Package: spatstat
Version: 3.6-1
Date: 2026-06-02
Title: Spatial Point Pattern Analysis, Model-Fitting, Simulation, Tests
Authors@R: c(person("Adrian", "Baddeley", 
                    role = c("aut", "cre"),
       	            email = "Adrian.Baddeley@curtin.edu.au",
		    comment = c(ORCID="0000-0001-9499-8382")),
	     person("Rolf", "Turner", 
                    role = "aut",
 	            email="rolfturner@posteo.net",
		    comment=c(ORCID="0000-0001-5521-5218")),
	     person("Ege",   "Rubak", 
                    role = "aut",
		    email = "rubak@math.aau.dk",
		    comment=c(ORCID="0000-0002-6675-533X")))
Maintainer: Adrian Baddeley <Adrian.Baddeley@curtin.edu.au>
Depends: R (>= 3.5.0), spatstat.data (>= 3.1-9), spatstat.univar (>=
        3.2-0), spatstat.geom (>= 3.8-1), spatstat.random (>= 3.5-0),
        spatstat.explore (>= 3.8-1), spatstat.model (>= 3.7-1),
        spatstat.linnet (>= 3.5-1), utils
Imports: spatstat.utils (>= 3.2-3)
Suggests:
Description: Comprehensive open-source toolbox for analysing Spatial Point Patterns. Focused mainly on two-dimensional point patterns, including multitype/marked points, in any spatial region. Also supports three-dimensional point patterns, space-time point patterns in any number of dimensions, point patterns on a linear network, and patterns of other geometrical objects. Supports spatial covariate data such as pixel images. 
	Contains over 3000 functions for plotting spatial data, exploratory data analysis, model-fitting, simulation, spatial sampling, model diagnostics, and formal inference. 
	Data types include point patterns, line segment patterns, spatial windows, pixel images, tessellations, and linear networks. 
	Exploratory methods include quadrat counts, K-functions and their simulation envelopes, nearest neighbour distance and empty space statistics, Fry plots, pair correlation function, kernel smoothed intensity, relative risk estimation with cross-validated bandwidth selection, mark correlation functions, segregation indices, mark dependence diagnostics, and kernel estimates of covariate effects. Formal hypothesis tests of random pattern (chi-squared, Kolmogorov-Smirnov, Monte Carlo, Diggle-Cressie-Loosmore-Ford, Dao-Genton, two-stage Monte Carlo) and tests for covariate effects (Cox-Berman-Waller-Lawson, Kolmogorov-Smirnov, ANOVA) are also supported.
	Parametric models can be fitted to point pattern data using the functions ppm(), kppm(), slrm(), dppm() similar to glm(). Types of models include Poisson, Gibbs and Cox point processes, Neyman-Scott cluster processes, and determinantal point processes. Models may involve dependence on covariates, inter-point interaction, cluster formation and dependence on marks. Models are fitted by maximum likelihood, logistic regression, minimum contrast, and composite likelihood methods. 
	A model can be fitted to a list of point patterns (replicated point pattern data) using the function mppm(). The model can include random effects and fixed effects depending on the experimental design, in addition to all the features listed above.
	Fitted point process models can be simulated, automatically. Formal hypothesis tests of a fitted model are supported (likelihood ratio test, analysis of deviance, Monte Carlo tests) along with basic tools for model selection (stepwise(), AIC()) and variable selection (sdr). Tools for validating the fitted model include simulation envelopes, residuals, residual plots and Q-Q plots, leverage and influence diagnostics, partial residuals, and added variable plots.
License: GPL (>= 2)
URL: http://spatstat.org/
NeedsCompilation: no
ByteCompile: true
BugReports: https://github.com/spatstat/spatstat/issues
Packaged: 2026-06-02 00:03:21 UTC; adrian
Author: Adrian Baddeley [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9499-8382>),
  Rolf Turner [aut] (ORCID: <https://orcid.org/0000-0001-5521-5218>),
  Ege Rubak [aut] (ORCID: <https://orcid.org/0000-0002-6675-533X>)
Repository: CRAN
Date/Publication: 2026-06-02 07:50:13 UTC
Built: R 4.5.2; ; 2026-06-02 11:12:08 UTC; unix

Package: spatstat
Version: 3.6-1
Date: 2026-06-02
Title: Spatial Point Pattern Analysis, Model-Fitting, Simulation, Tests
Authors@R: c(person("Adrian", "Baddeley", 
                    role = c("aut", "cre"),
       	            email = "Adrian.Baddeley@curtin.edu.au",
		    comment = c(ORCID="0000-0001-9499-8382")),
	     person("Rolf", "Turner", 
                    role = "aut",
 	            email="rolfturner@posteo.net",
		    comment=c(ORCID="0000-0001-5521-5218")),
	     person("Ege",   "Rubak", 
                    role = "aut",
		    email = "rubak@math.aau.dk",
		    comment=c(ORCID="0000-0002-6675-533X")))
Maintainer: Adrian Baddeley <Adrian.Baddeley@curtin.edu.au>
Depends: R (>= 3.5.0), spatstat.data (>= 3.1-9), spatstat.univar (>=
        3.2-0), spatstat.geom (>= 3.8-1), spatstat.random (>= 3.5-0),
        spatstat.explore (>= 3.8-1), spatstat.model (>= 3.7-1),
        spatstat.linnet (>= 3.5-1), utils
Imports: spatstat.utils (>= 3.2-3)
Suggests:
Description: Comprehensive open-source toolbox for analysing Spatial Point Patterns. Focused mainly on two-dimensional point patterns, including multitype/marked points, in any spatial region. Also supports three-dimensional point patterns, space-time point patterns in any number of dimensions, point patterns on a linear network, and patterns of other geometrical objects. Supports spatial covariate data such as pixel images. 
	Contains over 3000 functions for plotting spatial data, exploratory data analysis, model-fitting, simulation, spatial sampling, model diagnostics, and formal inference. 
	Data types include point patterns, line segment patterns, spatial windows, pixel images, tessellations, and linear networks. 
	Exploratory methods include quadrat counts, K-functions and their simulation envelopes, nearest neighbour distance and empty space statistics, Fry plots, pair correlation function, kernel smoothed intensity, relative risk estimation with cross-validated bandwidth selection, mark correlation functions, segregation indices, mark dependence diagnostics, and kernel estimates of covariate effects. Formal hypothesis tests of random pattern (chi-squared, Kolmogorov-Smirnov, Monte Carlo, Diggle-Cressie-Loosmore-Ford, Dao-Genton, two-stage Monte Carlo) and tests for covariate effects (Cox-Berman-Waller-Lawson, Kolmogorov-Smirnov, ANOVA) are also supported.
	Parametric models can be fitted to point pattern data using the functions ppm(), kppm(), slrm(), dppm() similar to glm(). Types of models include Poisson, Gibbs and Cox point processes, Neyman-Scott cluster processes, and determinantal point processes. Models may involve dependence on covariates, inter-point interaction, cluster formation and dependence on marks. Models are fitted by maximum likelihood, logistic regression, minimum contrast, and composite likelihood methods. 
	A model can be fitted to a list of point patterns (replicated point pattern data) using the function mppm(). The model can include random effects and fixed effects depending on the experimental design, in addition to all the features listed above.
	Fitted point process models can be simulated, automatically. Formal hypothesis tests of a fitted model are supported (likelihood ratio test, analysis of deviance, Monte Carlo tests) along with basic tools for model selection (stepwise(), AIC()) and variable selection (sdr). Tools for validating the fitted model include simulation envelopes, residuals, residual plots and Q-Q plots, leverage and influence diagnostics, partial residuals, and added variable plots.
License: GPL (>= 2)
URL: http://spatstat.org/
NeedsCompilation: no
ByteCompile: true
BugReports: https://github.com/spatstat/spatstat/issues
Packaged: 2026-06-02 00:03:21 UTC; adrian
Author: Adrian Baddeley [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9499-8382>),
  Rolf Turner [aut] (ORCID: <https://orcid.org/0000-0001-5521-5218>),
  Ege Rubak [aut] (ORCID: <https://orcid.org/0000-0002-6675-533X>)
Repository: CRAN
Date/Publication: 2026-06-02 07:50:13 UTC
Built: R 4.6.0; ; 2026-06-02 10:00:45 UTC; unix

# spatstat NAMESPACE file

importFrom(utils, RShowDoc, news, page, read.table)

import(spatstat.utils)
import(spatstat.data)
import(spatstat.univar)
import(spatstat.geom)
import(spatstat.random)
import(spatstat.explore)
import(spatstat.model)
import(spatstat.linnet)

# no dynamic library

# Do not edit the following.
# It is generated automatically.




# .................................................. 
#  load dynamic library
#      (native routines are now registered in init.c)
# .................................................. 

# There is no dynamic library
# useDynLib(spatstat, .registration=TRUE)

# .................................................. 
# Automatically-generated list of documented objects 
# .................................................. 
export("beginner")
export("bugfixes")
export("foo")
export("latest.changes")
export("latest.news")
export("plot.foo")
export("print.autoexec")    
export("print.bugtable")
export("print.changetable")
export("spatstat.family")

# .......  Special cases ........... 
# .......  End of special cases  ... 

# ......................................... 
# Automatically generated list of S3 methods
# ......................................... 
S3method("plot", "foo")
S3method("print", "autoexec")
S3method("print", "bugtable")
S3method("print", "changetable")
# ......................................... 
#      Assignment methods                   
# ......................................... 
# ......................................... 
#      End of methods                       
# ......................................... 

if(dev.cur() <= 1) {
  dd <- getOption("device")
  if(is.character(dd)) dd <- get(dd)
  dd()
}

oldpar <- par(ask = interactive() && dev.interactive(orNone=TRUE))
oldoptions <- options(warn=-1)

plot(amacrine)

plot(anemones, markscale=1)

ants.extra$plotit()

plot(austates)

plot(bei.extra$elev, main="Beilschmiedia")
plot(bei, add=TRUE, pch=16, cex=0.3)

plot(betacells)

plot(bramblecanes, cols=1:3)
plot(split(bramblecanes))

plot(bronzefilter,markscale=2)

plot(subset(btb, select=spoligotype), cols=2:5,
     main="Bovine Tuberculosis, by spoligotype")

plot(cells)

plot(chicago, main="Chicago Street Crimes", col="grey",
     cols=c("red", "blue", "black", "blue", "red", "blue", "blue"),
     chars=c(16,2,22,17,24,15,6), leg.side="left", show.window=FALSE)

chorley.extra$plotit()

plot(clmfires, which.marks="cause", cols=2:5, cex=0.25,
     main="Castilla-La Mancha forest fires")
plot(clmfires.extra$clmcov200, main="Covariates for forest fires")

plot(concrete)

plot(copper$Points, main="Copper")
plot(copper$Lines, add=TRUE)

plot(demohyper, quote({ plot(Image, main=""); plot(Points, add=TRUE) }),
      parargs=list(mar=rep(1,4)))

plot(dendrite, leg.side="bottom", main="Dendritic spines", cex=0.75, cols=2:4)

plot(demopat)

plot(finpines, main="Finnish pines")

wildM1 <- with(flu, virustype == "wt" & stain == "M2-M1")
plot(flu[wildM1, 1, drop=TRUE],
     main=c("flu data", "wild type virus, M2-M1 stain"),
     chars=c(16,3), cex=0.4, cols=2:3)

plot(gordon, main="People in Gordon Square", pch=16)

plot(gorillas, which.marks=1, chars=c(1,3), cols=2:3, main="Gorilla nest sites")

plot(hamster, cols=c(2,4))

plot(heather)

plot(humberside)

plot(hyytiala, cols=2:5)

plot(japanesepines)

plot(lansing)
plot(split(lansing))

plot(longleaf)

plot(mucosa, chars=c(1,3), cols=c("red", "green"))
plot(mucosa.subwin, add=TRUE, lty=3)

plot(murchison, main="Murchison data")

plot(murchison$greenstone, main="Murchison data", col="lightgreen")
plot(murchison$gold, add=TRUE, pch=3, col="blue")
plot(murchison$faults, add=TRUE, col="red")

plot(nbfires, use.marks=FALSE, pch=".")
plot(split(nbfires), use.marks=FALSE, chars=".")
plot(split(nbfires)$"2000", which.marks="fire.type",
     main=c("New Brunswick fires 2000", "by fire type"),
     cols=c("blue", "green", "red", "cyan"),
     leg.side="left")

plot(nztrees)
plot(trim.rectangle(as.owin(nztrees), c(0,5), 0), add=TRUE, lty=3)

plot(osteo[1:10,], tick.marks=FALSE, xlab="", ylab="", zlab="")

plot(paracou, cols=2:3, chars=c(16,3))

ponderosa.extra$plotit()

pyr <- pyramidal
pyr$grp <- abbreviate(pyramidal$group, minlength=7)
plot(pyr, quote(plot(Neurons, pch=16, main=grp)), main="Pyramidal Neurons")
rm(pyr)

plot(redwood)
plot(redwood3, add=TRUE, pch=20)

redwoodfull.extra$plotit()

plot(residualspaper$Fig1)
plot(residualspaper$Fig4a)
plot(residualspaper$Fig4b)
plot(residualspaper$Fig4c)

shapley.extra$plotit(main="Shapley")

plot(simdat)

plot(spiders, pch=16, show.window=FALSE)

plot(sporophores, chars=c(16,1,2), cex=0.6)
points(0,0,pch=16, cex=2)
text(15,8,"Tree", cex=0.75)

plot(spruces, maxsize=min(nndist(spruces)))

plot(subset(stonetools, select=TYPE),
     main="Palaeolithic stone tools and bone fragments",
     cols=2:3, cex=0.5)

plot(swedishpines)

plot(urkiola, cex=0.5, cols=2:3)

plot(waka, markscale=0.04, main=c("Waka national park", "tree diameters"))

plot(waterstriders, main.panel="",
     equal.scales=TRUE, mar.panel=0, hsep=1,
     cex=0.75)

par(oldpar)
options(oldoptions)

if(dev.cur() <= 1) {
  dd <- getOption("device")
  if(is.character(dd)) dd <- get(dd)
  dd()
}

oldpar <- par(ask = interactive() &&
              (.Device %in% c("X11", "GTK", "windows", "Macintosh")))
par(mfrow=c(1,1))
oldoptions <- options(warn = -1)

# 
#######################################################
#

X <- rpoispp(function(x,y) { 1000 * exp(- 4 * x)}, 1000)
plot(X, main="Inhomogeneous Poisson pattern")

fit.hom <- ppm(X ~1, Poisson())
fit.inhom <- ppm(X ~x, Poisson())

diagnose.ppm(fit.inhom, which="marks", type="Pearson",
             main=c("Mark plot",
               "Circles for positive residual mass",
               "Colour for negative residual density"))

par(mfrow=c(1,2))
diagnose.ppm(fit.hom, which="marks", 
             main=c("Wrong model", "(homogeneous Poisson)", "raw residuals"))
diagnose.ppm(fit.inhom, which="marks", 
             main=c("Right model", "(inhomogeneous Poisson)", "raw residuals"))
par(mfrow=c(1,1))

diagnose.ppm(fit.inhom, which="smooth", main="Smoothed residual field")

par(mfrow=c(1,2))
diagnose.ppm(fit.hom, which="smooth",
             main=c("Wrong model", "(homogeneous Poisson)",
                    "Smoothed residual field"))
diagnose.ppm(fit.inhom, which="smooth",
             main=c("Right model", "(inhomogeneous Poisson)",
                    "Smoothed residual field"))

par(mfrow=c(1,1))
diagnose.ppm(fit.inhom, which="x")

par(mfrow=c(1,2))
diagnose.ppm(fit.hom, which="x",
             main=c("Wrong model", "(homogeneous Poisson)",
                    "lurking variable plot for x"))
diagnose.ppm(fit.inhom, which="x",
             main=c("Right model", "(inhomogeneous Poisson)",
                    "lurking variable plot for x"))

par(mfrow=c(1,1))
diagnose.ppm(fit.hom, type="Pearson",main="standard diagnostic plots")

par(mfrow=c(1,2))
diagnose.ppm(fit.hom, main=c("Wrong model", "(homogeneous Poisson)"))
diagnose.ppm(fit.inhom,  main=c("Right model", "(inhomogeneous Poisson)"))
par(mfrow=c(1,1))


# 
#######################################################
#  LEVERAGE/INFLUENCE

plot(leverage(fit.inhom))

plot(influence(fit.inhom))

plot(dfbetas(fit.inhom))

# 
#######################################################
#  COMPENSATORS

## Takes a long time...
CF <- compareFit(listof(hom=fit.hom, inhom=fit.inhom),
                 Kcom, same="iso", different="icom")
plot(CF, main="model compensators", legend=FALSE)
legend("topleft",
       legend=c("empirical K function", "compensator of CSR",
         "compensator of inhomogeneous Poisson"), lty=1:3, col=1:3)

# 
#######################################################
#  Q - Q  PLOTS
#
qqplot.ppm(fit.hom, 40) 
#conclusion: homogeneous Poisson model is not correct
title(main="Q-Q plot of smoothed residuals")

qqplot.ppm(fit.inhom, 40) # TAKES A WHILE...
title(main=c("Right model", "(inhomogeneous Poisson)",
             "Q-Q plot of smoothed residuals"))
# conclusion: fitted inhomogeneous Poisson model looks OK
# 
#######################################################
#
plot(cells)
fitPoisson <- ppm(cells ~1, Poisson())
diagnose.ppm(fitPoisson, 
             main=c("CSR fitted to cells data",
                    "Raw residuals",
                    "No suggestion of departure from CSR"))
diagnose.ppm(fitPoisson, type="pearson",
             main=c("CSR fitted to cells data",
                    "Pearson residuals",
                    "No suggestion of departure from CSR"))
# These diagnostic plots do NOT show evidence of departure from uniform Poisson

plot(Kcom(fitPoisson), cbind(iso, icom) ~ r)
plot(Gcom(fitPoisson), cbind(han, hcom) ~ r)

# K compensator DOES show strong evidence of departure from uniform Poisson

qqplot.ppm(fitPoisson, 40)
title(main=c("CSR fitted to cells data",
        "Q-Q plot of smoothed raw residuals",
        "Strong suggestion of departure from CSR"))
           
# Q-Q plot DOES show strong evidence of departure from uniform Poisson.
#
fitStrauss <- ppm(cells ~1, Strauss(r=0.1))
diagnose.ppm(fitStrauss, 
             main=c("Strauss model fitted to cells data",
                    "Raw residuals"))
diagnose.ppm(fitStrauss, type="pearson",
             main=c("Strauss model fitted to cells data",
                    "Pearson residuals"))

plot(Kcom(fitStrauss), cbind(iso, icom) ~ r)
plot(Gcom(fitStrauss), cbind(han, hcom) ~ r)

# next line takes a LOOONG time ...
qqplot.ppm(fitStrauss, 40, type="pearson")
title(main=c("Strauss model fitted to cells data",
        "Q-Q plot of smoothed Pearson residuals",
        "Suggests adequate fit")) 
# Conclusion: Strauss model seems OK
# 
#######################################################
#
plot(nztrees)
fit <- ppm(nztrees ~1, Poisson())
diagnose.ppm(fit, type="pearson")
title(main=c("CSR fitted to NZ trees",
             "Pearson residuals"))
diagnose.ppm(fit, type="pearson", cumulative=FALSE)
title(main=c("CSR fitted to NZ trees",
             "Pearson residuals (non-cumulative)"))
lurking(fit, expression(x), type="pearson", cumulative=FALSE,
        splineargs=list(spar=0.3))
# Sharp peak at right is suspicious
qqplot.ppm(fit, 40, type="pearson")
title(main=c("CSR fitted to NZ trees",
        "Q-Q plot of smoothed Pearson residuals"))
# Slight suggestion of departure from Poisson at top right of pattern.
par(oldpar)
options(oldoptions)

### R code from vignette source 'updates.Rnw'

###################################################
### code chunk number 1: updates.Rnw:20-29
###################################################
library(spatstat)
x <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
              fields = c("Version", "Date"))
sdate    <- as.character(x[,"Date"])
sversion <- as.character(x[,"Version"])
## recognise fractional version number
IsDevel <- function(ver) { length(unlist(package_version(ver))) > 3 }
sdevel   <- IsDevel(x[,"Version"])
options(useFancyQuotes=FALSE)


###################################################
### code chunk number 2: updates.Rnw:39-142
###################################################
readSizeTable <- function(fname) {
  if(is.null(fname) || !file.exists(fname)) return(NULL)
  a <- read.table(fname, header=TRUE)
  a$date <- as.Date(a$date)
  return(a)
}
getSizeTable <- function(packagename="spatstat", tablename="packagesizes.txt") {
  fname <- system.file("info", tablename, package=packagename)
  out <- readSizeTable(fname)
  if(is.null(out)) {
    fname <- system.file("doc", tablename, package=packagename)
    out <- readSizeTable(fname)
  }
  return(out)
}
RemoveDevel <- function(sizetable) {
  ## remove entries with fractional version numbers
  if(is.null(sizetable)) return(NULL)
  isdevel <- sapply(sizetable$version, IsDevel)
  st <- if(all(isdevel)) NULL else sizetable[!isdevel, , drop=FALSE]
  return(st)
}
counts <- c("nhelpfiles", "nobjects", "ndatasets", "Rlines", "srclines")
mergeSizeTables <- function(a, b, breakupdate, allow.devel=sdevel) {
  #' a is the running total for spatstat; b is a sub-package.
  #' breakupdate is the date when the code in b was removed from spatstat
  #' so that the size of 'b' must be added to 'a' for all dates >= breakupdate
  if(!allow.devel) b <- RemoveDevel(b)
  if(is.null(b)) return(a)
  adates <- a$date
  bdates <- b$date
  alldates <- sort(unique(c(adates,bdates)))
  if(missing(breakupdate)) breakupdate <- min(bdates)
  #' functions to determine, for any given date,
  #' the relevant (latest) row of the table
  aok <- rev(!duplicated(rev(adates)))
  arowfun <- approxfun(adates[aok], seq_along(adates)[aok], 
                       method="constant", f=0, rule=2, yleft=0)
  bok <- rev(!duplicated(rev(bdates)))
  browfun <- approxfun(bdates[bok], seq_along(bdates)[bok], 
                       method="constant", f=0, rule=2, yleft=0)
  result <- NULL
  for(k in seq_along(alldates)) {
    thedate <- alldates[k]
    i <- arowfun(thedate)
    j <- browfun(thedate)
    #' i > 0 because spatstat's founding date is earlier than any sub-package
    nextrow <- a[i, ]
    if(j > 0 && thedate >= breakupdate) {
      #' add contribution from 'b'
      nextrow[, counts] <- nextrow[, counts] + b[j, counts]
    }
    result <- rbind(result, nextrow)
  }
  return(result)
}
## Get histories of all sub-packages
## Package formerly known as 'spatstat'
z <- getSizeTable() 
## installed sub-packages - access via the installed sub-packages
zutils   <- getSizeTable("spatstat.utils")
zdata    <- getSizeTable("spatstat.data")
zunivar  <- getSizeTable("spatstat.univar")
zsparse  <- getSizeTable("spatstat.sparse")
zgeom    <- getSizeTable("spatstat.geom")
zrandom  <- getSizeTable("spatstat.random")
zexplore <- getSizeTable("spatstat.explore")
zmodel   <- getSizeTable("spatstat.model")
zlinnet  <- getSizeTable("spatstat.linnet")
## other sub-packages - access via stored copies of package size files
##       defunct package spatstat.core 
zcore    <- getSizeTable("spatstat", "spatstatcoresize.txt")
##       extension packages 
zlocal   <- getSizeTable("spatstat", "spatstatlocalsize.txt")
zgui     <- getSizeTable("spatstat", "spatstatguisize.txt")
zKnet    <- getSizeTable("spatstat", "spatstatKnetsize.txt")
## Merge histories starting at the 'split dates'
z <- mergeSizeTables(z, zutils,  "2017-03-22")
z <- mergeSizeTables(z, zdata,   "2017-09-23")
z <- mergeSizeTables(z, zsparse, "2020-11-04")
BigSplitDay <- "2020-12-14"
z <- mergeSizeTables(z, zgeom,   BigSplitDay)
z <- mergeSizeTables(z, zcore,   BigSplitDay)
z <- mergeSizeTables(z, zlinnet, BigSplitDay)
z <- mergeSizeTables(z, zrandom, "2022-02-12")
CoreSplitDay <- "2020-05-25"   # size of 'core' drops to 0 on this date
z <- mergeSizeTables(z, zexplore, CoreSplitDay)
z <- mergeSizeTables(z, zmodel,   CoreSplitDay)
z <- mergeSizeTables(z, zunivar,  "2024-04-21")
## extension packages: these never overlapped spatstat
z <- mergeSizeTables(z, zlocal)
z <- mergeSizeTables(z, zgui)
z <- mergeSizeTables(z, zKnet)
## Now summarise
currentcount <- z[nrow(z), counts]
bookcount    <- z[z$version == "1.42-0", counts]
changes <- currentcount - bookcount
newobj <- changes[["nobjects"]]
newdat <- changes[["ndatasets"]] + 2  # redwood3, shelling2
newcode  <- changes[["Rlines"]] + changes[["srclines"]]
bookcode <- bookcount[["Rlines"]] + bookcount[["srclines"]]
currentcode <- currentcount[["Rlines"]] + currentcount[["srclines"]]
growth <- signif((100 * newcode)/bookcode, digits=2)


###################################################
### code chunk number 3: updates.Rnw:154-160
###################################################
options(SweaveHooks=list(fig=function() par(mar=0.2+c(2,4,2,0))))
Plot <- function(fmla, ..., dat=z, ylim=NULL) {
  yvals <- eval(as.expression(fmla[[2]]), envir=dat)
  if(is.null(ylim)) ylim <- c(0, max(yvals))
  plot(fmla, ..., data=dat, type="n", xlab="", ylim=ylim, las=1)
}


###################################################
### code chunk number 4: updates.Rnw:166-180
###################################################
getOption("SweaveHooks")[["fig"]]()
Plot((Rlines + srclines)/1000 ~ date, ylab="Lines of code (x 1000)", 
     main="Spatstat growth")
with(z, {
  polygon(c(date, rev(date)),
          c(srclines, rep(0, length(srclines)))/1000,
          angle=45, density=5, col="pink")
  polygon(c(date, rev(date)),
          c(srclines+Rlines, rev(srclines))/1000,
          angle=135, density=5, col="lightblue")
})
text(as.Date("2019-01-01"), 90, "R code", col="blue")
text(as.Date("2020-01-01"), 15, "C and C++ code", col="red")
lines((Rlines + srclines)/1000 ~ date, data=z, lwd=3)
lines(srclines/1000 ~ date, data=z, col="red")


###################################################
### code chunk number 5: updates.Rnw:197-221
###################################################
## Tabulate latest version numbers of packages
vtable <- data.frame(package="spatstat", version=sversion, date=as.Date(sdate))
AppendVersion <- function(pkg, sizetable, v, allow.devel=sdevel) {
  if(!allow.devel) sizetable <- RemoveDevel(sizetable)
  if(is.null(sizetable)) return(v)
  lastrow <- sizetable[nrow(sizetable), , drop=FALSE]
  if(is.null(lastrow)) return(v)
  rbind(v, data.frame(package=pkg, version=lastrow[,"version"], date=as.Date(lastrow[,"date"])))
}
vtable <- AppendVersion("spatstat.utils", zutils, vtable)
vtable <- AppendVersion("spatstat.data", zdata, vtable)
vtable <- AppendVersion("spatstat.sparse", zsparse, vtable)
vtable <- AppendVersion("spatstat.univar", zunivar, vtable)
vtable <- AppendVersion("spatstat.geom", zgeom, vtable)
vtable <- AppendVersion("spatstat.random", zrandom, vtable)
vtable <- AppendVersion("spatstat.explore", zexplore, vtable)
vtable <- AppendVersion("spatstat.model", zmodel, vtable)
vtable <- AppendVersion("spatstat.linnet", zlinnet, vtable)
## move spatstat to the bottom
vtable <- rbind(vtable[-1, ], vtable[1, ])
## add extras
vtable <- AppendVersion("spatstat.local", zlocal, vtable)
vtable <- AppendVersion("spatstat.Knet", zKnet, vtable)
vtable <- AppendVersion("spatstat.gui", zgui, vtable)


###################################################
### code chunk number 6: updates.Rnw:227-228
###################################################
print(vtable[,c(3,1,2)], row.names=FALSE)

if(dev.cur() <= 1) {
  dd <- getOption("device")
  if(is.character(dd)) dd <- get(dd)
  dd()
}

oldpar <- par(ask = interactive() && dev.interactive(orNone=TRUE))
oldoptions <- options(warn=-1)

fanfare <- function(stuff) {
  plot(c(0,1),c(0,1),type="n",axes=FALSE, xlab="", ylab="")
  text(0.5,0.5, stuff, cex=2.5)
}

par(mar=c(1,1,2,1)+0.1)

fanfare("Spatstat demonstration")

fanfare("I. Types of data")
plot(swedishpines, main="Point pattern")

plot(demopat, cols=c("green", "blue"), main="Multitype point pattern")

plot(longleaf, fg="blue", main="Marked point pattern")

plot(finpines, main="Point pattern with multivariate marks")

a <- psp(runif(20),runif(20),runif(20),runif(20), window=owin())
plot(a, main="Line segment pattern")
marks(a) <- sample(letters[1:4], 20, replace=TRUE)
plot(a, main="Multitype line segment pattern")
marks(a) <- runif(20)
plot(a, main="Marked line segment pattern", style="width")

plot(owin(), main="Rectangular window")
plot(letterR, main="Polygonal window")
plot(as.mask(letterR), main="Binary mask window")

par(mar=c(1,1,2,4)+0.1)

Z <- as.im(function(x,y){ sqrt((x - 1)^2 + (y-1)^2)}, square(2))
plot(Z, main="Pixel image")

X <- runifpoint(42)
plot(dirichlet(X), main="Tessellation")

plot(rpoispp3(100), main="Three-dimensional point pattern")

plot(simplenet, main="Linear network (linnet)")

X <- rpoislpp(20, simplenet)
plot(X,
     main="Point pattern on linear network (lpp)",
     show.window=FALSE)

V <- as.linim(function(x,y,seg,tp){x^2-y^2}, L=simplenet)
plot(V, main="Pixel image on a linear network")

par(mar=c(1,1,2,5)+0.1)
L <- infline(p=runif(5), theta=runif(5, max=pi/2))
Y <- chop.linnet(simplenet, L)
tilenames(Y) <- letters[seq_len(nobjects(Y))]
plot(Y, main="Tessellation on a linear network")
  
fanfare("II. Graphics")

par(mar=c(5,4,2,3)+0.1)

plot(letterR, col="green", border="red", lwd=2, main="Polygonal window with colour fill")
plot(letterR, hatch=TRUE, spacing=0.15, angle=30, main="Polygonal window with line shading")
plot(letterR, hatch=TRUE, hatchargs=list(texture=8, spacing=0.12),
     main="Polygonal window with texture fill")

plot(amacrine, chars=c(1,16),
     main="plot(X, chars = c(1,16))")
plot(amacrine, cols=c("red","blue"), chars=16,
     main="plot(X, cols=c(\"red\", \"blue\"))")

opa <- par(mfrow=c(1,2))
plot(longleaf, markscale=0.03, main="markscale=0.03")
plot(longleaf, markscale=0.09, main="markscale=0.09")           
par(opa)

par(mar=c(5,4,2,5)+0.1)
plot(longleaf, pch=21, cex=1,
     bg=colourmap(terrain.colors(128), range=c(0,80)),
     main="colourmap for numeric mark values")

par(mar=c(5,4,2,3)+0.1, xpd=NA)

Z <- as.im(function(x,y) { r <- sqrt(x^2+y^2); r * exp(-r) },
           owin(c(-5,5),c(-5,5)))
plot(Z, main="pixel image: image plot")
plot(Z, main="pixel image: image plot (heat colours)", col=heat.colors(256))
plot(Z, main="pixel image: logarithmic colour map", 
     log=TRUE, col=rainbow(128, end=5/6))
contour(Z, main="pixel image: contour plot", axes=FALSE)
plot(Z, main="pixel image: image + contour plot")
contour(Z, add=TRUE)
persp(Z, colmap=terrain.colors(128), shade=0.3, phi=30,theta=100,
      main="pixel image: perspective plot")

ct <- colourmap(rainbow(20), breaks=seq(-1,1,length=21))
plot(ct, main="Colour map for real numbers")

ca <- colourmap(rainbow(8), inputs=letters[1:8])
plot(ca, main="Colour map for discrete values")

Z <- as.im(nnfun(runifpoint(8)))
plot(Z, main="colour image for discrete values")
textureplot(Z, main="texture plot for discrete values")

W <- owin(c(1,5),c(0,4.5))
Lout <- scaletointerval(distmap(rebound.owin(letterR, W)))
Lin <- scaletointerval(distmap(complement.owin(letterR, W)))
L <- scaletointerval(eval.im(Lin-Lout))
D <- scaletointerval(density(runifpoint(30, W), adjust=0.3))
X <- scaletointerval(as.im(function(x,y){ x }, W=W))
plot(listof(L=L, D=D, X=X), main="Multiple images")
pairs(L, D, X, main="Multiple images: pairs plot")
persp(L, colin=D,
      theta=-24, phi=35, box=FALSE, apron=TRUE, 
      main="Two images:\nperspective + colours",
      shade=0.4, ltheta=225, lphi=10)
plot(rgbim(D,X,L,maxColorValue=1), valuesAreColours=TRUE,
     main="Three images: RGB display")
plot(hsvim(D,L,X), valuesAreColours=TRUE,
     main="Three images: HSV display")

V <- as.linim(function(x,y,seg,tp){(y/1000)^2-(x/1000)^3}, L=domain(chicago))
plot(V, main="Pixel image on a linear network (colour plot)")
plot(V, style="w", main="Pixel image on a linear network (width plot)")
persp(V, phi=25, theta=-35, expand=0.2, main="Pixel image on a linear network (perspective plot)")

XL <- rpoislpp(20, simplenet)
plot(XL, main="Point pattern on a network (default plot)")
plot(XL,
     shape="crossticks", size=0.04,
     main="Point pattern on a network (crossticks plot)")

plot(chicago, cols=1:7,
     main="Multitype point pattern on a network (symbols and colours)")
plot(dendrite, shape="crossticks", cols=2:4, size=8,
     leg.side="bottom", leg.args=list(lwd=3),
     main="Multitype point pattern on a network (crossticks and colours)")


fanfare("III. Conversion between types")

W <- as.owin(chorley)
plot(W, "window W")

plot(as.mask(W))
plot(as.mask(W, dimyx=1000))

plot(as.im(W, value=3))
plot(as.im(W, value=3, na.replace=0), ribbon=TRUE)

plot(as.im(function(x,y) {x^2 + y}, W=square(1)),
     main="as.im(function(x,y){x^2+y})")

V <- delaunay(runifpoint(12))
plot(V, main="Tessellation V")
plot(as.im(V, dimyx=256), main="as.im(V)")
plot(as.owin(V))

X <- swedishpines
plot(X, "point pattern X")

plot(as.im(X), col=c("white","red"), ribbon=FALSE, xlab="", ylab="")
plot(as.owin(X), add=TRUE)

fanfare("IV. Subsetting and splitting data")

plot(X, "point pattern X")
subset <- 1:20
plot(X[subset], main="subset operation: X[subset]")
subwindow <- owin(poly=list(x=c(0,96,96,40,40),y=c(0,0,100,100,50)))
plot(X[subwindow], main="subset operation: X[subwindow]")

plot(lansing, "Lansing Woods data")
plot(split(lansing),
     main="split operation: split(X)",
     mar.panel=c(0,0,2,0), hsep=1, pch=3)

plot(longleaf, main="Longleaf Pines data")
plot(cut(longleaf, breaks=3),
     main=c("cut operation", "cut(longleaf, breaks=3)"),
     cols=2:4)

Z <- dirichlet(runifpoint(10))
X <- runifpoint(100)

plot(cut(X,Z), main="points cut by tessellation", leg.side="left", cols=1:10)
plot(Z, add=TRUE)

plot(split(X, Z),
     main="points split by tessellation",
     mar.panel=c(0,0,2,2), hsep=1)

W <- square(1)
X <- as.im(function(x,y){sqrt(x^2+y^2)}, W)
Y <- dirichlet(runifpoint(12, W))
plot(split(X,Y), main="image split by tessellation")

fanfare("V. Tessellations")

plot(quadrats(square(1), nx=5, ny=3))
plot(hextess(square(1), 0.07))
plot(quantess(letterR, "x", 7))
plot(polartess(letterR, nangular=6, radii=(0:4)/2, origin=c(2.8, 1.5)),
     do.col=TRUE)
plot(delaunay(cells))
plot(cells, add=TRUE)
plot(dirichlet(cells))
plot(cells, add=TRUE)
plot(rpoislinetess(2.5), do.col=TRUE)

fanfare("VI. Exploratory data analysis")

par(mar=c(3,3,3,2)+0.1)

plot(swedishpines, main="Quadrat counts", pch="+")
tab <- quadratcount(swedishpines, 4)
plot(tab, add=TRUE, lty=2, cex=2, col="blue")

par(mar=c(5,3,3,2)+0.1)

plot(swedishpines, main="", pch="+")
title(main=expression(chi^2 * " test"), cex.main=2)
tes <- quadrat.test(swedishpines, 3)
tes
plot(tes, add=TRUE, col="red", cex=1.5, lty=2, lwd=3)
title(sub=paste("p-value =", signif(tes$p.value,3)), cex.sub=1.4)

par(mar=c(4,4,3,2)+0.1)

tesk <- cdf.test(nztrees, "x")
tesk
plot(tesk)


mur <- lapply(murchison, rescale, s=1000)
mur <- lapply(mur, "unitname<-", value="km")
X <- mur$gold
D <- distfun(mur$faults)
plot(X, main="Murchison gold deposits", cols="blue")
plot(mur$faults, add=TRUE, col="red")
rh <- rhohat(X,D)
plot(rh,
     main="Smoothed rate estimate",
     xlab="Distance to nearest fault (km)",
     legend=FALSE, xlim=c(0, 20))
plot(predict(rh), main="predict(rhohat(X,D))")

Z <- density(cells, 0.07)
plot(Z, main="Kernel smoothed intensity of point pattern")
plot(cells, add=TRUE)

plot(redwood, main="Redwood data")
te <- scan.test(redwood, 0.1, method="poisson")
plot(te, main=c("Scan Statistic for redwood data",
              paste("p-value =", signif(te$p.value,3))))
plot(redwood, add=TRUE)
te

X <- unique(unmark(shapley))
plot(X, "Shapley galaxy concentration", pch=".")
coco <-colourmap(rev(rainbow(128, end=2/3)), range=c(0,1))
pa <- function(i, ...) {
  if(i == 1) list(chars=c(".", "+"), cols=1:2) else
             list(size=0.5, pch=16, col=coco)
}
plot(nnclean(X, k=17), panel.args=pa,
     mar.panel=c(0,1,1,0), nrows=2,
     main="Byers-Raftery nearest neighbour cleaning",
     cex.title=1.2)
Y <- sharpen(X, sigma=0.5, edgecorrect=TRUE)
plot(Y, main="Choi-Hall data sharpening", pch=".")

  owpa <- par(mfrow=c(1,2))
  W <- grow.rectangle(as.rectangle(letterR), 1)
  X <- superimpose(runifpoint(300, letterR),
                   runifpoint(50, W), W=W)
  plot(W, main="clusterset(X, 'm')")
  plot(clusterset(X, 'marks', fast=TRUE), add=TRUE, chars=c("o", "+"), cols=1:2)
  plot(letterR, add=TRUE)
  plot(W, main="clusterset(X, 'd')")
  plot(clusterset(X, 'domain', exact=FALSE), add=TRUE)
  plot(letterR, add=TRUE)
  par(owpa)

D <- density(a, sigma=0.05)
plot(D, main="Kernel smoothed intensity of line segment pattern")
plot(a, add=TRUE)

X <- runifpoint(42)
plot(dirichlet(X))
plot(X, add=TRUE)

plot(delaunay(X))
plot(X, add=TRUE)

parsave <- par(mfrow=c(1,1), mar=0.2+c(0,1,3,1))
plot(listof("Longleaf Pines data"=longleaf,
            "Nearest mark"=nnmark(longleaf),
            "Kernel smoothing of marks"=Smooth(longleaf,10),
            "Inverse distance weighted\nsmoothing of marks"=idw(longleaf)),
     equal.scales=TRUE, halign=TRUE, valign=TRUE,
     main="", mar.panel=0.2+c(0,0,2,2))
par(parsave)

fryplot(cells, main=c("Fry plot","cells data"), pch="+")
miplot(longleaf, main="Morishita Index plot", pch=16, col="blue")

plot(swedishpines, main="Swedish Pines data")
K <- Kest(swedishpines)
plot(K, main="K function for Swedish Pines", legendmath=TRUE)

en <- envelope(swedishpines, fun=Kest, nsim=10, correction="translate")
plot(en, main="Envelopes of K function based on CSR", shade=c("hi", "lo"))

pc <- pcf(swedishpines)
plot(pc, main="Pair correlation function")

plot(swedishpines, main="nearest neighbours")
m <- nnwhich(swedishpines)
b <- swedishpines[m]
arrows(swedishpines$x, swedishpines$y, b$x, b$y,
       angle=12, length=0.1, col="red")

plot(swedishpines %mark% nndist(swedishpines),
     markscale=1, main="Stienen diagram", legend=FALSE, fg="blue")

plot(Gest(swedishpines),
     main=c("Nearest neighbour distance function G", "Gest(swedishpines)"),
     legendmath=TRUE)

Z <- distmap(swedishpines, dimyx=512)
plot(swedishpines$window, main="Distance map")
plot(Z, add=TRUE)
points(swedishpines)

plot(Fest(swedishpines),
     main=c("Empty space function F", "Fest(swedishpines)"),
     legendmath=TRUE)

W <- rebound.owin(letterR, square(5))
plot(distmap(W), main="Distance map")
plot(W, add=TRUE)

a <- psp(runif(20),runif(20),runif(20),runif(20), window=owin())
contour(distmap(a), main="Distance map")
plot(a, add=TRUE,col="red")

plot(Jest(swedishpines), main=c("J-function", "J(r)=(1-G(r))/(1-F(r))"))

X <- swedishpines
X <- X[sample(1:npoints(X))]
Z <- nnfun(X)
plot(as.owin(X), main="Nearest neighbour map")
plot(Z, add=TRUE)
points(X)

plot(allstats(swedishpines))

Fig4b <- residualspaper$Fig4b

plot(Fig4b, main="Inhomogeneous point pattern")
plot(Kinhom(Fig4b), main="Inhomogeneous K-function")
plot(pcfinhom(Fig4b, stoyan=0.1), main="Inhomogeneous pair correlation")
plot(Ginhom(Fig4b, sigma=0.06), main="Inhomogeneous G-function")
plot(Jinhom(Fig4b, sigma=0.06), main="Inhomogeneous J-function")

X <- unmark(bronzefilter)
plot(X, "Bronze filter data")
lam <- predict(ppm(X ~x))
plot(Kscaled(X, lam), xlim=c(0, 1.5), main="Locally-scaled K function")

plot(urkiola)
plot(split(urkiola), cex=0.5)
plot(density(split(urkiola)))
contour(density(split(urkiola)), panel.begin=as.owin(urkiola))
plot(relrisk(urkiola), main="Relative risk (cross-validated)")

plot(bramblecanes)
br <- rescale(bramblecanes)
plot(alltypes(br, "K"), mar.panel=c(4,5,2,2)+0.1)

ama <- rescale(amacrine)
plot(alltypes(ama, Lcross, envelope=TRUE, nsim=9), . - r ~ r, ylim=c(-25, 5))

ponderosa.extra$plotit(main="Ponderosa Pines")

L <- localL(ponderosa)
pL <- plot(L, lty=1, col=1, legend=FALSE,
           main=c("neighbourhood density functions",
             "for Ponderosa Pines"), cex.main=0.8)

parsave <- par(mfrow=c(1,2))
ponderosa.extra$plotit()
par(pty="s")
plot(L, iso007 ~ r, main="point B")

par(mar=0.2+c(1,1,3,1))
ponderosa.extra$plotit()
L12 <- localL(ponderosa, rvalue=12)
P12 <- ponderosa %mark% L12
Z12 <- Smooth(P12, sigma=5, dimyx=128)
plot(Z12, col=topo.colors(128),
     main=c("smoothed", "neighbourhood density"),
     cex.main=0.8)
contour(Z12, add=TRUE)
points(ponderosa, pch=16, cex=0.5)

plot(amacrine, main="Amacrine cells data", cex.main=0.8)
par(pty="s")
mkc <- markcorr(amacrine, 
                correction="translate", method="density",
                kernel="epanechnikov")
plot(mkc, main="Mark correlation function", legend=FALSE, cex.main=0.8)
par(parsave)

par(mar=0.2+c(4,4,3,1))
plot(alltypes(amacrine, markconnect), 
     title="Mark connection functions for amacrine cells")

parsave <- par(mfrow=c(1,2))

parspruce2 <- par(mar=0.2+c(0,2,2,0))
plot(spruces, cex.main=0.8, markscale=10)
par(pty="s", mar=0.2+c(2,3,2,0))
plot(markcorr(spruces), main="Mark correlation", legendpos="bottomright")

par(parspruce2)
plot(spruces, cex.main=0.8, markscale=10)
par(pty="s", mar=0.2+c(2,3,2,0))
plot(markvario(spruces), main="Mark variogram", legendpos="topright")
par(parsave)
par(mar=0.2+c(4,4,3,2))

plot(listof("Emark(spruces)"=Emark(spruces),
            "Vmark(spruces)"=Vmark(spruces)),
     main="Independence diagnostics", ylim.covers=0,
     legendpos="bottom")

par3 <- par(mfrow=c(1,2))
X <- rpoispp3(100)
plot(X, main="3D point pattern X")
plot(K3est(X), main="K-function in 3D")
plot(X, main="3D point pattern X")
plot(G3est(X), main="G-function in 3D", legendpos="bottomright")
par(par3)

par(mfrow=c(1,3))
X <- unmark(chicago)
plot(X, col="green", cols="red", pch=16,
     main="Chicago Street Crimes", cex.main=0.75,
     show.window=FALSE, claim.title.space=TRUE)
plot(density(X, 100, distance="e"), main=c("Kernel density estimate", "(Euclidean)"))
plot(density(X, 100, distance="p"), main=c("Kernel density estimate", "(shortest path)"))

plot(X, col="green", cols="red", pch=16,
     main="Chicago Street Crimes", cex.main=0.75,
     show.window=FALSE)
plot(linearK(X, correction="none"), main="Network K-function")
plot(linearK(X, correction="Ang"), main="Corrected K-function")

par(mfrow=c(1,1))

fanfare("VII. Model-fitting")

parsave <- par(mar=0.2+c(1,1,3,3))
plot(japanesepines)
fit <- ppm(japanesepines ~1)
print(fit)
fit <- ppm(japanesepines ~polynom(x,y,2))
print(fit)
plot(fit, how="image", se=FALSE, main=c("Inhomogeneous Poisson model",
                               "fit by maximum likelihood",
                               "Fitted intensity"))
plot(fit, how="image", trend=FALSE,
     main=c("Standard error", "of fitted intensity"))

plot(leverage(fit))
plot(influence(fit))

plot(mur$gold, main="Murchison gold deposits", cols="blue")
plot(mur$faults, add=TRUE, col="red")
fit <- ppm(mur$gold ~D, covariates=list(D=distfun(mur$faults)))
par(mar=0.2+c(4,4,4,2))
plot(parres(fit, "D"),
     main="Partial residuals from loglinear Poisson model",
     xlab="Distance to nearest fault (km)",
     ylab="log intensity of gold", legend=FALSE)
legend("bottomleft", legend=c("partial residual", "loglinear fit"),
       col=c(1,4), lty=c(1,4))

par(mar=rep(0.2, 4), mfrow=c(1,1))
fitT <- kppm(redwood ~1, clusters="Thomas")
simT <- simulate(fitT)[[1]]
plot(listof(redwood, simT),
     main.panel=c("Redwood", "simulation from\nfitted Thomas model"),
     main="", mar.panel=0.2, equal.scales=TRUE)

mop <- par(mfrow=c(1,2), pty="s", mar=rep(4.4, 4))
plot(fitT, xname=c("Thomas model", "minimum contrast fit"), pause=FALSE)
par(mop)

oop <- par(pty="s", mar=0.2+c(4,4,4,2))
os <- objsurf(fitT)
plot(os, main="Minimum contrast objective function", col=terrain.colors(128))
contour(os, add=TRUE)
par(oop)

parra <- par(mfrow=c(1,2), mar=0.2+c(3,3,4,2))
plot(swedishpines)
fit <- ppm(swedishpines ~1, Strauss(r=7))
print(fit)
plot(fit, how="image", main=c("Strauss model",
                               "fit by maximum pseudolikelihood",
                               "Conditional intensity plot"))
# fitted interaction
plot(swedishpines)
fit <- ppm(swedishpines ~1, PairPiece(c(3,5,7,9,11,13)))
plot(fitin(fit), legend=FALSE,
     main=c("Pairwise interaction model",
            "fit by maximum pseudolikelihood"))

# simulation
par(mfrow=c(1,1), mar=0.5+c(0,0,2,0))
Xsim <- rmh(model=fit,
            start=list(n.start=80),
            control=list(nrep=100))
plot(listof(swedishpines, Xsim),
     main="",
     main.panel=c("Swedish Pines",
       "Simulation from\nfitted Strauss model"),
     mar.panel=c(0,0,3,0),hsep=1,equal.scales=TRUE)

# model compensator
par(parra)
par(mar=0.2+c(4,4,3,1))
plot(swedishpines)
fit <- ppm(swedishpines ~1, Strauss(r=7))
plot(Kcom(fit), cbind(iso, icom, pois) ~ r,
     legend=FALSE, main="model compensators")
legend("topleft", legend=c("empirical K function",
                    "Strauss model compensator of K",
                    "Poisson theoretical K"), lty=1:3, col=1:3, inset=0.05)

par(parsave)

# Multitype data
dpat <- rescale(demopat, 8)
unitname(dpat) <- c("mile", "miles")
dpat

plot(dpat, cols=c("red", "blue"))
fit <- ppm(dpat ~marks + polynom(x,y,2), Poisson())
plot(fit, trend=TRUE, se=TRUE)

# Linear network data
plot(spiders)
fit <- lppm(spiders ~ polynom(x,y,2))
anova(fit, test="Chi")
(fit <- step(fit))
lam <- predict(fit)
plot(lam, main="Point process model on linear network", ribscale=1000)
plot(spiders, add=TRUE, pch=16, show.network=FALSE)

fanfare("VIII. Simulation")

plot(letterR, main="Poisson random points")
lambda <- 10/area.owin(letterR)
points(rpoispp(lambda, win=letterR))
points(rpoispp(9 * lambda, win=letterR))
points(rpoispp(90 * lambda, win=letterR))
plot(rpoispp(100))
plot(rpoispp(function(x,y){1000 * exp(-3*x)}, 1000),
     main="rpoispp(function)")

plot(rMaternII(200, 0.05))
plot(rSSI(0.05, 200))
plot(rThomas(10, 0.2, 5))
plot(rMatClust(10, 0.05, 4))
plot(rCauchy(30, 0.01, 5))
plot(rVarGamma(30, 2, 0.02, 5))
plot(rGaussPoisson(30, 0.05, 0.5))

X <- rLGCP("exp", 4, var=0.2, scale=0.1)
plot(attr(X, "Lambda"), main="log-Gaussian Cox process")
plot(X, add=TRUE, pch=16)

plot(rStrauss(200, 0.3, 0.07))
plot(rDiggleGratton(200,0.03,0.08))
plot(rDGS(300, 0.05))

plot(redwood, main="random thinning - rthin()")
points(rthin(redwood, 0.5), col="green", cex=1.4)

plot(rcell(nx=15))

plot(rsyst(nx=5))
abline(h=(1:4)/5, lty=2)
abline(v=(1:4)/5, lty=2)

plot(rstrat(nx=5))
abline(h=(1:4)/5, lty=2)
abline(v=(1:4)/5, lty=2)

X <- rsyst(nx=10)
plot(rjitter(X, 0.02))

Xg <- rmh(list(cif="geyer", par=list(beta=1.25, gamma=1.6, r=0.2, sat=4.5),
               w=c(0,10,0,10)),
          control=list(nrep=1e4), start=list(n.start=200))
plot(Xg, main=paste("Geyer saturation process\n",
                    "rmh() with cif=\"geyer\""))

L <- as.psp(matrix(runif(20), 5, 4), window=square(1))
plot(L, main="runifpointOnLines(30, L)")
plot(runifpointOnLines(30, L), add=TRUE, pch="+")

plot(L, main="rpoisppOnLines(3, L)")
plot(rpoisppOnLines(3, L), add=TRUE, pch="+")

plot(runiflpp(20, simplenet))
plot(rpoislpp(5, simplenet))

plot(rpoisline(10))

plot(rlinegrid(30, 0.1))

spatstat.options(npixel=256)
X <- dirichlet(runifpoint(30))
plot(rMosaicSet(X, 0.4), col="green", border=NA)
plot(X, add=TRUE)
plot(rMosaicField(X, runif))
plot(rMosaicSet(rpoislinetess(3), 0.5), col="green", border=NA, main="Switzer's random set")
spatstat.options(npixel=100)

plot(Halton(512, c(2,3)), main="quasirandom pattern")
plot(Halton(16384, c(2,3)), main="quasirandom pattern", pch=".")

fanfare("IX. Geometry")

A <- letterR

B <- shift(letterR, c(0.2,0.1))
plot(boundingbox(A,B), main="shift", type="n")
plot(A, add=TRUE)
plot(B, add=TRUE, border="red")

B <- rotate(letterR, 0.2)
plot(boundingbox(A,B), main="rotate", type="n")
plot(A, add=TRUE)
plot(B, add=TRUE, border="red")

mat <- matrix(c(1.1, 0, 0.3, 1), 2, 2)
B <- affine(letterR, mat=mat, vec=c(0.2,-0.1))
plot(boundingbox(A,B), main="affine", type="n")
plot(A, add=TRUE)
plot(B, add=TRUE, border="red")

par1x2 <- par(mfrow=c(1,2))
L <- rpoisline(10, owin(c(1.5,4.5),c(0.2,3.6)))
plot(L, main="Line segment pattern")
plot(L$window, main="L[window]", type="n")
plot(L[letterR], add=TRUE)
plot(letterR, add=TRUE, border="red")
par(par1x2)

a <- psp(runif(20),runif(20),runif(20),runif(20), window=owin())
plot(a, main="Self-crossing points")
plot(selfcrossing.psp(a), add=TRUE, col="red")

a <- as.psp(matrix(runif(20), 5, 4), window=square(1))
b <- rstrat(square(1), 5)
plot(a, lwd=3, col="green", main="project points to segments")
plot(b, add=TRUE, col="red", pch=16)
v <- project2segment(b, a)
Xproj <- v$Xproj
plot(Xproj, add=TRUE, pch=16)
arrows(b$x, b$y, Xproj$x, Xproj$y, angle=10, length=0.15, col="red")

plot(a, main="pointsOnLines(L)")
plot(pointsOnLines(a, np=100), add=TRUE, pch="+")

parry <- par(mfrow=c(1,3), mar=0.3+c(1,1,3,1))
X <- tess(xgrid=seq(2, 4, length=10), ygrid=seq(0, 3.5, length=8))
plot(X, cex.main=0.75)
plot(letterR, cex.main=0.75)
plot(intersect.tess(X, letterR), cex.main=0.75)

X <- dirichlet(runifpoint(10))
plot(X)
L <- infline(0.3,0.5)
plot(owin(), main="L", cex.main=0.75)
par(xpd=TRUE)
plot(L, col="red", lwd=2, cex.main=0.75)
plot(chop.tess(X,L), cex.main=0.75)
par(parry)

W <- chorley$window
plot(W, main="simplify.owin")
WS <- simplify.owin(W, 2)
plot(WS, add=TRUE, border="green")

nopa <- par(mfrow=c(2,2))
Rbox <- grow.rectangle(as.rectangle(letterR), 0.3)

v <- erosion.owin(letterR, 0.25)
plot(Rbox, type="n", main="erode.owin", cex.main=0.75)
plot(letterR, add=TRUE, col="red", cex.main=0.75)
plot(v, add=TRUE, col="blue")

v <- dilation.owin(letterR, 0.25)
plot(Rbox, type="n", main="dilate.owin", cex.main=0.75)
plot(v, add=TRUE, col="blue")
plot(letterR, add=TRUE, col="red")

v <- closing.owin(letterR, 0.3)
plot(Rbox, type="n", main="closing.owin", cex.main=0.75)
plot(v, add=TRUE, col="blue")
plot(letterR, add=TRUE, col="red")

v <- opening.owin(letterR, 0.3)
plot(Rbox, type="n", main="opening.owin", cex.main=0.75)
plot(letterR, add=TRUE, col="red")
plot(v, add=TRUE, col="blue")
par(nopa)

laslett(heather$fine, main="Laslett's Transform")

fanfare("X. Operations on pixel images")

Z <- distmap(swedishpines, dimyx=512)
plot(Z, main="An image Z")
plot(levelset(Z, 4))
plot(cut(Z, 5))
plot(eval.im(sqrt(Z) - 3))
plot(solutionset(abs(Z - 6) <= 1))
nopa <- par(mfrow=c(1,2))
plot(Z)
segments(0,0,96,100,lwd=2)
plot(transect.im(Z))
par(nopa)

d <- distmap(cells, dimyx=256)
W <- levelset(d, 0.06)
nopa <- par(mfrow=c(1,2))
plot(W)
plot(connected(W))
par(nopa)

Z <- as.im(function(x,y) { 4 * x^2 + 3 * y }, letterR)
plot(Z)
plot(letterR, add=TRUE)

plot(blur(Z, 0.3, bleed=TRUE))
plot(letterR, add=TRUE)

plot(blur(Z, 0.3, bleed=FALSE))
plot(letterR, add=TRUE)
          
plot(blur(Z, 0.3, bleed=FALSE))
plot(letterR, add=TRUE)
          
fanfare("XI. Programming tools")

showoffK <- function(Y, current, ..., fullpicture,rad) { 
	plot(fullpicture,
             main=c("Animation using `applynbd'", "explaining the K function"))
	points(Y, cex=2)
        ux <- current[["x"]]
        uy <- current[["y"]]
	points(ux,uy,pch="+",cex=3)
	theta <- seq(0,2*pi,length=100)
	polygon(ux + rad * cos(theta), uy + rad*sin(theta))
	text(ux+rad/3,uy+rad/2,Y$n,cex=3)
        if(runif(1) < 0.2) Sys.sleep(runif(1, max=0.4))
	return(npoints(Y))
}
par(ask=FALSE)
applynbd(redwood, R=0.2, showoffK, fullpicture=redwood, rad=0.2, exclude=TRUE)

par(oldpar)
options(oldoptions)

## demonstration of all summary functions

opa <- par(mfrow=c(1,1),
           mar=c(0,0,1,0)+0.2)

## Ripley's K-function
plot(swedishpines)

par(mar=c(4,4,2,1)+0.2)
plot(Kest(swedishpines))

## Besag's transformation
plot(Lest(swedishpines))

## pair correlation function
plot(pcf(swedishpines))

par(mfrow=c(2,3),
    mar=c(0,0,1,0)+0.2)
    
## Showing the utility of the K-function
plot(cells)
plot(nztrees)
plot(redwood)
par(mar=c(4,4,2,1)+0.2)
plot(Kest(cells))
plot(Kest(nztrees))
plot(Kest(redwood))
## Showing the utility of the pair correlation function
par(mar=c(0,0,1,0)+0.2)
plot(cells)
plot(nztrees)
plot(redwood)
par(mar=c(4,4,2,1)+0.2)
plot(pcf(cells))
plot(pcf(nztrees))
plot(pcf(redwood))
##
par(mfrow=c(1,1))

## Analogues for inhomogeneous patterns
## Reweighted K-function
plot(japanesepines)
fit <- ppm(japanesepines, ~polynom(x,y,2))
plot(predict(fit))
plot(Kinhom(japanesepines, fit))
plot(pcfinhom(japanesepines, fit))
plot(Linhom(japanesepines))

## Rescaled K-function
Bronze <- unmark(bronzefilter)
plot(Bronze)
plot(Kscaled(Bronze))
fit <- ppm(Bronze ~x)
plot(predict(fit))
plot(Bronze, add=TRUE)
plot(Kscaled(Bronze, fit))
plot(Lscaled(Bronze, fit))

## Local indicators of spatial association
plot(localL(swedishpines))
plot(localK(swedishpines))

## anisotropic
plot(Ksector(redwood, 0, 90))
plot(Rf <- pairorient(redwood, 0.05, 0.15))
rose(Rf, main="Rose diagram of pair orientation distribution")
plot(deriv(Rf, spar=0.6, Dperiodic=TRUE))
rose(nnorient(redwood))

##
par(mfrow=c(2,3),
    mar=rep(0.2, 4))
## Empty space function F
plot(cells)
plot(nztrees)
plot(redwood)
par(mar=c(4,4,2,1)+0.2)
plot(Fest(cells))
plot(Fest(nztrees))
plot(Fest(redwood))
## Nearest neighbour distance function G
par(mar=rep(0.2, 4))
plot(cells)
plot(nztrees)
plot(redwood)
par(mar=c(4,4,2,1)+0.2)
plot(Gest(cells))
plot(Gest(nztrees))
plot(Gest(redwood))
## J-function
par(mar=rep(0.2, 4))
plot(cells)
plot(nztrees)
plot(redwood)
par(mar=c(4,4,2,1)+0.2)
plot(Jest(cells))
plot(Jest(nztrees))
plot(Jest(redwood))
par(mfrow=c(1,1),
    mar=c(4,4,2,1)+0.2)

## versions for inhomogeneous patterns
plot(Finhom(japanesepines))
plot(Ginhom(japanesepines))
plot(Jinhom(japanesepines))

## Display F,G,J,K
plot(allstats(swedishpines))

## Multitype patterns
plot(amacrine)
plot(Kcross(amacrine))
plot(Kdot(amacrine))
I <- (marks(amacrine) == "on")
J <- (marks(amacrine) == "off")
plot(Kmulti(amacrine, I, J))

plot(alltypes(amacrine, "K"))

plot(Lcross(amacrine))
plot(Ldot(amacrine))

plot(pcfcross(amacrine))
plot(pcfdot(amacrine))
plot(pcfmulti(amacrine, I, J))

plot(Gcross(amacrine))
plot(Gdot(amacrine))
plot(Gmulti(amacrine, I, J))
plot(alltypes(amacrine, "G"))

plot(Jcross(amacrine))
plot(Jdot(amacrine))
plot(Jmulti(amacrine,I,J))
plot(alltypes(amacrine, "J"))

plot(alltypes(amacrine, "F"))

plot(Iest(amacrine))

plot(markconnect(amacrine))

## Multitype, inhomogeneous
plot(Kcross.inhom(amacrine))
plot(Kdot.inhom(amacrine))
plot(Kmulti.inhom(amacrine, I, J))
plot(Lcross.inhom(amacrine))
plot(Ldot.inhom(amacrine))

plot(pcfcross.inhom(amacrine))
plot(pcfdot.inhom(amacrine))
plot(pcfmulti.inhom(amacrine, I, J))	

## Numerical marks
plot(markcorr(longleaf))
plot(markvario(longleaf))
plot(Emark(longleaf))
plot(Vmark(longleaf))

## Linear networks
plot(chicago)
plot(linearK(chicago))
plot(linearKcross(chicago))
plot(linearKdot(chicago))
plot(linearpcf(chicago))
plot(linearpcfcross(chicago))
plot(linearpcfdot(chicago))

lam <- rep(intensity(unmark(chicago)), npoints(chicago))
A <- split(chicago)$assault
B <- split(chicago)$burglary
lamA <- rep(intensity(A), npoints(A))
lamB <- rep(intensity(B), npoints(B))
plot(linearKinhom(chicago, lam))
plot(linearKcross.inhom(chicago, "assault", "burglary", lamA, lamB))
plot(linearKdot.inhom(chicago, "assault", lamA, lam))
plot(linearpcfinhom(chicago, lam))
plot(linearpcfcross.inhom(chicago, "assault", "burglary", lamA, lamB))
plot(linearpcfdot.inhom(chicago, "assault", lamA, lam))

plot(linearmarkconnect(chicago))
plot(linearmarkequal(chicago))

rm(I,J,fit)

par(opa)

#'
#'        bugtable.R
#' 
#'    $Revision: 1.10 $ $Date: 2021/09/06 05:28:59 $

bugfixes <- function(sinceversion=NULL, sincedate=NULL,
                     package=spatstat.family(),
                     show=TRUE) {
  package <- as.character(package)
  is.spat <- package %in% spatstat.family(TRUE, TRUE)
  if("book" %in% list(sinceversion, sincedate)) {
    ## special usage
    sinceversion <- NULL
    sincedate    <- "2015-06-05"
  }
  n <- length(package)
  z <- NULL
  for(i in seq_len(n)) {
    packi <- package[i]
    if("all" %in% list(sinceversion, sincedate)) {
      ## no date constraint - show all news
      a <- eval(substitute(news(grepl("^BUG", Category),
                                package=packi)))
    } else if(!is.null(sincedate) && !is.spat[i]) {
      #' news items after specified date
      #' non-spatstat package
      ne <- news(package=packi)
      if(is.null(ne) || is.null(ne$Date) || anyNA(ne$Date))
        stop(paste(if(is.null(ne)) "News" else "Date",
                   "information is not available for package",
                   sQuote(packi)), call.=FALSE)
      a <- eval(substitute(news(Date >= SD & grepl("^BUG", Category),
                                package=packi),
                           list(SD=sincedate)))
    } else {
      #' determine a corresponding version number
      if(is.null(sinceversion) && is.null(sincedate)) {
        #' default is latest version
        dfile <- system.file("DESCRIPTION", package=packi)
        sinceversion <- read.dcf(file=dfile, fields="Version")
      } else if(!is.null(sincedate) && is.spat[i]) {
        #' read release history table
        fname <- system.file("doc", "packagesizes.txt", package=packi)
        p <- read.table(fname, header=TRUE, stringsAsFactors=FALSE)
        #' find earliest package version on or after the given date
        imin <- with(p, min(which(as.Date(date) >= sincedate)))
        sinceversion <- p[imin, "version"]
      }
      a <- eval(substitute(news(Version >= sv & grepl("^BUG", Category),
                                package=packi),
                           list(sv=sinceversion)))
    }
    #' convert format
    if(is.data.frame(a) && nrow(a) > 0) {
      #' split each entry into lines
      alines <- strsplit(a$Text, "\n")
      #' extract first line
      f <- unname(sapply(alines, "[", i=1L))
      #' extract body
      b <- unname(lapply(alines, "[", i=-1L))
      b <- unname(sapply(b, paste, collapse="\n"))
      #' extract header from first line
      h <- unname(sapply(strsplit(f, ":"), "[", i=1L))
      h <- unname(sapply(strsplit(h, ","), "[", i=1L))
      h <- unname(sapply(strsplit(h, " "), "[", i=1L))
      #' rebuild
      zi <- data.frame(Header=h,
                       Firstline=f,
                       Body=b,
                       Version=a$Version,
                       stringsAsFactors=FALSE)
      if(n > 1) zi$Package <- packi
      z <- if(i == 1) zi else rbind(z, zi)
    } 
  }
  if(is.null(z)) return(NULL)
  #' sort by header
  oo <- with(z, order(Header, Firstline))
  z <- z[oo, ]
  #' wrap up
  class(z) <- c("bugtable", class(z))
  if(show) page(z, method="print")
  return(invisible(z))
}

class(bugfixes) <- "autoexec"
              
print.bugtable <- function(x, ...) {
  hprev <- ""
  haspack <- "Package" %in% colnames(x)
  for(i in seq_len(nrow(x))) {
    h <- x$Header[i]
    f <- x$Firstline[i]
    v <- x$Version[i]
    p <- if(haspack) x$Package[i] else NULL
    b <- x$Body[i]
    if(h != hprev) {
      # new main header
      cat("\n***", h, "***\n", fill=TRUE)
    }
    if(haspack) cat(p, v, ":", f, fill=TRUE) else cat(v, ":", f, fill=TRUE)
    cat(b, "\n", fill=TRUE)
    hprev <- h
  }
  return(invisible(NULL))
}

##  spatstat/R/First.R

.onLoad <- function(...) {
  reset.spatstat.options()
  ##  umf <- system.file("doc", "umbrella.txt", package="spatstat")
  ##  isum <- !is.null(umf) && file.exists(umf)
  ##  putSpatstatVariable("Spatstat.Is.Umbrella", isum)
  putSpatstatVariable("Spatstat.Is.Umbrella", TRUE)
  invisible(NULL)
}
.onAttach <- function(libname, pkgname) {
  vs <- read.dcf(file=system.file("DESCRIPTION", package="spatstat"),
                 fields="Version")
  vs <- as.character(vs)
  putSpatstatVariable("SpatstatVersion", vs)
##  nickfile <- system.file("doc", "Nickname.txt", package="spatstat")
##  ni <- scan(file=nickfile, what=character(), n=1, quiet=TRUE)
  msg <- paste("\nspatstat", vs,
##               "     ",
##               paren(paste("nickname:", sQuote(ni))),
               "\nFor an introduction to spatstat, type",
               sQuote("beginner"), "\n")
  packageStartupMessage(msg)
  return(invisible(NULL))
}

  

#
#  beginner.R
#
# Helpful information for beginners
#
#  $Revision: 1.3 $  $Date: 2015/10/21 09:06:57 $
#

print.autoexec <- function(x, ...) { x() }

beginner <- function(package="spatstat") {
  package <- as.character(substitute(package))
  RShowDoc("BEGINNER.txt", type="txt", package=package)
  return(invisible(NULL))
}

class(beginner) <- "autoexec"

foo <- local({
  fooText <- paste0("Error: object 'foo' not found.\n\n",
                    "'foo' is not a defined variable or function.\n",
                    "It is a placeholder name, which serves only to ",
                    "demonstrate a concept. It represents the name of ",
                    "any desired object or function. ", 
                    "Other placeholder names popular with computer scientists ",
                    "are 'bar', 'foobar', 'qux' and 'mork'.")

  foo <- function() {
    splat(fooText) 
    return(invisible(NULL))
  }
  class(foo) <- "autoexec"
  foo
})

plot.foo <- function(x, ...) foo()

#
# news.R
#
#  News and warnings
#

latest.news <- function(package=spatstat.family(), doBrowse=FALSE, major=TRUE) {
  stopifnot(is.character(package))
  ## news from spatstat.core is subsumed in spatstat.explore + spatstat.model
  defunctpackages <- "spatstat.core"
  package <- setdiff(package, defunctpackages)
  n <- length(package)
  result <- vector(mode="list", length=n)
  names(result) <- package
  for(i in seq_len(n)) {
    pack <- package[i]
    ## get version number
    v <- read.dcf(file=system.file("DESCRIPTION", package=pack),
                  fields="Version")
    if(major) {
      ## the current major version
      vp <- package_version(v)
      vv <- unlist(vp)
      v <- paste0(vv[1], ".", vv[2])
    }
    ne <- eval(substitute(news(Version >= v0, package=pack), list(v0=v)))
    if(n > 1 && !doBrowse) {
      hdr <- paste0("Package ", sQuote(pack), ":")
      if(interactive()) readline(hdr) else cat(paste(hdr, "\n"))
    }
    page(ne, method="print", doBrowse=doBrowse)
    result[[i]] <- ne
  }
  if(n == 1) result <- result[[1]]
  return(invisible(result))
}

class(latest.news) <- "autoexec"

spatstat.family <- function(subpackages=TRUE, extensions=FALSE) {
  sub <- c("spatstat.utils",
           "spatstat.data",
           "spatstat.univar",
           "spatstat.sparse",
           "spatstat.geom",
           "spatstat.random",
           "spatstat.explore",
           "spatstat.model",
           "spatstat.linnet",
           "spatstat")
  ext <- c("spatstat.gui", "spatstat.local", "spatstat.Knet")
  result <- c(if(subpackages) sub else NULL,
              if(extensions) ext else NULL)
  as.character(result)
}

#'
#'        changes.R
#' 
#'    $Revision: 1.3 $ $Date: 2024/09/21 23:46:42 $

latest.changes <- function(x,
                           sinceversion=NULL, sincedate=NULL,
                           package=spatstat.family(),
                           show=TRUE) {
  x <- as.character(x)
  package <- as.character(package)
  is.spat <- package %in% spatstat.family(TRUE, TRUE)
  if("book" %in% list(sinceversion, sincedate)) {
    ## special usage
    sinceversion <- NULL
    sincedate    <- "2015-06-05"
  }
  n <- length(package)
  z <- NULL
  for(i in seq_len(n)) {
    packi <- package[i]
    if("all" %in% list(sinceversion, sincedate)) {
      ## no date constraint - show all news
      a <- eval(substitute(news(grepl("^SIGN", Category),
                                package=packi)))
    } else if(!is.null(sincedate) && !is.spat[i]) {
      #' news items after specified date
      #' non-spatstat package
      ne <- news(package=packi)
      if(is.null(ne) || is.null(ne$Date) || anyNA(ne$Date))
        stop(paste(if(is.null(ne)) "News" else "Date",
                   "information is not available for package",
                   sQuote(packi)), call.=FALSE)
      a <- eval(substitute(news(Date >= SD & grepl("^SIGN", Category),
                                package=packi),
                           list(SD=sincedate)))
    } else {
      #' determine a corresponding version number
      if(is.null(sinceversion) && is.null(sincedate)) {
        #' default is latest version
        dfile <- system.file("DESCRIPTION", package=packi)
        sinceversion <- read.dcf(file=dfile, fields="Version")
      } else if(!is.null(sincedate) && is.spat[i]) {
        #' read release history table
        fname <- system.file("doc", "packagesizes.txt", package=packi)
        p <- read.table(fname, header=TRUE, stringsAsFactors=FALSE)
        #' find earliest package version on or after the given date
        imin <- with(p, min(which(as.Date(date) >= sincedate)))
        sinceversion <- p[imin, "version"]
      }
      a <- eval(substitute(news(Version >= sv & grepl("^SIGN", Category),
                                package=packi),
                           list(sv=sinceversion)))
    }
    #' convert format
    if(is.data.frame(a) && nrow(a) > 0) {
      #' split each entry into lines
      alines <- strsplit(a$Text, "\n")
      #' extract first line
      f <- unname(sapply(alines, "[", i=1L))
      #' extract body
      b <- unname(lapply(alines, "[", i=-1L))
      b <- unname(sapply(b, paste, collapse="\n"))
      #' extract header from first line
      h <- unname(sapply(strsplit(f, ":"), "[", i=1L))
      h <- unname(sapply(strsplit(h, ","), "[", i=1L))
      h <- unname(sapply(strsplit(h, " "), "[", i=1L))
      #' rebuild
      zi <- data.frame(Header=h,
                       Firstline=f,
                       Body=b,
                       Version=a$Version,
                       stringsAsFactors=FALSE)
      if(n > 1) zi$Package <- packi
      z <- if(i == 1) zi else rbind(z, zi)
    } 
  }
  if(is.null(z)) return(NULL)
  #' sort by header
  oo <- with(z, order(Header, Firstline))
  z <- z[oo, ]
  #' finally filter items mentioning 'x'
  hit <- Reduce("|", lapply(x, function(s) with(z, grepl(s, Firstline))))
  z <- z[hit,]
  #' wrap up
  class(z) <- c("changetable", class(z))
  if(show) page(z, method="print")
  return(invisible(z))
}

print.changetable <- function(x, ...) {
  hprev <- ""
  haspack <- "Package" %in% colnames(x)
  for(i in seq_len(nrow(x))) {
    h <- x$Header[i]
    f <- x$Firstline[i]
    v <- x$Version[i]
    p <- if(haspack) x$Package[i] else NULL
    b <- x$Body[i]
    if(h != hprev) {
      # new main header
      cat("\n***", h, "***\n", fill=TRUE)
    }
    if(haspack) cat(p, v, ":", f, fill=TRUE) else cat(v, ":", f, fill=TRUE)
    cat(b, "\n", fill=TRUE)
    hprev <- h
  }
  return(invisible(NULL))
}

##
##   Math.im.R
##
##   $Revision: 1.8 $ $Date: 2020/05/08 06:09:14 $
##
Math.im <- function(x, ...){
    m <- do.call(.Generic, list(x$v, ...))
    rslt <- im(m, xcol = x$xcol, yrow = x$yrow, xrange = x$xrange,
               yrange = x$yrange, unitname = unitname(x))
    return(rslt)
}

Summary.im <- function(..., na.rm=FALSE, drop=TRUE){
  argh <- list(...)
  ims <- sapply(argh, is.im)
  argh[ims] <- lapply(argh[ims], getElement, name="v")
  do.call(.Generic, c(argh, list(na.rm = na.rm || drop)))
}

Complex.im <- function(z){
    m <- do.call(.Generic, list(z=z$v))
    rslt <- im(m, xcol = z$xcol, yrow = z$yrow, xrange = z$xrange,
               yrange = z$yrange, unitname = unitname(z))
    return(rslt)
}

## The following function defines what happens in Ops.im
## but the formal 'Ops' method is now in Ops.im.R

imageOp <- function(e1, e2=NULL, op) {
  ## operate on an image or pair of images
  if(is.null(e2)) {
    ## unary operation
    if(!is.element(op, c("!", "-", "+")))
      stop(paste("Unary operation", sQuote(op), "is undefined for images"),
           call.=FALSE)
    expr <- parse(text = paste(op, "e1"))
  } else {
    expr <- parse(text = paste("e1", op, "e2"))
  }
  return(do.call(eval.im, list(expr = expr)))
}                  

#'
#'   polartess.R
#'
#'   Tessellation using polar coordinates
#' 
#'   $Revision: 1.4 $  $Date: 2019/03/16 05:36:40 $

polartess <- function(W, ..., nradial=NULL, nangular=NULL,
                      radii=NULL, angles=NULL, origin=NULL,
                      sep="x") {
  trap.extra.arguments(...)
  W <- as.owin(W)
  if(!is.null(origin)) {
    origin <- interpretAsOrigin(origin, W)
    W <- shift(W, -origin)
  }

  V <- vertices(Frame(W))
  rmax <- sqrt(max(V$x^2 + V$y^2))

  if(!is.null(radii)) {
    if(!is.null(nradial))
      warning("nradial ignored because radii were specified")
    radii <- as.numeric(radii)
    stopifnot(length(radii) >= 2)
    stopifnot(all(radii >= 0))
    if(sum(is.infinite(radii)) > 1 || !all(diff(radii) > 0))
      stop("radii should be increasing")
    radnames <- paste(signif(radii, 4))
    radii[is.infinite(radii)] <- 1.01 * rmax
    rmax <- max(radii)
    nradial <- length(radii) - 1L
  } else if(!is.null(nradial)) {
    check.1.integer(nradial)
    radii <- seq(0, rmax, length.out=nradial+1L)
    radnames <- paste(signif(radii, 4))
  }
  nradii <- length(radii)
  
  if(!is.null(angles)) {
    if(!is.null(nangular))
      warning("nangular ignored because angles were specified")
    angles <- as.numeric(angles)
    stopifnot(length(angles) >= 2)
    if(!all(diff(angles) > 0))
      stop("angles should be increasing")
    if(diff(range(angles)) > 2 * pi + .Machine$double.eps)
      stop("The range of angles must not exceed 2 * pi")
    nangular <- length(angles) - 1L
  } else if(!is.null(nangular)) {
    check.1.integer(nangular)
    angles <- seq(0, 2*pi, length.out=nangular+1L)
  }
  nangles <- length(angles)
  
  #' build tessellations
  result <- as.tess(W)
  DD <- Dmax <- disc(rmax)
  
  if(!is.null(radii)) {
    rmin <- radii[1]
    if(rmin > 0) DD <- setminus.owin(DD, disc(rmin))
    Dlist <- lapply(radii[radii > 0], disc)
    if(rmin == 0) Dlist <- append(list(NULL), Dlist)
    Tlist <- list()
    for(i in 1:nradial) 
      Tlist <- append(Tlist, list(setminus.owin(Dlist[[i+1]], Dlist[[i]])))
    names(Tlist) <- paste0("[", radnames[-nradii], ", ", radnames[-1L],
                           c(rep(")", nradial-1L), "]"))
    Rtess <- tess(tiles=Tlist, window=DD)
    result <- intersect.tess(result, Rtess, sep=sep)
  } 
  
  if(!is.null(angles)) {
    Tlist <- list()
    aa <- seq(min(angles), max(angles), length.out=256)
    aa <- sort(c(aa, angles))
    xx <- rmax * cos(aa)
    yy <- rmax * sin(aa)
    for(i in 1:nangular) {
      jj <- (aa >= angles[i]) & (aa <= angles[i+1L])
      Tlist[[i]] <- owin(poly=list(x=c(0, xx[jj]),
                                   y=c(0, yy[jj])))
    }
    angnames <- lapply(angles/pi, simplenumber, unit="pi", multiply="")
    unknown <- sapply(angnames, is.null)
    angnames[unknown] <- paste(signif((angles/pi)[unknown], 4), "pi")
    angnames <- unlist(angnames)
    names(Tlist) <- paste0("[", angnames[-nangles], ", ", angnames[-1L],
                           c(rep(")", nangular-1L), "]"))
    gap <- abs(1 - diff(range(angles))/(2*pi))
    DDD <- if(gap < 0.01) Dmax else owin(poly=list(x=c(0, xx), y=c(0,yy)))
    Atess <- tess(tiles=Tlist, window=DDD)
    result <- intersect.tess(result, Atess, sep=sep)
  }
  if(!is.null(origin))
    result <- shift(result, vec=origin)
  return(result)
}

#
#    discarea.R
#
#  $Revision: 1.21 $  $Date: 2019/02/20 03:34:50 $
#
#
#  Compute area of intersection between a disc and a window,
#
discpartarea <- function(X, r, W=as.owin(X)) {
  if(!missing(W)) {
    verifyclass(W, "owin")
    if(!inherits(X, "ppp"))
      X <- as.ppp(X, W)
  }
  verifyclass(X, "ppp")

  n <- X$n
  if(is.matrix(r) && nrow(r) != n)
    stop("the number of rows of r should match the number of points in X")
  if(!is.matrix(r)) {
    nr <- length(r)
    r <- matrix(r, nrow=n, ncol=nr, byrow=TRUE)
  } else {
    nr <- ncol(r)
  }
  
  W <- as.polygonal(W)
  
  # convert polygon to line segments
  Y <- edges(W)
  # remove vertical segments (contribution is zero)
  vert <- (Y$ends$x1 == Y$ends$x0)
  Y <- Y[!vert]
  # go
  z <- .C("discareapoly",
          nc=as.integer(n),
          xc=as.double(X$x),
          yc=as.double(X$y),
          nr=as.integer(nr),
          rmat=as.double(r),
          nseg=as.integer(Y$n),
          x0=as.double(Y$ends$x0),
          y0=as.double(Y$ends$y0),
          x1=as.double(Y$ends$x1),
          y1=as.double(Y$ends$y1),
          eps=as.double(.Machine$double.eps),
          out=as.double(numeric(length(r))),
          PACKAGE = "spatstat")
  areas <- matrix(z$out, n, nr)
  return(areas)
}

# Compute area of dilation of point pattern
# using Dirichlet tessellation or distmap
#  (areas of other dilations using distmap)

dilated.areas <- function(X, r, W=as.owin(X), ...,
                          constrained=TRUE,
                          exact=FALSE) {
  if(is.matrix(r)) {
    if(sum(dim(r) > 1) > 1L)
      stop("r should be a vector or single value")
    r <- as.vector(r)
  }
  if(exact && !is.ppp(X)) {
    exact <- FALSE
    warning("Option exact=TRUE is only available for ppp objects")
  }
  if(!constrained || is.null(W)) {
    # unconstrained dilation
    bb <- as.rectangle(X)
    W <- grow.rectangle(bb, max(r))
    if(is.owin(X))
      X <- rebound.owin(X, W)
    else
      X$window <- W
  } else W <- as.owin(W)
  if(!exact) {
    D <- distmap(X, ...)
    pixelarea <- D$xstep * D$ystep
    Dvals <- D[W, drop=TRUE]
    if(is.im(Dvals))
      Dvals <- as.vector(as.matrix(Dvals))
    Dvals <- Dvals[!is.na(Dvals)]
    rr <- c(-1, r)
    h <- cumsum(whist(Dvals, rr))
    return(h * pixelarea)
  }
  npts <- npoints(X)
  nr <- length(r)
  if(npts == 0)
    return(numeric(nr))
  else if(npts == 1L) 
    return(discpartarea(X, r, W))
  samebox <- (W$type == "rectangle") &&
              isTRUE(all.equal(W, as.owin(X)))
  needclip <- constrained && !samebox
  X <- unique(X)
  dd <- dirichlet(X)
  til <- tiles(dd)
  #' some data points may not have a tile
  whichpoint <- as.integer(names(til))
  partareas <- matrix(0, length(til), nr)
  for(j in seq_along(til)) {
    Tj <- til[[j]]
    if(needclip)
      Tj <- intersect.owin(Tj, W)
    i <- whichpoint[j]
    partareas[j,] <- discpartarea(X[i], r, Tj)
  }
  return(colSums(partareas))
}

  

##
## convexify.R
##
## $Revision: 1.2 $ $Date: 2020/03/16 10:28:51 $

convexify <- function(W, eps) {
  if(!is.polygonal(W)) {
    if(missing(eps)) eps <- diameter(Frame(W))/20
    W <- simplify.owin(W, eps)
  }
  e <- edges(W)
  len <- lengths_psp(e)
  ang <- angles.psp(e, directed=TRUE)
  df <- data.frame(ang=ang, len=len)
  df <- df[order(df$ang), ]
  df <- within(df, { dx <- len * cos(ang); dy <- len * sin(ang)})
  owin(poly=with(df, list(x=cumsum(c(0,dx)), y=cumsum(c(0,dy)))))
}

    

#
# closepairs.R
#
#   $Revision: 1.44 $   $Date: 2019/10/28 08:48:47 $
#
#  simply extract the r-close pairs from a dataset
# 
#  Less memory-hungry for large patterns
#

closepairs <- function(X, rmax, ...) {
  UseMethod("closepairs")
}
  
closepairs.ppp <- function(X, rmax, twice=TRUE,
                           what=c("all", "indices", "ijd"),
                           distinct=TRUE, neat=TRUE,
                           periodic=FALSE,
                           ...) {
  verifyclass(X, "ppp")
  what <- match.arg(what)
  stopifnot(is.numeric(rmax) && length(rmax) == 1L)
  stopifnot(is.finite(rmax))
  stopifnot(rmax >= 0)
  ordered <- list(...)$ordered
  if(missing(twice) && !is.null(ordered)) {
    warning("Obsolete argument 'ordered' has been replaced by 'twice'")
    twice <- ordered
  }
  if(periodic && !is.rectangle(Window(X)))
    warning("Periodic edge correction applied in non-rectangular window",
            call.=FALSE)
  
  npts <- npoints(X)
  null.answer <- switch(what,
                        all = {
                          list(i=integer(0),
                               j=integer(0),
                               xi=numeric(0),
                               yi=numeric(0),
                               xj=numeric(0),
                               yj=numeric(0),
                               dx=numeric(0),
                               dy=numeric(0),
                               d=numeric(0))
                        },
                        indices = {
                          list(i=integer(0),
                               j=integer(0))
                        },
                        ijd = {
                          list(i=integer(0),
                               j=integer(0),
                               d=numeric(0))
                        })
  if(npts == 0)
    return(null.answer)
  ## sort points by increasing x coordinate
  if(!periodic) {
    oo <- fave.order(X$x)
    Xsort <- X[oo]
  } 
  ## First make an OVERESTIMATE of the number of unordered pairs
  nsize <- list(...)$nsize # secret option to test memory overflow code
  if(!is.null(nsize)) {
    splat("Using nsize =", nsize)
  } else {
    #' normal usage 
    nsize <- ceiling(2 * pi * (npts^2) * (rmax^2)/area(Window(X)))
    nsize <- max(1024, nsize)
    if(nsize > .Machine$integer.max) {
     warning("Estimated number of close pairs exceeds maximum possible integer",
              call.=FALSE)
      nsize <- .Machine$integer.max
    }
  }
  ## Now extract pairs
  if(periodic) {
    ## special algorithm for periodic distance
    got.twice <- TRUE
    x <- X$x
    y <- X$y
    r <- rmax
    p <- sidelengths(Frame(X))
    ng <- nsize
    storage.mode(x) <- "double"
    storage.mode(y) <- "double"
    storage.mode(r) <- "double"
    storage.mode(p) <- "double"
    storage.mode(ng) <- "integer"
    z <- .Call("closePpair",
               xx=x,
               yy=y,
               pp=p,
               rr=r,
               nguess=ng,
               PACKAGE="spatstat")
    i <- z[[1L]]
    j <- z[[2L]]
    d <- z[[3L]]
    if(what == "all") {
      xi <- x[i]
      yi <- y[i]
      xj <- x[j]
      yj <- y[j]
      dx <- xj - xi
      dy <- yj - yi
    }
  } else if(spatstat.options("closepairs.newcode")) {
    # ------------------- use new faster code ---------------------
    # fast algorithms collect each distinct pair only once
    got.twice <- FALSE
    ng <- nsize
    #
    x <- Xsort$x
    y <- Xsort$y
    r <- rmax
    storage.mode(x) <- "double"
    storage.mode(y) <- "double"
    storage.mode(r) <- "double"
    storage.mode(ng) <- "integer"
    switch(what,
           all = {
             z <- .Call("Vclosepairs",
                        xx=x, yy=y, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 9)
               stop("Internal error: incorrect format returned from Vclosepairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
             xi <- z[[3L]]
             yi <- z[[4L]]
             xj <- z[[5L]]
             yj <- z[[6L]]
             dx <- z[[7L]]
             dy <- z[[8L]]
             d  <- z[[9L]]
           },
           indices = {
             z <- .Call("VcloseIJpairs",
                        xx=x, yy=y, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 2)
               stop("Internal error: incorrect format returned from VcloseIJpairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
           },
           ijd = {
             z <- .Call("VcloseIJDpairs",
                        xx=x, yy=y, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 3)
               stop("Internal error: incorrect format returned from VcloseIJDpairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
             d  <- z[[3L]]
           })

  } else if(spatstat.options("closepairs.altcode")) {
    #' experimental alternative code
    got.twice <- FALSE
    ng <- nsize
    #
    x <- Xsort$x
    y <- Xsort$y
    r <- rmax
    storage.mode(x) <- "double"
    storage.mode(y) <- "double"
    storage.mode(r) <- "double"
    storage.mode(ng) <- "integer"
    switch(what,
           all = {
             z <- .Call("altVclosepairs",
                        xx=x, yy=y, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 9)
               stop("Internal error: incorrect format returned from altVclosepairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
             xi <- z[[3L]]
             yi <- z[[4L]]
             xj <- z[[5L]]
             yj <- z[[6L]]
             dx <- z[[7L]]
             dy <- z[[8L]]
             d  <- z[[9L]]
           },
           indices = {
             z <- .Call("altVcloseIJpairs",
                        xx=x, yy=y, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 2)
               stop("Internal error: incorrect format returned from altVcloseIJpairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
           },
           ijd = {
             z <- .Call("altVcloseIJDpairs",
                        xx=x, yy=y, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 3)
               stop("Internal error: incorrect format returned from altVcloseIJDpairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
             d  <- z[[3L]]
           })

  } else {
    # ------------------- use older code --------------------------
    if(!distinct) {
      ii <- seq_len(npts)
      xx <- X$x
      yy <- X$y
      zeroes <- rep(0, npts)
      null.answer <- switch(what,
                            all = {
                              list(i=ii,
                                   j=ii,
                                   xi=xx,
                                   yi=yy,
                                   xj=xx,
                                   yj=yy,
                                   dx=zeroes,
                                   dy=zeroes,
                                   d=zeroes)
                            },
                            indices = {
                              list(i=ii,
                                   j=ii)
                            },
                            ijd = {
                              list(i=ii,
                                   j=ii,
                                   d=zeroes)
                            })
    }

    got.twice <- TRUE
    nsize <- nsize * 2
    z <-
      .C("Fclosepairs",
         nxy=as.integer(npts),
         x=as.double(Xsort$x),
         y=as.double(Xsort$y),
         r=as.double(rmax),
         noutmax=as.integer(nsize), 
         nout=as.integer(integer(1L)),
         iout=as.integer(integer(nsize)),
         jout=as.integer(integer(nsize)), 
         xiout=as.double(numeric(nsize)),
         yiout=as.double(numeric(nsize)),
         xjout=as.double(numeric(nsize)),
         yjout=as.double(numeric(nsize)),
         dxout=as.double(numeric(nsize)),
         dyout=as.double(numeric(nsize)),
         dout=as.double(numeric(nsize)),
         status=as.integer(integer(1L)),
         PACKAGE = "spatstat")

    if(z$status != 0) {
      # Guess was insufficient
      # Obtain an OVERCOUNT of the number of pairs
      # (to work around gcc bug #323)
      rmaxplus <- 1.25 * rmax
      nsize <- .C("paircount",
                  nxy=as.integer(npts),
                  x=as.double(Xsort$x),
                  y=as.double(Xsort$y),
                  rmaxi=as.double(rmaxplus),
                  count=as.integer(integer(1L)),
                  PACKAGE = "spatstat")$count
      if(nsize <= 0)
        return(null.answer)
      # add a bit more for safety
      nsize <- ceiling(1.1 * nsize) + 2 * npts
      # now extract points
      z <-
        .C("Fclosepairs",
           nxy=as.integer(npts),
           x=as.double(Xsort$x),
           y=as.double(Xsort$y),
           r=as.double(rmax),
           noutmax=as.integer(nsize), 
           nout=as.integer(integer(1L)),
           iout=as.integer(integer(nsize)),
           jout=as.integer(integer(nsize)), 
           xiout=as.double(numeric(nsize)),
           yiout=as.double(numeric(nsize)),
           xjout=as.double(numeric(nsize)),
           yjout=as.double(numeric(nsize)),
           dxout=as.double(numeric(nsize)),
           dyout=as.double(numeric(nsize)),
           dout=as.double(numeric(nsize)),
           status=as.integer(integer(1L)),
           PACKAGE = "spatstat")
      if(z$status != 0)
        stop(paste("Internal error: C routine complains that insufficient space was allocated:", nsize))
    }
  # trim vectors to the length indicated
    npairs <- z$nout
    if(npairs <= 0)
      return(null.answer)
    actual <- seq_len(npairs)
    i  <- z$iout[actual]  # sic
    j  <- z$jout[actual]
    switch(what,
           indices={},
           all={
             xi <- z$xiout[actual]
             yi <- z$yiout[actual]
             xj <- z$xjout[actual]
             yj <- z$yjout[actual]
             dx <- z$dxout[actual]
             dy <- z$dyout[actual]
             d <-  z$dout[actual]
           },
           ijd = {
             d <- z$dout[actual]
           })
    # ------------------- end code switch ------------------------
  }
  
  if(!periodic) {
    ## convert i,j indices to original sequence
    i <- oo[i]
    j <- oo[j]
  }
  
  if(twice) {
    ## both (i, j) and (j, i) should be returned
    if(!got.twice) {
      ## duplication required
      iold <- i
      jold <- j
      i <- c(iold, jold)
      j <- c(jold, iold)
      switch(what,
             indices = { },
             ijd = {
               d <- rep(d, 2)
             },
             all = {
               xinew <- c(xi, xj)
               yinew <- c(yi, yj)
               xjnew <- c(xj, xi)
               yjnew <- c(yj, yi)
               xi <- xinew
               yi <- yinew
               xj <- xjnew
               yj <- yjnew
               dx <- c(dx, -dx)
               dy <- c(dy, -dy)
               d <- rep(d, 2)
             })
    }
  } else {
    ## only one of (i, j) and (j, i) should be returned
    if(got.twice) {
      ## remove duplication
      ok <- (i < j)
      i  <-  i[ok]
      j  <-  j[ok]
      switch(what,
             indices = { },
             all = {
               xi <- xi[ok]
               yi <- yi[ok]
               xj <- xj[ok]
               yj <- yj[ok]
               dx <- dx[ok]
               dy <- dy[ok]
               d  <-  d[ok]
             },
             ijd = {
               d  <-  d[ok]
             })
    } else if(neat) {
      ## enforce i < j
      swap <- (i > j)
      tmp <- i[swap]
      i[swap] <- j[swap]
      j[swap] <- tmp
      if(what == "all") {
        xinew <- ifelse(swap, xj, xi)
        yinew <- ifelse(swap, yj, yi)
        xjnew <- ifelse(swap, xi, xj)
        yjnew <- ifelse(swap, yi, yj)
        xi <- xinew
        yi <- yinew
        xj <- xjnew
        yj <- yjnew
        dx[swap] <- -dx[swap]
        dy[swap] <- -dy[swap]
      }
    } ## otherwise no action required
  }
  ## add pairs of identical points?
  if(!distinct) {
    ii <- seq_len(npts)
    xx <- X$x
    yy <- X$y
    zeroes <- rep(0, npts)
    i <- c(i, ii)
    j <- c(j, ii)
    switch(what,
           ijd={
             d  <- c(d, zeroes)
           },
           all = {
             xi <- c(xi, xx)
             yi <- c(yi, yy)
             xj <- c(xj, xx)
             yi <- c(yi, yy)
             dx <- c(dx, zeroes)
             dy <- c(dy, zeroes)
             d  <- c(d, zeroes)
           })
  }
  ## done
  switch(what,
         all = {
           answer <- list(i=i,
                          j=j,
                          xi=xi, 
                          yi=yi,
                          xj=xj,
                          yj=yj,
                          dx=dx,
                          dy=dy,
                          d=d)
         },
         indices = {
           answer <- list(i = i, j = j)
         },
         ijd = {
           answer <- list(i=i, j=j, d=d)
         })
  return(answer)
}

#######################

crosspairs <- function(X, Y, rmax, ...) {
  UseMethod("crosspairs")
}

crosspairs.ppp <- function(X, Y, rmax, what=c("all", "indices", "ijd"), ...) {
  verifyclass(X, "ppp")
  verifyclass(Y, "ppp")
  what <- match.arg(what)
  stopifnot(is.numeric(rmax) && length(rmax) == 1L && rmax >= 0)
  null.answer <- switch(what,
                        all = {
                          list(i=integer(0),
                               j=integer(0),
                               xi=numeric(0),
                               yi=numeric(0),
                               xj=numeric(0),
                               yj=numeric(0),
                               dx=numeric(0),
                               dy=numeric(0),
                               d=numeric(0))
                        },
                        indices = {
                          list(i=integer(0),
                               j=integer(0))
                        },
                        ijd = {
                          list(i=integer(0),
                               j=integer(0),
                               d=numeric(0))
                        })
  nX <- npoints(X)
  nY <- npoints(Y)
  if(nX == 0 || nY == 0) return(null.answer)
  # order patterns by increasing x coordinate
  ooX <- fave.order(X$x)
  Xsort <- X[ooX]
  ooY <- fave.order(Y$x)
  Ysort <- Y[ooY]
  if(spatstat.options("crosspairs.newcode")) {
    # ------------------- use new faster code ---------------------
    # First (over)estimate the number of pairs
    nsize <- ceiling(2 * pi * (rmax^2) * nX * nY/area(Window(Y)))
    nsize <- max(1024, nsize)
    if(nsize > .Machine$integer.max) {
      warning(
        "Estimated number of close pairs exceeds maximum possible integer",
        call.=FALSE)
      nsize <- .Machine$integer.max
    }
    # .Call
    Xx <- Xsort$x
    Xy <- Xsort$y
    Yx <- Ysort$x
    Yy <- Ysort$y
    r <- rmax
    ng <- nsize
    storage.mode(Xx) <- storage.mode(Xy) <- "double"
    storage.mode(Yx) <- storage.mode(Yy) <- "double"
    storage.mode(r) <- "double"
    storage.mode(ng) <- "integer"
    switch(what,
           all = {
             z <- .Call("Vcrosspairs",
                        xx1=Xx, yy1=Xy,
                        xx2=Yx, yy2=Yy,
                        rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 9)
               stop("Internal error: incorrect format returned from Vcrosspairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
             xi <- z[[3L]]
             yi <- z[[4L]]
             xj <- z[[5L]]
             yj <- z[[6L]]
             dx <- z[[7L]]
             dy <- z[[8L]]
             d  <- z[[9L]]
           },
           indices = {
             z <- .Call("VcrossIJpairs",
                        xx1=Xx, yy1=Xy,
                        xx2=Yx, yy2=Yy,
                        rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 2)
               stop("Internal error: incorrect format returned from VcrossIJpairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
           }, 
           ijd = {
             z <- .Call("VcrossIJDpairs",
                        xx1=Xx, yy1=Xy,
                        xx2=Yx, yy2=Yy,
                        rr=r, nguess=ng,
                        PACKAGE = "spatstat")
             if(length(z) != 3)
               stop("Internal error: incorrect format returned from VcrossIJDpairs")
             i  <- z[[1L]]  # NB no increment required
             j  <- z[[2L]]
             d  <- z[[3L]]
           })
           
  } else {
    # Older code 
    # obtain upper estimate of number of pairs
    # (to work around gcc bug 323)
    rmaxplus <- 1.25 * rmax
    nsize <- .C("crosscount",
                nn1=as.integer(X$n),
                x1=as.double(Xsort$x),
                y1=as.double(Xsort$y),
                nn2=as.integer(Ysort$n),
                x2=as.double(Ysort$x),
                y2=as.double(Ysort$y),
                rmaxi=as.double(rmaxplus),
                count=as.integer(integer(1L)),
                PACKAGE = "spatstat")$count
    if(nsize <= 0)
      return(null.answer)

    # allow slightly more space to work around gcc bug #323
    nsize <- ceiling(1.1 * nsize) + X$n + Y$n
    
    # now extract pairs
    z <-
      .C("Fcrosspairs",
         nn1=as.integer(X$n),
         x1=as.double(Xsort$x),
         y1=as.double(Xsort$y),
         nn2=as.integer(Y$n),
         x2=as.double(Ysort$x),
         y2=as.double(Ysort$y),
         r=as.double(rmax),
         noutmax=as.integer(nsize), 
         nout=as.integer(integer(1L)),
         iout=as.integer(integer(nsize)),
         jout=as.integer(integer(nsize)), 
         xiout=as.double(numeric(nsize)),
         yiout=as.double(numeric(nsize)),
         xjout=as.double(numeric(nsize)),
         yjout=as.double(numeric(nsize)),
         dxout=as.double(numeric(nsize)),
         dyout=as.double(numeric(nsize)),
         dout=as.double(numeric(nsize)),
         status=as.integer(integer(1L)),
         PACKAGE = "spatstat")
    if(z$status != 0)
      stop(paste("Internal error: C routine complains that insufficient space was allocated:", nsize))
    # trim vectors to the length indicated
    npairs <- z$nout
    if(npairs <= 0)
      return(null.answer)
    actual <- seq_len(npairs)
    i  <- z$iout[actual] # sic
    j  <- z$jout[actual] 
    xi <- z$xiout[actual]
    yi <- z$yiout[actual]
    xj <- z$xjout[actual]
    yj <- z$yjout[actual]
    dx <- z$dxout[actual]
    dy <- z$dyout[actual]
    d <-  z$dout[actual]
  }
  # convert i,j indices to original sequences
  i <- ooX[i]
  j <- ooY[j]
  # done
  switch(what,
         all = {
           answer <- list(i=i,
                          j=j,
                          xi=xi, 
                          yi=yi,
                          xj=xj,
                          yj=yj,
                          dx=dx,
                          dy=dy,
                          d=d)
         },
         indices = {
           answer <- list(i=i, j=j)
         },
         ijd = {
           answer <- list(i=i, j=j, d=d)
         })
  return(answer)
}

closethresh <- function(X, R, S, twice=TRUE, ...) {
  # list all R-close pairs
  # and indicate which of them are S-close (S < R)
  # so that results are consistent with closepairs(X,S)
  verifyclass(X, "ppp")
  stopifnot(is.numeric(R) && length(R) == 1L && R >= 0)
  stopifnot(is.numeric(S) && length(S) == 1L && S >= 0)
  stopifnot(S < R)
  ordered <- list(...)$ordered
  if(missing(twice) && !is.null(ordered)) {
    warning("Obsolete argument 'ordered' has been replaced by 'twice'")
    twice <- ordered
  }
  npts <- npoints(X)
   if(npts == 0)
     return(list(i=integer(0), j=integer(0), t=logical(0)))
  # sort points by increasing x coordinate
  oo <- fave.order(X$x)
  Xsort <- X[oo]
  # First make an OVERESTIMATE of the number of pairs
  nsize <- ceiling(4 * pi * (npts^2) * (R^2)/area(Window(X)))
  nsize <- max(1024, nsize)
  if(nsize > .Machine$integer.max) {
    warning("Estimated number of close pairs exceeds maximum possible integer",
            call.=FALSE)
    nsize <- .Machine$integer.max
  }
  # Now extract pairs
  x <- Xsort$x
  y <- Xsort$y
  r <- R
  s <- S
  ng <- nsize
  storage.mode(x) <- "double"
  storage.mode(y) <- "double"
  storage.mode(r) <- "double"
  storage.mode(s) <- "double"
  storage.mode(ng) <- "integer"
  z <- .Call("Vclosethresh",
             xx=x, yy=y, rr=r, ss=s, nguess=ng,
             PACKAGE = "spatstat")
  if(length(z) != 3)
    stop("Internal error: incorrect format returned from Vclosethresh")
  i  <- z[[1L]]  # NB no increment required
  j  <- z[[2L]]
  th <- as.logical(z[[3L]])
  
  # convert i,j indices to original sequence
  i <- oo[i]
  j <- oo[j]
  # fast C code only returns i < j
  if(twice) {
    iold <- i
    jold <- j
    i <- c(iold, jold)
    j <- c(jold, iold)
    th <- rep(th, 2)
  }
  # done
  return(list(i=i, j=j, th=th))
}

crosspairquad <- function(Q, rmax, what=c("all", "indices")) {
  # find all close pairs X[i], U[j]
  stopifnot(inherits(Q, "quad"))
  what <- match.arg(what)
  X <- Q$data
  D <- Q$dummy
  clX <- closepairs(X=X, rmax=rmax, what=what)
  clXD <- crosspairs(X=X, Y=D, rmax=rmax, what=what)
  # convert all indices to serial numbers in union.quad(Q)
  # assumes data are listed first
  clXD$j <- npoints(X) + clXD$j
  result <- as.list(rbind(as.data.frame(clX), as.data.frame(clXD)))
  return(result)
}

tweak.closepairs <- function(cl, rmax, i, deltax, deltay, deltaz) {
  stopifnot(is.list(cl))
  stopifnot(all(c("i", "j") %in% names(cl)))
  if(!any(c("xi", "dx") %in% names(cl)))
    stop("Insufficient data to update closepairs list")
  check.1.real(rmax)
  check.1.integer(i)
  check.1.real(deltax)
  check.1.real(deltay)
  if("dz" %in% names(cl)) check.1.real(deltaz) else { deltaz <- NULL }
  hit.i <- (cl$i == i)
  hit.j <- (cl$j == i)
  if(any(hit.i | hit.j)) {
    mm <- hit.i & !hit.j
    if(any(mm)) {
      cl$xi[mm] <- cl$xi[mm] + deltax
      cl$yi[mm] <- cl$yi[mm] + deltay
      cl$dx[mm] <- cl$dx[mm] - deltax
      cl$dy[mm] <- cl$dy[mm] - deltay
      if(is.null(deltaz)) {
        cl$d[mm] <- sqrt(cl$dx[mm]^2 + cl$dy[mm]^2)
      } else {
        cl$zi[mm] <- cl$zi[mm] + deltaz
        cl$dz[mm] <- cl$dz[mm] - deltaz
        cl$d[mm] <- sqrt(cl$dx[mm]^2 + cl$dy[mm]^2 + cl$dz[mm]^2)
      }
    }
    mm <- hit.j & !hit.i
    if(any(mm)) {
      cl$xj[mm] <- cl$xj[mm] + deltax
      cl$yj[mm] <- cl$yj[mm] + deltay
      cl$dx[mm] <- cl$dx[mm] + deltax
      cl$dy[mm] <- cl$dy[mm] + deltay
      if(is.null(deltaz)) {
        cl$d[mm] <- sqrt(cl$dx[mm]^2 + cl$dy[mm]^2)
      } else {
        cl$zj[mm] <- cl$zj[mm] + deltaz
        cl$dz[mm] <- cl$dz[mm] + deltaz
        cl$d[mm] <- sqrt(cl$dx[mm]^2 + cl$dy[mm]^2 + cl$dz[mm]^2)
      }
    }
    mm <- hit.i & hit.j
    if(any(mm)) {
      cl$xi[mm] <- cl$xi[mm] + deltax
      cl$xj[mm] <- cl$xj[mm] + deltax
      cl$yi[mm] <- cl$yi[mm] + deltay
      cl$yj[mm] <- cl$yj[mm] + deltay
      if(!is.null(deltaz)) {
        cl$zi[mm] <- cl$zi[mm] + deltaz
        cl$zj[mm] <- cl$zj[mm] + deltaz
      }
    }
    if(any(lost <- (cl$d > rmax)))
      cl <- as.list(as.data.frame(cl)[!lost, , drop=FALSE])
  }
  return(cl)
}

#
# close3Dpairs.R
#
#   $Revision: 1.14 $   $Date: 2020/02/06 05:53:02 $
#
#  extract the r-close pairs from a 3D dataset
# 
#
closepairs.pp3 <- local({

  closepairs.pp3 <- function(X, rmax, twice=TRUE,
                             what=c("all", "indices", "ijd"),
                             distinct=TRUE, neat=TRUE, ...) {
    verifyclass(X, "pp3")
    what <- match.arg(what)
    stopifnot(is.numeric(rmax) && length(rmax) == 1L)
    stopifnot(is.finite(rmax))
    stopifnot(rmax >= 0)
    ordered <- list(...)$ordered
    if(missing(twice) && !is.null(ordered)) {
      warning("Obsolete argument 'ordered' has been replaced by 'twice'")
      twice <- ordered
    }
    npts <- npoints(X)
    nama <- switch(what,
                   all = c("i", "j",
                           "xi", "yi", "zi",
                           "xj", "yj", "zj",
                           "dx", "dy", "dz",
                           "d"),
                   indices = c("i", "j"),
                   ijd     = c("i", "j", "d"))
    names(nama) <- nama
    if(npts == 0) {
      null.answer <- lapply(nama, nuttink)
      return(null.answer)
    }
    ## sort points by increasing x coordinate
    oo <- fave.order(coords(X)$x)
    Xsort <- X[oo]
    ## First make an OVERESTIMATE of the number of pairs
    nsize <- list(...)$nsize # secret option to test overflow code
    if(!is.null(nsize)) {
      splat("Using nsize =", nsize)
    } else {
      #' normal usage
      nsize <- ceiling(5 * pi * (npts^2) * (rmax^3)/volume(as.box3(X)))
      nsize <- max(1024, nsize)
      if(nsize > .Machine$integer.max) {
        warning(
          "Estimated number of close pairs exceeds maximum possible integer",
          call.=FALSE)
        nsize <- .Machine$integer.max
      }
    }
    ## Now extract pairs
    XsortC <- coords(Xsort)
    x <- XsortC$x
    y <- XsortC$y
    z <- XsortC$z
    r <- rmax
    ng <- nsize
    storage.mode(x) <- "double"
    storage.mode(y) <- "double"
    storage.mode(z) <- "double"
    storage.mode(r) <- "double"
    storage.mode(ng) <- "integer"
    ## go
    a <- switch(what,
                all = {
                  .Call("close3pairs",
                        xx=x, yy=y, zz=z, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
                },
                indices = {
                  .Call("close3IJpairs",
                        xx=x, yy=y, zz=z, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
                },
                ijd = {
                  .Call("close3IJDpairs",
                        xx=x, yy=y, zz=z, rr=r, nguess=ng,
                        PACKAGE = "spatstat")
                })
    names(a) <- nama
    ## convert i,j indices to original sequence
    a$i <- oo[a$i]
    a$j <- oo[a$j]
    ## handle options
    if(twice) {
      ## both (i, j) and (j, i) should be returned
      a <- as.data.frame(a)
      a <- as.list(rbind(a, swapdata(a, what)))
    } else if(neat) {
      ## enforce i < j
      swap <- with(a, (j < i))
      if(any(swap)) {
        a <- as.data.frame(a)
        a[swap,] <- swapdata(a[swap, ,drop=FALSE], what)
        a <- as.list(a)
      }
    }
    ## add pairs of identical points?
    if(!distinct) {
      ii <- seq_len(npts)
      xtra <- switch(what,
                     indices = {
                       data.frame(i = ii, j=ii)
                     },
                     ijd= {
                       data.frame(i = ii, j=ii, d=0)
                     },
                     all = {
                       cooi <- cooj <- coords(X)[, c("x","y","z")]
                       names(cooi) <- c("xi", "yi", "zi")
                       names(cooj) <- c("xj", "yj", "zj")
                       zero <- numeric(npts)
                       cbind(data.frame(i=ii, j=ii),
                             cooi,
                             cooj,
                             data.frame(dx=zero, dy=zero, dz=zero, d=zero))
                     })
      a <- as.list(rbind(as.data.frame(a), xtra))
    }
    ## done
    return(a)
  }

  swapdata <- function(a, what) {
    switch(what,
           all = {
             with(a, data.frame(i  =  j,
                                j  =  i,
                                xi =  xj,
                                yi =  yj,
                                zi =  zj,
                                xj =  xi,
                                yj =  yi,
                                zj =  zi,
                                dx = -dx,
                                dy = -dy,
                                dz = -dz,
                                d  =  d))
           },
           indices = {
             with(a, data.frame(i=j,
                                j=i))
           },
           ijd = {
             with(a, data.frame(i=j,
                                j=i,
                                d=d))
           })
  }
  
  nuttink <- function(x) numeric(0)

  closepairs.pp3
})

#######################

crosspairs.pp3 <- local({

  crosspairs.pp3 <- function(X, Y, rmax, what=c("all", "indices", "ijd"), ...) {
    verifyclass(X, "pp3")
    verifyclass(Y, "pp3")
    what <- match.arg(what)
    stopifnot(is.numeric(rmax) && length(rmax) == 1L && rmax >= 0)
    nama <- switch(what,
                   all = c("i", "j",
                           "xi", "yi", "zi",
                           "xj", "yj", "zj",
                           "dx", "dy", "dz",
                           "d"),
                   indices = c("i", "j"),
                   ijd = c("i", "j", "d"))
    names(nama) <- nama
    nX <- npoints(X)
    nY <- npoints(Y)
    if(nX == 0 || nY == 0) {
      null.answer <- lapply(nama, nuttink)
      return(null.answer)
    }
    ## order patterns by increasing x coordinate
    ooX <- fave.order(coords(X)$x)
    Xsort <- X[ooX]
    ooY <- fave.order(coords(Y)$x)
    Ysort <- Y[ooY]
    ## First (over)estimate the number of pairs
    nsize <- list(...)$nsize # secret option to test overflow code
    if(!is.null(nsize)) {
      splat("Using nsize =", nsize)
    } else {
      #' normal usage
      nsize <- ceiling(3 * pi * (rmax^3) * nX * nY/volume(as.box3(Y)))
      nsize <- max(1024, nsize)
    }
    if(nsize > .Machine$integer.max) {
      warning(
        "Estimated number of close pairs exceeds maximum possible integer",
        call.=FALSE)
      nsize <- .Machine$integer.max
    }
    ## .Call
    XsortC <- coords(Xsort)
    YsortC <- coords(Ysort)
    Xx <- XsortC$x
    Xy <- XsortC$y
    Xz <- XsortC$z
    Yx <- YsortC$x
    Yy <- YsortC$y
    Yz <- YsortC$z
    r <- rmax
    ng <- nsize
    storage.mode(Xx) <- storage.mode(Xy) <- storage.mode(Xz) <- "double"
    storage.mode(Yx) <- storage.mode(Yy) <- storage.mode(Yz) <- "double"
    storage.mode(r) <- "double"
    storage.mode(ng) <- "integer"
    ## go
    a <- switch(what,
                all = {
                  .Call("cross3pairs",
                        xx1=Xx, yy1=Xy, zz1=Xz,
                        xx2=Yx, yy2=Yy, zz2=Yz,
                        rr=r, nguess=ng,
                        PACKAGE = "spatstat")
                },
                indices = {
                  .Call("cross3IJpairs",
                        xx1=Xx, yy1=Xy, zz1=Xz,
                        xx2=Yx, yy2=Yy, zz2=Yz,
                        rr=r, nguess=ng,
                        PACKAGE = "spatstat")
                },
                ijd = {
                  .Call("cross3IJDpairs",
                        xx1=Xx, yy1=Xy, zz1=Xz,
                        xx2=Yx, yy2=Yy, zz2=Yz,
                        rr=r, nguess=ng,
                        PACKAGE = "spatstat")
                })
    names(a) <- nama
    ## convert i,j indices to original sequence
    a$i <- ooX[a$i]
    a$j <- ooY[a$j]
    return(a)
  }

  nuttink <- function(x) numeric(0)
  
  crosspairs.pp3
})

##
## disc.R
##
##  discs and ellipses
##
## $Revision: 1.18 $ $Date: 2017/01/15 05:25:16 $
##

disc <- local({

  indic <- function(x,y,x0,y0,r) { as.integer((x-x0)^2 + (y-y0)^2 < r^2) }
  
  disc <- function(radius=1, centre=c(0,0), ...,
                   mask=FALSE, npoly=128, delta=NULL) {
    check.1.real(radius)
    stopifnot(radius > 0)
    centre <- as2vector(centre)
    if(!missing(npoly) && !is.null(npoly) && !is.null(delta))
      stop("Specify either npoly or delta")
    if(!missing(npoly) && !is.null(npoly)) {
      stopifnot(length(npoly) == 1)
      stopifnot(npoly >= 3)
    } else if(!is.null(delta)) {
      check.1.real(delta)
      stopifnot(delta > 0)
      npoly <- pmax(16, ceiling(2 * pi * radius/delta))
    } else npoly <- 128
    if(!mask) {
      theta <- seq(from=0, to=2*pi, length.out=npoly+1)[-(npoly+1L)]
      x <- centre[1L] + radius * cos(theta)
      y <- centre[2L] + radius * sin(theta)
      W <- owin(poly=list(x=x, y=y), check=FALSE)
    } else {
      xr <- centre[1L] + radius * c(-1,1)
      yr <- centre[2L] + radius * c(-1,1)
      B <- owin(xr,yr)
      IW <- as.im(indic, B, x0=centre[1L], y0=centre[2L], r=radius, ...)
      W <- levelset(IW, 1, "==")
    }
    return(W)
  }

  disc
})

hexagon <- function(edge=1, centre=c(0,0), ...,
                    align=c("bottom", "top", "left", "right", "no")) {
  regularpolygon(6, edge, centre, align=align)
}

regularpolygon <- function(n, edge=1, centre=c(0,0), ...,
                           align=c("bottom", "top", "left", "right", "no")) {
  check.1.integer(n)
  check.1.real(edge)
  stopifnot(n >= 3)
  stopifnot(edge > 0)
  align <- match.arg(align)
  theta <- 2 * pi/n
  radius <- edge/(2 * sin(theta/2))
  result <- disc(radius, centre, npoly=n, mask=FALSE)
  if(align != "no") {
    k <- switch(align,
                bottom = 3/4,
                top = 1/4,
                left = 1/2,
                right = 1)
    alpha <- theta * (1/2 - (k * n) %% 1)
    result <- rotate(result, -alpha)
  }
  Frame(result) <- boundingbox(result)
  return(result)
}


ellipse <- local({
  
  indic <- function(x,y,x0,y0,a,b,co,si){
    x <- x-x0
    y <- y-y0
    as.integer(((x*co + y*si)/a)^2 + ((-x*si + y*co)/b)^2 < 1)
  }

  ellipse <- function(a, b, centre=c(0,0), phi=0, ...,
                      mask=FALSE, npoly = 128) {
    ## Czechs:
    stopifnot(length(a) == 1)
    stopifnot(a > 0)
    stopifnot(length(b) == 1)
    stopifnot(b > 0)
    centre <- as2vector(centre)
    stopifnot(length(phi) == 1)
    stopifnot(length(npoly) == 1)
    stopifnot(npoly > 2)
    ## Rotator cuff:
    co <- cos(phi)
    si <- sin(phi)
    ## Mask:
    if(mask) {
      ## Thetas maximizing x and y.
      tx <- atan(-b*tan(phi)/a)
      ty <- atan(b/(a*tan(phi)))
      ## Maximal x and y (for centre = c(0,0)).
      xm <- a*co*cos(tx) - b*si*sin(tx)
      ym <- a*si*cos(ty) + b*co*sin(ty)
      ## Range of x and y.
      xr <- xm*c(-1,1)+centre[1L]
      yr <- ym*c(-1,1)+centre[2L]
      ## Wrecked-angle to contain the mask.
      B  <- as.mask(owin(xr,yr),...)
      ## Build the mask as a level set.
      IW <- as.im(indic, B, x0=centre[1L], y0=centre[2L], a=a, b=b, co=co, si=si)
      return(levelset(IW, 1, "=="))
    }
    ## Polygonal.
    ## Build "horizontal" ellipse centred at 0:
    theta <- seq(0, 2 * pi, length = npoly+1)[-(npoly+1L)]
    xh <-  a * cos(theta)
    yh <-  b * sin(theta)

    ## Rotate through angle phi and shift centre:
    x  <- centre[1L] + co*xh - si*yh
    y  <- centre[2L] + si*xh + co*yh
    owin(poly=list(x = x, y = y))
  }

  ellipse
})

##  terse.R
##
##  code to control terseness and layout of printed output
##
##  $Revision: 1.11 $  $Date: 2016/09/23 02:07:24 $
##


## paragraph break in long output e.g. ppm
parbreak <- function(terse = spatstat.options("terse")) {
  if(waxlyrical('space', terse)) cat("\n")
  return(invisible(NULL))
}

waxlyrical <- local({

  ##  Values of spatstat.options('terse'):
  ##        0    default
  ##        1    suppress obvious wastage e.g. 'gory details'
  ##        2    contract space between paragraphs in long output
  ##        3    suppress extras e.g. standard errors and CI 
  ##        4    suppress error messages eg failed to converge

  TerseCutoff <- list(gory=1,
                      space=2,
                      extras=3,
                      errors=4)

  waxlyrical <- function(type, terse = spatstat.options("terse")) {
    if(!(type %in% names(TerseCutoff)))
      stop(paste("Internal error: unrecognised permission request",
                 sQuote(type)),
           call.=TRUE)
    return(terse < TerseCutoff[[type]])
  }
  
  waxlyrical
  
})

ruletextline <- function(ch="-", n=getOption('width'),
                         terse=spatstat.options('terse')) {
  if(waxlyrical('space', terse)) {
    chn <- paste(rep(ch, n), collapse="")
    chn <- substr(chn, 1, n)
    cat(chn, fill=TRUE)
  }
  return(invisible(NULL))
}
  

## scriptUtils.R
##       $Revision: 1.6 $ $Date: 2017/11/14 06:42:02 $

## slick way to use precomputed data
##    If the named file exists, it is loaded, giving access to the data.
##    Otherwise, 'expr' is evaluated, and all objects created
##    are saved in the designated file, for loading next time.

reload.or.compute <- function(filename, expr, 
                              objects=NULL,
                              destination=parent.frame(),
                              force=FALSE) {
  stopifnot(is.character(filename) && length(filename) == 1)
  if(force || !file.exists(filename)) {
    ## evaluate 'expr' in a fresh environment
    ee <- as.expression(substitute(expr))
    en <- new.env()
    local(eval(ee), envir=en)
    ## default is to save all objects that were created
    if(is.null(objects))
      objects <- ls(envir=en)
    ## save them in the designated file
    evalq(save(list=objects, file=filename, compress=TRUE), envir=en)
    ## assign them into the parent frame 
    for(i in seq_along(objects))
      assign(objects[i], get(objects[i], envir=en), envir=destination)
    result <- objects
  } else {
    result <- load(filename, envir=destination)
    if(!all(ok <- (objects %in% result))) {
      nbad <- sum(!ok)
      warning(paste(ngettext(nbad, "object", "objects"),
                    commasep(sQuote(objects[!ok])),
                    ngettext(nbad, "was", "were"),
                    "not present in data file", dQuote(filename)),
              call.=FALSE)
    }
  }
  return(invisible(result))
}

#
# periodify.R
#
# replicate a pattern periodically
#
#  $Revision: 1.3 $  $Date: 2011/04/17 05:52:50 $
#

periodify <- function(X, ...) {
  UseMethod("periodify")
}

periodify.ppp <- function(X, nx=1, ny=1, ...,
                          combine=TRUE, warn=TRUE, check=TRUE, 
                          ix=(-nx):nx, iy=(-ny):ny,
                          ixy=expand.grid(ix=ix,iy=iy)) {
  # sanity checks
  if(!missing(nx) || !missing(ny)) {
    if(is.null(nx)) nx <- 1
    if(is.null(ny)) ny <- 1
    if(length(nx) != 1 || length(ny) != 1)
      stop("nx and ny should be single integers")
    if(nx != round(nx) || ny != round(ny))
      stop("nx and ny should be integers")
  }
  force(ixy)
  W <- X$window
  isrect <- (W$type == "rectangle")
  if(warn && combine && !isrect)
    warning("X has a non-rectangular window")
  else 
   isrect <- isrect && all(diff(nx) == 1) && all(diff(ny) == 1)
  width <- diff(W$xrange)
  height <- diff(W$yrange)
  shifts <- cbind(ixy[,1] * width, ixy[,2] * height)
  Xshift <- list()
  for(i in 1:nrow(shifts))
    Xshift[[i]] <- shift(X, vec=as.numeric(shifts[i, ]))
  if(!combine)
    return(Xshift)
  Wnew <- if(isrect) {
    owin(range(range(W$xrange) + range(shifts[,1])),
         range(range(W$yrange) + range(shifts[,2])))
  } else NULL
  Z <- do.call(superimpose, append(Xshift, list(W=Wnew, check=check)))
  return(Z)
}

periodify.psp <- function(X, nx=1, ny=1, ...,
                          combine=TRUE, warn=TRUE, check=TRUE,
                          ix=(-nx):nx, iy=(-ny):ny,
                          ixy=expand.grid(ix=ix,iy=iy)) {
  # sanity checks
  if(!missing(nx) || !missing(ny)) {
    if(is.null(nx)) nx <- 1
    if(is.null(ny)) ny <- 1
    if(length(nx) != 1 || length(ny) != 1)
      stop("nx and ny should be single integers")
    if(nx != round(nx) || ny != round(ny))
      stop("nx and ny should be integers")
  }
  force(ixy)
  W <- X$window
  isrect <- (W$type == "rectangle")
  if(warn && combine && !isrect)
    warning("X has a non-rectangular window")
  else 
   isrect <- isrect && all(diff(nx) == 1) && all(diff(ny) == 1)
  width <- diff(W$xrange)
  height <- diff(W$yrange)
  shifts <- cbind(ixy[,1] * width, ixy[,2] * height)
  Xshift <- list()
  for(i in 1:nrow(shifts))
    Xshift[[i]] <- shift(X, vec=as.numeric(shifts[i, ]))
  if(!combine)
    return(Xshift)
  Wnew <- if(isrect) {
    owin(range(range(W$xrange) + range(shifts[,1])),
         range(range(W$yrange) + range(shifts[,2])))
  } else NULL
  Z <- do.call(superimpose, append(Xshift, list(W=Wnew, check=check)))
  return(Z)
}

periodify.owin <- function(X, nx=1, ny=1, ...,
                          combine=TRUE, warn=TRUE,
                          ix=(-nx):nx, iy=(-ny):ny,
                          ixy=expand.grid(ix=ix,iy=iy)) {
  # sanity checks
  if(!missing(nx) || !missing(ny)) {
    if(is.null(nx)) nx <- 1
    if(is.null(ny)) ny <- 1
    if(length(nx) != 1 || length(ny) != 1)
      stop("nx and ny should be single integers")
    if(nx != round(nx) || ny != round(ny))
      stop("nx and ny should be integers")
  }
  force(ixy)
  isrect <- (X$type == "rectangle")
  if(warn && combine && !isrect)
    warning("X is not rectangular")
  else 
    isrect <- isrect && all(diff(nx) == 1) && all(diff(ny) == 1)
  width <- diff(X$xrange)
  height <- diff(X$yrange)
  shifts <- cbind(ixy[,1] * width, ixy[,2] * height)
  if(combine) {
    if(isrect) {
      # result is a rectangle
      Y <-  owin(range(range(X$xrange) + range(shifts[,1])),
                    range(range(X$yrange) + range(shifts[,2])))
    } else {
      # result is another type of window
      for(i in 1:nrow(shifts)) {
        Xi <- shift(X, vec=as.numeric(shifts[i, ]))
        Y <- if(i == 1) Xi else union.owin(Y, Xi)
      }
    }
  } else {
    # result is a list
    Y <- list()
    for(i in 1:nrow(shifts))
      Y[[i]] <- shift(X, vec=as.numeric(shifts[i, ]))
  }
  return(Y)
}

#
# tess.R
#
# support for tessellations
#
#   $Revision: 1.97 $ $Date: 2019/09/13 04:29:08 $
#
tess <- function(..., xgrid=NULL, ygrid=NULL, tiles=NULL, image=NULL,
                 window=NULL, marks=NULL, keepempty=FALSE,
                 unitname=NULL, check=TRUE) {
  uname <- unitname
  if(!is.null(window)) {
    window <- as.owin(window)
    if(is.null(uname)) uname <- unitname(window) 
  }
  isrect <- !is.null(xgrid) && !is.null(ygrid)
  istiled <- !is.null(tiles)
  isimage <- !is.null(image)
  if(isrect + istiled + isimage != 1)
    stop("Must specify either (xgrid, ygrid) or tiles or img")
  if(isrect) {
    stopifnot(is.numeric(xgrid) && all(diff(xgrid) > 0))
    stopifnot(is.numeric(ygrid) && all(diff(ygrid) > 0))
    if(!is.null(window))
      warning("Argument 'window' ignored, because xgrid, grid are given")
    window <- owin(range(xgrid), range(ygrid), unitname=uname)
    ntiles <- (length(xgrid)-1) * (length(ygrid)-1)
    out <- list(type="rect", window=window, xgrid=xgrid, ygrid=ygrid, n=ntiles)
  } else if(istiled) {
    stopifnot(is.list(tiles))
    if(check) {
      if(!all(sapply(tiles, is.owin)))
        stop("Tiles must be a list of owin objects")
      if(!is.null(uname)) {
        ## attach new unit name to each tile
        tiles <- lapply(tiles, "unitname<-", value=uname)
      } else {
        ## extract unit names from tiles, check agreement, use as unitname
        uu <- unique(lapply(tiles, unitname))
        uu <- uu[!sapply(uu, is.null)]
        nun <- length(uu)
        if(nun > 1)
          stop("Tiles have inconsistent names for the unit of length")
        if(nun == 1) {
          ## use this unit name
          uname <- uu[[1]]
          if(!is.null(window))
            unitname(window) <- uname
        }
      }
    }
    if(!keepempty && check) {
      # remove empty tiles
      isempty <- sapply(tiles, is.empty)
      if(all(isempty))
        stop("All tiles are empty")
      if(any(isempty))
        tiles <- tiles[!isempty]
    }
    ntiles <- length(tiles)
    nam <- names(tiles)
    lev <- if(!is.null(nam) && all(nzchar(nam))) nam else 1:ntiles
    if(is.null(window)) 
      window <- do.call(union.owin, unname(tiles))
    if(is.mask(window) || any(sapply(tiles, is.mask))) {
      # convert to pixel image tessellation
      Grid <- do.call(commonGrid, append(list(window), unname(tiles)))
      ima <- as.im(window, W=Grid)
      ima$v[] <- NA
      for(i in 1:ntiles)
        ima[tiles[[i]]] <- i
      ima <- ima[window, drop=FALSE]
      ima <- eval.im(factor(ima, levels=1:ntiles))
      levels(ima) <- lev
      out <- list(type="image",
                  window=window, image=ima, n=length(lev))
    } else {
      # tile list
      window <- rescue.rectangle(window)
      out <- list(type="tiled", window=window, tiles=tiles, n=length(tiles))
    }
  } else if(isimage) {
    # convert to factor valued image
    image <- as.im(image)
    if(!is.null(uname)) unitname(image) <- uname
    switch(image$type,
           logical={
             # convert to factor
             if(keepempty) 
               image <- eval.im(factor(image, levels=c(FALSE,TRUE)))
             else
               image <- eval.im(factor(image))
           },
           factor={
             # eradicate unused levels
             if(!keepempty) 
               image <- eval.im(factor(image))
           },
           {
             # convert to factor
             image <- eval.im(factor(image))
           })
               
    if(is.null(window)) window <- as.owin(image)
    out <- list(type="image", window=window, image=image, n=length(levels(image)))
  } else stop("Internal error: unrecognised format")
  ## add marks!
  if(!is.null(marks)) {
    marks <- as.data.frame(marks)
    if(nrow(marks) != out$n)
      stop(paste("wrong number of marks:",
                 nrow(marks), "should be", out$n),
           call.=FALSE)
    out$marks <- marks
  }
  class(out) <- c("tess", class(out))
  return(out)
}

is.tess <- function(x) { inherits(x, "tess") }

print.tess <- function(x, ..., brief=FALSE) {
  full <- !brief
  if(full) cat("Tessellation\n")
  win <- x$window
  switch(x$type,
         rect={
           if(full) {
             unitinfo <- summary(unitname(win))
             if(equispaced(x$xgrid) && equispaced(x$ygrid)) 
               splat("Tiles are equal rectangles, of dimension",
                     signif(mean(diff(x$xgrid)), 5),
                     "x",
                     signif(mean(diff(x$ygrid)), 5),
                     unitinfo$plural, " ", unitinfo$explain)
             else
               splat("Tiles are unequal rectangles")
           }
           splat(length(x$xgrid)-1, "by", length(x$ygrid)-1, "grid of tiles")
         },
         tiled={
           if(full) {
             if(win$type == "polygonal")
               splat("Tiles are irregular polygons")
             else
               splat("Tiles are windows of general type")
           }
           splat(length(x$tiles), "tiles (irregular windows)")
         },
         image={
           nlev <- length(levels(x$image))
           if(full) {
             splat("Tessellation is determined by a factor-valued image with",
                   nlev, "levels")
           } else splat(nlev, "tiles (levels of a pixel image)")
         })
  if(!is.null(marx <- x$marks)) {
    m <- dim(marx)[2] %orifnull% 1
    if(m == 1) splat("Tessellation is marked") else
    splat("Tessellation has", m, "columns of marks:",
          commasep(sQuote(colnames(marx))))
  }
  if(full) print(win)
  invisible(NULL)
}

unitname.tess <- function(x) unitname(x$window)

"unitname<-.tess" <- function(x, value) {
  unitname(x$window) <- value
  switch(x$type,
         rect={},
         tiled={
           x$tiles <- lapply(x$tiles, "unitname<-", value)
         },
         image={
           unitname(x$image) <- value
         })
  return(x)
}

plot.tess <- local({

  plotpars <- c("sub", "lty", "lwd",
                "cex.main", "col.main", "font.main",
                "cex.sub", "col.sub", "font.sub", "border")

  plot.tess <- function(x, ..., main, add=FALSE, show.all=!add,
                        border=NULL,
                        do.plot=TRUE,
                        do.labels=FALSE, labels=tilenames(x),
                        labelargs=list(),
                        do.col=FALSE, 
                        values=marks(x),
                        multiplot=TRUE,
                        col=NULL,
                        ribargs=list()) {
    if(missing(main) || is.null(main))
      main <- short.deparse(substitute(x))
    ntiles <- x$n
    if(!do.col) {
      #' Plot tiles, with adornment
      y <- NULL
      result <- NULL
      bbox <- NULL
      need.legend <- FALSE
    } else {
      #' Fill tiles with colours determined by 'values'
      if(markformat(values) == "hyperframe") 
        values <- as.data.frame(values) #' automatic warning
      #' Determine values associated with each tile
      switch(markformat(values),
             none = {
               #' no values assigned.
               #' default is tile name
               tn <- tilenames(x)
               values <- factor(tn, levels=tn)
             },
             vector = {
               #' vector of values.
               #' validate length of vector
               check.anyvector(values, ntiles, things="tiles")
             },
             dataframe = {
               #' data frame or matrix of values.
               values <- as.data.frame(values)
               if(nrow(values) != ntiles)
                 stop(paste("Number of rows of values =", nrow(values),
                            "!=", ntiles, "= number of tiles"),
                      call.=FALSE)
               if(multiplot && ncol(values) > 1 && !add) {
                 #' Multiple Panel Plot
                 result <- multi.plot.tess(x, ...,
                                           main=main, show.all=show.all,
                                           border=border, do.plot=do.plot,
                                           do.labels=do.labels, labels=labels,
                                           labelargs=labelargs, do.col=do.col, 
                                           col=col, ribargs=ribargs)
                 return(invisible(result))
               }
               if(ncol(values) > 1)
                 warning("Using only the first column of values")
               values <- values[,1]
             },
             stop("Format of values is not understood")
             )
      #' Single Panel Plot
      #' Determine colour map and plan layout (including colour ribbon)
      #' using rules for pixel images
      y <- as.im(as.function(x, values=values))
      result <- do.call(plot.im,
                        resolve.defaults(
                          list(x=y,
                               do.plot=FALSE,
                               show.all=show.all, add=add, main=main,
                               col=col, ribargs=ribargs),
                          list(...),
                          list(valuesAreColours=FALSE)
                          ))
      #' exit if not actually plotting
      if(!do.plot) return(invisible(result))
      #' extract info
      colmap <- result
      bbox <- attr(result, "bbox")
      bbox.legend <- attr(result, "bbox.legend")
      need.legend <- !is.null(bbox.legend)
    }
    #'      Start Plot 
    #' initialise plot region if it is determined
    if(do.plot && !is.null(bbox) && !add) {
      plot(bbox, main=" ", type="n")
      add <- TRUE
    }
    switch(x$type,
           rect={
             win <- x$window
             z <- do.call.matched(plot.owin,
                                  resolve.defaults(list(x=win,
                                                        main=main,
                                                        add=add,
                                                        show.all=show.all,
                                                        do.plot=do.plot),
                                                   list(...)),
                                  extrargs=plotpars)
             if(is.null(result))
               result <- z
             if(do.plot) {
               #' actually plot
               if(do.col) {
                 #' fill tiles with colours
                 colours <- colmap(values)
                 til <- tiles(x)
                 for(i in seq_len(x$n))
                   plot(til[[i]], add=TRUE,
                        col=colours[i], border=border,
                        main="", ...)
               } else {
                 #' draw tile boundaries only
                 xg <- x$xgrid
                 yg <- x$ygrid
                 do.call.matched(segments,
                                 resolve.defaults(list(x0=xg, y0=win$yrange[1],
                                                       x1=xg, y1=win$yrange[2]),
                                                  list(col=border),
                                                  list(...),
                                                  .StripNull=TRUE))
                 do.call.matched(segments,
                                 resolve.defaults(list(x0=win$xrange[1], y0=yg,
                                                       x1=win$xrange[2], y1=yg),
                                                  list(col=border),
                                                  list(...),
                                                  .StripNull=TRUE))
               }
             }
           },
           tiled={
             z <- do.call.matched(plot.owin,
                                  resolve.defaults(list(x=x$window,
                                                        main=main,
                                                        add=add,
                                                        show.all=show.all,
                                                        do.plot=do.plot),
                                                   list(...)),
                                  extrargs=plotpars)
             if(is.null(result)) result <- z
             if(do.plot) {
               #' plot each tile
               til <- tiles(x)
               if(!do.col) {
                 #' border only
                 lapply(til, plot.owin, ..., add=TRUE, border=border)
               } else {
                 #' fill with colour
                 colours <- colmap(values)
                 mapply(plot.owin, x=til, col=colours,
                        MoreArgs=list(add=TRUE, main="", border=border, ...))
               } 
             }
           },
           image={
             if(is.null(y)) y <- x$image
             result <-
               do.call(plot,
                       resolve.defaults(list(y, add=add, main=main,
                                             show.all=show.all,
                                             do.plot=do.plot,
                                             col=col, ribargs=ribargs),
                                        list(...),
                                        list(valuesAreColours=FALSE)))
             need.legend <- FALSE
           })
    if(do.plot && do.labels) {
      labels <- paste(as.vector(labels))
      til <- tiles(x)
      incircles <- lapply(til, incircle)
      x0 <- sapply(incircles, getElement, name="x")
      y0 <- sapply(incircles, getElement, name="y")
      do.call.matched(text.default,
                      resolve.defaults(list(x=x0, y = y0),
                                       list(labels=labels),
                                       labelargs),
                      funargs=graphicsPars("text"))
    }
    if(do.plot && need.legend) {
      #' determine position of legend
      xlim <- bbox.legend$xrange
      ylim <- bbox.legend$yrange
      sidecode <- attr(colmap, "side.legend")
      vertical <- sidecode %in% c(2,4)
      do.call(plot.colourmap,
              resolve.defaults(list(x=colmap,
                                    add=TRUE, main="",
                                    xlim=xlim, ylim=ylim,
                                    side=sidecode, vertical=vertical),
                               ribargs,
                               list(...)))
    }
    return(invisible(result))
  }

  multi.plot.tess <- function(x, ..., zlim=NULL, col=NULL, equal.ribbon=FALSE) {
    if(equal.ribbon && is.null(zlim) && !inherits(col, "colourmap"))
      zlim <- range(marks(x))
    if(!is.null(zlim)) {
      result <- plot(unstack(x), ..., zlim=zlim, col=col)
    } else {
      result <- plot(unstack(x), ..., col=col)
    }
    return(invisible(result))
  }
  
  plot.tess
})


"[<-.tess" <- function(x, i, ..., value) {
  switch(x$type,
         rect=,
         tiled={
           til <- tiles(x)
           til[i] <- value
           ok <- !unlist(lapply(til, is.null))
           x <- tess(tiles=til[ok])
         },
         image={
           stop("Cannot assign new values to subsets of a pixel image")
         })
  return(x)
}
  
"[.tess" <- function(x, i, ...) {
  trap.extra.arguments(..., .Context="in [.tess")
  if(missing(i)) return(x)
  if(is.owin(i))
    return(intersect.tess(x, i))
  switch(x$type,
         rect=,
         tiled={
           til <- tiles(x)[i]
           return(tess(tiles=til))
         },
         image={
           img <- x$image
           oldlev <- levels(img)
           newlev <- unique(oldlev[i])
           img <- eval.im(factor(img, levels=newlev))
           return(tess(image=img))
         })
}

tiles <- function(x) {
  stopifnot(is.tess(x))
  switch(x$type,
         rect={
           out <- list()
           xg <- x$xgrid
           yg <- x$ygrid
           nx <- length(xg) - 1
           ny <- length(yg) - 1
           for(j in rev(seq_len(ny))) {
             for(i in seq_len(nx)) {
               winij <- owin(xg[c(i,i+1)], yg[c(j,j+1)])
               out <- append(out, list(winij))
             }
           }
         },
         tiled={
           out <- x$tiles
           },
         image={
           out <- list()
           ima <- x$image
           lev <- levels(ima)
           for(i in seq_along(lev))
             out[[i]] <- solutionset(ima == lev[i])
           })
  names(out) <- tilenames(x)
  out <- as.solist(out)
  return(out)
}

tiles.empty <- function(x) {
  stopifnot(is.tess(x))
  switch(x$type,
         rect = {
           nx <- length(x$xgrid) - 1
           ny <- length(x$ygrid) - 1
           ans <- rep(FALSE, nx * ny)
         },
         tiled = {
           ans <- sapply(x$tiles, is.empty)
         },
         image = {
           ans <- (table(x$image[]) == 0)
         })
  return(ans)
}
           
tilenames <- function(x) {
  UseMethod("tilenames")
}

tilenames.tess <- function(x) {
  switch(x$type,
         rect={
           if(!is.null(x$tilenames)) {
             out <- x$tilenames
           } else {
             nx <- length(x$xgrid) - 1
             ny <- length(x$ygrid) - 1
             ij <- expand.grid(1:nx, 1:ny)
             out <- paste0("Tile row ", ij[,2], ", col ", ij[,1])
           }
         },
         tiled={
           out <- names(x$tiles)
           if(sum(nzchar(out)) != x$n)
             out <- paste("Tile", seq_len(x$n))
         },
         image={
           out <- levels(x$image)
         }
         )
  return(as.character(out))
}

"tilenames<-" <- function(x, value) {
  UseMethod("tilenames<-")
}

"tilenames<-.tess" <- function(x, value) {
  if(!is.null(value)) {
    ## validate length
    value <- as.character(value)
    nv <- length(value)
    switch(x$type,
           rect = {
             nx <- length(x$xgrid) - 1
             ny <- length(x$ygrid) - 1
             n <- nx * ny
           },
           tiled = { n <- length(x$tiles) },
           image = { n <- length(levels(x$image)) })
    if(nv != n)
      stop("Replacement value has wrong length",
           paren(paste(nv, "instead of", n)))
  }
  switch(x$type,
         rect={
           x$tilenames <- value
         },
         tiled={
           names(x$tiles) <- value
         },
         image={
           levels(x$image) <- value %orifnull% (1:n)
         }
         )
  return(x)
}

marks.tess <- function(x, ...) {
  stopifnot(is.tess(x))
  return(x$marks)
}

"marks<-.tess" <- function(x, ..., value) {
  stopifnot(is.tess(x))
  if(!is.null(value)) {
    value <- as.data.frame(value)
    ntil <- x$n
    if(nrow(value) != ntil)
      stop(paste("replacement value for marks has wrong length:",
                 nrow(value), "should be", ntil),
           call.=FALSE)
    rownames(value) <- NULL
    if(ncol(value) == 1) colnames(value) <- "marks"
  }
  x$marks <- value
  return(x)
}

unmark.tess <- function(X) { marks(X) <- NULL; return(X) }

tile.areas <- function(x) {
  stopifnot(is.tess(x))
  switch(x$type,
         rect={
           xg <- x$xgrid
           yg <- x$ygrid
#           nx <- length(xg) - 1 
#           ny <- length(yg) - 1
           a <- outer(rev(diff(yg)), diff(xg), "*")
           a <- as.vector(t(a))
           names(a) <- as.vector(t(tilenames(x)))
         },
         tiled={
           a <- unlist(lapply(x$tiles, area))
         },
         image={
           z <- x$image
           a <- table(z$v) * z$xstep * z$ystep
         })
  return(a)
}

         
as.im.tess <- function(X, W=NULL, ...,
                       eps=NULL, dimyx=NULL, xy=NULL,
                       na.replace=NULL) {
  # if W is present, it may have to be converted
  if(!is.null(W)) {
    stopifnot(is.owin(W))
    if(W$type != "mask")
      W <- as.mask(W, eps=eps, dimyx=dimyx, xy=xy)
  } 
  switch(X$type,
         image={
           out <- as.im(X$image, W=W, eps=eps, dimyx=dimyx, xy=xy,
                        na.replace=na.replace)
         },
         tiled={
           if(is.null(W))
             W <- as.mask(as.owin(X), eps=eps, dimyx=dimyx, xy=xy)
           til <- X$tiles
           ntil <- length(til)
           nama <- names(til)
           if(is.null(nama) || !all(nzchar(nama)))
             nama <- paste(seq_len(ntil))
           xy <- list(x=W$xcol, y=W$yrow)
           for(i in seq_len(ntil)) {
             indic <- as.mask(til[[i]], xy=xy)
             tag <- as.im(indic, value=i)
             if(i == 1) {
               out <- tag
               outv <- out$v
             } else {
               outv <- pmin.int(outv, tag$v, na.rm=TRUE)
             }
           }
           out <- im(factor(outv, levels=seq_len(ntil), labels=nama),
                     out$xcol, out$yrow)
           unitname(out) <- unitname(W)
         },
         rect={
           if(is.null(W))
             out <- as.im(as.rectangle(X), eps=eps, dimyx=dimyx, xy=xy)
           else
             out <- as.im(W)
           xg <- X$xgrid
           yg <- X$ygrid
           nrows <- length(yg) - 1
           ncols <- length(xg) - 1
           jx <- findInterval(out$xcol, xg, rightmost.closed=TRUE)
           iy <- findInterval(out$yrow, yg, rightmost.closed=TRUE)
           M <- as.matrix(out)
           Jcol <- jx[col(M)]
           Irow <- nrows - iy[row(M)] + 1
           Ktile <- Jcol + ncols * (Irow - 1)
           Ktile <- factor(Ktile, levels=seq_len(nrows * ncols))
           out <- im(Ktile, xcol=out$xcol, yrow=out$yrow,
                     unitname=unitname(W))
         }
         )
  return(out)
}

nobjects.tess <- function(x) {
  switch(x$type,
         image = length(levels(x$image)),
         rect = (length(x$xgrid)-1) * (length(x$ygrid)-1),
         tiled = length(x$tiles))
}
  
as.function.tess <- function(x, ..., values=NULL) {
  V <- x
  if(is.null(values)) {
    f <- function(x,y) { tileindex(x,y,V) }
  } else {
    if(length(values) != nobjects(x))
      stop("Length of 'values' should equal the number of tiles", call.=FALSE)
    f <- function(x,y) { values[as.integer(tileindex(x,y,V))] }
  }
  g <- funxy(f, Window(V))
  return(g)
}

tileindex <- function(x, y, Z) {
  stopifnot(is.tess(Z))
  if((missing(y) || is.null(y)) && all(c("x", "y") %in% names(x))) {
    y <- x$y
    x <- x$x
  }
  stopifnot(length(x) == length(y))
  switch(Z$type,
         rect={
           jx <- findInterval(x, Z$xgrid, rightmost.closed=TRUE)
           iy <- findInterval(y, Z$ygrid, rightmost.closed=TRUE)
           nrows <- length(Z$ygrid) - 1
           ncols <- length(Z$xgrid) - 1
           iy[iy < 1 | iy > nrows] <- NA
           jx[jx < 1 | jx > ncols] <- NA
           jcol <- jx
           irow <- nrows - iy + 1
           ktile <- jcol + ncols * (irow - 1)
           m <- factor(ktile, levels=seq_len(nrows*ncols))
           ij <- expand.grid(j=seq_len(ncols),i=seq_len(nrows))
           levels(m) <- paste("Tile row ", ij$i, ", col ", ij$j, sep="")
         },
         tiled={
           n <- length(x)
           todo <- seq_len(n)
           nt <- length(Z$tiles)
           m <- integer(n)
           for(i in 1:nt) {
             ti <- Z$tiles[[i]]
             hit <- inside.owin(x[todo], y[todo], ti)
             if(any(hit)) {
               m[todo[hit]] <- i
               todo <- todo[!hit]
             }
             if(length(todo) == 0)
               break
           }
           m[m == 0] <- NA
           nama <- names(Z$tiles)
           lev <- seq_len(nt)
           lab <- if(!is.null(nama) && all(nzchar(nama))) nama else paste("Tile", lev)
           m <- factor(m, levels=lev, labels=lab)
         },
         image={
           Zim   <- Z$image
           m <- lookup.im(Zim, x, y, naok=TRUE)
           if(anyNA(m)) {
             #' look up neighbouring pixels
             isna <- is.na(m)
             rc <- nearest.valid.pixel(x[isna], y[isna], Zim, nsearch=2)
             m[isna] <- Zim$v[cbind(rc$row, rc$col)]
           }
         }
         )
  return(m)
}
  
as.tess <- function(X) {
  UseMethod("as.tess")
}

as.tess.tess <- function(X) {
  fields <- 
    switch(X$type,
           rect={ c("xgrid", "ygrid") },
           tiled={ "tiles" },
           image={ "image" },
           stop(paste("Unrecognised tessellation type", sQuote(X$type))))
  fields <- c(c("type", "window", "n", "marks"), fields)
  X <- unclass(X)[fields]
  class(X) <- c("tess", class(X))
  return(X)
}

as.tess.im <- function(X) {
  return(tess(image = X))
}

as.tess.list <- function(X) {
  W <- lapply(X, as.owin)
  return(tess(tiles=W))
}

as.tess.owin <- function(X) {
  return(tess(tiles=list(X)))
}

domain.tess <- Window.tess <- function(X, ...) { as.owin(X) } 

intersect.tess <- function(X, Y, ..., keepmarks=FALSE, sep="x") {
  X <- as.tess(X)
  check.1.string(sep)

  if(is.owin(Y)) {
    ## intersection of a tessellation with a window
    if(Y$type == "mask") {
      ## convert to pixel image 
      result <- as.im(Y)
      Xtiles <- tiles(X)
      seqXtiles <- seq_along(Xtiles)
      for(i in seqXtiles) {
        tilei <- Xtiles[[i]]
        result[tilei] <- i
      }
      result <- result[Y, drop=FALSE]
      out <- tess(image=result, window=Y)
      if(keepmarks && !is.null(marx <- marks(X))) {
        #' identify non-empty tiles
        tab <- table(factor(result[], levels=seqXtiles))
        marks(out) <- marksubset(marx, tab > 0)
      }
      return(out)
    } else {
      ## efficient code when Y is a window, retaining names of tiles of X
      Ztiles <- lapply(tiles(X), intersect.owin, B=Y, ..., fatal=FALSE)
      isempty <- sapply(Ztiles, is.empty)
      Ztiles <- Ztiles[!isempty]
      Xwin <- as.owin(X)
      Ywin <- Y
      Zwin <- intersect.owin(Xwin, Ywin)
      out <- tess(tiles=Ztiles, window=Zwin)
      if(keepmarks) {
        marx <- marks(X)
        if(!is.null(marx))
          marx <- as.data.frame(marx)[!isempty, ]
        marks(out) <- marx
      }
      return(out)
    }
  }
  
  ## general case: intersection of two tessellations
  Y <- as.tess(Y)
  Xtiles <- tiles(X)
  Ytiles <- tiles(Y)
  Ztiles <- list()
  namesX <- tilenames(X)
  namesY <- tilenames(Y)

  if(keepmarks) {
    ## initialise the mark variables to be inherited from parent tessellations
    Xmarks <- as.data.frame(marks(X))
    Ymarks <- as.data.frame(marks(Y))
    gotXmarks <- (ncol(Xmarks) > 0)
    gotYmarks <- (ncol(Ymarks) > 0)
    if(gotXmarks && gotYmarks) {
      colnames(Xmarks) <- paste0("X", colnames(Xmarks))
      colnames(Ymarks) <- paste0("Y", colnames(Ymarks))
    }
    if(gotXmarks || gotYmarks) {
      marx <- if(gotXmarks && gotYmarks) {
                cbind(Xmarks[integer(0), , drop=FALSE],
                      Ymarks[integer(0), , drop=FALSE])
              } else if(gotXmarks) {
                Xmarks[integer(0), , drop=FALSE]
              } else {
                Ymarks[integer(0), , drop=FALSE]
              }
    } else keepmarks <- FALSE
  }

  ## now compute intersection tiles
  Xtrivial <- (length(Xtiles) == 1)
  for(i in seq_along(Xtiles)) {
    Xi <- Xtiles[[i]]
    Ti <- lapply(Ytiles, intersect.owin, B=Xi, ..., fatal=FALSE)
    isempty <- sapply(Ti, is.empty)
    nonempty <- !isempty
    if(any(nonempty)) {
      Ti <- Ti[nonempty]
      names(Ti) <- if(Xtrivial) namesY[nonempty] else
                   paste(namesX[i], namesY[nonempty], sep=sep)
      Ztiles <- append(Ztiles, Ti)
      if(keepmarks) {
        extra <- if(gotXmarks && gotYmarks) {
                   data.frame(X=Xmarks[i, ,drop=FALSE],
                              Y=Ymarks[nonempty, ,drop=FALSE],
                              row.names=NULL)
                 } else if(gotYmarks) {
                   Ymarks[nonempty, ,drop=FALSE]
                 } else {
                   Xmarks[rep(i, sum(nonempty)), ,drop=FALSE]
                 }
        marx <- rbind(marx, extra)
      }
    }
  }
  ## form tessellation object
  Xwin <- as.owin(X)
  Ywin <- as.owin(Y)
  Zwin <- intersect.owin(Xwin, Ywin)
  out <- tess(tiles=Ztiles, window=Zwin)
  if(keepmarks) 
    marks(out) <- marx
  return(out)
}

venn.tess <- function(..., window=NULL) {
  argh <- list(...)
  nargh <- length(argh)
  if(nargh == 0) stop("No arguments given")
  iswin <- sapply(argh, is.owin)
  istes <- sapply(argh, is.tess)
  if(!all(iswin | istes)) 
    stop("All arguments must be windows or tessellations", call.=FALSE)
  nama <- names(argh)
  if(sum(nzchar(nama)) < nargh)
    nama <- paste0("T", seq_len(nargh))
  W <- window %orifnull% do.call(union.owin, unname(lapply(argh, as.owin)))
  for(i in seq_len(nargh)) {
    A <- argh[[i]]
    if(is.owin(A)) {
      Z <- list(A, Out=setminus.owin(W, A))
      names(Z) <- paste0(c("", "not"), nama[i])
      A <- tess(tiles=Z, window=W)
    }
    Y <- if(i == 1) A else intersect.tess(Y, A)
  }
  return(Y)
}

bdist.tiles <- local({

  vdist <- function(x,w) {
    z <- as.ppp(vertices(x), W=w, check=FALSE)
    min(bdist.points(z))
  }
  edist <- function(x,b) {
    xd <- crossdist(edges(x, check=FALSE), b, type="separation")
    min(xd)
  }

  bdist.tiles <-  function(X) {
    if(!is.tess(X))
      stop("X must be a tessellation")
    W <- as.owin(X)
    switch(X$type,
           rect=,
           tiled={
             tt <- tiles(X)
             if(is.convex(W)) {
               # distance is minimised at a tile vertex
               d <- sapply(tt, vdist, w=W)
             } else {
               # coerce everything to polygons
               W  <- as.polygonal(W)
               tt <- lapply(tt, as.polygonal)
               # compute min dist from tile edges to window edges
               d <- sapply(tt, edist, b=edges(W))
             }
           },
           image={
             Xim <- X$image
             # compute boundary distance for each pixel
             bd <- bdist.pixels(as.owin(Xim), style="image")
             bd <- bd[W, drop=FALSE]
             # split over tiles
             bX <- split(bd, X)
             # compute minimum distance over each level of factor
             d <- sapply(bX, function(z) { summary(z)$min })
           }
           )
    return(d)
  }
  bdist.tiles
})


## ......... geometrical transformations ..................

shift.tess <- function(X, ...) {
  Y <- X
  Y$window <- wY <- shift(X$window, ...)
  vec <- getlastshift(wY)
  switch(X$type,
         rect={
           Y$xgrid <- Y$xgrid + vec[1]
           Y$ygrid <- Y$ygrid + vec[2]
         },
         tiled={
           Y$tiles <- lapply(Y$tiles, shift, vec=vec)
         },
         image = {
           Y$image <- shift(Y$image, vec)
         })
  attr(Y, "lastshift") <- vec
  return(Y)
}

affine.tess <- function(X, mat=diag(c(1,1)), vec=c(0,0), ...) {
  Y <- X
  Y$window <- affine(X$window, mat=mat, vec=vec, ...)
  switch(Y$type,
         rect = {
           if(all(mat == diag(diag(mat)))) {
             ## result is rectangular
             Y$xgrid <- sort(mat[1,1] * X$xgrid + vec[1])
             Y$ygrid <- sort(mat[2,2] * X$ygrid + vec[2])
           } else {
             ## shear transformation; treat rectangles as general tiles
             Y <- tess(tiles=tiles(X), window=Y$window)
             Y$tiles <- lapply(Y$tiles, affine, mat=mat, vec=vec, ...)
           }
         },
         tiled={
           Y$tiles <- lapply(Y$tiles, affine, mat=mat, vec=vec, ...)
         },
         image = {
           Y$image <- affine(Y$image, mat=mat, vec=vec, ...)
         })
  return(Y)
}

reflect.tess <- function(X) {
  Y <- X
  Y$window <- reflect(Y$window)
  switch(X$type,
         rect = {
           Y$xgrid <- rev(- Y$xgrid)
           Y$ygrid <- rev(- Y$ygrid)
         },
         tiled = {
           Y$tiles <- lapply(Y$tiles, reflect)
         },
         image = {
           Y$image <- reflect(Y$image)
         })
  return(Y)
}

flipxy.tess <- function(X) {
  Y <- X
  Y$window <- flipxy(Y$window)
  switch(X$type,
         rect = {
           Y$xgrid <- X$ygrid
           Y$ygrid <- X$xgrid
         },
         tiled = {
           Y$tiles <- lapply(Y$tiles, flipxy)
         },
         image = {
           Y$image <- flipxy(Y$image)
         })
  return(Y)
}

scalardilate.tess <- function(X, f, ...) {
  Y <- X
  Y$window <- scalardilate(X$window, f, ...)
  switch(X$type,
         rect = {
           Y$xgrid <- f * Y$xgrid
           Y$ygrid <- f * Y$ygrid
         },
         tiled = {
           Y$tiles <- lapply(Y$tiles, scalardilate, f=f, ...)
         },
         image = {
           Y$image <- scalardilate(Y$image, f=f, ...)
         })
  return(Y)
}

rotate.tess <- function(X, angle=pi/2, ..., centre=NULL) {
  if(angle %% (2 * pi) == 0) return(X)
  if(!is.null(centre)) {
    X <- shift(X, origin=centre)
    negorigin <- getlastshift(X)
  } else negorigin <- NULL
  Y <- X
  Y$window <- rotate(X$window, angle=angle, ...)
  switch(X$type,
         rect = {
           if(angle %% (pi/2) == 0) {
             ## result is rectangular
             co <- round(cos(angle))
             si <- round(sin(angle))
             Y$xgrid <- sort((if(co == 0) 0 else (co * X$xgrid)) -
                             (if(si == 0) 0 else (si * X$ygrid)))
             Y$ygrid <- sort((if(si == 0) 0 else (si * X$xgrid)) +
                             (if(co == 0) 0 else (co * X$ygrid)))
           } else {
             ## general tessellation
             Y <- tess(tiles=lapply(tiles(X), rotate, angle=angle, ...),
                       window=Y$window)
           }
         },
         tiled = {
           Y$tiles <- lapply(X$tiles, rotate, angle=angle, ...)
         },
         image = {
           Y$image <- rotate(X$image, angle=angle, ...)
         })
  if(!is.null(negorigin))
    Y <- shift(Y, -negorigin)
  return(Y)
}
  
as.data.frame.tess <- function(x, ...) {
  switch(x$type,
         rect =,
         tiled = {
           y <- lapply(tiles(x), as.data.frame, ...)
           z <- mapply(assignDFcolumn,
                       x=y, value=tilenames(x),
                       MoreArgs=list(name="Tile", ...),
                       SIMPLIFY=FALSE)
           z <- do.call(rbind, z)
           row.names(z) <- NULL
         },
         image = {
           z <- as.data.frame(x$image, ...)
           if(!is.na(m <- match("value", colnames(z))))
             colnames(z)[m] <- "Tile"
         },
         {
           z <- NULL
           warning("Unrecognised type of tessellation")
         })
  return(z)
}

connected.tess <- function(X, ...) {
  Xim <- as.im(X, ...)
  X <- as.tess(Xim)
  tilesX <- tiles(X)
  namesX <- names(tilesX)
  shards <- lapply(tilesX, connected) # list of factor images
  shardnames <- lapply(shards, levels)
  nshards <- lengths(shardnames)
  broken <- (nshards > 1)
  #' unbroken tiles keep their original tile names
  shardnames[!broken] <- namesX[!broken]
  #' shards of broken tiles are named "tilename[i] shard j"
  shardnames[broken] <- mapply(paste,
                               namesX[broken], "shard", shardnames[broken],
                               SIMPLIFY=FALSE)
  #' rename them
  shards <- mapply("levels<-", shards, shardnames, SIMPLIFY=FALSE)
  #' separate them
  shards <- lapply(lapply(shards, as.tess), tiles)
  shards <- unlist(shards, recursive=FALSE, use.names=FALSE)
  names(shards) <- unlist(shardnames)
  #' form tessellation
  result <- tess(tiles=shards, window=as.owin(Xim))
  result
}

#
#   plot.im.R
#
#  $Revision: 1.137 $   $Date: 2020/02/21 08:45:54 $
#
#  Plotting code for pixel images
#
#  plot.im
#  image.im
#  contour.im
#
###########################################################################

plot.im <- local({

  ## auxiliary functions

  image.doit <- function(imagedata, ...,
                         extrargs=graphicsPars("image"), W,
                         workaround=FALSE) {
    aarg <- resolve.defaults(...)
    add      <- resolve.1.default(list(add=FALSE),     aarg)
    show.all <- resolve.1.default(list(show.all=!add), aarg)
    addcontour <- resolve.1.default(list(addcontour=FALSE), aarg)
    args.contour <- resolve.1.default(list(args.contour=list()), aarg)
    ##
    if(add && show.all) {
      ## set up the window space *with* the main title
      ## using the same code as plot.owin, for consistency
      do.call.matched(plot.owin,
                      resolve.defaults(list(x=W, type="n"), aarg), 
                      extrargs=graphicsPars("owin"))
    }
    if(workaround && identical(aarg$useRaster, TRUE)) {
      #' workaround for bug 16035
      #' detect reversed coordinates
      usr <- par('usr')
      xrev <- (diff(usr[1:2]) < 0) 
      yrev <- (diff(usr[3:4]) < 0)
      if(xrev || yrev) {
        #' flip matrix of pixel values, because the device driver does not
        z <- imagedata$z
        d <- dim(z) # z is in the orientation expected for image.default
        if(xrev) z <- z[d[1]:1,       , drop=FALSE]
        if(yrev) z <- z[      , d[2]:1, drop=FALSE]
        imagedata$z <- z
      }
    }
    extrargs <- setdiff(extrargs, c("claim.title.space", "box"))
    z <- do.call.matched(image.default,
                         append(imagedata, aarg),
                         extrargs=extrargs)
    if(addcontour)
      do.call(do.contour,
              resolve.defaults(imagedata,
                               list(add=TRUE),
                               args.contour,
                               list(col=par('fg')),
                               aarg,
                               .StripNull=TRUE))
    return(z)
  }

  do.contour <- function(x, y, z, ..., drawlabels=TRUE) {
    nx <- length(x)
    ny <- length(y)
    nz <- dim(z)
    if(nx > nz[1]) {
      if(nz[1] == 1) {
        z <- rbind(z, z)
        nz <- dim(z)
        drawlabels <- FALSE
      } else {
        x <- (x[-1] + x[-nx])/2
        nx <- nx-1
      }
    }
    if(ny > nz[2]) {
      if(nz[2] == 1) {
        z <- cbind(z, z)
        nz <- dim(z)
        drawlabels <- FALSE
      } else {
        y <- (y[-1] + y[-ny])/2
        ny <- ny-1
      }
    }
    do.call.matched(contour.default,
                    list(x=x, y=y, z=z, ..., drawlabels=drawlabels))
  }
                 
  do.box.etc <- function(bb, add, argh) {
    do.call(box.etc, append(list(bb=bb, add=add), argh))
  }
  
  box.etc <- function(bb, ..., add=FALSE, box=!add,
                      axes=FALSE, ann=FALSE, xlab="", ylab="") {
    # axes for image
    xr <- bb$xrange
    yr <- bb$yrange
    if(box)
      rect(xr[1], yr[1], xr[2], yr[2])
    if(axes) {
      px <- prettyinside(xr)
      py <- prettyinside(yr)
      do.call.plotfun(graphics::axis,
                      resolve.defaults(
                                       list(side=1, at=px), 
                                       list(...),
                                       list(pos=yr[1])),
                      extrargs=graphicsPars("axis"))
      do.call.plotfun(graphics::axis,
                      resolve.defaults(
                                       list(side=2, at=py), 
                                       list(...),
                                       list(pos=xr[1])),
                      extrargs=graphicsPars("axis"))
    }
    ## axis labels xlab, ylab
    if(ann) {
      dox <- any(nzchar(xlab))
      doy <- any(nzchar(ylab))
      line0 <- if(axes) 1 else 0
      if(dox || doy) {
        mtargs <- resolve.defaults(list(...), list(line=line0))
        if(dox)
          do.call.matched(mtext, append(list(text=xlab, side=1), mtargs))
        if(doy)
          do.call.matched(mtext, append(list(text=ylab, side=2), mtargs))
      }
    }
  }
  
  
  clamp <- function(x, v, tol=0.02 * diff(v)) {
    ok <- (x >= v[1] - tol) & (x <= v[2] + tol)
    x[ok]
  }
  
  cellbreaks <- function(x, dx) {
    nx <- length(x)
    seq(x[1] - dx/2, x[nx] + dx/2, length.out=nx+1)
  }

  log10orNA <- function(x) {
    y <- rep(NA_real_, length(x))
    ok <- !is.na(x) & (x > 0)
    y[ok] <- log10(x[ok])
    return(y)
  }

  Ticks <- function(usr, log=FALSE, nint=NULL, ..., clip=TRUE) {
    #' modification of grDevices::axisTicks
    #'      constrains ticks to be inside the specified range if clip=TRUE
    #'      accepts nint=NULL as if it were missing
    z <- if(is.null(nint)) axisTicks(usr=usr, log=log, ...) else
         axisTicks(usr=usr, log=log, nint=nint, ...) 
    if(clip) {
      zlimits <- if(log) 10^usr else usr
      z <- z[inside.range(z, zlimits)]
    }
    return(z)
  }

  numericalRange <- function(x, zlim=NULL) {
    xr <- suppressWarnings(range(x, finite=TRUE))
    if(!all(is.finite(xr)))
      warning("All pixel values are NA", call.=FALSE)
    if(!is.null(zlim)) 
      xr <- suppressWarnings(range(xr, zlim, finite=TRUE))
    if(!all(is.finite(xr))) {
      warning("Cannot determine range of values for colour map",
              call.=FALSE)
      xr <- c(0,0)
    }
    return(xr)
  }
  
  # main function
  PlotIm <- function(x, ...,
                     main, 
                     add=FALSE, clipwin=NULL,
                     col=NULL, valuesAreColours=NULL, log=FALSE,
                     ncolours=256, gamma=1, 
                     ribbon=show.all, show.all=!add,
                     ribside=c("right", "left", "bottom", "top"),
                     ribsep=0.15, ribwid=0.05, ribn=1024,
                     ribscale=1, ribargs=list(), riblab=NULL, colargs=list(),
                     useRaster=NULL, workaround=FALSE, zap=1,
                     do.plot=TRUE) {
    if(missing(main)) main <- short.deparse(substitute(x))
    verifyclass(x, "im")
    if(x$type == "complex") {
      cl <- match.call()
      cl$x <- solist(Re=Re(x), Im=Im(x), Mod=Mod(x), Arg=Arg(x))
      cl[[1]] <- as.name('plot')
      cl$main <- main
      out <- eval(cl, parent.frame())
      return(invisible(out))
    }
    ribside <- match.arg(ribside)
    col.given <- !is.null(col)
    dotargs <- list(...)

    stopifnot(is.list(ribargs))
    user.ticks <- ribargs$at
    user.nint <- ribargs$nint
    
    if(!is.null(clipwin)) {
      x <- x[as.rectangle(clipwin)]
      if(!is.rectangle(clipwin)) x <- x[clipwin, drop=FALSE]
    }

    zlim <- dotargs$zlim

    x <- repair.image.xycoords(x)

    xtype <- x$type
    xbox <- as.rectangle(x)
    
    do.log <- identical(log, TRUE)
    if(do.log && !(x$type %in% c("real", "integer")))
      stop(paste("Log transform is undefined for an image of type",
                 sQuote(xtype)))

    # determine whether pixel values are to be treated as colours
    if(!is.null(valuesAreColours)) {
      # argument given - validate
      stopifnot(is.logical(valuesAreColours))
      if(valuesAreColours) {
        ## pixel values must be factor or character
        if(!xtype %in% c("factor", "character")) {
          if(do.plot)
            warning(paste("Pixel values of type", sQuote(xtype),
                          "are not interpretable as colours"))
          valuesAreColours <- FALSE
        } else if(col.given) {
          ## colour info provided: contradictory
          if(do.plot)
            warning(paste("Pixel values are taken to be colour values,",
                          "because valuesAreColours=TRUE;", 
                          "the colour map (argument col) is ignored"),
                    call.=FALSE)
          col <- NULL
        }
        if(do.log && do.plot) 
          warning(paste("Pixel values are taken to be colour values,",
                        "because valuesAreColours=TRUE;", 
                        "the argument log=TRUE is ignored"),
                  call.=FALSE)
      }
    } else if(col.given) {
      # argument 'col' controls colours
      valuesAreColours <- FALSE
    } else if(spatstat.options("monochrome")) {
      valuesAreColours <- FALSE
    } else {
      ## default : determine whether pixel values are colours
      strings <- switch(xtype,
                        character = { as.vector(x$v) },
                        factor    = { levels(x) },
                        { NULL })
      valuesAreColours <- is.character(strings) && 
      !inherits(try(col2rgb(strings), silent=TRUE), "try-error")
      if(valuesAreColours && do.plot)
        splat("Interpreting pixel values as colours",
              "(valuesAreColours=TRUE)")
    }
    # 
    if(valuesAreColours) {
      # colour-valued images are plotted using the code for factor images
      # with the colour map equal to the levels of the factor
      switch(xtype,
             factor = {
               col <- levels(x)
             },
             character = {
               x <- eval.im(factor(x))
               xtype <- "factor"
               col <- levels(x)
             },
             {
               if(do.plot)
                 warning(paste("Pixel values of type", sQuote(xtype),
                               "are not interpretable as colours"))
             })
      # colours not suitable for ribbon
      ribbon <- FALSE
    } 
    
    # transform pixel values to log scale?
    if(do.log) {
      rx <- range(x, finite=TRUE)
      if(all(rx > 0)) {
        x <- eval.im(log10(x))
      } else {
        if(do.plot && any(rx < 0)) 
          warning(paste("Negative pixel values",
                        "omitted from logarithmic colour map;",
                        "range of values =", prange(rx)),
                  call.=FALSE)
        if(do.plot && !all(rx < 0))
          warning("Zero pixel values omitted from logarithmic colour map",
                  call.=FALSE)
        x <- eval.im(log10orNA(x))
      } 
      xtype <- x$type
      Log <- log10
      Exp <- function(x) { 10^x }
    } else {
      Log <- Exp <- function(x) { x }
    }
    
    imagebreaks <- NULL
#    ribbonvalues <- ribbonbreaks <- NULL
    ribbonvalues <- NULL

    ## NOW DETERMINE THE COLOUR MAP
    colfun <- colmap <- NULL
    if(valuesAreColours) {
      ## pixel values are colours; set of colours was determined earlier
      colmap <- colourmap(col=col, inputs=col)
    } else if(!col.given) {
      ## no colour information given: use default
      colfun <- spatstat.options("image.colfun")
    } else if(inherits(col, "colourmap")) {
      ## Bob's your uncle
      colmap <- col
    } else if(is.function(col)) {
      ## Some kind of function determining a colour map
      if(names(formals(col))[1] == "n") {
        ## function(n) -> colour values
        colfun <- col
      } else {
        ## colour map determined by a rule (e.g. 'beachcolours')
        colmap <- invokeColourmapRule(col, x, zlim=zlim, colargs=colargs)
        if(is.null(colmap))
          stop("Unrecognised syntax for colour function")
      }
    }

    switch(xtype,
           real    = {
             vrange <- numericalRange(x, zlim)
             if(!is.null(colmap)) {
               # explicit colour map
               s <- summary(colmap)
               if(s$discrete)
                 stop("Discrete colour map is not applicable to real values")
               imagebreaks <- s$breaks
               vrange <- range(imagebreaks)
               col <- s$outputs
             } 
             trivial <- (diff(vrange) <= zap * .Machine$double.eps)
             #' ribbonvalues: domain of colour map (pixel values)
             #' ribbonrange: (min, max) of pixel values in image
             #' nominalrange: range of values shown on ribbon 
             #' nominalmarks: values shown on ribbon at tick marks
             #' ribbonticks: pixel values of tick marks 
             #' ribbonlabels: text displayed at tick marks
             ribbonvalues <- if(trivial) vrange[1] else
                             seq(from=vrange[1], to=vrange[2], length.out=ribn)
             ribbonrange <- vrange
             nominalrange <- Log(ribscale * Exp(ribbonrange))
             nominalmarks <- if(trivial) nominalrange[1] else
                             (user.ticks %orifnull% Ticks(nominalrange,
                                                         log=do.log,
                                                         nint=user.nint))
             ribbonticks <- Log(nominalmarks/ribscale)
             ribbonlabels <- paste(nominalmarks)
           },
           integer = {
             values <- as.vector(x$v)
             values <- values[!is.na(values)]
             uv <- unique(values)
             vrange <- numericalRange(uv, zlim)
             nvalues <- length(uv)
             trivial <- (nvalues < 2)
             if(!trivial){
               nominalrange <- Log(ribscale * Exp(vrange))
               if(!is.null(user.ticks)) {
                 nominalmarks <- user.ticks
               } else {
                 nominalmarks <- Ticks(nominalrange,
                                       log=do.log,
                                       nint = user.nint)
                 nominalmarks <- nominalmarks[nominalmarks %% 1 == 0]
               }
               ribbonticks <- Log(nominalmarks/ribscale)
               ribbonlabels <- paste(nominalmarks)
               if(!do.log && isTRUE(all.equal(ribbonticks,
                                              vrange[1]:vrange[2]))) {
                 # each possible pixel value will appear in ribbon
                 ribbonvalues <- vrange[1]:vrange[2]
                 imagebreaks <- c(ribbonvalues - 0.5, vrange[2] + 0.5)
                 ribbonrange <- range(imagebreaks)
                 ribbonticks <- ribbonvalues
                 ribbonlabels <- paste(ribbonticks * ribscale)
               } else {
                 # not all possible values will appear in ribbon
                 ribn <- min(ribn, diff(vrange)+1)
                 ribbonvalues <- seq(from=vrange[1], to=vrange[2],
                                     length.out=ribn)
                 ribbonrange <- vrange
               }
             }
             if(!is.null(colmap)) {
               # explicit colour map
               s <- summary(colmap)
               imagebreaks <-
                 if(!s$discrete) s$breaks else
                 c(s$inputs[1] - 0.5, s$inputs + 0.5)
               col <- s$outputs
             }
           },
           logical = {
             values <- as.integer(as.vector(x$v))
             values <- values[!is.na(values)]
             uv <- unique(values)
             trivial <- (length(uv) < 2)
             vrange <- c(0,1)
             imagebreaks <- c(-0.5, 0.5, 1.5)
             ribbonvalues <- c(0,1)
             ribbonrange <- range(imagebreaks)
#             ribbonbreaks <- imagebreaks
             ribbonticks <- user.ticks %orifnull% ribbonvalues
             ribbonlabels <- c("FALSE", "TRUE")
             if(!is.null(colmap)) 
               col <- colmap(c(FALSE,TRUE))
           },
           factor  = {
             lev <- levels(x)
             nvalues <- length(lev)
             trivial <- (nvalues < 2)
             # ensure all factor levels plotted separately
             fac <- factor(lev, levels=lev)
             intlev <- as.integer(fac)
             imagebreaks <- c(intlev - 0.5, max(intlev) + 0.5)
             ribbonvalues <- intlev
             ribbonrange <- range(imagebreaks)
#             ribbonbreaks <- imagebreaks
             ribbonticks <- user.ticks %orifnull% ribbonvalues
             ribbonlabels <- paste(lev)
             vrange <- range(intlev)
             if(!is.null(colmap) && !valuesAreColours) 
               col <- colmap(fac)
           },
           character  = {
             x <- eval.im(factor(x))
             lev <- levels(x)
             nvalues <- length(lev)
             trivial <- (nvalues < 2)
             # ensure all factor levels plotted separately
             fac <- factor(lev, levels=lev)
             intlev <- as.integer(fac)
             imagebreaks <- c(intlev - 0.5, max(intlev) + 0.5)
             ribbonvalues <- intlev
             ribbonrange <- range(imagebreaks)
#             ribbonbreaks <- imagebreaks
             ribbonticks <- user.ticks %orifnull% ribbonvalues
             ribbonlabels <- paste(lev)
             vrange <- range(intlev)
             if(!is.null(colmap)) 
               col <- colmap(fac)
           },
           stop(paste("Do not know how to plot image of type", sQuote(xtype)))
           )
  
    ## Compute colour values to be passed to image.default
    if(!is.null(colmap)) {
      ## Explicit colour map object
      colourinfo <- list(breaks=imagebreaks, col=col)
    } else if(!is.null(colfun)) {
      ## Function colfun(n)
      if(trivial) ncolours <- 1
      colourinfo <-
        if(is.null(imagebreaks)) list(col=colfun(ncolours)) else
        list(breaks=imagebreaks, col=colfun(length(imagebreaks) - 1L))
    } else if(col.given) {
      ## Colour values
      if(inherits(try(col2rgb(col), silent=TRUE), "try-error"))
        stop("Unable to interpret argument col as colour values")
      if(is.null(imagebreaks)) {
        colourinfo <- list(col=col)
      } else {
        nintervals <- length(imagebreaks) - 1
        colourinfo <- list(breaks=imagebreaks, col=col)
        if(length(col) != nintervals)
          stop(paste("Length of argument", dQuote("col"),
                     paren(paste(length(col))),
                     "does not match the number of distinct values",
                     paren(paste(nintervals))))
      }
    } else stop("Internal error: unable to determine colour values")

    if(spatstat.options("monochrome")) {
      ## transform to grey scale
      colourinfo$col <- to.grey(colourinfo$col)
    }
    
    # colour map to be returned (invisibly)
    i.col <- colourinfo$col
    i.bks <- colourinfo$breaks
    output.colmap <-
      if(is.null(i.col)) NULL else
      if(inherits(i.col, "colourmap")) i.col else
      if(valuesAreColours) colourmap(col=i.col, inputs=i.col) else
      switch(xtype,
             integer=,
             real= {
               if(!is.null(i.bks)) {
                 colourmap(col=i.col, breaks=i.bks)
               } else colourmap(col=i.col, range=vrange, gamma=gamma)
             },
             logical={
               colourmap(col=i.col, inputs=c(FALSE,TRUE))
             },
             character=,
             factor={
               colourmap(col=i.col, inputs=lev)
             },
             NULL)

    ## gamma correction
    soc <- summary(output.colmap)
    if(!is.null(gamma <- soc$gamma) && gamma != 1)
      colourinfo$breaks <- soc$breaks

    ##  ........ decide whether to use rasterImage .........
    
    ## get device capabilities
    ##      (this will start a graphics device if none is active)
    rasterable <- dev.capabilities()$rasterImage
    if(is.null(rasterable)) rasterable <- "no"
    ##
    can.use.raster <-
      switch(rasterable,
             yes=TRUE,
             no=FALSE,
             "non-missing"=!anyNA(x$v),
             FALSE)
    if(is.null(useRaster)) {
      useRaster <- can.use.raster
    } else if(useRaster && !can.use.raster) {
        whinge <- "useRaster=TRUE is not supported by the graphics device"
        if(rasterable == "non-missing")
          whinge <- paste(whinge, "for images with NA values")
        warning(whinge, call.=FALSE)
    } 
    
    ## ........ start plotting .................

    if(!identical(ribbon, TRUE) || trivial) {
      ## no ribbon wanted

      attr(output.colmap, "bbox") <- as.rectangle(x)
      if(!do.plot)
        return(output.colmap)

      ## plot image without ribbon
      image.doit(imagedata=list(x=cellbreaks(x$xcol, x$xstep),
                                y=cellbreaks(x$yrow, x$ystep),
                                z=t(x$v)),
                 W=xbox,
                 workaround=workaround,
                 ##
                 list(axes=FALSE, xlab="",ylab=""), 
                 dotargs,
                 list(useRaster=useRaster, add=add, show.all=show.all),
                 colourinfo,
                 list(zlim=vrange),
                 list(asp = 1, main = main))
##      if(add && show.all)
##        fakemaintitle(x, main, dotargs)

      do.box.etc(Frame(x), add, dotargs)
      
      return(invisible(output.colmap))
    }
    
    # determine plot region
    bb <- owin(x$xrange, x$yrange)
    Width <- diff(bb$xrange)
    Height <- diff(bb$yrange)
    Size <- max(Width, Height)
    switch(ribside,
           right={
             # ribbon to right of image
             bb.rib <- owin(bb$xrange[2] + c(ribsep, ribsep+ribwid) * Size,
                            bb$yrange)
             rib.iside <- 4
           },
           left={
             # ribbon to left of image
             bb.rib <- owin(bb$xrange[1] - c(ribsep+ribwid, ribsep) * Size,
                            bb$yrange)
             rib.iside <- 2
           },
           top={
             # ribbon above image
             bb.rib <- owin(bb$xrange,
                            bb$yrange[2] + c(ribsep, ribsep+ribwid) * Size)
             rib.iside <- 3
           },
           bottom={
             # ribbon below image
             bb.rib <- owin(bb$xrange,
                            bb$yrange[1] - c(ribsep+ribwid, ribsep) * Size)
             rib.iside <- 1
           })
    bb.all <- boundingbox(bb.rib, bb)

    attr(output.colmap, "bbox") <- bb.all
    attr(output.colmap, "bbox.legend") <- bb.rib
    attr(output.colmap, "side.legend") <- rib.iside
    if(!do.plot)
      return(output.colmap)

    pt <- prepareTitle(main)
    
    if(!add) {
      ## establish coordinate system
      do.call.plotfun(plot.owin,
                      resolve.defaults(list(x=bb.all,
                                            type="n",
                                            main=pt$blank),
                                       dotargs),
                      extrargs=graphicsPars("owin"))
    }
    if(show.all) {
      ## plot title centred over main image area 'bb'
      do.call.plotfun(plot.owin,
                      resolve.defaults(list(x=bb,
                                            type="n",
                                            main=main,
                                            add=TRUE,
                                            show.all=TRUE),
                                       dotargs),
                      extrargs=graphicsPars("owin"))
      main <- ""
    }
    # plot image
    image.doit(imagedata=list(x=cellbreaks(x$xcol, x$xstep),
                              y=cellbreaks(x$yrow, x$ystep),
                              z=t(x$v)),
               W=xbox,
               workaround=workaround,
               list(add=TRUE, show.all=show.all),
               list(axes=FALSE, xlab="", ylab=""),
               dotargs,
               list(useRaster=useRaster),
               colourinfo,
               list(zlim=vrange),
               list(asp = 1, main = main))

##    if(add && show.all)
##      fakemaintitle(bb.all, main, ...)
    
    # box or axes for image
    do.box.etc(bb, add, dotargs)

    # plot ribbon image containing the range of image values
    rib.npixel <- length(ribbonvalues) + 1
    switch(ribside,
           left=,
           right={
             # vertical ribbon
             rib.xcoords <- bb.rib$xrange
             rib.ycoords <- seq(from=bb.rib$yrange[1],
                                to=bb.rib$yrange[2],
                                length.out=rib.npixel)
             rib.z <- matrix(ribbonvalues, ncol=1)
             rib.useRaster <- useRaster
           },
           top=,
           bottom={
             # horizontal ribbon
             rib.ycoords <- bb.rib$yrange
             rib.xcoords <- seq(from=bb.rib$xrange[1],
                                to=bb.rib$xrange[2],
                                length.out=rib.npixel)
             rib.z <- matrix(ribbonvalues, nrow=1)
             # bug workaround
             rib.useRaster <- FALSE 
           })
    image.doit(imagedata=list(x=rib.xcoords,
                              y=rib.ycoords,
                              z=t(rib.z)),
               W=bb.rib,
               workaround=workaround,
               list(add=TRUE,
                    show.all=show.all),
               ribargs,
               list(useRaster=rib.useRaster),
               list(main="", sub="", xlab="", ylab=""),
               dotargs,
               colourinfo)
    # box around ribbon?
    resol <- resolve.defaults(ribargs, dotargs)
    if(!identical(resol$box, FALSE))
      plot(as.owin(bb.rib), add=TRUE)
    # scale axis for ribbon image
    ribaxis <- !(identical(resol$axes, FALSE) || identical(resol$ann, FALSE))
    if(ribaxis) {
      ribaxis.iside <- rib.iside
      ## check for user-supplied xlim, ylim with reverse order
      ll <- resolve.defaults(ribargs, dotargs, list(xlim=NULL, ylim=NULL))
      xlimflip <- is.numeric(ll$xlim) && (diff(ll$xlim) < 0)
      ylimflip <- is.numeric(ll$ylim) && (diff(ll$ylim) < 0)
      if(xlimflip) ribaxis.iside <- c(1, 4, 3, 2)[ribaxis.iside] 
      if(ylimflip) ribaxis.iside <- c(3, 2, 1, 4)[ribaxis.iside]
      ##
      axisargs <- list(side=ribaxis.iside, labels=ribbonlabels)
      switch(ribside,
             right={
               scal <- diff(bb.rib$yrange)/diff(ribbonrange)
               at <- bb.rib$yrange[1] + scal * (ribbonticks - ribbonrange[1])
               axisargs <- append(axisargs, list(at=at))
               posargs <- list(pos=bb.rib$xrange[2],
                               yaxp=c(bb.rib$yrange, length(ribbonticks)))
             },
             left={
               scal <- diff(bb.rib$yrange)/diff(ribbonrange)
               at <- bb.rib$yrange[1] + scal * (ribbonticks - ribbonrange[1])
               axisargs <- append(axisargs, list(at=at))
               posargs <- list(pos=bb.rib$xrange[1],
                               yaxp=c(bb.rib$yrange, length(ribbonticks)))
             },
             top={
               scal <- diff(bb.rib$xrange)/diff(ribbonrange)
               at <- bb.rib$xrange[1] + scal * (ribbonticks - ribbonrange[1])
               axisargs <- append(axisargs, list(at=at))
               posargs <- list(pos=bb.rib$yrange[2],
                               xaxp=c(bb.rib$xrange, length(ribbonticks)))
             },
             bottom={
               scal <- diff(bb.rib$xrange)/diff(ribbonrange)
               at <- bb.rib$xrange[1] + scal * (ribbonticks - ribbonrange[1])
               axisargs <- append(axisargs, list(at=at))
               posargs <- list(pos=bb.rib$yrange[1],
                               xaxp=c(bb.rib$xrange, length(ribbonticks)))
             })
      do.call.plotfun(graphics::axis,
                      resolve.defaults(axisargs, ribargs, dotargs, posargs),
                      extrargs=graphicsPars("axis"))
    }
    if(!is.null(riblab)) {
      riblabel <- if(is.list(riblab)) riblab else list(text=riblab)
      riblabel$side <- rib.iside
      do.call(mtext, riblabel)
    }
    #
    return(invisible(output.colmap))
  }

  PlotIm
})

invokeColourmapRule <- function(colfun, x, ..., zlim=NULL, colargs=list()) {
  ## utility for handling special functions that generate colour maps
  ## either 
  ##        function(... range) -> colourmap
  ##        function(... inputs) -> colourmap
  stopifnot(is.im(x))
  stopifnot(is.function(colfun))
  colargnames <- names(formals(colfun))
  ## Convert it to a 'colourmap'
  colmap <- NULL
  xtype <- x$type
  if(xtype %in% c("real", "integer") && "range" %in% colargnames) {
    ## function(range) -> colourmap
    vrange <- range(range(x, finite=TRUE), zlim)
    cvals <- try(do.call.matched(colfun,
                                 append(list(range=vrange), colargs)),
                 silent=TRUE)
    if(!inherits(cvals, "try-error")) {
      colmap <- if(inherits(cvals, "colourmap")) cvals else
      if(is.character(cvals)) colourmap(cvals, range=vrange) else NULL
    }
  } else if(xtype != "real" && "inputs" %in% colargnames) {
    ## function(inputs) -> colourmap
    vpossible <- switch(xtype,
                        logical = c(FALSE, TRUE),
                        factor = levels(x),
                        unique(as.matrix(x)))
    if(!is.null(vpossible) && length(vpossible) < 256) {
      cvals <- try(do.call.matched(colfun,
                                   append(list(inputs=vpossible),
                                          colargs)),
                   silent=TRUE)
      if(!inherits(cvals, "try-error")) {
        colmap <- if(inherits(cvals, "colourmap")) cvals else
        if(is.character(cvals))
          colourmap(cvals, inputs=vpossible) else NULL
      }
    }
  }
  return(colmap)
}

########################################################################

image.im <- plot.im

######################################################################

contour.im <- function (x, ..., main, axes=FALSE, add=FALSE,
                        col=par("fg"), 
                        clipwin=NULL, show.all=!add, do.plot=TRUE)
{
  defaultmain <- deparse(substitute(x))
  dotargs <- list(...)
  bb <- Frame(x)
  ## return value
  result <- bb
  attr(result, "bbox") <- bb
  if(!do.plot) return(result)
  ## main title
  sop <- spatstat.options("par.contour")
  if(missing(main)) 
    main <- resolve.1.default(list(main=defaultmain), sop)
  pt <- prepareTitle(main)
  ## plotting parameters
  if(missing(add)) {
    force(add) ## use default in formal arguments, unless overridden
    add <- resolve.1.default(list(add=add), sop)
  }
  if(missing(axes)) {
    force(axes)
    axes <- resolve.1.default(list(axes=axes), sop)
  }
  axes <- axes && !add
  col0 <- if(inherits(col, "colourmap")) par("fg") else col
  ## clip to subset
  if(!is.null(clipwin))
    x <- x[clipwin, drop=FALSE]
  #' start plotting
  if(!add) {
    ## new plot - establish coordinate system
    if(axes && show.all) {
      #' standard plot initialisation in base graphics
      do.call.plotfun(plot.default,
                      resolve.defaults(
                                       list(x = range(x$xcol),
                                            y = range(x$yrow),
                                            type = "n"),
                                       list(...),
                                       list(asp = 1,
                                            xlab = "x",
                                            ylab = "y",
                                            col = col0,
                                            main = main)))
    } else {
      #' plot invisible bounding box
      do.call.plotfun(plot.owin,
                      resolve.defaults(list(x=bb,
                                            type="n",
                                            main=pt$blank),
                                       dotargs),
                      extrargs=graphicsPars("owin"))
    }
  } 
  if(show.all && !axes) {
    ## plot title centred over contour region
    do.call.plotfun(plot.owin,
                    resolve.defaults(list(x=bb,
                                          main=main,
                                          add=TRUE,
                                          show.all=TRUE),
                                     dotargs,
                                     list(col.main=col0)),
                    extrargs=graphicsPars("owin"))
  }
  #' plot contour lines
  if(!inherits(col, "colourmap")) {
    do.call.plotfun(contour.default,
                    resolve.defaults(list(x=x$xcol, y=x$yrow, z=t(x$v)),
                                     list(add=TRUE, col=col),
                                     list(...)))
  } else {
    clin <- do.call.matched(contourLines,
                            append(list(x=x$xcol, y=x$yrow, z=t(x$v)),
                                   list(...)))
    linpar <- graphicsPars("lines")
    for(i in seq_along(clin)) {
      lini <- clin[[i]]
      levi <- lini$level
      coli <- col(levi)
      argi <- resolve.defaults(lini[c("x", "y")],
                               list(...),
                               list(col=coli))
      do.call.matched(lines.default, argi, extrargs=linpar)
    }
  }
  return(invisible(result))
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#'  tests/aucroc.R
#'
#'  AUC and ROC code
#'
#'  $Revision: 1.4 $ $Date: 2020/04/28 12:58:26 $

local({
  if(FULLTEST) {
    A <- roc(spiders, "x")
    B <- auc(spiders, "y")
    fit <- kppm(redwood ~ I(y-x))
    a <- roc(fit)
    b <- auc(fit)
    fet <- ppm(amacrine~x+y+marks)
    d <- roc(fet)
    e <- auc(fet)
    fut <- lppm(spiders ~ I(y-x))
    f <- roc(fut)
    g <- auc(fut)
  }
})
## badwindowcheck.R
## $Revision: 1.3 $  $Date: 2020/04/28 12:58:26 $
##

local({
  if(ALWAYS) {
    ## Simple example of self-crossing polygon
    x <- read.table("selfcross.txt", header=TRUE)
    ## Auto-repair
    w <- owin(poly=x)

    ## Real data involving various quirks
    b <- read.table("badwindow.txt", header=TRUE)
    b <- split(b, factor(b$i))
    b <- lapply(b, function(z) { as.list(z[,-3]) })
    ## make owin without checking
    W <- owin(poly=b, check=FALSE, fix=FALSE)
    ## Apply stringent checks
    owinpolycheck(W,verbose=FALSE)
    ## Auto-repair
    W2 <- owin(poly=b)
  }
})




## tests/cdf.test.R

local({
  AC <- split(ants, un=FALSE)$Cataglyphis
  AM <- split(ants, un=FALSE)$Messor
  DM <- distmap(AM)
  if(ALWAYS) {
    ## (1) check cdf.test with strange data
    ## Marked point patterns with some marks not represented
    ## should produce a warning, rather than a crash:
    cdf.test(AC, DM)
  }
  if(FULLTEST) {
    ## should be OK:
    cdf.test(unmark(AC), DM)
    cdf.test(unmark(AC), DM, "cvm")
    cdf.test(unmark(AC), DM, "ad")
    ## other code blocks
    cdf.test(finpines, "x")
  }
  if(ALWAYS) {
    ## (2) linear networks
    set.seed(42)
    X <- runiflpp(20, simplenet)
    cdf.test(X, "x")
  }
  if(FULLTEST) {
    cdf.test(X, "x", "cvm")
    cdf.test(X %mark% runif(20), "x")
  }
  if(ALWAYS) {
    fit <- lppm(X ~1)
    cdf.test(fit, "y", "cvm")
  }
  if(FULLTEST) {
    cdf.test(fit, "y")
    cdf.test(fit, "y", "ad")
  }
  if(FULLTEST) {
    ## marked
    cdf.test(chicago, "y")
    cdf.test(subset(chicago, marks != "assault"), "y")
  }

  if(FULLTEST) {
    ## (3) Monte Carlo test for Gibbs model
    fit <- ppm(cells ~ 1, Strauss(0.07))
    cdf.test(fit, "x", nsim=9)

    ## cdf.test.slrm
    fut <- slrm(japanesepines ~ x + y)
    Z <- distmap(japanesepines)
    cdf.test(fut, Z)
  }
})
#'    tests/circular.R
#'
#'    Circular data and periodic distributions
#'
#'    $Revision: 1.4 $  $Date: 2020/04/28 12:58:26 $


local({
  if(ALWAYS) {
    a <- pairorient(redwood, 0.05, 0.15, correction="none")
    rose(a)
  }
  if(FULLTEST) {
    b <- pairorient(redwood, 0.05, 0.15, correction="best")
    rose(b, start="N", clockwise=TRUE)
  }
  if(ALWAYS) {
    #' arcs on the circle 
    #'       (depends on numerical behaviour)
    set.seed(19171025)
    aa <- replicate(7, runif(1, 0, 2*pi) + c(0, runif(1, 0, pi)),
                    simplify=FALSE)
    bb <- circunion(aa)

    assertsingle <- function(x, a, id) {
      y <- circunion(x)
      if(length(y) != 1 || max(abs(y[[1]] - a)) > .Machine$double.eps)
        stop(paste("Incorrect result from circunion in case", id),
             call.=FALSE)
      invisible(NULL)
    }

    assertsingle(list(c(pi/3, pi), c(pi/2, 3*pi/2)),
                 c(pi/3, 3*pi/2),
                 1)
    assertsingle(list(c(0, pi/2), c(pi/4, pi)),
                 c(0,pi),
                 2)
    assertsingle(list(c(-pi/4, pi/2), c(pi/4, pi)),
                 c((2-1/4)*pi, pi),
                 3)
  }
})

  
##  tests/closeshave.R
## check 'closepairs/crosspairs' code
## validity and memory allocation
## $Revision: 1.23 $ $Date: 2020/04/28 12:58:26 $

## ------- All this code must be run on every hardware -------
local({
  r <- 0.12
  close.all <- closepairs(redwood, r)
  close.ij <- closepairs(redwood, r, what="indices")
  close.ijd <- closepairs(redwood, r, what="ijd")
  close.every <- closepairs(redwood, r, what="all", distinct=FALSE)
  stopifnot(identical(close.ij, close.all[c("i","j")]))
  stopifnot(identical(close.ijd, close.all[c("i","j","d")]))

  #' test memory overflow code
  close.cigar <- closepairs(redwood, r, what="ijd", nsize=2)
  close.cigar <- closepairs(redwood, r, what="ijd", nsize=2, periodic=TRUE)
  
  Y <- split(amacrine)
  on <- Y$on
  off <- Y$off
  cross.all <- crosspairs(on, off, r)
  cross.ij <- crosspairs(on, off, r, what="indices")
  cross.ijd <- crosspairs(on, off, r, what="ijd")
  cross.every <- crosspairs(on, off, r, what="all", distinct=FALSE)
  stopifnot(identical(cross.ij, cross.all[c("i","j")]))
  stopifnot(identical(cross.ijd, cross.all[c("i","j","d")]))

  # closethresh vs closepairs: EXACT agreement
  thresh <- 0.08
  clt <- closethresh(redwood, r, thresh)
  cl <- with(closepairs(redwood, r),
             list(i=i, j=j, th = (d <= thresh)))
  if(!identical(cl, clt))
    stop("closepairs and closethresh disagree")

  reordered <- function(a) {
    o <- with(a, order(i,j))
    as.list(as.data.frame(a)[o,,drop=FALSE])
  }
  samesame <- function(a, b) {
    identical(reordered(a), reordered(b))
  }
  
  ## ...............................................
  #' compare with older, slower code
  op <- spatstat.options(closepairs.newcode=FALSE,
                         closepairs.altcode=FALSE,
                         crosspairs.newcode=FALSE)
  ## ...............................................
  old.close.ij <- closepairs(redwood, r, what="indices")
  old.cross.ij <- crosspairs(on, off, r, what="indices")
  stopifnot(samesame(close.ij, old.close.ij))
  stopifnot(samesame(cross.ij, old.cross.ij))
  # execute only:
  old.close.every <- closepairs(redwood, r, what="all", distinct=FALSE)
  old.close.once <- closepairs(redwood, r, what="all", twice=FALSE)
  #' test memory overflow code
  old.close.cigar <- closepairs(redwood, r, what="ijd", nsize=2)
  old.close.cigar <- closepairs(redwood, r, what="ijd", nsize=2, periodic=TRUE)
  
  ## ...............................................
  spatstat.options(op)
  ## ...............................................

  ## ...............................................
  #' alternative code - execution only
  op <- spatstat.options(closepairs.newcode=FALSE,
                         closepairs.altcode=TRUE)
  alt.close.ij <- closepairs(redwood, r, what="indices")
  alt.close.ijd <- closepairs(redwood, r, what="ijd")
  alt.close.all <- closepairs(redwood, r, what="all")
  #' test memory overflow code
  alt.close.cigar <- closepairs(redwood, r, what="ijd", nsize=2)
  alt.close.cigar <- closepairs(redwood, r, what="ijd", nsize=2, periodic=TRUE)
  spatstat.options(op)
  ## ...............................................
  
  # Rasmus' example
  R <- 0.04
  U <- as.ppp(gridcenters(owin(), 50, 50), W=owin())
  cp <- crosspairs(U, U, R)
  G <- matrix(0, npoints(U), npoints(U))
  G[cbind(cp$i, cp$j)] <- 1
  if(!isSymmetric(G))
    stop("crosspairs is not symmetric in Rasmus example")

  #' periodic distance
  pclose <- function(X, R, method=c("raw", "C")) {
    method <- match.arg(method)
    switch(method,
           raw = {
             D <- pairdist(X, periodic=TRUE)
             diag(D) <- Inf
             result <- which(D <= R, arr.ind=TRUE)
           },
           C = {
             result <- closepairs(X, R, periodic=TRUE, what="indices")
           })
    result <- as.data.frame(result)
    colnames(result) <- c("i","j")
    return(result)
  }
  #' pick a threshold value which avoids GCC bug 323
  RR <- 0.193
  A <- pclose(redwood, RR, "raw")
  B <- pclose(redwood, RR, "C")
  if(!samesame(A,B))
    stop("closepairs.ppp(periodic=TRUE) gives wrong answer")

  #' other functions that don't have a help file
  niets <- crosspairquad(quadscheme(cells), 0.1)

  #' other code blocks
  u <- closepairs(cells, 0.09, periodic=TRUE, what="all")
  v <- closepairs(cells, 0.07, twice=FALSE, neat=TRUE)
  #' tight cluster - guess count does not work
  Xc <- runifpoint(100, square(0.01))
  Window(Xc) <- square(1)
  z <- closepairs(Xc, 0.02, what="indices", distinct=FALSE)
  z <- closepairs(Xc, 0.02, what="ijd",     distinct=FALSE)
  z <- closepairs(Xc, 0.02, what="all",     distinct=FALSE)
  #' same task, older code
  aop <- spatstat.options(closepairs.newcode=FALSE)
  z <- closepairs(Xc, 0.02, what="indices", distinct=FALSE)
  z <- closepairs(Xc, 0.02, what="ijd",     distinct=FALSE)
  z <- closepairs(Xc, 0.02, what="all",     distinct=FALSE)
  spatstat.options(aop)
})

local({
  #' Three-dimensional
  X <- runifpoint3(100)
  cl <- closepairs(X, 0.2, what="indices")
  cl <- closepairs(X, 0.2, what="ijd")
  cl <- closepairs(X, 0.2, distinct=FALSE)
  cl <- closepairs(X, 0.2, distinct=FALSE, what="indices")
  cl <- closepairs(X, 0.2, distinct=FALSE, what="ijd")
  cl <- closepairs(X, 0.2, twice=FALSE, neat=TRUE)
  #' Test memory overflow code
  cl <- closepairs(X, 0.2, what="ijd", nsize=2)
  #' trap obsolete usage
  cl <- closepairs(X, 0.2, ordered=FALSE)
  #' crosspairs
  Y <- runifpoint3(100)
  cr <- crosspairs(X, Y, 0.2, what="indices")
  cr <- crosspairs(X, Y, 0.2, what="ijd")
  #' Test memory overflow code
  cr <- crosspairs(X, Y, 0.2, what="ijd", nsize=2)
  #' markmarkscatter uses closepairs.pp3
  marks(X) <- runif(npoints(X))
  markmarkscatter(X, 0.2)
  markmarkscatter(X[FALSE], 0.2)
})

local({
  #' weightedclosepairs is currently in strauss.R
  wi <- weightedclosepairs(redwood, 0.05, "isotropic")
  wt <- weightedclosepairs(redwood, 0.05, "translate")
  wp <- weightedclosepairs(redwood, 0.05, "periodic")
})

local({
  #' experimental
  r <- 0.08
  a <- closepairs(redwood, r)
  b <- tweak.closepairs(a, r, 26, 0.1, 0.1)
  X <- runifpoint3(30)
  rr <- 0.2
  cl <- closepairs(X, rr)
  ii <- cl$i[[1]]
  xl <- tweak.closepairs(cl, rr, ii, 0.05, -0.05, 0.05)
})

reset.spatstat.options()
#'
#'   tests/cluck.R
#'
#'   Tests of "click*" functions
#'   using queueing feature of spatstatLocator
#'
#'   $Revision: 1.4 $ $Date: 2020/04/28 12:58:26 $

require(spatstat)
local({
  #' clickppp
  if(ALWAYS) {
    spatstat.utils::queueSpatstatLocator(runif(5), runif(5))
    XA <- clickppp(hook=square(0.5))
  }
  if(FULLTEST) {
    spatstat.utils::queueSpatstatLocator(runif(6), runif(6))
    XB <- clickppp(n=3, types=c("a", "b"))
  }
  if(ALWAYS) {
    #' clickbox
    spatstat.utils::queueSpatstatLocator(runif(2), runif(2))
    BB <- clickbox()
    #' clickdist
    spatstat.utils::queueSpatstatLocator(runif(2), runif(2))
    dd <- clickdist()
    #' clickpoly
    hex <- vertices(disc(radius=0.4, centre=c(0.5, 0.5), npoly=6))
    spatstat.utils::queueSpatstatLocator(hex)
    PA <- clickpoly()
  }
  if(FULLTEST) {
    holy <- vertices(disc(radius=0.2, centre=c(0.5, 0.5), npoly=6))
    holy <- lapply(holy, rev)
    spatstat.utils::queueSpatstatLocator(concatxy(hex, holy))
    PB <- clickpoly(np=2, nv=6)
  }
  Y <- coords(runiflpp(6, simplenet))
  if(FULLTEST) {
    #' clicklpp
    spatstat.utils::queueSpatstatLocator(Y)
    XL <- clicklpp(simplenet)
  }
  if(ALWAYS) {
    spatstat.utils::queueSpatstatLocator(Y)
    XM <- clicklpp(simplenet, n=3, types=c("a", "b"))
  }
  if(ALWAYS) {
    #' identify.psp
    E <- edges(letterR)[c(FALSE, TRUE)]
    Z <- ppp(c(2.86, 3.65, 3.15), c(1.69, 1.98, 2.56), window=Frame(letterR))
    spatstat.utils::queueSpatstatLocator(Z)
    identify(E)
  }
  if(ALWAYS) {
    #' lineardisc
    plot(simplenet)
    spatstat.utils::queueSpatstatLocator(as.ppp(runiflpp(1, simplenet)))
    V <- lineardisc(simplenet, r=0.3)
  }
  if(FULLTEST) {
    #' transect.im
    Z <- density(cells)
    spatstat.utils::queueSpatstatLocator(runifpoint(2, Window(cells)))
    TZ <- transect.im(Z, click=TRUE)
  }
})
## tests/colour.R
##
##  Colour value manipulation and colour maps
##
## $Revision: 1.8 $ $Date: 2020/04/28 08:27:38 $
##

local({
  if(FULLTEST) {
    f <- function(n) grey(seq(0,1,length=n))
    z <- to.grey(f)

    h <- colourmap(rainbow(9), range=c(0.01, 0.1))
    plot(h, labelmap=100)
  }

  if(ALWAYS) {
    a <- colourmap(rainbow(12), range=as.Date(c("2018-01-01", "2018-12-31")))
    print(a)
    print(summary(a))
    a(as.Date("2018-06-15"))

    g <- colourmap(rainbow(4),
                   breaks=as.Date(c("2018-01-01", "2018-04-01",
                                    "2018-07-01", "2018-10-01", "2018-12-31")))
    print(g)
    print(summary(g))
    g(as.Date("2018-06-15"))
  }

  if(FULLTEST) {
    b <- colourmap(rainbow(12), inputs=month.name)
    print(b)
    print(summary(b))
    to.grey(b)
    to.grey(b, transparent=TRUE)
    plot(b, vertical=FALSE)
    plot(b, vertical=TRUE)
    plot(b, vertical=FALSE, gap=0)
    plot(b, vertical=TRUE, gap=0)
    plot(b, vertical=FALSE, xlim=c(0, 2))
    plot(b, vertical=TRUE, xlim=c(0,2))
    plot(b, vertical=FALSE, ylim=c(0, 2))
    plot(b, vertical=TRUE, ylim=c(0,2))

    argh <- list(a="iets", e="niets", col=b, f=42)
    arr <- col.args.to.grey(argh)
    rrgh <- col.args.to.grey(argh, transparent=TRUE)
  }

  if(ALWAYS) {
    #' constant colour map
    colourmap("grey", range=c(0.01, 0.1))
    colourmap("grey", range=as.Date(c("2018-01-01", "2018-12-31")))
    colourmap("grey",
              breaks=as.Date(c("2018-01-01", "2018-04-01",
                               "2018-07-01", "2018-10-01", "2018-12-31")))
    colourmap("grey", inputs=month.name)
  }

  if(FULLTEST) {
    #' empty colour map
    niets <- lut()
    print(niets)
    summary(niets)
    niets <- colourmap()
    print(niets)
    summary(niets)
    plot(niets)
  }

  if(ALWAYS) {
    #' interpolation - of transparent colours
    co <- colourmap(inputs=c(0, 0.5, 1),
                    rgb(red=c(1,0,0), green=c(0,1,0), blue=c(0,0,1),
                        alpha=c(0.3, 0.6, 0.9)))
    plot(interp.colourmap(co))
  }
})

#'
#'     contact.R
#'
#'   Check machinery for first contact distributions
#'
#'   $Revision: 1.6 $  $Date: 2020/04/28 12:58:26 $

local({
  if(ALWAYS) {
    #' reduce complexity
    Y <- as.mask(heather$coarse, dimyx=c(100, 50))
    
    X <- runifpoint(100, win = complement.owin(Y))
    G <- Gfox(X, Y)
    J <- Jfox(X, Y)

    Y <- as.polygonal(Y)
    X <- runifpoint(100, win = complement.owin(Y))
    G <- Gfox(X, Y)
    J <- Jfox(X, Y)

    op <- spatstat.options(exactdt.checks.data=TRUE)
    U <- exactdt(X)
    spatstat.options(op)
  }
})

reset.spatstat.options()
#'
#'   tests/contrib.R
#'
#'   Tests for user-contributed code in spatstat
#'
#'   $Revision: 1.2 $  $Date: 2020/04/28 12:58:26 $

local({
  #' Jinhom
  #' Marie-Colette van Lieshout and Ottmar Cronie
  X <- redwood3
  fit <- ppm(X ~ polynom(x,y,2))
  lam <- predict(fit)
  lamX <- fitted(fit, dataonly=TRUE)
  lmin <- 0.9 * min(lam)
  g1 <- Ginhom(X, lambda=fit, update=TRUE)
  if(FULLTEST) {
    g2 <- Ginhom(X, lambda=fit, update=FALSE, lmin = lmin)
    g3 <- Ginhom(X, lambda=lam,  lmin=lmin)
    g4 <- Ginhom(X, lambda=lamX, lmin=lmin)
  }
  if(ALWAYS) {
    f2 <- Finhom(X, lambda=fit, update=FALSE)
  }
  if(FULLTEST) {
    f1 <- Finhom(X, lambda=fit, update=TRUE)
    f3 <- Finhom(X, lambda=lam,  lmin=lmin)
  }
})
# tests/correctC.R
# check for agreement between C and interpreted code
# for interpoint distances etc.
# $Revision: 1.7 $ $Date: 2020/04/28 12:58:26 $

if(ALWAYS) { # depends on hardware
local({
  eps <- .Machine$double.eps * 4

  checkagree <- function(A, B, blurb) {
    maxerr <- max(abs(A-B))
    cat("Discrepancy", maxerr, "for", blurb, fill=TRUE)
    if(maxerr > eps) 
      stop(paste("Algorithms for", blurb, "disagree"))
    return(TRUE)
  }

  ## pairdist.ppp
  set.seed(190901)
  X <- rpoispp(42)
  dC <- pairdist(X, method="C")
  dR <- pairdist(X, method="interpreted")
  checkagree(dC, dR, "pairdist()")

  dCp <- pairdist(X, periodic=TRUE, method="C")
  dRp <- pairdist(X, periodic=TRUE, method="interpreted")
  checkagree(dCp, dRp, "pairdist(periodic=TRUE)")

  dCp2 <- pairdist(X, periodic=TRUE, squared=TRUE, method="C")
  dRp2 <- pairdist(X, periodic=TRUE, squared=TRUE, method="interpreted")
  checkagree(dCp2, dRp2, "pairdist(periodic=TRUE, squared=TRUE)")

  ## crossdist.ppp
  Y <- rpoispp(42)
  dC <- crossdist(X, Y, method="C")
  dR <- crossdist(X, Y, method="interpreted")
  checkagree(dC, dR, "crossdist()")

  dC <- crossdist(X, Y, periodic=TRUE, method="C")
  dR <- crossdist(X, Y, periodic=TRUE, method="interpreted")
  checkagree(dC, dR, "crossdist(periodic=TRUE)")

  dC2 <- crossdist(X, Y, periodic=TRUE, squared=TRUE, method="C")
  dR2 <- crossdist(X, Y, periodic=TRUE, squared=TRUE, method="interpreted")
  checkagree(dC2, dR2, "crossdist(periodic=TRUE, squared=TRUE)")

  # nndist.ppp
  nnC <- nndist(X, method="C")
  nnI <- nndist(X, method="interpreted")
  checkagree(nnC, nnI, "nndist()")

  nn3C <- nndist(X, k=3, method="C")
  nn3I <- nndist(X, k=3, method="interpreted")
  checkagree(nn3C, nn3I, "nndist(k=3)")

  # nnwhich.ppp
  nwC <- nnwhich(X, method="C")
  nwI <- nnwhich(X, method="interpreted")
  checkagree(nwC, nwI, "nnwhich()")

  nw3C <- nnwhich(X, k=3, method="C")
  nw3I <- nnwhich(X, k=3, method="interpreted")
  checkagree(nw3C, nw3I, "nnwhich(k=3)")

  # whist
  set.seed(98123)
  x <- runif(1000)
  w <- sample(1:5, 1000, replace=TRUE)
  b <- seq(0,1,length=101)
  op <- spatstat.options(Cwhist=TRUE)
  aT <- whist(x,b,w)
  spatstat.options(Cwhist=FALSE)
  aF <- whist(x,b,w)
  if(!all(aT == aF))
    stop("Algorithms for whist disagree")
  spatstat.options(op)
})

reset.spatstat.options()
}

#
# linearKmulti
#
# $Revision: 1.18 $ $Date: 2020/01/11 04:35:04 $
#
# K functions for multitype point pattern on linear network
#
#

linearKdot <- function(X, i, r=NULL, ..., correction="Ang") {
  if(!is.multitype(X, dfok=FALSE)) 
	stop("Point pattern must be multitype")
  marx <- marks(X)
  lev <- levels(marx)
  if(missing(i)) i <- lev[1L] else
    if(!(i %in% lev)) stop(paste("i = ", i , "is not a valid mark"))  
  I <- (marx == i)
  J <- rep(TRUE, npoints(X))  # i.e. all points
  result <- linearKmulti(X, I, J,
                         r=r, correction=correction, ...)
  correction <- attr(result, "correction")
  type <- if(correction == "Ang") "L" else "net"
  result <- rebadge.as.dotfun(result, "K", type, i)
  return(result)
}

linearKcross <- function(X, i, j, r=NULL, ..., correction="Ang") {
  if(!is.multitype(X, dfok=FALSE)) 
	stop("Point pattern must be multitype")
  marx <- marks(X)
  lev <- levels(marx)
  if(missing(i)) i <- lev[1L] else
    if(!(i %in% lev)) stop(paste("i = ", i , "is not a valid mark"))
  if(missing(j)) j <- lev[2L] else
    if(!(j %in% lev)) stop(paste("j = ", j , "is not a valid mark"))
  #
  if(i == j) {
    result <- linearK(X[marx == i], r=r, correction=correction, ...)
  } else {
    I <- (marx == i)
    J <- (marx == j)
    result <- linearKmulti(X, I, J, r=r, correction=correction, ...)
  }
  # rebrand
  correction <- attr(result, "correction")
  type <- if(correction == "Ang") "L" else "net"
  result <- rebadge.as.crossfun(result, "K", type, i, j)
  return(result)
}

linearKmulti <- function(X, I, J, r=NULL, ..., correction="Ang") {
  stopifnot(inherits(X, "lpp"))
  correction <- pickoption("correction", correction,
                           c(none="none",
                             Ang="Ang",
                             best="Ang"),
                           multi=FALSE)
  
  # extract info about pattern
  np <- npoints(X)
  lengthL <- volume(domain(X))
  # validate I, J
  if(!is.logical(I) || !is.logical(J))
    stop("I and J must be logical vectors")
  if(length(I) != np || length(J) != np)
    stop(paste("The length of I and J must equal",
               "the number of points in the pattern"))
	
  if(!any(I)) stop("no points satisfy I")
#  if(!any(J)) stop("no points satisfy J")
		
  nI <- sum(I)
  nJ <- sum(J)
  nIandJ <- sum(I & J)
#  lambdaI <- nI/lengthL
#  lambdaJ <- nJ/lengthL
  # compute K
  denom <- (nI * nJ - nIandJ)/lengthL
  K <- linearKmultiEngine(X, I, J, r=r, denom=denom,
                          correction=correction, ...)
  # set appropriate y axis label
  correction <- attr(K, "correction")
  type <- if(correction == "Ang") "L" else "net"
  K <- rebadge.as.crossfun(K, "K", type, "I", "J")
  attr(K, "correction") <- correction
  return(K)
}

# ................ inhomogeneous ............................

linearKdot.inhom <- function(X, i, lambdaI, lambdadot,
                             r=NULL, ..., correction="Ang", normalise=TRUE) {
  if(!is.multitype(X, dfok=FALSE)) 
	stop("Point pattern must be multitype")
  marx <- marks(X)
  lev <- levels(marx)
  if(missing(i)) i <- lev[1L] else
    if(!(i %in% lev)) stop(paste("i = ", i , "is not a valid mark"))  
  I <- (marx == i)
  J <- rep(TRUE, npoints(X))  # i.e. all points
  # compute
  result <- linearKmulti.inhom(X, I, J, lambdaI, lambdadot, 
                               r=r, correction=correction, normalise=normalise,
                               ...)
  ## relabel
  correction <- attr(result, "correction")
  type <- if(correction == "Ang") "L, inhom" else "net, inhom"
  result <- rebadge.as.dotfun(result, "K", type, i)
  attr(result, "correction") <- correction
  return(result)
}

linearKcross.inhom <- function(X, i, j, lambdaI, lambdaJ,
                               r=NULL, ...,
                               correction="Ang", normalise=TRUE) {
  if(!is.multitype(X, dfok=FALSE)) 
	stop("Point pattern must be multitype")
  marx <- marks(X)
  lev <- levels(marx)
  if(missing(i)) i <- lev[1L] else
    if(!(i %in% lev)) stop(paste("i = ", i , "is not a valid mark"))
  if(missing(j)) j <- lev[2L] else
    if(!(j %in% lev)) stop(paste("j = ", j , "is not a valid mark"))
  #
  if(i == j) {
    I <- (marx == i)
    result <- linearKinhom(X[I], lambda=lambdaI, r=r,
                           correction=correction, normalise=normalise, ...)
  } else {
    I <- (marx == i)
    J <- (marx == j)
    result <- linearKmulti.inhom(X, I, J, lambdaI, lambdaJ,
                                 r=r, correction=correction,
                                 normalise=normalise, ...)
  }
  # rebrand
  correction <- attr(result, "correction")
  type <- if(correction == "Ang") "L, inhom" else "net, inhom"
  result <- rebadge.as.crossfun(result, "K", type, i, j)
  attr(result, "correction") <- correction
  return(result)
}

linearKmulti.inhom <- function(X, I, J, lambdaI, lambdaJ,
                               r=NULL, ...,
                               correction="Ang", normalise=TRUE) {
  stopifnot(inherits(X, "lpp"))
  correction <- pickoption("correction", correction,
                           c(none="none",
                             Ang="Ang",
                             best="Ang"),
                           multi=FALSE)
  
  # extract info about pattern
  np <- npoints(X)
  lengthL <- volume(domain(X))
  #
  # validate I, J
  if(!is.logical(I) || !is.logical(J))
    stop("I and J must be logical vectors")
  if(length(I) != np || length(J) != np)
    stop(paste("The length of I and J must equal",
               "the number of points in the pattern"))
	
  if(!any(I)) stop("no points satisfy I")

  # validate lambda vectors
  lambdaI <- getlambda.lpp(lambdaI, X, subset=I, ...)
  lambdaJ <- getlambda.lpp(lambdaJ, X, subset=J, ...)

  # compute K
  weightsIJ <- outer(1/lambdaI, 1/lambdaJ, "*")
  denom <- if(!normalise) lengthL else sum(1/lambdaI) 
  K <- linearKmultiEngine(X, I, J, r=r,
                          reweight=weightsIJ, denom=denom,
                          correction=correction, ...)
  # set appropriate y axis label
  correction <- attr(K, "correction")
  type <- if(correction == "Ang") "L, inhom" else "net, inhom"
  K <- rebadge.as.crossfun(K, "K", type, "I", "J")
  # set markers for 'envelope'
  attr(K, "dangerous") <- union(attr(lambdaI, "dangerous"),
                                attr(lambdaJ, "dangerous"))
  attr(K, "correction") <- correction
  return(K)
}

# .............. internal ...............................

linearKmultiEngine <- function(X, I, J, ..., r=NULL, reweight=NULL, denom=1,
                          correction="Ang", showworking=FALSE) {
  # ensure distance information is present
  X <- as.lpp(X, sparse=FALSE)
  # extract info about pattern
  np <- npoints(X)
  # extract linear network
  L <- domain(X)
  W <- Window(L)
  # determine r values
  rmaxdefault <- 0.98 * boundingradius(L)
  breaks <- handle.r.b.args(r, NULL, W, rmaxdefault=rmaxdefault)
  r <- breaks$r
  rmax <- breaks$max
  #
  if(correction == "Ang") {
    fname <- c("K", "list(L, I, J)")
    ylab <- quote(K[L,I,J](r))
  } else {
    fname <- c("K", "list(net, I, J)")
    ylab <- quote(K[net,I,J](r))
  }
  #
  if(np < 2) {
    # no pairs to count: return zero function
    zeroes <- rep(0, length(r))
    df <- data.frame(r = r, est = zeroes)
    K <- fv(df, "r", ylab,
            "est", . ~ r, c(0, rmax),
            c("r", makefvlabel(NULL, "hat", fname)),
            c("distance argument r", "estimated %s"),
            fname = fname)
    attr(K, "correction") <- correction
    return(K)
  }
  #
  ## nI <- sum(I)
  ## nJ <- sum(J)
  ## whichI <- which(I)
  ## whichJ <- which(J)
  clash <- I & J
  has.clash <- any(clash)
  ## compute pairwise distances
  DIJ <- crossdist(X[I], X[J], check=FALSE)
  if(has.clash) {
    ## exclude pairs of identical points from consideration
    Iclash <- which(clash[I])
    Jclash <- which(clash[J])
    DIJ[cbind(Iclash,Jclash)] <- Inf
  }
  #---  compile into K function ---
  if(correction == "none" && is.null(reweight)) {
    # no weights (Okabe-Yamada)
    K <- compileK(DIJ, r, denom=denom, check=FALSE, fname=fname)
    K <- rebadge.as.crossfun(K, "K", "net", "I", "J")
    unitname(K) <- unitname(X)
    attr(K, "correction") <- correction
    return(K)
  }
  if(correction == "none") {
     edgewt <- 1
  } else {
    ## inverse m weights (Ang's correction)
    ## determine tolerance
    toler <- default.linnet.tolerance(L)
    ## compute m[i,j]
    m <- DoCountCrossEnds(X, I, J, DIJ, toler)
    edgewt <- 1/m
  }
  # compute K
  wt <- if(!is.null(reweight)) edgewt * reweight else edgewt
  K <- compileK(DIJ, r, weights=wt, denom=denom, check=FALSE, fname=fname)
  ## rebadge and tweak
  K <- rebadge.as.crossfun(K, "K", "L", "I", "J")
  fname <- attr(K, "fname")
  # tack on theoretical value
  K <- bind.fv(K, data.frame(theo=r),
               makefvlabel(NULL, NULL, fname, "pois"),
               "theoretical Poisson %s")
  ## 
  unitname(K) <- unitname(X)
  fvnames(K, ".") <- rev(fvnames(K, "."))
  # show working
  if(showworking)
    attr(K, "working") <- list(DIJ=DIJ, wt=wt)
  attr(K, "correction") <- correction
  return(K)
}

DoCountCrossEnds <- function(X, I, J, DIJ, toler) {
  stopifnot(is.lpp(X))
  stopifnot(is.logical(I) && is.logical(J))
  stopifnot(is.matrix(DIJ))
  nI <- sum(I)
  nJ <- sum(J)
  whichI <- which(I)
  whichJ <- which(J)
  m <- matrix(1, nI, nJ)
  easy <- list(is.connected=TRUE)
  L <- domain(X)
  if(is.connected(L)) {
    ## network is connected
    for(k in seq_len(nJ)) {
      j <- whichJ[k]
      I.j <- (whichI != j)
      i.j <- setdiff(whichI, j)
      m[I.j, k] <- countends(L, X[i.j], DIJ[I.j,k],
                              toler=toler, internal=easy)
    }
  } else {
    ## network is disconnected - split into components
    vlab <- connected(L, what="labels")
    subsets <- split(seq_len(nvertices(L)), factor(vlab))
    for(s in subsets) {
      ## extract one component and the points falling in it
      Xs <- thinNetwork(X, retainvertices=s)
      ns <- npoints(Xs)
      if(ns >= 2) {
        Ls <- domain(Xs)
        ## identify which points of X are involved
        relevant <- attr(Xs, "retainpoints")
        Xindex <- which(relevant)
        ## classify them
        Isub <- I[relevant]
        ## Jsub <- J[relevant]
        ## identify relevant submatrix of DIJ
        rowsub <- relevant[I]
        colsub <- relevant[J]
        ## corresponding indices in X
        ## rowXindex <- whichI[rowsub]
        ## colXindex <- whichJ[colsub]
        ## handle
        for(k in which(colsub)) {
          j <- whichJ[k]
          I.j <- rowsub & (whichI != j)
          i.j <- Isub & (Xindex != j)
          m[ I.j, k ] <- countends(Ls, Xs[i.j], DIJ[I.j, k],
                                   toler=toler,
                                   internal=easy)
        }
      }
    }
  }
  if(any(uhoh <- (m == 0) & is.finite(DIJ))) {
    warning("Internal error: disc boundary count equal to zero")
    m[uhoh] <- 1
  }
  return(m)
}

#
# covariates.R
#
# evaluate covariates
#
#   $Revision: 1.3 $  $Date: 2015/10/21 09:06:57 $
#

evalCovariate <- function(covariate, locations) {
  # evaluate covariate of any kind at specified locations
  covvalues <-
    if(is.im(covariate)) 
      safelookup(covariate, locations)
    else if(is.function(covariate)) 
      covariate(locations$x, locations$y)
    else if(is.numeric(covariate) || is.factor(covariate)) {
      if(length(covariate) == 1L)
        rep.int(covariate, length(locations$x))
      else if(length(covariate) == length(locations$x))
        covariate
      else stop("Inappropriate length for covariate vector")
    } else
  stop("Covariate should be an image, a function or a factor/numeric vector")
  return(covvalues)
}

ppmCovariates <- function(model) {
  # generate list of all covariates in ppm (excluding marks)
  stopifnot(is.ppm(model))
  co <- as.list(model$covariates)
  xy <- list(x=function(x,y){x}, y=function(x,y){y})
  coplus <- append(co, xy)
  return(as.anylist(coplus))
}

findCovariate <- function(covname, scope, scopename=NULL) {
  # find the named covariate in the given ppm object or list
  if(is.ppm(scope)) {
    covlist <- ppmCovariates(scope)
    if(missing(scopename)) scopename <- "covariates in model"
  } else if(is.list(scope)) {
    covlist <- scope
  } else stop("scope should be a named list of covariates, or a ppm object")
  if(!(covname %in% names(covlist))) 
    stop(paste("covariate", dQuote(covname), "not found",
               if(!is.null(scopename)) paste("amongst", scopename) else NULL))
  covlist[[covname]]
}

#'
#'   quantiledensity.R
#'
#'  quantile method for class 'density'
#'
#'  Also a CDF from a 'density'
#' 
#'  $Revision: 1.3 $ $Date: 2015/09/01 11:53:15 $

quantile.density <- local({

  quantile.density <- function(x, probs = seq(0, 1, 0.25), names = TRUE, ...,
                               warn=TRUE) {
    stopifnot(inherits(x, "density"))
    #' check whether density estimate was restricted to an interval
    if(warn && is.call(cl <- x$call) && any(c("from", "to") %in% names(cl)))
      warning(paste("Density was normalised within the computed range",
                    "of x values", prange(c(cl$from, cl$to))),
              call.=FALSE)
    #' validate probs
    eps <- 100 * .Machine$double.eps
    if(any((p.ok <- !is.na(probs)) & (probs < -eps | probs > 1 + eps))) 
      stop("'probs' outside [0,1]")
    if (na.p <- any(!p.ok)) {
      o.pr <- probs
      probs <- probs[p.ok]
      probs <- pmax(0, pmin(1, probs))
    }
    np <- length(probs)
    qs <- rep(NA_real_, np)
    if (np > 0) {
      #' extract density values 
      xx <- x$x
      yy <- x$y
      nn <- length(xx)
      #' integrate, normalise
      Fx <- cumsum(yy * c(0, diff(xx)))
      Fx <- Fx/Fx[nn]
      #' quantile
      for(j in 1:np) {
        ii <- min(which(Fx >= probs[j]))
        if(!is.na(ii) && ii >= 1 && ii <= nn) 
          qs[j] <- xx[ii]
      }
      if (names && np > 0L) {
        names(qs) <- format_perc(probs)
      }
    }
    if (na.p) {
      o.pr[p.ok] <- qs
      names(o.pr) <- rep("", length(o.pr))
      names(o.pr)[p.ok] <- names(qs)
      return(o.pr)
    } else return(qs)
  }

  format_perc <- function (x, digits = max(2L, getOption("digits")),
                           probability = TRUE, use.fC = length(x) < 100, ...) {
    if (length(x)) {
      if (probability) x <- 100 * x
      paste0(if (use.fC) 
             formatC(x, format = "fg", width = 1, digits = digits)
      else format(x, trim = TRUE, digits = digits, ...), "%")
    }
    else character(0)
  }

  quantile.density
})


CDF <- function(f, ...) {
  UseMethod("CDF")
}

CDF.density <- function(f, ..., warn=TRUE) {
  stopifnot(inherits(f, "density"))
  #' check whether density estimate was restricted to an interval
  if(warn && is.call(cl <- f$call) && any(c("from", "to") %in% names(cl)))
    warning(paste("Density was normalised within the computed range",
                  "of x values", prange(c(cl$from, cl$to))),
            call.=FALSE)
  #' integrate
  xx <- f$x
  yy <- f$y
  nn <- length(xx)
  Fx <- cumsum(yy * c(0, diff(xx)))
  #' normalise
  Fx <- Fx/Fx[nn]
  #' 
  FF <- approxfun(xx, Fx, method="linear", rule=2)
  return(FF)
}

#
#
#  defaultwin.R
#
#   $Revision: 1.10 $   $Date: 2015/10/21 09:06:57 $
#

default.expand <- function(object, m=2, epsilon=1e-6, w=Window(object)) {
  stopifnot(is.ppm(object) || inherits(object, "rmhmodel"))
  # no expansion necessary if model is Poisson
  if(is.poisson(object))
    return(.no.expansion)
  # default is no expansion if model is nonstationary
  if(!is.stationary(object))
    return(.no.expansion)
  
# Redundant since a non-expandable model is non-stationary
#  if(!is.expandable(object))
#    return(.no.expansion)
  
  # rule is to expand data window by distance d = m * reach
  rr <- reach(object, epsilon=epsilon)
  if(!is.finite(rr))
    return(rmhexpand())
  if(!is.numeric(m) || length(m) != 1 || m < 1)
    stop("m should be a single number >= 1")
  mr <- m * rr
  rule <- rmhexpand(distance = mr)
  # 
  if(is.owin(w)) {
    # apply rule to window
    wplus <- expand.owin(w, rule)
    # save as new expansion rule
    rule <- rmhexpand(wplus)
  }
  return(rule)
}

default.clipwindow <- function(object, epsilon=1e-6) {
  stopifnot(is.ppm(object) || inherits(object, "rmhmodel"))
  # data window
  w <- as.owin(object)
  if(is.null(w)) return(NULL)
  # interaction range of model
  rr <- reach(object, epsilon=epsilon)
  if(!is.finite(rr))
    return(NULL)
  if(rr == 0)
    return(w)
  else
    return(erosion(w, rr))
}

  

#'
#'   densityAdaptiveKernel.R
#'
#'   $Revision: 1.4 $  $Date: 2019/03/07 02:58:08 $
#'
#'
#'  Adaptive kernel smoothing via 3D FFT
#'

densityAdaptiveKernel <- function(X, ...) {
  UseMethod("densityAdaptiveKernel")
}

densityAdaptiveKernel.ppp <- function(X, bw, ...,
                                      weights=NULL,
                                      at=c("pixels", "points"),
                                      edge=TRUE, 
                                      ngroups) {
  stopifnot(is.ppp(X))
  at <- match.arg(at)
  nX <- npoints(X)

  if(nX == 0)
    switch(at,
           points = return(numeric(nX)),
           pixels = return(as.im(0, W=Window(X), ...)))
                     
  if(missing(ngroups) || is.null(ngroups)) {
    ngroups <- max(1L, floor(sqrt(npoints(X))))
  } else if(any(is.infinite(ngroups))) {
    ngroups <- nX
  } else {
    check.1.integer(ngroups)
    ngroups <- min(nX, ngroups)
  }

  if(weighted <- !is.null(weights)) {
    check.nvector(weights, nX, oneok=TRUE)
    if(length(weights) == 1) weights <- rep(weights, nX)
  } else weights <- rep(1,nX)

  ## determine bandwidth for each data point
  if(missing(bw) || is.null(bw)) {
    bw <- do.call.matched(bw.abram,
                          resolve.defaults(list(X=X, at="points"),
                                           list(...)),
                          extrargs=names(args(as.mask)))
  } else if(is.numeric(bw)) {
    check.nvector(bw, nX, oneok=TRUE)
    if(length(bw) == 1) bw <- rep(bw, nX)
  } else if(is.im(bw)) {
    bw <- safelookup(bw, X, warn=FALSE)
    if(anyNA(bw))
      stop("Some data points lie outside the domain of image 'bw'",
           call.=FALSE)
  } else if(inherits(bw, "funxy")) {
    bw <- bw(X)
    if(anyNA(bw))
      stop("Some data points lie outside the domain of function 'bw'",
           call.=FALSE)
  } else stop("Argument 'bw' should be a numeric vector or a pixel image")

  #' divide bandwidths into groups
  p <- seq(0,1,length=ngroups+1)
  qbands <- quantile(bw, p)
  groupid <- findInterval(bw,qbands,all.inside=TRUE)
  #' map to middle of group
  pmid <- (p[-1] + p[-length(p)])/2
  qmid   <- quantile(bw, pmid)

  marks(X) <- if(weighted) weights else NULL
  group <- factor(groupid, levels=1:ngroups)
  Y <- split(X, group)

  Z <- mapply(density.ppp,
              x=Y,
              sigma=as.list(qmid),
              weights=lapply(Y, marks),
              MoreArgs=list(edge=edge, at=at, ...),
              SIMPLIFY=FALSE)

  ZZ <- switch(at,
               pixels = im.apply(Z, "sum"),
               points = unsplit(Z, group))
  return(ZZ)
}

#
#    util.R    miscellaneous utilities
#
#    $Revision: 1.245 $    $Date: 2020/05/03 03:48:27 $
#

# common invocation of matrixsample

rastersample <- function(X, Y) {
  stopifnot(is.im(X) || is.mask(X))
  stopifnot(is.im(Y) || is.mask(Y))
  phase <- c((Y$yrow[1] - X$yrow[1])/X$ystep,
             (Y$xcol[1] - X$xcol[1])/X$xstep)
  scale <- c(Y$ystep/X$ystep,
             Y$xstep/X$xstep)
  if(is.im(X)) {
    # resample an image
    if(!is.im(Y))
      Y <- as.im(Y)
    Xtype <- X$type
    Xv    <- X$v
    # handle factor-valued image as integer
    if(Xtype == "factor") 
      Xv <- array(as.integer(Xv), dim=X$dim)
    # resample
    naval <- switch(Xtype,
                 factor=,
                 integer= NA_integer_, 
                 logical = as.logical(NA_integer_), 
                 real = NA_real_, 
                 complex = NA_complex_, 
                 character = NA_character_,
                 NA)
    Y$v <- matrixsample(Xv, Y$dim, phase=phase, scale=scale, na.value=naval)
    # inherit pixel data type from X
    Y$type <- Xtype
    if(Xtype == "factor") {
      lev <- levels(X)
      Y$v <- factor(Y$v, labels=lev, levels=seq_along(lev))
      dim(Y$v) <- Y$dim
    }
  } else {
    # resample a mask
    if(!is.mask(Y)) Y <- as.mask(Y)
    Y$m <- matrixsample(X$m, Y$dim, phase=phase, scale=scale, na.value=FALSE)
  }
  return(Y)
}

pointgrid <- function(W, ngrid) {
  W <- as.owin(W)
  masque <- as.mask(W, dimyx=ngrid)
  rxy <- rasterxy.mask(masque, drop=TRUE)
  xx <- rxy$x
  yy <- rxy$y
  return(ppp(xx, yy, W))
}

onecolumn <- function(m) {
  switch(markformat(m),
         none=stop("No marks provided"),
         vector=m,
         dataframe=m[,1, drop=TRUE],
         NA)
}


checkbigmatrix <- function(n, m, fatal=FALSE, silent=FALSE) {
  nm <- as.numeric(n) * as.numeric(m)
  if(nm <= spatstat.options("maxmatrix"))
    return(TRUE)
  whinge <- paste("Attempted to create binary mask with",
                  n, "*", m, "=", nm, "entries")
  if(fatal) stop(whinge, call.=FALSE)
  if(!silent) warning(whinge, call.=FALSE)
  return(FALSE)
}


## ........... progress reports .....................

progressreport <- local({

  Put <- function(name, value, state) {
    if(is.null(state)) {
      putSpatstatVariable(paste0("Spatstat.", name), value)
    } else {
      state[[name]] <- value
    }
    return(state)
  }
  Get <- function(name, state) {
    if(is.null(state)) {
      value <- getSpatstatVariable(paste0("Spatstat.", name))
    } else {
      value <- state[[name]] 
    }
    return(value)
  }

  IterationsPerLine <- function(charsperline, n, every, tick,
                                showtime, showevery) {
    # Calculate number of iterations that triggers a newline.
    # A dot is printed every 'tick' iterations
    # Iteration number is printed every 'every' iterations.
    # If showtime=TRUE, the time is shown every 'showevery' iterations
    # where showevery \in {1, every, n}.
    chars.report <- max(1, ceiling(log10(n)))
    if(showtime) {
      chars.time <- nchar(' [etd 12:00:00] ')
      timesperreport <- if(showevery == 1) every else
                        if(showevery == every) 1 else 0
      chars.report <- chars.report + timesperreport * chars.time
    }
    chars.ticks <- floor((every-1)/tick)
    chars.block <- chars.report + chars.ticks
    nblocks <- max(1, floor(charsperline/chars.block))
    nperline <- nblocks * every
    leftover <- charsperline - nblocks * chars.block
    if(leftover > 0)
      nperline <- nperline + min(leftover * tick, showevery - 1)
    return(nperline)
  }
  
  progressreport <- function(i, n,
                             every=min(100,max(1, ceiling(n/100))),
                             tick=1,
                             nperline=NULL,
                             charsperline=getOption("width"),
                             style=spatstat.options("progress"),
                             showtime=NULL,
                             state=NULL) {
    missevery <- missing(every)
    nperline.fixed <- !is.null(nperline)
    showtime.optional <- is.null(showtime)
    if(showtime.optional) showtime <- FALSE # initialise only
    if(i > n) {
      warning(paste("progressreport called with i =", i, "> n =", n))
      return(invisible(NULL))
    }
    if(style == "tk" && !requireNamespace("tcltk")) {
      warning("tcltk is unavailable; switching to style='txtbar'", call.=FALSE)
      style <- "txtbar"
    }
    if(is.null(state) && style != "tty")
      stop(paste("Argument 'state' is required when style =",sQuote(style)),
           call.=FALSE)
    switch(style,
           txtbar={
             if(i == 1) {
               ## initialise text bar
               state <- Put("ProgressBar",
                            txtProgressBar(1, n, 1, style=3),
                            state)
             } else {
               ## get text bar
               pbar <- Get("ProgressBar", state)
               ## update 
               setTxtProgressBar(pbar, i)
               if(i == n) {
                 close(pbar)
                 state <- Put("ProgressBar", NULL, state)
               } 
             }
           },
           tk={
             requireNamespace("tcltk")
             if(i == 1) {
               ## initialise text bar
               state <- Put("ProgressBar",
                            tcltk::tkProgressBar(title="progress",
                                                 min=0, max=n, width=300),
                            state)
             } else {
               ## get text bar
               pbar <- Get("ProgressBar", state)
               ## update 
               tcltk::setTkProgressBar(pbar, i,
                                       label=paste0(round(100 * i/n), "%"))
               if(i == n) {
                 close(pbar)
                 state <- Put("ProgressBar", NULL, state)
               } 
             }
           },
           tty={
             now <- proc.time()
             if(i == 1 || is.null(state)) {
               ## Initialise stuff
               if(missevery && every > 1 && n > 10) 
                 every <- niceround(every)
               showevery <- if(showtime) every else n
               if(!nperline.fixed) 
                 nperline <- IterationsPerLine(charsperline, n, every, tick,
                                               showtime, showevery)
               state <- Put("ProgressData",
                            list(every=every,
                                 tick=tick,
                                 nperline=nperline,
                                 starttime=now,
                                 showtime=showtime,
                                 showevery=showevery,
                                 nperline.fixed=nperline.fixed,
                                 showtime.optional=showtime.optional),
                            state)
             } else {
               pd <- Get("ProgressData", state)
               if(is.null(pd))
                 stop(paste("progressreport called with i =", i,
                            "before i = 1"))
               every     <- pd$every
               tick      <- pd$tick
               nperline  <- pd$nperline
               showtime  <- pd$showtime
               showevery <- pd$showevery
               showtime.optional <- pd$showtime.optional
               nperline.fixed    <- pd$nperline.fixed
               if(i < n) {
                 if(showtime || showtime.optional) {
                   ## estimate time remaining
                   starttime <- pd$starttime
                   elapsed <- now - starttime
                   elapsed <- unname(elapsed[3])
                   rate <- elapsed/(i-1)
                   remaining <- rate * (n-i)
                   if(!showtime) {
                     ## show time remaining if..
                     if(rate > 20) {
                       ## .. rate is very slow
                       showtime <- TRUE
                       showevery <- 1
                     } else if(remaining > 180) {
                       ## ... more than 3 minutes remaining
                       showtime <- TRUE
                       showevery <- every
                       aminute <- ceiling(60/rate)
                       if(aminute < showevery) 
                         showevery <- min(niceround(aminute), showevery)
                     }
                     # update number of iterations per line
                     if(showtime && !nperline.fixed) 
                       nperline <- IterationsPerLine(charsperline,
                                                     n, every, tick,
                                                     showtime, showevery)
                   }
                 }
                 state <- Put("ProgressData",
                              list(every=every,
                                   tick=tick,
                                   nperline=nperline,
                                   starttime=starttime,
                                   showtime=showtime,
                                   showevery=showevery,
                                   nperline.fixed=nperline.fixed,
                                   showtime.optional=showtime.optional),
                              state)
               }
             }
             if(i == n) 
               cat(paste(" ", n, ".\n", sep=""))
             else if(every == 1 || i <= 3)
               cat(paste(i, ",", if(i %% nperline == 0) "\n" else " ", sep=""))
             else {
               if(i %% every == 0) 
                 cat(i)
               else if(i %% tick == 0)
                 cat(".")
               if(i %% nperline == 0)
                 cat("\n")
             }
             if(i < n && showtime && (i %% showevery == 0)) {
               st <- paste("etd", codetime(round(remaining)))
               st <- paren(st, "[")
               cat(paste("", st, ""))
             }
             flush.console()
           },
           stop(paste("Unrecognised option for style:", dQuote(style)))
           )
    return(invisible(state))
  }

  progressreport
})
  


multiply.only.finite.entries <- function(x, a) {
  # In ppm a potential value that is -Inf must remain -Inf
  # and a potential value that is 0 multiplied by NA remains 0
  y <- x
  ok <- is.finite(x) & (x != 0)
  y[ok] <- a * x[ok]
  return(y)
}


 
## print names and version numbers of libraries loaded

sessionLibs <- function() {
  a <- sessionInfo()
  b <- unlist(lapply(a$otherPkgs, getElement, name="Version"))
  g <- rbind(names(b), unname(b))
  d <- apply(g, 2, paste, collapse=" ")
  if(length(d) > 0) {
    cat("Libraries loaded:\n")
    for(di in d) cat(paste("\t", di, "\n"))
  } else cat("Libraries loaded: None\n")
  return(invisible(d))
}



# ..................

prepareTitle <- function(main) {
  ## Count the number of lines in a main title
  ## Convert title to a form usable by plot.owin
  if(is.expression(main)) {
    nlines <- 1
  } else {
    main <- paste(main)
    ## break at newline 
    main <- unlist(strsplit(main, "\n"))
    nlines <- if(sum(nchar(main)) == 0) 0 else length(main)
  }
  return(list(main=main,
              nlines=nlines,
              blank=rep('  ', nlines)))
}

requireversion <- function(pkg, ver, fatal=TRUE) {
  pkgname <- deparse(substitute(pkg))
  pkgname <- gsub("\"", "", pkgname)
  pkgname <- gsub("'", "", pkgname)
  v <- read.dcf(file=system.file("DESCRIPTION", package=pkgname), 
                fields="Version")
  ok <- (package_version(v) >= ver)
  if(!ok && fatal) 
    stop(paste("Package",
               sQuote(pkgname),
               "is out of date: version >=",
               ver,
               "is needed"),
         call.=FALSE)
  return(if(ok) invisible(TRUE) else FALSE)
}

spatstatDiagnostic <- function(msg) {
  cat("-----------------------------\n")
  cat(paste(" >>> Spatstat Diagnostic: ", msg, "<<<\n"))
  cat("-----------------------------\n")
  invisible(NULL)
}

allElementsIdentical <- function(x, entry=NULL) {
  if(length(x) <= 1) return(TRUE)
  if(is.null(entry)) {
    x1 <- x[[1]]
    for(i in 2:length(x))
      if(!identical(x[[i]], x1)) return(FALSE)
  } else {
    e1 <- x[[1]][[entry]]
    for(i in 2:length(x))
      if(!identical(x[[i]][[entry]], e1)) return(FALSE)
  }
  return(TRUE)
}

#'
#'   adaptive.density.R
#'
#'   $Revision: 1.1 $  $Date: 2019/02/06 03:22:51 $
#'

adaptive.density <- function(X, ..., method=c("voronoi", "kernel")) {
  method <- match.arg(method)
  result <- switch(method,
                   voronoi = densityVoronoi(X, ...),
                   kernel  = densityAdaptiveKernel(X, ...))
  return(result)
}

#
# interp.im.R
#
#  $Revision: 1.6 $  $Date: 2018/07/30 14:29:25 $
#

interp.im <- local({

  lukimyu <- function(ccc, rrr, mat, defaults) {
    dimm <- dim(mat)
    within <- (rrr >= 1 & rrr <= dimm[1L] & ccc >= 1 & ccc <= dimm[2L])
    result <- defaults
    result[within] <- mat[cbind(rrr[within], ccc[within])]
    result
  }

  interp.im <- function(Z, x, y=NULL, bilinear=FALSE) {
    stopifnot(is.im(Z))
    if(!is.null(levels(Z)))
      stop("Interpolation is undefined for factor-valued images")
    xy <- xy.coords(x, y)
    x <- xy$x
    y <- xy$y
    ok <- inside.owin(x,y, as.owin(Z))
    V <- Z$v
    ## get default lookup values (for boundary cases)
    fallback <- Z[ppp(x[ok], y[ok], window=as.rectangle(Z), check=FALSE)]
    ## Transform to grid coordinates
    ## so that pixel centres are at integer points,
    ## bottom left of image is (0,0)
    xx <- (x[ok] - Z$xcol[1L])/Z$xstep
    yy <- (y[ok] - Z$yrow[1L])/Z$ystep
    ## find grid point to left and below
    ## (may transgress boundary)
    xlower <- floor(xx)
    ylower <- floor(yy)
    cc <- as.integer(xlower) + 1L
    rr <- as.integer(ylower) + 1L
    dx <- xx - xlower
    dy <- yy - ylower
    if(bilinear) {
      ## 'orthodox'
      values <- ((1-dx) * (1-dy) * lukimyu(cc,rr,V,fallback)
                 + dx   * (1-dy) * lukimyu(cc+1,rr,V,fallback)
                 + (1-dx) * dy   * lukimyu(cc,rr+1,V,fallback)
                 + dx    * dy    * lukimyu(cc+1,rr+1,V,fallback)
      )
    } else {
      ## original & default
      ## determine whether (x,y) is above or below antidiagonal in square
      below <- (dx + dy <= 1)
      ## if below,interpolate Z(x,y) = (1-x-y)Z(0,0) + xZ(1,0) + yZ(0,1)
      ## if above,interpolate Z(x,y) = (x+y-1)Z(1,1) + (1-x)Z(0,1) + (1-y)Z(1,0)
      values <- ifelse(below,
                      ( (1-dx-dy)*lukimyu(cc,rr,V,fallback)
                       + dx*lukimyu(cc+1,rr,V,fallback)
                       + dy*lukimyu(cc,rr+1,V,fallback)
                       ),
                      ( (dx+dy-1)*lukimyu(cc+1,rr+1,V,fallback)
                       + (1-dx)*lukimyu(cc,rr+1,V,fallback)
                       + (1-dy)*lukimyu(cc+1,rr,V,fallback)
                      ))
    }
    result <- numeric(length(x))
    result[ok] <- values
    result[!ok] <- NA
    return(result)
  }

  interp.im
})

#
#  morphology.R
#
#  dilation, erosion, opening, closing
#
#  generic functions
#  and methods for owin, psp, ppp
#
#  $Revision: 1.33 $   $Date: 2020/03/16 10:28:51 $
#

# ............ generic  ............................

erosion  <- function(w, r, ...) { UseMethod("erosion") }

dilation <- function(w, r, ...) { UseMethod("dilation") }

closing  <- function(w, r, ...) { UseMethod("closing") }

opening  <- function(w, r, ...) { UseMethod("opening") }

# ............ methods for class 'owin' ............................


# DELETED
# erode.owin <- function(...) {
#   .Deprecated("erosion.owin", package="spatstat")
#   erosion.owin(...)
# }

erosion.owin <- 
  function(w, r, shrink.frame=TRUE, ..., strict=FALSE, polygonal=NULL) {
  verifyclass(w, "owin")
  validradius(r, "erosion")
  if(r == 0 && !strict)
    return(w)

  xr <- w$xrange
  yr <- w$yrange
  
  if(2 * r >= max(diff(xr), diff(yr)))
    stop("erosion distance r too large for frame of window")

  # compute the dimensions of the eroded frame
  exr <- xr + c(r, -r)
  eyr <- yr + c(r, -r)
  ebox <- list(x=exr[c(1,2,2,1)], y=eyr[c(1,1,2,2)])

  ismask <- is.mask(w)
  if(is.empty(w))
    return(emptywindow(ebox))

  # determine type of computation
  if(is.null(polygonal))
    polygonal <- !ismask
  else {
    stopifnot(is.logical(polygonal))
    if(polygonal && ismask) {
      # try to convert
      w <- as.polygonal(w)
      if(is.mask(w))
        polygonal <- FALSE
    }
  }
  
  if(is.rectangle(w) && polygonal) {
    # result is a smaller rectangle
    if(shrink.frame) {
      return(owin(exr, eyr))  # type 'rectangle' 
    } else {
      return(owin(xr, yr, poly=ebox, check=FALSE)) # type 'polygonal'
    }
  }

  if(polygonal) {
    # compute polygonal region using polyclip package
    pnew <- polyclip::polyoffset(w$bdry, -r, jointype="round")
    # ensure correct polarity
    totarea <- sum(unlist(lapply(pnew, Area.xypolygon)))
    if(totarea < 0)
      pnew <- lapply(pnew, reverse.xypolygon)
    if(shrink.frame) {
      return(owin(poly=pnew, check=FALSE))
    } else {
      return(owin( xr,  yr, poly=pnew, check=FALSE))
    }
  }
  
  # otherwise erode the window in pixel image form
  if(w$type == "mask") 
    wnew <- erodemask(w, r, strict=strict)
  else {
    D <- distmap(w, invert=TRUE, ...)
    wnew <- levelset(D, r, if(strict) ">" else ">=")
  }
        
  if(shrink.frame) {
    # trim off some rows & columns of pixel raster
    keepcol <- (wnew$xcol >= exr[1] & wnew$xcol <= exr[2])
    keeprow <- (wnew$yrow >= eyr[1] & wnew$yrow <= eyr[2])
    wnew$xcol <- wnew$xcol[keepcol]
    wnew$yrow <- wnew$yrow[keeprow]
    wnew$dim <- c(sum(keeprow), sum(keepcol))
    wnew$m <- wnew$m[keeprow, keepcol]
    wnew$xrange <- exr
    wnew$yrange <- eyr
  }

  return(wnew)
}	

# DELETED
# dilate.owin <- function(...) {
#   .Deprecated("dilation.owin", package="spatstat")
#   dilation.owin(...)
# }

dilation.owin <- 
  function(w, r, ..., polygonal=NULL, tight=TRUE) {
  verifyclass(w, "owin")
  validradius(r, "dilation")
  
  if(r == 0)
    return(w)

  ismask <- is.mask(w)
  if(is.empty(w))
    return(w)

  # determine type of computation
  if(is.null(polygonal)) {
    polygonal <- !ismask
  } else stopifnot(is.logical(polygonal))
  
  if(polygonal) {
    # convert to polygonal 
    w <- as.polygonal(w)
    if(!is.polygonal(w))
      polygonal <- FALSE
  }
  
  # bounding frame
  bb <- if(tight) boundingbox(w) else as.rectangle(w)
  newbox <- grow.rectangle(bb, r)

  # compute dilation
  if(!polygonal) {
    # compute pixel approximation
    epsilon <- sqrt(w$xstep^2 + w$ystep^2)
    r <- max(r, epsilon)
    w <- rebound.owin(w, newbox)
    distant <- distmap(w, ...)
    dil <- levelset(distant, r, "<=")
    return(dil)
  } else {
    # compute polygonal region using polyclip package
    pnew <- polyclip::polyoffset(w$bdry, r, jointype="round")
    # ensure correct polarity
    totarea <- sum(unlist(lapply(pnew, Area.xypolygon)))
    if(totarea < 0)
      pnew <- lapply(pnew, reverse.xypolygon)
    # determine bounding frame, convert to owin
    if(tight) {
      out <- owin(poly=pnew, check=FALSE)
    } else {
      out <- owin(newbox$xrange, newbox$yrange, poly=pnew, check=FALSE)
    }
    return(out)
  }
}

closing.owin <- function(w, r, ..., polygonal=NULL) {
  if(missing(r))
    stop("r is required")
  validradius(r, "closing")
  wplus <- dilation.owin(w, r, ..., polygonal=polygonal, tight=FALSE)
  if(is.empty(wplus))
    return(wplus)
  wclose <- erosion.owin(wplus, r, strict=TRUE)
  b <- as.rectangle(w)
  wclose <- rebound.owin(wclose[b], b)
  return(wclose)
}

opening.owin <- function(w, r, ..., polygonal=NULL) {
  if(missing(r))
    stop("r is required")
  validradius(r, "opening")
  wminus <- erosion.owin(w, r, ..., polygonal=polygonal, shrink.frame=FALSE)
  if(is.empty(wminus))
    return(wminus)
  wopen <- dilation.owin(wminus, r, tight=FALSE)
  b <- as.rectangle(w)
  wopen <- rebound.owin(wopen[b], b)
  return(wopen)
}


border <- function(w, r, outside=FALSE, ...) {
  w <- as.owin(w)
  if(!outside) {
    e <- erosion(w, r, ...)
    b <- setminus.owin(w, e)
  } else {
    d <- dilation(w, r, ...)
    b <- setminus.owin(d, w)
  }
  return(b)
}

# ............ methods for class 'psp' ............................


dilation.psp <- function(w, r, ..., polygonal=TRUE, tight=TRUE) {
  verifyclass(w, "psp")
  x <- w
  validradius(r, "dilation")
  if(r == 0)
    return(w)

  if(is.empty(x))
    return(emptywindow(as.owin(w)))
  
  # bounding frame
  bb <- if(tight) boundingbox(x) else as.rectangle(x)
  newbox <- grow.rectangle(bb, r)
  
  # compute dilation
  if(!polygonal) {
    x <- rebound.psp(x, newbox)
    distant <- distmap(x, ...)
    dil <- levelset(distant, r, "<=")
    return(dil)
  } else if(spatstat.options("old.morpho.psp")) {
    # old code for polygonal case
    ends   <- x$ends
    angles <- angles.psp(x, directed=TRUE)
#    lengths <- lengths_psp(x)
    out <- NULL
    # dilate individual segments
    halfcircle <- seq(from=0, to=pi, length.out=128)[-c(1,128)]
    for(i in seq_len(x$n)) {
      seg <- ends[i,]
      co <- cos(angles[i])
      si <- sin(angles[i])
      # draw sausage around i-th segment
      xx <- c(seg$x0, seg$x1) + r * si
      yy <- c(seg$y0, seg$y1) - r * co
      rightcircle <- angles[i] - pi/2 + halfcircle
      xx <- c(xx, seg$x1 + r * cos(rightcircle))
      yy <- c(yy, seg$y1 + r * sin(rightcircle))
      xx <- c(xx, c(seg$x1, seg$x0) - r * si)
      yy <- c(yy, c(seg$y1, seg$y0) + r * co)
      leftcircle <- angles[i] + pi/2 + halfcircle
      xx <- c(xx, seg$x0 + r * cos(leftcircle))
      yy <- c(yy, seg$y0 + r * sin(leftcircle))
      sausage <- owin(newbox$xrange, newbox$yrange, poly=list(x=xx, y=yy), check=FALSE)
      # add to set
      out <- union.owin(out, sausage, ...)
    }
    return(out)
  } else {
    # new code using 'polyclip' package
    # convert to list of list(x,y)
    ends   <- as.matrix(x$ends)
    n <- nrow(ends)
    plines <- vector(mode="list", length=n)
    for(i in 1:n) plines[[i]] <- list(x=ends[i, c("x0","x1")],
                                      y=ends[i, c("y0","y1")])
    # call
    pnew <- polyclip::polylineoffset(plines, r,
                                     jointype="round", endtype="openround")
    # ensure correct polarity
    totarea <- sum(unlist(lapply(pnew, Area.xypolygon)))
    if(totarea < 0)
      pnew <- lapply(pnew, reverse.xypolygon)
    # convert to owin object
    out <- if(tight) owin(poly=pnew, check=FALSE) else
            owin(newbox$xrange, newbox$yrange, poly=pnew, check=FALSE)
    return(out)
  }
}

closing.psp <- function(w, r, ..., polygonal=TRUE) {
  if(missing(r))
    stop("r is required")
  validradius(r, "closing")
  wplus <- dilation.psp(w, r, ..., polygonal=polygonal, tight=FALSE)
  if(is.empty(wplus))
    return(emptywindow(as.owin(w)))
  wclose <- erosion.owin(wplus, r, strict=TRUE)
  wclose <- rebound.owin(wclose, as.rectangle(w))
  return(wclose)
}

erosion.psp <- function(w, r, ...) {
  idorempty(w, r, "erosion")
}

opening.psp <- function(w, r, ...) {
  idorempty(w, r,"opening")
}

  
# ............ methods for class 'ppp' ............................

dilation.ppp <- function(w, r, ..., polygonal=TRUE, tight=TRUE) {
  verifyclass(w, "ppp")
  validradius(r, "dilation")
  x <- w
  
  if(r == 0)
    return(x)

  if(is.empty(w))
    return(emptywindow(as.owin(w)))
  
  # bounding frame
  bb <- if(tight) boundingbox(x) else as.rectangle(x)
  releps <- 1e-6
  newbox <- grow.rectangle(bb, r * (1+releps))

  # compute dilation
  if(!polygonal) {
    # compute pixel approximation
    Window(x) <- newbox
    distant <- distmap(x, ...)
    dil <- levelset(distant, r, "<=")
    return(dil)
  } else {
    # compute polygonal approximation
    # generate discs
    coo <- coords(x)
    nn <- npoints(x)
    balls <- vector(mode="list", length=nn)
    ball0 <- disc(r, c(0,0), ...)
    for(i in seq_len(nn))
      balls[[i]] <- shift(ball0, vec=coo[i,])
    class(balls) <- c("solist", class(balls))
    out <- union.owin(balls)
    return(out)
  }
}

closing.ppp <- function(w, r, ..., polygonal=TRUE) {
  if(missing(r))
    stop("r is required")
  validradius(r, "closing")
  if(is.empty(w) || w$n <= 3)
    return(emptywindow(as.owin(w)))
  # remove `isolated' points
  ok <- (nndist(w) <= 2 * r)
  if(sum(ok) <= 3)
    return(emptywindow(as.owin(w)))
  w <- w[ok]
  # dilate
  wplus <- dilation.ppp(w, r, ..., polygonal=polygonal, tight=FALSE)
  wclose <- erosion.owin(wplus, r, strict=TRUE)
  wclose <- rebound.owin(wclose, as.rectangle(w))
  return(wclose)
}

erosion.ppp <- function(w, r, ...) {
  idorempty(w, r, "erosion")
}

opening.ppp <- function(w, r, ...) {
  idorempty(w, r,"opening")
}

# ............ utilities ............................

validradius <- local({

  validradius <- function(r, caller="morphological operator") {
  #  rname <- short.deparse(substitute(r))
    if(!is.numeric(r) || length(r) != 1)
      groan("radius r must be a single number", caller)
    if(r < 0)
      groan("radius r must be nonnegative", caller)
    return(TRUE)
  }

  groan <- function(whinge, caller) {
    stop(paste("for", paste(caller, ",", sep=""), whinge), call.=FALSE)
  }

  validradius
})
  
idorempty <- function(w, r, caller="morphological operator") {
  validradius(r, caller)
  if(r == 0)
    return(w)
  else
    return(emptywindow(w))
}
           

#'
#'     pool.R
#'
#'  $Revision: 1.5 $  $Date: 2017/06/05 10:31:58 $

pool <- function(...) {
  UseMethod("pool")
}

pool.fv <- local({

  Square <- function(A) { force(A); eval.fv(A^2, relabel=FALSE) }
  Add <- function(A,B){ force(A); force(B); eval.fv(A+B, relabel=FALSE) }
  Cmul <- function(A, f) { force(A); force(f); eval.fv(f * A, relabel=FALSE) }

  pool.fv <- function(..., weights=NULL, relabel=TRUE, variance=TRUE) {
    argh <- list(...)
    n <- narg <- length(argh)
    if(narg == 0) return(NULL)
    if(narg == 1) return(argh[[1]])
    ## validate 
    isfv <- unlist(lapply(argh, is.fv))
    if(!all(isfv))
      stop("All arguments must be fv objects")
    argh <- do.call(harmonise, append(argh, list(strict=TRUE)))
    template <- vanilla.fv(argh[[1]])
    ## compute products
    if(!is.null(weights)) {
      check.nvector(weights, narg, things="Functions")
      Y <- Map(Cmul, argh, weights)
      XY <- Map(Cmul, argh, weights^2)
      sumX <- sum(weights)
      sumX2 <- sum(weights^2)
    } else {
      ## default: weights=1
      Y <- XY <- argh
      sumX <- sumX2 <- narg
    }
    ## sum
    sumY <- Reduce(Add, Y)
    attributes(sumY) <- attributes(template)
    ## ratio-of-sums
    Ratio <- eval.fv(sumY/sumX, relabel=FALSE)
    if(variance) {
      ## variance calculation
      meanX <- sumX/n
      meanY <- eval.fv(sumY/n, relabel=FALSE)
      sumY2 <- Reduce(Add, lapply(Y, Square))
      varX   <- (sumX2 - n * meanX^2)/(n-1)
      varY   <- eval.fv((sumY2 - n * meanY^2)/(n-1), relabel=FALSE)
      sumXY <- Reduce(Add, XY)
      covXY <- eval.fv((sumXY - n * meanX * meanY)/(n-1), relabel=FALSE)
      ## variance by delta method
      relvar <- eval.fv(pmax.int(0, varY/meanY^2 + varX/meanX^2
                                 - 2 * covXY/(meanX * meanY)),
                        relabel=FALSE)
      Variance <- eval.fv(Ratio^2 * relvar/n,
                          relabel=FALSE)
      ## two sigma CI
      hiCI <- eval.fv(Ratio + 2 * sqrt(Variance), relabel=FALSE)
      loCI <- eval.fv(Ratio - 2 * sqrt(Variance), relabel=FALSE)
    }
    ## tweak labels of main estimate
    attributes(Ratio) <- attributes(template)
    if(relabel)
      Ratio <- prefixfv(Ratio,
                        tagprefix="pool",
                        descprefix="pooled ",
                        lablprefix="")
    if(!variance)
      return(Ratio)
    ## tweak labels of variance terms
    attributes(Variance) <- attributes(template)
    Variance <- prefixfv(Variance,
                         tagprefix="var",
                         descprefix="delta-method variance estimate of ",
                         lablprefix="bold(var)~")
    attributes(hiCI) <- attributes(loCI) <-  attributes(template)
    hiCI <- prefixfv(hiCI,
                     tagprefix="hi",
                     descprefix="upper limit of two-sigma CI based on ",
                     lablprefix="bold(hi)~")
    loCI <- prefixfv(loCI,
                     tagprefix="lo",
                     descprefix="lower limit of two-sigma CI based on ",
                     lablprefix="bold(lo)~")
    ## glue together
    result <- Reduce(bind.fv, list(Ratio, Variance, hiCI, loCI))
    ## don't plot variances, by default
    fvnames(result, ".") <- setdiff(fvnames(result, "."),
                                    fvnames(Variance, "."))
    return(result)
  }

  pool.fv
})


  

#'
#'       summary.kppm.R
#'
#'   $Revision: 1.6 $  $Date: 2018/11/10 09:43:05 $
#' 

summary.kppm <- function(object, ..., quick=FALSE) {
  nama <- names(object)
  result <- unclass(object)[!(nama %in% c("X", "po", "call", "callframe"))]
  ## handle old format
  if(is.null(result$isPCP)) result$isPCP <- TRUE
  ## summarise trend component
  result$trend <- summary(as.ppm(object), ..., quick=quick)
  if(identical(quick, FALSE)) {
    theta <- coef(object)
    if(length(theta) > 0) {
      vc <- vcov(object, matrix.action="warn")
      if(!is.null(vc)) {
        se <- if(is.matrix(vc)) sqrt(diag(vc)) else
        if(length(vc) == 1) sqrt(vc) else NULL
      }
      if(!is.null(se)) {
        two <- qnorm(0.975)
        lo <- theta - two * se
        hi <- theta + two * se
        zval <- theta/se
        pval <- 2 * pnorm(abs(zval), lower.tail=FALSE)
        psig <- cut(pval, c(0,0.001, 0.01, 0.05, 1),
                    labels=c("***", "**", "*", "  "),
                    include.lowest=TRUE)
        ## table of coefficient estimates with SE and 95% CI
        result$coefs.SE.CI <- data.frame(Estimate=theta, S.E.=se,
                                         CI95.lo=lo, CI95.hi=hi,
                                         Ztest=psig,
                                         Zval=zval)
      }
    }
  }
  class(result) <- "summary.kppm"
  return(result)
}

coef.summary.kppm <- function(object, ...) {
  return(object$coefs.SE.CI)
}

print.summary.kppm <- function(x, ...) {
  terselevel <- spatstat.options('terse')
  digits <- getOption('digits')
  isPCP <- x$isPCP
  splat(if(x$stationary) "Stationary" else "Inhomogeneous",
        if(isPCP) "cluster" else "Cox",
        "point process model")

  if(waxlyrical('extras', terselevel) && nchar(x$Xname) < 20)
    splat("Fitted to point pattern dataset", sQuote(x$Xname))

  Fit <- x$Fit
  
  if(waxlyrical('gory', terselevel)) {
    switch(Fit$method,
           mincon = {
             splat("Fitted by minimum contrast")
             splat("\tSummary statistic:", Fit$StatName)
             print(Fit$mcfit)
           },
           clik  =,
           clik2 = {
             splat("Fitted by maximum second order composite likelihood")
             splat("\trmax =", Fit$rmax)
             if(!is.null(wtf <- Fit$weightfun)) {
               a <- attr(wtf, "selfprint") %orifnull% pasteFormula(wtf)
               splat("\tweight function:", a)
             }
             printStatus(optimStatus(Fit$clfit))
           },
           palm = {
             splat("Fitted by maximum Palm likelihood")
             splat("\trmax =", Fit$rmax)
             if(!is.null(wtf <- Fit$weightfun)) {
               a <- attr(wtf, "selfprint") %orifnull% pasteFormula(wtf)
               splat("\tweight function:", a)
             }
             printStatus(optimStatus(Fit$clfit))
           },
           warning(paste("Unrecognised fitting method", sQuote(Fit$method)))
           )
  }

  # ............... trend .........................

  parbreak()
  splat("----------- TREND MODEL -----")
  print(x$trend, ...)

  # ..................... clusters ................

  tableentry <- spatstatClusterModelInfo(x$clusters)
  
  parbreak()
  splat("-----------", 
        if(isPCP) "CLUSTER" else "COX",
        "MODEL",
        "-----------")
  splat("Model:", tableentry$printmodelname(x))
  parbreak()
  
  cm <- x$covmodel
  if(!isPCP) {
    # Covariance model - LGCP only
    splat("\tCovariance model:", cm$model)
    margs <- cm$margs
    if(!is.null(margs)) {
      nama <- names(margs)
      tags <- ifelse(nzchar(nama), paste(nama, "="), "")
      tagvalue <- paste(tags, margs)
      splat("\tCovariance parameters:",
            paste(tagvalue, collapse=", "))
    }
  }
  pa <- x$clustpar
  if (!is.null(pa)) {
    splat("Fitted",
          if(isPCP) "cluster" else "covariance",
          "parameters:")
    print(pa, digits=digits)
  }

  if(!is.null(mu <- x$mu)) {
    if(isPCP) {
      splat("Mean cluster size: ",
            if(!is.im(mu)) paste(signif(mu, digits), "points") else "[pixel image]")
    } else {
      splat("Fitted mean of log of random intensity:",
            if(!is.im(mu)) signif(mu, digits) else "[pixel image]")
    }
  }
  # table of coefficient estimates with SE and 95% CI
  if(!is.null(cose <- x$coefs.SE.CI)) {
    parbreak()
    splat("Final standard error and CI")
    splat("(allowing for correlation of",
          if(isPCP) "cluster" else "Cox",
          "process):")
    print(cose)
  }
  invisible(NULL)
}

#
#    predictmppm.R
#
#	$Revision: 1.13 $	$Date: 2020/01/21 09:56:47 $
#
#
# -------------------------------------------------------------------

predict.mppm <- local({

  predict.mppm <- function(object, ..., newdata=NULL, type=c("trend", "cif"),
                           ngrid=40, locations=NULL, verbose=FALSE) {
    ##
    ##	'object' is the output of mppm()
    ##
    model <- object
    verifyclass(model, "mppm")
    ##
    ##  
    ##       'type'  
    type <- pickoption("type", type, c(trend="trend",
                                       lambda="cif",
                                       cif="cif"), multi=TRUE)
    want.trend <- "trend" %in% type
    want.cif   <- "cif"   %in% type
    ##
    selfcheck <- resolve.defaults(list(...), list(selfcheck=FALSE))$selfcheck
    ##
    ##
    if(verbose)
      cat("Inspecting arguments...")
    ##
    ##       'newdata'
    use.olddata <- is.null(newdata)
    if(use.olddata) {
      newdata <- model$data
      newdataname <- "Original data"
    } else {
      stopifnot(is.data.frame(newdata) || is.hyperframe(newdata))
      newdataname <- sQuote("newdata")
    }
    ##
    ##
    ##    Locations for prediction
    if(is.hyperframe(locations)) 
      locations <- locations[,1,drop=TRUE]
    if(is.list(locations))
      cls <- unique(sapply(locations, class))

    loctype <-
      if(is.null(locations)) "null" else
      if(is.data.frame(locations))  "data.frame" else
      if(is.list(locations)) {
        if(any(c("ppp", "quad") %in% cls)) "points"
        else if("owin" %in% cls) {
          if(all(sapply(locations, is.mask)))
            "mask"
          else
            "window"
        } else "unknown"
      } else "unknown"

    need.grid <- switch(loctype,
                        null      =TRUE,
                        data.frame=FALSE,
                        points    =FALSE,
                        mask      =FALSE,
                        window    =TRUE,
                        unknown   =stop("Unrecognised format for locations"))
    make.image <- need.grid || (loctype == "mask")
    ##  
    locationvars <- c("x", "y", "id")
    ##  
    ##
    if(verbose)
      cat("done.\nDetermining locations for prediction...")
    if(need.grid) {
      ## prediction on a grid is required
      if(is.data.frame(newdata))
        stop(paste("Cannot predict model on a grid;", newdataname,
                   "are a data frame"))
    } else {
      ## prediction at  `locations' is required
      if(is.hyperframe(newdata)) {
        ## check consistency between locations and newdata
        nloc <- length(locations)
        nnew <- summary(newdata)$ncases
        if(nloc != nnew)
          stop(paste("Length of argument", sQuote("locations"), paren(nloc),
                     "does not match number of rows in",
                     newdataname, paren(nnew)))
      } else {
        ## newdata is a data frame
        if(!is.data.frame(locations)) 
          stop(paste(newdataname,
                     "is a data frame; locations must be a data frame"))
        else {
          stopifnot(nrow(locations) == nrow(newdata))
          dup <- names(newdata) %in% names(locations)
          if(any(dup))
            for(nam in names(newdata)[dup])
              if(!isTRUE(all.equal(newdata[,nam], locations[,nam])))
                stop(paste("The data frames newdata and locations",
                           "both have a column called", sQuote(nam),
                           "but the entries differ"))
          nbg <- !(locationvars %in% c(names(newdata),names(locations)))
          if(any(nbg))
            stop(paste(ngettext(sum(nbg), "Variable", "Variables"),
                       commasep(locationvars[nbg]),
                       "not provided"))
          ## merge the two data frames
          newdata <- cbind(newdata[,!dup], locations)
          locations <- NULL
        }
      }
    }
    if(verbose)
      cat("done.\n Constructing data for prediction...")
    ##  
    ##
    ## extract fitted glm/gam/glmm object
    FIT <- model$Fit$FIT
    ## extract names of interaction variables
    Vnamelist <- model$Fit$Vnamelist
    vnames <- unlist(Vnamelist)
    ##
    ##  
    ## newdata is data frame
    if(is.data.frame(newdata)) {
      if(verbose)
        cat("(data frame)...")
      if(need.grid)
        stop("Cannot predict model on a grid; newdata is a data frame")
      ## use newdata as covariates
      nbg <- !(locationvars %in% names(newdata))
      if(any(nbg))
        stop(paste(ngettext(sum(nbg), "variable", "variables"),
                   commasep(locationvars[nbg]),
                   "not provided"))
      ## create output data frame
      answer <- as.data.frame(matrix(, nrow=nrow(newdata), ncol=0),
                              row.names=row.names(newdata))
      if(want.trend) {
        ## add interaction components, set to zero (if any)
        if(length(vnames) > 0)
          newdata[, vnames] <- 0
        ## compute fitted values
        answer$trend <- Predict(FIT, newdata=newdata, type="response")
      }
      if(want.cif) {
        if(is.poisson(object)) {
          ## cif = trend
          answer$cif <- if(want.trend) answer$trend else
                        Predict(FIT, newdata=newdata, type="response")
        } else {
          warning("Not yet implemented (computation of cif in data frame case)")        ## split data frame by 'id'
          ## compute interaction components using existing point patterns
          ## compute fitted values
        }
      }
      return(answer)
    }
  
    ## newdata is a hyperframe
    if(verbose)
      cat("(hyperframe)...")
    sumry <- summary(newdata)
    npat.new <- sumry$ncases
    ## name of response point pattern in model
    Yname <- model$Info$Yname
    ##
    ## Determine response point patterns if known.
    ## Extract from newdata if available
    ## Otherwise from the original data if appropriate
    if(verbose)
      cat("(responses)...")
    Y <- if(Yname %in% sumry$col.names) 
      newdata[, Yname, drop=TRUE, strip=FALSE]
    else if(npat.new == model$npat)
      data[, Yname, drop=TRUE, strip=FALSE]
    else NULL
    ##
    if(want.cif && is.null(Y))
      stop(paste("Cannot compute cif:",
                 "newdata does not contain column", dQuote(Yname),
                 "of response point patterns"))
    ##
    ## Determine windows for prediction 
    if(verbose)
      cat("(windows)...")
    Wins <- if(!need.grid)
      lapply(locations, as.owin, fatal=FALSE)
    else if(!is.null(Y))
      lapply(Y, as.owin, fatal=FALSE)
    else NULL
    if(is.null(Wins) || any(sapply(Wins, is.null)))
      stop("Cannot determine windows where predictions should be made")
    ##
    ##
    if(is.null(Y)) {
      ## only want trend; empty patterns will do
      Y <- lapply(Wins, emptypattern)
    }
    
    ## ensure Y contains data points only 
    if(is.quad(Y[[1]]))
      Y <- lapply(Y, getElement, name="data")

    ## Determine locations for prediction
    if(need.grid) {
      ## Generate grids of dummy locations 
      if(verbose)
        cat("(grids)...")
      Gridded <- lapply(Wins, gridsample, ngrid=ngrid)
      Dummies   <- lapply(Gridded, getElement, name="D")
      Templates <- lapply(Gridded, getElement, name="I")
    } else {
      ## locations are given somehow
      if(verbose)
        cat("(locations)...")
      if(loctype == "points")
        Dummies <- locations
      else if(loctype == "mask") {
        Dummies <- lapply(locations, punctify)
        Templates <- lapply(locations, as.im)
      } else
        stop("Internal error: illegal loctype")
    }
  
    ## Pack into quadschemes
    if(verbose)
      cat("(quadschemes)...")
    Quads <- list()
    for(i in seq(npat.new)) 
      Quads[[i]] <- quad(data=Y[[i]], dummy=Dummies[[i]])
    ## Insert quadschemes into newdata
    newdata[, Yname] <- Quads
    
    ## Determine interactions to be used
    if(verbose)
      cat("(interactions)...")
    interactions <- model$Inter$interaction
    ninter <- if(is.hyperframe(interactions)) nrow(interactions) else 1
    nnew <- nrow(newdata)
    if(ninter != nnew && ninter != 1) {
      if(!all(model$Inter$constant))
        stop(paste("Number of rows of newdata", paren(nnew),
                   "does not match number of interactions in model",
                   paren(ninter)))
      interactions <- interactions[1, ]
    }

    ## compute the Berman-Turner frame
    if(verbose)
      cat("done.\nStarting prediction...(Berman-Turner frame)...")
    moadf <- mppm(formula     = model$formula,
                  data        = newdata,
                  interaction = interactions,
                  iformula    = model$iformula,
#%^!ifdef RANDOMEFFECTS                
                  random      = model$random,
#%^!endif                
                  use.gam     = model$Fit$use.gam,
                  correction  = model$Info$correction,
                  rbord       = model$Info$rbord,
                  backdoor    = TRUE)
    ## compute fitted values
    if(verbose)
      cat("(glm prediction)...")
    values <- moadf[, c("x", "y", "id")]
    if(want.cif)
      values$cif <- Predict(FIT, newdata=moadf, type="response")
    if(want.trend) {
      if(length(vnames) == 0) {
        ## Poisson model: trend = cif 
        values$trend <-
          if(want.cif) values$cif else
          Predict(FIT, newdata=moadf, type="response")
      } else {
        ## zero the interaction components
        moadf[, vnames] <- 0
        ## compute fitted values
        values$trend <- Predict(FIT, newdata=moadf, type="response")
      }
    }
    if(verbose)
      cat("done.\nReshaping results...")
    ##
    ## Reshape results
    ## separate answers for each image
    values <- split(values, values$id)
    ## 
    Trends <- list()
    Lambdas <- list()
    if(!make.image) {
      if(verbose)
        cat("(marked point patterns)...")
      ## values become marks attached to locations
      for(i in seq(npat.new)) {
        Val <- values[[i]]
        Loc <- Dummies[[i]]
        isdum <- !is.data(Quads[[i]])
        if(selfcheck)
          if(length(isdum) != length(Val$trend))
            stop("Internal error: mismatch between data frame and locations")
        if(want.trend)
          Trends[[i]] <- Loc %mark% (Val$trend[isdum])
        if(want.cif)
          Lambdas[[i]] <- Loc %mark% (Val$cif[isdum])
      }
    } else {
      if(verbose)
        cat("(pixel images)...")
      ## assign values to pixel images
      for(i in seq(npat.new)) {
        values.i <- values[[i]]
        Q.i <- Quads[[i]]
        values.i <- values.i[!is.data(Q.i), ]
        Template.i <- Templates[[i]]
        ok.i <- !is.na(Template.i$v)
        if(sum(ok.i) != nrow(values.i))
          stop("Internal error: mismatch between data frame and image")
        if(selfcheck) {
          dx <- rasterx.im(Template.i)[ok.i] - values.i$x
          dy <- rastery.im(Template.i)[ok.i] - values.i$y
          cat(paste("i=", i, "range(dx) =", paste(range(dx), collapse=", "),
                    "range(dy) =", paste(range(dy), collapse=", "), "\n"))
        }
        if(want.trend) {
          Trend.i <- Template.i
          Trend.i$v[ok.i] <- values.i$trend
          Trends[[i]] <- Trend.i
        }
        if(want.cif) {
          Lambda.i <- Template.i
          Lambda.i$v[ok.i] <- values.i$cif
          Lambdas[[i]] <- Lambda.i
        }
      }
    }
    if(verbose)
      cat("done.\n")
    ## answer is a hyperframe
    Answer <- hyperframe(id=factor(levels(moadf$id)),
                         row.names=sumry$row.names)
    if(want.trend)
      Answer$trend <- Trends
    if(want.cif)
      Answer$cif <- Lambdas
    return(Answer)
  }

  ## helper functions
  emptypattern <- function(w) { ppp(numeric(0), numeric(0), window=w) }

  gridsample <- function(W, ngrid) {
    masque <- as.mask(W, dimyx=ngrid)
    xx <- raster.x(masque)
    yy <- raster.y(masque)
    xpredict <- xx[masque$m]
    ypredict <- yy[masque$m]
    Dummy <- ppp(xpredict, ypredict, window=W)
    Image <- as.im(masque)
    return(list(D=Dummy, I=Image))
  }

  punctify <- function(M) { 
    xx <- raster.x(M)
    yy <- raster.y(M)
    xpredict <- xx[M$m]
    ypredict <- yy[M$m]
    return(ppp(xpredict, ypredict, window=M))
  }

  Predict <- function(object, newdata, type=c("link", "response")) {
    type <- match.arg(type)
    if(inherits(object, "glmmPQL")) {
      class(object) <- class(object)[-1L]
      pred <- predict(object, newdata=newdata)
      if(type == "response") pred <- object$family$linkinv(pred)
    } else {
      pred <- predict(object, newdata=newdata, type=type)
    }
    return(as.numeric(pred))
  }
  
  predict.mppm
})

#
# rshift.psp.R
#
#  $Revision: 1.7 $  $Date: 2019/11/18 06:22:50 $
#


rshift.psp <- function(X, ..., group=NULL, which=NULL) {
  verifyclass(X, "psp")
  
  # process arguments
  W <- rescue.rectangle(X$window)
  arglist <- handle.rshift.args(W, ..., edgedefault="erode")
  radius <- arglist$radius
  width  <- arglist$width
  height <- arglist$height
  edge   <- arglist$edge
  clip   <- arglist$clip
  if(W$type != "rectangle")
    stop("Not yet implemented for non-rectangular windows")
  if(edge != "erode")
    stop(paste("Only implemented for edge=", dQuote("erode")))

  # split into groups
  if(is.null(group))
    Y <- list(X)
  else {
    stopifnot(is.factor(group))
    stopifnot(length(group) == X$n)
    Y <- lapply(levels(group),
                function(l, Z, group) {Z[group == l]},
                Z=X, group=group)
  }

  ############ loop ################
  result <- NULL
  
  for(i in seq_along(Y)) {
    
    Z <- Y[[i]]
    
    # generate random translation vector
    if(!is.null(radius)) 
      jump <- runifdisc(1, radius=radius)
    else {
      jump <- list(x=runif(1, min=0, max=width),
                   y=runif(1, min=0, max=height))
    }
    # translate segments
    Zsh <- shift(Z, c(jump$x, jump$y))
    Zsh$window <- W

    # append to result
    result <- append.psp(result, Zsh)
  }

  # clip 
  if(!is.null(clip))
   result <- result[clip]

  return(result)
}

#
#   rlabel.R
#
#   random (re)labelling
#
#   $Revision: 1.12 $   $Date: 2019/04/25 02:59:12 $
#
#
rlabel <- function(X, labels=marks(X), permute=TRUE, nsim=1, drop=TRUE) {
  stopifnot(is.ppp(X) || is.lpp(X) || is.pp3(X) || is.ppx(X) || is.psp(X))
  if(is.null(labels))
    stop("labels not given and marks not present")
  nthings <- nobjects(X)
  things <- if(is.psp(X)) "segments" else "points"
  if(is.vector(labels) || is.factor(labels)) {
    nlabels <- length(labels)
    if(permute && (nlabels != nthings))
      stop(paste("length of labels vector does not match number of", things))
    Y <- replicate(nsim,
                   X %mark% sample(labels, nthings, replace=!permute),
                   simplify=FALSE)
  } else if(is.data.frame(labels) || is.hyperframe(labels)) {
    nlabels <- nrow(labels)
    if(permute && (nlabels != nthings))
      stop(paste("number of rows of data frame does not match number of",
                 things))
    Y <- replicate(nsim,
                   X %mark% labels[sample(1:nlabels, nthings, replace=!permute), ,drop=FALSE],
                   simplify=FALSE)
  } else stop("Format of labels argument is not understood")
  return(simulationresult(Y, nsim, drop))
}

#
# connected.R
#
# connected component transform
#
#    $Revision: 1.20 $  $Date: 2017/11/06 02:01:55 $
#
# Interpreted code for pixel images by Julian Burgos <jmburgos@u.washington.edu>
# Rewritten in C by Adrian Baddeley
#
# Code for point patterns by Adrian Baddeley

connected <- function(X, ...) {
  UseMethod("connected")
}

connected.im <- function(X, ..., background=NA, method="C") {
  W <- if(!is.na(background)) solutionset(X != background) else 
       if(X$type == "logical") solutionset(X) else as.owin(X)
  connected.owin(W, method=method, ...)
}

connected.owin <- function(X, ..., method="C") {
  method <- pickoption("algorithm choice", method,
                       c(C="C", interpreted="interpreted"))
  # convert X to binary mask
  X <- as.mask(X, ...)
  #     
  Y <- X$m
  nr <- X$dim[1L]
  nc <- X$dim[2L]

  if(method == "C") {
################ COMPILED CODE #########################
# Pad border with FALSE
    M <- rbind(FALSE, Y, FALSE)
    M <- cbind(FALSE, M, FALSE)
    # assign unique label to each foreground pixel 
    L <- M
    L[M] <- seq_len(sum(M))
    L[!M] <- 0
    # resolve labels
    z <- .C("cocoImage",
            mat=as.integer(t(L)),
            nr=as.integer(nr),
            nc=as.integer(nc),
            PACKAGE = "spatstat")
    # unpack
    Z <- matrix(z$mat, nr+2, nc+2, byrow=TRUE)
  } else {
################ INTERPRETED CODE #########################
# by Julian Burgos
#  
# Pad border with zeros
    padY <- rbind(0, Y, 0)
    padY <- cbind(0, padY, 0)
    # Initialise 
    Z <- matrix(0, nrow(padY), ncol(padY))
    currentlab <- 1L
    todo <- as.vector(t(Y))
    equiv <- NULL
  
    # ........ main loop ..........................
    while(any(todo)){
      # pick first unresolved pixel
      one <- which(todo)[1L]
      onerow <- ceiling(one/nc)
      onecol <- one -((onerow-1L)*nc)
      parow=onerow+1L # Equivalent rows & column in padded matrix
      pacol=onecol+1L
      #Examine four previously scanned neighbors
      # (use padded matrix to avoid edge issues)
      nbrs <- rbind(c(parow-1L,pacol-1L),
                    c(parow-1L,pacol),
                    c(parow,  pacol-1L),
                    c(parow-1L,pacol+1L))
      px <- sum(padY[nbrs])
      if (px==0){
        # no neighbours: new component
        Z[parow,pacol] <- currentlab
        currentlab <- currentlab+1L
        todo[one] <- FALSE
      } else if(px==1L) {
        # one neighbour: assign existing label
        labs <- unique(Z[nbrs], na.rm=TRUE)
        labs <- labs[labs != 0]
        Z[parow,pacol] <- labs[1L]
        currentlab <- max(Z)+1L
        todo[one] <- FALSE
      } else {
        # more than one neighbour: possible merger of labels
        labs <- unique(Z[nbrs], na.rm=TRUE)
        labs <- labs[labs != 0]
        labs <- sort(labs)
        equiv <- rbind(equiv,c(labs,rep.int(0,times=4-length(labs))))
        Z[parow,pacol] <- labs[1L]
        currentlab <- max(Z)+1L
        todo[one] <- FALSE
      }
    }
    # ........... end of loop ............
    # Resolve equivalences ................

    if(length(equiv)>1L){
      merges <- (equiv[,2L] > 1L)
      nmerge <- sum(merges)
      if(nmerge==1L)
        equiv <- equiv[which(merges), , drop=FALSE]
      else if(nmerge > 1L) {
        relevant <- (equiv[,2L] > 0)
        equiv <- equiv[relevant, , drop=FALSE]
        equiv <- equiv[fave.order(equiv[,1L]),]
      }
      for (i in 1:nrow(equiv)){
        current <- equiv[i, 1L]
        for (j in 2:4){
          twin <- equiv[i,j]
          if (twin>0){
            # Change labels matrix
            Z[which(Z==twin)] <- current
            # Update equivalence table
            equiv[which(equiv==twin)] <- current
          }
        }
      }
    }
  }

  ########### COMMON CODE ############################
    
  # Renumber labels sequentially
  mapped <- (Z != 0)
  usedlabs <- sortunique(as.vector(Z[mapped]))
  nlabs <- length(usedlabs)
  labtable <- cumsum(seq_len(max(usedlabs)) %in% usedlabs)
  Z[mapped] <- labtable[Z[mapped]]

  # banish zeroes
  Z[!mapped] <- NA
  
  # strip borders
  Z <- Z[2:(nrow(Z)-1L),2:(ncol(Z)-1L)]
  # dress up 
  Z <- im(factor(Z, levels=1:nlabs),
          xcol=X$xcol, yrow=X$yrow, unitname=unitname(X))
  return(Z)
}

connected.ppp <- connected.pp3 <- function(X, R, ...) {
  methodname <- if(is.ppp(X)) "connected.ppp" else
                if(is.pp3(X)) "connected.pp3" else 
                stopifnot(is.ppp(X) || is.pp3(X))
  check.1.real(R, paste("In", methodname))
  stopifnot(R >= 0)
  internal <- resolve.1.default("internal", list(...), list(internal=FALSE))
  nv <- npoints(X)
  cl <- closepairs(X, R, what="indices")
  ie <- cl$i - 1L
  je <- cl$j - 1L
  ne <- length(ie)
  zz <- .C("cocoGraph",
           nv=as.integer(nv),
           ne=as.integer(ne),
           ie=as.integer(ie),
           je=as.integer(je),
           label=as.integer(integer(nv)),
           status=as.integer(integer(1L)),
           PACKAGE = "spatstat")
  if(zz$status != 0)
    stop(paste("Internal error:", methodname, "did not converge"), call.=FALSE)
  if(internal)
    return(zz$label)
  lab <- zz$label + 1L
  # Renumber labels sequentially 
  lab <- as.integer(factor(lab))
  # Convert labels to factor
  lab <- factor(lab)
  # Apply to points
  Y <- X %mark% lab
  return(Y)
}

# .................................................

is.connected <- function(X, ...) {
  UseMethod("is.connected")
}

is.connected.default <- function(X, ...) {
  y <- connected(X, ...)
  npieces <- length(levels(y))
  if(npieces == 0)
    stop("Unable to determine connectedness")
  return(npieces == 1)
}

is.connected.ppp <- function(X, R, ...) {
  lab <- connected(X, R, internal=TRUE)
  npieces <- length(unique(lab))
  return(npieces == 1)
}

#
#	exactPdt.R
#	R function exactPdt() for exact distance transform of pixel image
#
#	$Revision: 4.17 $	$Date: 2017/06/05 10:31:58 $
#

"exactPdt"<-
  function(w)
{
  verifyclass(w, "owin")
  if(w$type != "mask")
    stop(paste("Input must be a window of type", sQuote("mask")))
#	
  nr <- w$dim[1L]
  nc <- w$dim[2L]
# input image will be padded out with a margin of width 2 on all sides
  mr <- mc <- 2L
  # full dimensions of padded image
  Nnr <- nr + 2 * mr
  Nnc <- nc + 2 * mc
  N <- Nnr * Nnc
  # output image (subset): rows & columns (R indexing)
  rmin <- mr + 1L
  rmax <- Nnr - mr
  cmin <- mc + 1L
  cmax <- Nnc - mc
  # do padding
  x <- matrix(FALSE, nrow=Nnr, ncol=Nnc)
  x[rmin:rmax, cmin:cmax] <- w$m
  #
  res <- .C("ps_exact_dt_R",
            as.double(w$xrange[1L]),
            as.double(w$yrange[1L]),
            as.double(w$xrange[2L]),
            as.double(w$yrange[2L]),
            nr = as.integer(nr),
            nc = as.integer(nc),
            mr = as.integer(mr),
            mc = as.integer(mc),
            inp = as.integer(t(x)),
            distances = as.double (double(N)),
            rows      = as.integer(integer(N)),
            cols      = as.integer(integer(N)),
            boundary  = as.double (double(N)),
            PACKAGE = "spatstat")
  dist <- matrix(res$distances,
                 ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
  rows <- matrix(res$rows,
                 ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
  cols <- matrix(res$cols,
                 ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
  bdist<- matrix(res$boundary,
                 ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
  # convert from C to R indexing
  rows <- rows + 1L - as.integer(mr)
  cols <- cols + 1L - as.integer(mc)
  return(list(d=dist,row=rows,col=cols,b=bdist, w=w))
}

project2set <- function(X, W, ...) {
  stopifnot(is.ppp(X))
  W <- as.mask(W, ...)
  eW <- exactPdt(W)
  ## grid location of X
  XX <- nearest.raster.point(X$x, X$y, W)
  ijX <- cbind(XX$row, XX$col)
  ## look up values of 'eW' at this location 
  iY <- eW$row[ijX]
  jY <- eW$col[ijX]
  ## convert to spatial coordinates
  Y <- ppp(W$xcol[jY], W$yrow[iY], window=W)
  return(Y)
}

# 	applynbd.R
#
#     $Revision: 1.17 $     $Date: 2016/10/23 10:36:58 $
#
#  applynbd()
# For each point, identify either
#	 - all points within distance R
#        - the closest N points  
#        - those points satisfying some constraint
# and apply the function FUN to them
#
#  markstat()
#      simple application of applynbd
#################################################################


applynbd <- function(X, FUN, N=NULL, R=NULL, criterion=NULL, exclude=FALSE, ...) {

  if(is.null(N) && is.null(R) && is.null(criterion)) 
    stop(paste("must specify at least one of the arguments",
               commasep(sQuote(c("N","R","criterion")))))
     
  X <- as.ppp(X)
  npts <- npoints(X)

  # compute matrix of pairwise distances
  dist <- pairdist(X)

  # compute row ranks (avoid ties)
  rankit <- function(x) {  u <- numeric(length(x)); u[fave.order(x)] <- seq_along(x); return(u) }
  drank <- t(apply(dist, 1L, rankit)) - 1L

  included <- matrix(TRUE, npts, npts)
  if(!is.null(R)) {
    # select points closer than R
    included <- included & (dist <= R)
  }
  if(!is.null(N)) {
    # select N closest points
    if(N < 1)
      stop("Value of N must be at least 1")
    if(exclude)
      included <- included & (drank <= N) 
    else
      included <- included & (drank <= N-1)
  }
  if(!is.null(criterion)) {
    # some funny criterion
    for(i in 1L:npts) 
      included[i,] <- included[i,] & criterion(dist[i,], drank[i,])
  }
     
  if(exclude) 
    diag(included) <- FALSE

  # bind into an array
  a <- array(c(included, dist, drank, row(included)), dim=c(npts,npts,4))

  # what to do with a[i, , ]
  if(!is.marked(X)) 
    go <- function(ai, Z, fun, ...) { 
      which <- as.logical(ai[,1L])
      distances <- ai[,2L]
      dranks <- ai[,3L]
      here <- ai[1L,4L]
      fun(Y=Z[which],
          current=c(x=Z$x[here], y=Z$y[here]),
          dists=distances[which], dranks=dranks[which],
          ...) 
    }
  else
    go <- function(ai, Z, fun, ...) { 
      which <- as.logical(ai[,1L])
      distances <- ai[,2L]
      dranks <- ai[,3L]
      here <- ai[1L,4L]
      fun(Y=Z[which],
          current=Z[here],
          dists=distances[which], dranks=dranks[which],
          ...) 
    }
  
  # do it
  result <- apply(a, 1, go, Z=X, fun=FUN, ...)
  
  return(result)
}

markstat <- function(X, fun, N=NULL, R=NULL, ...) {
  verifyclass(X, "ppp")
  stopifnot(is.function(fun))
  statfun <- function(Y, current, dists, dranks, func, ...)
    { func(marks(Y, dfok=TRUE), ...) }
  applynbd(X, statfun, R=R, N=N, func=fun, ...)
}

#
#     options.R
#
#     Spatstat options and other internal states
#
#    $Revision: 1.87 $   $Date: 2020/03/26 03:10:54 $
#
#

.spEnv <- new.env()

putSpatstatVariable <- function(name, value) {
  assign(name, value, envir=.spEnv)
}
getSpatstatVariable <- function(name) {
  get(name, envir=.spEnv)
}
existsSpatstatVariable <- function(name) {
  exists(name, envir=.spEnv)
}

putSpatstatVariable("Spatstat.Options", list())
putSpatstatVariable("Spatstat.ProgressBar", NULL)
putSpatstatVariable("Spatstat.ProgressData", NULL)
putSpatstatVariable("warnedkeys", character(0))

## Kovesi's uniform colour map, row 29, linear 'bmy'
putSpatstatVariable("DefaultImageColours", 
c("#000C7D", "#000D7E", "#000D80", "#000E81", "#000E83", "#000E85", 
"#000F86", "#000F88", "#00108A", "#00108B", "#00118D", "#00118F", 
"#001190", "#001292", "#001293", "#001295", "#001396", "#001398", 
"#001399", "#00149A", "#00149C", "#00149D", "#00149E", "#00159F", 
"#0015A0", "#0015A1", "#0015A2", "#0015A3", "#0015A4", "#0016A5", 
"#0016A6", "#0016A6", "#0016A7", "#0016A8", "#0016A8", "#0016A8", 
"#0A16A9", "#1516A9", "#1D15A9", "#2315A9", "#2915A9", "#2F15A8", 
"#3414A8", "#3914A7", "#3E13A6", "#4313A5", "#4712A4", "#4C12A3", 
"#5011A2", "#5311A1", "#5710A0", "#5A0F9F", "#5E0F9E", "#610E9E", 
"#640E9D", "#670D9C", "#6A0D9B", "#6C0C9A", "#6F0B99", "#720B98", 
"#740A98", "#770A97", "#790996", "#7C0896", "#7E0895", "#800794", 
"#810794", "#840693", "#860692", "#880692", "#8A0591", "#8C0591", 
"#8E0490", "#900490", "#92048F", "#94038F", "#96038E", "#98038E", 
"#9A028D", "#9C028D", "#9E028D", "#A0018C", "#A2018C", "#A4018B", 
"#A6018B", "#A8008A", "#AA008A", "#AB0089", "#AD0089", "#AF0088", 
"#B10088", "#B30087", "#B50087", "#B70086", "#B80086", "#BA0086", 
"#BC0085", "#BE0085", "#C00084", "#C20084", "#C30083", "#C50083", 
"#C70082", "#C90082", "#CB0081", "#CD0081", "#CE0080", "#D00080", 
"#D20080", "#D40080", "#D5007F", "#D7007F", "#D9007E", "#DA007E", 
"#DC007D", "#DD007C", "#DF017C", "#E1027B", "#E2047B", "#E4067A", 
"#E5087A", "#E70B79", "#E80D78", "#E91078", "#EB1277", "#EC1477", 
"#ED1676", "#EF1875", "#F01A75", "#F11C74", "#F31E73", "#F42073", 
"#F52272", "#F62471", "#F72671", "#F82870", "#FA2A6F", "#FB2C6F", 
"#FC2E6E", "#FD306D", "#FE326C", "#FE346C", "#FE366B", "#FE386A", 
"#FE3A6A", "#FE3D69", "#FE3F68", "#FE4167", "#FE4366", "#FE4566", 
"#FE4765", "#FE4964", "#FE4B63", "#FE4D62", "#FE5062", "#FE5261", 
"#FE5460", "#FE565F", "#FE585E", "#FE5A5D", "#FE5D5C", "#FE5F5B", 
"#FE615B", "#FE635A", "#FE6559", "#FE6758", "#FE6A57", "#FE6C56", 
"#FE6E55", "#FE7054", "#FE7253", "#FE7452", "#FE7651", "#FE7850", 
"#FE7A4E", "#FE7C4D", "#FE7E4C", "#FE7F4B", "#FE804A", "#FE8249", 
"#FE8448", "#FE8647", "#FE8745", "#FE8944", "#FE8B43", "#FE8D42", 
"#FE8E40", "#FE903F", "#FE923E", "#FE943C", "#FE953B", "#FE9739", 
"#FE9938", "#FE9A36", "#FE9C35", "#FE9E33", "#FE9F32", "#FEA130", 
"#FEA22F", "#FEA42E", "#FEA52C", "#FEA72B", "#FEA82A", "#FEAA29", 
"#FEAB28", "#FEAD27", "#FEAE26", "#FEB026", "#FEB125", "#FEB324", 
"#FEB423", "#FEB523", "#FEB722", "#FEB822", "#FEBA21", "#FEBB20", 
"#FEBC20", "#FEBE1F", "#FEBF1F", "#FEC11F", "#FEC21E", "#FEC31E", 
"#FEC51E", "#FEC61D", "#FEC71D", "#FEC91D", "#FECA1D", "#FECB1D", 
"#FECD1D", "#FECE1C", "#FECF1C", "#FED11C", "#FED21C", "#FED31C", 
"#FED51C", "#FED61D", "#FED71D", "#FED91D", "#FEDA1D", "#FEDB1D", 
"#FEDD1D", "#FEDE1E", "#FEDF1E", "#FEE11E", "#FEE21E", "#FEE31F", 
"#FEE51F", "#FEE61F", "#FEE720", "#FEE820", "#FEEA21", "#FEEB21", 
"#FEEC22", "#FEEE22", "#FEEF23", "#FEF023"))

warn.once <- function(key, ...) {
  warned <- getSpatstatVariable("warnedkeys")
  if(!(key %in% warned)) {
    warning(paste(...), call.=FALSE)
    putSpatstatVariable("warnedkeys", c(warned, key))
  }
  return(invisible(NULL))
}

".Spat.Stat.Opt.Table" <-
  list(
       areainter.polygonal = list(
         ## use polygonal calculations in AreaInter
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1},
         valid="a single logical value"
         ),
       checkpolygons = list(
         ## superseded
         superseded=TRUE,
         default=FALSE,
         check=function(x) {
           warning("spatstat.options('checkpolygons') will be ignored in future versions of spatstat", call.=FALSE)
           return(is.logical(x) && length(x) == 1)
         },
         valid="a single logical value"
         ),
       checksegments = list(
         ## default value of 'check' for psp objects
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1},
         valid="a single logical value"
         ),
       closepairs.newcode=list(
         ## use new code for 'closepairs'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       closepairs.altcode=list(
         ## use alternative code for 'closepairs'
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       crossing.psp.useCall=list(
         ## use new code for 'crossing.psp'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       crosspairs.newcode=list(
         ## use new code for 'crosspairs'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       densityTransform=list(
         ## use experimental new C routines for 'density.ppp'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       densityC=list(
         ## use C routines for 'density.ppp'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       dpp.maxmatrix=list(
         ## maximum size of matrix in dppeigen
         default=2^24, # 16,777,216
         check=function(x) {
           is.numeric(x) && length(x) == 1 &&
             all(x == ceiling(x)) && all(x > 1024)
         },
         valid="a single integer, greater than 1024"
       ),
       exactdt.checks.data=list(
         ## whether 'exactdt' checks validity of return value
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       expand=list(
         ## default area expansion factor
         default=2,
         check=function(x) {
           is.numeric(x) && length(x) == 1 && all(x > 1)
         },
         valid="a single numeric value, greater than 1"
       ),
       expand.polynom=list(
         ## whether to expand polynom() in ppm formulae
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       fasteval=list(
         ## whether to use 'fasteval' code if available
         default="on",
         check=function(x) { x %in% c("off", "on", "test") },
         valid="one of the strings \'off\', \'on\' or \'test\'"
       ),
       fastpois=list(
         # whether to use fast algorithm for rpoispp() when lambda is an image
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       fastthin=list(
         # whether to use fast C algorithm for rthin() when P is constant
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       fastK.lgcp=list(
         ## whether to cut a few corners in 'lgcp.estK'
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       fast.trigraph=list(
         ## whether to use C function triograph or trigraph in edges2triangles
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       fixpolygons = list(
         ## whether to repair polygons automatically
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1},
         valid="a single logical value"
         ),
       gpclib=list(
         ## defunct!
         superseded=TRUE, 
         default=FALSE,
         check=function(x) {
           message("gpclib is no longer needed")
           return(TRUE)
         },
         valid="a single logical value"
         ),
       huge.npoints=list(
         ## threshold to trigger a warning from rpoispp 
         default=1e6,
         check=function(x) {
           is.numeric(x) && length(x) == 1 &&
             all(x == ceiling(x)) && all(x > 1024)
         },
         valid="a single integer, greater than 1024"
       ),
       image.colfun=list(
         ## default colour scheme for plot.im
#         default=function(n){topo.colors(n)},
         default=function(n) {
           z <- getSpatstatVariable("DefaultImageColours")
           interp.colours(z, n)
         },
         check=function(x) {
           if(!is.function(x) || length(formals(x)) == 0) return(FALSE)
           y <- x(42)
           if(length(y) != 42 || !is.character(y)) return(FALSE)
           z <- try(col2rgb(y), silent=TRUE)
           return(!inherits(z, "try-error"))
         },
         valid="a function f(n) that returns character strings, interpretable as colours"
         ),
       Kcom.remove.zeroes=list(
         ## whether Kcom removes zero distances
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       maxedgewt=list(
         ## maximum edge correction weight 
         default=100,
         check=function(x){
           is.numeric(x) && length(x) == 1 && is.finite(x) && all(x >= 1)
         },
         valid="a finite numeric value, not less than 1"
       ),
       maxmatrix=list(
         ## maximum size of matrix of pairs of points in mpl.R
         default=2^24, # 16,777,216
         check=function(x) {
           is.numeric(x) && length(x) == 1 &&
             all(x == ceiling(x)) && all(x > 1024)
         },
         valid="a single integer, greater than 1024"
       ),
       monochrome = list(
         ## switch for monochrome colour scheme
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1},
         valid="a single logical value"
         ),
       n.bandwidth=list(
         ## number of values of bandwidth to try in bandwidth selection
         default=32,
         check=function(x) {
           is.numeric(x) && (length(x) == 1) &&
             all(x == ceiling(x)) && all(x > 2)
         },
         valid="a single integer, greater than 2"
       ),
       ndummy.min=list(
         ## minimum grid size for dummy points
         default=32,
         check=function(x) {
           is.numeric(x) && length(x) <= 2 && all(x == ceiling(x)) && all(x > 1)
         },
         valid="a single integer or a pair of integers, greater than 1"
       ),
       ngrid.disc=list(
         ## number of grid points used to calculate area in area-interaction
         default=128,
         check=function(x) {
           is.numeric(x) && length(x) == 1 && all(x == ceiling(x)) && all(x > 1)
         },
         valid="a single integer, greater than 1"
       ),
       npixel=list(
         ## default pixel dimensions
         default=128,
         check=function(x){
           is.numeric(x) && (length(x) %in% c(1,2)) && all(is.finite(x)) &&
           all(x == ceiling(x)) && all(x > 1) 
         },
         valid="an integer, or a pair of integers, greater than 1"
        ),
       nvoxel=list(
         ## default total number of voxels
         default=2^22,
         check=function(x) {
           is.numeric(x) && length(x) == 1 &&
             all(x == ceiling(x)) && all(x > 2^12)
         },
         valid="a single integer, greater than 2^12"
       ),
       old.morpho.psp=list(
         ## use old code for morphological operations
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       par.binary=list(
         ## default graphics parameters for masks
         default=list(),
         check=is.list,
         valid="a list"
         ),
       par.contour=list(
         ## default graphics parameters for 'contour'
         default=list(),
         check=is.list,
         valid="a list"
         ),
       par.fv=list(
         ## default graphics parameters for 'plot.fv'
         default=list(),
         check=is.list,
         valid="a list"
         ),
       par.persp=list(
         ## default graphics parameters for 'persp' plots
         default=list(),
         check=is.list,
         valid="a list"
         ),
       par.points=list(
         ## default graphics parameters for 'points'
         default=list(),
         check=is.list,
         valid="a list"
         ),
       par.pp3=list(
         ## default graphics parameters for 'plot.pp3'
         default=list(),
         check=is.list,
         valid="a list"
         ),
       print.ppm.SE=list(
         ## under what conditions to print estimated SE in print.ppm
         default="poisson",
         check=function(x) { is.character(x) && length(x) == 1 &&
                             all(x %in% c("always", "poisson", "never")) },
         valid="one of the strings \'always\', \'poisson\' or \'never\'"
       ),
       progress = list(
         ## how to display progress reports
         default="tty",
         check=function(x){ all(x %in% c("tty", "tk", "txtbar")) },
         valid="one of the strings 'tty', 'tk' or 'txtbar'"
         ),
       project.fast=list(
         ## whether to cut corners when projecting an invalid ppm object
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       psstA.ngrid=list(
         ## size of point grid for computing areas in psstA
         default=32,
         check=function(x) {
           is.numeric(x) && length(x) == 1 &&
             all(x == ceiling(x)) && all(x >= 8)
         },
         valid="a single integer, greater than or equal to 8"
       ),
       psstA.nr=list(
         ## number of 'r' values to consider in psstA
         default=30,
         check=function(x) {
           is.numeric(x) && length(x) == 1 &&
             all(x == ceiling(x)) && all(x >= 4)
         },
         valid="a single integer, greater than or equal to 4"
       ),
       psstG.remove.zeroes=list(
         ## whether to remove zero distances in psstG
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
      eroded.intensity=list(
         ## whether to compute intensity estimate in eroded window
         ## e.g. for Kcom, Gcom
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       rmh.nrep=list(
         ## default value of parameter 'nrep' in rmh
         default=5e5, 
         check=function(x) {
           is.numeric(x) && length(x) == 1 && all(x == ceiling(x)) && all(x > 0)
         },
         valid="a single integer, greater than 0"
       ),
       rmh.p=list(
         ## default value of parameter 'p' in rmh
         default=0.9,
         check=function(x) { is.numeric(x) && length(x) == 1 &&
                             all(x >= 0) && all(x <= 1) },
         valid="a single numerical value, between 0 and 1"
       ),
       rmh.q=list(
         ## default value of parameter 'q' in rmh
         default=0.9,
         check=function(x) { is.numeric(x) && length(x) == 1 &&
                             all(x > 0) && all(x < 1) },
         valid="a single numerical value, strictly between 0 and 1"
       ),
       scalable = list(
         ## whether certain calculations in ppm should be scalable
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1},
         valid="a single logical value"
         ),
       selfcrossing.psp.useCall=list(
         ## whether to use new code in selfcrossing.psp
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       terse = list(
         ## Level of terseness in printed output (higher => more terse)
         default=0,
         check=function(x) { length(x) == 1 && all(x %in% 0:4) },
         valid="an integer between 0 and 4"
       ),
       transparent=list(
         ## whether to allow transparent colours in default colour maps
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       units.paren=list(
         default="(",
         check=function(x) {
           is.character(x) && (length(x) == 1) &&
             all(x %in% c("(", "[", "{", ""))
         },
         valid="one of the strings '(', '[', '{' or '' "
       ),
       use.Krect=list(
         ## whether to use function Krect in Kest(X) when window is rectangle
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       Cwhist=list(
         ## whether to use C code for whist
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       Cbdrymask=list(
         ## whether to use C code for bdry.mask
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       kppm.canonical=list(
         ## whether to use 'canonical' parameters in kppm
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       kppm.adjusted=list(
         ## experimental
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       check.rpanel.loaded=list(
         # internal debugging
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       check.RandomFields.loaded=list(
         # this is working OK so no need to check unless debugging
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       check.RandomFieldsUtils.loaded=list(
         # this is working OK so no need to check unless debugging
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       Clinequad = list(
         # use C code for 'linequad'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       Ccountends = list(
         # use C code for 'countends'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       Clinearradius = list(
         # use C code for 'boundingradius.linnet'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       Cnndistlpp = list(
         # use C code for 'nndist.lpp'/'nnwhich.lpp'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       Cnncrosslpp = list(
         # use C code for 'nncross.lpp'
         default=TRUE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       ),
       developer = list(
         # general purpose; user is a developer; use experimental code, etc
         default=FALSE,
         check=function(x) { is.logical(x) && length(x) == 1 },
         valid="a single logical value"
       )
    )
# end of options list

reset.spatstat.options <- function() {
  Spatstat.Options <- lapply(.Spat.Stat.Opt.Table, getElement, name="default")
  putSpatstatVariable("Spatstat.Options", Spatstat.Options)
  invisible(Spatstat.Options)  
}

reset.spatstat.options()

spatstat.options <- local({

  spatstat.options <- function (...) {
    Spatstat.Options <- getSpatstatVariable("Spatstat.Options")
    called <- list(...)    

    if(length(called) == 0) {
      # return all options, except superseded ones
      allofem <- .Spat.Stat.Opt.Table[names(Spatstat.Options)]
      retain <- sapply(lapply(allofem, getElement, name="superseded"), is.null)
      return(Spatstat.Options[retain])
    }
    
    if(is.null(names(called)) && length(called)==1) {
      # spatstat.options(x) 
      x <- called[[1]]
      if(is.null(x))
        return(Spatstat.Options)  # spatstat.options(NULL)
      if(is.list(x))
        called <- x 
    }
    
    if(is.null(names(called))) {
        # spatstat.options("par1", "par2", ...)
	ischar <- unlist(lapply(called, is.character))
	if(all(ischar)) {
          choices <- unlist(called)
          ok <- choices %in% names(Spatstat.Options)
          if(!all(ok))
            stop(paste("Unrecognised option(s):", called[!ok]))
          if(length(called) == 1)
            return(Spatstat.Options[[choices]])
          else
            return(Spatstat.Options[choices])
	} else {
	   wrong <- called[!ischar]
	   offending <- sapply(wrong, ShortDeparse)
	   offending <- paste(offending, collapse=",")
           stop(paste("Unrecognised mode of argument(s) [",
		offending,
	   "]: should be character string or name=value pair"))
    	}
    }
    ## spatstat.options(name=value, name2=value2,...)
    assignto <- names(called)
    if (is.null(assignto) || !all(nzchar(assignto)))
        stop("options must all be identified by name=value")
    recog <- assignto %in% names(.Spat.Stat.Opt.Table)
    if(!all(recog))
	stop(paste("Unrecognised option(s):", assignto[!recog]))
    ## validate new values
    for(i in seq_along(assignto)) {
      nama <- assignto[i]
      valo <- called[[i]]
      entry <- .Spat.Stat.Opt.Table[[nama]]
      ok <- entry$check(valo)
      if(!ok)
        stop(paste("Parameter", dQuote(nama), "should be",
                   entry$valid))
    }
    ## reassign
    changed <- Spatstat.Options[assignto]
    Spatstat.Options[assignto] <- called
    putSpatstatVariable("Spatstat.Options", Spatstat.Options)
  
    ## return 
    invisible(changed)
  }

  ShortDeparse <- function(x) {
    y <- x
    dont.complain.about(y)
    short.deparse(substitute(y))
  }
    
  spatstat.options
})

#
#	centroid.S	Centroid of a window
#			and related operations
#
#	$Revision: 1.6 $	$Date: 2014/11/10 08:20:59 $
#
# Function names (followed by "xypolygon" or "owin")
#	
#	intX            integral of x dx dy
#	intY            integral of y dx dy
#	meanX           mean of x dx dy
#	meanY           mean of y dx dy
#       centroid        (meanX, meanY)
#		
#-------------------------------------

intX.xypolygon <- function(polly) {
  #
  # polly: list(x,y) vertices of a single polygon (n joins to 1)
  #
  verify.xypolygon(polly)
  
  x <- polly$x
  y <- polly$y
  
#  nedges <- length(x)   # sic
  
  # place x axis below polygon
  y <- y - min(y) 

  # join vertex n to vertex 1
  xr <- c(x, x[1L])
  yr <- c(y, y[1L])

  # slope
  dx <- diff(xr)
  dy <- diff(yr)
  slope <- ifelseAX(dx == 0, 0, dy/dx)

  # integrate
  integrals <- x * y * dx + (y + slope * x) * (dx^2)/2 + slope * (dx^3)/3

  -sum(integrals)
}
		
intX.owin <- function(w) {
	verifyclass(w, "owin")
        switch(w$type,
               rectangle = {
		width  <- abs(diff(w$xrange))
		height <- abs(diff(w$yrange))
		answer <- width * height * mean(w$xrange)
               },
               polygonal = {
                 answer <- sum(unlist(lapply(w$bdry, intX.xypolygon)))
               },
               mask = {
                 pixelarea <- abs(w$xstep * w$ystep)
		 x <- rasterx.mask(w, drop=TRUE)
                 answer <- (pixelarea * length(x)) * mean(x)
               },
               stop("Unrecognised window type")
        )
        return(answer)
}

meanX.owin <- function(w) {
	verifyclass(w, "owin")
        switch(w$type,
               rectangle = {
		answer <- mean(w$xrange)
               },
               polygonal = {
	         area <- sum(unlist(lapply(w$bdry, Area.xypolygon)))
                 integrated <- sum(unlist(lapply(w$bdry, intX.xypolygon)))
		 answer <- integrated/area
               },
               mask = {
		 x <- rasterx.mask(w, drop=TRUE)
                 answer <- mean(x)
               },
               stop("Unrecognised window type")
        )
        return(answer)
}

intY.xypolygon <- function(polly) {
  #
  # polly: list(x,y) vertices of a single polygon (n joins to 1)
  #
  verify.xypolygon(polly)
  
  x <- polly$x
  y <- polly$y
  
#  nedges <- length(x)   # sic
  
  # place x axis below polygon
  yadjust <- min(y)
  y <- y - yadjust 

  # join vertex n to vertex 1
  xr <- c(x, x[1L])
  yr <- c(y, y[1L])

  # slope
  dx <- diff(xr)
  dy <- diff(yr)
  slope <- ifelseAX(dx == 0, 0, dy/dx)

  # integrate
  integrals <- (1/2) * (dx * y^2 + slope * y * dx^2 + slope^2 * dx^3/3)
  total <- sum(integrals) - yadjust * Area.xypolygon(polly)

  # change sign to adhere to anticlockwise convention
  -total
}
		
intY.owin <- function(w) {
	verifyclass(w, "owin")
        switch(w$type,
               rectangle = {
		width  <- abs(diff(w$xrange))
		height <- abs(diff(w$yrange))
		answer <- width * height * mean(w$yrange)
               },
               polygonal = {
                 answer <- sum(unlist(lapply(w$bdry, intY.xypolygon)))
               },
               mask = {
                 pixelarea <- abs(w$xstep * w$ystep)
		 y <- rastery.mask(w, drop=TRUE)
                 answer <- (pixelarea * length(y)) * mean(y)
               },
               stop("Unrecognised window type")
        )
        return(answer)
}

meanY.owin <- function(w) {
	verifyclass(w, "owin")
        switch(w$type,
               rectangle = {
		answer <- mean(w$yrange)
               },
               polygonal = {
	         area <- sum(unlist(lapply(w$bdry, Area.xypolygon)))
                 integrated <- sum(unlist(lapply(w$bdry, intY.xypolygon)))
		 answer <- integrated/area
               },
               mask = {
		 y <- rastery.mask(w, drop=TRUE)
                 answer <- mean(y)
               },
               stop("Unrecognised window type")
        )
        return(answer)
}

centroid.owin <- function(w, as.ppp = FALSE) {
	w <- as.owin(w)
        out <- list(x=meanX.owin(w), y=meanY.owin(w))
        if(as.ppp){
            if(!inside.owin(out$x, out$y, w))
                w <- as.rectangle(w)
            out <- as.ppp(out, W=w)
        }
	return(out)
}

#
#  pickoption.R
#
#  $Revision: 1.7 $  $Date: 2019/06/30 07:49:10 $
#

pickoption <- function(what="option", key, keymap, ...,
                       exact=FALSE, list.on.err=TRUE, die=TRUE, multi=FALSE,
                       allow.all=TRUE)
{
  keyname <- short.deparse(substitute(key))

  if(!is.character(key))
    stop(paste(keyname, "must be a character string",
               if(multi) "or strings" else NULL))
  if(length(key) == 0)
    stop(paste("Argument", sQuote(keyname), "has length zero"))
  key <- unique(key)
  if(!multi && length(key) > 1)
    stop(paste("Must specify only one", what, sQuote(keyname)))
  allow.all <- allow.all && multi

  id <-
    if(allow.all && "all" %in% key) {
      seq_along(keymap)
    } else if(exact) {
      match(key, names(keymap), nomatch=NA)
    } else {
      pmatch(key, names(keymap), nomatch=NA)
    }
  
  if(any(nbg <- is.na(id))) {
    # no match
    whinge <- paste("unrecognised", what,
                    paste(dQuote(key[nbg]), collapse=", "),
                    "in argument", sQuote(keyname))
    if(list.on.err) {
      cat(paste(whinge, "\n", "Options are:"),
          paste(dQuote(names(keymap)), collapse=","), "\n")
    }
    if(die) 
      stop(whinge, call.=FALSE)
    else
      return(NULL)
  }

  key <- unique(keymap[id])
  names(key) <- NULL
  return(key)
}

##
## hypersub.R
##
##
##  subset operations for hyperframes
##
##  $Revision: 1.28 $    $Date: 2019/08/17 13:48:03 $
##

"[.hyperframe" <- function(x, i, j, drop=FALSE, strip=drop, ...) {
  x <- unclass(x)
  if(!missing(i)) {
    y <- x
    y$df     <- x$df[i, , drop=FALSE]
    y$ncases <- nrow(y$df)
    y$hypercolumns <- lapply(x$hypercolumns, "[", i=i)
    x <- y
  }
  if(!missing(j)) {
    y <- x
    patsy <- seq_len(y$nvars)
    names(patsy) <- y$vname
    jj <- patsy[j]
    names(jj) <- NULL
    y$nvars <- length(jj)
    y$vname <- vname <- x$vname[jj]
    y$vtype <- vtype <- x$vtype[jj]
    y$vclass <- x$vclass[jj]
    if(ncol(x$df) != 0) 
      y$df    <- x$df[ , vname[vtype == "dfcolumn"], drop=FALSE]
    y$hyperatoms <- x$hyperatoms[ vname[ vtype == "hyperatom" ]]
    y$hypercolumns <- x$hypercolumns[ vname [ vtype == "hypercolumn" ] ]
    x <- y
  }
  if(drop) {
    nrows <- x$ncases
    ncols <- x$nvars
    if(nrows == 1 && ncols == 1 && strip) {
      ## return a single object 
      y <- switch(as.character(x$vtype),
                  dfcolumn    = x$df[, , drop=TRUE],
                  hypercolumn = (x$hypercolumns[[1L]])[[1L]],
                  hyperatom   = x$hyperatoms[[1L]])
      return(y)
    } else if(nrows == 1) {
      ## return the row as a vector or a list
      if(strip && all(x$vtype == "dfcolumn"))
        return(x$df[ , , drop=TRUE])
      n <- x$nvars
      y <- vector(mode="list", length=n)
      names(y) <- nama <- x$vname
      for(i in seq_len(n)) {
        nami <- nama[i]
        y[[i]] <- switch(as.character(x$vtype[i]),
                         dfcolumn = x$df[ , nami, drop=TRUE],
                         hyperatom = x$hyperatoms[[nami]],
                         hypercolumn = (x$hypercolumns[[nami]])[[1L]]
                         )
      }
      return(as.solist(y, demote=TRUE))
    } else if(ncols == 1) {
      ## return a column as an 'anylist'/'solist' or a vector
      switch(as.character(x$vtype),
             dfcolumn = {
               return(x$df[, , drop=TRUE])
             },
             hypercolumn = {
               y <- as.solist(x$hypercolumns[[1L]], demote=TRUE)
               names(y) <- row.names(x$df)
               return(y)
             },
             hyperatom = {
               ## replicate it to make a hypercolumn
               ha <- x$hyperatoms[1L]
               names(ha) <- NULL
               hc <- rep.int(ha, x$ncases)
               hc <- as.solist(hc, demote=TRUE)
               names(hc) <- row.names(x$df)
               return(hc)
             }
           )
    }
  }
  class(x) <- c("hyperframe", class(x))
  return(x)
}

"$.hyperframe" <- function(x,name) {
  m <- match(name, unclass(x)$vname)
  if(is.na(m))
    return(NULL)
  return(x[, name, drop=TRUE, strip=FALSE])
}

"$<-.hyperframe" <- function(x, name, value) {
  y <- as.list(x)
  if(is.hyperframe(value)) {
    if(ncol(value) == 1) {
      y[name] <- as.list(value)
    } else {
      y <- insertinlist(y, name, as.list(value))
    }
  } else {
    dfcol <- is.atomic(value) && (is.vector(value) || is.factor(value))
    if(!dfcol && !is.null(value))
      value <- as.list(value)
    y[[name]] <- value
  }
  z <- do.call(hyperframe, append(y, list(row.names=row.names(x),
                                            stringsAsFactors=FALSE)))
  return(z)
}

"[<-.hyperframe" <- 
function (x, i, j, value)
{
  sumry <- summary(x)
  colnam <- sumry$col.names
  dimx <- sumry$dim
  igiven <- !missing(i)
  jgiven <- !missing(j)
  if(!igiven) i <- seq_len(dimx[1L])
  if(!jgiven) j <- seq_len(dimx[2L])
  singlerow    <- ((is.integer(i) && length(i) == 1 && i > 0)
                   || (is.character(i) && length(i) == 1)
                   || (is.logical(i) && sum(i) == 1))
  singlecolumn <- ((is.integer(j) && length(j) == 1 && j > 0)
                   || (is.character(j) && length(j) == 1)
                   || (is.logical(j) && sum(j) == 1))
  
  if(!igiven && jgiven) {
    # x[, j] <- value
    if(singlecolumn) {
      # expecting single hypercolumn
      if(is.logical(j)) j <- names(x)[j]
      y <- get("$<-.hyperframe")(x, j, value)
    } else {
      # expecting hyperframe 
      xlist <- as.list(x)
      xlist[j] <- as.list(as.hyperframe(value))
      # the above construction accepts all indices including extra entries
      y <- do.call(hyperframe, append(xlist,
                                        list(row.names=row.names(x))))
    }
  } else {
    ## x[, ] <- value or x[i, ] <- value or x[i,j] <- value 
    ## convert indices to positive integers
    rowseq <- seq_len(dimx[1L])
    colseq <- seq_len(dimx[2L])
    names(rowseq) <- row.names(x)
    names(colseq) <- colnam
    I <- rowseq[i]
    J <- colseq[j]
    ## convert to lists 
    xlist <- as.list(x)
    if(singlerow && singlecolumn) {
      vlist <- list(anylist(value))
      nrowV <- ncolV <- 1
    } else {
      hv <- if(is.hyperframe(value)) value else
            as.hyperframe(as.solist(value, demote=TRUE))
      vlist <- as.list(hv)
      nrowV <- dim(hv)[1L]
      ncolV <- dim(hv)[2L]
    }
    if(nrowV != length(I)) {
      if(nrowV == 1) {
        ## replicate
        vlist <- lapply(vlist, rep, times=nrowV)
      } else stop(paste("Replacement value has wrong number of rows:",
                        nrowV, "should be", length(I)),
                  call.=FALSE)
    }
    if(ncolV != length(J)) {
      if(ncolV == 1) {
        ## replicate
        vlist <- rep(vlist, times=ncolV)
      } else stop(paste("Replacement value has wrong number of columns:",
                        ncolV, "should be", length(J)),
                  call.=FALSE)
    }
    ## replace entries
    for(k in seq_along(J)) {
      jj <- J[k]
      xlist[[jj]][I] <- vlist[[k]]
    }
    ## put back together
    y <- do.call(hyperframe, append(xlist,
                                      list(row.names=row.names(x))))
  } 
  return(y)
}


split.hyperframe <- local({

  split.hyperframe <- function(x, f, drop=FALSE, ...) {
    y <- data.frame(id=seq_len(nrow(x)))
    z <- split(y, f, drop=drop)
    z <- lapply(z, getElement, name="id")
    out <- lapply(z, indexi, x=x)
    return(out)
  }

  indexi <- function(i, x) x[i,]
  
  split.hyperframe
})


"split<-.hyperframe" <- function(x, f, drop=FALSE, ..., value) {
  ix <- split(seq_len(nrow(x)), f, drop = drop, ...)
  n <- length(value)
  j <- 0
  for (i in ix) {
    j <- j%%n + 1L
    x[i, ] <- value[[j]]
  }
  x
}
  

#'  perspective plot of 3D 
#'
#'  $Revision: 1.7 $ $Date: 2019/10/22 07:53:14 $
#'


project3Dhom <- local({

  check3dvector <- function(x) {
    xname <- deparse(substitute(x))
    if(!(is.numeric(x) && length(x) == 3))
      stop(paste(xname, "should be a numeric vector of length 3"),
           call.=FALSE)
    return(NULL)
  }

  normalise <- function(x) {
    len <- sqrt(sum(x^2))
    if(len == 0) stop("Attempted to normalise a vector of length 0")
    return(x/len)
  }

  innerprod <- function(a, b) sum(a*b)

  crossprod <- function(u, v) {
    c(u[2] * v[3] - u[3] * v[2],
      -(u[1] * v[3] - u[3] * v[1]),
      u[1] * v[2] - u[2] * v[1])
  }

  project3Dhom <- function(xyz, eye=c(0,-3,1), org=c(0,0,0), vert=c(0,0,1)) {
    ## xyz: data to be projected (matrix n * 3)
    stopifnot(is.matrix(xyz) && ncol(xyz) == 3)
    ## eye: eye position (x,y,z)
    check3dvector(eye)
    ## org: origin (x,y,z) becomes middle of projection plane
    check3dvector(org)
    ## vert: unit vector in direction to become the 'vertical'
    if(!missing(vert)) {
      check3dvector(vert)
      vert <- normalise(vert)
    }
    ## vector pointing into screen
    vin <- normalise(org - eye)
    ## projection of vertical onto screen
    vup <- normalise(vert - innerprod(vert, vin) * vin)
    ## horizontal axis in screen
    vhoriz <- crossprod(vin, vup)
    ##
#    dbg <- FALSE
#    if(dbg) {
#      cat("vin=")
#      print(vin)
#      cat("vup=")
#      print(vup)
#      cat("vhoriz=")
#      print(vhoriz)
#    }
    ## homogeneous coordinates
    hom <- t(t(xyz) - eye) %*% cbind(vhoriz, vup, vin)
    colnames(hom) <- c("x", "y", "d")
    return(hom)
  }

  project3Dhom
})

plot3Dpoints <- local({

  plot3Dpoints <- function(xyz, eye=c(2,-3,2), org=c(0,0,0),
                           ...,
                           type=c("p", "n", "h"),
                           xlim=c(0,1), ylim=c(0,1), zlim=c(0,1),
                           add=FALSE, box=TRUE, 
                           main, cex=par('cex'), 
                           box.back=list(col="pink"),
                           box.front=list(col="blue", lwd=2)
                           ) {
    if(missing(main)) main <- short.deparse(substitute(xyz))
    type <- match.arg(type)
    #'
    if(is.null(box.back) || (is.logical(box.back) && box.back))
      box.back <- list(col="pink")
    if(is.null(box.front) || (is.logical(box.front) && box.front))
      box.front <- list(col="blue", lwd=2)
    stopifnot(is.list(box.back) || is.logical(box.back))
    stopifnot(is.list(box.front) || is.logical(box.front))
    #'
    stopifnot(is.matrix(xyz) && ncol(xyz) == 3)
    if(nrow(xyz) > 0) {
      if(missing(xlim)) xlim <- range(pretty(xyz[,1]))
      if(missing(ylim)) ylim <- range(pretty(xyz[,2]))
      if(missing(zlim)) zlim <- range(pretty(xyz[,3]))
      if(missing(org)) org <- c(mean(xlim), mean(ylim), mean(zlim))
    }
    if(!add) {
      #' initialise plot
      bb <- plot3Dbox(xlim, ylim, zlim, eye=eye, org=org, do.plot=FALSE)
      plot(bb$xlim, bb$ylim, axes=FALSE, asp=1, type="n",
           xlab="", ylab="", main=main)
    }
    if(is.list(box.back)) {
      #' plot rear of box
      do.call(plot3DboxPart,
              resolve.defaults(list(xlim=xlim,
                                    ylim=ylim,
                                    zlim=zlim,
                                    eye=eye, org=org,
                                    part="back"),
                               box.back,
                               list(...)))
    }
    if(type != "n") {
      #' plot points
      uv <- project3Dhom(xyz, eye=eye, org=org)
      uv <- as.data.frame(uv)
      dord <- order(uv$d, decreasing=TRUE)
      uv <- uv[dord, , drop=FALSE]
      #' capture graphics arguments which might be vectors
      grarg <- list(..., cex=cex)
      grarg <- grarg[names(grarg) %in% parsAll]
      if(any(lengths(grarg) > 1L)) {
        grarg <- as.data.frame(grarg, stringsAsFactors=FALSE)
        grarg <- grarg[dord, , drop=FALSE]
        grarg <- as.list(grarg)
      }
      #' draw segments
      if(type == "h") {
        xy0 <- cbind(xyz[,1:2,drop=FALSE], zlim[1])
        uv0 <- as.data.frame(project3Dhom(xy0, eye=eye, org=org))
        uv0 <- uv0[dord, , drop=FALSE]
        segargs <- grarg[names(grarg) %in% parsSegments]
        do.call(segments,
                append(list(x0=with(uv0, x/d),
                            y0=with(uv0, y/d),
                            x1=with(uv,  x/d),
                            y1=with(uv,  y/d)),
                       segargs))
      }
      #' draw points
      ptargs <- grarg[names(grarg) %in% parsPoints]
      ptargs$cex <- ptargs$cex * with(uv, min(d)/d)
      do.call(points,
              c(list(x=with(uv, x/d),
                     y=with(uv, y/d)),
                ptargs))
    }
    if(is.list(box.front)) 
      do.call(plot3DboxPart,
              resolve.defaults(list(xlim=xlim,
                                    ylim=ylim,
                                    zlim=zlim,
                                    eye=eye, org=org,
                                    part="front"),
                               box.front,
                               list(...)))
    return(invisible(NULL))
  }

  vertexind <- data.frame(i=rep(1:2,4),
                          j=rep(rep(1:2,each=2),2),
                          k=rep(1:2, each=4))

  edgepairs <- data.frame(from=c(1, 1, 2, 3, 1, 2, 5, 3, 5, 4, 6, 7),
                          to = c(2, 3, 4, 4, 5, 6, 6, 7, 7, 8, 8, 8))
  
  vertexfrom <- vertexind[edgepairs$from,]
  vertexto   <- vertexind[edgepairs$to,]

  parsPoints <- c("cex", "col", "fg", "bg", "pch", "lwd") 
  parsSegments <- c("col", "lwd", "lty")
  parsAll <- union(parsPoints, parsSegments)
  
  hamming <- function(a, b) sum(abs(a-b))

  ## determine projected positions of box vertices
  ## and optionally plot the box
  plot3Dbox <- function(xlim=c(0,1), ylim=xlim, zlim=ylim,
                        eye=c(0,-3,1), org=c(0,0,0),
                        do.plot=TRUE) {
    fromxyz <- with(vertexfrom, cbind(xlim[i], ylim[j], zlim[k]))
    toxyz   <- with(vertexto,   cbind(xlim[i], ylim[j], zlim[k]))
    fromuv <-  project3Dhom(fromxyz, eye=eye, org=org)
    touv <-  project3Dhom(toxyz, eye=eye, org=org)
    xfrom <- fromuv[,1]/fromuv[,3]
    xto   <- touv[,1]/touv[,3]
    yfrom <- fromuv[,2]/fromuv[,3]
    yto   <- touv[,2]/touv[,3]
    if(do.plot) 
      segments(xfrom, yfrom, xto, yto)
    return(invisible(list(xlim=range(xfrom, xto), ylim=range(yfrom, yto))))
  }

  ## plot either back or front of box
  plot3DboxPart <- function(xlim=c(0,1), ylim=xlim, zlim=ylim,
                            eye=c(0,-3,1), org=c(0,0,0),
                            part=c("front", "back"), ...) {
    part <- match.arg(part)
    boxvert <- with(vertexind, cbind(xlim[i], ylim[j], zlim[k]))
    pvert <- project3Dhom(boxvert, eye=eye, org=org)
    xyvert <- pvert[,c("x","y")]/pvert[,"d"]
    ## find vertex which is furthest away
    nback <- which.max(pvert[,"d"])
    nearback <- with(edgepairs, (from==nback) | (to==nback))
    ind <- if(part == "back") nearback else !nearback
    ## draw lines
    with(edgepairs[ind,],
         do.call.matched(segments,
                         list(x0=xyvert[from, 1],
                              y0=xyvert[from, 2],
                              x1=xyvert[to,   1],
                              y1=xyvert[to,   2],
                              ...)))
  }

  plot3Dpoints
})

#
# split.ppx.R
#
# $Revision: 1.6 $ $Date: 2018/09/07 05:49:15 $
#
# split.ppx etc
#
#########################################

split.ppx <- function(x, f = marks(x), drop=FALSE, un=NULL, ...) {
  stopifnot(inherits(x, "ppx"))
  mf <- markformat(x)
  if(is.null(un))
    un <- missing(f) && !(mf %in% c("dataframe", "hyperframe"))

  if(missing(f)) {
    # f defaults to marks of x
    switch(mf,
           none={
             stop("f is missing and there are no marks")
           },
           vector={
             if(!is.multitype(x)) 
               stop("f is missing and the pattern is not multitype")
             f <- fsplit <- marks(x)
           },
           hyperframe=,
           dataframe={
             f <- fsplit <- firstfactor(marks(x))
             if(is.null(f))
               stop("Marks do not include a factor")
           })
    splittype <- "factor"
  } else{
    # f was given
    fsplit <- f
    if(is.factor(f)) {
      splittype <- "factor"
    } else if(is.logical(f)) {
      splittype <- "factor"
      f <- factor(f)
    } else if(is.character(f) && length(f) == 1) {
      # f is the name of a column of marks
      marx <- marks(x)
      if((is.data.frame(marx) || is.hyperframe(marx))
         && (f %in% names(marx))) {
        fsplit <- f <- as.factor(marx[ ,f,drop=TRUE])
      } else
        stop(paste("The name", sQuote(f), "does not match any column of marks"))
      splittype <- "factor"
    } else 
      stop(paste("f must be",
                 "a factor,",
                 "or the name of a column of marks"))
    if(length(f) != npoints(x))
      stop("length(f) must equal the number of points in x")
  }

  # At this point
  # 'f' is a factor that can be used to separate the points
  # 'fsplit' is the object (either a factor or a tessellation)
  # that determines the split (and can be "un-split")

  lev <- levels(f)
  if(drop) {
    # remove components that don't contain points
    retain <- (table(f) > 0)
    lev <- lev[retain]
    switch(splittype,
           factor = {
             # delete levels that don't occur
             fsplit <- factor(fsplit, levels=lev)
           },
           stop("Internal error: wrong format for fsplit"))
  }

  # split the data
  out <- list()
  for(l in lev) 
    out[[paste(l)]] <- x[!is.na(f) & (f == l)]
  
  if(un)
     out <- lapply(out, unmark)
  class(out) <- c("splitppx", "anylist", class(out))
  attr(out, "fsplit") <- fsplit
  return(out)
}

print.splitppx <- function(x, ...) {
  f <- attr(x, "fsplit")
  what <- if(is.factor(f)) "factor" else "unknown data"
  cat(paste("Multidimensional point pattern split by", what, "\n"))
  nam <- names(x)
  for(i in seq_along(x)) {
    cat(paste("\n", nam[i], ":\n", sep=""))
    print(x[[i]])
  }
  return(invisible(NULL))
}

summary.splitppx <- function(object, ...) {
  x <- lapply(object, summary, ...)
  class(x) <- "summary.splitppx"
  x
}

print.summary.splitppx <- function(x, ...) {
  class(x) <- "anylist"
  print(x)
  invisible(NULL)
}

"[.splitppx" <- function(x, ...) {
  f <- attr(x, "fsplit")
  # invoke list method on x
  class(x) <- "list"
  y <- x[...]
  # then make it a 'splitppx' object too
  class(y) <- c("splitppx", class(y))
  if(is.factor(f)) {
    lev <- levels(f)
    sublev <- lev[...]
    subf <- f[f %in% sublev]
    fsplit <- factor(subf, levels=lev)
  } else stop("Unknown splitting type")
  attr(y, "fsplit") <- fsplit
  y
}

"[<-.splitppx" <- function(x, ..., value) {
  if(!all(unlist(lapply(value, is.ppx))))
    stop("replacement value must be a list of point patterns (ppx)")
  f <- attr(x, "fsplit")
  # invoke list method
  class(x) <- "list"
  x[...] <- value
  # then make it a 'splitppx' object too
  class(x) <- c("splitppx", class(x))
  if(is.factor(f)) {
    lev <- levels(f)
    fsplit <- factor(rep.int(lev, unlist(lapply(x, npoints))), levels=lev)
  }
  attr(x, "fsplit") <- fsplit
  x
}
  

#'
#'   factors.R
#'
#'  Tools for manipulating factors and factor-valued things
#'
#'  $Revision: 1.4 $  $Date: 2016/04/25 02:34:40 $

relevel.im <- function(x, ref, ...) {
  if(x$type != "factor")
    stop("Only valid for factor-valued images")
  x[] <- relevel(x[], ref, ...)
  return(x)
}

relevel.ppp <- relevel.ppx <- function(x, ref, ...) {
  stopifnot(is.multitype(x))
  marks(x) <- relevel(marks(x), ref, ...)
  return(x)
}

mergeLevels <- function(.f, ...) {
  if(is.im(.f)) {
    aa <- mergeLevels(.f[], ...)
    .f[] <- aa
    return(.f)
  }
  if(is.multitype(.f)) {
    marks(.f) <- mergeLevels(marks(.f), ...)
    return(.f)
  }
  stopifnot(is.factor(.f))
  map <- list(...)
  n <- length(map)
  if(n == 0) return(.f)
  # mapping for 'other'
  if(any(isnul <- (lengths(map) == 0))) {
    if(sum(isnul) > 1)
      stop("At most one argument should be NULL or character(0)")
    otherlevels <- setdiff(levels(.f), unlist(map))
    map[[which(isnul)]] <- otherlevels
  }
  newlevels <- names(map)
  oldlevels <- levels(.f)
  mappedlevels <- unlist(map)
  if(sum(nzchar(newlevels)) != n)
    stop("Arguments must be in the form name=value")
  if(!all(mappedlevels %in% oldlevels))
    stop("Argument values must be levels of .f")
  ## construct mapping
  fullmap <- oldlevels
  for(i in seq_len(n)) {
    relevant <- oldlevels %in% map[[i]]
    fullmap[relevant] <- newlevels[i]
  }
  ## apply mapping
  newf <- factor(fullmap[.f], levels=unique(fullmap))
  return(newf)
}

levelsAsFactor <- function(x) {
  lev <- levels(x)
  if(is.null(lev)) return(NULL)
  return(factor(lev, levels=lev))
}

#'
#'	metricPdt.R
#'
#'     Metric distance transform of pixel mask
#'
#'	$Revision: 1.5 $	$Date: 2019/11/01 01:34:30 $

rectdistmap <- function(X, asp=1.0, npasses=1, verbose=FALSE) {
  w <- as.mask(X)
  check.1.real(asp)
  check.1.integer(npasses)
  stopifnot(asp > 0)
  #' ensure grid has suitable aspect ratio
  dx <- w$xstep
  dy <- w$ystep
  a <- dy/(asp*dx)
  if(verbose) 
    splat("grid aspect", signif(a, 3))
  refined <- (a > 1.2 || a < 0.8)
  if(refined) {
    flipped <- (a < 1)
    if(flipped) a <- 1/a
    n <- if(a > 10) 1 else if(a > 6) 2 else if(a > 4) 4 else 12
    an <- if(n > 1) round(a * n) else ceiling(a)
    k <- c(an, n)/greatest.common.divisor(an, n)
    if(flipped)
      k <- rev(k)
    woriginal <- w
    w <- as.owin(w, dimyx=k * dim(w))
    if(verbose) {
      splat("Grid expansion", k[1], "x", k[2])
      splat("Adjusted grid aspect", (a * k[2])/k[1])
    }
  }
  #'
  nr <- w$dim[1L]
  nc <- w$dim[2L]
  #' input image will be padded out with a margin of width 2 on all sides
  mr <- mc <- 2L
  #' full dimensions of padded image
  Nnr <- nr + 2 * mr
  Nnc <- nc + 2 * mc
  N <- Nnr * Nnc
  #' output image (subset): rows & columns (R indexing)
  rmin <- mr + 1L
  rmax <- Nnr - mr
  cmin <- mc + 1L
  cmax <- Nnc - mc
  #' do padding
  x <- matrix(FALSE, nrow=Nnr, ncol=Nnc)
  x[rmin:rmax, cmin:cmax] <- w$m
  #' compute distmap
  res <- .C("mdtPOrect",
            as.double(w$xrange[1L]),
            as.double(w$yrange[1L]),
            as.double(w$xrange[2L]),
            as.double(w$yrange[2L]),
            nr = as.integer(nr),
            nc = as.integer(nc),
            mr = as.integer(mr),
            mc = as.integer(mc),
            inp = as.integer(t(x)),
            asp = as.double(asp),
            npasses = as.integer(npasses),
            distances = as.double (double(N)),
            rows      = as.integer(integer(N)),
            cols      = as.integer(integer(N)),
            PACKAGE = "spatstat")
  dist <- matrix(res$distances,
                 ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
  result <- as.im(dist, w)
  if(refined) result <- as.im(result, W=woriginal) 
#  rows <- matrix(res$rows,
#                 ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
#  cols <- matrix(res$cols,
#                 ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
  # convert from C to R indexing
#  rows <- rows + 1L - as.integer(mr)
#  cols <- cols + 1L - as.integer(mc)
#  return(list(d=dist,row=rows,col=cols,b=bdist, w=w))
  edge <- TRUE
  if(edge) {
    #' calculate distance transform to boundary
    y <- x
    y[] <- TRUE
    y[rmin:rmax, cmin:cmax] <- FALSE
    y[rmin, ] <- TRUE
    y[rmax, ] <- TRUE
    y[, cmin] <- TRUE
    y[, cmax] <- TRUE
    #' compute distmap
    bres <- .C("mdtPOrect",
               as.double(w$xrange[1L]),
               as.double(w$yrange[1L]),
               as.double(w$xrange[2L]),
               as.double(w$yrange[2L]),
               nr = as.integer(nr),
               nc = as.integer(nc),
               mr = as.integer(mr),
               mc = as.integer(mc),
               inp = as.integer(t(y)),
               asp = as.double(asp),
               npasses = as.integer(npasses),
               distances = as.double (double(N)),
               rows      = as.integer(integer(N)),
               cols      = as.integer(integer(N)),
               PACKAGE = "spatstat")
    bdist <- matrix(bres$distances,
                    ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
    bdist <- as.im(bdist, w)
    if(refined) bdist <- as.im(bdist, W=woriginal) 
    attr(result, "bdist") <- bdist
  }
  return(result)
}

rectcontact <- function(X, ..., asp=1.0, npasses=4,
                        eps=NULL, r=NULL, breaks=NULL,
                        correction=c("rs", "km")) {
  verifyclass(X, "im")
  rorbgiven <- !is.null(r) || !is.null(breaks)
  checkspacing <- !isFALSE(list(...)$checkspacing)
  testme       <- isTRUE(list(...)$testme)
  
  check.1.real(asp)
  stopifnot(asp > 0)
  
  if(X$type != "logical")
    stop("X should be a logical-valued image")

  if(!missing(eps))
    X <- as.im(X, eps=eps)
  
  W <- as.mask(X)      # the region that is defined
  Y <- solutionset(X)  # the region that is TRUE
  fullframe  <- all(W$m)
  emptyframe <- !any(W$m)
  
  ## histogram breakpoints
  rmaxdefault <- rmax.rule("F", W)
  breaks <- handle.r.b.args(r, breaks, W, eps, rmaxdefault=rmaxdefault)
  rvals <- breaks$r
  rmax  <- breaks$max

  if(testme || (rorbgiven && checkspacing))
    check.finespacing(rvals,
                      if(is.null(eps)) NULL else eps/4,
                      W,
                      rmaxdefault=if(rorbgiven) NULL else rmaxdefault,
                      action="fatal",
                      rname="r", 
                      context="in rectcontact(X, r)")
                                
  correction <- pickoption("correction", correction,
                           c(border="rs",
                             rs="rs",
                             KM="km",
                             km="km",
                             Kaplan="km",
                             best="km"),
                           multi=TRUE)
  
  ##  compute distances and censoring distances
  if(!emptyframe) {
    dist <- rectdistmap(Y, asp, npasses=npasses)
    if(fullframe) {
      bdry <- attr(dist, "bdist")
    } else {
      bdry <- rectdistmap(complement.owin(W), asp, npasses=npasses)
    }
    #' extract corresponding values
    dist <- dist[W, drop=TRUE, rescue=FALSE]
    bdry <- bdry[W, drop=TRUE, rescue=FALSE]
    ## censoring indicators
    d <- (dist <= bdry)
    ##  observed distances
    o <- pmin.int(dist, bdry)
  }

  ## calculate Kaplan-Meier and/or border corrected (Reduced Sample) estimators
  want.rs <- "rs" %in% correction
  want.km <- "km" %in% correction
  selection <- c(want.rs, want.km)
  tags <- c("rs", "km")[selection]
  labels <- c("hat(%s)[bord](r)", "hat(%s)[km](r)")[selection]
  descr <- c("border corrected estimate of %s",
             "Kaplan-Meier estimate of %s")[selection]
  if(emptyframe) {
    df <- as.data.frame(matrix(0, length(rvals), length(tags)))
    names(df) <- tags
  } else {
    df  <- km.rs.opt(o, bdry, d, breaks, KM=want.km, RS=want.rs)
    df <- as.data.frame(df[tags])
  }
  ## create fv object
  df <- cbind(data.frame(r=rvals), df)
  Z <- fv(df, "r", quote(H(r)),
          if(want.km) "km" else "rs",
          . ~ r,
          c(0,rmax),
          c("r", labels),
          c("distance argument r", descr),
          fname="H")

  fvnames(Z, ".") <- rev(fvnames(Z, "."))
  attr(Z, "alim") <- with(Z, range(.x[is.finite(.y) & .y <= 0.95]))
  attr(Z, "conserve") <- list(checkspacing=FALSE)
  return(Z)
}

	
  

#'
#'     newformula.R
#'
#'    $Revision: 1.2 $ $Date: 2017/09/29 09:08:51 $
#' 
#'   Update formula and expand polynomial

newformula <- function(old, change, eold, enew) {
  old <- if(is.null(old)) ~1 else eval(old, eold)
  change <- if(is.null(change)) ~1 else eval(change, enew)
  old <- as.formula(old, env=eold)
  change <- as.formula(change, env=enew)
  if(spatstat.options("expand.polynom")) {
    old <- expand.polynom(old)
    change <- expand.polynom(change)
  }
  answer <- update.formula(old, change)
  return(answer)
}

#
# Functions for extracting and setting the name of the unit of length
#
#   $Revision: 1.29 $   $Date: 2019/02/20 03:34:50 $
#
#

unitname <- function(x) {
  UseMethod("unitname")
}

unitname.owin <- function(x) {
  u <- as.unitname(x$units)
  return(u)
}

unitname.ppp <- function(x) {
  u <- as.unitname(x$window$units)
  return(u)
}

unitname.im <- function(x) {
  u <- as.unitname(x$units)
  return(u)
}

unitname.default <- function(x) {
  return(as.unitname(attr(x, "units")))
}

"unitname<-" <- function(x, value) {
  UseMethod("unitname<-")
}

"unitname<-.owin" <- function(x, value) {
  x$units <- as.unitname(value)
  return(x)
}

"unitname<-.ppp" <- function(x, value) {
  w <- x$window
  unitname(w) <- value
  x$window <- w
  return(x)
}

"unitname<-.im" <- function(x, value) {
  x$units <- as.unitname(value)
  return(x)
}

"unitname<-.default" <- function(x, value) {
  if(is.null(x)) return(x)
  attr(x, "units") <- as.unitname(value)
  return(x)
}


###  class 'unitname'

makeunitname <- function(sing="unit", plur="units", mul = 1) {
  if(!is.character(sing))
    stop("In unit name, first entry should be a character string")
  if(!is.character(plur))
    stop("In unit name, second entry should be a character string")
  mul <- try(as.numeric(mul), silent=TRUE)
  if(inherits(mul, "try-error"))
    stop("In unit name, third entry should be a number")
  if(length(mul) != 1 || mul <= 0)
    stop("In unit name, third entry should be a single positive number")
  u <- list(singular=sing, plural=plur, multiplier=mul)
  if(mul != 1 && (sing=="unit" || plur=="units"))
    stop(paste("A multiplier is not allowed",
               "if the unit does not have a specific name"))
  class(u) <- "unitname"
  return(u)
}
  
as.unitname <- function(s) {
  if(inherits(s, "unitname")) return(s)
  s <- as.list(s)
  n <- length(s)
  if(n > 3)
    stop(paste("Unit name should be a character string,",
               "or a vector/list of 2 character strings,",
               "or a list(character, character, numeric)"))
  out <- switch(n+1,
                makeunitname(),
                makeunitname(s[[1]], s[[1]]),
                makeunitname(s[[1]], s[[2]]),
                makeunitname(s[[1]], s[[2]], s[[3]]))
  return(out)
}

print.unitname <- function(x, ...) {
  mul <- x$multiplier
  if(mul == 1)
    cat(paste(x$singular, "/", x$plural, "\n"))
  else 
    cat(paste(mul, x$plural, "\n"))
  return(invisible(NULL))
}

as.character.unitname <- function(x, ...) {
  mul <- x$multiplier
  return(if(mul == 1) x$plural else paste(mul, x$plural))
}

is.vanilla <- function(u) {
  u <- as.unitname(u)
  z <- (u$singular == "unit") && (u$multiplier == 1)
  return(z)
}

summary.unitname <- function(object, ...) {
  x <- object
  scaled <- (x$multiplier != 1)
  named  <- (x$singular != "unit")
  vanilla <- !named && !scaled
  out <-
    if(vanilla) {
      list(legend = NULL,
           axis   = NULL, 
           explain = NULL,
           singular = "unit",
           plural   = "units")
    } else if(named & !scaled) {
      list(legend = paste("Unit of length: 1", x$singular),
           axis   = paren(x$plural, type=spatstat.options('units.paren')),
           explain = NULL,
           singular = x$singular,
           plural   = x$plural)
    } else {
      expanded <- paste(x$multiplier, x$plural)
      expla <- paren(paste("one unit =", expanded),
                     type=spatstat.options('units.paren'))
      list(legend = paste("Unit of length:", expanded),
           axis   = expla, 
           explain  = expla,
           singular = "unit",
           plural   = "units")
    }
  out <- append(out, list(scaled  = scaled,
                          named   = named,
                          vanilla = vanilla))
  class(out) <- "summary.unitname"
  return(out)
}

print.summary.unitname <- function(x, ...) {
  if(x$vanilla)
    cat("Unit of length (unnamed)\n")
  else
    cat(paste(x$legend, "\n"))
  invisible(NULL)
}

compatible.unitname <- function(A, B, ..., coerce=TRUE) {
  A <- as.unitname(A)
  if(missing(B)) return(TRUE)
  B <- as.unitname(B)
  # check for null units
  Anull <- summary(A)$vanilla
  Bnull <- summary(B)$vanilla
  # `coerce' determines whether `vanilla' units are compatible with other units
  coerce <- as.logical(coerce)
  # 
  agree <- if(!Anull && !Bnull) isTRUE(all.equal(A,B)) else
           if(Anull && Bnull) TRUE else coerce 
  #
  if(!agree) return(FALSE)
  # A and B agree
  if(length(list(...)) == 0) return(TRUE)
  # recursion
  return(compatible.unitname(B, ...))
}

harmonize.unitname <-
harmonise.unitname <- function(..., coerce=TRUE, single=FALSE) {
  argh <- list(...)
  n <- length(argh)
  if(n == 0) return(NULL)
  u <- lapply(argh, as.unitname)
  if(n == 1) return(if(single) u[[1L]] else u)
  if(coerce) {
    #' vanilla units are compatible with another unit
    s <- lapply(u, summary)
    v <- sapply(s, getElement, name="vanilla")
    if(all(v))
      return(if(single) u[[1L]] else u)
    u <- u[!v]
  }
  z <- unique(u)
  if(length(z) > 1) stop("Unitnames are incompatible", call.=FALSE)
  if(single) return(z[[1]])
  z <- rep(z, n)
  names(z) <- names(argh)
  return(z)
}

# class 'numberwithunit':  numeric value(s) with unit of length

numberwithunit <- function(x, u) {
  u <- as.unitname(u)
  x <- as.numeric(x)
  unitname(x) <- u
  class(x) <- c(class(x), "numberwithunit")
  return(x)
}

"%unit%" <- function(x, u) {
  numberwithunit(x, u)
}

format.numberwithunit <- function(x, ..., collapse=" x ", modifier=NULL) {
  u <- summary(unitname(x))
  uname <- if(all(x == 1)) u$singular else u$plural
  y <- format(as.numeric(x), ...)
  z <- pasteN(paste(y, collapse=collapse), 
              modifier, uname, u$explain)
  return(z)
}

as.character.numberwithunit <- function(x, ...) {
  return(format(x))
}

print.numberwithunit <- function(x, ...) {
  cat(format(x, ...), fill=TRUE)
  return(invisible(NULL))
}


    

#
# marks.R
#
#   $Revision: 1.47 $   $Date: 2020/03/23 07:18:53 $
#
# stuff for handling marks
#
#

marks <- function(x, ...) {
  UseMethod("marks")
}

marks.default <- function(x, ...) { NULL }

# The 'dfok' switch is temporary
# while we convert the code to accept data frames of marks

marks.ppp <- function(x, ..., dfok=TRUE, drop=TRUE) {
  ma <- x$marks
  if((is.data.frame(ma) || is.matrix(ma))) {
    if(!dfok)
      stop("Sorry, not implemented when the marks are a data frame")
    if(drop && ncol(ma) == 1)
      ma <- ma[,1,drop=TRUE]
  }
  return(ma)
}

# ------------------------------------------------------------------

"marks<-" <- function(x, ..., value) {
  UseMethod("marks<-")
}

"marks<-.ppp" <- function(x, ..., dfok=TRUE, drop=TRUE, value) {
  np <- npoints(x)
  m <- value
  switch(markformat(m),
         none = {
           return(unmark(x))
         },
         vector = {
           # vector of marks
           if(length(m) == 1) m <- rep.int(m, np)
           else if(np == 0) m <- rep.int(m, 0) # ensures marked pattern obtained
           else if(length(m) != np) stop("number of points != number of marks")
           marx <- m
         },
         dataframe = {
           if(!dfok)
             stop("Sorry, data frames of marks are not yet implemented")
           m <- as.data.frame(m)
           # data frame of marks
           if(ncol(m) == 0) {
             # no mark variables
             marx <- NULL
           } else {
             # marks to be attached
             if(nrow(m) == np) {
               marx <- m
             } else {
               # lengths do not match
               if(nrow(m) == 1 || np == 0) {
               # replicate data frame
                 marx <- as.data.frame(lapply(as.list(m),
                                              function(x, k) { rep.int(x, k) },
                                              k=np))
               } else
               stop("number of rows of data frame != number of points")
             }
             # convert single-column data frame to vector?
             if(drop && ncol(marx) == 1)
               marx <- marx[,1,drop=TRUE]
           }
         },
         hyperframe = 
         stop("Hyperframes of marks are not supported in ppp objects; use ppx"),
         stop("Format of marks is not understood")
         )
  # attach/overwrite marks
  Y <- ppp(x$x,x$y,window=x$window,marks=marx, check=FALSE, drop=drop)
  return(Y)
}

"%mark%" <- setmarks <- function(x,value) {
  marks(x) <- value
  return(x)
}

# -------------------------------------------------

markformat <- function(x) {
  UseMethod("markformat")
}

markformat.ppp <- function(x) {
  mf <- x$markformat
  if(is.null(mf)) 
    mf <- markformat(marks(x))
  return(mf)
}

markformat.default <- function(x) {
  if(is.null(x)) return("none")
  if(is.null(dim(x))) {
    if(is.vector(x) || is.factor(x) || is.atomic(x)) return("vector")
    if(inherits(x, "POSIXt") || inherits(x, "Date")) return("vector")
  }
  if(is.data.frame(x) || is.matrix(x)) return("dataframe")
  if(is.hyperframe(x)) return("hyperframe")
  if(inherits(x, c("solist", "anylist", "listof"))) return("list")
  stop("Mark format not understood")
}

# ------------------------------------------------------------------

"is.marked" <-
function(X, ...) {
  UseMethod("is.marked")
}

"is.marked.ppp" <-
function(X, na.action="warn", ...) {
  marx <- marks(X, ...)
  if(is.null(marx))
    return(FALSE)
  if((length(marx) > 0) && anyNA(marx)) {
    gripe <- paste("some mark values are NA in the point pattern",
                   short.deparse(substitute(X)))
    switch(na.action,
           warn = warning(gripe, call.=FALSE),
           fatal = stop(gripe, call.=FALSE),
           ignore = {}
           )
  }
  return(TRUE)
}

"is.marked.default" <-
  function(...) { return(!is.null(marks(...))) }


# ------------------------------------------------------------------

is.multitype <- function(X, ...) {
  UseMethod("is.multitype")
}

is.multitype.default <- function(X, ...) {
  m <- marks(X)
  if(is.null(m))
    return(FALSE)
  if(!is.null(dim(m))) {
    # should have a single column
    if(dim(m)[2] != 1)
      return(FALSE)
    m <- m[,1,drop=TRUE]
  }
  return(is.factor(m))
}

is.multitype.ppp <- function(X, na.action="warn", ...) {
  marx <- marks(X, dfok=TRUE)
  if(is.null(marx))
    return(FALSE)
  if((is.data.frame(marx) || is.hyperframe(marx)) && ncol(marx) > 1)
    return(FALSE)
  if(!is.factor(marx))
    return(FALSE)
  if((length(marx) > 0) && anyNA(marx))
    switch(na.action,
           warn = {
             warning(paste("some mark values are NA in the point pattern",
                           short.deparse(substitute(X))))
           },
           fatal = {
             return(FALSE)
           },
           ignore = {}
           )
  return(TRUE)
}

# ------------------------------------------------------------------

unmark <- function(X) {
  UseMethod("unmark")
}

unmark.ppp <- function(X) {
  X$marks <- NULL
  X$markformat <- "none"
  return(X)
}

unmark.splitppp <- function(X) {
  Y <- lapply(X, unmark.ppp)
  class(Y) <- c("splitppp", class(Y))
  return(Y)
}

##### utility functions for subsetting & combining marks #########


marksubset <- function(x, index, format=NULL) {
  if(is.null(format)) format <- markformat(x)
  switch(format,
         none={return(NULL)},
         list=,
         vector={return(x[index])},
         hyperframe=,
         dataframe={return(x[index,,drop=FALSE])},
         stop("Internal error: unrecognised format of marks"))
}

"%msub%" <- marksubsetop <- function(x,i) { marksubset(x, i) }

"%mrep%" <- markreplicateop <- function(x,n) { 
  format <- markformat(x)
  switch(format,
         none={return(NULL)},
         list=,
         vector={ return(rep.int(x,n))},
         dataframe={
           return(as.data.frame(lapply(x, rep, times=n)))
         },
         hyperframe={
           xcols <- as.list(x)
           repxcols <- lapply(xcols, rep, times=n)
           return(do.call(hyperframe, repxcols))
         },
         stop("Internal error: unrecognised format of marks"))
}

"%mapp%" <- markappendop <- function(x,y) { 
  fx <- markformat(x)
  fy <- markformat(y)
  if(fx != fy) {
    ## vectors can be appended to single-column arrays
    xvec  <- (fx == "vector")
    yvec  <- (fy == "vector")
    xdf <- (fx %in% c("dataframe", "hyperframe")) && (ncol(x) == 1L)
    ydf <- (fy %in% c("dataframe", "hyperframe")) && (ncol(y) == 1L)
    if(xvec && ydf) {
      x <- as.data.frame(x)
      names(x) <- names(y)
      fx <- "dataframe"
    } else if(xdf && yvec) {
      y <- as.data.frame(y)
      names(y) <- names(x)
      fy <- "dataframe"
    } else stop("Attempted to concatenate marks that are not compatible")
  }
  switch(fx,
         none   = { return(NULL) },
         vector = {
           if(is.factor(x) || is.factor(y)) return(cat.factor(x,y))
           return(c(x,y))
         },
         hyperframe=,
         dataframe = { return(rbind(x,y)) },
         list = {
           z <- append(x,y)
           z <- as.solist(z, demote=TRUE)
           return(z)
         },
         stop("Internal error: unrecognised format of marks"))
}

markappend <- function(...) {
  # combine marks from any number of patterns
  marxlist <- list(...)
  # check on compatibility of marks
  mkfmt <- sapply(marxlist,markformat)
  if(length(ufm <- unique(mkfmt))>1)
    stop(paste("Cannot append marks of different formats:",
               commasep(sQuote(ufm))),
         call.=FALSE)
  mkfmt <- mkfmt[1]
  # combine the marks
  switch(mkfmt,
         none = {
           return(NULL)
         },
         vector = {
           marxlist <- lapply(marxlist,
                              function(x){as.data.frame.vector(x,nm="v1")})
           marx <- do.call(rbind, marxlist)[,1]
           return(marx)
         },
         hyperframe =,
         dataframe = {
           # check compatibility of data frames
           # (this is redundant but gives more helpful message)
           nama <- lapply(marxlist, names)
           dims <- lengths(nama)
           if(length(unique(dims)) != 1)
             stop("Data frames of marks have different column dimensions.")
           samenames <- unlist(lapply(nama,
                                      function(x,y) { identical(x,y) },
                                      y=nama[[1]]))
           if(!all(samenames))
             stop("Data frames of marks have different names.\n")
           marx <- do.call(rbind, marxlist)
           return(marx)
         },
         list = {
           marx <- do.call(c, marxlist)
           marx <- as.solist(marx, demote=TRUE) 
           return(marx)
         })
  stop("Unrecognised mark format")
}

markcbind <- function(...) {
  # cbind several columns of marks
  marxlist <- list(...)
  mkfmt <- unlist(lapply(marxlist, markformat))
  if(any(vacuous <- (mkfmt == "none"))) {
    marxlist <- marxlist[!vacuous]
    mkfmt    <- mkfmt[!vacuous]
  }
  if(any(isvec <- (mkfmt == "vector"))) {
    ## convert vectors to data frames with invented names
    for(i in which(isvec)) {
      mi <- as.data.frame(marxlist[i])
      colnames(mi) <- paste0("V", i)
      marxlist[[i]] <- mi
    }
    mkfmt[isvec] <- "dataframe"
  }
  if(all(mkfmt == "dataframe")) {
    ## result is a data frame
    marx <- do.call(data.frame, marxlist)
  } else {
    ## result is a hyperframe
    if(!all(ishyp <- (mkfmt == "hyperframe"))) 
      marxlist[!ishyp] <- lapply(marxlist[!ishyp], as.hyperframe)
    marx <- do.call(hyperframe, marxlist)
  }
  return(marx)
}

numeric.columns <- local({

  ## extract only the columns of (passably) numeric data from a data frame

  process <- function(z, logi, other) {
    if(is.numeric(z)) return(z)
    if(logi && is.logical(z)) return(as.integer(z))
    switch(other,
           na=rep.int(NA_real_, length(z)),
           discard=NULL,
           NULL)
  }

  numeric.columns <- function(M, logical=TRUE, others=c("discard", "na")) {
    others <- match.arg(others)
    M <- as.data.frame(M)
    if(ncol(M) == 1)
      colnames(M) <- NULL
    Mprocessed <- lapply(M, process, logi=logical, other=others)
    isnul <- unlist(lapply(Mprocessed, is.null))
    if(all(isnul)) {
      #' all columns have been removed
      #' return a data frame with no columns
      return(as.data.frame(matrix(, nrow=nrow(M), ncol=0)))
    }
    Mout <- do.call(data.frame, Mprocessed[!isnul])
    if(ncol(M) == 1 && ncol(Mout) == 1)
      colnames(Mout) <- NULL
    return(Mout)
  }

  numeric.columns
})


coerce.marks.numeric <- function(X, warn=TRUE) {
  marx <- marks(X)
  if(is.null(dim(marx))) {
    if(is.factor(marx)) {
      if(warn) warning("Factor-valued marks were converted to integer codes",
                       call.=FALSE)
      marx <- as.integer(marx)
      return(X %mark% marx)
    }
  } else {
    marx <- as.data.frame(marx)
    if(any(fax <- unlist(lapply(marx, is.factor)))) {
      if(warn) {
        nf <- sum(fax)
        whinge <- paste("Factor-valued mark",
                        ngettext(nf, "variable", "variables"),
                        commasep(sQuote(colnames(marx)[fax])),
                        ngettext(nf, "was", "were"),
                        "converted to integer codes")
        warning(whinge, call.=FALSE)
      }
      marx[fax] <- as.data.frame(lapply(marx[fax], as.integer))
      return(X %mark% marx)
    }
  }
  return(X)
}

#' for 'print' methods
markvaluetype <- function(x) {
  if(is.hyperframe(x)) return(unclass(x)$vclass)
  if(!is.null(dim(x))) x <- as.data.frame(x)
  if(is.data.frame(x)) return(sapply(x, markvaluetype))
  if(inherits(x, c("POSIXt", "Date"))) return("date-time")
  if(is.factor(x)) return("factor")
  return(typeof(x))
}

#
#  unnormdensity.R
#
#  $Revision: 1.5 $  $Date: 2016/02/11 10:17:12 $
#

unnormdensity <- function(x, ..., weights=NULL) {
  if(any(!nzchar(names(list(...)))))
    stop("All arguments must be named (tag=value)")
  if(is.null(weights)) {
    out <- do.call.matched(density.default, c(list(x=x), list(...)))
    out$y <- length(x) * out$y
  } else if(all(weights == 0)) {
    # result is zero
    out <- do.call.matched(density.default, c(list(x=x), list(...)))
    out$y <- 0 * out$y
  } else if(all(weights >= 0)) {
    # all masses are nonnegative
    w <- weights
    totmass <- sum(w)
    out <- do.call.matched(density.default,
                           c(list(x=x),
                             list(...),
                             list(weights=w/totmass)))
    out$y <- out$y * totmass
  } else if(all(weights <= 0)) {
    # all masses are nonpositive
    w <- (- weights)
    totmass <- sum(w)
    out <- do.call.matched(density.default,
                           c(list(x=x),
                             list(...),
                             list(weights=w/totmass)))
    out$y <- out$y * (- totmass)
  } else {
    # mixture of positive and negative masses
    w <- weights
    wabs <- abs(w)
    wpos <- pmax.int(0, w)
    wneg <- - pmin.int(0, w)
    # determine bandwidth using absolute masses
    dabs <- do.call.matched(density.default,
                            c(list(x=x),
                              list(...),
                              list(weights=wabs/sum(wabs))))
    bw <- dabs$bw
    # compute densities for positive and negative masses separately
    outpos <- do.call.matched(density.default,
                      resolve.defaults(list(x=x),
                                       list(bw=bw, adjust=1),
                                       list(weights=wpos/sum(wpos)),
                                       list(...),
                                       .StripNull=TRUE))
    outneg <- do.call.matched(density.default,
                      resolve.defaults(list(x=x),
                                       list(bw=bw, adjust=1),
                                       list(weights=wneg/sum(wneg)),
                                       list(...),
                                       .StripNull=TRUE))
    # combine
    out <- outpos
    out$y <- sum(wpos) * outpos$y - sum(wneg) * outneg$y
  }
  out$call <- match.call()
  return(out)
}

# eem.R
#
# Computes the Stoyan-Grabarnik "exponential energy weights" 
#
# $Revision: 1.4 $ $Date: 2008/07/25 19:51:05 $
#

eem <- function(fit, check=TRUE) {
  verifyclass(fit, "ppm")
  lambda <- fitted.ppm(fit, check=check)
  Q <- quad.ppm(fit)
  Z <- is.data(Q)
  eemarks <- 1/lambda[Z]
  attr(eemarks, "type") <- "eem"
  attr(eemarks, "typename") <- "exponential energy marks"
  return(eemarks)
}

##
## spatialcdf.R
##
##  $Revision: 1.5 $ $Date: 2019/03/12 11:45:26 $
##

spatialcdf <- function(Z, weights=NULL, normalise=FALSE, ...,
                       W=NULL, Zname=NULL) {
  Zdefaultname <- singlestring(short.deparse(substitute(Z)))
  if(is.character(Z) && length(Z) == 1) {
    if(is.null(Zname)) Zname <- Z
    switch(Zname,
           x={
             Z <- function(x,y) { x }
           }, 
           y={
             Z <- function(x,y) { y }
           },
           stop("Unrecognised covariate name")
         )
  }
  if(is.null(Zname)) Zname <- Zdefaultname
  ##
  if(is.ppm(weights) || is.kppm(weights) || is.dppm(weights)) {
    Q <- quad.ppm(as.ppm(weights))
    loc <- as.ppp(Q)
    df <- mpl.get.covariates(list(Z=Z), loc, covfunargs=list(...))
    df$wt <- fitted(weights) * w.quad(Q)
    G <- with(df, ewcdf(Z, wt, normalise=normalise))
    wtname <- if(normalise) "fraction of points" else "number of points"
  } else {
    if(is.null(W)) W <- as.owin(weights, fatal=FALSE)
    if(is.null(W)) W <- as.owin(Z, fatal=FALSE)
    if(is.null(W)) stop("No information specifying the spatial window")
    M <- as.mask(W, ...)
    loc <- rasterxy.mask(M, drop=TRUE)
    pixelarea <- with(unclass(M), xstep * ystep)
    if(is.null(weights)) {
      df <- mpl.get.covariates(list(Z=Z),
                               loc,
                               covfunargs=list(...))
      G <- with(df, ewcdf(Z, normalise=normalise, adjust=pixelarea))
      wtname <- if(normalise) "fraction of area" else "area"
    } else {
      df <- mpl.get.covariates(list(Z=Z, wt=weights),
                               loc,
                               covfunargs=list(...))
      G <- with(df, ewcdf(Z, wt, normalise=normalise, adjust=pixelarea))
      wtname <- if(normalise) "fraction of weight" else "weight"
    }
  }
  class(G) <- c("spatialcdf", class(G))
  attr(G, "call") <- sys.call()
  attr(G, "Zname") <- Zname
  attr(G, "ylab") <- paste("Cumulative", wtname)
  return(G)
}

plot.spatialcdf <- function(x, ..., xlab, ylab) {
  if(missing(xlab) || is.null(xlab))
    xlab <- attr(x, "Zname")
  if(missing(ylab) || is.null(ylab))
    ylab <- attr(x, "ylab")
  if(inherits(x, "ecdf")) {
    plot.ecdf(x, ..., xlab=xlab, ylab=ylab)
  } else {
    plot.stepfun(x, ..., xlab=xlab, ylab=ylab)
  }
}

#
# split.ppp.R
#
# $Revision: 1.36 $ $Date: 2020/05/08 01:22:18 $
#
# split.ppp and "split<-.ppp"
#
#########################################

split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, reduce=FALSE, ...) {
  verifyclass(x, "ppp")
  mf <- markformat(x)
  fgiven <- !missing(f)
  
  if(is.null(un)) {
    un <- !fgiven && (mf != "dataframe")
  } else un <- as.logical(un)

  if(!fgiven) {
    # f defaults to marks of x
    switch(mf,
           none={
             stop("f is missing and there are no marks")
           },
           vector={
             if(!is.multitype(x)) 
               stop("f is missing and the pattern is not multitype")
             f <- fsplit <- marks(x)
           },
           dataframe={
             f <- fsplit <- firstfactor(marks(x))
             if(is.null(f))
               stop("Data frame of marks contains no factors")
           })
    splittype <- "factor"
  } else {
    # f was given
    fsplit <- f
    if(is.factor(f)) {
      splittype <- "factor"
    } else if(is.logical(f)) {
      splittype <- "factor"
      f <- factor(f)
    } else if(is.tess(f)) {
      # f is a tessellation: determine the grouping
      f <- marks(cut(x, fsplit))
      splittype <- "tess"
    } else if(is.owin(f)) {
      # f is a window: coerce to a tessellation
      W <- as.owin(x)
      fsplit <- tess(tiles=list(fsplit, setminus.owin(W, fsplit)),
                     window=W)
      f <- marks(cut(x, fsplit))
      splittype <- "tess"
    } else if(is.im(f)) {
      # f is an image: coerce to a tessellation
      fsplit <- tess(image=f)
      f <- marks(cut(x, fsplit))
      splittype <- "tess"
    } else if(is.character(f) && length(f) == 1) {
      # f is the name of a column of marks
      marx <- marks(x)
      if(is.data.frame(marx) && (f %in% names(marx))) 
        fsplit <- f <- marx[[f]]
      else
        stop(paste("The name", sQuote(f), "does not match any column of marks"))
      splittype <- "factor"
    } else 
      stop(paste("f must be",
                 "a factor, a logical vector,", 
                 "a tessellation, a window, an image,",
                 "or the name of a column of marks"))
    if(length(f) != npoints(x))
      stop("length(f) must equal the number of points in x")
  }

  # At this point
  # 'f' is a factor that can be used to separate the points
  # 'fsplit' is the object (either a factor or a tessellation)
  # that determines the split (and can be "un-split")

  lev <- levels(f)
  if(drop) {
    # remove components that don't contain points
    retain <- (table(f) > 0)
    lev <- lev[retain]
    switch(splittype,
           tess = {
             # remove tiles that don't contain points
             fsplit <- fsplit[retain]
           },
           factor = {
             # delete levels that don't occur
             fsplit <- factor(fsplit, levels=lev)
           },
           stop("Internal error: wrong format for fsplit"))
  }

  ## remove marks that will not be retained
  if(un && reduce && mf == "dataframe")
    warning("Incompatible arguments un=TRUE and reduce=TRUE: assumed un=TRUE")
  if(un) {
    x <- unmark(x)
  } else if(reduce && !fgiven && mf == "dataframe") {
    # remove the column of marks that determined the split
    j <- findfirstfactor(marks(x))
    if(!is.null(j))
      marks(x) <- marks(x)[, -j]
  }
  
  ## split the data
  out <- list()
  fok <- !is.na(f)
  for(l in lev) 
    out[[paste(l)]] <- x[fok & (f == l)]

  ## 
  if(splittype == "tess") {
    til <- tiles(fsplit)
    for(i in seq_along(out))
      out[[i]]$window <- til[[i]]
  }
  class(out) <- c("splitppp", "ppplist", "solist", class(out))
  attr(out, "fsplit") <- fsplit
  attr(out, "fgroup") <- f
  return(out)
}

"split<-.ppp" <- function(x, f=marks(x), drop=FALSE, un=NULL,
                          ..., value) {
  verifyclass(x, "ppp")
  W <- x$window
  fgiven <- !missing(f)
  mf <- markformat(x)
  
  # evaluate `un' before assigning value of 'f'
  if(is.null(un)) {
    un <- !fgiven && (mf != "dataframe")
  } else un <- as.logical(un)

  # validate assignment value
  stopifnot(is.list(value))
  if(!all(unlist(lapply(value, is.ppp))))
    stop(paste("Each entry of", sQuote("value"),
               "must be a point pattern"))

  ismark <- unlist(lapply(value, is.marked))
  if(any(ismark) && !all(ismark))
    stop(paste("Some entries of",
               sQuote("value"),
               "are marked, and others are unmarked"))
  vmarked <- all(ismark)

  # determine type of splitting
  if(missing(f)) {
    # f defaults to marks of x
    switch(mf,
           none={
             stop("f is missing and there are no marks")
           },
           vector={
             if(!is.multitype(x)) 
               stop("f is missing and the pattern is not multitype")
             f <- fsplit <- marks(x)
           },
           dataframe={
             f <- fsplit <- firstfactor(marks(x))
             if(is.null(f))
               stop("Data frame of marks contains no factors")
           })
  } else {
    # f given
    fsplit <- f
    if(is.tess(f)) {
      # f is a tessellation: determine the grouping
      f <- marks(cut(x, fsplit))
    } else if(is.im(f)) {
      # f is an image: determine the grouping
      fsplit <- tess(image=f)
      f <- marks(cut(x, fsplit))
    } else if(is.character(f) && length(f) == 1) {
      # f is the name of a column of marks
      marx <- marks(x)
      if(is.data.frame(marx) && (f %in% names(marx))) 
        fsplit <- f <- marx[[f]]
      else
        stop(paste("The name", sQuote(f), "does not match any column of marks"))
    } else if(is.logical(f)) {
      f <- factor(f)
    } else if(!is.factor(f))
      stop(paste("f must be",
                 "a factor, a logical vector, a tessellation, an image,",
                 "or the name of a column of marks"))
    if(length(f) != x$n)
      stop("length(f) must equal the number of points in x")
  } 
  #
  all.levels <- lev <- levels(f)
  if(!drop) 
    levtype <- "levels of f"
  else {
    levtype <- "levels which f actually takes"
    # remove components that don't contain points
    lev <- lev[table(f) > 0]
  }
  if(length(value) != length(lev))
      stop(paste("length of", sQuote("value"),
                 "should equal the number of",
                 levtype))

  # ensure value[[i]] is associated with lev[i]
  if(!is.null(names(value))) {
    if(!all(names(value) %in% as.character(lev)))
      stop(paste("names of", sQuote("value"), "should be levels of f"))
    value <- value[lev]
  }
  names(value) <- NULL

  # restore the marks, if they were discarded
  if(un && is.marked(x)) {
    if(vmarked)
      warning(paste(sQuote("value"), "contains marked point patterns:",
                    "this is inconsistent with un=TRUE; marks ignored."))
    for(i in seq_along(value)) 
      value[[i]] <- value[[i]] %mark% factor(lev[i], levels=all.levels)
  }

  # handle NA's in splitting factor
  if(any(isNA <- is.na(f))) {
    xNA <- x[isNA]
    if(un && is.marked(x)) 
      xNA <- xNA %mark% factor(NA, levels=all.levels)
    value <- append(value, list(xNA))
  }

  # put Humpty together again
  if(npoints(x) == length(f) &&
     length(levels(f)) == length(value) &&
     all(table(f) == sapply(value, npoints))) {
    ## exact correspondence
    out <- x
    for(i in seq_along(levels(f)))
      out[ f == lev[i] ] <- value[[i]]
  } else {
    out <- do.call(superimpose,c(value,list(W=W)))
  }
  return(out)
}


print.splitppp <- function(x, ...) {
  f <- attr(x, "fsplit")
  what <- if(is.tess(f)) "tessellation" else
          if(is.factor(f)) "factor" else 
          if(is.logical(f)) "logical vector" else typeof(f)
  cat(paste("Point pattern split by", what, "\n"))
  nam <- names(x)
  for(i in seq_along(x)) {
    cat(paste("\n", nam[i], ":\n", sep=""))
    print(x[[i]])
  }
  return(invisible(NULL))
}

summary.splitppp <- function(object, ...) {
  x <- lapply(object, summary, ...)
  class(x) <- "summary.splitppp"
  x
}

print.summary.splitppp <- function(x, ...) {
  class(x) <- "anylist"
  print(x)
  invisible(NULL)
}

"[.splitppp" <- function(x, ...) {
  f <- attr(x, "fsplit")
  # invoke list method on x
  class(x) <- "list"
  y <- x[...]
  # then make it a 'splitppp' object too
  class(y) <- c("splitppp", class(y))
  if(is.tess(f)) {
    fsplit <- f[...]
  } else if(is.factor(f)) {
    lev <- levels(f)
    sublev <- lev[...]
    subf <- f[f %in% sublev]
    fsplit <- factor(subf, levels=lev)
  } else stop("Unknown splitting type")
  attr(y, "fsplit") <- fsplit
  y
}

"[<-.splitppp" <- function(x, ..., value) {
  if(!all(unlist(lapply(value, is.ppp))))
    stop("replacement value must be a list of point patterns")
  f <- attr(x, "fsplit")
  # invoke list method
  class(x) <- "list"
  x[...] <- value
  # then make it a 'splitppp' object too
  class(x) <- c("splitppp", class(x))
  if(is.tess(f)) {
    fsplit <- f
  } else if(is.factor(f)) {
    lev <- levels(f)
    fsplit <- factor(rep.int(lev, unlist(lapply(x, npoints))), levels=lev)
  }
  attr(x, "fsplit") <- fsplit
  x
}
  
density.ppplist <- 
density.splitppp <- function(x, ..., weights=NULL, se=FALSE) {
  if(is.null(weights) || is.im(weights) || is.expression(weights))
    weights <- rep(list(weights), length(x))
  y <- mapply(density.ppp, x=x, weights=weights,
              MoreArgs=list(se=se, ...),
              SIMPLIFY=FALSE)
  if(!se) return(as.solist(y, demote=TRUE))
  y.est <- lapply(y, getElement, name="estimate")
  y.se  <- lapply(y, getElement, name="SE")
  z <- list(estimate = as.solist(y.est, demote=TRUE),
            SE       = as.solist(y.se,  demote=TRUE))
  return(z)
}


plot.splitppp <- function(x, ..., main) {
  if(missing(main)) main <- short.deparse(substitute(x))
  do.call(plot.solist,
          resolve.defaults(list(x=x, main=main),
                           list(...),
                           list(equal.scales=TRUE)))
}

as.layered.splitppp <- function(X) { do.call(layered, X) }

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#
# tests/NAinCov.R
#
# Testing the response to the presence of NA's in covariates
#
# $Revision: 1.7 $ $Date: 2020/04/30 05:23:52 $

if(FULLTEST) {
local({
  X <- runifpoint(42)
  Y <- as.im(function(x,y) { x+y }, owin())
  Y[owin(c(0.2,0.4),c(0.2,0.4))] <- NA
  # fit model: should produce a warning but no failure
  misfit <- ppm(X ~Y, covariates=list(Y=Y))
  # prediction 
  Z <- predict(misfit, type="trend", se=TRUE)
  # covariance matrix: all should be silent
  v <- vcov(misfit)
  ss <- vcov(misfit, what="internals")
  NULL
  #' quantile.ewcdf
  f <- ewcdf(runif(100), runif(100))
  qf <- quantile(f, probs=c(0.1, NA, 0.8))
  #' quantile.density
  f <- density(runif(100))
  qf <- quantile(f, probs=c(0.1, NA, 0.8))
})
}
#
#    tests/nndist.R
#
# Check that nndist and nnwhich give
# results consistent with direct calculation from pairdist
#
# Similarly for nncross and distfun
#
# Also test whether minnndist(X) == min(nndist(X))
#
# Also test nnorient()
#
#   $Revision: 1.33 $  $Date: 2020/04/30 05:23:52 $
#


local({
  eps <- sqrt(.Machine$double.eps)
  f <- function(mat,k) { apply(mat, 1, function(z,n) { sort(z)[n]  }, n=k+1) }
  g <- function(mat,k) { apply(mat, 1, function(z,n) { order(z)[n] }, n=k+1) }

  ## .......  Two dimensions ................
  if(ALWAYS) {
    X <- runifpoint(24)

    nn <- nndist(X)
    nnP <- f(pairdist(X), 1)
    if(any(abs(nn - nnP) > eps))
      stop("nndist.ppp does not agree with pairdist")

    nn5 <- nndist(X, k=5)
    nn5P <- f(pairdist(X), 5)
    if(any(abs(nn5 - nn5P) > eps))
      stop("nndist.ppp(k=5) does not agree with pairdist")

    nw <- nnwhich(X)
    nwP <- g(pairdist(X), 1)
    if(any(nw != nwP))
      stop("nnwhich.ppp does not agree with pairdist")

    nw5 <- nnwhich(X, k=5)
    nw5P <- g(pairdist(X), 5)
    if(any(nw5 != nw5P))
      stop("nnwhich.ppp(k=5) does not agree with pairdist")
  }

  if(FULLTEST) {
    a <- nndist(X, method="test")
    b <- nnwhich(X, method="test")
    a <- nndist(X, method="test", k=1:2)
    b <- nnwhich(X, method="test", k=1:2)
    a2 <- nndist(cells[1:3], k=1:3)
    b2 <- nnwhich(cells[1:3], k=1:3)
    a3 <- nndist(cells[1])
    b3 <- nnwhich(cells[1])
    m <- factor((1:npoints(X)) %% 2 == 0)
    a4 <- nndist.default(X, by=m, k=2)
    b4 <- nnwhich.default(X, by=m, k=2)
  }

  if(ALWAYS) {
    ## nncross.ppp without options
    Y <- runifpoint(30)
    Y <- Y[nndist(Y) > 0.02]
    nc <- nncross(X,Y)
    ncd <- nc$dist
    ncw <- nc$which
    cd <- crossdist(X,Y)
    cdd <- apply(cd, 1, min)
    cdw <- apply(cd, 1, which.min)
    if(any(abs(ncd - cdd) > eps))
      stop("nncross()$dist does not agree with apply(crossdist(), 1, min)")
    if(any(ncw != cdw))
      stop("nncross()$which does not agree with apply(crossdist(), 1, which.min)")

    ## nncross with sort on x
    nc <- nncross(X,Y, sortby="x")
    ncd <- nc$dist
    ncw <- nc$which
    if(any(abs(ncd - cdd) > eps))
      stop("nncross(sortby=x)$dist does not agree with apply(crossdist(), 1, min)")
    if(any(ncw != cdw))
      stop("nncross(sortby=x)$which does not agree with apply(crossdist(), 1, which.min)")

    ## nncross with data pre-sorted on x
    Y <- Y[order(Y$x)]
    nc <- nncross(X,Y, is.sorted.Y=TRUE, sortby="x")
    ncd <- nc$dist
    ncw <- nc$which
    cd <- crossdist(X,Y)
    cdd <- apply(cd, 1, min)
    cdw <- apply(cd, 1, which.min)
    if(any(abs(ncd - cdd) > eps))
      stop("For sorted data, nncross()$dist does not agree with apply(crossdist(), 1, min)")
    if(any(ncw != cdw))
      stop("For sorted data, nncross()$which does not agree with apply(crossdist(), 1, which.min)")

    ## sanity check for nncross with k > 1
  ndw <- nncross(X, Y, k=1:4, what="which")
    if(any(is.na(ndw)))
      stop("NA's returned by nncross.ppp(k > 1, what='which')")
    nnc4 <- nncross(X, Y, k=1:4)
    iswhich <- (substr(colnames(nnc4), 1, nchar("which")) == "which")
    ndw <- nnc4[,iswhich]
    if(any(is.na(ndw)))
      stop("NA's returned by nncross.ppp(k > 1)$which")
  
    ## test of correctness for nncross with k > 1
    flipcells <- flipxy(cells)
    calcwhich <- nncross(cells, flipcells, k=1:4, what="which")
    truewhich <- t(apply(crossdist(cells,flipcells), 1, order))[,1:4]
    if(any(calcwhich != truewhich))
      stop("nncross(k > 1) gives wrong answer")
  }

  if(ALWAYS) {
    #' cover some C code blocks
    Z <- runifpoint(50)
    X <- Z[1:30]
    Y <- Z[20:50]
    iX <- 1:30
    iY <- 20:50
    Ndw <- nncross(X,Y, iX, iY, k=3)
    Nw  <- nncross(X,Y, iX, iY, k=3, what="which")
    Nd  <- nncross(X,Y, iX, iY, k=3, what="dist")

    ## special cases
    nndist(X[FALSE])
    nndist(X[1])
    nndist(X[1:3], k=4)
    nndist(X[1:3], k=1:4)
    nnwhich(X[FALSE])
    nnwhich(X[1])
    nnwhich(X[1:3], k=4)
    nnwhich(X[1:3], k=1:4)
    nncross(X[1:3], Y[FALSE])
    nncross(X[1:3], Y[1])
    nncross(X[1:3], Y[1:3], k=4)
    nncross(X[1:3], Y[1:3], k=1:4)
  }
  
  ## .......  Three dimensions ................

  if(ALWAYS) {
    X <- runifpoint3(42)

    nn <- nndist(X)
    nnP <- f(pairdist(X), 1)
    if(any(abs(nn - nnP) > eps))
      stop("nndist.pp3 does not agree with pairdist")

    nn5 <- nndist(X, k=5)
    nn5P <- f(pairdist(X), 5)
    if(any(abs(nn5 - nn5P) > eps))
      stop("nndist.pp3(k=5) does not agree with pairdist")

    nw <- nnwhich(X)
    nwP <- g(pairdist(X), 1)
    if(any(nw != nwP))
      stop("nnwhich.pp3 does not agree with pairdist")

    nw5 <- nnwhich(X, k=5)
    nw5P <- g(pairdist(X), 5)
    if(any(nw5 != nw5P))
      stop("nnwhich.pp3(k=5) does not agree with pairdist")

    ff <- function(mat,k) { apply(mat, 1, function(z,n) { sort(z)[n]  }, n=k) }
    gg <- function(mat,k) { apply(mat, 1, function(z,n) { order(z)[n] }, n=k) }

    Y <- runifpoint3(20)
    Y <- Y[nndist(Y) > 0.02]
    DXY <- crossdist(X,Y)
    a <- nncross(X,Y)
    a <- nncross(X,Y, what="dist")
    a <- nncross(X,Y, what="which")
    if(any(a != gg(DXY, 1)))
      stop("incorrect result from nncross.pp3(what='which')")
    a2 <- nncross(X,Y, k=2)
    a2 <- nncross(X,Y, what="dist", k=2)
    a2 <- nncross(X,Y, what="which", k=2)
    if(any(a2 != gg(DXY, 2)))
      stop("incorrect result from nncross.pp3(k=2, what='which')")
    iX <- 1:42
    iZ <- 30:42
    Z <- X[iZ]
    b <- nncross(X, Z, iX=iX, iY=iZ)
    b <- nncross(X, Z, iX=iX, iY=iZ, what="which")
    b <- nncross(X, Z, iX=iX, iY=iZ, what="dist")
    b2 <- nncross(X, Z, iX=iX, iY=iZ, k=2)
    b2 <- nncross(X, Z, iX=iX, iY=iZ, what="which", k=2)
    b2 <- nncross(X, Z, iX=iX, iY=iZ, what="dist", k=2)
    e1 <- nncross(X, Y[1:3], k=2:4)
    c1 <- nncross(X, Y, sortby="var")
    c2 <- nncross(X, Y, sortby="x")
    c3 <- nncross(X, Y, sortby="y")
    c4 <- nncross(X, Y, sortby="z")
    Xsort <- X[order(coords(X)$x)]
    c5 <- nncross(Xsort, Y, is.sorted.X=TRUE, sortby="x")
    Ysort <- Y[order(coords(Y)$x)]
    c6 <- nncross(Xsort, Ysort, is.sorted.X=TRUE, is.sorted.Y=TRUE, sortby="x")
  }

  if(FULLTEST) {
    ## special cases
    nndist(X[FALSE])
    nndist(X[1])
    nndist(X[1:3], k=4)
    nndist(X[1:3], k=1:4)
    nnwhich(X[FALSE])
    nnwhich(X[1])
    nnwhich(X[1:3], k=4)
    nnwhich(X[1:3], k=1:4)
    nncross(X[1:3], Y[FALSE])
    nncross(X[1:3], Y[1])
    nncross(X[1:3], Y[1:3], k=4)
    nncross(X[1:3], Y[1:3], k=1:4)
  }
  
  ## .......  m dimensions ................

  if(ALWAYS) {
    B <- boxx(c(0,1),c(0,1),c(0,1),c(0,1))
    X <- runifpointx(42, B)
    Y <- runifpointx(50, B)
    Y <- Y[nndist(Y) > 0.02]
    DXY <- crossdist(X,Y)
  
    nn <- nndist(X)
    nnP <- f(pairdist(X), 1)
    if(any(abs(nn - nnP) > eps))
      stop("nndist.ppx does not agree with pairdist")

    nn5 <- nndist(X, k=5)
    nn5P <- f(pairdist(X), 5)
    if(any(abs(nn5 - nn5P) > eps))
      stop("nndist.ppx(k=5) does not agree with pairdist")
  
    nw <- nnwhich(X)
    nwP <- g(pairdist(X), 1)
    if(any(nw != nwP))
      stop("nnwhich.ppx does not agree with pairdist")

    nw5 <- nnwhich(X, k=5)
    nw5P <- g(pairdist(X), 5)
    if(any(nw5 != nw5P))
      stop("nnwhich.ppx(k=5) does not agree with pairdist")

    a <- nncross(X,Y)
    ncd <- nncross(X,Y, what="dist")
    ncw <- nncross(X,Y, what="which")
    if(any(ncw != gg(DXY, 1)))
      stop("incorrect result from nncross.ppx(what='which')")
    a2 <- nncross(X,Y, k=2)
    ncd <- nncross(X,Y, what="dist", k=2)
    ncw <- nncross(X,Y, what="which", k=2)
    if(any(ncw != gg(DXY, 2)))
      stop("incorrect result from nncross.ppx(k=2, what='which')")
  }

  if(FULLTEST) {
    ## special cases
    nndist(X[FALSE])
    nndist(X[1])
    nndist(X[1:3], k=4)
    nndist(X[1:3], k=1:4)
    nnwhich(X[FALSE])
    nnwhich(X[1])
    nnwhich(X[1:3], k=4)
    nnwhich(X[1:3], k=1:4)
    nncross(X[1:3], Y[FALSE])
    nncross(X[1:3], Y[1])
    nncross(X[1:3], Y[1:3], k=4)
    nncross(X[1:3], Y[1:3], k=1:4)
  }

  if(ALWAYS) {
    ## test of agreement between nngrid.h and knngrid.h
    ##    dimyx=23 (found by trial-and-error) ensures that there are no ties 
    a <- as.matrix(nnmap(cells, what="which", dimyx=23))
    b <- as.matrix(nnmap(cells, what="which", dimyx=23, k=1:2)[[1]])
    if(any(a != b))
      stop("algorithms in nngrid.h and knngrid.h disagree")
    
    ## minnndist correctness
    X <- redwood3
    eps <- sqrt(.Machine$double.eps)
    mfast <- minnndist(X)
    mslow <- min(nndist(X))
    if(abs(mfast-mslow) > eps)
      stop("minnndist(X) disagrees with min(nndist(X))")

    ## maxnndist correctness
    mfast <- maxnndist(X)
    mslow <- max(nndist(X))
    if(abs(mfast-mslow) > eps)
      stop("maxnndist(X) disagrees with max(nndist(X))")
  }

  if(ALWAYS) {
    ## minnndist, maxnndist code blocks
    Y <- superimpose(amacrine, amacrine[10:20])
    a <- maxnndist(Y, positive=TRUE)
    u <- maxnndist(Y, positive=TRUE, by=marks(Y))
    b <- minnndist(Y, positive=TRUE)
    v <- minnndist(Y, positive=TRUE, by=marks(Y))

    ## nnmap code blocks
    A <- nnmap(cells[FALSE])
    A <- nnmap(cells, sortby="var")
    A <- nnmap(cells, sortby="x")
    A <- nnmap(cells, sortby="y")
    B <- nnmap(cells[1:3], k=4)
    B <- nnmap(cells[1:3], k=2:4)
    D <- nnmap(cells, outputarray=TRUE)
  }

  if(ALWAYS) {
    #' tests for has.close()
    #' (the default method uses nndist or pairdist, and can be trusted!)
    a <- has.close(redwood, 0.05)
    b <- has.close.default(redwood, 0.05)
    if(any(a != b)) stop("Incorrect result for has.close(X, r)")

    a <- has.close(redwood, 0.05, periodic=TRUE)
    a <- has.close.default(redwood, 0.05, periodic=TRUE)
    if(any(a != b)) stop("Incorrect result for has.close(X, r, periodic=TRUE)")

    Y <- split(amacrine)
    a <- with(Y, has.close(on, 0.05, off))
    b <- with(Y, has.close.default(on, 0.05, off))
    if(any(a != b)) stop("Incorrect result for has.close(X, r, Y)")

    a <- with(Y, has.close(on, 0.05, off, periodic=TRUE))
    b <- with(Y, has.close.default(on, 0.05, off, periodic=TRUE))
    if(any(a != b)) stop("Incorrect result for has.close(X, r, Y, periodic=TRUE)")
  }

  if(ALWAYS) {
    b <- bdist.pixels(letterR, style="coords")
    d <- bdist.pixels(letterR, dimyx=64, method="interpreted")
  }

  if(FULLTEST) {
    #' test nnorient
    nnorient(cells, domain=erosion(Window(cells), 0.1))
    #' degenerate case
    X <- cells[nndist(cells) > bdist.points(cells)]
    f <- nnorient(X)
    #' nnclean
    A <- nnclean(shapley, k=17, edge.correct=TRUE)
    B <- nnclean(runifpoint3(300), 3)
    #' stienen set
    #' bug when disc radius is zero
    Y <- unmark(humberside)[40:100] # contains duplicated points
    stienen(Y)
    Z <- stienenSet(Y)
    #' other cases
    U <- stienen(cells[1])
    V <- stienenSet(cells, edge=FALSE)

    ## nnfun.ppp
    f <- nnfun(cells)
    Z <- as.im(f)
    d <- domain(f)
    f <- nnfun(amacrine, value="mark")
    d <- domain(f)
    Z <- as.im(f)
    f <- nnfun(longleaf, value="mark")
    d <- domain(f)
    Z <- as.im(f)
  }

})

#
#  distances.psp.R
#
#  Hausdorff distance and Euclidean separation for psp objects
#
#  $Revision: 1.11 $ $Date: 2015/10/21 09:06:57 $
#
#

pairdist.psp <- function(X, ..., method="C", type="Hausdorff") {
  verifyclass(X, "psp")
  if(X$n == 0)
    return(matrix(, 0, 0))
  type <- pickoption("type", type,
                     c(Hausdorff="Hausdorff",
                       hausdorff="Hausdorff",
                       separation="separation"))

  D12 <- AsymmDistance.psp(X, X, metric=type, method=method)

  switch(type,
         Hausdorff={
           # maximum is Hausdorff metric
           D <- array(pmax.int(D12, t(D12)), dim=dim(D12))
         },
         separation={
           # Take minimum of endpoint-to-segment distances
           D <- array(pmin.int(D12, t(D12)), dim=dim(D12))
           # Identify any pairs of segments which cross
           cross <- test.selfcrossing.psp(X)
           # Assign separation = 0 to such pairs
           D[cross] <- 0
         })
  return(D)
}

crossdist.psp <- function(X, Y, ..., method="C", type="Hausdorff") {
  verifyclass(X, "psp")
  Y <- as.psp(Y)
  if(X$n * Y$n == 0)
    return(matrix(, X$n, Y$n))

  type <- pickoption("type", type,
                     c(Hausdorff="Hausdorff",
                       hausdorff="Hausdorff",
                       separation="separation"))
  
  DXY <- AsymmDistance.psp(X, Y, metric=type, method=method)
  DYX <- AsymmDistance.psp(Y, X, metric=type, method=method)
  
  switch(type,
         Hausdorff={
           # maximum is Hausdorff metric
           D <- array(pmax.int(DXY, t(DYX)), dim=dim(DXY))
         },
         separation={
           # Take minimum of endpoint-to-segment distances
           D <- array(pmin.int(DXY, t(DYX)), dim=dim(DXY))
           # Identify pairs of segments which cross
           cross <- test.crossing.psp(X, Y)
           # Assign separation = 0 to such pairs
           D[cross] <- 0
         })
  return(D)
}

nndist.psp <- function(X, ..., k=1, method="C") {
  verifyclass(X, "psp")
  if(!(is.vector(k) && all(k %% 1 == 0) && all(k >= 1)))
    stop("k should be a positive integer or integers")
  n <- nobjects(X)
  kmax <- max(k)
  lenk <- length(k)
  result <- if(lenk == 1) numeric(n) else matrix(, nrow=n, ncol=lenk)
  if(n == 0)
    return(result)
  if(kmax >= n) {
    # not enough objects 
    # fill with Infinite values
    result[] <- Inf
    if(any(ok <- (kmax < n))) {
      # compute the lower-order nnd's
      result[, ok] <- nndist.psp(X, ..., k=k[ok], method=method)
    }
    return(result)
  }
  # normal case:
  D <- pairdist.psp(X, ..., method=method)
  diag(D) <- Inf
  if(kmax == 1L) 
    NND <- apply(D, 1L, min)
  else 
    NND <- t(apply(D, 1L, orderstats, k=k))[, , drop=TRUE]
  return(NND)
}

# .....  AsymmDistance.psp .....
#
# If metric="Hausdorff":
#     this function computes, for each pair of segments A = X[i] and B = Y[j],
#     the value max_{a in A} d(a,B) = max_{a in A} min_{b in B} ||a-b||
#     which appears in the definition of the Hausdorff metric.
#     Since the distance function d(a,B) of a segment B is a convex function,
#     the maximum is achieved at an endpoint of A. So the algorithm
#     actually computes h(A,B) = max (d(e_1,B), d(e_2,B)) where e_1, e_2
#     are the endpoints of A. And H(A,B) = max(h(A,B),h(B,A)).
#
# If metric="separation":
#     the function computes, for each pair of segments A = X[i] and B = Y[j],
#     the MINIMUM distance from an endpoint of A to any point of B.
#        t(A,B) = min (d(e_1,B), d(e_2,B))
#     where e_1, e_2 are the endpoints of A.
#     Define the separation distance
#        s(A,B) = min_{a in A} min_{b in B} ||a-b||.
#     The minimum (a*, b*) occurs either when a* is an endpoint of A,
#     or when b* is an endpoint of B, or when a* = b* (so A and B intersect).
#     (If A and B are parallel, the minimum is still achieved at an endpoint)
#     Thus s(A,B) = min(t(A,B), t(B,A)) unless A and B intersect.


AsymmDistance.psp <- function(X, Y, metric="Hausdorff",
                              method=c("C", "Fortran", "interpreted")) {
  method <- match.arg(method)
  # Extract endpoints of X
  EX <- endpoints.psp(X, "both")
  idX <- attr(EX, "id")
  # compute shortest dist from each endpoint of X to each segment of Y
  DPL <- distppll(cbind(EX$x,EX$y), Y$ends, mintype=0, method=method)
  # for each segment in X, maximise or minimise over the two endpoints
  Dist <- as.vector(DPL)
  Point <- as.vector(idX[row(DPL)])
  Segment <- as.vector(col(DPL))
  switch(metric,
         Hausdorff={
           DXY <- tapply(Dist, list(factor(Point), factor(Segment)), max)
         },
         separation={
           DXY <- tapply(Dist, list(factor(Point), factor(Segment)), min)
           })
  return(DXY)
}

#
#
#   rmhResolveTypes.R
#
#   $Revision: 1.10 $   $Date: 2019/02/20 03:34:50 $
#
#
rmhResolveTypes <- function(model, start, control) {

# Decide whether a multitype point process is to be simulated.
# If so, determine the vector of types.

  verifyclass(model, "rmhmodel")
  verifyclass(start, "rmhstart")
  verifyclass(control, "rmhcontrol")

# Different ways of specifying types directly

  types.model <- model$types
  types.start <- if(start$given=="x" && is.marked(x.start <- start$x.start))
                     levels(marks(x.start, dfok=FALSE)) else NULL
  
# Check for inconsistencies  
  if(!is.null(types.model) && !is.null(types.start))
    if(!isTRUE(all.equal(types.model, types.start)))
      stop("marks in start$x.start do not match model$types")
  
  types.given <- if(!is.null(types.model)) types.model else types.start
  types.given.source <-
    if(!is.null(types.model)) "model$types" else "marks of x.start"
  
# Different ways of implying the number of types
  
  ntypes.beta <- length(model$par[["beta"]])
  ntypes.ptypes <- length(control$ptypes)
  ntypes.nstart <- if(start$given == "n") length(start$n.start) else 0
  mot <- model$trend
  ntypes.trend <-  if(is.null(mot)) 0 else
                   if(is.im(mot)) 1 else
                   if(is.list(mot) &&
                      all(unlist(lapply(mot, is.im))))
                     length(mot) else 0
  
# Check for inconsistencies in implied number of types (only for numbers > 1)

  nty <- c(ntypes.beta, ntypes.ptypes, ntypes.nstart, ntypes.trend)
  nam <- c("model$par$beta", "control$ptypes", "start$n.start", "model$trend")
  implied <- (nty > 1)
  if(!any(implied))
    ntypes.implied <- 1
  else {
    if(length(unique(nty[implied])) > 1)
      stop(paste("Mismatch in numbers of types implied by",
                 commasep(sQuote(nam[implied]))))
    ntypes.implied <- unique(nty[implied])
    ntypes.implied.source <- (nam[implied])[1]
  } 

# Check consistency between types.given and ntypes.implied 

  if(!is.null(types.given) && ntypes.implied > 1)
    if(length(types.given) != ntypes.implied)
      stop(paste("Mismatch between number of types in",
                 types.given.source,
                 "and length of",
                 ntypes.implied.source))

# Finally determine the types
  
  if(model$multitype.interact) {
    # There MUST be a types vector
    types <- if(!is.null(types.given)) types.given
             else if(ntypes.implied > 1) 1:ntypes.implied
             else stop("Cannot determine types for multitype process")
  } else {
    types <- if(!is.null(types.given)) types.given
             else if(ntypes.implied > 1) 1:ntypes.implied
             else 1
  }

  ntypes <- length(types)
  
# If we are conditioning on the number of points of each type,
# make sure the starting state is appropriate

  if(control$fixing == "n.each.type") {
    if(start$given == "n" && ntypes.nstart != ntypes)
      stop("Length of start$n.start not equal to number of types.\n")
    else if(start$given == "x" && length(types.given) != ntypes) 
      stop("Marks of start$x.start do not match number of types.\n")
  }
  
  return(types)
}

  

# Lurking variable plot for arbitrary covariate.
#
#
# $Revision: 1.64 $ $Date: 2019/01/10 00:21:28 $
#

lurking <- function(object, ...) {
  UseMethod("lurking")
}

lurking.ppp <- lurking.ppm <- local({

  cumsumna <- function(x) { cumsum(ifelse(is.na(x), 0, x)) }

  ## main function
  Lurking.ppm <- function(object, covariate,
                          type="eem",
                          cumulative=TRUE,
                          ..., 
                          plot.it=TRUE,
                          plot.sd=is.poisson(object), 
                          clipwindow=default.clipwindow(object),
                          rv = NULL,
                          envelope=FALSE, nsim=39, nrank=1,
                          typename,
                          covname, oldstyle=FALSE,
                          check=TRUE,
                          verbose=TRUE,
                          nx=128,
                          splineargs=list(spar=0.5),
                          internal=NULL) {
    cl <- match.call()

    ## validate object
    if(is.ppp(object)) {
      X <- object
      object <- ppm(X ~1, forcefit=TRUE)
      dont.complain.about(X)
    } else verifyclass(object, "ppm")

    ## default name for covariate
    if(missing(covname) || is.null(covname)) {
      co <- cl$covariate
      covname <- if(is.name(co)) as.character(co) else
                 if(is.expression(co)) format(co[[1]]) else NULL
    }

    ## handle secret data
    internal <- resolve.defaults(internal,
                                 list(saveworking=FALSE,
                                      Fisher=NULL,
                                      covrange=NULL))
    saveworking <- internal$saveworking
    Fisher      <- internal$Fisher  # possibly from a larger model
    covrange    <- internal$covrange
    
    if(!identical(envelope, FALSE)) {
      ## compute simulation envelope
      Xsim <- NULL
      if(!identical(envelope, TRUE)) {
        ## some kind of object
        Y <- envelope
        if(is.list(Y) && all(sapply(Y, is.ppp))) {
          Xsim <- Y
          envelope <- TRUE
        } else if(inherits(Y, "envelope")) {
          Xsim <- attr(Y, "simpatterns")
          if(is.null(Xsim))
            stop("envelope does not contain simulated point patterns")
          envelope <- TRUE
        } else stop("Unrecognised format of argument: envelope")
        nXsim <- length(Xsim)
        if(missing(nsim) && (nXsim < nsim)) {
          warning(paste("Only", nXsim, "simulated patterns available"))
          nsim <- nXsim
        }
      }
    }
    
    ## may need to refit the model
    if(plot.sd && is.null(getglmfit(object)))
      object <- update(object, forcefit=TRUE, use.internal=TRUE)

    ## match type argument
    type <- pickoption("type", type,
                       c(eem="eem",
                         raw="raw",
                         inverse="inverse",
                         pearson="pearson",
                         Pearson="pearson"))
    if(missing(typename))
      typename <- switch(type,
                         eem="exponential energy weights",
                         raw="raw residuals",
                         inverse="inverse-lambda residuals",
                         pearson="Pearson residuals")

    ## extract spatial locations
    Q <- quad.ppm(object)
    datapoints <- Q$data
    quadpoints <- union.quad(Q)
    Z <- is.data(Q)
    wts <- w.quad(Q)
    ## subset of quadrature points used to fit model
    subQset <- getglmsubset(object)
    if(is.null(subQset)) subQset <- rep.int(TRUE, n.quad(Q))
  
    #################################################################
    ## compute the covariate
    
    if(is.im(covariate)) {
      covvalues <- covariate[quadpoints, drop=FALSE]
      covrange <- covrange %orifnull% range(covariate, finite=TRUE)
    } else if(is.vector(covariate) && is.numeric(covariate)) {
      covvalues <- covariate
      covrange <- covrange %orifnull% range(covariate, finite=TRUE)
      if(length(covvalues) != quadpoints$n)
        stop("Length of covariate vector,", length(covvalues), "!=",
             quadpoints$n, ", number of quadrature points")
    } else if(is.expression(covariate)) {
      ## Expression involving covariates in the model
      glmdata <- getglmdata(object)
      ## Fix special cases
      if(is.null(glmdata)) {
        ## default 
        glmdata <- data.frame(x=quadpoints$x, y=quadpoints$y)
        if(is.marked(quadpoints))
          glmdata$marks <- marks(quadpoints)
      }
      ## ensure x and y are in data frame 
      if(!all(c("x","y") %in% names(glmdata))) {
        glmdata$x <- quadpoints$x
        glmdata$y <- quadpoints$y
      } 
      if(!is.null(object$covariates)) {
        ## Expression may involve an external covariate that's not used in model
        neednames <- all.vars(covariate)
        if(!all(neednames %in% colnames(glmdata))) {
          moredata <- mpl.get.covariates(object$covariates, quadpoints,
                                         covfunargs=object$covfunargs)
          use <- !(names(moredata) %in% colnames(glmdata))
          glmdata <- cbind(glmdata, moredata[,use,drop=FALSE])
        }
      }
      ## Evaluate expression
      sp <- parent.frame()
      covvalues <- eval(covariate, envir= glmdata, enclos=sp)
      covrange <- covrange %orifnull% range(covvalues, finite=TRUE)
      if(!is.numeric(covvalues))
        stop("The evaluated covariate is not numeric")
    } else 
      stop(paste("The", sQuote("covariate"), "should be either",
                 "a pixel image, an expression or a numeric vector"))

    #################################################################
    ## Secret exit
    if(identical(internal$getrange, TRUE))
      return(covrange)
    
    #################################################################
    ## Validate covariate values

    nbg <- is.na(covvalues)
    if(any(offending <- nbg & subQset)) {
      if(is.im(covariate))
        warning(paste(sum(offending), "out of", length(offending),
                      "quadrature points discarded because",
                      ngettext(sum(offending), "it lies", "they lie"),
                      "outside the domain of the covariate image"))
      else
        warning(paste(sum(offending), "out of", length(offending),
                      "covariate values discarded because",
                      ngettext(sum(offending), "it is NA", "they are NA")))
    }
    ## remove points
    ok <- !nbg & subQset
    Q <- Q[ok]
    covvalues <- covvalues[ok]
    quadpoints <- quadpoints[ok]
    ## adjust
    Z <- is.data(Q)
    wts <- w.quad(Q)
    if(any(is.infinite(covvalues) | is.nan(covvalues)))
      stop("covariate contains Inf or NaN values")

    ## Quadrature points marked by covariate value
    covq <- quadpoints %mark% as.numeric(covvalues)

    ################################################################
    ## Residuals/marks attached to appropriate locations.
    ## Stoyan-Grabarnik weights are attached to the data points only.
    ## Others (residuals) are attached to all quadrature points.

    resvalues <- 
      if(!is.null(rv)) rv
      else if(type=="eem") eem(object, check=check)
      else residuals.ppm(object, type=type, check=check)
  
    if(inherits(resvalues, "msr")) {
      ## signed or vector-valued measure
      resvalues <- resvalues$val
      if(ncol(as.matrix(resvalues)) > 1)
        stop("Not implemented for vector measures; use [.msr to split into separate components")
    }

    if(type != "eem")
      resvalues <- resvalues[ok]

    res <- (if(type == "eem") datapoints else quadpoints) %mark% as.numeric(resvalues)

    ## ... and the same locations marked by the covariate
    covres <- if(type == "eem") covq[Z] else covq

    ## NAMES OF THINGS
    ## name of the covariate
    if(is.null(covname)) 
      covname <- if(is.expression(covariate)) covariate else "covariate"
    ## type of residual/mark
    if(missing(typename)) 
      typename <- if(!is.null(rv)) "rv" else ""
    
    #######################################################################
    ## START ANALYSIS
    ## Clip to subwindow if needed
    clip <-
      (!is.poisson.ppm(object) || !missing(clipwindow)) &&
      !is.null(clipwindow)
    if(clip) {
      covq <- covq[clipwindow]
      res <- res[clipwindow]
      covres <- covres[clipwindow]
      clipquad <- inside.owin(quadpoints$x, quadpoints$y, clipwindow)
      wts <- wts[ clipquad ]
    }

    ## -----------------------------------------------------------------------
    ## (A) EMPIRICAL CUMULATIVE FUNCTION
    ## based on data points if type="eem", otherwise on quadrature points

    ## Reorder the data/quad points in order of increasing covariate value
    ## and then compute the cumulative sum of their residuals/marks
    markscovres <- marks(covres)
    o <- fave.order(markscovres)
    covsort <- markscovres[o]
    cummark <- cumsumna(marks(res)[o])
    if(anyDuplicated(covsort)) {
      right <- !duplicated(covsort, fromLast=TRUE)
      covsort <- covsort[right]
      cummark <- cummark[right]
    }
    ## we'll plot(covsort, cummark) in the cumulative case

    ## (B) THEORETICAL MEAN CUMULATIVE FUNCTION
    ## based on all quadrature points
    
    ## Range of covariate values
    covqmarks <- marks(covq)
    covrange <- covrange %orifnull% range(covqmarks, na.rm=TRUE)
    if(diff(covrange) > 0) {
      ## Suitable breakpoints
      cvalues <- seq(from=covrange[1L], to=covrange[2L], length.out=nx)
      csmall <- cvalues[1L] - diff(cvalues[1:2])
      cbreaks <- c(csmall, cvalues)
      ## cumulative area as function of covariate values
      covclass <- cut(covqmarks, breaks=cbreaks)
      increm <- tapply(wts, covclass, sum)
      cumarea <- cumsumna(increm)
    } else {
      ## Covariate is constant
      cvalues <- covrange[1L]
      covclass <- factor(rep(1, length(wts)))
      cumarea <- increm <- sum(wts)
    }
    ## compute theoretical mean (when model is true)
    mean0 <- if(type == "eem") cumarea else numeric(length(cumarea))
    ## we'll plot(cvalues, mean0) in the cumulative case

    ## (A'),(B') DERIVATIVES OF (A) AND (B)
    ##  Required if cumulative=FALSE  
    ##  Estimated by spline smoothing (with x values jittered)
    if(!cumulative) {
      ## fit smoothing spline to (A) 
      ss <- do.call(smooth.spline,
                    append(list(covsort, cummark),
                           splineargs)
                    )
      ## estimate derivative of (A)
      derivmark <- predict(ss, covsort, deriv=1)$y 
      ## similarly for (B) 
      ss <- do.call(smooth.spline,
                    append(list(cvalues, mean0),
                           splineargs)
                    )
      derivmean <- predict(ss, cvalues, deriv=1)$y
    }
  
    ## -----------------------------------------------------------------------
    ## Store what will be plotted
  
    if(cumulative) {
      empirical <- data.frame(covariate=covsort, value=cummark)
      theoretical <- data.frame(covariate=cvalues, mean=mean0)
    } else {
      empirical <- data.frame(covariate=covsort, value=derivmark)
      theoretical <- data.frame(covariate=cvalues, mean=derivmean)
    }

    ## ------------------------------------------------------------------------
  
    ## (C) STANDARD DEVIATION if desired
    ## (currently implemented only for Poisson)
    ## (currently implemented only for cumulative case)

    if(plot.sd && !is.poisson.ppm(object))
      warning(paste("standard deviation is calculated for Poisson model;",
                    "not valid for this model"))

    if(plot.sd && cumulative) {
      ## Fitted intensity at quadrature points
      lambda <- fitted.ppm(object, type="trend", check=check)
      lambda <- lambda[ok]
      ## Fisher information for coefficients
      asymp <- vcov(object,what="internals")
      Fisher <- Fisher %orifnull% asymp$fisher
      ## Local sufficient statistic at quadrature points
      suff <- asymp$suff
      suff <- suff[ok, ,drop=FALSE]
      ## Clip if required
      if(clip) {
        lambda <- lambda[clipquad]
        suff   <- suff[clipquad, , drop=FALSE]  ## suff is a matrix
      }
      ## First term: integral of lambda^(2p+1)
      switch(type,
             pearson={
               varI <- cumarea
             },
             raw={
               ## Compute sum of w*lambda for quadrature points in each interval
               dvar <- tapply(wts * lambda, covclass, sum)
               ## tapply() returns NA when the table is empty
               dvar[is.na(dvar)] <- 0
               ## Cumulate
               varI <- cumsum(dvar)
             },
             inverse=, ## same as eem
             eem={
               ## Compute sum of w/lambda for quadrature points in each interval
               dvar <- tapply(wts / lambda, covclass, sum)
               ## tapply() returns NA when the table is empty
               dvar[is.na(dvar)] <- 0
               ## Cumulate
               varI <- cumsum(dvar)
             })


      ## variance-covariance matrix of coefficients
      V <- try(solve(Fisher), silent=TRUE)
      if(inherits(V, "try-error")) {
        warning("Fisher information is singular; reverting to oldstyle=TRUE")
        oldstyle <- TRUE
      }
      if(any(dim(V) != ncol(suff))) {
        #' drop rows and columns
        nama <- colnames(suff)
        V <- V[nama, nama, drop=FALSE]
      }

      working <- NULL
      
      ## Second term: B' V B
      if(oldstyle) {
        varII <- 0
        if(saveworking) 
          working <- data.frame(varI=varI)
      } else {
        ## lamp = lambda^(p + 1)
        lamp <- switch(type,
                       raw     = lambda, 
                       pearson = sqrt(lambda),
                       inverse =,
                       eem     = as.integer(lambda > 0))
        ## Compute sum of w * lamp * suff for quad points in intervals
        Bcontrib <- as.vector(wts * lamp) * suff
        dB <- matrix(, nrow=length(cumarea), ncol=ncol(Bcontrib),
                     dimnames=list(NULL, colnames(suff)))
        for(j in seq_len(ncol(dB))) 
          dB[,j] <- tapply(Bcontrib[,j], covclass, sum, na.rm=TRUE)
        ## tapply() returns NA when the table is empty
        dB[is.na(dB)] <- 0
        ## Cumulate columns
        B <- apply(dB, 2, cumsum)
        if(!is.matrix(B)) B <- matrix(B, nrow=1)
        ## compute B' V B for each i 
        varII <- quadform(B, V)
        ##  was:   varII <- diag(B %*% V %*% t(B))
        if(saveworking) 
          working <- cbind(data.frame(varI=varI, varII=varII),
                           as.data.frame(B))
      }
      ##
      ## variance of residuals
      varR <- varI - varII
      ## trap numerical errors
      nbg <- (varR < 0)
      if(any(nbg)) {
        ran <- range(varR)
        varR[nbg] <- 0
        relerr <- abs(ran[1L]/ran[2L])
        nerr <- sum(nbg)
        if(relerr > 1e-6) {
          warning(paste(nerr, "negative",
                        ngettext(nerr, "value (", "values (min="),
                        signif(ran[1L], 4), ")",
                        "of residual variance reset to zero",
                        "(out of", length(varR), "values)"))
        }
      }
      theoretical$sd <- sqrt(varR)
    }

    ## 
    if(envelope) {
      ## compute envelopes by simulation
      cl$plot.it <- FALSE
      cl$envelope <- FALSE
      cl$rv <- NULL
      if(is.null(Xsim))
        Xsim <- simulate(object, nsim=nsim, progress=verbose)
      values <- NULL
      if(verbose) {
        cat("Processing.. ")
        state <- list()
      }
      for(i in seq_len(nsim)) {
        cl$object <- update(object, Xsim[[i]])
        result.i <- eval(cl, parent.frame())
        ## interpolate empirical values onto common sequence
        f.i <- with(result.i$empirical,
                    approxfun(covariate, value, rule=2))
        val.i <- f.i(theoretical$covariate)
        values <- cbind(values, val.i)
        if(verbose) state <- progressreport(i, nsim, state=state)
      }
      if(verbose) cat("Done.\n")
      hilo <- if(nrank == 1) apply(values, 1, range) else
                 apply(values, 1, orderstats, k=c(nrank, nsim-nrank+1))
      theoretical$upper <- hilo[1L,]
      theoretical$lower <- hilo[2L,]
    }
    ## ----------------  RETURN COORDINATES ----------------------------
    stuff <- list(empirical=empirical,
                  theoretical=theoretical)
    attr(stuff, "info") <- list(typename=typename,
                                cumulative=cumulative,
                                covrange=covrange,
                                covname=covname,
                                oldstyle=oldstyle)
    if(saveworking) attr(stuff, "working") <- working
    class(stuff) <- "lurk"
    ## ---------------  PLOT THEM  ----------------------------------
    if(plot.it) {
      plot(stuff, ...)
      return(invisible(stuff))
    } else {
      return(stuff)
    }
  }

  Lurking.ppm
})


# plot a lurk object


plot.lurk <- function(x, ..., shade="grey") {
  xplus <- append(x, attr(x, "info"))
  with(xplus, {
    ## work out plot range
    mr <- range(0, empirical$value, theoretical$mean, na.rm=TRUE)
    if(!is.null(theoretical$sd))
      mr <- range(mr,
                  theoretical$mean + 2 * theoretical$sd,
                  theoretical$mean - 2 * theoretical$sd,
                  na.rm=TRUE)
    if(!is.null(theoretical$upper))
      mr <- range(mr, theoretical$upper, theoretical$lower, na.rm=TRUE)

    ## start plot
    vname <- paste(if(cumulative)"cumulative" else "marginal", typename)
    do.call(plot,
            resolve.defaults(
              list(covrange, mr),
              list(type="n"),
              list(...),
              list(xlab=covname, ylab=vname)))
    ## Envelopes
    if(!is.null(theoretical$upper)) {
      Upper <- theoretical$upper
      Lower <- theoretical$lower
    } else if(!is.null(theoretical$sd)) {
      Upper <- with(theoretical, mean+2*sd)
      Lower <- with(theoretical, mean-2*sd)
    } else Upper <- Lower <- NULL
    if(!is.null(Upper) && !is.null(Lower)) {
      xx <- theoretical$covariate
      if(!is.null(shade)) {
        ## shaded envelope region
        shadecol <- if(is.colour(shade)) shade else "grey"
        xx <- c(xx,    rev(xx))
        yy <- c(Upper, rev(Lower))
        do.call.matched(polygon,
                        resolve.defaults(list(x=xx, y=yy),
                                         list(...),
                                         list(border=shadecol, col=shadecol)))
      } else {
        do.call(lines,
                resolve.defaults(
                  list(x = xx, y=Upper),
                  list(...),
                  list(lty=3)))
        do.call(lines,
                resolve.defaults(
                  list(x = xx, y = Lower),
                  list(...),
                  list(lty=3)))
      }
    }
    ## Empirical
    lines(value ~ covariate, empirical, ...)
    ## Theoretical mean
    do.call(lines,
            resolve.defaults(
              list(mean ~ covariate, theoretical),
              list(...),
              list(lty=2)))
  })
  return(invisible(NULL))
}

#'  print a lurk object

print.lurk <- function(x, ...) {
  splat("Lurking variable plot (object of class 'lurk')")
  info <- attr(x, "info")
  with(info, {
    splat("Residual type: ", typename)
    splat("Covariate on horizontal axis: ", covname)
    splat("Range of covariate values: ", prange(covrange))
    splat(if(cumulative) "Cumulative" else "Non-cumulative", "plot")
  })
  has.bands <- !is.null(x$theoretical$upper)
  has.sd    <- !is.null(x$theoretical$sd)
  if(!has.bands && !has.sd) {
    splat("No confidence bands computed")
  } else {
    splat("Includes",
          if(has.sd) "standard deviation for" else NULL,
          "confidence bands")
    if(!is.null(info$oldstyle)) 
      splat("Variance calculation:",
            if(info$oldstyle) "old" else "new",
            "style")
  }
  return(invisible(NULL))
}

#
#   ppx.R
#
#  class of general point patterns in any dimension
#
#  $Revision: 1.63 $  $Date: 2019/07/21 06:33:14 $
#

ppx <- local({
  
  ctype.table <- c("spatial", "temporal", "local", "mark")
  ctype.real  <- c(TRUE,      TRUE,       FALSE,   FALSE)

  ppx <- function(data, domain=NULL, coord.type=NULL, simplify=FALSE) {
    data <- as.hyperframe(data)
    # columns suitable for spatial coordinates
    suitable <- with(unclass(data),
                     vtype == "dfcolumn" &
                     (vclass == "numeric" | vclass == "integer"))
    if(is.null(coord.type)) {
      # assume all suitable columns of data are spatial coordinates
      # and all other columns are marks.
      ctype <- ifelse(suitable, "spatial", "mark")
    } else {
      stopifnot(is.character(coord.type))
      stopifnot(length(coord.type) == ncol(data))
      ctypeid <- pmatch(coord.type, ctype.table, duplicates.ok=TRUE)
      # validate
      if(any(uhoh <- is.na(ctypeid)))
        stop(paste("Unrecognised coordinate",
                   ngettext(sum(uhoh), "type", "types"),
                   commasep(sQuote(coord.type[uhoh]))))
      if(any(uhoh <- (!suitable & ctype.real[ctypeid]))) {
        nuh <- sum(uhoh)
        stop(paste(ngettext(nuh, "Coordinate", "Coordinates"),
                   commasep(sQuote(names(data)[uhoh])),
                   ngettext(nuh, "does not", "do not"),
                   "contain real numbers"))
      }
      ctype <- ctype.table[ctypeid]
    }
    ctype <- factor(ctype, levels=ctype.table)
    #
    if(simplify && all(ctype == "spatial")) {
       # attempt to reduce to ppp or pp3
      d <- length(ctype)
      if(d == 2) {
        ow <- try(as.owin(domain), silent=TRUE)
        if(!inherits(ow, "try-error")) {
          X <- try(as.ppp(as.data.frame(data), W=ow))
          if(!inherits(X, "try-error"))
            return(X)
        }
      } else if(d == 3) {
        bx <- try(as.box3(domain), silent=TRUE)
        if(!inherits(bx, "try-error")) {
          m <- as.matrix(as.data.frame(data))
          X <- try(pp3(m[,1], m[,2], m[,3], bx))
          if(!inherits(X, "try-error"))
            return(X)
        }
      }
    }
    out <- list(data=data, ctype=ctype, domain=domain)
    class(out) <- "ppx"
    return(out)
  }

  ppx
})


is.ppx <- function(x) { inherits(x, "ppx") }

nobjects.ppx <- npoints.ppx <- function(x) { nrow(x$data) }

print.ppx <- function(x, ...) {
  cat("Multidimensional point pattern\n")
  sd <- summary(x$data)
  np <- sd$ncases
  nama <- sd$col.names
  cat(paste(np, ngettext(np, "point", "points"), "\n"))
  if(any(iscoord <- (x$ctype == "spatial")))
    cat(paste(sum(iscoord), "-dimensional space coordinates ",
              paren(paste(nama[iscoord], collapse=",")), "\n", sep=""))
  if(any(istime <- (x$ctype == "temporal")))
    cat(paste(sum(istime), "-dimensional time coordinates ",
              paren(paste(nama[istime], collapse=",")), "\n", sep=""))
  if(any(islocal <- (x$ctype == "local"))) 
    cat(paste(sum(islocal), ngettext(sum(islocal), "column", "columns"),
              "of local coordinates:",
              commasep(sQuote(nama[islocal])), "\n"))
  if(any(ismark <- (x$ctype == "mark"))) 
    cat(paste(sum(ismark), ngettext(sum(ismark), "column", "columns"),
              "of marks:",
              commasep(sQuote(nama[ismark])), "\n"))
  if(!is.null(x$domain)) {
    cat("Domain:\n\t")
    print(x$domain)
  }
  invisible(NULL)
}

summary.ppx <- function(object, ...) { object }

plot.ppx <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  coo <- coords(x, local=FALSE)
  dom <- x$domain
  m <- ncol(coo)
  if(m == 1) {
    coo <- coo[,1]
    ran <- diff(range(coo))
    ylim <- c(-1,1) * ran/20
    do.call(plot.default,
            resolve.defaults(list(coo, numeric(length(coo))),
                             list(...),
                             list(asp=1, ylim=ylim,
                                  axes=FALSE, xlab="", ylab="")))
    axis(1, pos=ylim[1])
  } else if(m == 2) {
    if(is.null(dom)) {
      # plot x, y coordinates only
      nama <- names(coo)
      do.call.matched(plot.default,
                      resolve.defaults(list(x=coo[,1], y=coo[,2], asp=1),
                                       list(...),
                                       list(main=xname),
                                       list(xlab=nama[1], ylab=nama[2])))
    } else {
      add <- resolve.defaults(list(...), list(add=FALSE))$add
      if(!add) {
        # plot domain, whatever it is
        do.call(plot, resolve.defaults(list(dom),
                                       list(...),
                                       list(main=xname)))
      }
      # convert to ppp
      x2 <- ppp(coo[,1], coo[,2], window=as.owin(dom),
                marks=as.data.frame(marks(x)), check=FALSE)
      # invoke plot.ppp
      return(do.call(plot, resolve.defaults(list(x2),
                                              list(add=TRUE),
                                              list(...))))
    }
  } else if(m == 3) {
    # convert to pp3
    if(is.null(dom))
      dom <- box3(range(coo[,1]), range(coo[,2]), range(coo[,3]))
    x3 <- pp3(coo[,1], coo[,2], coo[,3], dom)
    # invoke plot.pp3
    nama <- names(coo)
    do.call(plot,
            resolve.defaults(list(x3),
                             list(...),
                             list(main=xname),
                             list(xlab=nama[1], ylab=nama[2], zlab=nama[3])))
  } else stop(paste("Don't know how to plot a general point pattern in",
               ncol(coo), "dimensions"))
  return(invisible(NULL))
}

"[.ppx" <- function (x, i, drop=FALSE, ...) {
  da <- x$data
  dom <- x$domain
  if(!missing(i)) {
    if(inherits(i, c("boxx", "box3"))) {
      dom <- i
      i <- inside.boxx(da, w=i)
    }
    da <- da[i, , drop=FALSE]
  }
  out <- list(data=da, ctype=x$ctype, domain=dom)
  class(out) <- "ppx"
  if(drop) {
    # remove unused factor levels
    mo <- marks(out)
    switch(markformat(mo),
           none = { },
           vector = {
             if(is.factor(mo))
               marks(out) <- factor(mo)
           },
           dataframe = {
             isfac <- sapply(mo, is.factor)
             if(any(isfac))
               mo[, isfac] <- lapply(mo[, isfac], factor)
             marks(out) <- mo
           },
           hyperframe = {
             lmo <- as.list(mo)
             isfac <- sapply(lmo, is.factor)
             if(any(isfac))
               mo[, isfac] <- as.hyperframe(lapply(lmo[isfac], factor))
             marks(out) <- mo
           })
  }
  return(out)
}

domain <- function(X, ...) { UseMethod("domain") }

domain.ppx <- function(X, ...) { X$domain }

coords <- function(x, ...) {
  UseMethod("coords")
}

coords.ppx <- function(x, ..., spatial=TRUE, temporal=TRUE, local=TRUE) {
  ctype <- x$ctype
  chosen <- (ctype == "spatial" & spatial) |
            (ctype == "temporal" & temporal) | 
            (ctype == "local" & local) 
  as.data.frame(x$data[, chosen, drop=FALSE])
}

coords.ppp <- function(x, ...) { data.frame(x=x$x,y=x$y) }

"coords<-" <- function(x, ..., value) {
  UseMethod("coords<-")
}

"coords<-.ppp" <- function(x, ..., value) {
  win <- x$window
  if(is.null(value)) {
    # empty pattern
    return(ppp(window=win))
  }
  value <- as.data.frame(value)
  if(ncol(value) != 2)
    stop("Expecting a 2-column matrix or data frame, or two vectors")
  result <- as.ppp(value, win)
  marks(result) <- marks(x)
  return(result)
}

"coords<-.ppx" <- function(x, ..., spatial=TRUE, temporal=TRUE, local=TRUE, value) {
  ctype <- x$ctype
  chosen <- (ctype == "spatial" & spatial) |
            (ctype == "temporal" & temporal) | 
            (ctype == "local" & local) 
  x$data[, chosen] <- value
  return(x)
}

as.hyperframe.ppx <- function(x, ...) { x$data }

as.data.frame.ppx <- function(x, ...) { as.data.frame(x$data, ...) } 

as.matrix.ppx <- function(x, ...) { as.matrix(as.data.frame(x, ...)) }

marks.ppx <- function(x, ..., drop=TRUE) {
  ctype <- x$ctype
  chosen <- (ctype == "mark")
  if(!any(chosen)) return(NULL)
  x$data[, chosen, drop=drop]
}

"marks<-.ppx" <- function(x, ..., value) {
  ctype <- x$ctype
  retain <- (ctype != "mark")
  coorddata <- x$data[, retain, drop=FALSE]
  if(is.null(value)) {
    newdata <- coorddata
    newctype <- ctype[retain]
  } else {
    if(is.matrix(value) && nrow(value) == nrow(x$data)) {
      # assume matrix is to be treated as data frame
      value <- as.data.frame(value)
    }
    if(!is.data.frame(value) && !is.hyperframe(value)) 
      value <- hyperframe(marks=value)
    if(is.hyperframe(value) || is.hyperframe(coorddata)) {
      value <- as.hyperframe(value)
      coorddata <- as.hyperframe(coorddata)
    }
    if(ncol(value) == 0) {
      newdata <- coorddata
      newctype <- ctype[retain]
    } else {
      if(nrow(coorddata) == 0) 
        value <- value[integer(0), , drop=FALSE]
      newdata <- cbind(coorddata, value)
      newctype <- factor(c(as.character(ctype[retain]),
                           rep.int("mark", ncol(value))),
                         levels=levels(ctype))
    }
  }
  out <- list(data=newdata, ctype=newctype, domain=x$domain)
  class(out) <- class(x)
  return(out)
}

unmark.ppx <- function(X) {
  marks(X) <- NULL
  return(X)
}

markformat.ppx <- function(x) {
  mf <- x$markformat
  if(is.null(mf)) 
    mf <- markformat(marks(x))
  return(mf)
}

boxx <- function(..., unitname=NULL) {
  if(length(list(...)) == 0)
    stop("No data")
  ranges <- data.frame(...)
  nama <- names(list(...))
  if(is.null(nama) || !all(nzchar(nama)))
    names(ranges) <- paste("x", 1:ncol(ranges),sep="")
  if(nrow(ranges) != 2)
    stop("Data should be vectors of length 2")
  if(any(unlist(lapply(ranges, diff)) <= 0))
    stop("Illegal range: Second element <= first element")
  out <- list(ranges=ranges, units=as.unitname(unitname))
  class(out) <- "boxx"
  return(out)
}

as.boxx <- function(..., warn.owin = TRUE) {
  a <- list(...)
  n <- length(a)
  if (n == 0) 
    stop("No arguments given")
  if (n == 1) {
    a <- a[[1]]
    if (inherits(a, "boxx")) 
      return(a)
    if (inherits(a, "box3")) 
      return(boxx(a$xrange, a$yrange, a$zrange, 
		  unitname = as.unitname(a$units)))
    if (inherits(a, "owin")) {
      if (!is.rectangle(a) && warn.owin) 
        warning("The owin object does not appear to be rectangular - the bounding box is used!")
      return(boxx(a$xrange, a$yrange, unitname = as.unitname(a$units)))
    }
    if (is.numeric(a)) {
      if ((length(a)%%2) == 0) 
        return(boxx(split(a, rep(1:(length(a)/2), each = 2))))
      stop(paste("Don't know how to interpret", length(a), "numbers as a box"))
    }
    if (!is.list(a)) 
      stop("Don't know how to interpret data as a box")
  }
  return(do.call(boxx, a))
}

print.boxx <- function(x, ...) {
  m <- ncol(x$ranges)
  cat(paste(m, "-dimensional box:\n", sep=""))
  bracket <- function(z) paste("[",
                               paste(signif(z, 5), collapse=", "),
                               "]", sep="")
  v <- paste(unlist(lapply(x$ranges, bracket)), collapse=" x ")
  s <- summary(unitname(x))
  cat(paste(v, s$plural, s$explain, "\n"))
  invisible(NULL)
}

unitname.boxx <- function(x) { as.unitname(x$units) }

"unitname<-.boxx" <- function(x, value) {
  x$units <- as.unitname(value)
  return(x)
}

unitname.ppx <- function(x) { unitname(x$domain) }

"unitname<-.ppx" <- function(x, value) {
  d <- x$domain
  unitname(d) <- value
  x$domain <- d
  return(x)
}

as.owin.boxx <- function(W, ..., fatal=TRUE) {
  ra <- W$ranges
  if(length(ra) == 2) return(owin(ra[[1]], ra[[2]]))
  if(fatal) stop(paste("Cannot interpret box of dimension",
                       length(ra), "as a window"))
  return(NULL)
}

sidelengths.boxx <- function(x) {
  stopifnot(inherits(x, "boxx"))
  y <- unlist(lapply(x$ranges, diff))
  return(y)
}
  
volume.boxx <- function(x) {
  prod(sidelengths(x))
}

diameter.boxx <- function(x) {
  d <- sqrt(sum(sidelengths(x)^2))
  return(d)
}

shortside.boxx <- function(x) {
  return(min(sidelengths(x)))
}

eroded.volumes.boxx <- local({

  eroded.volumes.boxx <- function(x, r) {
    len <- sidelengths(x)
    ero <- sapply(as.list(len), erode1side, r=r)
    apply(ero, 1, prod)
  }

  erode1side <- function(z, r) { pmax.int(0, z - 2 * r)}
  
  eroded.volumes.boxx
})


runifpointx <- function(n, domain, nsim=1, drop=TRUE) {
  check.1.integer(n)
  check.1.integer(nsim)
  stopifnot(inherits(domain, "boxx"))
  ra <- domain$ranges
  d <- length(ra)
  result <- vector(mode="list", length=nsim)
  for(i in 1:nsim) {
    if(n == 0) {
      coo <- matrix(numeric(0), nrow=0, ncol=d)
    } else {
      coo <- mapply(runif,
                    n=rep(n, d),
                    min=ra[1,],
                    max=ra[2,])
      if(!is.matrix(coo)) coo <- matrix(coo, ncol=d)
    }
    colnames(coo) <- colnames(ra)
    df <- as.data.frame(coo)
    result[[i]] <- ppx(df, domain, coord.type=rep("s", d))
  }
  if(nsim == 1 && drop)
    return(result[[1]])
  result <- as.anylist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}

rpoisppx <- function(lambda, domain, nsim=1, drop=TRUE) {
  stopifnot(inherits(domain, "boxx"))
  stopifnot(is.numeric(lambda) && length(lambda) == 1 && lambda >= 0)
  n <- rpois(nsim, lambda * volume.boxx(domain))
  result <- vector(mode="list", length=nsim)
  for(i in 1:nsim) 
    result[[i]] <- runifpointx(n[i], domain)
  if(nsim == 1 && drop)
    return(result[[1]])
  result <- as.anylist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}

unique.ppx <- function(x, ..., warn=FALSE) {
  dup <- duplicated(x, ...)
  if(!any(dup)) return(x)
  if(warn) warning(paste(sum(dup), "duplicated points were removed"),
                   call.=FALSE)
  y <- x[!dup]
  return(y)
}

duplicated.ppx <- function(x, ...) {
  dup <- duplicated(as.data.frame(x), ...)
  return(dup)
}

anyDuplicated.ppx <- function(x, ...) {
  anyDuplicated(as.data.frame(x), ...)
}


multiplicity.ppx <- function(x) {
  mul <- multiplicity(as.data.frame(x))
  return(mul)
}

intensity.ppx <- function(X, ...) {
  if(!is.multitype(X)) {
    n <- npoints(X)
  } else {
    mks <- marks(X)
    n <- as.vector(table(mks))
    names(n) <- levels(mks)
  }
  v <- volume(domain(X))
  return(n/v)
}

grow.boxx <- function(W, left, right = left){
  W <- as.boxx(W)
  ra <- W$ranges
  d <- length(ra)
  if(any(left < 0) || any(right < 0))
    stop("values of left and right margin must be nonnegative.")
  if(length(left)==1) left <- rep(left, d)
  if(length(right)==1) right <- rep(right, d)
  if(length(left)!=d || length(right)!=d){
    stop("left and right margin must be either of length 1 or the dimension of the boxx.")
  }
  W$ranges[1,] <- ra[1,]-left
  W$ranges[2,] <- ra[2,]+right
  return(W)
}

inside.boxx <- function(..., w = NULL){
  if(is.null(w))
    stop("Please provide a boxx using the named argument w.")
  w <- as.boxx(w)
  dat <- list(...)
  if(length(dat)==1){
    dat1 <- dat[[1]]
    if(inherits(dat1, "ppx"))
      dat <- coords(dat1)
    if(inherits(dat1, "hyperframe"))
      dat <- as.data.frame(dat1)
  }
  ra <- w$ranges
  if(length(ra)!=length(dat))
    stop("Mismatch between dimension of boxx and number of coordinate vectors.")
  ## Check coord. vectors have equal length
  n <- length(dat[[1]])
  if(any(lengths(dat)!=n))
    stop("Coordinate vectors have unequal length.")
  index <- rep(TRUE, n)
  for(i in seq_along(ra)){
    index <- index & inside.range(dat[[i]], ra[[i]])
  }
  return(index)
}


spatdim <- function(X, intrinsic=FALSE) {
  if(intrinsic) {
    if(inherits(X, c("lpp", "linnet", "linim", "linfun", "lintess"))) return(1L)
    if(inherits(X, c("s2pp", "s2", "s2region"))) return(2L)
  }
  if(is.sob(X)) 2L else
  if(inherits(X, "box3")) 3L else
  if(inherits(X, "boxx")) length(X$ranges) else 
  if(is.ppx(X)) as.integer(sum(X$ctype == "spatial")) else NA_integer_
}

#
# kernels.R
#
#  rXXX, dXXX, pXXX and qXXX for kernels
#
#  $Revision: 1.19 $  $Date: 2018/06/07 05:42:54 $
#

match.kernel <- function(kernel) {
  kernel.map <- c(Gaussian    ="gaussian",
                  gaussian    ="gaussian",
                  Normal      ="gaussian",
                  normal      ="gaussian",
                  rectangular ="rectangular",
                  triangular  ="triangular",
                  Epanechnikov="epanechnikov",
                  epanechnikov="epanechnikov",
                  biweight    ="biweight",
                  cosine      ="cosine",
                  optcosine   ="optcosine"
                  )
  ker <- pickoption("kernel", kernel, kernel.map)
  return(ker)
}

kernel.factor <- function(kernel="gaussian") {
  # This function returns the factor c such that
  #              h = c * sigma
  # where sigma is the standard deviation of the kernel, and
  # h is the corresponding bandwidth parameter as conventionally defined.

  # Conventionally h is defined as a scale factor
  # relative to the `standard form' of the kernel, namely the 
  # form with support [-1,1], except in the Gaussian case where
  # the standard form is N(0,1).
  
  # Thus the standard form of the kernel (h=1) has standard deviation 1/c.
  
  # The kernel with standard deviation 1 has support [-c,c]
  # except for gaussian case.
  
  kernel <- match.kernel(kernel)
  switch(kernel,
         gaussian     = 1,
         rectangular  = sqrt(3),
         triangular   = sqrt(6),
         epanechnikov = sqrt(5),
         biweight     = sqrt(7),
         cosine       = 1/sqrt(1/3 - 2/pi^2),
         optcosine    = 1/sqrt(1 - 8/pi^2))
}

rkernel <- function(n, kernel="gaussian", mean=0, sd=1) {
  kernel <- match.kernel(kernel)
  if(kernel == "gaussian")
    return(rnorm(n, mean=mean, sd=sd))
  # inverse cdf transformation
  u <- runif(n)
  qkernel(u, kernel, mean=mean, sd=sd)
}

dkernel <- function(x,  kernel="gaussian", mean=0, sd=1) {
  kernel <- match.kernel(kernel)
  stopifnot(is.numeric(x))
  stopifnot(is.numeric(sd) && length(sd) == 1 && sd > 0)
  a <- sd * kernel.factor(kernel)
  y <- abs(x-mean)/a
  dens <-
    switch(kernel,
           gaussian       = { dnorm(y) },
           rectangular    = { ifelse(y < 1, 1/2, 0) },
           triangular     = { ifelse(y < 1, (1 - y), 0) },
           epanechnikov   = { ifelse(y < 1, (3/4) * (1 - y^2), 0) },
           biweight       = { ifelse(y < 1, (15/16) * (1 - y^2)^2, 0) },
           cosine         = { ifelse(y < 1, (1 + cos(pi * y))/2, 0) },
           optcosine      = { ifelse(y < 1, (pi/4) * cos(pi * y/2), 0) }
           )
  dens/a
}

pkernel <- function(q, kernel="gaussian", mean=0, sd=1, lower.tail=TRUE){
  kernel <- match.kernel(kernel)
  stopifnot(is.numeric(q))
  stopifnot(is.numeric(sd) && length(sd) == 1 && sd > 0)
  a <- sd * kernel.factor(kernel)
  y <- (q-mean)/a
  switch(kernel,
         gaussian = {
           pnorm(y, lower.tail=lower.tail)
         },
         rectangular = {
           punif(y, min=-1, max=1, lower.tail=lower.tail)
         },
         triangular = {
           p <- ifelse(y < -1, 0, ifelse(y > 1, 1,
                       ifelse(y < 0, y + y^2/2 + 1/2,
                              y - y^2/2 + 1/2)))
           if(lower.tail) p else (1 - p)
         },
         epanechnikov = {
           p <- ifelse(y < -1, 0, ifelse(y > 1, 1,
                        (2 + 3 * y - y^3)/4))
           if(lower.tail) p else (1 - p)
         },
         biweight = {
           p <- ifelse(y < -1, 0, ifelse(y > 1, 1,
                       (15 * y - 10 * y^3 + 3 * y^5 + 8)/16))
           if(lower.tail) p else (1 - p)
         },
         cosine = {
           p <- ifelse(y < -1, 0, ifelse(y > 1, 1,
                       (y + sin(pi * y)/pi + 1)/2))
           if(lower.tail) p else (1 - p)
         },
         optcosine = {
           p <- ifelse(y < -1, 0, ifelse(y > 1, 1,
                       (sin(pi * y/2) + 1)/2))
           if(lower.tail) p else (1 - p)
         })
}

qkernel <- function(p, kernel="gaussian", mean=0, sd=1, lower.tail=TRUE) {
  kernel <- match.kernel(kernel)
  stopifnot(is.numeric(p))
  stopifnot(is.numeric(sd) && length(sd) == 1 && sd > 0)
  a <- sd * kernel.factor(kernel)
  if(!lower.tail)
    p <- 1 - p
  y <-
    switch(kernel,
           gaussian = {
             qnorm(p, lower.tail=lower.tail)
           },
           rectangular = {
             qunif(p, min=-1, max=1, lower.tail=lower.tail)
           },
           triangular = {
             ifelse(p < 1/2, sqrt(2 * p) - 1, 1 - sqrt(2 * (1-p)))
           },
           epanechnikov = {
             # solve using `polyroot'
             yy <- numeric(n <- length(p))
             yy[p == 0] <- -1
             yy[p == 1] <-  1
             inside <- (p != 0) & (p != 1)
             # coefficients of polynomial (2 + 3 y - y^3)/4
             z <- c(2, 3, 0, -1)/4
             for(i in seq(n)[inside]) {
               sol <- polyroot(z - c(p[i], 0, 0, 0))
               ok <- abs(Im(sol)) < 1e-6
               realpart <- Re(sol)
               ok <- ok & (abs(realpart) <= 1)
               if(sum(ok) != 1)
                 stop(paste("Internal error:", sum(ok), "roots of polynomial"))
               yy[i] <- realpart[ok]
             }
             yy
           },
           biweight = {
             # solve using `polyroot'
             yy <- numeric(n <- length(p))
             yy[p == 0] <- -1
             yy[p == 1] <-  1
             inside <- (p != 0) & (p != 1)
             # coefficients of polynomial (8 + 15 * y - 10 * y^3 + 3 * y^5)/16
             z <- c(8, 15, 0, -10, 0, 3)/16
             for(i in seq(n)[inside]) {
               sol <- polyroot(z - c(p[i], 0, 0, 0, 0, 0))
               ok <- abs(Im(sol)) < 1e-6
               realpart <- Re(sol)
               ok <- ok & (abs(realpart) <= 1)
               if(sum(ok) != 1) 
                 stop(paste("Internal error:", sum(ok), "roots of polynomial"))
               yy[i] <- realpart[ok]
             }
             yy
           },
           cosine = {
             # solve using `uniroot'
             g <- function(y, pval) { (y + sin(pi * y)/pi + 1)/2 - pval }
             yy <- numeric(n <- length(p))
             yy[p == 0] <- -1
             yy[p == 1] <-  1
             inside <- (p != 0) & (p != 1)
             for(i in seq(n)[inside]) 
               yy[i] <- uniroot(g, c(-1,1), pval=p[i])$root
             yy
           },
           optcosine = {
             (2/pi) * asin(2 * p - 1)
           })
  return(mean + a * y)
}

#' integral of t^m k(t) dt from -Inf to r
#'   where k(t) is the standard kernel with support [-1,1]
#' was:    nukernel(r, m, kernel)

kernel.moment <- local({

  kernel.moment <- function(m, r, kernel="gaussian") {
    ker <- match.kernel(kernel)
    check.1.integer(m)
    #' restrict to support
    if(ker != "gaussian") {
      r <- pmin(r, 1)
      r <- pmax(r, -1)
    }
    if(!(m %in% c(0,1,2)) || (ker %in% c("cosine", "optcosine"))) {
      ## use generic integration
      neginf <- if(ker == "gaussian") -10 else -1
      result <- numeric(length(r))
      for(i in seq_along(r))
        result[i] <- integralvalue(kintegrand,
                                   lower=neginf, upper=r[i],
                                   m=m, ker=ker)
      return(result)
    }
    switch(ker,
           gaussian={
             if(m == 0) return(pnorm(r)) else
             if(m == 1) return(-dnorm(r)) else
             return(pnorm(r) - r * dnorm(r))
           },
           rectangular = {
             if(m == 0) return((r + 1)/2) else
             if(m == 1) return((r^2 - 1)/4) else
             return((r^3 + 1)/6)
           },
           triangular={
             m1 <- m+1
             m2 <- m+2
             const <- ((-1)^m1)/m1 + ((-1)^m2)/m2
             answer <- (r^m1)/m1 + ifelse(r < 0, 1, -1) * (r^m2)/m2 - const
             return(answer)
           },
           epanechnikov = {
             if(m == 0)
               return((2 + 3*r - r^3)/4)
             else if(m == 1)
               return((-3 + 6*r^2 - 3*r^4)/16)
             else
               return(( 2 + 5*r^3  - 3* r^5)/20)
           },
           biweight = {
             if(m == 0)
               return((3*r^5 - 10*r^3 + 15*r + 8)/16)
             else if(m == 1)
               return((5*r^6 - 15*r^4 + 15*r^2 -5)/32)
             else 
               return((15*r^7 - 42*r^5 + 35*r^3 + 8)/112)
           },
           # never reached!
           cosine={stop("Sorry, not yet implemented for cosine kernel")},
           optcosine={stop("Sorry, not yet implemented for optcosine kernel")}
           )
  }

  integralvalue <- function(...) integrate(...)$value
  
  kintegrand <- function(x, m, ker) {
    (x^m) * dkernel(x, ker, mean=0, sd=1/kernel.factor(ker))
  }

  kernel.moment
})

kernel.squint <- function(kernel="gaussian", bw=1) {
  kernel <- match.kernel(kernel)
  check.1.real(bw)
  RK <- switch(kernel,
               gaussian = 1/(2 * sqrt(pi)),
               rectangular = sqrt(3)/6, 
               triangular = sqrt(6)/9,
               epanechnikov = 3/(5 * sqrt(5)), 
               biweight = 5 * sqrt(7)/49,
               cosine = 3/4 * sqrt(1/3 - 2/pi^2),
               optcosine = sqrt(1 - 8/pi^2) * pi^2/16)
  return(RK/bw)
}

#
#           pixellate.R
#
#           $Revision: 1.25 $    $Date: 2017/11/15 07:23:16 $
#
#     pixellate            convert an object to a pixel image
#
#     pixellate.ppp        convert a point pattern to a pixel image
#                          (pixel value = number of points in pixel)
#
#     pixellate.owin       convert a window to a pixel image
#                          (pixel value = area of intersection with pixel)
#

pixellate <- function(x, ...) {
  UseMethod("pixellate")
}

pixellate.ppp <- function(x, W=NULL, ..., weights=NULL, padzero=FALSE,
                          fractional=FALSE, preserve=FALSE,
                          DivideByPixelArea=FALSE, savemap=FALSE) {
  verifyclass(x, "ppp")

  if(is.null(W))
    W <- Window(x)
  isrect <- is.rectangle(W)
  preserve <- preserve && !isrect
  iscount <- is.null(weights) && !fractional && !preserve
  
  W <- do.call.matched(as.mask,
                       resolve.defaults(list(...),
                                        list(w=W)))

  nx <- npoints(x)
  
  insideW <- W$m
  dimW   <- W$dim
  nr <- dimW[1L]
  nc <- dimW[2L]
  xcolW <- W$xcol
  yrowW <- W$yrow
  xrangeW <- W$xrange
  yrangeW <- W$yrange
  unitsW <- unitname(W)
  
  # multiple columns of weights?
  if(is.data.frame(weights) || is.matrix(weights)) {
    k <- ncol(weights)
    stopifnot(nrow(weights) == npoints(x))
    weights <- if(k == 1) as.vector(weights) else as.data.frame(weights)
  } else {
    k <- 1
    if(length(weights) == 0) weights <- NULL else 
      stopifnot(length(weights) == npoints(x) || length(weights) == 1)
    if(length(weights) == 1)
      weights <- rep(weights, npoints(x))
  }

  # handle empty point pattern
  if(nx == 0) {
    zerovalue <- if(iscount) 0L else as.double(0)
    zeroimage <- as.im(zerovalue, W)
    if(padzero) # map NA to 0
      zeroimage <- na.handle.im(zeroimage, zerovalue)
    result <- zeroimage
    if(k > 1) {
      result <- as.solist(rep(list(zeroimage), k))
      names(result) <- colnames(weights)
    }
    return(result)
  }

  # map points to pixels 
  xx <- x$x
  yy <- x$y
  if(!fractional) {
    #' map (x,y) to nearest raster point
    pixels <- if(preserve) nearest.valid.pixel(xx, yy, W) else 
              nearest.raster.point(xx, yy, W)
    rowfac <- factor(pixels$row, levels=1:nr)
    colfac <- factor(pixels$col, levels=1:nc)
  } else {
    #' attribute fractional weights to the 4 pixel centres surrounding (x,y)
    #' find surrounding pixel centres
    jj <- findInterval(xx, xcolW, rightmost.closed=TRUE)
    ii <- findInterval(yy, yrowW, rightmost.closed=TRUE)
    jleft <- pmax(jj, 1)
    jright <- pmin(jj + 1, nr) 
    ibot <- pmax(ii, 1)
    itop <- pmin(ii+1, nc)
    #' compute fractional weights
    wleft <- pmin(1, abs(xcolW[jright] - xx)/W$xstep)
    wright <- 1 - wleft
    wbot <- pmin(1, abs(yrowW[itop] - yy)/W$ystep)
    wtop <- 1 - wbot
    #' pack together
    ww <- c(wleft * wbot, wleft * wtop, wright * wbot, wright * wtop)
    rowfac <- factor(c(ibot, itop, ibot, itop), levels=1:nr)
    colfac <- factor(c(jleft, jleft, jright, jright), levels=1:nc)
    if(preserve) {
      #' normalise fractions for each data point to sum to 1 inside window
      ok <- insideW[cbind(as.integer(rowfac), as.integer(colfac))]
      wwok <- ww * ok
      denom <- .colSums(wwok, 4, nx, na.rm=TRUE)
      recip <- ifelse(denom == 0, 1, 1/denom)
      ww <- wwok * rep(recip, each=4)
    }
    #' data weights must be replicated
    if(is.null(weights)) {
      weights <- ww
    } else if(k == 1) {
      weights <- ww * rep(weights, 4)
    } else {
      weights <- ww * apply(weights, 2, rep, times=4)
    }
  }
  
  #' sum weights
  if(is.null(weights)) {
    ta <- table(row = rowfac, col = colfac)
  } else if(k == 1) {
    ta <- tapplysum(weights, list(row = rowfac, col=colfac))
  } else {
    ta <- list()
    for(j in 1:k) {
      ta[[j]] <- tapplysum(weights[,j], list(row = rowfac, col=colfac))
    }
  }

  #' divide by pixel area?
  if(DivideByPixelArea) {
    pixelarea <- W$xstep * W$ystep
    if(k == 1) {
      ta <- ta/pixelarea
    } else {
      ta <- lapply(ta, "/", e2=pixelarea)
    }
  }

  # pack up as image(s)
  if(k == 1) {
    # single image
    # clip to window of data
    if(!padzero)
      ta[!insideW] <- NA
    out <- im(ta,
              xcol = xcolW, yrow = yrowW,
              xrange = xrangeW, yrange = yrangeW,
              unitname=unitsW)
  } else {
    # case k > 1
    # create template image to reduce overhead
    template <- im(ta[[1L]],
                   xcol = xcolW, yrow = yrowW,
                   xrange = xrangeW, yrange = yrangeW,
                   unitname=unitsW)
    out <- list()
    for(j in 1:k) {
      taj <- ta[[j]]
      # clip to window of data
      if(!padzero) 
        taj[!insideW] <- NA
      # copy template and reassign pixel values
      outj <- template
      outj$v <- taj
      # store
      out[[j]] <- outj
    }
    out <- as.solist(out)
    names(out) <- names(weights)
  }
  if(savemap)
    attr(out, "map") <- cbind(row=as.integer(rowfac),
                              col=as.integer(colfac))
  return(out)
}

pixellate.owin <- function(x, W=NULL, ..., DivideByPixelArea=FALSE) {
  stopifnot(is.owin(x))
  P <- as.polygonal(x)
  R <- as.rectangle(x)
  if(is.null(W)) 
    W <- R
  else if(!is.subset.owin(R, as.rectangle(W)))
    stop("W does not cover the domain of x")
  
  W <- do.call.matched(as.mask,
                       resolve.defaults(list(...),
                                        list(w=W)))
  ## compute
  Zmat <- polytileareaEngine(P, W$xrange, W$yrange, nx=W$dim[2L], ny=W$dim[1L],
                             DivideByPixelArea)
  ## convert to image
  Z <- im(Zmat, xcol=W$xcol, yrow=W$yrow, xrange=W$xrange, yrange=W$yrange,
          unitname=unitname(W))
  return(Z)
}

polytileareaEngine <- function(P, xrange, yrange, nx, ny,
                               DivideByPixelArea=FALSE) {
  x0 <- xrange[1L]
  y0 <- yrange[1L]
  dx <- diff(xrange)/nx
  dy <- diff(yrange)/ny
  # process each component polygon
  Z <- matrix(0.0, ny, nx)
  B <- P$bdry
  for(i in seq_along(B)) {
    PP <- B[[i]]
    # transform so that pixels become unit squares
    QQ <- affinexypolygon(PP, vec = c(-x0, -y0))
    RR <- affinexypolygon(QQ, mat = diag(1/c(dx, dy)))
    # 
    xx <- RR$x
    yy <- RR$y
    nn <- length(xx)
    # close polygon
    xx <- c(xx, xx[1L])
    yy <- c(yy, yy[1L])
    nn <- nn+1
    # call C routine
    zz <- .C("poly2imA",
             ncol=as.integer(nx),
             nrow=as.integer(ny),
             xpoly=as.double(xx),
             ypoly=as.double(yy),
             npoly=as.integer(nn),
             out=as.double(numeric(nx * ny)),
             status=as.integer(integer(1L)),
             PACKAGE = "spatstat")
    if(zz$status != 0)
      stop("Internal error")
    # increment output 
    Z[] <- Z[] + zz$out
  }
  if(!DivideByPixelArea) {
    #' revert to original scale
    pixelarea <- dx * dy
    Z <- Z * pixelarea
  }
  return(Z)
}


  

#
#    as.im.R
#
#    conversion to class "im"
#
#    $Revision: 1.58 $   $Date: 2020/01/21 04:50:04 $
#
#    as.im()
#

as.im <- function(X, ...) {
  UseMethod("as.im")
}

as.im.im <- function(X, W=NULL, ...,
                     eps=NULL, dimyx=NULL, xy=NULL,
                     na.replace=NULL) {
  X <- repair.old.factor.image(X)
  nopar <- is.null(eps) && is.null(dimyx) && is.null(xy)
  if(is.null(W)) {
    if(nopar) {
      X <- repair.image.xycoords(X)
      X <- na.handle.im(X, na.replace)
      return(X)
    }
    # pixel raster determined by dimyx etc
    W <- as.mask(as.rectangle(X), eps=eps, dimyx=dimyx, xy=xy)
    # invoke as.im.owin
    Y <- as.im(W)
  } else if(is.mask(W) || is.im(W) || !nopar) {
    #' raster information is present in { W, eps, dimyx, xy }
    Y <- as.im(W, eps=eps, dimyx=dimyx, xy=xy)
  } else {
    #' use existing raster information in X
    return(X[W, drop=FALSE, tight=TRUE])
  }
  # resample X onto raster of Y
  Y <- rastersample(X, Y)
  return(na.handle.im(Y, na.replace))
}

as.im.owin <- function(X, W=NULL, ...,
                       eps=NULL, dimyx=NULL, xy=NULL,
                       na.replace=NULL, value=1) {
  if(!(is.null(eps) && is.null(dimyx) && is.null(xy))) {
    # raster dimensions determined by dimyx etc
    # convert X to a mask 
    M <- as.mask(X, eps=eps, dimyx=dimyx, xy=xy)
    # convert mask to image
    d <- M$dim
    v <- matrix(value, d[1L], d[2L])
    m <- M$m
    v[!m] <- if(is.null(na.replace)) NA else na.replace
    out <- im(v, M$xcol, M$yrow,
              xrange=M$xrange, yrange=M$yrange,
              unitname=unitname(X))
    return(out)
  }
  if(!is.null(W) && is.owin(W) && W$type == "mask") {
    # raster dimensions determined by W
    # convert W to zero image
    d <- W$dim
    Z <- im(matrix(0, d[1L], d[2L]), W$xcol, W$yrow, unitname=unitname(X))    
    # adjust values to indicator of X
    Z[X] <- 1
    if(missing(value) && is.null(na.replace)) {
      # done
      out <- Z
    } else {
      # map {0, 1} to {na.replace, value}
      v <- matrix(ifelseAB(Z$v == 0, na.replace, value), d[1L], d[2L])
      out <- im(v, W$xcol, W$yrow, unitname=unitname(X))
    }
    return(out)
  }
  if(X$type == "mask") {
    # raster dimensions determined by X
    # convert X to image
    d <- X$dim
    v <- matrix(value, d[1L], d[2L])
    m <- X$m
    v[!m] <- if(is.null(na.replace)) NA else na.replace
    out <- im(v, xcol=X$xcol, yrow=X$yrow,
              xrange=X$xrange, yrange=X$yrange, unitname=unitname(X))
    return(out)
  }
  # X is not a mask.
  # W is either missing, or is not a mask.
  # Convert X to a image using default settings
  M <- as.mask(X)
  # convert mask to image
  d <- M$dim
  v <- matrix(value, d[1L], d[2L])
  m <- M$m
  v[!m] <- if(is.null(na.replace)) NA else na.replace
  out <- im(v, M$xcol, M$yrow, unitname=unitname(X))
  return(out)
}

as.im.funxy <- function(X, W=Window(X), ...) {
  as.im.function(X, W=W, ...)
}

as.im.function <- function(X, W=NULL, ...,
                           eps=NULL, dimyx=NULL, xy=NULL,
                           na.replace=NULL,
                           stringsAsFactors=default.stringsAsFactors(),
                           strict=FALSE, drop=TRUE) {
  f <- X
  if(is.null(W))
    stop("A window W is required")
  W <- as.owin(W)
  W <- as.mask(W, eps=eps, dimyx=dimyx, xy=xy)
  m <- W$m
  funnywindow <- !all(m)

  xx <- as.vector(rasterx.mask(W))
  yy <- as.vector(rastery.mask(W))

  argh <- list(...)
  if(strict) argh <- argh[names(argh) %in% names(formals(f))]

  #' evaluate function value at each pixel 
  if(!funnywindow) {
    values <- do.call(f, append(list(xx, yy), argh))
    slices <- as.list(as.data.frame(values,
                                    stringsAsFactors=stringsAsFactors))
    ns <- length(slices)
  } else {
    #' evaluate only inside window
    inside <- as.vector(m)
    values.inside <- do.call(f, append(list(xx[inside], yy[inside]), argh))
    slices.inside <- as.list(as.data.frame(values.inside,
                                           stringsAsFactors=stringsAsFactors))
    ns <- length(slices.inside)
    #' pad out
    msize <- length(m)
    slices <- vector(mode="list", length=ns)
    for(i in seq_len(ns)) {
      slice.inside.i <- slices.inside[[i]]
      #' create space for full matrix
      slice.i <- vector(mode=typeof(slice.inside.i), length=msize)
      if(is.factor(slice.inside.i))
        slice.i <- factor(slice.i, levels=levels(slice.inside.i))
      #' copy values, assigning NA outside window
      slice.i[inside] <- slice.inside.i
      slice.i[!inside] <- NA
      #'
      slices[[i]] <- slice.i
    }
  }

  outlist <- vector(mode="list", length=ns)
  nc <- length(W$xcol)
  nr <- length(W$yrow)
  for(i in seq_len(ns)) {
    if(nr == 1 || nc == 1) {
      #' exception: can't determine pixel width/height from centres
      mat.i <- matrix(slices[[i]], nr, nc)
      levels(mat.i) <- levels(slices[[i]])
      out.i <- im(mat.i, xrange=W$xrange, yrange=W$yrange, unitname=unitname(W))
    } else {
      out.i <- im(slices[[i]], W$xcol, W$yrow, unitname=unitname(W))
    }
    outlist[[i]] <- na.handle.im(out.i, na.replace)
  }
  if(ns == 1 && drop) return(outlist[[1L]])
  return(as.imlist(outlist)) 
}

as.im.expression <- function(X, W=NULL, ...) {
  e <- parent.frame()
  f <- function(x,y, ...) eval(X, envir=list(x=x, y=y), enclos=e)
  as.im(f, W=W, ...)
}

as.im.matrix <- function(X, W=NULL, ...) {
  nr <- nrow(X)
  nc <- ncol(X)
  if(is.null(W))
    return(im(X, ...))
  W <- as.owin(W)
  if(W$type == "mask") {
    xcol <- W$xcol
    yrow <- W$yrow
    # pixel coordinate information
    if(length(xcol) == nc && length(yrow) == nr)
      return(im(X, xcol, yrow, unitname=unitname(W)))
  }
  # range information
  R <- as.rectangle(W)
  xrange <- R$xrange
  yrange <- R$yrange
  return(im(X, xrange=xrange, yrange=yrange, unitname=unitname(W)))
}

as.im.default <- function(X, W=NULL, ...,
                          eps=NULL, dimyx=NULL, xy=NULL,
                          na.replace=NULL) {

  if((is.vector(X) || is.factor(X)) && length(X) == 1) {
    # numerical value: interpret as constant function
    xvalue <- X
    X <- function(xx, yy, ...) { rep.int(xvalue, length(xx)) }
    return(as.im(X, W, ..., eps=eps, dimyx=dimyx, xy=xy, na.replace=na.replace))
  }
  
  if(is.list(X) && checkfields(X, c("x","y","z"))) {
    stopifnot(is.matrix(X$z))
    z <- X$z
    y <- X$y
    x <- X$x
    # Usual S convention as in contour.default() and image.default()
    # Rows of z correspond to x values.
    nr <- nrow(z)
    nc <- ncol(z)
    lx <- length(x)
    ly <- length(y)
    if(lx == nr + 1)
      x <- (x[-1L] + x[-lx])/2
    else if(lx != nr)
      stop("length of x coordinate vector does not match number of rows of z")
    if(ly == nc + 1)
      y <- (y[-1L] + y[-ly])/2
    else if(ly != nc)
      stop("length of y coordinate vector does not match number of columns of z")
    # convert to class "im"
    out <- im(t(z), x, y)
    # now apply W and dimyx if present
    if(is.null(W) && !(is.null(eps) && is.null(dimyx) && is.null(xy)))
      out <- as.im(out, eps=eps, dimyx=dimyx, xy=xy)
    else if(!is.null(W))
      out <- as.im(out, W=W, eps=eps, dimyx=dimyx, xy=xy)
    return(na.handle.im(out, na.replace))
  }
  stop("Can't convert X to a pixel image")
}

as.im.ppp <- function(X, ...) {
  pixellate(X, ..., weights=NULL, zeropad=FALSE)
}

as.im.data.frame <- function(X, ..., step, fatal=TRUE, drop=TRUE) {
  if(missing(step)) {
    xstep <- ystep <- NULL
  } else {
    step <- ensure2vector(step)
    xstep <- step[1L]
    ystep <- step[2L]
  }
  if(ncol(X) < 3) {
    whinge <- "Argument 'X' must have at least 3 columns of data"
    if(fatal) stop(whinge)
    warning(whinge)
    return(NULL)
  }
  ## extract (x,y) coordinates
  mch <- matchNameOrPosition(c("x", "y", "z"), names(X))
  x <- X[, mch[1L]]
  y <- X[, mch[2L]]
  z <- X[, -mch[1:2], drop=FALSE]
  ## unique x,y coordinates
  xx <- sortunique(x)
  yy <- sortunique(y)
  jj <- match(x, xx)
  ii <- match(y, yy)
  iijj <- cbind(ii, jj)
  ## make matrix (for incomplete x, y sequence)
  ok <- checkbigmatrix(length(xx), length(yy), fatal=fatal)
  if(!ok) return(NULL)
  mm <- matrix(NA, length(yy), length(xx))
  ## ensure xx and yy are complete equally-spaced sequences
  fx <- fillseq(xx, step=xstep)
  fy <- fillseq(yy, step=ystep)
  xcol <- fx[[1L]]
  yrow <- fy[[1L]]
  ## trap very large matrices
  ok <- checkbigmatrix(length(xcol), length(yrow), fatal=fatal)
  if(!ok) return(NULL)
  ## mapping from xx to xcol, yy to yrow
  jjj <- fx[[2L]]
  iii <- fy[[2L]]
  ## make matrix for full sequence
  m <- matrix(NA, length(yrow), length(xcol))
  ## run through columns of pixel values
  nz <- ncol(z)
  result <- vector(mode="list", length=nz)
  names(result) <- colnames(z)
  for(k in seq_len(nz)) {
    zk <- z[,k]
    mm[] <- RNA <- RelevantNA(zk)
    mm[iijj] <- zk
    m[] <- RNA 
    m[iii,jjj] <- mm
    lev <- levels(zk)
    mo <- if(is.null(lev)) m else factor(m, levels=seq_along(lev), labels=lev)
    result[[k]] <- im(mat=mo, xcol=xcol, yrow=yrow) 
  }
  if(nz == 1 && drop) result <- result[[1L]]
  return(result)
}


# convert to image from some other format, then do something

do.as.im <- function(x, action, ...,
                     W = NULL, eps = NULL, dimyx = NULL, xy = NULL, 
                     na.replace = NULL) {
  Z <- as.im(x, W=W, eps=eps, dimyx=dimyx, xy=xy, na.replace=na.replace)
  Y <- do.call(action, list(Z, ...))
  return(Y)
}

na.handle.im <- function(X, na.replace) {
  if(is.null(na.replace))
    return(X)
  if(length(na.replace) != 1)
    stop("na.replace should be a single value")
  if(X$type == "factor") {
    lev <- levels(X)
    newlev <- union(lev, na.replace)
    if(length(newlev) > length(lev))
      levels(X) <- newlev
  }
  X$v[is.na(X$v)] <- na.replace
  return(X)
}

repair.old.factor.image <- function(x) {
  # convert from old to new representation of factor images
  if(x$type != "factor")
    return(x)
  v <- x$v
  isold <- !is.null(lev <- attr(x, "levels"))
  isnew <- is.factor(v) && is.matrix(v)
  if(isnew)
    return(x)
  if(!isold)
    stop("Internal error: unrecognised format for factor-valued image")
  v <- factor(v, levels=lev)
  dim(v) <- x$dim
  x$v <- v
  return(x)
}

repair.image.xycoords <- function(x) {
  v <- x$v
  if(is.null(dim(v))) 
    dim(v) <- c(length(x$yrow), length(x$xcol))
  im(v, xrange=x$xrange, yrange=x$yrange, unitname=unitname(x))
}

#'
#'  rhohat.R
#'
#'  $Revision: 1.86 $  $Date: 2020/03/16 10:28:51 $
#'
#'  Non-parametric estimation of a transformation rho(z) determining
#'  the intensity function lambda(u) of a point process in terms of a
#'  spatial covariate Z(u) through lambda(u) = rho(Z(u)).
#'  More generally allows offsets etc.

#' Copyright (c) Adrian Baddeley 2015-2019
#' GNU Public Licence GPL >= 2.0

rhohat <- function(object, covariate, ...) {
  UseMethod("rhohat")
}

rhohat.ppp <- rhohat.quad <- 
  function(object, covariate, ...,
           baseline=NULL, weights=NULL,
           method=c("ratio", "reweight", "transform"),
           horvitz=FALSE,
           smoother=c("kernel", "local", "decreasing", "increasing"),
           subset=NULL,
           dimyx=NULL, eps=NULL,
           n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
           bwref=bw, covname, confidence=0.95, positiveCI) {
  callstring <- short.deparse(sys.call())
  smoother <- match.arg(smoother)
  method <- match.arg(method)
  if(missing(positiveCI))
    positiveCI <- (smoother == "local")
  if(missing(covname)) 
    covname <- sensiblevarname(short.deparse(substitute(covariate)), "X")
  if(is.null(adjust))
    adjust <- 1
  # validate model
  if(is.null(baseline)) {
    model <- ppm(object ~1)
    reference <- "Lebesgue"
  } else {
    model <- ppm(object ~ offset(log(baseline)))
    reference <- "baseline"
  } 
  modelcall <- NULL

  if(is.character(covariate) && length(covariate) == 1) {
    covname <- covariate
    switch(covname,
           x={
             covariate <- function(x,y) { x }
           }, 
           y={
             covariate <- function(x,y) { y }
           },
           stop("Unrecognised covariate name")
         )
    covunits <- unitname(data.ppm(model))
  } else {
    covunits <- NULL
  }

  W <- Window(data.ppm(model))
  if(!is.null(subset)) W <- W[subset, drop=FALSE]
  areaW <- area(W)
  
  rhohatEngine(model, covariate, reference, areaW, ..., 
               subset=subset,
               weights=weights,
               method=method,
               horvitz=horvitz,
               smoother=smoother,
               resolution=list(dimyx=dimyx, eps=eps),
               n=n, bw=bw, adjust=adjust, from=from, to=to,
               bwref=bwref, covname=covname, covunits=covunits,
               confidence=confidence,
               positiveCI=positiveCI, 
               modelcall=modelcall, callstring=callstring)
}

rhohat.ppm <- function(object, covariate, ...,
                       weights=NULL,
                       method=c("ratio", "reweight", "transform"),
                       horvitz=FALSE,
                       smoother=c("kernel", "local", "decreasing", "increasing"),
                       subset=NULL,
                       dimyx=NULL, eps=NULL,
                       n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
                       bwref=bw, covname, confidence=0.95,
                       positiveCI) {
  callstring <- short.deparse(sys.call())
  smoother <- match.arg(smoother)
  method <- match.arg(method)
  if(missing(positiveCI))
    positiveCI <- (smoother == "local")
  if(missing(covname)) 
    covname <- sensiblevarname(short.deparse(substitute(covariate)), "X")
  if(is.null(adjust))
    adjust <- 1

  if("baseline" %in% names(list(...)))
    warning("Argument 'baseline' ignored: not available for rhohat.ppm")

  ## validate model
  model <- object
  reference <- "model"
  modelcall <- model$call

  if(is.character(covariate) && length(covariate) == 1) {
    covname <- covariate
    switch(covname,
           x={
             covariate <- function(x,y) { x }
           }, 
           y={
             covariate <- function(x,y) { y }
           },
           stop("Unrecognised covariate name")
         )
    covunits <- unitname(data.ppm(model))
  } else {
    covunits <- NULL
  }

  W <- Window(data.ppm(model))
  if(!is.null(subset)) W <- W[subset, drop=FALSE]
  areaW <- area(W)
  
  rhohatEngine(model, covariate, reference, areaW, ...,
               weights=weights,
               method=method,
               horvitz=horvitz,
               smoother=smoother,
               resolution=list(dimyx=dimyx, eps=eps),
               n=n, bw=bw, adjust=adjust, from=from, to=to,
               bwref=bwref, covname=covname, covunits=covunits,
               confidence=confidence, positiveCI=positiveCI,
               modelcall=modelcall, callstring=callstring)
}

rhohat.lpp <- rhohat.lppm <- 
  function(object, covariate, ...,
           weights=NULL,
           method=c("ratio", "reweight", "transform"),
           horvitz=FALSE,
           smoother=c("kernel", "local", "decreasing", "increasing"),
           subset=NULL,
           nd=1000, eps=NULL, random=TRUE, 
           n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
           bwref=bw, covname, confidence=0.95, positiveCI) {
  callstring <- short.deparse(sys.call())
  smoother <- match.arg(smoother)
  method <- match.arg(method)
  if(missing(positiveCI))
    positiveCI <- (smoother == "local")
  if(missing(covname)) 
    covname <- sensiblevarname(short.deparse(substitute(covariate)), "X")
  if(is.null(adjust))
    adjust <- 1
  # validate model
  if(is.lpp(object)) {
    X <- object
    model <- lppm(object, ~1, eps=eps, nd=nd, random=random)
    reference <- "Lebesgue"
    modelcall <- NULL
  } else if(inherits(object, "lppm")) {
    model <- object
    X <- model$X
    reference <- "model"
    modelcall <- model$call
  } else stop("object should be of class lpp or lppm")
  
  if("baseline" %in% names(list(...)))
    warning("Argument 'baseline' ignored: not available for ",
            if(is.lpp(object)) "rhohat.lpp" else "rhohat.lppm")

  if(is.character(covariate) && length(covariate) == 1) {
    covname <- covariate
    switch(covname,
           x={
             covariate <- function(x,y) { x }
           }, 
           y={
             covariate <- function(x,y) { y }
           },
           stop("Unrecognised covariate name")
         )
    covunits <- unitname(X)
  } else {
    covunits <- NULL
  }

  S <- as.psp(as.linnet(X))
  if(!is.null(subset)) S <- S[subset]
  totlen <- sum(lengths_psp(S))
  
  rhohatEngine(model, covariate, reference, totlen, ...,
               subset=subset,
               weights=weights,
               method=method,
               horvitz=horvitz,
               smoother=smoother,
               resolution=list(nd=nd, eps=eps, random=random),
               n=n, bw=bw, adjust=adjust, from=from, to=to,
               bwref=bwref, covname=covname, covunits=covunits,
               confidence=confidence, positiveCI=positiveCI,
               modelcall=modelcall, callstring=callstring)
}

rhohatEngine <- function(model, covariate,
                         reference=c("Lebesgue", "model", "baseline"),
                         volume,
                         ...,
                         subset=NULL,
                         weights=NULL,
                         method=c("ratio", "reweight", "transform"),
                         horvitz=FALSE,
                         smoother=c("kernel", "local", "decreasing", "increasing"),
                         resolution=list(), 
                         n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
                         bwref=bw, covname, covunits=NULL, confidence=0.95,
                         modelcall=NULL, callstring="rhohat") {
  reference <- match.arg(reference)
  # evaluate the covariate at data points and at pixels
  stuff <- do.call(evalCovar,
                   append(list(model=model,
                               covariate=covariate,
                               subset=subset),
                          resolution))
  # unpack
  values <- stuff$values
  # values at each data point
  ZX      <- values$ZX
  lambdaX <- values$lambdaX
  # values at each pixel
  Zimage      <- values$Zimage
  lambdaimage <- values$lambdaimage # could be multiple images
  # values at each pixel (for .ppp, .ppm) or quadrature point (for .lpp, .lppm)
  Zvalues <- values$Zvalues
  lambda  <- values$lambda
    ## weights
  if(!is.null(weights)) {
    X <- data.ppm(model)
    if(is.im(weights)) 
      weights <- safelookup(weights, X)
    else if(is.function(weights))
      weights <- weights(X$x, X$y)
    else if(is.numeric(weights) && is.vector(as.numeric(weights))) 
      check.nvector(weights, npoints(X))
    else stop(paste(sQuote("weights"),
                    "should be a vector, a pixel image, or a function"))
  }
  # normalising constants
  denom <- volume * (if(reference == "Lebesgue" || horvitz) 1 else mean(lambda))
  # info 
  savestuff <- list(reference   = reference,
                    horvitz     = horvitz,
                    Zimage      = Zimage,
                    lambdaimage = lambdaimage)
  # calculate rho-hat
  result <- rhohatCalc(ZX, Zvalues, lambda, denom,
                       ...,
                       weights=weights,
                       lambdaX=lambdaX,
                       method=method,
                       horvitz=horvitz,
                       smoother=smoother,
                       n=n, bw=bw, adjust=adjust, from=from, to=to,
                       bwref=bwref, covname=covname, confidence=confidence,
                       covunits=covunits,
                       modelcall=modelcall, callstring=callstring,
                       savestuff=savestuff)
  return(result)
}


# basic calculation of rhohat from covariate values

rhohatCalc <- local({
  
  interpolate <- function(x,y) {
    if(inherits(x, "density") && missing(y))
      approxfun(x$x, x$y, rule=2)
    else 
      approxfun(x, y, rule=2)
  }

  ## note: this function normalises the weights, like density.default
  LocfitRaw <- function(x, ..., weights=NULL) {
    if(is.null(weights)) weights <- 1
    requireNamespace("locfit", quietly=TRUE)
    do.call.matched(locfit::locfit.raw,
                    append(list(x=x, weights=weights), list(...)))
  }

  varlog <- function(obj,xx) {
    ## variance of log f-hat
    stopifnot(inherits(obj, "locfit"))
    if(!identical(obj$trans, exp))
      stop("internal error: locfit object does not have log link")
    ## the following call should have band="local" but that produces NaN's
    pred <- predict(obj, newdata=xx,
                    se.fit=TRUE, what="coef")
    se <- pred$se.fit
    return(se^2)
  }

  rhohatCalc <- function(ZX, Zvalues, lambda, denom, ...,
                         weights=NULL, lambdaX,
                         method=c("ratio", "reweight", "transform"),
                         horvitz=FALSE, 
                         smoother=c("kernel", "local", "decreasing", "increasing"),
                         n=512, bw="nrd0", adjust=1, from=NULL, to=NULL, 
                         bwref=bw, covname, confidence=0.95,
                         positiveCI=(smoother == "local"),
                         markovCI=TRUE,
                         covunits = NULL, modelcall=NULL, callstring=NULL,
                         savestuff=list()) {
    method <- match.arg(method)
    smoother <- match.arg(smoother)
    ## check availability of locfit package
    if(smoother == "local" && !requireNamespace("locfit", quietly=TRUE)) {
      warning(paste("In", paste0(dQuote(callstring), ":"),
                    "package", sQuote("locfit"), "is not available;",
                    "unable to perform local likelihood smoothing;",
                    "using kernel smoothing instead"),
              call.=FALSE)
      smoother <- "kernel"
    }
    ## validate
    stopifnot(is.numeric(ZX))
    stopifnot(is.numeric(Zvalues))
    stopifnot(is.numeric(lambda))
    stopifnot(length(lambda) == length(Zvalues))
    stopifnot(all(is.finite(lambda))) 
    check.1.real(denom)
    ## 
    if(horvitz) {
      ## data points will be weighted by reciprocal of model intensity
      weights <- (weights %orifnull% 1)/lambdaX
    }
    ## normalising constants
    nX   <- if(is.null(weights)) length(ZX) else sum(weights)
    kappahat <- nX/denom
    ## limits
    Zrange <- range(ZX, Zvalues)
    if(is.null(from)) from <- Zrange[1] 
    if(is.null(to))   to   <- Zrange[2]
    if(from > Zrange[1] || to < Zrange[2])
      stop(paste("In", paste0(dQuote(callstring), ":"),
                 "interval [from, to] =", prange(c(from,to)), 
                 "does not contain the range of data values =",
                 prange(Zrange)),
           call.=FALSE)
    ## critical constant for CI's
    crit <- qnorm((1+confidence)/2)
    percentage <- paste(round(100 * confidence), "%%", sep="")
    CIblurb <- paste("pointwise", percentage, "confidence interval")
    ## estimate densities
    switch(smoother,
    kernel = {
      ## ............... kernel smoothing ......................
      ## reference density (normalised) for calculation
      ghat <- density(Zvalues,weights=if(horvitz) NULL else lambda/sum(lambda),
                      bw=bwref,adjust=adjust,n=n,from=from,to=to, ...)
      xxx <- ghat$x
      ghatfun <- interpolate(ghat)
      ## relative density
      switch(method,
             ratio={
               ## compute ratio of smoothed densities
               fhat <- unnormdensity(ZX,weights=weights,
                                     bw=bw,adjust=adjust,
                                     n=n,from=from, to=to, ...)
               fhatfun <- interpolate(fhat)
               Ghat.xxx <- denom * ghatfun(xxx)
               yyy <- fhatfun(xxx)/Ghat.xxx
               ## compute variance approximation
               sigma <- fhat$bw
               weights2 <- if(is.null(weights)) NULL else weights^2
               fstar <- unnormdensity(ZX,weights=weights2,
                                      bw=bw,adjust=adjust/sqrt(2),
                                      n=n,from=from, to=to, ...)
               fstarfun <- interpolate(fstar)
               const <- 1/(2 * sigma * sqrt(pi))
               vvv  <- const * fstarfun(xxx)/Ghat.xxx^2
             },
             reweight={
               ## weight Z values by reciprocal of reference
               wt <- (weights %orifnull% 1)/(denom * ghatfun(ZX))
               rhat <- unnormdensity(ZX, weights=wt, bw=bw,adjust=adjust,
                                     n=n,from=from, to=to, ...)
               rhatfun <- interpolate(rhat)
               yyy <- rhatfun(xxx)
               ## compute variance approximation
               sigma <- rhat$bw
               rongstar <- unnormdensity(ZX, weights=wt^2,
                                         bw=bw,adjust=adjust/sqrt(2),
                                         n=n,from=from, to=to, ...)
               rongstarfun <- interpolate(rongstar)
               const <- 1/(2 * sigma * sqrt(pi))
               vvv  <- const * rongstarfun(xxx)
             },
             transform={
               ## probability integral transform
               Gfun <- interpolate(ghat$x, cumsum(ghat$y)/sum(ghat$y))
               GZX <- Gfun(ZX)
               ## smooth density on [0,1]
               qhat <- unnormdensity(GZX,weights=weights,
                                     bw=bw,adjust=adjust,
                                     n=n, from=0, to=1, ...)
               qhatfun <- interpolate(qhat)
               ## edge effect correction
               one <- density(seq(from=0,to=1,length.out=512),
                              bw=qhat$bw, adjust=1,
                              n=n,from=0, to=1, ...)
               onefun <- interpolate(one)
               ## apply to transformed values
               Gxxx <- Gfun(xxx)
               Dxxx <- denom * onefun(Gxxx)
               yyy <- qhatfun(Gxxx)/Dxxx
               ## compute variance approximation
               sigma <- qhat$bw
               weights2 <- if(is.null(weights)) NULL else weights^2
               qstar <- unnormdensity(GZX,weights=weights2,
                                      bw=bw,adjust=adjust/sqrt(2),
                                      n=n,from=0, to=1, ...)
               qstarfun <- interpolate(qstar)
               const <- 1/(2 * sigma * sqrt(pi))
               vvv  <- const * qstarfun(Gxxx)/Dxxx^2
             })
      vvvname <- "Variance of estimator"
      vvvlabel <- paste("bold(Var)~hat(%s)", paren(covname), sep="")
      sd <- sqrt(vvv)
      if(!positiveCI) {
        hi <- yyy + crit * sd
        lo <- yyy - crit * sd
      } else {
        sdlog <- ifelse(yyy > 0, sd/yyy, 0)
        sss <- exp(crit * sdlog)
        hi <- yyy * sss
        lo <- yyy / sss
        if(markovCI) {
          ## truncate extremely large confidence intervals
          ## using Markov's Inequality
          hi <- pmin(hi, yyy/(1-confidence))
        }
      }
    },
    local = {
      ## .................. local likelihood smoothing .......................
      xlim <- c(from, to)
      xxx <- seq(from, to, length=n)
      ## reference density
      ghat <- LocfitRaw(Zvalues,
                        weights=if(horvitz) NULL else lambda,
                        xlim=xlim, ...)
      ggg <- predict(ghat, xxx)
      ## relative density
      switch(method,
             ratio={
               ## compute ratio of smoothed densities
               fhat <- LocfitRaw(ZX, weights=weights, xlim=xlim, ...)
               fff <- predict(fhat, xxx)
               yyy <- kappahat * fff/ggg
               ## compute approximation to variance of log rho-hat
               varlogN <- 1/nX
               vvv <- varlog(fhat, xxx) + varlogN
             },
             reweight={
               ## weight Z values by reciprocal of reference
               wt <- (weights %orifnull% 1)/(denom * predict(ghat,ZX))
               sumwt <- sum(wt)
               rhat <- LocfitRaw(ZX, weights=wt, xlim=xlim, ...)
               rrr <- predict(rhat, xxx)
               yyy <- sumwt * rrr
               ## compute approximation to variance of log rho-hat
               varsumwt <- mean(yyy /(denom * ggg)) * diff(xlim)
               varlogsumwt <- varsumwt/sumwt^2
               vvv <- varlog(rhat, xxx) + varlogsumwt
             },
             transform={
               ## probability integral transform
               Gfun <- approxfun(xxx, cumsum(ggg)/sum(ggg), rule=2)
               GZX <- Gfun(ZX)
               ## smooth density on [0,1], end effect corrected
               qhat <- LocfitRaw(GZX, weights=weights, xlim=c(0,1), ...)
               ## apply to transformed values
               Gxxx <- Gfun(xxx)
               qqq <- predict(qhat, Gxxx)
               yyy <- kappahat * qqq
               ## compute approximation to variance of log rho-hat
               varlogN <- 1/nX
               vvv <- varlog(qhat, Gxxx) + varlogN
             })
      vvvname <- "Variance of log of estimator"
      vvvlabel <- paste("bold(Var)~log(hat(%s)", paren(covname), ")", sep="")
      sdlog <- sqrt(vvv)
      if(positiveCI) {
        sss <- exp(crit * sdlog)
        hi <- yyy * sss
        lo <- yyy / sss
        if(markovCI) {
          ## truncate extremely large confidence intervals
          ## using Markov's Inequality
          hi <- pmin(hi, yyy/(1-confidence))
        }
      } else {
        hi <- yyy * (1 + crit * sdlog)
        lo <- yyy * (1 - crit * sdlog)
      }
    },
    increasing = ,
    decreasing = {
      ## .................. nonparametric maximum likelihood ............
      if(is.null(weights)) weights <- rep(1, length(ZX))
      #' observed (sorted)
      oX <- order(ZX)
      ZX <- ZX[oX]
      weights <- weights[oX]
      #' reference CDF
      G <- ewcdf(Zvalues, lambda)
      #' reference denominator ('area') at each observed value
      if(smoother == "decreasing") {
        areas <- denom * G(ZX)
      } else {
        areas <- denom * (1 - G(rev(ZX)))
        weights <- rev(weights)
      }
      #' maximum upper sets algorithm
      rho <- numeric(0)
      darea <- diff(c(0, areas))
      dcount <- weights
      while(length(darea) > 0) {
        u <- cumsum(dcount)/cumsum(darea)
        if(any(bad <- !is.finite(u))) # divide by zero etc
          u[bad] <- max(u[!bad], 0)
        k <- which.max(u)
        rho <- c(rho, rep(u[k], k))
        darea <- darea[-(1:k)]
        dcount <- dcount[-(1:k)]
      }
      rho <- c(rho, 0)
      if(smoother == "increasing") rho <- rev(rho)
      #' compute as a stepfun
      rhofun <- stepfun(x = ZX, y=rho, right=TRUE, f=1)
      #' evaluate on a grid
      xlim <- c(from, to)
      xxx <- seq(from, to, length=n)
      yyy <- rhofun(xxx)
      #'
      vvv <- hi <- lo <- NULL
      savestuff$rhofun <- rhofun
    })
    ## pack into fv object
    df <- data.frame(xxx=xxx, rho=yyy)
    names(df)[1] <- covname
    desc <- c(paste("covariate", covname), "Estimated intensity")
    labl <- c(covname, paste("hat(%s)", paren(covname), sep=""))
    if(did.variance <- !is.null(vvv)) {
      df <- cbind(df, data.frame(var=vvv, hi=hi, lo=lo))
      desc <- c(desc,
                vvvname,
                paste("Upper limit of", CIblurb),
                paste("Lower limit of", CIblurb))
      labl <- c(labl,
                vvvlabel,
                paste("%s[hi]", paren(covname), sep=""),
                paste("%s[lo]", paren(covname), sep=""))
    }
    rslt <- fv(df,
               argu=covname,
               ylab=substitute(rho(X), list(X=as.name(covname))),
               valu="rho",
               fmla= as.formula(paste(". ~ ", covname)),
               alim=range(ZX),
               labl=labl,
               desc=desc,
               unitname=covunits,
               fname="rho",
               yexp=substitute(rho(X), list(X=as.name(covname))))
    if(did.variance) {
      fvnames(rslt, ".")  <- c("rho", "hi", "lo")
      fvnames(rslt, ".s") <- c("hi", "lo")
    } else fvnames(rslt, ".")  <- "rho"
    ## pack up
    class(rslt) <- c("rhohat", class(rslt))
    ## add info
    stuff <- 
      list(modelcall  = modelcall, 
           callstring = callstring,
           sigma      = switch(smoother, kernel=sigma, local=NULL),
           covname    = paste(covname, collapse=""),
           ZX         = ZX,
           lambda     = lambda,
           method     = method,
           smoother   = smoother,
           confidence = confidence,
           positiveCI = positiveCI)
    attr(rslt, "stuff") <- append(stuff, savestuff)
    return(rslt)
  }
  
  rhohatCalc
})

## ........... end of 'rhohatCalc' .................................


print.rhohat <- function(x, ...) {
  s <- attr(x, "stuff")
  splat("Intensity function estimate (class rhohat)",
        "for the covariate", s$covname)
  switch(s$reference,
         Lebesgue=splat("Function values are absolute intensities"),
         baseline=splat("Function values are relative to baseline"),
         model={
           splat("Function values are relative to fitted model")
           print(s$modelcall)
         })
  NPMLE <- s$smoother %in% c("increasing", "decreasing")
  cat("Estimation method: ")
  if(NPMLE) splat("nonparametric maximum likelihood") else
  switch(s$method,
         ratio={
           splat("ratio of fixed-bandwidth kernel smoothers")
         },
         reweight={
           splat("fixed-bandwidth kernel smoother of weighted data")
         },
         transform={
           splat("probability integral transform,",
                 "edge-corrected fixed bandwidth kernel smoothing",
                 "on [0,1]")
         },
         cat("UNKNOWN\n"))
  if(identical(s$horvitz, TRUE))
    splat("\twith Horvitz-Thompson weight")
  cat("Smoother: ")
  switch(s$smoother,
         kernel={
           splat("Kernel density estimator")
           splat("\tActual smoothing bandwidth sigma = ",
                 signif(s$sigma,5))
         },
         local = splat("Local likelihood density estimator"),
         increasing = splat("Increasing function of covariate"),
         decreasing = splat("Decreasing function of covariate"),
         splat("UNKNOWN")
         )
  if(!NPMLE) {
    positiveCI <- s$positiveCI %orifnull% (s$smoother == "local")
    confidence <- s$confidence %orifnull% 0.95
    splat("Pointwise", paste0(100 * confidence, "%"),
          "confidence bands for rho(x)\n\t based on asymptotic variance of",
          if(positiveCI) "log(rhohat(x))" else "rhohat(x)")
  }
  splat("Call:", s$callstring)
  cat("\n")
  NextMethod("print")
}

plot.rhohat <- function(x, ..., do.rug=TRUE) {
  xname <- short.deparse(substitute(x))
  s <- attr(x, "stuff")
  covname <- s$covname
  asked.rug <- !missing(do.rug) && identical(rug, TRUE)
  snam <- intersect(c("hi", "lo"), names(x))
  if(length(snam) == 0) snam <- NULL
  out <- do.call(plot.fv,
                 resolve.defaults(list(x=x), list(...),
                                  list(main=xname, shade=snam)))
  if(identical(list(...)$limitsonly, TRUE))
    return(out)
  if(do.rug) {
    rugx <- ZX <- s$ZX
    # check whether it's the default plot
    argh <- list(...)
    isfo <- unlist(lapply(argh, inherits, what="formula"))
    if(any(isfo)) {
      # a plot formula was given; inspect RHS
      fmla <- argh[[min(which(isfo))]]
      rhs <- rhs.of.formula(fmla)
      vars <- variablesinformula(rhs)
      vars <- vars[vars %in% c(colnames(x), ".x", ".y")]
      if(length(vars) == 1 && vars %in% c(covname, ".x")) {
        # expression in terms of covariate
        rhstr <- as.character(rhs)[2]
        dat <- list(ZX)
        names(dat) <- vars[1]
        rugx <- as.numeric(eval(parse(text=rhstr), dat))
      } else {
        if(asked.rug) warning("Unable to add rug plot")
        rugx <- NULL
      }
    } 
    if(!is.null(rugx)) {
      # restrict to x limits, if given
      if(!is.null(xlim <- list(...)$xlim))
        rugx <- rugx[rugx >= xlim[1] & rugx <= xlim[2]]
      # finally plot the rug
      if(length(rugx) > 0)
        rug(rugx)
    }
  }
  invisible(NULL)
}

predict.rhohat <- local({

  predict.rhohat <- function(object, ..., relative=FALSE,
                             what=c("rho", "lo", "hi", "se")) {
    trap.extra.arguments(..., .Context="in predict.rhohat")
    what <- match.arg(what)
    #' extract info
    s <- attr(object, "stuff")
    reference <- s$reference
    #' check availability
    if((what %in% c("lo", "hi", "se")) && !("hi" %in% names(object)))
      stop("Standard error and confidence bands are not available in this object",
           call.=FALSE)
    #' convert to (linearly interpolated) function 
    x <- with(object, .x)
    y <- if(what == "se") sqrt(object[["var"]]) else object[[what]]
    fun <- approxfun(x, y, rule=2)
    #' extract image(s) of covariate
    Z <- s$Zimage
    #' apply fun to Z
    Y <- if(is.im(Z)) evalfun(Z, fun) else solapply(Z, evalfun, f=fun)
    #' adjust to reference baseline
    if(reference != "Lebesgue" && !relative) {
      Lam <- s$lambdaimage # could be 'im' or 'imlist'
      Y <- Lam * Y 
    }
    return(Y)
  }

  evalfun <- function(X, f) {
    force(f)
    force(X)
    if(is.linim(X)) eval.linim(f(X)) else 
    if(is.im(X)) eval.im(f(X)) else NULL
  }
  
  predict.rhohat
})

as.function.rhohat <- function(x, ..., value=".y", extrapolate=TRUE) {
  NextMethod("as.function")
}

simulate.rhohat <- function(object, nsim=1, ..., drop=TRUE) {
  trap.extra.arguments(..., .Context="in simulate.rhohat")
  lambda <- predict(object)
  if(is.linim(lambda) || (is.solist(lambda) && all(sapply(lambda, is.linim)))) {
    result <- rpoislpp(lambda, nsim=nsim, drop=drop)
  } else {
    result <- rpoispp(lambda, nsim=nsim, drop=drop)
  }
  return(result)
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#'   tests/sdr.R
#'
#'   $Revision: 1.2 $ $Date: 2020/05/01 09:59:59 $

if(FULLTEST) {
local({
  AN <- sdr(bei, bei.extra, method="NNIR")
  AV <- sdr(bei, bei.extra, method="SAVE")
  AI <- sdr(bei, bei.extra, method="SIR")
  AT <- sdr(bei, bei.extra, method="TSE")
  subspaceDistance(AN$B, AV$B)
  dimhat(AN$M)
})
}
##
##  tests/segments.R
##   Tests of psp class and related code
##                      [SEE ALSO: tests/xysegment.R]
##
##  $Revision: 1.26 $  $Date: 2020/05/01 09:59:59 $

local({
  if(ALWAYS) { # depends on platform
    ## pointed out by Jeff Laake
    W <- owin()
    X <- psp(x0=.25,x1=.25,y0=0,y1=1,window=W)
    X[W]
  }

  X <- psp(runif(10),runif(10),runif(10),runif(10), window=owin())

  if(FULLTEST) {
    ## migrated from 'lpp'
    Z <- as.mask.psp(X)
    Z <- pixellate(X)
  }

  if(ALWAYS) { # platform dependent
    ## add short segment
    Shorty <- psp(0.5, 0.6, 0.5001, 0.6001, window=Window(X))
    XX <- superimpose(X[1:5], Shorty, X[6:10])
    ZZ <- as.mask.psp(XX)
    ZZ <- pixellate(XX)
  }

  if(FULLTEST) {
    #' misc
    PX <- periodify(X, 2)
  }

  if(FULLTEST) {
    ## more tests of lppm code
    fit <- lppm(unmark(chicago) ~ polynom(x,y,2))
    Z <- predict(fit)
  }

  if(ALWAYS) { # C code
    ## tests of pixellate.psp -> seg2pixL
    ns <- 50
    out <- numeric(ns)
    for(i in 1:ns) {
      X <- psp(runif(1), runif(1), runif(1), runif(1), window=owin())
      len <- lengths_psp(X)
      dlen <- sum(pixellate(X)$v)
      out[i] <- if(len > 1e-7) dlen/len else 1
    }
    if(diff(range(out)) > 0.01) stop(paste(
                                  "pixellate.psp test 1: relative error [",
                                  paste(diff(range(out)), collapse=", "),
                                  "]"))

    ## Michael Sumner's test examples
    set.seed(33)
    n <- 2001
    co <- cbind(runif(n), runif(n))
    ow <- owin()
    X <- psp(co[-n,1], co[-n,2], co[-1,1], co[-1,2], window=ow)
    s1 <- sum(pixellate(X))
    s2 <- sum(lengths_psp(X))
    if(abs(s1 - s2)/s2 > 0.01) {
      stop(paste("pixellate.psp test 2:",
                 "sum(pixellate(X)) = ", s1,
                 "!=", s2, "= sum(lengths_psp(X))"))
    }

    wts <- 1/(lengths_psp(X) * X$n)
    s1 <- sum(pixellate(X, weights=wts))
    if(abs(s1-1) > 0.01) {
      stop(paste("pixellate.psp test 3:",
                 "sum(pixellate(X, weights))=", s1,
                 " (should be 1)"))
    }
    
    X <- psp(0, 0, 0.01, 0.001, window=owin())
    s1 <- sum(pixellate(X))
    s2 <- sum(lengths_psp(X))
    if(abs(s1 - s2)/s2 > 0.01) {
      stop(paste("pixellate.psp test 4:",
                 "sum(pixellate(X)) = ", s1,
                 "!=", s2, "= sum(lengths_psp(X))"))
    }

    X <- psp(0, 0, 0.001, 0.001, window=owin())
    s1 <- sum(pixellate(X))
    s2 <- sum(lengths_psp(X))
    if(abs(s1 - s2)/s2 > 0.01) {
      stop(paste("pixellate.psp test 5:",
                 "sum(pixellate(X)) = ", s1,
                 "!=", s2, "= sum(lengths_psp(X))"))
    }
  }

  if(FULLTEST) {
    #' cases of superimpose.psp
    A <- as.psp(matrix(runif(40), 10, 4), window=owin())
    B <- as.psp(matrix(runif(40), 10, 4), window=owin())
    superimpose(A, B, W=ripras)
    superimpose(A, B, W="convex")
  }

  if(ALWAYS) { # C code
    #' tests of density.psp
    Y <- as.psp(simplenet)
    YC <- density(Y, 0.2, method="C", edge=FALSE, dimyx=64)
    YI <- density(Y, 0.2, method="interpreted", edge=FALSE, dimyx=64)
    YF <- density(Y, 0.2, method="FFT", edge=FALSE, dimyx=64)
    xCI <- max(abs(YC/YI - 1))
    xFI <- max(abs(YF/YI - 1))
    if(xCI > 0.01) stop(paste("density.psp C algorithm relative error =", xCI))
    if(xFI > 0.01) stop(paste("density.psp FFT algorithm relative error =", xFI))
    B <- square(0.3)
    density(Y, 0.2, at=B)
    density(Y, 0.2, at=B, edge=TRUE, method="C")
    Z <- runifpoint(3, B)
    density(Y, 0.2, at=Z)
    density(Y, 0.2, at=Z, edge=TRUE, method="C")
  }

  if(FULLTEST) {
    #' as.psp.data.frame
    df <- as.data.frame(matrix(runif(40), ncol=4))
    A <- as.psp(df, window=square(1))
    colnames(df) <- c("x0","y0","x1","y1")
    df <- cbind(df, data.frame(marks=1:nrow(df)))
    B <- as.psp(df, window=square(1))
    colnames(df) <- c("xmid", "ymid", "length", "angle", "marks")
    E <- as.psp(df, window=square(c(-1,2)))
    G <- E %mark% factor(sample(letters[1:3], nsegments(E), replace=TRUE))
    H <- E %mark% runif(nsegments(E))
  
    #' print and summary methods
    A
    B
    E
    G
    H
    summary(B)
    summary(G)
    summary(H)
    M <- B
    marks(M) <- data.frame(id=marks(B), len=lengths_psp(B))
    M
    summary(M)
    subset(M, select=len)

    #' plot method cases  
    spatstat.options(monochrome=TRUE)
    plot(B)
    plot(G)
    plot(M)
    spatstat.options(monochrome=FALSE)
    plot(B)
    plot(G)
    plot(M)
    #' misuse of 'col' argument - several cases
    plot(G, col="grey") # discrete
    plot(B, col="grey") 
    plot(unmark(B), col="grey") 
    plot(M, col="grey") 
  
    #' miscellaneous class support cases
    marks(M) <- marks(M)[1,,drop=FALSE]
    
    #' undocumented  
    as.ppp(B)
  }

  if(ALWAYS) { # C code
    #' segment crossing code
    X <- psp(runif(30),runif(30),runif(30),runif(30), window=owin())
    A <- selfcut.psp(X, eps=1e-11)
    B <- selfcut.psp(X[1])
    #' 
    Y <- psp(runif(30),runif(30),runif(30),runif(30), window=owin())
    Z <- edges(letterR)[c(FALSE,TRUE)]
    spatstat.options(selfcrossing.psp.useCall=FALSE, crossing.psp.useCall=FALSE)
    A <- selfcrossing.psp(X)
    B <- selfcrossing.psp(Z)
    D <- crossing.psp(X,Y,details=TRUE)
    spatstat.options(selfcrossing.psp.useCall=TRUE, crossing.psp.useCall=TRUE)
    A <- selfcrossing.psp(X)
    B <- selfcrossing.psp(Z)
    D <- crossing.psp(X,Y,details=TRUE)
  }

  if(FULLTEST) {
    #' test rshift.psp and append.psp with marks (Ute Hahn)
    m <- data.frame(A=1:10, B=letters[1:10])
    g <- gl(3, 3, length=10)
    X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin(), marks=m)
    Y <- rshift(X, radius = 0.1)
    Y <- rshift(X, radius = 0.1, group=g)
    #' mark management
    b <- data.frame(A=1:10)
    X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin(), marks=b)
    stopifnot(is.data.frame(marks(X)))
    Y <- rshift(X, radius = 0.1)
    Y <- rshift(X, radius = 0.1, group=g)
  }

  if(FULLTEST) {
    #' geometry
    m <- data.frame(A=1:10, B=letters[1:10])
    X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin(), marks=m)
    Z <- rotate(X, angle=pi/3, centre=c(0.5, 0.5))
    Y <- endpoints.psp(X, which="lower")
    Y <- endpoints.psp(X, which="upper")
    Y <- endpoints.psp(X, which="right")
    U <- flipxy(X)
  }
  
  if(ALWAYS) {
    ## nnfun.psp
    P <- psp(runif(10), runif(10), runif(10), runif(10),
             window=square(1), marks=runif(10))
    f <- nnfun(P)
    f <- nnfun(P, value="mark")
    d <- domain(f)
    Z <- as.im(f)
  }

})

reset.spatstat.options()
#
## tests/sigtraceprogress.R
#
## Tests of *.sigtrace and *.progress
#
## $Revision: 1.5 $ $Date: 2020/05/01 09:59:59 $

if(FULLTEST) {
local({
  plot(dclf.sigtrace(redwood, nsim=19, alternative="greater", rmin=0.02,
                     verbose=FALSE))
  plot(dclf.progress(redwood, nsim=19, alternative="greater", rmin=0.02,
                     verbose=FALSE))
  plot(dg.sigtrace(redwood, nsim=5, alternative="greater", rmin=0.02,
                     verbose=FALSE))
  plot(dg.progress(redwood, nsim=5, alternative="greater", rmin=0.02,
                   verbose=FALSE))
  ## test 'leave-two-out' algorithm
  a <- dclf.sigtrace(redwood, Lest, nsim=9, use.theory=FALSE, leaveout=2,
                     verbose=FALSE)
  aa <- dclf.progress(redwood, Lest, nsim=9, use.theory=FALSE, leaveout=2,
                      verbose=FALSE)
  b <- dg.sigtrace(redwood, Lest, nsim=5, use.theory=FALSE, leaveout=2)
  bb <- dg.progress(redwood, Lest, nsim=5, use.theory=FALSE, leaveout=2,
                    verbose=FALSE)
  ## other code blocks
  e <- mad.progress(redwood, nsim=5)
  e <- mad.progress(redwood, nsim=19, alpha=0.05)
  f <- dclf.progress(redwood, nsim=5, scale=function(x) x^2)
  f <- dclf.progress(redwood, nsim=5, normalize=TRUE, deflate=TRUE)
  g <- dg.progress(redwood, nsim=5, scale=function(x) x^2)
  g <- dg.progress(redwood, nsim=5, normalize=TRUE, deflate=TRUE)
})
}
#'
#'     tests/simplepan.R
#'
#'   Tests of user interaction in simplepanel
#'   Handled by spatstatLocator()
#'
#'   $Revision: 1.3 $  $Date: 2020/05/01 09:59:59 $
#'

if(ALWAYS) {  # may depend on platform
local({
  ## Adapted from example(simplepanel)
  ## make boxes
  outerbox <- owin(c(0,4), c(0,1))
  buttonboxes <- layout.boxes(outerbox, 4, horizontal=TRUE, aspect=1)
  ## make environment containing an integer count
  myenv <- new.env()
  assign("answer", 0, envir=myenv)
  ## what to do when finished: return the count.
  myexit <- function(e) { return(get("answer", envir=e)) }
  ## button clicks
  ## decrement the count
  Cminus <- function(e, xy) {
    ans <- get("answer", envir=e)
    assign("answer", ans - 1, envir=e)
    return(TRUE)
  }
  ## display the count (clicking does nothing)
  Cvalue <- function(...) { TRUE }
  ## increment the count
  Cplus <- function(e, xy) {
    ans <- get("answer", envir=e)
    assign("answer", ans + 1, envir=e)
    return(TRUE)
  }
  ## 'Clear' button
  Cclear <- function(e, xy) {
    assign("answer", 0, envir=e)
    return(TRUE)
  }
  ## quit button
  Cdone <- function(e, xy) { return(FALSE) }

  myclicks <- list("-"=Cminus,
                   value=Cvalue,
                   "+"=Cplus,
                   done=Cdone)
  ## redraw the button that displays the current value of the count
  Rvalue <- function(button, nam, e) {
     plot(button, add=TRUE)
     ans <- get("answer", envir=e)
     text(centroid.owin(button), labels=ans)
     return(TRUE)
  }
  ## make the panel
  P <- simplepanel("Counter",
                   B=outerbox, boxes=buttonboxes,
                   clicks=myclicks,
                   redraws = list(NULL, Rvalue, NULL, NULL),
                   exit=myexit, env=myenv)
  ## queue up a sequence of inputs
  boxcentres <- do.call(concatxy, unname(lapply(buttonboxes[c(3,3,1,3,2,4)],
                                                centroid.owin)))
  spatstat.utils::queueSpatstatLocator(boxcentres$x, boxcentres$y)
  ## go
  run.simplepanel(P)
})
}
#
# tests/slrm.R
#
# $Revision: 1.3 $ $Date: 2020/05/01 09:59:59 $
#
# Test slrm fitting and prediction when there are NA's
#

if(ALWAYS) {
local({
  X <- copper$SouthPoints
  W <- owin(poly=list(x=c(0,35,35,1),y=c(1,1,150,150)))
  Y <- X[W]
  fit <- slrm(Y ~ x+y)
  pred <- predict(fit)
  extractAIC(fit)
  fitx <- update(fit, . ~ x)
  simulate(fitx, seed=42)
  if(FULLTEST) {
    unitname(fitx)
    unitname(fitx) <- "km"

    mur <- solapply(murchison,rescale, 1000, "km")
    mur$dfault <- distfun(mur$faults)
    fut <- slrm(gold ~ dfault, data=mur, splitby="greenstone")
    A <- model.images(fut)
  }
})
}


#
#  tests/splitpea.R
#
#  Check behaviour of split.ppp etc
#
#  Thanks to Marcelino de la Cruz
#
#  $Revision: 1.14 $  $Date: 2020/05/01 09:59:59 $
#

local({
  W <- square(8)
  X <- ppp(c(2.98, 4.58, 7.27, 1.61, 7.19),
           c(7.56, 5.29, 5.03, 0.49, 1.65),
           window=W)
  Z <- quadrats(W, 4, 4)
  Yall <- split(X, Z, drop=FALSE)
  Ydrop <- split(X, Z, drop=TRUE)

  if(ALWAYS) { # may depend on platform
    P <- Yall[[1]]
    if(!all(inside.owin(P$x, P$y, P$window)))
      stop("Black hole detected when drop=FALSE")
    P <- Ydrop[[1]]
    if(!all(inside.owin(P$x, P$y, P$window)))
      stop("Black hole detected when drop=TRUE")
    Ydrop[[1]] <- P[1]
    split(X, Z, drop=TRUE) <- Ydrop
  }

  ## test NA handling
  Zbad <- quadrats(square(4), 2, 2)
  Ybdrop <- split(X, Zbad, drop=TRUE)
  Yball  <- split(X, Zbad, drop=FALSE)

  if(FULLTEST) {
    ## other bugs/ code blocks in split.ppp, split<-.ppp, [<-.splitppp
    flog <- rep(c(TRUE,FALSE), 21)
    fimg <- as.im(dirichlet(runifpoint(5, Window(cells))))
    A <- split(cells, flog)
    B <- split(cells, square(0.5))
    D <- split(cells, fimg)
    E <- split(cells, logical(42), drop=TRUE)
    Cellules <- cells
    split(Cellules, flog) <- solapply(A, rjitter)
    split(Cellules, fimg) <- solapply(D, rjitter)
    D[[2]] <- rjitter(D[[2]])
    Funpines <- finpines
    marks(Funpines)[,"diameter"] <- factor(marks(Funpines)[,"diameter"])
    G <- split(Funpines)
    H <- split(Funpines, "diameter")
    split(Funpines) <- solapply(G, rjitter)
    split(Funpines, "diameter") <- solapply(H, rjitter)

    ## From Marcelino
    set.seed(1)
    W<- square(10) # the big window
    puntos<- rpoispp(0.5, win=W)
    r00 <- letterR
    r05 <- shift(letterR,c(0,5))
    r50 <- shift(letterR,c(5,0))
    r55 <- shift(letterR,c(5,5))
    tessr4 <- tess(tiles=list(r00, r05,r50,r55))
    puntosr4 <- split(puntos, tessr4, drop=TRUE)
    split(puntos, tessr4, drop=TRUE) <- puntosr4

    ## More headaches with mark format
    A <- runifpoint(10)
    B <- runifpoint(10)
    AB <- split(superimpose(A=A, B=B))

    #' check that split<- respects ordering where possible
    X <- amacrine
    Y <- split(X)
    split(X) <- Y
    stopifnot(identical(X, amacrine))

    #' split.ppx
    df <- data.frame(x=runif(4),y=runif(4),t=runif(4),
                     age=rep(c("old", "new"), 2),
                     mineral=factor(rep(c("Au","Cu"), each=2),
                                    levels=c("Au", "Cu", "Pb")),
                     size=runif(4))
    X <- ppx(data=df, coord.type=c("s","s","t","m", "m","m"))
    Y <- split(X, "age")
    Y <- split(X, "mineral", drop=TRUE)
    Y <- split(X, "mineral")
    print(Y)
    print(summary(Y))
    Y[c(TRUE,FALSE,TRUE)]
    Y[1:2]
    Y[3] <- Y[1]
  }
})

#'
#'    tests/ssf.R
#'
#'   Tests of 'ssf' class
#'
#'   $Revision: 1.3 $ $Date: 2020/05/01 09:59:59 $
#'

if(FULLTEST) {
local({
  Y <- cells[1:5]
  X <- rsyst(Window(Y), 5)
  Z <- runifpoint(3, Window(Y))
  f1 <- ssf(X, nncross(X,Y,what="dist"))
  f2 <- ssf(X, nncross(X,Y,what="dist", k=1:2))
  image(f1)
  g1 <- as.function(f1)
  g1(Z)
  g2 <- as.function(f2)
  g2(Z)
  plot(f1, style="contour")
  plot(f1, style="imagecontour")
  contour(f1)
  apply.ssf(f2, 1, sum)
  range(f1)
  min(f1)
  max(f1)
  integral(f1, weights=tile.areas(dirichlet(X)))
})
}
#
#   tests/step.R
#
#   $Revision: 1.5 $  $Date: 2020/05/01 09:59:59 $
#
# test for step() operation
#
if(FULLTEST) {
local({
  Z <- as.im(function(x,y){ x^3 - y^2 }, nztrees$window)
  fitP <- ppm(nztrees ~x+y+Z, covariates=list(Z=Z))
  step(fitP)
  fitS <- update(fitP, Strauss(7))
  step(fitS)
  fitM <- ppm(amacrine ~ marks*(x+y),
              MultiStrauss(types=levels(marks(amacrine)), radii=matrix(0.04, 2, 2)))
  step(fitM)
})
}



#'
#'   tests/sumfun.R
#'
#'   Tests of code for summary functions
#'       including score residual functions etc
#'
#'   $Revision: 1.6 $ $Date: 2020/05/01 09:59:59 $

if(ALWAYS) { # involves C code 
local({
  W <- owin(c(0,1), c(-1/2, 0))
  Gr <- Gest(redwood, correction="all",domain=W)
  Fr <- Fest(redwood, correction="all",domain=W)
  Jr <- Jest(redwood, correction="all",domain=W)
  
  F0 <- Fest(redwood[FALSE], correction="all")
  Fh <- Fest(humberside, domain=erosion(Window(humberside), 100))

  FIr <- Finhom(redwood, savelambda=TRUE)
  JIr <- Jinhom(redwood, savelambda=TRUE)
  
  Ga <- Gcross(amacrine, correction="all")
  Ia <- Iest(amacrine, correction="all")
  lam <- intensity(amacrine)
  lmin <- 0.9 * min(lam)
  nJ <- sum(marks(amacrine) == "off")
  FM <- FmultiInhom(amacrine, marks(amacrine) == "off",
                    lambdaJ=rep(lam["off"], nJ),
                    lambdamin = lmin)
  GM <- GmultiInhom(amacrine, marks(amacrine) == "on",
                    marks(amacrine) == "off",
                    lambda=lam[marks(amacrine)],
                    lambdamin=lmin,
                    ReferenceMeasureMarkSetI=42)

  pt <- psst(cells, interaction=Strauss(0.1), fun=nndcumfun)

  a <- compileCDF(D=nndist(redwood),
                  B=bdist.points(redwood),
                  r=seq(0, 1, length=256))

  ## distance argument spacing and breakpoints
  e <- check.finespacing(c(0,1,2), eps=0.1, action="silent")
  b <- as.breakpts(pi, 20)
  b <- as.breakpts(42, max=pi, npos=20)
  b <- even.breaks.owin(letterR)
})
}
##
## tests/symbolmaps.R
##
##   Quirks associated with symbolmaps, etc.
##
## $Revision: 1.4 $ $Date: 2020/05/01 09:59:59 $

if(FULLTEST) {
local({
  set.seed(100)
  X <- runifpoint(8)

  ## symbolmap 
  g1 <- symbolmap(range=c(0,100), size=function(x) x/50)
  invoke.symbolmap(g1, 50, x=numeric(0), y=numeric(0), add=TRUE)
  plot(g1, labelmap=100)
  ## constant/trivial
  a <- symbolmap(pch=16)
  print(a)
  plot(a)
  symbolmapdomain(a)
  b <- symbolmap()
  print(b)

  ## textureplot
  V <- as.im(dirichlet(X))
  tmap <- textureplot(V)
  textureplot(V, textures=tmap, legend=TRUE, leg.side="left")
  textureplot(V, leg.side="bottom")
  textureplot(V, leg.side="top")
  ## spacing too large for tiles - upsets various pieces of code
  textureplot(V, spacing=2)

  ## plot.texturemap
  plot(tmap, vertical=TRUE)
  plot(tmap, vertical=TRUE, xlim=c(0,1))
  plot(tmap, vertical=TRUE, ylim=c(0,1))
  plot(tmap, vertical=FALSE, xlim=c(0,1))
  plot(tmap, vertical=FALSE, ylim=c(0,1))

  ## infrastructure
  plan.legend.layout(owin(), side="top", started=TRUE)
})
}

#
#   pcfinhom.R
#
#   $Revision: 1.23 $   $Date: 2019/05/24 10:24:48 $
#
#   inhomogeneous pair correlation function of point pattern 
#
#

pcfinhom <- function(X, lambda=NULL, ..., r=NULL,
                     kernel="epanechnikov", bw=NULL, stoyan=0.15,
                     correction=c("translate", "Ripley"),
                     divisor=c("r","d"),
                     renormalise=TRUE,
                     normpower=1,
                     update=TRUE, leaveoneout=TRUE,
                     reciplambda=NULL, 
                     sigma=NULL, varcov=NULL, close=NULL)
{
  verifyclass(X, "ppp")
#  r.override <- !is.null(r)
  miss.update <- missing(update)
  
  win <- X$window
  areaW <- area(win)
  npts <- npoints(X)

  kernel <- match.kernel(kernel)
  
  correction.given <- !missing(correction)
  correction <- pickoption("correction", correction,
                           c(isotropic="isotropic",
                             Ripley="isotropic",
                             trans="translate",
                             translate="translate",
                             translation="translate",
                             good="good",
                             best="best"),
                           multi=TRUE)
  if("good" %in% correction)
    correction[correction == "good"] <- good.correction.K(X)
  correction <- implemented.for.K(correction, win$type, correction.given)

  divisor <- match.arg(divisor)
  
  if(is.null(bw) && kernel=="epanechnikov") {
    # Stoyan & Stoyan 1995, eq (15.16), page 285
    h <- stoyan /sqrt(npts/areaW)
    hmax <- h
    # conversion to standard deviation
    bw <- h/sqrt(5)
  } else if(is.numeric(bw)) {
    # standard deviation of kernel specified
    # upper bound on half-width
    hmax <- 3 * bw
  } else {
    # data-dependent bandwidth selection: guess upper bound on half-width
    hmax <- 2 * stoyan /sqrt(npts/areaW)
  }


  ########## intensity values #########################

  dangerous <- c("lambda", "reciplambda")
  danger <- TRUE

  if(npts == 0) {
    lambda <- reciplambda <- numeric(0)
    danger <- FALSE
  } else if(missing(lambda) && is.null(reciplambda)) {
    # No intensity data provided
    danger <- FALSE
    # Estimate density by leave-one-out kernel smoothing
    lambda <- density(X, ..., sigma=sigma, varcov=varcov,
                      at="points", leaveoneout=TRUE)
    lambda <- as.numeric(lambda)
    reciplambda <- 1/lambda
  } else if(!is.null(reciplambda)) {
    # 1/lambda values provided
    if(is.im(reciplambda)) 
      reciplambda <- safelookup(reciplambda, X)
    else if(is.function(reciplambda))
      reciplambda <- reciplambda(X$x, X$y)
    else if(is.numeric(reciplambda) && is.vector(as.numeric(reciplambda)))
      check.nvector(reciplambda, npts)
    else stop(paste(sQuote("reciplambda"),
                    "should be a vector, a pixel image, or a function"))
  } else {
    # lambda values provided
    if(is.im(lambda)) 
      lambda <- safelookup(lambda, X)
    else if(is.ppm(lambda) || is.kppm(lambda) || is.dppm(lambda)) {
      model <- lambda
      if(!update) {
        ## just use intensity of fitted model
        lambda <- predict(model, locations=X, type="trend")
      } else {
        if(is.ppm(model)) {
          model <- update(model, Q=X)
          lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
        } else {
          model <- update(model, X=X)
          lambda <- fitted(model, dataonly=TRUE, leaveoneout=leaveoneout)
        }
        danger <- FALSE
        if(miss.update) 
          warn.once(key="pcfinhom.update",
                    "The behaviour of pcfinhom when lambda is a ppm object",
                    "has changed (in spatstat 1.45-0 and later).",
                    "See help(pcfinhom)")
      }
    } else if(is.function(lambda)) 
      lambda <- lambda(X$x, X$y)
    else if(is.numeric(lambda) && is.vector(as.numeric(lambda)))
      check.nvector(lambda, npts)
    else stop(paste(sQuote("lambda"),
         "should be a vector, a pixel image, a function, or a fitted model"))
    # evaluate reciprocal
    reciplambda <- 1/lambda
  }
  
  # renormalise
  if(renormalise && npts > 0) {
    check.1.real(normpower)
    stopifnot(normpower %in% 1:2)
    renorm.factor <- (areaW/sum(reciplambda))^normpower
  } 
  
  ########## r values ############################
  # handle arguments r and breaks 

  rmaxdefault <- rmax.rule("K", win, lambda)        
  breaks <- handle.r.b.args(r, NULL, win, rmaxdefault=rmaxdefault)
  if(!(breaks$even))
    stop("r values must be evenly spaced")
  # extract r values
  r <- breaks$r
  rmax <- breaks$max
  # recommended range of r values for plotting
  alim <- c(0, min(rmax, rmaxdefault))

  ########## smoothing parameters for pcf ############################  
  # arguments for 'density'

  denargs <- resolve.defaults(list(kernel=kernel, bw=bw),
                              list(...),
                              list(n=length(r), from=0, to=rmax))
  
  #################################################
  
  # compute pairwise distances

  if(npts > 1) {
    if(is.null(close)) {
      #' find close pairs
      close <- closepairs(X, rmax+hmax)
    } else {
      #' check 'close' has correct format
      needed <- c("i", "j", "xi", "yi", "xj", "yj", "dx", "dy", "d")
      if(any(is.na(match(needed, names(close)))))
        stop(paste("Argument", sQuote("close"),
                   "should have components named",
                   commasep(sQuote(needed))),
             call.=FALSE)
    }
    dIJ <- close$d
    I <- close$i
    J <- close$j
    XI <- ppp(close$xi, close$yi, window=win, check=FALSE)
    wIJ <- reciplambda[I] * reciplambda[J]
  } else {
    undefined <- rep(NaN, length(r))
  }

  # initialise fv object
  
  df <- data.frame(r=r, theo=rep.int(1,length(r)))
  out <- fv(df, "r",
            quote(g[inhom](r)), "theo", ,
            alim,
            c("r","{%s[%s]^{pois}}(r)"),
            c("distance argument r", "theoretical Poisson %s"),
            fname=c("g", "inhom"))

  ###### compute #######

  if(any(correction=="translate")) {
    # translation correction
    if(npts > 1) {
      XJ <- ppp(close$xj, close$yj, window=win, check=FALSE)
      edgewt <- edge.Trans(XI, XJ, paired=TRUE)
      gT <- sewpcf(dIJ, edgewt * wIJ, denargs, areaW, divisor)$g
      if(renormalise) gT <- gT * renorm.factor
    } else gT <- undefined
    out <- bind.fv(out,
                   data.frame(trans=gT),
                   "{hat(%s)[%s]^{Trans}}(r)",
                   "translation-corrected estimate of %s",
                   "trans")
  }
  if(any(correction=="isotropic")) {
    # Ripley isotropic correction
    if(npts > 1) {
      edgewt <- edge.Ripley(XI, matrix(dIJ, ncol=1))
      gR <- sewpcf(dIJ, edgewt * wIJ, denargs, areaW, divisor)$g
      if(renormalise) gR <- gR * renorm.factor
    } else gR <- undefined
    out <- bind.fv(out,
                   data.frame(iso=gR),
                   "{hat(%s)[%s]^{Ripley}}(r)",
                   "isotropic-corrected estimate of %s",
                   "iso")
  }
  
  # sanity check
  if(is.null(out)) {
    warning("Nothing computed - no edge corrections chosen")
    return(NULL)
  }
  
  # which corrections have been computed?
  corrxns <- rev(setdiff(names(out), "r"))

  # default is to display them all
  formula(out) <- . ~ r
  fvnames(out, ".") <- corrxns

  unitname(out) <- unitname(X)
  if(danger)
    attr(out, "dangerous") <- dangerous
  return(out)
}

#' Dominic Schuhmacher's idea
#'
#' $Revision: 1.17 $ $Date: 2019/11/15 07:12:52 $
#'

clickppp <- local({

  clickppp <- function(n=NULL, win=square(1), types=NULL, ...,
                     add=FALSE, main=NULL, hook=NULL) {
    win <- as.owin(win)
    instructions <-
      if(!is.null(n)) paste("click", n, "times in window") else
      paste("add points: click left mouse button in window\n",
            "exit: press ESC or another mouse button")
    if(is.null(main))
      main <- instructions
    

    ####  single type #########################
    if(is.null(types)) {
      plot(win, add=add, main=main, invert=TRUE)
      if(!is.null(hook))
        plot(hook, add=TRUE)
      splat("Ready to click..")
      if(!is.null(n))
        xy <- spatstatLocator(n=n, ...)
      else
        xy <- spatstatLocator(...)
      #' check whether all points lie inside window
      if((nout <- sum(!inside.owin(xy$x, xy$y, win))) > 0) {
        warning(paste(nout,
                      ngettext(nout, "point", "points"),
                      "lying outside specified window; window was expanded"))
        win <- boundingbox(win, xy)
      }
      X <- ppp(xy$x, xy$y, window=win)
      return(X)
    }
  
    ##### multitype #######################
    
    ftypes <- factor(types, levels=types)
    #' input points of type 1 
    X <- getem(ftypes[1L], instructions, n=n, win=win, add=add, ..., pch=1)
    X <- X %mark% ftypes[1L]
    #' input points of types 2, 3, ... in turn
    naughty <- FALSE
    for(i in 2:length(types)) {
      Xi <- getem(ftypes[i], instructions, n=n, win=win, add=add,
                  ..., hook=X, pch=i)
      Xi <- Xi %mark% ftypes[i]
      if(!naughty && identical(Xi$window, win)) {
        #' normal case
        X <- superimpose(X, Xi, W=win)
      } else {
        #' User has clicked outside original window.
        naughty <- TRUE
        #' Use bounding box for simplicity
        bb <- boundingbox(Xi$window, X$window)
        X <- superimpose(X, Xi, W=bb)
      } 
    }
    if(!add) {
      if(!naughty)
        plot(X, main="Final pattern")
      else {
        plot(X$window, main="Final pattern (in expanded window)", invert=TRUE)
        plot(win, add=TRUE, invert=TRUE)
        plot(X, add=TRUE)
      }
    }
    return(X)
  }

  getem <- function(i, instr, ...) {
    main <- paste("Points of type", sQuote(i), "\n", instr)
    do.call(clickppp, resolve.defaults(list(...), list(main=main)))
  }

  clickppp
})


clickdist <- function() {
  a <- spatstatLocator(2)
  return(pairdist(a)[1L,2L])
}

#
#   nnfun.R
#
#   nearest neighbour function (returns a function of x,y)
#
#   $Revision: 1.9 $   $Date: 2020/01/18 02:52:11 $
#

nnfun <- function(X, ...) {
  UseMethod("nnfun")
}

nnfun.ppp <- function(X, ..., k=1, value=c("index", "mark")) {
  # this line forces X to be bound
  stopifnot(is.ppp(X))
  if(length(k) != 1) stop("k should be a single integer")
  value <- match.arg(value)
  switch(value,
         index = {
           gi <- function(x,y=NULL) {
             Y <- xy.coords(x, y)[c("x", "y")]
             nncross(Y, X, what="which", k=k)
           }
           attr(gi, "Xclass") <- "ppp"
           g <- funxy(gi, as.rectangle(as.owin(X)))
         },
         mark = {
           stopifnot(is.marked(X))
           marx <- as.data.frame(marks(X))[,1]
           gm <- function(x,y=NULL) {
             Y <- xy.coords(x, y)[c("x", "y")]
             marx[nncross(Y, X, what="which", k=k)]
           }
           attr(gm, "Xclass") <- "ppp"
           g <- funxy(gm, as.rectangle(as.owin(X)))
         })
  class(g) <- c("nnfun", class(g))
  return(g)
}

nnfun.psp <- function(X, ..., value=c("index", "mark")) {
  # this line forces X to be bound
  stopifnot(is.psp(X))
  value <- match.arg(value)
  switch(value,
         index = {
           gi <- function(x,y=NULL) {
             Y <-  xy.coords(x, y)[c("x", "y")]
             nncross(Y, X, what="which")
           }
           attr(gi, "Xclass") <- "psp"
           g <- funxy(gi, as.rectangle(as.owin(X)))
         },
         mark = {
           stopifnot(is.marked(X))
           marx <- as.data.frame(marks(X))[,1]
           gm <- function(x,y=NULL) {
             Y <-  xy.coords(x, y)[c("x", "y")]
             marx[nncross(Y, X, what="which")]
           }
           attr(gm, "Xclass") <- "psp"
           g <- funxy(gm, as.rectangle(as.owin(X)))
         })
  class(g) <- c("nnfun", class(g))
  return(g)
}

as.owin.nnfun <- function(W, ..., fatal=TRUE) {
  X <- get("X", envir=environment(W))
  as.owin(X, ..., fatal=fatal)
}

domain.nnfun <- Window.nnfun <- function(X, ...) { as.owin(X) }

as.im.nnfun <- function(X, W=NULL, ...,
                           eps=NULL, dimyx=NULL, xy=NULL,
                           na.replace=NULL, approx=TRUE) {
  if(approx && is.null(W)) {
    env <- environment(X)
    Xdata  <- get("X", envir=env)
    if(is.ppp(Xdata)) {
      #' fast approximation is supported only for point patterns
      k <- mget("k", envir=env, inherits=FALSE, ifnotfound=list(1))[[1L]]
      value <- mget("value", envir=env, ifnotfound=list(NULL))[[1L]] 
      Z <- nnmap(Xdata, k=k, what="which", eps=eps, dimyx=dimyx, xy=xy)
      if(identical(value, "mark")) {
        marx <- get("marx", envir=env)
        Z <- eval.im(marx[Z])
      }
      if(!is.null(na.replace))
        Z$v[is.null(Z$v)] <- na.replace
      return(Z)
    }
  }
  if(is.null(W)) W <- Window(X)
  result <- as.im.function(X, W=W,
                           eps=eps, dimyx=dimyx, xy=xy,
                           na.replace=na.replace, ...)
  return(result)
}

print.nnfun <- function(x, ...) {
  env <- environment(x)
  X <- get("X", envir=env)
  k <- mget("k", envir=env, inherits=FALSE, ifnotfound=list(1))[[1L]]
  v <- mget("value", envir=env, ifnotfound=list(NULL))[[1L]]
  xtype <- attr(x, "Xclass")
  typestring <- switch(xtype,
                       ppp="point pattern",
                       psp="line segment pattern",
                       paste("object of class", sQuote(xtype)))
  Kth <- if(k == 1) "Nearest" else paste0(ordinal(k), "-Nearest")
  cat(paste(Kth, "Neighbour",
            if(is.null(v)) "Index" else "Mark",
            "function for ", typestring, "\n"))
  print(X)

  return(invisible(NULL))
}

#'
#'    boundingcircle.R
#'
#'  bounding circle and its centre
#'
#'  $Revision: 1.6 $ $Date: 2017/06/05 10:31:58 $
#'

circumradius <- function(x, ...) {
  .Deprecated("boundingradius")
  UseMethod("boundingradius")
}
circumradius.owin <- function(x, ...) {
  .Deprecated("boundingradius.owin")
  boundingradius.owin(x, ...)
}
circumradius.ppp <- function(x, ...) {
  .Deprecated("boundingradius.ppp")
  boundingradius.ppp(x, ...)
}

boundingradius <- function(x, ...) {
  UseMethod("boundingradius")
}

boundingcentre <- function(x, ...) {
  UseMethod("boundingcentre")
}

boundingcircle <- function(x, ...) {
  UseMethod("boundingcircle")
}

#' owin

boundingradius.owin <- function(x, ...) {
  sqrt(min(fardist(x, ..., squared=TRUE)))
}

boundingcentre.owin <- function(x, ...) {
  z <- where.min(fardist(x, ..., squared=TRUE))
  Window(z) <- x
  return(z)
}

boundingcircle.owin <- function(x, ...) {
  d2 <- fardist(x, ..., squared=TRUE)
  z <- where.min(d2)
  r <- sqrt(min(d2))
  w <- disc(centre=z, radius=r) 
  return(w)
}

#' ppp

boundingradius.ppp <- function(x, ...) {
  boundingradius(convexhull(x), ...)
}

boundingcentre.ppp <- function(x, ...) {
  z <- boundingcentre(convexhull(x), ...)
  Window(z) <- Window(x)
  return(z)
}

boundingcircle.ppp <- function(x, ...) {
  boundingcircle(convexhull(x), ...)
}

#
#  objsurf.R
#
#  surface of the objective function for an M-estimator
#
#  $Revision: 1.5 $ $Date: 2016/02/11 10:17:12 $
#

objsurf <- function(x, ...) {
  UseMethod("objsurf")
}

objsurf.kppm <- objsurf.dppm <- function(x, ..., ngrid=32, ratio=1.5, verbose=TRUE) {
  Fit <- x$Fit
  switch(Fit$method,
         mincon = {
           result <- objsurf(Fit$mcfit, ...,
                             ngrid=ngrid, ratio=ratio, verbose=verbose)
         },
         clik = {
           optpar  <- x$par
           objfun  <- Fit$objfun
           objargs <- Fit$objargs
           result  <- objsurfEngine(objfun, optpar, objargs, ...,
                                    ngrid=ngrid, ratio=ratio, verbose=verbose)
         })
  return(result)
}

objsurf.minconfit <- function(x, ..., ngrid=32, ratio=1.5, verbose=TRUE) {
  optpar  <- x$par.canon %orifnull% x$par
  objfun  <- x$objfun
  objargs <- x$objargs
  dotargs <- x$dotargs
  objsurfEngine(objfun, optpar, objargs, ...,
                dotargs=dotargs,
                ngrid=ngrid, ratio=ratio, verbose=verbose)
}

objsurfEngine <- function(objfun, optpar, objargs, 
                          ...,
                          dotargs=list(),
                          objname="objective", 
                          ngrid=32, ratio=1.5, verbose=TRUE) {
  trap.extra.arguments(...)
  if(!is.function(objfun))
    stop("Object is in an outdated format and needs to be re-fitted")
  npar    <- length(optpar)
  if(npar != 2)
    stop("Only implemented for functions of 2 arguments")
  # create grid of parameter values
  ratio <- ensure2vector(ratio)
  ngrid <- ensure2vector(ngrid)
  stopifnot(all(ratio > 1))
  xgrid <- seq(optpar[1]/ratio[1], optpar[1] * ratio[1], length=ngrid[1])
  ygrid <- seq(optpar[2]/ratio[2], optpar[2] * ratio[2], length=ngrid[2])
  pargrid <- expand.grid(xgrid, ygrid)
  colnames(pargrid) <- names(optpar)
  # evaluate
  if(verbose) cat(paste("Evaluating", nrow(pargrid), "function values..."))
  values <- do.call(apply,
                    append(list(pargrid, 1, objfun, objargs=objargs), dotargs))
  if(verbose) cat("Done.\n")
  result <- list(x=xgrid, y=ygrid, z=matrix(values, ngrid[1], ngrid[2]))
  attr(result, "optpar") <- optpar
  attr(result, "objname") <- "contrast"
  class(result) <- "objsurf"
  return(result)
}

print.objsurf <- function(x, ...) {
  cat("Objective function surface\n")
  optpar <- attr(x, "optpar")
  objname <- attr(x, "objname")
  nama <- names(optpar)
  cat("Parameter ranges:\n")
  cat(paste(paste0(nama[1], ":"), prange(range(x$x)), "\n"))
  cat(paste(paste0(nama[2], ":"), prange(range(x$y)), "\n"))
  cat(paste("Function value:", objname, "\n"))
  invisible(NULL)
}

image.objsurf <- plot.objsurf <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  optpar <- attr(x, "optpar")
  nama <- names(optpar)
  do.call(image,
          resolve.defaults(list(x=unclass(x)),
                           list(...),
                           list(xlab=nama[1], ylab=nama[2], main=xname)))
  abline(v=optpar[1], lty=3)
  abline(h=optpar[2], lty=3)
  invisible(NULL)
}

contour.objsurf <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  optpar <- attr(x, "optpar")
  nama <- names(optpar)
  do.call(contour,
          resolve.defaults(list(x=unclass(x)),
                           list(...),
                           list(xlab=nama[1], ylab=nama[2], main=xname)))
  abline(v=optpar[1], lty=3)
  abline(h=optpar[2], lty=3)
  invisible(NULL)
}

  
persp.objsurf <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  optpar <- attr(x, "optpar")
  objname <- attr(x, "objname")
  nama <- names(optpar)
  r <- do.call(persp,
               resolve.defaults(list(x=x$x, y=x$y, z=x$z),
                                list(...),
                                list(xlab=nama[1], ylab=nama[2],
                                     zlab=objname, main=xname)))
  invisible(r)
}

## lookup table of explicitly-known K functions and pcf
## and algorithms for computing sensible starting parameters

.Spatstat.ClusterModelInfoTable <- 
  list(
       Thomas=list(
         ## Thomas process: old par = (kappa, sigma2) (internally used everywhere)
         ## Thomas process: new par = (kappa, scale) (officially recommended for input/output)
         modelname = "Thomas process", # In modelname field of mincon fv obj.
         descname = "Thomas process", # In desc field of mincon fv obj.
         modelabbrev = "Thomas process", # In fitted obj.
         printmodelname = function(...) "Thomas process", # Used by print.kppm
         parnames = c("kappa", "sigma2"),
         clustargsnames = NULL,
         checkpar = function(par, old = TRUE){
             if(is.null(par))
                 par <- c(kappa=1,scale=1)
             if(any(par<=0))
                 stop("par values must be positive.")
             nam <- check.named.vector(par, c("kappa","sigma2"),
                                       onError="null")
             if(is.null(nam)) {
               check.named.vector(par, c("kappa","scale"))
               names(par)[2L] <- "sigma2"
               par[2L] <- par[2L]^2
             }
             if(!old){
                 names(par)[2L] <- "scale"
                 par[2L] <- sqrt(par[2L])
             }
             return(par)
         },
         checkclustargs = function(margs, old = TRUE) list(),
         resolvedots = function(...){
           return(list(...))
         },
         # density function for the distance to offspring
         ddist = function(r, scale, ...) {
             2 * pi * r * dnorm(r, 0, scale)/sqrt(2*pi*scale^2)
         },
         ## Practical range of clusters
         range = function(...){
             dots <- list(...)
             par <- dots$par
             # Choose the first of the possible supplied values for scale:
             scale <- c(dots$scale, dots$par[["scale"]], dots$sigma, dots$par[["sigma"]])[1L]
             if(is.null(scale))
                 stop("Argument ", sQuote("scale"), " must be given.")
             thresh <- dots$thresh
             if(!is.null(thresh)){
               ## The squared length of isotropic Gaussian (sigma)
               ## is exponential with mean 2 sigma^2
               rmax <- scale * sqrt(2 * qexp(thresh, lower.tail=FALSE))
               ## old code
               ##  ddist <- .Spatstat.ClusterModelInfoTable$Thomas$ddist
               ##  kernel0 <- clusterkernel("Thomas", scale = scale)(0,0)
               ##  f <- function(r) ddist(r, scale = scale)-thresh*kernel0
               ##  rmax <- uniroot(f, lower = scale, upper = 1000 * scale)$root
             } else{
                 rmax <- 4*scale
             }
             return(rmax)
         },
         kernel = function(par, rvals, ...) {
             scale <- sqrt(par[2L])
             dnorm(rvals, 0, scale)/sqrt(2*pi*scale^2)
         },
         isPCP=TRUE,
         ## K-function
         K = function(par,rvals, ...){
           if(any(par <= 0))
             return(rep.int(Inf, length(rvals)))
           pi*rvals^2+(1-exp(-rvals^2/(4*par[2L])))/par[1L]
         },
         ## pair correlation function
         pcf= function(par,rvals, ...){
           if(any(par <= 0))
             return(rep.int(Inf, length(rvals)))
           1 + exp(-rvals^2/(4 * par[2L]))/(4 * pi * par[1L] * par[2L])
         },
         ## sensible starting parameters
         selfstart = function(X) {
           kappa <- intensity(X)
           sigma2 <- 4 * mean(nndist(X))^2
           c(kappa=kappa, sigma2=sigma2)
         },
         ## meaningful model parameters
         interpret = function(par, lambda) {
           kappa <- par[["kappa"]]
           sigma <- sqrt(par[["sigma2"]])
           mu <- if(is.numeric(lambda) && length(lambda) == 1)
             lambda/kappa else NA
           c(kappa=kappa, sigma=sigma, mu=mu)
         },
         ## Experimental: convert to/from canonical cluster parameters
         tocanonical = function(par) {
           kappa <- par[[1L]]
           sigma2 <- par[[2L]]
           c(strength=1/(4 * pi * kappa * sigma2), scale=sqrt(sigma2))
         },
         tohuman = function(can) {
           strength <- can[[1L]]
           scale <- can[[2L]]
           sigma2 <- scale^2
           c(kappa=1/(4 * pi * strength * sigma2), sigma2=sigma2)
         }
         ),
       ## ...............................................
       MatClust=list(
         ## Matern Cluster process: old par = (kappa, R) (internally used everywhere)
         ## Matern Cluster process: new par = (kappa, scale) (officially recommended for input/output)
         modelname = "Matern cluster process", # In modelname field of mincon fv obj.
         descname = "Matern cluster process", # In desc field of mincon fv obj.
         modelabbrev = "Matern cluster process", # In fitted obj.
         printmodelname = function(...) "Matern cluster process", # Used by print.kppm
         parnames = c("kappa", "R"),
         clustargsnames = NULL,
         checkpar = function(par, old = TRUE){
             if(is.null(par))
                 par <- c(kappa=1,scale=1)
             if(any(par<=0))
                 stop("par values must be positive.")
             nam <- check.named.vector(par, c("kappa","R"), onError="null")
             if(is.null(nam)) {
               check.named.vector(par, c("kappa","scale"))
               names(par)[2L] <- "R"
             }
             if(!old){
                 names(par)[2L] <- "scale"
             }
             return(par)
         },
         # density function for the distance to offspring
         ddist = function(r, scale, ...) {
             ifelse(r>scale, 0, 2 * r / scale^2)
         },
         ## Practical range of clusters
         range = function(...){
             dots <- list(...)
             par <- dots$par
             # Choose the first of the possible supplied values for scale:
             scale <- c(dots$scale, dots$par[["scale"]], dots$R, dots$par[["R"]])[1L]
             if(is.null(scale))
                 stop("Argument ", sQuote("scale"), " must be given.")
           if(!is.null(dots$thresh))
               warning("Argument ", sQuote("thresh"), " is ignored for Matern Cluster model")
             return(scale)
         },
         checkclustargs = function(margs, old = TRUE) list(),
         resolvedots = function(...){
           return(list(...))
         },
         kernel = function(par, rvals, ...) {
             scale <- par[2L]
             ifelse(rvals>scale, 0, 1/(pi*scale^2))
         },
         isPCP=TRUE,
         K = function(par,rvals, ..., funaux){
           if(any(par <= 0))
             return(rep.int(Inf, length(rvals)))
           kappa <- par[1L]
           R <- par[2L]
           Hfun <- funaux$Hfun
           y <- pi * rvals^2 + (1/kappa) * Hfun(rvals/(2 * R))
           return(y)
         },
         pcf= function(par,rvals, ..., funaux){
             if(any(par <= 0))
               return(rep.int(Inf, length(rvals)))
             kappa <- par[1L]
             R <- par[2L]
             g <- funaux$g
             y <- 1 + (1/(pi * kappa * R^2)) * g(rvals/(2 * R))
             return(y)
           },
         funaux=list(
           Hfun=function(zz) {
             ok <- (zz < 1)
             h <- numeric(length(zz))
             h[!ok] <- 1
             z <- zz[ok]
             h[ok] <- 2 + (1/pi) * (
                                    (8 * z^2 - 4) * acos(z)
                                    - 2 * asin(z)
                                    + 4 * z * sqrt((1 - z^2)^3)
                                    - 6 * z * sqrt(1 - z^2)
                                    )
             return(h)
           },
           DOH=function(zz) {
             ok <- (zz < 1)
             h <- numeric(length(zz))
             h[!ok] <- 0
             z <- zz[ok]
             h[ok] <- (16/pi) * (z * acos(z) - (z^2) * sqrt(1 - z^2))
             return(h)
           },
           ## g(z) = DOH(z)/z has a limit at z=0.
           g=function(zz) {
             ok <- (zz < 1)
             h <- numeric(length(zz))
             h[!ok] <- 0
             z <- zz[ok]
             h[ok] <- (2/pi) * (acos(z) - z * sqrt(1 - z^2))
             return(h)
           }),
         ## sensible starting paramters
         selfstart = function(X) {
           kappa <- intensity(X)
           R <- 2 * mean(nndist(X)) 
           c(kappa=kappa, R=R)
         },
         ## meaningful model parameters
         interpret = function(par, lambda) {
           kappa <- par[["kappa"]]
           R     <- par[["R"]]
           mu    <- if(is.numeric(lambda) && length(lambda) == 1)
             lambda/kappa else NA           
           c(kappa=kappa, R=R, mu=mu)
         }
         ),
       ## ...............................................
       Cauchy=list(
         ## Neyman-Scott with Cauchy clusters: old par = (kappa, eta2) (internally used everywhere)
         ## Neyman-Scott with Cauchy clusters: new par = (kappa, scale) (officially recommended for input/output)
         modelname = "Neyman-Scott process with Cauchy kernel", # In modelname field of mincon fv obj.
         descname = "Neyman-Scott process with Cauchy kernel", # In desc field of mincon fv obj.
         modelabbrev = "Cauchy process", # In fitted obj.
         printmodelname = function(...) "Cauchy process", # Used by print.kppm
         parnames = c("kappa", "eta2"),
         clustargsnames = NULL,
         checkpar = function(par, old = TRUE){
             if(is.null(par))
                 par <- c(kappa=1,scale=1)
             if(any(par<=0))
                 stop("par values must be positive.")
             nam <- check.named.vector(par, c("kappa","eta2"), onError="null")
             if(is.null(nam)) {
                 check.named.vector(par, c("kappa","scale"))
                 names(par)[2L] <- "eta2"
                 par[2L] <- (2*par[2L])^2
             }
             if(!old){
                 names(par)[2L] <- "scale"
                 par[2L] <- sqrt(par[2L])/2
             }
             return(par)
         },
         checkclustargs = function(margs, old = TRUE) list(),
         resolvedots = function(...){
           return(list(...))
         },
         # density function for the distance to offspring
         ddist = function(r, scale, ...) {
             r/(scale^2) *  (1 + (r / scale)^2)^(-3/2)
         },
         ## Practical range of clusters
         range = function(...){
             dots <- list(...)
             # Choose the first of the possible supplied values for scale:
             scale <- c(dots$scale, dots$par[["scale"]])[1L]
             if(is.null(scale))
                 stop("Argument ", sQuote("scale"), " must be given.")
             thresh <- dots$thresh %orifnull% 0.01
             ## integral of ddist(r) dr is 1 - (1+(r/scale)^2)^(-1/2)
             ## solve for integral = 1-thresh:
             rmax <- scale * sqrt(1/thresh^2 - 1)
             ## old code
             ## ddist <- .Spatstat.ClusterModelInfoTable$Cauchy$ddist
             ## kernel0 <- clusterkernel("Cauchy", scale = scale)(0,0)
             ## f <- function(r) ddist(r, scale = scale)-thresh*kernel0
             ## rmax <- uniroot(f, lower = scale, upper = 1000 * scale)$root
             return(rmax)
         },
         kernel = function(par, rvals, ...) {
             scale <- sqrt(par[2L])/2
             1/(2*pi*scale^2)*((1 + (rvals/scale)^2)^(-3/2))
         },
         isPCP=TRUE,
         K = function(par,rvals, ...){
           if(any(par <= 0))
             return(rep.int(Inf, length(rvals)))
           pi*rvals^2 + (1 - 1/sqrt(1 + rvals^2/par[2L]))/par[1L]
         },
         pcf= function(par,rvals, ...){
           if(any(par <= 0))
             return(rep.int(Inf, length(rvals)))
           1 + ((1 + rvals^2/par[2L])^(-1.5))/(2 * pi * par[2L] * par[1L])
         },
         selfstart = function(X) {
           kappa <- intensity(X)
           eta2 <- 4 * mean(nndist(X))^2
           c(kappa = kappa, eta2 = eta2)
         },
         ## meaningful model parameters
         interpret = function(par, lambda) {
           kappa <- par[["kappa"]]
           omega <- sqrt(par[["eta2"]])/2
           mu <- if(is.numeric(lambda) && length(lambda) == 1)
             lambda/kappa else NA
           c(kappa=kappa, omega=omega, mu=mu)
         }
         ),
       ## ...............................................
       VarGamma=list(
         ## Neyman-Scott with VarianceGamma/Bessel clusters: old par = (kappa, eta) (internally used everywhere)
         ## Neyman-Scott with VarianceGamma/Bessel clusters: new par = (kappa, scale) (officially recommended for input/output)
         modelname = "Neyman-Scott process with Variance Gamma kernel", # In modelname field of mincon fv obj.
         descname = "Neyman-Scott process with Variance Gamma kernel", # In desc field of mincon fv obj.
         modelabbrev = "Variance Gamma process", # In fitted obj.
         printmodelname = function(obj){ # Used by print.kppm
             paste0("Variance Gamma process (nu=",
                    signif(obj$clustargs[["nu"]], 2), ")")
         },
         parnames = c("kappa", "eta"),
         clustargsnames = "nu",
         checkpar = function(par, old = TRUE){
             if(is.null(par))
                 par <- c(kappa=1,scale=1)
             if(any(par<=0))
                 stop("par values must be positive.")
             nam <- check.named.vector(par, c("kappa","eta"), onError="null")
             if(is.null(nam)) {
               check.named.vector(par, c("kappa","scale"))
               names(par)[2L] <- "eta"
             }
             if(!old) names(par)[2L] <- "scale"
             return(par)
         },
         checkclustargs = function(margs, old = TRUE){
             if(!old)
                 margs <- list(nu=margs$nu.ker)
             return(margs)
         },
         resolvedots = function(...){
           ## resolve dots for kppm and friends allowing for old/new par syntax
           dots <- list(...)
           out <- list()
           nu <- dots$nu
           if(is.null(nu)){
             nu <- try(resolve.vargamma.shape(nu.ker=dots$nu.ker,
                                              nu.pcf=dots$nu.pcf)$nu.ker,
                       silent = TRUE)
             if(inherits(nu, "try-error"))
               nu <- -1/4
           } else {
             check.1.real(nu)
             stopifnot(nu > -1/2)
           }
           out$margs <- list(nu.ker=nu, nu.pcf=2*nu+1)
           out$covmodel <- list(type="Kernel", model="VarGamma",
                                margs=out$margs)
           return(out)
         },
         # density function for the distance to offspring
         ddist = function(r, scale, nu, ...) {
             numer <- ((r/scale)^(nu+1)) * besselK(r/scale, nu)
             numer[r==0] <- 0
             denom <- (2^nu) * scale * gamma(nu + 1)
             numer/denom
         },
         ## Practical range of clusters
         range = function(...){
             dots <- list(...)
             # Choose the first of the possible supplied values for scale:
             scale <- c(dots$scale, dots$par[["scale"]])[1L]
             if(is.null(scale))
                 stop("Argument ", sQuote("scale"), " must be given.")
             # Find value of nu:
             extra <- .Spatstat.ClusterModelInfoTable$VarGamma$resolvedots(...)
             nu <- .Spatstat.ClusterModelInfoTable$VarGamma$checkclustargs(extra$margs, old=FALSE)$nu
             if(is.null(nu))
                 stop("Argument ", sQuote("nu"), " must be given.")
             thresh <- dots$thresh
             if(is.null(thresh))
                 thresh <- .001
             ddist <- .Spatstat.ClusterModelInfoTable$VarGamma$ddist
             f1 <- function(rmx) {
               integrate(ddist, 0, rmx, scale=scale, nu=nu)$value - (1 - thresh)
             }
             f <- Vectorize(f1)
             ## old code
             ## kernel0 <- clusterkernel("VarGamma", scale = scale, nu = nu)(0,0)
             ## f <- function(r) ddist(r, scale = scale, nu = nu) - thresh*kernel0
             rmax <- uniroot(f, lower = scale, upper = 1000 * scale)$root
             return(rmax)
         },
         ## kernel function in polar coordinates (no angular argument).
         kernel = function(par, rvals, ..., margs) {
             scale <- as.numeric(par[2L])
             nu <- margs$nu
             if(is.null(nu))
                 stop("Argument ", sQuote("nu"), " is missing.")
             numer <- ((rvals/scale)^nu) * besselK(rvals/scale, nu)
             numer[rvals==0] <- ifelse(nu>0, 2^(nu-1)*gamma(nu), Inf)
             denom <- pi * (2^(nu+1)) * scale^2 * gamma(nu + 1)
             numer/denom
         },
         isPCP=TRUE,
         K = local({
           ## K function requires integration of pair correlation
           xgx <- function(x, par, nu.pcf) {
             ## x * pcf(x) without check on par values
             numer <- (x/par[2L])^nu.pcf * besselK(x/par[2L], nu.pcf)
             denom <- 2^(nu.pcf+1) * pi * par[2L]^2 * par[1L] * gamma(nu.pcf + 1)
             return(x * (1 + numer/denom))
           }
           vargammaK <- function(par,rvals, ..., margs){
             ## margs = list(.. nu.pcf.. ) 
             if(any(par <= 0))
               return(rep.int(Inf, length(rvals)))
             nu.pcf <- margs$nu.pcf
             out <- numeric(length(rvals))
             ok <- (rvals > 0)
             rvalsok <- rvals[ok]
             outok <- numeric(sum(ok))
             for (i in 1:length(rvalsok))
               outok[i] <- 2 * pi * integrate(xgx,
                                              lower=0, upper=rvalsok[i],
                                              par=par, nu.pcf=nu.pcf)$value
             out[ok] <- outok
             return(out)
           }
           ## Initiated integration in sub-subintervals, but it is unfinished!
           ## vargammaK <- function(par,rvals, ..., margs){
           ##   ## margs = list(.. nu.pcf.. ) 
           ##   if(any(par <= 0))
           ##     return(rep.int(Inf, length(rvals)))
           ##   nu.pcf <- margs$nu.pcf
           ##   out <- numeric(length(rvals))
           ##   out[1L] <- if(rvals[1L] == 0) 0 else 
           ##   integrate(xgx, lower=0, upper=rvals[1L],
           ##             par = par, nu.pcf=nu.pcf)$value
           ##   for (i in 2:length(rvals)) {
           ##     delta <- integrate(xgx,
           ##                        lower=rvals[i-1L], upper=rvals[i],
           ##                        par=par, nu.pcf=nu.pcf)
           ##     out[i]=out[i-1L]+delta$value
           ##   }
           ##   return(out)
           ## }
           vargammaK
           }), ## end of 'local'
         pcf= function(par,rvals, ..., margs){
           ## margs = list(..nu.pcf..)
           if(any(par <= 0))
             return(rep.int(Inf, length(rvals)))
           nu.pcf <- margs$nu.pcf
           sig2 <- 1 / (4 * pi * (par[2L]^2) * nu.pcf * par[1L])
           denom <- 2^(nu.pcf - 1) * gamma(nu.pcf)
           rr <- rvals / par[2L]
           ## Matern correlation function
           fr <- ifelseXB(rr > 0,
                        (rr^nu.pcf) * besselK(rr, nu.pcf) / denom,
                        1)
           return(1 + sig2 * fr)
         },
         parhandler = function(..., nu.ker = -1/4) {
           check.1.real(nu.ker)
           stopifnot(nu.ker > -1/2)
           nu.pcf <- 2 * nu.ker + 1
           return(list(type="Kernel",
                       model="VarGamma",
                       margs=list(nu.ker=nu.ker,
                                  nu.pcf=nu.pcf)))
         },
         ## sensible starting values
         selfstart = function(X) {
           kappa <- intensity(X)
           eta <- 2 * mean(nndist(X))
           c(kappa=kappa, eta=eta)
         },
         ## meaningful model parameters
         interpret = function(par, lambda) {
           kappa <- par[["kappa"]]
           omega <- par[["eta"]]
           mu <- if(is.numeric(lambda) && length(lambda) == 1)
             lambda/kappa else NA
           c(kappa=kappa, omega=omega, mu=mu)
         }
         ),
       ## ...............................................
       LGCP=list(
         ## Log Gaussian Cox process: old par = (sigma2, alpha) (internally used everywhere)
         ## Log Gaussian Cox process: new par = (var, scale) (officially recommended for input/output)
         modelname = "Log-Gaussian Cox process", # In modelname field of mincon fv obj.
         descname = "LGCP", # In desc field of mincon fv obj.
         modelabbrev = "log-Gaussian Cox process", # In fitted obj.
         printmodelname = function(...) "log-Gaussian Cox process", # Used by print.kppm
         parnames = c("sigma2", "alpha"),
         checkpar = function(par, old = TRUE){
             if(is.null(par))
                 par <- c(var=1,scale=1)
             if(any(par<=0))
                 stop("par values must be positive.")
             nam <- check.named.vector(par, c("sigma2","alpha"), onError="null")
             if(is.null(nam)) {
                 check.named.vector(par, c("var","scale"))
                 names(par) <- c("sigma2", "alpha")
             }
             if(!old) names(par) <- c("var", "scale")
             return(par)
         },
         checkclustargs = function(margs, old = TRUE) return(margs),
         resolvedots = function(...){
           ## resolve dots for kppm and friends allowing for old/new par syntax
           dots <- list(...)
           nam <- names(dots)
           out <- list()
           cmod <- dots$covmodel
           model <- cmod$model %orifnull% dots$model %orifnull% "exponential"
           margs <- NULL
           if(!identical(model, "exponential")) {
             ## get the 'model generator' 
             modgen <- getRandomFieldsModelGen(model)
             attr(model, "modgen") <- modgen
             if(is.null(cmod)){
               margsnam <- names(formals(modgen))
               margsnam <- margsnam[!(margsnam %in% c("var", "scale"))]
               margs <- dots[nam %in% margsnam]
             } else{
               margs <- cmod[names(cmod)!="model"]
             }
           }
           if(length(margs)==0) {
             margs <- NULL
           } else {
             ## detect anisotropic model
             if("Aniso" %in% names(margs))
               stop("Anisotropic covariance models cannot be used",
                    call.=FALSE)
           }
           out$margs <- margs
           out$model <- model
           out$covmodel <- list(type="Covariance", model=model, margs=margs)
           return(out)
         },
         isPCP=FALSE,
         ## calls relevant covariance function from RandomFields package
         K = function(par, rvals, ..., model, margs) {
           if(any(par <= 0))
             return(rep.int(Inf, length(rvals)))
           if(model == "exponential") {
             ## For efficiency and to avoid need for RandomFields package
             integrand <- function(r,par,...) 2*pi*r*exp(par[1L]*exp(-r/par[2L]))
           } else {
             kraeverRandomFields()
             integrand <- function(r,par,model,margs) {
               modgen <- attr(model, "modgen")
               if(length(margs) == 0) {
                 mod <- modgen(var=par[1L], scale=par[2L])
               } else {
                 mod <- do.call(modgen,
                                append(list(var=par[1L], scale=par[2L]),
                                       margs))
               }
               2*pi *r *exp(RandomFields::RFcov(model=mod, x=r))
             }
           }
           nr <- length(rvals)
           th <- numeric(nr)
           if(spatstat.options("fastK.lgcp")) {
             ## integrate using Simpson's rule
             fvals <- integrand(r=rvals, par=par, model=model, margs=margs)
             th[1L] <- rvals[1L] * fvals[1L]/2
             if(nr > 1)
               for(i in 2:nr)
                 th[i] <- th[i-1L] +
                   (rvals[i] - rvals[i-1L]) * (fvals[i] + fvals[i-1L])/2
           } else {
             ## integrate using 'integrate'
             th[1L] <- if(rvals[1L] == 0) 0 else 
             integrate(integrand,lower=0,upper=rvals[1L],
                       par=par,model=model,margs=margs)$value
             for (i in 2:length(rvals)) {
               delta <- integrate(integrand,
                                  lower=rvals[i-1L],upper=rvals[i],
                                  par=par,model=model,margs=margs)
               th[i]=th[i-1L]+delta$value
             }
           }
           return(th)
         },
         pcf= function(par, rvals, ..., model, margs) {
           if(any(par <= 0))
             return(rep.int(Inf, length(rvals)))
           if(model == "exponential") {
             ## For efficiency and to avoid need for RandomFields package
             gtheo <- exp(par[1L]*exp(-rvals/par[2L]))
           } else {
             kraeverRandomFields()
             modgen <- attr(model, "modgen")
             if(length(margs) == 0) {
               mod <- modgen(var=par[1L], scale=par[2L])
             } else {
               mod <- do.call(modgen,
                              append(list(var=par[1L], scale=par[2L]),
                                     margs))
             }
             gtheo <- exp(RandomFields::RFcov(model=mod, x=rvals))
           }
           return(gtheo)
         },
         parhandler=function(model = "exponential", ...) {
           if(!is.character(model))
             stop("Covariance function model should be specified by name")
           margs <- c(...)
           if(!identical(model, "exponential")) {
             ## get the 'model generator' 
             modgen <- getRandomFieldsModelGen(model)
             attr(model, "modgen") <- modgen
           }
           return(list(type="Covariance", model=model, margs=margs))
         },
         ## sensible starting values
         selfstart = function(X) {
           alpha <- 2 * mean(nndist(X))
           c(sigma2=1, alpha=alpha)
         },
         ## meaningful model parameters
         interpret = function(par, lambda) {
           sigma2 <- par[["sigma2"]]
           alpha  <- par[["alpha"]]
           mu <- if(is.numeric(lambda) && length(lambda) == 1 && lambda > 0)
             log(lambda) - sigma2/2 else NA
           c(sigma2=sigma2, alpha=alpha, mu=mu)
         }
         )
  )

spatstatClusterModelInfo <- function(name, onlyPCP = FALSE) {
  if(inherits(name, "detpointprocfamily"))
    return(spatstatDPPModelInfo(name))
  if(!is.character(name) || length(name) != 1)
    stop("Argument must be a single character string", call.=FALSE)
  TheTable <- .Spatstat.ClusterModelInfoTable
  nama2 <- names(TheTable)
  if(onlyPCP){
    ok <- sapply(TheTable, getElement, name="isPCP")
    nama2 <- nama2[ok]
  } 
  if(!(name %in% nama2))
    stop(paste(sQuote(name), "is not recognised;",
               "valid names are", commasep(sQuote(nama2))),
         call.=FALSE)
  out <- TheTable[[name]]
  return(out)
}

#
#   rmhexpand.R
#
#   Rules/data for expanding the simulation window in rmh
#
#   $Revision: 1.8 $  $Date: 2016/02/11 10:17:12 $
#

# Establish names and rules for each type of expansion

RmhExpandRule <- local({

  .RmhExpandTable <-
  list(area=list(descrip ="Area expansion factor",
                 minval = 1,
                 expands = function(x) { unname(x) > 1 }),
       length=list(descrip ="Length expansion factor",
                   minval = 1,
                   expands = function(x) { unname(x) > 1 }),
       distance=list(descrip="Expansion buffer distance",
                     minval = 0,
                     expands = function(x) { unname(x) > 0 }))
  
  RmhExpandRule <- function(nama) {
    if(length(nama) == 0) nama <- "area"
    if(length(nama) > 1)
      stop("Internal error: too many names in RmhExpandRule", call.=FALSE)
    if(!(nama %in% names(.RmhExpandTable)))
      stop(paste("Internal error: unrecognised expansion type",
                 sQuote(nama)),
           call.=FALSE)
    return(.RmhExpandTable[[nama]])
  }
  RmhExpandRule
})
  
                        
rmhexpand <- function(x=NULL, ..., area=NULL, length=NULL, distance=NULL) {
  trap.extra.arguments(..., .Context="In rmhexpand")
  # check for incompatibility
  n <- (!is.null(x)) + (!is.null(area)) +
       (!is.null(length)) + (!is.null(distance))
  if(n > 1) stop("Only one argument should be given")
  # absorb other arguments into 'x'
  if(is.null(x) && n > 0) {
      if(!is.null(area)) x <- c(area=area)
      if(!is.null(length)) x <- c(length=length)
      if(!is.null(distance)) x <- c(distance=distance)
  }
  if(is.null(x)) {
    # No expansion rule supplied.
    # Use spatstat default, indicating that the user did not choose it.
    force.exp <- force.noexp <- FALSE
    x <- spatstat.options("expand")
    x <- rmhexpand(x)$expand
  } else {
    # process x
    if(inherits(x, "rmhexpand"))
      return(x)
    if(is.owin(x)) {
      force.exp <- TRUE
      force.noexp <- FALSE
    } else {
      # expecting c(name=value) or list(name=value)
      if(is.list(x))
        x <- unlist(x)
      if(!is.numeric(x))
        stop(paste("Expansion argument must be either",
                   "a number, a window, or NULL.\n"))
      # x is numeric
      check.1.real(x, "In rmhexpand(x)")
      explain.ifnot(is.finite(x), "In rmhexpand(x)")
      # an unlabelled numeric value is interpreted as an area expansion factor
      if(!any(nzchar(names(x))))
        names(x) <- "area"
      # validate
      rule <- RmhExpandRule(names(x))
      if(x < rule$minval) {
        warning(paste(rule$descrip, "<", rule$minval,
                      "has been reset to", rule$minval),
                call.=FALSE)
        x[] <- rule$minval
      }
      force.exp <- rule$expands(x)
      force.noexp <- !force.exp
    }
  }
  result <- list(expand=x, force.exp=force.exp, force.noexp=force.noexp)
  class(result) <- "rmhexpand"
  return(result)
}

.no.expansion <- list(expand=c(area=1), force.exp=FALSE, force.noexp=TRUE)
class(.no.expansion) <- "rmhexpand"

print.rmhexpand <- function(x, ..., prefix=TRUE) {
  if(prefix) cat("Expand the simulation window? ")
  if(x$force.noexp) {
    cat("No.\n")
  } else {
    if(x$force.exp) cat("Yes:\n") else cat("Not determined. Default is:\n")

    y <- x$expand
    if(is.null(y)) {
      print(rmhexpand(spatstat.options("expand")), prefix=FALSE)
    } else if(is.numeric(y)) {
      descrip <- RmhExpandRule(names(y))$descrip
      cat(paste("\t", descrip, unname(y), "\n"))
    } else {
      print(y)
    }
  }
  return(invisible(NULL))
}

summary.rmhexpand <- function(object, ...) {
  decided <- with(object, force.exp || force.noexp)
  ex <- object$expand
  if(is.null(ex))
    ex <- rmhexpand(spatstat.options("expand"))$expand
  if(is.owin(ex)) {
    willexpand <- TRUE
    descrip <- "Window"
  } else if(is.numeric(ex)) {
    rule <- RmhExpandRule(names(ex))
    descrip    <- rule$descrip
    willexpand <- if(object$force.exp) TRUE else
                  if(object$force.noexp) FALSE else
                  (unname(ex) > rule$minval)
  } else stop("Internal error: unrecognised format in summary.rmhexpand",
              call.=FALSE)
              
  out <- list(rule.decided=decided,
              window.decided=decided && is.owin(ex), 
              expand=ex,
              descrip=descrip,
              willexpand=willexpand)
  class(out) <- "summary.rmhexpand"
  return(out)
}

print.summary.rmhexpand <- function(x, ...) {
  cat("Expansion rule\n")
  ex <- x$expand
  if(x$window.decided) {
    cat("Window is decided.\n")
    print(ex)
  } else {
    if(x$rule.decided) {
      cat("Rule is decided.\n")
    } else {
      cat("Rule is not decided.\nDefault is:\n")
    }
    if(!x$willexpand) {
      cat("No expansion\n")
    } else {
      if(is.numeric(ex)) cat(paste(x$descrip, ex, "\n")) else print(ex)
    }
  }
  return(invisible(NULL))
}

expand.owin <- function(W, ...) {
  ex <- list(...)
  if(length(ex) > 1) stop("Too many arguments")
  # get an rmhexpand object
  if(inherits(ex[[1]], "rmhexpand")) {
    ex <- ex[[1]]
  } else ex <- do.call(rmhexpand, ex)
  f <- ex$expand
  if(is.null(f)) return(W)
  if(is.owin(f)) return(f)
  if(!is.numeric(f)) stop("Format not understood")
  switch(names(f),
         area = {
           if(f == 1)
             return(W)
           bb <- boundingbox(W)
           xr <- bb$xrange
           yr <- bb$yrange
           fff <- (sqrt(f) - 1)/2
           Wexp <- grow.rectangle(bb, fff * diff(xr), fff * diff(yr))
         },
         length = {
           if(f == 1)
             return(W)
           bb <- boundingbox(W)
           xr <- bb$xrange
           yr <- bb$yrange
           fff <- (f - 1)/2
           Wexp <- grow.rectangle(bb, fff * diff(xr), fff * diff(yr))
         },
         distance = {
           if(f == 0)
             return(W)
           Wexp <- if(is.rectangle(W)) grow.rectangle(W, f) else dilation(W, f)
         },
         stop("Internal error: unrecognised type")
         )
  return(Wexp)
}

will.expand <- function(x) {
  stopifnot(inherits(x, "rmhexpand"))
  if(x$force.exp) return(TRUE)
  if(x$force.noexp) return(FALSE)
  return(summary(x)$willexpand)
}

is.expandable <- function(x) { UseMethod("is.expandable") }

change.default.expand <- function(x, newdefault) {
  stopifnot(inherits(x, "rmhexpand"))
  decided <- with(x, force.exp || force.noexp)
  if(!decided)
    x$expand <- rmhexpand(newdefault)$expand
  return(x)
}

  

#
#      distances.R
#
#      $Revision: 1.47 $     $Date: 2018/10/07 11:07:54 $
#
#
#      Interpoint distances between pairs 
#
#

pairdist <- function(X, ...) {
  UseMethod("pairdist")
}

pairdist.ppp <- function(X, ..., periodic=FALSE, method="C", squared=FALSE) {
  verifyclass(X, "ppp")
  if(!periodic)
    return(pairdist.default(X$x, X$y, method=method, squared=squared))
  # periodic case
  W <- X$window
  if(W$type != "rectangle")
    stop(paste("periodic edge correction can't be applied",
               "in a non-rectangular window"))
  wide <- diff(W$xrange)
  high <- diff(W$yrange)
  return(pairdist.default(X$x, X$y, period=c(wide,high),
                          method=method, squared=squared))
}


pairdist.default <-
  function(X, Y=NULL, ..., period=NULL, method="C", squared=FALSE)
{
  if(!is.null(dim(X)) && ncol(X) > 2)
    stop("Data contain more than 2 coordinates")
  
  xy <- xy.coords(X,Y)[c("x","y")]

  if(identical(xy$xlab, "Index")) 
    stop("Cannot interpret data as 2-dimensional coordinates")

  x <- xy$x
  y <- xy$y

  n <- length(x)
  if(length(y) != n)
    stop("lengths of x and y do not match")

  # special cases
  if(n == 0)
    return(matrix(numeric(0), nrow=0, ncol=0))
  else if(n == 1L)
    return(matrix(0,nrow=1L,ncol=1L))

  if((periodic<- !is.null(period))) {
    stopifnot(is.numeric(period))
    stopifnot(length(period) == 2 || length(period) == 1)
    stopifnot(all(period > 0))
    if(length(period) == 1) period <- rep.int(period, 2)
    wide <- period[1L]
    high <- period[2L]
  }

  switch(method,
         interpreted={
           xx <- matrix(rep.int(x, n), nrow = n)
           yy <- matrix(rep.int(y, n), nrow = n)
           if(!periodic) {
             d2 <- (xx - t(xx))^2 + (yy - t(yy))^2
           } else {
             dx <- xx - t(xx)
             dy <- yy - t(yy)
             dx2 <- pmin.int(dx^2, (dx + wide)^2, (dx - wide)^2)
             dy2 <- pmin.int(dy^2, (dy + high)^2, (dy - high)^2)
             d2 <- dx2 + dy2
           }
           if(squared)
             dout <- d2
           else
             dout <- sqrt(d2)
         },
         C={
           d <- numeric( n * n)
           if(!periodic) {
               z<- .C("Cpairdist",
                      n = as.integer(n),
                      x= as.double(x),
                      y= as.double(y),
                      squared=as.integer(squared),
                      d= as.double(d),
                      PACKAGE = "spatstat")
           } else {
             z <- .C("CpairPdist",
                     n = as.integer(n),
                     x= as.double(x),
                     y= as.double(y),
                     xwidth=as.double(wide),
                     yheight=as.double(high),
                     squared = as.integer(squared),
                     d= as.double(d),
                     PACKAGE = "spatstat")
           }
           dout <- matrix(z$d, nrow=n, ncol=n)
         },
         stop(paste("Unrecognised method", sQuote(method)))
       )
  return(dout)
}


crossdist <- function(X, Y, ...) {
  UseMethod("crossdist")
}

crossdist.ppp <- function(X, Y, ...,
                          periodic=FALSE, method="C", squared=FALSE) {
  verifyclass(X, "ppp")
  Y <- as.ppp(Y)
  if(!periodic)
    return(crossdist.default(X$x, X$y, Y$x, Y$y,
                             method=method, squared=squared))
  # periodic case
  WX <- X$window
  WY <- Y$window
  if(WX$type != "rectangle" || WY$type != "rectangle")
    stop(paste("periodic edge correction can't be applied",
               "in non-rectangular windows"))
  if(!is.subset.owin(WX,WY) || !is.subset.owin(WY, WX))
    stop(paste("periodic edge correction is not implemented",
               "for the case when X and Y lie in different rectangles"))
  wide <- diff(WX$xrange)
  high <- diff(WX$yrange)
  return(crossdist.default(X$x, X$y, Y$x, Y$y,
                           period=c(wide,high),
                           method=method, squared=squared))
}

crossdist.default <-
  function(X, Y, x2, y2, ..., period=NULL, method="C", squared=FALSE)
{
  x1 <- X
  y1 <- Y
  # returns matrix[i,j] = distance from (x1[i],y1[i]) to (x2[j],y2[j])
  if(length(x1) != length(y1))
    stop("lengths of x and y do not match")
  if(length(x2) != length(y2))
    stop("lengths of x2 and y2 do not match")
  n1 <- length(x1)
  n2 <- length(x2)
  if(n1 == 0 || n2 == 0)
    return(matrix(numeric(0), nrow=n1, ncol=n2))

  if((periodic<- !is.null(period))) {
    stopifnot(is.numeric(period))
    stopifnot(length(period) == 2 || length(period) == 1)
    stopifnot(all(period > 0))
    if(length(period) == 1L) period <- rep.int(period, 2)
    wide <- period[1L]
    high <- period[2L]
  }

   switch(method,
         interpreted = {
                 X1 <- matrix(rep.int(x1, n2), ncol = n2)
                 Y1 <- matrix(rep.int(y1, n2), ncol = n2)
                 X2 <- matrix(rep.int(x2, n1), ncol = n1)
                 Y2 <- matrix(rep.int(y2, n1), ncol = n1)
                 if(!periodic) 
                   d2 <- (X1 - t(X2))^2 + (Y1 - t(Y2))^2
                 else {
                   dx <- X1 - t(X2)
                   dy <- Y1 - t(Y2)
                   dx2 <- pmin.int(dx^2, (dx + wide)^2, (dx - wide)^2)
                   dy2 <- pmin.int(dy^2, (dy + high)^2, (dy - high)^2)
                   d2 <- dx2 + dy2
                 }
                 return(if(squared) d2 else sqrt(d2))
               },
               C = {
                 if(!periodic) {
                   z<- .C("Ccrossdist",
                          nfrom = as.integer(n1),
                          xfrom = as.double(x1),
                          yfrom = as.double(y1),
                          nto = as.integer(n2),
                          xto = as.double(x2),
                          yto = as.double(y2),
                          squared = as.integer(squared),
                          d = as.double(matrix(0, nrow=n1, ncol=n2)),
                          PACKAGE = "spatstat")
                 } else {
                   z<- .C("CcrossPdist",
                          nfrom = as.integer(n1),
                          xfrom = as.double(x1),
                          yfrom = as.double(y1),
                          nto = as.integer(n2),
                          xto = as.double(x2),
                          yto = as.double(y2),
                          xwidth = as.double(wide),
                          yheight = as.double(high),
                          squared = as.integer(squared),
                          d = as.double(matrix(0, nrow=n1, ncol=n2)),
                          PACKAGE = "spatstat")
                 }
                 return(matrix(z$d, nrow=n1, ncol=n2))
               },
               stop(paste("Unrecognised method", method))
               )
      }

## hexagons.R
## $Revision: 1.6 $ $Date: 2017/02/07 07:35:32 $

hexgrid <- function(W, s, offset=c(0,0), origin=NULL, trim=TRUE) {
  W <- as.owin(W)
  check.1.real(s)
  stopifnot(s > 0)
  hstep <- 3 * s
  vstep <- sqrt(3) * s
  R <- grow.rectangle(as.rectangle(W), hstep)
  xr <- R$xrange
  yr <- R$yrange
  ## initial positions for 'odd' and 'even grids
  p0 <- as2vector(origin %orifnull% centroid.owin(R))
  p0 <- p0 + as2vector(offset)
  q0 <- p0 + c(hstep, vstep)/2
  ## 'even' points
  p0 <- c(startinrange(p0[1L], hstep, xr),
          startinrange(p0[2L], vstep, yr))
  if(!anyNA(p0)) {
    xeven <- prolongseq(p0[1L], xr, step=hstep)
    yeven <- prolongseq(p0[2L], yr, step=vstep)
    xyeven <- expand.grid(x=xeven, y=yeven)
  } else xyeven <- list(x=numeric(0), y=numeric(0))
  ## 'odd' points
  q0 <- c(startinrange(q0[1L], hstep, xr),
          startinrange(q0[2L], vstep, yr))
  if(!anyNA(q0)) {
    xodd <- prolongseq(q0[1L], xr, step=hstep)
    yodd <- prolongseq(q0[2L], yr, step=vstep)
    xyodd <- expand.grid(x=xodd, y=yodd)
  } else xyodd <- list(x=numeric(0), y=numeric(0))
  ##
  xy <- concatxy(xyeven, xyodd)
  XY <- as.ppp(xy, W=R)
  ##
  if(trim) return(XY[W])
  ok <- inside.owin(XY, w=dilation.owin(W, s))
  return(XY[ok])
}

hextess <- function(W, s, offset=c(0,0), origin=NULL, trim=TRUE) {
  W <- as.owin(W)
  G <- hexgrid(W=W, s=s, offset=offset, origin=origin, trim=FALSE)
  if(trim && is.mask(W)) {
    ## Result is a pixel image tessellation
    ## Determine pixel resolution by extending 'W' to larger domain of 'G'
    rasta <- harmonise.im(as.im(1, W), as.owin(G))[[1L]]
    rasta <- as.mask(rasta)
    ## Tweak G to have mask window
    G$window <- rasta
    ##
    img <- nnmap(G, what="which")
    result <- tess(image=img)
    return(result)
  }
  ## Result is a polygonal tessellation
  Gxy <- as.matrix(as.data.frame(G))
  n <- nrow(Gxy)
  ## Hexagon centred at origin
  hex0 <- disc(npoly=6, radius=s)
  ## Form hexagons
  hexes <- vector(mode="list", length=n)
  for(i in 1:n) 
    hexes[[i]] <- shift(hex0, Gxy[i,])
  ## Determine whether tiles intersect window wholly or partly
  suspect <- rep(TRUE, n)
  GW <- G[W]
  GinW <- inside.owin(G, w=W) 
  suspect[GinW] <- (bdist.points(GW) <= s)
  ## Compute intersection of tiles with window
  trimmed <- hexes
  trimmed[suspect] <- trimmed.suspect <- 
    lapply(trimmed[suspect], intersect.owin, B=W, fatal=FALSE)
  nonempty <- rep(TRUE, n)
  nonempty[suspect] <- !unlist(lapply(trimmed.suspect, is.empty))
  if(trim) {
    ## return the tiles intersected with W
    result <- tess(tiles=trimmed[nonempty], window=W)
  } else {
    ## return the tiles that have nonempty intersection with W
    result <- tess(tiles=hexes[nonempty])
  }
  return(result)
}


  
  
  

##
## symbolmap.R
##
##   $Revision: 1.37 $  $Date: 2020/01/11 04:45:05 $
##

symbolmap <- local({

  known.unknowns <- c("shape", "pch", "chars",
                      "size", "cex",
                      "direction", "arrowtype", "headlength", "headangle", 
                      "col", "cols", "fg", "bg",
                      "lty", "lwd", "border", "fill",
                      "etch")

  trycolourmap <- function(...) {
    try(colourmap(...), silent=TRUE)
  }

  symbolmap <- function(..., range=NULL, inputs=NULL) {
    if(!is.null(range) && !is.null(inputs))
      stop("Arguments range and inputs are incompatible")
    ## graphics parameters
    parlist <- list(...)
    ## remove unrecognised parameters and NULL values 
    if(length(parlist) > 0) {
      ok <- names(parlist) %in% known.unknowns
      ok <- ok & !unlist(lapply(parlist, is.null))
      parlist <- parlist[ok]
    }
    got.pars <- (length(parlist) > 0)
    parnames <- names(parlist)
    type <- if(is.null(inputs) && is.null(range)) "constant" else
            if(!is.null(inputs)) "discrete" else "continuous"
    if(got.pars) {
      ## validate parameters
      if(is.null(parnames) || !all(nzchar(parnames)))
        stop("All graphics parameters must have names")
          atomic <- unlist(lapply(parlist, is.atomic))
      functions <- unlist(lapply(parlist, is.function))
      lenfs <- lengths(parlist)
      constants <- atomic & (lenfs == 1)
      if(any(bad <- !(constants | functions))) {
        if(type == "discrete" && any(repairable <- atomic[bad])) {
          ## recycle data to desired length
          parlist[repairable] <- lapply(parlist[repairable],
                                        reptolength,
                                        n=length(inputs))
          bad[repairable] <- FALSE
        }
        if(type == "continuous") {
          ## look for vectors of colour values
          iscol <- bad & sapply(parlist, is.colour) &
            (names(parlist) %in% c("cols", "col", "fg", "bg"))
          ## convert colour values to colour map
          if(any(iscol)) {
            cmap <- lapply(parlist[iscol], trycolourmap, range=range)
            success <- sapply(cmap, inherits, what="colourmap")
            iscol[iscol] <- success
            if(any(iscol)) {
              parlist[iscol] <- cmap[success]
              bad[iscol] <- FALSE
              functions[iscol] <- TRUE
            }
          }
        }
        nbad <- sum(bad)
        if(nbad > 0) 
          stop(paste(ngettext(nbad, "Argument", "Arguments"),
                     commasep(sQuote(parnames[bad])),
                     ngettext(nbad, "is neither a function nor a constant",
                              "are neither functions nor constants")))
      }
      if(type == "constant" && any(functions))
        type <- "continuous"
    } 
    switch(type,
           constant ={
             ## set of constant graphics parameters defining a single symbol
             stuff <- list(type=type, parlist=parlist)
             ConstantValue <- as.data.frame(parlist, stringsAsFactors=FALSE)
             f <- function(x) ConstantValue
           },
           discrete = {
             ## finite set of inputs mapped to symbols
             stuff <- list(type=type, inputs=inputs, parlist=parlist)
             f <- function(x) ApplyDiscreteSymbolMap(x, stuff)
           },
           continuous = {
             got.shape <- "shape" %in% parnames
             got.size <- "size" %in% parnames
             got.cha <- any(c("pch", "chars") %in% parnames)
             ## interval of real line (etc) mapped to symbols or characters
             if(!got.cha) {
               ## mapped to symbols
               if(!got.shape)
                 parlist$shape <- "circles"
               if(!got.size)
                 stop("Parameter 'size' is missing")
             }
             rangetype <- if(is.null(range)) "numeric" else
                          if(inherits(range, "POSIXt")) "datetime" else
                          if(inherits(range, "Date")) "date" else
                          if(is.numeric(range)) "numeric" else "unknown"
             stuff <- list(type=type, range=range, rangetype=rangetype,
                           parlist=parlist)
             f <- function(x) ApplyContinuousSymbolMap(x, stuff)
           })
    attr(f, "stuff") <- stuff
    class(f) <- c("symbolmap", class(f))
    f
  }

  reptolength <- function(z, n) { rep.int(z, n)[1:n] }
  
  MapDiscrete <- function(f, x, i) {
    if(is.function(f)) f(x) else if(length(f) == 1) rep.int(f, length(x)) else f[i]
  }
  
  MapContinuous <- function(f, x) {
    if(is.function(f)) f(x) else rep.int(f, length(x))
  }

  ApplyContinuousSymbolMap <- function(x, stuff) {
    with(stuff, {
      y <- as.data.frame(lapply(parlist, MapContinuous, x=x),
                         stringsAsFactors=FALSE)
      return(y)
    })
  }
  
  ApplyDiscreteSymbolMap <- function(x, stuff) {
    with(stuff, {
      ii <- match(x, inputs)
      if(anyNA(ii))
        stop("Some values do not belong to the domain of the symbol map")
      y <- as.data.frame(lapply(parlist, MapDiscrete, x=x, i=ii),
                         stringsAsFactors=FALSE)
      return(y)
    })
  }
  symbolmap
})

symbolmaptype <- function(x) { attr(x, "stuff")$type }

symbolmapdomain <- function(x) {
  stuff <- attr(x, "stuff")
  d <- switch(stuff$type,
              constant = { integer(0) },
              discrete = { stuff$inputs },
              continuous = { stuff$range })
  return(d)
}

update.symbolmap <- function(object, ...) {
  y <- attr(object, "stuff")
  oldargs <- append(y[["parlist"]], y[c("inputs", "range")])
  do.call(symbolmap, resolve.defaults(list(...), oldargs))
}

print.symbolmap <- function(x, ...) {
  with(attr(x, "stuff"), {
    switch(type,
           constant = {
             if(length(parlist) == 0) {
               cat("Symbol map", "with no parameters", fill=TRUE)
             } else {
               cat("Symbol map", "with constant values", fill=TRUE)
             }
           },
           discrete = {
             cat("Symbol map", "for discrete inputs:", fill=TRUE)
             print(inputs)
           },
           continuous = {
             cat("Symbol map", "for",
                 switch(rangetype,
                        numeric="real numbers",
                        date = "dates",
                        datetime = "date/time values",
                        unknown = "unrecognised data"),
                 if(!is.null(range)) paste("in", prange(range)) else NULL,
                 fill=TRUE)
           })
    if(length(parlist) > 0) {
      for(i in seq_along(parlist)) {
        cat(paste0(names(parlist)[i], ": "))
        pari <- parlist[[i]]
        if(!is.function(pari) && length(pari) == 1)
          cat(pari, fill=TRUE) else print(pari)
      }
    }
    return(invisible(NULL))
  })
}

## Function which actually plots the symbols.
## Called by plot.ppp and plot.symbolmap
## Returns maximum size of symbols

invoke.symbolmap <- local({

  ## plot points, handling various arguments
  do.points <- function(x, y, ...,
                        cex=size, size=NULL, 
                        col=cols, pch=chars, cols=NULL, chars=NULL,
                        lwd=1, etch=FALSE, 
                        do.plot=TRUE) {
    if(do.plot) {
      if(length(cex) == 0) cex <- 1
      if(length(col) == 0) col <- par("col")
      if(length(pch) == 0) pch <- 1
      if(length(lwd) == 0) lwd <- 1
      n <- length(x)
      if(length(cex) == 1) cex <- rep(cex, n)
      if(length(col) == 1) col <- rep(col, n)
      if(length(pch) == 1) pch <- rep(pch, 1)
      if(length(lwd) == 1) lwd <- rep(lwd, n)
      if(length(etch) == 1) etch <- rep(etch, n)
      ## infer which arguments are parallelised
      other <- append(list(...), list(cex=cex, pch=pch))
      isvec <- (lengths(other) == n)
      other.fixed <- other[!isvec]
      other.vec   <- other[isvec]
      ##
      if(any(i <- as.logical(etch))) {
        anti.col <- complementarycolour(col)
        anti.lwd <- if(is.numeric(etch)) etch else 2 * lwd
        do.call.matched(points.default,
                        resolve.defaults(list(x=x[i], y=y[i]),
                                         other.fixed,
                                         lapply(other.vec, "[", i=i),
                                         list(col=anti.col[i],
                                              lwd=anti.lwd[i])),
                        extrargs=c("col", "pch", "type", "bg",
                                   "cex", "lwd", "lty"))
      }
      do.call.matched(points.default,
                    resolve.defaults(list(x=x, y=y),
                                     other,
                                     list(col=col, lwd=lwd)),
                    extrargs=c("col", "pch", "type", "bg", "cex", "lwd", "lty"))
    }
    return(max(cex %orifnull% 1))
  }
  ## plot symbols likewise
  do.symbols <- function(x, y, ..., 
                         shape,
                         size=cex, cex=NULL,
                         fg=col, col=cols, cols=NULL,
                         lwd=1, etch=FALSE, do.plot=TRUE) {
    if(do.plot) {
      ## zap tiny sizes
      tiny <- (size < (max(size)/1000))
      size[tiny] <- 0
      ## collect arguments
      n <- length(x)
      if(length(lwd) == 1) lwd <- rep(lwd, n)
      if(length(etch) == 1) etch <- rep(etch, n)
      if(length(fg) == 0) fg <- rep(par("col"), n) else
      if(length(fg) == 1) fg <- rep(fg, n)
      other <- resolve.defaults(list(...),
                                list(add=TRUE, inches=FALSE))
      ## infer which arguments are parallelised
      isvec <- (lengths(other) == n)
      other.fixed <- other[!isvec]
      other.vec   <- other[isvec]
      ##
      if(any(as.logical(etch))) {
        anti.fg <- complementarycolour(fg)
        anti.lwd <- if(is.numeric(etch)) etch else 2 * lwd
      }
      ## plot
      if(any(i <- (shape == "circles") & as.logical(etch))) 
        do.call.matched(symbols,
                        c(list(x=x[i], y=y[i], circles=size[i]/2),
                          other.fixed,
                          lapply(other.vec, "[", i=i),
                          list(lwd=anti.lwd[i], fg=anti.fg[i])),
                        extrargs=c("lwd", "lty"))
      if(any(i <- (shape == "circles")))
        do.call.matched(symbols,
                        c(list(x=x[i], y=y[i], circles=size[i]/2),
                          other.fixed,
                          lapply(other.vec, "[", i=i),
                          list(lwd=lwd[i], fg=fg[i])),
                        extrargs=c("lwd", "lty"))
      if(any(i <- (shape == "squares") & as.logical(etch)))
        do.call.matched(symbols,
                        c(list(x=x[i], y=y[i], squares=size[i]),
                          other.fixed,
                          lapply(other.vec, "[", i=i),
                          list(lwd=anti.lwd[i], fg=anti.fg[i])),
                        extrargs=c("lwd", "lty"))
      if(any(i <- (shape == "squares"))) 
        do.call.matched(symbols,
                        c(list(x=x[i], y=y[i], squares=size[i]),
                          other.fixed,
                          lapply(other.vec, "[", i=i),
                          list(lwd=lwd[i], fg=fg[i])),
                        extrargs=c("lwd", "lty"))
      if(any(i <- (shape == "arrows") & as.logical(etch)))
        do.call.matched(do.arrows,
                        c(list(x=x[i], y=y[i], len=size[i]),
                          other.fixed,
                          lapply(other.vec, "[", i=i),
                          list(lwd=anti.lwd[i], cols=anti.fg[i])),
                        extrargs=c("cols", "col", "lwd", "lty"))
      if(any(i <- (shape == "arrows"))) 
        do.call.matched(do.arrows,
                        c(list(x=x[i], y=y[i], len=size[i]),
                          other.fixed,
                          lapply(other.vec, "[", i=i),
                          list(lwd=lwd[i], cols=fg[i])),
                        extrargs=c("cols", "col", "lwd", "lty"))
    }
    return(max(size))
  }

  do.arrows <- function(x, y, len, direction=0, arrowtype=2, ...,
                        headlength=len * 0.4, 
                        headangle=40,
                        cols=col, col=par('fg'),
                        lwd=1, lty=1) {
    #' vectorise all arguments
    df <- data.frame(x=x, y=y, len=len, direction=direction,
                     arrowtype=arrowtype, headangle=headangle,
                     cols=cols, lwd=lwd, lty=lty)
    with(df, {
      alpha <- direction * pi/180
      dx <- len * cos(alpha)/2
      dy <- len * sin(alpha)/2
      x0 <- x - dx
      x1 <- x + dx
      y0 <- y - dy
      y1 <- y + dy
      segments(x0, y0, x1, y1, ..., col=cols, lty=lty, lwd=lwd)
      if(any(arrowtype != 0)) {
        halfangle <- (headangle/2) * pi/180
        beta1 <- alpha + halfangle
        beta2 <- alpha - halfangle
        hx1 <- headlength * cos(beta1)
        hy1 <- headlength * sin(beta1)
        hx2 <- headlength * cos(beta2)
        hy2 <- headlength * sin(beta2)
        if(any(left <- (arrowtype %in% c(1,3)))) {
          segments(x0[left], y0[left], (x0 + hx1)[left], (y0 + hy1)[left],
                   ..., col=cols[left], lwd=lwd[left], lty=lty[left])
          segments(x0[left], y0[left], (x0 + hx2)[left], (y0 + hy2)[left],
                   ..., col=cols[left], lwd=lwd[left], lty=lty[left])
        }
        if(any(right <- (arrowtype %in% c(2,3)))) {
          segments(x1[right], y1[right], (x1 - hx1)[right], (y1 - hy1)[right],
                   ..., col=cols[right], lwd=lwd[right], lty=lty[right])
          segments(x1[right], y1[right], (x1 - hx2)[right], (y1 - hy2)[right],
                   ..., col=cols[right], lwd=lwd[right], lty=lty[right])
        }
      }
    })
    return(invisible(NULL))
  }
  
  ## main function
  invoke.symbolmap <- function(map, values, x=NULL, y=NULL, ...,
                                 add=FALSE, do.plot=TRUE,
                                 started = add && do.plot) {
    if(!inherits(map, "symbolmap"))
      stop("map should be an object of class 'symbolmap'")
    if(hasxy <- (!is.null(x) || !is.null(y))) {
      xy <- xy.coords(x, y)
      x <- xy$x
      y <- xy$y
    } 
    ## function will return maximum size of symbols plotted.
    maxsize <- 0
    if(do.plot && !add) plot(x, y, type="n", ...)
    force(values)
    g <- map(values)
    parnames <- colnames(g)
    if(do.plot) {
      xydf <- data.frame(x=x, y=y)
      if(nrow(xydf) == 0)
        return(invisible(maxsize))
      g <- if(prod(dim(g)) == 0) xydf else 
           do.call(data.frame,
                   c(as.list(g), as.list(xydf), list(stringsAsFactors=FALSE)))
    }
    n <- nrow(g)
    ## figure out which function does the graphics job
    need.points <- any(c("pch", "chars") %in% parnames)
    need.symbols <- "shape" %in% parnames
    if(need.symbols && need.points) {
      worker <- with(g, ifelse(!is.na(shape), "symbols", "points"))
    } else if(need.symbols) {
      worker <- rep.int("symbols", n)
    } else {
      worker <- rep.int("points", n)
    } 
    ## split data according to graphics function involved
    z <- split(g, factor(worker))
    ## display using 'pch'
    zpoints <- z[["points"]]
    if(!is.null(zpoints) && nrow(zpoints) > 0) {
      ms <- do.call(do.points,
                    resolve.defaults(as.list(zpoints),
                                     list(...),
                                     list(do.plot=do.plot)))
      ## value is max(cex)
      ## guess size of one character
      charsize <- if(started) max(par('cxy')) else
                  if(hasxy) max(sidelengths(boundingbox(x,y))/40) else 1/40
      maxsize <- max(maxsize, charsize * ms)
    }
    ## display using 'symbols'
    zsymbols <- z[["symbols"]]
    if(!is.null(zsymbols) && nrow(zsymbols) > 0) {
      ms <- do.call(do.symbols,
                    resolve.defaults(as.list(zsymbols),
                                     list(...),
                                     list(do.plot=do.plot)))
      ## ms value is max physical size.
      maxsize <- max(maxsize, ms)
    }
    return(invisible(maxsize))
  }

  invoke.symbolmap
})


## Display the symbol map itself (`legend' style)

plot.symbolmap <- function(x, ..., main,
                           xlim=NULL, ylim=NULL,
                           vertical=FALSE,
                           side=c("bottom", "left", "top", "right"),
                           annotate=TRUE, labelmap=NULL, add=FALSE,
                           nsymbols=NULL) {
  if(missing(main))
    main <- short.deparse(substitute(x))
  miss.side <- missing(side)
  side <- match.arg(side)
  
  type <- symbolmaptype(x)
  map <- x
  stuff <- attr(map, "stuff")

  if(type == "constant" && length(stuff$parlist) == 0)
    return(invisible(NULL))

  if(is.null(labelmap)) {
    labelmap <- function(x) x
  } else if(type == "continuous" &&
            is.numeric(labelmap) && length(labelmap) == 1) {
    labscal <- labelmap
    labelmap <- function(x) { x * labscal }
  } else stopifnot(is.function(labelmap))

  ## determine the 'example' input values and their graphical representations
  switch(type,
         constant = {
           vv <- NULL
         },
         continuous = {
           ra <- stuff$range
           if(is.null(ra))
             stop("Cannot plot symbolmap with an infinite range")
           vv <- if(is.null(nsymbols)) prettyinside(ra) else
                 prettyinside(ra, n = nsymbols)
           if(is.numeric(vv))
             vv <- signif(vv, 4)
         },
         discrete = {
           vv <- if(is.null(nsymbols)) prettydiscrete(stuff$inputs) else
                 prettydiscrete(stuff$inputs, n = nsymbols)
           if(vertical) vv <- rev(vv)
         })
  nn <- length(vv)
  ##    gg <- map(vv)
  ll <- paste(labelmap(vv))
    
  ## determine position of plot and symbols
  if(add) {
    ## x and y limits must respect existing plot space
    usr <- par('usr')
    if(is.null(xlim)) xlim <- usr[1:2]
    if(is.null(ylim)) ylim <- usr[3:4]
  } else {
    ## create new plot
    maxdiam <- invoke.symbolmap(map, vv, do.plot=FALSE, started=FALSE)
    zz <- c(0, max(1, maxdiam))
    if(is.null(xlim) && is.null(ylim)) {
      if(vertical) {
        xlim <- zz
        ylim <- length(vv) * zz
      } else {
        xlim <- length(vv) * zz
        ylim <- zz
      }
    } else if(is.null(ylim)) {
      ylim <- zz
    } else if(is.null(xlim)) {
      xlim <- zz
    }
  }

  ## .......... initialise plot ...............................
  if(!add)
    do.call.matched(plot.default,
                    resolve.defaults(list(x=xlim, y=ylim,
                                          type="n", main=main,
                                          axes=FALSE, xlab="", ylab="",
                                          asp=1.0),
                                     list(...)))
  ## maximum symbol diameter
  maxdiam <- invoke.symbolmap(map, vv, do.plot=FALSE, started=TRUE)

  ## .......... plot symbols ....................
  if(type == "constant") {
    xp <- mean(xlim)
    yp <- mean(ylim)
  } else if(vertical) {
    ## vertical arrangement
    xp <- rep(mean(xlim), nn)
    vskip <- 1.1 * max(maxdiam, 3 * max(strheight(labelmap(vv))))
    if(diff(ylim) > nn * vskip) {
      yp <- (1:nn) * vskip
      yp <- yp - mean(yp) + mean(ylim)
    } else {
      z <- seq(ylim[1], ylim[2], length=nn+1)
      yp <- z[-1] - diff(z)/2
    }
  } else {
    ## horizontal arrangement
    yp <- rep(mean(ylim), nn)
    hskip <- 1.1 * max(maxdiam, max(strwidth(labelmap(vv))))
    if(diff(xlim) > nn * hskip) {
      xp <- (1:nn) * hskip
      xp <- xp - mean(xp) + mean(xlim)
    } else {
      z <- seq(xlim[1], xlim[2], length=nn+1)
      xp <- z[-1] - diff(z)/2
    }
  }
  invoke.symbolmap(map, vv, xp, yp, ..., add=TRUE)

  ## ................. draw annotation ..................
  if(annotate && length(ll) > 0) {
    if(vertical) {
      ## default axis position is to the right 
      if(miss.side) side <- "right"
      sidecode <- match(side, c("bottom", "left", "top", "right"))
      if(!(sidecode %in% c(2,4)))
        warning(paste("side =", sQuote(side),
                      "is not consistent with vertical orientation"))
      pos <- c(ylim[1], xlim[1], ylim[2], xlim[2])[sidecode]
      ## draw axis
      do.call.matched(graphics::axis,
                      resolve.defaults(list(...),
                                       list(side=sidecode, pos=pos, at=yp,
                                            labels=ll, tick=FALSE, las=1)),
                      extrargs=graphicsPars("axis"))
    } else {
      ## default axis position is below 
      if(miss.side) side <- "bottom"
      sidecode <- match(side, c("bottom", "left", "top", "right"))
      if(!(sidecode %in% c(1,3)))
        warning(paste("side =", sQuote(side),
                      "is not consistent with horizontal orientation"))
      pos <- c(ylim[1], xlim[1], ylim[2], xlim[2])[sidecode]
      ## draw axis
      do.call.matched(graphics::axis,
                      resolve.defaults(list(...),
                                       list(side = sidecode, pos = pos,
                                            at = xp, labels=ll, tick=FALSE)),
                      extrargs=graphicsPars("axis"))
    } 
  }
  return(invisible(NULL))
}

plan.legend.layout <- function(B, 
                               ..., 
                               side=c("bottom", "left", "top", "right"),
                               sep=NULL,
                               size=NULL,
                               sep.frac=0.05,
                               size.frac=0.05,
                               started=FALSE,
                               map=NULL) {
  ## Determine size and position of a box containing legend or symbolmap
  ## attached to a plot in region 'B'.
  ##   sep, size are absolute distances;
  ##   sep.frac, size.frac are fractions of the maximum sidelength of B.
  side <- match.arg(side)
  B <- as.rectangle(B)
  Bsize <- max(sidelengths(B))
  if(is.null(size)) {
    size <- size.frac * Bsize
  } else {
    check.1.real(size)
    stopifnot(size > 0)
  }
  if(is.null(sep)) {
    sep <- sep.frac * Bsize
  } else {
    check.1.real(sep)
    stopifnot(sep > 0)
  }
  if(is.null(map) || !inherits(map, "symbolmap")) {
    vv <- NULL
    textlength <- 8
  } else {
    vv <- with(attr(map, "stuff"),
               if(type == "discrete") inputs else prettyinside(range))
    textlength <- max(nchar(paste(vv)))
  }
  if(started && !is.null(vv)) {
    textwidth <- max(strwidth(vv))
    textheight <- max(strheight(vv))
  } else {
    ## the plot has not been initialised: guess character size
    charsize <- diff(if(side %in% c("left", "right")) B$yrange else B$xrange)/40
    textwidth <- charsize * textlength
    textheight <- charsize
  }
  switch(side,
         right={
           ## symbols to right of image
           b <- owin(B$xrange[2] + sep + c(0, size),
                     B$yrange)
           ## text to right of symbols
           tt <- owin(b$xrange[2] + sep + c(0, textwidth),
                      b$yrange)
           iside <- 4
         },
         left={
           ## symbols to left of image
           b <- owin(B$xrange[1] - sep - c(size, 0),
                     B$yrange)
           ## text to left of symbols
           tt <- owin(b$xrange[1] - sep - c(textwidth, 0),
                      b$yrange)
           iside <- 2
         },
         top={
           ## symbols above image
           b <- owin(B$xrange,
                     B$yrange[2] + sep + c(0, size))
           ## text above symbols
           tt <- owin(b$xrange,
                      b$yrange[2] + 3* charsize + c(0, textheight))
           iside <- 3
         },
         bottom={
           ## symbols below image
           b <- owin(B$xrange,
                     B$yrange[1] - sep - c(size, 0))
           ## text below symbols
           tt <- owin(b$xrange,
                      b$yrange[1] - 3 * charsize - c(textheight, 0))
           iside <- 1
         })
  A <- boundingbox(B, b, tt)
  return(list(A=A, B=B, b=b, tt=tt,
              iside=iside, side=side, size=size, charsize=charsize, sep=sep))
}

         
  
  

#
#	window.S
#
#	A class 'owin' to define the "observation window"
#
#	$Revision: 4.189 $	$Date: 2019/11/11 10:15:17 $
#
#
#	A window may be either
#
#		- rectangular:
#                       a rectangle in R^2
#                       (with sides parallel to the coordinate axes)
#
#		- polygonal:
#			delineated by 0, 1 or more non-self-intersecting
#                       polygons, possibly including polygonal holes.
#	
#		- digital mask:
#			defined by a binary image
#			whose pixel values are TRUE wherever the pixel
#                       is inside the window
#
#	Any window is an object of class 'owin', 
#       containing at least the following entries:	
#
#		$type:	a string ("rectangle", "polygonal" or "mask")
#
#		$xrange   
#		$yrange
#			vectors of length 2 giving the real dimensions 
#			of the enclosing box
#               $units
#                       name of the unit of length
#
#	The 'rectangle' type has only these entries.
#
#       The 'polygonal' type has an additional entry
#
#               $bdry
#                       a list of polygons.
#                       Each entry bdry[[i]] determines a closed polygon.
#
#                       bdry[[i]] has components $x and $y which are
#                       the cartesian coordinates of the vertices of
#                       the i-th boundary polygon (without repetition of
#                       the first vertex, i.e. same convention as in the
#                       plotting function polygon().)
#
#
#	The 'mask' type has entries
#
#		$m		logical matrix
#		$dim		its dimension array
#		$xstep,ystep	x and y dimensions of a pixel
#		$xcol	        vector of x values for each column
#               $yrow           vector of y values for each row
#	
#	(the row index corresponds to increasing y coordinate; 
#	 the column index "   "     "   "  "  "  x "   "    ".)
#
#
#-----------------------------------------------------------------------------
#

.Spatstat.Image.Warning <-
  c("Row index corresponds to increasing y coordinate; column to increasing x",
    "Transpose matrices to get the standard presentation in R",
    "Example: image(result$xcol,result$yrow,t(result$d))")

owin <- local({

  isxy <- function(x) { (is.matrix(x) || is.data.frame(x)) && ncol(x) == 2 }
  asxy <- function(xy) { list(x=xy[,1], y=xy[,2]) }

  owin <- function(xrange=c(0,1), yrange=c(0,1),
                 ..., poly=NULL, mask=NULL, unitname=NULL, xy=NULL) {

  # trap a common abuse of syntax
  if(nargs() == 1 && !missing(xrange) && is.owin(xrange))
    return(xrange)
  
  unitname <- as.unitname(unitname)

  ## Exterminate ambiguities
  poly.given <- !is.null(poly)
  mask.given <- !is.null(mask)
  if(poly.given  && mask.given)
     stop("Ambiguous -- both polygonal boundary and digital mask supplied")

  if(!is.null(xy) && !mask.given)
    warning("Argument xy ignored: it is only applicable when a mask is given")
     
  if(missing(xrange) != missing(yrange))
    stop("If one of xrange, yrange is specified then both must be.")
  if(!missing(xrange)) {
    xrange <- unname(xrange)
    yrange <- unname(yrange)
  }
    
  # convert data frames to vanilla lists
  if(poly.given) {
    if(is.data.frame(poly))
      poly <- as.list(poly)
    else if(is.list(poly) && any(unlist(lapply(poly, is.data.frame))))
      poly <- lapply(poly, as.list)
  }
  
  ## Hidden options controlling how much checking is performed
  check <- resolve.1.default(list(check=TRUE), list(...))
  calculate <- resolve.1.default(list(calculate=check), list(...))
  strict <- resolve.1.default(list(strict=spatstat.options("checkpolygons")),
                              list(...))
  fix <- resolve.1.default(list(fix=spatstat.options("fixpolygons")),
                           list(...))

  if(!poly.given && !mask.given) {
    ######### rectangle #################
    if(check) {
      if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] < xrange[1L])
        stop("xrange should be a vector of length 2 giving (xmin, xmax)")
      if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] < yrange[1L])
        stop("yrange should be a vector of length 2 giving (ymin, ymax)")
    }
    w <- list(type="rectangle", xrange=xrange, yrange=yrange, units=unitname)
    class(w) <- "owin"
    return(w)
  } else if(poly.given) {
    ######### polygonal boundary ########
    #
    if(length(poly) == 0) {
      # empty polygon
      if(check) {
        if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] < xrange[1L])
          stop("xrange should be a vector of length 2 giving (xmin, xmax)")
        if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] < yrange[1L])
          stop("yrange should be a vector of length 2 giving (ymin, ymax)")
      }
      w <- list(type="polygonal", xrange=xrange, yrange=yrange,
                bdry=list(), units=unitname)
      class(w) <- "owin"
      return(w)
    }
    # convert matrix or data frame to list(x,y)
    if(isxy(poly)) {
      poly <- asxy(poly)
    } else if(is.list(poly) && all(unlist(lapply(poly, isxy)))) {
      poly <- lapply(poly, asxy)
    }
    # nonempty polygon  
    # test whether it's a single polygon or multiple polygons
    if(verify.xypolygon(poly, fatal=FALSE))
      psingle <- TRUE
    else if(all(unlist(lapply(poly, verify.xypolygon, fatal=FALSE))))
      psingle <- FALSE
    else
      stop("poly must be either a list(x,y) or a list of list(x,y)")

    w.area <- NULL
    
    if(psingle) {
      # single boundary polygon
      bdry <- unname(list(poly))
      if(check || calculate) {
        w.area <- Area.xypolygon(poly)
        if(w.area < 0)
          stop(paste("Area of polygon is negative -",
                     "maybe traversed in wrong direction?"))
      }
    } else {
      # multiple boundary polygons
      bdry <- unname(poly)
      if(check || calculate) {
        w.area <- sapply(poly, Area.xypolygon)
        if(sum(w.area) < 0)
          stop(paste("Area of window is negative;\n",
                     "check that all polygons were traversed",
                     "in the right direction"))
      }
    }

    actual.xrange <- range(unlist(lapply(bdry, getElement, name="x")))
    if(missing(xrange))
      xrange <- actual.xrange
    else if(check) {
      if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] < xrange[1L])
        stop("xrange should be a vector of length 2 giving (xmin, xmax)")
      if(!all(xrange == range(c(xrange, actual.xrange))))
        stop("polygon's x coordinates outside xrange")
    }
    
    actual.yrange <- range(unlist(lapply(bdry, getElement, name="y")))
    if(missing(yrange))
      yrange <- actual.yrange
    else if(check) {
      if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] < yrange[1L])
        stop("yrange should be a vector of length 2 giving (ymin, ymax)")
      if(!all(yrange == range(c(yrange, actual.yrange))))
      stop("polygon's y coordinates outside yrange")
    }

    if(!is.null(w.area)) {
      # tack on area and hole data
      holes <- (w.area < 0)
      for(i in seq_along(bdry)) 
        bdry[[i]] <- append(bdry[[i]], list(area=w.area[i], hole=holes[i]))
    }
    
    w <- list(type="polygonal",
              xrange=xrange, yrange=yrange, bdry=bdry, units=unitname)
    class(w) <- "owin"

    if(check && strict) { 
      ## strict checks on geometry (self-intersection etc)
      ok <- owinpolycheck(w)
      if(!ok) {
        errors <- attr(ok, "err")
        stop(paste("Polygon data contain", commasep(errors)))
      }
    }
    if(check && fix) {
      if(length(bdry) == 1 &&
         length(bx <- bdry[[1L]]$x) == 4 &&
         length(unique(bx)) == 2 &&
         length(unique(bdry[[1L]]$y)) == 2) {
        ## it's really a rectangle
        if(Area.xypolygon(bdry[[1L]]) < 0)
          w$bdry <- lapply(bdry, reverse.xypolygon)
      } else {
        ## repair polygon data by invoking polyclip
        ##        to intersect polygon with larger-than-bounding rectangle
        ##        (Streamlined version of intersect.owin)
        ww <- lapply(bdry, reverse.xypolygon)
        xrplus <- mean(xrange) + c(-1,1) * diff(xrange)
        yrplus <- mean(yrange) + c(-1,1) * diff(yrange)
        bignum <- (.Machine$integer.max^2)/2
        epsclip <- max(diff(xrange), diff(yrange))/bignum
        rr <- list(list(x=xrplus[c(1,2,2,1)], y=yrplus[c(2,2,1,1)]))
        bb <- polyclip::polyclip(ww, rr, "intersection",
                                 fillA="nonzero", fillB="nonzero", eps=epsclip)
        ## ensure correct polarity
        totarea <- sum(unlist(lapply(bb, Area.xypolygon)))
        if(totarea < 0)
          bb <- lapply(bb, reverse.xypolygon)
        w$bdry <- bb
      }
    }
    return(w)
  } else if(mask.given) {
    ######### digital mask #####################

    if(is.data.frame(mask) &&
       ncol(mask) %in% c(2,3) &&
       sum(sapply(mask, is.numeric)) == 2) {
      # data frame with 2 columns of coordinates
      return(as.owin(W=mask, xy=xy))
    }
      
    if(!is.matrix(mask))
      stop(paste(sQuote("mask"), "must be a matrix"))
    if(!is.logical(mask))
      stop(paste("The entries of", sQuote("mask"), "must be logical"))
    
    nc <- ncol(mask)
    nr <- nrow(mask)

    if(!is.null(xy)) {
      # pixel coordinates given explicitly
      # validate dimensions
      if(!is.list(xy) || !checkfields(xy, c("x","y")))
        stop("xy should be a list with entries x and y")
      xcol <- xy$x
      yrow <- xy$y
      if(length(xcol) != nc)
        stop(paste("length of xy$x =", length(xcol),
                   "!=", nc, "= number of columns of mask"))
      if(length(yrow) != nr)
        stop(paste("length of xy$y =", length(yrow),
                   "!=", nr, "= number of rows of mask"))
      # x and y should be evenly spaced
      if(!evenly.spaced(xcol))
        stop("xy$x is not evenly spaced")
      if(!evenly.spaced(yrow))
        stop("xy$y is not evenly spaced")
      # determine other parameters
      xstep <- diff(xcol)[1L]
      ystep <- diff(yrow)[1L]
      if(missing(xrange) && missing(yrange)) {
        xrange <- range(xcol) + c(-1,1) * xstep/2
        yrange <- range(yrow) + c(-1,1) * ystep/2
      }
    } else {
      # determine pixel coordinates from xrange, yrange
      if(missing(xrange) && missing(yrange)) {
        # take pixels to be 1 x 1 unit
        xrange <- c(0,nc)
        yrange <- c(0,nr)
      } else if(check) {
        if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] < xrange[1L])
          stop("xrange should be a vector of length 2 giving (xmin, xmax)")
        if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] < yrange[1L])
          stop("yrange should be a vector of length 2 giving (ymin, ymax)")
      }
      xstep <- diff(xrange)/nc
      ystep <- diff(yrange)/nr
      xcol  <- seq(from=xrange[1L]+xstep/2, to=xrange[2L]-xstep/2, length.out=nc)
      yrow  <- seq(from=yrange[1L]+ystep/2, to=yrange[2L]-ystep/2, length.out=nr)
    }

    out <- list(type     = "mask",
                xrange   = unname(xrange),
                yrange   = unname(yrange),
                dim      = c(nr, nc),
                xstep    = unname(xstep),
                ystep    = unname(ystep),
                warnings = .Spatstat.Image.Warning,
                xcol    = unname(xcol), 
                yrow    = unname(yrow),
                m       = mask,
                units   = unitname)
    class(out) <- "owin"
    return(out)
  }
  # never reached
  NULL
}

  owin
})

#
#-----------------------------------------------------------------------------
#

is.owin <- function(x) { inherits(x, "owin") }

#
#-----------------------------------------------------------------------------
#

as.owin <- function(W, ..., fatal=TRUE) {
  UseMethod("as.owin")
}

as.owin.owin <- function(W, ..., fatal=TRUE) {
  if(verifyclass(W, "owin", fatal=fatal)) 
    return(owin(W$xrange, W$yrange, poly=W$bdry, mask=W$m, unitname=unitname(W), check=FALSE))
  else
    return(NULL)
}

as.owin.ppp <- function(W, ..., fatal=TRUE) {
  if(verifyclass(W, "ppp", fatal=fatal))
    return(W$window)
  else
    return(NULL)
}

as.owin.quad <- function(W, ..., fatal=TRUE) {
  if(verifyclass(W, "quad", fatal=fatal))
    return(W$data$window)
  else
    return(NULL)
}

as.owin.im <- function(W, ..., fatal=TRUE) {
  if(!verifyclass(W, "im", fatal=fatal))
    return(NULL)
  out <- list(type     = "mask",
              xrange   = W$xrange,
              yrange   = W$yrange,
              dim      = W$dim,
              xstep    = W$xstep,
              ystep    = W$ystep,
              warnings = .Spatstat.Image.Warning,
              xcol    = W$xcol,
              yrow    = W$yrow,
              m       = !is.na(W$v),
              units   = unitname(W))
  class(out) <- "owin"
  return(out)
}

as.owin.psp <- function(W, ..., fatal=TRUE) {
  if(!verifyclass(W, "psp", fatal=fatal))
    return(NULL)
  return(W$window)
}

as.owin.tess <- function(W, ..., fatal=TRUE) {
  if(!verifyclass(W, "tess", fatal=fatal))
    return(NULL)
  return(W$window)
}

as.owin.data.frame <- function(W, ..., step, fatal=TRUE) {
  if(!verifyclass(W, "data.frame", fatal=fatal))
    return(NULL)
  if(missing(step)) {
    xstep <- ystep <- NULL
  } else {
    step <- ensure2vector(step)
    xstep <- step[1L]
    ystep <- step[2L]
  }
  if(!(ncol(W) %in% c(2,3))) {
    whinge <- "need exactly 2 or 3 columns of data"
    if(fatal) stop(whinge)
    warning(whinge)
    return(NULL)
  }
  if(twocol <- (ncol(W) == 2)) {
    # assume data is a list of TRUE pixels
    W <- cbind(W, TRUE)
  } 
  mch <- matchNameOrPosition(c("x", "y", "z"), names(W))
  ix <- mch[1L]
  iy <- mch[2L]
  iz <- mch[3L]
  df <- data.frame(x=W[,ix], y=W[,iy], z=as.logical(W[,iz]))
  with(df, {
    xx <- sortunique(x)
    yy <- sortunique(y)
    jj <- match(x, xx)
    ii <- match(y, yy)
    ## make logical matrix (for incomplete x, y sequence)
    ok <- checkbigmatrix(length(xx), length(yy), fatal=fatal)
    if(!ok) return(NULL)
    mm <- matrix(FALSE, length(yy), length(xx))
    mm[cbind(ii,jj)] <- z
    ## ensure xx and yy are complete equally-spaced sequences
    fx <- fillseq(xx, step=xstep)
    fy <- fillseq(yy, step=ystep)
    xcol <- fx[[1L]]
    yrow <- fy[[1L]]
    ## trap very large matrices
    ok <- checkbigmatrix(length(xcol), length(yrow), fatal=fatal)
    if(!ok) return(NULL)
    ## mapping from xx to xcol, yy to yrow
    jjj <- fx[[2L]]
    iii <- fy[[2L]]
    ## make logical matrix for full sequence
    m <- matrix(FALSE, length(yrow), length(xcol))
    m[iii,jjj] <- mm
    ## make binary mask
    out <- owin(mask=m, xy=list(x=xcol, y=yrow))
    ## warn if area fraction is small: may be a misuse of as.owin
    if(twocol) {
      pcarea <- 100 * nrow(df)/prod(dim(m))
      if(pcarea < 1) 
       warning(paste("Window occupies only",
                     paste0(signif(pcarea, 2), "%"),
                     "of frame area. Did you mean owin(poly=df) ?"),
               call.=FALSE)
    }
    return(out)
  })
}

as.owin.default <- function(W, ..., fatal=TRUE) {
  ## Tries to interpret data as an object of class 'owin'
  ## W may be
  ##	a structure with entries xrange, yrange
  ##	a structure with entries xl, xu, yl, yu
  ##	a structure with entries xmin, xmax, ymin, ymax
  ##	a four-element vector (interpreted xmin, xmax, ymin, ymax)
  ##	an object with attribute "bbox"

  if(inherits(W, "box3")) {
    #' cannot be flattened
    if(fatal)
      stop("3D box cannot be converted to a 2D window")
    return(NULL)
  }
  
  if(checkfields(W, c("xrange", "yrange"))) {
    Z <- owin(W$xrange, W$yrange)
    return(Z)
  } else if(checkfields(W, c("xmin", "xmax", "ymin", "ymax"))) {
    W <- as.list(W)
    Z <- owin(c(W$xmin, W$xmax),c(W$ymin, W$ymax))
    return(Z)
  } else if(checkfields(W, c("xl", "xu", "yl", "yu"))) {
    W <- as.list(W)
    Z <- owin(c(W$xl, W$xu),c(W$yl, W$yu))
    return(Z)
  } else if(checkfields(W, c("x", "y", "area"))
            && checkfields(W$area, c("xl", "xu", "yl", "yu"))) {
    V <- as.list(W$area)
    Z <- owin(c(V$xl, V$xu),c(V$yl, V$yu))
    return(Z)
  } else if(is.vector(W) && is.numeric(W) && length(W) == 4) {
    Z <- owin(W[1:2], W[3:4])
    return(Z)
  } else if(!is.null(Z <- attr(W, "bbox"))) {
    return(as.owin(Z, ..., fatal=fatal))
  } else if(any(c("SpatialPolygons", "SpatialPolygonsDataFrame")
                %in% class(W))) {
    gripe <- "The package 'maptools' is needed to convert this data type"
    if(fatal) stop(gripe, call.=FALSE) else warning(gripe, call.=FALSE)
    return(NULL)
  } else {
    #' no idea
    if(fatal) stop("Can't interpret W as a window", call.=FALSE)
    return(NULL)
  }
}		

#
#-----------------------------------------------------------------------------
#
#
Frame <- function(X) { UseMethod("Frame") }

"Frame<-" <- function(X, value) { UseMethod("Frame<-") }

Frame.default <- function(X) { as.rectangle(X) }

"Frame<-.default" <- function(X, value) {
  Frame(Window(X)) <- value
  return(X)
}

## .........................................................

as.rectangle <- function(w, ...) {
  if(inherits(w, "owin"))
    return(owin(w$xrange, w$yrange, unitname=unitname(w)))
  else if(inherits(w, "im"))
    return(owin(w$xrange, w$yrange, unitname=unitname(w)))
  else if(inherits(w, "layered")) 
    return(do.call(boundingbox, unname(lapply(w, as.rectangle))))
  else {
    w <- as.owin(w, ...)
    return(owin(w$xrange, w$yrange, unitname=unitname(w)))
  }
}

#
#-----------------------------------------------------------------------------
#
as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
#	eps:		   grid mesh (pixel) size
#	dimyx:		   dimensions of pixel raster
#       xy:                coordinates of pixel raster
  nonamedargs <- is.null(eps) && is.null(dimyx) && is.null(xy)
  uname <- as.unitname(NULL)
  if(!missing(w) && !is.null(w)) {
    if(is.data.frame(w)) return(owin(mask=w, xy=xy))
    if(is.matrix(w)) {
      w <- as.data.frame(w)
      colnames(w) <- c("x", "y")
      return(owin(mask=w, xy=xy))
    }
    w <- as.owin(w)
    uname <- unitname(w)
  } else {
    if(is.null(xy)) 
      stop("If w is missing, xy is required")
  }
  # If it's already a mask, and no other arguments specified,
  # just return it.
  if(!missing(w) && w$type == "mask" && nonamedargs)
    return(w)
  
##########################
#  First determine pixel coordinates
##########################
  if(is.null(xy)) {
# Pixel coordinates to be computed from other dimensions
# First determine row & column dimensions
    if(!is.null(dimyx)) {
      dimyx <- ensure2vector(dimyx)
      nr <- dimyx[1L]
      nc <- dimyx[2L]
    } else {
    # use pixel size 'eps'
      if(!is.null(eps)) {
        eps <- ensure2vector(eps)
        nc <- diff(w$xrange)/eps[1L]
        nr <- diff(w$yrange)/eps[2L]
        if(nr < 1 || nc < 1)
          warning("pixel size parameter eps > size of window")
        nr <- ceiling(nr)
        nc <- ceiling(nc)
      } else {
    # use spatstat defaults
        np <- spatstat.options("npixel")
        if(length(np) == 1)
          nr <- nc <- np[1L]
        else {
          nr <- np[2L]  
          nc <- np[1L]
        }
      }
    }
    if((mpix <- (nr * nc)/1048576) >= 10) {
      whinge <- paste("Creating",
                      articlebeforenumber(mpix),
                      paste0(round(mpix, 1), "-megapixel"),
                      "window mask")
      message(whinge)
      warning(whinge, call.=FALSE)
    }
    # Initialise mask with all entries TRUE
    rasta <- owin(w$xrange, w$yrange, mask=matrix(TRUE, nr, nc))
  } else {
# 
# Pixel coordinates given explicitly:
#    xy is an image, a mask, or a list(x,y)
#
    if(is.im(xy)) {
      rasta <- as.owin(xy)
      rasta$m[] <- TRUE
    } else if(is.owin(xy)) {
      if(xy$type != "mask")
        stop("argument xy does not contain raster coordinates.")
      rasta <- xy
      rasta$m[] <- TRUE
    } else {
      if(!checkfields(xy, c("x", "y")))
        stop(paste(sQuote("xy"),
                   "should be a list containing two vectors x and y"))
      x <- sortunique(xy$x)
      y <- sortunique(xy$y)
      # derive other parameters
      nr <- length(y)
      nc <- length(x)
      # check size
      if((mpix <- (nr * nc)/1048576) >= 10) {
        whinge <- paste("Creating",
                        articlebeforenumber(mpix),
                        paste0(round(mpix, 1), "-megapixel"),
                        "window mask")
        message(whinge)
        warning(whinge, call.=FALSE)
      }
      # x and y pixel sizes
      dx <- diff(x)
      if(diff(range(dx)) > 0.01 * mean(dx))
        stop("x coordinates must be evenly spaced")
      xstep <- mean(dx)
      dy <- diff(y)
      if(diff(range(dy)) > 0.01 * mean(dy))
        stop("y coordinates must be evenly spaced")
      ystep <- mean(dy)
      xr <- range(x)
      yr <- range(y)
      xrange <-  xr + xstep * c(-1,1)/2
      yrange <-  yr + ystep * c(-1,1)/2
      # initialise mask with all entries TRUE
      rasta <- list(type     = "mask",
                    xrange   = xrange,
                    yrange   = yrange,
                    dim      = c(nr, nc),
                    xstep    = xstep,
                    ystep    = ystep,
                    warnings = .Spatstat.Image.Warning,
                    xcol    = seq(from=xr[1L], to=xr[2L], length.out=nc),
                    yrow    = seq(from=yr[1L], to=yr[2L], length.out=nr),
                    m       = matrix(TRUE, nr, nc),
                    units   = uname)
      class(rasta) <- "owin"
    }
    if(missing(w)) {
      # No more window information
      out <- rasta
      if(!(identical(x, xy$x) && identical(y, xy$y))) {
        ## xy is an enumeration of the TRUE pixels
        out$m[] <- FALSE
        ij <- cbind(i=match(xy$y, y), j=match(xy$x, x))
        out$m[ij] <- TRUE
      }
      return(out)
    }
  }

################################  
# Second, mask pixel raster with existing window
################################  
  switch(w$type,
         rectangle = {
           out <- rasta
           if(!all(w$xrange == rasta$xrange)
              || !all(w$yrange == rasta$yrange)) {
             xcol <- rasta$xcol
             yrow <- rasta$yrow
             wx <- w$xrange
             wy <- w$yrange
             badrow <- which(yrow > wy[2L] | yrow < wy[1L])
             badcol <- which(xcol > wx[2L] | xcol < wx[1L])
             out$m[badrow , ] <- FALSE
             out$m[ , badcol] <- FALSE
           }
         },
         mask = {
           # resample existing mask on new raster
           out <- rastersample(w, rasta)
         },
         polygonal = {
           # use C code
           out <- owinpoly2mask(w, rasta, FALSE)
         })

  unitname(out) <- uname
  return(out)
}

as.matrix.owin <- function(x, ...) {
  m <- as.mask(x, ...)
  return(m$m)
}

#
#
#-----------------------------------------------------------------------------
#
as.polygonal <- function(W, repair=FALSE) {
  verifyclass(W, "owin")
  switch(W$type,
         rectangle = {
           xr <- W$xrange
           yr <- W$yrange
           return(owin(xr, yr, poly=list(x=xr[c(1,2,2,1)],y=yr[c(1,1,2,2)]),
                       unitname=unitname(W),
                       check=FALSE))
         },
         polygonal = {
           if(repair)
             W <- owin(poly=W$bdry, unitname=unitname(W))
           return(W)
         },
         mask = {
           # This could take a while
           M <- W$m
           nr <- nrow(M)
           notM <- !M
           xcol <- W$xcol
           yrow <- W$yrow
           xbracket <- 1.1 * c(-1,1) * W$xstep/2
           ybracket <- 1.1 * c(-1,1) * W$ystep/2
           ## determine resolution for polyclip operations
           p <- list(x0 = xcol[1],
                     y0 = yrow[1],
                     eps = max(W$xstep, W$ystep)/(2^31))
           # identify runs of TRUE entries in each column
           start <- M & rbind(TRUE, notM[-nr, ])
           finish <- M & rbind(notM[-1, ], TRUE)
           #' build result
           out <- NULL
           for(j in 1:ncol(M)) {
             xj <- xcol[j]
             # identify start and end positions in column j
             starts <- which(start[,j])
             finishes <- which(finish[,j])
             ns <- length(starts)
             nf <- length(finishes)
             if(ns != nf)
               stop(paste("Internal error: length(starts)=", ns,
                          ", length(finishes)=", nf))
             if(ns > 0) {
               for(k in 1:ns) {
                 yfrom <- yrow[starts[k]]
                 yto   <- yrow[finishes[k]]
                 yk <- sort(c(yfrom,yto))
                 #' make rectangle boundary in reversed orientation
                 xrect <- xj + xbracket
                 yrect <- yk + ybracket
                 recto <- list(list(x = xrect[c(1,2,2,1)],
                                    y = yrect[c(2,2,1,1)]))
                 #' add to result
                 if(is.null(out)) {
                   out <- recto
                 } else {
                   out <- polyclip::polyclip(out, recto, "union",
                                             fillA="nonzero", fillB="nonzero",
                                             eps = p$eps,
                                             x0  = p$x0,
                                             y0  = p$y0)
                 }
               }
             }
           }
           if(is.null(out)) return(emptywindow(Frame(W)))
           totarea <- sum(sapply(out, Area.xypolygon))
           if(totarea < 0)
             out <- lapply(out, reverse.xypolygon)
           out <- owin(poly=out, check=FALSE, unitname=unitname(W))
           return(out)
         }
         )
}

#
# ----------------------------------------------------------------------

is.polygonal <- function(w) {
  return(inherits(w, "owin") && (w$type == "polygonal"))
}

is.rectangle <- function(w) {
  return(inherits(w, "owin") && (w$type == "rectangle"))
}

is.mask <- function(w) {
  return(inherits(w, "owin") && (w$type == "mask"))
}

validate.mask <- function(w, fatal=TRUE) {
  verifyclass(w, "owin", fatal=fatal)
  if(w$type == "mask")
    return(TRUE)
  if(fatal)
      stop(paste(short.deparse(substitute(w)), "is not a binary mask"))
  else {
      warning(paste(short.deparse(substitute(w)), "is not a binary mask"))
      return(FALSE)
  }
}

dim.owin <- function(x) { return(x$dim) } ## NULL unless it's a mask

## internal use only:

rasterx.mask <- function(w, drop=FALSE) {
  validate.mask(w)
  x <- w$xcol[col(w)]
  x <- if(drop) x[w$m, drop=TRUE] else array(x, dim=w$dim)
  return(x)
}

rastery.mask <- function(w, drop=FALSE) {
  validate.mask(w)
  y <- w$yrow[row(w)]
  y <- if(drop) y[w$m, drop=TRUE] else array(y, dim=w$dim)
  return(y)
}

rasterxy.mask <- function(w, drop=FALSE) {
  validate.mask(w)
  x <- w$xcol[col(w)]
  y <- w$yrow[row(w)]
  if(drop) {
    m <- w$m
    x <- x[m, drop=TRUE] 
    y <- y[m, drop=TRUE]
  }
  return(list(x=as.numeric(x),
              y=as.numeric(y)))
}


nearest.raster.point <- function(x,y,w, indices=TRUE) {
  stopifnot(is.mask(w) || is.im(w))
  nr <- w$dim[1L]
  nc <- w$dim[2L]
  if(length(x) == 0) {
    cc <- rr <- integer(0)
  } else {
    cc <- 1 + round((x - w$xcol[1L])/w$xstep)
    rr <- 1 + round((y - w$yrow[1L])/w$ystep)
    cc <- pmax.int(1,pmin.int(cc, nc))
    rr <- pmax.int(1,pmin.int(rr, nr))
  }
  if(indices) 
    return(list(row=rr, col=cc))
  else
    return(list(x=w$xcol[cc], y=w$yrow[rr]))
}

mask2df <- function(w) {
  stopifnot(is.owin(w) && w$type == "mask")
  xx <- raster.x(w)
  yy <- raster.y(w)
  ok <- w$m
  xx <- as.vector(xx[ok])
  yy <- as.vector(yy[ok])
  return(data.frame(x=xx, y=yy))
}

#------------------------------------------------------------------
		
complement.owin <- function(w, frame=as.rectangle(w)) {
  w <- as.owin(w)

  if(reframe <- !missing(frame)) {
    verifyclass(frame, "owin")
    w <- rebound.owin(w, frame)
    # if w was a rectangle, it's now polygonal
  }

  switch(w$type,
         mask = {
           w$m <- !(w$m)
         },
         rectangle = {
           # return empty window
           return(emptywindow(w))
         },
         polygonal = {
           bdry <- w$bdry
           if(length(bdry) == 0) {
             # w is empty
             return(frame)
           }
           # bounding box, in anticlockwise order
           box <- list(x=w$xrange[c(1,2,2,1)],
                       y=w$yrange[c(1,1,2,2)])
           boxarea <- Area.xypolygon(box)
                 
           # first check whether one of the current boundary polygons
           # is the bounding box itself (with + sign)
           if(reframe)
             is.box <- rep.int(FALSE, length(bdry))
           else {
             nvert <- lengths(lapply(bdry, getElement, name="x"))
             areas <- sapply(bdry, Area.xypolygon)
             boxarea.mineps <- boxarea * (0.99999)
             is.box <- (nvert == 4 & areas >= boxarea.mineps)
             if(sum(is.box) > 1)
               stop("Internal error: multiple copies of bounding box")
             if(all(is.box)) {
               return(emptywindow(box))
             }
           }
                 
           # if box is present (with + sign), remove it
           if(any(is.box))
             bdry <- bdry[!is.box]
                 
           # now reverse the direction of each polygon
           bdry <- lapply(bdry, reverse.xypolygon, adjust=TRUE)

           # if box was absent, add it
           if(!any(is.box))
             bdry <- c(bdry, list(box))   # sic

           # put back into w
           w$bdry <- bdry
         },
         stop("unrecognised window type", w$type)
         )
  return(w)
}

#-----------------------------------------------------------

inside.owin <- function(x, y, w) {
  # test whether (x,y) is inside window w
  # x, y may be vectors 

  if((missing(y) || is.null(y)) && all(c("x", "y") %in% names(x))) {
    y <- x$y
    x <- x$x
  }

  w <- as.owin(w)

  if(length(x)==0)
    return(logical(0))
  
  # test whether inside bounding rectangle
  xr <- w$xrange
  yr <- w$yrange
  eps <- sqrt(.Machine$double.eps)
  frameok <- (x >= xr[1L] - eps) & (x <= xr[2L] + eps) & 
             (y >= yr[1L] - eps) & (y <= yr[2L] + eps)
 
  if(!any(frameok))  # all points OUTSIDE window - no further work needed
    return(frameok)

  ok <- frameok
  switch(w$type,
         rectangle = {
           return(ok)
         },
         polygonal = {
           ## check scale
           framesize <- max(diff(xr), diff(yr))
           if(framesize > 1e6 || framesize < 1e-6) {
             ## rescale to avoid numerical overflow
             scalefac <- as.numeric(framesize)/100
             w <- as.polygonal(rescale(w, scalefac))
             x <- x/scalefac
             y <- y/scalefac
           }
           xy <- list(x=x,y=y)
           bdry <- w$bdry
           total <- numeric(length(x))
           on.bdry <- rep.int(FALSE, length(x))
           for(i in seq_along(bdry)) {
             score <- inside.xypolygon(xy, bdry[[i]], test01=FALSE)
             total <- total + score
             on.bdry <- on.bdry | attr(score, "on.boundary")
           }
           # any points identified as belonging to the boundary get score 1
           total[on.bdry] <- 1
           # check for sanity now..
           uhoh <- (total * (1-total) != 0)
           if(any(uhoh)) {
             nuh <- sum(uhoh)
             warning(paste("point-in-polygon test had difficulty with",
                           nuh,
                           ngettext(nuh, "point", "points"),
                           "(total score not 0 or 1)"),
                     call.=FALSE)
             total[uhoh] <- 0
           }
           return(ok & (total != 0))
         },
         mask = {
           # consider only those points which are inside the frame
           xf <- x[frameok]
           yf <- y[frameok]
           # map locations to raster (row,col) coordinates
           loc <- nearest.raster.point(xf,yf,w)
           # look up mask values
           okf <- (w$m)[cbind(loc$row, loc$col)]
           # insert into 'ok' vector
           ok[frameok] <- okf
           return(ok)
         },
         stop("unrecognised window type", w$type)
         )
}

#-------------------------------------------------------------------------
  
print.owin <- function(x, ..., prefix="window: ") {
  verifyclass(x, "owin")
  unitinfo <- summary(unitname(x))
  switch(x$type,
         rectangle={
           rectname <- paste0(prefix, "rectangle =")
         },
         polygonal={
           nonemp <- (length(x$bdry) != 0)
           splat(paste0(prefix,
                        if(nonemp) "polygonal boundary" else "empty"))
           rectname <- "enclosing rectangle:"
         },
         mask={
           splat(paste0(prefix, "binary image mask"))
           di <- x$dim
           splat(di[1L], "x", di[2L], "pixel array (ny, nx)")
           rectname <- "enclosing rectangle:"
         }
         )
  splat(rectname,
        prange(zapsmall(x$xrange)),
        "x",
        prange(zapsmall(x$yrange)),
        unitinfo$plural,
        unitinfo$explain)
  invisible(NULL)
}

summary.owin <- function(object, ...) {
  verifyclass(object, "owin")
  result <- list(xrange=object$xrange,
                 yrange=object$yrange,
                 type=object$type,
                 area=area(object),
                 units=unitname(object))
  result$areafraction <- with(result, area/(diff(xrange) * diff(yrange)))
  switch(object$type,
         rectangle={
         },
         polygonal={
           poly <- object$bdry
           result$npoly <- npoly <- length(poly)
           if(npoly == 0) {
             result$areas <- result$nvertices <- numeric(0)
           } else if(npoly == 1) {
             result$areas <- Area.xypolygon(poly[[1L]])
             result$nvertices <- length(poly[[1L]]$x)
           } else {
             result$areas <- unlist(lapply(poly, Area.xypolygon))
             result$nvertices <- lengths(lapply(poly, getElement, name="x"))
           }
           result$nhole <- sum(result$areas < 0)
         },
         mask={
           result$npixels <- object$dim
           result$xstep <- object$xstep
           result$ystep <- object$ystep
         }
         )
  class(result) <- "summary.owin"
  result
}

print.summary.owin <- function(x, ...) {
  verifyclass(x, "summary.owin")
  unitinfo <- summary(x$units)
  pluralunits <- unitinfo$plural
  singularunits <- unitinfo$singular
  switch(x$type,
         rectangle={
           rectname <- "Window: rectangle ="
         },
         polygonal={
           np <- x$npoly
           splat("Window: polygonal boundary")
           if(np == 0) {
             splat("window is empty")
           } else if(np == 1) {
             splat("single connected closed polygon with",
                   x$nvertices, "vertices")
           } else {
             nh <- x$nhole
             holy <- if(nh == 0) "(no holes)" else
                     if(nh == 1) "(1 hole)" else
                     paren(paste(nh, "holes"))
             splat(np, "separate polygons", holy)
             if(np > 0)
               print(data.frame(vertices=x$nvertices,
                                area=signif(x$areas, 6),
                                relative.area=signif(x$areas/x$area,3),
                                row.names=paste("polygon",
                                  1:np,
                                  ifelse(x$areas < 0, "(hole)", "")
                                  )))
           }
           rectname <- "enclosing rectangle:"
         },
         mask={
           splat("binary image mask")
           di <- x$npixels
           splat(di[1L], "x", di[2L], "pixel array (ny, nx)")
           splat("pixel size:",
                 signif(x$xstep,3), "by", signif(x$ystep,3),
                 pluralunits)
           rectname <- "enclosing rectangle:"
         }
         )
  splat(rectname,
        prange(zapsmall(x$xrange)),
        "x",
        prange(zapsmall(x$yrange)),
        pluralunits)
  if(x$xrange[1] != 0 || x$yrange[1] != 0) {
    width <- diff(x$xrange)
    height <- diff(x$yrange)
    blank <- paste(rep(" ", nchar(rectname)), collapse="") 
    splat(blank, paren(paste(signif(width, 4), "x", signif(height, 4),
                             pluralunits)))
  }
  Area <- signif(x$area, 6)
  splat("Window area =", Area, "square",
        if(Area == 1) singularunits else pluralunits)
  if(!is.null(ledge <- unitinfo$legend))
    splat(ledge)
  if(x$type != "rectangle")
    splat("Fraction of frame area:", signif(x$areafraction, 3))
  return(invisible(x))
}

as.data.frame.owin <- function(x, ..., drop=TRUE) {
  stopifnot(is.owin(x))
  switch(x$type,
         rectangle = { x <- as.polygonal(x) },
         polygonal = { },
         mask = {
           xy <- rasterxy.mask(x, drop=drop)
           if(!drop) xy <- append(xy, list(inside=as.vector(x$m)))
           return(as.data.frame(xy, ...))
         })
  b <- x$bdry
  ishole <- sapply(b, is.hole.xypolygon)
  sign <- (-1)^ishole
  b <- lapply(b, as.data.frame, ...)
  nb <- length(b)
  if(nb == 1)
    return(b[[1L]])
  dfs <- mapply(cbind, b, id=as.list(seq_len(nb)), sign=as.list(sign),
                SIMPLIFY=FALSE)
  df <- do.call(rbind, dfs)
  return(df)
}

discretise <- function(X,eps=NULL,dimyx=NULL,xy=NULL) {
  verifyclass(X,"ppp")
  W <- X$window
  ok <- inside.owin(X$x,X$y,W)
  if(!all(ok))
    stop("There are points of X outside the window of X")
  all.null <- is.null(eps) & is.null(dimyx) & is.null(xy)
  if(W$type=="mask" & all.null) return(X)
  WM  <- as.mask(W,eps=eps,dimyx=dimyx,xy=xy)
  nok <- !inside.owin(X$x,X$y,WM)
  if(any(nok)) {
    ifix <- nearest.raster.point(X$x[nok],X$y[nok], WM)
    ifix <- cbind(ifix$row,ifix$col)
    WM$m[ifix] <- TRUE
  }
  X$window <- WM
  X
}

pixelcentres <- function (X, W=NULL,...) {
  X <- as.mask(as.owin(X), ...)
  if(is.null(W)) W <- as.rectangle(X)
  Y <- as.ppp(raster.xy(X,drop=TRUE),W=W)
  return(Y)
}

owin2polypath <- function(w) {
  w <- as.polygonal(w)
  b <- w$bdry
  xvectors <- lapply(b, getElement, name="x")
  yvectors <- lapply(b, getElement, name="y")
  xx <- unlist(lapply(xvectors, append, values=NA, after=FALSE))[-1]
  yy <- unlist(lapply(yvectors, append, values=NA, after=FALSE))[-1]
  return(list(x=xx, y=yy))
}

## generics which extract and assign the window of some object

Window <- function(X, ...) { UseMethod("Window") }

"Window<-" <- function(X, ..., value) { UseMethod("Window<-") }

#'
#'     hasclose.R
#'
#'    Determine whether each point has a close neighbour
#'
#'    $Revision: 1.11 $  $Date: 2017/06/05 10:31:58 $

has.close <- function(X, r, Y=NULL, ...) {
  UseMethod("has.close")
}

has.close.default <- function(X, r, Y=NULL, ..., periodic=FALSE) {
  trap.extra.arguments(...)
  if(!periodic) {
    nd <- if(is.null(Y)) nndist(X) else nncross(X, Y, what="dist")
    return(nd <= r)
  }
  if(is.null(Y)) {
    pd <- pairdist(X, periodic=TRUE)
    diag(pd) <- Inf
  } else {
    pd <- crossdist(X, Y, periodic=TRUE)
  }
#  return(apply(pd <= r, 1, any))
  return(matrowany(pd <= r))
}

has.close.ppp <- function(X, r, Y=NULL, ..., periodic=FALSE, sorted=FALSE) {
  trap.extra.arguments(...)
  nX <- npoints(X)
  if(nX <= 1) return(logical(nX))
  #' sort by increasing x coordinate
  cX <- coords(X)
  if(!sorted) {
    oo <- order(cX$x)
    cX <- cX[oo,,drop=FALSE]
  }
  if(is.null(Y)) {
    if(!periodic) {
      zz <- .C("hasXclose",
               n = as.integer(nX),
               x = as.double(cX$x),
               y = as.double(cX$y),
               r = as.double(r),
               t = as.integer(integer(nX)),
               PACKAGE = "spatstat")
    } else {
      b <- sidelengths(Frame(X))
      zz <- .C("hasXpclose",
               n = as.integer(nX),
               x = as.double(cX$x),
               y = as.double(cX$y),
               r = as.double(r),
               b = as.double(b),
               t = as.integer(integer(nX)),
               PACKAGE = "spatstat")
    }
  } else {
    stopifnot(is.ppp(Y))
    nY <- npoints(Y)
    if(nY == 0) return(logical(nX))
    cY <- coords(Y)
    #' sort Y by increasing x coordinate
    if(!sorted) {
      ooY <- order(cY$x)
      cY <- cY[ooY, , drop=FALSE]
    }
    if(!periodic) {
      zz <- .C("hasXYclose",
               n1 = as.integer(nX),
               x1 = as.double(cX$x),
               y1 = as.double(cX$y),
               n2 = as.integer(nY),
               x2 = as.double(cY$x),
               y2 = as.double(cY$y),
               r = as.double(r),
               t = as.integer(integer(nX)),
               PACKAGE = "spatstat")
    } else {
      bX <- sidelengths(Frame(X))
      bY <- sidelengths(Frame(Y))
      if(any(bX != bY))
        warning("Windows are not equal: periodic distance may be erroneous")
      zz <- .C("hasXYpclose",
               n1 = as.integer(nX),
               x1 = as.double(cX$x),
               y1 = as.double(cX$y),
               n2 = as.integer(nY),
               x2 = as.double(cY$x),
               y2 = as.double(cY$y),
               r = as.double(r),
               b = as.double(bX),
               t = as.integer(integer(nX)),
               PACKAGE = "spatstat")
    }
  }
  tt <- as.logical(zz$t)
  if(sorted) return(tt)
  #' reinstate original order
  ans <- logical(nX)
  ans[oo] <- tt
  return(ans)
}

has.close.pp3 <- function(X, r, Y=NULL, ..., periodic=FALSE, sorted=FALSE) {
  trap.extra.arguments(...)
  nX <- npoints(X)
  if(nX <= 1) return(logical(nX))
  cX <- coords(X)
  if(!sorted) {
    #' sort by increasing x coordinate
    oo <- order(cX$x)
    cX <- cX[oo,,drop=FALSE]
  }
  if(is.null(Y)) {
    if(!periodic) {
      zz <- .C("hasX3close",
               n = as.integer(nX),
               x = as.double(cX$x),
               y = as.double(cX$y),
               z = as.double(cX$z),
               r = as.double(r),
               t = as.integer(integer(nX)),
               PACKAGE = "spatstat")
    } else {
      b <- sidelengths(as.box3(X))
      zz <- .C("hasX3pclose",
               n = as.integer(nX),
               x = as.double(cX$x),
               y = as.double(cX$y),
               z = as.double(cX$z),
               r = as.double(r),
               b = as.double(b), 
               t = as.integer(integer(nX)),
               PACKAGE = "spatstat")
    }
  } else {
    stopifnot(is.pp3(Y))
    nY <- npoints(Y)
    if(nY == 0) return(logical(nX))
    cY <- coords(Y)
    if(!sorted) {
      #' sort Y by increasing x coordinate
      ooY <- order(cY$x)
      cY <- cY[ooY, , drop=FALSE]
    }
    if(!periodic) {
      zz <- .C("hasXY3close",
               n1 = as.integer(nX),
               x1 = as.double(cX$x),
               y1 = as.double(cX$y),
               z1 = as.double(cX$z),
               n2 = as.integer(nY),
               x2 = as.double(cY$x),
               y2 = as.double(cY$y),
               z2 = as.double(cY$z),
               r = as.double(r),
               t = as.integer(integer(nX)),
               PACKAGE = "spatstat")
    } else {
      bX <- sidelengths(as.box3(X))
      bY <- sidelengths(as.box3(Y))
      if(any(bX != bY))
        warning("Domains are not equal: periodic distance may be erroneous")
      zz <- .C("hasXY3pclose",
               n1 = as.integer(nX),
               x1 = as.double(cX$x),
               y1 = as.double(cX$y),
               z1 = as.double(cX$z),
               n2 = as.integer(nY),
               x2 = as.double(cY$x),
               y2 = as.double(cY$y),
               z2 = as.double(cY$z),
               r = as.double(r),
               b = as.double(bX),
               t = as.integer(integer(nX)),
               PACKAGE = "spatstat")
    }
  }
  tt <- as.logical(zz$t)
  if(sorted) return(tt)
  #' reinstate original order
  ans <- logical(nX)
  ans[oo] <- tt
  return(ans)
}


  

#'  fourierbasis.R
#'    $Revision: 1.4 $  $Date: 2017/11/04 04:10:32 $

fourierbasis <- function(x, k, win = boxx(rep(list(0:1), ncol(k)))) {
  x <- as.matrix(x)
  k <- as.matrix(k)
  if (nrow(k) == 0 | nrow(x) == 0) 
    return(complex())
  d <- ncol(x)
  if (ncol(k) != d) 
    stop("Arguments x and k must have the same number of columns.")
  win <- as.boxx(win)
  boxlengths <- as.numeric(win$ranges[2L, ] - win$ranges[1L, ])
  if (length(boxlengths) != d) 
    stop("The box dimension differs from the number of columns in x and k")
  return(fourierbasisraw(x, k, boxlengths))
}

fourierbasisraw <- function(x, k, boxlengths) {
  two_pi_i <- 2 * pi * (0+1i)
  rslt <- outer(k[, 1L], x[, 1L]/boxlengths[1L])
  d <- ncol(x)
  if (d > 1) {
    for (i in 2:d) {
      rslt <- rslt + outer(k[, i], x[, i]/boxlengths[i])
    }
  }
  return(exp(two_pi_i * rslt)/sqrt(prod(boxlengths)))
}

#
# replace.ppp.R
#


"[<-.ppp" <-
  function(x, i, j, value) {
    verifyclass(x, "ppp")
    verifyclass(value, "ppp")
    
    if(missing(i) && missing(j))
      return(value)

    if(missing(i)) {
      message("The use of argument j in [<-.ppp is deprecated; use argument i")
      # invoke code below
      x[j] <- value
      return(x)
    }

    xmf <- markformat(x)
    vmf <- markformat(value)
    if(xmf != vmf) {
      if(xmf == "none")
        stop("Replacement points are marked, but x is not marked")
      else if(vmf == "none")
        stop("Replacement points have no marks, but x is marked")
      else
        stop("Format of marks in replacement is incompatible with original")
    }
    
    if(inherits(i, "owin")) {
      win <- i
      vok <- inside.owin(value$x, value$y, win)
      if(!all(vok)) {
        warning("Replacement points outside the specified window were deleted")
        value <- value[vok]
      }
      # convert to vector index
      i <- inside.owin(x$x, x$y, win)
    }
    if(!is.vector(i))
      stop("Unrecognised format for subset index i")
    
    # vector index
    # determine index subset
    n <- x$n
    SUB <- seq_len(n)[i]
    # anything to replace?
    if(length(SUB) == 0)
      return(x)
    # sanity checks
    if(anyNA(SUB))
      stop("Invalid subset: the resulting subscripts include NAs")
    # exact replacement of this subset?
    if(value$n == length(SUB)) {
      x$x[SUB] <- value$x
      x$y[SUB] <- value$y
      switch(xmf,
             none={},
             list=,
             vector={ x$marks[SUB] <- value$marks },
             dataframe={ x$marks[SUB,] <- value$marks })
    } else 
      x <- superimpose(x[-SUB], value, W=x$window)

    if(!missing(j)) {
      warning("The use of argument j in [<-.ppp is deprecated; use argument i")
      # invoke code above
      x[j] <- value
    }
      
    return(x)
}

#
# clip.psp.R
#
#    $Revision: 1.23 $   $Date: 2018/01/23 02:40:14 $
#
#
 
########################################################
# clipping operation (for subset)
########################################################

clip.psp <- function(x, window, check=TRUE, fragments=TRUE) {
  verifyclass(x, "psp")
  verifyclass(window, "owin")
  if(is.vanilla(unitname(window)))
     unitname(window) <- unitname(x)
  if(check && !is.subset.owin(window, x$window))
    warning("The clipping window is not a subset of the window containing the line segment pattern x")
  if(x$n == 0) {
    emptypattern <- psp(numeric(0), numeric(0), numeric(0), numeric(0),
                      window=window, marks=x$marks)
    return(emptypattern)
  }
  switch(window$type,
         rectangle={
           result <- cliprect.psp(x, window, fragments=fragments)
         },
         polygonal={
           result <- clippoly.psp(x, window, fragments=fragments)
         },
         mask={
           result <- clippoly.psp(x, as.polygonal(window), fragments=fragments)
           result$window <- window
         })
  return(result)
}


#####
#
#  clipping to a rectangle
#
cliprect.psp <- local({
  
  cliprect.psp <- function(x, window, fragments=TRUE) {
    verifyclass(x, "psp")
    verifyclass(window, "owin")
    ends <- x$ends
    marx <- marks(x, dfok=TRUE)
    #' find segments which are entirely inside the window
    #' (by convexity)
    in0 <- inside.owin(ends$x0, ends$y0, window)
    in1 <- inside.owin(ends$x1, ends$y1, window)
    ok <- in0 & in1
    #' if all segments are inside, return them
    if(all(ok))
      return(as.psp(ends, window=window, marks=marx, check=FALSE))
    #' otherwise, store those segments which are inside the window
    ends.inside <- ends[ok, , drop=FALSE]
    marks.inside <- marx %msub% ok
    x.inside <- as.psp(ends.inside, window=window, marks=marks.inside,
                       check=FALSE)
    if(!fragments)
      return(x.inside)
    #' now consider the rest
    ends <- ends[!ok, , drop=FALSE]
    in0 <- in0[!ok] 
    in1 <- in1[!ok]
    marx <- marx %msub% (!ok)
    #' first clip segments to the range x \in [xmin, xmax]
    #' use parametric coordinates
    tx <- cbind(ifelse0NA(between(ends$x0, window$xrange)),
                ifelse1NA(between(ends$x1, window$xrange)),
                tvalue(ends$x0, ends$x1, window$xrange[1L]),
                tvalue(ends$x0, ends$x1, window$xrange[2L]))
    #' discard segments which do not lie in the x range 
    nx <- apply(!is.na(tx), 1L, sum)
    ok <- (nx >= 2)
    if(!any(ok))
      return(x.inside)
    ends <- ends[ok, , drop=FALSE]
    tx   <- tx[ok, , drop=FALSE]
    in0  <- in0[ok]
    in1  <- in1[ok]
    marx <- marx %msub% ok
    #' Clip the segments to the x range
    tmin <- apply(tx, 1L, min, na.rm=TRUE)
    tmax <- apply(tx, 1L, max, na.rm=TRUE)
    dx <- ends$x1 - ends$x0
    dy <- ends$y1 - ends$y0
    ends.xclipped <- data.frame(x0=ends$x0 + tmin * dx,
                                y0=ends$y0 + tmin * dy,
                                x1=ends$x0 + tmax * dx,
                                y1=ends$y0 + tmax * dy)
    #' Now clip the segments to the range y \in [ymin, ymax]
    ends <- ends.xclipped
    in0 <- inside.owin(ends$x0, ends$y0, window)
    in1 <- inside.owin(ends$x1, ends$y1, window)
    ty <- cbind(ifelse0NA(in0),
                ifelse1NA(in1),
                tvalue(ends$y0, ends$y1, window$yrange[1L]),
                tvalue(ends$y0, ends$y1, window$yrange[2L]))
    #' discard segments which do not lie in the y range 
    ny <- apply(!is.na(ty), 1L, sum)
    ok <- (ny >= 2)
    if(!any(ok))
      return(x.inside)
    ends <- ends[ok, , drop=FALSE]
    ty   <- ty[ok, , drop=FALSE]
    in0  <- in0[ok]
    in1  <- in1[ok]
    marx <- marx %msub% ok
    #' Clip the segments to the y range
    tmin <- apply(ty, 1L, min, na.rm=TRUE)
    tmax <- apply(ty, 1L, max, na.rm=TRUE)
    dx <- ends$x1 - ends$x0
    dy <- ends$y1 - ends$y0
    ends.clipped <- data.frame(x0=ends$x0 + tmin * dx,
                               y0=ends$y0 + tmin * dy,
                               x1=ends$x0 + tmax * dx,
                               y1=ends$y0 + tmax * dy)
    marks.clipped <- marx
    #' OK - segments clipped
    #' Put them together with the unclipped ones
    ends.all <- rbind(ends.inside, ends.clipped)
    marks.all <- marks.inside %mapp% marks.clipped
    as.psp(ends.all, window=window, marks=marks.all, check=FALSE)
  }

  small <- function(x) { abs(x) <= .Machine$double.eps }

  tvalue <- function(z0, z1, zt) {
    y1 <- z1 - z0
    yt <- zt - z0
    tval <- ifelseAX(small(y1), 0.5, yt/y1)
    betwee <- (yt * (zt - z1)) <= 0
    result <- ifelseXB(betwee, tval, NA)
    return(result)
  }

  between <- function(x, r) { ((x-r[1L]) * (x-r[2L])) <= 0 }

  cliprect.psp
})

############################
#
#   clipping to a polygonal window
#

clippoly.psp <- function(s, window, fragments=TRUE) {
  verifyclass(s, "psp")
  verifyclass(window, "owin")
  stopifnot(window$type == "polygonal")
  marx <- marks(s)
  has.marks <- !is.null(marx)
  
  eps <- .Machine$double.eps

  # find the intersection points between segments and window edges
  
  ns <- s$n
  es <- s$ends
  x0s <- es$x0
  y0s <- es$y0
  x1s <- es$x1
  y1s <- es$y1
  dxs <- x1s - x0s
  dys <- y1s - y0s

  bdry <- edges(window)
  nw <- bdry$n
  ew <- bdry$ends
  x0w <- ew$x0
  y0w <- ew$y0
  dxw <- ew$x1 - ew$x0
  dyw <- ew$y1 - ew$y0

  out <- .C("xysegint",
            na=as.integer(ns),
            x0a=as.double(x0s),
            y0a=as.double(y0s),
            dxa=as.double(dxs),
            dya=as.double(dys), 
            nb=as.integer(nw),
            x0b=as.double(x0w),
            y0b=as.double(y0w),
            dxb=as.double(dxw),
            dyb=as.double(dyw), 
            eps=as.double(eps),
            xx=as.double(numeric(ns * nw)),
            yy=as.double(numeric(ns * nw)),
            ta=as.double(numeric(ns * nw)),
            tb=as.double(numeric(ns * nw)),
            ok=as.integer(integer(ns * nw)),
            PACKAGE = "spatstat")

  hitting <- (matrix(out$ok, ns, nw) != 0)
  ts <- matrix(out$ta, ns, nw)

  anyhit <- matrowany(hitting)
  
  if(!fragments) {
    #' retain only segments that avoid the boundary entirely
    leftin <- inside.owin(es$x0, es$y0, window)
    rightin <- inside.owin(es$x1, es$y1, window)
    ok <- !anyhit & leftin & rightin
    return(as.psp(es[ok,,drop=FALSE],
                  window=window,
		  marks=marx %msub% ok,
		  check=FALSE))
  }
  # form all the chopped segments (whether in or out)

  #' initially empty
  chopx0 <- chopy0 <- chopx1 <- chopy1 <- numeric(0)
  chopmarks <- marx %msub% integer(0)
    
  for(seg in seq_len(ns)) {
    #' coordinates of segment number 'seg'
    segx0 <- x0s[seg]
    segy0 <- y0s[seg]
    segx1 <- x1s[seg]
    segy1 <- y1s[seg]
    if(has.marks)
      segmarks <- marx %msub% seg
    if(!anyhit[seg]) {
      #' no intersection with boundary - add single segment
      chopx0 <- c(chopx0, segx0)
      chopy0 <- c(chopy0, segy0)
      chopx1 <- c(chopx1, segx1)
      chopy1 <- c(chopy1, segy1)
      if(has.marks)
        chopmarks <- chopmarks %mapp% segmarks
    } else {
      #' crosses boundary - add several pieces
      tvals <- ts[seg,]
      tvals <- sort(tvals[hitting[seg,]])
      dx <- segx1 - segx0
      dy <- segy1 - segy0
      chopx0 <- c(chopx0, segx0 + c(0,tvals) * dx)
      chopy0 <- c(chopy0, segy0 + c(0,tvals) * dy)
      chopx1 <- c(chopx1, segx0 + c(tvals,1) * dx)
      chopy1 <- c(chopy1, segy0 + c(tvals,1) * dy)
      if(has.marks) {
        npieces <- length(tvals) + 1L
        chopmarks <- chopmarks %mapp% (segmarks %mrep% npieces)
      }
    }
  }

  chopped <- psp(chopx0, chopy0, chopx1, chopy1,
                 window=boundingbox(Window(s), window),
                 marks=chopmarks)
  
  # select those chopped segments which are inside the window
  mid <- midpoints.psp(chopped)
  ins <- inside.owin(mid$x, mid$y, window)
  retained <- chopped[ins]
  retained$window <- window
  return(retained)
}

#
#	ppp.R
#
#	A class 'ppp' to define point patterns
#	observed in arbitrary windows in two dimensions.
#
#	$Revision: 4.113 $	$Date: 2020/05/03 03:10:05 $
#
#	A point pattern contains the following entries:	
#
#		$window:	an object of class 'owin'
#				defining the observation window
#
#		$n:	the number of points (for efficiency)
#	
#		$x:	
#		$y:	vectors of length n giving the Cartesian
#			coordinates of the points.
#
#	It may also contain the entry:	
#
#		$marks:	a vector of length n
#			whose entries are interpreted as the
#			'marks' attached to the corresponding points.	
#	
#--------------------------------------------------------------------------
ppp <- function(x, y, ..., window, marks,
                check=TRUE, checkdup=check, drop=TRUE) {
  # Constructs an object of class 'ppp'
  #
  if(!missing(window))
    verifyclass(window, "owin")
  else
    window <- owin(...)

  if((missing(x) && missing(y)) || (length(x) == 0 && length(y) == 0))
    x <- y <- numeric(0)

  n <- length(x)
  if(length(y) != n)
    stop("coordinate vectors x and y are not of equal length")
  
  # validate x, y coordinates
  stopifnot(is.numeric(x))
  stopifnot(is.numeric(y))
  good <- is.finite(x) & is.finite(y)
  if(naughty <- !all(good)) {
    #' bad values will be discarded
    nbad <- sum(!good)
    nna <- sum(is.na(x) | is.na(y))
    ninf <- nbad - nna
    if(nna > 0) 
      warning(paste(nna,  "out of", n, ngettext(n, "point", "points"),
                    "had NA or NaN coordinate values, and",
                    ngettext(nna, "was", "were"), "discarded"))
    if(ninf > 0) 
      warning(paste(ninf,  "out of", n, ngettext(n, "point", "points"),
                    "had infinite coordinate values, and",
                    ngettext(ninf, "was", "were"), "discarded"))
    #' chuck out
    x <- x[good]
    y <- y[good]
    n <- sum(good)
  }

  names(x) <- NULL
  names(y) <- NULL
  
  # check (x,y) points lie inside window
  if(check && n > 0) {
    ok <- inside.owin(x, y, window)
    nout <- sum(!ok)
    if(nout > 0) {
      warning(paste(nout,
                    ngettext(nout, "point was", "points were"),
                    "rejected as lying outside the specified window"),
              call.=FALSE)
      rr <- ripras(x,y)
      bb <- boundingbox(x,y)
      bb <- boundingbox(rr, bb, window)
      rejectwindow <-
        if(!is.null(rr)) rebound.owin(rr, bb) else bb
      rejects <- ppp(x[!ok], y[!ok], window=rejectwindow, check=FALSE)
      # discard illegal points
      x <- x[ok]
      y <- y[ok]
      n <- length(x)
    }
  } else nout <- 0
  # initialise ppp object
  pp <- list(window=window, n=n, x=x, y=y)
  # coerce marks to appropriate format
  if(missing(marks))
    marks <- NULL
  if(is.hyperframe(marks)) 
    stop("Hyperframes of marks are not implemented for ppp objects; use ppx")
  if(is.matrix(marks)) 
    marks <- as.data.frame(marks)
  ## drop dimensions?
  if(drop && is.data.frame(marks)) {
    nc <- ncol(marks)
    if(nc == 0)
      marks <- NULL
    else if(nc == 1)
      marks <- marks[,,drop=TRUE]
  }
  # attach marks 
  if(is.null(marks)) {
    # no marks
    pp$markformat <- "none"
  } else if(is.data.frame(marks)) {
    # data frame of marks
    pp$markformat <- "dataframe"
    if(naughty) {
      #' remove marks attached to discarded points with non-finite coordinates
      marks <- marks[good, ]
    }
    if(nout > 0) {
      #' sequester marks of points falling outside window
      marks(rejects) <- marks[!ok,]
      marks <- marks[ok, ]
    }
    if(nrow(marks) != n)
      stop("number of rows of marks != length of x and y")
    pp$marks <- marks
  } else {
    # should be a vector or factor
    # To recognise vector, strip attributes
    isspecial <- is.factor(marks) ||
                 inherits(marks, "POSIXt") || inherits(marks, "Date")
    if(!isspecial)
      attributes(marks) <- NULL
    if(!(is.vector(marks) || isspecial))
      stop("Format of marks not understood")
    # OK, it's a vector or factor
    pp$markformat <- "vector"
    if(naughty) {
      #' remove marks attached to discarded points with non-finite coordinates
      marks <- marks[good]
    }
    if(nout > 0) {
      #' sequester marks of points falling outside window
      marks(rejects) <- marks[!ok]
      marks <- marks[ok]
    }
    if(length(marks) != n)
      stop("length of marks vector != length of x and y")
    names(marks) <- NULL
    pp$marks <- marks
  }
  class(pp) <- "ppp"
  if(checkdup && anyDuplicated(pp))
    warning("data contain duplicated points", call.=FALSE)
  if(nout > 0) 
    attr(pp, "rejects") <- rejects
  pp
}

#
#--------------------------------------------------------------------------
#

is.ppp <- function(x) { inherits(x, "ppp") }

#
#--------------------------------------------------------------------------
#

as.ppp <- function(X, ..., fatal=TRUE) {
  UseMethod("as.ppp")
}

as.ppp.ppp <- function(X, ..., fatal=TRUE) {
  check <- resolve.defaults(list(...), list(check=FALSE))$check
  return(ppp(X$x, X$y, window=X$window, marks=X$marks, check=check))
}

as.ppp.quad <- function(X, ..., fatal=TRUE) {
  return(union.quad(X))
}

as.ppp.data.frame <- function(X, W = NULL, ..., fatal=TRUE) {
  X <- as.data.frame(X) #' swim against the tidyverse
  check <- resolve.defaults(list(...), list(check=TRUE))$check
  if(ncol(X) < 2) 
    return(complaining("X must have at least two columns",
                       fatal, value=NULL))

  if(is.null(W))
    return(complaining("x,y coords given but no window specified",
                       fatal, value=NULL))

  # columns 1 and 2 are assumed to be coordinates
  # marks from other columns
  marx <- if(ncol(X) > 2) X[, -(1:2)] else NULL

  if(is.function(W))
    Z <- cobble.xy(X[,1], X[,2], W, fatal, marks=marx, check=check)
  else {
    win <- as.owin(W)
    Z <- ppp(X[,1], X[,2], window = win, marks=marx, check=check)
  }

  return(Z)
}
    
as.ppp.matrix <- function(X, W = NULL, ..., fatal=TRUE) {
  check <- resolve.defaults(list(...), list(check=TRUE))$check
  if(!verifyclass(X, "matrix", fatal=fatal)
     || !is.numeric(X))
    return(complaining("X must be a numeric matrix",
                       fatal, value=NULL))

  if(ncol(X) < 2)
    return(complaining("X must have at least two columns",
                       fatal, value=NULL))

  if(is.null(W))
    return(complaining("x,y coords given but no window specified",
                       fatal, value=NULL))
    
  if(is.function(W))
    Z <- cobble.xy(X[,1], X[,2], W, fatal)
  else {
    win <- as.owin(W)
    Z <- ppp(X[,1], X[,2], window = win, check=check)
  }

  # add marks from other columns
  if(ncol(X) > 2)
    marks(Z) <- X[, -(1:2)]

  return(Z)
}
    
as.ppp.default <- function(X, W=NULL, ..., fatal=TRUE) {
	# tries to coerce data X to a point pattern
	# X may be:
	#	1. a structure with entries x, y, xl, xu, yl, yu
	#	2. a structure with entries x, y, area where
        #                    'area' has entries xl, xu, yl, yu
	#	3. a structure with entries x, y
        #       4. a vector of length 2, interpreted as a single point.
	# The second argument W is coerced to an object of class 'owin' by the 
	# function "as.owin" in window.S
        # If X also has an entry X$marks
        # then this will be interpreted as the marks vector for the pattern.
	#
  check <- resolve.defaults(list(...), list(check=TRUE))$check
  if(checkfields(X, c("x", "y", "xl", "xu", "yl", "yu"))) {
		xrange <- c(X$xl, X$xu)
		yrange <- c(X$yl, X$yu)
		if(is.null(X$marks))
			Z <- ppp(X$x, X$y, xrange, yrange, check=check)
		else
			Z <- ppp(X$x, X$y, xrange, yrange, 
				marks=X$marks, check=check)
		return(Z)
        } else if(checkfields(X, c("x", "y", "area"))
                  && checkfields(X$area, c("xl", "xu", "yl", "yu"))) {
                win <- as.owin(X$area)
                if (is.null(X$marks))
                  Z <- ppp(X$x, X$y, window=win, check=check)
                else
                  Z <- ppp(X$x, X$y, window=win, marks = X$marks, check=check)
                return(Z)
	} else if(checkfields(X, c("x", "y"))) {
                if(is.function(W))
                  return(cobble.xy(X$x, X$y, W, fatal))
		if(is.null(W)) {
                  if(fatal)
                    stop("x,y coords given but no window specified")
                  else
                    return(NULL)
                }
		win <- as.owin(W)
		if(is.null(X$marks))
                  Z <- ppp(X$x, X$y, window=win, check=check)
                else
                  Z <- ppp(X$x, X$y, window=win, marks=X$marks, check=check)
                return(Z)
        } else if(is.vector(X) && length(X) == 2) {
                win <- as.owin(W)
                Z <- ppp(X[1], X[2], window=win, check=check)
                return(Z)
	} else {
          if(fatal)
            stop("Can't interpret X as a point pattern")
          else
            return(NULL)
        }
}

cobble.xy <- function(x, y, f=ripras, fatal=TRUE, ...) {
  if(!is.function(f))
    stop("f is not a function")
  w <- f(x,y)
  if(!is.owin(w)) {
    gripe <- "Supplied function f did not return an owin object"
    if(fatal)
      stop(gripe)
    else {
      warning(gripe)
      return(NULL)
    }
  }
  return(ppp(x, y, window=w, ...))
}
  

# --------------------------------------------------------------

"[.ppp" <-
  function(x, i, j, drop=FALSE, ..., clip=FALSE) {

        verifyclass(x, "ppp")
        
        if(!missing(i)) {
          if(inherits(i, "owin")) {
            # i is a window
            window <- i
            if(clip) window <- intersect.owin(window, x$window)
            if(is.vanilla(unitname(window)))
              unitname(window) <- unitname(x)
            ok <- inside.owin(x$x, x$y, window)
            x <- ppp(x$x[ok], x$y[ok], window=window, #SIC
                     marks=marksubset(x$marks, ok),
                     check=FALSE)
          } else if(inherits(i, "im")) {
            # i is an image
            if(i$type != "logical")
              stop(paste("Subset operator X[i] undefined",
                         "when i is a pixel image",
                         "unless it has logical values"), call.=FALSE)
            # convert logical image to window
            e <- sys.frame(sys.nframe())
            window <- solutionset(i, e)
            if(clip) window <- intersect.owin(window, x$window)
            ok <- inside.owin(x$x, x$y, window)
            x <- ppp(x$x[ok], x$y[ok], window=window, #SIC
                     marks=marksubset(x$marks, ok),
                     check=FALSE)
          } else {
            # assume i is a subset index
            nx <- x$n
            if(nx == 0)
              return(x)
            subset <- seq_len(nx)[i]
            if(anyNA(subset))
              stop("Index out of bounds in [.ppp", call.=FALSE)
            x <- ppp(x$x[subset], x$y[subset], window=x$window,
                     marks=marksubset(x$marks, subset),
                     check=FALSE)
          } 
        }

        if(!missing(j))
          x <- x[j]   # invokes code above

        if(drop) {
          #' drop unused factor levels
          mx <- x$marks
          switch(markformat(mx),
                 none = { },
                 vector = {
                   if(is.factor(mx))
                     marks(x) <- factor(mx) # this preserves order of levels
                 },
                 dataframe = {
                   #' must be an actual data frame, not a matrix
                   if(is.data.frame(mx)) {
                     ml <- as.list(mx)
                     isfac <- sapply(ml, is.factor)
                     if(any(isfac))
                       mx[, isfac] <- as.data.frame(lapply(ml[isfac], factor))
                   }
                 },
                 hyperframe = { })
        }
               
        return(x)
}


# ------------------------------------------------------------------
#
#
scanpp <- function(filename, window, header=TRUE, dir="",
                   factor.marks = NULL, ...) {
  filename <- if(dir=="") filename else
              paste(dir, filename, sep=.Platform$file.sep)
  df <- read.table(filename, header=header,
                   stringsAsFactors = is.null(factor.marks))
  if(header) {
    # check whether there are columns named 'x' and 'y'
    colnames <- dimnames(df)[[2]]
    xycolumns <- match(c("x", "y"), colnames, 0)
    named <- all(xycolumns > 0)
  } else {
    named <- FALSE
  }
  if(named) {
    x <- df$x
    y <- df$y
  } else {
    # assume x, y given in columns 1, 2 respectively
    x <- df[,1]
    y <- df[,2]
    xycolumns <- c(1,2)
  }
  if(ncol(df) == 2) 
      X <- ppp(x, y, window=window)
  else {
      # Catch old argument "multitype":
      dots <- list(...)
      multi <- charmatch(names(dots), "multitype")
      argindex <- which(!is.na(multi))
      if(length(argindex)>0){
          if(missing(factor.marks)){
              factor.marks <- dots[[argindex]]
              ignored <- ""
          } else{
              ignored <- paste(" and it is ignored since",
                               sQuote("factor.marks"),
                               "is also supplied")
          }
          warning("It appears you have called scanpp ",
                  " with (something partially matching) ",
                  " the deprecated argument ",
                  paste0(sQuote("multitype"), ignored, "."),
                  " Please change to the new syntax.")
      }
    marks <- df[ , -xycolumns, drop=FALSE]
    if(any(factor.marks)){
        # Find indices to convert to factors (recycling to obtain correct length)
        factorid <- (1:ncol(marks))[factor.marks]
        # Convert relevant columns to factors
        marks[,factorid] <- lapply(marks[,factorid,drop=FALSE], factor)
    }
    X <- ppp(x, y, window=window, marks = marks)
  }
  X
}

#-------------------------------------------------------------------

"markspace.integral" <-
  function(X) {
  verifyclass(X, "ppp")
  if(!is.marked(X, dfok=TRUE))
    return(1)
  if(is.multitype(X))
    return(length(levels(marks(X))))
  else
    stop("Don't know how to compute total mass of mark space")
}

#-------------------------------------------------------------------

print.ppp <- function(x, ...) {
  verifyclass(x, "ppp")
  ism <- is.marked(x, dfok=TRUE)
  nx <- x$n
  splat(if(ism) "Marked planar" else "Planar",
        "point pattern:",
        nx, ngettext(nx, "point", "points"))
  if(ism) {
    mks <- marks(x, dfok=TRUE)
    if(is.data.frame(mks)) {
      ## data frame of marks
      exhibitStringList("Mark variables:", names(mks))
    } else {
      ## vector of marks
      if(is.factor(mks)) {
        exhibitStringList("Multitype, with levels =", levels(mks))
      } else {
        ## Numeric, or could be dates
        if(inherits(mks, "Date")) {
          splat("marks are dates, of class", sQuote("Date"))
        } else if(inherits(mks, "POSIXt")) {
          splat("marks are dates, of class", sQuote("POSIXt"))
        } else {
          splat(paste0("marks are", if(is.numeric(mks)) " numeric," else NULL),
                "of storage type ", sQuote(typeof(mks)))
        }
      }
    }
  }
  print(x$window)
  terselevel <- spatstat.options('terse')
  if(waxlyrical('errors', terselevel) &&
     !is.null(rejects <- attr(x, "rejects"))) {
    nrejects <- rejects$n
    splat("***",
          nrejects,
          ngettext(nrejects, "illegal point", "illegal points"),
          "stored in",
          paste0("attr(,", dQuote("rejects"), ")"),
          "***")
  }
  if(waxlyrical('extras', terselevel) &&
     !is.null(info <- attr(x, "info")) && inherits(info, "rmhInfoList"))
    splat("Pattern was generated by",
          if(is.poisson(info$model)) "Poisson" else "Metropolis-Hastings",
	  "simulation.")
  return(invisible(NULL))
}


summary.ppp <- function(object, ..., checkdup=TRUE) {
  verifyclass(object, "ppp")
  result <- list()
  result$is.marked <- is.marked(object, dfok=TRUE)
  result$n <- object$n
  result$window <- summary(object$window)
  result$intensity <- result$n/result$window$area
  if(checkdup) {
    result$nduplicated <- sum(duplicated(object))
    result$rounding <- rounding(object)
  }
  if(result$is.marked) {
    mks <- marks(object, dfok=TRUE)
    if(result$multiple.marks <- is.data.frame(mks)) {
      result$marknames <- names(mks)
      result$is.numeric <- FALSE
      result$marktype <- "dataframe"
      result$is.multitype <- FALSE
    } else {
      result$is.numeric <- is.numeric(mks)
      result$marknames <- "marks"
      result$marktype <- typeof(mks)
      result$is.multitype <- is.multitype(object)
    }
    if(result$is.multitype) {
      tm <- as.vector(table(mks))
      tfp <- data.frame(frequency=tm,
                        proportion=tm/sum(tm),
                        intensity=tm/result$window$area,
                        row.names=levels(mks))
      result$marks <- tfp
    } else 
      result$marks <- summary(mks)
  }
  class(result) <- "summary.ppp"
  if(!is.null(rejects <- attr(object, "rejects"))) 
    result$rejects <- rejects$n
  if(!is.null(info <- attr(object, "info")) && inherits(info, "rmhInfoList"))
    result$rmhinfo <- info
  return(result)
}

print.summary.ppp <- function(x, ..., dp=getOption("digits")) {
  verifyclass(x, "summary.ppp")
  terselevel <- spatstat.options("terse")
  splat(if(x$is.marked) "Marked planar" else "Planar",
        "point pattern: ",
        x$n,
        "points")
  oneline <- resolve.defaults(list(...), list(oneline=FALSE))$oneline
  if(oneline) return(invisible(NULL))
  unitinfo <- summary(x$window$units)
  splat("Average intensity",
        signif(x$intensity,dp),
        "points per square",
        unitinfo$singular,
        unitinfo$explain)
  ndup <- x$nduplicated
  if(waxlyrical('extras', terselevel) && !is.null(ndup) && (ndup > 0)) {
    parbreak(terselevel)
    splat("*Pattern contains duplicated points*")
  }
  rndg <- x$rounding
  if(waxlyrical('gory', terselevel) && !is.null(rndg)) {
    cat("\n")
    if(rndg >= 1) {
      cat("Coordinates are", "given to",
          rndg,
          "decimal", ngettext(rndg, "place", "places"),
          fill=TRUE)
      if(rndg <= 3) {
        cat("i.e. rounded to", "the nearest", "multiple of",
            10^(-rndg), unitinfo$plural, unitinfo$explain,
            fill=TRUE)
      }
    } else if(rndg == 0) {
      cat("Coordinates are", "integers", fill=TRUE)
      cat("i.e. rounded to", "the nearest", unitinfo$singular,
          unitinfo$explain, 
          fill=TRUE)
    } else {
      cat("Coordinates are", "multiples of",
          10^(-rndg), unitinfo$plural, unitinfo$explain, 
          fill=TRUE)
    }
    parbreak(terselevel)
  }
  if(x$is.marked) {
    if(x$multiple.marks) {
      splat("Mark variables:", commasep(x$marknames, ", "))
      cat("Summary:\n")
      print(x$marks)
    } else if(x$is.multitype) {
      cat("Multitype:\n")
      print(signif(x$marks,dp))
    } else {
      splat("marks are ",
            if(x$is.numeric) "numeric, ",
            "of type ", sQuote(x$marktype),
            sep="")
      cat("Summary:\n")
      print(x$marks)
    }
    parbreak(terselevel)
  }
  if(waxlyrical('extras', terselevel))
    print(x$window)
  if(waxlyrical('errors', terselevel) && !is.null(nrejects <- x$rejects)) {
    parbreak(terselevel)
    splat("***",
          nrejects,
          ngettext(nrejects, "illegal point", "illegal points"),
          "stored in",
          paste("attr(,", dQuote("rejects"), ")", sep=""),
          "***")
  }
  if(waxlyrical('gory', terselevel) && !is.null(info <- x$rmhinfo)) {
    cat("\nPattern was generated by",
        "Metropolis-Hastings algorithm rmh",
        fill=TRUE)
    print(info)
  }
  return(invisible(x))
}

# ---------------------------------------------------------------

identify.ppp <- function(x, ...) {
  verifyclass(x, "ppp")
  id <- identify(x$x, x$y, ...)
  if(!is.marked(x)) return(id)
  marks <- as.data.frame(x)[id, -(1:2)]
  out <- cbind(data.frame(id=id), marks)
  row.names(out) <- NULL
  return(out)
}

rebound <- function(x, rect) {
  UseMethod("rebound")
}

rebound.ppp <- function(x, rect) {
  verifyclass(x, "ppp")
  x$window <- rebound.owin(x$window, rect)
  return(x)
}

as.data.frame.ppp <- function(x, row.names=NULL, ...) {
  df <- data.frame(x=x$x, y=x$y, row.names=row.names)
  marx <- marks(x, dfok=TRUE)
  if(is.null(marx))
    return(df)
  if(is.data.frame(marx))
    df <- cbind(df, marx)
  else
    df <- data.frame(df, marks=marx)
  return(df)
}

is.empty.ppp <- function(x) { return(x$n == 0) }

npoints <- function(x) {
  UseMethod("npoints")
}

nobjects <- function(x) {
  UseMethod("nobjects")
}

nobjects.ppp <- npoints.ppp <- function(x) { x$n }


domain.ppp <- Window.ppp <- function(X, ...) { as.owin(X) }

"Window<-.ppp" <- function(X, ..., value) {
  verifyclass(value, "owin")
  return(X[value])
}

"Frame<-.ppp" <- function(X, value) {
  Frame(Window(X)) <- value
  return(X)
}

#' convert any appropriate subset index for any kind of point pattern
#' to a logical vector

ppsubset <- function(X, I, Iname, fatal=FALSE) {
  if(missing(Iname))
    Iname <- deparse(substitute(I))
  # I could be a window or logical image
  if(is.im(I))
    I <- solutionset(I)
  if((is.ppp(X) || is.lpp(X)) && is.owin(I)) {
    I <- inside.owin(X, w=I)
    return(I)
  }
  if((is.pp3(X) && inherits(I, "box3")) ||
     (is.ppx(X) && inherits(I, "boxx"))) {
    I <- inside.boxx(X, w=I)
    return(I)
  }
  # I could be a function to be applied to X
  if(is.function(I)) {
    I <- I(X)
    if(!is.vector(I)) {
      whinge <- paste("Function", sQuote(Iname), "did not return a vector")
      if(fatal) stop(whinge, call.=FALSE)
      warning(whinge, call.=FALSE)
      return(NULL)
    }
  }      
  # I is now a subset index: convert to logical
  I <- grokIndexVector(I, npoints(X))$strict$lo

  if(anyNA(I)) {
    #' illegal entries
    whinge <- paste("Indices in", sQuote(Iname), "exceed array limits")
    if(fatal) stop(whinge, call.=FALSE)
    warning(whinge, call.=FALSE)
    return(NULL)
  }

  return(I)
}

##
##   diagram.R
##
##   Simple objects for the elements of a diagram (text, arrows etc)
##    that are compatible with plot.layered and plot.solist
##
##   $Revision: 1.14 $ $Date: 2018/11/29 05:20:09 $

# ......... internal class 'diagramobj' supports other classes  .........

diagramobj <- function(X, ...) {
  if(inherits(try(Frame(X), silent=TRUE), "try-error"))
    stop("X is not a spatial object")
  a <- list(...)
  if(sum(nzchar(names(a))) != length(a))
    stop("All extra arguments must be named")
  attributes(X) <- append(attributes(X), a)
  class(X) <- c("diagramobj", class(X))
  return(X)
}

"[.diagramobj" <- function(x, ...) {
  y <- NextMethod("[")
  attributes(y) <- attributes(x)
  return(y)
}

shift.diagramobj <- function(X, ...) {
  y <- NextMethod("shift")
  attributes(y) <- attributes(X)
  return(y)
}

scalardilate.diagramobj <- function(X, f, ...) {
  y <- NextMethod("scalardilate")
  attributes(y) <- attributes(X)
  return(y)
}

# .............. user-accessible classes ................
# .........  (these only need a creator and a plot method) ......


## ...........  text .................

textstring <- function(x, y, txt=NULL, ...) {
  if(is.ppp(x) && missing(y)) {
    X <- x
    Window(X) <- boundingbox(x)
  } else {
    if(missing(y) && checkfields(x, c("x", "y"))) {
      y <- x$y
      x <- x$x
      stopifnot(length(x) == length(y))
    }
    X <- ppp(x, y, window=owin(range(x),range(y)))
  }
  marks(X) <- txt
  Y <- diagramobj(X, otherargs=list(...))
  class(Y) <- c("textstring", class(Y))
  return(Y)
}

plot.textstring <- function(x, ..., do.plot=TRUE) {
  txt <- marks(x)
  otha <- attr(x, "otherargs")
  if(do.plot) do.call.matched(text.default,
                              resolve.defaults(list(...),
                                               list(x=x$x, y=x$y, labels=txt),
                                               otha),
                              funargs=graphicsPars("text"))
  return(invisible(Frame(x)))
}

print.textstring <- function(x, ...) {
  splat("Text string object")
  txt <- marks(x)
  if(npoints(x) == 1) {
    splat("Text:", dQuote(txt))
    splat("Coordinates:", paren(paste(as.vector(coords(x)), collapse=", ")))
  } else {
    splat("Text:")
    print(txt)
    splat("Coordinates:")
    print(coords(x))
  }
  return(invisible(NULL))
}
  
## ...........  'yardstick' to display scale information  ................

yardstick <- function(x0, y0, x1, y1, txt=NULL, ...) {
  nomore <- missing(y0) && missing(x1) && missing(y1) 
  if(is.ppp(x0) && nomore) {
    if(npoints(x0) != 2) stop("x0 should consist of exactly 2 points")
    X <- x0
  } else if(is.psp(x0) && nomore) {
    if(nobjects(x0) != 1) stop("x0 should consist of exactly 1 segment")
    X <- endpoints.psp(x0)
  } else {
    xx <- c(x0, x1)
    yy <- c(y0, y1)
    B <- boundingbox(list(x=xx, y=yy))
    X <- ppp(xx, yy, window=B, check=FALSE)
  }
  Window(X) <- boundingbox(X)
  Y <- diagramobj(X, txt=txt, otherargs=list(...))
  class(Y) <- c("yardstick", class(Y))
  return(Y)
}

plot.yardstick <- local({

  mysegments <- function(x0, y0, x1, y1, ..., moreargs=list()) {
    ## ignore unrecognised arguments without whingeing
    do.call.matched(segments,
                    resolve.defaults(list(x0=x0, y0=y0, x1=x1, y1=y1),
                                     list(...),
                                     moreargs),
                    extrargs=c("col", "lty", "lwd", "xpd", "lend"))
  }
  
  myarrows <- function(x0, y0, x1, y1, ...,
                       left=TRUE, right=TRUE,
                       angle=20, frac=0.25,
                       main, show.all, add) {
    mysegments(x0, y0, x1, y1, ...)
    if(left || right) {
      ang <- angle * pi/180
      co <- cos(ang)
      si <- sin(ang)
      dx <- x1-x0
      dy <- y1-y0
      le <- sqrt(dx^2 + dy^2)
      rot <- matrix(c(dx, dy, -dy, dx)/le, 2, 2)
      arlen <- frac * le
      up <- arlen * (rot %*% c(co, si))
      lo <- arlen * (rot %*% c(co, -si))
      if(left) {
        mysegments(x0, y0, x0+up[1L], y0+up[2L], ...)
        mysegments(x0, y0, x0+lo[1L], y0+lo[2L], ...)
      }
      if(right) {
        mysegments(x1, y1, x1-up[1L], y1-up[2L], ...)
        mysegments(x1, y1, x1-lo[1L], y1-lo[2L], ...)
      }
    }
    return(invisible(NULL))
  }

  plot.yardstick <- function(x, ...,
                             angle=20,
                             frac=1/8,
                             split=FALSE,
                             shrink=1/4,
                             pos=NULL,
                             txt.args=list(),
                             txt.shift=c(0,0),
                             do.plot=TRUE) {
    if(do.plot) {
      txt <- attr(x, "txt")
      argh <- resolve.defaults(list(...), attr(x, "otherargs"))
      A <- as.numeric(coords(x)[1L,])
      B <- as.numeric(coords(x)[2L,])
      M <- (A+B)/2
      if(!split) {
        ## double-headed arrow
        myarrows(A[1L], A[2L], B[1L], y1=B[2L],
                 angle=angle, frac=frac, moreargs=argh)
        if(is.null(pos) && !("adj" %in% names(txt.args)))
          pos <- if(abs(A[1L] - B[1L]) < abs(A[2L] - B[2L])) 4 else 3
      } else {
        ## two single-headed arrows with text 
        dM <- (shrink/2) * (B - A)
        AM <- M - dM
        BM <- M + dM
        newfrac <- frac/((1-shrink)/2)
        myarrows(AM[1L], AM[2L], A[1L], A[2L],
                 angle=angle, frac=newfrac, left=FALSE, moreargs=argh)
        myarrows(BM[1L], BM[2L], B[1L], B[2L], 
                 angle=angle, frac=newfrac, left=FALSE, moreargs=argh)
      }
      if(is.null(txt.shift)) txt.shift <- rep(0, 2) else 
                             txt.shift <- ensure2vector(unlist(txt.shift))
      do.call.matched(text.default,
                      resolve.defaults(list(x=M[1L] + txt.shift[1L],
                                            y=M[2L] + txt.shift[2L]),
                                       txt.args,
                                       list(labels=txt, pos=pos),
                                       argh,
                                       .MatchNull=FALSE),
                      funargs=graphicsPars("text"))
    }
    return(invisible(Window(x)))
  }
  plot.yardstick
})


print.yardstick <- function(x, ...) {
  splat("Yardstick")
  if(!is.null(txt <- attr(x, "txt")))
    splat("Text:", txt)
  ui <- summary(unitname(x))
  splat("Length:", pairdist(x)[1L,2L], ui$plural, ui$explain)
  splat("Midpoint:",
        paren(paste(signif(c(mean(x$x), mean(x$y)), 3), collapse=", ")))
  dx <- diff(range(x$x))
  dy <- diff(range(x$y))
  orient <- if(dx == 0) "vertical" else
            if(dy == 0) "horizontal" else
            paste(atan2(dy, dx) * 180/pi, "degrees")
  splat("Orientation:", orient)
  return(invisible(NULL))
}


## code to draw a decent-looking arrow in spatstat diagrams
## (works in layered objects)

## The name 'onearrow' is used because R contains
## hidden functions [.arrow, length.arrow

onearrow <- function(x0, y0, x1, y1, txt=NULL, ...) {
  nomore <- missing(y0) && missing(x1) && missing(y1) 
  if(is.ppp(x0) && nomore) {
    if(npoints(x0) != 2) stop("x0 should consist of exactly 2 points")
    X <- x0
  } else if(is.psp(x0) && nomore) {
    if(nobjects(x0) != 1) stop("x0 should consist of exactly 1 segment")
    X <- endpoints.psp(x0)
  } else {
    xx <- c(x0, x1)
    yy <- c(y0, y1)
    B <- boundingbox(list(x=xx, y=yy))
    X <- ppp(xx, yy, window=B, check=FALSE)
  }
  Window(X) <- boundingbox(X)
  Y <- diagramobj(X, txt=txt, otherargs=list(...))
  class(Y) <- c("onearrow", class(Y))
  return(Y)
}

print.onearrow <- function(x, ...) {
  splat("Single arrow from",
        paren(paste0(x$x[1], ", ", x$y[1])),
        "to",
        paren(paste0(x$x[2], ", ", x$y[2])))
  if(!is.null(txt <- attr(x, "txt")))
    splat("Text:", sQuote(txt))
  if(length(oa <- attr(x, "otherargs"))) {
    cat("Graphical parameters:\n")
    print(unlist(oa))
  }
  return(invisible(NULL))
}

plot.onearrow <- function(x, ...,
                          add=FALSE,
                          main="",
                          retract=0.05,   
                          headfraction=0.25,
                          headangle=12, # degrees
                          headnick=0.1, # fraction of head length
                          col.head=NA,
                          lwd.head=lwd,
                          lwd=1,
                          col=1,
                          zap=FALSE,
                          zapfraction=0.07,
                          pch=1, cex=1,
                          do.plot=TRUE,
                          do.points=FALSE,
                          show.all=!add) {
  result <- plot.ppp(x, main=main, add=add,
                     pch=pch, cex=cex,
                     do.plot=do.plot && do.points,
                     show.all=show.all)
  if(do.plot && !do.points && !add)
    plot(Frame(x), main="", type="n")
  txt <- attr(x, "txt")
  ## resolve formal arguments with those stored in the object
  saved <- attr(x, "otherargs")
  if(missing(col))
    col <- saved[["col"]] %orifnull% col
  if(missing(lwd))
    lwd <- saved[["lwd"]] %orifnull% lwd
  if(missing(pch))
    pch <- saved[["pch"]] %orifnull% pch
  if(missing(cex))
    cex <- saved[["cex"]] %orifnull% cex
  if(missing(col.head))
    col.head <- saved[["col.head"]] %orifnull% col.head
  if(missing(lwd.head))
    lwd.head <- saved[["lwd.head"]] %orifnull% lwd.head
  if(missing(retract))
     retract <- saved[["retract"]] %orifnull% retract
  if(missing(headfraction))
    headfraction <- saved[["headfraction"]] %orifnull% headfraction
  if(missing(headangle))
    headangle <- saved[["headangle"]] %orifnull% headangle
  if(missing(headnick))
    headnick <- saved[["headnick"]] %orifnull% headnick
  if(missing(zap))
    zap <- saved[["zap"]] %orifnull% zap
  if(missing(zapfraction))
    zapfraction <- saved[["zapfraction"]] %orifnull% zapfraction
  argh <- list(col=col, lwd=lwd, cex=cex, pch=pch, ...)
  ## calculate 
  A <- as.numeric(coords(x)[1L,])
  B <- as.numeric(coords(x)[2L,])
  V <- B - A
  AR <- A + retract * V
  BR <- B - retract * V
  H <- B - headfraction * V
  HN <- H + headnick * headfraction * V
  headlength <- headfraction * sqrt(sum(V^2))
  halfwidth <- headlength * tan((headangle/2) * pi/180)
  alpha <- atan2(V[2L], V[1L]) + pi/2
  U <- c(cos(alpha), sin(alpha))
  HL <- H + halfwidth * U
  HR <- H - halfwidth * U
  Head <- rbind(HN, HR, BR, HL, HN)
  objHead <- owin(poly=Head[1:4,])
  parHead <- resolve.defaults(list(col=col.head, lwd=lwd.head),
                              argh)
  if(do.plot && !is.na(col.head))
    do.call.matched(polygon, append(list(x=Head), parHead))

  if(!zap) {
    Tail <- AR
  } else {
    M <- (AR+HN)/2
    dM <- (zapfraction/2) * (1-headfraction) * V
    dM <- dM + c(-dM[2L], dM[1L])
    ML <- M + dM
    MR <- M - dM
    Tail <- rbind(MR, ML, AR)
  }
  parLines <- argh
  if(do.plot) 
    do.call.matched(lines,
                    append(list(x=rbind(Head, Tail)),
                           parLines),
                    extrargs=c("col", "lwd", "lty", "xpd", "lend"))

  HT <- rbind(Head, Tail)
  W <- owin(range(HT[,1]), range(HT[,2]))
  nht <- nrow(HT)
  HT <- cbind(HT[-nht, , drop=FALSE], HT[-1, , drop=FALSE])
  objLines <- as.psp(HT, window=W)

  if(do.plot && !is.null(txt <- attr(x, "txt"))) {
    H <- (A+B)/2
    do.call.matched(text.default,
                    resolve.defaults(
                      list(x=H[1L], y=H[2L]),
                      argh,
                      list(labels=txt, pos=3 + (V[2L] != 0))),
                    funargs=graphicsPars("text"))
  }

  attr(result, "objects") <- layered(Head=objHead, Lines=objLines,
                                     plotargs=list(parHead, parLines))
  return(invisible(result))
}

#
# flipxy.R
#
# flip x and y coordinates
#
# $Revision: 1.3 $ $Date: 2017/02/07 07:22:47 $ 
#

flipxy <- function(X) {
  UseMethod("flipxy")
}

flipxy.ppp <- function(X) {
  stopifnot(is.ppp(X))
  ppp(X$y, X$x, marks=X$marks,
      window=flipxy(X$window), unitname=unitname(X),
      check=FALSE)
}

flipxypolygon <- function(p) {
  # flip x and y coordinates, and reinstate anticlockwise order
  oldy <- p$y
  p$y <- rev(p$x)
  p$x <- rev(oldy)
  # area and hole status unchanged
  return(p)
}

flipxy.owin <- function(X) {
  verifyclass(X, "owin")
  switch(X$type,
         rectangle={
           W <- owin(X$yrange, X$xrange, unitname=unitname(X))
         },
         polygonal={
           bdry <- lapply(X$bdry, flipxypolygon)
           W <- owin(poly=bdry, check=FALSE, unitname=unitname(X))
         },
         mask={
           W <- owin(mask=t(X$m),
                     xy=list(x=X$yrow, y=X$xcol),
                     unitname=unitname(X))
         },
         stop("Unrecognised window type")
         )
  return(W)
}

flipxy.psp <- function(X) {
  stopifnot(is.psp(X))
  flipends <- (X$ends)[, c(2L,1L,4L,3L), drop=FALSE]
  as.psp(flipends, window=flipxy(X$window), marks=X$marks,
         unitname=unitname(X), check=FALSE)
}

flipxy.im <- function(X) {
  im(t(X$v), xcol=X$yrow, yrow=X$xcol, unitname=unitname(X))
}

#
#  psp.R
#
#  $Revision: 1.104 $ $Date: 2020/04/01 04:36:39 $
#
# Class "psp" of planar line segment patterns
#
#
#################################################
# creator
#################################################
psp <- function(x0, y0, x1, y1, window, marks=NULL,
                check=spatstat.options("checksegments")) {
  stopifnot(is.numeric(x0))
  stopifnot(is.numeric(y0))
  stopifnot(is.numeric(x1))
  stopifnot(is.numeric(y1))
  stopifnot(is.vector(x0))
  stopifnot(is.vector(y0))
  stopifnot(is.vector(x1))
  stopifnot(is.vector(y1))
  stopifnot(length(x0) == length(y0))
  stopifnot(length(x1) == length(y1))
  stopifnot(length(x0) == length(x1))
  ends <- data.frame(x0=x0,y0=y0,x1=x1,y1=y1)
  if(!missing(window))
    verifyclass(window,"owin")
  if(check) {
    ok <- inside.owin(x0,y0, window) & inside.owin(x1,y1,window)
    if((nerr <- sum(!ok)) > 0)
      stop(paste(nerr, ngettext(nerr, "segment does not", "segments do not"),
                 "lie entirely inside the window.\n"), call.=FALSE)
  }
  out <- list(ends=ends,
              window=window,
              n = nrow(ends))

# add marks if any
  if(!is.null(marks)) {
    if(is.matrix(marks))
      marks <- as.data.frame(marks)
    if(is.data.frame(marks)) {
      omf <- "dataframe"
      nmarks <- nrow(marks)
      rownames(marks) <- seq_len(nmarks)
      whinge <- "The number of rows of marks"
    } else {
      omf <- "vector"
      names(marks) <- NULL
      nmarks <- length(marks)
      whinge <- "The length of the marks vector"
    }
    if(nmarks != out$n) stop(paste(whinge, "!= length of x and y.\n"))
    out$marks <- marks
    out$markformat <- omf
  } else {
    out$markformat <- "none"
  }

  class(out) <- c("psp", class(out))
  return(out)
}

######################################################
#  conversion
######################################################

is.psp <- function(x) { inherits(x, "psp") }

as.psp <- function(x, ..., from=NULL, to=NULL) {
  # special case: two point patterns
  if(is.null(from) != is.null(to))
    stop(paste("If one of", sQuote("from"), "and", sQuote("to"),
               "is specified, then both must be specified.\n"))
  if(!is.null(from) && !is.null(to)) {
    verifyclass(from, "ppp")
    verifyclass(to, "ppp")
    if(from$n != to$n)
      stop(paste("The point patterns", sQuote("from"), "and", sQuote("to"),
                 "have different numbers of points.\n"))
    uni <- union.owin(from$window, to$window)
    Y <- do.call(psp,
                 resolve.defaults(list(from$x, from$y, to$x, to$y),
                                  list(...),
                                  list(window=uni)))
    return(Y)
  }
  UseMethod("as.psp")
}

as.psp.psp <- function(x, ..., check=FALSE, fatal=TRUE) {
  if(!verifyclass(x, "psp", fatal=fatal))
    return(NULL)
  ends <- x$ends
  psp(ends$x0, ends$y0, ends$x1, ends$y1, window=x$window,
      marks=x$marks, check=check)
}

as.psp.data.frame <- function(x, ..., window=NULL, marks=NULL,
                              check=spatstat.options("checksegments"),
                              fatal=TRUE) {
  window <- suppressWarnings(as.owin(window,fatal=FALSE))
  if(!is.owin(window)) {
    if(fatal) stop("Cannot interpret \"window\" as an object of class owin.\n")
    return(NULL)
  }

  if(checkfields(x, "marks")) {
    if(is.null(marks)) marks <- x$marks
    else warning(paste("Column named \"marks\" ignored;\n",
                       "argument named \"marks\" has precedence.\n",sep=""))
    x$marks <- NULL
  }

  if(checkfields(x, c("x0", "y0", "x1", "y1"))) {
    out <- psp(x$x0, x$y0, x$x1, x$y1, window=window,
               check=check)
    x <- x[-match(c("x0","y0","x1","y1"),names(x))]
  } else if(checkfields(x, c("xmid", "ymid", "length", "angle"))) {
    rr <- x$length/2
    dx <- cos(x$angle) * rr
    dy <- sin(x$angle) * rr
    bb <- boundingbox(window)
    rmax <- max(rr)
    bigbox <- owin(bb$xrange + c(-1,1) * rmax, bb$yrange + c(-1,1) * rmax)
    pattern <- psp(x$xmid - dx, x$ymid - dy, x$xmid + dx, x$ymid + dy,
                   window=bigbox,check=FALSE)
    out <- pattern[window]
    x <- x[-match(c("xmid","ymid","length","angle"),names(x))]
  } else if(ncol(x) >= 4) {
    out <- psp(x[,1], x[,2], x[,3], x[,4], window=window,
               check=check)
    x <- x[-(1:4)]
  } else {
    ## data not understood
    if(fatal) 
      stop("Unable to interpret x as a line segment pattern.", call.=FALSE)
    return(NULL)
  }

  if(ncol(x) > 0) {
    #' additional columns of mark data in 'x'
    if(is.null(marks)) {
      marks <- x
    } else {
      warning(paste("Additional columns in x were ignored",
                    "because argument 'marks' takes precedence"),
              call.=FALSE)
    }
  }

  if(!is.null(marks)) {
    #' SUPPRESSED: if(identical(ncol(marks), 1L)) marks <- marks[,1L]
    #' assign marks directly to avoid infinite recursion
    out$marks <- marks
    out$markformat <- markformat(marks)
  }

  return(out)
}

as.psp.matrix <- function(x, ..., window=NULL, marks=NULL,
                          check=spatstat.options("checksegments"), fatal=TRUE) {
   x <- as.data.frame(x)
   as.psp(x,...,window=window,marks=marks,check=check,fatal=fatal)
}

as.psp.default <- function(x, ..., window=NULL, marks=NULL,
                           check=spatstat.options("checksegments"), fatal=TRUE) {
  if(checkfields(x,"marks")) {
	if(is.null(marks)) marks <- x$marks
	else warning(paste("Component of \"x\" named \"marks\" ignored;\n",
                             "argument named \"marks\" has precedence.\n",sep=""))
  }
  if(checkfields(x, c("x0", "y0", "x1", "y1")))
    return(psp(x$x0, x$y0, x$x1, x$y1, window=window, marks=marks,
               check=check))
  else if(checkfields(x, c("xmid", "ymid", "length", "angle"))) {
    rr <- x$length/2
    dx <- cos(x$angle) * rr
    dy <- sin(x$angle) * rr
    window <- as.owin(window)
    bb <- boundingbox(window)
    rmax <- max(rr)
    bigbox <- owin(bb$xrange + c(-1,1) * rmax, bb$yrange + c(-1,1) * rmax)
    pattern <- psp(x$x - dx, x$y - dy, x$x + dx, x$y + dy,
                   window=bigbox, marks=marks, check=FALSE)
    clipped <- pattern[window]
    return(clipped)
  }
  else if(fatal)
    stop("Unable to interpret x as a line segment pattern")
  return(NULL)
}

as.psp.owin <- function(x, ..., window=NULL,
                        check=spatstat.options("checksegments"), fatal=TRUE) {
  .Deprecated("edges", package="spatstat")
  edges(x, ..., window=window, check=check)
}


edges <- function(x, ...,
                  window=NULL, check=FALSE) {
  x <- as.owin(x)
  if(is.null(window)) window <- as.rectangle(x)
  x <- as.polygonal(x)
  x0 <- y0 <- x1 <- y1 <- numeric(0)
  bdry <- x$bdry
  for(i in seq_along(bdry)) {
    po <- bdry[[i]]
    ni <- length(po$x)
    nxt <- c(2:ni, 1)
    x0 <- c(x0, po$x)
    y0 <- c(y0, po$y)
    x1 <- c(x1, po$x[nxt])
    y1 <- c(y1, po$y[nxt])
  }
  out <- psp(x0, y0, x1, y1,  window=window, check=check)
  return(out)
}

xypolygon2psp <- function(p, w, check=spatstat.options("checksegments")) {
  verify.xypolygon(p)
  n <- length(p$x)
  nxt <- c(2:n, 1)
  return(psp(p$x, p$y, p$x[nxt], p$y[nxt], window=w, check=check))
}
         

#################

as.data.frame.psp <- function(x, row.names=NULL, ...) {
  df <- as.data.frame(x$ends, row.names=row.names)
  if(is.marked(x))
    df <- cbind(df, if(x$markformat=="dataframe") marks(x)
                    else data.frame(marks=marks(x)))
  return(df)
}

#######  manipulation ##########################

append.psp <- function(A,B) {
  if(is.null(A) && (is.psp(B) || is.null(B))) return(B)
  if(is.null(B) && is.psp(A)) return(A)
  verifyclass(A, "psp")
  verifyclass(B, "psp")
  stopifnot(identical(A$window, B$window))
  marks <- marks(A) %mapp% marks(B)
  ends <- rbind(A$ends, B$ends)
  out  <- as.psp(ends,window=A$window,marks=marks,check=FALSE)
  return(out)
}

rebound.psp <- function(x, rect) {
  verifyclass(x, "psp")
  x$window <- rebound.owin(x$window, rect)
  return(x)
}


#################################################
#  marks
#################################################

is.marked.psp <- function(X, ...) {
  marx <- marks(X, ...)
  return(!is.null(marx))
}

marks.psp <- function(x, ..., dfok = TRUE) {
  # data frames of marks are as of 19/March 2011 implemented for psp
    ma <- x$marks
    if ((is.data.frame(ma) || is.matrix(ma)) && !dfok) 
        stop("Sorry, not implemented when the marks are a data frame.\n")
    return(ma)
}

"marks<-.psp" <- function(x, ..., value) {
  stopifnot(is.psp(x))
  if(is.null(value)) return(unmark(x))
  m <- value
  if(is.hyperframe(m)) 
    stop("Hyperframes of marks are not supported in psp objects.\n")
  if(!(is.vector(m) || is.factor(m) || is.data.frame(m) || is.matrix(m)))
    stop("Incorrect format for marks")

  nseg <- nsegments(x)
  if (!is.data.frame(m) && !is.matrix(m)) {
    ## vector/factor
    if (length(m) == 1) 
      m <- rep.int(m, nseg)
    else if (nseg == 0) 
      m <- rep.int(m, 0)
    else if (length(m) != nseg) 
      stop("Number of marks != number of line segments.\n")
    marx <- m
  } else {
    ## multiple columns
    m <- as.data.frame(m)
    if (ncol(m) == 0) {
      marx <- NULL
    } else if (nrow(m) == nseg) {
      marx <- m
    } else if (nrow(m) == 1 || nseg == 0) {
      marx <- as.data.frame(lapply(as.list(m), rep.int, times=nseg))
    } else stop("Number of rows of data frame != number of line segments")
  }
  Y <- as.psp(x$ends, window = x$window, marks = marx, check = FALSE)
  return(Y)
}

markformat.psp <- function(x) {
  x$markformat %orifnull% markformat(marks(x))
}

unmark.psp <- function(X) {
  X$marks <- NULL
  X$markformat <- "none"
  return(X)
}

#################################################
#  plot and print methods
#################################################

plot.psp <- function(x, ..., main, add=FALSE,
                     show.all=!add, 
                     show.window=show.all,
                     which.marks=1,
                     style=c("colour", "width", "none"),
                     col=NULL,
                     ribbon=show.all, ribsep=0.15, ribwid=0.05, ribn=1024,
                     do.plot=TRUE) {
  if(missing(main) || is.null(main))
    main <- short.deparse(substitute(x))
  verifyclass(x, "psp")
  #'
  n <- nsegments(x)
  marx <- marks(x)
  #'
  style <- match.arg(style)
  use.marks <- !is.null(marx) && (n != 0) && (style != "none")
  #'
  if(use.marks && style == "width") {
    #' plot marks as line width
    #' temporary cheat using plot.linfun
    L <- linnet(endpoints.psp(x), edges=cbind(2*(1:n)-1, 2*(1:n)), sparse=TRUE)
    if(length(dim(marx))) marx <- marx[,which.marks]
    f <- function(x,y,seg,tp, values=marx) { values[seg] }
    g <- linfun(f, L)
    out <- plot(g, style="width", ..., main=main, add=add, col=col,
                show.all=show.all, show.window=show.window, do.plot=do.plot)
    return(invisible(out))
  }
  #' plot marks as colours, if present
  do.ribbon <- identical(ribbon, TRUE) && use.marks
  ##
  ## ....   initialise plot; draw observation window  ......
  owinpars <- setdiff(graphicsPars("owin"), "col")
  if(!do.ribbon) {
    ## window of x only
    bb.all <- as.rectangle(as.owin(x))
    if(do.plot && (!add || show.window))
      do.call.plotfun(plot.owin, 
                      resolve.defaults(list(x=x$window,
		                            main=if(show.all) main else "",
                                            add=add,
                                            type = if(show.window) "w" else "n",
                                            show.all=show.all),
                                       list(...)),
                      extrargs=owinpars)
  } else {
    ## enlarged window with room for colour ribbon
    ## x at left, ribbon at right
    bb <- as.rectangle(as.owin(x))
    xwidth <- diff(bb$xrange)
    xheight <- diff(bb$yrange)
    xsize <- max(xwidth, xheight)
    bb.rib <- owin(bb$xrange[2] + c(ribsep, ribsep+ribwid) * xsize,
                   bb$yrange)
    bb.all <- boundingbox(bb.rib, bb)
    if(do.plot) {
      pt <- prepareTitle(main)
      ## establish coordinate system
      if(!add)
      do.call.plotfun(plot.owin,
                      resolve.defaults(list(x=bb.all,
                                            type="n",
                                            main=pt$blank),
                                       list(...)),
                      extrargs=owinpars)
      ## now plot window of x
      ## with title centred on this window
      if(show.window) {
        do.call.plotfun(plot.owin, 
                        resolve.defaults(list(x=x$window,
                                              add=TRUE,
                                              main=main,
                                              show.all=TRUE),
                                         list(...)),
                        extrargs=owinpars)
        ## title done. 
        main <- ""
      }
    }
  }

  # plot segments
  if(n == 0) {
    result <- symbolmap()
    attr(result, "bbox") <- bb.all
    return(invisible(result))
  }
  
  ## determine colours if any
  colmap <- NULL
  if(use.marks) {
    ## use colours
    marx <- as.data.frame(marx)[, which.marks]
    if(is.character(marx) || length(unique(marx)) == 1)
      marx <- factor(marx)
    if(is.null(col)) {
      ## no colour info: use default colour palette
      nc <- if(is.factor(marx)) {
              length(levels(marx))
            } else {
              min(256, length(unique(marx)))
            }
      colfun <- spatstat.options("image.colfun")
      col <- colfun(nc)
    }
    ## determine colour map
    if(inherits(col, "colourmap")) {
      colmap <- colourmap
    } else if(is.colour(col)) {
      ## colour values given; create colour map
      if(is.factor(marx)) {
        lev <- levels(marx)
        colmap <- colourmap(col=col, inputs=factor(lev))
      } else {
        if(!all(is.finite(marx)))
          warning("Some mark values are infinite or NaN or NA")
        colmap <- colourmap(col=col, range=range(marx, finite=TRUE))
      }
    } else stop("Format of argument 'col' is not recognised")
    #' map the mark values to colours
    col <- colmap(marx)
  }
  ## convert to greyscale?
  if(spatstat.options("monochrome")) {
    col <- to.grey(col)
    colmap <- to.grey(colmap)
  }
  if(do.plot) {
    ## plot segments
    do.call.plotfun(segments,
                    resolve.defaults(as.list(x$ends),
                                     list(...),
                                     list(col=col),
                                     .MatchNull=FALSE, .StripNull=TRUE),
                    extrargs=names(par()))
    ## plot ribbon
    if(do.ribbon) 
      plot(colmap, vertical=TRUE, add=TRUE,
           xlim=bb.rib$xrange, ylim=bb.rib$yrange)
  }
  
  # return colour map
  result <- colmap %orifnull% colourmap()
  attr(result, "bbox") <- bb.all
  return(invisible(result))
}

print.psp <- function(x, ...) {
  verifyclass(x, "psp")
  n <- x$n
  ism <- is.marked(x, dfok = TRUE)
  splat(if(ism) "marked" else NULL,
        "planar line segment pattern:",
        n, ngettext(n, "line segment", "line segments"))
  if(ism) {
    mks <- marks(x, dfok = TRUE)
    if(is.data.frame(mks)) {
      splat("Mark variables: ",
            paste(names(mks), collapse = ", "))
    } else {
      if(is.factor(mks)) {
        splat("multitype, with levels =",
              paste(levels(mks), collapse = "\t"))
      } else {
        splat("marks are",
              if(is.numeric(mks)) "numeric," else NULL,
              "of type", sQuote(typeof(mks)))
      }
    }
  }
  print(x$window)
  return(invisible(NULL))
}

unitname.psp <- function(x) {
  return(unitname(x$window))
}

"unitname<-.psp" <- function(x, value) {
  w <- x$window
  unitname(w) <- value
  x$window <- w
  return(x)
}

####################################################
#    summary information
####################################################

endpoints.psp <- function(x, which="both") {
  verifyclass(x, "psp")
  ends <- x$ends
  n <- x$n
  switch(which,
         both={
           first <- second <- rep.int(TRUE, n)
         },
         first={
           first <- rep.int(TRUE, n)
           second <- rep.int(FALSE, n)
         },
         second={
           first <- rep.int(FALSE, n)
           second <- rep.int(TRUE, n)
         },
         left={
           first <- (ends$x0 < ends$x1)
           second <- !first
         },
         right={
           first <- (ends$x0 > ends$x1)
           second <- !first
         },
         lower={
           first <- (ends$y0 < ends$y1)
           second <- !first
         },
         upper={
           first <- (ends$y0 > ends$y1)
           second <- !first
         },
         stop(paste("Unrecognised option: which=", sQuote(which)))
         )
  ok <- rbind(first, second)
  xmat <- rbind(ends$x0, ends$x1)
  ymat <- rbind(ends$y0, ends$y1)
  idmat <- col(ok)
  xx <- as.vector(xmat[ok])
  yy <- as.vector(ymat[ok])
  id <- as.vector(idmat[ok])
  result <- ppp(xx, yy, window=x$window, check=FALSE)
  attr(result, "id") <- id
  return(result)
}

midpoints.psp <- function(x) {
  verifyclass(x, "psp")
  xm <- eval(expression((x0+x1)/2), envir=x$ends)
  ym <- eval(expression((y0+y1)/2), envir=x$ends)
  win <- x$window
  ok <- inside.owin(xm, ym, win)
  if(any(!ok)) {
    warning(paste("Some segment midpoints lie outside the original window;",
                  "window replaced by bounding box"))
    win <- boundingbox(win)
  }
  ppp(x=xm, y=ym, window=win, check=FALSE)
}

lengths_psp <- lengths.psp <- function(x, squared=FALSE) {
  verifyclass(x, "psp")
  lengths2 <- eval(expression((x1-x0)^2 + (y1-y0)^2), envir=x$ends)
  return(if(squared) lengths2 else sqrt(lengths2))
}

angles.psp <- function(x, directed=FALSE) {
  verifyclass(x, "psp")
  a <- eval(expression(atan2(y1-y0, x1-x0)), envir=x$ends)
  if(!directed) 
    a <- a %% pi
  return(a)
}

summary.psp <- function(object, ...) {
  verifyclass(object, "psp")
  len <- lengths_psp(object)
  out <- list(n = object$n,
              len = summary(len),
              totlen = sum(len),
              ang= summary(angles.psp(object)),
              w = summary.owin(object$window),
              marks=if(is.null(object$marks)) NULL else summary(object$marks),
              unitinfo=summary(unitname(object)))
  class(out) <- c("summary.psp", class(out))
  return(out)
}

print.summary.psp <- function(x, ...) {
  cat(paste(x$n, "line segments\n"))
  cat("Lengths:\n")
  print(x$len)
  unitblurb <- paste(x$unitinfo$plural, x$unitinfo$explain)
  cat(paste("Total length:", x$totlen, unitblurb, "\n"))
  cat(paste("Length per unit area:", x$totlen/x$w$area, "\n"))
  cat("Angles (radians):\n")
  print(x$ang)
  print(x$w)
  if(!is.null(x$marks)) {
    cat("Marks:\n")
    print(x$marks)
  }
  return(invisible(NULL))
}

extrapolate.psp <- function(x, ...) {
  verifyclass(x, "psp")
  theta <- (angles.psp(x) + pi/2) %% (2*pi)
  p <- with(x$ends, x1*cos(theta) + y1 * sin(theta))
  result <- infline(p=p, theta=theta)
  return(result)
}


########################################################
#  subsets
########################################################

"[.psp" <-
  function(x, i, j, drop, ..., fragments=TRUE) {

    verifyclass(x, "psp")
    
    if(missing(i) && missing(j))
      return(x)
        
    if(!missing(i)) {
      style <- if(inherits(i, "owin")) "window" else "index"
      switch(style,
             window={
               x <- clip.psp(x, window=i, check=FALSE, fragments=fragments)
             },
             index={
               enz <- x$ends[i, ]
               win <- x$window
               marx <- marksubset(x$marks, i, markformat(x))
               x <- with(enz, psp(x0, y0, x1, y1, window=win, marks=marx,
                                  check=FALSE))
             })
    }

    if(!missing(j))
      x <- x[j] # invokes code above
    
    return(x)
 }
  


####################################################
# affine transformations
####################################################

affine.psp <- function(X,  mat=diag(c(1,1)), vec=c(0,0), ...) {
  verifyclass(X, "psp")
  W <- affine.owin(X$window, mat=mat, vec=vec, ...)
  E <- X$ends
  ends0 <- affinexy(list(x=E$x0,y=E$y0), mat=mat, vec=vec)
  ends1 <- affinexy(list(x=E$x1,y=E$y1), mat=mat, vec=vec)
  psp(ends0$x, ends0$y, ends1$x, ends1$y, window=W, marks=marks(X, dfok=TRUE),
      check=FALSE)
}

shift.psp <- function(X, vec=c(0,0), ..., origin=NULL) {
  verifyclass(X, "psp")
  W <- Window(X)
  if(!is.null(origin)) {
    if(!missing(vec))
      warning("argument vec ignored; argument origin has precedence")
    locn <- interpretAsOrigin(origin, W)
    vec <- -locn
  }
  # perform shift
  W <- shift.owin(W, vec=vec, ...)
  E <- X$ends
  ends0 <- shiftxy(list(x=E$x0,y=E$y0), vec=vec, ...)
  ends1 <- shiftxy(list(x=E$x1,y=E$y1), vec=vec, ...)
  Y <- psp(ends0$x, ends0$y, ends1$x, ends1$y,
           window=W, marks=marks(X, dfok=TRUE),
           check=FALSE)
  # tack on shift vector
  attr(Y, "lastshift") <- vec
  return(Y)
}

rotate.psp <- function(X, angle=pi/2, ..., centre=NULL) {
  verifyclass(X, "psp")
  if(!is.null(centre)) {
    X <- shift(X, origin=centre)
    negorigin <- getlastshift(X)
  } else negorigin <- NULL
  W <- rotate.owin(X$window, angle=angle, ...)
  E <- X$ends
  ends0 <- rotxy(list(x=E$x0,y=E$y0), angle=angle)
  ends1 <- rotxy(list(x=E$x1,y=E$y1), angle=angle)
  Y <- psp(ends0$x, ends0$y, ends1$x, ends1$y,
           window=W, marks=marks(X, dfok=TRUE),
           check=FALSE)
  if(!is.null(negorigin))
    Y <- shift(Y, -negorigin)
  return(Y)
}

is.empty.psp <- function(x) { return(x$n == 0) } 

identify.psp <- function(x, ..., labels=seq_len(nsegments(x)),
                         n=nsegments(x), plot=TRUE) {
  Y <- x
  W <- as.owin(Y)
  mids <- midpoints.psp(Y)
  poz <- c(1, 2,4, 3)[(floor(angles.psp(Y)/(pi/4)) %% 4) + 1L]
  if(!(is.numeric(n) && (length(n) == 1) && (n %% 1 == 0) && (n >= 0)))
    stop("n should be a single integer")
  out <- integer(0)
  while(length(out) < n) {
    xy <- spatstatLocator(1)
    # check for interrupt exit
    if(length(xy$x) == 0)
      return(out)
    # find nearest segment
    X <- ppp(xy$x, xy$y, window=W)
    ident <- project2segment(X, Y)$mapXY
    # add to list
    if(ident %in% out) {
      cat(paste("Segment", ident, "already selected\n"))
    } else {
      if(plot) {
        # Display
        mi <- mids[ident]
        li <- labels[ident]
        po <- poz[ident]
        do.call.matched(graphics::text.default,
                        resolve.defaults(list(x=mi$x, y=mi$y, labels=li),
                                         list(...),
                                         list(pos=po)))
      }
      out <- c(out, ident)
    }
  }
  # exit if max n reached
  return(out)
}

nsegments <- function(x) {
	UseMethod("nsegments")
}

nobjects.psp <- nsegments.psp <- function(x) {
   x$n
}

as.ppp.psp <- function (X, ..., fatal=TRUE) 
{
  Y <- endpoints.psp(X, which="both")
  m  <- marks(X)
  marks(Y) <- markappend(m, m)
  return(Y)
}

domain.psp <- Window.psp <- function(X, ...) { as.owin(X) }

"Window<-.psp" <- function(X, ..., value) {
  verifyclass(value, "owin")
  X[value]
}

edit.psp <- function(name, ...) {
  x <- name
  y <- edit(as.data.frame(x), ...)
  xnew <- as.psp(y, window=Window(x))
  return(xnew)
}

text.psp <- function(x, ...) {
  mids <- midpoints.psp(x)
  poz <- c(1, 2,4, 3)[(floor(angles.psp(x)/(pi/4)) %% 4) + 1L]
  do.call.matched(graphics::text.default,
                  resolve.defaults(list(x=mids$x, y=mids$y),
                                   list(...),
                                   list(pos=poz),
                                   .StripNull=TRUE))
  return(invisible(NULL))
}

intensity.psp <- function(X, ..., weights=NULL) {
  len <- lengths_psp(X)
  a <- area(Window(X))
  if(is.null(weights)) {
    ## unweighted case - for efficiency
    if(is.multitype(X)) {
      mks <- marks(X)
      answer <- tapply(len, mks, sum)/a
      answer[is.na(answer)] <- 0
      names(answer) <- levels(mks)
    } else answer <- sum(len)/a
    return(answer)
  }
  ## weighted case 
  if(is.numeric(weights)) {
    check.nvector(weights, nsegments(X), things="segments")
  } else if(is.expression(weights)) {
    # evaluate expression in data frame of coordinates and marks
    df <- as.data.frame(X)
    pf <- parent.frame()
    eval.weights <- try(eval(weights, envir=df, enclos=pf))
    if(inherits(eval.weights, "try-error"))
      stop("Unable to evaluate expression for weights", call.=FALSE)
    if(!check.nvector(eval.weights, nsegments(X), fatal=FALSE, warn=TRUE))
      stop("Result of evaluating the expression for weights has wrong format")
    weights <- eval.weights
  } else stop("Unrecognised format for argument 'weights'")
  ##
  if(is.multitype(X)) {
    mks <- marks(X)
    answer <- as.vector(tapply(weights * len, mks, sum))/a
    answer[is.na(answer)] <- 0
    names(answer) <- levels(mks)
  } else {
    answer <- sum(weights)/a
  }
  return(answer)
}

#
# simplepanel.R
#
#  A simple, robust point & click interface
#     used in rmh visual debugger.
#
#  $Revision: 1.14 $  $Date: 2016/04/25 02:34:40 $
#

simplepanel <- function(title, B, boxes, clicks, redraws=NULL, exit=NULL, env) {
  stopifnot(is.rectangle(B))
  stopifnot(is.list(boxes))
  if(!all(unlist(lapply(boxes, is.rectangle))))
    stop("some of the boxes are not rectangles")
  if(!all(unlist(lapply(boxes, is.subset.owin, B=B))))
    stop("Some boxes do not lie inside the bounding box B")
  stopifnot(is.list(clicks) && length(clicks) == length(boxes))
  if(!all(unlist(lapply(clicks, is.function))))
    stop("clicks must be a list of functions")
  if(is.null(redraws)) {
    redraws <- rep.int(list(dflt.redraw), length(boxes))
  } else {
    stopifnot(is.list(redraws) && length(redraws) == length(boxes))
    if(any(isnul <- unlist(lapply(redraws, is.null))))
      redraws[isnul] <- rep.int(list(dflt.redraw), sum(isnul))
    if(!all(unlist(lapply(redraws, is.function))))
      stop("redraws must be a list of functions")
  }
  if(is.null(exit)) {
    exit <- function(...) { NULL}
  } else stopifnot(is.function(exit))
  stopifnot(is.environment(env))
  n <- length(boxes)
  bnames <- names(boxes) %orifnull% rep("", n)
  cnames <- names(clicks) %orifnull% rep("", n)
  dnames <- paste("Button", seq_len(n))
  nama <- ifelse(nzchar(bnames), bnames, ifelse(nzchar(cnames), cnames, dnames))
  out <- list(title=title, B=B,
              nama=nama, boxes=boxes, clicks=clicks, redraws=redraws,
              exit=exit, env=env)
  class(out) <- c("simplepanel", class(out))
  return(out)
}

grow.simplepanel <- function(P, side=c("right","left","top","bottom"),
                             len=NULL,
                             new.clicks, new.redraws=NULL, ..., aspect) {
  verifyclass(P, "simplepanel")
  side <- match.arg(side)
  stopifnot(is.list(new.clicks))
  if(!all(unlist(lapply(new.clicks, is.function))))
    stop("new.clicks must be a list of functions")
  if(is.null(new.redraws)) {
    new.redraws <- rep.int(list(dflt.redraw), length(new.clicks))
  } else {
    stopifnot(is.list(new.redraws) && length(new.redraws) == length(new.clicks))
    if(any(isnul <- sapply(new.redraws, is.null)))
      new.redraws[isnul] <- rep.int(list(dflt.redraw), sum(isnul))
    if(!all(unlist(lapply(new.redraws, is.function))))
      stop("new.redraws must be a list of functions")
  }
  if(missing(aspect) || is.null(aspect)) {
    # determine aspect ratio from length of longest text string
    n <- length(new.clicks)
    nama <- names(new.clicks)
    if(sum(nzchar(nama)) != n)
      nama <- names(new.redraws)
    if(sum(nzchar(nama)) != n)
      nama <- paste("Box", seq_len(n))
    aspect <- 3/max(4, nchar(nama))
  }
  B <- P$B
  n <- length(new.clicks)
  switch(side,
         right={
           new.width <- if(!is.null(len)) len else sidelengths(B)[1]/2
           extraspace <- owin(B$xrange[2] + c(0, new.width), B$yrange)
           new.boxes <- layout.boxes(extraspace, n, ..., aspect=aspect)
         },
         left={
           new.width <- if(!is.null(len)) len else sidelengths(B)[1]/2
           extraspace <- owin(B$xrange[1] - c(new.width, 0), B$yrange)
           new.boxes <- layout.boxes(extraspace, n, ..., aspect=aspect)
         },
         top={
           new.height <- if(!is.null(len)) len else sidelengths(B)[2]/2
           extraspace <- owin(B$xrange, B$yrange[2] + c(0, new.height))
           new.boxes <- layout.boxes(extraspace, n, ..., aspect=aspect,
                                     horizontal=TRUE)
         },
         bottom={
           new.height <- if(!is.null(len)) len else sidelengths(B)[2]/2
           extraspace <- owin(B$xrange, B$yrange[1] - c(new.height, 0))
           new.boxes <- layout.boxes(extraspace, n, ..., aspect=aspect,
                                     horizontal=TRUE)
         })
  with(P, simplepanel(title,
                      boundingbox(B, extraspace),
                      append(boxes, new.boxes),
                      append(clicks, new.clicks),
                      append(redraws, new.redraws),
                      exit, env))
}

                             
redraw.simplepanel <- function(P, verbose=FALSE) {
  verifyclass(P, "simplepanel")
  if(verbose)
    cat("Redrawing entire panel\n")
  with(P, {
#    ntitle <- sum(nzchar(title))
    plot(B, type="n", main=title)
    for(j in seq_along(nama)) 
      (redraws[[j]])(boxes[[j]], nama[j], env)
  })
  invisible(NULL)
}

clear.simplepanel <- function(P) {
  verifyclass(P, "simplepanel")
  plot(P$B, main="")
  invisible(NULL)
}
                             
run.simplepanel <- function(P, popup=TRUE, verbose=FALSE) {
  verifyclass(P, "simplepanel")
  if(popup) dev.new()
  ntitle <- sum(nzchar(P$title))
  opa <- par(mar=c(0,0,ntitle+0.2,0),ask=FALSE)
  with(P, {
    # interaction loop
    more <- TRUE
    while(more) {
      redraw.simplepanel(P, verbose=verbose)
      xy <- spatstatLocator(1)
      if(is.null(xy)) {
        if(verbose) cat("No (x,y) coordinates\n")
        break
      }
      found <- FALSE
      for(j in seq_along(boxes)) {
        if(inside.owin(xy$x, xy$y, boxes[[j]])) {
          found <- TRUE
          if(verbose) cat(paste("Caught click on", sQuote(nama[j]), "\n"))
          more <- (clicks[[j]])(env, xy)
          if(!is.logical(more) || length(more) != 1) {
            warning(paste("Click function for",
                          sQuote(nama[j]),
                          "did not return TRUE/FALSE"))
            more <- FALSE
          }
          if(verbose) cat(if(more) "Continuing\n" else "Terminating\n")
          break
        }
      }
      if(verbose && !found)
        cat(paste("Coordinates", paren(paste(xy, collapse=",")),
                  "not matched to any box\n"))
    }
  })
  if(verbose)
    cat("Calling exit function\n")

  rslt <- with(P, exit(env))
  
  # revert to original graphics parameters
  par(opa)
  # close popup window?
  if(popup) dev.off()
  
  # return value of 'exit' function
  return(rslt)
}

layout.boxes <- function(B, n, horizontal=FALSE, aspect=0.5, usefrac=0.9){
  # make n boxes in B
  stopifnot(is.rectangle(B))
  stopifnot(n > 0)
  width <- sidelengths(B)[1]
  height <- sidelengths(B)[2]
  if(!horizontal) {
    heightshare <- height/n
    useheight <- min(width * aspect, heightshare * usefrac)
    usewidth <-  min(useheight /aspect, width * usefrac)
    lostwidth <- width - usewidth
    lostheightshare <- heightshare - useheight
    template <- owin(c(0, usewidth), c(0, useheight))
    boxes <- list()
    boxes[[1]] <- shift(template,
                        c(B$xrange[1]+lostwidth/2,
                          B$yrange[1] + lostheightshare/2))
    if(n > 1) 
      for(j in 2:n) 
        boxes[[j]] <- shift(boxes[[j-1]], c(0, heightshare))
  } else {
    boxes <- layout.boxes(flipxy(B), n,
                            horizontal=FALSE, aspect=1/aspect, usefrac=usefrac)
    boxes <-  lapply(boxes, flipxy)
  }
  return(boxes)
}

# default redraw function for control buttons

dflt.redraw <- function(button, name, env) {
  plot(button, add=TRUE, border="pink")
  text(centroid.owin(button), labels=name)
}

print.simplepanel <- function(x, ...) {
  nama <- x$nama
  cat("simplepanel object\n")
  cat(paste("\tTitle:", sQuote(x$title), "\n"))
  cat("\tPanel names:")
  for(i in seq_along(nama)) {
    if(i %% 6 == 1) cat("\n\t")
    cat(paste0(sQuote(nama[i]), "  "))
  }
  cat("\n")
  return(invisible(NULL))
}

#
#	affine.R
#
#	$Revision: 1.53 $	$Date: 2019/02/22 10:36:51 $
#

affinexy <- function(X, mat=diag(c(1,1)), vec=c(0,0), invert=FALSE) {
  if(length(X$x) == 0 && length(X$y) == 0)
    return(list(x=numeric(0),y=numeric(0)))
  if(invert) {
    mat <- invmat <- solve(mat)
    vec <- - as.numeric(invmat %*% vec)
  }
  # Y = M X + V
  ans <- mat %*% rbind(X$x, X$y) + matrix(vec, nrow=2L, ncol=length(X$x))
  return(list(x = ans[1L,],
              y = ans[2L,]))
}

affinexypolygon <- function(p, mat=diag(c(1,1)), vec=c(0,0),
                             detmat=det(mat)) {
  # transform (x,y)
  p[c("x","y")] <- affinexy(p, mat=mat, vec=vec)
  # transform area
  if(!is.null(p$area))
    p$area <- p$area * detmat
  # if map has negative sign, cyclic order was reversed; correct it
  if(detmat < 0)
    p <- reverse.xypolygon(p, adjust=TRUE)
  return(p)
}
       
"affine" <- function(X, ...) {
  UseMethod("affine")
}

"affine.owin" <- function(X,  mat=diag(c(1,1)), vec=c(0,0), ...,
                          rescue=TRUE) {
  verifyclass(X, "owin")
  vec <- as2vector(vec)
  if(!is.matrix(mat) || any(dim(mat) != c(2,2)))
    stop(paste(sQuote("mat"), "should be a 2 x 2 matrix"))
  diagonalmatrix <- all(mat == diag(diag(mat)))
  scaletransform <- diagonalmatrix && (length(unique(diag(mat))) == 1)
  newunits <- if(scaletransform) unitname(X) else as.unitname(NULL)
  #
  switch(X$type,
         rectangle={
           if(diagonalmatrix) {
             # result is a rectangle
             Y <- owin(range(mat[1L,1L] * X$xrange + vec[1L]),
                       range(mat[2L,2L] * X$yrange + vec[2L]))
             unitname(Y) <- newunits
             return(Y)
           } else {
             # convert rectangle to polygon
             P <- as.polygonal(X)
             # call polygonal case
             return(affine.owin(P, mat, vec, rescue=rescue))
           }
         },
         polygonal={
           # Transform the polygonal boundaries
           bdry <- lapply(X$bdry, affinexypolygon, mat=mat, vec=vec,
                          detmat=det(mat))
           # Compile result
           W <- owin(poly=bdry, check=FALSE, unitname=newunits)
           # Result might be a rectangle: if so, convert to rectangle type
           if(rescue)
             W <- rescue.rectangle(W)
           return(W)
         },
         mask={
           # binary mask
           if(sqrt(abs(det(mat))) < .Machine$double.eps)
             stop("Matrix of linear transformation is singular")
           newframe <- boundingbox(affinexy(corners(X), mat, vec))
           W <- if(length(list(...)) > 0) as.mask(newframe, ...) else 
                   as.mask(newframe, eps=with(X, min(xstep, ystep)))
           pixelxy <- rasterxy.mask(W)
           xybefore <- affinexy(pixelxy, mat, vec, invert=TRUE)
           W$m[] <- with(xybefore, inside.owin(x, y, X))
           W <- intersect.owin(W, boundingbox(W))
           if(rescue)
             W <- rescue.rectangle(W)
           return(W)
         },
         stop("Unrecognised window type")
         )
}

"affine.ppp" <- function(X, mat=diag(c(1,1)), vec=c(0,0), ...) {
  verifyclass(X, "ppp")
  vec <- as2vector(vec)
  r <- affinexy(X, mat, vec)
  w <- affine.owin(X$window, mat, vec, ...)
  return(ppp(r$x, r$y, window=w, marks=marks(X, dfok=TRUE), check=FALSE))
}

"affine.im" <- function(X,  mat=diag(c(1,1)), vec=c(0,0), ...) {
  verifyclass(X, "im")
  vec <- as2vector(vec)
  if(!is.matrix(mat) || any(dim(mat) != c(2,2)))
    stop(paste(sQuote("mat"), "should be a 2 x 2 matrix"))
  # Inspect the determinant
  detmat <- det(mat)
  if(sqrt(abs(detmat)) < .Machine$double.eps)
    stop("Matrix of linear transformation is singular")
  #
  diagonalmatrix <- all(mat == diag(diag(mat)))
  scaletransform <- diagonalmatrix && (length(unique(diag(mat))) == 1L)
  newunits <- if(scaletransform) unitname(X) else as.unitname(NULL)
  newpixels <- (length(list(...)) > 0)
  #
  if(diagonalmatrix && !newpixels) {
    # diagonal matrix: apply map to row and column locations
    v      <- X$v
    d      <- X$dim
    newbox <- affine(as.rectangle(X), mat=mat, vec=vec)
    xscale <- diag(mat)[1L]
    yscale <- diag(mat)[2L]
    xcol <- xscale * X$xcol + vec[1L]
    yrow <- yscale * X$yrow + vec[2L]
    if(xscale < 0) {
      # x scale is negative
      xcol <- rev(xcol)
      v <- v[, (d[2L]:1)]
    }
    if(yscale < 0) {
      # y scale is negative
      yrow <- rev(yrow)
      v <- v[(d[1L]:1), ]
    }
    Y <- im(v, xcol=xcol, yrow=yrow,
            xrange=newbox$xrange, yrange=newbox$yrange,
            unitname=newunits)
  } else {
    # general case
    # create box containing transformed image
    newframe <- boundingbox(affinexy(corners(X), mat, vec))
    W <- if(length(list(...)) > 0) as.mask(newframe, ...) else 
    as.mask(newframe, eps=with(X, min(xstep, ystep)))
    unitname(W) <- newunits
    # raster for transformed image
    naval <- switch(X$type,
                    factor= , 
                    integer = NA_integer_,
                    logical = as.logical(NA_integer_),
                    real = NA_real_,
                    complex = NA_complex_, 
                    character = NA_character_,
                    NA)
    Y <- as.im(W, value=naval)
    # preimages of pixels of transformed image
    xx <- as.vector(rasterx.im(Y))
    yy <- as.vector(rastery.im(Y))
    pre <- affinexy(list(x=xx, y=yy), mat, vec, invert=TRUE)
    # sample original image
    if(X$type != "factor") {
      Y$v[] <- lookup.im(X, pre$x, pre$y, naok=TRUE)
    } else {
      lab <- levels(X)
      lev <- seq_along(lab)
      Y$v[] <- lookup.im(eval.im(as.integer(X)), pre$x, pre$y, naok=TRUE)
      Y <- eval.im(factor(Y, levels=lev, labels=lab))
    }
  }
  return(Y)
}


### ---------------------- reflect ----------------------------------

reflect <- function(X) {
  UseMethod("reflect")
}

reflect.default <- function(X) { affine(X, mat=diag(c(-1,-1))) }

reflect.im <- function(X) {
  stopifnot(is.im(X))
  out <- with(X,
              list(v      = v[dim[1L]:1, dim[2L]:1],
                   dim    = dim,
                   xrange = rev(-xrange),
                   yrange = rev(-yrange),
                   xstep  = xstep,
                   ystep  = ystep,
                   xcol   = rev(-xcol),
                   yrow   = rev(-yrow),
                   type   = type,
                   units  = units))
  class(out) <- "im"
  return(out)
}

### ---------------------- shift ----------------------------------

"shift" <- function(X, ...) {
  UseMethod("shift")
}

shiftxy <- function(X, vec=c(0,0)) {
  if(is.null(vec)) {
    warning("Null displacement vector; treated as zero")
    return(X)
  }
  list(x = X$x + vec[1L],
       y = X$y + vec[2L])
}

shiftxypolygon <- function(p, vec=c(0,0)) {
  # transform (x,y), retaining other data
  p[c("x","y")] <- shiftxy(p, vec=vec)
  return(p)
}

shift.owin <- function(X,  vec=c(0,0), ..., origin=NULL) {
  verifyclass(X, "owin")
  if(!is.null(origin)) {
    if(!missing(vec))
      warning("argument vec ignored; argument origin has precedence")
    locn <- interpretAsOrigin(origin, X)
    vec <- -locn
  }
  vec <- as2vector(vec)
  # Shift the bounding box
  X$xrange <- X$xrange + vec[1L]
  X$yrange <- X$yrange + vec[2L]
  switch(X$type,
         rectangle={
         },
         polygonal={
           # Shift the polygonal boundaries
           X$bdry <- lapply(X$bdry, shiftxypolygon, vec=vec)
         },
         mask={
           # Shift the pixel coordinates
           X$xcol <- X$xcol + vec[1L]
           X$yrow <- X$yrow + vec[2L]
           # That's all --- the mask entries are unchanged
         },
         stop("Unrecognised window type")
         )
  # tack on shift vector
  attr(X, "lastshift") <- vec
  # units are unchanged
  return(X)
}

shift.ppp <- function(X, vec=c(0,0), ..., origin=NULL) {
  verifyclass(X, "ppp")
  if(!is.null(origin)) {
    if(!missing(vec))
      warning("argument vec ignored; argument origin has precedence")
    locn <- interpretAsOrigin(origin, Window(X))
    vec <- -locn
  }
  vec <- as2vector(vec)
  # perform shift
  r <- shiftxy(X, vec)
  w <- shift.owin(X$window, vec)
  Y <- ppp(r$x, r$y, window=w, marks=marks(X, dfok=TRUE), check=FALSE)
  # tack on shift vector
  attr(Y, "lastshift") <- vec
  return(Y)
}

getlastshift <- function(X) {
  v <- attr(X, "lastshift")
  if(is.null(v))
    stop(paste("Internal error: shifted object of class",
               sQuote(as.character(class(X))[1L]),
               "does not have \"lastshift\" attribute"),
         call.=FALSE)
  if(!(is.numeric(v) && length(v) == 2L))
    stop("Internal error: \"lastshift\" attribute is not a vector",
         call.=FALSE)
  return(v)
}

putlastshift <- function(X, vec) {
  attr(X, "lastshift") <- vec
  return(X)
}

interpretAsOrigin <- function(x, W) {
  if(is.character(x)) {
    x <- paste(x, collapse="")
    x <- match.arg(x,
                   c("centroid", "midpoint",
                     "left", "right", "top", "bottom",
                     "bottomleft", "bottomright",
                     "topleft", "topright"))
    W <- as.owin(W)
    xr <- W$xrange
    yr <- W$yrange
    x <- switch(x,
                centroid    = { unlist(centroid.owin(W)) },
                midpoint    = { c(mean(xr), mean(yr)) },
                left        = { c(xr[1L],   mean(yr)) },
                right       = { c(xr[2L],   mean(yr)) },
                top         = { c(mean(xr), yr[2L]) },
                bottom      = { c(mean(xr), yr[1L]) },
                bottomleft  = { c(xr[1L],   yr[1L]) },
                bottomright = { c(xr[2L],   yr[1L]) },
                topleft     = { c(xr[1L],   yr[2L]) },
                topright    = { c(xr[2L],   yr[2L]) },
                stop(paste("Unrecognised option",sQuote(x)),
                     call.=FALSE))
  } 
  return(as2vector(x))
}

### ---------------------- scalar dilation ---------------------------------

scalardilate <- function(X, f, ...) {
  UseMethod("scalardilate")
}

scalardilate.default <- function(X, f, ...) {
  trap.extra.arguments(..., .Context="In scalardilate(X,f)")
  check.1.real(f, "In scalardilate(X,f)")
  stopifnot(is.finite(f) && f > 0)
  Y <- affine(X, mat=diag(c(f,f)))
  return(Y)
}

scalardilate.im <- scalardilate.owin <- scalardilate.psp <- scalardilate.ppp <-
  function(X, f, ..., origin=NULL) {
  trap.extra.arguments(..., .Context="In scalardilate(X,f)")
  check.1.real(f, "In scalardilate(X,f)")
  stopifnot(is.finite(f) && f > 0)
  if(!is.null(origin)) {
    X <- shift(X, origin=origin)
    negorig <- getlastshift(X)
  } else negorig <- c(0,0)
  Y <- affine(X, mat=diag(c(f, f)), vec = -negorig)
  return(Y)
}
  

##
##     quasirandom.R
##
##  Quasi-random sequence generators
##
##  $Revision: 1.6 $   $Date: 2017/06/05 10:31:58 $
##

vdCorput <- function(n, base) {
  stopifnot(is.prime(base))
  z <- .C("Corput",
          base=as.integer(base),
          n=as.integer(n),
          result=as.double(numeric(n)),
          PACKAGE = "spatstat")
  return(z$result)
}

Halton <- function(n, bases=c(2,3), raw=FALSE, simplify=TRUE) {
  d <- length(bases)
  if(d==2 && !raw && simplify)
    return(ppp(vdCorput(n, bases[1]),
               vdCorput(n, bases[2]),
               window=owin(), check=FALSE))
  z <- matrix(, nrow=n, ncol=d)
  for(j in 1:d)
    z[,j] <- vdCorput(n, bases[j])
  if(raw || d < 2) return(z)
  b <- do.call(boxx, rep(list(c(0,1)), d))
  return(ppx(z, b, simplify=simplify))
}

Hammersley <- function(n, bases=2, raw=FALSE, simplify=TRUE) {
  d <- length(bases) + 1
  z <- cbind(Halton(n, bases, raw=TRUE), (1:n)/n)
  dimnames(z) <- NULL
  if(raw || d < 2) return(z)
  b <- do.call(boxx, rep(list(c(0,1)), d))
  return(ppx(z, b, simplify=simplify))
}

rQuasi <- function(n, W, type=c("Halton", "Hammersley"), ...) {
  R <- as.rectangle(W)
  type <- match.arg(type)
  X <- switch(type,
              Halton=Halton(n, ...),
              Hammersley=Hammersley(n, ...))
  Y <- ppp(R$xrange[1] + diff(R$xrange) * X$x,
           R$yrange[1] + diff(R$yrange) * X$y,
           window=R, check=FALSE)
  if(!is.rectangle(W))
    Y <- Y[W]
  return(Y)
}

#
#  dist2dpath.R
#
#   $Revision: 1.10 $    $Date: 2017/06/05 10:31:58 $
#
#       dist2dpath    compute shortest path distances
#

dist2dpath <- function(dist, method="C") {
  ## given a matrix of distances between adjacent vertices
  ## (value = Inf if not adjacent)
  ## compute the matrix of shortest path distances
  stopifnot(is.matrix(dist) && isSymmetric(dist))
  stopifnot(all(diag(dist) == 0))
  findist <- dist[is.finite(dist)]
  if(any(findist < 0))
    stop("Some distances are negative")
  ##
  n <- nrow(dist)
  if(n <= 1L) return(dist)
  cols <- col(dist)
  ##
  tol <- .Machine$double.eps
  posdist <- findist[findist > 0]
  if(length(posdist) > 0) {
    shortest <- min(posdist)
    tol2 <- shortest/max(n,1024)
    tol <- max(tol, tol2)
  }
  ##
  switch(method,
         interpreted={
           dpathnew <- dpath <- dist
           changed <- TRUE
           while(changed) {
             for(j in 1:n) 
               dpathnew[,j] <- apply(dpath + dist[j,][cols], 1L, min)
             unequal <- (dpathnew != dpath)
             changed <- any(unequal) & any(abs(dpathnew-dpath)[unequal] > tol)
             dpath <- dpathnew
           }
         },
         C={
           adj <- is.finite(dist)
           diag(adj) <- TRUE
           d <- dist
           d[!adj] <- -1
           z <- .C("Ddist2dpath",
                   nv=as.integer(n),
                   d=as.double(d),
                   adj=as.integer(adj),
                   dpath=as.double(numeric(n*n)),
                   tol=as.double(tol),
                   niter=as.integer(integer(1L)),
                   status=as.integer(integer(1L)),
                   PACKAGE = "spatstat")
           if(z$status == -1L)
             warning(paste("C algorithm did not converge to tolerance", tol,
                           "after", z$niter, "iterations",
                           "on", n, "vertices and",
                           sum(adj) - n, "edges"))
           dpath <- matrix(z$dpath, n, n)
           ## value=-1 implies unreachable
           dpath[dpath < 0] <- Inf
         },
         stop(paste("Unrecognised method", sQuote(method))))
  return(dpath)
}

##
##  plot.anylist.R
##
##  Plotting functions for 'solist', 'anylist', 'imlist'
##       and legacy class 'listof'
##
##  $Revision: 1.29 $ $Date: 2019/09/11 11:52:28 $
##

plot.anylist <- plot.solist <- plot.listof <-
  local({

  ## auxiliary functions
  classes.with.do.plot <- c("im", "ppp", "psp", "msr", "layered", "tess")
  classes.with.multiplot <- c("ppp", "lpp", "msr", "tess",
                              "leverage.ppm", "influence.ppm")

  has.multiplot <- function(x) {
    inherits(x, classes.with.multiplot) ||
      (is.function(x) && "multiplot" %in% names(formals(x)))
  }
  
  extraplot <- function(nnn, x, ..., add=FALSE, extrargs=list(),
                        panel.args=NULL, plotcommand="plot") {
    argh <- list(...)
    if(has.multiplot(x) && identical(plotcommand,"plot"))
      argh <- c(argh, list(multiplot=FALSE))
    if(!is.null(panel.args)) {
      xtra <- if(is.function(panel.args)) panel.args(nnn) else panel.args
      if(!is.list(xtra))
        stop(paste0("panel.args",
                    if(is.function(panel.args)) "(i)" else "",
                    " should be a list"))
      argh <- resolve.defaults(xtra, argh)
    }
    if(length(extrargs) > 0)
      argh <- resolve.defaults(argh, extrargs)
    ## some plot commands don't recognise 'add'
    if(add)
      argh <- append(argh, list(add=TRUE))
    do.call(plotcommand, append(list(x=x), argh))
  }

  exec.or.plot <- function(cmd, i, xi, ..., extrargs=list(), add=FALSE) {
    if(is.null(cmd)) return(NULL)
    argh <-
      resolve.defaults(list(...),
                       extrargs,
                       ## some plot commands don't recognise 'add' 
                       if(add) list(add=TRUE) else NULL,
                       if(has.multiplot(cmd)) list(multiplot=FALSE) else NULL)
    if(is.function(cmd)) {
      do.call(cmd, resolve.defaults(list(i, xi), argh))
    } else {
      do.call(plot, resolve.defaults(list(cmd), argh))
    }
  }

  exec.or.plotshift <- function(cmd, i, xi, ..., vec=vec,
                                extrargs=list(), add=FALSE) {
    if(is.null(cmd)) return(NULL)
    argh <-
      resolve.defaults(list(...),
                       extrargs,
                       ## some plot commands don't recognise 'add' 
                       if(add) list(add=TRUE) else NULL,
                       if(has.multiplot(cmd)) list(multiplot=FALSE) else NULL)
    if(is.function(cmd)) {
      do.call(cmd, resolve.defaults(list(i, xi), argh))
    } else {
      cmd <- shift(cmd, vec)
      do.call(plot, resolve.defaults(list(cmd), argh))
    }
  }

  ## bounding box, including ribbon for images, legend for point patterns
  getplotbox <- function(x, ..., do.plot, plotcommand="plot", multiplot) {
    if(inherits(x, classes.with.do.plot)) {
      if(identical(plotcommand, "plot")) {
        y <- if(has.multiplot(x))
          plot(x, ..., multiplot=FALSE, do.plot=FALSE) else 
          plot(x, ..., do.plot=FALSE)
        return(as.owin(y))
      } else if(identical(plotcommand, "contour")) {
        y <- contour(x, ..., do.plot=FALSE)      
        return(as.owin(y))
      } else {
        plc <- plotcommand
        if(is.character(plc)) plc <- get(plc)
        if(!is.function(plc)) stop("Unrecognised plot function")
        if("do.plot" %in% names(args(plc))) {
          if(has.multiplot(plc)) {
            y <- do.call(plc, list(x=x, ..., multiplot=FALSE, do.plot=FALSE))
          } else {
            y <- do.call(plc, list(x=x, ...,                  do.plot=FALSE))
          }
          return(as.owin(y))
        }
      }
    }
    return(try(as.rectangle(x), silent=TRUE))
  }

  # calculate bounding boxes for each panel using intended arguments!
  getPlotBoxes <- function(xlist, ..., panel.args=NULL, extrargs=list()) {
    userargs <- list(...)
    n <- length(xlist)
    result <- vector(length=n, mode="list")
    for(i in seq_len(n)) {
      pai <- if(is.function(panel.args)) panel.args(i) else list()
      argh <- resolve.defaults(pai, userargs, extrargs)
      result[[i]] <- do.call(getplotbox, append(list(x=xlist[[i]]), argh))
    }
    return(result)
  }
    
  is.shiftable <- function(x) {
    if(is.null(x)) return(TRUE)
    if(is.function(x)) return(FALSE)
    y <- try(as.rectangle(x), silent=TRUE)
    return(!inherits(y, "try-error"))
  }

  maxassigned <- function(i, values) max(-1, values[i[i > 0]])
  
  plot.anylist <- function(x, ..., main, arrange=TRUE,
                            nrows=NULL, ncols=NULL,
                            main.panel=NULL,
                            mar.panel=c(2,1,1,2),
                            hsep = 0,
                            vsep = 0,
                            panel.begin=NULL,
                            panel.end=NULL,
                            panel.args=NULL,
                            panel.begin.args=NULL,
                            panel.end.args=NULL,
                            panel.vpad = 0.2,
                            plotcommand="plot",
                            adorn.left=NULL,
                            adorn.right=NULL,
                            adorn.top=NULL,
                            adorn.bottom=NULL,
                            adorn.size=0.2,
                            equal.scales=FALSE,
                            halign=FALSE, valign=FALSE
                           ) {
    xname <- short.deparse(substitute(x))

    ## recursively expand entries which are 'anylist' etc
    while(any(sapply(x, inherits, what="anylist"))) 
      x <- as.solist(expandSpecialLists(x, "anylist"), demote=TRUE)
    
    isSo <- inherits(x, "solist")
    isIm <- inherits(x, "imlist") || (isSo && all(unlist(lapply(x, is.im))))
    
    ## `boomerang despatch'
    cl <- match.call()
    if(missing(plotcommand) && isIm) {
      cl[[1]] <- as.name("image.imlist")
      parenv <- sys.parent()
      return(invisible(eval(cl, envir=parenv)))
    }

    if(isSo) {
      allfv <- somefv <- FALSE
    } else {
      isfv <- unlist(lapply(x, is.fv))
      allfv <- all(isfv)
      somefv <- any(isfv)
    }
    
    ## panel margins
    if(!missing(mar.panel)) {
      nm <- length(mar.panel)
      if(nm == 1) mar.panel <- rep(mar.panel, 4) else
      if(nm == 2) mar.panel <- rep(mar.panel, 2) else
      if(nm != 4) stop("mar.panel should have length 1, 2 or 4")
    } else if(somefv) {
      ## change default
      mar.panel <- 0.25+c(4,4,2,2)
    }
    
    n <- length(x)
    names(x) <- good.names(names(x), "Component_", 1:n)
    if(is.null(main.panel))
      main.panel <- names(x)
    else {
      if(!is.expression(main.panel))
        main.panel <- as.character(main.panel)
      nmp <- length(main.panel)
      if(nmp == 1)
        main.panel <- rep.int(main.panel, n)
      else if(nmp != n)
        stop("Incorrect length for main.panel")
    }

    if(allfv && equal.scales) {
      ## all entries are 'fv' objects: determine their plot limits
      fvlims <- lapply(x, plot, ..., limitsonly=TRUE)
      ## establish common x,y limits for all panels
      xlim <- range(unlist(lapply(fvlims, getElement, name="xlim")))
      ylim <- range(unlist(lapply(fvlims, getElement, name="ylim")))
      extrargs <- list(xlim=xlim, ylim=ylim)
    } else extrargs <- list()

    extrargs.begin <- resolve.defaults(panel.begin.args, extrargs)
    extrargs.end <- resolve.defaults(panel.end.args, extrargs)
    
    if(!arrange) {
      ## sequence of plots
      result <- vector(mode="list", length=n)
      for(i in 1:n) {
        xi <- x[[i]]
        exec.or.plot(panel.begin, i, xi, main=main.panel[i],
                     extrargs=extrargs.begin)
        result[[i]] <-
          extraplot(i, xi, ...,
                    add=!is.null(panel.begin),
                    main=main.panel[i],
                    panel.args=panel.args, extrargs=extrargs,
                    plotcommand=plotcommand) %orifnull% list()
        exec.or.plot(panel.end, i, xi, add=TRUE, extrargs=extrargs.end)
      }
      if(!is.null(adorn.left))
        warning("adorn.left was ignored because arrange=FALSE")
      if(!is.null(adorn.right))
        warning("adorn.right was ignored because arrange=FALSE")
      if(!is.null(adorn.top))
        warning("adorn.top was ignored because arrange=FALSE")
      if(!is.null(adorn.bottom))
        warning("adorn.bottom was ignored because arrange=FALSE")
      return(invisible(result))
    }

    ## ARRAY of plots
    ## decide whether to plot a main header
    main <- if(!missing(main) && !is.null(main)) main else xname
    if(!is.character(main)) {
      ## main title could be an expression
      nlines <- 1
      banner <- TRUE
    } else {
      ## main title is character string/vector, possibly ""
      banner <- any(nzchar(main))
      if(length(main) > 1)
        main <- paste(main, collapse="\n")
      nlines <- length(unlist(strsplit(main, "\n")))
    }
    ## determine arrangement of plots
    ## arrange like mfrow(nrows, ncols) plus a banner at the top
    if(is.null(nrows) && is.null(ncols)) {
      nrows <- as.integer(floor(sqrt(n)))
      ncols <- as.integer(ceiling(n/nrows))
    } else if(!is.null(nrows) && is.null(ncols))
      ncols <- as.integer(ceiling(n/nrows))
    else if(is.null(nrows) && !is.null(ncols))
      nrows <- as.integer(ceiling(n/ncols))
    else stopifnot(nrows * ncols >= length(x))
    nblank <- ncols * nrows - n
    if(allfv || list(plotcommand) %in% list("persp", persp)) {
      ## Function plots do not have physical 'size'
      sizes.known <- FALSE
    } else {
      ## Determine dimensions of objects
      ##     (including space for colour ribbons, if they are images)
      boxes <- getPlotBoxes(x, ..., plotcommand=plotcommand,
                            panel.args=panel.args, extrargs=extrargs)
      sizes.known <- !any(sapply(boxes, inherits, what="try-error"))
      if(sizes.known) {
        extrargs <- resolve.defaults(extrargs, list(claim.title.space=TRUE))
        boxes <- getPlotBoxes(x, ..., plotcommand=plotcommand,
                              panel.args=panel.args, extrargs=extrargs)
      }
      if(equal.scales && !sizes.known) {
        warning("Ignored equal.scales=TRUE; scales could not be determined")
        equal.scales <- FALSE
      }
    }
    if(sizes.known) {
      ## determine size of each panel
      if(equal.scales) {
        ## do not rescale panels
        scaledboxes <- boxes
      } else {
        ## rescale panels
        sides <- lapply(boxes, sidelengths)
        bwidths <- unlist(lapply(sides, "[", 1))
        bheights <- unlist(lapply(sides, "[", 2))
        ## Force equal heights, unless there is only one column
        scales <- if(ncols > 1) 1/bheights else 1/bwidths
        if(all(is.finite(scales))) {
          scaledboxes <- vector(mode="list", length=n)
          for(i in 1:n)
            scaledboxes[[i]] <- scalardilate(boxes[[i]], scales[i])
        } else {
          #' uh-oh
          equal.scales <- sizes.known <- FALSE
          scaledboxes <- boxes
        }
      }
    }
    ## determine whether to display all objects in one enormous plot
    ## Precondition is that everything has a spatial bounding box
    single.plot <- equal.scales && sizes.known
    if(equal.scales && !single.plot && !allfv)
      warning("equal.scales=TRUE ignored ", "because bounding boxes ",
              "could not be determined", call.=FALSE)
    ## enforce alignment by expanding boxes
    if(halign) {
      if(!equal.scales)
        warning("halign=TRUE ignored because equal.scales=FALSE")
      ## x coordinates align in each column
      xr <- range(sapply(scaledboxes, getElement, name="xrange"))
      scaledboxes <- lapply(scaledboxes, "[[<-", i="xrange", value=xr)
    }
    if(valign) {
      if(!equal.scales)
        warning("valign=TRUE ignored because equal.scales=FALSE")
      ## y coordinates align in each column
      yr <- range(sapply(scaledboxes, getElement, name="yrange"))
      scaledboxes <- lapply(scaledboxes, "[[<-", i="yrange", value=yr)
    }
    ## set up layout
    mat <- matrix(c(seq_len(n), integer(nblank)),
                  byrow=TRUE, ncol=ncols, nrow=nrows)
    if(sizes.known) {
      boxsides <- lapply(scaledboxes, sidelengths)
      xwidths <- sapply(boxsides, "[", i=1)
      xheights <- sapply(boxsides, "[", i=2)
      heights <- apply(mat, 1, maxassigned, values=xheights)
      widths <- apply(mat, 2, maxassigned, values=xwidths)
    } else {
      heights <- rep.int(1, nrows)
      widths <- rep.int(1, ncols)
    }
    #' negative heights/widths arise if a row/column is not used.
    meanheight <- mean(heights[heights > 0])
    meanwidth  <- mean(widths[heights > 0])
    heights[heights <= 0] <- meanheight
    widths[widths <= 0] <- meanwidth
    nall <- n
    ##
    if(single.plot) {
      ## .........  create a single plot ..................
      ## determine sizes
      ht <- max(heights)
      wd <- max(widths)
      marpar <- mar.panel * c(ht, wd, ht, wd)/6
      vsep <- vsep * ht/6
      hsep <- hsep * wd/6
      mainheight <- any(nzchar(main.panel)) * ht/5
      ewidths <- marpar[2] + widths + marpar[4]
      eheights <- marpar[1] + heights + marpar[3] + mainheight
      Width <- sum(ewidths) + hsep * (length(ewidths) - 1)
      Height <- sum(eheights) + vsep * (length(eheights) - 1)
      bigbox <- owin(c(0, Width), c(0, Height))
      ox <- marpar[2] + cumsum(c(0, ewidths + hsep))[1:ncols]
      oy <- marpar[1] + cumsum(c(0, rev(eheights) + vsep))[nrows:1]
      panelorigin <- as.matrix(expand.grid(x=ox, y=oy))
      ## initialise, with banner
      cex <- resolve.1.default(list(cex.title=1.5), list(...))/par('cex.main')
      plot(bigbox, type="n", main=main, cex.main=cex)
      ## plot individual objects
      result <- vector(mode="list", length=n)
      for(i in 1:n) {
        ## determine shift vector that moves bottom left corner of spatial box
        ## to bottom left corner of target area on plot device
        vec <- panelorigin[i,] - with(scaledboxes[[i]], c(xrange[1], yrange[1]))
        ## shift panel contents
        xi <- x[[i]]
        xishift <- shift(xi, vec)
        ## let rip
        if(!is.null(panel.begin))
          exec.or.plotshift(panel.begin, i, xishift,
                            add=TRUE,
                            main=main.panel[i], show.all=TRUE,
                            extrargs=extrargs.begin,
                            vec=vec)
        result[[i]] <-
          extraplot(i, xishift, ...,
                    add=TRUE, show.all=is.null(panel.begin),
                    main=main.panel[i],
                    extrargs=extrargs,
                    panel.args=panel.args,
                    plotcommand=plotcommand) %orifnull% list()
        exec.or.plotshift(panel.end, i, xishift, add=TRUE,
                          extrargs=extrargs.end,
                          vec=vec)
      }
      return(invisible(result))
    }
    ## ................. multiple logical plots using 'layout' ..............
    ## adjust panel margins to accommodate desired extra separation
    mar.panel <- pmax(0, mar.panel + c(vsep, hsep, vsep, hsep)/2)
    ## increase heights to accommodate panel titles
    if(sizes.known && any(nzchar(main.panel))) 
      heights <- heights * (1 + panel.vpad)
    ## check for adornment
    if(!is.null(adorn.left)) {
      ## add margin at left, of width adorn.size * meanwidth
      nall <- i.left <- n+1
      mat <- cbind(i.left, mat)
      widths <- c(adorn.size * meanwidth, widths)
    } 
    if(!is.null(adorn.right)) {
      ## add margin at right, of width adorn.size * meanwidth
      nall <- i.right <- nall+1
      mat <- cbind(mat, i.right)
      widths <- c(widths, adorn.size * meanwidth)
    } 
    if(!is.null(adorn.bottom)) {
      ## add margin at bottom, of height adorn.size * meanheight
      nall <- i.bottom <- nall+1
      mat <- rbind(mat, i.bottom)
      heights <- c(heights, adorn.size * meanheight)
    } 
    if(!is.null(adorn.top)) {
      ## add margin at top, of height adorn.size * meanheight
      nall <- i.top <- nall + 1
      mat <- rbind(i.top, mat)
      heights <- c(adorn.size * meanheight, heights)
    } 
    if(banner) {
      ## Increment existing panel numbers
      ## New panel 1 is the banner
      panels <- (mat > 0)
      mat[panels] <- mat[panels] + 1
      mat <- rbind(1, mat)
      heights <- c(0.1 * meanheight * (1 + nlines), heights)
    }
    ## declare layout
    layout(mat, heights=heights, widths=widths, respect=sizes.known)
    ## start output .....
    ## .... plot banner
    if(banner) {
      opa <- par(mar=rep.int(0,4), xpd=TRUE)
      plot(numeric(0),numeric(0),type="n",ann=FALSE,axes=FALSE,
           xlim=c(-1,1),ylim=c(-1,1))
      cex <- resolve.1.default(list(cex.title=1.5), list(...))/par('cex')
      text(0,0,main, cex=cex)
    }
    ## plot panels
    npa <- par(mar=mar.panel)
    if(!banner) opa <- npa
    result <- vector(mode="list", length=n)
    for(i in 1:n) {
      xi <- x[[i]]
      exec.or.plot(panel.begin, i, xi, main=main.panel[i],
                   extrargs=extrargs.begin)
      result <-
        extraplot(i, xi, ...,
                  add = !is.null(panel.begin), 
                  main = main.panel[i],
                  extrargs=extrargs,
                  panel.args=panel.args,
                  plotcommand=plotcommand) %orifnull% list()
      exec.or.plot(panel.end, i, xi, add=TRUE, extrargs=extrargs.end)
    }
    ## adornments
    if(nall > n) {
      par(mar=rep.int(0,4), xpd=TRUE)
      if(!is.null(adorn.left))
        adorn.left()
      if(!is.null(adorn.right))
        adorn.right()
      if(!is.null(adorn.bottom))
        adorn.bottom()
      if(!is.null(adorn.top))
        adorn.top()
    }
    ## revert
    layout(1)
    par(opa)
    return(invisible(result))
  }

  plot.anylist
})


contour.imlist <- contour.listof <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  do.call(plot.solist,
          resolve.defaults(list(x=x, plotcommand="contour"),
                           list(...),
                           list(main=xname)))
}

plot.imlist <- local({

  plot.imlist <- function(x, ..., plotcommand="image",
                          equal.ribbon = FALSE, ribmar=NULL) {
    xname <- short.deparse(substitute(x))
    if(missing(plotcommand) &&
       any(sapply(x, inherits, what=c("linim", "linfun"))))
      plotcommand <- "plot"
    if(equal.ribbon &&
       (list(plotcommand) %in% list("image", "plot", image, plot))) {
      out <- imagecommon(x, ..., xname=xname, ribmar=ribmar)
    } else {
      out <- do.call(plot.solist,
                     resolve.defaults(list(x=x, plotcommand=plotcommand), 
                                      list(...),
                                      list(main=xname)))
    }
    return(invisible(out))
  }

  imagecommon <- function(x, ...,
                          xname,
                          zlim=NULL,
                          ribbon=TRUE,
                          ribside=c("right", "left", "bottom", "top"),
                          ribsep=NULL, ribwid=0.5, ribn=1024,
                          ribscale=NULL, ribargs=list(),
                          ribmar = NULL, mar.panel = c(2,1,1,2)) {
    if(missing(xname))
      xname <- short.deparse(substitute(x))
    ribside <- match.arg(ribside)
    stopifnot(is.list(ribargs))
    if(!is.null(ribsep))
      warning("Argument ribsep is not yet implemented for image arrays")
    ## determine range of values
    if(is.null(zlim))
      zlim <- range(unlist(lapply(x, range)))
    ## determine common colour map
    imcolmap <- plot.im(x[[1]], do.plot=FALSE, zlim=zlim, ..., ribn=ribn)
    ## plot ribbon?
    if(!ribbon) {
      ribadorn <- list()
    } else {
      ## determine plot arguments for colour ribbon
      vertical <- (ribside %in% c("right", "left"))
      scaleinfo <- if(!is.null(ribscale)) list(labelmap=ribscale) else list()
      sidecode <- match(ribside, c("bottom", "left", "top", "right"))
      ribstuff <- c(list(x=imcolmap, main="", vertical=vertical),
                    ribargs,
                    scaleinfo,
                    list(side=sidecode))
      if (is.null(mar.panel)) 
        mar.panel <- c(2, 1, 1, 2)
      if (length(mar.panel) != 4) 
        mar.panel <- rep(mar.panel, 4)[1:4]
      if (is.null(ribmar)) {
        ribmar <- mar.panel/2
        newmar <- c(2, 0)
        switch(ribside,
               left   = { ribmar[c(2, 4)] <- newmar },
               right  = { ribmar[c(4, 2)] <- newmar },
               bottom = { ribmar[c(1, 3)] <- newmar },
               top    = { ribmar[c(3, 1)] <- newmar }
               )
      }
      ## bespoke function executed to plot colour ribbon
      do.ribbon <- function() {
        opa <- par(mar=ribmar)
        do.call(plot, ribstuff)
        par(opa)
      }
      ## ribbon plot function encoded as 'adorn' argument
      ribadorn <- list(adorn=do.ribbon, adorn.size=ribwid)
      names(ribadorn)[1] <- paste("adorn", ribside, sep=".")
    }
    ##
    result <- do.call(plot.solist,
                      resolve.defaults(list(x=x, plotcommand="image"),
                                       list(...),
                                       list(mar.panel=mar.panel,
                                            main=xname,
                                            col=imcolmap, zlim=zlim,
                                            ribbon=FALSE),
                                       ribadorn))
    return(invisible(result))
  }

  plot.imlist
})

image.imlist <- image.listof <-
  function(x, ..., equal.ribbon = FALSE, ribmar=NULL) {
    plc <- resolve.1.default(list(plotcommand="image"), list(...))
    if(list(plc) %in% list("image", "plot", image, plot)) {
      out <- plot.imlist(x, ..., plotcommand="image",
                         equal.ribbon=equal.ribbon, ribmar=ribmar)
    } else {
      out <- plot.solist(x, ..., ribmar=ribmar)
    }
    return(invisible(out))
  }

#
#	quadclass.S
#
#	Class 'quad' to define quadrature schemes
#	in (rectangular) windows in two dimensions.
#
#	$Revision: 4.28 $	$Date: 2019/01/14 06:44:41 $
#
# An object of class 'quad' contains the following entries:
#
#	$data:	an object of class 'ppp'
#		defining the OBSERVATION window, 
#		giving the locations (& marks) of the data points.
#
#	$dummy:	object of class 'ppp'
#		defining the QUADRATURE window, 
#		giving the locations (& marks) of the dummy points.
#	
#	$w: 	vector giving the nonnegative weights for the
#		data and dummy points (data first, followed by dummy)
#
#		w may also have an attribute attr(w, "zeroes")
#               equivalent to (w == 0). If this is absent
#               then all points are known to have positive weights.
#
#       $param:
#               parameters that were used to compute the weights
#               and possibly to create the dummy points (see below).
#              
#       The combined (data+dummy) vectors of x, y coordinates of the points, 
#       and their weights, are extracted using standard functions 
#       x.quad(), y.quad(), w.quad() etc.
#
# ----------------------------------------------------------------------
#  Note about parameters:
#
#       If the quadrature scheme was created by quadscheme(),
#       then $param contains
#
#           $param$weight
#                list containing the values of all parameters
#                actually used to compute the weights.
#
#           $param$dummy
#                list containing the values of all parameters
#                actually used to construct the dummy pattern
#                via default.dummy();
#                or NULL if the dummy pattern was provided externally
#
#           $param$sourceid
#                vector mapping the quadrature points to the
#                original data and dummy points.
#
#   If you constructed the quadrature scheme manually, this
#   structure may not be present.
#
#-------------------------------------------------------------

quad <- function(data, dummy, w, param=NULL) {
  
  data <- as.ppp(data)
  dummy <- as.ppp(dummy)

  n <- data$n + dummy$n
	
  if(missing(w))
    w <- rep.int(1, n)
  else {
    w <- as.vector(w)
    if(length(w) != n)
      stop("length of weights vector w is not equal to total number of points")
  }

  if(is.null(attr(w, "zeroes")) && any( w == 0))
	attr(w, "zeroes") <- (w == 0)

  Q <- list(data=data, dummy=dummy, w=w, param=param)
  class(Q) <- "quad"

  invisible(Q)
}

is.quad <- function(x) {
  inherits(x, "quad")
}

# ------------------ extractor functions ----------------------

x.quad <- function(Q) {
  verifyclass(Q, "quad")
  c(Q$data$x, Q$dummy$x)
}

y.quad <- function(Q) {
  verifyclass(Q, "quad")
  c(Q$data$y, Q$dummy$y)
}

coords.quad <- function(x, ...) {
  data.frame(x=x.quad(x),
             y=y.quad(x))
}

w.quad <- function(Q) {
  verifyclass(Q, "quad")
  Q$w
}

param.quad <- function(Q) {
  verifyclass(Q, "quad")
  Q$param
}
 
n.quad <- function(Q) {
  verifyclass(Q, "quad")
  Q$data$n + Q$dummy$n
}

marks.quad <- function(x, dfok=FALSE, ...) {
  verifyclass(x, "quad")
  dat <- x$data
  dum <- x$dummy
  if(dfok) warning("ignored dfok = TRUE; not implemented")
  mdat <- marks(dat, dfok=FALSE, ...)
  mdum <- marks(dum, dfok=FALSE, ...)
  if(is.null(mdat) && is.null(mdum))
    return(NULL)
  if(is.null(mdat))
    mdat <- rep.int(NA_integer_, dat$n)
  if(is.null(mdum))
    mdum <- rep.int(NA_integer_, dum$n)
  if(is.factor(mdat) && is.factor(mdum)) {
    mall <- cat.factor(mdat, mdum)
  } else mall <- c(mdat, mdum)
  return(mall)
}

is.marked.quad <- function(X, na.action="warn", ...) {
  marx <- marks(X, ...)
  if(is.null(marx))
    return(FALSE)
  if(anyNA(marx))
    switch(na.action,
           warn = {
             warning(paste("some mark values are NA in the point pattern",
                           short.deparse(substitute(X))))
           },
           fatal = {
             return(FALSE)
           },
           ignore = {}
           )
  return(TRUE)
}

is.multitype.quad <- function(X, na.action="warn", ...) {
  marx <- marks(X, ...)
  if(is.null(marx))
    return(FALSE)
  if(anyNA(marx))
    switch(na.action,
           warn = {
             warning(paste("some mark values are NA in the point pattern",
                           short.deparse(substitute(X))))
           },
           fatal = {
             return(FALSE)
           },
           ignore = {}
           )
  return(!is.data.frame(marx) && is.factor(marx))
}

is.data <- function(Q) {
  verifyclass(Q, "quad")
  return(c(rep.int(TRUE, Q$data$n),
	   rep.int(FALSE, Q$dummy$n)))
}

equals.quad <- function(Q) {
    # return matrix E such that E[i,j] = (X[i] == U[j])
    # where X = Q$data and U = union.quad(Q)
    n <- Q$data$n
    m <- Q$dummy$n
    E <- matrix(FALSE, nrow=n, ncol=n+m)
    diag(E) <- TRUE
    E
}

equalsfun.quad <- function(Q) {
  stopifnot(inherits(Q, "quad"))
  return(function(i,j) { i == j })
}

equalpairs.quad <- function(Q) {
  # return two-column matrix E such that
  #     X[E[i,1]] == U[E[i,2]] for all i
  # where X = Q$data and U = union.quad(Q)
  n <- Q$data$n
  return(matrix(rep.int(seq_len(n),2), ncol=2))
}
      
union.quad <- function(Q) {
  verifyclass(Q, "quad")
  ppp(x= c(Q$data$x, Q$dummy$x),
      y= c(Q$data$y, Q$dummy$y),
      window=Q$dummy$window,
      marks=marks.quad(Q),
      check=FALSE)
}
	
#
#   Plot a quadrature scheme
#
#
plot.quad <- function(x, ..., main, add=FALSE, dum=list(), tiles=FALSE) {
  if(missing(main) || is.null(main)) 
    main <- short.deparse(substitute(x))
  verifyclass(x, "quad")
  data <- x$data
  dummy <- x$dummy
  # determine plot parameters for dummy points
  dum <- resolve.defaults(dum, list(pch=".", add=TRUE))
  tt <- NULL
  if(tiles) {
    # show tiles that determined the weights
    wp <- x$param$weight
    tt <- NULL
    if(is.null(wp) || is.null(wp$method)) {
      warning("Tile information is not available")
    } else {
      switch(wp$method,
             grid = {
               ntile <- wp$ntile
               tt <- quadrats(as.owin(x), ntile[1], ntile[2])
             },
             dirichlet = {
               U <- union.quad(x)
               if(wp$exact) {
                 tt <- dirichlet(U)
               } else {
                 win <- as.mask(as.owin(U))
                 tileid <- im(exactdt(U)$i,
                              win$xcol, win$yrow,
                              win$xrange, win$yrange)
                 tt <- tess(image=tileid[win, drop=FALSE])
               }
             },
             warning("Unrecognised 'method' for tile weights")
             )
    }
  }
  pixeltiles <- !is.null(tt) && tt$type == "image"
  tileargs <- resolve.defaults(list(x=tt, main=main, add=add),
                               list(...),
                               if(!pixeltiles) list(col="grey") else NULL)
  if(!is.marked(data)) {
    if(!is.null(tt)) {
      do.call(plot, tileargs)
      add <- TRUE
    }
    plot(data, main=main, add=add, ...)
    do.call(plot, append(list(x=dummy), dum))
  } else if(is.multitype(data) && !add) {
    oldpar <- par(ask = interactive() &&
                  (.Device %in% c("X11", "GTK", "windows", "Macintosh")))
    on.exit(par(oldpar))
    data.marks <- marks(data)
    dummy.marks <- marks(dummy)
    types <- levels(data.marks)
    for(k in types) {
      add <- FALSE
      if(!is.null(tt)) {
        do.call(plot, tileargs)
        add <- TRUE
      }
      maink <- paste(main, "\n mark = ", k, sep="")
      plot(unmark(data[data.marks == k]), main=maink, add=add, ...)
      do.call(plot, append(list(x=unmark(dummy[dummy.marks == k])),
                             dum))
    }
  } else {
    if(!is.null(tt)) {
      do.call(plot, tileargs)
      add <- TRUE
    }
    plot(data, ..., main=main, add=add)
    do.call(plot, append(list(x=dummy), dum))
  }
  invisible(NULL)
}

# subset operator

"[.quad" <- function(x, ...) {
  U <- union.quad(x)
  Z <- is.data(x)
  w <- w.quad(x)
  # determine serial numbers of points to be included
  V <- U %mark% seq_len(U$n)
  i <- marks(V[...])
  # extract corresponding subsets of vectors
  Z <- Z[i]
  w <- w[i]
  # take subset of points, using any type of subset index
  U <- U[...]
  # stick together
  quad(U[Z], U[!Z], w)
}

domain.quad <- Window.quad <- function(X, ...) { as.owin(X) }

"Window<-.quad" <- function(X, ..., value) {
  verifyclass(value, "owin")
  return(X[value])
}

unitname.quad <- function(x) {
  return(unitname(x$data))
}

"unitname<-.quad" <- function(x, value) {
  unitname(x$data) <- value
  unitname(x$dummy) <- value
  return(x)
}

##
## solist.R
##
## Methods for class `solist' (spatial object list)
##
##      and related classes 'anylist', 'ppplist', 'imlist'
##
## plot.solist is defined in plot.solist.R
##
## $Revision: 1.21 $ $Date: 2020/05/07 13:04:05 $

anylist <- function(...) {
  x <- list(...)
  class(x) <- c("anylist", "listof", class(x))
  return(x)
}

print.anylist <- function (x, ...) {
  ll <- length(x)
  if(ll == 0) {
    splat("(Zero length list)")
    return(invisible(NULL))
  }
  nn <- names(x)
  if (length(nn) != ll) 
    nn <- paste("Component", seq.int(ll))
  spaceok <- waxlyrical('space')
  for (i in seq_len(ll)) {
    splat(paste0(nn[i], ":"))
    print(x[[i]], ...)
    if(spaceok && i < ll) cat("\n")
  }
  return(invisible(NULL))
}

as.anylist <- function(x) {
  if(inherits(x, "anylist")) return(x)
  if(!is.list(x))
    x <- list(x)
  class(x) <- c("anylist", "listof", class(x))
  return(x)
}
  
"[.anylist" <- function(x, i, ...) {
  cl <- oldClass(x)
  ## invoke list method
  y <- NextMethod("[")
  if(length(y) == 0) return(list())
  class(y) <- cl
  return(y)
}

"[<-.anylist" <- function(x, i, value) {
  as.anylist(NextMethod("[<-"))
}

summary.anylist <- function(object, ...) {
  as.anylist(lapply(object, summary, ...))
}

pool.anylist <- function(x, ...) {
  do.call(pool, append(x, list(...)))
}

## .................... solist .............................

is.sob <- local({
  ## test whether x is a spatial object suitable for solist
  sobjectclasses <- c("ppp", "psp", "im", "owin", 
                      "quad", "tess", "msr",
                      "quadratcount", "quadrattest", 
                      "layered",
                      "funxy", "distfun", "nnfun", 
                      "lpp", "linnet", "linfun", "lintess",  
                      "influence.ppm", "leverage.ppm")
  # Note 'linim' inherits 'im'
  #      'dfbetas.ppm' inherits 'msr'

  is.sob <- function(x) { inherits(x, what=sobjectclasses) }
  is.sob
})
  
solist <- function(..., check=TRUE, promote=TRUE, demote=FALSE, .NameBase) {
  stuff <- list(...)
  if(length(stuff) && !missing(.NameBase) && !any(nzchar(names(stuff))))
    names(stuff) <- paste(.NameBase, seq_along(stuff))
  if((check || demote) && !all(sapply(stuff, is.sob))) {
    if(demote)
      return(as.anylist(stuff))
    stop("Some arguments of solist() are not 2D spatial objects")
  }
  class(stuff) <- c("solist", "anylist", "listof", class(stuff))
  if(promote) {
    if(all(unlist(lapply(stuff, is.ppp)))) {
      class(stuff) <- c("ppplist", class(stuff))
    } else if(all(unlist(lapply(stuff, is.im)))) {
      class(stuff) <- c("imlist", class(stuff))
    }
  }
  return(stuff)
}

as.solist <- function(x, ...) {
  if(inherits(x, "solist") && length(list(...)) == 0) {
    #' wipe superfluous info
    if(inherits(x, "ppplist")) 
      attributes(x)[c("fsplit", "fgroup")] <- NULL
    class(x) <- c("solist", "anylist", "listof")
    return(x)
  }
  #' needs to be enclosed in list() ?
  if(!is.list(x) || (is.sob(x) && !inherits(x, "layered")))
    x <- list(x)
  return(do.call(solist, append(x, list(...))))
}

is.solist <- function(x) inherits(x, "solist")

print.solist <- function (x, ...) {
  what <- if(inherits(x, "ppplist")) "point patterns" else
          if(inherits(x, "imlist")) "pixel images" else "spatial objects"
  splat(paste("List of", what))
  parbreak()
  NextMethod("print")
}


"[.solist" <- function(x, i, ...) {
  cl <- oldClass(x)
  if(!missing(i) && is.owin(i)) {
    ## spatial subset
    y <- lapply(unclass(x), "[", i=i, ...)
  } else {
    ## invoke list method
    y <- NextMethod("[")
  }
  if(length(y) == 0) return(list())
  class(y) <- cl
  return(y)
}
  
"[<-.solist" <- function(x, i, value) {
  ## invoke list method
  y <- NextMethod("[<-")
  ## check again
  return(do.call(solist, y))
}
  
summary.solist <- function(object, ...) {
  x <- lapply(object, summary, ...)
  attr(x, "otype") <-
    if(inherits(object, "ppplist")) "ppp" else
    if(inherits(object, "imlist")) "im" else ""
  class(x) <- c("summary.solist", "anylist")
  x
}

print.summary.solist <- function(x, ...) {
  what <- switch(attr(x, "otype"),
                 ppp="point patterns",
                 im="pixel images",
                 "spatial objects")
  splat("Summary of", length(x), what)
  parbreak()
  NextMethod("print")
}

as.layered.solist <- function(X) {
  layered(LayerList=X)
}

#'  ----- ppplist and imlist ----------------------------
#'  for efficiency only

as.ppplist <- function(x, check=TRUE) {
  if(check) {
    x <- as.solist(x, promote=TRUE, check=TRUE)
    if(!inherits(x, "ppplist"))
      stop("some entries are not point patterns")
  }
  class(x) <- unique(c("ppplist", "solist", "anylist", "listof", class(x)))
  return(x)
}

is.ppplist <- function(x) inherits(x, "ppplist")

as.imlist <- function(x, check=TRUE) {
  if(check) {
    x <- as.solist(x, promote=TRUE, check=TRUE)
    if(!inherits(x, "imlist"))
      stop("some entries are not images")
  }
  class(x) <- unique(c("imlist", "solist", "anylist", "listof", class(x)))
  return(x)
}

is.imlist <- function(x) inherits(x, "imlist")

# --------------- counterparts of 'lapply' --------------------

anylapply <- function(X, FUN, ...) {
  v <- lapply(X, FUN, ...)
  return(as.anylist(v))
}

solapply <- function(X, FUN, ..., check=TRUE, promote=TRUE, demote=FALSE) {
  v <- lapply(X, FUN, ...)
  u <- as.solist(v, check=check, promote=promote, demote=demote)
  return(u)
}

expandSpecialLists <- function(x, special="solist") {
  ## x is a list which may include entries which are lists, of class 'special'
  ## unlist these entries only
  hit <- sapply(x, inherits, what=special) 
  if(!any(hit)) return(x)
  # wrap each *non*-special entry in list()
  x[!hit] <- lapply(x[!hit], list)
  # now strip one layer of list() from all entries
  return(unlist(x, recursive=FALSE))
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#'
#'    tests/deltasuffstat.R
#' 
#'    Explicit tests of 'deltasuffstat'
#' 
#' $Revision: 1.3 $ $Date: 2020/04/28 12:58:26 $

if(ALWAYS) {  # depends on C code
local({
  
  disagree <- function(x, y, tol=1e-7) { !is.null(x) && !is.null(y) && max(abs(x-y)) > tol }

  flydelta <- function(model, modelname="") {
    ## Check execution of different algorithms for 'deltasuffstat' 
    dSS <- deltasuffstat(model, sparseOK=TRUE)
    dBS <- deltasuffstat(model, sparseOK=TRUE,  use.special=FALSE, force=TRUE)
    dBF <- deltasuffstat(model, sparseOK=FALSE, use.special=FALSE, force=TRUE)
    ## Compare results
    if(disagree(dBS, dSS))
      stop(paste(modelname, "model: Brute force algorithm disagrees with special algorithm"))
    if(disagree(dBF, dBS))
      stop(paste(modelname, "model: Sparse and full versions of brute force algorithm disagree"))
    return(invisible(NULL))
  }

  modelS <- ppm(cells ~ x, Strauss(0.13), nd=10)
  flydelta(modelS, "Strauss")

  antsub <- ants[c(FALSE,TRUE,FALSE)]
  rmat <- matrix(c(130, 90, 90, 60), 2, 2)
  
  modelM <- ppm(antsub ~ 1, MultiStrauss(rmat), nd=16)
  flydelta(modelM, "MultiStrauss")
                
  modelA <- ppm(antsub ~ 1, HierStrauss(rmat, archy=c(2,1)), nd=16)
  flydelta(modelA, "HierStrauss")
})

}
#'
#'  tests/density.R
#'
#'  Test behaviour of density() methods,
#'                    relrisk(), Smooth()
#'                    and inhomogeneous summary functions
#'                    and idw, adaptive.density, intensity
#'
#'  $Revision: 1.56 $  $Date: 2020/04/28 12:58:26 $
#'

local({

  # test all cases of density.ppp and densityfun.ppp
  
  tryit <- function(..., do.fun=TRUE) {
    Z <- density(cells, ..., at="pixels")
    Z <- density(cells, ..., at="points")
    if(do.fun) {
      f <- densityfun(cells, ...)
      U <- f(0.1, 0.3)
    }
    return(invisible(NULL))
  }

  if(ALWAYS) {
    tryit(0.05)
    tryit(0.05, diggle=TRUE)
    tryit(0.05, se=TRUE)
    tryit(0.05, weights=expression(x))
    tryit(0.07, kernel="epa")
    tryit(sigma=Inf)
  }
  if(FULLTEST) {
    tryit(0.07, kernel="quartic")
    tryit(0.07, kernel="disc")
    tryit(0.07, kernel="epa", weights=expression(x))
    tryit(sigma=Inf, weights=expression(x))
  }
  
  V <- diag(c(0.05^2, 0.07^2))

  if(ALWAYS) {
    tryit(varcov=V)
  }
  if(FULLTEST) {
    tryit(varcov=V, diggle=TRUE)
    tryit(varcov=V, weights=expression(x))
    tryit(varcov=V, weights=expression(x), diggle=TRUE)
    Z <- distmap(runifpoint(5, Window(cells)))
    tryit(0.05, weights=Z)
    tryit(0.05, weights=Z, diggle=TRUE)
  }

  trymost <- function(...) tryit(..., do.fun=FALSE) 
  wdf <- data.frame(a=1:42,b=42:1)
  if(ALWAYS) {
    trymost(0.05, weights=wdf)
    trymost(sigma=Inf, weights=wdf)
  }
  if(FULLTEST) {
    trymost(0.05, weights=wdf, diggle=TRUE)
    trymost(varcov=V, weights=wdf)
    trymost(varcov=V, weights=expression(cbind(x,y)))
  }

  ## check conservation of mass
  checkconserve <- function(X, xname, sigma, toler=0.01) {
    veritas <- npoints(X)
    vino <- integral(density(X, sigma, diggle=TRUE))
    relerr <- abs(vino - veritas)/veritas
    if(relerr > toler)
      stop(paste("density.ppp(diggle=TRUE) fails to conserve mass:",
                 vino, "!=", veritas,
                 "for", sQuote(xname)),
           call.=FALSE)
    return(relerr)
  }
  if(FULLTEST) {
    checkconserve(cells, "cells", 0.15)
  }
  if(ALWAYS) {
    checkconserve(split(chorley)[["lung"]], "lung", 2)
  }
  
  ## run C algorithm 'denspt'
  opa <- spatstat.options(densityC=TRUE, densityTransform=FALSE)
  if(ALWAYS) {
    tryit(varcov=V)
  }
  if(FULLTEST) {
    tryit(varcov=V, weights=expression(x))
    trymost(varcov=V, weights=wdf)
  }
  spatstat.options(opa)

  crossit <- function(..., sigma=NULL) {
    U <- runifpoint(20, Window(cells))
    a <- densitycrossEngine(cells, U, ..., sigma=sigma)
    a <- densitycrossEngine(cells, U, ..., sigma=sigma, diggle=TRUE)
    invisible(NULL)
  }
  if(ALWAYS) {
    crossit(varcov=V, weights=cells$x)
    crossit(sigma=Inf)
  }
  if(FULLTEST) {
    crossit(varcov=V, weights=wdf)
    crossit(sigma=0.1, weights=wdf)
    crossit(sigma=0.1, kernel="epa", weights=wdf)
  }
  
  ## apply different discretisation rules
  if(ALWAYS) {
    Z <- density(cells, 0.05, fractional=TRUE)
  }
  if(FULLTEST) {
    Z <- density(cells, 0.05, preserve=TRUE)
    Z <- density(cells, 0.05, fractional=TRUE, preserve=TRUE)
  }
        
  ## compare results with different algorithms
  crosscheque <- function(expr) {
    e <- as.expression(substitute(expr))
    ename <- sQuote(deparse(substitute(expr)))
    ## interpreted R
    opa <- spatstat.options(densityC=FALSE, densityTransform=FALSE)
    val.interpreted <- eval(e)
    ## established C algorithm 'denspt'
    spatstat.options(densityC=TRUE, densityTransform=FALSE)
    val.C <- eval(e)
    ## new C algorithm 'Gdenspt' using transformed coordinates
    spatstat.options(densityC=TRUE, densityTransform=TRUE)
    val.Transform <- eval(e)
    spatstat.options(opa)
    if(max(abs(val.interpreted - val.C)) > 0.001)
      stop(paste("Numerical discrepancy between R and C algorithms in",
                 ename))
    if(max(abs(val.C - val.Transform)) > 0.001)
      stop(paste("Numerical discrepancy between C algorithms",
                 "using transformed and untransformed coordinates in",
                 ename))
    invisible(NULL)
  }

  ## execute & compare results of density(at="points") with different algorithms
  wdfr <- cbind(1:npoints(redwood), 2)
  if(ALWAYS) {
    crosscheque(density(redwood, at="points", sigma=0.13, edge=FALSE))
    crosscheque(density(redwood, at="points", sigma=0.13, edge=FALSE,
                        weights=wdfr[,1]))
    crosscheque(density(redwood, at="points", sigma=0.13, edge=FALSE,
                        weights=wdfr))
  }

  ## correctness of non-Gaussian kernel calculation
  leavein <- function(ker, maxd=0.025) {
    ZI <- density(redwood, 0.12, kernel=ker, edge=FALSE,
                  dimyx=256)[redwood]
    ZP <- density(redwood, 0.12, kernel=ker, edge=FALSE,
                  at="points", leaveoneout=FALSE)
    discrep <- max(abs(ZP - ZI))/npoints(redwood)
    if(discrep > maxd) 
      stop(paste("Discrepancy",
                 signif(discrep, 3),
                 "in calculation for", ker, "kernel"))
    return(invisible(NULL))
  }
  if(ALWAYS) {
    leavein("epanechnikov", 0.015)
  }
  if(FULLTEST) {
    leavein("quartic",      0.010)
    leavein("disc",         0.100)
  }

  ## bandwidth selection code blocks
  sigvec <- 0.01 * 2:15
  sigran <- range(sigvec)
  if(ALWAYS) {
    bw.ppl(redwood, sigma=sigvec)
    bw.CvL(redwood, sigma=sigvec)
  }
  if(FULLTEST) {
    bw.ppl(redwood, srange=sigran, ns=5)
    bw.CvL(redwood, srange=sigran, ns=5)
  }
  ## adaptive bandwidth
  if(ALWAYS) {
    a <- bw.abram(redwood)
  }
  if(FULLTEST) {
    a <- bw.abram(redwood, pilot=density(redwood, 0.2))
    a <- bw.abram(redwood, smoother="densityVoronoi", at="pixels")
  }
  
  ## Kinhom
  if(ALWAYS) {
    lam <- density(redwood)
    K <- Kinhom(redwood, lam)
  
    lamX <- density(redwood, at="points")
    KX <- Kinhom(redwood, lamX)
  }

  ## test all code cases of new 'relrisk.ppp' algorithm
  pants <- function(..., X=ants, sigma=100, se=TRUE) {
    a <- relrisk(X, sigma=sigma, se=se, ...)
    return(TRUE)
  }
  if(ALWAYS) {
    pants()
    pants(diggle=TRUE)
    pants(edge=FALSE)
    pants(at="points")
    pants(casecontrol=FALSE)
    pants(relative=TRUE)
    pants(sigma=Inf)
    pants(sigma=NULL, varcov=diag(c(100,100)^2))
  }
  if(FULLTEST) {
    pants(diggle=TRUE, at="points")
    pants(edge=FALSE, at="points")
    pants(casecontrol=FALSE, relative=TRUE)
    pants(casecontrol=FALSE,at="points")
    pants(relative=TRUE,at="points")
    pants(casecontrol=FALSE, relative=TRUE,at="points")
    pants(relative=TRUE, control="Cataglyphis", case="Messor")
    pants(relative=TRUE, control="Cataglyphis", case="Messor", at="points")
    pants(casecontrol=FALSE, case="Messor", se=FALSE)
    pants(case=2, at="pixels", relative=TRUE)
    pants(case=2, at="points", relative=TRUE)
    pants(case=2, at="pixels", relative=FALSE)
    pants(case=2, at="points", relative=FALSE)
  }
  ## more than 2 types
  if(ALWAYS) {
    pants(X=sporophores)
    pants(X=sporophores, sigma=20, at="points")
    bw.relrisk(sporophores, method="leastsquares")
  }
  if(FULLTEST) {
    pants(X=sporophores, sigma=20, relative=TRUE, at="points")
    pants(X=sporophores, sigma=20, at="pixels", se=FALSE)
    pants(X=sporophores, sigma=20, relative=TRUE, at="pixels", se=FALSE)
    bw.relrisk(sporophores, method="weightedleastsquares")
  }
  
  ## likewise 'relrisk.ppm'
  fit <- ppm(ants ~ x)
  rants <- function(..., model=fit) {
    a <- relrisk(model, sigma=100, se=TRUE, ...)
    return(TRUE)
  }
  if(ALWAYS) {
    rants()
    rants(diggle=TRUE)
    rants(edge=FALSE)
    rants(at="points")
    rants(casecontrol=FALSE)
    rants(relative=TRUE)
  }
  if(FULLTEST) {
    rants(diggle=TRUE, at="points")
    rants(edge=FALSE, at="points")
    rants(casecontrol=FALSE, relative=TRUE)
    rants(casecontrol=FALSE,at="points")
    rants(relative=TRUE,at="points")
    rants(casecontrol=FALSE, relative=TRUE,at="points")
    rants(relative=TRUE, control="Cataglyphis", case="Messor")
    rants(relative=TRUE, control="Cataglyphis", case="Messor", at="points")
  }
  ## more than 2 types
  fut <- ppm(sporophores ~ x)
  if(ALWAYS) {
    rants(model=fut)
  }
  if(FULLTEST) {
    rants(model=fut, at="points")
    rants(model=fut, relative=TRUE, at="points")
  }
  
  ## execute Smooth.ppp and Smoothfun.ppp in all cases
  stroke <- function(..., Y = longleaf) {
    Z <- Smooth(Y, ..., at="pixels")
    Z <- Smooth(Y, ..., at="points", leaveoneout=TRUE)
    Z <- Smooth(Y, ..., at="points", leaveoneout=FALSE)
    f <- Smoothfun(Y, ...)
    f(120, 80)
    f(Y[1:2])
    f(Y[FALSE])
    U <- as.im(f)
    return(invisible(NULL))
  }
  if(ALWAYS) {
    stroke()
    stroke(5, diggle=TRUE)
    stroke(5, geometric=TRUE)
    stroke(1e-6) # generates warning about small bandwidth
    stroke(5, weights=expression(x))
    stroke(5, kernel="epa")
    stroke(sigma=Inf)
  }
  if(FULLTEST) {
    Z <- as.im(function(x,y){abs(x)+1}, Window(longleaf))
    stroke(5, weights=Z)
    stroke(5, weights=runif(npoints(longleaf)))
    stroke(varcov=diag(c(25, 36)))
    stroke(varcov=diag(c(25, 36)), weights=runif(npoints(longleaf)))
    stroke(5, Y=longleaf %mark% 1)
    stroke(5, Y=cut(longleaf,breaks=3))
    stroke(5, weights=Z, geometric=TRUE)
  }
  
  markmean(longleaf, 9)
  
  strike <- function(..., Y=finpines) {
    Z <- Smooth(Y, ..., at="pixels")
    Z <- Smooth(Y, ..., at="points", leaveoneout=TRUE)
    Z <- Smooth(Y, ..., at="points", leaveoneout=FALSE)
    f <- Smoothfun(Y, ...)
    f(4, 1)
    f(Y[1:2])
    f(Y[FALSE])
    U <- as.im(f)
    return(invisible(NULL))
  }
  if(ALWAYS) {
    strike()
    strike(sigma=1.5, kernel="epa")
    strike(varcov=diag(c(1.2, 2.1)))
    strike(sigma=1e-6)
    strike(sigma=Inf)
  }
  if(FULLTEST) {
    strike(sigma=1e-6, kernel="epa")
    strike(1.5, weights=runif(npoints(finpines)))
    strike(1.5, weights=expression(y))
    strike(1.5, geometric=TRUE)
    strike(1.5, Y=finpines[FALSE])
    flatfin <- finpines %mark% data.frame(a=rep(1, npoints(finpines)), b=2)
    strike(1.5, Y=flatfin)
    strike(1.5, Y=flatfin, geometric=TRUE)
  }
  opx <- spatstat.options(densityTransform=FALSE)
  if(ALWAYS) {
    stroke(5, Y=longleaf[order(longleaf$x)], sorted=TRUE)
  }
  if(FULLTEST) {
    strike(1.5, Y=finpines[order(finpines$x)], sorted=TRUE)
  }
  spatstat.options(opx)

  ## detect special cases
  if(ALWAYS) {
    Smooth(longleaf[FALSE])
    Smooth(longleaf, minnndist(longleaf))
    Xconst <- cells %mark% 1
    Smooth(Xconst, 0.1)
    Smooth(Xconst, 0.1, at="points")
    Smooth(cells %mark% runif(42), sigma=Inf)
    Smooth(cells %mark% runif(42), sigma=Inf, at="points")
    Smooth(cells %mark% runif(42), sigma=Inf, at="points", leaveoneout=FALSE)
    Smooth(cut(longleaf, breaks=4))
  }
  
  ## code not otherwise reached
  if(ALWAYS) {
    smoothpointsEngine(cells, values=rep(1, npoints(cells)), sigma=0.2)
  }
  if(FULLTEST) {
    smoothpointsEngine(cells, values=runif(npoints(cells)), sigma=Inf)
    smoothpointsEngine(cells, values=runif(npoints(cells)), sigma=1e-16)
  }
  
  ## validity of Smooth.ppp(at='points')
  Y <- longleaf %mark% runif(npoints(longleaf), min=41, max=43)
  Z <- Smooth(Y, 5, at="points", leaveoneout=TRUE)
  rZ <- range(Z)
  if(rZ[1] < 40 || rZ[2] > 44)
    stop("Implausible results from Smooth.ppp(at=points, leaveoneout=TRUE)")

  Z <- Smooth(Y, 5, at="points", leaveoneout=FALSE)
  rZ <- range(Z)
  if(rZ[1] < 40 || rZ[2] > 44)
    stop("Implausible results from Smooth.ppp(at=points, leaveoneout=FALSE)")

  ## compare Smooth.ppp results with different algorithms
  if(ALWAYS) {
    crosscheque(Smooth(longleaf, at="points", sigma=6))
    wt <- runif(npoints(longleaf))
    crosscheque(Smooth(longleaf, at="points", sigma=6, weights=wt))
  }
  if(FULLTEST) {
    vc <- diag(c(25,36))
    crosscheque(Smooth(longleaf, at="points", varcov=vc))
    crosscheque(Smooth(longleaf, at="points", varcov=vc, weights=wt))
  }
  ## drop-dimension coding errors
  if(FULLTEST) {
    X <- longleaf
    marks(X) <- cbind(marks(X), 1)
    Z <- Smooth(X, 5)

    ZZ <- bw.smoothppp(finpines, hmin=0.01, hmax=0.012, nh=2) # reshaping problem
  }

  ## geometric-mean smoothing
  if(ALWAYS) {
    U <- Smooth(longleaf, 5, geometric=TRUE)
  }
  if(FULLTEST) {
    UU <- Smooth(X, 5, geometric=TRUE)
    V <- Smooth(longleaf, 5, geometric=TRUE, at="points")
    VV <- Smooth(X, 5, geometric=TRUE, at="points")
  }
})

reset.spatstat.options()

local({
  #' Kmeasure, second.moment.engine
  #' Expansion of window
  Zno  <- Kmeasure(redwood, sigma=0.2, expand=FALSE)
  Zyes <- Kmeasure(redwood, sigma=0.2, expand=TRUE)
  #' All code blocks
  sigmadouble <- rep(0.1, 2)
  diagmat <- diag(sigmadouble^2)
  generalmat <- matrix(c(1, 0.5, 0.5, 1)/100, 2, 2)
  Z <- Kmeasure(redwood, sigma=sigmadouble)
  Z <- Kmeasure(redwood, varcov=diagmat)
  Z <- Kmeasure(redwood, varcov=generalmat)
  A <- second.moment.calc(redwood, 0.1, what="all", debug=TRUE)
  B <- second.moment.calc(redwood, varcov=diagmat,    what="all")
  B <- second.moment.calc(redwood, varcov=diagmat,    what="all")
  D <- second.moment.calc(redwood, varcov=generalmat, what="all")
  PR <- pixellate(redwood)
  DRno  <- second.moment.calc(PR, 0.2, debug=TRUE, expand=FALSE,
                              npts=npoints(redwood), obswin=Window(redwood))
  DRyes <- second.moment.calc(PR, 0.2, debug=TRUE, expand=TRUE,
                              npts=npoints(redwood), obswin=Window(redwood))
  DR2 <- second.moment.calc(solist(PR, PR), 0.2, debug=TRUE, expand=TRUE,
                            npts=npoints(redwood), obswin=Window(redwood))
  Gmat <- generalmat * 100
  isoGauss <- function(x,y) {dnorm(x) * dnorm(y)}
  ee <- evaluate2Dkernel(isoGauss, runif(10), runif(10),
                         varcov=Gmat, scalekernel=TRUE)
  isoGaussIm <- as.im(isoGauss, square(c(-3,3)))
  gg <- evaluate2Dkernel(isoGaussIm, runif(10), runif(10),
                         varcov=Gmat, scalekernel=TRUE)
  ## experimental code
  op <- spatstat.options(developer=TRUE)
  DR <- density(redwood, 0.1)
  spatstat.options(op)
})

local({
  #' bandwidth selection
  op <- spatstat.options(n.bandwidth=8)
  bw.diggle(cells) 
  bw.diggle(cells, method="interpreted") # undocumented test
#  bw.relrisk(urkiola, hmax=20) is tested in man/bw.relrisk.Rd
  bw.relrisk(urkiola, hmax=20, method="leastsquares")
  bw.relrisk(urkiola, hmax=20, method="weightedleastsquares")
  ZX <- density(swedishpines, at="points")
  bw.pcf(swedishpines, lambda=ZX)
  bw.pcf(swedishpines, lambda=ZX,
         bias.correct=FALSE, simple=FALSE, cv.method="leastSQ")
  spatstat.options(op)
})

local({
  #' code in kernels.R
  kernames <- c("gaussian", "rectangular", "triangular",
                "epanechnikov", "biweight", "cosine", "optcosine")
  X <- rnorm(20)
  U <- runif(20)
  for(ker in kernames) {
    dX <- dkernel(X, ker)
    fX <- pkernel(X, ker)
    qU <- qkernel(U, ker)
    m0 <- kernel.moment(0, 0, ker)
    m1 <- kernel.moment(1, 0, ker)
    m2 <- kernel.moment(2, 0, ker)
    m3 <- kernel.moment(3, 0, ker)
  }
})

local({
  ## idw
  Z <- idw(longleaf, power=4)
  Z <- idw(longleaf, power=4, se=TRUE)
  ZX <- idw(longleaf, power=4, at="points")
  ZX <- idw(longleaf, power=4, at="points", se=TRUE)
  ## dodgy code blocks in densityVoronoi.R
  A <- adaptive.density(nztrees, nrep=2, f=0.5, counting=TRUE)
  B <- adaptive.density(nztrees, nrep=2, f=0.5, counting=TRUE, fixed=TRUE)
  D <- adaptive.density(nztrees, nrep=2, f=0.5, counting=FALSE)
  E <- adaptive.density(nztrees, nrep=2, f=0.5, counting=FALSE, fixed=TRUE)
  #' adaptive kernel estimation
  d10 <- nndist(nztrees, k=10)
  d10fun <- distfun(nztrees, k=10)
  d10im  <- as.im(d10fun)
  uN <- 2 * runif(npoints(nztrees))
  AA <- densityAdaptiveKernel(nztrees, bw=d10)
  BB <- densityAdaptiveKernel(nztrees, bw=d10, weights=uN)
  DD <- densityAdaptiveKernel(nztrees, bw=d10fun, weights=uN)
  EE <- densityAdaptiveKernel(nztrees, bw=d10im, weights=uN)
})

local({
  ## unnormdensity
  x <- rnorm(20) 
  d0 <- unnormdensity(x, weights=rep(0, 20))
  dneg <- unnormdensity(x, weights=c(-runif(19), 0))

  ## cases of 'intensity' etc
  a <- intensity(amacrine, weights=expression(x))
  a <- intensity(split(amacrine), weights=expression(x))
  a <- intensity(split(amacrine), weights=amacrine$x)
  a <- intensity(ppm(amacrine ~ 1))

  ## check infrastructure for 'densityfun'
  f <- densityfun(cells, 0.05)
  Z <- as.im(f)
  Z <- as.im(f, W=square(0.5))
})
reset.spatstat.options()

#'
#'      tests/diagnostique.R
#'
#'  Diagnostic tools such as diagnose.ppm, qqplot.ppm
#'
#'  $Revision: 1.6 $  $Date: 2020/04/28 12:58:26 $
#'

if(FULLTEST) {
local({
  fit <- ppm(cells ~ x)
  diagE <- diagnose.ppm(fit, type="eem")
  diagI <- diagnose.ppm(fit, type="inverse")
  diagP <- diagnose.ppm(fit, type="Pearson")
  plot(diagE, which="all")
  plot(diagI, which="smooth")
  plot(diagP, which="x")
  plot(diagP, which="marks", plot.neg="discrete")
  plot(diagP, which="marks", plot.neg="contour")
  plot(diagP, which="smooth", srange=c(-5,5))
  plot(diagP, which="smooth", plot.smooth="contour")
  plot(diagP, which="smooth", plot.smooth="image")

  fitS <- ppm(cells ~ x, Strauss(0.08))
  diagES <- diagnose.ppm(fitS, type="eem", clip=FALSE)
  diagIS <- diagnose.ppm(fitS, type="inverse", clip=FALSE)
  diagPS <- diagnose.ppm(fitS, type="Pearson", clip=FALSE)
  plot(diagES, which="marks", plot.neg="imagecontour")
  plot(diagPS, which="marks", plot.neg="discrete")
  plot(diagPS, which="marks", plot.neg="contour")
  plot(diagPS, which="smooth", plot.smooth="image")
  plot(diagPS, which="smooth", plot.smooth="contour")
  plot(diagPS, which="smooth", plot.smooth="persp")
  
  #' infinite reach, not border-corrected
  fut <- ppm(cells ~ x, Softcore(0.5), correction="isotropic")
  diagnose.ppm(fut)

  #' 
  diagPX <- diagnose.ppm(fit, type="Pearson", cumulative=FALSE)
  plot(diagPX, which="y")

  #' simulation based
  e <- envelope(cells, nsim=4, savepatterns=TRUE, savefuns=TRUE)
  Plist <- rpoispp(40, nsim=5)

  qf <- qqplot.ppm(fit, nsim=4, expr=e, plot.it=FALSE)
  print(qf)
  qp <- qqplot.ppm(fit, nsim=5, expr=Plist, fast=FALSE)
  print(qp)
  qp <- qqplot.ppm(fit, nsim=5, expr=expression(rpoispp(40)), plot.it=FALSE)
  print(qp)
  qg <- qqplot.ppm(fit, nsim=5, style="classical", plot.it=FALSE)
  print(qg)
  
  #' lurking.ppm
  #' covariate is numeric vector
  fitx <- ppm(cells ~ x)
  yvals <- coords(as.ppp(quad.ppm(fitx)))[,"y"]
  lurking(fitx, yvals)
  #' covariate is stored but is not used in model
  Z <- as.im(function(x,y){ x+y }, Window(cells))
  fitxx <- ppm(cells ~ x, data=solist(Zed=Z), allcovar=TRUE)
  lurking(fitxx, expression(Zed))
  #' envelope is a ppplist; length < nsim; glmdata=NULL
  fit <- ppm(cells ~ 1)
  stuff <- lurking(fit, expression(x), envelope=Plist, plot.sd=FALSE)
  #' plot.lurk
  plot(stuff, shade=NULL)
})
}

#'
#'  tests/discarea.R
#'
#'   $Revision: 1.3 $ $Date: 2020/04/28 12:58:26 $
#'

if(ALWAYS) {
local({
  u <- c(0.5,0.5)
  B <- owin(poly=list(x=c(0.3, 0.5, 0.7, 0.4), y=c(0.3, 0.3, 0.6, 0.8)))
  areaGain(u, cells, 0.1, exact=TRUE)
  areaGain(u, cells, 0.1, W=NULL)
  areaGain(u, cells, 0.1, W=B)

  X <- cells[square(0.4)]
  areaLoss(X, 0.1, exact=TRUE)  # -> areaLoss.diri
  areaLoss(X, 0.1, exact=FALSE) # -> areaLoss.grid
  areaLoss.poly(X, 0.1)

  areaLoss(X, 0.1, exact=FALSE, method="distmap")          # -> areaLoss.grid
  areaLoss(X, c(0.1, 0.15), exact=FALSE, method="distmap") # -> areaLoss.grid
})
}
#'
#'   tests/disconnected.R
#'
#'   disconnected linear networks
#'
#'    $Revision: 1.4 $ $Date: 2020/04/28 12:58:26 $

local({

  #'   disconnected network
  m <- simplenet$m
  m[4,5] <- m[5,4] <- m[6,10] <- m[10,6] <- m[4,6] <- m[6,4] <- FALSE
  L <- linnet(vertices(simplenet), m)
  if(FULLTEST) {
    L
    summary(L)
    is.connected(L)
    Z <- connected(L, what="components")
  }

  #' point pattern with no points in one connected component
  set.seed(42)
  X <- rpoislpp(lambda=function(x,y) { 10 * (x < 0.5)}, L)
  B <- lineardirichlet(X)
  if(FULLTEST) {
    plot(B)
    summary(B)
  }
  if(ALWAYS) {
    D <- pairdist(X)
    A <- nndist(X)
  }
  if(FULLTEST) {
    H <- nnwhich(X)
    Y <- rpoislpp(lambda=function(x,y) { 10 * (x < 0.5)}, L)
    G <- nncross(X, Y)
    J <- crossdist(X, Y)
    plot(distfun(X))  # includes evaluation of nncross(what="dist")
  }
  
  #' K functions in disconnected network
  if(ALWAYS) {
    K <- linearK(X)
    lamX <- intensity(X)
    nX <- npoints(X)
    KI <- linearKinhom(X, lambda=rep(lamX, nX))
    P <- linearpcf(X)
    PJ <- linearpcfinhom(X, lambda=rep(lamX, nX))
  }
  Y <- X %mark% factor(rep(1:2, nX)[1:nX])
  if(FULLTEST) {
    Y1 <- split(Y)[[1]]
    Y2 <- split(Y)[[2]]
    KY <- linearKcross(Y)
    PY <- linearpcfcross(Y)
    KYI <- linearKcross.inhom(Y, lambdaI=rep(intensity(Y1), npoints(Y1)),
                              lambdaJ=rep(intensity(Y2), npoints(Y2)))
    PYI <- linearpcfcross.inhom(Y, lambdaI=rep(intensity(Y1), npoints(Y1)),
                                lambdaJ=rep(intensity(Y2), npoints(Y2)))
  }

  #' internal utilities
  if(FULLTEST) {
    K <- ApplyConnected(X, linearK, rule=function(...) list())
  }
})
#'
#'   tests/dominic.R
#'
#'   Additional tests for Dominic Schuhmacher's code
#'
#'   $Revision: 1.4 $  $Date: 2020/04/28 12:58:26 $

if(ALWAYS) {   # tests C code
local({
  X <- runifpoint(10)
  Y <- runifpoint(10)

  d  <- pppdist(X, Y, type="ace", show.rprimal=TRUE)
  a <- matchingdist(d, type="ace")
  b <- matchingdist(d, type="mat")

  d2 <- pppdist(X, Y, type="spa", ccode=FALSE)
  d2 <- pppdist(X, Y, type="spa", ccode=TRUE, auction=FALSE)
  d3 <- pppdist(X, Y, type="mat", ccode=TRUE, auction=FALSE)
  d4 <- pppdist(X[FALSE], Y[FALSE], matching=TRUE, type="spa")
  d4 <- pppdist(X[FALSE], Y[FALSE], matching=FALSE, type="spa")
  d4 <- pppdist(X[FALSE], Y[FALSE], matching=TRUE, type="ace")
  d4 <- pppdist(X[FALSE], Y[FALSE], matching=FALSE, type="ace")

  m  <- pppdist.mat(X, Y, q=Inf, cutoff=0.001)
  m2 <- pppdist.mat(X[FALSE], Y[FALSE], q=Inf, cutoff=0.001)
  m3 <- pppdist.mat(X[FALSE], Y[FALSE], q=2, cutoff=0.001)

})
}



#'
#'    tests/deepeepee.R
#'
#'    Tests for determinantal point process models
#' 
#'    $Revision: 1.7 $ $Date: 2020/04/28 12:58:26 $

local({
  if(ALWAYS) {
    #' simulate.dppm
    jpines <- residualspaper$Fig1
    fit <- dppm(jpines ~ 1, dppGauss)
    set.seed(10981)
    simulate(fit, W=square(5))
  }
  if(FULLTEST) {
    #' simulate.detpointprocfamily - code blocks
    model <- dppGauss(lambda=100, alpha=.05, d=2)
    simulate(model, seed=1999, correction="border")
    u <- is.stationary(model)
    #' other methods for dppm
    kay <- Kmodel(fit)
    gee <- pcfmodel(fit)
    lam <- intensity(fit)
    arr <- reach(fit)
    pah <- parameters(fit)
  }
  #' dppeigen code blocks
  if(ALWAYS) {
    mod <- dppMatern(lambda=2, alpha=0.01, nu=1, d=2)
    uT <- dppeigen(mod, trunc=1.1,  Wscale=c(1,1), stationary=TRUE)
  }
  if(FULLTEST) {
    uF <- dppeigen(mod, trunc=1.1,  Wscale=c(1,1), stationary=FALSE)
    vT <- dppeigen(mod, trunc=0.98, Wscale=c(1,1), stationary=TRUE)
    vF <- dppeigen(mod, trunc=0.98, Wscale=c(1,1), stationary=FALSE)
  }
})
#'
#'   tests/duplicity.R
#'
#'  Tests of duplicated/multiplicity code
#'
#' $Revision: 1.8 $ $Date: 2020/04/28 12:58:26 $

if(ALWAYS) {
local({
   X <- ppp(c(1,1,0.5,1), c(2,2,1,2), window=square(3), check=FALSE)
   Y <- X %mark% factor(letters[c(3,2,4,3)])
   ZC <- X %mark% letters[c(3,2,4,3)]
   ZM <- Y %mark% matrix(c(3,2,4,3), 4, 2)
   ZD <- Y %mark% as.data.frame(marks(ZM))

   #' multiplicity
   m <- multiplicity(X)
   mf <- multiplicity(Y)
   mm <- multiplicity(ZM)
   mz <- multiplicity(ZD)
   mc <- multiplicity(ZC)
   ## default method
   kk <- c(1,2,3,1,1,2)
   mk <- multiplicity(kk)
   ml <- multiplicity(list(sin, cos, tan)[kk])
   mc <- multiplicity(c("sin", "cos", "tan")[kk])
   if(!identical(ml, mk))
     stop("multiplicity.default(<list>) disagrees with multiplicityNumeric")
   if(!identical(mc, mk))
     stop("multiplicity(<character>) disagrees with multiplicity(<numeric>)")
   ## data frame method
   df <- data.frame(x=c(1:4, 1,3,2,4, 0,0, 3,4),
                    y=factor(rep(letters[1:4], 3)))
   md <- multiplicity(df)

   ## uniquemap.ppp
   checkum <- function(X, blurb) {
     a <- uniquemap(X)
     if(any(a > seq_along(a)))
       stop(paste("uniquemap", blurb,
                  "does not respect sequential ordering"))
     return(invisible(NULL))
   }
   checkum(X, "<unmarked point pattern>")
   checkum(Y, "<multitype point pattern>")
   checkum(ZC, "<point pattern with character marks>")
   checkum(ZM, "<point pattern with matrix of marks>")
   checkum(ZD, "<point pattern with several columns of marks>")

   ## uniquemap.data.frame
   dfbase <- as.data.frame(replicate(3, sample(1:20, 10), simplify=FALSE))
   df <- dfbase[sample(1:10, 30, replace=TRUE), , drop=FALSE]
   #' faster algorithm for numeric values
   checkum(df, "<numeric data frame>")
   a <- uniquemap(df)
   #' general algorithm using 'duplicated' and 'match'
   dfletters <- as.data.frame(matrix(letters[as.matrix(df)], nrow=nrow(df)))
   checkum(dfletters, "<character data frame>")
   b <- uniquemap(dfletters)
   if(!isTRUE(all.equal(a,b)))
     stop("inconsistency between algorithms in uniquemap.data.frame")

   ## uniquemap.matrix
   M0 <- matrix(1:12, 3, 4)
   ii <- sample(1:3, 5, replace=TRUE)
   M4 <- M0[ii, , drop=FALSE]
   checkum(M4, "<integer matrix>")
   u4 <- uniquemap(M4)
   C4 <- matrix(letters[M4], 5, 4)
   uc4 <- uniquemap(C4)
   checkum(C4, "<character matrix>")
   if(!isTRUE(all.equal(u4, uc4)))
     stop("Inconsistency between algorithms in uniquemap.matrix")
   
   ## uniquemap.default
   a <- letters[c(1, 1:4, 3:2)]
   checkum(a, "<character>")
   checkum(as.list(a), "<list>")
   u1 <- uniquemap(a)
   u2 <- uniquemap(as.list(a))
   if(!isTRUE(all.equal(u1, u2)))
     stop("Inconsistency between algorithms in uniquemap.default")
})
}

#
#   nncross.R
#
#
#    $Revision: 1.31 $  $Date: 2020/01/05 00:46:13 $
#
#  Copyright (C) Adrian Baddeley, Jens Oehlschlaegel and Rolf Turner 2000-2012
#  Licence: GNU Public Licence >= 2

nncross <- function(X, Y, ...) {
  UseMethod("nncross")
}

nncross.default <- function(X, Y, ...) {
  X <- as.ppp(X, W=boundingbox)
  nncross(X, Y, ...)
}

nncross.ppp <- function(X, Y, iX=NULL, iY=NULL,
                    what = c("dist", "which"),
                    ...,
                    k = 1,
                    sortby=c("range", "var", "x", "y"),
                    is.sorted.X = FALSE,
                    is.sorted.Y = FALSE) {
  stopifnot(is.ppp(Y) || is.psp(Y))
  sortby <- match.arg(sortby)
  what   <- match.arg(what, choices=c("dist", "which"), several.ok=TRUE)
  want.dist  <- "dist" %in% what 
  want.which <- "which" %in% what
  want.both  <- want.dist && want.which

  if(!missing(k)) {
    # k can be a single integer or an integer vector
    if(length(k) == 0)
      stop("k is an empty vector")
    else if(length(k) == 1) {
      if(k != round(k) || k <= 0)
        stop("k is not a positive integer")
    } else {
      if(any(k != round(k)) || any(k <= 0))
        stop(paste("some entries of the vector",
                   sQuote("k"), "are not positive integers"))
    }
  }
  k <- as.integer(k)
  kmax <- max(k)
  nk <- length(k)

  nX <- npoints(X)
  nY <- nobjects(Y)
  ## trivial cases
  if(nX == 0 || nY == 0) {
    result <- list(dist=matrix(Inf, nrow=nX, ncol=nk),
                   which=matrix(NA_integer_, nrow=nX, ncol=nk))[what]
    result <- as.data.frame(result)
    if(ncol(result) == 1)
      result <- result[, , drop=TRUE]
    return(result)
  }
  
  # Y is a line segment pattern 
  if(is.psp(Y)) {
    if(!identical(k, 1L))
      stop("Sorry, the case k > 1 is not yet implemented for psp objects")
    return(ppllengine(X,Y,"distance")[, what])
  }

  # Y is a point pattern
  if(is.null(iX) != is.null(iY))
    stop("If one of iX, iY is given, then both must be given")
  exclude <- (!is.null(iX) || !is.null(iY))
  if(exclude) {
    stopifnot(is.integer(iX) && is.integer(iY))
    if(length(iX) != nX)
      stop("length of iX does not match the number of points in X")
    if(length(iY) != nY)
      stop("length of iY does not match the number of points in Y")
  }

  if((is.sorted.X || is.sorted.Y) && !(sortby %in% c("x", "y")))
     stop(paste("If data are already sorted,",
                "the sorting coordinate must be specified explicitly",
                "using sortby = \"x\" or \"y\""))

  # decide whether to sort on x or y coordinate
  switch(sortby,
         range = {
           WY <- as.owin(Y)
           sortby.y <- (diff(WY$xrange) < diff(WY$yrange))
         },
         var = {
           sortby.y <- (var(Y$x) < var(Y$y))
         },
         x={ sortby.y <- FALSE},
         y={ sortby.y <- TRUE}
         )

  # The C code expects points to be sorted by y coordinate.
  if(sortby.y) {
    Xx <- X$x
    Xy <- X$y
    Yx <- Y$x
    Yy <- Y$y
  } else {
    Xx <- X$y
    Xy <- X$x
    Yx <- Y$y
    Yy <- Y$x
  }
  # sort only if needed
  if(!is.sorted.X){
    oX <- fave.order(Xy)
    Xx <- Xx[oX]
    Xy <- Xy[oX]
    if(exclude) iX <- iX[oX]
  }
  if (!is.sorted.Y){
    oY <- fave.order(Yy)
    Yx <- Yx[oY]
    Yy <- Yy[oY]
    if(exclude) iY <- iY[oY]
  }

  # number of neighbours that are well-defined
  kmaxcalc <- min(nY, kmax)
  
  if(kmaxcalc == 1) {
    # ............... single nearest neighbour ..................
    # call C code
    nndv <- if(want.dist) numeric(nX) else numeric(1)
    nnwh <- if(want.which) integer(nX) else integer(1)
    if(!exclude) iX <- iY <- integer(1)

    huge <- 1.1 * diameter(boundingbox(as.rectangle(X), as.rectangle(Y)))

    z <- .C("nnXinterface",
            n1=as.integer(nX),
            x1=as.double(Xx),
            y1=as.double(Xy),
            id1=as.integer(iX),
            n2=as.integer(nY),
            x2=as.double(Yx),
            y2=as.double(Yy),
            id2=as.integer(iY),
            exclude = as.integer(exclude),
            wantdist = as.integer(want.dist),
            wantwhich = as.integer(want.which),
            nnd=as.double(nndv),
            nnwhich=as.integer(nnwh),
            huge=as.double(huge),
            PACKAGE = "spatstat")

    if(want.which) {
      nnwcode <- z$nnwhich #sic. C code now increments by 1
      if(any(uhoh <- (nnwcode == 0))) {
        warning("NA's produced in nncross()$which")
        nnwcode[uhoh] <- NA
      }
    }
  
    # reinterpret in original ordering
    if(is.sorted.X){
      if(want.dist) nndv <- z$nnd
      if(want.which) nnwh <- if(is.sorted.Y) nnwcode else oY[nnwcode]
    } else {
      if(want.dist) nndv[oX] <- z$nnd
      if(want.which) nnwh[oX] <- if(is.sorted.Y) nnwcode else oY[nnwcode]
    }

    if(want.both) return(data.frame(dist=nndv, which=nnwh))
    return(if(want.dist) nndv else nnwh)

  } else {
    # ............... k nearest neighbours ..................
    # call C code
    nndv <- if(want.dist) numeric(nX * kmaxcalc) else numeric(1)
    nnwh <- if(want.which) integer(nX * kmaxcalc) else integer(1)
    if(!exclude) iX <- iY <- integer(1)

    huge <- 1.1 * diameter(boundingbox(as.rectangle(X), as.rectangle(Y)))
  
    z <- .C("knnXinterface",
            n1=as.integer(nX),
            x1=as.double(Xx),
            y1=as.double(Xy),
            id1=as.integer(iX),
            n2=as.integer(nY),
            x2=as.double(Yx),
            y2=as.double(Yy),
            id2=as.integer(iY),
            kmax=as.integer(kmaxcalc),
            exclude = as.integer(exclude),
            wantdist = as.integer(want.dist),
            wantwhich = as.integer(want.which),
            nnd=as.double(nndv),
            nnwhich=as.integer(nnwh),
            huge=as.double(huge),
            PACKAGE = "spatstat")

    # extract results
    nnD <- z$nnd
    nnW <- z$nnwhich
    # map 0 to NA
    if(want.which && any(uhoh <- (nnW == 0))) {
      nnW[uhoh] <- NA
      if(want.dist) nnD[uhoh] <- Inf
    }
    # reinterpret indices in original ordering
    if(!is.sorted.Y) nnW <- oY[nnW]
    # reform as matrices
    NND <- if(want.dist) matrix(nnD, nrow=nX, ncol=kmaxcalc, byrow=TRUE) else 0
    NNW <- if(want.which) matrix(nnW, nrow=nX, ncol=kmaxcalc, byrow=TRUE) else 0
    if(!is.sorted.X){
      # rearrange rows to correspond to original ordering of points
      if(want.dist) NND[oX, ] <- NND
      if(want.which) NNW[oX, ] <- NNW
    }
    # the return value should correspond to the original vector k
    if(kmax > kmaxcalc) {
      # add columns of NA / Inf
      kextra <- kmax - kmaxcalc
      if(want.dist)
        NND <- cbind(NND, matrix(Inf, nrow=nX, ncol=kextra))
      if(want.which)
        NNW <- cbind(NNW, matrix(NA_integer_, nrow=nX, ncol=kextra))
    }
    if(length(k) < kmax) {
      # select only the specified columns
      if(want.dist)
        NND <- NND[, k, drop=TRUE]
      if(want.which)
        NNW <- NNW[, k, drop=TRUE]
    }

    result <- as.data.frame(list(dist=NND, which=NNW)[what])
    colnames(result) <- c(if(want.dist) paste0("dist.", k) else NULL,
                          if(want.which) paste0("which.",k) else NULL)
    if(ncol(result) == 1)
      result <- result[, , drop=TRUE]
    return(result)
  }
}

#'
#'     weightedStats.R
#'
#'   weighted versions of hist, var, median, quantile
#'
#'  $Revision: 1.3 $  $Date: 2017/06/05 10:31:58 $
#'


#'
#'    whist      weighted histogram
#'

whist <- function(x, breaks, weights=NULL) {
    N <- length(breaks)
    if(length(x) == 0) 
      h <- numeric(N+1)
    else {
      # classify data into histogram cells (breaks need not span range of data)
      cell <- findInterval(x, breaks, rightmost.closed=TRUE)
      # values of 'cell' range from 0 to N.
      nb <- N + 1L
      if(is.null(weights)) {
        ## histogram
        h <- tabulate(cell+1L, nbins=nb)
      } else {
        ##  weighted histogram
        if(!spatstat.options("Cwhist")) {
          cell <- factor(cell, levels=0:N)
          h <- unlist(lapply(split(weights, cell), sum, na.rm=TRUE))
        } else {
          h <- .Call("Cwhist",
                     as.integer(cell), as.double(weights), as.integer(nb),
                     PACKAGE = "spatstat")
        }
      }
    }
    h <- as.numeric(h)
    y <- h[2:N]
    attr(y, "low") <- h[1]
    attr(y, "high") <- h[N+1]
    return(y)
}

#' wrapper for computing weighted variance of a vector
#' Note: this includes a factor 1 - sum(v^2) in the denominator
#' where v = w/sum(w). See help(cov.wt)

weighted.var <- function(x, w, na.rm=TRUE) {
  bad <- is.na(w) | is.na(x)
  if(any(bad)) {
    if(!na.rm) return(NA_real_)
    ok <- !bad
    x <- x[ok]
    w <- w[ok]
  }
  cov.wt(matrix(x, ncol=1),w)$cov[]
}

#' weighted median

weighted.median <- function(x, w, na.rm=TRUE) {
  unname(weighted.quantile(x, probs=0.5, w=w, na.rm=na.rm))
}

#' weighted quantile

weighted.quantile <- function(x, w, probs=seq(0,1,0.25), na.rm=TRUE) {
  x <- as.numeric(as.vector(x))
  w <- as.numeric(as.vector(w))
  if(anyNA(x) || anyNA(w)) {
    ok <- !(is.na(x) | is.na(w))
    x <- x[ok]
    w <- w[ok]
  }
  stopifnot(all(w >= 0))
  if(all(w == 0)) stop("All weights are zero", call.=FALSE)
  #'
  oo <- order(x)
  x <- x[oo]
  w <- w[oo]
  Fx <- cumsum(w)/sum(w)
  #'
  result <- numeric(length(probs))
  for(i in seq_along(result)) {
    p <- probs[i]
    lefties <- which(Fx <= p)
    if(length(lefties) == 0) {
      result[i] <- x[1]
    } else {
      left <- max(lefties)
      result[i] <- x[left]
      if(Fx[left] < p && left < length(x)) {
        right <- left+1
        y <- x[left] + (x[right]-x[left]) * (p-Fx[left])/(Fx[right]-Fx[left])
        if(is.finite(y)) result[i] <- y
      }
    }
  }
  names(result) <- paste0(format(100 * probs, trim = TRUE), "%")
  return(result)
}

## transmat.R
##
## transform matrices between different spatial indexing conventions
##
##  $Revision: 1.1 $  $Date: 2015/03/04 07:13:10 $

transmat <- local({

  euro <- matrix(c(0,-1,1,0), 2, 2)
  spat <- matrix(c(0,1,1,0), 2, 2)
  cart <- diag(c(1,1))
  dimnames(euro) <- dimnames(spat) <- dimnames(cart) <- 
    list(c("x","y"), c("i","j"))

  known <- list(spatstat=spat,
                cartesian=cart,
                Cartesian=cart,
                european=euro,
                European=euro)

  cmap <- list(x=c(1,0),
               y=c(0,1),
               i=c(1,0),
               j=c(0,1))
  
  maptocoef <- function(s) { 
    e <- parse(text=s)[[1]]
    eval(eval(substitute(substitute(f, cmap), list(f=e)))) 
  }

  
  as.convention <- function(x) {
    if(is.character(x) && length(x) == 1) {
      k <- pmatch(x, names(known))
      if(is.na(k)) 
        stop(paste("Unrecognised convention", sQuote(x)), call.=FALSE)
      return(known[[k]])
    }
    if(is.list(x) && is.character(unlist(x))) {
      xx <- lapply(x, maptocoef)
      if(all(c("x", "y") %in% names(xx))) z <- rbind(xx$x, xx$y) else
      if(all(c("i", "j") %in% names(xx))) z <- cbind(xx$x, xx$y) else 
      stop("entries should be named i,j or x,y", call.=FALSE)
      dimnames(z) <- list(c("x","y"), c("i","j"))
      if(!(all(z == 0 | z == 1 | z == -1) && 
           all(rowSums(abs(z)) == 1) && 
           all(colSums(abs(z)) == 1)))
        stop("Illegal convention", call.=FALSE)
      return(z)
    }
    stop("Unrecognised format for spatial convention", call.=FALSE)
  }  

  transmat <- function(m, from, to) {
    m <- as.matrix(m)
    from <- as.convention(from)
    to <- as.convention(to)
    conv <- solve(from) %*% to
    flip <- apply(conv == -1, 2, any)
    if(flip[["i"]]) m <- m[nrow(m):1, , drop=FALSE]
    if(flip[["j"]]) m <- m[         , ncol(m):1, drop=FALSE]
    if(all(diag(conv) == 0))
       m <- t(m)
    return(m)
  }
  
  transmat
})

##
##  minnndist.R
##
## Fast versions of min(nndist(X)), max(nndist(X))
##
##  $Revision: 1.8 $  $Date: 2020/01/05 01:26:42 $

minnndist <- function(X, positive=FALSE, by=NULL) {
  stopifnot(is.ppp(X))
  if(!is.null(by)) {
    stopifnot(length(by) == npoints(X))
    if(positive) {
      retain <- !duplicated(X)
      X <- X[retain]
      by <- by[retain]
    }
    nn <- nndist(X, by=by)
    result <- aggregate(nn, by=list(from=by), min, drop=FALSE)[,-1,drop=FALSE]
    return(result)
  }
  n <- npoints(X)
  if(n <= 1) return(NA)
  x <- X$x
  y <- X$y
  o <- fave.order(y)
  big <- sqrt(.Machine$double.xmax)
  if(positive) {
      z <- .C("minPnnd2",
              n = as.integer(n),
              x = as.double(x[o]),
              y = as.double(y[o]),
              as.double(big),
              result = as.double(numeric(1)),
              PACKAGE = "spatstat")
  } else {
      z <- .C("minnnd2",
              n = as.integer(n),
              x = as.double(x[o]),
              y = as.double(y[o]),
              as.double(big),
              result = as.double(numeric(1)),
              PACKAGE = "spatstat")
  }
  return(sqrt(z$result))
}

maxnndist <- function(X, positive=FALSE, by=NULL) {
  stopifnot(is.ppp(X))
  if(!is.null(by)) {
    stopifnot(length(by) == npoints(X))
    if(positive) {
      retain <- !duplicated(X)
      X <- X[retain]
      by <- by[retain]
    }
    nn <- nndist(X, by=by)
    result <- aggregate(nn, by=list(from=by), max, drop=FALSE)[,-1,drop=FALSE]
    return(result)
  }
  n <- npoints(X)
  if(n <= 1) return(NA)
  x <- X$x
  y <- X$y
  o <- fave.order(y)
  big <- sqrt(.Machine$double.xmax)
  if(positive) {
      z <- .C("maxPnnd2",
              n = as.integer(n),
              x = as.double(x[o]),
              y = as.double(y[o]),
              as.double(big),
              result = as.double(numeric(1)),
              PACKAGE = "spatstat")
  } else {
      z <- .C("maxnnd2",
              n = as.integer(n),
              x = as.double(x[o]),
              y = as.double(y[o]),
              as.double(big),
              result = as.double(numeric(1)),
              PACKAGE = "spatstat")
  }
  return(sqrt(z$result))
}

#
#     ewcdf.R
#
#     $Revision: 1.19 $  $Date: 2019/10/14 08:40:51 $
#
#  With contributions from Kevin Ummel
#

ewcdf <- function(x, weights=NULL, normalise=TRUE, adjust=1)
{
  nx <- length(x)
  nw <- length(weights)
  weighted <- (nw > 0)

  if(weighted) {
    check.nvector(weights, things="entries of x", oneok=TRUE)
    stopifnot(all(weights >= 0))
    if(nw == 1) 
      weights <- rep(weights, nx)
  }
  
  ## remove NA's
  nbg <- is.na(x) 
  x <- x[!nbg]
  if(weighted) weights <- weights[!nbg]
  n <- length(x)
  if (n < 1)
    stop("'x' must have 1 or more non-missing values")

  ## sort in increasing order of x value
  if(!weighted) {
    x <- sort(x)
    w <- rep(1, n)
  } else {
    ox <- fave.order(x)
    x <- x[ox]
    w <- weights[ox]
  }
  ## find jump locations and match
  rl <- rle(x)
  vals <- rl$values
  if(!weighted) {
    wmatch <- rl$lengths
  } else {
    nv <- length(vals)
    wmatch <- .C("tabsumweight",
                 nx=as.integer(n),
                 x=as.double(x),
                 w=as.double(w),
                 nv=as.integer(nv),
                 v=as.double(vals),
                 z=as.double(numeric(nv)),
                 PACKAGE="spatstat")$z
  }
  ## cumulative weight in each interval
  cumwt <- cumsum(wmatch)
  totwt <- sum(wmatch)
  ## rescale ?
  if(normalise) {
    cumwt <- cumwt/totwt
    totwt <- 1
  } else if(adjust != 1) {
    cumwt <- adjust * cumwt
    totwt <- adjust * totwt
  }
  ## make function
  rval <- approxfun(vals, cumwt,
                    method = "constant", yleft = 0, yright = totwt,
                    f = 0, ties = "ordered")
  class(rval) <- c("ewcdf",
                   if(normalise) "ecdf" else NULL,
                   "stepfun", class(rval))
  assign("w", w, envir=environment(rval))
  attr(rval, "call") <- sys.call()
  return(rval)
}

  # Hacked from stats:::print.ecdf
print.ewcdf <- function (x, digits = getOption("digits") - 2L, ...) {
  cat("Weighted empirical CDF \nCall: ")
  print(attr(x, "call"), ...)
  env <- environment(x)
  xx <- get("x", envir=env)
  ww <- get("w", envir=env)
  n <- length(xx)
  i1 <- 1L:min(3L, n)
  i2 <- if (n >= 4L) max(4L, n - 1L):n else integer()
  numform <- function(x) paste(formatC(x, digits = digits), collapse = ", ")
  cat(" x[1:", n, "] = ", numform(xx[i1]), if (n > 3L) 
      ", ", if (n > 5L) 
      " ..., ", numform(xx[i2]), "\n", sep = "")
  cat(" weights[1:", n, "] = ", numform(ww[i1]), if (n > 3L) 
      ", ", if (n > 5L) 
      " ..., ", numform(ww[i2]), "\n", sep = "")
  invisible(x)
}

quantile.ewcdf <- function(x, probs=seq(0,1,0.25), names=TRUE, ...,
                           normalise=TRUE, type=1) {
  trap.extra.arguments(..., .Context="quantile.ewcdf")
  if(!(type %in% c(1,2)))
    stop("Only quantiles of type 1 and 2 are implemented", call.=FALSE)
  env <- environment(x)
  xx <- get("x", envir=env)
  n <- length(xx)
  Fxx <- get("y", envir=env)
  maxFxx <- max(Fxx)
  eps <- 100 * .Machine$double.eps
  if(normalise) {
    Fxx <- Fxx/maxFxx
    maxp <- 1
  } else {
    maxp <- maxFxx
  }
  if(any((p.ok <- !is.na(probs)) &
         (probs/maxp < -eps | probs/maxp > 1 + eps))) {
    allowed <- if(normalise) "[0,1]" else
               paste("permitted range", prange(c(0, maxp)))
    stop(paste("'probs' outside", allowed), call.=FALSE)
  }
  if (na.p <- any(!p.ok)) {
    o.pr <- probs
    probs <- probs[p.ok]
    probs <- pmax(0, pmin(maxp, probs))
  }
  np <- length(probs)
  if (n > 0 && np > 0) {
    qs <- numeric(np)
    if(type == 1) {
      ## right-continuous inverse
      for(k in 1:np) qs[k] <- xx[min(which(Fxx >= probs[k]))]
    } else {
      ## average of left and right continuous
      for(k in 1:np) {
        pk <- probs[k]
        ik <- min(which(Fxx >= probs[k]))
        qs[k] <- if(Fxx[ik] > pk) (xx[ik] + xx[ik-1L])/2 else xx[ik]
      }
    }
  } else {
    qs <- rep(NA_real_, np)
  }
  if (names && np > 0L) {
    dig <- max(2L, getOption("digits"))
    if(normalise) {
      probnames <-
        if(np < 100) formatC(100 * probs, format="fg", width=1, digits=dig) else
        format(100 * probs, trim = TRUE, digits = dig)
      names(qs) <- paste0(probnames, "%")
    } else {
      names(qs) <-
        if(np < 100) formatC(probs, format="fg", width=1, digits=dig) else
        format(probs, trim=TRUE, digits=dig)
    }
  }
  if (na.p) {
    o.pr[p.ok] <- qs
    names(o.pr) <- rep("NA", length(o.pr))
    names(o.pr)[p.ok] <- names(qs)
    o.pr
  } else qs
}

#
#
#      quadscheme.S
#
#      $Revision: 4.35 $    $Date: 2016/02/11 10:17:12 $
#
#      quadscheme()    generate a quadrature scheme from 
#		       data and dummy point patterns.
#
#      quadscheme.spatial()    case where both patterns are unmarked
#
#      quadscheme.replicated() case where data are multitype
#
#
#---------------------------------------------------------------------

quadscheme <- function(data, dummy, method="grid", ...) {
        #
	# generate a quadrature scheme from data and dummy patterns.
	#
	# Other arguments control how the quadrature weights are computed
        #

  data <- as.ppp(data)

  if(missing(dummy)) {
    # create dummy points
    dummy <- default.dummy(data, method=method, ...)
    # extract full set of parameters used to create dummy points
    dp <- attr(dummy, "dummy.parameters")
    # extract recommended parameters for computing weights
    wp <- attr(dummy, "weight.parameters")
  } else {
    # user-supplied dummy points
    if(!is.ppp(dummy)) {
      # convert to ppp object
      dummy <- as.ppp(dummy, data$window, check=FALSE)
      # confine dummy points to data window 
      dummy <- dummy[data$window]
      wp <- dp <- list()
    } else {
      # if it's already a ppp, it may have been created by default.dummy
     dp <- attr(dummy, "dummy.parameters")
     wp <- attr(dummy, "weight.parameters")     
   }
  }
  # arguments supplied directly to quadscheme()
  # override any arguments passed as attributes
  wp <- resolve.defaults(list(method=method), list(...), wp)
  
  mX <- is.marked(data)
  mD <- is.marked(dummy)

  if(!mX && !mD)
    Q <- do.call(quadscheme.spatial,
                 append(list(data, dummy, check=FALSE), wp))
  else if(mX && !mD)
    Q <- do.call(quadscheme.replicated,
                 append(list(data, dummy, check=FALSE), wp))
  else if(!mX && mD)
    stop("dummy points are marked but data are unmarked")
  else
    stop("marked data and marked dummy points -- sorry, this case is not implemented")

  # record parameters used to make dummy points
  Q$param$dummy <- dp

  return(Q)
}

quadscheme.spatial <-
  function(data, dummy, method=c("grid", "dirichlet"), ...) {
        #
	# generate a quadrature scheme from data and dummy patterns.
	#
	# The 'method' may be "grid" or "dirichlet"
	#
	# '...' are passed to gridweights() or dirichletWeights()
        #
        # quadscheme.spatial:
        #       for unmarked point patterns.
        #
        #       weights are determined only by spatial locations
        #       (i.e. weight computations ignore any marks)
	#
        # No two points should have the same spatial location
        # 

    check <- resolve.defaults(list(...), list(check=TRUE))$check
    method <- match.arg(method)
    
    data <- as.ppp(data, check=check)
    dummy <- as.ppp(dummy, data$window, check=check)
    # note data$window is the DEFAULT quadrature window
    # applicable when 'dummy' does not contain a window

    if(is.marked(data, dfok=TRUE))
      warning("marks in data pattern - ignored")
    if(is.marked(dummy, dfok=TRUE))
      warning("marks in dummy pattern - ignored")
    
    both <- as.ppp(concatxy(data, dummy), dummy$window, check=check)
    switch(method,
           grid={
             w <- gridweights(both, window= dummy$window, ...)
           },
           dirichlet = {
             w <- dirichletWeights(both, window=dummy$window, ...)
           },
           { 
             stop(paste("unrecognised method", sQuote(method)))
           }
           )
    # parameters actually used to make weights
    wp <- attr(w, "weight.parameters")
    param <- list(weight = wp, dummy = NULL)
    
    Q <- quad(data, dummy, w, param)
    return(Q)
  }

"quadscheme.replicated" <-
  function(data, dummy, method=c("grid", "dirichlet"), ...) {
    ##
    ## generate a quadrature scheme from data and dummy patterns.
    ##
    ## The 'method' may be "grid" or "dirichlet"
    ##
    ## '...' are passed to gridweights() or dirichletWeights()
    ##
    ## quadscheme.replicated:
    ##       for multitype point patterns.
    ##
    ## No two points in 'data'+'dummy' should have the same spatial location

    check <- resolve.defaults(list(...), list(check=TRUE))$check
    method <- match.arg(method)
    
    data <- as.ppp(data, check=check)
    dummy <- as.ppp(dummy, data$window, check=check)
		## note data$window is the DEFAULT quadrature window
		## unless otherwise specified in 'dummy'
    ndata <- data$n
    ndummy <- dummy$n

    if(!is.marked(data))
      stop("data pattern does not have marks")
    if(is.marked(dummy, dfok=TRUE) && npoints(dummy) > 0)
      warning("dummy points have marks --- ignored")

    ## first, ignore marks and compute spatial weights
    P <- quadscheme.spatial(unmark(data), dummy, method, ...)
    W <- w.quad(P)
    iz <- is.data(P)
    Wdat <- W[iz]
    Wdum <- W[!iz]

    ## find the set of all possible marks

    if(!is.multitype(data))
      stop("data pattern is not multitype")
    data.marks <- marks(data)
    markset <- levels(data.marks)
    nmarks <- length(markset)
    
    ## replicate dummy points, one copy for each possible mark
    ## -> dummy x {1,..,K}
        
    dumdum <- cartesian(dummy, markset)
    Wdumdum <- rep.int(Wdum, nmarks)
    Idumdum <- rep.int(ndata + seq_len(ndummy), nmarks)
        
    ## also make dummy marked points at same locations as data points
    ## but with different marks

    dumdat <- cartesian(unmark(data), markset)
    Wdumdat <- rep.int(Wdat, nmarks)
    Mdumdat <- marks(dumdat)
    Idumdat <- rep.int(1:ndata, nmarks)
        
    Mrepdat <- rep.int(data.marks, nmarks)

    ok <- (Mdumdat != Mrepdat)
    dumdat <- dumdat[ok,]
    Wdumdat <- Wdumdat[ok]
    Idumdat <- Idumdat[ok]

    ## combine the two dummy patterns
    dumb <- superimpose(dumdum, dumdat, W=dummy$window, check=FALSE)
    Wdumb <- c(Wdumdum, Wdumdat)
    Idumb <- c(Idumdum, Idumdat)
    
    ## record the quadrature parameters
    param <- list(weight = P$param$weight,
                  dummy = NULL,
                  sourceid=c(1:ndata, Idumb))

    ## wrap up
    Q <- quad(data, dumb, c(Wdat, Wdumb), param)
    return(Q)
}


"cartesian" <-
function(pp, markset, fac=TRUE) {
  ## given an unmarked point pattern 'pp'
  ## and a finite set of marks,
  ## create the marked point pattern which is
  ## the Cartesian product, consisting of all pairs (u,k)
  ## where u is a point of 'pp' and k is a mark in 'markset'
  nmarks <- length(markset)
  result <- ppp(rep.int(pp$x, nmarks),
                rep.int(pp$y, nmarks),
                window=pp$window,
                check=FALSE)
  marx <- rep.int(markset, rep.int(pp$n, nmarks))
  if(fac)
    marx <- factor(marx, levels=markset)
  marks(result) <- marx
  return(result)
}


validate.quad <- function(Q, fatal=FALSE, repair=TRUE, announce=FALSE) {
  X <- Q$data
  D <- Q$dummy
  mX <- is.marked(X)
  mD <- is.marked(D)
  nbg <- function(whinge, fatal=FALSE, announce=FALSE) {
    if(fatal)
      stop(whinge, call.=FALSE)
    else {
      if(announce)
        warning(whinge, call.=FALSE)
      return(FALSE)
    }
  }
  if(mX != mD) {
    whinge <-
      if(mX)
        "data points are marked, but dummy points are not"
      else
        "dummy points are marked, but data points are not"
    return(nbg(whinge, fatal, announce))
  }
  if(!mX)
    return(TRUE)
  # marked points 
  fX <- is.factor(Xmarx <- marks(X))
  fD <- is.factor(Dmarx <- marks(D))
  if(fX != fD) {
    whinge <-
      if(fX)
        "data points are multitype, but dummy points are not"
      else
        "dummy points are multitype, but data points are not"
    return(nbg(whinge, fatal, announce))
  }
  if(!fX)
    return(TRUE)
  # multitype points
  lX <- levels(Xmarx)
  lD <- levels(Dmarx)
  if(length(lX) != length(lD) || any(lX != lD)) {
    whinge <- "data and dummy points have different sets of possible marks"
    return(nbg(whinge, fatal, announce))
  }
  return(TRUE)
}

  

pixelquad <- function(X, W=as.owin(X)) {
  ## make a quadscheme with a dummy point at every pixel
  verifyclass(X, "ppp")
  
  ## convert window to mask if not already one
  W <- as.owin(W)
  M <- as.mask(W)
  MM <- M$m
  pixelarea <- M$xstep * M$ystep
  
  ## create pixel coordinates and corresponding row, column indices
  rxy <- rasterxy.mask(M, drop=TRUE)
  xx <- rxy$x
  yy <- rxy$y
  cc <- as.vector(col(MM)[MM])
  rr <- as.vector(row(MM)[MM])
  Nr <- M$dim[1]
  Nc <- M$dim[2]
  
  ## dummy point pattern
  dum <- ppp(xx, yy, window=W, check=FALSE)
  
  ## discretise data points
  ij <- nearest.raster.point(X$x, X$y, M)
  ijrow <- ij$row
  ijcol <- ij$col


  if(!is.marked(X)) {
    ## tabulate pixel locations of data points
    Xtab <- table(row=factor(ijrow, levels=1:Nr),
                  col=factor(ijcol, levels=1:Nc))
    ## every pixel contains exactly one dummy point,
    ## so the total count of quadrature points in each pixel is:
    Qtab <- Xtab + 1
    ## compute counting weights for data points
    wdat <- 1/Qtab[cbind(ijrow, ijcol)]
    ## compute counting weights for dummy points
    wdum <- 1/Qtab[cbind(rr, cc)]
  } else {
    marx <- marks(X)
    ## tabulate pixel locations and marks of data points
    Xtab <- table(row=factor(ijrow, levels=1:Nr),
                  col=factor(ijcol, levels=1:Nc),
                  mark=marx)
    ## replicate dummy points (pixel centres) for each mark
    dum <- cartesian(dum, levels(marx))
    ## every marked pixel contains exactly one dummy point,
    ## so the total count of quadrature points in each marked pixel is:
    Qtab <- Xtab + 1
    ## compute counting weights for data points
    wdat <- 1/Qtab[cbind(ijrow, ijcol, as.integer(marx))]
    ## compute counting weights for dummy points
    nm <- length(levels(marx))
    wdum <- 1/Qtab[cbind(rep.int(rr, nm),
                         rep.int(cc, nm),
                         rep(1:nm, each=length(rr)))]
  }
  ## create quadrature scheme
  wboth <- pixelarea * c(wdat, wdum)
  Q <- quad(X, dum, wboth)
  
  attr(Q, "M") <- M
  return(Q)
}

  
  

##
##    random.R
##
##    Functions for generating random point patterns
##
##    $Revision: 4.97 $   $Date: 2019/04/05 03:41:12 $
##
##
##    runifpoint()      n i.i.d. uniform random points ("binomial process")
##
##    runifpoispp()     uniform Poisson point process
##
##    rpoispp()         general Poisson point process (thinning method)
##
##    rpoint()          n independent random points (rejection/pixel list)
##
##    rMaternI()        Mat'ern model I 
##    rMaternII()       Mat'ern model II
##    rSSI()            Simple Sequential Inhibition process
##
##    rthin()           independent random thinning
##    rjitter()         random perturbation
##
##    Examples:
##          u01 <- owin(0:1,0:1)
##          plot(runifpoispp(100, u01))
##          X <- rpoispp(function(x,y) {100 * (1-x/2)}, 100, u01)
##          X <- rpoispp(function(x,y) {ifelse(x < 0.5, 100, 20)}, 100)
##          plot(X)
##          plot(rMaternI(100, 0.02))
##          plot(rMaternII(100, 0.05))
##

simulationresult <- function(resultlist, nsim, drop, NameBase="Simulation") {
  if(nsim == 1 && drop)
    return(resultlist[[1L]])
  #' return 'solist' if appropriate, otherwise 'anylist'
  return(as.solist(resultlist, .NameBase=NameBase, demote=TRUE))
}

runifrect <- function(n, win=owin(c(0,1),c(0,1)), nsim=1, drop=TRUE)
{
  ## no checking
  xr <- win$xrange
  yr <- win$yrange
  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    x <- runif(n, min=xr[1], max=xr[2])
    y <- runif(n, min=yr[1], max=yr[2])
    result[[isim]] <- ppp(x, y, window=win, check=FALSE)
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

runifdisc <- function(n, radius=1, centre=c(0,0), ..., nsim=1, drop=TRUE)
{
  ## i.i.d. uniform points in the disc of radius r and centre (x,y)
  check.1.real(radius)
  stopifnot(radius > 0)
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  disque <- disc(centre=centre, radius=radius, ...)
  twopi <- 2 * pi
  rad2 <- radius^2
  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    theta <- runif(n, min=0, max=twopi)
    s <- sqrt(runif(n, min=0, max=rad2))
    result[[isim]] <- ppp(centre[1] + s * cos(theta),
                          centre[2] + s * sin(theta),
                          window=disque, check=FALSE)
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}


runifpoint <- function(n, win=owin(c(0,1),c(0,1)),
                       giveup=1000, warn=TRUE, ...,
                       nsim=1, drop=TRUE, ex=NULL)
{
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  
  if(missing(n) && missing(win) && !is.null(ex)) {
    stopifnot(is.ppp(ex))
    n <- npoints(ex)
    win <- Window(ex)
  } else {
    win <- as.owin(win)
    check.1.integer(n)
    stopifnot(n >= 0)
  }

  if(n == 0) {
    emp <- ppp(window=win)
    result <- rep(list(emp), nsim)
    result <- simulationresult(result, nsim, drop)
    return(result)
  }

  if(warn) {
    nhuge <- spatstat.options("huge.npoints")
    if(n > nhuge) {
      whinge <- paste("Attempting to generate", n, "random points")
      message(whinge)
      warning(whinge, call.=FALSE)
    }
  }

  switch(win$type,
         rectangle = {
           return(runifrect(n, win, nsim=nsim, drop=drop))
         },
         mask = {
           dx <- win$xstep
           dy <- win$ystep
           ## extract pixel coordinates and probabilities
           rxy <- rasterxy.mask(win, drop=TRUE)
           xpix <- rxy$x
           ypix <- rxy$y
           ## make a list of nsim point patterns
           result <- vector(mode="list", length=nsim)
           for(isim in 1:nsim) {
             ## select pixels with equal probability
             id <- sample(seq_along(xpix), n, replace=TRUE)
             ## extract pixel centres and randomise within pixels
             x <- xpix[id] + runif(n, min= -dx/2, max=dx/2)
             y <- ypix[id] + runif(n, min= -dy/2, max=dy/2)
             result[[isim]] <- ppp(x, y, window=win, check=FALSE)
           }
         },
         polygonal={
           ## make a list of nsim point patterns
           result <- vector(mode="list", length=nsim)
           for(isim in 1:nsim) {
             ## rejection method
             ## initialise empty pattern
             x <- numeric(0)
             y <- numeric(0)
             X <- ppp(x, y, window=win)
             ##
             ## rectangle in which trial points will be generated
             box <- boundingbox(win)
             ## 
             ntries <- 0
             repeat {
               ntries <- ntries + 1
               ## generate trial points in batches of n
               qq <- runifrect(n, box) 
               ## retain those which are inside 'win'
               qq <- qq[win]
               ## add them to result
               X <- superimpose(X, qq, W=win, check=FALSE)
               ## if we have enough points, exit
               if(X$n > n) {
                 result[[isim]] <- X[1:n]
                 break
               } else if(X$n == n) {
                 result[[isim]] <- X
                 break
               } else if(ntries >= giveup) {
                 ## otherwise get bored eventually
                 stop(paste("Gave up after", giveup * n, "trials,",
                            X$n, "points accepted"))
               }
             }
           }
         },
         stop("Unrecognised window type")
         )
  
  ## list of point patterns produced.
  result <- simulationresult(result, nsim, drop)
  return(result)
}

runifpoispp <- function(lambda, win = owin(c(0,1),c(0,1)), ...,
                        nsim=1, drop=TRUE) {
  win <- as.owin(win)
  if(!is.numeric(lambda) || length(lambda) > 1 ||
     !is.finite(lambda) || lambda < 0)
    stop("Intensity lambda must be a single finite number >= 0")
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }

  if(lambda == 0) {
    ## return empty pattern
    emp <- ppp(window=win)
    result <- rep(list(emp), nsim)
    result <- simulationresult(result, nsim, drop)
    return(result)
  }

  ## will generate Poisson process in enclosing rectangle and trim it
  box <- boundingbox(win)
  meanN <- lambda * area(box)
  
  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    n <- rpois(1, meanN)
    if(!is.finite(n))
      stop(paste("Unable to generate Poisson process with a mean of",
                 meanN, "points"))
    X <- runifpoint(n, box)
    ## trim to window
    if(win$type != "rectangle")
      X <- X[win]
    result[[isim]] <- X
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

rpoint <- function(n, f, fmax=NULL,
                   win=unit.square(), ..., giveup=1000,verbose=FALSE,
                   nsim=1, drop=TRUE) {
  
  if(missing(f) || (is.numeric(f) && length(f) == 1))
    ## uniform distribution
    return(runifpoint(n, win, giveup, nsim=nsim, drop=drop))
  
  ## non-uniform distribution....
  if(!is.function(f) && !is.im(f))
    stop(paste(sQuote("f"),
               "must be either a function or an",
               sQuote("im"), "object"))

  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  
  if(is.im(f)) {
    ## ------------ PIXEL IMAGE ---------------------
    wf <- as.owin(f)
    if(n == 0) {
      ## return empty pattern(s)
      emp <- ppp(window=wf)
      result <- rep(list(emp), nsim)
      result <- simulationresult(result, nsim, drop)
      return(result)
    }
    w <- as.mask(wf)
    M <- w$m
    dx <- w$xstep
    dy <- w$ystep
    ## extract pixel coordinates and probabilities
    rxy <- rasterxy.mask(w, drop=TRUE)
    xpix <- rxy$x
    ypix <- rxy$y
    ppix <- as.vector(f$v[M]) ## not normalised - OK
    ##
    result <- vector(mode="list", length=nsim)
    for(isim in 1:nsim) {
      ## select pixels
      id <- sample(length(xpix), n, replace=TRUE, prob=ppix)
      ## extract pixel centres and randomise within pixels
      x <- xpix[id] + runif(n, min= -dx/2, max=dx/2)
      y <- ypix[id] + runif(n, min= -dy/2, max=dy/2)
      result[[isim]] <- ppp(x, y, window=wf, check=FALSE)
    }
    result <- simulationresult(result, nsim, drop)
    return(result)
  }

  ## ------------ FUNCTION  ---------------------  
  ## Establish parameters for rejection method

  verifyclass(win, "owin")

  if(n == 0) {
    ## return empty pattern(s)
    emp <- ppp(window=win)
    result <- rep(list(emp), nsim)
    result <- simulationresult(result, nsim, drop)
    return(result)
  }
  
  if(is.null(fmax)) {
    ## compute approx maximum value of f
    imag <- as.im(f, win, ...)
    summ <- summary(imag)
    fmax <- summ$max + 0.05 * diff(summ$range)
  }
  irregular <- (win$type != "rectangle")
  box <- boundingbox(win)

  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {

    ## initialise empty pattern
    X <- ppp(numeric(0), numeric(0), window=win)
  
    pbar <- 1
    nremaining <- n
    totngen <- 0
    
    ## generate uniform random points in batches
    ## and apply the rejection method.
    ## Collect any points that are retained in X

    ntries <- 0
    repeat{
      ntries <- ntries + 1
      ## proposal points
      ngen <- nremaining/pbar + 10
      totngen <- totngen + ngen
      prop <- runifrect(ngen, box)
      if(irregular)
        prop <- prop[win]
      if(prop$n > 0) {
        fvalues <- f(prop$x, prop$y, ...)
        paccept <- fvalues/fmax
        u <- runif(prop$n)
        ## accepted points
        Y <- prop[u < paccept]
        if(Y$n > 0) {
          ## add to X
          X <- superimpose(X, Y, W=win, check=FALSE)
          nX <- X$n
          pbar <- nX/totngen
          nremaining <- n - nX
          if(nremaining <= 0) {
            ## we have enough!
            if(verbose)
              splat("acceptance rate = ", round(100 * pbar, 2), "%")
            result[[isim]] <- if(nX == n) X else X[1:n]
            break
          }
        }
      }
      if(ntries > giveup)
        stop(paste("Gave up after",giveup * n,"trials with",
                   X$n, "points accepted"))
    }
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

rpoispp <- function(lambda, lmax=NULL, win = owin(), ...,
                    nsim=1, drop=TRUE, ex=NULL, warnwin=TRUE) {
  ## arguments:
  ##     lambda  intensity: constant, function(x,y,...) or image
  ##     lmax     maximum possible value of lambda(x,y,...)
  ##     win     default observation window (of class 'owin')
  ##   ...       arguments passed to lambda(x, y, ...)
  ##     nsim    number of replicate simulations

  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  
  if(missing(lambda) && is.null(lmax) && missing(win) && !is.null(ex)) {
    lambda <- intensity(unmark(ex))
    win <- Window(ex)
  } else {
    if(!(is.numeric(lambda) || is.function(lambda) || is.im(lambda)))
      stop(paste(sQuote("lambda"),
                 "must be a constant, a function or an image"))
    if(is.numeric(lambda) && !(length(lambda) == 1 && lambda >= 0))
      stop(paste(sQuote("lambda"),
                 "must be a single, nonnegative number"))
    if(!is.null(lmax)) {
      if(!is.numeric(lmax))
        stop("lmax should be a number")
      if(length(lmax) > 1)
        stop("lmax should be a single number")
    }
    if(is.im(lambda)) {
      if(warnwin && !missing(win))
        warning("Argument win ignored", call.=FALSE)
      win <- rescue.rectangle(as.owin(lambda))
    } else {
      win <- as.owin(win)
    }
  }
  
  if(is.numeric(lambda)) 
    ## uniform Poisson
    return(runifpoispp(lambda, win, nsim=nsim, drop=drop))

  ## inhomogeneous Poisson
  ## perform thinning of uniform Poisson
  ## determine upper bound
  if(is.null(lmax)) {
    imag <- as.im(lambda, win, ...)
    summ <- summary(imag)
    lmax <- summ$max + 0.05 * diff(summ$range)
  } 

  if(is.function(lambda)) {
    ## function lambda
    #'      runifpoispp checks 'lmax'
    result <- runifpoispp(lmax, win, nsim=nsim, drop=FALSE)
    #'      result is a 'ppplist' with appropriate names
    for(isim in 1:nsim) {
      X <- result[[isim]]
      if(X$n > 0) {
        prob <- lambda(X$x, X$y, ...)/lmax
        u <- runif(X$n)
        retain <- (u <= prob)
        result[[isim]] <- X[retain]
      }
    }
    if(nsim == 1 && drop)
       result <- result[[1L]]
    return(result)
  }

  if(is.im(lambda)) {
    ## image lambda
    if(spatstat.options("fastpois")) {
      ## new code: sample pixels directly
      mu <- integral(lambda)
      dx <- lambda$xstep/2
      dy <- lambda$ystep/2
      df <- as.data.frame(lambda)
      npix <- nrow(df)
      lpix <- df$value
      result <- vector(mode="list", length=nsim)
      nn <- rpois(nsim, mu)
      if(!all(is.finite(nn)))
        stop(paste("Unable to generate Poisson process with a mean of",
                   mu, "points"))
      for(isim in seq_len(nsim)) {
        ni <- nn[isim]
        ii <- sample.int(npix, size=ni, replace=TRUE, prob=lpix)
        xx <- df$x[ii] + runif(ni, -dx, dx)
        yy <- df$y[ii] + runif(ni, -dy, dy)
        result[[isim]] <- ppp(xx, yy, window=win, check=FALSE)
      }
      result <- simulationresult(result, nsim, drop)
      return(result)
    } else {
      ## old code: thinning
      result <- runifpoispp(lmax, win, nsim=nsim, drop=FALSE)
      for(isim in 1:nsim) {
        X <- result[[isim]]
        if(X$n > 0) {
          prob <- lambda[X]/lmax
          u <- runif(X$n)
          retain <- (u <= prob)
          result[[isim]] <- X[retain]
        }
      }
      if(nsim == 1 && drop)
         return(result[[1L]])
      return(result)
    }
  }
  stop(paste(sQuote("lambda"), "must be a constant, a function or an image"))
}
    
rMaternI <- function(kappa, r, win = owin(c(0,1),c(0,1)), stationary=TRUE,
                     ..., nsim=1, drop=TRUE)
{
  rMaternInhibition(type=1,
                    kappa=kappa, r=r, win=win, stationary=stationary,
                    ..., nsim=nsim, drop=drop)
}

rMaternII <- function(kappa, r, win = owin(c(0,1),c(0,1)), stationary=TRUE,
                     ..., nsim=1, drop=TRUE)
{
  rMaternInhibition(type=2,
                    kappa=kappa, r=r, win=win, stationary=stationary,
                    ..., nsim=nsim, drop=drop)
}

rMaternInhibition <- function(type, 
                              kappa, r, win = owin(c(0,1),c(0,1)),
                              stationary=TRUE,
                              ..., nsim=1, drop=TRUE) {
  stopifnot(is.numeric(r) && length(r) == 1)
  stopifnot(type %in% c(1,2))
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  ## Resolve window class
  if(!inherits(win, c("owin", "box3", "boxx"))) {
    givenwin <- win
    win <- try(as.owin(givenwin), silent = TRUE)
    if(inherits(win, "try-error"))
      win <- try(as.boxx(givenwin), silent = TRUE)
    if(inherits(win, "try-error"))
      stop("Could not coerce argument win to a window (owin, box3 or boxx).")
  }
  dimen <- spatdim(win)
  if(dimen == 2) {
    bigbox <- if(stationary) grow.rectangle(win, r) else win
    result <- rpoispp(kappa, win = bigbox, nsim = nsim, drop=FALSE)
  } else if(dimen == 3) {
    bigbox <- if(stationary) grow.box3(win, r) else win
    result <- rpoispp3(kappa, domain = bigbox, nsim = nsim, drop=FALSE)
  } else {
    bigbox <- if(stationary) grow.boxx(win, r) else win
    result <- rpoisppx(kappa, domain = bigbox, nsim = nsim, drop=FALSE)
  }
  for(isim in 1:nsim) {
    Y <- result[[isim]]
    nY <- npoints(Y)
    if(type == 1) {
      ## Matern Model I
      if(nY > 1) {
        d <- nndist(Y)
        Y <- Y[d > r]
      }
    } else {
      ## Matern Model II
      if(nY > 1) {
        ## matrix of squared pairwise distances
        d2 <- pairdist(Y, squared=TRUE)
        close <- (d2 <= r^2)
        ## random order 1:n
        age <- sample(seq_len(nY), nY, replace=FALSE)
        earlier <- outer(age, age, ">")
        conflict <- close & earlier
        ## delete <- apply(conflict, 1, any)
        delete <- matrowany(conflict)
        Y <- Y[!delete]
      }
    }
    if(stationary)
      Y <- Y[win]
    result[[isim]] <- Y
  }
  if(nsim == 1 && drop) return(result[[1L]])
  if(is.owin(win))
    result <- as.ppplist(result)
  return(result)
}

rSSI <- function(r, n=Inf, win = square(1), 
                 giveup = 1000, x.init=NULL, ...,
                 f=NULL, fmax=NULL,
                 nsim=1, drop=TRUE)
{
  win.given <- !missing(win) && !is.null(win)
  stopifnot(is.numeric(r) && length(r) == 1 && r >= 0)
  stopifnot(is.numeric(n) && length(n) == 1 && n >= 0)
  must.reach.n <- is.finite(n)
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  ##
  if(!is.null(f)) {
    stopifnot(is.numeric(f) || is.im(f) || is.function(f))
    if(is.null(fmax) && !is.numeric(f))
      fmax <- if(is.im(f)) max(f) else max(as.im(f, win))
  }
  ##
  result <- vector(mode="list", length=nsim)
  if(!win.given) win <- square(1)
  ## validate initial state
  if(is.null(x.init)) {
    ## start with empty pattern in specified window
    win <- as.owin(win)
    x.init <- ppp(numeric(0),numeric(0), window=win)
  } else {
    ## start with specified pattern
    stopifnot(is.ppp(x.init))
    if(!win.given) {
      win <- as.owin(x.init)
    } else {
      ## check compatibility of windows
      if(!identical(win, as.owin(x.init)))
        warning(paste("Argument", sQuote("win"),
                      "is not the same as the window of", sQuote("x.init")))
      x.init.new <- x.init[win]
      if(npoints(x.init.new) == 0)
        stop(paste("No points of x.init lie inside the specified window",
                   sQuote("win")))
      nlost <- npoints(x.init) - npoints(x.init.new)
      if(nlost > 0) 
        warning(paste(nlost, "out of",
                      npoints(x.init), "points of the pattern x.init",
                      "lay outside the specified window",
                      sQuote("win")))
      x.init <- x.init.new
    }
    if(n < npoints(x.init))
      stop(paste("x.init contains", npoints(x.init), "points",
                 "but a pattern containing only n =", n, "points", 
                 "is required"))
    if(n == npoints(x.init)) {
      warning(paste("Initial state x.init already contains", n, "points;",
                    "no further points were added"))
      result <- rep(list(x.init), nsim)
      result <- simulationresult(result, nsim, drop)
      return(result)
    }
  }
  #' validate radius
  r2 <- r^2
  if(!is.infinite(n) && (n * pi * r2/4  > area(win)))
      warning(paste("Window is too small to fit", n, "points",
                    "at minimum separation", r))
  #' start simulation 		    
  pstate <- list()
  for(isim in 1:nsim) {
    if(nsim > 1) pstate <- progressreport(isim, nsim, state=pstate)
    ## Simple Sequential Inhibition process
    ## fixed number of points
    ## Naive implementation, proposals are uniform
    X <- x.init
    ntries <- 0
    while(ntries < giveup) {
      ntries <- ntries + 1
      qq <- if(is.null(f)) runifpoint(1, win) else rpoint(1, f, fmax, win)
      dx <- qq$x[1] - X$x
      dy <- qq$y[1] - X$y
      if(all(dx^2 + dy^2 > r2)) {
        X <- superimpose(X, qq, W=win, check=FALSE)
        ntries <- 0
      }
      if(X$n >= n)
        break
    }
    if(must.reach.n && X$n < n)
      warning(paste("Gave up after", giveup,
                    "attempts with only", X$n, "points placed out of", n))
    result[[isim]] <- X
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

rPoissonCluster <-
  function(kappa, expand, rcluster, win = owin(c(0,1),c(0,1)), ...,
           lmax=NULL, nsim=1, drop=TRUE, saveparents=TRUE)
{
  ## Generic Poisson cluster process
  ## Implementation for bounded cluster radius
  ##
  ## 'rcluster' is a function(x,y) that takes the coordinates
  ## (x,y) of the parent point and generates a list(x,y) of offspring
  ##
  ## "..." are arguments to be passed to 'rcluster()'
  ##

  ## Catch old argument name rmax for expand, and allow rmax to be
  ## passed to rcluster (and then be ignored)
  if(missing(expand) && !is.null(rmax <- list(...)$rmax)){
      expand <- rmax
      f <- rcluster
      rcluster <- function(..., rmax) f(...)
  }
  win <- as.owin(win)
  
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }

  ## Generate parents in dilated window
  frame <- boundingbox(win)
  dilated <- owin(frame$xrange + c(-expand, expand),
                  frame$yrange + c(-expand, expand))
  if(is.im(kappa) && !is.subset.owin(dilated, as.owin(kappa)))
    stop(paste("The window in which the image",
               sQuote("kappa"),
               "is defined\n",
               "is not large enough to contain the dilation of the window",
               sQuote("win")))
  parentlist <- rpoispp(kappa, lmax=lmax, win=dilated, nsim=nsim)
  if(nsim == 1) parentlist <- list(parentlist)

  resultlist <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    parents <- parentlist[[isim]]
    result <- NULL
    ## generate clusters
    np <- parents$n
    if(np > 0) {
      xparent <- parents$x
      yparent <- parents$y
      for(i in seq_len(np)) {
        ## generate random offspring of i-th parent point
        cluster <- rcluster(xparent[i], yparent[i], ...)
        if(!inherits(cluster, "ppp"))
          cluster <- ppp(cluster$x, cluster$y, window=frame, check=FALSE)
        ## skip if cluster is empty
        if(cluster$n > 0) {
          ## trim to window
          cluster <- cluster[win]
          if(is.null(result)) {
            ## initialise offspring pattern and offspring-to-parent map
            result <- cluster
            parentid <- rep.int(1, cluster$n)
          } else {
            ## add to pattern
            result <- superimpose(result, cluster, W=win, check=FALSE)
            ## update offspring-to-parent map
            parentid <- c(parentid, rep.int(i, cluster$n))
          }
        }
      }
    } else {
      ## no parents - empty pattern
      result <- ppp(numeric(0), numeric(0), window=win)
      parentid <- integer(0)
    }

    if(saveparents) {
      attr(result, "parents") <- parents
      attr(result, "parentid") <- parentid
      attr(result, "expand") <- expand
    }
    
    resultlist[[isim]] <- result
  }

  result <- simulationresult(resultlist, nsim, drop)
  return(result)
}  

rGaussPoisson <- local({
  
  rGaussPoisson <- function(kappa, r, p2, win=owin(c(0,1), c(0,1)),
                            ..., nsim=1, drop=TRUE) {
    ## Gauss-Poisson process
    result <- rPoissonCluster(kappa, 1.05 * r, oneortwo,
                              win, radius=r/2, p2=p2, nsim=nsim, drop=drop)
    return(result)
  }

  oneortwo <- function(x0, y0, radius, p2) {
    if(runif(1) > p2) 
      ## one point
      return(list(x=x0, y=y0))
    ## two points
    theta <- runif(1, min=0, max=2*pi)
    return(list(x=x0+c(-1,1)*radius*cos(theta),
                y=y0+c(-1,1)*radius*sin(theta)))
  }

  rGaussPoisson
})

  
rstrat <- function(win=square(1), nx, ny=nx, k=1, nsim=1, drop=TRUE) {
  win <- as.owin(win)
  stopifnot(nx >= 1 && ny >= 1)
  stopifnot(k >= 1)
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    xy <- stratrand(win, nx, ny, k)
    Xbox <- ppp(xy$x, xy$y, win$xrange, win$yrange, check=FALSE)
    result[[isim]] <- Xbox[win]
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

xy.grid <- function(xr, yr, nx, ny, dx, dy) {
  nx.given <- !is.null(nx)
  ny.given <- !is.null(ny)
  dx.given <- !is.null(dx)
  dy.given <- !is.null(dy)
  if(nx.given && dx.given)
    stop("Do not give both nx and dx")    
  if(nx.given) {
    stopifnot(nx >= 1)
    x0 <- seq(from=xr[1], to=xr[2], length.out=nx+1)
    dx <- diff(xr)/nx
  } else if(dx.given) {
    stopifnot(dx > 0)
    x0 <- seq(from=xr[1], to=xr[2], by=dx)
    nx <- length(x0) - 1
  } else stop("Need either nx or dx")
  ## determine y grid
  if(ny.given && dy.given)
    stop("Do not give both ny and dy")    
  if(ny.given) {
    stopifnot(ny >= 1)
    y0 <- seq(from=yr[1], to=yr[2], length.out=ny+1)
    dy <- diff(yr)/ny
  } else {
    if(is.null(dy)) dy <- dx
    stopifnot(dy > 0)
    y0 <- seq(from=yr[1], to=yr[2], by=dy)
    ny <- length(y0) - 1
  }
  return(list(x0=x0, y0=y0, nx=nx, ny=ny, dx=dx, dy=dy))
}
  
rsyst <- function(win=square(1), nx=NULL, ny=nx, ..., dx=NULL, dy=dx,
                  nsim=1, drop=TRUE) {
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  win <- as.owin(win)
  xr <- win$xrange
  yr <- win$yrange
  ## determine grid coordinates 
  if(missing(ny)) ny <- NULL
  if(missing(dy)) dy <- NULL
  g <- xy.grid(xr, yr, nx, ny, dx, dy)
  x0 <- g$x0
  y0 <- g$y0
  dx <- g$dx
  dy <- g$dy
  ## assemble grid and randomise location
  xy0 <- expand.grid(x=x0, y=y0)
  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    x <- xy0$x + runif(1, min = 0, max = dx)
    y <- xy0$y + runif(1, min = 0, max = dy)
    Xbox <- ppp(x, y, xr, yr, check=FALSE)
    ## trim to window
    result[[isim]] <- Xbox[win]
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

rcellnumber <- local({

  rcellnumber <- function(n, N=10, mu=1) {
    if(missing(mu) || mu == 1) {
      z <- rCellUnit(n=n, N=N)
    } else {
      z <- replicate(n, rCellCumul(x=mu, N=N))
    }
    return(z)
  }
  
  rCellUnit <- function(n, N=10) {
    if(!missing(N)) {
      if(round(N) != N) stop("N must be an integer")
      stopifnot(is.finite(N))
      stopifnot(N > 1)
    }
    u <- runif(n, min=0, max=1)
    p0 <- 1/N
    pN <- 1/(N * (N-1))
    k <- ifelse(u < p0, 0, ifelse(u < (1 - pN), 1, N))
    return(k)
  }
  
  rCellCumul <- function(x, N=10) {
    check.1.real(x)
    n <- ceiling(x)
    if(n <= 0) return(0)
    y <- rCellUnit(n=n, N=N)
    if(n == x) return(sum(y))
    p <- x - (n-1)
    z <- sum(y[-1]) + rbinom(1, size=y[1], prob=p)
    return(z)
  }

  rcellnumber
})

rcell <- function(win=square(1), nx=NULL, ny=nx, ...,
                  dx=NULL, dy=dx, N=10, nsim=1, drop=TRUE) {
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  win <- as.owin(win)
  xr <- win$xrange
  yr <- win$yrange
  ## determine grid coordinates 
  if(missing(ny)) ny <- NULL
  if(missing(dy)) dy <- NULL
  g <- xy.grid(xr, yr, nx, ny, dx, dy)
  nx <- g$nx
  ny <- g$ny
  x0 <- g$x0
  y0 <- g$y0
  dx <- g$dx
  dy <- g$dy
  ## generate pattern(s)
  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    x <- numeric(0)
    y <- numeric(0)
    for(ix in seq_len(nx))
      for(iy in seq_len(ny)) {
        nij <- rcellnumber(1, N)
        x <- c(x, x0[ix] + runif(nij, min=0, max=dx))
        y <- c(y, y0[iy] + runif(nij, min=0, max=dy))
      }
    Xbox <- ppp(x, y, xr, yr, check=FALSE)
    result[[isim]] <- Xbox[win]
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}


thinjump <- function(n, p) {
  # equivalent to which(runif(n) < p) for constant p
  stopifnot(length(p) == 1)
  if(p <= 0) return(integer(0))
  if(p >= 1) return(seq_len(n))
  if(p > 0.5) {
    #' for retention prob > 0.5 we find the ones to discard instead
    discard <- thinjump(n, 1-p)
    retain <- if(length(discard)) -discard else seq_len(n)
    return(retain)
  }
  guessmaxlength <- ceiling(n * p + 2 * sqrt(n * p * (1-p)))
  i <- .Call("thinjumpequal",
             n, p, guessmaxlength,
             PACKAGE = "spatstat")
  return(i)
}

rthin <- function(X, P, ..., nsim=1, drop=TRUE) {
  if(!(is.ppp(X) || is.lpp(X) || is.pp3(X) || is.ppx(X) || is.psp(X)))
    stop(paste("X should be a point pattern (class ppp, lpp, pp3 or ppx)",
               "or a line segment pattern (class psp)"),
         call.=FALSE)
  if(!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  nX <- nobjects(X)
  if(nX == 0) {
    result <- rep(list(X), nsim)
    result <- simulationresult(result, nsim, drop)
    return(result)
  }

  if(is.numeric(P) && length(P) == 1 && spatstat.options("fastthin")) {
    # special algorithm for constant probability
    result <- vector(mode="list", length=nsim)
    for(isim in 1:nsim) {
      retain <- thinjump(nX, P)
      Y <- X[retain]
      ## also handle offspring-to-parent map if present
      if(!is.null(parentid <- attr(X, "parentid")))
        attr(Y, "parentid") <- parentid[retain]
      result[[isim]] <- Y
    }
    result <- simulationresult(result, nsim, drop)
    return(result)
  }

  if(is.numeric(P)) {
    ## vector of retention probabilities
    pX <- P
    if(length(pX) != nX) {
      if(length(pX) == 1)
        pX <- rep.int(pX, nX)
      else 
        stop("Length of vector P does not match number of points of X")
    }
    if(anyNA(pX))
      stop("P contains NA's")
  } else if(is.function(P)) {
    ## function - evaluate it at points of X
    if(!(is.ppp(X) || is.lpp(X)))
      stop(paste("Don't know how to apply a function to an object of class",
                 commasep(sQuote(class(X)))),
           call.=FALSE)
    pX <- if(inherits(P, c("linfun", "funxy"))) P(X, ...) else P(X$x, X$y, ...)
    if(length(pX) != nX)
      stop("Function P returned a vector of incorrect length")
    if(!is.numeric(pX))
      stop("Function P returned non-numeric values")
    if(anyNA(pX))
      stop("Function P returned some NA values")
  } else if(is.im(P)) {
    ## image - look it up
    if(!(is.ppp(X) || is.lpp(X)))
      stop(paste("Don't know how to apply image values to an object of class",
                 commasep(sQuote(class(X)))),
           call.=FALSE)
    if(!(P$type %in% c("integer", "real")))
      stop("Values of image P should be numeric")
    pX <- P[X, drop=FALSE]
    if(anyNA(pX))
      stop("some points of X lie outside the domain of image P")
  } else
  stop("Unrecognised format for P")

  if(min(pX) < 0) stop("some probabilities are negative")
  if(max(pX) > 1) stop("some probabilities are greater than 1")

  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    retain <- (runif(length(pX)) < pX)
    Y <- X[retain]
    ## also handle offspring-to-parent map if present
    if(!is.null(parentid <- attr(X, "parentid")))
      attr(Y, "parentid") <- parentid[retain]
    result[[isim]] <- Y
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}


## rjitter

rjitter <- function(X, radius, retry=TRUE, giveup=10000, ...,
                    nsim=1, drop=TRUE) {
  verifyclass(X, "ppp")
  check.1.integer(nsim)
  stopifnot(nsim >= 1)
  nX <- npoints(X)
  if(nX == 0) {
    result <- rep(list(X), nsim)
    result <- simulationresult(result, nsim, drop)
    return(result)
  }
  W <- X$window
  if(missing(radius) || is.null(radius))
    radius <- min(bw.stoyan(X), shortside(Frame(W)))
  
  result <- vector(mode="list", length=nsim)
  for(isim in 1:nsim) {
    if(!retry) {
      ## points outside window are lost
      D <- runifdisc(nX, radius=radius)
      xnew <- X$x + D$x
      ynew <- X$y + D$y
      ok <- inside.owin(xnew, ynew, W)
      result[[isim]] <- ppp(xnew[ok], ynew[ok], window=W, check=FALSE)
    } else {
      ## retry = TRUE: condition on points being inside window
      undone <- rep.int(TRUE, nX)
      triesleft <- giveup
      Xshift <- X
      while(any(undone)) {
        triesleft <- triesleft - 1
        if(triesleft <= 0) 
	  break
        Y <- Xshift[undone]
        D <- runifdisc(Y$n, radius=radius)
        xnew <- Y$x + D$x
        ynew <- Y$y + D$y
        ok <- inside.owin(xnew, ynew, W)
        if(any(ok)) {
          changed <- which(undone)[ok]
          Xshift$x[changed] <- xnew[ok]
          Xshift$y[changed] <- ynew[ok]
          undone[changed] <- FALSE
        }
      }
      result[[isim]] <- Xshift
    }
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

##
## boundingbox.R
##
## $Revision: 1.11 $ $Date: 2017/12/30 05:04:44 $

# bounding.box <- function(...) {
#  .Deprecated("boundingbox", "spatstat")
#  boundingbox(...)
# }

boundingbox <- function(...) {
  ## remove any NULL arguments
  arglist <- list(...)
  if(any(isnull <- sapply(arglist, is.null))) {
    if(length(arglist[!isnull]))
       return(do.call(boundingbox, arglist[!isnull]))
    stop("No non-null arguments given.\n")
  }
  UseMethod("boundingbox")
}

boundingbox.solist <- function(...) {
  argh <- list(...)
  issl <- sapply(argh, inherits, what="solist")
  yarg <- c(do.call(c, argh[issl]), argh[!issl])
  do.call(bbEngine, yarg)
}

boundingbox.ppp  <-
boundingbox.psp  <-
boundingbox.owin <-
boundingbox.list <-
boundingbox.linnet <-
boundingbox.lpp <-
boundingbox.im   <- function(...) {
   bbEngine(...)
}

recognise.spatstat.type <- local({

  knowntypes <- c("ppp","psp","owin","im", "lpp", "linnet")

  function(x) {
    for(kt in knowntypes)
      if(inherits(x, kt)) return(kt)
    if(is.list(x) && checkfields(x, c("x", "y"))
       && is.numeric(x$x) && is.numeric(x$y) &&
       is.vector(x$x) && is.vector(x$y) && length(x$x) == length(x$y))
        return("listxy")
    aso <- try(as.owin(x), silent=TRUE)
    if(!inherits(aso, "try-error")) return("as.owin")
    return("unknown")
  }
})

bbEngine <- local({

  bb.listxy <- function(X) owin(range(X$x), range(X$y))

  bb.linnet <- function(X) boundingbox(vertices(X))

  bb.lpp <- function(X) boundingbox(as.ppp(X))
  
  bbEngine <- function(...) {
    wins <- list(...)
    ## first detect any numeric vector arguments
    if(any(isnumvec <- unlist(lapply(wins, is.vector)) &
           unlist(lapply(wins, is.numeric)))) {
      ## invoke default method on these arguments
      bb <- do.call(boundingbox, wins[isnumvec])
      ## repack
      wins <- append(wins[!isnumvec], list(bb))
    }
    if(length(wins) > 1) {
      ## multiple arguments -- compute bounding box for each argument.
      objtype <- unlist(lapply(wins, recognise.spatstat.type))
      nbad <- sum(objtype == "unknown")
      if(nbad > 0) {
        whinge <- paste("Function boundingbox called with",
                        nbad,"unrecognised",
                        ngettext(nbad,"argument","arguments"))
        stop(whinge, call.=FALSE)
      }
      if(any(isppp <- (objtype == "ppp"))) 
        wins[isppp] <- lapply(wins[isppp], boundingbox)
      if(any(islistxy <- (objtype == "listxy")))
        wins[islistxy] <- lapply(wins[islistxy], bb.listxy)
      if(any(isnet <- (objtype == "linnet")))
        wins[isnet] <- lapply(wins[isnet], bb.linnet)
      if(any(islpp <- (objtype == "lpp")))
        wins[islpp] <- lapply(wins[islpp], bb.lpp)
      ## then convert all windows to owin
      wins <- lapply(wins, as.owin)
      ## then take bounding box of each window
      boxes <- lapply(wins, boundingbox)
      ## discard NULL values
      isnull <- unlist(lapply(boxes, is.null))
      boxes <- boxes[!isnull]
      ## take bounding box of these boxes
      xrange <- range(unlist(lapply(boxes, getElement, name="xrange")))
      yrange <- range(unlist(lapply(boxes, getElement, name="yrange")))
      W <- owin(xrange, yrange)
      ## If all of the windows have a common unit name, give
      ## that unit name to the bounding box.
      youse <- unique(t(sapply(boxes,unitname)))
      if(nrow(youse)==1) {
        ute <- unlist(youse[1L,])
        unitname(W) <- ute
      }
      return(W)
    }

    ## single argument
    w <- wins[[1L]]
    if(is.null(w))
      return(NULL)
    
    wtype <- recognise.spatstat.type(w)
    ## point pattern?
    if(wtype == "ppp")
      return(boundingbox(coords(w)))
    
    ## line segment pattern?
    if(wtype == "psp")
      return(boundingbox(endpoints.psp(w)))
    
    ## list(x,y)
    if(wtype == "listxy")
      return(bb.listxy(w))

    if(wtype == "linnet")
      w <- return(bb.linnet(w))

    if(wtype == "lpp")
      w <- return(bb.lpp(w))
    
    ## convert to window
    w <- as.owin(w)

    ## determine a tight bounding box for the window w
    switch(w$type,
           rectangle = {
             return(w)
           },
           polygonal = {
             bdry <- w$bdry
             if(length(bdry) == 0)
               return(NULL)
             xr <- range(unlist(lapply(bdry, rangeofx)))
             yr <- range(unlist(lapply(bdry, rangeofy)))
             return(owin(xr, yr, unitname=unitname(w)))
           },
           mask = {
             m <- w$m
             x <- rasterx.mask(w)
             y <- rastery.mask(w)
             xr <- range(x[m]) + c(-1,1) * w$xstep/2
             yr <- range(y[m]) + c(-1,1) * w$ystep/2
             return(owin(xr, yr, unitname=unitname(w)))
           },
           stop("unrecognised window type", w$type)
           )
  }

  rangeofx <- function(a) range(a$x)
  rangeofy <- function(a) range(a$y)
  
  bbEngine
})


boundingbox.default <- local({

  bb.listxy <- function(X) owin(range(X$x), range(X$y))

  boundingbox.default <- function(...) {
    arglist <- list(...)
    bb <- NULL
    if(length(arglist) == 0)
      return(bb)
    ## handle numeric vector arguments
    if(any(isnumvec <- unlist(lapply(arglist, is.vector)) &
           unlist(lapply(arglist, is.numeric)))) {
      nvec <- sum(isnumvec)
      if(nvec != 2)
        stop(paste("boundingbox.default expects 2 numeric vectors:",
                   nvec, "were supplied"),
             call.=FALSE)
      vecs <- arglist[isnumvec]
      x <- vecs[[1L]]
      y <- vecs[[2L]]
      bb <- if(length(x) == length(y)) owin(range(x), range(y)) else NULL
      arglist <- arglist[!isnumvec]
    }
    if(length(arglist) == 0)
      return(bb)
    ## other objects are present
    objtype <- unlist(lapply(arglist, recognise.spatstat.type))
    ## Unrecognised?
    nbad <- sum(objtype == "unknown")
    if(nbad > 0) {
      whinge <- paste("Function boundingbox called with",
                      nbad,"unrecognised",
                      ngettext(nbad,"argument","arguments"))
      stop(whinge, call.=FALSE)
    }
    if(any(aso <- (objtype == "as.owin"))) {
      ## promote objects to owin (to avoid infinite recursion!)
      arglist[aso] <- lapply(arglist[aso], as.owin)
    }
    if(any(lxy <- (objtype == "listxy"))) {
      ## handle list(x,y) objects 
      arglist[lxy] <- lapply(arglist[lxy], bb.listxy)
    }
    result <- do.call(boundingbox,
                      if(is.null(bb)) arglist else append(list(bb), arglist))
    return(result)
  }

  boundingbox.default
})

# levelset.R
#
#  $Revision: 1.5 $  $Date: 2015/01/15 07:10:37 $
#
# level set of an image

levelset <- function(X, thresh, compare="<=") {
  # force X and thresh to be evaluated in this frame
  verifyclass(X, "im")
  thresh <- thresh
  switch(compare,
         "<"  = { A <- eval.im(X < thresh) },
         ">"  = { A <- eval.im(X > thresh) },
         "<=" = { A <- eval.im(X <= thresh) },
         ">=" = { A <- eval.im(X >= thresh) },
         "==" = { A <- eval.im(X == thresh) },
         "!=" = { A <- eval.im(X != thresh) },
         stop(paste("unrecognised comparison operator", sQuote(compare))))
  W <- as.owin(eval.im(ifelse1NA(A)))
  return(W)
}

# compute owin containing all pixels where image expression is TRUE

solutionset <- function(..., envir) {
  if(missing(envir))
    envir <- parent.frame()
  A <- try(eval.im(..., envir=envir), silent=TRUE)
  if(inherits(A, "try-error"))
    A <- try(eval(..., envir=envir), silent=TRUE)
  if(inherits(A, "try-error"))
    stop("Unable to evaluate expression")
  if(!is.im(A))
    stop("Evaluating the expression did not yield a pixel image")
  if(A$type != "logical")
    stop("Evaluating the expression did not yield a logical-valued image")
  W <- as.owin(eval.im(ifelse1NA(A)))
  return(W)
}

#
#	plot.ppp.R
#
#	$Revision: 1.96 $	$Date: 2020/02/25 05:33:46 $
#
#
#--------------------------------------------------------------------------

plot.ppp <- local({

  transparencyfun <- function(n) {
    if(n <= 100) 1 else (0.2 + 0.8 * exp(-(n-100)/1000))
  }
  
  ## determine symbol map for marks of points
  default.symap.points <- function(x, ..., 
                                  chars=NULL, cols=NULL, 
                                  maxsize=NULL, meansize=NULL, markscale=NULL,
                                  markrange=NULL, marklevels=NULL) {
    marx <- marks(x)
    if(is.null(marx)) {
      ## null or constant symbol map
      ## consider using transparent colours
      if(is.null(cols) &&
         !any(c("col", "fg", "bg") %in% names(list(...))) &&
         (nx <- npoints(x)) > 100 &&
         identical(dev.capabilities()$semiTransparency, TRUE) &&
         spatstat.options("transparent"))
        cols <- rgb(0,0,0,transparencyfun(nx))
      return(symbolmap(..., chars=chars, cols=cols))
    }
    if(!is.null(dim(marx)))
      stop("Internal error: multivariate marks in default.symap.points")

    argnames <- names(list(...))
    shapegiven <- "shape" %in% argnames
    chargiven <- (!is.null(chars)) || ("pch" %in% argnames)
    assumecircles <- !(shapegiven || chargiven)
    sizegiven <- ("size" %in% argnames) ||
                 (("cex" %in% argnames) && !shapegiven)
    
    if(inherits(marx, c("Date", "POSIXt"))) {
      ## ......... marks are dates or date/times .....................
      timerange <- range(marx, na.rm=TRUE)
      shapedefault <- if(!assumecircles) list() else list(shape="circles")
      if(sizegiven) {
        g <- do.call(symbolmap,
          resolve.defaults(list(range=timerange),
                           list(...),
                           shapedefault,
                           list(chars=chars, cols=cols)))
        return(g)
      }
      ## attempt to determine a scale for the marks 
      y <- scaletointerval(marx, 0, 1, timerange)
      y <- y[is.finite(y)]
      if(length(y) == 0) return(symbolmap(..., chars=chars, cols=cols))
      scal <- mark.scale.default(y, as.owin(x), 
                                 markscale=markscale, maxsize=maxsize,
                                 meansize=meansize, 
                                 characters=chargiven)
      if(is.na(scal)) return(symbolmap(..., chars=chars, cols=cols))
      ## scale determined
      sizefun <- function(x, scal=1) {
        (scal/2) * scaletointerval(x, 0, 1, timerange)
      }
      formals(sizefun)[[2]] <- scal  ## ensures value of 'scal' is printed
      ##
      g <- do.call(symbolmap,
                   resolve.defaults(list(range=timerange),
                                    list(...),
                                    shapedefault,
                                    list(size=sizefun)))
      return(g)
    }
    if(is.numeric(marx)) {
      ## ............. marks are numeric values ...................
      marx <- marx[is.finite(marx)]
      if(is.null(markrange)) {
        #' usual case
        if(length(marx) == 0)
          return(symbolmap(..., chars=chars, cols=cols))
        markrange <- range(marx)
      } else {
        if(!all(inside.range(marx, markrange)))
          warning("markrange does not encompass the range of mark values",
                  call.=FALSE)
      }
      ## 
      if(sizegiven) {
        g <- do.call(symbolmap,
          resolve.defaults(list(range=markrange),
                           list(...),
                           if(assumecircles) list(shape="circles") else list(),
                           list(chars=chars, cols=cols)))
        return(g)
      }
      ## attempt to determine a scale for the marks 
      if(all(markrange == 0))
        return(symbolmap(..., chars=chars, cols=cols))
      scal <- mark.scale.default(marx, as.owin(x), 
                                 markscale=markscale, maxsize=maxsize,
                                 meansize=meansize,
                                 characters=chargiven)
      if(is.na(scal)) return(symbolmap(..., chars=chars, cols=cols))
      ## scale determined
      if(markrange[1] >= 0) {
        ## all marks are nonnegative
        shapedefault <-
          if(!assumecircles) list() else list(shape="circles")
        cexfun <- function(x, scal=1) { scal * x }
        circfun <- function(x, scal=1) { scal * x }
        formals(cexfun)[[2]] <- formals(circfun)[[2]] <- scal
        sizedefault <-
          if(sizegiven) list() else
          if(chargiven) list(cex=cexfun) else list(size=circfun)
      } else {
        ## some marks are negative
        shapedefault <-
          if(!assumecircles) list() else
          list(shape=function(x) { ifelse(x >= 0, "circles", "squares") })
        cexfun <- function(x, scal=1) { scal * abs(x) }
        circfun <- function(x, scal=1) { scal * abs(x) }
        formals(cexfun)[[2]] <- formals(circfun)[[2]] <- scal
        sizedefault <-
          if(sizegiven) list() else
          if(chargiven) list(cex=cexfun) else list(size=circfun)
      }
      g <- do.call(symbolmap,
                   resolve.defaults(list(range=markrange),
                                    list(...),
                                    shapedefault,
                                    sizedefault,
                                    list(chars=chars, cols=cols)))
      return(g)
    }
    ##  ...........  non-numeric marks .........................
    um <- marklevels %orifnull%
          if(is.factor(marx)) levels(marx) else sortunique(marx)
    ntypes <- length(um)
    if(!is.null(cols))
      cols <- rep.int(cols, ntypes)[1:ntypes]
    if(shapegiven && sizegiven) {
      #' values mapped to symbols (shape and size specified)
      g <- symbolmap(inputs=um, ..., cols=cols)
    } else if(!shapegiven) {
      #' values mapped to 'pch'
      chars <- default.charmap(ntypes, chars)
      g <- symbolmap(inputs=um, ..., chars=chars, cols=cols)
    } else {
      #' values mapped to symbols
      #' determine size
      scal <- mark.scale.default(rep(1, npoints(x)),
                                 Window(x), 
                                 maxsize=maxsize,
                                 meansize=meansize,
                                 characters=FALSE)
      g <- symbolmap(inputs=um, ..., size=scal, cols=cols)
    }
    return(g)
  }
                                  
  default.charmap <- function(n, ch=NULL) {
    if(!is.null(ch))
      return(rep.int(ch, n)[1:n])
    if(n <= 25)
      return(1:n)
    ltr <- c(letters, LETTERS)
    if(n <= 52)
      return(ltr[1:n])
    ## wrapped sequence of letters
    warning("Too many types to display every type as a different character")
    return(ltr[1 + (0:(n - 1) %% 52)])
  }

  ## main function
  plot.ppp <-
    function(x, main, ..., clipwin=NULL,
             chars=NULL, cols=NULL, use.marks=TRUE,
             which.marks=NULL, add=FALSE, type=c("p", "n"), 
             legend=TRUE, leg.side=c("left", "bottom", "top", "right"),
             leg.args=list(),
             symap=NULL, maxsize=NULL, meansize=NULL, markscale=NULL, zap=0.01, 
             show.window=show.all, show.all=!add, do.plot=TRUE,
             multiplot=TRUE)
{
  if(missing(main))
    main <- short.deparse(substitute(x))

  type <- match.arg(type)
  if(missing(legend)) legend <- (type == "p")

#  if(!missing(maxsize) || !missing(markscale) || !missing(meansize))
#    warn.once("circlescale",
#              "Interpretation of arguments maxsize and markscale",
#              "has changed (in spatstat version 1.37-0 and later).",
#              "Size of a circle is now measured by its diameter.")

  if(clipped <- !is.null(clipwin)) {
    stopifnot(is.owin(clipwin))
    W <- Window(x)
    clippy <- if(is.mask(W)) intersect.owin(W, clipwin) else edges(W)[clipwin]
    x <- x[clipwin]
  } else clippy <- NULL
  
  ## sensible default position
  legend <- legend && show.all
  if(legend) {
    leg.side <- match.arg(leg.side)
    vertical <- (leg.side %in% c("left", "right"))
  }
  
#  if(type == "n" || npoints(x) == 0) {
#    ## plot the window only
#    xwindow <- x$window
#    if(do.plot) 
#      do.call(plot.owin,
#              resolve.defaults(list(xwindow),
#                               list(...),
#                               list(main=main, invert=TRUE, add=add,
#                                    type=if(show.window) "w" else "n")))
#    if(is.null(symap)) symap <- symbolmap()
#    attr(symap, "bbox") <- as.rectangle(xwindow)
#    return(invisible(symap))
#  }

  ## ................................................................
  ## Handle multiple columns of marks as separate plots
  ##  (unless add=TRUE or which.marks selects a single column
  ##   or multipage = FALSE)
  if(use.marks && is.data.frame(mx <- marks(x))) {
    implied.all <- is.null(which.marks)
    want.several <- implied.all || is.data.frame(mx <- mx[,which.marks])
    do.several <- want.several && !add && multiplot
    if(do.several) {
      ## generate one plot for each column of marks
      y <- solapply(mx, setmarks, x=x)
      out <- do.call(plot,
                     resolve.defaults(list(x=y, main=main,
                                           show.window=show.window && !clipped,
                                           do.plot=do.plot,
                                           type=type),
                                      list(...),
                                      list(equal.scales=TRUE),
                                      list(panel.end=clippy),
                                      list(legend=legend,
                                           leg.side=leg.side,
                                           leg.args=leg.args),
                                      list(chars=chars, cols=cols,
                                           maxsize=maxsize,
                                           meansize=meansize,
                                           markscale=markscale,
                                           zap=zap)))
      return(invisible(out))
    } 
    if(is.null(which.marks)) {
      which.marks <- 1
      if(do.plot) message("Plotting the first column of marks")
    }
  }
  
  ## ............... unmarked, or single column of marks ....................

  ## Determine symbol map and mark values to be used
  y <- x
  if(!is.marked(x, na.action="ignore") || !use.marks) {
    ## Marks are not mapped.
    marx <- NULL
    if(is.null(symap))
      symap <- default.symap.points(unmark(x), ..., chars=chars, cols=cols)
  } else {
    ## Marked point pattern
    marx <- marks(y, dfok=TRUE)
    if(is.data.frame(marx)) {
      ## select column or take first colum
      marx <- marx[, which.marks]
      y <- setmarks(y, marx)
    }
    if(npoints(y) > 0) {
      ok <- complete.cases(as.data.frame(y))
      if(!any(ok)) {
        warning("All mark values are NA; plotting locations only.")
        if(is.null(symap))
          symap <- default.symap.points(unmark(x), ..., chars=chars, cols=cols)
      } else if(any(!ok)) {
        warning(paste("Some marks are NA;",
                      "corresponding points are omitted."))
        x <- x[ok]
        y <- y[ok]
        marx <- marks(y)
      }
    }
    ## apply default symbol map
    if(is.null(symap))
      symap <- default.symap.points(y, chars=chars, cols=cols, 
                                    maxsize=maxsize, meansize=meansize,
                                    markscale=markscale,
                                    ...)
  }
#  gtype <- symbolmaptype(symap)

  ## Determine bounding box for main plot
  BB <- as.rectangle(x)
  sick <- inherits(x, "ppp") && !is.null(rejects <- attr(x, "rejects"))
  if(sick) {
    ## Get relevant parameters
    par.direct <- list(main=main, use.marks=use.marks,
                   maxsize=maxsize, meansize=meansize, markscale=markscale)
    par.rejects <- resolve.1.default(list(par.rejects=list(pch="+")),
                                     list(...))
    par.all <- resolve.defaults(par.rejects, par.direct)
    rw <- resolve.defaults(list(...), list(rejectwindow=NULL))$rejectwindow
    ## determine window for rejects
    rwin <-
      if(is.null(rw))
        rejects$window
      else if(is.logical(rw) && rw)
        rejects$window
      else if(inherits(rw, "owin"))
        rw
      else if(is.character(rw)) {
        switch(rw,
               box={boundingbox(rejects, x)},
               ripras={ripras(c(rejects$x, x$x), c(rejects$y, x$y))},
               stop(paste("Unrecognised option: rejectwindow=", rw)))
      } else stop("Unrecognised format for rejectwindow")
    if(is.null(rwin))
      stop("Selected window for rejects pattern is NULL")
    BB <- boundingbox(BB, as.rectangle(rwin))
  }

  ## Augment bounding box with space for legend, if appropriate
  legend <- legend && (symbolmaptype(symap) != "constant") 
  if(legend) {
    ## guess maximum size of symbols
    maxsize <- invoke.symbolmap(symap, symbolmapdomain(symap),
                                corners(as.rectangle(x)),
                                add=add, do.plot=FALSE)
    sizeguess <- if(maxsize <= 0) NULL else (1.5 * maxsize)
    leg.args <- append(list(side=leg.side, vertical=vertical), leg.args)
    ## draw up layout
    legbox <- do.call.matched(plan.legend.layout,
                              append(list(B=BB, size = sizeguess,
                                          started=FALSE, map=symap),
                                     leg.args))
    ## bounding box for everything
    BB <- legbox$A
  }

  ## return now if not plotting
  attr(symap, "bbox") <- BB
  if(!do.plot)
    return(invisible(symap))
    
  ## ............. start plotting .......................
  pt <- prepareTitle(main)
  main <- pt$main
  nlines <- pt$nlines
  blankmain <- if(nlines == 0) "" else rep("  ", nlines)
  dflt <- list(cex.main=1, xlim=NULL, ylim=NULL,
               ann=FALSE, axes=FALSE, xlab="", ylab="")
  rez <- resolve.defaults(list(...), dflt)[names(dflt)]
  do.call(plot.owin,
          append(list(x=BB, type="n", add=add,
                      main=blankmain, show.all=show.all),
                 rez))
  if(sick) {
    if(show.window) {
      ## plot windows
      if(!is.null(rw)) {
        ## plot window for rejects
        rwinpardefault <- list(lty=2,lwd=1,border=1)
        rwinpars <-
          resolve.defaults(par.rejects, rwinpardefault)[names(rwinpardefault)]
        do.call(plot.owin, append(list(rwin, add=TRUE), rwinpars))
      }
      ## plot window of main pattern
      if(!clipped) {
        do.call(plot.owin,
                resolve.defaults(list(x$window, add=TRUE),
                                 list(...),
                                 list(invert=TRUE)))
      } else plot(clippy, add=TRUE, ...)
    }
    if(type != "n") {
      ## plot reject points
      do.call(plot.ppp, append(list(rejects, add=TRUE), par.all))
      warning(paste(rejects$n, "illegal points also plotted"))
    }
    ## the rest is added
    add <- TRUE
  }

  ## Now convert to bona fide point pattern
  x <- as.ppp(x)
  xwindow <- x$window

  ## Plot observation window (or at least the main title)
  do.call(plot.owin,
          resolve.defaults(list(x=xwindow,
                                add=TRUE,
                                main=main,
                                type=if(show.window && !clipped) "w" else "n",
                                show.all=show.all),
                           list(...),
                           list(invert=TRUE)))
  ## If clipped, plot visible part of original window
  if(show.window && clipped)
    plot(clippy, add=TRUE, ...)
  # else if(show.all) fakemaintitle(as.rectangle(xwindow), main, ...)

  if(type != "n") {
    ## plot symbols ##
    invoke.symbolmap(symap, marx, x, add=TRUE)
  }
  
  ## add legend
  if(legend) {
    b <- legbox$b
    legendmap <- if(length(leg.args) == 0) symap else 
                 do.call(update, append(list(object=symap), leg.args))
    do.call(plot,
            append(list(x=legendmap, main="", add=TRUE,
                        xlim=b$xrange, ylim=b$yrange),
                   leg.args))
  }
  
  return(invisible(symap))
}

plot.ppp

})


mark.scale.default <- function(marx, w, ..., markscale=NULL,
                               maxsize=NULL, meansize=NULL,
                               characters=FALSE) {
  ## establish values of markscale, maxsize, meansize
  ngiven <- (!is.null(markscale)) +
            (!is.null(maxsize)) +
            (!is.null(meansize))
  if(ngiven > 1)
     stop("Only one of the arguments markscale, maxsize, meansize",
          " should be given", call.=FALSE)
  if(ngiven == 0) {
    ## if ALL are absent, enforce the spatstat defaults
    ## (which could also be null)
    pop <- spatstat.options("par.points")
    markscale <- pop$markscale
    maxsize   <- pop$maxsize
    meansize <- pop$meansize
  }
  ## Now check whether markscale is fixed
  if(!is.null(markscale)) {
    stopifnot(markscale > 0)
    return(markscale)
  }
  # Usual case: markscale is to be determined from maximum/mean physical size
  if(is.null(maxsize) && is.null(meansize)) {
    ## compute default value of 'maxsize'
    ## guess appropriate max physical size of symbols
    bb <- as.rectangle(w)
    maxsize <- 1.4/sqrt(pi * length(marx)/area(bb))
    maxsize <- min(maxsize, diameter(bb) * 0.07)
    ## updated: maxsize now represents *diameter*
    maxsize <- 2 * maxsize
  } else {
    if(!is.null(maxsize)) stopifnot(maxsize > 0) else stopifnot(meansize > 0)
  }
  
  # Examine mark values
  absmarx <- abs(marx)
  maxabs <- max(absmarx)
  tiny <- (maxabs < 4 * .Machine$double.eps)
  if(tiny)
    return(NA)

  ## finally determine physical scale for symbols
  if(!is.null(maxsize)) {
    scal <- maxsize/maxabs
  } else {
    meanabs <- mean(absmarx)
    scal <- meansize/meanabs
  }
  if(!characters) return(scal)

  ## if using characters ('pch') we need to
  ## convert physical sizes to 'cex' values
  charsize <- max(sidelengths(as.rectangle(w)))/40
  return(scal/charsize)
}

fakemaintitle <- function(bb, main, ...) {
  ## Try to imitate effect of 'title(main=main)' above a specified box
  if(!any(nzchar(main))) return(invisible(NULL))
  bb <- as.rectangle(bb)
  x0 <- mean(bb$xrange)
  y0 <- bb$yrange[2] + length(main) * diff(bb$yrange)/12
  parnames <- c('cex.main', 'col.main', 'font.main')
  parlist <- par(parnames)
  parlist <- resolve.defaults(list(...), parlist)[parnames]
  names(parlist) <- c('cex', 'col', 'font')
  do.call.matched(text.default,
                  resolve.defaults(list(x=x0, y=y0, labels=main),
                                   parlist,    list(...)),
                  funargs=graphicsPars("text"))
  return(invisible(NULL))
}

text.ppp <- function(x, ...) {
  graphics::text.default(x=x$x, y=x$y, ...)
}

#
#   distfun.R
#
#   distance function (returns a function of x,y)
#
#   $Revision: 1.27 $   $Date: 2018/04/23 05:12:30 $
#

distfun <- function(X, ...) {
  UseMethod("distfun")
}

distfun.ppp <- function(X, ..., k=1, undef=Inf) {
  # this line forces X to be bound
  stopifnot(is.ppp(X))
  stopifnot(length(k) == 1)
  force(undef)
  g <- function(x,y=NULL) {
    Y <- xy.coords(x, y)[c("x", "y")]
    if(npoints(X) < k) rep(undef, length(Y$x)) else
    nncross(Y, X, what="dist", k=k) 
  }
  attr(g, "Xclass") <- "ppp"
  g <- funxy(g, as.rectangle(as.owin(X)))
  attr(g, "k") <- k
  attr(g, "extrargs") <- list(k=k, undef=undef)
  class(g) <- c("distfun", class(g))
  return(g)
}

distfun.psp <- function(X, ...) {
  # this line forces X to be bound
  stopifnot(is.psp(X))
  g <- function(x,y=NULL) {
    Y <-  xy.coords(x, y)[c("x", "y")]
    nncross(Y, X, what="dist")
  }
  attr(g, "Xclass") <- "psp"
  g <- funxy(g, as.rectangle(as.owin(X)))
  class(g) <- c("distfun", class(g))
  attr(g, "extrargs") <- list()
  return(g)
}

distfun.owin <- function(X, ..., invert=FALSE) {
  # this line forces X to be bound
  stopifnot(is.owin(X))
  force(invert)
  #
  P <- edges(X)
  #
  g <- function(x,y=NULL) {
    Y <-  xy.coords(x, y)
    inside <- inside.owin(Y$x, Y$y, X)
    D <- nncross(Y, P, what="dist")
    out <- if(!invert) ifelseAX(inside, 0, D) else ifelseXB(inside, D, 0)
    return(out)
  }
  attr(g, "Xclass") <- "owin"
  g <- funxy(g, as.rectangle(as.owin(X)))
  attr(g, "extrargs") <- list(invert=invert)
  class(g) <- c("distfun", class(g))
  return(g)
}

as.owin.distfun <- function(W, ..., fatal=TRUE) {
  X <- get("X", envir=environment(W))
  result <- if(is.owin(X)) as.rectangle(X) else as.owin(X, ..., fatal=fatal)
  return(result)
}

domain.distfun <- Window.distfun <- function(X, ...) { as.owin(X) }

as.im.distfun <- function(X, W=NULL, ...,
                           eps=NULL, dimyx=NULL, xy=NULL,
                           na.replace=NULL, approx=TRUE) {
  k <- attr(X, "k")
  if(approx && is.null(W) && (is.null(k) || (k == 1))) {
    # use 'distmap' for speed
    env <- environment(X)
    Xdata  <- get("X",      envir=env)
    args <- list(X=Xdata, eps=eps, dimyx=dimyx, xy=xy)
    if(is.owin(Xdata)) {
      args <- append(args, list(invert = get("invert", envir=env)))
    }
    D <- do.call(distmap, args = args)
    if(!is.null(na.replace))
      D$v[is.null(D$v)] <- na.replace
  } else if(identical(attr(X, "Xclass"), "ppp")) {
    # point pattern --- use nngrid/knngrid
    env <- environment(X)
    Xdata  <- get("X",      envir=env)
    D <- nnmap(Xdata, W=W, what="dist", k=k, 
               eps=eps, dimyx=dimyx, xy=xy, na.replace=na.replace,
               ...)
  } else {
    # evaluate function at pixel centres
    D <- as.im.function(X, W=W,
                        eps=eps, dimyx=dimyx, xy=xy, na.replace=na.replace)
  }
  return(D)
}

print.distfun <- function(x, ...) {
  xtype <- attr(x, "Xclass")
  typestring <- switch(xtype,
                       ppp="point pattern",
                       psp="line segment pattern",
                       owin="window",
                       "unrecognised object")
  objname <- switch(xtype,
                    ppp="point",
                    psp="line segment",
                    "object")
  splat("Distance function for", typestring)
  X <- get("X", envir=environment(x))
  print(X)
  if(!is.null(k <- attr(x, "k")) && k > 1)
    splat("Distance to", ordinal(k), "nearest", objname, "will be computed")
  return(invisible(NULL))
}

summary.distfun <- function(object, ...) {
  xtype <- attr(object, "Xclass")
  w <- as.owin(object)
  fundef <- attr(object, "f")
  attr(fundef, "Xclass") <- NULL
  X <- get("X", envir=environment(object))
  z <- list(xtype   = xtype,
            k       = attr(object, "k") %orifnull% 1,
            Xsumry  = summary(X),
            values  = summary(as.im(object, ...)),
            wintype = w$type,
            frame   = Frame(w),
            units   = unitname(w))
  class(z) <- "summary.distfun"
  return(z)
}

print.summary.distfun <- function(x, ...) {
  typestring <- switch(x$xtype,
                       ppp="point pattern",
                       psp="line segment pattern",
                       owin="window",
                       "unrecognised object")
  objname <- switch(x$xtype,
                    ppp="point",
                    psp="line segment",
                    "object")
  splat("Distance function for", typestring)
  if(x$k > 1)
    splat("Distance to", ordinal(x$k), "nearest", objname, "will be computed")
  windesc <- switch(x$wintype,
                    rectangle="the rectangle",
                    polygonal="a polygonal window inside the frame",
                    mask="a binary mask in the rectangle")
  unitinfo <- summary(x$units)
  sigdig <- getOption('digits')
  splat("defined in",
        windesc,
        prange(signif(x$frame$xrange, sigdig)),
        "x",
        prange(signif(x$frame$yrange, sigdig)),
        unitinfo$plural,
        unitinfo$explain
        )
  v <- x$values
  splat("\nDistance function values:")
  splat("\trange =", prange(signif(v$range, sigdig)))
#  splat("\tintegral =", signif(v$integral, sigdig))
  splat("\tmean =", signif(v$mean, sigdig))
  invisible(NULL)
}

shift.distfun <- rotate.distfun <- scalardilate.distfun <- 
  affine.distfun <- function(X, ...) {
    f <- X
    extrargs <- attr(f, "extrargs")
    if(is.null(extrargs))
      stop(paste("distfun object has outdated format;",
                 "cannot apply geometrical transformation"),
           call.=FALSE)
    Y <- get("X", envir=environment(f))
    Ynew <- do.call(.Generic, list(Y, ...))
    fnew <- do.call(distfun, append(list(Ynew), extrargs))
    return(fnew)
  }


flipxy.distfun <- reflect.distfun <- function(X) {
  f <- X
  extrargs <- attr(f, "extrargs")
  if(is.null(extrargs))
    stop(paste("distfun object has outdated format;",
               "cannot apply geometrical transformation"),
         call.=FALSE)
  Y <- get("X", envir=environment(f))
  Ynew <- do.call(.Generic, list(Y))
  fnew <- do.call(distfun, append(list(Ynew), extrargs))
  return(fnew)
}

rescale.distfun <- function(X, s, unitname) {
  if(missing(s)) s <- NULL
  if(missing(unitname)) unitname <- NULL
  f <- X
  Y <- get("X", envir=environment(f))
  Ynew <- rescale(Y, s, unitname)
  extrargs <- attr(f, "extrargs")
  if(is.null(extrargs))
    stop(paste("distfun object has outdated format;",
               "cannot rescale it"),
         call.=FALSE)
  fnew <- do.call(distfun, append(list(Ynew), extrargs))
  return(fnew)
}
  

#
#   pairs.im.R
#
#   $Revision: 1.17 $   $Date: 2020/04/04 04:22:50 $
#

pairs.listof <- pairs.solist <- function(..., plot=TRUE) {
  argh <- expandSpecialLists(list(...), special=c("solist", "listof"))
  haslines <- any(sapply(argh, inherits, what="linim"))
  names(argh) <- good.names(names(argh), "V", seq_along(argh))
  if(haslines) {
    do.call(pairs.linim, append(argh, list(plot=plot)))
  } else {
    do.call(pairs.im, append(argh, list(plot=plot)))
  }
}

pairs.im <- function(..., plot=TRUE) {
  argh <- list(...)
  cl <- match.call()
  ## unpack single argument which is a list of images
  if(length(argh) == 1) {
    arg1 <- argh[[1]]
    if(is.list(arg1) && all(unlist(lapply(arg1, is.im))))
      argh <- arg1
  }
  ## identify which arguments are images
  isim <- unlist(lapply(argh, is.im))
  nim <- sum(isim)
  if(nim == 0) 
    stop("No images provided")
  ## separate image arguments from others
  imlist <- argh[isim]
  rest   <- argh[!isim]
  ## determine image names for plotting
  imnames <- argh$labels %orifnull% names(imlist)
  if(length(imnames) != nim || !all(nzchar(imnames))) {
    #' names not given explicitly
    callednames <- paste(cl)[c(FALSE, isim, FALSE)]
    backupnames <- paste0("V", seq_len(nim))
    if(length(callednames) != nim) {
      callednames <- backupnames
    } else if(any(toolong <- (nchar(callednames) > 15))) {
      callednames[toolong] <- backupnames[toolong]
    }
    imnames <- good.names(imnames, good.names(callednames, backupnames))
  }
  ## 
  if(nim == 1) {
    ## one image: plot histogram
    Z <- imlist[[1L]]
    xname <- imnames[1L]
    do.call(hist,
            resolve.defaults(list(x=Z, plot=plot),
                             rest, 
                             list(xlab=xname,
                                  main=paste("Histogram of", xname))))
    ## save pixel values
    pixvals <- list(Z[])
    names(pixvals) <- xname
  } else {
    ## extract pixel rasters and reconcile them
    imwins <- lapply(imlist, as.owin)
    names(imwins) <- NULL
    rasta    <- do.call(intersect.owin, imwins)
    ## extract image pixel values on common raster
    pixvals <- lapply(imlist, "[.im", i=rasta, raster=rasta, drop=TRUE)
  }
  ## combine into data frame
  pixdf <- do.call(data.frame, pixvals)
  ## pairs plot
  if(plot && nim > 1)
    do.call(pairs, resolve.defaults(list(x=pixdf),
                                      rest,
                                      list(labels=imnames, pch=".")))
  labels <- resolve.defaults(rest, list(labels=imnames))$labels
  colnames(pixdf) <- labels
  class(pixdf) <- c("plotpairsim", class(pixdf))
  return(invisible(pixdf))
}

plot.plotpairsim <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  x <- as.data.frame(x)
  if(ncol(x) == 1) {
    do.call(hist.default,
            resolve.defaults(list(x=x[,1]),
                             list(...),
                             list(main=xname, xlab=xname)))
  } else {
    do.call(pairs.default,
            resolve.defaults(list(x=x),
                             list(...),
                             list(pch=".")))
  }
  return(invisible(NULL))
}

print.plotpairsim <- function(x, ...) {
  cat("Object of class plotpairsim\n")
  cat(paste("contains pixel data for", commasep(sQuote(colnames(x))), "\n"))
  return(invisible(NULL))
}

panel.image <- function(x, y, ..., sigma=NULL) {
  usr <- par("usr"); on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))
  xx <- scaletointerval(x)
  yy <- scaletointerval(y)
  p <- ppp(xx, yy, window=square(1), check=FALSE)
  plot(density(p, sigma=sigma), add=TRUE, ...)
}

panel.contour <- function(x, y, ..., sigma=NULL) {
  usr <- par("usr"); on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))
  xx <- scaletointerval(x)
  yy <- scaletointerval(y)
  p <- ppp(xx, yy, window=square(1), check=FALSE)
  Z <- density(p, sigma=sigma)
  do.call(contour,
          resolve.defaults(list(x=Z, add=TRUE),
                           list(...),
                           list(drawlabels=FALSE)))
}

panel.histogram <- function(x, ...) {
  usr <- par("usr"); on.exit(par(usr))
  par(usr = c(usr[1:2], 0, 1.5) )
  h <- hist(x, plot = FALSE)
  breaks <- h$breaks; nB <- length(breaks)
  y <- h$counts; y <- y/max(y)
  do.call(rect,
          resolve.defaults(list(xleft   = breaks[-nB],
                                ybottom = 0,
                                xright  = breaks[-1],
                                ytop    = y),
                           list(...),
                           list(col="grey")))
}

  
  
  

#
#      distan3D.R
#
#      $Revision: 1.13 $     $Date: 2017/06/05 10:31:58 $
#
#      Interpoint distances for 3D points
#
#      Methods for pairdist, nndist, nnwhich, crossdist
#

pairdist.pp3 <- function(X, ..., periodic=FALSE, squared=FALSE) {
  verifyclass(X, "pp3")
  # extract point coordinates
  xyz <- coords(X)
  n <- nrow(xyz)
  x <- xyz$x
  y <- xyz$y
  z <- xyz$z
  #   
  # special cases
  if(n == 0)
    return(matrix(numeric(0), nrow=0, ncol=0))
  else if(n == 1L)
    return(matrix(0,nrow=1L,ncol=1L))
  #
  if(!periodic) {
    Cout <- .C("D3pairdist",
               n = as.integer(n),
               x = as.double(x),
               y = as.double(y),
               z = as.double(z),
               squared = as.integer(squared),
               d = as.double(numeric(n*n)),
               PACKAGE = "spatstat")
  } else {
    b <- as.box3(X)
    wide <- diff(b$xrange)
    high <- diff(b$yrange)
    deep <- diff(b$zrange)
    Cout <- .C("D3pairPdist",
               n = as.integer(n),
               x = as.double(x),
               y = as.double(y),
               z = as.double(z),
               xwidth=as.double(wide),
               yheight=as.double(high),
               zdepth=as.double(deep),
               squared = as.integer(squared),
               d= as.double(numeric(n*n)),
               PACKAGE = "spatstat")
  }
  dout <- matrix(Cout$d, nrow=n, ncol=n)
  return(dout)
}

nndist.pp3 <- function(X, ..., k=1, by=NULL) {
  verifyclass(X, "pp3")

  if((narg <- length(list(...))) > 0) 
    warning(paste(narg, "unrecognised",
                  ngettext(narg, "argument was", "arguments were"),
                  "ignored"))

  if(!is.null(by)) 
    return(genericNNdistBy(X, by, k=k))
  
  # extract point coordinates
  xyz <- coords(X)
  n <- nrow(xyz)
  x <- xyz$x
  y <- xyz$y
  z <- xyz$z
  
  # k can be a single integer or an integer vector
  if(length(k) == 0)
    stop("k is an empty vector")
  else if(length(k) == 1L) {
    if(k != round(k) || k <= 0)
      stop("k is not a positive integer")
  } else {
    if(any(k != round(k)) || any(k <= 0))
      stop(paste("some entries of the vector",
           sQuote("k"), "are not positive integers"))
  }
  k <- as.integer(k)
  kmax <- max(k)

  # trivial cases
  if(n <= 1L) {
    # empty pattern => return numeric(0)
    # or pattern with only 1 point => return Inf
    nnd <- matrix(Inf, nrow=n, ncol=kmax)
    nnd <- nnd[,k, drop=TRUE]
    return(nnd)
  }
  
  # number of neighbours that are well-defined
  kmaxcalc <- min(n-1L, kmax)

  # calculate k-nn distances for k <= kmaxcalc
  
  if(kmaxcalc == 1L) {
    # calculate nearest neighbour distance only
    nnd<-numeric(n)
    o <- fave.order(z)
    big <- sqrt(.Machine$double.xmax)
    Cout <- .C("nnd3D",
               n= as.integer(n),
               x= as.double(x[o]),
               y= as.double(y[o]),
               z= as.double(z[o]),
               nnd= as.double(nnd),
               nnwhich = as.integer(integer(1L)),
               huge=as.double(big),
               PACKAGE = "spatstat")
    nnd[o] <- Cout$nnd
  } else {
    # case kmaxcalc > 1
    nnd<-numeric(n * kmaxcalc)
    o <- fave.order(z)
    big <- sqrt(.Machine$double.xmax)
    Cout <- .C("knnd3D",
               n    = as.integer(n),
               kmax = as.integer(kmaxcalc),
               x    = as.double(x[o]),
               y    = as.double(y[o]),
               z    = as.double(z[o]),
               nnd  = as.double(nnd),
               nnwhich = as.integer(integer(1L)),
               huge = as.double(big),
               PACKAGE = "spatstat")
    nnd <- matrix(nnd, nrow=n, ncol=kmaxcalc)
    nnd[o, ] <- matrix(Cout$nnd, nrow=n, ncol=kmaxcalc, byrow=TRUE)
  }

  # post-processing
  if(kmax > kmaxcalc) {
    # add columns of Inf's
    infs <- matrix(as.numeric(Inf), nrow=n, ncol=kmax-kmaxcalc)
    nnd <- cbind(nnd, infs)
  }

  if(length(k) < kmax) {
    # select only the specified columns
    nnd <- nnd[, k, drop=TRUE]
  }
  
  return(nnd)
}

nnwhich.pp3 <- function(X, ..., k=1) {
  verifyclass(X, "pp3")
  if((narg <- length(list(...))) > 0) 
    warning(paste(narg, "unrecognised",
                  ngettext(narg, "argument was", "arguments were"),
                  "ignored"))
  
  # k can be a single integer or an integer vector
  if(length(k) == 0)
    stop("k is an empty vector")
  else if(length(k) == 1L) {
    if(k != round(k) || k <= 0)
      stop("k is not a positive integer")
  } else {
    if(any(k != round(k)) || any(k <= 0))
      stop(paste("some entries of the vector",
           sQuote("k"), "are not positive integers"))
  }
  k <- as.integer(k)
  kmax <- max(k)

  # extract point coordinates
  xyz <- coords(X)
  n <- nrow(xyz)
  x <- xyz$x
  y <- xyz$y
  z <- xyz$z
  
  # special cases
  if(n <= 1L) {
    # empty pattern => return integer(0)
    # or pattern with only 1 point => return NA
    nnw <- matrix(as.integer(NA), nrow=n, ncol=kmax)
    nnw <- nnw[,k, drop=TRUE]
    return(nnw)
  }

  # number of neighbours that are well-defined
  kmaxcalc <- min(n-1L, kmax)

  # identify k-nn for k <= kmaxcalc

  if(kmaxcalc == 1L) {
    # identify nearest neighbour only
    nnw <- integer(n)
    o <- fave.order(z)
    big <- sqrt(.Machine$double.xmax)
    Cout <- .C("nnw3D",
               n = as.integer(n),
               x = as.double(x[o]),
               y = as.double(y[o]),
               z = as.double(z[o]),
               nnd = as.double(numeric(1L)),
               nnwhich = as.integer(nnw),
               huge = as.double(big),
               PACKAGE = "spatstat")
    # [sic] Conversion from C to R indexing is done in C code.
    witch <- Cout$nnwhich
    if(any(witch <= 0))
      stop("Internal error: illegal index returned from C code")
    if(any(witch > n))
      stop("Internal error: index returned from C code exceeds n")
    nnw[o] <- o[witch]
  } else {
    # case kmaxcalc > 1
    nnw <- matrix(integer(n * kmaxcalc), nrow=n, ncol=kmaxcalc)
    o <- fave.order(z)
    big <- sqrt(.Machine$double.xmax)
    Cout <- .C("knnw3D",
               n = as.integer(n),
               kmax = as.integer(kmaxcalc),
               x = as.double(x[o]),
               y = as.double(y[o]),
               z = as.double(z[o]),
               nnd = as.double(numeric(1L)),
               nnwhich = as.integer(nnw),
               huge = as.double(big),
               PACKAGE = "spatstat")
    # [sic] Conversion from C to R indexing is done in C code.
    witch <- Cout$nnwhich 
    witch <- matrix(witch, nrow=n, ncol=kmaxcalc, byrow=TRUE)
    if(any(witch <= 0))
      stop("Internal error: illegal index returned from C code")
    if(any(witch > n))
      stop("Internal error: index returned from C code exceeds n")
    # convert back to original ordering
    nnw[o,] <- matrix(o[witch], nrow=n, ncol=kmaxcalc)
  }
  
  # post-processing
  if(kmax > kmaxcalc) {
    # add columns of NA's
    nas <- matrix(as.integer(NA), nrow=n, ncol=kmax-kmaxcalc)
    nnw <- cbind(nnw, nas)
  }

  if(length(k) < kmax) {
    # select only the specified columns
    nnw <- nnw[, k, drop=TRUE]
  }
  return(nnw)
}

crossdist.pp3 <- function(X, Y, ..., periodic=FALSE, squared=FALSE) {
  verifyclass(X, "pp3")
  verifyclass(Y, "pp3")

  cX <- coords(X)
  cY <- coords(Y)
  nX <- nrow(cX)
  nY <- nrow(cY)

  if(nX == 0 || nY == 0)
    return(matrix(numeric(0), nrow=nX, ncol=nY))

  if(!periodic) {
    Cout <- .C("D3crossdist",
               nfrom = as.integer(nX),
               xfrom = as.double(cX$x),
               yfrom = as.double(cX$y),
               zfrom = as.double(cX$z),
               nto = as.integer(nY),
               xto = as.double(cY$x),
               yto = as.double(cY$y),
               zto = as.double(cY$z),
               squared = as.integer(squared),
               d = as.double(matrix(0, nrow=nX, ncol=nY)),
               PACKAGE = "spatstat")
  } else {
    b <- as.box3(X)
    wide <- diff(b$xrange)
    high <- diff(b$yrange)
    deep <- diff(b$zrange)
    Cout <- .C("D3crossPdist",
               nfrom = as.integer(nX),
               xfrom = as.double(cX$x),
               yfrom = as.double(cX$y),
               zfrom = as.double(cX$z),
               nto = as.integer(nY),
               xto = as.double(cY$x),
               yto = as.double(cY$y),
               zto = as.double(cY$z),
               xwidth = as.double(wide),
               yheight = as.double(high),
               zheight = as.double(deep),
               squared = as.integer(squared),
               d = as.double(matrix(0, nrow=nX, ncol=nY)),
               PACKAGE = "spatstat")
  }
  return(matrix(Cout$d, nrow=nX, ncol=nY))
}
  

##
##     texture.R
##
##     Texture plots and texture maps
##
##  $Revision: 1.15 $ $Date: 2016/02/16 01:39:12 $

### .................. basic graphics .............................

## put hatching in a window
add.texture <- function(W, texture=4, spacing=NULL, ...) {
  if(is.data.frame(texture)) {
    ## texture = f(x) where f is a texturemap
    out <- do.call(add.texture,
                   resolve.defaults(list(W=W, spacing=spacing),
                                    list(...),
                                    as.list(texture)))
    return(out)
  }
  ## texture should be an integer
  stopifnot(is.owin(W))
  stopifnot(texture %in% 1:8)
  if(is.null(spacing)) {
    spacing <- diameter(as.rectangle(W))/50
  } else {
    check.1.real(spacing)
    stopifnot(spacing > 0)
  }
  P <- L <- NULL
  switch(texture,
         {
           ## texture 1: graveyard
           P <- rsyst(W, dx=3*spacing)
         },
         {
           ## texture 2: vertical lines
           L <- rlinegrid(90, spacing, W)[W]
         },
         {
           ## texture 3: horizontal lines
           L <- rlinegrid(0, spacing, W)[W]
         },
         {
           ## texture 4: forward slashes
           L <- rlinegrid(45, spacing, W)[W]
         },
         {
           ## texture 5: back slashes
           L <- rlinegrid(135, spacing, W)[W]
         },
         {
           ## texture 6: horiz/vert grid
           L0 <- rlinegrid(0, spacing, W)[W]
           L90 <- rlinegrid(90, spacing, W)[W]
           L <- superimpose(L0, L90, W=W, check=FALSE)
         },
         {
           ## texture 7: diagonal grid
           L45 <- rlinegrid(45, spacing, W)[W]
           L135 <- rlinegrid(135, spacing, W)[W]
           L <- superimpose(L45, L135, W=W, check=FALSE)
         },
         {
           ## texture 8: hexagons
           H <- hextess(W, spacing, offset=runifpoint(1, W))
           H <- intersect.tess(H, W)
           do.call.matched(plot.tess,
                           resolve.defaults(list(x=H, add=TRUE),
                                            list(...)))
         })
  if(!is.null(P))
    do.call.matched(plot.ppp,
                    resolve.defaults(list(x=P, add=TRUE),
                                     list(...),
                                     list(chars=3, cex=0.2)),
                    extrargs=c("lwd", "col", "cols", "pch"))
  if(!is.null(L))
    do.call.matched(plot.psp,
                    resolve.defaults(list(x=L, add=TRUE),
                                     list(...)),
                    extrargs=c("lwd","lty","col"))
  return(invisible(NULL))
}

## .................. texture maps ................................

## create a texture map

texturemap <- function(inputs, textures, ...) {
  argh <- list(...)
  if(length(argh) > 0) {
    isnul <- unlist(lapply(argh, is.null))
    argh <- argh[!isnul]
  }
  if(missing(textures) || is.null(textures)) textures <- seq_along(inputs)
  df <- do.call(data.frame,
                append(list(input=inputs, texture=textures), argh))
  f <- function(x) {
    df[match(x, df$input), -1, drop=FALSE]
  }
  class(f) <- c("texturemap", class(f))
  attr(f, "df") <- df
  return(f)
}

print.texturemap <- function(x, ...) {
  cat("Texture map\n")
  print(attr(x, "df"))
  return(invisible(NULL))
}

## plot a texture map

plot.texturemap <- local({

  ## recognised additional arguments to and axis()
  axisparams <- c("cex", 
                  "cex.axis", "cex.lab",
                  "col.axis", "col.lab",
                  "font.axis", "font.lab",
                  "las", "mgp", "xaxp", "yaxp",
                  "tck", "tcl", "xpd")

  # rules to determine the map dimensions when one dimension is given
  widthrule <- function(heightrange, separate, n, gap) {
    if(separate) 1 else diff(heightrange)/10
  }
  heightrule <- function(widthrange, separate, n, gap) {
    (if(separate) (n + (n-1)*gap) else 10) * diff(widthrange) 
  }

  plot.texturemap <- function(x, ..., main,
                             xlim=NULL, ylim=NULL, vertical=FALSE, axis=TRUE,
                             labelmap=NULL, gap=0.25,
                             spacing=NULL, add=FALSE) {
    if(missing(main))
      main <- short.deparse(substitute(x))
    df <- attr(x, "df")
#    textures <- df$textures
    n   <- nrow(df)
    check.1.real(gap, "In plot.texturemap")
    explain.ifnot(gap >= 0, "In plot.texturemap")
    separate <- (gap > 0)
    if(is.null(labelmap)) {
      labelmap <- function(x) x
    } else stopifnot(is.function(labelmap))
    ## determine rectangular window for display
    rr <- c(0, n + (n-1)*gap)
    if(is.null(xlim) && is.null(ylim)) {
      u <- widthrule(rr, separate, n, gap)
      if(!vertical) {
        xlim <- rr
        ylim <- c(0,u)
      } else {
        xlim <- c(0,u)
        ylim <- rr
      }
    } else if(is.null(ylim)) {
      if(!vertical) 
        ylim <- c(0, widthrule(xlim, separate, n, gap))
      else 
        ylim <- c(0, heightrule(xlim, separate, n, gap))
    } else if(is.null(xlim)) {
      if(!vertical) 
        xlim <- c(0, heightrule(ylim, separate, n, gap))
      else 
        xlim <- c(0, widthrule(ylim, separate, n, gap))
    } 
    width <- diff(xlim)
    height <- diff(ylim)
    ## determine boxes to be filled with textures,
    if(vertical) {
      boxheight <- min(width, height/(n + (n-1) * gap))
      vgap   <- (height - n * boxheight)/(n-1)
      boxes <- list()
      for(i in 1:n) boxes[[i]] <-
        owin(xlim, ylim[1] + c(i-1, i) * boxheight + (i-1) * vgap)
    } else {
      boxwidth <- min(height, width/(n + (n-1) * gap))
      hgap   <- (width - n * boxwidth)/(n-1)
      boxes <- list()
      for(i in 1:n) boxes[[i]] <-
        owin(xlim[1] + c(i-1, i) * boxwidth + (i-1) * hgap, ylim)
    }
    boxsize <- shortside(boxes[[1]])
    if(is.null(spacing))
      spacing <- 0.1 * boxsize
    
    # .......... initialise plot ...............................
    if(!add)
      do.call.matched(plot.default,
                      resolve.defaults(list(x=xlim, y=ylim,
                                            type="n", main=main,
                                            axes=FALSE, xlab="", ylab="",
                                            asp=1.0),
                                       list(...)))
    
    ## ................ plot texture blocks .................
    for(i in 1:n) {
      dfi <- df[i,,drop=FALSE]
      add.texture(W=boxes[[i]], texture=dfi, ..., spacing=spacing)
      plot(boxes[[i]], add=TRUE)
    }

    if(axis) {
      # ................. draw annotation ..................
      la <- paste(labelmap(df$input))
      if(!vertical) {
        ## add horizontal axis/annotation
        at <- lapply(lapply(boxes, centroid.owin), "getElement", name="x")
        # default axis position is below the ribbon (side=1)
        sidecode <- resolve.1.default("side", list(...), list(side=1))
        if(!(sidecode %in% c(1,3)))
          warning(paste("side =", sidecode,
                        "is not consistent with horizontal orientation"))
        pos <- c(ylim[1], xlim[1], ylim[2], xlim[2])[sidecode]
        # don't draw axis lines if plotting separate blocks
        lwd0 <- if(separate) 0 else 1
        # draw axis
        do.call.matched(graphics::axis,
                        resolve.defaults(list(...),
                                         list(side = 1, pos = pos, at = at),
                                         list(labels=la, lwd=lwd0)),
                        extrargs=axisparams)
      } else {
        ## add vertical axis
        at <- lapply(lapply(boxes, centroid.owin), "getElement", name="y")
        # default axis position is to the right of ribbon (side=4)
        sidecode <- resolve.1.default("side", list(...), list(side=4))
        if(!(sidecode %in% c(2,4)))
          warning(paste("side =", sidecode,
                        "is not consistent with vertical orientation"))
        pos <- c(ylim[1], xlim[1], ylim[2], xlim[2])[sidecode]
        # don't draw axis lines if plotting separate blocks
        lwd0 <- if(separate) 0 else 1
        # draw labels horizontally if plotting separate blocks
        las0 <- if(separate) 1 else 0
        # draw axis
        do.call.matched(graphics::axis,
                        resolve.defaults(list(...),
                                         list(side=4, pos=pos, at=at),
                                         list(labels=la, lwd=lwd0, las=las0)),
                        extrargs=axisparams)
      }
    }
    invisible(NULL)
  }

  plot.texturemap
})

## plot a pixel image using textures

textureplot <- local({

  textureplot <- function(x, ..., main, add=FALSE, clipwin=NULL, do.plot=TRUE,
                          border=NULL, col=NULL, lwd=NULL, lty=NULL,
                          spacing=NULL, textures=1:8,
                          legend=TRUE,
                          leg.side=c("right", "left", "bottom", "top"),
                          legsep=0.1, legwid=0.2) {
    if(missing(main))
      main <- short.deparse(substitute(x))
    if(!(is.im(x) || is.tess(x))) {
      x <- try(as.tess(x), silent=TRUE)
      if(inherits(x, "try-error")) 
        x <- try(as.im(x), silent=TRUE)
      if(inherits(x, "try-error")) 
        stop("x must be a pixel image or a tessellation", call.=FALSE)
    }
    leg.side <- match.arg(leg.side)
    if(!is.null(clipwin))
      x <- x[clipwin, drop=FALSE]
    if(is.im(x)) {
      if(x$type != "factor")
        x <- eval.im(factor(x))
      levX <- levels(x)
    } else {
      tilX <- tiles(x)
      levX <- names(tilX)
    }
    n <- length(levX)
    if(n > 8)
      stop("Too many factor levels or tiles: maximum is 8")
    ## determine texture map
    if(inherits(textures, "texturemap")) {
      tmap <- textures
    } else {
      stopifnot(all(textures %in% 1:8))
      stopifnot(length(textures) >= n)
      mono <- spatstat.options("monochrome")
      col <- enforcelength(col, n, if(mono) 1 else 1:8)
      lwd <- if(is.null(lwd)) NULL else enforcelength(lwd, n, 1)
      lty <- if(is.null(lty)) NULL else enforcelength(lwd, n, 1)
      tmap <- texturemap(inputs=levX, textures=textures[1:n],
                         col=col, lwd=lwd, lty=lty)
    }
    ## determine plot region
    bb <- as.rectangle(x)
    if(!legend) {
      bb.all <- bb
    } else {
      Size <- max(sidelengths(bb))
      bb.leg <-
        switch(leg.side,
               right={
                 ## legend to right of image
                 owin(bb$xrange[2] + c(legsep, legsep+legwid) * Size,
                      bb$yrange)
               },
               left={
                 ## legend to left of image
                 owin(bb$xrange[1] - c(legsep+legwid, legsep) * Size,
                      bb$yrange)
               },
               top={
                 ## legend above image
                 owin(bb$xrange,
                      bb$yrange[2] + c(legsep, legsep+legwid) * Size)
               },
               bottom={
                 ## legend below image
                 owin(bb$xrange,
                      bb$yrange[1] - c(legsep+legwid, legsep) * Size)
           })
      iside <- match(leg.side, c("bottom", "left", "top", "right"))
      bb.all <- boundingbox(bb.leg, bb)
    }
    ## 
    result <- tmap
    attr(result, "bbox") <- bb
    ##
    if(do.plot) {
      ## Plot textures
      if(!add) {
        plot(bb.all, type="n", main="")
        fakemaintitle(bb, main, ...)
      }
      if(is.null(spacing)) spacing <- diameter(as.rectangle(x))/50
      areas <- if(is.im(x)) table(x$v) else tile.areas(x)
      for(i in which(areas > 0)) {
        Zi <- if(is.tess(x)) tilX[[i]] else levelset(x, levX[i], "==")
        Zi <- as.polygonal(Zi)
        if(is.null(border) || !is.na(border))
          plot(Zi, add=TRUE, border=border)
        add.texture(Zi, texture=tmap(levX[i]), spacing=spacing, ...)
      }
      vertical <- leg.side %in% c("left", "right")
      if(legend)
        do.call(plot.texturemap,
                resolve.defaults(list(x=tmap, add=TRUE,
                                      vertical=vertical,
                                      side=iside,
                                      xlim=bb.leg$xrange,
                                      ylim=bb.leg$yrange,
                                      spacing=spacing),
                                 list(...)))
    }
    return(invisible(result))
  }

  enforcelength <- function(x, n, x0) {
    if(is.null(x)) x <- x0
    if(length(x) < n) x <- rep(x, n)
    return(x[1:n])
  }

  textureplot
})



  

#
# randomOnLines.R
#
# $Revision: 1.10 $  $Date: 2020/03/16 10:28:51 $
#
# Generate random points on specified lines
#

runifpointOnLines <- function(n, L, nsim=1, drop=TRUE) {
  if(!is.numeric(n) || any(n < 0) || any(n %% 1 != 0))
    stop("n should be a nonnegative integer or integers")
  if(!is.psp(L))
    L <- as.psp(L)
  W <- as.owin(L)
  result <- vector(mode="list", length=nsim)
  for(i in 1:nsim) {
    X <- datagen.runifpointOnLines(n, L)
    Y <- ppp(X$x, X$y, marks=X$marks, window=W, check=FALSE)
    result[[i]] <- Y
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

datagen.runifpointOnLines <- function(n, L) {
  stopifnot(is.psp(L))
  m <- length(n)
  ismarked <- (m > 1)
  if(m == 0 || (m == 1 && n == 0))
    return(data.frame(x=numeric(0),
                      y=numeric(0),
                      seg=integer(0),
                      tp=numeric(0)))
  # extract segment information
  len <- lengths_psp(L)
  sumlen <- sum(len)
  cumlen <- cumsum(len)
  cum0len <- c(0, cumlen)
  Ldf <- as.data.frame(L)
  x0 <- with(Ldf, x0)
  y0 <- with(Ldf, y0)
  dx <- with(Ldf, x1-x0)
  dy <- with(Ldf, y1-y0)
  # determine mark space
  if(ismarked) {
    markvalues <- names(n)
    if(sum(nzchar(markvalues)) < m)
      markvalues <- paste(1:m)
  }
  # initialise output data.frame
  out <- data.frame(x=numeric(0), y=numeric(0), seg=integer(0), tp=numeric(0))
  if(ismarked) 
    out <- cbind(out, data.frame(marks=character(0)))
  # generate points of each mark in turn
  for(j in 1:m) {
    if(n[[j]] > 0) {
      # generate random positions
      uu <- runif(n[[j]], min=0, max=sumlen)
      # identify segment for each point
      kk <- findInterval(uu, cum0len, rightmost.closed=TRUE, all.inside=TRUE)
      # parametric position along segment
      tt <- (uu - cum0len[kk])/len[kk]
      tt[!is.finite(tt)] <- 0
      # convert to (x,y)
      x <- x0[kk] + tt * dx[kk]
      y <- y0[kk] + tt * dy[kk]
      # assemble result
      if(!ismarked) {
        out <- data.frame(x=x, y=y, seg=kk, tp=tt)
      } else {
        outj <- data.frame(x=x, y=y, seg=kk, tp=tt, marks=markvalues[j])
        out <- rbind(out, outj)
      }
    }
  }
  if(ismarked) out$marks <- factor(out$marks, levels=markvalues)
  return(out)
}

runifpoisppOnLines <- function(lambda, L, nsim=1, drop=TRUE) {
  if(!is.numeric(lambda) || !all(is.finite(lambda) && (lambda >= 0)))
    stop("lambda should be a finite, nonnegative number or numbers")
  if(!is.psp(L))
    L <- as.psp(L)
  W <- as.owin(L)
  result <- vector(mode="list", length=nsim)
  for(i in 1:nsim) {
    X <- datagen.runifpoisppOnLines(lambda, L)
    Y <- ppp(X$x, X$y, marks=X$marks, window=W, check=FALSE)
    result[[i]] <- Y
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

datagen.runifpoisppOnLines <- function(lambda, L) {
  stopifnot(is.psp(L))
  mu <- lambda * sum(lengths_psp(L))
  n <- rpois(rep.int(1, length(mu)), mu)
  if(length(n) > 1)
    names(n) <- names(lambda)
  df <- datagen.runifpointOnLines(n, L)
  return(df)
}

rpoisppOnLines <- function(lambda, L, lmax=NULL, ..., nsim=1, drop=TRUE) {
  if(!is.psp(L))
    L <- as.psp(L)
  W <- as.owin(L)
  result <- vector(mode="list", length=nsim)
  for(i in 1:nsim) {
    X <- datagen.rpoisppOnLines(lambda, L, lmax=lmax, ...)
    Y <- ppp(X$x, X$y, marks=X$marks, window=W, check=FALSE)
    result[[i]] <- Y
  }
  result <- simulationresult(result, nsim, drop)
  return(result)
}

datagen.rpoisppOnLines <- function(lambda, L, lmax=NULL, ..., check=TRUE)  {
  stopifnot(is.psp(L))
  if(is.numeric(lambda)) 
    return(datagen.runifpoisppOnLines(lambda, L))
  # ensure lambda is a list
  if(is.function(lambda) || is.im(lambda))
    lambda <- list(lambda)
  m <- length(lambda)
  # determine type of argument
  argtype <-
    if(all(unlist(lapply(lambda, is.im)))) "im" else
    if(all(unlist(lapply(lambda, is.function)))) "function" else
    stop(paste(sQuote("lambda"),
               "must be a numeric vector, a function, an image,",
               "a list of functions, or a list of images"))
  # check values of lambda
  if(argtype == "im") {
    for(j in seq_len(m)) {
      lamj <- lambda[[j]]
      if(!(lamj$type %in% c("real", "integer")))
        stop("lambda must be numeric-valued or integer-valued")
      lrange <- range(lamj)
      if(any(is.infinite(lrange)))
        stop("Infinite pixel values not permitted")
      if(lrange[1] < 0)
        stop("Negative pixel values not permitted")
    }
  }
  # determine uniform bound
  if(!is.null(lmax)) {
    stopifnot(is.numeric(lmax))
    if(length(lmax) != m) {
      if(length(lmax) == 1) {
        lmax <- rep.int(lmax, m)
      } else stop("Length of lmax does not match length of lambda")
    }
  } else {
    # compute lmax
    lmax <- numeric(m)
    for(j in seq_len(m)) {
      lamj <- lambda[[j]]
      if(is.function(lamj)) {
        X <- pointsOnLines(L, np=10000)
        lambdaX <- lamj(X$x, X$y, ...)
        lmax[j] <- max(lambdaX, na.rm=TRUE)
      } else if(is.im(lamj)) 
        lmax[j] <- max(lamj)
    }
    if(!all(is.finite(lmax)))
      stop("Infinite values of lambda obtained")
    if(any(lmax < 0))
      stop("Negative upper bound for lambda obtained")
    names(lmax) <- names(lambda)
  } 
  # Lewis-Shedler (rejection) method
  Y <- datagen.runifpoisppOnLines(lmax, L)
  n <- nrow(Y)
  if(n == 0)
    return(Y)
  # evaluate lambda at each simulated point
  if(m == 1) {
    lambda <- lambda[[1]]
    markindex <- 1
    if(is.function(lambda)) 
      lambdaY <- lambda(Y$x, Y$y, ...)
    else
      lambdaY <- safelookup(lambda, as.ppp(Y, W=as.owin(L)))
  } else {
    lambdaY <- numeric(n)
    markindex <- as.integer(Y$marks)
    for(j in seq_len(m)) {
      lamj <- lambda[[j]]
      jrows <- (markindex == j)
      Yj <- Y[jrows, , drop=FALSE]
      if(is.function(lamj)) 
        lambdaY[jrows] <- lamj(Yj$x, Yj$y, ...)
      else
        lambdaY[jrows] <- safelookup(lamj, as.ppp(Yj, W=as.owin(L)))
    }
  }
  lambdaY[is.na(lambdaY)] <- 0
  # accept/reject
  pY <- lambdaY/lmax[markindex]
  if(check) {
    if(any(pY < 0))
      warning("Negative values of lambda obtained")
    if(any(pY > 1))
      warning("lmax is not an upper bound for lambda")
  }
  retain <- (runif(n) < pY)
  Y <- Y[retain, , drop=FALSE]
  return(Y)
}

      
  
  
  

#
# listof.R
#
# Methods for class `listof'
#
# plot.listof is defined in plot.splitppp.R
#

"[<-.listof" <- function(x, i, value) {
  # invoke list method
  class(x) <- "list"
  x[i] <- value
  # then make it a 'listof' object too
  class(x) <- c("listof", class(x))
  x
}
  
summary.listof <- function(object, ...) {
  x <- lapply(object, summary, ...)
  class(x) <- "summary.listof"
  x
}

print.summary.listof <- function(x, ...) {
  class(x) <- "listof"
  print(x)
  invisible(NULL)
}

listof <- function(...) {
#  warn.once("listof",
#            "The class listof will be Deprecated",
#            "in future versions of spatstat.",
#            "Use anylist or solist")
  stuff <- list(...)
  class(stuff) <- c("listof", class(stuff))
  return(stuff)
}

as.listof <- function(x) {
  if(!is.list(x))
    x <- list(x)
  if(!inherits(x, "listof"))
    class(x) <- c("listof", class(x))
#  warn.once("listof",
#            "The class listof will be Deprecated",
#            "in future versions of spatstat.",
#            "Use anylist or solist")
  return(x)
}

as.layered.listof <- function(X) {
  layered(LayerList=X)
}

#
#     eval.im.R
#
#        eval.im()             Evaluate expressions involving images
#
#        compatible.im()       Check whether two images are compatible
#
#        harmonise.im()       Harmonise images
#        commonGrid()
#
#     $Revision: 1.54 $     $Date: 2019/04/21 11:42:27 $
#

eval.im <- local({

  eval.im <- function(expr, envir, harmonize=TRUE, warn=TRUE) {
    e <- as.expression(substitute(expr))
    ## get names of all variables in the expression
    varnames <- all.vars(e)
    allnames <- all.names(e, unique=TRUE)
    funnames <- allnames[!(allnames %in% varnames)]
    if(length(varnames) == 0)
      stop("No variables in this expression")
    ## get the values of the variables
    if(missing(envir)) {
      envir <- parent.frame() # WAS: sys.parent()
    } else if(is.list(envir)) {
      envir <- list2env(envir, parent=parent.frame())
    }
    vars <- mget(varnames, envir=envir, inherits=TRUE, ifnotfound=list(NULL))
    funs <- mget(funnames, envir=envir, inherits=TRUE, ifnotfound=list(NULL))
    ## WAS: vars <- lapply(as.list(varnames), get, envir=envir)
    ## WAS: funs <- lapply(as.list(funnames), get, envir=envir)
    ##
    ## find out which variables are images
    ims <- unlist(lapply(vars, is.im))
    if(!any(ims))
      stop("No images in this expression")
    images <- vars[ims]
    nimages <- length(images)
    ## test that the images are compatible
    if(!(do.call(compatible, unname(images)))) {
      whinge <- paste(if(nimages > 2) "some of" else NULL,
                      "the images",
                      commasep(sQuote(names(images))),
                      if(!harmonize) "are" else "were",
                      "not compatible")
      if(!harmonize) {
        stop(whinge, call.=FALSE)
      } else if(warn) {
        warning(whinge, call.=FALSE)
      }
      images <- do.call(harmonise.im, images)
    }
    ## trap a common error: using fv object as variable
    isfun <- unlist(lapply(vars, is.fv))
    if(any(isfun))
      stop("Cannot use objects of class fv as variables in eval.im")
    ## replace each image by its matrix of pixel values, and evaluate
    imagevalues <- lapply(images, getImValues)
    template <- images[[1L]]
    ## This bit has been repaired:
    vars[ims] <- imagevalues
    v <- eval(e, append(vars, funs))
    ##
    ## reshape, etc
    result <- im(v,
                 xcol=template$xcol, yrow=template$yrow,
                 xrange=template$xrange, yrange=template$yrange, 
                 unitname=unitname(template))
    return(result)
  }
  
  ## extract pixel values without destroying type information
  getImValues <- function(x) {
    v <- as.matrix(x)
    dim(v) <- NULL
    return(v)
  }

  eval.im
})

compatible.im <- function(A, B, ..., tol=1e-6) {
  verifyclass(A, "im")
  if(missing(B)) return(TRUE)
  verifyclass(B, "im")
  if(!all(A$dim == B$dim))
    return(FALSE)
  xdiscrep <- max(abs(A$xrange - B$xrange),
                 abs(A$xstep - B$xstep),
                 abs(A$xcol - B$xcol))
  ydiscrep <- max(abs(A$yrange - B$yrange),
                 abs(A$ystep - B$ystep),
                 abs(A$yrow - B$yrow))
  xok <- (xdiscrep < tol * min(A$xstep, B$xstep))
  yok <- (ydiscrep < tol * min(A$ystep, B$ystep))
  uok <- compatible.unitname(unitname(A), unitname(B))
  if(!(xok && yok && uok))
    return(FALSE)
  ## A and B are compatible
  if(length(list(...)) == 0)
    return(TRUE)
  ## recursion
  return(compatible.im(B, ..., tol=tol))
}

## force a list of images to be compatible

harmonize.im <- harmonise.im <- function(...) {
  argz <- list(...)
  n <- length(argz)
  if(n < 2) return(argz)
  result <- vector(mode="list", length=n)
  isim <- unlist(lapply(argz, is.im))
  if(!any(isim))
    stop("No images supplied")
  imgs <- argz[isim]
  ## if any windows are present, extract bounding box
  iswin <- unlist(lapply(argz, is.owin))
  bb0 <- if(!any(iswin)) NULL else do.call(boundingbox, unname(argz[iswin]))
  if(length(imgs) == 1L && is.null(bb0)) {
    ## only one 'true' image: use it as template.
    result[isim] <- imgs
    Wtemplate <- imgs[[1L]]
  } else {
    ## test for compatible units
    un <- lapply(imgs, unitname)
    uok <- unlist(lapply(un, compatible.unitname, y=un[[1L]]))
    if(!all(uok))
      stop("Images have incompatible units of length")
    ## find the image with the highest resolution
    xsteps <- unlist(lapply(imgs, getElement, name="xstep"))
    which.finest <- which.min(xsteps)
    finest <- imgs[[which.finest]]
    ## get the bounding box
    bb <- do.call(boundingbox, lapply(unname(imgs), as.rectangle))
    if(!is.null(bb0)) bb <- boundingbox(bb, bb0)
    ## determine new raster coordinates
    xcol <- prolongseq(finest$xcol, bb$xrange)
    yrow <- prolongseq(finest$yrow, bb$yrange)
    xy <- list(x=xcol, y=yrow)
    ## resample all images on new raster
    newimgs <- lapply(imgs, as.im, xy=xy)
    result[isim] <- newimgs
    Wtemplate <- newimgs[[which.finest]]
  }
  ## convert other data to images
  if(any(notim <- !isim)) 
    result[notim] <- lapply(argz[notim], as.im, W=as.mask(Wtemplate))
  names(result) <- names(argz)
  return(result)
}

## Return just the corresponding template window

commonGrid <- local({
  ## auxiliary function
  gettype <- function(x) {
    if(is.im(x) || is.mask(x)) "raster" else
    if(is.owin(x) || is.ppp(x) || is.psp(x)) "spatial" else
    "none"
  }

  commonGrid <- function(...) {
    argz <- list(...)
    type <- unlist(lapply(argz, gettype))
    israster <- (type == "raster")
    haswin   <- (type != "none")

    if(any(israster)) {
      ## Get raster data
      rasterlist <- argz[israster]
    } else {
      ## No existing raster data - apply default resolution
      if(!any(haswin))
        stop("No spatial data supplied")
      wins <- lapply(argz[haswin], as.owin)
      rasterlist <- lapply(wins, as.mask)
    }

    ## Find raster object with finest resolution
    if(length(rasterlist) == 1L) {
      ## only one raster object
      finest <- rasterlist[[1L]]
    } else {
      ## test for compatible units
      un <- lapply(rasterlist, unitname)
      uok <- unlist(lapply(un, compatible.unitname, y=un[[1L]]))
      if(!all(uok))
        stop("Objects have incompatible units of length")
      ## find the image/mask with the highest resolution
      xsteps <- unlist(lapply(rasterlist, getElement, name="xstep"))
      which.finest <- which.min(xsteps)
      finest <- rasterlist[[which.finest]]
    }
    ## determine the bounding box
    bb <- do.call(boundingbox, lapply(unname(argz[haswin]), as.rectangle))
    ## determine new raster coordinates
    xcol <- prolongseq(finest$xcol, bb$xrange)
    yrow <- prolongseq(finest$yrow, bb$yrange)
    xy <- list(x=xcol, y=yrow)
    ## generate template
    Wtemplate <- as.mask(bb, xy=xy)
    return(Wtemplate)
  }

  commonGrid
})

im.apply <- function(X, FUN, ..., fun.handles.na=FALSE, check=TRUE) {
  if(!inherits(X, "imlist")) {
    stopifnot(is.list(X))
    if(!all(sapply(X, is.im)))
      stop("All elements of X must be pixel images")
  }
  ## determine function to be applied
  fun <- if(is.character(FUN)) get(FUN, mode="function") else
         if(is.function(FUN)) FUN else stop("Unrecognised format for FUN")
  funcode <- match(list(fun),
                   list(base::sum,
                        base::mean,
                        base::mean.default,
                        stats::var,
                        stats::sd),
                   nomatch=0L)
  funtype <- c("general", "sum", "mean", "mean", "var", "sd")[funcode+1L]
  if(funcode != 0)
    na.rm <- resolve.1.default(list(na.rm=FALSE), list(...))
  ## ensure images are compatible
  if(check && !do.call(compatible, unname(X)))
    X <- do.call(harmonise.im, X)
  template <- X[[1L]]
  d <- dim(template)
  ## extract numerical values and convert to matrix with one column per image
  vals <- sapply(X, getElement, name="v")
  ## apply to all pixels ?
  full <- fun.handles.na || !anyNA(vals)
  if(!full) {
    ## NA present
    ok <- complete.cases(vals)
    if(!any(ok)) {
      ## empty result
      return(as.im(NA, W=template))
    }
    ## restrict to pixels where all data are non-NA
    vals <- vals[ok, , drop=FALSE]
  }
  n <- nrow(vals)
  ## calculate
  y <- switch(funtype,
              general = apply(vals, 1L, fun, ...),
              sum     = rowSums(vals,  na.rm=na.rm),
              mean    = rowMeans(vals, na.rm = na.rm),
              sd = ,
              var = {
                sumx  <- rowSums(vals,   na.rm = na.rm)
                sumx2 <- rowSums(vals^2, na.rm = na.rm)
                if(!anyNA(vals)) {
                  m <- ncol(vals)
                  v <- (sumx2 - sumx^2/m)/(m-1)
                } else {
                  m <- rowSums(!is.na(vals)) 
                  v <- ifelse(m < 2, NA, (sumx2 - sumx^2/m)/(m-1))
                }
                if(funtype == "var") v else sqrt(v)
              })
  if(funtype == "general" && length(y) != n)
    stop("FUN should yield one value per pixel")
  
  if(!full) {
    ## put the NA's back (preserving type of 'y')
    yfull <- rep(y[1L], prod(d))
    yfull[ok] <- y
    yfull[!ok] <- NA
    y <- yfull
  }
  
  ## pack up (preserving type of 'y')
  result <- im(y,
               xcol=template$xcol, yrow=template$yrow,
               xrange=template$xrange, yrange=template$yrange,
               unitname=template$unitname)
  return(result)
}

#
#  hyperframe.R
#
# $Revision: 1.73 $  $Date: 2020/04/29 12:37:48 $
#

hyperframe <- local({

  hyperframe <- function(...,
                         row.names=NULL, check.rows=FALSE, check.names=TRUE,
                         stringsAsFactors=default.stringsAsFactors()) {
    aarg <- list(...)
    nama <- names(aarg)

    ## number of columns (= variables)
    nvars <- length(aarg)
  
    if(nvars == 0) {
      ## zero columns - return
      result <- list(nvars=0,
                     ncases=0,
                     vname=character(0),
                     vtype=factor(,
                       levels=c("dfcolumn","hypercolumn","hyperatom")),
                     vclass=character(0),
                     df=data.frame(),
                     hyperatoms=list(),
                     hypercolumns=list())
      class(result) <- c("hyperframe", class(result))
      return(result)
    }

    ## check column names
    if(is.null(nama))
      nama <- paste("V", 1:nvars, sep="")
    else if(any(unnamed <- (nama == ""))) 
      nama[unnamed] <- paste("V", seq_len(sum(unnamed)), sep="")
    nama <- make.names(nama, unique=TRUE)
    names(aarg) <- nama
  
    ## Each argument must be either
    ##    - a vector suitable as a column in a data frame
    ##    - a list of objects of the same class
    ##    - a single object of some class
  
    dfcolumns    <- sapply(aarg, is.dfcolumn)
    hypercolumns <- sapply(aarg, is.hypercolumn)
    hyperatoms   <- !(dfcolumns | hypercolumns)

    ## Determine number of rows (= cases) 
    columns <- dfcolumns | hypercolumns
    if(!any(columns)) {
      ncases <- 1
    } else {
      heights <- rep.int(1, nvars)
      heights[columns] <-  lengths(aarg[columns])
      u <- unique(heights)
      if(length(u) > 1) {
        u <- u[u != 1]
        if(length(u) > 1)
          stop(paste("Column lengths are inconsistent:",
                     paste(u, collapse=",")))
      }
      ncases <- u
      if(ncases > 1 && all(heights[dfcolumns] == 1)) {
        ## force the data frame to have 'ncases' rows
        aarg[dfcolumns] <- lapply(aarg[dfcolumns], rep, ncases)
        heights[dfcolumns] <- ncases
      }
      if(any(stubs <- hypercolumns & (heights != ncases))) {
        ## hypercolumns of height 1 should be hyperatoms
        aarg[stubs] <- lapply(aarg[stubs], "[[", i=1L)
        hypercolumns[stubs] <- FALSE
        hyperatoms[stubs] <- TRUE
      }
    }
  
    ## Collect the data frame columns into a data frame
    if(!any(dfcolumns))
      df <- as.data.frame(matrix(, ncases, 0), row.names=row.names)
    else {
      df <- do.call(data.frame,
                    append(aarg[dfcolumns],
                           list(row.names=row.names,
                                check.rows=check.rows,
                                check.names=check.names,
                                stringsAsFactors=stringsAsFactors)))
      names(df) <- nama[dfcolumns]
    }

    ## Storage type of each variable
    vtype <- character(nvars)
    vtype[dfcolumns] <- "dfcolumn"
    vtype[hypercolumns] <- "hypercolumn"
    vtype[hyperatoms] <- "hyperatom"
    vtype=factor(vtype, levels=c("dfcolumn","hypercolumn","hyperatom"))

    ## Class of each variable
    vclass <- character(nvars)
    if(any(dfcolumns))
      vclass[dfcolumns] <- unlist(lapply(as.list(df), class1))
    if(any(hyperatoms))
      vclass[hyperatoms] <- unlist(lapply(aarg[hyperatoms], class1))
    if(any(hypercolumns))
      vclass[hypercolumns] <- unlist(lapply(aarg[hypercolumns], class1of1))
    ## Put the result together
    result <- list(nvars=nvars,
                   ncases=ncases,
                   vname=nama,
                   vtype=vtype,
                   vclass=vclass,
                   df=df,
                   hyperatoms=aarg[hyperatoms],
                   hypercolumns=aarg[hypercolumns])
    
    class(result) <- c("hyperframe", class(result))
    return(result)
  }

  dateclasses <- 
  
  is.dfcolumn <- function(x) {
    is.atomic(x) &&
      (is.vector(x) ||
       is.factor(x) ||
       inherits(x, c("POSIXlt", "POSIXct", "Date")))
  }
  
  is.hypercolumn <- function(x) {
    if(!is.list(x))
      return(FALSE)
    if(inherits(x, c("listof", "anylist")))
      return(TRUE)
    if(length(x) <= 1)
      return(TRUE)
    cla <- lapply(x, class)
    return(length(unique(cla)) == 1)
  }

  class1 <- function(x) { class(x)[1L] }

  class1of1 <- function(x) { class(x[[1L]])[1L] }
  
  hyperframe
})


is.hyperframe <- function(x) inherits(x, "hyperframe")

print.hyperframe <- function(x, ...) {
  ux <- unclass(x)
  nvars <- ux$nvars
  ncases <- ux$ncases
  if(nvars * ncases == 0) {
    splat("NULL hyperframe with", ncases,
          ngettext(ncases, "row (=case)", "rows (=cases)"),
          "and", nvars,
          ngettext(nvars, "column (=variable)", "columns (=variables)"))
  } else {
    if(waxlyrical('gory')) cat("Hyperframe:\n")
    print(as.data.frame(x, discard=FALSE), ...)
  }
  return(invisible(NULL))
}

dim.hyperframe <- function(x) {
  with(unclass(x), c(ncases, nvars))
}

summary.hyperframe <- function(object, ..., brief=FALSE) {
  x <- unclass(object)
  y <- list(
            nvars = x$nvars,
            ncases = x$ncases,
            dim = c(x$ncases, x$nvars),
            typeframe = data.frame(VariableName=x$vname, Class=x$vclass),
            storage = x$vtype,
            col.names = x$vname)
  classes <- x$vclass
  names(classes) <- x$vname
  y$classes <- classes
  # Ordinary data frame columns
  df <- x$df
  y$dfnames <- colnames(df)
  y$df <- if(length(df) > 0 && !brief) summary(df) else NULL
  y$row.names <- row.names(df)
  # insert into full array
  if(!brief && x$nvars > 0) {
    isobject <- (x$vtype != "dfcolumn")
    nobj <- sum(isobject)
    if(nobj == 0) {
      allcols <- y$df
    } else {
      nas <- rep(list(NA_character_), nobj)
      names(nas) <- x$vname[isobject]
      allcols <- do.call(cbind, append(list(y$df), nas))
      acnames <- c(colnames(df), names(nas))
      allcols <- allcols[ , match(x$vname, acnames), drop=FALSE]
    }
    pclass <- padtowidth(paren(classes), colnames(allcols), justify="right")
    allcols <- as.table(rbind(class=pclass, as.table(allcols)))
    row.names(allcols) <- rep("", nrow(allcols))
    y$allcols <- allcols
  }
  class(y) <- c("summary.hyperframe", class(y))
  return(y)
}

print.summary.hyperframe <- function(x, ...) {
  nvars <- x$nvars
  ncases <- x$ncases
  splat(if(nvars * ncases == 0) "NULL hyperframe" else "hyperframe",
        "with", ncases,
        ngettext(ncases, "row", "rows"),
        "and", nvars,
        ngettext(nvars, "column", "columns"))
  if(nvars == 0)
    return(invisible(NULL))
  print(if(any(x$storage == "dfcolumn")) x$allcols else noquote(x$classes))
  return(invisible(NULL))
}

names.hyperframe <- function(x) { unclass(x)$vname }

"names<-.hyperframe" <- function(x, value) {
  x <- unclass(x)
  stopifnot(is.character(value))
  value <- make.names(value)
  if(length(value) != x$nvars)
    stop("Incorrect length for vector of names")
  vtype <- x$vtype
  names(x$df)           <- value[vtype == "dfcolumn"]
  names(x$hyperatoms)   <- value[vtype == "hyperatom"]
  names(x$hypercolumns) <- value[vtype == "hypercolumn"]
  x$vname <- value
  class(x) <- c("hyperframe", class(x))
  return(x)
}

row.names.hyperframe <- function(x) {
  return(row.names(unclass(x)$df))
}

"row.names<-.hyperframe" <- function(x, value) {
  y <- unclass(x)
  row.names(y$df) <- value
  class(y) <- c("hyperframe", class(y))
  return(y)
}

dimnames.hyperframe <- function(x) {
  ux <- unclass(x)
  return(list(row.names(ux$df), ux$vname))
}

"dimnames<-.hyperframe" <- function(x, value) {
  if(!is.list(value) || length(value) != 2 || !all(sapply(value, is.character)))
    stop("Invalid 'dimnames' for a hyperframe", call.=FALSE)
  rn <- value[[1L]]
  cn <- value[[2L]]
  d <- dim(x)
  if(length(rn) != d[1L])
    stop(paste("Row names have wrong length:",
               length(rn), "should be", d[1L]),
         call.=FALSE)
  if(length(cn) != d[2L])
    stop(paste("Column names have wrong length:",
               length(cn), "should be", d[2L]),
         call.=FALSE)
  y <- unclass(x)
  row.names(y$df) <- value[[1L]]
  y$vname <- value[[2]]
  class(y) <- c("hyperframe", class(y))
  return(y)
}

## conversion to hyperframe

as.hyperframe <- function(x, ...) {
  UseMethod("as.hyperframe")
}

as.hyperframe.hyperframe <- function(x, ...) {
  return(x)
}

as.hyperframe.data.frame <- function(x, ..., stringsAsFactors=FALSE) {
  xlist <- if(missing(x)) NULL else as.list(x)
  do.call(hyperframe,
          resolve.defaults(
                           xlist,
                           list(...),
                           list(row.names=rownames(x),
                                stringsAsFactors=stringsAsFactors),
                           .StripNull=TRUE))
}

as.hyperframe.anylist <- 
as.hyperframe.listof <- function(x, ...) {
  if(!missing(x)) {
    xname <- sensiblevarname(short.deparse(substitute(x)), "x")
    xlist <- list(x)
    names(xlist) <- xname
  } else xlist <- NULL
  do.call(hyperframe,
          resolve.defaults(
                           xlist,
                           list(...),
                           list(row.names=rownames(x)),
                           .StripNull=TRUE))
}

as.hyperframe.default <- function(x, ...) {
  as.hyperframe(as.data.frame(x, ...))
}

#### conversion to other types

as.data.frame.hyperframe <- function(x, row.names = NULL,
                                     optional = FALSE, ...,
                                     discard=TRUE, warn=TRUE) {
  ux <- unclass(x)
  if(is.null(row.names))
    row.names <- row.names(ux$df)
  vtype <- ux$vtype
  vclass <- ux$vclass
  dfcol <- (vtype == "dfcolumn")
  if(discard) { 
    nhyper <- sum(!dfcol)
    if(nhyper > 0 && warn)
      warning(paste(nhyper, 
                    ngettext(nhyper, "variable", "variables"),
                    "discarded in conversion to data frame"))
    df <- as.data.frame(ux$df, row.names=row.names, optional=optional, ...)
  } else {
    lx <- as.list(x)
    nrows <- ux$ncases
    vclassstring <- paren(vclass)
    if(any(!dfcol)) 
      lx[!dfcol] <- lapply(as.list(vclassstring[!dfcol]),
                           rep.int, times=nrows)
    df <- do.call(data.frame, append(lx, list(row.names=row.names)))
    colnames(df) <- ux$vname
  }
  return(df)
}

as.list.hyperframe <- function(x, ...) {
  ux <- unclass(x)
  out <- vector(mode="list", length=ux$nvars)
  vtype <- ux$vtype
  df <- ux$df
  if(any(dfcol <- (vtype == "dfcolumn")))
    out[dfcol] <- as.list(df)
  if(any(hypcol <- (vtype == "hypercolumn"))) {
    hc <- lapply(ux$hypercolumns, as.solist, demote=TRUE)
    out[hypcol] <- hc
  }
  if(any(hatom <- (vtype == "hyperatom"))) {
    ha <- ux$hyperatoms
    names(ha) <- NULL
    hacol <- lapply(ha, list)
    hacol <- lapply(hacol, rep.int, times=ux$ncases)
    hacol <- lapply(hacol, as.solist, demote=TRUE)
    out[hatom] <- hacol
  }
  out <- lapply(out, "names<-", value=row.names(df))
  names(out) <- names(x)
  return(out)
}

# evaluation

# eval.hyper <- function(e, h, simplify=TRUE, ee=NULL) {
#   .Deprecated("with.hyperframe", package="spatstat")
#   if(is.null(ee))
#     ee <- as.expression(substitute(e))
#   with.hyperframe(h, simplify=simplify, ee=ee)
# }

with.hyperframe <- function(data, expr, ..., simplify=TRUE, ee=NULL,
                            enclos=NULL) {
  if(!inherits(data, "hyperframe"))
    stop("data must be a hyperframe")
  if(is.null(ee))
    ee <- as.expression(substitute(expr))
  if(is.null(enclos))
    enclos <- parent.frame()
  n <- nrow(data)
  out <- vector(mode="list", length=n)
  datalist <- as.list(data)
  for(i in 1:n) {
    rowi <- lapply(datalist, "[[", i=i)  # ensures the result is always a list
    outi <- eval(ee, rowi, enclos)
    if(!is.null(outi))
      out[[i]] <- outi
  }
  names(out) <- row.names(data)
  if(simplify && all(unlist(lapply(out, is.vector)))) {
    # if all results are atomic vectors of equal length,
    # return a matrix or vector.
    lenfs <- lengths(out)
    if(all(unlist(lapply(out, is.atomic))) &&
            length(unique(lenfs)) == 1) {
      out <- t(as.matrix(as.data.frame(out)))
      row.names(out) <- row.names(data)
      out <- out[,,drop=TRUE]
      return(out)
    }
  }
  out <- hyperframe(result=out, row.names=row.names(data))$result
  return(out)
}

cbind.hyperframe <- function(...) {
  aarg <- list(...)
  narg <- length(aarg)
  if(narg == 0)
    return(hyperframe())
  namarg <- names(aarg)
  if(is.null(namarg))
    namarg <- rep.int("", narg)
  ishyper <- unlist(lapply(aarg, inherits, what="hyperframe"))
  isdf <- unlist(lapply(aarg, inherits, what="data.frame"))
  columns <- list()
  for(i in 1:narg) {
    if(ishyper[i] || isdf[i]){
      if(ncol(aarg[[i]]) > 0) {
        newcolumns <- as.list(aarg[[i]])
        if(namarg[i] != "")
          names(newcolumns) <- paste(namarg[i], ".", names(newcolumns), sep="")
        columns <- append(columns, newcolumns)
      }
    } else {
      nextcolumn <- list(aarg[[i]])
      names(nextcolumn) <- namarg[i]
      columns <- append(columns, nextcolumn)
    }
  }
  result <- do.call(hyperframe, columns)
  ## tack on row names
  rona <- lapply(aarg, row.names)
  good <- (lengths(rona) == nrow(result))
  if(any(good)) {
    rona <- rona[[min(which(good))]]
    row.names(result) <- make.names(rona, unique=TRUE)
  }
  return(result)
}

rbind.hyperframe <- function(...) {
  argh <- list(...)
  if(length(argh) == 0)
    return(NULL)
  # convert them all to hyperframes
  argh <- lapply(argh, as.hyperframe)
  #
  nargh <- length(argh)
  if(nargh == 1)
    return(argh[[1L]])
  # check for compatibility of dimensions & names
  dfs <- lapply(argh, as.data.frame, discard=FALSE)
  dfall <- do.call(rbind, dfs)
  # check that data frame columns also match
  dfs0 <- lapply(argh, as.data.frame, discard=TRUE, warn=FALSE)
  df0all <- do.call(rbind, dfs0)
  # assemble data
  rslt <- list()
  nam <- names(dfall) 
  nam0 <- names(df0all)
  for(k in seq_along(nam)) {
    nama <- nam[k]
    if(nama %in% nam0) {
      # data frame column: already made
      rslt[[k]] <- dfall[,k]
    } else {
      ## hypercolumns or hyperatoms: extract them
      hdata <- lapply(argh, "[", j=nama, drop=FALSE)
      hdata <- lapply(lapply(hdata, as.list), getElement, name=nama)
      ## bind them
      hh <- Reduce(append, hdata)
      rslt[[k]] <- hh
    }
  }
  ## make hyperframe
  names(rslt) <- nam
  rona <- row.names(dfall)
  out <- do.call(hyperframe, append(rslt,
                                    list(stringsAsFactors=FALSE,
                                         row.names=rona)))
  return(out)
}

plot.hyperframe <-
  function(x, e, ..., main, arrange=TRUE,
           nrows=NULL, ncols=NULL,
           parargs=list(mar=mar * marsize),
           marsize=1, mar=c(1,1,3,1)) {
  xname <- short.deparse(substitute(x))
  main <- if(!missing(main)) main else xname
  mar <- rep(mar, 4)[1:4]
  
  if(missing(e)) {
    # default: plot first column that contains objects
    ok <- (summary(x)$storage %in% c("hypercolumn", "hyperatom"))
    if(any(ok)) {
      j <- min(which(ok))
      x <- x[,j, drop=TRUE, strip=FALSE]
      x <- as.solist(x, demote=TRUE)
      plot(x, ..., main=main, arrange=arrange, nrows=nrows, ncols=ncols)
      return(invisible(NULL))
    } else {
      # hyperframe does not contain any objects
      # invoke plot.data.frame
      x <- as.data.frame(x)
      plot(x, ..., main=main)
      return(invisible(NULL))
    }
  }

  if(!is.language(e))
    stop(paste("Argument e should be a call or an expression;",
               "use quote(...) or expression(...)"))
  ee <- as.expression(e)

  if(!arrange) {
    # No arrangement specified: just evaluate the plot expression 'nr' times
    with(x, ee=ee)
    return(invisible(NULL))
  }

  # Arrangement
  # Decide whether to plot a main header
  banner <- (sum(nchar(as.character(main))) > 0)
  if(length(main) > 1)
    main <- paste(main, collapse="\n")
  nlines <- if(!is.character(main)) 1 else length(unlist(strsplit(main, "\n")))
  # determine arrangement of plots
  # arrange like mfrow(nrows, ncols) plus a banner at the top
  n <- summary(x)$ncases
  if(is.null(nrows) && is.null(ncols)) {
    nrows <- as.integer(floor(sqrt(n)))
    ncols <- as.integer(ceiling(n/nrows))
  } else if(!is.null(nrows) && is.null(ncols))
    ncols <- as.integer(ceiling(n/nrows))
  else if(is.null(nrows) && !is.null(ncols))
    nrows <- as.integer(ceiling(n/ncols))
  else stopifnot(nrows * ncols >= length(x))
  nblank <- ncols * nrows - n
  # declare layout
  mat <- matrix(c(seq_len(n), numeric(nblank)),
                byrow=TRUE, ncol=ncols, nrow=nrows)
  heights <- rep.int(1, nrows)
  if(banner) {
    # Increment existing panel numbers
    # New panel 1 is the banner
    panels <- (mat > 0)
    mat[panels] <- mat[panels] + 1L
    mat <- rbind(rep.int(1,ncols), mat)
    heights <- c(0.1 * (1 + nlines), heights)
  }
  # initialise plot
  layout(mat, heights=heights)
  # plot banner
  if(banner) {
    opa <- par(mar=rep.int(0,4), xpd=TRUE)
    plot(numeric(0),numeric(0),type="n",ann=FALSE,axes=FALSE,
         xlim=c(-1,1),ylim=c(-1,1))
    cex <- resolve.defaults(list(...), list(cex.title=2))$cex.title
    text(0,0,main, cex=cex)
  }
  # plot panels
  npa <- do.call(par, parargs)
  if(!banner) opa <- npa
  with(x, ee=ee)
  # revert
  layout(1)
  par(opa)
  return(invisible(NULL))
}


str.hyperframe <- function(object, ...) {
  d <- dim(object)
  x <- unclass(object)
  argh <- resolve.defaults(list(...), list(nest.lev=0, indent.str="  .."))
  cat(paste("'hyperframe':\t",
            d[1L], ngettext(d[1L], "row", "rows"),
            "and",
            d[2L], ngettext(d[2L], "column", "columns"),
            "\n"))
  nr <- d[1L]
  nc <- d[2L]
  if(nc > 0) {
    vname <- x$vname
    vclass <- x$vclass
    vtype  <- as.character(x$vtype)
    indentstring <- with(argh, paste(rep.int(indent.str, nest.lev), collapse=""))
    for(j in 1:nc) {
      tag <- paste("$", vname[j])
      switch(vtype[j],
             dfcolumn={
               desc <- vclass[j]
               if(nr > 0) {
                 vals <- object[1:min(nr,3),j,drop=TRUE]
                 vals <- paste(paste(format(vals), collapse=" "), "...")
               } else vals <- ""
             },
             hypercolumn=,
             hyperatom={
               desc <- "objects of class"
               vals <- vclass[j]
             })
      cat(paste(paste(indentstring, tag, sep=""),
                ":", desc, vals, "\n"))
    }
  }
  return(invisible(NULL))
}

subset.hyperframe <- function(x, subset, select, ...) {
  stopifnot(is.hyperframe(x))
  r <- if(missing(subset)) {
    rep_len(TRUE, nrow(x))
  } else {
      r <- eval(substitute(
        with(x, e, enclos=parent.frame()),
        list(e=substitute(subset))))
    if (!is.logical(r)) 
      stop("'subset' must be logical")
    r & !is.na(r)
  }
  vars <- if(missing(select)) { 
    TRUE
  } else {
    nl <- as.list(seq_len(ncol(x)))
    names(nl) <- names(x)
    eval(substitute(select), nl, parent.frame())
  }
  nama <- names(x)
  names(nama) <- nama
  vars <- nama[vars]
  z <- x[i=r, j=vars, ...]
  return(z)
}

head.hyperframe <- function (x, n = 6L, ...) {
  stopifnot(length(n) == 1L)
  n <- if(n < 0L) max(nrow(x) + n, 0L) else min(n, nrow(x))
  x[seq_len(n), , drop = FALSE]
}

tail.hyperframe <- function(x, n = 6L, ...) {
  stopifnot(length(n) == 1L)
  nrx <- nrow(x)
  n <- if(n < 0L) max(nrx + n, 0L) else min(n, nrx)
  sel <- seq.int(to = nrx, length.out = n)
  x[sel, , drop = FALSE]
}

edit.hyperframe <- function(name, ...) {
  x <- name
  isdf <- unclass(x)$vtype == "dfcolumn"
  if(!any(isdf)) {
    warning("No columns of editable data", call.=FALSE)
    return(x)
  }
  y <- x[,isdf]
  ynew <- edit(as.data.frame(y), ...)
  xnew <- x
  for(na in names(ynew)) xnew[,na] <- ynew[,na]
  losenames <- setdiff(names(y), names(ynew))
  for(na in losenames) xnew[,na] <- NULL
  return(xnew)
}

#
#  colourtools.R
#
#   $Revision: 1.21 $   $Date: 2019/04/05 09:20:59 $
#


rgb2hex <- function(v, maxColorValue=255) {
  stopifnot(is.numeric(v))
  if(!is.matrix(v))
    v <- matrix(v, nrow=1L)
  if(ncol(v) %in% c(3, 4)) {
    out <- rgb(v, maxColorValue=maxColorValue)
    return(out)
  } 
  stop("v should be a vector of length 3 or 4, or a matrix with 3 or 4 columns")
}

rgb2hsva <- function(red, green=NULL, blue=NULL, alpha=NULL,
                     maxColorValue=255) {
  if(is.null(green) && is.null(blue) && is.null(alpha)) {
    ## red should be a 3-row matrix of RGB values
    ## or a 4-row matrix of RGBA values 
    if(!is.matrix(red))
      red <- matrix(red, ncol=1L)
    ## check for an alpha channel
    if(nrow(red) == 4) {
      alpha <- red[4L,]
      red <- red[-4L, , drop=FALSE]
    }
  }
  y <- rgb2hsv(red, green, blue, maxColorValue=maxColorValue)
  if(!is.null(alpha))
    y <- rbind(y, alpha=alpha/maxColorValue)
  return(y)
}
 
col2hex <- function(x) {
  # convert to RGBA
  y <- col2rgb(x, alpha=TRUE)
  # remove alpha channel if all colours are opaque
  if(all(y["alpha", ] == 255))
    y <- y[1:3, , drop=FALSE]
  # convert to hex 
  z <- rgb2hex(t(y))
  return(z)
}

paletteindex <- function(x) {
  x <- col2hex(x)
  p <- col2hex(palette())
  m <- match(x, p)
  return(m)
}

is.colour <- function(x) {
  if(length(x) == 0) return(FALSE)
  cx <- try(col2rgb(x), silent=TRUE)
  bad <- inherits(cx, "try-error")
  return(!bad)
}

samecolour <- function(x, y) { col2hex(x) == col2hex(y) }

complementarycolour <- function(x) {
  if(is.null(x)) return(NULL)
  if(inherits(x, "colourmap")) {
    colouroutputs(x) <- complementarycolour(colouroutputs(x))
    return(x)
  }
  # convert to RGBA
  y <- col2rgb(x, alpha=TRUE)
  # complement of R, G, B
  y[1:3, ] <- 255 - y[1:3, ]
  # convert to colours
  z <- rgb2hex(t(y))
  return(z)
}

is.grey <- function(x) {
  if(inherits(x, "colourmap")) x <- colouroutputs(x)
  if(is.function(x)) return(NA)
  y <- rgb2hsva(col2rgb(x, alpha=TRUE))
  sat <- y["s", ]
  alp <- y["alpha", ]
  return(sat == 0 & alp == 1)
}

to.opaque <- function(x) {
  if(all(!is.na(paletteindex(x))))
    return(x) # preserve palette colours
  rgb(t(col2rgb(x)), maxColorValue=255)
}

to.transparent <- function(x, fraction) {
  if(all(fraction == 1))
    return(to.opaque(x))
  rgb(t(col2rgb(x))/255, alpha=fraction, maxColorValue=1)
}

to.saturated <- function(x, s=1) {
  y <- rgb2hsv(col2rgb(x))
  ## map grey to black, otherwise saturate the colour
  notwhite <- !(y["h",] == 0 & y["s",] == 0 & y["v", ] == 1)
  isgrey <- (y["s", ] == 0) 
  y["v",  isgrey & notwhite] <- 0
  y["s", !isgrey & notwhite] <- s
  ## convert back
  z <- hsv(y["h",], y["s",], y["v",])
  return(z)
}
  
to.grey <- function(x, weights=c(0.299, 0.587, 0.114), transparent=FALSE) {
  if(is.null(x)) return(NULL)
  if(inherits(x, "colourmap")) {
    colouroutputs(x) <- to.grey(colouroutputs(x),
                                weights=weights, transparent=transparent)
    return(x)
  }
  if(is.function(x)) {
    f <- x
    g <- function(...) to.grey(f(...), weights=weights, transparent=transparent)
    return(g)
  }
  ## preserve palette indices, if only using black/grey
  if(all(!is.na(paletteindex(x))) && all(is.grey(x)))
    return(x)
  if(!transparent) {
    y <- col2rgb(x)
    g <- (weights %*% y)/(255 * sum(weights))
    z <- grey(g)
  } else {
    yy <- col2rgb(x, alpha=TRUE)
    y <- yy[1:3, , drop=FALSE]
    g <- (weights %*% y)/(255 * sum(weights))
    z <- grey(g, alpha=yy[4L,]/255.0)
  }
  return(z)
}

is.col.argname <- function(x) {
  return(nzchar(x) & ((x == "col") | (substr(x, 1L, 4L) == "col.")))
}

col.args.to.grey <- function(x, ...) {
  if(any(hit <- is.col.argname(names(x))))
    x[hit] <- lapply(x[hit], to.grey, ...)
  return(x)
}

# versions of rgb() and hsv() that work with NA values

rgbNA <- function(red, green, blue, alpha=NULL, maxColorValue=1) {
  df <- if(is.null(alpha)) data.frame(red=red, green=green, blue=blue) else
        data.frame(red=red, green=green, blue=blue, alpha=alpha)
  result <- rep(NA_character_, nrow(df))
  ok <- complete.cases(df)
  result[ok] <- if(is.null(alpha)) {
    with(df, rgb(red[ok], green[ok], blue[ok],
                 maxColorValue=maxColorValue))
  } else {
    with(df, rgb(red[ok], green[ok], blue[ok], alpha[ok],
                 maxColorValue=maxColorValue))
  }
  return(result)
}

hsvNA <- function(h, s, v, alpha=NULL) {
  df <- if(is.null(alpha)) data.frame(h=h, s=s, v=v) else
                           data.frame(h=h, s=s, v=v, alpha=alpha)
  result <- rep(NA_character_, nrow(df))
  ok <- complete.cases(df)
  result[ok] <- if(is.null(alpha)) {
    with(df, hsv(h[ok], s[ok], v[ok]))
  } else {  
    with(df, hsv(h[ok], s[ok], v[ok], alpha[ok]))
  }
  return(result)
}

## This function traps the colour arguments
## and converts to greyscale if required.

do.call.plotfun <- function(fun, arglist, ...) {
  if(spatstat.options("monochrome")) {
    keys <- names(arglist)
    if(!is.null(keys)) {
      cols <- nzchar(keys) & ((keys %in% c("border", "col", "fg", "bg")) |
                              (substr(keys, 1, 4) == "col."))
      if(any(cols))
        arglist[cols] <- lapply(arglist[cols], to.grey)
    }
  }
  do.call.matched(fun, arglist, ...)
}

gammabreaks <- function(ra, n, gamma=1) {
  # make breaks for x which are evenly spaced on the scale y = x^gamma
  check.1.real(gamma)
  stopifnot(gamma > 0)
  y <- seq(from=0, to=1, length.out=n)
  breaks <- ra[1L] + diff(ra) * y^(1/gamma)
  breaks[1L] <- ra[1L]
  breaks[n]  <- ra[2L]
  return(breaks)
}

##
##   Math.linim.R
##
##   $Revision: 1.8 $ $Date: 2020/04/09 01:59:33 $
##

Ops.linim <- function(e1,e2=NULL){
    unary <- nargs() == 1L
    if(unary){
        if(!is.element(.Generic, c("!", "-", "+")))
            stop("Unary usage is undefined for this operation for images.")
        callstring <- paste(.Generic, "e1")
    } else {
        callstring <- paste("e1", .Generic, "e2")
    }
    expr <- parse(text = callstring)
    return(do.call(eval.linim, list(expr = expr)))
}

Math.linim <- function(x, ...){
    m <- do.call(.Generic, list(x[,,drop=FALSE], ...))
    Z <- im(m, xcol = x$xcol, yrow = x$yrow, xrange = x$xrange,
            yrange = x$yrange, unitname = unitname(x))
    df <- attr(x, "df")
    df$values <- do.call(.Generic, list(df$values, ...))
    L <- attr(x, "L")
    rslt <- linim(L, Z, df=df, restrict=FALSE)
    return(rslt)
}

Summary.linim <- function(..., na.rm, finite){
  if(missing(finite)) finite <- FALSE
  if(missing(na.rm)) na.rm <- FALSE
  argh <- list(...)
  values <- lapply(argh, "[")
  dfvalues <- if(is.element(.Generic, c("sum", "prod"))) list() else 
              lapply(lapply(argh, attr, which="df"), getElement, name="values")
  vals <- unlist(c(values, dfvalues))
  logique <- is.element(.Generic, c("all", "any"))
  vals <- if(logique) as.logical(vals) else as.numeric(vals)
  if(finite && !logique) {
    vals <- vals[is.finite(vals)]
  } else if(na.rm) {
    vals <- vals[!is.na(vals)]
  }
  do.call(.Generic, list(vals))
}

Complex.linim <- function(z){
    L <- attr(z, "L")
    df <- attr(z, "df")
    m <- do.call(.Generic, list(z=z[,,drop=FALSE]))
    Z <- im(m, xcol = z$xcol, yrow = z$yrow, xrange = z$xrange,
               yrange = z$yrange, unitname = unitname(z))
    df$values <- do.call(.Generic, list(z=df$values))
    rslt <- linim(L, Z, df=df, restrict=FALSE)
    return(rslt)
}

#'
#'   hotrod.R
#'
#'  Heat kernel for a one-dimensional rod
#'
#'  Copyright (c) Greg McSwiggan and Adrian Baddeley 2017-2020
#'
#'  $Revision: 1.2 $ $Date: 2020/04/05 03:44:31 $

hotrod <- function(len, xsource, xquery, sigma, 
                   ends=c("insulated", "absorbing"),
                   nmax=20) {
  ends <- match.arg(ends)
  len <- as.numeric(len)
  xsource <- as.numeric(xsource)
  xquery <- as.numeric(xquery)
  sigma <- as.numeric(sigma)
  nmax <- as.integer(nmax)
  df <- data.frame(len=len, xsource=xsource, xquery=xquery,
                   sigma=sigma, nmax=nmax)
  n <- nrow(df)
  switch(ends,
         insulated = {
           z <- with(df,
                     .C("hotrodInsul",
                        n = as.integer(n),
                        a = as.double(len),
                        x = as.double(xsource),
                        y = as.double(xquery),
                        s = as.double(sigma),
                        m = as.integer(nmax),
                        z = as.double(numeric(n)),
                        PACKAGE="spatstat")$z)
         },
         absorbing = {
           z <- with(df,
                     .C("hotrodAbsorb",
                        n = as.integer(n),
                        a = as.double(len),
                        x = as.double(xsource),
                        y = as.double(xquery),
                        s = as.double(sigma),
                        m = as.integer(nmax),
                        z = as.double(numeric(n)),
                        PACKAGE="spatstat")$z)
         })
  return(z)
}

#' indicator function for window

as.function.owin <- function(x, ...) {
  W <- x
  g <- function(x, y=NULL) {
    xy <- xy.coords(x, y)
    inside.owin(xy$x, xy$y, W)
  }
  class(g) <- c("indicfun", class(g))
  return(g)
}

print.indicfun <- function(x, ...) {
  W <- get("W", envir=environment(x))
  nama <- names(formals(x))
  splat(paste0("function", paren(paste(nama, collapse=","))))
  splat("Indicator function (returns 1 inside window, 0 outside)")
  print(W)
  return(invisible(NULL))
}

plot.indicfun <- function(x, W, ..., main) {
  if(missing(main)) main <- short.deparse(substitute(x))
  if(missing(W) || is.null(W)) {
    w <- get("W", envir=environment(x))
    R <- Frame(w)
    W <- grow.rectangle(R, min(sidelengths(R))/5)
  }
  result <- do.as.im(x, plot, W=W, ..., main=main)
  return(invisible(result))
}

#
# randomseg.R
#
# $Revision: 1.12 $ $Date: 2016/12/01 09:32:41 $
#

rpoisline <- function(lambda, win=owin()) {
  win <- as.owin(win)
  # determine circumcircle
  xr <- win$xrange
  yr <- win$yrange
  xmid <- mean(xr)
  ymid <- mean(yr)
  width <- diff(xr)
  height <- diff(yr)
  rmax <- sqrt(width^2 + height^2)/2
  boundbox <- owin(xmid + c(-1,1) * rmax, ymid + c(-1,1) * rmax)
  # generate poisson lines through circumcircle
  n <- rpois(1, lambda * 2 * pi * rmax)
  if(n == 0) {
    X <- psp(numeric(0), numeric(0), numeric(0), numeric(0),
             marks=integer(0), 
             window=win)
    attr(X, "lines") <- infline(p=numeric(0), theta=numeric(0))
    attr(X, "linemap") <- integer(0)
    return(X)
  }
  theta <- runif(n, max= 2 * pi)
  p <- runif(n, max=rmax)
  # compute intersection points with circle
  q <- sqrt(rmax^2 - p^2)
  co <- cos(theta)
  si <- sin(theta)
  X <- psp(x0= xmid + p * co + q * si,
           y0= ymid + p * si - q * co,
           x1= xmid + p * co - q * si,
           y1= ymid + p * si + q * co,
           marks = seq_len(n),
           window=boundbox, check=FALSE)
  # infinite lines
  L <- infline(p = p + xmid * co + ymid * si,
               theta = theta)
  # clip to window
  X <- X[win]
  # append info
  linemap <- as.integer(marks(X))
  X <- unmark(X)
  attr(X, "lines") <- L
  attr(X, "linemap") <- linemap
  return(X)
}

rlinegrid <- function(angle=45, spacing=0.1, win=owin()) {
  win <- as.owin(win)
  # determine circumcircle
  width <- diff(win$xrange)
  height <- diff(win$yrange)
  rmax <- sqrt(width^2 + height^2)/2
  xmid <- mean(win$xrange)
  ymid <- mean(win$yrange)
  # generate randomly-displaced grid of lines through circumcircle
  u <- runif(1, min=0, max=spacing) - rmax
  if(u >= rmax)   
    return(psp(numeric(0), numeric(0), numeric(0), numeric(0),
               window=win, check=FALSE))
  p <- seq(from=u, to=rmax, by=spacing)
  # compute intersection points with circle
  q <- sqrt(rmax^2 - p^2)
  theta <- pi * ((angle - 90)/180)
  co <- cos(theta)
  si <- sin(theta)
  X <- psp(x0= xmid + p * co + q * si,
           y0= ymid + p * si - q * co,
           x1= xmid + p * co - q * si,
           y1= ymid + p * si + q * co,
           window=owin(xmid+c(-1,1)*rmax, ymid+c(-1,1)*rmax), check=FALSE)
  # clip to window
  X <- X[win]
  return(X)
}

##
##  fardist.R
##
## Farthest distance to boundary
##
##  $Revision: 1.11 $ $Date: 2017/06/05 10:31:58 $

fardist <- function(X, ...) {
  UseMethod("fardist")
}

fardist.owin <- function(X, ..., squared=FALSE) {
  verifyclass(X, "owin")
  M <- as.mask(X, ...)
  V <- if(is.mask(X)) vertices(M) else vertices(X)
  nx <- dim(M)[2L]
  ny <- dim(M)[1L]
  x0 <- M$xcol[1L]
  y0 <- M$yrow[1L]
  xstep <- M$xstep
  ystep <- M$ystep
  if(squared) {
    z <- .C("fardist2grid",
            nx = as.integer(nx),
            x0 = as.double(x0),
            xstep = as.double(xstep),
            ny = as.integer(ny),
            y0 = as.double(y0),
            ystep = as.double(ystep),
            np = as.integer(length(V$x)),
            xp = as.double(V$x),
            yp = as.double(V$y),
            dfar = as.double(numeric(nx * ny)),
            PACKAGE = "spatstat")
  } else {
    z <- .C("fardistgrid",
            nx = as.integer(nx),
            x0 = as.double(x0),
            xstep = as.double(xstep),
            ny = as.integer(ny),
            y0 = as.double(y0),
            ystep = as.double(ystep),
            np = as.integer(length(V$x)),
            xp = as.double(V$x),
            yp = as.double(V$y),
            dfar = as.double(numeric(nx * ny)),
            PACKAGE = "spatstat")
  }
  out <- im(z$dfar, xcol=M$xcol, yrow=M$yrow,
            xrange=M$xrange, yrange=M$yrange, unitname=unitname(M))
  if(!is.rectangle(X))
    out <- out[X, drop=FALSE]
  return(out)
}
  
fardist.ppp <- function(X, ..., squared=FALSE) {
  verifyclass(X, "ppp")
  V <- vertices(Window(X))
  D2 <- crossdist(X$x, X$y, V$x, V$y, squared=TRUE) 
  D2max <- apply(D2, 1L, max)
  if(squared) return(D2max) else return(sqrt(D2max))
}

#
#    rescue.rectangle.R
# 
#    $Revision: 1.7 $   $Date: 2018/09/28 05:05:49 $
#
rescue.rectangle <- function(W) {
  verifyclass(W, "owin")

  if(W$type == "mask" && all(W$m))
     return(owin(W$xrange, W$yrange, unitname=unitname(W)))

  if(W$type == "polygonal" && length(W$bdry) == 1) {
    x <- W$bdry[[1]]$x
    y <- W$bdry[[1]]$y
    if(length(x) == 4 && length(y) == 4) {
      # could be a rectangle
      ux <- veryunique(x)
      uy <- veryunique(y)
      if(length(ux) == 2 && length(uy) == 2)
        return(owin(ux,uy, unitname=unitname(W)))
    }
  }
  return(W)
}

veryunique <- function(z) {
  uz <- sortunique(z)
  epsilon <- 2 * .Machine$double.eps * diff(range(uz))
  close <- (diff(uz) <= epsilon)
  uz <- uz[c(TRUE, !close)]
  return(uz)
}

#
#  nnmap.R
#
#    nearest or k-th nearest neighbour of each pixel
#
#  $Revision: 1.10 $  $Date: 2018/02/11 06:33:09 $
#

nnmap <- function(X, k=1, what = c("dist", "which"), ...,
                  W=as.owin(X),
                  is.sorted.X=FALSE,
                  sortby=c("range", "var", "x", "y")) {
  stopifnot(is.ppp(X))
  sortby <- match.arg(sortby)
  outputarray <- resolve.1.default("outputarray", ..., outputarray=FALSE)

  W <- as.owin(W %orifnull% X)
  huge <- 1.1 * diameter(boundingbox(as.rectangle(X), as.rectangle(W)))
  
  what   <- match.arg(what, choices=c("dist", "which"), several.ok=TRUE)
  want.dist  <- "dist" %in% what 
  want.which <- "which" %in% what
  want.both  <- want.dist && want.which

  if(!missing(k)) {
    # k can be a single integer or an integer vector
    if(length(k) == 0)
      stop("k is an empty vector")
    else if(length(k) == 1) {
      if(k != round(k) || k <= 0)
        stop("k is not a positive integer")
    } else {
      if(any(k != round(k)) || any(k <= 0))
        stop(paste("some entries of the vector",
                   sQuote("k"), "are not positive integers"))
    }
  }
  k <- as.integer(k)
  kmax <- max(k)
  nk <- length(k)

  # note whether W is `really' a rectangle
  isrect <- is.rectangle(rescue.rectangle(W))

  # set up pixel array
  M <- do.call.matched(as.mask,
                       resolve.defaults(list(...), list(w=W)))
  Mdim <- M$dim
  nxcol <- Mdim[2]
  nyrow <- Mdim[1]
  npixel <- nxcol * nyrow
  
  nX <- npoints(X)
  if(nX == 0) {
    # trivial - avoid potential problems in C code
    NND <- if(want.dist) array(Inf, dim=c(nk, Mdim)) else 0
    NNW <- if(want.which) array(NA_integer_, dim=c(nk, Mdim)) else 0
  } else {
    # usual case 
    if(is.sorted.X && !(sortby %in% c("x", "y")))
      stop(paste("If data are already sorted,",
                 "the sorting coordinate must be specified explicitly",
                 "using sortby = \"x\" or \"y\""))

    # decide whether to sort on x or y coordinate
    switch(sortby,
           range = {
             s <- sidelengths(as.rectangle(X))
             sortby.y <- (s[1] < s[2])
           },
           var = {
             sortby.y <- (var(X$x) < var(X$y))
           },
           x={ sortby.y <- FALSE},
           y={ sortby.y <- TRUE}
           )

    # The C code expects points to be sorted by x coordinate.
    if(sortby.y) {
      oldM <- M
      X <- flipxy(X)
      W <- flipxy(W)
      M <- flipxy(M)
      Mdim <- M$dim
      nxcol <- Mdim[2]
      nyrow <- Mdim[1]
    }
    xx <- X$x
    yy <- X$y
    # sort only if needed
    if(!is.sorted.X){
      oX <- fave.order(xx)
      xx <- xx[oX]
      yy <- yy[oX]
    }

    # number of neighbours that are well-defined
    kmaxcalc <- min(nX, kmax)

    # prepare to call C code
    nndv <- if(want.dist) numeric(npixel * kmaxcalc) else numeric(1)
    nnwh <- if(want.which) integer(npixel * kmaxcalc) else integer(1)

    # ............. call C code ............................
    
    if(kmaxcalc == 1) {
      zz <- .C("nnGinterface",
               nx = as.integer(nxcol),
               x0 = as.double(M$xcol[1]),
               xstep = as.double(M$xstep),
               ny = as.integer(nyrow),
               y0 = as.double(M$yrow[1]),
               ystep = as.double(M$ystep),
               np = as.integer(nX),
               xp = as.double(xx),
               yp = as.double(yy),
               wantdist = as.integer(want.dist),
               wantwhich = as.integer(want.which),
               nnd = as.double(nndv),
               nnwhich = as.integer(nnwh),
               huge = as.double(huge),
               PACKAGE = "spatstat")
    } else {
      zz <- .C("knnGinterface",
               nx = as.integer(nxcol),
               x0 = as.double(M$xcol[1]),
               xstep = as.double(M$xstep),
               ny = as.integer(nyrow),
               y0 = as.double(M$yrow[1]),
               ystep = as.double(M$ystep),
               np = as.integer(nX),
               xp = as.double(xx),
               yp = as.double(yy),
               kmax = as.integer(kmaxcalc),
               wantdist = as.integer(want.dist),
               wantwhich = as.integer(want.which),
               nnd = as.double(nndv),
               nnwhich = as.integer(nnwh),
               huge = as.double(huge),
               PACKAGE = "spatstat")
    }
    
    # extract results
    nnW <- zz$nnwhich
    nnD <- zz$nnd
    # map index 0 to NA
    if(want.which && any(uhoh <- (nnW == 0))) {
      nnW[uhoh] <- NA
      if(want.dist) nnD[uhoh] <- Inf
    }
    # reinterpret indices in original ordering
    if(!is.sorted.X) nnW <- oX[nnW]
  
    # reform as arrays 
    NND <- if(want.dist) array(nnD, dim=c(kmaxcalc, Mdim)) else 0
    NNW <- if(want.which) array(nnW, dim=c(kmaxcalc, Mdim)) else 0
    if(sortby.y) {
      # flip x and y back again
      if(want.dist) NND <- aperm(NND, c(1, 3, 2))
      if(want.which) NNW <- aperm(NNW, c(1, 3, 2))
      M <- oldM
      Mdim <- dim(M)
    }
    
    # the return value should correspond to the original vector k
    if(kmax > kmaxcalc) {
      # pad with NA / Inf
      if(want.dist) {
        NNDcalc <- NND
        NND <- array(Inf, dim=c(kmax, Mdim))
        NND[1:kmaxcalc, , ] <- NNDcalc
      }
      if(want.which) {
        NNWcalc <- NNW
        NNW <- array(NA_integer_, dim=c(kmax, Mdim))
        NNW[1:kmaxcalc, , ] <- NNWcalc
      }
    }
    if(length(k) < kmax) {
      # select only the specified planes
      if(want.dist)
        NND <- NND[k, , , drop=FALSE]
      if(want.which)
        NNW <- NNW[k, , , drop=FALSE]
    }
  }

  # secret backdoor
  if(outputarray) {
    # return result as an array or pair of arrays
    result <- if(want.both) { list(dist=NND, which=NNW) } else
              if(want.dist) NND else NNW
    attr(result, "pixarea") <- with(M, xstep * ystep)
    return(result)
  }

  # format result as a list of images
  result <- list()
  if(want.dist) {
    dlist <- list()
    for(i in 1:nk) {
      DI <- as.im(NND[i,,], M)
      if(!isrect) DI <- DI[M, drop=FALSE]
      dlist[[i]] <- DI
    }
    names(dlist) <- k
    result[["dist"]] <- if(nk > 1) dlist else dlist[[1]]
  }
  if(want.which) {
    wlist <- list()
    for(i in 1:nk) {
      WI <- as.im(NNW[i,,], M)
      if(!isrect) WI <- WI[M, drop=FALSE]
      wlist[[i]] <- WI
    }
    names(wlist) <- k
    result[["which"]] <- if(nk > 1) wlist else wlist[[1]]
  }
  if(!want.both) result <- result[[1]]
  return(result)
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
##
##    tests/gcc323.R
##
##    $Revision: 1.3 $  $Date: 2020/04/28 12:58:26 $
##
if(ALWAYS) { # depends on hardware
local({
  # critical R values that provoke GCC bug #323
  a <- marktable(lansing, R=0.25)
  a <- marktable(lansing, R=0.21)
  a <- marktable(lansing, R=0.20)
  a <- marktable(lansing, R=0.10)
})
}
#       
#        tests/hobjects.R
#
#   Validity of methods for ppm(... method="ho")
#


if(FULLTEST) {
local({
  set.seed(42)
  fit  <- ppm(cells ~1,         Strauss(0.1), method="ho", nsim=10)
  fitx <- ppm(cells ~offset(x), Strauss(0.1), method="ho", nsim=10)

  a  <- AIC(fit)
  ax <- AIC(fitx)

  f  <- fitted(fit)
  fx <- fitted(fitx)

  p  <- predict(fit)
  px <- predict(fitx)
})
}


#
# tests/hyperframe.R
#
# test "[.hyperframe" etc
#
#  $Revision: 1.8 $  $Date: 2020/04/28 12:58:26 $
#

if(FULLTEST) {
local({
  lambda <- runif(4, min=50, max=100)
  X <- lapply(as.list(lambda), function(x) { rpoispp(x) })
  h <- hyperframe(lambda=lambda, X=X)
  h$lambda2 <- lambda^2
  h[, "lambda3"] <- lambda^3
  h[, "Y"] <- X
  h[, "X"] <- lapply(X, flipxy)
  h[, c("X", "Y")] <- hyperframe(X=X, Y=X)

  names(h) <- LETTERS[1:5]
  print(h)

  summary(h)
  str(h)
  head(h)
  tail(h)

  rn <- rownames(h)
  r.n <- row.names(h)
  if(!identical(rn, r.n))
    stop("rownames and row.names conflict for hyperframes")

  dn <- dimnames(h)
  dimnames(h) <- dn
  dimnames(h)[[2]][2] <- "copacetic"
  dimnames(h)[[1]][2] <- "second"

  #' hyperframe with a hyperatom
  H <- hyperframe(A=runif(3), B=1:3, D=runifpoint(10))
  H[,3]
  H[,3,drop=TRUE]
  #' special cases of [<-
  H$B <- H[,1]
  H[2:3,1] <- H[2:3,2]
  H[2:3,1] <- H[2,2]
  H[2,1:2] <- H[3,1:2]

  #' split
  f <- factor(c("a", "a", "b"))
  G <- split(H, f)
  G[["a"]]$B <- 42
  split(H, f) <- G
})
}
#'     tests/hypotests.R
#'     Hypothesis tests
#' 
#'  $Revision: 1.6 $ $Date: 2020/04/28 12:58:26 $

if(FULLTEST) {
local({
  hopskel.test(redwood, method="MonteCarlo", nsim=5)
  
  berman.test(spiders, "x")
  berman.test(lppm(spiders ~ x), "y")

  #' quadrat test - spatial methods
  a <- quadrat.test(redwood, 3)
  domain(a)
  shift(a, c(1,1))

  #' cases of studpermu.test
  #' X is a hyperframe
  b <- studpermu.test(pyramidal, nperm=9)
  b <- studpermu.test(pyramidal, nperm=9, use.Tbar=TRUE)
  #' X is a list of lists of ppp
  ZZ <- split(pyramidal$Neurons, pyramidal$group)
  bb <- studpermu.test(ZZ, nperm=9)

  #' Issue #115
  X <- runifpoint(50, nsim = 3)
  Y <- runifpoint(3000, nsim = 3)
  h <- hyperframe(ppp = c(X, Y), group = rep(1:2, 3))
  studpermu.test(h, ppp ~ group)

  #' scan test
  Z <- scanmeasure(cells, 0.1, method="fft")
  rr <- c(0.05, 1)
  scan.test(amacrine, rr, nsim=5,
            method="binomial", alternative="less")
  fit <- ppm(cells ~ x)
  lam <- predict(fit)
  scan.test(cells, rr, nsim=5,
            method="poisson", baseline=fit, alternative="less")
  scan.test(cells, rr, nsim=5,
            method="poisson", baseline=lam, alternative="less")
})
}
#
#  tests/imageops.R
#
#   $Revision: 1.29 $   $Date: 2020/04/28 12:58:26 $
#

if(FULLTEST) {
local({
  #' cases of 'im' data
  tab <- table(sample(factor(letters[1:10]), 30, replace=TRUE))
  b <- im(tab, xrange=c(0,1), yrange=c(0,10))
  b <- update(b)

  mat <- matrix(sample(0:4, 12, replace=TRUE), 3, 4)
  a <- im(mat)
  levels(a$v) <- 0:4
  a <- update(a)
  
  levels(mat) <- 0:4
  b <- im(mat)
  b <- update(b)

  D <- as.im(mat, letterR)
  df <- as.data.frame(D)
  DD <- as.im(df, step=c(D$xstep, D$ystep))
  
  #' various manipulations
  AA <- A <- as.im(owin())
  BB <- B <- as.im(owin(c(1.1, 1.9), c(0,1)))
  Z <- imcov(A, B)
  stopifnot(abs(max(Z) - 0.8) < 0.1)

  Frame(AA) <- Frame(B)
  Frame(BB) <- Frame(A)
  
  ## handling images with 1 row or column
  
  ycov <- function(x, y) y
  E <- as.im(ycov, owin(), dimyx = c(2,1))
  G <- cut(E, 2)
  H <- as.tess(G)

  E12 <- as.im(ycov, owin(), dimyx = c(1,2))
  G12 <- cut(E12, 2)
  H12 <- as.tess(G12)

  AAA <- as.array(AA)
  EEE <- as.array(E)
  AAD <- as.double(AA)
  EED <- as.double(E)
  aaa <- xtfrm(AAA)
  eee <- xtfrm(E)
  
  ##
  d <- distmap(cells, dimyx=32)
  Z <- connected(d <= 0.06, method="interpreted")

  a <- where.max(d, first=FALSE)
  a <- where.min(d, first=FALSE)

  dx <- raster.x(d)
  dy <- raster.y(d)
  dxy <- raster.xy(d)
  xyZ <- raster.xy(Z, drop=TRUE)

  horosho <- conform.imagelist(cells, list(d, Z))

  #' split.im
  W <- square(1)
  X <- as.im(function(x,y){x}, W)
  Y <- dirichlet(runifpoint(7, W))
  Z <- split(X, as.im(Y))
  
  ## ...........  cases of "[.im" ........................
  ee  <- d[simplenet, drop=FALSE]
  eev <- d[simplenet]
  Empty <- cells[FALSE]
  EmptyFun <- ssf(Empty, numeric(0))
  ff <- d[Empty]
  ff <- d[EmptyFun]
  gg <- d[2,]
  gg <- d[,2]
  gg <- d[j=2]
  gg <- d[2:4, 3:5]
  hh <- d[2:4, 3:5, rescue=TRUE]
  if(!is.im(hh)) stop("rectangle was not rescued in [.im")
  ## factor and NA values
  f <- cut(d, breaks=4)
  f <- f[f != levels(f)[1], drop=FALSE]
  fff <- f[, , drop=FALSE]
  fff <- f[cells]
  fff <- f[cells, drop=FALSE]
  fff <- f[Empty]

  ## ...........  cases of "[<-.im"  .......................
  d[,] <- d[] + 1
  d[Empty] <- 42
  d[EmptyFun] <- 42
  ## smudge() and rasterfilter()
  dd <- smudge(d)

  ## rgb/hsv options
  X <- setcov(owin())
  M <- Window(X)
  Y <- as.im(function(x,y) x, W=M)
  Z <- as.im(function(x,y) y, W=M)
  # convert after rescaling
  RGBscal <- rgbim(X, Y, Z, autoscale=TRUE, maxColorValue=1)
  HSVscal <- hsvim(X, Y, Z, autoscale=TRUE)

  #' cases of [.im
  Ma <- as.mask(M, dimyx=37)
  ZM <- Z[raster=Ma, drop=FALSE]
  ZM[solutionset(Y+Z > 0.4)] <- NA
  ZF <- cut(ZM, breaks=5)
  ZL <- (ZM > 0)
  P <- list(x=c(0.511, 0.774, 0.633, 0.248, 0.798),
            y=c(0.791, 0.608, 0.337, 0.613, 0.819))
  zmp <- ZM[P, drop=TRUE]
  zfp <- ZF[P, drop=TRUE]
  zlp <- ZL[P, drop=TRUE]
  P <- as.ppp(P, owin())
  zmp <- ZM[P, drop=TRUE]
  zfp <- ZF[P, drop=TRUE]
  zlp <- ZL[P, drop=TRUE]

  #' miscellaneous
  ZZ <- zapsmall.im(Z, digits=6)
  ZZ <- zapsmall.im(Z)

  ZS <- shift(Z, origin="centroid")
  ZS <- shift(Z, origin="bottomleft")

  ZA <- affine(Z, mat=diag(c(-1,-2)))

  U <- scaletointerval(Z)
  C <- as.im(1, W=U)
  U <- scaletointerval(C)
  
  #' hist.im
  h <- hist(Z)
  h <- hist(Z, probability=TRUE)
  h <- hist(Z, plot=FALSE)
  Zcut <- cut(Z, breaks=5)
  h <- hist(Zcut) # barplot
  hp <- hist(Zcut, probability=TRUE) # barplot
  plot(h) # plot.barplotdata

  #' plot.im code blocks
  plot(Z, ribside="left")
  plot(Z, ribside="top")
  plot(Z, riblab="value")
  plot(Z, clipwin=square(0.5))
  plot(Z - mean(Z), log=TRUE)
  plot(Z, valuesAreColours=TRUE) # rejected with a warning
  IX <- as.im(function(x,y) { as.integer(round(3*x)) }, square(1))
  co <- colourmap(rainbow(4), inputs=0:3)
  plot(IX, col=co)
  CX <- eval.im(col2hex(IX+1L))
  plot(CX, valuesAreColours=TRUE)
  plot(CX, valuesAreColours=FALSE)

  #' pairs.im 
  pairs(solist(Z))
  pairs(solist(A=Z))
  
  #' handling and plotting of character and factor images
  Afactor    <- as.im(col2hex("green"), letterR, na.replace=col2hex("blue"))
  Acharacter <- as.im(col2hex("green"), letterR, na.replace=col2hex("blue"),
                      stringsAsFactors=FALSE)
  plot(Afactor)
  plot(Acharacter, valuesAreColours=TRUE)
  print(summary(Afactor))
  print(summary(Acharacter))

  #' safelookup (including extrapolation case)
  Z <- as.im(function(x,y) { x - y }, letterR)
  B <- grow.rectangle(Frame(letterR), 1)
  X <- superimpose(runifpoint(10,letterR),
                   runifpoint(20, setminus.owin(B, letterR)),
                   vertices(Frame(B)),
                   W=B)
  a <- safelookup(Z, X)
  b <- safelookup(cut(Z, breaks=4), X)
  aa <- lookup.im(Z, X)
  
  #' Smooth.im -> blur.im with sigma=NULL
  ZS <- Smooth(Z)
  
  #' cases of distcdf
  distcdf(cells[1:5])
  distcdf(W=cells[1:5], dW=1:5)
  distcdf(W=Window(cells), V=cells[1:5])
  distcdf(W=Window(cells), V=cells[1:5], dV=1:5)

  #' im.apply
  DA <- density(split(amacrine))
  Z <- im.apply(DA, sd)
  Z <- which.max.im(DA) # deprecated -> im.apply(DA, which.max)

  #' Math.imlist, Ops.imlist, Complex.imlist
  U <- Z+2i
  B <- U * (2+1i)
  print(summary(B))
  V <- solist(A=U, B=B)
  negV <- -V
  E <- Re(V)
  negE <- -E

  #' rotmean
  U <- rotmean(Z, origin="midpoint", result="im", padzero=FALSE)
})
}

if(ALWAYS) {
  local({
    #' check nearest.valid.pixel
    W <- Window(demopat)
    set.seed(911911)
    X <- runifpoint(1000, W)
    Z <- quantess(W, function(x,y) { x }, 9)$image
    nearest.valid.pixel(numeric(0), numeric(0), Z)
    x <- X$x
    y <- X$y
    a <- nearest.valid.pixel(x, y, Z, method="interpreted")
    b <- nearest.valid.pixel(x, y, Z, method="C")
    if(!isTRUE(all.equal(a,b)))
      stop("Unequal results in nearest.valid.pixel")
      if(!identical(a,b)) 
        stop("Equal, but not identical, results in nearest.valid.pixel")
  })
}

#'
#'  tests/interact.R
#'
#'  Support for interaction objects
#'
#'  $Revision: 1.2 $ $Date: 2020/04/28 12:58:26 $

if(FULLTEST) {
local({
  #' print.intermaker
  Strauss
  Geyer
  Ord
  #' intermaker
  BS <- get("BlankStrauss", envir=environment(Strauss))
  BD <- function(r) { instantiate.interact(BS, list(r=r)) }
  BlueDanube <- intermaker(BD, BS) 
})
}

#'   tests/ippm.R
#'   Tests of 'ippm' class
#'   $Revision: 1.6 $ $Date: 2020/04/28 12:58:26 $

if(FULLTEST) {
local({
  # .......... set up example from help file .................
  nd <- 10
  gamma0 <- 3
  delta0 <- 5
  POW <- 3
  # Terms in intensity
  Z <- function(x,y) { -2*y }
  f <- function(x,y,gamma,delta) { 1 + exp(gamma - delta * x^POW) }
  # True intensity
  lamb <- function(x,y,gamma,delta) { 200 * exp(Z(x,y)) * f(x,y,gamma,delta) }
  # Simulate realisation
  lmax <- max(lamb(0,0,gamma0,delta0), lamb(1,1,gamma0,delta0))
  set.seed(42)
  X <- rpoispp(lamb, lmax=lmax, win=owin(), gamma=gamma0, delta=delta0)
  # Partial derivatives of log f
  DlogfDgamma <- function(x,y, gamma, delta) {
    topbit <- exp(gamma - delta * x^POW)
    topbit/(1 + topbit)
  }
  DlogfDdelta <- function(x,y, gamma, delta) {
    topbit <- exp(gamma - delta * x^POW)
    - (x^POW) * topbit/(1 + topbit)
  }
  # irregular score
  Dlogf <- list(gamma=DlogfDgamma, delta=DlogfDdelta)
  # fit model
  fit <- ippm(X ~Z + offset(log(f)),
              covariates=list(Z=Z, f=f),
              iScore=Dlogf,
              start=list(gamma=1, delta=1),
              nd=nd)
  # fit model with logistic likelihood but without iScore
  fitlo <- ippm(X ~Z + offset(log(f)),
                method="logi",
                covariates=list(Z=Z, f=f),
                start=list(gamma=1, delta=1),
                nd=nd)

  ## ............. test ippm class support ......................
  Ar <- model.matrix(fit)
  Ai <- model.matrix(fit, irregular=TRUE)
  An <- model.matrix(fit, irregular=TRUE, keepNA=FALSE)
  AS <- model.matrix(fit, irregular=TRUE, subset=(abs(Z) < 0.5))

  Zr <- model.images(fit)
  Zi <- model.images(fit, irregular=TRUE)
  ## update.ippm
  fit2 <- update(fit, . ~ . + I(Z^2))
  fit0 <- update(fit,
                 . ~ . - Z,
                 start=list(gamma=2, delta=4))
  oldfit <- ippm(X,
              ~Z + offset(log(f)),
              covariates=list(Z=Z, f=f),
              iScore=Dlogf,
              start=list(gamma=1, delta=1),
              nd=nd)
  oldfit2 <- update(oldfit, . ~ . + I(Z^2))
  oldfit0 <- update(oldfit,
                    . ~ . - Z,
                    start=list(gamma=2, delta=4))
  ## again with logistic
  fitlo2 <- update(fitlo, . ~ . + I(Z^2))
  fitlo0 <- update(fitlo,
                   . ~ . - Z,
                   start=list(gamma=2, delta=4))
  oldfitlo <- ippm(X,
                   ~Z + offset(log(f)),
                   method="logi",
                   covariates=list(Z=Z, f=f),
                   start=list(gamma=1, delta=1),
                   nd=nd)
  oldfitlo2 <- update(oldfitlo, . ~ . + I(Z^2))
  oldfitlo0 <- update(oldfitlo,
                      . ~ . - Z,
                      start=list(gamma=2, delta=4))
  ## anova.ppm including ippm objects
  fit0 <- update(fit, . ~ Z)
  fit0lo <- update(fitlo, . ~ Z)
  A <- anova(fit0, fit)
  Alo <- anova(fit0lo, fitlo)
})
}

#
#	exactdt.S
#	S function exactdt() for exact distance transform
#
#	$Revision: 4.17 $	$Date: 2017/06/05 10:31:58 $
#

exactdt <- local({

  die <- function(why) { stop(paste("ppp object format corrupted:", why)) }

  exactdt <- function(X, ...) {
    verifyclass(X, "ppp")
    w <- X$window
    if(spatstat.options("exactdt.checks.data")) {
      ## check validity of ppp structure 
      bb <- as.rectangle(w)
      xr <- bb$xrange
      yr <- bb$yrange
      rx <- range(X$x)
      ry <- range(X$y)
      if(rx[1L] < xr[1L] || rx[2L] > xr[2L]) die("x-coordinates out of bounds")
      if(ry[1L] < yr[1L] || ry[2L] > yr[2L]) die("y-coordinates out of bounds")
      if(length(X$x) != length(X$y))
        die("x and y vectors have different length")
      if(length(X$x) != X$n) die("length of x,y vectors does not match n")
    }
    w <- as.mask(w, ...)
    ## dimensions of result
    nr <- w$dim[1L]
    nc <- w$dim[2L]
    ## margins in C array 
    mr <- 2
    mc <- 2
    ## full dimensions of allocated storage
    Nnr <- nr + 2 * mr
    Nnc <- nc + 2 * mc
    N <- Nnr * Nnc
    ## output rows & columns (R indexing)
    rmin <- mr + 1
    rmax <- Nnr - mr
    cmin <- mc + 1
    cmax <- Nnc - mc
    ## go
    res <- .C("exact_dt_R",
              as.double(X$x),
              as.double(X$y),
              as.integer(X$n),
              as.double(w$xrange[1L]),
              as.double(w$yrange[1L]),
              as.double(w$xrange[2L]),
              as.double(w$yrange[2L]),
              nr = as.integer(nr),
              nc = as.integer(nc),
              mr = as.integer(mr),
              mc = as.integer(mc),
              distances = as.double(double(N)),
              indices = as.integer(integer(N)),
              boundary = as.double(double(N)),
              PACKAGE = "spatstat")
    ## extract 
    dist <- matrix(res$distances,
                   ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
    inde <- matrix(res$indices,
                   ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
    bdry <- matrix(res$boundary,
                   ncol=Nnc, nrow=Nnr, byrow = TRUE)[rmin:rmax, cmin:cmax]
    ## convert index from C to R indexing
    inde <- inde + 1L
    return(list(d = dist, i = inde, b = bdry, w=w))
  }

  exactdt
})

#
# clickpoly.R
#
#
# $Revision: 1.10 $  $Date: 2015/10/21 09:06:57 $
#
#

clickpoly <- function(add=FALSE, nv=NULL, np=1, ...) {
  if((!add) | dev.cur() == 1L) {
    plot(0,0,type="n", xlab="", ylab="", xlim=c(0,1), ylim=c(0,1), asp=1.0,
         axes=FALSE)
    rect(0,0,1,1)
  }
  spatstatLocator(0) ## check locator is enabled
  gon <- list()
  stopifnot(np >= 1)
  #
  for(i in 1:np) {
    if(np > 1)
      cat(paste(".... Polygon number", i, ".....\n"))
    if(!is.null(nv)) 
      cat(paste("click", nv, "times in window\n"))
    else
      cat(paste("to add points: click left mouse button in window\n",
                "      to exit: press ESC or click middle mouse button\n",
                "[The last point should NOT repeat the first point]\n"))
    xy <- do.call(spatstatLocator,
                  resolve.defaults(if(!is.null(nv)) list(n=nv) else list(),
                                   list(...),
                                   list(type="o")))
    if(Area.xypolygon(xy) < 0)
      xy <- lapply(xy, rev)
    gon[[i]] <- xy
    plotPolygonBdry(owin(poly=xy), ...)
  }
  result <- owin(poly=gon)
  plotPolygonBdry(result, ...)
  return(result)
}

clickbox <- function(add=TRUE, ...) {
  spatstatLocator(0) # check locator enabled
  cat("Click two corners of a box\n")
  if(!add) plot(owin(), main="Click two corners of a box") 
  a <- try(spatstatLocator(1), silent=TRUE)
  if(inherits(a, "try-error")) {
    ## add=TRUE but there is no current plot
    plot.new()
    a <- spatstatLocator(1, ...)
  }
  abline(v=a$x)
  abline(h=a$y)
  b <- spatstatLocator(1, ...)
  abline(v=b$x)
  abline(h=b$y)
  ab <- concatxy(a, b)
  result <- owin(range(ab$x), range(ab$y))
  plotPolygonBdry(result, ...)
  return(result)
}

plotPolygonBdry <- function(x, ...) {
  # filter appropriate arguments
  argh <- list(...)
  polyPars <- union(graphicsPars("lines"), graphicsPars("owin"))
  polyargs <- argh[names(argh) %in% polyPars]
  # change 'col' to 'border'
  nama <- names(polyargs)
  if(any(nama == "col") && !any(nama == "border"))
    names(polyargs)[nama == "col"] <- "border"
  # plot
  do.call(plot.owin,
          append(list(x=x, add=TRUE), polyargs))
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#'   tests/tessera.R
#'   Tessellation code, not elsewhere tested
#'   $Revision: 1.8 $ $Date: 2020/05/02 01:32:58 $
#'
if(FULLTEST) {
local({
  W <- owin()
  Wsub <- square(0.5)
  X <- runifpoint(7, W)
  A <- dirichlet(X)
  marks(A) <- 1:nobjects(A)
  Z <- distmap(letterR, invert=TRUE)[letterR, drop=FALSE]
  H <- tess(xgrid=0:2, ygrid=0:3)
  #' discretisation of tiles
  V <- as.im(A)
  B <- tess(window=as.mask(W), tiles=tiles(A))
  #' logical images
  D <- tess(image=(Z > 0.2))
  U <- (Z > -0.2) # TRUE or NA
  E <- tess(image=U, keepempty=TRUE)
  G <- tess(image=U, keepempty=FALSE)
  #' methods
  flay <- function(op, ..., Rect=H, Poly=A, Img=E) {
    a <- do.call(op, list(Rect, ...))
    b <- do.call(op, list(Poly, ...))
    e <- do.call(op, list(Img, ...))
  }
  flay(reflect)
  flay(flipxy)
  flay(shift, vec=c(1,2))
  flay(scalardilate, f=2) 
  flay(rotate, angle=pi/3, centre=c(0, 0))
  flay(rotate, angle=pi/2)
  flay(affine, mat=matrix(c(1,2,0,1), 2, 2), vec=c(1,2))
  flay(affine, mat=diag(c(1,2)))
  flay(as.data.frame)
  ##
  unitname(A) <- "km"
  unitname(B) <- c("metre", "metres")
  unitname(B)
  print(B)
  Bsub <- B[c(3,5,7)]
  print(Bsub)
  tilenames(H) <- letters[seq_along(tilenames(H))]
  G <- tess(xgrid=(0:3)/3, ygrid=(0:3)/3)
  tilenames(G) <- letters[1:9]
  h <- tilenames(G)
  GG <- as.tess(tiles(G))
  #'
  Pe <- intersect.tess(A, Wsub, keepmarks=TRUE)
  Pm <- intersect.tess(A, as.mask(Wsub), keepmarks=TRUE)
  H <- dirichlet(runifpoint(4, W))
  AxH <- intersect.tess(A, H, keepmarks=TRUE) # A is marked, H is not
  HxA <- intersect.tess(H, A, keepmarks=TRUE) # A is marked, H is not
  
  b <- bdist.tiles(D)
  b <- bdist.tiles(A[c(3,5,7)])
  #'
  Eim <- as.im(E, W=letterR)
  #'
  #' chop.tess
  #'    horiz/vert lines
  W <- square(1)
  H <- infline(h=(2:4)/5)
  V <- infline(v=(3:4)/5)
  WH <- chop.tess(W, H)
  WV <- chop.tess(W, V)
  #'     polygonal tessellation
  D <- dirichlet(runifpoint(4))
  DH <- chop.tess(D, H)
  DV <- chop.tess(D, V)
  #'     image-based tessellation
  f <- function(x,y){factor(round(4* (x^2 + y^2)))}
  A <- tess(image=as.im(f, W=W))
  L <- infline(p=(1:3)/3, theta=pi/4)
  AL <- chop.tess(A, L)
  AH <- chop.tess(A, H)
  AV <- chop.tess(A, V)
  #'
  #' quantess
  #' quantess.owin
  a <- quantess(square(1), "x", 3)
  a <- quantess(square(1), "y", 3)
  a <- quantess(square(1), "rad", 5, origin=c(1/2, 1/3))
  a <- quantess(square(1), "ang", 7, origin=c(1/2, 1/3))
  ZFUN <- function(x,y){y-x}
  a <- quantess(square(1), ZFUN, 3)
  b <- quantess(letterR, "y", 3)
  #' quantess.ppp
  d <- quantess(cells, "y", 4)
  g <- quantess(demopat, "x", 5)
  g <- quantess(demopat, "y", 5)
  g <- quantess(demopat, "rad", 5, origin=c(4442, 4214))
  g <- quantess(demopat, "ang", 5, origin=c(4442, 4214))
  g <- quantess(demopat, ZFUN, 7)
  #' quantess.im
  D <- distmap(demopat)
  h <- quantess(D, "y", 4)
  h <- quantess(D, ZFUN, 5)
  g <- quantess(D, "rad", 5, origin=c(4442, 4214))
  g <- quantess(D, "ang", 5, origin=c(4442, 4214))
  #'
  X <- shift(chorley, vec = c(1e6, 0))
  tes <- quantess(X, "x", 4)
  if(anyDuplicated(tilenames(tes)))
    stop("quantess produced non-unique tilenames")
  ## 
  ## 
  da <- dirichletAreas(discretise(runifpoint(15, letterR)))
})
}
#
#   tests/testaddvar.R
#
# test addvar options
#
#   $Revision: 1.3 $  $Date: 2020/05/02 01:32:58 $

if(FULLTEST) {
local({
  X <-  rpoispp(function(x,y){exp(3+3*x)})
  model <- ppm(X ~y)
  addvar(model, "x", crosscheck=TRUE)
  addvar(model, "x", bw.input="quad")
  w <- square(0.5)
  addvar(model, "x", subregion=w)
  addvar(model, "x", subregion=w, bw.input="points")
  Z <- as.im(function(x,y) { x }, Window(X))
  addvar(model, Z)
})
}
#
#   tests/testparres.R
#
# additional test of parres
#
#  $Revision: 1.7 $  $Date: 2020/05/02 01:32:58 $
#

if(FULLTEST) {
local({
X <-  rpoispp(function(x,y){exp(3+x+2*x^2)})
model <- ppm(X ~x+y)

# options in parres (and code blocks in print.parres)
parres(model, "x")
parres(model, "x", smooth.effect=TRUE)
parres(model, "x", bw.input="quad")
w <- square(0.5)
parres(model, "x", subregion=w)
parres(model, "x", subregion=w, bw.input="quad")
f <- function(x,y) { x + y }
parres(model, f)

# check whether 'update.ppm' has messed up internals
mod2 <- update(model, ~x)
parres(mod2, "x")

#' other kinds of covariates
mod3 <- ppm(X ~ x + offset(y))
parres(mod3, "offset(y)")
Z <- distmap(runifpoint(3))
parres(mod3, Z)
mod4 <- ppm(X ~ sin(x), data=solist(B=Z))
parres(mod4, "sin(x)")
parres(mod4, "B")

#' models with interaction
mod5 <- ppm(cells ~ x, AreaInter(0.06))
parres(mod5, "x")
dlin <- distfun(copper$SouthLines)
copfit <- ppm(copper$SouthPoints ~ dlin, Geyer(1,1))
parres(copfit, "dlin")

#' covariate need not be specified if there is only one.
parres(mod5)
parres(copfit)

#' infrastructure
ltuae <- evalCovariate(42, cells)
LTUAE <- evalCovariate(ltuae, cells)

fit <- ppm(amacrine ~ x * marks, nd=16)
dmat <- model.depends(fit)
check.separable(dmat, "x", c(x=FALSE, marks=FALSE), FALSE)
check.separable(dmat, "x", c(FALSE, FALSE), FALSE)
check.separable(dmat, "x", c(x=FALSE, marks=TRUE), FALSE)
})
}
#'
#'     tests/threedee.R
#'
#'     Tests of 3D code 
#'
#'      $Revision: 1.8 $ $Date: 2020/05/02 01:32:58 $
#'

local({
  X <- runifpoint3(30)
  Y <- runifpoint3(20)
  if(FULLTEST) {
    A <- runifpoint3(10, nsim=2)
    Z <- ppsubset(X, 2:4)
  }
  ##
  if(ALWAYS) { # includes C code
    d <- pairdist(X, periodic=TRUE, squared=TRUE)
    d <- crossdist(X, Y, squared=TRUE)
    d <- crossdist(X, Y, squared=TRUE, periodic=TRUE)
    #' 
    h <- has.close(X, 0.2)
    h <- has.close(X, 0.2, periodic=TRUE)
    h <- has.close(X, 0.2, Y=Y)
    h <- has.close(X, 0.2, Y=Y, periodic=TRUE)
    #' code blocks not otherwise reached
    rmax <- 0.6 * max(nndist(X))
    g <- G3est(X, rmax=rmax, correction="rs")
    g <- G3est(X, rmax=rmax, correction="km")
    g <- G3est(X, rmax=rmax, correction="Hanisch")
    g <- G3est(X, rmax=rmax, sphere="ideal")
    g <- G3est(X, rmax=rmax, sphere="digital")
    v <- sphere.volume()
    v <- digital.volume()
    #' older code
    co <- coords(X)
    xx <- co$x
    yy <- co$y
    zz <- co$z
    gg1 <- g3engine(xx, yy, zz, correction="Hanisch G3")
    gg2 <- g3engine(xx, yy, zz, correction="minus sampling")
    ff1 <- f3engine(xx, yy, zz, correction="no")
    ff2 <- f3engine(xx, yy, zz, correction="minus sampling")
  }
  ##
  if(ALWAYS) {
    #'class support
    X <- runifpoint3(10)
    print(X)
    print(X %mark% runif(10))
    print(X %mark% factor(letters[c(1:5,5:1)]))
    print(X %mark% data.frame(a=1:10, b=runif(10)))
    da <- as.Date(paste0("2020-01-0", c(1:5,5:1)))
    print(X %mark% da)
    print(X %mark% data.frame(a=1:10, b=da))
  }
})
#'    tests/trigraph.R
#'
#'   Tests for C code in trigraf.c
#'   
#'  $Revision: 1.4 $  $Date: 2020/05/02 01:32:58 $
#'
if(ALWAYS) { # depends on C code 
local({
  #' called from deldir.R
  spatstat.deldir.setopt(FALSE, TRUE)
  A <- delaunay(redwood)
  spatstat.deldir.setopt(FALSE, FALSE)
  B <- delaunay(redwood)
  spatstat.deldir.setopt(TRUE, TRUE)
  #' called from edges2triangles.R
  tryangles <- function(iedge, jedge, nt=0) {
    spatstat.options(fast.trigraph=FALSE)
    A <- edges2triangles(iedge, jedge)
    spatstat.options(fast.trigraph=TRUE)
    B <- edges2triangles(iedge, jedge)
    if(!all(dim(A) == dim(B)) || !all(A == B))
      stop(paste("Discrepancy in edges2triangles (with", nt, "triangles)"))
  }
  ii <- simplenet$from
  jj <- simplenet$to
  tryangles(ii,          jj,          0)
  tryangles(c(ii, 1),    c(jj, 5),    1)
  tryangles(c(ii, 1, 8), c(jj, 5, 9), 2)
})
}
reset.spatstat.options()


#
# tests/triplets.R
#
# test code for triplet interaction and associated summary function Tstat 
#
# $Revision: 1.8 $ $Date: 2020/05/02 01:32:58 $
#
if(ALWAYS) { # C code, platform dependence
local({
  #' valid model
  fit <- ppm(cells ~1, Triplets(0.1))
  fit
  suffstat(fit)
  #' invalid model 
  fitR <- ppm(redwood ~1, Triplets(0.1))
  fitR
  suffstat(fitR)
  #' hard core (zero triangles, coefficient is NA)
  fit0 <- ppm(cells ~1, Triplets(0.05))
  fit0
  suffstat(fit0)
  #' bug case (1 triangle in data)
  fit1 <- ppm(cells ~1, Triplets(0.15))
  fit1
  suffstat(fit1)
  #' Tstat function, all code blocks
  a <- Tstat(redwood, ratio=TRUE,
             correction=c("none", "border", "bord.modif", "translate"))
  #' simulation
  X <- simulate(fit)
  mod <- list(cif="triplets",par=list(beta=50,gamma=0.2,r=0.07), w=square(1))
  Xm <- rmh(model=mod,start=list(n.start=5), control=list(nrep=1e5))
  #' hard core
  mod$par$gamma <- 0
  XmHard <- rmh(model=mod,start=list(n.start=5), control=list(nrep=1e5))
})
}

#'
#'    varcount.R
#'
#'   Variance of N(B)
#'
#'  $Revision: 1.8 $  $Date: 2015/11/21 07:02:51 $
#'

varcount <- function(model, B, ..., dimyx=NULL) {
  stopifnot(is.owin(B) || is.im(B) || is.function(B))
  g <- pcfmodel(model)
  if(!is.function(g))
    stop("Pair correlation function cannot be computed")
  if(is.owin(B)) {
    lambdaB <- predict(model, locations=B, ngrid=dimyx, type="intensity")
    v <- varcountEngine(g, B, lambdaB)
  } else {
    f <- if(is.im(B)) B else as.im(B, W=as.owin(model), ..., dimyx=dimyx)
    B <- as.owin(f)
    lambdaB <- predict(model, locations=B, type="intensity")
    v <- varcountEngine(g, B, lambdaB, f)
  } 
  return(v)
}

varcountEngine <- local({

  varcountEngine <- function(g, B, lambdaB, f=1) {
    if(missing(f) || identical(f, 1)) {
      v <- integral(lambdaB) + covterm(g, B, lambdaB)
    } else if(min(f) >= 0) {
      ## nonnegative integrand
      v <- integral(lambdaB * f^2) + covterm(g, B, lambdaB * f)
    } else if(max(f) <= 0) {
      ## nonpositive integrand
      v <- integral(lambdaB * f^2) + covterm(g, B, lambdaB * (-f))
    } else {
      ## integrand has both positive and negative parts
      lamfplus <- eval.im(lambdaB * pmax(0, f))
      lamfminus <- eval.im(lambdaB * pmax(0, -f))
      v <- integral(lambdaB * f^2) +
        (covterm(g, B, lamfplus) + covterm(g, B, lamfminus)
         - covterm(g, B, lamfplus, lamfminus)
         - covterm(g, B, lamfminus, lamfplus))
    }
    return(v)
  }

  covterm <- function(g, B, f, f2) {
    if(missing(f2)) {
      # \int_B \int_B (g(u-v) - 1) f(u) f(v) du dv
      H <- distcdf(B, dW=f)
      a <- integral(f)^2 * (as.numeric(stieltjes(g, H)) - 1)
    } else {
      # \int_B \int_B (g(u-v) - 1) f(u) f2(v) du dv
      H <- distcdf(B, dW=f, dV=f2)
      a <- integral(f) * integral(f2) * (as.numeric(stieltjes(g, H)) - 1)
    }
    return(a)
  }
  
  varcountEngine
})

#
#  slrm.R
#
#  Spatial Logistic Regression
#
#  $Revision: 1.29 $   $Date: 2018/05/12 16:19:22 $
#

slrm <- function(formula, ..., data=NULL, offset=TRUE, link="logit",
                 dataAtPoints=NULL, splitby=NULL) {
  
  # remember call
  CallInfo <- list(callstring = short.deparse(sys.call()),
                   cl = match.call(),
                   formula = formula,
                   offset=offset,
                   link=link,
                   splitby=splitby,
                   dotargs=list(...))
  if(!(link %in% c("logit", "cloglog")))
    stop(paste("Unrecognised link", dQuote(link)))

  ########### INTERPRET FORMULA ##############################
  
  if(!inherits(formula, "formula"))
    stop(paste("Argument", dQuote("formula"), "should be a formula"))

  # check formula has LHS and RHS. Extract them
  if(length(formula) < 3)
    stop(paste("Argument", sQuote("formula"),
               "must have a left hand side"))
  Yname <- formula[[2]]
  trend <- rhs <- formula[c(1,3)]
  if(!is.name(Yname))
    stop("Left hand side of formula should be a single name")
  Yname <- paste(Yname)
  if(!inherits(trend, "formula"))
    stop("Internal error: failed to extract RHS of formula")

  varnames <- unique(variablesinformula(trend))
  specials <- c("x", "y", "logpixelarea")
  covnames <- varnames[!(varnames %in% specials)]

  # add 'splitby' to covariate names
  if(!is.null(splitby)) {
    if(!is.character(splitby) || length(splitby) != 1)
      stop("splitby should be a single character string")
    covnames <- unique(c(covnames, splitby))
  }

  CallInfo$responsename <- Yname
  CallInfo$varnames     <- varnames
  CallInfo$covnames     <- covnames
  
  # Parent environment
  parenv <- environment(formula)

  ########  FIND DATA AND RESHAPE #######################

  Data <- slr.prepare(CallInfo, parenv, data, dataAtPoints, splitby)

#  W  <- Data$W
  df <- Data$df
  
  ########  FIT MODEL ###############################

  dformula <- formula
  if(offset) {
    # insert offset term in formula
    rhs <- paste(as.character(rhs), collapse=" ")
    rhs <- paste(c(rhs, "offset(logpixelarea)"), collapse="+")
    dformula <- as.formula(paste(Yname, rhs))
  }

  linkname <- link
  FIT  <- glm(dformula, family=binomial(link=linkname),
              data=df, na.action=na.exclude)

  result <- list(call     = CallInfo$cl,
                 CallInfo = CallInfo,
                 Data     = Data,
                 Fit      = list(FIT=FIT, dformula=dformula),
                 terms    = terms(formula))

  class(result) <- c("slrm", class(result))
  return(result)
}

################ UTILITY TO FIND AND RESHAPE DATA #################

slr.prepare <- function(CallInfo, envir, data,
                        dataAtPoints=NULL, splitby=NULL,
                        clip=TRUE) {
  # CallInfo is produced by slrm()
  # envir is parent environment of model formula
  # data  is 'data' argument that takes precedence over 'envir'
  # 'clip' is TRUE if the data should be clipped to the domain of Y
  Yname    <- CallInfo$responsename
#  varnames <- CallInfo$varnames
  covnames <- CallInfo$covnames
  dotargs  <- CallInfo$dotargs
  #
  getobj <- function(nama, env, dat) {
    if(!is.null(dat) && !is.null(x <- dat[[nama]]))
      return(x)
    else return(get(nama, envir=env))
  }
  # Get the response point pattern Y 
  Y <- getobj(Yname, envir, data)
  if(!is.ppp(Y))
    stop(paste("The response", sQuote(Yname), "must be a point pattern"))
  #
  if(!is.null(dataAtPoints)) {
    dataAtPoints <- as.data.frame(dataAtPoints)
    if(nrow(dataAtPoints) != npoints(Y))
      stop(paste("dataAtPoints should have one row for each point in",
                 dQuote(Yname)))
  }
  # Find the covariates
  ncov <- length(covnames)
  covlist <- lapply(as.list(covnames), getobj, env = envir, dat=data)
  names(covlist) <- covnames
  # Each covariate should be an image, a window, a function, or a single number
  if(ncov == 0) {
    isim <- isowin <- ismask <- isfun <- isnum <- isspatial <- israster <- logical(0)
  } else {
    isim  <- sapply(covlist, is.im)
    isowin  <- sapply(covlist, is.owin)
    ismask  <- sapply(covlist, is.mask)
    isfun  <- sapply(covlist, is.function)
    isspatial <- isim | isowin | isfun
    israster <- isim | ismask
    isnum <- sapply(covlist, is.numeric) & (lengths(covlist) == 1)
  }
  if(!all(ok <- (isspatial | isnum))) {
    n <- sum(!ok)
    stop(paste(ngettext(n, "The argument", "Each of the arguments"),
               commasep(sQuote(covnames[!ok])),
               "should be either an image, a window, or a single number"))
  }
  # 'splitby' 
  if(!is.null(splitby)) {
    splitwin <- covlist[[splitby]]
    if(!is.owin(splitwin))
      stop("The splitting covariate must be a window")
    # ensure it is a polygonal window
    covlist[[splitby]] <- splitwin <- as.polygonal(splitwin)
    # delete splitting covariate from lists to be processed
    issplit <- (covnames == splitby)
    isspatial[issplit] <- FALSE
    israster[issplit] <- FALSE
  }
  # 
#  nnum <- sum(isnum)
#  nspatial <- sum(isspatial)
  nraster <- sum(israster)
  #
  numlist <- covlist[isnum]
  spatiallist <- covlist[isspatial]
  rasterlist <- covlist[israster]
  #
  numnames <- names(numlist)
  spatialnames <- names(spatiallist)
#  rasternames <- names(rasterlist)
  #
  
  ########  CONVERT TO RASTER DATA  ###############################

  convert <- function(x,W) {
    if(is.im(x) || is.function(x)) return(as.im(x,W))
    if(is.owin(x)) return(as.im(x, W, value=TRUE, na.replace=FALSE))
    return(NULL)
  }

  # determine spatial domain & common resolution: convert all data to it
  if(length(dotargs) > 0 || nraster == 0) {
    # Pixel resolution is determined by explicit arguments
    if(clip) {
      # Window extent is determined by response point pattern
      D <- as.owin(Y)
    } else {
      # Window extent is union of domains of data
      domains <- lapply(append(spatiallist, list(Y)), as.owin)
      D <- do.call(union.owin, domains)
    }
    # Create template mask
    W <- do.call(as.mask, append(list(D), dotargs))
    # Convert all spatial objects to this resolution
    spatiallist <- lapply(spatiallist, convert, W=W)
  } else {
    # Pixel resolution is determined implicitly by covariate data
    W <- do.call(commonGrid, rasterlist)
    if(clip) {
      # Restrict data to spatial extent of response point pattern
      W <- intersect.owin(W, as.owin(Y))
    }
    # Adjust spatial objects to this resolution
    spatiallist <- lapply(spatiallist, convert, W=W)
  }
  # images containing coordinate values
  xcoordim <- as.im(function(x,y){x}, W=W)
  ycoordim <- as.im(function(x,y){y}, W=W)
  #
  # create a list of covariate images, with names as in formula
  covimages <- append(list(x=xcoordim, y=ycoordim), spatiallist)

  basepixelarea <- W$xstep * W$ystep

  ########  ASSEMBLE DATA FRAME  ###############################

  if(is.null(splitby)) {
    df <- slrAssemblePixelData(Y, Yname, W,
                               covimages, dataAtPoints, basepixelarea)
    sumYloga <- Y$n * log(basepixelarea)
    serial <- attr(df, "serial")
  } else {
    # fractional pixel areas
    pixsplit <- pixellate(splitwin, W)
    splitpixelarea <- as.vector(as.matrix(pixsplit))
    # determine which points of Y are inside/outside window
    ins <- inside.owin(Y$x, Y$y, splitwin)
    # split processing
    dfIN <- slrAssemblePixelData(Y[ins], Yname, W, covimages,
                                 dataAtPoints[ins, ], splitpixelarea)
    serialIN <- attr(dfIN, "serial")
    dfIN[[splitby]] <- TRUE
    dfOUT <- slrAssemblePixelData(Y[!ins], Yname, W, covimages,
                                  dataAtPoints[!ins, ],
                                  basepixelarea - splitpixelarea)
    serialOUT <- attr(dfOUT, "serial")
    dfOUT[[splitby]] <- FALSE
    df <- rbind(dfIN, dfOUT)
    serial <- c(serialIN, serialOUT)
    # sum of log pixel areas associated with points
    Ysplit <- pixsplit[Y]
    sumYloga <- sum(log(ifelseXY(ins, Ysplit, basepixelarea - Ysplit)))
  }
  
  # tack on any numeric values
  df <- do.call(cbind, append(list(df), numlist))
  
  ### RETURN ALL 
  Data <- list(response=Y,
               covariates=covlist,
               spatialnames=spatialnames,
               numnames=numnames,
               W=W,
               df=df,
               serial=serial,
               sumYloga=sumYloga,
               dataAtPoints=dataAtPoints)
  return(Data)
}

#  
slrAssemblePixelData <- function(Y, Yname, W,
                                 covimages, dataAtPoints, pixelarea) {
  # pixellate point pattern
  Z <- pixellate(Y, W=W)
  Z <- eval.im(as.integer(Z>0))
  # overwrite pixel entries for data points using exact values
  # coordinates
  xcoordim <- covimages[["x"]]
  ycoordim <- covimages[["y"]]
  xcoordim[Y] <- Y$x
  ycoordim[Y] <- Y$y
  covimages[["x"]] <- xcoordim
  covimages[["y"]] <- ycoordim
  # overwrite pixel entries
  if(!is.null(dataAtPoints)) {
    enames <- colnames(dataAtPoints)
    relevant <- enames %in% names(covimages)
    for(v in enames[relevant]) {
      cova <- covimages[[v]]
      cova[Y] <- dataAtPoints[, v, drop=TRUE]
      covimages[[v]] <- cova
    }
  }
  # assemble list of all images
  Ylist <- list(Z)
  names(Ylist) <- Yname
  allimages <- append(Ylist, covimages)
  # extract pixel values of each image
  pixelvalues <-
    function(z) {
      v <- as.vector(as.matrix(z))
      if(z$type != "factor") return(v)
      lev <- levels(z)
      return(factor(v, levels=seq_along(lev), labels=lev))
    }
  pixdata <- lapply(allimages, pixelvalues)
  df <- as.data.frame(pixdata)
  serial <- seq_len(nrow(df))
  # add log(pixel area) column
  if(length(pixelarea) == 1) {
    df <- cbind(df, logpixelarea=log(pixelarea))
  } else {
    ok <- (pixelarea > 0)
    df <- cbind(df[ok, ], logpixelarea=log(pixelarea[ok]))
    serial <- serial[ok]
  }
  attr(df, "serial") <- serial
  return(df)
}

is.slrm <- function(x) {
  inherits(x, "slrm")
}

coef.slrm <- function(object, ...) {
  coef(object$Fit$FIT)
}

print.slrm <- function(x, ...) {
  lk <- x$CallInfo$link
  switch(lk,
         logit= {
           splat("Fitted spatial logistic regression model")
         },
         cloglog= {
           splat("Fitted spatial regression model (complementary log-log)")
         },
         {
           splat("Fitted spatial regression model")
           splat("Link =", dQuote(lk))
         })
  cat("Formula:\t")
  print(x$CallInfo$formula)
  splat("Fitted coefficients:")
  print(coef(x))
  return(invisible(NULL))
}

logLik.slrm <- function(object, ..., adjust=TRUE) {
  FIT  <- object$Fit$FIT
  ll <- -deviance(FIT)/2
  if(adjust) {
    sumYloga <- object$Data$sumYloga
    ll <- ll - sumYloga
  }
  attr(ll, "df") <- length(coef(object))
  class(ll) <- "logLik"
  return(ll)
}

fitted.slrm <- function(object, ...) {
  if(length(list(...)) > 0)
    warning("second argument (and any subsequent arguments) ignored")
  predict(object, type="probabilities")
}

predict.slrm <- function(object, ..., type="intensity",
                         newdata=NULL, window=NULL) {
  type <- pickoption("type", type,
                     c(probabilities="probabilities",
                       link="link",
                       intensity="intensity",
                       lambda="intensity"))
  
  FIT  <- object$Fit$FIT
  link <- object$CallInfo$link
  W    <- object$Data$W
  df   <- object$Data$df
  loga <- df$logpixelarea

  if(is.null(newdata) && is.null(window)) {
    # fitted values from existing fit
    switch(type,
           probabilities={
             values <- fitted(FIT)
           },
           link={
             values <- predict(FIT, type="link")
           },
           intensity={
             # this calculation applies whether an offset was included or not
             if(link == "cloglog") {
               linkvalues <- predict(FIT, type="link")
               values <- exp(linkvalues - loga)
             } else {
               probs <- fitted(FIT)
               values <- -log(1-probs)/exp(loga)
             }
           }
           )
    out <- im(values, xcol=W$xcol, yrow=W$yrow, unitname=unitname(W))
    return(out)
  } else {
    # prediction using new values
    # update arguments that may affect pixel resolution
    CallInfo <- object$CallInfo
    CallInfo$dotargs <- resolve.defaults(list(...), CallInfo$dotargs)
    #
    if(!is.null(window)) {
      # insert fake response in new window
      if(is.null(newdata)) newdata <- list()
      window <- as.owin(window)
      newdata[[CallInfo$responsename]] <- ppp(numeric(0), numeric(0),
                                            window=window)
    }
    # process new data
    newData <- slr.prepare(CallInfo, environment(CallInfo$formula), newdata,
                           clip=!is.null(window))
    newdf   <- newData$df
    newW    <- newData$W
    newloga <- newdf$logpixelarea
    # avoid NA etc
    npixel <- nrow(newdf)
    ok <- complete.cases(newdf)
    if(!all(ok)) {
      newdf   <- newdf[ok, , drop=FALSE]
      newloga <- newloga[ok]
    }
    # compute link values
    linkvalues <- predict(FIT, newdata=newdf, type="link")
    # transform to desired scale
    linkinv <- family(FIT)$linkinv
    switch(type,
           probabilities={
             values <- linkinv(linkvalues)
           },
           link={
             values <- linkvalues
           },
           intensity={
             # this calculation applies whether an offset was included or not
             if(link == "cloglog") {
               values <- exp(linkvalues - newloga)
             } else {
               probs <- linkinv(linkvalues)
               values <- -log(1-probs)/exp(newloga)
             }
           }
           )
    # form image
    v <- rep.int(NA_real_, npixel)
    v[ok] <- values
    out <- im(v, xcol=newW$xcol, yrow=newW$yrow, unitname=unitname(W))
    return(out)
  }
}

plot.slrm <- function(x, ..., type="intensity") {
  xname <- short.deparse(substitute(x))
  y <- predict(x, type=type)
  do.call(plot.im, resolve.defaults(list(x=y), list(...), list(main=xname)))
}

formula.slrm <- function(x, ...) {
  f <- x$CallInfo$formula
  return(f)
}

terms.slrm <- function(x, ...) {
  terms(formula(x), ...)
}

labels.slrm <- function(object, ...) {
  # extract fitted trend coefficients
  co <- coef(object)
  # model terms
  tt <- terms(object)
  lab <- attr(tt, "term.labels")
  if(length(lab) == 0)
    return(character(0))
  # model matrix
  mm <- model.matrix(object)
  ass <- attr(mm, "assign")
  # 'ass' associates coefficients with model terms
  # except ass == 0 for the Intercept
  coef.ok <- is.finite(co)
  relevant <- (ass > 0) 
  okterms <- unique(ass[coef.ok & relevant])
  return(lab[okterms])
}

extractAIC.slrm <- function (fit, scale = 0, k = 2, ...)
{
    edf <- length(coef(fit))
    aic <- AIC(fit)
    c(edf, aic + (k - 2) * edf)
}

model.matrix.slrm <- function(object,..., keepNA=TRUE) {
  FIT <- object$Fit$FIT
  mm <- model.matrix(FIT, ...)
  if(!keepNA)
    return(mm)
  df <- object$Data$df
  comp <- complete.cases(df)
  if(all(comp))
    return(mm)
  if(sum(comp) != nrow(mm))
      stop("Internal error in patching NA's")
  mmplus <- matrix(NA, nrow(df), ncol(mm))
  mmplus[comp, ] <- mm
  return(mmplus)
}

model.images.slrm <- function(object, ...) {
  mm <- model.matrix(object, ...)
  mm <- as.data.frame(mm)
  Data <- object$Data
  W      <- Data$W
  serial <- Data$serial
  splitby <- object$CallInfo$splitby
  blank   <- as.im(NA_real_, W)
  assignbyserial <- function(values, serial, template) {
    Z <- template
    Z$v[serial] <- values
    return(Z)
  }
  if(is.null(splitby)) {
    result <- lapply(as.list(mm), assignbyserial, serial=serial, template=blank)
  } else {
    df <- Data$df
    IN <- as.logical(df[[splitby]])
    OUT <- !IN
    mmIN <- mm[IN, , drop=FALSE]
    mmOUT <- mm[OUT, , drop=FALSE]
    resultIN <- lapply(as.list(mmIN), assignbyserial,
                       serial=serial[IN], template=blank)
    resultOUT <- lapply(as.list(mmOUT), assignbyserial,
                       serial=serial[OUT], template=blank)
    names(resultIN) <- paste(names(resultIN), splitby, "TRUE", sep="")
    names(resultOUT) <- paste(names(resultOUT), splitby, "FALSE", sep="")
    result <- c(resultIN, resultOUT)
  }
  return(as.solist(result))
}

update.slrm <- function(object, ..., evaluate=TRUE, env=parent.frame()) {
  e <- update.default(object, ..., evaluate=FALSE)
  if(evaluate)
    e <- eval(e, envir=env)
  return(e)
}

anova.slrm <- local({

  anova.slrm <- function(object, ..., test=NULL) {
    objex <- append(list(object), list(...))
    if(!all(unlist(lapply(objex, is.slrm))))
      stop("Some arguments are not of class slrm")
    fitz <- lapply(objex, getFIT)
    do.call(anova, append(fitz, list(test=test)))
  }

  getFIT <- function(z) {z$Fit$FIT}

  anova.slrm
})


vcov.slrm <- function(object, ..., what=c("vcov", "corr", "fisher", "Fisher")) {
  stopifnot(is.slrm(object))
  what <- match.arg(what)
  vc <- vcov(object$Fit$FIT)
  result <- switch(what,
                   vcov = vc,
                   corr = {
                     sd <- sqrt(diag(vc))
                     vc / outer(sd, sd, "*")
                   },
                   fisher=,
                   Fisher={
                     solve(vc)
                   })
  return(result)
}

unitname.slrm <- function(x) {
  return(unitname(x$Data$response))
}

"unitname<-.slrm" <- function(x, value) {
  unitname(x$Data$response) <- value
  return(x)
}

is.stationary.slrm <- function(x) {
  fo <- formula(x)
  trend <- fo[c(1,3)]
  return(identical.formulae(trend, ~1))
}

is.poisson.slrm <- function(x) { TRUE }


simulate.slrm <- function(object, nsim=1, seed=NULL, ...,
                          window=NULL, covariates=NULL, 
                          verbose=TRUE, drop=FALSE) {
  # .... copied from simulate.lm ....
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
    runif(1)
  if (is.null(seed))
    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
  else {
    R.seed <- get(".Random.seed", envir = .GlobalEnv)
    set.seed(seed)
    RNGstate <- structure(seed, kind = as.list(RNGkind()))
    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
  }
  starttime <- proc.time()
  
  # determine simulation window and compute intensity
  if(!is.null(window))
    stopifnot(is.owin(window))
  lambda <- predict(object, type="intensity", newdata=covariates, window=window)

  # max lambda (for efficiency)
  summ <- summary(lambda)
  lmax <- summ$max + 0.05 * diff(summ$range)

  # run
  out <- list()
  verbose <- verbose && (nsim > 1)
  if(verbose) {
    cat(paste("Generating", nsim, "simulations... "))
    pstate <- list()
  }
  for(i in 1:nsim) {
    out[[i]] <- rpoispp(lambda, lmax=lmax)
    if(verbose) pstate <- progressreport(i, nsim, state=pstate)
  }
  #' pack up
  out <- simulationresult(out, nsim, drop)
  out <- timed(out, starttime=starttime)
  attr(out, "seed") <- RNGstate
  return(out)
}

#
#       images.R
#
#      $Revision: 1.157 $     $Date: 2019/08/31 05:19:39 $
#
#      The class "im" of raster images
#
#     im()     object creator
#
#     is.im()   tests class membership
#
#     rasterx.im(), rastery.im()    
#                      raster X and Y coordinates
#
#     nearest.pixel()   
#     lookup.im()
#                      facilities for looking up pixel values
#
################################################################
########   basic support for class "im"
################################################################
#
#   creator 

im <- function(mat, xcol=seq_len(ncol(mat)), yrow=seq_len(nrow(mat)), 
               xrange=NULL, yrange=NULL,
               unitname=NULL) {

  typ <- typeof(mat)
  if(typ == "double")
    typ <- "real"

  miss.xcol <- missing(xcol)
  miss.yrow <- missing(yrow)
  
  # determine dimensions
  if(!is.null(dim(mat))) {
    nr <- nrow(mat)
    nc <- ncol(mat)
    if(is.na(nc)) {
      #' handle one-dimensional tables
      nc <- 1
      nr <- length(mat)
      if(missing(xcol)) xcol <- seq_len(nc)
    }
    if(length(xcol) != nc)
      stop("Length of xcol does not match ncol(mat)")
    if(length(yrow) != nr)
      stop("Length of yrow does not match nrow(mat)")
  } else {
    if(miss.xcol || miss.yrow)
      stop(paste(sQuote("mat"),
                 "is not a matrix and I can't guess its dimensions"))
    stopifnot(length(mat) == length(xcol) * length(yrow))
    nc <- length(xcol)
    nr <- length(yrow)
  }

  # deal with factor case
  if(is.factor(mat)) {
    typ <- "factor"
  } else if(!is.null(lev <- levels(mat))) {
    typ <- "factor"
    mat <- factor(mat, levels=lev)
  }

  # Ensure 'mat' is a matrix (without destroying factor information)
  if(!is.matrix(mat))
    dim(mat) <- c(nr, nc)

  # set up coordinates
  if((miss.xcol || length(xcol) <= 1) && !is.null(xrange) ) {
    # use 'xrange' 
    xstep <- diff(xrange)/nc
    xcol <- seq(from=xrange[1L] + xstep/2, to=xrange[2L] - xstep/2, length.out=nc)
  } else if(length(xcol) > 1) {
    # use 'xcol'
    # ensure spacing is constant
    xcol <- seq(from=min(xcol), to=max(xcol), length.out=length(xcol))
    xstep <- diff(xcol)[1L]
    xrange <- range(xcol) + c(-1,1) * xstep/2
  } else stop("Cannot determine pixel width")
  
  if((miss.yrow || length(yrow) <= 1) && !is.null(yrange)) {
    # use 'yrange'
    ystep <- diff(yrange)/nr
    yrow <- seq(from=yrange[1L] + ystep/2, to=yrange[2L] - ystep/2, length.out=nr)
  } else if(length(yrow) > 1) {
    # use 'yrow'
    # ensure spacing is constant
    yrow <- seq(from=min(yrow), to=max(yrow), length.out=length(yrow))
    ystep <- diff(yrow)[1L]
    yrange <- range(yrow) + c(-1,1) * ystep/2
  }  else stop("Cannot determine pixel height")

  unitname <- as.unitname(unitname)

  out <- list(v   = mat,
              dim = c(nr, nc),
              xrange   = xrange,
              yrange   = yrange,
              xstep    = xstep,
              ystep    = ystep,
              xcol    = xcol,
              yrow    = yrow,
              type    = typ,
              units   = unitname)
  class(out) <- "im"
  return(out)
}

is.im <- function(x) {
  inherits(x,"im")
}

levels.im <- function(x) {
  levels(x$v)
}

"levels<-.im" <- function(x, value) {
  if(x$type != "factor") 
    stop("image is not factor-valued")
  levels(x$v) <- value
  x
}

################################################################
########   methods for class "im"
################################################################

shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
  verifyclass(X, "im")
  if(!is.null(origin)) {
    if(!missing(vec))
      warning("argument vec ignored; argument origin has precedence")
    locn <- interpretAsOrigin(origin, Window(X))
    vec <- -locn
  }
  vec <- as2vector(vec)
  X$xrange <- X$xrange + vec[1L]
  X$yrange <- X$yrange + vec[2L]
  X$xcol <- X$xcol + vec[1L]
  X$yrow <- X$yrow + vec[2L]
  attr(X, "lastshift") <- vec
  return(X)
}

"Frame<-.im" <- function(X, value) {
  stopifnot(is.rectangle(value))
  if(!is.subset.owin(value, Frame(X))) {
    ## first expand
    X <- X[value, drop=FALSE]
  }
  X[value, drop=TRUE]
}


"[.im" <- local({

  disjoint <- function(r, s) { (r[2L] < s[1L]) || (r[1L] > s[2L])  }
  clip <- function(r, s) { c(max(r[1L],s[1L]), min(r[2L],s[2L])) }
  inrange <- function(x, r) { (x >= r[1L]) & (x <= r[2L]) }

  Extract.im <- function(x, i, j, ...,
                         drop=TRUE, tight=FALSE, raster=NULL,
                         rescue=is.owin(i)) {

    ## detect 'blank' arguments like second argument in x[i, ] 
    ngiven <- length(sys.call())
    nmatched <- length(match.call())
    nblank <- ngiven - nmatched
    itype <- if(missing(i)) "missing" else "given"
    jtype <- if(missing(j)) "missing" else "given"
    if(nblank == 1) {
      if(!missing(i)) jtype <- "blank"
      if(!missing(j)) itype <- "blank"
    } else if(nblank == 2) {
      itype <- jtype <- "blank"
    }

    if(missing(rescue) && itype != "given")
      rescue <- FALSE
    
    if(itype == "missing" && jtype == "missing") {
      ## no indices: return entire image 
      out <- if(is.null(raster)) x else as.im(raster)
      xy <- expand.grid(y=out$yrow,x=out$xcol)
      if(!is.null(raster)) {
        ## resample image on new pixel raster
        values <- lookup.im(x, xy$x, xy$y, naok=TRUE)
        out <- im(values, out$xcol, out$yrow, unitname=unitname(out))
      }
      if(!drop)
        return(out)
      else {
        v <- out$v
        return(v[!is.na(v)])
      }
    }

    if(itype == "given") {
      ## .................................................................
      ## Try spatial index
      ## .................................................................
      if(verifyclass(i, "owin", fatal=FALSE)) {

        if(jtype == "given")
          warning("Argument j ignored")
      
        ## 'i' is a window
        ## if drop = FALSE, just set values outside window to NA
        ## if drop = TRUE, extract values for all pixels inside window
        ##                 as an image (if 'i' is a rectangle)
        ##                 or as a vector (otherwise)

        ## determine pixel raster for output
        if(!is.null(raster)) {
          out <- as.im(raster)
          do.resample <- TRUE
        } else if(is.subset.owin(i, as.owin(x))) {
          out <- x
          do.resample <- FALSE
        } else {
          ## new window does not contain data window: expand it
          bb <- boundingbox(as.rectangle(i), as.rectangle(x))
          rr <- if(is.mask(i)) i else x
          xcol <- prolongseq(rr$xcol, bb$xrange, rr$xstep)
          yrow <- prolongseq(rr$yrow, bb$yrange, rr$ystep)
          out <- list(xcol=xcol, yrow=yrow)
          do.resample <- TRUE
        }
        xy <- expand.grid(y=out$yrow,x=out$xcol)
        if(do.resample) {
          ## resample image on new pixel raster
          values <- lookup.im(x, xy$x, xy$y, naok=TRUE)
          out <- im(values, out$xcol, out$yrow, unitname=unitname(out))
        }
        inside <- inside.owin(xy$x, xy$y, i)
        if(!drop) {
          ## set other pixels to NA and return image
          out$v[!inside] <- NA
          if(!tight)
            return(out)
        } else if(!(rescue && i$type == "rectangle")) {
          ## return pixel values
          values <- out$v[inside]
          return(values)
        }
        ## return image in smaller rectangle
        if(disjoint(i$xrange, x$xrange) || disjoint(i$yrange, x$yrange))
          ## empty intersection
          return(numeric(0))
        xr <- clip(i$xrange, x$xrange)
        yr <- clip(i$yrange, x$yrange)
        colsub <- inrange(out$xcol, xr)
        rowsub <- inrange(out$yrow, yr)
        ncolsub <- sum(colsub)
        nrowsub <- sum(rowsub)
        if(ncolsub == 0 || nrowsub == 0)
          return(numeric(0))
        marg <- list(mat=out$v[rowsub, colsub, drop=FALSE],
                     unitname=unitname(x))
        xarg <-
          if(ncolsub > 1) list(xcol = out$xcol[colsub]) else list(xrange=xr)
        yarg <-
          if(nrowsub > 1) list(yrow = out$yrow[rowsub]) else list(yrange=yr)
        result <- do.call(im, c(marg, xarg, yarg))
        return(result)
      }
      if(verifyclass(i, "im", fatal=FALSE)) {
        if(jtype == "given")
          warning("Argument j ignored")
        ## logical images OK
        if(i$type == "logical") {
          ## convert to window
          w <- as.owin(eval.im(ifelse1NA(i)))
          return(x[w, drop=drop, ..., raster=raster])
        } else stop("Subset argument \'i\' is an image, but not of logical type")
      }

      if(inherits(i, "linnet")) {
        #' linear network
        if(jtype == "given")
          warning("Argument j ignored")
        W <- raster %orifnull% as.owin(x)
        M <- as.mask.psp(as.psp(i), W=W, ...)
        xM <- x[M, drop=drop]
        if(is.im(xM)) xM <- linim(i, xM)
        return(xM)
      }
      
      if(is.ppp(i)) {
        ## 'i' is a point pattern 
        if(jtype == "given")
          warning("Argument j ignored")
        ## Look up the greyscale values for the points of the pattern
        values <- lookup.im(x, i$x, i$y, naok=TRUE)
        if(drop) 
          values <- values[!is.na(values)]
        if(length(values) == 0) 
          ## ensure the zero-length vector is of the right type
          values <- 
            switch(x$type,
                   factor={ factor(, levels=levels(x)) },
                   integer = { integer(0) },
                   logical = { logical(0) },
                   real = { numeric(0) },
                   complex = { complex(0) },
                   character = { character(0) },
                   { values }
                   )
        return(values)
      }
    }
    ## ............... not a spatial index .............................

    ## Try indexing as a matrix

    ## Construct a matrix index call for possible re-use
    M <- as.matrix(x)
    ## suppress warnings from code checkers
    dont.complain.about(M)
    ##
    ycall <- switch(itype,
                    given = {
                      switch(jtype,
                             given   = quote(M[i, j, drop=FALSE]),
                             blank   = quote(M[i,  , drop=FALSE]),
                             missing = quote(M[i,    drop=FALSE]))
                    },
                    blank = {
                      switch(jtype,
                             given   = quote(M[ , j, drop=FALSE]),
                             blank   = quote(M[ ,  , drop=FALSE]),
                             missing = quote(M[ ,    drop=FALSE]))
                    },
                    missing = {
                      switch(jtype,
                             given   = quote(M[j=j,  drop=FALSE]),
                             blank   = quote(M[j= ,  drop=FALSE]),
                             missing = quote(M[      drop=FALSE]))
                    })
    ## try it
    y <- try(eval(as.call(ycall)), silent=TRUE)
    if(!inherits(y, "try-error")) {
      ## valid subset index for a matrix
      if(rescue) {
        ## check whether it's a rectangular block, in correct order
        RR <- row(x$v)
        CC <- col(x$v)
        rcall <- ycall
        rcall[[2L]] <- quote(RR)
        ccall <- ycall
        ccall[[2L]] <- quote(CC)
        rr <- eval(as.call(rcall))
        cc <- eval(as.call(ccall))
        rseq <- sortunique(as.vector(rr))
        cseq <- sortunique(as.vector(cc))
        if(all(diff(rseq) == 1) && all(diff(cseq) == 1) &&
           (length(rr) == length(rseq) * length(cseq)) &&
           all(rr == RR[rseq, cseq]) && all(cc == CC[rseq,cseq])) {
          ## yes - make image
          dim(y) <- c(length(rseq), length(cseq))
          Y <- x
          Y$v <- y
          Y$dim <- dim(y)
          Y$xcol <- x$xcol[cseq]
          Y$yrow <- x$yrow[rseq]
          Y$xrange <- range(Y$xcol) + c(-1,1) * x$xstep/2
          Y$yrange <- range(Y$yrow) + c(-1,1) * x$ystep/2
          return(Y)
        }
      }
      ## return pixel values (possibly as matrix)
      return(y)
    }

    ## Last chance!
    if(itype == "given" &&
       !is.matrix(i) &&
       !is.null(ip <- as.ppp(i, W=as.owin(x), fatal=FALSE, check=FALSE))) {
      ## 'i' is convertible to a point pattern 
      ## Look up the greyscale values for the points of the pattern
      values <- lookup.im(x, ip$x, ip$y, naok=TRUE)
      if(drop) 
        values <- values[!is.na(values)]
      if(length(values) == 0) 
        ## ensure the zero-length vector is of the right type
        values <- 
          switch(x$type,
                 factor={ factor(, levels=levels(x)) },
                 integer = { integer(0) },
                 logical = { logical(0) },
                 real = { numeric(0) },
                 complex = { complex(0) },
                 character = { character(0) },
                 { values }
                 )
      return(values)
    }
  
    stop("The subset operation is undefined for this type of index")
  }

  Extract.im
})

update.im <- function(object, ...) {
  ## update internal structure of image after manipulation
  X <- object
  mat <- X$v
  typ <- typeof(mat)
  if(typ == "double")
    typ <- "real"
  ## deal with factor case
  if(is.factor(mat)) {
    typ <- "factor"
  } else if(!is.null(lev <- levels(mat))) {
    typ <- "factor"
    X$v <- factor(mat, levels=lev)
  }
  X$type <- typ
  return(X)
}

"[<-.im" <- function(x, i, j, ..., drop=TRUE, value) {
  # detect 'blank' arguments like second argument of x[i, ] 
  ngiven <- length(sys.call())
  nmatched <- length(match.call())
  nblank <- ngiven - nmatched
  itype <- if(missing(i)) "missing" else "given"
  jtype <- if(missing(j)) "missing" else "given"
  if(nblank == 1) {
    if(!missing(i)) jtype <- "blank"
    if(!missing(j)) itype <- "blank"
  } else if(nblank == 2) {
    itype <- jtype <- "blank"
  }

  X <- x
  W <- as.owin(X)

  stopifnot(is.im(value) || is.vector(value) ||
            is.matrix(value) || is.array(value) || is.factor(value))
  if(is.im(value)) 
    value <- value$v

  if(itype == "missing" && jtype == "missing") {
    #' no index provided
    #' set all pixels to 'value'
    #' (if drop=TRUE, this applies only to pixels inside the window)
    v <- X$v
    if(!is.factor(value)) {
      if(!drop) {
        v[] <- value
      } else {
        v[!is.na(v)] <- value
      }
    } else {
      vnew <- matrix(NA_integer_, ncol(v), nrow(v))
      if(!drop) {
        vnew[] <- as.integer(value)
      } else {
        vnew[!is.na(v)] <- as.integer(value)
      }
      v <- factor(vnew, labels=levels(value))
    }
    X$v <- v
    return(update(X))
  }
  if(itype == "given") {
    # ..................... Try a spatial index ....................
    if(verifyclass(i, "owin", fatal=FALSE)) {
      if(jtype == "given") warning("Index j ignored")
      # 'i' is a window
      if(is.empty(i))
        return(X)
      rxy <- rasterxy.mask(W)
      xx <- rxy$x
      yy <- rxy$y
      ok <- inside.owin(xx, yy, i)
      X$v[ok] <- value
      X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
      return(update(X))
    }
    if(verifyclass(i, "im", fatal=FALSE) && i$type == "logical") {
      if(jtype == "given") warning("Index j ignored")
      # convert logical vector to window where entries are TRUE
      i <- as.owin(eval.im(ifelse1NA(i)))
      # continue as above
      rxy <- rasterxy.mask(W)
      xx <- rxy$x
      yy <- rxy$y
      ok <- inside.owin(xx, yy, i)
      X$v[ok] <- value
      X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
      return(update(X))
    }
    if(is.ppp(i)) {
      # 'i' is a point pattern
      if(jtype == "given") warning("Index j ignored")
      nv <- length(value)
      np <- npoints(i)
      if(nv != np && nv != 1)
        stop("Length of replacement value != number of point locations")
      # test whether all points are inside window FRAME
      ok <- inside.owin(i$x, i$y, as.rectangle(W))
      if(any(!ok)) {
        warning("Some points are outside the outer frame of the image")
        if(nv == np)
          value <- value[ok]
        i <- i[ok]
      }
      if(npoints(i) > 0) {
        # determine row & column positions for each point 
        loc <- nearest.pixel(i$x, i$y, X)
        # set values
        X$v[cbind(loc$row, loc$col)] <- value
      }
      X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
      return(update(X))
    }
  }
  # .................. 'i' is not a spatial index ....................
  
  # Construct a matrix replacement call 
  ycall <- switch(itype,
                  given = {
                    switch(jtype,
                           given   = quote(X$v[i, j] <- value),
                           blank   = quote(X$v[i,  ] <- value),
                           missing = quote(X$v[i]    <- value))
                  },
                  blank = {
                    switch(jtype,
                           given   = quote(X$v[ , j] <- value),
                           blank   = quote(X$v[ ,  ] <- value),
                           missing = quote(X$v[ ] <- value))
                  },
                  missing = {
                    switch(jtype,
                           given   = quote(X$v[j=j] <- value),
                           blank   = quote(X$v[j= ] <- value),
                           missing = quote(X$v[] <- value))
                  })
  # try it
  litmus <- try(eval(as.call(ycall)), silent=TRUE)
  if(!inherits(litmus, "try-error")){
    X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
    return(update(X))
  }
  #  Last chance!
  if(itype == "given" &&
     !is.matrix(i) &&
     !is.null(ip <- as.ppp(i, W=W, fatal=FALSE, check=TRUE))) {
    # 'i' is convertible to a point pattern
    if(jtype == "given") warning("Index j ignored")
    nv <- length(value)
    np <- npoints(ip)
    if(nv != np && nv != 1)
      stop("Length of replacement value != number of point locations")
    # test whether all points are inside window FRAME
    ok <- inside.owin(ip$x, ip$y, as.rectangle(W))
    if(any(!ok)) {
      warning("Some points are outside the outer frame of the image")
      if(nv == np)
        value <- value[ok]
      ip <- ip[ok]
    }
    if(npoints(ip) > 0) {
      # determine row & column positions for each point 
      loc <- nearest.pixel(ip$x, ip$y, X)
      # set values
      X$v[cbind(loc$row, loc$col)] <- value
    }
    X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
    return(update(X))
  }

  stop("The subset operation is undefined for this type of index")
}

################################################################
########   other tools
################################################################

#
# This function is similar to nearest.raster.point except for
# the third argument 'im' and the different idiom for calculating
# row & column - which could be used in nearest.raster.point()

nearest.pixel <- function(x,y, Z) {
  stopifnot(is.im(Z) || is.mask(Z))
  if(length(x) > 0) {
    nr <- Z$dim[1L]
    nc <- Z$dim[2L]
    cc <- as.integer(round(1 + (x - Z$xcol[1L])/Z$xstep))
    rr <- as.integer(round(1 + (y - Z$yrow[1L])/Z$ystep))
    cc <- pmax.int(1L,pmin.int(cc, nc))
    rr <- pmax.int(1L,pmin.int(rr, nr))
  } else cc <- rr <- integer(0)
  return(list(row=rr, col=cc))
}

# Explores the 3 x 3 neighbourhood of nearest.pixel
# and finds the nearest pixel that is not NA

nearest.valid.pixel <- function(x, y, Z,
                                method=c("C","interpreted"),
                                nsearch=1) {
  method <- match.arg(method)
  switch(method,
         interpreted = {
           rc <- nearest.pixel(x,y,Z) # checks that Z is an 'im' or 'mask'
           rr <- rc$row
           cc <- rc$col
           #' check whether any pixels are outside image domain
           inside <- as.owin(Z)$m
           miss <- !inside[cbind(rr, cc)]
           if(!any(miss))
             return(rc)
           #' for offending pixels, explore 3 x 3 neighbourhood
           nr <- Z$dim[1L]
           nc <- Z$dim[2L]
           xcol <- Z$xcol
           yrow <- Z$yrow
           searching <- (-nsearch):(nsearch)
           for(i in which(miss)) {
             rows <- rr[i] + searching
             cols <- cc[i] + searching
             rows <- unique(pmax.int(1L, pmin.int(rows, nr)))
             cols <- unique(pmax.int(1L, pmin.int(cols, nc)))
             rcp <- expand.grid(row=rows, col=cols)
             ok <- inside[as.matrix(rcp)]
             if(any(ok)) {
               #' At least one of the neighbours is valid
               #' Find the closest one
               rcp <- rcp[ok,]
               dsq <- with(rcp, (x[i] - xcol[col])^2 + (y[i] - yrow[row])^2)
               j <- which.min(dsq)
               rc$row[i] <- rcp$row[j]
               rc$col[i] <- rcp$col[j]
             }
           }
         },
         C = {
           stopifnot(is.im(Z) || is.mask(Z))
           n <- length(x)
           if(n == 0) {
             cc <- rr <- integer(0)
           } else {
             nr <- Z$dim[1L]
             nc <- Z$dim[2L]
             xscaled <- (x - Z$xcol[1])/Z$xstep
             yscaled <- (y - Z$yrow[1])/Z$ystep
             aspect  <- Z$ystep/Z$xstep
             inside <- as.owin(Z)$m
             zz <- .C("nearestvalidpixel",
                      n = as.integer(n),
                      x = as.double(xscaled),
                      y = as.double(yscaled),
                      nr = as.integer(nr),
                      nc = as.integer(nc),
                      aspect = as.double(aspect),
                      z = as.integer(inside),
                      nsearch = as.integer(nsearch),
                      rr = as.integer(integer(n)),
                      cc = as.integer(integer(n)),
                      PACKAGE="spatstat")
             rr <- zz$rr + 1L
             cc <- zz$cc + 1L
             if(any(bad <- (rr == 0 | cc == 0))) {
               rr[bad] <- NA
               cc[bad] <- NA
             }
           }
           rc <- list(row=rr, col=cc)
         })  
  return(rc)
}
  

# This function is a generalisation of inside.owin()
# to images other than binary-valued images.

lookup.im <- function(Z, x, y, naok=FALSE, strict=TRUE) {
  verifyclass(Z, "im")

  if(Z$type == "factor")
    Z <- repair.old.factor.image(Z)

  if((missing(y) || is.null(y)) && all(c("x", "y") %in% names(x))) {
    y <- x$y
    x <- x$x
  }
  
  if(length(x) != length(y))
    stop("x and y must be numeric vectors of equal length")

  # initialise answer to NA 
  if(Z$type != "factor") {
    niets <- NA
    mode(niets) <- mode(Z$v)
  } else {
    niets <- factor(NA, levels=levels(Z))
  }
  value <- rep.int(niets, length(x))
               
  # test whether inside bounding rectangle
  xr <- Z$xrange
  yr <- Z$yrange
  eps <- sqrt(.Machine$double.eps)
  frameok <- (x >= xr[1L] - eps) & (x <= xr[2L] + eps) & 
             (y >= yr[1L] - eps) & (y <= yr[2L] + eps)
  
  if(!any(frameok)) {
    # all points OUTSIDE range - no further work needed
    if(!naok)
      warning("Internal error: all values NA")
    return(value)  # all NA
  }

  # consider only those points which are inside the frame
  xf <- x[frameok]
  yf <- y[frameok]
  # map locations to raster (row,col) coordinates
  if(strict)
    loc <- nearest.pixel(xf,yf,Z)
  else
    loc <- nearest.valid.pixel(xf,yf,Z)
  # look up image values
  vf <- Z$v[cbind(loc$row, loc$col)]
  
  # insert into answer
  value[frameok] <- vf

  if(!naok && anyNA(value))
    warning("Internal error: NA's generated")

  return(value)
}
  

## low level

rasterx.im <- function(x) {
  verifyclass(x, "im")
  xx <- x$xcol
  matrix(xx[col(x)], ncol=ncol(x), nrow=nrow(x))
}

rastery.im <- function(x) {
  verifyclass(x, "im")
  yy <- x$yrow
  matrix(yy[row(x)], ncol=ncol(x), nrow=nrow(x))
}

rasterxy.im <- function(x, drop=FALSE) {
  verifyclass(x, "im")
  xx <- x$xcol
  yy <- x$yrow
  ans <- cbind(x=as.vector(xx[col(x)]),
               y=as.vector(yy[row(x)]))
  if(drop) {
    ok <- as.vector(!is.na(x$v))
    ans <- ans[ok, , drop=FALSE]
  }
  return(ans)
}

## user interface 

raster.x <- function(w, drop=FALSE) {
  if(is.owin(w)) return(rasterx.mask(w, drop=drop))
  if(!is.im(w)) stop("w should be a window or an image")
  x <- w$xcol[col(w)]
  x <- if(drop) x[!is.na(w$v), drop=TRUE] else array(x, dim=w$dim)
  return(x)
}
  
raster.y <- function(w, drop=FALSE) {
  if(is.owin(w)) return(rastery.mask(w, drop=drop))
  if(!is.im(w)) stop("w should be a window or an image")
  y <- w$yrow[row(w)]
  y <- if(drop) y[!is.na(w$v), drop=TRUE] else array(y, dim=w$dim)
  return(y)
}

raster.xy <- function(w, drop=FALSE) {
  if(is.owin(w)) return(rasterxy.mask(w, drop=drop))
  if(!is.im(w)) stop("w should be a window or an image")
  x <- w$xcol[col(w)]
  y <- w$yrow[row(w)]
  if(drop) {
    ok <- !is.na(w$v)
    x <- x[ok, drop=TRUE]
    y <- y[ok, drop=TRUE]
  }
  return(list(x=as.numeric(x),
              y=as.numeric(y)))
}

##############

# methods for other functions

xtfrm.im <- function(x) { as.numeric(as.matrix.im(x)) }

as.matrix.im <- function(x, ...) {
  return(x$v)
}

as.array.im <- function(x, ...) {
  m <- as.matrix(x)
  a <- do.call(array, resolve.defaults(list(m),
                                       list(...),
                                       list(dim=c(dim(m), 1))))
  return(a)
}

as.data.frame.im <- function(x, ...) {
  verifyclass(x, "im")
  v <- x$v
  xx <- x$xcol[col(v)]
  yy <- x$yrow[row(v)]
  ok <- !is.na(v)
  xx <- as.vector(xx[ok])
  yy <- as.vector(yy[ok])
  # extract pixel values without losing factor info
  vv <- v[ok]
  dim(vv) <- NULL
  # 
  data.frame(x=xx, y=yy, value=vv, ...)
}

mean.im <- function(x, trim=0, na.rm=TRUE, ...) {
  verifyclass(x, "im")
  xvalues <- x[drop=na.rm]
  return(mean(xvalues, trim=trim, na.rm=na.rm))
}

## arguments of generic 'median' will change in R 3.4
median.im <- if("..." %in% names(formals(median))) {
   function(x, na.rm=TRUE, ...) {
    verifyclass(x, "im")
    xvalues <- x[drop=na.rm]
    return(median(xvalues, ...))
  }
} else {
   function(x, na.rm=TRUE) {
    verifyclass(x, "im")
    xvalues <- x[drop=na.rm]
    return(median(xvalues))
  }
}

where.max <- function(x, first=TRUE) {
  stopifnot(is.im(x))
  if(first) { 
    ## find the first maximum
    v <- x$v
    locn <- which.max(as.vector(v))  # ignores NA, NaN
    locrow <- as.vector(row(v))[locn]
    loccol <- as.vector(col(v))[locn]
  } else {
    ## find all maxima
    xmax <- max(x)
    M <- solutionset(x == xmax)
    loc <- which(M$m, arr.ind=TRUE)
    locrow <- loc[,1L]
    loccol <- loc[,2L]
  }
  xx <- x$xcol[loccol]
  yy <- x$yrow[locrow]
  return(ppp(x=xx, y=yy, window=Window(x)))
}

where.min <- function(x, first=TRUE) {
  stopifnot(is.im(x))
  if(first) { 
    ## find the first minimum
    v <- x$v
    locn <- which.min(as.vector(v))  # ignores NA, NaN
    locrow <- as.vector(row(v))[locn]
    loccol <- as.vector(col(v))[locn]
  } else {
    ## find all minima
    xmin <- min(x)
    M <- solutionset(x == xmin)
    loc <- which(M$m, arr.ind=TRUE)
    locrow <- loc[,1L]
    loccol <- loc[,2L]
  }
  xx <- x$xcol[loccol]
  yy <- x$yrow[locrow]
  return(ppp(x=xx, y=yy, window=Window(x)))
}

## the following ensures that 'sd' works

as.double.im <- function(x, ...) { as.double(x[], ...) }

##

hist.im <- function(x, ..., probability=FALSE, xname) {
  if(missing(xname) || is.null(xname)) xname <- short.deparse(substitute(x))
  verifyclass(x, "im")
  main <- paste("Histogram of", xname)
  # default plot arguments
  # extract pixel values
  values <- as.matrix(x)
  dim(values) <- NULL
  # barplot or histogram
  if(x$type %in% c("logical", "factor")) {
    # barplot
    tab <- table(values)
    probs <- tab/sum(tab)
    if(probability) {
      heights <- probs
      ylab <- "Probability"
    } else {
      heights <- tab
      ylab <- "Number of pixels"
    }
    mids <- do.call(barplot,
                   resolve.defaults(list(heights),
                                    list(...),
                                    list(xlab=paste("Pixel value"),
                                         ylab=ylab,
                                         main=main)))
    out <- list(counts=tab, probs=probs, heights=heights,
                mids=mids, xname=xname)
    class(out) <- "barplotdata"
  } else {
    # histogram
    values <- values[!is.na(values)]
    plotit <- resolve.defaults(list(...), list(plot=TRUE))$plot
    if(plotit) {
      ylab <- if(probability) "Probability density" else "Number of pixels"
      out <- do.call(hist.default,
                     resolve.defaults(list(values),
                                      list(...),
                                      list(freq=!probability,
                                           xlab="Pixel value",
                                           ylab=ylab,
                                           main=main)))
      out$xname <- xname
    } else {
      # plot.default whinges if `probability' given when plot=FALSE
      out <- do.call(hist.default,
                   resolve.defaults(list(values),
                                    list(...)))
      # hack!
      out$xname <- xname
    }
  }
  return(invisible(out))
}

plot.barplotdata <- function(x, ...) {
  do.call(barplot,
          resolve.defaults(list(height=x$heights),
                           list(...),
                           list(main=paste("Histogram of ", x$xname))))
}

cut.im <- function(x, ...) {
  verifyclass(x, "im")
  typ <- x$type
  if(typ %in% c("factor", "logical", "character")) 
    stop(paste0("cut.im is not defined for ", typ, "-valued images"),
         call.=FALSE)
  vcut <- cut(as.numeric(as.matrix(x)), ...)
  return(im(vcut,
            xcol=x$xcol, yrow=x$yrow,
            xrange=x$xrange, yrange=x$yrange,
            unitname=unitname(x)))
}

quantile.im <- function(x, ...) {
  verifyclass(x, "im")
  q <- do.call(quantile,
               resolve.defaults(list(as.numeric(as.matrix(x))),
                                list(...),
                                list(na.rm=TRUE)))
  return(q)
}

integral <- function(f, domain=NULL, ...) {
  UseMethod("integral")
}

integral.im <- function(f, domain=NULL, ...) {
  verifyclass(f, "im")
  typ <- f$type
  if(!any(typ == c("integer", "real", "complex", "logical")))
    stop(paste("Don't know how to integrate an image of type", sQuote(typ)))
  if(!is.null(domain)) {
    if(is.tess(domain)) return(sapply(tiles(domain), integral.im, f=f))
    f <- f[domain, drop=FALSE, tight=TRUE]
  }
  a <- with(f, sum(v, na.rm=TRUE) * xstep * ystep)
  return(a)
}

conform.imagelist <- function(X, Zlist) {
  # determine points of X where all images in Zlist are defined
  ok <- rep.int(TRUE, length(X$x))
  for(i in seq_along(Zlist)) {
    Zi <- Zlist[[i]]
    ZiX <- Zi[X, drop=FALSE]
    ok <- ok & !is.na(ZiX)
  }
  return(ok)
}

split.im <- function(x, f, ..., drop=FALSE) {
  stopifnot(is.im(x))
  if(inherits(f, "tess")) 
    subsets <- tiles(f)
  else if(is.im(f)) {
    if(f$type != "factor")
      f <- eval.im(factor(f))
    subsets <- tiles(tess(image=f))
  } else stop("f should be a tessellation or a factor-valued image")
  if(!is.subset.owin(as.owin(x), as.owin(f)))
    stop("f does not cover the window of x")
  n <- length(subsets)
  out <- vector(mode="list", length=n)
  names(out) <- names(subsets)
  for(i in 1:n)
    out[[i]] <- x[subsets[[i]], drop=drop]
  if(drop)
    return(out)
  else 
    return(as.solist(out))
}

by.im <- function(data, INDICES, FUN, ...) {
  stopifnot(is.im(data))
  V <- split(data, INDICES)
  U <- lapply(V, FUN, ...)
  return(as.solist(U, demote=TRUE))
}

rebound.im <- function(x, rect) {
  stopifnot(is.im(x))
  stopifnot(is.owin(rect))
  rect <- as.rectangle(rect)
  stopifnot(is.subset.owin(as.rectangle(x), rect))
  # compute number of extra rows/columns
  dx <- x$xstep
  nleft  <- max(0, floor((x$xrange[1L]-rect$xrange[1L])/dx))
  nright <- max(0, floor((rect$xrange[2L]-x$xrange[2L])/dx))
  dy <- x$ystep
  nbot <- max(0, floor((x$yrange[1L]-rect$yrange[1L])/dy))
  ntop <- max(0, floor((rect$yrange[2L]-x$yrange[2L])/dy))
  # determine exact x and y ranges (to preserve original pixel locations)
  xrange.new <- x$xrange + c(-nleft, nright) * dx
  yrange.new <- x$yrange + c(-nbot,  ntop) * dy
  # expand pixel data matrix
  nr <- x$dim[1L]
  nc <- x$dim[2L]
  nrnew <- nbot  + nr + ntop
  ncnew <- nleft + nc + nright
  naval <- switch(x$type,
                  factor=,
                  integer=NA_integer_,
                  real=NA_real_,
                  character=NA_character_,
                  complex=NA_complex_,
                  NA)
  vnew <- matrix(naval, nrnew, ncnew)
  if(x$type != "factor") {
    vnew[nbot + (1:nr), nleft + (1:nc)] <- x$v
  } else {
    vnew[nbot + (1:nr), nleft + (1:nc)] <- as.integer(x$v)
    vnew <- factor(vnew, labels=levels(x))
    dim(vnew) <- c(nrnew, ncnew)
  }
  # build new image object
  xnew <- im(vnew,
             xrange = xrange.new,
             yrange = yrange.new,
             unitname = unitname(x))
  return(xnew)
}

sort.im <- function(x, ...) {
  verifyclass(x, "im")
  sort(as.vector(as.matrix(x)), ...)
}

dim.im <- function(x) { x$dim }

# colour images
rgbim <- function(R, G, B, A=NULL, maxColorValue=255, autoscale=FALSE) {
  if(autoscale) {
    R <- scaletointerval(R, 0, maxColorValue)
    G <- scaletointerval(G, 0, maxColorValue)
    B <- scaletointerval(B, 0, maxColorValue)
    if(!is.null(A))
      A <- scaletointerval(A, 0, maxColorValue)
  }
  Z <- eval.im(factor(rgbNA(as.vector(R), as.vector(G), as.vector(B),
                            as.vector(A),
                            maxColorValue=maxColorValue)))
  return(Z)
}

hsvim <- function(H, S, V, A=NULL, autoscale=FALSE) {
  if(autoscale) {
    H <- scaletointerval(H, 0, 1)
    S <- scaletointerval(S, 0, 1)
    V <- scaletointerval(V, 0, 1)
    if(!is.null(A))
      A <- scaletointerval(A, 0, 1)
  }
  Z <- eval.im(factor(hsvNA(as.vector(H), as.vector(S), as.vector(V),
                            as.vector(A))))
  return(Z)
}

scaletointerval <- function(x, from=0, to=1, xrange=range(x)) {
  UseMethod("scaletointerval")
}

scaletointerval.default <- function(x, from=0, to=1, xrange=range(x)) {
  x <- as.numeric(x)
  rr <- if(missing(xrange)) range(x, na.rm=TRUE) else as.numeric(xrange)
  b <- as.numeric(to - from)/diff(rr)
  if(is.finite(b)) {
    y <- from + b * (x - rr[1L])
  } else {
    y <- (from+to)/2 + 0 * x
  }
  y[] <- pmin(pmax(y[], from), to)
  return(y)
}

scaletointerval.im <- function(x, from=0, to=1, xrange=range(x)) {
  v <- scaletointerval(x$v, from, to, xrange=xrange)
  y <- im(v, x$xcol, x$yrow, x$xrange, x$yrange, unitname(x))
  return(y)
}

zapsmall.im <- function(x, digits) {
  if(missing(digits))
    return(eval.im(zapsmall(x)))
  return(eval.im(zapsmall(x, digits=digits)))
}

domain.im <- Window.im <- function(X, ...) { as.owin(X) }

"Window<-.im" <- function(X, ..., value) {
  verifyclass(value, "owin")
  X[value, drop=FALSE]
}

padimage <- function(X, value=NA, n=1, W=NULL) {
  stopifnot(is.im(X))
  stopifnot(length(value) == 1)
  if(!missing(n) && !is.null(W)) 
    stop("Arguments n and W are incompatible", call.=FALSE)
  padW <- !is.null(W)
  if(isfac <- (X$type == "factor")) {
    ## handle factors
    levX <- levels(X)
    if(is.factor(value)) {
      stopifnot(identical(levels(X), levels(value)))
    } else {
      value <- factor(value, levels=levX)
    }
    X <- eval.im(as.integer(X))
    value <- as.integer(value)
  }
  if(!padW) {
    ## pad by 'n' pixels
    nn <- rep(n, 4)
    nleft   <- nn[1L]
    nright  <- nn[2L]
    nbottom <- nn[3L]
    ntop    <- nn[4L]
  } else {
    ## pad out to window W
    FX <- Frame(X)
    B <- boundingbox(Frame(W), FX)
    nleft   <- max(1, round((FX$xrange[1L] - B$xrange[1L])/X$xstep))
    nright  <- max(1, round((B$xrange[2L] - FX$xrange[2L])/X$xstep))
    nbottom <- max(1, round((FX$yrange[1L] - B$yrange[1L])/X$ystep))
    ntop    <- max(1, round((B$yrange[2L] - FX$yrange[2L])/X$ystep))
  }
  mX <- as.matrix(X)
  dd <- dim(mX)
  mX <- cbind(matrix(value, dd[1L], nleft, byrow=TRUE),
              as.matrix(X),
              matrix(value, dd[1L], nright, byrow=TRUE))
  dd <- dim(mX)
  mX <- rbind(matrix(rev(value), nbottom, dd[2L]),
              mX,
              matrix(value, ntop, dd[2L]))
  xcol <- with(X,
               c(xcol[1L]     - (nleft:1) * xstep,
                 xcol,
                 xcol[length(xcol)] + (1:nright) * xstep))
  yrow <- with(X,
               c(yrow[1L]     - (nbottom:1) * ystep,
                 yrow,
                 yrow[length(yrow)] + (1:ntop) * ystep))
  xr <- with(X, xrange + c(-nleft, nright) * xstep)
  yr <- with(X, yrange + c(-nbottom, ntop) * ystep)
  Y <- im(mX,
          xcol=xcol, yrow=yrow, xrange=xr, yrange=yr,
          unitname=unitname(X))
  if(isfac)
    Y <- eval.im(factor(Y, levels=seq_along(levX), labels=levX))
  if(padW && !is.rectangle(W)) 
    Y <- Y[W, drop=FALSE]
  return(Y)
}

as.function.im <- function(x, ...) {
  Z <- x
  f <- function(x,y) { Z[list(x=x, y=y)] }
  g <- funxy(f, Window(x))
  return(g)
}

anyNA.im <- function(x, recursive=FALSE) {
  anyNA(x$v)
}

ZeroValue <- function(x) {
  UseMethod("ZeroValue")
}

ZeroValue.im <- function(x) {
  lev <- levels(x)
  z <- switch(x$type,
              factor  = factor(lev[1L], levels=lev),
              integer = integer(1L),
              logical = logical(1L),
              real    = numeric(1L),
              complex = complex(1L),
              character = character(1L),
              x$v[!is.na(x$v),drop=TRUE][1])
  return(z)
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#'
#'    tests/quadschemes.R
#'
#'    Quadrature schemes, dummy points etc
#' 
#'   $Revision: 1.7 $ $Date: 2020/05/01 02:42:58 $
#'

if(FULLTEST) {
local({
  ##  class 'quad' 
  qu <- quadscheme(cells)
  qm <- quadscheme(amacrine)
  plot(qu)
  plot(qm)
  is.multitype(qu)
  is.multitype(qm)
  a <- param.quad(qu)
  a <- param.quad(qm)
  a <- equals.quad(qu)
  a <- equals.quad(qm)
  a <- domain(qu)
  unitname(qu) <- c("Furlong", "Furlongs")
  
  ## utilities
  b <- cellmiddles(square(1), 3, 4)
  b <- cellmiddles(letterR, 3, 4, distances=FALSE)
  b <- cellmiddles(letterR, 3, 4, distances=TRUE)
  v <- tilecentroids(square(1), 3, 4)
  v <- tilecentroids(letterR, 3, 4)
  n <- default.n.tiling(cells)
  n <- default.n.tiling(cells, nd=4)
  n <- default.n.tiling(cells, ntile=4)
  n <- default.n.tiling(cells, ntile=4, quasi=TRUE)

  ## quadrature weights - special cases
  X <- runifpoint(10, as.mask(letterR))
  gr <- gridweights(X, ntile=12, npix=7) # causes warnings about zero digital area
  
  ## plot.quad 
  plot(quadscheme(cells, method="dirichlet", nd=7),              tiles=TRUE)
  plot(quadscheme(cells, method="dirichlet", nd=7, exact=FALSE), tiles=TRUE)
  
  ## logistic
  d <- quadscheme.logi(cells, logi.dummy(cells, "binomial"))
  print(summary(d))
  d <- quadscheme.logi(cells, logi.dummy(cells, "poisson"))
  print(summary(d))
  d <- quadscheme.logi(cells, logi.dummy(cells, "grid"))
  print(summary(d))
  d <- quadscheme.logi(cells, logi.dummy(cells, "transgrid"))
  print(summary(d))
  d <- quadscheme.logi(amacrine,
                       logi.dummy(amacrine, "binomial", mark.repeat=TRUE))
  print(summary(d))
  d <- quadscheme.logi(amacrine,
                       logi.dummy(amacrine, "poisson", mark.repeat=FALSE))
  print(summary(d))
})
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#
#  tests/undoc.R
#
#   $Revision: 1.14 $   $Date: 2020/05/02 01:32:58 $
#
#  Test undocumented hacks, experimental code, etc

local({
  if(FULLTEST) {
    ## cases of 'pickoption'
    aliases <- c(Lenin="Ulyanov", Stalin="Djugashvili", Trotsky="Bronstein")
    surname <- "Trot"
    pickoption("leader", surname,  aliases)
    pickoption("leader", surname,  aliases, exact=TRUE, die=FALSE)
  }
  if(ALWAYS) {
    ## pixellate.ppp accepts a data frame of weights
    pixellate(cells, weights=data.frame(a=1:42, b=42:1))
    ## test parts of 'rmhsnoop' that don't require interaction with user
    rmhSnoopEnv(cells, Window(cells), 0.1)
  }
  if(FULLTEST) {
    ## linim helper functions
    df <- pointsAlongNetwork(simplenet, 0.2)
    ## Berman-Turner frame
    A <- bt.frame(quadscheme(cells), ~x, Strauss(0.07), rbord=0.07)
    print(A)
  }
  if(FULLTEST) {
    ## digestCovariates
    D <- distfun(cells)
    Z <- distmap(cells)
    U <- dirichlet(cells)
    stopifnot(is.scov(D))
    stopifnot(is.scov(Z))
    stopifnot(is.scov(U))
    stopifnot(is.scov("x"))
    dg <- digestCovariates(D=D,Z=Z,U=U,"x",list(A="x", B=D))
    ##
    a <- getfields(dg, c("A", "D", "niets"), fatal=FALSE)
    ## util.R
    gg <- pointgrid(owin(), 7)
    checkbigmatrix(1000000L, 1000000L, FALSE, TRUE)
    spatstatDiagnostic("whatever")
    M <- list(list(a=2, b=FALSE),
              list(a=2, b=TRUE))
    stopifnot(!allElementsIdentical(M))
    stopifnot(allElementsIdentical(M, "a"))
    ##
    A <- Strauss(0.1)
    A <- reincarnate.interact(A)
    ##
    ## special lists
    B <- solist(a=cells, b=redwood, c=japanesepines)
    BB <- as.ppplist(B)
    BL <- as.layered(B)
    DB <- as.imlist(lapply(B, density))
    is.solist(B)
    is.ppplist(B)
    is.imlist(DB)
    ## case of density.ppplist 
    DEB <- density(BB, se=TRUE)
  }

  if(ALWAYS) {
    ## fft
    z <- matrix(1:16, 4, 4)
    a <- fft2D(z, west=FALSE)
    if(fftwAvailable())
      b <- fft2D(z, west=TRUE)
  }

  if(ALWAYS) {
    ## experimental interactions
    pot <- function(d, par) { d <= 0.1 }
    A <- Saturated(pot)
    print(A)
    A <- update(A, name="something")
    ppm(amacrine ~ x, A, rbord=0.1)
  }

  if(ALWAYS) { # platform dependent
    #' version-checking
    now <- Sys.Date()
    versioncurrency.spatstat(now + 80, FALSE)
    versioncurrency.spatstat(now + 140, FALSE)
    versioncurrency.spatstat(now + 400, FALSE)
    versioncurrency.spatstat(now + 1000)
  }

  if(FULLTEST) {
    #' general Ord interaction
    gradual <- function(d, pars) {
      y <- pmax(0, 0.005 - d)/0.005
      if(is.matrix(d)) y <- matrix(y, nrow(d), ncol(d))
      return(y)
    }
    B <- Ord(gradual, "gradual Ord process")
  }
})
##
##  tests/updateppm.R
##
##  Check validity of update.ppm
##
##  $Revision: 1.5 $ $Date: 2020/05/02 01:32:58 $

local({

  if(ALWAYS) {
    require(spatstat.utils)
    h <- function(m1, m2) {
      mc <- short.deparse(sys.call())
      cat(paste(mc, "\t... "))
      m1name <- short.deparse(substitute(m1))
      m2name <- short.deparse(substitute(m2))
      if(!identical(names(coef(m1)), names(coef(m2))))
        stop(paste("Differing results for", m1name, "and", m2name,
                   "in updateppm.R"),
             call.=FALSE)
      cat("OK\n")
    }
    
    X <- redwood[c(TRUE,FALSE)]
    Y <- redwood[c(FALSE,TRUE)]
    fit0f <- ppm(X ~ 1, nd=8)
    fit0p <- ppm(X, ~1, nd=8)
    fitxf <- ppm(X ~ x, nd=8)
    fitxp <- ppm(X, ~x, nd=8)

    cat("Basic consistency ...\n")
    h(fit0f, fit0p)
    h(fitxf, fitxp)
  
    cat("\nTest correct handling of model formulas ...\n")
    h(update(fitxf, Y), fitxf)
    h(update(fitxf, Q=Y), fitxf)
    h(update(fitxf, Y~x), fitxf)
    h(update(fitxf, Q=Y~x), fitxf)
    h(update(fitxf, ~x), fitxf)
  }

  if(FULLTEST) {
    h(update(fitxf, Y~1), fit0f)
    h(update(fitxf, ~1), fit0f)
    h(update(fit0f, Y~x), fitxf)
    h(update(fit0f, ~x), fitxf)

    h(update(fitxp, Y), fitxp)
    h(update(fitxp, Q=Y), fitxp)
    h(update(fitxp, Y~x), fitxp)
    h(update(fitxp, Q=Y~x), fitxp)
    h(update(fitxp, ~x), fitxp)

    h(update(fitxp, Y~1), fit0p)
    h(update(fitxp, ~1), fit0p)
    h(update(fit0p, Y~x), fitxp)
    h(update(fit0p, ~x), fitxp)
  }

  if(ALWAYS) {
    cat("\nTest scope handling for left hand side ...\n")
    X <- Y
    h(update(fitxf), fitxf)
  }

  if(ALWAYS) {
    cat("\nTest scope handling for right hand side ...\n")
    Z <- distmap(X)
    fitZf <- ppm(X ~ Z)
    fitZp <- ppm(X, ~ Z)
    h(update(fitxf, X ~ Z), fitZf)
  }
  if(FULLTEST) {
    h(update(fitxp, X ~ Z), fitZp)
    h(update(fitxf, . ~ Z), fitZf)
    h(update(fitZf, . ~ x), fitxf)
    h(update(fitZf, . ~ . - Z), fit0f)
    h(update(fitxp, . ~ Z), fitZp)
    h(update(fitZp, . ~ . - Z), fit0p)
    h(update(fit0p, . ~ . + Z), fitZp)
    h(update(fitZf, . ~ . ), fitZf)
    h(update(fitZp, . ~ . ), fitZp)
  }
  if(ALWAYS) {
    cat("\nTest use of internal data ...\n")
    h(update(fitZf, ~ x, use.internal=TRUE), fitxf)
    fitsin <- update(fitZf, X~sin(Z))
    h(update(fitZf, ~ sin(Z), use.internal=TRUE), fitsin)
  }
  if(FULLTEST) {
    cat("\nTest step() ... ")
    fut <- ppm(X ~ Z + x + y, nd=8)
    fut0 <- step(fut, trace=0)
    cat("OK\n")
  }

  if(ALWAYS) {
    ## test update.lppm
    X <- runiflpp(20, simplenet)
    fit0 <- lppm(X ~ 1)
    fit1 <- update(fit0, ~ x)
    anova(fit0, fit1, test="LR")
    cat("update.lppm(fit, ~trend) is OK\n")
    fit2 <- update(fit0, . ~ x)
    anova(fit0, fit2, test="LR")
    cat("update.lppm(fit, . ~ trend) is OK\n")
  }
})
#
#  tests/vcovppm.R
#
#  Check validity of vcov.ppm algorithms
#
#  Thanks to Ege Rubak
#
#  $Revision: 1.12 $  $Date: 2020/05/02 01:32:58 $
#

local({

  set.seed(42)
  X <- rStrauss(200, .5, .05)
  model <- ppm(X, inter = Strauss(.05))

  if(ALWAYS) {
    b  <- vcov(model, generic = TRUE, algorithm = "basic")
    v  <- vcov(model, generic = TRUE, algorithm = "vector")
    vc <- vcov(model, generic = TRUE, algorithm = "vectorclip")
    vn <- vcov(model, generic = FALSE)

    disagree <- function(x, y, tol=1e-7) { max(abs(x-y)) > tol }
    asymmetric <- function(x) { disagree(x, t(x)) }

    if(asymmetric(b))
      stop("Non-symmetric matrix produced by vcov.ppm 'basic' algorithm")
    if(asymmetric(v))
      stop("Non-symmetric matrix produced by vcov.ppm 'vector' algorithm")
    if(asymmetric(vc))
      stop("Non-symmetric matrix produced by vcov.ppm 'vectorclip' algorithm")
    if(asymmetric(vn))
      stop("Non-symmetric matrix produced by vcov.ppm Strauss algorithm")
    
    if(disagree(v, b))
      stop("Disagreement between vcov.ppm algorithms 'vector' and 'basic' ")
    if(disagree(v, vc))
      stop("Disagreement between vcov.ppm algorithms 'vector' and 'vectorclip' ")
    if(disagree(vn, vc))
      stop("Disagreement between vcov.ppm generic and Strauss algorithms")
  }

  if(ALWAYS) { # C code
    ## Geyer code
    xx <- c(0.7375956, 0.6851697, 0.6399788, 0.6188382)
    yy <- c(0.5816040, 0.6456319, 0.5150633, 0.6191592)
    Y <- ppp(xx, yy, window=square(1))
    modelY <- ppm(Y ~1, Geyer(0.1, 1))
    
    b  <- vcov(modelY, generic = TRUE, algorithm = "basic")
    v  <- vcov(modelY, generic = TRUE, algorithm = "vector")
    vc <- vcov(modelY, generic = TRUE, algorithm = "vectorclip")
    
    if(asymmetric(b))
      stop("Non-symmetric matrix produced by vcov.ppm 'basic' algorithm for Geyer model")
    if(asymmetric(v))
      stop("Non-symmetric matrix produced by vcov.ppm 'vector' algorithm for Geyer model")
    if(asymmetric(vc))
      stop("Non-symmetric matrix produced by vcov.ppm 'vectorclip' algorithm for Geyer model")
  
    if(disagree(v, b))
      stop("Disagreement between vcov.ppm algorithms 'vector' and 'basic' for Geyer model")
    if(disagree(v, vc))
      stop("Disagreement between vcov.ppm algorithms 'vector' and 'vectorclip' for Geyer model")
  }

  if(ALWAYS) { # C code
    ## tests of 'deltasuffstat' code
    ##     Handling of offset terms
    modelH <- ppm(cells ~x, Hardcore(0.05))
    a <- vcov(modelH, generic=TRUE) ## may fall over
    b <- vcov(modelH, generic=FALSE)
    if(disagree(a, b))
      stop("Disagreement between vcov.ppm algorithms for Hardcore model")
  
    ##     Correctness of pairwise.family$delta2
    modelZ <- ppm(amacrine ~1, MultiStrauss(radii=matrix(0.1, 2, 2)))
    b <- vcov(modelZ, generic=FALSE)
    g <- vcov(modelZ, generic=TRUE)
    if(disagree(b, g))
      stop("Disagreement between vcov.ppm algorithms for MultiStrauss model")

    ## Test that 'deltasuffstat' works for Hybrids
    modelHyb <- ppm(japanesepines ~ 1, Hybrid(Strauss(0.05), Strauss(0.1)))
    vHyb <- vcov(modelHyb)
  }

  if(FULLTEST) {
    ## Code blocks for other choices of 'what'
    model <- ppm(X ~ 1, Strauss(.05))
    cG <- vcov(model, what="corr")
    cP <- vcov(update(model, Poisson()), what="corr")
    ## outdated usage
    cX <- vcov(model, A1dummy=TRUE)

    ## Model with zero-length coefficient vector
    lam <- intensity(X)
    f <- function(x,y) { rep(lam, length(x)) }
    model0 <- ppm(X ~ offset(log(f)) - 1)
    dd <- vcov(model0)
    cc <- vcov(model0, what="corr")
  
    ## Model with NA coefficients
    fit <- ppm(X ~ log(f))
    vcov(fit)
    fitE <- emend(fit, trace=TRUE)
  
    ## Other weird stuff
    su <- suffloc(ppm(X ~ x))
  }
})
#
# tests/windows.R
#
# Tests of owin geometry code
#
#  $Revision: 1.16 $  $Date: 2020/05/02 01:32:58 $

local({
  if(ALWAYS) { # C code
    ## Ege Rubak spotted this problem in 1.28-1
    A <- as.owin(ants)
    B <- dilation(A, 140)
    if(!is.subset.owin(A, B))
      stop("is.subset.owin fails in polygonal case")

    ## thanks to Tom Rosenbaum
    A <- shift(square(3), origin="midpoint")
    B <- shift(square(1), origin="midpoint")
    AB <- setminus.owin(A, B)
    D <- shift(square(2), origin="midpoint")
    if(is.subset.owin(D,AB))
      stop("is.subset.owin fails for polygons with holes")

    ## thanks to Brian Ripley / SpatialVx
    M <- as.mask(letterR)
    stopifnot(area(bdry.mask(M)) > 0)
    stopifnot(area(convexhull(M)) > 0)
    R <- as.mask(square(1))
    stopifnot(area(bdry.mask(R)) > 0)
    stopifnot(area(convexhull(R)) > 0)
  }

  if(FULLTEST) {
    RR <- convexify(as.mask(letterR))
    CC <- covering(letterR, 0.05, eps=0.1)
  
    #' as.owin.data.frame
    V <- as.mask(letterR, eps=0.2)
    Vdf <- as.data.frame(V)
    Vnew <- as.owin(Vdf)
    zz <- mask2df(V)
  }

  if(ALWAYS) { # C code
    RM <- owinpoly2mask(letterR, as.mask(Frame(letterR)), check=TRUE)
  }

  if(FULLTEST) {
    #' as.owin
    U <- as.owin(quadscheme(cells))
    U2 <- as.owin(list(xmin=0, xmax=1, ymin=0, ymax=1))
  }

  if(FULLTEST) {
    #' intersections involving masks
    B1 <- square(1)
    B2 <- as.mask(shift(B1, c(0.2, 0.3)))
    o12 <- overlap.owin(B1, B2)
    o21 <- overlap.owin(B2, B1)
    i12 <- intersect.owin(B1, B2, eps=0.01)
    i21 <- intersect.owin(B2, B1, eps=0.01)
    E2 <- emptywindow(square(2))
    e12 <- intersect.owin(B1, E2)
    e21 <- intersect.owin(E2, B1)
  
    #' geometry
    inradius(B1)
    inradius(B2)
    inradius(letterR)
    inpoint(B1)
    inpoint(B2)
    inpoint(letterR)
    is.convex(B1)
    is.convex(B2)
    is.convex(letterR)
    volume(letterR)
    perimeter(as.mask(letterR))
    boundingradius(cells)
  
    boundingbox(letterR)
    boundingbox(letterR, NULL)
    boundingbox(cells, ppm(cells ~ 1))
    boundingbox(solist(letterR))

  }

  if(ALWAYS) { # C code
    spatstat.options(Cbdrymask=FALSE)
    bb <- bdry.mask(letterR)
    spatstat.options(Cbdrymask=TRUE)
  }

  if(FULLTEST) {
    X <- longleaf[square(50)]
    marks(X) <- marks(X)/8
    D <- discs(X)
    D <- discs(X, delta=5, separate=TRUE)
  }

  if(ALWAYS) { # C code
    AD <- dilated.areas(cells,
                        r=0.01 * matrix(1:10, 10,1),
                        constrained=FALSE, exact=FALSE)
  }

  if(FULLTEST) {
    periodify(B1, 2)
    periodify(union.owin(B1, B2), 2)
    periodify(letterR, 2)
  }

  if(ALWAYS) {
    #' Ancient bug in inside.owin
    W5 <- owin(poly=1e5*cbind(c(-1,1,1,-1),c(-1,-1,1,1)))
    W6 <- owin(poly=1e6*cbind(c(-1,1,1,-1),c(-1,-1,1,1)))
    i5 <- inside.owin(0,0,W5)
    i6 <- inside.owin(0,0,W6)
    if(!i5) stop("Wrong answer from inside.owin")
    if(i5 != i6) stop("Results from inside.owin are scale-dependent")
  }

  if(FULLTEST) {
    #' miscellaneous utilities
    thrash <- function(f) {
      f(letterR)
      f(Frame(letterR))
      f(as.mask(letterR))
    }
    thrash(meanX.owin)
    thrash(meanY.owin)
    thrash(intX.owin)
    thrash(intY.owin)

    interpretAsOrigin("right", letterR)
    interpretAsOrigin("bottom", letterR)
    interpretAsOrigin("bottomright", letterR)
    interpretAsOrigin("topleft", letterR)
    interpretAsOrigin("topright", letterR)
  }

  if(ALWAYS) { # depends on polyclip
    A <- break.holes(letterR)
    B <- break.holes(letterR, splitby="y")
    plot(letterR, col="blue", use.polypath=FALSE)
  }

  if(ALWAYS) {  # C code
    #' mask conversion
    M <- as.mask(letterR)
    D2 <- as.data.frame(M)              # two-column
    D3 <- as.data.frame(M, drop=FALSE)  # three-column
    M2 <- as.owin(D2)
    M3 <- as.owin(D3)
    W2 <- owin(mask=D2)
    W3 <- owin(mask=D3)
  }

  if(FULLTEST) {
    #' void/empty cases
    nix <- nearest.raster.point(numeric(0), numeric(0), M)
    E <- emptywindow(Frame(letterR))
    print(E)
    #' cases of summary.owin
    print(summary(E)) # empty
    print(summary(Window(humberside))) # single polygon
    #' additional cases of owin()
    B <- owin(mask=M$m) # no pixel size or coordinate info
    xy <- as.data.frame(letterR)
    xxyy <- split(xy[,1:2], xy$id)
    spatstat.options(checkpolygons=TRUE)
    H <- owin(poly=xxyy, check=TRUE)
  }

  #' Code for/using intersection and union of windows

  if(FULLTEST) {
    Empty <- emptywindow(Frame(letterR))
    a <- intersect.owin()
    a <- intersect.owin(Empty)
    a <- intersect.owin(Empty, letterR)
    a <- intersect.owin(letterR, Empty)
    b <- intersect.owin()
    b <- intersect.owin(Empty)
    b <- intersect.owin(Empty, letterR)
    b <- intersect.owin(letterR, Empty)
    d <- union.owin(as.mask(square(1)), as.mask(square(2)))
    #' [.owin
    A <- erosion(letterR, 0.2)
    Alogi <- as.im(TRUE, W=A)
    B <- letterR[A]
    B <- letterR[Alogi]
    #' miscellaneous
    D <- convexhull(Alogi)
  }
})

reset.spatstat.options()
##
## tests/xysegment.R
##                      [SEE ALSO tests/segments.R]
##
##    Test weird problems and boundary cases for line segment code
##
##    $Version$ $Date: 2020/05/02 01:32:58 $ 
##
local({
  if(FULLTEST) {
    ## segment of length zero
    B <- psp(1/2, 1/2, 1/2, 1/2, window=square(1))
    BB <- angles.psp(B)
    A <- runifpoint(3)
    AB <- project2segment(A,B)

    ## mark inheritance
    X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
    marks(X) <- 1:10
    Y <- selfcut.psp(X)
    marks(X) <- data.frame(A=1:10, B=factor(letters[1:10]))
    Z <- selfcut.psp(X)

    #' psp class support
    S <- as.psp(simplenet)
    marks(S) <- sample(factor(c("A","B")), nobjects(S), replace=TRUE)
    intensity(S)
    intensity(S, weights=runif(nsegments(S)))
  }
})

#
#
#     setcov.R
#
#     $Revision: 1.16 $ $Date: 2018/08/07 11:44:46 $
#
#    Compute the set covariance function of a window
#    or the (noncentred) spatial covariance function of an image
#

setcov <- function(W, V=W, ...) {
  W <- as.owin(W)
  # pixel approximation
  mW <- as.mask(W, ...)
  Z <- as.im(mW, na.replace=0)
  if(missing(V)) 
    return(imcov(Z))
  # cross-covariance
  V <- as.owin(V)
  mV <- as.mask(V, ...)
  Z2 <- as.im(mV, na.replace=0)
  imcov(Z, Z2)
}

imcov <- function(X, Y=X) {
  if(missing(Y)) Y <- NULL
  convolve.im(X, Y, reflectX = FALSE, reflectY=TRUE)
}

convolve.im <- function(X, Y=X, ..., reflectX=FALSE, reflectY=FALSE) {
  stopifnot(is.im(X))
  have.Y <- !missing(Y) && !is.null(Y)
  crosscov <- have.Y || reflectX || reflectY
  trap.extra.arguments(..., .Context="In convolve.im")
  #' Check whether Fastest Fourier Transform in the West is available
  west <- fftwAvailable()
  #'
  if(have.Y) {
    # cross-covariance 
    stopifnot(is.im(Y))
    Xbox <- as.rectangle(X)
    Ybox <- as.rectangle(Y)
    # first shift images to common midpoint, to reduce storage
    Xmid <- centroid.owin(Xbox)
    Ymid <- centroid.owin(Ybox)
    svec <- as.numeric(Xmid) - as.numeric(Ymid)
    Y <- shift(Y, svec)
    # ensure images are compatible
    XY <- harmonise.im(X=X, Y=Y)
    X <- XY$X
    Y <- XY$Y
  } else {
    # Y is missing or NULL
    Y <- X
    Xbox <- Ybox <- as.rectangle(X)
  }
  M <- X$v
  M[is.na(M)] <- 0
  xstep <- X$xstep
  ystep <- X$ystep
  # pad with zeroes
  nr <- nrow(M)
  nc <- ncol(M)
  Mpad <- matrix(0, ncol=2*nc, nrow=2*nr)
  Mpad[1:nr, 1:nc] <- M
  lengthMpad <- 4 * nc * nr
  fM <- fft2D(Mpad, west=west)
  if(!crosscov) {
    # compute convolution square
    G <- fft2D(fM^2, inverse=TRUE, west=west)/lengthMpad
  } else {
    # compute set cross-covariance or convolution by FFT
    N <- Y$v
    N[is.na(N)] <- 0
    Npad <- matrix(0, ncol=2*nc, nrow=2*nr)
    Npad[1:nr, 1:nc] <- N
    fN <- fft2D(Npad, west=west)
    if(reflectY) fN <- Conj(fN)
    if(reflectX) fM <- Conj(fM)
    G <- fft2D(fM * fN, inverse=TRUE, west=west)/lengthMpad
  }
#  cat(paste("maximum imaginary part=", max(Im(G)), "\n"))
  G <- Mod(G) * xstep * ystep
  if(reflectX != reflectY) {
    # Currently G[i,j] corresponds to a vector shift of
    #     dy = (i-1) mod nr, dx = (j-1) mod nc.
    # Rearrange this periodic function so that 
    # the origin of translations (0,0) is at matrix position (nr,nc)
    # NB this introduces an extra row and column
    G <- G[ ((-nr):nr) %% (2 * nr) + 1, (-nc):nc %% (2*nc) + 1]
  }
  # Determine spatial domain of full raster image
  XB <- as.rectangle(X)
  YB <- as.rectangle(Y)
  # undo shift
  if(have.Y) YB <- shift(YB, -svec)
  # reflect
  if(reflectX) XB <- reflect(XB)
  if(reflectY) YB <- reflect(YB)
  # Minkowski sum of covering boxes
  xran <- XB$xrange + YB$xrange
  yran <- XB$yrange + YB$yrange
  # Declare spatial domain
  out <- im(G, xrange = xran, yrange=yran)
  if(crosscov) {
    # restrict to actual spatial domain of function
    if(reflectX) Xbox <- reflect(Xbox)
    if(reflectY) Ybox <- reflect(Ybox)
   # Minkowski sum 
    xran <- Xbox$xrange + Ybox$xrange
    yran <- Xbox$yrange + Ybox$yrange   
    XYbox <- owin(xran, yran)
    out <- out[XYbox, rescue=TRUE]
  }
  unitname(out) <- unitname(X)
  return(out)
}

#
#   round.R
#
#   discretisation of coordinates
#
#   $Revision: 1.6 $  $Date: 2019/02/20 03:34:50 $

round.ppp <- round.pp3 <- round.ppx <- function(x, digits=0) {
  coords(x) <- round(as.matrix(coords(x)), digits=digits)
  return(x)
}

rounding <- function(x) {
  UseMethod("rounding")
}

rounding.ppp <- rounding.pp3 <- rounding.ppx <- function(x) {
  rounding(as.matrix(coords(x)))
}

rounding.default <- function(x) {
  # works for numeric, complex, matrix etc
  if(all(x == 0))
    return(NULL)
  if(isTRUE(all.equal(x, round(x)))) { 
    # integers: go up
    k <- 0
    smallk <- -log10(.Machine$double.xmax)
    repeat {
      if(k < smallk || !isTRUE(all.equal(x, round(x, k-1))))
        return(k)
      k <- k-1
    }
  } else {
    # not integers: go down
    k <- 1
    bigk <- -log10(.Machine$double.eps)
    repeat {
      if(k > bigk || isTRUE(all.equal(x, round(x, k))))
        return(k)
      k <- k+1
    }
  }
}

#'
#'   bw.ppl.R
#'
#'   Likelihood cross-validation for kernel smoother of point pattern
#'
#'   $Revision: 1.12 $ $Date: 2020/04/08 04:27:12 $
#'

bw.ppl <- function(X, ..., srange=NULL, ns=16, sigma=NULL,
                   weights=NULL, shortcut=FALSE, warn=TRUE) {
  stopifnot(is.ppp(X))
  if(!is.null(sigma)) {
    stopifnot(is.numeric(sigma) && is.vector(sigma))
    ns <- length(sigma)
  } else {
    if(!is.null(srange)) check.range(srange) else {
      nnd <- nndist(X)
      srange <- c(min(nnd[nnd > 0]), diameter(as.owin(X))/2)
    }
    sigma <- geomseq(from=srange[1L], to=srange[2L], length.out=ns)
  }
  cv <- numeric(ns)
  if(shortcut) {
    for(i in 1:ns) {
      si <- sigma[i]
      lamx <- density(X, sigma=si, at="points", leaveoneout=TRUE,
                    weights=weights, ...)
      cv[i] <- sum(log(lamx))
    }
  } else {
    for(i in 1:ns) {
      si <- sigma[i]
      lamx <- density(X, sigma=si, at="points", leaveoneout=TRUE,
                      weights=weights, ...)
      lam <- density(X, sigma=si,
                     weights=weights, ...)
      cv[i] <- sum(log(lamx)) - integral.im(lam)
    }
  }
  result <- bw.optim(cv, sigma, iopt=which.max(cv), 
                     creator="bw.ppl",
                     criterion="Likelihood Cross-Validation",
                     warnextreme=warn,
                     hargnames="srange",
                     unitname=unitname(X))
  return(result)
}

# superimpose.R
#
# $Revision: 1.37 $ $Date: 2017/12/30 05:03:15 $
#
#
############################# 

superimpose <- function(...) {
  # remove any NULL arguments
  arglist <- list(...)
  if(any(isnull <- sapply(arglist, is.null)))
    return(do.call(superimpose, arglist[!isnull]))
  UseMethod("superimpose")
}

superimpose.default <- function(...) {
  argh <- list(...)
  #' First expand any arguments which are lists of objects
  argh <- expandSpecialLists(argh, "solist")
  #' Now dispatch
  if(any(sapply(argh, is.lpp)) || any(sapply(argh, inherits, what="linnet")))
    return(do.call(superimpose.lpp, argh))
  if(any(sapply(argh, is.psp)))
    return(do.call(superimpose.psp, argh))
  #' default
  return(do.call(superimpose.ppp, argh))
}

superimpose.ppp <- function(..., W=NULL, check=TRUE) {
  arglist <- list(...)
  # Check that all "..." arguments have x, y coordinates
  hasxy <- unlist(lapply(arglist, checkfields, L=c("x", "y")))
  if(!all(hasxy)) {
    nbad <- sum(bad <- !hasxy)
    stop(paste(ngettext(nbad, "Argument", "Arguments"),
               commasep(which(bad)),
               ngettext(nbad, "does not", "do not"),
               "have components x and y"),
         call.=FALSE)
  }
  
  # concatenate lists of (x,y) coordinates
  XY <- do.call(concatxy, arglist)
  needcheck <- TRUE

  # determine whether there is any window information
  if(!is.owin(W)) {
    ## we have to compute the final window
    if(is.function(W)) {
      ## W is a function like bounding.box.xy or ripras
      ## Apply function to the x,y coordinates; it should return an owin
      WXY <- W(XY)
      if(!is.owin(WXY))
        stop("Function W did not return an owin object", call.=FALSE)
      W <- WXY
    } else if(is.character(W)) {
      ## character string identifies a function
      pW <- pmatch(W, c("convex", "rectangle", "bbox", "none"))
      if(is.na(pW))
        stop(paste("Unrecognised option W=", sQuote(W)), call.=FALSE)
      WXY <- switch(pW,
                    convex=ripras(XY),
                    rectangle=ripras(XY, shape="rectangle"),
                    bbox=boundingbox(XY),
                    none=NULL)
      # in these cases we don't need to verify that the points are inside.
      needcheck <- !is.null(WXY)
      if(!is.null(WXY)) W <- WXY
    } else if(is.null(W)) {
      if(any(isppp <- unlist(lapply(arglist, is.ppp)))) {
        ## extract windows from ppp objects
        wins <- unname(lapply(arglist[isppp], as.owin))
        ## take union
        W <- if(length(wins) == 1) wins[[1]] else do.call(union.owin, wins)
      } else {
        ## no window information
        return(XY)
      }
    } else stop("Argument W is not understood")
  }
  # extract the marks if any
  nobj <- lengths(lapply(arglist, getElement, name="x"))
  marx  <- superimposeMarks(arglist, nobj)
  #
  ppp(XY$x, XY$y, window=W, marks=marx, check=check & needcheck)
}

superimpose.splitppp <- superimpose.ppplist <-
  function(..., W=NULL, check=TRUE) {
    arglist <- list(...)
    while(any(h <- sapply(arglist, inherits, what=c("splitppp", "ppplist")))) {
      i <- min(which(h))
      arglist <- insertinlist(arglist, i, arglist[[i]])
    }
    do.call(superimpose, append(arglist, list(W=W, check=check)))
  }

superimpose.psp <- function(..., W=NULL, check=TRUE) {
  # superimpose any number of line segment patterns
  arglist <- list(...)
  misscheck <- missing(check)

  if(!all(sapply(arglist, is.psp)))
    stop("Patterns to be superimposed must all be psp objects", call.=FALSE)

  # extract segment coordinates
  matlist <- lapply(lapply(arglist, getElement, name="ends"),
                    asNumericMatrix)
  
  # tack them together
  mat <- do.call(rbind, matlist)

  # determine whether there is any window information
  needcheck <- FALSE
  if(!is.owin(W)) {
    # we have to compute the final window
    WXY <- NULL
#    Wpsp <- NULL
    if(any(ispsp <- unlist(lapply(arglist, is.psp)))) {
      # extract windows from psp objects
      wins <- unname(lapply(arglist[ispsp], as.owin))
      # take union
      Wppp <- if(length(wins) == 1) wins[[1]] else do.call(union.owin, wins)
    }
    if(is.function(W) || is.character(W)) {
      # guess window from x, y coordinates
      XY <- list(x=cbind(mat[,1], mat[,3]),
                 y=cbind(mat[,2], mat[,4]))
      if(is.function(W)) {
        # W is a function like bounding.box.xy or ripras
        # Apply function to the x,y coordinates; it should return an owin
        WXY <- W(XY)
        if(!is.owin(WXY))
          stop("Function W did not return an owin object", call.=FALSE)
      }
      if(is.character(W)) {
        # character string identifies a function
        pW <- pmatch(W, c("convex", "rectangle", "bbox", "none"))
        if(is.na(pW))
          stop(paste("Unrecognised option W=", sQuote(W)), call.=FALSE)
        WXY <- switch(pW,
                      convex=ripras(XY),
                      rectangle=ripras(XY, shape="rectangle"),
                      bbox=boundingbox(XY),
                      none=NULL)
      # in these cases we don't need to verify that the points are inside.
        needcheck <- !is.null(WXY)
      }
    }
    W <- union.owin(WXY, Wppp)
  }
  
  # extract marks, if any
  nobj <- sapply(arglist, nsegments)
  marx <- superimposeMarks(arglist, nobj)

  if(misscheck && !needcheck) check <- FALSE
  return(as.psp(mat, window=W, marks=marx, check=check))
}

superimposeMarks <- function(arglist, nobj) {
  # combine marks from the objects in the argument list
  marxlist <- lapply(arglist, marks)
  marx <- do.call(markappend, unname(marxlist))
  nama <- names(arglist)
  if(length(nama) == length(arglist) && all(nzchar(nama))) {
    # arguments are named: use names as (extra) marks
    newmarx <- factor(rep.int(nama, nobj), levels=nama)
    marx <- markcbind(marx, newmarx)
    if(ncol(marx) == 2) {
      ## component marks were not named: call them 'origMarks'
      colnames(marx) <- c("origMarks", "pattern")
    } else colnames(marx)[ncol(marx)] <- "pattern"
  }
  return(marx)
}

#
#	breakpts.S
#
#	A simple class definition for the specification
#       of histogram breakpoints in the special form we need them.
#
#	even.breaks()
#
#	$Revision: 1.25 $	$Date: 2020/04/12 08:34:19 $
#
#
#       Other functions in this directory use the standard Splus function
#	hist() to compute histograms of distance values.
#       One argument of hist() is the vector 'breaks'
#	of breakpoints for the histogram cells. 
#
#       The breakpoints must
#            (a) span the range of the data
#            (b) be given in increasing order
#            (c) satisfy breaks[2] = 0,
#
#	The function make.even.breaks() will create suitable breakpoints.
#
#       Condition (c) means that the first histogram cell has
#       *right* endpoint equal to 0.
#
#       Since all our distance values are nonnegative, the effect of (c) is
#       that the first histogram cell counts the distance values which are
#       exactly equal to 0. Hence F(0), the probability P{X = 0},
#       is estimated without a discretisation bias.
#
#	We assume the histograms have followed the default counting rule
#	in hist(), which is such that the k-th entry of the histogram
#	counts the number of data values in 
#		I_k = ( breaks[k],breaks[k+1] ]	for k > 1
#		I_1 = [ breaks[1],breaks[2]   ]
#
#	The implementations of estimators of c.d.f's in this directory
#       produce vectors of length = length(breaks)-1
#       with value[k] = estimate of F(breaks[k+1]),
#       i.e. value[k] is an estimate of the c.d.f. at the RIGHT endpoint
#       of the kth histogram cell.
#
#       An object of class 'breakpts' contains:
#
#              $val     the actual breakpoints
#              $max     the maximum value (= last breakpoint)
#              $ncells  total number of histogram cells
#              $r       right endpoints, r = val[-1]
#              $even    logical = TRUE if cells known to be evenly spaced
#              $npos    number of histogram cells on the positive halfline
#                        = length(val) - 2,
#                       or NULL if cells not evenly spaced
#              $step    histogram cell width
#                       or NULL if cells not evenly spaced
#       
# --------------------------------------------------------------------
breakpts <- function(val, maxi, even=FALSE, npos=NULL, step=NULL) {
  out <- list(val=as.numeric(val),
              max=as.numeric(maxi),
              ncells=length(val)-1L, r = val[-1L],
              even=isTRUE(even),
              npos=npos, step=step)
  class(out) <- "breakpts"
  out
}

scalardilate.breakpts <- function(X, f, ...) {
  out <- with(X,
              list(val    = f*val,
                   max    = f*max,
                   ncells = ncells,
                   r      = f*r,
                   even   = even,
                   npos   = npos,
                   step   = if(is.null(step)) NULL else (f*step)))
  class(out) <- "breakpts"
  out
}  
                            
make.even.breaks <- function(bmax, npos, bstep) {
  bmax <- as.numeric(bmax)
  if(bmax <= 0)
    stop("bmax must be positive")
  if(missing(bstep) && missing(npos))
    stop(paste("Must specify either", sQuote("bstep"),
               "or", sQuote("npos")))
  if(!missing(npos)) {
    npos <- as.integer(npos)
    bstep <- bmax/npos
    val <- seq(from=0, to=bmax, length.out=npos+1L)
    val <- c(-bstep,val)
    right <- bmax
  } else {
    bstep <- as.numeric(bstep)
    npos <- ceiling(bmax/bstep)
    right <- bstep * npos
    val <- seq(from=0, to=right, length.out=npos+1L)
    val <- c(-bstep,val)
  }
  breakpts(val, right, TRUE, npos, bstep)
}

"as.breakpts" <- function(...) {

  XL <- list(...)

  if(length(XL) == 1L) {
    # single argument
    X <- XL[[1L]]

    if(!is.null(class(X)) && class(X) == "breakpts")
    # X already in correct form
      return(X)
  
    if(is.vector(X) && length(X) > 2) {
      ## it's a vector
      X <- as.numeric(X)
      if(X[2L] != 0)
        stop("breakpoints do not satisfy breaks[2] = 0")
      # The following test for equal spacing is used in hist.default
      steps <- diff(X)
      if(diff(range(steps)) < 1e-07 * mean(steps))
        # equally spaced
        return(breakpts(X, max(X), TRUE, length(X)-2, steps[1L]))
      else
        # unknown spacing
        return(breakpts(X, max(X), FALSE))
    }
  } else {

    # There are multiple arguments.
  
    # exactly two arguments - interpret as even.breaks()
    if(length(XL) == 2)
      return(make.even.breaks(XL[[1L]], XL[[2L]]))

    # two arguments 'max' and 'npos'
  
    if(!is.null(XL$max) && !is.null(XL$npos))
      return(make.even.breaks(XL$max, XL$npos))

    # otherwise
    stop("Don't know how to convert these data to breakpoints")
  }
  # never reached
}


check.hist.lengths <- function(hist, breaks) {
  verifyclass(breaks, "breakpts")
  nh <- length(hist)
  nb <- breaks$ncells
  if(nh != nb)
    stop(paste("Length of histogram =", nh,
               "not equal to number of histogram cells =", nb))
}

breakpts.from.r <- function(r) {
  if(!is.numeric(r) && !is.vector(r))
    stop("r must be a numeric vector")
  r <- as.numeric(r)
  if(length(r) < 2)
    stop(paste("r has length", length(r), "- must be at least 2"))
  if(r[1L] != 0)
    stop("First r value must be 0")
  if(any(diff(r) <= 0))
    stop("successive values of r must be increasing")
  dr <- r[2L] - r[1L]
  b <- c(-dr, r)
  return(as.breakpts(b))
}

handle.r.b.args <- function(r=NULL, breaks=NULL, window, pixeps=NULL,
                            rmaxdefault=NULL) {
  if(!is.null(r) && !is.null(breaks))
    stop(paste("Do not specify both",
               sQuote("r"), "and", sQuote("breaks")))
  if(!is.null(breaks)) {
    breaks <- as.breakpts(breaks)
  } else if(!is.null(r)) {
    breaks <- breakpts.from.r(r)
  } else {
    #' determine rmax
    #' ignore infinite or NA values of rmaxdefault
    if(!isTRUE(is.finite(rmaxdefault)))
      rmaxdefault <- NULL
    rmax <- rmaxdefault %orifnull% diameter(Frame(window))
    #' determine spacing
    if(is.null(pixeps)) {
      pixeps <-
        if(is.mask(window)) min(window$xstep, window$ystep) else rmax/128
    }
    rstep <- pixeps/4
    breaks <- make.even.breaks(rmax, bstep=rstep)
  }
  return(breaks)
}

check.finespacing <- function(r, eps=NULL, win=NULL,
                              rmaxdefault = max(r), 
                              context="",
                              action=c("fatal", "warn", "silent"),
                              rname) {
  if(missing(rname)) rname <- deparse(substitute(r))
  action <- match.arg(action)
  if(is.null(eps)) {
    b <- handle.r.b.args(window=win, rmaxdefault=rmaxdefault)
    eps <- b$step
  }
  dr <- max(diff(r))
  if(dr > eps * 1.01) {
    whinge <- paste(context, "the successive", rname,
                    "values must be finely spaced:",
                    "given spacing =",
                    paste0(signif(dr, 5), ";"),
                    "required spacing <= ",
                    signif(eps, 3))
    switch(action,
           fatal = stop(whinge, call.=FALSE),
           warn = warning(whinge, call.=FALSE),
           silent = {})
    return(FALSE)
  }
  return(TRUE)
}

## clusterfunctions.R
##
## Contains the generic functions:
##  - clusterkernel
##  - clusterfield
##  - clusterradius.
##
##   $Revision: 1.4 $  $Date: 2019/10/17 08:46:15 $
##

clusterkernel <- function(model, ...) {
  UseMethod("clusterkernel")
}

clusterkernel.kppm <- function(model, ...) {
  kernelR <- Kpcf.kppm(model, what = "kernel")
  f <- function(x, y = 0, ...){
    kernelR(sqrt(x^2+y^2))
  }
  return(f)
}

clusterkernel.character <- function(model, ...){
  info <- spatstatClusterModelInfo(model, onlyPCP = TRUE)
  internalkernel <- info$kernel
  dots <- list(...)
  par <- c(kappa = 1, scale = dots$scale)
  par <- info$checkpar(par, old = TRUE)
  nam <- info$clustargsnames
  margs <- NULL
  if(!is.null(nam))
    margs <- dots[nam]
  f <- function(x, y = 0, ...){
    internalkernel(par = par, rvals = sqrt(x^2+y^2), margs = margs)
  }
  return(f)
}

clusterfield <- function(model, locations = NULL, ...) {
    UseMethod("clusterfield")
}

clusterfield.kppm <- function(model, locations = NULL, ...) {
    f <- clusterkernel(model)
    if(is.null(locations)){
        if(!is.stationary(model))
            stop("The model is non-stationary. The argument ",
                 sQuote("locations"), " must be given.")
        locations <- centroid.owin(Window(model), as.ppp = TRUE)
    }
    clusterfield.function(f, locations, ..., mu = model$mu)
}

clusterfield.character <- function(model, locations = NULL, ...){
    f <- clusterkernel(model, ...)
    clusterfield.function(f, locations, ...)
}

clusterfield.function <- function(model, locations = NULL, ..., mu = NULL) {
    if(is.null(locations)){
        locations <- ppp(.5, .5, window=square(1))
    }
    if(!inherits(locations, "ppp"))
        stop("Argument ", sQuote("locations"), " must be a point pattern (ppp).")

    if("sigma" %in% names(list(...)) && "sigma" %in% names(formals(model)))
        warning("Currently ", sQuote("sigma"),
                "cannot be passed as an extra argument to the kernel function. ",
                "Please redefine the kernel function to use another argument name.")

    rslt <- density(locations, kernel=model, ...)
    if(is.null(mu))
        return(rslt)
    mu <- as.im(mu, W=rslt)
    if(min(mu)<0)
        stop("Cluster reference intensity ", sQuote("mu"), " is negative.")
    return(rslt*mu)
}

clusterradius <- function(model, ...){
    UseMethod("clusterradius")
}

clusterradius.character <- function(model, ..., thresh = NULL, precision = FALSE){
    info <- spatstatClusterModelInfo(model, onlyPCP = TRUE)
    rmax <- info$range(..., thresh = thresh)
    if(precision){
        ddist <- function(r) info$ddist(r, ...)
        prec <- integrate(ddist, 0, rmax)
        attr(rmax, "prec") <- prec
    }
    return(rmax)
}

clusterradius.kppm <- function(model, ..., thresh = NULL, precision = FALSE){
    a <- list(model = model$clusters,
              thresh = thresh,
              precision = precision)
    a <- append(a, as.list(c(model$clustpar, model$clustargs)))
    do.call(clusterradius.character, a)
}

reach.kppm <- function(x, ..., epsilon) {
  thresh <- if(missing(epsilon)) NULL else epsilon
  2 * clusterradius.kppm(x, ..., thresh=thresh)
}

#
#   edges2triangles.R
#
#   $Revision: 1.16 $  $Date: 2018/09/23 09:21:22 $
#

edges2triangles <- function(iedge, jedge, nvert=max(iedge, jedge),
                            ..., check=TRUE, friendly=rep(TRUE, nvert)) {
  usefriends <- !missing(friendly)
  if(check) {
    stopifnot(length(iedge) == length(jedge))
    stopifnot(all(iedge > 0))
    stopifnot(all(jedge > 0))
    if(!missing(nvert)) {
      stopifnot(all(iedge <= nvert))
      stopifnot(all(jedge <= nvert))
    }
    if(usefriends) {
      stopifnot(is.logical(friendly))
      stopifnot(length(friendly) == nvert)
      usefriends <- !all(friendly)
    }
  }
  # zero length data, or not enough to make triangles
  if(length(iedge) < 3)
    return(matrix(integer(0), nrow=0, ncol=3,
                  dimnames=list(NULL, c("i", "j", "k"))))
  # sort in increasing order of 'iedge'
  oi <- fave.order(iedge)
  iedge <- iedge[oi]
  jedge <- jedge[oi]
  # call C
  storage.mode(nvert) <- storage.mode(iedge) <- storage.mode(jedge) <- "integer"
  if(usefriends) {
    fr <- as.logical(friendly)
    storage.mode(fr) <- "integer"
    zz <- .Call("trioxgraph",
                nv=nvert, iedge=iedge, jedge=jedge, friendly=fr,
                PACKAGE="spatstat")
  } else if(spatstat.options("fast.trigraph")) {
    zz <- .Call("triograph",
                nv=nvert, iedge=iedge, jedge=jedge,
                PACKAGE="spatstat")
  } else {
    #' testing purposes only
    zz <- .Call("trigraph",
                nv=nvert, iedge=iedge, jedge=jedge,
                PACKAGE="spatstat")
  }
  mat <- as.matrix(as.data.frame(zz, col.names=c("i", "j", "k")))
  return(mat)
}

# compute triangle diameters as well

trianglediameters <- function(iedge, jedge, edgelength, ..., 
                              nvert=max(iedge, jedge),
                              dmax=Inf, check=TRUE) {
  if(check) {
    stopifnot(length(iedge) == length(jedge))
    stopifnot(length(iedge) == length(edgelength))
    stopifnot(all(iedge > 0))
    stopifnot(all(jedge > 0))
    if(!missing(nvert)) {
      stopifnot(all(iedge <= nvert))
      stopifnot(all(jedge <= nvert))
    }
    if(is.finite(dmax)) check.1.real(dmax)
  }
  # zero length data
  if(length(iedge) == 0 || dmax < 0)
    return(data.frame(i=integer(0),
                      j=integer(0),
                      k=integer(0),
                      diam=numeric(0)))

  # call C
  storage.mode(nvert) <- storage.mode(iedge) <- storage.mode(jedge) <- "integer"
  storage.mode(edgelength) <- "double"
  if(is.infinite(dmax)) {
    zz <- .Call("triDgraph",
                nv=nvert, iedge=iedge, jedge=jedge, edgelength=edgelength,
                PACKAGE = "spatstat")
  } else {
    storage.mode(dmax) <- "double"
    zz <- .Call("triDRgraph",
                nv=nvert, iedge=iedge, jedge=jedge, edgelength=edgelength,
                dmax=dmax,
                PACKAGE = "spatstat")
  }    
  df <- as.data.frame(zz)
  colnames(df) <- c("i", "j", "k", "diam")
  return(df)
}

closetriples <- function(X, rmax) {
  a <- closepairs(X, rmax, what="ijd", twice=FALSE, neat=FALSE)
  tri <- trianglediameters(a$i, a$j, a$d, nvert=npoints(X), dmax=rmax)
  return(tri)
}

# extract 'vees', i.e. triples (i, j, k) where i ~ j and i ~ k

edges2vees <- function(iedge, jedge, nvert=max(iedge, jedge),
                            ..., check=TRUE) {
  if(check) {
    stopifnot(length(iedge) == length(jedge))
    stopifnot(all(iedge > 0))
    stopifnot(all(jedge > 0))
    if(!missing(nvert)) {
      stopifnot(all(iedge <= nvert))
      stopifnot(all(jedge <= nvert))
    }
  }
  # zero length data, or not enough to make vees
  if(length(iedge) < 2)
    return(data.frame(i=numeric(0),
                      j=numeric(0),
                      k=numeric(0)))
  # call 
  vees <- .Call("graphVees",
                nv = nvert,
                iedge = iedge,
                jedge = jedge,
                PACKAGE="spatstat")
  names(vees) <- c("i", "j", "k")
  vees <- as.data.frame(vees)
  return(vees)
}

  

#
#  Perfect Simulation 
#
#  $Revision: 1.21 $ $Date: 2017/06/05 10:31:58 $
#
#  rStrauss
#  rHardcore
#  rStraussHard
#  rDiggleGratton
#  rDGS
#  rPenttinen

rStrauss <- function(beta, gamma=1, R=0, W=owin(), expand=TRUE,
                     nsim=1, drop=TRUE) {

  if(!missing(W)) 
    verifyclass(W, "owin")

  check.1.real(beta)
  check.1.real(gamma)
  check.1.real(R)

  check.finite(beta)
  check.finite(gamma)
  check.finite(R)
  
  stopifnot(beta > 0)
  stopifnot(gamma >= 0)
  stopifnot(gamma <= 1)
  stopifnot(R >= 0)

  runif(1)

  Wsim <- expandwinPerfect(W, expand, rmhexpand(distance=2*R))
  xrange <- Wsim$xrange
  yrange <- Wsim$yrange

  result <- vector(mode="list", length=nsim)

  for(i in 1:nsim) {
    storage.mode(beta) <- storage.mode(gamma) <- storage.mode(R) <- "double"
    storage.mode(xrange) <- storage.mode(yrange) <- "double"
  
    z <- .Call("PerfectStrauss",
               beta,
               gamma,
               R,
               xrange,
               yrange,
               PACKAGE = "spatstat")

    X <- z[[1]]
    Y <- z[[2]]
    nout <- z[[3]]
    times <- c(start=z[[4]], end=z[[5]])
    
    if(nout<0)
      stop("internal error: copying failed in PerfectStrauss")

    seqn <- seq_len(nout)
    P <- ppp(X[seqn], Y[seqn], window=Wsim, check=FALSE)
    if(attr(Wsim, "changed"))
      P <- P[W]
    attr(P, "times") <- times

    if(nsim == 1 && drop) return(P)
    
    result[[i]] <- P
  }
  result <- as.solist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}

#  Perfect Simulation of Hardcore process

rHardcore <- function(beta, R=0, W=owin(), expand=TRUE, nsim=1, drop=TRUE) {
  if(!missing(W)) 
    verifyclass(W, "owin")

  check.1.real(beta)
  check.1.real(R)

  check.finite(beta)
  check.finite(R)

  stopifnot(beta > 0)
  stopifnot(R    >= 0)

  runif(1)

  Wsim <- expandwinPerfect(W, expand, rmhexpand(distance=2*R))
  xrange <- Wsim$xrange
  yrange <- Wsim$yrange

  result <- vector(mode="list", length=nsim)

  for(i in 1:nsim) {
    storage.mode(beta) <- storage.mode(R) <- "double"
    storage.mode(xrange) <- storage.mode(yrange) <- "double"
  
    z <- .Call("PerfectHardcore",
               beta,
               R,
               xrange,
               yrange,
               PACKAGE = "spatstat")

    X <- z[[1]]
    Y <- z[[2]]
    nout <- z[[3]]
    
    if(nout<0)
      stop("internal error: copying failed in PerfectHardcore")

    seqn <- seq_len(nout)
    P <- ppp(X[seqn], Y[seqn], window=Wsim, check=FALSE)
    if(attr(Wsim, "changed"))
      P <- P[W]

    if(nsim == 1 && drop) return(P)
    result[[i]] <- P
  }
  result <- as.solist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}

#
#  Perfect simulation of hybrid Strauss-Hardcore
#        provided gamma <= 1
#

rStraussHard <- function(beta, gamma=1, R=0, H=0, W=owin(),
                         expand=TRUE, nsim=1, drop=TRUE) {
  if(!missing(W)) 
    verifyclass(W, "owin")

  check.1.real(beta)
  check.1.real(gamma)
  check.1.real(R)
  check.1.real(H)

  check.finite(beta)
  check.finite(gamma)
  check.finite(R)
  check.finite(H)
  
  stopifnot(beta > 0)
  stopifnot(gamma >= 0)
  if(gamma > 1)
    stop("Sorry, perfect simulation is only implemented for gamma <= 1")
  stopifnot(R >= 0)
  stopifnot(H >= 0)
  stopifnot(H <= R)

  runif(1)

  Wsim <- expandwinPerfect(W, expand, rmhexpand(distance=2*R))
  xrange <- Wsim$xrange
  yrange <- Wsim$yrange

  result <- vector(mode="list", length=nsim)

  for(i in 1:nsim) {
    storage.mode(beta) <- storage.mode(gamma) <-
      storage.mode(R) <- storage.mode(H) <- "double"
    storage.mode(xrange) <- storage.mode(yrange) <- "double"
  
    z <- .Call("PerfectStraussHard",
               beta,
               gamma,
               R,
               H,
               xrange,
               yrange,
               PACKAGE = "spatstat")

    X <- z[[1]]
    Y <- z[[2]]
    nout <- z[[3]]

    if(nout<0)
      stop("internal error: copying failed in PerfectStraussHard")

    seqn <- seq_len(nout)
    P <- ppp(X[seqn], Y[seqn], window=Wsim, check=FALSE)
    if(attr(Wsim, "changed"))
      P <- P[W]

    if(nsim == 1 && drop) return(P)
    result[[i]] <- P
  }
  result <- as.solist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}

#
#  Perfect Simulation of Diggle-Gratton process
#

rDiggleGratton <- function(beta, delta, rho, kappa=1, W=owin(),
                           expand=TRUE, nsim=1, drop=TRUE) {
  if(!missing(W)) 
    verifyclass(W, "owin")

  check.1.real(beta)
  check.1.real(delta)
  check.1.real(rho)
  check.1.real(kappa)

  check.finite(beta)
  check.finite(delta)
  check.finite(rho)
  check.finite(kappa)

  stopifnot(beta > 0)
  stopifnot(delta >= 0)
  stopifnot(rho   >= 0)
  stopifnot(delta <= rho)
  stopifnot(kappa >= 0)

  runif(1)

  Wsim <- expandwinPerfect(W, expand, rmhexpand(distance=2*rho))
  xrange <- Wsim$xrange
  yrange <- Wsim$yrange

  result <- vector(mode="list", length=nsim)

  for(i in 1:nsim) {
    storage.mode(beta) <- "double"
    storage.mode(delta) <- storage.mode(rho) <- storage.mode(kappa) <- "double"
    storage.mode(xrange) <- storage.mode(yrange) <- "double"
  
    z <- .Call("PerfectDiggleGratton",
               beta,
               delta,
               rho,
               kappa,
               xrange,
               yrange,
               PACKAGE = "spatstat")

    X <- z[[1]]
    Y <- z[[2]]
    nout <- z[[3]]

    if(nout<0)
      stop("internal error: copying failed in PerfectDiggleGratton")

    seqn <- seq_len(nout)
    P <- ppp(X[seqn], Y[seqn], window=Wsim, check=FALSE)
    if(attr(Wsim, "changed"))
      P <- P[W]

    if(nsim == 1 && drop) return(P)
    result[[i]] <- P
  }
  result <- as.solist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}


#
#  Perfect Simulation of Diggle-Gates-Stibbard process
#

rDGS <- function(beta, rho, W=owin(), expand=TRUE, nsim=1, drop=TRUE) {
  if(!missing(W)) 
    verifyclass(W, "owin")

  check.1.real(beta)
  check.1.real(rho)

  check.finite(beta)
  check.finite(rho)

  stopifnot(beta > 0)
  stopifnot(rho  >= 0)

  runif(1)

  Wsim <- expandwinPerfect(W, expand, rmhexpand(distance=2*rho))
  xrange <- Wsim$xrange
  yrange <- Wsim$yrange

  result <- vector(mode="list", length=nsim)

  for(i in 1:nsim) {
    storage.mode(beta) <- "double"
    storage.mode(rho) <- "double"
    storage.mode(xrange) <- storage.mode(yrange) <- "double"
  
    z <- .Call("PerfectDGS",
               beta,
               rho,
               xrange,
               yrange,
               PACKAGE = "spatstat")

    X <- z[[1]]
    Y <- z[[2]]
    nout <- z[[3]]
    
    if(nout<0)
      stop("internal error: copying failed in PerfectDGS")
    
    seqn <- seq_len(nout)
    P <- ppp(X[seqn], Y[seqn], window=Wsim, check=FALSE)
    if(attr(Wsim, "changed"))
      P <- P[W]

    if(nsim == 1 && drop) return(P)
    result[[i]] <- P
  }
  result <- as.solist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}


#
#  Perfect Simulation of Penttinen process
#

rPenttinen <- function(beta, gamma=1, R, W=owin(),
                       expand=TRUE, nsim=1, drop=TRUE) {
  if(!missing(W)) 
    verifyclass(W, "owin")

  check.1.real(beta)
  check.1.real(gamma)
  check.1.real(R)

  check.finite(beta)
  check.finite(gamma)
  check.finite(R)

  stopifnot(beta > 0)
  stopifnot(gamma >= 0)
  stopifnot(gamma <= 1)
  stopifnot(R >= 0)

  runif(1)

  Wsim <- expandwinPerfect(W, expand, rmhexpand(distance=2*R))
  xrange <- Wsim$xrange
  yrange <- Wsim$yrange

  result <- vector(mode="list", length=nsim)

  for(i in 1:nsim) {
    storage.mode(beta) <- storage.mode(gamma) <- storage.mode(R) <- "double"
    storage.mode(xrange) <- storage.mode(yrange) <- "double"
  
    z <- .Call("PerfectPenttinen",
               beta,
               gamma,
               R,
               xrange,
               yrange,
               PACKAGE = "spatstat")

    X <- z[[1]]
    Y <- z[[2]]
    nout <- z[[3]]
    
    if(nout<0)
      stop("internal error: copying failed in PerfectPenttinen")
    
    seqn <- seq_len(nout)
    P <- ppp(X[seqn], Y[seqn], window=Wsim, check=FALSE)
    if(attr(Wsim, "changed"))
      P <- P[W]

    if(nsim == 1 && drop) return(P)
    result[[i]] <- P
  }
  result <- as.solist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}


## .......  utilities .................................

expandwinPerfect <- function(W, expand, amount) {
  ## expand 'W' if expand=TRUE according to default 'amount'
  ## or expand 'W' using rmhexpand(expand)
  if(!is.logical(expand)) {
    amount <- rmhexpand(expand)
    expand <- TRUE
  }
  changed <- FALSE
  if(expand) {
    W <- expand.owin(W, amount)
    changed <- TRUE
  }
  if(!is.rectangle(W)) {
    W <- as.rectangle(W)
    changed <- TRUE
    warning(paste("Simulation will be performed in the containing rectangle",
                  "and clipped to the original window."),
            call.=FALSE)
  }
  attr(W, "changed") <- changed
  return(W)
}

#
#  quadratcount.R
#
#  $Revision: 1.57 $  $Date: 2016/08/15 03:05:15 $
#

quadratcount <- function(X, ...) {
  UseMethod("quadratcount")
}

quadratcount.splitppp <- function(X, ...) {
  solapply(X, quadratcount, ...)
}

quadratcount.ppp <- function(X, nx=5, ny=nx, ...,
                             xbreaks=NULL, ybreaks=NULL,
                             tess=NULL)  {
  verifyclass(X, "ppp")
  W <- X$window

  if(is.null(tess)) {
    # rectangular boundaries 
    if(!is.numeric(nx))
      stop("nx should be numeric")
    # start with rectangular tessellation
    tess <- quadrats(as.rectangle(W),
                     nx=nx, ny=ny, xbreaks=xbreaks, ybreaks=ybreaks)
    # fast code for counting points in rectangular grid
    Xcount <- rectquadrat.countEngine(X$x, X$y, tess$xgrid, tess$ygrid)
    #
    if(W$type != "rectangle") {
      # intersections of rectangles with window including empty intersections
      tess <- quadrats(X,
                       nx=nx, ny=ny, xbreaks=xbreaks, ybreaks=ybreaks,
                       keepempty=TRUE)
      # now delete the empty quadrats and the corresponding counts
      nonempty <- !tiles.empty(tess)
#     WAS: nonempty <- !unlist(lapply(tiles(tess), is.empty))
      if(!any(nonempty))
        stop("All tiles are empty")
      if(!all(nonempty)) {
#        ntiles <- sum(nonempty)
        tess   <- tess[nonempty]
        Xcount <- t(Xcount)[nonempty]
        # matrices and tables are in row-major order,
        # tiles in a rectangular tessellation are in column-major order
        Xcount <- array(Xcount,
                        dimnames=list(tile=tilenames(tess)))
        class(Xcount) <- "table"
      }
    }
  } else {
    # user-supplied tessellation
    if(!inherits(tess, "tess")) {
      tess <- try(as.tess(tess), silent=TRUE)
      if(inherits(tess, "try-error"))
        stop("The argument tess should be a tessellation", call.=FALSE)
    }
    if(tess$type == "rect") {
      # fast code for counting points in rectangular grid
      Xcount <- rectquadrat.countEngine(X$x, X$y, tess$xgrid, tess$ygrid)
    } else {
      # quadrats are another type of tessellation
      Y <- cut(X, tess)
      if(anyNA(marks(Y)))
        warning("Tessellation does not contain all the points of X")
      Xcount <- table(tile=marks(Y))
    }
  }
  attr(Xcount, "tess") <- tess
  class(Xcount) <- c("quadratcount", class(Xcount))
  return(Xcount)
}

plot.quadratcount <- function(x, ...,
                              add=FALSE, entries=as.vector(t(as.table(x))),
                              dx=0, dy=0, show.tiles=TRUE,
                              textargs = list()) {
  xname <- short.deparse(substitute(x))
  tess <- attr(x, "tess")
  # add=FALSE, show.tiles=TRUE  => plot tiles + numbers
  # add=FALSE, show.tiles=FALSE => plot window (add=FALSE) + numbers
  # add=TRUE,  show.tiles=TRUE  => plot tiles  (add=TRUE) + numbers
  # add=TRUE,  show.tiles=FALSE => plot numbers
  if(show.tiles || !add) {
    context <- if(show.tiles) tess else as.owin(tess)
    do.call(plot,
            resolve.defaults(list(context, add=add),
                             list(...),
                             list(main=xname),
                             .StripNull=TRUE))
  }
  if(!is.null(entries)) {
    labels <- paste(as.vector(entries))
    til <- tiles(tess)
    incircles <- lapply(til, incircle)
    x0 <- sapply(incircles, getElement, name="x")
    y0 <- sapply(incircles, getElement, name="y")
    ra <- sapply(incircles, getElement, name="r")
    do.call.matched(text.default,
                    resolve.defaults(list(x=x0 + dx * ra, y = y0 + dy * ra),
                                     list(labels=labels),
                                     textargs, 
                                     list(...)),
                    funargs=graphicsPars("text"))
  }
  return(invisible(NULL))
}

rectquadrat.breaks <- function(xr, yr, nx=5, ny=nx, xbreaks=NULL, ybreaks=NULL) {
  if(is.null(xbreaks))
    xbreaks <- seq(from=xr[1], to=xr[2], length.out=nx+1)
  else if(min(xbreaks) > xr[1] || max(xbreaks) < xr[2])
    stop("xbreaks do not span the range of x coordinates in the window")
  if(is.null(ybreaks))
    ybreaks <- seq(from=yr[1], to=yr[2], length.out=ny+1)
  else if(min(ybreaks) > yr[1] || max(ybreaks) < yr[2])
    stop("ybreaks do not span the range of y coordinates in the window")
  return(list(xbreaks=xbreaks, ybreaks=ybreaks))
}

rectquadrat.countEngine <- function(x, y, xbreaks, ybreaks, weights) {
  if(length(x) > 0) {
    # check validity of breaks
    if(!all(inside.range(range(x), range(xbreaks))))
      stop("xbreaks do not span the actual range of x coordinates in data")
    if(!all(inside.range(range(y), range(ybreaks))))
      stop("ybreaks do not span the actual range of y coordinates in data")
  }
  # WAS: 
  # xg <- cut(x, breaks=xbreaks, include.lowest=TRUE)
  # yg <- cut(y, breaks=ybreaks, include.lowest=TRUE)
  xg <- fastFindInterval(x, xbreaks, labels=TRUE)
  yg <- fastFindInterval(y, ybreaks, labels=TRUE)
  if(missing(weights)) {
    sumz <- table(list(y=yg, x=xg))
  } else {
    # was: 
    # sumz <- tapply(weights, list(y=yg, x=xg), sum)
    # if(any(nbg <- is.na(sumz)))
    #  sumz[nbg] <- 0
    sumz <- tapplysum(weights, list(y=yg, x=xg), do.names=TRUE)
  }
  # reverse order of y 
  sumz <- sumz[rev(seq_len(nrow(sumz))), ]
  sumz <- as.table(sumz)
  #
  attr(sumz, "xbreaks") <- xbreaks
  attr(sumz, "ybreaks") <- ybreaks
  return(sumz)
}

quadrats <- function(X, nx=5, ny=nx, xbreaks = NULL, ybreaks = NULL,
                     keepempty=FALSE) {
  W <- as.owin(X)
  xr <- W$xrange
  yr <- W$yrange
  b <- rectquadrat.breaks(xr, yr, nx, ny, xbreaks, ybreaks)
  # rectangular tiles
  Z <- tess(xgrid=b$xbreaks, ygrid=b$ybreaks, unitname=unitname(W))
  if(W$type != "rectangle") {
    # intersect rectangular tiles with window W
    if(!keepempty) {
      Z <- intersect.tess(Z, W)
    } else {
      til <- tiles(Z)
      for(i in seq_along(til))
        til[[i]] <- intersect.owin(til[[i]], W)
      Z <- tess(tiles=til, window=W, keepempty=TRUE)
    }
  }
  return(Z)
}

as.tess.quadratcount <- function(X) {
  return(attr(X, "tess"))
}

as.owin.quadratcount <- function(W, ..., fatal=TRUE) {
  return(as.owin(as.tess(W), ..., fatal=fatal))
}

domain.quadratcount <- Window.quadratcount <- function(X, ...) { as.owin(X) }

intensity.quadratcount <- function(X, ..., image=FALSE) {
  Y <- as.tess(X)
  a <- tile.areas(Y)
  ## in the rectangular case, tiles are indexed in column-major order
  if(Y$type == "rect" && length(dim(X)) > 1) 
    a <- matrix(a, byrow=TRUE, nrow(X), ncol(X))
  lambda <- X/a
  if(!image) {
    trap.extra.arguments(...)
    class(lambda) <- "table"
    attr(lambda, "tess") <- NULL
    return(lambda)
  }
  ## again to handle rectangular case
  lambda <- as.vector(t(lambda))
  tileid <- as.im(Y, ...)
  result <- eval.im(lambda[tileid])
  return(result)
}

## The shift method is undocumented.
## It is only needed in plot.listof / plot.solist / plot.layered

shift.quadratcount <- function(X, ...) {
  attr(X, "tess") <- te <- shift(attr(X, "tess"), ...)
  attr(X, "lastshift") <- getlastshift(te)
  return(X)
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#  tests/emptymarks.R
#
# test cases where there are no (rows or columns of) marks
#
#  $Revision: 1.4 $ $Date: 2020/04/28 12:58:26 $

if(ALWAYS) {
local({
  n <- npoints(cells)
  df <- data.frame(x=1:n, y=factor(sample(letters, n, replace=TRUE)))
  nocolumns <- c(FALSE, FALSE)
  norows <- rep(FALSE, n)
  X <- cells
  marks(X) <- df
  marks(X) <- df[,1]
  marks(X) <- df[,nocolumns]
  Z <- Y <- X[integer(0)]
  marks(Y) <- df[norows,]
  stopifnot(is.marked(Y))
  marks(Z) <- df[norows,nocolumns]
  stopifnot(!is.marked(Z))
})
}
#
#  tests/envelopes.R
#
#  Test validity of envelope data
#
#  $Revision: 1.22 $  $Date: 2020/04/28 12:58:26 $
#

local({
checktheo <- function(fit) {
  fitname <- deparse(substitute(fit))
  en <- envelope(fit, nsim=4, verbose=FALSE, nrep=1e3)
  nama <- names(en)
  expecttheo <- is.poisson(fit) && is.stationary(fit)
  context <- paste("Envelope of", fitname)
  if(expecttheo) {
    if(!("theo" %in% nama))
      stop(paste(context, "did not contain", sQuote("theo")))
    if("mmean" %in% nama)
      stop(paste(context, "unexpectedly contained", sQuote("mmean")))
  } else {
    if("theo" %in% nama)
      stop(paste(context, "unexpectedly contained", sQuote("theo")))
    if(!("mmean" %in% nama))
      stop(paste(context, "did not contain", sQuote("mmean")))
  }
  cat(paste(context, "has correct format\n"))
}

if(ALWAYS) {
  checktheo(ppm(cells ~x))
}
if(FULLTEST) {
  checktheo(ppm(cells))
  checktheo(ppm(cells ~1, Strauss(0.1)))
}

# check envelope calls from 'alltypes'
if(ALWAYS) a <- alltypes(demopat, Kcross, nsim=4, envelope=TRUE)
if(FULLTEST) b <- alltypes(demopat, Kcross, nsim=4, envelope=TRUE, global=TRUE)

# check 'transform' idioms
if(ALWAYS) A <- envelope(cells, Kest, nsim=4, transform=expression(. - .x))
if(FULLTEST) B <- envelope(cells, Kest, nsim=4, transform=expression(sqrt(./pi) - .x))

#' check savefuns/savepatterns with global 
fit <- ppm(cells~x)
if(ALWAYS) Ef <- envelope(fit, Kest, nsim=4, savefuns=TRUE, global=TRUE)
if(FULLTEST) Ep <- envelope(fit, Kest, nsim=4, savepatterns=TRUE, global=TRUE)

#' check handling of 'dangerous' cases
if(FULLTEST) {
  fut <- ppm(redwood ~ x)
  Ek <- envelope(fut, Kinhom, update=FALSE, nsim=4)
  kfut <- kppm(redwood3 ~ x)
  Ekk <- envelope(kfut, Kinhom, lambda=density(redwood3), nsim=7)
}

# check conditional simulation
if(FULLTEST) {
  e1 <- envelope(cells, Kest, nsim=4, fix.n=TRUE)
  e2 <- envelope(amacrine, Kest, nsim=4, fix.n=TRUE)
  e3 <- envelope(amacrine, Kcross, nsim=4, fix.marks=TRUE)
  e4 <- envelope(finpines, Kest, nsim=4, fix.n=TRUE) # multiple columns of marks
  e5 <- envelope(finpines, Kest, nsim=4, fix.marks=TRUE)
}
if(ALWAYS) { # invokes C code
  fit <- ppm(japanesepines ~ 1, Strauss(0.04))
  e6 <- envelope(fit, Kest, nsim=4, fix.n=TRUE)
  fit2 <- ppm(amacrine ~ 1, Strauss(0.03))
  e7 <- envelope(fit2, Gcross, nsim=4, fix.marks=TRUE)
}

# check pooling of envelopes in global case
E1 <- envelope(cells, Kest, nsim=5, savefuns=TRUE, global=TRUE)
E2 <- envelope(cells, Kest, nsim=12, savefuns=TRUE, global=TRUE)
p12 <- pool(E1, E2)
p12 <- pool(E1, E2, savefuns=TRUE)
if(FULLTEST) {
  F1 <- envelope(cells, Kest, nsim=5,
                 savefuns=TRUE, savepatterns=TRUE, global=TRUE)
  F2 <- envelope(cells, Kest, nsim=12,
                 savefuns=TRUE, savepatterns=TRUE, global=TRUE)
  p12 <- pool(F1, F2)
  p12 <- pool(F1, F2, savefuns=TRUE, savepatterns=TRUE)
  E1r <- envelope(cells, Kest, nsim=5, savefuns=TRUE, global=TRUE,
                  ginterval=c(0.05, 0.15))
  E2r <- envelope(cells, Kest, nsim=12, savefuns=TRUE, global=TRUE,
                  ginterval=c(0.05, 0.15))
  p12r <- pool(E1r, E2r)
}
})

if(FULLTEST) {
local({
  #' as.data.frame.envelope
  Nsim <- 5
  E <- envelope(cells, nsim=Nsim, savefuns=TRUE)
  A <- as.data.frame(E)
  B <- as.data.frame(E, simfuns=TRUE)
  stopifnot(ncol(B) - ncol(A) == Nsim)
})
}

if(FULLTEST) {
local({
  #' cases not covered elsewhere
  A <- envelope(cells, nsim=5, alternative="less",
                do.pwrong=TRUE, use.theory=FALSE,
                savepatterns=TRUE, savefuns=TRUE)
  print(A)
  B <- envelope(A, nsim=5, savefuns=TRUE)
  D <- envelope(cells, "Lest", nsim=5)
  
  UU <- envelope(cells, nsim=5, foreignclass="ppp", clipdata=TRUE)
  
  AA <- envelope(cells, nsim=5, jsim=5, alternative="greater", global=TRUE)
  AA <- envelope(cells, nsim=5, jsim=5, alternative="less", global=TRUE)
  AA <- envelope(cells, nsim=5, jsim=5, alternative="greater", VARIANCE=TRUE)
  AA <- envelope(cells, nsim=5, jsim=5, alternative="greater", VARIANCE=TRUE)

  fit <- ppm(cells ~ 1, Strauss(0.07))
  U <- envelope(fit, nsim=3, simulate=expression(runifpoint(20)))
  kfit <- kppm(redwood3 ~ x)
  UU <- envelope(kfit, nsim=7, simulate=expression(simulate(kfit, drop=TRUE)))
  VV <- envelope(kfit, nsim=7, weights=1:7)
  MM <- envelope(kfit, nsim=7, Kinhom, lambda=density(redwood3))
  #'  envelopes based on sample variance
  E <- envelope(cells, nsim=8, VARIANCE=TRUE)
  G <- envelope(cells, nsim=8, VARIANCE=TRUE,
                use.theory=FALSE, do.pwrong=TRUE)
  print(G)
  #' summary method
  summary(E)
  summary(envelope(cells, nsim=5, simulate=expression(runifpoint(42))))
  #' weights argument
  H1 <- envelope(cells, nsim=4, weights=npoints, savefuns=TRUE)
  H2 <- envelope(cells, nsim=4, weights=npoints, savefuns=TRUE)
  J1 <- envelope(cells, nsim=4, weights=npoints, VARIANCE=TRUE)
  J2 <- envelope(cells, nsim=4, weights=npoints, VARIANCE=TRUE)
  #' pooling with weights
  H <- pool(H1, H2)
  J <- pool(J1, J2)
  #' pooling envelopes with non-identical attributes
  H0 <- envelope(cells, nsim=4, savefuns=TRUE)
  HH <- pool(H0, H1)
  #' undocumented/secret
  K <- envelope(cells, nsim=4, saveresultof=npoints, collectrubbish=TRUE)
  #' so secret I've even forgotten how to do it
  M <- envelope(cells, nsim=4, internal=list(eject="patterns"))
})
}

if(FULLTEST) {
local({
  #' envelope computations in other functions
  P <- lurking(cells, expression(x), envelope=TRUE, nsim=9)
  print(P)
  #' re-using envelope objects in other functions
  A <- envelope(cells, nsim=9, savepatterns=TRUE, savefuns=TRUE)
  S <- lurking(cells, expression(x), envelope=A, nsim=9)
  #' envelope.envelope
  B <- envelope(cells, nsim=5, savepatterns=TRUE, savefuns=FALSE)
  envelope(B)
})
}

if(FULLTEST) {
local({
  X <- runiflpp(10, simplenet)
  Xr <- X %mark% runif(10)
  Xc <- X %mark% factor(letters[c(1:4,3,2,4:1)])
  X2 <- X %mark% data.frame(height=runif(10), width=runif(10))

  E  <- envelope(X,  linearK, nsim=9)
  Er <- envelope(Xr, linearK, nsim=9)
  Ec <- envelope(Xc, linearK, nsim=9)
  E2 <- envelope(X2, linearK, nsim=9)
  
  Erf <- envelope(Xr, linearK, nsim=9, fix.n=TRUE)
  E2f <- envelope(X2, linearK, nsim=9, fix.n=TRUE)

  Ecf <- envelope(Xc, linearK,      nsim=9, fix.n=TRUE)
  Ecm <- envelope(Xc, linearKcross, nsim=9, fix.n=TRUE, fix.marks=TRUE)

  fut <- lppm(Xc ~ marks)
  EEf <- envelope(fut, linearK,      fix.n=TRUE)
  EEm <- envelope(fut, linearKcross, fix.n=TRUE, fix.marks=TRUE)
})
}

if(ALWAYS) {
local({
  #' Test robustness of envelope() sorting procedure when NA's are present
  #' Fails with spatstat.utils 1.12-0
  set.seed(42)
  EP <- envelope(longleaf, pcf, nsim=10, nrank=2)

  #' Test case when the maximum permitted number of failures is exceeded
  X <- amacrine[1:153] # contains exactly one point with mark='off'
  #' High probability of generating a pattern with no marks = 'off'
  E <- envelope(X, Kcross, nsim=39, maxnerr=2, maxerr.action="warn")
  A <- alltypes(X, Kcross, envelope=TRUE, nsim=39, maxnerr=2)
})
}

if(ALWAYS) {
local({
  #' Internals: envelope.matrix
  Y <- matrix(rnorm(200), 10, 20)
  rr <- 1:10
  oo <- rnorm(10)
  zz <- numeric(10)
  E <- envelope(Y, rvals=rr, observed=oo, nsim=10)
  E <- envelope(Y, rvals=rr, observed=oo, jsim=1:10)
  E <- envelope(Y, rvals=rr, observed=oo, theory=zz,
                type="global", use.theory=TRUE)
  E <- envelope(Y, rvals=rr, observed=oo, theory=zz,
                type="global", use.theory=TRUE, nsim=10)
  E <- envelope(Y, rvals=rr, observed=oo, theory=zz,
                type="global", use.theory=FALSE, nsim=10)
  E <- envelope(Y, rvals=rr, observed=oo, type="global",
                nsim=10, nsim2=10)
  E <- envelope(Y, rvals=rr, observed=oo, type="global",
                jsim=1:10, jsim.mean=11:20)
  if(FULLTEST) print(E)
  E <- envelope(Y, rvals=rr, observed=oo, type="global",
                nsim=10, jsim.mean=11:20)
  E <- envelope(Y, rvals=rr, observed=oo, type="global",
                jsim=1:10, nsim2=10)
})
}

if(ALWAYS) {
local({
  #' quirk with handmade summary functions ('conserve' attribute)
  Kdif <- function(X, r=NULL) { # note no ellipsis
    Y <- split(X)
    K1 <- Kest(Y[[1]], r=r)
    K2 <- Kest(Y[[2]], r=r)
    D <- eval.fv(K1-K2)
    return(D)
  }
  envelope(amacrine, Kdif, nsim=3)
})
}
#'  tests/enveltest.R
#'     Envelope tests (dclf.test, mad.test)
#'     and two-stage tests (bits.test, dg.test, bits.envelope, dg.envelope)
#' 
#'     $Revision: 1.3 $  $Date: 2020/04/28 12:58:26 $ 
#'
if(FULLTEST) {
local({
  #' handling of NA function values (due to empty point patterns)
  set.seed(1234)
  X <- rThomas(5, 0.05, 10) 
  fit <- kppm(X ~ 1, "Thomas")
  set.seed(100000)
  dclf.test(fit)
  set.seed(909)
  dg.test(fit, nsim=9)
  #' other code blocks
  dclf.test(fit, rinterval=c(0, 3), nsim=9)
  envelopeTest(X, exponent=3, clamp=TRUE, nsim=9)
})
}
#
#    tests/factorbugs.R
#
# check for various bugs related to factor conversions
#
#    $Revision: 1.5 $  $Date: 2020/04/28 12:58:26 $
#

if(ALWAYS) {
local({
  ## make a factor image
  m <- factor(rep(letters[1:4], 4))
  Z <- im(m, xcol=1:4, yrow=1:4)
  ## make a point pattern
  set.seed(42)
  X <- runifpoint(20, win=as.owin(Z))
  ## look up the image at the points of X
  ## (a) internal
  ans1 <- lookup.im(Z, X$x, X$y)
  stopifnot(is.factor(ans1))
  ## (b) user level
  ans2 <- Z[X]
  stopifnot(is.factor(ans2))
  ## (c) turn the image into a tessellation
  ##  and apply quadratcount
  V <- tess(image = Z)
  quadratcount(X, tess=V)
  ## (d) pad image
  Y <- padimage(Z, factor("b", levels=levels(Z)))
  stopifnot(Y$type == "factor")
  U <- padimage(Z, "b")
  stopifnot(U$type == "factor")
  ## (e) manipulate levels
  Zb <- relevel(Z, "b")
  Zv <- mergeLevels(Z, vowel="a", consonant=c("b","c","d"))
  P <- X %mark% Z[X]
  Pv <- mergeLevels(P, vowel="a", consonant=c("b","c","d"))
})
}
#
#    tests/fastgeyer.R
#
# checks validity of fast C implementation of Geyer interaction
#
#    $Revision: 1.4 $  $Date: 2020/04/28 12:58:26 $
#
if(FULLTEST) {  # depends on hardware
local({
  X <- redwood
  Q <- quadscheme(X)
  U <- union.quad(Q)
  EP <- equalpairs.quad(Q)
  G <- Geyer(0.11, 2)
# The value r=0.11 is chosen to avoid hardware numerical effects (gcc bug 323).
# It avoids being close any value of pairdist(redwood).
# The nearest such values are 0.1077.. and 0.1131..
# By contrast if r = 0.1 there are values differing from 0.1 by 3e-17
  a <- pairsat.family$eval(X,U,EP,G$pot,G$par,"border")
  b <-          G$fasteval(X,U,EP,G$pot,G$par,"border")
  if(!all(a==b))
    stop("Results of Geyer()$fasteval and pairsat.family$eval do not match")
# ...
# and again for a non-integer value of 'sat'
# (spotted by Thordis Linda Thorarinsdottir)  
  G <- Geyer(0.11, 2.5)
  a <- pairsat.family$eval(X,U,EP,G$pot,G$par,"border")
  b <-          G$fasteval(X,U,EP,G$pot,G$par,"border")
  if(!all(a==b))
    stop("Results of Geyer()$fasteval and pairsat.family$eval do not match when sat is not an integer")
# and again for sat < 1
# (spotted by Rolf)  
  G <- Geyer(0.11, 0.5)
  a <- pairsat.family$eval(X,U,EP,G$pot,G$par,"border")
  b <-          G$fasteval(X,U,EP,G$pot,G$par,"border")
  if(!all(a==b))
    stop("Results of Geyer()$fasteval and pairsat.family$eval do not match when sat < 1")
})
}

#
#  tests/fastK.R
#
# check fast and slow code for Kest
#       and options not tested elsewhere
#
#   $Revision: 1.5 $   $Date: 2020/04/28 12:58:26 $
#
if(ALWAYS) {
local({
  ## fast code
  Kb <- Kest(cells, nlarge=0)
  Ku <- Kest(cells, correction="none")
  Kbu <- Kest(cells, correction=c("none", "border"))
  ## slow code, full set of corrections, sqrt transformation, ratios
  Ldd <- Lest(unmark(demopat), correction="all", var.approx=TRUE, ratio=TRUE)
  ## Lotwick-Silverman var approx (rectangular window)
  Loo <- Lest(cells, correction="all", var.approx=TRUE, ratio=TRUE)
  ## Code for large dataset
  nbig <- .Machine$integer.max
  if(!is.null(nbig)) {
    nn <- ceiling(sqrt(nbig))
    if(nn < 1e6) Kbig <- Kest(runifpoint(nn),
                              correction=c("border", "bord.modif", "none"),
                              ratio=TRUE)
  }
  
  ## Kinhom
  lam <- density(cells, at="points", leaveoneout=TRUE)
  ## fast code
  Kib <- Kinhom(cells, lam, nlarge=0)
  Kiu <- Kest(cells, lam, correction="none")
  Kibu <- Kest(cells, lam, correction=c("none", "border"))
  ## slow code
  Lidd <- Linhom(unmark(demopat), sigma=bw.scott)
})

}
#' tests/formuli.R
#'
#'  Test machinery for manipulating formulae
#' 
#' $Revision: 1.7 $  $Date: 2020/04/28 12:58:26 $

local({

  ff <- function(A, deletevar, B) {
    D <- reduceformula(A, deletevar)
    if(!spatstat.utils::identical.formulae(D, B)) {
      AD <- as.expression(substitute(reduceformula(A,d),
                                     list(A=A, d=deletevar)))
      stop(paste(AD, "\n\tyields ", spatstat.utils::pasteFormula(D),
                 " instead of ", spatstat.utils::pasteFormula(B)),
           call.=FALSE)
    }
    invisible(NULL)
  }

  ff(~ x + z, "x", ~z)

  ff(y ~ x + z, "x", y~z)

  ff(~ I(x^2) + z, "x",  ~z)

  ff(y ~ poly(x,2) + poly(z,3), "x", y ~poly(z,3))

  ff(y ~ x + z, "g", y ~ x + z)

  reduceformula(y ~ x+z, "g", verbose=TRUE)
  reduceformula(y ~ sin(x-z), "z", verbose=TRUE)
  
  illegal.iformula(~str*g, itags="str", dfvarnames=c("marks", "g", "x", "y"))
})



#
#  tests/func.R
#
#   $Revision: 1.5 $   $Date: 2020/04/28 12:58:26 $
#
#  Tests of 'funxy' infrastructure etc

if(FULLTEST) {
local({
  ## Check the peculiar function-building code in funxy
  W <- square(1)
  f1a <- function(x, y) sqrt(x^2 + y^2)
  f1b <- function(x, y) { sqrt(x^2 + y^2) }
  f2a <- function(x, y) sin(x)
  f2b <- function(x, y) { sin(x) } 
  f3a <- function(x, y) sin(x) + cos(x) 
  f3b <- function(x, y) { sin(x) + cos(x) } 
  f4a <- function(x, y) { z <- x + y ; z }
  f4b <- function(x, y) { x + y } 
  F1a <- funxy(f1a, W)
  F1b <- funxy(f1b, W)
  F2a <- funxy(f2a, W)
  F2b <- funxy(f2b, W)
  F3a <- funxy(f3a, W)
  F3b <- funxy(f3b, W)
  F4a <- funxy(f4a, W)
  F4b <- funxy(f4b, W)
  stopifnot(identical(F1a(cells), F1b(cells)))
  stopifnot(identical(F2a(cells), F2b(cells)))
  stopifnot(identical(F3a(cells), F3b(cells)))
  stopifnot(identical(F4a(cells), F4b(cells)))
  ## check coordinate extraction from objects
  X <- runifpoint(9)
  Y <- runiflpp(5, simplenet)
  Q <- quadscheme(X)
  a <- F1a(X)
  b <- F1a(Y)
  d <- F1a(Q)
})
}
##  
##     tests/funnymarks.R
##
## tests involving strange mark values
## $Revision: 1.7 $ $Date: 2020/04/28 12:58:26 $

if(ALWAYS) { # depends on locale
local({
  ## ppm() where mark levels contain illegal characters
  hyphenated <- c("a", "not-a")
  spaced <- c("U", "non U")
  suffixed <- c("a+", "a*")
  charred <- c("+", "*")

  irad <- matrix(0.1, 2,2)
  hrad <- matrix(0.005, 2, 2)

  tryit <- function(types, X, irad, hrad) { 
    levels(marks(X)) <- types
    fit <- ppm(X ~marks + polynom(x,y,2),
               MultiStraussHard(types=types,iradii=irad,hradii=hrad))
    print(fit)
    print(coef(fit))
    val <- fitted(fit)
    pred <- predict(fit)
    return(invisible(NULL))
  }

  tryit(hyphenated, amacrine, irad, hrad)
  tryit(spaced, amacrine, irad, hrad)
  tryit(suffixed, amacrine, irad, hrad)
  tryit(charred, amacrine, irad, hrad)

  ## marks which are dates
  X <- cells
  n <- npoints(X)
  endoftime <- rep(ISOdate(2001,1,1), n)
  eotDate   <- rep(as.Date("2001-01-01"), n)
  markformat(endoftime)
  markformat(eotDate)
  marks(X) <- endoftime
  print(X)
  Y <- X %mark% data.frame(id=1:42, date=endoftime, dd=eotDate)
  print(Y)
  md <- markformat(endoftime)
  
  ## mark formats
  Z <- Y
  marks(Z) <- marks(Z)[1,,drop=FALSE]
  ms <- markformat(solist(cells, redwood))
  marks(Z) <- factor(1:npoints(Z))
  marks(Z)[12] <- NA
  mz <- is.multitype(Z)
  cZ <- coerce.marks.numeric(Z)
  marks(Z) <- data.frame(n=1:npoints(Z),
                         a=factor(sample(letters, npoints(Z), replace=TRUE)))
  cZ <- coerce.marks.numeric(Z)
  stopifnot(is.multitype(cells %mark% data.frame(a=factor(1:npoints(cells)))))

  a <- numeric.columns(finpines)
  b1 <- numeric.columns(amacrine)
  b2 <- coerce.marks.numeric(amacrine)
  d <- numeric.columns(cells)
  f <- numeric.columns(longleaf)
  ff <- data.frame(a=factor(letters[1:10]), y=factor(sample(letters, 10)))
  numeric.columns(ff)

  ## mark operations
  df <- data.frame(x=1:2, y=sample(letters, 2))
  h <- hyperframe(z=1:2, p=solist(cells, cells))
  a <- NULL %mrep% 3
  a <- 1:4 %mrep% 3
  a <- df %mrep% 3
  a <- h %mrep% 3
  b <- markcbind(df, h)
  b <- markcbind(h, df)
})
}
## 
##    tests/fvproblems.R
##
##    problems with fv, ratfv and fasp code
##
##    $Revision: 1.15 $  $Date: 2020/04/28 12:58:26 $

#' This appears in the workshop notes
#' Problem detected by Martin Bratschi

if(FULLTEST) {
local({
  Jdif <- function(X, ..., i) {
    Jidot <- Jdot(X, ..., i=i)
    J <- Jest(X, ...)
    dif <- eval.fv(Jidot - J)
    return(dif)
  }
  Z <- Jdif(amacrine, i="on")
})
}
#'
#'  Test mathlegend code
#'
local({
  K <- Kest(cells)
  if(FULLTEST) {
    plot(K)
    plot(K, . ~ r)
    plot(K, . - theo ~ r)
  }
  if(ALWAYS) {
    plot(K, sqrt(./pi)  ~ r)
  }
  if(FULLTEST) {
    plot(K, cbind(iso, theo) ~ r)
    plot(K, cbind(iso, theo) - theo ~ r)
    plot(K, sqrt(cbind(iso, theo)/pi)  ~ r)
    plot(K, cbind(iso/2, -theo) ~ r)
    plot(K, cbind(iso/2, trans/2) - theo ~ r)
  }
  if(FULLTEST) {
    ## test expansion of .x and .y
    plot(K, . ~ .x)
    plot(K, . - theo ~ .x)
    plot(K, .y - theo ~ .x)
  }
  if(ALWAYS) {
    plot(K, sqrt(.y) - sqrt(theo) ~ .x)
  }

  # problems with parsing weird strings in levels(marks(X))
  # noted by Ulf Mehlig
  if(ALWAYS) {
    levels(marks(amacrine)) <- c("Nasticreechia krorluppia", "Homo habilis")
    plot(Kcross(amacrine))
    plot(alltypes(amacrine, "K"))
  }
  if(FULLTEST) {
    plot(alltypes(amacrine, "J"))
    plot(alltypes(amacrine, pcfcross))
  }
})

#'
#'  Test quirks related to 'alim' attribute

if(FULLTEST) {
local({
  K <- Kest(cells)
  attr(K, "alim") <- NULL
  plot(K)
  attr(K, "alim") <- c(0, 0.1)
  plot(tail(K))
})
}

#'
#' Check that default 'r' vector passes the test for fine spacing

if(ALWAYS) {
local({
  a <- Fest(cells)
  A <- Fest(cells, r=a$r)
  b <- Hest(heather$coarse)
  B <- Hest(heather$coarse, r=b$r)
  # from Cenk Icos
  X <- runifpoint(100, owin(c(0,3), c(0,10)))
  FX <- Fest(X)
  FXr <- Fest(X, r=FX$r)
  JX <- Jest(X)
})
}

##' various functionality in fv.R

if(ALWAYS) {
local({
  M <- cbind(1:20, matrix(runif(100), 20, 5))
  A <- as.fv(M)
  fvlabels(A) <- c("r","%s(r)", "%s[A](r)", "%s[B](r)", "%s[C](r)", "%s[D](r)")
  A <- rename.fv(A, "M", quote(M(r)))
  A <- tweak.fv.entry(A, "V1", new.tag="r")
  A[,3] <- NULL
  A$hogwash <- runif(nrow(A))
  fvnames(A, ".") <- NULL
  #' bind.fv with qualitatively different functions
  GK <- harmonise(G=Gest(cells), K=Kest(cells))
  G <- GK$G
  K <- GK$K
  ss <- c(rep(TRUE, nrow(K)-10), rep(FALSE, 10))
  U <- bind.fv(G, K[ss, ], clip=TRUE)
  #'
  H <- rebadge.as.crossfun(K, "H", "inhom", 1, 2)
  H <- rebadge.as.dotfun(K, "H", "inhom", 3)
  #' text layout
  op <- options(width=27)
  print(K)
  options(width=18)
  print(K)
  options(op)
  #' collapse.fv
  Kb <- Kest(cells, correction="border")
  Ki <- Kest(cells, correction="isotropic")
  collapse.fv(Kb, Ki, same="theo")
  collapse.fv(anylist(B=Kb, I=Ki), same="theo")
  collapse.fv(anylist(B=Kb), I=Ki, same="theo")
  Xlist <- replicate(3, runifpoint(30), simplify=FALSE)
  Klist <- anylapply(Xlist, Kest) 
  collapse.fv(Klist, same="theo", different=c("iso", "border"))
  names(Klist) <- LETTERS[24:26]
  collapse.fv(Klist, same="theo", different=c("iso", "border"))
})
}

if(FULLTEST) {
local({
  ## rat
  K <- Kest(cells, ratio=TRUE)
  G <- Gest(cells, ratio=TRUE)
  print(K)
  compatible(K, K)
  compatible(K, G)
  H <- rat(K, attr(K, "numerator"), attr(K, "denominator"), check=TRUE)
})
}

if(FULLTEST) {
local({
  ## bug in Jmulti.R colliding with breakpts.R
  B <- owin(c(0,3), c(0,10))
  Y <- superimpose(A=runifpoint(1212, B), B=runifpoint(496, B))
  JDX <- Jdot(Y)
  JCX <- Jcross(Y)
  Jdif <- function(X, ..., i) {
    Jidot <- Jdot(X, ..., i=i)
    J <- Jest(X, ...)
    dif <- eval.fv(Jidot - J)
    return(dif)
  }
  E <- envelope(Y, Jdif, nsim=19, i="A", simulate=expression(rlabel(Y)))
})
}

if(FULLTEST) {
local({
  #' fasp axes, title, dimnames
  a <- alltypes(amacrine)
  a$title <- NULL
  plot(a, samex=TRUE, samey=TRUE)
  dimnames(a) <- lapply(dimnames(a), toupper)

  b <- as.fv(a)
})
}

if(FULLTEST) {
local({
  ## plot.anylist (fv)
  b <- anylist(A=Kcross(amacrine), B=Kest(amacrine))
  plot(b, equal.scales=TRUE, main=expression(sqrt(pi)))
  plot(b, arrange=FALSE)
})
}

#
# quadratresample.R
#
# resample a point pattern by resampling quadrats
#
# $Revision: 1.7 $  $Date: 2015/10/21 09:06:57 $
#

quadratresample <- function(X, nx, ny=nx, ...,
                            replace=FALSE, nsamples=1,
                            verbose=(nsamples > 1)) {
  stopifnot(is.ppp(X))
  if(X$window$type != "rectangle")
    stop("Resampling is only implemented for rectangular windows")
  # create tessellation
  A <- quadrats(X, nx=nx, ny=ny)
  # split data over tessellation
  B <- split(X, A)
  nq <- length(B)
  # determine bottom left corner of each tile
  V <- lapply(B, framebottomleft)
  out <- list()
  if(verbose) {
    cat("Generating resampled patterns...")
    pstate <- list()
  }
  for(i in 1:nsamples) {
    # resample tiles
    ind <- sample(1:nq, nq, replace=replace)
    Xresampled <- X
    Bresampled <- B
    for(j in 1:nq) {
      k <- ind[j]
      Bresampled[[j]] <- shift(B[[k]], unlist(V[[j]]) - unlist(V[[k]]))
    }
    split(Xresampled, A) <- Bresampled
    out[[i]] <- Xresampled
    if(verbose)
      pstate <- progressreport(i, nsamples, state=pstate)
  }
  if(nsamples == 1)
    return(out[[1]])
  return(as.solist(out))
}

#
#   rLGCP.R
#
#   simulation of log-Gaussian Cox process
#
#   original code by Abdollah Jalilian
#
#  $Revision: 1.21 $    $Date: 2019/07/09 09:52:18 $
#

rLGCP <- local({

  rLGCP <- function(model="exp", mu = 0, param = NULL, ...,
                    win=NULL, saveLambda=TRUE, nsim=1, drop=TRUE) {
    ## validate
    if (is.numeric(mu)) {
      check.1.real(mu, paste("if", sQuote("mu"), "is numeric,"))
    } else if(!is.function(mu) && !is.im(mu))
      stop(paste(sQuote("mu"), "must be a constant, a function or an image"))
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
    ## check for outdated usage
    if(!all(nzchar(names(param))))
      stop("Outdated syntax of argument 'param' to rLGCP", call.=FALSE)
    ## 
    do.rLGCP(model=model, mu=mu, param=param, ...,
             win=win, saveLambda=saveLambda, nsim=nsim, drop=drop)
  }

  do.rLGCP <- function(model="exp", mu = 0, param = NULL, ...,
                       win=NULL, saveLambda=TRUE,
                       eps = NULL, dimyx = NULL, xy = NULL,
                       modelonly=FALSE, nsim=1, drop=TRUE) {
    ## make RF model object from RandomFields package
    ## get the 'model generator'
    modgen <- getRandomFieldsModelGen(model)
    ## now create a RandomFields 'model' object
    rfmodel <- do.call(modgen, append(as.list(param), list(...)))
    if(!inherits(rfmodel, "RMmodel"))
      stop("Unable to create RandomFields model object", call.=FALSE)

    ## secret exit
    if(modelonly)
      return(rfmodel)

    ## simulation window
    win.given <- !is.null(win)
    mu.image <- is.im(mu)
    win <- if(win.given) as.owin(win) else if(mu.image) as.owin(mu) else owin()
  
    if(win.given && mu.image && !is.subset.owin(win, as.owin(mu)))
      stop(paste("The spatial domain of the pixel image", sQuote("mu"),
                 "does not cover the simulation window", sQuote("win")))

    ## convert win to a mask
    w <- as.mask(w=win, eps=eps, dimyx=dimyx, xy=xy)
    xcol <- w$xcol
    yrow <- w$yrow
    dimw <- w$dim

    ## evaluate 'mu' at pixels of mask
    if(is.numeric(mu)) {
      muxy <- mu
    } else {
      xy <- rasterxy.mask(w, drop=FALSE)
      xx <- xy$x
      yy <- xy$y
      muxy <- if (is.function(mu)) mu(xx,yy) else
              lookup.im(mu, xx, yy, naok=TRUE, strict=TRUE)
      muxy[is.na(muxy)] <- -Inf
    }
    ## corresponding image template
    Lambda <- as.im(w)

    ## generate 'nsim' realisations of a zero-mean Gaussian random field Z
    spc <- RandomFields::RFoptions()$general$spConform
    if(spc) RandomFields::RFoptions(spConform=FALSE)
    z <- RandomFields::RFsimulate(rfmodel, xcol, yrow, grid = TRUE, n=nsim)
    if(spc) RandomFields::RFoptions(spConform=TRUE)
    ## ensure 3D array
    if(length(dim(z)) == 2) z <- array(z, dim=c(dim(z), 1))

    ## generate realisations of LGCP
    result <- vector(mode="list", length=nsim)
    for(i in 1:nsim) {
      ## Extract i-th realisation of Z; convert to log-Gaussian image
      Lambda$v[] <- exp(muxy + z[,,i])
      ## generate Poisson points
      X <- rpoispp(Lambda)[win]
      ## 
      if(saveLambda)
        attr(X, "Lambda") <- Lambda
      result[[i]] <- X
    }
    if(drop && nsim == 1)
      return(result[[1]])
    names(result) <- paste("Simulation", 1:nsim)
    return(as.solist(result))
  }

  rLGCP
})

#
#   rshift.R
#
#   random shift with optional toroidal boundary
#
#   $Revision: 1.19 $   $Date: 2020/04/29 13:20:21 $
#
#
rshift <- function(X, ...) {
  UseMethod("rshift")
}

rshift.splitppp <- function(X, ..., which=seq_along(X),
                            nsim=1, drop=TRUE)
{
  verifyclass(X, "splitppp")
  check.1.integer(nsim)
  
  if("group" %in% names(list(...)))
    stop(paste("argument", sQuote("group"),
               "not implemented for splitppp objects"))

  if(is.null(which)) {
    iwhich <- which <- seq_along(X)
  } else {
    id <- seq_along(X)
    names(id) <- names(X)
    iwhich <- id[which]
    if(length(iwhich) == 0)
      stop(paste("Argument", sQuote("which"), "did not match any marks"))
  }
  
  # validate arguments and determine common clipping window
  arglist <- handle.rshift.args(X[[1]]$window, ..., edgedefault="torus")

  if(!is.null(clip <- arglist$clip)) {
    # clip the patterns that are not to be shifted
    if(length(iwhich) < length(X)) 
      X[-iwhich] <- lapply(X[-iwhich], "[.ppp", i=clip)
  }
  Xvariable <- X[iwhich]

  resultlist <- vector(mode="list", length=nsim)

  for(isim in seq_len(nsim)) {
    Xsim <- X
    ## perform shift on selected patterns
    ## (setting group = NULL ensures each pattern is not split further)
    shiftXsub <- do.call(lapply, append(list(Xvariable, rshift.ppp, group=NULL),
                                        arglist))
    ## put back
    Xsim[iwhich] <- shiftXsub
    resultlist[[isim]] <- Xsim
  }

  return(simulationresult(resultlist, nsim, drop))
}

rshift.ppp <- function(X, ..., which=NULL, group, nsim=1, drop=TRUE)
{
  verifyclass(X, "ppp")
  check.1.integer(nsim)
  
  # validate arguments and determine common clipping window
  arglist <- handle.rshift.args(X$window, ..., edgedefault="torus")

  # default grouping
  #   (NULL is not the default)
  #   (NULL means all points shifted in parallel)
  if(missing(group))
    group <- if(is.multitype(X)) marks(X) else NULL

  # if no grouping, use of `which' is undefined
  if(is.null(group) && !is.null(which))
    stop(paste("Cannot apply argument", sQuote("which"),
               "; no grouping defined"))

  resultlist <- vector(mode="list", length=nsim)
  
  # if grouping, use split
  if(!is.null(group)) {
    Y <- split(X, group)
    splitshifts <- do.call(rshift.splitppp,
                           append(list(Y, which=which, nsim=nsim, drop=FALSE),
                                  arglist))
    for(isim in seq_len(nsim)) {
      Xsim <- X
      split(Xsim, group) <- splitshifts[[isim]]
      resultlist[[isim]] <- Xsim
    }
    return(simulationresult(resultlist, nsim, drop))
  } 
    
  # ungrouped point pattern
  # shift all points in parallel

  # recover arguments
  radius <- arglist$radius
  width  <- arglist$width
  height <- arglist$height
  edge   <- arglist$edge
  clip   <- arglist$clip
 
  W <- rescue.rectangle(Window(X))

  if(edge == "torus") {
    if(!is.rectangle(W))
      stop("edge = 'torus' is only meaningful for rectangular windows")
    xr <- W$xrange
    yr <- W$yrange
    Wide <- diff(xr)
    High <- diff(yr)
  }

  ## .......... simulation loop ..................
  resultlist <- vector(mode="list", length=nsim)
  for(isim in seq_len(nsim)) {
    #' generate random translation vector
    if(!is.null(radius)) {
      jump <- runifdisc(1, radius=radius)
    } else {
      jump <- list(x=runif(1, min=0, max=width),
                   y=runif(1, min=0, max=height))
    }
    #' translate points of X
    x <- X$x + jump$x
    y <- X$y + jump$y
    #' wrap points
    if(edge == "torus") {
      x <- xr[1] + (x - xr[1]) %% Wide
      y <- yr[1] + (y - yr[1]) %% High
    }
    #' save as point pattern
    Xsim <- X
    Xsim$x <- x
    Xsim$y <- y
    #' clip to window
    if(!is.null(clip))
      Xsim <- Xsim[clip]
    #' save result
    resultlist[[isim]] <- Xsim
  }
  ## ................ end loop ..................
  return(simulationresult(resultlist, nsim, drop))
}


handle.rshift.args <- function(W, ...,
                               radius=NULL, width=NULL, height=NULL,
                               edge=NULL, clip=NULL, edgedefault)
{
  verifyclass(W, "owin")
  W <- rescue.rectangle(W)
  
  if(length(aargh <- list(...)) > 0)
    stop(paste("Unrecognised arguments:",
               paste(names(aargh), collapse=",")))
  
  if(!is.null(radius)) {
    # radial generator
    if(!(is.null(width) && is.null(height)))
    stop(paste(sQuote("radius"), "is incompatible with",
               sQuote("width"), "and", sQuote("height")))
  } else {
    # rectangular generator
    if(is.null(width) != is.null(height))
      stop("Must specify both width and height, if one is specified")
    if(is.null(width)) width <- diff(W$xrange)
    if(is.null(height)) height <- diff(W$yrange)
  }
  
  if(is.null(edge))
    edge <- edgedefault
  else if(!(edge %in% c("torus", "erode", "none")))
    stop(paste("Unrecognised option erode=", sQuote(edge)))

  # determine whether clipping window is needed
  if(is.null(clip))
    clip <- switch(edge,
                   torus= NULL,
                   none= W,
                   erode={
                     if(!is.null(radius))
                       erosion.owin(W, radius)
                     else if(W$type == "rectangle")
                       trim.rectangle(W, width, height)
                     else
                       erosion.owin(W, max(width, height))
                   })

  return(list(radius=radius, width=width, height=height,
              edge=edge, clip=clip))
}

# rtoro <- function(X, which=NULL, radius=NULL, width=NULL, height=NULL) {
#  .Deprecated("rshift", package="spatstat")
#  rshift(X, which=which, radius=radius, width=width, height=height)
# }

#'
#'   unstack.R
#'
#'   Methods for generic 'unstack'
#' 
#'   $Revision: 1.5 $  $Date: 2019/09/12 04:05:58 $

unstack.ppp <- unstack.psp <- unstack.lpp <- unstack.tess <- function(x, ...) {
  trap.extra.arguments(...)
  marx <- marks(x)
  d <- dim(marx)
  if(is.null(d)) return(solist(x))
  y <- rep(list(unmark(x)), d[2])
  for(j in seq_along(y))
    marks(y[[j]]) <- marx[,j,drop=FALSE]
  names(y) <- colnames(marx)
  return(as.solist(y))
}


unstack.msr <- function(x, ...) {
  trap.extra.arguments(...)
  d <- dim(x)
  if(is.null(d)) return(solist(x))
  smo <- attr(x, "smoothdensity")
  if(!inherits(smo, "imlist")) smo <- NULL
  nc <- d[2]
  y <- vector(mode="list", length=nc)
  for(j in seq_len(nc)) {
    xj <- x[,j,drop=FALSE]
    if(!is.null(smo)) attr(xj, "smoothdensity") <- smo[[j]]
    y[[j]] <- xj
  }
  names(y) <- colnames(x)
  return(as.solist(y))
}

unstackFilter <- function(x) {
  ## deal with a whole swag of classes that do not need to be unstacked
  nonvectorclasses <- c("im", "owin", "quad", 
                        "quadratcount", "quadrattest", 
                        "funxy", "distfun", "nnfun", 
                        "linnet", "linfun", 
                        "influence.ppm", "leverage.ppm")
  y <- if(inherits(x, nonvectorclasses)) solist(x) else unstack(x)
  return(y)
}

unstack.solist <- function(x, ...) {
  trap.extra.arguments(...)
  y <- lapply(x, unstackFilter)
  z <- as.solist(unlist(y, recursive=FALSE))
  return(z)
}

unstack.layered <- function(x, ...) {
  trap.extra.arguments(...)
  y <- lapply(x, unstackFilter)
  ny <- lengths(y)
  nx <- length(ny)
  if(all(ny == 1) || nx == 0) return(solist(x))
  pax <- layerplotargs(x)
  pay <- rep(pax, times=ny)
  z <- unlist(y, recursive=FALSE)
  z <- layered(LayerList=z, plotargs=pay)
  return(z)
}


  




  


  

#
#	ripras.S	Ripley-Rasson estimator of domain
#
#
#	$Revision: 1.14 $	$Date: 2014/10/24 00:22:30 $
#
#
#
#
#-------------------------------------
bounding.box.xy <- function(x, y=NULL) {
  xy <- xy.coords(x,y)
  if(length(xy$x) == 0)
    return(NULL)
  owin(range(xy$x), range(xy$y), check=FALSE)
}

convexhull.xy <- function(x, y=NULL) {
  xy <- xy.coords(x, y)
  x <- xy$x
  y <- xy$y
  if(length(x) < 3)
    return(NULL)
  h <- rev(chull(x, y))  # must be anticlockwise
  if(length(h) < 3)
    return(NULL)
  w <- owin(poly=list(x=x[h], y=y[h]), check=FALSE)
  return(w)
}

ripras <- function(x, y=NULL, shape="convex", f) {
  xy <- xy.coords(x, y)
  n <- length(xy$x)
  w <- switch(shape,
              convex = convexhull.xy(xy),
              rectangle = boundingbox(xy),
              stop(paste("Unrecognised option: shape=", dQuote(shape))))
  if(is.null(w))
    return(NULL)
  # expansion factor
  if(!missing(f))
    stopifnot(is.numeric(f) && length(f) == 1 && f >= 1)
  else switch(shape,
              convex = {
                # number of vertices
                m <- summary(w)$nvertices
                f <- if(m < n) 1/sqrt(1 - m/n) else 2
              },
              rectangle = {
                f <- (n+1)/(n-1)
              })
  # centroid
  ce <- unlist(centroid.owin(w))
  # shift centroid to origin
  W <- shift(w, -ce)
  # rescale
  W <- affine(W, mat=diag(c(f,f)))
  # shift origin to centroid
  W <- shift(W, ce)
  return(W)
}

#
#	rotate.S
#
#	$Revision: 1.21 $	$Date: 2014/10/24 00:22:30 $
#

rotxy <- function(X, angle=pi/2) {
  co <- cos(angle)
  si <- sin(angle)
  list(x = co * X$x - si * X$y,
       y = si * X$x + co * X$y)
}

rotxypolygon <- function(p, angle=pi/2) {
  p[c("x","y")] <- rotxy(p, angle=angle)
  # area and hole status are invariant under rotation
  return(p)
}

rotate <- function(X, ...) {
  UseMethod("rotate")
}

rotate.owin <- function(X, angle=pi/2, ..., rescue=TRUE, centre=NULL) {
  verifyclass(X, "owin")
  if(!is.null(centre)) {
    ## rotation about designated centre
    X <- shift(X, origin=centre)
    negorig <- getlastshift(X)
  } else negorig <- NULL
  switch(X$type,
         rectangle={
           # convert rectangle to polygon
           P <- owin(X$xrange, X$yrange, poly=
                     list(x=X$xrange[c(1,2,2,1)],
                          y=X$yrange[c(1,1,2,2)]),
                     unitname=unitname(X))
           # call polygonal case
           Y <- rotate.owin(P, angle, rescue=rescue)
         },
         polygonal={
           # First rotate the polygonal boundaries
           bdry <- lapply(X$bdry, rotxypolygon, angle=angle)
           # wrap up
           Y <- owin(poly=bdry, check=FALSE, unitname=unitname(X))
           if(rescue)
             Y <- rescue.rectangle(Y)
         },
         mask={
           newframe <- boundingbox(rotxy(corners(X), angle))
           Y <- if(length(list(...)) > 0) as.mask(newframe, ...) else 
                   as.mask(newframe, eps=with(X, min(xstep, ystep)))
           pixelxy <- rasterxy.mask(Y)
           xybefore <- rotxy(pixelxy, -angle)
           Y$m[] <- with(xybefore, inside.owin(x, y, X))
           Y <- intersect.owin(Y, boundingbox(Y))
           if(rescue)
             Y <- rescue.rectangle(Y)
           unitname(Y) <- unitname(X)
         },
         stop("Unrecognised window type")
         )
  if(!is.null(negorig))
    Y <- shift(Y, -negorig)
  return(Y)
}

rotate.ppp <- function(X, angle=pi/2, ..., centre=NULL) {
  verifyclass(X, "ppp")
  if(!is.null(centre)) {
    X <- shift(X, origin=centre)
    negorigin <- getlastshift(X)
  } else negorigin <- NULL
  r <- rotxy(X, angle)
  w <- rotate.owin(X$window, angle, ...)
  Y <- ppp(r$x, r$y, window=w, marks=marks(X, dfok=TRUE), check=FALSE)
  if(!is.null(negorigin))
    Y <- shift(Y, -negorigin)
  return(Y)
}

rotate.im <- function(X, angle=pi/2, ..., centre=NULL) {
  if(!is.null(centre)) {
    X <- shift(X, origin=centre)
    negorigin <- getlastshift(X)
  } else negorigin <- NULL
  co <- cos(angle)
  si <- sin(angle)
  m <- matrix(c(co,si,-si,co), nrow=2, ncol=2)
  Y <- affine(X, mat=m)
  if(!is.null(negorigin))
    Y <- shift(Y, -negorigin)
  return(Y)
}

#
#	distbdry.S		Distance to boundary
#
#	$Revision: 4.45 $	$Date: 2019/02/06 10:53:48 $
#
# -------- functions ----------------------------------------
#
#	bdist.points()
#                       compute vector of distances 
#			from each point of point pattern
#                       to boundary of window
#
#       bdist.pixels()
#                       compute matrix of distances from each pixel
#                       to boundary of window
#
#       erodemask()    erode the window mask by a distance r
#                       [yields a new window]
#
#
# 
"bdist.points"<-
function(X)
{
	verifyclass(X, "ppp") 
        if(X$n == 0)
          return(numeric(0))
	x <- X$x
	y <- X$y
	window <- X$window
        switch(window$type,
               rectangle = {
		xmin <- min(window$xrange)
		xmax <- max(window$xrange)
		ymin <- min(window$yrange)
		ymax <- max(window$yrange)
		result <- pmin.int(x - xmin, xmax - x, y - ymin, ymax - y)
               },
               polygonal = {
                 xy <- cbind(x,y)
                 ll <- edges(window)$ends
                 result <- distppllmin(xy, ll)$min.d
               },
               mask = {
                 b <- bdist.pixels(window, style="matrix")
                 loc <- nearest.raster.point(x,y,window)
                 result <- b[cbind(loc$row, loc$col)]
               },
               stop("Unrecognised window type", window$type)
               )
        return(result)
}

"bdist.pixels" <- function(w, ..., style="image",
                           method=c("C", "interpreted")) {
	verifyclass(w, "owin")

        masque <- as.mask(w, ...)
        
        switch(w$type,
               mask = {
                 neg <- complement.owin(masque)
                 m <- exactPdt(neg)
                 b <- pmin.int(m$d,m$b)
               },
               rectangle = {
                 rxy <- rasterxy.mask(masque)
                 x <- rxy$x
                 y <- rxy$y
                 xmin <- w$xrange[1L]
                 xmax <- w$xrange[2L]
                 ymin <- w$yrange[1L]
                 ymax <- w$yrange[2L]
                 b <- pmin.int(x - xmin, xmax - x, y - ymin, ymax - y)
               },
               polygonal = {
                 # set up pixel raster
                 method <- match.arg(method)
                 rxy <- rasterxy.mask(masque)
                 x <- rxy$x
                 y <- rxy$y
                 b <- numeric(length(x))
                 # test each pixel in/out, analytically
                 inside <- inside.owin(x, y, w)
                 # compute distances for these pixels
                 xy <- cbind(x[inside], y[inside])
                 switch(method,
                        C = {
                          #' C code
                          ll <- as.data.frame(edges(w))
                          dxy <- distppllmin(xy, ll)$min.d
                        },
                        interpreted = {
                          #' ancient R code
                          dxy <- rep.int(Inf, sum(inside))
                          bdry <- w$bdry
                          for(i in seq_along(bdry)) {
                            polly <- bdry[[i]]
                            nsegs <- length(polly$x)
                            for(j in 1:nsegs) {
                              j1 <- if(j < nsegs) j + 1L else 1L
                              seg <- c(polly$x[j],  polly$y[j],
                                       polly$x[j1], polly$y[j1])
                              dxy <- pmin.int(dxy, distppl(xy, seg))
                            }
                          }
                        })
                 b[inside] <- dxy
               },
               stop("unrecognised window type", w$type)
               )

        # reshape it
        b <- matrix(b, nrow=masque$dim[1L], ncol=masque$dim[2L])

        switch(style,
               coords={
                 # format which can be plotted by image(), persp() etc
                 return(list(x=masque$xcol, y=masque$yrow, z=t(b)))
               },
               matrix={
                 # return matrix (for internal use by package)
                 return(b)
               },
               image={
                 bim <- im(b, xcol=masque$xcol, yrow=masque$yrow,
                           unitname=unitname(masque))
                 return(bim)
               },
               stop(paste("Unrecognised option for style:", style)))
} 

erodemask <- function(w, r, strict=FALSE) {
  # erode a binary image mask without changing any other entries
  verifyclass(w, "owin")
  if(w$type != "mask")
    stop(paste("window w is not of type", sQuote("mask")))
  if(!is.numeric(r) || length(r) != 1L)
    stop("r must be a single number")
  if(r < 0)
    stop("r must be nonnegative")
        
  bb <- bdist.pixels(w, style="matrix")

  if(r > max(bb))
    warning("eroded mask is empty")

  if(identical(strict, TRUE))
    w$m <- (bb > r)
  else 
    w$m <- (bb >= r)
  return(w)
}

"Frame<-.owin" <- function(X, value) {
  stopifnot(is.rectangle(value))
  W <- Frame(X)
  if(!is.subset.owin(W, value))
    W <- intersect.owin(W, value)
  rebound.owin(X, value)
}

rebound.owin <- local({

  rebound.owin <- function(x, rect) {
    w <- x
    verifyclass(rect, "owin")
    if(is.empty(w))
      return(emptywindow(rect))
    verifyclass(w, "owin")
    if(!is.subset.owin(as.rectangle(w), rect)) {
      bb <- boundingbox(w)
      if(!is.subset.owin(bb, rect))
        stop(paste("The new rectangle",
                   sQuote("rect"),
                   "does not contain the window",
                   sQuote("win")))
    }
    xr <- rect$xrange
    yr <- rect$yrange
    ## determine unitname
    uu <- list(unitname(x), unitname(rect))
    uu <- unique(uu[sapply(uu, is.vanilla)])
    if(length(uu) > 1) {
      warning("Arguments of rebound.owin have incompatible unitnames",
              call.=FALSE)
      uu <- list()
    }
    un <- if(length(uu)) uu[[1]] else NULL
    ## 
    switch(w$type,
           rectangle={
             return(owin(xr, yr,
                         poly=list(x=w$xrange[c(1L,2L,2L,1L)],
                                   y=w$yrange[c(1L,1L,2L,2L)]),
                         unitname = un,
                         check=FALSE))
           },
           polygonal={
             return(owin(xr, yr, poly=w$bdry, unitname=un, check=FALSE))
           },
           mask={
             xcol <- newseq(w$xcol, xr)
             yrow <- newseq(w$yrow, yr)
             newmask <- as.mask(xy=list(x=xcol, y=yrow))
             xx <- rasterx.mask(newmask)
             yy <- rastery.mask(newmask)
             newmask$m <- inside.owin(xx, yy, w)
             unitname(newmask) <- un
             return(newmask)
           }
           )
  }

  newseq <- function(oldseq, newrange) {
    oldrange <- range(oldseq)
    dstep <- mean(diff(oldseq))
    nleft <- max(0, floor((oldrange[1L] - newrange[1L])/dstep))
    nright <- max(0, floor((newrange[2L] - oldrange[2L])/dstep))
    newstart <- max(oldrange[1L] - nleft * dstep, newrange[1L])
    newend <- min(oldrange[2L] + nright * dstep, newrange[2L])
    seq(from=newstart, by=dstep, to=newend)
  }

  rebound.owin
})

    
  

#'
#'     timed.R
#'
#'   Timed objects
#'
#'   $Revision: 1.3 $ $Date: 2017/07/31 01:08:55 $

timed <- function(x, ..., starttime=NULL, timetaken=NULL) {
  if(is.null(starttime) && is.null(timetaken)) # time starts now.
    starttime <- proc.time()
  # evaluate expression if any
  object <- x
  if(is.null(timetaken))
    timetaken <- proc.time() - starttime
  if(!inherits(object, "timed"))
    class(object) <- c("timed", class(object))
  attr(object, "timetaken") <- timetaken
  return(object)
}

print.timed <- function(x, ...) {
  # strip the timing information and print the rest.
  taken <- attr(x, "timetaken")
  cx <- class(x)
  attr(x, "timetaken") <- NULL
  class(x) <- cx[cx != "timed"]
  NextMethod("print")
  # Now print the timing info
  cat(paste("\nTime taken:", codetime(taken), "\n"))
  return(invisible(NULL))
}

timeTaken <- function(..., warn=TRUE) {
  allargs <- list(...)
  hastime <- sapply(allargs, inherits, what="timed")
  if(!any(hastime)) {
    if(warn) warning("Data did not contain timing information", call.=FALSE)
    return(NULL)
  }
  if(warn && !all(hastime))
    warning("Some arguments did not contain timing information", call.=FALSE)
  times <- sapply(allargs[hastime], attr, which="timetaken")
  tottime <- rowSums(times)
  class(tottime) <- "proc_time"
  return(tottime)
}

#'  ..............  codetime ....................................
#'  Basic utility for converting times in seconds to text strings

codetime <- local({
  uname <- c("min", "hours", "days", "years",
             "thousand years", "million years", "billion years")
  u1name <- c("min", "hour", "day", "year",
             "thousand years", "million years", "billion years")
  multiple <- c(60, 60, 24, 365, 1e3, 1e3, 1e3)
  codehms <- function(x) {
    sgn <- if(x < 0) "-" else ""
    x <- round(abs(x))
    hours <- x %/% 3600
    mins  <- (x %/% 60) %% 60
    secs  <- x %% 60
    h <- if(hours > 0) paste(hours, ":", sep="") else ""
    started <- (hours > 0)
    m <- if(mins > 0) {
      paste(if(mins < 10 && started) "0" else "", mins, ":", sep="")
    } else if(started) "00:" else ""
    started <- started | (mins > 0)
    s <- if(secs > 0) {
      paste(if(secs < 10 && started) "0" else "", secs, sep="")
    } else if(started) "00" else "0"
    if(!started) s <- paste(s, "sec")
    paste(sgn, h, m, s, sep="")
  }
  codetime <- function(x, hms=TRUE, what=c("elapsed","user","system")) {
    if(inherits(x, "proc_time")) {
      what <- match.arg(what)
      x <- summary(x)[[match(what, c("user", "system", "elapsed"))]]
    }
    if(!is.numeric(x) || length(x) != 1)
      stop("codetime: x must be a proc_time object or a single number")
    sgn <- if(x < 0) "-" else ""
    x <- abs(x)
    if(x < 60)
      return(paste(sgn, signif(x, 3), " sec", sep=""))
    # more than 1 minute: round to whole number of seconds
    x <- round(x)
    if(hms && (x < 60 * 60 * 24))
      return(paste(sgn, codehms(x), sep=""))
    u <- u1 <- "sec"
    for(k in seq_along(multiple)) {
      if(x >= multiple[k]) {
        x <- x/multiple[k]
        u <- uname[k]
        u1 <- u1name[k]
      } else break
    }
    xx <- round(x, 1)
    ux <- if(xx == 1) u1 else u
    paste(sgn, xx, " ", ux, sep="")
  }
  codetime
})

#
#	weights.S
#
#	Utilities for computing quadrature weights
#
#	$Revision: 4.40 $	$Date: 2020/01/10 06:54:23 $
#
#
# Main functions:
		
#	gridweights()	    Divide the window frame into a regular nx * ny
#			    grid of rectangular tiles. Given an arbitrary
#			    pattern of (data + dummy) points derive the
#			    'counting weights'.
#
#	dirichletWeights()  Compute the areas of the tiles of the
#			    Dirichlet tessellation generated by the 
#			    given pattern of (data+dummy) points,
#			    restricted to the window.
#	
# Auxiliary functions:	
#			
#       countingweights()   compute the counting weights
#                           for a GENERIC tiling scheme and an arbitrary
#			    pattern of (data + dummy) points,
#			    given the tile areas and the information
#			    that point number k belongs to tile number id[k]. 
#
#
#	gridindex()	    Divide the window frame into a regular nx * ny
#			    grid of rectangular tiles. 
#			    Compute tile membership for arbitrary x,y.
#				    
#       grid1index()        1-dimensional analogue of gridindex()
#
#
#-------------------------------------------------------------------
	
countingweights <- function(id, areas, check=TRUE) {
	#
	# id:        cell indices of n points
	#                     (length n, values in 1:k)
	#
	# areas:     areas of k cells 
	#                     (length k)
	#
  id <- as.integer(id)
  fid <- factor(id, levels=seq_along(areas))
  counts <- table(fid)
  w <- areas[id] / counts[id]     # ensures denominator > 0
  w <- as.vector(w)
#	
# that's it; but check for funny business
#
  if(check) {
    zerocount <- (counts == 0)
    zeroarea <- (areas == 0)
    if(any(!zeroarea & zerocount)) {
      lostfrac <- 1 - sum(w)/sum(areas)
      lostpc <- round(100 * lostfrac, 1)
      if(lostpc >= 1) 
        warning(paste("some tiles with positive area",
                      "do not contain any quadrature points:",
                      "relative error =",
                      paste0(lostpc, "%")))
    }
    if(any(!zerocount & zeroarea)) {
	warning("Some tiles with zero area contain quadrature points")
	warning("Some weights are zero")
	attr(w, "zeroes") <- zeroarea[id]
    }
  }
#
  names(w) <- NULL
  return(w)
}

gridindex <- function(x, y, xrange, yrange, nx, ny) {
	#
	# The box with dimensions xrange, yrange is divided
	# into nx * ny cells.
	#
	# For each point (x[i], y[i]) compute the index (ix, iy)
	# of the cell containing the point.
	# 
	ix <- grid1index(x, xrange, nx)
	iy <- grid1index(y, yrange, ny)
	#
	return(list(ix=ix, iy=iy, index=as.integer((iy-1) * nx + ix)))
}

grid1index <- function(x, xrange, nx) {
	i <- ceiling( nx * (x - xrange[1])/diff(xrange))
	i <- pmax.int(1, i)
	i <- pmin.int(i, nx)
	i
}

gridweights <- function(X, ntile=NULL, ..., window=NULL, verbose=FALSE,
                        npix=NULL, areas=NULL) {
	#
	# Compute counting weights based on a regular tessellation of the
	# window frame into ntile[1] * ntile[2] rectangular tiles.
	#
	# Arguments X and (optionally) 'window' are interpreted as a
	# point pattern.
	#
	# The window frame is divided into a regular ntile[1] * ntile[2] grid
	# of rectangular tiles. The counting weights based on this tessellation
	# are computed for the points (x, y) of the pattern.
	#
        # 'npix' determines the dimensions of the pixel raster used to
        # approximate tile areas.
	
	X <- as.ppp(X, window)
	x <- X$x
	y <- X$y
        win <- X$window

        # determine number of tiles
        if(is.null(ntile))
          ntile <- default.ntile(X)
        if(length(ntile) == 1)
          ntile <- rep.int(ntile, 2)
        nx <- ntile[1]
        ny <- ntile[2]

        if(verbose) 
          cat(paste("grid weights for a", nx, "x", ny, "grid of tiles\n"))

        ## determine pixel resolution in case it is required
        if(!is.null(npix)) {
          npix <- ensure2vector(npix)
        } else {
          npix <- pmax(rev(spatstat.options("npixel")),
                       c(nx, ny))
          if(is.mask(win))
            npix <- pmax(npix, rev(dim(win)))
        }
        
        if(is.null(areas)) {
  	  # compute tile areas
          switch(win$type,
                 rectangle = {
                   nxy <- nx * ny
                   tilearea <- area(win)/nxy
                   areas <- rep.int(tilearea, nxy)
                   zeroareas <- rep(FALSE, nxy)
                 },
                 polygonal = {
                   areamat <- polytileareaEngine(win,
                                                 win$xrange, win$yrange,
                                                 nx, ny)
                   ## convert from 'im' to 'gridindex' ordering
                   areas <- as.vector(t(areamat))
                   zeroareas <- (areas == 0)
                   if(verbose)
                     splat("Split polygonal window of area", area(win),
                           "into", nx, "x", ny, "grid of tiles",
                           "of total area", sum(areas))
                 },
                 mask = {
                   win <- as.mask(win, dimyx=rev(npix))
                   if(verbose) 
                     splat("Converting mask dimensions to",
                           npix[1], "x", npix[2], "pixels")
                   ## extract pixel coordinates inside window
                   rxy <- rasterxy.mask(win, drop=TRUE)
                   xx <- rxy$x
                   yy <- rxy$y
                   
                   ## classify all pixels into tiles
                   pixelid <- gridindex(xx, yy, 
                                        win$xrange, win$yrange, nx, ny)$index
                   pixelid <- factor(pixelid, levels=seq_len(nx * ny))
                                
                   ## compute digital areas of tiles
                   tilepixels <- as.vector(table(pixelid))
                   pixelarea <- win$xstep * win$ystep
                   areas <- tilepixels * pixelarea
                   zeroareas <- (tilepixels == 0)
                 }
               )
        } else zeroareas <- (areas == 0)
        
        id <- gridindex(x, y, win$xrange, win$yrange, nx, ny)$index

        if(win$type != "rectangle" && any(uhoh <- zeroareas[id])) {
          # this can happen: the tile has digital area zero
          # but contains a data/dummy point
          slivers <- unique(id[uhoh])
          switch(win$type,
                 mask = {
                   offence <- "digital area zero"
                   epsa <- pixelarea/2
                 },
                 polygonal = {
                   offence <- "very small area"
                   epsa <- min(areas[!zeroareas])/10
                 })
          areas[slivers] <- epsa
          nsliver <- length(slivers)
          extraarea <- nsliver * epsa
          extrafrac <- extraarea/area(win)
          if(verbose || extrafrac > 0.01) {
            splat(nsliver, ngettext(nsliver, "tile", "tiles"),
                  "of", offence,
                  ngettext(nsliver, "was", "were"),
                  "given nominal area", signif(epsa, 3),
                  "increasing the total area by",
                  signif(extraarea, 5), "square units or",
                  paste0(round(100 * extrafrac, 1), "% of total area"))
            if(extrafrac > 0.01)
              warning(paste("Repairing tiles with", offence,
                            "caused a",
                            paste0(round(100 * extrafrac), "%"),
                            "change in total area"),
                      call.=FALSE)
          }
        }
 
	# compute counting weights 
	w <- countingweights(id, areas)

        # attach information about weight construction parameters
        attr(w, "weight.parameters") <-
          list(method="grid", ntile=ntile, npix=npix, areas=areas)
        
	return(w)
}


# dirichlet.weights <- function(...) {
#   .Deprecated("dirichletWeights", package="spatstat")
#   dirichletWeights(...)
# }

dirichletWeights <- function(X, window = NULL, exact=TRUE, ...) {
  #'
  #' Compute weights based on Dirichlet tessellation of the window 
  #' induced by the point pattern X. 
  #' The weights are just the tile areas.
  #'
  #' NOTE:	X should contain both data and dummy points,
  #' if you need these weights for the B-T-B method.
  #'
  #' Arguments X and (optionally) 'window' are interpreted as a
  #' point pattern.
  #'
  #' If the window is a rectangle, we invoke Rolf Turner's "deldir"
  #' package to compute the areas of the tiles of the Dirichlet
  #' tessellation of the window frame induced by the points.
  #' [NOTE: the functionality of deldir to create dummy points
  #' is NOT used. ]
  #'	if exact=TRUE	compute the exact areas, using "deldir"
  #'	if exact=FALSE      compute the digital areas using exactdt()
  #' 
  #' If the window is a mask, we compute the digital area of
  #' each tile of the Dirichlet tessellation by counting pixels.
  #'
  #'
  #' 
  X <- as.ppp(X, window)

  if(!exact && is.polygonal(Window(X)))
    Window(X) <- as.mask(Window(X))
  
  #' compute tile areas
  w <- dirichletAreas(X)

  #' zero areas can occur due to discretisation or weird geometry
  zeroes <- (w == 0)
  if(any(zeroes)) {
    #' compute weights for subset
    nX <- npoints(X)
    Xnew <- X[!zeroes]
    wnew <- dirichletAreas(Xnew)
    w    <- numeric(nX)
    w[!zeroes] <- wnew
    #' map deleted points to nearest retained point
    jj <- nncross(X[zeroes], Xnew, what="which")
    #' map retained points to themselves
    ii <- Xseq <- seq_len(nX)
    ii[zeroes] <- (ii[!zeroes])[jj]
    #' redistribute weights
    nshare <- table(factor(ii, levels=Xseq))
    w <- w[ii]/nshare[ii]
  }
  #' attach information about weight construction parameters
  attr(w, "weight.parameters") <- list(method="dirichlet", exact=exact)
  return(w)
}

default.ntile <- function(X) { 
	# default number of tiles (n x n) for tile weights
        # when data and dummy points are X
  X <- as.ppp(X)
  guess.ngrid <- 10 * floor(sqrt(X$n)/10)
  max(5, guess.ngrid/2)
}

#
#    pspcross.R
#
#    Intersections of line segments
#    
#    $Revision: 1.26 $   $Date: 2020/03/23 01:15:26 $
#
#
crossing.psp <- function(A,B,fatal=TRUE,details=FALSE) {
  verifyclass(A, "psp")
  verifyclass(B, "psp")
  
  # first check for intersection of windows
  ABW <- intersect.owin(A$window, B$window, fatal=fatal)
  if(is.null(ABW)) 
    return(NULL)
  
  eps <- .Machine$double.eps

  na <- A$n
  eA <- A$ends
  x0a <- eA$x0
  y0a <- eA$y0
  dxa <- eA$x1 - eA$x0
  dya <- eA$y1 - eA$y0

  nb <- B$n
  eB <- B$ends
  x0b <- eB$x0
  y0b <- eB$y0
  dxb <- eB$x1 - eB$x0
  dyb <- eB$y1 - eB$y0

  useCall <- spatstat.options("crossing.psp.useCall")
  if(!useCall) {
    # old C routine
    out <- .C("xysegint",
              na=as.integer(na),
              x0a=as.double(x0a),
              y0a=as.double(y0a),
              dxa=as.double(dxa),
              dya=as.double(dya), 
              nb=as.integer(nb),
              x0b=as.double(x0b),
              y0b=as.double(y0b),
              dxb=as.double(dxb),
              dyb=as.double(dyb), 
              eps=as.double(eps),
              xx=as.double(numeric(na * nb)),
              yy=as.double(numeric(na * nb)),
              ta=as.double(numeric(na * nb)),
              tb=as.double(numeric(na * nb)),
              ok=as.integer(integer(na * nb)),
              PACKAGE = "spatstat")
    
    ok <- (matrix(out$ok, na, nb) != 0)
    xx <- matrix(out$xx, na, nb)
    yy <- matrix(out$yy, na, nb)
    xx <- as.vector(xx[ok])
    yy <- as.vector(yy[ok])
    if(details) {
      ia <- as.vector(row(ok)[ok])
      jb <- as.vector(col(ok)[ok])
      ta <- as.vector(matrix(out$ta, na, nb)[ok])
      tb <- as.vector(matrix(out$tb, na, nb)[ok])
    }
  } else {
    # new
    storage.mode(x0a) <- storage.mode(y0a) <- "double"
    storage.mode(dxa) <- storage.mode(dya) <- "double"
    storage.mode(x0b) <- storage.mode(y0b) <- "double"
    storage.mode(dxb) <- storage.mode(dyb) <- "double"
    storage.mode(eps) <- "double"
    out <- .Call("Cxysegint",
                 x0a, 
                 y0a, 
                 dxa, 
                 dya, 
                 x0b, 
                 y0b, 
                 dxb, 
                 dyb, 
    	           eps,
                 PACKAGE="spatstat")
    xx <- out[[5]]
    yy <- out[[6]]
    if(details) {
      ia <- out[[1L]] + 1L
      jb <- out[[2L]] + 1L
      ta <- out[[3L]]
      tb <- out[[4L]]
    }
  }
  result <- ppp(xx, yy, window=ABW, check=FALSE)
  if(details)
    marks(result) <- data.frame(iA=ia, jB=jb, tA=ta, tB=tb)
  return(result)
}

test.crossing.psp <- function(A,B) {
  # return logical matrix specifying whether A[i] and B[j] cross
  verifyclass(A, "psp")
  verifyclass(B, "psp")
  eps <- .Machine$double.eps

  na <- A$n
  eA <- A$ends
  x0a <- eA$x0
  y0a <- eA$y0
  dxa <- eA$x1 - eA$x0
  dya <- eA$y1 - eA$y0

  nb <- B$n
  eB <- B$ends
  x0b <- eB$x0
  y0b <- eB$y0
  dxb <- eB$x1 - eB$x0
  dyb <- eB$y1 - eB$y0

  out <- .C("xysi",
            na=as.integer(na),
            x0a=as.double(x0a),
            y0a=as.double(y0a),
            dxa=as.double(dxa),
            dya=as.double(dya), 
            nb=as.integer(nb),
            x0b=as.double(x0b),
            y0b=as.double(y0b),
            dxb=as.double(dxb),
            dyb=as.double(dyb), 
            eps=as.double(eps),
            ok=as.integer(integer(na * nb)),
            PACKAGE = "spatstat")

  hit <- (matrix(out$ok, na, nb) != 0)
  return(hit)
}

anycrossing.psp <- function(A,B) {
  # equivalent to: any(test.crossing.psp(A,B))
  # Test whether two psp objects have at least one crossing point
  verifyclass(A, "psp")
  verifyclass(B, "psp")
  eps <- .Machine$double.eps

  na <- A$n
  eA <- A$ends
  x0a <- eA$x0
  y0a <- eA$y0
  dxa <- eA$x1 - eA$x0
  dya <- eA$y1 - eA$y0

  nb <- B$n
  eB <- B$ends
  x0b <- eB$x0
  y0b <- eB$y0
  dxb <- eB$x1 - eB$x0
  dyb <- eB$y1 - eB$y0

  out <- .C("xysiANY",
            na=as.integer(na),
            x0a=as.double(x0a),
            y0a=as.double(y0a),
            dxa=as.double(dxa),
            dya=as.double(dya), 
            nb=as.integer(nb),
            x0b=as.double(x0b),
            y0b=as.double(y0b),
            dxb=as.double(dxb),
            dyb=as.double(dyb), 
            eps=as.double(eps),
            ok=as.integer(integer(1L)),
            PACKAGE = "spatstat")
  hit <- (out$ok != 0)
  return(hit)
}

selfcrossing.psp <- function(A) {
  verifyclass(A, "psp")
  eps <- .Machine$double.eps

  n <- A$n
  eA <- A$ends
  x0 <- eA$x0
  y0 <- eA$y0
  dx <- eA$x1 - eA$x0
  dy <- eA$y1 - eA$y0

  useCall <- spatstat.options("selfcrossing.psp.useCall")
  if(!useCall) {
    # old C routine
    out <- .C("xysegXint",
              n=as.integer(n),
              x0=as.double(x0),
              y0=as.double(y0),
              dx=as.double(dx),
              dy=as.double(dy), 
              eps=as.double(eps),
              xx=as.double(numeric(n^2)),
              yy=as.double(numeric(n^2)),
              ti=as.double(numeric(n^2)),
              tj=as.double(numeric(n^2)),
              ok=as.integer(integer(n^2)),
              PACKAGE = "spatstat")

    ok <- (matrix(out$ok, n, n) != 0)
    xx <- matrix(out$xx, n, n)
    yy <- matrix(out$yy, n, n)
    xx <- as.vector(xx[ok])
    yy <- as.vector(yy[ok])
  } else {
    # new
    storage.mode(x0) <- storage.mode(y0) <- "double"
    storage.mode(dx) <- storage.mode(dy) <- "double"
    storage.mode(eps) <- "double"
    out <- .Call("CxysegXint",
                 x0, 
                 y0, 
                 dx, 
                 dy, 
    	           eps,
                 PACKAGE="spatstat")
    xx <- out[[5L]]
    yy <- out[[6L]]
  }
  result <- ppp(xx, yy, window=A$window, check=FALSE)
  return(result)
}


test.selfcrossing.psp <- function(A) {
  verifyclass(A, "psp")
  eps <- .Machine$double.eps

  n <- A$n
  eA <- A$ends
  x0 <- eA$x0
  y0 <- eA$y0
  dx <- eA$x1 - eA$x0
  dy <- eA$y1 - eA$y0

  out <- .C("xysxi",
            na=as.integer(n),
            x0=as.double(x0),
            y0=as.double(y0),
            dx=as.double(dx),
            dy=as.double(dy), 
            eps=as.double(eps),
            ok=as.integer(integer(n*n)),
            PACKAGE = "spatstat")
  hit <- (matrix(out$ok, n, n) != 0)
  return(hit)
}

selfcut.psp <- function(A, ..., eps) {
  stopifnot(is.psp(A))
  n <- A$n
  eA <- A$ends
  x0 <- eA$x0
  y0 <- eA$y0
  x1 <- eA$x1
  y1 <- eA$y1
  dx <- x1 - x0
  dy <- y1 - y0
  if(missing(eps) || is.null(eps)) {
    eps <- sqrt(.Machine$double.eps) * diameter(Frame(A))
  } else {
    check.1.real(eps)
    stopifnot(eps >= 0)
  }
  ## identify self-crossings
  eps <- .Machine$double.eps
  storage.mode(x0) <- storage.mode(y0) <- "double"
  storage.mode(dx) <- storage.mode(dy) <- "double"
  storage.mode(eps) <- "double"
  zz <- .Call("CxysegXint",
              x0, 
              y0, 
              dx, 
              dy, 
              eps,
              PACKAGE = "spatstat")
  if(length(zz[[1]]) == 0) {
    ## no dissection required
    attr(A, "camefrom") <- seq_len(n)
    return(A)
  }
  ##
  names(zz) <- c("i", "j", "ti", "tj", "x", "y")
  icross  <- zz$i + 1L
  jcross  <- zz$j + 1L
  ticross <- zz$ti
  tjcross <- zz$tj
  xcross  <- zz$x
  ycross  <- zz$y
  ## which segments are split...
  gone <- unique(c(icross, jcross))
  ## ... and which are not
  retained <- setdiff(seq_len(n), gone)
  ## initialise result
  ## start with all segments which are retained
  x0out <- x0[retained]
  y0out <- y0[retained]
  x1out <- x1[retained]
  y1out <- y1[retained]
  camefrom <- retained
  ## cut each segment using the *provided* values of x,y
  for(ii in gone) {
    ## assemble cuts through segment ii
    imatch <- which(icross == ii)
    jmatch <- which(jcross == ii)
    ijmatch <- c(imatch, jmatch)
    tt <- c(ticross[imatch], tjcross[jmatch])
    xx <- xcross[ijmatch]
    yy <- ycross[ijmatch]
    # discard T-junctions
    ok <- (tt > 0 & tt < 1)
    tt <- tt[ok]
    xx <- xx[ok]
    yy <- yy[ok]
    # order the pieces
    ord <- order(tt)
    xx <- xx[ord]
    yy <- yy[ord]
    ## add endpoints of old segment
    xnew <- c(x0[ii], xx, x1[ii])
    ynew <- c(y0[ii], yy, y1[ii])
    ## append to result
    m <- length(xnew)
    x0out <- c(x0out, xnew[-m])
    y0out <- c(y0out, ynew[-m])
    x1out <- c(x1out, xnew[-1L])
    y1out <- c(y1out, ynew[-1L])
    camefrom <- c(camefrom, rep(ii, m-1L))
  }
  marx <- marks(A)
  marxout <- if(is.null(marx)) NULL else
             as.data.frame(marx)[camefrom, , drop=FALSE]
  Y <- psp(x0out, y0out, x1out, y1out, window=Window(A), marks=marxout,
           check=FALSE)
  if(eps > 0) {
    ok <- (lengths_psp(Y) > eps)
    if(!all(ok)) {
      Y <- Y[ok]
      camefrom <- camefrom[ok]
    }
  }
  attr(Y, "camefrom") <- camefrom
  return(Y)
}
  

#
#  colourschemes.R
#
#  $Revision: 1.5 $  $Date: 2019/12/15 05:25:53 $
#

beachcolourmap <- function(range, ...) {
  col <- beachcolours(range, ...)
  z <- colourmap(col, range=range)
  return(z)
}

beachcolours <- function(range, sealevel = 0, monochrome=FALSE,
                         ncolours=if(monochrome) 16 else 64,
                         nbeach=1) {
  check.range(range)
  stopifnot(all(is.finite(range)))
  check.1.real(sealevel)
  range <- range(c(sealevel,range))
  check.1.integer(ncolours)
  stopifnot(ncolours >= 3)
  check.1.integer(nbeach)
  stopifnot(nbeach >= 0)
  stopifnot(nbeach <= ncolours + 2)
  if(monochrome)
    return(grey(seq(from=0,to=1,length.out=ncolours)))
  depths <- range[1L]
  peaks <- range[2L]
  dv <- diff(range)/(ncolours - 1L)
  epsilon <- nbeach * dv/2
  lowtide <- max(sealevel - epsilon, depths)
  hightide <-  min(sealevel + epsilon, peaks)
  nsea <- max(0L, floor((lowtide - depths)/dv))
  nland <- max(0L, floor((peaks  - hightide)/dv))
  discrep <- nsea + nland + nbeach - ncolours
  if(discrep != 0) {
    dd <- abs(discrep)
    ss <- as.integer(-sign(discrep))
    smallhalf <- dd/2L
    largehalf <- dd - smallhalf
    if(nsea < nland) {
      nsea <- nsea + ss * smallhalf
      nland <- nland + ss * largehalf
    } else {
      nland <- nland + ss * smallhalf
      nsea <- nsea + ss * largehalf
    }      
    if(nsea + nland + nbeach != ncolours)
      warning("Internal error: incorrect adjustment of length in beachcolours")
  }
  colours <- character(0)
  if(nsea > 0)  colours <- rev(rainbow(nsea, start=3/6,end=4/6)) # cyan/blue
  if(nbeach > 0)  colours <- c(colours,
                               rev(rainbow(nbeach, start=3/12,end=5/12))) # green
  if(nland > 0)  colours <- c(colours,
                              rev(rainbow(nland, start=0, end=1/6)))  # red/yellow
  return(colours)
}

#'
#' bw.pcf.R
#'
#' $Revision: 1.5 $  $Date: 2019/09/30 07:51:52 $
#'
#' bandwidth selection for pcf
#' with least-squares cross-validation method
#' 
#' Original code by: Rasmus Waagepetersen and Abdollah Jalilian
#'
#' References:
#' Guan, Y. (2007). A composite likelihood cross-validation approach in 
#'   selecting bandwidth for the estimation of the pair correlation function. 
#'   Scandinavian Journal of Statistics, 34(2), 336--346. 
#'   DOI: http://doi.org/10.1111/j.1467-9469.2006.00533.x
#' Guan, Y. (2007). A least-squares cross-validation bandwidth 
#'   selection approach in pair correlation function estimations. 
#'   Statistics & Probability Letters, 77(18), 1722--1729. 
#'   DOI: http://doi.org/10.1016/j.spl.2007.04.016

bw.pcf <- function(X, rmax=NULL, lambda=NULL, divisor="r", 
                   kernel="epanechnikov", nr=10000, bias.correct=TRUE, 
                   cv.method=c("compLik", "leastSQ"), simple=TRUE,
                   srange=NULL, ..., verbose=FALSE, warn=TRUE)
{
  stopifnot(is.ppp(X))
  X <- unmark(X)
  win <- Window(X)
  areaW <- area(win)
  nX <- npoints(X)

  cv.method <- match.arg(cv.method)
  kernel <- match.kernel(kernel)
  
  #' maximum distance lag: rmax
  if (is.null(rmax))
    rmax <- rmax.rule("K", win,  nX/areaW)
  if(is.null(srange))
    srange <- c(0, rmax/4)
  #' number of subintervals for discretization of [0, rmax]: nr
  #' length of subintervals
  discr <- rmax / nr
  #' breaks of subintervals
  rs <- seq(0, rmax, length.out= nr + 1)

  #' closepairs distances: \\ u - v \\
  #' Pre-compute close pair distances for use in 'pcf'
  #'   we need close pairs up to a distance rmax + smax
  #'   where 'smax' is the maximum halfwidth of the support of the kernel
  smax <- srange[2] * (if(kernel == "gaussian") 2 else kernel.factor(kernel))
  cpfull <- closepairs(X, rmax + smax, what="all", twice=TRUE)
  
  #' For cross-validation, restrict close pairs to distance rmax 
  ok <- (cpfull$d <= rmax)
  cp <- lapply(cpfull, "[", i=ok)

  ds <- cp$d
  #' determining closepairs distances are in which subinterval
  idx <- round(ds / discr) + 1L
  idx <- pmin.int(idx, nr+1L)
  
  #' translation edge correction factor: /W|/|W \cap W_{u-v}|
  edgewt <- edge.Trans(dx=cp$dx, dy=cp$dy, W=win, paired=TRUE)
  
  if(homogeneous <- is.null(lambda)) {
    #' homogeneous case
    lambda <- nX/areaW
    lambda2area <- lambda^2 * areaW
    pcfargs <- list(X=X, r=rs,
                    divisor=divisor, kernel=kernel, correction="translate",
                    close=cpfull)
    renorm.factor <- 1
  } else {
    # inhomogeneous case: lambda is assumed to be a numeric vector giving
    # the intensity at the points of the point pattern X
    check.nvector(lambda, nX)
    lambda2area <- lambda[cp$i] * lambda[cp$j] * areaW
    pcfargs <- list(X=X, lambda=lambda, r=rs,
                    divisor=divisor, kernel=kernel, correction="translate",
                    close=cpfull)
    renorm.factor <- (areaW/sum(1/lambda))
  }
  
  stuff <- list(cv.method=cv.method,
                kernel=kernel,
                homogeneous=homogeneous,
                bias.correct=bias.correct,
                simple = simple,
                discr=discr,
                rs=rs,
                cp=cp,
                ds=ds,
                idx=idx,
                edgewt=edgewt,
                pcfargs=pcfargs,
                lambda=lambda,
                lambda2area=lambda2area,
                renorm.factor=renorm.factor,
		show=verbose)
  stuff <- list2env(stuff)

  #' find optimum bandwidth
  z <- optimizeWithTrace(CVforPCF, srange, maximum=TRUE, stuff=stuff)

  #' pack up
  ox <- order(z$x)
  sigma  <- z$x[ox]
  cv     <- z$y[ox]
  criterion <- switch(cv.method,
                      compLik = "composite likelihood cross-validation",
                      leastSQ = "least squares cross-validation")
  result <- bw.optim(cv, sigma, which.max(cv),
                     criterion = criterion,
                     warnextreme=warn, hargnames=c("rmax", "srange"),
                     unitname=unitname(X))
  return(result)
}

CVforPCF <- function(bw, stuff) {
  stuff$bw <- bw
  with(stuff, {
    if(show) splat("bw=", bw)
    #' values of pair correlation at breaks of subintervals
    a <- append(pcfargs, list(bw=bw))
    grs <- if(homogeneous) do.call(pcf.ppp, a) else do.call(pcfinhom, a)
    grs <- grs$trans
    #' bias correction
    if (bias.correct) {
      grs <- grs / pkernel(rs, kernel, 0, bw)
      dcorrec <- pkernel(ds, kernel, 0, bw)
    } else {
      dcorrec <- 1
    }
    #' make sure that the estimated pair correlation at origin is finite
    if (!is.finite(grs[1]))
      grs[1] <- grs[2]
    #' approximate the pair correlation values at closepairs distances
    gds <- grs[idx]
    wt <- edgewt / (2 * pi * ds * lambda2area * dcorrec) * renorm.factor
    #' remove pairs to approximate the cross-validation term: g^{-(u, v)}
    if (simple) {
      gds <- gds - 2 * wt * dkernel(0, kernel, 0, bw)
    } else {
      cpi <- cp$i
      cpj <- cp$j
      for (k in 1:length(ds)) {
        exclude <- (cpi == cpi[k]) | (cpj == cpj[k])
        gds[k] <- gds[k] - 2 * sum(wt[exclude] * 
                                   dkernel(ds[k] - ds[exclude],
                                           kernel, 0, bw))
      }
    }
    #' remove negative and zero values
    gds <- pmax.int(.Machine$double.eps, gds)
    switch(cv.method,
           compLik={
             #' composite likelihood cross-validation
             #' the integral term: 2 \pi \int_{0}^{rmax} \hat g(r) r dr
             normconst <- 2 * pi * sum(grs * rs) * discr
             value <- mean(log(gds)) - log(normconst)
           },
           leastSQ={
             #' least squares cross-validation
             #' the integral term: 2 \pi \int_{0}^{rmax} \hat g^2(r) r dr
             normconst <- 2 * pi * sum(grs^2 * rs) * discr
             value <- 2 * sum(gds * edgewt / (lambda2area)) - normconst
           },
           stop("Unrecognised cross-validation method"))
    if(show) splat("value=", value)
    return(value)
  })
}

##
## auc.R
##
##  Calculate ROC curve or area under it
##
## $Revision: 1.6 $ $Date: 2016/11/10 01:08:04 $

roc <- function(X, ...) { UseMethod("roc") }

roc.ppp <- function(X, covariate, ..., high=TRUE) {
  nullmodel <- ppm(X)
  result <- rocData(covariate, nullmodel, ..., high=high)
  return(result)
}

roc.lpp <- function(X, covariate, ..., high=TRUE) {
  nullmodel <- lppm(X)
  result <- rocData(covariate, nullmodel, ..., high=high)
  return(result)
}

rocData <- function(covariate, nullmodel, ..., high=TRUE) {
  d <- spatialCDFframe(nullmodel, covariate, ...)
  U <- d$values$U
  ec <- if(high) ecdf(1-U) else ecdf(U)
  p <- seq(0,1,length=1024)
  df <- data.frame(p=p, fobs=ec(p), fnull=p)
  result <- fv(df,
               argu="p",
               ylab=quote(roc(p)),
               valu="fobs",
               desc=c("fraction of area",
                      "observed fraction of points",
                      "expected fraction if no effect"),
               fname="roc")
  fvnames(result, ".") <- c("fobs", "fnull")
  return(result)
}

roc.ppm <- function(X, ...) {
  stopifnot(is.ppm(X))
  model <- X
  lambda <- predict(model, ...)
  Y <- data.ppm(model)
  nullmodel <- ppm(Y)
  result <- rocModel(lambda, nullmodel, ...)
  return(result)
}

roc.kppm <- function(X, ...) {
  stopifnot(is.kppm(X))
  model <- as.ppm(X)
  lambda <- predict(model, ...)
  Y <- data.ppm(model)
  nullmodel <- ppm(Y)
  result <- rocModel(lambda, nullmodel, ...)
  return(result)
}

roc.lppm <- function(X, ...) {
  stopifnot(is.lppm(X))
  model <- X
  lambda <- predict(model, ...)
  Y <- X$X
  nullmodel <- lppm(Y)
  result <- rocModel(lambda, nullmodel, ...)
  return(result)
}

rocModel <- function(lambda, nullmodel, ..., high) {
  if(!missing(high))
    warning("Argument 'high' is ignored when computing ROC for a fitted model")
  d<- spatialCDFframe(nullmodel, lambda, ...) 
  U <- d$values$U
  ec <- ecdf(1-U) 
  p <- seq(0,1,length=1024)
  fobs <- ec(p)
  FZ <- d$values$FZ
  lambdavalues <- if(is.im(lambda)) lambda[] else unlist(lapply(lambda, "["))
  F1Z <- ewcdf(lambdavalues, lambdavalues/sum(lambdavalues))    
  pZ <- get("y", environment(FZ))
  qZ <- get("x", environment(FZ))
  FZinverse <- approxfun(pZ, qZ, rule=2)
  ftheo <- 1 - F1Z(FZinverse(1-p))
  df <- data.frame(p=p, fobs=fobs, ftheo=ftheo, fnull=p)
  result <- fv(df,
               argu="p",
               ylab=quote(roc(p)),
               valu="fobs",
               fmla = . ~ p,
               desc=c("fraction of area",
                 "observed fraction of points",
                 "expected fraction of points",
                 "expected fraction if no effect"),
               fname="roc")
  fvnames(result, ".") <- c("fobs", "ftheo", "fnull")
  return(result)
}

#    ......................................................

auc <- function(X, ...) { UseMethod("auc") }

auc.ppp <- function(X, covariate, ..., high=TRUE) {
  d <- spatialCDFframe(ppm(X), covariate, ...)
  U <- d$values$U
  EU <- mean(U)
  result <- if(high) EU else (1 - EU) 
  return(result)
}

auc.lpp <- function(X, covariate, ..., high=TRUE) {
  d <- spatialCDFframe(lppm(X), covariate, ...)
  U <- d$values$U
  EU <- mean(U)
  result <- if(high) EU else (1 - EU) 
  return(result)
}

auc.kppm <- function(X, ...) { auc(as.ppm(X), ...) }

auc.ppm <- function(X, ...) {
  model <- X
  if(is.multitype(model)) {
    # cheat
    ro <- roc(model, ...)
    aobs <- with(ro, mean(fobs))
    atheo <- with(ro, mean(ftheo))
  } else if(is.stationary(model)) {
    aobs <- atheo <- 1/2
  } else {
    lambda <- intensity(model)
    Fl <- ecdf(lambda[])
    lambda <- as.im(lambda, Window(model))
    X <- data.ppm(model)
    lamX <- lambda[X]
    aobs <- mean(Fl(lamX))
    atheo <- mean(lambda[] * Fl(lambda[]))/mean(lambda)
  }
  result <- c(aobs, atheo)
  names(result) <- c("obs", "theo")
  return(result)
}

auc.lppm <- function(X, ...) {
  stopifnot(inherits(X, "lppm"))
  model <- X
  if(is.multitype(model)) {
    # cheat
    ro <- roc(model, ...)
    aobs <- with(ro, mean(fobs))
    atheo <- with(ro, mean(ftheo))
  } else {
    lambda <- predict(model, ...)
    Fl <- ecdf(lambda[])
    lamX <- lambda[model$X]
    aobs <- mean(Fl(lamX))
    atheo <- mean(lambda[] * Fl(lambda[]))/mean(lambda)
  }
  result <- c(aobs, atheo)
  names(result) <- c("obs", "theo")
  return(result)
}

#
# nnmark.R
#
# $Revision: 1.7 $ $Date: 2018/02/14 08:00:59 $

nnmark <- local({

  nnmark <- function(X, ..., k=1, at=c("pixels", "points")) {
    stopifnot(is.ppp(X))
    stopifnot(is.marked(X))
    at <- match.arg(at)
    mX <- marks(X)
    switch(at,
           pixels = {
             Y <- nnmap(X, k=k, what="which", ...)
             switch(markformat(X),
                    vector={
                      result <- eval.im(mX[Y])
                    },
                    dataframe = {
                      mX <- as.list(as.data.frame(mX))
                      result <- solapply(mX, lookedup, indeximage=Y)
                    },
                    stop("Marks must be a vector or dataframe"))
           },
           points = {
             Y <- nnwhich(X, k=k)
             switch(markformat(X),
                    vector={
                      result <- mX[Y]
                    },
                    dataframe = {
                      result <- mX[Y,, drop=FALSE]
                      row.names(result) <- NULL
                    },
                    stop("Marks must be a vector or dataframe"))
           })
    return(result)
  }

  lookedup <- function(xvals, indeximage) eval.im(xvals[indeximage])

  nnmark
})




  

#'
#'       minkowski.R
#' 
#'  Minkowski Sum and related operations
#'
#'  $Revision: 1.7 $ $Date: 2017/06/05 10:31:58 $


"%(+)%" <- MinkowskiSum <- local({

  MinkowskiSum <- function(A, B) {
    if(is.ppp(A)) return(UnionOfShifts(B, A))
    if(is.ppp(B)) return(UnionOfShifts(A, B))
    ## extract lists of simply-connected polygons
    AA <- simplepolygons(A)
    BB <- simplepolygons(B)
    ## determine common resolution for polyclip operations
    eps <- mean(c(sidelengths(Frame(A)), sidelengths(Frame(B))))/2^30
    ## compute Minkowski sums of pieces
    pieces <- NULL
    for(b in BB) 
      pieces <- append(pieces, lapply(AA, MinkSumConnected, b=b, eps=eps))
    ## form union in one step, to avoid artefacts
    result <- union.owin(solapply(pieces, poly2owin))
    return(result)
  }

  poly2owin <- function(z) owin(poly=z, check=FALSE)

  MinkSumConnected <- function(a, b, eps) {
    ## a and b are list(x,y) simply-connected polygons
    out <- polyclip::polyminkowski(a, b, x0=0, y0=0, eps=eps)
    if(length(out) == 1) return(out)
    ispos <- (sapply(out, Area.xypolygon) >= 0)
    if(sum(ispos) > 1) {
      stop("Internal error: result of sumconnected is not simply connected",
           call.=FALSE)
    }
    return(out[ispos])
  }

  simplepolygons <- function(A) {
    if(is.psp(A)) return(psp2poly(A))
    ## convert to owin, then polygonal
    A <- as.polygonal(A)
    ## separate into simply-connected pieces
    AA <- break.holes(A)$bdry
    return(AA)
  }
  
  ## handle segment patterns as well 
  psp2poly <- function(X) apply(as.matrix(X$ends), 1, seg2poly)

  seg2poly <- function(z) with(as.list(z), list(x=c(x0, x1, x0), y=c(y0,y1,y0)))

  ##
  UnionOfShifts <- function(X, V) {
    #' compute the union or superposition of copies of X by vectors in V
    v <- as.matrix(coords(V))
    n <- nrow(v)
    Y <- vector(mode="list", length=n)
    for(i in seq_len(n)) 
      Y[[i]] <- shift(X, v[i,])
    Y <- as.solist(Y)
    if(is.owin(X)) {
      Z <- union.owin(Y)
    } else {
      #' X is a pattern of objects in a window
      W <- MinkowskiSum(Window(X), Window(V))
      Z <- superimpose(Y, W=W)
    }
    return(Z)
  }

  MinkowskiSum
})

dilationAny <- function(A, B) { MinkowskiSum(A, reflect(B)) }

"%(-)%" <- erosionAny <- function(A, B) {
  D <- Frame(A)
  Dplus <- grow.rectangle(D, 0.1 * shortside(D))
  Ac <- complement.owin(A, Dplus)
  AcB <- MinkowskiSum(Ac, reflect(B))
  if(is.subset.owin(D, AcB))
    return(emptywindow(D))
  C <- complement.owin(AcB[Dplus], Dplus)[D]
  return(C)
}

#'  pointweights.R
#'
#'  get a valid vector of weights for a point pattern
#'
#'  Argument 'weights' is usually passed from a user-level function
#'  It may be:
#'        a numeric vector
#'        a single number
#'        a function(x,y)
#'        a pixel image
#'        an expression involving the coordinates and marks
#' 
#'  $Revision: 1.2 $ $Date: 2020/01/27 09:08:06 $

pointweights <- function(X, ..., weights=NULL, parent=NULL) {
  if(is.null(weights)) return(NULL)
  nX <- npoints(X)
  if(is.numeric(weights) && is.vector(as.numeric(weights))) {
    if(length(weights) == 1) weights <- rep(weights, nX)
  } else if(is.im(weights)) {
    weights <- safelookup(weights, X) # includes warning if NA
  } else if(is.function(weights)) {
      weights <- weights(X$x, X$y)
  } else if(is.expression(weights)) {
    #' evaluate expression in data frame of coordinates and marks
    df <- as.data.frame(X)
    weights <- try(eval(weights, envir=df, enclos=parent))
    if(inherits(weights, "try-error"))
      stop("Unable to evaluate expression for weights", call.=FALSE)
    if(length(weights) == 0) return(NULL)
  } else stop(paste("Argument 'weights' should be",
                    "a numeric vector, a function, an image,",
                    "or an expression"), call.=FALSE)
  check.nvector(weights, nX)
  return(weights)
}

#
#   pointsonlines.R
#
# place points at regular intervals along line segments
#
#   $Revision: 1.9 $  $Date: 2020/03/16 10:28:51 $
#

pointsOnLines <- function(X, eps=NULL, np=1000, shortok=TRUE) {
  stopifnot(is.psp(X))
  len <- lengths_psp(X)
  nseg <- length(len)
  if(is.null(eps)) {
    stopifnot(is.numeric(np) && length(np) == 1)
    stopifnot(is.finite(np) && np > 0)
    eps <- sum(len)/np
  } else {
    stopifnot(is.numeric(eps) && length(eps) == 1)
    stopifnot(is.finite(eps) && eps > 0)
  }
  # initialise
  Xdf    <- as.data.frame(X)
  xmid <- with(Xdf, (x0+x1)/2)
  ymid <- with(Xdf, (y0+y1)/2)
  # handle very short segments
#  allsegs <- 1:nseg
  if(any(short <- (len <= eps)) && shortok) {
    # very short segments: use midpoints
    Z <- data.frame(x = xmid[short], y = ymid[short], seg=which(short), tp=0.5)
  } else Z <- data.frame(x=numeric(0), y=numeric(0),
                         seg=integer(0), tp=numeric(0))
  # handle other segments
  for(i in (1:nseg)[!short]) {
    # divide segment into pieces of length eps
    # with shorter bits at each end
    leni <- len[i]
    nwhole <- floor(leni/eps)
    if(leni/eps - nwhole < 0.5 && nwhole > 2)
      nwhole <- nwhole - 1
    rump <- (leni - nwhole * eps)/2
    brks <- c(0, rump + (0:nwhole) * eps, leni)
    nbrks <- length(brks)
    # points at middle of each piece
    ss <- (brks[-1] + brks[-nbrks])/2
    tp <- ss/leni
    x <- with(Xdf, x0[i] + tp * (x1[i]-x0[i]))
    y <- with(Xdf, y0[i] + tp * (y1[i]-y0[i]))
    Z <- rbind(Z, data.frame(x=x, y=y, seg=i, tp=tp))
  }
  result <- as.ppp(Z[,c("x","y")], W=X$window)
  attr(result, "map") <- Z[,c("seg", "tp")]
  return(result)
}

#
#   by.ppp.R
#
#  $Revision: 1.6 $  $Date: 2015/10/21 09:06:57 $
#

by.ppp <- function(data, INDICES=marks(data), FUN, ...) {
  if(missing(INDICES))
    INDICES <- marks(data, dfok=FALSE)
  if(missing(FUN))
    stop("FUN is missing")
  y <- split(data, INDICES)
  z <- list()
  for(i in seq_along(y))
    z[[i]] <- FUN(y[[i]], ...)
  names(z) <- names(y)
  z <- as.solist(z, demote=TRUE)
  return(z)
}

#
#	wingeom.R	Various geometrical computations in windows
#
#	$Revision: 4.136 $	$Date: 2020/03/16 10:28:51 $
#

volume.owin <- function(x) { area.owin(x) }

area <- function(w) UseMethod("area")

area.default <- function(w) area.owin(as.owin(w))

area.owin <- function(w) {
    stopifnot(is.owin(w))
        switch(w$type,
               rectangle = {
		width <- abs(diff(w$xrange))
		height <- abs(diff(w$yrange))
		area <- width * height
               },
               polygonal = {
                 area <- sum(unlist(lapply(w$bdry, Area.xypolygon)))
               },
               mask = {
                 pixelarea <- abs(w$xstep * w$ystep)
                 npixels <- sum(w$m)
                 area <- pixelarea * npixels
               },
               stop("Unrecognised window type")
        )
        return(area)
}

perimeter <- function(w) {
  w <- as.owin(w)
  switch(w$type,
         rectangle = {
           return(2*(diff(w$xrange)+diff(w$yrange)))
         },
         polygonal={
           return(sum(lengths_psp(edges(w))))
         },
         mask={
           p <- as.polygonal(w)
           if(is.null(p)) return(NA)
           delta <- sqrt(w$xstep^2 + w$ystep^2)
           p <- simplify.owin(p, delta * 1.15)
           return(sum(lengths_psp(edges(p))))
         })
  return(NA)
}

framebottomleft <- function(w) {
  f <- Frame(w)
  c(f$xrange[1L], f$yrange[1L])
}

sidelengths.owin <- function(x) {
  if(x$type != "rectangle")
    warning("Computing the side lengths of a non-rectangular window")
  with(x, c(diff(xrange), diff(yrange)))
}

shortside.owin <- function(x) { min(sidelengths(x)) }

eroded.areas <- function(w, r, subset=NULL) {
  w <- as.owin(w)
  if(!is.null(subset) && !is.mask(w))
    w <- as.mask(w)
  switch(w$type,
         rectangle = {
           width <- abs(diff(w$xrange))
           height <- abs(diff(w$yrange))
           areas <- pmax(width - 2 * r, 0) * pmax(height - 2 * r, 0)
         },
         polygonal = {
           ## warning("Approximating polygonal window by digital image")
           w <- as.mask(w)
           areas <- eroded.areas(w, r)
         },
         mask = {
           ## distances from each pixel to window boundary
           b <- if(is.null(subset)) bdist.pixels(w, style="matrix") else 
                bdist.pixels(w)[subset, drop=TRUE, rescue=FALSE]
           ## histogram breaks to satisfy hist()
           Bmax <- max(b, r)
           breaks <- c(-1,r,Bmax+1)
           ## histogram of boundary distances
           h <- hist(b, breaks=breaks, plot=FALSE)$counts
           ## reverse cumulative histogram
           H <- revcumsum(h)
           ## drop first entry corresponding to r=-1
           H <- H[-1]
           ## convert count to area
           pixarea <- w$xstep * w$ystep
           areas <- pixarea * H
         },
         stop("unrecognised window type")
         )
  areas
}	

even.breaks.owin <- function(w) {
  verifyclass(w, "owin")
  Rmax <- diameter(w)
  make.even.breaks(bmax=Rmax, npos=128)
}

unit.square <- function() { owin(c(0,1),c(0,1)) }

square <- function(r=1, unitname=NULL) {
  stopifnot(is.numeric(r))
  if(is.numeric(unitname) && length(unitname) == 1 && length(r) == 1) {
    #' common error
    warning("Interpreting square(a, b) as square(c(a,b))", call.=FALSE)
    r <- c(r, unitname)
    unitname <- NULL
  }
  if(!all(is.finite(r)))
    stop("argument r is NA or infinite")
  if(length(r) == 1) {
    stopifnot(r > 0)
    r <- c(0,r)
  } else if(length(r) == 2) {
    stopifnot(r[1L] < r[2L])
  } else stop("argument r must be a single number, or a vector of length 2")
  owin(r,r, unitname=unitname)
}

# convert polygonal window to mask window 
owinpoly2mask <- function(w, rasta, check=TRUE) {
  if(check) {
    verifyclass(w, "owin")
    stopifnot(w$type == "polygonal")
    verifyclass(rasta, "owin")
    stopifnot(rasta$type == "mask")
  }
  
  bdry <- w$bdry

  x0    <- rasta$xcol[1L]
  y0    <- rasta$yrow[1L]
  xstep <- rasta$xstep
  ystep <- rasta$ystep
  dimyx <- rasta$dim
  nx    <- dimyx[2L]
  ny    <- dimyx[1L]

  epsilon <- with(.Machine, double.base^floor(double.ulp.digits/2))

  score <- numeric(nx*ny)
  
  for(i in seq_along(bdry)) {
    p <- bdry[[i]]
    xp <- p$x
    yp <- p$y
    np <- length(p$x)
    # repeat last vertex
    xp <- c(xp, xp[1L])
    yp <- c(yp, yp[1L])
    np <- np + 1
    # rescale coordinates so that pixels are at integer locations
    xp <- (xp - x0)/xstep
    yp <- (yp - y0)/ystep
    # avoid exact integer locations for vertices
    whole <- (ceiling(xp) == floor(xp))
    xp[whole] <-  xp[whole] + epsilon
    whole <- (ceiling(yp) == floor(yp))
    yp[whole] <-  yp[whole] + epsilon
    # call C
    z <- .C("poly2imI",
            xp=as.double(xp),
            yp=as.double(yp),
            np=as.integer(np),
            nx=as.integer(nx),
            ny=as.integer(ny),
            out=as.integer(integer(nx * ny)),
            PACKAGE="spatstat")
    if(i == 1)
      score <- z$out
    else 
      score <- score + z$out
  }
  status <- (score != 0)
  out <- owin(rasta$xrange, rasta$yrange, mask=matrix(status, ny, nx))
  return(out)
}



overlap.owin <- function(A, B) {
  # compute the area of overlap between two windows
  
  # check units
  if(!compatible(unitname(A), unitname(B)))
    warning("The two windows have incompatible units of length")
  
  At <- A$type
  Bt <- B$type
  if(At=="rectangle" && Bt=="rectangle") {
    xmin <- max(A$xrange[1L],B$xrange[1L])
    xmax <- min(A$xrange[2L],B$xrange[2L])
    if(xmax <= xmin) return(0)
    ymin <- max(A$yrange[1L],B$yrange[1L])
    ymax <- min(A$yrange[2L],B$yrange[2L])
    if(ymax <= ymin) return(0)
    return((xmax-xmin) * (ymax-ymin))
  }
  if((At=="rectangle" && Bt=="polygonal")
     || (At=="polygonal" && Bt=="rectangle")
     || (At=="polygonal" && Bt=="polygonal"))
  {
    AA <- as.polygonal(A)$bdry
    BB <- as.polygonal(B)$bdry
    area <- 0
    for(i in seq_along(AA))
      for(j in seq_along(BB))
        area <- area + overlap.xypolygon(AA[[i]], BB[[j]])
    # small negative numbers can occur due to numerical error
    return(max(0, area))
  }
  if(At=="mask") {
    # count pixels in A that belong to B
    pixelarea <- abs(A$xstep * A$ystep)
    rxy <- rasterxy.mask(A, drop=TRUE)
    x <- rxy$x
    y <- rxy$y
    ok <- inside.owin(x, y, B) 
    return(pixelarea * sum(ok))
  }
  if(Bt== "mask") {
    # count pixels in B that belong to A
    pixelarea <- abs(B$xstep * B$ystep)
    rxy <- rasterxy.mask(B, drop=TRUE)
    x <- rxy$x
    y <- rxy$y
    ok <- inside.owin(x, y, A)
    return(pixelarea * sum(ok))
  }
  stop("Internal error")
}

#
#  subset operator for window
#

"[.owin" <- 
function(x, i, ...) {
  if(!missing(i) && !is.null(i)) {
    if(is.im(i) && i$type == "logical") {
      # convert to window
      i <- as.owin(eval.im(ifelse1NA(i)))
    } else stopifnot(is.owin(i))
    x <- intersect.owin(x, i, fatal=FALSE)
  }
  return(x)
}
    
#
#
#  Intersection and union of windows
#
#

intersect.owin <- function(..., fatal=FALSE, p) {
  argh <- list(...)
  ## p is a list of arguments to polyclip::polyclip
  if(missing(p) || is.null(p)) p <- list()
  ## handle 'solist' objects
  argh <- expandSpecialLists(argh, "solist")
  rasterinfo <- list()
  if(length(argh) > 0) {
    # explicit arguments controlling raster info
    israster <- names(argh) %in% names(formals(as.mask))
    if(any(israster)) {
      rasterinfo <- argh[israster]
      ## remaining arguments
      argh <- argh[!israster]
    }
  }
  ## look for window arguments
  isowin <- as.logical(sapply(argh, is.owin))
  if(any(!isowin))
    warning("Some arguments were not windows")
  argh <- argh[isowin]
  nwin <- length(argh)

  if(nwin == 0) {
    warning("No windows were given")
    return(NULL)
  }

  ## at least one window
  A <- argh[[1L]]
  if(is.empty(A)) {
    if(fatal) stop("Intersection is empty", call.=FALSE)
    return(A)
  }
  if(nwin == 1) return(A)

  ## at least two windows
  B <- argh[[2L]]
  if(is.empty(B)) {
    if(fatal) stop("Intersection is empty", call.=FALSE)
    return(B)
  }
  
  if(nwin > 2) {
    ## handle union of more than two windows
    windows <- argh[-c(1,2)]
    ## determine a common set of parameters for polyclip
    p <- commonPolyclipArgs(A, B, do.call(boundingbox, windows), p=p)
    ## absorb all windows into B
    for(i in seq_along(windows)) {
      B <- do.call(intersect.owin,
                   append(list(B, windows[[i]], p=p, fatal=fatal),
                          rasterinfo))
      if(is.empty(B)) return(B)
    }
  }

  ## There are now only two windows, which are not empty.
  if(identical(A, B))
    return(A)

  # check units
  if(!compatible(unitname(A), unitname(B)))
    warning("The two windows have incompatible units of length")
  uname <- harmonise(unitname(A), unitname(B), single=TRUE)

  # determine intersection of x and y ranges
  xr <- intersect.ranges(A$xrange, B$xrange, fatal=fatal)
  yr <- intersect.ranges(A$yrange, B$yrange, fatal=fatal)
  if(!fatal && (is.null(xr) || is.null(yr)))
    return(emptywindow(A))
  #' non-empty intersection of Frames
  C <- owin(xr, yr, unitname=uname)

  # Determine type of intersection
  Arect <- is.rectangle(A)
  Brect <- is.rectangle(B)
#  Apoly <- is.polygonal(A)
#  Bpoly <- is.polygonal(B)
  Amask <- is.mask(A)
  Bmask <- is.mask(B)
  
  # Rectangular case
  if(Arect && Brect)
    return(C)

  if(!Amask && !Bmask) {
    ####### Result is polygonal ############
    a <- lapply(as.polygonal(A)$bdry, reverse.xypolygon)
    b <- lapply(as.polygonal(B)$bdry, reverse.xypolygon)
    ab <- do.call(polyclip::polyclip,
                  append(list(a, b, "intersection",
                              fillA="nonzero", fillB="nonzero"),
                         p))
    if(length(ab)==0) {
      if(fatal) stop("Intersection is empty", call.=FALSE)
      return(emptywindow(C))
    }
    # ensure correct polarity
    totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
    if(totarea < 0)
      ab <- lapply(ab, reverse.xypolygon)
    AB <- owin(poly=ab, check=FALSE, unitname=uname)
    AB <- rescue.rectangle(AB)
    return(AB)
  }

  ######### Result is a mask ##############
  
  # Restrict domain where possible 
  if(Arect)
    A <- C
  if(Brect)
    B <- C
  if(Amask)
    A <- trim.mask(A, C)
  if(Bmask)
    B <- trim.mask(B, C)

  #' trap empty windows
  if(is.empty(A)) {
    if(fatal) stop("Intersection is empty", call.=FALSE)
    return(A)
  }
  if(is.empty(B)) {
    if(fatal) stop("Intersection is empty", call.=FALSE)
    return(B)
  }
  
  # Did the user specify the pixel raster?
  if(length(rasterinfo) > 0) {
    # convert to masks with specified parameters, and intersect
    if(Amask) {
      A <- do.call(as.mask, append(list(A), rasterinfo))
      AB <- restrict.mask(A, B)
      if(fatal && is.empty(AB)) stop("Intersection is empty", call.=FALSE)
      return(AB)
    } else {
      B <- do.call(as.mask, append(list(B), rasterinfo))
      BA <- restrict.mask(B,A)
      if(fatal && is.empty(BA)) stop("Intersection is empty", call.=FALSE)
      return(BA)
    }
  } 
  
  # One mask and one rectangle?
  if(Arect && Bmask)
      return(B)
  if(Amask && Brect)
      return(A)

  # One mask and one polygon?
  if(Amask && !Bmask) {
    AB <- restrict.mask(A, B)
    if(fatal && is.empty(AB)) stop("Intersection is empty", call.=FALSE)
    return(AB)
  }
  if(!Amask && Bmask) {
    BA <- restrict.mask(B, A)
    if(fatal && is.empty(BA)) stop("Intersection is empty", call.=FALSE)
    return(BA)
  }

  # Two existing masks?
  if(Amask && Bmask) {
    # choose the finer one
    AB <- if(A$xstep <= B$xstep) restrict.mask(A, B) else restrict.mask(B, A)
    if(fatal && is.empty(AB)) stop("Intersection is empty", call.=FALSE)
    return(AB)
  }

  stop("Internal error: never reached")
#  # No existing masks. No clipping applied so far.
#  # Convert one window to a mask with default pixel raster, and intersect.
#  if(Arect) {
#    A <- as.mask(A)
#    AB <- restrict.mask(A, B)
#    if(fatal && is.empty(AB)) stop("Intersection is empty", call.=FALSE)
#    return(AB)
#  } else {
#    B <- as.mask(B)
#    BA <- restrict.mask(B, A)
#    if(fatal && is.empty(BA)) stop("Intersection is empty", call.=FALSE)
#    return(BA)
#  }
}


union.owin <- function(..., p) {
  argh <- list(...)
  ## weed out NULL arguments
  argh <- argh[!sapply(argh, is.null)]
  ## p is a list of arguments to polyclip::polyclip
  if(missing(p) || is.null(p)) p <- list()
  ## handle 'solist' objects
  argh <- expandSpecialLists(argh, "solist")
  rasterinfo <- list()
  if(length(argh) > 0) {
    ## arguments controlling raster info
    israster <- names(argh) %in% names(formals(as.mask))
    if(any(israster)) {
      rasterinfo <- argh[israster]
      ## remaining arguments
      argh <- argh[!israster]
    }
  }
  ## look for window arguments
  isowin <- as.logical(sapply(argh, is.owin))
  if(any(!isowin))
    warning("Some arguments were not windows")
  argh <- argh[isowin]
  ## 
  nwin <- length(argh)
  if(nwin == 0) {
    warning("No windows were given")
    return(NULL)
  }
  ## find non-empty ones
  if(any(isemp <- sapply(argh, is.empty)))
    argh <- argh[!isemp]
  nwin <- length(argh)
  if(nwin == 0) {
    warning("All windows were empty")
    return(NULL)
  }
  ## at least one window
  A <- argh[[1L]]
  if(nwin == 1) return(A)

  ## more than two windows
  if(nwin > 2) {
    ## check if we need polyclip
    somepoly <- !all(sapply(argh, is.mask))
    if(somepoly) {
      ## determine a common set of parameters for polyclip
      p <- commonPolyclipArgs(do.call(boundingbox, argh), p=p)
      ## apply these parameters now to avoid numerical errors
      argh <- applyPolyclipArgs(argh, p=p)
      A <- argh[[1L]]
    } 
    ## absorb all windows into A without rescaling
    nullp <- list(eps=1, x0=0, y0=0)
    for(i in 2:nwin) 
      A <- do.call(union.owin,
                   append(list(A, argh[[i]], p=nullp),
                          rasterinfo))
    if(somepoly) {
      ## undo rescaling
      A <- reversePolyclipArgs(A, p=p)
    }
    return(A)
  }

  ## Exactly two windows
  B <- argh[[2L]]
  if(identical(A, B))
    return(A)

  ## check units
  if(!compatible(unitname(A), unitname(B)))
    warning("The two windows have incompatible units of length")
  uname <- harmonise(unitname(A), unitname(B), single=TRUE)
  
  ## Determine type of intersection
  
##  Arect <- is.rectangle(A)
##  Brect <- is.rectangle(B)
##  Apoly <- is.polygonal(A)
##  Bpoly <- is.polygonal(B)
  Amask <- is.mask(A)
  Bmask <- is.mask(B)

  ## Create a rectangle to contain the result
  
  C <- owin(range(A$xrange, B$xrange),
            range(A$yrange, B$yrange),
            unitname=uname)

  if(!Amask && !Bmask) {
    ####### Result is polygonal (or rectangular) ############
    a <- lapply(as.polygonal(A)$bdry, reverse.xypolygon)
    b <- lapply(as.polygonal(B)$bdry, reverse.xypolygon)
    ab <- do.call(polyclip::polyclip,
                  append(list(a, b, "union",
                              fillA="nonzero", fillB="nonzero"),
                         p))
    if(length(ab) == 0)
      return(emptywindow(C))
    ## ensure correct polarity
    totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
    if(totarea < 0)
      ab <- lapply(ab, reverse.xypolygon)
    AB <- owin(poly=ab, check=FALSE, unitname=uname)
    AB <- rescue.rectangle(AB)
    return(AB)
  }

  ####### Result is a mask ############

  ## Determine pixel raster parameters
  if(length(rasterinfo) == 0) {
    rasterinfo <-
      if(Amask)
        list(xy=list(x=as.numeric(prolongseq(A$xcol, C$xrange)),
               y=as.numeric(prolongseq(A$yrow, C$yrange))))
      else if(Bmask)
        list(xy=list(x=as.numeric(prolongseq(B$xcol, C$xrange)),
               y=as.numeric(prolongseq(B$yrow, C$yrange))))
      else
        list()
  }

  ## Convert C to mask
  C <- do.call(as.mask, append(list(w=C), rasterinfo))

  rxy <- rasterxy.mask(C)
  x <- rxy$x
  y <- rxy$y
  ok <- inside.owin(x, y, A) | inside.owin(x, y, B)

  if(all(ok)) {
    ## result is a rectangle
    C <- as.rectangle(C)
  } else {
    ## result is a mask
    C$m[] <- ok
  }
  return(C)
}

setminus.owin <- function(A, B, ..., p) {
  if(is.null(B)) return(A)
  verifyclass(B, "owin")
  if(is.null(A)) return(emptywindow(Frame(B)))
  verifyclass(A, "owin")
  if(is.empty(A) || is.empty(B)) return(A)
  if(identical(A, B))
    return(emptywindow(Frame(A)))

  ## p is a list of arguments to polyclip::polyclip
  if(missing(p) || is.null(p)) p <- list()

  ## check units
  if(!compatible(unitname(A), unitname(B)))
    warning("The two windows have incompatible units of length")
  uname <- harmonise(unitname(A), unitname(B), single=TRUE)
  
  ## Determine type of arguments
  
  Arect <- is.rectangle(A)
  Brect <- is.rectangle(B)
##  Apoly <- is.polygonal(A)
##  Bpoly <- is.polygonal(B)
  Amask <- is.mask(A)
  Bmask <- is.mask(B)

  ## Case where A and B are both rectangular
  if(Arect && Brect) {
    if(is.subset.owin(A, B))
      return(emptywindow(B))
    C <- intersect.owin(A, B, fatal=FALSE)
    if(is.null(C) || is.empty(C)) return(A)
    return(complement.owin(C, A))
  }
    
  ## Polygonal case

  if(!Amask && !Bmask) {
    ####### Result is polygonal ############
    a <- lapply(as.polygonal(A)$bdry, reverse.xypolygon)
    b <- lapply(as.polygonal(B)$bdry, reverse.xypolygon)
    ab <- do.call(polyclip::polyclip,
                  append(list(a, b, "minus",
                              fillA="nonzero", fillB="nonzero"),
                         p))
    if(length(ab) == 0)
      return(emptywindow(B))
    ## ensure correct polarity
    totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
    if(totarea < 0)
      ab <- lapply(ab, reverse.xypolygon)
    AB <- owin(poly=ab, check=FALSE, unitname=uname)
    AB <- rescue.rectangle(AB)
    return(AB)
  }

  ####### Result is a mask ############

  ## Determine pixel raster parameters
  rasterinfo <- 
    if((length(list(...)) > 0))
      list(...)
    else if(Amask)
      list(xy=list(x=A$xcol,
                   y=A$yrow))
    else if(Bmask)
      list(xy=list(x=B$xcol,
                   y=B$yrow))
    else
      list()

  ## Convert A to mask
  AB <- do.call(as.mask, append(list(w=A), rasterinfo))

  rxy <- rasterxy.mask(AB)
  x <- rxy$x
  y <- rxy$y
  ok <- inside.owin(x, y, A) & !inside.owin(x, y, B)

  if(!all(ok))
    AB$m[] <- ok
  else
    AB <- rescue.rectangle(AB)

  return(AB)
}

## auxiliary functions

commonPolyclipArgs <- function(..., p=NULL) {
  # compute a common resolution for polyclip operations
  # on several windows
  if(!is.null(p) && !is.null(p$eps) && !is.null(p$x0) && !is.null(p$y0))
    return(p)
  bb <- boundingbox(...)
  xr <- bb$xrange
  yr <- bb$yrange
  eps <- p$eps %orifnull% max(diff(xr), diff(yr))/(2^31)
  x0  <- p$x0  %orifnull% mean(xr)
  y0  <- p$y0  %orifnull% mean(yr)
  return(list(eps=eps, x0=x0, y0=y0))
}

applyPolyclipArgs <- function(x, p=NULL) {
  if(is.null(p)) return(x)
  y <- lapply(x, shift, vec=-c(p$x0, p$y0))
  z <- lapply(y, scalardilate, f=1/p$eps)
  return(z)
}

reversePolyclipArgs <- function(x, p=NULL) {
  if(is.null(p)) return(x)
  y <- scalardilate(x, f=p$eps)
  z <- shift(y, vec=c(p$x0, p$y0))
  return(z)
}

trim.mask <- function(M, R, tolerant=TRUE) {
    ## M is a mask,
    ## R is a rectangle

    ## Ensure R is a subset of bounding rectangle of M
    R <- owin(intersect.ranges(M$xrange, R$xrange),
              intersect.ranges(M$yrange, R$yrange))
    
    ## Deal with very thin rectangles
    if(tolerant) {
      R$xrange <- adjustthinrange(R$xrange, M$xstep, M$xrange)
      R$yrange <- adjustthinrange(R$yrange, M$ystep, M$yrange)
    }

    ## Extract subset of image grid
    yrowok <- inside.range(M$yrow, R$yrange)
    xcolok <- inside.range(M$xcol, R$xrange)
    if((ny <- sum(yrowok)) == 0 || (nx <- sum(xcolok)) == 0) 
      return(emptywindow(R))
    Z <- M
    Z$xrange <- R$xrange
    Z$yrange <- R$yrange
    Z$yrow <- M$yrow[yrowok]
    Z$xcol <- M$xcol[xcolok]
    Z$m <- M$m[yrowok, xcolok]
    if(ny < 2 || nx < 2)
      Z$m <- matrix(Z$m, nrow=ny, ncol=nx)
    Z$dim <- dim(Z$m)
    return(Z)
}

restrict.mask <- function(M, W) {
  ## M is a mask, W is any window
  stopifnot(is.mask(M))
  stopifnot(inherits(W, "owin"))
  if(is.rectangle(W))
    return(trim.mask(M, W))
  M <- trim.mask(M, as.rectangle(W))
  ## Determine which pixels of M are inside W
  rxy <- rasterxy.mask(M, drop=TRUE)
  x <- rxy$x
  y <- rxy$y
  ok <- inside.owin(x, y, W)
  Mm <- M$m
  Mm[Mm] <- ok
  M$m <- Mm
  return(M)
}

# SUBSUMED IN rmhexpand.R
# expand.owin <- function(W, f=1) {
#
#  # expand bounding box of 'win'
#  # by factor 'f' in **area**
#  if(f <= 0)
#    stop("f must be > 0")
#  if(f == 1)
#    return(W)
#  bb <- boundingbox(W)
#  xr <- bb$xrange
#  yr <- bb$yrange
#  fff <- (sqrt(f) - 1)/2
#  Wexp <- owin(xr + fff * c(-1,1) * diff(xr),
#               yr + fff * c(-1,1) * diff(yr),
#               unitname=unitname(W))
#  return(Wexp)
#}

trim.rectangle <- function(W, xmargin=0, ymargin=xmargin) {
  if(!is.rectangle(W))
    stop("Internal error: tried to trim margin off non-rectangular window")
  xmargin <- ensure2vector(xmargin)
  ymargin <- ensure2vector(ymargin)
  if(any(xmargin < 0) || any(ymargin < 0))
    stop("values of xmargin, ymargin must be nonnegative")
  if(sum(xmargin) > diff(W$xrange))
    stop("window is too small to cut off margins of the width specified")
  if(sum(ymargin) > diff(W$yrange))
    stop("window is too small to cut off margins of the height specified")
  owin(W$xrange + c(1,-1) * xmargin,
       W$yrange + c(1,-1) * ymargin,
       unitname=unitname(W))
}

grow.rectangle <- function(W, xmargin=0, ymargin=xmargin, fraction=NULL) {
  if(!is.null(fraction)) {
    fraction <- ensure2vector(fraction)
    if(any(fraction < 0)) stop("fraction must be non-negative")
    if(missing(xmargin)) xmargin <- fraction[1L] * diff(W$xrange)
    if(missing(ymargin)) ymargin <- fraction[2L] * diff(W$yrange)
  }
  xmargin <- ensure2vector(xmargin)
  ymargin <- ensure2vector(ymargin)
  if(any(xmargin < 0) || any(ymargin < 0))
    stop("values of xmargin, ymargin must be nonnegative")
  owin(W$xrange + c(-1,1) * xmargin,
       W$yrange + c(-1,1) * ymargin,
       unitname=unitname(W))
}

grow.mask <- function(M, xmargin=0, ymargin=xmargin) {
  stopifnot(is.mask(M))
  m <- as.matrix(M)
  Rplus <- grow.rectangle(as.rectangle(M), xmargin, ymargin)
  ## extend the raster
  xcolplus <- prolongseq(M$xcol, Rplus$xrange)
  yrowplus <- prolongseq(M$yrow, Rplus$yrange)
  mplus <- matrix(FALSE, length(yrowplus), length(xcolplus))
  ## pad out the mask entries
  nleft <- attr(xcolplus, "nleft")
  nright <- attr(xcolplus, "nright")
  nbot <- attr(yrowplus, "nleft")
  ntop <- attr(yrowplus, "nright")
  mplus[ (nbot+1):(length(yrowplus)-ntop),
         (nleft+1):(length(xcolplus)-nright) ] <- m
  ## pack up
  result <- owin(xrange=Rplus$xrange,
                 yrange=Rplus$yrange,
                 xcol=as.numeric(xcolplus),
                 yrow=as.numeric(yrowplus),
                 mask=mplus,
                 unitname=unitname(M))
  return(result)
}
  
bdry.mask <- function(W) {
  verifyclass(W, "owin")
  W <- as.mask(W)
  m <- W$m
  nr <- nrow(m)
  nc <- ncol(m)
  if(!spatstat.options('Cbdrymask')) {
    ## old interpreted code
    b <-     (m != rbind(FALSE,       m[-nr, ]))
    b <- b | (m != rbind(m[-1, ], FALSE))
    b <- b | (m != cbind(FALSE,       m[, -nc]))
    b <- b | (m != cbind(m[, -1], FALSE))
  } else {
    b <- integer(nr * nc)
    z <- .C("bdrymask",
            nx = as.integer(nc),
            ny = as.integer(nr),
            m = as.integer(m),
            b = as.integer(b),
            PACKAGE = "spatstat")
    b <- matrix(as.logical(z$b), nr, nc)
  }
  W$m <- b
  return(W)
}

nvertices <- function(x, ...) {
  UseMethod("nvertices")
}

nvertices.default <- function(x, ...) {
  v <- vertices(x)
  vx <- v$x
  n <- if(is.null(vx)) NA else length(vx)
  return(n)
}

nvertices.owin <- function(x, ...) {
  if(is.empty(x))
    return(0)
  n <- switch(x$type,
              rectangle=4,
              polygonal=sum(lengths(lapply(x$bdry, getElement, name="x"))),
              mask=sum(bdry.mask(x)$m))
  return(n)
}

vertices <- function(w) {
  UseMethod("vertices")
}

vertices.owin <- function(w) {
  verifyclass(w, "owin")
  if(is.empty(w))
    return(NULL)
  switch(w$type,
         rectangle={
           xr <- w$xrange
           yr <- w$yrange
           vert <- list(x=xr[c(1,2,2,1)], y=yr[c(1,1,2,2)])
         },
         polygonal={
           vert <- do.call(concatxy,w$bdry)
         },
         mask={
           bm <- bdry.mask(w)$m
           rxy <- rasterxy.mask(w)
           xx <- rxy$x
           yy <- rxy$y
           vert <- list(x=as.vector(xx[bm]),
                        y=as.vector(yy[bm]))
         })
  return(vert)
}

diameter <- function(x) { UseMethod("diameter") }

diameter.owin <- function(x) {
  w <- as.owin(x)
  if(is.empty(w))
    return(NULL)
  vert <- vertices(w)
  if(length(vert$x) > 3) {
    # extract convex hull
    h <- with(vert, chull(x, y))
    vert <- with(vert, list(x=x[h], y=y[h]))
  }
  d <- pairdist(vert, squared=TRUE)
  return(sqrt(max(d)))
}

##    radius of inscribed circle

inradius <- function(W) {
  stopifnot(is.owin(W))
  if(W$type == "rectangle") diameter(W)/2 else max(distmap(W, invert=TRUE))
}

  
incircle <- function(W) {
  # computes the largest circle contained in W
  verifyclass(W, "owin")
  if(is.empty(W))
    return(NULL)
  if(is.rectangle(W)) {
    xr <- W$xrange
    yr <- W$yrange
    x0 <- mean(xr)
    y0 <- mean(yr)
    radius <- min(diff(xr), diff(yr))/2
    return(list(x=x0, y=y0, r=radius))
  }
  # compute distance to boundary
  D <- distmap(W, invert=TRUE)
  D <- D[W, drop=FALSE]
  # find maximum distance
  v <- D$v
  ok <- !is.na(v)
  Dvalues <- as.vector(v[ok])
  if(length(Dvalues) == 0) return(NULL)
  Dmax <- max(Dvalues)
  # find location of maximum
  locn <- which.max(Dvalues)
  locrow <- as.vector(row(v)[ok])[locn]
  loccol <- as.vector(col(v)[ok])[locn]
  x0 <- D$xcol[loccol]
  y0 <- D$yrow[locrow]
  if(is.mask(W)) {
    # radius could be one pixel diameter shorter than Dmax
    Dpixel <- sqrt(D$xstep^2 + D$ystep^2)
    radius <- max(0, Dmax - Dpixel)
  } else radius <- Dmax
  return(list(x=x0, y=y0, r=radius))
}

inpoint <- function(W) {
  # selects a point that is always inside the window.
  verifyclass(W, "owin")
  if(is.empty(W))
    return(NULL)
  if(is.rectangle(W))
    return(c(mean(W$xrange), mean(W$yrange)))
  if(is.polygonal(W)) {
    xy <- centroid.owin(W)
    if(inside.owin(xy$x, xy$y, W))
      return(xy)
  }
  W <- as.mask(W)
  Mm <- W$m
  if(!any(Mm)) return(NULL)
  Mrow <- as.vector(row(Mm)[Mm])
  Mcol <- as.vector(col(Mm)[Mm])
  selectmiddle <- function(x) { x[ceiling(length(x)/2)] }
  midcol <- selectmiddle(Mcol)
  midrow <- selectmiddle(Mrow[Mcol==midcol])
  x <- W$xcol[midcol]
  y <- W$yrow[midrow]
  return(c(x,y))
}

simplify.owin <- function(W, dmin) {
  verifyclass(W, "owin")
  if(is.empty(W))
    return(W)
  W <- as.polygonal(W)
  W$bdry <- lapply(W$bdry, simplify.xypolygon, dmin=dmin)
  return(W)
}

  
is.convex <- function(x) {
  verifyclass(x, "owin")
  if(is.empty(x))
    return(TRUE)
  switch(x$type,
         rectangle={return(TRUE)},
         polygonal={
           b <- x$bdry
           if(length(b) > 1)
             return(FALSE)
           b <- b[[1L]]
           xx <- b$x
           yy <- b$y
           ch <- chull(xx,yy)
           return(length(ch) == length(xx))
         },
         mask={
           v <- vertices(x)
           v <- as.ppp(v, W=as.rectangle(x))
           ch <- convexhull.xy(v)
           edg <- edges(ch)
           edgedist <- nncross(v, edg, what="dist")
           pixdiam <- sqrt(x$xstep^2 + x$ystep^2)
           return(all(edgedist <= pixdiam))
         })
  return(as.logical(NA))
}

convexhull <- function(x) {
  if(inherits(x, "owin")) 
    v <- vertices(x)
  else if(inherits(x, "psp"))
    v <- endpoints.psp(x)
  else if(inherits(x, "ppp"))
    v <- x
  else {
    x <- as.owin(x)
    v <- vertices(x)
  }
  b <- as.rectangle(x)
  if(is.empty(x))
    return(emptywindow(b))
  ch <- convexhull.xy(v)
  out <- rebound.owin(ch, b)
  return(out)
}

  
is.empty <- function(x) { UseMethod("is.empty") }

is.empty.default <- function(x) { length(x) == 0 }

is.empty.owin <- function(x) {
  switch(x$type,
         rectangle=return(FALSE),
         polygonal=return(length(x$bdry) == 0),
         mask=return(!any(x$m)))
  return(NA)
}

emptywindow <- function(w) {
  w <- as.owin(w)
  out <- owin(w$xrange, w$yrange, poly=list(), unitname=unitname(w))
  return(out)
}

discs <- function(centres, radii=marks(centres)/2, ..., 
                  separate=FALSE, mask=FALSE, trim=TRUE,
                  delta=NULL, npoly=NULL) {
  stopifnot(is.ppp(centres))
  n <- npoints(centres)
  if(n == 0) return(emptywindow(Frame(centres)))
  check.nvector(radii, npoints(centres), oneok=TRUE)
  stopifnot(all(radii > 0))
  
  if(sameradius <- (length(radii) == 1)) 
    radii <- rep(radii, npoints(centres))

  if(!separate && mask) {
    #' compute pixel approximation
    M <- as.mask(Window(centres), ...)
    z <- .C("discs2grid",
            nx    = as.integer(M$dim[2L]),
            x0    = as.double(M$xcol[1L]),
            xstep = as.double(M$xstep),  
            ny    = as.integer(M$dim[1L]),
            y0    = as.double(M$yrow[1L]),
            ystep = as.double(M$ystep), 
            nd    = as.integer(n),
            xd    = as.double(centres$x),
            yd    = as.double(centres$y),
            rd    = as.double(radii), 
            out   = as.integer(integer(prod(M$dim))),
            PACKAGE = "spatstat")
    M$m[] <- as.logical(z$out)
    return(M)
  }
  #' construct a list of discs
  D <- list()
  if(!sameradius && length(unique(radii)) > 1) {
    if(is.null(delta) && is.null(npoly)) {
      ra <- range(radii)
      rr <- ra[2L]/ra[1L]
      mm <- ceiling(128/rr)
      mm <- max(16, mm) ## equals 16 unless ra[2]/ra[1] < 8 
      delta <- 2 * pi * ra[1L]/mm
    }
    for(i in 1:n)
      D[[i]] <- disc(centre=centres[i], radius=radii[i],
                     delta=delta, npoly=npoly)
  } else {
    #' congruent discs -- use 'shift'
    W0 <- disc(centre=c(0,0), radius=radii[1L],
               delta=delta, npoly=npoly)
    for(i in 1:n) 
      D[[i]] <- shift(W0, vec=centres[i])
  } 
  D <- as.solist(D)
  #' return list of discs?
  if(separate)
    return(D)
  #' return union of discs
  W <- union.owin(D)
  if(trim) W <- intersect.owin(W, Window(centres))
  return(W)
}

harmonise.owin <- harmonize.owin <- function(...) {
  argz <- list(...)
  wins <- solapply(argz, as.owin)
  if(length(wins) < 2L) return(wins)
  ismask <- sapply(wins, is.mask)
  if(!any(ismask)) return(wins)
  comgrid <- do.call(commonGrid, lapply(argz, as.owin))
  result <- solapply(argz, "[", i=comgrid, drop=FALSE)
  return(result)
}

#
#
#   rescale.R
#
#   $Revision: 1.8 $ $Date: 2019/02/10 06:42:26 $
#
#

rescale <- function(X, s, unitname) {
  UseMethod("rescale")
}

rescale.ppp <- function(X, s, unitname) {
  if(missing(unitname)) unitname <- NULL
  if(missing(s) || is.null(s)) s <- 1/unitname(X)$multiplier
  Y <- affine.ppp(X, mat=diag(c(1/s,1/s)), rescue=FALSE)
  unitname(Y) <- rescale(unitname(X), s, unitname)
  return(Y)
}

rescale.owin <- function(X, s, unitname) {
  if(missing(unitname)) unitname <- NULL
  if(missing(s) || is.null(s)) s <- 1/unitname(X)$multiplier
  Y <- affine.owin(X, mat=diag(c(1/s,1/s)), rescue=FALSE)
  unitname(Y) <- rescale(unitname(X), s, unitname)
  return(Y)
}

rescale.im <- function(X, s, unitname) {
  if(missing(unitname)) unitname <- NULL
  if(missing(s) || is.null(s)) s <- 1/unitname(X)$multiplier
  Y <- X
  Y$xrange <- X$xrange/s
  Y$yrange <- X$yrange/s
  Y$xstep  <- X$xstep/s
  Y$ystep  <- X$ystep/s
  Y$xcol   <- X$xcol/s
  Y$yrow   <- X$yrow/s
  unitname(Y) <- rescale(unitname(X), s, unitname)
  return(Y)
}

rescale.psp <- function(X, s, unitname) {
  if(missing(unitname)) unitname <- NULL
  if(missing(s) || is.null(s)) s <- 1/unitname(X)$multiplier
  Y <- affine.psp(X, mat=diag(c(1/s,1/s)), rescue=FALSE)
  unitname(Y) <- rescale(unitname(X), s, unitname)
  return(Y)
}
  
rescale.unitname <- function(X, s, unitname) {
  if(!missing(unitname) && !is.null(unitname)) return(as.unitname(unitname))
  if(summary(X)$vanilla)
    return(X)
  if(missing(s)) {
    X$multiplier <- 1
  } else {
    if(!is.numeric(s) || length(s) != 1 || s <= 0)
      stop("s should be a positive number")
    X$multiplier <- s * X$multiplier
  }
  return(X)
}

#'
#'      rmhtemper.R
#'
#'   $Revision: 1.4 $  $Date: 2018/10/18 02:07:56 $
#'

reheat <- local({

  expon <- function(x, alpha) {
    if(is.null(x)) return(NULL)
    if(is.numeric(x)) return(x^alpha)
    if(is.im(x)) return(x^alpha)
    if(is.function(x)) {
      f <- x
      g <- function(...) { f(...)^alpha }
      if(!inherits(f, "funxy")) return(g)
      return(funxy(g, W=as.owin(f)))
    }
    if(is.list(x)) return(lapply(x, expon))
    stop("Unrecognised format for x in x^alpha", call.=FALSE)
  }
    
  reheat <- function(model, invtemp) {
    model <- rmhmodel(model)
    cif   <- model$cif
    par   <- model$par
    w     <- model$w
    trend <- model$trend
    types <- model$types

    newtrend <- expon(trend, invtemp)

    rules <- lapply(cif, spatstatRmhInfo)
    temperfuns <- lapply(rules, getElement, name="temper")
    if(any(bad <- sapply(temperfuns, is.null)))
      stop(paste("reheating the", commasep(sQuote(cif[bad])),
                 ngettext(sum(bad), "cif", "cifs"),
                 "is not supported"))

    Ncif <- length(cif)
    if(Ncif == 1) {
      newpar <- temperfuns[[1]](par, invtemp)
    } else {
      newpar <- par
      for(i in 1:Ncif) 
        newpar[[i]] <- temperfuns[[i]](par[[i]], invtemp)
    }
    newmodel <- rmhmodel(cif=cif,
                         par=newpar, trend=newtrend,
                         w=w, types=types)
    return(newmodel)
  } 

  reheat
  
})


rtemper <- function(model, invtemp, nrep, ...,
                    track=FALSE, start=NULL, verbose=FALSE){
  df <- data.frame(invtemp, nrep)
  ndf <- nrow(df)
  X <- NULL
  h <- NULL
  for(i in 1:ndf) {
    if(verbose)
      cat(paste("Step", i, "of", paste0(ndf, ":"),
                "Running", nrep[i], "iterations",
                "at inverse temperature", signif(invtemp[i], 4), "... "))
    model.i <- reheat(model, invtemp[i])
    X <- rmh(model.i, nrep=nrep[i], ...,
             start=start,
             overrideXstart = X,
             overrideclip = (i != ndf),
             track=track,
             saveinfo = FALSE, verbose=FALSE)
    if(track) {
      hnew <- attr(X, "history")
      h <- rbind(h, hnew)
    }
  }
  if(verbose) cat("Done.\n")
  if(track)
    attr(X, "history") <- h
  return(X)
}

#
# $Id: rmh.default.R,v 1.115 2020/01/07 05:53:17 adrian Exp adrian $
#
rmh.default <- function(model,start=NULL,
                        control=default.rmhcontrol(model),
                        ...,
                        nsim=1, drop=TRUE, saveinfo=TRUE,
                        verbose=TRUE, snoop=FALSE) {
#
# Function rmh.  To simulate realizations of 2-dimensional point
# patterns, given the conditional intensity function of the 
# underlying process, via the Metropolis-Hastings algorithm.
#
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
#
#     V A L I D A T E
#
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
  
  if(verbose)
    cat("Checking arguments..")

# validate arguments and fill in the defaults

  model <- rmhmodel(model)
  start <- rmhstart(start)
  if(is.null(control)) {
    control <- default.rmhcontrol(model)
  } else {
    control <- rmhcontrol(control)
  }
  # override 
  if(length(list(...)) > 0)
    control <- update(control, ...)

  control <- rmhResolveControl(control, model)

  saveinfo <- as.logical(saveinfo)
  
  # retain "..." arguments unrecognised by rmhcontrol
  # These are assumed to be arguments of functions defining the trend
  argh <- list(...)
  known <- names(argh) %in% names(formals(rmhcontrol.default))
  f.args <- argh[!known]

#### Multitype models
  
# Decide whether the model is multitype; if so, find the types.

  types <- rmhResolveTypes(model, start, control)
  ntypes <- length(types)
  mtype <- (ntypes > 1)

# If the model is multitype, check that the model parameters agree with types
# and digest them
  
  if(mtype && !is.null(model$check)) {
    model <- rmhmodel(model, types=types)
  } else {
    model$types <- types
  }
  
######## Check for illegal combinations of model, start and control  ########

  # No expansion can be done if we are using x.start

  if(start$given == "x") {
    if(control$expand$force.exp)
      stop("Cannot expand window when using x.start.\n", call.=FALSE)
    control$expand <- .no.expansion
  }

# Warn about a silly value of fixall:
  if(control$fixall & ntypes==1) {
    warning("control$fixall applies only to multitype processes. Ignored.",
            call.=FALSE)
    control$fixall <- FALSE
    if(control$fixing == "n.each.type")
      control$fixing <- "n.total"
  }

  
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
#
#     M O D E L   P A R A M E T E R S
#
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===

#######  Determine windows  ################################

  if(verbose)
    cat("determining simulation windows...")
  
# these may be NULL  
  w.model <- model$w
  x.start <- start$x.start
  trend <- model$trend
  trendy <- !is.null(trend)

  singletrend <- trendy && (is.im(trend) ||
                            is.function(trend) ||
                            (is.numeric(trend) && length(trend) == 1))
  trendlist <- if(singletrend) list(trend) else trend
  
# window implied by trend image, if any
  
  w.trend <- 
    if(is.im(trend))
      as.owin(trend)
    else if(is.list(trend) && any(ok <- unlist(lapply(trend, is.im))))
      as.owin((trend[ok])[[1L]])
    else NULL

##  Clipping window (for final result)
  
  w.clip <-
    if(!is.null(w.model))
      w.model
    else if(!will.expand(control$expand)) {
      if(start$given == "x" && is.ppp(x.start))
        x.start$window
      else if(is.owin(w.trend))
        w.trend
    } else NULL

  if(!is.owin(w.clip))
    stop("Unable to determine window for pattern", call.=FALSE)

  
##  Simulation window 

  xpn <- rmhResolveExpansion(w.clip, control, trendlist, "trend")
  w.sim <- xpn$wsim
  expanded <- xpn$expanded

## Check the fine print   

  if(expanded) {

    if(control$fixing != "none")
      stop(paste("If we're conditioning on the number of points,",
                 "we cannot clip the result to another window."),
           call.=FALSE)

    if(!is.subset.owin(w.clip, w.sim))
      stop("Expanded simulation window does not contain model window",
           call.=FALSE)
  }


#######  Trend  ################################
  
# Check that the expanded window fits inside the window
# upon which the trend(s) live if there are trends and
# if any trend is given by an image.

  if(expanded && !is.null(trend)) {
    trends <- if(is.im(trend)) list(trend) else trend
    images <- unlist(lapply(trends, is.im))
    if(any(images)) {
      iwindows <- lapply(trends[images], as.owin)
      nimages <- length(iwindows)
      misfit <- !sapply(iwindows, is.subset.owin, A=w.sim)
      nmisfit <- sum(misfit)
      if(nmisfit > 1) 
        stop(paste("Expanded simulation window is not contained in",
                   "several of the trend windows.\n",
                   "Bailing out."), call.=FALSE)
      else if(nmisfit == 1) {
        warning(paste("Expanded simulation window is not contained in",
                      if(nimages == 1) "the trend window.\n"
                      else "one of the trend windows.\n",
                      "Expanding to this trend window (only)."),
                call.=FALSE)
        w.sim <- iwindows[[which(misfit)]]
      }
    }
  }

# Extract the 'beta' parameters

  if(length(model$cif) == 1) {
    # single interaction
    beta <- model$C.beta
    betalist <- list(beta)
  } else {
    # hybrid
    betalist <- model$C.betalist
    # multiply beta vectors for each component
    beta <- Reduce("*", betalist)
  }
  
##### .................. CONDITIONAL SIMULATION ...................

#####  
#||   Determine windows for conditional simulation
#||
#||      w.state  = window for the full configuration
#||  
#||      w.sim    = window for the 'free' (random) points
#||

  w.state <- w.sim
  
  condtype <- control$condtype
  x.cond   <- control$x.cond
#  n.cond   <- control$n.cond

  switch(condtype,
         none={
           w.cond <- NULL
         },
         window={
           # conditioning on the realisation inside a subwindow
           w.cond <- as.owin(x.cond)
           # subtract from w.sim
           w.sim <- setminus.owin(w.state, w.cond)
           if(is.empty(w.sim))
             stop(paste("Conditional simulation is undefined;",
                        "the conditioning window",
                        sQuote("as.owin(control$x.cond)"),
                        "covers the entire simulation window"), call.=FALSE)
         },
         Palm={
           # Palm conditioning
           w.cond <- NULL
         })

#####  
#||   Convert conditioning points to appropriate format


  x.condpp <- switch(condtype,
                        none=NULL,
                        window=x.cond,
                        Palm=as.ppp(x.cond, w.state))

# validate  
  if(!is.null(x.condpp)) {
    if(mtype) {
      if(!is.marked(x.condpp))
        stop("Model is multitype, but x.cond is unmarked", call.=FALSE)
      if(!isTRUE(all.equal(types, levels(marks(x.condpp)))))
        stop("Types of points in x.cond do not match types in model", call.=FALSE)
    }
  }
  

#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
#
#     S T A R T I N G      S T A T E
#
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===

###################### Starting state data ############################

# whether the initial state should be thinned
  
  thin <- (start$given != "x") && (control$fixing == "none")
  
# There must now be a starting state.
  
  if(start$given == "none") {
    # For conditional simulation, the starting state must be given
    if(condtype != "none")
      stop("No starting state given", call.=FALSE)
    # Determine integral of beta * trend over data window.
    # This is the expected number of points in the reference Poisson process.
    area.w.clip <- area(w.clip)
    if(trendy) {
      tsummaries <- summarise.trend(trend, w=w.clip, a=area.w.clip)
      En <- beta * sapply(tsummaries, getElement, name="integral")
    } else {
      En <- beta * area.w.clip
    }
    # Fix n.start equal to this integral
    n.start <- if(spatstat.options("scalable")) round(En) else ceiling(En)
    start <- rmhstart(n.start=n.start)
  }
  
# In the case of conditional simulation, the start data determine
# the 'free' points (i.e. excluding x.cond) in the initial state.

  switch(start$given,
         none={
           stop("No starting state given", call.=FALSE)
         },
         x = {
           # x.start was given
           # coerce it to a ppp object
           if(!is.ppp(x.start))
             x.start <- as.ppp(x.start, w.state)
           if(condtype == "window") {
             # clip to simulation window
             xs <- x.start[w.sim]
             nlost <- x.start$n - xs$n
             if(nlost > 0) 
               warning(paste(nlost,
                             ngettext(nlost, "point","points"),
                             "of x.start",
                             ngettext(nlost, "was", "were"),
                             "removed because",
                             ngettext(nlost, "it", "they"),
                             "fell in the window of x.cond"),
                       call.=FALSE)
             x.start <- xs
           }
           npts.free <- x.start$n
         },
         n = {
           # n.start was given
           n.start <- start$n.start
           # Adjust the number of points in the starting state in accordance
           # with the expansion that has occurred.  
           if(expanded) {
	     holnum <- if(spatstat.options("scalable")) round else ceiling
             n.start <- holnum(n.start * area(w.sim)/area(w.clip))
           }
           #
           npts.free <- sum(n.start) # The ``sum()'' is redundant if n.start
                                # is scalar; no harm, but.
         },
         stop("Internal error: start$given unrecognized"), call.=FALSE)

#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
#
#     C O N T R O L    P A R A M E T E R S
#
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===

###################  Periodic boundary conditions #########################

  periodic <- control$periodic
  
  if(is.null(periodic)) {
    # undecided. Use default rule
    control$periodic <- periodic <- expanded && is.rectangle(w.state)
  } else if(periodic && !is.rectangle(w.state)) {
    # if periodic is TRUE we have to be simulating in a rectangular window.
    stop("Need rectangular window for periodic simulation.", call.=FALSE)
  }

# parameter passed to C:  
  period <-
    if(periodic)
      c(diff(w.state$xrange), diff(w.state$yrange))
    else
      c(-1,-1)



#### vector of proposal probabilities 

  if(!mtype) 
    ptypes <- 1
  else {
    ptypes <- control$ptypes
    if(is.null(ptypes)) {
      # default proposal probabilities
      ptypes <- if(start$given == "x" && (nx <- npoints(x.start)) > 0) {
        table(marks(x.start, dfok=FALSE))/nx
      } else rep.int(1/ntypes, ntypes)
    } else {
      # Validate ptypes
      if(length(ptypes) != ntypes | sum(ptypes) != 1)
        stop("Argument ptypes is mis-specified.", call.=FALSE)
    }
  } 


  
########################################################################
#  Normalising constant for proposal density
# 
# Integral of trend over the expanded window (or area of window):
# Iota == Integral Of Trend (or) Area.

  area.w.sim <- area(w.sim)
  if(trendy) {
    if(verbose)
      cat("Evaluating trend integral...")
    tsummaries <- summarise.trend(trend, w=w.sim, a=area.w.sim)
    mins  <- sapply(tsummaries, getElement, name="min")
    if(any(mins < 0))
      stop("Trend has negative values", call.=FALSE)
    iota <- sapply(tsummaries, getElement, name="integral")
    tmax <- sapply(tsummaries, getElement, name="max")
  } else {
    iota <- area.w.sim
    tmax <- NULL
  }

  
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
#
#     A.S. EMPTY PROCESS
#
#         for conditional simulation, 'empty' means there are no 'free' points
#  
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===

  a.s.empty <- FALSE
  
#
#  Empty pattern, simulated conditional on n
#  
  if(npts.free == 0 && control$fixing != "none") {
    a.s.empty <- TRUE
    if(verbose) {
      mess <- paste("Initial pattern has 0 random points,",
                    "and simulation is conditional on the number of points -")
      if(condtype == "none")
        warning(paste(mess, "returning an empty pattern"), call.=FALSE)
      else
        warning(paste(mess, "returning a pattern with no random points"), call.=FALSE)
    }
  }

#
#  If beta = 0, the process is almost surely empty
#
  
  if(all(beta < .Machine$double.eps)) {
    if(control$fixing == "none" && condtype == "none") {
      # return empty pattern
      if(verbose)
        warning("beta = 0 implies an empty pattern", call.=FALSE)
      a.s.empty <- TRUE
    } else 
      stop("beta = 0 implies an empty pattern, but we are simulating conditional on a nonzero number of points", call.=FALSE)
  }

#
# If we're conditioning on the contents of a subwindow,
# and the subwindow covers the clipping region,
# the result is deterministic.  

  if(condtype == "window" && is.subset.owin(w.clip, w.cond)) {
    a.s.empty <- TRUE
    warning(paste("Model window is a subset of conditioning window:",
              "result is deterministic"), call.=FALSE)
  }    

#
#  
  if(a.s.empty) {
    # create empty pattern, to be returned
    if(!is.null(x.condpp)) 
      empty <- x.condpp[w.clip]
    else {
      empty <- ppp(numeric(0), numeric(0), window=w.clip)
      if(mtype) {
        vide <- factor(types[integer(0)], levels=types)
        empty <- empty %mark% vide
      }
    }
  }

#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
#
#     PACK UP
#
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===

######### Store decisions

  Model <- model
  Start <- start
  Control <- control

  Model$w <- w.clip
  Model$types <- types
  
  Control$expand <- if(expanded) rmhexpand(w.state) else .no.expansion

  Control$internal <- list(w.sim=w.sim,
                           w.state=w.state,
                           x.condpp=x.condpp,
                           ptypes=ptypes,
                           period=period,
                           thin=thin)

  Model$internal <- list(a.s.empty=a.s.empty,
                         empty=if(a.s.empty) empty else NULL,
                         mtype=mtype,
                         trendy=trendy,
                         betalist=betalist,
                         beta=beta,
                         iota=iota,
                         tmax=tmax)

  Start$internal <- list(npts.free=npts.free)

  InfoList <- list(model=Model, start=Start, control=Control)
  class(InfoList) <- c("rmhInfoList", class(InfoList))

  # go
  if(nsim == 1 && drop) {
    result <- do.call(rmhEngine,
                      append(list(InfoList,
                                  verbose=verbose,
                                  snoop=snoop,
                                  kitchensink=saveinfo),
                             f.args))
  } else {
    result <- vector(mode="list", length=nsim)
    if(verbose) {
      splat("Generating", nsim, "point patterns...")
      pstate <- list()
    }
    subverb <- verbose && (nsim == 1)
    for(isim in 1:nsim) {
      if(verbose) pstate <- progressreport(isim, nsim, state=pstate)
      result[[isim]] <- do.call(rmhEngine,
                                append(list(InfoList,
                                            verbose=subverb,
                                            snoop=snoop,
                                            kitchensink=saveinfo),
                                       f.args))
    }
    if(verbose) splat("Done.\n")
    result <- simulationresult(result, nsim, drop)
  }
  return(result)
}

print.rmhInfoList <- function(x, ...) {
  cat("\nPre-digested Metropolis-Hastings algorithm parameters (rmhInfoList)\n")
  print(as.anylist(x))
}

#---------------  rmhEngine -------------------------------------------
#
# This is the interface to the C code.
#
# InfoList is a list of pre-digested, validated arguments
# obtained from rmh.default.
#
# This function is called by rmh.default to generate one simulated
# realisation of the model.
# It's called repeatedly by ho.engine and qqplot.ppm to generate multiple
# realisations (saving time by not repeating the argument checking
# in rmh.default).

# arguments:  
# kitchensink: whether to tack InfoList on to the return value as an attribute
# preponly: whether to just return InfoList without simulating
#
#   rmh.default digests arguments and calls rmhEngine with kitchensink=T
#
#   qqplot.ppm first gets InfoList by calling rmh.default with preponly=T
#              (which digests the model arguments and calls rmhEngine
#               with preponly=T, returning InfoList),
#              then repeatedly calls rmhEngine(InfoList) to simulate.
#
# -------------------------------------------------------

rmhEngine <- function(InfoList, ...,
                       verbose=FALSE, kitchensink=FALSE,
                       preponly=FALSE, snoop=FALSE,
                       overrideXstart=NULL, overrideclip=FALSE) {
# Internal Use Only!
# This is the interface to the C code.

  if(!inherits(InfoList, "rmhInfoList"))
    stop("data not in correct format for internal function rmhEngine", call.=FALSE)

  
  if(preponly)
    return(InfoList)

  model <- InfoList$model
  start <- InfoList$start
  control <- InfoList$control

  w.sim <- control$internal$w.sim
  w.state <- control$internal$w.state
  w.clip <- model$w

  condtype <- control$condtype
  x.condpp <- control$internal$x.condpp

  types <- model$types
  ntypes <- length(types)
  
  ptypes <- control$internal$ptypes
  period <- control$internal$period

  mtype <- model$internal$mtype

  trend <- model$trend
  trendy <- model$internal$trendy
#  betalist <- model$internal$betalist
  beta <- model$internal$beta
  iota <- model$internal$iota
  tmax <- model$internal$tmax

  npts.free <- start$internal$npts.free

  n.start <- start$n.start
  x.start <- start$x.start

  
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
#
#     E M P T Y   P A T T E R N
#
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===

  if(model$internal$a.s.empty) {
    if(verbose) cat("\n")
    empty <- model$internal$empty
    attr(empty, "info") <- InfoList
    return(empty)
  }
  
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
#
#     S I M U L A T I O N     
#
#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===

#############################################
####  
####  Random number seed: initialisation & capture
####  
#############################################  
  
  if(!exists(".Random.seed"))
    runif(1L)

  saved.seed <- .Random.seed
  

#############################################
####  
####  Poisson case
####  
#############################################  
  
  if(is.poisson.rmhmodel(model)) {
    if(verbose) cat("\n")
    intensity <- if(!trendy) beta else model$trend
    Xsim <-
      switch(control$fixing,
             none= {
               # Poisson process 
               if(!mtype)
                 rpoispp(intensity, win=w.sim, ..., warnwin=FALSE)
               else
                 rmpoispp(intensity, win=w.sim, types=types, warnwin=FALSE)
             },
             n.total = {
               # Binomial/multinomial process with fixed total number of points
               if(!mtype) 
                 rpoint(npts.free, intensity, win=w.sim, verbose=verbose)
               else
                 rmpoint(npts.free, intensity, win=w.sim, types=types,
                         verbose=verbose)
             },
             n.each.type = {
               # Multinomial process with fixed number of points of each type
               npts.each <-
                 switch(start$given,
                        n = n.start,
                        x = as.integer(table(marks(x.start, dfok=FALSE))),
  stop("No starting state given; can't condition on fixed number of points", call.=FALSE))
               rmpoint(npts.each, intensity, win=w.sim, types=types,
                       verbose=verbose)
             },
             stop("Internal error: control$fixing unrecognised", call.=FALSE)
             )
    # if conditioning, add fixed points
    if(condtype != "none")
      Xsim <- superimpose(Xsim, x.condpp, W=w.state)
    # clip result to output window
    Xclip <- if(!overrideclip) Xsim[w.clip] else Xsim
    attr(Xclip, "info") <- InfoList
    return(Xclip)
  }

  
########################################################################  
#      M e t r o p o l i s  H a s t i n g s    s i m u l a t i o n
########################################################################

  if(verbose)
    cat("Starting simulation.\nInitial state...")
  

#### Build starting state

  npts.cond  <- if(condtype != "none") x.condpp$n else 0
#  npts.total <- npts.free + npts.cond

#### FIRST generate the 'free' points
  
#### First the marks, if any.
#### The marks must be integers 0 to (ntypes-1) for passing to C

  Ctypes <- if(mtype) 0:(ntypes-1) else 0
  
  Cmarks <-
    if(!mtype)
      0
    else
      switch(start$given,
             n = {
               # n.start given
               if(control$fixing=="n.each.type")
                 rep.int(Ctypes,n.start)
               else
                 sample(Ctypes,npts.free,TRUE,ptypes)
             },
             x = {
               # x.start given
               as.integer(marks(x.start, dfok=FALSE))-1L
             },
             stop("internal error: start$given unrecognised", call.=FALSE)
             )
#
# Then the x, y coordinates
#
  switch(start$given,
         x = {
           x <- x.start$x
           y <- x.start$y
         },
         n = {
           xy <-
             if(!trendy)
               runifpoint(npts.free, w.sim, ...)
             else
               rpoint.multi(npts.free, trend, tmax,
                      factor(Cmarks,levels=Ctypes), w.sim, ...)
           x <- xy$x
           y <- xy$y
         })

## APPEND the free points AFTER the conditioning points

  if(condtype != "none") {
    x <- c(x.condpp$x, x)
    y <- c(x.condpp$y, y)
    if(mtype)
      Cmarks <- c(as.integer(marks(x.condpp))-1L, Cmarks)
  }

  if(!is.null(overrideXstart)) {
    #' override the previous data
    x <- overrideXstart$x
    y <- overrideXstart$y
    if(mtype) Cmarks <- as.integer(marks(overrideXstart))-1L
  }

# decide whether to activate visual debugger
  if(snoop) {
    Xinit <- ppp(x, y, window=w.sim)
    if(mtype) 
      marks(Xinit) <- factor(Cmarks, levels=Ctypes, labels=types)
    if(verbose) cat("\nCreating debugger environment..")
    snoopenv <- rmhSnoopEnv(Xinit=Xinit, Wclip=w.clip, R=reach(model))
    if(verbose) cat("Done.\n")
  } else snoopenv <- "none"

  
#######################################################################
#  Set up C call
######################################################################    

# Determine the name of the cif used in the C code

  C.id <- model$C.id
  ncif <- length(C.id)
  
# Get the parameters in C-ese
    
  ipar <- model$C.ipar
  iparlist <- if(ncif == 1) list(ipar) else model$C.iparlist
  iparlen <- lengths(iparlist)

  beta <- model$internal$beta
  
# Absorb the constants or vectors `iota' and 'ptypes' into the beta parameters
  beta <- (iota/ptypes) * beta
  
# Algorithm control parameters

  p       <- control$p
  q       <- control$q
  nrep    <- control$nrep
#  fixcode <- control$fixcode
#  fixing  <- control$fixing
  fixall  <- control$fixall
  nverb   <- control$nverb
  saving  <- control$saving
  nsave   <- control$nsave
  nburn   <- control$nburn
  track   <- control$track
  thin    <- control$internal$thin
  pstage  <- control$pstage %orifnull% "start"
  if(pstage == "block" && !saving) pstage <- "start"
  temper  <- FALSE
  invertemp <- 1.0

  if(verbose)
    cat("Ready to simulate. ")

  storage.mode(ncif)   <- "integer"
  storage.mode(C.id)   <- "character"
  storage.mode(beta)    <- "double"
  storage.mode(ipar)    <- "double"
  storage.mode(iparlen) <- "integer"
  storage.mode(period) <- "double"
  storage.mode(ntypes) <- "integer"
  storage.mode(nrep)   <- "integer"
  storage.mode(p) <- storage.mode(q) <- "double"
  storage.mode(nverb)  <- "integer"
  storage.mode(x) <- storage.mode(y) <- "double"
  storage.mode(Cmarks) <- "integer"
  storage.mode(fixall) <- "integer"
  storage.mode(npts.cond) <- "integer"
  storage.mode(track) <- "integer"
  storage.mode(thin) <- "integer"
  storage.mode(temper) <- "integer"
  storage.mode(invertemp) <- "double"

  if(pstage == "start" || !saving) {
    #' generate all proposal points now.
    if(verbose)
      cat("Generating proposal points...")

    #' If the pattern is multitype, generate the mark proposals (0 to ntypes-1)
    Cmprop <- if(mtype) sample(Ctypes,nrep,TRUE,prob=ptypes) else 0
    storage.mode(Cmprop) <- "integer"

    #' Generate the ``proposal points'' in the expanded window.
    xy <- if(trendy) {
      rpoint.multi(nrep,trend,tmax,
                   factor(Cmprop, levels=Ctypes),
                   w.sim, ..., warn=FALSE)
    } else runifpoint(nrep, w.sim, warn=FALSE)
    xprop <- xy$x
    yprop <- xy$y
    storage.mode(xprop)  <- storage.mode(yprop) <- "double"
  }
  
  if(!saving) {
    # ////////// Single block /////////////////////////////////

    nrep0 <- 0
    storage.mode(nrep0)  <- "integer"

    # Call the Metropolis-Hastings C code:
    if(verbose)
      cat("Running Metropolis-Hastings.\n")
    out <- .Call("xmethas",
                 ncif,
                 C.id,
                 beta,
                 ipar,
                 iparlen,
                 period,
                 xprop, yprop, Cmprop,
                 ntypes,
                 nrep,
                 p, q,
                 nverb,
                 nrep0,
                 x, y, Cmarks,
                 npts.cond,
                 fixall,
                 track,
                 thin,
                 snoopenv,
                 temper,
                 invertemp,
                 PACKAGE="spatstat")
  
    # Extract the point pattern returned from C
    X <- ppp(x=out[[1L]], y=out[[2L]], window=w.state, check=FALSE)
    if(mtype) {
      #' convert integer marks from C to R
      #' then restore original type levels
      marks(X) <- factor(out[[3L]], levels=Ctypes, labels=types)
    }

    # Now clip the pattern to the ``clipping'' window:
    if(!overrideclip && !control$expand$force.noexp)
      X <- X[w.clip]

    # Extract transition history:
    if(track) {
      usedout <- if(mtype) 3 else 2
      proptype <- factor(out[[usedout+1]], levels=1:3,
                         labels=c("Birth", "Death", "Shift"))
      accepted <- as.logical(out[[usedout+2]])
      History <- data.frame(proposaltype=proptype, accepted=accepted)
      if(length(out) >= usedout + 4) {
        # history includes numerator & denominator of Hastings ratio
        numerator <- as.double(out[[usedout + 3]])
        denominator <- as.double(out[[usedout + 4]])
        History <- cbind(History,
                         data.frame(numerator=numerator,
                                    denominator=denominator))
      }
    }
  } else {
    # ////////// Multiple blocks /////////////////////////////////
    ## determine length of each block of simulations
    nsuperblocks <- as.integer(1L + ceiling((nrep - nburn)/sum(nsave)))
    block <- c(nburn, rep.int(nsave, nsuperblocks-1L))
    block <- block[cumsum(block) <= nrep]
    if((tot <- sum(block)) < nrep)
      block <- c(block, nrep-tot)
    block <- block[block >= 1L]
    nblocks <- length(block)
    blockend <- cumsum(block)
    ## set up list to contain the saved point patterns
    Xlist <- vector(mode="list", length=nblocks+1L)
    ## save initial state
    Xinit <- ppp(x=x, y=y, window=w.state, check=FALSE)
    if(mtype) {
      ## convert integer marks from C to R
      ## then restore original type levels
      marks(Xinit) <- factor(Cmarks, levels=Ctypes, labels=types)
    }
    Xlist[[1L]] <- Xinit
    # Call the Metropolis-Hastings C code repeatedly:
    xprev <- x
    yprev <- y
    Cmarksprev <- Cmarks
    #
    thinFALSE <- as.integer(FALSE)
    storage.mode(thinFALSE) <- "integer"
    # ................ loop .........................
    for(I in 1:nblocks) {
      # number of iterations for this block
      nrepI <- block[I]
      storage.mode(nrepI) <- "integer"
      # number of previous iterations
      nrep0 <- if(I == 1) 0 else blockend[I-1]
      storage.mode(nrep0)  <- "integer"
      # Generate or extract proposals
      switch(pstage,
             start = {
               #' extract proposals from previously-generated vectors
               if(verbose)
                 cat("Extracting proposal points...")
               seqI <- 1:nrepI
               xpropI <- xprop[seqI]
               ypropI <- yprop[seqI]
               CmpropI <- Cmprop[seqI]
               storage.mode(xpropI) <- storage.mode(ypropI) <- "double"
               storage.mode(CmpropI) <- "integer"
             },
             block = {
               # generate 'nrepI' random proposals
               if(verbose)
                 cat("Generating proposal points...")
               #' If the pattern is multitype, generate the mark proposals 
               CmpropI <- if(mtype) sample(Ctypes,nrepI,TRUE,prob=ptypes) else 0
               storage.mode(CmpropI) <- "integer"
               #' Generate the ``proposal points'' in the expanded window.
               xy <- if(trendy) {
                 rpoint.multi(nrepI,trend,tmax,
                              factor(CmpropI, levels=Ctypes),
                              w.sim, ..., warn=FALSE)
               } else runifpoint(nrepI, w.sim, warn=FALSE)
               xpropI <- xy$x
               ypropI <- xy$y
               storage.mode(xpropI)  <- storage.mode(ypropI) <- "double"
             })
      # no thinning in subsequent blocks
      if(I > 1) thin <- thinFALSE
      #' call
      if(verbose)
        cat("Running Metropolis-Hastings.\n")
      out <- .Call("xmethas",
                   ncif,
                   C.id,
                   beta,
                   ipar,
                   iparlen,
                   period,
                   xpropI, ypropI, CmpropI,
                   ntypes,
                   nrepI,
                   p, q,
                   nverb,
                   nrep0,
                   xprev, yprev, Cmarksprev,
                   npts.cond,
                   fixall,
                   track,
                   thin,
                   snoopenv,
                   temper,
                   invertemp,
                   PACKAGE = "spatstat")
      # Extract the point pattern returned from C
      X <- ppp(x=out[[1L]], y=out[[2L]], window=w.state, check=FALSE)
      if(mtype) {
        # convert integer marks from C to R
        # then restore original type levels
        marks(X) <- factor(out[[3L]], levels=Ctypes, labels=types)
      }
      
      # Now clip the pattern to the ``clipping'' window:
      if(!overrideclip && !control$expand$force.noexp)
        X <- X[w.clip]

      # commit to list
      Xlist[[I+1L]] <- X
      
      # Extract transition history:
      if(track) {
        usedout <- if(mtype) 3 else 2
        proptype <- factor(out[[usedout+1]], levels=1:3,
                           labels=c("Birth", "Death", "Shift"))
        accepted <- as.logical(out[[usedout+2]])
        HistoryI <- data.frame(proposaltype=proptype, accepted=accepted)
        if(length(out) >= usedout + 4) {
          # history includes numerator & denominator of Hastings ratio
          numerator <- as.double(out[[usedout + 3]])
          denominator <- as.double(out[[usedout + 4]])
          HistoryI <- cbind(HistoryI,
                            data.frame(numerator=numerator,
                                       denominator=denominator))
        }
        # concatenate with histories of previous blocks
        History <- if(I == 1) HistoryI else rbind(History, HistoryI)
      }

      # update 'previous state'
      xprev <- out[[1L]]
      yprev <- out[[2L]]
      Cmarksprev <- if(!mtype) 0 else out[[3]]
      storage.mode(xprev) <- storage.mode(yprev) <- "double"
      storage.mode(Cmarksprev) <- "integer"

      if(pstage == "start") {
        #' discard used proposals
        xprop <- xprop[-seqI]
        yprop <- yprop[-seqI]
        Cmprop <- Cmprop[-seqI]
      }
    }
    # .............. end loop ...............................
    
    # Result of simulation is final state 'X'
    # Tack on the list of intermediate states
    names(Xlist) <- paste("Iteration", c(0,as.integer(blockend)), sep="_")
    attr(X, "saved") <- as.solist(Xlist)
  }

# Append to the result information about how it was generated.
  if(kitchensink) {
    attr(X, "info") <- InfoList
    attr(X, "seed") <- saved.seed
  }
  if(track)
    attr(X, "history") <- History
  
  return(X)
}


# helper function

summarise.trend <- local({
  # main function
  summarise.trend <- function(trend, w, a=area(w)) {
    tlist <- if(is.function(trend) || is.im(trend)) list(trend) else trend
    return(lapply(tlist, summarise1, w=w, a=a))
  }
  # 
  summarise1 <-  function(x, w, a) {
    if(is.numeric(x)) {
      mini <- maxi <- x
      integ <- a*x
    } else {
      Z  <- as.im(x, w)[w, drop=FALSE]
      ran <- range(Z)
      mini <- ran[1L]
      maxi <- ran[2L]
      integ <- integral.im(Z)
    }
    return(list(min=mini, max=maxi, integral=integ))
  }
  summarise.trend
})
  

#
#   nndist.R
#
#   nearest neighbour distances (nndist) and identifiers (nnwhich)
#
#   $Revision: 1.12 $ $Date: 2020/01/10 06:37:05 $
#

nndist <- function(X, ...) {
  UseMethod("nndist")
}

nndist.ppp <- local({

  nndist.ppp <- function(X, ..., k=1, by=NULL, method="C") {
    verifyclass(X, "ppp")
    trap.extra.arguments(..., .Context="In nndist.ppp")
    if(is.null(by)) # usual case
      return(nndist.default(X$x, X$y, k=k, by=by, method=method))
    return(nndistby(X, k=k, by=by))
  }

  nndistby <- function(X, k, by) {
    # split by factor 
    idX <- seq_len(npoints(X))
    Y <- split(X %mark% idX, f=by, un=FALSE)
    distY <- lapply(Y, nndistsub, XX=X, iX=idX, k=k)
    result <- do.call(cbind, distY)
    return(result)
  }

  nndistsub <- function(Z, XX, iX, k) {
    nncross(XX, Z, iX=iX, iY=marks(Z), k=k, what="dist")
  }

  nndist.ppp
})

nndist.default <-
  function(X, Y=NULL, ..., k=1, by=NULL, method="C")
{
	#  computes the vector of nearest-neighbour distances 
	#  for the pattern of points (x[i],y[i])
	#
  xy <- xy.coords(X,Y)[c("x","y")]
  x <- xy$x
  y <- xy$y

  # validate
  n <- length(x)
  if(length(y) != n)
    stop("lengths of x and y do not match")

  method <- match.arg(method, c("C", "interpreted", "test"))
  # other arguments ignored
  trap.extra.arguments(..., .Context="In nndist.default")

  # split by factor ?
  if(!is.null(by)) {
    X <- as.ppp(xy, W=boundingbox)
    return(nndist(X, by=by, k=k))
  }
  
  # k can be a single integer or an integer vector
  if(length(k) == 0)
    stop("k is an empty vector")
  else if(length(k) == 1) {
    if(k != round(k) || k <= 0)
      stop("k is not a positive integer")
  } else {
    if(any(k != round(k)) || any(k <= 0))
      stop(paste("some entries of the vector",
           sQuote("k"), "are not positive integers"))
  }
  k <- as.integer(k)
  kmax <- max(k)

  # trivial cases
  if(n <= 1) {
    # empty pattern => return numeric(0)
    # or pattern with only 1 point => return Inf
    nnd <- matrix(Inf, nrow=n, ncol=kmax)
    nnd <- nnd[,k, drop=TRUE]
    return(nnd)
  }
  
  # number of neighbours that are well-defined
  kmaxcalc <- min(n-1, kmax)

  # calculate k-nn distances for k <= kmaxcalc
  
  if(kmaxcalc == 1) {
    # calculate nearest neighbour distance only
    switch(method,
         test = ,
         interpreted={
           #  matrix of squared distances between all pairs of points
           sq <- function(a, b) { (a-b)^2 }
           squd <-  outer(x, x, sq) + outer(y, y, sq)
           #  reset diagonal to a large value so it is excluded from minimum
           diag(squd) <- Inf
           #  nearest neighbour distances
           nnd <- sqrt(apply(squd,1,min))
         },
         C={
           nnd<-numeric(n)
           o <- fave.order(y)
           big <- sqrt(.Machine$double.xmax)
           z<- .C("nndistsort",
                  n= as.integer(n),
                  x= as.double(x[o]), y= as.double(y[o]), nnd= as.double(nnd),
                  as.double(big),
                  PACKAGE = "spatstat")
           nnd[o] <- z$nnd
         },
         stop(paste("Unrecognised method", sQuote(method)))
         )
  } else {
    # case kmaxcalc > 1
    switch(method,
           test = ,
           interpreted={
             if(n <= 1000 && method == "interpreted") {
               # form n x n matrix of squared distances
               D2 <- pairdist.default(x, y, method=method, squared=TRUE)
               # find k'th smallest squared distance
               diag(D2) <- Inf
               NND2 <- t(apply(D2, 1, sort))[, 1:kmaxcalc]
               nnd <- sqrt(NND2)
             } else {
               # avoid creating huge matrix
               # handle one row of D at a time
               NND2 <- matrix(numeric(n * kmaxcalc), nrow=n, ncol=kmaxcalc)
               for(i in seq_len(n)) {
                 D2i <- (x - x[i])^2 + (y - y[i])^2
                 D2i[i] <- Inf
                 NND2[i,] <- orderstats(D2i, k=1:kmaxcalc)
               }
               nnd <- sqrt(NND2)
             }
           },
           C={
             nnd<-numeric(n * kmaxcalc)
             o <- fave.order(y)
             big <- sqrt(.Machine$double.xmax)
             z<- .C("knndsort",
                    n    = as.integer(n),
                    kmax = as.integer(kmaxcalc),
                    x    = as.double(x[o]),
                    y    = as.double(y[o]),
                    nnd  = as.double(nnd),
                    huge = as.double(big),
                    PACKAGE = "spatstat")
             nnd <- matrix(nnd, nrow=n, ncol=kmaxcalc)
             nnd[o, ] <- matrix(z$nnd, nrow=n, ncol=kmaxcalc, byrow=TRUE)
           },
           stop(paste("Unrecognised method", sQuote(method)))
           )
  }

  # post-processing
  if(kmax > kmaxcalc) {
    # add columns of Inf
    infs <- matrix(Inf, nrow=n, ncol=kmax-kmaxcalc)
    nnd <- cbind(nnd, infs)
  }

  if(kmax > 1)
    colnames(nnd) <- paste0("dist.", 1:kmax)
  
  if(length(k) < kmax) {
    # select only the specified columns
    nnd <- nnd[, k, drop=TRUE]
  }
  
  return(nnd)
}


nnwhich <- function(X, ...) {
  UseMethod("nnwhich")
}

nnwhich.ppp <- local({

  nnwhich.ppp <- function(X, ..., k=1, by=NULL, method="C") {
    verifyclass(X, "ppp")
    trap.extra.arguments(..., .Context="In nnwhich.ppp")
    if(is.null(by))
      return(nnwhich.default(X$x, X$y, k=k, method=method))
    return(nnwhichby(X, k=k, by=by))
  }

  nnwhichby <- function(X, k, by) {
    # split by factor 
    idX <- seq_len(npoints(X))
    Y <- split(X %mark% idX, f=by, un=FALSE)
    whichY <- lapply(Y, nnwhichsub, XX=X, iX=idX, k=k)
    result <- do.call(cbind, whichY)
    return(result)
  }

  nnwhichsub <- function(Z, XX, iX, k) {
    # marks(Z) gives original serial numbers of subset Z
    iY <- marks(Z)
    Zid <- nncross(XX, Z, iX=iX, iY=iY, k=k, what="which")
    nk <- length(k)
    if(nk == 1) {
      Yid <- iY[Zid]
    } else {
      Zid <- as.vector(as.matrix(Zid))
      Yid <- iY[Zid]
      Yid <- data.frame(which=matrix(Yid, ncol=nk))
    }
    return(Yid)
  }

  nnwhich.ppp
})


nnwhich.default <-
  function(X, Y=NULL, ..., k=1, by=NULL, method="C")
{
	#  identifies nearest neighbour of each point in
	#  the pattern of points (x[i],y[i])
	#
  xy <- xy.coords(X,Y)[c("x","y")]
  x <- xy$x
  y <- xy$y

  # validate
  n <- length(x)
  if(length(y) != n)
    stop("lengths of x and y do not match")
  
  method <- match.arg(method, c("C", "interpreted", "test"))
  # other arguments ignored
  trap.extra.arguments(..., .Context="In nnwhich.default")

  # split by factor ?
  if(!is.null(by)) {
    X <- as.ppp(xy, W=boundingbox)
    return(nnwhich(X, by=by, k=k))
  }
  
  # k can be a single integer or an integer vector
  if(length(k) == 0)
    stop("k is an empty vector")
  else if(length(k) == 1) {
    if(k != round(k) || k <= 0)
      stop("k is not a positive integer")
  } else {
    if(any(k != round(k)) || any(k <= 0))
      stop(paste("some entries of the vector",
           sQuote("k"), "are not positive integers"))
  }
  k <- as.integer(k)
  kmax <- max(k)

  # special cases
  if(n <= 1) {
    # empty pattern => return integer(0)
    # or pattern with only 1 point => return NA
    nnw <- matrix(as.integer(NA), nrow=n, ncol=kmax)
    nnw <- nnw[,k, drop=TRUE]
    return(nnw)
  }

  # number of neighbours that are well-defined
  kmaxcalc <- min(n-1, kmax)

  # identify k-nn for k <= kmaxcalc

  if(kmaxcalc == 1) {
    # identify nearest neighbour only
    switch(method,
           test = ,
           interpreted={
             #  matrix of squared distances between all pairs of points
             sq <- function(a, b) { (a-b)^2 }
             squd <-  outer(x, x, sq) + outer(y, y, sq)
             #  reset diagonal to a large value so it is excluded from minimum
             diag(squd) <- Inf
             #  nearest neighbours
             nnw <- apply(squd,1,which.min)
           },
           C={
             nnw <- integer(n)
             o <- fave.order(y)
             big <- sqrt(.Machine$double.xmax)
             z<- .C("nnwhichsort",
                    n = as.integer(n),
                    x = as.double(x[o]),
                    y = as.double(y[o]),
                    nnwhich = as.integer(nnw),
                    huge = as.double(big),
                    PACKAGE = "spatstat")
             witch <- z$nnwhich # sic 
             if(any(witch <= 0))
               stop("Internal error: non-positive index returned from C code")
             if(any(witch > n))
               stop("Internal error: index returned from C code exceeds n")
             nnw[o] <- o[witch]
           },
           stop(paste("Unrecognised method", sQuote(method)))
           )
  } else {
    # case kmaxcalc > 1
    switch(method,
           test = ,
           interpreted={
             if(n <= 1000 && method == "interpreted") {
               # form n x n matrix of squared distances
               D2 <- pairdist.default(x, y, method=method, squared=TRUE)
               # find k'th smallest squared distance
               diag(D2) <- Inf
               nnw <- t(apply(D2, 1, fave.order))[, 1:kmaxcalc]
             } else {
               # avoid creating huge matrix
               # handle one row of D at a time
               nnw <- matrix(as.integer(NA), nrow=n, ncol=kmaxcalc)
               for(i in seq_len(n)) {
                 D2i <- (x - x[i])^2 + (y - y[i])^2
                 D2i[i] <- Inf
                 nnw[i,] <- fave.order(D2i)[1:kmaxcalc]
               }      
             }
           },
           C={
             nnw <- matrix(integer(n * kmaxcalc), nrow=n, ncol=kmaxcalc)
             o <- fave.order(y)
             big <- sqrt(.Machine$double.xmax)
             z<- .C("knnwhich",
                    n = as.integer(n),
                    kmax = as.integer(kmaxcalc),
                    x = as.double(x[o]),
                    y = as.double(y[o]),
                    nnwhich = as.integer(nnw),
                    huge = as.double(big),
                    PACKAGE = "spatstat")
             witch <- z$nnwhich # sic
             witch <- matrix(witch, nrow=n, ncol=kmaxcalc, byrow=TRUE)
             if(any(witch <= 0))
               stop("Internal error: non-positive index returned from C code")
             if(any(witch > n))
               stop("Internal error: index returned from C code exceeds n")
             # convert back to original ordering
             nnw[o,] <- matrix(o[witch], nrow=n, ncol=kmaxcalc)
           },
           stop(paste("Unrecognised method", sQuote(method)))
           )
  }
  
  # post-processing
  if(kmax > kmaxcalc) {
    # add columns of NA's
    nas <- matrix(as.numeric(NA), nrow=n, ncol=kmax-kmaxcalc)
    nnw <- cbind(nnw, nas)
  }

  if(kmax > 1)
    colnames(nnw) <- paste0("which.", 1:kmax)

  if(length(k) < kmax) {
    # select only the specified columns
    nnw <- nnw[, k, drop=TRUE]
  }
  return(nnw)
}

#'
#'   uniquemap.R
#'
#'   Copyright (C) Adrian Baddeley, Ege Rubak and Rolf Turner 2001-2019
#'   Licence: GNU Public Licence >= 2
#'
#'   $Revision: 1.16 $  $Date: 2020/01/26 03:50:10 $

uniquemap <- function(x) { UseMethod("uniquemap") }

uniquemap.default <- function(x) {
  result <- seqn <- seq_along(x)
  if(length(x) <= 1) return(result)
  if(is.atomic(x) && (is.factor(x) || (is.vector(x) && is.numeric(x)))) {
    if(is.factor(x)) x <- as.integer(x)
    o <- order(x, seqn)
    isfirst <- c(TRUE, (diff(x[o]) != 0))
    omap <- cumsum(isfirst)
    result <- seqn
    result[o] <- o[isfirst][omap]
    return(result)
  }
  dup <- duplicated(x)
  ux <- x[!dup]
  mapdup <- match(x[dup], ux)
  result[dup] <- which(!dup)[mapdup]
  return(result)
}

uniquemap.matrix <- function(x) {
  n <- nrow(x)
  result <- seqn <- seq_len(n)
  if(n <= 1)
    return(result)
  #' faster algorithms for special cases
  nc <- ncol(x)
  if(nc == 1L) return(uniquemap(x[,1]))
  if(is.numeric(x)) {
    if(nc == 2L) {
      oo <- order(x[,1], x[,2], seqn)
      xx <- x[oo, , drop=FALSE]
      isfirst <- c(TRUE, (diff(xx[,1]) != 0) | (diff(xx[,2]) != 0))
    } else {
      ## y <- asplit(x, 2) would require R 3.6.0
      y <- split(as.vector(x), factor(as.vector(col(x)), levels=1:nc))
      oo <- do.call(order, append(unname(y), list(seqn)))
      xx <- x[oo, , drop=FALSE]
      isfirst <- c(TRUE, matrowany(apply(xx, 2, diff) != 0))
    }
    uniqueids <- seqn[oo][isfirst]
    lastunique <- cumsum(isfirst)
    result[oo] <- uniqueids[lastunique]
    return(result)
  }
  #' non-numeric matrix e.g. character
  if(!anyDuplicated(x))
    return(result)
  dup <- duplicated(x)
  uni <- which(!dup)
  for(j in which(dup)) {
    for(i in uni[uni < j]) {
      if(IdenticalRowPair(i, j, x)) {
        result[j] <- i
        break
      }
    }
  }
  return(result)
}

uniquemap.data.frame <- function(x) {
  n <- nrow(x)
  result <- seqn <- seq_len(n)
  if(n <= 1)
    return(result)
  #' faster algorithms for special cases
  nc <- ncol(x)
  if(nc == 1L) return(uniquemap(x[,1]))
  if(all(sapply(x, is.numeric))) {
    if(nc == 2L) {
      oo <- order(x[,1], x[,2], seqn)
      xx <- x[oo, , drop=FALSE]
      isfirst <- c(TRUE, (diff(xx[,1]) != 0) | (diff(xx[,2]) != 0))
    } else {
      oo <- do.call(order, append(unname(as.list(x)), list(seqn)))
      xx <- x[oo, , drop=FALSE]
      isfirst <- c(TRUE, matrowany(apply(xx, 2, diff) != 0))
    }
    uniqueids <- seqn[oo][isfirst]
    lastunique <- cumsum(isfirst)
    result[oo] <- uniqueids[lastunique]
    return(result)
  }
  #' general case
  if(!anyDuplicated(x))
    return(result)
  dup <- duplicated(x)
  uni <- which(!dup)
  for(j in which(dup)) {
    for(i in uni[uni < j]) {
      if(IdenticalRowPair(i, j, x)) {
        result[j] <- i
        break
      }
    }
  }
  return(result)
}

uniquemap.ppp <- function(x) {
  n <- npoints(x)
  seqn <- seq_len(n)
  if(n <= 1) return(seqn)
  marx <- marks(x)
  switch(markformat(marx),
         none = {
           useC <- TRUE
         },
         vector = {
           #' convert to integers if possible
           if(is.integer(marx) || is.factor(marx)) {
             marx <- as.integer(marx)
             useC <- TRUE
           } else {
             um <- unique(marx)
             if(length(um) <= 2^30) {
               marx <- match(marx, um)
               useC <- TRUE
             } else {
               useC <- FALSE
             }
           }
         },
         {
           useC <- FALSE
         })

  if(!useC) {
    #' first find duplicated spatial coordinates
    u <- uniquemap(unmark(x))
    #' add marks
    df <- cbind(data.frame(ind=seqn, uni=u), as.data.frame(marx))
    bb <- split(df, factor(u))
    #' consider each set of duplicated locations
    for(b in bb) {
      #' find unique rows of marks, as a list
      mrows <- lapply(seq_len(nrow(b)), function(i) b[i, -(1:2)])
      um <- unique(mrows)
      #' match other rows to them
      ma <- match(mrows, um)
      #' map to original index
      u[b$ind] <- b$ind[ma]
    }
    return(u)
  }

  #' unmarked or integer/factor marked
  xx <- x$x
  yy <- x$y
  o <- order(xx, seqn)

  if(is.null(marx)) {
    umap <- .C("uniqmapxy",
               n=as.integer(n),
               x=as.double(xx[o]),
               y=as.double(yy[o]),
               uniqmap=as.integer(integer(n)),
               PACKAGE="spatstat")$uniqmap
  } else {
    #' marks are (converted to) integers
    umap <- .C("uniqmap2M",
               n=as.integer(n),
               x=as.double(xx[o]),
               y=as.double(yy[o]),
               marks=as.integer(marx[o]),
               uniqmap=as.integer(integer(n)),
               PACKAGE="spatstat")$uniqmap
  }
  nodup <- (umap == 0)
  umap[nodup] <- which(nodup)
  result <- integer(n)
  result[o] <- o[umap]
  return(result)
}

uniquemap.lpp <- function(x) {
  n <- npoints(x)
  if(n <= 1 || !anyDuplicated(as.ppp(x))) return(seq_len(n))
  result <- uniquemap(as.data.frame(x))
  return(result)
}

uniquemap.ppx <- function(x) {
  uniquemap(as.data.frame(x))
}

#
#	Kmulti.S		
#
#	Compute estimates of cross-type K functions
#	for multitype point patterns
#
#	$Revision: 5.50 $	$Date: 2019/10/16 03:09:01 $
#
#
# -------- functions ----------------------------------------
#	Kcross()	cross-type K function K_{ij}
#                       between types i and j
#
#	Kdot()          K_{i\bullet}
#                       between type i and all points regardless of type
#
#       Kmulti()        (generic)
#
#
# -------- standard arguments ------------------------------	
#	X		point pattern (of class 'ppp')
#				including 'marks' vector
#	r		distance values at which to compute K	
#
# -------- standard output ------------------------------
#      A data frame with columns named
#
#	r:		same as input
#
#	trans:		K function estimated by translation correction
#
#	iso:		K function estimated by Ripley isotropic correction
#
#	theo:		K function for Poisson ( = pi * r ^2 )
#
#	border:		K function estimated by border method
#			using standard formula (denominator = count of points)
#
#       bord.modif:	K function estimated by border method
#			using modified formula 
#			(denominator = area of eroded window
#
# ------------------------------------------------------------------------

"Lcross" <- function(X, i, j, ..., from, to, correction) {
  if(!is.multitype(X, dfok=FALSE)) 
	stop("Point pattern must be multitype")
  if(missing(i)) i <- if(!missing(from)) from else levels(marks(X))[1]
  if(missing(j)) j <- if(!missing(to)) to else levels(marks(X))[2]
  if(missing(correction)) correction <- NULL
  K <- Kcross(X, i, j, ..., correction=correction)
  L <- eval.fv(sqrt(K/pi))
  # relabel the fv object
  iname <- make.parseable(paste(i))
  jname <- make.parseable(paste(j))
  L <- rebadge.fv(L,
                  substitute(L[i,j](r),
                             list(i=iname,j=jname)),
                  c("L", paste0("list(", iname, ",", jname, ")")),
                  new.yexp=substitute(L[list(i,j)](r),
                                      list(i=iname,j=jname)))
  attr(L, "labl") <- attr(K, "labl")
  return(L)  
}

"Ldot" <- function(X, i, ..., from, correction) {
  if(!is.multitype(X, dfok=FALSE)) 
	stop("Point pattern must be multitype")
  if(missing(i)) i <- if(!missing(from)) from else levels(marks(X))[1]
  if(missing(correction)) correction <- NULL
  K <- Kdot(X, i, ..., correction=correction)
  L <- eval.fv(sqrt(K/pi))
  # relabel the fv object
  iname <- make.parseable(paste(i))
  L <- rebadge.fv(L,
                  substitute(L[i ~ dot](r), list(i=iname)),
                  c("L", paste(iname, "~ symbol(\"\\267\")")), 
                  new.yexp=substitute(L[i ~ symbol("\267")](r), list(i=iname)))
  attr(L, "labl") <- attr(K, "labl")
  return(L)  
}

"Kcross" <- 
function(X, i, j, r=NULL, breaks=NULL,
         correction =c("border", "isotropic", "Ripley", "translate") , ...,
         ratio=FALSE, from, to)
{
  verifyclass(X, "ppp")
  if(!is.multitype(X, dfok=FALSE)) 
	stop("Point pattern must be multitype")
  if(missing(correction))
    correction <- NULL
  marx <- marks(X)
  if(missing(i))
    i <- if(!missing(from)) from else levels(marx)[1]
  if(missing(j))
    j <- if(!missing(to)) to else levels(marx)[2]
  I <- (marx == i)
  if(!any(I))
    stop(paste("No points have mark i =", i))

  if(i == j) {
    ## use Kest
    result <- do.call(Kest,
                      resolve.defaults(list(X=X[I],
                                            r=r, breaks=breaks,
                                            correction=correction, ratio=ratio),
                                       list(rmax=NULL), ## forbidden 
                                       list(...)))
  } else {
    J <- (marx == j)
    if(!any(J))
      stop(paste("No points have mark j =", j))
    result <- Kmulti(X, I, J,
                     r=r, breaks=breaks,
                     correction=correction, ratio=ratio, ...)
  }
  iname <- make.parseable(paste(i))
  jname <- make.parseable(paste(j))
  result <-
    rebadge.fv(result, 
               substitute(Kcross[i,j](r), list(i=iname,j=jname)),
               c("K", paste0("list(", iname, ",", jname, ")")), 
               new.yexp=substitute(K[list(i,j)](r),
                                   list(i=iname,j=jname)))
  return(result)
}

"Kdot" <- 
function(X, i, r=NULL, breaks=NULL,
         correction = c("border", "isotropic", "Ripley", "translate") , ...,
         ratio=FALSE, from)
{
  verifyclass(X, "ppp")
  if(!is.multitype(X, dfok=FALSE)) 
	stop("Point pattern must be multitype")
  if(missing(correction))
    correction <- NULL

  marx <- marks(X)
  if(missing(i))
    i <- if(!missing(from)) from else levels(marx)[1]
        
  I <- (marx == i)
  J <- rep.int(TRUE, X$n)  # i.e. all points
	
  if(!any(I)) stop(paste("No points have mark i =", i))
	
  result <- Kmulti(X, I, J,
                   r=r, breaks=breaks, correction=correction, ..., ratio=ratio)
  iname <- make.parseable(paste(i))
  result <-
    rebadge.fv(result,
               substitute(K[i ~ dot](r), list(i=iname)),
               c("K", paste0(iname, "~ symbol(\"\\267\")")),
               new.yexp=substitute(K[i ~ symbol("\267")](r), list(i=iname)))
  return(result)
}


"Kmulti"<-
function(X, I, J, r=NULL, breaks=NULL,
         correction = c("border", "isotropic", "Ripley", "translate") , ...,
         ratio=FALSE)
{
  verifyclass(X, "ppp")

  npts <- npoints(X)
  W <- X$window
  areaW <- area(W)

  correction.given <- !missing(correction) && !is.null(correction)
  if(is.null(correction))
    correction <- c("border", "isotropic", "Ripley", "translate")
  
  correction <- pickoption("correction", correction,
                           c(none="none",
                             border="border",
                             "bord.modif"="bord.modif",
                             isotropic="isotropic",
                             Ripley="isotropic",
                             trans="translate",
                             translate="translate",
                             translation="translate",
                             best="best"),
                           multi=TRUE)

  correction <- implemented.for.K(correction, W$type, correction.given)

  I <- ppsubset(X, I)
  J <- ppsubset(X, J)
  if(is.null(I) || is.null(J))
    stop("I and J must be valid subset indices")
	
  if(!any(I)) stop("no points belong to subset I")
  if(!any(J)) stop("no points belong to subset J")
		
  nI <- sum(I)
  nJ <- sum(J)
  lambdaI <- nI/areaW
  lambdaJ <- nJ/areaW

  # r values 
  rmaxdefault <- rmax.rule("K", W, lambdaJ)
  breaks <- handle.r.b.args(r, breaks, W, rmaxdefault=rmaxdefault)
  r <- breaks$r
  rmax <- breaks$max
        
  # recommended range of r values
  alim <- c(0, min(rmax, rmaxdefault))
        
  # this will be the output data frame
  # It will be given more columns later
  K <- data.frame(r=r, theo= pi * r^2)
  desc <- c("distance argument r", "theoretical Poisson %s")
  K <- fv(K, "r", quote(K[IJ](r)), 
          "theo", , alim, c("r","{%s[%s]^{pois}}(r)"),
          desc, fname=c("K", "list(I,J)"),
          yexp=quote(K[list(I,J)](r)))
  
  # save numerator and denominator?
  if(ratio) {
    denom <- lambdaI * lambdaJ * areaW
    numK <- eval.fv(denom * K)
    denK <- eval.fv(denom + K * 0)
    attributes(numK) <- attributes(denK) <- attributes(K)
    attr(numK, "desc")[2] <- "numerator for theoretical Poisson %s"
    attr(denK, "desc")[2] <- "denominator for theoretical Poisson %s"
  }

  # find close pairs of points
  XI <- X[I]
  XJ <- X[J]
  close <- crosspairs(XI, XJ, max(r), what="ijd")
# close$i and close$j are serial numbers in XI and XJ respectively;        
# map them to original serial numbers in X
  orig <- seq_len(npts)
  imap <- orig[I]
  jmap <- orig[J]
  iX <- imap[close$i]
  jX <- jmap[close$j]
# eliminate any identical pairs
  if(any(I & J)) {
    ok <- (iX != jX)
    if(!all(ok)) {
      close$i  <- close$i[ok]
      close$j  <- close$j[ok]
      close$d  <- close$d[ok]
    }
  }
# extract information for these pairs (relative to orderings of XI, XJ)
  dcloseIJ <- close$d
  icloseI  <- close$i
  jcloseJ  <- close$j
        
# Compute estimates by each of the selected edge corrections.
        
  if(any(correction == "none")) {
    # uncorrected! 
    wh <- whist(dcloseIJ, breaks$val)  # no weights
    numKun <- cumsum(wh)
    denKun <- lambdaI * lambdaJ * areaW
    Kun <- numKun/denKun
    K <- bind.fv(K, data.frame(un=Kun), "{hat(%s)[%s]^{un}}(r)",
                 "uncorrected estimate of %s",
                 "un")
    if(ratio) {
      # save numerator and denominator
      numK <- bind.fv(numK, data.frame(un=numKun), "{hat(%s)[%s]^{un}}(r)",
                 "numerator of uncorrected estimate of %s",
                 "un")
      denK <- bind.fv(denK, data.frame(un=denKun), "{hat(%s)[%s]^{un}}(r)",
                 "denominator of uncorrected estimate of %s",
                 "un")
    }

  }
  if(any(correction == "border" | correction == "bord.modif")) {
    # border method
    # distance to boundary from each point of type I
    bI <- bdist.points(XI)
    # distance to boundary from first element of each (i, j) pair
    bcloseI <- bI[icloseI]
    # apply reduced sample algorithm
    RS <- Kount(dcloseIJ, bcloseI, bI, breaks)
    if(any(correction == "bord.modif")) {
      denom.area <- eroded.areas(W, r)
      numKbm <- RS$numerator
      denKbm <- denom.area * nI * nJ
      Kbm <- numKbm/denKbm
      K <- bind.fv(K, data.frame(bord.modif=Kbm), "{hat(%s)[%s]^{bordm}}(r)",
                   "modified border-corrected estimate of %s",
                   "bord.modif")
      if(ratio) {
        # save numerator and denominator
        numK <- bind.fv(numK, data.frame(bord.modif=numKbm),
                        "{hat(%s)[%s]^{bordm}}(r)",
                        "numerator of modified border-corrected estimate of %s",
                        "bord.modif")
        denK <- bind.fv(denK, data.frame(bord.modif=denKbm),
                        "{hat(%s)[%s]^{bordm}}(r)",
                        "denominator of modified border-corrected estimate of %s",
                        "bord.modif")
      }
    }
    if(any(correction == "border")) {
      numKb <- RS$numerator
      denKb <- lambdaJ * RS$denom.count
      Kb <- numKb/denKb
      K <- bind.fv(K, data.frame(border=Kb), "{hat(%s)[%s]^{bord}}(r)",
                   "border-corrected estimate of %s",
                   "border")
      if(ratio) {
        numK <- bind.fv(numK, data.frame(border=numKb),
                        "{hat(%s)[%s]^{bord}}(r)",
                        "numerator of border-corrected estimate of %s",
                        "border")
        denK <- bind.fv(denK, data.frame(border=denKb),
                        "{hat(%s)[%s]^{bord}}(r)",
                        "denominator of border-corrected estimate of %s",
                        "border")
      }
    }
  }
  if(any(correction == "translate")) {
    # translation correction
    edgewt <- edge.Trans(XI[icloseI], XJ[jcloseJ], paired=TRUE)
    wh <- whist(dcloseIJ, breaks$val, edgewt)
    numKtrans <- cumsum(wh)
    denKtrans <- lambdaI * lambdaJ * areaW
    Ktrans <- numKtrans/denKtrans
    rmax <- diameter(W)/2
    Ktrans[r >= rmax] <- NA
    K <- bind.fv(K, data.frame(trans=Ktrans), "{hat(%s)[%s]^{trans}}(r)", 
                 "translation-corrected estimate of %s",
                 "trans")
    if(ratio) {
      numK <- bind.fv(numK, data.frame(trans=numKtrans),
                      "{hat(%s)[%s]^{trans}}(r)",
                      "numerator of translation-corrected estimate of %s",
                      "trans")
      denK <- bind.fv(denK, data.frame(trans=denKtrans),
                      "{hat(%s)[%s]^{trans}}(r)",
                      "denominator of translation-corrected estimate of %s",
                      "trans")
    }
  }
  if(any(correction == "isotropic")) {
    # Ripley isotropic correction
    edgewt <- edge.Ripley(XI[icloseI], matrix(dcloseIJ, ncol=1))
    wh <- whist(dcloseIJ, breaks$val, edgewt)
    numKiso <- cumsum(wh)
    denKiso <- lambdaI * lambdaJ * areaW
    Kiso <- numKiso/denKiso
    rmax <- diameter(W)/2
    Kiso[r >= rmax] <- NA
    K <- bind.fv(K, data.frame(iso=Kiso), "{hat(%s)[%s]^{iso}}(r)",
                 "Ripley isotropic correction estimate of %s",
                 "iso")
   if(ratio) {
      numK <- bind.fv(numK, data.frame(iso=numKiso), "{hat(%s)[%s]^{iso}}(r)",
                      "numerator of Ripley isotropic correction estimate of %s",
                      "iso")
      denK <- bind.fv(denK, data.frame(iso=denKiso), "{hat(%s)[%s]^{iso}}(r)",
                      "denominator of Ripley isotropic correction estimate of %s",
                      "iso")
    }
  }
  # default is to display them all
  formula(K) <- . ~ r
  unitname(K) <- unitname(X)
  
  if(ratio) {
    # finish up numerator & denominator
    formula(numK) <- formula(denK) <- . ~ r
    unitname(numK) <- unitname(denK) <- unitname(K)
    # tack on to result
    K <- rat(K, numK, denK, check=FALSE)
  }
  return(K)
}

#
# areadiff.R
#
#  $Revision: 1.38 $  $Date: 2019/01/20 08:46:55 $
#
# Computes sufficient statistic for area-interaction process
#
# Invokes areadiff.c
#
# areaLoss = area lost by removing X[i] from X

areaLoss <- function(X, r, ..., W=as.owin(X),
                     subset=NULL, exact=FALSE,
                     ngrid=spatstat.options("ngrid.disc")) {
  if(exact)
    areaLoss.diri(X, r, ..., W=W, subset=subset)
  else
    areaLoss.grid(X, r, ..., W=W, subset=subset, ngrid=ngrid)
}

# areaGain = area gained by adding u[i] to X

areaGain <- function(u, X, r, ..., W=as.owin(X), exact=FALSE,
                     ngrid=spatstat.options("ngrid.disc")) {
  if(exact)
    areaGain.diri(u, X, r, ..., W=W)
  else
    areaGain.grid(u, X, r, W=W, ..., ngrid=ngrid)
}

#////////////////////////////////////////////////////////////
#    algorithms using polygon geometry
#///////////////////////////////////////////////////////////

areaLoss.poly <- function(X, r, ..., W=as.owin(X), subset=NULL,
                            splitem=TRUE) {
  check.1.real(r)
  nX <- npoints(X)
  if(r <= 0 || nX == 0) return(numeric(nX))
  cooX <- coords(X)
  if(useW <- is.owin(W))
    W <- as.polygonal(W)
  #' initialise result
  result <- rep(pi * r^2, nX)
  wanted <- 1:nX
  if(!is.null(subset))
    wanted <- wanted[subset]
  #' split into connected components
  if(splitem) {
    Y <- connected(X, 2 * r)
    Z <- split(Y)
    V <- lapply(Z, areaLoss.poly, r=r, W=W, splitem=FALSE)
    return(unsplit(V, marks(Y))[wanted])
  }
  #' determine which pairs of points interact
  cl <- closepairs(X, 2 * r, what="indices")
  if(length(cl$i) == 0)
    return(result[wanted])
  #' determine scale parameters for polyclip
  p <- commonPolyclipArgs(Frame(X))
  #' template disc
  ball0 <- disc(r, c(0,0), ...)
  #' discs centred on data points
  balls <- vector(mode="list", length=nX)
  for(i in seq_len(nX))
    balls[[i]] <- shift(ball0, vec=cooX[i,])
  balls <- as.solist(balls, check=FALSE)
  #' start computin'
  for(i in wanted) {
    jj <- cl$j[cl$i == i]
    nn <- length(jj)
    if(nn > 0) {
      #' union of balls close to i
      u <- if(nn == 1) balls[[ jj ]] else union.owin(balls[jj], p=p)
      #' subtract from ball i
      v <- setminus.owin(balls[[i]], u)
      #' clip to window
      if(useW) 
        v <- intersect.owin(v, W)
      #' compute
      result[i] <- area(v)
    }
  }
  return(result[wanted])
}

#////////////////////////////////////////////////////////////
#    algorithms using Dirichlet tessellation
#///////////////////////////////////////////////////////////

areaLoss.diri <- function(X, r, ..., W=as.owin(X), subset=NULL) {
  stopifnot(is.ppp(X))
  npts <- npoints(X)
  if(is.matrix(r)) {
    if(sum(dim(r) > 1) > 1)
      stop("r should be a vector or single value")
    r <- as.vector(r)
  }
  nr <- length(r)
  if(npts == 0)
    return(matrix(, nrow=0, ncol=nr))
  else if(npts == 1) 
    return(matrix(discpartarea(X, r, W), nrow=1))
  #' set up output array
  indices <- 1L:npts
  if(!is.null(subset))
    indices <- indices[subset]
  out <- matrix(0, nrow=length(indices), ncol=nr)
  #' handle duplicate points
  retain <- !duplicated(X)
  getzero <- (multiplicity(X) > 1)
  uX <- X[retain]
  newserial <- cumsum(retain)
  # dirichlet neighbour relation in entire pattern 
  w <- X$window
  dd <- deldir(uX$x, uX$y, rw=c(w$xrange, w$yrange))
  a <- dd$delsgs[,5L]
  b <- dd$delsgs[,6L]
  pir2 <- pi * r^2
  for(k in seq_along(indices)) {
    ind <- indices[k]
    if(!getzero[ind]) {
      #' find serial number in uX
      i <- newserial[ind]
      #' find all Delaunay neighbours of i 
      jj <- c(b[a==i], a[b==i])
      jj <- sortunique(jj)
      #' extract only these points
      Yminus <- uX[jj]
      Yplus  <- uX[c(jj, i)]
      #' dilate
      aplus <- dilated.areas(Yplus, r, W, exact=TRUE, ...)
      aminus <- dilated.areas(Yminus, r, W, exact=TRUE, ...)
      areas <- aplus - aminus
      #' area/(pi * r^2) must be positive and nonincreasing
      y <- ifelseAX(r == 0, 1, areas/pir2)
      y <- pmin.int(1, y)
      ok <- is.finite(y)
      y[ok] <- rev(cummax(rev(y[ok])))
      areas <- pmax.int(0, y * pir2)
      #' save
      out[k, ] <- areas
    }
  }
  return(out)
}

areaGain.diri <- function(u, X, r, ..., W=as.owin(X), verbose=FALSE) {
  stopifnot(is.ppp(X))
  Y <- as.ppp(u, W=W)
  nX <- X$n
  nY <- Y$n
  if(is.matrix(r)) {
    if(sum(dim(r) > 1) > 1)
      stop("r should be a vector or single value")
    r <- as.vector(r)
  }
  nr <- length(r)
  if(nY == 0)
    return(matrix(, nrow=0, ncol=nr))
  if(nX == 0)
    return(matrix(pi * r^2, nrow=nY, ncol=nr, byrow=TRUE))
  if(verbose)
    splat("areaGain,",
          nY, ngettext(nY, "point,", "points,"),
          nr, ngettext(nr, "rvalue", "r values"))
  out <- matrix(0, nrow=nY, ncol=nr)
  pir2 <- pi * r^2
  wbox <- as.rectangle(as.owin(X))
  #
  state <- list()
  for(i in 1L:nY) {
    if(verbose) state <- progressreport(i, nY, state=state)
    V <- superimpose(Y[i], X, W=wbox, check=FALSE)
    # Dirichlet neighbour relation for V
    dd <- deldir(V$x, V$y, rw=c(wbox$xrange, wbox$yrange))
    aa <- dd$delsgs[,5L]
    bb <- dd$delsgs[,6L]
    # find all Delaunay neighbours of Y[1] in V
    jj <- c(bb[aa==1L], aa[bb==1L])
    jj <- sortunique(jj)
    # extract only these points
    Zminus <- V[jj]
    Zplus  <- V[c(1, jj)]
    # dilate
    aplus <- dilated.areas(Zplus, r, W, exact=TRUE)
    aminus <- dilated.areas(Zminus, r, W, exact=TRUE)
    areas <- aplus - aminus
    # area/(pi * r^2) must be in [0,1] and nonincreasing
    y <- ifelseAX(r == 0, 1, areas/pir2)
    y <- pmin.int(1, y)
    ok <- is.finite(y)
    y[ok] <- rev(cummax(rev(y[ok])))
    areas <- pmax.int(0, y * pir2)
    # save
    out[i,] <- areas
  }
  return(out)
}

#////////////////////////////////////////////////////////////////////////
#    alternative implementations using grid counting in C
#////////////////////////////////////////////////////////////////////////

areaGain.grid <- function(u, X, r, ..., W=NULL, ngrid=spatstat.options("ngrid.disc")) {
  verifyclass(X, "ppp")
  u <- as.ppp(u, W=as.owin(X))
  stopifnot(is.numeric(r) && all(is.finite(r)) && all(r >= 0))
  #
  nu <- u$n
  nr <- length(r)
  if(nr == 0)
    return(numeric(0))
  rmax <- max(r)
  #
  constrain <- !is.null(W)
  if(constrain && (W$type != "rectangle")) {
    # Constrained to an irregular window
    # initialise to value for small-r
    result <- matrix(pi * r^2, nrow=nu, ncol=nr, byrow=TRUE)    
    # vector of radii below which b(u,r) is disjoint from U(X,r)
    rcrit.u <- nncross(u, X, what="dist")/2
    rcrit.min <- min(rcrit.u)
    #' determine pixel resolution
    eps <- unclass(as.mask(Window(X), ...))[c("xstep", "ystep")]
    eps <- as.numeric(eps)
    eps <- eps * min(1, (rmax/4)/max(eps))
    #' Use distance transform and set covariance
    D <- distmap(X, eps=eps)
    DW <- D[W, drop=FALSE]
    # distance from (0,0) - thresholded to make digital discs
    discWin <- owin(c(-rmax,rmax),c(-rmax,rmax))
    discWin <- as.mask(discWin, eps=eps)
    rad <- as.im(function(x,y){sqrt(x^2+y^2)}, W=discWin)
    # 
    for(j in which(r > rcrit.min)) {
      # rj is above the critical radius rcrit.u[i] for at least one point u[i]
      rj <- r[j]
      if(any(above <- (rj > rcrit.u))) {
        Uncovered  <- levelset(DW, rj, ">")
        DiscRj     <- levelset(rad, rj, "<=")
        AreaGainIm <- setcov(Uncovered, DiscRj)
        result[above, j] <- safelookup(AreaGainIm, u[above])
      }
    }
    return(result)
  }
  #
  #
  xx <- X$x
  yy <- X$y
  result <- matrix(, nrow=nu, ncol=nr)
  #
  for(i in 1L:nu) {
    # shift u[i] to origin
    xu <- u$x[i]
    yu <- u$y[i]
    xshift <- xx - xu
    yshift <- yy - yu
    # find points within distance 2 rmax of origin
    close <- (xshift^2 + yshift^2 < 4 * rmax^2)
    nclose <- sum(close)
    # invoke C routine
    if(!constrain) {
      z <- .C("areadifs",
              rad = as.double(r),
              nrads = as.integer(nr),
              x   = as.double(xshift[close]),
              y   = as.double(yshift[close]),
              nn  = as.integer(nclose),
              ngrid = as.integer(ngrid),
              answer = as.double(numeric(nr)),
              PACKAGE = "spatstat")
      result[i,] <- z$answer
    } else {
      z <- .C("areaBdif",
              rad = as.double(r),
              nrads = as.integer(nr),
              x   = as.double(xshift[close]),
              y   = as.double(yshift[close]),
              nn  = as.integer(nclose),
              ngrid = as.integer(ngrid),
              x0 = as.double(W$xrange[1L] - xu),
              y0 = as.double(W$yrange[1L] - yu),
              x1 = as.double(W$xrange[2L] - xu),
              y1 = as.double(W$yrange[2L] - yu),
              answer = as.double(numeric(nr)),
              PACKAGE = "spatstat")
      result[i,] <- z$answer
    }
  }
  return(result)
}

areaLoss.grid <- function(X, r, ...,
                          W=as.owin(X), subset=NULL,
                          method = c("count", "distmap"),
                          ngrid = spatstat.options("ngrid.disc"),
                          exact = FALSE) {
  verifyclass(X, "ppp")
  n <- npoints(X)
  nr <- length(r)
  indices <- if(is.null(subset)) 1L:n else (1L:n)[subset]
  answer <- matrix(, nrow=length(indices), ncol=nr)
  if(missing(method)) {
    method <- if(nr <= 20 || exact) "count" else "distmap"
  } else method <- match.arg(method)
  switch(method,
         count = {
           # one value of r: use grid-counting
           for(k in seq_along(indices)) {
             i <- indices[k]
             answer[k,] <- areaGain(X[i], X[-i], r, W=W,
                                    ngrid=ngrid, exact=exact, ...)
           }
         },
         distmap = {
           # Many values of r: use distance transform
           D <- distmap(X, ...)
           DW <- D[W, drop=FALSE]
           a <- area(Window(DW))
           # empirical cdf of distance values
           FW <- ecdf(DW[drop=TRUE])
           # radii below which there are no overlaps
           rcrit <- nndist(X)/2
           for(k in seq_along(indices)) {
             i <- indices[k]
             Di <- distmap(X[-i], ...)
             FiW <- ecdf(Di[W, drop=TRUE])
             answer[k, ] <-
               ifelseXY(r > rcrit[i], a * (FW(r) - FiW(r)), pi * r^2)
           }
         })
  return(answer)
}

#
#  is.subset.owin.R
#
#  $Revision: 1.16 $   $Date: 2019/03/01 06:02:27 $
#
#  Determine whether a window is a subset of another window
#
#  is.subset.owin()
#

is.subset.owin <- local({
  
  is.subset.owin <- function(A, B) {
    A <- as.owin(A)
    B <- as.owin(B)

    if(identical(A, B))
      return(TRUE)

    A <- rescue.rectangle(A)
    B <- rescue.rectangle(B)
  
    if(is.rectangle(B)) {
      # Some cases can be resolved using convexity of B
    
      # (1) A is also a rectangle
      if(is.rectangle(A)) {
        xx <- A$xrange[c(1L,2L,2L,1L)]
        yy <- A$yrange[c(1L,1L,2L,2L)]
        ok <- inside.owin(xx, yy, B)
        return(all(ok))
      } 
      # (2) A is polygonal
      # Then A is a subset of B iff,
      # for every constituent polygon of A with positive sign,
      # the vertices are all in B
      if(is.polygonal(A)) {
        ok <- unlist(lapply(A$bdry, okpolygon, B=B))
        return(all(ok))
      }
      # (3) Feeling lucky
      # Test whether the bounding box of A is a subset of B
      # Then a fortiori, A is a subset of B
      AA <- boundingbox(A)
      if(is.subset.owin(AA, B))
        return(TRUE)
    }

    if(!is.mask(A) && !is.mask(B)) {
      ## rectangles or polygonal domains
      if(!all(inside.owin(vertices(A), , B)))
        return(FALSE)
      ## all vertices of A are inside B.
      if(is.convex(B)) return(TRUE)
      ## check for boundary crossings
      bx <- crossing.psp(edges(A), edges(B))
      if(npoints(bx) > 0) return(FALSE)
      ## Absence of boundary crossings is sufficient if B has no holes
      if(length(B$bdry) == 1 || !any(sapply(B$bdry, is.hole.xypolygon)))
        return(TRUE)
      ## Compare area of intersection with area of putative subset
      ## (these are subject to numerical rounding error)
      areaA <- area(A)
      if(overlap.owin(A,B) >= areaA ||
         overlap.owin(B,A) >= areaA) return(TRUE)
      ## continue...
    }
  
   # Discretise
    a <- as.mask(A)
    b <- as.mask(B)
    rxy <- rasterxy.mask(a, drop=TRUE)
    xx <- rxy$x
    yy <- rxy$y
    ok <- inside.owin(xx, yy, b)
    return(all(ok))
    
  }

  okpolygon <- function(a, B) {
    if(Area.xypolygon(a) < 0) return(TRUE)
    ok <- inside.owin(a$x, a$y, B)
    return(all(ok))
  }

  is.subset.owin
})

#'
#'    polygood.R
#'
#'   Check validity of polygon data
#'
#'  $Revision: 1.2 $  $Date: 2017/06/05 10:31:58 $
#'

#' check validity of a polygonal owin

owinpolycheck <- function(W, verbose=TRUE) {
  verifyclass(W, "owin")
  stopifnot(W$type == "polygonal")

  # extract stuff
  B <- W$bdry
  npoly <- length(B)
  outerframe <- owin(W$xrange, W$yrange)
  # can't use as.rectangle here; we're still checking validity
  boxarea.mineps <- area.owin(outerframe) * (1 - 0.00001)

  # detect very large datasets
  BS <- object.size(B)
  blowbyblow <- verbose && (BS > 1e4 || npoly > 20)
  #
  
  answer <- TRUE
  notes <- character(0)
  err <- character(0)
  
  # check for duplicated points, self-intersection, outer frame
  if(blowbyblow) {
    cat(paste("Checking", npoly, ngettext(npoly, "polygon...", "polygons...")))
    pstate <- list()
  }

  dup <- self <- is.box <- logical(npoly)
  
  for(i in 1:npoly) {
    if(blowbyblow && npoly > 1L)
      pstate <- progressreport(i, npoly, state=pstate)
    Bi <- B[[i]]
    # check for duplicated vertices
    dup[i] <- as.logical(anyDuplicated(ppp(Bi$x, Bi$y,
                                           window=outerframe, check=FALSE)))
    if(dup[i] && blowbyblow)
      message(paste("Polygon", i, "contains duplicated vertices"))
    # check for self-intersection
    self[i] <- xypolyselfint(B[[i]], proper=TRUE, yesorno=TRUE)
    if(self[i] && blowbyblow)
      message(paste("Polygon", i, "is self-intersecting"))
    # check whether one of the current boundary polygons
    # is the bounding box itself (with + sign)
    is.box[i] <- (length(Bi$x) == 4) && (Area.xypolygon(Bi) >= boxarea.mineps)
  }
  if(blowbyblow)
    cat("done.\n")
  
  if((ndup <- sum(dup)) > 0) {
    whinge <- paste(ngettext(ndup, "Polygon", "Polygons"),
                    if(npoly == 1L) NULL else
                    commasep(which(dup)), 
                    ngettext(ndup, "contains", "contain"),
                    "duplicated vertices")
    notes <- c(notes, whinge)
    err <- c(err, "duplicated vertices")
    if(verbose) 
      message(whinge)
    answer <- FALSE
  }
  
  if((nself <- sum(self)) > 0) {
    whinge <-  paste(ngettext(nself, "Polygon", "Polygons"),
                     if(npoly == 1L) NULL else
                     commasep(which(self)),
                     ngettext(nself, "is", "are"),
                     "self-intersecting")
    notes <- c(notes, whinge)
    if(verbose) 
      message(whinge)
    err <- c(err, "self-intersection")
    answer <- FALSE
  }
  
  if(sum(is.box) > 1L) {
    answer <- FALSE
    whinge <- paste("Polygons",
                    commasep(which(is.box)),
                    "coincide with the outer frame")
    notes <- c(notes, whinge)
    err <- c(err, "polygons duplicating the outer frame")
  }
  
  # check for crossings between different polygons
  cross <- matrix(FALSE, npoly, npoly)
  if(npoly > 1L) {
    if(blowbyblow) {
      cat(paste("Checking for cross-intersection between",
                npoly, "polygons..."))
      pstate <- list()
    }
    P <- lapply(B, xypolygon2psp, w=outerframe, check=FALSE)
    for(i in seq_len(npoly-1L)) {
      if(blowbyblow)
        pstate <- progressreport(i, npoly-1L, state=pstate)
      Pi <- P[[i]]
      for(j in (i+1L):npoly) {
        crosses <- if(is.box[i] || is.box[j]) FALSE else {
          anycrossing.psp(Pi, P[[j]])
        }
        cross[i,j] <- cross[j,i] <- crosses
        if(crosses) {
          answer <- FALSE
          whinge <- paste("Polygons", i, "and", j, "cross over")
          notes <- c(notes, whinge)
          if(verbose) 
            message(whinge)
          err <- c(err, "overlaps between polygons")
        }
      }
    }
    if(blowbyblow)
      cat("done.\n")
  }

  err <- unique(err)
  attr(answer, "notes") <- notes
  attr(answer, "err") <-  err
  return(answer)
}

#' check for self-intersections in an xypolygon

xypolyselfint <- function(p, eps=.Machine$double.eps,
                          proper=FALSE, yesorno=FALSE, checkinternal=FALSE) {
  verify.xypolygon(p)
  n <- length(p$x)
  verbose <- (n > 1000)
  if(verbose)
    cat(paste("[Checking polygon with", n, "edges..."))
  x0 <- p$x
  y0 <- p$y
  dx <- diff(x0[c(1:n,1L)])
  dy <- diff(y0[c(1:n,1L)])
  if(yesorno) {
    # get a yes-or-no answer
    answer <- .C("xypsi",
                 n=as.integer(n),
                 x0=as.double(x0),
                 y0=as.double(y0),
                 dx=as.double(dx),
                 dy=as.double(dy),
                 xsep=as.double(2 * max(abs(dx))),
                 ysep=as.double(2 * max(abs(dy))),
                 eps=as.double(eps),
                 proper=as.integer(proper),
                 answer=as.integer(integer(1L)),
                 PACKAGE = "spatstat")$answer
    if(verbose)
      cat("]\n")
    return(answer != 0)
  }
  out <- .C("Cxypolyselfint",
            n=as.integer(n),
            x0=as.double(x0),
            y0=as.double(y0),
            dx=as.double(dx),
            dy=as.double(dy), 
            eps=as.double(eps),
            xx=as.double(numeric(n^2)),
            yy=as.double(numeric(n^2)),
            ti=as.double(numeric(n^2)),
            tj=as.double(numeric(n^2)),
            ok=as.integer(integer(n^2)),
            PACKAGE = "spatstat")

  uhoh <- (matrix(out$ok, n, n) != 0)
  if(proper) {
    # ignore cases where two vertices coincide 
    ti <- matrix(out$ti, n, n)[uhoh]
    tj <- matrix(out$tj, n, n)[uhoh]
    i.is.vertex <- (abs(ti) < eps) | (abs(ti - 1) < eps)
    j.is.vertex <- (abs(tj) < eps) | (abs(tj - 1) < eps)
    dup <- i.is.vertex & j.is.vertex
    uhoh[uhoh] <- !dup
  }
  if(checkinternal && any(uhoh != t(uhoh)))
    warning("Internal error: incidence matrix is not symmetric")
  xx <- matrix(out$xx, n, n)
  yy <- matrix(out$yy, n, n)
  uptri <- (row(uhoh) < col(uhoh))
  xx <- as.vector(xx[uhoh & uptri])
  yy <- as.vector(yy[uhoh & uptri])
  result <- list(x=xx, y=yy)
  if(verbose)
    cat("]\n")
  return(result)
}
  

##   edit.R
##
##   Methods for 'edit'
##
##   $Revision: 1.3 $ $Date: 2015/04/19 06:14:21 $

edit.ppp <- local({

  edit.ppp <- function(name, ...) {
    X <- name
    df <- as.data.frame(X)
    df <- as.data.frame(lapply(df, as.num.or.char))
    Y <- edit(df, ...)
    Z <- as.ppp(Y, W=Window(X))
    return(Z)
  }

  as.num.or.char <- function(x) {
    if (is.character(x)) x else
    if (is.numeric(x)) {
      storage.mode(x) <- "double"
      x
    } else as.character(x)
  }

  edit.ppp
})

edit.im <- function(name, ...) {
  X <- name
  M <- transmat(as.matrix(X), from="spatstat", to="European")
  Y <- as.data.frame(M)
  Z <- edit(Y, ...)
  X[] <- transmat(as.matrix(Z), from="European", to="spatstat")
  return(X)
}

  

#'
#'   covering.R
#'
#'  $Revision: 1.4 $  $Date: 2019/12/31 03:57:05 $
#'

covering <- function(W, r, ..., giveup=1000) {
  W <- as.owin(W)
  ## compute distance to boundary
  D <- distmap(W, invert=TRUE, ...)
  D <- D[W, drop=FALSE]
  M <- as.owin(D)
  pixstep <- max(M$xstep, M$ystep)
  ## very small distances
  if(r <= pixstep) {
    warning("r is smaller than the pixel resolution: returning pixel centres",
            call.=FALSE)
    xy <- rasterxy.mask(M, drop=TRUE)
    return(ppp(xy$x, xy$y, window=W, check=FALSE))
  }
  ## find the point of W farthest from the boundary
  X <- where.max(D)
  ## build a hexagonal grid through this point
  ruse <- if(is.convex(W)) r else (r * 2/3)
  ruse <- max(pixstep, ruse - pixstep)
  H <- hexgrid(W, ruse, offset=c(X$x, X$y), origin=c(0,0))
  if(npoints(H) == 0) H <- X
  ## this may not suffice if W is irregular
  for(i in 1:giveup) {
    DH <- distmap(H)
    if(max(DH) < ruse && npoints(H) > 0) return(H)
    Hnew <- where.max(DH)
    H <- superimpose(H, Hnew, W=W)
  }
  stop(paste("Failed to converge after adding", giveup, "points"), call.=FALSE)
}

#'
#'           otherpackages.R
#' 
#'    Dealing with other packages
#' 
#'    $Revision: 1.18 $  $Date: 2019/02/14 02:02:11 $

fft2D <- function(z, inverse=FALSE, west=fftwAvailable()) {
  if(west) return(fftwtools::fftw2d(data=z, inverse=inverse))
  return(stats::fft(z=z, inverse=inverse))
}

fftwAvailable <- function() {
  # including temporary check for recent version
  ok <- requireNamespace("fftwtools", quietly=TRUE)
  return(ok)
}

kraeverRandomFields <- function() {
  kraever("RandomFieldsUtils")
  kraever("RandomFields")
# should no longer be needed:  
#  capture.output(RandomFieldsUtils:::.onLoad())
#  capture.output(RandomFields:::.onLoad())
  return(invisible(NULL))
}

# require a namespace and optionally check whether it is attached
kraever <- function(package, fatal=TRUE) {
  if(!requireNamespace(package, quietly=TRUE)) {
    if(fatal)
      stop(paste("The package", sQuote(package), "is required"),
           call.=FALSE)
    return(FALSE)
  }
  if(spatstat.options(paste("check", package, "loaded", sep=".")) &&
    !isNamespaceLoaded(package)){
    if(fatal)
      stop(paste("The package", sQuote(package),
                 "must be loaded: please type",
                 sQuote(paste0("library", paren(package)))),
           call.=FALSE)
    return(FALSE)
  }
  return(TRUE)
}

getRandomFieldsModelGen <- function(model) {
  kraeverRandomFields()
  if(inherits(model, "RMmodelgenerator"))
    return(model)
  if(!is.character(model))
    stop(paste("'model' should be a character string",
               "or one of the functions in the RandomFields package",
               "with a name beginning 'RM'"),
         call.=FALSE)
  f <- switch(model,
              cauchy    = RandomFields::RMcauchy,
              exponential = ,
              exp       = RandomFields::RMexp,
              gencauchy = RandomFields::RMgencauchy,
              gauss     = RandomFields::RMgauss,
              gneiting  = RandomFields::RMgneiting,
              matern    = RandomFields::RMmatern,
              nugget    = RandomFields::RMnugget,
              spheric   = RandomFields::RMspheric,
              stable    = RandomFields::RMstable,
              whittle   = RandomFields::RMwhittle,
              {
                modgen <- try(getExportedValue("RandomFields", 
                                               paste0("RM", model)),
                              silent=TRUE)
                if(inherits(modgen, "try-error") ||
                   !inherits(modgen, "RMmodelgenerator"))
                  stop(paste("Model", sQuote(model), "is not recognised"))
                modgen
              })
  if(!is.function(f))
    stop(paste0("Unable to retrieve RandomFields::RM", model))
  return(f)
}

#
# colourtables.R
#
# support for colour maps and other lookup tables
#
# $Revision: 1.46 $ $Date: 2019/12/10 01:08:19 $
#

colourmap <- function(col, ..., range=NULL, breaks=NULL, inputs=NULL, gamma=1) {
  if(nargs() == 0) {
    ## null colour map
    f <- lut()
  } else {
    ## validate colour data 
    col2hex(col)
    ## store without conversion
    f <- lut(col, ..., range=range, breaks=breaks, inputs=inputs, gamma=gamma)
  }
  class(f) <- c("colourmap", class(f))
  f
}

lut <- function(outputs, ..., range=NULL, breaks=NULL, inputs=NULL, gamma=1) {
  if(nargs() == 0) {
    ## null lookup table
    f <- function(x, what="value"){NULL}
    class(f) <- c("lut", class(f))
    attr(f, "stuff") <- list(n=0)
    return(f)
  }
  if(is.null(gamma)) gamma <- 1
  n <- length(outputs)
  given <- c(!is.null(range), !is.null(breaks), !is.null(inputs))
  names(given) <- nama <- c("range", "breaks", "inputs")
  ngiven <- sum(given)
  if(ngiven == 0L)
    stop(paste("One of the arguments",
               commasep(sQuote(nama), "or"),
               "should be given"))
  if(ngiven > 1L) {
    offending <- nama[given]
    stop(paste("The arguments",
               commasep(sQuote(offending)),
               "are incompatible"))
  }
  if(!is.null(inputs)) {
    #' discrete set of input values mapped to output values
    if(n == 1L) {
      #' constant output
      n <- length(inputs)
      outputs <- rep(outputs, n)
    } else stopifnot(length(inputs) == length(outputs))
    stuff <- list(n=n, discrete=TRUE, inputs=inputs, outputs=outputs)
    f <- function(x, what="value") {
      m <- match(x, stuff$inputs)
      if(what == "index")
        return(m)
      cout <- stuff$outputs[m]
      return(cout)
    }
  } else {
    #' range of numbers, or date/time interval, mapped to colours
    #' determine type of domain
    timeclasses <- c("Date", "POSIXt")
    is.time <- inherits(range, timeclasses) || inherits(breaks, timeclasses)
    #' determine breaks
    if(is.null(breaks)) {
      breaks <- gammabreaks(range, n + 1L, gamma)
      gamma.used <- gamma
    } else {
      stopifnot(length(breaks) >= 2)
      if(length(outputs) == 1L) {
        n <- length(breaks) - 1L
        outputs <- rep(outputs, n)
      } else stopifnot(length(breaks) == length(outputs) + 1L)
      if(!all(diff(breaks) > 0))
        stop("breaks must be increasing")
      gamma.used <- NULL
    }
    stuff <- list(n=n, discrete=FALSE, breaks=breaks, outputs=outputs,
                  gamma=gamma.used)
    #' use appropriate function
    if(is.time) {
      f <- function(x, what="value") {
        x <- as.vector(as.numeric(x))
        z <- findInterval(x, stuff$breaks,
                          rightmost.closed=TRUE)
        if(what == "index")
          return(z)
        cout <- stuff$outputs[z]
        return(cout)
      }
    } else {
      f <- function(x, what="value") {
        stopifnot(is.numeric(x))
        x <- as.vector(x)
        z <- findInterval(x, stuff$breaks,
                          rightmost.closed=TRUE)
        if(what == "index")
          return(z)
        cout <- stuff$outputs[z]
        return(cout)
      }
    }
  }
  attr(f, "stuff") <- stuff
  class(f) <- c("lut", class(f))
  f
}

print.lut <- function(x, ...) {
  if(inherits(x, "colourmap")) {
    tablename <- "Colour map"
    outputname <- "colour"
  } else {
    tablename  <- "Lookup table"
    outputname <- "output"
  }
  stuff <- attr(x, "stuff")
  n <- stuff$n
  if(n == 0) {
    ## Null map
    cat(paste("Null", tablename, "\n"))
    return(invisible(NULL))
  }
  if(stuff$discrete) {
    cat(paste(tablename, "for discrete set of input values\n"))
    out <- data.frame(input=stuff$inputs, output=stuff$outputs)
  } else {
    b <- stuff$breaks
    cat(paste(tablename, "for the range", prange(b[c(1L,n+1L)]), "\n"))
    leftend  <- rep("[", n)
    rightend <- c(rep(")", n-1), "]")
    inames <- paste(leftend, b[-(n+1L)], ", ", b[-1L], rightend, sep="")
    out <- data.frame(interval=inames, output=stuff$outputs)
  }
  colnames(out)[2L] <- outputname
  print(out)
  if(!is.null(gamma <- stuff$gamma) && gamma != 1)
    cat(paste("Generated using gamma =", gamma, "\n"))
  invisible(NULL)
}

print.colourmap <- function(x, ...) {
  NextMethod("print")
}

summary.lut <- function(object, ...) {
  s <- attr(object, "stuff")
  if(inherits(object, "colourmap")) {
    s$tablename <- "Colour map"
    s$outputname <- "colour"
  } else {
    s$tablename  <- "Lookup table"
    s$outputname <- "output"
  }
  class(s) <- "summary.lut"
  return(s)
}

print.summary.lut <- function(x, ...) {
  n <- x$n
  if(n == 0) {
    cat(paste("Null", x$tablename, "\n"))
    return(invisible(NULL))
  }
  if(x$discrete) {
    cat(paste(x$tablename, "for discrete set of input values\n"))
    out <- data.frame(input=x$inputs, output=x$outputs)
  } else {
    b <- x$breaks
    cat(paste(x$tablename, "for the range", prange(b[c(1L,n+1L)]), "\n"))
    leftend  <- rep("[", n)
    rightend <- c(rep(")", n-1L), "]")
    inames <- paste(leftend, b[-(n+1L)], ", ", b[-1L], rightend, sep="")
    out <- data.frame(interval=inames, output=x$outputs)
  }
  colnames(out)[2L] <- x$outputname
  print(out)  
}

plot.colourmap <- local({

  # recognised additional arguments to image.default() and axis()
  
  imageparams <- c("main", "asp", "sub", "axes", "ann",
                   "cex", "font", 
                   "cex.axis", "cex.lab", "cex.main", "cex.sub",
                   "col.axis", "col.lab", "col.main", "col.sub",
                   "font.axis", "font.lab", "font.main", "font.sub")
  axisparams <- c("cex", 
                  "cex.axis", "cex.lab",
                  "col.axis", "col.lab",
                  "font.axis", "font.lab",
                  "las", "mgp", "xaxp", "yaxp",
                  "tck", "tcl", "xpd")

  linmap <- function(x, from, to) {
    dFrom <- as.numeric(diff(from))
    dTo <- as.numeric(diff(to))
    b <- dTo/dFrom
    if(is.nan(b)) b <- 0
    if(!is.finite(b)) stop("Internal error: Cannot map zero width interval")
    to[1L] + b * (x - from[1L])
  }

  ensurenumeric <- function(x) { if(is.numeric(x)) x else as.numeric(x) }

  # rules to determine the ribbon dimensions when one dimension is given
  widthrule <- function(heightrange, separate, n, gap) {
    dh <- diff(heightrange)
    if(separate || dh == 0) 1 else dh/10
  }
  heightrule <- function(widthrange, separate, n, gap) {
    dw <- diff(widthrange)
    if(dw == 0) 1 else (dw * (if(separate) (n + (n-1)*gap) else 10))
  }

  plot.colourmap <- function(x, ..., main,
                             xlim=NULL, ylim=NULL, vertical=FALSE, axis=TRUE,
                             labelmap=NULL, gap=0.25, add=FALSE,
                             increasing=NULL) {
    if(missing(main))
      main <- short.deparse(substitute(x))
    stuff <- attr(x, "stuff")
    col <- stuff$outputs
    n   <- stuff$n
    if(n == 0) {
      ## Null map
      return(invisible(NULL))
    }
    discrete <- stuff$discrete
    if(discrete) {
      check.1.real(gap, "In plot.colourmap")
      explain.ifnot(gap >= 0, "In plot.colourmap")
    }
    separate <- discrete && (gap > 0)
    if(is.null(labelmap)) {
      labelmap <- function(x) x
    } else if(is.numeric(labelmap) && length(labelmap) == 1L && !discrete) {
      labscal <- labelmap
      labelmap <- function(x) { x * labscal }
    } else stopifnot(is.function(labelmap))

    if(is.null(increasing))
      increasing <- !(discrete && vertical)
    reverse <- !increasing

    #' determine pixel entries 'v' and colour map breakpoints 'bks'
    #' to be passed to 'image.default'
    trivial <- FALSE
    if(!discrete) {
      # real numbers: continuous ribbon
      bks <- stuff$breaks
      rr <- range(bks)
      trivial <- (diff(rr) == 0)
      v <- if(trivial) rr[1] else
           seq(from=rr[1L], to=rr[2L], length.out=max(n+1L, 1024))
    } else if(!separate) {
      # discrete values: blocks of colour, run together
      v <- (1:n) - 0.5
      bks <- 0:n
      rr <- c(0,n)
    } else {
      # discrete values: separate blocks of colour
      vleft <- (1+gap) * (0:(n-1L))
      vright <- vleft + 1
      v <- vleft + 0.5
      rr <- c(0, n + (n-1)*gap)
    }
    # determine position of ribbon or blocks of colour
    if(is.null(xlim) && is.null(ylim)) {
      u <- widthrule(rr, separate, n, gap)
      if(!vertical) {
        xlim <- rr
        ylim <- c(0,u)
      } else {
        xlim <- c(0,u)
        ylim <- rr
      }
    } else if(is.null(ylim)) {
      if(!vertical) 
        ylim <- c(0, widthrule(xlim, separate, n, gap))
      else 
        ylim <- c(0, heightrule(xlim, separate, n, gap))
    } else if(is.null(xlim)) {
      if(!vertical) 
        xlim <- c(0, heightrule(ylim, separate, n, gap))
      else 
        xlim <- c(0, widthrule(ylim, separate, n, gap))
    } 

    # .......... initialise plot ...............................
    if(!add)
      do.call.matched(plot.default,
                      resolve.defaults(list(x=xlim, y=ylim,
                                            type="n", main=main,
                                            axes=FALSE, xlab="", ylab="",
                                            asp=1.0),
                                       list(...)))

    if(separate) {
      # ................ plot separate blocks of colour .................
      if(reverse) 
        col <- rev(col)
      if(!vertical) {
        # horizontal arrangement of blocks
        xleft <- linmap(vleft, rr, xlim)
        xright <- linmap(vright, rr, xlim)
        y <- ylim
        z <- matrix(1, 1L, 1L)
        for(i in 1:n) {
          x <- c(xleft[i], xright[i])
          do.call.matched(image.default,
                          resolve.defaults(list(x=ensurenumeric(x),
                                                y=ensurenumeric(y),
                                                z=z, add=TRUE),
                                       list(...),
                                       list(col=col[i])),
                      extrargs=imageparams)
                          
        }
      } else {
        # vertical arrangement of blocks
        x <- xlim 
        ylow <- linmap(vleft, rr, ylim)
        yupp <- linmap(vright, rr, ylim)
        z <- matrix(1, 1L, 1L)
        for(i in 1:n) {
          y <- c(ylow[i], yupp[i])
          do.call.matched(image.default,
                          resolve.defaults(list(x=ensurenumeric(x),
                                                y=ensurenumeric(y),
                                                z=z, add=TRUE),
                                       list(...),
                                       list(col=col[i])),
                      extrargs=imageparams)
                          
        }
      }
    } else {
      # ................... plot ribbon image .............................
      if(!vertical) {
        # horizontal colour ribbon
        x <- linmap(v, rr, xlim)
        y <- ylim
        z <- matrix(v, ncol=1L)
      } else {
        # vertical colour ribbon
        y <- linmap(v, rr, ylim)
        z <- matrix(v, nrow=1L)
        x <- xlim
      }
      #' deal with Date or integer values
      x <- ensurenumeric(x)
      if(!trivial) {
        if(any(diff(x) == 0)) 
          x <- seq(from=x[1L], to=x[length(x)], length.out=length(x))
        y <- ensurenumeric(y)
        if(any(diff(y) == 0)) 
          y <- seq(from=y[1L], to=y[length(y)], length.out=length(y))
        bks <- ensurenumeric(bks)
        if(any(diff(bks) <= 0)) {
          ok <- (diff(bks) > 0)
          bks <- bks[ok]
          col <- col[ok]
        }
      }
      if(reverse)
        col <- rev(col)
      do.call.matched(image.default,
                      resolve.defaults(list(x=x, y=y, z=z, add=TRUE),
                                       list(...),
                                       list(breaks=ensurenumeric(bks),
                                            col=col)),
                      extrargs=imageparams)
    }
    if(axis) {
      # ................. draw annotation ..................
      if(!vertical) {
          # add horizontal axis/annotation
        if(discrete) {
          la <- paste(labelmap(stuff$inputs))
          at <- linmap(v, rr, xlim)
        } else {
          la <- prettyinside(rr)
          at <- linmap(la, rr, xlim)
          la <- labelmap(la)
        }
        if(reverse)
          at <- rev(at)
        # default axis position is below the ribbon (side=1)
        sidecode <- resolve.1.default("side", list(...), list(side=1L))
        if(!(sidecode %in% c(1L,3L)))
          warning(paste("side =", sidecode,
                        "is not consistent with horizontal orientation"))
        pos <- c(ylim[1L], xlim[1L], ylim[2L], xlim[2L])[sidecode]
        # don't draw axis lines if plotting separate blocks
        lwd0 <- if(separate) 0 else 1
        # draw axis
        do.call.matched(graphics::axis,
                        resolve.defaults(list(...),
                                         list(side = 1L, pos = pos,
                                              at = ensurenumeric(at)),
                                         list(labels=la, lwd=lwd0)),
                        extrargs=axisparams)
      } else {
        # add vertical axis
        if(discrete) {
          la <- paste(labelmap(stuff$inputs))
          at <- linmap(v, rr, ylim)
        } else {
          la <- prettyinside(rr)
          at <- linmap(la, rr, ylim)
          la <- labelmap(la)
        }
        if(reverse)
          at <- rev(at)
        # default axis position is to the right of ribbon (side=4)
        sidecode <- resolve.1.default("side", list(...), list(side=4))
        if(!(sidecode %in% c(2L,4L)))
          warning(paste("side =", sidecode,
                        "is not consistent with vertical orientation"))
        pos <- c(ylim[1L], xlim[1L], ylim[2L], xlim[2L])[sidecode]
        # don't draw axis lines if plotting separate blocks
        lwd0 <- if(separate) 0 else 1
        # draw labels horizontally if plotting separate blocks
        las0 <- if(separate) 1 else 0
        # draw axis
        do.call.matched(graphics::axis,
                        resolve.defaults(list(...),
                                         list(side=4, pos=pos,
                                              at=ensurenumeric(at)),
                                         list(labels=la, lwd=lwd0, las=las0)),
                        extrargs=axisparams)
      }
    }
    invisible(NULL)
  }

  plot.colourmap
})


# Interpolate a colourmap or lookup table defined on real numbers

interp.colourmap <- function(m, n=512) {
  if(!inherits(m, "colourmap"))
    stop("m should be a colourmap")
  st <- attr(m, "stuff")
  if(st$discrete) {
    # discrete set of input values mapped to colours
    xknots <- st$inputs
    # Ensure the inputs are real numbers
    if(!is.numeric(xknots))
      stop("Cannot interpolate: inputs are not numerical values")
  } else {
    # interval of real line, chopped into intervals, mapped to colours
    # Find midpoints of intervals
    bks <- st$breaks
    nb <- length(bks)
    xknots <- (bks[-1L] + bks[-nb])/2
  }
  # corresponding colours in hsv coordinates
  yknots.hsv <- rgb2hsva(col2rgb(st$outputs, alpha=TRUE))
  # transform 'hue' from polar to cartesian coordinate
  # divide domain into n equal intervals
  xrange <- range(xknots)
  xbreaks <- seq(xrange[1L], xrange[2L], length=n+1L)
  xx <- (xbreaks[-1L] + xbreaks[-(n+1L)])/2
  # interpolate saturation and value in hsv coordinates
  yy.sat <- approx(x=xknots, y=yknots.hsv["s", ], xout=xx)$y
  yy.val <- approx(x=xknots, y=yknots.hsv["v", ], xout=xx)$y
  # interpolate hue by first transforming polar to cartesian coordinate
  yknots.hue <- 2 * pi * yknots.hsv["h", ]
  yy.huex <- approx(x=xknots, y=cos(yknots.hue), xout=xx)$y
  yy.huey <- approx(x=xknots, y=sin(yknots.hue), xout=xx)$y
  yy.hue <- (atan2(yy.huey, yy.huex)/(2 * pi)) %% 1
  # handle transparency
  yknots.alpha <- yknots.hsv["alpha", ]
  if(all(yknots.alpha == 1)) {
    ## opaque colours: form using hue, sat, val
    yy <- hsv(yy.hue, yy.sat, yy.val)
  } else {
    ## transparent colours: interpolate alpha
    yy.alpha <- approx(x=xknots, y=yknots.alpha, xout=xx)$y
    ## form colours using hue, sat, val, alpha
    yy <- hsv(yy.hue, yy.sat, yy.val, yy.alpha)    
  }
  # done
  f <- colourmap(yy, breaks=xbreaks)
  return(f)
}

interp.colours <- function(x, length.out=512) {
  y <- colourmap(x, range=c(0,1))
  z <- interp.colourmap(y, length.out)
  oo <- attr(z, "stuff")$outputs
  return(oo)
}

tweak.colourmap <- local({

  is.hex <- function(z) {
    is.character(z) &&
    all(nchar(z, keepNA=TRUE) %in% c(7L,9L)) &&
    identical(substr(z, 1L, 7L), substr(col2hex(z), 1L, 7L))
  }

  tweak.colourmap <- function(m, col, ..., inputs=NULL, range=NULL) {
    if(!inherits(m, "colourmap"))
      stop("m should be a colourmap")
    if(is.null(inputs) && is.null(range)) 
      stop("Specify either inputs or range")
    if(!is.null(inputs) && !is.null(range))
      stop("Do not specify both inputs and range")
    ## determine indices of colours to be changed
    if(!is.null(inputs)) {
      ix <- m(inputs, what="index")
    } else {
      if(!(is.numeric(range) && length(range) == 2 && diff(range) > 0))
        stop("range should be a numeric vector of length 2 giving (min, max)")
      if(length(col2hex(col)) != 1L)
        stop("When range is given, col should be a single colour value")
      ixr <- m(range, what="index")
      ix <- (ixr[1L]):(ixr[2L])
    }
    ## reassign colours
    st <- attr(m, "stuff")
    outputs <- st$outputs
    result.hex <- FALSE
    if(is.hex(outputs)) {
      ## convert replacement data to hex
      col <- col2hex(col)
      result.hex <- TRUE
    } else if(is.hex(col)) {
      ## convert existing data to hex
      outputs <- col2hex(outputs)
      result.hex <- TRUE
    } else if(!(is.character(outputs) && is.character(col))) {
      ## unrecognised format - convert both to hex
      outputs <- col2hex(outputs)
      col <- col2hex(col)
      result.hex <- TRUE
    }
    if(result.hex) {
      ## hex codes may be 7 or 9 characters
      outlen <- nchar(outputs)
      collen <- nchar(col)
      if(length(unique(c(outlen, collen))) > 1L) {
        ## convert all to 9 characters
        if(any(bad <- (outlen == 7))) 
          outputs[bad] <- paste0(outputs[bad], "FF")
        if(any(bad <- (collen == 7))) 
          col[bad] <- paste0(col[bad], "FF")
      }
    }
    ## Finally, replace
    outputs[ix] <- col
    st$outputs <- outputs
    attr(m, "stuff") <- st
    assign("stuff", st, envir=environment(m))
    return(m)
  }

  tweak.colourmap
})

colouroutputs <- function(x) {
  stopifnot(inherits(x, "colourmap"))
  attr(x, "stuff")$outputs
}

"colouroutputs<-" <- function(x, value) {
  stopifnot(inherits(x, "colourmap"))
  st <- attr(x, "stuff")
  col2hex(value) # validates colours
  st$outputs[] <- value
  attr(x, "stuff") <- st
  assign("stuff", st, envir=environment(x))
  return(x)
}

#'
#'     headtail.R
#'
#'   Methods for head() and tail()
#'
#'   $Revision: 1.1 $  $Date: 2016/12/20 01:11:29 $

head.tess <- head.psp <- head.ppx <- head.ppp <- function(x, n=6L, ...) {
  stopifnot(length(n) == 1L)
  xlen <- nobjects(x)
  n <- if (n < 0L) max(xlen + n, 0L) else min(n, xlen)
  x[seq_len(n)]
}

tail.tess <- tail.psp <- tail.ppx <- tail.ppp <- function (x, n = 6L, ...) {
  stopifnot(length(n) == 1L)
  xlen <- nobjects(x)
  n <- if (n < 0L) max(xlen + n, 0L) else min(n, xlen)
  x[seq.int(to = xlen, length.out = n)]
}

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#' spatstat/tests/package.R
#' Package information
#' $Revision: 1.2 $ $Date: 2020/04/30 05:23:52 $

if(FULLTEST) {
local({
  a <- bugfixes("book", show=FALSE)
  bugfixes(package="deldir")
})
}
## 
##    tests/percy.R
##
## Tests of Percus-Yevick approximations
##
##    $Revision: 1.3 $ $Date: 2020/04/30 05:23:52 $

if(FULLTEST) {
local({
  fit <- ppm(swedishpines ~1, DiggleGatesStibbard(6))
  K <- Kmodel(fit)
})
}

#'   tests/perspim.R
#'
#'   Check persp.im handling of NA, etc
#' 
#'   $Revision: 1.2 $  $Date: 2020/04/30 05:23:52 $

if(FULLTEST) {
local({
  set.seed(42)
  Z <- distmap(letterR, invert=TRUE)[letterR, drop=FALSE]
  X <- runifpoint(100, Frame(Z))
  M <- persp(Z, colin=Z, visible=TRUE, phi=50)
  perspPoints(X, Z=Z, M=M)
  P <- psp(c(2.360, 3.079, 2.211),
           c(0.934, 1.881, 2.184),
           c(2.337, 3.654, 3.274),
           c(1.829, 0.883, 2.093), window=letterR)
  perspSegments(P, Z=Z, M=M)
  
  persp(Z, colmap=rainbow)
  persp(Z, colmap=beachcolours, sealevel=mean(Z))
  persp(Z, colin=as.im(Z, dimyx=dim(Z)/4))
})
}
##
## tests/pixelgripes.R
##     Problems related to pixellation of windows
##
## $Revision: 1.5 $ $Date: 2020/04/30 05:23:52 $

if(FULLTEST) {
local({
  ## From Philipp Hunziker: bug in rNeymanScott (etc)
  ## Create an irregular window
  PM <- matrix(c(1,0,0.5,1,0,0), 3, 2, byrow=TRUE)
  P <- owin(poly=PM)
  ## Generate Matern points
  X <- rMatClust(50, 0.05, 5, win=P)
  ## Some distance function as a covariate
  distorigin <- function(x, y) { sqrt(x^2 + y^2) }
  ## No covariates: works fine
  fit0 <- kppm(X ~ 1, clusters="MatClust")
  Y0 <- simulate(fit0, retry=0)
  ## Covariates: Simulation fails
  fit1 <- kppm(X ~ distorigin, clusters="MatClust")
  Y1 <- simulate(fit1, retry=0)

  ## pixellate.ppp includes mapping from (x,y) to (row, col)
  Z <- pixellate(cells, savemap=TRUE)
  ind <- attr(Z, "map")
  m <- (as.matrix(Z))[ind]
  if(!all(m == 1)) stop("Coordinate mismatch in pixellate.ppp")
})
}
## 
## tests/polygons.R
##
##  $Revision: 1.5 $ $Date: 2020/04/30 05:23:52 $
##
if(ALWAYS) { # involves C code
local({
  co <- as.ppp(corners(letterR), letterR, check=FALSE)
  co[letterR]

  b <- letterR$bdry
  a <- sapply(b, xypolyselfint, yesorno=TRUE)
  a <- lapply(b, xypolyselfint, proper=TRUE)
  
  ## Simple example of self-crossing polygon
  x <- read.table("selfcross.txt", header=TRUE)
  y <- xypolyselfint(x)
})
}

#
#  mincontrast.R
#
#  Functions for estimation by minimum contrast
#

##################  base ################################

mincontrast <- local({

  ## objective function (in a format that is re-usable by other code)
  contrast.objective <- function(par, objargs, ...) {
    with(objargs, {
      theo <- theoretical(par=par, rvals, ...)
      if(!is.vector(theo) || !is.numeric(theo))
        stop("theoretical function did not return a numeric vector")
      if(length(theo) != nrvals)
        stop("theoretical function did not return the correct number of values")
      ## experimental
      if(!is.null(adjustment)) {
        theo <- adjustment$fun(theo=theo, par=par,
                               auxdata=adjustment$auxdata, ...)
        if(!is.vector(theo) || !is.numeric(theo))
	  stop("adjustment did not return a numeric vector")
        if(length(theo) != nrvals)
          stop("adjustment did not return the correct number of values")
      }
      ##
      discrep <- (abs(theo^qq - obsq))^pp
      value <- mean(discrep)
      value <- min(value, .Machine$double.xmax)
      return(value)
    })
  }

  mincontrast <- function(observed, theoretical, startpar,
                          ...,
                          ctrl=list(q = 1/4, p = 2, rmin=NULL, rmax=NULL),
                          fvlab=list(label=NULL, desc="minimum contrast fit"),
                          explain=list(dataname=NULL,
                            modelname=NULL, fname=NULL),
			  adjustment=NULL) {
    verifyclass(observed, "fv")

    stopifnot(is.function(theoretical))
    if(!any("par" %in% names(formals(theoretical))))
      stop(paste("Theoretical function does not include an argument called",
                 sQuote("par")))

    ## enforce defaults
    ctrl <- resolve.defaults(ctrl, list(q = 1/4, p = 2, rmin=NULL, rmax=NULL))
    fvlab <- resolve.defaults(fvlab,
                              list(label=NULL, desc="minimum contrast fit"))
    explain <- resolve.defaults(explain,
                                list(dataname=NULL, modelname=NULL, fname=NULL))
  
    ## extract vector of r values
    argu <- fvnames(observed, ".x")
    rvals <- observed[[argu]]
    
    ## determine range of r values
    rmin <- ctrl$rmin
    rmax <- ctrl$rmax
    if(!is.null(rmin) && !is.null(rmax)) 
      stopifnot(rmin < rmax && rmin >= 0)
    else {
      alim <- attr(observed, "alim") %orifnull% range(rvals)
      if(is.null(rmax)) rmax <- alim[2]
      if(is.null(rmin)) {
        rmin <- alim[1]
        if(rmin == 0 && identical(explain$fname,"g"))
          rmin <- rmax/1e3 # avoid artefacts at zero in pcf
      }
    }
    ## extract vector of observed values of statistic
    valu <- fvnames(observed, ".y")
    obs <- observed[[valu]]
    ## restrict to [rmin, rmax]
    if(max(rvals) < rmax)
      stop(paste("rmax=", signif(rmax,4),
                 "exceeds the range of available data",
                 "= [", signif(min(rvals),4), ",", signif(max(rvals),4), "]"))
    sub <- (rvals >= rmin) & (rvals <= rmax)
    rvals <- rvals[sub]
    obs <- obs[sub]
    ## sanity clause
    if(!all(ok <- is.finite(obs))) {
      whinge <- paste("Some values of the empirical function",
                      sQuote(explain$fname),
                      "were infinite or NA.")
      iMAX <- max(which(ok))
      iMIN <- min(which(!ok)) + 1
      if(iMAX > iMIN && all(ok[iMIN:iMAX])) {
        rmin <- rvals[iMIN]
        rmax <- rvals[iMAX]
        obs   <- obs[iMIN:iMAX]
        rvals <- rvals[iMIN:iMAX]
        sub[sub] <- ok
        warning(paste(whinge,
                      "Range of r values was reset to",
                      prange(c(rmin, rmax))),
                call.=FALSE)
      } else stop(paste(whinge, "Please choose a narrower range [rmin, rmax]"),
                  call.=FALSE)
    }
    ## pack data into a list
    objargs <- list(theoretical = theoretical,
                    rvals       = rvals,
                    nrvals      = length(rvals),
                    obsq        = obs^(ctrl$q),   ## for efficiency
                    qq          = ctrl$q,
                    pp          = ctrl$p,
                    rmin        = rmin,
                    rmax        = rmax,
		    adjustment  = adjustment)
    ## go
    minimum <- optim(startpar, fn=contrast.objective, objargs=objargs, ...)
    ## if convergence failed, issue a warning 
    signalStatus(optimStatus(minimum), errors.only=TRUE)
    ## evaluate the fitted theoretical curve
    fittheo <- theoretical(minimum$par, rvals, ...)
    ## pack it up as an `fv' object
    label <- fvlab$label %orifnull% "%s[fit](r)"
    desc  <- fvlab$desc
    fitfv <- bind.fv(observed[sub, ],
                     data.frame(fit=fittheo),
                     label, desc)
    if(!is.null(adjustment)) {
      adjtheo <- adjustment$fun(theo=fittheo,
      	                        par=minimum$par,
				auxdata=adjustment$auxdata, ...)
      fitfv <- bind.fv(fitfv,
                       data.frame(adjfit=adjtheo),
		       "%s[adjfit](r)",
		       paste("adjusted", desc))
    }				
    result <- list(par      = minimum$par,
                   fit      = fitfv,
                   opt      = minimum,
                   ctrl     = list(p=ctrl$p,q=ctrl$q,rmin=rmin,rmax=rmax),
                   info     = explain,
                   startpar = startpar,
                   objfun   = contrast.objective,
                   objargs  = objargs,
                   dotargs  = list(...))
    class(result) <- c("minconfit", class(result))
    return(result)
  }

  mincontrast
})

print.minconfit <- function(x, ...) {
  terselevel <- spatstat.options('terse')
  digits <- getOption('digits')
  ## explanatory
  cat(paste("Minimum contrast fit ",
            "(",
            "object of class ",
            dQuote("minconfit"),
            ")",
            "\n", sep=""))
  mo <- x$info$modelname
  fu <- x$info$fname
  da <- x$info$dataname
  cm <- x$covmodel
  if(!is.null(mo))
    cat("Model:", mo, fill=TRUE)
  if(!is.null(cm)) {
    ## Covariance/kernel model and nuisance parameters 
    cat("\t", cm$type, "model:", cm$model, fill=TRUE)
    margs <- cm$margs
    if(!is.null(margs)) {
      nama <- names(margs)
      tags <- ifelse(nzchar(nama), paste(nama, "="), "")
      tagvalue <- paste(tags, margs)
      splat("\t", cm$type, "parameters:",
            paste(tagvalue, collapse=", "))
    }
  }
  if(!is.null(fu) && !is.null(da))
    splat("Fitted by matching theoretical", fu, "function to", da)
  else {
    if(!is.null(fu))
      splat(" based on", fu)
    if(!is.null(da))
      splat(" fitted to", da)
  }

  if(waxlyrical('space', terselevel))
      cat("\n")
  ## Values
  splat("Internal parameters fitted by minimum contrast ($par):")
  print(x$par, ...)
  if(waxlyrical('space', terselevel))
      cat("\n")
  
  ## Handling new parameters
  isPCP <- x$isPCP %orifnull% x$internal$model!="lgcp"
  cpar <- x$clustpar
  if (!is.null(cpar)) {
    splat("Fitted",
          if(isPCP) "cluster" else "covariance",
          "parameters:")
    print(cpar, digits=digits)
  } else{
    ## Old modelpar field if necessary
    mp <- x$modelpar
    if(!is.null(mp)) {
      splat("Derived parameters of",
            if(!is.null(mo)) mo else "model",
            "($modelpar):")
      print(mp)
    }
  }
  if(length(mu <- x$mu)) {
    if(isPCP) {
      splat("Mean cluster size: ",
            if(is.numeric(mu)) paste(signif(mu, digits), "points") else
            if(is.im(mu)) "[pixel image]" else "[unknown]")
    } else {
      splat("Fitted mean of log of random intensity: ",
            if(is.numeric(mu)) signif(mu, digits) else
            if(is.im(mu)) "[pixel image]" else "[unknown]")
    }
  }
  if(waxlyrical('space', terselevel))
      cat("\n")
  ## Diagnostics
  printStatus(optimStatus(x$opt))
  ## Starting values
  if(waxlyrical('gory', terselevel)){
      cat("\n")
      splat("Starting values of parameters:")
      print(x$startpar)
      ## Algorithm parameters
      ct <- x$ctrl
      splat("Domain of integration:",
            "[",
            signif(ct$rmin,4),
            ",",
            signif(ct$rmax,4),
            "]")
      splat("Exponents:",
            "p=", paste(signif(ct$p, 3), ",",  sep=""),
            "q=", signif(ct$q,3))
  }
  invisible(NULL)
}
              

plot.minconfit <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  do.call(plot.fv,
          resolve.defaults(list(x$fit),
                           list(...),
                           list(main=xname)))
}

unitname.minconfit <- function(x) {
  unitname(x$fit)
}

"unitname<-.minconfit" <- function(x, value) {
  unitname(x$fit) <- value
  return(x)
}

as.fv.minconfit <- function(x) x$fit

######  convergence status of 'optim' object

optimStatus <- function(x, call=NULL) {
  cgce <- x$convergence
  neval <- x$counts[["function"]]
  switch(paste(cgce),
         "0" = {
           simpleMessage(
                         paste("Converged successfully after", 
                               neval, "function evaluations"),
                         call)
         },
         "1" = simpleWarning(
           paste("Iteration limit maxit was reached after",
                 neval, "function evaluations"),
           call),
         "10" = simpleWarning("Nelder-Mead simplex was degenerate", call),
         "51"= {
           simpleWarning(
                         paste("Warning message from L-BGFS-B method:",
                               sQuote(x$message)),
                         call)
         },
         "52"={
           simpleError(
                         paste("Error message from L-BGFS-B method:",
                               sQuote(x$message)),
                         call)
         },
         simpleWarning(paste("Unrecognised error code", cgce), call)
         )
}

#' general code for collecting status reports

signalStatus <- function(x, errors.only=FALSE) {
  stopifnot(inherits(x, "condition"))
  if(inherits(x, "error")) stop(x)
  if(inherits(x, "warning")) warning(x) 
  if(inherits(x, "message") && !errors.only) message(x)
  return(invisible(NULL))
}

printStatus <- function(x, errors.only=FALSE) {
  prefix <-
    if(inherits(x, "error")) "error: " else 
    if(inherits(x, "warning")) "warning: " else NULL
  if(!is.null(prefix) || !errors.only)
    cat(paste(prefix, conditionMessage(x), "\n", sep=""))
  return(invisible(NULL))
}

accumulateStatus <- function(x, stats=NULL) {
  values      <- stats$values      %orifnull% list()
  frequencies <- stats$frequencies %orifnull% integer(0)
  if(inherits(x, c("error", "warning", "message"))) {
    same <- unlist(lapply(values, identical, y=x))
    if(any(same)) {
      i <- min(which(same))
      frequencies[i] <- frequencies[i] + 1L
    } else {
      values <- append(values, list(x))
      frequencies <- c(frequencies, 1L)
    }
  }
  stats <- list(values=values, frequencies=frequencies)
  return(stats)
}

printStatusList <- function(stats) {
  with(stats,
       {
         for(i in seq_along(values)) {
           printStatus(values[[i]])
           fi <- frequencies[i]
           splat("\t", paren(paste(fi, ngettext(fi, "time", "times"))))
         }
       }
       )
  invisible(NULL)
}

  
############### applications (specific models) ##################


getdataname <- function(defaultvalue, ..., dataname=NULL) {
  if(!is.null(dataname)) dataname else defaultvalue
}
  
thomas.estK <- function(X, startpar=c(kappa=1,scale=1),
                        lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, ...) {

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)

  if(inherits(X, "fv")) {
    K <- X
    if(!identical(attr(K, "fname")[1], "K"))
      warning("Argument X does not appear to be a K-function")
  } else if(inherits(X, "ppp")) {
    K <- Kest(X)
    dataname <- paste("Kest(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  info <- spatstatClusterModelInfo("Thomas")
  startpar <- info$checkpar(startpar)
  theoret <- info$K
  
  result <- mincontrast(K, theoret, startpar,
                        ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)",
                          desc="minimum contrast fit of Thomas process"),
                        explain=list(dataname=dataname,
                          fname=attr(K, "fname"),
                          modelname="Thomas process"), ...)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("kappa", "sigma2")
  result$par <- par
  ## infer meaningful model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="Thomas")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  return(result)
}

lgcp.estK <- function(X, startpar=c(var=1,scale=1),
                      covmodel=list(model="exponential"), 
                      lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, ...) {

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)
  
  if(inherits(X, "fv")) {
    K <- X
    if(!identical(attr(K, "fname")[1], "K"))
      warning("Argument X does not appear to be a K-function")
  } else if(inherits(X, "ppp")) {
    K <- Kest(X)
    dataname <- paste("Kest(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  info <- spatstatClusterModelInfo("LGCP")
  startpar <- info$checkpar(startpar)

  ## digest parameters of Covariance model and test validity
  ph <- info$parhandler
  cmodel <- do.call(ph, covmodel)
  
  theoret <- info$K

  result <- mincontrast(K, theoret, startpar,
                        ctrl=list(q=q, p=p, rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)",
                          desc="minimum contrast fit of LGCP"),
                        explain=list(dataname=dataname,
                          fname=attr(K, "fname"),
                          modelname="log-Gaussian Cox process"),
                        ...,
                        model=cmodel$model,
                        margs=cmodel$margs)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("sigma2", "alpha")
  result$par <- par
  result$covmodel <- cmodel
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="lgcp")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  result$clustargs <- info$checkclustargs(cmodel$margs, old=FALSE)
  return(result)
}

matclust.estK <- function(X, startpar=c(kappa=1,scale=1),
                          lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, ...) {

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)

  if(inherits(X, "fv")) {
    K <- X
    if(!identical(attr(K, "fname")[1], "K"))
      warning("Argument X does not appear to be a K-function")
  } else if(inherits(X, "ppp")) {
    K <- Kest(X)
    dataname <- paste("Kest(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  info <- spatstatClusterModelInfo("MatClust")
  startpar <- info$checkpar(startpar)
  theoret <- info$K
  funaux <-  info$funaux
  
  result <- mincontrast(K, theoret, startpar,
                        ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)",
                          desc="minimum contrast fit of Matern Cluster process"),
                        explain=list(dataname=dataname,
                          fname=attr(K, "fname"),
                          modelname="Matern Cluster process"),
                        ...,
                        funaux=funaux)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("kappa", "R")
  result$par <- par
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="MatClust")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  return(result)
}

## versions using pcf (suggested by Jan Wild)

thomas.estpcf <- function(X, startpar=c(kappa=1,scale=1),
                          lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, ...,
                          pcfargs=list()){

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)

  if(inherits(X, "fv")) {
    g <- X
    if(!identical(attr(g, "fname")[1], "g"))
      warning("Argument X does not appear to be a pair correlation function")
  } else if(inherits(X, "ppp")) {
    g <- do.call(pcf.ppp, append(list(X), pcfargs))
    dataname <- paste("pcf(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  info <- spatstatClusterModelInfo("Thomas")
  startpar <- info$checkpar(startpar)
  theoret <- info$pcf
  
  ## avoid using g(0) as it may be infinite
  argu <- fvnames(g, ".x")
  rvals <- g[[argu]]
  if(rvals[1] == 0 && (is.null(rmin) || rmin == 0)) {
    rmin <- rvals[2]
  }
  result <- mincontrast(g, theoret, startpar,
                        ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
                        fvlab=list(
                          label="%s[fit](r)",
                          desc="minimum contrast fit of Thomas process"),
                        explain=list(
                          dataname=dataname,
                          fname=attr(g, "fname"),
                          modelname="Thomas process"), ...)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("kappa", "sigma2")
  result$par <- par
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="Thomas")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  return(result)
}

matclust.estpcf <- function(X, startpar=c(kappa=1,scale=1),
                            lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, ...,
                            pcfargs=list()){

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)

  if(inherits(X, "fv")) {
    g <- X
    if(!identical(attr(g, "fname")[1], "g"))
      warning("Argument X does not appear to be a pair correlation function")
  } else if(inherits(X, "ppp")) {
    g <- do.call(pcf.ppp, append(list(X), pcfargs))
    dataname <- paste("pcf(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  info <- spatstatClusterModelInfo("MatClust")
  startpar <- info$checkpar(startpar)
  theoret <- info$pcf
  funaux <-  info$funaux
  
  ## avoid using g(0) as it may be infinite
  argu <- fvnames(g, ".x")
  rvals <- g[[argu]]
  if(rvals[1] == 0 && (is.null(rmin) || rmin == 0)) {
    rmin <- rvals[2]
  }
  result <- mincontrast(g, theoret, startpar,
                        ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)",
                          desc="minimum contrast fit of Matern Cluster process"),
                        explain=list(dataname=dataname,
                          fname=attr(g, "fname"),
                          modelname="Matern Cluster process"),
                        ...,
                        funaux=funaux)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("kappa", "R")
  result$par <- par
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="MatClust")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  return(result)
}

lgcp.estpcf <- function(X, startpar=c(var=1,scale=1),
                      covmodel=list(model="exponential"), 
                        lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, ...,
                        pcfargs=list()) {
  
  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)
  
  if(inherits(X, "fv")) {
    g <- X
    if(!identical(attr(g, "fname")[1], "g"))
      warning("Argument X does not appear to be a pair correlation function")
  } else if(inherits(X, "ppp")) {
    g <- do.call(pcf.ppp, append(list(X), pcfargs))
    dataname <- paste("pcf(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  info <- spatstatClusterModelInfo("LGCP")
  startpar <- info$checkpar(startpar)

  ## digest parameters of Covariance model and test validity
  ph <- info$parhandler
  cmodel <- do.call(ph, covmodel)
  
  theoret <- info$pcf
  
  result <- mincontrast(g, theoret, startpar,
                        ctrl=list(q=q, p=p, rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)",
                          desc="minimum contrast fit of LGCP"),
                        explain=list(dataname=dataname,
                          fname=attr(g, "fname"),
                          modelname="log-Gaussian Cox process"),
                        ...,
                        model=cmodel$model,
                        margs=cmodel$margs)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("sigma2", "alpha")
  result$par <- par
  result$covmodel <- cmodel
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="lgcp")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  result$clustargs <- info$checkclustargs(cmodel$margs, old=FALSE)
  return(result)
}


cauchy.estK <- function(X, startpar=c(kappa=1,scale=1),
                        lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, ...) {

## omega: scale parameter of Cauchy kernel function
## eta: scale parameter of Cauchy pair correlation function
## eta = 2 * omega

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)

  if(inherits(X, "fv")) {
    K <- X
    if(!identical(attr(K, "fname")[1], "K"))
      warning("Argument X does not appear to be a K-function")
  } else if(inherits(X, "ppp")) {
    K <- Kest(X)
    dataname <- paste("Kest(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  info <- spatstatClusterModelInfo("Cauchy")
  startpar <- info$checkpar(startpar)
  theoret <- info$K

  desc <- "minimum contrast fit of Neyman-Scott process with Cauchy kernel"
  result <- mincontrast(K, theoret, startpar,
                        ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)", desc=desc),
                        explain=list(dataname=dataname,
                          fname=attr(K, "fname"),
                          modelname="Cauchy process"), ...)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("kappa", "eta2")
  result$par <- par
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="Cauchy")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  return(result)
}


cauchy.estpcf <- function(X, startpar=c(kappa=1,scale=1),
                          lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, ...,
                          pcfargs=list()) {

## omega: scale parameter of Cauchy kernel function
## eta: scale parameter of Cauchy pair correlation function
## eta = 2 * omega

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)

  if(inherits(X, "fv")) {
    g <- X
    if(!identical(attr(g, "fname")[1], "g"))
      warning("Argument X does not appear to be a pair correlation function")
  } else if(inherits(X, "ppp")) {
    g <- do.call(pcf.ppp, append(list(X), pcfargs))
    dataname <- paste("pcf(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  info <- spatstatClusterModelInfo("Cauchy")
  startpar <- info$checkpar(startpar)
  theoret <- info$pcf

  ## avoid using g(0) as it may be infinite
  argu <- fvnames(g, ".x")
  rvals <- g[[argu]]
  if(rvals[1] == 0 && (is.null(rmin) || rmin == 0)) {
    rmin <- rvals[2]
  }
  
  desc <- "minimum contrast fit of Neyman-Scott process with Cauchy kernel"
  result <- mincontrast(g, theoret, startpar,
                        ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)", desc=desc),
                        explain=list(dataname=dataname,
                          fname=attr(g, "fname"),
                          modelname="Cauchy process"), ...)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("kappa", "eta2")
  result$par <- par
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="Cauchy")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  return(result)
}

## user-callable
resolve.vargamma.shape <- function(..., nu.ker=NULL, nu.pcf=NULL, default = FALSE) {
  if(is.null(nu.ker) && is.null(nu.pcf)){
    if(!default)
        stop("Must specify either nu.ker or nu.pcf", call.=FALSE)
    nu.ker <- -1/4
  }
  if(!is.null(nu.ker) && !is.null(nu.pcf))
    stop("Only one of nu.ker and nu.pcf should be specified",
         call.=FALSE)
  if(!is.null(nu.ker)) {
    check.1.real(nu.ker)
    stopifnot(nu.ker > -1/2)
    nu.pcf <- 2 * nu.ker + 1
  } else {
    check.1.real(nu.pcf)
    stopifnot(nu.pcf > 0)
    nu.ker <- (nu.pcf - 1)/2
  }
  return(list(nu.ker=nu.ker, nu.pcf=nu.pcf))
}

vargamma.estK <- function(X, startpar=c(kappa=1,scale=1), nu = -1/4,
                          lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL,
                          ...) {

## nu.ker: smoothness parameter of Variance Gamma kernel function
## omega: scale parameter of kernel function
## nu.pcf: smoothness parameter of Variance Gamma pair correlation function
## eta: scale parameter of Variance Gamma pair correlation function
## nu.pcf = 2 * nu.ker + 1    and    eta = omega

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)
  
  if(inherits(X, "fv")) {
    K <- X
    if(!identical(attr(K, "fname")[1], "K"))
      warning("Argument X does not appear to be a K-function")
  } else if(inherits(X, "ppp")) {
    K <- Kest(X)
    dataname <- paste("Kest(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
    stop("Unrecognised format for argument X")

  ## Catch old nu.ker/nu.pcf syntax and resolve nu-value.
  dots <- list(...)
  if(missing(nu)){
      nu <- resolve.vargamma.shape(nu.ker=dots$nu.ker, nu.pcf=dots$nu.pcf, default = TRUE)$nu.ker
  }
  check.1.real(nu)
  stopifnot(nu > -1/2)

  info <- spatstatClusterModelInfo("VarGamma")
  startpar <- info$checkpar(startpar)
  theoret <- info$K
  
  ## test validity of parameter nu and digest
  ph <- info$parhandler
  cmodel <- ph(nu.ker=nu)
  margs <- cmodel$margs

  desc <- "minimum contrast fit of Neyman-Scott process with Variance Gamma kernel"
  result <- mincontrast(K, theoret, startpar,
                        ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)", desc=desc),
                        explain=list(dataname=dataname,
                          fname=attr(K, "fname"),
                          modelname="Variance Gamma process"),
                        margs=margs, ...)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("kappa", "eta")
  result$par <- par
  result$covmodel <- cmodel
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="VarGamma")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  result$clustargs <- info$checkclustargs(cmodel$margs, old=FALSE)
  return(result)
}


vargamma.estpcf <- function(X, startpar=c(kappa=1,scale=1), nu=-1/4, 
                            lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL, 
                            ..., pcfargs=list()) {

## nu.ker: smoothness parameter of Variance Gamma kernel function
## omega: scale parameter of kernel function
## nu.pcf: smoothness parameter of Variance Gamma pair correlation function
## eta: scale parameter of Variance Gamma pair correlation function
## nu.pcf = 2 * nu.ker + 1    and    eta = omega

  dataname <-
    getdataname(short.deparse(substitute(X), 20), ...)

  if(inherits(X, "fv")) {
    g <- X
    if(!identical(attr(g, "fname")[1], "g"))
      warning("Argument X does not appear to be a pair correlation function")
  } else if(inherits(X, "ppp")) {
    g <- do.call(pcf.ppp, append(list(X), pcfargs))
    dataname <- paste("pcf(", dataname, ")", sep="")
    if(is.null(lambda))
      lambda <- summary(X)$intensity
  } else 
      stop("Unrecognised format for argument X")
  
  ## Catch old nu.ker/nu.pcf syntax and resolve nu-value.
  dots <- list(...)
  if(missing(nu)){
      ## nutmp <- try(resolve.vargamma.shape(nu.ker=dots$nu.ker, nu.pcf=dots$nu.pcf)$nu.ker, silent=TRUE)
      ## if(!inherits(nutmp, "try-error")) nu <- nutmp
      nu <- resolve.vargamma.shape(nu.ker=dots$nu.ker, nu.pcf=dots$nu.pcf, default = TRUE)$nu.ker
  }
  check.1.real(nu)
  stopifnot(nu > -1/2)

  info <- spatstatClusterModelInfo("VarGamma")
  startpar <- info$checkpar(startpar)
  theoret <- info$pcf

  ## test validity of parameter nu and digest 
  ph <- info$parhandler
  cmodel <- ph(nu.ker=nu)
  margs <- cmodel$margs
  
  ## avoid using g(0) as it may be infinite
  argu <- fvnames(g, ".x")
  rvals <- g[[argu]]
  if(rvals[1] == 0 && (is.null(rmin) || rmin == 0)) {
    rmin <- rvals[2]
  }
  
  desc <- "minimum contrast fit of Neyman-Scott process with Variance Gamma kernel"
  result <- mincontrast(g, theoret, startpar,
                        ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
                        fvlab=list(label="%s[fit](r)", desc=desc),
                        explain=list(dataname=dataname,
                          fname=attr(g, "fname"),
                          modelname="Variance Gamma process"),
                        margs=margs,
                        ...)
  ## imbue with meaning
  par <- result$par
  names(par) <- c("kappa", "eta")
  result$par <- par
  result$covmodel <- cmodel
  ## infer model parameters
  result$modelpar <- info$interpret(par, lambda)
  result$internal <- list(model="VarGamma")
  ## add new parametrisation to object
  result$clustpar <- info$checkpar(par, old=FALSE)
  result$clustargs <- info$checkclustargs(cmodel$margs, old=FALSE)
  return(result)
}

#
# psp2pix.R
#
#  $Revision: 1.12 $  $Date: 2017/11/15 07:21:21 $
#
#

as.mask.psp <- function(x, W=NULL, ...) {
  L <- as.psp(x)
  if(is.null(W))
    W <- as.owin(L)
  else
    W <- as.owin(W)

  W <- do.call.matched(as.mask,
                       resolve.defaults(list(...),
                                        list(w=W)))

  ends <- L$ends
  nseg <- nrow(ends)
  
  if(nseg == 0) {
    # empty
    W$m[] <- FALSE
    return(W)
  }
    
  x0 <- (ends$x0 - W$xrange[1])/W$xstep
  x1 <- (ends$x1 - W$xrange[1])/W$xstep
  y0 <- (ends$y0 - W$yrange[1])/W$ystep
  y1 <- (ends$y1 - W$yrange[1])/W$ystep
  nr <- W$dim[1]
  nc <- W$dim[2]
  zz <- .C("seg2pixI",
           ns=as.integer(nseg),
           x0=as.double(x0),
           y0=as.double(y0),
           x1=as.double(x1),
           y1=as.double(y1),
           nx=as.integer(nc),
           ny=as.integer(nr),
           out=as.integer(integer(nr * nc)),
           PACKAGE = "spatstat")
  mm <- matrix(zz$out, nr, nc)
  # intersect with existing window
  W$m <- W$m & mm
  W
}


pixellate.psp <- function(x, W=NULL, ..., weights=NULL,
                          what=c("length", "number"),
                          DivideByPixelArea=FALSE) {
  L <- as.psp(x)
  what <- match.arg(what)
  
  if(is.null(W))
    W <- as.owin(L)
  else
    W <- as.owin(W)

  W <- do.call.matched(as.mask,
                         resolve.defaults(list(...),
                                          list(w=W)))

  Z <- as.im(W)

  ends <- L$ends
  nseg <- nrow(ends)

  if(nseg == 0) {
    # empty
    Z$v[] <- 0
    return(Z)
  }

  
  if(is.null(weights))
    weights <- rep.int(1, nseg)
  else {
    if(!is.numeric(weights)) stop("weights must be numeric")
    if(anyNA(weights)) stop("weights must not be NA")
    if(!all(is.finite(weights))) stop("weights must not be infinite")
    if(length(weights) == 1)
      weights <- rep.int(weights, nseg)
    else if(length(weights) != nseg)
      stop(paste("weights vector has length", length(weights),
                 "but there are", nseg, "line segments"))
  }
      
  x0 <- (ends$x0 - Z$xrange[1])/Z$xstep
  x1 <- (ends$x1 - Z$xrange[1])/Z$xstep
  y0 <- (ends$y0 - Z$yrange[1])/Z$ystep
  y1 <- (ends$y1 - Z$yrange[1])/Z$ystep
  nr <- Z$dim[1]
  nc <- Z$dim[2]
  switch(what,
         length = {
           zz <- .C("seg2pixL",
                    ns=as.integer(nseg),
                    x0=as.double(x0),
                    y0=as.double(y0),
                    x1=as.double(x1),
                    y1=as.double(y1),
                    weights=as.double(weights),
                    pixwidth=as.double(Z$xstep),
                    pixheight=as.double(Z$ystep),
                    nx=as.integer(nc),
                    ny=as.integer(nr),
                    out=as.double(numeric(nr * nc)),
                    PACKAGE = "spatstat")
         },
         number = {
           zz <- .C("seg2pixN",
                    ns=as.integer(nseg),
                    x0=as.double(x0),
                    y0=as.double(y0),
                    x1=as.double(x1),
                    y1=as.double(y1),
                    w=as.double(weights),
                    nx=as.integer(nc),
                    ny=as.integer(nr),
                    out=as.double(numeric(nr * nc)),
                    PACKAGE = "spatstat")
         })
  mm <- matrix(zz$out, nr, nc)
  ## intersect with existing window
  mm[is.na(Z$v)] <- NA
  ##
  if(DivideByPixelArea) {
    pixelarea <- W$xstep * W$ystep
    mm <- mm/pixelarea
  }
  ## pack up
  Z$v <- mm
  return(Z)
}

#'
#'    triangulate.R
#'
#'   Decompose a polygon into triangles
#'
#'    $Revision: 1.4 $  $Date: 2015/11/21 11:13:00 $
#'

triangulate.owin <- local({

  is.triangle <- function(p) {
    return((length(p$bdry) == 1) && (length(p$bdry[[1]]$x) == 3))
  }

  triangulate.owin <- function(W) {
    stopifnot(is.owin(W))
    W <- as.polygonal(W, repair=TRUE)
    P <- as.ppp(vertices(W), W=Frame(W), check=FALSE)
    D <- delaunay(P)
    V <- intersect.tess(W, D)
    Candidates <- tiles(V)
    istri <- sapply(Candidates, is.triangle)
    Accepted <- Candidates[istri]
    if(any(!istri)) {
      # recurse
      Worries <- unname(Candidates[!istri])
      Fixed <- lapply(Worries, triangulate.owin)
      Fixed <- do.call(c, lapply(Fixed, tiles))
      Accepted <- append(Accepted, Fixed)
    }
    result <- tess(tiles=Accepted, window=W)
    return(result)
  }

  triangulate.owin
})

#'
#'    rasterfilter.R
#'
#'    raster filters implemented directly
#'
#'    $Revision: 1.5 $ $Date: 2017/11/18 07:17:18 $
#'

rasterfilter <- function(X, f) {
  X <- as.im(X)
  dimX <- dim(X)
  f <- as.matrix(f)
  if(!all(dim(f) == 3))
    stop("f should be a 3 x 3 matrix or image")
  #' handle NA
  v <- as.double(X$v)
  if(hasna <- anyNA(v)) {
    isna <- is.na(v)
    v[isna] <- 0
  }
  #' compute
  z <- .C("raster3filter",
          nx = as.integer(dimX[2]),
          ny = as.integer(dimX[1]),
          a  = as.double(v),
          w  = as.double(f),
          b  = as.double(numeric(prod(dimX))),
          PACKAGE="spatstat")
  z <- z$b
  #' handle NA
  if(hasna)
    z[isna] <- NA
  # replace
  X[] <- z
  return(X)
}

#'  antialiasing
smudge <- function(X) {
  stopifnot(is.im(X))
  xstep <- X$xstep
  ystep <- X$ystep
  #' choose a very small bandwidth
  sigma <- min(xstep, ystep)/2
  #' match variance: 2 p step^2 = sigma^2
  px <- sigma^2/(2 * xstep^2)
  py <- sigma^2/(2 * ystep^2)
  f <- outer(c(py, 1-2*py, py), c(px, 1-2*px, px), "*")
  #' compute 
  Z <- rasterfilter(X, f)
  attr(Z, "sigma") <- sigma
  return(Z)
}

#
#
#  density.psp.R
#
#  $Revision: 1.16 $    $Date: 2020/03/16 10:28:51 $
#
#

density.psp <- function(x, sigma, ..., edge=TRUE,
                        method=c("FFT", "C", "interpreted"),
                        at=NULL) {
  verifyclass(x, "psp")
  method <- match.arg(method)
  w <- x$window
  n <- x$n
  len <- lengths_psp(x)
  ang <- angles.psp(x, directed=TRUE)
  ux <- unitname(x)
  if(missing(sigma))
    sigma <- 0.1 * diameter(w)
  #' determine locations for evaluation of density
  if(is.null(at)) {
    atype <- "window"
    w <- do.call.matched(as.mask, resolve.defaults(list(w=w, ...)))
  } else if(is.owin(at)) {
    atype <- "window"
    w <- do.call.matched(as.mask, resolve.defaults(list(w=at, ...)))
  } else {  
    atype <- "points"
    atY <- try(as.ppp(at, W=w))
    if(inherits(atY, "try-error"))
      stop("Argument 'at' should be a window or a point pattern", call.=FALSE)
  }
  #' detect empty pattern
  if(n == 0 || all(len == 0)) 
    switch(atype,
           window = return(as.im(0, w)),
           points = return(rep(0, npoints(atY))))
  #' determine prediction coordinates
  switch(atype,
         window = {
           xy <- rasterxy.mask(w)
           xx <- xy$x
           yy <- xy$y
         },
         points = {
           xx <- atY$x
           yy <- atY$y
         })
  #' c o m p u t e
  switch(method,
         interpreted = {
           #' compute matrix contribution from each segment 
           coz <- cos(ang)
           zin <- sin(ang)
           for(i in seq_len(n)) {
             en <- x$ends[i,]
             dx <- xx - en$x0
             dy <- yy - en$y0
             u1 <- dx * coz[i] + dy * zin[i]
             u2 <- - dx * zin[i] + dy * coz[i]
             value <- dnorm(u2, sd=sigma) *
               (pnorm(u1, sd=sigma) - pnorm(u1-len[i], sd=sigma))
             totvalue <- if(i == 1L) value else (value + totvalue)
           }
           dens <- switch(atype,
                          window = im(totvalue, w$xcol, w$yrow, unitname=ux),
                          points = totvalue)
         },
         C = {
           #' C implementation of the above
           xs <- x$ends$x0
           ys <- x$ends$y0
           xp <- as.numeric(as.vector(xx))
           yp <- as.numeric(as.vector(yy))
           np <- length(xp)
           z <- .C("segdens",
                   sigma = as.double(sigma),
                   ns = as.integer(n),
                   xs = as.double(xs),
                   ys = as.double(ys),
                   alps = as.double(ang),
                   lens = as.double(len),
                   np = as.integer(np),
                   xp = as.double(xp), 
                   yp = as.double(yp),
                   z = as.double(numeric(np)),
                   PACKAGE = "spatstat")
           dens <- switch(atype,
                          window = im(z$z, w$xcol, w$yrow, unitname=ux),
                          points = z$z)
         },
         FFT = {
           Y <- pixellate(x, ..., DivideByPixelArea=TRUE)
           dens <- blur(Y, sigma, normalise=edge, bleed=FALSE, ...)
           if(atype == "points") dens <- dens[atY, drop=FALSE]
         })
  if(edge && method != "FFT") {
    edg <- second.moment.calc(midpoints.psp(x), sigma, what="edge", ...)
    switch(atype,
           window = {
             dens <- eval.im(dens/edg)
           },
           points = {
             edgY <- edg[atY, drop=FALSE]
             dens <- dens/edgY
           })
  }
  if(atype == "window")
    dens <- dens[x$window, drop=FALSE]
  attr(dens, "sigma") <- sigma
  return(dens)
}

#
#	plot.owin.S
#
#	The 'plot' method for observation windows (class "owin")
#
#	$Revision: 1.60 $	$Date: 2020/02/18 03:33:02 $
#
#
#

plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
                      type = c("w", "n"), show.all=!add,
                      hatch=FALSE,
                      hatchargs=list(), 
                      invert=FALSE, do.plot=TRUE,
                      claim.title.space=FALSE, use.polypath=TRUE) 
{
#
# Function plot.owin.  A method for plot.
#
  if(missing(main))
    main <- short.deparse(substitute(x))

  W <- x
  verifyclass(W, "owin")
  if(!do.plot) 
    return(invisible(as.rectangle(W)))
  
  type <- match.arg(type)

  if(missing(box) || is.null(box)) {
    box <- is.mask(W) && show.all
  } else stopifnot(is.logical(box) && length(box) == 1)

####
  pt <- prepareTitle(main)
  main <- pt$main
  nlines <- pt$nlines
#########        
  xlim <- xr <- W$xrange
  ylim <- yr <- W$yrange

####################################################

  ## graphics parameters that can be overridden by user
  gparam <- resolve.defaults(list(...), par())
  ## character expansion factors
  ##     main title size = 'cex.main' * par(cex.main) * par(cex)
  ## user's graphics expansion factor (*multiplies* par)
  cex.main.user <- resolve.1.default(list(cex.main=1), list(...))
  ## size of main title as multiple of par('cex')
  cex.main.rela <- cex.main.user * par('cex.main') 
  ## absolute size
  cex.main.absol <- cex.main.rela * par('cex')
    
  if(!add) {
    ## new plot
    if(claim.title.space && nlines > 0) {
      ## allow space for main title (only in multi-panel plots)
      guesslinespace <- 0.07 * sqrt(diff(xr)^2 + diff(yr)^2) * cex.main.absol
      added <- (nlines + 1) * guesslinespace
      ylim[2] <- ylim[2] + added
    }
    ## set up plot with equal scales
    do.call.plotfun(plot.default,
                    resolve.defaults(list(x=numeric(0), y=numeric(0),
                                          type="n"),
                                     list(...),
                                     list(xlim=xlim, ylim=ylim,
                                          ann=FALSE, axes=FALSE,
                                          asp=1.0,
                                          xaxs="i", yaxs="i",
                                          xlab="", ylab=""),
                                     .MatchNull=FALSE))
  }
  if(show.all && nlines > 0) {
    ## add title 
    if(claim.title.space) {
      mainheight <- sum(strheight(main, units="user", cex=cex.main.rela))
      gapheight <- (strheight("b\nb", units="user", cex=cex.main.rela)
                    - 2 * strheight("b", units="user", cex=cex.main.rela))
      if(nlines > 1 && !is.expression(main))
        main <- paste(main, collapse="\n")
      text(x=mean(xr), y=yr[2] + mainheight + 0.5 * gapheight, labels=main,
           cex=cex.main.rela,
           col=gparam$col.main,
           font=gparam$font.main)
    } else {
      title(main=main,
            cex=cex.main.rela,
            col=gparam$col.main,
            font=gparam$font.main)
    }
  }
  
# Draw surrounding box
  if(box)
    do.call.plotfun(segments,
                    resolve.defaults(
                                     list(x0=xr[c(1,2,2,1)],
                                          y0=yr[c(1,1,2,2)],
                                          x1=xr[c(2,2,1,1)],
                                          y1=yr[c(1,2,2,1)]),
                                     list(...)))
  
# If type = "n", do not plot the window.
    if(type == "n")
      return(invisible(as.rectangle(W)))

  
# Draw window

  switch(W$type,
         rectangle = {
           Wpoly <- as.polygonal(W)
           po <- Wpoly$bdry[[1]]
           do.call.plotfun(polygon,
                           resolve.defaults(list(x=po),
                                            list(...)),
                           extrargs="lwd")
           if(hatch)
             do.call(add.texture, append(list(W=W), hatchargs))
         },
         polygonal = {
           p <- W$bdry
           # Determine whether user wants to fill the interior
           col.poly <- resolve.defaults(list(...), list(col=NA))$col
           den.poly <- resolve.defaults(list(...), list(density=NULL))$density
           no.fill  <- is.null(den.poly) &&
                       (is.null(col.poly) || is.na(col.poly))
           # Determine whether we need to triangulate the interior.
           # If it is required to fill the interior,
           # this can be done directly using polygon() provided
           # there are no holes. Otherwise we must triangulate the interior.
           if(no.fill)
             triangulate <- FALSE
           else {
             # Determine whether there are any holes
             holes <- unlist(lapply(p, is.hole.xypolygon))
             triangulate <- any(holes)
           }

           if(!triangulate) {
             # No triangulation required;
             # simply plot the polygons
             for(i in seq_along(p))
               do.call.plotfun(polygon,
                               resolve.defaults(
                                                list(x=p[[i]]),
                                                list(...)),
                               extrargs="lwd")
           } else {
              # Try using polypath():
             if(use.polypath &&
                !(names(dev.cur()) %in% c("xfig","pictex","X11"))) {
               ppa <- owin2polypath(W)
               do.call.plotfun(polypath,
                               resolve.defaults(ppa,
                                                list(border=col.poly),
                                                list(...)))
             } else {
               # decompose window into simply-connected pieces
               broken <- try(break.holes(W))
               if(inherits(broken, "try-error")) {
                 warning("Unable to plot filled polygons")
               } else {
                 # Fill pieces with colour (and draw border in same colour)
                 pp <- broken$bdry
                 for(i in seq_len(length(pp)))
                   do.call.plotfun(polygon,
                                   resolve.defaults(list(x=pp[[i]],
                                                         border=col.poly),
                                                    list(...)))
               }
             }
             # Now draw polygon boundaries
             for(i in seq_along(p))
               do.call.plotfun(polygon,
                               resolve.defaults(
                                                list(x=p[[i]]),
                                                list(density=0, col=NA),
                                                list(...)),
                               extrargs="lwd")
           }
           if(hatch)
             do.call(add.texture, append(list(W=W), hatchargs))
         },
         mask = {
           # capture 'col' argument and ensure it's at least 2 values
           coldefault <- c(par("bg"), par("fg"))
           col <- resolve.defaults(
                                   list(...),
                                   spatstat.options("par.binary"),
                                   list(col=coldefault)
                                   )$col
           if(length(col) == 1) {
             col <- unique(c(par("bg"), col))
             if(length(col) == 1) 
               col <- c(par("fg"), col)
           }
           ## invert colours?
           if(invert)
             col <- rev(col)
           ## convert to greyscale?
           if(spatstat.options("monochrome"))
             col <- to.grey(col)
           
           do.call.matched(image.default,
                           resolve.defaults(
                           list(x=W$xcol, y=W$yrow, z=t(W$m), add=TRUE),
                           list(col=col),       
                           list(...),
                           spatstat.options("par.binary"),
                           list(zlim=c(FALSE, TRUE))))
           if(hatch)
             do.call(add.texture, append(list(W=W), hatchargs))
         },
         stop(paste("Don't know how to plot window of type", sQuote(W$type)))
         )
  return(invisible(as.rectangle(W)))
}

break.holes <- local({

  insect <- function(A, Box) {
    ## efficient version of intersect.owin which doesn't 'fix' the polygons
    a <- lapply(A$bdry, reverse.xypolygon)
    b <- lapply(as.polygonal(Box)$bdry, reverse.xypolygon)
    ab <- polyclip::polyclip(a, b, "intersection",
                             fillA="nonzero", fillB="nonzero")
    if(length(ab)==0)
      return(emptywindow(Box))
    # ensure correct polarity
    totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
    if(totarea < 0)
      ab <- lapply(ab, reverse.xypolygon)
    AB <- owin(Box$xrange, Box$yrange,
               poly=ab, check=FALSE, strict=FALSE, fix=FALSE,
               unitname=unitname(A))
    return(AB)
  }

  break.holes <- function(x, splitby=NULL, depth=0, maxdepth=100) {
    if(is.null(splitby)) {
      ## first call: validate x
      stopifnot(is.owin(x))
      splitby <- "x"
    }
    if(depth > maxdepth)
      stop("Unable to divide window into simply-connected pieces")
    p <- x$bdry
    holes <- unlist(lapply(p, is.hole.xypolygon))
    if(!any(holes)) return(x)
    nholes <- sum(holes)
    maxdepth <- max(maxdepth, 4 * nholes)
    i <- min(which(holes))
    p.i <- p[[i]]
    b <- as.rectangle(x)
    xr <- b$xrange
    yr <- b$yrange
    switch(splitby,
           x = {
             xsplit <- mean(range(p.i$x))
             left <- c(xr[1], xsplit)
             right <- c(xsplit, xr[2])
             xleft <- insect(x, owin(left, yr))
             xright <- insect(x, owin(right, yr))
             ## recurse
             xleft <- break.holes(xleft, splitby="y",
                                  depth=depth+1, maxdepth=maxdepth)
             xright <- break.holes(xright, splitby="y",
                                  depth=depth+1, maxdepth=maxdepth)
             ## recombine (without fusing polygons again!)
             result <- owin(xr, yr, poly=c(xleft$bdry, xright$bdry),
                            check=FALSE, strict=FALSE, fix=FALSE)
           },
           y = {
             ysplit <- mean(range(p.i$y))
             lower <- c(yr[1], ysplit)
             upper <- c(ysplit, yr[2])
             xlower <- insect(x, owin(xr, lower))
             xupper <- insect(x, owin(xr, upper))
             ## recurse
             xlower <- break.holes(xlower, splitby="x",
                                   depth=depth+1, maxdepth=maxdepth)
             xupper <- break.holes(xupper, splitby="x",
                                   depth=depth+1, maxdepth=maxdepth)
             ## recombine (without fusing polygons again!)
             result <- owin(xr, yr, poly=c(xlower$bdry, xupper$bdry),
                            check=FALSE, strict=FALSE, fix=FALSE)
           })
    return(result)
  }

  break.holes
})

#
# pppmatch.R
#
# $Revision: 1.25 $  $Date: 2018/10/26 08:06:28 $
#
# Code by Dominic Schuhmacher
#
#
# -----------------------------------------------------------------
# The standard functions for the new class pppmatching
#
# Objects of class pppmatching consist of two point patterns pp1 and pp2,
# and either an adjacency matrix ((i,j)-th entry 1 if i-th point of pp1 and j-th
# point of pp2 are matched, 0 otherwise) for "full point matchings" or
# a "generalized adjacency matrix" (or flow matrix; positive values are
# no longer limited to 1, (i,j)-th entry gives the "flow" between
# the i-th point of pp1 and the j-th point of pp2) for "fractional matchings".
# Optional elements are the type
# of the matching, the cutoff value for distances in R^2, the order
# of averages taken, and the resulting distance for the matching.
# Currently recognized types are "spa" (subpattern assignment,
# where dummy points at maximal dist are introduced if cardinalities differ), 
# "ace" (assignment if cardinalities equal, where dist is maximal if cards differ),
# and "mat" (mass transfer, fractional matching that belongs to the
# Wasserstein distance obtained if point patterns are normalized to probability measures).
# -----------------------------------------------------------------

pppmatching <- function(X, Y, am, type = NULL, cutoff = NULL,
   q = NULL, mdist = NULL) {
   verifyclass(X, "ppp")
   verifyclass(Y, "ppp")
   n1 <- X$n
   n2 <- Y$n
   am <- as.matrix(am)
   if (length(am) == 0) {
      if (min(n1,n2) == 0) 
         am <- matrix(am, nrow=n1, ncol=n2)
      else
         stop("Adjacency matrix does not have the right dimensions")
   }
   if (dim(am)[1] != n1 || dim(am)[2] != n2)
      stop("Adjacency matrix does not have the right dimensions")
   am <- matrix(as.numeric(am), n1, n2)
   #am <- apply(am, c(1,2), as.numeric)
   res <- list("pp1" = X, "pp2" = Y, "matrix" = am,
               "type" = type, "cutoff" = cutoff, 
               "q" = q, "distance" = mdist)
   class(res) <- "pppmatching"
   res
}

plot.pppmatching <- function(x, addmatch = NULL, main = NULL, ..., adjust=1) {
   if (is.null(main))
     main <- short.deparse(substitute(x))
   pp1 <- x$pp1
   pp2 <- x$pp2
   do.call.matched(plot.owin,
                   list(x=pp1$window, main = main, ...),
                   extrargs=graphicsPars("owin"))
   here <- which((x$matrix > 0), arr.ind = TRUE)
   if (!is.null(addmatch)) {
     stopifnot(is.matrix(addmatch))
     addhere <- which((addmatch > 0), arr.ind = TRUE)
     seg <- as.psp(from=pp1[addhere[,1]], to=pp2[addhere[,2]])
     plot(seg, add=TRUE, lty = 2, col="gray70")
   }
   if (length(here) > 0) {
     seg <- as.psp(from=pp1[here[,1]], to=pp2[here[,2]])
     marks(seg) <- x$matrix[here]
     plot(seg, add=TRUE, ..., style="width", adjust=adjust)
   }
   plot(x$pp1, add=TRUE, pch=20, col=2, ...)
   plot(x$pp2, add=TRUE, pch=20, col=4, ...)
   return(invisible(NULL))
}

print.pppmatching <- function(x, ...) {
   n1 <- x$pp1$n
   n2 <- x$pp2$n
   if (is.null(x$type) || is.null(x$q) || is.null(x$cutoff))
     splat("Generic matching of two planar point patterns")
   else
     splat(x$type, "-",
           x$q, " matching of two planar point patterns (cutoff = ",
           x$cutoff, ")", sep = "")
   splat("pp1:", n1, ngettext(n1, "point", "points"))
   splat("pp2:", n2, ngettext(n2, "point", "points"))
   print(Window(x$pp1))
   npair <- sum(x$matrix > 0)
   if (npair == 0)
     splat("matching is empty")
   else {
     if (any(x$matrix != trunc(x$matrix)))
       splat("fractional matching,", npair, ngettext(npair, "flow", "flows"))
     else
       splat("point matching,", npair, ngettext(npair, "line", "lines"))
   }
   if (!is.null(x$distance))
     splat("distance:", x$distance)
   return(invisible(NULL))
}

summary.pppmatching <- function(object, ...) {
   X <- object$pp1
   Y <- object$pp2
   n1 <- X$n
   n2 <- Y$n
   if (is.null(object$type) || is.null(object$q) || is.null(object$cutoff))
     splat("Generic matching of two planar point patterns")
   else
     splat(object$type, "-", object$q,
           " matching of two planar point patterns (cutoff = ",
           object$cutoff, ")", sep = "")
   splat("pp1:", n1, ngettext(n1, "point", "points"))
   splat("pp2:", n2, ngettext(n2, "point", "points"))
   print(Window(X))
   npair <- sum(object$matrix > 0)
   if (npair == 0)
     splat("matching is empty") 
   else {
     if (any(object$matrix != trunc(object$matrix))) {
       splat("fractional matching,", npair, ngettext(npair, "flow", "flows"))
     }
     else {
       splat("point matching,", npair, ngettext(npair, "line", "lines"))
       rowsum <- rowSums(object$matrix)
       colsum <- colSums(object$matrix)
       lt <- ifelse(min(rowsum) >= 1, TRUE, FALSE)
       ru <- ifelse(max(rowsum) <= 1, TRUE, FALSE)
       rt <- ifelse(min(colsum) >= 1, TRUE, FALSE)
       lu <- ifelse(max(colsum) <= 1, TRUE, FALSE)
       if (lt && ru && rt && lu)
         splat("matching is 1-1")
       else if (any(lt, ru, rt, lu)) {
         splat("matching is",
                   ifelse(lt, " left-total", ""),
                   ifelse(lu, " left-unique", ""),
                   ifelse(rt, " right-total", ""),
                   ifelse(ru, " right-unique", ""),
                   sep="")
         }
     }
   }
   if (!is.null(object$distance))
     splat("distance:", object$distance)
   return(invisible(NULL))
}


# -----------------------------------------------------------------
# matchingdist computes the distance associated with a certain kind of matching.
# Any of the arguments type, cutoff and order (if supplied) override the 
# the corresponding arguments in the matching.
# This function is useful for verifying the distance element of an
# object of class pppmatching as well as for comparing different
# (typically non-optimal) matchings.
# -----------------------------------------------------------------

matchingdist <- function(matching, type = NULL, cutoff = NULL, q = NULL) {
  verifyclass(matching, "pppmatching")
  if (is.null(type))
    if (is.null(matching$type))
      stop("Type of matching unknown. Distance cannot be computed")
    else
      type <- matching$type
  if (is.null(cutoff))
    if (is.null(matching$cutoff))
      stop("Cutoff value unknown. Distance cannot be computed")
    else
      cutoff <- matching$cutoff
  if (is.null(q))
    if (is.null(matching$q))
      stop("Order unknown. Distance cannot be computed")
    else
      q <- matching$q

  X <- matching$pp1
  Y <- matching$pp2
  n1 <- X$n
  n2 <- Y$n
  Lpexpect <- function(x, w, p) {
    f <- max(x)
      return(ifelse(f==0, 0, f * sum((x/f)^p * w)^(1/p)))
  }

  if (type == "spa") {
    n <- max(n1,n2) # divisor for Lpexpect
    if (n == 0)
      return(0)
    else if (min(n1,n2) == 0)
      return(cutoff)
    shortdim <- which.min(c(n1,n2))
    shortsum <- apply(matching$matrix, shortdim, sum)
    if (any(shortsum != 1))
      warning("matching does not attribute mass 1 to each point of point pattern with smaller cardinality")
#    dfix <- apply(crossdist(X,Y), c(1,2), function(x) { min(x,cutoff) })
    dfix <- pmin(crossdist(X,Y), cutoff)
    if (is.finite(q))
      resdist <- (Lpexpect(dfix, matching$matrix/n, q)^q + abs(n2-n1)/n * cutoff^q)^(1/q)
    else
      resdist <- ifelse(n1==n2, max(dfix[matching$matrix > 0]), cutoff)
  }
  else if (type == "ace") {
    n <- n1 # divisor for Lpexpect
    if (n1 != n2)
      return(cutoff)
    if (n == 0)
      return(0)
    rowsum <- rowSums(matching$matrix)
    colsum <- colSums(matching$matrix)
    if (any(c(rowsum, colsum) != 1))
      warning("matching is not 1-1")
#    dfix <- apply(crossdist(X,Y), c(1,2), function(x) { min(x,cutoff) })
    dfix <- pmin(crossdist(X,Y), cutoff)
    if (is.finite(q))
      resdist <- Lpexpect(dfix, matching$matrix/n, q)
    else
      resdist <- max(dfix[matching$matrix > 0])
  }
  else if (type == "mat") {
    n <- min(n1,n2) # divisor for Lpexpect
    if (min(n1,n2) == 0)
      return(NaN)
    shortdim <- which.min(c(n1,n2))
    shortsum <- apply(matching$matrix, shortdim, sum)
    if (any(shortsum != 1))
      warning("matching does not attribute mass 1 to each point of point pattern with smaller cardinality")
#    dfix <- apply(crossdist(X,Y), c(1,2), function(x) { min(x,cutoff) })
    dfix <- pmin(crossdist(X,Y), cutoff)
    if (is.finite(q))
      resdist <- Lpexpect(dfix, matching$matrix/n, q)
    else
      resdist <- max(dfix[matching$matrix > 0])
  }
  else 
    stop(paste("Unrecognised type", sQuote(type)))
  return(resdist)
}


# -----------------------------------------------------------------
# The main function for computation of distances and finding optimal
# matchings between point patterns: pppdist
# -----------------------------------------------------------------
#
# pppdist uses several helper functions not normally called by the user 
#
# The arguments of pppdist are 
#
# x and y of class ppp (the two point patterns for which we want to compute
#   a distance)
# The type of distance to be computed; any one of "spa" (default), "ace", "mat".
#   For details of this and the following two arguments see above (description
#   for class "pppmatching")
# cutoff and order q of the distance
# Set matching to TRUE if the full point matching (including distance)
#   should be returned; otherwise only the distance is returned
# If ccode is FALSE R code is used where available. This may be useful if q
#   is high (say above 10) and severe warning messages pop up. R can
#   (on most machines) deal with a higher number of significant digits per
#   number than C (at least with the code used below)
# precision should only be entered by advanced users. Empirically reasonable defaults
#   are used otherwise. As a rule of thumb, if ccode is TRUE, precision should
#   be the highest value that does not give an error (typically 9); if ccode
#   is FALSE, precision should be balanced (typically between 10 and 100) in
#   such a way that the sum of the  number of zeroes and pseudo-zeroes given in the
#   warning messages is minimal
# approximation: if q = Inf, by the distance of which order should 
#   the true distance be approximated. If approximation is Inf, brute force
#   computation is used, which is only practicable for point patterns with
#   very few points (see also the remarks just before the pppdist.prohorov
#   function below).  
# show.rprimal=TRUE shows at each stage of the algorithm what the current restricted
#   primal problem and its solution are (algorithm jumps between restricted primal
#   and dual problem until the solution to the restricted primal (a partial
#   matching of the point patterns) is a full matching)
# timelag gives the number of seconds of pause added each time a solution to
#   the current restricted primal is found (has only an effect if show.primal=TRUE) 
# -----------------------------------------------------------------

pppdist <- function(X, Y, type = "spa", cutoff = 1, q = 1, matching = TRUE,
                    ccode = TRUE, auction = TRUE, precision = NULL,
                    approximation = 10, show.rprimal = FALSE, timelag = 0) {
  verifyclass(X, "ppp")
  verifyclass(Y, "ppp")
  if (!ccode && type == "mat") {
    warning("R code is not available for type = ", dQuote("mat"), ". C code is
    used instead")
    ccode <- TRUE
  }
  if (!ccode && is.infinite(q) && is.infinite(approximation)) {
    warning("R code is not available for q = Inf and approximation = Inf. C code is
    used instead")
    ccode <- TRUE
  }
  if (ccode && is.infinite(q) && is.infinite(approximation) && type == "spa" && X$n != Y$n) {
    warning("approximation = Inf not available for type = ",
        dQuote("spa"), " and point patterns with differing cardinalities")
    approximation <- 10
  }
  if (is.infinite(q) && is.infinite(approximation) && type == "mat") {
    warning("approximation = Inf not available for type = ",
        dQuote("mat"))
    approximation <- 10
  }
  if (show.rprimal) {
    ccode <- FALSE
    auction <- FALSE
      if (type != "ace"){
        warning("show.rprimal = TRUE not available for type = ",
        dQuote(type), ". Type is changed to ", dQuote("ace"))
        type <- "ace"
    }
  }

  if (is.null(precision)) {
    if (ccode)
      precision <- trunc(log10(.Machine$integer.max))
    else {
      db <- .Machine$double.base
      minprec <- trunc(log10(.Machine$double.base^.Machine$double.digits))
      if (is.finite(q))
        precision <- min(max(minprec,2*q),
                         (.Machine$double.max.exp-1)*log(db)/log(10))
      else
        precision <- min(max(minprec,2*approximation),
                         (.Machine$double.max.exp-1)*log(db)/log(10))
    }
  }

  if (type == "spa") {
    if (X$n == 0 && Y$n == 0) {
      if (!matching)
        return(0)
      else {
        return(pppmatching(X, Y, matrix(0, nrow=0,ncol=0), type, cutoff, q, 0))
      }
    }
    n1 <- X$n
    n2 <- Y$n
    n <- max(n1,n2)
    dfix <- matrix(cutoff,n,n)
    if (min(n1,n2) > 0)
      dfix[1:n1,1:n2] <- crossdist(X,Y)
#    d <- dfix <- apply(dfix, c(1,2), function(x) { min(x,cutoff) })
    d <- dfix <- pmin(dfix,cutoff)
    if (is.infinite(q)) {
      if (n1 == n2 || matching)
        return(pppdist.prohorov(X, Y, n, d, type, cutoff, matching, ccode,
        auction, precision, approximation))
      else
        return(cutoff)
      # in the case n1 != n2 the distance is clear, and in a sense any
      # matching would be correct. We go here the extra mile and call
      # pppdist.prohorov in order to find (approximate) the matching
      # that is intuitively most interesting (i.e. the one that
      # pairs the points of the
      # smaller cardinality point pattern with the points of the larger
      # cardinality point pattern in such a way that the maximal pairing distance
      # is minimal (for q < Inf the q-th order pairing distance before the introduction
      # of dummy points is automatically minimal if it is minimal after the
      # introduction of dummy points)
      # which would be the case for the obtained pairing if q < Inf
    }
  }
  else if (type == "ace") {
    if (X$n != Y$n) {
      if (!matching)
        return(cutoff)
      else {
        return(pppmatching(X, Y, matrix(0, nrow=X$n, ncol=Y$n), type, cutoff, q, cutoff))
      }
    }
    if (X$n == 0) {
      if (!matching)
        return(0)
      else {
        return(pppmatching(X, Y, matrix(0, nrow=0,ncol=0), type, cutoff, q, 0))
      }
    }
    n <- n1 <- n2 <- X$n
    dfix <- crossdist(X,Y)
#    d <- dfix <- apply(dfix, c(1,2), function(x) { min(x,cutoff) })
    d <- dfix <- pmin(dfix, cutoff)
    if (is.infinite(q))
      return(pppdist.prohorov(X, Y, n, d, type, cutoff, matching, ccode,
      auction, precision, approximation))
  }
  else if (type == "mat") {
    if (!ccode)
      warning("R code is not available for type = ", dQuote("mat"), ". C code is used instead")
    if (auction)
      warning("Auction algorithm is not available for type = ", dQuote("mat"), ". Primal-dual algorithm is used instead")
    return(pppdist.mat(X, Y, cutoff, q, matching, precision, approximation))
  }
  else stop(paste("Unrecognised type", sQuote(type)))

  d <- d/max(d)
  d <- round((d^q)*(10^precision))
  nzeroes <- sum(d == 0 & dfix > 0)
  if(nzeroes > 0)
    warning(paste(nzeroes, ngettext(nzeroes, "zero", "zeroes"), "introduced, while rounding the q-th powers of distances"))
  if(ccode & any(d > .Machine$integer.max))
    stop("integer overflow, while rounding the q-th powers of distances")
  if(!ccode) {
    if (any(is.infinite(d)))
      stop("Inf obtained, while taking the q-th powers of distances")
    maxd <- max(d)
    npszeroes <- sum(maxd/d[d>0] >= .Machine$double.base^.Machine$double.digits)
    if (npszeroes > 0)
      warning(paste(npszeroes, ngettext(npszeroes, "pseudo-zero", "pseudo-zeroes"), "introduced, while taking the q-th powers of distances"))
      # a pseudo-zero is a value that is positive but contributes nothing to the
      # q-th order average because it is too small compared to the other values
  }

  Lpmean <- function(x, p) {
    f <- max(x)
    return(ifelse(f==0, 0, f * mean((x/f)^p)^(1/p)))
  }
  
  if (show.rprimal && type == "ace") {
    assig <- acedist.show(X, Y, n, d, timelag)
    am <- matrix(0, n, n)
    am[cbind(1:n, assig[1:n])] <- 1
  }
  else if (ccode) {
    if (auction) {
      dupper <- max(d)/10	
      lasteps <- 1/(n+1)
      epsfac <- 10
      epsvec <- lasteps
      ## Bertsekas: from dupper/2 to 1/(n+1) divide repeatedly by a constant
      while (lasteps < dupper) {
        lasteps <- lasteps*epsfac
        epsvec <- c(epsvec,lasteps)
      }
      epsvec <- rev(epsvec)[-1]
      neps <- length(epsvec)
      stopifnot(neps >= 1)
      d <- max(d)-d
      ## auctionbf uses a "desire matrix"
      res <- .C("auctionbf",
                as.integer(d),
                as.integer(n),
                pers_to_obj = as.integer(rep(-1,n)),
                price       = as.double(rep(0,n)),
                profit      = as.double(rep(0,n)),
                as.integer(neps),
                as.double(epsvec),
                PACKAGE = "spatstat")
      am <- matrix(0, n, n)
      am[cbind(1:n,res$pers_to_obj+1)] <- 1
    }
    else {           
      res <- .C("dwpure",
                as.integer(d),
                as.integer(rep.int(1,n)),
                as.integer(rep.int(1,n)),
                as.integer(n),
                as.integer(n),
                flowmatrix = as.integer(integer(n^2)),
                PACKAGE = "spatstat")
      am <- matrix(res$flowmatrix, n, n)
    }
  }
  else {
    assig <- acedist.noshow(X, Y, n, d)
    am <- matrix(0, n, n)
    am[cbind(1:n, assig[1:n])] <- 1
  }
  resdist <- Lpmean(dfix[am == 1], q)
  if (!matching)
    return(resdist)
  else {
    amsmall <- suppressWarnings(matrix(am[1:n1,1:n2], nrow=n1, ncol=n2))
    # previous line solves various problems associated with min(n1,n2) = 0 or = 1
    return(pppmatching(X, Y, amsmall, type, cutoff, q, resdist))
  }
}   

#
#
# ===========================================================
# ===========================================================
#                   Anything below:
#    Internal functions usually not to be called by user
# ===========================================================
# ===========================================================
#

#
#   Called if show.rprimal is true
#

acedist.show <- function(X, Y, n, d, timelag = 0) {
      plot(pppmatching(X, Y, matrix(0, n, n)))
      # initialization of dual variables
      u <- apply(d, 1, min)
      d <- d - u
      v <- apply(d, 2, min)
      d <- d - rep(v, each=n)
      # the main loop
      feasible <- FALSE
      while (!feasible) {
         rpsol <- maxflow(d)  # rpsol = restricted primal, solution
         am <- matrix(0, n, n)
         for (i in 1:n) {
            if (rpsol$assignment[i] > -1) am[i, rpsol$assignment[i]] <- TRUE
         }
         Sys.sleep(timelag)
         channelmat <- (d == 0 & !am)
         plot(pppmatching(X, Y, am), addmatch = channelmat)
         # if the solution of the restricted primal is not feasible for  
         # the original primal, update dual variables
         if (min(rpsol$assignment) == -1) {
            w1 <- which(rpsol$fi_rowlab > -1)
            w2 <- which(rpsol$fi_collab == -1)
            subtractor <- min(d[w1, w2])
            d[w1,] <- d[w1,] - subtractor
            d[,-w2] <- d[,-w2] + subtractor 
         }
         # otherwise break the loop
         else {
            feasible <- TRUE
         }   
      }
      return(rpsol$assignment)
}

#
#   R-version of hungarian algo without the pictures
#   useful if q is large
#

acedist.noshow <- function(X, Y, n, d) {
      # initialization of dual variables
      u <- apply(d, 1, min)
      d <- d - u
      v <- apply(d, 2, min)
      d <- d - rep(v, each=n)
      # the main loop
      feasible <- FALSE
      while (!feasible) {
         rpsol <- maxflow(d)  # rpsol = restricted primal, solution
# ~~~~~~~~~ deleted by AJB ~~~~~~~~~~~~~~~~~
#         am <- matrix(0, n, n)
#         for (i in 1:n) {
#            if (rpsol$assignment[i] > -1) am[i, rpsol$assignment[i]] <- TRUE
#         }
#         channelmat <- (d == 0 & !am)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~         
         # if the solution of the restricted primal is not feasible for  
         # the original primal, update dual variables
         if (min(rpsol$assignment) == -1) {
            w1 <- which(rpsol$fi_rowlab > -1)
            w2 <- which(rpsol$fi_collab == -1)
            subtractor <- min(d[w1, w2])
            d[w1,] <- d[w1,] - subtractor
            d[,-w2] <- d[,-w2] + subtractor 
         }
         # otherwise break the loop
         else {
            feasible <- TRUE
         }   
      }
      return(rpsol$assignment)
}

#  
# Solution of restricted primal
# 

maxflow <- function(costm) {
  stopifnot(is.matrix(costm))
  stopifnot(nrow(costm) == ncol(costm))
  if(!all(apply(costm == 0, 1, any)))
    stop("Each row of the cost matrix must contain a zero")
  
  m <- dim(costm)[1]   # cost matrix is square m * m
  assignment <- rep.int(-1, m)   # -1 means no pp2-point assigned to i-th pp1-point
  ## initial assignment or rowlabel <- source label (= 0) where not possible
  for (i in 1:m) {
    j <- match(0, costm[i,])
    if (!(j %in% assignment))
      assignment[i] <- j
  }
  newlabelfound <- TRUE
  while (newlabelfound) {
    rowlab <- rep.int(-1, m)   # -1 means no label given, 0 stands for source label
    collab <- rep.int(-1, m)
    rowlab <- ifelse(assignment == -1, 0, rowlab)
    ## column and row labeling procedure until either breakthrough occurs
    ## (which means that there is a better point assignment, i.e. one that
    ## creates more point pairs than the current one (flow can be increased))
    ## or no more labeling is possible
    breakthrough <- -1
    while (newlabelfound && breakthrough == -1) { 
      newlabelfound <- FALSE
      for (i in 1:m) {
        if (rowlab[i] != -1) {
          for (j in 1:m) {
            if (costm[i,j] == 0 && collab[j] == -1) {
              collab[j] <- i
              newlabelfound <- TRUE
              if (!(j %in% assignment) && breakthrough == -1)
                breakthrough <- j
            }
          }
        }
      }
      for (j in 1:m) {
        if (collab[j] != -1) {
          for (i in 1:m) {
            if (assignment[i] == j && rowlab[i] == -1) {
              rowlab[i] <- j
              newlabelfound <- TRUE
            }
          }
        }
      }
    }
    ## if the while-loop was left due to breakthrough,
    ## reassign points (i.e. redirect flow) and restart labeling procedure
    if (breakthrough != -1) {
      l <- breakthrough
      while (l != 0) {
        k <- collab[l]
        assignment[k] <- l
        l <- rowlab[k] 
      }
    }
  }
  ## the outermost while-loop is left, no more labels can be given; hence
  ## the maximal number of points are paired given the current restriction
  ## (flow is maximal given the current graph)
  return(list("assignment"=assignment, "fi_rowlab"=rowlab, "fi_collab"=collab))  
}

# 
# Prohorov distance computation/approximation (called if q = Inf in pppdist
#   and type = "spa" or "ace")
# Exact brute force computation of distance if approximation = Inf,
#   scales very badly, should not be used for cardinality n larger than 10-12
# Approximation by order q distance gives often (if the warning messages 
#   are not too extreme) the right matching and therefore the exact Prohorov distance,
#   but in very rare cases the result can be very wrong. However, it is always
#   an exact upper bound of the Prohorov distance (since based on *a* pairing
#   as opposed to optimal pairing.
#

pppdist.prohorov <- function(X, Y, n, dfix, type, cutoff = 1, matching = TRUE,
                             ccode = TRUE, auction = TRUE,
                             precision = 9, approximation = 10) {
  n1 <- X$n
  n2 <- Y$n
  d <- dfix/max(dfix)
  if (is.finite(approximation)) {
    warning(paste("distance with parameter q = Inf is approximated by distance with parameter q =", approximation))
    d <- round((d^approximation)*(10^precision)) 
    nzeroes <- sum(d == 0 & dfix > 0)
    if (nzeroes > 0)
      warning(paste(nzeroes,
                    ngettext(nzeroes, "zero", "zeroes"),
                    "introduced, while rounding distances"))
    if (ccode) {
      if (any(d > .Machine$integer.max))
        stop("integer overflow, while rounding the q-th powers of distances")
      if (auction) {
      	dupper <- max(d)/10
        lasteps <- 1/(n+1)
        epsfac <- 10
        epsvec <- lasteps
        ## Bertsekas: from dupper/2 to 1/(n+1) divide repeatedly by a constant
        while (lasteps < dupper) {
          lasteps <- lasteps*epsfac
          epsvec <- c(epsvec,lasteps)
        }
        epsvec <- rev(epsvec)[-1]
        neps <- length(epsvec)
        stopifnot(neps >= 1)
        d <- max(d)-d
        ## auctionbf uses a "desire matrix"
        res <- .C("auctionbf",
                  as.integer(d),
                  as.integer(n),
                  pers_to_obj = as.integer(rep(-1,n)),
                  price       = as.double(rep(0,n)),
                  profit      = as.double(rep(0,n)),
                  as.integer(neps),
                  as.double(epsvec),
                  PACKAGE = "spatstat")
        am <- matrix(0, n, n)
        am[cbind(1:n,res$pers_to_obj+1)] <- 1
      }
      else {           
        res <- .C("dwpure",
                  as.integer(d),
                  as.integer(rep.int(1,n)),
                  as.integer(rep.int(1,n)),
                  as.integer(n),
                  as.integer(n),
                  flowmatrix = as.integer(integer(n^2)),
                  PACKAGE = "spatstat")
        am <- matrix(res$flowmatrix, n, n)
      }
    }
    else {
      if (any(is.infinite(d)))
        stop("Inf obtained, while taking the q-th powers of distances")
      maxd <- max(d)
      npszeroes <- sum(maxd/d[d>0] >= .Machine$double.base^.Machine$double.digits)
      if (npszeroes > 0)
        warning(paste(npszeroes,
                      ngettext(npszeroes, "pseudo-zero", "pseudo-zeroes"),
                      "introduced, while taking the q-th powers of distances"))
      assig <- acedist.noshow(X, Y, n, d)
      am <- matrix(0, n, n)
      am[cbind(1:n, assig[1:n])] <- 1
    }
  }
  else {
    d <- round(d*(10^precision))
    nzeroes <- sum(d == 0 & dfix > 0)
    if (nzeroes > 0)
      warning(paste(nzeroes, ngettext(nzeroes, "zero", "zeroes"),
                    "introduced, while rounding distances"))
    if (any(d > .Machine$integer.max))
      stop("integer overflow, while rounding the q-th powers of distances")
    res <- .C("dinfty_R",
              as.integer(d),
              as.integer(n),
              assignment = as.integer(rep.int(-1,n)),
              PACKAGE = "spatstat")
    assig <- res$assignment
    am <- matrix(0, n, n)
    am[cbind(1:n, assig[1:n])] <- 1
  }
  if (n1 == n2)
    resdist <- max(dfix[am == 1])
  else
    resdist <- cutoff
  if (!matching)
    return(resdist)
  else {
    amsmall <- suppressWarnings(matrix(am[1:n1,1:n2], nrow=n1, ncol=n2))
    ## previous line solves various problems associated with min(n1,n2) = 0 or = 1
    return(pppmatching(X, Y, amsmall, type, cutoff, Inf, resdist))
  }
}   

# 
# Computation of "pure Wasserstein distance" for any q (called if type="mat"
#   in pppdist, no matter if q finite or not).
# If q = Inf, approximation using ccode is enforced
# (approximation == Inf is not allowed here).
#

pppdist.mat <- function(X, Y, cutoff = 1, q = 1, matching = TRUE, precision = 9,
  approximation = 10) {
  n1 <- X$n
  n2 <- Y$n
  n <- min(n1,n2)
  if (n == 0) {
    if (!matching)
      return(NaN)
    else
      return(pppmatching(X, Y, matrix(0, nrow=0,ncol=0), "mat", cutoff, q, NaN))
  }

  dfix <- crossdist(X,Y)
#  d <- dfix <- apply(dfix, c(1,2), function(x) { min(x,cutoff) })
  d <- dfix <- pmin(dfix, cutoff)
  d <- d/max(d)
  if (is.infinite(q)) {
    if (is.infinite(approximation))
      stop("approximation = Inf")
    warning(paste("distance with parameter q = Inf is approximated by distance with parameter q =", approximation))
    d <- round((d^approximation)*(10^precision)) 
    nzeroes <- sum(d == 0 & dfix > 0)
    if (nzeroes > 0)
      warning(paste(nzeroes, "zeroes introduced, while rounding distances"))
    if (any(d > .Machine$integer.max))
      stop("integer overflow, while rounding the q-th powers of distances")
    gcd <- greatest.common.divisor(n1,n2)
    mass1 <- n2/gcd
    mass2 <- n1/gcd

    res <- .C("dwpure",
             as.integer(d),
             as.integer(rep.int(mass1,n1)),
             as.integer(rep.int(mass2,n2)),
             as.integer(n1),
             as.integer(n2),
             flowmatrix = as.integer(integer(n1*n2)),
             PACKAGE = "spatstat")
    am <- matrix(res$flowmatrix/(max(n1,n2)/gcd), n1, n2)
    resdist <- max(dfix[am > 0])
  }
  else {
    d <- round((d^q)*(10^precision))
    nzeroes <- sum(d == 0 & dfix > 0)
    if(nzeroes > 0)
      warning(paste(nzeroes, ngettext(nzeroes, "zero", "zeroes"), "introduced, while rounding the q-th powers of distances"))
    if(any(d > .Machine$integer.max))
      stop("integer overflow, while rounding the q-th powers of distances")
    gcd <- greatest.common.divisor(n1,n2)
    mass1 <- n2/gcd
    mass2 <- n1/gcd

    Lpexpect <- function(x, w, p) {
      f <- max(x)
      return(ifelse(f==0, 0, f * sum((x/f)^p * w)^(1/p)))
    }

    res <- .C("dwpure",
             as.integer(d),
             as.integer(rep.int(mass1,n1)),
             as.integer(rep.int(mass2,n2)),
             as.integer(n1),
             as.integer(n2),
             flowmatrix = as.integer(integer(n1*n2)),
             PACKAGE = "spatstat")
    am <- matrix(res$flowmatrix/(max(n1,n2)/gcd), n1, n2)
    # our "adjacency matrix" in this case is standardized to have
    # rowsum 1 if n1 <= n2 and colsum 1 if n1 >= n2
    resdist <- Lpexpect(dfix, am/n, q)
  }
  if (!matching)
    return(resdist)
  else {
   amsmall <- suppressWarnings(matrix(am[1:n1,1:n2], nrow=n1, ncol=n2))
   # previous line solves various problems associated with min(n1,n2) = 0 or = 1
   return(pppmatching(X, Y, amsmall, "mat", cutoff, q, resdist))
  }
}

#
#   unique.ppp.R
#
# $Revision: 1.37 $  $Date: 2019/05/22 09:04:57 $
#
# Methods for 'multiplicity' co-authored by Sebastian Meyer
# Copyright 2013 Adrian Baddeley and Sebastian Meyer 

unique.ppp <- function(x, ..., warn=FALSE) {
  verifyclass(x, "ppp")
  dupe <- duplicated.ppp(x, ...) 
  if(!any(dupe)) return(x)
  if(warn) warning(paste(sum(dupe), "duplicated points were removed"),
                   call.=FALSE)
  return(x[!dupe])
}

duplicated.ppp <- function(x, ...,
                           rule=c("spatstat", "deldir", "unmark")) {
  verifyclass(x, "ppp")
  rule <- match.arg(rule)
  if(rule == "deldir")
    return(deldir::duplicatedxy(x))
  n <- npoints(x)
  xloc <- unmark(x)
  if(!anyDuplicated(xloc))
    return(logical(n)) # i.e. vector of FALSE
  if(rule == "unmark")
    x <- xloc
  switch(markformat(x),
         none = {
           #' unmarked points
           u <- uniquemap(x)
           result <- (u != seq_along(u))
         },
         vector = {
           #' marked points - convert mark to integer
           m <- marks(x)
           if(is.factor(m)) {
             marks(x) <- as.integer(m)
           } else {
             um <- unique(m)
             marks(x) <- match(m, um)
           }
           result <- duplicated(as.data.frame(x))
         },
         dataframe = {
           result <- duplicated(as.data.frame(x))
         },
         # the following are currently not supported
         hyperframe = {
           result <- duplicated(as.data.frame(x))
         }, 
         list = {
           result <- duplicated(as.data.frame(as.hyperframe(x)))
         },
         stop(paste("Unknown mark type", sQuote(markformat(x))))
         )
  return(result)
}

anyDuplicated.ppp <- function(x, ...) {
  #' first check duplication of coordinates using fast code
  n <- npoints(x)
  if(n <= 1) return(FALSE)
  xx <- x$x
  yy <- x$y
  o <- order(xx, seq_len(n))
  anydupXY <- .C("anydupxy",
                 n=as.integer(n),
                 x=as.double(xx[o]),
                 y=as.double(yy[o]),
                 anydup=as.integer(integer(1)),
                 PACKAGE="spatstat")$anydup
  anydupXY && (!is.marked(x) || anyDuplicated(as.data.frame(x), ...))
}

## utility to check whether two rows are identical

IdenticalRowPair <- function(i,j, a, b=a) {
  #' i and j are row indices (single integers)
  ai <- a[i,]
  bj <- b[j,]
  row.names(ai) <- row.names(bj) <- NULL
  identical(ai, bj)
}

## vectorised

IdenticalRows <- function(i, j, a, b=a) {
  #' i and j are row index vectors of equal length
  #' result[k] = identical( a[i[k],]  , b[j[k],] )
  Mo <- if(missing(b)) list(a=a) else list(a=a, b=b)
  mapply(IdenticalRowPair, i=i, j=j, MoreArgs=Mo,
         SIMPLIFY=TRUE, USE.NAMES=FALSE)
}

## .......... multiplicity .............

multiplicity <- function(x) {
  UseMethod("multiplicity")
}
  
multiplicity.ppp <- function(x) {
  verifyclass(x, "ppp")
  np <- npoints(x)
  if(np == 0) return(integer(0))
  cl <- closepairs(x, 0, what="indices")
  I <- cl$i
  J <- cl$j
  if(length(I) == 0)
    return(rep.int(1L, np))
  switch(markformat(x),
         none = { },
         vector = {
           marx <- as.data.frame(marks(x))
           agree <- IdenticalRows(I, J, marx)
           I <- I[agree]
           J <- J[agree]
         },
         dataframe = {
           marx <- marks(x)
           agree <- IdenticalRows(I, J, marx)
           I <- I[agree]
           J <- J[agree]
         },
         hyperframe = {
           marx <- as.data.frame(marks(x)) # possibly discards columns
           agree <- IdenticalRows(I, J, marx)
           I <- I[agree]
           J <- J[agree]
         }, 
         list = stop("Not implemented for lists of marks")
         )
  if(length(I) == 0)
    return(rep.int(1L, np))
  JbyI <- split(J, factor(I, levels=1:np))
  result <- 1 + lengths(JbyI)
  return(result)
}
  
multiplicity.data.frame <- function (x) {
  if(all(unlist(lapply(x, is.numeric))))
    return(multiplicityNumeric(as.matrix(x)))
  ## result template (vector of 1's)
  result <- setNames(rep.int(1L, nrow(x)), rownames(x))
  ## check for duplicates (works for data frames, arrays and vectors)
  ## CAVE: comparisons are based on a character representation of x
  if (!any(dup <- duplicated(x)))
    return(result)
  ux <- x[!dup, , drop=FALSE]
  dx <- x[dup,  , drop=FALSE]
  nu <- nrow(ux)
  nd <- nrow(dx)
  hit <- outer(seq_len(nu), seq_len(nd), IdenticalRows, a=ux, b=dx)
  counts <- as.integer(1L + .rowSums(hit, nu, nd))
  result[!dup] <- counts
  dumap <- apply(hit, 2, match, x=TRUE) # equivalent to min(which(z))
  result[dup] <- counts[dumap]
  return(result)
}

### multiplicity method for NUMERIC arrays, data frames, and vectors
### This implementation is simply based on checking for dist(x)==0

multiplicityNumeric <- function(x)
{
  if (anyDuplicated(x)) {
    distmat <- as.matrix(dist(x, method="manhattan"))  # faster than euclid.
    result <- as.integer(rowSums(distmat == 0))        # labels are kept
    if(is.null(names(result)))
      names(result) <- seq_along(result)
  } else {                                             # -> vector of 1's
    nx <- NROW(x)
    labels <- if (length(dim(x))) rownames(x) else names(x)
    if (is.null(labels)) labels <- seq_len(nx)
    result <- setNames(rep.int(1L, nx), labels)
  }
  return(result)
}

### multiplicity method for arrays, data frames, and vectors (including lists)
### It also works for non-numeric data, since it is based on duplicated().

multiplicity.default <- function (x) {
  if(is.numeric(x))
    return(multiplicityNumeric(x))
  nx <- NROW(x)                   # also works for a vector x
  ## result template (vector of 1's)
  labels <- if (length(dim(x))) rownames(x) else names(x)
  if (is.null(labels)) labels <- seq_len(nx)
  result <- setNames(rep.int(1L, nx), labels)
  ## check for duplicates (works for data frames, arrays and vectors)
  ## CAVE: comparisons are based on a character representation of x
  if (!any(dup <- duplicated(x)))
    return(result)

  ## convert x to a matrix for IdenticalRows()
  x <- as.matrix(x)
  dimnames(x) <- NULL             # discard any names!
  ux <- x[!dup, , drop=FALSE]
  dx <- x[dup,  , drop=FALSE]
  nu <- nrow(ux)
  nd <- nrow(dx)
  hit <- outer(seq_len(nu), seq_len(nd), IdenticalRows, a=ux, b=dx)
  counts <- as.integer(1L + .rowSums(hit, nu, nd))
  dumap <- apply(hit, 2, match, x=TRUE) # was: function(z) min(which(z)))
  result[!dup] <- counts
  result[dup]  <- counts[dumap]
  return(result)
}

#
# versions.R
#
# version numbers
#
# $Revision: 1.13 $  $Date: 2019/11/29 03:38:28 $
#
#####################


# Extract version string from ppm object

versionstring.ppm <- function(object) {
  verifyclass(object, "ppm")
  v <- object$version
  if(is.null(v) || !is.list(v))
    v <- list(major=1, minor=3, release=4)
  vs <- paste(v$major, ".", v$minor, "-", v$release, sep="")
  return(vs)
}

# Extract version string from interact object

versionstring.interact <- function(object) {
  verifyclass(object, "interact")
  v <- object$version
  return(v)  # NULL before 1.11-0
}

# Get version number of current spatstat installation
# This is now saved in the spatstat cache environment
# rather than read from file every time

versionstring.spatstat <- function() {
  if(!existsSpatstatVariable("SpatstatVersion"))
    store.versionstring.spatstat()    
  getSpatstatVariable("SpatstatVersion")
}

store.versionstring.spatstat <- function() {
  vs <- read.dcf(file=system.file("DESCRIPTION", package="spatstat"),
                 fields="Version")
  vs <- as.character(vs)
  putSpatstatVariable("SpatstatVersion", vs)
}


# Extract major and minor versions only.

majorminorversion <- function(v) {
  vp <- package_version(v)
  return(package_version(paste(vp$major, vp$minor, sep=".")))
}

versioncurrency.spatstat <- function(today=Sys.Date(), checkR=TRUE) {
  ## check version currency using dates
  msg <- NULL
  if(checkR && exists("getRversion") && getRversion() >= "3.2.2") {
    ## check version of R
    rv <- R.Version()
    rdate <- with(rv, ISOdate(year, month, day))
    if(today - as.Date(rdate) > 365) {
      ## R version is really old; just warn about this
      msg <- paste(rv$version.string,
                   "is more than a year old;",
                   "we strongly recommend upgrading to the latest version")
    }
  }
  if(is.null(msg)) {
    ## check version of spatstat
    descfile <- system.file("DESCRIPTION", package="spatstat")
    packdate <- as.Date(read.dcf(file=descfile, fields="Date"))
    elapsed <- today - packdate
    if(elapsed > 75) {
      if(elapsed > 365) {
        n <- floor(elapsed/365)
        unit <- "year"
        sowhat <- "we strongly recommend upgrading to the latest version."
      } else if(elapsed > 100) {
        n <- floor(elapsed/30)
        unit <- "month"
        sowhat <- "we recommend upgrading to the latest version."
      } else {
        n <- floor(elapsed/7)
        unit <- "week"
        sowhat <- "a newer version should be available."
      }
      expired <- if(n == 1) paste("a", unit) else paste(n, paste0(unit, "s"))
      ver <- versionstring.spatstat()
      msg <- paste("spatstat version", ver,
                   "is out of date by more than",
                   paste0(expired, ";"), 
                   sowhat)
    }
  }
  return(msg)
}

# legacy function

RandomFieldsSafe <- function() { TRUE }

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#'
#'   tests/Kfuns.R
#'
#'   Various K and L functions and pcf
#'
#'   $Revision: 1.39 $  $Date: 2020/04/28 08:18:41 $
#'
#'   Assumes 'EveryStart.R' was run

myfun <- function(x,y){(x+1) * y } # must be outside

local({
  if(FULLTEST) {
    #' supporting code
    rmax.rule("Kscaled", owin(), 42)
    implemented.for.K(c("border", "bord.modif", "translate", "good", "best"),
                      "polygonal", TRUE)
    implemented.for.K(c("border", "bord.modif", "translate", "good", "best"),
                      "mask", TRUE)
    implemented.for.K(c("border", "isotropic"), "mask", TRUE)
    implemented.for.K(c("border", "isotropic"), "mask", FALSE)
    #' shortcuts
    D <- density(cells)
    K <- Kborder.engine(cells, rmax=0.4, weights=D, ratio=TRUE)
    K <- Knone.engine(cells, rmax=0.4, weights=D, ratio=TRUE)
    allcor <- c("none", "border", "bord.modif","isotropic", "translate")
    K <- Krect.engine(cells, rmax=0.4, ratio=TRUE, correction=allcor)
    K <- Krect.engine(cells, rmax=0.4, ratio=TRUE, correction=allcor,
                      weights=D)
    K <- Krect.engine(cells, rmax=0.4, ratio=TRUE, correction=allcor,
                      use.integers=FALSE)
    #' Kest special code blocks
    K <- Kest(cells, var.approx=TRUE, ratio=FALSE)
    Z <- distmap(cells) + 1
    Kb <- Kest(cells, correction=c("border","bord.modif"),
               weights=Z, ratio=TRUE)
    Kn <- Kest(cells, correction="none",
               weights=Z, ratio=TRUE)
    Knb <- Kest(cells, correction=c("border","bord.modif","none"),
                weights=Z, ratio=TRUE)
  }
  if(ALWAYS) {
    bigint <- 50000 # This is only "big" on a 32-bit system where
                    # sqrt(.Machine$integer.max) = 46340.9
    X <- runifpoint(bigint)
    Z <- as.im(1/bigint, owin())
    Kb <- Kest(X, correction=c("border","bord.modif"),
               rmax=0.02, weights=Z, ratio=TRUE)
  }
  if(FULLTEST) {
    Kn <- Kest(X, correction="none",
               rmax=0.02, weights=Z, ratio=TRUE)
    Knb <- Kest(X, correction=c("border","bord.modif","none"),
                rmax=0.02, weights=Z, ratio=TRUE)
    #' pcf.ppp special code blocks
    pr  <- pcf(cells, ratio=TRUE, var.approx=TRUE)
    pc  <- pcf(cells, domain=square(0.5))
    pcr <- pcf(cells, domain=square(0.5), ratio=TRUE)
    pw <- pcf(redwood, correction="none")
    pwr <- pcf(redwood, correction="none", ratio=TRUE)
    pv <- pcf(redwood, kernel="rectangular")
    p1 <- pcf(redwood[1])
    #' pcf.fv
    K <- Kest(redwood)
    g <- pcf(K, method="a")
    g <- pcf(K, method="c")
    g <- pcf(K, method="d")
    #' Kinhom code blocks
    X <- rpoispp(function(x,y) { 100 * x }, 100, square(1))
    lambda <- 100 * X$x
    Kin <- Kinhom(X, lambda, correction=c("none", "border"))
    lambda2 <- outer(lambda, lambda, "*")
    Ki2 <- Kinhom(X, lambda2=lambda2, diagonal=FALSE,
                  correction=c("translate", "isotropic"))
    fut <- ppm(X ~ x)
    Kio <- Kinhom(X, fut, update=FALSE)
    Kiu <- Kinhom(X, fut, update=TRUE, diagonal=FALSE)
  }
  if(ALWAYS) {
    #' edge corrections
    rr <- rep(0.1, npoints(cells))
    eC <- edge.Ripley(cells, rr)
    eI <- edge.Ripley(cells, rr, method="interpreted")
    if(max(abs(eC-eI)) > 0.1)
      stop("Ripley edge correction results do not match")
  }
  if(FULLTEST) {
    a <- rmax.Ripley(square(1))
    a <- rmax.Rigid(square(1))
    a <- rmax.Ripley(as.polygonal(square(1)))
    a <- rmax.Rigid(as.polygonal(square(1)))
    a <- rmax.Ripley(letterR)
    a <- rmax.Rigid(letterR)
  }
  if(ALWAYS) {
    #' run slow code for edge correction and compare results
    X <- redwood[c(TRUE, FALSE, FALSE)]
    Window(X) <- as.polygonal(Window(X))
    Eapprox <- edge.Trans(X)
    Eexact <- edge.Trans(X, exact=TRUE)
    maxrelerr <- max(abs(1 - range(Eapprox/Eexact)))
    if(maxrelerr > 0.1)
      stop(paste("Exact and approximate algorithms for edge.Trans disagree by",
                 paste0(round(100*maxrelerr), "%")),
           call.=FALSE)
  }
})

local({
  if(FULLTEST) {
    #' ----  multitype ------
    K <- Kcross(amacrine, correction=c("none", "bord.modif"))
    #' inhomogeneous multitype
    fit <- ppm(amacrine ~ marks)
    K1 <- Kcross.inhom(amacrine, lambdaX=fit)
    K2 <- Kcross.inhom(amacrine, lambdaX=densityfun(amacrine))
    K3 <- Kcross.inhom(amacrine, lambdaX=density(amacrine, at="points"))
    On <- split(amacrine)$on
    Off <- split(amacrine)$off
    K4 <- Kcross.inhom(amacrine, lambdaI=ppm(On), lambdaJ=ppm(Off))
    K5 <- Kcross.inhom(amacrine, correction="bord.modif")
    #' markconnect, markcorr
    M <- markconnect(amacrine, "on", "off", normalise=TRUE)
    M <- markcorr(longleaf, normalise=TRUE,
                  correction=c("isotropic", "translate", "border", "none"))
    M <- markcorr(longleaf, normalise=TRUE, fargs=list())
    #' Kmark (=markcorrint)
    X <- runifpoint(100) %mark% runif(100)
    km <- Kmark(X, f=atan2)
    km <- Kmark(X, f1=sin)
    km <- Kmark(X, f="myfun")
    aa <- Kmark(X, normalise=FALSE, returnL=FALSE)
    aa <- Kmark(X, normalise=FALSE, returnL=TRUE)
    aa <- Kmark(X, normalise=TRUE,  returnL=FALSE)
    aa <- Kmark(X, normalise=TRUE,  returnL=TRUE)
  }
})

local({
  if(FULLTEST) {
    #'    various modified K functions
    #'
    #'   directional K functions
    #'
    a <- Ksector(swedishpines,
                 -pi/2, pi/2, units="radians",
                 correction=c("none", "border", "bord.modif",
                              "Ripley", "translate"),
                 ratio=TRUE)
    plot(a)
    #'
    #'   local K functions
    #'
    fut <- ppm(swedishpines ~ polynom(x,y,2))
    Z <- predict(fut)
    Lam <- fitted(fut, dataonly=TRUE)
    a <- localLinhom(swedishpines, lambda=fut)
    a <- localLinhom(swedishpines, lambda=Z)
    a <- localLinhom(swedishpines, lambda=Lam)
    a <- localLinhom(swedishpines, lambda=Z, correction="none")
    a <- localLinhom(swedishpines, lambda=Z, correction="translate")
    a <- localLcross(amacrine)
    a <- localLcross(amacrine, from="off", to="off")
    a <- localKdot(amacrine)
    a <- localLdot(amacrine)
    a <- localKcross.inhom(amacrine)
    a <- localLcross.inhom(amacrine)
    fat <- ppm(amacrine ~ x * marks)
    Zed <- predict(fat)
    Lum <- fitted(fat, dataonly=TRUE)
    moff <- (marks(amacrine) == "off")
    a <- localLcross.inhom(amacrine, from="off", to="on", lambdaX=Zed)
    a <- localLcross.inhom(amacrine, from="off", to="on", lambdaX=Lum)
    a <- localLcross.inhom(amacrine, from="off", to="on", lambdaX=fat)
    a <- localLcross.inhom(amacrine, from="off", to="on",
                           lambdaFrom=Lum[moff], lambdaTo=Lum[!moff])
    a <- localLcross.inhom(amacrine, from="off", to="on", lambdaX=Zed,
                           correction="none")
    a <- localLcross.inhom(amacrine, from="off", to="on", lambdaX=Zed,
                           correction="translate")
    #'
    #' cases of resolve.lambda.cross
    #'
    h <- resolve.lambda.cross(amacrine, moff, !moff)
    h <- resolve.lambda.cross(amacrine, moff, !moff, lambdaX=Zed)
    h <- resolve.lambda.cross(amacrine, moff, !moff, lambdaX=Lum)
    h <- resolve.lambda.cross(amacrine, moff, !moff, lambdaX=fat)
    h <- resolve.lambda.cross(amacrine, moff, !moff, lambdaX=fat, update=FALSE)
    h <- resolve.lambda.cross(amacrine, moff, !moff,
                              lambdaI=Zed[["off"]], lambdaJ=Zed[["on"]])
    h <- resolve.lambda.cross(amacrine, moff, !moff,
                              lambdaI=Lum[moff], lambdaJ=Lum[!moff])
    h <- resolve.lambda.cross(amacrine, moff, !moff,
                              lambdaI=fat, lambdaJ=fat)
    h <- resolve.lambda.cross(amacrine, moff, !moff,
                              lambdaI=fat, lambdaJ=fat,
                              update=FALSE)
    d <- densityfun(unmark(amacrine), sigma=0.1)
    dm <- lapply(split(amacrine), densityfun, sigma=0.1)
    h <- resolve.lambda.cross(amacrine, moff, !moff, lambdaX=d)
    h <- resolve.lambda.cross(amacrine, moff, !moff,
                              lambdaI=dm[["off"]], lambdaJ=dm[["on"]])
    h <- resolve.lambda.cross(amacrine, moff, !moff,
                              lambdaX=function(x,y,m){ d(x,y) })
    #'
    #' multitype inhomogeneous pcf
    #'
    g <- pcfcross.inhom(amacrine, 
                        lambdaI=dm[["off"]], lambdaJ=dm[["on"]])

    #'
    #'   lohboot code blocks
    #'
    Ared <- lohboot(redwood, fun="Kest", block=TRUE,
                    Vcorrection=TRUE, global=FALSE, correction="none")
    Bred <- lohboot(redwood, block=TRUE, basicboot=TRUE, global=FALSE)
    Cred <- lohboot(redwood, fun=Kest, block=TRUE, global=TRUE,
                    correction="translate")
    Dred <- lohboot(redwood, Lest)
    Kred <- lohboot(redwood, Kinhom)
    Lred <- lohboot(redwood, Linhom)
    gred <- lohboot(redwood, pcfinhom, sigma=0.1)
    Zred <- predict(ppm(redwood ~ x+y))
    Lred <- lohboot(redwood, Linhom, lambda=Zred)
    #'
    X <- runifpoint(100, letterR)
    AX <- lohboot(X, block=TRUE, nx=7, ny=10)
    #'    multitype
    b <- lohboot(amacrine, Kcross)
    b <- lohboot(amacrine, Lcross)
    b <- lohboot(amacrine, Kdot)
    b <- lohboot(amacrine, Ldot)
    b <- lohboot(amacrine, Kcross.inhom)
    b <- lohboot(amacrine, Lcross.inhom)
    b <- lohboot(amacrine, Lcross.inhom, from="off", to="on", lambdaX=Zed)
    b <- lohboot(amacrine, Lcross.inhom, from="off", to="on", lambdaX=Lum)
    b <- lohboot(amacrine, Lcross.inhom, from="off", to="on", lambdaX=fat)
    b <- lohboot(amacrine, Lcross.inhom, from="off", to="on", 
                 lambdaFrom=Lum[moff], lambdaTo=Lum[!moff])
    #'
    #'  residual K functions etc
    #'
    rco <- compareFit(cells, Kcom,
                      interaction=anylist(P=Poisson(), S=Strauss(0.08)),
                      same="trans", different="tcom")
    fit <- ppm(cells ~ x, Strauss(0.07))
    K <- Kcom(cells, model=fit, restrict=TRUE)
    
    ##  Kscaled
    A <- Lscaled(japanesepines, renormalise=TRUE, correction="all")
  }
})
  
local({
  if(ALWAYS) {
    #' From Ege, in response to a stackoverflow question.
    #' The following example has two points separated by r = 1 with 1/4 of the
    #' circumference outside the 10x10 window (i.e. area 100).
    #' Thus the value of K^(r) should jump from 0 to 
    #' 100/(2\cdot 1)\cdot ((3/4)^{-1} + (3/4)^{-1}) = 100 \cdot 4/3 = 133.333.
    x <- c(4.5,5.5)
    y <- c(10,10)-sqrt(2)/2
    W <- square(10)
    X <- ppp(x, y, W)
    compere <- function(a, b, where, tol=1e-6) {
      descrip <- paste("discrepancy in isotropic edge correction", where)
      err <- as.numeric(a) - as.numeric(b)
      maxerr <- max(abs(err))
      blurb <- paste(descrip, "is", paste0(signif(maxerr, 4), ","), 
                     if(maxerr > tol) "exceeding" else "within",
                     "tolerance of", tol)
      message(blurb)
      if(maxerr > tol) {
        message(paste("Discrepancies:", paste(err, collapse=", ")))
        stop(paste("excessive", descrip), call.=FALSE)
      }
      invisible(TRUE)
    }
    ## Testing:
    eX <- edge.Ripley(X, c(1,1))
    compere(eX, c(4/3,4/3), "at interior point of rectangle")
    ## Corner case:
    Y <- X
    Y$x <- X$x-4.5+sqrt(2)/2
    eY <- edge.Ripley(Y, c(1,1))
    compere(eY, c(2,4/3), "near corner of rectangle")
    ## Invoke polygonal code
    Z <- rotate(Y, pi/4)
    eZdebug <- edge.Ripley(Z, c(1,1), internal=list(debug=TRUE))
    compere(eZdebug, c(2,4/3), "at interior point of polygon (debug on)")
    ## test validity without debugger,in case of quirks of compiler optimisation
    eZ <- edge.Ripley(Z, c(1,1))
    compere(eZ,      c(2,4/3), "at interior point of polygon (debug off)")
  }
})
#
# tests/kppm.R
#
# $Revision: 1.33 $ $Date: 2020/04/28 12:58:26 $
#
# Test functionality of kppm that depends on RandomFields
# Test update.kppm for old style kppm objects

local({

 fit <- kppm(redwood ~1, "Thomas") # sic
 fitx <- kppm(redwood ~x, "Thomas", verbose=TRUE)
 if(FULLTEST) {
   fitx <- update(fit, ~ . + x)
   fitM <- update(fit, clusters="MatClust")
   fitC <- update(fit, cells)
   fitCx <- update(fit, cells ~ x)
   #'
   Wsub <- owin(c(0, 0.5), c(-0.5, 0))
   Zsub <- (bdist.pixels(Window(redwood)) > 0.1)
   fitWsub <- kppm(redwood ~1, "Thomas", subset=Wsub)
   fitZsub <- kppm(redwood ~1, "Thomas", subset=Zsub)
   fitWsub
 
   #' various methods
   ff <- as.fv(fitx)
   uu <- unitname(fitx)
   unitname(fitCx) <- "furlong"
   mo <- model.images(fitCx)
   p <- psib(fit)
   px <- psib(fitx)
 }
 if(ALWAYS) {
   Y <- simulate(fitx, seed=42, saveLambda=TRUE)[[1]]
 }

 if(FULLTEST) {
   #' vcov.kppm different algorithms
   vc  <- vcov(fitx)
   vc2 <- vcov(fitx, fast=TRUE)
   vc3 <- vcov(fitx, fast=TRUE, splitup=TRUE)
   vc4 <- vcov(fitx,            splitup=TRUE)

   ## other code blocks
   a <- varcount(fitx, function(x,y){x+1}) # always positive
   a <- varcount(fitx, function(x,y){y-1}) # always negative
   a <- varcount(fitx, function(x,y){x+y}) # positive or negative

   #' improve.kppm
   fitI <- update(fit, improve.type="quasi")
   fitxI <- update(fitx, improve.type="quasi")
   fitxIs <- update(fitx, improve.type="quasi", fast=FALSE) 
   #' vcov.kppm
   vcI <- vcov(fitxI)
 }

  ## plot.kppm including predict.kppm
 if(ALWAYS) {
   fitMC <- kppm(redwood ~ x, "Thomas")
   plot(fitMC)
 }
 if(FULLTEST) {
   fitCL <- kppm(redwood ~ x, "Thomas", method="c")
   fitPA <- kppm(redwood ~ x, "Thomas", method="p")
   plot(fitCL)
   plot(fitPA)

   ## fit with composite likelihood method [thanks to Abdollah Jalilian]
   fut <- kppm(redwood ~ x, "VarGamma", method="clik2", nu.ker=-3/8)
   kfut <- as.fv(fut)
 }
 
 if(require(RandomFields)) {
   fit0 <- kppm(redwood ~1, "LGCP")
   is.poisson(fit0)
   Y0 <- simulate(fit0, saveLambda=TRUE)[[1]]
   stopifnot(is.ppp(Y0))
   p0 <- psib(fit0) # issues a warning

   if(FULLTEST) {
     ## fit LGCP using K function: slow
     fit1 <- kppm(redwood ~x, "LGCP",
                  covmodel=list(model="matern", nu=0.3),
                  control=list(maxit=3))
     Y1 <- simulate(fit1, saveLambda=TRUE)[[1]]
     stopifnot(is.ppp(Y1))
   }

   ## fit LGCP using pcf
   fit1p <- kppm(redwood ~x, "LGCP",
                 covmodel=list(model="matern", nu=0.3),
                 statistic="pcf")
   Y1p <- simulate(fit1p, saveLambda=TRUE)[[1]]
   stopifnot(is.ppp(Y1p))

   ## .. and using different fitting methods
   if(FULLTEST) {
     fit1pClik <- update(fit1p, method="clik")
     fit1pPalm <- update(fit1p, method="palm")
   }
   ## image covariate (a different code block) 
   xx <- as.im(function(x,y) x, Window(redwood))
   fit1xx <- update(fit1p, . ~ xx, data=solist(xx=xx))
   Y1xx <- simulate(fit1xx, saveLambda=TRUE)[[1]]
   stopifnot(is.ppp(Y1xx))
   if(FULLTEST) {
     fit1xxVG <- update(fit1xx, clusters="VarGamma", nu=-1/4)
     Y1xxVG <- simulate(fit1xxVG, saveLambda=TRUE)[[1]]
     stopifnot(is.ppp(Y1xxVG))
   }
   fit1xxLG <- update(fit1xx, clusters="LGCP",
                      covmodel=list(model="matern", nu=0.3),
                      statistic="pcf")
   Y1xxLG <- simulate(fit1xxLG, saveLambda=TRUE, drop=TRUE)
   stopifnot(is.ppp(Y1xxLG))
   
   # ... and Abdollah's code
   if(FULLTEST) {
     fit2 <- kppm(redwood ~x, cluster="Cauchy", statistic="K")
     Y2 <- simulate(fit2, saveLambda=TRUE)[[1]]
     stopifnot(is.ppp(Y2))
   }
   # check package mechanism
   kraever("RandomFields")
   
 }

})

if(FULLTEST) {
local({
  #' various code blocks
  fut <- kppm(redwood, ~x)
  fet <- update(fut, redwood3)
  fot <- update(fut, trend=~y)
  fit <- kppm(redwoodfull ~ x)
  Y <- simulate(fit, window=redwoodfull.extra$regionII, saveLambda=TRUE)
  gut <- improve.kppm(fit, type="wclik1")
  gut <- improve.kppm(fit, vcov=TRUE, fast.vcov=TRUE, save.internals=TRUE)
  hut <- kppm(redwood ~ x, method="clik", weightfun=NULL)
  hut <- kppm(redwood ~ x, method="palm", weightfun=NULL)
  mut <- kppm(redwood)
  nut <- update(mut, Y)
})
}

if(FULLTEST) {
local({
  #' minimum contrast code
  K <- Kest(redwood)
  a <- matclust.estK(K)
  a <- thomas.estK(K)
  a <- cauchy.estK(K)
  a <- vargamma.estK(K)
  a <- lgcp.estK(K)

  print(a)
  u <- unitname(a)
  
  g <- pcf(redwood)
  a <- matclust.estpcf(g)
  a <- thomas.estpcf(g)
  a <- cauchy.estpcf(g)
  a <- vargamma.estpcf(g)
  a <- lgcp.estpcf(g)

  #' auxiliary functions
  b <- resolve.vargamma.shape(nu.pcf=1.5)
  Z <- clusterfield("Thomas", kappa=1, scale=0.2)
  
  aa <- NULL
  aa <- accumulateStatus(simpleMessage("Woof"), aa)
  aa <- accumulateStatus(simpleMessage("Sit"),  aa)
  aa <- accumulateStatus(simpleMessage("Woof"), aa)
  printStatusList(aa)

  RMIN <- 0.01
  fit <- kppm(redwood ~ 1, ctrl=list(rmin=RMIN,q=1/2))
  if(fit$Fit$mcfit$ctrl$rmin != RMIN)
    stop("kppm did not handle parameter 'rmin' in argument 'ctrl' ")
  fit <- kppm(redwood ~ 1, ctrl=list(rmin=0,q=1/2), rmin=RMIN)
  if(fit$Fit$mcfit$ctrl$rmin != RMIN)
    stop("kppm did not handle parameter 'rmin' in argument 'ctrl'")

  RMIN <- 2
  fit <- dppm(swedishpines~1, dppGauss(), ctrl=list(rmin=RMIN,q=1))
  if(fit$Fit$mcfit$ctrl$rmin != RMIN)
    stop("dppm did not handle parameter 'rmin' in argument 'ctrl'")
  fit <- dppm(swedishpines~1, dppGauss(), ctrl=list(rmin=0,q=1), rmin=RMIN)
  if(fit$Fit$mcfit$ctrl$rmin != RMIN)
    stop("dppm did not handle argument 'rmin'")
})
}

local({
  #'  experimental
  spatstat.options(kppm.canonical=TRUE, kppm.adjusted=TRUE)
  futTT1 <- kppm(redwood)
  futTT2 <- kppm(redwood, method="palm")
  futTT3 <- kppm(redwood, method="clik2")
  spatstat.options(kppm.canonical=TRUE, kppm.adjusted=FALSE)
  futTF1 <- kppm(redwood)
  futTF2 <- kppm(redwood, method="palm")
  futTF3 <- kppm(redwood, method="clik2")
  spatstat.options(kppm.canonical=FALSE, kppm.adjusted=TRUE)
  futFT1 <- kppm(redwood)
  futFT2 <- kppm(redwood, method="palm")
  futFT3 <- kppm(redwood, method="clik2")
  spatstat.options(kppm.canonical=FALSE, kppm.adjusted=FALSE)
  futFF1 <- kppm(redwood)
  futFF2 <- kppm(redwood, method="palm")
  futFF3 <- kppm(redwood, method="clik2")
  ## unsupported options that give a warning
  spatstat.options(kppm.canonical=TRUE, kppm.adjusted=TRUE)
  futXX1 <- kppm(redwood, clusters="MatClust") 
  futXX2 <- kppm(redwood, clusters="MatClust", method="palm")
  futXX3 <- kppm(redwood, clusters="MatClust", method="clik2")
  jpines <- residualspaper$Fig1
  fut <- dppm(jpines ~ 1, dppGauss)
  print(fut)
  spatstat.options(kppm.canonical=FALSE, kppm.adjusted=FALSE)
})

if(FULLTEST) {
local({
  #' cover a few code blocks
  fut <- kppm(redwood ~ x, method="clik")
  print(summary(fut))
  a <- residuals(fut)
  fut2 <- kppm(redwood ~ x, "LGCP", method="palm")
  print(summary(fut2))
  b <- residuals(fut2)
  #'
  po <- ppm(redwood ~ 1)
  A <- kppmComLik(redwood, Xname="redwood", po=po, clusters="Thomas",
                  statistic="pcf", statargs=list(), control=list(),
                  weightfun=NULL, rmax=0.1)
  A <- kppmPalmLik(redwood, Xname="redwood", po=po, clusters="Thomas",
                   statistic="pcf", statargs=list(), control=list(),
                   weightfun=NULL, rmax=0.1)
})
}

reset.spatstat.options()
  

#'     quantess.R
#' 
#'     Quantile Tessellation
#'
#'   $Revision: 1.22 $  $Date: 2019/10/17 01:49:01 $

quantess <- function(M, Z, n, ...) {
  UseMethod("quantess")
}

quantess.owin <- function(M, Z, n, ..., type=2, origin=c(0,0), eps=NULL) {
  W <- as.owin(M)
  B <- boundingbox(W)
  tcross <- MinimalTess(W, ...)
  force(n)
  if(!is.character(Z)) {
    Zim <- as.im(Z, W, eps=eps)
    Zrange <- range(Zim)
  } else {
    Z <- match.arg(Z, c("x", "y", "rad", "ang"))
    if(Z %in% c("x", "y") && is.rectangle(W)) {
      out <- switch(Z,
                    x={ quadrats(W, nx=n, ny=1) },
                    y={ quadrats(W, nx=1, ny=n) })
      if(!is.null(tcross)) out <- intersect.tess(out, tcross)
      return(out)
    }
    a <- qtPrepareCoordinate(Z, W, origin)
    Zfun <- a$Zfun
    Zrange <- a$Zrange
    Zim <- as.im(Zfun, W, eps=eps)
  }
  qZ <- quantile(Zim, probs=(0:n)/n, type=type)
  qZ[1] <- min(qZ[1], Zrange[1])
  qZ[n+1] <- max(qZ[n+1], Zrange[2])
  if(is.polygonal(W) && is.character(Z)) {
    R <- Frame(W)
    strips <- switch(Z,
                     x = tess(xgrid=qZ, ygrid=R$yrange),
                     y = tess(xgrid=R$xrange, ygrid=qZ),
                     rad = polartess(B, radii=qZ, origin=origin),
                     ang = polartess(B, angles=qZ, origin=origin))
    out <- intersect.tess(strips, tess(tiles=list(W)))
    tilenames(out) <- makeCutLabels(qZ, include.lowest=TRUE)
  } else {
    ZC <- cut(Zim, breaks=qZ, include.lowest=TRUE)
    out <- tess(image=ZC)
  }
  if(!is.null(tcross)) out <- intersect.tess(out, tcross)
  return(out)
}

qtPrepareCoordinate <- function(covname, W, origin=c(0,0)) {
  switch(covname,
         x={
           Zfun <- function(x,y){x}
           Zrange <- boundingbox(W)$xrange
         },
         y={
           Zfun <- function(x,y){y}
           Zrange <- boundingbox(W)$yrange
         },
         rad={
           origin <- interpretAsOrigin(origin, W)
           Zfun <- function(x,y) { sqrt((x-origin[1])^2+(y-origin[2])^2) }
           V <- vertices(W)
           Zrange <- range(Zfun(V$x, V$y))
         },
         ang={
           origin <- interpretAsOrigin(origin, W)
           Zstart <- 0
           Zfun <- function(x,y) {
             angle <- atan2(y-origin[2], x-origin[1]) %% (2*pi)
             if(Zstart < 0) {
               negangle <- angle - 2*pi
               angle <- ifelse(negangle >= Zstart, negangle, angle)
             }
             return(angle)
           }
           S <- as.data.frame(edges(W))
           a <- Zfun(S[,"x0"], S[,"y0"])
           b <- Zfun(S[,"x1"], S[,"y1"])
           bmina <- b - a
           swap <- (bmina > pi) | (bmina < 0 & bmina > -pi)
           arcs <- cbind(ifelse(swap, b, a), ifelse(swap, a, b))
           arcs <- lapply(apply(arcs, 1, list), unlist)
           Zunion <- circunion(arcs)
           Zrange <- c(Zunion[[1]][1], Zunion[[length(Zunion)]][2])
           if(diff(Zrange) < 0) {
             #' first interval straddles the positive x-axis
             Zstart <- Zrange[1] <- Zrange[1] - 2*pi
           } 
         })
  return(list(Zrange=Zrange, Zfun=Zfun))
}

quantess.ppp <- function(M, Z, n, ..., type=2, origin=c(0,0), eps=NULL) {
  W <- as.owin(M)
  B <- boundingbox(W)
  tcross <- MinimalTess(W, ...)
  force(n)
  if(!is.character(Z)) {
    Zim <- as.im(Z, W, eps=eps)
    ZM <- if(is.function(Z)) Z(M$x, M$y) else Zim[M]
    Zrange <- range(range(Zim), ZM)
  } else {
    Z <- match.arg(Z, c("x", "y", "rad", "ang"))
    if(Z %in% c("x", "y") && is.rectangle(W)) {
      switch(Z,
             x={
               qx <- quantile(M$x, probs=(1:(n-1))/n, type=type)
               qx <- c(W$xrange[1], qx, W$xrange[2])
               out <- tess(xgrid=qx, ygrid=W$yrange)
             },
             y={
               qy <- quantile(M$y, probs=(1:(n-1))/n, type=type)
               qy <- c(W$yrange[1], qy, W$yrange[2])
               out <- tess(xgrid=W$xrange, ygrid=qy)
             })
      if(!is.null(tcross)) out <- intersect.tess(out, tcross)
      return(out)
    }
    a <- qtPrepareCoordinate(Z, W, origin)
    Zrange <- a$Zrange
    Zfun <- a$Zfun
    ZM <- Zfun(M$x, M$y)
    Zrange <- range(Zrange, range(ZM))
    Zim <- as.im(Zfun, W, eps=eps)
  } 
  qZ <- quantile(Zim, probs=(0:n)/n, type=type)
  qZ[1] <- min(qZ[1], Zrange[1])
  qZ[n+1] <- max(qZ[n+1], Zrange[2])
  if(is.polygonal(W) && is.character(Z)) {
    R <- Frame(W)
    strips <- switch(Z,
                     x = tess(xgrid=qZ, ygrid=R$yrange),
                     y = tess(xgrid=R$xrange, ygrid=qZ),
                     rad = polartess(B, radii=qZ, origin=origin),
                     ang = polartess(B, angles=qZ, origin=origin))
    out <- intersect.tess(strips, tess(tiles=list(W)))
    tilenames(out) <- makeCutLabels(qZ, include.lowest=TRUE)
  } else {
    ZC <- cut(Zim, breaks=qZ, include.lowest=TRUE)
    out <- tess(image=ZC)
  }
  if(!is.null(tcross)) out <- intersect.tess(out, tcross)
  return(out)
}

quantess.im <- function(M, Z, n, ..., type=2, origin=c(0,0)) {
  W <- Window(M)
  tcross <- MinimalTess(W, ...)
  force(n)
  if(!(type %in% c(1,2)))
    stop("Only quantiles of type 1 and 2 are implemented for quantess.im")
  if(is.character(Z)) {
    Z <- match.arg(Z, c("x", "y", "rad", "ang"))
    a <- qtPrepareCoordinate(Z, W, origin)
    Z <- a$Zfun
    Zrange <- a$Zrange
  } else Zrange <- NULL
  MZ <- harmonise(M=M, Z=Z)
  M <- MZ$M[W, drop=FALSE]
  Z <- MZ$Z[W, drop=FALSE]
  Zrange <- range(c(range(Z), Zrange))
  Fun <- ewcdf(Z[], weights=M[]/sum(M[]))
  qZ <- quantile(Fun, probs=(1:(n-1))/n, type=type)
  qZ <- c(Zrange[1], qZ, Zrange[2])
  ZC <- cut(Z, breaks=qZ, include.lowest=TRUE)
  out <- tess(image=ZC)
  tilenames(out) <- makeCutLabels(qZ, include.lowest=TRUE)
  if(!is.null(tcross)) out <- intersect.tess(out, tcross)
  return(out)
}

MinimalTess <- function(W, ...) {
  ## find the minimal tessellation of W consistent with the arguments
  argh <- list(...)
  v <- NULL
  if(length(argh)) {
    nama <- names(argh)
    known <- union(names(formals(quadrats)),
                   names(formals(tess)))
    recognised <- !is.na(match(nama, known))
    if(any(recognised)) {
      if(any(c("nx", "ny") %in% nama)) {
        v <- do.call(quadrats,
                     resolve.defaults(list(X=W),
                                      argh[recognised],
                                      list(nx=1, ny=1)))
      } else if(any(c("xbreaks", "ybreaks") %in% nama)) {
        v <- do.call(quadrats,
                     resolve.defaults(list(X=W),
                                      argh[recognised],
                                      list(xbreaks=W$xrange,
                                           ybreaks=W$yrange)))
      } else {
        v <- do.call(tess,
                     resolve.defaults(argh[recognised],
                                      list(window=W,
                                           keepempty=TRUE)))
      }
    }
  } 
  return(v)
}

nestsplit <- function(X, ...) {
  stopifnot(is.ppp(X))
  flist <- list(...)
  cansplit <- sapply(flist, inherits,
                     what=c("factor", "tess", "owin", "im", "character"))
  splitted <- lapply(flist[cansplit], split, x=X)
  splitters <- lapply(splitted, attr, which="fsplit")
  if(any(!cansplit)) {
    extra <- do.call(MinimalTess, append(list(W=Window(X)), flist[!cansplit]))
    pos <- min(which(!cansplit))
    ns <- length(splitters)
    if(pos > ns) {
      splitters <- append(splitters, list(extra))
    } else {
      before <- splitters[seq_len(pos-1)]
      after  <- splitters[pos:ns]
      splitters <- c(before, list(extra), after)
    }
  }
  ns <- length(splitters)
  if(ns == 0) return(X)
  if(ns == 1) return(split(X, splitters[[1]]))
  if(ns > 2) stop("Nesting depths greater than 2 are not yet implemented")
  names(splitters) <- good.names(names(splitters), paste0("f", 1:ns))
  fax1 <- is.factor(sp1 <- splitters[[1]])
  fax2 <- is.factor(sp2 <- splitters[[2]])
  lev1 <- if(fax1) levels(sp1) else seq_len(sp1$n)
  lev2 <- if(fax2) levels(sp2) else seq_len(sp2$n)
  if(!fax1 && !fax2) {
    ## two tessellations
    marks(sp1) <- factor(lev1, levels=lev1)
    marks(sp2) <- factor(lev2, levels=lev2)
    sp12 <- intersect.tess(sp1, sp2, keepmarks=TRUE)
    pats <- split(X, sp12)
    f1 <- marks(sp12)[,1]
    f2 <- marks(sp12)[,2]
  } else {
    if(fax1 && fax2) {
      ## two grouping factors
      Xsp1 <- split(X, sp1)
      sp2.1 <- split(sp2, sp1)
      ll <- mapply(split, Xsp1, sp2.1, SIMPLIFY=FALSE)
    } else if(fax1 && !fax2) {
      ## grouping factor and tessellation
      Xsp1 <- split(X, sp1)
      ll <- lapply(Xsp1, split, f=sp2)
    } else if(!fax1 && fax2) {
      ## tessellation and grouping factor
      Xsp1 <- split(X, sp1)
      sp2.1 <- split(sp2, attr(Xsp1, "fgroup"))
      ll <- mapply(split, Xsp1, sp2.1, SIMPLIFY=FALSE)
    }
    neach <- lengths(ll)
    f1 <- rep(factor(lev1, levels=lev1), neach)
    f2 <- rep(factor(lev2, levels=lev2), length(Xsp1))
    pats <- do.call(c, unname(ll))
  }
  h <- hyperframe(pts=pats, f1=f1, f2=f2)
  names(h)[2:3] <- names(splitters)
  return(h)
}

#
#    summary.im.R
#
#    summary() method for class "im"
#
#    $Revision: 1.21 $   $Date: 2016/09/01 02:31:52 $
#
#    summary.im()
#    print.summary.im()
#    print.im()
#
summary.im <- function(object, ...) {
  verifyclass(object, "im")

  x <- object

  y <- unclass(x)[c("dim", "xstep", "ystep")]
  pixelarea <- y$xstep * y$ystep

  # extract image values
  v <- x$v
  inside <- !is.na(v)
  v <- v[inside]

  # type of values?
  y$type <- x$type
  
  # factor-valued?
  lev <- levels(x)
  if(!is.null(lev) && !is.factor(v))
    v <- factor(v, levels=seq_along(lev), labels=lev)

  switch(x$type,
         integer=,
         real={
           y$mean <- mv <- mean(v)
           y$integral <- mv * length(v) * pixelarea
           y$range <- ra <- range(v)
           y$min <- ra[1]  
           y$max <- ra[2]
         },
         factor={
           y$levels <- lev
           y$table <- table(v, dnn="")
         },
         complex={
           y$mean <- mv <- mean(v)
           y$integral <- mv * length(v) * pixelarea
           rr <- range(Re(v))
           y$Re <- list(range=rr, min=rr[1], max=rr[2])
           ri <- range(Im(v))
           y$Im <- list(range=ri, min=ri[1], max=ri[2])
         },
         {
           # another unknown type
           pixelvalues <- v
           y$summary <- summary(pixelvalues)
         })
    
  # summarise pixel raster
  win <- as.owin(x)
  y$window <- summary.owin(win)

  y$fullgrid <- (rescue.rectangle(win)$type == "rectangle")

  y$units <- unitname(x)
  
  class(y) <- "summary.im"
  return(y)
}

print.summary.im <- function(x, ...) {
  verifyclass(x, "summary.im")
  splat(paste0(x$type, "-valued"), "pixel image")
  unitinfo <- summary(x$units)
  pluralunits <- unitinfo$plural
  sigdig <- getOption('digits')
  di <- x$dim
  win <- x$window
  splat(di[1], "x", di[2], "pixel array (ny, nx)")
  splat("enclosing rectangle:",
        prange(signif(x$window$xrange, sigdig)),
        "x",
        prange(signif(x$window$yrange, sigdig)),
        unitinfo$plural,
        unitinfo$explain)
  splat("dimensions of each pixel:",
        signif(x$xstep, 3), "x", signif(x$ystep, sigdig),
        pluralunits)
  if(!is.null(explain <- unitinfo$explain))
    splat(explain)
  fullgrid <- x$fullgrid
  if(fullgrid) {
    splat("Image is defined on the full rectangular grid")
    whatpart <- "Frame"
  } else {
    splat("Image is defined on a subset of the rectangular grid")
    whatpart <- "Subset"
  }
  splat(whatpart, "area =", win$area, "square", pluralunits)
  if(!fullgrid) {
    af <- signif(win$areafraction, min(3, sigdig))
    splat(whatpart, "area fraction =", af)
  }
  if(fullgrid) splat("Pixel values") else
                 splat("Pixel values (inside window):")
  switch(x$type,
         integer=,
         real={
           splat("\trange =", prange(signif(x$range, sigdig)))
           splat("\tintegral =", signif(x$integral, sigdig))
           splat("\tmean =", signif(x$mean, sigdig))
         },
         factor={
           print(x$table)
         },
         complex={
           splat("\trange: Real",
                 prange(signif(x$Re$range, sigdig)),
                 "Imaginary",
                 prange(signif(x$Im$range, sigdig)))
           splat("\tintegral =", signif(x$integral, sigdig))
           splat("\tmean =", signif(x$mean, sigdig))
         },
         {
           print(x$summary)
         })

  return(invisible(NULL))
}

print.im <- function(x, ...) {
  splat(paste0(x$type, "-valued"), "pixel image")
  if(x$type == "factor") {
    splat("factor levels:")
    print(levels(x))
  }
  sigdig <- min(5, getOption('digits'))
  unitinfo <- summary(unitname(x))
  di <- x$dim
  splat(di[1], "x", di[2], "pixel array (ny, nx)")
  splat("enclosing rectangle:",
        prange(signif(zapsmall(x$xrange), sigdig)),
        "x",
        prange(signif(zapsmall(x$yrange), sigdig)),
        unitinfo$plural,
        unitinfo$explain)
  return(invisible(NULL))
}

#
#	kmrs.S
#
#	S code for Kaplan-Meier, Reduced Sample and Hanisch
#	estimates of a distribution function
#	from _histograms_ of censored data.
#
#	kaplan.meier()
#	reduced.sample()
#       km.rs()
#
#	$Revision: 3.26 $	$Date: 2013/06/27 08:59:16 $
#
#	The functions in this file produce vectors `km' and `rs'
#	where km[k] and rs[k] are estimates of F(breaks[k+1]),
#	i.e. an estimate of the c.d.f. at the RIGHT endpoint of the interval.
#

"kaplan.meier" <-
function(obs, nco, breaks, upperobs=0) {
#	obs: histogram of all observations : min(T_i,C_i)
#	nco: histogram of noncensored observations : T_i such that T_i <= C_i
# 	breaks: breakpoints (vector or 'breakpts' object, see breaks.S)
#       upperobs: number of observations beyond rightmost breakpoint
#  
        breaks <- as.breakpts(breaks)

	n <- length(obs)
	if(n != length(nco)) 
		stop("lengths of histograms do not match")
	check.hist.lengths(nco, breaks)
#
#	
#   reverse cumulative histogram of observations
	d <- revcumsum(obs) + upperobs
#
#  product integrand
	s <- ifelseXB(d > 0, 1 - nco/d, 1)
#
	km <- 1 - cumprod(s)
#  km has length n;  km[i] is an estimate of F(r) for r=breaks[i+1]
#	
	widths <- diff(breaks$val)
        lambda <- numeric(n)
        pos <- (s > 0)
        lambda[pos] <- -log(s[pos])/widths[pos]
#  lambda has length n; lambda[i] is an estimate of
#  the average of \lambda(r) over the interval (breaks[i],breaks[i+1]).
#	
	return(list(km=km, lambda=lambda))
}

"reduced.sample" <-
function(nco, cen, ncc, show=FALSE, uppercen=0)
#	nco: histogram of noncensored observations: T_i such that T_i <= C_i
#	cen: histogram of all censoring times: C_i
#	ncc: histogram of censoring times for noncensored obs:
#		C_i such that T_i <= C_i
#
#	Then nco[k] = #{i: T_i <= C_i, T_i \in I_k}
#	     cen[k] = #{i: C_i \in I_k}
#	     ncc[k] = #{i: T_i <= C_i, C_i \in I_k}.
#
#       The intervals I_k must span an interval [0,R] beginning at 0.
#       If this interval did not include all censoring times,
#       then `uppercen' must be the number of censoring times
#       that were not counted in 'cen'.
{
	n <- length(nco)
	if(n != length(cen) || n != length(ncc))
		stop("histogram lengths do not match")
#
#	denominator: reverse cumulative histogram of censoring times
#		denom(r) = #{i : C_i >= r}
#	We compute 
#		cc[k] = #{i: C_i > breaks[k]}	
#	except that > becomes >= for k=0.
#
	cc <- revcumsum(cen) + uppercen
#
#
#	numerator
#	#{i: T_i <= r <= C_i }
#	= #{i: T_i <= r, T_i <= C_i} - #{i: C_i < r, T_i <= C_i}
#	We compute
#		u[k] = #{i: T_i <= C_i, T_i <= breaks[k+1]}
#			- #{i: T_i <= C_i, C_i <= breaks[k]}
#		     = #{i: T_i <= C_i, C_i > breaks[k], T_i <= breaks[k+1]}
#	this ensures that numerator and denominator are 
#	comparable, u[k] <= cc[k] always.
#
	u <- cumsum(nco) - c(0,cumsum(ncc)[1:(n-1)])
	rs <- u/cc
#
#	Hence rs[k] = u[k]/cc[k] is an estimator of F(r) 
#	for r = breaks[k+1], i.e. for the right hand end of the interval.
#
        if(!show)
          return(rs)
        else
          return(list(rs=rs, numerator=u, denominator=cc))
}

"km.rs" <-
function(o, cc, d, breaks) {
#	o: censored lifetimes min(T_i,C_i)
#	cc: censoring times C_i
#	d: censoring indicators 1(T_i <= C_i)
#	breaks: histogram breakpoints (vector or 'breakpts' object)
#
  breaks <- as.breakpts(breaks)
  bval <- breaks$val
# compile histograms (breakpoints may not span data)
  obs <- whist( o,     breaks=bval)
  nco <- whist( o[d],  breaks=bval)
  cen <- whist( cc,    breaks=bval)
  ncc <- whist( cc[d], breaks=bval)
# number of observations exceeding largest breakpoint
  upperobs <- attr(obs, "high")
  uppercen <- attr(cen, "high")
# go
  km <- kaplan.meier(obs, nco, breaks, upperobs=upperobs)
  rs <- reduced.sample(nco, cen, ncc, uppercen=uppercen)
#
  return(list(rs=rs, km=km$km, hazard=km$lambda,
              r=breaks$r, breaks=bval))
}

"km.rs.opt" <-
function(o, cc, d, breaks, KM=TRUE, RS=TRUE) {
#	o: censored lifetimes min(T_i,C_i)
#	cc: censoring times C_i
#	d: censoring indicators 1(T_i <= C_i)
#	breaks: histogram breakpoints (vector or 'breakpts' object)
#
  breaks <- as.breakpts(breaks)
  bval <- breaks$val
  out <- list(r=breaks$r, breaks=bval)
  if(KM || RS)
    nco <- whist( o[d],  breaks=bval)
  if(KM) {
    obs <- whist( o,     breaks=bval)
    upperobs <- attr(obs, "high")
    km <- kaplan.meier(obs, nco, breaks, upperobs=upperobs)
    out <- append(list(km=km$km, hazard=km$lambda), out)
  }
  if(RS) {
    cen <- whist( cc,    breaks=bval)
    ncc <- whist( cc[d], breaks=bval)
    uppercen <- attr(cen, "high")
    rs <- reduced.sample(nco, cen, ncc, uppercen=uppercen)
    out <- append(list(rs=rs), out)
  }
  return(out)
}


censtimeCDFest <- function(o, cc, d, breaks, ...,
                           KM=TRUE, RS=TRUE, HAN=TRUE, RAW=TRUE,
                           han.denom=NULL, tt=NULL, pmax=0.9) {
# Histogram-based estimation of cumulative distribution function
# of lifetimes subject to censoring.
#	o: censored lifetimes min(T_i,C_i)
#	cc: censoring times C_i
#	d: censoring indicators 1(T_i <= C_i)
#	breaks: histogram breakpoints (vector or 'breakpts' object)
#       han.denom: denominator (eroded area) for each value of r
#       tt: uncensored lifetimes T_i, if known  
  breaks <- as.breakpts(breaks)
  bval <- breaks$val
  rval <- breaks$r
  rmax <- breaks$max
  # Kaplan-Meier and/or Reduced Sample
  out <- km.rs.opt(o, cc, d, breaks, KM=KM, RS=RS)
  # convert to data frame
  out$breaks <- NULL
  df <- as.data.frame(out)
  # Raw ecdf of observed lifetimes if available
  if(RAW && !is.null(tt)) {
    h <- whist(tt[tt <= rmax], breaks=bval)
    df <- cbind(df, data.frame(raw=cumsum(h)/length(tt)))
  }
  # Hanisch
  if(HAN) {
    if(is.null(han.denom))
      stop("Internal error: missing denominator for Hanisch estimator")
    if(length(han.denom) != length(rval))
      stop(paste("Internal error:",
                 "length(han.denom) =", length(han.denom),
                 "!=", length(rval), "= length(rvals)"))
    #  uncensored distances
    x <- o[d]
    # calculate Hanisch estimator
    h <- whist(x[x <= rmax], breaks=bval)
    H <- cumsum(h/han.denom)
    df <- cbind(df, data.frame(han=H/max(H[is.finite(H)])))
  }
  # determine appropriate plotting range
  bestest <- if(KM) "km" else if(HAN) "han" else if(RS) "rs" else "raw"
  alim <- range(df$r[df[[bestest]] <= pmax])
  # convert to fv object
  nama <-  c("r",  "km", "hazard", "han", "rs", "raw")
  avail <- c(TRUE,  KM,  KM,       HAN,   RS,   RAW)
  iscdf <- c(FALSE, TRUE, FALSE,   TRUE,  TRUE, TRUE)
  labl <- c("r", "hat(%s)[km](r)", "lambda(r)", "hat(%s)[han](r)",
            "hat(%s)[bord](r)", "hat(%s)[raw](r)")[avail]
  desc <- c("distance argument r",
            "Kaplan-Meier estimate of %s",
            "Kaplan-Meier estimate of hazard function lambda(r)",
            "Hanisch estimate of %s",
            "border corrected estimate of %s",
            "uncorrected estimate of %s")[avail]
  df <- df[, nama[avail]]
  Z <- fv(df, "r", substitute(CDF(r), NULL), bestest, . ~ r, alim, labl, desc,
          fname="CDF")
  fvnames(Z, ".") <- nama[iscdf & avail]
  return(Z)
}

# simple interface for students and code development

compileCDF <- function(D, B, r, ..., han.denom=NULL, check=TRUE) {
  han <- !is.null(han.denom)
  breaks <- breakpts.from.r(r)
  if(check) {
    stopifnot(length(D) == length(B) && all(D >= 0) && all(B >= 0))
    if(han)
      stopifnot(length(han.denom) == length(r))
  }
  D <- as.vector(D)
  B <- as.vector(B)
  # observed (censored) lifetimes
  o <- pmin.int(D, B)
  # censoring indicators
  d <- (D <= B)
  # go
  result <- censtimeCDFest(o, B, d, breaks,
                           HAN=han, 
                           han.denom=han.denom,
                           RAW=TRUE, tt=D)
  result <- rebadge.fv(result, new.fname="compileCDF")
}

#'
#' circarcs.R
#'
#'  Circular Arcs
#'
#'   An interval on the circle is specified by [left, right]
#'   meaning the arc starting at 'left' going anticlockwise until 'right'.
#'   Here 'left' and 'right' are angles in radians (mod 2*pi)
#'   from the positive x-axis.
#' 
#'   $Revision: 1.5 $ $Date: 2019/12/06 06:15:29 $

check.arc <- function(arc, fatal=TRUE) {
  if(is.numeric(arc) && length(arc) == 2)
    return(TRUE)
  if(fatal)
    stop("arc should be a numeric vector of length 2")
  return(FALSE)
}

inside.arc <- function(theta, arc) {
  check.arc(arc)
  arc <- arc %% (2*pi)
  theta <- theta %% (2*pi)
  if(arc[1] <= arc[2]) {
    #' arc does not cross the positive x-axis
    result <- (arc[1] <= theta) & (theta <= arc[2])
  } else {
    #' arc crosses the positive x-axis
    result <- (arc[1] <= theta) | (theta <= arc[2])
  }
  return(result)
}

circunion <- function(arcs) {
  stopifnot(is.list(arcs))
  nothing <- list()
  everything <- list(c(0, 2*pi))
  if(length(arcs) == 0) return(nothing)
  lapply(arcs, check.arc)
  #' extract all endpoints
  allends <- unlist(arcs)
  allends <- sortunique(as.numeric(allends) %% (2*pi))
  #' compute midpoints between each successive pair of endpoints (mod 2pi)
  midpts <- allends + diff(c(allends, allends[1] + 2*pi))/2
  #' determine which midpoints lie inside one of the arcs
  midinside <- Reduce("|", lapply(arcs, inside.arc, theta=midpts))
  zeroinside <- any(sapply(arcs, inside.arc, theta=0))
  if(!any(midinside) && !zeroinside)
    return(nothing)
  if(all(midinside) && zeroinside)
    return(everything)
  result <- nothing
  if(zeroinside) {
    #' First deal with the connected component containing 0
    #' Scan clockwise from 2*pi for left endpoint of interval
    n <- length(midinside)
    ileft <- (max(which(!midinside)) %% n) + 1L
    aleft <- allends[ileft]
    #' then anticlockwise for right endpoint
    iright <- min(which(!midinside))
    aright <- allends[iright]
    #' save this interval
    result <- append(result, list(c(aleft, aright)))
    #' remove data from consideration
    seqn <- seq_len(n)
    retain <- seqn > iright & seqn < (ileft-1L)
    midinside <- midinside[retain]
    allends   <- allends[retain]
  }
  #' Now scan anticlockwise for first midpoint that is inside the union
  while(any(midinside)) {
    ileft <- min(which(midinside))
    toright  <- (seq_along(midinside) > ileft)
    iright <- min(c(length(allends), which(!midinside & toright)))
    aleft <- allends[ileft]
    aright <- allends[iright]
    #' save this interval
    result <- append(result, list(c(aleft, aright)))
    #' throw away points that are not endpoints of the union
    midinside <- midinside[seq_along(midinside) > iright]
    allends   <- allends[seq_along(allends) > iright]
  }
  return(result)
}

# plotarc <- function(arc, ..., add=TRUE, lwd=3, rad=1){
#   if(!add || is.null(dev.list()))
#     plot(disc(), main="")
#   if(diff(arc) < 0)
#     arc[2] <- arc[2] + 2*pi
#   ang <- seq(arc[1], arc[2], by=0.01)
#   lines(rad * cos(ang), rad * sin(ang), ..., lwd=lwd)
# }
# 
# plotarcs <- function(arcs, ..., rad=1, jitter=FALSE, add=FALSE) {
#   if(length(rad) == 1) rad <- rep(rad, length(arcs))
#   if(jitter) rad <- rad * seq(0.9, 1.05, length=length(rad))
#   rad <- as.list(rad)
#   if(!add) plot(disc(), main="")
#   mapply(plotarc, arc=arcs,rad=rad, MoreArgs=list(...))
#   invisible(NULL)
# }
#   
# runifarc <- function(n=1, maxlen=pi) {
#   replicate(n, runif(1, 0, 2*pi) + c(0, runif(1, 0, maxlen)), simplify=FALSE)
# }
# 
# tryit <- function(n=5, maxlen=pi) {
#   a <- runifarc(n, maxlen=maxlen)
#   plotarcs(circunion(a), col=3, jitter=FALSE, lwd=6)
#   plotarcs(a, jitter=TRUE, lwd=2, add=TRUE)
# }

#
#
#   randommk.R
#
#   Random generators for MULTITYPE point processes
#
#   $Revision: 1.39 $   $Date: 2018/05/07 04:34:35 $
#
#   rmpoispp()   random marked Poisson pp
#   rmpoint()    n independent random marked points
#   rmpoint.I.allim()  ... internal
#   rpoint.multi()   temporary wrapper 
#
rmpoispp <- local({

  ## Argument checking
  is.numvector <- function(x) {is.numeric(x) && is.vector(x)}
  is.constant <- function(x) {is.numvector(x) && length(x) == 1}
  checkone <- function(x) {
    if(is.constant(x)) {
      if(x >= 0) return(TRUE) else stop("Intensity is negative!")
    }
    return(is.function(x) || is.im(x))
  }

  ## Ensure that m can be passed as a single value to function(x,y,m,...)
  slice.fun <- function(x,y,fun,mvalue, ...) {
    m <- if(length(mvalue) == 1) rep.int(mvalue, length(x)) else mvalue
    result <- fun(x,y,m, ...)
    return(result)
  }

  ## Main function
  rmpoispp <- 
    function(lambda, lmax=NULL, win = owin(c(0,1),c(0,1)),
             types, ..., nsim=1, drop=TRUE, warnwin=!missing(win)) {
      ## arguments:
      ##     lambda  intensity:
      ##                constant, function(x,y,m,...), image,
      ##                vector, list of function(x,y,...) or list of images
      ##
      ##     lmax     maximum possible value of lambda
      ##                constant, vector, or list
      ##
      ##     win     default observation window (of class 'owin')
      ##
      ##     types    possible types for multitype pattern
      ##    
      ##     ...     extra arguments passed to lambda()
      ##

      if(missing(types)) types <- NULL
      force(warnwin)
      
      if(nsim > 1) {
        result <- vector(mode="list", length=nsim)
        for(i in 1:nsim)
          result[[i]] <- rmpoispp(lambda, lmax, win, types, ...,
                                  warnwin=warnwin)
        return(simulationresult(result, nsim, drop))
      }
      
      ## Validate arguments
      single.arg <- checkone(lambda)
      vector.arg <- !single.arg && is.numvector(lambda) 
      list.arg <- !single.arg && is.list(lambda)
      if(! (single.arg || vector.arg || list.arg))
        stop(paste("argument", sQuote("lambda"), "not understood"))
    
      if(list.arg && !all(unlist(lapply(lambda, checkone))))
        stop(paste("Each entry in the list",
                   sQuote("lambda"),
                   "must be either a constant, a function or an image"))
      if(vector.arg && any(lambda < 0))
        stop(paste("Some entries in the vector",
                   sQuote("lambda"), "are negative"))

      ## Determine & validate the set of possible types
      if(is.null(types)) {
        if(single.arg) {
          stop(paste(sQuote("types"), "must be given explicitly when",
                     sQuote("lambda"), "is a constant, a function or an image"))
        } else if(!is.null(nama <- names(lambda)) &&
                  sum(nzchar(nama)) == length(lambda)) {
          types <- nama
        } else {
          types <- seq_along(lambda)
        }
      } 

      ntypes <- length(types)
      if(!single.arg && (length(lambda) != ntypes))
        stop(paste("The lengths of", sQuote("lambda"),
                   "and", sQuote("types"), "do not match"))

      factortype <- factor(types, levels=types)

      ## Validate `lmax'
      if(! (is.null(lmax) || is.numvector(lmax) || is.list(lmax) ))
        stop(paste(sQuote("lmax"),
                   "should be a constant, a vector, a list or NULL"))
       
      ## coerce lmax to a vector, to save confusion
      if(is.null(lmax))
        maxes <- rep(NULL, ntypes)
      else if(is.numvector(lmax) && length(lmax) == 1)
        maxes <- rep.int(lmax, ntypes)
      else if(length(lmax) != ntypes)
        stop(paste("The length of",
                   sQuote("lmax"),
                   "does not match the number of possible types"))
      else if(is.list(lmax))
        maxes <- unlist(lmax)
      else maxes <- lmax

      ## coerce lambda to a list, to save confusion
      lam <- if(single.arg) rep(list(lambda), ntypes) else
             if(vector.arg) as.list(lambda) else lambda

      ## Simulate
      for(i in 1:ntypes) {
        if(single.arg && is.function(lambda)) {
          ## call f(x,y,m, ...)
          Y <- rpoispp(slice.fun, lmax=maxes[i], win=win,
                       fun=lambda, mvalue=types[i], ..., warnwin=warnwin)
        } else {
          ## call f(x,y, ...) or use other formats
          Y <- rpoispp(lam[[i]], lmax=maxes[i], win=win, ..., warnwin=warnwin)
        }
        Y <- Y %mark% factortype[i]
        X <- if(i == 1) Y else superimpose(X, Y, W=X$window, check=FALSE)
      }

      ## Randomly permute, just in case the order is important
      permu <- sample(X$n)
      X <- X[permu]
      return(simulationresult(list(X), 1, drop))
    }

  rmpoispp
})

## ------------------------------------------------------------------------

rmpoint <- local({

  ## argument validation
  is.numvector <- function(x) {is.numeric(x) && is.vector(x)}
  is.constant <- function(x) {is.numvector(x) && length(x) == 1}
  checkone <- function(x) {
    if(is.constant(x)) {
      if(x >= 0) return(TRUE) else stop("Intensity is negative!")
    }
    return(is.function(x) || is.im(x))
  }

  # integration..
  integratexy <- function(f, win, ...) {
    imag <- as.im(f, W=win, ...)
    integral.im(imag)
  }
  ## create a counterpart of f(x,y,m) that works when m is a single value
  funwithfixedmark <- function(xx, yy, ..., m, fun) {
    mm <- rep.int(m, length(xx))
    fun(xx, yy, mm, ...)
  }
  integratewithfixedmark <- function(m, fun, win, ...) {
    integratexy(funwithfixedmark, win=win, m=m, fun=fun, ...)
  }

  # Main function
  rmpoint <- function(n, f=1, fmax=NULL, 
                      win = unit.square(), 
                      types, ptypes, ...,
                      giveup = 1000, verbose = FALSE,
                      nsim = 1, drop=TRUE) {
    if(!is.numeric(n))
      stop("n must be a scalar or vector")
    if(any(ceiling(n) != floor(n)))
      stop("n must be an integer or integers")
    if(any(n < 0))
      stop("n must be non-negative")
    if(missing(types)) types <- NULL
    if(missing(ptypes)) ptypes <- NULL

    if(nsim > 1) {
      result <- vector(mode="list", length=nsim)
      for(i in 1:nsim)
        result[[i]] <- rmpoint(n, f, fmax, win, types, ptypes, ...,
                               giveup=giveup, verbose=verbose)
      return(simulationresult(result, nsim, drop))
    }
      
    if(sum(n) == 0) {
      nopoints <- ppp(x=numeric(0), y=numeric(0), window=win, check=FALSE)
      if(!is.null(types)) {
        nomarks <- factor(types[numeric(0)], levels=types)
        nopoints <- nopoints %mark% nomarks
      }
      return(simulationresult(list(nopoints), 1, drop))
    }         
    #############
  
    Model <- if(length(n) == 1) {
      if(is.null(ptypes)) "I" else "II"
    } else "III"
  
    ##############  Validate f argument
    single.arg <- checkone(f)
    vector.arg <- !single.arg && is.numvector(f) 
    list.arg <- !single.arg && is.list(f)
    if(! (single.arg || vector.arg || list.arg))
      stop(paste("argument", sQuote("f"), "not understood"))
    
    if(list.arg && !all(unlist(lapply(f, checkone))))
      stop(paste("Each entry in the list", sQuote("f"),
                 "must be either a constant, a function or an image"))
    if(vector.arg && any(f < 0))
      stop(paste("Some entries in the vector",
                 sQuote("f"), "are negative"))
    
    ## cases where it's known that all types of points 
    ## have the same conditional density of location (x,y)
    const.density <- vector.arg ||
                     (list.arg && all(unlist(lapply(f, is.constant))))
    same.density <- const.density || (single.arg && !is.function(f))

    ################   Determine & validate the set of possible types
    if(is.null(types)) {
      if(single.arg && length(n) == 1)
        stop(paste(sQuote("types"), "must be given explicitly when",
                   sQuote("f"),
                   "is a single number, a function or an image and",
                   sQuote("n"), "is a single number"))
      else {
        basis <- if(single.arg) n else f
        if(!is.null(nama <- names(basis)) &&
           sum(nzchar(nama)) == length(basis)) {
          types <- nama
        } else {
          types <- seq_along(basis)
        }
      }
    }

    ntypes <- length(types)
    if(!single.arg && (length(f) != ntypes))
      stop(paste("The lengths of",
                 sQuote("f"), "and", sQuote("types"),
                 "do not match"))
    if(length(n) > 1 && ntypes != length(n))
      stop(paste("The lengths of",
                 sQuote("n"), "and", sQuote("types"),
                 "do not match"))

    factortype <- factor(types, levels=types)
  
    #######################  Validate `fmax'
    if(! (is.null(fmax) || is.numvector(fmax) || is.list(fmax) ))
      stop(paste(sQuote("fmax"),
                 "should be a constant, a vector, a list or NULL"))
       
    ## coerce fmax to a vector, to save confusion
    if(is.null(fmax))
      maxes <- rep(NULL, ntypes)
    else if(is.constant(fmax))
      maxes <- rep.int(fmax, ntypes)
    else if(length(fmax) != ntypes)
      stop(paste("The length of", sQuote("fmax"),
                 "does not match the number of possible types"))
    else if(is.list(fmax))
      maxes <- unlist(fmax)
    else maxes <- fmax

    ## coerce f to a list, to save confusion
    flist <- if(single.arg) rep(list(f), ntypes) else
             if(vector.arg) as.list(f) else f

    #################### START ##################################

    ## special algorithm for Model I when all f[[i]] are images

    if(Model == "I" && !same.density && all(unlist(lapply(flist, is.im)))) {
      X <- rmpoint.I.allim(n, flist, types)
      return(simulationresult(list(X), 1, drop))
    }

    ## otherwise, first select types, then locations given types
  
    if(Model == "I") {
      ## Compute approximate marginal distribution of type
      if(vector.arg)
        ptypes <- f/sum(f)
      else if(list.arg) {
        fintegrals <- unlist(lapply(flist, integratexy, win=win, ...))
        ptypes <- fintegrals/sum(fintegrals)
      } else {
        ## single argument
        if(is.constant(f)) {
          ptypes <- rep.int(1/ntypes, ntypes)
        } else {
          ## f is a function (x,y,m)
          ## convert to images and integrate
          fintegrals <- unlist(lapply(types,
                                      integratewithfixedmark,
                                      win=win, fun=f, ...))
          ## normalise
          ptypes <- fintegrals/sum(fintegrals)
        }
      }
    }

    ## Generate marks 

    if(Model == "I" || Model == "II") {
      ## i.i.d.: n marks with distribution 'ptypes'
      marques <- sample(factortype, n, prob=ptypes, replace=TRUE)
      nn <- table(marques)
    } else {
      ## multinomial: fixed number n[i] of types[i]
      repmarks <- factor(rep.int(types, n), levels=types)
      marques <- sample(repmarks)
      nn <- n
    }
    ntot <- sum(nn)

    ##############  SIMULATE !!!  #########################

    ## If all types have the same conditional density of location,
    ## generate the locations using rpoint, and return.
    if(same.density) {
      X <- rpoint(ntot, flist[[1]], maxes[[1]], win=win, ...,
                  giveup=giveup, verbose=verbose)
      X <- X %mark% marques
      return(simulationresult(list(X), 1, drop))
    }
    ## Otherwise invoke rpoint() for each type separately
    X <- ppp(numeric(ntot), numeric(ntot), window=win, marks=marques,
              check=FALSE)

    for(i in 1:ntypes) {
      if(verbose) cat(paste("Type", i, "\n"))
      if(single.arg && is.function(f)) {
        ## want to call f(x,y,m, ...)
        Y <- rpoint(nn[i], funwithfixedmark, fmax=maxes[i], win=win,
                    ..., m=factortype[i], fun=f, giveup=giveup, verbose=verbose)
      } else {
        ## call f(x,y, ...) or use other formats
        Y <- rpoint(nn[i], flist[[i]], fmax=maxes[i], win=win,
                    ..., giveup=giveup, verbose=verbose)
      }
      Y <- Y %mark% factortype[i]
      X[marques == factortype[i]] <- Y
    }
    return(simulationresult(list(X), 1, drop))
  }

  rmpoint
})

rmpoint.I.allim <- local({

  ## Extract pixel coordinates and probabilities
  get.stuff <- function(imag) {
    w <- as.mask(as.owin(imag))
    dx <- w$xstep
    dy <- w$ystep
    rxy <- rasterxy.mask(w, drop=TRUE)
    xpix <- rxy$x
    ypix <- rxy$y
    ppix <- as.vector(imag$v[w$m]) ## not normalised - OK
    npix <- length(xpix)
    return(list(xpix=xpix, ypix=ypix, ppix=ppix,
                dx=rep.int(dx,npix), dy=rep.int(dy, npix),
                npix=npix))
  }

  rmpoint.I.allim <- function(n, f, types) {
    ## Internal use only!
    ## Generates random marked points (Model I *only*)
    ## when all f[[i]] are pixel images.
    ##
    stuff <- lapply(f, get.stuff)
    ## Concatenate into loooong vectors
    xpix <- unlist(lapply(stuff, getElement, name="xpix"))
    ypix <- unlist(lapply(stuff, getElement, name="ypix"))
    ppix <- unlist(lapply(stuff, getElement, name="ppix"))
    dx   <- unlist(lapply(stuff, getElement, name="dx"))
    dy   <- unlist(lapply(stuff, getElement, name="dy"))
    ## replicate types
    numpix <- unlist(lapply(stuff, getElement, name="npix"))
    tpix <- rep.int(seq_along(types), numpix)
    ##
    ## sample pixels from union of all images
    ##
    npix <- sum(numpix)
    id <- sample(npix, n, replace=TRUE, prob=ppix)
    ## get pixel centre coordinates and randomise within pixel
    x <- xpix[id] + (runif(n) - 1/2) * dx[id]
    y <- ypix[id] + (runif(n) - 1/2) * dy[id]
    ## compute types
    marx <- factor(types[tpix[id]],levels=types)
    ## et voila!
    return(ppp(x, y, window=as.owin(f[[1]]), marks=marx, check=FALSE))
  }

  rmpoint.I.allim
})

##
##     wrapper for Rolf's function
##
rpoint.multi <- function (n, f, fmax=NULL, marks = NULL,
                          win = unit.square(),
                          giveup = 1000, verbose = FALSE,
                          warn=TRUE, nsim=1, drop=TRUE) {
  if(nsim > 1) {
    result <- vector(mode="list", length=nsim)
    for(i in 1:nsim)
      result[[i]] <- rpoint.multi(n, f, fmax, marks, win, giveup, verbose)
    return(simulationresult(result, nsim, drop))
  }
  
  no.marks <- is.null(marks) ||
               (is.factor(marks) && length(levels(marks)) == 1)
  if(warn) {
    nhuge <- spatstat.options("huge.npoints")
    if(n > nhuge)
      warning(paste("Attempting to generate", n, "random points"))
  }
  ## unmarked case
  if (no.marks) {
    X <- if(is.function(f)) {
      rpoint(n, f, fmax, win, giveup=giveup, verbose=verbose)
    } else {
      rpoint(n, f, fmax, giveup=giveup, verbose=verbose)
    }
    return(simulationresult(list(X), 1, drop))
  }
  ## multitype case
  if(length(marks) != n)
    stop("length of marks vector != n")
  if(!is.factor(marks))
    stop("marks should be a factor")
  types <- levels(marks)
  types <- factor(types, levels=types)
  ## generate required number of points of each type
  nums <- table(marks)
  X <- rmpoint(nums, f, fmax, win=win, types=types,
               giveup=giveup, verbose=verbose)
  if(any(table(marks(X)) != nums))
    stop("Internal error: output of rmpoint illegal")
  ## reorder them to correspond to the desired 'marks' vector
  Y <- X
  Xmarks <- marks(X)
  for(ty in types) {
    to   <- (marks == ty)
    from <- (Xmarks == ty)
    if(sum(to) != sum(from))
      stop(paste("Internal error: mismatch for mark =", ty))
    if(any(to)) {
      Y$x[to] <- X$x[from]
      Y$y[to] <- X$y[from]
      Y$marks[to] <- ty
    }
  }
  return(simulationresult(list(Y), 1, drop))
}


  
  

    

#
#   pp3.R
#
#  class of three-dimensional point patterns in rectangular boxes
#
#  $Revision: 1.30 $  $Date: 2019/01/23 02:41:43 $
#

box3 <- function(xrange=c(0,1), yrange=xrange, zrange=yrange, unitname=NULL) {
  stopifnot(is.numeric(xrange) && length(xrange) == 2 && diff(xrange) > 0)
  stopifnot(is.numeric(yrange) && length(yrange) == 2 && diff(yrange) > 0)
  stopifnot(is.numeric(zrange) && length(zrange) == 2 && diff(zrange) > 0)
  out <- list(xrange=xrange, yrange=yrange, zrange=zrange,
              units=as.unitname(unitname))
  class(out) <- "box3"
  return(out)
}

as.box3 <- function(...) {
  a <- list(...)
  n <- length(a)
  if(n == 0)
    stop("No arguments given")
  if(n == 1) {
    a <- a[[1]]
    if(inherits(a, "box3"))
      return(a)
    if(inherits(a, "pp3"))
      return(a$domain)
    if(inherits(a, "boxx")){
      if(ncol(a$ranges)==3)
        return(box3(a$ranges[,1], a$ranges[,2], a$ranges[,3]))
      stop("Supplied boxx object does not have dimension three")
    }
    if(inherits(a, "ppx"))
      return(as.box3(a$domain))
    if(is.numeric(a)) {
      if(length(a) == 6)
        return(box3(a[1:2], a[3:4], a[5:6]))
      stop(paste("Don't know how to interpret", length(a), "numbers as a box"))
    }
    if(!is.list(a))
      stop("Don't know how to interpret data as a box")
  }
  return(do.call(box3, a))
}

print.box3 <- function(x, ...) {
  bracket <- function(z) paste("[",
                               paste(signif(z, 5), collapse=", "),
                               "]", sep="")
  v <- paste(unlist(lapply(x[1:3], bracket)), collapse=" x ")
  s <- summary(unitname(x))
  splat("Box:", v, s$plural, s$explain)
  invisible(NULL)
}

unitname.box3 <- function(x) { as.unitname(x$units) }

"unitname<-.box3" <- function(x, value) {
  x$units <- as.unitname(value)
  return(x)
}

grow.box3 <- function(W, left, right=left) {
  as.box3(grow.boxx(as.boxx(W), left, right))
}

eroded.volumes <- function(x, r) { UseMethod("eroded.volumes") }

eroded.volumes.box3 <- function(x, r) {
  b <- as.box3(x)
  ax <- pmax.int(0, diff(b$xrange) - 2 * r)
  ay <- pmax.int(0, diff(b$yrange) - 2 * r)
  az <- pmax.int(0, diff(b$zrange) - 2 * r)
  ax * ay * az
}

shortside <- function(x) { UseMethod("shortside") }

shortside.box3 <- function(x) {
  min(sidelengths(x))
}

sidelengths <- function(x) { UseMethod("sidelengths") }

sidelengths.box3 <- function(x) {
  with(x, c(diff(xrange), diff(yrange), diff(zrange)))
}

bounding.box3 <- function(...) {
  wins <- list(...)
  boxes <- lapply(wins, as.box3)
  xr <- range(unlist(lapply(boxes, getElement, name="xrange")))
  yr <- range(unlist(lapply(boxes, getElement, name="yrange")))
  zr <- range(unlist(lapply(boxes, getElement, name="zrange")))
  box3(xr, yr, zr)
}

pp3 <- function(x, y, z, ..., marks=NULL) {
  stopifnot(is.numeric(x))
  stopifnot(is.numeric(y))
  stopifnot(is.numeric(z)) 
  b <- as.box3(...)
  out <- ppx(data=data.frame(x=x,y=y,z=z), domain=b)
  class(out) <- c("pp3", class(out))
  if(!is.null(marks)) marks(out) <- marks
  return(out)
}

domain.pp3 <- function(X, ...) { X$domain }

is.pp3 <- function(x) { inherits(x, "pp3") }

npoints.pp3 <- function(x) { nrow(x$data) }

print.pp3 <- function(x, ...) {
  ism <- is.marked(x, dfok=TRUE)
  nx <- npoints(x)
  splat(if(ism) "Marked three-dimensional" else "Three-dimensional",
        "point pattern:",
        nx, ngettext(nx, "point", "points"))
  if(ism) {
    mks <- marks(x, dfok=TRUE)
    if(is.data.frame(mks) | is.hyperframe(mks)) {
      ## data frame of marks
      exhibitStringList("Mark variables:", names(mks))
    } else {
      ## vector of marks
      if(is.factor(mks)) {
        exhibitStringList("Multitype, with levels =", levels(mks))
      } else {
        ## Numeric, or could be dates
        if(inherits(mks, "Date")) {
          splat("marks are dates, of class", sQuote("Date"))
        } else if(inherits(mks, "POSIXt")) {
          splat("marks are dates, of class", sQuote("POSIXt"))
        } else {
          splat(paste0("marks are", if(is.numeric(mks)) " numeric," else NULL),
                "of storage type ", sQuote(typeof(mks)))
        }
      }
    }
  }
  print(x$domain)
  invisible(NULL)
}

summary.pp3 <- function(object, ...) {
  sd <- summary(object$data)
  np <- sd$ncases
  dom <- object$domain
  v <- volume.box3(dom)
  u <- summary(unitname(dom))
  intens <- np/v
  out <-  list(np=np, sumdat=sd, dom=dom, v=v, u=u, intensity=intens)
  class(out) <- "summary.pp3"
  return(out)
}

print.summary.pp3 <- function(x, ...) {
  splat("Three-dimensional point pattern")
  splat(x$np, ngettext(x$np, "point", "points"))
  print(x$dom)
  u <- x$u
  v <- x$v
  splat("Volume", v, "cubic",
        if(v == 1) u$singular else u$plural,
        u$explain)
  splat("Average intensity", x$intensity,
        "points per cubic", u$singular, u$explain)
  invisible(NULL)
}

plot.pp3 <- function(x, ..., eye=NULL, org=NULL, theta=25, phi=15,
                     type=c("p", "n", "h"),
                     box.back=list(col="pink"),
                     box.front=list(col="blue", lwd=2)) {
  xname <- short.deparse(substitute(x))
  type <- match.arg(type)
  # given arguments
  argh <- list(...)
  if(!missing(box.front)) argh$box.front <- box.front
  if(!missing(box.back))  argh$box.back  <- box.back
  # Now apply formal defaults above
  formaldefaults <- list(box.front=box.front, box.back=box.back)
  #'
  coo <- as.matrix(coords(x))
  xlim <- x$domain$xrange
  ylim <- x$domain$yrange
  zlim <- x$domain$zrange
  if(is.null(org)) org <- c(mean(xlim), mean(ylim), mean(zlim))
  if(is.null(eye)) {
    theta <- theta * pi/180
    phi   <- phi * pi/180
    d <- 2 * diameter(x$domain)
    eye <- org + d * c(cos(phi) * c(sin(theta), -cos(theta)), sin(phi))
  }
  deefolts <- spatstat.options('par.pp3')
  ## determine default eye position and centre of view
  do.call(plot3Dpoints,
          resolve.defaults(list(xyz=coo, eye=eye, org=org, type=type),
                           argh,
                           deefolts,
                           formaldefaults,
                           list(main=xname,
                                xlim=xlim,
                                ylim=ylim,
                                zlim=zlim)))
}

"[.pp3" <- function(x, i, drop=FALSE, ...) {
  answer <- NextMethod("[")
  if(is.ppx(answer))
    class(answer) <- c("pp3", class(answer))
  return(answer)
}
  
unitname.pp3 <- function(x) { unitname(x$domain) }

"unitname<-.pp3" <- function(x, value) {
  d <- x$domain
  unitname(d) <- value
  x$domain <- d
  return(x)
}

diameter.box3 <- function(x) {
  stopifnot(inherits(x, "box3"))
  with(x, sqrt(diff(xrange)^2+diff(yrange)^2+diff(zrange)^2))
}

volume <- function(x) { UseMethod("volume") }

volume.box3 <- function(x) {
  stopifnot(inherits(x, "box3"))
  with(x, prod(diff(xrange), diff(yrange), diff(zrange)))
}

runifpoint3 <- function(n, domain=box3(), nsim=1, drop=TRUE) {
  domain <- as.box3(domain)
  result <- vector(mode="list", length=nsim)
  dd <- as.list(domain)[c("xrange", "yrange", "zrange")]
  for(i in 1:nsim) {
    x <- with(dd, runif(n, min=xrange[1], max=xrange[2]))
    y <- with(dd, runif(n, min=yrange[1], max=yrange[2]))
    z <- with(dd, runif(n, min=zrange[1], max=zrange[2]))
    result[[i]] <- pp3(x,y,z,domain)
  }
  if(drop && nsim == 1) return(result[[1]])
  result <- as.anylist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}

rpoispp3 <- function(lambda, domain=box3(), nsim=1, drop=TRUE) {
  domain <- as.box3(domain)
  v <- volume(domain)
  if(!(is.numeric(lambda) && length(lambda) == 1))
    stop("lambda must be a single numeric value")
  np <- rpois(nsim, lambda * v)
  dd <- as.list(domain)[c("xrange", "yrange", "zrange")]
  result <- vector(mode="list", length=nsim)
  for(i in 1:nsim) {
    ni <- np[i]
    x <- with(dd, runif(ni, min=xrange[1], max=xrange[2]))
    y <- with(dd, runif(ni, min=yrange[1], max=yrange[2]))
    z <- with(dd, runif(ni, min=zrange[1], max=zrange[2]))
    result[[i]] <- pp3(x,y,z,domain)
  }
  if(drop && nsim == 1) return(result[[1]])
  result <- as.anylist(result)
  names(result) <- paste("Simulation", 1:nsim)
  return(result)
}

#
#  nearestsegment.R
#
#  $Revision: 1.12 $  $Date: 2018/03/07 01:56:36 $
#
# Given a point pattern X and a line segment pattern Y,
# for each point x of X, determine which segment of Y is closest to x
# and find the point on Y closest to x.
#

nearestsegment <- function(X,Y) {
  return(ppllengine(X,Y,"identify"))
}

project2segment <- function(X, Y) {
  return(ppllengine(X,Y,"project"))
}
  
ppllengine <- function(X, Y, action="project", check=FALSE) {
  stopifnot(is.ppp(X))
  stopifnot(is.psp(Y))
  stopifnot(action %in% c("distance", "identify", "project"))
  # deal with empty patterns
  if(X$n == 0) {
    nowt <- numeric(0)
    none <- integer(0)
    switch(action,
           identify = return(none),
           distance = return(list(dist=nowt, which=none)),
           project  = return(list(Xproj=X, mapXY=none, d=nowt, tp=nowt)))
  }
  if(Y$n == 0)
    stop("Segment pattern Y contains 0 segments; projection undefined")
  #              
  XX <- as.matrix(as.data.frame(unmark(X)))
  YY <- as.matrix(as.data.frame(unmark(Y)))
  # determine which segment lies closest to each point
  huge <- max(diameter(as.rectangle(as.owin(X))),
              diameter(as.rectangle(as.owin(Y))))
  d <- distppllmin(XX, YY, huge^2)
  mapXY <- d$min.which
  if(action == "identify")
    return(mapXY)
  else if(action == "distance") 
    return(data.frame(dist=d$min.d, which=mapXY))
  
  # combine relevant rows of data
  alldata <- as.data.frame(cbind(XX, YY[mapXY, ,drop=FALSE]))
  colnames(alldata) <- c("x", "y", "x0", "y0", "x1", "y1")
  # coordinate geometry
  dx <- with(alldata, x1-x0)
  dy <- with(alldata, y1-y0)
  leng <- sqrt(dx^2 + dy^2)
  # rotation sines & cosines (may include 0/0)
  co <- dx/leng
  si <- dy/leng
  # vector to point from first endpoint of segment
  xv <- with(alldata, x - x0)
  yv <- with(alldata, y - y0)
  # rotate coordinate system so that x axis is parallel to line segment
  xpr <- xv * co + yv * si
#  ypr <- - xv * si + yv * co
  # determine whether projection is an endpoint or interior point of segment
  ok <- is.finite(xpr)
  left <- !ok | (xpr <= 0)
  right <- ok &  (xpr >= leng)
  # location of projected point in rotated coordinates
  xr <- with(alldata, ifelseAX(left, 0, ifelseXY(right, leng, xpr)))
  # back to standard coordinates
  xproj <- with(alldata, x0 + ifelseXB(ok, xr * co, 0))
  yproj <- with(alldata, y0 + ifelseXB(ok, xr * si, 0))
  Xproj <- ppp(xproj, yproj, window=X$window, marks=X$marks, check=check)
  # parametric coordinates
  tp <- xr/leng
  tp[!is.finite(tp)] <- 0
  # 
  return(list(Xproj=Xproj, mapXY=mapXY, d=d$min.d, tp=tp))
}

#
#	dummy.S
#
#	Utilities for generating patterns of dummy points
#
#       $Revision: 5.32 $     $Date: 2018/09/28 05:11:55 $
#
#	corners()	corners of window
#	gridcenters()	points of a rectangular grid
#	stratrand()	random points in each tile of a rectangular grid
#	spokes()	Rolf's 'spokes' arrangement
#	
#	concatxy()	concatenate any lists of x, y coordinates
#
#	default.dummy()	Default action to create a dummy pattern
#		
	
corners <- function(window) {
	window <- as.owin(window)
	x <- window$xrange[c(1L,2L,1L,2L)]
	y <- window$yrange[c(1L,1L,2L,2L)]
	return(list(x=x, y=y))
}

gridcenters <-	
gridcentres <- function(window, nx, ny) {
	window <- as.owin(window)
	xr <- window$xrange
	yr <- window$yrange
	x <- seq(from=xr[1L], to=xr[2L], length.out = 2L * nx + 1L)[2L * (1:nx)]
	y <- seq(from=yr[1L], to=yr[2L], length.out = 2L * ny + 1L)[2L * (1:ny)]
	x <- rep.int(x, ny)
	y <- rep.int(y, rep.int(nx, ny))
	return(list(x=x, y=y))
}

stratrand <- function(window,nx,ny, k=1) {
	
	# divide window into an nx * ny grid of tiles
	# and place k points at random in each tile
	
	window <- as.owin(window)

	wide  <- diff(window$xrange)/nx
	high  <- diff(window$yrange)/ny
        cent <- gridcentres(window, nx, ny)
	cx <- rep.int(cent$x, k)
	cy <- rep.int(cent$y, k)
	n <- nx * ny * k
	x <- cx + runif(n, min = -wide/2, max = wide/2)
	y <- cy + runif(n, min = -high/2, max = high/2)
	return(list(x=x,y=y))
}

tilecentroids <- function (W, nx, ny)
{
  W <- as.owin(W)
  if(W$type == "rectangle")
    return(gridcentres(W, nx, ny))
  else {
    # approximate
    W   <- as.mask(W)
    rxy <- rasterxy.mask(W, drop=TRUE)
    xx  <- rxy$x
    yy  <- rxy$y
    pid <- gridindex(xx,yy,W$xrange,W$yrange,nx,nx)$index
    x   <- tapply(xx,pid,mean)
    y   <- tapply(yy,pid,mean)
    return(list(x=x,y=y))
  }
}

cellmiddles <- local({
  # auxiliary 
  middle <- function(v) { n <- length(v);
                          mid <- ceiling(n/2);
                          v[mid]}

  dcut <- function(x, nx, xrange) {
    dx <- diff(xrange)/nx
    fx <- ((x - xrange[1L])/dx) %% 1
    bx <- dx * pmin(fx, 1-fx)
    bx
  }
  
  # main
  cellmiddles <- function (W, nx, ny, npix=NULL, distances=FALSE) {
    if(W$type == "rectangle")
      return(gridcentres(W, nx, ny))

    # pixel approximation to window
    # This matches the pixel approximation used to compute tile areas
    # and ensures that dummy points are generated only inside those tiles
    # that have nonzero digital area
    M   <- as.mask(W, dimyx=rev(npix))
    xx <- as.vector(rasterx.mask(M, drop=TRUE))
    yy <- as.vector(rastery.mask(M, drop=TRUE))
    pid <- gridindex(xx,yy,W$xrange,W$yrange,nx,ny)$index

    # compute tile centroids
    xmid <- tapply(xx, pid, mean)
    ymid <- tapply(yy, pid, mean)
    # check whether they are inside window
    ok <- inside.owin(xmid, ymid, W)
    if(all(ok))
      return(list(x=xmid, y=ymid))

    # some problem tiles
    bad <- rep.int(TRUE, nx * ny)
    bad[as.integer(names(xmid))] <- !ok
    badpid <- bad[pid]
    if(!distances) {
       midpix <- tapply(seq_along(pid)[badpid], pid[badpid], middle)
    } else {
      # find 'middle' points using boundary distances
      Dlines <- im(outer(dcut(M$yrow,ny,M$yrange),
                         dcut(M$xcol,nx,M$xrange),
                         "pmin"),
                   M$xcol, M$yrow, M$xrange, M$yrange)
      Dbdry <- bdist.pixels(M)
      Dtile <- eval.im(pmin(Dlines, Dbdry))
      dtile <- as.vector(Dtile[M])
      df <- data.frame(dtile=dtile, id=seq_along(dtile))[badpid, , drop=FALSE]
      midpix <- by(df, pid[badpid], midpixid)
    }
    xmid[!ok] <- xx[midpix]
    ymid[!ok] <- yy[midpix]
    return(list(x=xmid,y=ymid))
  }

  midpixid <- function(z) { z$id[which.max(z$dtile)] }
  
  cellmiddles
})

spokes <- function(x, y, nrad = 3, nper = 3, fctr = 1.5, Mdefault=1) {
	#
	# Rolf Turner's "spokes" arrangement
	#
	# Places dummy points on radii of circles 
	# emanating from each data point x[i], y[i]
	#
	#       nrad:    number of radii from each data point
	#       nper:	 number of dummy points per radius
	#       fctr:	 length of largest radius = fctr * M
	#                where M is mean nearest neighbour distance in data
	#
        pat <- inherits(x,"ppp")
        if(pat) w <- x$w
        if(checkfields(x,c("x","y"))) {
          y <- x$y
          x <- x$x
        }
        M <- if(length(x) > 1) mean(nndist(x,y)) else Mdefault
	lrad  <- fctr * M / nper
	theta <- 2 * pi * (1:nrad)/nrad
	cs    <- cos(theta)
	sn    <- sin(theta)
	xt    <- lrad * as.vector((1:nper) %o% cs)
	yt    <- lrad * as.vector((1:nper) %o% sn)
	xd    <- as.vector(outer(x, xt, "+"))
	yd    <- as.vector(outer(y, yt, "+"))
	
        tmp <- list(x = xd, y = yd)
        if(pat) return(as.ppp(tmp,W=w)[w]) else return(tmp)
}
	
# concatenate any number of list(x,y) into a list(x,y)
		
concatxy <- function(...) {
	x <- unlist(lapply(list(...), getElement, name="x"))
	y <- unlist(lapply(list(...), getElement, name="y"))
	if(length(x) != length(y))
		stop("Internal error: lengths of x and y unequal")
	return(list(x=x,y=y))
}

#------------------------------------------------------------

default.dummy <- function(X, nd=NULL, random=FALSE, ntile=NULL, npix = NULL,
                          quasi=FALSE, ..., eps=NULL, verbose=FALSE) {
  # default action to create dummy points.
  # regular grid of nd[1] * nd[2] points
  # plus corner points of window frame,
  # all clipped to window.
  orig <- list(nd=nd, eps=eps, ntile=ntile, npix=npix)
  orig <- orig[!sapply(orig, is.null)]
  # 
  X <- as.ppp(X)
  win <- X$window
  #
  #
  # default dimensions
  a <- default.n.tiling(X, nd=nd, ntile=ntile, npix=npix,
                        eps=eps, random=random, quasi=quasi, verbose=verbose)
  nd    <- a$nd
  ntile <- a$ntile
  npix  <- a$npix
  periodsample <- !quasi && !random &&
                  is.mask(win) &&
                  all(nd %% win$dim == 0)
  # make dummy points
  dummy <- if(quasi) rQuasi(prod(nd), as.rectangle(win)) else
           if(random) stratrand(win, nd[1L], nd[2L], 1) else 
           cellmiddles(win, nd[1L], nd[2L], npix)
  dummy <- as.ppp(dummy, win, check=FALSE)
  # restrict to window
  if(!is.rectangle(win) && !periodsample)
    dummy <- dummy[win]
  # corner points
  corn <- as.ppp(corners(win), win, check=FALSE)
  corn <- corn[win]
  dummy <- superimpose(dummy, corn, W=win, check=FALSE)
  if(dummy$n == 0)
    stop("None of the dummy points lies inside the window")
  # pass parameters for computing weights
  attr(dummy, "weight.parameters") <-
    append(list(...), list(ntile=ntile, verbose=verbose, npix=npix))
  # record parameters used to create dummy locations
  attr(dummy, "dummy.parameters") <-
    list(nd=nd, random=random, quasi=quasi, verbose=verbose, orig=orig)
  return(dummy)
}


# Criteria:
#   for rectangular windows,
#       R1.  nd >= ntile
#   for non-rectangular windows,
#       R2. nd should be a multiple of ntile
#       R3. each dummy point is also a pixel of the npix grid
#       R4. npix should ideally be a multiple of nd, for speed
#       R5. npix should be large, for accuracy
#       R6. npix should not be too large, for speed
#       R7. if the window is a mask, npix should ideally be
#           a multiple of the mask array dimensions, for speed.
#

default.n.tiling <- local({
  # auxiliary
  ensure2print <- function(x, verbose=TRUE, blah="user specified") {
    xname <- short.deparse(substitute(x))
    x <- ensure2vector(x)
    if(verbose)
      cat(paste(blah, xname, "=", x[1L], "*", x[2L], "\n"))
    x
  }
  minmultiple <- function(n, lo, hi) {
    if(lo > hi) {
      temp <- hi
      hi <- lo
      lo <- temp
    }
    if(n > hi) return(hi)
    m <- n * (floor(lo/n):ceiling(hi/n))
    m <- m[m >= n & m >= lo & m <= hi]
    if(length(m) > 0) min(m) else hi
  }
    
  mindivisor <- function(N, lo, Nbig) {
    d <- divisors(N)
    ok <- (d >= lo)
    if(any(ok)) return(min(d[ok]))
    m <- floor(Nbig/N)
    d <- unlist(lapply(as.list(seq_len(m) * N), divisors))
    d <- sortunique(d)
    ok <- (d >= lo)
    if(any(ok)) return(min(d[ok]))
    return(Nbig)
  }

  min2mul <- function(n, lo, hi) 
    c(minmultiple(n[1L], lo[1L], hi[1L]),
      minmultiple(n[2L], lo[2L], hi[2L]))

  min2div <- function(N, lo, Nbig) 
    c(mindivisor(N[1L], lo[1L], Nbig[1L]),
      mindivisor(N[2L], lo[2L], Nbig[2L]))

  maxdiv <- function(n, k=1) {
    if(length(n) > 1L)
      return(c(maxdiv(n[1L], k),
               maxdiv(n[2L], k)))
    ## k-th largest divisor other than n
    d <- divisors(n)
    m <- length(d)
    ans <- if(m == 2L) n else if(m < 2+k) d[2L] else d[m-k]
    return(ans)
  }

  # main
  default.n.tiling <- function(X,
                               nd=NULL, ntile=NULL, npix=NULL,
                               eps=NULL,
                               random=FALSE, quasi=FALSE, verbose=TRUE) {
  # computes dimensions of rectangular grids of 
  #     - dummy points  (nd) (eps)
  #     - tiles for grid weights (ntile)
  #     - pixels for approximating area (npix)
  # for data pattern X.
  #
  verifyclass(X, "ppp")
  win <- X$window
  pixels <- (win$type != "rectangle")
  
  if(nd.given <- !is.null(nd)) 
    nd <- ensure2print(nd, verbose)
  if(ntile.given <- !is.null(ntile)) 
    ntile <- ensure2print(ntile, verbose)
  if(npix.given <- !is.null(npix)) 
    npix <- ensure2print(npix, verbose)

  if(pixels) 
    sonpixel <- rev(ensure2print(spatstat.options("npixel"), verbose, ""))

  ndummy.min <- ensure2print(spatstat.options("ndummy.min"), verbose, "")
  ndminX <- pmax(ndummy.min, 10 * ceiling(2 * sqrt(X$n)/10))
  ndminX <- ensure2vector(ndminX)

  if(!is.null(eps)) {
    eps <- ensure2print(eps, verbose)
    Xbox <- as.rectangle(as.owin(X))
    sides <- with(Xbox, c(diff(xrange), diff(yrange)))
    ndminX <- pmax(ndminX, ceiling(sides/eps))
  }

  # range of acceptable values for npix
  if(npix.given)
    Nmin <- Nmax <- npix
  else 
    switch(win$type,
           rectangle = {
             Nmin <- ensure2vector(X$n)
             Nmax <- Inf
           },
           polygonal = {
             Nmin <- sonpixel
             Nmax <- 4 * sonpixel
           },
           mask={
             nmask <- rev(win$dim)
             Nmin <- nmask
             Nmax <- pmax(2 * nmask, 4 * sonpixel)
           })

  # determine values of nd and ntile

  if(nd.given && !ntile.given) {
    # ntile must be a divisor of nd
    if(any(nd > Nmax))
      warning("number of dummy points nd exceeds maximum pixel dimensions")
    ntile <- min2div(nd, ndminX, nd)
  } else if(!nd.given && ntile.given) {
    # nd must be a multiple of ntile
    nd <- min2mul(ntile, ndminX, Nmin)
    if(any(nd >= Nmin))
      nd <- ntile
  } else if(!nd.given && !ntile.given) {
     if(!pixels) {
       nd <- ntile <- ensure2vector(ndminX)
       if(verbose)
         cat(paste("nd and ntile default to", nd[1L], "*", nd[2L], "\n"))
     } else {
       # find suitable divisors of the number of pixels
       nd <- ntile <- min2div(Nmin, ndminX, Nmax)
       if(any(nd >= Nmin)) { # none suitable
         if(verbose)
           cat("No suitable divisor of pixel dimensions\n")
         nd <- ntile <- ndminX
       }
     }
  } else {
    # both nd, ntile were given
    if(any(ntile > nd))
      warning("the number of tiles (ntile) exceeds the number of dummy points (nd)")
  }

  if(!ntile.given && quasi) {
    if(verbose) cat("Adjusting ntile because quasi=TRUE\n")
    ntile <- maxdiv(ntile, if(pixels) 2L else 1L)
  } 
 
  if(!npix.given && pixels) 
    npix <- min2mul(nd, Nmin, Nmax)

  if(verbose) {
    if(!quasi)
      cat(paste("dummy points:",
                paste0(if(random) "stratified random in" else NULL,
                       "grid"),
                nd[1L], "x", nd[2L], "\n"))
    else
      cat(paste("dummy points:",
                nd[1L], "x", nd[2L], "=", prod(nd),
                "quasirandom points\n"))
    cat(paste("weighting tiles", ntile[1L], "x", ntile[2L], "\n"))
    if(pixels) cat(paste("pixel grid", npix[1L], "x", npix[2L], "\n"))
  }

  if(pixels) 
    return(list(nd=nd, ntile=ntile, npix=npix))
  else
    return(list(nd=nd, ntile=ntile, npix=npix))
}

  default.n.tiling
})

#
#   deltametric.R
#
#   Delta metric
#
#   $Revision: 1.4 $  $Date: 2014/10/24 00:22:30 $
#

deltametric <- function(A, B, p=2, c=Inf, ...) {
  stopifnot(is.numeric(p) && length(p) == 1L && p > 0)
  # ensure frames are identical
  bb <- boundingbox(as.rectangle(A), as.rectangle(B))
  # enforce identical frames
  A <- rebound(A, bb)
  B <- rebound(B, bb)
  # compute distance functions
  dA <- distmap(A, ...)
  dB <- distmap(B, ...)
  if(!is.infinite(c)) {
    dA <- eval.im(pmin.int(dA, c))
    dB <- eval.im(pmin.int(dB, c))
  }
  if(is.infinite(p)) {
    # L^infinity
    Z <- eval.im(abs(dA-dB))
    delta <- summary(Z)$max
  } else {
    # L^p
    Z <- eval.im(abs(dA-dB)^p)
    iZ <- summary(Z)$mean
    delta <- iZ^(1/p)
  }
  return(delta)
}

##
##   randomNS.R
##
##   simulating from Neyman-Scott processes
##
##   $Revision: 1.26 $  $Date: 2019/03/19 08:32:39 $
##
##    Original code for rCauchy and rVarGamma by Abdollah Jalilian
##    Other code and modifications by Adrian Baddeley
##    Bug fixes by Abdollah, Adrian, and Rolf Turner

rNeymanScott <- 
  function(kappa, expand, rcluster, win = owin(c(0,1),c(0,1)), ...,
           lmax=NULL, nsim=1, drop=TRUE, nonempty=TRUE, saveparents=TRUE)
{
  ## Generic Neyman-Scott process
  ## Implementation for bounded cluster radius
  ##

  ## Catch old argument name rmax for expand
  if(missing(expand) && !is.null(rmax <- list(...)$rmax))
    expand <- rmax
    
  ## 'rcluster' may be
  ##
  ##     (1) a function(x,y, ...) that takes the coordinates
  ##         (x,y) of the parent point and generates a list(x,y) of offspring
  ##

  if(is.function(rcluster))
    return(rPoissonCluster(kappa, expand, rcluster, win, ...,
                           lmax=lmax, nsim=nsim, drop=drop,
                           saveparents=saveparents))

  ##     (2) a list(mu, f) where mu is a numeric value, function, or pixel image
  ##         and f is a function(n, ...) generating n i.i.d. offspring at 0,0
  
  if(!(is.list(rcluster) && length(rcluster) == 2))
    stop("rcluster should be either a function, or a list of two elements")
  win <- as.owin(win)
  mu <- rcluster[[1]]
  rdisplace <- rcluster[[2]]
  if(is.numeric(mu)) {
    ## homogeneous
    if(!(length(mu) == 1 && mu >= 0))
      stop("rcluster[[1]] should be a single nonnegative number")
    mumax <- mu
  } else if (is.im(mu) || is.function(mu)) {
      ## inhomogeneous
    if(is.function(mu)) mu <- as.im(mu, W=win, ..., strict=TRUE)
    mumax <- max(mu)
  } else stop("rcluster[[1]] should be a number, a function or a pixel image")  
  if(!is.function(rdisplace))
    stop("rcluster[[2]] should be a function")

  ## Generate parents in dilated window
  frame <- boundingbox(win)
  dilated <- grow.rectangle(frame, expand)
  if(is.im(kappa) && !is.subset.owin(dilated, as.owin(kappa)))
    stop(paste("The window in which the image",
               sQuote("kappa"),
               "is defined\n",
               "is not large enough to contain the dilation of the window",
               sQuote("win")))
  if(nonempty) {
    if(is.function(kappa)) {
      kappa <- as.im(kappa, W=dilated, ..., strict=TRUE)
      lmax <- NULL
    }
    ## intensity of parents with at least one offspring point
    kappa <- kappa * (1 - exp(-mumax))
  }
  ## generate
  parentlist <- rpoispp(kappa, lmax=lmax, win=dilated, nsim=nsim,
                        drop=FALSE, warnwin=FALSE)

  resultlist <- vector(mode="list", length=nsim)
  for(i in 1:nsim) {
    parents <- parentlist[[i]]
    
    np <- npoints(parents)
    ## generate cluster sizes
    if(np == 0) {
      ## no parents - empty pattern
      result <- ppp(numeric(0), numeric(0), window=win)
      parentid <- integer(0)
    } else {
      if(!nonempty) {
        ## cluster sizes are Poisson
        csize <- rpois(np, mumax)
      } else {
        ## cluster sizes are Poisson conditional on > 0
        csize <- qpois(runif(np, min=dpois(0, mumax)), mumax)
      }
      noff <- sum(csize)
      xparent <- parents$x
      yparent <- parents$y
      x0 <- rep.int(xparent, csize)
      y0 <- rep.int(yparent, csize)
      ## invoke random generator
      dd <- rdisplace(noff, ...)
      mm <- if(is.ppp(dd)) marks(dd) else NULL
      ## validate
      xy <- xy.coords(dd)
      dx <- xy$x
      dy <- xy$y
      if(!(length(dx) == noff))
        stop("rcluster returned the wrong number of points")
      ## create offspring and offspring-to-parent map
      xoff <- x0 + dx
      yoff <- y0 + dy
      parentid <- rep.int(1:np, csize)
      ## trim to window
      retain <- inside.owin(xoff, yoff, win)
      if(is.im(mu))
        retain[retain] <- inside.owin(xoff[retain], yoff[retain], as.owin(mu))
      xoff <- xoff[retain]
      yoff <- yoff[retain]
      parentid <- parentid[retain]
      if(!is.null(mm)) mm <- marksubset(mm, retain)
      ## done
      result <- ppp(xoff, yoff, window=win, check=FALSE, marks=mm)
    }

    if(is.im(mu)) {
      ## inhomogeneously modulated clusters a la Waagepetersen
      P <- eval.im(mu/mumax)
      result <- rthin(result, P)
    }

    if(saveparents) {
      attr(result, "parents") <- parents
      attr(result, "parentid") <- parentid
      attr(result, "expand") <- expand
    }
    
    resultlist[[i]] <- result
  }

  result <- simulationresult(resultlist, nsim, drop)
  return(result)
}  

rMatClust <- local({
  
  ## like runifdisc but returns only the coordinates
  rundisk <- function(n, radius) {
    R <- radius * sqrt(runif(n, min=0, max=1))
    Theta <- runif(n, min=0, max=2*pi)
    cbind(R * cos(Theta), R * sin(Theta))
  }

  rMatClust <- 
  function(kappa, scale, mu, win = owin(c(0,1),c(0,1)),
           nsim=1, drop=TRUE, saveLambda=FALSE, expand = scale, ...,
           poisthresh=1e-6, saveparents=TRUE) {
    ## Matern Cluster Process with Poisson (mu) offspring distribution
    ## Catch old scale syntax (r)
    if(missing(scale)) scale <- list(...)$r
    check.1.real(scale)
    stopifnot(scale > 0)

    ## trap case of large clusters, close to Poisson
    kok <- is.numeric(kappa) || is.im(kappa)
    if(kok) {
      kappamax <- max(kappa)
    } else {
      kim <- as.im(kappa, W=win, ..., strict=TRUE)
      kra <- range(kim)
      kappamax <- kra[2] + 0.05 * diff(kra)
    }
    if(1/(pi * kappamax * scale^2) < poisthresh) {
      kapmu <- mu * (if(kok) kappa else kim)
      result <- rpoispp(kapmu, win=win, nsim=nsim, drop=drop, warnwin=FALSE)
      return(result)
    }

    result <- rNeymanScott(kappa, scale, list(mu, rundisk), win, radius=scale,
                           nsim=nsim, drop=FALSE,
                           saveparents = saveparents || saveLambda)
    if(saveLambda){
      for(i in 1:nsim) {
        parents <- attr(result[[i]], "parents")
        Lambda <- clusterfield("MatClust", parents, scale=scale, mu=mu, ...)
        attr(result[[i]], "Lambda") <- Lambda[win, drop=FALSE]
      }
    }
    return(if(nsim == 1 && drop) result[[1]] else result)
  }

  rMatClust
})

                  
rThomas <- local({

  ## random displacements
  gaus <- function(n, sigma) {
    matrix(rnorm(2 * n, mean=0, sd=sigma), ncol=2)
  }

  ## main function
  rThomas <-
      function(kappa, scale, mu, win = owin(c(0,1),c(0,1)), nsim=1, drop=TRUE, 
               saveLambda=FALSE, expand = 4*scale, ...,
               poisthresh=1e-6, saveparents=TRUE) {
      ## Thomas process with Poisson(mu) number of offspring
      ## at isotropic Normal(0,sigma^2) displacements from parent
      ##
      ## Catch old scale syntax (sigma)
      if(missing(scale)) scale <- list(...)$sigma
      check.1.real(scale)
      stopifnot(scale > 0)

      ## trap case of large clusters, close to Poisson
      kok <- is.numeric(kappa) || is.im(kappa)
      if(kok) {
        kappamax <- max(kappa)
      } else {
        kim <- as.im(kappa, W=win, ..., strict=TRUE)
        kra <- range(kim)
        kappamax <- kra[2] + 0.05 * diff(kra)
      }
      if(1/(4*pi * kappamax * scale^2) < poisthresh) {
        kapmu <- mu * (if(kok) kappa else kim)
        result <- rpoispp(kapmu, win=win, nsim=nsim, drop=drop, warnwin=FALSE)
        return(result)
      }
      
      ## determine the maximum radius of clusters
      if(missing(expand))
          expand <- clusterradius("Thomas", scale = scale, ...)

      result <- rNeymanScott(kappa, expand, list(mu, gaus),
                             win, sigma=scale,
                             nsim=nsim, drop=FALSE,
                             saveparents = saveparents || saveLambda)  
      if(saveLambda){
        for(i in 1:nsim) {
          parents <- attr(result[[i]], "parents")
          Lambda <- clusterfield("Thomas", parents, scale=scale, mu=mu, ...)
          attr(result[[i]], "Lambda") <- Lambda[win, drop=FALSE]
        }
      }
      return(if(nsim == 1 && drop) result[[1]] else result)
    }

  rThomas
})


## ================================================
## Neyman-Scott process with Cauchy kernel function
## ================================================

## scale / omega: scale parameter of Cauchy kernel function
## eta: scale parameter of Cauchy pair correlation function
## eta = 2 * omega

rCauchy <- local({

  ## simulate mixture of normals with inverse-gamma distributed variance
  rnmix.invgam <- function(n = 1, rate) {
    V <- matrix(rnorm(2 * n, 0, 1), nrow = n, ncol = 2)
    s <- 1/rgamma(n, shape=1/2, rate=rate)
    return(sqrt(s) * V)
  }

  ## main function
  rCauchy <- function (kappa, scale, mu, win = owin(), thresh = 0.001,
                       nsim=1, drop=TRUE, saveLambda=FALSE, expand = NULL,
                       ..., poisthresh=1e-6, saveparents=TRUE) {
    ## scale / omega: scale parameter of Cauchy kernel function
    ## eta: scale parameter of Cauchy pair correlation function

    ## Catch old scale syntax (omega)
    dots <- list(...)
    if(missing(scale)) scale <- dots$omega
    
    ## Catch old name 'eps' for 'thresh':
    if(missing(thresh))
        thresh <- dots$eps %orifnull% 0.001

    ## trap case of large clusters, close to Poisson
    kok <- is.numeric(kappa) || is.im(kappa)
    if(kok) {
      kappamax <- max(kappa)
    } else {
      kim <- as.im(kappa, W=win, ..., strict=TRUE)
      kra <- range(kim)
      kappamax <- kra[2] + 0.05 * diff(kra)
    }
    if(1/(pi * kappamax * scale^2) < poisthresh) {
      kapmu <- mu * (if(kok) kappa else kim)
      result <- rpoispp(kapmu, win=win, nsim=nsim, drop=drop, warnwin=FALSE)
      return(result)
    }
    
    ## determine the maximum radius of clusters
    if(missing(expand)){
        expand <- clusterradius("Cauchy", scale = scale, thresh = thresh, ...)
    } else if(!missing(thresh)){
        warning("Argument ", sQuote("thresh"), " is ignored when ", sQuote("expand"), " is given")
    }

    ## simulate
    result <- rNeymanScott(kappa, expand,
                           list(mu, rnmix.invgam),
                           win, rate = scale^2/2, nsim=nsim, drop=FALSE,
                           saveparents = saveparents || saveLambda)
    ## correction from Abdollah: the rate is beta = omega^2 / 2 = eta^2 / 8.
    if(saveLambda){
      for(i in 1:nsim) {
        parents <- attr(result[[i]], "parents")
        Lambda <- clusterfield("Cauchy", parents, scale=scale, mu=mu, ...)
        attr(result[[i]], "Lambda") <- Lambda[win, drop=FALSE]
      }
    }
    return(if(nsim == 1 && drop) result[[1]] else result)
  }

  rCauchy })

##    
## =================================================================
## Neyman-Scott process with Variance Gamma (Bessel) kernel function
## =================================================================

## nu.ker: smoothness parameter of Variance Gamma kernel function
## omega: scale parameter of kernel function
## nu.pcf: smoothness parameter of Variance Gamma pair correlation function
## eta: scale parameter of Variance Gamma pair correlation function
## nu.pcf = 2 * nu.ker + 1    and    eta = omega

rVarGamma <- local({
  
  ## simulates mixture of isotropic Normal points in 2D with gamma variances
  rnmix.gamma <- function(n = 1, shape, rate) {
    V <- matrix(rnorm(2 * n, 0, 1), nrow = n, ncol = 2)
    s <- rgamma(n, shape=shape, rate=rate)
    return(sqrt(s) * V)
  }

  ## main function
  rVarGamma <- function (kappa, nu, scale, mu, win = owin(),
                         thresh = 0.001, nsim=1, drop=TRUE, saveLambda=FALSE,
                         expand = NULL, ..., poisthresh=1e-6,
                         saveparents=TRUE) {
    ## nu / nu.ker: smoothness parameter of Variance Gamma kernel function
    ## scale / omega: scale parameter of kernel function
    ## Catch old nu.ker/nu.pcf syntax and resolve nu-value.
    dots <- list(...)
    if(missing(nu)){
        nu <- resolve.vargamma.shape(nu.ker=dots$nu.ker, nu.pcf=dots$nu.pcf)$nu.ker
    } else{
        check.1.real(nu)
        stopifnot(nu > -1/2)
    }
    ## Catch old scale syntax (omega)
    if(missing(scale)) scale <- dots$omega
    
    ## Catch old name 'eps' for 'thresh':
    if(missthresh <- missing(thresh))
        thresh <- dots$eps %orifnull% 0.001

    ## trap case of large clusters, close to Poisson
    kok <- is.numeric(kappa) || is.im(kappa)
    if(kok) {
      kappamax <- max(kappa)
    } else {
      kim <- as.im(kappa, W=win, ..., strict=TRUE)
      kra <- range(kim)
      kappamax <- kra[2] + 0.05 * diff(kra)
    }
    if(1/(4 * pi * kappamax * scale^2) < poisthresh) {
      kapmu <- mu * (if(kok) kappa else kim)
      result <- rpoispp(kapmu, win=win, nsim=nsim, drop=drop, warnwin=FALSE)
      return(result)
    }
    
     ## determine the maximum radius of clusters
    if(missing(expand)){
        expand <- clusterradius("VarGamma", scale = scale, nu = nu,
                             thresh = thresh, ...)
    } else if(!missthresh){
        warning("Argument ", sQuote("thresh"), " is ignored when ", sQuote("expand"), " is given")
    }

    ## simulate
    result <- rNeymanScott(kappa, expand,
                           list(mu, rnmix.gamma), win,
##                          WAS:  shape = 2 * (nu.ker + 1)
                           shape = nu + 1,
                           rate = 1/(2 * scale^2),
                           nsim=nsim, drop=FALSE,
                           saveparents = saveparents || saveLambda)
    if(saveLambda){
      for(i in 1:nsim) {
        parents <- attr(result[[i]], "parents")
        Lambda <- clusterfield("VarGamma", parents, scale=scale,
                               nu=nu, mu=mu, ...)
        attr(result[[i]], "Lambda") <- Lambda[win, drop=FALSE]
      }
    }
    return(if(nsim == 1 && drop) result[[1]] else result)
  }

  rVarGamma
})

##
## persp.im.R
##
##  'persp' method for image objects
##      plus annotation
##  
##  $Revision: 1.20 $ $Date: 2016/09/01 05:49:42 $
##

persp.im <- local({

  persp.im <- function(x, ...,
                       colmap=NULL, colin=x, apron=FALSE,
                       visible=FALSE) {
    xname <- deparse(substitute(x))
    xinfo <- summary(x)
    if(xinfo$type == "factor")
      stop("Perspective plot is inappropriate for factor-valued image")
    ## check whether 'col' was specified when 'colmap' was intended
    Col <- list(...)$col
    if(is.null(colmap) && !is.null(Col) && !is.matrix(Col) && length(Col) != 1)
      warning("Argument col is not a matrix. Did you mean colmap?")
    if(!missing(colin)) {
      ## separate image to determine colours
      verifyclass(colin, "im")
      if(!compatible(colin, x)) {
        ## resample 'colin' onto grid of 'x'
        colin <- as.im(colin, W=x)
      }
      if(is.null(colmap))
        colmap <- spatstat.options("image.colfun")(128)
    }
    pop <- spatstat.options("par.persp")
    ##
    if(is.function(colmap) && !inherits(colmap, "colourmap")) {
      ## coerce to a 'colourmap' if possible
      clim <- range(colin, finite=TRUE)
      if(names(formals(colmap))[1] == "n") {
        colval <- colmap(128)
        colmap <- colourmap(colval, range=clim)
      } else {
        ## colour map determined by a rule (e.g. 'beachcolours')
        colmap <- invokeColourmapRule(colmap, colin,
                                      zlim=clim, colargs=list(...))
        if(is.null(colmap))
          stop("Unrecognised syntax for colour function")
      }
    }
    ## colour map?
    if(is.null(colmap)) {
      colinfo <- list(col=NULL)
    } else if(inherits(colmap, "colourmap")) {
      ## colour map object
      ## apply colour function to image data
      colval <- eval.im(colmap(colin))
      colval <- t(as.matrix(colval))
      ## strip one row and column for input to persp.default
      colval <- colval[-1, -1]
      ## replace NA by arbitrary value
      isna <- is.na(colval)
      if(any(isna)) {
        stuff <- attr(colmap, "stuff")
        colvalues <- stuff$outputs
        colval[isna] <- colvalues[1]
      }
      ## pass colour matrix (and suppress lines)
      colinfo <- list(col=colval, border=NA)
    } else {
      ## interpret 'colmap' as colour map
      if(is.list(colmap) && all(c("breaks", "col") %in% names(colmap))) {
        breaks <- colmap$breaks
        colvalues <- colmap$col
      } else if(is.vector(colmap)) {
        colvalues <- colmap
        breaks <- quantile(colin,
                           seq(from=0,to=1,length.out=length(colvalues)+1))
        if(!all(ok <- !duplicated(breaks))) {
          breaks <- breaks[ok]
          colvalues <- colvalues[ok[-1]]
        }
      } else warning("Unrecognised format for colour map")
      ## apply colour map to image values
      colid <- cut.im(colin, breaks=breaks, include.lowest=TRUE)
      colval <- eval.im(colvalues[unclass(colid)])
      colval <- t(as.matrix(colval))
#      nr <- nrow(colval)
#      nc <- ncol(colval)
      ## strip one row and column for input to persp.default
      colval <- colval[-1, -1]
      colval[is.na(colval)] <- colvalues[1]
      ## pass colour matrix (and suppress lines)
      colinfo <- list(col=colval, border=NA)
    }

    if(apron) {
      ## add an 'apron'
      zlim <- list(...)$zlim
      bottom <- if(!is.null(zlim)) zlim[1] else min(x)
      x <- na.handle.im(x, na.replace=bottom)
      x <- padimage(x, bottom)
      xinfo <- summary(x)
      if(is.matrix(colval <- colinfo$col)) {
        colval <- matrix(col2hex(colval), nrow(colval), ncol(colval))
        grijs <- col2hex("lightgrey")
        colval <- cbind(grijs, rbind(grijs, colval, grijs), grijs)
        colinfo$col <- colval
      }
    }

    if(spatstat.options("monochrome"))
      colinfo$col <- to.grey(colinfo$col)
  
    ## get reasonable z scale while fixing x:y aspect ratio
    if(xinfo$type %in% c("integer", "real")) {
      zrange <- xinfo$range
      if(diff(zrange) > 0) {
        xbox <- as.rectangle(x)
        zscale <- 0.5 * mean(diff(xbox$xrange), diff(xbox$yrange))/diff(zrange)
        zlim <- zrange
      } else {
        zscale <- NULL
        mx <- xinfo$mean
        zlim <- mx + c(-1,1) * if(mx == 0) 0.1 else min(abs(mx), 1)
      }
    } else 
      zscale <- zlim <- NULL

    dotargs <- list(...)
    if(spatstat.options("monochrome"))
      dotargs <- col.args.to.grey(dotargs)
    
    yargh <- resolve.defaults(list(x=x$xcol, y=x$yrow, z=t(x$v)),
                              dotargs,
                              pop,
                              colinfo,
                              list(xlab="x", ylab="y", zlab=xname),
                              list(scale=FALSE, expand=zscale,
                                   zlim=zlim),
                              list(main=xname),
                              .StripNull=TRUE)

    jawab <- do.call.matched(persp, yargh, 
                             funargs=graphicsPars("persp"))

    attr(jawab, "expand") <- yargh$expand
    
    if(visible)
      attr(jawab, "visible") <- perspvis(x, M=jawab)
    
    return(invisible(jawab))
  }

  diffit <- function(x) {
    y <- diff(x)
    return(c(y[1], y))
  }
  
  perspvis <- function(X, ..., M=NULL) {
    stopifnot(is.im(X))
    ## determine perspective matrix
    if(is.null(M))
      M <- persp(X, ...)
    ## project the coordinates
    ## onto (x,y) plane of plot and z axis pointing out of it
    xy <- rasterxy.im(X, drop=TRUE)
    z <- X[drop=TRUE]
    xyz <- cbind(xy, z)
    v <- cbind(xyz, 1) %*% M
    pxyz <- v[,1:3]/v[,4]
    px <- pxyz[,1]
    py <- pxyz[,2]
    pz <- pxyz[,3]
    ## determine greatest possible difference in 'depth' in one pixel step
    PZ <- as.matrix(X)
    ok <- !is.na(PZ)
    PZ[ok] <- pz
    maxslip <- max(0, abs(apply(PZ, 1, diff)),
                      abs(apply(PZ, 2, diff)), na.rm=TRUE)
    ## determine which pixels are in front
    d <- ceiling(dim(X)/2)
    jx <- cut(px, breaks=d[2])
    iy <- cut(py, breaks=d[1])
    zmax <- tapply(pz, list(iy,jx), max)
    isvis <- infront <- (pz > zmax[cbind(iy,jx)] - 2 * maxslip)
    ##
    if(TRUE) {
      ## Additionally check whether unit normal to surface is pointing to viewer
      Xmat <- as.matrix(X)
      dzdx <- cbind(0, t(apply(Xmat, 1, diff)))/X$xstep
      dzdy <- rbind(0, apply(Xmat, 2, diff))/X$ystep
      dzdx <- as.vector(dzdx[ok])
      dzdy <- as.vector(dzdy[ok])
      ## unscaled normal is (-dzdx, -dzdy, 1)
      if(FALSE) {
        ## THIS DOESN'T WORK - not sure why.
        ## rescale so that length is half diameter of pixel
        fac <- sqrt(X$xstep^2 + X$ystep^2)/(2 * sqrt(dzdx^2+dzdy^2+1))
        ## add to spatial coordinates
        xyzplus <- xyz + fac * cbind(-dzdx, -dzdy, 1)
        ## transform
        vplus <- cbind(xyzplus, 1) %*% M
        pplus <- vplus[,1:3]/vplus[,4]
        ## determine whether normal is pointing toward viewer
        deltaz <- pplus[,3] - pz
        isvis <- infront & (deltaz > 0)
      } else {
        theta <- atan2(M[2,1],M[1,1]) + pi/2
        phi <-  - atan2(M[3,3], M[3,2])
        ## check agreement
        ## cat(paste("Guess: theta=", theta * 180/pi, "\n"))
        ## cat(paste("Guess: phi=", phi * 180/pi, "\n"))
        ## view vector
        viewer <- cos(phi) * c(cos(theta), sin(theta), 0)
                       + c(0, 0, sin(phi))
        ## inner product
        dotprod <- -dzdx * viewer[1] - dzdy * viewer[2] + viewer[3]
        isvis <- infront & (dotprod < 0)
      }
    }
    ## put into image
    Y <- eval.im(X > 0)
    Y[] <- isvis
    ## replace 'NA' by 'FALSE'
    if(anyNA(Y))
      Y <- as.im(Y, na.replace=FALSE)
    return(Y)
  }

  persp.im
})


perspPoints <- function(x, y=NULL, ..., Z, M) {
  xy <- xy.coords(x, y)
  stopifnot(is.im(Z))
  X <- as.ppp(xy, W=Frame(Z))
  if(!(is.matrix(M) && all(dim(M) == 4)))
    stop("M should be a 4 x 4 matrix, returned from persp()")
  V <- attr(M, "visible")
  if(is.null(V)) {
    warning(paste("M does not contain visibility information;",
               "it should be recomputed by persp() with visible=TRUE"))
  } else {
    ## restrict to visible points
    VX <- V[X, drop=FALSE]
    VX[is.na(VX)] <- FALSE
    X <- X[VX]
  }
  #' determine heights
  ZX <- Z[X, drop=FALSE] # may contain NA
  #' transform and plot
  points(trans3d(X$x, X$y, ZX, M), ...)
}

perspSegments <- local({
  perspSegments <- function(x0, y0=NULL, x1=NULL, y1=NULL, ..., Z, M) {
    stopifnot(is.im(Z))
    if(!(is.matrix(M) && all(dim(M) == 4)))
      stop("M should be a 4 x 4 matrix, returned from persp()")
    V <- attr(M, "visible")
    if(is.null(V))
      warning(paste("M does not contain visibility information;",
                 "it should be recomputed by persp() with visible=TRUE"))
    
    if(is.psp(X <- x0) && is.null(y0) && is.null(x1) && is.null(y1)) {
      eX <- X$ends
#      nX <- nrow(eX)
    } else {
#      nX <- length(x0)
      check.nvector(x0, naok=TRUE)
      check.nvector(y0, naok=TRUE)
      check.nvector(x1, naok=TRUE)
      check.nvector(y1, naok=TRUE)
      eX <- cbind(x0, y0, x1, y1)
    }
    if(is.null(V)) {
      Y <- eX
    } else {
      ## chop segments to length of single pixel
      eps <- with(Z, min(xstep,ystep))
      Y <- do.call(rbind, lapply(as.data.frame(t(eX)), chopsegment, eps=eps))
      ## determine which segments are visible
      yleft  <- list(x=Y[,1], y=Y[,2])
      yright <- list(x=Y[,3], y=Y[,4])
      ok <- V[yleft, drop=FALSE] & V[yright, drop=FALSE]
      ok[is.na(ok)] <- FALSE
      Y <- Y[ok, ,drop=FALSE]
    }
    if(nrow(Y) == 0) return(invisible(NULL))
    ## map to projected plane
    x0y0 <- trans3d(Y[,1], Y[,2], Z[list(x=Y[,1],y=Y[,2]), drop=FALSE], M)
    x1y1 <- trans3d(Y[,3], Y[,4], Z[list(x=Y[,3],y=Y[,4]), drop=FALSE], M)
    segments(x0y0$x, x0y0$y, x1y1$x, x1y1$y, ...)
  }

  chopsegment <- function(x, eps) {
    len2 <- (x[3] - x[1])^2 + (x[4] - x[2])^2
    if(len2 <= eps^2) return(x)
    n <- ceiling(sqrt(len2)/eps)
    b <- (1:n)/n
    a <- (0:(n-1))/n
    return(cbind(x[1] + a * (x[3]-x[1]),
                 x[2] + a * (x[4]-x[2]),
                 x[1] + b * (x[3]-x[1]),
                 x[2] + b * (x[4]-x[2])))
  }
      
  perspSegments
})

perspLines <- function(x, y=NULL, ..., Z, M) {
  xy <- xy.coords(x, y)
  n <- length(xy$x)
  perspSegments(x[-n], y[-n], x[-1], y[-1], Z=Z, M=M, ...)
}

perspContour <- function(Z, M, ...,
                         nlevels=10, levels=pretty(range(Z), nlevels)) {
  cl <- contourLines(x=Z$xcol,
                     y=Z$yrow,
                     z=t(Z$v),
                     nlevels=nlevels, levels=levels)
  for(i in seq_along(cl)) {
    cli <- cl[[i]]
    perspLines(cli$x, cli$y, ..., Z=Z, M=M)
  }
  invisible(NULL)
}

#
#
#      distmap.R
#
#      $Revision: 1.23 $     $Date: 2017/06/05 10:31:58 $
#
#
#     Distance transforms
#
#
distmap <- function(X, ...) {
  UseMethod("distmap")
}

distmap.ppp <- function(X, ...) {
  verifyclass(X, "ppp")
  e <- exactdt(X, ...)
  W <- e$w
  uni <- unitname(W)
  dmat <- e$d
  imat <- e$i
  V <- im(dmat, W$xcol, W$yrow, unitname=uni)
  I <- im(imat, W$xcol, W$yrow, unitname=uni)
  if(X$window$type == "rectangle") {
    # distance to frame boundary
    bmat <- e$b
    B <- im(bmat, W$xcol, W$yrow, unitname=uni)
  } else {
    # distance to window boundary, not frame boundary
    bmat <- bdist.pixels(W, style="matrix")
    B <- im(bmat, W$xcol, W$yrow, unitname=uni)
    # clip all to window
    V <- V[W, drop=FALSE]
    I <- I[W, drop=FALSE]
    B <- B[W, drop=FALSE]
  }
  attr(V, "index") <- I
  attr(V, "bdry")  <- B
  return(V)
}

distmap.owin <- function(X, ..., discretise=FALSE, invert=FALSE) {
  verifyclass(X, "owin")
  uni <- unitname(X)
  if(X$type == "rectangle") {
    M <- as.mask(X, ...)
    Bdry <- im(bdist.pixels(M, style="matrix"),
               M$xcol, M$yrow, unitname=uni)
    if(!invert)
      Dist <- as.im(M, value=0)
    else 
      Dist <- Bdry
  } else if(X$type == "polygonal" && !discretise) {
    Edges <- edges(X)
    Dist <- distmap(Edges, ...)
    Bdry <- attr(Dist, "bdry")
    if(!invert) 
      Dist[X] <- 0
    else {
      bb <- as.rectangle(X)
      bigbox <- grow.rectangle(bb, diameter(bb)/4)
      Dist[complement.owin(X, bigbox)] <- 0
    }
  } else {
    X <- as.mask(X, ...)
    if(invert)
      X <- complement.owin(X)
    xc <- X$xcol
    yr <- X$yrow
    nr <- X$dim[1L]
    nc <- X$dim[2L]
# pad out the input image with a margin of width 1 on all sides
    mat <- X$m
    pad <- invert # boundary condition is opposite of value inside W
    mat <- cbind(pad, mat, pad)
    mat <- rbind(pad, mat, pad)
# call C routine
    res <- .C("distmapbin",
              xmin=as.double(X$xrange[1L]),
              ymin=as.double(X$yrange[1L]),
              xmax=as.double(X$xrange[2L]),
              ymax=as.double(X$yrange[2L]),
              nr = as.integer(nr),
              nc = as.integer(nc),
              inp = as.integer(as.logical(t(mat))),
              distances = as.double(matrix(0, ncol = nc + 2, nrow = nr + 2)),
              boundary = as.double(matrix(0, ncol = nc + 2, nrow = nr + 2)),
              PACKAGE = "spatstat")
  # strip off margins again
    dist <- matrix(res$distances,
                   ncol = nc + 2, byrow = TRUE)[2:(nr + 1), 2:(nc +1)]
    bdist <- matrix(res$boundary,
                    ncol = nc + 2, byrow = TRUE)[2:(nr + 1), 2:(nc +1)]
  # cast as image objects
    Dist <- im(dist,  xc, yr, unitname=uni)
    Bdry <- im(bdist, xc, yr, unitname=uni)
  }
  attr(Dist, "bdry")  <- Bdry
  return(Dist)
}

distmap.psp <- function(X, ...) {
  verifyclass(X, "psp")
  W <- as.mask(Window(X), ...)
  uni <- unitname(W)
  rxy <- rasterxy.mask(W)
  xp <- rxy$x
  yp <- rxy$y
  E <- X$ends
  big <- 2 * diameter(Frame(W))^2
  z <- NNdist2segments(xp, yp, E$x0, E$y0, E$x1, E$y1, big)
  xc <- W$xcol
  yr <- W$yrow
  Dist <- im(array(sqrt(z$dist2), dim=W$dim), xc, yr, unitname=uni)
  Indx <- im(array(z$index, dim=W$dim), xc, yr, unitname=uni)
  Bdry <- im(bdist.pixels(W, style="matrix"), xc, yr, unitname=uni)
  attr(Dist, "index") <- Indx
  attr(Dist, "bdry")  <- Bdry
  return(Dist)
}

##
## subset.R
##
## Methods for 'subset'
##
##   $Revision: 1.6 $  $Date: 2018/10/02 03:50:44 $

subset.ppp <- function(x, subset, select, drop=FALSE, ...) {
  stopifnot(is.ppp(x))
  w <- as.owin(x)
  y <- as.data.frame(x)
  r <- if (missing(subset)) {
    rep_len(TRUE, nrow(y))
  } else {
    e <- substitute(subset)
    r <- eval(e, y, parent.frame())
    if(!is.logical(r))
      r <- ppsubset(x, r, "subset", fatal=TRUE)
    r & !is.na(r)
  }
  vars <- if (missing(select)) {
    TRUE
  } else {
    ## create an environment in which column names are mapped to their positions
    nl <- as.list(seq_along(y))
    names(nl) <- names(y)
    if(length(nl) > 3) {
      ## multiple columns of marks: add the name 'marks'
      nl <- append(nl, list(marks=3:length(nl)))
    }
    eval(substitute(select), nl, parent.frame())
  }
  ## ensure columns include coordinates
  nama <- names(y)
  names(nama) <- nama
  vars <- union(c("x", "y"), nama[vars])
  ## take subset
  z <- y[r, vars, drop = FALSE]
  ## reinstate as point pattern
  out <- as.ppp(z, W=w, check=FALSE)
  if(drop)
    out <- out[drop=TRUE]
  return(out)
}

subset.pp3 <- subset.lpp <- subset.ppx <- function(x, subset, select, drop=FALSE, ...) {
  y <- as.data.frame(x)
  r <- if (missing(subset)) 
    rep_len(TRUE, nrow(y))
  else {
    e <- substitute(subset)
    r <- eval(e, y, parent.frame())
    if(!is.logical(r))
      r <- ppsubset(x, r, "subset", fatal=TRUE)
    r & !is.na(r)
  }
  vars <- if (missing(select)) 
    TRUE
  else {
    ## create an environment in which column names are mapped to their positions
    nl <- as.list(seq_along(y))
    names(nl) <- names(y)
    if(!("marks" %in% names(y)) && any(ismark <- (x$ctype == "mark"))) {
      ## add the symbol 'marks' 
      nl <- append(nl, list(marks=which(ismark)))
    }
    eval(substitute(select), nl, parent.frame())
  }
  ## ensure columns include coordinates 
  nama <- names(y)
  names(nama) <- nama
  vars <- union(names(coords(x)), nama[vars])
  ## take subset
  z <- y[r, vars, drop = FALSE]
  ## reinstate as point pattern
  ctype <- as.character(x$ctype)[match(vars, nama)]
  out <- ppx(z, domain=x$domain, coord.type=ctype)
  ## drop unused factor levels
  if(drop)
    out <- out[drop=TRUE]
  ## reinstate class
  class(out) <- class(x)
  return(out)
}

subset.psp <- function(x, subset, select, drop=FALSE, ...) {
  stopifnot(is.psp(x))
  w <- Window(x)
  y <- as.data.frame(x)
  r <- if (missing(subset)) {
    rep_len(TRUE, nrow(y))
  } else {
    e <- substitute(subset)
    r <- eval(e, y, parent.frame())
    if(!is.logical(r))
      stop("Argument 'subset' should be a logical vector", call.=FALSE)
    r & !is.na(r)
  }
  vars <- if (missing(select)) {
    TRUE
  } else {
    ## create an environment in which column names are mapped to their positions
    nl <- as.list(seq_along(y))
    names(nl) <- names(y)
    if(length(nl) > 3) {
      ## multiple columns of marks: add the name 'marks'
      nl <- append(nl, list(marks=3:length(nl)))
    }
    eval(substitute(select), nl, parent.frame())
  }
  ## ensure columns include coordinates
  nama <- names(y)
  names(nama) <- nama
  vars <- union(c("x0", "y0", "x1", "y1"), nama[vars])
  ## take subset
  z <- y[r, vars, drop = FALSE]
  ## reinstate as line segment pattern
  out <- as.psp(z, window=w, check=FALSE)
  if(drop)
    out <- out[drop=TRUE]
  return(out)
}

#'
#' deldir.R
#'
#' Interface to deldir package
#'
#'  $Revision: 1.32 $ $Date: 2020/01/10 06:54:35 $
#'

#' ..............................................
#' Internal options 
#'      deldir suggests spatstat (!!!)
#'      so we must save options here, not in spatstat.options
  .spst.triEnv <- new.env()
  assign("use.trigraf",  TRUE, envir=.spst.triEnv)
  assign("use.trigrafS", TRUE, envir=.spst.triEnv)
  assign("debug.delaunay", FALSE, envir=.spst.triEnv)
#' for testing purposes only
  spatstat.deldir.setopt <- function(use.trigrafS=TRUE,
                                     use.trigraf=TRUE,
                                     debug.delaunay=FALSE) {
    assign("use.trigrafS", use.trigrafS, envir=.spst.triEnv)
    assign("use.trigraf",  use.trigraf, envir=.spst.triEnv)
    assign("debug.delaunay", debug.delaunay, envir=.spst.triEnv)
    return(invisible(NULL))
  }
#'..............................................


dirichlet <- local({

  dirichlet <- function(X) {
    stopifnot(is.ppp(X))
    X <- unique(X, rule="deldir", warn=TRUE)
    nX <- npoints(X)
    w <- X$window
    if(nX == 0) return(NULL)
    if(nX == 1) return(as.tess(w))
    dd <- safedeldir(X)
    if(is.null(dd)) return(NULL)
    pp <- lapply(tile.list(dd), df2poly)
    if(length(pp) == npoints(X))
      names(pp) <- seq_len(npoints(X))
    dir <- tess(tiles=pp, window=as.rectangle(w))
    if(w$type != "rectangle")
      dir <- intersect.tess(dir, w)
    return(dir)
  }

  df2poly <- function(z) { owin(poly=z[c("x","y")]) }
  
  dirichlet
})

delaunay <- function(X) {
  stopifnot(is.ppp(X))
  X <- unique(X, rule="deldir", warn=TRUE)
  nX <- npoints(X)
  if(nX < 3) return(NULL)
  w <- X$window
  dd <- safedeldir(X)
  if(is.null(dd)) return(NULL)
  a <- dd$delsgs[,5L]
  b <- dd$delsgs[,6L]
  use.trigraf  <- get("use.trigraf", envir=.spst.triEnv)
  use.trigrafS <- get("use.trigrafS", envir=.spst.triEnv)
  debug.delaunay <- get("debug.delaunay", envir=.spst.triEnv)
  if(use.trigrafS) {
    # first ensure a[] < b[]
    swap <- (a > b)
    if(any(swap)) {
      oldb <- b
      b[swap] <- a[swap]
      a[swap] <- oldb[swap]
    }
    # next ensure a is sorted
    o <- order(a, b)
    a <- a[o]
    b <- b[o]
    # 
    nv <- nX
    ne <- length(a)
    ntmax <- ne
    z <- .C("trigrafS",
            nv = as.integer(nv),
            ne = as.integer(ne),
            ie = as.integer(a),
            je = as.integer(b),
            ntmax = as.integer(ntmax),
            nt = as.integer(integer(1L)),
            it = as.integer(integer(ne)),
            jt = as.integer(integer(ne)),
            kt = as.integer(integer(ne)),
            status = as.integer(integer(1L)),
            PACKAGE = "spatstat")
    if(z$status != 0)
      stop("Internal error: overflow in trigrafS")
    tlist <- with(z, cbind(it, jt, kt)[1:nt, ])
  } else if(use.trigraf) {
    nv <- nX
    ne <- length(a)
    ntmax <- ne
    z <- .C("trigraf",
            nv = as.integer(nv),
            ne = as.integer(ne),
            ie = as.integer(a),
            je = as.integer(b),
            ntmax = as.integer(ntmax),
            nt = as.integer(integer(1L)),
            it = as.integer(integer(ntmax)),
            jt = as.integer(integer(ntmax)),
            kt = as.integer(integer(ntmax)),
            status = as.integer(integer(1L)),
            PACKAGE = "spatstat")
    if(z$status != 0)
      stop("Internal error: overflow in trigraf")
    tlist <- with(z, cbind(it, jt, kt)[1:nt, ])
  } else {
    tlist <- matrix(integer(0), 0, 3)
    for(i in seq_len(nX)) {
      # find all Delaunay neighbours of i 
      jj <- c(b[a==i], a[b==i])
      jj <- sortunique(jj)
      # select those with a higher index than i
      jj <- jj[jj > i]
      # find pairs of neighbours which are Delaunay neighbours
      # (thus, triangles where the first numbered vertex is i)
      if(length(jj) > 0) 
        for(j in jj) {
          kk <- c(b[a == j], a[b == j])
          kk <- kk[(kk %in% jj) & (kk > j)]
          if(length(kk) > 0)
            for(k in kk) 
              # add (i,j,k) to list of triangles (i < j < k)
              tlist <- rbind(tlist, c(i, j, k))
        }
    }
  }
  # At this point, `tlist' contains all triangles formed by the Delaunay edges,
  # with vertices given in ascending order i < j < k in the 3 columns of tlist.
  # Some of these triangles may not belong to the Delaunay triangulation.
  # They will be weeded out later.
  
  # Assemble coordinates of triangles
  x <- X$x
  y <- X$y
  xtri <- matrix(x[tlist], nrow(tlist), 3L)
  ytri <- matrix(y[tlist], nrow(tlist), 3L)
  # ensure triangle vertices are in anticlockwise order
  ztri <- ytri - min(y)
  dx <- cbind(xtri[,2L]-xtri[,1L], xtri[,3L]-xtri[,2L], xtri[,1L]-xtri[,3L])
  zm <- cbind(ztri[,1L]+ztri[,2L], ztri[,2L]+ztri[,3L], ztri[,3L]+ztri[,1L])
  negareas <- apply(dx * zm, 1L, sum)
  clockwise <- (negareas > 0)
  #
  if(any(clockwise)) {
    xc <- xtri[clockwise, , drop=FALSE]
    yc <- ytri[clockwise, , drop=FALSE]
    tc <- tlist[clockwise, , drop=FALSE]
    xtri[clockwise,]  <- xc[,c(1L,3L,2L)]
    ytri[clockwise,]  <- yc[,c(1L,3L,2L)]
    tlist[clockwise,] <- tc[, c(1L,3L,2L)]
  }
  # At this point, triangle vertices are listed in anticlockwise order.
  # The same directed edge (i, j) cannot appear twice.
  # To weed out invalid triangles, check for such duplication
  triedges <- rbind(tlist[, c(1L,2L)],
                    tlist[, c(2L,3L)],
                    tlist[, c(3L,1L)])
  if(any(bad <- duplicated(triedges))) {
    badedges <- unique(triedges[bad, , drop=FALSE])
    ntri <- nrow(tlist)
    triid <- rep.int(seq_len(ntri), 3)
    illegal <- rep.int(FALSE, ntri)
    for(j in seq_len(nrow(badedges))) {
      from <- badedges[j, 1L]
      to   <- badedges[j, 2L]
      if(debug.delaunay)
        cat(paste("Suspect edge from vertex", from, "to vertex", to, "\n"))
      # find all triangles sharing this edge in this orientation
      sustri <- triid[(triedges[,1L] == from) & (triedges[,2L] == to)]
      if(debug.delaunay)
        cat(paste("\tInvestigating triangles", commasep(sustri), "\n"))
      # list all vertices associated with the suspect triangles
      susvert <- sortunique(as.vector(tlist[sustri, ]))
      if(debug.delaunay)
        cat(paste("\tInvestigating vertices", commasep(susvert), "\n"))
      xsusvert <- x[susvert]
      ysusvert <- y[susvert]
      # take each triangle in turn and check whether it contains a data point
      for(k in sustri) {
        if(!illegal[k] &&
           any(inside.triangle(xsusvert, ysusvert, xtri[k,], ytri[k,]))) {
          if(debug.delaunay)
            cat(paste("Triangle", k, "is illegal\n"))
          illegal[k] <- TRUE
        }
      }
    }
    if(!any(illegal)) {
      if(debug.delaunay)
        cat("No illegal triangles found\n")
    } else {
      if(debug.delaunay)
        cat(paste("Removing", sum(illegal), "triangles\n"))
      tlist <- tlist[!illegal, , drop=FALSE]
      xtri  <- xtri[!illegal, , drop=FALSE]
      ytri  <- ytri[!illegal, , drop=FALSE]
    }
  }
  # make tile list
  tiles <- list()
  for(m in seq_len(nrow(tlist))) {
    p <- list(x=xtri[m,], y=ytri[m,])
    tiles[[m]] <- owin(poly=p, check=FALSE)
  }

  wc <- convexhull.xy(x, y)
  del <- tess(tiles=tiles, window=wc)
  if(w$type != "rectangle")
    del <- intersect.tess(del, w)
  return(del)
}

delaunayDistance <- function(X) {
  stopifnot(is.ppp(X))
  nX <- npoints(X)
  w <- as.owin(X)
  ok <- !duplicated(X, rule="deldir")
  Y <- X[ok] 
  nY <- npoints(Y)
  if(nY < 3) 
    return(matrix(Inf, nX, nX))
  dd <- deldir(Y$x, Y$y, rw=c(w$xrange,w$yrange))
  if(is.null(dd)) return(NULL)
  joins <- as.matrix(dd$delsgs[,5:6])
  joins <- rbind(joins, joins[,2:1])
  d <- matrix(-1L, nY, nY)
  diag(d) <- 0
  d[joins] <- 1
  adj <- matrix(FALSE, nY, nY)
  diag(adj) <- TRUE
  adj[joins] <- TRUE
  z <- .C("Idist2dpath",
          nv = as.integer(nY),
          d = as.integer(d), 
          adj = as.integer(adj),
          dpath = as.integer(integer(nY * nY)),
          tol = as.integer(0),
          niter = as.integer(integer(1L)), 
          status = as.integer(integer(1L)),
          PACKAGE = "spatstat")
  if (z$status == -1L)
    warning(paste("graph connectivity algorithm did not converge after", 
                  z$niter, "iterations", "on", nY, "vertices and", 
                  sum(adj) - nY, "edges"))
  dpathY <- matrix(z$dpath, nY, nY)
  if(all(ok)) {
    dpathX <- dpathY
  } else {
    dpathX <- matrix(NA_integer_, nX, nX)
    dpathX[ok, ok] <- dpathY
  }
  return(dpathX)
}

safedeldir <- function(X) {
  rw <- with(X$window, c(xrange,yrange))
  dd <- try(deldir(X$x, X$y, rw=rw))
  if(!inherits(dd, "try-error") && inherits(dd, "deldir"))
    return(dd)
  warning("deldir failed; re-trying with slight perturbation of coordinates.",
          call.=FALSE)
  Y <- rjitter(X, mean(nndist(X))/100)
  dd <- try(deldir(Y$x, Y$y, rw=rw))
  if(!inherits(dd, "try-error") && inherits(dd, "deldir"))
    return(dd)
  warning("deldir failed even after perturbation of coordinates.", call.=FALSE)
  return(NULL)
}

dirichletVertices <- function(X) {
  DT <- tiles(dirichlet(X))
  xy <- do.call(concatxy, lapply(DT, vertices))
  Y <- unique(ppp(xy$x, xy$y, window=Window(X), check=FALSE))
  b <- bdist.points(Y)
  thresh <- diameter(Frame(X))/1000
  Y <- Y[b > thresh]
  return(Y)
}

dirichletAreas <- function(X) {
  stopifnot(is.ppp(X))
  X <- unmark(X)
  win <- Window(X)
  dup <- duplicated(X, rule="deldir")
  if((anydup <- any(dup))) {
    oldX <- X
    X <- X[!dup]
  }
  switch(win$type,
         rectangle = {
           rw <- c(win$xrange, win$yrange)
           dd <- deldir(X$x, X$y, dpl=NULL, rw=rw)
           w <- dd$summary[, 'dir.area']
         },
         polygonal = {
           w <- tile.areas(dirichlet(X))
         },
         mask = {
           #' Nearest data point to each pixel:
           tileid <- exactdt(X)$i
           #' Restrict to window (result is a vector - OK)
           tileid <- tileid[win$m]
           #' Count pixels in each tile
           id <- factor(tileid, levels=seq_len(X$n))
           counts <- table(id)
           #' Convert to digital area
           pixelarea <- win$xstep * win$ystep
           w <- pixelarea * as.numeric(counts)
         })
  if(!anydup)
    return(w)
  oldw <- numeric(npoints(oldX))
  oldw[!dup] <- w
  return(oldw)
}

delaunayNetwork <- function(X) {
  stopifnot(is.ppp(X))
  X <- unique(X, rule="deldir")
  nX <- npoints(X)
  if(nX == 0) return(NULL)
  if(nX == 1L) return(linnet(X, !diag(TRUE)))
  if(nX == 2L) return(linnet(X, !diag(c(TRUE,TRUE))))
  dd <- safedeldir(X)
  if(is.null(dd)) 
    return(NULL)
  joins <- as.matrix(dd$delsgs[, 5:6])
  return(linnet(X, edges=joins))
}

dirichletEdges <- function(X) {
  stopifnot(is.ppp(X))
  X <- unique(X, rule="deldir")
  nX <- npoints(X)
  W <- Window(X)
  if(nX < 2)
    return(edges(W))
  dd <- safedeldir(X)
  if(is.null(dd))
    return(edges(W))
  return(as.psp(dd$dirsgs[,1:4], window=W))
}

dirichletNetwork <- function(X, ...) as.linnet(dirichletEdges(X), ...)

## deprecated older names

# delaunay.distance <- function(...) {
#   .Deprecated("delaunayDistance", package="spatstat")
#   delaunayDistance(...)
# }

# delaunay.network <- function(...) {
#   .Deprecated("delaunayNetwork", package="spatstat")
#   delaunayNetwork(...)
# }

# dirichlet.edges <- function(...) {
#   .Deprecated("dirichletEdges", package="spatstat")
#   dirichletEdges(...)
# }

# dirichlet.network <- function(...) {
#   .Deprecated("dirichletNetwork", package="spatstat")
#   dirichletNetwork(...)
# }

# dirichlet.vertices <- function(...) {
#   .Deprecated("dirichletVertices", package="spatstat")
#   dirichletVertices(...)
# }

#
# randomtess.R
#
# Random tessellations
#
# $Revision: 1.7 $  $Date: 2015/10/21 09:06:57 $
#

# Poisson line tessellation

rpoislinetess <- function(lambda, win=owin()) {
  win <- as.owin(win)
  if(win$type == "mask")
    stop("Not implemented for masks")
  # determine circumcircle
  xr <- win$xrange
  yr <- win$yrange
  xmid <- mean(xr)
  ymid <- mean(yr)
  width <- diff(xr)
  height <- diff(yr)
  rmax <- sqrt(width^2 + height^2)/2
  boundbox <- owin(xmid + c(-1,1) * rmax, ymid + c(-1,1) * rmax)
  # generate poisson lines through circumcircle
  n <- rpois(1, lambda * 2 * pi * rmax)
  if(n == 0)
    return(tess(tiles=list(win)))
  theta <- runif(n, max= 2 * pi)
  p <- runif(n, max=rmax)
  Y <- infline(p=p, theta=theta)
  # form the induced tessellation in bounding box
  Z <- chop.tess(boundbox, Y)
  # clip to window
  Z <- intersect.tess(Z, win)
  attr(Z, "lines") <- Y
  return(Z)
}

rMosaicSet <- function(X, p=0.5) {
  stopifnot(is.tess(X))
  Y <- tiles(X)
  Y <- Y[runif(length(Y)) < p]
  if(length(Y) == 0)
    return(NULL)
  Z <- NULL
  for(i in seq_along(Y))
    Z <- union.owin(Z, Y[[i]])
  return(Z)
}

rMosaicField <- function(X,
                    rgen=function(n) { sample(0:1, n, replace=TRUE)},
                    ..., 
                    rgenargs=NULL ) {
  stopifnot(is.tess(X))
  Y <- as.im(X, ...)
  ntiles <- length(levels(Y))
  values <- do.call(rgen, append(list(ntiles),rgenargs))
  Z <- eval.im(values[as.integer(Y)])
  return(Z)
}

#
# infline.R
#
# Infinite lines
#
# $Revision: 1.28 $ $Date: 2017/02/07 07:47:20 $
#

infline <- function(a=NULL, b=NULL, h=NULL, v=NULL, p=NULL, theta=NULL) {
  if(is.null(a) != is.null(b))
    stop("invalid specification of a,b")
  if(is.null(p) != is.null(theta))
    stop("invalid specification of p,theta")
  if(!is.null(h)) 
    out <- data.frame(a=h, b=0, h=h, v=NA, p=h, theta=pi/2)
  else if(!is.null(v)) 
    out <- data.frame(a=NA,b=NA,h=NA,v=v,p=v,theta=ifelseAB(v < 0, pi, 0))
  else if(!is.null(a)) {
    # a, b specified
    z <- data.frame(a=a,b=b)
    a <- z$a
    b <- z$b
    theta <- ifelseAX(b == 0, pi/2, atan(-1/b))
    theta <- theta %% pi
    p <- a * sin(theta)
    out <- data.frame(a=a, b=b,
                      h=ifelseXB(b==0, a, NA),
                      v=NA, p=p, theta=theta)
  } else if(!is.null(p)) {
    # p, theta specified
    z <- data.frame(p=p,theta=theta)
    p <- z$p
    theta <- z$theta
    theta <- theta %% (2*pi)
    if(any(reverse <- (theta >= pi))) {
      theta[reverse] <- theta[reverse] - pi
      p[reverse]     <- -p[reverse]
    }
    vert <- (theta == 0)
    horz <- (cos(theta) == 0)
    gene <- !(vert | horz)
    v <- ifelseXB(vert, p, NA)
    h <- ifelseXB(horz, p, NA)
    a <- ifelseXB(gene, p/sin(theta), NA)
    b <- ifelseXB(gene, -cos(theta)/sin(theta), NA)
    out <- data.frame(a=a,b=b,h=h,v=v,p=p,theta=theta)
  } else stop("No data given!")
  class(out) <- c("infline", class(out))
  return(out)
}

is.infline <- function(x) { inherits(x, "infline") }

plot.infline <- function(x, ...) {
  for(i in seq_len(nrow(x))) {
    xi <- as.list(x[i, 1:4])
    xi[sapply(xi, is.na)] <- NULL
    do.call(abline, append(xi, list(...)))
  }
  return(invisible(NULL))
}

print.infline <- function(x, ...) {
  n <- nrow(x)
  splat(n, "infinite", ngettext(n, "line", "lines"))
  print(as.data.frame(x), ...)
  return(invisible(NULL))
}

clip.infline <- function(L, win) {
  # clip a set of infinite straight lines to a window
  win <- as.owin(win)
  stopifnot(inherits(L, "infline"))
  nL <- nrow(L)
  if(nL == 0)
    return(psp(numeric(0),numeric(0),numeric(0),numeric(0), window=win))
  seqL <- seq_len(nL)
  # determine circumcircle of win
  xr <- win$xrange
  yr <- win$yrange
  xmid <- mean(xr)
  ymid <- mean(yr)
  width <- diff(xr)
  height <- diff(yr)
  rmax <- sqrt(width^2 + height^2)/2
  boundbox <- owin(xmid + c(-1,1) * rmax, ymid + c(-1,1) * rmax)
  # convert line coordinates to origin (xmid, ymid)
  p <- L$p
  theta <- L$theta
  co <- cos(theta)
  si <- sin(theta)
  p <- p - xmid * co - ymid * si
  # compute intersection points with circumcircle 
  hit <- (abs(p) < rmax)
  if(!any(hit)) 
    return(psp(numeric(0),numeric(0),numeric(0),numeric(0), window=win))
  p <- p[hit]
  theta <- theta[hit]
  q <- sqrt(rmax^2 - p^2)
  co <- co[hit]
  si <- si[hit]
  id <- seqL[hit]
  X <- psp(x0= xmid + p * co + q * si,
           y0= ymid + p * si - q * co,
           x1= xmid + p * co - q * si,
           y1= ymid + p * si + q * co,
           marks = factor(id, levels=seqL),
           window=boundbox, check=FALSE)
  # clip to window
  X <- X[win]
  return(X)
}
  
chop.tess <- function(X, L) {
  stopifnot(is.infline(L))
  stopifnot(is.tess(X)||is.owin(X))
  X <- as.tess(X)

  if(X$type == "image") {
    Xim <- X$image
    xr <- Xim$xrange
    yr <- Xim$yrange
    # extract matrices of pixel values and x, y coordinates
    Zmat <- as.integer(as.matrix(Xim))
    xmat <- rasterx.im(Xim)
    ymat <- rastery.im(Xim)
    # process lines
    for(i in seq_len(nrow(L))) {
      # line i chops window into two pieces
      if(!is.na(h <- L[i, "h"])) {
        # horizontal line
        if(h > yr[1L] && h < yr[2L]) 
          Zmat <- 2 * Zmat + (ymat > h)
      } else if(!is.na(v <- L[i, "v"])) {
        # vertical line
        if(v > xr[1L] && v < xr[2L])
          Zmat <- 2 * Zmat + (xmat < v)
      } else {
        # generic line y = a + bx
        a <- L[i, "a"]
        b <- L[i, "b"]
        Zmat <- 2 * Zmat + (ymat > a + b * xmat)
      }
    }
    # Now just put back as factor image
    Zim <- im(Zmat, xcol=Xim$xcol, yrow=Xim$yrow, unitname=unitname(Xim))
    Z <- tess(image=Zim)
    return(Z)
  }

  #---- polygonal computation --------
  # get bounding box
  B <- as.rectangle(as.owin(X))
  xr <- B$xrange
  yr <- B$yrange

  # get coordinates
  for(i in seq_len(nrow(L))) {
    # line i chops box B into two pieces
    if(!is.na(h <- L[i, "h"])) {
      # horizontal line
      if(h < yr[1L] || h > yr[2L])
        Z <- NULL
      else {
        lower <- owin(xr, c(yr[1L], h))
        upper <- owin(xr, c(h, yr[2L]))
        Z <- tess(tiles=list(lower,upper), window=B)
      }
    } else if(!is.na(v <- L[i, "v"])) {
      # vertical line
      if(v < xr[1L] || v > xr[2L])
        Z <- NULL
      else {
        left <- owin(c(xr[1L], v), yr)
        right <- owin(c(v, xr[2L]), yr)
        Z <- tess(tiles=list(left,right), window=B)
      }
    } else {
      # generic line
      a <- L[i, "a"]
      b <- L[i, "b"]
      # Intersect with extended left and right sides of B
      yleft <- a + b * xr[1L]
      yright <- a + b * xr[2L]
      ylo <- min(yleft, yright, yr[1L]) - 1
      yhi <- max(yleft, yright, yr[2L]) + 1
      lower <- owin(poly=list(x=xr[c(1L,1L,2L,2L)],
                              y=c(yleft,ylo,ylo,yright)))
      upper <- owin(poly=list(x=xr[c(1L,2L,2L,1L)],
                              y=c(yleft,yright,yhi,yhi)))
      Bplus <- owin(xr, c(ylo, yhi), unitname=unitname(B))
      Z <- tess(tiles=list(lower,upper), window=Bplus)
    }
    # intersect this simple tessellation with X
    if(!is.null(Z)) {
      X <- intersect.tess(X, Z)
      tilenames(X) <- paste("Tile", seq_len(length(tiles(X))))
    }
  }
  return(X)
}

whichhalfplane <- function(L, x, y=NULL) {
  verifyclass(L, "infline")
  xy <- xy.coords(x, y)
  x <- xy$x
  y <- xy$y
  m <- length(x)
  n <- nrow(L)
  Z <- matrix(as.logical(NA_integer_), n, m)
  for(i in seq_len(n)) {
    if(!is.na(h <- L[i, "h"])) {
      #' horizontal line
      Z[i,] <- (y < h)
    } else if(!is.na(v <- L[i, "v"])) {
      #' vertical line
      Z[i,] <- (x < v)
    } else {
      #' generic line y = a + bx
      a <- L[i, "a"]
      b <- L[i, "b"]
      Z[i,] <- (y < a + b * x)
    }
  }
  return(Z)
}

rotate.infline <- function(X, angle=pi/2, ...) {
  if(nrow(X) == 0) return(X)
  Y <- with(X, infline(p = p, theta=theta + angle))
  return(Y)
}

shift.infline <- function(X, vec=c(0,0), ...) {
  if(nrow(X) == 0) return(X)
  vec <- as2vector(vec)
  Y <- with(X, infline(p = p + vec[1L] * cos(theta) + vec[2L] * sin(theta),
                       theta=theta))
  return(Y)
}

reflect.infline <- function(X) {
  if(nrow(X) == 0) return(X)
  Y <- with(X, infline(p = p,
                       theta=(theta + pi) %% (2 * pi)))
  return(Y)
}

flipxy.infline <- function(X) {
  if(nrow(X) == 0) return(X)
  Y <- with(X, infline(p = p,
                       theta=(pi/2 - theta) %% (2 * pi)))
  return(Y)
}

#'
#'  distcdf.R
#'
#' cdf of |X1-X2| when X1,X2 are iid uniform in W, etc
#'
#'  $Revision: 1.10 $  $Date: 2016/02/11 10:17:12 $
#'

distcdf <- function(W, V=W, ..., dW=1, dV=dW, nr=1024, regularise=TRUE) {
  reflexive <- missing(V) && missing(dV)
  diffuse <- is.owin(W) && is.owin(V)
  uniformW <- identical(dW, 1)
  uniformV <- identical(dV, 1)
  uniform <- uniformW && uniformV

  if(is.owin(W)) {
    W <- as.mask(as.owin(W), ...)
    dW <- as.im(dW, W=W)
  } else if(is.ppp(W)) {
    if(uniformW) {
      #' discrete uniform distribution on W
      dW <- pixellate(W, ...)
    } else {
      #' dW should be a weight or vector of weights
      if(!is.vector(dW) || !is.numeric(dW))
        stop("If W is a point pattern, dW should be a vector of weights")
      if(length(dW) == 1L) {
        dW <- rep(dW, npoints(W))
      } else stopifnot(length(dW) == npoints(W))
      dW <- pixellate(W, weights=dW, ...)
    }
  } else stop("W should be a point pattern or a window")
  
  if(is.owin(V)) {
    V <- as.mask(as.owin(V), ...)
    dV <- as.im(dV, W=V)
  } else if(is.ppp(V)) {
    if(uniformV) {
      #' discrete uniform distribution on V
      dV <- pixellate(V, ...)
    } else {
      #' dV should be a weight or vector of weights
      if(!is.vector(dV) || !is.numeric(dV))
        stop("If V is a point pattern, dV should be a vector of weights")
      if(length(dV) == 1L) {
        dV <- rep(dV, npoints(V))
      } else stopifnot(length(dV) == npoints(V))
      dV <- pixellate(V, weights=dV, ...)
    }
  } else stop("V should be a point pattern or a window")

  if(!uniformW && min(dW) < 0) 
    stop("Negative values encountered in dW")
  
  if(!uniformV && min(dV) < 0) 
    stop("Negative values encountered in dV")

  #' compute
  if(diffuse && uniform) {
    #' uniform distributions on windows 
    g <- if(reflexive) setcov(W, ...) else setcov(W, V, ...)
  } else {
    g <- if(reflexive) imcov(dW) else imcov(dW, dV)
  }
  r <- as.im(function(x,y) { sqrt(x^2 + y^2) }, g)
  rvals <- as.vector(as.matrix(r))
  gvals <- as.vector(as.matrix(g))
  rgrid <- seq(0, max(rvals), length=nr)
  dr <- max(rvals)/(nr-1)
  h <- whist(rvals, breaks=rgrid, weights=gvals/sum(gvals))
  ch <- c(0,cumsum(h))
  #' regularise at very short distances
  if(regularise) {
    sevenpix <- 7 * with(r, max(xstep, ystep))
    ii <- round(sevenpix/dr)
    ch[1:ii] <- ch[ii] * (rgrid[1:ii]/rgrid[ii])^2
  }
  #' ok
  result <- fv(data.frame(r=rgrid, f=ch),
                "r", quote(CDF(r)),
               "f", , range(rvals), c("r","%s(r)"),
               c("Interpoint distance","Cumulative probability"),
               fname="CDF")
  return(result)
}

bw.frac <- function(X, ..., f=1/4) {
  X <- as.owin(X)
  g <- distcdf(X, ...)
  r <- with(g, .x)
  Fr <- with(g, .y)
  iopt <- min(which(Fr >= f))
  ropt <- r[iopt]
  attr(ropt, "f") <- f
  attr(ropt, "g") <- g
  class(ropt) <- c("bw.frac", class(ropt))
  return(ropt)
}

print.bw.frac <- function(x, ...) {
  print(as.numeric(x), ...)
}

plot.bw.frac <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  g <- attr(x, "g")
  f <- attr(x, "f")
  ropt <- as.numeric(x)
  do.call(plot,
          resolve.defaults(list(g),
                             list(...),
                             list(main=xname)))
  abline(v=ropt, lty=3)
  abline(h=f, lty=3)
  invisible(NULL)
}

##
##  hermite.R
##
##  Gauss-Hermite quadrature
##
##  $Revision: 1.5 $  $Date: 2017/02/07 07:35:32 $
##

HermiteCoefs <- function(order) {
  ## compute coefficients of Hermite polynomial (unnormalised)
  x <- 1
  if(order > 0) 
    for(n in 1:order)
      x <- c(0, 2 * x) - c(((0:(n-1)) * x)[-1L], 0, 0)
  return(x)
}

gauss.hermite <- function(f, mu=0, sd=1, ..., order=5) {
  stopifnot(is.function(f))
  stopifnot(length(mu) == 1)
  stopifnot(length(sd) == 1)
  ## Hermite polynomial coefficients (un-normalised)
  Hn <- HermiteCoefs(order)
  Hn1 <- HermiteCoefs(order-1)
  ## quadrature points
  x <- sort(Re(polyroot(Hn)))
  ## weights
  Hn1x <- matrix(Hn1, nrow=1) %*% t(outer(x, 0:(order-1), "^"))
  w <- 2^(order-1) * factorial(order) * sqrt(pi)/(order * Hn1x)^2
  ## adjust
  ww <- w/sqrt(pi)
  xx <- mu + sd * sqrt(2) * x
  ## compute
  ans <- 0
  for(i in seq_along(x))
    ans <- ans + ww[i] * f(xx[i], ...)
  return(ans)
}

dmixpois <- local({

  dpoisG <- function(x, ..., k, g) dpois(k, g(x))

  function(x, mu, sd, invlink=exp, GHorder=5) 
    gauss.hermite(dpoisG, mu=mu, sd=sd, g=invlink, k=x, order=GHorder)
})

pmixpois <- local({
  ppoisG <- function(x, ..., q, g, lot) ppois(q, g(x), lower.tail=lot)

  function(q, mu, sd, invlink=exp, lower.tail = TRUE, GHorder=5) 
    gauss.hermite(ppoisG, mu=mu, sd=sd, g=invlink, q=q, order=GHorder,
                 lot=lower.tail)
})
  
qmixpois <- function(p, mu, sd, invlink=exp, lower.tail = TRUE, GHorder=5) {
  ## guess upper limit
  ## Guess upper and lower limits
  pmin <- min(p, 1-p)/2
  lam.hi <- invlink(qnorm(pmin, mean=max(mu), sd=max(sd), lower.tail=FALSE))
  lam.lo <- invlink(qnorm(pmin, mean=min(mu), sd=max(sd), lower.tail=TRUE))
  kmin <- qpois(pmin, lam.lo, lower.tail=TRUE)
  kmax <- qpois(pmin, lam.hi, lower.tail=FALSE)
  kk <- kmin:kmax
  pp <- pmixpois(kk, mu, sd, invlink, lower.tail=TRUE, GHorder)
  ans <- if(lower.tail) kk[findInterval(p, pp, all.inside=TRUE)] else
         rev(kk)[findInterval(1-p, rev(1-pp), all.inside=TRUE)]
  return(ans)
}
  
rmixpois <- function(n, mu, sd, invlink=exp) {
  lam <- invlink(rnorm(n, mean=mu, sd=sd))
  y <- rpois(n, lam)
  return(y)
}

#
#  transect.R
#
# Line transects of pixel images
#
#  $Revision: 1.7 $  $Date: 2020/04/22 05:18:40 $
#

transect.im <- local({

  specify.location <- function(loc, rect) {
    lname <- short.deparse(substitute(loc))
    if(is.numeric(loc) && length(loc) == 2)
      return(list(x=loc[1], y=loc[2]))
    if(is.list(loc))
      return(xy.coords(loc))
    if(!(is.character(loc) && length(loc) == 1))
      stop(paste("Unrecognised format for", sQuote(lname)), call.=FALSE)
    xr <- rect$xrange
    yr <- rect$yrange
    switch(loc,
           bottomleft  = list(x=xr[1],    y=yr[1]),
           bottom      = list(x=mean(xr), y=yr[1]),
           bottomright = list(x=xr[2],    y=yr[1]),
           right       = list(x=xr[2],    y=mean(yr)),
           topright    = list(x=xr[2],    y=yr[2]),
           top         = list(x=mean(xr), y=yr[2]),
           topleft     = list(x=xr[1],    y=yr[2]),
           left        = list(x=xr[1],    y=mean(yr)),
           centre=,
           center      = list(x=mean(xr), y=mean(yr)),
           stop(paste("Unrecognised location",
                      sQuote(lname), "=", dQuote(loc)),
                call.=FALSE)
           )
  }

  transect.im <- 
    function(X, ..., from="bottomleft", to="topright",
             nsample=512, click=FALSE, add=FALSE) {
      Xname <- short.deparse(substitute(X))
      Xname <- sensiblevarname(Xname, "X")
      stopifnot(is.im(X))
      check.1.integer(nsample)
      # determine transect position
      if(click) {
        # interactive
        if(!add) plot(X)
        from <- spatstatLocator(1)
        points(from)
        to <- spatstatLocator(1)
        points(to)
        segments(from$x, from$y, to$x, to$y)
      } else {
        # data defining a line segment
        R <- as.rectangle(X)
        from <- specify.location(from, R)
        to   <- specify.location(to,   R)
      }
      # create sample points along transect
      if(identical(from,to))
        stop(paste(sQuote("from"), "and", sQuote("to"),
                   "must be distinct points"), call.=FALSE)
      u <- seq(0,1,length.out=nsample)
      x <- from$x + u * (to$x - from$x)
      y <- from$y + u * (to$y - from$y)
      leng <- sqrt( (to$x - from$x)^2 +  (to$y - from$y)^2)
      t <- u * leng
      # look up pixel values (may be NA)
      v <- X[list(x=x, y=y), drop=FALSE]
      # package into fv object
      df <- data.frame(t=t, v=v)
      colnames(df)[2] <- Xname
      fv(df, argu = "t",
         ylab = substitute(Xname(t), list(Xname=as.name(Xname))),
         valu=Xname,
         labl = c("t", "%s(t)"),
         desc = c("distance along transect",
           "pixel value of %s"),
         unitname = unitname(X), fname = Xname)
    }

  transect.im
})

#
#   funxy.R
#
#   Class of functions of x,y location with a spatial domain
#
#   $Revision: 1.19 $   $Date: 2018/02/26 01:41:27 $
#

spatstat.xy.coords <- function(x,y) {
  if(missing(y) || is.null(y)) {
    xy <- if(is.ppp(x) || is.lpp(x) || is.quad(x)) coords(x) else
          if(checkfields(x, c("x", "y"))) x else 
          stop("Argument y is missing", call.=FALSE)
    x <- xy$x
    y <- xy$y
  }
  xy.coords(x,y)[c("x","y")]
}

funxy <- function(f, W=NULL) {
  stopifnot(is.function(f))
  stopifnot(is.owin(W))
  if(!identical(names(formals(f))[1:2], c("x", "y")))
    stop("The first two arguments of f should be named x and y", call.=FALSE)
  if(is.primitive(f))
    stop("Not implemented for primitive functions", call.=FALSE)
  ## copy 'f' including formals, environment, attributes
  h <- f
  ## make new function body:
  ## paste body of 'f' into last line of 'spatstat.xy.coords'
  b <- body(spatstat.xy.coords)
  b[[length(b)]] <- body(f)
  ## transplant the body 
  body(h) <- b
  ## reinstate attributes
  attributes(h) <- attributes(f)
  ## stamp it
  class(h) <- c("funxy", class(h))
  attr(h, "W") <- W
  attr(h, "f") <- f
  return(h)  
}

print.funxy <- function(x, ...) {
  nama <- names(formals(x))
  splat(paste0("function", paren(paste(nama,collapse=","))),
        "of class", sQuote("funxy"))
  print(as.owin(x))
  splat("\nOriginal function definition:")
  print(attr(x, "f"))
}

summary.funxy <- function(object, ...) {
  w <- Window(object)
  z <- list(argues  = names(formals(object)),
            fundef  = attr(object, "f"),
            values  = summary(as.im(object, ...)),
            wintype = w$type,
            frame   = Frame(w),
            units   = unitname(w))
  class(z) <- "summary.funxy"
  return(z)
}

print.summary.funxy <- function(x, ...) {
  sigdig <- getOption('digits')
  splat(paste0("function", paren(paste(x$argues,collapse=","))),
        "of class", sQuote("funxy"))
  windesc <- switch(x$wintype,
                    rectangle="the rectangle",
                    polygonal="a polygonal window inside the frame",
                    mask="a binary mask in the rectangle")
  unitinfo <- summary(x$units)
  splat("defined in",
        windesc,
        prange(signif(x$frame$xrange, sigdig)),
        "x",
        prange(signif(x$frame$yrange, sigdig)),
        unitinfo$plural,
        unitinfo$explain
        )
  splat("\nOriginal function definition:")
  print(x$fundef)
  v <- x$values
  splat("\nFunction values are", v$type)
  switch(v$type,
         integer=,
         real={
           splat("\trange =", prange(signif(v$range, sigdig)))
           splat("\tintegral =", signif(v$integral, sigdig))
           splat("\tmean =", signif(v$mean, sigdig))
         },
         factor={
           print(v$table)
         },
         complex={
           splat("\trange: Real",
                 prange(signif(v$Re$range, sigdig)),
                 "Imaginary",
                 prange(signif(v$Im$range, sigdig)))
#           splat("\tintegral =", signif(v$integral, sigdig))
           splat("\tmean =", signif(v$mean, sigdig))
         },
         {
           print(v$summary)
         })
  invisible(NULL)
}

as.owin.funxy <- function(W, ..., fatal=TRUE) {
  W <- attr(W, "W")
  as.owin(W, ..., fatal=fatal)
}

domain.funxy <- Window.funxy <- function(X, ...) { as.owin(X) }

#   Note that 'distfun' (and other classes inheriting from funxy)
#   has a method for as.owin that takes precedence over as.owin.funxy
#   and this will affect the behaviour of the following plot methods
#   because 'distfun' does not have its own plot method.

plot.funxy <- function(x, ...) {
  xname <- short.deparse(substitute(x))
  W <- as.owin(x)
  do.call(do.as.im,
          resolve.defaults(list(x, action="plot"),
                           list(...),
                           list(main=xname, W=W)))
  invisible(NULL)
}

contour.funxy <- function(x, ...) {
  xname <- deparse(substitute(x))
  W <- as.owin(x)
  do.call(do.as.im,
          resolve.defaults(list(x, action="contour"),
                           list(...),
                           list(main=xname, W=W)))
  invisible(NULL)
}

persp.funxy <- function(x, ...) {
  xname <- deparse(substitute(x))
  zlab <- substitute(expression(f(x,y)), list(f=as.name(xname)))
  W <- as.rectangle(as.owin(x))
  do.call(do.as.im,
          resolve.defaults(list(x, action="persp"),
                           list(...),
                           list(main=xname, W=W, zlab=zlab)))
  invisible(NULL)
}

hist.funxy <- function(x, ..., xname) {
  if(missing(xname) || is.null(xname)) xname <- short.deparse(substitute(x))
  a <- do.call.matched(as.im,
                       list(X=x, ...),
                       c("X", "W",
		         "dimyx", "eps", "xy",
   		         "na.replace", "strict"),
		       sieve=TRUE)
  Z <- a$result
  do.call(hist.im, append(list(x=Z, xname=xname), a$otherargs))
}

#
# blur.R
#
# apply Gaussian blur to an image
#
#    $Revision: 1.23 $   $Date: 2020/02/04 07:05:28 $
#
fillNA <- function(x, value=0) {
  stopifnot(is.im(x))
  v <- x$v
  v[is.na(v)] <- value
  x$v <- v
  return(x)
}

Smooth.im <- function(X, sigma=NULL, ..., kernel="gaussian",
                      normalise=FALSE, bleed=TRUE, varcov=NULL) {
  blur(X, sigma=sigma, ..., kernel=kernel,
       normalise=normalise, bleed=bleed, varcov=varcov)
}

blur <- function(x, sigma=NULL, ..., kernel="gaussian",
                 normalise=FALSE, bleed=TRUE, varcov=NULL) {
  stopifnot(is.im(x))
  # determine smoothing kernel 
  sigma.given <- !is.null(sigma)
  varcov.given <- !is.null(varcov)
  if (sigma.given) {
    stopifnot(is.numeric(sigma))
    stopifnot(length(sigma) %in% c(1, 2))
    stopifnot(all(sigma > 0))
  }
  if (varcov.given)
    stopifnot(is.matrix(varcov) && nrow(varcov) == 2 && ncol(varcov) == 2)
  ngiven <- varcov.given + sigma.given
  switch(ngiven + 1L,
         {
           sigma <- (1/8) * min(diff(x$xrange), diff(x$yrange))
         }, {
           if (sigma.given && length(sigma) == 2)
             varcov <- diag(sigma^2)
           if (!is.null(varcov))
             sigma <- NULL
         }, {
           stop(paste("Give only one of the arguments", sQuote("sigma"),
                      "and", sQuote("varcov")))
         })
  # replace NA's in image raster by zeroes 
  X <- fillNA(x, 0)
  # convolve with Gaussian
  Y <- second.moment.calc(X, sigma=sigma, ..., kernel=kernel,
                          varcov=varcov, what="smooth")
  # if no bleeding, we restrict data to the original boundary
  if(!bleed)
    Y$v[is.na(x$v)] <- NA
  # 
  if(!normalise)
    return(Y)
  # normalisation:
  # convert original image to window (0/1 image)
  Xone <- x
  isna <- is.na(x$v)
  Xone$v[isna] <- 0
  Xone$v[!isna] <- 1
  # convolve with Gaussian
  Ydenom <- second.moment.calc(Xone, sigma=sigma, ..., kernel=kernel,
                               varcov=varcov, what="smooth")
  # normalise
  Z <- eval.im(Y/Ydenom)
  return(Z)
}
  
safelookup <- function(Z, x, factor=2, warn=TRUE) {
  #' x is a ppp
  #' evaluates Z[x], replacing any NA's by blur(Z)[x]
  Zvals <- Z[x, drop=FALSE]
  isna <- is.na(Zvals)
  if(!any(isna))
     return(Zvals)
  #' First pass - look up values at neighbouring pixels if valid
  Xbad <- x[isna]
  rc <- nearest.valid.pixel(Xbad$x, Xbad$y, Z)
  Nvals <- Z$v[cbind(rc$row, rc$col)]
  fixed <- !is.na(Nvals)
  Zvals[isna] <- Nvals
  if(all(fixed))
    return(Zvals)
  isna[isna] <- !fixed
  #' Second pass 
  Xbad <- x[isna]
  #' expand domain of Z 
  RX <- as.rectangle(x)
  RZ <- as.rectangle(Z)
  bb <- boundingbox(RX, RZ)
  pixdiam <- sqrt(Z$xstep^2 + Z$ystep^2)
  big <- grow.rectangle(bb, 2 * pixdiam)
  Z <- rebound.im(Z, big)
  #'
  isfac <- (Z$type == "factor")
  if(!isfac) {
    #' Numerical extrapolation: blur by a few pixels
    Zblur <- blur(Z, factor * pixdiam, bleed=TRUE, normalise=TRUE)
    Bvals <- Zblur[Xbad, drop=FALSE]
    Zvals[isna] <- Bvals
    fixed <- !is.na(Bvals)
    if(warn && any(fixed))
      warning(paste("Values for", sum(fixed), 
                    "points lying slightly outside the pixel image domain",
                    "were estimated by convolution"),
              call.=FALSE)
    if(all(fixed))
      return(Zvals)
    isna[isna] <- notfixed <- !fixed
    Xbad <- Xbad[notfixed]
  }
  #' Third pass
  #' last resort: project to nearest pixel at any distance
  W <- as.mask(Z)
  eW <- exactPdt(W)
  ## discretise points of Xbad
  Gbad <- nearest.raster.point(Xbad$x, Xbad$y, W)
  ijGbad <- cbind(Gbad$row, Gbad$col)
  ## find nearest pixels inside domain
  iclosest <- eW$row[ijGbad]
  jclosest <- eW$col[ijGbad]
  ## look up values of Z
  Cvals <- Z$v[cbind(iclosest, jclosest)]
  fixed <- !is.na(Cvals)
  Zvals[isna] <- Cvals
  if(warn && any(fixed)) 
    warning(paste(if(isfac) "Categorical values" else "Values",
                  "for", sum(fixed), "points lying",
                  if(isfac) "outside" else "far outside",
                  "the pixel image domain",
                  "were estimated by projection to the nearest pixel"),
            call.=FALSE)
  if(!all(fixed))
    stop(paste("Internal error:", sum(!fixed),
               "pixel values were NA, even after projection"),
         call.=FALSE)
  return(Zvals)
}

nearestValue <- function(X) {
  #' for each raster location, look up the nearest defined pixel value
  X <- as.im(X)
  if(!anyNA(X)) return(X)
  Y <- X ## copy dimensions, value type, units etc etc
  W <- as.mask(X)
  eW <- exactPdt(W)
  iclosest <- as.vector(eW$row)
  jclosest <- as.vector(eW$col)
  ## look up values of Z
  Y$v[] <- X$v[cbind(iclosest, jclosest)]
  return(Y)
}

#
#   pcf.R
#
#   $Revision: 1.68 $   $Date: 2019/02/13 07:21:23 $
#
#
#   calculate pair correlation function
#   from point pattern (pcf.ppp)
#   or from estimate of K or Kcross (pcf.fv)
#   or from fasp object
#
#
pcf <- function(X, ...) {
  UseMethod("pcf")
}

pcf.ppp <- function(X, ..., r=NULL,
                    kernel="epanechnikov", bw=NULL, stoyan=0.15,
                    correction=c("translate", "Ripley"),
                    divisor=c("r", "d"),
                    var.approx=FALSE,
                    domain=NULL, ratio=FALSE,
                    close=NULL)
{
  verifyclass(X, "ppp")
#  r.override <- !is.null(r)

  win <- Window(X)
  areaW <- area(win)
  npts <- npoints(X)
  lambda <- npts/areaW
  lambda2area <- areaW * lambda^2

  kernel <- match.kernel(kernel)

  rmaxdefault <- rmax.rule("K", win, lambda)        
  
  if(!is.null(domain)) {
    # estimate based on contributions from a subdomain
    domain <- as.owin(domain)
    if(!is.subset.owin(domain, win))
      stop(paste(dQuote("domain"),
                 "is not a subset of the window of X"))
    # trick pcfdot() into doing it
    indom <- factor(inside.owin(X$x, X$y, domain), levels=c(FALSE,TRUE))
    g <- pcfdot(X %mark% indom,
                i="TRUE",
                r=r,
                correction=correction, kernel=kernel, bw=bw, stoyan=stoyan,
                divisor=divisor,
                ...)
    if(!ratio) {
      ## relabel
      g <- rebadge.fv(g, quote(g(r)), "g")
    } else {
      ## construct ratfv object
      denom <- sum(indom == "TRUE") * lambda
      g <- ratfv(as.data.frame(g), NULL, denom,
                 "r", quote(g(r)),
                 "theo", NULL, c(0, rmaxdefault), 
                 attr(g, "labl"), attr(g, "desc"), fname="g",
                 ratio=TRUE)
    }
    unitname(g) <- unitname(X)
    if(var.approx)
      warning("var.approx is not implemented when 'domain' is given")
    return(g)
  }

  correction.given <- !missing(correction)
  correction <- pickoption("correction", correction,
                           c(isotropic="isotropic",
                             Ripley="isotropic",
                             trans="translate",
                             translate="translate",
                             translation="translate",
                             good="translate",
                             best="best",
                             none="none"),
                           multi=TRUE)

  correction <- implemented.for.K(correction, win$type, correction.given)

  divisor <- match.arg(divisor)
  
  # bandwidth
  if(is.null(bw) && (kernel == "epanechnikov")) {
    # Stoyan & Stoyan 1995, eq (15.16), page 285
    h <- stoyan /sqrt(lambda)
    hmax <- h
    # conversion to standard deviation
    bw <- h/sqrt(5)
  } else if(is.numeric(bw)) {
    # standard deviation of kernel specified
    # upper bound on half-width
    hmax <- 3 * bw
  } else {
    # data-dependent bandwidth selection: guess upper bound on half-width
    hmax <- 2 * stoyan /sqrt(lambda)
  }

  ########## r values ############################
  # handle arguments r and breaks 

  breaks <- handle.r.b.args(r, NULL, win, rmaxdefault=rmaxdefault)
  if(!(breaks$even))
    stop("r values must be evenly spaced")
  # extract r values
  r <- breaks$r
  rmax <- breaks$max
  # recommended range of r values for plotting
  alim <- c(0, min(rmax, rmaxdefault))

  # arguments for 'density'
  denargs <- resolve.defaults(list(kernel=kernel, bw=bw),
                              list(...),
                              list(n=length(r), from=0, to=rmax),
                              .StripNull = TRUE)
  
  #################################################
  
  # compute pairwise distances
  if(npts > 1) {
    needall <- any(correction %in% c("translate", "isotropic"))
    if(is.null(close)) {
      what <- if(needall) "all" else "ijd"
      close <- closepairs(X, rmax + hmax, what=what)
    } else {
      #' check 'close' has correct format
      needed <- if(!needall) c("i", "j", "d") else
                 c("i", "j", "xi", "yi", "xj", "yj", "dx", "dy", "d")
      if(any(is.na(match(needed, names(close)))))
        stop(paste("Argument", sQuote("close"),
                   "should have components named",
                   commasep(sQuote(needed))),
             call.=FALSE)
    }
    dIJ <- close$d
  } else {
    undefined <- rep(NaN, length(r))
  }

  # initialise fv object
  
  df <- data.frame(r=r, theo=rep.int(1,length(r)))
  out <- ratfv(df,
               NULL, lambda2area,
               "r", quote(g(r)),
               "theo", NULL,
               alim,
               c("r","%s[Pois](r)"),
               c("distance argument r", "theoretical Poisson %s"),
               fname="g",
               ratio=ratio)

  ###### compute #######

  bw.used <- NULL
  
  if(any(correction=="none")) {
    #' uncorrected
    if(npts > 1) {
      kdenN <- sewpcf(dIJ, 1, denargs, lambda2area, divisor)
      gN <- kdenN$g
      bw.used <- attr(kdenN, "bw")
    } else gN <- undefined
    if(!ratio) {
      out <- bind.fv(out,
                     data.frame(un=gN),
                     "hat(%s)[un](r)",
                     "uncorrected estimate of %s",
                     "un")
    } else {
      out <- bind.ratfv(out,
                        data.frame(un=gN * lambda2area),
                        lambda2area,
                        "hat(%s)[un](r)",
                        "uncorrected estimate of %s",
                        "un")
    }
  }
  
  if(any(correction=="translate")) {
    # translation correction
    if(npts > 1) {
      edgewt <- edge.Trans(dx=close$dx, dy=close$dy, W=win, paired=TRUE)
      kdenT <- sewpcf(dIJ, edgewt, denargs, lambda2area, divisor)
      gT <- kdenT$g
      bw.used <- attr(kdenT, "bw")
    } else gT <- undefined
    if(!ratio) {
      out <- bind.fv(out,
                     data.frame(trans=gT),
                     "hat(%s)[Trans](r)",
                     "translation-corrected estimate of %s",
                     "trans")
    } else {
      out <- bind.ratfv(out,
                        data.frame(trans=gT * lambda2area),
                        lambda2area,
                        "hat(%s)[Trans](r)",
                        "translation-corrected estimate of %s",
                        "trans")
    }
  }

  if(any(correction=="isotropic")) {
    # Ripley isotropic correction
    if(npts > 1) {
      XI <- ppp(close$xi, close$yi, window=win, check=FALSE)
      edgewt <- edge.Ripley(XI, matrix(dIJ, ncol=1))
      kdenR <- sewpcf(dIJ, edgewt, denargs, lambda2area, divisor)
      gR <- kdenR$g
      bw.used <- attr(kdenR, "bw")
    } else gR <- undefined
    if(!ratio) {
      out <- bind.fv(out,
                     data.frame(iso=gR),
                     "hat(%s)[Ripley](r)",
                     "isotropic-corrected estimate of %s",
                     "iso")
    } else {
      out <- bind.ratfv(out,
                        data.frame(iso=gR * lambda2area),
                        lambda2area,
                        "hat(%s)[Ripley](r)",
                        "isotropic-corrected estimate of %s",
                        "iso")
    }
  }
  
  # sanity check
  if(is.null(out)) {
    warning("Nothing computed - no edge corrections chosen")
    return(NULL)
  }

  ## variance approximation
  ## Illian et al 2008 p 234 equation 4.3.42
  if(var.approx) {
    gr <- if(any(correction == "isotropic")) gR else gT
    # integral of squared kernel
    intk2 <- kernel.squint(kernel, bw.used)
    # isotropised set covariance of window
    gWbar <- as.function(rotmean(setcov(win), result="fv"))
    vest <- gr * intk2/(pi * r * gWbar(r) * lambda^2)
    if(!ratio) {
      out <- bind.fv(out,
                     data.frame(v=vest),
                     "v(r)",
                     "approximate variance of %s",
                     "v")
    } else {
      vden <- rep((npts-1)^2, length(vest))
      vnum <- vden * vest
      out <- bind.ratfv(out,
                        data.frame(v=vnum),
                        data.frame(v=vden),
                        "v(r)", 
                        "approximate variance of %s",
                        "v")
    }
  }

  ## Finish off
  ## default is to display all corrections
  formula(out) <- . ~ r
  fvnames(out, ".") <- setdiff(rev(colnames(out)), c("r", "v"))
  ##
  unitname(out) <- unitname(X)
  ## copy to other components
  if(ratio)
    out <- conform.ratfv(out)

  attr(out, "bw") <- bw.used
  return(out)
}

# Smoothing Estimate of Weighted Pair Correlation
# d = vector of relevant distances
# w = vector of edge correction weights (in normal use)
# denargs = arguments to density.default
# lambda2area = constant lambda^2 * areaW (in normal use)

sewpcf <- function(d, w, denargs, lambda2area, divisor=c("r","d")) {
  divisor <- match.arg(divisor)
  nw <- length(w)
  if(nw != length(d) && nw != 1)
    stop("Internal error: incorrect length of weights vector in sewpcf")
  if(divisor == "d") {
    w <- w/d
    if(!all(good <- is.finite(w))) {
      nbad <- sum(!good)
      warning(paste(nbad, "infinite or NA",
                    ngettext(nbad, "contribution was", "contributions were"),
                    "deleted from pcf estimate"))
      d <- d[good]
      w <- w[good]
    }
  }
  if(nw == 1) {
    #' weights are equal
    kden <- do.call.matched(density.default,
                            append(list(x=d), denargs))
    wtot <- length(d)
  } else {
    #' weighted 
    wtot <- sum(w)
    kden <- do.call.matched(density.default,
                            append(list(x=d, weights=w/wtot), denargs))
  }
  r <- kden$x
  y <- kden$y * wtot
  if(divisor == "r")
    y <- y/r
  g <- y/(2 * pi * lambda2area)
  result <- data.frame(r=r,g=g)
  attr(result, "bw") <- kden$bw
  return(result)
}

#
#---------- OTHER METHODS FOR pcf --------------------
#

"pcf.fasp" <- function(X, ..., method="c") {
  verifyclass(X, "fasp")
  Y <- X
  Y$title <- paste("Array of pair correlation functions",
                   if(!is.null(X$dataname)) "for",
                   X$dataname)
  # go to work on each function
  for(i in seq_along(X$fns)) {
    Xi <- X$fns[[i]]
    PCFi <- pcf.fv(Xi, ..., method=method)
    Y$fns[[i]] <- PCFi
    if(is.fv(PCFi))
      Y$default.formula[[i]] <- formula(PCFi)
  }
  return(Y)
}


pcf.fv <- local({

  callmatched <- function(fun, argue) {
    formalnames <- names(formals(fun))
    formalnames <- formalnames[formalnames != "..."]
    do.call(fun, argue[names(argue) %in% formalnames])
  }

  pcf.fv <- function(X, ..., method="c") {
    verifyclass(X, "fv")
  
    # extract r and the recommended estimate of K
    r <- with(X, .x)
    K <- with(X, .y)
    alim <- attr(X, "alim")

    # remove NA's
    ok <- !is.na(K)
    K <- K[ok]
    r <- r[ok]
    switch(method,
           a = {
             ss <- callmatched(smooth.spline,
                               list(x=r, y=K, ...))
             dK <- predict(ss, r, deriv=1)$y
             g <- dK/(2 * pi * r)
           },
           b = {
             y <- K/(2 * pi * r)
             y[!is.finite(y)] <- 0
             ss <- callmatched(smooth.spline,
                               list(x=r, y=y, ...))
             dy <- predict(ss, r, deriv=1)$y
             g <- dy + y/r
           },
           c = {
             z <- K/(pi * r^2)
             z[!is.finite(z)] <- 1
             ss <- callmatched(smooth.spline,
                               list(x=r, y=z, ...))
             dz <- predict(ss, r, deriv=1)$y
             g <- (r/2) * dz + z
           },
           d = {
             z <- sqrt(K)
             z[!is.finite(z)] <- 0
             ss <- callmatched(smooth.spline,
                               list(x=r, y=z, ...))
             dz <- predict(ss, r, deriv=1)$y
             g <- z * dz/(pi * r)
           },
           stop(paste("unrecognised method", sQuote(method)))
           )

    # pack result into "fv" data frame
    Z <- fv(data.frame(r=r,
                       theo=rep.int(1, length(r)),
                       pcf=g),
            "r", substitute(g(r), NULL), "pcf", . ~ r, alim,
            c("r", "%s[pois](r)", "%s(r)"),
            c("distance argument r",
              "theoretical Poisson value of %s",
              "estimate of %s by numerical differentiation"),
            fname="g")
    unitname(Z) <- unitname(X)
    return(Z)
  }

  pcf.fv
})

#
#    rat.R
#
#   Ratio objects
#
#   Numerator and denominator are stored as attributes
#
#   $Revision: 1.12 $   $Date: 2020/01/10 03:10:03 $
#

rat <- function(ratio, numerator, denominator, check=TRUE) {
  if(check) {
    stopifnot(compatible(numerator, denominator))
    stopifnot(compatible(ratio, denominator))
  }
  attr(ratio, "numerator") <- numerator
  attr(ratio, "denominator") <- denominator
  class(ratio) <- unique(c("rat", class(ratio)))
  return(ratio)
}

print.rat <- function(x, ...) {
  NextMethod("print")
  cat("[Contains ratio information]\n")
  return(invisible(NULL))
}

compatible.rat <- function(A, B, ...) {
  NextMethod("compatible")
}

pool.rat <- local({

  Add <- function(A,B){ force(A); force(B); eval.fv(A+B, relabel=FALSE) }
  Square <- function(A) { force(A);         eval.fv(A^2, relabel=FALSE) }
  Mul <- function(A,B){ force(A); force(B); eval.fv(A*B, relabel=FALSE) }

  pool.rat <- function(..., weights=NULL, relabel=TRUE, variance=TRUE) {
    argh <- list(...)
    n <- narg <- length(argh)
    if(narg == 0) return(NULL)
    if(narg == 1) return(argh[[1]])
    ##
    israt <- unlist(lapply(argh, inherits, what="rat"))
    if(any(bad <- !israt)) {
      nbad <- sum(bad)
      stop(paste(ngettext(nbad, "Argument", "Arguments"),
                 commasep(which(bad)),
                 ngettext(nbad, "does not", "do not"),
                 "contain ratio (numerator/denominator) information"))
    }
    isfv <- unlist(lapply(argh, is.fv))
    if(!all(isfv))
      stop("All arguments must be fv objects")
    ## extract
    template <- vanilla.fv(argh[[1]])
    Y <- lapply(argh, attr, which="numerator")
    X <- lapply(argh, attr, which="denominator")
    X <- do.call(harmonise, X)
    Y <- do.call(harmonise, Y)
    templateX <- vanilla.fv(X[[1]])
    templateY <- vanilla.fv(Y[[1]])
    ## compute products
    if(!is.null(weights)) {
      check.nvector(weights, narg, things="Functions")
      X <- Map(Mul, X, weights)
      Y <- Map(Mul, Y, weights)
    } 
    ## sum
    sumX <- Reduce(Add, X)
    sumY <- Reduce(Add, Y)
    attributes(sumX) <- attributes(templateX)
    attributes(sumY) <- attributes(templateY)
    ## ratio-of-sums
    Ratio <- eval.fv(sumY/sumX, relabel=FALSE)
    attributes(Ratio) <- attributes(template)
    ## variance calculation
    if(variance) { 
      meanX <- eval.fv(sumX/n, relabel=FALSE)
      meanY <- eval.fv(sumY/n, relabel=FALSE)
      sumX2 <- Reduce(Add, lapply(X, Square))
      sumY2 <- Reduce(Add, lapply(Y, Square))
      varX   <- eval.fv((sumX2 - n * meanX^2)/(n-1), relabel=FALSE)
      varY   <- eval.fv((sumY2 - n * meanY^2)/(n-1), relabel=FALSE)
      XY <- Map(Mul, X, Y)
      sumXY <- Reduce(Add, XY)
      covXY <- eval.fv((sumXY - n * meanX * meanY)/(n-1), relabel=FALSE)
      ## variance by delta method
      relvar <- eval.fv(pmax.int(0, varY/meanY^2 + varX/meanX^2
                                 - 2 * covXY/(meanX * meanY)),
		        relabel=FALSE)
      Variance <- eval.fv(Ratio^2 * relvar/n, relabel=FALSE)
      attributes(Variance) <- attributes(template)
      ## two sigma CI
      hiCI <- eval.fv(Ratio + 2 * sqrt(Variance), relabel=FALSE)
      loCI <- eval.fv(Ratio - 2 * sqrt(Variance), relabel=FALSE)
      attributes(hiCI) <- attributes(loCI) <-  attributes(template)
    }
    ## dress up
    if(relabel) {
      Ratio <- prefixfv(Ratio,
                        tagprefix="pool",
                        descprefix="pooled ",
                        lablprefix="")
      if(variance) {		      
        Variance <- prefixfv(Variance,
                             tagprefix="var",
                             descprefix="delta-method variance estimate of ",
                             lablprefix="bold(var)~")
        hiCI <- prefixfv(hiCI,
                         tagprefix="hi",
                         descprefix="upper limit of two-sigma CI based on ",
                         lablprefix="bold(hi)~")
        loCI <- prefixfv(loCI,
                         tagprefix="lo",
                         descprefix="lower limit of two-sigma CI based on ",
                         lablprefix="bold(lo)~")
      }
    }
    result <- if(!variance) Ratio else
              Reduce(bind.fv, list(Ratio, Variance, hiCI, loCI))
    return(result)
  }

  pool.rat
  
})

adjust.ratfv <- function(f, columns=fvnames(f, "*"), numfactor=1, denfactor=1) {
  stopifnot(is.fv(f))
  f[,columns] <- (numfactor/denfactor) * as.data.frame(f)[,columns]
  if(numfactor != 1 && !is.null(num <- attr(f, "numerator"))) {
    num[,columns] <- numfactor * as.data.frame(num)[,columns]
    attr(f, "numerator") <- num
  }	    
  if(denfactor != 1 && !is.null(den <- attr(f, "denominator"))) {
    den[,columns] <- denfactor * as.data.frame(den)[,columns]
    attr(f, "denominator") <- den
  }
  return(f)
}  

tweak.ratfv.entry <- function(x, ...) {
  # apply same tweak to function, numerator and denominator.
  x <- tweak.fv.entry(x, ...)
  if(!is.null(num <- attr(x, "numerator")))
    attr(x, "numerator") <- tweak.fv.entry(num, ...)
  if(!is.null(den <- attr(x, "denominator")))
    attr(x, "denominator") <- tweak.fv.entry(den, ...)
  return(x)
}

"[.rat" <- function(x, ...) {
   if(!is.fv(x)) stop("Not yet implemented for non-fv ratios")
   num <- attr(x, "numerator")
   den <- attr(x, "denominator")
   class(x) <- "fv"
   x <- x[...]
   den <- den[...]
   num <- num[...]
   attr(x, "numerator") <- num
   attr(x, "denominator") <- den
   class(x) <- unique(c("rat", class(x)))
   return(x)
}
  

#
#  cut.ppp.R
#
#  cut method for ppp objects
#
#  $Revision: 1.15 $   $Date: 2016/10/26 09:29:57 $
#

cut.ppp <- function(x, z=marks(x), ...) {
  x <- as.ppp(x)
  if(missing(z) || is.null(z)) {
    z <- marks(x, dfok=TRUE)
    if(is.null(z))
      stop("x has no marks to cut")
  }
  if(is.character(z)) {
    if(length(z) == npoints(x)) {
      # interpret as a factor
      z <- factor(z)
    } else if((length(z) == 1L) && (z %in% colnames(df <- as.data.frame(x)))) {
      # interpret as the name of a column of marks or coordinates
      z <- df[, z]
    } else stop("format of argument z not understood") 
  }
  if(is.factor(z) || is.vector(z)) {
    stopifnot(length(z) == npoints(x))
    g <- if(is.factor(z)) z else if(is.numeric(z)) cut(z, ...) else factor(z)
    marks(x) <- g
    return(x)
  }
  if(is.data.frame(z) || is.matrix(z)) {
    stopifnot(nrow(z) == npoints(x))
    # take first column 
    z <- z[,1L]
    g <- if(is.numeric(z)) cut(z, ...) else factor(z)
    marks(x) <- g
    return(x)
  }
  if(is.im(z)) 
    return(cut(x, z[x, drop=FALSE], ...))

  if(is.owin(z)) {
    marks(x) <- factor(inside.owin(x$x, x$y, z), levels=c(FALSE, TRUE))
    return(x)
  }
  
  if(is.tess(z)) {
    marks(x) <- tileindex(x$x, x$y, z)
    return(x)
  }

  stop("Format of z not understood")
} 

#'
#'   Header for all (concatenated) test files
#'
#'   Require spatstat.
#'   Obtain environment variable controlling tests.
#'
#'   $Revision: 1.5 $ $Date: 2020/04/30 05:31:37 $

require(spatstat)
FULLTEST <- (nchar(Sys.getenv("SPATSTAT_TEST", unset="")) > 0)
ALWAYS   <- TRUE
cat(paste("--------- Executing",
          if(FULLTEST) "** ALL **" else "**RESTRICTED** subset of",
          "test code -----------\n"))
#'  tests/randoms.R
#'   Further tests of random generation code
#'  $Revision: 1.12 $ $Date: 2020/05/01 02:42:58 $


local({
  if(FULLTEST) {
    A <- runifrect(6, nsim=2)
    A <- runifdisc(6, nsim=2)
    A <- runifpoispp(5, nsim=2)
    A <- runifpoispp(0, nsim=2)
    A <- rSSI(0.05, 6, nsim=2)
    A <- rSSI(0.05, 10, win=square(c(-0.5, 1.5)), x.init=A[[1]], nsim=2)  
    A <- rstrat(nx=4, nsim=2)
    A <- rsyst(nx=4, nsim=2)
  }
  if(ALWAYS) { # involves C code etc
    A <- rthin(cells, P=0.5, nsim=2)
    A <- rthin(cells, runif(42))
    A <- rthin(cells[FALSE], P=0.5, nsim=2)
  }
  if(FULLTEST) {
    A <- rjitter(cells, nsim=2, retry=FALSE)
    A <- rjitter(cells[FALSE])
    A <- rcell(square(1), nx=5, nsim=2)
  }
  f <- function(x,y) { 10*x }
  Z <- as.im(f, square(1))
  if(ALWAYS) {
    A <- rpoint(n=6, f=f, fmax=10, nsim=2)
    A <- rpoint(n=6, f=Z, fmax=10, nsim=2)
    A <- rpoint(n=0, f=f, fmax=10, nsim=2)
    A <- rpoint(n=0, f=Z, fmax=10, nsim=2)

    op <- spatstat.options(fastpois=FALSE)
    A <- runifpoispp(5, nsim=2)
    A <- rpoispp(Z)
    spatstat.options(op)
  }
  if(FULLTEST) {
    b3 <- box3(c(0,1))
    b4 <- boxx(c(0,1), c(0,1), c(0,1), c(0,1))
    b5 <- c(0, 2, 0, 2)
    X <- rMaternInhibition(2, kappa=20, r=0.1, win=b3)
    Y <- rMaternInhibition(2, kappa=20, r=0.1, win=b4)
    Y <- rMaternInhibition(2, kappa=20, r=0.1, win=b5, nsim=2)

    X <- rSSI(0.05, 6)
    Y <- rSSI(0.05, 6, x.init=X) # no extra points

    Z <- rlabel(finpines)
  }

  f1 <- function(x,y){(x^2 + y^3)/10}
  f2 <- function(x,y){(x^3 + y^2)/10}
  ZZ <- solist(A=as.im(f1, letterR),
               B=as.im(f2, letterR))
  g <- function(x,y,m){(10+as.integer(m)) * (x^2 + y^3)}
  if(FULLTEST) {
    XX <- rmpoispp(ZZ, nsim=3)
    YY <- rmpoint(10, f=ZZ, nsim=3)
    VV <- rpoint.multi(10, f=g,
                       marks=factor(sample(letters[1:3], 10, replace=TRUE)),
                       nsim=3)
  }
  if(ALWAYS) { # depends on C code
    L <- edges(letterR)
    E <- runifpoisppOnLines(5, L)
    G <- rpoisppOnLines(ZZ, L)
    G2 <- rpoisppOnLines(list(A=f1, B=f2), L, lmax=max(sapply(ZZ, max)))
  }

  if(FULLTEST) {
    #' cluster models + bells + whistles
    X <- rThomas(10, 0.2, 5, saveLambda=TRUE)
    if(is.null(attr(X, "Lambda")))
      stop("rThomas did not save Lambda image")
    Y <- rThomas(0, 0.2, 5, saveLambda=TRUE)
    if(is.null(attr(Y, "Lambda")))
      stop("rThomas did not save Lambda image when kappa=0")
    X <- rMatClust(10, 0.05, 4, saveLambda=TRUE)
    X <- rCauchy(30, 0.01, 5, saveLambda=TRUE)
    X <- rVarGamma(30, 2, 0.02, 5, saveLambda=TRUE)
    Z <- as.im(function(x,y){ 5 * exp(2 * x - 1) }, owin())
    Y <- rThomas(10, 0.2, Z, saveLambda=TRUE)
    Y <- rMatClust(10, 0.05, Z, saveLambda=TRUE)
    Y <- rCauchy(30, 0.01, Z, saveLambda=TRUE)
    Y <- rVarGamma(30, 2, 0.02, Z, saveLambda=TRUE)
  }

  if(FULLTEST) {
    #' perfect simulation code infrastructure
    expandwinPerfect(letterR, 2, 3)
  }

})
reset.spatstat.options()


#'  tests/resid.R
#'
#'  Stuff related to residuals and residual diagnostics
#'
#'   $Revision: 1.6 $  $Date: 2020/05/01 02:42:58 $
#'

local({
  fit <- ppm(cells ~x, Strauss(r=0.15))
  rr <- residuals(fit, quad=quadscheme(cells, nd=128))
  diagnose.ppm(fit, cumulative=FALSE, type="pearson")

  if(FULLTEST) {
    diagnose.ppm(fit, cumulative=FALSE)

    fitoff <- ppm(cells ~ sin(x) + offset(y))
    plot(a <- parres(fitoff, "x"))
    plot(b <- parres(fitoff, "y"))
    print(a)
    print(b)
  
    d <- diagnose.ppm(fit, which="marks")
    plot(d, plot.neg="discrete")
    plot(d, plot.neg="imagecontour")

    d <- diagnose.ppm(fit, type="pearson", which="smooth")
    plot(d, plot.smooth="image")
    plot(d, plot.smooth="contour")
    plot(d, plot.smooth="imagecontour")
  
    d <- diagnose.ppm(fit, type="pearson", which="x")
    plot(d)
    d <- diagnose.ppm(fit, type="pearson", which="y")
    plot(d)
  
    diagnose.ppm(fit, type="pearson", which="x", cumulative=FALSE)
    diagnose.ppm(fit, type="pearson", which="x", cumulative=FALSE)
    diagnose.ppm(fit, type="raw", plot.neg="discrete", plot.smooth="image")
    diagnose.ppm(fit, type="pearson", plot.neg="contour", plot.smooth="contour")

    diagnose.ppm(fitoff, type="raw", which="smooth", plot.smooth="persp")
    diagnose.ppm(fitoff, type="pearson", plot.neg="imagecontour")

    plot(Frame(letterR), main="")
    ploterodewin(letterR, erosion(letterR, 0.05), main="jeans")
    W <- as.mask(letterR)
    plot(Frame(W), main="")
    ploterodewin(W, erosion(W, 0.05), main="JeAnS")

    #' entangled terms in model
    U <- as.im(1, owin())
    Z <- as.im(function(x,y) x, owin())
    X <- runifpoint(40)
    fut <- ppm(X ~ Z:U)
    a <- parres(fut, "Z")
    futoff <- ppm(X ~ offset(Z*U))
    a <- parres(futoff, "Z")
  }
})



##
## tests/rhohat.R
##
## Test all combinations of options for rhohatCalc
##
## $Revision: 1.5 $ $Date: 2020/05/01 02:42:58 $

local({
  if(FULLTEST) {
    X <-  rpoispp(function(x,y){exp(3+3*x)})
    Z <- as.im(function(x,y) { x }, Window(X))
    f <- funxy(function(x,y) { y + 1 }, Window(X))
    ## rhohat.ppp
    ## done in example(rhohat):
    ## rhoA <- rhohat(X, "x")
    ## rhoB <- rhohat(X, "x", method="reweight")
    ## rhoC <- rhohat(X, "x", method="transform")
    ## alternative smoother (if package locfit available)
    rhoA <- rhohat(X, "x", smoother="local")
    rhoB <- rhohat(X, "x", smoother="local", method="reweight")
    rhoC <- rhohat(X, "x", smoother="local", method="transform")

    #' code blocks
    rhoD <- rhohat(X, "y", positiveCI=TRUE)
    rhoE <- rhohat(X, Z,   positiveCI=TRUE)
    #' weights 
    rhoF <- rhohat(X, Z,   weights=f(X))
    rhoG <- rhohat(X, Z,   weights=f)
    rhoH <- rhohat(X, Z,   weights=as.im(f))
    
    ## rhohat.ppm
    fit <- ppm(X ~x)
    rhofitA <- rhohat(fit, "x")
    rhofitB <- rhohat(fit, "x", method="reweight")
    rhofitC <- rhohat(fit, "x", method="transform")
    rhofitD <- rhohat(fit, Z)
    rhofitD <- rhohat(fit, Z, positiveCI=TRUE)

    ## Baseline
    lam <- predict(fit)
    rhoAb <- rhohat(X, "x", baseline=lam)
    rhoBb <- rhohat(X, "x", method="reweight", baseline=lam)
    rhoCb <- rhohat(X, "x", method="transform", baseline=lam)

    ## Horvitz-Thompson
    rhoAH <- rhohat(X, "x", horvitz=TRUE) 
    rhoBH <- rhohat(X, "x", method="reweight", horvitz=TRUE)
    rhoCH <- rhohat(X, "x", method="transform", horvitz=TRUE)
    rhofitAH <- rhohat(fit, "x", horvitz=TRUE)
    rhofitBH <- rhohat(fit, "x", method="reweight", horvitz=TRUE)
    rhofitCH <- rhohat(fit, "x", method="transform", horvitz=TRUE)

    ## class support
    plot(rhoA)
    plot(rhoA, rho ~ x, shade=NULL)
    plot(rhoA, log(rho) ~ x, shade=NULL)
    plot(rhoA, log(.) ~ x)

    ## rho2hat
    r2xy <- rho2hat(X, "x", "y")
    r2xyw <- rho2hat(X, "x", "y", method="reweight")
    print(r2xyw)
    plot(r2xy, do.points=TRUE)
    xcoord <- function(x,y) x
    ycoord <- function(x,y) y
    xim <- as.im(xcoord, W=Window(X))
    r2fi <- rho2hat(X, ycoord, xim)
    r2if <- rho2hat(X, xim, ycoord)
    r2myx <- rho2hat(fit, "y", "x")
    r2myxw <- rho2hat(fit, "y", "x", method="reweight")
    plot(r2myx)
    plot(r2myxw)
    print(r2myxw)
    predict(r2myxw)
    predict(r2myxw, relative=TRUE)
  }
})

#
# layered.R
#
# Simple mechanism for layered plotting
#
#  $Revision: 1.39 $  $Date: 2017/06/05 10:31:58 $
#

layered <- function(..., plotargs=NULL, LayerList=NULL) {
  argh <- list(...)
  if(length(argh) > 0 && !is.null(LayerList))
    stop("LayerList is incompatible with other arguments")
  out <- if(!is.null(LayerList)) LayerList else argh
  n <- length(out)
  if(sum(nzchar(names(out))) != n)
    names(out) <- paste("Layer", seq_len(n))
  if(is.null(plotargs)) {
    plotargs <- rep.int(list(list()), n)
  } else {
    if(!is.list(plotargs))
      stop("plotargs should be a list of lists")
    if(!all(unlist(lapply(plotargs, is.list))))
      plotargs <- list(plotargs)
    np <- length(plotargs)
    if(np == 1) plotargs <- rep(plotargs, n) else if(np != n)
      stop("plotargs should have one component for each element of the list")
  }
  names(plotargs) <- names(out)
  attr(out, "plotargs") <- plotargs
  class(out) <- c("layered", class(out))
  return(out)
}

print.layered <- function(x, ...) {
  splat("Layered object")
  if(length(x) == 0) splat("(no entries)")
  for(i in seq_along(x)) {
    cat(paste("\n", names(x)[i], ":\n", sep=""))
    print(x[[i]])
  }
  pl <- layerplotargs(x)
  hasplot <- (lengths(pl) > 0)
  if(any(hasplot)) 
    splat("Includes plot arguments for", commasep(names(pl)[hasplot]))
  invisible(NULL)
}

plot.layered <- function(x, ..., which=NULL, plotargs=NULL,
                         add=FALSE, show.all=!add, main=NULL,
                         do.plot=TRUE) {
  if(is.null(main))
    main <- short.deparse(substitute(x))
  n <- length(x)
  if(!is.null(plotargs)) {
    np <- length(plotargs)
    if(!(is.list(plotargs) && all(unlist(lapply(plotargs, is.list)))))
      stop("plotargs should be a list of lists")
  }
  ## select layers
  if(!is.null(which)) {
    x <- x[which]
    nw <- length(x)
    if(!is.null(plotargs)) {
      if(np == n) plotargs <- plotargs[which] else
      if(np == 1) plotargs <- rep(plotargs, nw) else
      if(np != nw) 
        stop("plotargs should have one component for each layer to be plotted")
    }
    n <- nw
  } else if(!is.null(plotargs)) {
    if(np == 1) plotargs <- rep(plotargs, n) else
    if(np != n) stop("plotargs should have one component for each layer")
  }
  ## remove null layers
  if(any(isnul <- unlist(lapply(x, is.null)))) {
    x <- x[!isnul]
    if(!is.null(plotargs))
      plotargs <- plotargs[!isnul]
    n <- length(x)
  }
  ## anything to plot?
  if(n == 0)
    return(invisible(NULL))
  ## Merge plotting arguments
  xplotargs <- layerplotargs(x)
  if(is.null(plotargs)) {
    plotargs <- xplotargs
  } else if(length(xplotargs) > 0) {
    for(i in 1:n)
      plotargs[[i]] <- resolve.defaults(plotargs[[i]], xplotargs[[i]])
  }
  ## Determine bounding box
  a <- plotEachLayer(x, ..., plotargs=plotargs, add=add,
                     show.all=show.all, do.plot=FALSE)
  if(!do.plot)
    return(a)
  bb <- as.rectangle(as.owin(a))
  ## Start plotting
  if(!add && !is.null(bb)) {
    ## initialise new plot using bounding box
    pt <- prepareTitle(main)
    plot(bb, type="n", main=pt$blank)
    add <- TRUE
  }
  # plot the layers
  out <- plotEachLayer(x, ..., main=main,
                       plotargs=plotargs, add=add,
                       show.all=show.all, do.plot=TRUE)
  return(invisible(out))
}

plotEachLayer <- function(x, ..., main,
                          plotargs, add, show.all, do.plot=TRUE) {
  main.given <- !missing(main)
  ## do.plot=TRUE    =>   plot the layers 
  ## do.plot=FALSE   =>   determine bounding boxes
  out <- boxes <- list()
  nama <- names(x)
  firstlayer <- TRUE
  for(i in seq_along(x)) {
    xi <- x[[i]]
    if(length(xi) == 0) {
      # null layer - no plotting
      out[[i]] <- boxes[[i]] <- NULL
    } else {
      ## plot layer i on top of previous layers if any.
      ## By default,
      ##    - show all graphic elements of the first component only;
      ##    - show title 'firstmain' on first component;
      ##    - do not show any component names.
      add.i <- add || !firstlayer
      if(main.given) {
        main.i <- if(firstlayer) main else ""
      } else {
        show.all.i <- resolve.1.default(list(show.all=FALSE),
                                         list(...), 
                                         plotargs[[i]])
        main.i <- if(show.all.i) nama[i] else ""
      }
      dflt <- list(main=main.i,
                   show.all=show.all && firstlayer)
      pla.i <- plotargs[[i]]
      defaultplot <- !(".plot" %in% names(pla.i))
      ## plot layer i, or just determine bounding box
      if(defaultplot &&
         inherits(xi, c("ppp", "psp", "owin",
                        "lpp", "linnet", 
                        "im", "msr", "layered"))) {
        ## plot method for 'xi' has argument 'do.plot'.
        mplf <-
          if(inherits(xi, c("ppp", "lpp"))) list(multiplot=FALSE) else list()
        out[[i]] <- outi <- do.call(plot,
                                    resolve.defaults(list(x=xi,
                                                          add=add.i,
                                                          do.plot=do.plot),
                                                     list(...),
                                                     mplf,
                                                     pla.i,
                                                     dflt))
        boxes[[i]] <- as.rectangle(as.owin(outi))
      } else {
        ## plot method for 'xi' does not have argument 'do.plot'
        if(do.plot) {
          if(defaultplot) {
            plotfun <- "plot"
          } else {
            plotfun <- pla.i[[".plot"]]
            pla.i <- pla.i[names(pla.i) != ".plot"]
          }
          out[[i]] <- outi <- do.call(plotfun,
                                      resolve.defaults(list(x=xi,
                                                            add=add.i),
                                                       list(...),
                                                       pla.i,
                                                       dflt))
        }
        ## convert layer i to box
        boxi <- try(as.rectangle(xi), silent=TRUE)
        boxes[[i]] <- if(!inherits(boxi, "try-error")) boxi else NULL
      }
      firstlayer <- FALSE
    }
  }
  ## one box to bound them all
  if(!all(unlist(lapply(boxes, is.null))))
    attr(out, "bbox") <- do.call(boundingbox, boxes)
  return(out)
}


"[.layered" <- function(x, i, j, drop=FALSE, ...) {
  i.given <- !missing(i) && !is.null(i)
  j.given <- !missing(j) && !is.null(j)
  if(!i.given && !j.given)
    return(x)
  p <- attr(x, "plotargs")
  x <- unclass(x)
  nx <- length(x)
  if(i.given) {
    if(is.owin(i)) {
      #' spatial window subset
      nonemp <- (lengths(x) != 0)
      x[nonemp] <- lapply(x[nonemp], "[", i=i, ...)
    } else {
      #' vector subset index
      x <- x[i]
      p <- p[i]
      nx <- length(x)
    }
  }
  if(j.given) {
    nonemp <- (lengths(x) != 0)
    x[nonemp] <- lapply(x[nonemp], "[", i=j, ...)
  }
  if(drop && nx == 1)
    return(x[[1L]])
  y <- layered(LayerList=x, plotargs=p)
  return(y)
}

"[[<-.layered" <- function(x, i, value) {
  x[i] <- if(!is.null(value)) list(value) else NULL
  return(x)
}

"[<-.layered" <- function(x, i, value) {
  p <- layerplotargs(x)
  ## invoke list method
  y <- x
  class(y) <- "list"
  y[i] <- value
  # make it a 'layered' object too
  class(y) <- c("layered", class(y))
  # update names and plotargs
  if(any(blank <- !nzchar(names(y)))) {
    names(y)[blank] <- paste("Layer", which(blank))
    pnew <- rep(list(list()), length(y))
    names(pnew) <- names(y)
    m <- match(names(y), names(x))
    mok <- !is.na(m)
    pnew[mok] <- p[m[mok]]
    layerplotargs(y) <- pnew
  } else layerplotargs(y) <- layerplotargs(x)[names(y)]
  return(y)
}

layerplotargs <- function(L) {
  stopifnot(inherits(L, "layered"))
  attr(L, "plotargs")
}

"layerplotargs<-" <- function(L, value) {
  if(!inherits(L, "layered"))
    L <- layered(L)
  if(!is.list(value))
    stop("Replacement value should be a list, or a list-of-lists")
  n <- length(L)
  if(!all(unlist(lapply(value, is.list)))) 
    value <- unname(rep(list(value), n))
  if(length(value) != n) {
    if(length(value) == 1) value <- unname(rep(value, n)) else
    stop("Replacement value is wrong length")
  }
  if(is.null(names(value))) names(value) <- names(L) else
  if(!identical(names(value), names(L)))
    stop("Mismatch in names of list elements")
  attr(L, "plotargs") <- value
  return(L)
}

applytolayers <- function(L, FUN, ...) {
  # Apply FUN to each **non-null** layer,
  # preserving the plot arguments
  pla <- layerplotargs(L)
  if(length(L) > 0) {
    ok <- !unlist(lapply(L, is.null))
    L[ok] <- lapply(L[ok], FUN, ...)
  }
  Z <- layered(LayerList=L, plotargs=pla)
  return(Z)
}
  
shift.layered <- function(X, vec=c(0,0), ...) {
  if(length(list(...)) > 0) {
    if(!missing(vec)) 
      warning("Argument vec ignored; overridden by other arguments")
    ## ensure the same shift is applied to all layers
    s <- shift(X[[1L]], ...)
    vec <- getlastshift(s)
  }
  Y <- applytolayers(X, shift, vec=vec)
  attr(Y, "lastshift") <- vec
  return(Y)
}

affine.layered <- function(X, ...) {
  applytolayers(X, affine, ...)
}

rotate.layered <- function(X, ..., centre=NULL) {
  if(!is.null(centre)) {
    X <- shift(X, origin=centre)
    negorigin <- getlastshift(X)
  } else negorigin <- NULL
  Y <- applytolayers(X, rotate, ...)
  if(!is.null(negorigin))
    Y <- shift(Y, -negorigin)
  return(Y)
}

reflect.layered <- function(X) {
  applytolayers(X, reflect)
}

flipxy.layered <- function(X) {
  applytolayers(X, flipxy)
}

scalardilate.layered <- function(X, ...) {
  applytolayers(X, scalardilate, ...)
}
  
rescale.layered <- function(X, s, unitname) {
  if(missing(s)) s <- NULL
  if(missing(unitname)) unitname <- NULL
  applytolayers(X, rescale, s=s, unitname=unitname) 
}


as.owin.layered <- local({

  as.owin.layered <- function(W, ..., fatal=TRUE) {
    if(length(W) == 0) {
      if(fatal) stop("Layered object is empty: no window data")
      return(NULL)
    }
    ## remove null layers
    isnul <- unlist(lapply(W, is.null))
    W <- W[!isnul]
    if(length(W) == 0) {
      if(fatal) stop("Layered object has no window data")
      return(NULL)
    }
    Wlist <- lapply(unname(W), as.owin, ..., fatal=fatal)
    Wlist <- lapply(Wlist, rescue.rectangle)
    Wlist <- lapply(Wlist, puffbox)
    Z <- Wlist[[1L]]
    if(length(Wlist) > 1) {
      same <- unlist(lapply(Wlist[-1L], identical, y=Z))
      if(!all(same))
        Z <- do.call(union.owin, Wlist)
    }
    return(Z)
  }

  puffbox <- function(W) {
    ## union.owin will delete boxes that have width zero or height zero
    ## so 'puff' them out slightly
    ss <- sidelengths(Frame(W))
    if(ss[1L] == 0) W$xrange <- W$xrange + 1e-6 * c(-1,1) * ss[2L]
    if(ss[2L] == 0) W$yrange <- W$yrange + 1e-6 * c(-1,1) * ss[1L]
    return(W)
  }
  
  as.owin.layered
})


domain.layered <- Window.layered <- function(X, ...) { as.owin(X) }

as.layered <- function(X) {
  UseMethod("as.layered")
}

as.layered.default <- function(X) {
  if(is.list(X) && all(sapply(X, is.sob))) layered(LayerList=X) else 
  layered(X)
}

as.layered.ppp <- function(X) {
  if(!is.marked(X)) return(layered(X))
  if(is.multitype(X)) return(layered(LayerList=split(X)))
  mX <- marks(X)
  if(!is.null(d <- dim(mX)) && d[2L] > 1) {
    mx <- as.data.frame(marks(X))
    Y <- lapply(mx, setmarks, x=X)
    return(layered(LayerList=Y))
  }
  return(layered(X))
}


  

#
# summary.quad.R
#
#  summary() method for class "quad"
#
#  $Revision: 1.12 $ $Date: 2018/07/06 02:05:31 $
#

summary.quad <- local({

  sumriz <- function(ww) {
    if(length(ww) > 0) 
      return(list(range=range(ww), sum=sum(ww)))
    else
      return(NULL)
  }

  summary.quad <- function(object, ..., checkdup=FALSE) {
    verifyclass(object, "quad")
    X <- object$data
    D <- object$dummy
    s <- list(
      logi  = inherits(object, "logiquad"),
      data  = summary.ppp(X, checkdup=checkdup),
      dummy = summary.ppp(D, checkdup=checkdup),
      param = object$param)
    ## make description of dummy point arrangement
    dpar <- object$param$dummy
    eps.given <- dpar$orig$eps # could be NULL
    eps.actual <- NULL
    if(is.null(dpar)) {
      descrip <- "(provided manually)"
    } else if(is.character(dmethod <- dpar$method)) {
      descrip <- dmethod
    } else if(identical(dpar$quasi, TRUE)) {
      descrip <- paste(npoints(D), "quasirandom dummy points",
                       "plus 4 corner points")
      eps.actual <- 1/(2 * sqrt(intensity(D)))
    } else if(!is.null(nd <- dpar$nd)) {
      nd <- ensure2vector(nd)
      eps.actual <- unique(sidelengths(Frame(D))/nd)
      if(identical(dpar$random, TRUE)) {
        descrip <- paste("systematic random dummy points in",
                         nd[1], "x", nd[2], "grid",
                         "plus 4 corner points")
      } else {
        descrip <- paste(nd[1], "x", nd[2],
                         "grid of dummy points, plus 4 corner points")
      }
    } else descrip <- "(rule for creating dummy points not understood)"
    
    if(!is.null(eps.actual)) {
      uD <- unitname(D)
      s$resolution <- numberwithunit(eps.actual, uD)
      if(!is.null(eps.given)) {
        descrip2 <- paste("dummy spacing:",
                          format(eps.given %unit% uD), "requested,", 
                          format(eps.actual %unit% uD), "actual")
      } else {
        descrip2 <- paste("dummy spacing:", format(eps.actual %unit% uD))
      }
      descrip <- c(descrip, descrip2)
    }
    s$descrip <- descrip
    
    w <- object$w
    Z <- is.data(object)
    s$w <- list(all   = sumriz(w),
                data  = sumriz(w[Z]),
                dummy = sumriz(w[!Z]))
    class(s) <- "summary.quad"
    return(s)
  }

  summary.quad
})

print.summary.quad <- local({

  summariseweights <- function(ww, blah, dp=3) {
    cat(paste(blah, ":\n\t", sep=""))
    if(is.null(ww)) {
      cat("(None)\n")
      return()
    }
    splat(paste0("range: ",
              "[",
              paste(signif(ww$range, digits=dp), collapse=", "),
              "]\t",
              "total: ",
              signif(ww$sum, digits=dp)))
  }

  print.summary.quad <- function(x, ..., dp=3) {
    splat("Quadrature scheme (Berman-Turner) = data + dummy + weights")
    pa <- x$param
    if(is.null(pa))
      splat("created by an unknown function.")

    parbreak()

    splat("Data pattern:")
    print(x$data, dp=dp)

    parbreak()

    splat("Dummy quadrature points:")
    ## How they were computed
    splat(x$descrip, indent=5)
    parbreak()
    ## What arguments were given
    if(!is.null(orig <- pa$dummy$orig))
      splat("Original dummy parameters:",
            paste0(names(orig), "=", orig, collapse=", "))
    ## Description of the dummy points
    print(x$dummy, dp=dp)

    splat("Quadrature weights:")
    ## How they were computed
    if(!is.null(pa)) {
      wpar <- pa$weight
      if(is.null(wpar))
        splat("(values provided manually)", indent=5)
      else if(is.character(wmethod <- wpar$method)) {
        switch(wmethod,
               grid = {
                 splat("(counting weights based on",
                       wpar$ntile[1], "x", wpar$ntile[2],
                       "array of rectangular tiles)",
                       indent=5)
               },
               dirichlet = {
                 splat("(Dirichlet tile areas, computed",
                       if(wpar$exact) "exactly)" else "by pixel approximation)",
                       indent=5)
               },
               splat(wmethod, indent=5)
               )
      } else splat("(rule for creating dummy points not understood)")
    }
    if(waxlyrical('extras')) {
      summariseweights(x$w$all, "All weights", dp)
      summariseweights(x$w$data, "Weights on data points", dp)
      summariseweights(x$w$dummy, "Weights on dummy points", dp)
    }
    return(invisible(NULL))
  }

  print.summary.quad
})

print.quad <- function(x, ...) {
  logi <- inherits(x, "logiquad")
  splat("Quadrature scheme",
        paren(if(logi) "logistic" else "Berman-Turner"))
  splat(x$data$n, "data points,", x$dummy$n, "dummy points")
  if(waxlyrical('extras')) {
    sx <- summary(x)
    splat(sx$descrip, indent=5)
  }
  splat("Total weight", sum(x$w), indent=5)
  return(invisible(NULL))
}

#
#      distanxD.R
#
#      $Revision: 1.15 $     $Date: 2020/04/24 03:33:47 $
#
#      Interpoint distances for multidimensional points
#
#      Methods for pairdist, nndist, nnwhich, crossdist
#

pairdist.ppx <- function(X, ...) {
  verifyclass(X, "ppx")
  # extract point coordinates
  coo <- as.matrix(coords(X, ...))
  n <- nrow(coo)
  if(n == 0)
    return(matrix(numeric(0), nrow=0, ncol=0))
  return(as.matrix(dist(coo)))
}

crossdist.ppx <- function(X, Y, ...) {
  verifyclass(X, "ppx")
  verifyclass(Y, "ppx")
  # extract point coordinates
  cooX <- as.matrix(coords(X, ...))
  cooY <- as.matrix(coords(Y, ...))
  nX <- nrow(cooX)
  nY <- nrow(cooY)
  if(ncol(cooX) != ncol(cooY))
    stop("X and Y have different dimensions (different numbers of coordinates)")
  if(nX == 0 || nY == 0)
    return(matrix(numeric(0), nrow=nX, ncol=nY))
  coo <- rbind(cooX, cooY)
  dis <- as.matrix(dist(coo))
  ans <- dis[1:nX, nX + (1:nY)]
  return(ans)
}

genericNNdistBy <- function(X, by, k=1) {
  #' performs nndist(X, by) using generic nndist and nncross
  #' assuming they conform to the standard output format
  Y <- split(X, by) 
  m <- length(Y)
  lev <- names(Y)
  blanklist <- vector(mode="list", length=m)
  partresults <- blanklist
  for(i in 1:m) {
    Yi <- Y[[i]]
    contrib <- blanklist
    for(j in 1:m)
      contrib[[j]] <-
        if(i == j) nndist(Yi, k=k) else nncross(Yi, Y[[j]], k=k, what="dist")
    names(contrib) <- lev
    partresults[[i]] <- do.call(cbind, contrib)
  }
  names(partresults) <- lev
  result <- as.data.frame(matrix(, npoints(X), m * length(k)))
  colnames(result) <- colnames(partresults[[1L]])
  split(result, marks(X)) <- partresults
  return(result)
}

nndist.ppx <- function(X, ..., k=1, by=NULL) {
  verifyclass(X, "ppx")

  if(!is.null(by)) 
    return(genericNNdistBy(X, by, k=k))
  
  ## extract point coordinates
  coo <- as.matrix(coords(X, ...))
  n <- nrow(coo)
  m <- ncol(coo)

  if(m == 0) {
    warning("nndist.ppx: Zero-dimensional coordinates: returning NA")
    if(length(k) == 1L)
      return(rep.int(NA_real_, n))
    else
      return(matrix(NA_real_, n, length(k)))
  }
  
  # k can be a single integer or an integer vector
  if(length(k) == 0)
    stop("k is an empty vector")
  else if(length(k) == 1L) {
    if(k != round(k) || k <= 0)
      stop("k is not a positive integer")
  } else {
    if(any(k != round(k)) || any(k <= 0))
      stop(paste("some entries of the vector",
           sQuote("k"), "are not positive integers"))
  }
  k <- as.integer(k)
  kmax <- max(k)

  # trivial cases
  if(n <= 1L) {
    # empty pattern => return numeric(0)
    # or pattern with only 1 point => return Inf
    nnd <- matrix(Inf, nrow=n, ncol=kmax)
    nnd <- nnd[,k, drop=TRUE]
    return(nnd)
  }
  
  # number of neighbours that are well-defined
  kmaxcalc <- min(n-1L, kmax)

  # calculate k-nn distances for k <= kmaxcalc
  
  if(kmaxcalc == 1L) {
    # calculate nearest neighbour distance only
    nnd<-numeric(n)
    o <- fave.order(coo[,1L])
    big <- sqrt(.Machine$double.xmax)
    Cout <- .C("nndMD",
               n= as.integer(n),
               m=as.integer(m),
               x= as.double(t(coo[o,])),
               nnd= as.double(nnd),
               as.double(big),
               PACKAGE = "spatstat")
    nnd[o] <- Cout$nnd
  } else {
    # case kmaxcalc > 1
    nnd<-numeric(n * kmaxcalc)
    o <- fave.order(coo[,1L])
    big <- sqrt(.Machine$double.xmax)
    Cout <- .C("knndMD",
               n    = as.integer(n),
               m    = as.integer(m),
               kmax = as.integer(kmaxcalc),
               x    = as.double(t(coo[o,])),
               nnd  = as.double(nnd),
               huge = as.double(big),
               PACKAGE = "spatstat")
    nnd <- matrix(nnd, nrow=n, ncol=kmaxcalc)
    nnd[o, ] <- matrix(Cout$nnd, nrow=n, ncol=kmaxcalc, byrow=TRUE)
  }

  # post-processing
  if(kmax > kmaxcalc) {
    # add columns of Inf's
    infs <- matrix(as.numeric(Inf), nrow=n, ncol=kmax-kmaxcalc)
    nnd <- cbind(nnd, infs)
  }

  # add labels
  if(kmax > 1)
    colnames(nnd) <- paste0("dist.", 1:kmax)

  if(length(k) < kmax) {
    # select only the specified columns
    nnd <- nnd[, k, drop=TRUE]
  }
  
  return(nnd)
}

nnwhich.ppx <- function(X, ..., k=1) {
  verifyclass(X, "ppx")
  # k can be a single integer or an integer vector
  if(length(k) == 0)
    stop("k is an empty vector")
  else if(length(k) == 1L) {
    if(k != round(k) || k <= 0)
      stop("k is not a positive integer")
  } else {
    if(any(k != round(k)) || any(k <= 0))
      stop(paste("some entries of the vector",
           sQuote("k"), "are not positive integers"))
  }
  k <- as.integer(k)
  kmax <- max(k)

  # extract point coordinates
  coo <- coords(X, ...)
  n <- nrow(coo)
  m <- ncol(coo)
  
  if(m == 0) {
    warning("nnwhich.ppx: Zero-dimensional coordinates: returning NA")
    if(length(k) == 1L)
      return(rep.int(NA_real_, n))
    else
      return(matrix(NA_real_, n, length(k)))
  }
  
  # special cases
  if(n <= 1L) {
    # empty pattern => return integer(0)
    # or pattern with only 1 point => return NA
    nnw <- matrix(NA_integer_, nrow=n, ncol=kmax)
    nnw <- nnw[,k, drop=TRUE]
    return(nnw)
  }

  # number of neighbours that are well-defined
  kmaxcalc <- min(n-1L, kmax)

  # identify k-nn for k <= kmaxcalc

  if(kmaxcalc == 1L) {
    # identify nearest neighbour only
    nnw <- integer(n)
    o <- fave.order(coo[,1L])
    big <- sqrt(.Machine$double.xmax)
    Cout <- .C("nnwMD",
               n = as.integer(n),
               m = as.integer(m),
               x = as.double(t(coo[o,])),
               nnd = as.double(numeric(n)),
               nnwhich = as.integer(nnw),
               huge = as.double(big),
               PACKAGE = "spatstat")
    witch <- Cout$nnwhich
    if(any(witch <= 0))
      stop("Internal error: non-positive index returned from C code")
    if(any(witch > n))
      stop("Internal error: index returned from C code exceeds n")
    nnw[o] <- o[witch]
  } else {
    # case kmaxcalc > 1
    nnw <- matrix(integer(n * kmaxcalc), nrow=n, ncol=kmaxcalc)
    o <- fave.order(coo[,1L])
    big <- sqrt(.Machine$double.xmax)
    Cout <- .C("knnwMD",
               n = as.integer(n),
               m = as.integer(m),
               kmax = as.integer(kmaxcalc),
               x = as.double(t(coo[o,])),
               nnd = as.double(numeric(n * kmaxcalc)),
               nnwhich = as.integer(nnw),
               huge = as.double(big),
               PACKAGE = "spatstat")
    witch <- Cout$nnwhich
    witch <- matrix(witch, nrow=n, ncol=kmaxcalc, byrow=TRUE)
    if(any(witch <= 0))
      stop("Internal error: non-positive index returned from C code")
    if(any(witch > n))
      stop("Internal error: index returned from C code exceeds n")
    # convert back to original ordering
    nnw[o,] <- matrix(o[witch], nrow=n, ncol=kmaxcalc)
  }
  
  # post-processing
  if(kmax > kmaxcalc) {
    # add columns of NA's
    nas <- matrix(NA_integer_, nrow=n, ncol=kmax-kmaxcalc)
    nnw <- cbind(nnw, nas)
  }

  if(kmax > 1)
    colnames(nnw) <- paste0("which.", 1:kmax)
  
  if(length(k) < kmax) {
    # select only the specified columns
    nnw <- nnw[, k, drop=TRUE]
  }
  return(nnw)
}

nncross.ppx <- function(X, Y, iX=NULL, iY=NULL,
                        what = c("dist", "which"),
                        ...,
                        k = 1) {
  verifyclass(X, "ppx")
  verifyclass(Y, "ppx")
  what <- match.arg(what, several.ok=TRUE)
  want.dist  <- "dist" %in% what
  want.which <- "which" %in% what
  ## k can be a single integer or an integer vector
  if(length(k) == 0)
    stop("k is an empty vector")
  else if(length(k) == 1L) {
    if(k != round(k) || k <= 0)
      stop("k is not a positive integer")
  } else {
    if(any(k != round(k)) || any(k <= 0))
      stop(paste("some entries of the vector",
           sQuote("k"), "are not positive integers"))
  }
  k <- as.integer(k)
  kmax <- max(k)
  nk <- length(k)
  ## extract point coordinates
  cooX <- as.matrix(coords(X, ...))
  nX <- nrow(cooX)
  m  <- ncol(cooX)
  cooY <- as.matrix(coords(Y, ...))
  nY <- nrow(cooY)
  mY <- ncol(cooY)
  ## check dimensions
  if(mY != m)
    stop(paste("Point patterns have different spatial dimensions:",
               m, "!=", mY),
         call.=FALSE)
  if(m == 0) {
    warning("nncross.ppx: Zero-dimensional coordinates: returning NA")
    if(nk == 1L) {
      NND <- if(want.dist) rep.int(NA_real_,     nX) else 0
      NNW <- if(want.which) rep.int(NA_integer_, nX) else 0
    } else {
      NND <- if(want.dist) matrix(NA_real_,     nX, nk) else 0
      NNW <- if(want.which) matrix(NA_integer_, nX, nk) else 0
    }
    return(packupNNdata(NND, NNW, what, k))
  }
  ## trivial cases
  if(nX == 0L || nY == 0L) {
    NND <- matrix(Inf,         nrow=nX, ncol=nk)
    NNW <- matrix(NA_integer_, nrow=nX, ncol=nk)
    return(packupNNdata(NND, NNW, what, k))
  }

  ## exclusion arguments
  if(is.null(iX) != is.null(iY))
    stop("If one of iX, iY is given, then both must be given")
  exclude <- (!is.null(iX) || !is.null(iY))
  if(exclude) {
    stopifnot(is.integer(iX) && is.integer(iY))
    if(length(iX) != nX)
      stop("length of iX does not match the number of points in X")
    if(length(iY) != nY)
      stop("length of iY does not match the number of points in Y")
  }

  ## number of neighbours that are well-defined
  kmaxcalc <- min(nY, kmax)

  ## find k-nearest neighbours for k <= kmaxcalc
  oX <- fave.order(cooX[,1L])
  oY <- fave.order(cooY[,1L])
  big <- sqrt(.Machine$double.xmax)

  if(kmaxcalc == 1L) {
    ## find nearest neighbour only
    nnd <- numeric(nX)
    nnw <- integer(nX)
    if(!exclude) {
      Cout <- .C("nnXwMD",
                 m  =as.integer(m),
                 n1 = as.integer(nX),
                 x1 = as.double(t(cooX[oX,])),
                 n2 = as.integer(nY),
                 x2 = as.double(t(cooY[oY,])),
                 nnd = as.double(nnd),
                 nnwhich = as.integer(nnw),
                 as.double(big),
                 PACKAGE = "spatstat")
    } else {
      Cout <- .C("nnXxMD",
                 m  =as.integer(m),
                 n1 = as.integer(nX),
                 x1 = as.double(t(cooX[oX,])),
                 id1 = as.integer(iX[oX]),
                 n2 = as.integer(nY),
                 x2 = as.double(t(cooY[oY,])),
                 id2 = as.integer(iY[oY]),
                 nnd = as.double(nnd),
                 nnwhich = as.integer(nnw),
                 as.double(big),
                 PACKAGE = "spatstat")
    }
    if(want.dist)
      nnd[oX] <- Cout$nnd
    if(want.which) {
      witch <- Cout$nnwhich
      if(any(witch <= 0))
        stop("Internal error: non-positive index returned from C code")
      if(any(witch > nY))
        stop("Internal error: index returned from C code exceeds npoints(Y)")
      nnw[oX] <- oY[witch]
    }
  } else {
    ## k-nearest
    nnd <- matrix(0,  nX, kmaxcalc)
    nnw <- matrix(0L, nX, kmaxcalc)
    if(!exclude) {
      Cout <- .C("knnXwMD",
                 m       = as.integer(m),
                 n1      = as.integer(nX),
                 x1      = as.double(t(cooX[oX,])),
                 n2      = as.integer(nY),
                 x2      = as.double(t(cooY[oY,])),
                 kmax    = as.integer(kmaxcalc),
                 nnd     = as.double(nnd),
                 nnwhich = as.integer(nnw),
                 huge    = as.double(big),
                 PACKAGE = "spatstat")
    } else {
      Cout <- .C("knnXxMD",
                 m       = as.integer(m),
                 n1      = as.integer(nX),
                 x1      = as.double(t(cooX[oX,])),
                 id1     = as.integer(iX[oX]),
                 n2      = as.integer(nY),
                 x2      = as.double(t(cooY[oY,])),
                 id2     = as.integer(iY[oY]),
                 kmax    = as.integer(kmaxcalc),
                 nnd     = as.double(nnd),
                 nnwhich = as.integer(nnw),
                 huge    = as.double(big),
                 PACKAGE = "spatstat")
    }
    dust <- Cout$nnd
    witch <- Cout$nnwhich
    if(any(notfound <- (witch <= 0 | witch > nY))) {
      dust[notfound] <- Inf
      witch[notfound] <- NA
    }
    nnd[oX, ] <- matrix(dust, nrow=nX, ncol=kmaxcalc, byrow=TRUE)
    nnw[oX, ] <- matrix(oY[witch], nrow=nX, ncol=kmaxcalc, byrow=TRUE)
  }

  ## post-processing
  if(kmax > kmaxcalc) {
    ## add columns of Inf's/NA's
    if(want.dist) {
      infs <- matrix(as.numeric(Inf), nrow=nX, ncol=kmax-kmaxcalc)
      nnd <- cbind(nnd, infs)
    }
    if(want.which) {
      nas <- matrix(NA_integer_,      nrow=nX, ncol=kmax-kmaxcalc)
      nnw <- cbind(nnw, nas)
    }
  }
  if(length(k) < kmax) {
    ## select only the specified columns
    if(want.dist) nnd <- nnd[, k, drop=TRUE]
    if(want.which) nnw <- nnw[, k, drop=TRUE]
  }
  return(packupNNdata(nnd, nnw, what, k))  
}


packupNNdata <- function(NND, NNW, what, k) {
  result <- as.data.frame(list(dist=NND, which=NNW)[what])
  if(max(k) > 1L) {
    colnames(result) <- c(if("dist" %in% what) paste0("dist.", k) else NULL,
                          if("which" %in% what) paste0("which.",k) else NULL)
  }
  if(ncol(result) == 1L)
    result <- result[, , drop=TRUE]
  return(result)
}

#
#
#	classes.S
#
#	$Revision: 1.7 $	$Date: 2006/10/09 03:38:14 $
#
#	Generic utilities for classes
#
#
#--------------------------------------------------------------------------

verifyclass <- function(X, C, N=deparse(substitute(X)), fatal=TRUE) {
  if(!inherits(X, C)) {
    if(fatal) {
        gripe <- paste("argument", sQuote(N),
                       "is not of class", sQuote(C))
	stop(gripe)
    } else 
	return(FALSE)
  }
  return(TRUE)
}

#--------------------------------------------------------------------------

checkfields <- function(X, L) {
	  # X is a list, L is a vector of strings
	  # Checks for presence of field named L[i] for all i
	return(all(!is.na(match(L,names(X)))))
}

getfields <- function(X, L, fatal=TRUE) {
	  # X is a list, L is a vector of strings
	  # Extracts all fields with names L[i] from list X
	  # Checks for presence of all desired fields
	  # Returns the sublist of X with fields named L[i]
	absent <- is.na(match(L, names(X)))
	if(any(absent)) {
		gripe <- paste("Needed the following components:",
				paste(L, collapse=", "),
				"\nThese ones were missing: ",
				paste(L[absent], collapse=", "))
		if(fatal)
			stop(gripe)
		else 
			warning(gripe)
	} 
	return(X[L[!absent]])
}

\documentclass[11pt]{article}

% \VignetteIndexEntry{Analysing Replicated Point Patterns in Spatstat}

\usepackage{graphicx}
\usepackage{Sweave}
\usepackage{bm}
\usepackage{anysize}

\marginsize{2cm}{2cm}{2cm}{2cm}

\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\R}{{\sf R}}
\newcommand{\spst}{\pkg{spatstat}}
\newcommand{\Spst}{\pkg{Spatstat}}

\newcommand{\bold}[1]{{\textbf {#1}}}

\newcommand{\indicate}[1]{\boldmaths{1}\{ {#1} \}}
\newcommand{\dee}[1]{\, {\rm d}{#1}}
\newcommand{\boldmaths}[1]{{\ensuremath\boldsymbol{#1}}}
\newcommand{\xx}{\boldmaths{x}}

\begin{document}
\bibliographystyle{plain}
\thispagestyle{empty}

<<echo=FALSE,results=hide,fig=FALSE>>=
options(SweaveHooks=list(fig=function() par(mar=c(1,1,1,1))))
@ 
\SweaveOpts{eps=TRUE}
\setkeys{Gin}{width=0.6\textwidth}

<<echo=FALSE,results=hide>>=
library(spatstat)
spatstat.options(image.colfun=function(n) { grey(seq(0,1,length=n)) })
sdate <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
         fields = "Date")
sversion <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
         fields = "Version")
options(useFancyQuotes=FALSE)
@ 

\title{Analysing replicated point patterns in \texttt{spatstat}}
\author{Adrian Baddeley}
\date{For \spst\ version \texttt{\Sexpr{sversion}}}
\maketitle

\begin{abstract}
This document describes \spst's capabilities for
fitting models to replicated point patterns.
More generally it applies to data from a designed experiment
in which the response from each unit is a spatial point pattern.
\end{abstract}

\tableofcontents
\newpage

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Introduction}

`Replicated point patterns' are datasets consisting of 
several point patterns which can be 
regarded as independent repetitions of the same experiment. For example,
three point patterns taken from micrographs of three pipette samples of the 
same jug of milk, could be assumed to be replicated observations.

More generally we could have several experimental groups, with 
replicated point pattern data in each group. For example there may be
two jugs of milk that were treated differently, and we take three
pipette samples from each jug.

Even more generally our point patterns could be the result
of a designed experiment involving
control and treatment groups, covariates such as temperature,
and even spatial covariates (such as image data). 

This document describes some capabilities available in the \spst\ package
for analysing such data. 
\textbf{For further detail, see Chapter 16 of the spatstat book \cite{TheBook}.}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Overview of software}

The main components needed are:

\begin{itemize}
\item the model-fitting function \texttt{mppm}, an extension of the
  \texttt{spatstat} function \texttt{ppm}, that will fit Gibbs point process
  models to multiple point pattern datasets;
\item support for the class \texttt{"mppm"} of point process models 
   fitted by \texttt{mppm} (e.g. functions to print and plot the fitted model,
   analysis of deviance for Poisson models)
\item some tools for exploratory data analysis;
\item basic support for the data from such experiments
  by storing the data in a \emph{``hyperframe''}. A hyperframe is like
  a data frame, except that each entry in a column can be a point pattern
  or a pixel image, as well as a single number or categorical value.
\item four example datasets.
\end{itemize}

\section{Formulating the problem}

We view the experiment as involving a series of
{\em `units'\/}.
Each unit is subjected to a known set of experimental conditions 
(described by the values of the {\em covariates\/}), and
each unit yields a {\em response\/} which is a spatial point pattern.
The value of a particular covariate for each unit can be
either a single value (numerical, logical or factor),
or a pixel image.

Three important cases are:
\begin{description}
\item[independent replicates:]
We observe $n$
different point patterns that can be regarded as independent replicates,
i.e.\ independent realisations of the same point process.
The `responses' are the point patterns; there are no covariates.
\item[replication in groups:]
there are $K$ different experimental groups (e.g. control, aspirin,
nurofen). In group $k$ ($k=1,\ldots,K$) we observe $n_k$
point patterns which can be regarded as independent replicates within
this group. We regard this as an experiment with $n = \sum_k n_k$ 
units. The responses are the point patterns; there is one covariate
which is a factor (categorical variable) identifying which group 
each point pattern belongs to.
\item[general case:] 
there are covariates other than factors that influence
the response. The point patterns are assumed to be independent, 
but no two patterns have the same distribution.
\end{description}

Examples of these three cases are given in the 
datasets \texttt{waterstriders}, \texttt{pyramidal} and \texttt{demohyper}
respectively, which are installed in \spst.

\section{Installed datasets}

The following datasets are currently installed in \spst.

\begin{itemize}
\item \texttt{waterstriders}: Penttinen's \cite{pent84} waterstriders data
recording the locations of insect larvae on a pond in 3 independent
experiments.
\item \texttt{pyramidal}: data from Diggle, Lange and Benes 
\cite{digglangbene91} on the locations of pyramidal neurons in 
human brain, 31 human subjects grouped into 3 groups (controls,
schizoaffective and schizophrenic).
\item \texttt{flu}: data from Chen et al \cite{chenetal08}
giving the locations of two different virus proteins 
on the membranes of cells infected with influenza virus;
41 multitype point patterns divided into two virus types
(wild and mutant) and two stain types. 
\item \texttt{simba}: simulated data from an experiment with two groups
and 5 replicate point patterns per group.
\item \texttt{demohyper}: simulated data from an experiment with two 
  groups in which each experimental unit has a point pattern response
  and a pixel image covariate. 
\end{itemize}

\section{Lists of point patterns}

First we need a convenient way to store the \emph{responses}
from all the units in an experiment.

An individual point pattern is stored as an object of class \verb!"ppp"!. 
The easiest way to store all the responses is to form a list
of \verb!"ppp"! objects.

\subsection{Waterstriders data}

The \texttt{waterstriders} data are an example of this type.
The data consist of 3 independent point patterns representing the 
locations of insect larvae on a pond. See \texttt{help(waterstriders)}.

<<>>=
waterstriders
@ 

The \texttt{waterstriders} dataset is a list of point patterns.
It is a list, each of whose entries is a point pattern (object of class
\verb!"ppp"!). Note that the observation windows of the
three point patterns are {\tt not\/} identical. 

\subsection{The class \texttt{listof}}

For convenience, the \texttt{waterstriders} dataset also belongs to the
class \verb!"listof"!. This is a simple mechanism to allow us to
handle the list neatly --- for example, we can provide
special methods for printing, plotting and summarising the list.

\SweaveOpts{width=6,height=2}
\setkeys{Gin}{width=0.9\textwidth}

<<fig=TRUE>>=
plot(waterstriders, main="")
@ 

Notice that the 
plot method displays each entry of the list in a separate panel.
There's also the summary method:

<<>>=
summary(waterstriders)
@ 

\subsection{Creating a \texttt{listof} object}

For example, here is a simulated dataset containing three
independent realisations of the Poisson process with intensity 100.

<<>>=
X <- listof(rpoispp(100), rpoispp(100), rpoispp(100))
@ 

Then it can be printed and plotted.

<<fig=TRUE>>=
plot(X)
X
@ 

To convert an existing list to the class \code{listof}, use
\code{as.listof}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Hyperframes}

A \emph{hyperframe} is like a data frame, except that its entries
can be objects of any kind.
A hyperframe is effectively a two-dimensional array
in which each column consists of
values of one type (as in a data frame) or consists of
objects of one class. 

The entries in a hyperframe can be point patterns, pixel images,
windows, or any other objects. 

To analyse an experiment, we will store {\bf all} the data from the experiment
in a single hyperframe. The rows of the hyperframe will correspond 
to different experimental units,
while the columns represent different variables 
(response variables or covariates).

\subsection{Creating hyperframes}

The function \texttt{hyperframe} will create a hyperframe.

<<eval=FALSE>>=
hyperframe(...)
@ 

The arguments \verb!...! are any number of arguments of
the form \texttt{tag=value}. Each \texttt{value} will
become a column of the array. The \texttt{tag} determines the name
of the column.

Each \texttt{value} can be either
\begin{itemize}
\item an atomic vector or factor
  (i.e. numeric vector, integer vector, character vector, logical
  vector, complex vector or factor)
\item a list of objects which are all of the same class
\item one atomic value, which will be replicated to make an atomic
  vector or factor
\item one object, which will be replicated to make a list of identical objects.
\end{itemize}
    
All columns (vectors, factors and lists) must be of the same length,
if their length is greater than 1. 

For example, here is a hyperframe containing a column of
numbers and a column of \emph{functions}:

<<>>=
H <- hyperframe(X=1:3, Y=list(sin,cos,tan))
H
@ 

Note that a column of character strings will be converted to a factor, 
unless you set \texttt{stringsAsFactors=FALSE} in the call to
\code{hyperframe}. This is the same behaviour as for the function 
\code{data.frame}.

<<>>=
G <- hyperframe(X=1:3, Y=letters[1:3], Z=factor(letters[1:3]),
                W=list(rpoispp(100),rpoispp(100), rpoispp(100)),
                U=42,
                V=rpoispp(100), stringsAsFactors=FALSE)
G
@ 

This hyperframe has 3 rows and 6 columns. 
The columns named \texttt{U} and \texttt{V}
are constant (all entries in a column are the same). The column named
\texttt{Y} is a character vector while \texttt{Z} is a factor.

\subsection{Hyperframes of data}

To analyse an experiment, we will store {\bf all} the data from the experiment
in a single hyperframe. The rows of the hyperframe will correspond 
to different experimental units,
while the columns represent different variables 
(response variables or covariates).

Several examples of hyperframes are provided with the package,
including \texttt{demohyper}, \texttt{flu}, \texttt{simba}
and \texttt{pyramidal}, described above.

The \texttt{simba} dataset contains simulated data from an
experiment with a `control' group and a `treatment' group, each 
group containing 5 experimental units. The responses in the control group are 
independent Poisson point patterns with intensity 80.
The responses in the treatment group are independent realisations of
a Strauss process (see \texttt{help(simba)} for details).
The \texttt{simba} dataset is a hyperframe with 10 rows and 2 columns:
\texttt{Points} (the point patterns) and \texttt{group} (a factor 
with levels \texttt{control} and \texttt{treatment}).

<<>>=
simba
@ 

The \texttt{pyramidal} dataset contains data from Diggle, Lange and Benes 
\cite{digglangbene91} on the locations of pyramidal neurons in 
human brain. One point pattern was observed in each of 31 human subjects.
The subjects were classified
into 3 groups (controls, schizoaffective and schizophrenic).
The \texttt{pyramidal} dataset is a hyperframe with 31 rows
and 2 columns: \code{Neurons} (the point patterns) and \code{group}
(a factor with levels \texttt{control}, \texttt{schizoaffective} 
and \texttt{schizophrenic}).

<<>>=
pyramidal
@ 

The \texttt{waterstriders} dataset is not a hyperframe; it's just a 
list of point patterns. It can easily be converted into a hyperframe:

<<>>=
ws <- hyperframe(Striders=waterstriders)
@ 

\subsection{Columns of a hyperframe}

Individual columns of a hyperframe can be extracted using \verb!$!:

<<>>=
H$X
H$Y
@ 

The result of \verb!$! is a vector or factor if the column contains
atomic values; otherwise it is a list of objects (with class \texttt{"listof"}
to make it easier to print and plot).

Individual columns can also be assigned (overwritten or created)
using \verb!$<-!:

<<>>=
H$U <- letters[1:3]
H
@ 

This can be used to build up a hyperframe column-by-column:

<<>>=
G <- hyperframe()
G$X <- waterstriders
G$Y <- 1:3
G
@ 

\subsection{Subsets of a hyperframe}

Other subsets of a hyperframe
can be extracted with \verb![!:

<<>>=
H[,1]
H[2,]
H[2:3, ]
H[1,1]
@ 

The result of \verb![! is a hyperframe, unless you set \verb!drop=TRUE!
and the subset consists of only one element or one column:

<<>>=
H[,1,drop=TRUE]
H[1,1,drop=TRUE]
H[1,2,drop=TRUE]
@ 

There is also a method for \verb![<-! that allows
you to assign values to a subset of a hyperframe.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Plotting}

\subsection{Plotting a \code{listof} object}

The plot method for \code{listof} objects has formal arguments

<<eval=FALSE>>=
plot.listof(x, ..., main, arrange = TRUE, nrows = NULL, ncols = NULL)
@ 

where \code{main} is a title for the entire page.

If \code{arrange=TRUE} then the entries of the list are displayed  
in separate panels on the same page (with \code{nrows} rows and
\code{ncols} columns of panels), while if \code{arrange=FALSE} then 
the entries are just plotted as a series of plot frames.

The extra arguments \verb!...! control the individual plot panels.
These arguments will be passed to the plot method
that displays each entry of the list. Suitable arguments depend on the
type of entries.

<<fig=TRUE>>= 
plot(waterstriders, pch=16, nrows=1)
@ 

\subsection{Plotting a hyperframe}

\subsubsection{Plotting one column}

If \code{h} is a hyperframe, then the default action of
\code{plot(h)} is to extract the first column
of \code{h} and plot each of the entries in a separate panel on 
one page (actually using the plot method for class \verb!"listof"!). 

\SweaveOpts{width=7,height=5}
\setkeys{Gin}{width=0.9\textwidth}

<<fig=TRUE>>=
plot(simba)
@ 

This only works if the entries in the first column are objects 
for which a plot method is defined (for example, point patterns, images,
windows). 

To select a different column, use \verb!$! or \verb![!:

\SweaveOpts{width=6,height=2}
\setkeys{Gin}{width=0.9\textwidth}

<<fig=TRUE>>=
H <- hyperframe(X=1:3, Y=list(sin,cos,tan))
plot(H$Y)
@ 

The plot can be controlled using the arguments for \code{plot.listof}
(and, in this case, \code{plot.function}, since \verb!H$Y! consists of 
functions).

\subsubsection{Complex plots}

More generally, we can display any kind of higher-order plot
involving one or more columns of a hyperframe:

<<eval=FALSE>>=
plot(h, e)
@ 

where \code{h} is a hyperframe and \code{e} is an \R\ language call 
or expression that must be evaluated in each row to generate each plot panel.

\SweaveOpts{width=9,height=5}
\setkeys{Gin}{width=0.9\textwidth}

<<fig=TRUE>>=
plot(demohyper, quote({ plot(Image, main=""); plot(Points, add=TRUE) }))
@ 

Note the use of \code{quote}, which prevents the code
inside the braces from being evaluated immediately.

To plot the $K$-functions of each of the patterns in the
\code{waterstriders} dataset,

\SweaveOpts{width=6,height=2}
\setkeys{Gin}{width=0.9\textwidth}

<<fig=TRUE>>=
H <- hyperframe(Bugs=waterstriders)
plot(H, quote(plot(Kest(Bugs))), marsize=1)
@ 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Data analysis}

\subsection{Computing with hyperframes}

Often we want to perform some computation on each row of
a hyperframe. 

In a data frame, this can be done using the command \code{with}:

<<>>=
df <- data.frame(A=1:10, B=10:1)
with(df, A-B)
@ 

In this example, the expression \code{A-B} is evaluated
in each row of the data frame, and the result is a vector
containing the computed values for each row.
The function \code{with} is generic, and has a method for data frames,
\code{with.data.frame}. The computation above
was executed by \code{with.data.frame}. 

The same syntax is available for hyperframes
using the method \code{with.hyperframe}:

<<eval=FALSE>>=
with(h,e)
@ 

Here \code{h} is a hyperframe, and 
\code{e} is an {\sf R} language construct involving the names
of columns in \code{h}.
For each row of \code{h}, the expression \code{e} will be evaluated
in such a way that each entry in the row is identified by its
column name. 

<<>>=
H <- hyperframe(Bugs=waterstriders)
with(H, npoints(Bugs))
with(H, distmap(Bugs))
@

The result of \code{with.hyperframe} 
is a list of objects (of class \verb!"listof"!),
or a vector or factor if appropriate.

Notice that (unlike the situation for data frames)
the operations in the expression \code{e} do not have to
be vectorised. For example, \code{distmap} expects 
a single point pattern, and is not vectorised to deal with 
a list of point patterns. Instead, the expression \code{distmap(Bugs)}
is evaluated separately in each row of the hyperframe.

\subsection{Summary statistics}

One application of \code{with.hyperframe} is to calculate summary statistics
for each row of a hyperframe.

For example, the number of points in a point pattern \code{X}
is returned by \code{npoints(X)}. To calculate this for each of the 
responses in the \code{simba} dataset,

<<>>=
with(simba, npoints(Points))
@ 

The summary statistic can be any kind of object. For example, to
compute the empirical $K$-functions for each of the 
patterns in the \code{waterstriders} dataset,

<<>>=
H <- hyperframe(Bugs=waterstriders)
K <- with(H, Kest(Bugs))
@ 

To plot these $K$-functions you can then just type 

\SweaveOpts{width=6,height=2}
\setkeys{Gin}{width=0.9\textwidth}

<<fig=TRUE>>=
plot(K)
@ 

The summary statistic for each row could be a numeric vector:

<<>>=
H <- hyperframe(Bugs=waterstriders)
with(H, nndist(Bugs))
@ 

The result is a list, each entry being a vector of nearest neighbour distances.
To find the minimum interpoint distance in each pattern:

<<>>=
with(H, min(nndist(Bugs)))
@ 

\subsection{Generating new columns}

New columns of a hyperframe can be created by computation
from the existing columns. 

For example, I can add a new column to the \code{simba} dataset
that contains pixel images of the distance maps for each of the
point pattern responses. 

<<fig=FALSE>>=
simba$Dist <- with(simba, distmap(Points))
@ 

\subsection{Simulation}


This can be useful for simulation. For example, to generate
Poisson point patterns with different intensities, where the 
intensities are given by a numeric vector \code{lambda}:

\SweaveOpts{width=6,height=6}
\setkeys{Gin}{width=0.7\textwidth}

<<fig=TRUE>>=
lambda <- rexp(6, rate=1/50)
H <- hyperframe(lambda=lambda)
H$Points <- with(H, rpoispp(lambda))
plot(H, quote(plot(Points, main=paste("lambda=", signif(lambda, 4)))))
@ 

It's even simpler to generate 10 independent Poisson point patterns
with the \emph{same} intensity 50, say:

<<fig=FALSE>>=
H$X <- with(H, rpoispp(50))
@ 

\noindent The expression \code{rpoispp(50)} is evaluated once in each row,
yielding a different point pattern in each row because of the
randomness.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Exploratory data analysis}

Before fitting models to the data, it is prudent to explore 
the data to detect unusual features and to suggest appropriate 
models. 

\subsection{Exploring spatial trend and covariate effects}

Points may be distributed non-uniformly either because they are
intrinsically non-uniform (``spatial trend'') or because their abundance
depends on a spatial covariate (``covariate effects''). 

Non-uniformity of a point pattern can be investigated using 
the kernel smoothed intensity. This is the convolution of the point pattern
with a smooth density called the kernel. Effectively each point
in the pattern is replaced by a copy of the kernel, and the sum of all
copies of the kernel is the kernel-smoothed intensity function.
It is computed by \texttt{density.ppp} separately for each point pattern.

<<fig=TRUE>>=
plot(simba, quote(plot(density(Points), main="")), nrows=2)
@ 

Covariate effects due to a real-valued spatial covariate (a real-valued
pixel image) can be investigated
using the command \code{rhohat}. This uses a kernel smoothing
technique to fit a model of the form 
\[
           \lambda(u) = \rho(Z(u))
\]
where $\lambda(u)$ is the point process intensity at a location $u$,
and $Z(u)$ is the value of the spatial covariate at that location.
Here $\rho$ is an unknown, smooth function which is to be estimated.
The function $\rho$ expresses the effect of the
spatial covariate on the point process intensity. If $\rho$ turns out to
be constant, then the covariate has no effect on point process intensity
(and the constant value of $\rho$ is the constant intensity of the
point process). 

<<fig=TRUE>>=
rhos <- with(demohyper, rhohat(Points, Image))
plot(rhos)
@ 

\SweaveOpts{width=6,height=4}
\setkeys{Gin}{width=0.9\textwidth}

\subsection{Exploring interpoint interaction}

Still to be written.
See Chapter 16 of \cite{baddrubaturn15}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Fitting models of spatial trend}

The command \code{mppm} fits models to multiple point patterns.
Its syntax is very similar to that of \code{lm} and \code{glm}:

<<eval=FALSE>>=
mppm(formula, data, interaction, ...)
@ 

where \code{formula} is a formula describing the systematic trend
part of the model, \code{data} is a hyperframe containing all the data 
(responses and covariates), and \code{interaction} determines the
stochastic interpoint interaction part of the model.

For example:

<<eval=FALSE>>=
mppm(Points ~ group, simba, Poisson())
@ 

Note that the formula has a left hand side, which identifies
the response. This should be the name of a column of \code{data}.

\subsection{Trend formula}

The right side of \code{formula} is an expression for the
linear predictor (effectively the {\bf logarithm} 
of the spatial trend). 

The variables appearing in the right hand side
of \code{formula} should be either
\begin{itemize}
\item names of columns in \code{data}
\item objects in the {\sf R} global
environment (such as \code{pi} and \code{log})
\item the reserved names \code{x}, \code{y} 
(representing Cartesian coordinates), \code{marks} (representing mark values
attached to points) or \code{id} (a factor representing the row number
in the hyperframe).
\end{itemize}

\subsubsection{Design covariates}

The variables in the trend could be `design covariates'.

For example, to fit a model to the \code{simba} dataset
in which all patterns are independent replicates of the
same uniform Poisson process, with the same constant
intensity:

<<>>=
mppm(Points ~ 1, simba)
@ 

To fit a model in which the two groups of patterns (control and treatment
groups) each consist of independent replicates of a uniform Poisson process,
but with possibly different intensity in each group:

<<>>=
mppm(Points ~ group, simba)
@ 

To fit a uniform Poisson process to each pattern, with 
different intensity for each pattern:

<<>>=
mppm(Points ~ id, simba)
@ 

\subsubsection{Spatial covariates}

The variables in the trend could be `spatial covariates'.

For example, the \code{demohyper} dataset has a column \code{Image}
containing pixel images. 

<<>>=
mppm(Points ~ Image, data=demohyper)
@ 

This model postulates that each pattern is a Poisson process
with intensity of the form
\[
      \lambda(u) = \exp(\beta_0 + \beta_1 Z(u))
\]
at location $u$, where $\beta_0, \beta_1$ are coefficients
to be estimated, and $Z(u)$ is the value of the pixel image
\code{Image} at location $u$.

It may or may not be appropriate to assume that the intensity of the points
is an exponential function of the image pixel value $Z$.
If instead 
we wanted the intensity $\lambda(u)$ to be \emph{proportional} to $Z(u)$,
the appropriate model is 

<<eval=FALSE>>=
mppm(Points ~ offset(log(Image)), data=demohyper)
@ 

which corresponds to an intensity proportional to \code{Image},
\[
      \lambda(u) = \exp(\beta_0 + \log Z(u)) = e^{\beta_0} \; Z(u).
\]
The \code{offset} indicates that there is no coefficient in front
of $\log Z(u)$. 

Alternatively we could allow a coefficient:

<<eval=FALSE>>=
mppm(Points ~ log(Image), data=demop)
@ 

which corresponds to a gamma transformation of \code{Image},
\[
      \lambda(u) = \exp(\beta_0 + \beta_1 \log Z(u)) 
      = e^{\beta_0} \; Z(u)^{\beta_1}.
\]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Interpoint interaction}

The stochastic interpoint interaction in a point process model
is specified by the arguments \code{interaction} and (optionally)
\code{iformula} in 

<<eval=FALSE>>=
mppm(formula, data, interaction, ..., iformula=NULL)
@ 

\subsection{Same interaction for all patterns}

In the simplest case,
the argument \texttt{interaction} is one of the familiar 
objects that describe the point process interaction structure.
It is an object of class \texttt{"interact"}
created by calling one of the functions

\begin{center}
  \begin{tabular}{rl}
    \texttt{Poisson()}  & the Poisson point process\\     
    \texttt{Hardcore()}  & the hard core process \\ 
    \texttt{Strauss()}  & the Strauss process \\ 
    \texttt{StraussHard()}  & the Strauss/hard core point process\\ 
    \texttt{Softcore()}  & pairwise interaction, soft core potential\\  
    \texttt{PairPiece()}  & pairwise interaction, piecewise constant \\ 
    \texttt{DiggleGatesStibbard() }  & Diggle-Gates-Stibbard pair potential \\ 
    \texttt{DiggleGratton() }  & Diggle-Gratton pair potential \\ 
    \texttt{Fiksel() }  & Fiksel pair potential \\ 
    \texttt{LennardJones() }  & Lennard-Jones pair potential \\ 
    \texttt{Pairwise()}  &	pairwise interaction, user-supplied potential\\ 
    \texttt{AreaInter()}  &		area-interaction potential\\ 
    \texttt{Geyer()}	  & Geyer's saturation process\\ 
    \texttt{BadGey()}	  & multiscale Geyer saturation process\\ 
    \texttt{Saturated()}  &	Saturated pair model, user-supplied potential\\ 
    \texttt{OrdThresh()}  &		Ord process, threshold potential\\ 
    \texttt{Ord()}  & 		        Ord model, user-supplied potential \\ 
    \texttt{MultiStrauss()}  & 		multitype Strauss process \\ 
    \texttt{MultiStraussHard()}  & 	multitype Strauss/hard core process \\ 
    \texttt{Concom()}  &	connected component interaction \\ 
    \texttt{Hybrid()}  &	hybrid of several interactions \\ 
  \end{tabular}
\end{center}

In this `simple' usage of \texttt{mppm}, the point process model
assumes that all point patterns have exactly the same interpoint interaction,
(with the same interaction parameters), and only differ in their spatial trend.

\subsection{Hyperframe of interactions}

More generally the argument \code{interaction} can be a hyperframe
containing objects of class \texttt{"interact"}. 

For example, we
might want to fit a Strauss process to each point pattern, but with
a different Strauss interaction radius for each pattern. 

<<echo=FALSE,results=hide>>=
radii <- with(simba, mean(nndist(Points)))
@ 

Then \code{radii} is a vector of numbers which we could use
as the values of the interaction radius for each case. 
First we need to make the interaction objects:

<<>>=
Rad <- hyperframe(R=radii)
Str <- with(Rad, Strauss(R))
@ 

Then we put them into a hyperframe and fit the model:

<<>>=
Int <- hyperframe(str=Str)
mppm(Points ~ 1, simba, interaction=Int)
@ 

An important constraint is that all of the interaction objects 
in one column must be \emph{instances of the same process} (e.g. Strauss)
albeit possibly having different parameter values. For example, you cannot
put Poisson and Strauss processes in the same column. 

\subsection{Interaction formula}

If \code{interaction} is a hyperframe, then 
the additional argument \code{iformula} may be used to
fully specify the interaction.

(An \code{iformula} is also required if \code{interaction}
has more than one column.)

The \code{iformula} should be a formula without a left hand side.
Variables on the right hand side are typically the names of
columns in \code{interaction}.

\subsubsection{Selecting one column}

If the right hand side of \code{iformula} is a single name,
then this identifies the column in \code{interaction}
to be used as the interpoint interaction structure.

<<>>=
h <- hyperframe(Y=waterstriders)
g <- hyperframe(po=Poisson(), str4 = Strauss(4), str7= Strauss(7))
mppm(Y ~ 1, data=h, interaction=g, iformula=~str4)
@ 

\subsubsection{Interaction depending on design} 

The \code{iformula} can also involve columns of \code{data}, but
only those columns that are vectors or factors. This allows us to
specify an interaction that depends on the experimental design.
[This feature is {\bf experimental}.]
For example

<<>>=
fit <- mppm(Points ~ 1, simba, Strauss(0.07), iformula = ~Interaction*group)
@ 

Since \code{Strauss(0.1)} is not a hyperframe, it is first converted
to a hyperframe with a single column named \code{Interaction}.

The \code{iformula = ~Interaction*group} specifies (since \code{group}
is a factor) that the interpoint interaction shall have a different 
coefficient in each experimental group. That is, we fit a model
which has two different values for the Strauss interaction parameter $\gamma$,
one for the control group and one for the treatment group.

When you print the result of such a fit, 
the package tries to 
do `automatic interpretation' of the fitted model (translating the
fitted interaction coefficients into meaningful numbers like $\gamma$). 
This will be successful in \emph{most} cases:

<<>>=
fit
@ 

<<echo=FALSE,results=hide>>=
co <- coef(fit)
si <- function(x) { signif(x, 4) }
@ 

Thus we see that the estimate of the Strauss parameter $\gamma$ 
for the control group is \Sexpr{si(exp(co[2]))}, and 
for the treatment group \Sexpr{si(exp(sum(co[c(2,4)])))}
(the correct values in this simulated dataset were $1$ and $0.5$).

The fitted model can also be interpreted directly from the fitted
canonical coefficients:

<<>>=
coef(fit)
@ 

The last output shows all the coefficients $\beta_j$
in the linear predictor for the (log) conditional intensity. 

The interpretation of the model coefficients, for any fitted model in \R,
depends on the \emph{contrasts} which were applicable when the model was
fitted. This is part of the core {\sf R} system: see \code{help(contrasts)}
or \code{options(contrasts)}. If you did not specify otherwise, 
the default is to use \emph{treatment contrasts}. This means that,
for an explanatory variable which is a \texttt{factor} with $N$ levels, 
the first level of the factor is used as a baseline, and the
fitted model coefficients represent the factor levels $2, 3, \ldots, N$
relative to this baseline. 

In the output above, there is a coefficient for \code{(Intercept)} 
and one for \code{grouptreatment}. These are coefficients related to
the \code{group} factor. According to the ``treatment contrasts'' rule, 
the \code{(Intercept)} coefficient is
the estimated effect for the control group, and the
\code{grouptreatment} coefficient is the estimated difference between
the treatment and control groups. Thus the fitted first order
trend is $\exp(\Sexpr{si(co[1])}) = \Sexpr{si(exp(co[1]))}$ 
for the control group
and $\exp(\Sexpr{si(co[1])} + \Sexpr{si(co[3])}) 
 = \Sexpr{si(exp(sum(co[c(1,3)])))}$ for the treatment group.
The correct values in this simulated dataset were 
$80$ and $100$.

The remaining coefficients in the output are \code{Interaction}
and \code{Interaction:grouptreatment}. Recall that the Strauss process
interaction term 
is $\gamma^{t(u,\xx)} = \exp(t(u,\xx) \log\gamma)$
at a spatial location $u$, for a point pattern $\xx$. 
Since we're using treatment contrasts, the coefficient 
\code{Interaction} is the estimate of 
$\log\gamma$ for the control group.
The coefficient  \code{Interaction:grouptreatment} is the 
estimate of the difference in $\log\gamma$ between the 
treatment and control groups. Thus the estimated Strauss interaction
parameter $\gamma$ is $\exp(\Sexpr{si(co[2])}) = \Sexpr{si(exp(co[2]))}$ 
for the control group and 
$\exp(\Sexpr{si(co[2])} + (\Sexpr{si(co[4])})) = \Sexpr{si(exp(co[2]+co[4]))}$ 
for the treatment group.
The correct values were $1$ and $0.5$.

\subsubsection{Completely different interactions for different cases} 

In the previous example, when we fitted a Strauss model to all 
point patterns in the \code{simba} dataset, the fitted model for the
patterns in the control group was close to Poisson ($\gamma \approx 1$).
Suppose we now want to fit a model which {\it is}
Poisson in the control group, and Strauss in the treatment group. 
The Poisson and Strauss interactions must be given as separate columns
in a hyperframe of interactions:

<<eval=FALSE>>=
interaction=hyperframe(po=Poisson(), str=Strauss(0.07))
@ 

What do we write for the 
\code{iformula}? The following \emph{will not} work:
<<eval=FALSE>>=
iformula=~ifelse(group=="control", po, str)
@ 
This does not work because the Poisson and Strauss models are `incompatible'
inside such expressions. The canonical sufficient statistics for
the Poisson and Strauss processes do not have the same dimension. 
Internally in \code{mppm} we translate the symbols \code{po} and \code{str}
into matrices; the dimensions of these matrices are different,
so the \code{ifelse} expression cannot be evaluated.

Instead we need something like the following:
<<eval=FALSE>>=
iformula=~I((group=="control")*po) + I((group=="treatment") * str)
@ 
The letter \code{I} here is a standard R function that prevents its argument
from being interpreted as a formula (thus the \code{*} is interpreted
as multiplication instead of a model interaction). The expression
\code{(group=="control")} is logical, and when multiplied by the matrix
\code{po}, yields a matrix.

So the following does work:

<<>>=
g <- hyperframe(po=Poisson(), str=Strauss(0.07))
fit2 <- mppm(Points ~ 1, simba, g, 
             iformula=~I((group=="control")*po) 
                     + I((group=="treatment") * str))
fit2
@ 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%#%^!ifdef RANDOMEFFECTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Random effects}

\subsection{Mixed effects models}

It is also possible to fit models that include `random effects'.
Effectively, some of the coefficients
in the model are assumed to be Normally-distributed random variables
instead of constants.

\subsubsection{Mixed Poisson model}

Consider the simplest model of a uniform Poisson process which we fitted to
the 3 point patterns of waterstriders. It might be sensible to assume that
each pattern is a realisation of a Poisson process, but with 
{\em random intensity\/}. In each realisation the intensity $\lambda$
is constant across different locations, but it
is a different, random value in different realisations.
This example is called a `mixed Poisson process' and belongs to the
class of `Cox processes' (Poisson processes with random intensity 
functions).
Let's assume further that the log-intensity 
is a Normal random variable.
Then the model is a (very degenerate) special case of
a `log-Gaussian Cox process'. 

To fit such a model we use the standard techniques of mixed effects
models \cite{lairware82,davigilt95,pinhbate00}.
The mixed Poisson process which we discussed above would
be written in standard form
\begin{equation}
\label{mixPois}
   \lambda_i(u) = \exp(\mu + Z_i)
\end{equation}
for the $i$th point pattern, where $\mu$ is a parameter to be estimated 
(the `fixed effect')
and $Z_i \sim N(0, \sigma^2)$ is a zero-mean Normal random variable
(the `random effect' for point pattern $i$). In the simplest case we
would assume that $Z_1, \ldots, Z_n$ are independent.
The variance $\sigma^2$ of the random effects
would be estimated. One can also estimate the individual realised values
$z_i$ of the random effects for each point pattern, although these are
usually not of such great interest. Since the model includes both
fixed and random effects, it is called a ``mixed-effects'' model.

\subsubsection{Dependence structure}

When we formulate a random-effects or mixed-effects model, we must 
specify the dependence structure of the random effects. In the model above
we assumed that the $Z_i$ are independent for all point patterns $i$. 
If the experiment consists of two groups, we could alternatively assume
that $Z_i = Z_j$ whenever $i$ and $j$ belong to the same group. In other words
all the patterns in one group have the same value of the random effect.
So the random effect is associated with the group rather than with
individual patterns. This could be appropriate if, for example, 
the groups represent different
batches of a chemical. Each batch is prepared under slightly different 
conditions so we believe that there are random variations between batches,
but within a batch we believe that the chemical is well-mixed.

\subsubsection{Random effects are coefficients}

In the mixed Poisson model (\ref{mixPois}), 
the random effect is an additive constant
(with a random value) in the log-intensity. In general, a
random effect is a \emph{coefficient} of one of the covariates.
For example if $v$ is a real-valued design covariate (e.g. `temperature'), 
with value $v_i$ for the $i$th point pattern, then we could assume
\begin{equation}
\label{ranef2}
   \lambda_i(u) = \exp(\mu + Z_i v_i)
\end{equation}
where $Z_i \sim N(0, \sigma^2)$ are independent for different $i$.
This model has a random effect in the dependence on $v$.

We could also have a random effect for a spatial covariate $V$.
Suppose $V_i$ is a real-valued image for the $i$th pattern 
(so that $V_i(u)$ is the value of some covariate at the location $u$
for the $i$th case). Then we could assume
\begin{equation}
\label{ranef3}
   \lambda_i(u) = \exp(\mu + Z_i V_i(u))
\end{equation}
where $Z_i \sim N(0, \sigma^2)$ are independent for different $i$.
This kind of random effect would be appropriate if, for example, the 
images $V_i$ are not `normalised' or `standardised' relative to each 
other (e.g.\ they are images taken under different illumination). Then the
coefficients $Z_i$ effectively include the rescaling necessary to standardise
the images.

\subsection{Fitting a mixed-effects model}

The call to \texttt{mppm} can also include the argument
\texttt{random}. This should be a formula (with no left-hand side) 
describing the structure of random effects. 

The formula for random effects
must be recognisable to \texttt{lme}. It is typically of the form
\begin{verbatim}
      ~x1 + ... + xn | g
\end{verbatim}
or
\begin{verbatim}
      ~x1 + ... + xn | g1/.../gm
\end{verbatim}
where \verb!x1 + ... + xn! specifies the covariates for the random effects
and \texttt{g} or \verb!g1/.../gm! determines the grouping (dependence) 
structure. Here \code{g} or \code{g1, \ldots, gm} should be factors.

To fit the mixed Poisson model (\ref{mixPois}) to the waterstriders,
we want to have a random intercept coefficient
(so \texttt{x} is \texttt{1}) that varies for different point patterns
(so \texttt{g} is \texttt{id}). 
The reserved name \code{id} is a factor referring to the individual
point pattern. Thus

<<>>=
H <- hyperframe(P=waterstriders)
mppm(P ~ 1, H, random=~1|id)
@ 

To fit the mixed effects model (\ref{ranef2}) to the coculture data
with the \code{AstroIm} covariate, with a random effect associated
with each well,

<<eval=FALSE>>=
mppm(Neurons ~ AstroIm, random=~AstroIm|WellNumber)
@ 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%#%^!endif
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Studying the fitted model}

Fitted models produced by \code{mppm}
can be examined and validated in many ways.

\subsection{Fits for each pattern}


\subsubsection{Subfits}

The command \code{subfits} takes an \code{mppm} object
and extracts, for each individual point pattern, 
the fitted point process model for that pattern
\emph{that is implied by the overall fit}. It returns a list of 
objects of class \code{ppm}. 

<<>>=
H <- hyperframe(W=waterstriders)
fit <- mppm(W ~ 1, H)
subfits(fit)
@ 

In this example the result is a list of three \code{ppm} objects
representing the implied fits for each of the three point patterns
in the \code{waterstriders} dataset.
Notice that {\bf the fitted coefficients are the same} in all three 
models. 

Note that there are some unresolved difficulties with the implementation of
\code{subfits}. Two completely different implementations are supplied
in the package; they are called \code{subfits.old} 
%(used in versions 0.1--1 and earlier) 
and \code{subfits.new}.% (introduced in 0.1--2).
The old version would occasionally crash. 
Unfortunately the newer version \code{subfits.new} is quite memory-hungry
and sometimes causes R to hang.
We're still working on this problem. So for the time being,
\code{subfits} is the same as \code{subfits.old}. You can change this
simply by reassigning, e.g.

<<eval=FALSE>>=
subfits <- subfits.new
@ 

\subsubsection{Fitting separately to each pattern} 

For comparison, we could fit a point process model separately
to each point pattern dataset using \code{ppm}. The easy way to do this is 
with \code{with.hyperframe}.

To fit a \emph{separate} uniform Poisson 
point process to each of the three waterstriders patterns,

<<>>=
H <- hyperframe(W=waterstriders)
with(H, ppm(W))
@ 

The result is again a list of three fitted point process models
(objects of class \code{ppm}), but now the fitted coefficients
are different.

\subsection{Residuals}

One standard way to check a fitted model
is to examine  the residuals. 

\subsubsection{Point process residuals}

Some recent papers \cite{baddetal05,baddmollpake08} have defined
residuals for a fitted point process model (fitted to a \emph{single}
point pattern). These residuals are implemented in \code{spatstat}
as \code{residuals.ppm} and apply to an object of class \code{ppm}, 
that is, a model fitted to a \emph{single} point pattern. 

The command \code{residuals.mppm} computes the point process residuals
for an \code{mppm} object. 

<<>>=
fit <- mppm(P ~ x, hyperframe(P=waterstriders))
res <- residuals(fit)
@ 

The result is a list, with one entry for each of the point pattern
datasets. Each list entry contains the point process residuals 
for the corresponding point pattern dataset.
Each entry in the list is a signed measure (object of class \code{"msr"})
as explained in the help for \code{residuals.ppm}).
It can be plotted:

<<fig=TRUE>>=
plot(res)
@ 

You probably want the smoothed residual field:

<<fig=TRUE>>=
smor <- with(hyperframe(res=res), Smooth(res, sigma=4))
plot(smor)
@ 

\subsubsection{Sums of residuals}

It would be useful to have a residual that is a single value
for each point pattern (representing how much that point pattern
departs from the model fitted to all the point patterns).

That can be computed by \emph{integrating} the residual measures
using the function \code{integral.msr}:

<<>>=
fit <- mppm(P ~ x, hyperframe(P=waterstriders))
res <- residuals(fit)
totres <- sapply(res, integral.msr)
@ 

In designed experiments we can plot these total residuals against
the design covariates:

<<fig=TRUE>>=
fit <- mppm(Points~Image, data=demohyper)
resids <- residuals(fit, type="Pearson")
totres <- sapply(resids, integral.msr)
areas <- with(demohyper, area.owin(as.owin(Points)))
df <- as.data.frame(demohyper[, "Group"])
df$resids <- totres/areas
plot(resids~Group, df)
@ 

\subsubsection{Four-panel diagnostic plots} 

Sometimes a more useful tool is the function \code{diagnose.ppm}
which produces a four-panel diagnostic plot based on the 
point process residuals. However, it is only available for 
\code{ppm} objects.

To obtain a four-panel diagnostic plot for each of the 
point patterns, do the following:

\begin{enumerate}
\item fit a model to multiple point patterns using \code{mppm}.
\item extract the individual fits using \code{subfits}.
\item plot the residuals of the individual fits.
\end{enumerate}

For example:

<<fig=TRUE>>=
fit <- mppm(P ~ 1, hyperframe(P=waterstriders))
sub <- hyperframe(Model=subfits(fit))
plot(sub, quote(diagnose.ppm(Model)))
@ 

(One could also do this for models fitted separately to the 
individual point patterns.)

\subsubsection{Residuals of the parameter estimates}

We can also compare the parameter estimates obtained 
by fitting the model simultaneously to all patterns (using \code{mppm})
with those obtained by fitting the model separately to each 
pattern (using \code{ppm}). 

<<>>=
H <- hyperframe(P = waterstriders)
fitall <- mppm(P ~ 1, H)
together <- subfits(fitall)
separate <- with(H, ppm(P))
Fits <- hyperframe(Together=together, Separate=separate)
dr <- with(Fits, unlist(coef(Separate)) - unlist(coef(Together)))
dr
exp(dr)
@ 

One could also try deletion residuals, etc.

\subsection{Goodness-of-fit tests} 

\subsubsection{Quadrat count test}

The $\chi^2$ goodness-of-fit test based on quadrat counts is implemented
for objects of class \code{ppm} (in \code{quadrat.test.ppm})
and also for objects of class \code{mppm} (in \code{quadrat.test.mppm}).

This is a goodness-of-fit test for a fitted {\bf Poisson} point process
model only. The model could be uniform or non-uniform and the intensity
might depend on covariates. 

<<>>=
H <- hyperframe(X=waterstriders)

# Poisson with constant intensity for all patterns
fit1 <- mppm(X~1, H)
quadrat.test(fit1, nx=2)

# uniform Poisson with different intensity for each pattern
fit2 <- mppm(X ~ id, H)
quadrat.test(fit2, nx=2)
@ 

See the help for \code{quadrat.test.ppm} and \code{quadrat.test.mppm}
for further details.

\subsubsection{Kolmogorov-Smirnov test}

The Kolmogorov-Smirnov test of goodness-of-fit of a Poisson
point process model compares the observed and predicted
distributions of the values of a spatial covariate.

We want to test the null hypothesis $H_0$ that the observed point pattern
${\mathbf x}$ is a realisation from the Poisson process with intensity 
function $\lambda(u)$ (for locations $u$ in the window $W$).
Let $Z(u)$ be a given, real-valued covariate defined at each spatial location
$u$. Under $H_0$, the \emph{observed} values of $Z$ at the 
data points, $Z(x_i)$ for each $x_i \in {\mathbf x}$, are independent
random variables with common probability distribution function
\[
    F_0(z) = \frac{\int_W \lambda(u) \indicate{Z(u) \le z} \dee u}
                {\int_W \lambda(u)                       \dee u}.
\]
We can therefore apply the Kolmogorov-Smirnov test 
of goodness-of-fit. This compares the empirical cumulative distribution of
the observed values $Z(x_i)$ to the predicted c.d.f. $F_0$.

The test is implemented as \code{kstest.ppm}. The syntax is 

<<eval=FALSE>>=
kstest.mppm(model, covariate)
@ 

where \code{model} is a fitted model (of class \texttt{"mppm"})
and \code{covariate} is either 
\begin{itemize}
\item a \code{function(x,y)} making it possible to compute the value 
of the covariate at any location \code{(x,y)}
\item a pixel image containing the covariate values 
\item a list of functions, one for each row of the hyperframe of
original data
\item a list of pixel images, one for each row of the hyperframe of 
original data
\item a hyperframe with one column containing either functions or 
pixel images.
\end{itemize}

See Chapter 16 of \cite{baddrubaturn15} for further information.

\newpage

\addcontentsline{toc}{section}{Bibliography}

%\bibliography{%
%extra,%
%extra2,%
%biblio/badd,%
%biblio/bioscience,%
%biblio/censoring,%
%biblio/mcmc,%
%biblio/spatstat,%
%biblio/stat,%
%biblio/stochgeom%
%}

\begin{thebibliography}{1}

\bibitem{baddmollpake08}
A. Baddeley, J. M{\o}ller, and A.G. Pakes.
\newblock Properties of residuals for spatial point processes.
\newblock {\em Annals of the Institute of Statistical Mathematics},
  60:627--649, 2008.

\bibitem{TheBook}
A. Baddeley, E. Rubak, and R. Turner.
\newblock {\em Spatial Point Patterns: Methodology and Applications with R}.
\newblock Chapman \& Hall/CRC Press, 2015.

\bibitem{statpaper}
A. Baddeley, I. Sintorn, L. Bischof, R. Turner, and S. Heggarty.
\newblock Analysing designed experiments where the response is a spatial point
  pattern.
\newblock In preparation.

\bibitem{baddetal05}
A. Baddeley, R. Turner, J. M{\o}ller, and M. Hazelton.
\newblock Residual analysis for spatial point processes (with discussion).
\newblock {\em Journal of the Royal Statistical Society, series B},
  67(5):617--666, 2005.

\bibitem{chenetal08}
B.J. Chen, G.P. Leser, D. Jackson, and R.A. Lamb.
\newblock The influenza virus {M2} protein cytoplasmic tail interacts with the
  {M1} protein and influences virus assembly at the site of virus budding.
\newblock {\em Journal of Virology}, 82:10059--10070, 2008.

%#%^!ifdef RANDOMEFFECTS  
\bibitem{davigilt95}
M. Davidian and D.M. Giltinan.
\newblock {\em Nonlinear Mixed Effects Models for Repeated Measurement Data}.
\newblock Chapman and Hall, 1995.
%#%^!endif

\bibitem{digglangbene91}
P.J. Diggle, N. Lange, and F. M. Benes.
\newblock Analysis of variance for replicated spatial point patterns in
  clinical neuroanatomy.
\newblock {\em Journal of the {A}merican {S}tatistical {A}ssociation},
  86:618--625, 1991.

%#%^!ifdef RANDOMEFFECTS  
\bibitem{lairware82}
N.M. Laird and J.H. Ware.
\newblock Random-effects models for longitudinal data.
\newblock {\em Biometrics}, 38:963--974, 1982.
%#%^!endif

\bibitem{pent84}
A. Penttinen.
\newblock {\em Modelling Interaction in Spatial Point Patterns: Parameter
  Estimation by the Maximum Likelihood Method}.
\newblock Number 7 in {Jyv\"askyl\"a} Studies in Computer Science, Economics
  and Statistics. University of {Jyv\"askyl\"a}, 1984.

%#%^!ifdef RANDOMEFFECTS  
\bibitem{pinhbate00}
J.C. Pinheiro and D.M. Bates.
\newblock {\em Mixed-Effects Models in {S} and {S-PLUS}}.
\newblock Springer, 2000.
%#%^!endif

\end{thebibliography}

%\addcontentsline{toc}{section}{Index}

%\printindex

\end{document}

\documentclass[11pt]{article}

% \VignetteIndexEntry{Getting Started with Spatstat}

<<echo=FALSE,results=hide,fig=FALSE>>=
options(SweaveHooks=list(fig=function() par(mar=c(1,1,1,1))))
@ 

\usepackage{graphicx}
\usepackage{anysize}
\marginsize{2cm}{2cm}{2cm}{2cm}

\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\bold}[1]{{\textbf {#1}}}
\newcommand{\R}{{\sf R}}
\newcommand{\spst}{\pkg{spatstat}}
\newcommand{\Spst}{\pkg{Spatstat}}

\begin{document}
\bibliographystyle{plain}
\thispagestyle{empty}
\SweaveOpts{eps=TRUE}
\setkeys{Gin}{width=0.6\textwidth}

<<echo=FALSE,results=hide>>=
library(spatstat)
spatstat.options(image.colfun=function(n) { grey(seq(0,1,length=n)) })
sdate <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
         fields = "Date")
sversion <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
         fields = "Version")
options(useFancyQuotes=FALSE)
@ 

\title{Getting started with \texttt{spatstat}}
\author{Adrian Baddeley, Rolf Turner and Ege Rubak}
\date{For \spst\ version \texttt{\Sexpr{sversion}}}
\maketitle

Welcome to \spst, a package in the \R\ language
for analysing spatial point patterns.

This document will help you to get started with \spst.
It gives you a quick overview of \spst, and some cookbook
recipes for doing basic calculations. 

\section*{What kind of data does \spst\ handle?}

\Spst\ is mainly designed for analysing \emph{spatial point patterns}.
For example, suppose you are an ecologist studying plant seedlings. 
You have pegged out a $10 \times 10$ metre rectangle for your survey.
Inside the rectangle you identify all the seedlings of the species
you want, and record their $(x,y)$ locations. You can plot the 
$(x,y)$ locations:

<<fig=TRUE,echo=FALSE,results=hide>>=
data(redwood)
plot(redwood, pch=16, main="")
@ 

This is a \emph{spatial point pattern} dataset. 

Methods for
analysing this kind of data are summarised in the
highly recommended book by Diggle \cite{digg03},
or our own book \cite{baddrubaturn15}, or other references
in the bibliography below. \nocite{handbook10,bivapebegome08}

Alternatively the points could be locations in one dimension
(such as road accidents recorded on a road network) or 
in three dimensions (such as cells observed in 3D microscopy).

You might also have recorded additional information about each seedling,
such as its height, or the number of fronds. Such information, attached to
each point in the point pattern, is called a \emph{mark} variable. For example,
here is a stand of pine trees, with each tree marked by its
diameter at breast height (dbh). The circle radii represent the dbh values
(not to scale).

<<fig=TRUE,echo=FALSE,results=hide>>=
data(longleaf)
plot(longleaf, main="")
@ 

You might also have recorded supplementary data, 
such as the terrain elevation, which might serve as explanatory variables.
These data can be in any format. \Spst\ does not usually provide 
capabilities for analysing such data in their own right, but 
\spst\ does allow such explanatory data to be taken into account
in the analysis of a spatial point pattern. 

\Spst\ is \underline{\bf not} designed to handle point data where
the $(x,y)$ locations are fixed (e.g.\ temperature records 
from the state capital cities in Australia) or where the different
$(x,y)$ points represent the same object at different times (e.g.\ 
hourly locations of a tiger shark with a GPS tag). These are different 
statistical problems, for which you need different methodology.
  
\section*{What can \spst\ do?}

\Spst\ supports a very wide range of popular techniques 
for statistical analysis for spatial point patterns,
for example 
\begin{itemize}
\item kernel estimation of density/intensity
\item quadrat counting and clustering indices
\item detection of clustering using Ripley's $K$-function
\item spatial logistic regression
\item model-fitting
\item Monte Carlo tests
\end{itemize}
as well as some advanced statistical techniques.

\Spst\ is one of the largest packages available for \R,
containing over 1000 commands. It is the product of 25 years of software 
development by leading researchers in spatial statistics.

\section*{How do I start using \spst?}

\begin{enumerate}
\item Install \R\ on your computer
  \begin{quote}
  Go to \texttt{r-project.org} and follow the installation
  instructions.
  \end{quote}
\item Install the \spst\ package in your \R\ system
  \begin{quote}
  Start \R\ and type \verb!install.packages("spatstat")!. 
  If that doesn't work, go to \texttt{r-project.org} to learn how
  to install Contributed Packages. 
  \end{quote}
\item Start \R\
\item Type \texttt{library(spatstat)} to load the package.
\item Type \texttt{help(spatstat)} for information.
\end{enumerate}

\section*{How do I get my data into \spst?}

<<echo=FALSE,results=hide>>=
data(finpines)
mypattern <- unmark(finpines)
mydata <- round(as.data.frame(finpines), 2)
@ 

Here is a cookbook example. Suppose you've recorded the 
$(x,y)$ locations of seedlings, in an Excel spreadsheet.
You should also have recorded the dimensions of the survey area
in which the seedlings were mapped. 

\begin{enumerate}
\item In Excel, save the spreadsheet into a comma-separated values (CSV) file.
\item Start \R\ 
\item Read your data into \R\ using \texttt{read.csv}.
  \begin{quote}
    If your CSV file is called \texttt{myfile.csv} then you could 
    type something like 
<<eval=FALSE>>=
mydata <- read.csv("myfile.csv")
@ 
    to read the data from the file and save them in an object called 
    \texttt{mydata} (or whatever you want to call it).
    You may need to set various options inside the \texttt{read.csv()} command
    to get this to work for your file format: 
    type \texttt{help(read.csv)} for information.
  \end{quote}
\item Check that \texttt{mydata} contains the data you expect. 
  \begin{quote}
  For example, to see the first few rows of data from the spreadsheet, type
<<>>=
head(mydata)
@ 
  To select a particular column of data, you can type
  \texttt{mydata[,3]} to extract the third column, or
  \verb!mydata$x! to extract the column labelled \texttt{x}.
  \end{quote}
\item Type \texttt{library(spatstat)} to load the \spst\ package
\item Now convert the data to a point pattern object using the
  \spst\ command \texttt{ppp}. 
  \begin{quote}
    Suppose that the \texttt{x} and \texttt{y} coordinates were 
    stored in columns 3 and 7 of the spreadsheet. Suppose that the
    sampling plot was a rectangle, with the $x$ coordinates ranging
    from 100 to 200, and the $y$ coordinates ranging from 10 to 90. Then
    you would type
<<eval=FALSE>>=
  mypattern <- ppp(mydata[,3], mydata[,7], c(100,200), c(10,90))
@ 
   The general form is 
<<eval=FALSE>>=
ppp(x.coordinates, y.coordinates, x.range, y.range)
@ 

  Note that this only stores the seedling locations. 
  If you have additional columns of data
  (such as seedling height, seedling sex, etc) these can be added as
  \emph{marks}, later.
  \end{quote}
\item Check that the point pattern looks right by plotting it:
<<fig=TRUE,results=hide>>=
plot(mypattern)
@ 
\item Now you are ready to do some statistical analysis. 
  Try the following:
  \begin{itemize}
  \item 
    Basic summary of data: type 
<<eval=FALSE>>=
summary(mypattern)
@ 
  \item 
  Ripley's $K$-function:
<<echo=FALSE,results=hide,fig=FALSE>>=
options(SweaveHooks=list(fig=function() par(mar=rep(4,4)+0.1)))
@ 
<<fig=TRUE,results=hide>>=
plot(Kest(mypattern))
@ 

For more information, type \texttt{help(Kest)}
  \item 
  Envelopes of $K$-function:
<<eval=FALSE>>=
plot(envelope(mypattern,Kest))
@ 
<<echo=FALSE,results=hide>>=
env <- envelope(mypattern,Kest, nsim=39)
@ 
<<fig=TRUE,echo=FALSE,results=hide>>=
plot(env, main="envelope(mypattern, Kest)")
@ 
<<echo=FALSE,results=hide,fig=FALSE>>=
options(SweaveHooks=list(fig=function() par(mar=c(1,1,1,1))))
@ 

  For more information, type \texttt{help(envelope)}
  \item 
  kernel smoother of point density:
<<fig=TRUE,results=hide>>=
plot(density(mypattern))
@ 

For more information, type \texttt{help(density.ppp)}
\end{itemize}
\item 
  Next if you have additional columns of data
  recording (for example) the seedling height and seedling sex,
  you can add these data as \emph{marks}. Suppose that columns 
  5 and 9 of the spreadsheet contained such values. Then do something like
<<eval=FALSE>>=
marks(mypattern) <- mydata[, c(5,9)]
@ 
<<echo=FALSE,results=hide>>=
mypattern <-finpines
@ 
Now you can try things like the kernel smoother of mark values:
<<eval=FALSE>>=
plot(Smooth(mypattern))
@ 
\setkeys{Gin}{width=0.8\textwidth}
<<fig=TRUE,echo=FALSE,results=hide>>=
plot(Smooth(mypattern, sigma=1.2), main="Smooth(mypattern)")
@ 
\setkeys{Gin}{width=0.4\textwidth}
\item
  You are airborne!
  Now look at the book \cite{baddrubaturn15} for more hints.
\end{enumerate}

\section*{How do I find out which command to use?}

Information sources for \spst\ include:
\begin{itemize}
\item the Quick Reference guide: a list of the most useful commands.
  \begin{quote}
    To view the quick reference guide, 
    start \R, then type \texttt{library(spatstat)}
    and then \texttt{help(spatstat)}.
    Alternatively you can download a pdf of the Quick Reference
    guide from the website \texttt{www.spatstat.org}
  \end{quote}
\item online help:
  \begin{quote}
    The online help files are useful --- they
    give detailed information and advice about each command.
    They are available when you are running \spst. 
    To get help about a particular command \texttt{blah}, 
    type \texttt{help(blah)}.
    There is a graphical help interface, 
    which you can start by typing \texttt{help.start()}.
    Alternatively you can download a pdf of the entire manual (1000 pages!)
    from the website \texttt{www.spatstat.org}. 
  \end{quote}
\item vignettes:
  \begin{quote}
    \Spst\ comes installed with several `vignettes' (introductory documents
    with examples) which can be accessed using the graphical help interface.
    They include a document about \texttt{Handling shapefiles}.
  \end{quote}
\item book:
  \begin{quote}
    Our book \cite{baddrubaturn15} 
    contains a complete course on \texttt{spatstat}.
  \end{quote}
\item website:
  \begin{quote}
    Visit the \spst\ package website \texttt{www.spatstat.org} 
  \end{quote}
\item forums:
  \begin{quote}
    Join the forum \texttt{R-sig-geo} by visiting \texttt{r-project.org}.
    Then email your questions to the forum. 
    Alternatively you can ask the authors of the \spst\ package
    (their email addresses are given in the package documentation).
  \end{quote}
\end{itemize}

\begin{thebibliography}{10}
% \bibitem{badd10wshop}
% A. Baddeley.
% \newblock Analysing spatial point patterns in {{R}}.
% \newblock Technical report, CSIRO, 2010.
% \newblock Version 4.
% \newblock URL \texttt{https://research.csiro.au/software/r-workshop-notes/}
%
\bibitem{baddrubaturn15}
A. Baddeley, E. Rubak, and R. Turner.
\newblock {\em Spatial Point Patterns: Methodology and Applications with {{R}}}.
\newblock Chapman \& Hall/CRC Press, 2015.

\bibitem{bivapebegome08}
R. Bivand, E.J. Pebesma, and V. G{\'{o}}mez-Rubio.
\newblock {\em Applied spatial data analysis with {R}}.
\newblock Springer, 2008.

\bibitem{cres93}
N.A.C. Cressie.
\newblock {\em Statistics for Spatial Data}.
\newblock {John Wiley and Sons}, {New York}, second edition, 1993.

\bibitem{digg03}
P.J. Diggle.
\newblock {\em Statistical Analysis of Spatial Point Patterns}.
\newblock Hodder Arnold, London, second edition, 2003.

\bibitem{fortdale05}
M.J. Fortin and M.R.T. Dale.
\newblock {\em Spatial analysis: a guide for ecologists}.
\newblock Cambridge University Press, Cambridge, UK, 2005.

\bibitem{fothroge09handbook}
A.S. Fotheringham and P.A. Rogers, editors.
\newblock {\em The {SAGE} {H}andbook on {S}patial {A}nalysis}.
\newblock SAGE Publications, London, 2009.

\bibitem{gaetguyo09}
C. Gaetan and X. Guyon.
\newblock {\em Spatial statistics and modeling}.
\newblock Springer, 2009.
\newblock Translated by Kevin Bleakley.

\bibitem{handbook10}
A.E. Gelfand, P.J. Diggle, M. Fuentes, and P. Guttorp, editors.
\newblock {\em Handbook of Spatial Statistics}.
\newblock CRC Press, 2010.

\bibitem{illietal08}
J. Illian, A. Penttinen, H. Stoyan, and D. Stoyan.
\newblock {\em Statistical Analysis and Modelling of Spatial Point Patterns}.
\newblock John Wiley and Sons, Chichester, 2008.

\bibitem{mollwaag04}
J. M{\o}ller and R.P. Waagepetersen.
\newblock {\em Statistical Inference and Simulation for Spatial Point
  Processes}.
\newblock Chapman and Hall/CRC, Boca Raton, 2004.

\bibitem{pfeietal08}
D.U. Pfeiffer, T. Robinson, M. Stevenson, K. Stevens, D. Rogers, and
  A. Clements.
\newblock {\em Spatial analysis in epidemiology}.
\newblock Oxford University Press, Oxford, UK, 2008.

\bibitem{wallgotw04}
L.A. Waller and C.A. Gotway.
\newblock {\em Applied spatial statistics for public health data}.
\newblock Wiley, 2004.

\end{thebibliography}



\end{document}

\documentclass[twoside,11pt]{article}

% \VignetteIndexEntry{Handling shapefiles in the spatstat package}

\SweaveOpts{eps=TRUE}

<<echo=FALSE,results=hide,fig=FALSE>>=
options(SweaveHooks = list(fig=function() par(mar=c(1,1,1,1))))
@ 

\usepackage{graphicx}
\usepackage[colorlinks=true,urlcolor=blue]{hyperref}
\usepackage{color}
\usepackage{anysize}
\marginsize{2cm}{2cm}{2cm}{2cm}

\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\bold}[1]{{\textbf {#1}}}
\newcommand{\R}{{\sf R}}

\begin{document}
\SweaveOpts{concordance=TRUE}
%\bibliographystyle{plain}
\thispagestyle{empty}

<<echo=FALSE,results=hide,eval=TRUE>>=
library(spatstat)
options(useFancyQuotes=FALSE)
sdate <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
         fields = "Date")
sversion <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
         fields = "Version")
@ 

\title{Handling shapefiles in the \texttt{spatstat} package}
\author{Adrian Baddeley, Rolf Turner and Ege Rubak}
\date{
\Sexpr{sdate} \\ 
\pkg{spatstat} version \texttt{\Sexpr{sversion}}
}
\maketitle

This vignette explains how to read data into the \pkg{spatstat} package
from files in the popular `shapefile' format. 

This vignette is part of the documentation included in 
\pkg{spatstat} version \texttt{\Sexpr{sversion}}.
The information applies to 
\pkg{spatstat} versions \texttt{1.36-0} and above.

\section{Shapefiles}

A shapefile represents a list of spatial objects 
--- a list of points, a list of lines, or a list of polygonal regions --- 
and each object in the list may have additional variables attached to it.

A dataset stored in shapefile format is actually stored in a
collection of text files, for example
\begin{verbatim}
     mydata.shp
     mydata.prj
     mydata.sbn
     mydata.dbf
\end{verbatim}
which all have the same base name \texttt{mydata} but different file extensions.
To refer to this collection you will always use the filename with the 
extension \texttt{shp}, for example \texttt{mydata.shp}. 

\section{Helper package}
\label{S:helpers}

We'll use the \pkg{sf} package to handle shapefile data.
Previously the now defunct package \pkg{maptools} was used in this vignette together with the older (still functional) \pkg{sp}.

Both \pkg{sf} and \pkg{sp} support a standard set of spatial data types in \R.
The \pkg{sp} package uses S4 classes and \pkg{sf} uses S3 classes. 
These standard data types can be handled by many other packages, so it is useful to convert 
your spatial data into one of the data types supported by \pkg{sf} or \pkg{sp}. 
With the retirement of \pkg{maptools} there are no direct conversion tools between \pkg{spatstat} data types and \pkg{sp} data types and for this reason we recommend using \pkg{sf} if possible. 
However, if you are already using \pkg{sp} formats you can convert data from 
\pkg{sp} format to \pkg{sf} format and then to \pkg{spatstat} format.

To read and write files in shapefile format \pkg{sf} uses the system library 
\texttt{GDAL} and newer versions of \pkg{sp} uses \pkg{sf} under the hood.

\section{Caveat about longitude-latitude coordinates}

The shapefile format supports geographical coordinates, usually 
longitude-latitude coordinates, which specify locations on the curved surface of
the Earth. 
However, \texttt{spatstat} deals only with spatial data on 
a flat two-dimensional plane. 

If you follow the recommendation to read in shapefile (or other formats) data 
with \pkg{sf} and then converting to \pkg{spatstat} format you should encounter 
an error if you try to convert data in longitude-latitude coordinates directly 
to \pkg{spatstat} format. However, if you manage to read in longitude-latitude 
data and convert them into \texttt{spatstat} objects without \pkg{sf},
longitude and latitude coordinates will most likely be
treated as $x$ and $y$ coordinates,
so that the Earth's surface is effectively mapped to a rectangle.
This mapping distorts distances and areas. 

If your study region is a \emph{small} region of the Earth's surface 
(about 3 degrees, 180 nautical miles, 200 statute miles, 320 km across) then
a reasonable approach is to use the latitude and longitude as 
$x$ and $y$ coordinates, after multiplying the longitude coordinates by the 
cosine of the latitude of the centre of the region. This will
approximately preserve areas and distances. 
This calculation is a simple example of 
a \emph{geographical projection} and there are some much better 
projections available. It may be wise to use \verb!st_transform()! in \pkg{sf}
to perform the appropriate projection for you, and then to convert
the projected data into \pkg{spatstat} objects.

If your study region is a large part of the sphere, then your data 
may not be amenable to the techniques provided by \pkg{spatstat}
because the geometry is fundamentally different.
Please consider the extension package \pkg{spatstat.sphere}.

\section{How to read shapefiles into \pkg{spatstat}} 

To read shapefile data into \pkg{spatstat}, you follow two steps:
\begin{enumerate}
\item 
  using the facilities of \pkg{sf}, read the shapefiles
  and store the data in one of the standard formats supported by \pkg{sf}.
\item
  convert the \pkg{sf} data type into one of the data types
  supported by \pkg{spatstat}.
\end{enumerate}

\subsection{Read shapefiles using \pkg{sf}} 

Here's how to read shapefile data.

\begin{enumerate}
\item ensure that the package \pkg{sf} is installed.
\item start R and load the package: 
<<eval=FALSE>>=
library(sf)
@ 
\item read the shapefile into an object in the \pkg{sf} package
using \verb!st_read!, for example
<<eval=FALSE>>=
x <- st_read(system.file("shape/nc.shp", package="sf"))
@ 
\item 
This will read in the data as an object of class \texttt{sf}. This is basically a \texttt{data.frame} with a designated geometry column (of class \texttt{sfc}). All other columns contain information/features (marks in \pkg{spatstat} terminology) related to the geometries. For example the geometry column could be a list of polygonal boundaries of the counties of a state and the other columns could contain the name of the county and other registered values of interest. To find out what kind of spatial objects are represented by the dataset,
inspect the class of the geometry column:
<<eval=FALSE>>=
st_geometry_type(x, by_geometry = FALSE)
@ 
There are many possible classes but the ones of main interest here are:
\begin{itemize}
\item \texttt{POINT} (or \texttt{MULTIPOINT}) indicating each row refers to a point (or several points)
\item \texttt{LINESTRING} (or \texttt{MULTILINESTRING}) indicating each row refers to a collection of sequentially connected straight line segments (or several of these)
\item \texttt{POLYGON} (or \texttt{MULTIPOLYGON}) indicating each row refers to a polygon which may have holes inside (or several of these)
\end{itemize}
\end{enumerate}

\subsection{Convert data to \pkg{spatstat} format}

To convert the dataset to an object in the
\pkg{spatstat} package, the procedure depends on the 
type of the geometry column, as explained below. 

Both packages \pkg{sf} and \pkg{spatstat} must be \textbf{loaded}
in order to convert the data.

\subsubsection{Geometries of class \texttt{POINT}/\texttt{MULTIPOINT}}

A \texttt{sf} object \texttt{x} with geometry column of class \texttt{POINT}
represents a spatial point pattern.
Use \texttt{as.ppp(x)} to convert it to a 
spatial point pattern in \pkg{spatstat}:
<<eval=FALSE>>=
X <- as.ppp(x)
@ 

(The conversion is performed by \texttt{as.ppp.sf}, a function in \pkg{sf}.)

The window for the point pattern will be taken from
the bounding box of the points. You will probably wish to change this window,
usually by taking another dataset to provide the window information.
Use \verb![.ppp! to change the window: if \texttt{X} is a point pattern
object of class \verb!"ppp"! and \texttt{W} is a window object of class
\verb!"owin"!, type
<<eval=FALSE>>=
X <- X[W]
@ 

If the \texttt{sf} object \texttt{x} contains other columns than the geometry column these are additional variables (`marks') attached to each point.

At the time of writing \texttt{as.ppp(x)} will unfortunately only use the first column of additional data as the \texttt{marks} of the point pattern \texttt{X}.
In that case you can extract the data frame of auxiliary data by 
\verb!df <- st_drop_geometry(x)! and manually assign them as marks in \pkg{spatstat}:

<<eval=FALSE>>=
df <- st_drop_geometry(x)
X <- as.ppp(x)
marks(X) <- df
@ 

If the class of the geometry column is \texttt{MULTIPOINT} you need to first cast to \texttt{POINT} and then convert as described above:
<<eval=FALSE>>=
x_point <- st_cast(x, "POINT")
X <- as.ppp(x_point)
@ 

If you have a combination of \texttt{POINT} and \texttt{MULTIPOINT} you may first need an intermediate cast to \texttt{MULTIPOINT} before casting to \texttt{POINT}:
<<eval=FALSE>>=
x_multipoint <- st_cast(x, "MULTIPOINT")
x_point <- st_cast(x, "POINT")
X <- as.ppp(x_point)
@ 


\subsubsection{Geometries of class \texttt{LINESTRING} or \texttt{MULTILINESTRING}}
\label{spatiallines.2.psp}

A ``line segment'' is the straight line between two points in the plane.

In the \pkg{spatstat} package, an object of class \texttt{psp}
(``planar segment pattern'')
represents a pattern of line segments, which may or may not be
connected to each other (like matches which have fallen at random
on the ground). 

In the \pkg{sf} package, a geometry column of class \texttt{LINESTRING}
represents a \textbf{list} of \textbf{connected curves}, 
each curve consisting of a sequence of straight 
line segments that are joined together (like
several pieces of a broken bicycle chain.) 

For a geometry column of class \texttt{MULTILINESTRING} each element of the top list (the geometry column) is it self a list of connected curves.

\textbf{list of lists}

So the \texttt{spatstat} and \texttt{sf} data types do not correspond exactly.

The list of connected curves in a \texttt{LINESTRING} column may be useful when representing a single river system where each branch of the river may be in its own element of the list (row of the column).

The list-of-lists hierarchy in a \texttt{MULTILINESTRING} column
is useful when representing river of a continent where each element of the primary list (row in the column) would correspond to a river system and each of these would be a list of the branches of the river system.

For example, if \texttt{Africa} is an object of class \texttt{sf} with geometry column of class \texttt{MULTILINESTRING}
representing the main rivers of Africa, then \texttt{Africa} will have hundreds of rows representing each of the rivers. The geometry column \verb!geo <- st_geometry(Africa)! is then a list, where \verb!geo[[i]]! might represent the total river system the \texttt{i}-th river (e.g. the Nile). 
The branches of each river system consist of several different curved lines. Thus 
\verb!geo[[i]][[j]]! would represent the \texttt{j}th 
branch of the \texttt{i}-th river and would be of class \texttt{LINESTRING}.

For an object \texttt{x} of class \texttt{sf} with geometry column of class \texttt{MULTILINESTRING} or \texttt{LINESTRING},
there are several things that you might want to do:
\begin{enumerate}
\item 
  collect together all the line segments (all the segments that make up all the
  connected curves) and store them as a single object of class \texttt{psp}.
\begin{quote}
  To do this, 
  use \texttt{as.psp(x)} to convert it to a 
  spatial line segment pattern. 
\end{quote}
Note: Any auxilliary information stored in other columns is automatically attached as marks to the line segments of the \pkg{spatstat} \texttt{psp} object.
\item 
  convert each connected curve to an object of class \texttt{psp},
  keeping different connected curves separate.

To do this, type something like the following:
<<eval=FALSE>>=
out <- lapply(geo, function(z) { lapply(z, as.psp) })
@ 

(The conversion is performed by \texttt{as.psp.MULTILINESTRING},
a function in \pkg{sf}. So the \pkg{sf} and \pkg{spatstat}
packages must be loaded in order for this to work.)

Any auxiliary data in other columns can be attached as marks by this command:
<<eval=FALSE>>=
dat <- st_drop_geometry(Africa)
for(i in seq(nrow(dat))){
  out[[i]] <- lapply(out[[i]], "marks<-", value=dat[i, , drop=FALSE])
}
@

The result will be a \textbf{list of lists} of objects of class \texttt{psp}.
Each one of these objects represents a connected curve,
although the \pkg{spatstat} package does not know that.
The list structure will reflect the list structure of the original
\texttt{MULTILINESTRING} object \texttt{x}. If that's not what you want,
then use \verb!curvelist <- do.call("c", out)! or
<<eval=FALSE>>=
curvegroup <- lapply(out, function(z) { do.call("superimpose", z)})
@ 
to collapse the list-of-lists-of-\texttt{psp}'s 
into a list-of-\texttt{psp}'s. In the first case, \texttt{curvelist[[i]]}
is a \texttt{psp} object representing the \texttt{i}-th connected curve. 
In the second case, \texttt{curvegroup[[i]]}
is a \texttt{psp} object containing all the line segments in
the \texttt{i}-th group of connected curves (for example the 
\texttt{i}-th river system -- the Nile -- in the \texttt{Africa} example).
\end{enumerate}

The window for the spatial line segment pattern can be specified
as an argument \texttt{window} to the function \texttt{as.psp}.

In the \pkg{spatstat} package, an object of class \texttt{psp}
(representing a collection of line segments)
may have a data frame of marks. Note that each \emph{line segment}
in a \texttt{psp} object may have different mark values. 
For the converted data the mark variables attached to a particular \emph{group of connected lines}
in the \texttt{sf} object, will be duplicated
and attached to each \emph{line segment} in the resulting \texttt{psp} object.

\subsubsection{Geometries of class \texttt{POLYGON} or \texttt{MULTIPOLYGON}}

First, so that we don't go completely crazy, let's introduce some terminology.
A \emph{polygon} is a closed curve that is composed of 
straight line segments. You can draw a polygon
without lifting your pen from the paper. This is called a \texttt{POLYGON} in 
\texttt{sf} terminology.

\setkeys{Gin}{width=0.4\textwidth}
\begin{center}
<<echo=FALSE,results=hide,fig=TRUE>>=
data(chorley)
plot(as.owin(chorley), lwd=3, main="polygon")
@ 
\end{center}

A \emph{polygonal region}
is a region in space whose boundary is composed of straight line segments.
A polygonal region may consist of several unconnected pieces, and each piece
may have holes. The boundary of a polygonal region
consists of one or more polygons. To draw the boundary of a polygonal 
region, you may need to lift and drop the pen several times. This is called a 
\texttt{MULTIPOLYGON} in \texttt{sf} terminology.

\setkeys{Gin}{width=0.4\textwidth}
\begin{center}
<<echo=FALSE,results=hide,fig=TRUE>>=
data(demopat)
plot(as.owin(demopat), col="blue", main="polygonal region")
@ 
\end{center}

An object of class \texttt{owin} in \pkg{spatstat}
represents a polygonal region. It is a region of space that is delimited
by boundaries made of lines.

An object \texttt{x} with geometry column of class \texttt{MULTIPOLYGON}
represents a \textbf{list of polygonal regions}. For example,
a single geometry column of class \texttt{MULTIPOLYGON} could 
store information about every State in the United States of America
(or the United States of Malaysia). Each State would be a separate
polygonal region (and it might contain holes such as lakes).

There are two things
that you might want to do with a geometry column of class \texttt{MULTIPOLYGON}:
\begin{enumerate}
\item 
  combine all the polygonal regions together into a single
  polygonal region, and convert this to a single object of class \texttt{owin}.
  \begin{quote}
    For example, you could combine all the States of the USA together
    and obtain a single object that represents the territory of the USA. 

    To do this, use \texttt{as.owin(x)}.
    The result is a single window (object of class \texttt{"owin"}) 
    in the \pkg{spatstat} package.
  \end{quote}
\item keep the different polygonal regions separate; convert each 
  one of the polygonal regions to an object of class \texttt{owin}.
  \begin{quote}
    For example, you could keep the States of the USA separate,
    and convert each State to an object of class \texttt{owin}.
  \end{quote}
  To do this, type the following:
<<eval=FALSE>>=
geo <- st_geometry(x)
windows <- lapply(geo, as.owin)
@ 
  The result is a list of objects of class \texttt{owin}. 
  Often it would make sense to convert this to a 
  tessellation object, by typing
<<eval=FALSE>>=
te <- tess(tiles=windows)
@ 
\end{enumerate}

(The conversion is performed by \texttt{as.owin.MULTIPOLYGON},
a function in \pkg{sf}. So the \pkg{sf} and \pkg{spatstat}
packages must be loaded in order for this to work.)

{\bf The following is different from what happened in
  previous versions of \pkg{spatstat}} (prior to version \texttt{1.36-0}.)

During the conversion process, the geometry of the polygons
will be automatically ``repaired'' if needed.
Polygon data from shapefiles often contain geometrical inconsistencies 
such as self-intersecting boundaries and overlapping pieces. 
For example, these can arise from small errors in curve-tracing.
Geometrical inconsistencies are tolerated in 
an object with geometry column of class \texttt{MULTIPOLYGON} which
is a list of lists of polygonal curves.
However, they are not tolerated in an object of class \texttt{owin},
because an \texttt{owin} must specify a well-defined region of space.
These data inconsistencies must be repaired to prevent technical problems. 
In \pkg{spatstat} polygon-clipping code is used to automatically convert 
polygonal lines into valid polygon boundaries. 
The repair process changes the number of vertices in each polygon,
and the number of polygons (if you chose option 1).
To disable the repair process, set 
\texttt{spatstat.options(fixpolygons=FALSE)}.

\subsubsection{Auxiliary information}

Typically an object \texttt{x} of class \texttt{sf} with geometry column of type 
(\texttt{MULTI})\texttt{POLYGON} has other columns
with additional variables attached to
each polygon. 
The data frame of auxiliary data is extracted by 
\verb!df <- st_drop_geometry(x)!.

There is currently no facility in \pkg{spatstat} for attaching
marks to an \texttt{owin} object directly, but if you have collected the 
separate regions in a tessellation you can attach marks to the tessellation, so 
the entire workflow becomes:
<<eval=FALSE>>=
geo <- st_geometry(x)
df <- st_drop_geometry(x)
windows <- lapply(geo, as.owin)
te <- tess(tiles=windows)
marks(te) <- df
@ 

However, if the regions are kept as a list of \texttt{owin} objects it is 
possible to take advantage of \pkg{spatstat}'s support of objects called 
\textbf{hyperframes}, which are like data frames except that the entries can be 
any type of object. Thus we can represent the data in \pkg{spatstat} as follows:
<<eval=FALSE>>=
h <- hyperframe(window=windows)
h <- cbind.hyperframe(h, df)
@ 

Then \texttt{h} is a hyperframe containing a column of \texttt{owin}
objects followed by the columns of auxiliary data.

% \subsubsection{Objects of class \texttt{SpatialGridDataFrame}
%   and \texttt{SpatialPixelsDataFrame}}
% 
% An object \texttt{x} of class \texttt{SpatialGridDataFrame} represents
% a pixel image on a rectangular grid. It includes a \texttt{SpatialGrid}
% object \texttt{slot(x, "grid")} defining the full rectangular grid of pixels, 
% and a data frame \texttt{slot(x, "data")} containing the pixel values
% (which may include \texttt{NA} values).
% 
% The command \texttt{as(x, "im")} converts \texttt{x} to a pixel image
% of class \texttt{"im"}, taking the pixel values from the \emph{first column}
% of the data frame. If the data frame has multiple columns, these 
% have to be converted to separate pixel images in \pkg{spatstat}.
% For example
% <<eval=FALSE>>=
%   y <- as(x, "im")
%   ylist <- lapply(slot(x, "data"), function(z, y) { y[,] <- z; y }, y=y)
% @ 
% 
% An object \texttt{x} of class  \texttt{SpatialPixelsDataFrame} 
% represents a \emph{subset} of a pixel image. 
% To convert this to a \pkg{spatstat} object, it should first be converted to
% a \texttt{SpatialGridDataFrame} by \texttt{as(x, "SpatialGridDataFrame")},
% then handled as described above.

\end{document}

\documentclass[11pt]{article}

% \VignetteIndexEntry{Datasets Provided for the Spatstat Package}

<<echo=FALSE,results=hide,fig=FALSE>>=
options(SweaveHooks=list(fig=function() par(mar=c(1,1,1,1))))
@ 

\usepackage{graphicx}
\usepackage{anysize}
\marginsize{1.7cm}{1.7cm}{1.7cm}{1.7cm}

\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\bold}[1]{{\textbf {#1}}}
\newcommand{\R}{{\sf R}}
\newcommand{\spst}{\pkg{spatstat}}
\newcommand{\Spst}{\pkg{Spatstat}}
\newcommand{\sdat}{\pkg{spatstat.data}}
\newcommand{\Sdat}{\pkg{Spatstat.data}}

\begin{document}
\bibliographystyle{plain}
\thispagestyle{empty}
\SweaveOpts{eps=TRUE}
\setkeys{Gin}{width=0.6\textwidth}

<<echo=FALSE,results=hide>>=
library(spatstat)
sdate <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
         fields = "Date")
sversion <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
         fields = "Version")
spatstat.options(transparent=FALSE)
options(useFancyQuotes=FALSE)
@ 

\title{Datasets provided for \spst}
\author{Adrian Baddeley, Rolf Turner and Ege Rubak}
\date{For \spst\ version \texttt{\Sexpr{sversion}}}
\maketitle

This document is an overview of the spatial datasets
that are provided for the \spst\ package. 

To flick through a nice display of all the data sets that come with
\spst\ type \texttt{demo(data)}.  To see information about a given
data set, type \texttt{help({\em name})} where \emph{name} is the
name of the data set.  To plot a given data set, 
type \texttt{plot({\em name})}. A nicer plot
will often be produced by typing \texttt{example(\emph{name})}.

Datasets in \spst\ are ``lazy-loaded'', which means that they can
be accessed simply by typing their name. Not all packages do this;
in some packages you have to type \texttt{data({\em name})} in
order to access a data set.

To list all the datasets in \spst, you need to type
\texttt{data(package="spatstat.data")}. This is because, 
for efficiency, the datasets are actually installed in a
sub-package \sdat. This is the only time you should ever need to
mention \sdat\ explicitly. When the \spst\ package 
is loaded by the command \texttt{library(spatstat)},
the sub-package \sdat\ is automatically loaded.

\section{List of datasets}

\subsection{Point patterns in 2D}

\newcommand{\recto}{\framebox{\hphantom{re}\vphantom{re}}}
\newcommand{\irregpoly}{\includegraphics*[width=6mm]{irregpoly}}
\newcommand{\masque}{\includegraphics*[width=6mm]{mask}}
\newcommand{\convpoly}{\includegraphics*[width=4mm]{hexagon}}
\newcommand{\disc}{$\bigcirc$}
\newcommand{\nomarks}{$\cdot$}
\newcommand{\nocov}{$\cdot$}

Here is a list of the standard point pattern data sets 
that are supplied with the current installation of \sdat.

The shape of the window containing the point pattern
is indicated by the symbols \recto\ (rectangle), 
\disc\ (disc), \convpoly\ (convex polygon), \irregpoly\ (irregular polygon)
and \masque (binary mask).

\begin{tabular}{l|l|ccc}
{\sf name} & {\sf description} &
            {\sf marks} & {\sf covariates} & {\sf window} \\ \hline
{\tt amacrine} & rabbit amacrine cells &
            cell type & \nocov & \recto \\  
{\tt anemones} & sea anemones  & 
            diameter & \nocov & \recto \\
{\tt ants} & ant nests& 
            species & zones  & \convpoly \\
{\tt bdspots} & breakdown spots & 
           \nomarks           & \nocov & \disc \\
{\tt bei} & rainforest trees & 
           \nomarks           & topography & \recto \\
{\tt betacells} & cat retinal ganglia & 
            cell type, area & \nocov & \recto \\
{\tt bramblecanes} & bramble canes & 
            age & \nocov & \recto \\
{\tt bronzefilter} & bronze particles & 
            diameter & \nocov & \recto \\
{\tt btb} & bovine tuberculosis & 
            type, year & \nocov & \irregpoly \\
{\tt cells} & biological cells &
             \nomarks &\nocov & \recto \\
{\tt chorley} & cancers & 
            case/control &\nocov  & \irregpoly \\
{\tt clmfires} & forest fires & 
            cause, size, date & 
            \shortstack[c]{elevation, orientation,\\ slope, land use}  
            & \irregpoly \\
{\tt concrete} & air bubbles in concrete & 
             \nomarks & \nocov & \masque  \\
{\tt copper} & copper deposits & 
             \nomarks & fault lines & \recto  \\
{\tt demopat} & artificial data & 
             type & \nocov & \irregpoly \\
{\tt finpines} & trees & 
             diam, height & \nocov & \recto  \\
{\tt gordon} & people in a park & 
             \nomarks & \nocov & \irregpoly  \\
{\tt gorillas} & gorilla nest sites & 
             group, season & 
            \shortstack[c]{terrain, vegetation,\\ heat, water} & 
             \irregpoly  \\
{\tt hamster} & hamster tumour cells & 
              cell type &\nocov  & \recto \\
{\tt humberside} & child leukaemia & 
              case/control & \nocov & \irregpoly\\
{\tt hyytiala} & mixed forest & 
              species &\nocov  & \recto \\
{\tt japanesepines} & Japanese pines & \nomarks &\nocov & \recto \\
{\tt lansing} & mixed forest &
               species & \nocov & \recto \\
{\tt longleaf} & trees & 
              diameter & \nocov &  \recto \\
{\tt meningitis} & disease cases & 
              serotype & population density &  \irregpoly \\
{\tt mucosa}   & gastric mucosa cells & 
              cell type & \nocov &  \recto \\
{\tt murchison} & gold deposits & \nomarks & faults, rock type & \irregpoly \\
{\tt nbfires} & wildfires & several & \nocov & \irregpoly \\
{\tt nztrees} & trees & \nomarks & \nocov & \recto \\
{\tt paracou} & trees & adult/juvenile & \nocov & \recto \\
{\tt ponderosa} & trees & \nomarks & \nocov & \recto \\
{\tt redwood} & saplings & \nomarks & \nocov & \recto \\
{\tt redwood3} & saplings & \nomarks & \nocov & \recto \\
{\tt redwoodfull} & saplings & 
              \nomarks & zones & \recto \\
{\tt shapley} & galaxies & magnitude, recession, SE & \nocov & \convpoly \\
{\tt shelling} & artillery impacts & \nomarks & \nocov & \irregpoly \\
{\tt simdat} & simulated pattern & \nomarks & \nocov & \recto \\
{\tt sporophores} & fungi & species & \nocov &  \disc \\
{\tt spruces} & trees & diameter & \nocov &  \recto \\
{\tt stonetools} & palaeolithic tools & type, depth & \nocov &  \irregpoly \\
{\tt swedishpines} & trees & \nomarks & \nocov & \recto \\
{\tt urkiola} & mixed forest & species & \nocov & \irregpoly \\
{\tt vesicles} & synaptic vesicles & \nomarks & zones & \irregpoly \\
{\tt waka} & trees & diameter & \nocov & \recto \\
\hline
\end{tabular}

\subsubsection*{Auxiliary information}

Additional information about a point pattern data set \texttt{\em name}
may be stored in a separate list \texttt{{\em name}.extra}.
Currently these are the auxiliary data sets:

\begin{tabular}[!h]{ll}
  {\sc Name} & {\sc Contents} \\ 
  \hline
  {\tt ants.extra} & field and scrub subregions; \\
                   & additional map elements; plotting function \\
  {\tt bei.extra} & covariate images \\
  {\tt chorley.extra} & incinerator location; plotting function \\
  {\tt gorillas.extra} & covariate images\\
  {\tt nbfires.extra} & inscribed rectangle; border type labels \\
  {\tt ponderosa.extra} & data points of interest; plotting function\\
  {\tt redwoodfull.extra} & subregions; plotting function \\
  {\tt shapley.extra} & individual survey fields; plotting function \\
  {\tt vesicles.extra} & anatomical regions \\
  \hline
\end{tabular}

Additionally, raw data files are available for the datasets 
\texttt{amacrine}, \texttt{finpines}, \texttt{gorillas},
\texttt{vesicles} and \texttt{gorillas}.
See the help files for these datasets for information about the
raw data files.

\subsection{Other Spatial Data Types}

There are also the following spatial data sets which are not 2D point patterns:

\begin{tabular}[c]{l|l|l|l}
{\sf name} & {\sf description} & {\sf class} & {\sf format} \\ \hline
{\tt austates} & Australian states & \texttt{tess} & tessellation \\
{\tt cetaceans} & marine survey & \texttt{hyperframe} & replicated 2D point patterns \\
{\tt chicago} & crimes & \texttt{lpp} & point pattern on linear network \\
{\tt demohyper} & simulated data & \texttt{hyperframe} & replicated 2D point patterns with covariates\\
{\tt dendrite} & dendritic spines & \texttt{lpp} & point pattern on linear network \\
{\tt flu} & virus proteins & \texttt{hyperframe} & replicated 2D point patterns \\
{\tt heather} & heather mosaic & \texttt{solist} & binary image (three versions) \\
{\tt osteo} & osteocyte lacunae & \texttt{hyperframe} & replicated 3D point patterns with covariates\\
{\tt pyramidal} & pyramidal neurons & \texttt{hyperframe} & replicated 2D point patterns in 3 groups\\
{\tt residualspaper} & data \& code & \texttt{list} & 2D point patterns, \R\ function \\
{\tt simba} & simulated data & \texttt{hyperframe} & replicated 2D point patterns in 2 groups\\
{\tt spiders} & spider webs & \texttt{lpp} & point pattern on linear network \\
{\tt waterstriders} & insects on water & \texttt{solist} & replicated 2D point patterns\\
\hline
\end{tabular}

Additionally there is a dataset \texttt{Kovesi} containing
several colour maps with perceptually uniform contrast. 

Raw data files are available for the \texttt{osteo} dataset.
See the help for \texttt{osteo} for information.

\section{Information on each dataset}

Here we give basic information about each dataset.
For further information, consult the help file for the 
particular dataset.

<<echo=FALSE>>=
opa <- par()
## How to set all margins to zero and eliminate all outer spaces
zeromargins <- function() {
  par(
      mar=rep(0,4),
      omd=c(0,1,0,1),
      xaxs="i",
      yaxs="i"
  )
  invisible(NULL)
}
## Set 'mar'
setmargins <- function(...) {
  x <- c(...)
  x <- rep(x, 4)[1:4]
  par(mar=x)
  invisible(NULL)
}
@ 

\subsubsection*{\texttt{amacrine}: Amacrine cells}

Locations of displaced amacrine cells in the retina of a rabbit.
There are two types of points, ``on'' and ``off''.

\SweaveOpts{width=5.5,height=3}\setkeys{Gin}{width=0.8\textwidth}
<<eval=FALSE>>=
plot(amacrine)
@ 
<<fig=TRUE,echo=FALSE,results=hide>>=
setmargins(0,1,2,0)
plot(amacrine)
@ 

\subsubsection*{\texttt{anemones}: Sea Anemones}

These data give the spatial locations and diameters
of sea anemones on a boulder near sea level.

\SweaveOpts{width=7,height=4.5}\setkeys{Gin}{width=0.8\textwidth}
<<eval=FALSE>>=
plot(anemones, markscale=1)
@ 
<<fig=TRUE,echo=FALSE,results=hide>>=
setmargins(0,0,2,0)
plot(anemones, markscale=1)
@ 

\subsubsection*{\texttt{ants}: Ants' nests}

Spatial locations of nests of two species of
ants at a site in Greece.
The full dataset (supplied here) has an irregular polygonal boundary,
while most analyses have been confined to two rectangular
subsets of the pattern (also supplied here).

% Parameters for Ants data with key at right
\SweaveOpts{width=6.3,height=4}\setkeys{Gin}{width=0.7\textwidth}
<<eval=FALSE>>=
ants.extra$plotit()
@ %$
<<fig=TRUE,echo=FALSE,results=hide>>=
setmargins(0,0,1,0)
ants.extra$plotit()
@ %$

\subsubsection*{\texttt{austates}: Australian states}

  The states and large mainland territories of Australia are
  represented as polygonal regions forming a tessellation.

<<fig=TRUE>>=
plot(austates)
@   

\subsubsection*{\texttt{bdspots}: Breakdown spots}

A list of three point patterns, each giving the locations of
electrical breakdown spots on a circular electrode in
a microelectronic capacitor.

\SweaveOpts{width=12,height=6}\setkeys{Gin}{width=\textwidth}
<<eval=FALSE>>=
plot(bdspots, equal.scales=TRUE, pch="+", 
     panel.args=function(i)list(cex=c(0.15, 0.2, 0.7)[i]))
@   
<<fig=TRUE,echo=FALSE>>=
zeromargins()
plot(bdspots, equal.scales=TRUE, pch="+", main="",
     mar.panel=0, hsep=1,
     panel.args=function(i)list(cex=c(0.15, 0.2, 0.7)[i]))
@   

\subsubsection*{\texttt{bei}: Beilschmiedia data}

Locations of 3605 trees in a tropical rain forest.
Accompanied by covariate data giving the elevation (altitude)
and slope of elevation in the study region.
  
\SweaveOpts{width=12,height=6}\setkeys{Gin}{width=0.8\textwidth}
<<eval=FALSE>>=
plot(bei.extra$elev, main="Beilschmiedia")
plot(bei, add=TRUE, pch=16, cex=0.3)
@ 
<<fig=TRUE,echo=FALSE,results=hide>>=
setmargins(0,0,2,0)
plot(bei.extra$elev, main="Beilschmiedia")
plot(bei, add=TRUE, pch=16, cex=0.3)
@ 

The following command gives a perspective display
similar to the front cover of Baddeley, Rubak and Turner (2015):

<<eval=FALSE>>=
M <- persp(bei.extra$elev, 
           theta=-45, phi=18, expand=7,
           border=NA, apron=TRUE, shade=0.3, 
           box=FALSE, visible=TRUE,
           main="")
perspPoints(bei, Z=bei.extra$elev, M=M, pch=16, cex=0.3)
@ 

\subsubsection*{\texttt{betacells}: Beta ganglion cells}

Locations of beta ganglion cells in cat retina,
each cell classified as `on' or `off'
and also labelled with the cell profile area.
  
<<fig=TRUE>>=
plot(betacells)
@ 

\subsubsection*{\texttt{bramblecanes}: Bramble canes}

<<fig=TRUE>>=
plot(bramblecanes, cols=1:3)
@ 

Try the following 

<<eval=FALSE>>=
plot(split(bramblecanes))
@ 

\subsubsection*{\texttt{bronzefilter}: Bronze filter section profiles}

Spatially inhomogeneous pattern of
circular section profiles of particles, observed in a
longitudinal plane section through a gradient sinter
filter made from bronze powder.

<<fig=TRUE>>=
plot(bronzefilter,markscale=2)
@ 

\subsubsection*{\texttt{btb}: bovine tuberculosis}

Locations of farms where bovine tuberculosis was detected,
marked by year of detection and spoligotype of tuberculosis.

<<fig=TRUE>>=
plot(btb, which.marks="spoligotype", cols=2:5, chars=1:4)
@ 

\subsubsection*{\texttt{cells}: Biological cells}

Locations of the centres of 42 biological cells
observed under optical microscopy in a histological section.
Often used as a demonstration example.

<<fig=TRUE>>=
plot(cells)
@ 

\subsubsection*{\texttt{cetaceans}: Survey of marine species}

Recorded sightings of whales, dolphins and other marine species
in a series of surveys. Replicated 2D marked point patterns.

<<fig=TRUE>>=
plot(cetaceans.extra$patterns, main="Cetaceans data", cols=1:5, hsep=1)
@ 

\subsubsection*{\texttt{chicago}: Chicago crimes}

Locations (street addresses) of crimes reported in a two-week period
in an area close to the University of Chicago.
A multitype point pattern on a linear network.

<<fig=TRUE>>=
plot(chicago, main="Chicago Crimes", col="grey",
     cols=c("red", "blue", "black", "blue", "red", "blue", "blue"),
     chars=c(16,2,22,17,24,15,6), leg.side="left", show.window=FALSE)
@ 

\subsubsection*{\texttt{chorley}: Chorley-Ribble cancer data}

Spatial locations of cases of cancer of the larynx
and cancer of the lung, and the location of a disused industrial
incinerator. A marked point pattern, with an irregular window
and a simple covariate.

<<fig=TRUE>>=
chorley.extra$plotit()
@ %$

\subsubsection*{\texttt{clmfires}: Castilla-La Mancha Fires}

Forest fires in the Castilla-La Mancha
region of Spain between 1998 and 2007.
A point pattern with 4 columns of marks:

\begin{tabular}{ll}
  \texttt{cause} & cause of fire\\ 
  \texttt{burnt.area} & total area burned, in hectares \\
  \texttt{date} & date of fire \\
  \texttt{julian.date} & date of fire in days since 1.1.1998
\end{tabular}
  
<<fig=TRUE>>=
plot(clmfires, which.marks="cause", cols=2:5, cex=0.25,
     main="Castilla-La Mancha forest fires")
@ 

The accompanying dataset \texttt{clmfires.extra} is a list
of two items \texttt{clmcov100} and \texttt{clmcov200} containing covariate
information for the entire Castilla-La Mancha region. Each
of these two elements is a list of four pixel images 
named \texttt{elevation}, \texttt{orientation},
\texttt{slope} and \texttt{landuse}. 

<<fig=TRUE>>=
plot(clmfires.extra$clmcov100$elevation, main="Elevation")
@ %$ 

\subsubsection*{\texttt{concrete}: Air bubbles in concrete}

Prof.\ Shin-ichi Igarashi's data:  a point pattern of the locations
of centroids of air bubbles seen in a cross-section of concrete.
Air bubbles are present in the matrix of cement paste
which surrounds the particles of aggregate. The outline of the aggregate,
and the locations of the centroids of the bubble profiles, are recorded.

<<fig=TRUE>>=
plot(concrete,chars="+",cols="blue",col="yellow")
@ 

\subsubsection*{\texttt{copper}: Queensland copper data}

These data come from an intensive geological survey 
in central Queensland, Australia.
They consist of 67 points representing copper ore deposits,
and 146 line segments representing geological `lineaments',
mostly faults. 

<<fig=TRUE>>=
plot(copper$Points, main="Copper")
plot(copper$Lines, add=TRUE)
@ 

\subsubsection*{\texttt{demohyper}}

A synthetic example of a \texttt{hyperframe} for demonstration purposes.

<<fig=TRUE>>=
plot(demohyper, quote({ plot(Image, main=""); plot(Points, add=TRUE) }),
      parargs=list(mar=rep(1,4)))
@ 

\subsubsection*{\texttt{demopat}}

A synthetic example of a point pattern for demonstration purposes.

<<fig=TRUE>>=
plot(demopat)
@ 

\subsubsection*{\texttt{dendrite}}

  Dendrites are branching filaments which extend from the
  main body of a neuron (nerve cell) to propagate electrochemical
  signals. Spines are small protrusions on the dendrites.

  This dataset gives the locations of 566 spines
  observed on one branch of the dendritic tree of a rat neuron.
  The spines are classified according to their shape into three types:
  mushroom, stubby or thin.

<<fig=TRUE>>=
plot(dendrite, leg.side="bottom", main="", cex=0.75, cols=2:4)
@ 

\subsubsection*{\texttt{finpines}: Finnish pine saplings}

Locations of 126 pine saplings
in a Finnish forest, their heights and their diameters.

<<fig=TRUE>>=
plot(finpines, main="Finnish pines")
@ 

\subsubsection*{\texttt{flu}: Influenza virus proteins}

  The \texttt{flu} dataset contains
  replicated spatial point patterns giving the locations of two
  different virus proteins on the membranes of cells infected with
  influenza virus.
  
  It is a \texttt{hyperframe} containing
  point patterns and explanatory variables.
  
<<fig=TRUE>>=
wildM1 <- with(flu, virustype == "wt" & stain == "M2-M1")
plot(flu[wildM1, 1, drop=TRUE],
     main=c("flu data", "wild type virus, M2-M1 stain"),
     chars=c(16,3), cex=0.4, cols=2:3)
@ 

\subsubsection*{\texttt{gordon}: People in Gordon Square}

Locations of people sitting on a grass patch on a sunny afternoon.

  
<<fig=TRUE>>=
plot(gordon, main="People in Gordon Square", pch=16)
@ 

\subsubsection*{\texttt{gorillas}: Gorilla nesting sites}

 Locations of nesting sites of gorillas, and associated covariates,
  in a National Park in Cameroon.  
  
  \texttt{gorillas} is a marked point pattern (object
  of class \texttt{"ppp"}) representing nest site locations.

  \texttt{gorillas.extra} is a named list of 7 pixel images (objects of
  class \texttt{"im"}) containing spatial covariates.
  It also belongs to the class \texttt{"listof"}.
  
<<fig=TRUE>>=
plot(gorillas, which.marks=1, chars=c(1,3), cols=2:3, main="Gorilla nest sites")
@ 

The \texttt{vegetation} covariate is also available as a raw ASCII format file,
<<eval=FALSE>>=
system.file("rawdata/gorillas/vegetation.asc", package="spatstat")
@ 

\subsubsection*{\texttt{hamster}: Hamster kidney cells}

 Cell nuclei in hamster kidney, each nucleus classified as
 either `dividing' or `pyknotic'.
 A multitype point pattern.
 
<<fig=TRUE>>=
plot(hamster, cols=c(2,4))
@ 

\subsubsection*{\texttt{heather}: Heather mosaic}

The spatial mosaic of vegetation of the heather plant,
recorded in a 10 by 20 metre sampling plot in Sweden.
A list with three entries, representing the same data at
different spatial resolutions.
  
<<fig=TRUE>>=
plot(heather$coarse)
@ 

Type the following to see all three images:
<<eval=FALSE>>=
plot(heather)
@ 

\subsubsection*{\texttt{humberside}: Childhood Leukemia and Lymphoma}

Spatial locations of cases of childhood leukaemia
and lymphoma, and randomly-selected controls,
in North Humberside.
A marked point pattern.

<<fig=TRUE>>=
plot(humberside)
@ 

The dataset \texttt{humberside.convex} is an object of the
same format, representing the same point pattern data,
but contained in a larger, 5-sided convex polygon.

\subsubsection*{\texttt{hyytiala}: Mixed forest}

Spatial locations and species classification for
trees in a Finnish forest.

<<fig=TRUE>>=
plot(hyytiala, cols=2:5)
@ 

\subsubsection*{\texttt{japanesepines}: Japanese black pine saplings}

Locations of Japanese black pine saplings
in a square sampling region in a natural forest.
Often used as a standard example.

<<fig=TRUE>>=
plot(japanesepines)
@ 

\subsubsection*{\texttt{lansing}: Lansing Woods}

Locations and botanical classification of trees in a forest.
A multitype point pattern with 6 different types of points.
Includes duplicated points.

<<fig=TRUE>>=
plot(lansing)
@ 

Type the following to see 6 plots, each showing the location
of one of the types of points:

<<eval=FALSE>>=
plot(split(lansing))
@ 

\subsubsection*{\texttt{longleaf}: Longleaf Pines}

Locations and diameters of Longleaf pine trees.
  
<<fig=TRUE>>=
plot(longleaf)
@ 

\subsubsection*{\texttt{meningitis}: Meningitis Cases}

Spatial locations of cases of invasive meningitis in Germany,
labelled by serotype. Auxiliary data gives average population density
in each administrative district.
  
<<fig=TRUE>>=
pa <- function(i) {
   if(i == 1) list(cols=c("red", "green")) else 
   list(do.col=TRUE, col=grey(seq(1,0,length=32)))
}
plot(meningitis, panel.args=pa)
@ 

\subsubsection*{\texttt{mucosa}: Gastric Mucosa Cells}

A bivariate inhomogeneous point pattern, giving the locations of
the centres of two types of cells in a cross-section of the
gastric mucosa of a rat.
  
<<fig=TRUE>>=
plot(mucosa, chars=c(1,3), cols=c("red", "green"))
plot(mucosa.subwin, add=TRUE, lty=3)
@ 

\subsubsection*{\texttt{murchison}: Murchison Gold Deposits}

Spatial locations of gold deposits and associated
geological features in the Murchison area of Western Australia.
A list of three elements:
\begin{itemize}
\item \texttt{gold}, the point pattern of gold deposits;
\item \texttt{faults}, the line segment pattern of geological faults;
\item \texttt{greenstone}, the subregion of greenstone outcrop.
\end{itemize}

Type the following to see the full Murchison dataset:

<<fig=TRUE>>=
plot(murchison$greenstone, main="Murchison data", col="lightgreen")
plot(murchison$gold, add=TRUE, pch=3, col="blue")
plot(murchison$faults, add=TRUE, col="red")
@ 

Some analysis of the data uses a rectangle close to the abandoned town
of Reedy:

<<>>=
reedy <- owin(c(580, 650) * 1000, c(6986, 7026) * 1000)
@ 

Here are the data in that area:

<<fig=TRUE>>=
plot(murchison$greenstone[reedy], 
     main="Detail around Reedy township", 
     col="lightgreen")
plot(murchison$gold[reedy], add=TRUE, pch=3, col="blue")
plot(murchison$faults[reedy], add=TRUE, col="red")
@ 

\subsubsection*{\texttt{nbfires}: New Brunswick Fires}

Fires in New Brunswick (Canada) 
with marks giving information about each fire.

<<fig=TRUE>>=
plot(nbfires, use.marks=FALSE, pch=".")
@ 

The following command would show the data for each year
in a separate panel:

<<eval=FALSE>>=
plot(split(nbfires), use.marks=FALSE, chars=".")
@ 

<<fig=TRUE>>=
par(mar=c(0,0,2,0))
plot(split(nbfires)$"2000", which.marks="fire.type",
     main=c("New Brunswick fires 2000", "by fire type"),
     cols=c("blue", "green", "red", "cyan"),
     leg.side="left")
@ 

\subsubsection*{\texttt{nztrees}: New Zealand Trees}

Locations of trees in a forest plot in New Zealand.
Often used as a demonstration example.

<<fig=TRUE>>=
plot(nztrees)
plot(trim.rectangle(as.owin(nztrees), c(0,5), 0), add=TRUE, lty=3)
@ 

\subsubsection*{\texttt{osteo}: Osteocyte Lacunae}

Replicated three-dimensional point patterns:
the three-dimensional locations of 
  osteocyte lacunae observed in rectangular volumes of
  solid bone using a confocal microscope.
A \texttt{hyperframe} containing 3D point patterns
and explanatory variables.
  
  
<<fig=TRUE>>=
plot(osteo[1:4,], main.panel="", pch=21, bg='white')
@ 

For demonstration and instruction purposes, the 
raw data from the 36th point pattern are available in a plain ascii file in the
\texttt{spatstat} installation,
<<eval=FALSE>>=
system.file("rawdata/osteo/osteo36.txt", package="spatstat")
@ 

\subsubsection*{\texttt{paracou}: Kimboto trees}

Point pattern of adult and juvenile Kimboto trees
recorded at Paracou in French Guiana.
A bivariate point pattern.

<<fig=TRUE>>=
plot(paracou, cols=2:3, chars=c(16,3))
@ 

\subsubsection*{\texttt{ponderosa}: Ponderosa Pines}

Locations of Ponderosa Pine trees in a forest.
Several special points are identified.

<<fig=TRUE>>=
ponderosa.extra$plotit()
@  %$

\subsubsection*{\texttt{pyramidal}: Pyramidal Neurons in Brain}

Locations of pyramidal neurons in sections of human brain.
There is one point pattern from each of 31 human subjects.
The subjects are divided into three groups:
controls (12 subjects), schizoaffective (9  subjects)
and schizophrenic (10 subjects).

To reduce space, we show only the odd-numbered patterns:
<<>>=
pyr <- pyramidal[c(FALSE,TRUE), ]
@ 

<<fig=TRUE>>=
pyr$grp <- abbreviate(pyr$group, minlength=7)
plot(pyr, quote(plot(Neurons, pch=16, main=grp)), main="Pyramidal Neurons")
@ 

\subsubsection*{\texttt{redwood}, \texttt{redwood3}, \texttt{redwoodfull}: Redwood seedlings and saplings}

California Redwood seedlings and saplings in a forest.
There are two versions of this dataset:
\texttt{redwood} and \texttt{redwoodfull}.

The \texttt{redwoodfull} dataset is the full data.
It is spatially inhomogeneous in density and spacing of points.

The \texttt{redwood} dataset is a subset of the full data,
selected because it is apparently homogeneous, and has often 
been used as a demonstration example. This comes in two versions
commonly used in the literature:
\texttt{redwood} (coordinates given to 2 decimal places)
and \texttt{redwood3} (coordinates given to 3 decimal places).


<<fig=TRUE>>=
plot(redwood)
plot(redwood3, add=TRUE, pch=20)
@ 

<<fig=TRUE>>=
redwoodfull.extra$plotit()
@ %$

\subsubsection*{\texttt{residualspaper}: Data from residuals paper}

Contains the point patterns used as examples in 
\begin{quote}
  A. Baddeley, R. Turner, J. M{\o}ller and M. Hazelton (2005)
  Residual analysis for spatial point processes.
  \emph{Journal of the Royal Statistical Society, Series B}
  \textbf{67}, 617--666
\end{quote}
along with {\sf R} code.

<<fig=TRUE>>=
plot(as.solist(residualspaper[c("Fig1", "Fig4a", "Fig4b", "Fig4c")]), 
     main="")
@ 

\subsubsection*{\texttt{shapley}: Shapley Galaxy Concentration}

Sky positions of 4215 galaxies in the Shapley Supercluster
(mapped by radioastronomy).

<<fig=TRUE>>=
shapley.extra$plotit(main="Shapley")
@  %$

\subsubsection*{\texttt{shelling}: Artillery shell impacts}

Spatial locations of artillery shell impacts in a field in Ukraine.

<<fig=TRUE>>=
plot(shelling, pch=3)
plot(onearrow(830, 400, 830, 530, "N"), add=TRUE)
@

\subsubsection*{\texttt{simdat}: Simulated data}

Another simulated dataset used for demonstration purposes.

<<fig=TRUE>>=
plot(simdat)
@ 

\subsubsection*{\texttt{spiders}: Spider webs}

Spider webs across the mortar lines of a brick wall. 
A point pattern on a linear network.

<<fig=TRUE>>=
plot(spiders, pch=16, show.window=FALSE)
@ 

\subsubsection*{\texttt{sporophores}: Sporophores}

Sporophores of three species of fungi around a tree.

<<fig=TRUE>>=
plot(sporophores, chars=c(16,1,2), cex=0.6)
points(0,0,pch=16, cex=2)
text(15,8,"Tree", cex=0.75)
@ 

\subsubsection*{\texttt{stonetools}: 
  Palaeolithic stone tools and bone fragments}

Palaeolithic tools and bone fragments
uncovered in an archaeological dig
in Tanzania. Each find is marked by its type (either \texttt{BONE}
or \texttt{LITHIC}) and by the height $z$ (above a reference level)
of the location in the soil where it was found.

<<fig=TRUE>>=
plot(stonetools, which.marks=2, cols=c(2,3), chars=c(1,3), cex=0.5)
@ 

\subsubsection*{\texttt{spruces}: Spruces in Saxony}

Locations of Norwegian spruce trees 
in a natural forest stand in Saxonia, Germany.
Each tree is marked with its diameter at breast height.
 
<<fig=TRUE>>=
plot(spruces, maxsize=min(nndist(spruces)))
@ 

\subsubsection*{\texttt{swedishpines}: Swedish Pines}

Locations of pine saplings
in a Swedish forest.
Often used as a demonstration example.

<<fig=TRUE>>=
plot(swedishpines)
@ 

\subsubsection*{\texttt{urkiola}: trees in a wood}

Locations of birch and oak trees  in a secondary wood in
Urkiola Natural Park (Basque country, northern Spain). 
Irregular window, bivariate point pattern.

<<fig=TRUE>>=
plot(urkiola, cex=0.5, cols=2:3)
@ 

\subsubsection*{\texttt{waka}: trees in Waka National Park}

Spatial coordinates of each tree, marked by the tree diameter at breast height.
    
<<fig=TRUE>>=
par(mar=c(0,0,2,0))
plot(waka, markscale=0.04, main=c("Waka national park", "tree diameters"))
@ 

\subsubsection*{\texttt{vesicles}: synaptic vesicles}

Point pattern of synaptic vesicles observed in rat brain tissue.

<<fig=TRUE>>=
v <- rotate(vesicles, pi/2)
ve <- lapply(vesicles.extra, rotate, pi/2)
plot(v, main="Vesicles")
plot(ve$activezone, add=TRUE, lwd=3)
@ 

The auxiliary dataset \texttt{vesicles.extra} is a list with entries\\ 
\begin{tabular}{ll}
  \texttt{presynapse} & outer polygonal boundary of presynapse \\
  \texttt{mitochondria} & polygonal boundary of mitochondria \\
  \texttt{mask} & binary mask representation of vesicles window \\
  \texttt{activezone} & line segment pattern representing the active zone.
\end{tabular}

For demonstration and training purposes,
the raw data files for this dataset are also
provided in the \pkg{spatstat} package installation:\\ 
\begin{tabular}{ll}
  \texttt{vesicles.txt} &  spatial locations of vesicles \\
  \texttt{presynapse.txt} &  vertices of \texttt{presynapse} \\
  \texttt{mitochondria.txt}  &  vertices of \texttt{mitochondria} \\
  \texttt{vesiclesimage.tif}  &  greyscale microscope image \\
  \texttt{vesiclesmask.tif}  &  binary image of \texttt{mask} \\
  \texttt{activezone.txt}  &  coordinates of \texttt{activezone} 
\end{tabular}
The files are in the folder \texttt{rawdata/vesicles} in the
\texttt{spatstat} installation directory. The precise location of the
files can be obtained using \texttt{system.file}, for example
<<eval=FALSE>>=
system.file("rawdata/vesicles/mitochondria.txt", package="spatstat")
@ 

\subsubsection*{\texttt{waterstriders}: Insects on a pond}

Three independent replications of a point pattern
formed by insects on the surface of a pond.
  
<<fig=TRUE>>=
plot(waterstriders)
@ 

\end{document}

\documentclass[11pt]{article}

% \VignetteIndexEntry{Missing or unavailable (NA) objects in Spatstat}

\usepackage{graphicx}
\usepackage{Sweave}
\usepackage{bm}
\usepackage{anysize}

\marginsize{2cm}{2cm}{2cm}{2cm}

\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\R}{{\sf R}}
\newcommand{\spst}{\pkg{spatstat}}
\newcommand{\spg}{\pkg{spatstat.geom}}

\begin{document}
\bibliographystyle{plain}
\thispagestyle{empty}

<<echo=FALSE,results=hide,fig=FALSE>>=
options(SweaveHooks=list(fig=function() par(mar=c(1,1,1,1))))
@ 
\SweaveOpts{eps=TRUE}
\setkeys{Gin}{width=0.6\textwidth}

<<echo=FALSE,results=hide>>=
library(spatstat)
requireversion(spatstat.geom, "3.5-0.003")
spatstat.options(image.colfun=function(n) { grey(seq(0,1,length=n)) })
sgversion <- read.dcf(file = system.file("DESCRIPTION", 
                                         package = "spatstat.geom"),
                      fields = "Version")
options(useFancyQuotes=FALSE)
set.seed(42) # for repeatability
@ 

\title{Missing or unavailable (NA) objects in \texttt{spatstat}}
\author{Adrian Baddeley}
\date{\today\\ For \spg\ version \texttt{\Sexpr{sgversion}}}
\maketitle

\begin{abstract}
This document describes experimental new code in \spst\ 
which supports missing or unavailable (NA) objects.
\end{abstract}

\tableofcontents
\newpage

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Introduction}

This document describes a new, experimental feature of \spst\ which supports
missing or unavailable (``\code{NA}'') spatial objects.

The base \R\ system 
allows for missing or unavailable entries
in a numeric vector, logical vector, character vector and so on.
The value \code{NA} is assigned to these missing
entries. 
% Data which include \code{NA} values can be handled by 
% the vast majority of functions in base \R. 

Similarly the new code in \spst\ allows for missing or unavailable entries
in a list of \textbf{spatial objects}.
For example, in a list of spatial point patterns, 
one of the entries in the list 
could be designated as missing or unavailable
--- that is, the entire point pattern is not available.
This could happen because a microscope slide was broken,
a patient refused to participate, a simulation algorithm failed
to generate a realisation, etc.

Additionally the new code in \spst\ allows for missing or unavailable 
\textbf{entries in a hyperframe}.
For example, in a hyperframe representing the results of a designed
experiment, in which the response from each experimental unit is a 
spatial point pattern, the column of point patterns could include
entries which are missing or unavailable. There could also be 
unavailable entries in a column of pixel images, and so on.

There are two ways to indicate that an entry in a list or hyperframe
is missing/unavailable:
\begin{enumerate}
\item a ``missing object'' can be created using the function \code{NAobject}.
\item in an existing list or hyperframe, the relevant entry can be assigned
  the value \code{NA}, and this will be coerced to a missing object.
\end{enumerate}


\section{Creating a missing object}

\subsection{A missing object belongs to a particular class}

In base \R, there are \code{NA} values
of different types. The missing
entries in a numeric vector are numeric \code{NA} values
(equal to \verb!NA_real_!) 
so that the vector is nevertheless treated as
a vector of numeric values.
The missing entries in a character vector
are character \code{NA} values (\verb!NA_character_!), and so on.
We use a similar approach in \spst: each missing/unavailable object 
\emph{belongs to a particular class}.

\subsection{Creating a missing object}

In \spst, use the function \code{NAobject} to create a missing or unavailable 
\textbf{object}.

To create a missing object that belongs to class \code{"foo"},
use \texttt{NAobject("foo")}. For example, a 
missing spatial point pattern (class \code{"ppp"}) is created by:

<<>>=
X <- NAobject("ppp")
X
@ 

\noindent
The printout indicates that \spst\ recognises \code{X}
as a missing object of class \code{"ppp"}.

A missing or unavailable object of any particular class \code{"foo"}
is represented in \spst\ by an object of class \code{c("NAobject", "foo")}.
For example, a missing spatial point pattern (class \code{"ppp"}) is
represented as an object of class \code{c("NAobject", "ppp")}. This ensures
that the object is treated as both a point pattern and a missing object
in appropriate circumstances.

A missing object can be included as an entry in a list:

<<>>=
pats <- solist(cells, NAobject("ppp"), redwood)
pats
@ 

\noindent
The printout indicates that the second entry in the list is a missing
point pattern, and the entire list is nevertheless a list of point patterns.

A missing object can be included in a column of a hyperframe:

<<>>=
m <- hyperframe(X=runif(3), Y=pats)
m
@

\section{Recognising a missing object}

\subsection{Testing whether an object is a missing object}

The function \code{is.NAobject} can be used to test whether an
object is a missing/unavailable object.

<<>>=
Z <- NAobject("ppp")
is.NAobject(Z)
is.NAobject(cells)
@ 

Of course one could also use \code{inherits}:

<<>>=
inherits(Z, what="NAobject")
@ 

\subsection{Missing entries in a list of objects}

The generic function \code{is.na} is used in base \R\ to 
determine which \textbf{entries} of a vector or matrix are missing values.

Similarly in \spst, for lists which belong to class \code{"solist"},
\code{"ppplist"}, \code{"imlist"} or \code{"anylist"}, 
missing entries can be detected using methods for \code{is.na}:

<<>>=
is.na(pats)
@ 

For a list that does not belong to one of these special types,
there may not be a method for \texttt{is.na}. Instead
one could use \code{lapply} and friends:

<<>>=
U <- list(cells, Z, cells)
sapply(U, is.NAobject)
sapply(U, inherits, what="NAobject")
@ 

\subsection{Missing entries in a hyperframe}

A missing object can also be included as an entry in a hyperframe:

<<>>=
h <- hyperframe(z=1:3, p=pats)
h
@ 

The generic \code{is.na} has a method for hyperframes, and returns 
a logical matrix indicating whether each entry of the hyperframe
is missing:

<<>>=
is.na(h)
@ 

\section{Coercion of \code{NA} to \code{NAobject}}

\subsection{Coercion of \code{NA} in base \R}

In base \R, an assignment of the form \verb!x[i] <- NA!
works for an atomic vector \code{x} of any type. 
The \code{NA} will be converted (`coerced') to an \code{NA} value of the type 
appropriate to \code{x}. For example:

<<>>=
blah <- letters[1:4]
blah[2] <- NA
blah
@ 

\noindent In this case \code{blah} is a character vector,
so the \code{NA} has been coerced to a character value:

<<>>=
is.character(blah[2])
identical(blah[2], NA_character_)
@ 

\subsection{Coercion of \code{NA} in \spst}

Similarly in \spst\ an assignment of the form \verb!x[i] <- NA! or
\verb!x[[i]] <- NA! works for a \textbf{list of objects of the same class}.
The value \code{NA} will be coerced to 
an \code{"NAobject"} of the appropriate class.
For example:

<<>>=
Y <- rpoispp(10, nsim=3)
Y[[2]] <- NA
Y
@

Here \code{Y} is a list of point patterns (objects of class \code{"ppp"})
so \code{NA} is coerced to \code{NAobject("ppp")}.
The coercion can occur when the list is created:

<<>>=
solist(cells, NA, redwood)
@ 

(Note that, for lists of class \code{"solist"} or \code{"anylist"}, 
this only works if all of the non-missing entries belong to the same class, 
so that the intended class is unambiguous.)

Similarly in hyperframes,

<<>>=
g <- hyperframe(A=letters[1:3], B=rpoispp(10, nsim=3), D=runif(3))
g
g[2,2] <- NA
g
@ 

Each individual \code{NA} entry will be coerced to the appropriate
kind of missing value:

<<>>=
g[3, ] <- NA
g
@ 

If an entire column of a hyperframe is replaced by \code{NA}, 
the result will be an atomic column of logical \code{NA} values
(since otherwise the intended class of objects is ambiguous):

<<>>=
g[,2] <- NA
g
@ 


\section{Handling missing objects}

\subsection{Basic support}

Missing objects are handled by the code for
\begin{enumerate}
\item creating lists of class 
\code{"solist"}, \code{"ppplist"}, \code{"imlist"} or \code{"anylist"}
\item creating hyperframes
\item extracting or replacing subsets of
  a list of class \code{"solist"}, \code{"ppplist"}, \code{"imlist"} 
  or \code{"anylist"}
\item extracting or replacing subsets of a hyperframe
\item printing and plotting 
\end{enumerate}

There are methods for \code{print}, \code{plot} and \code{summary}
for the class \code{"NAobject"}. The \code{print} and \code{summary}
methods simply indicate that the object is missing. The \code{plot}
method does not generate a plot, and just prints a message that the
object was missing.

\subsection{Most functions in \spst\ do not recognise \texttt{NA} objects}

Most functions in \spst\ do not
handle objects of class \code{"NAobject"}. For example the 
following would generate an \textbf{error}:

<<eval=FALSE>>=
  X <- NAobject("ppp") 
  K <- Kest(X)
@ 

The object \code{X} is recognised as a point pattern, but does not contain
any of the data that are expected for such an object, so \code{Kest} will 
fail with some peculiar error message. 

Such eventualities can be handled by checking the object first:

<<>>=
  X <- NAobject("ppp")
  K <- if(is.NAobject(X)) NAobject("fv") else Kest(X)
@ 


\subsection{\texttt{solapply} and friends}

\texttt{NA} objects are automatically handled by the \spst\ functions
 \texttt{solapply},  \texttt{anylapply} and \texttt{with.hyperframe}.

\subsubsection{\texttt{solapply} and \texttt{anylapply}}

The functions \texttt{solapply} and \texttt{anylapply} are wrappers for
\texttt{lapply}, called in the form
\begin{verbatim}
    solapply(X, FUN, ...)
    anylapply(X, FUN, ...)
\end{verbatim}
where \texttt{X} is a list and \texttt{FUN} is a function that will be applied
to each element of \texttt{X}. The difference is that \texttt{solapply} 
expects the results to be spatial objects (e.g.\ point patterns, windows), 
while \texttt{anylapply} allows them to be any kind of object 
(e.g.\ numbers, matrices, \verb!"fv"! objects).

The functions \texttt{solapply} and \texttt{anylapply} now check whether 
any elements of \texttt{X} are missing or unavailable, and if so,
they return an \texttt{NAobject} as the result for each such element.
The function \texttt{FUN} is only applied to the entries which are not
missing. 

For example, using the list of point patterns \texttt{pats}
which contains some missing entries:

<<>>=
A <- solapply(pats, Window)
B <- anylapply(pats, Kest)
D <- solapply(pats, Kest, demote=TRUE)
E <- anylapply(pats, npoints)
@ 

These tricks do \textbf{not} work with the base \R\ functions
\texttt{lapply}, \texttt{sapply} etc.

\subsubsection{\texttt{with.hyperframe}}

The \spst\ function \texttt{with.hyperframe} is a method for the 
generic \texttt{with}. It evaluates a given expression in each row of the
hyperframe, and returns a list containing the result for each row.

This function now checks for missing or unavailable entries in the 
hyperframe, and if they are needed to evaluate the expression, the result
is returned as an \texttt{NAobject} for each row in which the data are missing.

For example, using the hyperframe \texttt{m} which contains some missing
entries:

<<>>=
K <- with(m, Kest(Y))
m$G <- with(m, Gest(Y))
m$u <- with(m, clarkevans.test(Y))
with(m, u$p.value)
@ 




\end{document}

\documentclass[10pt]{article}
\usepackage{graphicx}
\usepackage{Sweave}
\usepackage{bm}
\usepackage[bottom=0.5cm, right=1.5cm, left=1.5cm, top=1.5cm]{geometry} 

% \VignetteIndexEntry{Bugs Fixed in Spatstat}

%  $Revision: 1.37 $ $Date: 2026/05/18 06:38:53 $

\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\R}{{\sf R}}
\newcommand{\spst}{\pkg{spatstat}}
\newcommand{\Spst}{\pkg{Spatstat}}

\begin{document}
\bibliographystyle{plain}

<<echo=FALSE,results=hide>>=
library(spatstat)
x <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
              fields = c("Version", "Date"))
sversion <- as.character(x[,"Version"])
sdate    <- as.character(x[,"Date"])
options(useFancyQuotes=FALSE)
@ 

\title{Bugs fixed in \spst}
\author{Adrian Baddeley, Rolf Turner and Ege Rubak}
\date{For \spst\ version \texttt{\Sexpr{sversion}}}
\maketitle

\thispagestyle{empty}

This vignette lists all \emph{important} bugs detected and fixed in the
\spst\ package since 2010. It also explains how to search the list of
all recorded bugs in the \spst\ family of packages.

<<echo=FALSE,results=hide>>=
nbugs      <- nrow(bugfixes("all",  show=FALSE))
nbugssince <- nrow(bugfixes("book", show=FALSE))
@ 

\tableofcontents

\pagebreak
\section{Bug history}

Thousands of bugs have been detected and fixed in \spst\ since it was
first released in 2001. We started recording the bug history in 2010.

\subsection{Documentation of bugs}

Bugs that may have affected the user are listed in the 
package \texttt{NEWS} file, and can be searched
using the \R\ command \texttt{news} or the \spst\ command \texttt{bugfixes}. 

To see the bugs which have just been fixed in the latest version of \spst, type 
<<eval=FALSE>>=
bugfixes
@ 
To see all bugs which were fixed after a particular version of \spst,
for example, bugs that were fixed in version \texttt{1.50-0} or later, type
<<eval=FALSE>>=
bugfixes(sinceversion="1.50-0")
@ 
To see all bugs in \spst\ that were fixed after a particular date,
for example 30 June 2017, type
<<eval=FALSE>>=
bugfixes(sincedate="2017-06-30")
@ 
To see all bugs fixed after the book \cite{baddrubaturn15} was written, type
<<eval=FALSE>>=
bugfixes("book")
@ 
To see all bugs in the entire recorded history of \spst, type
<<eval=FALSE>>=
bugfixes("all")
@ 
which currently produces a list of \Sexpr{nbugs} bugs,
of which \Sexpr{nbugssince} were detected after publication of the
book \cite{baddrubaturn15}.

\subsection{Bugs in the \texttt{spatstat} family of packages}

Recently \spst\ was divided into a family of sub-packages.
The command \texttt{bugfixes} now covers the bug history in 
all of these sub-packages. See the help for \texttt{bugfixes} 
for further details.

<<echo=FALSE,results=hide>>=
getstuff <- function(pkg) {
  x <- read.dcf(file=system.file("DESCRIPTION", package=pkg),
                fields=c("Version", "Date"))
  xversion <- as.character(x[,"Version"])
  xdate    <- as.character(x[,"Date"])
  data.frame(date=as.Date(xdate), package=pkg, version=xversion)
}
vtable <- do.call(rbind,
                  lapply(c("spatstat.utils",
                           "spatstat.data", 
                           "spatstat.sparse",
                           "spatstat.geom",
                           "spatstat.random",
                           "spatstat.explore",
                           "spatstat.model",
                           "spatstat.linnet",
                           "spatstat"),
                         getstuff))
@ 

The current versions of the \spst\ family of packages
(used to produce this document) are:

<<echo=FALSE>>=
print(vtable, row.names=FALSE)
@ 

\pagebreak
\section{List of bugs}

Following is a list of the {\bf most serious bugs}, 
in decreasing order of potential impact.

A bug is classified as ``serious'' if it produced incorrect results
without the user's knowledge.

Bugs which cause an error exit are not listed here, because 
the presence of a bug is obvious, and the bug would not have misled the user.

\newcommand\bugger[4]{%  
  \\  {} %
  {\small (Bug introduced in \texttt{spatstat {#1}}, {#2}; %
    fixed in \texttt{spatstat {#3}}, {#4})}%
}
\newcommand\bugin[4]{%  
  \\  {} %
  {\small (Bug introduced in \texttt{{#1}}, {#2}; %
    fixed in \texttt{{#3}}, {#4})}%
}
\newcommand\bugbefore[4]{%  
  \\  {} %
  {\small (Bug introduced before \texttt{{#1}}, {#2}; %
    fixed in \texttt{{#3}}, {#4})}%
}
\newcommand\bugalways[2]{\bugbefore{spatstat 1.19-0}{may 2010}{#1}{#2}}


  %%% LEVEL 1
\subsection{Serious Bugs, Always Wrong, Broad Impact} 

\begin{itemize}
\item \texttt{nncross.ppp}:
  Results were completely incorrect if $k > 1$.
  \bugger{1.31-2}{april 2013}{1.35-0}{december 2013}

\item \texttt{nncross.pp3}:
  Results were completely incorrect in some cases.
  \bugger{1.32-0}{august 2013}{1.34-0}{october 2013}

 \item \texttt{cdf.test.ppm}:
    Calculation of $p$-values was incorrect for Gibbs models: 
    $1-p$ was computed instead of $p$.
  \bugger{1.40-0}{december 2014}{1.45-2}{may 2016}

\item \texttt{Smooth.ppp}:
  Results of \verb!Smooth(X, at="points", leaveoneout=FALSE)!
  were completely incorrect.
  \bugger{1.20-5}{august 2010}{1.46-0}{july 2016}
  
\item \texttt{rmh}:
  
   \begin{itemize}
   \item Simulation was completely incorrect in the case of 
     a multitype point process with an interaction that does not depend
     on the marks, such as \verb!ppm(betacells, ~marks, Strauss(60))!
     due to a coding error in the \texttt{C} interface.
     \bugger{1.22-3}{march 2010}{1.22-3}{june 2011}
   \item 
     Simulation of the Area-Interaction model was completely incorrect.
     \bugger{1.23-6}{october 2011}{1.31-0}{january 2013}
   \item 
     Simulation of the Geyer saturation process was completely incorrect.
     \bugger{1.31-0}{january 2013}{1.31-1}{march 2013}
   \item 
     Simulation of the Strauss-Hard Core process was partially incorrect,
     giving point patterns with a slightly lower intensity.
     \bugger{1.31-0}{january 2013}{1.37-0}{may 2014}
   \item 
     Simulation of the \emph{multitype} hard core model
     was completely incorrect (the interaction was effectively removed,
     changing the model into a Poisson process).
     \bugger{1.31-0}{january 2013}{1.63-0}{january 2020}   
   \item
     The result of simulating a model with a hard core
     did not necessarily respect the hard core constraint,
     and simulation of a model with strong inhibition
     did not necessarily converge. 
     This only happened if the first order trend was large,
     the starting state (\texttt{n.start} or \texttt{x.start}) was not given,
     and the number of iterations \texttt{nrep} was not very large.
     It occurred because of a poor choice for the default starting state.
     {\small (Bug was present since about 2010.  
       Fixed in \texttt{spatstat 1.40-0}, december 2014)}
   \item 
     Simulation was incorrect in the case of an inhomogeneous multitype model
     with \texttt{fixall=TRUE} (i.e.\ with a fixed number of points 
     of each type) if the model was segregated (i.e.\ if different types
     of points had different first order trend). 
     The effect of the error was that all types of points
     had the same first order trend.
     {\small (Bug was present since about 2010.
       Fixed in \texttt{spatstat 1.43-0}, september 2015)}
  \item 
     Simulation of the Geyer saturation process was 
     incorrectly initialised, so that the results of a short run 
     (i.e. small value of \texttt{nrep}) were incorrect, 
     while long runs were correct.
     \bugger{1.17-0}{october 2009}{1.31-1}{march 2013}
   \end{itemize}

\item \texttt{nnmark, as.im.ssf}:
  If \code{marks(X)} was a matrix rather than a data frame,
  the results were completely incorrect.
  \bugger{1.32-0}{august 2013}{1.55-1}{april 2018}

 \item \texttt{rVarGamma}:
   Simulations were incorrect; they were generated using the wrong value
   of the parameter \texttt{nu.ker}.  
   \bugger{1.25-0}{december 2011}{1.35-0}{december 2013}

 \item \texttt{rCauchy}:
   Simulations were incorrect; they were generated using the wrong value
   of the parameter \texttt{omega}.
   \bugger{1.25-0}{december 2011}{1.25-2}{january 2012}
   
 \item \texttt{lppm}:
   For multitype patterns, the fitted model was completely incorrect
   due to an error in constructing the quadrature scheme.
   \bugger{1.23-0}{july 2011}{1.30-0}{december 2012}
   
 \item \verb![.lpp!:
   The local coordinate \texttt{seg} was completely incorrect,
   when \texttt{i} was a window.
   \bugger{1.31-2}{april 2013}{1.45-0}{march 2016}
   
 \item \texttt{lohboot}:  Implementation was completely incorrect.
   \bugger{1.26-1}{april 2012}{1.53-2}{october 2017}
   
 \item \texttt{leverage.ppm}, \texttt{influence.ppm}, \texttt{dfbetas.ppm}:
   Results were incorrect for non-Poisson processes
   due to a mathematical error.
   \bugger{1.25-0}{december 2011}{1.51-0}{may 2017}

 \end{itemize}
 
  %%% LEVEL 2
 \subsection{Serious Bugs, Often Completely Wrong, Moderate Impact}

 \begin{itemize}
 \item \texttt{MinkowskiSum}, operator \verb!%(+)%!:
   Result was often incorrect for non-convex sets
    (and was not symmetric in \texttt{A} and \texttt{B}).
    \bugbefore{spatstat 1.65-0}{december 2020}{spatstat.geom 3.7-2}{march 2026} 
    
 \item \texttt{pairdist.lpp}:
   Results could have been completely incorrect, due to an internal bug,
    if the linear network data was in the non-sparse representation.
    \bugbefore{spatstat 1.65-0}{december 2020}{spatstat.linnet 3.0-3}{january 2023}
    
 \item \texttt{matrixsqrt}, \texttt{matrixinvsqrt}, \texttt{matrixpower}:
    If the result was a complex-valued matrix, the values were completely 
    incorrect.
    \bugin{spatstat 1.48-0}{december 2016}{spatstat.sparse 2.1-1}{february 2021}

  \item \texttt{pcf.ppp}:
    Estimates were incorrectly scaled (they were incorrectly multiplied
    by the area of the window.) Spotted by Maximilian Hesselbarth.
    \bugin{spatstat.explore 3.0-0}{may 2022}{spatstat.explore 3.0-6}{january 2023}
    
  \item \texttt{rLGCP}, \texttt{simulate.kppm}:
    Simulation results for log-Gaussian Cox processes were incorrect
    unless the pixel dimensions and pixel spacings were identical
    on the horizontal and vertical axes. (If pixel dimensions were not
    specified, then the results were incorrect whenever the Frame of the
    simulation window was not a square.) 
   \bugger{1.22-2}{june 2011}{1.65-0}{december 2020}   

 \item \texttt{rLGCP}, \texttt{simulate.kppm}:
    Simulation results for log-Gaussian Cox processes were incorrect
    unless the pixel dimensions and pixel spacings were identical
    on the horizontal and vertical axes. (If pixel dimensions were not
    specified, then the results were incorrect whenever the Frame of the
    simulation window was not a square.) 
   \bugin{spatstat.random 3.2-0}{oct 2023}{spatstat.random 3.2-4}{june 2024}
      
  \item \texttt{deviance.lppm}, \texttt{pseudoR2.lppm}:
    Results were completely incorrect, due to a coding error.
   \bugger{1.44-0}{december 2015}{1.64-2}{november 2020}   

  \item \texttt{kppm}: 
    Results were sometimes incorrect for \texttt{method='clik2'}
    and \texttt{method='palm'}
    because the log composite likelihood was erroneously truncated to
    positive values. Any fitted model for which \verb!logLik(model) = 2.2e-16!
    should be suspected of being incorrect.
    \bugin{spatstat.core 2.0-0}{march 2021}{spatstat.core 2.3-2}{november 2021}   
    
 \item \texttt{bw.pcf}:
   Results were totally incorrect due to a typo.
   \bugger{1.51-0}{may 2017}{1.52-0}{august 2017}   

  \item \texttt{density.ppp}: 
    edge correction factors were calculated incorrectly when
    the window was not a rectangle, causing a negative bias in
    the estimated intensity.
    \bugger{1.57-0}{oct 2018}{1.64-0}{april 2020}.
    
 \item \texttt{density.ppp}:
   The standard error (calculated when \texttt{se=TRUE})
   was incorrect when \texttt{sigma} was a single numeric value.
   The output was equal to \texttt{sqrt(sigma)} times the correct answer.
   \bugger{1.41-1}{february 2015}{1.57-0}{october 2018}   
   
 \item \texttt{density.ppp}:
   The standard error (calculated when \texttt{se=TRUE})
   was incorrect for non-Gaussian kernels.
   \bugbefore{spatstat 1.57-0}{october 2018}{spatstat.explore 3.2-0}{may 2023}
  
 \item \texttt{relrisk.ppp}:
   The standard error (calculated when \texttt{se=TRUE})
   was incorrect for non-Gaussian kernels.
   \bugbefore{spatstat 1.57-0}{october 2018}{spatstat.explore 3.2-0}{may 2023}
  
  \item \texttt{rthin}:
   If \texttt{P} was close to 1, the result was sometimes an empty point pattern
   when it should have been identical to \texttt{X}.
   \bugger{1.43-0}{october 2015}{1.57-0}{october 2018}   
   
 \item \texttt{predict.mppm}:
   If the model included random effects, and if the library \pkg{MASS} was not loaded,
   the predictions were on the log scale (i.e.\ they were logarithms of the correct values).
   \bugger{1.43-0}{october 2015}{1.55-1}{april 2018}   
   
 \item \texttt{nnmap}, \texttt{nnmark}:
   Values were incorrect if the resulting pixel image
    had unequal numbers of rows and columns.
   \bugger{1.35-0}{december 2013}{1.55-0}{january 2018}   

 \item \texttt{vcov.mppm}:
    Format was incorrect (rows/columns were omitted) in some cases.
   \bugger{1.45-1}{may 2016}{1.55-0}{january 2018}   

  \item \texttt{model.matrix.ppm}, \texttt{model.frame.ppm}:
    Values were sometimes incorrect when applied to the result of
    \texttt{subfits}.
    To be precise, if \texttt{fit} was an \texttt{mppm}
    object fitted to a hyperframe
    that included ``design covariates'' (covariates that take a constant value
    in each row of the hyperframe), and if \verb!futs <- subfits(fit)!, then
    \verb!model.matrix(futs[[i]])! gave incorrect values in the columns
    corresponding to the design covariates. 
    \bugger{1.45-1}{may 2016}{1.55-0}{january 2018}   
   
 \item \texttt{predict.rhohat}, \texttt{simulate.rhohat}:
   Results were incorrect for a \texttt{rhohat} object computed
   from linear network data (class \texttt{"lpp"} or \texttt{"lppm"}).
   \bugger{1.31-0}{march 2013}{1.63-1}{february 2020}   

 \item \texttt{predict.rho2hat}:
   Results were incorrect for a \texttt{rho2hat} object computed
   from a point pattern.
   \bugger{1.42-0}{may 2015}{1.52-0}{august 2017}   

\item \texttt{density.ppp}:
  Result was incorrect for non-Gaussian kernels when
  \texttt{at="points"} and \texttt{leaveoneout=FALSE}.
   \bugger{1.47-0}{october 2016}{1.57-0}{october 2018}   
  
\item \texttt{envelope.ppm}:
   If the model was an inhomogeneous Poisson process, 
   the resulting envelope object was incorrect
   (the simulations were correct, but the envelopes were calculated
   assuming the model was CSR). 
   \bugger{1.23-5}{september 2011}{1.23-6}{october 2011}

 \item \texttt{linearK}, \texttt{linearpcf},
   \texttt{linearKinhom}, \texttt{linearpcfinhom}
   and multitype versions:
    These functions were sometimes greatly underestimated
    when the network had segments shorter than 10 coordinate units.
   \bugger{1.44-0}{december 2015}{1.46-2}{july 2016}

 \item \texttt{nncross}, \texttt{distfun}, \texttt{AreaInter}:
  Results of \texttt{nncross} were possibly incorrect 
  when \code{X} and \code{Y} did not have the same window. 
  This bug affected values of \texttt{distfun} and may also 
  have affected ppm objects with interaction \texttt{AreaInter}.
  \bugger{1.9-4}{june 2006}{1.25-2}{january 2012}

 \item \texttt{update.kppm}:
   \begin{itemize}
   \item Did not function correctly when several additional arguments
     were given.
     \bugger{1.42-2}{june 2015}{1.54-0}{november 2017}
   \item 
  If the call to \texttt{update} did not include a formula argument
   or a point pattern argument, then all arguments were ignored. 
   Example: \texttt{update(fit, improve.type="quasi")} was identical to 
   \texttt{fit}.
   \bugger{1.42-2}{june 2015}{1.45-0}{march 2016}
   \end{itemize}

  \item \texttt{markcorrint}:
   Results were completely incorrect.
   \bugger{1.39-0}{october 2014}{1.40-0}{december 2014}

    \item \texttt{leverage.ppm}, \texttt{influence.ppm}, \texttt{dfbetas.ppm}:
   Results were slightly incorrect for models with a hard core,
   due to a mathematical error.
   \bugger{1.51-0}{may 2017}{1.55-1}{april 2018}

    \item \texttt{Ops.msr}:
    If the input data contained a pixel image of the smoothed density,
    this image was not updated; it was copied to the output unchanged.
    Plots of the resulting measure were incorrect. 
    \bugger{1.52-0}{august 2017}{1.55-1}{april 2018}

  \item \verb![.linnet!: 
    in calculating \verb!L[W]! where \texttt{W} is a window,
    the code ignored segments of \code{L} that crossed \code{W}
    without having a vertex in \code{W}.
    \bugger{1.53-0}{september 2017}{1.55-1}{april 2015}
    
  \item \verb!as.im.function!:
    if the function domain was not a rectangle
    and the function values were categorical (factor) values,
    the result was an empty image.
    \bugger{1.42-0}{may 2015}{1.57-0}{october 2018}

  \item \texttt{density.lpp}:
    If \texttt{weights} were given, the results were completely incorrect
    if \texttt{leaveoneout=TRUE} (the default) and \texttt{at="points"}.
    \bugin{spatstat 1.51-0}{may 2017}{spatstat.linnet 3.0-0}{june 2022}
    
  \end{itemize}
  
  %%% LEVEL 3
  \subsection{Bugs, Substantially Incorrect, Moderate Impact}

  \begin{itemize}

  \item \texttt{spatstat.sparse} (numerous calculations):
    Results were sometimes incorrect for symmetric sparse matrices
    inheriting class \texttt{"symmetricMatrix"}.
    \bugin{spatstat.sparse 1.0-0}{may 2020}{spatstat.sparse 3.2-0}{may 2026}    
   
  \item \texttt{fitted.slrm}:
    Calculation was incorrect when \texttt{leaveoneout=TRUE}
    and \texttt{type != "intensity"}.
    \bugbefore{spatstat 1.65-0}{december 2020}{spatstat.model 3.5-0}{november 2025}    
    
  \item \texttt{Kmark}:
    Calculation was incorrect for multitype patterns (categorical marks).
    \bugbefore{spatstat 1.41-0}{january 2015}{spatstat.explore 3.6-0}{november 2025}
    
  \item \texttt{rpoislinetess}:
    Results were incorrect unless the window was centred at the origin.
    \bugbefore{spatstat 1.42-0}{may 2015}{spatstat.random 3.2-1}{october 2023}
    
  \item \texttt{closepairs.ppp}:
    If \texttt{distinct=FALSE} and \texttt{what="all"}, the resulting vectors
    \texttt{yi} and \texttt{yj} contained incorrect values, 
    and had the wrong length.
    \bugbefore{spatstat 1.56-0}{june 2018}{spatstat.geom 2.4-0}{march 2022}

  \item \texttt{nncross.ppp}:
    When \texttt{k > 1}, distance values were incorrectly replaced by
    \texttt{Inf} in some cases.
    \bugbefore{spatstat 1.56-0}{june 2018}{spatstat.geom 2.4-0}{march 2022}
    
  \item \texttt{nncross.ppp}:
    If the argument \texttt{by} was given, some of the results were incorrect.
    [Spotted by Hank Stevens.]
    \bugger{1.32-0}{august 2013}{2.2-0}{june 2021}
    
  \item \verb!"[<-.im"!:
    Incorrect values were assigned in \verb!x[] <- v! 
    when \texttt{x} and \texttt{v} were both factor-valued
    but with different sets of levels.
    \bugbefore{spatstat 1.56-0}{june 2018}{spatstat.geom 3.0-5}{january 2023}
     
 \item \texttt{vcov.mppm}:
    For Gibbs (non-Poisson) models, the variance matrix was
    calculated incorrectly in some cases. 
   \bugin{spatstat 1.45-1}{may 2016}{spatstat.core 2.4-1}{may 2022}   

 \item \texttt{vcov.mppm}:
    Results were sometimes incorrect if the two models
    had different interactions (e.g. Strauss vs Poisson).
   \bugin{spatstat 1.45-1}{may 2016}{spatstat.core 2.4-1}{may 2022}   

 \item \texttt{distmap.owin}:
   If the window was a binary mask, the distance values were
    slightly too large (by a factor \texttt{1 + 1/n} where \texttt{n}
    is the pixel grid dimension).
    \bugalways{spatstat.geom 2.4-0}{march 2022}
    
 \item \texttt{distfun.owin}:
   If the window was a binary mask, the distance values were
    slightly too small (typically reduced by 1/20 of a pixel width).
    \bugalways{spatstat.geom 2.4-0}{march 2022}
    
\item \texttt{nncross.ppp}, \texttt{nncross.pp3}:
  If \texttt{iX} and \texttt{iY} were given, some of the results were incorrect.
  \bugger{1.32-0}{august 2013}{2.2-0}{june 2021}

  \item \texttt{vcov.ppm}:
    Result was sometimes incorrect for Gibbs models.
    The Fisher information was slightly underestimated.
    \bugger{1.31-1}{march 2013}{1.64-1}{may 2020}
    
  \item \texttt{as.linnet.psp}:
    Sometimes produced a network with duplicated segments.
    [Such objects can be repaired using \texttt{repairNetwork}.]
     \bugger{1.41-1}{february 2015}{1.62-0}{december 2019}
    
   \item \texttt{addvar}:
    If the covariate contained \texttt{NA}, \texttt{NaN} or \texttt{Inf} values,
    the calculations were sometimes incorrect.
    \bugger{1.45-0}{march 2016}{.core 2.4-1}{march 2022}
    
    \item \texttt{rlpp}:
       The resulting pattern was unmarked 
       even when it should have been multitype.
       \bugger{1.48-0}{december 2016}{1.63-0}{january 2020}
       
 \item \texttt{spatialcdf}:
   Argument \texttt{weights} was ignored, unless it was a fitted model.
   {\small (Bug was present since about 2010.  
     Fixed in \texttt{spatstat 1.59-0}, march 2019)}

  \item \texttt{ppp}:
    Points inside the window were erroneously rejected as lying outside
    the window, if the window was a polygon equivalent to a rectangle
    with sides longer than $10^6$ units.
     {\small (Bug was present since the beginning.
       Fixed in \texttt{spatstat 1.59-0}, march 2019)}
    
  \item \texttt{inside.owin}:
    All results were \texttt{FALSE}
    if the window was a polygon equivalent to a rectangle
    with sides longer than $10^6$ units.
     {\small (Bug was present since the beginning.
       Fixed in \texttt{spatstat 1.59-0}, march 2019)}
     
   \item \texttt{sumouter}:
     result was incorrect (all entries were zero)
     if \texttt{w} was missing and \texttt{y} was given.
     \bugger{1.47-0}{october 2016}{1.59-0}{march 2019}
     
   \item \texttt{simulate.dppm}, \texttt{simulate.detpointprocfamily}:
    In dimensions higher than 2, the result was shifted so that
    it was centred at the origin.
    \bugger{1.54-0}{december 2017}{1.55-0}{january 2018}   

    \item \texttt{integral.msr}: If the result was a matrix, 
      it was the transpose of the correct answer.
    \bugger{1.35-0}{december 2012}{1.55-1}{april 2018}   
      
\item \texttt{density.ppp}:
  Values of \verb!density(X, at="points")!
  and \verb!Smooth(X, at="points")!
  were sometimes incorrect, due to omission of
  the contribution from the data point with the smallest $x$ coordinate.
  \bugger{1.26-0}{april 2012}{1.46-1}{july 2016}
  
  \item \texttt{multiplicity.default}:
    The first occurrence of any value in the input
    was incorrectly assigned a multiplicity of 1.
    \bugger{1.32-0}{december 2013}{1.57-1}{november 2018}   
    
 \item \texttt{update.ppm}:
   If the argument \texttt{Q} was given,
   the results were usually incorrect, or an error was generated.
   \bugger{1.38-0}{august 2014}{1.38-1}{august 2014}

\item \texttt{subfits}:
    The interaction coefficients of the submodels were incorrect
    for Gibbs models with a multitype interaction (\texttt{MultiStrauss}, etc).
   \bugger{1.35-0}{december 2013}{1.45-2}{may 2016}
   
\item \texttt{F3est}:
   Estimates of $F(r)$ for the largest value of $r$ 
   were wildly incorrect. 
     {\small (Bug was present since about 2010.
       Fixed in \texttt{spatstat 1.48-0}, december 2016)}
   
 \item \texttt{kppm}, \texttt{matclust.estpcf}, \texttt{pcfmodel}:
    The pair correlation function of the M\'atern Cluster Process
    was evaluated incorrectly at distances close to 0.
    This could have affected the fitted parameters 
    in \texttt{matclust.estpcf()} or \texttt{kppm(clusters="MatClust")}.
    \bugger{1.20-2}{august 2010}{1.33-0}{september 2013}
    
 \item \texttt{ppm}:
   Results were incorrect for the Geyer saturation model
   with a non-integer value of the saturation parameter \texttt{sat}.
   \bugger{1.20-0}{july 2010}{1.31-2}{april 2013}
   
 \item \texttt{clip.infline}: 
   Results were incorrect unless the midpoint of the window
   was the coordinate origin.
   \bugger{1.15-1}{april 2009}{1.48-0}{december 2016}
   
 \item \texttt{intensity.ppm}:
   Result was incorrect for Gibbs models if the model was 
    exactly equivalent to a Poisson process (i.e. if all interaction
    coefficients were exactly zero).
   \bugger{1.28-1}{june 2012}{1.47-0}{october 2016}
   
 \item \texttt{idw}: 
   Results were incorrect if 
   \texttt{se=TRUE} and \verb!at="pixels"! and 
   \texttt{power} was not equal to 2.
   The pixel values of \verb!$estimate! were all equal to zero.
   \bugger{1.58-0}{january 2019}{1.63-0}{january 2020}
   
 \item \texttt{funxy}: 
   Did not correctly handle one-line functions. 
   The resulting objects evaluated the wrong function in some cases.
   \bugger{1.45-0}{march 2016}{1.46-0}{july 2016}   

 \item \texttt{kernel.moment}:
   Result was incorrect for \texttt{kernel="cosine"} 
   and \texttt{kernel="optcosine"}.
   \bugger{1.45-2}{may 2016}{1.56-0}{june 2018}   
   
 \item \verb![.msr!:
   Format was mangled if the subset contained exactly one quadrature point.
   \bugger{1.21-3}{january 2011}{1.56-0}{june 2018}   

\item \texttt{hyperframe}:
  Did not correctly handle date-time values (columns of class \texttt{"Date"}, 
  etc).
     \bugger{1.19-1}{may 2010}{1.63-0}{january 2020}   
  
 \item \texttt{tess}:
    If a list of tiles was given, and the tiles were pixel images or masks,
    their pixel resolutions were ignored, and reset to the default 
    $128 \times 128$.
   {\small (Bug fixed in \texttt{spatstat 1.56-0}, june 2018)}   
   
 \item \texttt{nnorient}:
   crashed if the point pattern was empty.
   \bugger{1.40-0}{december 2015}{1.57-0}{october 2018}   

   \item \verb!as.im.data.frame!:
     Results were incorrect for factor-valued data.
    \bugger{1.45-2}{may 2016}{1.63-0}{january 2020}

  \item \texttt{predict.ppm}:
   Argument \texttt{new.coef} was ignored in calculating the standard error
   when \texttt{se=TRUE}.
   \bugin{spatstat 1.29-0}{october 2012}{spatstat.model 3.0-0}{june 2022}

 \item \texttt{predict.ppm}:
   Argument \texttt{new.coef} was ignored in calculating the standard error
   (and therefore the width of the interval) when \texttt{type="count"} and
   (\texttt{interval="confidence"} or \texttt{interval="prediction"}).
   \bugin{spatstat 1.29-0}{october 2012}{spatstat.model 3.0-0}{june 2022}

   \item \texttt{unnormdensity}:
     If \texttt{weights} was a single numerical value 
     \texttt{w}, the calculation incorrectly
    assigned the weight for each observation to be \texttt{w/n} where 
    \texttt{n=length(x)}.
    \bugalways{spatstat.geom 3.0-7}{march 2023}
    
    \item \texttt{SpatialMedian.ppp}, \texttt{Spatialquantile.ppp}:
      Argument \texttt{sigma} was ignored in some calculations.
      \bugin{spatstat.explore 3.2-6}{february 2024}{spatstat.explore 3.2-7}{march 2024}
      
  \item \texttt{weighted.quantile}:
    Results were incorrect when \texttt{type=2}.
    \bugbefore{spatstat 1.42-0}{may 2015}{spatstat.univar 2.0-4}{may 2024}
    
 \end{itemize}   

 %% LEVEL 4: 
 \subsection{Partially Incorrect or Having Only Minor Impact}
 \begin{itemize}
  \item \texttt{intersect.lintess}:
    Result was incorrect if \texttt{X}
    was a tessellation and \texttt{Y} was a linear network.
    \bugbefore{spatstat 1.46-0}{july 2016}{spatstat.linnet 3.3-3}{november 2025}    
 \item \texttt{incircle}:
   Gave incorrect results for some irregular windows.
   \bugin{spatstat 1.14-4}{october 2008}{spatstat.explore 3.6-0}{november 2025}   
  \item \texttt{kernel.squint}:
    The return value was incorrect if the argument \texttt{bw} was given.
    \bugin{spatstat 1.43-0}{october 2015}{spatstat.explore 3.2-4}{october 2023}
 
  \item \texttt{pcf}
    The variance approximation was calculated incorrectly (due to a bug 
    in \texttt{kernel.squint}).
    \bugin{spatstat 1.43-0}{october 2015}{spatstat.explore 3.2-4}{october 2023}
 
 \item \texttt{rjitter.ppp}: If \texttt{retry=FALSE}, marks were ignored.
   \bugbefore{spatstat 1.65-0}{december 2020}{spatstat.geom 3.0-5}{january 2023}
   
  \item \texttt{rhohat.lpp}:
    The argument \texttt{subset} was not handled correctly in the internal data.
    The estimated function \texttt{rho} was correct, but if
    \texttt{predict.rhohat} was applied,
    predictions were computed only in the \texttt{subset}, 
    and were possibly incorrect values.
    \bugbefore{spatstat 1.65-0}{december 2020}{spatstat.linnet 3.0-0}{june 2022}

  \item \texttt{rhohat.ppp}:
    The argument \texttt{subset} was not handled correctly in the internal data.
    The estimated function \texttt{rho} was correct, but if
    \texttt{predict.rhohat} was applied,
    predictions were computed only in the \texttt{subset}, 
    and were possibly incorrect values.
    \bugbefore{spatstat 1.65-0}{december 2020}{spatstat.explore 3.0-0}{june 2022}

 \item \texttt{density.lpp}:
   The result had the wrong length if \texttt{x} contained duplicated points
    when \texttt{weights} were given and \texttt{at="points"}.
    [Spotted by Andrea Gilardi]
    \bugger{1.55-1}{april 2018}{2.2-0}{june 2021}
    
  \item \texttt{crossdist.pp3}:
    Results with \texttt{periodic=TRUE} were partially incorrect.
    \bugger{1.34-1}{dec 2013}{1.65-0}{dec 2020}
    
 \item \texttt{mppm}:
   Internal data were malformed if the interaction was \texttt{Hardcore}
    or \texttt{MultiHard} or a hybrid involving these interactions.
    This caused various errors when the fitted model was used.
    \bugger{1.61-0}{september 2019}{1.64-0}{april 2020}.

 \item \texttt{mppm}:
   Ignored the arguments \texttt{nd} and \texttt{eps} controlling the 
   quadrature scheme. 
    \bugger{1.35-0}{december 2013}{1.64-0}{april 2020}.
    
  \item \texttt{edge.Ripley}:
    Results were incorrect for data points lying exactly at the corners
    of a rectangle.
    \bugbefore{spatstat 1.21-0}{november 2010}{spatstat.core 2.3-2}{november 2021}   

 \item \texttt{kppm}, \texttt{AIC}:
    For kppm models fitted with \verb!method='clik2'!, 
    the resulting value of \texttt{logLik()}
    was equal to $1/2$ of the correct value. This would have affected
    model comparison using AIC, and model selection using \texttt{step}.
    \bugger{1.42-0}{may 2015}{1.63-0}{january 2020}.
    
 \item \texttt{edge.Ripley}, \texttt{Kest}, \texttt{Kinhom}:
    Isotropic correction weights for polygonal windows were sometimes incorrect
    for small radius \texttt{r} if the polygon contained many small segments
    or if the polygon was very long and thin.
    \bugger{1.60-0}{june 2019}{1.62-0}{december 2019}.
    
 \item \texttt{edge.Ripley}, \texttt{Kest}, \texttt{Kinhom}:    
    Isotropic edge correction weight was computed incorrectly
    for a data point lying exactly on a corner of a rectangular window.
    \bugin{spatstat 1.60-0}{june 2019}{spatstat.explore 3.0-4}{november 2022}
 
 \item \texttt{beachcolours}, \texttt{beachcolourmap}:
   The number of colours was not always equal to \texttt{ncolours}.
   \bugger{1.32-0}{august 2013}{1.59-0}{march 2019}

 \item \texttt{extractbranch.lpp}:
   Point pattern coordinates were sometimes erroneously set to \texttt{NA}.
   \bugger{1.42-0}{may 2015}{1.59-0}{march 2019}
   
 \item \texttt{rotmean}:
       When \texttt{result="im"}
       the resulting image did not have the same dimensions
       as the input.
   \bugger{1.42-2}{june 2015}{1.58-0}{january 2019}   

 \item \texttt{quadratcount.ppp}:
   Sometimes issued an incorrect warning that data points were 
   outside the tessellation, when \texttt{tess} was a tessellation
   represented by a pixel image.
   {\small (Bug fixed in \texttt{spatstat 1.59-0}, march 2019)}   
   
 \item \texttt{quadrat.test}:
   the $p$-value was \texttt{NA} if one of the observed counts was zero,
   for the Cressie-Read tests with \texttt{CR} not equal to $1$ or $-1$.
   \bugger{1.38-0}{august 2014}{1.59-0}{march 2019}
   
 \item \texttt{quadrat.test}:
   argument \texttt{CR} was ignored if \texttt{method="MonteCarlo"}.
   \bugger{1.38-0}{august 2014}{1.61-0}{september 2019}
   
 \item \texttt{rotmean}:
    If argument \texttt{origin} was given, and if \texttt{result="im"}
    was specified,
    the resulting image was wrongly displaced.
   \bugger{1.42-2}{june 2015}{1.58-0}{january 2019}   

   
 \item \texttt{runifpointx}:
    Result was mangled when \texttt{n=0} or \texttt{n=1}.
   \bugger{1.50-0}{march 2017}{1.58-0}{january 2019}   
   
 \item \texttt{model.matrix.ppm}:
   The attribute \texttt{assign} was omitted in some cases.
   \bugger{1.45-1}{may 2016}{1.55-0}{january 2018}   

 \item \texttt{model.matrix.mppm}:
   Sometimes returned a matrix with the wrong number of rows.
   \bugger{1.55-0}{january 2018}{1.63-0}{january 2020}   

\item \texttt{density.ppp}:
  If the smoothing bandwidth \texttt{sigma} was very small 
  (e.g.\ less than the width of a pixel), 
  results were inaccurate if the default resolution was used,
  and completely incorrect if a user-specified resolution was given.
  \bugger{1.26-0}{april 2012}{1.52-0}{august 2017}
  
 \item \texttt{selfcrossing.psp}:
   $y$ coordinate values were incorrect.   
   \bugger{1.23-2}{august 2011}{1.25-3}{february 2012}
     
 \item \texttt{Geyer}:
   For point process models with the \texttt{Geyer} interaction, 
   \texttt{vcov.ppm} and \texttt{suffstat} sometimes gave incorrect answers.
   \bugger{1.27-0}{may 2012}{1.30-0}{december 2012}
   
 \item \texttt{leverage.ppm}, \texttt{influence.ppm}, \texttt{dfbetas.ppm}:
    Calculations were incorrect for a Geyer model fitted using
    an edge correction other than \texttt{"border"} or \texttt{"none"}.
   \bugger{1.25-0}{december 2011}{1.51-0}{may 2017}

 \item \texttt{leverage.ppm}, \texttt{influence.ppm}, \texttt{dfbetas.ppm}:
   Results were slightly incorrect for models fitted using the border
   correction.
   \bugger{1.25-0}{december 2011}{1.54-0}{november 2017}

 \item \texttt{leverage.ppm}:
   The mean leverage value (shown as a contour level in \texttt{plot.leverage.ppm})
   was slightly incorrect for Gibbs models.
   \bugger{1.25-0}{december 2011}{1.54-0}{november 2017}

 \item \texttt{vcov.ppm}, \texttt{suffstat}:
   These functions sometimes gave incorrect values 
   for marked point process models.   
   \bugger{1.27-0}{may 2012}{1.29-0}{october 2012}
   
 \item \texttt{diagnose.ppm}:
   When applied to a model obtained from \texttt{subfits()}, 
   in the default case (\texttt{oldstyle=FALSE}) 
   the variance calculations were incorrect.
   Consequently the dotted lines representing significance bands were 
   incorrect. An error or warning about negative variances occurred sometimes.
   However, calculations with \texttt{oldstyle=TRUE} were correct.
   The default has now been changed to \texttt{oldstyle=TRUE} for such models.
   \bugger{1.35-0}{december 2013}{1.45-0}{march 2016}

 \item \texttt{Smooth.ppp}:
   Results for \verb!at="points"! were garbled, for some values of 
   \texttt{sigma}, if \texttt{X} had more than one column of marks.
   \bugger{1.38-0}{october 2014}{1.46-0}{july 2016}
    
 \item \texttt{linearK}, \texttt{linearKinhom}:
   If any data points were located exactly at a vertex of the 
   linear network, the weights for Ang's correction were incorrect, 
   due to numerical error. This sometimes produced infinite 
   or NA values of the linear $K$ function.  
   \bugger{1.23-0}{july 2011}{1.27-0}{may 2012}

 \item \texttt{Kinhom}, \texttt{Linhom}:
   the results were not renormalised (even if \texttt{renormalise=TRUE})
   in some cases.
   \bugger{1.21-0}{december 2010}{1.37-0}{may 2014}
     
 \item \texttt{Kinhom}, \texttt{Linhom}:
   Ignored argument \texttt{reciplambda2} in some cases.
   \bugger{1.39-0}{october 2014}{1.40-0}{december 2014}

 \item \texttt{Kinhom}, \texttt{Linhom}:
   Calculations were incorrect if \texttt{lambda} was a fitted point
   process model.
   \bugger{1.38-0}{august 2014}{1.38-1}{august 2014}
     
 \item \texttt{integral.linim}, \texttt{integral.linfun}:
   \begin{itemize}
   \item 
   results were inaccurate because of a bias in the distribution of
   sample points.
   \bugger{1.41-0}{february 2015}{1.47-0}{october 2016}
   \item 
   results were inaccurate if many of the segment lengths were
   shorter than the width of a pixel.
   \bugger{1.41-0}{february 2015}{1.48-0}{december 2016}
    \item 
   results were wildly inaccurate in some extreme cases where many segments
   were very short.
   \bugger{1.41-0}{february 2015}{1.54-0}{november 2017}
   \end{itemize}
   
 \item \texttt{predict.ppm}:
   Calculation of the conditional intensity omitted the edge correction
   if \texttt{correction='translate'} or \texttt{correction='periodic'}. 
   \bugger{1.17-0}{october 2009}{1.31-3}{may 2013}

 \item \texttt{varblock}:
   Calculations were incorrect if more than one column of 
   edge corrections was computed. 
   \bugger{1.21-1}{november 2010}{1.39-0}{october 2014}
   
 \item \texttt{scan.test}
   Results were sometimes incorrect due to numerical instability
   (a 'Gibbs phenomenon').    
   \bugger{1.24-1}{october 2011}{1.26-1}{april 2012}

 \item \texttt{relrisk}:
   When \verb!at="pixels"!, a small fraction of pixel values were sometimes
   wildly inaccurate, due to numerical errors. This affected the 
   range of values in the result, and therefore the appearance of plots.
   {\small (Bug fixed in \texttt{spatstat 1.40-0}, december 2014)}

 \item \texttt{predict.slrm}:
   Results of \texttt{predict(object, newdata)} were incorrect 
   if the spatial domain of \texttt{newdata}
   was larger than the original domain.
   \bugger{1.21-0}{november 2010}{1.25-3}{february 2012}
   
 \item \texttt{Lest}:
   The variance approximations (Lotwick-Silverman and Ripley)
   obtained with \texttt{var.approx=TRUE} were incorrect for \texttt{Lest}
   (although they were correct for \texttt{Kest}) due to a coding error.
   \bugger{1.24-1}{october 2011}{1.24-2}{november 2011}
 
 \item \texttt{bw.diggle}:
   Bandwidth was too large by a factor of 2.
   \bugger{1.23-4}{september 2011}{1.23-5}{september 2011}
 
 \item pair correlation functions (\texttt{pcf.ppp}, \texttt{pcfdot}, 
    \texttt{pcfcross} etc:)
    The result had a negative bias at the maximum $r$ value,
    because contributions to the pcf estimate from interpoint distances
    greater than \texttt{max(r)} were mistakenly omitted. 
    {\small (Bugs fixed in \texttt{spatstat 1.35-0}, december 2013)}
    
 \item \texttt{Kest}, \texttt{Lest}:
   Gave incorrect values in very large datasets, due to numerical overflow.
   `Very large' typically means about 1 million points in a random pattern, 
   or 100,000 points in a tightly clustered pattern.
   [Overflow cannot occur unless there are at least 46,341 points.]
   
 \item \texttt{bw.relrisk}:
    Implementation of \texttt{method="weightedleastsquares"} was incorrect
    and was equivalent to \texttt{method="leastsquares"}.
    \bugger{1.21-0}{november 2010}{1.23-4}{september 2011}
    
\item \texttt{triangulate.owin}:
   Results were incorrect in some special cases.
   \bugger{1.42-2}{june 2015}{1.44-0}{december 2015}
  
\item \texttt{crosspairs}:
   If \texttt{X} and \texttt{Y} were identical point patterns,
   the result was not necessarily symmetric
   (on some machines) due to numerical artifacts.
   \bugger{1.35-0}{december 2013}{1.44-0}{december 2015}

 \item \texttt{bdist.tiles}:
   Values were incorrect in some cases due to numerical error.
    {\small (Bug fixed in \texttt{spatstat 1.29-0}, october 2012)}
   
\item \texttt{Kest.fft}:
  Result was incorrectly normalised.
   \bugger{1.21-2}{january 2011}{1.44-0}{december 2015}
  
\item \texttt{crossdist.ppp}:
  Ignored argument \texttt{squared} if \texttt{periodic=FALSE}.
    {\small (Bug fixed in \texttt{spatstat 1.38-0}, july 2014)}

\item polygon geometry:
    The point-in-polygon test gave the wrong answer in some boundary cases.
    {\small (Bug fixed in \texttt{spatstat 1.23-2}, august 2011)}

\item \texttt{MultiStraussHard}:
    If a fitted model with \texttt{MultiStraussHard} interaction was invalid,
    \texttt{project.ppm} sometimes yielded a model that was still invalid.
    {\small (Bug fixed in \texttt{spatstat 1.42-0}, may 2015)}
    
\item \texttt{pool.envelope}:  
  Did not always respect the value of \texttt{use.theory}.
    \bugger{1.23-5}{september 2011}{1.43-0}{september 2015}

\item \texttt{nncross.lpp}, \texttt{nnwhich.lpp}, \texttt{distfun.lpp}:
  Sometimes caused a segmentation fault.
    \bugger{1.44-0}{december 2015}{1.44-1}{december 2015}

\item \texttt{anova.ppm}:
  If a single \texttt{object} was given, and it was a Gibbs model,
  then \texttt{adjust} was effectively set to \texttt{FALSE}.
  \bugger{1.39-0}{october 2014}{1.44-1}{december 2015}

  \item \verb![.linim!: the result sometimes had the wrong class.
  \bugger{1.53-0}{september 2017}{1.55-1}{april 2015}

  \item \verb![.linim!: factor values were erroneously converted to 
    integers, in some cases.
  \bugger{1.53-0}{september 2017}{1.61-0}{september 2019}

\item \verb!is.subset.owin!: sometimes gave the wrong result for polygonal
  windows due to numerical rounding error.
   {\small (Bug was always present.
     Fixed in \texttt{spatstat 1.59-0}, march 2019)}

 \item \texttt{plot.tess}:  the legend showed the tile names
   in lexicographical order, rather than their original order.
   \bugger{1.55-1}{april 2018}{1.59-0}{march 2019}

  \item \texttt{rThomas}, \texttt{rMatClust},
    \texttt{rCauchy}, \texttt{rVarGamma}:
    If the simulation window was not a rectangle,
    the attribute \texttt{Lambda} was a numeric vector, 
    rather than a pixel image as intended.
    \bugger{1.43-0}{october 2015}{1.59-0}{march 2019}

  \item \texttt{effectfun}:
    In a multitype point process model, \texttt{effectfun} ignored any
    user-specified value of \texttt{marks}.
    \bugger{1.52-0}{august 2017}{1.61-0}{september 2019}

  \item \verb!"[<-.hyperframe"!:
    Some classes of objects were not handled correctly.
    \bugger{1.37-0}{may 2014}{1.61-0}{september 2019}

  \item \texttt{relrisk.ppp}:
    Crashed if there were more than 2 types of points
    and \texttt{method = "leastsquares"} or 
    \texttt{method = "weightedleastsquares"}.
    \bugger{1.23-4}{september 2011}{1.63-0}{january 2020}
    
  \item \texttt{nncross.ppp}:
    Format of output was incorrect if \texttt{X} was an empty pattern.
    \bugger{1.56-0}{june 2018}{1.63-0}{january 2020}
  
  \item \texttt{rmh}, \texttt{rmh.default}:
    For a marked point process, the debugger did not display the marks.
    (The \texttt{rmh} debugger is invoked by calling \texttt{rmh}
    with \texttt{snoop=TRUE}).
    \bugger{1.31-1}{march 2013}{1.63-0}{january 2020}
    
  \item \texttt{model.matrix.mppm}:
    If the model was fitted using \texttt{gam}, the resulting matrix
    did not have an \texttt{"assign"} attribute.
    \bugger{1.55-0}{march 2018}{2.2-0}{june 2021}
    
  \item \texttt{update.slrm}:
    Failed to find covariates that were provided in \texttt{env}.
    \bugger{1.33-0}{september 2013}{2.2-0}{june 2021}

   \item \texttt{distmap.owin}:
     Values were incorrect if X was an empty window 
     (\texttt{is.empty(X) = TRUE}).
     \bugalways{spatstat.geom 2.4-0}{march 2022}
    
   \item \texttt{distmap.ppp}, \texttt{distmap.psp}:
    Values were incorrect if X was an empty pattern (\texttt{npoints(X) = 0}).
     \bugalways{spatstat.geom 2.4-0}{march 2022}
    
   \item \texttt{distmap.psp}:
    Values were incorrect if X was an empty pattern (\texttt{nsegments(X) = 0}).
     \bugalways{spatstat.geom 2.4-0}{march 2022}
    
   \item \texttt{envelope} methods:
    Results were malformed if the name of the function argument 
    was not \texttt{r}. 
    \bugalways{spatstat.explore 3.2-3}{september 2023}

    \item \texttt{gorillas} dataset:
      The mark variables \texttt{group} and \texttt{season} were
      character vectors, when they were intended to be factors
      (categorical variables) as stated in the help file.
      \bugalways{spatstat.data 3.0-4}{january 2024}
      
   \item \texttt{marginSumsSparse}:
     If the result of \texttt{marginSumsSparse} 
     was a one-dimensional sparse vector,
     the entries were incorrectly rearranged so that the non-zero entries
     were all at the beginning of the vector.
     \bugalways{spatstat.sparse 3.0-2}{october 2023}
     
   \item \texttt{scanLRTS}:
     Pixel resolution arguments \texttt{dimyx}, \texttt{eps}, \texttt{xy}
     were not correctly handled.
     \bugalways{spatstat.explore 3.2-6}{february 2024}
     
   \item \texttt{Jest}:
     Pixel resolution argument \texttt{eps} was ignored.
     \bugalways{spatstat.explore 3.2-6}{february 2024}

   \item \texttt{rLGCP}:
     In certain special cases, an error message about incompatible images
     was issued, and in the resulting point pattern object \texttt{X},
     the driving intensity image \texttt{attr(X,"Lambda")}
     had incorrect dimensions or spatial coordinates.
     \bugalways{spatstat.random 3.2-3}{march 2024}
     
   \item \texttt{primefactors}:
     The default method (\verb!method = "C"!) did not handle numbers
    greater than the largest integer (\verb!n > .Machine$integer.max!).
     \bugalways{spatstat.utils 3.0-4}{february 2024}

   \item \texttt{predict.ppm}:
     Argument \texttt{eps} was ignored in many cases.
     \bugalways{spatstat.model 3.2-12}{may 2024}

   \item \texttt{lppm}:
    For models involving covariates of class \texttt{lintess}, the internal
    structure of the fitted model was corrupted, leading to errors in
    calculating properties of the fitted model, such as \texttt{predict.lppm}.
    \bugin{spatstat.linnet 3.1-2}{october 2023}{spatstat.linnet 3.2-5}{january 2025}
    
  \item \verb!"[.lpp"!: if the subset index \texttt{i} or \texttt{j} was a
    window, network segments which crossed the boundary of this window
    were not correctly handled in some cases.
    \bugin{spatstat.linnet 3.0-2}{december 2022}{spatstat.linnet 3.2-6}{may 2025}
 
\end{itemize}

\begin{thebibliography}{1}
\bibitem{baddrubaturn15}
A. Baddeley, E. Rubak, and R. Turner.
\newblock {\em Spatial Point Patterns: Methodology and Applications with {{R}}}.
\newblock Chapman \& Hall/CRC Press, 2015.
\end{thebibliography}
 
\end{document}

\documentclass[10pt]{article}
\usepackage{graphicx}
\usepackage{Sweave}
\usepackage{bm}
\usepackage[bottom=0.5cm, right=1.5cm, left=1.5cm, top=1.5cm]{geometry} 

% \VignetteIndexEntry{Guide to Function Objects in Spatstat}

% $Revision: 1.5 $ $Date: 2024/07/25 01:50:47 $

\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\link}[1]{#1}
\newcommand{\R}{{\sf R}}
\newcommand{\spst}{\pkg{spatstat}}
\newcommand{\Spst}{\pkg{Spatstat}}

\newcommand{\fv}{\texttt{"fv"}}
\newcommand{\env}{\texttt{"envelope"}}
\newcommand{\rat}{\texttt{"rat"}}
\newcommand{\obj}[1]{object of class {#1}}
\newcommand{\objs}[1]{objects of class {#1}}
\newcommand{\objsfvenv}{\objs\fv{} and \env}

\newcommand{\fun}[1]{\texttt{#1}}
\newcommand{\class}[1]{\texttt{"{#1}"}}

\newcommand{\Kfun}{$K$-function}
\newcommand{\Lfun}{$L$-function}
\newcommand{\pois}[1]{{#1}_{\mbox{\scriptsize pois}}}
\newcommand{\isoest}[1]{\widehat{#1}_{\mbox{\scriptsize iso}}}

\newcommand{\figref}[1]{Figure~\ref{#1}}
\newcommand{\secref}[1]{Section~\ref{#1}}
\newcommand{\eqref}[1]{(\ref{#1})}

\begin{document}
\bibliographystyle{plain}

<<echo=FALSE,results=hide>>=
library(spatstat)
x <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
              fields = c("Version", "Date"))
sversion <- as.character(x[,"Version"])
sdate    <- as.character(x[,"Date"])
options(useFancyQuotes=FALSE)
setmargins <- function(...) {
  options(SweaveHooks=list(fig=function() par(mar=c(...)+0.1)))
}
@ 
<<echo=FALSE,results=hide,fig=FALSE>>=
options(SweaveHooks=list(fig=function() par(mar=c(5,4,2,4)+0.1)))
options(width=100)
@ 
\SweaveOpts{eps=TRUE}
\setkeys{Gin}{width=0.5\textwidth}

\title{A guide to function objects (class \fv\ and \env) in \spst}
\author{Adrian Baddeley, Rolf Turner and Ege Rubak}
\date{For \spst\ version \texttt{\Sexpr{sversion}}}
\maketitle

\thispagestyle{empty}

\begin{abstract}
  This vignette explains how to use and manipulate 
  function objects (\objs{}\fv) and envelope objects (\objs{}\env)
  in the \spst\ package.
\end{abstract}


\setcounter{tocdepth}{1}
\tableofcontents

\newpage
\section{Introduction}

\subsection{Functional summary statistics}

An \obj\fv\ (`function value table') is a
convenient way of storing several different
estimates of the same function. 

It is common practice to summarise a spatial point pattern dataset
using a summary function, such as Ripley's \Kfun\ $K(r)$, rather than a 
single numerical summary value. 
Typically, an empirical estimate of the function, obtained from the data,
will be compared with the `theoretical' version of the function
that would be expected if the point pattern was completely random.
There may be several different empirical estimates of the function,
based on different estimation techniques, and we also want to compare
these estimates with one another.

The \spst{} family of packages makes it very easy to
compute and handle multiple versions of a summary function.
Taking the Finnish Pines data \texttt{finpines} as an example,
we can compute and plot estimates of Ripley's \Kfun\ by typing

<<>>=
K <- Kest(finpines)
@ 

<<K,fig=TRUE>>=
plot(K)
@ 

The plot shows several curves, which
represent the different empirical estimates of the \Kfun\
(namely the 
isotropic correction $\widehat K_{\mbox{\scriptsize iso}}(r)$,
translation correction $\widehat K_{\mbox{\scriptsize trans}}(r)$,
and border correction $\widehat K_{\mbox{\scriptsize bord}}(r)$)
and also the theoretical value $K_{\mbox{\scriptsize pois}}(r)$
that would be expected if the point pattern was completely random. 
All these functions are plotted against the distance argument $r$.

The object \texttt{K} 
belongs to class \fv{} (``function value table'').
It is a data frame
(that is, it also belongs to the class \class{data.frame})
with attributes giving extra information such as the 
recommended way of plotting the function.
One column of the data frame contains evenly spaced values of
the distance argument $r$, while the other columns contain 
estimates of the value of the function, or the theoretical value
of the function under CSR, corresponding to these distance values.

More information is given by the print method \texttt{print.fv}, 
which can be invoked just by typing the name of the object:
<<>>=
  K
@ 
The output indicates that the columns in the
data frame are named \texttt{r}, \texttt{theo}, \texttt{border},
\texttt{trans}, and \texttt{iso}, and explains their contents. 
For example, 
the column \texttt{iso} contains estimates of the \Kfun{} using the 
isotropic edge correction.
This column is labelled in the plot by the \R\ expression
\texttt{hat(K)[iso](r)} which is rendered 
as the mathematical notation $\widehat K_{\mbox{\scriptsize iso}}(r)$.

The function argument in an \class{fv} object is usually,
but not always, called \texttt{r}. (Counterexamples include
\fun{transect.im} which returns an \fv\ object with 
function argument \texttt{t}, and \fun{roc} which returns an \fv\ object 
with function argument \texttt{p}.)

The command \texttt{plot(K)} is dispatched to the method
\texttt{plot.fv} to generate the graphic shown above. The plot method
uses the auxiliary information contained in \texttt{K} to 
attach meaningful labels to the graphic. 

Stripping off the auxiliary information we can inspect the
data frame itself:
<<>>=
head(as.data.frame(K))
@ 


This vignette explains how to plot, manipulate and create objects of 
class \fv.

\subsection{Simulation envelopes}

Simulation envelopes of summary functions
are often used to assess statistical significance
in early stages of analysis. The \spst{} command \texttt{envelope}
generates simulation envelopes of a summary function:
<<results=hide>>=
E <- envelope(finpines, Kest, nsim=39)
@ 
<<E,fig=TRUE>>=
plot(E)
@ 

In this example, the command \verb!E <- envelope(finpines, Kest, nsim=39)!
generates 39 simulated point patterns according to a completely random process,
computes the estimated \Kfun{} for each simulated pattern, 
and finds the simulation envelopes
by identifying the pointwise minimum and maximum of the 39 simulated functions.
The result \texttt{E} is again an \obj\fv, but additionally belongs to the 
class \env, and contains additional information about how the
envelopes were computed. 

In the resulting plot, generated by the method \texttt{plot.envelope},
the region between the upper and lower
simulation envelopes is filled in grey shading. The solid black line
is the estimated \Kfun{} for the original \texttt{finpines} dataset,
and the dashed red line is the theoretical \Kfun{} for a completely
random pattern.

There is a lot of auxiliary information, displayed by \texttt{print.envelope}:
<<>>=
E
@ 

This vignette also explains how to plot, manipulate and create \objs\env.
Since envelope objects also belong to class \fv, the vignette first focuses
on the capabilities of class \fv.

\subsection{Why bother?}
\label{S:whybother}

Any self-respecting programmer would regard it as a 
trivial task to organise data in a data frame
and plot each column of data as a curve in a graph.
Although the task is trivial, it can be time-consuming,
it is prone to error, and it can take many attempts to get it 
exactly right. The authors of \spst\ developed the class \fv\ 
to make this job easier.

The class \fv{} is designed to
\begin{itemize}
\item 
  support \emph{multiple versions of a function},
  such as the different estimates of the \Kfun{} obtained using
  different edge corrections, the theoretical version of the \Kfun{}
  for a completely random process, the upper and lower simulation envelopes
  of the \Kfun, and so on. 
\item 
  do the \emph{``book-keeping''} about the different versions
  of the function, such as the names of the different columns.
\item 
  perform automatic \emph{plotting} of the function, handling all the details
  of layout and labelling, including generating the mathematical labels
  for each curve.
\item 
  support \emph{calculations} that will be applied automatically
  to all the versions of the function.
\item 
  support \emph{conversion} to other data types in base \R,
  such as data frames and functions.
\end{itemize}

For example, Besag's $L$ function is defined as 
$L(r) = \sqrt{K(r)/\pi}$. Since we have already computed the 
\Kfun{} in the example above, we can compute and plot the $L$-function
just by typing

<<>>=
L <- sqrt(K/pi)
@ 
<<L,fig=TRUE>>=
plot(L)
@ 

Several kinds of magic have happened here:
\begin{itemize}
\item The expression \texttt{sqrt(K/pi)}, 
  where \texttt{K} is an \obj\fv, has been evaluated
  automatically by calculating $\sqrt{K(r)/\pi}$ for each of the versions of the
  function stored in \texttt{K};
\item The internal data in the object \texttt{K}, which 
  provide mathematical labels for each version of the \Kfun,
  have been modified according to the algebraic operation 
  that was just performed;
\item The result has been saved as a new \obj\fv{} named \texttt{L};
\item The \texttt{plot} method has correctly displayed each version
  of the modified function using the modified mathematical labels, both
  on the vertical axis and in the legend box;
\item The \texttt{plot} method has \textbf{automatically computed the position
    of the legend box} to prevent it from overlapping the plotted curves;
\item The unit of length for the function argument has been
  correctly saved in the object \texttt{L} and 
  correctly reported on the horizontal axis label.
\end{itemize}

The class \env{} extends the class \fv{} to handle additional information
about how the envelopes were computed. The code supporting the class \env{}
performs many of the ``trivial'' but error-prone calculations involving
envelopes. An \obj\env{} can also contain the simulated data (the point patterns
and/or the summary functions) that were used to compute the envelopes, 
which makes it possible to re-use the simulated data to compute a different
version of the envelope. 

\newpage
\section{Plotting}
\label{S:plot.fv}

\subsection{Default plot}

If \texttt{f} is an object of class \class{fv},
the command \texttt{plot(f)} is dispatched to the method \fun{plot.fv}.
The default behaviour of \texttt{plot(f)} is to generate a plot
containing several curves, each representing a different
version of the same target function,
plotted against the distance argument $r$. 

<<eval=FALSE>>=
plot(Gest(finpines))
@ 
<<Gplot,fig=TRUE,echo=FALSE,results=hide>>=
aa <- plot(Gest(finpines))
@ 

Here \texttt{Gest} computes estimates of the nearest-neighbour
distance distribution function $G(r)$. The plot shows three
empirical estimates of $G(r)$ for the \texttt{finpines}
dataset, together with the `theoretical' 
curve $\pois G(r)$ expected for a completely random pattern,
all plotted against the distance argument $r$. 
The legend indicates the meaning of each curve. 
The main title identifies the object in \R\ that was plotted. 

The return value from \fun{plot.fv} is a data frame containing
more detailed information about the meaning of the curves. 
For the plot generated above, the return value is
<<eval=FALSE>>=
aa <- plot(Gest(finpines))
aa
@ 
<<echo=FALSE>>=
aa
@ 
Here \texttt{lty} and \texttt{col} are the graphics parameters
controlling the line type and line colour, and \texttt{label} is 
the mathematical notation for each edge-corrected estimate,
in the syntax recognised by \R{} graphics functions. 

The plot generated by \texttt{plot.fv} uses the base \R\ graphics system
(not \texttt{lattice} or \texttt{ggplot}). 
and is affected by graphics parameters
specified by \texttt{par()}.

\subsection{Modifying parameters of the default plot}

The default plot can easily be modified:
\begin{description}
\item[margin space:]
  To change the amount of white space around the plot,
  use \texttt{par('mar')}.
\item[main title:]
  use \texttt{main=""} to suppress the main title.
\item[legend:] 
  Set \texttt{legend=FALSE} to suppress the legend.
  Use the argument \texttt{legendargs} to modify the legend.
  The legend position is 
  automatically computed to avoid overlap with the plotted curves,
  but this can be overridden by \texttt{legendpos}. 
\item[range of values:] 
  Use \texttt{xlim} and \texttt{ylim} to specify the ranges
  of values on the $x$ and $y$ axes.
  \textbf{See the note below about the ``recommended range''.}
  Use \texttt{ylim.covers} to specify a numerical value or values that
  must be covered by the $y$ axis. For example, \texttt{ylim.covers=0}
  means that the $y$ axis will always include the origin.
\end{description}
For further information, see \texttt{help(plot.fv)}.

\subsection{Recommended range and recommended columns}

The default plot of an \fv\ object does not necessarily display
all the data that is contained in the object:
\begin{description}
\item[shorter range of distances:]
  the range of values of the distance argument $r$ 
  displayed in the default plot may be shorter than the 
  range of values actually contained in the data frame. 
\item[not all columns of data:] 
  the plot may not display all the columns of data contained in the data frame.
\end{description}

This happens because an \obj\fv\ contains ``recommendations'' 
about the range of distances that should be displayed,
and about the columns of data that should be shown.
These recommendations are based on standard statistical practice.
The recommendations are followed when the default plot is generated,
unless they are specifically overridden.

Consider this example:
<<>>=
G <- Gest(finpines)
G
@ 

The printout shows the range of values of \texttt{r} that are present 
in the table as the `\texttt{available range}'. It also gives a 
`\texttt{recommended range}' which is generally shorter than the
available range. \emph{The default plot of the object will only show the 
function values over the recommended range} and not over the full range of
values available. This is done so that the interesting detail
is clearly visible in the default plot. Values outside the recommended range 
may be unreliable due to increased variance or bias, depending on the
edge correction. To prevent this behaviour and use the full range
of function values available, set \texttt{clip.xlim=FALSE} in the 
plot command. Alternatively, specify the desired range of \texttt{r} values
using the argument \texttt{xlim} in the plot command.

The printout also says that the default plot formula is \verb! . ~ r !
where ``\verb!.!'' stands for \texttt{"km", "rs", "han", "theo"}.
This means that the default plot will display only the columns 
named \texttt{"km", "rs", "han"} and \texttt{"theo"} and will \textbf{not}
display the columns named \texttt{"hazard"} and \texttt{"theohaz"} which
are mentioned in the printout. This is consistent with the graphic shown
above. 

In this example, the column named \texttt{"hazard"} is an estimate
of the \emph{hazard rate} $h(r) = G'(r)/(1-G(r))$ of the nearest neighbour
distance function, rather than an estimate of $G(r)$ itself. 
The column named \texttt{"theohaz"} is the corresponding 
theoretical value of the hazard rate, expected 
if the point pattern is completely random.
It makes sense that the hazard rate $h(r)$ and distribution function $G(r)$
should not normally be plotted together.
Therefore when \texttt{Gest} is executed, it designates
\texttt{"km", "rs", "han", "theo"} as the ``recommended columns''
that should be displayed by default, and it stores this information
in the resulting object \texttt{G}. When \texttt{plot(G)} 
is executed, \texttt{plot.fv} uses this information to determine which
columns are to be plotted.

\subsection{Plot specified by a formula}
\label{S:plot.formula}

Different kinds of plots can be specified using a \texttt{formula}
as the second argument to \texttt{plot.fv}.
The left side of the formula represents what variables will
be plotted on the vertical ($y$) axis, and the right side determines the 
variable on the horizontal ($x$) axis.
For example, in the object
\texttt{K <- Kest(finpines)}, the column named \texttt{iso}
contains the values of the isotropic correction estimate. 
To plot the isotropic correction estimate against $r$, simply do
<<Kiso,fig=TRUE>>=
plot(K,  iso ~ r)
@ 

In \fun{plot.fv}, both sides of the plot formula are interpreted
as mathematical expressions, so that operators like
`\verb!+!', `\verb!-!', `\verb!*!', `\verb!/!'
have their usual meaning in arithmetic. 
The right-hand side of the formula can be any expression
that, when evaluated, yields a numeric vector, and the left-hand side is any
expression that evaluates to a vector or matrix of compatible
dimensions. 

If the left-hand side of the formula, when evaluated, yields a matrix, 
then each column of that
matrix is plotted against the specified $x$ variable as a separate curve.  
In particular the left-hand side of the formula may invoke the
function \fun{cbind} to indicate that several different curves
should be plotted. For example, to plot only the 
isotropic correction estimator and the theoretical curve:
<<Kit,fig=TRUE>>=
plot(K, cbind(iso, theo) ~ r)
@ 

Notice that, in this example, \texttt{plot.fv} is clever enough to recognise
that \texttt{iso} and \texttt{theo} are both versions of the \Kfun\ $K(r)$, 
and to decide that the appropriate label for the vertical axis is just $K(r)$.

The plot formula may also involve the names of constants like \texttt{pi},
standard functions like \texttt{sqrt}, and some special abbreviations listed
in Table~\ref{tab:fvnames}. 

\begin{table}[!h]
  \begin{tabular}{ll}
    \verb!.x! & argument of function \\
    \verb!.y! & best estimate of function \\
    \verb!.! & all recommended estimates of function \\
    \verb!.a! & all columns of function values \\
    \verb!.s! & upper and lower limits of shading
  \end{tabular}
    \caption{
      Recognised abbreviations for columns of an \class{fv} object.
    }
    \label{tab:fvnames}
\end{table}

The symbol \verb!.x! represents the function argument, usually \texttt{"r"}. 
The symbol \verb!.y! represents one of the columns of function values
which has been designated as the `best' estimate, for use by some other
commands in \spst.  
The symbol `\verb!.!' represents the `recommended' estimates.
The default plotting formula is \verb!. ~ .x!  indicating that
each of the recommended estimates will be plotted against
the function argument. The formula \verb!.y ~ .x! means that the
best estimate of the function will be plotted against the function argument.

To expand these abbreviations for a particular \fv\ object, 
use the function \texttt{fvnames}.
<<>>=
fvnames(K, ".y")
fvnames(K, ".")
@ 

A plot formula can be used to specify a transformation
that should be applied to the function values before they are displayed.
For example, to subtract the theoretical Poisson value
from each of the function estimates:
<<Ksubtheo,fig=TRUE>>=
plot(K, . - theo  ~ r)
@ 

Alternatively one could plot the function estimates \emph{against} the
Poisson value:
<<Ktheo,fig=TRUE>>=
plot(K, . ~ theo)
@ 

This plot has some theoretical support. 
In the discussion of Ripley's paper, Cox \cite{cox77discuss}
proposed that $\widehat K(r)$ should be plotted against $r^2$,
which is almost equivalent. We can follow Cox's recommendation
exactly:
<<Kcox,fig=TRUE>>=
plot(K, . ~ r^2)
@ 

The mathematical labels for the plot axes, and for the individual curves,
are constructed automatically by \spst\ from the plot formula. 
If the plot formula involves the names of external variables,
these will be rendered in Greek where possible. For example, to 
plot the average number of trees surrounding a typical tree in the
Swedish Pines data, 
<<Kswed,fig=TRUE>>=
lambda <- intensity(swedishpines)
plot(K, lambda * . ~ r)
@ 

Here we use the name \texttt{lambda} 
so that it will be rendered as the Greek letter $\lambda$ in
the graphics: the $y$-axis will be labelled $\lambda K(r)$. 

\section{Calculating with an \fv\ object}

This section explains how to do calculations involving a single \obj\fv.
The next section covers calculations involving several \objs\fv.

\subsection{Arithmetic and mathematical operators}

Arithmetic and mathematical operations on an \obj\fv\ 
can be performed by simply writing the arithmetic expression
involving the name of the object. The following are valid:
<<eval=FALSE>>=
K <- Kest(cells)
K/pi
sqrt(K/pi)
@ 

These inline calculations are performed by the operators 
\texttt{Ops.fv} and \texttt{Math.fv}.
The operation is applied to each column of \emph{function values};
the function argument \texttt{r} will not be affected.
The result is another \obj\fv\ with the same number of columns,
with the same column names, but with appropriately adjusted 
auxiliary information.

The expression can involve a command which returns an \obj\fv:
<<eval=FALSE>>=
sqrt(Kest(cells)/pi)
@ 
The auxiliary information contained in the resulting object
will be slightly less elegant in this case.

These arithmetic and mathematical operations are applied only to the 
\emph{recommended} columns of function values
identified by \texttt{fvnames(, ".")}. 

\subsection{Other vectorised operations}

Functions such as \texttt{pmax} and \texttt{cumsum} apply to vector data,
but are not recognised as arithmetic or mathematical operators 
by the \R\ parser,
so they are not covered by \texttt{Ops.fv} and \texttt{Math.fv}. 

For expressions involving \texttt{pmax} and \texttt{cumsum} (or indeed
any algebraic expression whatsoever), 
use the command \texttt{eval.fv} to perform the
calculation simultaneously for each column of function values:
<<eval=FALSE>>=
Kpos <- eval.fv(pmax(0, K))
@ 
The result \texttt{Kpos} is another \obj\fv\
in which the function values are all non-negative.

The first argument of \texttt{eval.fv} should be an expression
involving the \textbf{name} of the \obj\fv.

By default, the calculation is only applied to the 
\emph{recommended} columns of function values
identified by \texttt{fvnames(, ".")}.
This may be overridden by setting \texttt{dotonly=FALSE} in the call
to \texttt{eval.fv}.

The computations of \texttt{Ops.fv} and \texttt{Math.fv} are implemented
using \texttt{eval.fv} but there may be slight 
differences in the handling of the
auxiliary information. 

\subsection{Calculations involving specific columns}
\label{p:with.fv}

To manipulate or combine one or more columns of data in an \class{fv} object,
it is typically easiest to use \fun{with.fv}, 
a method for the generic \fun{with}. 
This behaves in a very similar way to \texttt{with.data.frame}.
For example:
<<>>= 
Kr <- Kest(redwood)
z <- with(Kr, iso - theo)
x <- with(Kr, r)
@ 
The results \texttt{x} and \texttt{z} are numeric
vectors, where \texttt{x} contains the values of the distance
argument $r$, and \texttt{z} contains the difference between
the columns \texttt{iso} (isotropic correction estimate) and
\texttt{theo} (theoretical value for CSR) for the \Kfun{}
estimate of the redwood seedlings data.  For this to work, we
have to know that \texttt{Kr} contains columns named \texttt{r},
\texttt{iso} and \texttt{theo}.  Printing
the object will reveal this information, as would typing
\texttt{names(Kr)} or \texttt{colnames(Kr)}. 

The general syntax is \texttt{with(X, expr)} where \texttt{X}
is an \class{fv} object and \texttt{expr} can be any expression
involving the names of columns of \texttt{X}. The expression
can include functions, so long as they are capable of operating
on numeric vectors.  The expression can also involve the abbreviations
listed in Table~\ref{tab:fvnames}:
<<>>=
Kcen <- with(Kr, . - theo)
@ 
subtracts the
`theoretical' value from all the available edge correction
estimates.  The result \texttt{Kcen} is another \class{fv} object.   

You can also get a result which is a vector or single number:
<<>>=
with(Kr, max(abs(iso-theo)))
@ 

\subsection{Extracting data}

An \obj\fv\ is essentially a data frame with additional attributes.
It contains the values of the desired function (such as $K(r)$) 
at a finely spaced grid of values of the function argument $r$.

The data frame can be extracted (and the additional attributes
removed) using \texttt{as.data.frame.fv}:
<<>>=
df <- as.data.frame(K)
@ 

A single column of values can be extracted using the \verb!$! 
operator in the usual way: \verb!K$iso! %$
would extract a vector containing the isotropic correction estimates of $K(r)$. 

The subset extraction operator `\verb![!' has a method  %]
for \class{fv} objects. This always returns another 
\class{fv} object, so it will refuse to remove the column 
containing values of the function argument \texttt{r}, for example.
To override this refusal, 
convert the object to a data frame using \fun{as.data.frame} 
and then use `\verb![!': % ]
the result will be a data frame or a vector.

Commands designed for data frames often work for \class{fv} objects as well. 
The functions \texttt{head} and \texttt{tail} extract the top (first few rows)
and bottom (last few rows) of a data frame. They also work on \class{fv}
objects: the result is a new \class{fv} object containing the 
function values for a short interval of $r$ values 
at the beginning or end of the range. The function \texttt{subset} 
selects designated subsets of a data frame using an elegant syntax
and this also works on \class{fv} objects. To restrict \texttt{K}
to the range $r \le 0.1$ and remove the border correction,
<<>>=
Ko <- subset(K, r < 0.1, select= -border)
@ 

\subsection{Converting to a true function}

An \obj\fv\ is meant to represent a function, but it contains only sample values
of the function at a grid of values of the function argument. 
The table of function values can also be converted to a true function
in the \R{} language using \fun{as.function}. This makes it easy
to evaluate the function at any desired distance $r$.
<<>>=
Ks <- Kest(swedishpines)
kfun <- as.function(Ks)
kfun(9)
@ 
By default, the result \texttt{kfun} is a function in \R, 
with a single argument 
\texttt{r} (or whatever the original function argument was called).
The new function accepts numeric values or numeric vectors of distance values, 
and returns the values of the `best' estimate of the function, 
interpolated linearly between entries in the table. 

If one of the other function estimates is required, 
use the argument \texttt{value} to \fun{as.function} to select it. 
<<>>=
kt <- as.function(Ks, value="trans")
kt(9)
@ 
To retain the option to select any one of the function estimates, type
<<>>=
kf <- as.function(Ks, value=".")
kf(9, "trans")
@ 

\subsection{Special operations}
\label{S:manip.fv}

An \class{fv} object can be manipulated using the operations
listed in Table~\ref{tab:fvmethods}. 

\begin{table}[!h]
\begin{tabular}[c]{ll}
  \texttt{f} & print a description \\
  \texttt{print(f)} & print a description \\
  \texttt{plot(f)} & plot the function estimates \\
  \texttt{as.data.frame(f)} & strip extra information (returns a data frame) \\
  \verb!f$iso! & extract column named \texttt{iso} (returns a numeric vector) \\
  \verb!f[i,j]! & extract subset (returns an \class{fv} object) \\
  \verb!subset(f, ...)! & extract subset (returns an \class{fv} object) \\
  \texttt{with(f, expr)} & perform calculations with columns of data frame\\
  \texttt{eval.fv(expr)} & perform calculations with several \class{fv} objects \\
  \texttt{bind.fv(f, d)} & combine an \class{fv} object \texttt{f} and data frame \texttt{d} \\
  \texttt{min(f)}, \texttt{max(f)}, \texttt{range(f)} & 
                range of function values \\
  \texttt{Smooth(f)} & apply smoothing to function values \\
  \texttt{deriv(f)} & derivative of function\\
  \texttt{stieltjes(g,f)} & compute Stieltjes integral with respect to \texttt{f} \\
  \texttt{as.function(f)} & convert to a function
\end{tabular}
\caption{Operations for manipulating an \class{fv} object \code{f}.}
\label{tab:fvmethods}
\end{table}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Calculating with several \fv\ objects}

\subsection{Arithmetic and mathematical operators}

Arithmetic and mathematical operations involving several \objs\fv\ 
can be performed by simply writing the arithmetic expression
involving the objects:
<<eval=FALSE>>=
Kcel <- Kest(cells)
Kred <- Kest(redwood)
Kdif <- Kcel - Kred
@ 
These inline calculations are performed by the operators 
\texttt{Ops.fv} and \texttt{Math.fv}.

The operation is applied to each column of \emph{function values};
the function argument \texttt{r} will not be affected.
The result is another \obj\fv\ with the same number of columns,
with the same column names, but with appropriately adjusted 
auxiliary information.

The \fv\ objects should be `compatible' 
in the sense that they have the same column names, 
and the same vector of $r$ values. However, \texttt{eval.fv} will
attempt to reconcile incompatible objects. 
(The \spst\ generic function \fun{compatible} determines whether two or more
objects are compatible, and the generic function \fun{harmonise}
makes them compatible, if possible.)

The expression can involve sub-expressions which return \objs\fv:
<<eval=FALSE>>=
Kest(cells) - Kest(redwood)
@ 
The auxiliary information contained in the resulting object
will be slightly less elegant in this case.

\subsection{Other vectorised operations}

For expressions involving \texttt{pmax} and \texttt{cumsum} (or indeed
any algebraic expression whatsoever), 
use the command \texttt{eval.fv} to perform the
calculation simultaneously for each column of function values.
<<eval=FALSE>>=
Kcel <- Kest(cells)
Kred <- Kest(redwood)
Kmax <- eval.fv(pmax(Kcel, Kred))
@ 
The result \texttt{Kmax} is another \obj\fv.

The first argument of \texttt{eval.fv} should be an expression
involving the \textbf{names} of the \objs\fv.

By default, the calculation is only applied to the 
\emph{recommended} columns of function values
identified by \texttt{fvnames(x, ".")} where \texttt{x} is the \obj\fv.
This may be overridden by setting \texttt{dotonly=FALSE} in the call
to \texttt{eval.fv}.

The expression is not permitted to contain sub-expressions
that evaluate to \objs\fv. However, you can use the argument \texttt{envir} to 
supply such sub-expressions:
<<eval=FALSE>>=
  Kmax <- eval.fm(pmax(Kcel, Kred),
                  envir=list(Kcel=Kest(cells), Kred=Kest(redwood)))
@ 

The computations of \texttt{Ops.fv} and \texttt{Math.fv} are implemented
using \texttt{eval.fv} but there may be slight 
differences in the handling of the auxiliary information. 

\subsection{Combining objects}

Several \class{fv} objects can be combined using the operations
listed in Table~\ref{tab:fvmethods.multi}. 

\begin{table}[!h]
\begin{tabular}[c]{ll}
  \texttt{eval.fv(expr)} & perform calculations with several \class{fv} objects \\
  \verb!cbind(f1, f2, ...)! & combine \class{fv} objects \texttt{f1, f2, ...} \\
  \texttt{bind.fv(f, d)} & combine an \class{fv} object \texttt{f} and data frame \texttt{d} \\
  \verb!collapse.fv(f1, f2, ...)! & 
      combine several redundant \class{fv} objects  \\
  \verb!compatible(f1, f2, ...)! & 
  check whether \class{fv} objects are compatible  \\
  \verb!harmonise(f1, f2, ...)! & 
  make \class{fv} objects compatible  
\end{tabular}
\caption{Operations for manipulating 
  several \class{fv} objects \code{f1}, \code{f2}.}
\label{tab:fvmethods.multi}
\end{table}

Use \code{\link{cbind.fv}} to combine several \code{"fv"} objects.
Use \code{\link{bind.fv}} to glue additional columns onto an existing
\code{"fv"} object.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Creating \fv\ objects from raw data}

This section explains how to create \objs\fv\ from raw numerical data.
This would be useful if you are implementing a completely new kind of
summary function. 

Subsection~\ref{S:creator} explains how to create an \obj\fv\ 
by providing the numerical data and the required auxiliary information. 
Section~\ref{S:as.fv} describes an easier way to convert a data frame
(or similar object) to an \obj\fv\ without specifying the auxiliary
information, using default rules for the auxiliary information.
Section~\ref{S:compileK} describes special tools
\texttt{compileK, compilepcf, compileCDF} for creating an
\obj\fv\ from a numeric vector of distance values, using the rules that 
apply to the \Kfun, or the pair correlation function,
or the nearest-neighbour distance distribution function.

\subsection{The creator function \fun{fv}}
\label{S:creator}

\subsubsection{The creator function}

The low-level function \code{fv} is used to create an object of
class \code{"fv"} from raw numerical data. It has the following syntax:
\begin{verbatim}
fv(x, argu = "r", ylab = NULL, valu, fmla = NULL, 
   alim = NULL, labl = names(x), desc = NULL, 
   unitname = NULL, fname = NULL, yexp = ylab)
\end{verbatim}

The arguments are as follows:
\begin{itemize}
\item 
  \code{x} contains the numerical data. It should be a data frame,
  in which one column gives the values of the function argument for which
  the function has been evaluated, and at least one other column
  contains the corresponding values of the function.

  These other columns typically give the values of different versions
  or estimates of the same function,
  for example, different estimates of the \Kfun{}  
  obtained using different edge corrections.
  However they may also contain the values of related functions
  such as the derivative or hazard rate.
\item 
  \code{argu} specifies the name of the column of
  \code{x} that contains the values of the function argument
  (typically \code{argu="r"} but this is not compulsory).
\item 
  \code{valu} specifies the name of another column
  that contains the `recommended' estimate of the function.
  It will be used to provide function values in those situations where
  a single column of data is required. For example,
  \code{envelope} computes its simulation envelopes
  using the recommended value of the summary function.
\item 
  \code{fmla} specifies the default plotting behaviour.
  It should be a formula, or a string that can be converted to a
  formula. Variables in the formula are names of columns of \code{x}.
  See \code{plot.fv} for the interpretation of this
  formula.
\item 
  \code{alim} specifies the recommended range of the
  function argument. This is used in situations where statistical
  theory or statistical practice indicates that the computed
  estimates of the function are not trustworthy outside a certain
  range of values of the function argument. By default,
  \code{plot.fv} will restrict the plot to this range.
\item 
  \code{fname} is a character string (or a vector of 2 character strings)
  giving the name of the function itself.
  For example, the \Kfun{}  would have \code{fname="K"},
  while the inhomogeneous \Kfun\ has \code{fname=c("K", "inhom")}.
\item 
  \code{ylab} is a mathematical expression
  for the function value, used when labelling an axis
  of the plot, or when printing a description of the
  function. It should be an \R{} language object. 
  For example the \Kfun's mathematical name $K(r)$ is rendered
  by \code{ylab=quote(K(r))}. 
\item
  \code{yexp} is another mathematical expression for the function value.
  If \code{yexp} is present, then \code{ylab} will be
  used only for printing, and \code{yexp} will be used for
  annotating axes in a plot. 
  (Otherwise \code{yexp} defaults to \code{ylab}).
\item 
  \code{labl} is a character vector specifying plot labels
  for each column of \code{x}. These labels will appear on the
  plot axes (in non-default plots), legends and printed output.
  Entries in \code{labl}
  may contain the string \code{"\%s"} which will be replaced
  by \code{fname} when plotted or printed. 
  For example the border-corrected estimate
  of the \Kfun{}  has label \code{"\%s[bord](r)"} which
  becomes \code{"K[bord](r)"} when it is used in \texttt{plot.fv} 
  or \texttt{print.fv}.
\item 
  \code{desc} is a character vector containing intelligible
  explanations of each column of \code{x}. Entries in
  \code{desc} may contain the string \code{"\%s"} which will be replaced
  by \code{ylab}. For example the border correction estimate of the
  \Kfun{}  has description \code{"border correction estimate of \%s"}.
  This will be replaced by 
  \code{"border correction estimate of K(r)"} when it is used in 
  \texttt{print.fv}.
\item 
  \code{unitname} is the name of the unit of length 
  for the \underline{function argument}. Typically the function argument
  \code{"r"} represents distance between points. The distance values are 
  typically expressed in terms of a distance unit, such as metres or feet.
  This unit will be printed on the horizontal axis. 
  The argument \code{unitname} is an object of
  class \class{unitname}, or \code{NULL} representing dimensionless values.
\end{itemize}

\subsubsection{Syntax for \texttt{ylab} and \texttt{yexp}}

Mathematical symbols and notation are supported 
in \R\ base graphics. The labels on the axes of a graph,
in the body of the graph, and in graph legends, can all include
mathematical notation. The notation has to be encoded as an 
\R\ language expression. The decoding is slightly idiosyncratic,
and this affects the programming of the class \fv.

The arguments \code{ylab} and \code{yexp} are mathematical 
expressions for the function value: \texttt{ylab} is used when
printing a description of the function,
and \texttt{yexp} is used when labelling an axis. 
Usually \texttt{ylab} and \texttt{yexp} are the same.
For example the \Kfun's mathematical name $K(r)$ is rendered
by \code{ylab=quote(K(r))} and \code{yexp=ylab}.
An example where they are different is the multitype \Kfun\ $K_{1,2}(r)$ 
where we set  \code{ylab=quote(Kcross[1,2](r))}
and 
\code{yexp=quote(Kcross[list(1,2)](r))}
to get the most satisfactory behaviour.

A useful programming tip is to use \code{substitute} instead of
\code{quote} to insert values of variables into an expression,
e.g. \code{substitute(Kcross[i,j](r), list(i=42,j=97))}
yields the same as \code{quote(Kcross[42, 97](r))}.)

\subsubsection{Syntax for \texttt{labl}} 

The argument \texttt{labl} is a character vector 
specifying plot labels
for each column of \code{x}. These labels will appear on the
plot axes (in non-default plots), legends and printed output.

Entries in \code{labl}
may contain the string \code{"\%s"} which will be replaced
by \code{fname} when plotted or printed. 
For example the border-corrected estimate
of the \Kfun{}  has label \code{"\%s[bord](r)"} which
becomes \code{"K[bord](r)"} when it is used in \texttt{plot.fv} 
or \texttt{print.fv}. This mechanism allows the code to adjust the
labels when the object is changed --- for example, to produce the correct
labels in \code{plot(sqrt(K/pi))} as shown in Section~\ref{S:whybother}.

Things become more complicated if \texttt{fname} is a character vector
of length 2. In that case the appropriate expression for the border-corrected
estimate is \verb!"{hat(%s)[%s]^{bord}}(r)"! which becomes
\verb!{hat(K)[inhom]^{bord}}(r)! when it is used in \texttt{plot.fv}
or \texttt{print.fv}.

We strongly recommend using the function \fun{makefvlabel} to create the
appropriate labels. Its syntax is:
\begin{verbatim}
   makefvlabel(op=NULL, accent=NULL, fname, sub=NULL, argname="r")
\end{verbatim}
where the arguments are character strings:
\begin{description}
\item[op] is a prefix or operator such as \code{"var"} (rarely used);
\item[accent] is an accent that should be applied to the main function symbol, usually \texttt{"hat"} for empirical estimates;
\item[fname] is the name of the function (usually a single letter or
  a character vector of length 2);
\item[sub] is an optional subscript, typically used to discriminate 
  between different estimates of the function, such as different edge corrections;
\item[argname] is the name of the function argument.
\end{description}

Examples:
<<>>=
  makefvlabel(NULL, NULL, "K", "pois")
  makefvlabel(NULL, "hat", "K", "bord")
  makefvlabel(NULL, "hat", c("K", "inhom"), "bord")
  makefvlabel("var", "hat", c("K", "inhom"), "bord")
@ 

\subsubsection{Syntax for \texttt{desc}}

Each entry of \texttt{desc} is a single character string. It may 
contain a \underline{single} instance of \code{"\%s"}, which will be replaced by
the function name when required.


\subsection{Conversion function \fun{as.fv}}
\label{S:as.fv}

The generic function \texttt{as.fv} converts other kinds of data to an \obj\fv. 

The methods \fun{as.fv.matrix} and \fun{as.fv.data.frame} provide a lazy way
to convert a table of function data to an \obj\fv. The auxiliary information
is determined by applying default rules.

Other methods apply to classes of objects which intrinsically contain
an \obj\fv, and they simply extract the \fv\ object.
For example, a fitted model of class \class{kppm} contains the 
summary function (either the $K$ function or the pair correlation function)
that was used to fit the model; so the method \fun{as.fun.kppm} simply extracts
this summary function.

\subsection{compileK, compilepcf, compileCDF}
\label{S:compileK}

A shortcut is provided for programmers wishing to implement 
a summary function that is similar to 
Ripley's $K$ function, the pair correlation function $g$,
the empty space function $F$ or
the nearest-neighbour distance distribution function $G$.

\subsubsection{$K$ functions and pair correlation functions}

Programmers who wish to implement a summary function similar to 
Ripley's $K$ function or the pair correlation function
can use the commands \texttt{compileK} or \texttt{compilepcf}.

These low-level functions construct estimates of the $K$ function or
pair correlation function, or any similar functions, given only
the matrix of pairwise distances and optional weights associated
with these distances.

These functions are useful for code development and for teaching,
because they perform a common task, and do the housekeeping
required to make an object of class \fv\ that represents the
estimated function. However, they are not very efficient.

The basic syntax of \texttt{compileK} and \texttt{compilepcf} is:

<<eval=FALSE>>=
compileK(D, r, weights = NULL, denom = 1, ...)
compilepcf(D, r, weights = NULL, denom = 1, ...)
@ 

where

\begin{itemize}
\item \texttt{D} is a square matrix giving the distances between all pairs of
  points;
\item \texttt{r} is a vector of distance values, equally spaced, at which 
  the summary function should be calculated;
\item \texttt{weights} is an optional matrix of numerical weights
  for the pairwise distances;
\item \texttt{denom} is the denominator for the estimator.
  It may be a single number, or a numeric
  vector with the same length as \texttt{r}.
\end{itemize}

The command \texttt{compileK} calculates the weighted estimate of the
$K$ function,
\[
  K(r) = \frac{1}{v(r)} \sum_i \sum_{j \neq i} w_{i,j} \; 1\{ d_{i,j} \le r\}  
\]
and \texttt{compilepcf} calculates the weighted estimate of the pair
correlation function,
\[
  g(r) = \frac{1}{v(r)} \sum_i \sum_{j \neq i} w_{i,j}\; \kappa ( d_{i,j} - r) 
\]     
where $d_{i,j}$ is the distance between spatial points $i$ and $j$, with
corresponding weight $w_{i,j}$, and $v(r)$ is the specified denominator.
Here $\kappa$ is a fixed-bandwidth smoothing kernel.

For a point pattern in two dimensions, the usual denominator $v(r)$
is constant for the $K$ function, and proportional to $r$ for the pair
correlation function:

<<>>=
X <- japanesepines
D <- pairdist(X)
Wt <- edge.Ripley(X, D)
lambda <- intensity(X)
a <- (npoints(X)-1) * lambda
r <- seq(0, 0.25, by=0.01)
K <- compileK(D=D, r=r, weights=Wt, denom=a)
g <- compilepcf(D=D, r=r, weights=Wt, denom= a * 2 * pi * r)
@ 

The result of \texttt{compileK} or \texttt{compilepcf} can then
be edited (as explained in the next section)
to change the function name and other information as desired.

\subsubsection{Cumulative distribution functions}

Programmers wishing to implement a summary function 
which is a cumulative distribution function, similar to
the functions \texttt{Gest} or \texttt{Fest},
can use the command \texttt{compileCDF}.

The basic syntax of \texttt{compileCDF} is:
<<eval=FALSE>>=
compileCDF(D, B, r, ..., han.denom = NULL)
@ 
where
\begin{itemize}
\item \texttt{D} is a numeric vector of observed distances (such as the distance
  from each data point to its nearest neighbour);
\item \texttt{B} is a numeric vector of censoring distances 
  (such as the distance from each data point to the boundary of the window);
\item \texttt{r} is a vector of distance values, equally spaced, at which 
  the summary function should be calculated;
\item \texttt{han.denom} is the denominator for the Hanisch estimator.
  It is usually a numeric
  vector with the same length as \texttt{r}.
\end{itemize}

An example for the nearest-neighbour distance distribution function $G(r)$:
<<>>=
X <- japanesepines
D <- nndist(X)
B <- bdist.points(X)
r <- seq(0, 1, by=0.01)
h <- eroded.areas(Window(X), r)
G <- compileCDF(D=D, B=B, r=r, han.denom=h)
## give it a better name
G <- rebadge.fv(G, new.fname="G", new.ylab=quote(G(r)))
@ 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 



\section{Editing the auxiliary information in \fv\ objects}

The ``auxiliary information'' in an  \obj\fv\ consists of the 
function name, a mathematical expression for the function,
mathematical expressions for each version of the function contained in a 
column of data, the choice of which columns will be plotted by default,
and other information. 

A programmer will often wish to create an \fv\ object first,
perhaps using some existing code, and then edit the auxiliary information.

The safe way to edit the auxiliary information 
is to \textbf{use the internal functions in \spst} which support the \fv\ class:

\begin{itemize}
\item \texttt{rebadge.fv} is a low-level function which 
  allows the user to change any of the entries in the auxiliary information
  as desired.
\item \texttt{rebadge.as.crossfun} and \texttt{rebadge.as.dotfun}
  are wrappers for \texttt{rebadge.fv} which change the auxiliary information
  into the form expected for a cross-type or dot-type summary function.
\item \texttt{fvlabels} extracts the mathematical code for each
  version of the summary function, and \verb!fvlabels<-! changes the codes.
\item \texttt{makefvlabel} creates suitable mathematical code
  for a version of the summary function.
\item The functions \texttt{fvnames} and \verb!fvnames<-! manage the
definition of the abbreviations listed in Table~\ref{tab:fvnames}.
\item The methods \texttt{formula.fv} and \verb!formula<-.fv! manage the
default plotting formula.
\item \verb!names<-.fv! changes the names of the columns in the 
  \fv\ object, and adjusts the internal data accordingly.
\item \texttt{tweak.fv.entry} 
  is a very low-level function that changes the auxiliary information
  about one of the columns of data.
\item \texttt{prefixfv} is another wrapper for \texttt{rebadge.fv}
  that adds a prefix to the name of the function.
\end{itemize}

\subsection{Low-level editing}

\texttt{rebadge.fv} is a low-level function which 
allows the user to change any of the entries in the auxiliary information
as desired. It has syntax
<<eval=FALSE>>=
rebadge.fv(x, new.ylab, new.fname,
           tags, new.desc, new.labl,
           new.yexp=new.ylab, new.dotnames,
           new.preferred, new.formula, new.tags)
@ 
where \texttt{x} is the \obj\fv.

The arguments \texttt{new.fname}, \texttt{new.ylab} and 
\texttt{new.yexp} (if present) specify new values for the
function name \texttt{fname} and the mathematical expressions
for the function, \texttt{ylab} and \texttt{yexp}, described in
Section~\ref{S:creator}.

The argument \texttt{new.dotnames} specifies a new value for
the selection of columns that are plotted by default.
This is a character vector of column names of \texttt{x}
and is associated with the abbreviation ``\verb!.!'' in 
Table~\ref{tab:fvnames}.

The argument \texttt{new.preferred} specifies a new value for
the choice of column that is designated the ``preferred'' column
and is used in calculations which require a single column of data,
such as simulation envelopes. 
This is a single character string which must be a column 
name of \texttt{x}
and is associated with the abbreviation ``\verb!.y!'' in 
Table~\ref{tab:fvnames}.

The argument \texttt{new.formula} specifies a new default plotting formula
for the summary function. 

The argument \texttt{new.desc} specifies new values for the string
descriptions of the individual columns, replacing the argument \texttt{desc}
described in Section~\ref{S:creator}. It should be a character vector 
with one entry for every column of \texttt{x} (or see below).

The argument \texttt{new.labl} specifies new values for the mathematical labels
of the individual columns, replacing the argument \texttt{desc}
described in Section~\ref{S:creator}. It should be a character vector 
with one entry for every column of \texttt{x} (or see below).

The argument \texttt{tags} can be used to select some of the columns of data
so that only the auxiliary data for the selected columns will be changed.
It should be a character vector with entries which match the names
of columns of \texttt{x}. In that case, \texttt{new.desc} and \texttt{new.labl}
should have the same length as \texttt{tags}, 
and they will be taken as replacement values for the selected columns only.

The optional argument \texttt{new.tags} changes the names of the
columns of \texttt{x} (or the columns selected by \texttt{tags}) to new values.

\subsection{Changing information about one column}

The method \verb!names<-.fv! changes the names of the columns in the 
\fv\ object, and adjusts the internal data accordingly.

The function \texttt{tweak.fv.entry} 
is a very low-level function that changes the auxiliary information
about one of the columns of data. It has syntax
<<eval=FALSE>>=
tweak.fv.entry(x, current.tag, new.labl=NULL, new.desc=NULL, new.tag=NULL)
@ 
where \texttt{current.tag} is the current name of the column for which
the information should be changed, \texttt{new.labl} is the new
mathematical label for the column, \texttt{new.desc} is the 
new text description of the column, and \texttt{new.tag} is the new 
name of the column. All these arguments are single strings 
or \texttt{NULL}. 

\subsection{Special idioms}

A few functions are available for performing special idioms.

The function \texttt{prefixfv} is a wrapper for \texttt{rebadge.fv}
that adds a prefix to the name of the function, and to all the 
relevant auxiliary information. It has syntax
<<eval=FALSE>>=
prefixfv(x, tagprefix="", descprefix="", lablprefix=tagprefix,
         whichtags=fvnames(x, "*"))
@ 
where \texttt{tagprefix}, \texttt{descprefix} and \texttt{lablprefix}
are strings that should be added to the beginning
of the column name, the text description, and the mathematical expression
for each column of data. The argument \texttt{whichtags} specifies which
columns of data should be changed; the default is to change all columns.

The function \texttt{rebadge.as.crossfun} changes the auxiliary information
into the form expected for a bivariate, cross-type summary function,
analogous to the bivariate $K$-function $K_{i,j}(r)$ between two types of points
labelled $i$ and $j$ that is computed by the \spst\ function \texttt{Kcross}.
It has syntax
<<eval=FALSE>>=
rebadge.as.crossfun(x, main, sub=NULL, i, j)
@ 
where \texttt{main} is the main part of the function name,
\texttt{sub} is the subscript part of the function name,
and \texttt{i} and \texttt{j} are the type labels.
For example
<<eval=FALSE>>=
rebadge.as.crossfun(x, "L", i="A", j="B")
@ 
would create a function $L_{A,B}(r)$, and
<<eval=FALSE>>=
rebadge.as.crossfun(x, "L", "inhom", "A", "B")
@ 
would create a function $L_{\mbox{\scriptsize inhom},A,B}(r)$.

Similarly the function \texttt{rebadge.as.dotfun}
changes the auxiliary information
into the form expected for a ``one type-to-any type'' summary function,
analogous to the function $K_{i \bullet}(r)$ 
that is computed by the \spst\ function \texttt{Kdot}.
It has syntax
<<eval=FALSE>>=
rebadge.as.dotfun(x, main, sub=NULL, i)
@ 

\subsection{Handling mathematical labels}

The auxiliary information in an \fv\ object includes
mathematical labels for the different versions of the function,
which are displayed by \texttt{plot.fv}.

The function \texttt{fvlabels} extracts the mathematical code for each
version of the summary function from the \fv\ object, 
and \verb!fvlabels<-! changes the codes.

The mathematical codes are strings 
which must be recognisable to the \texttt{plotmath}
code in the \R\ base graphics system which is somewhat idiosyncratic. 
The strings may also (and usually do)
include the substring \verb!%s! (appearing once or twice)
which will be replaced by the 
function name. For example, if the function name is 
\texttt{"K"} and the label is \verb!"hat(%s)[iso](r)"! this will be
parsed as \verb!hat(K)[iso](r)! which is rendered 
as $\widehat K_{\mbox{\scriptsize iso}}(r)$.

The function \texttt{makefvlabel} creates suitable mathematical code
for a version of the summary function. Programmers are strongly advised
to use \texttt{makefvlabel}. 

\subsection{Changing default behaviour}

The default behaviour for plotting an \fv\ object depends
on its default \texttt{plot formula} and typically on its
\texttt{dot names}.

The default plot formula is printed when the object is printed,
and can be extracted using \texttt{formula.fv}.
<<>>=
K <- Kest(cells)
formula(K)
@ 
The interpretation of the plot formula is explained 
in Section~\ref{S:plot.formula}. In the example above, the left hand side
of the formula uses the abbreviation ``\verb!.!'' which stands
for ``the default list of columns to be plotted''. This abbreviation can
be expanded using \texttt{fvnames}:
<<>>=
fvnames(K, ".")
@ 
which indicates that the columns named
\texttt{"iso"},
\texttt{"trans"},
\texttt{"border"} and
\texttt{"theo"} will be plotted.

The choice of ``dot names'' can be changed using \verb!fvnames<-!:
<<>>=
fvnames(K, ".") <- c("iso", "theo")
@ 

In general the functions \texttt{fvnames} and \verb!fvnames<-! 
manage the
definition of all the abbreviations listed in Table~\ref{tab:fvnames}.


\section{Pooling several function estimates}

\subsection{Pooling}

``Pooling'' or combining several datasets into a single dataset
is a common statistical procedure. If we are only interested in a summary
statistic of the data, then in some special circumstances, the 
summary statistic of the pooled dataset can be calculated from the
summary statistics of the original, separate datasets. For example,
if we have a set of $n_1$ observations with sample mean $m_1$,
and another set of $n_2$ observations with sample mean $m_2$,
then the sample mean of the pooled set of $n_1+n_2$ observations has
sample mean $(n_1 m_1 + n_2 m_2)/(n_1+n_2)$, a weighted average of the
sample means of the original datasets. This procedure is loosely called
``pooling'' the sample mean.

If we have two point pattern datasets, observed in different windows,
we can ``pool'' the patterns by simply
treating them as a single point pattern observed in the combined window.
If we pool two point pattern datasets, the estimated $K$-function of the 
pooled pattern can be calculated from the estimated $K$-functions 
$K_1(r)$ and $K_2(r)$ of the original point patterns, if we know the number of
points in each of the two original patterns. 
That is, Ripley's $K$-function can be ``pooled''. 

The summary functions used in spatial statistics can be pooled, 
provided they are able to be expressed as a ratio
$f(r) = A(r)/B$ or $f(r) = A(r)/B(r)$ where $A(r)$ is the ``numerator''
and $B$ or $B(r)$ is the ``denominator''. The pooled estimate is the
ratio of the sum of numerators divided by the sum of denominators.
For details, see section 16.8.1 of \cite{baddrubaturn15}.

\subsection{Pooling summary functions}

The generic function \texttt{pool} performs pooling of summary statistics
(including summary functions like the $K$-function).
In order for this to work correctly, we must know the numerator and 
denominator for each of the individual summary statistics or summary functions. 
For this purpose there is a special class \class{rat} (for ``ratio object'').
An \obj\rat\ contains two attributes named
\texttt{"numerator"} and \texttt{"denominator"} which contain the 
numerator and denominator of the ratio. 

For many of the summary functions provided in \spst, 
if we set the argument \texttt{ratio=TRUE}, the numerator and 
denominator will be calculated separately and saved in the resulting
object, which will belong to the class \class{rat} (``ratio object'')
as well as \fv. 
<<>>=
class(Kest(cells))
class(Kest(cells, ratio=TRUE))
@ 
This capability is currently available for the functions 
\texttt{compileK},
\texttt{compilepcf},
\texttt{Finhom},
\texttt{Gcross.inhom},
\texttt{Gdot.inhom},
\texttt{Ginhom},
\texttt{GmultiInhom},
\texttt{Jcross.inhom},
\texttt{Jdot.inhom},
\texttt{Jinhom},
\texttt{Jmulti.inhom},
\texttt{K3est},
\texttt{Kcross},
\texttt{Kdot},
\texttt{Kest},
\texttt{Kinhom},
\texttt{Kmulti},
\texttt{Ksector},
\texttt{linearKinhom},
\texttt{linearK},
\texttt{linearpcfinhom},
\texttt{linearpcf},
\texttt{nnorient},
\texttt{pairorient},
\texttt{pcfcross},
\texttt{pcfdot},
\texttt{pcfmulti},
\texttt{pcf.ppp} and
\texttt{Tstat}.

The method \texttt{pool.rat} will pool several objects which all belong
to the classes \class{fv} and \class{rat}:

<<>>=
X1 <- runifpoint(50)
X2 <- runifpoint(50)
K1 <- Kest(X1, ratio=TRUE)
K2 <- Kest(X2, ratio=TRUE)
K <- pool(K1, K2)
@ 

<<>>=
Xlist <- runifpoint(50, nsim=6)
Klist <- lapply(Xlist, Kest, ratio=TRUE)
K <- do.call(pool, Klist)
@ 

There is also a fallback method \texttt{pool.fv} 
which is used when some of the objects do not contain ratio information.
This method effectively pretends that all the objects have the same
denominator. 

\subsection{Low level utilities}

Programmers wishing to implement a summary function with 
ratio information can use the following low-level utilities:

\begin{itemize}
\item \texttt{ratfv} is the creator function, analogous to \texttt{fv}.
  Its syntax is
<<eval=FALSE>>=
ratfv(df, numer, denom, ..., ratio=TRUE)
@ 
where \texttt{df} is a data frame, \texttt{numer} and \texttt{denom}
are \objs\fv, and additional arguments \verb!...! are passed to \texttt{fv}.
It is sufficient to specify either \texttt{df} or \texttt{numer},
in addition to \texttt{denom}.
\item \texttt{bind.ratfv} glues extra columns onto an existing 
  \obj\fv and \class{rat}.
\item \texttt{conform.ratfv} forces the auxiliary information
  in the numerator and denominator of an \obj\fv and \class{rat}
  to agree with the auxiliary information of the main object.
\end{itemize}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Structure of \objs\fv}

This section explains the information contained in \objs\fv.

\subsection{Advice}

We strongly discourage the user from unpacking the internal
contents of \objs\fv{} and manipulating the contents directly.
Instead, we recommend using the functions that are available in \spst{} 
for handling these objects. 

Although it is easy to extract the internal data contained in an object
in \R, the structure of \objs\fv\ is idiosyncratic, and the internal format
is variable. Looking at one example of an \obj\fv\ will not tell you how
it all works. This is because there are many cases to handle,
and many quirks in the formatting of algebraic expressions in \R.

Using the functions provided in \spst\ is also more efficient 
than extracting data yourself, because it avoids creating multiple 
copies of the data.

Most of all, \textbf{do not change the internal contents of \objs\fv}.
This can easily violate the internal format and cause errors.
Use the functions supplied for handling these objects.


\subsection{Objects of class \fv}

Objects of class \fv\ are returned by many commands in the 
\spst\ packages. Usually these objects are obtained by
analysing a spatial point pattern dataset.
There are also functions to create such objects from raw data.

An \obj\fv{} is essentially a data frame with
additional attributes containing auxiliary information.

\subsubsection*{Data frame structure}

The first column of the data frame contains values of the 
function argument. These values are arranged in increasing order,
are usually evenly-spaced, and usually start from zero. The first
column usually (but not always) has the column name \texttt{r}.

Subsequent columns of the data frame contain the values of
different versions of the summary function, corresponding to the
values of the function argument. These columns may have any column names.
These versions of the function may be referred to by their column names
when plotting and manipulating the object.

<<>>=
G <- Gest(finpines)
df <- as.data.frame(G)
head(df)
@ 

In this example, the object \texttt{G} contains estimates of
the nearest-neighbour distance distribution function $G(r)$
for the \texttt{finpines} dataset. For the distance
value $r = $ \Sexpr{round(df[6, "r"], 9)} metres, the estimate of $G(r)$ 
using the \texttt{han} method is \Sexpr{round(df[6, "han"],8)}. 

Columns of data can be extracted using the data frame structure.
To extract the sequence of \texttt{r} values, 
use \verb!df$r! or \verb!G$r! or \verb!df[, "r"]!. 
To extract the corresponding values of
\texttt{han}, use \verb!df$han! or \verb!G$han! or \verb!df[, "han"]!.

\subsubsection*{Auxiliary information}

In the example above,
to find out what the column \texttt{han} means, we need the
auxiliary information stored in the object \texttt{G}. 
This can be printed out directly in readable form:
<<>>=
G
@ 
Thus, \texttt{han} refers to the estimate of $G(r)$ using Hanisch's method.

The auxiliary information is stored in attributes
of the object. The full list of attributes is as follows:

\begin{tabular}{lll}
\texttt{argu} & character(1) & Name of function argument (usually \texttt{"r"}) \\
\texttt{valu} & character(1) & Name of preferred function value \\
\texttt{ylab} & language  & Mathematical expression for function (for vertical axis of plot) \\
\texttt{yexp} & language  & Mathematical expression for function (in algebra) \\
\texttt{fmla} & character(1) & Default plotting formula \\
\texttt{alim} & numeric(2) & Recommended range of function argument \\
\texttt{labl} & character($m$) & Mathematical labels for each column\\
\texttt{desc} & character($m$) & Text descriptions of each column\\
\texttt{units} & unitname & Unit of length (for function argument) \\
\texttt{fname} & character(1 or 2) & Symbol for function only \\
\texttt{dotnames} & character($k \le m$) & Column names of all recommended versions \\
\texttt{shade} & character(0 or 2) & Column names of limits of grey shading\\
\end{tabular}

\code{argu} is the name of the column of the data frame
that contains the values of the function argument
(typically \code{argu="r"} but this is not compulsory).

\code{valu} specifies the name of another column
that contains the `recommended' estimate of the function.
It will be used to provide function values in those situations where
a single column of data is required. For example,
\code{envelope} computes its simulation envelopes
using the recommended value of the summary function.

\code{fmla} specifies the default plotting behaviour,
as explained in Section~\ref{S:plot.fv}. It is a character string
that can be converted to a \texttt{formula} in the \R\ language.

\code{alim} specifies the recommended range of the
function argument. It is a numeric vector of length 2.
This is used in situations where statistical
theory or statistical practice indicates that the computed
estimates of the function are not trustworthy outside a certain
range of values of the function argument. By default,
\code{plot.fv} will restrict the plot to this range.

\code{fname} gives the name of the function itself.
For example, the \Kfun{}  would have \code{fname="K"}.
It is either a character string, or a vector of two character strings,
where the second element is interpreted as a subscript.
For example, the inhomogeneous \Kfun{} computed by \code{Kinhom}
has \code{fname=c("K", "inhom")}.
  
\code{ylab} is a mathematical expression
for the function value, used when printing a description of the
function. It is an \R{} language object. 
For example the \Kfun's mathematical name $K(r)$ is rendered
by \code{ylab=quote(K(r))}. 

\code{yexp} is another mathematical expression
for the function value, used for annotating axes in a plot. 

\code{labl} is a character vector specifying plot labels
for each column of the data frame. These labels will appear on the
plot axes (in non-default plots), legends and printed output.
Entries in \code{labl}
may contain the string \code{"\%s"} which will be replaced
by \code{fname}. 

\code{desc} is a character vector containing intelligible
explanations of each column of the data frame. Entries in
\code{desc} may contain the string \code{"\%s"} which will be replaced
by \code{ylab}. 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Structure of \objs\env}

This section explains the information contained in \objs\env.

\subsection{The \texttt{envelope} command}

The \spst\ function \fun{envelope} performs the
calculations required for envelopes. It computes the 
summary function for a point pattern dataset, generates
simulated point patterns, computes the summary functions for the
simulated patterns, and computes the envelopes of these summary 
functions. 

<<eval=FALSE>>=
E <- envelope(swp, Kest, nsim=39, fix.n=TRUE)
@ 
The result is an object of class \class{envelope} and \class{fv}
which can be printed and plotted and manipulated
using the tools for \class{fv} objects, and by additional tools
provided for \class{envelope} objects.

The print method gives a lot of detail:
<<>>=
E
@ 

\subsection{Re-using envelope data}

The method \texttt{envelope.envelope} allows new \fun{envelope} commands to
be applied to a previously computed \class{envelope} object,
provided it contains the necessary data.

In the original call to \fun{envelope}, if the argument
\texttt{savepatterns=TRUE} was given, the resulting \class{envelope}
object contains all the simulated point patterns.  Alternatively
if the argument \texttt{savefuns=TRUE} was given, the resulting
object contains the individual summary functions for each of the
simulated patterns.  This information is not saved, by default,
for efficiency's sake.

Envelopes created with \texttt{savepatterns=TRUE} allow any kind of
new envelopes to be computed using the same simulated point patterns:
<<eval=FALSE>>=
E1 <- envelope(redwood, Kest, savepatterns=TRUE)
E2 <- envelope(E1, Gest, global=TRUE, 
               transform=expression(fisher(.)))
@ 

Envelopes created with \texttt{savefuns=TRUE} allow 
the user to switch between pointwise and global envelopes
of the same summary function, to apply different transformations
of the summary function, and to change some parameters:
<<eval=FALSE>>=
A1 <- envelope(redwood, Kest, nsim=39, savefuns=TRUE)
A2 <- envelope(A1, global=TRUE, nsim=19, 
               transform=expression(sqrt(./pi)))
@ 

\subsection{Pooling several envelopes}

It is also possible to combine the simulation data
from several envelope objects 
and to compute envelopes based on the combined data.
This is done using \fun{pool.envelope}, a method for
the \spst\ generic \fun{pool}.
The envelopes must be compatible,
in that they are envelopes for the same function,
and were computed using the same options. The
individual summary functions must have been saved.
<<eval=FALSE>>=
E1 <- envelope(cells, Kest, nsim=10, savefuns=TRUE)
E2 <- envelope(cells, Kest, nsim=20, savefuns=TRUE)
E <- pool(E1, E2)
@ 

\subsection{Structure of envelope objects}

An \obj\env{} is an \obj\fv{} with additional auxiliary information:
\begin{itemize}
\item 
  the names of two of the columns of function values,
  designated as the upper and lower simulation envelopes of the function,
  saved in \texttt{attr(, "shade")} and retrievable as 
  \texttt{fvnames(, .s)}
\item 
  details of how the envelopes were computed,
  saved in \texttt{attr(, "einfo")}
\item 
  optionally, the simulated point patterns used to compute the envelopes,
  saved in \texttt{attr(, "simpatterns")}
\item 
  optionally, the simulated summary functions
  (the summary functions computed for the simulated point patterns)
  used to compute the envelopes,
  saved in \texttt{attr(, "simfuns")}
\end{itemize}

Objects of class \env\ inherit the class \fv, so they can be manipulated
using methods for class \fv, but there are extra methods for the
special class \env. 

\subsection{The \texttt{einfo} list}

Additional attribute \texttt{einfo} is a list of:

\begin{tabular}{lll}
  \texttt{call} & character(1) & original function call \\
  \texttt{Yname} & character(1) & name of original dataset \\
  \texttt{valname} & character(1) & column name of function values used\\
  \texttt{csr} & logical(1) & \texttt{TRUE} if simulations based on CSR \\
  \texttt{csr.theo} & logical (1) & see below\\
  \texttt{use.theory} & logical (1) & see below\\
  \texttt{pois} & logical(1) & \texttt{TRUE} if simulations 
                               are Poisson process\\
  \texttt{simtype} & character(1) & Type of simulation (see below) \\
  \texttt{constraints} & character(1) & Additional information (see below) \\
  \texttt{nrank} & integer(1) & Rank of envelopes \\
  \texttt{nsim} & integer(1) & Number of simulations for envelope \\
  \texttt{Nsim} & integer(1) & Total number of simulations\\
  \texttt{global} & logical(1) & \texttt{TRUE} if global envelopes\\
  \texttt{ginterval} & numeric(0 or 2) & Domain of function argument for global envelopes \\
  \texttt{dual} & logical(1) & \texttt{TRUE} if two sets of simulations performed\\
  \texttt{nsim2} & integer(1) & Number of simulations in second set \\
  \texttt{VARIANCE} & logical(1) & \texttt{TRUE} if limits are based on standard deviation \\
  \texttt{nSD} & numeric(1) & Number of standard deviations defining limits \\
  \texttt{alternative} & character(1) & \texttt{two.sided}, \texttt{less} or \texttt{greater} \\
  \texttt{scale} & \texttt{NULL} or function & Scaling function for function argument \\
  \texttt{clamp} & logical(1) & \texttt{TRUE} if one-sided deviations must be positive \\
  \texttt{use.weights} & logical(1) & \texttt{TRUE} if sample mean is weighted\\
  \texttt{do.pwrong} & logical(1) & \texttt{TRUE} if ``wrong $p$-value'' should be calculated \\
  \texttt{gaveup} &  logical(1) & \texttt{TRUE} if simulations terminated early
\end{tabular}


\begin{thebibliography}{1}
\bibitem{baddrubaturn15}
A. Baddeley, E. Rubak, and R. Turner.
\newblock {\em Spatial Point Patterns: Methodology and Applications with {{R}}}.
\newblock Chapman \& Hall/CRC Press, 2015.
\bibitem{besa77d}
J.E. Besag. 
\newblock Contribution to the discussion of the paper by Ripley (1977).
\newblock \emph{Journal of the Royal Statistical Society, Series B}
\textbf{39} (1977) 193--195.
\bibitem{cox77discuss}
D.R. Cox.
\newblock Contribution to the discussion of the paper by Ripley (1977).
\newblock \emph{Journal of the Royal Statistical Society, Series B}
\textbf{39} (1977) 206.
\end{thebibliography}

\end{document}

\name{add.texture}
\alias{add.texture}
\title{
  Fill Plot With Texture
}
\description{
  Draws a simple texture inside a region on the plot.
}
\usage{
add.texture(W, texture = 4, spacing = NULL, ...)
}
\arguments{
  \item{W}{
    Window (object of class \code{"owin"}) inside which the
    texture should be drawn.
  }
  \item{texture}{
    Integer from 1 to 8 identifying the type of texture. See Details.
  }
  \item{spacing}{
    Spacing between elements of the texture, in units of the
    current plot.
  }
  \item{\dots}{
    Further arguments controlling the plot colour, line width etc.
  }
}
\details{
  The chosen texture, confined to the window \code{W},
  will be added to the current plot.
  The available textures are:
  \describe{
    \item{texture=1:}{
      Small crosses arranged in a square grid.
    }
    \item{texture=2:}{
      Parallel vertical lines.
    }
    \item{texture=3:}{
      Parallel horizontal lines.
    }
    \item{texture=4:}{ 
      Parallel diagonal lines at 45 degrees from the horizontal.
   }
    \item{texture=5:}{
      Parallel diagonal lines at 135 degrees from the horizontal.
    }
    \item{texture=6:}{
      Grid of horizontal and vertical lines.
    }
    \item{texture=7:}{
      Grid of diagonal lines at 45 and 135 degrees from the horizontal.
    }
    \item{texture=8:}{
      Grid of hexagons.
    }
  }
}
\author{
  \adrian
  and
  \rolf
}
\seealso{
  \code{\link{owin}}, 
  \code{\link{plot.owin}},
  \code{\link{textureplot}},
  \code{\link{texturemap}}.
}
\examples{
  W <- Window(chorley)
  plot(W, main="")
  add.texture(W, 7)
}
\keyword{spatial}
\keyword{hplot}

\name{foo}
\alias{foo}
\alias{plot.foo}
\title{
  Foo is Not a Real Name
}
\description{
  The name \code{foo} is not a real name: it is a place holder,
  used to represent the name of any desired thing.

  The functions defined here simply print an explanation of the
  placeholder name \code{foo}.
}
\usage{
foo()

\method{plot}{foo}(x, \dots)
}
\arguments{
  \item{x}{Ignored.}
  \item{\dots}{Ignored.}
}
\details{
  The name \code{foo} is used by computer scientists as a
  \emph{place holder}, to represent the name of any desired object or
  function. It is not the name of an actual object or function;
  it serves only as an example, to explain a concept.

  However, many users misinterpret this convention, and actually
  type the command \code{foo} or \code{foo()}. Then they email the
  package author to inform them that \code{foo} is not defined.

  To avoid this correspondence, we have now defined an object
  called \code{foo}.
  
  The function \code{foo()} prints a message explaining that \code{foo}
  is not really the name of a variable. 

  The function can be executed simply by typing \code{foo}
  without parentheses.
}
\value{
  Null.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{beginner}}
}
\examples{
  foo
}
\keyword{documentation}

\name{beginner}
\alias{beginner}
\title{
  Print Introduction For Beginners
}
\description{
  Prints an introduction for beginners to the \code{spatstat} package,
  or another specified package.
}
\usage{
beginner(package = "spatstat")
}
\arguments{
  \item{package}{
    Name of package.
  }
}
\details{
  This function prints an introduction for beginners
  to the \pkg{spatstat} package.

  The function can be executed simply by typing \code{beginner}
  without parentheses.

  If the argument \code{package} is given, then the function prints the
  beginner's help file \code{BEGINNER.txt} from the specified package
  (if it has one).
}
\value{
  Null.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{latest.news}}
}
\examples{
  beginner
}
\keyword{documentation}

\name{spatstat-package}
\alias{spatstat-package} 
\alias{spatstat} 
\docType{package}
\title{The Spatstat Package}
\description{
  This is a summary of the features of 
  \pkg{spatstat}, a family of \R packages
  for the statistical analysis of spatial point patterns.
}
\details{
  \pkg{spatstat} is a family of \R packages
  for the statistical analysis
  of spatial data. Its main focus is the analysis of
  spatial patterns of points in two-dimensional space.

  \pkg{spatstat} is designed to support a complete statistical analysis
  of spatial data. It supports
  \itemize{
    \item creation, manipulation and plotting of point patterns;
    \item exploratory data analysis;
    \item spatial random sampling;
    \item simulation of point process models;
    \item parametric model-fitting;
    \item non-parametric smoothing and regression;
    \item formal inference (hypothesis tests, confidence intervals);
    \item model diagnostics.
  }
  
  Apart from two-dimensional point patterns and point processes,
  \pkg{spatstat} also supports point patterns in three dimensions, 
  point patterns in multidimensional space-time,
  point patterns on a linear network,
  patterns of line segments in two dimensions, 
  and spatial tessellations and random sets in two dimensions.

  The package can fit several types of point process models
  to a point pattern dataset:
  \itemize{
    \item Poisson point process models
    (by Berman-Turner approximate maximum likelihood
    or by spatial logistic regression)
    \item Gibbs/Markov point process models
    (by Baddeley-Turner approximate maximum pseudolikelihood,
    Coeurjolly-Rubak logistic likelihood, or
    Huang-Ogata approximate maximum likelihood)
    \item Cox/cluster point process models
    (by Waagepetersen's two-step fitting procedure
    and minimum contrast, composite likelihood, or Palm likelihood)
    \item determinantal point process models
    (by Waagepetersen's two-step fitting procedure
    and minimum contrast, composite likelihood, or Palm likelihood)
  }
  The models may include spatial trend,
  dependence on covariates, and complicated interpoint interactions.
  Models are specified by 
  a \code{formula} in the \R  language, and are fitted using
  a function analogous to \code{\link[stats]{lm}} and \code{\link[stats]{glm}}.
  Fitted models can be printed, plotted, predicted, simulated and so on.
}
\section{Getting Started}{
  For a quick introduction to \pkg{spatstat}, read
  the package vignette \emph{Getting started with spatstat}
  installed with \pkg{spatstat}. To read that document, you can either
  \itemize{
    \item visit \url{https://cran.r-project.org/package=spatstat}
    and click on \code{Getting Started with Spatstat}
    \item
    start \R, type \code{library(spatstat)} and \code{vignette('getstart')}
    \item
    start \R, type \code{help.start()} to open the help
    browser, and navigate to \code{Packages > spatstat > Vignettes}.
  }
  Once you have installed \pkg{spatstat}, start \R and type
  \code{library(spatstat)}. Then type \code{beginner}
  for a beginner's introduction, or 
  \code{demo(spatstat)} for a demonstration of the package's capabilities.

  For a complete course on \pkg{spatstat},
  and on statistical analysis of spatial point patterns,
  read the book by Baddeley, Rubak and Turner (2015).
  Other recommended books on spatial point process methods
  are Diggle (2014), Gelfand et al (2010) and Illian et al (2008).

  The \pkg{spatstat} package includes over 50 datasets,
  which can be useful when learning the package.
  Type \code{demo(data)} to see plots of all datasets
  available in the package.
  Type \code{vignette('datasets')} for detailed background information
  on these datasets, and plots of each dataset.
      
  For information on converting your data into \pkg{spatstat} format,
  read Chapter 3 of Baddeley, Rubak and Turner (2015).
  This chapter is available free online, as one of the sample chapters
  at the book companion website, \url{https://book.spatstat.org/}.

%  For information about handling data in \bold{shapefiles},
%  see Chapter 3, or the Vignette
%  \emph{Handling shapefiles in the spatstat package},
%  installed with \pkg{spatstat}, accessible as
%  \code{vignette('shapefiles')}.
}
\section{Structure of the spatstat family}{  
  The original \pkg{spatstat} package grew to be very large.
  It has now been divided into several \bold{sub-packages}:
  \itemize{
    \item \pkg{spatstat.utils} containing basic utilities
    \item \pkg{spatstat.sparse} containing linear algebra utilities
    \item \pkg{spatstat.data} containing datasets
    \item \pkg{spatstat.univar} containing functions for estimating
    probability distributions of random variables
    \item \pkg{spatstat.geom} containing functionality for geometrical
    operations, and defining the main classes of spatial objects
    \item \pkg{spatstat.random} containing procedures for random generation
    and simulation of spatial data
    \item \pkg{spatstat.explore} containing the main functions for exploratory
    analysis of spatial data
    \item \pkg{spatstat.model} containing the main functions for
    parametric statistical modelling and analysis,
    and formal inference, for spatial data
    \item \pkg{spatstat.linnet} containing functions for
    spatial data on a linear network
    \item \pkg{spatstat}, which simply loads the other sub-packages
    listed above, and provides documentation.
  }
  The breakup has been done in such a way that the user should not notice
  any difference. Source code that worked with the old \pkg{spatstat}
  package should work with the new \pkg{spatstat} family.
  Code that is documented in our books, journal articles and vignettes
  should still work.
  
  When you install \pkg{spatstat}, the sub-packages listed above are also
  installed. Then if you load the \pkg{spatstat} package by typing
  \code{library(spatstat)}, the other sub-packages listed above will
  automatically be loaded or imported. 

  This help file covers all the functionality and datasets
  that are provided in the sub-packages listed above.
}
\section{Extension packages}{
  Additionally there are several \bold{extension packages:}
  \itemize{
    \item \pkg{spatstat.gui} for interactive graphics
    \item \pkg{spatstat.local} for local likelihood
    (including geographically weighted regression)
    \item \pkg{spatstat.Knet} for additional, computationally efficient code
    for linear networks
    \item \pkg{spatstat.sphere} (under development) for spatial data
    on a sphere, including spatial data on the earth's surface
  }
  The extension packages must be installed separately
  and loaded explicitly if needed. They also have separate documentation.
}
\section{Updates}{
  New versions of \pkg{spatstat} are released every 8 weeks.
  Users are advised to update their installation of \pkg{spatstat}
  regularly.
  
  Type \code{latest.news} to read the news documentation about
  changes to the current installed version of \pkg{spatstat}.

  See the Vignette \emph{Summary of recent updates},
  installed with \pkg{spatstat}, which describes the main changes
  to \pkg{spatstat} since the book (Baddeley, Rubak and Turner, 2015)
  was published. It is accessible as \code{vignette('updates')}.
  
  Type \code{news(package="spatstat")} to read news documentation about
  all previous versions of the package.
}
\section{FUNCTIONS AND DATASETS}{
  Following is a summary of the main functions and datasets
  in the \pkg{spatstat} package.
  Alternatively an alphabetical list of all functions and
  datasets is available by typing \code{library(help=spatstat)}.

  For further information on any of these,
  type \code{help(name)} or \code{?name}
  where \code{name} is the name of the function
  or dataset.
}
\section{CONTENTS:}{
  \tabular{ll}{
    I. \tab Creating and manipulating data \cr
    II. \tab Exploratory Data Analysis \cr
    III. \tab Model fitting (Cox and cluster models) \cr
    IV. \tab Model fitting (Poisson and Gibbs models) \cr
    V. \tab Model fitting (determinantal point process models)\cr
    VI. \tab Model fitting (spatial logistic regression)\cr
    VII. \tab Model fitting (recursive partition models)\cr
    VIII. \tab Simulation \cr
    IX. \tab Tests and diagnostics\cr
    X. \tab Documentation
  }
}

\section{I. CREATING AND MANIPULATING DATA}{

  \bold{Types of spatial data:}

  The main types of spatial data supported by \pkg{spatstat} are:

  \tabular{ll}{
    \code{\link[spatstat.geom]{ppp}} \tab point pattern \cr
    \code{\link[spatstat.geom]{owin}} \tab window (spatial region) \cr
    \code{\link[spatstat.geom]{im}} \tab pixel image \cr
    \code{\link[spatstat.geom]{psp}} \tab line segment pattern \cr
    \code{\link[spatstat.geom]{tess}} \tab tessellation \cr
    \code{\link[spatstat.geom]{pp3}} \tab three-dimensional point pattern \cr
    \code{\link[spatstat.geom]{ppx}} \tab point pattern in any number of dimensions \cr
    \code{\link[spatstat.linnet]{lpp}} \tab point pattern on a linear network
  }

  \bold{To create a point pattern:}
  
  \tabular{ll}{
    \code{\link[spatstat.geom]{ppp}} \tab
    create a point pattern from \eqn{(x,y)} and window information
    \cr
    \tab
    \code{ppp(x, y, xlim, ylim)} for rectangular window\cr
    \tab
    \code{ppp(x, y, poly)} for polygonal window \cr
    \tab
    \code{ppp(x, y, mask)} for binary image window \cr
    \code{\link[spatstat.geom]{as.ppp}} \tab
    convert other types of data to a \code{ppp} object \cr
    \code{\link[spatstat.geom]{clickppp}} \tab
    interactively add points to a plot \cr
    \code{\link[spatstat.geom]{marks<-}}, \code{\%mark\%}  \tab
    attach/reassign marks to a point pattern
  }
      
  \bold{To simulate a random point pattern:}
  
  \tabular{ll}{
	\code{\link[spatstat.random]{runifpoint}} \tab
	generate \eqn{n} independent uniform random points \cr
	\code{\link[spatstat.random]{rpoint}} \tab
	generate \eqn{n} independent random points \cr
	\code{\link[spatstat.random]{rmpoint}} \tab
	generate \eqn{n} independent multitype random points \cr
	\code{\link[spatstat.random]{rpoispp}} \tab
	simulate the (in)homogeneous Poisson point process \cr
	\code{\link[spatstat.random]{rmpoispp}} \tab
	simulate the (in)homogeneous multitype Poisson point process \cr
	\code{\link[spatstat.random]{runifdisc}} \tab
	generate \eqn{n} independent uniform random points in disc\cr
	\code{\link[spatstat.random]{rstrat}} \tab
	stratified random sample of points \cr
	\code{\link[spatstat.geom]{rsyst}} \tab
	systematic random sample of points \cr
	\code{\link[spatstat.geom]{rjitter}} \tab
	apply random displacements to points in a pattern\cr
	\code{\link[spatstat.random]{rMaternI}}  \tab
	simulate the \Matern Model I inhibition process\cr
	\code{\link[spatstat.random]{rMaternII}} \tab
	simulate the \Matern Model II inhibition process\cr
	\code{\link[spatstat.random]{rSSI}} \tab
	simulate Simple Sequential Inhibition process\cr
	\code{\link[spatstat.random]{rStrauss}} \tab
	simulate Strauss process (perfect simulation)\cr
	\code{\link[spatstat.random]{rHardcore}} \tab
	simulate Hard Core process (perfect simulation)\cr
	\code{\link[spatstat.random]{rStraussHard}} \tab
	simulate Strauss-hard core process (perfect simulation)\cr
	\code{\link[spatstat.random]{rDiggleGratton}} \tab
	simulate Diggle-Gratton process (perfect simulation)\cr
	\code{\link[spatstat.random]{rDGS}} \tab
	simulate Diggle-Gates-Stibbard process (perfect simulation)\cr
	\code{\link[spatstat.random]{rPenttinen}} \tab
	simulate Penttinen process (perfect simulation)\cr
	\code{\link[spatstat.random]{rNeymanScott}} \tab
	simulate a general Neyman-Scott process\cr
	\code{\link[spatstat.random]{rPoissonCluster}} \tab
	simulate a general Poisson cluster process\cr
	\code{\link[spatstat.random]{rMatClust}} \tab
	simulate the \Matern Cluster process\cr
	\code{\link[spatstat.random]{rThomas}} \tab
	simulate the Thomas process  \cr
	\code{\link[spatstat.random]{rGaussPoisson}}  \tab
	simulate the Gauss-Poisson cluster process\cr
	\code{\link[spatstat.random]{rCauchy}} \tab
	simulate Neyman-Scott Cauchy cluster process \cr
	\code{\link[spatstat.random]{rVarGamma}} \tab
	simulate Neyman-Scott Variance Gamma cluster process \cr
	\code{\link[spatstat.random]{rthin}} \tab  random thinning  \cr
	\code{\link[spatstat.random]{rthinclumps}} \tab  random thinning  \cr
	\code{\link[spatstat.random]{rcell}} \tab
	simulate the Baddeley-Silverman cell process  \cr
	\code{\link[spatstat.random]{rmh}} \tab
	simulate Gibbs point process using Metropolis-Hastings \cr
	\code{\link[spatstat.model]{simulate.ppm}} \tab
	simulate Gibbs point process using Metropolis-Hastings \cr
	\code{\link[spatstat.random]{runifpointOnLines}} \tab
	generate \eqn{n} random points along specified line segments \cr
	\code{\link[spatstat.random]{rpoisppOnLines}} \tab
	generate Poisson random points along specified line segments 
      }

      \bold{To randomly change an existing point pattern:}
      
      \tabular{ll}{
	\code{\link[spatstat.random]{rshift}} \tab random shifting of points \cr
        \code{\link[spatstat.geom]{rjitter}} \tab
	apply random displacements to points in a pattern\cr
	\code{\link[spatstat.random]{rthin}} \tab  random thinning \cr
	\code{\link[spatstat.random]{rlabel}} \tab random (re)labelling of a multitype
	point pattern \cr
        \code{\link[spatstat.random]{quadratresample}} \tab block resampling 
      }

      \bold{Standard point pattern datasets:}

      Datasets in \pkg{spatstat} are lazy-loaded, so you can simply
      type the name of the dataset to use it; there is no need
      to type \code{\link[utils]{data}(amacrine)} etc.

      Type \code{demo(data)} to see a display of all the datasets
      installed with the package.

      Type \code{vignette('datasets')} for a document giving an overview
      of all datasets, including background information, and plots.
      
      \tabular{ll}{
	\code{\link[spatstat.data]{amacrine}} \tab Austin Hughes' rabbit amacrine cells \cr
	\code{\link[spatstat.data]{anemones}} \tab Upton-Fingleton sea anemones data\cr
	\code{\link[spatstat.data]{ants}} \tab Harkness-Isham ant nests data\cr
	\code{\link[spatstat.data]{bdspots}} \tab Breakdown spots in microelectrodes \cr
	\code{\link[spatstat.data]{bei}} \tab Tropical rainforest trees \cr
	\code{\link[spatstat.data]{betacells}} \tab Waessle et al. cat retinal ganglia data \cr
	\code{\link[spatstat.data]{bramblecanes}} \tab Bramble Canes data \cr
	\code{\link[spatstat.data]{bronzefilter}} \tab Bronze Filter Section data \cr
	\code{\link[spatstat.data]{cells}} \tab Crick-Ripley biological cells data \cr
	\code{\link[spatstat.data]{chicago}} \tab Chicago crimes \cr
	\code{\link[spatstat.data]{chorley}} \tab Chorley-Ribble cancer data \cr
	\code{\link[spatstat.data]{clmfires}} \tab Castilla-La Mancha forest fires \cr
	\code{\link[spatstat.data]{copper}} \tab Berman-Huntington copper deposits data \cr
	\code{\link[spatstat.data]{dendrite}} \tab Dendritic spines \cr
	\code{\link[spatstat.data]{demohyper}} \tab Synthetic point patterns\cr
	\code{\link[spatstat.data]{demopat}} \tab Synthetic point pattern \cr
	\code{\link[spatstat.data]{finpines}} \tab Finnish Pines data \cr
	\code{\link[spatstat.data]{flu}} \tab Influenza virus proteins \cr
	\code{\link[spatstat.data]{gordon}} \tab People in Gordon Square, London \cr
	\code{\link[spatstat.data]{gorillas}} \tab Gorilla nest sites \cr
	\code{\link[spatstat.data]{hamster}} \tab Aherne's hamster tumour data \cr
	\code{\link[spatstat.data]{humberside}} \tab North Humberside childhood leukaemia data \cr
	\code{\link[spatstat.data]{hyytiala}} \tab {Mixed forest in
              \ifelse{latex}{\out{Hyyti{\"a}l{\"a}}}{Hyytiala}, Finland}\cr
	\code{\link[spatstat.data]{japanesepines}} \tab Japanese Pines data \cr
	\code{\link[spatstat.data]{lansing}} \tab Lansing Woods data \cr
	\code{\link[spatstat.data]{longleaf}} \tab Longleaf Pines data \cr
	\code{\link[spatstat.data]{mucosa}} \tab Cells in gastric mucosa \cr
	\code{\link[spatstat.data]{murchison}} \tab Murchison gold deposits \cr
	\code{\link[spatstat.data]{nbfires}} \tab New Brunswick fires data \cr
	\code{\link[spatstat.data]{nztrees}} \tab Mark-Esler-Ripley trees data \cr
	\code{\link[spatstat.data]{osteo}} \tab Osteocyte lacunae (3D, replicated) \cr
	\code{\link[spatstat.data]{paracou}} \tab Kimboto trees in Paracou, French Guiana \cr
	\code{\link[spatstat.data]{ponderosa}} \tab Getis-Franklin ponderosa pine trees data \cr
	\code{\link[spatstat.data]{pyramidal}} \tab Pyramidal neurons from 31 brains \cr
	\code{\link[spatstat.data]{redwood}} \tab Strauss-Ripley redwood saplings data \cr
	\code{\link[spatstat.data]{redwoodfull}} \tab Strauss redwood saplings data (full set) \cr
	\code{\link[spatstat.data]{residualspaper}} \tab Data from Baddeley et al (2005) \cr
	\code{\link[spatstat.data]{shapley}} \tab Galaxies in an astronomical survey \cr
	\code{\link[spatstat.data]{simdat}} \tab Simulated point pattern (inhomogeneous, with interaction) \cr
	\code{\link[spatstat.data]{spiders}} \tab Spider webs on mortar lines of brick wall \cr
	\code{\link[spatstat.data]{sporophores}} \tab Mycorrhizal fungi around a tree \cr
	\code{\link[spatstat.data]{spruces}} \tab Spruce trees in Saxonia \cr
	\code{\link[spatstat.data]{swedishpines}} \tab Strand-Ripley Swedish pines data \cr
	\code{\link[spatstat.data]{urkiola}} \tab Urkiola Woods data \cr
	\code{\link[spatstat.data]{waka}} \tab Trees in Waka national park \cr
	\code{\link[spatstat.data]{waterstriders}} \tab Insects on water surface 
      }

      \bold{To display a 2D point pattern:}

      \tabular{ll}{
	\code{\link[spatstat.geom]{plot.ppp}} \tab
	plot a point pattern (e.g. \code{plot(X)}) \cr
	\code{spatstat.gui::iplot} \tab
	plot a point pattern interactively \cr
	\code{\link[spatstat.geom]{persp.ppp}} \tab
	perspective plot of marked point pattern
      }

      \bold{To manipulate a 2D point pattern:}

      \tabular{ll}{
	\code{\link[spatstat.geom]{edit.ppp}} \tab interactive text editor \cr
	\code{\link[spatstat.geom]{[.ppp}} \tab
	extract or replace a subset of a point pattern \cr
        \tab \code{pp[subset]} or \code{pp[subwindow]} \cr
	\code{\link[spatstat.geom]{subset.ppp}} \tab
	extract subset of point pattern satisfying a condition \cr
	\code{\link[spatstat.geom]{superimpose}} \tab
	combine several point patterns  \cr
	\code{\link[spatstat.geom]{by.ppp}} \tab
	apply a function to sub-patterns of a point pattern \cr
	\code{\link[spatstat.geom]{cut.ppp}} \tab
	classify the points in a point pattern \cr
	\code{\link[spatstat.geom]{split.ppp}} \tab
	divide pattern into sub-patterns \cr
	\code{\link[spatstat.geom]{unmark}} \tab
	remove marks  \cr
	\code{\link[spatstat.geom]{npoints}} \tab
	count the number of points  \cr
	\code{\link[spatstat.geom]{coords}} \tab
	extract coordinates, change coordinates  \cr
	\code{\link[spatstat.geom]{marks}} \tab
	extract marks, change marks or attach marks  \cr
	\code{\link[spatstat.geom]{rotate}} \tab
	rotate pattern  \cr
	\code{\link[spatstat.geom]{shift} } \tab
	translate pattern  \cr
	\code{\link[spatstat.geom]{flipxy} } \tab
	swap \eqn{x} and \eqn{y} coordinates  \cr
	\code{\link[spatstat.geom]{reflect} } \tab
	reflect in the origin  \cr
	\code{\link[spatstat.geom]{periodify} } \tab
	make several translated copies  \cr
	\code{\link[spatstat.geom]{affine}} \tab
	apply affine transformation\cr
	\code{\link[spatstat.geom]{scalardilate}} \tab
	apply scalar dilation\cr
	\code{\link[spatstat.explore]{density.ppp}} \tab
	kernel estimation of point pattern intensity\cr
	\code{\link[spatstat.explore]{densityHeat.ppp}} \tab
	diffusion kernel estimation of point pattern intensity\cr
	\code{\link[spatstat.explore]{Smooth.ppp}} \tab
	kernel smoothing of marks of point pattern\cr
	\code{\link[spatstat.explore]{SmoothHeat.ppp}} \tab
	diffusion smoothing of marks of point pattern\cr
	\code{\link[spatstat.geom]{nnmark}} \tab
	mark value of nearest data point\cr
	\code{\link[spatstat.explore]{sharpen.ppp}} \tab
	data sharpening\cr
	\code{\link[spatstat.geom]{identify.ppp}} \tab
	interactively identify points \cr
	\code{\link[spatstat.geom]{unique.ppp}} \tab
	remove duplicate points \cr
	\code{\link[spatstat.geom]{duplicated.ppp}} \tab
	determine which points are duplicates \cr
	\code{\link[spatstat.geom]{uniquemap.ppp}} \tab
	map duplicated points to unique points \cr
	\code{\link[spatstat.geom]{connected.ppp}} \tab find clumps of points \cr
	\code{\link[spatstat.geom]{dirichlet}} \tab
	compute Dirichlet-Voronoi tessellation \cr
	\code{\link[spatstat.geom]{delaunay}} \tab
	compute Delaunay triangulation \cr 
	\code{\link[spatstat.geom]{delaunayDistance}} \tab
	graph distance in Delaunay triangulation \cr 
	\code{\link[spatstat.geom]{convexhull}} \tab compute convex hull \cr
	\code{\link[spatstat.geom]{discretise}} \tab discretise coordinates \cr
	\code{\link[spatstat.geom]{pixellate.ppp}} \tab approximate point pattern by 
	pixel image \cr
	\code{\link[spatstat.geom]{as.im.ppp}} \tab approximate point pattern by 
	pixel image 
      }
      See \code{\link[spatstat.geom]{spatstat.options}} to control plotting behaviour.
      
    \bold{To create a window:}

    An object of class \code{"owin"} describes a spatial region
      (a window of observation).

      \tabular{ll}{
	\code{\link[spatstat.geom]{owin}}	\tab	Create a window object \cr
	\tab \code{owin(xlim, ylim)} for rectangular window \cr
	\tab \code{owin(poly)} for polygonal window \cr
	\tab \code{owin(mask)} for binary image window \cr
	\code{\link[spatstat.geom]{Window}}	\tab
	Extract window of another object \cr
	\code{\link[spatstat.geom]{Frame}}	\tab
	Extract the containing rectangle ('frame') of another object \cr
	\code{\link[spatstat.geom]{as.owin}}	\tab
	Convert other data to a window object \cr
	\code{\link[spatstat.geom]{square}}    \tab	make a square window \cr
	\code{\link[spatstat.geom]{disc}}    \tab	make a circular window \cr
	\code{\link[spatstat.geom]{ellipse}}    \tab	make an elliptical window \cr
	\code{\link[spatstat.geom]{ripras}}    \tab
	Ripley-Rasson estimator of window, given only the points \cr
	\code{\link[spatstat.geom]{convexhull}} \tab compute convex hull of something \cr 
	\code{\link[spatstat.data]{letterR}}    \tab
	polygonal window in the shape of the \R logo \cr
	\code{\link[spatstat.geom]{clickpoly}}    \tab
	interactively draw a polygonal window  \cr
	\code{\link[spatstat.geom]{clickbox}}    \tab
	interactively draw a rectangle  
      }

    \bold{To manipulate a window:}

    \tabular{ll}{
	\code{\link[spatstat.geom]{plot.owin}}	\tab	plot a window. \cr
	\tab		\code{plot(W)}\cr
	\code{\link[spatstat.geom]{boundingbox}} \tab
	Find a tight bounding box for the window \cr
	\code{\link[spatstat.geom]{erosion}}	\tab
	erode window by a distance r\cr
	\code{\link[spatstat.geom]{dilation}}	\tab
	dilate window by a distance r\cr
	\code{\link[spatstat.geom]{closing}}	\tab
	close window by a distance r\cr
	\code{\link[spatstat.geom]{opening}}	\tab
	open window by a distance r\cr
	\code{\link[spatstat.geom]{border}}	\tab
	difference between window and its erosion/dilation \cr 
	\code{\link[spatstat.geom]{complement.owin}}	\tab
	invert (swap inside and outside)\cr
	\code{\link[spatstat.geom]{simplify.owin}}	\tab
	approximate a window by a simple polygon  \cr
	\code{\link[spatstat.geom]{fillholes.owin}}	\tab
	remove small holes from a window  \cr
	\code{\link[spatstat.geom]{rotate}} \tab rotate window  \cr
	\code{\link[spatstat.geom]{flipxy}} \tab swap \eqn{x} and \eqn{y} coordinates  \cr
	\code{\link[spatstat.geom]{shift} } \tab translate window  \cr
	\code{\link[spatstat.geom]{periodify} } \tab make several translated copies  \cr
	\code{\link[spatstat.geom]{affine}} \tab apply affine transformation \cr
	\code{\link[spatstat.geom]{as.data.frame.owin}} \tab
	convert window to data frame 
      }

    \bold{Digital approximations:}

    \tabular{ll}{
	\code{\link[spatstat.geom]{as.mask}}	\tab
	Make a discrete pixel approximation of a given window \cr
	\code{\link[spatstat.geom]{as.im.owin}} \tab convert window to pixel image \cr
	\code{\link[spatstat.geom]{pixellate.owin}} \tab convert window to pixel image \cr
	\code{\link[spatstat.geom]{commonGrid}} \tab find common pixel grid for windows \cr
	\code{\link[spatstat.geom]{nearest.raster.point}} \tab
	map continuous coordinates to raster locations\cr
	\code{\link[spatstat.geom]{raster.x}} \tab
	raster x coordinates \cr
	\code{\link[spatstat.geom]{raster.y}} \tab
	raster y coordinates \cr
	\code{\link[spatstat.geom]{raster.xy}} \tab
	raster x and y coordinates \cr
	\code{\link[spatstat.geom]{as.polygonal}} \tab
	convert pixel mask to polygonal window 
      }
      See \code{\link[spatstat.geom]{spatstat.options}} to control the approximation

    \bold{Geometrical computations with windows:}

    \tabular{ll}{
	\code{\link[spatstat.geom]{edges}}	\tab	extract boundary edges \cr
	\code{\link[spatstat.geom]{intersect.owin}}	\tab	intersection of two windows\cr
	\code{\link[spatstat.geom]{union.owin}}	\tab	union of two windows\cr
	\code{\link[spatstat.geom]{setminus.owin}}	\tab	set subtraction of two windows\cr
	\code{\link[spatstat.geom]{inside.owin}}	\tab	determine whether a point is inside a window\cr
	\code{\link[spatstat.geom]{area.owin}}	\tab	compute area \cr
	\code{\link[spatstat.geom]{perimeter}}	\tab	compute perimeter length \cr
	\code{\link[spatstat.geom]{diameter.owin}}	\tab	compute diameter\cr
	\code{\link[spatstat.geom]{incircle}}	\tab	find largest circle inside a window \cr
	\code{\link[spatstat.geom]{inradius}}	\tab	radius of incircle \cr
	\code{\link[spatstat.geom]{connected.owin}}    \tab find connected components of window \cr
	\code{\link[spatstat.geom]{eroded.areas}}	\tab	compute areas of eroded windows\cr
	\code{\link[spatstat.geom]{dilated.areas}}	\tab	compute areas of dilated windows\cr
	\code{\link[spatstat.geom]{bdist.points}}	\tab	compute distances from data points to window boundary \cr
	\code{\link[spatstat.geom]{bdist.pixels}}	\tab	compute distances from all pixels to window boundary \cr
	\code{\link[spatstat.geom]{bdist.tiles}}	\tab
	boundary distance for each tile in tessellation \cr
	\code{\link[spatstat.geom]{distmap.owin}}	\tab	distance transform image \cr
	\code{\link[spatstat.geom]{distfun.owin}}	\tab	distance transform \cr
	\code{\link[spatstat.geom]{centroid.owin}}	\tab compute centroid (centre of mass) of window\cr
	\code{\link[spatstat.geom]{is.subset.owin}}    \tab determine whether one
	window contains another \cr
	\code{\link[spatstat.geom]{is.convex}} \tab determine whether a window is convex \cr 
	\code{\link[spatstat.geom]{convexhull}} \tab compute convex hull \cr
	\code{\link[spatstat.geom]{triangulate.owin}} \tab decompose into triangles \cr
	\code{\link[spatstat.geom]{as.mask}} \tab pixel approximation of window \cr
	\code{\link[spatstat.geom]{as.polygonal}} \tab polygonal approximation of window \cr
	\code{\link[spatstat.geom]{is.rectangle}} \tab test whether window is a rectangle \cr 
	\code{\link[spatstat.geom]{is.polygonal}} \tab test whether window is polygonal \cr 
	\code{\link[spatstat.geom]{is.mask}} \tab test whether window is a mask \cr 
	\code{\link[spatstat.geom]{setcov}} \tab spatial covariance function of window \cr
	\code{\link[spatstat.geom]{pixelcentres}} \tab extract centres of pixels in mask \cr
	\code{\link[spatstat.geom]{clickdist}}    \tab
	measure distance between two points clicked by user
      }

    \bold{Pixel images:}
    An object of class \code{"im"} represents a pixel image. 
    Such objects are returned by some of the functions in
      \pkg{spatstat} including \code{\link[spatstat.explore]{Kmeasure}},
      \code{\link[spatstat.geom]{setcov}} and \code{\link[spatstat.explore]{density.ppp}}. 
      \tabular{ll}{
	\code{\link[spatstat.geom]{im}} \tab create a pixel image\cr
	\code{\link[spatstat.geom]{as.im}} \tab convert other data to a pixel image\cr
	\code{\link[spatstat.geom]{pixellate}} \tab convert other data to a pixel image\cr
	\code{\link[spatstat.geom]{as.matrix.im}} \tab convert pixel image to matrix\cr
	\code{\link[spatstat.geom]{as.data.frame.im}} \tab convert pixel image to data frame\cr
	\code{\link[spatstat.geom]{as.function.im}} \tab convert pixel image to function\cr
	\code{\link[spatstat.geom]{plot.im}}	\tab	plot a pixel image on screen as a digital image\cr
	\code{\link[spatstat.geom]{contour.im}}	\tab draw contours of a pixel image \cr
	\code{\link[spatstat.geom]{persp.im}}	\tab draw perspective plot of a pixel image \cr
	\code{\link[spatstat.geom]{rgbim}}	\tab create colour-valued pixel image \cr
	\code{\link[spatstat.geom]{hsvim}}	\tab create colour-valued pixel image \cr
	\code{\link[spatstat.geom]{[.im}} 	\tab extract a subset of a pixel image\cr
	\code{\link[spatstat.geom]{[<-.im}} 	\tab replace a subset of a pixel image\cr
	\code{\link[spatstat.geom]{rotate.im}} \tab rotate pixel image \cr
	\code{\link[spatstat.geom]{shift.im}} \tab apply vector shift to pixel image \cr
	\code{\link[spatstat.geom]{affine.im}} \tab apply affine transformation to image \cr
	\code{X}	\tab	print very basic information about image \code{X}\cr
	\code{\link[spatstat.geom:summary.im]{summary}(X)} \tab	summary of image \code{X} \cr
	\code{\link[spatstat.geom]{hist.im}} \tab	histogram of image \cr
	\code{\link[spatstat.geom]{mean.im}} \tab	mean pixel value of image  \cr
	\code{\link[spatstat.geom]{integral.im}} \tab	integral of pixel values  \cr
	\code{\link[spatstat.geom]{quantile.im}} \tab	quantiles of image  \cr
	\code{\link[spatstat.geom]{cut.im}} \tab	convert numeric image to factor image \cr
	\code{\link[spatstat.geom]{is.im}} \tab test whether an object is a pixel image\cr
	\code{\link[spatstat.geom]{interp.im}} \tab interpolate a pixel image\cr
	\code{\link[spatstat.explore]{blur}} \tab apply Gaussian blur to image\cr
	\code{\link[spatstat.explore]{blurHeat}} \tab apply diffusion blur to image\cr
	\code{\link[spatstat.explore]{Smooth.im}} \tab apply Gaussian blur to image\cr
	\code{\link[spatstat.explore]{SmoothHeat.im}} \tab apply diffusion blur to image\cr
	\code{\link[spatstat.geom]{connected.im}} \tab find connected components \cr
	\code{\link[spatstat.geom]{compatible.im}} \tab test whether two images have
	compatible dimensions \cr
	\code{\link[spatstat.geom]{harmonise.im}} \tab make images compatible \cr
	\code{\link[spatstat.geom]{commonGrid}} \tab find a common pixel grid for images \cr
	\code{\link[spatstat.geom]{eval.im}} \tab evaluate any expression involving images\cr
	\code{\link[spatstat.geom]{im.apply}} \tab evaluate a function of several images \cr
	\code{\link[spatstat.geom]{scaletointerval}} \tab rescale pixel values \cr
	\code{\link[spatstat.geom]{zapsmall.im}} \tab set very small pixel values to zero \cr
	\code{\link[spatstat.geom]{levelset}} \tab level set of an image\cr 
	\code{\link[spatstat.geom]{solutionset}} \tab region where an expression is true \cr
	\code{\link[spatstat.geom]{imcov}} \tab covariance image \cr
	\code{\link[spatstat.explore]{spatcov}} \tab spatial covariance function of image \cr
	\code{\link[spatstat.geom]{convolve.im}} \tab spatial convolution of images \cr
	\code{\link[spatstat.explore]{transect.im}} \tab line transect of image \cr
	\code{\link[spatstat.geom]{pixelcentres}} \tab extract centres of pixels \cr
	\code{\link[spatstat.geom]{transmat}} \tab convert matrix of pixel values \cr
	                       \tab to a different indexing convention \cr
	\code{\link[spatstat.random]{rnoise}} \tab random pixel noise
     }

    \bold{Line segment patterns}

    An object of class \code{"psp"} represents a pattern of straight line
    segments.
    \tabular{ll}{
      \code{\link[spatstat.geom]{psp}} \tab create a line segment pattern \cr
      \code{\link[spatstat.geom]{as.psp}} \tab convert other data into a line segment pattern \cr
      \code{\link[spatstat.geom]{edges}} \tab extract edges of a window \cr
      \code{\link[spatstat.geom]{is.psp}} \tab determine whether a dataset has class \code{"psp"} \cr
      \code{\link[spatstat.geom]{plot.psp}} \tab plot a line segment pattern \cr
      \code{\link[spatstat.geom]{print.psp}} \tab print basic information \cr
      \code{\link[spatstat.geom]{summary.psp}} \tab print summary information \cr
      \code{\link[spatstat.geom]{[.psp}} \tab extract a subset of a line segment pattern \cr
      \code{\link[spatstat.geom]{subset.psp}} \tab extract subset of line segment pattern \cr
      \code{\link[spatstat.geom]{as.data.frame.psp}} \tab
      convert line segment pattern to data frame \cr
      \code{\link[spatstat.geom]{marks.psp}} \tab extract marks of line segments \cr
      \code{\link[spatstat.geom]{marks<-.psp}} \tab assign new marks to line segments \cr
      \code{\link[spatstat.geom]{unmark.psp}} \tab delete marks from line segments \cr
      \code{\link[spatstat.geom]{midpoints.psp}} \tab
      compute the midpoints of line segments \cr
      \code{\link[spatstat.geom]{endpoints.psp}} \tab extract the endpoints of line segments \cr
      \code{\link[spatstat.geom]{lengths_psp}} \tab compute the lengths of line segments \cr
      \code{\link[spatstat.geom]{angles.psp}} \tab compute the orientation angles of line segments \cr
      \code{\link[spatstat.geom]{superimpose}} \tab combine several line segment patterns  \cr
      \code{\link[spatstat.geom]{flipxy}} \tab swap \eqn{x} and \eqn{y} coordinates \cr      
      \code{\link[spatstat.geom]{rotate.psp}} \tab rotate a line segment pattern \cr      
      \code{\link[spatstat.geom]{shift.psp}} \tab shift a line segment pattern \cr      
      \code{\link[spatstat.geom]{periodify}} \tab make several shifted copies \cr      
      \code{\link[spatstat.geom]{affine.psp}} \tab apply an affine transformation \cr      
      \code{\link[spatstat.geom]{pixellate.psp}} \tab approximate line segment pattern
    by pixel image \cr      
      \code{\link[spatstat.geom]{psp2mask}} \tab approximate line segment pattern
    by binary mask \cr      
      \code{\link[spatstat.geom]{distmap.psp}} \tab compute the distance map of a line
      segment pattern \cr
      \code{\link[spatstat.geom]{distfun.psp}} \tab compute the distance map of a line
      segment pattern \cr
      \code{\link[spatstat.explore]{density.psp}} \tab kernel smoothing of line segments\cr
      \code{\link[spatstat.geom]{selfcrossing.psp}} \tab find crossing points between
      line segments \cr
      \code{\link[spatstat.geom]{selfcut.psp}} \tab cut segments where they cross \cr
      \code{\link[spatstat.geom]{crossing.psp}} \tab find crossing points between
      two line segment patterns \cr
      \code{\link[spatstat.geom]{extrapolate.psp}} \tab extrapolate line segments to
      infinite lines \cr
      \code{\link[spatstat.geom]{nncross}} \tab find distance to nearest line segment
      from a given point\cr
      \code{\link[spatstat.geom]{nearestsegment}} \tab find line segment closest to a
      given point \cr
      \code{\link[spatstat.geom]{project2segment}} \tab find location along a line segment
      closest to a given point \cr
      \code{\link[spatstat.geom]{pointsOnLines}} \tab generate points evenly spaced
      along line segment \cr
      \code{\link[spatstat.random]{rpoisline}} \tab generate a realisation of the
      Poisson line process inside a window\cr
      \code{\link[spatstat.geom]{rlinegrid}} \tab generate a random array of parallel
      lines through a window
    }

    \bold{Tessellations}

    An object of class \code{"tess"} represents a tessellation.

    \tabular{ll}{
      \code{\link[spatstat.geom]{tess}} \tab create a tessellation \cr
      \code{\link[spatstat.geom]{quadrats}} \tab create a tessellation of rectangles\cr
      \code{\link[spatstat.geom]{hextess}} \tab create a tessellation of hexagons \cr
      \code{\link[spatstat.geom]{polartess}} \tab tessellation using polar coordinates \cr
      \code{\link[spatstat.geom]{quantess}} \tab quantile tessellation \cr
      \code{\link[spatstat.geom]{venn.tess}} \tab Venn diagram tessellation \cr 
      \code{\link[spatstat.geom]{dirichlet}} \tab compute Dirichlet-Voronoi tessellation of points\cr
      \code{\link[spatstat.geom]{delaunay}} \tab compute Delaunay triangulation of points\cr
      \code{\link[spatstat.geom]{bufftess}} \tab Distance buffer tessellation \cr 
      \code{\link[spatstat.geom]{as.tess}} \tab convert other data to a tessellation \cr
      \code{\link[spatstat.geom]{plot.tess}} \tab plot a tessellation \cr
      \code{\link[spatstat.geom]{identify.tess}} \tab interactively identify
      tiles of a tessellation \cr
      \code{\link[spatstat.geom]{tiles}} \tab extract all the tiles of a tessellation \cr
      \code{\link[spatstat.geom]{[.tess}} \tab extract some tiles of a tessellation \cr
      \code{\link[spatstat.geom]{[<-.tess}} \tab change some tiles of a tessellation \cr
      \code{\link[spatstat.geom]{intersect.tess}} \tab intersect two tessellations \cr
                              \tab or restrict a tessellation to a window \cr
      \code{\link[spatstat.geom]{chop.tess}} \tab subdivide a tessellation by a line \cr
      \code{\link[spatstat.random]{rpoislinetess}} \tab generate tessellation using Poisson line
      process \cr
      \code{\link[spatstat.geom]{tilenames}}	\tab string name of each tile \cr
      \code{\link[spatstat.geom]{tile.areas}}	\tab
      area of each tile in tessellation \cr
      \code{\link[spatstat.geom]{tile.centroids}}	\tab
      centroid of each tile in tessellation \cr
      \code{\link[spatstat.geom]{bdist.tiles}}	\tab
      boundary distance for each tile in tessellation \cr
      \code{\link[spatstat.geom]{connected.tess}} \tab find connected components of tiles \cr
      \code{\link[spatstat.geom]{shift.tess}} \tab shift a tessellation \cr
      \code{\link[spatstat.geom]{rotate.tess}} \tab rotate a tessellation \cr
      \code{\link[spatstat.geom]{reflect.tess}} \tab reflect about the origin \cr
      \code{\link[spatstat.geom]{flipxy.tess}} \tab reflect about the diagonal \cr
      \code{\link[spatstat.geom]{affine.tess}} \tab apply affine transformation 
    }

    \bold{Three-dimensional point patterns}

    An object of class \code{"pp3"} represents a three-dimensional
    point pattern in a rectangular box. The box is represented by
    an object of class \code{"box3"}.

    \tabular{ll}{
      \code{\link[spatstat.geom]{pp3}} \tab create a 3-D point pattern \cr
      \code{\link[spatstat.geom]{plot.pp3}} \tab plot a 3-D point pattern \cr
      \code{\link[spatstat.geom]{coords}} \tab extract coordinates \cr
      \code{\link[spatstat.geom]{as.hyperframe}} \tab extract coordinates \cr
      \code{\link[spatstat.geom]{subset.pp3}} \tab extract subset of 3-D point pattern \cr
      \code{\link[spatstat.geom]{unitname.pp3}} \tab name of unit of length \cr
      \code{\link[spatstat.geom]{npoints}} \tab count the number of points  \cr
      \code{\link[spatstat.random]{runifpoint3}} \tab generate uniform random points in 3-D \cr
      \code{\link[spatstat.random]{rpoispp3}} \tab generate Poisson random points in 3-D \cr
      \code{\link[spatstat.explore]{envelope.pp3}} \tab generate simulation envelopes for
      3-D pattern \cr
      \code{\link[spatstat.geom]{box3}} \tab create a 3-D rectangular box \cr
      \code{\link[spatstat.geom]{as.box3}} \tab convert data to 3-D rectangular box \cr
      \code{\link[spatstat.geom]{unitname.box3}} \tab name of unit of length \cr
      \code{\link[spatstat.geom]{diameter.box3}} \tab diameter of box \cr
      \code{\link[spatstat.geom]{volume.box3}} \tab volume of box \cr
      \code{\link[spatstat.geom]{shortside.box3}} \tab shortest side of box \cr
      \code{\link[spatstat.geom]{eroded.volumes}} \tab volumes of erosions of box 
    }

    \bold{Multi-dimensional space-time point patterns}

    An object of class \code{"ppx"} represents a 
    point pattern in multi-dimensional space and/or time.

    \tabular{ll}{
      \code{\link[spatstat.geom]{ppx}} \tab create a multidimensional space-time point pattern \cr
      \code{\link[spatstat.geom]{coords}} \tab extract coordinates \cr
      \code{\link[spatstat.geom]{as.hyperframe}} \tab extract coordinates \cr
      \code{\link[spatstat.geom]{subset.ppx}} \tab extract subset \cr
      \code{\link[spatstat.geom]{unitname.ppx}} \tab name of unit of length \cr
      \code{\link[spatstat.geom]{npoints}} \tab count the number of points  \cr
      \code{\link[spatstat.random]{runifpointx}} \tab generate uniform random points \cr
      \code{\link[spatstat.random]{rpoisppx}} \tab generate Poisson random points \cr
      \code{\link[spatstat.geom]{boxx}} \tab define multidimensional box  \cr
      \code{\link[spatstat.geom]{diameter.boxx}} \tab diameter of box \cr
      \code{\link[spatstat.geom]{volume.boxx}} \tab volume of box \cr
      \code{\link[spatstat.geom]{shortside.boxx}} \tab shortest side of box \cr
      \code{\link[spatstat.geom]{eroded.volumes.boxx}} \tab volumes of erosions of box 
    }
    
    \bold{Point patterns on a linear network}

    An object of class \code{"linnet"} represents a linear network
    (for example, a road network).

    \tabular{ll}{
      \code{\link[spatstat.linnet]{linnet}} \tab create a linear network \cr
      \code{\link[spatstat.linnet]{clickjoin}} \tab interactively join vertices in network \cr
      \code{\link[spatstat.linnet]{identify.linnet}} \tab interactively identify
      segments of a network \cr
      \code{spatstat.gui::iplot.linnet} \tab interactively plot network \cr
      \code{\link[spatstat.data]{simplenet}} \tab simple example of network \cr
      \code{\link[spatstat.linnet]{lineardisc}} \tab disc in a linear network \cr
      \code{\link[spatstat.linnet]{delaunayNetwork}} \tab network of Delaunay triangulation \cr
      \code{\link[spatstat.linnet]{dirichletNetwork}} \tab network of Dirichlet edges \cr
      \code{\link[spatstat.linnet]{methods.linnet}} \tab methods for \code{linnet} objects\cr
      \code{\link[spatstat.linnet]{vertices.linnet}} \tab nodes of network \cr
      \code{\link[spatstat.linnet]{joinVertices}} \tab join existing vertices in a network \cr
      \code{\link[spatstat.linnet]{insertVertices}} \tab insert new vertices at positions
      along a network \cr
      \code{\link[spatstat.linnet]{addVertices}} \tab add new vertices, extending a network \cr
      \code{\link[spatstat.linnet]{thinNetwork}} \tab remove vertices or lines from a network \cr
      \code{\link[spatstat.linnet]{repairNetwork}} \tab repair internal format \cr
      \code{\link[spatstat.linnet]{terminalvertices}} \tab identify terminal vertices \cr
      \code{\link[spatstat.linnet]{pixellate.linnet}} \tab approximate by pixel
      image \cr
      \code{\link[spatstat.linnet]{shortestpath}} \tab shortest path between two
      points on network 
    }
    
    An object of class \code{"lpp"} represents a 
    point pattern on a linear network (for example,
    road accidents on a road network).
    
    \tabular{ll}{
      \code{\link[spatstat.linnet]{lpp}} \tab create a point pattern on a linear network \cr
      \code{\link[spatstat.linnet]{identify.lpp}} \tab interactively identify
      points on a network \cr
      \code{\link[spatstat.linnet]{methods.lpp}} \tab methods for \code{lpp} objects \cr
      \code{\link[spatstat.linnet]{subset.lpp}} \tab method for \code{subset} \cr
      \code{\link[spatstat.linnet]{rpoislpp}} \tab simulate Poisson points on linear network \cr
      \code{\link[spatstat.linnet]{runiflpp}} \tab simulate random points on a
      linear network \cr      
      \code{\link[spatstat.linnet]{rcelllpp}} \tab simulate cell process on a network \cr
      \code{\link[spatstat.linnet]{rSwitzerlpp}} \tab simulate Switzer-type process on network \cr
      \code{\link[spatstat.data]{chicago}} \tab Chicago crime data \cr
      \code{\link[spatstat.data]{dendrite}} \tab Dendritic spines data \cr
      \code{\link[spatstat.data]{spiders}} \tab Spider webs on mortar lines of brick wall 
    }

    \bold{Tessellations on a network}

    An object of class \code{"lintess"} represents a tessellation of a
    linear network, that is, a division of the network into disjoint
    pieces.

    \tabular{ll}{
      \code{\link[spatstat.linnet]{lintess}} \tab create a tessellation of a network \cr
      \code{\link[spatstat.linnet]{as.lintess}} \tab
      convert data to a tessellation \cr
      \code{\link[spatstat.linnet]{identify.lintess}} \tab interactively identify
      tiles of a tessellation\cr
      \code{\link[spatstat.linnet]{nobjects.lintess}} \tab number of tiles \cr
      \code{\link[spatstat.linnet]{marks.lintess}} \tab extract marks attached to
      tiles of tessellation\cr
      \code{\link[spatstat.linnet]{tilenames.lintess}} \tab extract names of
      tiles of tessellation\cr
      \code{\link[spatstat.linnet]{intersect.lintess}} \tab form intersection of
      two tessellations on a network
    }
    
    \bold{Hyperframes}

    A hyperframe is like a data frame, except that the entries
    may be objects of any kind.

    \tabular{ll}{
      \code{\link[spatstat.geom]{hyperframe}} \tab create a hyperframe \cr
      \code{\link[spatstat.geom]{as.hyperframe}} \tab convert data to hyperframe \cr
      \code{\link[spatstat.geom]{plot.hyperframe}} \tab plot hyperframe \cr
      \code{\link[spatstat.geom]{with.hyperframe}} \tab evaluate expression using each row
      of hyperframe \cr
      \code{\link[spatstat.geom]{cbind.hyperframe}} \tab combine hyperframes by columns\cr
      \code{\link[spatstat.geom]{rbind.hyperframe}} \tab combine hyperframes by rows\cr
      \code{\link[spatstat.geom]{as.data.frame.hyperframe}} \tab convert hyperframe to
      data frame \cr
      \code{\link[spatstat.geom]{subset.hyperframe}} \tab method for \code{subset} \cr
      \code{\link[spatstat.geom]{head.hyperframe}} \tab first few rows of hyperframe \cr
      \code{\link[spatstat.geom]{tail.hyperframe}} \tab last few rows of hyperframe
    }
    
    \bold{Layered objects}

    A layered object represents data that should be plotted in
    successive layers, for example, a background and a foreground.

     \tabular{ll}{
      \code{\link[spatstat.geom]{layered}} \tab create layered object \cr
      \code{\link[spatstat.geom]{plot.layered}} \tab plot layered object\cr
      \code{\link[spatstat.geom]{[.layered}} \tab extract subset of layered object
    }    

    \bold{Colour maps}

    A colour map is a mechanism for associating colours with data.
    It can be regarded as a function, mapping data to colours.
    Using a \code{colourmap} object in a plot command
    ensures that the mapping from numbers to colours is
    the same in different plots. 

     \tabular{ll}{
      \code{\link[spatstat.geom]{colourmap}} \tab create a colour map  \cr
      \code{\link[spatstat.geom]{plot.colourmap}} \tab plot the colour map only\cr
      \code{\link[spatstat.geom]{tweak.colourmap}} \tab alter individual colour values \cr
      \code{\link[spatstat.geom]{interp.colourmap}} \tab make a smooth transition
      between colours \cr
      \code{\link[spatstat.geom]{restrict.colourmap}} \tab restrict colours to
      narrower range \cr
      \code{\link[spatstat.geom]{beachcolourmap}} \tab one special colour map \cr
      \code{\link[spatstat.geom]{pHcolourmap}} \tab colour map for pH values \cr
      \code{\link[spatstat.geom]{default.image.colours}} \tab control the default colours
    }

    \bold{Missing objects}

    An object of class \code{"NAobject"} represents a spatial dataset
    which is entirely missing. For example
    \code{NAobject("ppp")} creates a missing spatial point pattern dataset.

    \tabular{ll}{
      \code{\link[spatstat.geom]{NAobject}} \tab Create a missing object \cr
      \code{\link[spatstat.geom]{is.NAobject}} \tab Test whether object is missing \cr
      \code{\link[spatstat.geom]{is.na.hyperframe}} \tab Test which entries
      in a hyperframe are missing
    }
}
  
\section{II. EXPLORATORY DATA ANALYSIS}{

  \bold{Inspection of data:}
  \tabular{ll}{
    \code{\link[spatstat.geom:summary.ppp]{summary}(X)} \tab
    print useful summary of point pattern \code{X}\cr
    \code{X} \tab
    print basic description of point pattern \code{X}  \cr
    \code{any(duplicated(X))} \tab
    check for duplicated points in pattern \code{X} \cr
    \code{spatstat.gui::istat(X)} \tab Interactive exploratory analysis \cr
    \code{spatstat.gui::View.ppp(X)} \tab spreadsheet-style viewer
  }

  \bold{Classical exploratory tools:}
  \tabular{ll}{
    \code{\link[spatstat.explore]{clarkevans}} \tab Clark and Evans aggregation index \cr
    \code{\link[spatstat.explore]{fryplot}} \tab Fry plot \cr
    \code{\link[spatstat.explore]{miplot}} \tab Morisita Index plot
  }

  \bold{Smoothing:}
  \tabular{ll}{
    \code{\link[spatstat.explore]{density.ppp}} \tab kernel smoothed density/intensity\cr
    \code{\link[spatstat.explore]{relrisk}} \tab kernel estimate of relative risk\cr
    \code{\link[spatstat.explore]{relriskHeat}} \tab diffusion estimate of relative risk\cr
    \code{\link[spatstat.explore]{Smooth.ppp}} \tab spatial interpolation of marks  \cr
    \code{\link[spatstat.explore]{SmoothHeat.ppp}} \tab spatial interpolation of marks  \cr
    \code{\link[spatstat.explore]{SpatialMedian.ppp}} \tab locally weighted median mark  \cr
    \code{\link[spatstat.explore]{SpatialQuantile.ppp}} \tab locally weighted
    quantile of marks  \cr
    \code{\link[spatstat.explore]{bw.diggle}} \tab cross-validated bandwidth selection
                               for \code{\link[spatstat.explore]{density.ppp}}\cr
    \code{\link[spatstat.explore]{bw.ppl}} \tab likelihood cross-validated bandwidth selection
                               for \code{\link[spatstat.explore]{density.ppp}}\cr
    \code{\link[spatstat.explore]{bw.CvL}} \tab Cronie-Van Lieshout bandwidth selection
                                for density estimation\cr
    \code{\link[spatstat.explore]{bw.scott}} \tab Scott's rule of thumb bandwidth
                                for density estimation\cr
    \code{\link[spatstat.univar]{bw.taylor}} \tab Taylor's nonrandom bootstrap bandwidth
                                for density estimation\cr
    \code{\link[spatstat.univar]{bw.abram}} \tab Abramson's rule for adaptive bandwidths\cr
    \code{\link[spatstat.explore]{bw.relrisk}} \tab cross-validated bandwidth selection
                               for \code{\link[spatstat.explore]{relrisk}} \cr
    \code{\link[spatstat.explore]{bw.relriskHeatppp}} \tab cross-validated bandwidth selection
                               for \code{\link[spatstat.explore]{relriskHeat.ppp}} \cr
    \code{\link[spatstat.explore]{bw.smoothppp}} \tab cross-validated bandwidth selection
                               for \code{\link[spatstat.explore]{Smooth.ppp}} \cr
    \code{\link[spatstat.explore]{bw.frac}} \tab bandwidth selection using window geometry
  }

  \bold{Modern exploratory tools:}
  \tabular{ll}{
    \code{\link[spatstat.explore]{clusterset}} \tab Allard-Fraley feature detection  \cr
    \code{\link[spatstat.explore]{nnclean}} \tab Byers-Raftery feature detection  \cr
    \code{\link[spatstat.explore]{sharpen.ppp}} \tab Choi-Hall data sharpening \cr
    \code{\link[spatstat.explore]{rhohat}} \tab Kernel estimate of covariate effect\cr
    \code{\link[spatstat.explore]{rho2hat}} \tab Kernel estimate of effect of two covariates\cr
    \code{\link[spatstat.explore]{spatialcdf}} \tab Spatial cumulative distribution function\cr
    \code{\link[spatstat.explore]{roc}} \tab Receiver operating characteristic curve
  }

  \bold{Summary statistics for a point pattern:}
  Type \code{demo(sumfun)} for a demonstration of many
  of the summary statistics.
  \tabular{ll}{
    \code{\link[spatstat.geom]{intensity}} \tab Mean intensity \cr
    \code{\link[spatstat.geom]{quadratcount}} \tab Quadrat counts \cr
    \code{\link[spatstat.geom]{intensity.quadratcount}} \tab Mean intensity in quadrats \cr
    \code{\link[spatstat.explore]{Fest}} \tab empty space function \eqn{F} \cr
    \code{\link[spatstat.explore]{Gest}} \tab nearest neighbour distribution function \eqn{G} \cr
    \code{\link[spatstat.explore]{Jest}} \tab \eqn{J}-function \eqn{J = (1-G)/(1-F)} \cr
    \code{\link[spatstat.explore]{Kest}} \tab Ripley's \eqn{K}-function\cr
    \code{\link[spatstat.explore]{Lest}} \tab Besag \eqn{L}-function\cr
    \code{\link[spatstat.explore]{Tstat}} \tab Third order \eqn{T}-function \cr
    \code{\link[spatstat.explore]{allstats}} \tab all four functions \eqn{F}, \eqn{G}, \eqn{J}, \eqn{K} \cr
    \code{\link[spatstat.explore]{pcf}} \tab 	pair correlation function \cr
    \code{\link[spatstat.explore]{bw.stoyan}} \tab Stoyan's rule of thumb for bandwidth
                               for \code{\link[spatstat.explore]{pcf}} \cr
    \code{\link[spatstat.explore]{bw.pcf}} \tab Cross-validated bandwidth selection
                               for \code{\link[spatstat.explore]{pcf}} \cr
    \code{\link[spatstat.explore]{Kinhom}} \tab \eqn{K} for inhomogeneous point patterns \cr
    \code{\link[spatstat.explore]{Linhom}} \tab \eqn{L} for inhomogeneous point patterns \cr
    \code{\link[spatstat.explore]{pcfinhom}} \tab pair correlation for inhomogeneous patterns\cr
    \code{\link[spatstat.explore]{bw.pcfinhom}} \tab Cross-validated bandwidth selection
                               for \code{\link[spatstat.explore]{pcfinhom}} \cr
    \code{\link[spatstat.explore]{bw.bdh}} \tab Bandwidth adjustment 
                               for \code{\link[spatstat.explore]{pcfinhom}} \cr
    \code{\link[spatstat.explore]{Finhom}} \tab \eqn{F} for inhomogeneous point patterns \cr
    \code{\link[spatstat.explore]{Ginhom}} \tab \eqn{G} for inhomogeneous point patterns \cr
    \code{\link[spatstat.explore]{Jinhom}} \tab \eqn{J} for inhomogeneous point patterns \cr
    \code{\link[spatstat.explore]{localL}} \tab Getis-Franklin neighbourhood density function\cr
    \code{\link[spatstat.explore]{localK}} \tab neighbourhood K-function\cr
    \code{\link[spatstat.explore]{localpcf}} \tab local pair correlation function\cr
    \code{\link[spatstat.explore]{localKinhom}} \tab local \eqn{K} for inhomogeneous point patterns \cr
    \code{\link[spatstat.explore]{localLinhom}} \tab local \eqn{L} for inhomogeneous point patterns \cr
    \code{\link[spatstat.explore]{localpcfinhom}} \tab local pair correlation for inhomogeneous patterns\cr
    \code{\link[spatstat.explore]{Ksector}} \tab Directional \eqn{K}-function\cr
    \code{\link[spatstat.explore]{Kscaled}} \tab locally scaled \eqn{K}-function \cr
    \code{\link[spatstat.explore]{Kest.fft}} \tab fast \eqn{K}-function using FFT for large datasets \cr
    \code{\link[spatstat.explore]{Kmeasure}} \tab reduced second moment measure \cr
    \code{\link[spatstat.explore]{envelope}} \tab simulation envelopes for a summary
    function \cr
    \code{\link[spatstat.explore]{varblock}} \tab variances and confidence intervals\cr
                   \tab for a summary function \cr
    \code{\link[spatstat.explore]{lohboot}} \tab bootstrap for a summary function 
  }

  Related facilities:
  \tabular{ll}{
    \code{\link[spatstat.explore]{plot.fv}} \tab plot a summary function\cr
    \code{\link[spatstat.explore]{eval.fv}} \tab evaluate any expression involving
    summary functions\cr
    \code{\link[spatstat.explore]{harmonise.fv}} \tab make functions compatible \cr
    \code{\link[spatstat.explore]{eval.fasp}} \tab evaluate any expression involving
    an array of functions\cr
    \code{\link[spatstat.explore]{with.fv}} \tab evaluate an expression for a 
    summary function\cr
    \code{\link[spatstat.explore]{integral.fv}} \tab integral of summary function\cr
    \code{\link[spatstat.explore]{Smooth.fv}} \tab apply smoothing to a summary function\cr
    \code{\link[spatstat.explore]{deriv.fv}} \tab calculate derivative of a summary function\cr
    \code{\link[spatstat.explore]{pool.fv}} \tab pool several estimates of a summary function\cr
    \code{\link[spatstat.geom]{nndist}} \tab nearest neighbour distances \cr
    \code{\link[spatstat.geom]{nnwhich}} \tab find nearest neighbours \cr
    \code{\link[spatstat.geom]{pairdist}} \tab distances between all pairs of points\cr
    \code{\link[spatstat.geom]{crossdist}} \tab distances between points in two patterns\cr
    \code{\link[spatstat.geom]{nncross}} \tab nearest neighbours between two point patterns \cr
    \code{\link[spatstat.geom]{exactdt}} \tab distance from any location to nearest data point\cr 
    \code{\link[spatstat.geom]{distmap}} \tab distance map image\cr
    \code{\link[spatstat.geom]{distfun}} \tab distance map function\cr
    \code{\link[spatstat.geom]{nnmap}} \tab nearest point image \cr
    \code{\link[spatstat.geom]{nnfun}} \tab nearest point function \cr
    \code{\link[spatstat.explore]{density.ppp}} \tab kernel smoothed density\cr
    \code{\link[spatstat.explore]{densityAdaptiveKernel.ppp}} \tab variable
  bandwidth kernel smoothed intensity\cr
    \code{\link[spatstat.explore]{densityHeat.ppp}} \tab diffusion kernel smoothed density\cr
    \code{\link[spatstat.explore]{Smooth.ppp}} \tab spatial interpolation of marks  \cr
    \code{\link[spatstat.explore]{relrisk}} \tab kernel estimate of relative risk\cr
    \code{\link[spatstat.explore]{sharpen.ppp}} \tab data sharpening  \cr
    \code{\link[spatstat.random]{rknn}} \tab theoretical distribution of nearest
    neighbour distance
 }

  \bold{Summary statistics for a multitype point pattern:}
  A multitype point pattern is represented by an object \code{X}
  of class \code{"ppp"} such that \code{marks(X)} is a factor. 
  \tabular{ll}{
    \code{\link[spatstat.explore]{relrisk}} \tab kernel estimation of relative risk  \cr
    \code{\link[spatstat.explore]{tolcon}} \tab Tolerance contours for relative risk  \cr
    \code{\link[spatstat.explore]{scan.test}} \tab spatial scan test of elevated risk  \cr
    \code{\link[spatstat.explore]{Gcross},\link[spatstat.explore]{Gdot},\link[spatstat.explore]{Gmulti}} \tab
    multitype nearest neighbour distributions 
    \eqn{G_{ij}, G_{i\bullet}}{G[i,j], G[i.]} \cr
    \code{\link[spatstat.explore]{Kcross},\link[spatstat.explore]{Kdot}, \link[spatstat.explore]{Kmulti}} \tab
    multitype \eqn{K}-functions 
    \eqn{K_{ij}, K_{i\bullet}}{K[i,j], K[i.]} \cr
    \code{\link[spatstat.explore]{Lcross},\link[spatstat.explore]{Ldot}} \tab
    multitype \eqn{L}-functions 
    \eqn{L_{ij}, L_{i\bullet}}{L[i,j], L[i.]} \cr
    \code{\link[spatstat.explore]{Jcross},\link[spatstat.explore]{Jdot},\link[spatstat.explore]{Jmulti}} \tab
    multitype \eqn{J}-functions
    \eqn{J_{ij}, J_{i\bullet}}{J[i,j],J[i.]} \cr
    \code{\link[spatstat.explore]{pcfcross}} \tab
    multitype pair correlation function \eqn{g_{ij}}{g[i,j]} \cr
    \code{\link[spatstat.explore]{pcfdot}} \tab
    multitype pair correlation function \eqn{g_{i\bullet}}{g[i.]} \cr
    \code{\link[spatstat.explore]{pcfmulti}} \tab
    general pair correlation function \cr
    \code{\link[spatstat.explore]{markconnect}} \tab
    marked connection function \eqn{p_{ij}}{p[i,j]} \cr
    \code{\link[spatstat.explore]{markequal}} \tab mark equality function \cr
    \code{\link[spatstat.explore]{alltypes}} \tab  estimates of the above
    for all \eqn{i,j} pairs \cr
    \code{\link[spatstat.explore]{Iest}} \tab  multitype \eqn{I}-function\cr
    \code{\link[spatstat.explore]{Kcross.inhom},\link[spatstat.explore]{Kdot.inhom}} \tab
    inhomogeneous counterparts of \code{Kcross}, \code{Kdot} \cr
    \code{\link[spatstat.explore]{Lcross.inhom},\link[spatstat.explore]{Ldot.inhom}} \tab
    inhomogeneous counterparts of \code{Lcross}, \code{Ldot} \cr
    \code{\link[spatstat.explore]{pcfcross.inhom},\link[spatstat.explore]{pcfdot.inhom}} \tab
    inhomogeneous counterparts of \code{pcfcross}, \code{pcfdot} \cr
    \code{\link[spatstat.explore]{localKcross},\link[spatstat.explore]{localKdot}} \tab
    local counterparts of \code{Kcross}, \code{Kdot} \cr
    \code{\link[spatstat.explore]{localLcross},\link[spatstat.explore]{localLdot}} \tab
    local counterparts of \code{Lcross}, \code{Ldot} \cr
    \code{\link[spatstat.explore]{localKcross.inhom},\link[spatstat.explore]{localLcross.inhom}} \tab
    local counterparts of \code{Kcross.inhom}, \code{Lcross.inhom} 
  }

  \bold{Summary statistics for a marked point pattern:}
  A marked point pattern is represented by an object \code{X}
  of class \code{"ppp"} with a component \code{X$marks}.
  The entries in the vector \code{X$marks} may be numeric, complex,
  string or any other atomic type. For numeric marks, there are the
  following functions:
  \tabular{ll}{
    \code{\link[spatstat.explore]{markmean}} \tab smoothed local average of marks \cr
    \code{\link[spatstat.explore]{markvar}} \tab smoothed local variance of marks \cr
    \code{\link[spatstat.explore]{markcorr}} \tab mark correlation function \cr
    \code{\link[spatstat.explore]{markcrosscorr}} \tab mark cross-correlation function \cr
    \code{\link[spatstat.explore]{markvario}} \tab mark variogram \cr
    \code{\link[spatstat.explore]{markmarkscatter}} \tab mark-mark scatterplot \cr
    \code{\link[spatstat.explore]{Kmark}} \tab mark-weighted \eqn{K} function \cr
    \code{\link[spatstat.explore]{Emark}} \tab mark independence diagnostic \eqn{E(r)} \cr
    \code{\link[spatstat.explore]{Vmark}} \tab mark independence diagnostic \eqn{V(r)} \cr
    \code{\link[spatstat.explore]{nnmean}} \tab nearest neighbour mean index \cr
    \code{\link[spatstat.explore]{nnvario}} \tab nearest neighbour mark variance index \cr
    \code{\link[spatstat.explore]{markmarkscatter}} \tab mark-mark scatterplot
  }
  For marks of any type, there are the following:
  \tabular{ll}{
    \code{\link[spatstat.explore]{Gmulti}} \tab multitype nearest neighbour distribution \cr
    \code{\link[spatstat.explore]{Kmulti}} \tab multitype \eqn{K}-function \cr
    \code{\link[spatstat.explore]{Jmulti}} \tab multitype \eqn{J}-function 
  }
  Alternatively use \code{\link[spatstat.geom]{cut.ppp}} to convert a marked point pattern
  to a multitype point pattern.

  \bold{Programming tools:}
  \tabular{ll}{
    \code{\link[spatstat.geom]{applynbd}} \tab apply function to every neighbourhood
    in a point pattern \cr
    \code{\link[spatstat.geom]{markstat}} \tab apply function to the marks of neighbours
    in a point pattern \cr
    \code{\link[spatstat.explore]{marktable}} \tab tabulate the marks of neighbours
    in a point pattern \cr
    \code{\link[spatstat.geom]{pppdist}} \tab find the optimal match between two point
    patterns
  }

  \bold{Summary statistics for a point pattern on a linear network:}

  These are for point patterns on a linear network (class \code{lpp}).
  For unmarked patterns:
  
  \tabular{ll}{
    \code{\link[spatstat.linnet]{linearK}} \tab
    \eqn{K} function on linear network \cr
    \code{\link[spatstat.linnet]{linearKinhom}} \tab
    inhomogeneous \eqn{K} function on linear network \cr
    \code{\link[spatstat.linnet]{linearpcf}} \tab
    pair correlation function on linear network \cr
    \code{\link[spatstat.linnet]{linearpcfinhom}} \tab
    inhomogeneous pair correlation on linear network \cr
    \code{\link[spatstat.linnet]{quadratcount.lpp}} \tab quadrat counts  \cr
    \code{\link[spatstat.linnet]{quadrat.test.lpp}} \tab quadrat counting test \cr
    \code{\link[spatstat.linnet]{linearJinhom}} \tab
    inhomogeneous \eqn{J} function on linear network \cr
    \code{\link[spatstat.linnet]{linearKEuclid}} \tab
    \eqn{K} function using Euclidean distance \cr
    \code{\link[spatstat.linnet]{linearpcfEuclid}} \tab
    pair correlation function using Euclidean distance \cr
  }

  For multitype patterns:
  \tabular{ll}{
    \code{\link[spatstat.linnet]{linearKcross}} \tab
    \eqn{K} function between two types of points \cr
    \code{\link[spatstat.linnet]{linearKdot}} \tab
    \eqn{K} function from one type to any type \cr
    \code{\link[spatstat.linnet]{linearKcross.inhom}} \tab
    Inhomogeneous version of \code{\link[spatstat.linnet]{linearKcross}} \cr
    \code{\link[spatstat.linnet]{linearKdot.inhom}} \tab
    Inhomogeneous version of \code{\link[spatstat.linnet]{linearKdot}} \cr
    \code{\link[spatstat.linnet]{linearmarkconnect}} \tab
    Mark connection function  on linear network \cr
    \code{\link[spatstat.linnet]{linearmarkequal}} \tab
    Mark equality function on linear network \cr
    \code{\link[spatstat.linnet]{linearpcfcross}} \tab
    Pair correlation between two types of points \cr
    \code{\link[spatstat.linnet]{linearpcfdot}} \tab
    Pair correlation from one type to any type \cr
    \code{\link[spatstat.linnet]{linearpcfcross.inhom}} \tab
    Inhomogeneous version of \code{\link[spatstat.linnet]{linearpcfcross}} \cr
    \code{\link[spatstat.linnet]{linearpcfdot.inhom}} \tab
    Inhomogeneous version of \code{\link[spatstat.linnet]{linearpcfdot}} 
  }

  Related facilities:
  
  \tabular{ll}{
    \code{\link[spatstat.linnet]{pairdist.lpp}} \tab distances between pairs  \cr
    \code{\link[spatstat.linnet]{crossdist.lpp}} \tab distances between pairs \cr
    \code{\link[spatstat.linnet]{nndist.lpp}} \tab nearest neighbour distances  \cr
    \code{\link[spatstat.linnet]{nncross.lpp}} \tab nearest neighbour distances  \cr
    \code{\link[spatstat.linnet]{nnwhich.lpp}} \tab find nearest neighbours  \cr
    \code{\link[spatstat.linnet]{nnfun.lpp}} \tab find nearest data point  \cr
    \code{\link[spatstat.linnet]{density.lpp}} \tab kernel smoothing estimator of intensity  \cr
    \code{\link[spatstat.linnet]{densityQuick.lpp}} \tab 2D kernel estimate  \cr
    \code{\link[spatstat.linnet]{densityHeat.lpp}} \tab diffusion kernel estimate  \cr
    \code{\link[spatstat.linnet]{distfun.lpp}} \tab distance transform  \cr
    \code{\link[spatstat.linnet]{envelope.lpp}} \tab simulation envelopes  \cr
    \code{\link[spatstat.linnet]{rpoislpp}} \tab simulate Poisson points on linear network \cr
    \code{\link[spatstat.linnet]{runiflpp}} \tab simulate random points on a linear network 
  }
  
  It is also possible to fit point process models to \code{lpp} objects.
  See Section IV.
  
  \bold{Summary statistics for a three-dimensional point pattern:}

  These are for 3-dimensional point pattern objects (class \code{pp3}).

  \tabular{ll}{
    \code{\link[spatstat.explore]{F3est}} \tab empty space function \eqn{F} \cr
    \code{\link[spatstat.explore]{G3est}} \tab nearest neighbour function \eqn{G} \cr
    \code{\link[spatstat.explore]{K3est}} \tab \eqn{K}-function \cr
    \code{\link[spatstat.explore]{pcf3est}} \tab pair correlation function
  }

  Related facilities:
  \tabular{ll}{
    \code{\link[spatstat.explore]{envelope.pp3}} \tab simulation envelopes \cr
    \code{\link[spatstat.geom]{pairdist.pp3}} \tab distances between all pairs of
    points \cr
    \code{\link[spatstat.geom]{crossdist.pp3}} \tab distances between points in
    two patterns \cr
    \code{\link[spatstat.geom]{nndist.pp3}} \tab nearest neighbour distances \cr
    \code{\link[spatstat.geom]{nnwhich.pp3}} \tab find nearest neighbours \cr
    \code{\link[spatstat.geom]{nncross.pp3}} \tab find nearest neighbours in another pattern
  }

  \bold{Computations for multi-dimensional point pattern:}

  These are for multi-dimensional space-time
  point pattern objects (class \code{ppx}).

  \tabular{ll}{
    \code{\link[spatstat.geom]{pairdist.ppx}} \tab distances between all pairs of
    points \cr
    \code{\link[spatstat.geom]{crossdist.ppx}} \tab distances between points in
    two patterns \cr
    \code{\link[spatstat.geom]{nndist.ppx}} \tab nearest neighbour distances \cr
    \code{\link[spatstat.geom]{nnwhich.ppx}} \tab find nearest neighbours
  }

  \bold{Summary statistics for random sets:}
  
  These work for point patterns (class \code{ppp}),
  line segment patterns (class \code{psp})
  or windows (class \code{owin}).
  
  \tabular{ll}{
    \code{\link[spatstat.explore]{Hest}} \tab spherical contact distribution \eqn{H} \cr
    \code{\link[spatstat.explore]{Gfox}} \tab Foxall \eqn{G}-function \cr
    \code{\link[spatstat.explore]{Jfox}} \tab Foxall \eqn{J}-function
  }
  
}

\section{III. MODEL FITTING (COX AND CLUSTER MODELS)}{
  
  Cluster process models (with homogeneous or inhomogeneous intensity)
  and Cox processes can be fitted by the function \code{\link[spatstat.model]{kppm}}.
  Its result is an object of class \code{"kppm"}.
  The fitted model can be printed, plotted, predicted, simulated
  and updated.

  \tabular{ll}{
    \code{\link[spatstat.model]{kppm}} \tab  Fit model\cr
    \code{\link[spatstat.model]{plot.kppm}} \tab  Plot the fitted model\cr
    \code{\link[spatstat.model]{summary.kppm}} \tab  Summarise the fitted model\cr
    \code{\link[spatstat.model]{fitted.kppm}} \tab Compute fitted intensity \cr
    \code{\link[spatstat.model]{predict.kppm}} \tab Compute fitted intensity \cr
    \code{\link[spatstat.model]{update.kppm}} \tab Update the model \cr
    \code{\link[spatstat.model]{improve.kppm}} \tab Refine the estimate of trend \cr
    \code{\link[spatstat.model]{simulate.kppm}} \tab Generate simulated realisations \cr
  }

  For model selection, you can also use 
  the generic functions \code{\link[stats]{step}}, \code{\link[stats]{drop1}} 
  and \code{\link[stats]{AIC}} on fitted point process models.
  For variable selection, see \code{\link[spatstat.explore]{sdr}}.
  
  The theoretical models can also be simulated,
  for any choice of parameter values,
  using \code{\link[spatstat.random]{rThomas}}, \code{\link[spatstat.random]{rMatClust}},
  \code{\link[spatstat.random]{rCauchy}}, \code{\link[spatstat.random]{rVarGamma}},
  and \code{\link[spatstat.random]{rLGCP}}.
  
  \bold{Characteristics of a cluster process model:}

  \tabular{ll}{
    \code{\link[spatstat.model:methods.kppm]{coef.kppm}}
    \tab Extract trend coefficients \cr
    \code{\link[spatstat.model]{vcov.kppm}} \tab Variance-covariance matrix \cr
                                  \tab of trend coefficients \cr
    \code{\link[spatstat.model:methods.kppm]{formula.kppm}}
    \tab Extract trend formula \cr
    \code{\link[spatstat.model]{parameters}} \tab Extract all model parameters \cr
    \code{\link[spatstat.model]{clusterfield.kppm}} \tab Compute offspring density \cr
    \code{\link[spatstat.model]{clusterradius.kppm}} \tab Radius of support of offspring density \cr
    \code{\link[spatstat.model]{reach.kppm}} \tab Interaction distance \cr
    \code{\link[spatstat.model]{Kmodel.kppm}} \tab \eqn{K} function of fitted model \cr
    \code{\link[spatstat.model]{pcfmodel.kppm}} \tab Pair correlation of fitted model \cr
    \code{\link[spatstat.model]{psib}} \tab Sibling probability \cr
    \code{\link[spatstat.model]{panysib}} \tab Probability of any siblings \cr
    \code{\link[spatstat.model]{persist}} \tab Spatial persistence index \cr
    \code{\link[spatstat.model]{clusterstrength}} \tab Strength of clustering
  }
  
  Lower-level fitting functions include:

  \tabular{ll}{
    \code{\link[spatstat.model]{lgcp.estK}} \tab fit a log-Gaussian Cox process model\cr
    \code{\link[spatstat.model]{lgcp.estpcf}} \tab fit a log-Gaussian Cox process model\cr
    \code{\link[spatstat.model]{thomas.estK}} \tab fit the Thomas process model \cr
    \code{\link[spatstat.model]{thomas.estpcf}} \tab fit the Thomas process model \cr
    \code{\link[spatstat.model]{matclust.estK}} \tab fit the \Matern Cluster process model \cr
    \code{\link[spatstat.model]{matclust.estpcf}} \tab fit the \Matern Cluster process model \cr
    \code{\link[spatstat.model]{cauchy.estK}} \tab fit a Neyman-Scott Cauchy cluster process \cr
   \code{\link[spatstat.model]{cauchy.estpcf}} \tab fit a Neyman-Scott Cauchy cluster process\cr
   \code{\link[spatstat.model]{vargamma.estK}} \tab fit a Neyman-Scott Variance Gamma process\cr
   \code{\link[spatstat.model]{vargamma.estpcf}} \tab fit a Neyman-Scott Variance Gamma process\cr
    \code{\link[spatstat.model]{mincontrast}} \tab low-level algorithm for fitting models
    \cr \tab by the method of minimum contrast 
  }
}

\section{IV. MODEL FITTING (POISSON AND GIBBS MODELS)}{
  
  \bold{Types of models}
  
  Poisson point processes are the simplest models for point patterns.
  A Poisson model assumes that the points are stochastically
  independent. It may allow the points to have a non-uniform spatial
  density. The special case of a Poisson process with a uniform
  spatial density is often called Complete Spatial Randomness.
  
  Poisson point processes are included in the more general class of Gibbs point
  process models. In a Gibbs model, there is \emph{interaction}
  or dependence between points. Many different types of interaction
  can be specified.
  
  For a detailed explanation of how to fit Poisson or
  Gibbs point process models to point pattern data using \pkg{spatstat},
  see Baddeley and Turner (2005b) or Baddeley (2008).
  
  \bold{To fit a Poisson or Gibbs point process model:}

  Model fitting in \pkg{spatstat} is performed mainly by the function
  \code{\link[spatstat.model]{ppm}}. Its result is an object of class \code{"ppm"}.
  
  Here are some examples, where \code{X} is a point pattern (class
  \code{"ppp"}):
  
  \tabular{ll}{
    \emph{command} \tab \emph{model} \cr
    \code{ppm(X)} \tab Complete Spatial Randomness \cr
    \code{ppm(X ~ 1)} \tab Complete Spatial Randomness \cr
    \code{ppm(X ~ x)} \tab Poisson process with \cr
                             \tab intensity loglinear in \eqn{x} coordinate \cr
    \code{ppm(X ~ 1, Strauss(0.1))} \tab Stationary Strauss process \cr
    \code{ppm(X ~ x, Strauss(0.1))} \tab Strauss process with \cr
                              \tab conditional intensity loglinear in \eqn{x}
  }
  It is also possible to fit models that depend on
  other covariates.

  \bold{Manipulating the fitted model:}

  \tabular{ll}{
    \code{\link[spatstat.model]{plot.ppm}} \tab 		Plot the fitted model\cr
    \code{\link[spatstat.model]{predict.ppm}}
    \tab   Compute the spatial trend and conditional intensity\cr
    \tab   of the fitted point process model \cr
    \code{\link[spatstat.model]{coef.ppm}} \tab Extract the fitted model coefficients\cr
    \code{\link[spatstat.model]{parameters}} \tab Extract all model parameters\cr
    \code{\link[spatstat.model]{formula.ppm}} \tab Extract the trend formula\cr
    \code{\link[spatstat.model]{intensity.ppm}} \tab Compute fitted intensity \cr
    \code{\link[spatstat.model]{Kmodel.ppm}} \tab \eqn{K} function of fitted model \cr
    \code{\link[spatstat.model]{pcfmodel.ppm}} \tab pair correlation of fitted model \cr
    \code{\link[spatstat.model]{fitted.ppm}} \tab Compute fitted conditional intensity at quadrature points \cr
    \code{\link[spatstat.model]{residuals.ppm}} \tab Compute point process residuals at quadrature points \cr
    \code{\link[spatstat.model]{update.ppm}} \tab Update the fit \cr
    \code{\link[spatstat.model]{vcov.ppm}} \tab Variance-covariance matrix of estimates\cr
    \code{\link[spatstat.model]{rmh.ppm}} \tab Simulate from fitted model  \cr
    \code{\link[spatstat.model]{simulate.ppm}} \tab Simulate from fitted model  \cr
    \code{\link[spatstat.model]{print.ppm}} \tab Print basic information about a fitted model\cr
    \code{\link[spatstat.model]{summary.ppm}} \tab Summarise a fitted model\cr
    \code{\link[spatstat.model]{effectfun}} \tab Compute the fitted effect of one covariate\cr
    \code{\link[spatstat.model]{logLik.ppm}} \tab log-likelihood or log-pseudolikelihood\cr
    \code{\link[spatstat.model]{anova.ppm}} \tab Analysis of deviance \cr
    \code{\link[spatstat.model]{model.frame.ppm}} \tab Extract data frame used to fit model  \cr
    \code{\link[spatstat.model]{model.images}} \tab Extract spatial data used to fit model  \cr
    \code{\link[spatstat.model]{model.depends}} \tab Identify variables in the model \cr
    \code{\link[spatstat.model]{as.interact}} \tab Interpoint interaction component of model \cr
    \code{\link[spatstat.model]{fitin}} \tab Extract fitted interpoint interaction \cr
    \code{\link[spatstat.model]{is.hybrid}} \tab Determine whether the model is a hybrid \cr
    \code{\link[spatstat.model]{valid.ppm}} \tab Check the model is a valid point process \cr
    \code{\link[spatstat.model]{project.ppm}} \tab Ensure the model is a valid point process \cr
    \code{\link[spatstat.model]{hardcoredist}} \tab Extract hard core distance of model 
  }
  For model selection, you can also use 
  the generic functions \code{\link[stats]{step}}, \code{\link[stats]{drop1}} 
  and \code{\link[stats]{AIC}} on fitted point process models.
  For variable selection, see \code{\link[spatstat.explore]{sdr}}.
  
  See \code{\link[spatstat.geom]{spatstat.options}} to control plotting of fitted model.
  
  \bold{To specify a point process model:}
  
  The first order ``trend'' of the model is determined by an \R 
  language formula. The formula specifies the form of the
  \emph{logarithm} of the trend.
  
  \tabular{ll}{
    \code{X ~ 1}  \tab No trend (stationary) \cr
    \code{X ~ x}   \tab Loglinear trend
      \eqn{\lambda(x,y) = \exp(\alpha + \beta x)}{lambda(x,y) =	exp(alpha + beta * x)} \cr
    \tab where \eqn{x,y} are Cartesian coordinates \cr
    \code{X ~ polynom(x,y,3)}  \tab Log-cubic polynomial trend  \cr
    \code{X ~ harmonic(x,y,2)}  \tab Log-harmonic polynomial trend \cr
    \code{X ~ Z}   \tab Loglinear function of covariate \code{Z} \cr
    \tab  \eqn{\lambda(x,y) = \exp(\alpha + \beta Z(x,y))}{lambda(x,y) =	exp(alpha + beta * Z(x,y))} 
  }

  The higher order (``interaction'') components are described by
  an object of class \code{"interact"}. Such objects are created by:
  \tabular{ll}{
    \code{\link[spatstat.model]{Poisson}()} \tab		the Poisson point process\cr
    \code{\link[spatstat.model]{AreaInter}()}	 \tab Area-interaction process\cr
    \code{\link[spatstat.model]{BadGey}()} \tab	multiscale Geyer process\cr
    \code{\link[spatstat.model]{Concom}()} \tab	connected component interaction\cr
    \code{\link[spatstat.model]{DiggleGratton}() } \tab Diggle-Gratton potential \cr
    \code{\link[spatstat.model]{DiggleGatesStibbard}() } \tab Diggle-Gates-Stibbard potential \cr
    \code{\link[spatstat.model]{Fiksel}()}	 \tab Fiksel pairwise interaction process\cr
    \code{\link[spatstat.model]{Geyer}()}	 \tab Geyer's saturation process\cr
    \code{\link[spatstat.model]{Hardcore}()}	 \tab Hard core process\cr
    \code{\link[spatstat.model]{HierHard}()}	 \tab Hierarchical multiype hard core process\cr
    \code{\link[spatstat.model]{HierStrauss}()}	 \tab Hierarchical multiype Strauss process\cr
    \code{\link[spatstat.model]{HierStraussHard}()}	 \tab
    Hierarchical multiype Strauss-hard core process\cr
    \code{\link[spatstat.model]{Hybrid}()}	 \tab Hybrid of several interactions\cr
    \code{\link[spatstat.model]{LennardJones}() } \tab Lennard-Jones potential \cr
    \code{\link[spatstat.model]{MultiHard}()} \tab 		multitype hard core process \cr
    \code{\link[spatstat.model]{MultiStrauss}()} \tab 		multitype Strauss process \cr
    \code{\link[spatstat.model]{MultiStraussHard}()} \tab 	multitype Strauss/hard core process \cr
    \code{\link[spatstat.model]{OrdThresh}()} \tab		Ord process, threshold potential\cr
    \code{\link[spatstat.model]{Ord}()} \tab 		        Ord model, user-supplied potential \cr
    \code{\link[spatstat.model]{PairPiece}()} \tab		pairwise interaction, piecewise constant \cr
    \code{\link[spatstat.model]{Pairwise}()} \tab	pairwise interaction, user-supplied potential\cr
    \code{\link[spatstat.model]{Penttinen}()} \tab	Penttinen pairwise interaction\cr
    \code{\link[spatstat.model]{SatPiece}()} \tab	Saturated pair model, piecewise  constant potential\cr
    \code{\link[spatstat.model]{Saturated}()} \tab	Saturated pair model, user-supplied potential\cr
    \code{\link[spatstat.model]{Softcore}()} \tab pairwise interaction, soft core potential\cr
    \code{\link[spatstat.model]{Strauss}()} \tab  Strauss process \cr
    \code{\link[spatstat.model]{StraussHard}()} \tab Strauss/hard core point process \cr
    \code{\link[spatstat.model]{Triplets}()} \tab Geyer triplets process
  }
  Note that it is also possible to combine several such interactions
  using \code{\link[spatstat.model]{Hybrid}}.
  
  \bold{Finer control over model fitting:}
  
  A quadrature scheme is represented by an object of
  class \code{"quad"}. To create a quadrature scheme, typically
  use \code{\link[spatstat.geom]{quadscheme}}.
  
  \tabular{ll}{
    \code{\link[spatstat.geom]{quadscheme}} \tab default quadrature scheme \cr
                             \tab using rectangular cells or Dirichlet cells\cr
    \code{\link[spatstat.geom]{pixelquad}}  \tab quadrature scheme based on image pixels \cr
    \code{\link[spatstat.geom]{quad}}       \tab create an object of class \code{"quad"}
  }
  
  To inspect a quadrature scheme:
  \tabular{ll}{
    \code{plot(Q)} \tab plot quadrature scheme \code{Q}\cr
    \code{print(Q)} \tab print basic information about quadrature scheme \code{Q}\cr
    \code{\link[spatstat.geom:summary.quad]{summary}(Q)} \tab summary of quadrature scheme \code{Q}
  }

  A quadrature scheme consists of data points, dummy points, and
  weights. To generate dummy points:
  \tabular{ll}{
    \code{\link[spatstat.geom]{default.dummy}} \tab default pattern of dummy points \cr
    \code{\link[spatstat.geom]{gridcentres}} \tab dummy points in a rectangular grid \cr
    \code{\link[spatstat.random]{rstrat}} \tab stratified random dummy pattern \cr
    \code{\link[spatstat.geom]{spokes}} \tab radial pattern of dummy points  \cr
    \code{\link[spatstat.geom]{corners}} \tab dummy points at corners of the window
  }
  
  To compute weights:
  \tabular{ll}{
    \code{\link[spatstat.geom]{gridweights}} \tab quadrature weights by the grid-counting rule  \cr
    \code{\link[spatstat.geom]{dirichletWeights}} \tab quadrature weights are
    Dirichlet tile areas
  }

  \bold{Simulation and goodness-of-fit for fitted models:}
  
  \tabular{ll}{
    \code{\link[spatstat.model]{rmh.ppm}} \tab simulate realisations of a fitted model \cr
    \code{\link[spatstat.model]{simulate.ppm}} \tab simulate realisations of a fitted model \cr
    \code{\link[spatstat.explore]{envelope}} \tab compute simulation envelopes for a
    fitted model \cr
    \code{\link[spatstat.explore]{MISE.envelope}} \tab mean integrated squared error \cr
    \code{\link[spatstat.explore]{RMSE.envelope}} \tab pointwise root-mean-square error 
  }

  \bold{Point process models on a linear network:}

  An object of class \code{"lpp"} represents a pattern of points on
  a linear network. Point process models can also be fitted to these
  objects. Currently only Poisson models can be fitted.

  \tabular{ll}{
    \code{\link[spatstat.linnet]{lppm}} \tab point process model on linear network \cr
    \code{\link[spatstat.linnet]{anova.lppm}} \tab analysis of deviance for \cr
    \tab point process model on linear network \cr
    \code{\link[spatstat.linnet]{envelope.lppm}} \tab simulation envelopes for \cr
    \tab point process model on linear network \cr
    \code{\link[spatstat.linnet]{fitted.lppm}} \tab fitted intensity values \cr
    \code{\link[spatstat.linnet]{predict.lppm}} \tab model prediction on linear network \cr
    \code{\link[spatstat.linnet]{linim}} \tab pixel image on linear network \cr
    \code{\link[spatstat.linnet]{plot.linim}} \tab plot a pixel image on linear network \cr
    \code{\link[spatstat.linnet]{eval.linim}} \tab evaluate expression involving images \cr
    \code{\link[spatstat.linnet]{linfun}} \tab function defined on linear network \cr
    \code{\link[spatstat.linnet]{methods.linfun}} \tab conversion facilities
  }

  For model diagnostics on a linear network, see the section on Diagnostics.
}


\section{V. MODEL FITTING (DETERMINANTAL POINT PROCESS MODELS)}{

  Code for fitting \emph{determinantal point process models} (DPPs) has 
  recently been added to \pkg{spatstat}.

  \tabular{ll}{
    \code{\link[spatstat.model]{dppm}} \tab Fit a determinantal point process
    model to data \cr
    \code{\link[spatstat.model]{update.dppm}} \tab Update model \cr
    \code{\link[spatstat.model]{summary.dppm}} \tab Summarise fitted model \cr
    \code{\link[spatstat.model]{predict.dppm}} \tab Compute fitted intensity \cr
    \code{\link[spatstat.model]{fitted.dppm}} \tab Compute fitted intensity \cr
    \code{\link[spatstat.model]{Kmodel.dppm}} \tab \eqn{K} function of model \cr
    \code{\link[spatstat.model]{pcfmodel.dppm}} \tab Pair correlation function of
    model \cr
    \code{\link[spatstat.model]{simulate.dppm}} \tab Simulate the fitted model \cr
    \code{\link[spatstat.model]{rdpp}} \tab Simulate model \cr
    \code{\link[spatstat.model]{logLik.dppm}} \tab log composite likelihood \cr
    \code{\link[spatstat.model]{AIC.dppm}} \tab Akaike Information Criterion \cr
    \code{\link[spatstat.model]{coef.dppm}} \tab Extract fitted coefficients \cr
    \code{\link[spatstat.model]{parameters.dppm}} \tab Extract all model parameters \cr
    \code{\link[spatstat.model]{formula.dppm}} \tab Extract model formula \cr
    \code{\link[spatstat.model]{objsurf.dppm}} \tab Objective function surface \cr
    \code{\link[spatstat.model]{roc.dppm}} \tab ROC curve  \cr
    \code{\link[spatstat.model]{repul}} \tab Index of strength of repulsion
  }

  Interaction structures in a DPP are specified by the model families
  \code{\link[spatstat.model]{dppBessel}},
  \code{\link[spatstat.model]{dppCauchy}},
  \code{\link[spatstat.model]{dppGauss}},
  \code{\link[spatstat.model]{dppMatern}},
  \code{\link[spatstat.model]{dppPowerExp}}.
  
}

\section{VI. MODEL FITTING (SPATIAL LOGISTIC REGRESSION)}{
  
  \bold{Logistic regression}
  
  Pixel-based spatial logistic regression is an alternative
  technique for analysing spatial point patterns
  that is widely used in Geographical Information Systems.
  It is approximately equivalent to fitting a Poisson point process
  model.
  
  In pixel-based logistic regression, the spatial domain is
  divided into small pixels, the presence or absence of a
  data point in each pixel is recorded, and logistic regression
  is used to model the presence/absence indicators as a function
  of any covariates.
  
  Facilities for performing spatial logistic regression are
  provided in \pkg{spatstat} for comparison purposes.
  
  \bold{Fitting a spatial logistic regression}
  
  Spatial logistic regression is performed by the function
  \code{\link[spatstat.model]{slrm}}. Its result is an object of class \code{"slrm"}.
  There are many methods for this class, including methods for
  \code{print}, \code{fitted}, \code{predict}, \code{simulate},
  \code{anova}, \code{coef}, \code{logLik}, \code{terms},
  \code{update}, \code{formula} and \code{vcov}. 
  
  For example, if \code{X} is a point pattern (class
  \code{"ppp"}):
  
  \tabular{ll}{
    \emph{command} \tab \emph{model} \cr
    \code{slrm(X ~ 1)} \tab Complete Spatial Randomness \cr
    \code{slrm(X ~ x)} \tab Poisson process with \cr
                \tab intensity loglinear in \eqn{x} coordinate \cr
    \code{slrm(X ~ Z)} \tab Poisson process with \cr
                \tab intensity loglinear in covariate \code{Z}
  }

  \bold{Manipulating a fitted spatial logistic regression}
  
  \tabular{ll}{
    \code{\link[spatstat.model]{anova.slrm}} \tab Analysis of deviance \cr
    \code{\link[spatstat.model]{coef.slrm}}  \tab Extract fitted coefficients \cr
    \code{\link[spatstat.model]{vcov.slrm}}  \tab Variance-covariance matrix of fitted coefficients \cr
    \code{\link[spatstat.model]{fitted.slrm}} \tab Compute fitted probabilities or
    intensity \cr
    \code{\link[spatstat.model]{logLik.slrm}}   \tab Evaluate loglikelihood of fitted
    model \cr
    \code{\link[spatstat.model]{plot.slrm}}    \tab Plot fitted probabilities or
    intensity \cr
    \code{\link[spatstat.model]{predict.slrm}} \tab Compute predicted probabilities or
    intensity with new data \cr
    \code{\link[spatstat.model]{simulate.slrm}} \tab Simulate model
  }
  
  There are many other undocumented methods for this class,
  including methods for \code{print}, \code{update}, \code{formula}
  and \code{terms}. Stepwise model selection is
  possible using \code{step} or \code{stepAIC}.
  For variable selection, see \code{\link[spatstat.explore]{sdr}}.
}

\section{VII. MODEL FITTING (RECURSIVE PARTITION MODELS)}{

  A recursively partitioned point process model, depending on
  a list of spatial covariates, is a Poisson point process model
  in which the intensity function is determined by a regression tree
  applied to the covariates.

  \tabular{ll}{
    \code{\link[spatstat.model]{rppm}} \tab Fit a recursively partitioned model \cr
    \code{\link[spatstat.model]{predict.rppm}} \tab Compute the fitted intensity \cr
    \code{\link[spatstat.model]{plot.rppm}} \tab Compute the regression tree \cr
    \code{\link[spatstat.model]{prune.rppm}} \tab Prune the regression tree \cr
    \code{\link[spatstat.model]{as.tess.rppm}} \tab Interpret the model as a
    subdivision of space \cr
    \code{\link[spatstat.model]{formula.rppm}} \tab Extract the model formula \cr
    \code{\link[spatstat.model]{update.rppm}} \tab Update the model \cr
    \code{\link[spatstat.model]{residuals.rppm}} \tab residual measure \cr
    \code{\link[spatstat.model]{effectfun}} \tab intensity as a function of covariate \cr
  }
}

\section{VIII. SIMULATION}{

  There are many ways to generate a random point pattern,
  line segment pattern, pixel image or tessellation
  in \pkg{spatstat}. 

  \bold{Random point patterns:}

  \tabular{ll}{
    \code{\link[spatstat.random]{runifpoint}} \tab
    generate \eqn{n} independent uniform random points \cr
    \code{\link[spatstat.random]{rpoint}} \tab
    generate \eqn{n} independent random points \cr
    \code{\link[spatstat.random]{rmpoint}} \tab
    generate \eqn{n} independent multitype random points \cr
    \code{\link[spatstat.random]{rpoispp}} \tab
    simulate the (in)homogeneous Poisson point process \cr
    \code{\link[spatstat.random]{rmpoispp}} \tab
    simulate the (in)homogeneous multitype Poisson point process \cr
    \code{\link[spatstat.random]{runifdisc}} \tab
    generate \eqn{n} independent uniform random points in disc\cr
    \code{\link[spatstat.random]{rstrat}} \tab
    stratified random sample of points \cr
    \code{\link[spatstat.geom]{rsyst}} \tab
    systematic random sample (grid) of points \cr
    \code{\link[spatstat.random]{rMaternI}}  \tab
    simulate the \Matern Model I inhibition process\cr
    \code{\link[spatstat.random]{rMaternII}} \tab
    simulate the \Matern Model II inhibition process\cr
    \code{\link[spatstat.random]{rSSI}} \tab
    simulate Simple Sequential Inhibition process\cr
    \code{\link[spatstat.random]{rHardcore}} \tab
    simulate hard core process (perfect simulation)\cr
    \code{\link[spatstat.random]{rStrauss}} \tab
    simulate Strauss process (perfect simulation)\cr
    \code{\link[spatstat.random]{rStraussHard}} \tab
    simulate Strauss-hard core process (perfect simulation)\cr
    \code{\link[spatstat.random]{rDiggleGratton}} \tab
    simulate Diggle-Gratton process (perfect simulation)\cr
    \code{\link[spatstat.random]{rDGS}} \tab
    simulate Diggle-Gates-Stibbard process (perfect simulation)\cr
    \code{\link[spatstat.random]{rPenttinen}} \tab
    simulate Penttinen process (perfect simulation)\cr
    \code{\link[spatstat.random]{rNeymanScott}} \tab
    simulate a general Neyman-Scott process\cr
    \code{\link[spatstat.random]{rMatClust}} \tab
    simulate the \Matern Cluster process\cr
    \code{\link[spatstat.random]{rThomas}} \tab
    simulate the Thomas process  \cr
    \code{\link[spatstat.random]{rLGCP}} \tab
    simulate the log-Gaussian Cox process  \cr
    \code{\link[spatstat.random]{rGaussPoisson}}  \tab
    simulate the Gauss-Poisson cluster process\cr
    \code{\link[spatstat.random]{rCauchy}} \tab
    simulate Neyman-Scott process with Cauchy clusters \cr
    \code{\link[spatstat.random]{rVarGamma}} \tab
    simulate Neyman-Scott process with Variance Gamma clusters \cr
    \code{\link[spatstat.random]{rcell}} \tab
    simulate the Baddeley-Silverman cell process  \cr
    \code{\link[spatstat.random]{runifpointOnLines}} \tab
    generate \eqn{n} random points along specified line segments \cr
    \code{\link[spatstat.random]{rpoisppOnLines}} \tab
    generate Poisson random points along specified line segments 
  }
      
  \bold{Resampling a point pattern:}

  \tabular{ll}{
    \code{\link[spatstat.random]{quadratresample}} \tab block resampling \cr
    \code{\link[spatstat.geom]{rjitter}} \tab
    apply random displacements to points in a pattern\cr
    \code{\link[spatstat.random]{rshift}} \tab random shifting of (subsets of) points\cr
    \code{\link[spatstat.random]{rthin}} \tab  random thinning 
  }
  
  See also \code{\link[spatstat.explore]{varblock}} for estimating the variance
  of a summary statistic by block resampling, and
  \code{\link[spatstat.explore]{lohboot}} for another bootstrap technique.
  
  \bold{Fitted point process models:}

   If you have fitted a point process model to a point pattern dataset,
   the fitted model can be simulated.

   Cluster process models 
   are fitted by the function \code{\link[spatstat.model]{kppm}} yielding an
   object of class \code{"kppm"}. To generate one or more simulated
   realisations of this fitted model, use 
   \code{\link[spatstat.model]{simulate.kppm}}.

   Gibbs point process models 
   are fitted by the function \code{\link[spatstat.model]{ppm}} yielding an
   object of class \code{"ppm"}. To generate a simulated
   realisation of this fitted model, use \code{\link[spatstat.random]{rmh}}.
   To generate one or more simulated realisations of the fitted model,
   use \code{\link[spatstat.model]{simulate.ppm}}.

   \bold{Other random patterns:}

   \tabular{ll}{
     \code{\link[spatstat.geom]{rlinegrid}} \tab
     generate a random array of parallel lines through a window \cr
     \code{\link[spatstat.random]{rpoisline}} \tab
     simulate the Poisson line process within a window \cr
     \code{\link[spatstat.random]{rpoislinetess}} \tab
     generate random tessellation using Poisson line process \cr
     \code{\link[spatstat.random]{rMosaicSet}} \tab
     generate random set by selecting some tiles of a tessellation \cr
     \code{\link[spatstat.random]{rMosaicField}} \tab
     generate random pixel image by assigning random values
     in each tile of a tessellation
   }

   \bold{Simulation-based inference}

   \tabular{ll}{
    \code{\link[spatstat.explore]{envelope}} \tab critical envelope for Monte Carlo
    test of goodness-of-fit \cr
    \code{\link[spatstat.explore]{bits.envelope}} \tab critical envelope for balanced
    two-stage Monte Carlo test \cr
    \code{\link[spatstat.model]{qqplot.ppm}} \tab diagnostic plot for interpoint
    interaction \cr
    \code{\link[spatstat.explore]{scan.test}} \tab spatial scan statistic/test \cr
    \code{\link[spatstat.explore]{studpermu.test}} \tab studentised permutation test\cr
    \code{\link[spatstat.explore]{segregation.test}} \tab test of segregation of types 
  }
}


\section{IX. TESTS AND DIAGNOSTICS}{

  \bold{Hypothesis tests:}

  \tabular{ll}{
    \code{\link[spatstat.explore]{quadrat.test}} \tab \eqn{\chi^2}{chi^2} goodness-of-fit
    test on quadrat counts \cr
    \code{\link[spatstat.explore]{clarkevans.test}} \tab Clark and Evans test \cr
    \code{\link[spatstat.explore]{cdf.test}} \tab Spatial distribution goodness-of-fit test\cr
    \code{\link[spatstat.explore]{berman.test}} \tab Berman's goodness-of-fit tests\cr
    \code{\link[spatstat.explore]{envelope}} \tab critical envelope for Monte Carlo
    test of goodness-of-fit \cr
    \code{\link[spatstat.explore]{scan.test}} \tab spatial scan statistic/test \cr
    \code{\link[spatstat.explore]{dclf.test}} \tab Diggle-Cressie-Loosmore-Ford test \cr
    \code{\link[spatstat.explore]{mad.test}} \tab Mean Absolute Deviation test \cr
    \code{\link[spatstat.model]{anova.ppm}} \tab Analysis of Deviance for
    point process models 
  }

  More recently-developed tests and plots:
  
  \tabular{ll}{
    \code{\link[spatstat.explore]{dg.test}} \tab Dao-Genton test \cr
    \code{\link[spatstat.explore]{bits.test}} \tab Balanced independent two-stage test \cr
    \code{\link[spatstat.explore]{dclf.progress}} \tab Progress plot for DCLF test \cr
    \code{\link[spatstat.explore]{mad.progress}} \tab Progress plot for MAD test \cr
  }

  \bold{Sensitivity diagnostics:}

  Classical measures of model sensitivity such as leverage and influence
  have been adapted to point process models.
  
    \tabular{ll}{
      \code{\link[spatstat.model]{leverage.ppm}} \tab Leverage for point process model\cr
      \code{\link[spatstat.model]{influence.ppm}} \tab Influence for point process model\cr
      \code{\link[spatstat.model]{dfbetas.ppm}} \tab Parameter influence\cr
      \code{\link[spatstat.model]{dffit.ppm}} \tab Effect change diagnostic
    }
  
  \bold{Diagnostics for covariate effect:}

  Classical diagnostics for covariate effects have been adapted to
  point process models.

  \tabular{ll}{
    \code{\link[spatstat.model]{parres}} \tab Partial residual plot\cr
    \code{\link[spatstat.model]{addvar}} \tab Added variable plot \cr
    \code{\link[spatstat.explore]{rhohat}} \tab Kernel estimate of covariate effect\cr
    \code{\link[spatstat.explore]{rho2hat}} \tab Kernel estimate of covariate effect
    (bivariate) \cr
    \code{\link[spatstat.model]{addROC}} \tab Partial ROC curve for adding a
    covariate \cr
    \code{\link[spatstat.model]{dropROC}} \tab Partial ROC curve for removing a
    covariate \cr
    \code{\link[spatstat.model]{dropply}} \tab Consider all single-variable
    deletions from a model 
  }
  
  \bold{Residual diagnostics:}
  
  Residuals for a fitted point process model, and diagnostic plots
  based on the residuals, were introduced in Baddeley et al (2005) and
  Baddeley, Rubak and \Moller (2011).
  
  Type \code{demo(diagnose)}
  for a demonstration of the diagnostics features.

  \tabular{ll}{
    \code{\link[spatstat.model]{residuals.ppm}} \tab Residual measure \cr
    \code{\link[spatstat.model]{diagnose.ppm}} \tab diagnostic plots for spatial trend\cr
    \code{\link[spatstat.model]{qqplot.ppm}} \tab diagnostic Q-Q plot for interpoint interaction\cr
    \code{\link[spatstat.model]{lurking}} \tab Lurking variable plot \cr
    \code{\link[spatstat.data]{residualspaper}} \tab examples from Baddeley et al (2005) \cr
    \code{\link[spatstat.model]{eem.ppm}} \tab Stoyan-Grabarnik diagnostic \cr
    \code{\link[spatstat.model]{Kcom}} \tab model compensator of \eqn{K} function \cr
    \code{\link[spatstat.model]{Gcom}} \tab model compensator of \eqn{G} function \cr
    \code{\link[spatstat.model]{Kres}} \tab score residual of \eqn{K} function \cr
    \code{\link[spatstat.model]{Gres}} \tab score residual of \eqn{G} function \cr
    \code{\link[spatstat.model]{psst}} \tab pseudoscore residual of summary function \cr
    \code{\link[spatstat.model]{psstA}} \tab pseudoscore residual of empty space function \cr
    \code{\link[spatstat.model]{psstG}} \tab pseudoscore residual of \eqn{G} function \cr
    \code{\link[spatstat.model]{compareFit}} \tab compare compensators of several fitted
    models
  }

  \bold{Diagnostics for cluster processes:}

  \tabular{ll}{
    \code{\link[spatstat.model]{palmdiagnose}} \tab Palm diagnostic \cr
    \code{\link[spatstat.model]{as.fv.kppm}} \tab Compare empirical and fitted
    summary functions
  }

  \bold{Residual diagnostics on a linear network:}

  An object of class \code{"lppm"} represents a fitted point process model
  on a linear network.
  
  \tabular{ll}{
    \code{\link[spatstat.linnet]{diagnose.lppm}} \tab diagnostic plots \cr
    \code{\link[spatstat.linnet]{lurking.lppm}} \tab lurking variable plot \cr
    \code{\link[spatstat.linnet]{residuals.lppm}} \tab residual measure \cr
  }

  \bold{Resampling and randomisation procedures}

  You can build your own tests based on randomisation
  and resampling using the following capabilities:
  
  \tabular{ll}{
    \code{\link[spatstat.random]{quadratresample}} \tab block resampling \cr
    \code{\link[spatstat.geom]{rjitter}} \tab
    apply random displacements to points in a pattern\cr
    \code{\link[spatstat.random]{rshift}} \tab random shifting of (subsets of) points\cr
    \code{\link[spatstat.random]{rthin}} \tab  random thinning  
  }
}


\section{X. DOCUMENTATION}{
  The online manual entries are quite detailed and should be consulted
  first for information about a particular function.
  
  The book
  Baddeley, Rubak and Turner (2015) is a complete course
  on analysing spatial point patterns, with full details about
  \pkg{spatstat}.

  Older material (which is now out-of-date but is freely available)
  includes Baddeley and Turner (2005a), a brief overview of
  the package in its early development;
  Baddeley and Turner (2005b), a more detailed explanation of
  how to fit point process models to data; and 
  Baddeley (2010), a complete set of notes from a 2-day workshop
  on the use of \pkg{spatstat}. 

  Type \code{citation("spatstat")} to get a list of these references.
}
\references{
  Baddeley, A. (2010)
  \emph{Analysing spatial point patterns in R}.
  Workshop notes, Version 4.1.
  Online technical publication, CSIRO.
  \url{https://research.csiro.au/software/wp-content/uploads/sites/6/2015/02/Rspatialcourse_CMIS_PDF-Standard.pdf}

  Baddeley, A., Rubak, E. and Turner, R. (2015)
  \emph{Spatial Point Patterns: Methodology and Applications with R}.
  Chapman and Hall/CRC Press.
  
  Baddeley, A. and Turner, R. (2005a)
  Spatstat: an R package for analyzing spatial point patterns.
  \emph{Journal of Statistical Software} \bold{12}:6, 1--42.
  \code{DOI: 10.18637/jss.v012.i06}. 

  Baddeley, A. and Turner, R. (2005b)
  Modelling spatial point patterns in R.
  In: A. Baddeley, P. Gregori, J. Mateu, R. Stoica, and D. Stoyan,
  editors, \emph{Case Studies in Spatial Point Pattern Modelling},
  Lecture Notes in Statistics number 185. Pages 23--74.
  Springer-Verlag, New York, 2006. 
  ISBN: 0-387-28311-0.

  Baddeley, A., Turner, R.,
  \Moller, J. and Hazelton, M. (2005)
  Residual analysis for spatial point processes.
  \emph{Journal of the Royal Statistical Society, Series B}
  \bold{67}, 617--666.

  Baddeley, A., Rubak, E. and \Moller, J. (2011)
  Score, pseudo-score and residual
  diagnostics for spatial point process models.
  \emph{Statistical Science} \bold{26}, 613--646.

  Baddeley, A., Turner, R., Mateu, J. and Bevan, A. (2013)
  Hybrids of Gibbs point process models and their implementation.
  \emph{Journal of Statistical Software} \bold{55}:11, 1--43.
  \url{https://www.jstatsoft.org/v55/i11/}

  Diggle, P.J. (2003)
  \emph{Statistical analysis of spatial point patterns},
  Second edition. Arnold.

  Diggle, P.J. (2014)
  \emph{Statistical Analysis of Spatial and Spatio-Temporal Point Patterns},
  Third edition. {Chapman and Hall/CRC}.

  Gelfand, A.E., Diggle, P.J., Fuentes, M. and Guttorp, P., editors (2010)
  \emph{Handbook of Spatial Statistics}.
  CRC Press.

  Huang, F. and Ogata, Y. (1999)
  Improvements of the maximum pseudo-likelihood
  estimators in various spatial statistical models.
  \emph{Journal of Computational and Graphical Statistics}
  \bold{8}, 510--530.

  Illian, J., Penttinen, A., Stoyan, H. and Stoyan, D. (2008)
  \emph{Statistical Analysis and Modelling of Spatial Point Patterns.}
  Wiley.
  
  Waagepetersen, R.
  An estimating function approach to inference for
  inhomogeneous Neyman-Scott processes.
  \emph{Biometrics} \bold{63} (2007) 252--258.
}
\section{Licence}{
  This library and its documentation are usable under the terms of the "GNU 
  General Public License", a copy of which is distributed with the package.
}
\author{
  \spatstatAuthors.
}
\section{Acknowledgements}{
  Kasper Klitgaard Berthelsen,
  Ottmar Cronie,
  Tilman Davies,
  Yongtao Guan,
  Ute Hahn,
  Abdollah Jalilian,
  Marie-Colette van Lieshout,
  Greg McSwiggan,
  Tuomas Rajala,
  Suman Rakshit,
  Dominic Schuhmacher,
  Rasmus Waagepetersen
  and
  Hangsheng Wang
  made substantial contributions of code.

  Additional contributions and suggestions from
  Mohomed Abraj,
  Monsuru Adepeju,
  Corey Anderson,
  Ang Qi Wei,
  Ryan Arellano,
  Jens \ifelse{latex}{\out{{\AA}str{\" o}m}}{Astrom},
  Robert Aue,
  Marcel Austenfeld,
  Sandro Azaele,
  Guy Bayegnak,
  Colin Beale,
  Melanie Bell,
  Thomas Bendtsen,
  Ricardo Bernhardt,
  Andrew Bevan,
  Brad Biggerstaff,
  Anders Bilgrau,
  Leanne Bischof,
  Christophe Biscio,
  Roger Bivand,
  Jose M. Blanco Moreno,
  Florent Bonneu,
  Jordan Brown,
  Ian Buller,
  Julian Burgos,
  Simon Byers,
  Ya-Mei Chang,
  Jianbao Chen,
  Igor Chernayavsky,
  Y.C. Chin,
  Bjarke Christensen,
  \Lucia Cobo \Sanchez,
  Jean-\Francois Coeurjolly,
  Kim Colyvas,
  Hadrien Commenges,
  Rochelle Constantine,
  Robin Corria Ainslie,
  Richard Cotton,
  Marcelino de la Cruz,
  Peter Dalgaard,
  Mario D'Antuono,
  Sourav Das,
  Peter Diggle,
  Patrick Donnelly,
  Ian Dryden,
  Murray Efford, 
  Stephen Eglen,
  Ahmed El-Gabbas,
  Belarmain Fandohan,
  Olivier Flores,
  David Ford,
  Peter Forbes,
  Shane Frank,
  Janet Franklin,
  Funwi-Gabga Neba,
  Oscar Garcia,
  Agnes Gault,
  Jonas Geldmann,
  Marc Genton,
  Shaaban Ghalandarayeshi,
  Julian Gilbey,
  Jason Goldstick,
  Pavel Grabarnik,
  C. Graf,
  Ute Hahn,
  Andrew Hardegen,
  Martin \Bogsted Hansen,
  Martin Hazelton,
  Juha Heikkinen,
  Mandy Hering,
  Markus Herrmann,
  Maximilian Hesselbarth,
  Paul Hewson,
  Hamidreza Heydarian,
  Kassel Hingee,
  Stephanie Hogg,
  Kurt Hornik,
  Philipp Hunziker,
  Jack Hywood,
  Ross Ihaka,
  \ifelse{latex}{\out{\u{C}enk I\c{c}\"{o}s}}{Cenk Icos},
  Aruna Jammalamadaka,
  Jakob Jentschke,
  Robert John-Chandran,
  Devin Johnson,
  Mahdieh Khanmohammadi,
  Bob Klaver,
  Lily Kozmian-Ledward,
  Peter Kovesi,
  Mike Kuhn,
  Jeff Laake,
  Robert Lamb,
  \Frederic Lavancier,
  Tom Lawrence,
  Tomas Lazauskas,
  Jonathan Lee,
  George Leser,
  Angela Li,
  Li Haitao,
  George Limitsios,
  Andrew Lister,
  Nestor Luambua,
  Bethany Macdonald,
  Ben Madin,
  Martin Maechler,
  Daniel Manrique-\ifelse{latex}{\out{Casta{\~n}o}}{Castano},
  Kiran Marchikanti,
  Jeff Marcus,
  Robert Mark,
  Peter McCullagh,
  Monia Mahling,
  Jorge Mateu Mahiques,
  Ulf Mehlig,
  Frederico Mestre,
  Sebastian Wastl Meyer,
  Mi Xiangcheng,
  Lore De Middeleer,
  Robin Milne,
  Enrique Miranda,
  Jesper \Moller,
  Annie \ifelse{latex}{\out{Molli{\'e}}}{Mollie},
  Ines Moncada,
  Mehdi Moradi,
  Virginia Morera Pujol,
  Erika Mudrak,
  Gopalan Nair,
  Nader Najari,
  Nicoletta Nava,
  Linda Stougaard Nielsen,
  Felipe Nunes,
  Jens Randel Nyengaard,
  Jens \Oehlschlaegel,
  Thierry Onkelinx,
  Sean O'Riordan,
  Evgeni Parilov,
  Jeff Picka,
  Nicolas Picard,
  Tim Pollington,
  Mike Porter,
  Sergiy Protsiv,
  Adrian Raftery,
  Suman Rakshit,
  Ben Ramage,
  Pablo Ramon,
  Xavier Raynaud,
  Nicholas Read,
  Matt Reiter,
  Ian Renner,
  Tom Richardson,
  Brian Ripley,
  Yonatan Rosen,
  Ted Rosenbaum,
  Barry Rowlingson,
  Jason Rudokas,
  Tyler Rudolph,
  John Rudge,
  Christopher Ryan,
  Farzaneh Safavimanesh,
  Aila \Sarkka,
  Cody Schank,
  Katja Schladitz,
  Sebastian Schutte,
  Bryan Scott,
  Olivia Semboli,
  \Francois \ifelse{latex}{\out{S\'{e}m\'{e}curbe}}{Semecurbe},
  Alexey Sergushichev,
  Vadim Shcherbakov,
  Shen Guochun,
  Shi Peijian,
  Harold-Jeffrey Ship,
  Tammy L Silva,
  Ida-Maria Sintorn,
  Yong Song, 
  Malte Spiess,
  Mark Stevenson,
  Kaspar Stucki,
  Jan Sulavik,
  Michael Sumner,
  P. Surovy,
  Ben Taylor,
  Thordis Linda Thorarinsdottir,
  Leigh Torres,
  Berwin Turlach,
  Torben Tvedebrink,
  Kevin Ummer,
  Medha Uppala,
  Malissa Usher,
  Andrew van Burgel,
  Tobias Verbeke,
  Mikko Vihtakari,
  Alexendre Villers,
  Fabrice Vinatier,
  Maximilian Vogtland,
  Sasha Voss,
  Sven Wagner,
  Hao Wang,
  H. Wendrock,
  Jan Wild,
  Carl G. Witthoft,
  Selene Wong,
  Maxime Woringer,
  Luke Yates,
  Mike Zamboni,
  Achim Zeileis
  and
  Tingting Zhan.
}
\keyword{spatial}
\keyword{package}

\name{spatstat.family}
\alias{spatstat.family}
\title{Names of All Packages in the Spatstat Family}
\description{
  Provides the names of all the packages belonging to the \pkg{spatstat}
  family of packages.
}
\usage{
  spatstat.family(subpackages=TRUE, extensions=FALSE)
}
\arguments{
  \item{subpackages}{
    Logical value specifying whether to include sub-packages.
  }
  \item{extensions}{
    Logical value specifying whether to include extension packages.
  }
}
\value{
  Character vector of package names.
}
\details{
  This function returns a character vector containing the names
  of the packages that belong to the \pkg{spatstat} family.

  By default, only the sub-packages are listed, and not the
  extension packages.
  
  A \dQuote{sub-package} is a package which is implicitly loaded or imported
  when the command \code{library(spatstat)} is issued. Currently the
  sub-packages are:
  \itemize{
    \item \code{spatstat.utils}
    \item \code{spatstat.data}
    \item \code{spatstat.univar}
    \item \code{spatstat.sparse}
    \item \code{spatstat.geom}
    \item \code{spatstat.random}
    \item \code{spatstat.explore}
    \item \code{spatstat.model}
    \item \code{spatstat.linnet}
    \item \code{spatstat}
  }
  An \dQuote{extension package} is a package which must be loaded
  explicitly. The extension packages are:
  \itemize{
    \item \code{spatstat.gui}
    \item \code{spatstat.local}
    \item \code{spatstat.Knet}
  }
}
\seealso{
  \code{\link{latest.news}}
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}

\name{yardstick}
\alias{textstring}
\alias{onearrow}
\alias{yardstick}
\title{
  Text, Arrow or Scale Bar in a Diagram
}
\description{
  Create spatial objects that represent a text string,
  an arrow, or a yardstick (scale bar).
}
\usage{
textstring(x, y, txt = NULL, \dots)

onearrow(x0, y0, x1, y1, txt = NULL, \dots)

yardstick(x0, y0, x1, y1, txt = NULL, \dots)
}
\arguments{
  \item{x,y}{
    Coordinates where the text should be placed.
  }
  \item{x0,y0,x1,y1}{
    Spatial coordinates of both ends of the arrow or yardstick.
    Alternatively \code{x0} can be a point pattern (class \code{"ppp"})
    containing exactly two points, or a line segment pattern
    (class \code{"psp"}) consisting of exactly one line segment.
  }
  \item{txt}{
    The text to be displayed beside the line segment.
    Either a character string or an expression.
  }
  \item{\dots}{
    Additional named arguments for plotting the object.
  }
}
\details{
  These commands create objects that represent components of a
  diagram:
  \itemize{
    \item \code{textstring} creates an object that represents a string
    of text at a particular spatial location.
    \item \code{onearrow} creates an object that represents an arrow
    between two locations.
    \item \code{yardstick} creates an object that represents
    a scale bar: a line segment indicating the scale of the plot.
  }
  To display the relevant object, it should be plotted, using
  \code{plot}. See the help files for the plot methods
  \code{\link{plot.textstring}}, \code{\link{plot.onearrow}}
  and \code{\link{plot.yardstick}}.

  These objects are designed to be included as components in a
  \code{\link{layered}} object or a \code{\link{solist}}. This makes it
  possible to build up a diagram consisting of many spatial objects,
  and to annotate the diagram with arrows, text and so on, so that
  ultimately the entire diagram is plotted using \code{plot}.
}
\value{
  An object of class \code{"diagramobj"} which also
  belongs to one of the special classes \code{"textstring"}, \code{"onearrow"}
  or \code{"yardstick"}. There are methods for \code{plot},
  \code{print}, \code{"["} and \code{\link{shift}}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{plot.textstring}},
  \code{\link{plot.onearrow}},
  \code{\link{plot.yardstick}}.
}
\examples{
  X <- rescale(swedishpines)
  plot(X, pch=16, main="")
  yd <- yardstick(0,0,1,1, "diagonal")
  yy <- yardstick(X[1:2])
  ys <- yardstick(as.psp(list(xmid=4, ymid=0.5, length=1, angle=0),
                         window=Window(X)),
                  txt="1 m")
  ys
  plot(ys, angle=90)
  scalardilate(ys, 2)
}
\keyword{spatial}
\keyword{hplot}

\name{expand.owin}
\alias{expand.owin}
\title{Apply Expansion Rule}
\description{
  Applies an expansion rule to a window.
}
\usage{
 expand.owin(W, \dots)
}
\arguments{
  \item{W}{A window.}
  \item{\dots}{
    Arguments passed to \code{\link{rmhexpand}} to
    determine an expansion rule.
  }
}
\value{
  A window (object of class \code{"owin"}).
}
\details{
  The argument \code{W} should be a window (an object of class
  \code{"owin"}).

  This command applies the expansion rule specified by the
  arguments \code{\dots} to the window \code{W}, yielding another
  window.

  The arguments \code{\dots} are passed to \code{\link{rmhexpand}}
  to determine the expansion rule.

  For other transformations of the scale, location and orientation
  of a window, see \code{\link{shift}}, \code{\link{affine}}
  and \code{\link{rotate}}.
}
\seealso{
  \code{\link{rmhexpand}} about expansion rules.
  
  \code{\link{shift}},
  \code{\link{rotate}},
  \code{\link{affine}} for other types of manipulation.
}
\examples{
   expand.owin(square(1), 9)
   expand.owin(square(1), distance=0.5)
   expand.owin(letterR, length=2)
   expand.owin(letterR, distance=0.1)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{latest.changes}
\alias{latest.changes}
\title{
  List Recent Significant Changes to a Function
}
\description{
  List all the changes to a particular function in a package,
  starting from a certain date or version of the package.
  The default is to list changes in the latest
  version of the \pkg{spatstat} family of packages.  
}
\usage{
  latest.changes(x,
          sinceversion = NULL, sincedate = NULL,
          package = spatstat.family(), show = TRUE)
}
\arguments{
  \item{x}{
    Character string giving the name
    of the function of interest, or a search pattern to be matched.
    A character vector is permitted.
  }
  \item{sinceversion}{
    Earliest version of \code{package}
    for which changes should be listed.
    A character string.
    Default is the most recent version.
  }
  \item{sincedate}{
    Earliest release date of \code{package}
    for which changes should be listed.
    A character string or a date-time object.
    Default is the date of the most recent version.
  }
  \item{package}{
    The name of the package (or packages) for which changes are to be listed.
    A character string or character vector.
  }
  \item{show}{
    Logical value indicating whether to display the table of changes
    on the terminal.
  }
}
\details{
  Details of changes are extracted from the NEWS file
  of the specified \code{package} under the heading
  \sQuote{Significant User-Visible Changes}.
  All entries for which the first line matches \code{x} are selected.
  The table of changes
  is displayed (if \code{show=TRUE}) and returned invisibly.
  
  The argument \code{sinceversion} should be a character string
  like \code{"1.2-3"}. The default is the version string of the most
  recent version.

  The argument \code{sincedate} should be a 
  character string like \code{"2015-05-27"}, or a date-time object.
  The default is the date of the most recent version.
  
  If \code{sinceversion="all"} or \code{sincedate="all"}
  then all recorded changes will be listed. 
  
  The special options \code{sinceversion="book"} and \code{sincedate="book"}
  are interpreted to mean \code{sincedate="2015-06-05"},
  which gives all changes reported after publication of
  the book by Baddeley, Rubak and Turner (2015).

  By default, changes in the \emph{extension} packages
  \pkg{spatstat.local}, \pkg{spatstat.Knet}, \pkg{spatstat.gui}
  are \emph{not} reported. To include these changes as well,
  set \code{package=spatstat.family(TRUE,TRUE)}.
}
\section{Pattern matching}{
  The first argument \code{x} is interpreted as a pattern to be matched
  using \code{\link[base]{grepl}}.
  
  For example if \code{x = "ppp"}, then this will match any news items
  that are about \code{ppp}, but will also match any news items
  about \code{plot.ppp}, \code{print.ppp}, \code{ppplist},
  \code{pppmatch}, \code{[.ppp} and so on.

  The pattern \code{x = "^ppp"} will match only those strings
  which begin with \code{"ppp"}, such as \code{ppp}, \code{ppplist},
  \code{pppmatch}. The symbol \code{^} means the start of the string.

  The pattern \code{x = "^ppp$"} will match only the string
  \code{"ppp"}. The symbol \code{$} means the end of the string.

  The pattern \code{x = "\\.ppp$"} will match only those strings which
  end with \code{".ppp"}, such as \code{plot.ppp}, \code{print.ppp},
  \code{[.ppp}. The symbol \code{.} has a special meaning, but here we
  want  the character \code{"."},
  so we use \code{\\} to escape the special
  meaning, so that \code{\\.} means the character \code{"."},
  and \code{"\\.ppp$"} means any string ending in \code{".ppp"}.
}
\value{
  (Invisibly) a data frame, belonging to the class \code{"changetable"},
  which has a \code{print} method.
}
\author{
  \adrian.
}
\seealso{
  \code{\link{bugfixes}},
  \code{\link{latest.news}},
  \code{\link[utils]{news}}.
}
\references{
  \baddrubaturnbook
}
\examples{
   latest.changes("plot.symbolmap")
}
\keyword{documentation}

\name{latest.news}
\alias{latest.news}
\title{
  Print News About Latest Version of Package
}
\description{
  Prints the news documentation for the current version of \code{spatstat}
  or another specified package.
}
\usage{
latest.news(package = spatstat.family(), doBrowse=FALSE, major=TRUE)
}
\arguments{
  \item{package}{
    Name of package for which the latest news should be printed.
    A character string, or vector of character strings.
  }
  \item{doBrowse}{
    Logical value indicating whether to display the results in a browser
    window instead of printing them.
  }
  \item{major}{
    Logical value.
    If \code{TRUE} (the default), print all information for the
    current major version \code{"x.y"}.
    If \code{FALSE}, print only the information
    for the current minor version \code{"x.y-z"}.
  }
}
\details{
  This function prints the news documentation about changes in the
  current installed version of a package.

  By default, it prints the latest news about all the sub-packages in
  the \pkg{spatstat} family.
  
  The function can be called simply by typing its
  name without parentheses (see the Examples).

  If \code{major=FALSE}, only information for the current minor version
  \code{"x.y-z"} will be printed. If \code{major=TRUE} (the default),
  all information for the current major version \code{"x.y"} will be
  printed, encompassing versions \code{"x.y-0"}, \code{"x.y-1"},
  up to \code{"x.y-z"}.
  
  If \code{package} is given, then the function reads the
  news for the specified package from its \code{NEWS} file (if it has one) 
  and prints only the entries that refer to the current version
  of the package.

  To see the news for all previous versions as well as the current
  version, use the \R utility \code{\link[utils]{news}}. See the Examples.
}
\value{
  Null.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{spatstat.family}} lists the packages in the \pkg{spatstat} family.

  \code{\link{bugfixes}} lists bug fixes.

  \code{\link[utils]{news}} 
}
\examples{
  if(interactive()) {

    # current news
    latest.news

    # all news
    # news(package="spatstat")

  }
}
\keyword{documentation}

\name{convolve.im}
\alias{convolve.im}
\title{Convolution of Pixel Images}
\description{
  Computes the convolution of two pixel images.
}
\usage{
 convolve.im(X, Y=X, \dots, reflectX=FALSE, reflectY=FALSE)
}
\arguments{
  \item{X}{
    A pixel image (object of class \code{"im"}.
  }
  \item{Y}{
    Optional. Another pixel image.
  }
  \item{\dots}{Ignored.}
  \item{reflectX,reflectY}{
    Logical values specifying whether the images \code{X} and \code{Y}
    (respectively) should be reflected in the origin before computing
    the convolution.
  }
}
\value{
  A pixel image (an object of class \code{"im"}) representing the
  convolution of \code{X} and \code{Y}.
}
\details{
  The \emph{convolution} of two pixel images \eqn{X} and \eqn{Y} in the plane
  is the function \eqn{C(v)} defined for each vector \eqn{v} as
  \deqn{
    C(v) = \int X(u)Y(v-u)\, {\rm d}u
  }{
    C(v) = integral of X(u) * Y(v-u) du
  }
  where the integral is
  over all spatial locations \eqn{u}, and where \eqn{X(u)} and
  \eqn{Y(u)} denote the pixel values of \eqn{X} and \eqn{Y} respectively
  at location \eqn{u}.
  
  This command computes a discretised approximation to
  the convolution, using the Fast Fourier Transform.
  The return value is
  another pixel image (object of class \code{"im"}) whose greyscale values
  are values of the convolution.

  If \code{reflectX = TRUE} then the pixel image \code{X} is reflected
  in the origin (see \code{\link{reflect}})
  before the convolution is computed, so that
  \code{convolve.im(X,Y,reflectX=TRUE)} is mathematically
  equivalent to \code{convolve.im(reflect(X), Y)}. (These two commands
  are not exactly equivalent, because the reflection is performed
  in the Fourier domain in the first command, and reflection is
  performed in the spatial domain in the second command).

  Similarly if \code{reflectY = TRUE} then the pixel image \code{Y} is reflected
  in the origin before the convolution is computed, so that
  \code{convolve.im(X,Y,reflectY=TRUE)} is mathematically
  equivalent to \code{convolve.im(X, reflect(Y))}.
}
\seealso{
  \code{\link{imcov}},
  \code{\link{reflect}}
}
\examples{
  X <- as.im(letterR)
  Y <- as.im(square(1))
  plot(convolve.im(X, Y))
  plot(convolve.im(X, Y, reflectX=TRUE))
  plot(convolve.im(X))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{discs}
\alias{discs}
\title{
 Union of Discs
}
\description{
  Make a spatial region composed of discs
  with given centres and radii.  
}
\usage{
  discs(centres, radii = marks(centres)/2, \dots,
        separate = FALSE, mask = FALSE, trim = TRUE,
        delta = NULL, npoly=NULL)
}
\arguments{
  \item{centres}{
    Point pattern giving the locations of centres for the discs.
  }
  \item{radii}{
    Vector of radii for each disc, or a single number giving a common
    radius.
    (Notice that the default assumes that the marks of \code{X} are
    \emph{diameters}.)
  }
  \item{\dots}{
    Optional arguments passed to \code{\link{as.mask}} to determine
    the pixel resolution, if \code{mask=TRUE}.
  }
  \item{separate}{
    Logical. If \code{TRUE}, the result is a list
    containing each disc as a separate entry.
    If \code{FALSE} (the default), the result is a window
    obtained by forming the union of the discs.
  }
  \item{mask}{
    Logical. If \code{TRUE}, the result is a binary mask window.
    If \code{FALSE}, the result is a polygonal window.
    Applies only when \code{separate=FALSE}.
  }
  \item{trim}{
    Logical value indicating whether to restrict the result
    to the original window of the \code{centres}.
    Applies only when \code{separate=FALSE}.
  }
  \item{delta}{
    Argument passed to \code{\link{disc}} to determine the 
    tolerance for the polygonal approximation of each disc.
    Applies only when \code{mask=FALSE}.
    Incompatible with \code{npoly}.
  }
  \item{npoly}{
    Argument passed to \code{\link{disc}} to determine the 
    number of edges in the polygonal approximation of each disc.
    Applies only when \code{mask=FALSE}.
    Incompatible with \code{delta}.
  }
}
\details{
  This command is typically applied to a marked point pattern
  dataset \code{X} in which the marks represent the sizes of objects.
  The result is a spatial region representing the space occupied by
  the objects.
  
  If the marks of \code{X} represent the diameters of circular objects,
  then the result of \code{discs(X)}
  is a spatial region constructed by taking discs, of the specified
  diameters, centred at the points of \code{X}, and forming the union
  of these discs. If the marks of \code{X} represent the areas of
  objects, one could take \code{discs(X, sqrt(marks(X)/pi))}
  to produce discs of equivalent area.

  A fast algorithm is used to compute the result as a binary mask, when
  \code{mask=TRUE}. This option is recommended unless polygons are
  really necessary.

  If \code{mask=FALSE}, the discs will be constructed as polygons
  by the function \code{\link{disc}}. To avoid computational problems,
  by default, the discs will all be constructed using
  the same physical tolerance value \code{delta}
  passed to \code{\link{disc}}. The default is such that the smallest
  disc will be approximated by a 16-sided polygon.
  (The argument \code{npoly} should not normally be used, to avoid
  computational problems arising with small radii.)
}
\value{
  If \code{separate=FALSE}, a window (object of class \code{"owin"}).

  If \code{separate=TRUE}, a list of windows.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{disc}},
  \code{\link{union.owin}}
}
\examples{
  plot(discs(anemones, mask=TRUE, eps=0.5))
}
\keyword{spatial}
\keyword{datagen}

\name{as.box3}
\Rdversion{1.1}
\alias{as.box3}
\title{
  Convert Data to Three-Dimensional Box
}
\description{
  Interprets data as the dimensions of a three-dimensional box.
}
\usage{
as.box3(...)
}
\arguments{
  \item{\dots}{
    Data that can be interpreted as giving the dimensions of a
    three-dimensional box. See Details.
  }
}
\details{
  This function converts data in various formats to
  an object of class \code{"box3"} representing a three-dimensional
  box (see \code{\link{box3}}). The arguments \code{\dots} may be
  \itemize{
    \item an object of class \code{"box3"}
    \item arguments acceptable to \code{box3}
    \item a numeric vector of length 6, interpreted as
    \code{c(xrange[1],xrange[2],yrange[1],yrange[2],zrange[1],zrange[2])}
    \item an object of class \code{"pp3"} representing a
    three-dimensional point pattern contained in a box.
  }
}
\value{
  Object of class \code{"box3"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{box3}}, 
  \code{\link{pp3}}
}
\examples{
    X <- c(0,10,0,10,0,5)
    as.box3(X)
    X <- pp3(runif(42),runif(42),runif(42), box3(c(0,1)))
    as.box3(X)
}
\keyword{spatial}
\keyword{manip}

\name{bugfixes}
\alias{bugfixes}
\title{
  List Recent Bug Fixes
}
\description{
  List all bug fixes in a package, starting from a certain date or
  version of the package. Fixes are sorted alphabetically by the name of the
  affected function. The default is to list bug fixes in the latest
  version of the \pkg{spatstat} family of packages.  
}
\usage{
  bugfixes(sinceversion = NULL, sincedate = NULL,
           package = spatstat.family(), show = TRUE)
}
\arguments{
  \item{sinceversion}{
    Earliest version of \code{package}
    for which bugs should be listed.
    A character string.
    The default is the current installed version.
  }
  \item{sincedate}{
    Earliest release date of \code{package}
    for which bugs should be listed.
    A character string or a date-time object.
  }
  \item{package}{
    The name of the package (or packages) for which bugs are to be listed.
    A character string or character vector.
  }
  \item{show}{
    Logical value indicating whether to display the bug table
    on the terminal.
  }
}
\details{
  Bug reports are extracted from the NEWS file
  of the specified \code{package}.
  Only those after a specified date, or after a specified version
  of the package, are retained.
  The bug reports are then sorted alphabetically, so that all bugs affecting a
  particular function are listed consecutively. Finally the table of bug
  reports is displayed (if \code{show=TRUE}) and returned invisibly.
  
  The argument \code{sinceversion} should be a character string
  like \code{"1.2-3"}. The default is the current installed version of the
  package.

  The argument \code{sincedate} should be a 
  character string like \code{"2015-05-27"}, or a date-time object.

  If \code{sinceversion="all"} or \code{sincedate="all"}
  then all recorded bugs will be listed.
  
  The special options \code{sinceversion="book"} and \code{sincedate="book"}
  are interpreted to mean \code{sincedate="2015-06-05"},
  which gives all bugs reported after publication of
  the book by Baddeley, Rubak and Turner (2015).

  Typing \code{bugfixes} without parentheses will display a table of
  all bugs that were fixed in the current installed version of
  \pkg{spatstat} and its sub-packages.

  By default, bugs in the \emph{extension} packages
  \pkg{spatstat.local}, \pkg{spatstat.Knet}, \pkg{spatstat.gui}
  are \emph{not} reported. To include these bugs as well,
  set \code{package=spatstat.family(TRUE,TRUE)}.
}
\value{
  (Invisibly) a data frame, belonging to the class \code{"bugtable"},
  which has a \code{print} method.
}
\author{
  \adrian.
}
\seealso{
  \code{\link{latest.changes}},
  \code{\link{latest.news}},
  \code{\link[utils]{news}}.  
}
\references{
  \baddrubaturnbook
}
\examples{
   bugfixes
   ## show all bugs reported after publication of the spatstat book
   if(interactive()) bugfixes(sinceversion="book")
}
\keyword{documentation}

\name{intersect.tess}
\alias{intersect.tess}
\title{Intersection of Two Tessellations}
\description{
  Yields the intersection of two tessellations,
  or the intersection of a tessellation with a window.
}
\usage{
   intersect.tess(X, Y, \dots, keepmarks=FALSE, sep="x")
}
\arguments{
  \item{X,Y}{Two tessellations (objects of class \code{"tess"}),
    or windows (objects of class \code{"tess"}),
    or other data that can be converted to 
    tessellations by \code{\link{as.tess}}.
  }
  \item{\dots}{
    Optional arguments passed to \code{\link{as.mask}}
    to control the discretisation, if required.
  }
  \item{keepmarks}{
    Logical value. If \code{TRUE}, the marks attached to the
    tiles of \code{X} and \code{Y} will be retained as marks of
    the intersection tiles. 
  }
  \item{sep}{
    Character string used to separate the names of tiles from \code{X}
    and from \code{Y}, when forming the name of the tiles of the
    intersection.
  }
}
\value{
  A tessellation (object of class \code{"tess"}).
}
\details{
  A tessellation is a collection of disjoint spatial regions
  (called \emph{tiles}) that fit together to form a larger spatial
  region. See \code{\link{tess}}.

  If \code{X} and \code{Y} are not tessellations, they are first
  converted into tessellations by \code{\link{as.tess}}.

  The function \code{intersect.tess} then computes the intersection between
  the two tessellations. This is another tessellation, each of whose
  tiles is the intersection of a tile from \code{X} and a tile from \code{Y}.

  One possible use of this function is to slice a window \code{W} into
  subwindows determined by a tessellation. See the Examples.
}

\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{tess}},
  \code{\link{as.tess}},
  \code{\link{intersect.owin}}
}
\examples{
  opa <- par(mfrow=c(1,3))
# polygon
  data(letterR)
  plot(letterR)
# tessellation of rectangles
  X <- tess(xgrid=seq(2, 4, length=10), ygrid=seq(0, 3.5, length=8))
  plot(X)
  plot(intersect.tess(X, letterR))

  A <- runifpoint(10)
  B <- runifpoint(10)
  plot(DA <- dirichlet(A))
  plot(DB <- dirichlet(B))
  plot(intersect.tess(DA, DB))
  par(opa)

  marks(DA) <- 1:10
  marks(DB) <- 1:10
  plot(Z <- intersect.tess(DA,DB, keepmarks=TRUE))
  mZ <- marks(Z)
  tZ <- tiles(Z)
  for(i in which(mZ[,1] == 3)) plot(tZ[[i]], add=TRUE, col="pink")
}
\keyword{spatial}
\keyword{math}

\name{spatstat-internal} 
\title{Internal spatstat functions}
\alias{spatstat-internal} %DoNotExport



\alias{print.autoexec}    
\alias{print.bugtable}
\alias{print.changetable}

%%%%%%%
\description{
  Internal spatstat functions.
}
\usage{





\method{print}{autoexec}(x, \dots)
\method{print}{bugtable}(x, \dots)
\method{print}{changetable}(x, \dots)
}
\details{
  These internal \pkg{spatstat} functions should not be called
  directly by the user. Their names and capabilities may change
  without warning from one version of \pkg{spatstat} to the next.
}
\value{
The return values of these functions are not documented,
and may change without warning.
}
\keyword{internal}

\name{append.psp}
\alias{append.psp}
\title{Combine Two Line Segment Patterns}
\description{
  Combine two line segment patterns into a single pattern.
}
\usage{
  append.psp(A, B)
}
\arguments{
  \item{A,B}{
    Line segment patterns (objects of class \code{"psp"}).
  }
}
\value{
  Another line segment pattern (object of class \code{"psp"}).
}
\details{
  This function is used to superimpose two line segment patterns
  \code{A} and \code{B}.
  
  The two patterns must have \bold{identical} windows. If one
  pattern has marks, then the other must also have marks
  of the same type.  It the marks are data frames then the
  number of columns of these data frames, and the names of
  the columns must be identical.

  (To combine two point patterns, see \code{superimpose}).

  If one of the arguments is \code{NULL}, it will be ignored
  and the other argument will be returned.
}
\seealso{
  \code{\link{psp}},
  \code{\link{as.psp}},
  \code{\link{superimpose}},
}
\examples{
  X <- psp(runif(20), runif(20), runif(20), runif(20),  window=owin())
  Y <- psp(runif(5), runif(5), runif(5), runif(5),  window=owin())
  append.psp(X,Y)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{as.data.frame.im}
\alias{as.data.frame.im}
\title{Convert Pixel Image to Data Frame}
\description{
  Convert a pixel image to a data frame
}
\usage{
  \method{as.data.frame}{im}(x, ...)
}
\arguments{
  \item{x}{A pixel image (object of class \code{"im"}).}
  \item{\dots}{Further arguments passed to
    \code{\link[base:as.data.frame]{as.data.frame.default}}
    to determine the row names and other features.}
}
\details{
  This function takes the pixel image \code{x}
  and returns a data frame with three columns
  containing the pixel coordinates and the pixel values.

  The data frame entries are automatically sorted in increasing order of
  the \code{x} coordinate (and in increasing order of \code{y} within
  \code{x}). 
}
\value{
  A data frame.
}
\examples{
   # artificial image
   Z <- setcov(square(1))

   Y <- as.data.frame(Z)

   head(Y)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{methods}

\name{dppapproxpcf}
\alias{dppapproxpcf}
\title{Approximate Pair Correlation Function of Determinantal Point Process Model}
\description{
  Returns an approximation to the
  theoretical pair correlation function of a
  determinantal point process model, as a
  function of one argument \eqn{x}.
}
\usage{dppapproxpcf(model, trunc = 0.99, W = NULL)}
\arguments{
  \item{model}{Object of class \code{"detpointprocfamily"}.}
  \item{trunc}{Numeric specifying how the model truncation is
    performed. See Details section of \code{\link{simulate.detpointprocfamily}}.}
  \item{W}{Optional window -- undocumented at the moment.}
}
\details{This function is usually NOT needed for anything. It
only exists for investigative purposes.}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\examples{
f <- dppapproxpcf(dppMatern(lambda = 100, alpha=.028, nu=1, d=2))
plot(f, xlim = c(0,0.1))
}

\name{triangulate.owin}
\alias{triangulate.owin}
\title{
  Decompose Window into Triangles
}
\description{
  Given a spatial window, this function decomposes the window
  into disjoint triangles. 
  The result is a tessellation of the window
  in which each tile is a triangle.
}
\usage{
triangulate.owin(W)
}
\arguments{
  \item{W}{Window (object of class \code{"owin"}).}
}
\details{
  The window \code{W} will be decomposed into disjoint triangles.
  The result is a tessellation of \code{W} in which each tile is a
  triangle. All triangle vertices lie on the boundary
  of the original polygon.
  
  The window is first converted to a polygonal window using
  \code{\link{as.polygonal}}. The vertices of the polygonal window
  are extracted, and the Delaunay triangulation of these vertices
  is computed using \code{\link{delaunay}}. Each Delaunay triangle
  is intersected with the window: if the result is not a triangle, 
  the triangulation procedure is applied recursively to this smaller polygon. 
}
\value{
  Tessellation (object of class \code{"tess"}).
}
\author{
  \spatstatAuthors
}
\seealso{
  \code{\link{tess}},
  \code{\link{delaunay}},
  \code{\link{as.polygonal}}
}
\examples{
  plot(triangulate.owin(letterR))
}
\keyword{spatial}
\keyword{manip}

\name{square}
\alias{square}
\alias{unit.square}
\title{Square Window}
\description{
Creates a square window
}
\usage{
 square(r=1, unitname=NULL)
 unit.square()
}
\arguments{
  \item{r}{Numeric. The side length of the square,
    or a vector giving the minimum and maximum coordinate values.
  }
  \item{unitname}{
    Optional. Name of unit of length. Either a single character string,
    or a vector of two character strings giving the
    singular and plural forms, respectively.
  }
}
\value{
  An object of class \code{"owin"} (see \code{\link{owin.object}})
  specifying a window. 
}
\details{
  If \code{r} is a number, \code{square(r)}
  is a shortcut for creating a window object
  representing the square 
  \eqn{[0,r] \times [0,r]}{[0,r] * [0,r]}.
  It is equivalent to the command
  \code{owin(c(0,r),c(0,r))}.

  If \code{r} is a vector of length 2, then
  \code{square(r)} creates the square with \code{x} and \code{y} coordinates
  ranging from \code{r[1]} to \code{r[2]}.
  
  \code{unit.square} creates the unit square
  \eqn{[0,1] \times [0,1]}{[0,1] * [0,1]}.
  It is equivalent to
  \code{square(1)} or \code{square()} or \code{owin(c(0,1),c(0,1))}.

  These commands are included for convenience,
  and to improve the readability of some code.
}
\seealso{
  \code{\link{owin.object}},
  \code{\link{owin}}
}
\examples{
 W <- square(10)
 W <- square(c(-1,1))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}
 
 

\name{symbolmap}
\alias{symbolmap}
\title{
  Graphics Symbol Map
}
\description{
  Create a graphics symbol map that associates data values with
  graphical symbols.
}
\usage{
symbolmap(\dots, range = NULL, inputs = NULL)
}
\arguments{
  \item{\dots}{
    Named arguments specifying the graphical parameters.
    See Details.
  }
  \item{range}{
    Optional. Range of numbers that are mapped.
    A numeric vector of length 2 giving the minimum and maximum
    values that will be mapped.
    Incompatible with \code{inputs}.
  }
  \item{inputs}{
    Optional. A vector containing all the data values
    that will be mapped to symbols.
    Incompatible with \code{range}.
  }
}
\details{
  A graphical symbol map is an association between
  data values and graphical symbols. 
  The command \code{symbolmap} creates an object of class
  \code{"symbolmap"} that represents a graphical symbol map.

  Once a symbol map has been created, it can be applied to any
  suitable data to generate a plot of those data.
  This makes it easy to ensure that
  the \emph{same} symbol map is used in two different plots.
  The symbol map can be plotted as a legend to the plots,
  and can also be plotted in its own right.
  
  The possible values of data that will be mapped
  are specified by \code{range} or \code{inputs}.
  \itemize{
    \item if \code{range} is given, it should be a numeric vector
    of length 2 giving the minimum and maximum values of the range
    of numbers that will be mapped. These limits must be finite.
    \item if \code{inputs} is given, it should be a vector
    of any atomic type (e.g. numeric, character, logical, factor).
    This vector contains all the possible data values
    that will be mapped.
    \item If neither \code{range} nor \code{inputs} is given,
    it is assumed that the possible values are real numbers.
  }
  The association of data values with graphical symbols
  is specified by the other arguments \code{\dots}
  which are given in \code{name=value} form.
  These arguments specify the kinds of symbols that will be
  used, the sizes of the symbols, and graphics parameters for
  drawing the symbols.

  Each graphics parameter can be either a single
  value, for example \code{shape="circles"},
  or a \code{function(x)} which determines the value
  of the graphics parameter as a function of the data \code{x},
  for example \code{shape=function(x) ifelse(x > 0, "circles", "squares")}.
  Colourmaps (see \code{\link{colourmap}}) are also acceptable
  because they are functions.
  
  Currently recognised graphics parameters, and their
  allowed values, are:
  \describe{
    \item{shape}{
      The shape of the symbol: currently
      either \code{"circles"}, \code{"squares"}, \code{"arrows"}
      or \code{NA}.
      This parameter takes precedence over \code{pch}.
    }
    \item{size}{
      The size of the symbol: a positive number or zero.
    }
    \item{pch}{
      Graphics character code:
      a positive integer, or a single character.
      See \code{\link[graphics]{par}}.
    }
    \item{cex}{
      Graphics character expansion factor.
    }
    \item{cols}{
      Colour of plotting characters.
    }
    \item{fg,bg}{
      Colour of foreground (or symbol border) and background
      (or symbol interior).
    }
    \item{col,lwd,lty}{
      Colour, width and style of lines.
    }
    \item{etch}{
      Logical. If \code{TRUE}, each symbol is surrounded
      by a border drawn in the opposite colour,
      which improves its visibility against the background.
      Default is \code{FALSE}.
    }
    \item{direction,headlength,headangle,arrowtype}{
      Numeric parameters of arrow symbols, applicable when
      \code{shape="arrows"}. Here \code{direction} is the direction
      of the arrow in degrees anticlockwise from the \eqn{x} axis;
      \code{headlength} is the length of the head of the arrow in
      coordinate units; \code{headangle} is the angle subtended by the point
      of the arrow; and \code{arrowtype} is an integer code
      specifying which ends of the shaft have arrowheads
      attached (0 means no arrowheads, 1 is an arrowhead at the start
      of the shaft, 2 is an arrowhead at the end of the shaft, and
      3 is arrowheads at both ends). 
    }
  }
  A vector of colour values is also acceptable for the arguments
  \code{col,cols,fg,bg} if
  \code{range} is specified.
}
\value{
  An object of class \code{"symbolmap"}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{plot.symbolmap}} to plot the symbol map itself.
  
  \code{\link{invoke.symbolmap}} to apply the symbol map to some data
  and plot the resulting symbols.
  
  \code{\link{update.symbolmap}} to change the symbol map.
}
\examples{
  g <- symbolmap(inputs=letters[1:10], pch=11:20)

  g1 <- symbolmap(range=c(0,100), size=function(x) x/50)

  g2 <- symbolmap(shape=function(x) ifelse(x > 0, "circles", "squares"),
                    size=function(x) sqrt(ifelse(x > 0, x/pi, -x)),
                    bg = function(x) ifelse(abs(x) < 1, "red", "black"))

  colmap <- colourmap(topo.colors(20), range=c(0,10))
  g3 <- symbolmap(pch=21, bg=colmap, range=c(0,10))
  plot(g3)
}
\keyword{spatial}
\keyword{hplot}

\name{plot.linnet}
\alias{plot.linnet}
\title{
  Plot a linear network
}
\description{
  Plots a linear network
}
\usage{
 \method{plot}{linnet}(x, ..., main=NULL, add=FALSE, 
                               vertices=FALSE, window=FALSE,
                               do.plot=TRUE)
}
\arguments{
  \item{x}{
    Linear network (object of class \code{"linnet"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{plot.psp}}
    controlling the plot.
  }
  \item{main}{
    Main title for plot. Use \code{main=""} to suppress it.
  }
  \item{add}{
    Logical. If code{TRUE}, superimpose the graphics
    over the current plot. If \code{FALSE}, generate a new plot.
  }
  \item{vertices}{
    Logical. Whether to plot the vertices as well.
  }
  \item{window}{
    Logical. Whether to plot the window containing the linear network.
  }
  \item{do.plot}{
    Logical. Whether to actually perform the plot.
  }
}
\details{
  This is the plot method for class \code{"linnet"}.
}
\value{
  An (invisible) object of class \code{"owin"}
  giving the bounding box of the network.
}
\author{
  Ang Qi Wei \email{aqw07398@hotmail.com} and
  \adrian 
  
}
\seealso{
   \code{\link{linnet}}
}
\examples{
   plot(simplenet)
}
\keyword{spatial}

\name{clickdist}
\alias{clickdist}
\title{Interactively Measure Distance}
\description{
  Measures the distance between two points
  which the user has clicked on.
}
\usage{
  clickdist()
}
\value{
  A single nonnegative number.
}
\details{
  This function allows the user to measure the distance
  between two spatial locations, interactively,
  by clicking on the screen display.

  When \code{clickdist()} is called, the user is expected to
  click two points in the current graphics device. The distance
  between these points will be returned.

  This function uses the \R{} command \code{\link[graphics]{locator}} to
  input the mouse clicks. It only works on screen devices such as
  \sQuote{X11}, \sQuote{windows} and \sQuote{quartz}.
}
\seealso{
  \code{\link[graphics]{locator}},
  \code{\link{clickppp}},
  \code{\link{clicklpp}},
  \code{\link{clickpoly}},
  \code{\link{clickbox}}
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{iplot}

\name{plot.owin}
\alias{plot.owin}
\title{Plot a Spatial Window}
\description{
  Plot a two-dimensional window of observation for a spatial point pattern
}
\usage{
 \method{plot}{owin}(x, main, add=FALSE, \dots, box, edge=0.04,
                      type=c("w","n"), show.all=!add, 
                      hatch=FALSE,
                      hatchargs=list(), 
                      invert=FALSE, do.plot=TRUE,
                      claim.title.space=FALSE, use.polypath=TRUE)
}
\arguments{
  \item{x}{
    The window to be plotted.
    An object of class \code{\link{owin}},
    or data which can be converted into 
    this format by \code{\link{as.owin}()}.
  }
  \item{main}{
    text to be displayed as a title above the plot.
  }
  \item{add}{
    logical flag: if \code{TRUE}, draw the window in 
    the current plot; if \code{FALSE}, generate a new plot.
  }
  \item{\dots}{
    extra arguments controlling the appearance of the plot.
    These arguments are passed to \code{\link[graphics]{polygon}}
    if \code{x} is a
    polygonal or rectangular window, or passed to
    \code{\link[graphics]{image.default}}
    if \code{x} is a binary mask.
    Some arguments are passed to \code{\link[graphics]{plot.default}}.
    See Details.
  }
  \item{box}{
    logical flag; if \code{TRUE}, plot the enclosing rectangular box
  }
  \item{edge}{
    nonnegative number; the plotting region will have coordinate limits
    that are \code{1 + edge}
    times as large as the limits of the rectangular box
    that encloses the pattern.
  }
  \item{type}{
    Type of plot: either \code{"w"} or \code{"n"}.
    If \code{type="w"} (the default), the window is plotted.
    If \code{type="n"} and \code{add=TRUE}, a new plot is initialised
    and the coordinate system is established, but nothing is drawn.
  }
  \item{show.all}{
    Logical value indicating whether to plot everything
    including the main title.
  }
  \item{hatch}{
    logical flag; if \code{TRUE}, the interior of the window will
    be shaded by texture, such as a grid of parallel lines.
  }
  \item{hatchargs}{
    List of arguments passed to \code{\link{add.texture}}
    to control the texture shading when \code{hatch=TRUE}. 
  }
  \item{invert}{
    logical flag; when the window is a binary pixel mask,
    the mask colours will be inverted if \code{invert=TRUE}.
  }
  \item{do.plot}{
    Logical value indicating whether to actually perform the plot.
  }
  \item{claim.title.space}{
    Logical value indicating whether extra space for the main title
    should be allocated when declaring the plot dimensions.
    Should be set to \code{FALSE} under normal conditions.
  }
  \item{use.polypath}{
    Logical value indicating what graphics capabilities should be used
    to draw a polygon filled with colour when the polygon has holes.
    If \code{TRUE} (the default), then the polygon will be filled using
    \code{\link[graphics]{polypath}}, provided the graphics device
    supports this function. If \code{FALSE}, the polygon will be
    decomposed into simple closed polygons, which will be colour filled
    using \code{\link[graphics]{polygon}}.
  }
}
\value{
  none.
}
\details{
  This is the \code{plot} method for the class \code{\link{owin}}.
  The action is to plot the boundary of the window on the current plot device,
  using equal scales on the \code{x} and \code{y} axes.

  If the window \code{x} is of type \code{"rectangle"} or \code{"polygonal"},
  the boundary of the window is plotted as a polygon or series of polygons.
  If \code{x} is of type \code{"mask"} 
  the discrete raster approximation of the window is displayed
  as a binary image (white inside the window, black outside).

  Graphical parameters controlling the display (e.g. setting the
  colours) may be passed directly via the \code{...} arguments,
  or indirectly reset using 
  \code{\link{spatstat.options}}.

  If \code{add=FALSE} (the default), the plot is initialised
  by calling the base graphics function
  \code{\link[graphics]{plot.default}}
  to create the plot area. By default, coordinate axes
  and axis labels are not plotted. To plot coordinate axes,
  use the argument \code{axes=TRUE}; 
  to plot axis labels, use the argument \code{ann=TRUE}
  and then specify the labels with \code{xlab} and \code{ylab};
  see the help file for \code{\link[graphics]{plot.default}} for
  information on these arguments, and for additional
  arguments controlling the appearance of the axes.
  See the Examples also.

  When \code{x} is of type \code{"rectangle"} or \code{"polygonal"}, it
  is plotted by the \R function \code{\link[graphics]{polygon}}. To control the
  appearance (colour, fill density, line density etc) of the polygon
  plot, determine the required argument of \code{\link[graphics]{polygon}} and
  pass it through \code{...} For example, to paint the interior of the
  polygon in red, use the argument \code{col="red"}. To draw the polygon
  edges in green, use \code{border="green"}. To suppress the drawing of
  polygon edges, use \code{border=NA}.

  When \code{x} is of type \code{"mask"}, it is plotted by
  \code{\link[graphics]{image.default}}. The appearance of the image plot
  can be controlled by passing arguments to
  \code{\link[graphics]{image.default}}
  through \code{...}. The default appearance can also be changed
  by setting the parameter \code{par.binary} of
  \code{\link{spatstat.options}}. 

  To zoom in (to view only a subset of the window at higher
  magnification), use the graphical arguments
  \code{xlim} and \code{ylim} to specify the desired rectangular field of
  view. (The actual field of view may be larger, depending on the
  graphics device).
}
\section{Notes on Filled Polygons with Holes}{
  The function \code{\link[graphics]{polygon}} can only handle
  polygons without holes. To plot polygons with holes in a solid colour,
  we have implemented two workarounds.
  
  \describe{
    \item{polypath function:}{
      The first workaround uses
      the relatively new function \code{\link[graphics]{polypath}} which
      \emph{does} have the capability to handle polygons with holes.
      However, not all graphics devices support
      \code{\link[graphics]{polypath}}.
      The older devices \code{\link{xfig}} and \code{\link{pictex}}
      do not support \code{\link[graphics]{polypath}}.
      On a Windows system, the default graphics device
#ifdef windows      
      \code{\link{windows}} 
#endif
#ifndef windows      
      \code{windows}
#endif
      supports \code{\link[graphics]{polypath}}.
#ifdef unix
      On a Linux system, the default graphics device
      \code{X11(type="Xlib")} does \emph{not} support
      \code{\link[graphics]{polypath}}
      but \code{X11(type="cairo")} does support it.
      See \code{\link{X11}} and the section on Cairo below.
#endif
    }
    \item{polygon decomposition:}{
      The other workaround involves decomposing the polygonal window
      into pieces which do not have holes. This code is experimental
      but works in all our test cases. If this code fails, a warning
      will be issued, and the filled colours will not be plotted.
    }
  }
}
#ifdef unix
\section{Cairo graphics on a Linux system}{
  Linux systems support
  the graphics device \code{X11(type="cairo")} (see \code{\link{X11}})
  provided the external library \pkg{cairo} is installed
  on the computer. See \code{www.cairographics.org}
  for instructions on obtaining and installing \pkg{cairo}.  After having
  installed \pkg{cairo} one needs to re-install \R from source so
  that it has \pkg{cairo} capabilites.  To check whether your
  current installation of R has \pkg{cairo} capabilities, type
  (in \R) \code{capabilities()["cairo"]}.
  The default type for \code{\link{X11}} is controlled by
  \code{\link[grDevices]{X11.options}}.
  You may find it convenient to
  make \pkg{cairo} the default, e.g. via your \code{.Rprofile}.
  The magic incantation to put into \code{.Rprofile} is
  \preformatted{
    setHook(packageEvent("graphics", "onLoad"),
    function(...) grDevices::X11.options(type="cairo"))
  }
}
#endif
\seealso{
  \code{\link{owin.object}},
  \code{\link{plot.ppp}},
  \code{\link[graphics]{polygon}},
  \code{\link[graphics]{image.default}},
  \code{\link{spatstat.options}}
}
\examples{
  # rectangular window
   plot(Window(nztrees))
   abline(v=148, lty=2)

  # polygonal window
  w <- Window(demopat)
  plot(w)
  plot(w, col="red", border="green", lwd=2)
  plot(w, hatch=TRUE, lwd=2)

  # binary mask
  we <- as.mask(w)
  plot(we)
  op <- spatstat.options(par.binary=list(col=grey(c(0.5,1))))
  plot(we)
  spatstat.options(op)

  ## axis annotation
  plot(letterR, axes=TRUE, ann=TRUE, xlab="Easting", ylab="Northing")
  plot(letterR,            ann=TRUE, xlab="Declination", ylab="Right Ascension")
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{hplot}

\name{midpoints.psp}
\alias{midpoints.psp}
\title{Midpoints of Line Segment Pattern}
\description{
  Computes the midpoints of each line segment
  in a line segment pattern.
}
\usage{
  midpoints.psp(x)
}
\arguments{
  \item{x}{
    A line segment pattern (object of class \code{"psp"}).
  }
}
\value{
  Point pattern (object of class \code{"ppp"}).
}
\details{
  The midpoint of each line segment is computed.
}
\seealso{
  \code{\link{marks.psp}},
  \code{\link{summary.psp}},
  \code{\link{lengths_psp}}
  \code{\link{angles.psp}},
  \code{\link{endpoints.psp}},
  \code{\link{extrapolate.psp}}.
}
\examples{
  a <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  b <- midpoints.psp(a)   
}
\author{
  \adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{as.psp}
\alias{as.psp}
\alias{as.psp.psp}
\alias{as.psp.data.frame}
\alias{as.psp.matrix}
\alias{as.psp.default}
\title{Convert Data To Class psp}
\description{
  Tries to coerce any reasonable kind of data object to a line segment pattern
  (an object of class \code{"psp"})
  for use by the \pkg{spatstat} package.
}
\usage{
  as.psp(x, \dots, from=NULL, to=NULL)

  \method{as.psp}{psp}(x, \dots, check=FALSE, fatal=TRUE)

  \method{as.psp}{data.frame}(x, \dots, window=NULL, marks=NULL,
      check=spatstat.options("checksegments"), fatal=TRUE)

  \method{as.psp}{matrix}(x, \dots, window=NULL, marks=NULL,
       check=spatstat.options("checksegments"), fatal=TRUE)

  \method{as.psp}{default}(x, \dots, window=NULL, marks=NULL,
       check=spatstat.options("checksegments"), fatal=TRUE)
}
\arguments{
  \item{x}{Data which will be converted into a line segment pattern}
  \item{window}{Data which define a window for the pattern.}
  \item{\dots}{Ignored.}
  \item{marks}{(Optional) vector or data frame of marks for the pattern}
  \item{check}{
    Logical value indicating whether to check the validity of the data,
    e.g. to check that the line segments lie inside the window.
  }
  \item{fatal}{Logical value. See Details.}
  \item{from,to}{Point patterns (object of class \code{"ppp"})
    containing the first and second endpoints (respectively) of each
    segment. Incompatible with \code{x}.
  }
}
\value{
  An object of class \code{"psp"} (see \code{\link{psp.object}})
  describing the line segment pattern and its window of observation.
  The value \code{NULL} may also be returned; see Details.
}
\details{
  Converts the dataset \code{x} to a line segment pattern
  (an object of class \code{"psp"}; see \code{\link{psp.object}} for
  an overview).

  This function is normally used to convert an existing line segment pattern
  dataset, stored in another format, to the \code{"psp"} format.
  To create a new point pattern from raw data such as \eqn{x,y}
  coordinates, it is normally easier to use the creator function
  \code{\link{psp}}.

  The dataset \code{x} may be:
  \itemize{
    \item
    an object of class \code{"psp"}  
    \item
    a data frame with at least 4 columns
    \item a structure (list) with elements named  \code{x0, y0,
    x1, y1} or elements named \code{xmid, ymid, length, angle}
    and possibly a fifth element named \code{marks}
  }
  If \code{x} is a data frame the interpretation of its columns is
  as follows:
  \itemize{
    \item
    If there are columns named \code{x0, y0, x1, y1} then these
    will be interpreted as the coordinates of the endpoints of
    the segments and used to form the \code{ends} component of
    the \code{psp} object to be returned.
    \item
    If there are columns named \code{xmid, ymid, length, angle}
    then these will be interpreted as the coordinates of the segment
    midpoints, the lengths of the segments, and the orientations
    of the segments in radians and used to form the \code{ends}
    component of the \code{psp} object to be returned.
    \item
    If there is a column named \code{marks} then this will
    be interpreted as the marks of the pattern provided that
    the argument \code{marks} of this function is \code{NULL}.
    If argument \code{marks} is not \code{NULL} then the value
    of this argument is taken to be the marks of the pattern and
    the column named \code{marks} is ignored (with a warning).
    In either case the column named marks is deleted and omitted
    from further consideration.
    \item
    If there is no column named \code{marks} and if the \code{marks}
    argument of this function is \code{NULL}, and if after
    interpreting 4 columns of \code{x} as determining the \code{ends}
    component of the \code{psp} object to be returned, there remain
    other columns of \code{x}, then these remaining columns will be
    taken to form a data frame of marks for the \code{psp} object
    to be returned.
  }

  If \code{x} is a structure (list) with elements named \code{x0,
  y0, x1, y1, marks} or \code{xmid, ymid, length, angle, marks},
  then the element named \code{marks} will be interpreted as the
  marks of the pattern provide that the argument \code{marks} of
  this function is \code{NULL}.  If this argument is non-\code{NULL}
  then it is interpreted as the marks of the pattern and the element
  \code{marks} of \code{x} is ignored --- with a warning.

  Alternatively, you may specify two point patterns
  \code{from} and \code{to} containing the first and second endpoints
  of the line segments.
  
  The argument \code{window} is converted to a window object
  by the function \code{\link{as.owin}}.

  The argument \code{fatal} indicates what to do when
  the data cannot be converted to a line segment pattern.
  If \code{fatal=TRUE}, a fatal error
  will be generated; if \code{fatal=FALSE}, the
  value \code{NULL} is returned.

  The function \code{as.psp} is generic, with methods for the
  classes \code{"psp"}, \code{"data.frame"}, \code{"matrix"}
  and a default method.
  
  Point pattern datasets can also be created by the function
  \code{\link{psp}}.
}
\section{Warnings}{
If only a proper subset of the names \code{x0,y0,x1,y1} or
\code{xmid,ymid,length,angle} appear amongst the names of the
columns of \code{x} where \code{x} is a data frame, then these
special names are ignored.

For example if the names of the columns were
\code{xmid,ymid,length,degrees}, then these columns would be
interpreted as if the represented \code{x0,y0,x1,y1} in that order.

Whether it gets used or not, column named \code{marks} is
\emph{always} removed from \code{x} before any attempt to form the
\code{ends} component of the \code{psp} object that is returned.
}
\seealso{
  \code{\link{psp}},
  \code{\link{psp.object}},
  \code{\link{as.owin}},
  \code{\link{owin.object}}.

  See \code{\link{edges}} for extracting the edges of a polygonal window
  as a \code{"psp"} object.
}
\examples{
   mat <- matrix(runif(40), ncol=4)
   mx <- data.frame(v1=sample(1:4,10,TRUE),
                    v2=factor(sample(letters[1:4],10,TRUE),levels=letters[1:4]))
   a <- as.psp(mat, window=owin(),marks=mx)
   mat <- cbind(as.data.frame(mat),mx)
   b <- as.psp(mat, window=owin()) # a and b are identical.
   stuff <- list(xmid=runif(10),
                 ymid=runif(10),
                 length=rep(0.1, 10),
                 angle=runif(10, 0, 2 * pi))
   a <- as.psp(stuff, window=owin())
   b <- as.psp(from=runifpoint(10), to=runifpoint(10))
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{selfcut.psp}
\alias{selfcut.psp}
\title{Cut Line Segments Where They Intersect}
\description{
  Finds any crossing points between the line segments in a 
  line segment pattern, and cuts the segments into pieces
  at these crossing-points.
}
\usage{
selfcut.psp(A, \dots, eps)
}
\arguments{
  \item{A}{
    Line segment pattern (object of class \code{"psp"}).
  }
  \item{eps}{
    Optional. Smallest permissible length of the resulting
    line segments. There is a sensible default.
  }
  \item{\dots}{Ignored.}
}
\details{
  This function finds any crossing points between
  different line segments in the line segment pattern \code{A},
  and cuts the line segments into pieces at these intersection points.

  A crossing point occurs whenever one of the line segments in \code{A}
  intersects another line segment in \code{A}, at a nonzero
  angle of intersection.
}
\value{
  Another line segment pattern (object of class \code{"psp"})
  in the same window as \code{A} with the same kind of marks as
  \code{A}.

  The result also has an attribute \code{"camefrom"} indicating
  the provenance of each segment in the result.
  For example \code{camefrom[3]=2} means that the third segment in the
  result is a piece of the second segment of \code{A}.
}
\author{
\spatstatAuthors.
}
\seealso{
  \code{\link{selfcrossing.psp}}
}
\examples{
  X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  Y <- selfcut.psp(X)
  n <- nsegments(Y)
  plot(Y \%mark\% factor(sample(seq_len(n), n, replace=TRUE)))
}
\keyword{spatial}
\keyword{manip}

\name{nnwhich.ppx}
\alias{nnwhich.ppx}
\title{Nearest Neighbours in Any Dimensions}
\description{
  Finds the nearest neighbour of each point in a multi-dimensional
  point pattern.
}
\usage{
  \method{nnwhich}{ppx}(X, \dots, k=1)
}
\arguments{
  \item{X}{
    Multi-dimensional point pattern 
    (object of class \code{"ppx"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{coords.ppx}} to determine
    which coordinates should be used.
  }
  \item{k}{
    Integer, or integer vector. The algorithm will compute the distance to the
    \code{k}th nearest neighbour. 
  }
}
\value{
  Numeric vector or matrix giving, for each point,
  the index of its nearest neighbour (or \code{k}th nearest neighbour).

  If \code{k = 1} (the default), the return value is a
  numeric vector \code{v} giving the indices of the nearest neighbours
  (the nearest neighbout of the \code{i}th point is
  the \code{j}th point where \code{j = v[i]}).
  
  If \code{k} is a single integer, then the return value is a
  numeric vector giving the indices of the
  \code{k}th nearest neighbours.

  If \code{k} is a vector, then the return value is a
  matrix \code{m} such that \code{m[i,j]} is the
  index of the \code{k[j]}th nearest neighbour for the
  \code{i}th data point.
}
\details{
  For each point in the given multi-dimensional
  point pattern, this function finds
  its nearest neighbour (the nearest other point of the pattern).
  By default it returns a vector giving, for each point,
  the index of the point's
  nearest neighbour. If \code{k} is specified, the algorithm finds
  each point's \code{k}th nearest neighbour.

  The function \code{nnwhich} is generic. This is the method
  for the class \code{"ppx"}.

  If there are no points in the pattern,
  a numeric vector of length zero is returned.
  If there is only one point,
  then the nearest neighbour is undefined, and a value of \code{NA}
  is returned. In general if the number of points is less than or equal
  to \code{k}, then a vector of \code{NA}'s is returned.

  To evaluate the \emph{distance} between a point and its nearest
  neighbour, use \code{\link{nndist}}.

  To find the nearest neighbours from one point pattern
  to another point pattern, use \code{\link{nncross}}.
  
  By default, both spatial and temporal coordinates are extracted.
  To obtain the spatial distance between points in a space-time point
  pattern, set \code{temporal=FALSE}.
}
\section{Warnings}{
  A value of \code{NA} is returned if there is only one point
  in the point pattern. 
}
\seealso{
  \code{\link{nnwhich}},
  \code{\link{nndist}},
  \code{\link{nncross}}
}
\examples{
   df <- data.frame(x=runif(5),y=runif(5),z=runif(5),w=runif(5))
   X <- ppx(data=df)
   m <- nnwhich(X)
   m2 <- nnwhich(X, k=2)
}
\author{
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{nnmark}
\alias{nnmark}
\title{
  Mark of Nearest Neighbour 
}
\description{
  Given a marked point pattern dataset \code{X}
  this function computes, for each desired location \code{y},
  the mark attached to the nearest neighbour of \code{y} in \code{X}.
  The desired locations \code{y} can be either a pixel grid
  or the point pattern \code{X} itself.
}
\usage{
nnmark(X, \dots, k = 1, at=c("pixels", "points"))
}
\arguments{
  \item{X}{
    A marked point pattern (object of class \code{"ppp"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine the
    pixel resolution.
  }
  \item{k}{
    Single integer. The \code{k}th nearest data point will be used.
  }
  \item{at}{
    String specifying whether to compute the values
    at a grid of pixel locations (\code{at="pixels"}) or
    only at the points of \code{X} (\code{at="points"}).
  }
}
\details{
  Given a marked point pattern dataset \code{X}
  this function computes, for each desired location \code{y},
  the mark attached to the point of \code{X} that is nearest
  to \code{y}. The desired locations \code{y} can be either a pixel grid
  or the point pattern \code{X} itself.

  The argument \code{X} must be a marked point pattern (object
  of class \code{"ppp"}, see \code{\link{ppp.object}}).
  The marks are allowed to be a vector or a data frame.

  \itemize{
    \item
    If \code{at="points"}, then for each point in \code{X},
    the algorithm finds the nearest \emph{other} point in \code{X},
    and extracts the mark attached to it.
    The result is a vector or data frame containing the marks
    of the neighbours of each point.
    \item
    If \code{at="pixels"} (the default), then for each pixel
    in a rectangular grid, the algorithm finds the nearest point in \code{X},
    and extracts the mark attached to it.
    The result is an image or a list of images containing the marks
    of the neighbours of each pixel.
    The pixel resolution is controlled by the arguments \code{\dots}
    passed to \code{\link{as.mask}}.
  }

  If the argument \code{k} is given, then the \code{k}-th nearest
  neighbour will be used.
}
\value{
  \emph{If \code{X} has a single column of marks:}
  \itemize{
    \item 
    If \code{at="pixels"} (the default), the result is
    a pixel image (object of class \code{"im"}). 
    The value at each pixel is the mark attached
    to the nearest point of \code{X}.
    \item
    If \code{at="points"}, the result is a vector or factor
    of length equal to the number of points in \code{X}.
    Entries are the mark values of the
    nearest neighbours of each point of \code{X}.
  }
  \emph{If \code{X} has a data frame of marks:}
  \itemize{
    \item 
    If \code{at="pixels"} (the default), the result is a named list of 
    pixel images (object of class \code{"im"}). There is one
    image for each column of marks. This list also belongs to
    the class \code{"solist"}, for which there is a plot method.
    \item
    If \code{at="points"}, the result is a data frame
    with one row for each point of \code{X},
    Entries are the mark values of the
    nearest neighbours of each point of \code{X}.
  }
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link{Smooth.ppp}},
  \code{\link{marktable}},
  \code{\link{nnwhich}}
}
\examples{
  plot(nnmark(ants))
  v <- nnmark(ants, at="points")
  v[1:10]
  plot(nnmark(finpines))
  vf <- nnmark(finpines, at="points")
  vf[1:5,]
}
\keyword{spatial}
\keyword{methods}
\keyword{smooth}

\name{unmark}
\alias{unmark}
\alias{unmark.ppp}
\alias{unmark.splitppp}
\alias{unmark.psp}
\alias{unmark.ppx}
\title{Remove Marks}
\description{
  Remove the mark information from a spatial dataset. 
}
\usage{
 unmark(X)
 \method{unmark}{ppp}(X)
 \method{unmark}{splitppp}(X)
 \method{unmark}{psp}(X)
 \method{unmark}{ppx}(X)
}
\arguments{
  \item{X}{A point pattern (object of class \code{"ppp"}),
    a split point pattern (object of class \code{"splitppp"}),
    a line segment pattern (object of class \code{"psp"})
    or a multidimensional space-time point pattern
    (object of class \code{"ppx"}).
  }
}
\value{
  An object of the same class as \code{X}
  with any mark information deleted.
}
\details{
  A `mark' is a value attached to each point in a spatial point pattern,
  or attached to each line segment in a line segment pattern, etc.
  
  The function \code{unmark} is a simple way to 
  remove the marks from such a dataset.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{psp.object}}
}
\examples{
  data(lansing)
  hicks <- lansing[lansing$marks == "hickory", ]
  \dontrun{
  plot(hicks)  # still a marked point pattern, but only 1 value of marks
  plot(unmark(hicks)) # unmarked
  }
}
\author{
  \adrian
  and \rolf
}
\keyword{spatial}
\keyword{manip}

\name{is.connected.ppp}  
\Rdversion{1.1}
\alias{is.connected.ppp}
\title{
  Determine Whether a Point Pattern is Connected
}
\description{
  Determine whether a point pattern is topologically connected
  when all pairs of points closer than a threshold distance are joined.
}
\usage{
\method{is.connected}{ppp}(X, R, \dots)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"}).
  }
  \item{R}{
    Threshold distance. Pairs of points closer than \code{R} units apart
    will be joined together.
  }
  \item{\dots}{
     Ignored.
  }
}
\details{
  The function \code{is.connected} is generic. This is the method for
  point patterns (objects of class \code{"ppp"}).

  The point pattern \code{X} is first converted into an abstract graph
  by joining every pair of points that lie closer than \code{R} units
  apart. Then the algorithm determines whether this graph is connected.

  That is, the result of \code{is.connected(X)} is \code{TRUE}
  if any point in \code{X} can be reached from any other point,
  by a series of steps between points of \code{X},
  each step being shorter than \code{R} units in length.
}
\value{
  A logical value.
}
\seealso{
  \code{\link{is.connected}}, 
  \code{\link{connected.ppp}}.
}
\examples{
  is.connected(redwoodfull, 0.1)
  is.connected(redwoodfull, 0.2)
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{math}

\name{pairdist.ppp}
\alias{pairdist.ppp}
\title{Pairwise distances}
\description{
  Computes the matrix of distances between all pairs of points
  in a point pattern.
}
\usage{
  \method{pairdist}{ppp}(X, \dots, periodic=FALSE, method="C", squared=FALSE)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{periodic}{
    Logical. Specifies whether to apply a periodic edge correction.
  }
  \item{method}{
    String specifying which method of calculation to use.
    Values are \code{"C"} and \code{"interpreted"}.
    Usually not specified.
  }
  \item{squared}{
    Logical. If \code{squared=TRUE}, the squared distances are
    returned instead (this computation is faster).
  }
}
\value{
  A square matrix whose \code{[i,j]} entry is the distance
  between the points numbered \code{i} and \code{j}.
}
\details{
  This is a method for the generic function \code{pairdist}.

  Given a point pattern \code{X} (an object of class \code{"ppp"}),
  this function computes the Euclidean distances between all pairs of
  points in \code{X}, and returns the matrix of distances.

  Alternatively if \code{periodic=TRUE} and the window containing \code{X} is a
  rectangle, then the distances will be computed in the `periodic'
  sense (also known as `torus' distance): opposite edges of the
  rectangle are regarded as equivalent.
  This is meaningless if the window is not a rectangle.

  If \code{squared=TRUE} then the \emph{squared} Euclidean distances
  \eqn{d^2} are returned, instead of the Euclidean distances \eqn{d}.
  The squared distances are faster to calculate, and are sufficient for
  many purposes (such as finding the nearest neighbour of a point).
  
  The argument \code{method} is not normally used. It is
  retained only for checking the validity of the software.
  If \code{method = "interpreted"} then the distances are
  computed using interpreted R code only. If \code{method="C"}
  (the default) then C code is used. The C code is somewhat faster.
}
\seealso{
  \code{\link{pairdist}},
  \code{\link{pairdist.default}},
  \code{\link{pairdist.psp}},
  \code{\link{crossdist}},
  \code{\link{nndist}},
  \code{\link{Kest}}
}
\examples{
   data(cells)
   d <- pairdist(cells)
   d <- pairdist(cells, periodic=TRUE)
   d <- pairdist(cells, squared=TRUE)
}
\author{Pavel Grabarnik
  \email{pavel.grabar@issp.serpukhov.su}
  and 
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{scanpp}
\alias{scanpp}
\title{Read Point Pattern From Data File} 
\description{
Reads a point pattern dataset from a text file.
}
\usage{
   scanpp(filename, window, header=TRUE, dir="", factor.marks=NULL, ...) 
}
\arguments{
  \item{filename}{
    String name of the file containing
    the coordinates of the points in the point pattern,
    and their marks if any.
    }
    \item{window}{
      Window for the point pattern. An object of class \code{"owin"}.
    }
    \item{header}{
      Logical flag indicating whether the first line of the
      file contains headings for the columns.
      Passed to \code{\link[utils]{read.table}}.
    }
    \item{dir}{
      String containing the path name of the directory
      in which \code{filename} is to be found.
      Default is the current directory.
    }
    \item{factor.marks}{
      Logical vector (or NULL) indicating whether marks are to be
      interpreted as factors. Defaults to \code{NULL} which means that
      strings will be interpreted as factors while numeric variables
      will not. See details.
    }
    \item{\dots}{
      Ignored.
    }
}
\value{
  A point pattern (an object of class \code{"ppp"},
  see \code{\link{ppp.object}}).
}
\details{
  This simple function reads a point pattern dataset from a file
  containing the cartesian coordinates of its points,
  and optionally the mark values for these points.

  The file identified by \code{filename} in directory \code{dir}
  should be a text file that can be read using \code{\link[utils]{read.table}}.
  Thus, each line of the file (except possibly the first line)
  contains data for one point in the
  point pattern. Data are arranged in columns. There should be either
  two columns (for an unmarked point pattern) or more columns (for a
  marked point pattern).

  If \code{header=FALSE} then the first two columns of data
  will be interpreted as the \eqn{x} and \eqn{y} coordinates
  of points. Remaining columns, if present, will be interpreted as
  containing the marks for these points.

  If \code{header=TRUE} then the first line of the file should contain
  string names for each of the columns of data. If there are columns
  named \code{x} and \code{y} then these will be taken as the
  cartesian coordinates, and any remaining columns will be taken as
  the marks. If there are no columns named \code{x} and \code{y}
  then the first and second columns will be taken as the cartesian
  coordinates.

  If a logical vector is provided for \code{factor.marks} the length
  should equal the number of mark columns (a shorter \code{factor.marks}
  is recycled to this length). This vector is then used to determine
  which mark columns should be interpreted as factors. Note: Strings will
  not be interpreted as factors if the corresponding entry in
  \code{factor.marks} is \code{FALSE}.

  Note that there is intentionally no default for \code{window}.
  The window of observation should be specified.
  If you really need to estimate the window, use the
  Ripley-Rasson estimator \code{\link{ripras}}.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{ppp}},
  \code{\link{as.ppp}},
  \code{\link{ripras}}
}
\author{
  \adrian
  and \rolf.
}
\examples{
  ##  files installed with spatstat, for demonstration
  d <- system.file("rawdata", "finpines", package="spatstat.data")
  if(nzchar(d)) {
    W <- owin(c(-5,5), c(-8,2))
    X <- scanpp("finpines.txt", dir=d, window=W)
    print(X)
  }
  d <- system.file("rawdata", "amacrine", package="spatstat.data")
  if(nzchar(d)) {
    W <- owin(c(0, 1060/662), c(0, 1))
    Y <- scanpp("amacrine.txt", dir=d, window=W, factor.marks=TRUE)
    print(Y)
  }
}
\keyword{spatial}
\keyword{IO}
 
 

\name{discretise}
\alias{discretise}
\title{
  Safely Convert Point Pattern Window to Binary Mask
}
\description{
  Given a point pattern, discretise its window by converting it to a
  binary pixel mask, adjusting the mask so that it still contains all
  the points.
}
\usage{
  discretise(X, eps = NULL, dimyx = NULL, xy = NULL)
}
\arguments{
  \item{X}{A point pattern (object of class \code{"ppp"}) to be converted.}
  \item{eps}{(optional) width and height of each pixel}
  \item{dimyx}{(optional) pixel array dimensions}
  \item{xy}{(optional) pixel coordinates}
}
\details{
  This function modifies the point pattern \code{X} by converting its
  observation window \code{Window(X)} to a binary pixel image (a window
  of type \code{"mask"}). It ensures that no points of \code{X} are
  deleted by the discretisation.

  The window is first discretised using \code{\link{as.mask}}. 
  It can happen that points of \code{X} that were inside the original
  window may fall outside the new mask.
  The \code{discretise} function corrects this by augmenting the mask
  (so that the mask includes any pixel that contains a point of the pattern).

  The arguments \code{eps}, \code{dimyx} and \code{xy}
  control the fineness of the pixel array. They are passed to
  \code{\link{as.mask}}.
  
  If \code{eps}, \code{dimyx} and \code{xy} are all absent or
  \code{NULL}, and if the window of \code{X} is of type \code{"mask"}
  to start with, then \code{discretise(X)} returns \code{X} unchanged.

  See \code{\link{as.mask}} for further details
  about the arguments \code{eps}, \code{dimyx},
  and \code{xy}, and the process of converting
  a window to one of type \code{mask}.
}
\section{Error checking}{
  Before doing anything, \code{discretise} checks that
  all the points of the pattern are actually
  inside the original window.  This is guaranteed to
  be the case if the pattern was constructed using \code{\link{ppp}}
  or \code{\link{as.ppp}}. However anomalies are possible if the
  point pattern was created or manipulated inappropriately.
  These will cause an error.
}
\value{
  A point pattern (object of class \code{"ppp"}),
  identical to \code{X}, except that
  its observation window has been converted to one
  of type \code{mask}. 
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{as.mask}}
}
\examples{
  data(demopat)
  X <- demopat
  plot(X, main="original pattern")
  Y <- discretise(X, dimyx=50)
  plot(Y, main="discretise(X)")
  stopifnot(npoints(X) == npoints(Y))

  # what happens if we just convert the window to a mask?
  W <- Window(X)
  M <- as.mask(W, dimyx=50)
  plot(M, main="window of X converted to mask")
  plot(X, add=TRUE, pch=16)
  plot(X[M], add=TRUE, pch=1, cex=1.5)
  XM <- X[M]
  cat(paste(npoints(X) - npoints(XM), "points of X lie outside M\n"))
}
\keyword{spatial}
\keyword{manip}

\name{extrapolate.psp}
\alias{extrapolate.psp}
\title{
  Extrapolate Line Segments to Obtain Infinite Lines
}
\description{
  Given a spatial pattern of line segments,
  extrapolate the segments to infinite lines.
}
\usage{
extrapolate.psp(x, \dots)
}
\arguments{
  \item{x}{
    Spatial pattern of line segments (object of class \code{"psp"}).
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  Each line segment in the pattern \code{x} is
  extrapolated to an infinite line, drawn through its two endpoints.
  The resulting pattern of infinite lines
  is returned as an object of class \code{"infline"}.

  If a segment's endpoints are identical
  (so that it has zero length)
  the resulting infinite line is vertical (i.e. parallel to the \eqn{y}
  coordinate axis).
}
\value{
  An object of class \code{"infline"} representing the pattern of
  infinite lines. See \code{\link{infline}} for details of
  structure.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{psp}},
  \code{\link{infline}}

  \code{\link{midpoints.psp}},
  \code{\link{lengths_psp}}
  \code{\link{angles.psp}},
  \code{\link{endpoints.psp}}.
}
\examples{
   X <- psp(runif(4), runif(4), runif(4), runif(4), window=owin())
   Y <- extrapolate.psp(X)
   plot(X, col=3, lwd=4)
   plot(Y, lty=3)
   Y
}
\keyword{spatial}
\keyword{math}

\name{dirichletAreas}
\alias{dirichletAreas}
\title{
  Compute Areas of Tiles in Dirichlet Tessellation
}
\description{
  Calculates the area of each tile in the 
  Dirichlet-Voronoi tessellation of a point pattern.
}
\usage{
dirichletAreas(X)
}
\arguments{
  \item{X}{
    Point pattern (object of class \code{"ppp"}).
  }
}
\details{
  This is an efficient algorithm to calculate the areas
  of the tiles in the Dirichlet-Voronoi tessellation.

  If the window of \code{X} is a binary pixel mask, the
  tile areas are computed by counting pixels. Otherwise the
  areas are computed exactly using analytic geometry.

  If any points of \code{X} are duplicated, the duplicates will
  have tile area zero.
}
\value{
  Numeric vector with one entry for each point of \code{X}.
}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
\code{\link{dirichlet}},
\code{\link{dirichletVertices}}
}
\examples{
 aa <- dirichletAreas(cells)
}
\keyword{spatial}
\keyword{math}
\keyword{manip}

\name{edges2vees}
\alias{edges2vees}
\title{
  List Dihedral Triples in a Graph
}
\description{
  Given a list of edges between vertices,
  compile a list of all \sQuote{vees} or dihedral triples
  formed by these edges.
}
\usage{
edges2vees(iedge, jedge, nvert=max(iedge, jedge), \dots,
                check=TRUE)
}
\arguments{
  \item{iedge,jedge}{
    Integer vectors, of equal length, specifying the edges.
  }
  \item{nvert}{
    Number of vertices in the network.
  }
  \item{\dots}{Ignored}
  \item{check}{Logical. Whether to check validity of input data.}
}
\details{
  Given a finite graph with \code{nvert} vertices and with edges
  specified by \code{iedge, jedge}, this low-level function
  finds all \sQuote{vees} or \sQuote{dihedral triples}
  in the graph, that is, all triples
  of vertices \code{(i,j,k)} where \code{i} and \code{j} are joined by
  an edge and \code{i} and \code{k} are joined by an edge. 

  The interpretation of \code{iedge, jedge} is that each successive
  pair of entries specifies an edge in the graph.
  The \eqn{k}th edge joins vertex \code{iedge[k]} to vertex \code{jedge[k]}.
  Entries of \code{iedge} and \code{jedge} must be integers
  from 1 to \code{nvert}.
}
\value{
  A 3-column matrix of integers, in which each row represents a triple
  of vertices, with the first vertex joined to the other two vertices.
}
\seealso{
  \code{\link{edges2triangles}}
}
\author{\adrian
  
  
  and \rolf
  
}
\examples{
 i <- c(1, 2, 5, 5, 1, 4, 2)
 j <- c(2, 3, 3, 1, 3, 2, 5)
 edges2vees(i, j)
}
\keyword{spatial}
\keyword{manip}

\name{rthin}
\alias{rthin}
\title{Random Thinning}
\description{
  Applies independent random thinning to a point pattern or segment pattern.
}
\usage{
rthin(X, P, \dots, nsim=1, drop=TRUE)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"} or \code{"lpp"}
    or \code{"pp3"} or \code{"ppx"}) or line segment pattern
    (object of class \code{"psp"}) that will be thinned.
  }
  \item{P}{
    Data giving the retention probabilities, i.e. the probability
    that each point or line in \code{X} will be retained.
    Either a single number, or a vector of numbers,
    or a \code{function(x,y)} in the \R language,
    or a function object (class \code{"funxy"} or \code{"linfun"}),
    or a pixel image (object of class \code{"im"} or \code{"linim"}).
  }
  \item{\dots}{
    Additional arguments passed to \code{P}, if it is a function.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  An object of the same kind as \code{X}
  if \code{nsim=1}, or a list of such objects if \code{nsim > 1}.
}
\details{
  In a random thinning operation, each point of the point pattern \code{X}
  is randomly either deleted or retained (i.e. not deleted).
  The result is a point pattern,
  consisting of those points of \code{X} that were retained.

  Independent random thinning means that the retention/deletion of each
  point is independent of other points.

  The argument \code{P} determines the probability of \bold{retaining}
  each point. It may be
  \describe{
    \item{a single number,}{so that each point will be retained with
      the same probability \code{P};
    }
    \item{a vector of numbers,}{so that the \code{i}th point of \code{X}
      will be retained with probability \code{P[i]};
    }
    \item{a function \code{P(x,y)},}{so that a point at a location
      \code{(x,y)} will be retained with probability \code{P(x,y)};
    }
    \item{an object of class \code{"funxy"} or \code{"linfun"},}{so that
    points in the pattern \code{X} will be retained with probabilities
    \code{P(X)};
    }
    \item{a pixel image,}{containing values of the retention probability
      for all locations in a region encompassing the point pattern.
    }
  }
  If \code{P} is a function \code{P(x,y)},
  it should be \sQuote{vectorised}, that is,
  it should accept vector arguments \code{x,y} and should yield a
  numeric vector of the same length. The function may have extra
  arguments which are passed through the \code{\dots} argument.
}
\section{Reproducibility}{
  The algorithm for random thinning was changed in \pkg{spatstat}
  version \code{1.42-3}. Set \code{spatstat.options(fastthin=FALSE)}
  to use the previous, slower algorithm, if it is desired to reproduce
  results obtained with earlier versions.
}
\examples{
  plot(redwood, main="thinning")
  
  # delete 20\% of points
  Y <- rthin(redwood, 0.8)
  points(Y, col="green", cex=1.4)

  # function
  f <- function(x,y) { ifelse(x < 0.4, 1, 0.5) }
  Y <- rthin(redwood, f)

  # pixel image
  Z <- as.im(f, Window(redwood))
  Y <- rthin(redwood, Z)

  # pattern on a linear network
  A <- runiflpp(30, simplenet)
  B <- rthin(A, 0.2)
  g <- function(x,y,seg,tp) { ifelse(y < 0.4, 1, 0.5) }
  B <- rthin(A, linfun(g, simplenet))

  # thin other kinds of patterns
  E <- rthin(osteo$pts[[1]], 0.6)
  L <- rthin(copper$Lines, 0.5)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{datagen}
\keyword{manip}

\name{boxx}
\Rdversion{1.1}
\alias{boxx}
\title{
  Multi-Dimensional Box
}
\description{
  Creates an object representing a multi-dimensional box.
}
\usage{
boxx(..., unitname = NULL)
}
\arguments{
  \item{\dots}{
    Dimensions of the box. Vectors of length 2.
  }
  \item{unitname}{
    Optional. Name of the unit of length. See Details.
  }
}
\details{
  This function creates an object representing
  a multi-dimensional rectangular parallelepiped (box)
  with sides parallel to the coordinate axes.

  The object can be used to specify the domain of a multi-dimensional
  point pattern (see \code{\link{ppx}}) and in various
  geometrical calculations (see \code{\link{volume.boxx}},
  \code{\link{diameter.boxx}}, \code{\link{eroded.volumes}}). 
  
  The optional argument \code{unitname} specifies the name
  of the unit of length. See \code{\link{unitname}}
  for valid formats.
}
\value{
  An object of class \code{"boxx"}. There is a print method for this class.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{ppx}},
  \code{\link{volume.boxx}},
  \code{\link{diameter.boxx}},
  \code{\link{eroded.volumes.boxx}}.
}
\examples{
    boxx(c(0,10),c(0,10),c(0,5),c(0,1), unitname=c("metre","metres"))
}
\keyword{spatial}
\keyword{datagen}

\name{range.fv}
\alias{range.fv}
\alias{max.fv}
\alias{min.fv}
\title{
  Range of Function Values
}
\description{
  Compute the range, maximum, or minimum of the
  function values in a summary function.
}
\usage{
  \method{range}{fv}(\dots, na.rm = TRUE, finite = na.rm)

  \method{max}{fv}(\dots, na.rm = TRUE, finite = na.rm)

  \method{min}{fv}(\dots, na.rm = TRUE, finite = na.rm)
}
\arguments{
  \item{\dots}{
    One or more function value tables (objects of class \code{"fv"}
    representing summary functions) or other data.
  }
  \item{na.rm}{
    Logical. Whether to ignore \code{NA} values.
  }
  \item{finite}{
    Logical. Whether to ignore values that are
    infinite, \code{NaN} or \code{NA}.
  }
}
\details{
  These are methods for the generic \code{\link[base]{range}},
  \code{\link[base]{max}} and \code{\link[base]{min}}.
  They compute the range, maximum, and minimum of the \emph{function} values
  that would be plotted on the \eqn{y} axis by default.

  For more complicated calculations, use \code{\link{with.fv}}.
}
\value{
  Numeric vector of length 2.
}
\seealso{
  \code{\link{with.fv}}
}
\examples{
   G <- Gest(cells)
   range(G)
   max(G)
   min(G)
}
\author{
  \adrian
  
  ,
  \rolf and \ege.
}
\keyword{spatial}
\keyword{math}

\name{nsegments}
\alias{nsegments}
\alias{nsegments.psp}
\title{
   Number of Line Segments in a Line Segment Pattern
}
\description{
   Returns the number of line segments in a line segment pattern.
}
\usage{
   nsegments(x)

   \method{nsegments}{psp}(x)
}
\arguments{
  \item{x}{
    A line segment pattern, i.e. an object of class \code{psp},
    or an object containing a linear network.
  }
}
\details{
   This function is generic, with methods for 
   classes \code{psp}, \code{linnet} and \code{lpp}.
}
\value{
   Integer.
}
\author{
  \spatstatAuthors
}
\seealso{
  \code{\link{npoints}()}, \code{\link{psp.object}()}
}
\examples{
nsegments(copper$Lines)
nsegments(copper$SouthLines)
}
\keyword{spatial}
\keyword{manip}

\name{reflect} 
\alias{reflect}
\alias{reflect.im}
\alias{reflect.default}
\title{Reflect In Origin}
\description{
  Reflects a geometrical object through the origin.
}
\usage{
  reflect(X)

  \method{reflect}{im}(X)

  \method{reflect}{default}(X)
}
\arguments{
  \item{X}{Any suitable dataset representing a two-dimensional
    object, such as a point pattern (object of class \code{"ppp"}),
    or a window (object of class \code{"owin"}).}
}
\value{
  Another object of the same type, representing the
  result of reflection.
}
\details{
  The object \code{X} is reflected through the origin.
  That is, each point in \code{X} with coordinates
  \eqn{(x,y)} is mapped to the position \eqn{(-x, -y)}.

  This is equivalent to applying the affine transformation with matrix
  \code{diag(c(-1,-1))}. It is also equivalent to rotation about the origin
  by 180 degrees.

  The command \code{reflect} is generic, with a method for
  pixel images and a default method.
}
\seealso{
  \code{\link{affine}},
  \code{\link{flipxy}}
}
\examples{
  plot(reflect(as.im(letterR)))
  plot(reflect(letterR), add=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{Extract.psp}
\alias{[.psp}
\title{Extract Subset of Line Segment Pattern}
\description{
  Extract a subset of a line segment pattern.
}
\usage{
  \method{[}{psp}(x, i, j, drop, \dots, fragments=TRUE)
}
\arguments{
  \item{x}{
    A two-dimensional line segment pattern.
    An object of class \code{"psp"}.
  }
  \item{i}{
    Subset index. Either a valid subset index in the usual \R sense,
   indicating which segments should be retained, or a window
   (an object of class \code{"owin"})
    delineating a subset of the original observation window. 
  }
  \item{j}{
   Redundant - included for backward compatibility.
  }
  \item{drop}{
    Ignored. Required for compatibility with generic function.
  }
  \item{\dots}{
    Ignored.
  }
  \item{fragments}{
    Logical value indicating whether to retain all pieces of line segments
    that intersect the new window (\code{fragments=TRUE}, the default)
    or to retain only those line segments
    that lie entirely inside the new window (\code{fragments=FALSE}).
  }
} 
\value{
  A line segment pattern (of class \code{"psp"}).
}
\details{
  These functions extract a designated subset of a line segment pattern.

  The function \code{[.psp} is a method for \code{\link{[}} for the
  class \code{"psp"}. It extracts a designated subset of a line segment pattern,
  either by ``\emph{thinning}''
  (retaining/deleting some line segments of a line segment pattern)
  or ``\emph{trimming}'' (reducing the window of observation
  to a smaller subregion and clipping the line segments to
  this boundary) or both.
 
  The pattern will be ``thinned''
  if  \code{subset} is specified. The line segments designated by \code{subset}
  will be retained. Here \code{subset} can be a numeric vector
  of positive indices (identifying the line segments to be retained),
  a numeric vector of negative indices (identifying the line segments
  to be deleted) or a logical vector of length equal to the number
  of line segments in the line segment pattern \code{x}. In the latter case,
  the line segments for which 
  \code{subset[i]=TRUE} will be retained, and the others
  will be deleted.
 
  The pattern will be ``trimmed''
  if \code{window} is specified. This should
  be an object of class \code{\link{owin}} specifying a window of observation
  to which the line segment pattern \code{x} will be
  trimmed. Line segments of \code{x} lying inside the new
  \code{window} will be retained unchanged. Line segments lying
  partially inside the new \code{window} and partially outside it
  will, by default, be clipped so that they lie entirely inside the window;
  but if \code{fragments=FALSE}, such segments will be removed.
 
  Both ``thinning'' and ``trimming'' can be performed together.
}
\seealso{
  \code{\link{psp.object}},
  \code{\link{owin.object}}
}
\examples{
    a <- psp(runif(20),runif(20),runif(20),runif(20), window=owin())
    plot(a)
  # thinning
    id <- sample(c(TRUE, FALSE), 20, replace=TRUE)
    b <- a[id]
    plot(b, add=TRUE, lwd=3)
 # trimming
    plot(a)
    w <- owin(c(0.1,0.7), c(0.2, 0.8))
    b <- a[w]
    plot(b, add=TRUE, col="red", lwd=2)
    plot(w, add=TRUE)
    u <- a[w, fragments=FALSE]
    plot(u, add=TRUE, col="blue", lwd=3)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{nearest.raster.point}
\alias{nearest.raster.point}
\title{Find Pixel Nearest to a Given Point}
\description{
  Given cartesian coordinates, find the nearest pixel.
}
\usage{
 nearest.raster.point(x,y,w, indices=TRUE)
}
\arguments{
  \item{x}{Numeric vector of \eqn{x} coordinates of any points}
  \item{y}{Numeric vector of \eqn{y} coordinates of any points}
  \item{w}{An image (object of class \code{"im"})
    or a binary mask window (an object of class \code{"owin"}
    of type \code{"mask"}).
  }
  \item{indices}{Logical flag indicating whether to return the
    row and column indices, or the actual \eqn{x,y} coordinates.
    }
}
\value{
  If \code{indices=TRUE}, a 
  list containing two vectors \code{rr} and \code{cc}
  giving row and column positions (in the image matrix).
  If \code{indices=FALSE}, a list containing
  vectors \code{x} and \code{y} giving actual coordinates
  of the pixels.
}
\details{
  The argument \code{w} should be either a pixel image
  (object of class \code{"im"}) or a window (an object of class
  \code{"owin"}, see \code{\link{owin.object}} for details)
  of type \code{"mask"}. 

  The arguments \code{x} and \code{y} should be numeric vectors
  of equal length. They are interpreted as the coordinates of
  points in space. For each point \code{(x[i], y[i])}, the function
  finds the nearest pixel in the grid of pixels for \code{w}.

  If \code{indices=TRUE},
  this function returns a list containing two vectors \code{rr} and
  \code{cc} giving row and column positions (in the image matrix).
  For the location \code{(x[i],y[i])} the nearest
  pixel is at row \code{rr[i]} and column \code{cc[i]} of
  the image.

  If \code{indices=FALSE}, the function returns a list containing
  two vectors \code{x} and \code{y} giving the actual coordinates
  of the pixels.
}
\seealso{
  \code{\link{owin.object}},
  \code{\link{as.mask}}
}
\examples{
  w <- owin(c(0,1), c(0,1), mask=matrix(TRUE, 100,100))  # 100 x 100 grid
  nearest.raster.point(0.5, 0.3, w)
  nearest.raster.point(0.5, 0.3, w, indices=FALSE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{tess}
\alias{tess}
\title{Create a Tessellation}
\description{
  Creates an object of class \code{"tess"} representing a tessellation
  of a spatial region.
}
\usage{
  tess(..., xgrid = NULL, ygrid = NULL, tiles = NULL, image = NULL,
            window=NULL, marks=NULL, keepempty=FALSE, unitname=NULL, check=TRUE)
}
\arguments{
  \item{\dots}{Ignored.}
  \item{xgrid,ygrid}{Cartesian coordinates of vertical and
    horizontal lines determining a grid of rectangles.
    Incompatible with other arguments.
  }
  \item{tiles}{List of tiles in the tessellation.
    A list, each of whose elements is a window
    (object of class \code{"owin"}). Incompatible with other arguments.
  }
  \item{image}{
    Pixel image which specifies the tessellation.
    Incompatible with other arguments.
  }
  \item{window}{
    Optional.
    The spatial region which is tessellated (i.e. the union of all the tiles).
    An object of class \code{"owin"}.
  }
  \item{marks}{
    Optional vector or data frame of marks associated with the tiles.
  }
  \item{keepempty}{
    Logical flag indicating whether empty tiles should be retained
    or deleted.
  }
  \item{unitname}{
    Optional. Name of unit of length. Either a single character string,
    or a vector of two character strings giving the
    singular and plural forms, respectively.
    If this argument is missing or \code{NULL},
    information about the unitname will be
    extracted from the other arguments.
    If this argument is given, it overrides any other information
    about the unitname.
  }
  \item{check}{
    Logical value indicating whether to check the validity of the
    input data. It is strongly recommended to use the default
    value \code{check=TRUE}.
  }
}
\details{
  A tessellation is a collection of disjoint spatial regions
  (called \emph{tiles}) that fit together to form a larger spatial
  region. This command creates an object of class \code{"tess"} that
  represents a tessellation.

  Three types of tessellation are supported:
  \describe{
    \item{rectangular:}{
      tiles are rectangles, with sides parallel to the \code{x} and
      \code{y} axes. They may or may not have equal size and shape.
      The arguments \code{xgrid} and \code{ygrid} determine
      the positions of the vertical and horizontal grid lines,
      respectively. (See \code{\link{quadrats}} for another way to do this.)
    }
    \item{tile list:}{
      tiles are arbitrary spatial regions.
      The argument \code{tiles} is a list of these tiles,
      which are objects of class \code{"owin"}.
    }
    \item{pixel image:}{
      Tiles are subsets of a fine grid of pixels.
      The argument \code{image} is a pixel image (object of class
      \code{"im"}) with factor values. Each level of the factor
      represents a different tile of the tessellation. The pixels that
      have a particular value of the factor constitute a tile. 
    }
  }

  The optional argument \code{window} specifies the spatial region
  formed by the union of all the tiles. In other words it specifies the
  spatial region that is divided into tiles by the tessellation.
  If this argument is missing or \code{NULL}, it will be determined by
  computing the set union of all the tiles. This is a time-consuming
  computation. For efficiency it is advisable to specify the window.
  Note that the validity of the window will not be checked.

  Empty tiles may occur, either because one of the entries in the list
  \code{tiles} is an empty window, or because one of the levels of the
  factor-valued pixel image \code{image} does not occur in the pixel data.
  When \code{keepempty=TRUE}, empty tiles are permitted. 
  When \code{keepempty=FALSE} (the default), tiles are not allowed to be
  empty, and any empty tiles will be removed from the tessellation.

  There are methods for \code{print}, \code{plot}, \code{[} and \code{[<-}
  for tessellations. Use \code{\link{tiles}} to extract the list of
  tiles in a tessellation, \code{\link{tilenames}} to extract the names
  of the tiles, and \code{\link{tile.areas}} to compute their
  areas.

  The tiles may have marks, which can be extracted by
  \code{\link{marks.tess}} and changed by \code{\link{marks<-.tess}}.

  Tessellations can be used to classify the points of
  a point pattern, in \code{\link{split.ppp}}, \code{\link{cut.ppp}} and
  \code{\link{by.ppp}}.

  To construct particular tessellations, see
  \code{\link{quadrats}}, \code{\link{hextess}}, 
  \code{\link{dirichlet}}, \code{\link{delaunay}},
  \code{\link{venn.tess}},
  \code{\link{polartess}},
  \code{\link{quantess}}
  and \code{\link{rpoislinetess}}.
}
\value{
  An object of class \code{"tess"} representing the tessellation.
}
\seealso{
  \code{\link{marks.tess}},
  \code{\link{plot.tess}},
  \code{\link{[.tess}},
  \code{\link{as.tess}},
  \code{\link{tiles}},
  \code{\link{intersect.tess}},
  \code{\link{split.ppp}},
  \code{\link{cut.ppp}},
  \code{\link{by.ppp}},
  \code{\link{bdist.tiles}},
  \code{\link{tile.areas}},
  \code{\link{as.function.tess}}.

  To construct particular tessellations, see
  \code{\link{quadrats}}, \code{\link{hextess}},
  \code{\link{venn.tess}},
  \code{\link{polartess}},
  \code{\link{dirichlet}}, \code{\link{delaunay}}, \code{\link{quantess}}
  and \code{\link{rpoislinetess}}.

  To divide space into pieces containing equal
  amounts of stuff, use \code{\link{quantess}}.

  To convert a tessellation to a function, for use as a spatial covariate
  (associating a numerical value with each tile of the tessellation) use
  \code{\link{as.function.tess}}.
}
\examples{
  A <- tess(xgrid=0:4,ygrid=0:4)
  A
  plot(A)
  B <- A[c(1, 2, 5, 7, 9)]
  B
  v <- as.im(function(x,y){factor(round(5 * (x^2 + y^2)))}, W=owin())
  levels(v) <- letters[seq(length(levels(v)))]
  E <- tess(image=v)
  plot(E)
  G <- tess(image=v, marks=toupper(levels(v)), unitname="km")
  G
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{datagen}

\name{Extract.hyperframe}
\alias{[.hyperframe}
\alias{[<-.hyperframe}
\alias{$.hyperframe}
\alias{$<-.hyperframe}
\title{Extract or Replace Subset of Hyperframe}
\description{
  Extract or replace a subset of a hyperframe.
}
\usage{
  \method{[}{hyperframe}(x, i, j, drop, strip=drop, \dots)
  \method{[}{hyperframe}(x, i, j) <- value
  \method{$}{hyperframe}(x, name)
  \method{$}{hyperframe}(x, name) <- value
}
\arguments{
  \item{x}{
    A hyperframe (object of class \code{"hyperframe"}).
  }
  \item{i,j}{
    Row and column indices.
  }
  \item{drop,strip}{
     Logical values indicating what to do when the
     hyperframe has only one row or column.
     See Details.
  }
  \item{\dots}{
    Ignored. 
  }
  \item{name}{
    Name of a column of the hyperframe.
  }
  \item{value}{
    Replacement value for the subset. A hyperframe
    or (if the subset is a single column) a list or an atomic vector.
  }
}
\value{
  A hyperframe (of class \code{"hyperframe"}).
}
\details{
  These functions extract a designated subset of a hyperframe,
  or replace the designated subset with another hyperframe.

  The function \code{[.hyperframe} is a method for the subset operator
  \code{\link{[}} for the
  class \code{"hyperframe"}. It extracts the subset of \code{x}
  specified by the row index \code{i} and column index \code{j}.

  The argument \code{drop} determines whether the array structure
  will be discarded if possible. The argument \code{strip} determines
  whether the list structure in a row or column or cell will be discarded if
  possible.
  If \code{drop=FALSE} (the default), the return value is always
  a hyperframe or data frame.
  If \code{drop=TRUE}, and if the selected subset has only one row,
  or only one column, or both, then
  \itemize{
    \item{
       if \code{strip=FALSE}, the result is a
       list, with one entry for each array cell that was selected.
    }
    \item{
       if \code{strip=TRUE},
       \itemize{
         \item if the subset has one row containing several columns,
         the result is a list or (if possible) an atomic vector;
         \item if the subset has one column containing several rows,
         the result is a list or (if possible) an atomic vector;
         \item if the subset has exactly one row and exactly one column,
         the result is the object (or atomic value)
         contained in this row and column.
       }
    }
  }
  
  The function \code{[<-.hyperframe} is a method for the
  subset replacement operator \code{\link{[<-}} for the
  class \code{"hyperframe"}. It replaces the designated
  subset with the hyperframe \code{value}.
  The subset of \code{x} to be replaced is designated by
  the arguments \code{i} and \code{j} as above.
  The replacement \code{value} should be a hyperframe with the
  appropriate dimensions, or (if the specified subset is a single
  column) a list of the appropriate length.

  The function \code{$.hyperframe} is a method for \code{\link{$}}
  for hyperframes. It extracts the relevant column of the hyperframe.
  The result is always a list (i.e. equivalent to using
  \code{[.hyperframe} with \code{strip=FALSE}).

  The function \code{$<-.hyperframe} is a method for \code{\link{$<-}}
  for hyperframes. It replaces the relevant column of the hyperframe.
  The replacement value should be a list of the appropriate length.
}
\seealso{
  \code{\link{hyperframe}}
}
\examples{
  h <- hyperframe(X=list(square(1), square(2)), Y=list(sin, cos))
  h
  h[1, ]
  h[1, ,drop=TRUE]
  h[ , 1]
  h[ , 1, drop=TRUE]
  h[1,1]
  h[1,1,drop=TRUE]
  h[1,1,drop=TRUE,strip=FALSE]
  h[1,1] <- list(square(3))
  # extract column
  h$X
  # replace existing column
  h$Y <- list(cells, cells)
  # add new column
  h$Z <- list(cells, cells)
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{rpoispp3}
\alias{rpoispp3}
\title{
  Generate Poisson Point Pattern in Three Dimensions
}
\description{
  Generate a random three-dimensional point pattern
  using the homogeneous Poisson process.
}
\usage{
rpoispp3(lambda, domain = box3(), nsim=1, drop=TRUE)
}
\arguments{
  \item{lambda}{
    Intensity of the Poisson process.
    A single positive number.
  }
  \item{domain}{
    Three-dimensional box in which the process should be generated.
    An object of class \code{"box3"}.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  If \code{nsim = 1} and \code{drop=TRUE}, a point pattern in
  three dimensions (an object of class \code{"pp3"}).
  If \code{nsim > 1}, a list of such point patterns.
}
\details{
  This function generates a realisation
  of the homogeneous Poisson process in three dimensions,
  with intensity \code{lambda} (points per unit volume).
  
  The realisation is generated inside the three-dimensional region
  \code{domain} which currently must be a rectangular box (object of
  class \code{"box3"}).
}
\note{
  The intensity \code{lambda} is the expected number of points
  \emph{per unit volume}. 
}
\seealso{
  \code{\link{runifpoint3}}, 
  \code{\link{pp3}}, 
  \code{\link{box3}}
}
\examples{
   X <- rpoispp3(50)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{where.max}
\alias{where.max}
\alias{where.min}
\title{
  Find Location of Maximum in a Pixel Image
}
\description{
  Finds the spatial location(s) where a given pixel image
  attains its maximum or minimum value.
}
\usage{
  where.max(x, first = TRUE)
  where.min(x, first = TRUE)
}
\arguments{
  \item{x}{
    A pixel image (object of class \code{"im"}).
  }
  \item{first}{
    Logical value. If \code{TRUE} (the default), then only one location
    will be returned. If \code{FALSE}, then all locations where the
    maximum is achieved will be returned.
  }
}
\details{
  This function finds the spatial location or locations where the
  pixel image \code{x} attains its maximum or minimum value.
  The result is a point pattern giving the locations.

  If \code{first=TRUE} (the default), then only one location will
  be returned, namely the location with the smallest \eqn{y} coordinate
  value which attains the maximum or minimum.
  This behaviour is analogous to the functions
  \code{\link[base]{which.min}} and
  \code{\link[base:which.min]{which.max}}.

  If \code{first=FALSE}, then the function returns
  the locations of all pixels where the
  maximum (or minimum) value is attained. This could be a large
  number of points.
}
\value{
  A point pattern (object of class \code{"ppp"}).
}
\author{
  \adrian
}
\seealso{
  \code{\link{Summary.im}} for computing the minimum and maximum
  of pixel values;
  \code{\link{eval.im}} and \code{\link{Math.im}} for mathematical
  expressions involving images; \code{\link{solutionset}} for finding
  the set of pixels where a statement is true.
}
\examples{
   D <- distmap(letterR, invert=TRUE)
   plot(D)
   plot(where.max(D), add=TRUE, pch=16, cols="green")
}
\keyword{spatial}
\keyword{math}

\name{hotrod}
\alias{hotrod}
\title{
  Heat Kernel for a One-Dimensional Rod
}
\description{
  Calculate values of the heat kernel on a one-dimensional rod.
  The ends of the rod may be assumed to be insulated,
  or absorbing.
}
\usage{
hotrod(len, xsource, xquery, sigma, ends=c("insulated", "absorbing"), nmax=20)
}
\arguments{
  \item{len}{
    Length of the rod. A single number or numeric vector.
  }
  \item{xsource}{
    Positions of the source points, from the left end of the rod
    (in the same distance units as \code{len}).
    A single number or numeric vector.
  }
  \item{xquery}{
    Positions of the query points, from the left end of the rod
    (in the same distance units as \code{len}).
    A single number or numeric vector.
  }
  \item{sigma}{
    Bandwidth for kernel. A single number or a numeric vector.
  }
  \item{ends}{
    Character string (partially matched) specifying whether the
    ends of the rod are assumed to be insulated or absorbing.
  }
  \item{nmax}{
    Number of terms in the infinite sum to use.
    A single integer or an integer vector.
  }
}
\details{
  Computes the heat kernel as an infinite sum.
}
\value{
  Number or numeric vector.
}
\author{
  Greg McSwiggan and \adrian.
}
\examples{
  curve(hotrod(1, 0.1, x, 0.7))

  # check it's a probability density
  f <- function(x) hotrod(1, 0.1, x, 0.7)
  integrate(f, 0, 1)

  ## absorbing ends
  curve(hotrod(1, 0.1, x, 0.7, ends="a"))
}
\keyword{math}

\name{summary.solist}
\alias{summary.solist}
\title{Summary of a List of Spatial Objects}
\description{
  Prints a useful summary of each entry in a list of two-dimensional
  spatial objects.
}
\usage{
 \method{summary}{solist}(object, \dots)
}
\arguments{
  \item{object}{
    An object of class \code{"solist"}.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  This is a method for the generic function \code{\link{summary}}.

  An object of the class \code{"solist"} is effectively a list
  of two-dimensional spatial datasets.
  See \code{\link{solist}}.

  This function extracts a useful summary of each of the datasets.
}
\seealso{
  \code{\link{solist}},
  \code{\link{summary}},
  \code{\link{plot.solist}}
}
\examples{
  x <- solist(cells, japanesepines, redwood)
  summary(x)
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{methods}

\name{flipxy} 
\alias{flipxy}
\alias{flipxy.owin}
\alias{flipxy.ppp}
\alias{flipxy.psp}
\alias{flipxy.im}
\title{Exchange X and Y Coordinates}
\description{
  Exchanges the \eqn{x} and \eqn{y} coordinates in a spatial dataset.
}
\usage{
 flipxy(X)
 \method{flipxy}{owin}(X)
 \method{flipxy}{ppp}(X)
 \method{flipxy}{psp}(X)
 \method{flipxy}{im}(X)
}
\arguments{
  \item{X}{Spatial dataset. An object of class
    \code{"owin"}, \code{"ppp"}, \code{"psp"} or \code{"im"}.
  }
}
\value{
  Another object of the same type, representing the
  result of swapping the \eqn{x} and \eqn{y} coordinates.
}
\details{
  This function swaps the \eqn{x} and \eqn{y} coordinates of a spatial
  dataset. This could also be performed using the command \code{\link{affine}},
  but \code{flipxy} is faster.
  
  The function \code{\link{flipxy}} is generic, with methods
  for the classes of objects listed above.
}
\seealso{
  \code{\link{affine}},
  \code{\link{reflect}},
  \code{\link{rotate}},
  \code{\link{shift}}
}
\examples{
  data(cells)
  X <- flipxy(cells)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{summary.distfun}
\alias{summary.distfun}
\alias{summary.funxy}
\title{
  Summarizing a Function of Spatial Location
}
\description{
  Prints a useful summary of a function of spatial location.
}
\usage{
  \method{summary}{distfun}(object, \dots)

  \method{summary}{funxy}(object, \dots)
}
\arguments{
  \item{object}{
    An object of class \code{"distfun"} or \code{"funxy"}
    representing a function of spatial coordinates.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} controlling the
    pixel resolution used to compute the summary.
  }
}
\details{
  These are the \code{\link[base]{summary}} methods
  for the classes \code{"funxy"} and \code{"distfun"}.
  
  An object of class \code{"funxy"} represents a function
  of spatial location, defined in a particular region of space.
  This includes objects of the special class \code{"distfun"} which represent
  distance functions.

  The \code{summary} method computes a summary of the function values.
  The function is evaluated on a grid of locations using
  \code{\link{as.im}} and numerical values at these locations
  are summarised using \code{\link{summary.im}}. The pixel resolution
  for the grid of locations is determined by the arguments
  \code{\dots} which are passed to \code{\link{as.mask}}.
}
\value{
  For \code{summary.funxy} the result is
  an object of class \code{"summary.funxy"}.
  For \code{summary.distfun} the result is
  an object of class \code{"summary.distfun"}.
  There are \code{print} methods for these classes.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{distfun}}, \code{\link{funxy}}
}
\examples{
   f <- function(x,y) { x^2 + y^2 - 1} 
   g <- funxy(f, square(2))
   summary(g)

   summary(distfun(cells))
   summary(distfun(cells), dimyx=256)
}
\keyword{spatial}
\keyword{methods}

\name{boundingbox}
\alias{boundingbox}
\alias{boundingbox.default}
\alias{boundingbox.im}
\alias{boundingbox.owin}
\alias{boundingbox.ppp}
\alias{boundingbox.psp}
\alias{boundingbox.lpp}
\alias{boundingbox.linnet}
\alias{boundingbox.solist}
\title{
  Bounding Box of a Window, Image, or Point Pattern
}
\description{
  Find the smallest rectangle containing a given window(s),
  image(s) or point pattern(s).
}
\usage{
boundingbox(\dots)

\method{boundingbox}{default}(\dots)

\method{boundingbox}{im}(\dots)

\method{boundingbox}{owin}(\dots)

\method{boundingbox}{ppp}(\dots)

\method{boundingbox}{psp}(\dots)

\method{boundingbox}{lpp}(\dots)

\method{boundingbox}{linnet}(\dots)

\method{boundingbox}{solist}(\dots)
}
\arguments{
  \item{\dots}{One or more windows (objects of class \code{"owin"}),
    pixel images (objects of class \code{"im"}) or
    point patterns (objects of class \code{"ppp"} or \code{"lpp"})
    or line segment patterns (objects of class \code{"psp"})
    or linear networks (objects of class \code{"linnet"})
    or any combination of such objects.
    Alternatively, the argument may be a list of such objects,
    of class \code{"solist"}.
  }
}
\details{
  This function finds the smallest rectangle (with sides parallel to
  the coordinate axes) that contains all the given objects.

  For a window (object of class \code{"owin"}), the bounding box
  is the smallest rectangle that contains all the vertices of the
  window (this is generally smaller than the enclosing frame,
  which is returned by \code{\link{as.rectangle}}).

  For a point pattern (object of class \code{"ppp"} or \code{"lpp"}),
  the bounding box
  is the smallest rectangle that contains all the points of the pattern.
  This is usually smaller than the bounding box of the window of the
  point pattern.

  For a line segment pattern (object of class \code{"psp"})
  or a linear network (object of class \code{"linnet"}), the
  bounding box is the smallest rectangle that contains all endpoints
  of line segments.
  
  For a pixel image (object of class \code{"im"}), the image will
  be converted to a window using \code{\link{as.owin}},
  and the bounding box of this window is obtained.
  
  If the argument is a list of several objects, then
  this function finds the smallest rectangle that contains
  all the bounding boxes of the objects.
}
\value{
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{as.rectangle}}
}
\examples{
  w <- owin(c(0,10),c(0,10), poly=list(x=c(1,2,3,2,1), y=c(2,3,4,6,7)))
  r <- boundingbox(w)
  # returns rectangle [1,3] x [2,7]

  w2 <- unit.square()
  r <- boundingbox(w, w2)
  # returns rectangle [0,3] x [0,7]
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{utilities}

\name{intersect.owin}
\alias{intersect.owin}
\alias{union.owin}
\alias{setminus.owin}
\title{Intersection, Union or Set Subtraction of Windows}
\description{
  Yields the intersection, union or set subtraction of windows.
}
\usage{
   intersect.owin(\dots, fatal=FALSE, p)
   union.owin(\dots, p)
   setminus.owin(A, B, \dots, p)
}
\arguments{
  \item{A,B}{Windows (objects of class \code{"owin"}).}
  \item{\dots}{
    Windows,
    or arguments passed to \code{\link{as.mask}}
    to control the discretisation.
  }
  \item{fatal}{Logical.
    Determines what happens if the intersection is empty.
  }
  \item{p}{
    Optional list of parameters passed to
    \code{\link[polyclip]{polyclip}} to control the
    accuracy of polygon geometry.
  }
}
\value{
  A window (object of class \code{"owin"})
  or possibly \code{NULL}.
}
\details{
  The function \code{intersect.owin} computes the intersection between
  the windows given in \code{\dots}, while 
  \code{union.owin} computes their union.
  The function \code{setminus.owin} computes the intersection of
  \code{A} with the complement of \code{B}.

  For \code{intersect.owin} and \code{union.owin}, 
  the arguments \code{\dots} must be either
  \itemize{
    \item window objects of class \code{"owin"},
    \item data that can be coerced to this class by
    \code{\link{as.owin}}),
    \item lists of windows, of class \code{"solist"},
    \item named arguments of \code{\link{as.mask}} to control
    the discretisation if required.
  }
  For \code{setminus.owin}, the arguments \code{\dots}
  must be named arguments of \code{\link{as.mask}}.

  If the intersection is empty, then if \code{fatal=FALSE}
  the result is an empty window or \code{NULL},
  while if \code{fatal=TRUE} an error occurs.
}

\author{
\spatstatAuthors.
}
\seealso{
  \code{\link{is.subset.owin}},
  \code{\link{overlap.owin}},
  \code{\link{is.empty}},
  \code{\link{boundingbox}},
  \code{\link{owin.object}}
}
\examples{
# rectangles
   u <- unit.square()
   v <- owin(c(0.5,3.5), c(0.4,2.5))
# polygon
   data(letterR)
# mask
   m <- as.mask(letterR)

# two rectangles
   intersect.owin(u, v) 
   union.owin(u,v)
   setminus.owin(u,v)

# polygon and rectangle
   intersect.owin(letterR, v)
   union.owin(letterR,v)
   setminus.owin(letterR,v)

# mask and rectangle
   intersect.owin(m, v)
   union.owin(m,v)
   setminus.owin(m,v)

# mask and polygon
   p <- rotate(v, 0.2)
   intersect.owin(m, p)
   union.owin(m,p)
   setminus.owin(m,p)

# two polygons
   A <- letterR
   B <- rotate(letterR, 0.2)
   plot(boundingbox(A,B), main="intersection")
   w <- intersect.owin(A, B)
   plot(w, add=TRUE, col="lightblue")
   plot(A, add=TRUE)
   plot(B, add=TRUE)

   plot(boundingbox(A,B), main="union")
   w <- union.owin(A,B)
   plot(w, add=TRUE, col="lightblue")   
   plot(A, add=TRUE)
   plot(B, add=TRUE)

   plot(boundingbox(A,B), main="set minus")
   w <- setminus.owin(A,B)
   plot(w, add=TRUE, col="lightblue")   
   plot(A, add=TRUE)
   plot(B, add=TRUE)

# intersection and union of three windows
   C <- shift(B, c(0.2, 0.3))
   plot(union.owin(A,B,C))
   plot(intersect.owin(A,B,C))
}
\keyword{spatial}
\keyword{math}

\name{dirichletWeights}
\alias{dirichletWeights}
\title{Compute Quadrature Weights Based on Dirichlet Tessellation}
\description{
  Computes quadrature weights for a given set of points,
  using the areas of tiles in the Dirichlet tessellation.
}
\usage{
 dirichletWeights(X, window=NULL, exact=TRUE, \dots)
}
\arguments{
  \item{X}{Data defining a point pattern.}
  \item{window}{Default window for the point pattern}
  \item{exact}{Logical value. If \code{TRUE}, compute exact areas
    using the package \code{deldir}. If \code{FALSE}, compute
    approximate areas using a pixel raster.
  }
  \item{\dots}{
    Ignored.
  }
}
\value{
  Vector of nonnegative weights for each point in \code{X}.
}
\details{
  This function computes a set of quadrature weights
  for a given pattern of points
  (typically comprising both ``data'' and `dummy'' points).
  See \code{\link{quad.object}} for an explanation of quadrature
  weights and quadrature schemes.

  The weights are computed using the Dirichlet tessellation.
  First \code{X} and (optionally) \code{window} are converted into a
  point pattern object. Then the Dirichlet tessellation of the points
  of \code{X} is computed.
  The weight attached to a point of \code{X} is the area of
  its Dirichlet tile (inside the window \code{Window(X)}).

  If \code{exact=TRUE} the Dirichlet tessellation is computed exactly
  by the Lee-Schachter algorithm using the package \code{deldir}.
  Otherwise a pixel raster approximation is constructed and the areas
  are approximations to the true weights. In all cases the sum of the
  weights is equal to the area of the window.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{gridweights}}
}
\examples{
  Q <- quadscheme(runifpoispp(10))
  X <- as.ppp(Q) # data and dummy points together
  w <- dirichletWeights(X, exact=FALSE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{utilities}

\name{summary.im}
\alias{summary.im}
\alias{print.summary.im}
\title{Summarizing a Pixel Image}
\description{
  \code{summary} method for class \code{"im"}.
}
\usage{
  \method{summary}{im}(object, \dots)
  \method{print}{summary.im}(x, \dots)
}
\arguments{
  \item{object}{A pixel image.}
  \item{\dots}{Ignored.}
  \item{x}{Object of class \code{"summary.im"} as returned by
    \code{summary.im}.
  }
}
\details{
  This is a method for the generic \code{\link{summary}}
  for the class \code{"im"}. An object of class \code{"im"}
  describes a pixel image. See \code{\link{im.object}})
  for details of this class. 
  
  \code{summary.im} extracts information about the pixel image,
  and \code{print.summary.im} prints this information in a
  comprehensible format.

  In normal usage, \code{print.summary.im} is invoked implicitly
  when the user calls \code{summary.im} without assigning its value
  to anything. See the examples.

  The information extracted by \code{summary.im} includes
  \describe{
    \item{range}{The range of the image values.}
    \item{mean}{The mean of the image values.}
    \item{integral}{The ``integral'' of the image values,
      calculated as the sum of the image values
      multiplied by the area of one pixel.}
    \item{dim}{The dimensions of the pixel array:
      \code{dim[1]} is the number of rows in the array,
      corresponding to the \bold{y} coordinate.}
  }
}
\value{
  \code{summary.im} returns an object of class \code{"summary.im"},
  while \code{print.summary.im} returns \code{NULL}.
}
\seealso{
  \code{\link{mean.im}}, \code{\link{integral.im}}, \code{\link{anyNA.im}}
}
\examples{
  # make an image
  X <- as.im(function(x,y) {x^2}, unit.square())
  # summarize it
  summary(X)
  # save the summary
  s <- summary(X)
  # print it
  print(X)
  s
  # extract stuff
  X$dim
  X$range
  X$integral
}
\author{\adrian
  and \rolf
}
\keyword{spatial}
\keyword{methods}

\name{npoints}
\alias{npoints}
\alias{npoints.ppp}
\alias{npoints.pp3}
\alias{npoints.ppx}
\title{Number of Points in a Point Pattern}
\description{
  Returns the number of points in a point pattern of any kind.
}
\usage{
  npoints(x)
  \method{npoints}{ppp}(x)
  \method{npoints}{pp3}(x)
  \method{npoints}{ppx}(x)
}
\arguments{
  \item{x}{
    A point pattern (object of class \code{"ppp"},
    \code{"pp3"}, \code{"ppx"} or some other suitable class).
  }
}
\value{
  Integer.
}
\details{
  This function returns the number of points in
  a point pattern. The function \code{npoints} is generic
  with methods for the classes \code{"ppp"}, \code{"pp3"},
  \code{"ppx"} and possibly other classes.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{print.pp3}},
  \code{\link{print.ppx}}.
}
\examples{
   data(cells)
   npoints(cells)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{is.linim}
\alias{is.linim}
\title{Test Whether an Object is a Pixel Image on a Linear Network}
\description{
  Tests whether its argument is a pixel image on a linear network
  (object of class \code{"linim"}).
}
\usage{
is.linim(x)
}
\arguments{
  \item{x}{Any object.}
}
\details{
  This function tests whether the argument \code{x} is a
  pixel image on a linear network
  (object of class \code{"linim"}). 
  
  The object is determined to be an image if it inherits from
  class \code{"linim"}.
}
\value{
  \code{TRUE} if \code{x} is a pixel image on a linear network,
  otherwise \code{FALSE}.
}

\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{split.ppx}
\alias{split.ppx}
\title{Divide Multidimensional Point Pattern into Sub-patterns}
\description{
  Divides a multidimensional point pattern into several sub-patterns,
  according to their marks, or according to any user-specified grouping.
}
\usage{
  \method{split}{ppx}(x, f = marks(x), drop=FALSE, un=NULL, \dots)
}
\arguments{
  \item{x}{
    A multi-dimensional point pattern.
    An object of class \code{"ppx"}.
  }
  \item{f}{
    Data determining the grouping. Either a factor, a logical vector,
    or the name of one of the columns of marks.
  }
  \item{drop}{
    Logical. Determines whether empty groups will
    be deleted.
  }
  \item{un}{
    Logical. Determines whether the resulting subpatterns will be unmarked
    (i.e. whether marks will be removed	from the points in each
    subpattern). 
  }
  \item{\dots}{
    Other arguments are ignored.
  }
} 
\value{
  A list of point patterns.
  The components of the list are named by the levels of \code{f}.
  The list also has the class \code{"splitppx"} and \code{"anylist"}.
}
\details{
  The generic command \code{\link[base]{split}} allows a dataset to be separated
  into subsets according to the value of a grouping variable.
  
  The function \code{split.ppx} is a method for the generic
  \code{\link[base]{split}} for the class \code{"ppx"} of multidimensional
  point patterns. It divides up the points of the point pattern \code{x}
  into several sub-patterns according to the values of \code{f}.
  The result is a list of point patterns.
  
  The argument \code{f} may be
  \itemize{
    \item
    a factor, of length equal to the number of points in \code{x}.
    The levels of \code{f}
    determine the destination of each point in \code{x}.
    The \code{i}th point of \code{x} will be placed in the sub-pattern
    \code{split.ppx(x)$l} where \code{l = f[i]}.
    \item
    a character string, matching the name of one of the columns of
    marks, if \code{marks(x)} is a data frame. This column should
    be a factor.
  }
  If \code{f} is missing, then it will be determined by the
  marks of the point pattern. The pattern \code{x} can be either
  \itemize{
    \item 
    a multitype point pattern
    (a marked point pattern whose marks vector is a factor).
    Then \code{f} is taken to be the marks vector.
    The effect is that the points of each type
    are separated into different point patterns.
    \item 
    a marked point pattern with a data frame or hyperframe
    of marks, containing at least one
    column that is a factor. The first such column will be used to
    determine the splitting factor \code{f}.
  }

  Some of the sub-patterns created by the split
  may be empty. If \code{drop=TRUE}, then empty sub-patterns will
  be deleted from the list. If \code{drop=FALSE} then they are retained.

  The argument \code{un} determines how to handle marks 
  in the case where \code{x} is a marked point pattern.
  If \code{un=TRUE} then the marks of the 
  points will be discarded when they are split into groups,
  while if \code{un=FALSE} then the marks will be retained.

  If \code{f} and \code{un} are both missing,
  then the default is \code{un=TRUE} for multitype point patterns
  and \code{un=FALSE} for marked point patterns with a data frame of
  marks.
  
  The result of \code{split.ppx} has class \code{"splitppx"}
  and \code{"anylist"}. There are methods for \code{print},
  \code{summary} and \code{plot}.
}
\seealso{
  \code{\link{ppx}},
  \code{\link{plot.anylist}}
}
\examples{
   df <- data.frame(x=runif(4),y=runif(4),t=runif(4),
                    age=factor(rep(c("old", "new"), 2)),
                    size=runif(4))
   X <- ppx(data=df, coord.type=c("s","s","t","m","m"))
   X
   split(X)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{methods}
\keyword{manip}

\name{methods.ppx}
\Rdversion{1.1}
\alias{methods.ppx} %DoNotExport
\alias{print.ppx}
\alias{plot.ppx}
\alias{unitname.ppx}
\alias{unitname<-.ppx}
\title{
  Methods for Multidimensional Space-Time Point Patterns
}
\description{
  Methods for printing and plotting a general multidimensional
  space-time point pattern.
}
\usage{
\method{print}{ppx}(x, ...)
\method{plot}{ppx}(x, ...)
\method{unitname}{ppx}(x)
\method{unitname}{ppx}(x) <- value
}
\arguments{
  \item{x}{
    Multidimensional point pattern (object of class \code{"ppx"}).
  }
  \item{\dots}{
    Additional arguments passed to plot methods.
  }
  \item{value}{
    Name of the unit of length. See \code{\link{unitname}}.
  }
}
\details{
  These are methods for the generic functions
  \code{\link{print}}, \code{\link{plot}},
  \code{\link{unitname}} and  \code{\link{unitname<-}}
  for the class \code{"ppx"} of multidimensional point patterns.
  
  The \code{print} method prints a description
  of the point pattern and its spatial domain.

  The \code{unitname} method extracts the name of the unit of
  length in which the point coordinates are expressed.
  The \code{unitname<-} method assigns the name of the unit of length.
}
\value{
  For \code{print.ppx} the value is \code{NULL}.
  For \code{unitname.ppx} an object of class \code{"units"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{ppx}},
  \code{\link{unitname}}  
}
\keyword{spatial}

\name{rescale.owin}
\alias{rescale.owin}
\title{Convert Window to Another Unit of Length}
\description{
  Converts a window to another unit of length.
}
\usage{
  \method{rescale}{owin}(X, s, unitname)
}
\arguments{
  \item{X}{Window (object of class \code{"owin"}).}
  \item{s}{Conversion factor: the new units are \code{s} times the old units.}
  \item{unitname}{
    Optional. New name for the unit of length.
    See \code{\link{unitname}}.
  }
}
\value{
  Another window object (of class \code{"owin"}) representing the same
  window, but expressed in the new units.
}
\details{
  This is a method for the generic function \code{\link{rescale}}.

  The spatial coordinates in the window \code{X}
  (and its window) will be re-expressed
  in terms of a new unit of length that is \code{s} times the current
  unit of length given in \code{X}.
  (Thus, the coordinate values are \emph{divided} by \code{s},
  while the unit value is multiplied by \code{s}).

  The result is a window representing the \emph{same} region of space,
  but re-expressed in a different unit.

  If \code{s} is missing, then the coordinates will be re-expressed
  in \sQuote{native} units; for example if the current unit is
  equal to 0.1 metres, then the coordinates will be re-expressed in metres.
}
\section{Note}{
  The result of this operation is equivalent to the original window.
  If you want to actually change the coordinates by
  a linear transformation, producing a window that is larger or smaller
  than the original one, use \code{\link{affine}}.
}
\seealso{
  \code{\link{unitname}},
  \code{\link{rescale}},
  \code{\link{rescale.owin}},
  \code{\link{affine}},
  \code{\link{rotate}},
  \code{\link{shift}}
}
\examples{
  data(swedishpines)
  W <- Window(swedishpines)
  W
# coordinates are in decimetres (0.1 metre)
# convert to metres:
  rescale(W, 10)
# or equivalently
  rescale(W)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{Extract.ppp}
\alias{[.ppp}
\alias{[<-.ppp}
\title{Extract or Replace Subset of Point Pattern}
\description{
  Extract or replace a subset of a point pattern.
  Extraction of a subset has the effect of thinning the 
  points and/or trimming the window.
}
\usage{
  \method{[}{ppp}(x, i, j, drop=FALSE, \dots, clip=FALSE)
  \method{[}{ppp}(x, i, j) <- value
}
\arguments{
  \item{x}{
    A two-dimensional point pattern.
    An object of class \code{"ppp"}.
  }
  \item{i}{
   Subset index. Either a valid subset index in the usual \R sense,
   indicating which points should be retained, or a window
   (an object of class \code{"owin"}) 
   delineating a subset of the original observation window,
   or a pixel image with logical values defining a subset of the
   original observation window.
  }
  \item{value}{
    Replacement value for the subset. A point pattern.
  }
  \item{j}{
    Redundant. Included for backward compatibility.
  }
  \item{drop}{
    Logical value indicating whether to remove unused levels
    of the marks, if the marks are a factor.
  }
  \item{clip}{
    Logical value indicating how to form the window of the resulting
    point pattern, when \code{i} is a window. 
    If \code{clip=FALSE} (the default), the result has window
    equal to \code{i}. If \code{clip=TRUE}, the resulting window
    is the intersection between the window of \code{x} and the
    window \code{i}.
  }
  \item{\dots}{
    Ignored. This argument is required for compatibility
    with the generic function.
  }
}
\value{
  A point pattern (of class \code{"ppp"}).
}
\details{
  These functions extract a designated subset of a point pattern,
  or replace the designated subset with another point pattern.

  The function \code{[.ppp} is a method for \code{\link{[}} for the
  class \code{"ppp"}. It extracts a designated subset of a point pattern,
  either by ``\emph{thinning}''
  (retaining/deleting some points of a point pattern)
  or ``\emph{trimming}'' (reducing the window of observation
  to a smaller subregion and retaining only
  those points which lie in the subregion) or both.

  The pattern will be ``thinned''
  if \code{i} is a subset index in the usual \R sense:
  either a numeric vector
  of positive indices (identifying the points to be retained),
  a numeric vector of negative indices (identifying the points
  to be deleted) or a logical vector of length equal to the number of
  points in the point pattern \code{x}. In the latter case, 
  the points \code{(x$x[i], x$y[i])} for which 
  \code{subset[i]=TRUE} will be retained, and the others
  will be deleted.
 
  The pattern will be ``trimmed''
  if \code{i} is an object of class 
  \code{"owin"} specifying a window of observation.
  The points of \code{x} lying inside the new
  window \code{i} will be retained. Alternatively \code{i} may be a
  pixel image (object of class \code{"im"}) with logical values;
  the pixels with the value \code{TRUE} will be interpreted as a window.

  The argument \code{drop} determines whether to remove
  unused levels of a factor, if the point pattern is multitype
  (i.e. the marks are a factor) or if the marks are a data frame
  in which some of the columns are factors.

  The function \code{[<-.ppp} is a method for \code{\link{[<-}} for the
  class \code{"ppp"}. It replaces the designated
  subset with the point pattern \code{value}.
  The subset of \code{x} to be replaced is designated by
  the argument \code{i} as above.

  The replacement point pattern \code{value} must lie inside the
  window of the original pattern \code{x}.
  The ordering of points in \code{x} will be preserved
  if the replacement pattern \code{value} has the same number of points
  as the subset to be replaced.  Otherwise the ordering is
  unpredictable.

  If the original pattern \code{x} has marks, then the replacement
  pattern \code{value} must also have marks, of the same type.

  Use the function \code{\link{unmark}} to remove marks from a
  marked point pattern.

  Use the function \code{\link{split.ppp}} to select those points
  in a marked point pattern which have a specified mark.
}
\seealso{
  \code{\link{subset.ppp}}.

  \code{\link{ppp.object}},
  \code{\link{owin.object}},
  \code{\link{unmark}},
  \code{\link{split.ppp}},
  \code{\link{cut.ppp}}
}
\section{Warnings}{
  The function does not check whether \code{i} is a subset of
  \code{Window(x)}. Nor does it check whether \code{value} lies
  inside \code{Window(x)}.
}
\examples{
 # Longleaf pines data
 lon <- longleaf
 \dontrun{
 plot(lon)
 }
 \testonly{lon <- lon[seq(1,npoints(lon),by=10)]}

 # adult trees defined to have diameter at least 30 cm
 longadult <- subset(lon, marks >= 30)
 \dontrun{
 plot(longadult)
 }
 # note that the marks are still retained.
 # Use unmark(longadult) to remove the marks
 
 # New Zealand trees data
 \dontrun{
 plot(nztrees)          # plot shows a line of trees at the far right
 abline(v=148, lty=2)   # cut along this line
 }
 nzw <- owin(c(0,148),c(0,95)) # the subwindow
 # trim dataset to this subwindow
 nzsub <- nztrees[nzw]
 \dontrun{
 plot(nzsub)
 }

 # Redwood data
 \dontrun{
 plot(redwood)
 }
 # Random thinning: delete 60\% of data
 retain <- (runif(npoints(redwood)) < 0.4)
 thinred <- redwood[retain]
 \dontrun{
 plot(thinred)
 }

 # Scramble 60\% of data
 X <- redwood
 modif <- (runif(npoints(X)) < 0.6)
 X[modif] <- runifpoint(ex=X[modif])

 # Lansing woods data - multitype points
 lan <- lansing
 \testonly{
    lan <- lan[seq(1, npoints(lan), length=100)]
 }

 # Hickory trees
  hicks <- split(lansing)$hickory

 # Trees in subwindow
  win <- owin(c(0.3, 0.6),c(0.2, 0.5))
  lsub <- lan[win]

 # Scramble the locations of trees in subwindow, retaining their marks
  lan[win] <- runifpoint(ex=lsub) \%mark\% marks(lsub)

 # Extract oaks only
 oaknames <- c("redoak", "whiteoak", "blackoak")
 oak <- lan[marks(lan) \%in\% oaknames, drop=TRUE]
 oak <- subset(lan, marks \%in\% oaknames, drop=TRUE)

 # To clip or not to clip
 X <- runifpoint(25, letterR)
 B <- owin(c(2.2, 3.9), c(2, 3.5))
 opa <- par(mfrow=c(1,2))
 plot(X, main="X[B]")
 plot(X[B], border="red", cols="red", add=TRUE, show.all=TRUE, main="")
 plot(X, main="X[B, clip=TRUE]")
 plot(B, add=TRUE, lty=2)
 plot(X[B, clip=TRUE], border="blue", cols="blue", add=TRUE,
      show.all=TRUE, main="")
 par(opa)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{is.dppm}
\alias{is.dppm}
\title{Recognise Fitted Determinantal Point Process Models}
\description{Check that an object inherits the class dppm}
\usage{is.dppm(x)}
\arguments{
  \item{x}{Any object.}
}

\value{A single logical value.}

\author{\ege <rubak@math.aau.dk>,
\adrian <Adrian.Baddeley@uwa.edu.au>
and \rolf <r.turner@auckland.ac.nz>}

\keyword{spatial}
\keyword{manip}
\keyword{models}

\name{rotate.owin}
\alias{rotate.owin}
\title{Rotate a Window}
\description{
  Rotates a window
}
\usage{
 \method{rotate}{owin}(X, angle=pi/2, \dots, rescue=TRUE, centre=NULL)
}
\arguments{
  \item{X}{A window (object of class \code{"owin"}).}
  \item{angle}{Angle of rotation.}
  \item{rescue}{
    Logical. If \code{TRUE}, the rotated window
    will be processed by \code{\link{rescue.rectangle}}.
  }
  \item{\dots}{
    Optional arguments passed to \code{\link{as.mask}}
    controlling the resolution of the rotated window, if \code{X} is a
    binary pixel mask. Ignored if \code{X} is not a binary mask.
  }
  \item{centre}{
    Centre of rotation.
    Either a vector of length 2, or a character string
    (partially matched to \code{"centroid"}, \code{"midpoint"}
    or \code{"bottomleft"}).
    The default is the coordinate origin \code{c(0,0)}.
  }
}
\value{
  Another object of class \code{"owin"} representing the
  rotated window.
}
\details{
  Rotates the window by the specified angle. Angles are measured in
  radians, anticlockwise. The default is to rotate the window 90 degrees
  anticlockwise. The centre of rotation is the origin, by default,
  unless \code{centre} is specified.
}
\seealso{
  \code{\link{owin.object}}
}
\examples{
  w <- owin(c(0,1),c(0,1))
  v <- rotate(w, pi/3)
  e <- rotate(w, pi/2, centre="midpoint")
  \dontrun{
  plot(v)
  }
  w <- as.mask(letterR)
  v <- rotate(w, pi/5)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{hist.im}
\alias{hist.im}
\title{Histogram of Pixel Values in an Image}
\description{
  Computes and displays a histogram of the pixel values in a pixel image.
  The \code{hist} method for class \code{"im"}.
}
\usage{
  \method{hist}{im}(x, \dots, probability=FALSE, xname)
}
\arguments{
  \item{x}{A pixel image (object of class \code{"im"}).}
  \item{\dots}{Arguments passed to \code{\link{hist.default}}
    or \code{\link{barplot}}.}
  \item{probability}{Logical. If \code{TRUE}, the histogram will be
    normalised to give probabilities or probability densities.
  }
  \item{xname}{Optional. Character string to be used as the
  name of the dataset \code{x}.
  }
}
\details{
  This function computes and (by default) displays a histogram
  of the pixel values in the image \code{x}.

  An object of class \code{"im"}
  describes a pixel image. See \code{\link{im.object}})
  for details of this class.

  The function \code{hist.im} is a method for the generic
  function \code{\link{hist}} for the class \code{"im"}. 
  
  Any arguments in \code{...} are passed to \code{\link{hist.default}}
  (for numeric valued images) or \code{\link{barplot}} (for factor or
  logical images).
  For example, such arguments control the axes, and may be used to
  suppress the plotting.
}
\value{
  For numeric-valued images, an object of class \code{"histogram"} as returned
  by \code{\link[graphics:hist]{hist.default}}. This object can be
  plotted.
  
  For factor-valued or logical images, an object of class
  \code{"barplotdata"}, which can be plotted.
  This is a list with components
  called \code{counts} (contingency table of counts of the numbers of
  pixels taking each possible value), \code{probs} (corresponding relative
  frequencies) and \code{mids} (graphical \eqn{x}-coordinates of the
  midpoints of the bars in the barplot). 
}
\seealso{
  \code{\link{spatialcdf}} for the cumulative distribution function
  of an image.

  \code{\link{hist}},
  \code{\link{hist.default}},
  \code{\link{barplot}}.

  For other statistical graphics such as Q-Q plots,
  use \code{X[]} to extract the pixel values of image \code{X},
  and apply the usual statistical graphics commands.
  
  For information about pixel images see
  \code{\link{im.object}},
  \code{\link{summary.im}}.
}
\examples{
  X <- as.im(function(x,y) {x^2}, unit.square())
  hist(X)
  hist(cut(X,3))
}
\author{
  \adrian
  and \rolf
}
\keyword{spatial}
\keyword{methods}

\name{Extract.quad}
\alias{[.quad}
\title{Subset of Quadrature Scheme}
\description{
  Extract a subset of a quadrature scheme.
}
\usage{
  \method{[}{quad}(x, ...)
}
\arguments{
  \item{x}{
    A quadrature scheme (object of class \code{"quad"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{[.ppp}} to determine the
    subset.
  }
}
\value{
  A quadrature scheme (object of class \code{"quad"}).
}
\details{
  This function extracts a designated subset of a quadrature scheme.

  The function \code{[.quad} is a method for \code{\link{[}} for the
  class \code{"quad"}. It extracts a designated subset of a quadrature
  scheme.

  The subset to be extracted is determined by the arguments \code{\dots} which
  are interpreted by \code{\link{[.ppp}}. Thus it is possible to take the
  subset consisting of all quadrature points that lie inside a
  given region, or a subset of quadrature points identified by
  numeric indices.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{[.ppp}}.
}
\examples{
 Q <- quadscheme(nztrees)
 W <- owin(c(0,148),c(0,95)) # a subwindow
 Q[W]
}
\author{
  \adrian
  and
  \rolf
}
\keyword{spatial}
\keyword{manip}

\name{diameter.boxx} 
\Rdversion{1.1}
\alias{diameter.boxx}
\alias{volume.boxx}
\alias{shortside.boxx}
\alias{sidelengths.boxx}
\alias{eroded.volumes.boxx}
\title{
  Geometrical Calculations for Multi-Dimensional Box
}
\description{
  Calculates the volume, diameter, shortest side,
  side lengths, or eroded volume of a multi-dimensional box.
}
\usage{
\method{diameter}{boxx}(x)

\method{volume}{boxx}(x)

\method{shortside}{boxx}(x)

\method{sidelengths}{boxx}(x)

\method{eroded.volumes}{boxx}(x, r)
}
\arguments{
  \item{x}{
    Multi-dimensional box (object of class \code{"boxx"}).
  }
  \item{r}{
    Numeric value or vector of numeric values for which eroded volumes
    should be calculated.
  }
}
\details{
  \code{diameter.boxx}, 
  \code{volume.boxx} and \code{shortside.boxx}
  compute the diameter, volume and shortest side length of the box.
  \code{sidelengths.boxx} returns the lengths of each side of the box.
  
  \code{eroded.volumes.boxx} computes, for each entry \code{r[i]},
  the volume of the smaller box obtained by removing a slab of
  thickness \code{r[i]} from each face of the box. This smaller box is
  the subset consisting of points that lie at least \code{r[i]} units
  away from the boundary of the box.
}
\value{
  For \code{diameter.boxx}, \code{shortside.boxx} and
  \code{volume.boxx}, a single numeric value.
  For \code{sidelengths.boxx}, a numeric vector of length equal to
  the number of spatial dimensions.
  For \code{eroded.volumes.boxx}, a numeric vector of the same length
  as \code{r}.  
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{boxx}}
}
\examples{
    X <- boxx(c(0,10),c(0,10),c(0,5),c(0,2))
    diameter(X) 
    volume(X)
    shortside(X)
    sidelengths(X)
    hd <- shortside(X)/2
    eroded.volumes(X, seq(0,hd, length=10))
}
\keyword{spatial}
\keyword{math}

\name{compatible.im}  
\alias{compatible.im}
\title{Test Whether Pixel Images Are Compatible}
\description{
  Tests whether two or more pixel image objects have compatible dimensions.
}
\usage{
 \method{compatible}{im}(A, B, \dots, tol=1e-6)
}
\arguments{
  \item{A,B,\dots}{Two or more pixel images (objects of class \code{"im"}).}
  \item{tol}{Tolerance factor}
}
\details{
  This function tests whether the pixel images \code{A} and \code{B}
  (and any additional images \code{\dots})
  have compatible pixel dimensions. They are compatible if they have
  the same number of rows and columns, the same physical pixel
  dimensions, and occupy the same rectangle in the plane.

  The argument \code{tol} specifies the maximum tolerated error
  in the pixel coordinates, expressed as a
  fraction of the dimensions of a single pixel.
}
\value{
  Logical value: \code{TRUE} if the images are compatible, and \code{FALSE}
  if they are not.
}
\seealso{
  \code{\link{eval.im}},
  \code{\link{harmonise.im}},
  \code{\link{commonGrid}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{covering}
\alias{covering}
\title{Cover Region with Discs}
\description{
  Given a spatial region, this function finds an efficient
  covering of the region using discs of a chosen radius.
}
\usage{
 covering(W, r, \dots, giveup=1000)
}
\arguments{
  \item{W}{
    A window (object of class \code{"owin"}).
  }
  \item{r}{positive number: the radius of the covering discs.}
  \item{\dots}{
    extra arguments passed to \code{\link{as.mask}}
    controlling the pixel resolution for the calculations.
  }
  \item{giveup}{
    Maximum number of attempts to place additional discs.
  }
}
\value{
  A point pattern (object of class \code{"ppp"})
  giving the centres of the discs.
}
\details{
  This function finds an efficient covering of the
  window \code{W} using discs of the given radius \code{r}.
  The result is a point pattern giving the centres of the discs.

  The algorithm tries to use as few discs as possible,
  but is not guaranteed to find the minimal number of discs.
  It begins by placing a hexagonal grid of points inside \code{W},
  then adds further points until every location inside \code{W} lies
  no more than \code{r} units away from one of the points.
}
\examples{
  rr <- 0.5
  X <- covering(letterR, rr)
  plot(grow.rectangle(Frame(X), rr), type="n", main="")
  plot(X, pch=16, add=TRUE, col="red")
  plot(letterR, add=TRUE, lwd=3)
  plot(X \%mark\% (2*rr), add=TRUE, markscale=1)
}
\author{
  \adrian
}
\keyword{spatial}
\keyword{math}

\name{eval.im}
\alias{eval.im}
\title{Evaluate Expression Involving Pixel Images}
\description{
  Evaluates any expression involving one or more pixel images,
  and returns a pixel image.
}
\usage{
  eval.im(expr, envir, harmonize=TRUE, warn=TRUE)
}
\arguments{
  \item{expr}{An expression.}
  \item{envir}{Optional. The environment in which to evaluate the
    expression, or a named list containing pixel images to be used
    in the expression.}
  \item{harmonize}{
    Logical. Whether to resolve inconsistencies between
    the pixel grids.
  }
  \item{warn}{
    Logical. Whether to issue a warning if the pixel grids were
    inconsistent.
  }
}
\details{
  This function is a wrapper to make it easier to perform
  pixel-by-pixel calculations in an image. 

  Pixel images in \pkg{spatstat}
  are represented by objects of class \code{"im"}
  (see \code{\link{im.object}}). These are essentially matrices of
  pixel values, with extra attributes recording the pixel dimensions,
  etc.

  Suppose \code{X} is a pixel image. Then \code{eval.im(X+3)}
  will add 3 to the value of every pixel in \code{X}, and return
  the resulting pixel image.

  Suppose \code{X} and \code{Y} are two pixel images with compatible
  dimensions: they have the same number of pixels, the same physical
  size of pixels, and the same bounding box. Then
  \code{eval.im(X + Y)} will add the corresponding pixel values in
  \code{X} and \code{Y}, and return the resulting pixel image.

  In general, \code{expr} can be any expression in the R language involving
  (a) the \emph{names} of pixel images, (b) scalar
  constants, and (c) functions which are vectorised.
  See the Examples.

  First \code{eval.im} determines which of the \emph{variable names}
  in the expression \code{expr} refer to pixel images. Each such name
  is replaced by a matrix containing the pixel values. The expression is
  then evaluated. The result should be a matrix; it is taken as
  the matrix of pixel values.
  
  The expression \code{expr} must be vectorised.
  There must be at least one pixel image in the expression.

  All images must have compatible dimensions.
  If \code{harmonize=FALSE}, images that are incompatible will cause an error.
  If \code{harmonize=TRUE}, images that have incompatible dimensions
  will be resampled so that they are compatible; if \code{warn=TRUE},
  a warning will be issued.
}
\value{
  An image object of class \code{"im"}.
}
\seealso{
  \code{\link{im.apply}} for operations similar to \code{\link{apply}},
  such as taking the sum of a list of images.
  
  \code{\link{as.im}},
  \code{\link{compatible.im}},
  \code{\link{harmonise.im}},
  \code{\link{im.object}}
}
\examples{
  # test images
  X <- as.im(function(x,y) { x^2 - y^2 }, unit.square())
  Y <- as.im(function(x,y) { 3 * x + y }, unit.square())

  eval.im(X + 3)
  eval.im(X - Y)
  eval.im(abs(X - Y))
  Z <- eval.im(sin(X * pi) + Y)

  ## Use of 'envir'
  W <- eval.im(sin(U), list(U=density(cells)))
}
\author{
  \adrian
  and \rolf
}
\keyword{spatial}
\keyword{manip}
\keyword{programming}

\name{tile.areas}
\alias{tile.areas}
\title{Compute Areas of Tiles in a Tessellation}
\description{
  Computes the area of each tile in a tessellation.
}
\usage{
tile.areas(x)
}
\arguments{
  \item{x}{A tessellation (object of class \code{"tess"}).}
}
\details{
  A tessellation is a collection of disjoint spatial regions
  (called \emph{tiles}) that fit together to form a larger spatial
  region. See \code{\link{tess}}.

  This command computes the area of each of the tiles 
  that make up the tessellation \code{x}.
  The result is a numeric vector
  in the same order as the tiles would be listed by \code{tiles(x)}.
}
\value{
  A numeric vector.
}
\seealso{
  \code{\link{tess}},
  \code{\link{tiles}},
  \code{\link{tilenames}},
  \code{\link{tiles.empty}}
}
\examples{
  A <- tess(xgrid=0:2,ygrid=0:2)
  tile.areas(A)
  v <- as.im(function(x,y){factor(round(x^2 + y^2))}, W=owin())
  E <- tess(image=v)
  tile.areas(E)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{is.multitype}
\alias{is.multitype}
\title{Test whether Object is Multitype}
\description{
  Generic function to test whether a given object (usually a point
  pattern or something related to a point pattern) has ``marks''
  attached to the points which classify the points into several types.
}
\usage{
  is.multitype(X, \dots) 
}
\arguments{
  \item{X}{
    Object to be inspected
  }
  \item{\dots}{
    Other arguments.
  }
}
\value{
  Logical value, equal to \code{TRUE} if \code{X} is multitype.
}
\details{
  ``Marks'' are observations attached to each point of a point pattern.
  For example the \code{\link[spatstat.data]{longleaf}} dataset
  contains the locations of trees, each tree being marked by its diameter;
  the \code{\link[spatstat.data]{amacrine}} dataset gives the locations of cells
  of two types (on/off) and the type of cell may be regarded as a mark attached
  to the location of the cell.
  Other objects related to point patterns, such as point process models,
  may involve marked points.

  This function tests whether the object \code{X}
  contains or involves marked points, \bold{and} that the
  marks are a factor.

  For example, the \code{\link[spatstat.data]{amacrine}}
  dataset is multitype (there are two types of cells, on and off),
  but the \code{\link[spatstat.data]{longleaf}} dataset is \emph{not} multitype
  (the marks are real numbers).
  
  This function is generic; methods are provided
  for point patterns (objects of class \code{"ppp"})
  and point process models (objects of class \code{"ppm"}).
}
\seealso{
  \code{\link{is.multitype.ppp}},
  \code{\link{is.multitype.ppm}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{texturemap}
\alias{texturemap}
\title{
  Texture Map
}
\description{
  Create a map that associates data values with graphical textures.
}
\usage{
texturemap(inputs, textures, ...)
}
\arguments{
  \item{inputs}{
    A vector containing all the data values
    that will be mapped to textures.
  }
  \item{textures}{
    Optional. A vector of integer codes specifying the textures
    to which the \code{inputs} will be mapped.
  }
  \item{\dots}{
    Other graphics parameters such as \code{col}, \code{lwd}, \code{lty}.
  }
}
\details{
  A texture map is an association between data values and graphical
  textures. The command \code{texturemap} creates an object of class
  \code{"texturemap"} that represents a texture map.

  Once a texture map has been created, it can be applied to any
  suitable data to generate a texture plot of those data
  using \code{\link{textureplot}}.
  This makes it easy to ensure that
  the \emph{same} texture map is used in two different plots.
  The texture map can also be plotted in its own right.

  The argument \code{inputs} should be a vector containing all the
  possible data values (such as the levels of a factor) that are to be
  mapped.

  The \code{textures} should be integer values between 1 and 8,
  representing the eight possible textures
  described in the help for \code{\link{add.texture}}.
  The default is \code{textures = 1:n} where
  \code{n} is the length of \code{inputs}.
}
\value{
  An object of class \code{"texturemap"} representing the texture map.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{textureplot}}
}
\examples{
   texturemap(letters[1:4], 2:5, col=1:4, lwd=2)
}
\keyword{spatial}
\keyword{hplot}

\name{rescale}
\alias{rescale}
\title{Convert dataset to another unit of length}
\description{
  Converts between different units of length
  in a spatial dataset, such as a point pattern or a window. 
}
\usage{
  rescale(X, s, unitname)
}
\arguments{
  \item{X}{Any suitable dataset representing a two-dimensional
    object, such as a point pattern (object of class \code{"ppp"}),
    or a window (object of class \code{"owin"}).}
  \item{s}{Conversion factor: the new units are \code{s} times the old
    units.}
  \item{unitname}{
    Optional. New name for the unit of length.
    See \code{\link{unitname}}.
  }
}
\value{
  Another object of the same type, representing the same
  data, but expressed in the new units.
}
\details{
  This is generic. Methods are provided for many spatial objects.

  The spatial coordinates in the dataset \code{X} will be re-expressed
  in terms of a new unit of length that is \code{s} times the current
  unit of length given in \code{X}. The name of the unit of length
  will also be adjusted. The result is an object of the same type,
  representing the same data, but expressed in the new units.

  For example if \code{X} is a dataset giving coordinates in metres,
  then \code{rescale(X,1000)} will take the new unit of length
  to be 1000 metres.
  To do this, it will divide the old coordinate values by 1000
  to obtain coordinates expressed in kilometres, and change the
  name of the unit of length from \code{"metres"} to \code{"1000 metres"}.

  If \code{unitname} is given, it will be taken as the new name of the unit
  of length. It should be a valid name for the
  unit of length, as described in the help for \code{\link{unitname}}.
  For example if \code{X} is a dataset giving coordinates in metres,
  \code{rescale(X, 1000, "km")} will divide the coordinate
  values by 1000 to obtain coordinates in
  kilometres, and the unit name will be changed to \code{"km"}.
}
\section{Note}{
  The result of this operation is equivalent to the original dataset.
  If you want to actually change the coordinates by
  a linear transformation, producing a dataset that is not equivalent
  to the original one, use \code{\link{affine}}.
}
\seealso{
Available methods:
\code{\link{rescale.im}},
\code{\link{rescale.layered}},
\code{\link{rescale.linnet}},
\code{\link{rescale.lpp}},
\code{\link{rescale.owin}},
\code{\link{rescale.ppp}},
\code{\link{rescale.psp}}
and \code{\link{rescale.unitname}}.

Other generics: 
\code{\link{unitname}},
  \code{\link{affine}},
  \code{\link{rotate}},
  \code{\link{shift}}.
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{update.symbolmap}
\alias{update.symbolmap}
\title{
  Update a Graphics Symbol Map.
}
\description{
  This command updates the \code{object} using the
  arguments given.
}
\usage{
\method{update}{symbolmap}(object, \dots)
}
\arguments{
  \item{object}{
    Graphics symbol map (object of class \code{"symbolmap"}).
  }
  \item{\dots}{
    Additional or replacement arguments to \code{\link{symbolmap}}.
  }
}
\details{
  This is a method for the generic function \code{\link[stats]{update}}
  for the class \code{"symbolmap"} of graphics symbol maps.
  It updates the \code{object} using the parameters given in the
  extra arguments \code{\dots}.

  The extra arguments must be given in
  the form \code{name=value} and must be recognisable to
  \code{\link{symbolmap}}. They override any parameters
  of the same name in \code{object}.
}
\value{
  Another object of class \code{"symbolmap"}.
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege.
}
\seealso{
  \code{\link{symbolmap}} to create a graphics symbol map.
}
\examples{
  g <- symbolmap(size=function(x) x/50)
  g
  update(g, range=c(0,1))
  update(g, size=42)
  update(g, shape="squares", range=c(0,1))
}
\keyword{spatial}
\keyword{hplot}

\name{im}
\alias{im}
\title{Create a Pixel Image Object}
\description{
  Creates an object of
  class \code{"im"} representing a two-dimensional pixel image.
}
\usage{
  im(mat, xcol=seq_len(ncol(mat)), yrow=seq_len(nrow(mat)),
   xrange=NULL, yrange=NULL,
   unitname=NULL)
}
\arguments{
  \item{mat}{
    matrix or vector containing the pixel values of the image.
  }
  \item{xcol}{
    vector of \eqn{x} coordinates for the pixel grid
  }
  \item{yrow}{
    vector of \eqn{y} coordinates for the pixel grid
  }
  \item{xrange,yrange}{
    Optional. Vectors of length 2 giving the \eqn{x} and \eqn{y}
    limits of the enclosing rectangle.
    (Ignored if \code{xcol}, \code{yrow} are present.)
  }
  \item{unitname}{
    Optional. Name of unit of length. Either a single character string,
    or a vector of two character strings giving the
    singular and plural forms, respectively.
  }
}
\details{
  This function creates an object of class \code{"im"} representing
  a \sQuote{pixel image} or two-dimensional array of values.

  The pixel grid is rectangular and occupies a rectangular window
  in the spatial coordinate system. 
  The pixel values are \emph{scalars}: they can be real numbers, integers, 
  complex numbers, single characters or strings, 
  logical values, or categorical values. A pixel's
  value can also be \code{NA}, meaning that no value is defined
  at that location, and effectively that pixel is \sQuote{outside} the window. 
  Although the pixel values must be scalar,
  photographic colour images (i.e., with red, green, and blue brightness
  channels) can be represented as character-valued images in \pkg{spatstat},
  using \R's standard encoding of colours as character strings.

  The matrix \code{mat} contains the \sQuote{greyscale} values
  for a rectangular grid of pixels.
  Note carefully that the entry \code{mat[i,j]}
  gives the pixel value at the location \code{(xcol[j],yrow[i])}.
  That is, the \bold{row} index of the matrix \code{mat} corresponds
  to increasing \bold{y} coordinate, while the column index of \code{mat}
  corresponds to increasing \bold{x} coordinate.
  Thus \code{yrow} has one entry for each row of \code{mat}
  and \code{xcol} has one entry for each column of \code{mat}.
  Under the usual convention in \R, a correct
  display of the image would be obtained by transposing the matrix, e.g.
  \code{image.default(xcol, yrow, t(mat))}, if you wanted to do it by hand.

  The entries of \code{mat} may be numeric (real or integer), complex, 
  logical, character, or factor values.
  If \code{mat} is not a matrix, it will be converted into
  a matrix with \code{nrow(mat) = length(yrow)} and
  \code{ncol(mat) = length(xcol)}.
  
  To make a factor-valued image, note that 
  \R has a quirky way of handling matrices with
  factor-valued entries. The command \code{\link{matrix}} cannot be used
  directly, because it destroys factor information.
  To make a factor-valued image, do one of the following:
  \itemize{
    \item
    Create a \code{factor} containing the pixel values,
    say \code{mat <- factor(.....)}, 
    and then assign matrix dimensions to it by \code{dim(mat) <- c(nr, nc)}
    where \code{nr, nc} are the numbers of rows and columns. The
    resulting object \code{mat} is both a factor and a vector.
    \item
    Supply \code{mat} as a one-dimensional factor
    and specify the arguments \code{xcol} and \code{yrow}
    to determine the dimensions of the image.
    \item
    Use the functions
    \code{\link{cut.im}} or \code{\link{eval.im}} to make factor-valued
    images from other images).
  }
    
  For a description of the methods available for pixel image objects,
  see \code{\link{im.object}}.

  To convert other kinds of data to a pixel image (for example,
  functions or windows), use \code{\link{as.im}}.
}
\seealso{
  \code{\link{im.object}} for details of the class.
  
  \code{\link{as.im}} for converting other kinds of data to an image.
  
  \code{\link{as.matrix.im}},
  \code{\link{[.im}},
  \code{\link{eval.im}} for manipulating images.
}
\section{Warnings}{
  The internal representation of images is likely to change in future
  releases of \pkg{spatstat}. The safe way to extract pixel values
  from an image object is to use \code{\link{as.matrix.im}}
  or \code{\link{[.im}}.
}
\examples{
   vec <- rnorm(1200)
   mat <- matrix(vec, nrow=30, ncol=40)
   whitenoise <- im(mat)
   whitenoise <- im(mat, xrange=c(0,1), yrange=c(0,1))
   whitenoise <- im(mat, xcol=seq(0,1,length=40), yrow=seq(0,1,length=30))
   whitenoise <- im(vec, xcol=seq(0,1,length=40), yrow=seq(0,1,length=30))
   plot(whitenoise)

   # Factor-valued images:
   f <- factor(letters[1:12])
   dim(f) <- c(3,4)
   Z <- im(f)

   # Factor image from other image:
   cutwhite <- cut(whitenoise, 3)
   plot(cutwhite)

   # Factor image from raw data
   cutmat <- cut(mat, 3)
   dim(cutmat) <- c(30,40)
   cutwhite <- im(cutmat)
   plot(cutwhite)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}
\keyword{datagen}
 
 

\name{plot.imlist}
\alias{plot.imlist}
\alias{image.imlist}
\alias{image.listof}
\title{Plot a List of Images}
\description{
  Plots an array of pixel images.
}
\usage{
  \method{plot}{imlist}(x, \dots, plotcommand="image",
                          equal.ribbon=FALSE, ribmar=NULL)

  \method{image}{imlist}(x, \dots, equal.ribbon=FALSE, ribmar=NULL)

  \method{image}{listof}(x, \dots, equal.ribbon=FALSE, ribmar=NULL)
}
\arguments{
  \item{x}{
    An object of the class \code{"imlist"}
    representing a list of pixel images.
    Alternatively \code{x} may belong to the outdated class \code{"listof"}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{plot.solist}} to control the
    spatial arrangement of panels, and arguments passed to \code{\link{plot.im}}
    to control the display of each panel.
  }
  \item{equal.ribbon}{
    Logical. If \code{TRUE}, the colour maps of all the images will be
    the same. If \code{FALSE}, the colour map of each
    image is adjusted to the range of values of that image.
  }
  \item{ribmar}{
    Numeric vector of length 4 specifying the margins around the
    colour ribbon, if \code{equal.ribbon=TRUE}. Entries in the vector
    give the margin at the bottom, left, top, and right respectively,
    as a multiple of the height of a line of text.
  }
  \item{plotcommand}{
    Character string giving the name of a function
    to be used to display each image.
    Recognised by \code{plot.imlist} only.
  }
}
\value{
  Null.
}
\details{
  These are methods for the generic plot commands
  \code{plot} and \code{image} for the class \code{"imlist"}.
  They are currently identical.
  
  An object of class \code{"imlist"} represents a list of pixel images.
  (The outdated class \code{"listof"} is also handled.)

  Each entry in the list \code{x} will be displayed as a pixel image,
  in an array of panels laid out on the same graphics display,
  using \code{\link{plot.solist}}. Individual panels are plotted by
  \code{\link{plot.im}}.

  If \code{equal.ribbon=FALSE} (the default), 
  the images are rendered using different colour maps,
  which are displayed as colour ribbons beside each image.
  If \code{equal.ribbon=TRUE}, the images are rendered using the
  same colour map, and a single colour ribbon will be displayed at the right
  side of the array. 
  The colour maps and the placement of the colour ribbons are
  controlled by arguments \code{\dots} passed to \code{\link{plot.im}}.
}
\seealso{
  \code{\link{plot.solist}},
  \code{\link{plot.im}}
}
\examples{
 D <- density(split(amacrine))
 image(D, equal.ribbon=TRUE, main="", col.ticks="red", col.axis="red")
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{hplot}

\name{default.rmhcontrol}
\alias{default.rmhcontrol}
\title{Set Default Control Parameters for Metropolis-Hastings Algorithm.}
\description{
  Given a fitted point process model, this command
  sets appropriate default values of the
  parameters controlling the iterative behaviour
  of the Metropolis-Hastings algorithm.
}
\usage{
   default.rmhcontrol(model, w=NULL)
}
\arguments{
  \item{model}{
    A fitted point process model (object of class \code{"ppm"})
  }
  \item{w}{
    Optional. Window for the resulting simulated patterns.
  }
}
\value{
  An object of class \code{"rmhcontrol"}. See \code{\link{rmhcontrol}}.
}
\details{
  This function sets the values of 
  the parameters controlling the iterative behaviour
  of the Metropolis-Hastings
  simulation algorithm. It uses default values
  that would be appropriate for the fitted point
  process model \code{model}.

  The expansion parameter \code{expand} is set to
  \code{\link{default.expand}(model, w)}.

  All other parameters revert to their defaults given in
  \code{\link{rmhcontrol.default}}.
  
  See \code{\link{rmhcontrol}} for the full list of control parameters.
  To override default parameters, use \code{\link{update.rmhcontrol}}.
}
\seealso{
  \code{\link{rmhcontrol}},
  \code{\link{update.rmhcontrol}},
  \code{\link{ppm}},
  \code{\link{default.expand}}
}
\examples{
  fit <- ppm(cells, ~1, Strauss(0.1))
  default.rmhcontrol(fit)
  default.rmhcontrol(fit, w=square(2))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{print.im}
\alias{print.im}
\title{Print Brief Details of an Image}
\description{
  Prints a very brief description of a pixel image object.
}
\usage{
  \method{print}{im}(x, \dots)
}
\arguments{
  \item{x}{Pixel image (object of class \code{"im"}).}
  \item{\dots}{Ignored.}
}
\details{
  A very brief description of the pixel image \code{x} is printed.

  This is a method for the generic function \code{\link{print}}.
}
\seealso{
  \code{\link{print}},
  \code{\link{im.object}},
  \code{\link{summary.im}}
}
\examples{
  data(letterR)
  U <- as.im(letterR)
  U
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{print}

\name{quantile.im}
\alias{quantile.im}
\title{Sample Quantiles of Pixel Image}
\description{
Compute the sample quantiles of the pixel values of a
given pixel image.
}
\usage{
\method{quantile}{im}(x, \dots)
}
\arguments{
\item{x}{
A pixel image.
An object of class \code{"im"}.
}
\item{\dots}{
Optional arguments passed to \code{\link{quantile.default}}.
They determine the probabilities for which quantiles should be
computed. See \code{\link{quantile.default}}.
}
} 
\value{
A vector of quantiles. 
}
\details{
This simple function applies the generic \code{\link{quantile}} operation
to the pixel values of the image \code{x}. 

This function is a convenient
way to inspect an image and to obtain summary statistics.
See the examples.
}
\seealso{
\code{\link{quantile}},
\code{\link{cut.im}},
\code{\link{im.object}}
}
\examples{
# artificial image data
Z <- setcov(square(1))

# find the quartiles
quantile(Z)

# find the deciles
quantile(Z, probs=(0:10)/10)
}

\author{\adrian


and \rolf

}
\keyword{spatial}
\keyword{methods}
\keyword{univar}

\name{pppmatching}
\alias{pppmatching}
\title{Create a Point Matching}
\description{
  Creates an object of class \code{"pppmatching"} representing
  a matching of two planar point patterns (objects of class \code{"ppp"}).
}
\usage{
  pppmatching(X, Y, am, type = NULL, cutoff = NULL, q = NULL,
    mdist = NULL)
}
\arguments{
  \item{X,Y}{Two point patterns (objects of class \code{"ppp"}).}
  \item{am}{
    An \code{npoints(X)} by \code{npoints(Y)} matrix with entries \eqn{\geq 0}{>= 0} 
    that specifies which points are matched and with what weight;
    alternatively, an object that can be coerced to this form
    by \code{as.matrix}.
  }
  \item{type}{
    A character string giving the type of the matching.
    One of \code{"spa"}, \code{"ace"} or \code{"mat"}, or \code{NULL}
    for a generic or unknown matching.
  }
  \item{cutoff, q}{
    Numerical values specifying the cutoff value \eqn{> 0} for interpoint distances and
    the order \eqn{q \in [1,\infty]}{q in [0,Inf]} of the average that is applied to them.
    \code{NULL} if not applicable or unknown.
  }
  \item{mdist}{
    Numerical value for the distance to be associated with the matching.
  }
}
\details{
  The argument \code{am} is interpreted as a "generalized adjacency matrix":
  if the \code{[i,j]}-th entry is positive, then the \code{i}-th point
  of \code{X} and the \code{j}-th point of \code{Y} are matched and the
  value of the entry gives the corresponding weight of the match. For
  an unweighted matching all the weights should be set to \eqn{1}.

  The remaining arguments are optional and allow to save
  additional information about the matching. See the help files for
  \code{\link{pppdist}} and \code{\link{matchingdist}} for details on
  the meaning of these parameters.
}
\author{
  Dominic Schuhmacher
  \email{dominic.schuhmacher@stat.unibe.ch}
  \url{http://www.dominic.schuhmacher.name}
}
\seealso{
  \code{\link{pppmatching.object}}
  \code{\link{matchingdist}}
}
\examples{
  # a random unweighted complete matching
  X <- runifpoint(10)
  Y <- runifpoint(10)
  am <- r2dtable(1, rep(1,10), rep(1,10))[[1]]
        # generates a random permutation matrix
  m <- pppmatching(X, Y, am)
  summary(m)
  m$matrix
  plot(m)

  # a random weighted complete matching
  X <- runifpoint(7)
  Y <- runifpoint(7)
  am <- r2dtable(1, rep(10,7), rep(10,7))[[1]]/10
        # generates a random doubly stochastic matrix
  m2 <- pppmatching(X, Y, am)
  summary(m2)
  m2$matrix
  plot(m2)
  m3 <- pppmatching(X, Y, am, "ace")
  m4 <- pppmatching(X, Y, am, "mat")
}
\keyword{spatial}
\keyword{datagen}

\name{dirichlet}
\alias{dirichlet}
\title{Dirichlet Tessellation of Point Pattern}
\description{
  Computes the Dirichlet tessellation of a spatial point pattern.
  Also known as the Voronoi or Thiessen tessellation.
}
\usage{
dirichlet(X)
}
\arguments{
  \item{X}{Spatial point pattern (object of class \code{"ppp"}).}
}
\details{
  In a spatial point pattern \code{X}, the Dirichlet tile associated
  with a particular point \code{X[i]} is the region of space that is
  closer to \code{X[i]} than to any other point in \code{X}. The
  Dirichlet tiles divide the two-dimensional plane into disjoint
  regions, forming a tessellation.

  The Dirichlet tessellation is also known as the Voronoi or
  Thiessen tessellation.

  This function computes the Dirichlet tessellation (within the original
  window of \code{X}) using the function \code{\link[deldir]{deldir}}
  in the package \pkg{deldir}.

  To ensure that there is a one-to-one correspondence between the
  points of \code{X} and the tiles of \code{dirichlet(X)},
  duplicated points in \code{X} should first be removed by
  \code{X <- unique(X, rule="deldir")}. 
  
  The tiles of the tessellation will be computed as polygons
  if the original window is a rectangle or a polygon.
  Otherwise the tiles will be computed as binary masks. 
}
\value{
  A tessellation (object of class \code{"tess"}).
}
\seealso{
  \code{\link{tess}},
  \code{\link{delaunay}},
  \code{\link{ppp}},
  \code{\link{dirichletVertices}}
}
\examples{
  X <- runifpoint(42)
  plot(dirichlet(X))
  plot(X, add=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{intensity}
\alias{intensity}
\title{
  Intensity of a Dataset or a Model
}
\description{
  Generic function for computing the intensity of a spatial dataset
  or spatial point process model.
}
\usage{
intensity(X, ...)
}
\arguments{
  \item{X}{
    A spatial dataset or a spatial point process model.
  }
  \item{\dots}{
    Further arguments depending on the class of \code{X}.
  }
}
\details{
  This is a generic function for computing the intensity of a spatial dataset
  or spatial point process model. There are methods for point patterns
  (objects of class \code{"ppp"}) and fitted point process models
  (objects of class \code{"ppm"}).
  
  The empirical intensity of a dataset is the average density
  (the average amount of \sQuote{stuff} per unit area or volume).
  The empirical intensity of a point pattern is computed by the
  method \code{\link{intensity.ppp}}.

  The theoretical intensity of a stochastic model is the expected density
  (expected  amount of \sQuote{stuff} per unit area or volume).
  The theoretical intensity of a fitted point process model is computed by the
  method \code{\link{intensity.ppm}}.
}
\value{
  Usually a numeric value or vector.
}
\seealso{
  \code{\link{intensity.ppp}},
  \code{\link{intensity.ppm}}.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{models}

\name{kaplan.meier}
\alias{kaplan.meier}
\title{Kaplan-Meier Estimator using Histogram Data}
\description{
  Compute the Kaplan-Meier estimator of a survival time distribution
  function, from histogram data
}
\usage{
  kaplan.meier(obs, nco, breaks, upperobs=0)
}
\arguments{
  \item{obs}{vector of \eqn{n} integers giving the histogram of
    all observations (censored or uncensored survival times)
  }
  \item{nco}{vector of \eqn{n} integers giving the histogram of
    uncensored observations (those survival times that are less than or
    equal to the censoring time)
  }
  \item{breaks}{Vector of \eqn{n+1} breakpoints which were used to form
    both histograms.
  }
  \item{upperobs}{
    Number of observations beyond the rightmost breakpoint, if any.
  }
}
\value{
  A list with two elements:
  \item{km}{Kaplan-Meier estimate of the survival time c.d.f. \eqn{F(t)}
  }
  \item{lambda}{corresponding Nelson-Aalen estimate of the
    hazard rate \eqn{\lambda(t)}{lambda(t)}
  }
  These are numeric vectors of length \eqn{n}.
}
\details{
  This function is needed mainly for internal use in \pkg{spatstat},
  but may be useful in other applications where you want to form the
  Kaplan-Meier estimator from a huge dataset.

  Suppose \eqn{T_i}{T[i]} are the survival times of individuals
  \eqn{i=1,\ldots,M} with unknown distribution function \eqn{F(t)}
  which we wish to estimate. Suppose these times are right-censored
  by random censoring times \eqn{C_i}{C[i]}.
  Thus the observations consist of right-censored survival times
  \eqn{\tilde T_i = \min(T_i,C_i)}{T*[i] = min(T[i],C[i])}
  and non-censoring indicators
  \eqn{D_i = 1\{T_i \le C_i\}}{D[i] = 1(T[i] <= C[i])}
  for each \eqn{i}.

  If the number of observations \eqn{M} is large, it is efficient to
  use histograms.
  Form the histogram \code{obs} of all observed times \eqn{\tilde T_i}{T*[i]}.
  That is, \code{obs[k]} counts the number of values 
  \eqn{\tilde T_i}{T*[i]} in the interval
  \code{(breaks[k],breaks[k+1]]} for \eqn{k > 1}
  and \code{[breaks[1],breaks[2]]} for \eqn{k = 1}.
  Also form the histogram \code{nco} of all uncensored times,
  i.e. those \eqn{\tilde T_i}{T*[i]} such that \eqn{D_i=1}{D[i]=1}.
  These two histograms are the arguments passed to \code{kaplan.meier}.
  
  The vectors \code{km} and \code{lambda} returned by \code{kaplan.meier}
  are (histogram approximations to) the Kaplan-Meier estimator
  of \eqn{F(t)} and its hazard rate \eqn{\lambda(t)}{lambda(t)}.
  Specifically, \code{km[k]} is an estimate of
  \code{F(breaks[k+1])}, and \code{lambda[k]} is an estimate of
  the average of \eqn{\lambda(t)}{lambda(t)} over the interval
  \code{(breaks[k],breaks[k+1])}.

  The histogram breaks must include \eqn{0}.
  If the histogram breaks do not span the range of the observations,
  it is important to count how many survival times
  \eqn{\tilde T_i}{T*[i]} exceed the rightmost breakpoint,
  and give this as the value \code{upperobs}.
}
\seealso{
  \code{\link{reduced.sample}},
  \code{\link{km.rs}}
}
\author{\adrian
  
  
  and \rolf
  
  }
\keyword{spatial}
\keyword{nonparametric}

\name{simplepanel}
\alias{simplepanel}
\alias{grow.simplepanel}
\title{Simple Point-and-Click Interface Panels}
\description{
  These functions enable the user to create a simple, robust, 
  point-and-click interface to any \R code.
}
\usage{
   simplepanel(title, B, boxes, clicks,
      redraws=NULL, exit = NULL, env)

   grow.simplepanel(P, side = c("right", "left", "top", "bottom"),
      len = NULL, new.clicks, new.redraws=NULL, \dots, aspect)
}
\arguments{
  \item{title}{
    Character string giving the title of the interface panel.
  }
  \item{B}{
    Bounding box of the panel coordinates.
    A rectangular window (object of class \code{"owin"})
  }
  \item{boxes}{
    A list of rectangular windows (objects of class \code{"owin"})
    specifying the placement of the buttons
    and other interactive components of the panel.
  }
  \item{clicks}{
    A list of \R functions, of the same length as \code{boxes},
    specifying the operations to be performed when each button
    is clicked. Entries can also be \code{NULL} indicating that no
    action should occur. See Details.
  }
  \item{redraws}{
    Optional list of \R functions, of the same length as \code{boxes},
    specifying how to redraw each button. Entries can also be
    \code{NULL} indicating a simple default. See Details.
  }
  \item{exit}{
    An \R function specifying actions to be taken when the
    interactive panel terminates.
  }
  \item{env}{
    An \code{environment} that will be passed as an argument
    to all the functions in \code{clicks}, \code{redraws} and
    \code{exit}.
  }
  \item{P}{
    An existing interaction panel (object of class \code{"simplepanel"}).
  }
  \item{side}{
    Character string identifying which side of the panel \code{P}
    should be grown to accommodate the new buttons.
  }
  \item{len}{
    Optional. Thickness of the new panel area that should be grown
    to accommodate the new buttons. A single number in the same units
    as the coordinate system of \code{P}.
  }
  \item{new.clicks}{
    List of \R functions defining the operations to be performed
    when each of the new buttons is clicked.
  }
  \item{new.redraws}{
    Optional. List of \R functions, of the same length as
    \code{new.clicks}, defining how to redraw each of the new buttons.
  }
  \item{\dots}{
    Arguments passed to \code{\link{layout.boxes}} to determine the
    layout of the new buttons.
  }
  \item{aspect}{
    Optional. Aspect ratio (height/width) of the new buttons.
  }
}
\details{
  These functions enable the user to create a simple, robust,
  point-and-click interface to any \R code.

  The functions \code{simplepanel} and \code{grow.simplepanel}
  create an object of class \code{"simplepanel"}. Such an object defines
  the graphics to be displayed and the actions to be performed
  when the user interacts with the panel.

  The panel is activated by calling \code{\link{run.simplepanel}}.

  The function \code{simplepanel} creates a panel object
  from basic data.
  The function \code{grow.simplepanel} modifies an existing
  panel object \code{P} by growing an additional row or column
  of buttons. 

  For \code{simplepanel},
  \itemize{
    \item 
    The spatial layout of the panel is determined by the rectangles
    \code{B} and \code{boxes}.
    \item 
    The argument \code{clicks} must be a list of functions
    specifying the action to be taken when each button is clicked
    (or \code{NULL} to indicate that no action should be taken).
    The list entries should have names (but there are sensible defaults).
    Each function should be of the form \code{function(env, xy)} where
    \code{env} is an \code{environment} that may contain shared data,
    and \code{xy} gives the coordinates of the mouse click, in the format
    \code{list(x, y)}.
    The function returns \code{TRUE} if the
    panel should continue running, and \code{FALSE} if the panel
    should terminate.
    \item 
    The argument \code{redraws}, if given, must be a list of functions
    specifying the action to be taken when each button is to be redrawn.
    Each function should be of the form \code{function(button, name, env)} where
    \code{button} is a rectangle specifying the location of the button
    in the current coordinate system; \code{name} is a character string
    giving the name of the button; and \code{env} is the
    \code{environment} that may contain shared data.
    The function returns \code{TRUE} if the
    panel should continue running, and \code{FALSE} if the panel
    should terminate. 
    If \code{redraws} is not given (or if one of the entries in
    \code{redraws} is \code{NULL}), the default action is to draw a pink
    rectangle showing the button position,
    draw the name of the button in the middle of this rectangle,
    and return \code{TRUE}. 
    \item
    The argument \code{exit}, if given, must be a function
    specifying the action to be taken when the panel terminates.
    (Termination occurs when one of the \code{clicks} functions
    returns \code{FALSE}).
    The \code{exit} function should be of the form \code{function(env)} where
    \code{env} is the \code{environment} that may contain shared data.
    Its return value will be used as the return value
    of \code{\link{run.simplepanel}}.
    \item
    The argument \code{env} should be an \R environment.
    The panel buttons will have access to this environment,
    and will be able to read and write data in it. This mechanism is used
    to exchange data between the panel and other \R code.
  }
  For \code{grow.simplepanel},
  \itemize{
    \item the spatial layout of the new boxes
    is determined by the arguments \code{side}, \code{len},
    \code{aspect} and by the additional \code{\dots} arguments passed to
    \code{\link{layout.boxes}}.
    \item the argument \code{new.clicks} 
    should have the same format as \code{clicks}.
    It implicitly specifies the number of new buttons to be added,
    and the actions to be performed when they are clicked.
    \item the optional argument \code{new.redraws}, if given,
    should have the same format as \code{redraws}.
    It specifies the actions to be performed when the
    new buttons are clicked.
  }
}
\value{
  An object of class \code{"simplepanel"}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{run.simplepanel}},
  \code{\link{layout.boxes}}
}
\examples{
  # make boxes (alternatively use layout.boxes())
  Bminus <- square(1)
  Bvalue <- shift(Bminus, c(1.2, 0))
  Bplus <- shift(Bvalue, c(1.2, 0))
  Bdone <- shift(Bplus, c(1.2, 0))
  myboxes <- list(Bminus, Bvalue, Bplus, Bdone)
  myB <- do.call(boundingbox,myboxes)

  # make environment containing an integer count
  myenv <- new.env()
  assign("answer", 0, envir=myenv)

  # what to do when finished: return the count.
  myexit <- function(e) { return(get("answer", envir=e)) }

  # button clicks
  # decrement the count
  Cminus <- function(e, xy) {
    ans <- get("answer", envir=e)
    assign("answer", ans - 1, envir=e)
    return(TRUE)
  }
  # display the count (clicking does nothing)
  Cvalue <- function(...) { TRUE }
  # increment the count
  Cplus <- function(e, xy) {
    ans <- get("answer", envir=e)
    assign("answer", ans + 1, envir=e)
    return(TRUE)
  }
  # 'Clear' button
  Cclear <- function(e, xy) {
    assign("answer", 0, envir=e)
    return(TRUE)
  }
  # quit button
  Cdone <- function(e, xy) { return(FALSE) }

  myclicks <- list("-"=Cminus,
                   value=Cvalue,
                   "+"=Cplus,
                   done=Cdone)

  # redraw the button that displays the current value of the count
  Rvalue <- function(button, nam, e) {
     plot(button, add=TRUE)
     ans <- get("answer", envir=e)
     text(centroid.owin(button), labels=ans)
     return(TRUE)
  }

  # make the panel
  P <- simplepanel("Counter",
                   B=myB, boxes=myboxes,
                   clicks=myclicks,
                   redraws = list(NULL, Rvalue, NULL, NULL),
                   exit=myexit, env=myenv)
  # print it
  P
  # show what it looks like
  redraw.simplepanel(P)

  # ( type run.simplepanel(P) to run the panel interactively )

  # add another button to right
  Pplus <- grow.simplepanel(P, "right", new.clicks=list(clear=Cclear))
}
\keyword{iplot}
\keyword{utilities}

\name{kernel.factor}
\alias{kernel.factor}
\title{Scale factor for density kernel} 
\description{
  Returns a scale factor for the kernels used in density estimation
  for numerical data.
}
\usage{
  kernel.factor(kernel = "gaussian")
}
\arguments{
  \item{kernel}{
    String name of the kernel.
    Options are
    \code{"gaussian"}, \code{"rectangular"},
    \code{"triangular"},
    \code{"epanechnikov"},
    \code{"biweight"},
    \code{"cosine"} and \code{"optcosine"}.
    (Partial matching is used).
  }
}
\details{
  Kernel estimation of a probability density in one dimension
  is performed by \code{\link[stats]{density.default}}
  using a kernel function selected from the list above.

  This function computes a scale constant for the kernel.
  For the Gaussian kernel, this constant is equal to 1.
  Otherwise, the constant \eqn{c} is such that the kernel
  with standard deviation \eqn{1} is supported on the interval
  \eqn{[-c,c]}. 
  
  For more information about these kernels,
  see \code{\link[stats]{density.default}}. 
}
\value{
  A single number.
}
\seealso{
  \code{\link[stats]{density.default}},
  \code{\link{dkernel}},
  \code{\link{kernel.moment}},
  \code{\link{kernel.squint}}
}
\examples{
   kernel.factor("rect")
   # bandwidth for Epanechnikov kernel with half-width h=1
   h <- 1
   bw <- h/kernel.factor("epa")
}
\author{\adrian
  and Martin Hazelton
}
\keyword{methods}
\keyword{nonparametric}
\keyword{smooth}

\name{with.hyperframe}
\alias{with.hyperframe}
\title{Evaluate an Expression in Each Row of a Hyperframe}
\description{
  An expression, involving the names of columns in a hyperframe,
  is evaluated separately for each row of the hyperframe.
}
\usage{
\method{with}{hyperframe}(data, expr, ...,
                         simplify = TRUE,
                         ee = NULL, enclos=NULL)
}
\arguments{
  \item{data}{A hyperframe (object of class \code{"hyperframe"})
    containing data.
  }
  \item{expr}{An \R language expression to be evaluated.}
  \item{\dots}{Ignored.}
  \item{simplify}{
    Logical. If \code{TRUE}, the return value
    will be simplified to a vector whenever possible.
  }
  \item{ee}{
    Alternative form of \code{expr}, as an object of class
    \code{"expression"}.
  }
  \item{enclos}{
    An environment in which to search for objects that are
    not found in the hyperframe. Defaults to \code{\link{parent.frame}()}.
  }
}
\details{
  This function evaluates the expression \code{expr} in each row
  of the hyperframe \code{data}. It is a method for the generic
  function \code{\link{with}}.
  
  The argument \code{expr} should be an \R language expression
  in which each variable name is either the name of a column in the
  hyperframe \code{data}, or the name of an object in the parent frame
  (the environment in which \code{with} was called.)
  The argument \code{ee} can be used as an alternative
  to \code{expr} and should be an expression object (of
  class \code{"expression"}).
  
  For each row of \code{data}, the expression will be evaluated
  so that variables which are column names of \code{data} are
  interpreted as the entries for those columns in the current row.

  For example, if a hyperframe \code{h} has columns
  called \code{A} and \code{B}, then \code{with(h, A != B)} inspects
  each row of \code{data} in turn,
  tests whether the entries in columns \code{A} and \code{B} are
  equal, and returns the \eqn{n} logical values.
}
\value{
  Normally a list of length
  \eqn{n} (where \eqn{n} is the number of rows) containing the results
  of evaluating the expression for each row. 
  If \code{simplify=TRUE} and each result is a single atomic value,
  then the result is a vector or factor 
  containing the same values.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{hyperframe}},
  \code{\link{plot.hyperframe}}
}
\examples{
  # generate Poisson point patterns with intensities 10 to 100
   H <- hyperframe(L=seq(10,100, by=10))
   X <- with(H, rpoispp(L))
}
\keyword{spatial}
\keyword{manip}
\keyword{programming}

\name{ellipse}
\alias{ellipse}
\title{
  Elliptical Window.
}
\description{
  Create an elliptical window.
}
\usage{
   ellipse(a, b, centre=c(0,0), phi=0, \dots, mask=FALSE, npoly = 128)
}
\arguments{
  \item{a,b}{
    The half-lengths of the axes of the ellipse.
  }
  \item{centre}{
    The centre of the ellipse.
  }
  \item{phi}{
    The (anti-clockwise) angle through which the ellipse should be
    rotated (about its centre) starting from an orientation in which
    the axis of half-length \code{a} is horizontal.
  }
  \item{mask}{
    Logical value controlling the type of approximation
    to a perfect ellipse. See Details.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine the
    pixel resolution, if \code{mask} is \code{TRUE}.
  }
  \item{npoly}{
    The number of edges in the polygonal approximation to the ellipse.
  }
}
\details{
  This command creates a window object
  representing an ellipse with the given centre and axes.

  By default, the ellipse is
  approximated by a polygon with \code{npoly} edges.

  If \code{mask=TRUE}, then the ellipse is approximated by a binary pixel
  mask. The resolution of the mask is controlled by
  the arguments \code{\dots} which are passed to \code{\link{as.mask}}.

  The arguments \code{a} and \code{b} must be single positive numbers.
  The argument \code{centre} specifies the ellipse centre: it can be either 
  a numeric vector of length 2 giving the coordinates,
  or a \code{list(x,y)} giving the coordinates of exactly one point, or a
  point pattern (object of class \code{"ppp"}) containing exactly one point.
}
\value{
  An object of class \code{owin} (either of type \dQuote{polygonal}
  or of type \dQuote{mask}) specifying an elliptical window.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{disc}},
  \code{\link{owin.object}},
  \code{\link{owin}},
  \code{\link{as.mask}}
}
\examples{
  W <- ellipse(a=5,b=2,centre=c(5,1),phi=pi/6)
  plot(W,lwd=2,border="red")
  WM <- ellipse(a=5,b=2,centre=c(5,1),phi=pi/6,mask=TRUE,dimyx=512)
  plot(WM,add=TRUE,box=FALSE)
}
\keyword{spatial}
\keyword{datagen}

\name{anylist}
\alias{anylist}
\alias{as.anylist}
\title{
  List of Objects
}
\description{
  Make a list of objects of any type.
}
\usage{
anylist(\dots)
as.anylist(x)
}
\arguments{
  \item{\dots}{
    Any number of arguments of any type.
  }
  \item{x}{
    A list.
  }
}
\details{
  An object of class \code{"anylist"} is
  a list of objects that the user intends to treat in a similar fashion.

  For example it may be desired to plot each of the objects
  side-by-side: this can be done using the function
  \code{\link{plot.anylist}}.
  
  The objects can belong to any class;
  they may or may not all belong to the same class.

  In the \pkg{spatstat} package, various functions produce
  an object of class \code{"anylist"}.
}
\value{
  A list, belonging to the class \code{"anylist"},
  containing the original objects.
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link{solist}},
  \code{\link{as.solist}},
  \code{\link{anylapply}}.
}
\examples{
  anylist(cells, intensity(cells), Kest(cells))
  anylist()
}
\keyword{list}
\keyword{manip}

\name{venn.tess}
\alias{venn.tess}
\title{
  Tessellation Delimited by Several Sets
}
\description{
  Given a list of windows, construct the tessellation
  formed by all combinations of inclusion/exclusion of these
  windows. 
}
\usage{
venn.tess(\dots, window = NULL)
}
\arguments{
  \item{\dots}{
    Sets which delimit the tessellation.
    Any number of windows (objects of class \code{"owin"})
    or tessellations (objects of class \code{"tess"}).
  }
  \item{window}{
    Optional. The bounding window of the resulting tessellation.
    If not specified, the default is the union of all the
    arguments \code{\dots}.
  }
}
\details{
  The arguments \code{\dots} may be any number of windows.
  This function constructs a tessellation, like a Venn diagram,
  whose boundaries are made up of the boundaries of these sets.
  Each tile of the tessellation is defined by one of the 
  possible combinations in which each set is either included or
  excluded. 

  If the arguments \code{\dots} are named, then the resulting tiles
  will also have names, which identify the inclusion/exclusion
  combinations defining each tile. See the Examples.
}
\value{
  A tessellation (object of class \code{"tess"}).
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{intersect.tess}}.

  To construct other kinds of tessellations, see
  \code{\link{tess}},
  \code{\link{quadrats}}, \code{\link{hextess}},
  \code{\link{polartess}},
  \code{\link{dirichlet}}, \code{\link{delaunay}}, \code{\link{quantess}}
  and \code{\link{rpoislinetess}}.
}
\examples{
   V <- venn.tess(A=square(1),
                  B=square(c(-0.5, 0.5)),
                  window=square(c(-1,1.5)))
   V
   plot(V, do.labels=TRUE)
}
\keyword{spatial}
\keyword{math}

\name{inside.owin}
\alias{inside.owin}
\title{Test Whether Points Are Inside A Window}
\description{
 Test whether points lie inside or outside
 a given window.
}
\usage{
 inside.owin(x, y, w)
}
\arguments{
  \item{x}{
    Vector of \eqn{x} coordinates of points to be tested.
    (Alternatively, a point pattern object providing both
    \eqn{x} and \eqn{y} coordinates.)
  }
  \item{y}{
    Vector of \eqn{y} coordinates of points to be tested.
  }
  \item{w}{A window.
    This should be an object of class \code{\link{owin}},
    or can be given in any format acceptable to \code{\link{as.owin}()}.
  }
}
\value{
  Logical vector whose \code{i}th entry is 
  \code{TRUE} if the corresponding point \code{(x[i],y[i])}
  is inside \code{w}. 
}
\details{
  This function tests whether each of the points 
  \code{(x[i],y[i])} lies inside or outside
  the window \code{w} and returns \code{TRUE} if it is inside.
  
  The boundary of the window is treated as being inside.

  If \code{w} is of type \code{"rectangle"} or 
  \code{"polygonal"}, the algorithm uses analytic geometry
  (the discrete Stokes theorem).
  Computation time is linear in the number of points
  and (for polygonal windows) in the number of vertices of the
  boundary polygon. Boundary cases are correct to single
  precision accuracy.
  
  If \code{w} is of type \code{"mask"} then the 
  pixel closest to \code{(x[i],y[i])} is tested. The
  results may be incorrect for points lying within 
  one pixel diameter of the window boundary.

  Normally \code{x} and \code{y} must be numeric vectors of
  equal length (length zero is allowed) containing the coordinates
  of points. Alternatively \code{x}
  can be a point pattern (object of class \code{"ppp"}) while \code{y}
  is missing; then the coordinates of the point pattern are extracted.
}
\seealso{
  \code{\link{owin.object}},
  \code{\link{as.owin}}
}
\examples{
  # hexagonal window
  k <- 6
  theta <- 2 * pi * (0:(k-1))/k
  co <- cos(theta)
  si <- sin(theta)
  mas <- owin(c(-1,1), c(-1,1), poly=list(x=co, y=si))
  \dontrun{
  plot(mas)
  }

  # random points in rectangle
  x <- runif(30,min=-1, max=1)
  y <- runif(30,min=-1, max=1)

  ok <- inside.owin(x, y, mas)

  \dontrun{
  points(x[ok], y[ok])
  points(x[!ok], y[!ok], pch="x")
  }  
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{timeTaken}
\alias{timeTaken}
\title{
  Extract the Total Computation Time
}
\description{
  Given an object or objects that contain timing information
  (reporting the amount of computer time taken to compute each object),
  this function extracts the timing data and evaluates the total time taken.
}
\usage{
timeTaken(..., warn=TRUE)
}
\arguments{
  \item{\dots}{
    One or more objects of class \code{"timed"} containing
    timing data. 
  }
  \item{warn}{
    Logical value indicating whether a warning should be issued if
    some of the arguments do not contain timing information.
  }
}
\details{
  An object of class \code{"timed"} contains information on
  the amount of computer time that was taken to compute the object.
  See \code{\link{timed}}.

  This function extracts the timing information from one or more
  such objects, and calculates the total time.
}
\value{
  An object inheriting the class \code{"timed"}.
}
\examples{
  A <- timed(Kest(cells))
  B <- timed(Gest(cells))
  A
  B
  timeTaken(A,B)
}
\seealso{
  \code{\link{timed}}
}
\author{
  \spatstatAuthors.
}
\keyword{utilities}

\name{as.data.frame.ppp}
\alias{as.data.frame.ppp}
\title{Coerce Point Pattern to a Data Frame}
\description{
  Extracts the coordinates of the points in a point pattern,
  and their marks if any, and returns them in a data frame.
}
\usage{
\method{as.data.frame}{ppp}(x, row.names = NULL, ...)
}
\arguments{
  \item{x}{Point pattern (object of class \code{"ppp"}).}
  \item{row.names}{Optional character vector of row names.}
  \item{\dots}{Ignored.}
}
\details{
  This is a method for the generic function \code{\link{as.data.frame}}
  for the class \code{"ppp"} of point patterns.

  It extracts the coordinates of the points in the point pattern, and
  returns them as columns named \code{x} and \code{y} in a data frame.
  If the points were marked, the marks are returned as a column
  named \code{marks} with the same type as in the point pattern dataset.
}
\value{
  A data frame.
}
\examples{
  data(amacrine)
  df <- as.data.frame(amacrine)
  df[1:5,]
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{layerplotargs}
\alias{layerplotargs}
\alias{layerplotargs<-}
\title{
  Extract or Replace the Plot Arguments of a Layered Object
}
\description{
  Extracts or replaces the plot arguments of a layered object.
}
\usage{
layerplotargs(L)

layerplotargs(L) <- value
}
\arguments{
  \item{L}{
    An object of class \code{"layered"}
    created by the function \code{\link{layered}}.
  }
  \item{value}{
    Replacement value. 
    A list, with the same length as \code{L},
    whose elements are lists of plot arguments.
  }
}
\details{
  These commands extract or replace the \code{plotargs}
  in a layered object. See \code{\link{layered}}.

  The replacement \code{value} should normally have the same
  length as the current value. However, it can also be a list with
  \emph{one} element which is a list of parameters. This will be
  replicated to the required length.

  For the assignment function \code{layerplotargs<-},
  the argument \code{L} can be any spatial object; it will be converted
  to a \code{layered} object with a single layer.
}
\value{
  \code{layerplotargs} returns a list of lists of plot arguments.

  \code{"layerplotargs<-"} returns the updated object 
  of class \code{"layered"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{layered}},
  \code{\link{methods.layered}},
  \code{\link{[.layered}}.
}
\examples{
   W <- square(2)
   L <- layered(W=W, X=cells)
   ## The following are equivalent
   layerplotargs(L) <- list(list(), list(pch=16))
   layerplotargs(L)[[2]] <- list(pch=16)
   layerplotargs(L)$X <- list(pch=16)

   ## The following are equivalent
   layerplotargs(L) <- list(list(cex=2), list(cex=2))
   layerplotargs(L) <- list(list(cex=2))
}
\keyword{spatial}
\keyword{hplot}

\name{summary.splitppp}
\alias{summary.splitppp}
\title{Summary of a Split Point Pattern}
\description{
  Prints a useful summary of a split point pattern.
}
\usage{
 \method{summary}{splitppp}(object, \dots)
}
\arguments{
  \item{object}{
    Split point pattern (object of class \code{"splitppp"}, effectively a
    list of point patterns, usually created by \code{\link{split.ppp}}).
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  This is a method for the generic function \code{\link{summary}}.

  An object of the class \code{"splitppp"} is effectively a list
  of point patterns (objects of class \code{"ppp"}) representing
  different sub-patterns of an original point pattern.

  This function extracts a useful summary of each of the sub-patterns.
}
\seealso{
  \code{\link{summary}},
  \code{\link{split}},
  \code{\link{split.ppp}}
}
\examples{
  data(amacrine)      # multitype point pattern
  summary(split(amacrine))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}

\name{intensity.quadratcount} 
\alias{intensity.quadratcount}
\title{
  Intensity Estimates Using Quadrat Counts
}
\description{
  Uses quadrat count data to estimate the intensity of a point pattern
  in each tile of a tessellation, assuming the intensity is constant in
  each tile.
}
\usage{
\method{intensity}{quadratcount}(X, ..., image=FALSE)
}
\arguments{
  \item{X}{
    An object of class \code{"quadratcount"}.
  }
  \item{image}{
    Logical value specifying whether to return
    a table of estimated intensities (the default)
    or a pixel image of the estimated intensity (\code{image=TRUE}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine
    the resolution of the pixel image, if \code{image=TRUE}.
  }
}
\details{
  This is a method for the generic function \code{\link{intensity}}.
  It computes an estimate of the intensity of a point pattern
  from its quadrat counts.

  The argument \code{X} should be an object of class
  \code{"quadratcount"}. It would have been obtained by applying the function
  \code{\link{quadratcount}} to a point pattern
  (object of class \code{"ppp"}). It contains
  the counts of the numbers of points of the point pattern falling in each
  tile of a tessellation.

  Using this information, \code{intensity.quadratcount}
  divides the quadrat counts by the tile areas,
  yielding the average density of points per unit area
  in each tile of the tessellation.

  If \code{image=FALSE} (the default), these intensity values
  are returned in a contingency table. Cells of the contingency
  table correspond to tiles of the tessellation.

  If \code{image=TRUE}, the estimated intensity function is
  returned as a pixel image. For each pixel, the pixel value is the
  estimated intensity in the tile which contains that pixel.
}
\value{
  If \code{image=FALSE} (the default), a contingency table.
  If \code{image=TRUE}, a pixel image (object of class \code{"im"}).
}
\seealso{
  \code{\link{intensity}}, 
  \code{\link{quadratcount}}
}
\examples{
  qa <- quadratcount(swedishpines, 4,3)
  qa
  intensity(qa)
  plot(intensity(qa, image=TRUE))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{nonparametric}

\name{plot.symbolmap}
\alias{plot.symbolmap}
\title{
  Plot a Graphics Symbol Map
}
\description{
  Plot a representation of a graphics symbol map,
  similar to a plot legend.
}
\usage{
\method{plot}{symbolmap}(x, \dots, main, xlim = NULL, ylim = NULL,
                         vertical = FALSE,
                         side = c("bottom", "left", "top", "right"),
                         annotate = TRUE, labelmap = NULL, add = FALSE,
                         nsymbols = NULL)
} 
\arguments{
  \item{x}{
    Graphics symbol map (object of class \code{"symbolmap"}).
  }
  \item{\dots}{
    Additional graphics arguments passed to
    \code{\link{points}}, \code{\link{symbols}} or \code{\link{axis}}.
  }
  \item{main}{
    Main title for the plot. A character string.
  }
  \item{xlim,ylim}{
    Coordinate limits for the plot.
    Numeric vectors of length 2.
  }
  \item{vertical}{
    Logical. Whether to plot the symbol map in a vertical orientation.
  }
  \item{side}{
    Character string specifying the position of the text
    that annotates the symbols.
  }
  \item{annotate}{
    Logical. Whether to annotate the symbols with labels.
  }
  \item{labelmap}{
    Transformation of the labels.
    A function or a scale factor which will be applied to
    the data values corresponding to the plotted symbols.
  }
  \item{add}{
    Logical value indicating whether to add the plot to the
    current plot (\code{add=TRUE}) or to initialise a new plot.
  }
  \item{nsymbols}{
    Optional. The number of symbols that should be displayed.
    (This may not be exactly obeyed.)
  }
}
\details{
  A graphics symbol map is
  an association between data values and graphical symbols. 

  This command plots the graphics symbol map itself,
  in the style of a plot legend.
}
\value{
  None.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{symbolmap}} to create a symbol map.
  
  \code{\link{invoke.symbolmap}} to apply the symbol map to some data
  and plot the resulting symbols.
}
\examples{
  g <- symbolmap(inputs=letters[1:10], pch=11:20)
  plot(g)

  g2 <- symbolmap(range=c(-1,1),
                    shape=function(x) ifelse(x > 0, "circles", "squares"),
                    size=function(x) sqrt(ifelse(x > 0, x/pi, -x)),
                    bg = function(x) ifelse(abs(x) < 1, "red", "black"))
  plot(g2, vertical=TRUE, side="left", col.axis="blue", cex.axis=2)
}
\keyword{spatial}
\keyword{hplot}

\name{nearestValue}
\alias{nearestValue}
\title{
  Image of Nearest Defined Pixel Value
}
\description{
  Given a pixel image defined on a subset of a rectangle,
  this function assigns a value to every pixel in the rectangle,
  by looking up the value of the nearest pixel that has a value.
}
\usage{
nearestValue(X)
}
\arguments{
  \item{X}{
    A pixel image (object of class \code{"im"}).
  }
}
\details{
  A pixel image in \pkg{spatstat} is always stored on
  a rectangular grid of pixels, but its value may be \code{NA}
  on some pixels, indicating that the image is not defined at those
  pixels.

  This function assigns a value to every pixel in the rectangular grid.
  For each pixel \code{a} in the grid, if the value of \code{X} is not
  defined at \code{a}, the function finds the nearest other pixel \code{b}
  at which the value of \code{X} is defined, and takes the pixel value at
  \code{b} as the new pixel value at \code{a}.
}
\value{
  Another image of the same kind as \code{X}.
}
\author{
  \adrian.
}
\seealso{
  \code{\link{blur}}, \code{\link{Smooth.ppp}}
}
\examples{
  X <- as.im(function(x,y) { x + y }, letterR)
  Y <- nearestValue(X)
  plot(solist("X"=X,"nearestValue(X)"=Y), main="", panel.end=letterR)
}
\keyword{spatial}
\keyword{manip}

\name{discpartarea}
\Rdversion{1.1}
\alias{discpartarea}
\title{
  Area of Part of Disc
}
\description{
  Compute area of intersection between a disc and a window  
}
\usage{
discpartarea(X, r, W=as.owin(X))
}
\arguments{
  \item{X}{
    Point pattern (object of class \code{"ppp"})
    specifying the centres of the discs.
    Alternatively, \code{X} may be in any format
    acceptable to \code{\link{as.ppp}}.
  }
  \item{r}{
    Matrix, vector or numeric value specifying the
    radii of the discs.
  }
  \item{W}{
    Window (object of class \code{"owin"}) with which the
    discs should be intersected.
  }
}
\details{
  This algorithm computes the exact area of the intersection between
  a window \code{W} and a disc (or each of several discs).
  The centres of the discs are specified by the point pattern
  \code{X}, and their radii are specified by \code{r}.

  If \code{r} is a single numeric value, then the algorithm computes the
  area of intersection between \code{W} and the disc of radius \code{r} centred
  at each point of \code{X}, and returns a one-column matrix
  containing one entry for each point of \code{X}.

  If \code{r} is a vector of length \code{m}, then the algorithm
  returns an \code{n * m} matrix in which the entry on row \code{i},
  column \code{j} is the area of the
  intersection between \code{W} and the disc centred at \code{X[i]}
  with radius \code{r[j]}.
  
  If \code{r} is a matrix, it should have one row for each point in
  \code{X}. The algorithm
  returns a matrix in which the entry on row \code{i},
  column \code{j} is the area of the
  intersection between \code{W} and the disc centred at \code{X[i]}
  with radius \code{r[i,j]}.

  Areas are computed by analytic geometry.
}
\value{
  Numeric matrix, with one row for each point of \code{X}.
}
\seealso{
  \code{\link{owin}},
  \code{\link{disc}}
}
\examples{
  data(letterR)
  X <- runifpoint(3, letterR)
  discpartarea(X, 0.2)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{Math.im}
\alias{Math.im}
\alias{Ops.im}
\alias{Complex.im}
\alias{Summary.im}
\title{S3 Group Generic methods for images}
\description{
  These are group generic methods for images of class \code{"im"}, which
  allows for usual mathematical functions and operators to be applied
  directly to images. See Details for a list of implemented functions.
}
\usage{
## S3 methods for group generics have prototypes:
\special{Math(x, \dots)}
\special{Ops(e1, e2)}
\special{Complex(z)}
\special{Summary(\dots, na.rm=FALSE, drop=TRUE)}
%NAMESPACE S3method("Math", "im")
%NAMESPACE S3method("Ops", "im")
%NAMESPACE S3method("Complex", "im")
%NAMESPACE S3method("Summary", "im")
}
\arguments{
  \item{x, z, e1, e2}{objects of class \code{"im"}.}
  \item{\dots}{further arguments passed to methods.}
  \item{na.rm,drop}{
    Logical values specifying whether missing values
    should be removed. This will happen if either
    \code{na.rm=TRUE} or \code{drop=TRUE}. See Details.
  }
}

\details{
  Below is a list of mathematical functions and operators which are
  defined for images. Not all functions will make sense for all
  types of images. For example, none of the functions in the \code{"Math"} group
  make sense for character-valued images. Note that the \code{"Ops"}
  group methods are implemented using \code{\link{eval.im}}, which tries
  to harmonise images via \code{\link{harmonise.im}} if they aren't
  compatible to begin with.

  \enumerate{
    \item Group \code{"Math"}:
    \itemize{
      \item
      \code{abs}, \code{sign}, \code{sqrt},\cr
      \code{floor}, \code{ceiling}, \code{trunc},\cr
      \code{round}, \code{signif}

      \item
      \code{exp}, \code{log},  \code{expm1}, \code{log1p},\cr
      \code{cos}, \code{sin}, \code{tan},\cr
      \code{cospi}, \code{sinpi}, \code{tanpi},\cr
      \code{acos}, \code{asin}, \code{atan}

      \code{cosh}, \code{sinh}, \code{tanh},\cr
      \code{acosh}, \code{asinh}, \code{atanh}

      \item
      \code{lgamma}, \code{gamma}, \code{digamma}, \code{trigamma}
      \item \code{cumsum}, \code{cumprod}, \code{cummax}, \code{cummin}
    }

    \item Group \code{"Ops"}:
    \itemize{
      \item
      \code{"+"}, \code{"-"}, \code{"*"}, \code{"/"},
      \code{"^"}, \code{"\%\%"}, \code{"\%/\%"}

      \item \code{"&"}, \code{"|"}, \code{"!"}

      \item \code{"=="}, \code{"!="},
      \code{"<"}, \code{"<="}, \code{">="}, \code{">"}
    }

    \item Group \code{"Summary"}:
    \itemize{
      \item \code{all}, \code{any}
      \item \code{sum}, \code{prod}
      \item \code{min}, \code{max}
      \item \code{range}
    }

    \item Group \code{"Complex"}:
    \itemize{
      \item \code{Arg}, \code{Conj}, \code{Im}, \code{Mod}, \code{Re}
    }
  }

  For the \code{Summary} group, the generic has an argument
  \code{na.rm=FALSE}, but for pixel images it makes
  sense to set \code{na.rm=TRUE} so that pixels outside the domain of
  the image are ignored. To enable this, we added the argument
  \code{drop}. Pixel values that are \code{NA} are removed
  if \code{drop=TRUE} or if \code{na.rm=TRUE}.

  For the \code{Ops} group, one of the arguments is permitted to be
  a single atomic value instead of an image.
}
\seealso{
  \code{\link{eval.im}} for evaluating expressions involving images.
}
\examples{
  ## Convert gradient values to angle of inclination:
  V <- atan(bei.extra$grad) * 180/pi
  ## Make logical image which is TRUE when heat equals 'Moderate': 
  A <- (gorillas.extra$heat == "Moderate")
  ## Summary:
  any(A)
  ## Complex:
  Z <- exp(1 + V * 1i)
  Z
  Re(Z)
}
\author{
  \spatstatAuthors and Kassel Hingee.
}
\keyword{spatial}
\keyword{methods}

\name{is.subset.owin}
\alias{is.subset.owin}
\title{Determine Whether One Window is Contained In Another}
\description{
  Tests whether window \code{A} is a subset of window \code{B}.
}
\usage{
is.subset.owin(A, B)
}
\arguments{
  \item{A}{A window object (see Details).}
  \item{B}{A window object (see Details).}
}
\value{
  Logical scalar; \code{TRUE} if \code{A} is a sub-window of \code{B},
  otherwise \code{FALSE}.
}
\details{
  This function tests whether the window \code{A} is a subset
  of the window \code{B}.

  The arguments \code{A} and \code{B} must be window objects
  (either objects of class \code{"owin"}, or data that can be
  coerced to this class by \code{\link{as.owin}}).

  Various algorithms are used, depending on the geometrical type
  of the two windows.

  Note that if \code{B} is not rectangular, the algorithm proceeds by
  discretising \code{A}, converting it to a pixel mask using 
  \code{\link{as.mask}}. In this case the resulting
  answer is only ``approximately correct''. The accuracy of the
  approximation can be controlled: see \code{\link{as.mask}}.
}

\author{\adrian
  
  
  and \rolf
  
}

\examples{
   w1 <- as.owin(c(0,1,0,1))
   w2 <- as.owin(c(-1,2,-1,2))
   is.subset.owin(w1,w2)  # Returns TRUE.
   is.subset.owin(w2,w1)  # Returns FALSE.
}
\keyword{spatial}
\keyword{math}

\name{intersect.lintess}
\alias{intersect.lintess}
\title{
  Intersection of Tessellations on a Linear Network
}
\description{
  Yields the intersection (common refinement) of two tessellations
  on a linear network.
}
\usage{
intersect.lintess(X, Y)
}
\arguments{
  \item{X,Y}{
    Tessellations (objects of class \code{"lintess"})
    on the same linear network.
  }
}
\value{
  Another tessellation (object of class \code{"lintess"})
  on the same linear network as \code{X} and \code{Y}.
}
\details{
  \code{X} and \code{Y} should be tessellations defined on the same
  linear network.
  
  Each tile in the resulting tessellation is the intersection of
  a tile of \code{X} with a tile of \code{Y}.
}
\author{
  \adrian.
}
\seealso{
  \code{\link{lintess}},
  \code{\link{divide.linnet}},
  \code{\link{chop.linnet}}
}
\examples{
  X <- divide.linnet(runiflpp(4, simplenet))
  Y <- divide.linnet(runiflpp(3, simplenet))
  opa <- par(mfrow=c(1,3))
  plot(X)
  plot(Y)
  plot(intersect.lintess(X,Y))
  par(opa)
}
\keyword{spatial}
\keyword{manip}

\name{plot.lpp}
\alias{plot.lpp}
\title{
  Plot Point Pattern on Linear Network
}
\description{
  Plots a point pattern on a linear network.
  Plot method for the class \code{"lpp"} of point patterns on a linear network.
}
\usage{
\method{plot}{lpp}(x, \dots, main, add = FALSE,
                   use.marks=TRUE, which.marks=NULL,
                   show.all = !add, show.window=FALSE, show.network=TRUE, 
                   do.plot = TRUE, multiplot=TRUE) 
}
\arguments{
  \item{x}{
    Point pattern on a linear network (object of class \code{"lpp"}).
  }
  \item{\dots}{
    Additional arguments passed to \code{\link{plot.linnet}}
    or \code{\link{plot.ppp}}.
  }
  \item{main}{
    Main title for plot.
  }
  \item{add}{
    Logical value indicating whether the plot is to be added to the
    existing plot (\code{add=TRUE}) or whether a new plot should be
    initialised (\code{add=FALSE}, the default).
  }
  \item{use.marks}{
    logical flag; if \code{TRUE}, plot points using a different
    plotting symbol for each mark;
    if \code{FALSE}, only the locations of the points will be plotted,
    using \code{\link{points}()}.
  }
  \item{which.marks}{
    Index determining which column of marks to use,
    if the marks of \code{x} are a data frame.
    A character or integer vector identifying one or more
    columns of marks. 
    If \code{add=FALSE} then
    the default is to plot all columns of marks, in a series of separate
    plots.
    If \code{add=TRUE} then only one column of marks can be plotted,
    and the default is \code{which.marks=1}
    indicating the first column of marks.
  }
  \item{show.all}{
    Logical value indicating whether to plot everything
    including the main title and the window containing the network.
  }
  \item{show.window}{
    Logical value indicating whether to plot the
    window containing the network. Overrides \code{show.all}.
  }
  \item{show.network}{
    Logical value indicating whether to plot the network.
  }
  \item{do.plot}{
    Logical value determining whether to actually perform the plotting.
  }
  \item{multiplot}{
    Logical value giving permission to display multiple plots.
  }
}
\details{
  The linear network is plotted by \code{\link{plot.linnet}},
  then the points are plotted by \code{\link{plot.ppp}}.

  Commonly-used arguments include:
  \itemize{
    \item \code{col} and \code{lwd} for the colour and width of lines
    in the linear network
    \item \code{cols} for the colour or colours of the points
    \item \code{chars} for the plot characters representing different
    types of points
    \item \code{legend} and \code{leg.side} to control the graphics
    legend
  }

  Note that the linear network will be plotted
  even when \code{add=TRUE}, unless \code{show.network=FALSE}.
}
\value{
  (Invisible) object of class \code{"symbolmap"}
  giving the correspondence between 
  mark values and plotting characters.
}
\seealso{
  \code{\link{lpp}}.

  See \code{\link{plot.ppp}} for options for representing the points.

  See also \code{\link{points.lpp}}, \code{\link{text.lpp}}.
}
\examples{
  plot(chicago, cols=1:6)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{hplot}

\name{ripras}
\alias{ripras}
\title{Estimate window from points alone}
\description{
  Given an observed pattern of points,
  computes the Ripley-Rasson estimate of 
  the spatial domain from which they came.
}
\usage{
 ripras(x, y=NULL, shape="convex", f)
}
\arguments{
  \item{x}{
    vector of \code{x} coordinates of observed points,
    or a 2-column matrix giving \code{x,y} coordinates,
    or a list with components \code{x,y} giving coordinates
    (such as a point pattern object of class \code{"ppp"}.)
  }
  \item{y}{(optional) vector of \code{y} coordinates of observed points,
    if \code{x} is a vector.}
  \item{shape}{String indicating the type of window to be estimated:
    either \code{"convex"} or \code{"rectangle"}.
  }
  \item{f}{
    (optional) scaling factor. See Details.
  }
}
\value{
  A window (an object of class \code{"owin"}).
}
\details{
  Given an observed pattern of points with coordinates 
  given by \code{x} and \code{y}, this function computes 
  an estimate due to Ripley and Rasson (1977) of the
  spatial domain from which the points came. 

  The points are
  assumed to have been generated independently and uniformly
  distributed inside an unknown domain \eqn{D}.

  If \code{shape="convex"} (the default), the domain \eqn{D} is assumed
  to be a convex set. The maximum
  likelihood estimate of \eqn{D} is the convex hull of the 
  points (computed by \code{\link{convexhull.xy}}).
  Analogously to the problems of estimating the endpoint
  of a uniform distribution, the MLE is not optimal.
  Ripley and Rasson's estimator is a rescaled copy of the convex hull,
  centred at the centroid of the convex hull.
  The scaling factor is 
  \eqn{1/sqrt(1 - m/n)}{1/\sqrt{1 - \frac m n}}
  where \eqn{n} is the number of data points and 
  \eqn{m} the number of vertices of the convex hull.
  The scaling factor may be overridden using the argument \code{f}.

  If \code{shape="rectangle"}, the domain \eqn{D} is assumed
  to be a rectangle with sides parallel to the coordinate axes. The maximum
  likelihood estimate of \eqn{D} is the bounding box of the points
  (computed by \code{\link{bounding.box.xy}}). The Ripley-Rasson
  estimator is a rescaled copy of the bounding box,
  with scaling factor \eqn{(n+1)/(n-1)}
  where \eqn{n} is the number of data points,
  centred at the centroid of the bounding box.
  The scaling factor may be overridden using the argument \code{f}.
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{bounding.box.xy}},
  \code{\link{convexhull.xy}}
}
\examples{
  x <- runif(30)
  y <- runif(30)
  w <- ripras(x,y)
  plot(owin(), main="ripras(x,y)")
  plot(w, add=TRUE)
  points(x,y)

  X <- rpoispp(15)
  plot(X, main="ripras(X)")
  plot(ripras(X), add=TRUE)

  # two points insufficient
  ripras(c(0,1),c(0,0))
  # triangle
  ripras(c(0,1,0.5), c(0,0,1))
  # three collinear points
  ripras(c(0,0,0), c(0,1,2))
}
\references{
  Ripley, B.D. and Rasson, J.-P. (1977)
  Finding the edge of a Poisson forest.
  \emph{Journal of Applied Probability},
  \bold{14}, 483 -- 491.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{utilities}

\name{quantile.ewcdf}
\alias{quantile.ewcdf}
\title{
  Quantiles of Weighted Empirical Cumulative Distribution Function
}
\description{
  Compute quantiles of a weighted empirical cumulative distribution function.  
}
\usage{
  \method{quantile}{ewcdf}(x, probs = seq(0, 1, 0.25),
                 names = TRUE, \dots,
                 normalise = TRUE, type=1)
}
\arguments{
  \item{x}{
    A weighted empirical cumulative distribution function
    (object of class \code{"ewcdf"}, produced by \code{\link{ewcdf}})
    for which the quantiles are desired.
  }
  \item{probs}{
    probabilities for which the quantiles are desired.
    A numeric vector of values between 0 and 1.
  }
  \item{names}{
    Logical. If \code{TRUE}, the resulting vector of quantiles is
    annotated with names corresponding to \code{probs}.
  }
  \item{\dots}{
    Ignored.
  }
  \item{normalise}{
    Logical value indicating whether \code{x} should first be normalised
    so that it ranges between 0 and 1. 
  }
  \item{type}{
    Integer specifying the type of quantile to be calculated,
    as explained in \code{\link[stats]{quantile.default}}.
    Only types 1 and 2 are currently implemented.
  }
}
\details{
  This is a method for the generic \code{\link[stats]{quantile}}
  function for the class \code{ewcdf} of empirical weighted cumulative
  distribution functions.

  The quantile for a probability \code{p} is computed
  as the right-continuous inverse of the cumulative
  distribution function \code{x} (assuming \code{type=1}, the default).

  If \code{normalise=TRUE} (the default),
  the weighted cumulative function \code{x} is first normalised to
  have total mass \code{1} so that it can be interpreted as a
  cumulative probability distribution function. 
}
\value{
  Numeric vector of quantiles, of the same length as \code{probs}.
}
\seealso{
  \code{\link{ewcdf}}, 
  \code{\link[stats]{quantile}}
}
\examples{
  z <- rnorm(50)
  w <- runif(50)
  Fun <- ewcdf(z, w)
  quantile(Fun, c(0.95,0.99))
}
\author{
  \spatstatAuthors
  and Kevin Ummel.
}
\keyword{spatial}
\keyword{nonparametric}

\name{plot.texturemap}
\alias{plot.texturemap}
\title{
  Plot a Texture Map
}
\description{
  Plot a representation of a texture map,
  similar to a plot legend.
}
\usage{
 \method{plot}{texturemap}(x, \dots, main, xlim = NULL, ylim = NULL,
                           vertical = FALSE, axis = TRUE,
                           labelmap = NULL, gap = 0.25,
                           spacing = NULL, add = FALSE)
}
\arguments{
  \item{x}{
    Texture map object (class \code{"texturemap"}).
  }
  \item{\dots}{
    Additional graphics arguments passed to
    \code{\link{add.texture}} or \code{\link{axis.default}}.
  }
  \item{main}{
    Main title for plot.
  }
  \item{xlim,ylim}{
    Optional vectors of length 2 giving the \eqn{x} and \eqn{y} limits
    of the plot.
  }
  \item{vertical}{
    Logical value indicating whether to arrange the texture boxes
    in a vertical column (\code{vertical=TRUE} or a horizontal row
    (\code{vertical=FALSE}, the default).
  }
  \item{axis}{
    Logical value indicating whether to plot an axis line
    joining the texture boxes.
  }
  \item{labelmap}{
    Optional. A \code{function} which will be applied to the
    data values (the inputs of the texture map) before they are
    displayed on the plot.
  }
  \item{gap}{
    Separation between texture boxes, as a fraction of the
    width or height of a box.
  }
  \item{spacing}{
    Argument passed to \code{\link{add.texture}} controlling the density
    of lines in a texture. Expressed in spatial coordinate units.
  }
  \item{add}{
    Logical value indicating whether to add the graphics to an existing
    plot (\code{add=TRUE}) or to initialise a new plot
    (\code{add=FALSE}, the default).
  }
}
\details{
  A texture map is an association between data values and graphical
  textures. An object of class \code{"texturemap"} represents a texture
  map. Such objects are returned from the plotting function
  \code{\link{textureplot}}, and can be created directly by the function
  \code{\link{texturemap}}.

  This function \code{plot.texturemap} is a method for the generic
  \code{\link{plot}} for the class \code{"texturemap"}. It displays
  a sample of each of the textures in the texture map, in a separate
  box, annotated by the data value which is mapped to that texture.

  The arrangement and position of the boxes is controlled by
  the arguments \code{vertical}, \code{xlim}, \code{ylim} and
  \code{gap}.
}
\value{
  Null.
}
\author{\adrian
  
  
  \rolf
  
  and \ege
}
\seealso{
  \code{\link{texturemap}},
  \code{\link{textureplot}},
  \code{\link{add.texture}}.
}
\examples{
   tm <- texturemap(c("First", "Second", "Third"), 2:4, col=2:4)
   plot(tm, vertical=FALSE)
   ## abbreviate the labels
   plot(tm, labelmap=function(x) substr(x, 1, 2))
}
\keyword{spatial}
\keyword{hplot}

\name{FmultiInhom}
\alias{FmultiInhom}
\title{
  Inhomogeneous Marked F-Function
}
\description{
  For a marked point pattern, 
  estimate the inhomogeneous version of the multitype \eqn{F} function,
  effectively the cumulative distribution function of the distance from
  a fixed point to the nearest point in subset \eqn{J},
  adjusted for spatially varying intensity.
}
\usage{
  FmultiInhom(X, J,
              lambda = NULL, lambdaJ = NULL, lambdamin = NULL,
              \dots,
              r = NULL)
}
\arguments{
  \item{X}{
    A spatial point pattern (object of class \code{"ppp"}.
  }
  \item{J}{
    A subset index specifying the subset of points to which
    distances are measured. Any kind of subset index acceptable
    to \code{\link{[.ppp}}.
  }
  \item{lambda}{
    Intensity estimates for each point of \code{X}.
    A numeric vector of length equal to \code{npoints(X)}.
    Incompatible with \code{lambdaJ}.
  }
  \item{lambdaJ}{
    Intensity estimates for each point of \code{X[J]}.
    A numeric vector of length equal to \code{npoints(X[J])}.
    Incompatible with \code{lambda}.
  }
  \item{lambdamin}{
    A lower bound for the intensity,
    or at least a lower bound for the values in \code{lambdaJ}
    or \code{lambda[J]}.
  }
  \item{\dots}{
    Ignored. 
  }
  \item{r}{
    Vector of distance values at which the inhomogeneous \eqn{G}
    function should be estimated. There is a sensible default.
  }
}
\details{
  See Cronie and Van Lieshout (2015).
}
\value{
  Object of class \code{"fv"} containing the estimate of the
  inhomogeneous multitype \eqn{F} function.
}
\references{
  Cronie, O. and Van Lieshout, M.N.M. (2015)
  Summary statistics for inhomogeneous marked point processes.
  \emph{Annals of the Institute of Statistical Mathematics}
  DOI: 10.1007/s10463-015-0515-z
}
\author{
  Ottmar Cronie and Marie-Colette van Lieshout.
  Rewritten for \pkg{spatstat} by \adrian.
}
\seealso{
  \code{\link{Finhom}}
}
\examples{
  X <- amacrine
  J <- (marks(X) == "off")
  mod <- ppm(X ~ marks * x)
  lam <- fitted(mod, dataonly=TRUE)
  lmin <- min(predict(mod)[["off"]]) * 0.9
  plot(FmultiInhom(X, J, lambda=lam, lambdamin=lmin))
}
\keyword{spatial}
\keyword{nonparametric}

\name{pixelquad}
\alias{pixelquad}
\title{Quadrature Scheme Based on Pixel Grid}
\description{
  Makes a quadrature scheme with a dummy point at every pixel
  of a pixel image.
}
\usage{
pixelquad(X, W = as.owin(X))
}
\arguments{
  \item{X}{Point pattern (object of class \code{"ppp"}) containing the
    data points for the quadrature scheme.
  }
  \item{W}{
    Specifies the pixel grid.
    A pixel image (object of class \code{"im"}),
    a window (object of class \code{"owin"}), or anything that can
    be converted to a window by \code{\link{as.owin}}.
  }
}
\value{
  An object of class \code{"quad"} describing the quadrature scheme
  (data points, dummy points, and quadrature weights)
  suitable as the argument \code{Q} of the function \code{\link{ppm}()} for
  fitting a point process model.

  The quadrature scheme can be inspected using the
  \code{print} and \code{plot} methods for objects
  of class \code{"quad"}. 
}
\details{
  This is a method for producing a quadrature scheme
  for use by \code{\link{ppm}}. It is an alternative to
  \code{\link{quadscheme}}.
  
  The function \code{\link{ppm}} fits a point process model to an
  observed point pattern using 
  the Berman-Turner quadrature approximation (Berman and Turner, 1992;
  Baddeley and Turner, 2000) to the pseudolikelihood of the model. 
  It requires a quadrature scheme consisting of 
  the original data point pattern, an additional pattern of dummy points,
  and a vector of quadrature weights for all these points.
  Such quadrature schemes are represented by objects of class
  \code{"quad"}. See \code{\link{quad.object}} for a description of this class.

  Given a grid of pixels, this function creates a quadrature scheme
  in which there is one dummy point at the centre of each pixel. The
  counting weights are used (the weight attached to each quadrature
  point is 1 divided by the number of quadrature points falling
  in the same pixel).

  The argument \code{X} specifies the locations of the data points
  for the quadrature scheme. Typically this would be a point pattern
  dataset.

  The argument \code{W} specifies the grid of pixels for the dummy
  points of the quadrature scheme. It should be a pixel image
  (object of class \code{"im"}), a window (object of class
  \code{"owin"}), or anything that can
  be converted to a window by \code{\link{as.owin}}. If \code{W} is a
  pixel image or a binary mask (a window of type \code{"mask"})
  then the pixel grid of \code{W} will be used. If \code{W} is a
  rectangular or polygonal window, then it will first be converted to a
  binary mask using \code{\link{as.mask}} at the default pixel
  resolution.
}
\examples{
  W <- owin(c(0,1),c(0,1))
  X <- runifpoint(42, W)
  W <- as.mask(W,dimyx=128)
  pixelquad(X,W)
}
\seealso{
  \code{\link{quadscheme}},
  \code{\link{quad.object}},
  \code{\link{ppm}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{zapsmall.im}
\alias{zapsmall.im}
\title{Rounding of Pixel Values}
\description{
  Modifies a pixel image, identifying those pixels that have values 
  very close to zero, and replacing the value by zero.
}
\usage{
  zapsmall.im(x, digits)
}
\arguments{
  \item{x}{Pixel image (object of class \code{"im"}).}
  \item{digits}{
    Argument passed to \code{\link{zapsmall}}
    indicating the precision to be used.
  }
}
\details{
  The function \code{\link{zapsmall}} is applied to each pixel value
  of the image \code{x}.
}
\value{
  Another pixel image.
}
\seealso{
  \code{\link{zapsmall}}
}
\examples{
  data(cells)
  D <- density(cells)
  zapsmall.im(D)
}
\author{\ege
  and \adrian
  
  
}
\keyword{spatial}
\keyword{methods}
\keyword{univar}

\name{clip.infline}
\alias{clip.infline}
\title{Intersect Infinite Straight Lines with a Window}
\description{
  Take the intersection between a set of infinite straight lines
  and a window, yielding a set of line segments.
}
\usage{
clip.infline(L, win)
}
\arguments{
  \item{L}{
    Object of class \code{"infline"} specifying a set of infinite
    straight lines in the plane.
  }
  \item{win}{
    Window (object of class \code{"owin"}).
  }
}
\details{
  This function computes the intersection between
  a set of infinite straight lines in the plane
  (stored in an object \code{L} of class \code{"infline"} created by the
  function \code{\link{infline}}) and a window \code{win}.
  The result is a pattern of line segments. Each line segment carries a
  mark indicating which line it belongs to.
}
\value{
  A line segment pattern (object of class \code{"psp"})
  with a single column of marks.
}
\author{
  \adrian
  and \rolf.
}
\seealso{
  \code{\link{infline}},\code{\link{psp}}.

  To divide a window into pieces using infinite lines,
  use \code{\link{chop.tess}}.
}
\examples{
  L <- infline(p=1:3, theta=pi/4)
  W <- square(4)
  clip.infline(L, W)
}
\keyword{spatial}
\keyword{math}

\name{ewcdf}
\alias{ewcdf}
\title{Weighted Empirical Cumulative Distribution Function}
\description{
  Compute a weighted version of the
  empirical cumulative distribution function.
}
\usage{
ewcdf(x, weights = NULL, normalise=TRUE, adjust=1)
}
\arguments{
  \item{x}{Numeric vector of observations.}
  \item{weights}{
    Optional. Numeric vector of non-negative weights for \code{x}.
    Defaults to equal weight 1 for each entry of \code{x}.
  }
  \item{normalise}{
    Logical value indicating whether the weights should be rescaled
    so that they sum to 1.
  }
  \item{adjust}{
    Numeric value. Adjustment factor.
    The weights will be multiplied by \code{adjust}.
  }
}
\details{
  This is a modification of the standard function \code{\link{ecdf}}
  allowing the observations \code{x} to have weights.

  The weighted e.c.d.f. (empirical cumulative distribution function)
  \code{Fn} is defined so that, for any real number \code{y}, the value of
  \code{Fn(y)} is equal to the total weight of all entries of
  \code{x} that are less than or equal to \code{y}. That is
  \code{Fn(y) = sum(weights[x <= y])}.

  Thus \code{Fn} is a step function which jumps at the
  values of \code{x}. The height of the jump at a point \code{y}
  is the total weight of all entries in \code{x} 
  number of tied observations at that value.  Missing values are
  ignored.

  If \code{weights} is omitted, the default is equivalent to
  \code{ecdf(x)} except for the class membership.

  The result of \code{ewcdf} is a function, of class \code{"ewcdf"},
  inheriting from the classes \code{"ecdf"} (if \code{normalise=TRUE})
  and \code{"stepfun"}.
  The class \code{ewcdf} has methods for \code{print} and \code{quantile}.
  The inherited classes \code{ecdf} and \code{stepfun}
  have methods for \code{plot} and \code{summary}.
}
\value{
  A function, of class \code{"ewcdf"}, inheriting from 
  \code{"ecdf"} (if \code{normalise=TRUE}) and \code{"stepfun"}. 
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{ecdf}}.
  
  \code{\link{quantile.ewcdf}}
}
\examples{
   x <- rnorm(100)
   w <- runif(100)
   plot(e <- ewcdf(x,w))
   e
}
\keyword{nonparametric}
\keyword{univar}

\name{closepairs}
\alias{closepairs}
\alias{crosspairs}
\alias{closepairs.ppp}
\alias{crosspairs.ppp}
\alias{closepaircounts}
\alias{crosspaircounts}
\title{
  Close Pairs of Points
}
\description{
  Low-level functions to find all close pairs of points.
}
\usage{
closepaircounts(X, r)

crosspaircounts(X, Y, r)

closepairs(X, rmax, \dots)

\method{closepairs}{ppp}(X, rmax, twice=TRUE,
                         what=c("all","indices","ijd"),
                         distinct=TRUE, neat=TRUE,
                         periodic=FALSE, \dots)

crosspairs(X, Y, rmax, \dots)

\method{crosspairs}{ppp}(X, Y, rmax, what=c("all", "indices", "ijd"), \dots)
}
\arguments{
  \item{X,Y}{
    Point patterns (objects of class \code{"ppp"}).
  }
  \item{r,rmax}{
    Maximum distance between pairs of points to be counted as close pairs.
  }
  \item{twice}{
    Logical value indicating whether all ordered pairs of close points
    should be returned. If \code{twice=TRUE} (the default),
    each pair will appear twice in the output, as \code{(i,j)}
    and again as \code{(j,i)}. If \code{twice=FALSE},
    then each pair will appear only once, as the pair \code{(i,j)}
    with \code{i < j}.
  }
  \item{what}{
    String specifying the data to be returned for each close pair of points.
    If \code{what="all"} (the default) then the
    returned information includes the indices \code{i,j} of each pair,
    their \code{x,y} coordinates, and the distance between them.
    If \code{what="indices"} then only the indices \code{i,j} are
    returned.
    If \code{what="ijd"} then the indices \code{i,j} and the
    distance \code{d} are returned.
  }
  \item{distinct}{
    Logical value indicating whether to return only the
    pairs of points with different indices \code{i} and \code{j}
    (\code{distinct=TRUE}, the default) or to also include
    the pairs where \code{i=j} (\code{distinct=FALSE}).
  }
  \item{neat}{
    Logical value indicating whether to ensure that \code{i < j}
    in each output pair, when \code{twice=FALSE}. 
  }
  \item{periodic}{
    Logical value indicating whether to use the periodic edge correction.
    The window of \code{X} should be a rectangle.
    Opposite pairs of edges of the window will be treated as identical.
  }
  \item{\dots}{Extra arguments, ignored by methods.}
}
\details{
  These are the efficient low-level functions used by \pkg{spatstat}
  to find all close pairs of points in a point pattern
  or all close pairs between two point patterns. 

  \code{closepaircounts(X,r)} counts the number of neighbours for
  each point in the pattern \code{X}. That is, for each point
  \code{X[i]}, it counts the number of other points \code{X[j]}
  with \code{j != i} such that \code{d(X[i],X[j]) <= r} where
  \code{d} denotes Euclidean distance. The result is an integer vector
  \code{v} such that \code{v[i]} is the number of neighbours of
  \code{X[i]}.

  \code{crosspaircounts(X,Y,r)} counts, for each point 
  in the pattern \code{X}, the number of neighbours in the pattern
  \code{Y}. That is, for each point
  \code{X[i]}, it counts the number of points \code{Y[j]}
  such that \code{d(X[i],X[j]) <= r}. The result is an integer vector
  \code{v} such that \code{v[i]} is the number of neighbours of
  \code{X[i]} in the pattern \code{Y}.

  \code{closepairs(X,rmax)} identifies all pairs of distinct neighbours 
  in the pattern \code{X} and returns them. The result is
  a list with the following components:
  \describe{
    \item{i}{Integer vector of indices of the first point in each pair.}
    \item{j}{Integer vector of indices of the second point in each pair.}
    \item{xi,yi}{Coordinates of the first point in each pair.}
    \item{xj,yj}{Coordinates of the second point in each pair.}
    \item{dx}{Equal to \code{xj-xi}}
    \item{dy}{Equal to \code{yj-yi}}
    \item{d}{Euclidean distance between each pair of points.}
  }
  If \code{what="indices"} then only the components \code{i} and
  \code{j} are returned. This is slightly faster and more efficient
  with use of memory.

  \code{crosspairs(X,rmax)} identifies all pairs of neighbours
  \code{(X[i], Y[j])} between the patterns \code{X} and \code{Y},
  and returns them. The result is
  a list with the same format as for \code{closepairs}.
}
\section{Warning about accuracy}{
  The results of these functions may not agree exactly with
  the correct answer (as calculated by a human) and may not
  be consistent between different computers and different installations
  of \R. The discrepancies arise in marginal cases where the interpoint
  distance is equal to, or very close to, the threshold \code{rmax}.

  Floating-point numbers in a computer
  are not mathematical Real Numbers: they are approximations using
  finite-precision binary arithmetic.
  The approximation is accurate to a tolerance of about
  \code{.Machine$double.eps}.

  If the true interpoint distance \eqn{d} and the threshold \code{rmax}
  are equal, or if their difference is no more than \code{.Machine$double.eps},
  the result may be incorrect.
}
\value{
  For \code{closepaircounts} and \code{crosspaircounts}, an integer
  vector of length equal to the number of points in \code{X}.

  For \code{closepairs} and \code{crosspairs}, 
  a list with components \code{i} and \code{j},
  and possibly other components as described under Details.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{closepairs.pp3}} for the corresponding
  functions for 3D point patterns.
  
  \code{\link{Kest}}, \code{\link{Kcross}},
  \code{\link{nndist}}, \code{\link{nncross}},
  \code{\link{applynbd}}, \code{\link{markstat}}
  for functions which use these capabilities.
}
\examples{
   a <- closepaircounts(cells, 0.1)
   sum(a)

   Y <- split(amacrine)
   b <- crosspaircounts(Y$on, Y$off, 0.1)

   d <- closepairs(cells, 0.1)
   e <- crosspairs(Y$on, Y$off, 0.1)
}
\keyword{spatial}
\keyword{math}

\name{distcdf}
\alias{distcdf}
\title{Distribution Function of Interpoint Distance }
\description{
  Computes the cumulative distribution function of the distance
  between two independent random points in a given window
  or windows.
}
\usage{
  distcdf(W, V=W, \dots, dW=1, dV=dW, nr=1024, regularise=TRUE)
}
\arguments{
  \item{W}{
    A window (object of class \code{"owin"}) containing the
    first random point.
  }
  \item{V}{
    Optional. Another window containing the second random point.
    Defaults to \code{W}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine the
    pixel resolution for the calculation.
  }
  \item{dV, dW}{
    Optional. Probability densities (not necessarily normalised)
    for the first and second random points respectively.
    Data in any format acceptable
    to \code{\link{as.im}}, for example, a \code{function(x,y)}
    or a pixel image or a numeric value. The default
    corresponds to a uniform distribution over the window.
  }
  \item{nr}{
    Integer. The number of values of interpoint distance \eqn{r}
    for which the CDF will be computed.
    Should be a large value!
  }
  \item{regularise}{
    Logical value indicating whether to smooth the results
    for very small distances, to avoid discretisation artefacts.
  }
}
\value{
  An object of class \code{"fv"}, see \code{\link{fv.object}},
  which can be plotted directly using \code{\link{plot.fv}}.
}
\details{
  This command computes the Cumulative Distribution Function
  \eqn{
    CDF(r) = Prob(T \le r)
  }{
    CDF(r) = Prob(T \le r)
  }
  of the Euclidean distance \eqn{T = \|X_1 - X_2\|}{T = |X1-X2|}
  between two independent random points \eqn{X_1}{X1} and \eqn{X_2}{X2}.

  In the simplest case, the command \code{distcdf(W)}, the random points are 
  assumed to be uniformly distributed in the same
  window \code{W}.

  Alternatively the two random points may be 
  uniformly distributed in two different windows \code{W} and \code{V}.

  In the most general case the first point \eqn{X_1}{X1} is random
  in the window \code{W} with a probability density proportional to
  \code{dW}, and the second point \eqn{X_2}{X2} is random in
  a different window \code{V} with probability density proportional
  to \code{dV}. The values of \code{dW} and \code{dV} must be
  finite and nonnegative.

  The calculation is performed by numerical integration of the set covariance
  function \code{\link{setcov}} for uniformly distributed points, and
  by computing the covariance function \code{\link{imcov}} in the
  general case. The accuracy of the result depends on
  the pixel resolution used to represent the windows: this is controlled
  by the arguments \code{\dots} which are passed to \code{\link{as.mask}}.
  For example use \code{eps=0.1} to specify pixels of size 0.1 units.

  The arguments \code{W} or \code{V} may also be point patterns
  (objects of class \code{"ppp"}).
  The result is the cumulative distribution function
  of the distance from a randomly selected point in the point pattern,
  to a randomly selected point in the other point pattern or window.

  If \code{regularise=TRUE} (the default), values of the cumulative
  distribution function for very short distances are smoothed to avoid
  discretisation artefacts. Smoothing is applied to all distances
  shorter than the width of 7 pixels. 
}
\seealso{
  \code{\link{setcov}},
  \code{\link{as.mask}}.
}
\examples{
 # The unit disc
 B <- disc()
 plot(distcdf(B))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

 
 

\name{print.psp}
\alias{print.psp}
\title{Print Brief Details of a Line Segment Pattern Dataset}
\description{
  Prints a very brief description of a line segment pattern dataset.
}
\usage{
  \method{print}{psp}(x, \dots)
}
\arguments{
  \item{x}{Line segment pattern (object of class \code{"psp"}).}
  \item{\dots}{Ignored.}
}
\details{
  A very brief description of the line segment pattern \code{x} is printed.

  This is a method for the generic function \code{\link{print}}.
}
\seealso{
  \code{\link{print}},
  \code{\link{print.owin}},
  \code{\link{summary.psp}}
}
\examples{
  a <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  a
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{print}

\name{rotate.infline}
\alias{rotate.infline}
\alias{shift.infline}
\alias{reflect.infline}
\alias{flipxy.infline}
\title{
  Rotate or Shift Infinite Lines
}
\description{
  Given the coordinates of one or more infinite straight lines in the
  plane, apply a rotation or shift.
}
\usage{
\method{rotate}{infline}(X, angle = pi/2, \dots)

\method{shift}{infline}(X, vec = c(0,0), \dots)

\method{reflect}{infline}(X)

\method{flipxy}{infline}(X)
}
\arguments{
  \item{X}{
    Object of class \code{"infline"} representing one or more
    infinite straight lines in the plane.
}
  \item{angle}{
    Angle of rotation, in radians.
  }
  \item{vec}{
    Translation (shift) vector: a numeric vector of length 2,
    or a \code{list(x,y)}, or a point pattern containing one point.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  These functions are methods for the generic
  \code{\link{shift}}, \code{\link{rotate}},
  \code{\link{reflect}} and \code{\link{flipxy}}
  for the class \code{"infline"}.

  An object of class \code{"infline"} represents one or more
  infinite lines in the plane.
}
\value{
  Another \code{"infline"} object representing the
  result of the transformation.
}
\author{
  \adrian.
}
\seealso{
  \code{\link{infline}}
}
\examples{
  L <- infline(v=0.5)

  plot(square(c(-1,1)), main="rotate lines", type="n")
  points(0, 0, pch=3)
  plot(L, col="green")
  plot(rotate(L, pi/12), col="red")
  plot(rotate(L, pi/6), col="red")
  plot(rotate(L, pi/4), col="red")

  L <- infline(p=c(0.4, 0.9), theta=pi* c(0.2, 0.6))

  plot(square(c(-1,1)), main="shift lines", type="n")
  L <- infline(p=c(0.7, 0.8), theta=pi* c(0.2, 0.6))
  plot(L, col="green")
  plot(shift(L, c(-0.5, -0.4)), col="red")

  plot(square(c(-1,1)), main="reflect lines", type="n")
  points(0, 0, pch=3)
  L <- infline(p=c(0.7, 0.8), theta=pi* c(0.2, 0.6))
  plot(L, col="green")
  plot(reflect(L), col="red")

}
\keyword{spatial}
\keyword{manip}

\name{ppp}
\alias{ppp}
\title{Create a Point Pattern}
\description{
  Creates an object of class \code{"ppp"} representing 
  a point pattern dataset in the two-dimensional plane.
}
\usage{
  ppp(x,y, \dots, window, marks,
      check=TRUE, checkdup=check, drop=TRUE)
}
\arguments{
  \item{x}{Vector of \eqn{x} coordinates of data points}
  \item{y}{Vector of \eqn{y} coordinates of data points}
  \item{window}{window of observation,
    an object of class \code{"owin"}}
  \item{\dots}{arguments passed to \code{\link{owin}} to create the
    window, if \code{window} is missing}
  \item{marks}{(optional) mark values for the points.
    A vector or data frame.}
  \item{check}{
    Logical value indicating whether to check
    that all the \eqn{(x,y)} points lie inside the specified window.
    Do not set this to \code{FALSE} unless you are absolutely sure that this
    check is unnecessary. See Warnings below.
  }
  \item{checkdup}{
    Logical value indicating whether to check for duplicated
    coordinates. See Warnings below.
  }
  \item{drop}{
    Logical flag indicating whether to simplify data frames of marks.
    See Details.
  }
}
\value{
  An object of class \code{"ppp"} 
  describing a point pattern in the two-dimensional plane
  (see \code{\link{ppp.object}}).
}
\details{
  In the \pkg{spatstat} library, a point pattern dataset is
  described by an object of class \code{"ppp"}. This function
  creates such objects.

  The vectors \code{x} and \code{y} must be numeric vectors of
  equal length. They are interpreted as the cartesian coordinates
  of the points in the pattern. Note that \code{x} and \code{y} are
  permitted to have length zero, corresponding to an empty point
  pattern; this is the default if these arguments are missing.

  A point pattern dataset is assumed to have been observed within a specific
  region of the plane called the observation window.
  An object of class \code{"ppp"} representing a point pattern
  contains information specifying the observation window.
  This window must always be specified when creating a point pattern dataset;
  there is intentionally no default action of ``guessing'' the window
  dimensions from the data points alone. 

  You can specify the observation window in several
  (mutually exclusive) ways:
  \itemize{
    \item
    \code{xrange, yrange} specify a rectangle
    with these dimensions;
    \item
    \code{poly} specifies a polygonal boundary.
    If the boundary is a single polygon then \code{poly}
    must be a list with components \code{x,y}
    giving the coordinates of the vertices.
    If the boundary consists of several disjoint polygons
    then \code{poly} must be a list of such lists
    so that \code{poly[[i]]$x} gives the \eqn{x} coordinates
    of the vertices of the \eqn{i}th boundary polygon.
    \item
    \code{mask} specifies a binary pixel image with entries
    that are \code{TRUE} if the corresponding pixel is inside
    the window.
    \item
    \code{window} is an object of class \code{"owin"}
    specifying the window. A window object can be created
    by \code{\link{owin}} from raw coordinate data. Special shapes
    of windows can be created by the functions
    \code{\link{square}}, \code{\link{hexagon}},
    \code{\link{regularpolygon}}, \code{\link{disc}}
    and \code{\link{ellipse}}. See the Examples.
  }
  The arguments \code{xrange, yrange} or \code{poly}
  or \code{mask} are passed to the window creator function
  \code{\link{owin}} for interpretation. See
  \code{\link{owin}} for further details.

  The argument \code{window}, if given, must be an object of class
  \code{"owin"}. It is a full description of the window geometry,
  and could have been obtained from \code{\link{owin}} or
  \code{\link{as.owin}}, or by just extracting the observation window
  of another point pattern, or by manipulating such windows.
  See \code{\link{owin}} or the Examples below.

  The points with coordinates \code{x} and \code{y}
  \bold{must} lie inside the specified window, in order to
  define a valid object of this class. 
  Any points which do not lie inside the window will be
  removed from the point pattern, and a warning will be issued.
  See the section on Rejected Points.

  The name of the unit of length for the \code{x} and \code{y} coordinates
  can be specified in the dataset, using the argument \code{unitname}, which is
  passed to \code{\link{owin}}. See the examples below, or the help file
  for \code{\link{owin}}.
  
  The optional argument \code{marks} is given if the point pattern
  is marked, i.e. if each data point carries additional information.
  For example, points which are classified into two or more different
  types, or colours, may be regarded as having a mark which identifies
  which colour they are. Data recording the locations and heights of
  trees in a forest can be regarded as a marked point pattern where the
  mark is the tree height.
  
  The argument \code{marks} can be either
  \itemize{
    \item a vector, of
    the same length as \code{x} and \code{y}, which is interpreted so
    that \code{marks[i]} is the mark attached to the point
    \code{(x[i],y[i])}. If the mark is a real number then \code{marks}
    should be a numeric vector, while if the mark takes only a finite
    number of possible values (e.g. colours or types) then
    \code{marks} should be a \code{factor}.
    \item
    a data frame, with the number of rows equal to the number of points
    in the point pattern. The \code{i}th row of the data frame is interpreted
    as containing the mark values for the \code{i}th point in the point
    pattern. The columns of the data frame correspond to different
    mark variables (e.g. tree species and tree diameter).
  }
  If \code{drop=TRUE} (the default), then 
  a data frame with only one column will be
  converted to a vector, and a data frame with no columns will be
  converted to \code{NULL}.
  
  See \code{\link{ppp.object}} for a description of the
  class \code{"ppp"}.

  Users would normally invoke \code{ppp} to create a point pattern,
  but the functions \code{\link{as.ppp}} and 
  \code{scanpp} may sometimes be convenient.
}
\section{Invalid coordinate values}{
  The coordinate vectors \code{x} and \code{y} must contain only
  finite numerical values. If the coordinates include
  any of the values \code{NA}, \code{NaN}, \code{Inf} or \code{-Inf},
  these will be removed. 
}
\section{Rejected points}{
  The points with coordinates \code{x} and \code{y}
  \bold{must} lie inside the specified window, in order to
  define a valid object of class \code{"ppp"}.
  Any points which do not lie inside the window will be
  removed from the point pattern, and a warning will be issued.

  The rejected points are still accessible: they are stored
  as an attribute of the point pattern called \code{"rejects"}
  (which is an object of class \code{"ppp"} containing the rejected points
  in a large window). However, rejected points in a point pattern
  will be ignored by all other functions except
  \code{\link{plot.ppp}}.

  To remove the rejected points altogether,
  use \code{\link{as.ppp}}. To include the rejected points,
  you will need to find a larger window that contains them,
  and use this larger window in a call to \code{ppp}.
}
\section{Warnings}{
  The code will check for problems with the data,
  and issue a warning if any problems are found.
  The checks and warnings can be switched off, for efficiency's sake,
  but this should only be done if you are confident that the data
  do not have these problems. 

  Setting \code{check=FALSE} will disable all the checking procedures:
  the check for points outside the window, and the check for
  duplicated points. This is extremely dangerous, because points lying
  outside the window will break many of the procedures in
  \pkg{spatstat}, causing crashes and strange errors.
  Set \code{check=FALSE} only if you are absolutely
  sure that there are no points outside the window.

  If duplicated points are found, a warning is issued, but no action is
  taken. Duplicated points are not illegal, but may cause unexpected problems
  later. Setting \code{checkdup=FALSE} will disable the check for duplicated
  points. Do this only if you already know the answer.

  Methodology and software for spatial point patterns often assume
  that all points are distinct so that there are no duplicated points.
  If duplicated points are present, the consequence could be
  an incorrect result or a software crash. To the best of our knowledge,
  all \pkg{spatstat} code handles duplicated points correctly.
  However, if duplicated points are present, we advise using
  \code{\link{unique.ppp}} or \code{\link{multiplicity.ppp}}
  to eliminate duplicated points and re-analyse the data.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{as.ppp}},
  \code{\link{owin.object}},
  \code{\link{owin}},
  \code{\link{as.owin}}
}
\examples{
  # some arbitrary coordinates in [0,1]
  x <- runif(20)
  y <- runif(20)

  # the following are equivalent
  X <- ppp(x, y, c(0,1), c(0,1))
  X <- ppp(x, y)
  X <- ppp(x, y, window=owin(c(0,1),c(0,1)))

  # specify that the coordinates are given in metres
  X <- ppp(x, y, c(0,1), c(0,1), unitname=c("metre","metres"))

  \dontrun{plot(X)}

  # marks
  m <- sample(1:2, 20, replace=TRUE)
  m <- factor(m, levels=1:2)
  X <- ppp(x, y, c(0,1), c(0,1), marks=m)
  \dontrun{plot(X)}

  # polygonal window
  X <- ppp(x, y, poly=list(x=c(0,10,0), y=c(0,0,10)))
  \dontrun{plot(X)}

  # circular window of radius 2
  X <- ppp(x, y, window=disc(2))

  # copy the window from another pattern
  data(cells)
  X <- ppp(x, y, window=Window(cells))
}
\author{\adrian
  and \rolf
}
\keyword{spatial}
\keyword{datagen}

\name{nobjects}
\alias{nobjects}
\alias{nobjects.ppp}
\alias{nobjects.ppx}
\alias{nobjects.psp}
\alias{nobjects.tess}
\alias{nobjects.lintess}
\title{
  Count Number of Geometrical Objects in a Spatial Dataset
}
\description{
  A generic function to count the number of geometrical objects in a
  spatial dataset.
}
\usage{
  nobjects(x)

  \method{nobjects}{ppp}(x)

  \method{nobjects}{ppx}(x)

  \method{nobjects}{psp}(x)

  \method{nobjects}{tess}(x)

  \method{nobjects}{lintess}(x)
}
\arguments{
  \item{x}{A dataset.}
}
\details{
  The generic function \code{nobjects} counts the number of
  geometrical objects in the spatial dataset \code{x}.

  The methods for point patterns (classes \code{"ppp"} and \code{"ppx"},
  embracing \code{"pp3"} and \code{"lpp"})
  count the number of points in the pattern.

  The method for line segment patterns (class \code{"psp"})
  counts the number of line segments in the pattern.

  The method for tessellations (class \code{"tess"} or \code{"lintess"})
  counts the number of tiles of the tessellation.
}
\value{
  A single integer.
}
\author{
  \spatstatAuthors
}
\seealso{
  \code{\link{npoints}}
}
\examples{
   nobjects(redwood)
   nobjects(edges(letterR))
   nobjects(dirichlet(cells))
   nobjects(lineardirichlet(runiflpp(5, simplenet)))
}
\keyword{spatial}
\keyword{manip}

\name{print.ppp}
\alias{print.ppp}
\title{Print Brief Details of a Point Pattern Dataset}
\description{
  Prints a very brief description of a point pattern dataset.
}
\usage{
  \method{print}{ppp}(x, \dots)
}
\arguments{
  \item{x}{Point pattern (object of class \code{"ppp"}).}
  \item{\dots}{Ignored.}
}
\details{
  A very brief description of the point pattern \code{x} is printed.

  This is a method for the generic function \code{\link{print}}.
}
\seealso{
  \code{\link{print}},
  \code{\link{print.owin}},
  \code{\link{summary.ppp}}
}
\examples{
  data(cells)      # plain vanilla point pattern
  cells

  data(lansing)    # multitype point pattern
  lansing          
  
  data(longleaf)    # numeric marks
  longleaf          

  data(demopat)     # weird polygonal window
  demopat
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{print}

\name{psp.object}
\alias{psp.object} %DoNotExport
\title{Class of Line Segment Patterns}
\description{
  A class \code{"psp"} to represent a spatial pattern of
  line segments in the plane.
  Includes information about the window in which the
  pattern was observed. Optionally includes marks.
}
\details{
  An object of this class represents 
  a two-dimensional pattern of line segments. It specifies
  \itemize{
    \item the locations of the line segments (both endpoints)
    \item the window in which the pattern was observed
    \item optionally, a ``mark'' attached to each line segment
      (extra information such as a type label).
    }
  If \code{X} is an object of type \code{psp},
  it contains the following elements:
  \tabular{ll}{
    \code{ends} \tab data frame with entries \code{x0, y0, x1, y1} \cr
                \tab giving coordinates of segment endpoints \cr 
    \code{window} \tab window of observation \cr
                  \tab (an object of class \code{\link{owin}}) \cr
    \code{n} \tab number of line segments \cr
    \code{marks} \tab optional vector or data frame of marks \cr
    \code{markformat} \tab character string specifying the format of the \cr
                      \tab marks; \dQuote{none}, \dQuote{vector}, or
                      \dQuote{dataframe}
  }
  Users are strongly advised not to manipulate these entries
  directly.
  
  Objects of class \code{"psp"}
  may be created by the function
  \code{\link{psp}}
  and converted from other types of data by the function
  \code{\link{as.psp}}.
  Note that you must always specify the window of observation;
  there is intentionally no default action of ``guessing'' the window
  dimensions from the line segments alone.

  Subsets of a line segment pattern may be obtained by the functions
  \code{\link{[.psp}} and \code{\link{clip.psp}}.

  Line segment pattern objects can be plotted just by typing \code{plot(X)}
  which invokes the \code{plot} method for line segment pattern objects,
  \code{\link{plot.psp}}. See \code{\link{plot.psp}} for further information.

  There are also methods for \code{summary} and \code{print}
  for line segment patterns. Use \code{summary(X)} to see a useful description
  of the data.

  Utilities for line segment patterns include
  \code{\link{midpoints.psp}} (to compute the midpoints of each segment),
  \code{\link{lengths_psp}}, (to compute the length of each segment),
  \code{\link{angles.psp}}, (to compute the angle of orientation of
  each segment), and 
  \code{\link{distmap.psp}} to compute the distance map of a
  line segment pattern.
}
\seealso{
  \code{\link{psp}},
  \code{\link{as.psp}},
  \code{\link{[.psp}}
}
\examples{
# creating 
    a <- psp(runif(20),runif(20),runif(20),runif(20), window=owin())
# converting from other formats
    a <- as.psp(matrix(runif(80), ncol=4), window=owin())
    a <- as.psp(data.frame(x0=runif(20), y0=runif(20),
                            x1=runif(20), y1=runif(20)), window=owin())
# clipping
    w <- owin(c(0.1,0.7), c(0.2, 0.8))
    b <- clip.psp(a, w)
    b <- a[w]
# the last two lines are equivalent.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{attribute}
 
 

\name{hextess}
\alias{hexgrid}
\alias{hextess}
\title{
  Hexagonal Grid or Tessellation
}
\description{
  Construct a hexagonal grid of points,
  or a hexagonal tessellation.
}
\usage{
hexgrid(W, s, offset = c(0, 0), origin=NULL, trim = TRUE)

hextess(W, s, offset = c(0, 0), origin=NULL, trim = TRUE)
}

\arguments{
  \item{W}{
    Window in which to construct the hexagonal grid or tessellation.
    An object of class \code{"owin"}.
  }
  \item{s}{
    Side length of hexagons. A positive number.
  }
  \item{offset}{
    Numeric vector of length 2 specifying a shift of the
    hexagonal grid. See Details.
  }
  \item{origin}{
    Numeric vector of length 2 specifying the initial origin
    of the hexagonal grid, before the offset is applied.
    See Details.
  }
  \item{trim}{
    Logical value indicating whether to restrict the result to
    the window \code{W}. See Details.
  }
}
\details{
  \code{hexgrid} constructs a hexagonal grid of points
  on the window \code{W}. If \code{trim=TRUE} (the default),
  the grid is intersected with \code{W} so that all points lie
  inside \code{W}. If \code{trim=FALSE}, then we retain all grid points
  which are the centres of hexagons that intersect \code{W}.

  \code{hextess} constructs a tessellation of hexagons
  on the window \code{W}. If \code{trim=TRUE} (the default),
  the tessellation is restricted to the interior of \code{W},
  so that there will be some fragmentary hexagons near the
  boundary of \code{W}. If \code{trim=FALSE}, the tessellation
  consists of all hexagons which intersect \code{W}.

  The points of \code{hexgrid(...)} are the
  centres of the tiles of \code{hextess(...)} 
  in the same order.
  
  In the initial position of the grid or tessellation,
  one of the grid points (tile centres) is placed at the 
  \code{origin}, which defaults to the midpoint of the
  bounding rectangle of \code{W}. The grid can be shifted
  relative to this origin by specifing the \code{offset}.
}
\value{
  The value of \code{hexgrid} is a point pattern (object of class
  \code{"ppp"}).

  The value of \code{hextess} is a tessellation (object of class \code{"tess"}).
}
\seealso{
  \code{\link{tess}}

  \code{\link{hexagon}}
}
\examples{
  if(interactive()) {
    W <- Window(chorley)
    s <- 0.7
  } else {
    W <- letterR
    s <- 0.3
  }
  plot(hextess(W, s))
  plot(hexgrid(W, s), add=TRUE)
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{datagen}

\name{quadscheme}
\alias{quadscheme}
\title{Generate a Quadrature Scheme from a Point Pattern}
\description{
  Generates a quadrature scheme (an object of class \code{"quad"})
  from point patterns of data and dummy points.
}
\usage{
 quadscheme(data, dummy, method="grid", \dots)
}
\arguments{
  \item{data}{
    The observed data point pattern.
    An object of class \code{"ppp"}
    or in a format recognised by \code{\link{as.ppp}()}
  }
  \item{dummy}{
    The pattern of dummy points for the quadrature. 
    An object of class \code{"ppp"}
    or in a format recognised by \code{\link{as.ppp}()}
    Defaults to \code{default.dummy(data, ...)}
  }
  \item{method}{
    The name of the method for calculating quadrature weights: either
    \code{"grid"} or \code{"dirichlet"}.
  }
  \item{\dots}{
    Parameters of the weighting method (see below)
    and parameters for constructing the dummy points if necessary.
  }
} 
\value{
  An object of class \code{"quad"} describing the quadrature scheme
  (data points, dummy points, and quadrature weights)
  suitable as the argument \code{Q} of the function \code{\link{ppm}()} for
  fitting a point process model.

  The quadrature scheme can be inspected using the
  \code{print} and \code{plot} methods for objects
  of class \code{"quad"}. 
}
\details{
  This is the primary method for producing a quadrature schemes
  for use by \code{\link{ppm}}.
  
  The function \code{\link{ppm}} fits a point process model to an
  observed point pattern using 
  the Berman-Turner quadrature approximation (Berman and Turner, 1992;
  Baddeley and Turner, 2000) to the pseudolikelihood of the model. 
  It requires a quadrature scheme consisting of 
  the original data point pattern, an additional pattern of dummy points,
  and a vector of quadrature weights for all these points.
  Such quadrature schemes are represented by objects of class
  \code{"quad"}. See \code{\link{quad.object}} for a description of this class.

  Quadrature schemes are created by the function
  \code{quadscheme}.
  The arguments \code{data} and \code{dummy} specify the data and dummy
  points, respectively. There is a sensible default for the dummy 
  points (provided by \code{\link{default.dummy}}).
  Alternatively the dummy points
  may be specified arbitrarily and given in any format recognised by
  \code{\link{as.ppp}}.
  There are also functions for creating dummy patterns
  including \code{\link{corners}},
  \code{\link{gridcentres}},
  \code{\link{stratrand}} and
  \code{\link{spokes}}.
 
  The quadrature region is the region over which we are
  integrating, and approximating integrals by finite sums.
  If \code{dummy} is a point pattern object (class \code{"ppp"})
  then the quadrature region is taken to be \code{Window(dummy)}.
  If \code{dummy} is just a list of \eqn{x, y} coordinates
  then the quadrature region defaults to the observation window
  of the data pattern, \code{Window(data)}.

  If \code{dummy} is missing, then a pattern of dummy points
  will be generated using \code{\link{default.dummy}}, taking account
  of the optional arguments \code{...}.
  By default, the dummy points are arranged in a
  rectangular grid; recognised arguments
  include \code{nd} (the number of grid points
  in the horizontal and vertical directions)
  and \code{eps} (the spacing between dummy points).
  If \code{random=TRUE}, a systematic random pattern
  of dummy points is generated instead.
  See \code{\link{default.dummy}} for details. 
  
  If \code{method = "grid"} then the optional arguments (for \code{\dots}) are 
  \code{(nd, ntile, eps)}.
  The quadrature region (defined above) is divided into
  an \code{ntile[1]} by \code{ntile[2]} grid of rectangular tiles.
  The weight for each
  quadrature point is the area of a tile divided by the number of
  quadrature points in that tile. 
 
  If \code{method="dirichlet"} then the optional arguments are
  \code{(exact=TRUE, nd, eps)}.
  The quadrature points (both data and dummy) are used to construct the
  Dirichlet tessellation. The quadrature weight of each point is the
  area of its Dirichlet tile inside the quadrature region.
  If \code{exact == TRUE} then this area is computed exactly
  using the package \code{deldir}; otherwise it is computed
  approximately by discretisation.
}
\references{
  Baddeley, A. and Turner, R.
  Practical maximum pseudolikelihood for spatial point patterns.
  \emph{Australian and New Zealand Journal of Statistics}
  \bold{42} (2000) 283--322.
 
  Berman, M. and Turner, T.R. 
  Approximating point process likelihoods with GLIM.
  \emph{Applied Statistics} \bold{41} (1992) 31--38.
}
\seealso{
  \code{\link{ppm}},
  \code{\link{as.ppp}},
  \code{\link{quad.object}},
  \code{\link{gridweights}},
  \code{\link{dirichletWeights}},
  \code{\link{corners}},
  \code{\link{gridcentres}},
  \code{\link{stratrand}},
  \code{\link{spokes}}
}
\examples{
  data(simdat)

  # grid weights
  Q <- quadscheme(simdat)
  Q <- quadscheme(simdat, method="grid")
  Q <- quadscheme(simdat, eps=0.5)         # dummy point spacing 0.5 units

  Q <- quadscheme(simdat, nd=50)           # 1 dummy point per tile
  Q <- quadscheme(simdat, ntile=25, nd=50) # 4 dummy points per tile

  # Dirichlet weights
  Q <- quadscheme(simdat, method="dirichlet", exact=FALSE)

  # random dummy pattern
  \dontrun{
  D <- runifpoint(250, Window(simdat))
  Q <- quadscheme(simdat, D, method="dirichlet", exact=FALSE)
  }

  # polygonal window
  data(demopat)
  X <- unmark(demopat)
  Q <- quadscheme(X)

  # mask window
  Window(X) <- as.mask(Window(X))
  Q <- quadscheme(X)
  
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{kernel.moment}
\alias{kernel.moment}
\title{Moment of Smoothing Kernel}
\description{
  Computes the complete or incomplete \eqn{m}th moment of a
  smoothing kernel.
}
\usage{
  kernel.moment(m, r, kernel = "gaussian")
}
\arguments{
  \item{m}{
    Exponent (order of moment).
    An integer.
  }
  \item{r}{
    Upper limit of integration for the incomplete moment.
    A numeric value or numeric vector.
    Set \code{r=Inf} to obtain the complete moment.
  }
  \item{kernel}{
    String name of the kernel.
    Options are
    \code{"gaussian"}, \code{"rectangular"},
    \code{"triangular"},
    \code{"epanechnikov"},
    \code{"biweight"},
    \code{"cosine"} and \code{"optcosine"}.
    (Partial matching is used).
  }
}
\details{
  Kernel estimation of a probability density in one dimension
  is performed by \code{\link[stats]{density.default}}
  using a kernel function selected from the list above.
  For more information about these kernels,
  see \code{\link[stats]{density.default}}. 

  The function \code{kernel.moment} computes the partial integral 
  \deqn{
    \int_{-\infty}^r t^m k(t) dt
  }{
    integral[-Inf][r] t^m k(t) dt
  }
  where \eqn{k(t)} is the selected kernel, \eqn{r} is the upper limit of
  integration, and \eqn{m} is the exponent or order.
  Here \eqn{k(t)} is the \bold{standard form} of the kernel,
  which has support \eqn{[-1,1]} and
  standard deviation \eqn{sigma = 1/c} where \code{c = kernel.factor(kernel)}.
}
\value{
  A single number, or a numeric vector of the same length as \code{r}.
}
\seealso{
  \code{\link[stats]{density.default}},
  \code{\link{dkernel}},
  \code{\link{kernel.factor}},
}
\examples{
   kernel.moment(1, 0.1, "epa")
   curve(kernel.moment(2, x, "epa"), from=-1, to=1)
}
\author{
  \adrian
  and Martin Hazelton.
}
\keyword{methods}
\keyword{nonparametric}
\keyword{smooth}

\name{gridweights}
\alias{gridweights}
\title{Compute Quadrature Weights Based on Grid Counts}
\description{
  Computes quadrature weights for a given set of points,
  using the ``counting weights'' for a grid of rectangular tiles.
}
\usage{
 gridweights(X, ntile, \dots, window=NULL, verbose=FALSE, npix=NULL, areas=NULL)
}
\arguments{
  \item{X}{Data defining a point pattern.}
  \item{ntile}{Number of tiles
    in each row and column of the rectangular grid.
    An integer vector of length 1 or 2.
  }
  \item{\dots}{Ignored.}
  \item{window}{Default window for the point pattern}
  \item{verbose}{Logical flag. If \code{TRUE}, information will be printed
    about the computation of the grid weights.
  }
  \item{npix}{Dimensions of pixel grid to use when
    computing a digital approximation to the tile areas.
  }
  \item{areas}{Vector of areas of the tiles, if they are already known.}
}
\value{
  Vector of nonnegative weights for each point in \code{X}.
}
\details{
  This function computes a set of quadrature weights
  for a given pattern of points
  (typically comprising both ``data'' and `dummy'' points).
  See \code{\link{quad.object}} for an explanation of quadrature
  weights and quadrature schemes.

  The weights are computed by the ``counting weights'' rule
  based on a regular grid of rectangular tiles.
  First \code{X} and (optionally) \code{window} are converted into a
  point pattern object. Then the bounding rectangle of the window of
  the point pattern is
  divided into a regular \code{ntile[1] * ntile[2]} grid of rectangular tiles.
  The weight attached to a point of \code{X} is the area of the tile
  in which it lies, divided by the number of points of \code{X} lying in
  that tile.

  For non-rectangular windows the tile areas are currently calculated
  by approximating the window as a binary mask. The accuracy of this
  approximation is controlled by \code{npix}, which becomes
  the argument \code{dimyx} of \code{\link{as.mask}}. 
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{dirichletWeights}}
}
\examples{
  Q <- quadscheme(runifpoispp(10))
  X <- as.ppp(Q) # data and dummy points together
  w <- gridweights(X, 10)
  w <- gridweights(X, c(10, 10))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{is.owin}
\alias{is.owin}
\title{Test Whether An Object Is A Window}
\description{
  Checks whether its argument is a window
  (object of class \code{"owin"}).
}
\usage{
is.owin(x)
}
\arguments{
  \item{x}{Any object.}
}
\details{
  This function tests whether the object \code{x}
  is a window object of class
  \code{"owin"}. See \code{\link{owin.object}} for details
  of this class.

  The result is determined to be \code{TRUE} if \code{x}
  inherits from \code{"owin"}, i.e. if \code{x}
  has \code{"owin"} amongst its classes.
}
\value{
  \code{TRUE} if \code{x} is a point pattern,
  otherwise \code{FALSE}.
}

\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{methods.layered} 
\Rdversion{1.1}
\alias{methods.layered} %DoNotExport
\alias{shift.layered}
\alias{reflect.layered}
\alias{flipxy.layered}
\alias{rotate.layered}
\alias{affine.layered}
\alias{rescale.layered}
\alias{scalardilate.layered}
\title{
  Methods for Layered Objects
}
\description{
  Methods for geometrical transformations of
  layered objects (class \code{"layered"}).
}
\usage{
  \method{shift}{layered}(X, vec=c(0,0), ...)

  \method{rotate}{layered}(X, ..., centre=NULL)

  \method{affine}{layered}(X, ...)

  \method{reflect}{layered}(X)

  \method{flipxy}{layered}(X)

  \method{rescale}{layered}(X, s, unitname)

  \method{scalardilate}{layered}(X, ...)
}
\arguments{
  \item{X}{
    Object of class \code{"layered"}.
  }
  \item{\dots}{
    Arguments passed to the relevant methods
    when applying the operation to each layer of \code{X}.
  }
  \item{s}{
    Rescaling factor passed to the relevant method for
    \code{\link{rescale}}. May be missing.
  }
  \item{vec}{
    Shift vector (numeric vector of length 2).
  }
  \item{centre}{
    Centre of rotation.
    Either a vector of length 2, or a character string
    (partially matched to \code{"centroid"}, \code{"midpoint"}
    or \code{"bottomleft"}).
    The default is the coordinate origin \code{c(0,0)}.
  }
  \item{unitname}{
    Optional. New name for the unit of length.
    A value acceptable to the function \code{\link{unitname<-}}
  }
}
\details{
  These are methods for the generic functions
  \code{\link{shift}},
  \code{\link{rotate}},
  \code{\link{reflect}},
  \code{\link{affine}},
  \code{\link{rescale}},
  \code{\link{scalardilate}} and
  \code{\link{flipxy}}
  for the class of layered objects.

  A layered object represents data that should be plotted in
  successive layers, for example, a background and a foreground.
  See \code{\link{layered}}.
}
\value{
  Another object of class \code{"layered"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{layered}}
}
\examples{
  L <- layered(letterR, runifpoint(20, letterR))
  plot(L)
  plot(rotate(L, pi/4))
}
\keyword{spatial}
\keyword{methods}

\name{split.hyperframe}
\alias{split.hyperframe}
\alias{split<-.hyperframe}
\title{
  Divide Hyperframe Into Subsets and Reassemble
}
\description{
  \code{split} divides the data \code{x} into subsets defined
  by \code{f}. The replacement form replaces values corresponding to
  such a division. 
}
\usage{
  \method{split}{hyperframe}(x, f, drop = FALSE, ...)

  \method{split}{hyperframe}(x, f, drop = FALSE, ...) <- value
}
\arguments{
  \item{x}{
    Hyperframe (object of class \code{"hyperframe"}).
  }
  \item{f}{
    a \code{factor} in the sense that \code{as.factor(f)} defines the
    grouping, or a list of such factors in which case their
    interaction is used for the grouping.
  }
  \item{drop}{
    logical value, indicating whether levels that do not occur should be
    dropped from the result.
  }
  \item{value}{
    a list of hyperframes which arose (or could have arisen)
    from the command \code{split(x,f,drop=drop)}.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  These are methods for the generic functions
  \code{\link{split}} and \code{\link{split<-}}
  for hyperframes (objects of class \code{"hyperframe"}).

  A hyperframe is like a data frame, except that its entries
  can be objects of any kind. The behaviour of these methods
  is analogous to the corresponding methods for data frames.
}
\value{
  The value returned from \code{split.hyperframe} is a list of
  hyperframe containing
  the values for the groups.  The components of the list are named
  by the levels of \code{f} (after converting to a factor, or if already
  a factor and \code{drop = TRUE}, dropping unused levels).

  The replacement method \code{split<-.hyperframe} returns
  a new hyperframe \code{x} for which \code{split(x,f)} equals \code{value}.
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege
}
\seealso{
  \code{\link{hyperframe}}, \code{\link{[.hyperframe}}
}
\examples{
   split(pyramidal, pyramidal$group)
}
\keyword{spatial}
\keyword{manip}

\name{by.ppp}
\alias{by.ppp}
\title{Apply a Function to a Point Pattern Broken Down by Factor}
\description{
  Splits a point pattern into sub-patterns, and applies the function to each
  sub-pattern.
}
\usage{
 \method{by}{ppp}(data, INDICES=marks(data), FUN, ...)
}
\arguments{
  \item{data}{Point pattern (object of class \code{"ppp"}).}
  \item{INDICES}{Grouping variable. Either a factor, a pixel image with
    factor values, or a tessellation.}
  \item{FUN}{Function to be applied to subsets of \code{data}.}
  \item{\dots}{Additional arguments to \code{FUN}.}
}
\details{
  This is a method for the generic function \code{\link{by}}
  for point patterns (class \code{"ppp"}).

  The point pattern \code{data} is first divided into subsets
  according to \code{INDICES}. Then the function \code{FUN}
  is applied to each subset. The results of each computation are
  returned in a list.

  The argument \code{INDICES} may be
  \itemize{
    \item
    a factor, of length equal to the number of points in \code{data}.
    The levels of \code{INDICES}
    determine the destination of each point in \code{data}.
    The \code{i}th point of \code{data} will be placed in the sub-pattern
    \code{split.ppp(data)$l} where \code{l = f[i]}.
    \item
    a pixel image (object of class \code{"im"}) with factor values.
    The pixel value of \code{INDICES}
    at each point of \code{data} will be used as the classifying variable.
    \item
    a tessellation (object of class \code{"tess"}).
    Each point of \code{data} will be classified according to
    the tile of the tessellation into which it falls.
  }
  If \code{INDICES} is missing, then \code{data} must be a multitype point pattern
  (a marked point pattern whose marks vector is a factor).
  Then the effect is that the points of each type
  are separated into different point patterns.
}
\value{
  A list (also of class \code{"anylist"} or \code{"solist"} as
  appropriate) containing the results returned
  from \code{FUN} for each of the subpatterns.
}
\seealso{
  \code{\link{ppp}},
  \code{\link{split.ppp}},
  \code{\link{cut.ppp}},
  \code{\link{tess}},
  \code{\link{im}}.
}
\examples{
  # multitype point pattern, broken down by type
  data(amacrine)
  by(amacrine, FUN=density)
  by(amacrine, FUN=function(x) { min(nndist(x)) } )

  # how to pass additional arguments to FUN
  by(amacrine, FUN=clarkevans, correction=c("Donnelly","cdf"))

  # point pattern broken down by tessellation
  data(swedishpines)
  tes <- quadrats(swedishpines, 5, 5)
  B <- by(swedishpines, tes, clarkevans, correction="Donnelly")
  unlist(lapply(B, as.numeric))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}
\keyword{manip}

\name{rMaternII}
\alias{rMaternII}
\title{Simulate Matern Model II}
\description{
  Generate a random point pattern, a simulated realisation of the
  \Matern Model II inhibition process.
}
\usage{
 rMaternII(kappa, r, win = owin(c(0,1),c(0,1)), stationary=TRUE, ...,
           nsim=1, drop=TRUE)
}
\arguments{
  \item{kappa}{
    Intensity of the Poisson process of proposal points.
    A single positive number.
  }
  \item{r}{
    Inhibition distance.
  }
  \item{win}{
    Window in which to simulate the pattern.
    An object of class \code{"owin"}
    or something acceptable to \code{\link{as.owin}}.
    Alternatively a higher-dimensional box of class
    \code{"box3"} or \code{"boxx"}.
  }
  \item{stationary}{
    Logical. Whether to start with a stationary process of proposal points
    (\code{stationary=TRUE}) or to generate the
    proposal points only inside the window (\code{stationary=FALSE}).
  }
  \item{\dots}{Ignored.}
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  A point pattern
  if \code{nsim=1}, or a list of point patterns if \code{nsim > 1}.
  Each point pattern is normally an object of class \code{"ppp"},
  but may be of class \code{"pp3"} or \code{"ppx"} depending on the window.
}
\details{
  This algorithm generates one or more realisations
  of \Matern's Model II
  inhibition process inside the window \code{win}.

  The process is constructed by first
  generating a uniform Poisson point process of ``proposal'' points 
  with intensity \code{kappa}. If \code{stationary = TRUE} (the
  default), the proposal points are generated in a window larger than
  \code{win} that effectively means the proposals are stationary.
  If \code{stationary=FALSE} then the proposal points are
  only generated inside the window \code{win}.
  
  Then each proposal point is marked by an ``arrival time'', a number
  uniformly distributed in \eqn{[0,1]} independently of other variables.
  
  A proposal point is deleted if it lies within \code{r} units' distance
  of another proposal point \emph{that has an earlier arrival time}.
  Otherwise it is retained.
  The retained points constitute \Matern's Model II.

  The difference between \Matern's Model I and II is the italicised
  statement above. Model II has a higher intensity
  for the same parameter values.
}
\seealso{
\code{\link{rpoispp}},
\code{\link{rMatClust}},
\code{\link{rMaternI}}
}
\examples{
 X <- rMaternII(20, 0.05)
 Y <- rMaternII(20, 0.05, stationary=FALSE)
}
\author{
  \adrian
  
  ,
  Ute Hahn,
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{datagen}

\name{distmap.psp}  
\alias{distmap.psp}
\title{
  Distance Map of Line Segment Pattern
}
\description{
  Computes the distance from each pixel to the nearest
  line segment in the given line segment pattern.
}
\usage{
  \method{distmap}{psp}(X, \dots)
}
\arguments{
  \item{X}{A line segment pattern (object of class \code{"psp"}).
  }
  \item{\dots}{Arguments passed to \code{\link{as.mask}}
    to control pixel resolution.
  }
}
\value{
  A pixel image (object of class \code{"im"}) whose greyscale values
  are the values of the distance map.
  The return value has attributes \code{"index"} and \code{"bdry"}
  which are also pixel images. 
}
\details{
  The ``distance map'' of a line segment pattern \eqn{X} is the function
  \eqn{f} whose value \code{f(u)} is defined for any two-dimensional
  location \eqn{u} as the shortest distance from \eqn{u} to \eqn{X}.

  This function computes the distance map of the line segment pattern \code{X}
  and returns the distance map as a pixel image. The greyscale value
  at a pixel \eqn{u} equals the distance from \eqn{u}
  to the nearest line segment of the pattern \code{X}.
  Distances are computed using analytic geometry.

  Additionally, the return value 
  has two attributes, \code{"index"} and \code{"bdry"}, which are
  also pixel images. The grey values in \code{"bdry"} give the
  distance from each pixel to the bounding rectangle of the image.
  The grey values in \code{"index"} are integers identifying which
  line segment of \code{X} is closest. 
  
  This is a method for the generic function \code{\link{distmap}}.

  Note that this function gives the exact distance from the
  centre of each pixel to the nearest line segment.
  To compute the exact distance from the points in a point pattern
  to the nearest line segment, use \code{\link{distfun}} or one of the
  low-level functions \code{\link{nncross}}
  or \code{\link{project2segment}}.
}
\seealso{
  \code{\link{distmap}},
  \code{\link{distmap.owin}},
  \code{\link{distmap.ppp}},
  \code{\link{distfun}},
  \code{\link{nncross}},
  \code{\link{nearestsegment}},
  \code{\link{project2segment}}.
}
\examples{
    a <- psp(runif(20),runif(20),runif(20),runif(20), window=owin())
    Z <- distmap(a)
    plot(Z)
    plot(a, add=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{Replace.im}
\alias{[<-.im}
\title{Reset Values in Subset of Image}
\description{
  Reset the values in a subset of a pixel image.
}
\usage{
  \method{[}{im}(x, i, j, \dots, drop=TRUE) <- value
}
\arguments{
  \item{x}{
    A two-dimensional pixel image.
    An object of class \code{"im"}.
  }
  \item{i}{
    Object defining the subregion or subset to be replaced.
    Either a spatial window (an object of class \code{"owin"}), or a
    pixel image with logical values, or a point pattern (an object
    of class \code{"ppp"}), or any type of index that applies to a
    matrix, or something that can be converted to a point pattern
    by \code{\link{as.ppp}} (using the window of \code{x}).
  }
  \item{j}{
    An integer or logical vector serving as the column index
    if matrix indexing is being used.  Ignored if \code{i} is
    appropriate to some sort of replacement \emph{other than}
    matrix indexing.
  }
  \item{\dots}{Ignored.}
  \item{drop}{
    Logical value specifying what happens when \code{i} and \code{j}
    are both missing. See Details.
  }
  \item{value}{
     Vector, matrix, factor or pixel image
     containing the replacement values.
     Short vectors will be recycled.
  }
} 
\value{
  The image \code{x} with the values replaced.
}
\details{
  This function changes some of the pixel values in a
  pixel image. The image \code{x} must be an object of class
  \code{"im"} representing a pixel image defined inside a
  rectangle in two-dimensional space (see \code{\link{im.object}}).

  The subset to be changed is determined by the arguments \code{i,j}
  according to the following rules (which are checked in this order):
  \enumerate{
    \item
    \code{i} is a spatial object such as a window,
    a pixel image with logical values, or a point pattern;  or
    \item
    \code{i,j} are indices for the matrix \code{as.matrix(x)}; or
    \item
    \code{i} can be converted to a point pattern
    by \code{\link{as.ppp}(i, W=Window(x))},
    and \code{i} is not a matrix.
  }

  If \code{i} is a spatial window (an object of class \code{"owin"}),
  the values of the image inside this window are changed.

  If \code{i} is a point pattern (an object of class
  \code{"ppp"}), then the values of the pixel image at the points of
  this pattern are changed.

  If \code{i} does not satisfy any of the conditions above, then
  the algorithm tries to interpret \code{i,j} as indices for the matrix
  \code{as.matrix(x)}. Either \code{i} or \code{j} may be missing or blank.

  If none of the conditions above are met, and if \code{i} is not
  a matrix, then \code{i} is converted into a point pattern
  by \code{\link{as.ppp}(i, W=Window(x))}.
  Again the values of the pixel image at the points of
  this pattern are changed.

  If \code{i} and \code{j} are both missing, as in the call
  \code{x[] <- value}, then all pixel values in \code{x}
  are replaced by \code{value}:
  \itemize{
    \item
    If \code{drop=TRUE} (the default), then
    this replacement applies only to pixels whose values are currently defined
    (i.e. where the current pixel value is not \code{NA}).
    If \code{value} is a vector, then its length must equal
    the number of pixels whose values are currently defined.
    \item
    If \code{drop=FALSE} then the replacement applies to all
    pixels inside the rectangle \code{Frame(x)}.
    If \code{value} is a vector, then its length 
    must equal the number of pixels in the entire rectangle.
  }
}
\section{Warning}{
  If you have a 2-column matrix containing the \eqn{x,y} coordinates
  of point locations, then to prevent this being interpreted as an
  array index, you should convert it to a \code{data.frame}
  or to a point pattern.
}
\seealso{
  \code{\link{im.object}},
  \code{\link{[.im}},
  \code{\link{[}},
  \code{\link{ppp.object}},
  \code{\link{as.ppp}},
  \code{\link{owin.object}}
}
\examples{
 # make up an image
 X <- setcov(unit.square())
 plot(X)

 # a rectangular subset
 W <- owin(c(0,0.5),c(0.2,0.8))
 X[W] <- 2
 plot(X)

 # a polygonal subset
 data(letterR)
 R <- affine(letterR, diag(c(1,1)/2), c(-2,-0.7))
 X[R] <- 3
 plot(X)

 # a point pattern
 P <- rpoispp(20)
 X[P] <- 10
 plot(X)

 # change pixel value at a specific location
 X[list(x=0.1,y=0.2)] <- 7

 # matrix indexing --- single vector index
 X[1:2570] <- 10
 plot(X)

 # matrix indexing using double indices
 X[1:257,1:10] <- 5
 plot(X)

 # matrix indexing using a matrix of indices
 X[cbind(1:257,1:257)] <- 10
 X[cbind(257:1,1:257)] <- 10
 plot(X)

 # Blank indices
 Y <- as.im(letterR)
 plot(Y)
 Y[] <- 42  # replace values only inside the window 'R'
 plot(Y)
 Y[drop=FALSE] <- 7 # replace all values in the rectangle
 plot(Y)

 Z <- as.im(letterR)
 Z[] <- raster.x(Z, drop=TRUE) # excludes NA
 plot(Z)
 Z[drop=FALSE] <- raster.y(Z, drop=FALSE) # includes NA
 plot(Z)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{rounding} 
\alias{rounding}
\alias{rounding.default}
\alias{rounding.ppp}
\alias{rounding.pp3}
\alias{rounding.ppx}
\title{
  Detect Numerical Rounding 
}
\description{
  Given a numeric vector, or an object containing numeric spatial
  coordinates, determine whether the values have been rounded
  to a certain number of decimal places.
}
\usage{
rounding(x)

\method{rounding}{default}(x)

\method{rounding}{ppp}(x)

\method{rounding}{pp3}(x)

\method{rounding}{ppx}(x)
}
\arguments{
  \item{x}{
    A numeric vector, or an object containing numeric spatial coordinates.
  }
}
\details{
  For a numeric vector \code{x},
  this function determines whether the values have been rounded
  to a certain number of decimal places.
  \itemize{
    \item 
    If the entries of \code{x} are not all integers, then
    \code{rounding(x)} returns the smallest number of digits \code{d}
    after the decimal point
    such that \code{\link[base]{round}(x, digits=d)} is identical to
    \code{x}.
    For example if \code{rounding(x) = 2} then the entries of
    \code{x} are rounded to 2 decimal places, and are multiples of 0.01.
    \item 
    If all the entries of \code{x} are integers, then
    \code{rounding(x)} returns \code{-d}, where
    \code{d} is the smallest number of digits \emph{before} the decimal point
    such that \code{\link[base]{round}(x, digits=-d)} is identical to
    \code{x}.
    For example if \code{rounding(x) = -3} then the entries of
    \code{x} are multiples of 1000.
    If \code{rounding(x) = 0} then the entries of \code{x} are integers
    but not multiples of 10.
    \item
    If all entries of \code{x} are equal to 0, the rounding is
    not determined, and a value of \code{NULL} is returned.
  }
  
  For a point pattern (object of class \code{"ppp"})
  or similar object \code{x} containing numeric spatial
  coordinates, this procedure is applied to the spatial coordinates.
}
\value{
  An integer.
}
\author{
  \adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{round.ppp}}
}
\examples{
   rounding(c(0.1, 0.3, 1.2))
   rounding(c(1940, 1880, 2010))
   rounding(0)
   rounding(cells)
}
\keyword{spatial}
\keyword{math}

\name{affine.ppp} 
\alias{affine.ppp} 
\title{Apply Affine Transformation To Point Pattern}
\description{
  Applies any affine transformation of the plane (linear transformation
  plus vector shift) to a point pattern. 
}
\usage{
 \method{affine}{ppp}(X, mat=diag(c(1,1)), vec=c(0,0), \dots)
}
\arguments{
  \item{X}{Point pattern (object of class \code{"ppp"}).}
  \item{mat}{Matrix representing a linear transformation.}
  \item{vec}{Vector of length 2 representing a translation.}
  \item{\dots}{Arguments passed to \code{\link{affine.owin}} affecting
    the handling of the observation window, if it is a binary pixel
    mask.
  }
}
\value{
  Another point pattern (of class \code{"ppp"}) representing the
  result of applying the affine transformation.
}
\details{
  The point pattern, and its window, are subjected first to the
  linear transformation represented by
  \code{mat} (multiplying on the left by \code{mat}),
  and are then translated by the vector \code{vec}.
  
  The argument \code{mat} must be a nonsingular \eqn{2 \times 2}{2 * 2}
  matrix.

  This is a method for the generic function \code{\link{affine}}.
}
\seealso{
  \code{\link{affine}},
  \code{\link{affine.owin}},
  \code{\link{affine.psp}},
  \code{\link{affine.im}},
  \code{\link{flipxy}},
  \code{\link{rotate}},
  \code{\link{shift}}
}
\examples{
  data(cells)
  # shear transformation
  X <- affine(cells, matrix(c(1,0,0.6,1),ncol=2))
  \dontrun{
  plot(X)
  # rescale y coordinates by factor 1.3
  plot(affine(cells, diag(c(1,1.3))))
  }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{dilated.areas}
\Rdversion{1.1}
\alias{dilated.areas}
\title{
  Areas of Morphological Dilations
}
\description{
  Computes the areas of successive morphological dilations.
}
\usage{
  dilated.areas(X, r, W=as.owin(X), ..., constrained=TRUE, exact = FALSE)
}
\arguments{
  \item{X}{
    Object to be dilated.
    A point pattern (object of class \code{"ppp"}),
    a line segment pattern (object of class \code{"psp"}),
    or a window (object of class \code{"owin"}).
  }
  \item{r}{
    Numeric vector of radii for the dilations. 
  }
  \item{W}{
    Window (object of class \code{"owin"}) inside which the areas
    will be computed, if \code{constrained=TRUE}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{distmap}} to control the
    pixel resolution, if \code{exact=FALSE}.
  }
  \item{constrained}{
    Logical flag indicating whether areas should be restricted
    to the window \code{W}.
  }
  \item{exact}{
    Logical flag indicating whether areas should be computed
    using analytic geometry (which is slower but more accurate).
    Currently available only when \code{X} is a point pattern.
  }
}
\details{
  This function computes the areas of the dilations of \code{X}
  by each of the radii \code{r[i]}. Areas may also be computed
  inside a specified window \code{W}.

  The morphological dilation of a set \eqn{X} by a distance \eqn{r > 0}
  is the subset 
  consisting of all points \eqn{x}{x} such that the
  distance from \eqn{x} to \eqn{X} is less than 
  or equal to \eqn{r}.

  When \code{X} is a point pattern, the dilation by a distance
  \eqn{r} is the union of
  discs of radius \eqn{r} centred at the points of \code{X}.

  The argument \code{r} should be a vector of nonnegative numbers.

  If \code{exact=TRUE} and if \code{X} is a point pattern,
  then the areas are computed using analytic geometry, which is
  slower but much more accurate. Otherwise the computation is performed
  using \code{\link{distmap}}.

  To compute the dilated object itself, use \code{\link{dilation}}.
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{dilation}},
  \code{\link{eroded.areas}}
}
\examples{
  X <- runifpoint(10)
  a <- dilated.areas(X, c(0.1,0.2), W=square(1), exact=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{as.im}  
\alias{as.im}
\alias{as.im.im}
\alias{as.im.leverage.ppm}
\alias{as.im.owin}
\alias{as.im.matrix}
\alias{as.im.tess}
\alias{as.im.function}
\alias{as.im.funxy}
\alias{as.im.expression}
\alias{as.im.distfun}
\alias{as.im.nnfun}
\alias{as.im.densityfun}
\alias{as.im.Smoothfun}
\alias{as.im.data.frame}
\alias{as.im.default}
\title{Convert to Pixel Image}
\description{
  Converts various kinds of data to a pixel image
}
\usage{
  as.im(X, \dots)

  \method{as.im}{im}(X, W=NULL, \dots,
        eps=NULL, dimyx=NULL, xy=NULL,
        na.replace=NULL)

  \method{as.im}{owin}(X, W=NULL, \dots,
        eps=NULL, dimyx=NULL, xy=NULL,
        na.replace=NULL, value=1)

  \method{as.im}{matrix}(X, W=NULL, \dots)

  \method{as.im}{tess}(X, W=NULL, \dots, 
        eps=NULL, dimyx=NULL, xy=NULL,
        na.replace=NULL)

  \method{as.im}{function}(X, W=NULL, \dots,
        eps=NULL, dimyx=NULL, xy=NULL,
        na.replace=NULL,
        stringsAsFactors=default.stringsAsFactors(),
        strict=FALSE, drop=TRUE)

  \method{as.im}{funxy}(X, W=Window(X), \dots)

  \method{as.im}{expression}(X, W=NULL, \dots)

  \method{as.im}{distfun}(X, W=NULL, \dots,
        eps=NULL, dimyx=NULL, xy=NULL,
        na.replace=NULL, approx=TRUE)

  \method{as.im}{nnfun}(X, W=NULL, \dots,
        eps=NULL, dimyx=NULL, xy=NULL,
        na.replace=NULL, approx=TRUE)

  \method{as.im}{densityfun}(X, W=Window(X), \dots, approx=TRUE)

  \method{as.im}{Smoothfun}(X, W=Window(X), \dots, approx=TRUE)

  \method{as.im}{leverage.ppm}(X, \dots, what=c("smooth", "nearest"))

  \method{as.im}{data.frame}(X, \dots, step, fatal=TRUE, drop=TRUE)

  \method{as.im}{default}(X, W=NULL, \dots,
        eps=NULL, dimyx=NULL, xy=NULL,
        na.replace=NULL)
}
\arguments{
  \item{X}{Data to be converted to a pixel image.}
  \item{W}{Window object which determines the spatial domain
    and pixel array geometry.
  }
  \item{\dots}{Additional arguments passed to \code{X} when \code{X}
    is a function.}
  \item{eps,dimyx,xy}{
    Optional parameters passed to \code{\link{as.mask}}
    which determine the pixel array geometry.
    See \code{\link{as.mask}}.
  }
  \item{na.replace}{Optional value to replace \code{NA} entries in the
    output image.
  }
  \item{value}{Optional.
    The value to be assigned to pixels inside the window,
    if \code{X} is a window.
  }
  \item{strict}{
    Logical value indicating whether to match formal arguments
    of \code{X} when \code{X} is a function. If \code{strict=FALSE}
    (the default), all the \code{\dots} arguments are passed to
    \code{X}. If \code{strict=TRUE}, only named arguments
    are passed, and only if they match the names of formal arguments of
    \code{X}. 
  }
  \item{step}{
    Optional. A single number, or numeric vector of length 2,
    giving the grid step lengths
    in the \eqn{x} and \eqn{y} directions.
  }
  \item{fatal}{
    Logical value indicating what to do if the resulting image
    would be too large for available memory. If \code{fatal=TRUE} (the
    default), an error occurs. If \code{fatal=FALSE}, a warning is
    issued and \code{NULL} is returned.
  }
  \item{drop}{
    Logical value indicating what to do if the result would
    normally be a list of pixel images but the list contains
    only one image.
    If \code{drop=TRUE} (the default), the pixel image is extracted
    and the result is a pixel image.
    If \code{drop=FALSE}, this list is returned as the result.
  }
  \item{stringsAsFactors}{
    Logical value (passed to \code{\link[base]{data.frame}})
    specifying how to handle pixel values which
    are character strings. If \code{TRUE}, character values are
    interpreted as factor levels. If \code{FALSE}, they remain
    as character strings. The factory-fresh befault is \code{TRUE},
    but that can be changed by setting
    \code{options(stringsAsFactors=FALSE)}.
  }
  \item{approx}{
   Logical value indicating whether to compute an approximate result
   at faster speed.
 }
 \item{what}{
   Character string (partially matched) specifying which image data
   should be extracted. See \code{\link{plot.leverage.ppm}} for
   explanation.
 }
}
\details{
  This function converts the data \code{X} into a pixel image
  object of class \code{"im"} (see \code{\link{im.object}}).
  The function \code{as.im} is generic, with methods for the classes
  listed above.

  Currently \code{X} may be any of the following:
  \itemize{
    \item
    a pixel image object, of class \code{"im"}.
    \item
    a window object, of class \code{"owin"} (see
    \code{\link{owin.object}}). The result is an image
    with all pixel entries equal to \code{value} inside the window \code{X},
    and \code{NA} outside.
    \item
    a matrix. 
    \item
    a tessellation (object of class \code{"tess"}).
    The result is a factor-valued image, with one factor level
    corresponding to each tile of the tessellation. Pixels are classified
    according to the tile of the tessellation into which they fall.
    \item
    a single number (or a single logical, complex, factor or character
    value). The result is an image
    with all pixel entries equal to this constant value
    inside the window \code{W} (and \code{NA} outside, unless the
    argument \code{na.replace} is given).
    Argument \code{W} is required.
    \item
    a function of the form \code{function(x, y, ...)}
    which is to be evaluated to yield the image pixel values.
    In this case, the additional argument \code{W} must be present.
    This window will be converted to
    a binary image mask. Then the function \code{X} will be evaluated
    in the form \code{X(x, y, ...)} where \code{x} and \code{y} are
    \bold{vectors} containing the \eqn{x} and \eqn{y} coordinates
    of all the pixels in the image mask, and \code{...} are any extra
    arguments given. This function must return a
    vector or factor of the same length as the input vectors,
    giving the pixel values.
    \item
    an object of class \code{"funxy"} representing a
    \code{function(x,y,...)} defined in a spatial region.
    The function will be evaluated as described above.
    The window \code{W} defaults to the domain of definition of the
    function.
    \item
    an object of class \code{"funxy"} which also belongs to one of
    the following special classes. If \code{approx=TRUE} (the default),
    the function will be evaluated approximately using a very fast
    algorithm. If \code{approx=FALSE}, the function will be evaluated
    exactly at each grid location as described above.
    \itemize{
      \item
      an object of class \code{"distfun"} representing a distance function
      (created by the command \code{\link{distfun}}).
      The fast approximation is 
      the distance transform \code{\link{distmap}}.
      \item
      an object of class \code{"nnfun"}
      representing a nearest neighbour function
      (created by the command \code{\link{nnfun}}).
      The fast approximation is \code{\link{nnmap}}.
      \item
      an object of class \code{"densityfun"} representing a kernel
      estimate of intensity (created by the command \code{\link{densityfun}}).
      The fast approximation is the Fast Fourier Transform
      algorithm in \code{\link{density.ppp}}.
      \item
      an object of class \code{"Smoothfun"} 
      representing kernel-smoothed values
      (created by the command \code{\link{Smoothfun}}).
      The fast approximation is the Fast Fourier Transform
      algorithm in \code{\link{Smooth.ppp}}.
    }
    \item
    An \code{expression} involving the variables \code{x} and \code{y}
    representing the spatial coordinates, and possibly also involving other
    variables.
    The additional argument \code{W} must be present; it will be converted to
    a binary image mask. The expression \code{X} will be evaluated
    in an environment where \code{x} and \code{y} are
    \bold{vectors} containing the spatial coordinates
    of all the pixels in the image mask.
    Evaluation of the expression \code{X} must yield a vector or factor,
    of the same length as \code{x} and \code{y}, giving the pixel values.
    \item
    a list with entries \code{x, y, z} in the format expected by
    the standard \code{R} functions
    \code{\link{image.default}} and \code{\link{contour.default}}.
    That is, \code{z} is a matrix of pixel values, \code{x} and \code{y}
    are vectors of \eqn{x} and \eqn{y} coordinates respectively,
    and \code{z[i,j]} is the pixel value for the location
    \code{(x[i],y[j])}.
    \item
    a point pattern (object of class \code{"ppp"}).
    See the separate documentation for \code{\link{as.im.ppp}}.
    \item
    A data frame with at least three columns.
    Columns named \code{x}, \code{y} and \code{z}, if present,
    will be assumed to contain the spatial coordinates and the pixel
    values, respectively. Otherwise the \code{x} and \code{y}
    coordinates will be taken from the first two columns of the
    data frame, and any remaining columns will be interpreted as
    pixel values.
  }

  The spatial domain (enclosing rectangle) of the pixel image
  is determined by the argument \code{W}. If \code{W} is absent, 
  the spatial domain is determined by \code{X}.
  When \code{X} is a function, a matrix, or a single numerical value,
  \code{W} is required.
  
  The pixel array dimensions of the final resulting image are determined
  by (in priority order)
  \itemize{
    \item the argument \code{eps}, \code{dimyx} or \code{xy} if present;
    \item the pixel dimensions of the window \code{W}, if it is
    present and if it is a binary mask;
    \item the pixel dimensions of \code{X} if it is an image,
    a binary mask, or a \code{list(x,y,z)};
    \item the default pixel dimensions, 
    controlled by \code{\link{spatstat.options}}.
  }

  Note that if \code{eps}, \code{dimyx} or \code{xy}
  is given, this will override
  the pixel dimensions of \code{X} if it has them.
  Thus, \code{as.im} can be used to change an image's pixel dimensions.

  If the argument \code{na.replace} is given, then all \code{NA} entries
  in the image will be replaced by this value.  The resulting image is
  then defined everwhere on the full rectangular domain, instead of a
  smaller window. Here \code{na.replace} should be a single value,
  of the same type as the other entries in the image.

  If \code{X} is a pixel image that was created by an older version
  of \pkg{spatstat}, the command \code{X <- as.im(X)} will
  repair the internal format of \code{X} so that it conforms to the
  current version of \pkg{spatstat}.

  If \code{X} is a data frame with \code{m} columns,
  then \code{m-2} columns of data are interpreted as pixel values,
  yielding \code{m-2} pixel images. The result of
  \code{as.im.data.frame} is a list of pixel
  images, belonging to the class \code{"imlist"}.
  If \code{m = 3} and \code{drop=TRUE} (the default), then the
  result is a pixel image rather than a list containing this image.

  If \code{X} is a \code{function(x,y)} which returns a matrix of
  values, then \code{as.im(X, W)} will be a list of pixel images.
}
\section{Character-valued images}{
  By default, if the pixel value data are character strings, they will be
  treated as levels of a factor, and the resulting image will be
  factor-valued. To prevent the conversion of character strings to
  factors, use the argument \code{stringsAsFactors=FALSE},
  which is recognised by most of the
  methods for \code{as.im}, or alternatively set
  \code{options(stringsAsFactors=FALSE)}.
}
\value{
  A pixel image (object of class \code{"im"}),
  or a list of pixel images, 
  or \code{NULL} if the conversion failed.
}
\seealso{
  Separate documentation for \code{\link{as.im.ppp}}
}
\examples{
  data(demopat)
  # window object
  W <- Window(demopat)
  plot(W)
  Z <- as.im(W)
  image(Z)
  # function
  Z <- as.im(function(x,y) {x^2 + y^2}, unit.square())
  image(Z)
  # or as an expression
  Z <- as.im(expression(x^2+y^2), square(1))

  # function with extra arguments
  f <- function(x, y, x0, y0) {
      sqrt((x - x0)^2 + (y-y0)^2)
  }
  Z <- as.im(f, unit.square(), x0=0.5, y0=0.5)
  image(Z)

  # Revisit the Sixties
  Z <- as.im(f, letterR, x0=2.5, y0=2)
  image(Z)
  # usual convention in R
  stuff <- list(x=1:10, y=1:10, z=matrix(1:100, nrow=10))
  Z <- as.im(stuff)
  # convert to finer grid
  Z <- as.im(Z, dimyx=256)

  #' distance functions
  d <- distfun(redwood)
  Zapprox <- as.im(d)
  Zexact <- as.im(d, approx=FALSE)
  plot(solist(approx=Zapprox, exact=Zexact), main="")

  # pixellate the Dirichlet tessellation
  Di <- dirichlet(runifpoint(10))
  plot(as.im(Di))
  plot(Di, add=TRUE)

  # as.im.data.frame is the reverse of as.data.frame.im
  grad <- bei.extra$grad
  slopedata <- as.data.frame(grad)
  slope <- as.im(slopedata)
  unitname(grad) <- unitname(slope) <- unitname(grad) # for compatibility
  all.equal(slope, grad) # TRUE

  ## handling of character values
  as.im("a", W=letterR, na.replace="b")
  as.im("a", W=letterR, na.replace="b", stringsAsFactors=FALSE)
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{as.function.im}
\alias{as.function.im}
\title{
  Convert Pixel Image to Function of Coordinates
}
\description{
  Converts a pixel image to a function of the
  \eqn{x} and \eqn{y} coordinates.
}
\usage{
 \method{as.function}{im}(x, ...)
}
\arguments{
  \item{x}{
    Pixel image (object of class \code{"im"}).
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  This command converts a pixel image
  (object of class \code{"im"}) to a \code{function(x,y)}
  where the arguments \code{x} and \code{y} are (vectors of) spatial
  coordinates. This function returns the pixel values at the
  specified locations.
}
\value{
  A function in the \R language, also belonging to the
  class \code{"funxy"}.
}
\author{
\adrian


\rolf

and \ege

}
\seealso{
  \code{\link{[.im}}
}
\examples{
  d <- density(cells)
  f <- as.function(d)
  f(0.1, 0.3)
}
\keyword{spatial}
\keyword{manip}

\name{as.polygonal}
\Rdversion{1.1}
\alias{as.polygonal}
\title{
Convert a Window to a Polygonal Window
}
\description{
  Given a window \code{W} of any geometric type (rectangular,
  polygonal or binary mask),  this function returns a polygonal window
  that represents the same spatial domain.
}
\usage{
as.polygonal(W, repair=FALSE)
}
\arguments{
  \item{W}{
    A window (object of class \code{"owin"}).
  }
  \item{repair}{
    Logical value indicating whether to check the validity
    of the polygon data and repair it,
    if \code{W} is already a polygonal window.
  }
}
\details{
  Given a window \code{W} of any geometric type (rectangular,
  polygonal or binary mask),  this function returns a polygonal window
  that represents the same spatial domain.

  If \code{W} is a rectangle, it is converted to a polygon with
  4 vertices.

  If \code{W} is already polygonal, it is returned unchanged, by default.  
  However if \code{repair=TRUE} then the validity of the polygonal coordinates
  will be checked (for example to check the boundary is not self-intersecting)
  and repaired if necessary, so that the result could be different
  from \code{W}.

  If \code{W} is a binary mask, then each pixel in the
  mask is replaced by a small square or rectangle, and the union of these
  squares or rectangles is computed. The result is a polygonal window
  that has only horizontal and vertical edges. 
  (Use \code{\link{simplify.owin}} to remove the staircase
  appearance, if desired).
}
\value{
  A polygonal window (object of class \code{"owin"}
  and of type \code{"polygonal"}). 
}
\author{
  \spatstatAuthors
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{as.mask}},
  \code{\link{simplify.owin}}
}
\examples{
   data(letterR)
   m <- as.mask(letterR, dimyx=32)
   p <- as.polygonal(m)
   if(interactive()) {
      plot(m)
      plot(p, add=TRUE, lwd=2)
   }
}
\keyword{spatial}
\keyword{manip}

\name{layout.boxes}
\alias{layout.boxes}
\title{
  Generate a Row or Column Arrangement of Rectangles.
}
\description{
  A simple utility to generate a row or column of boxes (rectangles)
  for use in point-and-click panels.
}
\usage{
layout.boxes(B, n, horizontal = FALSE, aspect = 0.5, usefrac = 0.9)
}
\arguments{
  \item{B}{
    Bounding rectangle for the boxes.
    An object of class \code{"owin"}.
  }
  \item{n}{
    Integer. The number of boxes.
  }
  \item{horizontal}{
    Logical. If \code{TRUE}, arrange the boxes in a horizontal row.
    If \code{FALSE} (the default), arrange them in a vertical column.
  }
  \item{aspect}{
    Aspect ratio (height/width) of each box.
  }
  \item{usefrac}{
    Number between 0 and 1. The 
    fraction of height or width of \code{B} that should be
    occupied by boxes.
  }
}
\details{
  This simple utility generates a list of boxes (rectangles)
  inside the bounding box \code{B} arranged in a regular
  row or column. It is useful for generating the
  positions of the panel buttons in the function \code{\link{simplepanel}}.
}
\value{
  A list of rectangles.
}
\examples{
  B <- owin(c(0,10),c(0,1))
  boxes <- layout.boxes(B, 5, horizontal=TRUE)
  plot(B, main="", col="blue")
  niets <- lapply(boxes, plot, add=TRUE, col="grey")
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{simplepanel}}
}
\keyword{utilities}

\name{diameter.owin}  
\alias{diameter.owin}
\title{Diameter of a Window}
\description{
  Computes the diameter of a window.
}
\usage{
 \method{diameter}{owin}(x)
}
\arguments{
  \item{x}{
    A window whose diameter will be computed.
  }
}
\value{
  The numerical value of the diameter of the window. 
}
\details{
  This function computes the 
  diameter of a window of arbitrary shape,
  i.e. the maximum distance between any two points
  in the window.

  The argument \code{x} should be a window (an object of class
  \code{"owin"}, see \code{\link{owin.object}} for details)
  or can be given in any format acceptable to \code{\link{as.owin}()}.
  
  The function \code{diameter} is generic. This function is the
  method for the class \code{"owin"}.
}
\seealso{
  \code{\link{area.owin}},
  \code{\link{perimeter}},
  \code{\link{edges}},
  \code{\link{owin}},
  \code{\link{as.owin}}
}
\examples{
  w <- owin(c(0,1),c(0,1))
  diameter(w) 
  # returns sqrt(2)
  data(letterR)
  diameter(letterR)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{rsyst}
\alias{rsyst}
\title{Simulate systematic random point pattern}
\description{
  Generates a \dQuote{systematic random} pattern of points in a window,
  consisting of a grid of equally-spaced points with a random common
  displacement.
}
\usage{
 rsyst(win=square(1), nx=NULL, ny=nx, \dots, dx=NULL, dy=dx,
       nsim=1, drop=TRUE)
}
\arguments{
  \item{win}{
    A window. 
    An object of class \code{\link{owin}},
    or data in any format acceptable to \code{\link{as.owin}()}.
  }
  \item{nx}{Number of columns of grid points in the window.
    Incompatible with \code{dx}.
  }
  \item{ny}{Number of rows of grid points in the window.
    Incompatible with \code{dy}.
  }
  \item{\dots}{Ignored.}
  \item{dx}{Spacing of grid points in \eqn{x} direction.
    Incompatible with \code{nx}.
  }
  \item{dy}{Spacing of grid points in \eqn{y} direction.
    Incompatible with \code{ny}.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  A point pattern (an object of class \code{"ppp"})
  if \code{nsim=1}, or a list of point patterns if \code{nsim > 1}.
}
\details{
  This function generates a \dQuote{systematic random} pattern
  of points in the window \code{win}. The pattern consists of a
  rectangular grid of points with a random common displacement.

  The grid spacing in the \eqn{x} direction is determined
  either by the number of columns \code{nx} or by the
  horizontal spacing \code{dx}.
  The grid spacing in the \eqn{y} direction is determined
  either by the number of rows \code{ny} or by the
  vertical spacing \code{dy}. 
  
  The grid is then given a random displacement (the common displacement
  of the grid points is a uniformly distributed random vector in the
  tile of dimensions \code{dx, dy}).

  Some of the resulting grid points may lie outside the window
  \code{win}: if they do, they are deleted.  The result is a point
  pattern inside the window \code{win}.

  This function is useful in creating dummy points for quadrature
  schemes (see \code{\link{quadscheme}}) as well as in simulating
  random point patterns.
}
\seealso{
  \code{\link{rstrat}},
  \code{\link{runifpoint}},
  \code{\link{quadscheme}}
}
\examples{
  X <- rsyst(nx=10)
  plot(X)

  # polygonal boundary
  data(letterR)
  X <- rsyst(letterR, 5, 10)
  plot(X)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{rknn}
\alias{dknn}
\alias{pknn}
\alias{qknn}
\alias{rknn}
\title{
  Theoretical Distribution of Nearest Neighbour Distance
}
\description{
  Density, distribution function, quantile function and random
  generation for the random distance to the \eqn{k}th nearest neighbour
  in a Poisson point process in \eqn{d} dimensions.
}
\usage{
dknn(x, k = 1, d = 2, lambda = 1)
pknn(q, k = 1, d = 2, lambda = 1)
qknn(p, k = 1, d = 2, lambda = 1)
rknn(n, k = 1, d = 2, lambda = 1)
}
\arguments{
  \item{x,q}{vector of quantiles.}
  \item{p}{vector of probabilities.}
  \item{n}{number of observations to be generated.}
  \item{k}{order of neighbour.}
  \item{d}{dimension of space.}
  \item{lambda}{intensity of Poisson point process.}
}
\details{
  In a Poisson point process in \eqn{d}-dimensional space, let
  the random variable \eqn{R} be
  the distance from a fixed point to the \eqn{k}-th nearest random point,
  or the distance from a random point to the
  \eqn{k}-th nearest other random point.

  Then \eqn{R^d} has a Gamma distribution with shape parameter \eqn{k}
  and rate \eqn{\lambda * \alpha}{lambda * alpha} where
  \eqn{\alpha}{alpha} is a constant (equal to the volume of the
  unit ball in \eqn{d}-dimensional space).
  See e.g. Cressie (1991, page 61).

  These functions support calculation and simulation for the
  distribution of \eqn{R}. 
}
\value{
  A numeric vector:
  \code{dknn} returns the probability density,
  \code{pknn} returns cumulative probabilities (distribution function),
  \code{qknn} returns quantiles,
  and \code{rknn} generates random deviates.
}
\references{
  Cressie, N.A.C. (1991)
  \emph{Statistics for spatial data}.
  John Wiley and Sons, 1991.
}
\author{\adrian
  
  
  and \rolf
  
}
\examples{
  x <- seq(0, 5, length=20)
  densities <- dknn(x, k=3, d=2)
  cdfvalues <- pknn(x, k=3, d=2)
  randomvalues <- rknn(100, k=3, d=2)
  deciles <- qknn((1:9)/10, k=3, d=2)
}
\keyword{spatial}
\keyword{distribution}

\name{rmhmodel.list}
\alias{rmhmodel.list}
\title{Define Point Process Model for Metropolis-Hastings Simulation.}
\description{
  Given a list of parameters,
  builds a description of a point process model
  for use in simulating the model by the Metropolis-Hastings
  algorithm. 
}
\usage{
   \method{rmhmodel}{list}(model, ...)
}
\arguments{
  \item{model}{A list of parameters. See Details.}
  \item{\dots}{
    Optional list of additional named parameters.
  }
}
\value{
  An object of class \code{"rmhmodel"}, which is essentially
  a validated list of parameter values for the model.
  
  There is a \code{print} method for this class, which prints
  a sensible description of the model chosen.
}
\details{
  The generic function \code{\link{rmhmodel}} takes a
  description of a point process model in some format, and
  converts it into an object of class \code{"rmhmodel"}
  so that simulations of the model can be generated using
  the Metropolis-Hastings algorithm \code{\link{rmh}}. 
  
  This function \code{rmhmodel.list} is the method
  for lists. The argument \code{model} should be a named list of parameters
  of the form
  
  \code{list(cif, par, w, trend, types)}

  where \code{cif} and \code{par} are required and the others are
  optional. For details about these components,
  see \code{\link{rmhmodel.default}}.

  The subsequent arguments \code{\dots} (if any) may also
  have these names, and they will take precedence over
  elements of the list \code{model}. 
}  
\references{
   Diggle, P. J. (2003) \emph{Statistical Analysis of Spatial Point
   Patterns} (2nd ed.) Arnold, London.

   Diggle, P.J. and Gratton, R.J. (1984)
   Monte Carlo methods of inference for implicit statistical models.
   \emph{Journal of the Royal Statistical Society, series B}
   \bold{46}, 193 -- 212.

   Diggle, P.J., Gates, D.J., and Stibbard, A. (1987)
   A nonparametric estimator for pairwise-interaction point processes.
   Biometrika \bold{74}, 763 -- 770.
   \emph{Scandinavian Journal of Statistics} \bold{21}, 359--373.

   Geyer, C.J. (1999)
   Likelihood Inference for Spatial Point
   Processes. Chapter 3 in  O.E. Barndorff-Nielsen, W.S. Kendall and
   M.N.M. Van Lieshout (eds) \emph{Stochastic Geometry: Likelihood and
   Computation}, Chapman and Hall / CRC,  Monographs on Statistics and
   Applied Probability, number 80. Pages 79--140.
}

\seealso{
  \code{\link{rmhmodel}},
  \code{\link{rmhmodel.default}},
  \code{\link{rmhmodel.ppm}},
  \code{\link{rmh}},
  \code{\link{rmhcontrol}},
  \code{\link{rmhstart}},
  \code{\link{ppm}},
  \code{\link{Strauss}},
  \code{\link{Softcore}},
  \code{\link{StraussHard}},
  \code{\link{MultiStrauss}},
  \code{\link{MultiStraussHard}},
  \code{\link{DiggleGratton}},
  \code{\link{PairPiece}}
}
\examples{
   # Strauss process:
   mod01 <- list(cif="strauss",par=list(beta=2,gamma=0.2,r=0.7),
                 w=c(0,10,0,10))
   mod01 <- rmhmodel(mod01)

   # Strauss with hardcore:
   mod04 <- list(cif="straush",par=list(beta=2,gamma=0.2,r=0.7,hc=0.3),
                w=owin(c(0,10),c(0,5)))
   mod04 <- rmhmodel(mod04)

   # Soft core:
   w    <- square(10)
   mod07 <- list(cif="sftcr",
                     par=list(beta=0.8,sigma=0.1,kappa=0.5),
                     w=w)
   mod07 <- rmhmodel(mod07)
   
   # Multitype Strauss:
   beta <- c(0.027,0.008)
   gmma <- matrix(c(0.43,0.98,0.98,0.36),2,2)
   r    <- matrix(c(45,45,45,45),2,2)
   mod08 <- list(cif="straussm",
                     par=list(beta=beta,gamma=gmma,radii=r),
                     w=square(250))
   mod08 <- rmhmodel(mod08)

   # specify types
   mod09 <- rmhmodel(list(cif="straussm",
                     par=list(beta=beta,gamma=gmma,radii=r),
                     w=square(250),
                     types=c("A", "B")))

   # Multitype Strauss hardcore with trends for each type:
   beta  <- c(0.27,0.08)
   ri    <- matrix(c(45,45,45,45),2,2)
   rhc  <- matrix(c(9.1,5.0,5.0,2.5),2,2)
   tr3   <- function(x,y){x <- x/250; y <- y/250;
   			   exp((6*x + 5*y - 18*x^2 + 12*x*y - 9*y^2)/6)
                         }
                         # log quadratic trend
   tr4   <- function(x,y){x <- x/250; y <- y/250;
                         exp(-0.6*x+0.5*y)}
                        # log linear trend
   mod10 <- list(cif="straushm",par=list(beta=beta,gamma=gmma,
                 iradii=ri,hradii=rhc),w=c(0,250,0,250),
                 trend=list(tr3,tr4))
   mod10 <- rmhmodel(mod10)

   # Lookup (interaction function h_2 from page 76, Diggle (2003)):
   r <- seq(from=0,to=0.2,length=101)[-1] # Drop 0.
   h <- 20*(r-0.05)
   h[r<0.05] <- 0
   h[r>0.10] <- 1
   mod17 <- list(cif="lookup",par=list(beta=4000,h=h,r=r),w=c(0,1,0,1))
   mod17 <- rmhmodel(mod17)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{boundingcircle}
\alias{boundingradius}
\alias{boundingradius.owin}
\alias{boundingradius.ppp}
\alias{boundingcentre}
\alias{boundingcircle}
\alias{boundingcentre.owin}
\alias{boundingcircle.owin}
\alias{boundingcentre.ppp}
\alias{boundingcircle.ppp}
\title{
  Smallest Enclosing Circle
}
\description{
  Find the smallest circle enclosing a spatial window
  or other object. Return its radius, or the location of its centre,
  or the circle itself.
}
\usage{
boundingradius(x, \dots)

boundingcentre(x, \dots)

boundingcircle(x, \dots)

\method{boundingradius}{owin}(x, \dots)

\method{boundingcentre}{owin}(x, \dots)

\method{boundingcircle}{owin}(x, \dots)

\method{boundingradius}{ppp}(x, \dots)

\method{boundingcentre}{ppp}(x, \dots)

\method{boundingcircle}{ppp}(x, \dots)
}
\arguments{
  \item{x}{
    A window (object of class \code{"owin"}), or another spatial object. 
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine the
    pixel resolution for the calculation.
  }
}
\details{
  The \code{boundingcircle} of a spatial region \eqn{W} is the smallest circle
  that contains \eqn{W}. The \code{boundingradius} is the radius of this circle,
  and the \code{boundingcentre} is the centre of the circle.
  
  The functions \code{boundingcircle}, \code{boundingcentre} and
  \code{boundingradius} are generic. There are methods
  for objects of class \code{"owin"}, \code{"ppp"} and \code{"linnet"}.
}
\value{
  The result of \code{boundingradius} is a single numeric value.

  The result of \code{boundingcentre} is a point pattern containing a
  single point.

  The result of \code{boundingcircle} is a window representing the
  boundingcircle.
}
\author{
  \adrian 
}
\seealso{
 \code{\link{boundingradius.linnet}}
}
\examples{
  boundingradius(letterR)

  plot(grow.rectangle(Frame(letterR), 0.2), main="", type="n")
  plot(letterR, add=TRUE, col="grey")
  plot(boundingcircle(letterR), add=TRUE, border="green", lwd=2)
  plot(boundingcentre(letterR), pch="+", cex=2, col="blue", add=TRUE)

  X <- runifpoint(5)
  plot(X)
  plot(boundingcircle(X), add=TRUE)
  plot(boundingcentre(X), pch="+", cex=2, col="blue", add=TRUE)
}
\keyword{spatial}
\keyword{math}

\name{edges2triangles}
\alias{edges2triangles}
\title{
  List Triangles in a Graph
}
\description{
  Given a list of edges between vertices,
  compile a list of all triangles formed by these edges.
}
\usage{
edges2triangles(iedge, jedge, nvert=max(iedge, jedge), \dots,
                check=TRUE, friendly=rep(TRUE, nvert))
}
\arguments{
  \item{iedge,jedge}{
    Integer vectors, of equal length, specifying the edges.
  }
  \item{nvert}{
    Number of vertices in the network.
  }
  \item{\dots}{Ignored}
  \item{check}{Logical. Whether to check validity of input data.}
  \item{friendly}{
    Optional. For advanced use. See Details.
  }
}
\details{
  This low level function finds all the triangles (cliques of size 3)
  in a finite graph with \code{nvert} vertices and with edges
  specified by \code{iedge, jedge}. 

  The interpretation of \code{iedge, jedge} is that each successive
  pair of entries specifies an edge in the graph.
  The \eqn{k}th edge joins vertex \code{iedge[k]} to vertex \code{jedge[k]}.
  Entries of \code{iedge} and \code{jedge} must be integers
  from 1 to \code{nvert}.

  To improve efficiency in some applications, the optional argument
  \code{friendly} can be used. It should be a logical vector of
  length \code{nvert} specifying a labelling of the vertices,
  such that two vertices \code{j,k} which are \emph{not} friendly
  (\code{friendly[j] = friendly[k] = FALSE})
  are \emph{never} connected by an edge. 
}
\value{
  A 3-column matrix of integers, in which each row represents a triangle.
}
\seealso{
  \code{\link{edges2vees}}
}
\author{\adrian
  
  
  and \rolf
  
}
\examples{
 i <- c(1, 2, 5, 5, 1, 4, 2)
 j <- c(2, 3, 3, 1, 3, 2, 5)
 edges2triangles(i, j)
}
\keyword{spatial}
\keyword{manip}

\name{overlap.owin}
\alias{overlap.owin}
\title{
  Compute Area of Overlap
}
\description{
  Computes the area of the overlap (intersection) of
  two windows.
}
\usage{
overlap.owin(A, B)
}
\arguments{
  \item{A,B}{
    Windows (objects of class \code{"owin"}).
  }
}
\details{
  This function computes the area of the overlap between the two
  windows \code{A} and \code{B}.

  If one of the windows is a binary mask, then both windows
  are converted to masks on the same grid, and the area is
  computed by counting pixels. Otherwise, the area is computed
  analytically (using the discrete Stokes theorem).
}
\value{
  A single numeric value.
}
\seealso{
  \code{\link{intersect.owin}}, 
  \code{\link{area.owin}},
  \code{\link{setcov}}.
}
\examples{
  A <- square(1)
  B <- shift(A, c(0.3, 0.2))
  overlap.owin(A, B)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{rQuasi}
\alias{rQuasi}
\title{
  Generate Quasirandom Point Pattern in Given Window
}
\description{
  Generates a quasirandom pattern of points in any two-dimensional window.
}
\usage{
rQuasi(n, W, type = c("Halton", "Hammersley"), ...)
}
\arguments{
  \item{n}{
    Maximum number of points to be generated.
  }
  \item{W}{
    Window (object of class \code{"owin"}) in which to generate
    the points.
  }
  \item{type}{
    String identifying the quasirandom generator.
  }
  \item{\dots}{
    Arguments passed to the quasirandom generator.
  }
}
\details{
  This function generates a quasirandom point pattern,
  using the quasirandom sequence generator
  \code{\link{Halton}} or \code{\link{Hammersley}}
  as specified.

  If \code{W} is a rectangle, 
  exactly \code{n} points will be generated.

  If \code{W} is not a rectangle, \code{n} points will be generated in the
  containing rectangle \code{as.rectangle(W)},
  and only the points lying inside \code{W} will be retained.
}
\value{
  Point pattern (object of class \code{"ppp"}) inside the window \code{W}.
}
\seealso{
  \code{\link{Halton}}
}
\examples{
   plot(rQuasi(256, letterR))
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege.
}
\keyword{spatial}
\keyword{datagen}

\name{affine.owin} 
\alias{affine.owin} 
\title{Apply Affine Transformation To Window}
\description{
  Applies any affine transformation of the plane (linear transformation
  plus vector shift) to a window. 
}
\usage{
  \method{affine}{owin}(X, mat=diag(c(1,1)), vec=c(0,0), \dots, rescue=TRUE)
}
\arguments{
  \item{X}{Window (object of class \code{"owin"}).}
  \item{mat}{Matrix representing a linear transformation.}
  \item{vec}{Vector of length 2 representing a translation.}
  \item{rescue}{
    Logical. If \code{TRUE}, the transformed window
    will be processed by \code{\link{rescue.rectangle}}.
  }
  \item{\dots}{
    Optional arguments passed to \code{\link{as.mask}}
    controlling the pixel resolution of the transformed window,
    if \code{X} is a binary pixel mask.
  }
}
\value{
  Another window (of class \code{"owin"}) representing the
  result of applying the affine transformation.
}
\details{
  The window is subjected first to the linear transformation represented by
  \code{mat} (multiplying on the left by \code{mat}),
  and then the result is translated by the vector \code{vec}.
  
  The argument \code{mat} must be a nonsingular \eqn{2 \times 2}{2 * 2}
  matrix.

  This is a method for the generic function \code{\link{affine}}.
}
\seealso{
  \code{\link{affine}},
  \code{\link{affine.ppp}},
  \code{\link{affine.psp}},
  \code{\link{affine.im}},
  \code{\link{rotate}},
  \code{\link{shift}}
}
\examples{
  # shear transformation
  shear <- matrix(c(1,0,0.6,1),ncol=2)
  X <- affine(owin(), shear)
  \dontrun{
  plot(X)
  }
  data(letterR)
  affine(letterR, shear, c(0, 0.5))
  affine(as.mask(letterR), shear, c(0, 0.5))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{is.marked.ppm}
\alias{is.marked.ppm}
\alias{is.marked.lppm}
\title{Test Whether A Point Process Model is Marked}
\description{
  Tests whether a fitted point process model involves ``marks''
  attached to the points.
}
\usage{
  \method{is.marked}{ppm}(X, \dots) 

  \method{is.marked}{lppm}(X, \dots) 
}
\arguments{
  \item{X}{
    Fitted point process model (object of class \code{"ppm"})
    usually obtained from \code{\link{ppm}}.
    Alternatively, a model of class \code{"lppm"}.
  }
  \item{\dots}{
    Ignored.
  }
}
\value{
  Logical value, equal to \code{TRUE} if
  \code{X} is a model that was fitted to a marked point pattern dataset.
}
\details{
  ``Marks'' are observations attached to each point of a point pattern.
  For example the \code{\link[spatstat.data]{longleaf}} dataset contains
  the locations of trees, each tree being marked by its diameter;
  the \code{\link[spatstat.data]{amacrine}} dataset gives the locations of cells
  of two types (on/off) and the type of cell may be regarded as a mark attached
  to the location of the cell.

  The argument \code{X} is a fitted point process model
  (an object of class \code{"ppm"}) typically obtained
  by fitting a model to point pattern data using \code{\link{ppm}}.

  This function returns \code{TRUE} if the \emph{original data}
  (to which the model \code{X} was fitted) were a marked point pattern.

  Note that this is not the same as testing whether the
  model involves terms that depend on the marks (i.e. whether the
  fitted model ignores the marks in the data).
  Currently we have not implemented a test for this.

  If this function returns \code{TRUE}, the implications are
  (for example) that
  any simulation of this model will require simulation of random marks
  as well as random point locations.
}
\seealso{
  \code{\link{is.marked}},
  \code{\link{is.marked.ppp}}
}
\examples{
   X <- lansing
   # Multitype point pattern --- trees marked by species

   \testonly{
      # Smaller dataset
      X <- amacrine
   }

  fit1 <- ppm(X, ~ marks, Poisson())
  is.marked(fit1)
  # TRUE

  fit2 <- ppm(X, ~ 1, Poisson())
  is.marked(fit2)
  # TRUE

  # Unmarked point pattern
  fit3 <- ppm(cells, ~ 1, Poisson())
  is.marked(fit3)
  # FALSE

}

\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}
\keyword{models}

\name{summary.ppp}
\alias{summary.ppp}
\title{Summary of a Point Pattern Dataset}
\description{
  Prints a useful summary of a point pattern dataset.
}
\usage{
 \method{summary}{ppp}(object, \dots, checkdup=TRUE)
}
\arguments{
  \item{object}{
    Point pattern (object of class \code{"ppp"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{checkdup}{
    Logical value indicating whether to check
    for the presence of duplicate points.
  }
}
\details{
  A useful summary of the point pattern \code{object} is printed.

  This is a method for the generic function \code{\link{summary}}.

  If \code{checkdup=TRUE}, the pattern will be checked for the
  presence of duplicate points, using \code{\link{duplicated.ppp}}.
  This can be time-consuming if the pattern contains many points,
  so the checking can be disabled by setting \code{checkdup=FALSE}.

  If the point pattern was generated by simulation
  using \code{\link{rmh}}, the parameters of the algorithm
  are printed.
}
\seealso{
  \code{\link{summary}},
  \code{\link{summary.owin}},
  \code{\link{print.ppp}}
}
\examples{
  summary(cells)  # plain vanilla point pattern

  # multitype point pattern
  woods <- lansing
  \testonly{woods <- woods[seq(1, npoints(woods), length=40)]}
  summary(woods) # tabulates frequencies of each mark
  
  # numeric marks
  trees <- longleaf
  \testonly{trees <- trees[seq(1, npoints(trees), length=40)]}
  summary(trees) # prints summary.default(marks(trees))

  # weird polygonal window
  summary(demopat)  # describes it
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{methods}

\name{unstack.solist}
\alias{unstack.solist}
\alias{unstack.layered}
\title{
  Unstack Each Spatial Object in a List of Objects
}
\description{
  Given a list of two-dimensional spatial objects,
  apply 
  
}
\usage{
\method{unstack}{solist}(x, \dots)

\method{unstack}{layered}(x, \dots)
}
\arguments{
  \item{x}{
    An object of class \code{"solist"} or \code{"layered"}
    representing a list of two-dimensional spatial objects.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  The functions defined here are methods for the generic
  \code{\link[utils]{unstack}}. They expect the argument \code{x}
  to be a list of spatial objects, of class \code{"solist"}
  or \code{"layered"}.

  Each spatial object in the list \code{x}
  will be unstacked by applying the relevant method for
  \code{\link[utils]{unstack}}. This means that
  \itemize{
    \item a marked point pattern with several columns of marks
    will be separated into several point patterns, each having
    a single column of marks
    \item a measure with \eqn{k}-dimensional vector values
    will be separated into \eqn{k} measures with scalar values
  }
  The resulting unstacked objects
  will be collected into a list of the same kind as \code{x}.
  Typically the length of \code{unstack(x)} is greater than
  the length of \code{x}.
}
\value{
  A list belonging to the same class as \code{x}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link[utils]{unstack}}

  \code{\link{unstack.msr}},
  \code{\link{unstack.ppp}},
  \code{\link{unstack.lpp}},
  \code{\link{unstack.psp}}
}
\examples{
  A <- solist(finpines=finpines, cells=cells)
  A
  unstack(A)
  B <- layered(fin=finpines, loc=unmark(finpines),
              plotargs=list(list(), list(pch=16)))
  B
  plot(B)
  unstack(B)
  plot(unstack(B))
}
\keyword{spatial}
\keyword{manip}

\name{pairdist}
\alias{pairdist}
\title{Pairwise distances}
\description{
  Computes the matrix of distances between all pairs of `things'
  in a dataset
}
\usage{
  pairdist(X, \dots)
}
\arguments{
  \item{X}{
    Object specifying the locations of a set of `things'
    (such as a set of points or a set of line segments).
  }
  \item{\dots}{
    Further arguments depending on the method.
  }
}
\value{
  A square matrix whose \code{[i,j]} entry is the distance
  between the `things' numbered \code{i} and \code{j}.
}
\details{
  Given a dataset \code{X} and \code{Y}
  (representing either a point pattern or
  a line segment pattern)
  \code{pairdist} computes the distance between each pair of
  `things' in the dataset, and returns 
  a matrix containing these distances.

  The function \code{pairdist} is generic, with
  methods for point patterns (objects of class \code{"ppp"}),
  line segment patterns (objects of class \code{"psp"})
  and a default method. See the documentation
  for \code{\link{pairdist.ppp}}, \code{\link{pairdist.psp}}
  or \code{\link{pairdist.default}} for details.
}
\seealso{
  \code{\link{pairdist.ppp}},
  \code{\link{pairdist.psp}},
  \code{\link{pairdist.default}},
  \code{\link{crossdist}},
  \code{\link{nndist}},
  \code{\link{Kest}}
}
\author{Pavel Grabarnik
  \email{pavel.grabar@issp.serpukhov.su}
  and 
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{bounding.box.xy}
\alias{bounding.box.xy}
\title{Convex Hull of Points}
\description{
  Computes the smallest rectangle containing a set of points.
}
\usage{
bounding.box.xy(x, y=NULL)
}
\arguments{
  \item{x}{
    vector of \code{x} coordinates of observed points,
    or a 2-column matrix giving \code{x,y} coordinates,
    or a list with components \code{x,y} giving coordinates
    (such as a point pattern object of class \code{"ppp"}.)
  }
  \item{y}{(optional) vector of \code{y} coordinates of observed points,
    if \code{x} is a vector.}
}
\value{
  A window (an object of class \code{"owin"}).
}
\details{
  Given an observed pattern of points with coordinates 
  given by \code{x} and \code{y}, this function finds the smallest
  rectangle, with sides parallel to the coordinate axes, that contains
  all the points, and returns it as a window.
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{convexhull.xy}},
  \code{\link{ripras}}
}
\examples{
  x <- runif(30)
  y <- runif(30)
  w <- bounding.box.xy(x,y)
  plot(owin(), main="bounding.box.xy(x,y)")
  plot(w, add=TRUE)
  points(x,y)

  X <- rpoispp(30)
  plot(X, main="bounding.box.xy(X)")
  plot(bounding.box.xy(X), add=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{utilities}

\name{perimeter}
\Rdversion{1.1}
\alias{perimeter}
\title{
  Perimeter Length of Window
}
\description{
  Computes the perimeter length of a window
}
\usage{
perimeter(w)
}
\arguments{
  \item{w}{
    A window (object of class \code{"owin"})
    or data that can be converted to a window by \code{\link{as.owin}}.
  }
}
\details{
  This function computes the perimeter (length of the boundary) of the
  window \code{w}. If \code{w} is a rectangle or a polygonal window,
  the perimeter is the sum of the lengths of the edges of \code{w}.
  If \code{w} is a mask, it is first converted to a polygonal window
  using \code{\link{as.polygonal}}, then staircase edges are removed
  using \code{\link{simplify.owin}}, and the perimeter of the resulting
  polygon is computed.
}
\value{
  A numeric value giving the perimeter length of the window.
}
\seealso{
  \code{\link{area.owin}}
  \code{\link{diameter.owin}},
  \code{\link{owin.object}},
  \code{\link{as.owin}}
}
\examples{
  perimeter(square(3))
  data(letterR)
  perimeter(letterR)
  if(interactive()) print(perimeter(as.mask(letterR)))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{mean.im} %DontDeclareMethods
\alias{mean.im}
\alias{median.im}
\title{Mean and Median of Pixel Values in an Image}
\description{
  Calculates the mean or median
  of the pixel values in a pixel image.
}
%NAMESPACE S3method("mean", "im")
%NAMESPACE S3method("median", "im")
\usage{
## S3 method for class 'im'
## mean(x, trim=0, na.rm=TRUE, ...) 

## S3 method for class 'im'
## median(x, na.rm=TRUE)        [R < 3.4.0]
## median(x, na.rm=TRUE, ...)   [R >= 3.4.0]
}
\arguments{
  \item{x}{A pixel image (object of class \code{"im"}).}
  \item{na.rm}{
    Logical value indicating whether \code{NA} values should be
    stripped before the computation proceeds.
  }
  \item{trim}{
    The fraction (0 to 0.5) of pixel values to be trimmed from
    each end of their range, before the mean is computed.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  These functions calculate the mean and median
  of the pixel values in the image \code{x}.

  An object of class \code{"im"}
  describes a pixel image. See \code{\link{im.object}})
  for details of this class.

  The function \code{mean.im} is a method for the generic
  function \code{\link[base]{mean}} for the class \code{"im"}.
  Similarly \code{median.im} is a method for the generic
  \code{\link[stats]{median}}.
  
  If the image \code{x} is logical-valued, the mean value of \code{x} is
  the fraction of pixels that have the value \code{TRUE}. The median is
  not defined. 

  If the image \code{x} is factor-valued, then the mean of \code{x}
  is the mean of the integer codes of the pixel values. The median is
  are not defined.

  Other mathematical operations on images are supported by
  \code{\link{Math.im}}, \code{\link{Summary.im}}
  and \code{\link{Complex.im}}.

  Other information about an image can be obtained using
  \code{\link{summary.im}} or \code{\link{quantile.im}}.
}
\value{
  A single number.
}
\seealso{
  \code{\link{Math.im}} for other operations.

  Generics and default methods:
  \code{\link[base]{mean}},
  \code{\link[stats]{median}}.
  
  \code{\link{quantile.im}},
  \code{\link{anyNA.im}},
  \code{\link{im.object}},
  \code{\link{summary.im}}.
}
\examples{
  X <- as.im(function(x,y) {x^2}, unit.square())
  mean(X)
  median(X)
  mean(X, trim=0.05)
}
\author{
  \spatstatAuthors and Kassel Hingee.
}
\keyword{spatial}
\keyword{methods}
\keyword{univar}

\name{gridcentres}
\alias{gridcentres}
\alias{gridcenters}
\title{Rectangular grid of points}
\description{
  Generates a rectangular grid of points in a window
}
\usage{
 gridcentres(window, nx, ny)
}
\arguments{
  \item{window}{A window. 
    An object of class \code{\link{owin}},
    or data in any format acceptable to \code{\link{as.owin}()}.
  }
  \item{nx}{Number of points
    in each row of the rectangular grid.
  }
  \item{ny}{Number of points
    in each column of the rectangular grid.
  }
}
\value{
  A list with two components \code{x} and \code{y}, which are numeric
  vectors giving the coordinates of the points of the
  rectangular grid.
}
\details{
  This function creates a rectangular grid of points in the window.
  
  The bounding rectangle of the \code{window} is divided into
  a regular \eqn{nx \times ny}{nx * ny} grid of rectangular tiles.
  The function returns the \eqn{x,y} coordinates of the
  centres of these tiles.

  Note that some of these grid points may lie outside the window,
  if \code{window} is not of type \code{"rectangle"}. The function
  \code{\link{inside.owin}} can be used to select those grid points
  which do lie inside the window. See the examples.

  This function is useful in creating dummy points for quadrature
  schemes (see \code{\link{quadscheme}}) and for other miscellaneous
  purposes.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{quadscheme}},
  \code{\link{inside.owin}},
  \code{\link{stratrand}}
}
\examples{
  w <- unit.square()
  xy <- gridcentres(w, 10,15)
  \dontrun{
  plot(w)
  points(xy)
  }

  bdry <- list(x=c(0.1,0.3,0.7,0.4,0.2),
               y=c(0.1,0.1,0.5,0.7,0.3))
  w <- owin(c(0,1), c(0,1), poly=bdry)
  xy <- gridcentres(w, 30, 30)
  ok <- inside.owin(xy$x, xy$y, w)
  \dontrun{
  plot(w)
  points(xy$x[ok], xy$y[ok])
  }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{as.rectangle}
\alias{as.rectangle}
\title{Window Frame}
\description{
  Extract the window frame of a window
  or other spatial dataset
}
\usage{
 as.rectangle(w, \dots)
}
\arguments{
  \item{w}{A window, or a dataset that has a window.
    Either a window (object of class \code{"owin"}),
    a pixel image (object of class \code{"im"})
    or other data determining such a window.
  }
  \item{\dots}{
    Optional. Auxiliary data to help determine the window. If
    \code{w} does not belong to a recognised class,
    the arguments \code{w} and \code{\dots} 
    are passed to \code{\link{as.owin}} to determine the window.
  }
}
\value{
  A window (object of class \code{"owin"})
  of type \code{"rectangle"} representing a rectangle.
}
\details{
  This function is the quickest way to determine a bounding rectangle
  for a spatial dataset.

  If \code{w} is a window, the function
  just extracts the outer bounding rectangle
  of \code{w} as given by its elements \code{xrange,yrange}.

  The function can also be applied to any spatial dataset that has a window:
  for example, a point pattern (object of class \code{"ppp"}) or
  a line segment pattern (object of class \code{"psp"}).
  The bounding rectangle of the window of the dataset is extracted.

  Use the function \code{\link{boundingbox}} to compute the \emph{smallest}
  bounding rectangle of a dataset.
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{boundingbox}}
}
\examples{
  w <- owin(c(0,10),c(0,10), poly=list(x=c(1,2,3,2,1), y=c(2,3,4,6,7)))
  r <- as.rectangle(w)
  # returns a 10 x 10 rectangle

  data(lansing)
  as.rectangle(lansing)

  data(copper)
  as.rectangle(copper$SouthLines)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{summary.psp}
\alias{summary.psp}
\title{Summary of a Line Segment Pattern Dataset}
\description{
  Prints a useful summary of a line segment pattern dataset.
}
\usage{
  \method{summary}{psp}(object, \dots)
}
\arguments{
  \item{object}{Line segment pattern (object of class \code{"psp"}).}
  \item{\dots}{Ignored.}
}
\details{
  A useful summary of the line segment pattern \code{object} is printed.

  This is a method for the generic function \code{\link{summary}}.
}
\seealso{
  \code{\link{summary}},
  \code{\link{summary.owin}},
  \code{\link{print.psp}}
}
\examples{
  a <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  summary(a)  # describes it
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}

\name{ppp.object}
\alias{ppp.object} %DoNotExport
\title{Class of Point Patterns}
\description{
  A class \code{"ppp"} to represent a two-dimensional point
  pattern. Includes information about the window in which the
  pattern was observed. Optionally includes marks.
}
\details{
  This class represents
  a two-dimensional point pattern dataset. It specifies
  \itemize{
    \item the locations of the points
    \item the window in which the pattern was observed
    \item optionally, ``marks'' attached to each point
      (extra information such as a type label).
    }
  If \code{X} is an object of type \code{ppp},
  it contains the following elements:
  \tabular{ll}{
    \code{x} \tab vector of \eqn{x} coordinates of data points \cr
    \code{y} \tab vector of \eqn{y} coordinates of data points \cr
    \code{n} \tab number of points \cr
    \code{window} \tab window of observation \cr
                  \tab (an object of class \code{\link{owin}}) \cr
    \code{marks} \tab optional vector or data frame of marks
  }
  Users are strongly advised not to manipulate these entries
  directly.
  
  Objects of class \code{"ppp"}
  may be created by the function
  \code{\link{ppp}}
  and converted from other types of data by the function
  \code{\link{as.ppp}}.
  Note that you must always specify the window of observation;
  there is intentionally no default action of ``guessing'' the window
  dimensions from the data points alone.

  Standard point pattern datasets provided with the package
  include
  \code{\link[spatstat.data]{amacrine}},
  \code{\link[spatstat.data]{betacells}},
  \code{\link[spatstat.data]{bramblecanes}},
  \code{\link[spatstat.data]{cells}},
  \code{\link[spatstat.data]{demopat}},
  \code{\link[spatstat.data]{ganglia}},
  \code{\link[spatstat.data]{lansing}},
  \code{\link[spatstat.data]{longleaf}},
  \code{\link[spatstat.data]{nztrees}},
  \code{\link[spatstat.data]{redwood}},
  \code{\link[spatstat.data]{simdat}} and
  \code{\link[spatstat.data]{swedishpines}}.
  
  Point patterns may be scanned from your own data files by
  \code{\link{scanpp}} or by using \code{\link{read.table}} and
  \code{\link{as.ppp}}.
  
  They may be manipulated by the functions 
  \code{\link{[.ppp}} and
  \code{\link{superimpose}}.

  Point pattern objects can be plotted just by typing \code{plot(X)}
  which invokes the \code{plot} method for point pattern objects,
  \code{\link{plot.ppp}}. See \code{\link{plot.ppp}} for further information.

  There are also methods for \code{summary} and \code{print}
  for point patterns. Use \code{summary(X)} to see a useful description
  of the data.
  
  Patterns may be generated at random by
   \code{\link{runifpoint}},
   \code{\link{rpoispp}},
   \code{\link{rMaternI}},
   \code{\link{rMaternII}},
   \code{\link{rSSI}},
   \code{\link{rNeymanScott}},
   \code{\link{rMatClust}},
   and
   \code{\link{rThomas}}.

   Most functions which are intended to operate on a window
   (of class \code{\link{owin}})
   will, if presented with a \code{\link{ppp}} object instead,
   automatically extract the window information from the point pattern.
}
\seealso{
  \code{\link{owin}},
  \code{\link{ppp}},
  \code{\link{as.ppp}},
  \code{\link{[.ppp}}
}
\section{Warnings}{
  The internal representation of marks is likely to change in the
  next release of this package.
}
\examples{
  x <- runif(100)
  y <- runif(100)
  X <- ppp(x, y, c(0,1),c(0,1))
  X
  \dontrun{plot(X)}
  mar <- sample(1:3, 100, replace=TRUE)
  mm <- ppp(x, y, c(0,1), c(0,1), marks=mar)
  \dontrun{plot(mm)}
  # points with mark equal to 2
  ss <- mm[ mm$marks == 2 , ]
  \dontrun{plot(ss)}
  # left half of pattern 'mm'
  lu <- owin(c(0,0.5),c(0,1))
  mmleft <- mm[ , lu]
  \dontrun{plot(mmleft)}
  \dontrun{
  if(FALSE) {
  # input data from file
  qq <- scanpp("my.table", unit.square())

  # interactively build a point pattern
  plot(unit.square())
  X <- as.ppp(locator(10), unit.square())
  plot(X)
  }
 }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{attribute}
 
 

\name{sessionLibs}
\alias{sessionLibs}
\title{
  Print Names and Version Numbers of Libraries Loaded
}
\description{
  Prints the names and version numbers
  of libraries currently loaded by the user.
}
\usage{
  sessionLibs()
}
\details{
  This function prints a list of the libraries loaded by the user
  in the current session, giving just their name and version number.
  It obtains this information from \code{\link[utils]{sessionInfo}}.

  This function is not needed in an interactive \R session
  because the package startup messages will usually provide this
  information.
  
  Its main use is in an \code{\link{Sweave}} script, where it is needed
  because the package startup messages are not printed. 
}
\value{
  Null.
}
\examples{
  sessionLibs()
}
\author{
  \adrian
  and \rolf.
}
\seealso{
  \code{\link[utils]{sessionInfo}}
}
\keyword{data}

\name{textureplot}
\alias{textureplot}
\title{
  Plot Image or Tessellation Using Texture Fill
}
\description{
  For a factor-valued pixel image, this command plots each level
  of the factor using a different texture.
  For a tessellation, each tile is plotted using a different texture.
}
\usage{
textureplot(x, \dots,
            main, add=FALSE, clipwin=NULL, do.plot = TRUE,
            border=NULL, col = NULL, lwd = NULL, lty = NULL, spacing = NULL,
            textures=1:8,
            legend=TRUE,
            leg.side=c("right", "left", "bottom", "top"),
            legsep=0.1, legwid=0.2)
}
\arguments{
  \item{x}{
    A tessellation (object of class \code{"tess"} or something
    acceptable to \code{\link{as.tess}}) with at most 8 tiles,
    or a pixel image (object of class \code{"im"} or something
    acceptable to \code{\link{as.im}}) whose pixel values
    are a \code{factor} with at most 8 levels.
  }
  \item{\dots}{
    Other arguments passed to \code{\link{add.texture}}.
  }
  \item{main}{
    Character string giving a main title for the plot.
  }
  \item{add}{
    Logical value indicating whether to draw on
    the current plot (\code{add=TRUE}) or to initialise a new plot
    (\code{add=FALSE}).
  }
  \item{clipwin}{
    Optional. A window (object of class \code{"owin"}).
    Only this subset of the image will be displayed.
  }
  \item{do.plot}{
    Logical. Whether to actually do the plot.
  }
  \item{border}{
    Colour for drawing the boundaries between the different regions.
    The default (\code{border=NULL}) means to use \code{par("fg")}.
    Use \code{border=NA} to omit borders.
  }
  \item{col}{
    Numeric value or vector giving the colour or colours in which
    the textures should be plotted.
  }
  \item{lwd}{
    Numeric value or vector giving the line width or widths to be used.
  }
  \item{lty}{
    Numeric value or vector giving the line type or types to be used.
  }
  \item{spacing}{
    Numeric value or vector giving the \code{spacing} parameter
    for the textures.
  }
  \item{textures}{
    Textures to be used for each level. Either a texture map (object of
    class \code{"texturemap"}) or a vector of integer codes
    (to be interpreted by \code{\link{add.texture}}).
  }
  \item{legend}{
    Logical. Whether to display an explanatory legend.
  }
  \item{leg.side}{Position of legend relative to main plot.}
  \item{legsep}{
    Separation between legend and main plot, as a fraction
    of the shortest side length of the main plot.
  }
  \item{legwid}{
    Width (if vertical) or height (if horizontal) of the legend
    as a fraction of the shortest side length of the main plot.
  }
}
\details{
  If \code{x} is a tessellation, then each tile of the tessellation is
  plotted and filled with a texture using \link{add.texture}.

  If \code{x} is a factor-valued pixel image, then
  for each level of the factor, the algorithm finds the region where the image
  takes this value, and fills the region with a texture using
  \code{\link{add.texture}}.
}
\value{
  (Invisible) A texture map (object of class \code{"texturemap"})
  associating a texture with each level of the factor.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{im}}, 
  \code{\link{plot.im}},
  \code{\link{add.texture}}.
}
\examples{
  nd <- if(interactive()) 128 else 32
  Z <- setcov(owin(), dimyx=nd)
  Zcut <- cut(Z, 3, labels=c("Lo", "Med", "Hi"))
  textureplot(Zcut)
  textureplot(dirichlet(runifpoint(6)))
}
\keyword{spatial}
\keyword{hplot}

\name{areaGain}
\alias{areaGain}
\title{Difference of Disc Areas}
\description{
  Computes the area of that part of a disc
  that is not covered by other discs.
}
\usage{
   areaGain(u, X, r, ..., W=as.owin(X), exact=FALSE,
                     ngrid=spatstat.options("ngrid.disc"))
}
\arguments{
  \item{u}{
    Coordinates of the centre of the disc of interest.
    A vector of length 2.
    Alternatively, a point pattern (object of class \code{"ppp"}).
  }
  \item{X}{
    Locations of the centres of other discs.
    A point pattern (object of class \code{"ppp"}).
  }
  \item{r}{
    Disc radius, or vector of disc radii.
  }
  \item{\dots}{
    Arguments passed to \code{\link{distmap}} to determine the pixel
    resolution, when \code{exact=FALSE}.
  }
  \item{W}{
    Window (object of class \code{"owin"})
    in which the area should be computed.
  }
  \item{exact}{
    Choice of algorithm.
    If \code{exact=TRUE}, areas are computed exactly using
    analytic geometry. If \code{exact=FALSE} then a faster algorithm
    is used to compute a discrete approximation to the areas.
  }
  \item{ngrid}{
    Integer. Number of points in the square grid used to compute
    the discrete approximation, when \code{exact=FALSE}.
}
}
\value{
  A matrix with one row for each point in \code{u}
  and one column for each value in \code{r}. 
}
\details{
  This function computes the area of that part of
  the disc of radius \code{r} centred at the location \code{u}
  that is \emph{not} covered by any of the discs of radius \code{r}
  centred at the points of the pattern \code{X}.
  This area is important in some calculations related to
  the area-interaction model \code{\link{AreaInter}}.

  If \code{u} is a point pattern and \code{r} is a vector,
  the result is a matrix, with one row for each point in \code{u}
  and one column for each entry of \code{r}. The \code{[i,j]} entry
  in the matrix is the area of that part of the disc of radius
  \code{r[j]} centred at the location \code{u[i]} that is
  \emph{not} covered by any of the discs of radius \code{r[j]}
  centred at the points of the pattern \code{X}.

  If \code{W} is not \code{NULL}, then the areas are computed only
  inside the window \code{W}. 
}
\seealso{
  \code{\link{AreaInter}},
  \code{\link{areaLoss}}
}
\examples{
   data(cells)
   u <- c(0.5,0.5)
   areaGain(u, cells, 0.1)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{matchingdist}
\alias{matchingdist}
\title{Distance for a Point Pattern Matching}
\description{
  Computes the distance associated with a matching between two point patterns.
}
\usage{
  matchingdist(matching, type = NULL, cutoff = NULL, q = NULL)
}
\arguments{
  \item{matching}{A point pattern matching (an object of class \code{"pppmatching"}).}
  \item{type}{
    A character string giving the type of distance to be computed.
    One of \code{"spa"}, \code{"ace"} or \code{"mat"}. See details below. 
  }
  \item{cutoff}{
    The value \eqn{> 0} at which interpoint distances are cut off.
  }
  \item{q}{
    The order of the average that is applied to the interpoint distances.
    May be \code{Inf}, in which case the maximum of the interpoint distances is taken.
  }
}
\details{
  Computes the distance specified by \code{type}, \code{cutoff}, and \code{order}
  for a point matching. If any of these arguments are not provided, the function
  uses the corresponding elements of \code{matching} (if available).

  For the type \code{"spa"} (subpattern assignment) it is assumed that the points
  of the point pattern with the smaller cardinality \eqn{m} are matched to a
  \eqn{m}-point subpattern of the point pattern with the larger
  cardinality \eqn{n} in a 1-1 way. The distance
  is then given as the \code{q}-th order average of the \eqn{m} distances between
  matched points (minimum of Euclidean distance and \code{cutoff})
  and \eqn{n-m} "penalty distances" of value \code{cutoff}.

  For the type \code{"ace"} (assignment only if cardinalities equal) the matching
  is assumed to be 1-1 if the cardinalities of the point patterns are
  the same, in which case the \code{q}-th order average of the matching distances
  (minimum of Euclidean distance and \code{cutoff}) is taken. If the cardinalities
  are different, the matching may be arbitrary and the distance returned is always
  equal to \code{cutoff}.

  For the type \code{mat} (mass transfer) it is assumed that each point of
  the point pattern with the smaller cardinality \eqn{m} has mass \eqn{1},
  each point of the point pattern with the larger cardinality \eqn{n}
  has mass \eqn{m/n},
  and fractions of these masses are matched in such a way that each point
  contributes exactly its mass. The distance is then given as the \code{q}-th
  order weighted average of all distances (minimum of Euclidean distance
  and \code{cutoff}) of (partially) matched points with weights equal to the
  fractional masses divided by \eqn{m}.

  If the cardinalities of the two point patterns are equal,
  \code{matchingdist(m, type, cutoff, q)} yields the same result
  no matter if \code{type} is \code{"spa"}, \code{"ace"} or
  \code{"mat"}.
}
\value{
  Numeric value of the distance associated with the matching.
}
\author{
  Dominic Schuhmacher
  \email{dominic.schuhmacher@stat.unibe.ch}
  \url{http://www.dominic.schuhmacher.name}
}
\seealso{
  \code{\link{pppdist}}
  \code{\link{pppmatching.object}}
}
\examples{
  # an optimal matching
  X <- runifpoint(20)
  Y <- runifpoint(20)
  m.opt <- pppdist(X, Y)
  summary(m.opt)
  matchingdist(m.opt)
       # is the same as the distance given by summary(m.opt)
  
  # sequential nearest neighbour matching
  # (go through all points of point pattern X in sequence
  # and match each point with the closest point of Y that is
  # still unmatched)
  am <- matrix(0, 20, 20)
  h <- matrix(c(1:20, rep(0,20)), 20, 2)
  h[1,2] = nncross(X[1],Y)[1,2]
  for (i in 2:20) {
    nn <- nncross(X[i],Y[-h[1:(i-1),2]])[1,2]
    h[i,2] <- ((1:20)[-h[1:(i-1),2]])[nn]
  }
  am[h] <- 1
  m.nn <- pppmatching(X, Y, am)
  matchingdist(m.nn, type="spa", cutoff=1, q=1)
       # is >= the distance obtained for m.opt
       # in most cases strictly >

  opa <- par(mfrow=c(1,2))
  plot(m.opt, main="optimal")
  plot(m.nn,  main="nearest neighbour")
  text(X, 1:20, pos=1, offset=0.3, cex=0.8)
  par(opa)
}
\keyword{spatial}
\keyword{math}

\name{as.matrix.im}
\alias{as.matrix.im}
\alias{as.array.im}
\title{Convert Pixel Image to Matrix or Array}
\description{
  Converts a pixel image to a matrix or an array.
}
\usage{
  \method{as.matrix}{im}(x, ...)
  \method{as.array}{im}(x, ...)
}
\arguments{
  \item{x}{A pixel image (object of class \code{"im"}).}
  \item{\dots}{See below.}
}
\details{
  The function \code{as.matrix.im} converts the pixel image \code{x}
  into a matrix containing the pixel values.
  It is handy when you want to extract a summary of the pixel values.
  See the Examples.
  
  The function \code{as.array.im} converts the pixel image to an array.
  By default this is a three-dimensional array of dimension \eqn{n} by
  \eqn{m} by \eqn{1}. 
  If the extra arguments \code{\dots} are given, they will be passed
  to \code{\link{array}}, and they may change the dimensions of the
  array.
}
\value{
  A matrix or array.
}
\seealso{
  \code{\link{as.matrix.owin}}
}
\examples{
   # artificial image
   Z <- setcov(square(1))

   M <- as.matrix(Z)

   median(M)
   
   \dontrun{
   # plot the cumulative distribution function of pixel values
   plot(ecdf(as.matrix(Z)))
   }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}

\name{reduced.sample}
\alias{reduced.sample}
\title{Reduced Sample Estimator using Histogram Data}
\description{
  Compute the Reduced Sample estimator of a survival time distribution
  function, from histogram data
}
\usage{
  reduced.sample(nco, cen, ncc, show=FALSE, uppercen=0)
}
\arguments{
  \item{nco}{vector of counts giving the histogram of
    uncensored observations (those survival times that are less than or
    equal to the censoring time)
  }
  \item{cen}{vector of counts giving the histogram of
    censoring times
  }
  \item{ncc}{vector of counts giving the histogram of
    censoring times for the uncensored observations only 
  }
  \item{uppercen}{
    number of censoring times greater than the rightmost
    histogram breakpoint (if there are any)
  }
  \item{show}{Logical value controlling the amount of detail
    returned by the function value (see below)
  }
}
\value{
  If \code{show = FALSE}, a numeric vector giving the values of
  the reduced sample estimator.
  If \code{show=TRUE}, a list with three components which are
  vectors of equal length,
  \item{rs}{Reduced sample estimate of the survival time c.d.f. \eqn{F(t)}
  }
  \item{numerator}{numerator of the reduced sample estimator
  }
  \item{denominator}{denominator of the reduced sample estimator
  }
}
\details{
  This function is needed mainly for internal use in \pkg{spatstat},
  but may be useful in other applications where you want to form the
  reduced sample estimator from a huge dataset.

  Suppose \eqn{T_i}{T[i]} are the survival times of individuals
  \eqn{i=1,\ldots,M} with unknown distribution function \eqn{F(t)}
  which we wish to estimate. Suppose these times are right-censored
  by random censoring times \eqn{C_i}{C[i]}.
  Thus the observations consist of right-censored survival times
  \eqn{\tilde T_i = \min(T_i,C_i)}{T*[i] = min(T[i],C[i])}
  and non-censoring indicators
  \eqn{D_i = 1\{T_i \le C_i\}}{D[i] = 1(T[i] <= C[i])}
  for each \eqn{i}.

  If the number of observations \eqn{M} is large, it is efficient to
  use histograms.
  Form the histogram \code{cen} of all censoring times \eqn{C_i}{C[i]}.
  That is, \code{obs[k]} counts the number of values 
  \eqn{C_i}{C[i]} in the interval
  \code{(breaks[k],breaks[k+1]]} for \eqn{k > 1}
  and \code{[breaks[1],breaks[2]]} for \eqn{k = 1}.
  Also form the histogram \code{nco} of all uncensored times,
  i.e. those \eqn{\tilde T_i}{T*[i]} such that \eqn{D_i=1}{D[i]=1},
  and the histogram of all censoring times for which the survival time
  is uncensored,
  i.e. those \eqn{C_i}{C[i]} such that \eqn{D_i=1}{D[i]=1}.
  These three histograms are the arguments passed to \code{kaplan.meier}.

  The return value \code{rs} is the reduced-sample estimator
  of the distribution function \eqn{F(t)}. Specifically,
  \code{rs[k]} is the reduced sample estimate of \code{F(breaks[k+1])}.
  The value is exact, i.e. the use of histograms does not introduce any
  approximation error.

  Note that, for the results to be valid, either the histogram breaks
  must span the censoring times, or the number of censoring times
  that do not fall in a histogram cell must have been counted in
  \code{uppercen}.
}
\seealso{
  \code{\link{kaplan.meier}},
  \code{\link{km.rs}}
}
\author{\adrian
  
  
  and \rolf
  
  }
\keyword{spatial}
\keyword{nonparametric}

\name{print.owin}
\alias{print.owin}
\title{Print Brief Details of a Spatial Window}
\description{
  Prints a very brief description of a window object.
}
\usage{
  \method{print}{owin}(x, \dots, prefix="window: ")
}
\arguments{
  \item{x}{Window (object of class \code{"owin"}).}
  \item{\dots}{Ignored.}
  \item{prefix}{Character string to be printed at the start of the output.}
}
\details{
  A very brief description of the window \code{x} is printed.

  This is a method for the generic function \code{\link{print}}.
}
\seealso{
  \code{\link{print}},
  \code{\link{print.ppp}},
  \code{\link{summary.owin}}
}
\examples{
  owin()  # the unit square

  data(demopat)
  W <- Window(demopat)
  W                    # just says it is polygonal
  as.mask(W)           # just says it is a binary image 

}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{print}

\name{unnormdensity}
\alias{unnormdensity}
\title{
  Weighted kernel smoother
}
\description{
  An unnormalised version of kernel density estimation
  where the weights are not required to sum to 1.
  The weights may be positive, negative or zero.
}
\usage{
unnormdensity(x, ..., weights = NULL)
}
\arguments{
  \item{x}{
    Numeric vector of data
  }
  \item{\dots}{
    Arguments passed to \code{\link{density.default}}.
    Arguments must be \emph{named}.
  }`
  \item{weights}{
    Optional numeric vector of weights for the data.
  }
}
\details{
  This is an alternative to the standard \R kernel density estimation function
  \code{\link{density.default}}.

  The standard \code{\link{density.default}}
  requires the \code{weights} to be nonnegative numbers that add up to 1,
  and returns a probability density (a function that integrates to 1).
  
  This function \code{unnormdensity} does not impose any requirement
  on the \code{weights} except that they be finite. Individual weights may be
  positive, negative or zero. The result is a function that does not
  necessarily integrate to 1 and may be negative. The result is
  the convolution of the kernel \eqn{k} with the weighted data,
  \deqn{
    f(x) = \sum_i w_i k(x- x_i)
  }{
    f(x) = sum of w[i] * k(x - x[i])
  }
  where \eqn{x_i}{x[i]} are the data points and \eqn{w_i}{w[i]} are the
  weights.

  The algorithm first selects the kernel bandwidth by
  applying \code{\link{density.default}} to the data
  \code{x} with normalised, positive weight vector
  \code{w = abs(weights)/sum(abs(weights))} and
  extracting the selected bandwidth.
  Then the result is computed by applying
  applying \code{\link{density.default}} to \code{x} twice
  using the normalised positive and negative parts of the weights.
  
  Note that the arguments \code{\dots} must be passed by name,
  i.e. in the form (\code{name=value}). Arguments that do not match
  an argument of \code{\link{density.default}} will be ignored
  \emph{silently}.
}
\value{
  Object of class \code{"density"} as described in
  \code{\link{density.default}}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
 \code{\link{density.default}}
}
\examples{
  d <- unnormdensity(1:3, weights=c(-1,0,1))
  if(interactive()) plot(d)
}
\keyword{smooth}

\name{plot.yardstick}
\alias{plot.yardstick}
\title{
  Plot a Yardstick or Scale Bar
}
\description{
  Plots an object of class \code{"yardstick"}.
}
\usage{
 \method{plot}{yardstick}(x, \dots,
        angle = 20, frac = 1/8, 
        split = FALSE, shrink = 1/4,
        pos = NULL,
        txt.args=list(),
        txt.shift=c(0,0),
        do.plot = TRUE)
}
\arguments{
  \item{x}{
    Object of class \code{"yardstick"} to be plotted.
    This object is created by the command \code{\link{yardstick}}.
  }
  \item{\dots}{
    Additional graphics arguments passed to
    \code{\link[graphics]{segments}} to control the appearance of the line.
  }
  \item{angle}{
    Angle between the arrows and the line segment, in degrees. 
  }
  \item{frac}{
    Length of arrow as a fraction of total length of the line segment.
  }
  \item{split}{
    Logical. If \code{TRUE}, then the line will be broken in the
    middle, and the text will be placed in this gap. If \code{FALSE},
    the line will be unbroken, and the text will be placed beside
    the line.
  }
  \item{shrink}{
    Fraction of total length to be removed from the middle of the
    line segment, if \code{split=TRUE}.
  }
  \item{pos}{
    Integer (passed to \code{\link[graphics]{text}}) determining the
    position of the annotation text relative to the line segment,
    if \code{split=FALSE}. Values of 1, 2, 3 and 4
    indicate positions below, to the left of,
    above and to the right of the line, respectively.
  }
  \item{txt.args}{
    Optional list of additional arguments passed to
    \code{\link[graphics]{text}} controlling the appearance of the text.
    Examples include \code{adj}, \code{srt}, \code{col}, \code{cex},
    \code{font}.
  }
  \item{txt.shift}{
    Optional numeric vector of length 2 specifying displacement
    of the text position relative to the centre of the yardstick.
  }
  \item{do.plot}{
    Logical. Whether to actually perform the plot (\code{do.plot=TRUE}).
  }
}
\details{
  A yardstick or scale bar is a line segment, drawn on any spatial
  graphics display, indicating the scale of the plot. 

  The argument \code{x} should be an object of class \code{"yardstick"}
  created by the command \code{\link{yardstick}}. 
}
\value{
  A window (class \code{"owin"}) enclosing the plotted graphics.
}
\examples{
  plot(owin(), main="Yardsticks")
  ys <- yardstick(as.psp(list(xmid=0.5, ymid=0.1, length=0.4, angle=0),
                         window=owin(c(0.2, 0.8), c(0, 0.2))),
                  txt="1 km")
  plot(ys)
  ys <- shift(ys, c(0, 0.3))
  plot(ys, angle=90, frac=0.08)
  ys <- shift(ys, c(0, 0.3))
  plot(ys, split=TRUE)
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link{yardstick}}
}
\keyword{spatial}
\keyword{hplot}

\name{bw.stoyan}
\alias{bw.stoyan}
\title{
  Stoyan's Rule of Thumb for Bandwidth Selection
}
\description{
  Computes a rough estimate of the appropriate bandwidth
  for kernel smoothing estimators of the pair correlation function
  and other quantities.
}
\usage{
bw.stoyan(X, co=0.15)
}

\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"}).
  }
  \item{co}{
    Coefficient appearing in the rule of thumb. See Details.
  }
}
\details{
  Estimation of the pair correlation function and other quantities
  by smoothing methods requires a choice of the smoothing bandwidth.
  Stoyan and Stoyan (1995, equation (15.16), page 285) proposed a
  rule of thumb for choosing the smoothing bandwidth.
  
  For the Epanechnikov kernel, the rule of thumb is to set
  the kernel's half-width \eqn{h} to
  \eqn{0.15/\sqrt{\lambda}}{0.15/sqrt(\lambda)} where
  \eqn{\lambda}{\lambda} is the estimated intensity of the point pattern,
  typically computed as the number of points of \code{X} divided by the
  area of the window containing \code{X}.

  For a general kernel, the corresponding rule is to set the
  standard deviation of the kernel to
  \eqn{\sigma = 0.15/\sqrt{5\lambda}}{\sigma = 0.15/sqrt(5 * \lambda)}.
  
  The coefficient \eqn{0.15} can be tweaked using the
  argument \code{co}.

  To ensure the bandwidth is finite, an empty point pattern is treated
  as if it contained 1 point.
}
\value{
  A finite positive numerical value giving the selected bandwidth (the standard
  deviation of the smoothing kernel).
}
\seealso{
  \code{\link{pcf}},
  \code{\link{bw.relrisk}}
}
\examples{
  data(shapley)
  bw.stoyan(shapley)
}
\references{
  Stoyan, D. and Stoyan, H. (1995)
  Fractals, random shapes and point fields:
  methods of geometrical statistics.
  John Wiley and Sons.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}
\keyword{smooth}

\name{solist}
\alias{solist}
\title{
  List of Two-Dimensional Spatial Objects
}
\description{
  Make a list of two-dimensional spatial objects.
}
\usage{
solist(\dots, check=TRUE, promote=TRUE, demote=FALSE, .NameBase)
}
\arguments{
  \item{\dots}{
    Any number of objects, each representing a two-dimensional
    spatial dataset.
  }
  \item{check}{
    Logical value. If \code{TRUE}, check that each of the
    objects is a 2D spatial object.
  }
  \item{promote}{
    Logical value. If \code{TRUE}, test whether all objects belong to
    the \emph{same} class, and if so, promote the list of objects
    to the appropriate class of list.
  }
  \item{demote}{
    Logical value determining what should happen if any of the
    objects is not a 2D spatial object: if \code{demote=FALSE} (the
    default), a fatal error occurs; if \code{demote=TRUE},
    a list of class \code{"anylist"} is returned.
  }
  \item{.NameBase}{
    Optional. Character string. If the \code{\dots} arguments have no
    names, then the entries of the resulting list will be given
    names that start with \code{.NameBase}.
  }
}
\details{
  This command creates an object of class \code{"solist"}
  (spatial object list)
  which represents a list of two-dimensional spatial datasets.
  The datasets do not necessarily belong to the same class.

  Typically the intention is that the datasets in the list
  should be treated in the same way, for example, they should
  be plotted side-by-side. The \pkg{spatstat} package
  provides a plotting function, \code{\link{plot.solist}},
  and many other functions for this class.
  
  In the \pkg{spatstat} package, various functions produce
  an object of class \code{"solist"}. For example, when
  a point pattern is split into several point patterns by
  \code{\link{split.ppp}}, or an image is split into several
  images by \code{\link{split.im}}, the result is of
  class \code{"solist"}.

  If \code{check=TRUE} then the code will check whether all
  objects in \code{\dots} belong to the classes
  of two-dimensional spatial objects defined in the
  \pkg{spatstat} package. They do not have to belong to the
  \emph{same} class. Set \code{check=FALSE}
  for efficiency, but only if you are sure that all the objects are valid.

  If some of the objects in \code{\dots} are
  not two-dimensional spatial objects,
  the action taken depends on the argument \code{demote}.
  If \code{demote=TRUE}, the result will belong to the more general
  class \code{"anylist"} instead of \code{"solist"}.
  If \code{demote=FALSE} (the default), an error occurs.

  If \code{promote=TRUE} then the code will check whether all
  the objects \code{\dots} belong to the same class.
  If they are all point patterns (class \code{"ppp"}),
  the result will also belong to the class \code{"ppplist"}.
  If they are all pixel images (class \code{"im"}), the result
  will also belong to the class \code{"imlist"}.

  Use \code{\link{as.solist}} to convert a list to a \code{"solist"}.
}
\value{
  A list, usually belonging to the class \code{"solist"}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{as.solist}},
  \code{\link{anylist}},
  \code{\link{solapply}}
}
\examples{
  solist(cells, density(cells))
  solist(cells, japanesepines, redwood, .NameBase="Pattern")
}
\keyword{spatial}
\keyword{list}
\keyword{manip}

\name{plot.quadratcount}
\alias{plot.quadratcount}
\title{
  Plot Quadrat Counts
}
\description{
  Given a table of quadrat counts for a spatial point pattern,
  plot the quadrats which were used, and display the quadrat count
  as text in the centre of each quadrat.
}
\usage{
\method{plot}{quadratcount}(x, \dots, add = FALSE,
              entries = as.vector(t(as.table(x))),
              dx = 0, dy = 0, show.tiles = TRUE,
              textargs = list())
}
\arguments{
  \item{x}{
    Object of class \code{"quadratcount"}
    produced by the function \code{\link{quadratcount}}.
  }
  \item{\dots}{
    Additional arguments passed to \code{\link{plot.tess}}
    to plot the quadrats.
  }
  \item{add}{
    Logical. Whether to add the graphics to an existing plot.
  }
  \item{entries}{
    Vector of numbers to be plotted in each quadrat.
    The default is to plot the quadrat counts.
  }
  \item{dx,dy}{
    Horizontal and vertical displacement of text
    relative to centroid of quadrat.
  }
  \item{show.tiles}{
    Logical value indicating whether to plot the quadrats.
  }
  \item{textargs}{
    List containing extra arguments
    passed to \code{\link[graphics]{text.default}}
    to control the annotation.
  }
}
\details{
  This is the plot method for the objects 
  of class \code{"quadratcount"} that are
  produced by the function \code{\link{quadratcount}}.
  Given a spatial point pattern, \code{\link{quadratcount}}
  divides the observation window into disjoint tiles or quadrats,
  counts the number of points in each quadrat, and stores the
  result as a contingency table which also belongs to the
  class \code{"quadratcount"}.

  First the quadrats are plotted
  (provided \code{show.tiles=TRUE}, the default).
  This display can be controlled by passing additional arguments \code{\dots}
  to \code{\link{plot.tess}}.
  
  Then the quadrat counts are printed using
  \code{\link[graphics]{text.default}}. This display can be controlled
  using the arguments \code{dx,dy} and \code{textargs}.
}
\value{
  Null.
}
\seealso{
  \code{\link{quadratcount}},
  \code{\link{plot.tess}},
  \code{\link[graphics]{text.default}},
  \code{\link{plot.quadrattest}}.
}
\examples{
   plot(quadratcount(swedishpines, 5))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{hplot}

\name{colourmap}
\alias{colourmap}
\title{Colour Lookup Tables}
\description{
  Create a colour map (colour lookup table).
}
\usage{
colourmap(col, \dots, range=NULL, breaks=NULL, inputs=NULL, gamma=1)
}
\arguments{
  \item{col}{Vector of values specifying colours}
  \item{\dots}{Ignored.}
  \item{range}{
    Interval to be mapped.
    A numeric vector of length 2, specifying the endpoints of the
    range of values to be mapped.
    Incompatible with \code{breaks} or \code{inputs}.
  }
  \item{inputs}{
    Values to which the colours are associated.
    A factor or vector of the same length as \code{col}.
    Incompatible with \code{breaks} or \code{range}.
  }
  \item{breaks}{
    Breakpoints for the colour map.
    A numeric vector of length equal to \code{length(col)+1}.
    Incompatible with \code{range} or \code{inputs}.
  }
  \item{gamma}{
    Exponent for the gamma correction, when \code{range} is given.
    A single positive number.
    See Details.
  }
}
\details{
  A colour map is a mechanism for associating colours with data.
  It can be regarded as a function, mapping data to colours.

  The command \code{colourmap} creates an object representing
  a colour map, which can then be used to control the plot commands
  in the \pkg{spatstat} package. It can also be used to compute the
  colour assigned to any data value. 

  The argument \code{col} specifies the colours to which
  data values will be mapped. It should be a vector
  whose entries can be interpreted as colours by the standard
  \R graphics system. The entries can be string names of colours
  like \code{"red"}, or integers that refer to
  colours in the standard palette, or strings containing
  six-letter hexadecimal codes like \code{"#F0A0FF"}. 

  Exactly one of the arguments \code{range}, \code{inputs} or \code{breaks}
  must be specified by name.

  \itemize{
    \item
    If \code{inputs} is given, then it should be a vector or factor,
    of the same length as \code{col}. The entries of \code{inputs} can be
    any atomic type (e.g. numeric, logical, character, complex) or factor
    values. The resulting colour map associates the value \code{inputs[i]}
    with the colour \code{col[i]}.
    The argument \code{col} should have the same length as \code{inputs}.
    \item
    If \code{range} is given, then it determines the interval of the real
    number line that will be mapped. It should be a numeric vector of
    length 2. The interval will be divided evenly into bands, each of which is
    assigned one of the colours in \code{col}. (If \code{gamma} is given,
    then the bands are equally spaced on a scale where the original values
    are raised to the power \code{gamma}.)
    \item
    If \code{breaks} is given, then it determines the precise intervals
    of the real number line
    which are mapped to each colour. It should be a numeric vector,
    of length at least 2, with entries that are in increasing order.
    Infinite values are allowed. Any number in the range
    between \code{breaks[i]} and \code{breaks[i+1]} will be mapped to the
    colour \code{col[i]}. 
    The argument \code{col} should have length equal to
    \code{length(breaks) - 1}.
  }
  It is also permissible for \code{col} to be a single colour value,
  representing a trivial colour map in which all data values are mapped to
  the same colour.
  
  The result is an object of class \code{"colourmap"}. 
  There are \code{print} and \code{plot} methods for this class.
  Some plot commands in the \pkg{spatstat} package accept an object
  of this class as a specification of the colour map.

  The result is also a function \code{f} which can be used to compute
  the colour assigned to any data value. 
  That is, \code{f(x)} returns the character value of the colour assigned
  to \code{x}. This also works for vectors of data values.
}
\value{
  A function, which is also an object of class \code{"colourmap"}.
}
\seealso{
  The plot method \code{\link{plot.colourmap}}.
  
  See the \R help file on
  \code{\link[grDevices:colors]{colours}} for information about the colours
  that \R recognises, and how to manipulate them.
  
  To make a smooth transition between colours, see
  \code{\link{interp.colourmap}}.
  To alter individual colour values, see
  \code{\link{tweak.colourmap}}.
  To extract or replace all colour values, see
  \code{\link{colouroutputs}}.
    
  See \code{\link[spatstat:colourtools]{colourtools}}
  for more tools to manipulate colour values.

  See \code{\link{lut}} for lookup tables.
}
\examples{
  # colour map for real numbers, using breakpoints
  cr <- colourmap(c("red", "blue", "green"), breaks=c(0,5,10,15))
  cr
  cr(3.2)
  cr(c(3,5,7))
  # a large colour map
  co <- colourmap(rainbow(100), range=c(-1,1))
  co(0.2)
  # colour map for discrete set of values
  ct <- colourmap(c("red", "green"), inputs=c(FALSE, TRUE))
  ct(TRUE)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{color}

\name{whist}
\alias{whist}
\title{
  Weighted Histogram
}
\description{
  Computes the weighted histogram of a set of observations
  with a given set of weights.
}
\usage{
whist(x, breaks, weights = NULL)
}
\arguments{
  \item{x}{
    Numeric vector of observed values.
  }
  \item{breaks}{
    Vector of breakpoints for the histogram. 
  }
  \item{weights}{
    Numeric vector of weights for the observed values. 
  }
}
\details{
  This low-level function computes (but does not plot) the weighted
  histogram of a vector of observations \code{x} using a given
  vector of \code{weights}.

  The arguments \code{x} and \code{weights} should be numeric vectors of
  equal length. They may include \code{NA} or infinite values.

  The argument \code{breaks} should be a numeric vector whose entries
  are strictly increasing. These values define the boundaries between the
  successive histogram cells.
  The breaks \emph{do not} have to span the range
  of the observations.

  There are \code{N-1} histogram cells, where \code{N = length(breaks)}.
  An observation \code{x[i]} falls in the \code{j}th cell if
  \code{breaks[j] <= x[i] < breaks[j+1]} (for \code{j < N-1})
  or
  \code{breaks[j] <= x[i] <= breaks[j+1]} (for \code{j = N-1}).
  The weighted histogram value \code{h[j]} for the \code{j}th cell is
  the sum of \code{weights[i]} for all observations \code{x[i]} that
  fall in the cell.
  
  Note that, in contrast to the function \code{\link{hist}},
  the function \code{whist} does not require the breakpoints to span the
  range of the observations \code{x}. Values of \code{x} that fall outside the
  range of \code{breaks} are handled separately; their total weight
  is returned as an attribute of the histogram. 
}
\value{
  A numeric vector of length \code{N-1} containing the
  histogram values, where \code{N = length(breaks)}.

  The return value also has attributes \code{"low"} and \code{"high"}
  giving the total weight of all observations that are less than
  the lowest breakpoint, or greater than the highest breakpoint,
  respectively.
}
\examples{
  x <- rnorm(100)
  b <- seq(-1,1,length=21)
  w <- runif(100)
  whist(x,b,w)
}
\author{\adrian
  
  
  and \rolf
  
  with thanks to Peter Dalgaard.
}
\keyword{arith}

\name{pairs.im}
\alias{pairs.im}
\title{
  Scatterplot Matrix for Pixel Images
}
\description{
  Produces a scatterplot matrix of the pixel values
  in two or more pixel images.
}
\usage{
\method{pairs}{im}(..., plot=TRUE)
}
\arguments{
  \item{\dots}{
    Any number of arguments, each of which is either
    a pixel image (object of class \code{"im"})
    or a named argument to be passed to \code{\link{pairs.default}}.
  }
  \item{plot}{
    Logical. If \code{TRUE}, the scatterplot matrix is plotted.
  }
}
\details{
  This is a method for the generic function \code{\link{pairs}}
  for the class of pixel images.
  
  It produces a square array of plot panels, in which each panel shows
  a scatterplot of the pixel values of one image against the
  corresponding pixel values of another image.

  At least two of the arguments \code{\dots}
  should be pixel images (objects of class \code{"im"}).
  Their spatial domains must overlap, but need not have the same
  pixel dimensions.

  First the pixel image domains are intersected, and converted to a
  common pixel resolution. Then the corresponding pixel values of each image are
  extracted. Then \code{\link{pairs.default}} is called to 
  plot the scatterplot matrix.

  Any arguments in \code{\dots} which are not pixel images will be
  passed to \code{\link{pairs.default}} to control the plot.
}
\section{Image or Contour Plots}{
  Since the scatterplots may show very dense concentrations of points,
  it may be useful to set \code{panel=panel.image}
  or \code{panel=panel.contour} to draw a colour image or contour plot
  of the kernel-smoothed density of the scatterplot in each panel.
  The argument \code{panel} is passed
  to \code{\link{pairs.default}}. See the help for 
  \code{\link{panel.image}} and \code{\link{panel.contour}}.
}
\section{Low Level Control of Graphics}{
  To control the appearance of the individual scatterplot panels,
  see \code{\link{pairs.default}}, \code{\link{points}}
  or \code{\link{par}}.
  To control the plotting symbol for the points in the scatterplot,
  use the arguments \code{pch}, \code{col}, \code{bg} as described
  under \code{\link{points}} (because 
  the default panel plotter is the function \code{\link{points}}).
  To suppress the tick marks on the plot axes,
  type \code{par(xaxt="n", yaxt="n")} before calling \code{pairs}.
}
\value{
  Invisible. A \code{data.frame} containing the
  corresponding pixel values for each image.
  The return value also belongs to the class \code{plotpairsim} which has
  a plot method, so that it can be re-plotted.
}
\seealso{
  \code{\link{pairs}},
  \code{\link{pairs.default}},
  \code{\link{panel.contour}},
  \code{\link{panel.image}},
  \code{\link{plot.im}},
  \code{\link{im}},
  \code{\link{par}}
}
\examples{
  X <- density(rpoispp(30))
  Y <- density(rpoispp(40))
  Z <- density(rpoispp(30))
  p <- pairs(X,Y,Z)
  p
  plot(p)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{hplot}

\name{identify.psp}
\alias{identify.psp}
\title{Identify Segments in a Line Segment Pattern}
\description{
  If a line segment pattern is plotted in the graphics window,
  this function will find the segment which is nearest to
  the mouse position, and print its serial number.
}
\usage{
  \method{identify}{psp}(x, \dots, labels=seq_len(nsegments(x)), n=nsegments(x), plot=TRUE)
}
\arguments{
  \item{x}{
    A line segment pattern (object of class \code{"psp"}).
  }
  \item{labels}{
    Labels associated with the segments, to be plotted when the
    segments are identified. A character vector or numeric vector
    of length equal to the number of segments in \code{x}.
  }
  \item{n}{
    Maximum number of segments to be identified.
  }
  \item{plot}{
    Logical. Whether to plot the labels when a segment is identified.
  }
  \item{\dots}{
    Arguments passed to \code{\link[graphics]{text.default}}
    controlling the plotting of the labels.
  }
}
\value{
  Vector containing the serial numbers of the segments in the pattern
  \code{x} that were identified.
}
\details{
  This is a method for the generic function \code{\link[graphics]{identify}}
  for line segment pattern objects.

  The line segment pattern \code{x} should first be plotted
  using \code{\link{plot.psp}}. Then \code{identify(x)}
  reads the position of the graphics pointer each time the
  left mouse button is pressed.  It then finds 
  the segment in the pattern \code{x} that is closest to the mouse position.
  This segment's index will be returned as part of the value of the call.

  Each time a segment is identified,
  text will be displayed next to the point,
  showing its serial number (or the relevant entry of \code{labels}).
}
\seealso{
  \code{\link[graphics]{identify}},
  \code{\link{identify.ppp}}.
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{iplot}

\name{nncross.ppx}
\alias{nncross.ppx}
\title{Nearest Neighbours Between Two Patterns in Any Dimensions}
\description{
  Given two point patterns \code{X} and \code{Y} in
  many dimensional space,
  finds the nearest neighbour in \code{Y} of each point of \code{X}.
}
\usage{
  \method{nncross}{ppx}(X, Y,
          iX=NULL, iY=NULL,
          what = c("dist", "which"),
          \dots,
          k = 1)
}
\arguments{
  \item{X,Y}{
    Point patterns in any number of spatial dimensions
    (objects of class \code{"ppx"}).
  }
  \item{iX, iY}{Optional identifiers, 
    used to determine whether a point in
    \code{X} is identical to a point in \code{Y}. See Details.
  }
  \item{what}{
    Character string specifying what information should be returned.
    Either the nearest neighbour distance (\code{"dist"}),
    the identifier of the nearest neighbour (\code{"which"}),
    or both.
  }
  \item{k}{
    Integer, or integer vector. The algorithm will compute the distance to the
    \code{k}th nearest neighbour.
  }
  \item{\dots}{Ignored.}
}
\details{
  Given two point patterns \code{X} and \code{Y} in
  \eqn{m}-dimensional space,
  this function finds, for each point of \code{X}, 
  the nearest point of \code{Y}. The distance between these points
  is also computed.
  If the argument \code{k} is specified, then the \code{k}-th nearest
  neighbours will be found.

  The return value is a data frame, with rows corresponding to
  the points of \code{X}.  The first column gives the nearest neighbour
  distances (i.e. the \code{i}th entry is the distance 
  from the \code{i}th point of \code{X} to the nearest element of
  \code{Y}). The second column gives the indices of the nearest
  neighbours (i.e.\ the \code{i}th entry is the index of
  the nearest element in \code{Y}.)
  If \code{what="dist"} then only the vector of distances is returned.
  If \code{what="which"} then only the vector of indices is returned.

  The argument \code{k} may be an integer or an integer vector.
  If it is a single integer, then the \code{k}-th nearest neighbours
  are computed. If it is a vector, then the \code{k[i]}-th nearest
  neighbours are computed for each entry \code{k[i]}. For example, setting
  \code{k=1:3} will compute the nearest, second-nearest and
  third-nearest neighbours. The result is a data frame.

  Note that this function is not symmetric in \code{X} and \code{Y}.
  To find the nearest neighbour in \code{X} of each point in \code{Y},
  use \code{nncross(Y,X)}.

  The arguments \code{iX} and \code{iY} are used when
  the two point patterns \code{X} and \code{Y} have some points in
  common.  In this situation \code{nncross(X, Y)} would return some zero
  distances. To avoid this, attach a unique integer identifier to
  each point, such that two points are identical if their
  identifying numbers are equal. Let \code{iX} be the vector of
  identifier values for the points in \code{X}, and \code{iY}
  the vector of identifiers for points in \code{Y}. Then the code
  will only compare two points if they have different values of the
  identifier. See the Examples.
}
\value{
  A data frame, or a vector if the data frame would contain only one column.
  
  By default (if \code{what=c("dist", "which")} and \code{k=1})
  a data frame with two columns:
  \item{dist}{Nearest neighbour distance}
  \item{which}{Nearest neighbour index in \code{Y}}

  If \code{what="dist"} and \code{k=1}, a vector of nearest neighbour distances.

  If \code{what="which"} and \code{k=1}, a vector of nearest neighbour
  indices.

  If \code{k} is specified, the result is a data frame with
  columns containing the \code{k}-th nearest neighbour distances
  and/or nearest neighbour indices.
}
\seealso{
  \code{\link{nndist}} for nearest neighbour
  distances in a single point pattern.
}
\examples{
  B <- boxx(c(0,1), c(0,1), c(0,1), c(0,1))
  ## two different point patterns
  X <- runifpointx(5, B)
  Y <- runifpointx(10, B)
  nncross(X,Y)
  N23 <- nncross(X,Y, k=2:3)

  ## two patterns with some points in common
  Z <- runifpointx(20, B)
  X <- Z[1:15]
  Y <- Z[10:20]
  iX <- 1:15
  iY <- 10:20
  N <- nncross(X,Y, iX, iY, what="which")
  N4 <- nncross(X,Y, iX, iY, k=4)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{predict.lppm}
\alias{predict.lppm}
\title{
  Predict Point Process Model on Linear Network
}
\description{
  Given a fitted point process model on a linear network,
  compute the fitted intensity or conditional intensity of the model.
}
\usage{
\method{predict}{lppm}(object, ...,
                       type = "trend", locations = NULL, new.coef=NULL)
}
\arguments{
  \item{object}{
    The fitted model. An object of class \code{"lppm"},
    see \code{\link{lppm}}.
  }
  \item{type}{
    Type of values to be computed. Either \code{"trend"},
    \code{"cif"} or \code{"se"}.
  }
  \item{locations}{
    Optional. Locations at which predictions should be computed.
    Either a data frame with two columns of coordinates,
    or a binary image mask.
  }
  \item{new.coef}{
    Optional. Numeric vector of model coefficients,
    to be used instead of the fitted coefficients
    \code{coef(object)} when calculating
    the prediction.
  }
  \item{\dots}{
    Optional arguments passed to \code{\link{as.mask}}
    to determine the
    pixel resolution (if \code{locations} is missing).
  }
}
\details{
  This function computes the fitted poin process intensity,
  fitted conditional intensity, or standard error of the fitted
  intensity, for a point process model on a linear network.
  It is a method for the generic \code{\link[stats]{predict}}
  for the class \code{"lppm"}.

  The argument \code{object} should be an object of class \code{"lppm"}
  (produced by \code{\link{lppm}}) representing a point process model
  on a linear network.

  Predicted values are computed at the locations given by the
  argument \code{locations}. If this argument is missing,
  then predicted values are computed at a fine grid of points
  on the linear network.

  \itemize{
    \item 
    If \code{locations} is missing or \code{NULL} (the default),
    the return value is a pixel image (object of class \code{"linim"}
    which inherits class \code{"im"})
    corresponding to a discretisation
    of the linear network, with numeric pixel values giving the
    predicted values at each location on the linear network.
    \item
    If \code{locations} is a data frame, the result is a 
    numeric vector of predicted values at the locations specified by
    the data frame.
    \item
    If \code{locations} is a binary mask, the result is a pixel image
    with predicted values computed at the pixels of the mask.
  }
}
\value{
  A pixel image (object of class \code{"linim"} which inherits
  class \code{"im"}) or
  a numeric vector, depending on the argument \code{locations}.
  See Details.
}
\author{
  \adrian
}
\seealso{
  \code{\link{lpp}},
  \code{\link{linim}}
}
\examples{
  X <- runiflpp(12, simplenet)
  fit <- lppm(X ~ x)
  v <- predict(fit, type="trend")
  plot(v)
}
\references{
  Ang, Q.W. (2010)
  \emph{Statistical methodology for events on a network}.
  Master's thesis, School of Mathematics and Statistics, University of
  Western Australia.
  
  Ang, Q.W., Baddeley, A. and Nair, G. (2012)
  Geometrically corrected second-order analysis of 
  events on a linear network, with applications to
  ecology and criminology.
  \emph{Scandinavian Journal of Statistics} \bold{39}, 591--617.

  McSwiggan, G., Nair, M.G. and Baddeley, A. (2012)
  Fitting Poisson point process models to events 
  on a linear network. Manuscript in preparation.
}
\keyword{spatial}
\keyword{models}

\name{shift.im}
\alias{shift.im}
\title{Apply Vector Translation To Pixel Image}
\description{
  Applies a vector shift to a pixel image
}
\usage{
 \method{shift}{im}(X, vec=c(0,0), \dots, origin=NULL)
}
\arguments{
  \item{X}{Pixel image (object of class \code{"im"}).}
  \item{vec}{Vector of length 2 representing a translation.}
  \item{\dots}{Ignored}
  \item{origin}{
    Location that will be shifted to the origin.
    Either a numeric vector of length 2 giving the location,
    or a point pattern containing only one point,
    or a list with two entries named \code{x} and \code{y},
    or one of the character strings
    \code{"centroid"}, \code{"midpoint"},
    \code{"left"}, \code{"right"}, \code{"top"}, \code{"bottom"},
    \code{"topleft"}, \code{"bottomleft"}, \code{"topright"} or
    \code{"bottomright"} (partially matched).
  }
}
\value{
  Another pixel image (of class \code{"im"}) representing the
  result of applying the vector shift.
}
\details{
  The spatial location of each pixel in the image
  is translated by the vector \code{vec}.
  This is a method for the generic function \code{\link{shift}}.

  If \code{origin} is given,
  the argument \code{vec} will be ignored; instead the shift will be performed
  so that the specified geometric location is shifted to the
  coordinate origin \eqn{(0,0)}.
  The argument \code{origin} should be either a numeric vector of length
  2 giving the spatial coordinates of a location, or one of the character
  strings \code{"centroid"}, \code{"midpoint"},
  \code{"left"}, \code{"right"}, \code{"top"}, \code{"bottom"},
  \code{"topleft"}, \code{"bottomleft"}, \code{"topright"} or
  \code{"bottomright"} (partially matched).
  If \code{origin="centroid"} then the centroid of the window will be
  shifted to the origin. If \code{origin="midpoint"} then the centre of
  the bounding rectangle of the window will be shifted to the origin.
  If \code{origin="bottomleft"} then the bottom left corner of the
  bounding rectangle of the window will be shifted to the origin,
  and so on.
}
\seealso{
  \code{\link{shift}}
}
\examples{
 # make up an image
 X <- setcov(unit.square())
 plot(X)

 Y <- shift(X, c(10,10))
 plot(Y)
 # no discernible difference except coordinates are different

 shift(X, origin="c")
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{transect.im}
\alias{transect.im}
\title{
  Pixel Values Along a Transect
}
\description{
  Extract the pixel values of a pixel image at each point along
  a linear transect.
}
\usage{
transect.im(X, ..., from="bottomleft", to="topright",
            nsample=512, click=FALSE, add=FALSE)
}
\arguments{
  \item{X}{
    A pixel image (object of class \code{"im"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{from,to}{
    Optional. Start point and end point of the transect.
    Pairs of \eqn{(x,y)} coordinates
    in a format acceptable to \code{\link{xy.coords}},
    or keywords \code{"bottom"}, \code{"left"}, \code{"top"},
    \code{"right"}, \code{"bottomleft"} etc.
  }
  \item{nsample}{
    Integer. Number of sample locations along the transect.
  }
  \item{click}{
    Optional.
    Logical value.
    If \code{TRUE}, the linear transect is determined interactively
    by the user, who clicks two points on the current plot.
  }
  \item{add}{
    Logical. If \code{click=TRUE}, this argument determines
    whether to perform interactive tasks on the current plot (\code{add=TRUE})
    or to start by plotting \code{X} (\code{add=FALSE}).
  }
}
\details{
  The pixel values of the image \code{X} along a line segment 
  will be extracted. The result is a function table (\code{"fv"} object)
  which can be plotted directly.

  If \code{click=TRUE}, then the user is prompted to click two points on
  the plot of \code{X}. These endpoints define the transect.

  Otherwise, the transect is defined by the endpoints
  \code{from} and \code{to}. The default is a diagonal transect from
  bottom left to top right of the frame.
}
\value{
  An object of class \code{"fv"} which can be plotted.
}
\author{
  \adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{im}}
}
\examples{
  Z <- density(redwood)
  plot(transect.im(Z))
  \dontrun{
   if(FALSE) {
    plot(transect.im(Z, click=TRUE))
   }
  }
}
\keyword{spatial}
\keyword{manip}
\keyword{iplot}

\name{chop.tess}
\alias{chop.tess}
\title{Subdivide a Window or Tessellation using a Set of Lines}
\description{
  Divide a given window into tiles
  delineated by a set of infinite straight lines, obtaining
  a tessellation of the window.
  Alternatively, given a tessellation, divide each tile of the
  tessellation into sub-tiles delineated by the lines.
}
\usage{
chop.tess(X, L)
}
\arguments{
  \item{X}{
    A window (object of class \code{"owin"}) or tessellation
    (object of class \code{"tess"}) to be subdivided by lines.
  }
  \item{L}{
    A set of infinite straight lines (object of class \code{"infline"})
  }
}
\details{
  The argument \code{L} should be a set of infinite straight lines in the plane
  (stored in an object \code{L} of class \code{"infline"} created by the
  function \code{\link{infline}}).
  
  If \code{X} is a window, then it is divided into tiles
  delineated by the lines in \code{L}.

  If \code{X} is a tessellation, then each tile of \code{X} is
  subdivided into sub-tiles delineated by the lines in \code{L}.

  The result is a tessellation. 
}
\section{Warning}{
  If \code{X} is a non-convex window, or a tessellation containing
  non-convex tiles, then \code{chop.tess(X,L)} may contain a tile
  which consists of several unconnected pieces. 
}
\value{
  A tessellation (object of class \code{"tess"}).
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{infline}},
  \code{\link{clip.infline}}
}
\examples{
  L <- infline(p=1:3, theta=pi/4)
  W <- square(4)
  chop.tess(W, L)
}
\keyword{spatial}
\keyword{math}

\name{whichhalfplane}
\alias{whichhalfplane}
\title{
  Test Which Side of Infinite Line a Point Falls On
}
\description{
  Given an infinite line
  and a spatial point location,
  determine which side of the line the point falls on.
}
\usage{
whichhalfplane(L, x, y = NULL)
}
\arguments{
  \item{L}{
    Object of class \code{"infline"} specifying one or more
    infinite straight lines in two dimensions.
  }
  \item{x,y}{
    Arguments acceptable to \code{\link[grDevices]{xy.coords}}
    specifying the locations of the points.
  }
}
\details{
  An infinite line \eqn{L} divides the two-dimensional plane into
  two half-planes. This function returns a matrix \code{M} of logical values
  in which \code{M[i,j] = TRUE} if the \code{j}th spatial point
  lies below or to the left of the \code{i}th line.
}
\value{
  A logical matrix.
}
\author{
  \adrian.
}
\seealso{
  \code{\link{infline}}
}
\examples{
  L <- infline(p=runif(3), theta=runif(3, max=2*pi))
  X <- runifpoint(4)
  whichhalfplane(L, X)
}
\keyword{spatial}
\keyword{manip}

\name{as.data.frame.hyperframe}
\alias{as.data.frame.hyperframe}
\title{Coerce Hyperframe to Data Frame}
\description{
  Converts a hyperframe to a data frame.
}
\usage{
\method{as.data.frame}{hyperframe}(x, row.names = NULL,
                                  optional = FALSE, ..., 
                                  discard=TRUE, warn=TRUE)
}
\arguments{
  \item{x}{Hyperframe (object of class \code{"hyperframe"}).}
  \item{row.names}{Optional character vector of row names.}
  \item{optional}{Argument passed to \code{\link{as.data.frame}}
    controlling what happens to row names.}
  \item{\dots}{Ignored.}
  \item{discard}{Logical. Whether to discard columns of the hyperframe
    that do not contain atomic data. See Details.
  }
  \item{warn}{Logical. Whether to issue a warning when columns are discarded.}
}
\details{
  This is a method for the generic function \code{\link{as.data.frame}}
  for the class of hyperframes (see \code{\link{hyperframe}}.

  If \code{discard=TRUE}, any columns of the hyperframe that
  do not contain atomic data will be removed (and a warning will
  be issued if \code{warn=TRUE}). 
  If \code{discard=FALSE}, then such columns are converted to
  strings indicating what class of data
  they originally contained.
}
\value{
  A data frame.
}
\examples{
  h <- hyperframe(X=1:3, Y=letters[1:3], f=list(sin, cos, tan))
  as.data.frame(h, discard=TRUE, warn=FALSE)
  as.data.frame(h, discard=FALSE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{contour.im}
\alias{contour.im}
\title{Contour plot of pixel image}
\description{
  Generates a contour plot of a pixel image.
}
\usage{
   \method{contour}{im}(x, \dots, main,
            axes=FALSE, add=FALSE, col=par("fg"), 
            clipwin=NULL, show.all=!add, do.plot=TRUE)
}
\arguments{
  \item{x}{
    Pixel image to be plotted.
    An object of class \code{"im"}.
  }
  \item{main}{
    Character string to be displayed as the main title.
  }
  \item{axes}{
    Logical. If \code{TRUE}, coordinate axes are plotted
    (with tick marks) around a region slightly larger than the image window.
    If \code{FALSE} (the default), no axes are plotted,
    and a box is drawn tightly around the image window.
    Ignored if \code{add=TRUE}.
  }
  \item{add}{
    Logical. If \code{FALSE}, a new plot is created. If \code{TRUE},
    the contours are drawn over the existing plot.
  }
  \item{col}{
    Colour in which to draw the contour lines.
    Either a single value that can be interpreted as a colour value,
    or a \code{colourmap} object.
  }
  \item{clipwin}{
    Optional. A window (object of class \code{"owin"}).
    Only this subset of the data will be displayed.
  }
  \item{\dots}{
    Other arguments passed to \code{\link{contour.default}}
    controlling the contour plot; see Details.
  }
  \item{show.all}{
    Logical value indicating whether to display all plot elements
    including the main title, bounding box, and (if
    \code{axis=TRUE}) coordinate axis markings.
    Default is \code{TRUE} for new plots and \code{FALSE} for added plots.
  }
  \item{do.plot}{
    Logical value indicating whether to actually perform the plot.
  }
}
\details{
  This is a method for the generic \code{contour} function,
  for objects of the class \code{"im"}.
  
  An object of class \code{"im"} represents a pixel image;
  see \code{\link{im.object}}.

  This function displays the values of the pixel image \code{x}
  as a contour plot on the current plot device,
  using equal scales on the \eqn{x} and \eqn{y} axes.

  The appearance of the plot can be modified
  using any of the arguments listed in the help for
  \code{\link{contour.default}}.
  Useful ones include:
  \describe{
    \item{nlevels}{
      Number of contour levels to plot.
    }
    \item{drawlabels}{
      Whether to label the contour lines with text.
    }
    \item{col,lty,lwd}{
      Colour, type, and width of contour lines.
    }
  }
  See \code{\link{contour.default}} for a full list of these arguments.

  The defaults for any of the abovementioned arguments
  can be reset using \code{\link{spatstat.options}("par.contour")}.

  If \code{col} is a colour map (object of class \code{"colourmap"},
  see \code{\link{colourmap}}) then the contours will be plotted in
  different colours as determined by the colour map.
  The contour at level \code{z} will be plotted
  in the colour \code{col(z)} associated with this level in the colour map.
}
\value{
  none.
}
\examples{
   # an image
   Z <- setcov(owin())
   contour(Z, axes=TRUE)
   contour(Z)

   co <- colourmap(rainbow(100), range=c(0,1))
   contour(Z, col=co, lwd=2)
}
\seealso{
  \code{\link{im.object}},
  \code{\link{plot.im}},
  \code{\link{persp.im}}
}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{hplot}

\name{periodify} 
\alias{periodify}
\alias{periodify.owin}
\alias{periodify.ppp}
\alias{periodify.psp}
\title{
  Make Periodic Copies of a Spatial Pattern  
}
\description{
  Given a spatial pattern (point pattern, line segment pattern,
  window, etc) make shifted copies of the pattern
  and optionally combine them to make a periodic pattern.
}
\usage{
periodify(X, ...)
\method{periodify}{ppp}(X, nx = 1, ny = 1, ...,
           combine=TRUE, warn=TRUE, check=TRUE,
           ix=(-nx):nx, iy=(-ny):ny,
           ixy=expand.grid(ix=ix,iy=iy))
\method{periodify}{psp}(X, nx = 1, ny = 1, ..., 
           combine=TRUE, warn=TRUE, check=TRUE,
           ix=(-nx):nx, iy=(-ny):ny, 
           ixy=expand.grid(ix=ix,iy=iy))
\method{periodify}{owin}(X, nx = 1, ny = 1, ...,
           combine=TRUE, warn=TRUE, 
           ix=(-nx):nx, iy=(-ny):ny,
           ixy=expand.grid(ix=ix,iy=iy))
}
\arguments{
  \item{X}{
    An object representing a spatial pattern
    (point pattern, line segment pattern or window).
  }
  \item{nx,ny}{
    Integers.
    Numbers of additional copies of \code{X} in each direction.
    The result will be a grid of \code{2 * nx + 1} by \code{2 * ny + 1}
    copies of the original object.
    (Overruled by \code{ix, iy, ixy}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{combine}{
    Logical flag determining whether the copies should be superimposed
    to make an object like \code{X} (if \code{combine=TRUE}) or
    simply returned as a list of objects (\code{combine=FALSE}).
  }
  \item{warn}{
    Logical flag determining whether to issue warnings.
  }
  \item{check}{
    Logical flag determining whether to check the validity of the
    combined pattern.
  }
  \item{ix, iy}{
    Integer vectors determining the grid positions of the copies
    of \code{X}. (Overruled by \code{ixy}).
  }
  \item{ixy}{
    Matrix or data frame with two columns, giving the 
    grid positions of the copies of \code{X}. 
  }
}
\details{
  Given a spatial pattern (point pattern, line segment pattern, etc)
  this function makes a number of shifted copies of the pattern
  and optionally combines them. The function \code{periodify} is
  generic, with methods for various kinds of spatial objects.

  The default is to make a 3 by 3 array of copies of \code{X} and
  combine them into a single pattern of the same kind as \code{X}.
  This can be used (for example) to compute toroidal or periodic
  edge corrections for various operations on \code{X}.

  If the arguments \code{nx}, \code{ny} are given
  and other arguments are missing,
  the original object will be copied \code{nx} times to the right
  and \code{nx} times to the left, then \code{ny} times upward and
  \code{ny} times downward, making \code{(2 * nx + 1) * (2 * ny + 1)} 
  copies altogether, arranged in a grid, centred on the original object.
  
  If the arguments \code{ix}, \code{iy} or \code{ixy} are specified,
  then these determine the grid positions of the copies of \code{X}
  that will be made. For example \code{(ix,iy) = (1, 2)} means a
  copy of \code{X} shifted by the vector \code{(ix * w, iy * h)} where
  \code{w,h} are the width and height of the bounding rectangle of \code{X}.

  If \code{combine=TRUE} (the default) the copies of \code{X} are
  superimposed to create an object of the same kind as \code{X}.
  If \code{combine=FALSE} the copies of \code{X} are returned as a list.
}
\value{
  If \code{combine=TRUE}, an object of the same class as \code{X}.
  If \code{combine=FALSE}, a list of objects of the same class as \code{X}.
}
\seealso{
  \code{\link{shift}}
}
\examples{
  data(cells)
  plot(periodify(cells))
  a <- lapply(periodify(Window(cells), combine=FALSE),
        plot, add=TRUE,lty=2)

}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{as.data.frame.psp}
\alias{as.data.frame.psp}
\title{Coerce Line Segment Pattern to a Data Frame}
\description{
  Extracts the coordinates of the endpoints in a line segment pattern,
  and their marks if any, and returns them in a data frame.
}
\usage{
\method{as.data.frame}{psp}(x, row.names = NULL, ...)
}
\arguments{
  \item{x}{Line segment pattern (object of class \code{"psp"}).}
  \item{row.names}{Optional character vector of row names.}
  \item{\dots}{Ignored.}
}
\details{
  This is a method for the generic function \code{\link{as.data.frame}}
  for the class \code{"psp"} of line segment patterns.

  It extracts the coordinates of the endpoints of the line segments,
  and returns them as columns named \code{x0}, \code{y0}, \code{x1}
  and \code{y1} in a data frame. If the line segments were marked,
  the marks are appended as an extra column or columns to the
  data frame which is returned.  If the marks are a vector then a
  single column named \code{marks} is appended. in the data frame,
  with the same type as in the line segment pattern dataset.  If the
  marks are a data frame, then the columns of this data frame are
  appended (retaining their names).

}
\value{
  A data frame with 4 or 5 columns.
}
\examples{
  data(copper)
  df <- as.data.frame(copper$Lines)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{levelset}
\alias{levelset}
\title{Level Set of a Pixel Image}
\description{
  Given a pixel image, find all pixels which have values less than a
  specified threshold value (or greater than a threshold, etc),
  and assemble these pixels into a window.
}
\usage{
  levelset(X, thresh, compare="<=")
}
\arguments{
  \item{X}{A pixel image (object of class "im")}.
  \item{thresh}{Threshold value.
    A single number or value compatible with the pixel values in
    \code{X}}.
  \item{compare}{Character string specifying one of the comparison
    operators \code{"<", ">", "==", "<=", ">=", "!="}. 
  }
}
\details{
  If \code{X} is a pixel image with numeric values,
  then \code{levelset(X, thresh)} finds the region of space
  where the pixel values are less than or equal to
  the threshold value \code{thresh}.
  This region is returned as a spatial window.

  The argument \code{compare} specifies how the pixel values should be
  compared with the threshold value.
  Instead of requiring pixel values to be less than or equal to
  \code{thresh}, you can specify that they must be less than (\code{<}),
  greater than (\code{>}), equal to (\code{==}), greater than or equal
  to (\code{>=}), or not equal to (\code{!=}) the threshold value
  \code{thresh}.

  If \code{X} has non-numeric pixel values (for example, logical
  or factor values) it is advisable to use only the comparisons
  \code{==} and \code{!=}, unless you really know what you are doing.

  For more complicated logical comparisons, see \code{\link{solutionset}}.
}
\value{
  A spatial window (object of class \code{"owin"},
  see \code{\link{owin.object}}) containing the pixels satisfying the
  constraint.
}
\seealso{
  \code{\link{im.object}},
  \code{\link{as.owin}},
  \code{\link{solutionset}}.
}
\examples{
  # test image
  X <- as.im(function(x,y) { x^2 - y^2 }, unit.square())

  W <- levelset(X, 0.2)
  W <- levelset(X, -0.3, ">")

  # compute area of level set
  area(levelset(X, 0.1))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{programming}
\keyword{manip}

\name{summary.quad}
\alias{summary.quad}
\alias{print.summary.quad}
\title{Summarizing a Quadrature Scheme}
\description{
  \code{summary} method for class \code{"quad"}.
}
\usage{
  \method{summary}{quad}(object, \dots, checkdup=FALSE)
  \method{print}{summary.quad}(x, \dots, dp=3)
}
\arguments{
  \item{object}{A quadrature scheme.}
  \item{\dots}{Ignored.}
  \item{checkdup}{
    Logical value indicating whether to test for duplicated points.
  }
  \item{dp}{Number of significant digits to print.} 
  \item{x}{Object of class \code{"summary.quad"} returned by
    \code{summary.quad}.}
}
\details{
  This is a method for the generic \code{\link{summary}}
  for the class \code{"quad"}. An object of class \code{"quad"}
  describes a quadrature scheme, used to fit a point process model.
  See \code{\link{quad.object}}) for details of this class. 
  
  \code{summary.quad} extracts information about the
  quadrature scheme,
  and \code{print.summary.quad} prints this information in a
  comprehensible format.

  In normal usage, \code{print.summary.quad} is invoked implicitly
  when the user calls \code{summary.quad} without assigning its value
  to anything. See the examples.
}
\value{
  \code{summary.quad} returns an object of class \code{"summary.quad"},
  while \code{print.summary.quad} returns \code{NULL}.
}
\examples{
  # make a quadrature scheme
  Q <- quadscheme(rpoispp(42))
  # summarize it
  summary(Q)
  # save the summary
  s <- summary(Q)
  # print it
  print(s)
  s
  # extract total quadrature weight
  s$w$all$sum
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}

\name{box3}
\Rdversion{1.1}
\alias{box3}
\title{
  Three-Dimensional Box
}
\description{
  Creates an object representing a three-dimensional box.
}
\usage{
box3(xrange = c(0, 1), yrange = xrange, zrange = yrange, unitname = NULL)
}
\arguments{
  \item{xrange, yrange, zrange}{
    Dimensions of the box in the \eqn{x,y,z} directions.
    Each of these arguments should be a numeric vector of length 2.
  }
  \item{unitname}{
    Optional. Name of the unit of length. See Details.
}
}
\details{
  This function creates an object representing
  a three-dimensional rectangular parallelepiped (box)
  with sides parallel to the coordinate axes.

  The object can be used to specify the domain of a three-dimensional
  point pattern (see \code{\link{pp3}}) and in various
  geometrical calculations (see \code{\link{volume.box3}},
  \code{\link{diameter.box3}}, \code{\link{eroded.volumes}}). 
  
  The optional argument \code{unitname} specifies the name
  of the unit of length. See \code{\link{unitname}}
  for valid formats.

  The function \code{\link{as.box3}} can be used to convert other kinds
  of data to this format.
}
\value{
  An object of class \code{"box3"}. There is a print method for this class.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{as.box3}}, 
  \code{\link{pp3}},
  \code{\link{volume.box3}},
  \code{\link{diameter.box3}},
  \code{\link{eroded.volumes}}.
}
\examples{
    box3()
    box3(c(0,10),c(0,10),c(0,5), unitname=c("metre","metres"))
    box3(c(-1,1))
}
\keyword{spatial}
\keyword{datagen}

\name{subset.hyperframe}
\alias{subset.hyperframe}
\title{
  Subset of Hyperframe Satisfying A Condition
}
\description{
  Given a hyperframe, return the subset specified by
  imposing a condition on each row, and optionally by choosing
  only some of the columns.
}
\usage{
\method{subset}{hyperframe}(x, subset, select, \dots)
}
\arguments{
  \item{x}{
    A hyperframe pattern (object of class \code{"hyperframe"}.
  }
  \item{subset}{
    Logical expression indicating which points are to be kept.
    The expression may involve the names of columns of \code{x}
    and will be evaluated by \code{\link{with.hyperframe}}.
  }
  \item{select}{
    Expression indicating which columns of marks should be kept.
  }
  \item{\dots}{
    Arguments passed to \code{\link{[.hyperframe}}
    such as \code{drop} and \code{strip}.
  }
}
\details{
  This is a method for the generic function \code{\link{subset}}.
  It extracts the subset of rows of \code{x}
  that satisfy the logical expression
  \code{subset}, and retains only the columns of \code{x} that are
  specified by the expression \code{select}. The result is always a
  hyperframe.

  The argument \code{subset} determines the subset of rows that
  will be extracted. It should be a logical expression.
  It may involve the names of columns of \code{x}.
  The default is to keep all points.

  The argument \code{select} determines which columns of \code{x}
  will be retained.
  It should be an expression involving the names of columns
  (which will be interpreted as integers representing the positions of
  these columns). For example if there are columns named
  \code{A} to \code{Z}, then \code{select=D:F} is a valid expression
  and means that columns \code{D}, \code{E} and \code{F} will be
  retained. Similarly \code{select=-(A:C)} is valid and means that columns
  \code{A} to \code{C} will be deleted.  
  The default is to retain all columns.

  Setting \code{subset=FALSE} will remove all the rows.
  Setting \code{select=FALSE} will remove all the columns.
  
  The result is always a hyperframe.
}
\value{
  A hyperframe.
}  
\author{\adrian
  
  ,
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link[base]{subset}},
  \code{\link{[.hyperframe}}
}
\examples{
 a <- subset(flu, virustype=="wt")

 aa <- subset(flu, minnndist(pattern) > 10)

 aaa <- subset(flu, virustype=="wt", select = -pattern)
}
\keyword{spatial}
\keyword{manip}

\name{crossdist.ppp} 
\alias{crossdist.ppp}
\title{Pairwise distances between two different point patterns}
\description{
  Computes the distances between pairs of points
  taken from two different point patterns.
}
\usage{
  \method{crossdist}{ppp}(X, Y, \dots, periodic=FALSE, method="C", squared=FALSE)
}
\arguments{
  \item{X,Y}{
    Point patterns (objects of class \code{"ppp"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{periodic}{
    Logical. Specifies whether to apply a periodic edge correction.
  }
  \item{method}{
    String specifying which method of calculation to use.
    Values are \code{"C"} and \code{"interpreted"}.
  }
  \item{squared}{
    Logical. If \code{squared=TRUE}, the squared distances are
    returned instead (this computation is faster).
  }
}
\value{
  A matrix whose \code{[i,j]} entry is the distance
  from the \code{i}-th point in \code{X}
  to the \code{j}-th point in \code{Y}.
}
\details{
  Given two point patterns, 
  this function computes the Euclidean distance from each point
  in the first pattern to each point in the second pattern,
  and returns a matrix containing these distances.

  This is a method for the generic function \code{\link{crossdist}}
  for point patterns (objects of class \code{"ppp"}).

  This function expects two
  point patterns \code{X} and \code{Y}, and returns the matrix
  whose \code{[i,j]} entry is the distance from \code{X[i]} to
  \code{Y[j]}.

  Alternatively if \code{periodic=TRUE}, then provided the windows
  containing \code{X} and \code{Y} are identical and are rectangular,
  then the distances will be computed in the `periodic'
  sense (also known as `torus' distance): opposite edges of the
  rectangle are regarded as equivalent.
  This is meaningless if the window is not a rectangle.
  
  The argument \code{method} is not normally used. It is
  retained only for checking the validity of the software.
  If \code{method = "interpreted"} then the distances are
  computed using interpreted R code only. If \code{method="C"}
  (the default) then C code is used. 
  The C code is faster by a factor of 4.
}
\seealso{
  \code{\link{crossdist}},
  \code{\link{crossdist.default}},
  \code{\link{crossdist.psp}},
  \code{\link{pairdist}},
  \code{\link{nndist}},
  \code{\link{Gest}}
}
\examples{
   data(cells)
   d <- crossdist(cells, runifpoint(6))
   d <- crossdist(cells, runifpoint(6), periodic=TRUE)
}
\author{Pavel Grabarnik
  \email{pavel.grabar@issp.serpukhov.su}
  and
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{harmonise.im}
\alias{harmonise.im}
\alias{harmonize.im}
\title{Make Pixel Images Compatible}
\description{
  Convert several pixel images to a common pixel raster.
}
\usage{
\method{harmonise}{im}(\dots)

\method{harmonize}{im}(\dots)
}
\arguments{
  \item{\dots}{
    Any number of pixel images (objects of class \code{"im"})
    or data which can be converted to pixel images by \code{\link{as.im}}.
  } 
}
\details{
  This function makes any number of pixel images compatible,
  by converting them all to a common pixel grid.

  The command \code{\link{harmonise}} is generic. This is the
  method for objects of class \code{"im"}.
  
  At least one of the arguments \code{\dots} must be a pixel image.
  Some arguments may be windows (objects of class \code{"owin"}),
  functions (\code{function(x,y)}) or numerical constants. These will be
  converted to images using \code{\link{as.im}}.
  
  The common pixel grid is determined by inspecting all the pixel
  images in the argument list, computing the bounding box of all the
  images, then finding the image with the highest spatial resolution, 
  and extending its pixel grid to cover the bounding box. 

  The return value is a list with entries corresponding to the input
  arguments.
  If the arguments were named (\code{name=value}) then the return value
  also carries these names.

  If you just want to determine the appropriate pixel resolution,
  without converting the images, use \code{\link{commonGrid}}.
}
\value{
  A list,
  of length equal to the number of arguments \code{\dots},
  whose entries are pixel images.
}
\author{\adrian
  
  
  and \rolf
  
}
\examples{
   A <- setcov(square(1))
   B <- function(x,y) { x }
   G <- density(runifpoint(42))
   harmonise(X=A, Y=B, Z=G)
}
\seealso{
  \code{\link{commonGrid}},
  \code{\link{compatible.im}},
  \code{\link{as.im}}
}
\keyword{spatial}
\keyword{manip}

\name{perspPoints}
\alias{perspPoints}
\alias{perspSegments}
\alias{perspLines}
\alias{perspContour}
\title{
  Draw Points or Lines on a Surface Viewed in Perspective
}
\description{
  After a surface has been plotted in a perspective view
  using \code{\link{persp.im}}, these functions can be used to
  draw points or lines on the surface.
}
\usage{
perspPoints(x, y=NULL, \dots, Z, M)

perspLines(x, y = NULL, \dots, Z, M)

perspSegments(x0, y0 = NULL, x1 = NULL, y1 = NULL, \dots, Z, M)

perspContour(Z, M, \dots,
             nlevels=10, levels=pretty(range(Z), nlevels))
}
\arguments{
  \item{x,y}{
    Spatial coordinates,
    acceptable to \code{\link[grDevices]{xy.coords}},
    for the points or lines on the horizontal plane.
  }
  \item{Z}{
    Pixel image (object of class \code{"im"})
    specifying the surface heights.
  }
  \item{M}{
    Projection matrix returned from \code{\link{persp.im}}
    when \code{Z} was plotted.
  }
  \item{\dots}{
    Graphical arguments passed to
    \code{\link[graphics]{points}},
    \code{\link[graphics]{lines}} or
    \code{\link[graphics]{segments}}
    to control the drawing.
  }
  \item{x0,y0,x1,y1}{
    Spatial coordinates of the line segments, on the
    horizontal plane. Alternatively \code{x0} can be a
    line segment pattern (object of class \code{"psp"})
    and \code{y0,x1,y1} can be \code{NULL}.
  }
  \item{nlevels}{Number of contour levels}
  \item{levels}{Vector of heights of contours.}
}
\details{
  After a surface has been plotted in a perspective view,
  these functions can be used to draw points or lines on the surface.

  The user should already have called \code{\link{persp.im}}
  in the form \code{M <- persp(Z, visible=TRUE, ...)} 
  to display the perspective view of the surface \code{Z}.

  Only points and lines which are visible from the viewer's standpoint
  will be drawn.  
}
\value{
  Same as the return value from
  \code{\link[graphics]{points}}
  or \code{\link[graphics]{segments}}. 
}
\seealso{
  \code{\link{persp.im}}
}
\examples{
  M <- persp(bei.extra$elev, colmap=terrain.colors(128),
             apron=TRUE, theta=-30, phi=20,
             zlab="Elevation", main="", 
             expand=6, visible=TRUE, shade=0.3)

  perspContour(bei.extra$elev, M=M, col="pink", nlevels=12)
  perspPoints(bei, Z=bei.extra$elev, M=M, pch=16, cex=0.3, col="chartreuse")
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{hplot}

\name{setcov}
\alias{setcov}
\title{Set Covariance of a Window}
\description{
  Computes the set covariance function of a window.
}
\usage{
 setcov(W, V=W, \dots)
}
\arguments{
  \item{W}{
    A window (object of class \code{"owin"}.
  }
  \item{V}{
    Optional. Another window.
  }
  \item{\dots}{
    Optional arguments passed to \code{\link{as.mask}}
    to control the pixel resolution.
  }
}
\value{
  A pixel image (an object of class \code{"im"}) representing the
  set covariance function of \code{W},
  or the cross-covariance of \code{W} and \code{V}.
}
\details{
  The set covariance function of a region \eqn{W} in the plane
  is the function \eqn{C(v)} defined for each vector \eqn{v}
  as the area of the intersection between \eqn{W} and \eqn{W+v},
  where \eqn{W+v} is the set obtained by shifting (translating)
  \eqn{W} by \eqn{v}.

  We may interpret \eqn{C(v)} as the area of the set of
  all points \eqn{x} in \eqn{W} such that \eqn{x+v} also lies in
  \eqn{W}.
  
  This command computes a discretised approximation to
  the set covariance function of any
  plane region \eqn{W} represented as a window object (of class
  \code{"owin"}, see \code{\link{owin.object}}). The return value is
  a pixel image (object of class \code{"im"}) whose greyscale values
  are values of the set covariance function.

  The set covariance is computed using the Fast Fourier Transform,
  unless \code{W} is a rectangle, when an exact formula is used.

  If the argument \code{V} is present, then \code{setcov(W,V)}
  computes the set \emph{cross-covariance} function \eqn{C(x)}
  defined for each vector \eqn{x}
  as the area of the intersection between \eqn{W} and \eqn{V+x}.
}
\seealso{
  \code{\link{imcov}},
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{erosion}}
}
\examples{
  w <- owin(c(0,1),c(0,1))
  v <- setcov(w)
  plot(v)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{interp.im}
\alias{interp.im}
\title{Interpolate a Pixel Image}
\description{
  Interpolates the values of a pixel image at any
  desired location in the frame.
}
\usage{
interp.im(Z, x, y=NULL, bilinear=FALSE)
}
\arguments{
  \item{Z}{
    Pixel image (object of class \code{"im"})
    with numeric or integer values.
  }
  \item{x,y}{
    Vectors of Cartesian coordinates.
    Alternatively \code{x} can be a point pattern and \code{y} can be missing.
  }
  \item{bilinear}{
    Logical value specifying the choice of interpolation rule.
    If \code{bilinear=TRUE} then a bilinear interpolation rule is used.
    If \code{bilinear=FALSE} (the default) then a slightly biased rule
    is used; this rule is consistent with earlier versions of
    \pkg{spatstat}.
  }
}
\details{
  A value at each location \code{(x[i],y[i])} will be
  interpolated using the pixel values of \code{Z} at the four
  surrounding pixel centres, by simple bilinear interpolation.

  At the boundary (where \code{(x[i],y[i])} is not surrounded by
  four pixel centres) the value at the nearest pixel
  is taken.

  The arguments \code{x,y} can be anything acceptable to
  \code{\link[grDevices]{xy.coords}}.
}
\value{
  Vector of interpolated values, with \code{NA} for points that lie
  outside the domain of the image.
}
\examples{
   opa <- par(mfrow=c(1,2))
   # coarse image
   V <- as.im(function(x,y) { x^2 + y }, owin(), dimyx=10)
   plot(V, main="coarse image", col=terrain.colors(256))

   # lookup value at location (0.5,0.5)
   V[list(x=0.5,y=0.5)]
   # interpolated value at location (0.5,0.5)
   interp.im(V, 0.5, 0.5)
   interp.im(V, 0.5, 0.5, bilinear=TRUE)
   # true value is 0.75

   # how to obtain an interpolated image at a desired resolution
   U <- as.im(interp.im, W=owin(), Z=V, dimyx=256)
   plot(U, main="interpolated image", col=terrain.colors(256))
   par(opa)
}

\author{
  \adrian
  and \rolf,
  with a contribution from an anonymous user.
}
\keyword{spatial}
\keyword{manip}

\name{rMatClust}
\alias{rMatClust}
\title{Simulate Matern Cluster Process}
\description{
  Generate a random point pattern, a simulated realisation of the
  \Matern Cluster Process.
}
\usage{
 rMatClust(kappa, scale, mu, win = owin(c(0,1),c(0,1)),
           nsim=1, drop=TRUE, 
           saveLambda=FALSE, expand = scale, ...,
           poisthresh=1e-6, saveparents=TRUE)
}
\arguments{
  \item{kappa}{
    Intensity of the Poisson process of cluster centres.
    A single positive number, a function, or a pixel image.
  }
  \item{scale}{
    Radius parameter of the clusters.
  }
  \item{mu}{
    Mean number of points per cluster (a single positive number)
    or reference intensity for the cluster points (a function or
    a pixel image).
  }
  \item{win}{
    Window in which to simulate the pattern.
    An object of class \code{"owin"}
    or something acceptable to \code{\link{as.owin}}.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
  \item{saveLambda}{
    Logical. If \code{TRUE} then the random intensity corresponding to
    the simulated parent points will also be calculated and saved,
    and returns as an attribute of the point pattern.
  }
  \item{expand}{Numeric. Size of window expansion for generation of
    parent points. Defaults to \code{scale} which is the cluster
    radius.
  }
  \item{\dots}{Passed to \code{\link{clusterfield}} to control the image
    resolution when \code{saveLambda=TRUE}.
  }
  \item{poisthresh}{
    Numerical threshold below which the model will be treated
    as a Poisson process. See Details.
  }
  \item{saveparents}{
    Logical value indicating whether to save the locations of the
    parent points as an attribute.
  }
}
\value{
  A point pattern (an object of class \code{"ppp"}) if \code{nsim=1},
  or a list of point patterns if \code{nsim > 1}.

  Additionally, some intermediate results of the simulation are returned
  as attributes of this point pattern (see
  \code{\link{rNeymanScott}}). Furthermore, the simulated intensity
  function is returned as an attribute \code{"Lambda"}, if
  \code{saveLambda=TRUE}.
}
\details{
  This algorithm generates a realisation of
  \Matern's cluster process,
  a special case of the Neyman-Scott process, inside the window \code{win}.

  In the simplest case, where \code{kappa} and \code{mu}
  are single numbers, the algorithm 
  generates a uniform Poisson point process of \dQuote{parent} points 
  with intensity \code{kappa}. Then each parent point is
  replaced by a random cluster of \dQuote{offspring} points,
  the number of points per cluster being Poisson (\code{mu})
  distributed, and their
  positions being placed and uniformly inside
  a disc of radius \code{scale} centred on the parent point.
  The resulting point pattern
  is a realisation of the classical
  \dQuote{stationary \Matern cluster process}
  generated inside the window \code{win}.
  This point process has intensity \code{kappa * mu}.

  The algorithm can also generate spatially inhomogeneous versions of
  the \Matern cluster process:
  \itemize{
    \item The parent points can be spatially inhomogeneous.
    If the argument \code{kappa} is a \code{function(x,y)}
    or a pixel image (object of class \code{"im"}), then it is taken
    as specifying the intensity function of an inhomogeneous Poisson
    process that generates the parent points.
    \item The offspring points can be inhomogeneous. If the
    argument \code{mu} is a \code{function(x,y)}
    or a pixel image (object of class \code{"im"}), then it is
    interpreted as the reference density for offspring points,
    in the sense of Waagepetersen (2007).
    For a given parent point, the offspring constitute a Poisson process
    with intensity function equal to 
    \code{mu/(pi * scale^2)}
    inside the disc of radius \code{scale} centred on the parent
    point, and zero intensity outside this disc.
    Equivalently we first generate,
    for each parent point, a Poisson (\eqn{M}) random number of
    offspring (where \eqn{M} is the maximum value of \code{mu})
    placed independently and uniformly in the disc of radius \code{scale}
    centred on the parent location, and then randomly thin the
    offspring points, with retention probability \code{mu/M}.
    \item
    Both the parent points and the offspring points can be
    inhomogeneous, as described above.
  }

  Note that if \code{kappa} is a pixel image, its domain must be larger
  than the window \code{win}. This is because an offspring point inside
  \code{win} could have its parent point lying outside \code{win}.
  In order to allow this, the simulation algorithm
  first expands the original window \code{win}
  by a distance \code{expand} and generates the Poisson process of
  parent points on this larger window. If \code{kappa} is a pixel image,
  its domain must contain this larger window.

  The intensity of the \Matern cluster
  process is \code{kappa * mu}
  if either \code{kappa} or \code{mu} is a single number. In the general
  case the intensity is an integral involving \code{kappa}, \code{mu}
  and \code{scale}.

  The \Matern cluster process model
  with homogeneous parents (i.e. where \code{kappa} is a single number)
  can be fitted to data using \code{\link{kppm}}.
  Currently it is not possible to fit the
  \Matern cluster process model
  with inhomogeneous parents.

  If the pair correlation function of the model is very close
  to that of a Poisson process, deviating by less than
  \code{poisthresh}, then the model is approximately a Poisson process,
  and will be simulated as a Poisson process with intensity
  \code{kappa * mu}, using \code{\link{rpoispp}}.
  This avoids computations that would otherwise require huge amounts
  of memory.
}
\seealso{
  \code{\link{rpoispp}},
  \code{\link{rThomas}},
  \code{\link{rCauchy}},
  \code{\link{rVarGamma}},
  \code{\link{rNeymanScott}},
  \code{\link{rGaussPoisson}},
  \code{\link{kppm}},
  \code{\link{clusterfit}}.
}
\examples{
 # homogeneous
 X <- rMatClust(10, 0.05, 4)
 # inhomogeneous
 ff <- function(x,y){ 4 * exp(2 * abs(x) - 1) }
 Z <- as.im(ff, owin())
 Y <- rMatClust(10, 0.05, Z)
 YY <- rMatClust(ff, 0.05, 3)
}
\references{
  \Matern, B. (1960)
  \emph{Spatial Variation}.
  Meddelanden \ifelse{latex}{\out{fr\r{a}n}}{fraan}
  Statens Skogsforskningsinstitut,
  volume 59, number 5.  Statens Skogsforskningsinstitut, Sweden.

  \Matern, B. (1986)
  \emph{Spatial Variation}.
  Lecture Notes in Statistics 36, Springer-Verlag, New York.

  Waagepetersen, R. (2007)
  An estimating function approach to inference for
  inhomogeneous Neyman-Scott processes.
  \emph{Biometrics} \bold{63}, 252--258.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{shift}
\alias{shift}
\title{Apply Vector Translation}
\description{
  Applies a vector shift of the plane 
  to a geometrical object,
  such as a point pattern or a window. 
}
\usage{
  shift(X, \dots)
}
\arguments{
  \item{X}{Any suitable dataset representing a two-dimensional
    object, such as a point pattern (object of class \code{"ppp"}),
    or a window (object of class \code{"owin"}).}
  \item{\dots}{Arguments determining the shift vector.}
}
\value{
  Another object of the same type, representing the
  result of applying the shift.
}
\details{
  This is generic. Methods are provided for
  point patterns (\code{\link{shift.ppp}})
  and windows (\code{\link{shift.owin}}).

  The object is translated by the vector \code{vec}.
}
\seealso{
  \code{\link{shift.ppp}},
  \code{\link{shift.owin}},
  \code{\link{rotate}},
  \code{\link{affine}},
  \code{\link{periodify}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{is.empty}  
\alias{is.empty}
\alias{is.empty.owin}
\alias{is.empty.ppp}
\alias{is.empty.psp}
\alias{is.empty.default}
\title{Test Whether An Object Is Empty}
\description{
  Checks whether the argument is an empty window,
  an empty point pattern, etc.
}
\usage{
is.empty(x)
\method{is.empty}{owin}(x)
\method{is.empty}{ppp}(x)
\method{is.empty}{psp}(x)
\method{is.empty}{default}(x)
}
\arguments{
  \item{x}{
    A window (object of class \code{"owin"}),
    a point pattern (object of class \code{"ppp"}), or
    a line segment pattern (object of class \code{"psp"}).
  }
}
\details{
  This function tests whether the object \code{x}
  represents an empty spatial object, such as an empty window,
  a point pattern with zero points, or a line segment pattern
  with zero line segments. 

  An empty window can be obtained as the output of
  \code{\link{intersect.owin}}, \code{\link{erosion}},
  \code{\link{opening}}, \code{\link{complement.owin}}
  and some other operations.

  An empty point pattern or line segment pattern can be
  obtained as the result of simulation.
}
\value{
  Logical value.
}

\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{pixellate.owin}  
\Rdversion{1.1}
\alias{pixellate.owin}
\title{
  Convert Window to Pixel Image
}
\description{
  Convert a window to 
  a pixel image by measuring the area of intersection
  between the window and each pixel in a raster.
}
\usage{
\method{pixellate}{owin}(x, W = NULL, ..., DivideByPixelArea=FALSE)
}
\arguments{
  \item{x}{
    Window (object of class \code{"owin"}) to be converted.
  }
  \item{W}{
    Optional. Window determining the pixel raster on which the conversion
    should occur. 
  }
  \item{\dots}{
    Optional. Extra arguments passed to \code{\link{as.mask}}
    to determine the pixel raster.
  }
  \item{DivideByPixelArea}{
    Logical value, indicating whether the resulting pixel values
    should be divided by the pixel area.
  }
}
\details{
  This is a method for the generic function \code{pixellate}.

  It converts a window \code{x} into
  a pixel image, by measuring the \emph{amount} of \code{x} that is
  inside each pixel.

  (The related function \code{\link{as.im}}
  also converts \code{x} into a pixel image,
  but records only the presence or absence of
  \code{x} in each pixel.)

  The pixel raster for the conversion is determined by the
  argument \code{W} and the extra arguments \code{\dots}.
  \itemize{
    \item 
    If \code{W} is given, and it is a binary mask (a window
    of type \code{"mask"}) then it determines the pixel raster.
    \item
    If \code{W} is given, but it is not a binary mask (it is a
    window of another type) then it will be converted to a binary mask
    using \code{as.mask(W, \dots)}.
    \item
    If \code{W} is not given, it defaults to
    \code{as.mask(as.rectangle(x), \dots)}
  }
  In the second and third cases it would be common to use the
  argument \code{dimyx} to control the number of pixels. See the Examples.

  The algorithm then computes the area
  of intersection of each pixel with the window.

  The result is a pixel image with pixel entries equal to these
  intersection areas.
}
\value{
  A pixel image (object of class \code{"im"}).
}
\seealso{
  \code{\link{pixellate.ppp}},
  \code{\link{pixellate}},
  \code{\link{as.im}}
}
\examples{
   data(letterR)
   plot(pixellate(letterR, dimyx=15))
   W <- grow.rectangle(as.rectangle(letterR), 0.2)
   plot(pixellate(letterR, W, dimyx=15))
}
\author{\adrian
  and \rolf
}
\keyword{spatial}
\keyword{manip}

\name{Extract.im}
\alias{[.im}
\title{Extract Subset of Image}
\description{
  Extract a subset or subregion of a pixel image.
}
\usage{
  \method{[}{im}(x, i, j, \dots, drop=TRUE, tight=FALSE,
                                 raster=NULL, rescue=is.owin(i))
}
\arguments{
  \item{x}{
    A two-dimensional pixel image.
    An object of class \code{"im"}.
  }
  \item{i}{
    Object defining the subregion or subset to be extracted.
    Either a spatial window (an object of class \code{"owin"}), or a
    pixel image with logical values, or a linear network (object of
    class \code{"linnet"}) or a point pattern (an object
    of class \code{"ppp"}), or any type of index that applies to a
    matrix, or something that can be converted to a point pattern
    by \code{\link{as.ppp}} (using the window of \code{x}).
  }
  \item{j}{
    An integer or logical vector serving as the column index if
    matrix indexing is being used. Ignored if \code{i} is a spatial object.
  }
  \item{\dots}{Ignored.}
  \item{drop}{
    Logical value. 
    Locations in \code{w} that lie outside the spatial domain of the
    image \code{x} return a pixel value of \code{NA} if
    \code{drop=FALSE}, and are omitted if \code{drop=TRUE}.
  }
  \item{tight}{
    Logical value. If \code{tight=TRUE}, and if the result of the
    subset operation is an image, the image will be trimmed
    to the smallest possible rectangle.
  }
  \item{raster}{
     Optional. An object of class \code{"owin"} or \code{"im"}
     determining a pixel grid.
  }
  \item{rescue}{
   Logical value indicating whether rectangular blocks of data
   should always be returned as pixel images.
  }
}
\value{
  Either a pixel image or a vector of pixel values. See Details.
}
\details{
  This function extracts a subset of the pixel values in a
  pixel image. (To reassign the pixel values, see \code{\link{[<-.im}}).

  The image \code{x} must be an object of class
  \code{"im"} representing a pixel image defined inside a
  rectangle in two-dimensional space (see \code{\link{im.object}}).

  The subset to be extracted is determined by the arguments \code{i,j}
  according to the following rules (which are checked in this order):
  \enumerate{
    \item
    \code{i} is a spatial object such as a window,
    a pixel image with logical values,
    a linear network, or a point pattern;  or
    \item
    \code{i,j} are indices for the matrix \code{as.matrix(x)}; or
    \item
    \code{i} can be converted to a point pattern
    by \code{\link{as.ppp}(i, W=Window(x))},
    and \code{i} is not a matrix.
  }
    
  If \code{i} is a spatial window (an object of class \code{"owin"}),
  the values of the image inside this window are extracted
  (after first clipping the window to the spatial domain of the image
  if necessary).

  If \code{i} is a linear network (object of class \code{"linnet"}),
  the values of the image on this network are extracted.

  If \code{i} is a pixel image with logical values,
  it is interpreted as a spatial window (with \code{TRUE} values
  inside the window and \code{FALSE} outside).

  If \code{i} is a point pattern (an object of class
  \code{"ppp"}), then the values of the pixel image at the points of
  this pattern are extracted. This is a simple way to read the
  pixel values at a given spatial location.

  At locations outside the spatial domain of the image, the pixel
  value is undefined, and is taken to be \code{NA}.  The logical
  argument \code{drop} determines whether such \code{NA} values
  will be returned or omitted.  It also influences the format of
  the return value.

  If \code{i} is a point pattern (or something that can be converted
  to a point pattern), then \code{X[i, drop=FALSE]} is a numeric
  vector containing the pixel values at each of the points of
  the pattern.  Its length is equal to the number of points in the
  pattern \code{i}.  It may contain \code{NA}s corresponding to
  points which lie outside the spatial domain of the image \code{x}.
  By contrast, \code{X[i]} or \code{X[i, drop=TRUE]} contains only
  those pixel values which are not \code{NA}. It may be shorter.

  If \code{i} is a spatial window then \code{X[i, drop=FALSE]} is
  another pixel image of the same dimensions as \code{X} obtained
  by setting all pixels outside the window \code{i} to have value
  \code{NA}. When the result is displayed by \code{\link{plot.im}}
  the effect is that the pixel image \code{x} is clipped to the
  window \code{i}.

  If \code{i} is a linear network (object of class \code{"linnet"})
  then \code{X[i, drop=FALSE]} is another pixel image of the same
  dimensions as \code{X} obtained by restricting the pixel image
  \code{X} to the linear network. The result also belongs to the
  class \code{"linim"} (pixel image on a linear network). 

  If \code{i} is a spatial window then \code{X[i, drop=TRUE]}
  is either:
  \itemize{
    \item a numeric vector containing the pixel values for all pixels
          that lie inside the window \code{i}.
          This happens if \code{i} is \emph{not} a rectangle
          (i.e. \code{i$type != "rectangle"})
          or if \code{rescue=FALSE}.
    \item  a pixel image.
          This happens only if
          \code{i} is a rectangle (\code{i$type = "rectangle"})
          and \code{rescue=TRUE} (the default).
  }

  If the optional argument \code{raster} is given, then it should
  be a binary image mask or a pixel image. Then
  \code{x} will first be converted to an image defined on the
  pixel grid implied by \code{raster}, before the subset operation
  is carried out.
  In particular, \code{x[i, raster=i, drop=FALSE]} will return
  an image defined on the same pixel array as the object \code{i}.

  If \code{i} does not satisfy any of the conditions above, then
  the algorithm attempts to interpret \code{i} and \code{j}
  as indices for the matrix \code{as.matrix(x)}.
  Either \code{i} or \code{j} may be missing or blank.
  The result is usually a vector or matrix of pixel values.
  Exceptionally the result is a pixel image if \code{i,j} determines
  a rectangular subset of the pixel grid, and if the user specifies
  \code{rescue=TRUE}.

  Finally, if none of the above conditions is met,
  the object \code{i} may also be a data frame or list of \code{x,y}
  coordinates which will be converted to a point pattern, taking the
  observation window to be \code{Window(x)}. Then the pixel values
  at these points will be extracted as a vector.
}
\section{Warnings}{
  If you have a 2-column matrix containing the \eqn{x,y} coordinates
  of point locations, then to prevent this being interpreted as an
  array index, you should convert it to a \code{data.frame}
  or to a point pattern.

  If \code{W} is a window or a pixel image, then \code{x[W, drop=FALSE]} 
  will return an image defined on the same pixel array
  as the original image \code{x}. If you want to obtain an image
  whose pixel dimensions agree with those of \code{W}, use the
  \code{raster} argument, \code{x[W, raster=W, drop=FALSE]}.
}
\seealso{
  \code{\link{im.object}},
  \code{\link{[<-.im}},
  \code{\link{ppp.object}},
  \code{\link{as.ppp}},
  \code{\link{owin.object}},
  \code{\link{plot.im}}
}
\examples{
 # make up an image
 X <- setcov(unit.square())
 plot(X)

 # a rectangular subset
 W <- owin(c(0,0.5),c(0.2,0.8))
 Y <- X[W]
 plot(Y)

 # a polygonal subset
 R <- affine(letterR, diag(c(1,1)/2), c(-2,-0.7))
 plot(X[R, drop=FALSE])
 plot(X[R, drop=FALSE, tight=TRUE])

 # a point pattern
 P <- rpoispp(20)
 Y <- X[P]

 # look up a specified location
 X[list(x=0.1,y=0.2)]

 # 10 x 10 pixel array
 X <- as.im(function(x,y) { x + y }, owin(c(-1,1),c(-1,1)), dimyx=10)
 # 100 x 100 
 W <- as.mask(disc(1, c(0,0)), dimyx=100)
 # 10 x 10 raster
 X[W,drop=FALSE]
 # 100 x 100 raster
 X[W, raster=W, drop=FALSE]
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{is.convex}
\alias{is.convex}
\title{Test Whether a Window is Convex}
\description{
  Determines whether a window is convex.
}
\usage{
  is.convex(x)
}
\arguments{
  \item{x}{
    Window (object of class \code{"owin"}).
  }
}
\value{
  Logical value, equal to \code{TRUE} if \code{x} is convex.
}
\details{
  If \code{x} is a rectangle, the result is TRUE.

  If \code{x} is polygonal, the result is TRUE if \code{x} consists of a
  single polygon and this polygon is equal to the minimal convex hull
  of its vertices computed by \code{\link[grDevices]{chull}}. 

  If \code{x} is a mask, the algorithm first extracts all boundary
  pixels of \code{x} using \code{\link{vertices}}. Then it computes
  the (polygonal) convex hull \eqn{K} of the boundary pixels.
  The result is TRUE if every boundary pixel lies within 
  one pixel diameter of an edge of \eqn{K}.
}
\seealso{
  \code{\link{owin}},
  \code{\link{convexhull.xy}},
  \code{\link{vertices}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{vertices}
\alias{vertices}
\alias{vertices.owin}
\title{Vertices of a Window}
\description{
  Finds the vertices of a window, or similar object.
}
\usage{
 vertices(w)

 \method{vertices}{owin}(w)
}
\arguments{
  \item{w}{A window (object of class \code{"owin"}) or similar object.}
}
\value{
  A list with components \code{x} and \code{y} giving the coordinates
  of the vertices.
}
\details{
  This function computes the vertices (`corners') of a spatial window
  or other object.

  For \code{vertices.owin},
  the argument \code{w} should be a window (an object of class
  \code{"owin"}, see \code{\link{owin.object}} for details).

  If \code{w} is a rectangle, the coordinates of the
  four corner points are returned.

  If \code{w} is a polygonal window (consisting of one or more
  polygons), the coordinates of the vertices of all polygons are
  returned.

  If \code{w} is a binary mask, then a `boundary pixel' is defined to be
  a pixel inside the window
  which has at least one neighbour outside the window. The coordinates
  of the centres of all boundary pixels are returned.
}
\seealso{
  \code{\link{owin.object}}.
}
\examples{
  data(letterR)
  vert <- vertices(letterR)

  plot(letterR, main="Polygonal vertices")
  points(vert)
  plot(letterR, main="Boundary pixels")
  points(vertices(as.mask(letterR)))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{pixellate.psp}  
\alias{pixellate.psp}
\title{
  Convert Line Segment Pattern to Pixel Image
}
\description{
  Converts a line segment pattern to a pixel image
  by measuring the length or number of lines intersecting each pixel.
}
\usage{
\method{pixellate}{psp}(x, W=NULL, ..., weights = NULL,
                        what=c("length", "number"),
                        DivideByPixelArea=FALSE)
}
\arguments{
  \item{x}{
    Line segment pattern (object of class \code{"psp"}).
  }
  \item{W}{
    Optional window (object of class \code{"owin"}) determining
    the pixel resolution.
  }
  \item{\dots}{
    Optional arguments passed to \code{\link{as.mask}} to determine
    the pixel resolution.
  }
  \item{weights}{
    Optional vector of weights associated with each line segment.
  }
  \item{what}{
    String (partially matched) indicating whether to compute the
    total length of intersection (\code{what="length"}, the default)
    or the total number of segments intersecting each pixel
    (\code{what="number"}).
  }
  \item{DivideByPixelArea}{
    Logical value, indicating whether the resulting pixel values
    should be divided by the pixel area.
  }
}
\details{
  This function converts a line segment pattern to a pixel image by computing,
  for each pixel, the total length of
  intersection between the pixel and the line segments.
  Alternatively it can count the number of line segments intersecting
  each pixel. 
  
  This is a method for the generic function
  \code{\link{pixellate}} for the class of line segment patterns.

  The pixel raster is determined by \code{W}
  and the optional arguments \code{\dots}.
  If \code{W} is missing or \code{NULL}, it defaults to the window
  containing \code{x}.
  Then \code{W} is converted to a
  binary pixel mask using \code{\link{as.mask}}. The arguments
  \code{\dots} are passed to \code{\link{as.mask}} to
  control the pixel resolution.

  If \code{weights} are given, then the length of the intersection
  between line segment \code{i} and pixel \code{j} is multiplied by
  \code{weights[i]} before the lengths are summed for each pixel.
}
\value{
  A pixel image (object of class \code{"im"}) with numeric values.
}
\seealso{
  \code{\link{pixellate}},
  \code{\link{as.mask}},
  \code{\link{as.mask.psp}}.
  
  Use \code{\link{as.mask.psp}} if you only want to know
  which pixels are intersected by lines.
}
\examples{
  X <- psp(runif(10),runif(10), runif(10), runif(10), window=owin())
  plot(pixellate(X))
  plot(X, add=TRUE)
  sum(lengths_psp(X))
  sum(pixellate(X))
  plot(pixellate(X, what="n"))
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{requireversion}
\alias{requireversion}
\title{
  Require a Specific Version of a Package
}
\description{
  Checks that the version number of a specified package is greater than or
  equal to the specified version number.
  For use in stand-alone \R scripts.
}
\usage{
requireversion(pkg, ver, fatal=TRUE)
}
\arguments{
  \item{pkg}{
    Package name.
  }
  \item{ver}{
    Character string containing version number.
  }
  \item{fatal}{
    Logical value indicating whether an error should occur
    when the package version is less than \code{ver}.
  }
}
\details{
  This function checks whether the installed version of the
  package \code{pkg} is greater than or equal to \code{ver}.
  By default, an error occurs if this condition is not met.

  It is useful in stand-alone \R scripts, which often require
  a particular version of a package in order to work correctly.

  \bold{This function should not be used inside a package}:
  for that purpose, the dependence on packages and versions
  should be specified in the package description file.
}
\value{
  A logical value. 
}
\author{
  \adrian
}
\examples{
  requireversion(spatstat, "1.42-0")
  requireversion(spatstat, "999.999-999", fatal=FALSE)
}
\keyword{environment}

\name{eroded.areas}
\alias{eroded.areas}
\title{Areas of Morphological Erosions}
\description{
  Computes the areas of successive morphological erosions of a window.
}
\usage{
 eroded.areas(w, r, subset=NULL)
}
\arguments{
  \item{w}{A window.}
  \item{r}{Numeric vector of radii at which erosions will be performed.}
  \item{subset}{
    Optional window inside which the areas should be computed.
  }
}
\value{
  Numeric vector, of the same length as \code{r},
  giving the areas of the successive erosions.
}
\details{
  This function computes the areas of the erosions of the window
  \code{w} by each of the radii \code{r[i]}. 

  The morphological erosion of a set \eqn{W} by a distance \eqn{r > 0}
  is the subset 
  consisting of all points \eqn{x \in W}{x in W} such that the
  distance from \eqn{x} to the boundary of \eqn{W} is greater than
  or equal to \eqn{r}. In other words it is the result of trimming
  a margin of width \eqn{r} off the set \eqn{W}.

  The argument \code{r} should be a vector of positive numbers.
  The argument \code{w} should be a window (an object of class
  \code{"owin"}, see \code{\link{owin.object}} for details)
  or can be given in any format acceptable to \code{\link{as.owin}()}.

  Unless \code{w} is a rectangle, the computation is performed
  using a pixel raster approximation.

  To compute the eroded window itself, use \code{\link{erosion}}.
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{erosion}}
}
\examples{
  w <- owin(c(0,1),c(0,1))
  a <- eroded.areas(w, seq(0.01,0.49,by=0.01))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{fourierbasis}
\alias{fourierbasis}
\alias{fourierbasisraw}
\title{Fourier Basis Functions}
\description{Evaluates the Fourier basis functions
  on a \eqn{d}-dimensional box
  with \eqn{d}-dimensional frequencies \eqn{k_i} at the
  \eqn{d}-dimensional coordinates \eqn{x_j}.
}
\usage{
  fourierbasis(x, k, win = boxx(rep(list(0:1), ncol(k))))
  fourierbasisraw(x, k, boxlengths)
}
\arguments{
  \item{x}{
    Coordinates. 
    A \code{data.frame} or matrix with
    \eqn{n} rows and \eqn{d} columns giving
    the \eqn{d}-dimensional coordinates.
  }
  \item{k}{Frequencies.
    A \code{data.frame} or matrix with \eqn{m} rows and \eqn{d} columns
    giving the frequencies of the Fourier-functions.
  }
  \item{win}{
    window (of class \code{"owin"}, \code{"box3"} or \code{"boxx"})
    giving the \eqn{d}-dimensional box domain of the Fourier functions.
  }
  \item{boxlengths}{
    numeric giving the side lengths of the box domain of the Fourier functions.
  }
}
\details{
  The result is an \eqn{m} by \eqn{n} matrix where the \eqn{(i,j)}'th
  entry is the \eqn{d}-dimensional Fourier basis function with
  frequency \eqn{k_i} evaluated at the point \eqn{x_j}, i.e.,
  \deqn{
    \frac{1}{\sqrt{|W|}}
    \exp(2\pi i \sum{l=1}^d k_{i,l} x_{j,l}/L_l)
  }{
    1/sqrt(|W|) * exp(2*pi*i*(k_{i,1}*x_{j,1}/L_1 + ... + k_{i,d}*x_{j,d}/L_d))
  }
  where \eqn{L_l}, \eqn{l=1,...,d} are the box side lengths
  and \eqn{|W|} is the volume of the 
  domain (window/box). Note that the algorithm does not check whether
  the coordinates given in \code{x} are contained in the given box.
  Actually the box is only used to determine the side lengths and volume of the
  domain for normalization.
  
  The stripped down faster version \code{fourierbasisraw} doesn't do checking or
  conversion of arguments and requires \code{x} and \code{k} to be matrices.
}
\value{An \code{m} by \code{n} matrix of complex values.}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\examples{
## 27 rows of three dimensional Fourier frequencies:
k <- expand.grid(-1:1,-1:1, -1:1)
## Two random points in the three dimensional unit box:
x <- rbind(runif(3),runif(3))
## 27 by 2 resulting matrix:
v <- fourierbasis(x, k)
head(v)
}

\name{quad.object}
\alias{quad.object} %DoNotExport
\title{Class of Quadrature Schemes}
\description{
  A class \code{"quad"} to represent a quadrature scheme.
}
\details{
  A (finite) quadrature scheme is a list of quadrature points
  \eqn{u_j}{u[j]} and associated weights \eqn{w_j}{w[j]}
  which is used to approximate an integral by a finite sum:
  \deqn{
    \int f(x) dx \approx \sum_j f(u_j) w_j
  }{
    integral(f(x) dx) ~= sum( f(u[j]) w[j] )
  }
  Given a point pattern dataset, a \emph{Berman-Turner}
  quadrature scheme is one which includes all these data points,
  as well as a nonzero number of other (``dummy'') points.

  These quadrature schemes are used to approximate the
  pseudolikelihood of a point process, in the method of
  Baddeley and Turner (2000) (see Berman and Turner (1992)).
  Accuracy and computation time both increase with the number of
  points in the quadrature scheme.

  An object of class \code{"quad"} represents a Berman-Turner
  quadrature scheme. It can be passed as an argument to
  the model-fitting function \code{\link{ppm}}, which
  requires a quadrature scheme.

  An object of this class contains at least the following elements:
  \tabular{ll}{
    \code{data}: \tab an object of class \code{"ppp"} \cr
                 \tab giving the locations (and marks) of the data points.\cr
    \code{dummy}: \tab an object of class \code{"ppp"} \cr
                 \tab giving the locations (and marks) of the dummy points.\cr
    \code{w}: \tab vector of nonnegative weights for the quadrature points\cr
  }
  Users are strongly advised not to manipulate these entries
  directly.
  
  The domain of quadrature is specified by \code{Window(dummy)}
  while the observation window (if this needs to be specified
  separately) is taken to be \code{Window(data)}. 

  The weights vector \code{w} may also have an attribute
  \code{attr(w, "zeroes")} equivalent to the logical vector
  \code{(w == 0)}. If this is absent then all points are known to
  have positive weights.
  
  To create an object of class \code{"quad"},
  users would typically call the
  high level function \code{\link{quadscheme}}.
  (They are actually
  created by the low level function \code{quad}.)
  
  Entries are extracted from a \code{"quad"} object by the functions
  \code{x.quad},
  \code{y.quad},
  \code{w.quad} and
  \code{marks.quad},
  which extract the \eqn{x} coordinates, \eqn{y} coordinates,
  weights, and marks, respectively. The function
  \code{n.quad} returns the total number of quadrature points
  (dummy plus data).

  An object of class \code{"quad"} can be converted into an ordinary
  point pattern by the function \code{\link{union.quad}} which simply
  takes the union of the data and dummy points.

  Quadrature schemes can be plotted using \code{\link{plot.quad}}
  (a method for the generic \code{\link{plot}}).
}
\seealso{
  \code{\link{quadscheme}},
  \code{\link{ppm}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{attribute}
 
 

\name{split.im}
\alias{split.im}
\title{Divide Image Into Sub-images}
\description{
  Divides a pixel image into several sub-images according to the
  value of a factor, or according to the tiles of a tessellation.
}
\usage{
\method{split}{im}(x, f, ..., drop = FALSE)
}
\arguments{
  \item{x}{Pixel image (object of class \code{"im"}).}
  \item{f}{
    Splitting criterion. Either a tessellation (object of class
    \code{"tess"}) or a pixel image with factor values.
  }
  \item{\dots}{Ignored.}
  \item{drop}{Logical value determining whether each subset should
    be returned as a pixel images (\code{drop=FALSE}) or 
    as a one-dimensional vector of pixel values (\code{drop=TRUE}).
  }
}
\details{
  This is a method for the generic function \code{\link{split}}
  for the class of pixel images. The image \code{x} will be divided
  into subsets determined by the data \code{f}. The result is a list
  of these subsets.

  The splitting criterion may be either
  \itemize{
    \item a tessellation (object of class \code{"tess"}). Each tile of
    the tessellation delineates a subset of the spatial domain.
    \item a pixel image (object of class \code{"im"}) with factor
    values. The levels of the factor determine subsets of the spatial
    domain.
  }
  If \code{drop=FALSE} (the default), the result is a list of pixel
  images, each one a subset of the pixel image \code{x},
  obtained by restricting the pixel domain to one of the subsets.
  If \code{drop=TRUE}, then the pixel values are returned as
  numeric vectors. 
}
\value{
  If \code{drop=FALSE}, a list of pixel images (objects of class
  \code{"im"}). It is also of class \code{"solist"} so that it can be
  plotted immediately.

  If \code{drop=TRUE}, a list of numeric vectors.
}
\seealso{
  \code{\link{by.im}},
  \code{\link{tess}},
  \code{\link{im}}
}
\examples{
  W <- square(1)
  X <- as.im(function(x,y){sqrt(x^2+y^2)}, W)
  Y <- dirichlet(runifpoint(12, W))
  plot(split(X,Y))
}
\author{\adrian
  and \rolf.
}
\keyword{spatial}
\keyword{methods}
\keyword{manip}

\name{simplify.owin}
\Rdversion{1.1}
\alias{simplify.owin}
\title{
  Approximate a Polygon by a Simpler Polygon
}
\description{
  Given a polygonal window, this function finds a simpler polygon
  that approximates it.
}
\usage{
simplify.owin(W, dmin)
}
\arguments{
  \item{W}{
    The polygon which is to be simplied.
    An object of class \code{"owin"}.
  }
  \item{dmin}{
    Numeric value. The smallest permissible length of an edge.
  }
}
\details{
  This function simplifies a polygon \code{W}
  by recursively deleting the shortest edge of \code{W}
  until all remaining edges are longer than the specified
  minimum length \code{dmin}, or until there are only three edges left.

  The argument \code{W} must be a window (object of class
  \code{"owin"}). It should be of type \code{"polygonal"}.
  If \code{W} is a rectangle, it is returned without alteration.

  The simplification algorithm is not yet implemented for
  binary masks. If \code{W} is a mask, an error is generated.
}
\value{
  Another window (object of class \code{"owin"})
  of type \code{"polygonal"}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{owin}}
}
\examples{
  plot(letterR, col="red")
  plot(simplify.owin(letterR, 0.3), col="blue", add=TRUE)

  W <- Window(chorley)
  plot(W)
  WS <- simplify.owin(W, 2)
  plot(WS, add=TRUE, border="green")
  points(vertices(WS))
}
\keyword{spatial}
\keyword{math}

\name{owin.object}
\alias{owin.object} %DoNotExport
\title{Class owin}
\description{
  A class \code{owin} to define the ``observation window'' of a point pattern
}
\details{
  In the \pkg{spatstat} library, a point pattern dataset must include
  information about the window or region in which the pattern was
  observed. A window is described by an object of class \code{"owin"}.
  Windows of arbitrary shape are supported.
  
  An object of class \code{"owin"} has one of three types:
  \tabular{ll}{
    \code{"rectangle"}: \tab
      a rectangle in the two-dimensional plane with edges parallel to the axes
    \cr
    \code{"polygonal"}: \tab
    a region whose boundary is a polygon or several polygons.
    The region may have holes and may consist of several disconnected pieces.
    \cr
    \code{"mask"}: \tab
      a binary image (a logical matrix)
      set to \code{TRUE} for pixels inside the window and
      \code{FALSE} outside the window.
  }
  Objects of class \code{"owin"} may be created by the function
  \code{\link{owin}}
  and converted from other types of data by the function
  \code{\link{as.owin}}.

  They may be manipulated by the functions 
  \code{\link{as.rectangle}},
  \code{\link{as.mask}}, 
  \code{\link{complement.owin}},
  \code{\link{rotate}},
  \code{\link{shift}},
  \code{\link{affine}},
  \code{\link{erosion}},
  \code{\link{dilation}},
  \code{\link{opening}}
  and
  \code{\link{closing}}.

  Geometrical calculations available for windows include
  \code{\link{area.owin}},
  \code{\link{perimeter}},
  \code{\link{diameter.owin}},
  \code{\link{boundingbox}},
  \code{\link{eroded.areas}},
  \code{\link{bdist.points}},
  \code{\link{bdist.pixels}},
  and
  \code{even.breaks.owin}.
  The mapping between continuous coordinates and pixel raster indices
  is facilitated by the functions
  \code{\link{raster.x}},
  \code{\link{raster.y}} and
  \code{\link{nearest.raster.point}}.

  There is a \code{plot} method for window objects,
  \code{\link{plot.owin}}. This may be useful if you wish to
  plot a point pattern's window without the points for graphical
  purposes.

  There are also methods for
  \code{summary} and \code{print}. 
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{as.rectangle}},
  \code{\link{as.mask}},
  \code{\link{summary.owin}},
  \code{\link{print.owin}},
  \code{\link{complement.owin}},
  \code{\link{erosion}},
  \code{\link{dilation}},
  \code{\link{opening}},
  \code{\link{closing}},
  \code{\link{affine.owin}},
  \code{\link{shift.owin}},
  \code{\link{rotate.owin}},
  \code{\link{raster.x}},
  \code{\link{raster.y}},
  \code{\link{nearest.raster.point}},
  \code{\link{plot.owin}},
  \code{\link{area.owin}},
  \code{\link{boundingbox}},
  \code{\link{diameter}},
  \code{\link{eroded.areas}},
  \code{\link{bdist.points}},
  \code{\link{bdist.pixels}}
}
\section{Warnings}{
  In a window of type \code{"mask"}, the 
  row index corresponds to increasing \eqn{y} coordinate,
  and the column index corresponds to increasing \eqn{x} coordinate.
}
\examples{
 w <- owin()
 w <- owin(c(0,1), c(0,1))
 # the unit square
  
 w <- owin(c(0,1), c(0,2))
 \dontrun{
 if(FALSE) {
   plot(w)
   # plots edges of a box 1 unit x 2 units
   v <- locator() 
   # click on points in the plot window
   # to be the vertices of a polygon 
   # traversed in anticlockwise order 
   u <- owin(c(0,1), c(0,2), poly=v)
   plot(u)
   # plots polygonal boundary using polygon()
   plot(as.mask(u, eps=0.02))
   # plots discrete pixel approximation to polygon
 }
}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{attribute}
 
 

\name{compatible}
\alias{compatible}
\title{Test Whether Objects Are Compatible}
\description{
  Tests whether two or more objects of the same class 
  are compatible.
}
\usage{
  compatible(A, B, \dots)
}
\arguments{
  \item{A,B,\dots}{Two or more objects of the same class}
}
\details{
  This generic function is used to check whether the
  objects \code{A} and \code{B} (and any additional
  objects \code{\dots}) are compatible.

  What is meant by \sQuote{compatible} depends on the class of object.

  There are methods for the classes \code{"fv"}, \code{"fasp"},
  \code{"im"} and \code{"unitname"}.
}
\value{
  Logical value: \code{TRUE} if the objects are compatible, and \code{FALSE}
  if they are not.
}
\seealso{
  \code{\link{compatible.fv}},
  \code{\link{compatible.fasp}},
  \code{\link{compatible.im}},
  \code{\link{compatible.unitname}} 
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{endpoints.psp}
\alias{endpoints.psp}
\title{Endpoints of Line Segment Pattern}
\description{
  Extracts the endpoints of each line segment
  in a line segment pattern.
}
\usage{
  endpoints.psp(x, which="both")
}
\arguments{
  \item{x}{
    A line segment pattern (object of class \code{"psp"}).
  }
  \item{which}{
    String specifying which endpoint or endpoints should be
    returned. See Details.
  }
}
\value{
  Point pattern (object of class \code{"ppp"}).
}
\details{
  This function extracts one endpoint, or both endpoints,
  from each of the line segments in \code{x},
  and returns these points as a point pattern object.
  
  The argument \code{which} determines which endpoint or endpoints
  of each line segment should be returned:
  \describe{
    \item{\code{which="both"}}{
      (the default): both endpoints
      of each line segment are returned. The result is a point pattern
      with twice as many points as there are line segments in \code{x}.
    }
    \item{\code{which="first"}}{
      select the first endpoint
      of each line segment (returns the points with coordinates
      \code{x$ends$x0, x$ends$y0}).
    }
    \item{\code{which="second"}}{
      select the second endpoint
      of each line segment (returns the points with coordinates
      \code{x$ends$x1, x$ends$y1}).
    }
    \item{\code{which="left"}}{
      select the left-most endpoint
      (the endpoint with the smaller \eqn{x} coordinate)
      of each line segment.
    }
    \item{\code{which="right"}}{
      select the right-most endpoint
      (the endpoint with the greater \eqn{x} coordinate)
      of each line segment.
    }
    \item{\code{which="lower"}}{
      select the lower endpoint
      (the endpoint with the smaller \eqn{y} coordinate)
      of each line segment.
    }
    \item{\code{which="upper"}}{
      select the upper endpoint
      (the endpoint with the greater \eqn{y} coordinate)
      of each line segment.
    }
  }
  The result is a point pattern. It also has an attribute
  \code{"id"} which is an integer vector identifying
  the segment which contributed each point.
}
\seealso{
  \code{\link{psp.object}},
  \code{\link{ppp.object}},
  \code{\link{marks.psp}},
  \code{\link{summary.psp}},
  \code{\link{midpoints.psp}},
  \code{\link{lengths_psp}},
  \code{\link{angles.psp}},
  \code{\link{extrapolate.psp}}.
}
\examples{
  a <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  plot(a)
  b <- endpoints.psp(a, "left")
  plot(b, add=TRUE)
}
\author{
  \adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{run.simplepanel}
\alias{clear.simplepanel}
\alias{redraw.simplepanel}
\alias{run.simplepanel}
\title{
  Run Point-and-Click Interface
}
\description{
  Execute various operations in a simple
  point-and-click user interface.
}
\usage{
run.simplepanel(P, popup=TRUE, verbose = FALSE)
clear.simplepanel(P)
redraw.simplepanel(P, verbose = FALSE)
}
\arguments{
  \item{P}{
    An interaction panel (object of class \code{"simplepanel"},
    created by \code{\link{simplepanel}} or \code{\link{grow.simplepanel}}).
  }
  \item{popup}{
    Logical. If \code{popup=TRUE} (the default),
    the panel will be displayed in a
    new popup window. If \code{popup=FALSE},
    the panel will be displayed on the current graphics
    window if it already exists, and on a new window otherwise.
  }
  \item{verbose}{
    Logical. If \code{TRUE}, debugging information will be printed.
  }
}
\details{
  These commands enable the user to run a simple, robust,
  point-and-click interface to any \R code.
  The interface is implemented
  using only the basic graphics package in \R.

  The argument \code{P} is an object of class \code{"simplepanel"},
  created by \code{\link{simplepanel}} or \code{\link{grow.simplepanel}},  
  which specifies the graphics to be displayed and the actions to be performed
  when the user interacts with the panel.

  The command \code{run.simplepanel(P)} activates the panel:
  the display is initialised and the graphics system waits for the
  user to click the panel. 
  While the panel is active, the user can only interact with the panel;
  the \R command line interface and the \R GUI cannot be used.
  When the panel terminates (typically because the user
  clicked a button labelled Exit), control returns to the \R command line
  interface and the \R GUI. 

  The command \code{clear.simplepanel(P)} clears all the display
  elements in the panel, resulting in a blank display except for the
  title of the panel.

  The command \code{redraw.simplepanel(P)} redraws all the buttons
  of the panel, according to the \code{redraw} functions contained
  in the panel.

  If \code{popup=TRUE} (the default), \code{run.simplepanel} begins by
  calling \code{\link[grDevices]{dev.new}} so that a new popup window
  is created; this window is closed using
  \code{\link[grDevices]{dev.off}} when \code{run.simplepanel} terminates.
  If \code{popup=FALSE}, the panel will be displayed on the current graphics
  window if it already exists, and on a new window otherwise;
  this window is not closed when \code{run.simplepanel} terminates.

  For more sophisticated control of the graphics focus (for example, to
  use the panel to control the display on another window),
  initialise the graphics devices yourself using
  \code{\link[grDevices]{dev.new}} or similar commands; save these devices
  in the shared environment \code{env} of the panel \code{P};
  and write the click/redraw functions of \code{P} in such a way that
  they access these devices using \code{\link[grDevices]{dev.set}}.
  Then use \code{run.simplepanel} with \code{popup=FALSE}.
}
\value{
  The return value of \code{run.simplepanel(P)} is the value returned
  by the \code{exit} function of \code{P}. See \code{\link{simplepanel}}.

  The functions \code{clear.simplepanel} and \code{redraw.simplepanel}
  return \code{NULL}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{simplepanel}}
}
\examples{
  if(interactive()) {
    # make boxes (alternatively use layout.boxes())
    Bminus <- square(1)
    Bvalue <- shift(Bminus, c(1.2, 0))
    Bplus <- shift(Bvalue, c(1.2, 0))
    Bdone <- shift(Bplus, c(1.2, 0))
    myboxes <- list(Bminus, Bvalue, Bplus, Bdone)
    myB <- do.call(boundingbox,myboxes)

    # make environment containing an integer count
    myenv <- new.env()
    assign("answer", 0, envir=myenv)

    # what to do when finished: return the count.
    myexit <- function(e) { return(get("answer", envir=e)) }

    # button clicks
    # decrement the count
    Cminus <- function(e, xy) {
     ans <- get("answer", envir=e)
     assign("answer", ans - 1, envir=e)
     return(TRUE)
   }
   # display the count (clicking does nothing)
   Cvalue <- function(...) { TRUE }
   # increment the count
   Cplus <- function(e, xy) {
    ans <- get("answer", envir=e)
    assign("answer", ans + 1, envir=e)
    return(TRUE)
   }
   # quit button
   Cdone <- function(e, xy) { return(FALSE) }

   myclicks <- list("-"=Cminus,
                    value=Cvalue,
                    "+"=Cplus,
                    done=Cdone)

   # redraw the button that displays the current value of the count
   Rvalue <- function(button, nam, e) {
     plot(button, add=TRUE)
     ans <- get("answer", envir=e)
     text(centroid.owin(button), labels=ans)
     return(TRUE)
  }

  # make the panel
  P <- simplepanel("Counter",
                   B=myB, boxes=myboxes,
                   clicks=myclicks,
                   redraws = list(NULL, Rvalue, NULL, NULL),
                   exit=myexit, env=myenv)
  P

  run.simplepanel(P)
  }
}
\keyword{iplot}
\keyword{utilities}

\name{scalardilate} 
\alias{scalardilate}
\alias{scalardilate.im}
\alias{scalardilate.owin}
\alias{scalardilate.ppp}
\alias{scalardilate.psp}
\alias{scalardilate.default}
\title{Apply Scalar Dilation}
\description{
  Applies scalar dilation to a plane geometrical object,
  such as a point pattern or a window,
  relative to a specified origin.
}
\usage{
  scalardilate(X, f, \dots)

  \method{scalardilate}{im}(X, f, \dots, origin=NULL)

  \method{scalardilate}{owin}(X, f, \dots, origin=NULL)

  \method{scalardilate}{ppp}(X, f, \dots, origin=NULL)

  \method{scalardilate}{psp}(X, f, \dots, origin=NULL)

  \method{scalardilate}{default}(X, f, \dots)
}
\arguments{
  \item{X}{Any suitable dataset representing a two-dimensional
    object, such as a point pattern (object of class \code{"ppp"}),
    a window (object of class \code{"owin"}), a pixel image
    (class \code{"im"}) and so on.
  }
  \item{f}{
    Scalar dilation factor. A finite number greater than zero.
  }
  \item{\dots}{Ignored by the methods.}
  \item{origin}{
    Origin for the scalar dilation. Either a vector of 2 numbers,
    or one of the character strings
    \code{"centroid"}, \code{"midpoint"},
    \code{"left"}, \code{"right"}, \code{"top"}, \code{"bottom"},
    \code{"topleft"}, \code{"bottomleft"}, \code{"topright"} or
    \code{"bottomright"} (partially matched).
  }
}
\value{
  Another object of the same type, representing the
  result of applying the scalar dilation.
}
\details{
  This command performs scalar dilation of the object \code{X}
  by the factor \code{f} relative to the origin specified by
  \code{origin}. 

  The function \code{scalardilate} is generic, with methods for
  windows (class \code{"owin"}), point patterns (class \code{"ppp"}),
  pixel images (class \code{"im"}), line segment patterns (class \code{"psp"})
  and a default method.

  If the argument \code{origin} is not given,
  then every spatial coordinate is multiplied by the factor \code{f}.

  If \code{origin} is given, then scalar dilation is performed
  relative to the specified origin. Effectively, \code{X} is shifted
  so that \code{origin} is moved to \code{c(0,0)}, then scalar dilation
  is performed, then the result is shifted so that \code{c(0,0)} is
  moved to \code{origin}.

  This command is a special case of an affine transformation:
  see \code{\link{affine}}.
}
\seealso{
  \code{\link{affine}},
  \code{\link{shift}}
}
\examples{
plot(letterR)
plot(scalardilate(letterR, 0.7, origin="left"), col="red", add=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{as.data.frame.owin}
\alias{as.data.frame.owin}
\title{Convert Window to Data Frame}
\description{
  Converts a window object to a data frame.
}
\usage{
\method{as.data.frame}{owin}(x, \dots, drop=TRUE)
}
\arguments{
  \item{x}{
    Window (object of class \code{"owin"}).
  }
  \item{\dots}{Further arguments passed to
    \code{\link[base:as.data.frame]{as.data.frame.default}}
    to determine the row names and other features.
  }
  \item{drop}{
    Logical value indicating whether to discard pixels that are
    outside the window, when \code{x} is a binary mask.
  }
}
\details{
  This function returns a data frame specifying the coordinates of the
  window.
  
  If \code{x} is a binary mask window,
  the result is a data frame with columns \code{x} and \code{y}
  containing the spatial coordinates of each \emph{pixel}.
  If \code{drop=TRUE} (the default), only pixels inside the window are retained.
  If \code{drop=FALSE}, all pixels are retained, and the data frame has
  an extra column \code{inside} containing the logical value of each pixel
  (\code{TRUE} for pixels inside the window, \code{FALSE} for outside).
  
  If \code{x} is a rectangle or a polygonal window,
  the result is a data frame with columns \code{x} and \code{y}
  containing the spatial coordinates of the \emph{vertices} of the
  window. If the boundary consists of several polygons, the data frame
  has additional columns \code{id}, identifying which polygon is being
  traced, and \code{sign}, indicating whether the polygon is an
  outer or inner boundary (\code{sign=1} and \code{sign=-1}
  respectively).
}
\value{
  A data frame with columns named \code{x} and \code{y},
  and possibly other columns.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{as.data.frame.im}},
  \code{\link{as.owin.data.frame}}
}
\examples{
   as.data.frame(square(1))

   holey <- owin(poly=list(
                        list(x=c(0,10,0), y=c(0,0,10)),
                        list(x=c(2,2,4,4), y=c(2,4,4,2))))
   as.data.frame(holey)

   M <- as.mask(holey, eps=0.5)
   Mdf <- as.data.frame(M)
}
\keyword{spatial}
\keyword{methods}

\name{integral.im}
\alias{integral}
\alias{integral.im}
\title{
  Integral of a Pixel Image
}
\description{
  Computes the integral of a pixel image.
}
\usage{
integral(f, domain=NULL, \dots)

\method{integral}{im}(f, domain=NULL, \dots)
}
\arguments{
  \item{f}{
    A pixel image (object of class \code{"im"}) with pixel values
    that can be treated as numeric or complex values.
  }
  \item{domain}{
    Optional. Window specifying the domain of integration.
    Alternatively a tessellation.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  The function \code{integral} is generic, with methods
  for \code{"im"}, \code{"msr"}, \code{"linim"} and \code{"linfun"}.
  
  The method \code{integral.im} treats the pixel image \code{f} as a function of
  the spatial coordinates, and computes its integral.
  The integral is calculated
  by summing the pixel values and multiplying by the area of one pixel.

  The pixel values of \code{f} may be numeric, integer, logical or
  complex. They cannot be factor or character values.

  The logical values \code{TRUE} and \code{FALSE} are converted to
  \code{1} and \code{0} respectively, so that the integral of a logical
  image is the total area of the \code{TRUE} pixels, in the same units
  as \code{unitname(x)}.

  If \code{domain} is a window (class \code{"owin"}) then the integration
  will be restricted to this window. If \code{domain} is a tessellation
  (class \code{"tess"}) then the integral of \code{f} in each
  tile of \code{domain} will be computed.
}
\value{
  A single numeric or complex value (or a vector of such values
  if \code{domain} is a tessellation).
}
\seealso{
  \code{\link{eval.im}},
  \code{\link{[.im}}
}
\examples{
   # approximate integral of f(x,y) dx dy
   f <- function(x,y){3*x^2 + 2*y}
   Z <- as.im(f, square(1))
   integral.im(Z)
   # correct answer is 2

   D <- density(cells)
   integral.im(D)
   # should be approximately equal to number of points = 42

   # integrate over the subset [0.1,0.9] x [0.2,0.8]
   W <- owin(c(0.1,0.9), c(0.2,0.8))
   integral.im(D, W)
}
\author{
\adrian


\rolf

and \ege

}
\keyword{spatial}
\keyword{math}

\name{stratrand}
\alias{stratrand}
\title{Stratified random point pattern}
\description{
  Generates a \dQuote{stratified random} pattern of points in a window,
  by dividing the window into rectangular tiles and placing
  \code{k} random points in each tile.
}
\usage{
 stratrand(window, nx, ny, k = 1)
}
\arguments{
  \item{window}{A window. 
    An object of class \code{\link{owin}},
    or data in any format acceptable to \code{\link{as.owin}()}.
  }
  \item{nx}{Number of tiles in each row.
  }
  \item{ny}{Number of tiles in each column.
  }
  \item{k}{Number of random points to generate in each tile.
  }
}
\value{
  A list with two components \code{x} and \code{y}, which are numeric
  vectors giving the coordinates of the random points.
}
\details{
  The bounding rectangle of \code{window} is divided into
  a regular \eqn{nx \times ny}{nx * ny} grid of rectangular tiles.
  In each tile, \code{k} random points are generated independently
  with a uniform distribution in that tile. 

  Note that some of these grid points may lie outside the window,
  if \code{window} is not of type \code{"rectangle"}. The function
  \code{\link{inside.owin}} can be used to select those grid points
  which do lie inside the window. See the examples.

  This function is useful in creating dummy points for quadrature
  schemes (see \code{\link{quadscheme}}) as well as in simulating
  random point patterns.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{quadscheme}},
  \code{\link{inside.owin}},
  \code{\link{gridcentres}}
}
\examples{
  w <- unit.square()
  xy <- stratrand(w, 10, 10)
  \dontrun{
  plot(w)
  points(xy)
  }

  # polygonal boundary
  bdry <- list(x=c(0.1,0.3,0.7,0.4,0.2),
               y=c(0.1,0.1,0.5,0.7,0.3))
  w <- owin(c(0,1), c(0,1), poly=bdry)
  xy <- stratrand(w, 10, 10, 3)
  \dontrun{
  plot(w)
  points(xy)
  }
  # determine which grid points are inside polygon
  ok <- inside.owin(xy$x, xy$y, w)
  \dontrun{
  plot(w)
  points(xy$x[ok], xy$y[ok])
  }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{diameter.box3} 
\Rdversion{1.1}
\alias{diameter.box3}
\alias{volume.box3}
\alias{shortside.box3}
\alias{sidelengths.box3}
\alias{eroded.volumes.box3}
\alias{shortside}
\alias{sidelengths}
\alias{eroded.volumes}
\title{
  Geometrical Calculations for Three-Dimensional Box
}
\description{
  Calculates the volume, diameter, shortest side, side lengths, 
  or eroded volume of a three-dimensional box.
}
\usage{
\method{diameter}{box3}(x)

\method{volume}{box3}(x)

shortside(x)
sidelengths(x)
eroded.volumes(x, r)

\method{shortside}{box3}(x)

\method{sidelengths}{box3}(x)

\method{eroded.volumes}{box3}(x, r)
}
\arguments{
  \item{x}{
    Three-dimensional box (object of class \code{"box3"}).
  }
  \item{r}{
    Numeric value or vector of numeric values for which eroded volumes
    should be calculated.
  }
}
\details{
  \code{diameter.box3} computes the diameter of the box.
  \code{volume.box3} computes the volume of the box.
  \code{shortside.box3} finds the shortest of the three side lengths
  of the box.
  \code{sidelengths.box3} returns all three side lengths
  of the box.

  \code{eroded.volumes} computes, for each entry \code{r[i]},
  the volume of the smaller box obtained by removing a slab of
  thickness \code{r[i]} from each face of the box. This smaller box is
  the subset consisting of points that lie at least \code{r[i]} units
  away from the boundary of the box.
}
\value{
  For \code{diameter.box3}, \code{shortside.box3} and
  \code{volume.box3}, a single numeric
  value. For \code{sidelengths.box3}, a vector of three numbers.
  For \code{eroded.volumes}, a numeric vector of the same length
  as \code{r}.  
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{as.box3}}
}
\examples{
    X <- box3(c(0,10),c(0,10),c(0,5))
    diameter(X) 
    volume(X)
    sidelengths(X)
    shortside(X)
    hd <- shortside(X)/2
    eroded.volumes(X, seq(0,hd, length=10))
}
\keyword{spatial}
\keyword{math}

\name{dmixpois}
\alias{dmixpois}
\alias{pmixpois}
\alias{qmixpois}
\alias{rmixpois}
\title{
  Mixed Poisson Distribution
}
\description{
  Density, distribution function, quantile function and random
  generation for a mixture of Poisson distributions.
}
\usage{
dmixpois(x, mu, sd, invlink = exp, GHorder = 5)
pmixpois(q, mu, sd, invlink = exp, lower.tail = TRUE, GHorder = 5)
qmixpois(p, mu, sd, invlink = exp, lower.tail = TRUE, GHorder = 5)
rmixpois(n, mu, sd, invlink = exp)
}
\arguments{
  \item{x}{vector of (non-negative integer) quantiles.}
  \item{q}{vector of quantiles.}
  \item{p}{vector of probabilities.}
  \item{n}{number of random values to return.}
  \item{mu}{
    Mean of the linear predictor. A single numeric value.
  }
  \item{sd}{
    Standard deviation of the linear predictor. A single numeric value.
  }
  \item{invlink}{
    Inverse link function. A function in the \R language,
    used to transform the linear predictor into the
    parameter \code{lambda} of the Poisson distribution.
  }
  \item{lower.tail}{
    Logical. If \code{TRUE} (the default), probabilities are
    \eqn{P[X \le x]}, otherwise, \eqn{P[X > x]}.
  }
  \item{GHorder}{
    Number of quadrature points in the Gauss-Hermite quadrature approximation.
    A small positive integer.
  }
}
\details{
  These functions are analogous to
  \code{\link{dpois}}
  \code{\link{ppois}},
  \code{\link{qpois}} and
  \code{\link{rpois}}
  except that they apply to a mixture of Poisson distributions.

  In effect, the Poisson mean parameter \code{lambda} is randomised
  by setting \code{lambda = invlink(Z)} where \code{Z}
  has a Gaussian \eqn{N(\mu,\sigma^2)}{N(\mu, \sigma^2)} distribution.
  The default is \code{invlink=exp} which means that
  \code{lambda} is lognormal. Set \code{invlink=I} to assume
  that \code{lambda} is approximately Normal.

  For \code{dmixpois}, \code{pmixpois} and \code{qmixpois},
  the probability distribution is approximated using Gauss-Hermite
  quadrature. For \code{rmixpois}, the deviates are simulated
  exactly.
}
\value{
  Numeric vector:
  \code{dmixpois} gives probability masses,
  \code{ppois} gives cumulative probabilities,
  \code{qpois} gives (non-negative integer) quantiles, and
  \code{rpois} generates (non-negative integer) random deviates.
}
\seealso{
  \code{\link{dpois}},
  \code{\link{gauss.hermite}}.
}
\examples{
  dmixpois(7, 10, 1, invlink = I)
  dpois(7, 10)

  pmixpois(7, log(10), 0.2)
  ppois(7, 10)

  qmixpois(0.95, log(10), 0.2)
  qpois(0.95, 10)

  x <- rmixpois(100, log(10), log(1.2))
  mean(x)
  var(x)
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege
}
\keyword{distribution}

\name{spokes}
\alias{spokes}
\title{Spokes pattern of dummy points}
\description{
  Generates a pattern of dummy points in a window,
  given a data point pattern.
  The dummy points lie on the radii of circles emanating from each
  data point.
}
\usage{
 spokes(x, y, nrad = 3, nper = 3, fctr = 1.5, Mdefault = 1)
}
\arguments{
  \item{x}{
    Vector of \eqn{x} coordinates of data points, or a list
    with components \code{x} and \code{y}, or a point pattern
    (an object of class \code{ppp}).
  }
  \item{y}{
    Vector of \eqn{y} coordinates of data points.  Ignored
    unless \code{x} is a vector.
  }
  \item{nrad}{
    Number of radii emanating from each data point.
  }
  \item{nper}{
    Number of dummy points per radius.
  }
  \item{fctr}{
    Scale factor.
    Length of largest spoke radius is \code{fctr * M}
    where \code{M} is the mean nearest neighbour distance
    for the data points.
  }
  \item{Mdefault}{
    Value of \code{M} to be used if \code{x} has length 1.
  }
}
\value{
  If argument \code{x} is a point pattern, a point pattern with
  window equal to that of \code{x}.  Otherwise a list with two
  components \code{x} and \code{y}.  In either case the components
  \code{x} and \code{y} of the value are numeric vectors giving
  the coordinates of the dummy points.
}
\details{
  This function is useful in creating dummy points for quadrature
  schemes (see \code{\link{quadscheme}}).

  Given the data points, the function creates a collection of
  \code{nrad * nper * length(x)} dummy points.

  Around each data point \code{(x[i],y[i])} there are
  \code{nrad * nper} dummy points, lying on \code{nrad} radii
  emanating from \code{(x[i],y[i])}, with \code{nper} dummy points
  equally spaced along each radius.

  The (equal) spacing of dummy points along each radius is
  controlled by the factor \code{fctr}. 
  The distance from a data point to the furthest of its associated
  dummy points is \code{fctr * M}
  where \code{M} is the mean nearest neighbour distance
  for the data points.

  If there is only one data point the nearest neighbour distance
  is infinite, so the value \code{Mdefault} will be used in place
  of \code{M}.

  If \code{x} is a point pattern, then the value returned is
  also a point pattern, which is clipped to the window
  of \code{x}.  Hence there may be fewer than
  \code{nrad * nper * length(x)} dummy points in the pattern
  returned.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{quadscheme}},
  \code{\link{inside.owin}},
  \code{\link{gridcentres}},
  \code{\link{stratrand}}
}
\examples{
  dat <- runifrect(10)
  dum <- spokes(dat$x, dat$y, 5, 3, 0.7)
  plot(dum)
  Q <- quadscheme(dat, dum, method="dirichlet")
  plot(Q, tiles=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{bw.pcf}
\alias{bw.pcf}
\title{
  Cross Validated Bandwidth Selection for Pair Correlation Function
}
\description{
  Uses composite likelihood or generalized least squares 
  cross-validation to select a smoothing bandwidth
  for the kernel estimation of pair correlation function.
}
\usage{
  bw.pcf(X, rmax=NULL, lambda=NULL, divisor="r", 
         kernel="epanechnikov", nr=10000, bias.correct=TRUE, 
         cv.method=c("compLik", "leastSQ"), simple=TRUE, srange=NULL,
	 \dots, verbose=FALSE, warn=TRUE)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"}).
  }
  \item{rmax}{
    Numeric. Maximum value of the spatial lag distance \eqn{r} 
    for which \eqn{g(r)} should be evaluated.
  }
  \item{lambda}{
    Optional.
    Values of the estimated intensity function.
    A vector giving the intensity values
    at the points of the pattern \code{X}.
  }
  \item{divisor}{
    Choice of divisor in the estimation formula:
    either \code{"r"} (the default) or \code{"d"}. 
    See \code{pcf.ppp}.
  }
  \item{kernel}{
    Choice of smoothing kernel, passed to \code{density}; 
    see \code{\link{pcf}} and \code{\link{pcfinhom}}.
  }
  \item{nr}{
    Integer. Number of subintervals for discretization of 
    [0, rmax] to use in computing numerical integrals.
  }
  \item{bias.correct}{
    Logical. Whether to use bias corrected version of the kernel 
    estimate. See Details.
  }
  \item{cv.method}{
    Choice of cross validation method: either
    \code{"compLik"} or \code{"leastSQ"} (partially matched).
  }
  \item{simple}{
    Logical. Whether to use simple removal of spatial lag 
    distances. See Details.
  }
  \item{srange}{
    Optional. Numeric vector of length 2 giving the range of
    bandwidth values that should be searched to find the optimum
    bandwidth.
  }
  \item{\dots}{
    Other arguments, passed to \code{\link{pcf}} or 
    \code{\link{pcfinhom}}.
  }
  \item{verbose}{
    Logical value indicating whether to print progress reports
    during the optimization procedure.
  }
  \item{warn}{
    Logical. If \code{TRUE}, issue a warning if the optimum value of
    the cross-validation criterion occurs at one of the ends of the
    search interval.
  }
}
\details{
  This function selects an appropriate bandwidth \code{bw}
  for the kernel estimator of the pair correlation function 
  of a point process intensity computed by \code{\link{pcf.ppp}} 
  (homogeneous case) or \code{\link{pcfinhom}} 
  (inhomogeneous case).

  With \code{cv.method="leastSQ"}, the bandwidth 
  \eqn{h} is chosen to minimise an unbiased 
  estimate of the integrated mean-square error criterion 
  \eqn{M(h)} defined in equation (4) in Guan (2007a).
  The code implements the fast algorithm of Jalilian and Waagepetersen
  (2018).
    
  With \code{cv.method="compLik"}, the bandwidth 
  \eqn{h} is chosen to maximise a likelihood 
  cross-validation criterion \eqn{CV(h)} defined in 
  equation (6) of Guan (2007b).
  
  \deqn{
    M(b) = \frac{\mbox{MSE}(\sigma)}{\lambda^2} - g(0)
  }{
    M(b) = \int_{0}^{rmax} \hat{g}^2(r;b) r dr - \sum_{u,v}
  }

  The result is a numerical value giving the selected bandwidth.
}
\section{Definition of bandwidth}{
  The bandwidth \code{bw} returned by \code{bw.pcf}
  corresponds to the standard deviation of the smoothoing 
  kernel. As mentioned in the documentation of 
  \code{\link{density.default}} and \code{\link{pcf.ppp}}, 
  this differs from the scale parameter \code{h} of 
  the smoothing kernel which is often considered in the 
  literature as the bandwidth of the kernel function.
  For example for the Epanechnikov kernel, \code{bw=h/sqrt(h)}.
}
\value{
  A numerical value giving the selected bandwidth.
  The result also belongs to the class \code{"bw.optim"}
  which can be plotted.
}
\seealso{
  \code{\link{pcf.ppp}},
  \code{\link{pcfinhom}}
}
\examples{
  b <- bw.pcf(redwood)
  plot(pcf(redwood, bw=b))
}
\references{
  Guan, Y. (2007a). 
  A composite likelihood cross-validation approach in selecting 
  bandwidth for the estimation of the pair correlation function. 
  \emph{Scandinavian Journal of Statistics}, 
  \bold{34}(2), 336--346.
  
  Guan, Y. (2007b). 
  A least-squares cross-validation bandwidth selection approach 
  in pair correlation function estimations. 
  \emph{Statistics & Probability Letters}, 
  \bold{77}(18), 1722--1729.

  Jalilian, A. and Waagepetersen, R. (2018)
  Fast bandwidth selection for estimation of the pair correlation
  function.
  \emph{Journal of Statistical Computation and Simulation},
  \bold{88}(10), 2001--2011.
  \url{https://www.tandfonline.com/doi/full/10.1080/00949655.2018.1428606}
}
\author{
  Rasmus Waagepetersen and Abdollah Jalilian. 
  Adapted for \pkg{spatstat} by \spatstatAuthors.
}
\keyword{spatial}
\keyword{methods}
\keyword{smooth}

\name{rmhstart}
\alias{rmhstart}
\alias{rmhstart.default}
\title{Determine Initial State for Metropolis-Hastings Simulation.}
\description{
  Builds a description of the initial state
  for the Metropolis-Hastings
  algorithm. 
}
\usage{
   rmhstart(start, \dots)
   \method{rmhstart}{default}(start=NULL, \dots, n.start=NULL, x.start=NULL)
}
\arguments{
  \item{start}{An existing description of the initial state in some
    format. Incompatible with the arguments listed below.
  }
  \item{\dots}{There should be no other arguments.}
  \item{n.start}{
    Number of initial points (to be randomly generated).
    Incompatible with \code{x.start}.
  }
  \item{x.start}{
    Initial point pattern configuration.
    Incompatible with \code{n.start}.
  }
}
\value{
  An object of class \code{"rmhstart"}, which is essentially
  a list of parameters describing the initial point pattern
  and (optionally) the initial state of the random number generator.
  
  There is a \code{print} method for this class, which prints
  a sensible description of the initial state.
}
\details{
  Simulated realisations of many point process models
  can be generated using the Metropolis-Hastings algorithm
  implemented in \code{\link{rmh}}.
  
  This function \code{rmhstart}
  creates a full description of the initial state of the
  Metropolis-Hastings algorithm,
  \emph{including possibly the initial state of the random number generator},
  for use in a subsequent call to \code{\link{rmh}}. It also
  checks that the initial state is valid.

  The initial state should be specified \bold{either} by the
  first argument \code{start} \bold{or} by the other arguments
  \code{n.start}, \code{x.start} etc.
  
  If \code{start} is a list, then it should have components named 
  \code{n.start} or \code{x.start},
  with the same interpretation as described below.

  The arguments are:
    \describe{
      \item{n.start}{
	The number of \dQuote{initial} points to be randomly
	(uniformly) generated in the simulation window \code{w}.
	Incompatible with \code{x.start}.

	For a multitype point process, \code{n.start} may be a vector
	(of length equal to the number of types) giving the number
	of points of each type to be generated.  

	If expansion of the simulation window is selected (see the argument
	\code{expand} to \code{\link{rmhcontrol}}),
	then the actual number of starting points in the simulation
	will be \code{n.start} multiplied by the expansion factor
	(ratio of the areas of the expanded window and original window).

	For faster convergence of the Metropolis-Hastings algorithm,
	the value of \code{n.start} should be roughly equal to
	(an educated guess at) the expected number of points
	for the point process inside the window.
      }
      \item{x.start}{
	Initial point pattern configuration. Incompatible with
	\code{n.start}.

	\code{x.start} may be a point pattern (an object
	of class \code{ppp}), or an object which can be coerced
	to this class by \code{\link{as.ppp}}, or a dataset containing
	vectors \code{x} and \code{y}.  

	If \code{x.start} is specified, then expansion of the
	simulation window (the argument \code{expand}
	of \code{\link{rmhcontrol}}) is not permitted.
      }
    }
    The parameters \code{n.start} and \code{x.start} are
    \emph{incompatible}.
}  
\seealso{
  \code{\link{rmh}},
  \code{\link{rmhcontrol}},
  \code{\link{rmhmodel}}
}
\examples{
   # 30 random points
   a <- rmhstart(n.start=30)
   a

   # a particular point pattern
   b <- rmhstart(x.start=cells)
}
\author{
  \adrian
  and \rolf
}
\keyword{spatial}
\keyword{datagen}

\name{nearestsegment}
\alias{nearestsegment}
\title{Find Line Segment Nearest to Each Point}
\description{
  Given a point pattern and a line segment pattern,
  this function finds the nearest line segment for each point.
}
\usage{
nearestsegment(X, Y)
}
\arguments{
  \item{X}{A point pattern (object of class \code{"ppp"}).}
  \item{Y}{A line segment pattern (object of class \code{"psp"}).}
}
\details{
  The distance between a point \code{x} and a straight line segment \code{y} is
  defined to be the shortest Euclidean distance between \code{x} and any
  location on \code{y}. This algorithm first calculates the distance
  from each point of \code{X} to
  each segment of \code{Y}. Then it determines, for each point \code{x} in
  \code{X}, which segment of \code{Y} is closest. The index of this
  segment is returned.
}
\value{
  Integer vector \code{v} (of length equal to the number of points in
  \code{X}) identifying the nearest segment to each point.
  If \code{v[i] = j}, then
  \code{Y[j]} is the line segment lying closest to \code{X[i]}.
}
\author{
  \adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{project2segment}} to project each point of \code{X} to
  a point lying on one of the line segments.

  Use \code{\link{distmap.psp}} to 
  identify the nearest line segment for each pixel in a grid.
}
\examples{
  X <- runifpoint(3)
  Y <- as.psp(matrix(runif(20), 5, 4), window=owin())
  v <- nearestsegment(X,Y)
  plot(Y)
  plot(X, add=TRUE)
  plot(X[1], add=TRUE, col="red")
  plot(Y[v[1]], add=TRUE, lwd=2, col="red") 
}
\keyword{spatial}
\keyword{math}

\name{test.crossing.psp}
\alias{test.crossing.psp}
\alias{test.selfcrossing.psp}
\title{
  Check Whether Segments Cross
}
\description{
  Determine whether there is a crossing (intersection) between
  each pair of line segments.
}
\usage{
test.crossing.psp(A, B)
test.selfcrossing.psp(A)
}
\arguments{
  \item{A,B}{
    Line segment patterns (objects of class \code{"psp"}).
  }
}
\details{
  These functions decide whether the given line segments intersect
  each other.

  If \code{A} and \code{B} are two spatial patterns of line segments,
  \code{test.crossing.psp(A, B)}
  returns a logical matrix in which the entry on row \code{i}, column
  \code{j} is equal to \code{TRUE} if segment \code{A[i]}
  has an intersection with segment \code{B[j]}.

  If \code{A} is a pattern of line segments, \code{test.selfcross.psp(A)}
  returns a symmetric logical matrix in which the entry on row \code{i}, column
  \code{j} is equal to \code{TRUE} if segment \code{A[i]}
  has an intersection with segment \code{A[j]}.
}
\value{
  A logical matrix.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{psp}}
}
\examples{
  B <- edges(letterR)
  A <- rpoisline(5, Frame(B))
  MA <- test.selfcrossing.psp(A)
  MAB <- test.crossing.psp(A, B)
}
\keyword{spatial}
\keyword{math}

\name{harmonise}
\alias{harmonise}
\alias{harmonize}
\title{Make Objects Compatible}
\description{
  Converts several objects of the same class to a common format
  so that they can be combined or compared.
}
\usage{
harmonise(\dots)
harmonize(\dots)
}
\arguments{
  \item{\dots}{
    Any number of objects of the same class.
  } 
}
\details{
  This generic command takes any number of objects of the same
  class, and \emph{attempts} to make them compatible in the sense
  of \code{\link{compatible}} so that they can be combined or compared.

  There are methods for the classes \code{"fv"}
  (\code{\link{harmonise.fv}})
  and \code{"im"} (\code{\link{harmonise.im}}).
  
  All arguments \code{\dots} must be objects of the same class.
  The result will be a list, of length equal to the number of
  arguments \code{\dots}, containing new versions of each of these
  objects, converted to a common format.
  If the arguments were named (\code{name=value}) then the return value
  also carries these names.
}
\value{
  A list, of length equal to the number of arguments \code{\dots},
  whose entries are objects of the same class.
  If the arguments were named (\code{name=value}) then the return value
  also carries these names.
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege.
}
\seealso{
  \code{\link{compatible}},
  \code{\link{harmonise.fv}},
  \code{\link{harmonise.im}}
}
\keyword{spatial}
\keyword{manip}

\name{crossdist}
\alias{crossdist}
\title{Pairwise distances}
\description{
  Computes the distances between pairs of `things'
  taken from two different datasets.
}
\usage{
  crossdist(X, Y, \dots)
}
\arguments{
  \item{X,Y}{
    Two objects of the same class.
  }
  \item{\dots}{
    Additional arguments depending on the method.
  }
}
\value{
  A matrix whose \code{[i,j]} entry is the distance
  from the \code{i}-th thing in the first dataset
  to the \code{j}-th thing in the second dataset.
}
\details{
  Given two datasets \code{X} and \code{Y}
  (representing either two point patterns or
  two line segment patterns)
  \code{crossdist} computes the Euclidean distance from each thing
  in the first dataset to each thing in the second dataset,
  and returns a matrix containing these distances.

  The function \code{crossdist} is generic, with
  methods for point patterns (objects of class \code{"ppp"}),
  line segment patterns (objects of class \code{"psp"}),
  and a default method. See the documentation for
  \code{\link{crossdist.ppp}},
  \code{\link{crossdist.psp}} or
  \code{\link{crossdist.default}} for further details.
}
\seealso{
  \code{\link{crossdist.ppp}},
  \code{\link{crossdist.psp}},
  \code{\link{crossdist.default}},
  \code{\link{pairdist}},
  \code{\link{nndist}}
}
\author{
  \adrian
  
  
}
\keyword{spatial}

\keyword{math}

\name{nestsplit}
\alias{nestsplit}
\title{
  Nested Split
}
\description{
  Applies two splitting operations to a point pattern,
  producing a list of lists of patterns.
}
\usage{
  nestsplit(X, \dots)
}
\arguments{
  \item{X}{
    Point pattern to be split.
    Object of class \code{"ppp"}.
  }
  \item{\dots}{
    Data determining the splitting factors or splitting regions.
    See Details.
  }
}
\details{
  This function splits the point pattern \code{X} into several
  sub-patterns using \code{\link{split.ppp}}, then splits each of the
  sub-patterns into sub-sub-patterns using \code{\link{split.ppp}}
  again.
  The result is a hyperframe containing the sub-sub-patterns
  and two factors indicating the grouping.

  The arguments \code{\dots} determine the two splitting factors
  or splitting regions. Each argument may be:
  \itemize{
    \item a factor (of length equal to the number of points in \code{X})
    \item the name of a column of marks of \code{X} (provided this
    column contains factor values)
    \item a tessellation (class \code{"tess"})
    \item a pixel image (class \code{"im"}) with factor values
    \item a window (class \code{"owin"})
    \item identified by name (in the form \code{name=value})
    as one of the formal arguments of \code{\link{quadrats}} or
    \code{\link{tess}}
  }
  The arguments will be processed to yield a list of two
  splitting factors/tessellations. The splits will be applied
  to \code{X} consecutively to produce the sub-sub-patterns.
}
\value{
  A hyperframe with three columns. The first column contains the
  sub-sub-patterns. The second and third columns are factors
  which identify the grouping according to the two splitting factors.
}
\author{
  Original idea by Ute Hahn.
  Code by \spatstatAuthors.
}
\seealso{
  \code{\link{split.ppp}},
  \code{\link{quantess}}
}
\examples{
  # factor and tessellation
  Nft <- nestsplit(amacrine, marks(amacrine), quadrats(amacrine, 3, 1))
  Ntf <- nestsplit(amacrine, quadrats(amacrine, 3, 1), marks(amacrine))
  Ntf

  # two factors
  big <- with(marks(betacells), area > 300)
  Nff <- nestsplit(betacells, "type", factor(big))

  # two tessellations
  Tx <- quantess(redwood, "x", 4)
  Td <- dirichlet(runifpoint(5, Window(redwood)))
  Ntt <- nestsplit(redwood, Td, Tx)
  Ntt2 <- nestsplit(redwood, Td, ny=3)
}
\keyword{spatial}
\keyword{manip}

\name{Frame}
\alias{Frame}
\alias{Frame<-}
\alias{Frame.default}
\alias{Frame<-.default}
\alias{Frame<-.owin}
\alias{Frame<-.ppp}
\alias{Frame<-.im}
\title{
  Extract or Change the Containing Rectangle of a Spatial Object
}
\description{
  Given a spatial object (such as a point pattern or pixel image)
  in two dimensions, these functions extract or change the
  containing rectangle inside which the object is defined.
}
\usage{
   Frame(X)

   \method{Frame}{default}(X)

   Frame(X) <- value

   \method{Frame}{owin}(X) <- value

   \method{Frame}{ppp}(X) <- value

   \method{Frame}{im}(X) <- value

   \method{Frame}{default}(X) <- value
}
\arguments{
  \item{X}{
    A spatial object such as a point pattern, line segment pattern
    or pixel image.
  }
  \item{value}{
    A rectangular window (object of class \code{"owin"}
    of type \code{"rectangle"}) to be used as the new containing
    rectangle for \code{X}.
  }
}
\details{
  The functions \code{Frame} and \code{Frame<-} are generic.

  \code{Frame(X)} extracts the rectangle inside which \code{X} is
  defined. 

  \code{Frame(X) <- R} changes the rectangle inside which \code{X} is defined
  to the new rectangle \code{R}.
}
\value{
  The result of \code{Frame} is a rectangular window (object of class
  \code{"owin"} of type \code{"rectangle"}).

  The result of \code{Frame<-} is the updated object \code{X},
  of the same class as \code{X}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{Window}}
}
\examples{
   Frame(cells)
   X <- demopat
   Frame(X)
   Frame(X) <- owin(c(0, 11000), c(400, 8000))
}
\keyword{spatial}
\keyword{manip}

\name{padimage}
\alias{padimage}
\title{
  Pad the Border of a Pixel Image
}
\description{
  Fills the border of a pixel image with a given value or values,
  or extends a pixel image to fill a larger window.
}
\usage{
padimage(X, value=NA, n=1, W=NULL)
}
\arguments{
  \item{X}{
    Pixel image (object of class \code{"im"}).
  }
  \item{value}{
    Single value to be placed around the border of \code{X}.    
  }
  \item{n}{
    Width of border, in pixels.
    See Details.
  }
  \item{W}{
    Window for the resulting image.
    Incompatible with \code{n}.
  }
}
\details{
  The image \code{X} will be expanded by a margin of \code{n}
  pixels, or extended to fill the window \code{W},
  with new pixel values set to \code{value}.

  The argument \code{value} should be a single value (a vector of length
  1), normally a value of the same type as the pixel values of \code{X}.
  It may be \code{NA}. Alternatively if \code{X} is a
  factor-valued image, \code{value} can be one of the levels of
  \code{X}.

  If \code{n} is given, it may be a single number, specifying the
  width of the border in pixels. Alternatively it may be a vector
  of length 2 or 4. It will be replicated to length 4, and these numbers
  will be interpreted as the border widths for the
  (left, right, top, bottom) margins respectively.

  Alternatively if \code{W} is given, the image will be extended to
  the window \code{W}. 
}
\value{
  Another object of class \code{"im"}, of the same type as \code{X}.
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link{im}}
}
\examples{
 Z <- setcov(owin())
 plot(padimage(Z, 1, 10))
}
\keyword{spatial}
\keyword{manip}

\name{disc}
\alias{disc}
\title{Circular Window}
\description{
Creates a circular window
}
\usage{
 disc(radius=1, centre=c(0,0), \dots, mask=FALSE, npoly=128, delta=NULL)
}
\arguments{
  \item{radius}{Radius of the circle.}
  \item{centre}{The centre of the circle.}
  \item{mask}{Logical flag controlling the type of approximation
    to a perfect circle. See Details.
  }
  \item{npoly}{Number of edges of the polygonal approximation,
    if \code{mask=FALSE}. Incompatible with \code{delta}.
  }
  \item{delta}{
    Tolerance of polygonal approximation: the length of arc
    that will be replaced by one edge of the polygon.
    Incompatible with \code{npoly}.
  }
  \item{\dots}{Arguments passed to \code{as.mask} determining the
    pixel resolution, if \code{mask=TRUE}.
  }
}
\value{
  An object of class \code{"owin"} (see \code{\link{owin.object}})
  specifying a window. 
}
\details{
  This command creates a window object
  representing a disc, with the given radius and centre.

  By default, the circle is
  approximated by a polygon with \code{npoly} edges.

  If \code{mask=TRUE}, then the disc is approximated by a binary pixel
  mask. The resolution of the mask is controlled by
  the arguments \code{\dots} which are passed to \code{\link{as.mask}}.

  The argument \code{radius} must be a single positive number.
  The argument \code{centre} specifies the disc centre: it can be either 
  a numeric vector of length 2 giving the coordinates,
  or a \code{list(x,y)} giving the coordinates of exactly one point, or a
  point pattern (object of class \code{"ppp"}) containing exactly one point.
}
\seealso{
  \code{\link{ellipse}},
  \code{\link{discs}},
  \code{\link{owin.object}},
  \code{\link{owin}},
  \code{\link{as.mask}}
}
\note{This function can also be used to generate regular polygons,
  by setting \code{npoly} to a small integer value. For example
  \code{npoly=5} generates a pentagon and \code{npoly=13} a triskaidecagon.
}
\examples{
 # unit disc
 W <- disc()
 # disc of radius 3 centred at x=10, y=5
 W <- disc(3, c(10,5))
 #
 plot(disc())
 plot(disc(mask=TRUE))
 # nice smooth circle
 plot(disc(npoly=256))
 # how to control the resolution of the mask
 plot(disc(mask=TRUE, dimyx=256))
 # check accuracy of approximation
 area(disc())/pi
 area(disc(mask=TRUE))/pi
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}
 
 

\name{clickbox}
\alias{clickbox}
\title{Interactively Define a Rectangle}
\description{
  Allows the user to specify a rectangle by
  point-and-click in the display.
}
\usage{
  clickbox(add=TRUE, \dots)
}
\arguments{
  \item{add}{
    Logical value indicating whether to create a new plot
    (\code{add=FALSE}) or draw over the existing plot (\code{add=TRUE}).
  }
  \item{\dots}{
    Graphics arguments passed to \code{\link[graphics]{polygon}} to plot the
    box.
  }
}
\value{
  A window (object of class \code{"owin"}) representing the
  selected rectangle.
}
\details{
  This function allows the user to create a rectangular window 
  by interactively clicking on the screen display.

  The user is prompted to point the mouse at any desired locations
  for two corners of the rectangle,
  and click the left mouse button to add each point.

  The return value is a window (object of class \code{"owin"})
  representing the rectangle.

  This function uses the \R command \code{\link[graphics]{locator}} to
  input the mouse clicks. It only works on screen devices such as
  \sQuote{X11}, \sQuote{windows} and \sQuote{quartz}. 
}
\seealso{
  \code{\link{clickpoly}},
  \code{\link{clickppp}},
  \code{\link{clickdist}},
  \code{\link[graphics]{locator}}
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{iplot}

\name{rotate.im}
\alias{rotate.im}
\title{Rotate a Pixel Image}
\description{
  Rotates a pixel image
}
\usage{
 \method{rotate}{im}(X, angle=pi/2, \dots, centre=NULL)
}
\arguments{
  \item{X}{A pixel image (object of class \code{"im"}).}
  \item{angle}{Angle of rotation, in radians.}
  \item{\dots}{Ignored.}
  \item{centre}{
    Centre of rotation.
    Either a vector of length 2, or a character string
    (partially matched to \code{"centroid"}, \code{"midpoint"}
    or \code{"bottomleft"}).
    The default is the coordinate origin \code{c(0,0)}.
  }
}
\value{
  Another object of class \code{"im"} representing the
  rotated pixel image.
}
\details{
  The image is rotated by the angle specified.
  Angles are measured in radians, anticlockwise.
  The default is to rotate the image 90 degrees anticlockwise. 
}
\seealso{
  \code{\link{affine.im}},
  \code{\link{shift.im}},
  \code{\link{rotate}}
}
\examples{
  Z <- distmap(letterR)
  X <- rotate(Z)
  \dontrun{
  plot(X)
  }
  Y <- rotate(X, centre="midpoint")
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{plot.layered}
\alias{plot.layered}
\title{
Layered Plot
}
\description{
  Generates a layered plot.
  The plot method for objects of class \code{"layered"}.
}
\usage{
\method{plot}{layered}(x, ..., which = NULL, plotargs = NULL,
                         add=FALSE, show.all=!add, main=NULL,
                         do.plot=TRUE)
}
\arguments{
  \item{x}{
    An object of class \code{"layered"}
    created by the function \code{\link{layered}}.
  }
  \item{\dots}{
    Arguments to be passed to the \code{plot} method
    for \emph{every} layer.
  }
  \item{which}{
    Subset index specifying which layers should be plotted. 
  }
  \item{plotargs}{
    Arguments to be passed to the  \code{plot} methods
    for individual layers. A list of lists of arguments of the
    form \code{name=value}.
  }
  \item{add}{Logical value indicating whether to add the graphics
    to an existing plot.
  }
  \item{show.all}{
     Logical value indicating whether the \emph{first} layer should be
     displayed in full (including the main title, bounding window,
     coordinate axes, colour ribbon, and so on). 
  }
  \item{main}{Main title for the plot}
  \item{do.plot}{Logical value indicating whether to actually do the plotting.}
}
\details{
  Layering is a simple mechanism for controlling
  a high-level plot that is composed of
  several successive plots, for example, a background and a foreground
  plot. The layering mechanism makes it easier to plot,
  to switch on or off the plotting of each individual layer,
  to control the plotting arguments that are passed to each layer,
  and to zoom in on a subregion.

  The layers of data to be plotted should first be converted
  into a single object of class \code{"layered"} using the
  function \code{\link{layered}}. Then the layers can be plotted using
  the method \code{plot.layered}.

  To zoom in on a subregion,
  apply the subset operator \code{\link{[.layered}} to \code{x}
  before plotting.

  Graphics parameters for each layer are determined by (in order of precedence)
  \code{\dots}, \code{plotargs}, and \code{\link{layerplotargs}(x)}.

  The graphics parameters may also include the special argument
  \code{.plot} specifying (the name of) a function which will be used to
  perform the plotting instead of the generic \code{plot}.
  
  The argument \code{show.all} is recognised by many plot methods
  in \pkg{spatstat}. It determines whether a plot is drawn
  with all its additional components such as the main title, bounding window,
  coordinate axes, colour ribbons and legends. The default is
  \code{TRUE} for new plots and \code{FALSE} for added plots.
  
  In \code{plot.layered}, the argument \code{show.all} applies only to the
  \bold{first} layer. The subsequent layers are plotted
  with \code{show.all=FALSE}. 

  To override this, that is, if you really want to draw all
  the components of \bold{all} layers of \code{x},
  insert the argument \code{show.all=TRUE} in each
  entry of \code{plotargs} or \code{\link{layerplotargs}(x)}.
}
\value{
  (Invisibly) a list containing the return values from
  the plot commands for each layer. This list has
  an attribute \code{"bbox"} giving a bounding box for the entire plot.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{layered}},
  \code{\link{layerplotargs}},
  \code{\link{[.layered}},
  \code{\link{plot}}.
}
\examples{
   data(cells)
   D <- distmap(cells)
   L <- layered(D, cells)
   plot(L)
   plot(L, which = 2)
   plot(L, plotargs=list(list(ribbon=FALSE), list(pch=3, cols="white")))
   # plot a subregion
   plot(L[, square(0.5)])
}
\keyword{spatial}
\keyword{hplot}

\name{nnfun}
\Rdversion{1.1}
\alias{nnfun}
\alias{nnfun.ppp}
\alias{nnfun.psp}
\title{
  Nearest Neighbour Index Map as a Function 
}
\description{
  Compute the nearest neighbour index map of an object,
  and return it as a function.
}
\usage{
  nnfun(X, ...)

  \method{nnfun}{ppp}(X, ..., k=1, value=c("index", "mark"))

  \method{nnfun}{psp}(X, ..., value=c("index", "mark"))
}
\arguments{
  \item{X}{
    Any suitable dataset representing a two-dimensional
    collection of objects,
    such as a point pattern (object of class \code{"ppp"})
    or a line segment pattern (object of class \code{"psp"}).
  }
  \item{k}{
    A single integer. The \code{k}th nearest neighbour will be found.
  }
  \item{\dots}{
    Extra arguments are ignored. 
  }
  \item{value}{
    String (partially matched) specifying whether to return the
    index of the neighbour (\code{value="index"}, the default)
    or the mark value of the neighbour (\code{value="mark"}).
  }
}
\details{
  For a collection \eqn{X} of two dimensional objects
  (such as a point pattern or a line segment pattern),
  the \dQuote{nearest neighbour index function}
  of \eqn{X} is the mathematical function \eqn{f} such that, for any 
  two-dimensional spatial location \eqn{(x,y)},
  the function value \code{f(x,y)}
  is the index \eqn{i} identifying the closest member of \eqn{X}.
  That is, if \eqn{i = f(x,y)} then \eqn{X[i]} is the closest member of
  the collection \eqn{X} to the location \eqn{(x,y)}.
  
  The command \code{f <- nnfun(X)} returns a \emph{function}
  in the \R language, with arguments \code{x,y}, that represents the
  nearest neighbour index function of \code{X}. Evaluating the function \code{f}
  in the form \code{v <- f(x,y)}, where \code{x} and \code{y}
  are any numeric vectors of equal length containing coordinates of
  spatial locations, yields the indices of the nearest neighbours
  to these locations.

  If the argument \code{k} is specified then the \code{k}-th nearest
  neighbour will be found.

  The result of \code{f <- nnfun(X)} also belongs to the class
  \code{"funxy"} and to the special class \code{"nnfun"}.
  It can be printed and plotted immediately as shown in the Examples.
  
  A \code{nnfun} object can be converted to a pixel image
  using \code{\link{as.im}}.
}
\value{
  A \code{function} with arguments \code{x,y}.
  The function also belongs to the class \code{"nnfun"} which has
  a method for \code{print}.
  It also belongs to the class \code{"funxy"} which has methods
  for \code{plot}, \code{contour} and \code{persp}.
}
\seealso{
  \code{\link{distfun}},
  \code{\link{plot.funxy}}
}
\examples{
   f <- nnfun(cells)
   f
   plot(f)
   f(0.2, 0.3)

   g <- nnfun(cells, k=2)
   g(0.2, 0.3)

   plot(nnfun(amacrine, value="m"))

   L <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
   h <- nnfun(L)
   h(0.2, 0.3)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{plot.hyperframe}
\alias{plot.hyperframe}
\title{Plot Entries in a Hyperframe}
\description{
  Plots the entries in a hyperframe, in a series of
  panels, one panel for each row of the hyperframe.
}
\usage{
   \method{plot}{hyperframe}(x, e, \dots, main, arrange=TRUE,
                            nrows=NULL, ncols=NULL,
                            parargs=list(mar=mar * marsize),
                            marsize=1, mar=c(1,1,3,1))
}
\arguments{
  \item{x}{
    Data to be plotted. A hyperframe (object of class \code{"hyperframe"}, see
    \code{\link{hyperframe}}).
  }
  \item{e}{
    How to plot each row. Optional. An \R language call or expression
    (typically enclosed in \code{\link{quote}()} that will be
    evaluated in each row of the hyperframe to generate the plots.
  }
  \item{\dots}{
    Extra arguments controlling the plot (when \code{e} is missing).
  }
  \item{main}{Overall title for the array of plots.}
  \item{arrange}{
    Logical flag indicating whether to plot the objects
    side-by-side on a single page (\code{arrange=TRUE})
    or plot them individually in a succession of frames
    (\code{arrange=FALSE}).
  }
  \item{nrows,ncols}{
    Optional. The number of rows/columns in the plot layout
    (assuming \code{arrange=TRUE}).
    You can specify either or both of these numbers.
  }
  \item{parargs}{
    Optional list of arguments passed to \code{\link{par}} before
    plotting each panel. Can be used to control margin sizes, etc.
  }
  \item{marsize}{
    Optional scale parameter controlling the sizes of margins around
    the panels. Incompatible with \code{parargs}.
  }
  \item{mar}{
    Optional numeric vector of length 1, 2 or 4
    controlling the relative sizes of margins between
    the panels. Incompatible with \code{parargs}.
  }
}
\details{
  This is the \code{plot} method for the class \code{"hyperframe"}.

  The argument \code{x} must be a hyperframe (like a data frame,
  except that the entries can be objects of any class; see
  \code{\link{hyperframe}}).

  This function generates a series of plots, one plot for each
  row of the hyperframe. If \code{arrange=TRUE} (the default), then
  these plots are arranged in a neat array of panels within a single
  plot frame. If \code{arrange=FALSE}, the plots are simply executed
  one after another.

  Exactly what is plotted, and how it is plotted, depends on the
  argument \code{e}. The default (if \code{e} is missing) is to plot
  only the first column of \code{x}. Each entry in the first column
  is plotted using the generic \code{\link{plot}} command, together with
  any extra arguments given in \code{\dots}.

  If \code{e} is present, it should be an \R language expression
  involving the column names of \code{x}.
  (It is typically created using \code{\link{quote}} or
  \code{\link{expression}}.)
  The expression will be evaluated once for each row of \code{x}.
  It will be evaluated in an environment where each column name of
  \code{x} is interpreted as meaning the object in that column
  in the current row.
  See the Examples.
}
\value{
  \code{NULL}.
}
\seealso{
  \code{\link{hyperframe}},
  \code{\link{with.hyperframe}}
}
\examples{
   H <- hyperframe(id=1:10)
   H$X <- with(H, rpoispp(100))
   H$D <- with(H, distmap(X))
   # points only
   plot(H[,"X"])
   plot(H, quote(plot(X, main=id)))
   # points superimposed on images
   plot(H, quote({plot(D, main=id); plot(X, add=TRUE)}))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{hplot}

\name{rotate.psp}
\alias{rotate.psp}
\title{Rotate a Line Segment Pattern}
\description{
  Rotates a line segment pattern
}
\usage{
 \method{rotate}{psp}(X, angle=pi/2, \dots, centre=NULL)
}
\arguments{
  \item{X}{A line segment pattern (object of class \code{"psp"}).}
  \item{angle}{Angle of rotation.}
  \item{\dots}{
    Arguments passed to \code{\link{rotate.owin}} affecting
    the handling of the observation window, if it is a binary pixel
    mask.
  }
  \item{centre}{
    Centre of rotation.
    Either a vector of length 2, or a character string
    (partially matched to \code{"centroid"}, \code{"midpoint"}
    or \code{"bottomleft"}).
    The default is the coordinate origin \code{c(0,0)}.
  }
}
\value{
  Another object of class \code{"psp"} representing the
  rotated line segment pattern.
}
\details{
  The line segments of the pattern, and the window of observation, 
  are rotated about the origin by the angle specified.
  Angles are measured in
  radians, anticlockwise. The default is to rotate the pattern 90 degrees
  anticlockwise. If the line segments carry marks, these are preserved.
}
\seealso{
  \code{\link{psp.object}},
  \code{\link{rotate.owin}},
  \code{\link{rotate.ppp}}
}
\examples{
  oldpar <- par(mfrow=c(2,1))
  X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  plot(X, main="original")
  Y <- rotate(X, pi/4)
  plot(Y, main="rotated")
  par(oldpar)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{border}
\alias{border}
\title{Border Region of a Window}
\description{
  Computes the border region of a window,
  that is, the region lying within a specified distance of the boundary
  of a window.
}
\usage{
border(w, r, outside=FALSE, ...)
}
\arguments{
  \item{w}{A window (object of class \code{"owin"})
    or something acceptable to \code{\link{as.owin}}.
  }
  \item{r}{Numerical value.}
  \item{outside}{Logical value determining whether to compute the
    border outside or inside \code{w}.}
  \item{\dots}{
    Optional arguments passed to \code{\link{erosion}}
    (if \code{outside=FALSE}) or to \code{\link{dilation}}
    (if \code{outside=TRUE}).
  }
}
\value{
  A window (object of class \code{"owin"}).
}
\details{
  By default (if \code{outside=FALSE}),
  the border region is the subset of \code{w}
  lying within a distance \code{r} of the boundary of \code{w}.
  It is computed by eroding \code{w} by the distance \code{r} (using
  \code{\link{erosion}}) and
  subtracting this eroded window from the original window \code{w}.

  If \code{outside=TRUE}, the border region is the set of locations
  outside \code{w} lying within a distance \code{r} of \code{w}.
  It is computed by dilating \code{w} by the distance \code{r}
  (using \code{\link{dilation}}) and
  subtracting the original window \code{w} from the dilated window.
}

\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{erosion}},
  \code{\link{dilation}}
}
\examples{
# rectangle
   u <- unit.square()
   border(u, 0.1)
   border(u, 0.1, outside=TRUE)
# polygon
   \testonly{opa <- spatstat.options(npixel=32)}
   data(letterR)
   plot(letterR)
   plot(border(letterR, 0.1), add=TRUE)
   plot(border(letterR, 0.1, outside=TRUE), add=TRUE)
   \testonly{spatstat.options(opa)}
}
\keyword{spatial}
\keyword{math}

\name{deltametric}
\Rdversion{1.1}
\alias{deltametric}
\title{
  Delta Metric
}
\description{
  Computes the discrepancy between two sets \eqn{A} and \eqn{B}
  according to Baddeley's delta-metric.
}
\usage{
deltametric(A, B, p = 2, c = Inf, ...)
}
\arguments{
  \item{A,B}{
    The two sets which will be compared.
    Windows (objects of class \code{"owin"}),
    point patterns (objects of class \code{"ppp"})
    or line segment patterns (objects of class \code{"psp"}).
  }
  \item{p}{
    Index of the \eqn{L^p} metric.
    Either a positive numeric value, or \code{Inf}.
  }
  \item{c}{
    Distance threshold. 
    Either a positive numeric value, or \code{Inf}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine the
    pixel resolution of the distance maps computed by \code{\link{distmap}}.
  }
}
\details{
  Baddeley (1992a, 1992b) defined a distance
  between two sets \eqn{A} and \eqn{B} contained in a space \eqn{W} by 
  \deqn{
    \Delta(A,B) = \left[
    \frac 1 {|W|}
    \int_W
    \left| \min(c, d(x,A)) - \min(c, d(x,B)) \right|^p \, {\rm d}x
    \right]^{1/p}
  }{
    \Delta(A,B) = [ (1/|W|) * integral of |min(c, d(x,A))-min(c, d(x,B))|^p dx ]^(1/p)
  }
  where \eqn{c \ge 0}{c \ge 0} is a distance threshold parameter,
  \eqn{0 < p \le \infty}{0 < p \le Inf} is the exponent parameter,
  and \eqn{d(x,A)} denotes the 
  shortest distance from a point \eqn{x} to the set \eqn{A}.
  Also \code{|W|} denotes the area or volume of the containing space \eqn{W}.

  This is defined so that it is a \emph{metric}, i.e.
  \itemize{
    \item \eqn{\Delta(A,B)=0}{\Delta(A,B)=0} if and only if \eqn{A=B}
    \item \eqn{\Delta(A,B)=\Delta(B,A)}{\Delta(A,B)=\Delta(B,A)}
    \item \eqn{\Delta(A,C) \le \Delta(A,B) + \Delta(B,C)}{\Delta(A,C) \le
      \Delta(A,B) + \Delta(B,C)}
  }
  It is topologically equivalent to the Hausdorff metric
  (Baddeley, 1992a) but has better stability properties
  in practical applications (Baddeley, 1992b).

  If \eqn{p=\infty}{p=Inf} and \eqn{c=\infty}{c=Inf} the Delta metric
  is equal to the Hausdorff metric.

  The algorithm uses \code{\link{distmap}} to compute the distance maps
  \eqn{d(x,A)} and \eqn{d(x,B)}, then approximates the integral
  numerically.
  The accuracy of the computation depends on the pixel resolution
  which is controlled through the extra arguments \code{\dots} passed
  to \code{\link{as.mask}}.
}
\value{
  A numeric value.
}
\references{
  Baddeley, A.J. (1992a)
  Errors in binary images and an \eqn{L^p} version of the Hausdorff metric.
  \emph{Nieuw Archief voor Wiskunde} \bold{10}, 157--183.

  Baddeley, A.J. (1992b)
  An error metric for binary images.
  In W. Foerstner and S. Ruwiedel (eds)
  \emph{Robust Computer Vision}. Karlsruhe: Wichmann.
  Pages 59--78.
}
\author{
  \adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{distmap}}
}
\examples{
  X <- runifpoint(20)
  Y <- runifpoint(10)
  deltametric(X, Y, p=1,c=0.1)
}
\keyword{spatial}
\keyword{math}

\name{rpoislinetess}
\alias{rpoislinetess}
\title{Poisson Line Tessellation}
\description{
  Generate a tessellation delineated by the lines of
  the Poisson line process
}
\usage{
rpoislinetess(lambda, win = owin())
}
\arguments{
  \item{lambda}{
    Intensity of the Poisson line process.
    A positive number.
  }
  \item{win}{
    Window in which to simulate the pattern.
    An object of class \code{"owin"}
    or something acceptable to \code{\link{as.owin}}.
    Currently, the window must be a rectangle.
  }
}
\details{
  This algorithm generates a realisation
  of the uniform Poisson line process, and divides the window
  \code{win} into tiles separated by these lines.

  The argument \code{lambda} must be a positive number.
  It controls the intensity of the process. The expected number of
  lines intersecting a convex region of the plane is equal to
  \code{lambda} times the perimeter length of the region.
  The expected total length of the lines crossing a region of the plane
  is equal to \code{lambda * pi} times the area of the region.
}
\value{
  A tessellation (object of class \code{"tess"}).

  Also has an attribute \code{"lines"} containing the realisation of the
  Poisson line process, as an object of class \code{"infline"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{rpoisline}}
  to generate the lines only.
}
\examples{
 X <- rpoislinetess(3)
 plot(as.im(X), main="rpoislinetess(3)")
 plot(X, add=TRUE)
}
\keyword{spatial}
\keyword{datagen}

\name{intensity.ppm}  
\alias{intensity.ppm}
\title{
  Intensity of Fitted Point Process Model
}
\description{
  Computes the intensity of a fitted point process model.
}
\usage{
\method{intensity}{ppm}(X, \dots)
}
\arguments{
  \item{X}{
    A fitted point process model (object of class \code{"ppm"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{predict.ppm}} in some cases.
    See Details.
  }
}
\details{
  This is a method for the generic function \code{\link{intensity}}
  for fitted point process models (class \code{"ppm"}).

  The intensity of a point process model is the expected
  number of random points per unit area. 

  If \code{X} is a Poisson point process model, the intensity of the
  process is computed exactly.
  The result is a numerical value if \code{X}
  is a stationary Poisson point process, and a pixel image if \code{X}
  is non-stationary. (In the latter case, the resolution of the pixel
  image is controlled by the arguments \code{\dots} which are passed
  to \code{\link{predict.ppm}}.)

  If \code{X} is another Gibbs point process model, the intensity is
  computed approximately using the Poisson-saddlepoint approximation
  (Baddeley and Nair, 2012a, 2012b, 2016; Anderssen et al, 2014). 
  The approximation is currently available for pairwise-interaction
  models (Baddeley and Nair, 2012a, 2012b)
  and for the area-interaction model and Geyer saturation model
  (Baddeley and Nair, 2016).

  For a non-stationary Gibbs model, the 
  pseudostationary solution (Baddeley and Nair, 2012b;
  Anderssen et al, 2014) is used. The result is a pixel image,
  whose resolution is controlled by the arguments \code{\dots} which are passed
  to \code{\link{predict.ppm}}.
}
\value{
  A numeric value (if the model is stationary)
  or a pixel image.
}
\references{
  Anderssen, R.S., Baddeley, A., DeHoog, F.R. and Nair, G.M. (2014)
  Solution of an integral equation arising in spatial point process theory.
  \emph{Journal of Integral Equations and Applications} 
  \bold{26} (4) 437--453.

  Baddeley, A. and Nair, G. (2012a) 
  Fast approximation of the intensity of Gibbs point processes.
  \emph{Electronic Journal of Statistics} \bold{6} 1155--1169.

  Baddeley, A. and Nair, G. (2012b)
  Approximating the moments of a spatial point process.
  \emph{Stat} \bold{1}, 1, 18--30.
  doi: 10.1002/sta4.5

  Baddeley, A. and Nair, G. (2016)
  Poisson-saddlepoint approximation for spatial point processes
  with infinite order interaction.
  Submitted for publication.
}
\seealso{
  \code{\link{intensity}},
  \code{\link{intensity.ppp}}
}
\examples{
  fitP <- ppm(swedishpines ~ 1)
  intensity(fitP)
  fitS <- ppm(swedishpines ~ 1, Strauss(9))
  intensity(fitS)
  fitSx <- ppm(swedishpines ~ x, Strauss(9))
  lamSx <- intensity(fitSx)
  fitG <- ppm(swedishpines ~ 1, Geyer(9, 1))
  lamG <- intensity(fitG)
  fitA <- ppm(swedishpines ~ 1, AreaInter(7))
  lamA <- intensity(fitA)
}
\author{
  \adrian
  and Gopalan Nair.
}
\keyword{spatial}
\keyword{models}

\name{harmonise.owin}
\alias{harmonise.owin}
\alias{harmonize.owin}
\title{Make Windows Compatible}
\description{
  Convert several windows to a common pixel raster.
}
\usage{
\method{harmonise}{owin}(\dots)

\method{harmonize}{owin}(\dots)
}
\arguments{
  \item{\dots}{
    Any number of windows (objects of class \code{"owin"})
    or data which can be converted to windows by \code{\link{as.owin}}.
  } 
}
\details{
  This function makes any number of windows compatible,
  by converting them all to a common pixel grid.

  This only has an effect if one of the windows is a binary mask.
  If all the windows are rectangular or polygonal, they are
  returned unchanged.

  The command \code{\link{harmonise}} is generic. This is the
  method for objects of class \code{"owin"}.
  
  Each argument must be a window (object of class \code{"owin"}),
  or data that can be converted to a window by \code{\link{as.owin}}.
  
  The common pixel grid is determined by inspecting all the windows
  in the argument list, computing the bounding box of all the
  windows, then finding the binary mask with the finest spatial resolution, 
  and extending its pixel grid to cover the bounding box. 

  The return value is a list with entries corresponding to the input
  arguments.
  If the arguments were named (\code{name=value}) then the return value
  also carries these names.

  If you just want to determine the appropriate pixel resolution,
  without converting the windows, use \code{\link{commonGrid}}.
}
\value{
  A list of windows, of length equal to the number of arguments
  \code{\dots}. The list belongs to the class \code{"solist"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\examples{
   harmonise(X=letterR,
             Y=grow.rectangle(Frame(letterR), 0.2),
             Z=as.mask(letterR, eps=0.1),
             V=as.mask(letterR, eps=0.07))
}
\seealso{
  \code{\link{commonGrid}},
  \code{\link{harmonise.im}},
  \code{\link{as.owin}}
}
\keyword{spatial}
\keyword{manip}

\name{cut.ppp}
\alias{cut.ppp}
\title{Classify Points in a Point Pattern}
\description{
  Classifies the points in a point pattern into distinct types
  according to the numerical marks in the pattern, or according to
  another variable.
}
\usage{
  \method{cut}{ppp}(x, z=marks(x), ...)
}
\arguments{
  \item{x}{
    A two-dimensional point pattern.
    An object of class \code{"ppp"}.
  }
  \item{z}{
    Data determining the classification. A numeric vector,
    a factor, a pixel image, a window, a tessellation, or a string
    giving the name of a column of marks or the name of a spatial coordinate.
  }
  \item{\dots}{
    Arguments passed to \code{\link{cut.default}}.
    They determine the breakpoints for the mapping from numerical values
    in \code{z} to factor values in the output.
    See \code{\link{cut.default}}.
  }
} 
\value{
  A multitype point pattern, that is, a point pattern object
  (of class \code{"ppp"}) with a \code{marks} vector that is a factor.
}
\details{
  This function has the effect of classifying each point in the point
  pattern \code{x} into one of several possible types. The
  classification is based on the dataset \code{z}, which may be either
  \itemize{
    \item
    a factor (of length equal to the number of points in \code{z})
    determining the classification of each point in \code{x}.
    Levels of the factor determine the classification.
    \item
    a numeric vector (of length equal to the number of points in
    \code{z}). The range of values of \code{z} will be divided into
    bands (the number of bands is determined by \code{\dots})
    and \code{z} will be converted to a factor using
    \code{\link{cut.default}}.
    \item
    a pixel image (object of class \code{"im"}).
    The value of \code{z} at each point of \code{x} will be
    used as the classifying variable.
    \item
    a tessellation (object of class \code{"tess"}, see
    \code{\link{tess}}). Each point of \code{x} will be classified
    according to the tile of the tessellation into which it falls.
    \item
    a window (object of class \code{"owin"}).
    Each point of \code{x} will be classified
    according to whether it falls inside or outside this window.
    \item
    a character string, giving the name of one of the columns
    of \code{marks(x)}, if this is a data frame.
    \item
    a character string \code{"x"} or \code{"y"} identifying one of the
    spatial coordinates.
  }
  The default is to take \code{z} to be the vector of marks in
  \code{x} (or the first column in the data frame of marks of \code{x},
  if it is a data frame). If the marks are numeric, then the range of values
  of the numerical marks is divided into several intervals, and each
  interval is associated with a level of a factor. 
  The result is a
  marked point pattern, with the same window and point locations as
  \code{x}, but with the numeric mark of each point discretised
  by replacing it by the factor level.
  This is a convenient way to transform a marked point pattern
  which has numeric marks into a multitype point pattern,
  for example to plot it or analyse it. See the examples.

  To select some points from a point pattern, use the subset operators
  \code{\link{[.ppp}} or \code{\link{subset.ppp}} instead.
}
\seealso{
  \code{\link{cut}},
  \code{\link{ppp.object}},
  \code{\link{tess}}
}
\examples{
 # (1) cutting based on numeric marks of point pattern
 
 trees <- longleaf
 # Longleaf Pines data
 # the marks are positive real numbers indicating tree diameters.

 \testonly{
	# smaller dataset
	trees <- trees[seq(1, npoints(trees), by=80)]
 }
 \dontrun{
 plot(trees)
 }

 # cut the range of tree diameters into three intervals
 long3 <- cut(trees, breaks=3)
 \dontrun{
 plot(long3)
 }

 # adult trees defined to have diameter at least 30 cm
 long2 <- cut(trees, breaks=c(0,30,100), labels=c("Sapling", "Adult"))
 plot(long2)
 plot(long2, cols=c("green","blue"))

 # (2) cutting based on another numeric vector
 # Divide Swedish Pines data into 3 classes
 # according to nearest neighbour distance

 swedishpines
 plot(cut(swedishpines, nndist(swedishpines), breaks=3))

 # (3) cutting based on tessellation
 # Divide Swedish Pines study region into a 4 x 4 grid of rectangles
 # and classify points accordingly

 tes <- tess(xgrid=seq(0,96,length=5),ygrid=seq(0,100,length=5))
 plot(cut(swedishpines, tes))
 plot(tes, lty=2, add=TRUE)

 # (4) inside/outside a given region
 with(murchison, cut(gold, greenstone))

 # (5) multivariate marks
 finpines
 cut(finpines, "height", breaks=4)
}

\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{methods}

\name{raster.x}
\alias{raster.x}
\alias{raster.y}
\alias{raster.xy}
\title{Cartesian Coordinates for a Pixel Raster}
\description{
  Return the \eqn{x} and \eqn{y} coordinates
  of each pixel in a pixel image or binary mask.
}
\usage{
 raster.x(w, drop=FALSE)
 raster.y(w, drop=FALSE)
 raster.xy(w, drop=FALSE)
}
\arguments{
  \item{w}{
    A pixel image (object of class \code{"im"}) or a 
    mask window (object of class \code{"owin"}
    of type \code{"mask"}).
  }
  \item{drop}{
    Logical.
    If \code{TRUE}, then coordinates of pixels that lie outside the
    window are removed.
    If \code{FALSE} (the default) then the coordinates of every
    pixel in the containing rectangle are retained.
  }
}
\value{
  \code{raster.xy} returns a list with components \code{x} and \code{y}
  which are numeric vectors of equal length containing the pixel coordinates.

  If \code{drop=FALSE}, 
  \code{raster.x} and \code{raster.y} return 
  a matrix of the same dimensions as the pixel grid in \code{w},
  and giving the value of the \eqn{x} (or \eqn{y}) coordinate
  of each pixel in the raster.

  If \code{drop=TRUE}, 
  \code{raster.x} and \code{raster.y} return numeric vectors.
}
\details{
  The argument \code{w} should be either
  a pixel image (object of class \code{"im"})
  or a mask window (an object of class
  \code{"owin"} of type \code{"mask"}).

  If \code{drop=FALSE} (the default), the 
  functions \code{raster.x} and \code{raster.y} return
  a matrix of the same dimensions as the
  pixel image or mask itself, with entries giving the \eqn{x} coordinate
  (for \code{raster.x}) or \eqn{y} coordinate (for \code{raster.y})
  of each pixel in the pixel grid.

  If \code{drop=TRUE}, pixels that lie outside the
  window \code{w} (or outside the domain of the image \code{w})
  are removed, and \code{raster.x} and \code{raster.y} 
  return numeric vectors containing the coordinates of the
  pixels that are inside the window \code{w}.

  The function \code{raster.xy} returns a list
  with components \code{x} and \code{y}
  which are numeric vectors of equal length containing the pixel coordinates.
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.mask}},
  \code{\link{pixelcentres}}
}
\examples{
  u <- owin(c(-1,1),c(-1,1)) # square of side 2
  w <- as.mask(u, eps=0.01) # 200 x 200 grid
  X <- raster.x(w)
  Y <- raster.y(w)
  disc <- owin(c(-1,1), c(-1,1), mask=(X^2 + Y^2 <= 1))
  \dontrun{plot(disc)}
  # approximation to the unit disc
}
\author{\adrian
  ,
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{manip}

\name{Smooth}
\alias{Smooth}
\title{Spatial smoothing of data}
\description{
  Generic function to perform spatial smoothing of spatial data.
}
\usage{
  Smooth(X, ...)
}
\arguments{
  \item{X}{Some kind of spatial data}
  \item{\dots}{Arguments passed to methods.}
}
\details{
  This generic function calls an appropriate method
  to perform spatial smoothing on the spatial dataset \code{X}.

  Methods for this function include
  \itemize{
    \item \code{\link{Smooth.ppp}} for point patterns
    \item \code{\link{Smooth.msr}} for measures
    \item \code{\link{Smooth.fv}} for function value tables
  }
}
\seealso{
  \code{\link{Smooth.ppp}},
  \code{\link{Smooth.im}},
  \code{\link{Smooth.msr}},
  \code{\link{Smooth.fv}}.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}
\keyword{smooth}

\name{distmap.ppp}  
\alias{distmap.ppp}
\title{
  Distance Map of Point Pattern
}
\description{
  Computes the distance from each pixel to the nearest
  point in the given point pattern.
}
\usage{
  \method{distmap}{ppp}(X, \dots)
}
\arguments{
  \item{X}{A point pattern (object of class \code{"ppp"}).
  }
  \item{\dots}{Arguments passed to \code{\link{as.mask}}
    to control pixel resolution.
  }
}
\value{
  A pixel image (object of class \code{"im"}) whose greyscale values
  are the values of the distance map.
  The return value has attributes \code{"index"} and \code{"bdry"}
  which are also pixel images. 
}
\details{
  The ``distance map'' of a point pattern \eqn{X} is the function
  \eqn{f} whose value \code{f(u)} is defined for any two-dimensional
  location \eqn{u} as the shortest distance from \eqn{u} to \eqn{X}.

  This function computes the distance map of the point pattern \code{X}
  and returns the distance map as a pixel image. The greyscale value
  at a pixel \eqn{u} equals the distance from \eqn{u} to the nearest
  point of the pattern \code{X}.

  Additionally, the return value 
  has two attributes, \code{"index"} and \code{"bdry"}, which are
  also pixel images. The grey values in \code{"bdry"} give the
  distance from each pixel to the bounding rectangle of the image.
  The grey values in \code{"index"} are integers identifying which
  point of \code{X} is closest. 
  
  This is a method for the generic function \code{\link{distmap}}.
  
  Note that this function gives the distance from the
  \emph{centre of each pixel} to the nearest data point.
  To compute the exact distance from a given spatial location
  to the nearest data point in \code{X}, use \code{\link{distfun}} or 
  \code{\link{nncross}}.
}
\seealso{
  Generic function \code{\link{distmap}} and other methods
  \code{\link{distmap.psp}},
  \code{\link{distmap.owin}}.
  
  Generic function \code{\link{distfun}}.

  Nearest neighbour distance \code{\link{nncross}}
}
\examples{
  data(cells)
  U <- distmap(cells)
  \dontrun{
  plot(U)
  plot(attr(U, "bdry"))
  plot(attr(U, "index"))
  }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{Math.imlist}
\alias{Math.imlist}
\alias{Ops.imlist}
\alias{Complex.imlist}
\alias{Summary.imlist}
\title{S3 Group Generic methods for List of Images}
\description{
  These are group generic methods for the class \code{"imlist"}
  of lists of images. These methods 
  allows the usual mathematical functions and operators to be applied
  directly to lists of images. See Details for a list of implemented functions.
}
\usage{
## S3 methods for group generics have prototypes:
\special{Math(x, \dots)}
\special{Ops(e1, e2)}
\special{Complex(z)}
\special{Summary(\dots, na.rm = TRUE)}
%NAMESPACE S3method("Math", "imlist")
%NAMESPACE S3method("Ops", "imlist")
%NAMESPACE S3method("Complex", "imlist")
%NAMESPACE S3method("Summary", "imlist")
}
\arguments{
  \item{x,z,e1,e2}{
    Lists of pixel images (objects of class \code{"imlist"}).
  }
  \item{\dots}{further arguments passed to methods.}
  \item{na.rm}{logical: should missing values be removed?}
}
\details{
  An object of class \code{"imlist"} represents a list of pixel images.
  It is a \code{list}, whose entries are pixel images
  (objects of class \code{"im"}).
  
  The following mathematical functions and operators are
  defined for lists of images.

  Not all functions will make sense for all
  types of images. For example, none of the functions in the \code{"Math"} group
  make sense for character-valued images. Note that the \code{"Ops"}
  group methods are implemented using \code{\link{eval.im}}, which tries
  to harmonise images via \code{\link{harmonise.im}} if they aren't
  compatible to begin with.

  \enumerate{
    \item Group \code{"Math"}:
    \itemize{
      \item
      \code{abs}, \code{sign}, \code{sqrt},\cr
      \code{floor}, \code{ceiling}, \code{trunc},\cr
      \code{round}, \code{signif}

      \item
      \code{exp}, \code{log},  \code{expm1}, \code{log1p},\cr
      \code{cos}, \code{sin}, \code{tan},\cr
      \code{cospi}, \code{sinpi}, \code{tanpi},\cr
      \code{acos}, \code{asin}, \code{atan}

      \code{cosh}, \code{sinh}, \code{tanh},\cr
      \code{acosh}, \code{asinh}, \code{atanh}

      \item
      \code{lgamma}, \code{gamma}, \code{digamma}, \code{trigamma}
      \item \code{cumsum}, \code{cumprod}, \code{cummax}, \code{cummin}
    }

    \item Group \code{"Ops"}:
    \itemize{
      \item
      \code{"+"}, \code{"-"}, \code{"*"}, \code{"/"},
      \code{"^"}, \code{"\%\%"}, \code{"\%/\%"}

      \item \code{"&"}, \code{"|"}, \code{"!"}

      \item \code{"=="}, \code{"!="},
      \code{"<"}, \code{"<="}, \code{">="}, \code{">"}
    }

    \item Group \code{"Summary"}:
    \itemize{
      \item \code{all}, \code{any}
      \item \code{sum}, \code{prod}
      \item \code{min}, \code{max}
      \item \code{range}
    }

    \item Group \code{"Complex"}:
    \itemize{
      \item \code{Arg}, \code{Conj}, \code{Im}, \code{Mod}, \code{Re}
    }
  }

  For the binary operations in \code{"Ops"},
  either
  \itemize{
    \item \code{e1} and \code{e2} are lists of pixel images,
    and contain the same number of images.
%    \item one of \code{e1,e2} is a list of pixel images,
%    and the other is an individual pixel image (class \code{"im"}).
    \item one of \code{e1,e2} is a list of pixel images,
    and the other is a single atomic value.
  }
  
}
\value{
  The result of \code{"Math"}, \code{"Ops"} and \code{"Complex"} group
  operations is another list of images.
  The result of \code{"Summary"} group operations is a numeric vector of
  length 1 or 2.
}
\seealso{
  \code{\link{Math.im}} or \code{\link{eval.im}}
  for evaluating expressions involving images.
  \code{\link{solapply}} for a wrapper for \code{\link[base]{lapply}}.
}
\examples{
  a <- Smooth(finpines, 2)
  log(a)/2 - sqrt(a)
  range(a)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{methods}

\name{solapply}
\alias{solapply}
\alias{anylapply}
\title{
  Apply a Function Over a List and Obtain a List of Objects
}
\description{
  Applies the function \code{FUN} to each element of the list \code{X},
  and returns the result as a list of class \code{"solist"}
  or \code{"anylist"} as appropriate.
}
\usage{
  anylapply(X, FUN, \dots)

  solapply(X, FUN, \dots, check = TRUE, promote = TRUE, demote = FALSE)
}
\arguments{
  \item{X}{A list.}
  \item{FUN}{
    Function to be applied to each element of \code{X}.
  }
  \item{\dots}{
    Additional arguments to \code{FUN}.
  }
  \item{check,promote,demote}{
    Arguments passed to \code{\link{solist}} which determine
    how to handle different classes of objects.
  }
}
\details{
  These convenience functions are similar to \code{\link[base]{lapply}}
  except that they return a list of class \code{"solist"} or
  \code{"anylist"}.

  In both functions, the result is computed by 
  \code{lapply(X, FUN, \dots)}.

  In \code{anylapply} the result is converted to a list of class
  \code{"anylist"} and returned.

  In \code{solapply} the result is converted to
  a list of class \code{"solist"} \bold{if possible}, using 
  \code{\link{as.solist}}. If this is not possible,
  then the behaviour depends on the argument \code{demote}.
  If \code{demote=TRUE} the result will be returned as a
  list of class \code{"anylist"}. If \code{demote=FALSE} (the default),
  an error occurs.
}
\value{
  A list, usually of class \code{"solist"}.
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link{solist}}, 
  \code{\link{anylist}}.
}
\examples{
  solapply(waterstriders, density)
}
\keyword{spatial}
\keyword{list}
\keyword{manip}

\name{as.hyperframe.ppx}
\Rdversion{1.1}
\alias{as.hyperframe.ppx}
\alias{as.data.frame.ppx}
\alias{as.matrix.ppx}
\title{
  Extract coordinates and marks of multidimensional point pattern
}
\description{
  Given any kind of spatial or space-time point pattern,
  extract the coordinates and marks of the points.
}
\usage{
\method{as.hyperframe}{ppx}(x, ...)
\method{as.data.frame}{ppx}(x, ...)
\method{as.matrix}{ppx}(x, ...)
}
\arguments{
  \item{x}{
    A general multidimensional space-time point pattern 
    (object of class \code{"ppx"}).
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  An object of class \code{"ppx"} (see \code{\link{ppx}})
  represents a marked point pattern
  in multidimensional space and/or time. There may be any
  number of spatial coordinates, any number of temporal coordinates,
  and any number of mark variables. The individual marks may be
  atomic (numeric values, factor values, etc) or objects of any kind.

  The function \code{as.hyperframe.ppx} extracts the coordinates
  and the marks as a \code{"hyperframe"} (see
  \code{\link{hyperframe}}) with one row of data for each
  point in the pattern. This is a method for the generic
  function \code{\link{as.hyperframe}}.

  The function \code{as.data.frame.ppx} discards those mark variables
  which are not atomic values, and extracts the coordinates
  and the remaining marks as a \code{data.frame}
  with one row of data for each
  point in the pattern. This is a method for the generic
  function \code{\link{as.data.frame}}.

  Finally \code{as.matrix(x)} is equivalent to
  \code{as.matrix(as.data.frame(x))} for an object of class
  \code{"ppx"}. Be warned that, if there are any columns of non-numeric
  data (i.e. if there are mark variables that are factors),
  the result will be a matrix of character values.
}
\value{
  A \code{hyperframe}, \code{data.frame} or \code{matrix} as appropriate.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{ppx}},
  \code{\link{hyperframe}},
  \code{\link{as.hyperframe}}.
}
\examples{
   df <- data.frame(x=runif(4),y=runif(4),t=runif(4))
   X <- ppx(data=df, coord.type=c("s","s","t"))
   as.data.frame(X)
   val <- runif(4)
   E <- lapply(val, function(s) { rpoispp(s) })
   hf <- hyperframe(t=val, e=as.listof(E))
   Z <- ppx(data=hf, domain=c(0,1))
   as.hyperframe(Z)
   as.data.frame(Z)
}
\keyword{spatial}
\keyword{manip}

\name{densityHeat}
\alias{densityHeat}
\title{
  Kernel Density on a Network using Heat Equation
}
\description{
  Computes a kernel density estimate on a linear network
  using the heat equation.
}
\usage{
  densityHeat(x, sigma, \dots,
              at=c("pixels", "points"), leaveoneout=TRUE,
              weights = NULL,
              dx = NULL, dt = NULL, iterMax = 1e+06, verbose=FALSE)
}
\arguments{
  \item{x}{
    Point pattern on a linear network (object of class \code{"lpp"})
    to be smoothed.
  }
  \item{sigma}{
    Smoothing bandwidth (standard deviation of the kernel)
    in the same units as the spatial coordinates of \code{x}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} determining the
    resolution of the result. (Any other arguments are ignored.)
  }
  \item{at}{
    String specifying whether to compute the intensity values
    at a grid of pixel locations (\code{at="pixels"}) or
    only at the points of \code{x} (\code{at="points"}).
  }
  \item{leaveoneout}{
    Logical value indicating whether to compute a leave-one-out
    estimator. Applicable only when \code{at="points"}.
  }
  \item{weights}{
    Optional. Numeric vector of weights associated with the
    points of \code{x}. Weights may be positive, negative or zero.
  }
  \item{dx}{
    Optional. Spacing of the sampling points along the network.
    A single number giving a distance value in the same units as \code{x}.
  }
  \item{dt}{
    Optional. Time step in the heat equation solver.
    A single number. 
  }
  \item{iterMax}{
    Maximum number of iterations.
  }
  \item{verbose}{
    Logical value specifying whether to print progress reports.
  }
}
\details{
  Kernel smoothing is applied to the points of \code{x}
  using a kernel based on path distances in the network.
  If \code{at="pixels"} (the default),
  the result is a pixel image on the linear network (class
  \code{"linim"}) which can be plotted.
  If \code{at="points"} the result is a numeric vector giving the
  density estimates at the data points of \code{x}. 

  Smoothing is performed using the
  \dQuote{equal-split continuous} rule described in
  Section 9.2.3 of Okabe and Sugihara (2012).
  However, the actual computation is performed rapidly, by solving the classical
  time-dependent heat equation on the network,
  as described in McSwiggan et al (2016).
  Computational time is short, but increases quadratically with
  \code{sigma}.

  If \code{at="points"} and \code{leaveoneout=TRUE}, 
  a leave-one-out estimate is computed at each data point
  (that is, the estimate at each data point \code{x[i]} is based
  on all of the points except \code{x[i]})
  using the truncated series approximation
  of McSwiggan et al (2019).
}
\section{Infinite bandwidth}{
  If \code{sigma=Inf}, the resulting density estimate is 
  constant over all locations,
  and is equal to the average density of points per unit length.
  (If the network is not connected, then this rule
  is applied separately to each connected component of the network).
}
\value{
  A pixel image on the linear network (object of class \code{"linim"}).
}
\references{
  McSwiggan, G., Baddeley, A. and Nair, G. (2016)
  Kernel density estimation on a linear network.
  \emph{Scandinavian Journal of Statistics} \bold{44}, 324--345.

  McSwiggan, G., Baddeley, A. and Nair, G. (2019)
  Estimation of relative risk for events on a linear network.
  \emph{Statistics and Computing} \bold{30}, 469--484.

  Okabe, A. and Sugihara, K. (2012)
  \emph{Spatial analysis along networks}.
  Wiley.
}
\author{
  \adrian and Greg McSwiggan.
}
\seealso{
  \code{\link{density.lpp}}
}
\examples{
  X <- runiflpp(3, simplenet)
  D <- densityHeat(X, 0.2)
  plot(D, style="w", main="", adjust=2)
  densityHeat(X, 0.2, at="points")
  Dw <- densityHeat(X, 0.2, weights=c(1,2,-1))
}
\keyword{spatial}
\keyword{methods}
\keyword{smooth}

\name{by.im}
\alias{by.im}
\title{Apply Function to Image Broken Down by Factor}
\description{
  Splits a pixel image into sub-images and applies a function to each
  sub-image.
}
\usage{
\method{by}{im}(data, INDICES, FUN, ...)
}
\arguments{
  \item{data}{A pixel image (object of class \code{"im"}).}
  \item{INDICES}{Grouping variable.
    Either a tessellation (object of class \code{"tess"}) or
    a factor-valued pixel image.
  }
  \item{FUN}{Function to be applied to each sub-image of \code{data}.}
  \item{\dots}{Extra arguments passed to \code{FUN}.}
}
\details{
  This is a method for the generic function \code{\link{by}} for
  pixel images (class \code{"im"}).

  The pixel image \code{data} is first divided into sub-images according
  to \code{INDICES}. Then the function \code{FUN} is applied to each subset.
  The results of each computation are returned in a list.

  The grouping variable \code{INDICES} may be either
  \itemize{
    \item a tessellation (object of class \code{"tess"}). Each tile of
    the tessellation delineates a subset of the spatial domain.
    \item a pixel image (object of class \code{"im"}) with factor
    values. The levels of the factor determine subsets of the spatial
    domain.
  }
}
\value{
  A list containing the results of each evaluation of \code{FUN}.
}
\seealso{
  \code{\link{split.im}},
  \code{\link{tess}},
  \code{\link{im}}
}
\examples{
  W <- square(1)
  X <- as.im(function(x,y){sqrt(x^2+y^2)}, W)
  Y <- dirichlet(runifpoint(12, W))
  # mean pixel value in each subset
  unlist(by(X, Y, mean))
  # trimmed mean
  unlist(by(X, Y, mean, trim=0.05))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}
\keyword{manip}

\name{inside.boxx}
\alias{inside.boxx}
\title{Test Whether Points Are Inside A Multidimensional Box}
\description{
 Test whether points lie inside or outside
 a given multidimensional box.
}
\usage{
 inside.boxx(\dots, w)
}
\arguments{
  \item{\dots}{
    Coordinates of points to be tested.
    One vector for each dimension (all of same length).
    (Alternatively, a single point pattern object
    of class \code{"\link{ppx}"}
    or its coordinates as a \code{"\link{hyperframe}"})
  }
  \item{w}{A window.
    This should be an object of class \code{\link{boxx}},
    or can be given in any format acceptable to \code{\link{as.boxx}()}.
  }
}
\value{
  Logical vector whose \code{i}th entry is 
  \code{TRUE} if the corresponding point is inside \code{w}. 
}
\details{
  This function tests whether each of the points 
  \code{(x[i],y[i])} lies inside or outside
  the window \code{w} and returns \code{TRUE} if it is inside.
  
  The boundary of the window is treated as being inside.

  Normally each argument provided (except \code{w}) must be numeric vectors of
  equal length (length zero is allowed) containing the coordinates
  of points.
  Alternatively a single point pattern (object of class \code{"ppx"})
  can be given; then the coordinates of the point pattern are extracted.
}
\seealso{
  \code{\link{boxx}},
  \code{\link{as.boxx}}
}
\examples{
  # Random points in box with side [0,2]
  w <- boxx(c(0,2), c(0,2), c(0,2))

  # Random points in box with side [-1,3]
  x <- runif(30, min=-1, max=3)
  y <- runif(30, min=-1, max=3)
  z <- runif(30, min=-1, max=3)

  # Points falling in smaller box
  ok <- inside.boxx(x, y, z, w=w)

  # Same using a point pattern as argument:
  X <- ppx(data = cbind(x, y, z), domain = boxx(c(0,3), c(0,3), c(0,3)))
  ok2 <- inside.boxx(X, w=w)
}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{math}

\name{methods.box3}
\Rdversion{1.1}
\alias{methods.box3}  %DoNotExport
\alias{print.box3}
\alias{unitname.box3}
\alias{unitname<-.box3}
\title{
  Methods for Three-Dimensional Box
}
\description{
  Methods for class \code{"box3"}.
}
\usage{
  \method{print}{box3}(x, ...)
  \method{unitname}{box3}(x) 
  \method{unitname}{box3}(x) <- value
}
\arguments{
  \item{x}{
    Object of class \code{"box3"} representing a three-dimensional box.
  }
  \item{\dots}{
    Other arguments passed to \code{print.default}.
  }
  \item{value}{
    Name of the unit of length. See \code{\link{unitname}}.
  }
}
\details{
  These are methods for the generic functions
  \code{\link{print}} and \code{\link{unitname}}
  for the class \code{"box3"} of three-dimensional boxes.

  The \code{print} method prints a description of the box,
  while the \code{unitname} method extracts the name of the unit of
  length in which the box coordinates are expressed.
}
\value{
  For \code{print.box3} the value is \code{NULL}.
  For \code{unitname.box3} an object of class \code{"units"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{box3}},
  \code{\link{print}},
  \code{\link{unitname}}
}
\examples{
   X <- box3(c(0,10),c(0,10),c(0,5), unitname=c("metre", "metres"))
   X
   unitname(X)
   # Northern European usage
   unitname(X) <- "meter"
}
\keyword{spatial}
\keyword{methods}

\name{trim.rectangle}
\alias{trim.rectangle}
\title{Cut margins from rectangle}
\description{
  Trims a margin from a rectangle.
}
\usage{
 trim.rectangle(W, xmargin=0, ymargin=xmargin)
}
\arguments{
  \item{W}{
    A window (object of class \code{"owin"}).
    Must be of type \code{"rectangle"}.
  }
  \item{xmargin}{Width of horizontal margin to be trimmed.
    A single nonnegative number, or a vector of length 2
    indicating margins of unequal width at left and right.
  }
  \item{ymargin}{Height of vertical margin to be trimmed.
    A single nonnegative number, or a vector of length 2
    indicating margins of unequal width at bottom and top.
  }
}
\value{
  Another object of class \code{"owin"} representing the
  window after margins are trimmed.
}
\details{
  This is a simple convenience function to trim off a
  margin of specified width and height from each side of a
  rectangular window. Unequal margins can also be trimmed.
}
\seealso{
  \code{\link{grow.rectangle}},
  \code{\link{erosion}},
  \code{\link{owin.object}}
}
\examples{
  w <- square(10)
  # trim a margin of width 1 from all four sides
  square9 <- trim.rectangle(w, 1)

  # trim margin of width 3 from the right side
  # and margin of height 4 from top edge.
  v <- trim.rectangle(w, c(0,3), c(0,4))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{rectdistmap}
\alias{rectdistmap}
\title{
  Distance Map Using Rectangular Distance Metric
}
\description{
  Computes the distance map of a spatial region
  based on the rectangular distance metric.
}
\usage{
rectdistmap(X, asp = 1, npasses=1, verbose=FALSE)
}
\arguments{
  \item{X}{
    A window (object of class \code{"owin"}).
  }
  \item{asp}{
    Aspect ratio for the metric.
    See Details.
  }
  \item{npasses}{
    Experimental. 
  }
  \item{verbose}{
    Logical value indicating whether to print trace information.
  }
}
\details{
  This function computes the distance map of the
  spatial region \code{X} using the rectangular distance metric
  with aspect ratio \code{asp}. This metric is defined so that
  the set of all points lying at most 1 unit away from the origin
  (according to the metric) form a rectangle of width 1
  and height \code{asp}.
}
\value{
  A pixel image (object of class \code{"im"}).
}
\author{
  \adrian
}
\seealso{
  \code{\link{distmap}}
}
\examples{
  V <- letterR
  Frame(V) <- grow.rectangle(Frame(V), 0.5)
  plot(rectdistmap(V))
}
\keyword{spatial}
\keyword{math}

\name{pppdist}
\alias{pppdist}
\title{Distance Between Two Point Patterns}
\description{
  Given two point patterns, find the distance between them based on
  optimal point matching.
}
\usage{
  pppdist(X, Y, type = "spa", cutoff = 1, q = 1, matching = TRUE,
    ccode = TRUE, auction = TRUE, precision = NULL, approximation = 10,
    show.rprimal = FALSE, timelag = 0)
}
\arguments{
  \item{X,Y}{Two point patterns (objects of class \code{"ppp"}).}
  \item{type}{
    A character string giving the type of distance to be computed.
    One of \code{"spa"} (default), \code{"ace"} or \code{"mat"}, indicating
    whether the algorithm should find the optimal matching based on
    \dQuote{subpattern assignment},
    \dQuote{assignment only if cardinalities are equal}
    or \dQuote{mass transfer}. See Details. 
  }
  \item{cutoff}{
    The value \eqn{> 0} at which interpoint distances are cut off.
  }
  \item{q}{
    The order of the average that is applied to the interpoint distances.
    May be \code{Inf}, in which case the maximum of the interpoint
    distances is taken.
  }
  \item{matching}{
    Logical. Whether to return the optimal matching or only the
    associated distance.
  }
  \item{ccode}{
    Logical. If \code{FALSE}, \R code is used which allows for higher
    precision, but is much slower.
  }
  \item{auction}{
    Logical. By default a version of Bertsekas' auction algorithm
    is used to compute an optimal point matching if \code{type} is
    either \code{"spa"} or \code{"ace"}.
    If \code{auction} is \code{FALSE} (or \code{type} is \code{"mat"})
    a specialized primal-dual algorithm is used instead.
    This was the standard in earlier versions
    of \pkg{spatstat}, but is several orders of magnitudes slower. 
  }
  \item{precision}{
    Index controlling accuracy of algorithm. The \code{q}-th powers of
    interpoint distances will be rounded to the nearest multiple of
    \code{10^(-precision)}. There is a sensible default which depends
    on \code{ccode}.
  }
  \item{approximation}{
    If \code{q = Inf}, compute distance based on the optimal matching for the
    corresponding distance of order \code{approximation}. Can be
    \code{Inf}, but this makes computations extremely slow.
  }
  \item{show.rprimal}{
    Logical. Whether to plot the progress of the primal-dual
    algorithm. If \code{TRUE}, slow primal-dual \R code is used,
    regardless of the arguments \code{ccode} and \code{auction}.
  }
  \item{timelag}{
    Time lag, in seconds, between successive displays of the
    iterative solution of the restricted primal problem.
  }
}
\details{
  Computes the distance between point patterns \code{X} and \code{Y} based
  on finding the matching between them which minimizes the average of
  the distances between matched points
  (if \code{q=1}), the maximum distance between matched points
  (if \code{q=Inf}), and in general the \code{q}-th order average
  (i.e. the \code{1/q}th power of the sum of
  the \code{q}th powers) of the distances between matched points.
  Distances between matched points are Euclidean distances cut off at
  the value of \code{cutoff}.

  The parameter \code{type} controls the behaviour of the algorithm if
  the cardinalities of the point patterns are different. For the type
  \code{"spa"} (subpattern assignment) the subpattern of the point pattern
  with the larger cardinality \eqn{n} that is closest to the point pattern
  with the smaller cardinality \eqn{m} is determined; then the \code{q}-th order
  average is taken over \eqn{n} values: the \eqn{m} distances of matched points
  and \eqn{n-m} "penalty distances" of value \code{cutoff} for
  the unmatched points. For the type \code{"ace"} (assignment only if 
  cardinalities equal) the matching is empty and the distance returned is equal
  to \code{cutoff} if the cardinalities differ. For the
  type \code{"mat"} (mass transfer) each point pattern is assumed
  to have total mass \eqn{m} (= the smaller cardinality) distributed evenly
  among its points; the algorithm finds then the "mass transfer plan" that
  minimizes the \code{q}-th order weighted average of the distances, where 
  the weights are given by the transferred mass divided by \eqn{m}. The
  result is a fractional matching (each match of two points has a weight
  in \eqn{(0,1]}) with the minimized quantity as the associated distance.

  The central problem to be solved is the assignment problem (for types
  \code{"spa"} and \code{"ace"}) or the more general transport problem
  (for type \code{"mat"}). Both are well-known problems in discrete
  optimization, see e.g. Luenberger (2003). 

  For the assignment problem \code{pppdist} uses by default the
  forward/backward version of Bertsekas' auction algorithm with
  automated epsilon scaling; see Bertsekas (1992). The implemented
  version gives good overall performance and can handle point patterns
  with several thousand points. 
  
  For the transport problem a specialized primal-dual algorithm is
  employed; see Luenberger (2003), Section 5.9. The C implementation
  used by default can handle patterns with a few hundreds of points, but
  should not be used with thousands of points. By setting
  \code{show.rprimal = TRUE}, some insight in the working of the
  algorithm can be gained. 

  For a broader selection of optimal transport algorithms that are not
  restricted to spatial point patterns and allow for additional fine
  tuning, we recommend the \R package \pkg{transport}. 
   
  For moderate and large values of \code{q} there can be numerical
  issues based on the fact that the \code{q}-th powers of distances are
  taken and some positive values enter the optimization algorithm as
  zeroes because they are too small in comparison with the larger
  values. In this case the number of zeroes introduced is given in a
  warning message, and it is possible then that the matching obtained is
  not optimal and the associated distance is only a strict upper bound
  of the true distance. As a general guideline (which can be very wrong
  in special situations) a small number of zeroes (up to about 50\% of
  the smaller point pattern cardinality \eqn{m}) usually still results
  in the right matching, and the number can even be quite a bit higher
  and usually still provides a highly accurate upper bound for the
  distance. These numerical problems can be reduced by enforcing (much
  slower) \R code via the argument \code{ccode = FALSE}. 

  For \code{q = Inf} there is no fast algorithm available, which is why
  approximation is normally used: for finding the optimal matching,
  \code{q} is set to the value of \code{approximation}. The
  resulting distance is still given as the maximum rather than the
  \code{q}-th order average in the corresponding distance computation.
  If \code{approximation = Inf}, approximation is suppressed and a very
  inefficient exhaustive search for the best matching is performed.

  The value of \code{precision} should normally not be supplied by the
  user. If \code{ccode = TRUE}, this value is preset to the highest
  exponent of 10 that the C code still can handle (usually \eqn{9}). If
  \code{ccode = FALSE}, the value is preset according to \code{q}
  (usually \eqn{15} if \code{q} is small), which can sometimes be
  changed to obtain less severe warning messages. 
}
\value{
  Normally an object of class \code{pppmatching} that contains detailed
  information about the parameters used and the resulting distance.
  See \code{\link{pppmatching.object}} for details.
  If \code{matching = FALSE}, only the numerical value of the distance
  is returned.
}
\references{
  Bertsekas, D.P. (1992).
  Auction algorithms for network flow problems: a tutorial introduction.
  Computational Optimization and Applications 1, 7-66.

  Luenberger, D.G. (2003). \emph{Linear and nonlinear programming.}
  Second edition. Kluwer.

  Schuhmacher, D. (2014).
  \emph{transport: optimal transport in various forms.}
  R package version 0.6-2 (or later)

  Schuhmacher, D. and Xia, A. (2008).
  A new metric between distributions of point processes.
  \emph{Advances in Applied Probability} \bold{40}, 651--672

  Schuhmacher, D., Vo, B.-T. and Vo, B.-N. (2008).
  A consistent metric for performance evaluation of multi-object
  filters.
  \emph{IEEE Transactions on Signal Processing} \bold{56}, 3447--3457.
}
\author{
  Dominic Schuhmacher
  \email{dominic.schuhmacher@mathematik.uni-goettingen.de} \cr
  \url{http://www.dominic.schuhmacher.name}
}
\seealso{
  \code{\link{pppmatching.object}}, \code{\link{matchingdist}},
  \code{\link{plot.pppmatching}}
}
\examples{
# equal cardinalities
set.seed(140627)
X <- runifpoint(500)
Y <- runifpoint(500)
m <- pppdist(X, Y)
m
\dontrun{
plot(m)}
  
# differing cardinalities
X <- runifpoint(14)
Y <- runifpoint(10)
m1 <- pppdist(X, Y, type="spa")
m2 <- pppdist(X, Y, type="ace")
m3 <- pppdist(X, Y, type="mat", auction=FALSE)
summary(m1)
summary(m2)
summary(m3)
\dontrun{
m1$matrix
m2$matrix
m3$matrix}

# q = Inf
X <- runifpoint(10)
Y <- runifpoint(10)
mx1 <- pppdist(X, Y, q=Inf, matching=FALSE)
mx2 <- pppdist(X, Y, q=Inf, matching=FALSE, ccode=FALSE, approximation=50)
mx3 <- pppdist(X, Y, q=Inf, matching=FALSE, approximation=Inf)
all.equal(mx1,mx2,mx3)
# sometimes TRUE
all.equal(mx2,mx3)
# very often TRUE
}
\keyword{spatial}
\keyword{math}

\name{is.lpp}
\alias{is.lpp}
\title{Test Whether An Object Is A Point Pattern on a Linear Network}
\description{
  Checks whether its argument is a point pattern
  on a linear network
  (object of class \code{"lpp"}).
}
\usage{
is.lpp(x)
}
\arguments{
  \item{x}{Any object.}
}
\details{
  This function tests whether the object \code{x}
  is a point pattern object of class \code{"lpp"}. 
}
\value{
  \code{TRUE} if \code{x} is a point pattern of class \code{"lpp"},
  otherwise \code{FALSE}.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{rVarGamma}
\alias{rVarGamma}
\title{Simulate Neyman-Scott Point Process with Variance Gamma cluster kernel}
\description{
  Generate a random point pattern, a simulated realisation of the
  Neyman-Scott process with Variance Gamma (Bessel) cluster kernel.
}
\usage{
 rVarGamma(kappa, nu, scale, mu, win = owin(), 
           thresh = 0.001, nsim=1, drop=TRUE,
           saveLambda=FALSE, expand = NULL, ...,
           poisthresh=1e-6, saveparents=TRUE)
}
\arguments{
  \item{kappa}{
    Intensity of the Poisson process of cluster centres.
    A single positive number, a function, or a pixel image.
  }
  \item{nu}{
    Shape parameter for the cluster kernel. A number greater than -1.
  }
  \item{scale}{
    Scale parameter for cluster kernel. Determines the size of clusters.
    A positive number in the same units as the spatial coordinates.
  }
  \item{mu}{
    Mean number of points per cluster (a single positive number)
    or reference intensity for the cluster points (a function or
    a pixel image).
  }
  \item{win}{
    Window in which to simulate the pattern.
    An object of class \code{"owin"}
    or something acceptable to \code{\link{as.owin}}.
  }
  \item{thresh}{
    Threshold relative to the cluster kernel value at the origin (parent
    location) determining when the cluster kernel will be treated as
    zero for simulation purposes. Will be overridden by argument
    \code{expand} if that is given.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
  \item{saveLambda}{
    Logical. If \code{TRUE} then the random intensity corresponding to
    the simulated parent points will also be calculated and saved,
    and returns as an attribute of the point pattern.
  }
  \item{expand}{
    Numeric. Size of window expansion for generation of parent
    points. By default determined by calling
    \code{\link{clusterradius}} with the numeric threshold value given
    in \code{thresh}.
  }
  \item{\dots}{
    Passed to \code{\link{clusterfield}} to control the image resolution
    when \code{saveLambda=TRUE} and to \code{\link{clusterradius}} when
    \code{expand} is missing or \code{NULL}.
  }
  \item{poisthresh}{
    Numerical threshold below which the model will be treated
    as a Poisson process. See Details.
  }
  \item{saveparents}{
    Logical value indicating whether to save the locations of the
    parent points as an attribute.
  }
}
\value{
  A point pattern (an object of class \code{"ppp"}) if \code{nsim=1},
  or a list of point patterns if \code{nsim > 1}.

  Additionally, some intermediate results of the simulation are returned
  as attributes of this point pattern (see
  \code{\link{rNeymanScott}}). Furthermore, the simulated intensity
  function is returned as an attribute \code{"Lambda"}, if
  \code{saveLambda=TRUE}.
}
\details{
  This algorithm generates a realisation of the Neyman-Scott process
  with Variance Gamma (Bessel) cluster kernel, inside the window \code{win}.

  The process is constructed by first
  generating a Poisson point process of ``parent'' points 
  with intensity \code{kappa}. Then each parent point is
  replaced by a random cluster of points, the number of points in each
  cluster being random with a Poisson (\code{mu}) distribution,
  and the points being placed independently and uniformly
  according to a Variance Gamma kernel.

  The shape of the kernel is determined by the dimensionless
  index \code{nu}. This is the parameter
  \eqn{\nu^\prime = \alpha/2-1}{nu' = alpha/2 - 1} appearing in
  equation (12) on page 126 of Jalilian et al (2013).

  The scale of the kernel is determined by the argument \code{scale},
  which is the parameter
  \eqn{\eta}{eta} appearing in equations (12) and (13) of
  Jalilian et al (2013). 
  It is expressed in units of length (the same as the unit of length for 
  the window \code{win}).
  
  In this implementation, parent points are not restricted to lie in the
  window; the parent process is effectively the uniform
  Poisson process on the infinite plane.

  This model can be fitted to data by the method of minimum contrast,
  maximum composite likelihood or Palm likelihood using
  \code{\link{kppm}}.
  
  The algorithm can also generate spatially inhomogeneous versions of
  the cluster process:
  \itemize{
    \item The parent points can be spatially inhomogeneous.
    If the argument \code{kappa} is a \code{function(x,y)}
    or a pixel image (object of class \code{"im"}), then it is taken
    as specifying the intensity function of an inhomogeneous Poisson
    process that generates the parent points.
    \item The offspring points can be inhomogeneous. If the
    argument \code{mu} is a \code{function(x,y)}
    or a pixel image (object of class \code{"im"}), then it is
    interpreted as the reference density for offspring points,
    in the sense of Waagepetersen (2006).
  }
  When the parents are homogeneous (\code{kappa} is a single number)
  and the offspring are inhomogeneous (\code{mu} is a
  function or pixel image), the model can be fitted to data
  using \code{\link{kppm}}, or using \code{\link{vargamma.estK}}
  or \code{\link{vargamma.estpcf}}
  applied to the inhomogeneous \eqn{K} function. 

  If the pair correlation function of the model is very close
  to that of a Poisson process, deviating by less than
  \code{poisthresh}, then the model is approximately a Poisson process,
  and will be simulated as a Poisson process with intensity
  \code{kappa * mu}, using \code{\link{rpoispp}}.
  This avoids computations that would otherwise require huge amounts
  of memory.
}
\seealso{
  \code{\link{rpoispp}},
  \code{\link{rNeymanScott}},
  \code{\link{kppm}}.

  \code{\link{vargamma.estK}},
  \code{\link{vargamma.estpcf}}.
}
\examples{
 # homogeneous
 X <- rVarGamma(30, 2, 0.02, 5)
 # inhomogeneous
 ff <- function(x,y){ exp(2 - 3 * abs(x)) }
 Z <- as.im(ff, W= owin())
 Y <- rVarGamma(30, 2, 0.02, Z)
 YY <- rVarGamma(ff, 2, 0.02, 3)
}
\references{
  Jalilian, A., Guan, Y. and Waagepetersen, R. (2013)
  Decomposition of variance for spatial Cox processes.
  \emph{Scandinavian Journal of Statistics} \bold{40}, 119-137.
  
  Waagepetersen, R. (2007)
  An estimating function approach to inference for
  inhomogeneous Neyman-Scott processes.
  \emph{Biometrics} \bold{63}, 252--258.
}
\author{Abdollah Jalilian and Rasmus Waagepetersen.
  Adapted for \pkg{spatstat} by \adrian
  
  
}
\keyword{spatial}
\keyword{datagen}

\name{edit.hyperframe}
\alias{edit.hyperframe}
\title{
  Invoke Text Editor on Hyperframe
}
\description{
  Invokes a text editor allowing the user to inspect and change
  entries in a hyperframe.
}
\usage{
\method{edit}{hyperframe}(name, \dots)
}
\arguments{
  \item{name}{
    A hyperframe (object of class \code{"hyperframe"}).
  }
  \item{\dots}{
    Other arguments passed to \code{\link[utils]{edit.data.frame}}.
  }
}
\details{
  The function \code{\link[utils]{edit}} is generic. This function
  is the methods for objects of class \code{"hyperframe"}.

  The hyperframe \code{name} is converted to a data frame or array,
  and the text editor is invoked. The user can change entries in the
  columns of data, and create new columns of data.

  Only the columns of atomic data
  (numbers, characters, factor values etc) can be edited.
  
  Note that the original object \code{name} is not changed;
  the function returns the edited dataset.
}
\value{
  Another hyperframe.
}
\author{
\adrian


\rolf

and \ege

}
\seealso{
  \code{\link[utils]{edit.data.frame}},
  \code{\link{edit.ppp}}
}
\examples{
  if(interactive()) Z <- edit(flu)
}
\keyword{spatial}
\keyword{manip}

\name{spatstat.options}
\alias{spatstat.options}
\alias{reset.spatstat.options}
\title{Internal Options in Spatstat Package}
\description{
  Allows the user to examine and reset the values
  of global parameters which control actions in the
  \pkg{spatstat} package.
}
\usage{
  spatstat.options(...)
  reset.spatstat.options()
}
\arguments{
  \item{\dots}{
    Either empty,
    or a succession of parameter names in quotes,
    or a succession of \code{name=value} pairs.
    See below for the parameter names.
  }
}
\value{
  Either a list of parameters and their values,
  or a single value. See Details.
}
\details{
  The function \code{spatstat.options}
  allows the user to examine and reset the values
  of global parameters which control actions in the
  \pkg{spatstat} package.
  It is analogous to the system function \code{\link[base]{options}}.

  The function \code{reset.spatstat.options} resets all the
  global parameters in \pkg{spatstat} to their original,
  default values.

  The global parameters of interest to the user are:
  \describe{
    \item{checkpolygons}{
      Logical flag indicating whether the functions
      \code{\link[spatstat]{owin}} and \code{\link[spatstat]{as.owin}} should
      apply very strict checks on the validity of polygon data. 
      These strict checks are no longer necessary, and the default is
      \code{checkpolygons=FALSE}.
      See also \code{fixpolygons} below.
    }
    \item{checksegments}{
      Logical flag indicating whether the functions
      \code{\link[spatstat]{psp}} and \code{\link[spatstat]{as.psp}} should check
      the validity of line segment data (in particular, checking that
      the endpoints of the line segments are inside the specified
      window). It is advisable to leave this flag set to \code{TRUE}. 
    }
    \item{dpp.maxmatrix}{
      Integer specifying the maximum size of matrices
      generated by \code{\link{dppeigen}}. Defaults to \code{2^24}.
    }
    \item{eroded.intensity}{
      Logical flag affecting the behaviour of the
      score and pseudo-score residual functions
      \code{\link[spatstat]{Gcom}}, \code{\link[spatstat]{Gres}}
      \code{\link[spatstat]{Kcom}}, \code{\link[spatstat]{Kres}}, 
      \code{\link[spatstat]{psstA}}, 
      \code{\link[spatstat]{psstG}}, 
      \code{\link[spatstat]{psst}}.
      The flag indicates whether to compute intensity estimates
      on an eroded window (\code{eroded.intensity=TRUE})
      or on the original data window
      (\code{eroded.intensity=FALSE}, the default). 
    }
    \item{expand}{
      The default expansion factor (area inflation factor)
      for expansion of the simulation window in
      \code{\link[spatstat]{rmh}} (see \code{\link[spatstat]{rmhcontrol}}).
      Initialised to \code{2}.
    }
    \item{expand.polynom}{
      Logical. Whether expressions involving \code{\link[spatstat]{polynom}}
      in a model formula should be expanded, so that
      \code{polynom(x,2)} is replaced by \code{x + I(x^2)} and so on.
      Initialised to \code{TRUE}.
    }
    \item{fastpois}{
      Logical. Whether to use a fast algorithm
      (introduced in \pkg{spatstat 1.42-3}) for
      simulating the Poisson point process in \code{\link[spatstat]{rpoispp}}
      when the argument \code{lambda} is a pixel image.
      Initialised to \code{TRUE}.
      Should be set to \code{FALSE} if needed to guarantee repeatability
      of results computed using earlier versions of \pkg{spatstat}.
    }
    \item{fastthin}{
      Logical. Whether to use a fast C language algorithm
      (introduced in \pkg{spatstat 1.42-3}) for
      random thinning in \code{\link[spatstat]{rthin}} when the argument
      \code{P} is a single number.
      Initialised to \code{TRUE}.
      Should be set to \code{FALSE} if needed to guarantee repeatability
      of results computed using earlier versions of \pkg{spatstat}.
    }
    \item{fastK.lgcp}{
      Logical. Whether to use fast or slow algorithm to compute the
      (theoretical) \eqn{K}-function of a log-Gaussian Cox process
      for use in \code{\link[spatstat]{lgcp.estK}} or \code{\link[spatstat]{Kmodel}}.
      The slow algorithm uses accurate numerical integration; the
      fast algorithm uses Simpson's Rule for numerical integration,
      and is about two orders of magnitude faster.
      Initialised to \code{FALSE}. 
    }
    \item{fixpolygons}{
      Logical flag indicating whether the functions
      \code{\link[spatstat]{owin}} and \code{\link[spatstat]{as.owin}} should
      repair errors in polygon data. For example,
      self-intersecting polygons and overlapping polygons will
      be repaired. The default is \code{fixpolygons=TRUE}.
    }
    \item{fftw}{
      Logical value indicating whether the two-dimensional
      Fast Fourier Transform
      should be computed using the package \pkg{fftwtools},
      instead of the \code{fft} function in the \pkg{stats} package.
      This affects the speed of \code{\link[spatstat]{density.ppp}},
      \code{\link[spatstat]{density.psp}}, \code{\link[spatstat]{blur}}
      \code{\link[spatstat]{setcov}} and \code{\link[spatstat]{Smooth.ppp}}.
    }
    \item{gpclib}{
      Defunct. This parameter was used to permit or forbid the use of the
      package \pkg{gpclib}, because of its restricted software licence.
      This package is no longer needed.
    }
    \item{huge.npoints}{
      The maximum value of \code{n} for which \code{runif(n)} will
      not generate an error (possible errors include failure to allocate
      sufficient memory, and integer overflow of \code{n}).
      An attempt to generate more than this number of random points
      triggers a warning from \code{\link[spatstat]{runifpoint}} and other
      functions. Defaults to \code{1e6}.
    }
    \item{image.colfun}{
      Function determining the default colour map for
      \code{\link[spatstat]{plot.im}}. When called with one integer argument
      \code{n}, this function should return a character vector of length
      \code{n} specifying \code{n} different colours.
    }
    \item{Kcom.remove.zeroes}{
      Logical value, determining whether the algorithm in
      \code{\link[spatstat]{Kcom}} and \code{\link[spatstat]{Kres}}
      removes or retains the contributions to the function
      from pairs of points that are identical. If these are
      retained then the function has a jump at \eqn{r=0}.
      Initialised to \code{TRUE}.
    }
    \item{maxedgewt}{
      Edge correction weights will be trimmed
      so as not to exceed this value.
      This applies to the weights computed by
      \code{\link[spatstat]{edge.Trans}} or \code{\link[spatstat]{edge.Ripley}}
      and used in \code{\link[spatstat]{Kest}} and its relatives.
    }
    \item{maxmatrix}{
      The maximum permitted size (rows times columns)
      of matrices generated by \pkg{spatstat}'s internal code.
      Used by \code{\link[spatstat]{ppm}} and \code{\link[spatstat]{predict.ppm}}
      (for example) to decide when to
      split a large calculation into blocks.
      Defaults to \code{2^24=16777216}. 
    }
    \item{monochrome}{
      Logical flag indicating whether graphics should be
      plotted in grey scale (\code{monochrome=TRUE}) or
      in colour (\code{monochrome=FALSE}, the default).
    }
    \item{n.bandwidth}{
      Integer. Number of trial values of smoothing bandwidth to use for
      cross-validation in \code{\link[spatstat]{bw.relrisk}} and similar
      functions.
    }
    \item{ndummy.min}{
      The minimum number of dummy points in a quadrature scheme
      created by \code{\link[spatstat]{default.dummy}}.
      Either an integer or a pair of integers
      giving the minimum number of dummy points in the \code{x} and \code{y}
      directions respectively. 
    }
    \item{ngrid.disc}{
      Number of points in the square grid used to compute
      a discrete approximation to the areas of discs
      in \code{\link[spatstat]{areaLoss}} and \code{\link[spatstat]{areaGain}}
      when exact calculation is not available.
      A single integer.
    }
    \item{npixel}{
      Default number of pixels in a binary mask or pixel image.
      Either an integer, or a pair of integers,
      giving the number of pixels in the \code{x} and \code{y}
      directions respectively.
    }
    \item{nvoxel}{
      Default number of voxels in a 3D image,
      typically for calculating the distance transform in \code{\link[spatstat]{F3est}}.
      Initialised to 4 megavoxels: \code{nvoxel = 2^22 = 4194304}.
    }
    \item{par.binary}{
      List of arguments to be passed to the function
      \code{\link[graphics]{image}}
      when displaying a binary image mask (in \code{\link[spatstat]{plot.owin}}
      or \code{\link[spatstat]{plot.ppp}}).
      Typically used to reset the colours of foreground and background.
    }
    \item{par.contour}{
      List of arguments controlling contour plots of pixel images
      by \code{\link[spatstat]{contour.im}}.
    }
    \item{par.fv}{
      List of arguments controlling the plotting of functions 
      by \code{\link[spatstat]{plot.fv}} and its relatives.
    }
    \item{par.persp}{
      List of arguments to be passed to the function
      \code{\link[graphics]{persp}}
      when displaying a real-valued image, such as the fitted surfaces
      in \code{\link[spatstat]{plot.ppm}}.
    }
    \item{par.points}{
      List of arguments controlling the plotting of point patterns
      by \code{\link[spatstat]{plot.ppp}}.
    }
    \item{par.pp3}{
      List of arguments controlling the plotting of three-dimensional
      point patterns by \code{\link[spatstat]{plot.pp3}}.
    }
    \item{print.ppm.SE}{
      Default rule used by \code{\link[spatstat]{print.ppm}}
      to decide whether to calculate and print standard errors
      of the estimated coefficients of the model.
      One of the strings \code{"always"}, \code{"never"} or
      \code{"poisson"} (the latter indicating that standard errors
      will be calculated only for Poisson models). The default is
      \code{"poisson"} because the calculation for non-Poisson models
      can take a long time.
    }
    \item{progress}{
      Character string determining the style of progress reports
      printed by \code{\link[spatstat]{progressreport}}. Either
      \code{"tty"}, \code{"tk"} or \code{"txtbar"}.
      For explanation of these options, see \code{\link[spatstat]{progressreport}}.
    }
    \item{project.fast}{
      Logical. If \code{TRUE}, the algorithm of
      \code{\link[spatstat]{project.ppm}}
      will be accelerated using a shorcut.
      Initialised to \code{FALSE}.
    }
    \item{psstA.ngrid}{
      Single integer, 
      controlling the accuracy of the discrete approximation
      of areas computed in the function \code{\link[spatstat]{psstA}}.
      The area of a disc is approximated by counting points on
      an \eqn{n \times n}{n * n} grid.
      Initialised to 32.
    }
    \item{psstA.nr}{
      Single integer,
      determining the number of distances \eqn{r}
      at which the function \code{\link[spatstat]{psstA}} will be evaluated
      (in the default case where argument \code{r} is absent).
      Initialised to 30.
    }
    \item{psstG.remove.zeroes}{
      Logical value, determining whether the algorithm in
      \code{\link[spatstat]{psstG}} 
      removes or retains the contributions to the function
      from pairs of points that are identical. If these are
      retained then the function has a jump at \eqn{r=0}.
      Initialised to \code{TRUE}.
    }
    \item{rmh.p, rmh.q, rmh.nrep}{
      New default values for the parameters \code{p}, \code{q}
      and \code{nrep} in the Metropolis-Hastings simulation
      algorithm. These override the defaults
      in \code{\link[spatstat]{rmhcontrol.default}}.
    }
    \item{scalable}{
      Logical flag indicating whether the new code in \code{rmh.default}
      which makes the results scalable (invariant to change of units)
      should be used.  In order to recover former behaviour (so that
      previous results can be reproduced) set this option equal to
      \code{FALSE}.  See the \dQuote{Warning} section in the help for
      \code{\link[spatstat]{rmh}()} for more detail.
    }
    \item{terse}{
      Integer between 0 and 4.
      The level of terseness (brevity) in printed output
      from many functions in \pkg{spatstat}.
      Higher values mean shorter output.
      A rough guide is the following:
      \tabular{ll}{
	0 \tab Full output\cr
        1 \tab Avoid wasteful output \cr
        2 \tab Remove space between paragraphs\cr
        3 \tab Suppress extras such as standard errors \cr
        4 \tab Compress text, suppress internal warnings
      }
      The value of \code{terse} is initialised to 0.
    }
    \item{transparent}{
      Logical value indicating whether default colour maps
      are allowed to include semi-transparent colours, where possible.
      Default is \code{TRUE}.
      Currently this only affects \code{\link[spatstat]{plot.ppp}}. 
    }
    \item{units.paren}{
      The kind of parenthesis which encloses the text that
      explains a \code{unitname}. This text is seen in
      the text output of functions like \code{\link[spatstat]{print.ppp}} and
      in the graphics generated by \code{\link[spatstat]{plot.fv}}.
      The value should be one of the character strings \code{'('}, \code{'['},
      \code{'{'} or \code{''}. The default is \code{'('}.
    }
  }
  If no arguments are given, 
  the current values of all parameters are returned,
  in a list.

  If one parameter name is given, the current value of this
  parameter is returned (\bold{not} in a list, just the value).
  
  If several parameter names are given,
  the current values of these parameters are returned, in a list.
  
  If \code{name=value} pairs are given, the named parameters
  are reset to the given values, and the \bold{previous} values of
  these parameters are returned, in a list.
}
\section{Internal parameters}{
  The following parameters may also be specified to
  \code{spatstat.options} but are intended for
  software development or testing purposes.
  \describe{
    \item{closepairs.newcode}{
      Logical. Whether to use new version of the
      code for \code{\link[spatstat]{closepairs}}.
      Initialised to \code{TRUE}.
    }
    \item{crossing.psp.useCall}{
      Logical. Whether to use new version of the
      code for \code{\link[spatstat]{crossing.psp}}.
      Initialised to \code{TRUE}.
    }
    \item{crosspairs.newcode}{
      Logical. Whether to use new version of the
      code for \code{\link[spatstat]{crosspairs}}.
      Initialised to \code{TRUE}.
    }
    \item{densityC}{
      Logical. 
      Indicates whether to use accelerated C code
      (\code{densityC=TRUE}) or interpreted R code (\code{densityC=FALSE})
      to evaluate \code{density.ppp(X, at="points")}.
      Initialised to \code{TRUE}.
    }
    \item{exactdt.checks.data}{
      Logical.
      Do not change this value, unless you are
      \adrian. 
    }
    \item{fasteval}{
      One of the strings \code{'off'}, \code{'on'} or
      \code{'test'} determining whether to use accelerated C code
      to evaluate the conditional intensity of a Gibbs model.
      Initialised to \code{'on'}.
    }
    \item{old.morpho.psp}{
      Logical. Whether to use old R code for
      morphological operations.
      Initialise to \code{FALSE}.
    }
    \item{selfcrossing.psp.useCall}{
      Logical. Whether to use new version of the
      code for \code{\link[spatstat]{selfcrossing.psp}}.
      Initialised to \code{TRUE}.
    }
    \item{use.Krect}{
      Logical. Whether to use specialised code for the
      K-function in a rectangular window.
      Initialised to \code{TRUE}.
    }
  }
}
\seealso{
  \code{\link[base]{options}}
}
\examples{
  # save current values whatever they are
  oldopt <- spatstat.options()

  spatstat.options("npixel")
  spatstat.options(npixel=150)
  spatstat.options(npixel=c(100,200))

  spatstat.options(par.binary=list(col=grey(c(0.5,1))))

  spatstat.options(par.persp=list(theta=-30,phi=40,d=4))
  # see help(persp.default) for other options

  # revert to the state at the beginning of these examples
  spatstat.options(oldopt)

  # revert to 'factory defaults'
  reset.spatstat.options()
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}

\name{pixellate.ppp} 
\alias{pixellate.ppp}
\alias{as.im.ppp}
\title{Convert Point Pattern to Pixel Image}
\description{
  Converts a point pattern to a pixel image. The value in each pixel
  is the number of points falling in that pixel, and is typically
  either 0 or 1.
}
\usage{
\method{pixellate}{ppp}(x, W=NULL, \dots, weights = NULL,
                        padzero=FALSE, fractional=FALSE, preserve=FALSE,
                        DivideByPixelArea=FALSE, savemap=FALSE)

\method{as.im}{ppp}(X, \dots)
}
\arguments{
  \item{x,X}{Point pattern (object of class \code{"ppp"}).}
  \item{\dots}{Arguments passed to \code{\link{as.mask}} to determine
    the pixel resolution}
  \item{W}{Optional window mask (object of class \code{"owin"}) determining
    the pixel raster.
  }
  \item{weights}{Optional vector of weights associated with the points.}
  \item{padzero}{
    Logical value indicating whether to set pixel values
    to zero outside the window. 
  }
  \item{fractional,preserve}{
    Logical values determining the type of discretisation.
    See Details.
  }
  \item{DivideByPixelArea}{
    Logical value, indicating whether the resulting pixel values
    should be divided by the pixel area.
  }
  \item{savemap}{
    Logical value, indicating whether to save information
    about the discretised coordinates of the points of \code{x}.
  }
}
\details{
  The functions \code{pixellate.ppp} and \code{as.im.ppp}
  convert a spatial point pattern \code{x} into a pixel
  image, by counting the number of points (or the total weight of
  points) falling in each pixel. 

  Calling \code{as.im.ppp} is equivalent to
  calling \code{pixellate.ppp} with its default arguments.
  Note that \code{pixellate.ppp} is more general than \code{as.im.ppp}
  (it has additional arguments for greater flexibility).

  The functions \code{as.im.ppp} and \code{pixellate.ppp}
  are methods for the generic functions \code{\link{as.im}}
  and \code{\link{pixellate}} respectively,
  for the class of point patterns.

  The pixel raster (in which points are counted) is determined
  by the argument \code{W} if it is present (for \code{pixellate.ppp} only).
  In this case \code{W} should be a binary mask (a window object of
  class \code{"owin"} with type \code{"mask"}).
  Otherwise the pixel raster is determined by
  extracting the window containing \code{x} and converting it to a
  binary pixel mask using \code{\link{as.mask}}. The arguments
  \code{\dots} are passed to \code{\link{as.mask}} to
  control the pixel resolution.

  If \code{weights} is \code{NULL}, then for each pixel
  in the mask, the algorithm counts how many points in \code{x} fall
  in the pixel. This count is usually either 0 (for a pixel with no data
  points in it) or 1 (for a pixel containing one data point) but may be
  greater than 1. The result is an image with these counts as its pixel values.

  If \code{weights} is given, it should be a numeric vector of the same
  length as the number of points in \code{x}. For each pixel, the
  algorithm finds the total weight associated with points in \code{x} that fall
  in the given pixel. The result is an image with these total weights
  as its pixel values.

  By default (if \code{zeropad=FALSE}) the resulting pixel image has the same
  spatial domain as the window of the point pattern \code{x}. If
  \code{zeropad=TRUE} then the resulting pixel image has a rectangular
  domain; pixels outside the original window are assigned the value zero.

  The discretisation procedure is controlled by the arguments
  \code{fractional} and \code{preserve}.
  \itemize{
    \item 
    The argument \code{fractional} specifies how data points are mapped to
    pixels. If \code{fractional=FALSE} (the default),
    each data point is allocated to the nearest pixel centre.
    If \code{fractional=TRUE}, each data point is allocated
    with fractional weight to four pixel centres
    (the corners of a rectangle containing the data
    point).
    \item
    The argument \code{preserve} specifies what to do with pixels
    lying near the boundary of the window, if the window is not a rectangle.
    If \code{preserve=FALSE} (the default), any contributions
    that are attributed to pixel centres lying outside the window
    are reset to zero. If \code{preserve=TRUE}, any such contributions
    are shifted to the nearest pixel lying inside the window, so that
    the total mass is preserved.
  }

  If \code{savemap=TRUE} then the result has an attribute
  \code{"map"} which is a 2-column matrix containing the row and column
  indices of the discretised positions of the points of \code{x}
  in the pixel grid.
}
\value{
  A pixel image (object of class \code{"im"}).
}
\seealso{
  \code{\link{pixellate}},
  \code{\link{im}}, \code{\link{as.im}},
  \code{\link{density.ppp}}, \code{\link{Smooth.ppp}}.
}
\examples{
  plot(pixellate(humberside))
  plot(pixellate(humberside, fractional=TRUE))
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{invoke.symbolmap}
\alias{invoke.symbolmap}
\title{
  Plot Data Using Graphics Symbol Map
}
\description{
  Apply a graphics symbol map to a vector of data values and
  plot the resulting symbols.
}
\usage{
invoke.symbolmap(map, values, x=NULL, y = NULL, \dots, add = FALSE,
               do.plot = TRUE, started = add && do.plot)
}
\arguments{
  \item{map}{
    Graphics symbol map (object of class \code{"symbolmap"}).
  }
  \item{values}{
    Vector of data that can be mapped by the symbol map.
  }
  \item{x,y}{
    Coordinate vectors for the spatial locations of the
    symbols to be plotted.
  }
  \item{\dots}{
    Additional graphics parameters.
  }
  \item{add}{
    Logical value indicating whether to add the symbols to
    an existing plot (\code{add=TRUE}) or to initialise a new
    plot (\code{add=FALSE}, the default).
  }
  \item{do.plot}{
    Logical value indicating whether to actually perform the plotting.
  }
  \item{started}{
    Logical value indicating whether the plot has already been initialised.
  }
}
\details{
  A symbol map is an association between data values and graphical symbols. 

  This command applies the symbol map \code{map} to the data 
  \code{values} and plots the resulting symbols at the locations
  given by \code{\link{xy.coords}(x,y)}.
}
\value{
  (Invisibly) the maximum diameter of the symbols, in user coordinate units.
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege.
}
\seealso{
  \code{\link{plot.symbolmap}} to plot the graphics map itself.
  
  \code{\link{symbolmap}} to create a graphics map.
}
\examples{
  g <- symbolmap(range=c(-1,1),
                   shape=function(x) ifelse(x > 0, "circles", "squares"),
                   size=function(x) sqrt(ifelse(x > 0, x/pi, -x))/15,
                   bg=function(x) ifelse(x > 0, "green", "red"))
  plot(square(1), main="")
  a <- invoke.symbolmap(g, runif(10, -1, 1), runifpoint(10), add=TRUE)
  a 
}
\keyword{spatial}
\keyword{hplot}

\name{Extract.owin}
\alias{[.owin}
\title{Extract Subset of Window}
\description{
  Extract a subset of a window.
}
\usage{
  \method{[}{owin}(x, i, \dots)
}
\arguments{
  \item{x}{
    A spatial window (object of class \code{"owin"}).
  }
  \item{i}{
    Object defining the subregion.
    Either a spatial window, or a
    pixel image with logical values.
  }
  \item{\dots}{Ignored.}
}
\value{
  Another spatial window (object of class \code{"owin"}).
}
\details{
  This function computes the intersection
  between the window \code{x} and the domain specified by \code{i},
  using \code{\link{intersect.owin}}.
  
  This function is a method for the subset operator \code{"["} for
  spatial windows (objects of class \code{"owin"}). It is provided
  mainly for completeness.

  The index \code{i} may be either a window, or a pixel image with
  logical values (the \code{TRUE} values of the
  image specify the spatial domain).
}
\seealso{
  \code{\link{intersect.owin}}
}
\examples{
 W <- owin(c(2.5, 3.2), c(1.4, 2.9))
 plot(letterR)
 plot(letterR[W], add=TRUE, col="red")
}
\author{
  \adrian
  and
  \rolf
}
\keyword{spatial}
\keyword{manip}

\name{rStrauss}
\alias{rStrauss}
\title{Perfect Simulation of the Strauss Process}
\description{
  Generate a random pattern of points, a simulated realisation
  of the Strauss process, using a perfect simulation algorithm.
}
\usage{
  rStrauss(beta, gamma = 1, R = 0, W = owin(), expand=TRUE, nsim=1, drop=TRUE)
}
\arguments{
  \item{beta}{
    intensity parameter (a positive number).
  }
  \item{gamma}{
    interaction parameter (a number between 0 and 1, inclusive).
  }
  \item{R}{
    interaction radius (a non-negative number).
  }
  \item{W}{
    window (object of class \code{"owin"}) in which to
    generate the random pattern. 
  }
  \item{expand}{
    Logical. If \code{FALSE}, simulation is performed
    in the window \code{W}, which must be rectangular.
    If \code{TRUE} (the default), simulation is performed
    on a larger window, and the result is clipped to the original
    window \code{W}.
    Alternatively \code{expand} can be an object of class 
    \code{"rmhexpand"} (see \code{\link{rmhexpand}})
    determining the expansion method.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\details{
  This function generates a realisation of the 
  Strauss point process in the window \code{W}
  using a \sQuote{perfect simulation} algorithm.

  The Strauss process (Strauss, 1975; Kelly and Ripley, 1976)
  is a model for spatial inhibition, ranging from
  a strong `hard core' inhibition to a completely random pattern
  according to the value of \code{gamma}.

  The Strauss process with interaction radius \eqn{R} and 
  parameters \eqn{\beta}{beta} and \eqn{\gamma}{gamma}
  is the pairwise interaction point process
  with probability density 
  \deqn{
    f(x_1,\ldots,x_n) =
    \alpha \beta^{n(x)} \gamma^{s(x)}
  }{
    f(x_1,\ldots,x_n) =
    alpha . beta^n(x) gamma^s(x)
  }
  where \eqn{x_1,\ldots,x_n}{x[1],\ldots,x[n]} represent the 
  points of the pattern, \eqn{n(x)} is the number of points in the
  pattern, \eqn{s(x)} is the number of distinct unordered pairs of
  points that are closer than \eqn{R} units apart,
  and \eqn{\alpha}{alpha} is the normalising constant.
  Intuitively, each point of the pattern
  contributes a factor \eqn{\beta}{beta} to the 
  probability density, and each pair of points
  closer than \eqn{r} units apart contributes a factor
  \eqn{\gamma}{gamma} to the density.

  The interaction parameter \eqn{\gamma}{gamma} must be less than
  or equal to \eqn{1} in order that the process be well-defined
  (Kelly and Ripley, 1976).
  This model describes an ``ordered'' or ``inhibitive'' pattern.
  If \eqn{\gamma=1}{gamma=1} it reduces to a Poisson process
  (complete spatial randomness) with intensity \eqn{\beta}{beta}.
  If \eqn{\gamma=0}{gamma=0} it is called a ``hard core process''
  with hard core radius \eqn{R/2}, since no pair of points is permitted
  to lie closer than \eqn{R} units apart.

  The simulation algorithm used to generate the point pattern
  is \sQuote{dominated coupling from the past}
  as implemented by Berthelsen and \Moller (2002, 2003).
  This is a \sQuote{perfect simulation} or \sQuote{exact simulation}
  algorithm, so called because the output of the algorithm is guaranteed
  to have the correct probability distribution exactly (unlike the
  Metropolis-Hastings algorithm used in \code{\link{rmh}}, whose output
  is only approximately correct).

  There is a tiny chance that the algorithm will 
  run out of space before it has terminated. If this occurs, an error
  message will be generated.
}
\value{
  If \code{nsim = 1}, a point pattern (object of class \code{"ppp"}).
  If \code{nsim > 1}, a list of point patterns.
}
\references{
  Berthelsen, K.K. and \Moller, J. (2002)
  A primer on perfect simulation for spatial point processes.
  \emph{Bulletin of the Brazilian Mathematical Society} 33, 351-367.

  Berthelsen, K.K. and \Moller, J. (2003)
  Likelihood and non-parametric Bayesian MCMC inference 
  for spatial point processes based on perfect simulation and
  path sampling. 
  \emph{Scandinavian Journal of Statistics} 30, 549-564.

  Kelly, F.P. and Ripley, B.D. (1976)
  On Strauss's model for clustering.
  \emph{Biometrika} \bold{63}, 357--360.

  \Moller, J. and Waagepetersen, R. (2003).
  \emph{Statistical Inference and Simulation for Spatial Point Processes.}
  Chapman and Hall/CRC.

  Strauss, D.J. (1975)
  A model for clustering.
  \emph{Biometrika} \bold{62}, 467--475.
}
\author{
  Kasper Klitgaard Berthelsen,
  adapted for \pkg{spatstat} by \adrian
}
\examples{
   X <- rStrauss(0.05,0.2,1.5,square(141.4))
   Z <- rStrauss(100,0.7,0.05, nsim=2)
}
\seealso{
  \code{\link{rmh}},
  \code{\link{Strauss}},
  \code{\link{rHardcore}},
  \code{\link{rStraussHard}},
  \code{\link{rDiggleGratton}},
  \code{\link{rDGS}},
  \code{\link{rPenttinen}}.
}

\keyword{spatial}
\keyword{datagen}

\name{plot.pp3}
\Rdversion{1.1}
\alias{plot.pp3}
\title{
  Plot a Three-Dimensional Point Pattern
}
\description{
  Plots a three-dimensional point pattern.
}
\usage{
 \method{plot}{pp3}(x, ..., eye=NULL, org=NULL, theta=25, phi=15,
                    type=c("p", "n", "h"),
                    box.back=list(col="pink"),
                    box.front=list(col="blue", lwd=2))
}
\arguments{
  \item{x}{
    Three-dimensional point pattern (object of class \code{"pp3"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link[graphics]{points}}
    controlling the appearance of the points.
  }
  \item{eye}{
    Optional. Eye position. A numeric vector of length 3 giving the
    location from which the scene is viewed.
  }
  \item{org}{
    Optional. Origin (centre) of the view. A numeric vector of length
    3 which will be at the centre of the view.
  }
  \item{theta,phi}{
    Optional angular coordinates (in degrees) specifying the direction
    from which the scene is viewed: \code{theta} is the azimuth
    and \code{phi} is the colatitude. Ignored if \code{eye} is given.
  }
  \item{type}{
    Type of plot: \code{type="p"} for points,
    \code{type="h"} for points on vertical lines,
    \code{type="n"} for box only.
  }
  \item{box.front,box.back}{
    How to plot the three-dimensional box that contains the points.
    A list of graphical arguments passed to \code{\link[graphics]{segments}},
    or a logical value indicating whether or not to plot
    the relevant part of the box. See Details.
  }
}
\details{
  This is the plot method for objects of class \code{"pp3"}.
  It generates a two-dimensional plot of the point pattern \code{x}
  and its containing box as if they had been viewed from the
  location specified by \code{eye} (or from the direction
  specified by \code{theta} and \code{phi}). 

  The edges of the box at the \sQuote{back} of the scene (as viewed from the
  eye position) are plotted first. Then the points are added. Finally the
  remaining \sQuote{front} edges are plotted. The arguments
  \code{box.back} and \code{box.front} specify graphical parameters for
  drawing the back and front edges, respectively. Alternatively
  \code{box.back=FALSE} specifies that the back edges shall not be drawn.
  
  Note that default values of arguments to \code{plot.pp3}
  can be set by \code{\link{spatstat.options}("par.pp3")}.
}
\value{Null.}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{pp3}},
  \code{\link{spatstat.options}}.
}
\examples{
  X <- osteo$pts[[1]]
  plot(X, main="Osteocyte lacunae, animal 1, brick 1",
       cex=1.5, pch=16)
  plot(X, type="h", main="", box.back=list(lty=3))
}
\keyword{spatial}
\keyword{hplot}

\name{distmap.owin}  
\alias{distmap.owin}
\title{Distance Map of Window}
\description{
  Computes the distance from each pixel to the nearest point
  in the given window.
}
\usage{
  \method{distmap}{owin}(X, \dots, discretise=FALSE, invert=FALSE)
}
\arguments{
  \item{X}{
    A window (object of class \code{"owin"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}}
    to control pixel resolution.
  }
  \item{discretise}{
    Logical flag controlling the choice of algorithm when \code{X} is
    a polygonal window. See Details.
  }
  \item{invert}{
    If \code{TRUE}, compute the distance transform of the
    complement of the window.
  }
}
\value{
  A pixel image (object of class \code{"im"}) whose greyscale values
  are the values of the distance map.
  The return value has an attribute \code{"bdry"}
  which is a pixel image.
}
\details{
  The ``distance map'' of a window \eqn{W} is the function
  \eqn{f} whose value \code{f(u)} is defined for any two-dimensional
  location \eqn{u} as the shortest distance from \eqn{u} to \eqn{W}.

  This function computes the distance map of the window \code{X}
  and returns the distance map as a pixel image. The greyscale value
  at a pixel \eqn{u} equals the distance from \eqn{u} to the nearest
  pixel in \code{X}.

  Additionally, the return value 
  has an attribute \code{"bdry"} which is 
  also a pixel image. The grey values in \code{"bdry"} give the
  distance from each pixel to the bounding rectangle of the image.

  If \code{X} is a binary pixel mask,
  the distance values computed are not the
  usual Euclidean distances. Instead the distance between two pixels
  is measured by the length of the
  shortest path connecting the two pixels. A path is a series of steps
  between neighbouring pixels (each pixel has 8 neighbours). 
  This is the standard `distance transform' algorithm of image
  processing (Rosenfeld and Kak, 1968; Borgefors, 1986).

  If \code{X} is a polygonal window, then exact Euclidean distances
  will be computed if \code{discretise=FALSE}. If \code{discretise=TRUE}
  then the window will first be converted to a binary pixel mask
  and the discrete path distances will be computed.

  The arguments \code{\dots} are passed to \code{\link{as.mask}}
  to control the pixel resolution.
  
  This function is a method for the generic \code{\link{distmap}}.
}
\seealso{
  \code{\link{distmap}},
  \code{\link{distmap.ppp}},
  \code{\link{distmap.psp}}
}
\examples{
  data(letterR)
  U <- distmap(letterR)
  \dontrun{
  plot(U)
  plot(attr(U, "bdry"))
  }
}
\references{
  Borgefors, G.
  Distance transformations in digital images.
  \emph{Computer Vision, Graphics and Image Processing} \bold{34}
  (1986) 344--371.
  
  Rosenfeld, A. and Pfalz, J.L.
  Distance functions on digital pictures.
  \emph{Pattern Recognition} \bold{1} (1968) 33-61.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{plot.quad}
\alias{plot.quad}
\title{Plot a Spatial Quadrature Scheme}
\description{
  Plot a two-dimensional spatial quadrature scheme.
}
\usage{
  \method{plot}{quad}(x, ..., main, add=FALSE, dum=list(), tiles=FALSE)
}
\arguments{
  \item{x}{
    The spatial quadrature scheme to be plotted.
    An object of class \code{"quad"}.
  }
  \item{\dots}{
    extra arguments controlling the plotting
    of the data points of the quadrature scheme.
  }
  \item{main}{
    text to be displayed as a title above the plot.
  }
  \item{add}{
    Logical value indicating whether the graphics should be added to the
    current plot if there is one (\code{add=TRUE})
    or whether a new plot should be initialised (\code{add=FALSE}, the default).
  }
  \item{dum}{
    list of extra arguments controlling the plotting of the dummy points of the
    quadrature scheme. See below.
  }
  \item{tiles}{
    Logical value indicating whether to display the tiles used to
    compute the quadrature weights.
  }
}
\value{
  \code{NULL}.
}
\details{
  This is the \code{plot} method for 
  quadrature schemes (objects of class \code{"quad"},
  see \code{\link{quad.object}}).

  First the data points of the quadrature scheme
  are plotted (in their observation window) using 
  \code{\link{plot.ppp}} with any arguments specified in \code{...}

  Then the dummy points of the quadrature scheme are plotted
  using \code{\link{plot.ppp}} with any arguments specified in
  \code{dum}.

  By default the dummy points are superimposed onto the plot of data
  points. This can be overridden by including the argument
  \code{add=FALSE} in the list \code{dum} as shown in the examples.
  In this case the data and dummy point patterns are plotted separately.

  See \code{\link[graphics]{par}} and \code{\link{plot.ppp}}
  for other possible arguments controlling the plots.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{plot.ppp}},
  \code{\link[graphics]{par}}
}
\examples{
   data(nztrees)
   Q <- quadscheme(nztrees)

   plot(Q, main="NZ trees: quadrature scheme")

   oldpar <- par(mfrow=c(2,1))
   plot(Q, main="NZ trees", dum=list(add=FALSE))
   par(oldpar)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
 
 \keyword{hplot}

\name{weighted.median}
\alias{weighted.median}
\alias{weighted.quantile}
\alias{weighted.var}
\title{
  Weighted Median, Quantiles or Variance
}
\description{
  Compute the median, quantiles or variance of a set of numbers which have
  weights associated with them.
}
\usage{
weighted.median(x, w, na.rm = TRUE)

weighted.quantile(x, w, probs=seq(0,1,0.25), na.rm = TRUE)

weighted.var(x, w, na.rm = TRUE)
}
\arguments{
  \item{x}{
    Data values.
    A vector of numeric values, for which the median or quantiles are required.
  }
  \item{w}{
    Weights.
    A vector of nonnegative numbers, of the same length as \code{x}.
  }
  \item{probs}{
    Probabilities for which the quantiles should be computed.
    A numeric vector of values between 0 and 1.
  }
  \item{na.rm}{
    Logical. Whether to ignore \code{NA} values.
  }
}
\details{
  The \code{i}th observation \code{x[i]} is treated as having
  a weight proportional to \code{w[i]}.

  The weighted median is a value \code{m}
  such that the total weight of data to the left of \code{m}
  is equal to half the total weight.
  If there is no such value, linear interpolation is performed.
}
\value{
  A numeric value or vector.
}
\author{
  \adrian.
}
\seealso{
  \code{\link[stats]{quantile}}, \code{\link[stats]{median}}.
}
\examples{
  x <- 1:20
  w <- runif(20)
  weighted.median(x, w)
  weighted.quantile(x, w)
  weighted.var(x, w)
}
\keyword{math}

\name{round.ppp}
\alias{round.ppp}
\alias{round.pp3}
\alias{round.ppx}
\title{
  Apply Numerical Rounding to Spatial Coordinates
}
\description{
  Apply numerical rounding
  to the spatial coordinates of a point pattern.
}
\usage{
\method{round}{ppp}(x, digits = 0)

\method{round}{pp3}(x, digits = 0)

\method{round}{ppx}(x, digits = 0)
}
\arguments{
  \item{x}{
    A spatial point pattern in any dimension
    (object of class \code{"ppp"}, \code{"pp3"} or \code{"ppx"}).
  }
  \item{digits}{
    integer indicating the number of decimal places. 
  }
}
\details{
  These functions are methods for the generic function
  \code{\link[base]{round}}.
  They apply numerical rounding to the spatial coordinates of the
  point pattern \code{x}. 
}
\value{
  A point pattern object, of the same class as \code{x}.
}
\author{
  \adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{rounding}} to determine whether numbers have been
  rounded.
  
  \code{\link[base]{round}} in the Base package.
}
\examples{
  round(cells, 1)
}
\keyword{spatial}
\keyword{manip}

\name{pixellate}
\Rdversion{1.1}
\alias{pixellate}
\title{
  Convert Spatial Object to Pixel Image
}
\description{
  Convert a spatial object to 
  a pixel image by measuring the amount of stuff
  in each pixel.
}
\usage{
pixellate(x, ...)
}
\arguments{
  \item{x}{
    Spatial object to be converted.
    A point pattern (object of class \code{"ppp"}),
    a window (object of class \code{"owin"}),
    a line segment pattern (object of class \code{"psp"}),
    or some other suitable data.
}
  \item{\dots}{
    Arguments passed to methods.
}
}
\details{
  The function \code{pixellate} converts a geometrical object \code{x} into
  a pixel image, by measuring the \emph{amount} of \code{x} that is
  inside each pixel.
  
  If \code{x} is a point pattern, \code{pixellate(x)} counts the
  number of points of \code{x} falling in each pixel.
  If \code{x} is a window, \code{pixellate(x)} measures the area
  of intersection of each pixel with the window.

  The function \code{pixellate} is generic, with methods for
  point patterns (\code{\link{pixellate.ppp}}),
  windows (\code{\link{pixellate.owin}}),
  and 
  line segment patterns (\code{\link{pixellate.psp}}),
  See the separate documentation for these methods.

  The related function \code{\link{as.im}}
  also converts \code{x} into a pixel image,
  but typically measures only the presence or absence of
  \code{x} inside each pixel.
}
\value{
  A pixel image (object of class \code{"im"}).
}
\seealso{
  \code{\link{pixellate.ppp}},
  \code{\link{pixellate.owin}},
  \code{\link{pixellate.psp}},
  \code{\link{pixellate.linnet}},
  \code{\link{as.im}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{grow.boxx}
\alias{grow.boxx}
\alias{grow.box3}
\title{Add margins to box in any dimension}
\description{
  Adds a margin to a box of class boxx.
}
\usage{
 grow.boxx(W, left, right = left)
 grow.box3(W, left, right = left)
}
\arguments{
  \item{W}{
    A box (object of class \code{"boxx"} or \code{"box3"}).
  }
  \item{left}{Width of margin to be added to left endpoint
    of box side in every dimension.
    A single nonnegative number, or a vector of same length
    as the dimension of the box to add different left margin in each dimension.
  }
  \item{right}{Width of margin to be added to right endpoint
    of box side in every dimension.
    A single nonnegative number, or a vector of same length
    as the dimension of the box to add different right margin in each dimension.
  }
}
\value{
  Another object of the same class \code{"boxx"} or \code{"box3"}
  representing the window after margins are added.
}
\seealso{
  \code{\link{grow.rectangle}},
  \code{\link{boxx}},
  \code{\link{box3}}
}
\examples{
  w <- boxx(c(0,10), c(0,10), c(0,10), c(0,10))
  # add a margin of size 1 on both sides in all four dimensions
  b12 <- grow.boxx(w, 1)

  # add margin of size 2 at left, and margin of size 3 at right,
  # in each dimension.
  v <- grow.boxx(w, 2, 3)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{as.data.frame.tess}
\alias{as.data.frame.tess}
\title{Convert Tessellation to Data Frame}
\description{
  Converts a spatial tessellation object to a data frame.
}
\usage{
\method{as.data.frame}{tess}(x, \dots)
}
\arguments{
  \item{x}{
    Tessellation (object of class \code{"tess"}).
  }
  \item{\dots}{Further arguments passed to
    \code{\link{as.data.frame.owin}} or
    \code{\link{as.data.frame.im}} and ultimately to
    \code{\link[base:as.data.frame]{as.data.frame.default}}
    to determine the row names and other features.
  }
}
\details{
  This function converts the tessellation \code{x} to a data frame.

  If \code{x} is a pixel image tessellation (a pixel image with factor
  values specifying the tile membership of each pixel) then this
  pixel image is converted to a data frame by
  \code{\link{as.data.frame.im}}. The result is a data frame with
  columns \code{x} and \code{y} giving the pixel coordinates,
  and \code{Tile} identifying the tile containing the pixel.

  If \code{x} is a tessellation consisting of a rectangular grid of tiles
  or a list of polygonal tiles, then each tile is converted to a
  data frame by \code{\link{as.data.frame.owin}}, and these data frames
  are joined together, yielding a single large data frame containing
  columns \code{x}, \code{y} giving the coordinates of vertices of the
  polygons, and \code{Tile} identifying the tile.
}
\value{
  A data frame with columns named \code{x}, \code{y}, \code{Tile},
  and possibly other columns.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{as.data.frame.owin}},
  \code{\link{as.data.frame.im}}
}
\examples{
  Z <- as.data.frame(dirichlet(cells))
  head(Z, 10)
}
\keyword{spatial}
\keyword{methods}

\name{infline}
\alias{infline}
\alias{plot.infline}
\alias{print.infline}
\title{Infinite Straight Lines}
\description{
  Define the coordinates of one or more straight lines in the plane
}
\usage{
infline(a = NULL, b = NULL, h = NULL, v = NULL, p = NULL, theta = NULL)

\method{print}{infline}(x, \dots)

\method{plot}{infline}(x, \dots)
}
\arguments{
  \item{a,b}{Numeric vectors of equal length giving the
    intercepts \eqn{a} and slopes \eqn{b} of the lines.
    Incompatible with \code{h,v,p,theta}
  }
  \item{h}{Numeric vector giving the positions of horizontal lines when
    they cross the \eqn{y} axis.
    Incompatible with \code{a,b,v,p,theta}
  }
  \item{v}{Numeric vector giving the positions of vertical lines when
    they cross the \eqn{x} axis.
    Incompatible with \code{a,b,h,p,theta}
  }
  \item{p,theta}{Numeric vectors of equal length
    giving the polar coordinates of the line.
    Incompatible with \code{a,b,h,v}
  }
  \item{x}{An object of class \code{"infline"}}
  \item{\dots}{
    Extra arguments passed to \code{\link[base]{print}}
    for printing or \code{\link[graphics]{abline}} for plotting
  }
}
\details{
  The class \code{infline} is a convenient way to handle
  infinite straight lines in the plane.

  The position of a line can be specified in several ways:
  \itemize{
    \item
    its intercept \eqn{a} and slope \eqn{b} 
    in the equation \eqn{y = a + b x}{y = a + b * x}
    can be used unless the line is vertical.
    \item
    for vertical lines we can use the
    position \eqn{v} where the line crosses the \eqn{y} axis
    \item
    for horizontal lines we can use the
    position \eqn{h} where the line crosses the \eqn{x} axis
    \item
    the polar coordinates \eqn{p} and \eqn{\theta}{theta}
    can be used for any line. The line equation is
    \deqn{
      x \cos\theta + y \sin\theta = p
    }{
      x * cos(theta) + y * sin(theta) = p
    }
  }
  The command \code{infline} will accept line coordinates in any
  of these formats. The arguments \code{a,b,h,v} have the same interpretation
  as they do in the line-plotting function
  \code{\link[graphics]{abline}}.

  The command \code{infline} converts between different coordinate
  systems (e.g. from \code{a,b} to \code{p,theta}) and returns an
  object of class \code{"infline"}
  that contains a representation of the lines in
  each appropriate coordinate system. This object can be printed
  and plotted.
}
\value{
  The value of \code{infline} is an object of class \code{"infline"}
  which is basically a data frame with columns \code{a,b,h,v,p,theta}.
  Each row of the data frame represents one line. 
  Entries may be \code{NA} if a coordinate is not applicable to
  a particular line. 
}
\seealso{
  \code{\link{rotate.infline}},
  \code{\link{clip.infline}},
  \code{\link{chop.tess}},
  \code{\link{whichhalfplane}}
}
\examples{
  infline(a=10:13,b=1)
  infline(p=1:3, theta=pi/4)
  plot(c(-1,1),c(-1,1),type="n",xlab="",ylab="", asp=1)
  plot(infline(p=0.4, theta=seq(0,pi,length=20)))
}
\author{
  \adrian
  and \rolf.
}
\keyword{spatial}
\keyword{datagen}

\name{grow.rectangle}
\alias{grow.rectangle}
\title{Add margins to rectangle}
\description{
  Adds a margin to a rectangle.
}
\usage{
 grow.rectangle(W, xmargin=0, ymargin=xmargin, fraction=NULL)
}
\arguments{
  \item{W}{
    A window (object of class \code{"owin"}).
    Must be of type \code{"rectangle"}.
  }
  \item{xmargin}{Width of horizontal margin to be added.
    A single nonnegative number, or a vector of length 2
    indicating margins of unequal width at left and right.
  }
  \item{ymargin}{Height of vertical margin to be added.
    A single nonnegative number, or a vector of length 2
    indicating margins of unequal width at bottom and top.
  }
  \item{fraction}{
    Fraction of width and height to be added.
    A number greater than zero, or a numeric vector of length 2 indicating
    different fractions of width and of height, respectively.
    Incompatible with specifying \code{xmargin} and \code{ymargin}.
  }
}
\value{
  Another object of class \code{"owin"} representing the
  window after margins are added.
}
\details{
  This is a simple convenience function to add a
  margin of specified width and height on each side of a
  rectangular window. Unequal margins can also be added.
}
\seealso{
  \code{\link{trim.rectangle}},
  \code{\link{dilation}},
  \code{\link{erosion}},
  \code{\link{owin.object}}
}
\examples{
  w <- square(10)
  # add a margin of width 1 on all four sides
  square12 <- grow.rectangle(w, 1)

  # add margin of width 3 on the right side
  # and margin of height 4 on top.
  v <- grow.rectangle(w, c(0,3), c(0,4))

  # grow by 5 percent on all sides
  grow.rectangle(w, fraction=0.05)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{is.im}
\alias{is.im}
\title{Test Whether An Object Is A Pixel Image}
\description{
  Tests whether its argument is a pixel image
  (object of class \code{"im"}).
}
\usage{
is.im(x)
}
\arguments{
  \item{x}{Any object.}
}
\details{
  This function tests whether the argument \code{x} is a
  pixel image object of class \code{"im"}. For details of this
  class, see \code{\link{im.object}}.
  
  The object is determined to be an image if it inherits from
  class \code{"im"}.
}
\value{
  \code{TRUE} if \code{x} is a pixel image, otherwise \code{FALSE}.
}

\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{crossdist.default}
\alias{crossdist.default}
\title{Pairwise distances between two different sets of points}
\description{
  Computes the distances between each pair of points
  taken from two different sets of points.
}
\usage{
  \method{crossdist}{default}(X, Y, x2, y2, \dots,
             period=NULL, method="C", squared=FALSE)
}
\arguments{
  \item{X,Y}{
    Numeric vectors of equal length specifying the coordinates of
    the first set of points.
  }
  \item{x2,y2}{
    Numeric vectors of equal length specifying the coordinates of
    the second set of points.
  }
  \item{\dots}{
    Ignored.
  }
  \item{period}{
    Optional. Dimensions for periodic edge correction.
  }
  \item{method}{
    String specifying which method of calculation to use.
    Values are \code{"C"} and \code{"interpreted"}.
  }
  \item{squared}{
    Logical. If \code{squared=TRUE}, the squared distances are
    returned instead (this computation is faster).
  }
}
\value{
  A matrix whose \code{[i,j]} entry is the distance
  from the \code{i}-th point in the first set of points
  to the \code{j}-th point in the second set of points.
}
\details{
  Given two sets of points,
  this function computes the Euclidean distance from each point
  in the first set to each point in the second set,
  and returns a matrix containing these distances.

  This is a method for the generic function \code{\link{crossdist}}.

  This function expects \code{X} and \code{Y} to be numeric vectors
  of equal length specifying the coordinates of the first set of points.
  The arguments \code{x2},\code{y2} specify the coordinates of the
  second set of points.
  
  Alternatively if \code{period} is given,
  then the distances will be computed in the `periodic'
  sense (also known as `torus' distance).
  The points will be treated as if they are in a rectangle
  of width \code{period[1]} and height \code{period[2]}.
  Opposite edges of the rectangle are regarded as equivalent.
  
  The argument \code{method} is not normally used. It is
  retained only for checking the validity of the software.
  If \code{method = "interpreted"} then the distances are
  computed using interpreted R code only. If \code{method="C"}
  (the default) then C code is used. 
  The C code is faster by a factor of 4.
}
\seealso{
  \code{\link{crossdist}},
  \code{\link{crossdist.ppp}},
  \code{\link{crossdist.psp}},
  \code{\link{pairdist}},
  \code{\link{nndist}},
  \code{\link{Gest}}
}
\examples{
   d <- crossdist(runif(7), runif(7), runif(12), runif(12))
   d <- crossdist(runif(7), runif(7), runif(12), runif(12), period=c(1,1))
}
\author{Pavel Grabarnik
  \email{pavel.grabar@issp.serpukhov.su}
  and
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{as.mask}
\alias{as.mask}
\title{Pixel Image Approximation of a Window}
\description{
  Obtain a discrete (pixel image) approximation of a given window
}
\usage{
 as.mask(w, eps=NULL, dimyx=NULL, xy=NULL)
}
\arguments{
  \item{w}{A window (object of class \code{"owin"}) or data acceptable
    to \code{\link{as.owin}}.}
  \item{eps}{(optional) width and height of pixels.}
  \item{dimyx}{(optional) pixel array dimensions}
  \item{xy}{(optional) data containing pixel coordinates}
}
\value{
  A window (object of class \code{"owin"})
  of type \code{"mask"} representing a binary pixel image.
}
\details{
  This function generates a rectangular grid of locations in the plane,
  tests whether each of these locations lies inside the
  window \code{w}, and stores the results as a binary pixel image
  or `mask' (an object of class \code{"owin"}, see \code{\link{owin.object}}).

  The most common use of this function is to approximate the shape
  of another window \code{w} by a binary pixel image. In this case,
  we will usually want to have a very fine grid of pixels.

  This function can also be used to generate a coarsely-spaced grid of
  locations inside a window, for purposes such as subsampling
  and prediction.

  The grid spacing and location are controlled by the
  arguments \code{eps}, \code{dimyx} and \code{xy},
  which are mutually incompatible.

  If \code{eps} is given, then it determines the grid spacing.
  If \code{eps} is a single number,
  then the grid spacing will be approximately \code{eps}
  in both the \eqn{x} and \eqn{y} directions. If \code{eps} is a
  vector of length 2, then the grid spacing will be approximately
  \code{eps[1]} in the \eqn{x} direction and 
  \code{eps[2]} in the \eqn{y} direction.

  If \code{dimyx} is given, then the pixel grid will be an
  \eqn{m \times n}{m * n} rectangular grid
  where \eqn{m, n} are given by \code{dimyx[2]}, \code{dimyx[1]}
  respectively. \bold{Warning:} \code{dimyx[1]} is the number of
  pixels in the \eqn{y} direction, and \code{dimyx[2]} is the number
  in the \eqn{x} direction. 

  If \code{xy} is given, then this should be some kind of
  data specifing the coordinates of a pixel grid. It may be
  \itemize{
    \item 
    a list or structure containing elements \code{x} and \code{y}
    which are numeric vectors of equal length. These will be taken as
    \eqn{x} and \code{y} coordinates of the margins
    of the grid. The pixel coordinates will be generated
    from these two vectors.
    \item
    a pixel image (object of class \code{"im"}).
    \item
    a window (object of class \code{"owin"}) which is
    of type \code{"mask"} so that it contains pixel coordinates.
  }
  If \code{xy} is given, \code{w} may be omitted.
  
  If neither \code{eps} nor \code{dimyx} nor \code{xy} is given,
  the pixel raster dimensions are obtained from
  \code{\link{spatstat.options}("npixel")}.

  There is no inverse of this function. However, the function
  \code{\link{as.polygonal}} will compute a polygonal approximation
  of a binary mask.
}
\seealso{
  \code{\link{owin.object}},
  \code{\link{as.rectangle}},
  \code{\link{as.polygonal}},
  \code{\link{spatstat.options}}
}
\examples{
  w <- owin(c(0,10),c(0,10), poly=list(x=c(1,2,3,2,1), y=c(2,3,4,6,7)))
  \dontrun{plot(w)}
  m <- as.mask(w)
  \dontrun{plot(m)}
  x <- 1:9
  y <- seq(0.25, 9.75, by=0.5)
  m <- as.mask(w, xy=list(x=x, y=y))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{kppm}
\alias{kppm}
\alias{kppm.formula}
\alias{kppm.ppp}
\alias{kppm.quad}
\concept{point process model}
\concept{Cox point process}
\concept{cluster process}
\concept{Neyman-Scott cluster process}
\title{Fit Cluster or Cox Point Process Model}
\description{
  Fit a homogeneous or inhomogeneous cluster process or
  Cox point process model to a point pattern.
}
\usage{
  kppm(X, \dots)

  \method{kppm}{formula}(X,
                clusters = c("Thomas","MatClust","Cauchy","VarGamma","LGCP"),
                \dots,
                data=NULL)

  \method{kppm}{ppp}(X,
       trend = ~1,
       clusters = c("Thomas","MatClust","Cauchy","VarGamma","LGCP"),
       data = NULL,
       ...,
       covariates=data,
       subset,
       method = c("mincon", "clik2", "palm"),
       improve.type = c("none", "clik1", "wclik1", "quasi"),
       improve.args = list(),
       weightfun=NULL,
       control=list(),
       algorithm="Nelder-Mead",
       statistic="K",
       statargs=list(),
       rmax = NULL,
       covfunargs=NULL,
       use.gam=FALSE,
       nd=NULL, eps=NULL)

\method{kppm}{quad}(X,
       trend = ~1,
       clusters = c("Thomas","MatClust","Cauchy","VarGamma","LGCP"),
       data = NULL,
       ...,
       covariates=data,
       subset,
       method = c("mincon", "clik2", "palm"),
       improve.type = c("none", "clik1", "wclik1", "quasi"),
       improve.args = list(),
       weightfun=NULL,
       control=list(),
       algorithm="Nelder-Mead",
       statistic="K",
       statargs=list(),
       rmax = NULL,
       covfunargs=NULL,
       use.gam=FALSE,
       nd=NULL, eps=NULL)
}
\arguments{
  \item{X}{
    A point pattern dataset (object of class \code{"ppp"} or
    \code{"quad"}) to which the model should be fitted, or a
    \code{formula} in the \R language defining the model. See Details.
  }
  \item{trend}{
    An \R formula, with no left hand side,
    specifying the form of the log intensity.
  }
  \item{clusters}{
    Character string determining the cluster model.
    Partially matched.
    Options are \code{"Thomas"}, \code{"MatClust"},
    \code{"Cauchy"}, \code{"VarGamma"} and \code{"LGCP"}.
  }
  \item{data,covariates}{
    The values of spatial covariates (other than the Cartesian
    coordinates) required by the model.
    A named list of pixel images, functions, windows,
    tessellations or numeric constants.
  }
  \item{\dots}{
    Additional arguments. See Details.
  }
  \item{subset}{
    Optional.
    A subset of the spatial domain,
    to which the model-fitting should be restricted.
    A window (object of class \code{"owin"})
    or a logical-valued pixel image (object of class \code{"im"}),
    or an expression (possibly involving the names of entries in \code{data})
    which can be evaluated to yield a window or pixel image.
  }
  \item{method}{
    The fitting method. Either 
    \code{"mincon"} for minimum contrast,
    \code{"clik2"} for second order composite likelihood,
    or \code{"palm"} for Palm likelihood.
    Partially matched.
  }
  \item{improve.type}{
    Method for updating the initial estimate of the trend.
    Initially the trend is estimated as if the process
    is an inhomogeneous Poisson process.
    The default, \code{improve.type = "none"}, is to use this initial estimate.
    Otherwise, the trend estimate is
    updated by \code{\link{improve.kppm}}, using information
    about the pair correlation function.
    Options are \code{"clik1"}
    (first order composite likelihood, essentially equivalent to \code{"none"}),
    \code{"wclik1"} (weighted first order composite likelihood) and
    \code{"quasi"} (quasi likelihood).
  }
  \item{improve.args}{
    Additional arguments passed to \code{\link{improve.kppm}} when
    \code{improve.type != "none"}. See Details.
  }
  \item{weightfun}{
    Optional weighting function \eqn{w}
    in the composite likelihood or Palm likelihood.
    A \code{function} in the \R language.
    See Details.
  }
  \item{control}{
    List of control parameters passed to the optimization function
    \code{\link[stats]{optim}}.
  }
  \item{algorithm}{
    Character string determining the mathematical optimisation algorithm
    to be used by \code{\link[stats]{optim}}. This argument is
    passed to  \code{\link[stats]{optim}} as the argument \code{method}.
  }
  \item{statistic}{
    Name of the summary statistic to be used
    for minimum contrast estimation: either \code{"K"} or \code{"pcf"}.
  }
  \item{statargs}{
    Optional list of arguments to be used when calculating
    the \code{statistic}. See Details.
  }
  \item{rmax}{
    Maximum value of interpoint distance
    to use in the composite likelihood.
  }
  \item{covfunargs,use.gam,nd,eps}{
    Arguments passed to \code{\link{ppm}} when fitting the intensity.
  }
}
\details{
  This function fits a clustered point process model to the
  point pattern dataset \code{X}. 

  The model may be either a \emph{Neyman-Scott cluster process}
  or another \emph{Cox process}.
  The type of model is determined by the argument \code{clusters}.
  Currently the options 
  are \code{clusters="Thomas"} for the Thomas process,
  \code{clusters="MatClust"} for the \Matern cluster process,
  \code{clusters="Cauchy"} for the Neyman-Scott cluster process
  with Cauchy kernel,
  \code{clusters="VarGamma"} for the Neyman-Scott cluster process
  with Variance Gamma kernel (requires an additional argument \code{nu}
  to be passed through the dots; see \code{\link{rVarGamma}} for details),
  and \code{clusters="LGCP"} for the log-Gaussian Cox process (may
  require additional arguments passed through \code{\dots}; see
  \code{\link{rLGCP}} for details on argument names).
  The first four models are Neyman-Scott cluster processes.
  
  The algorithm first estimates the intensity function
  of the point process using \code{\link{ppm}}.
  The argument \code{X} may be a point pattern
  (object of class \code{"ppp"}) or a quadrature scheme
  (object of class \code{"quad"}). The intensity is specified by
  the \code{trend} argument.
  If the trend formula is \code{~1} (the default)
  then the model is \emph{homogeneous}. The algorithm begins by
  estimating the intensity as the number of points divided by
  the area of the window.
  Otherwise, the model is \emph{inhomogeneous}. 
  The algorithm begins by fitting a Poisson process with log intensity
  of the form specified by the formula \code{trend}.
  (See \code{\link{ppm}} for further explanation).

  The argument \code{X} may also be a \code{formula} in the
  \R language. The right hand side of the formula gives the
  \code{trend} as described above. The left hand side of the formula
  gives the point pattern dataset to which the model should be fitted.

  If \code{improve.type="none"} this is the final estimate of the
  intensity. Otherwise, the intensity estimate is updated, as explained in
  \code{\link{improve.kppm}}. Additional arguments to
  \code{\link{improve.kppm}} are passed as a named list in
  \code{improve.args}.
  
  The clustering parameters of the model are then fitted
  either by minimum contrast estimation, or by maximising a
  composite likelihood.

  \describe{
   \item{Minimum contrast:}{
      If \code{method = "mincon"} (the default) clustering parameters of
      the model will be fitted
      by minimum contrast estimation, that is, by matching the theoretical
      \eqn{K}-function of the model to the empirical \eqn{K}-function
      of the data, as explained in \code{\link{mincontrast}}.

      For a homogeneous model (\code{ trend = ~1 })
      the empirical \eqn{K}-function of the data is computed
      using \code{\link{Kest}},
      and the parameters of the cluster model are estimated by
      the method of minimum contrast.

      For an inhomogeneous model, 
      the inhomogeneous \eqn{K} function is estimated
      by \code{\link{Kinhom}} using the fitted intensity.
      Then the parameters of the cluster model
      are estimated by the method of minimum contrast using the
      inhomogeneous \eqn{K} function. This two-step estimation
      procedure is due to Waagepetersen (2007).
  
      If \code{statistic="pcf"} then instead of using the
      \eqn{K}-function, the algorithm will use
      the pair correlation function \code{\link{pcf}} for homogeneous
      models and the inhomogeneous pair correlation function
      \code{\link{pcfinhom}} for inhomogeneous models.
      In this case, the smoothing parameters of the pair correlation
      can be controlled using the argument \code{statargs},
      as shown in the Examples.

      Additional arguments \code{\dots} will be passed to
      \code{\link{clusterfit}} to control the minimum contrast fitting
      algorithm.
    }
    \item{Composite likelihood:}{
      If \code{method = "clik2"} the clustering parameters of the
      model will be fitted by maximising the second-order composite likelihood
      (Guan, 2006). The log composite likelihood is
      \deqn{
	\sum_{i,j} w(d_{ij}) \log\rho(d_{ij}; \theta)
	- \left( \sum_{i,j} w(d_{ij}) \right)
	\log \int_D \int_D w(\|u-v\|) \rho(\|u-v\|; \theta)\, du\, dv
      }{
	sum[i,j] w(d[i,j]) log(rho(d[i,j]; theta))
	- (sum[i,j] w(d[i,j]))
	log(integral[D,D] w(||u-v||) rho(||u-v||; theta) du dv)
      }
      where the sums are taken over all pairs of data points
      \eqn{x_i, x_j}{x[i], x[j]} separated by a distance
      \eqn{d_{ij} = \| x_i - x_j\|}{d[i,j] = ||x[i] - x[j]||}
      less than \code{rmax},
      and the double integral is taken over all pairs of locations
      \eqn{u,v} in the spatial window of the data.
      Here \eqn{\rho(d;\theta)}{rho(d;theta)} is the
      pair correlation function of the model with
      cluster parameters \eqn{\theta}{theta}.
      
      The function \eqn{w} in the composite likelihood
      is a weighting function and may be chosen arbitrarily.
      It is specified by the argument \code{weightfun}.
      If this is missing or \code{NULL} then the default is
      a threshold weight function,
      \eqn{w(d) = 1(d \le R)}{w(d) = 1(d <= R)}, where \eqn{R} is \code{rmax/2}.
    }
    \item{Palm likelihood:}{
      If \code{method = "palm"} the clustering parameters of the
      model will be fitted by maximising the Palm loglikelihood
      (Tanaka et al, 2008)
      \deqn{
	\sum_{i,j} w(x_i, x_j) \log \lambda_P(x_j \mid x_i; \theta)
	- \int_D w(x_i, u) \lambda_P(u \mid x_i; \theta) {\rm d} u
      }{
	sum[i,j] w(x[i], x[j]) log(lambdaP(x[j] | x[i]; theta)
	- integral[D] w(x[i], u) lambdaP(u | x[i]; theta) du
      }
      with the same notation as above. Here
      \eqn{\lambda_P(u|v;\theta}{lambdaP(u|v;theta)} is the Palm intensity of
      the model at location \eqn{u} given there is a point at \eqn{v}.
    }
  }
  In all three methods, the optimisation is performed by the generic
  optimisation algorithm \code{\link[stats]{optim}}.
  The behaviour of this algorithm can be modified using the
  arguments \code{control} and \code{algorithm}.
  Useful control arguments include
  \code{trace}, \code{maxit} and \code{abstol}
  (documented in the help for \code{\link[stats]{optim}}).

  Fitting the LGCP model requires the \pkg{RandomFields} package,
  except in the default case where the exponential covariance
  is assumed.
}
\section{Log-Gaussian Cox Models}{
  To fit a log-Gaussian Cox model with non-exponential covariance,
  specify \code{clusters="LGCP"} and use additional arguments
  to specify the covariance structure. These additional arguments can
  be given individually in the call to \code{kppm}, or they can be
  collected together in a list called \code{covmodel}.

  For example a \Matern model with parameter \eqn{\nu=0.5} could be specified
  either by \code{kppm(X, clusters="LGCP", model="matern", nu=0.5)} or by
  \code{kppm(X, clusters="LGCP", covmodel=list(model="matern", nu=0.5))}.

  The argument \code{model} specifies the type of covariance
  model: the default is \code{model="exp"} for an exponential covariance.
  Alternatives include \code{"matern"}, \code{"cauchy"} and \code{"spheric"}.
  Model names correspond to functions beginning with \code{RM} in the
  \pkg{RandomFields} package: for example \code{model="matern"}
  corresponds to the function \code{RMmatern} in the 
  \pkg{RandomFields} package.
  
  Additional arguments are passed to the
  relevant function in the \pkg{RandomFields} package:
  for example if \code{model="matern"} then the additional argument
  \code{nu} is required, and is passed to the function
  \code{RMmatern} in the \pkg{RandomFields} package.

  Note that it is not possible to use \emph{anisotropic} covariance models
  because the \code{kppm} technique assumes the pair correlation function
  is isotropic.
}
\value{
  An object of class \code{"kppm"} representing the fitted model.
  There are methods for printing, plotting, predicting, simulating
  and updating objects of this class.
}
\section{Error and warning messages}{
  See \code{\link{ppm.ppp}} for a list of common error messages
  and warnings originating from the first stage of model-fitting.
}
\seealso{
  Methods for \code{kppm} objects:
  \code{\link{plot.kppm}},
  \code{\link{fitted.kppm}},
  \code{\link{predict.kppm}},
  \code{\link{simulate.kppm}},
  \code{\link{update.kppm}},
  \code{\link{vcov.kppm}},
  \code{\link[spatstat:methods.kppm]{methods.kppm}},
  \code{\link{as.ppm.kppm}},
  \code{\link{as.fv.kppm}},
  \code{\link{Kmodel.kppm}},
  \code{\link{pcfmodel.kppm}}.

  Minimum contrast fitting algorithm:
  higher level interface \code{\link{clusterfit}};
  low-level algorithm \code{\link{mincontrast}}.

  Alternative fitting algorithms:
  \code{\link{thomas.estK}},
  \code{\link{matclust.estK}},
  \code{\link{lgcp.estK}},
  \code{\link{cauchy.estK}},
  \code{\link{vargamma.estK}},
  \code{\link{thomas.estpcf}},
  \code{\link{matclust.estpcf}},
  \code{\link{lgcp.estpcf}},
  \code{\link{cauchy.estpcf}},
  \code{\link{vargamma.estpcf}},

  Summary statistics:
  \code{\link{Kest}},
  \code{\link{Kinhom}},
  \code{\link{pcf}},
  \code{\link{pcfinhom}}.

  See also \code{\link{ppm}}
}
\references{
  Guan, Y. (2006) 
  A composite likelihood approach in fitting spatial point process
  models.
  \emph{Journal of the American Statistical Association}
  \bold{101}, 1502--1512.

  Jalilian, A., Guan, Y. and Waagepetersen, R. (2012)
  Decomposition of variance for spatial Cox processes.
  \emph{Scandinavian Journal of Statistics} \bold{40}, 119--137.

  Tanaka, U. and Ogata, Y. and Stoyan, D. (2008)
  Parameter estimation and model selection for
  Neyman-Scott point processes. 
  \emph{Biometrical Journal} \bold{50}, 43--57.

  Waagepetersen, R. (2007)
  An estimating function approach to inference for
  inhomogeneous Neyman-Scott processes.
  \emph{Biometrics} \bold{63}, 252--258.
}
\examples{
  # method for point patterns
  kppm(redwood, ~1, "Thomas")
  # method for formulas
  kppm(redwood ~ 1, "Thomas")

  # different models for clustering
  kppm(redwood ~ x, "MatClust") 
  kppm(redwood ~ x, "MatClust", statistic="pcf", statargs=list(stoyan=0.2)) 
  kppm(redwood ~ x, cluster="Cauchy", statistic="K")
  kppm(redwood, cluster="VarGamma", nu = 0.5, statistic="pcf")

  # log-Gaussian Cox process (LGCP) models
  kppm(redwood ~ 1, "LGCP", statistic="pcf")
  if(require("RandomFields")) {
    # Random Fields package is needed for non-default choice of covariance model
    kppm(redwood ~ x, "LGCP", statistic="pcf",
                              model="matern", nu=0.3,
                              control=list(maxit=10))
  }

  # Different fitting techniques
  kppm(redwood ~ 1, "Thomas", method="c")
  kppm(redwood ~ 1, "Thomas", method="p")
  # composite likelihood method
  kppm(redwood ~ x, "VarGamma", method="clik2", nu.ker=-3/8)
  # quasi-likelihood method
  kppm(redwood ~ x, "Thomas", improve.type = "quasi")

}
\author{
  \spatstatAuthors,
  with contributions from Abdollah Jalilian and Rasmus Waagepetersen.
}
\keyword{spatial}
\keyword{models}

\name{kernel.squint}
\alias{kernel.squint}
\title{Integral of Squared Kernel} 
\description{
  Computes the integral of the squared kernel,
  for the kernels used in density estimation
  for numerical data.
}
\usage{
  kernel.squint(kernel = "gaussian", bw=1)
}
\arguments{
  \item{kernel}{
    String name of the kernel.
    Options are
    \code{"gaussian"}, \code{"rectangular"},
    \code{"triangular"},
    \code{"epanechnikov"},
    \code{"biweight"},
    \code{"cosine"} and \code{"optcosine"}.
    (Partial matching is used).
  }
  \item{bw}{
    Bandwidth (standard deviation) of the kernel.
  }
}
\details{
  Kernel estimation of a probability density in one dimension
  is performed by \code{\link[stats]{density.default}}
  using a kernel function selected from the list above.

  This function computes the integral of the squared kernel,
  \deqn{
    R = \int_{-\infty}^{\infty} k(x)^2 \, {\rm d}x
  }{
    R = integral of k(x)^2 dx from x = -infinity to x = +infinity
  }
  where \eqn{k(x)} is the kernel with bandwidth \code{bw}.
}
\value{
  A single number.
}
\seealso{
  \code{\link[stats]{density.default}},
  \code{\link{dkernel}},
  \code{\link{kernel.moment}},
  \code{\link{kernel.factor}}
}
\examples{
   kernel.squint("gaussian", 3)

   # integral of squared Epanechnikov kernel with half-width h=1
   h <- 1
   bw <- h/kernel.factor("epa")
   kernel.squint("epa", bw)
}
\author{
  \spatstatAuthors
  and Martin Hazelton
}
\keyword{methods}
\keyword{nonparametric}
\keyword{smooth}

\name{as.mask.psp}
\alias{as.mask.psp}
\title{
  Convert Line Segment Pattern to Binary Pixel Mask
}
\description{
  Converts a line segment pattern to a binary pixel mask
  by determining which pixels intersect the lines.
}
\usage{
as.mask.psp(x, W=NULL, ...)
}

\arguments{
  \item{x}{
    Line segment pattern (object of class \code{"psp"}).
  }
  \item{W}{
    Optional window (object of class \code{"owin"})
    determining the pixel raster.
  }
  \item{\dots}{
    Optional extra arguments passed to \code{\link{as.mask}}
    to determine the pixel resolution.
  }
}
\details{
  This function converts a line segment pattern to a binary
  pixel mask by determining which pixels intersect the lines.

  The pixel raster is determined by \code{W}
  and the optional arguments \code{\dots}.
  If \code{W} is missing or \code{NULL}, it defaults to the window
  containing \code{x}.
  Then \code{W} is converted to a
  binary pixel mask using \code{\link{as.mask}}. The arguments
  \code{\dots} are passed to \code{\link{as.mask}} to
  control the pixel resolution.
}
\value{
  A window (object of class \code{"owin"})
  which is a binary pixel mask (type \code{"mask"}).
}
\seealso{
  \code{\link{pixellate.psp}},
  \code{\link{as.mask}}.

  Use \code{\link{pixellate.psp}} if you want to measure the
  length of line in each pixel.
}
\examples{
  X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  plot(as.mask.psp(X))
  plot(X, add=TRUE, col="red")
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{methods.funxy}
\alias{methods.funxy} %DoNotExport
\alias{contour.funxy}
\alias{persp.funxy}
\alias{plot.funxy}
\Rdversion{1.1}
\title{
  Methods for Spatial Functions
}
\description{
  Methods for objects of the class \code{"funxy"}.
}
\usage{
\method{contour}{funxy}(x, \dots)
\method{persp}{funxy}(x, \dots)
\method{plot}{funxy}(x, \dots)
}
\arguments{
  \item{x}{
    Object of class \code{"funxy"} representing a
    function of \eqn{x,y} coordinates.
  }
  \item{\dots}{
    Named arguments controlling the plot. See Details.
  }
}
\details{
  These are methods for the generic functions
  \code{\link{plot}},
  \code{\link{contour}}
  and \code{\link{persp}}
  for the class \code{"funxy"} of spatial functions.

  Objects of class \code{"funxy"} are created, for example,
  by the commands \code{\link{distfun}} and \code{\link{funxy}}.
  
  The \code{plot}, \code{contour} and \code{persp} methods first convert
  \code{x} to a pixel image object using \code{\link{as.im}},
  then display it using \code{\link{plot.im}}, \code{\link{contour.im}} or
  \code{\link{persp.im}}.

  Additional arguments \code{\dots} are either
  passed to \code{\link{as.im.function}} to
  control the spatial resolution of the pixel image, or passed to 
  \code{\link{contour.im}},
  \code{\link{persp.im}} or 
  \code{\link{plot.im}} to control the appearance of the plot. 
}
\value{
  \code{NULL}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{funxy}},
  \code{\link{distfun}},
  \code{\link{as.im}},
  \code{\link{plot.im}},
  \code{\link{persp.im}},
  \code{\link{contour.im}},
  \code{\link{spatstat.options}}
}
\examples{
   f <- distfun(letterR)
   contour(f)
   B <- owin(c(1,5), c(-1, 4))
   contour(f, W=B)
   persp(f, W=B, theta=40, phi=40, border=NA, shade=0.7)
}
\keyword{spatial}
\keyword{methods}

\name{plot.fv}
\alias{plot.fv}
\title{Plot Function Values}
\description{
  Plot method for the class \code{"fv"}.
}
\usage{
 \method{plot}{fv}(x, fmla, \dots, subset=NULL, lty=NULL, col=NULL, lwd=NULL,
           xlim=NULL, ylim=NULL, xlab=NULL, ylab=NULL, ylim.covers=NULL,
           legend=!add, legendpos="topleft", legendavoid=missing(legendpos),
           legendmath=TRUE, legendargs=list(),
           shade=fvnames(x, ".s"), shadecol="grey",
           add=FALSE, log="",
           mathfont=c("italic", "plain", "bold", "bolditalic"), 
           limitsonly=FALSE)
}
\arguments{
  \item{x}{
    An object of class \code{"fv"}, containing the variables to be plotted
    or variables from which the plotting coordinates can be computed.
  }
  \item{fmla}{
    an R language formula 
    determining which variables or expressions are plotted.
    Either a formula object, or a string that can be parsed as a
    formula.
    See Details.
  }
  \item{subset}{
    (optional) subset of rows of the data frame that will be plotted.
  }
  \item{lty}{
    (optional) numeric vector of values of the graphical parameter
    \code{lty} controlling the line style of each plot.
  }
  \item{col}{
    (optional) numeric vector of values of the graphical parameter
    \code{col} controlling the colour of each plot.
  }
  \item{lwd}{
    (optional) numeric vector of values of the graphical parameter
    \code{lwd} controlling the line width of each plot.
  }
  \item{xlim}{
    (optional) range of x axis
  }
  \item{ylim}{
    (optional) range of y axis
  }
  \item{xlab}{
    (optional) label for x axis
  }
  \item{ylab}{
    (optional) label for y axis
  }
  \item{\dots}{
    Extra arguments passed to \code{plot.default}.
  }
  \item{ylim.covers}{
    Optional vector of \eqn{y} values that must be included in the
    \eqn{y} axis. For example \code{ylim.covers=0} will ensure that the
    \eqn{y} axis includes the origin.
  }
  \item{legend}{
    Logical flag or \code{NULL}. If \code{legend=TRUE}, the algorithm
    plots a legend in the top left corner of the plot,
    explaining the meaning of the different line types and colours.
  }
  \item{legendpos}{
    The position of the legend. Either a character string keyword
    (see \code{\link[graphics]{legend}} for keyword options)
    or a pair of coordinates in the format \code{list(x,y)}.
    Alternatively if \code{legendpos="float"}, a location will be
    selected inside the plot region, avoiding the graphics.
  }
  \item{legendavoid}{
    Whether to avoid collisions between the legend and the graphics.
    Logical value.
    If \code{TRUE}, the code will check for collisions between the
    legend box and the graphics, and will override \code{legendpos}
    if a collision occurs.
    If \code{FALSE}, the value of \code{legendpos} is always respected.
  }
  \item{legendmath}{
    Logical. If \code{TRUE}, the legend will display the
    mathematical notation for each curve. If \code{FALSE}, the legend text
    is the identifier (column name) for each curve.
  }
  \item{legendargs}{
    Named list containing additional arguments to be passed to
    \code{\link{legend}} controlling the appearance of the legend.
  }
  \item{shade}{
    A character vector giving the names of two columns of \code{x},
    or another type of index that identifies two columns.
    When the corresponding curves are plotted,
    the region between the curves will be shaded in light grey.
    The object \code{x} may or may not contain two columns which are designated
    as boundaries for shading; they are identified by \code{fvnames(x, ".s")}.
    The default is to shade between these two curves
    if they exist. To suppress this behaviour, set \code{shade=NULL}.
  }
  \item{shadecol}{
    The colour to be used in the \code{shade} plot.
    A character string or an integer specifying a colour.
  }
  \item{add}{
    Logical. Whether the plot should be added to an existing plot
  }
  \item{log}{
    A character string which contains \code{"x"} if the x axis is to
    be logarithmic, \code{"y"} if the y axis is to be logarithmic and
    \code{"xy"} or \code{"yx"} if both axes are to be logarithmic.
  }
  \item{mathfont}{
    Character string. The font to be used for mathematical
    expressions in the axis labels and the legend.
  }
  \item{limitsonly}{
    Logical. If \code{FALSE}, plotting is performed normally.
    If \code{TRUE}, no plotting is performed at all; 
    just the \eqn{x} and \eqn{y} limits of the plot are computed
    and returned. 
  }
}
\value{
  Invisible: either \code{NULL}, or a data frame giving the meaning of the
  different line types and colours.
}
\details{
  This is the \code{plot} method for the class \code{"fv"}.

  The use of the argument \code{fmla} is like \code{plot.formula}, but offers
  some extra functionality.

  The left and right hand sides of \code{fmla} are evaluated,
  and the results are plotted against each other
  (the left side on the \eqn{y} axis 
  against the right side on the \eqn{x} axis).

  The left and right hand sides of \code{fmla} may be
  the names of columns of the data frame \code{x},
  or expressions involving these names. If a variable in \code{fmla}
  is not the name of a column of \code{x}, the algorithm will search for
  an object of this name in the environment where \code{plot.fv} was
  called, and then in the enclosing environment, and so on. 

  Multiple curves may be specified by a single formula
  of the form 
  \code{cbind(y1,y2,\dots,yn) ~ x}, where \code{x,y1,y2,\dots,yn} are
  expressions involving the variables in the data frame.
  Each of the variables \code{y1,y2,\dots,yn} in turn will be plotted
  against \code{x}. 
  See the examples.

  Convenient abbreviations which can be used in the formula
  are 
  \itemize{
    \item the symbol \code{.} which represents all the
    columns in the data frame that will be plotted by default;
    \item the symbol \code{.x} which represents the function argument;
    \item the symbol \code{.y} which represents the recommended value
    of the function.
  }
  For further information, see \code{\link{fvnames}}.

  The value returned by this plot function indicates the
  meaning of the line types and colours in the plot. It can be used
  to make a suitable legend for the plot if you want to do this
  by hand. See the examples.

  The argument \code{shade} can be used to display critical bands
  or confidence intervals. If it is not \code{NULL}, then it should be
  a subset index for the columns of \code{x}, that identifies exactly
  2 columns. When the corresponding curves are plotted, the region
  between the curves will be shaded in light grey. See the Examples.

  The default values of \code{lty}, \code{col} and \code{lwd} can
  be changed using \code{\link{spatstat.options}("plot.fv")}.

  Use \code{type = "n"} to create the plot region and draw the axes
  without plotting any data.

  Use \code{limitsonly=TRUE} to suppress all plotting
  and just compute the \eqn{x} and \eqn{y} limits. This can be used
  to calculate common \eqn{x} and \eqn{y} scales for several plots.

  To change the kind of parenthesis enclosing the
  explanatory text about the unit of length, use
  \code{\link{spatstat.options}('units.paren')}
}
\examples{
   K <- Kest(cells)
   # K is an object of class "fv"

   plot(K, iso ~ r)                # plots iso against r

   plot(K, sqrt(iso/pi) ~ r)   # plots sqrt(iso/r)  against r

   plot(K, cbind(iso,theo) ~ r)   # plots iso against r  AND theo against r

   plot(K, .  ~ r)            # plots all available estimates of K against r

   plot(K, sqrt(./pi) ~ r)   # plots all estimates of L-function
                             # L(r) = sqrt(K(r)/pi)

   plot(K, cbind(iso,theo) ~ r, col=c(2,3))
                                   # plots iso against r  in colour 2
                                   # and theo against r in colour 3

   plot(K, iso ~ r, subset=quote(r < 0.2))
                                   # plots iso against r for r < 10

   # Can't remember the names of the columns? No problem..
   plot(K, sqrt(./pi) ~ .x)

   # making a legend by hand
   v <- plot(K, . ~ r, legend=FALSE)
   legend("topleft", legend=v$meaning, lty=v$lty, col=v$col)

   # significance bands
   KE <- envelope(cells, Kest, nsim=19)
   plot(KE, shade=c("hi", "lo"))

   # how to display two functions on a common scale
   Kr <- Kest(redwood)
   a <- plot(K, limitsonly=TRUE)
   b <- plot(Kr, limitsonly=TRUE)
   xlim <- range(a$xlim, b$xlim)
   ylim <- range(a$ylim, b$ylim)
   opa <- par(mfrow=c(1,2))
   plot(K, xlim=xlim, ylim=ylim)
   plot(Kr, xlim=xlim, ylim=ylim)
   par(opa)
}
\seealso{
  \code{\link{fv.object}},
  \code{\link{Kest}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{hplot}
 

\name{cut.im}
\alias{cut.im}
\title{Convert Pixel Image from Numeric to Factor}
\description{
  Transform the values of a pixel image
  from numeric values into a factor.
}
\usage{
  \method{cut}{im}(x, \dots)
}
\arguments{
  \item{x}{
    A pixel image.
    An object of class \code{"im"}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{cut.default}}.
    They determine the breakpoints for the mapping from numerical values to
    factor values. See \code{\link{cut.default}}.
  }
} 
\value{
  A pixel image (object of class \code{"im"}) with 
  pixel values that are a factor.
  See \code{\link{im.object}}.
}
\details{
  This simple function applies the generic \code{\link{cut}} operation
  to the pixel values of the image \code{x}. The range of pixel values
  is divided into several intervals, and each
  interval is associated with a level of a factor. 
  The result is another pixel image,
  with the same window and pixel grid as
  \code{x}, but with the numeric value of each pixel discretised
  by replacing it by the factor level. 

  This function is a convenient
  way to inspect an image and to obtain summary statistics.
  See the examples.

  To select a subset of an image, use the subset operator
  \code{\link{[.im}} instead.
}
\seealso{
  \code{\link{cut}},
  \code{\link{im.object}}
}
\examples{
  # artificial image data
  Z <- setcov(square(1))

  Y <- cut(Z, 3)
  Y <- cut(Z, breaks=seq(0,1,length=5))

  # cut at the quartiles
  # (divides the image into 4 equal areas)
  Y <- cut(Z, quantile(Z))
}

\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}

\name{selfcrossing.psp}
\alias{selfcrossing.psp}
\title{Crossing Points in a Line Segment Pattern}
\description{
  Finds any crossing points between the line segments in a 
  line segment pattern.
}
\usage{
  selfcrossing.psp(A)
}
\arguments{
  \item{A}{
    Line segment pattern (object of class \code{"psp"}).
  }
}
\value{
  Point pattern (object of class \code{"ppp"}).
}
\details{
  This function finds any crossing points between
  different line segments in the line segment pattern \code{A}.

  A crossing point occurs whenever one of the line segments in \code{A}
  intersects another line segment in \code{A}, at a nonzero
  angle of intersection.
}
\seealso{
  \code{\link{crossing.psp}},
  \code{\link{psp.object}},
  \code{\link{ppp.object}}.
}
\examples{
  a <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  plot(a, col="green", main="selfcrossing.psp")
  P <- selfcrossing.psp(a)
  plot(P, add=TRUE, col="red")
}
\author{
  \adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{contour.imlist}
\alias{contour.imlist}
\alias{contour.listof}
\title{Array of Contour Plots}
\description{
  Generates an array of contour plots.
}
\usage{
  \method{contour}{imlist}(x, \dots)

  \method{contour}{listof}(x, \dots)
}
\arguments{
  \item{x}{
    An object of the class \code{"imlist"}
    representing a list of pixel images.
    Alternatively \code{x} may belong to the outdated class \code{"listof"}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{plot.solist}} to control the
    spatial arrangement of panels, and arguments passed
    to \code{\link{contour.im}} to control the display of each panel.
  }
}
\value{
  Null.
}
\details{
  This is a method for the generic command
  \code{contour} for the class \code{"imlist"}.
  An object of class \code{"imlist"} represents a list of pixel images.

  (The outdated class \code{"listof"} is also handled.)

  Each entry in the list \code{x} will be displayed as a contour plot,
  in an array of panels laid out on the same graphics display,
  using \code{\link{plot.solist}}. Invididual panels are plotted
  by \code{\link{contour.im}}.
}
\seealso{
  \code{\link{plot.solist}},
  \code{\link{contour.im}}
}
\examples{
# Multitype point pattern
 contour(D <- density(split(amacrine)))
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{hplot}

\name{has.close}
\alias{has.close}
\alias{has.close.default}
\alias{has.close.ppp}
\alias{has.close.pp3}
\title{
  Check Whether Points Have Close Neighbours
}
\description{
  For each point in a point pattern, determine whether the
  point has a close neighbour in the same pattern.
}
\usage{
  has.close(X, r, Y=NULL, \dots)

  \method{has.close}{default}(X,r, Y=NULL, \dots, periodic=FALSE)

  \method{has.close}{ppp}(X,r, Y=NULL, \dots, periodic=FALSE, sorted=FALSE)

  \method{has.close}{pp3}(X,r, Y=NULL, \dots, periodic=FALSE, sorted=FALSE)
}
\arguments{
  \item{X,Y}{
    Point patterns of class \code{"ppp"} or \code{"pp3"} or \code{"lpp"}.
  }
  \item{r}{
    Threshold distance: a number greater than zero.
  }
  \item{periodic}{
    Logical value indicating whether to measure distances in the
    periodic sense, so that opposite sides of the (rectangular) window
    are treated as identical.
  }
  \item{sorted}{
    Logical value, indicating whether the points of \code{X}
    (and \code{Y}, if given) are already sorted into increasing order of the
    \eqn{x} coordinates.
  }
  \item{\dots}{Other arguments are ignored.}
}
\details{
  This is simply a faster version of \code{(nndist(X) <= r)}
  or \code{(nncross(X,Y,what="dist") <= r)}.

  \code{has.close(X,r)} determines, for each point in the pattern \code{X},
  whether or not this point has a neighbour in the same pattern \code{X}
  which lies at a distance less than or equal to \code{r}.

  \code{has.close(X,r,Y)} determines, for each point in the pattern \code{X},
  whether or not this point has a neighbour in the \emph{other} pattern
  \code{Y} which lies at a distance less than or equal to \code{r}.

  The function \code{has.close} is generic, with methods for
  \code{"ppp"} and \code{"pp3"} and a default method.
}
\value{
  A logical vector, with one entry for each point of \code{X}.
}
\author{
  \adrian.
}
\seealso{
  \code{\link{nndist}}
}
\examples{
  has.close(redwood, 0.05)
  with(split(amacrine), has.close(on, 0.05, off))
  with(osteo, sum(has.close(pts, 20)))
}
\keyword{spatial}
\keyword{math}

\name{hyperframe}
\alias{hyperframe}
\title{Hyper Data Frame}
\description{
  Create a hyperframe: a two-dimensional array in which each column
  consists of values of the same atomic type (like the columns
  of a data frame) or objects of the same class. 
}
\usage{
   hyperframe(...,
             row.names=NULL, check.rows=FALSE, check.names=TRUE,
             stringsAsFactors=default.stringsAsFactors())
}
\arguments{
  \item{\dots}{
    Arguments of the form \code{value} or \code{tag=value}.
    Each \code{value} is either an atomic vector, or a list of objects of the
    same class, or a single atomic value, or a single object.
    Each \code{value} will become a column of the array.
    The \code{tag} determines the name of the column. See Details.
  }
  \item{row.names,check.rows,check.names,stringsAsFactors}{
    Arguments passed to \code{\link{data.frame}} controlling the
    names of the rows, whether to check that rows are consistent,
    whether to check validity of the column names, and whether to
    convert character columns to factors.
  }
}
\details{
  A hyperframe is like a data frame, except that its entries
  can be objects of any kind.

  A hyperframe is a two-dimensional array in which each column consists of
  values of one atomic type (as in a data frame) or consists of
  objects of one class.

  The arguments \code{\dots} are any number of arguments of
  the form \code{value} or \code{tag=value}. Each \code{value} will
  become a column of the array. The \code{tag} determines the name
  of the column.

  Each \code{value} can be either
  \itemize{
    \item an atomic vector or factor
    (i.e. numeric vector, integer vector, character vector, logical
    vector, complex vector or factor)
    \item a list of objects which are all of the same class
    \item one atomic value, which will be replicated to make an atomic
    vector or factor
    \item one object, which will be replicated to make a list of objects.
  }

  All columns (vectors, factors and lists) must be of the same length,
  if their length is greater than 1. 
}
\section{Methods for Hyperframes}{
  There are methods for
  \code{print}, \code{plot}, \code{summary}, \code{with}, \code{split},
  \code{[}, \code{[<},\code{$}, \code{$<-},
  \code{names}, \code{as.data.frame} \code{as.list},
  \code{cbind} and \code{rbind} for the class of hyperframes. There is also
  \code{is.hyperframe} and \code{\link{as.hyperframe}}.
}
\value{
  An object of class \code{"hyperframe"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{as.hyperframe}},
  \code{\link{as.hyperframe.ppx}},
  \code{\link{plot.hyperframe}},
  \code{\link{[.hyperframe}},
  \code{\link{with.hyperframe}},
  \code{\link{split.hyperframe}},
  \code{\link{as.data.frame.hyperframe}},
  \code{\link{cbind.hyperframe}},
  \code{\link{rbind.hyperframe}}
}
\examples{
 # equivalent to a data frame
  hyperframe(X=1:10, Y=3)

 # list of functions
  hyperframe(f=list(sin, cos, tan))

 # table of functions and matching expressions
  hyperframe(f=list(sin, cos, tan),
             e=list(expression(sin(x)), expression(cos(x)), expression(tan(x))))

  hyperframe(X=1:10, Y=letters[1:10], Z=factor(letters[1:10]),
    stringsAsFactors=FALSE)

  lambda <- runif(4, min=50, max=100)
  X <- lapply(as.list(lambda), function(x) { rpoispp(x) })
  h <- hyperframe(lambda=lambda, X=X)
  h

  h$lambda2 <- lambda^2
  h[, "lambda3"] <- lambda^3
  h[, "Y"] <- X
}
\keyword{spatial}
\keyword{manip}

\name{pairdist.pp3}
\alias{pairdist.pp3}
\title{Pairwise distances in Three Dimensions}
\description{
  Computes the matrix of distances between all pairs of points
  in a three-dimensional point pattern.
}
\usage{
  \method{pairdist}{pp3}(X, \dots, periodic=FALSE, squared=FALSE)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"pp3"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{periodic}{
    Logical. Specifies whether to apply a periodic edge correction.
  }
  \item{squared}{
    Logical. If \code{squared=TRUE}, the squared distances are
    returned instead (this computation is faster).
  }
}
\value{
  A square matrix whose \code{[i,j]} entry is the distance
  between the points numbered \code{i} and \code{j}.
}
\details{
  This is a method for the generic function \code{pairdist}.

  Given a three-dimensional point pattern \code{X}
  (an object of class \code{"pp3"}),
  this function computes the Euclidean distances between all pairs of
  points in \code{X}, and returns the matrix of distances.

  Alternatively if \code{periodic=TRUE} and the window containing \code{X} is a
  box, then the distances will be computed in the `periodic'
  sense (also known as `torus' distance): opposite faces of the
  box are regarded as equivalent.
  This is meaningless if the window is not a box.

  If \code{squared=TRUE} then the \emph{squared} Euclidean distances
  \eqn{d^2} are returned, instead of the Euclidean distances \eqn{d}.
  The squared distances are faster to calculate, and are sufficient for
  many purposes (such as finding the nearest neighbour of a point).
}
\seealso{
  \code{\link{pairdist}},
  \code{\link{crossdist}},
  \code{\link{nndist}},
  \code{\link{K3est}}
}
\examples{
   X <- runifpoint3(20)
   d <- pairdist(X)
   d <- pairdist(X, periodic=TRUE)
   d <- pairdist(X, squared=TRUE)
}
\author{
  \adrian
  
  
  based on two-dimensional code by 
  Pavel Grabarnik.
}
\keyword{spatial}
\keyword{math}

\name{volume}
\alias{volume}
\title{Volume of an Object}
\description{
  Computes the volume of a spatial object such as a three-dimensional box. 
}
\usage{
 volume(x)
}
\arguments{
  \item{x}{
    An object whose volume will be computed.
  }
}
\value{
  The numerical value of the volume of the object.
}
\details{
  This function computes the volume of an object
  such as a three-dimensional box. 

  The function \code{volume} is generic, with methods for
  the classes \code{"box3"} (three-dimensional boxes) and
  \code{"boxx"} (multi-dimensional boxes).

  There is also a method for the class \code{"owin"}
  (two-dimensional windows), which is identical to
  \code{\link{area.owin}},
  and a method for the class \code{"linnet"} of linear networks,
  which returns the length of the network.
}
\seealso{
  \code{\link{area.owin}},
  \code{\link{volume.box3}},
  \code{\link{volume.boxx}},
  \code{\link{volume.linnet}}
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{math}

\name{Window}
\alias{Window}
\alias{Window<-}
\alias{Window.ppp}
\alias{Window<-.ppp}
\alias{Window.psp}
\alias{Window<-.psp}
\alias{Window.im}
\alias{Window<-.im}
\title{
  Extract or Change the Window of a Spatial Object
}
\description{
  Given a spatial object (such as a point pattern or pixel image)
  in two dimensions, these functions extract or change the window
  in which the object is defined.
}
\usage{
   Window(X, \dots)

   Window(X, \dots) <- value

   \method{Window}{ppp}(X, \dots)

   \method{Window}{ppp}(X, \dots) <- value

   \method{Window}{psp}(X, \dots)

   \method{Window}{psp}(X, \dots) <- value

   \method{Window}{im}(X, \dots)

   \method{Window}{im}(X, \dots) <- value
}
\arguments{
  \item{X}{
    A spatial object such as a point pattern, line segment pattern
    or pixel image.
  }
  \item{\dots}{
    Extra arguments. They are ignored by all the methods listed here.
  }
  \item{value}{
    Another window (object of class \code{"owin"}) to be used as the
    window for \code{X}.
  }
}
\details{
  The functions \code{Window} and \code{Window<-} are generic.

  \code{Window(X)} extracts the spatial window in which \code{X} is
  defined.

  \code{Window(X) <- W} changes the window in which \code{X} is defined
  to the new window \code{W}, and \emph{discards any data outside} \code{W}.
  In particular:
  \itemize{
    \item If \code{X} is a point pattern (object of class \code{"ppp"})
    then \code{Window(X) <- W} discards any points of \code{X} which
    fall outside \code{W}.
    \item If \code{X} is a line segment pattern (object of class
    \code{"psp"}) then \code{Window(X) <- W} clips the segments of \code{X}
    to the boundaries of \code{W}.
    \item If \code{X} is a pixel image (object of class \code{"im"})
    then \code{Window(X) <- W} has the effect that pixels
    lying outside \code{W} are retained but their pixel values
    are set to \code{NA}.
  }

  Many other classes of spatial object have a method
  for \code{Window}, but not \code{Window<-}.
  See \code{\link{Window.ppm}}.
}
\value{
  The result of \code{Window} is a window (object of class
  \code{"owin"}).

  The result of \code{Window<-} is the updated object \code{X},
  of the same class as \code{X}.
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link{Window.ppm}}
}
\examples{
   ## point patterns
   Window(cells)
   X <- demopat
   Window(X)
   Window(X) <- as.rectangle(Window(X))

   ## line segment patterns
   X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
   Window(X)
   Window(X) <- square(0.5)

   ## images
   Z <- setcov(owin())
   Window(Z)
   Window(Z) <- square(0.5)
}
\keyword{spatial}
\keyword{manip}

\name{quasirandom}
\alias{quasirandom} %DoNotExport
\alias{vdCorput}
\alias{Halton}
\alias{Hammersley}
\title{
  Quasirandom Patterns
}
\description{
  Generates quasirandom sequences of numbers and
  quasirandom spatial patterns of points in any dimension.
}
\usage{
vdCorput(n, base)

Halton(n, bases = c(2, 3), raw = FALSE, simplify = TRUE)

Hammersley(n, bases = 2, raw = FALSE, simplify = TRUE)
}
\arguments{
  \item{n}{
    Number of points to generate.
  }
  \item{base}{
    A prime number giving the base of the sequence.
  }
  \item{bases}{
    Vector of prime numbers giving the bases of the sequences
    for each coordinate axis.
  }
  \item{raw}{
    Logical value indicating whether to return the coordinates
    as a matrix (\code{raw=TRUE}) or as a spatial point
    pattern (\code{raw=FALSE}, the default).
  }
  \item{simplify}{
    Argument passed to \code{\link{ppx}} indicating whether
    point patterns of dimension 2 or 3 should be
    returned as objects of class \code{"ppp"} or \code{"pp3"}
    respectively (\code{simplify=TRUE}, the default)
    or as objects of class \code{"ppx"} (\code{simplify=FALSE}).
  }
}
\details{
  The function \code{vdCorput} generates the quasirandom sequence
  of Van der Corput (1935) of length \code{n} with the given
  \code{base}. These are numbers between 0 and 1 which are in
  some sense uniformly distributed over the interval.

  The function \code{Halton} generates the Halton quasirandom sequence
  of points in \code{d}-dimensional space, where
  \code{d = length(bases)}. The values of the \eqn{i}-th coordinate
  of the points are generated using the van der Corput sequence with
  base equal to \code{bases[i]}.

  The function \code{Hammersley} generates the Hammersley set
  of points in \code{d+1}-dimensional space, where 
  \code{d = length(bases)}. The first \code{d} coordinates
  of the points are generated using the van der Corput sequence with
  base equal to \code{bases[i]}. The \code{d+1}-th coordinate
  is the sequence \code{1/n, 2/n, ..., 1}.

  If \code{raw=FALSE} (the default) then the Halton and Hammersley
  sets are interpreted as spatial point patterns of the
  appropriate dimension. They are returned as objects of
  class \code{"ppx"} (multidimensional point patterns)
  unless \code{simplify=TRUE} and \code{d=2} or \code{d=3}
  when they are returned as objects of class \code{"ppp"}
  or \code{"pp3"}.
  If \code{raw=TRUE}, the coordinates are returned as a matrix
  with \code{n} rows and \code{D} columns where \code{D} is the spatial
  dimension.
}
\value{
  For \code{vdCorput}, a numeric vector.

  For \code{Halton} and \code{Hammersley},
  an object of class \code{"ppp"}, \code{"pp3"} or
  \code{"ppx"}; or if \code{raw=TRUE}, a numeric matrix.
}
\references{
  Van der Corput, J. G. (1935) Verteilungsfunktionen.
  \emph{Proc. Ned. Akad. v. Wetensch.} \bold{38}: 813--821.

  Kuipers, L. and Niederreiter, H. (2005)
  \emph{Uniform distribution of sequences},
  Dover Publications.
}
\seealso{
  \code{\link{rQuasi}}
}
\examples{
   vdCorput(10, 2)

   plot(Halton(256, c(2,3)))

   plot(Hammersley(256, 3))
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege.
}
\keyword{spatial}
\keyword{datagen}

\name{persp.im}
\alias{persp.im}
\title{Perspective Plot of Pixel Image}
\description{
  Displays a perspective plot of a pixel image.
}
\usage{
  \method{persp}{im}(x, \dots,
                     colmap=NULL, colin=x, apron=FALSE, visible=FALSE)
}
\arguments{
  \item{x}{
    The pixel image to be plotted as a surface.
    An object of class \code{"im"} (see \code{\link{im.object}}).
  }
  \item{\dots}{
    Extra arguments passed to \code{\link{persp.default}} to control the
    display.
  }
  \item{colmap}{
    Optional data controlling the colour map. See Details.
  }
  \item{colin}{
    Optional. Colour input. Another pixel image (of the same dimensions
    as \code{x}) containing the values that will be mapped to colours. 
  }
  \item{apron}{
    Logical. If \code{TRUE}, a grey apron is placed
    around the sides of the perspective plot.
  }
  \item{visible}{
    Logical value indicating whether to compute which pixels of \code{x}
    are visible in the perspective view. See Details.
  }
}
\value{
  (invisibly) the 3D transformation matrix
  returned by \code{\link{persp.default}},
  together with an attribute \code{"expand"} which gives the
  relative scale of the \eqn{z} coordinate.

  If argument \code{visible=TRUE} was given, the return value
  also has an attribute \code{"visible"} which is a pixel image,
  compatible with \code{x}, with logical values which are \emph{TRUE}
  when the corresponding pixel is visible in the perspective view,
  and \code{FALSE} when it is obscured.
}
\details{
  This is the \code{persp} method for the class \code{"im"}.

  The pixel image \code{x} must have real or integer values.
  These values are treated as heights of a surface, and the
  surface is displayed as a perspective plot on the current plot device,
  using equal scales on the \code{x} and \code{y} axes.

  The optional argument \code{colmap} gives an easy way to display
  different altitudes in different colours (if this is what you
  want).
  \itemize{
    \item If \code{colmap} is a colour map (object of class \code{"colourmap"},
    created by the function \code{\link{colourmap}})
    then this colour map will be used to associate
    altitudes with colours.
    \item If \code{colmap} is a character vector, then the range of
    altitudes in the perspective plot will be divided into
    \code{length(colmap)} intervals, and those parts of the surface
    which lie in a particular altitude range will be assigned
    the corresponding colour from \code{colmap}.
    \item If \code{colmap} is a function in the \R language
    of the form \code{function(n, ...)}, this function will be called
    with an appropriate value of \code{n} to generate a character
    vector of \code{n} colours. 
      Examples of such functions are
      \code{\link[grDevices]{heat.colors}},
      \code{\link[grDevices]{terrain.colors}},
      \code{\link[grDevices]{topo.colors}} and
      \code{\link[grDevices]{cm.colors}}. 
    \item If \code{colmap} is a function in the \R language of the
    form \code{function(range, ...)} then it will be called
    with \code{range} equal to the range of altitudes,
    to determine the colour values or colour map.
    Examples of such functions are \code{\link{beachcolours}}
    and \code{\link{beachcolourmap}}.
    \item If \code{colmap} is a list with entries \code{breaks} and \code{col},
    then \code{colmap$breaks} determines the breakpoints of the altitude
    intervals, and \code{colmap$col} provides the corresponding colours.
  }
  Alternatively, if the argument \code{colin} (\emph{colour input}) is present,
  then the colour map \code{colmap} will be applied to
  the pixel values of \code{colin} instead of the pixel values of
  \code{x}. The result is a perspective view of a surface with
  heights determined by \code{x} and colours determined by \code{colin}
  (mapped by \code{colmap}).  
  
  If \code{apron=TRUE}, vertical surface is drawn around the boundary
  of the perspective plot, so that the terrain appears to have been
  cut out of a solid material. If colour data were supplied, then
  the apron is coloured light grey.

  Graphical parameters controlling the perspective plot
  are passed through the \code{...} arguments
  directly to the function \code{\link{persp.default}}.
  See the examples in \code{\link{persp.default}} or in
  \code{demo(persp)}.

  The vertical scale is controlled by the argument \code{expand}:
  setting \code{expand=1} will interpret the pixel values as being in
  the same units as the spatial coordinates \eqn{x} and \eqn{y} and
  represent them at the same scale.

  If \code{visible=TRUE}, the algorithm also computes whether each pixel
  in \code{x} is visible in the perspective view. In order to be
  visible, a pixel must not be obscured by another pixel which lies
  in front of it (as seen from the viewing direction), and the
  three-dimensional vector normal to the surface must be pointing toward the
  viewer. The return value of \code{persp.im} then has an attribute
  \code{"visible"} which is a pixel image, compatible with \code{x},
  with pixel value equal to \code{TRUE} if the corresponding pixel in
  \code{x} is visible, and \code{FALSE} if it is not visible.
}
\seealso{
  \code{\link{perspPoints}},
  \code{\link{perspLines}}
  for drawing additional points or lines \emph{on the surface}.

  \code{\link[grDevices]{trans3d}} for mapping arbitrary
  \eqn{(x,y,z)} coordinate locations to the plotting coordinates.
  
  \code{\link{im.object}},
  \code{\link{plot.im}},
  \code{\link{contour.im}}
}
\examples{
   # an image
   Z <- setcov(owin())
   persp(Z, colmap=terrain.colors(128))
   co <- colourmap(range=c(0,1), col=rainbow(128))
   persp(Z, colmap=co, axes=FALSE, shade=0.3)

   ## Terrain elevation
   persp(bei.extra$elev, colmap=terrain.colors(128),
         apron=TRUE, theta=-30, phi=20,
         zlab="Elevation", main="", ticktype="detailed",
         expand=6)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
 
 \keyword{hplot}

\name{as.solist}
\alias{as.solist}
\title{
  Convert List of Two-Dimensional Spatial Objects
}
\description{
  Given a list of two-dimensional spatial objects,
  convert it to the class \code{"solist"}.
}
\usage{
as.solist(x, \dots)
}
\arguments{
  \item{x}{
    A list of objects, each representing a two-dimensional
    spatial dataset.
  }
  \item{\dots}{
    Additional arguments passed to \code{\link{solist}}.
  }
}
\details{
  This command makes the list \code{x} into an object of class \code{"solist"}
  (spatial object list).
  See \code{\link{solist}} for details.

  The entries in the list \code{x} should be
  two-dimensional spatial datasets
  (not necessarily of the same class).
}
\value{
  A list, usually of class \code{"solist"}.
}
\seealso{
  \code{\link{solist}},
  \code{\link{as.anylist}},
  \code{\link{solapply}}.
}
\examples{
  x <- list(cells, density(cells))
  y <- as.solist(x)
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{list}
\keyword{manip}

\name{Extract.tess}
\alias{[.tess}
\alias{[<-.tess}
\title{Extract or Replace Subset of Tessellation}
\description{
  Extract, change or delete a subset of the tiles of a tessellation,
  to make a new tessellation.
}
\usage{
  \method{[}{tess}(x, i, \dots)
  \method{[}{tess}(x, i, \dots) <- value
}
\arguments{
  \item{x}{A tessellation (object of class \code{"tess"}).}
  \item{i}{
     Subset index for the tiles of the tessellation.
     Alternatively a window (object of class \code{"owin"}).
  }
  \item{\dots}{
     One argument that specifies the subset to be extracted or changed.
     Any valid format for the subset index in a list.
  }
  \item{value}{
     Replacement value for the selected tiles of the tessellation.
     A list of windows (objects of class \code{"owin"}) or \code{NULL}. 
  }
}
\details{
  A tessellation (object of class \code{"tess"}, see \code{\link{tess}})
  is effectively a list of tiles (spatial regions) that cover a spatial region.
  The subset operator \code{[.tess} extracts some
  of these tiles and forms a new tessellation, which of course covers a
  smaller region than the original. 

  For \code{[.tess} only, the subset index can also be a window
  (object of class \code{"owin"}). The tessellation \code{x}
  is then intersected with the window.

  The replacement operator changes the selected tiles. The replacement
  \code{value} may be either \code{NULL} (which causes the selected tiles
  to be removed from \code{x}) or a list of the same length as
  the selected subset. The entries of \code{value} may be windows
  (objects of class \code{"owin"}) or \code{NULL} to indicate that the
  corresponding tile should be deleted.

  Generally it does not make sense to replace a tile in a tessellation
  with a completely different tile, because the tiles are expected to
  fit together. However this facility is sometimes useful for making
  small adjustments to polygonal tiles.
}
\value{
  A tessellation (object of class \code{"tess"}).
}
\seealso{
  \code{\link{tess}}, \code{\link{tiles}}, \code{\link{intersect.tess}}.
}
\examples{
   \testonly{op <- spatstat.options(npixel=10)}
   A <- tess(xgrid=0:4, ygrid=0:3)
   B <- A[c(1, 3, 7)]
   E <- A[-1]
   A[c(2, 5, 11)] <- NULL
   \testonly{spatstat.options(op)}
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{intensity.ppx}
\alias{intensity.ppx}
\title{Intensity of a Multidimensional Space-Time Point Pattern}
\description{
  Calculates the intensity of points in a multi-dimensional point
  pattern of class \code{"ppx"} or \code{"pp3"}.
}
\usage{
  \method{intensity}{ppx}(X, \dots)
}
\arguments{
  \item{X}{Point pattern of class \code{"ppx"} or \code{"pp3"}.}
  \item{\dots}{Ignored.}
}
\value{
  A single number or a numeric vector.
}
\details{
  This is a method for the generic function \code{\link{intensity}}.
  It computes the empirical intensity of a multi-dimensional point pattern
  (object of class \code{"ppx"} including \code{"pp3"}),
  i.e. the average density of points per unit volume.

  If the point pattern is multitype, the intensities of the
  different types are computed separately.
}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\examples{
  X <- osteo$pts[[1]]
  intensity(X)
  marks(X) <- factor(sample(letters[1:3], npoints(X), replace=TRUE))
  intensity(X)
}

\name{mergeLevels}
\alias{mergeLevels}
\title{
  Merge Levels of a Factor
}
\description{
  Specified levels of the factor will be merged into a single level.
}
\usage{
mergeLevels(.f, \dots)
}
\arguments{
  \item{.f}{
    A factor (or a factor-valued pixel image
    or a point pattern with factor-valued marks).
  }
  \item{\dots}{
    List of \code{name=value} pairs, where \code{name} is the
    new merged level, and \code{value} is the vector of old
    levels that will be merged.
  }
}
\details{
  This utility function takes a factor \code{.f}
  and merges specified levels of the factor.

  The grouping is specified by the arguments \code{\dots}
  which must each be given in the form \code{new=old}, where
  \code{new} is the name for the new merged level,
  and \code{old} is a character vector containing the old levels
  that are to be merged.

  The result is a new factor (or factor-valued object),
  in which the levels listed in \code{old}
  have been replaced by a single level \code{new}.

  An argument of the form \code{name=character(0)} or \code{name=NULL}
  is interpreted to mean that all other levels of the old factor
  should be mapped to \code{name}.
}
\value{
  Another factor of the same length as \code{.f}
  (or object of the same kind as \code{.f}).
}
\section{Tips for manipulating factor levels}{
  To remove unused levels from a factor \code{f},
  just type \code{f <- factor(f)}.

  To change the ordering of levels in a factor,
  use \code{\link[base]{factor}(f, levels=l)} or
  \code{\link[stats]{relevel}(f, ref)}.
}
\seealso{
  \code{\link[base]{factor}},
  \code{\link[stats]{relevel}}
}
\author{
\adrian


\rolf

and \ege

}
\examples{
   likert <- c("Strongly Agree", "Agree", "Neutral",
               "Disagree", "Strongly Disagree")
   answers <- factor(sample(likert, 15, replace=TRUE), levels=likert)
   answers
   mergeLevels(answers, Positive=c("Strongly Agree", "Agree"),
                        Negative=c("Strongly Disagree", "Disagree"))
}
\keyword{manip}
\keyword{spatial}

\name{maxnndist}
\alias{maxnndist}
\alias{minnndist}
\title{
  Compute Minimum or Maximum Nearest-Neighbour Distance
}
\description{
  A faster way to compute the minimum or maximum
  nearest-neighbour distance in a point pattern.
}
\usage{
minnndist(X, positive=FALSE, by=NULL)
maxnndist(X, positive=FALSE, by=NULL)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"}).
  }
  \item{positive}{
    Logical. If \code{FALSE} (the default), compute the usual
    nearest-neighbour distance.
    If \code{TRUE}, ignore coincident points, so that the
    nearest neighbour distance for each point is greater than zero.
  }
  \item{by}{
    Optional. A factor, which separates \code{X} into groups.
    The algorithm will compute the distance to
    the nearest point in each group. 
  }
}
\details{
  These functions find the minimum and maximum values
  of nearest-neighbour distances in the point pattern \code{X}.
  \code{minnndist(X)} and \code{maxnndist(X)} are 
  equivalent to, but faster than, \code{min(nndist(X))}
  and \code{max(nndist(X))} respectively.

  The value is \code{NA} if \code{npoints(X) < 2}.
}
\value{
  A single numeric value (possibly \code{NA}).

  If \code{by} is given, the result is a numeric matrix
  giving the minimum or maximum nearest neighbour distance
  between each subset of \code{X}.
}
\seealso{
  \code{\link{nndist}}
}
\examples{
  min(nndist(swedishpines))
  minnndist(swedishpines)

  max(nndist(swedishpines))
  maxnndist(swedishpines)

  minnndist(lansing, positive=TRUE)

  if(interactive()) {
     X <- rpoispp(1e6)
     system.time(min(nndist(X)))
     system.time(minnndist(X))
  }

  minnndist(amacrine, by=marks(amacrine))
  maxnndist(amacrine, by=marks(amacrine))
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{pcf.ppp}
\alias{pcf.ppp}
\title{Pair Correlation Function of Point Pattern}
\description{
  Estimates the pair correlation function of
  a point pattern using kernel methods.
}
\usage{
  \method{pcf}{ppp}(X, \dots, r = NULL, kernel="epanechnikov", bw=NULL,
                    stoyan=0.15,
                    correction=c("translate", "Ripley"),
                    divisor = c("r", "d"),
                    var.approx = FALSE,
                    domain=NULL,
                    ratio=FALSE, close=NULL)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"}).
  }
  \item{r}{
    Vector of values for the argument \eqn{r} at which \eqn{g(r)} 
    should be evaluated. There is a sensible default.
  }
  \item{kernel}{
    Choice of smoothing kernel,
    passed to \code{\link{density.default}}.
  }
  \item{bw}{
    Bandwidth for smoothing kernel,
    passed to \code{\link{density.default}}.
    Either a single numeric value,
    or a character string specifying a bandwidth selection rule
    recognised by \code{\link{density.default}}.
    If \code{bw} is missing or \code{NULL},
    the default value is computed using
    Stoyan's rule of thumb: see Details.
  }
  \item{\dots}{
    Other arguments passed to the kernel density estimation 
    function \code{\link{density.default}}.
  }
  \item{stoyan}{
    Coefficient for Stoyan's bandwidth selection rule; see Details.
  }
  \item{correction}{
    Edge correction. A character vector specifying the choice
    (or choices) of edge correction. See Details.
  }
  \item{divisor}{
    Choice of divisor in the estimation formula:
    either \code{"r"} (the default) or \code{"d"}. See Details.
  }
  \item{var.approx}{
    Logical value indicating whether to compute an analytic
    approximation to the variance of the estimated pair correlation.
  }
  \item{domain}{
    Optional. Calculations will be restricted to this subset
    of the window. See Details.
  }
  \item{ratio}{
    Logical. 
    If \code{TRUE}, the numerator and denominator of
    each edge-corrected estimate will also be saved,
    for use in analysing replicated point patterns.
  }
  \item{close}{
    Advanced use only. Precomputed data. See section on Advanced Use.
  }
} 
\value{
  A function value table
  (object of class \code{"fv"}).
  Essentially a data frame containing the variables
  \item{r}{the vector of values of the argument \eqn{r} 
    at which the pair correlation function \eqn{g(r)} has been  estimated
  }
  \item{theo}{vector of values equal to 1,
    the theoretical value of \eqn{g(r)} for the Poisson process
  }
  \item{trans}{vector of values of \eqn{g(r)}
    estimated by translation correction
  }
  \item{iso}{vector of values of \eqn{g(r)}
    estimated by Ripley isotropic correction
  }
  \item{v}{vector of approximate values of the variance of
    the estimate of \eqn{g(r)}
  }
  as required.

  If \code{ratio=TRUE} then the return value also has two
  attributes called \code{"numerator"} and \code{"denominator"}
  which are \code{"fv"} objects
  containing the numerators and denominators of each
  estimate of \eqn{g(r)}.

  The return value also has an attribute \code{"bw"} giving the
  smoothing bandwidth that was used.
}
\details{
  The pair correlation function \eqn{g(r)} 
  is a summary of the dependence between points in a spatial point
  process. The best intuitive interpretation is the following: the probability
  \eqn{p(r)} of finding two points at locations \eqn{x} and \eqn{y}
  separated by a distance \eqn{r} is equal to
  \deqn{
    p(r) = \lambda^2 g(r) \,{\rm d}x \, {\rm d}y
  }{
    p(r) = lambda^2 * g(r) dx dy
  }
  where \eqn{\lambda}{lambda} is the intensity of the point process.
  For a completely random (uniform Poisson) process,
  \eqn{p(r) = \lambda^2 \,{\rm d}x \, {\rm d}y}{p(r) = lambda^2 dx dy}
  so \eqn{g(r) = 1}.
  Formally, the pair correlation function of a stationary point process
  is defined by 
  \deqn{
    g(r) = \frac{K'(r)}{2\pi r}
  }{
    g(r) = K'(r)/ ( 2 * pi * r) 
  }
  where \eqn{K'(r)} is the derivative of \eqn{K(r)}, the
  reduced second moment function (aka ``Ripley's \eqn{K} function'')
  of the point process. See \code{\link{Kest}} for information
  about \eqn{K(r)}.

  For a stationary Poisson process, the
  pair correlation function is identically equal to 1. Values
  \eqn{g(r) < 1} suggest inhibition between points;
  values greater than 1 suggest clustering.

  This routine computes an estimate of \eqn{g(r)}
  by kernel smoothing. 

  \itemize{
    \item
    If \code{divisor="r"} (the default), then the standard
    kernel estimator (Stoyan and Stoyan, 1994, pages 284--285)
    is used. By default, the recommendations of Stoyan and Stoyan (1994)
    are followed exactly. 
    \item
    If \code{divisor="d"} then a modified estimator is used:
    the contribution from
    an interpoint distance \eqn{d_{ij}}{d[ij]} to the
    estimate of \eqn{g(r)} is divided by \eqn{d_{ij}}{d[ij]}
    instead of dividing by \eqn{r}. This usually improves the
    bias of the estimator when \eqn{r} is close to zero.
  }

  There is also a choice of spatial edge corrections
  (which are needed to avoid bias due to edge effects
  associated with the boundary of the spatial window):

  \itemize{
    \item
    If \code{correction="translate"} or \code{correction="translation"}
    then the translation correction
    is used. For \code{divisor="r"} the translation-corrected estimate
    is given in equation (15.15), page 284 of Stoyan and Stoyan (1994).
    \item
    If \code{correction="Ripley"} or \code{correction="isotropic"}
    then Ripley's isotropic edge correction
    is used. For \code{divisor="r"} the isotropic-corrected estimate
    is given in equation (15.18), page 285 of Stoyan and Stoyan (1994). 
    \item
    If \code{correction="none"} then no edge correction is used,
    that is, an uncorrected estimate is computed. 
  }
  Multiple corrections can be selected. The default is
  \code{correction=c("translate", "Ripley")}.
  
  Alternatively \code{correction="all"} selects all options;
  \code{correction="best"} selects the option which has the best
  statistical performance; \code{correction="good"} selects the option
  which is the best compromise between statistical performance and speed
  of computation.
  
  The choice of smoothing kernel is controlled by the 
  argument \code{kernel} which is passed to \code{\link{density.default}}.
  The default is the Epanechnikov kernel, recommended by
  Stoyan and Stoyan (1994, page 285).

  The bandwidth of the smoothing kernel can be controlled by the
  argument \code{bw}. Its precise interpretation
  is explained in the documentation for \code{\link{density.default}}.
  For the Epanechnikov kernel, the argument \code{bw} is
  equivalent to \eqn{h/\sqrt{5}}{h/sqrt(5)}.

  Stoyan and Stoyan (1994, page 285) recommend using the Epanechnikov
  kernel with support \eqn{[-h,h]} chosen by the rule of thumn
  \eqn{h = c/\sqrt{\lambda}}{h = c/sqrt(lambda)},
  where \eqn{\lambda}{lambda} is the (estimated) intensity of the
  point process, and \eqn{c} is a constant in the range from 0.1 to 0.2.
  See equation (15.16).
  If \code{bw} is missing or \code{NULL},
  then this rule of thumb will be applied.
  The argument \code{stoyan} determines the value of \eqn{c}.
  The smoothing bandwidth that was used in the calculation is returned
  as an attribute of the final result.
  
  The argument \code{r} is the vector of values for the
  distance \eqn{r} at which \eqn{g(r)} should be evaluated.
  There is a sensible default.
  If it is specified, \code{r} must be a vector of increasing numbers
  starting from \code{r[1] = 0}, 
  and \code{max(r)} must not exceed half the diameter of 
  the window.

  If the argument \code{domain} is given, estimation will be restricted
  to this region. That is, the estimate of 
  \eqn{g(r)} will be based on pairs of points in which the first point lies
  inside \code{domain} and the second point is unrestricted.
  The argument \code{domain}
  should be a window (object of class \code{"owin"}) or something acceptable to
  \code{\link{as.owin}}. It must be a subset of the
  window of the point pattern \code{X}.

  To compute a confidence band for the true value of the
  pair correlation function, use \code{\link{lohboot}}.

  If \code{var.approx = TRUE}, the variance of the
  estimate of the pair correlation will also be calculated using
  an analytic approximation (Illian et al, 2008, page 234)
  which is valid for stationary point processes which are not
  too clustered. This calculation is not yet implemented when
  the argument \code{domain} is given.
}
\section{Advanced Use}{
  To perform the same computation using several different bandwidths \code{bw},
  it is efficient to use the argument \code{close}.
  This should be the result of \code{\link{closepairs}(X, rmax)}
  for a suitably large value of \code{rmax}, namely
  \code{rmax >= max(r) + 3 * bw}.
}
\references{
  Illian, J., Penttinen, A., Stoyan, H. and Stoyan, D. (2008)
  \emph{Statistical Analysis and Modelling of Spatial Point Patterns.}
  Wiley.

  Stoyan, D. and Stoyan, H. (1994)
  \emph{Fractals, random shapes and point fields:
  methods of geometrical statistics.}
  John Wiley and Sons.
}
\seealso{
  \code{\link{Kest}},
  \code{\link{pcf}},
  \code{\link{density.default}},
  \code{\link{bw.stoyan}},
  \code{\link{bw.pcf}},
  \code{\link{lohboot}}.
}
\examples{
  X <- simdat
  \testonly{
    X <- X[seq(1,npoints(X), by=4)]
  }
  p <- pcf(X)
  plot(p, main="pair correlation function for X")
  # indicates inhibition at distances r < 0.3

  pd <- pcf(X, divisor="d")

  # compare estimates
  plot(p, cbind(iso, theo) ~ r, col=c("blue", "red"),
         ylim.covers=0, main="", lwd=c(2,1), lty=c(1,3), legend=FALSE)
  plot(pd, iso ~ r, col="green", lwd=2, add=TRUE)
  legend("center", col=c("blue", "green"), lty=1, lwd=2,
         legend=c("divisor=r","divisor=d"))

  # calculate approximate variance and show POINTWISE confidence bands
  pv <- pcf(X, var.approx=TRUE)
  plot(pv, cbind(iso, iso+2*sqrt(v), iso-2*sqrt(v)) ~ r)
}
\author{
  \spatstatAuthors
  and Martin Hazelton.
}
\keyword{spatial}
\keyword{nonparametric}

\name{rGaussPoisson}
\alias{rGaussPoisson}
\title{Simulate Gauss-Poisson Process}
\description{
  Generate a random point pattern, a simulated realisation of the
  Gauss-Poisson Process.
}
\usage{
 rGaussPoisson(kappa, r, p2, win = owin(c(0,1),c(0,1)),
               \dots, nsim=1, drop=TRUE)
}
\arguments{
  \item{kappa}{
    Intensity of the Poisson process of cluster centres.
    A single positive number, a function, or a pixel image.
  }
  \item{r}{
    Diameter of each cluster that consists of exactly 2 points.
  }
  \item{p2}{
    Probability that a cluster contains exactly 2 points.
  }
  \item{win}{
    Window in which to simulate the pattern.
    An object of class \code{"owin"}
    or something acceptable to \code{\link{as.owin}}.
  }
  \item{\dots}{Ignored.}
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  A point pattern (an object of class \code{"ppp"})
  if \code{nsim=1}, or a list of point patterns if \code{nsim > 1}.

  Additionally, some intermediate results of the simulation are
  returned as attributes of the point pattern.
  See \code{\link{rNeymanScott}}.
}
\details{
  This algorithm generates a realisation of the Gauss-Poisson
  point process inside the window \code{win}.
  The process is constructed by first
  generating a Poisson point process of parent points 
  with intensity \code{kappa}. Then each parent point is either retained
  (with probability \code{1 - p2})
  or replaced by a pair of points at a fixed distance \code{r} apart
  (with probability \code{p2}). In the case of clusters of 2 points,
  the line joining the two points has uniform random orientation.

  In this implementation, parent points are not restricted to lie in the
  window; the parent process is effectively the uniform
  Poisson process on the infinite plane.
}
\seealso{
  \code{\link{rpoispp}},
  \code{\link{rThomas}},
  \code{\link{rMatClust}},
  \code{\link{rNeymanScott}}
}
\examples{
 pp <- rGaussPoisson(30, 0.07, 0.5)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{Extract.layered}
\alias{[.layered}
\alias{[<-.layered}
\alias{[[<-.layered}
\title{Extract or Replace Subset of a Layered Object}
\description{
  Extract or replace some or all of the layers of a layered object,
  or extract a spatial subset of each layer.
}
\usage{
  \method{[}{layered}(x, i, j, drop=FALSE, ...)

  \method{[}{layered}(x, i) <- value

  \method{[[}{layered}(x, i) <- value
}
\arguments{
  \item{x}{
    A layered object (class \code{"layered"}).
  }
  \item{i}{
    Subset index for the list of layers.
    A logical vector, integer vector or character vector
    specifying which layers are to be extracted or replaced.
  }
  \item{j}{
    Subset index to be applied to the data in each layer.
    Typically a spatial window (class \code{"owin"}).
  }
  \item{drop}{
    Logical. If \code{i} specifies only a single layer
    and \code{drop=TRUE}, then the contents of this layer
    will be returned.
  }
  \item{\dots}{
     Additional arguments, passed to other subset methods
     if the subset index is a window.
  }
  \item{value}{List of objects which shall replace the designated
    subset, or an object which shall replace the designated element.
  }
}
\value{
  Usually an object of class \code{"layered"}.
}
\details{
  A layered object represents data that should be plotted in
  successive layers, for example, a background and a foreground.
  See \code{\link{layered}}.

  The function \code{[.layered}
  extracts a designated subset of a layered object.
  It is a method for \code{\link{[}} for the
  class \code{"layered"}.

  The functions \code{[<-.layered} and \code{[[<-.layered}
  replace a designated subset or designated entry of the object by new
  values. They are methods for \code{\link{[<-}} and \code{\link{[[<-}}
  for the \code{"layered"} class.

  The index \code{i} specifies which layers will be retained.
  It should be a valid subset index for the list of layers.

  The index \code{j} will be applied to each layer. It is typically
  a spatial window (class \code{"owin"}) so that each of the layers
  will be restricted to the same spatial region.
  Alternatively \code{j} may be any subset index
  which is permissible for the \code{"["} method for each of the layers.
}
\seealso{
  \code{\link{layered}}
}
\examples{
 D <- distmap(cells)
 L <- layered(D, cells,
              plotargs=list(list(ribbon=FALSE), list(pch=16)))

 L[-2]
 L[, square(0.5)]

 L[[3]] <- japanesepines
 L
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{quadrats}
\alias{quadrats}
\title{Divide Region into Quadrats}
\description{
  Divides window into rectangular quadrats and 
  returns the quadrats as a tessellation.
}
\usage{
quadrats(X, nx = 5, ny = nx, xbreaks = NULL, ybreaks = NULL, keepempty=FALSE)
}
\arguments{
  \item{X}{
    A window (object of class \code{"owin"})
    or anything that can be coerced to a window using
    \code{\link{as.owin}}, such as a point pattern.
  }
  \item{nx,ny}{
    Numbers of quadrats in the \eqn{x} and \eqn{y} directions.
    Incompatible with \code{xbreaks} and \code{ybreaks}.
  }
  \item{xbreaks}{
    Numeric vector giving the \eqn{x} coordinates of the
    boundaries of the quadrats. Incompatible with \code{nx}.
  }
  \item{ybreaks}{
    Numeric vector giving the \eqn{y} coordinates of the
    boundaries of the quadrats. Incompatible with \code{ny}.
  }
  \item{keepempty}{
    Logical value indicating whether to delete or retain
    empty quadrats. See Details.
  }
}
\details{
  If the window \code{X} is a rectangle, it is divided into
  an \code{nx * ny} grid of rectangular tiles or `quadrats'.

  If \code{X} is not a rectangle, then the bounding rectangle of
  \code{X} is first divided into an \code{nx * ny} grid of rectangular
  tiles, and these tiles are then intersected with the window \code{X}.

  The resulting tiles are returned as a tessellation (object of class
  \code{"tess"}) which can be plotted and used in other analyses.

  If \code{xbreaks} is given, it should be a numeric vector
  giving the \eqn{x} coordinates of the quadrat boundaries.
  If it is not given, it defaults to a
  sequence of \code{nx+1} values equally spaced
  over the range of \eqn{x} coordinates in the window \code{Window(X)}.

  Similarly if \code{ybreaks} is given, it should be a numeric
  vector giving the \eqn{y} coordinates of the quadrat boundaries.
  It defaults to a vector of \code{ny+1} values
  equally spaced over the range of \eqn{y} coordinates in the window.
  The lengths of \code{xbreaks} and \code{ybreaks} may be different.

  By default (if \code{keepempty=FALSE}), any rectangular tile which
  does not intersect the window \code{X} is
  ignored, and only the non-empty intersections are treated as quadrats,
  so the tessellation may consist of fewer than \code{nx * ny} tiles.
  If \code{keepempty=TRUE}, empty intersections are retained,
  and the tessellation always contains exactly \code{nx * ny} tiles,
  some of which may be empty.
}
\value{
  A tessellation (object of class \code{"tess"}) as described under
  \code{\link{tess}}.
}
\examples{
 W <- square(10)
 Z <- quadrats(W, 4, 5)
 plot(Z)

 data(letterR)
 plot(quadrats(letterR, 5, 7))
}
\seealso{
  For calculations using quadrats, see
  \code{\link{quadratcount}},
  \code{\link{quadrat.test}},
  \code{\link{quadratresample}}

  For other kinds of tessellations, see 
  \code{\link{tess}},
  \code{\link{hextess}},
  \code{\link{venn.tess}},
  \code{\link{polartess}},
  \code{\link{dirichlet}}, \code{\link{delaunay}},
  \code{\link{rpoislinetess}} and \code{\link{quantess}}.

}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{utilities}
\keyword{datagen}

\name{rMosaicField}
\alias{rMosaicField}
\title{Mosaic Random Field}
\description{
  Generate a realisation of a random field
  which is piecewise constant on the tiles of a given tessellation.
}
\usage{
rMosaicField(X, 
    rgen = function(n) { sample(0:1, n, replace = TRUE)},
    ...,
    rgenargs=NULL)
}
\arguments{
  \item{X}{
    A tessellation (object of class \code{"tess"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}}
    determining the pixel resolution.
  }
  \item{rgen}{
    Function that generates random values for the
    tiles of the tessellation.
  }
  \item{rgenargs}{
    List containing extra arguments that should be passed
    to \code{rgen} (typically specifying parameters of the
    distribution of the values).
  }
}
\details{
  This function generates a realisation of a random field
  which is piecewise constant on the tiles of the given
  tessellation \code{X}. The values in each tile
  are independent and identically distributed.
}
\value{
  A pixel image (object of class \code{"im"}).
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{rpoislinetess}},
  \code{\link{rMosaicSet}}
}
\examples{
   X <- rpoislinetess(3)
   plot(rMosaicField(X, runif))
   plot(rMosaicField(X, runif, dimyx=256))
   plot(rMosaicField(X, rnorm, rgenargs=list(mean=10, sd=2)))

   plot(rMosaicField(dirichlet(runifpoint(30)), rnorm))
}
\keyword{spatial}
\keyword{datagen}

\name{colourtools}
\alias{colourtools} %DoNotExport
\alias{paletteindex}
\alias{rgb2hex}
\alias{rgb2hsva}
\alias{col2hex}
\alias{paletteindex}
\alias{samecolour}
\alias{complementarycolour}
\alias{interp.colours}
\alias{is.colour}
\alias{is.grey}
\alias{to.grey}
\alias{to.opaque}
\alias{to.transparent}
\alias{to.saturated}
\title{
  Convert and Compare Colours in Different Formats
}
\description{
  These functions convert between different formats for specifying
  a colour in \R, determine whether colours are equivalent,
  and convert colour to greyscale.
}
\usage{
col2hex(x)
rgb2hex(v, maxColorValue=255)
rgb2hsva(red, green=NULL, blue=NULL, alpha=NULL, maxColorValue=255)
paletteindex(x)
samecolour(x,y)
complementarycolour(x)
interp.colours(x, length.out=512)
is.colour(x)
to.grey(x, weights=c(0.299, 0.587, 0.114), transparent=FALSE)
is.grey(x)
to.opaque(x)
to.transparent(x, fraction)
to.saturated(x, s=1)
}

\arguments{
  \item{x,y}{
    Any valid specification for a colour or sequence of colours
    accepted by \code{\link[grDevices]{col2rgb}}.
  }
  \item{v}{
    A numeric vector of length 3, giving the RGB values of
    a single colour, or a 3-column matrix giving the RGB values of
    several colours. Alternatively a vector of length 4 or a matrix with
    4 columns, giving the RGB and alpha (transparency) values.
  }
  \item{red,green,blue,alpha}{
    Arguments acceptable to \code{\link[grDevices]{rgb}}
    determining the red, green, blue channels and optionally the
    alpha (transparency) channel.
    Note that \code{red} can also be a matrix with 3 \bold{rows}
    giving the RGB values, or a matrix with 4 rows
    giving RGB and alpha values.
  }
  \item{maxColorValue}{
    Number giving the maximum possible value for the entries in
    \code{v} or \code{red,green,blue,alpha}.
  }
  \item{weights}{
    Numeric vector of length 3 giving 
    relative weights for the red, green, and blue
    channels respectively.
  }
  \item{transparent}{
    Logical value indicating whether transparent colours should
    be converted to transparent grey values (\code{transparent=TRUE})
    or converted to opaque grey values (\code{transparent=FALSE}, the
    default).
  }
  \item{fraction}{
    Transparency fraction. Numerical value or vector of values
    between 0 and 1, giving the opaqueness of a colour.
    A fully opaque colour has \code{fraction=1}.
  }
  \item{length.out}{
    Integer. Length of desired sequence.
  }
  \item{s}{
    Saturation value (between 0 and 1).
  }
}
\details{
  \code{is.colour(x)} can be applied to any kind of data \code{x}
  and returns \code{TRUE} if \code{x} can be interpreted as a colour or
  colours. The remaining functions expect data that can be interpreted
  as colours.
  
  \code{col2hex} converts colours specified in any format
  into their hexadecimal character codes.

  \code{rgb2hex} converts RGB colour values into their hexadecimal
  character codes. It is a very minor extension to \code{\link[grDevices]{rgb}}.
  Arguments to \code{rgb2hex} should be similar to
  arguments to \code{\link[grDevices]{rgb}}.

  \code{rgb2hsva} converts RGB colour values into HSV colour values
  including the alpha (transparency) channel.
  It is an extension of \code{\link[grDevices]{rgb2hsv}}.
  Arguments to \code{rgb2hsva} should be similar to arguments to
  \code{\link[grDevices]{rgb2hsv}}.
  
  \code{paletteindex} checks whether the colour or colours specified
  by \code{x} are available in the default palette returned by
  \code{\link[grDevices]{palette}()}. If so, it returns the index or indices of
  the colours in the palette. If not, it returns \code{NA}.

  \code{samecolour} decides whether two colours \code{x} and \code{y}
  are equivalent.

  \code{is.grey} determines whether each entry of \code{x} is a
  greyscale colour, and returns a logical vector.
  
  \code{to.grey} converts the colour data in \code{x} to greyscale
  colours. Alternatively \code{x} can be an object of class \code{"colourmap"}
  and \code{to.grey(x)} is the modified colour map.

  \code{to.opaque} converts the colours in \code{x} to opaque
  (non-transparent) colours, and \code{to.transparent} converts them
  to transparent colours with a specified transparency value.
  Note that \code{to.transparent(x,1)} is equivalent to \code{to.opaque(x)}.

  For \code{to.grey}, \code{to.opaque} and
  \code{to.transparent}, if all the data in \code{x} specifies colours from the
  standard palette, and if the result would be equivalent to \code{x},
  then the result is identical to \code{x}.

  \code{to.saturated} converts each colour in \code{x} to
  its fully-saturated equivalent. For example, pink is mapped to red.
  Shades of grey are converted to black; white is unchanged.
  
  \code{complementarycolour} replaces each colour by its
  complementary colour in RGB space (the colour obtained by replacing
  RGB values \code{(r, g, b)} by \code{(255-r, 255-g, 255-b)}).
  The transparency value is not changed.
  Alternatively \code{x} can be an object of class \code{"colourmap"}
  and \code{complementarycolour(x)} is the modified colour map.

  \code{interp.colours} interpolates between each successive pair of
  colours in a sequence of colours, to generate a more finely-spaced
  sequence. It uses linear interpolation in HSV space (with hue
  represented as a two-dimensional unit vector).
}
\section{Warning}{
  \code{paletteindex("green")} returns \code{NA} because
  the green colour in the default palette is called \code{"green3"}.
}
\value{
  For \code{col2hex} and \code{rgb2hex} 
  a character vector containing hexadecimal colour codes.

  For \code{to.grey}, \code{to.opaque} and \code{to.transparent},
  either a character vector containing hexadecimal colour codes,
  or a value identical to the input \code{x}.
  
  For \code{rgb2hsva}, a matrix with 3 or 4 rows containing
  HSV colour values.
  
  For \code{paletteindex}, an integer vector, possibly containing
  \code{NA} values.

  For \code{samecolour} and \code{is.grey},
  a logical value or logical vector.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link[grDevices]{col2rgb}},
  \code{\link[grDevices]{rgb2hsv}},
  \code{\link[grDevices]{palette}}.

  See also the class of colour map objects in the \pkg{spatstat} package:
  \code{\link{colourmap}}, 
  \code{\link{interp.colourmap}}, 
  \code{\link{tweak.colourmap}}.
}
\examples{
  samecolour("grey", "gray")
  paletteindex("grey")
  col2hex("orange")
  to.grey("orange")
  to.saturated("orange")
  complementarycolour("orange")
  is.grey("lightgrey")
  is.grey(8)
  to.transparent("orange", 0.5)
  to.opaque("red")
  interp.colours(c("orange", "red", "violet"), 5)
}
\keyword{color}

\name{nnwhich}
\alias{nnwhich}
\alias{nnwhich.ppp}
\alias{nnwhich.default}
\title{Nearest neighbour}
\description{
  Finds the nearest neighbour of each point in a point pattern.
}
\usage{
  nnwhich(X, \dots)
  \method{nnwhich}{ppp}(X, \dots, k=1, by=NULL, method="C")
  \method{nnwhich}{default}(X, Y=NULL, \dots, k=1, by=NULL, method="C")
}
\arguments{
  \item{X,Y}{
    Arguments specifying the locations of
    a set of points.
    For \code{nnwhich.ppp}, the argument \code{X} should be a point
    pattern (object of class \code{"ppp"}).
    For \code{nnwhich.default}, typically \code{X} and \code{Y} would be
    numeric vectors of equal length. Alternatively \code{Y} may be
    omitted and \code{X} may be
    a list with two components \code{x} and \code{y},
    or a matrix with two columns.
  }
  \item{\dots}{
    Ignored by \code{nnwhich.ppp}
    and \code{nnwhich.default}.
  }
  \item{k}{
    Integer, or integer vector. The algorithm will compute the distance to the
    \code{k}th nearest neighbour. 
  }
  \item{by}{
    Optional. A factor, which separates \code{X} into groups.
    The algorithm will find the nearest neighbour in each group. 
  }
  \item{method}{String specifying which method of calculation to use.
    Values are \code{"C"} and \code{"interpreted"}.
  }
}
\value{
  Numeric vector or matrix giving, for each point,
  the index of its nearest neighbour (or \code{k}th nearest neighbour).

  If \code{k = 1} (the default), the return value is a
  numeric vector \code{v} giving the indices of the nearest neighbours
  (the nearest neighbout of the \code{i}th point is
  the \code{j}th point where \code{j = v[i]}).
  
  If \code{k} is a single integer, then the return value is a
  numeric vector giving the indices of the
  \code{k}th nearest neighbours.

  If \code{k} is a vector, then the return value is a
  matrix \code{m} such that \code{m[i,j]} is the
  index of the \code{k[j]}th nearest neighbour for the
  \code{i}th data point.

  If the argument \code{by} is given, then the result is a data frame
  containing the indices described above, from each point of \code{X},
  to the nearest point in each subset of \code{X}
  defined by the factor \code{by}.
}
\details{
  For each point in the given point pattern, this function finds
  its nearest neighbour (the nearest other point of the pattern).
  By default it returns a vector giving, for each point,
  the index of the point's
  nearest neighbour. If \code{k} is specified, the algorithm finds
  each point's \code{k}th nearest neighbour.

  The function \code{nnwhich} is generic, with
  method for point patterns (objects of class \code{"ppp"})
  and a default method which are described here, as well as a method for
  three-dimensional point patterns (objects of class \code{"pp3"},
  described in \code{\link{nnwhich.pp3}}.

  The method \code{nnwhich.ppp} expects a single
  point pattern argument \code{X}.
  The default method expects that \code{X} and \code{Y} will determine
  the coordinates of a set of points. Typically \code{X} and
  \code{Y} would be numeric vectors of equal length. Alternatively
  \code{Y} may be omitted and \code{X} may be a list with two components
  named \code{x} and \code{y}, or a matrix or data frame with two columns.
  
  The argument \code{k} may be a single integer, or an integer vector.
  If it is a vector, then the \eqn{k}th nearest neighbour distances are
  computed for each value of \eqn{k} specified in the vector.

  If the argument \code{by} is given, it should be a \code{factor},
  of length equal to the number of points in \code{X}.
  This factor effectively partitions \code{X} into subsets,
  each subset associated with one of the levels of \code{X}.
  The algorithm will then find, for each point of \code{X},
  the nearest neighbour \emph{in each subset}.

  If there are no points (if \code{x} has length zero)
  a numeric vector of length zero is returned.
  If there is only one point (if \code{x} has length 1),
  then the nearest neighbour is undefined, and a value of \code{NA}
  is returned. In general if the number of points is less than or equal
  to \code{k}, then a vector of \code{NA}'s is returned.

  The argument \code{method} is not normally used. It is
  retained only for checking the validity of the software.
  If \code{method = "interpreted"} then the distances are
  computed using interpreted R code only. If \code{method="C"}
  (the default) then C code is used. 
  The C code is faster by two to three orders of magnitude
  and uses much less memory.
  
  To evaluate the \emph{distance} between a point and its nearest
  neighbour, use \code{\link{nndist}}.

  To find the nearest neighbours from one point pattern
  to another point pattern, use \code{\link{nncross}}.
}
\section{Nearest neighbours of each type}{
  If \code{X} is a multitype point pattern 
  and \code{by=marks(X)}, then the algorithm will find,
  for each point of \code{X}, the nearest neighbour
  of each type. See the Examples. 
}
\section{Warnings}{
  A value of \code{NA} is returned if there is only one point
  in the point pattern. 
}
\seealso{
  \code{\link{nndist}},
  \code{\link{nncross}}
}
\examples{
   data(cells)
   plot(cells)
   m <- nnwhich(cells)
   m2 <- nnwhich(cells, k=2)

   # plot nearest neighbour links
   b <- cells[m]
   arrows(cells$x, cells$y, b$x, b$y, angle=15, length=0.15, col="red")

   # find points which are the neighbour of their neighbour
   self <- (m[m] == seq(m))
   # plot them
   A <- cells[self]
   B <- cells[m[self]]
   plot(cells)
   segments(A$x, A$y, B$x, B$y)

   # nearest neighbours of each type
   head(nnwhich(ants, by=marks(ants)))
}
\author{Pavel Grabarnik
  \email{pavel.grabar@issp.serpukhov.su}
  and
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{imcov}
\alias{imcov}
\title{Spatial Covariance of a Pixel Image}
\description{
  Computes the unnormalised spatial covariance function of a pixel image.
}
\usage{
 imcov(X, Y=X)
}
\arguments{
  \item{X}{
    A pixel image (object of class \code{"im"}.
  }
  \item{Y}{
    Optional. Another pixel image.
  }
}
\value{
  A pixel image (an object of class \code{"im"}) representing the
  spatial covariance function of \code{X},
  or the cross-covariance of \code{X} and \code{Y}.
}
\details{
  The (uncentred, unnormalised)
  \emph{spatial covariance function} of a pixel image \eqn{X} in the plane
  is the function \eqn{C(v)} defined for each vector \eqn{v} as
  \deqn{
    C(v) = \int X(u)X(u-v)\, {\rm d}u
  }{
    C(v) = integral of X(u) * X(u-v) du
  }
  where the integral is
  over all spatial locations \eqn{u}, and where \eqn{X(u)} denotes the
  pixel value at location \eqn{u}.
  
  This command computes a discretised approximation to
  the spatial covariance function, using the Fast Fourier Transform.
  The return value is
  another pixel image (object of class \code{"im"}) whose greyscale values
  are values of the spatial covariance function.

  If the argument \code{Y} is present, then \code{imcov(X,Y)}
  computes the set \emph{cross-covariance} function \eqn{C(u)}
  defined as 
  \deqn{
    C(v) = \int X(u)Y(u-v)\, {\rm d}u.
  }{
    C(v) = integral of X(u) * Y(u-v) du.
  }

  Note that \code{imcov(X,Y)} is equivalent to
  \code{convolve.im(X,Y,reflectY=TRUE)}.
}
\seealso{
  \code{\link{setcov}},
  \code{\link{convolve.im}},
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{erosion}}
}
\examples{
  X <- as.im(square(1))
  v <- imcov(X)
  plot(v)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{edges}
\alias{edges}
\title{
  Extract Boundary Edges of a Window.
}
\description{
  Extracts the boundary edges of a window
  and returns them as a line segment pattern.
}
\usage{
  edges(x, \dots, window = NULL, check = FALSE) 
}
\arguments{
  \item{x}{
    A window (object of class \code{"owin"}), or
    data acceptable to \code{\link{as.owin}}, specifying the window
    whose boundary is to be extracted.
  }
  \item{\dots}{
    Ignored.
  }
  \item{window}{
    Window to contain the resulting line segments.
    Defaults to \code{as.rectangle(x)}.
  }
  \item{check}{
    Logical. Whether to check the validity of the resulting
    segment pattern.
  }
}
\details{
  The boundary edges of the window \code{x} will be extracted as a line
  segment pattern.
}
\value{
  A line segment pattern (object of class \code{"psp"}).
}
\seealso{
  \code{\link{perimeter}} for calculating the total length of the boundary.
}
\examples{
  edges(square(1))
  edges(letterR)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{nnmap}
\alias{nnmap}
\title{
  K-th Nearest Point Map
}
\description{
  Given a point pattern, this function constructs pixel images
  giving the distance from each pixel to its \eqn{k}-th nearest
  neighbour in the point pattern, and the index of the \eqn{k}-th nearest
  neighbour.
}
\usage{
  nnmap(X, k = 1, what = c("dist", "which"),
  \dots, W = as.owin(X),
  is.sorted.X = FALSE, sortby = c("range", "var", "x", "y"))
}
\arguments{
  \item{X}{
    Point pattern (object of class \code{"ppp"}).
  }
  \item{k}{
    Integer, or integer vector. The algorithm will find the 
    \code{k}th nearest neighbour. 
  }
  \item{what}{
    Character string specifying what information should be returned.
    Either the nearest neighbour distance (\code{"dist"}),
    the index of the nearest neighbour (\code{"which"}),
    or both.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine the pixel
    resolution of the result.
  }
  \item{W}{
    Window (object of class \code{"owin"}) specifying the spatial domain
    in which the distances will be computed.
    Defaults to the window of \code{X}.
  }
  \item{is.sorted.X}{
    Logical value attesting whether the point pattern \code{X}
    has been sorted. See Details.
  }
  \item{sortby}{
    Determines which coordinate to use to sort the point pattern.
    See Details.
  }
}
\details{
  Given a point pattern \code{X}, this function constructs two pixel images:
  \itemize{
    \item a distance map giving, for each pixel,
    the distance to the nearest point of \code{X};
    \item a nearest neighbour map giving, for each pixel,
    the identifier of the nearest point of \code{X}.
  }
  If the argument \code{k} is specified, then the \code{k}-th nearest
  neighbours will be found.

  If \code{what="dist"} then only the distance map is returned.
  If \code{what="which"} then only the nearest neighbour map
  is returned.

  The argument \code{k} may be an integer or an integer vector.
  If it is a single integer, then the \code{k}-th nearest neighbours
  are computed. If it is a vector, then the \code{k[i]}-th nearest
  neighbours are computed for each entry \code{k[i]}. For example, setting
  \code{k=1:3} will compute the nearest, second-nearest and
  third-nearest neighbours. 
}
\section{Sorting data and pre-sorted data}{
  Read this section if you care about the speed of computation.
  
  For efficiency, the algorithm sorts the point pattern \code{X}
  into increasing order of the \eqn{x} coordinate
  or increasing order of the the \eqn{y} coordinate.
  Sorting is only an intermediate step;
  it does not affect the output, which is always given in the same
  order as the original data.
  
  By default (if \code{sortby="range"}),
  the sorting will occur on the coordinate that has the larger range of
  values (according to the frame of the enclosing window of \code{X}).
  If \code{sortby = "var"}), sorting will occur on the coordinate that
  has the greater variance (in the pattern \code{X}).
  Setting \code{sortby="x"} or \code{sortby = "y"} will specify that
  sorting should occur on the \eqn{x} or \eqn{y} coordinate, respectively.

  If the point pattern \code{X} is already
  sorted, then the argument \code{is.sorted.X}
  should be set to \code{TRUE}, and \code{sortby} should be set
  equal to \code{"x"} or \code{"y"} to indicate which coordinate
  is sorted.
}
\section{Warning About Ties}{
  Ties are possible: there may be two data points
  which lie exactly the same distance away from a particular pixel.
  This affects the results from \code{nnmap(what="which")}.
  The handling of ties is not well-defined: it is not consistent
  between different computers and different installations of \R.
  If there are ties, then different calls to \code{nnmap(what="which")}
  may give inconsistent results. For example, you may get a different answer
  from \code{nnmap(what="which",k=1)}
  and \code{nnmap(what="which", k=1:2)[[1]]}.
}
\value{
  A pixel image, or a list of pixel images.

  By default (if \code{what=c("dist", "which")}), the result is
  a list with two components \code{dist} and \code{which}
  containing the distance map and the nearest neighbour map.

  If \code{what="dist"} then the result is a real-valued pixel image
  containing the distance map.
  
  If \code{what="which"} then the result is an integer-valued pixel image
  containing the nearest neighbour map.

  If \code{k} is a vector of several integers, then the result is
  similar except that each pixel image is replaced by a list of
  pixel images, one for each entry of \code{k}.
}
\seealso{
  \code{\link{distmap}}
}
\examples{
  plot(nnmap(cells, 2, what="which"))
}
\author{
  \adrian
  
  ,
  \rolf
  ,
  and Jens Oehlschlaegel
}
\keyword{spatial}
\keyword{math}

\name{multiplicity.ppp}
\alias{multiplicity}
\alias{multiplicity.default}
\alias{multiplicity.data.frame}
\alias{multiplicity.ppp}
\alias{multiplicity.ppx}
\title{Count Multiplicity of Duplicate Points}
\description{
  Counts the number of duplicates for each point
  in a spatial point pattern.
}
\usage{
 multiplicity(x)

 \method{multiplicity}{ppp}(x)

 \method{multiplicity}{ppx}(x)

 \method{multiplicity}{data.frame}(x)

 \method{multiplicity}{default}(x)
}
\arguments{
  \item{x}{
    A spatial point pattern
    (object of class \code{"ppp"} or \code{"ppx"})
    or a vector, matrix or data frame.
  }
}
\value{
  A vector of integers (multiplicities) of length equal to the
  number of points in \code{x}.
}
\details{
  Two points in a point pattern are deemed to be identical
  if their \eqn{x,y} coordinates are the same,
  and their marks are also the same (if they carry marks).
  The Examples section illustrates how it is possible for
  a point pattern to contain a pair of identical points.

  For each point in \code{x}, the function \code{multiplicity} counts how many
  points are identical to it, and returns the vector of counts.

  The argument \code{x} can also be a vector, a matrix or a data frame.
  When \code{x} is a vector, \code{m <- multiplicity(x)} is a vector
  of the same length as \code{x}, and \code{m[i]} is the
  number of elements of \code{x} that are identical to \code{x[i]}.
  When \code{x} is a matrix or data frame,
  \code{m <- multiplicity(x)} is a vector
  of length equal to the number of rows of \code{x}, and \code{m[i]}
  is the number of rows of \code{x} that are identical to
  the \code{i}th row.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{duplicated.ppp}},
  \code{\link{unique.ppp}}
}
\examples{
   X <- ppp(c(1,1,0.5,1), c(2,2,1,2), window=square(3), check=FALSE)
   m <- multiplicity(X)

   # unique points in X, marked by their multiplicity
   first <- !duplicated(X)
   Y <- X[first] \%mark\% m[first]
}

\author{\adrian
  
  ,
  \rolf
  
  and Sebastian Meyer.
}
\keyword{spatial}
\keyword{utilities}
 

\name{pointsOnLines}
\alias{pointsOnLines}
\title{Place Points Evenly Along Specified Lines}
\description{
  Given a line segment pattern, place a series of points
  at equal distances along each line segment.
}
\usage{
pointsOnLines(X, eps = NULL, np = 1000, shortok=TRUE)
}
\arguments{
  \item{X}{A line segment pattern (object of class \code{"psp"}).}
  \item{eps}{Spacing between successive points.}
  \item{np}{Approximate total number of points (incompatible with
    \code{eps}).}
  \item{shortok}{
    Logical. If \code{FALSE}, very short segments
    (of length shorter than \code{eps}) will not generate any points.
    If \code{TRUE}, a very short segment will be represented by its
    midpoint.
  }
}
\details{
  For each line segment in the pattern \code{X}, a succession of points
  is placed along the line segment. These points are equally spaced at
  a distance \code{eps}, except for the first and last points
  in the sequence. 

  The spacing \code{eps} is measured in coordinate units of \code{X}.
  
  If \code{eps} is not given, then it is determined by
  \code{eps = len/np} where \code{len} is the total length of the
  segments in \code{X}. The actual number of points will then be slightly
  larger than \code{np}.
}
\value{
  A point pattern (object of class \code{"ppp"}) in the same window
  as \code{X}. The result also has an attribute called \code{"map"} which
  maps the points to their parent line segments.
}
\seealso{
  \code{\link{psp}},
  \code{\link{ppp}},
  \code{\link{runifpointOnLines}}
}
\examples{
  X <- psp(runif(20), runif(20), runif(20), runif(20),  window=owin())
  Y <- pointsOnLines(X, eps=0.05)
  plot(X, main="")
  plot(Y, add=TRUE, pch="+")
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{duplicated.ppp}
\alias{duplicated.ppp}
\alias{duplicated.ppx}
\alias{anyDuplicated.ppp}
\alias{anyDuplicated.ppx}
\title{Determine Duplicated Points in a Spatial Point Pattern}
\description{
  Determines which points in a spatial point pattern
  are duplicates of previous points, and returns a logical vector.
}
\usage{
 \method{duplicated}{ppp}(x, \dots, rule=c("spatstat", "deldir", "unmark"))

 \method{duplicated}{ppx}(x, \dots)

 \method{anyDuplicated}{ppp}(x, \dots)

 \method{anyDuplicated}{ppx}(x, \dots)
}
\arguments{
  \item{x}{
    A spatial point pattern
    (object of class \code{"ppp"} or \code{"ppx"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{rule}{
    Character string.
    The rule for determining duplicated points.
  }
}
\value{
  \code{duplicated(x)} returns
  a logical vector of length equal to the number of points in \code{x}.

  \code{anyDuplicated(x)} is a number equal to 0 if there are no
  duplicated points, and otherwise is equal to the index of the first
  duplicated point.
}
\details{
  These are methods for the generic functions \code{\link{duplicated}}
  and \code{\link{anyDuplicated}} for 
  point pattern datasets (of class \code{"ppp"}, see
  \code{\link{ppp.object}}, or class \code{"ppx"}).

  \code{anyDuplicated(x)} is a faster version of
  \code{any(duplicated(x))}.

  Two points in a point pattern are deemed to be identical
  if their \eqn{x,y} coordinates are the same,
  and their marks are also the same (if they carry marks).
  The Examples section illustrates how it is possible for
  a point pattern to contain a pair of identical points.

  This function determines which points in \code{x} duplicate
  other points that appeared earlier in the sequence. It 
  returns a logical vector with entries that are \code{TRUE}
  for duplicated points and \code{FALSE} for unique (non-duplicated)
  points.

  If \code{rule="spatstat"} (the default), two points are deemed
  identical if their coordinates are equal according to \code{==},
  \emph{and} their marks are equal according to \code{==}.
  This is the most stringent possible test.
  If \code{rule="unmark"}, duplicated points are
  determined by testing equality of their coordinates only,
  using \code{==}.
  If \code{rule="deldir"}, duplicated points are
  determined by testing equality of their coordinates only,
  using the function \code{\link[deldir]{duplicatedxy}}
  in the package \pkg{deldir}, which currently uses
  \code{\link{duplicated.data.frame}}. Setting \code{rule="deldir"}
  will ensure consistency with functions in the \pkg{deldir} package.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{unique.ppp}},
  \code{\link{multiplicity.ppp}}
}
\examples{
   X <- ppp(c(1,1,0.5), c(2,2,1), window=square(3))
   duplicated(X)
   duplicated(X, rule="deldir")
}

\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}
 

\name{owin}
\alias{owin}
\title{Create a Window}
\description{
  Creates an object of class \code{"owin"} representing 
  an observation window in the two-dimensional plane
}
\usage{
  owin(xrange=c(0,1), yrange=c(0,1), ..., poly=NULL, mask=NULL,
unitname=NULL, xy=NULL)
}
\arguments{
  \item{xrange}{\eqn{x} coordinate limits of enclosing box}
  \item{yrange}{\eqn{y} coordinate limits of enclosing box}
  \item{\dots}{Ignored.}
  \item{poly}{
    Optional. Polygonal boundary of window.
    Incompatible with \code{mask}.
  }
  \item{mask}{
    Optional. Logical matrix giving binary image of window.
    Incompatible with \code{poly}.
  }
  \item{unitname}{
    Optional. Name of unit of length. Either a single character string,
    or a vector of two character strings giving the
    singular and plural forms, respectively.
  }
  \item{xy}{
    Optional. List with components \code{x} and \code{y} specifying the
    pixel coordinates for \code{mask}.
  }
}
\value{
An object of class \code{"owin"} 
describing a window in the two-dimensional plane.
}
\details{
  In the \pkg{spatstat} library, a point pattern dataset must include
  information about the window of observation. This is represented by
  an object of class \code{"owin"}.
  See \code{\link{owin.object}} for an overview.

  To create a window in its own right, 
  users would normally invoke \code{owin},
  although sometimes \code{\link{as.owin}} may be convenient.

  A window may be rectangular, polygonal, or a mask (a binary image).
  \itemize{
    \item
    \bold{rectangular windows:}
    If only \code{xrange} and \code{yrange} are given, then
    the window will be rectangular, with its \eqn{x} and \eqn{y}
    coordinate dimensions given by these two arguments
    (which must be vectors of length 2).
    If no arguments are given at all, the default is the unit square
    with dimensions \code{xrange=c(0,1)} and \code{yrange=c(0,1)}.
    \item
    \bold{polygonal windows:}
    If \code{poly} is given, then the window will be polygonal.
    \itemize{
      \item
      \emph{single polygon:}
      If \code{poly} is a matrix or data frame with two columns, or
      a structure with two component vectors \code{x} and \code{y}
      of equal length,
      then these values are interpreted as the cartesian coordinates
      of the vertices of a polygon circumscribing the window.
      The vertices must be listed \emph{anticlockwise}.
      No vertex should be repeated (i.e. do not repeat the first
      vertex).
      \item
      \emph{multiple polygons or holes:}
      If \code{poly} is a list, each entry \code{poly[[i]]}
      of which is a matrix or data frame with two columns
      or a structure with
      two component vectors \code{x} and \code{y} of equal length,
      then the successive list members \code{poly[[i]]}
      are interpreted as separate polygons which together
      make up the boundary of the window.
      The vertices of each polygon must be listed \emph{anticlockwise}
      if the polygon is part of the external boundary,
      but \emph{clockwise}
      if the polygon is the boundary of a hole in the window.
      Again, do not repeat any vertex.
    }
    \item
    \bold{binary masks:}
    If \code{mask} is given, then the window will be a binary image.
    \itemize{
      \item
      \emph{Specified by logical matrix:}
      Normally the argument \code{mask} should be a logical matrix
      such that \code{mask[i,j]} is \code{TRUE} if the point
      \code{(x[j],y[i])} belongs to the window, and \code{FALSE} if it
      does not. Note carefully that rows of \code{mask} correspond to the
      \eqn{y} coordinate, and columns to the \eqn{x} coordinate.
      Here \code{x} and \code{y} are vectors of \eqn{x} and \eqn{y}
      coordinates equally spaced over \code{xrange} and \code{yrange}
      respectively. The pixel coordinate vectors \code{x} and \code{y}
      may be specified explicitly using the argument \code{xy}, which
      should be a list containing components \code{x} and \code{y}.
      Alternatively there is a sensible default.
      \item
      \emph{Specified by list of pixel coordinates:}
      Alternatively the argument \code{mask} can be a data frame
      with 2 or 3 columns. If it has 2 columns, it is expected to
      contain the spatial coordinates of all the
      pixels which are inside the window.
      If it has 3 columns,
      it should contain the spatial coordinates \eqn{(x,y)}
      of every pixel in the grid, and the logical value associated
      with each pixel. The pixels may be listed in any order.
    }
  }
  To create a window which is mathematically
  defined by inequalities in the Cartesian coordinates, 
  use \code{\link{raster.x}()} and \code{\link{raster.y}()}
  as in the examples below.

  Functions \code{\link{square}} and \code{\link{disc}}
  will create square and circular windows, respectively.
}
\section{Validity of polygon data}{
  Polygon data may contain geometrical inconsistencies such as
  self-intersections and overlaps. These inconsistencies must be
  removed to prevent problems in other \pkg{spatstat} functions.
  By default, polygon data will be repaired automatically
  using polygon-clipping code.
  The repair process may change the number of vertices in a polygon
  and the number of polygon components. 
  To disable the repair process, set \code{spatstat.options(fixpolygons=FALSE)}.
}
\seealso{
  \code{\link{owin.object}},
  \code{\link{as.owin}},
  \code{\link{complement.owin}},
  \code{\link{ppp.object}},
  \code{\link{ppp}}

  \code{\link{square}},
  \code{\link{hexagon}},
  \code{\link{regularpolygon}}, \code{\link{disc}},
  \code{\link{ellipse}}. 
}
\examples{
  w <- owin()
  w <- owin(c(0,1), c(0,1))
  # the unit square

  w <- owin(c(10,20), c(10,30), unitname=c("foot","feet"))
  # a rectangle of dimensions 10 x 20 feet
  # with lower left corner at (10,10)

  # polygon (diamond shape)
  w <- owin(poly=list(x=c(0.5,1,0.5,0),y=c(0,1,2,1)))
  w <- owin(c(0,1), c(0,2), poly=list(x=c(0.5,1,0.5,0),y=c(0,1,2,1)))

  # polygon with hole
  ho <- owin(poly=list(list(x=c(0,1,1,0), y=c(0,0,1,1)),
                       list(x=c(0.6,0.4,0.4,0.6), y=c(0.2,0.2,0.4,0.4))))
  
  w <- owin(c(-1,1), c(-1,1), mask=matrix(TRUE, 100,100))
          # 100 x 100 image, all TRUE
  X <- raster.x(w)
  Y <- raster.y(w)
  wm <- owin(w$xrange, w$yrange, mask=(X^2 + Y^2 <= 1))
          # discrete approximation to the unit disc

  \dontrun{
  if(FALSE) {
    plot(c(0,1),c(0,1),type="n")
    bdry <- locator()
    # click the vertices of a polygon (anticlockwise)
  }
  }
  \testonly{
  bdry <- list(x=c(0.1,0.3,0.7,0.4,0.2),
               y=c(0.1,0.1,0.5,0.7,0.3))
  }
  w <- owin(poly=bdry)
  \dontrun{plot(w)}
 
 \dontrun{
 im <- as.logical(matrix(scan("myfile"), nrow=128, ncol=128))
 # read in an arbitrary 128 x 128 digital image from text file
 rim <- im[, 128:1]
 # Assuming it was given in row-major order in the file
 # i.e. scanning left-to-right in rows from top-to-bottom,
 # the use of matrix() has effectively transposed rows & columns,
 # so to convert it to our format just reverse the column order.
 w <- owin(mask=rim)
 plot(w)
 # display it to check!
 }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{bdist.tiles}
\alias{bdist.tiles}
\title{Distance to Boundary of Window}
\description{
  Computes the shortest distances 
  from each tile in a tessellation
  to the boundary of the window.
}
\usage{
 bdist.tiles(X)
}
\arguments{
  \item{X}{A tessellation (object of class \code{"tess"}).}
}
\value{
  A numeric vector, 
  giving the shortest distance from each tile in the tessellation
  to the boundary of the window.
  Entries of the vector correspond to the entries of \code{tiles(X)}.
}
\details{
  This function computes, for each tile \eqn{s_i}{s[[i]]}
  in the tessellation \code{X}, the shortest distance
  from \eqn{s_i}{s[[i]]}
  to the boundary of the window \eqn{W} containing the tessellation.
}
\seealso{
  \code{\link{tess}},
  \code{\link{bdist.points}},
  \code{\link{bdist.pixels}}
}
\examples{
  P <- runifpoint(15)
  X <- dirichlet(P)
  plot(X, col="red")
  B <- bdist.tiles(X)
  # identify tiles that do not touch the boundary
  plot(X[B > 0], add=TRUE, col="green", lwd=3)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{pp3}
\Rdversion{1.1}
\alias{pp3}
\title{
  Three Dimensional Point Pattern
}
\description{
  Create a three-dimensional point pattern
}
\usage{
pp3(x, y, z, ..., marks=NULL)
}
\arguments{
  \item{x,y,z}{
    Numeric vectors of equal length, containing Cartesian coordinates
    of points in three-dimensional space.
}
  \item{\dots}{
    Arguments passed to \code{\link{as.box3}} to determine
    the three-dimensional box in which the points have been observed.
  }
  \item{marks}{
    Optional.
    Vector, data frame, or hyperframe of mark values associated with the points.
  }
}
\details{
  An object of class \code{"pp3"} represents a pattern of points in
  three-dimensional space. The points are assumed to have been observed
  by exhaustively inspecting a three-dimensional rectangular box. The
  boundaries of the box are included as part of the dataset.
}
\value{
  Object of class \code{"pp3"} representing a three dimensional
  point pattern. Also belongs to class \code{"ppx"}. 
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{box3}}, 
  \code{\link{print.pp3}}, 
  \code{\link{ppx}} 
}
\examples{
   X <- pp3(runif(10), runif(10), runif(10),
            box3(c(0,1)),
            marks=rnorm(10))
   X
}
\keyword{spatial}
\keyword{datagen}

\name{affine}
\alias{affine}
\title{Apply Affine Transformation}
\description{
  Applies any affine transformation of the plane (linear transformation
  plus vector shift) to a plane geometrical object,
  such as a point pattern or a window. 
}
\usage{
  affine(X, \dots)
}
\arguments{
  \item{X}{Any suitable dataset representing a two-dimensional
    object, such as a point pattern (object of class \code{"ppp"}),
    a line segment pattern (object of class \code{"psp"}),
    a window (object of class \code{"owin"}) or a pixel image
    (object of class \code{"im"}).
  }
  \item{\dots}{Arguments determining the affine transformation.}
}
\value{
  Another object of the same type, representing the
  result of applying the affine transformation.
}
\details{
  This is generic. Methods are provided for
  point patterns (\code{\link{affine.ppp}})
  and windows (\code{\link{affine.owin}}).
}
\seealso{
  \code{\link{affine.ppp}},
  \code{\link{affine.psp}},
  \code{\link{affine.owin}},
  \code{\link{affine.im}},
  \code{\link{flipxy}},
  \code{\link{reflect}},
  \code{\link{rotate}},
  \code{\link{shift}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{timed}
\alias{timed}
\title{
  Record the Computation Time
}
\description{
  Saves the result of a calculation as an object of class \code{"timed"}
  which includes information about the time taken to compute the result.
  The computation time is printed when the object is printed.
}
\usage{
timed(x, ..., starttime = NULL, timetaken = NULL)
}
\arguments{
  \item{x}{
    An expression to be evaluated, or an object that has already
    been evaluated.
  }
  \item{starttime}{
    The time at which the computation is defined to have started.
    The default is the current time.
    Ignored if \code{timetaken} is given.
  }
  \item{timetaken}{
    The length of time taken to perform the computation.
    The default is the time taken to evaluate \code{x}.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  This is a simple mechanism for recording how long it takes to
  perform complicated calculations (usually for the purposes of
  reporting in a publication).
  
  If \code{x} is an expression to be evaluated, \code{timed(x)}
  evaluates the expression and measures the
  time taken to evaluate it.
  The result is saved as an object of the class
  \code{"timed"}. Printing this object displays the computation time.

  If \code{x} is an object which has already been computed,
  then the time taken to compute the object can be specified either directly
  by the argument \code{timetaken}, or indirectly by the argument
  \code{starttime}.
  \itemize{
    \item
    \code{timetaken} is the duration of time taken to perform
    the computation. It should be the difference of two clock times
    returned by \code{\link{proc.time}}. Typically the user
    sets \code{begin <- proc.time()} before commencing the calculations,
    then \code{end <- proc.time()} after completing the calculations,
    and then sets \code{timetaken <- end - begin}.
    \item
    \code{starttime} is the clock time at which the computation started.
    It should be a value that was returned by \code{\link{proc.time}}
    at some earlier time when the calculations commenced.
    When \code{timed} is called, the computation time will be taken
    as the difference between the current clock time and
    \code{starttime}. Typically the user
    sets \code{begin <- proc.time()} before commencing the calculations,
    and when the calculations are completed, the user calls
    \code{result <- timed(result, starttime=begin)}. 
  }
  
  If the result of evaluating \code{x} belongs to other S3 classes,
  then the result of \code{timed(x, \dots)} also inherits these classes,
  and printing the object will display the appropriate information for these
  classes as well.
}
\value{
  An object inheriting the class \code{"timed"}.
}
\examples{
  timed(clarkevans(cells))

  timed(Kest(cells))

  answer <- timed(42, timetaken=4.1e17)
  answer
}
\seealso{
  \code{\link{timeTaken}} to extract the time taken.
}
\author{
  \spatstatAuthors.
}
\keyword{utilities}

\name{plot.textstring}
\alias{plot.textstring}
\title{Plot a Text String}
\description{Plots an object of class \code{"textstring"}.}
\usage{
\method{plot}{textstring}(x, \dots, do.plot = TRUE)
}
\arguments{
  \item{x}{
    Object of class \code{"textstring"} to be plotted.
    This object is created by the command \code{\link{textstring}}.
  }
  \item{\dots}{
    Additional graphics arguments passed to
    \code{\link[graphics]{text}} to control the plotting of text.
  }
  \item{do.plot}{
    Logical value indicating whether to actually plot the text.
  }
}
\details{
  The argument \code{x} should be an object of class \code{"textstring"} 
  created by the command \code{\link{textstring}}. 

  This function displays the text using 
  \code{\link[graphics]{text}}.
}
\value{
  A window (class \code{"owin"}) enclosing the plotted graphics.
}
\examples{
  W <- Window(humberside)
  te <- textstring(centroid.owin(W), txt="Humberside", cex=2.5)
  te
  plot(layered(W, te), main="")
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{onearrow}},
  \code{\link{yardstick}}
}
\keyword{spatial}
\keyword{hplot}

\name{centroid.owin}
\alias{centroid.owin}
\title{Centroid of a window}
\description{
  Computes the centroid (centre of mass) of a window 
}
\usage{
 centroid.owin(w, as.ppp = FALSE)
}
\arguments{
  \item{w}{A window}
  \item{as.ppp}{Logical flag indicating whether to return the centroid
    as a point pattern (\code{ppp} object)}
}
\value{
  Either a list with components \code{x, y}, or a point pattern (of class
  \code{ppp}) consisting of a single point, giving the coordinates of the
  centroid of the window \code{w}.
}
\details{
  The centroid of the window \code{w} is computed.
  The centroid (``centre of mass'') 
  is the point whose \eqn{x} and \eqn{y} coordinates 
  are the mean values of the \eqn{x} and \eqn{y} coordinates
  of all points in the window.

  The argument \code{w} should be a window (an object of class
  \code{"owin"}, see \code{\link{owin.object}} for details)
  or can be given in any format acceptable to \code{\link{as.owin}()}.

  The calculation uses an exact analytic formula for the case
  of polygonal windows.

  Note that the centroid of a window is not necessarily inside 
  the window, unless the window is convex.
  If  \code{as.ppp=TRUE} and
  the centroid of \code{w} lies outside \code{w},
  then the window of the returned point pattern
  will be a rectangle containing the
  original window (using \code{\link{as.rectangle}}. 
}
\seealso{
  \code{\link{owin}},
  \code{\link{as.owin}}
}
\examples{
  w <- owin(c(0,1),c(0,1))
  centroid.owin(w)
  # returns 0.5, 0.5

  data(demopat)
  w <- Window(demopat)
  # an irregular window
  cent <- centroid.owin(w, as.ppp = TRUE)
  \dontrun{
  plot(cent)
  # plot the window and its centroid
  }

  wapprox <- as.mask(w)
  # pixel approximation of window
  \dontrun{
  points(centroid.owin(wapprox))
  # should be indistinguishable 
  }
  \testonly{
  centroid.owin(w)
  centroid.owin(wapprox)
  }	
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege
}
\keyword{spatial}
\keyword{math}

\name{rotate}
\alias{rotate}
\title{Rotate}
\description{
  Applies a rotation to any two-dimensional object,
  such as a point pattern or a window. 
}
\usage{
  rotate(X, \dots)
}
\arguments{
  \item{X}{Any suitable dataset representing a two-dimensional
    object, such as a point pattern (object of class \code{"ppp"}),
    or a window (object of class \code{"owin"}).}
  \item{\dots}{Data specifying the rotation.}
}
\value{
  Another object of the same type, representing the
  result of rotating \code{X} through the specified angle.
}
\details{
  This is generic. Methods are provided for
  point patterns (\code{\link{rotate.ppp}})
  and windows (\code{\link{rotate.owin}}).
}
\seealso{
  \code{\link{rotate.ppp}},
  \code{\link{rotate.owin}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{plot.colourmap}
\alias{plot.colourmap}
\title{Plot a Colour Map}
\description{
  Displays a colour map as a colour ribbon
}
\usage{
\method{plot}{colourmap}(x, ...,
       main, xlim = NULL, ylim = NULL, vertical = FALSE, axis = TRUE,
       labelmap=NULL, gap=0.25, add=FALSE, increasing=NULL)
}
\arguments{
  \item{x}{Colour map to be plotted. An object of class \code{"colourmap"}.}
  \item{\dots}{
    Graphical arguments passed to \code{\link{image.default}}
    or \code{\link{axis}}.
  }
  \item{main}{Main title for plot. A character string.}
  \item{xlim}{
    Optional range of \code{x} values for the location of the
    colour ribbon.
  }
  \item{ylim}{
    Optional range of \code{y} values for the location of the
    colour ribbon.
  }
  \item{vertical}{Logical flag determining whether the colour ribbon
    is plotted as a horizontal strip (\code{FALSE}) or a vertical strip
    (\code{TRUE}).}
  \item{axis}{Logical flag determining whether an axis should be plotted
    showing the numerical values that are mapped to the colours.
  }
  \item{labelmap}{
    Function. If this is present, then the labels on the plot,
    which indicate the input values corresponding to particular colours,
    will be transformed by \code{labelmap} before being displayed
    on the plot. Typically used to simplify or shorten the
    labels on the plot.
  }
  \item{gap}{
    Distance between separate blocks of colour, as a fraction of
    the width of one block, if the colourmap is discrete.
  }
  \item{add}{
    Logical value indicating whether to add the colourmap to the
    existing plot (\code{add=TRUE}), or to start a new plot
    (\code{add=FALSE}, the default).
  }
  \item{increasing}{
    Logical value indicating whether to display the colour map
    in increasing order. See Details.
  }
}
\details{
  This is the plot method for the class \code{"colourmap"}.
  An object of this class
  (created by the function \code{\link{colourmap}})
  represents a colour map or
  colour lookup table associating colours with each data value.

  The command \code{plot.colourmap} displays the colour map as a colour
  ribbon or as a colour legend (a sequence of blocks of colour).
  This plot can be useful on its own to inspect the colour map.

  If the domain of the colourmap is an interval of real numbers,
  the colourmap is displayed as a continuous ribbon of colour.
  If the domain of the colourmap is a finite set of inputs,
  the colours are displayed as separate blocks of colour.
  The separation between blocks is equal to \code{gap} times
  the width of one block. 

  To annotate an existing plot with an explanatory colour ribbon
  or colour legend,
  specify \code{add=TRUE} and use the arguments \code{xlim}
  and/or \code{ylim} to control the physical position of the ribbon
  on the plot.

  Labels explaining the colour map are
  drawn by \code{\link[graphics]{axis}} and can be modified by
  specifying arguments that will be passed to this function.

  The argument \code{increasing} indicates whether the
  colourmap should be displayed so that the
  input values are increasing with the spatial coordinate:
  that is, increasing from left to right (if \code{vertical=FALSE})
  or increasing from bottom to top (if \code{vertical=TRUE}).
  If \code{increasing=FALSE}, this ordering will be reversed.
  The default is \code{increasing=TRUE} in all cases
  except when \code{vertical=TRUE} and
  the domain of the colourmap is a finite set of discrete inputs.
}
\value{
  None.
}
\seealso{\code{\link{colourmap}}}
\examples{
  co <- colourmap(rainbow(100), breaks=seq(-1,1,length=101))
  plot(co)
  plot(co, col.ticks="pink")
  ca <- colourmap(rainbow(8), inputs=letters[1:8])
  plot(ca, vertical=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{color}
\keyword{hplot}

\name{project2set}
\alias{project2set}
\title{
  Find Nearest Point in a Region
}
\description{
  For each data point in a point pattern \code{X},
  find the nearest location in a given spatial region \code{W}.
}
\usage{
  project2set(X, W, \dots)
}
\arguments{
  \item{X}{
    Point pattern (object of class \code{"ppp"}).
  }
  \item{W}{
    Window (object of class \code{"owin"})
    or something acceptable to \code{\link{as.owin}}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} controlling the
    pixel resolution.
  }
}
\details{
  The window \code{W} is first discretised as a binary mask
  using \code{\link{as.mask}}.
  
  For each data point \code{X[i]} in the point pattern \code{X},
  the algorithm finds the nearest pixel in \code{W}.

  The result is a point pattern \code{Y} containing these nearest points,
  that is, \code{Y[i]} is the nearest point in \code{W} to the
  point \code{X[i]}.
}
\value{
  A point pattern (object of class \code{"ppp"}) with the same
  number of points as \code{X} in the window \code{W}.
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link{project2segment}},
  \code{\link{nncross}}
}
\examples{
  He <- heather$fine[owin(c(2.8, 7.4), c(4.0, 7.8))]
  plot(He, main="project2set")
  X <- runifpoint(4, erosion(complement.owin(He), 0.2))
  points(X, col="red")
  Y <- project2set(X, He)
  points(Y, col="green")
  arrows(X$x, X$y, Y$x, Y$y, angle=15, length=0.2)
}
\keyword{spatial}
\keyword{math}

\name{corners}
\alias{corners}
\title{Corners of a rectangle}
\description{
  Returns the four corners of a rectangle 
}
\usage{
 corners(window)
}
\arguments{
  \item{window}{A window. 
    An object of class \code{\link{owin}},
    or data in any format acceptable to \code{\link{as.owin}()}.
  }
}
\value{
  A list with two components \code{x} and \code{y}, which are numeric
  vectors of length 4 giving the coordinates of the four corner points
  of the (bounding rectangle of the) window.
}
\details{
  This trivial function is occasionally convenient.
  If \code{window} is of type \code{"rectangle"} this returns the
  four corners of the window itself; 
  otherwise, it returns the corners of the bounding rectangle
  of the window.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{quadscheme}}
}
\examples{
  w <- unit.square()
  corners(w)
       # returns list(x=c(0,1,0,1),y=c(0,0,1,1))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{utilities}

\name{quantile.density}
\alias{quantile.density}
\title{
  Quantiles of a Density Estimate
}
\description{
  Given a kernel estimate of a probability density, compute quantiles.
}
\usage{
\method{quantile}{density}(x, probs = seq(0, 1, 0.25), names = TRUE,
        \dots, warn = TRUE)
}
\arguments{
  \item{x}{
    Object of class \code{"density"} computed by a method for
    \code{\link[stats]{density}}
  }
  \item{probs}{
    Numeric vector of probabilities for which the quantiles are required.
  }
  \item{names}{
    Logical value indicating whether to attach names (based on
    \code{probs}) to the result.
  }
  \item{\dots}{
    Ignored.
  }
  \item{warn}{
    Logical value indicating whether to issue a warning if the
    density estimate \code{x} had to be renormalised because it
    was computed in a restricted interval.
  }
}
\details{
  This function calculates quantiles of the probability distribution
  whose probability density has been estimated and stored in the object
  \code{x}. The object \code{x} must belong to the class \code{"density"},
  and would typically have been obtained from a call to the function
  \code{\link[stats]{density}}.

  The probability density is first normalised so that the total
  probability is equal to 1. A warning is issued if the density
  estimate was restricted to an interval (i.e. if \code{x}
  was created by a call to   \code{\link[stats]{density}} which
  included either of the arguments \code{from} and \code{to}).

  Next, the density estimate is numerically integrated to obtain an estimate
  of the cumulative distribution function \eqn{F(x)}. Then
  for each desired probability \eqn{p}, the algorithm finds the
  corresponding quantile \eqn{q}.

  The quantile \eqn{q} corresponding to probability \eqn{p}
  satisfies \eqn{F(q) = p} up to
  the resolution of the grid of values contained in \code{x}.
  The quantile is computed from the right, that is,
  \eqn{q} is the smallest available value of \eqn{x} such that
  \eqn{F(x) \ge p}{F(x) >= p}.
}
\value{
  A numeric vector containing the quantiles.
}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link[stats]{quantile}}, 
  \code{\link{quantile.ewcdf}}, 
  \code{\link{quantile.im}},
  \code{\link{CDF}}.
}
\examples{
   dd <- density(runif(10))
   quantile(dd)
}
\keyword{methods}
\keyword{univar}
\keyword{nonparametric}

\name{pairdist.psp}
\alias{pairdist.psp}
\title{Pairwise distances between line segments}
\description{
  Computes the matrix of distances between all pairs of line segments
  in a line segment pattern.
}
\usage{
  \method{pairdist}{psp}(X, \dots, method="C", type="Hausdorff")
}
\arguments{
  \item{X}{
    A line segment pattern (object of class \code{"psp"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{method}{
    String specifying which method of calculation to use.
    Values are \code{"C"} and \code{"interpreted"}.
    Usually not specified.
  }
  \item{type}{
    Type of distance to be computed. Options are
    \code{"Hausdorff"} and \code{"separation"}. Partial matching is used.
  }
}
\value{
  A square matrix whose \code{[i,j]} entry is the distance
  between the line segments numbered \code{i} and \code{j}.
}
\details{
  This function computes the distance between each pair of
  line segments in \code{X}, and returns the matrix of distances.
  
  This is a method for the generic function \code{\link{pairdist}}
  for the class \code{"psp"}.

  The distances between line segments are measured in one of two ways:
  \itemize{
    \item if \code{type="Hausdorff"}, distances are computed
    in the Hausdorff metric. The Hausdorff
    distance between two line segments is the \emph{maximum} distance
    from any point on one of the segments to the nearest point on
    the other segment.
    \item if \code{type="separation"}, distances are computed
    as the \emph{minimum} distance from a point on one line segment to
    a point on the other line segment. For example, line segments which
    cross over each other have separation zero.
  }
  
  The argument \code{method} is not normally used. It is
  retained only for checking the validity of the software.
  If \code{method = "interpreted"} then the distances are
  computed using interpreted \R code only. If \code{method="C"}
  (the default) then compiled \code{C} code is used,
  which is somewhat faster.
}
\seealso{
  \code{\link{crossdist}},
  \code{\link{nndist}},
  \code{\link{pairdist.ppp}}
}
\examples{
   L <- psp(runif(10), runif(10), runif(10), runif(10), owin())
   D <- pairdist(L)
   S <- pairdist(L, type="sep")
}
\author{
  \adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{im.object}
\alias{im.object} %DoNotExport
\title{Class of Images}
\description{
  A class \code{"im"} to represent a two-dimensional pixel image.
}
\details{
  An object of this class represents
  a two-dimensional pixel image. It specifies
  \itemize{
    \item the dimensions of the rectangular array of pixels
    \item \eqn{x} and \eqn{y} coordinates for the pixels
    \item a numeric value (``grey value'') at each pixel
  }
  If \code{X} is an object of type \code{im},
  it contains the following elements:
  \tabular{ll}{
    \code{v} \tab matrix of values \cr
    \code{dim} \tab dimensions of matrix \code{v} \cr
    \code{xrange} \tab range of \eqn{x} coordinates of image window \cr
    \code{yrange} \tab range of \eqn{y} coordinates of image window \cr
    \code{xstep} \tab width of one pixel \cr
    \code{ystep} \tab height of one pixel \cr
    \code{xcol} \tab vector of \eqn{x} coordinates of centres of pixels \cr
    \code{yrow} \tab vector of \eqn{y} coordinates of centres of pixels 
  }
  Users are strongly advised not to manipulate these entries
  directly.

  Objects of class \code{"im"}
  may be created by the functions
  \code{\link{im}} and \code{\link{as.im}}.
  Image objects are also returned by various functions including
  \code{\link{distmap}}, \code{\link{Kmeasure}}, \code{\link{setcov}}, 
  \code{\link{eval.im}} and \code{\link{cut.im}}.

  Image objects may be displayed using the methods
  \code{\link{plot.im}}, \code{image.im}, \code{\link{persp.im}}
  and \code{contour.im}. There are also methods 
  \code{\link{print.im}} for printing information about an image,
  \code{\link{summary.im}} for summarising an image,
  \code{\link{mean.im}} for calculating the average pixel value, 
  \code{\link{hist.im}} for plotting a histogram of pixel values,
  \code{\link{quantile.im}} for calculating quantiles of pixel values,
  and \code{\link{cut.im}} for dividing the range of pixel values into
  categories.

  Pixel values in an image may be extracted
  using the subset operator \code{\link{[.im}}.
  To extract all pixel values from an image object,
  use \code{\link{as.matrix.im}}.
  The levels of a factor-valued image can be extracted and
  changed with \code{levels} and \code{levels<-}.

  Calculations involving one or more images (for example,
  squaring all the pixel values in an image, converting numbers to 
  factor levels, or 
  subtracting one image from another) can often be done
  easily using \code{\link{eval.im}}.
  To find all pixels satisfying
  a certain constraint, use \code{\link{solutionset}}.
  
  Note carefully that the entry \code{v[i,j]}
  gives the pixel value at the location \code{(xcol[j],yrow[i]}.
  That is, the \bold{row} index of the matrix \code{v} corresponds
  to increasing \bold{y} coordinate, while the column index of \code{mat}
  corresponds to increasing \bold{x} coordinate.
  Thus \code{yrow} has one entry for each row of \code{v}
  and \code{xcol} has one entry for each column of \code{v}.
  Under the usual convention in \R, a correct
  display of the image would be obtained by transposing the matrix, e.g.
  \code{image.default(xcol, yrow, t(v))}, if you wanted to do it by hand.
}
\seealso{
  \code{\link{im}},
  \code{\link{as.im}},
  \code{\link{plot.im}},
  \code{\link{persp.im}},
  \code{\link{eval.im}},
  \code{\link{[.im}}
}
\section{Warnings}{
  The internal representation of images is likely to change in future
  releases of \pkg{spatstat}. Do not address the entries in an
  image directly. To extract all pixel values from an image object,
  use \code{\link{as.matrix.im}}.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{attribute}
 
 

\name{pairdist.default}
\alias{pairdist.default}
\title{Pairwise distances}
\description{
  Computes the matrix of distances between all pairs of points
  in a set of points in two dimensional space
}
\usage{
  \method{pairdist}{default}(X, Y=NULL, \dots, period=NULL, method="C", squared=FALSE)
}
\arguments{
  \item{X,Y}{
    Arguments specifying the coordinates of a set of points.
    Typically \code{X} and \code{Y} would be
    numeric vectors of equal length.
    Alternatively \code{Y} may be omitted and \code{X} may be
    a list with two components \code{x} and \code{y},
    or a matrix with two columns.
  }
  \item{\dots}{
    Ignored.
  }
  \item{period}{
    Optional. Dimensions for periodic edge correction.
  }
  \item{method}{
    String specifying which method of calculation to use.
    Values are \code{"C"} and \code{"interpreted"}.
    Usually not specified.
  }
  \item{squared}{
    Logical. If \code{squared=TRUE}, the squared distances are
    returned instead (this computation is faster).
  }
}
\value{
  A square matrix whose \code{[i,j]} entry is the distance
  between the points numbered \code{i} and \code{j}.
}
\details{
  Given the coordinates of a set of points in two dimensional space,
  this function computes the Euclidean distances between all pairs of
  points, and returns the matrix of distances.
  It is a method for the generic function \code{pairdist}.
  
  Note: If only pairwise distances within some threshold value are
  needed the low-level function \code{\link{closepairs}} may be much
  faster to use.

  The arguments \code{X} and \code{Y} must determine
  the coordinates of a set of points. Typically \code{X} and
  \code{Y} would be numeric vectors of equal length. Alternatively
  \code{Y} may be omitted and \code{X} may be a list with two components
  named \code{x} and \code{y}, or a matrix or data frame with two columns.
  
  For typical input the result is numerically equivalent to
  (but computationally faster than) \code{as.matrix(dist(x))} where
  \code{x = cbind(X, Y)}, but that command is useful for calculating 
  all pairwise distances between points in \eqn{k}-dimensional space
  when \code{x} has \eqn{k} columns.
  
  Alternatively if \code{period} is given,
  then the distances will be computed in the `periodic'
  sense (also known as `torus' distance).
  The points will be treated as if they are in a rectangle
  of width \code{period[1]} and height \code{period[2]}.
  Opposite edges of the rectangle are regarded as equivalent.
  
  If \code{squared=TRUE} then the \emph{squared} Euclidean distances
  \eqn{d^2} are returned, instead of the Euclidean distances \eqn{d}.
  The squared distances are faster to calculate, and are sufficient for
  many purposes (such as finding the nearest neighbour of a point).
  
  The argument \code{method} is not normally used. It is
  retained only for checking the validity of the software.
  If \code{method = "interpreted"} then the distances are
  computed using interpreted R code only. If \code{method="C"}
  (the default) then C code is used. The C code is somewhat faster.
}
\seealso{
  \code{\link{crossdist}},
  \code{\link{nndist}},
  \code{\link{Kest}},
  \code{\link{closepairs}}
}
\examples{
   x <- runif(100)
   y <- runif(100)
   d <- pairdist(x, y)
   d <- pairdist(cbind(x,y))
   d <- pairdist(x, y, period=c(1,1))
   d <- pairdist(x, y, squared=TRUE)
}
\author{Pavel Grabarnik
  \email{pavel.grabar@issp.serpukhov.su}
  and 
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{rNeymanScott}
\alias{rNeymanScott}
\title{Simulate Neyman-Scott Process}
\description{
  Generate a random point pattern, a realisation of the
  Neyman-Scott cluster process.
}
\usage{
 rNeymanScott(kappa, expand, rcluster, win = owin(c(0,1),c(0,1)),
              \dots, lmax=NULL, nsim=1, drop=TRUE,
              nonempty=TRUE, saveparents=TRUE)
}
\arguments{
  \item{kappa}{
    Intensity of the Poisson process of cluster centres.
    A single positive number, a function, or a pixel image.
  }
  \item{expand}{
    Size of the expansion of the simulation window for generating parent
    points. A single non-negative number.
  }
  \item{rcluster}{
    A function which generates random clusters,
    or other data specifying the random cluster mechanism.
    See Details.
  }
  \item{win}{
    Window in which to simulate the pattern.
    An object of class \code{"owin"}
    or something acceptable to \code{\link{as.owin}}.
  }
  \item{\dots}{
    Arguments passed to \code{rcluster}.
  }
  \item{lmax}{
    Optional. Upper bound on the values of \code{kappa}
    when \code{kappa} is a function or pixel image.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
  \item{nonempty}{
    Logical. If \code{TRUE} (the default), a more efficient algorithm is
    used, in which parents are generated conditionally on having at
    least one offspring point. If \code{FALSE}, parents are generated
    even if they have no offspring. Both choices are valid; the default
    is recommended unless you need to simulate all the parent points
    for some other purpose.
  }
  \item{saveparents}{
    Logical value indicating whether to save the locations of the
    parent points as an attribute.
  }
}
\value{
  A point pattern (an object of class \code{"ppp"}) if \code{nsim=1},
  or a list of point patterns if \code{nsim > 1}.
  
  Additionally,  some intermediate results of the simulation are
  returned as attributes of this point pattern: see Details.
}
\details{
  This algorithm generates a realisation of the
  general Neyman-Scott process, with the cluster mechanism
  given by the function \code{rcluster}. 

  First, the algorithm generates a Poisson point process of
  \dQuote{parent} points with intensity \code{kappa} in an expanded
  window as explained below. Here \code{kappa}
  may be a single positive number,
  a function \code{kappa(x,y)},
  or a pixel image object of class \code{"im"} (see
  \code{\link{im.object}}).  See \code{\link{rpoispp}} for details.

  Second, each parent point is replaced by a random cluster
  of points. These clusters are combined together to yield a
  single point pattern, and the restriction of this pattern to the
  window \code{win} is then returned as the result of
  \code{rNeymanScott}.

  The expanded window consists of \code{\link{as.rectangle}(win)}
  extended by the amount \code{expand} in each direction. The size of
  the expansion is saved in the attribute \code{"expand"} and may be
  extracted by \code{attr(X, "expand")} where \code{X} is the generated
  point pattern.  

  The argument \code{rcluster} specifies the cluster mechanism.
  It may be either:
  \itemize{
    \item
    A \code{function} which will be called to generate each random
    cluster (the offspring points of each parent point).
    The function should expect to be called
    in the form \code{rcluster(x0,y0,\dots)} for a parent point at a location
    \code{(x0,y0)}. The return value of \code{rcluster}
    should specify the coordinates of the points in the cluster;
    it may be a list containing elements
    \code{x,y}, or a point pattern (object of
    class \code{"ppp"}). If it is a marked point pattern then the result of
    \code{rNeymanScott} will be a marked point pattern.
    \item
    A \code{list(mu, f)} where \code{mu} specifies the mean
    number of offspring points in each cluster, and \code{f}
    generates the random displacements (vectors pointing from the parent
    to the offspring). In this case, the number of offspring
    in a cluster is assumed to have a Poisson distribution, implying
    that the Neyman-Scott process is also a Cox process.
    The first element \code{mu} should be either a single nonnegative
    number (interpreted as the mean of the Poisson distribution of
    cluster size)
    or a pixel image or a \code{function(x,y)} giving a spatially
    varying mean cluster size (interpreted in the sense of
    Waagepetersen, 2007).
    The second element \code{f} should be a function that will be
    called once in the form \code{f(n)} to generate \code{n} independent
    and identically distributed displacement vectors (i.e. as if there
    were a cluster of size \code{n} with a parent at the origin
    \code{(0,0)}). 
    The function should return
    a point pattern (object of class \code{"ppp"})
    or something acceptable to \code{\link[grDevices]{xy.coords}}
    that specifies the coordinates of \code{n} points. 
  }

  If required, the intermediate stages of the simulation (the
  parents and the individual clusters) can also be extracted from
  the return value of \code{rNeymanScott} through the attributes
  \code{"parents"} and \code{"parentid"}.  The attribute
  \code{"parents"} is the point pattern of parent points.
  The attribute \code{"parentid"} is an integer vector specifying
  the parent for each of the points in the simulated pattern.

  Neyman-Scott models where \code{kappa} is a single number
  and \code{rcluster = list(mu,f)} can be fitted to data
  using the function \code{\link{kppm}}.
}
\section{Inhomogeneous Neyman-Scott Processes}{
  There are several different ways of specifying a spatially inhomogeneous
  Neyman-Scott process:
  \itemize{
    \item
    The point process of parent points can be inhomogeneous. 
    If the argument \code{kappa} is a \code{function(x,y)} or a pixel
    image (object of class \code{"im"}), then it is taken as specifying
    the intensity function of an inhomogeneous Poisson process according
    to which the parent points are generated.
    \item
    The number of points in a typical cluster can
    be spatially varying.
    If the argument \code{rcluster} is a list of two elements
    \code{mu, f} and the first entry \code{mu} is a 
    \code{function(x,y)} or a pixel image (object of class \code{"im"}),
    then \code{mu} is interpreted as the reference intensity
    for offspring points, in the sense of Waagepetersen (2007).
    For a given parent point, the offspring constitute a Poisson process
    with intensity function equal to \code{mu(x, y) * g(x-x0, y-y0)}
    where \code{g} is the probability density of the offspring
    displacements generated by the function \code{f}.

    Equivalently, clusters are first generated with a constant
    expected number of points per cluster: the constant is \code{mumax}, the
    maximum of \code{mu}. Then the offspring are randomly \emph{thinned}
    (see \code{\link{rthin}}) with spatially-varying retention
    probabilities given by \code{mu/mumax}.  
    \item
    The entire mechanism for generating a cluster can
    be dependent on the location of the parent point.
    If the argument \code{rcluster} is a function,
    then the cluster associated with a parent point at location
    \code{(x0,y0)} will be generated by calling
    \code{rcluster(x0, y0, \dots)}. The behaviour of this function
    could depend on the location \code{(x0,y0)} in any fashion.
  }

  Note that if \code{kappa} is an
  image, the spatial domain covered by this image must be large
  enough to include the \emph{expanded} window in which the parent
  points are to be generated. This requirement means that \code{win} must
  be small enough so that the expansion of \code{as.rectangle(win)}
  is contained in the spatial domain of \code{kappa}.  As a result,
  one may wind up having to simulate the process in a window smaller
  than what is really desired.

  In the first two cases, the intensity of the Neyman-Scott process
  is equal to \code{kappa * mu} if at least one of \code{kappa} or
  \code{mu} is a single number, and is otherwise equal to an
  integral involving \code{kappa}, \code{mu} and \code{f}.
}
\seealso{
  \code{\link{rpoispp}},
  \code{\link{rThomas}},
  \code{\link{rGaussPoisson}},
  \code{\link{rMatClust}},
  \code{\link{rCauchy}},
  \code{\link{rVarGamma}}
}
\examples{
  # each cluster consist of 10 points in a disc of radius 0.2
  nclust <- function(x0, y0, radius, n) {
              return(runifdisc(n, radius, centre=c(x0, y0)))
            }
  plot(rNeymanScott(10, 0.2, nclust, radius=0.2, n=5))

  # multitype Neyman-Scott process (each cluster is a multitype process)
  nclust2 <- function(x0, y0, radius, n, types=c("a", "b")) {
     X <- runifdisc(n, radius, centre=c(x0, y0))
     M <- sample(types, n, replace=TRUE)
     marks(X) <- M
     return(X)
  }
  plot(rNeymanScott(15,0.1,nclust2, radius=0.1, n=5))
}
\references{
  Neyman, J. and Scott, E.L. (1958)
  A statistical approach to problems of cosmology.
  \emph{Journal of the Royal Statistical Society, Series B}
  \bold{20}, 1--43.

  Waagepetersen, R. (2007)
  An estimating function approach to inference for
  inhomogeneous Neyman-Scott processes.
  \emph{Biometrics} \bold{63}, 252--258.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{connected.tess}  
\Rdversion{1.1}
\alias{connected.tess}
\title{
  Connected Components of Tiles of a Tessellation
}
\description{
  Given a tessellation, find the topologically-connected pieces
  of each tile, and make a new tessellation using these pieces.
}
\usage{
\method{connected}{tess}(X, \dots)
}
\arguments{
  \item{X}{
    A tessellation (object of class \code{"tess"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine the
    pixel resolution.
  }
}
\details{
  The function \code{connected} is generic.
  This function \code{connected.tess} is the method for tessellations.

  Given the tessellation \code{X}, the algorithm considers each tile
  of the tessellation, and identifies its connected components
  (topologically-connected pieces) using \code{\link{connected.owin}}.
  Each of these pieces is treated as a distinct tile
  and a new tessellation is made from these pieces.

  The result is another tessellation obtained by subdividing each tile
  of \code{X} into one or more new tiles.
}
\value{
  Another tessellation (object of class \code{"tess"}).
}
\seealso{
  \code{\link{connected.owin}}
}
\examples{
  BB <- grow.rectangle(Frame(letterR), 0.2)
  H <- tess(tiles=list(IN=letterR, OUT=complement.owin(letterR, BB)))
  opa <- par(mfrow=c(1,2))
  plot(H, do.col=TRUE)
  plot(connected(H), do.col=TRUE, col=2:4)
  par(opa)
}
\author{
  \adrian.
}
\keyword{spatial}
\keyword{math}

\name{as.boxx}
\alias{as.boxx}
\title{Convert Data to Multi-Dimensional Box}
\description{Interprets data as the dimensions of a multi-dimensional box.}
\usage{
  as.boxx(\dots, warn.owin = TRUE)
}
\arguments{
  \item{\dots}{
    Data that can be interpreted as giving the dimensions of a
    multi-dimensional box. See Details.
  }
  \item{warn.owin}{
    Logical value indicating whether to print a warning
    if a non-rectangular window (object of class \code{"owin"})
    is supplied.
  }
}
\details{
  Either a single argument should be provided which is one
  of the following:
  \itemize{
    \item an object of class \code{"boxx"}
    \item an object of class \code{"box3"}
    \item an object of class \code{"owin"}
    \item a numeric vector of even length, specifying the corners
    of the box. See Examples
  }
  or a list of arguments acceptable to \code{\link{boxx}}.
}
\value{A \code{"boxx"} object.}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\examples{
 # Convert unit square to two dimensional box.
 W <- owin()
 as.boxx(W)
 # Make three dimensional box [0,1]x[0,1]x[0,1] from numeric vector
 as.boxx(c(0,1,0,1,0,1))
}

\name{delaunay}
\alias{delaunay}
\title{Delaunay Triangulation of Point Pattern}
\description{
  Computes the Delaunay triangulation of a spatial point pattern.
}
\usage{
delaunay(X)
}
\arguments{
  \item{X}{Spatial point pattern (object of class \code{"ppp"}).}
}
\details{
  The Delaunay triangulation of a spatial point pattern \code{X}
  is defined as follows. First the Dirichlet/Voronoi tessellation of \code{X}
  computed; see \code{\link{dirichlet}}. Then two points of \code{X}
  are defined to be Delaunay neighbours if their Dirichlet/Voronoi tiles
  share a common boundary. Every pair of Delaunay neighbours is
  joined by a straight line. The result is a tessellation, consisting of
  disjoint triangles. The union of these triangles is the convex hull of
  \code{X}.
}
\value{
  A tessellation (object of class \code{"tess"}). The window of the
  tessellation is the convex hull of \code{X}, not the original window
  of \code{X}.
}
\seealso{
  \code{\link{tess}},
  \code{\link{dirichlet}},
  \code{\link{convexhull.xy}},
  \code{\link{ppp}},
  \code{\link{delaunayDistance}},
  \code{\link{delaunayNetwork}}
}
\examples{
  X <- runifpoint(42)
  plot(delaunay(X))
  plot(X, add=TRUE)
}
\author{
\adrian


\rolf

and \ege

}
\keyword{spatial}
\keyword{manip}

\name{affine.im}
\alias{affine.im} 
\title{Apply Affine Transformation To Pixel Image}
\description{
  Applies any affine transformation of the plane (linear transformation
  plus vector shift) to a pixel image. 
}
\usage{
  \method{affine}{im}(X, mat=diag(c(1,1)), vec=c(0,0), \dots) 
}
\arguments{
  \item{X}{Pixel image (object of class \code{"im"}).}
  \item{mat}{Matrix representing a linear transformation.}
  \item{vec}{Vector of length 2 representing a translation.}
  \item{\dots}{
    Optional arguments passed to \code{\link{as.mask}}
    controlling the pixel resolution of the transformed image.
  }
}
\value{
  Another pixel image (of class \code{"im"}) representing the
  result of applying the affine transformation.
}
\details{
  The image is subjected first to the linear transformation represented by
  \code{mat} (multiplying on the left by \code{mat}),
  and then the result is translated by the vector \code{vec}.
  
  The argument \code{mat} must be a nonsingular \eqn{2 \times 2}{2 * 2}
  matrix.

  This is a method for the generic function \code{\link{affine}}.
}
\seealso{
  \code{\link{affine}},
  \code{\link{affine.ppp}},
  \code{\link{affine.psp}},
  \code{\link{affine.owin}},
  \code{\link{rotate}},
  \code{\link{shift}}
}
\examples{
  X <- setcov(owin())
  stretch <- diag(c(2,3))
  Y <- affine(X, mat=stretch)
  shear <- matrix(c(1,0,0.6,1),ncol=2, nrow=2)
  Z <- affine(X, mat=shear)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{distfun}  
\Rdversion{1.1}
\alias{distfun}
\alias{distfun.ppp}
\alias{distfun.psp}
\alias{distfun.owin}
\title{
  Distance Map as a Function 
}
\description{
  Compute the distance function of an object, and return it as a function.
}
\usage{
  distfun(X, \dots)

  \method{distfun}{ppp}(X, \dots, k=1, undef=Inf)

  \method{distfun}{psp}(X, \dots)

  \method{distfun}{owin}(X, \dots, invert=FALSE)
}
\arguments{
  \item{X}{Any suitable dataset representing a two-dimensional
    object, such as a point pattern (object of class \code{"ppp"}),
    a window (object of class \code{"owin"}) or a
    line segment pattern (object of class \code{"psp"}).
  }
  \item{\dots}{
    Extra arguments are ignored. 
  }
  \item{k}{
    An integer. The distance to the \code{k}th nearest point
    will be computed.
  }
  \item{undef}{
    The value that should be returned if the distance is undefined
    (that is, if \code{X} contains fewer than \code{k} points).
  }
  \item{invert}{
    If \code{TRUE}, compute the distance transform of the
    complement of \code{X}.
  }
}
\details{
  The \dQuote{distance function} of a set of points \eqn{A} is the
  mathematical function \eqn{f} such that, for any 
  two-dimensional spatial location \eqn{(x,y)},
  the function value \code{f(x,y)}
  is the shortest distance from \eqn{(x,y)} to \eqn{A}.

  The command \code{f <- distfun(X)} returns a \emph{function}
  in the \R language, with arguments \code{x,y}, that represents the
  distance function of \code{X}. Evaluating the function \code{f}
  in the form \code{v <- f(x,y)}, where \code{x} and \code{y}
  are any numeric vectors of equal length containing coordinates of
  spatial locations, yields the values of the distance function at these
  locations. Alternatively \code{x} can be a
  point pattern (object of class \code{"ppp"} or \code{"lpp"}) of
  locations at which the distance function should be computed (and then
  \code{y} should be missing).

  This should be contrasted with the related command \code{\link{distmap}}
  which computes the distance function of \code{X}
  on a grid of locations, and returns the distance
  values in the form of a pixel image.

  The result of \code{f <- distfun(X)} also belongs to the class
  \code{"funxy"} and to the special class \code{"distfun"}.
  It can be printed and plotted immediately as shown in the Examples.
  
  A \code{distfun} object can be converted to a pixel image
  using \code{\link{as.im}}.
}
\value{
  A \code{function} with arguments \code{x,y}.
  The function belongs to the class \code{"distfun"} which has
  methods for \code{print} and \code{summary}, and for geometric
  operations like \code{shift}.
  It also belongs to the class \code{"funxy"} which has methods
  for \code{plot}, \code{contour} and \code{persp}.
}
\seealso{
  \code{\link{distmap}},
  \code{\link{summary.distfun}},
  \code{\link{methods.distfun}},
  \code{\link{methods.funxy}},
  \code{\link{plot.funxy}}
}
\examples{
   data(letterR)
   f <- distfun(letterR)
   f
   plot(f)
   f(0.2, 0.3)

   plot(distfun(letterR, invert=TRUE), eps=0.1)

   d <- distfun(cells)
   d2 <- distfun(cells, k=2)
   d(0.5, 0.5)
   d2(0.5, 0.5)
   domain(d)
   summary(d)

   z <- d(japanesepines)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{as.ppp}
\alias{as.ppp}
\alias{as.ppp.ppp}
\alias{as.ppp.psp}
\alias{as.ppp.quad}
\alias{as.ppp.matrix}
\alias{as.ppp.data.frame}
\alias{as.ppp.influence.ppm}
\alias{as.ppp.default}
\title{Convert Data To Class ppp}
\description{
  Tries to coerce any reasonable kind of data to a spatial point pattern
  (an object of class \code{"ppp"})
  for use by the \pkg{spatstat} package).
}
\usage{
  as.ppp(X, \dots, fatal=TRUE)

  \method{as.ppp}{ppp}(X, \dots, fatal=TRUE)

  \method{as.ppp}{psp}(X, \dots, fatal=TRUE)

  \method{as.ppp}{quad}(X, \dots, fatal=TRUE)

  \method{as.ppp}{matrix}(X, W=NULL, \dots, fatal=TRUE)

  \method{as.ppp}{data.frame}(X, W=NULL, \dots, fatal=TRUE)

  \method{as.ppp}{influence.ppm}(X, \dots)

  \method{as.ppp}{default}(X, W=NULL, \dots, fatal=TRUE)
}
\arguments{
  \item{X}{Data which will be converted into a point pattern}
  \item{W}{
    Data which define a window for the pattern,
    when \code{X} does not contain a window.
    (Ignored if \code{X} contains window information.)
  }
  \item{\dots}{Ignored.}
  \item{fatal}{
    Logical value specifying what to do if the
    data cannot be converted.
    See Details.
  }
}
\value{
  An object of class \code{"ppp"} (see \code{\link{ppp.object}})
  describing the point pattern and its window of observation.
  The value \code{NULL} may also be returned; see Details.
}
\details{
  Converts the dataset \code{X} to a point pattern
  (an object of class \code{"ppp"}; see \code{\link{ppp.object}} for
  an overview).

  This function is normally used to convert an existing point pattern
  dataset, stored in another format, to the \code{"ppp"} format.
  To create a new point pattern from raw data such as \eqn{x,y}
  coordinates, it is normally easier to use the creator function
  \code{\link{ppp}}.

  The function \code{as.ppp} is generic, with methods for the
  classes \code{"ppp"}, \code{"psp"}, \code{"quad"}, \code{"matrix"},
  \code{"data.frame"} and a default method.

  The dataset \code{X} may be:
  \itemize{
    \item
    an object of class \code{"ppp"}  
    \item
    an object of class \code{"psp"}
    \item
    a point pattern object created by the \pkg{spatial} library
    \item
    an object of class \code{"quad"} representing a quadrature scheme
    (see \code{\link{quad.object}})
    \item
    a matrix or data frame with at least two columns
    \item
    a structure with entries \code{x}, \code{y} which are numeric vectors
    of equal length
    \item
    a numeric vector of length 2, interpreted as the coordinates
    of a single point.
  }
  In the last three cases, we need the second argument \code{W}
  which is converted to a window object
  by the function \code{\link{as.owin}}.
  In the first four cases, \code{W} will be ignored.

  If \code{X} is a line segment pattern (an object of class \code{psp})
  the point pattern returned consists of the endpoints of the segments.
  If \code{X} is marked then the point pattern returned will also be
  marked, the mark associated with a point being the mark of the segment
  of which that point was an endpoint.

  If \code{X} is a matrix or data frame, the first and second columns will
  be interpreted as the \eqn{x} and \eqn{y} coordinates respectively.
  Any additional columns will be interpreted as marks.
  
  The argument \code{fatal} indicates what to do when 
  \code{W} is missing and \code{X} contains no
  information about the window. If \code{fatal=TRUE}, a fatal error
  will be generated; if \code{fatal=FALSE}, the
  value \code{NULL} is returned.

  In the \pkg{spatial} library, a point pattern is represented
  in either of the following formats:
  \itemize{
    \item
    (in \pkg{spatial} versions 1 to 6)
    a structure with entries \code{x}, \code{y}
    \code{xl}, \code{xu}, \code{yl}, \code{yu}
    \item
    (in \pkg{spatial} version 7)
    a structure with entries
    \code{x}, \code{y} and \code{area},
    where \code{area} is a structure with entries
    \code{xl}, \code{xu}, \code{yl}, \code{yu}
  }
  where \code{x} and \code{y} are vectors of equal length
  giving the point coordinates, and \code{xl}, \code{xu}, \code{yl},
  \code{yu} are numbers giving the dimensions of a rectangular window.

  Point pattern datasets can also be created by the function
  \code{\link{ppp}}.
}
\seealso{
  \code{\link{ppp}}, \code{\link{ppp.object}}, \code{\link{as.owin}},
  \code{\link{owin.object}}
}
\examples{
 xy <- matrix(runif(40), ncol=2)
 pp <- as.ppp(xy, c(0,1,0,1))

 # Venables-Ripley format
 # check for 'spatial' package
 spatialpath <- system.file(package="spatial")
 if(nchar(spatialpath) > 0) {
   require(spatial)
   towns <- ppinit("towns.dat")
   pp <- as.ppp(towns) # converted to our format
   detach(package:spatial)
 }

 xyzt <- matrix(runif(40), ncol=4)
 Z <- as.ppp(xyzt, square(1))
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{fardist}
\alias{fardist}
\alias{fardist.ppp}
\alias{fardist.owin}
\title{
  Farthest Distance to Boundary of Window
}
\description{
  Computes the farthest distance from each pixel, or each data point,
  to the boundary of the window.
}
\usage{
  fardist(X, \dots)

  \method{fardist}{owin}(X, \dots, squared=FALSE)

  \method{fardist}{ppp}(X, \dots, squared=FALSE)
}
\arguments{
  \item{X}{
    A spatial object such as a window or point pattern.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine
    the pixel resolution, if required.
  }
  \item{squared}{
    Logical. If \code{TRUE}, the squared distances will be
    returned.
  }
}
\details{
  The function \code{fardist} is generic, with methods for
  the classes \code{owin} and \code{ppp}.

  For a window \code{W}, the command \code{fardist(W)}
  returns a pixel image in which the value at each pixel
  is the \emph{largest} distance from that pixel to the boundary of
  \code{W}.

  For a point pattern \code{X}, with window \code{W}, the
  command \code{fardist(X)} returns a numeric vector
  with one entry for each point of \code{X}, giving the
  largest distance from that data point to the boundary of \code{W}.
}
\value{
  For \code{fardist.owin}, a pixel image (object of class \code{"im"}).

  For \code{fardist.ppp}, a numeric vector.
}
\examples{
  fardist(cells)

  plot(FR <- fardist(letterR))
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{math}

\name{methods.unitname}
\Rdversion{1.1}
\alias{methods.unitname} %DoNotExport
\alias{print.unitname}
\alias{summary.unitname}
\alias{rescale.unitname}
\alias{compatible.unitname}
\alias{harmonise.unitname}
\alias{harmonize.unitname}
\title{
  Methods for Units
}
\description{
  Methods for class \code{"unitname"}.
}
\usage{
  \method{print}{unitname}(x, ...)
  \method{summary}{unitname}(object, ...)
  \method{rescale}{unitname}(X, s, unitname)
  \method{compatible}{unitname}(A,B, ..., coerce=TRUE)
  \method{harmonise}{unitname}(..., coerce=TRUE, single=FALSE)
  \method{harmonize}{unitname}(..., coerce=TRUE, single=FALSE)
}
\arguments{
  \item{x,X,A,B,object}{
    Objects of class \code{"unitname"} representing
    units of length.
  }
  \item{\dots}{
    Other arguments.
    For \code{print.unitname} these arguments are passed to
    \code{\link[base]{print.default}}.
    For \code{summary.unitname} they are ignored.
    For \code{compatible.unitname} and \code{harmonise.unitname} these arguments
    are other objects of class \code{"unitname"}.
  }
  \item{s}{
    Conversion factor: the new units are \code{s} times the old units.
  }
  \item{unitname}{
    Optional new name for the unit. If present, this overrides the
    rescaling operation and simply substitutes the new name for the old
    one.
  }
  \item{coerce}{
    Logical. If \code{TRUE}, a null unit of length is compatible with
    any non-null unit.
  }
  \item{single}{
    Logical value indicating whether to return a single unitname,
    or a list of unitnames.
  }
}
\details{
  These are methods for the generic functions
  \code{\link{print}}, \code{\link{summary}}, \code{\link{rescale}}
  and \code{\link{compatible}}
  for the class \code{"unitname"}.

  An object of class \code{"unitname"} represents a unit of length.

  The \code{print} method prints a description of the unit of length,
  and the \code{summary} method gives a more detailed description.

  The \code{rescale} method changes the unit of length by rescaling it.
  
  The \code{compatible} method tests whether two or more units of length
  are compatible.

  The \code{harmonise} method returns the common unit of length
  if there is one. For consistency with other methods for
  \code{\link{harmonise}}, the result is a list of unitname objects,
  with one entry for each argument in \code{\dots}. All of these
  entries are identical. This can be overridden by setting
  \code{single=TRUE} when the result will be a single unitname object.
}
\value{
  For \code{print.unitname} the value is \code{NULL}.
  For \code{summary.unitname} the value is an object of class
  \code{summary.unitname} (with its own print method).
  For \code{rescale.unitname} the value is another object of class \code{"unitname"}.
  For \code{compatible.unitname} the result is logical.
  For \code{harmonise.unitname} the result is a list of
  identical unitnames if \code{single=FALSE} (the default),
  or a single unitname if \code{single=TRUE}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{box3}},
  \code{\link{print}},
  \code{\link{unitname}}
}
\keyword{spatial}
\keyword{methods}

\name{Extract.ppx}
\alias{[.ppx}
\title{Extract Subset of Multidimensional Point Pattern}
\description{
  Extract a subset of a multidimensional point pattern.
}
\usage{
  \method{[}{ppx}(x, i, drop=FALSE, ...)
}
\arguments{
  \item{x}{
    A multidimensional point pattern (object of class \code{"ppx"}).
  }
  \item{i}{
   Subset index. A valid subset index in the usual \R sense,
   indicating which points should be retained;
   or a spatial domain of class \code{"boxx"} or \code{"box3"}.
  }
  \item{drop}{
    Logical value indicating whether to remove unused levels
    of the marks, if the marks are a factor.
  }
  \item{\dots}{
    Ignored. 
  }
}
\value{
  A multidimensional point pattern (of class \code{"ppx"}).
}
\details{
  This function extracts a designated subset of a multidimensional
  point pattern.

  The function \code{[.ppx} is a method for \code{\link{[}} for the
  class \code{"ppx"}. It extracts a designated subset of a point pattern.
  The argument \code{i} may be either
  \itemize{
    \item a subset index in the usual \R sense:
    either a numeric vector
    of positive indices (identifying the points to be retained),
    a numeric vector of negative indices (identifying the points
    to be deleted) or a logical vector of length equal to the number of
    points in the point pattern \code{x}. In the latter case, 
    the points \code{(x$x[i], x$y[i])} for which 
    \code{subset[i]=TRUE} will be retained, and the others
    will be deleted.
  \item
    a spatial domain of class \code{"boxx"} or \code{"box3"}.
    Points falling inside this region will be retained.
  }
  The argument \code{drop} determines whether to remove
  unused levels of a factor, if the point pattern is multitype
  (i.e. the marks are a factor) or if the marks are a data frame or hyperframe
  in which some of the columns are factors.

  Use the function \code{\link{unmark}} to remove marks from a
  marked point pattern.
}
\seealso{
  \code{\link{ppx}}
}
\examples{
   df <- data.frame(x=runif(4),y=runif(4),z=runif(4))
   X <- ppx(data=df, coord.type=c("s","s","t"))
   X[-2]
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{erosionAny}
\alias{erosionAny}
\alias{\%(-)\%}  %DoNotExport 
%NAMESPACE export("%(-)%")
\title{Morphological Erosion of Windows}
\description{
  Compute the morphological erosion of one spatial window by another.
}
\usage{
erosionAny(A, B)

A \%(-)\% B
}
\arguments{
  \item{A,B}{
    Windows (objects of class \code{"owin"}).
  }
}
\value{
  Another window (object of class \code{"owin"}).
}
\details{
  The operator \code{A \%(-)\% B} and function \code{erosionAny(A,B)}
  are synonymous: they both compute the
  morphological erosion of the window \code{A} by the window \code{B}.

  The morphological erosion
  \eqn{A \ominus B}{A \%(-)\% B}
  of region \eqn{A} by region \eqn{B}
  is the spatial region consisting of all vectors \eqn{z}
  such that, when \eqn{B} is shifted by the vector \eqn{z}, the result
  is a subset of \eqn{A}.

  Equivalently
  \deqn{
    A \ominus B = ((A^c \oplus (-B))^c
  }{
    (A^c \%+\% (-B))^c
  }
  where \eqn{\oplus}{\%+\%} is the Minkowski sum,
  \eqn{A^c} denotes the set complement, and \eqn{(-B)} is
  the reflection of \eqn{B} through the origin, consisting of all
  vectors \eqn{-b} where \eqn{b} is a point in \eqn{B}.

  If \code{B} is a disc of radius \code{r}, then
  \code{erosionAny(A, B)} is equivalent to \code{erosion(A, r)}.
  See \code{\link{erosion}}.
  
  The algorithm currently computes the result as a polygonal
  window using the \pkg{polyclip} library. 
  It will be quite slow if applied to binary mask windows.
}
\seealso{
  \code{\link{erosion}},
  \code{\link{MinkowskiSum}}
}
\examples{
  B <- square(c(-0.1, 0.1))
  RminusB <- letterR \%(-)\% B
  FR <- grow.rectangle(Frame(letterR), 0.3)
  plot(FR, main="", type="n")
  plot(letterR, add=TRUE, lwd=2, hatch=TRUE, box=FALSE)
  plot(RminusB, add=TRUE, col="blue", box=FALSE)
  plot(shift(B, vec=c(3.49, 2.98)),
       add=TRUE, border="red", lwd=2)
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{math}

\name{headtail}
\alias{head.ppp}
\alias{head.ppx}
\alias{head.psp}
\alias{head.tess}
\alias{tail.ppp}
\alias{tail.ppx}
\alias{tail.psp}
\alias{tail.tess}
\title{
  First or Last Part of a Spatial Pattern
}
\description{
  Returns the first few elements (\code{head}) or the last few
  elements (\code{tail}) of a spatial pattern.
}
\usage{
  \method{head}{ppp}(x, n = 6L, \dots)

  \method{head}{ppx}(x, n = 6L, \dots)

  \method{head}{psp}(x, n = 6L, \dots)

  \method{head}{tess}(x, n = 6L, \dots)

  \method{tail}{ppp}(x, n = 6L, \dots)

  \method{tail}{ppx}(x, n = 6L, \dots)

  \method{tail}{psp}(x, n = 6L, \dots)

  \method{tail}{tess}(x, n = 6L, \dots)
}
\arguments{
  \item{x}{
    A spatial pattern of geometrical figures,
    such as a spatial pattern of points
    (an object of class \code{"ppp"}, \code{"pp3"}, \code{"ppx"} or
    \code{"lpp"}) or a spatial pattern of line segments
    (an object of class \code{"psp"}) or a tessellation
    (object of class \code{"tess"}).
  }
  \item{n}{
    Integer. The number of elements of the pattern that should be extracted.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  These are methods for the generic functions \code{\link[utils]{head}}
  and \code{\link[utils]{tail}}. They extract the first or last
  \code{n} elements from \code{x} and return them as an object of the
  same kind as \code{x}.

  To inspect the spatial coordinates themselves, use
  \code{\link[utils]{View}(x)}
  or \code{head(as.data.frame(x))}.
}
\value{
  An object of the same class as \code{x}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link[utils]{View}}, \code{\link[utils]{edit}}.
  
  Conversion to data frame:
  \code{\link{as.data.frame.ppp}},
  \code{\link{as.data.frame.ppx}},
  \code{\link{as.data.frame.psp}}
}
\examples{
  head(cells)
  tail(as.psp(spiders), 10)
  head(dirichlet(cells), 4)
}
\keyword{spatial}
\keyword{manip}

\name{nvertices}
\alias{nvertices}
\alias{nvertices.owin}
\alias{nvertices.default}
\title{
  Count Number of Vertices
}
\description{
  Count the number of vertices in an object for which vertices are well-defined.
}
\usage{
  nvertices(x, \dots)

  \method{nvertices}{owin}(x, \dots)

  \method{nvertices}{default}(x, \dots)
}
\arguments{
  \item{x}{
    A window (object of class \code{"owin"}),
    or some other object which has vertices.
  }
  \item{\dots}{
    Currently ignored.
  }
}
\details{
  This function counts the number of vertices of \code{x}
  as they would be returned by \code{\link{vertices}(x)}.
  It is more efficient than executing \code{npoints(vertices(x))}.
}
\value{
  A single integer.
}
\author{
  \spatstatAuthors
  and Suman Rakshit.
}
\seealso{
  \code{\link{vertices}}
}
\examples{
  nvertices(square(2))
  nvertices(letterR)
}
\keyword{spatial}
\keyword{manip}

\name{distmap}
\alias{distmap}
\title{
  Distance Map 
}
\description{
  Compute the distance map of an object, and return it as a pixel image.
  Generic.
}
\usage{
  distmap(X, \dots)
}
\arguments{
  \item{X}{Any suitable dataset representing a two-dimensional
    object, such as a point pattern (object of class \code{"ppp"}),
    a window (object of class \code{"owin"}) or a
    line segment pattern (object of class \code{"psp"}).
  }
  \item{\dots}{Arguments passed to \code{\link{as.mask}}
    to control pixel resolution.
  }
}
\value{
  A pixel image (object of class \code{"im"}) whose grey scale values
  are the values of the distance map.
}
\details{
  The \dQuote{distance map} of a set of points \eqn{A} is the function
  \eqn{f} whose value \code{f(x)} is defined for any two-dimensional
  location \eqn{x} as the shortest distance from \eqn{x} to \eqn{A}.

  This function computes the distance map of the set \code{X}
  and returns the distance map as a pixel image.
  
  This is generic. Methods are provided for
  point patterns (\code{\link{distmap.ppp}}),
  line segment patterns  (\code{\link{distmap.psp}})
  and windows (\code{\link{distmap.owin}}).
}
\seealso{
  \code{\link{distmap.ppp}},
  \code{\link{distmap.psp}},
  \code{\link{distmap.owin}},
  \code{\link{distfun}}
}
\examples{
  data(cells)
  U <- distmap(cells)
  data(letterR)
  V <- distmap(letterR)
  \dontrun{
  plot(U)
  plot(V)
  }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{lut}
\alias{lut}
\title{Lookup Tables}
\description{
  Create a lookup table.
}
\usage{
lut(outputs, ..., range=NULL, breaks=NULL, inputs=NULL, gamma=1)
}
\arguments{
  \item{outputs}{Vector of output values}
  \item{\dots}{Ignored.}
  \item{range}{
    Interval of numbers to be mapped.
    A numeric vector of length 2, specifying the ends of the range of values
    to be mapped.
    Incompatible with \code{breaks} or \code{inputs}.
  }
  \item{inputs}{
    Input values to which the output values are associated.
    A factor or vector of the same length as \code{outputs}.
    Incompatible with \code{breaks} or \code{range}.
  }
  \item{breaks}{
    Breakpoints for the lookup table.
    A numeric vector of length equal to \code{length(outputs)+1}.
    Incompatible with \code{range} or \code{inputs}.
  }
  \item{gamma}{
    Exponent for gamma correction, when \code{range} is given.
    A single positive number.
    See Details.
  }
}
\details{
  A lookup table is a function, mapping input values to output values.

  The command \code{lut} creates an object representing
  a lookup table, which can then be used to control various behaviour
  in the \pkg{spatstat} package. It can also be used to compute the
  output value assigned to any input value. 

  The argument \code{outputs} specifies the output values to which
  input data values will be mapped. It should be a vector of
  any atomic type (e.g. numeric, logical, character, complex) or factor
  values. 

  Exactly one of the arguments \code{range}, \code{inputs} or \code{breaks}
  must be specified by name.
  \itemize{
    \item 
    If \code{inputs} is given, then it should be a vector or factor,
    of the same length as \code{outputs}. The entries of \code{inputs} can be
    any atomic type (e.g. numeric, logical, character, complex) or factor
    values. The resulting lookup table associates the value \code{inputs[i]}
    with the value \code{outputs[i]}.
    The argument \code{outputs} should have the same length as \code{inputs}.
    \item
    If \code{range} is given, then it determines the interval of the real
    number line that will be mapped. It should be a numeric vector of
    length 2. 
    The interval will be divided evenly into bands, each of which is
    mapped to an entry of \code{outputs}. (If \code{gamma} is given,
    then the bands are equally spaced on a scale where the original values
    are raised to the power \code{gamma}.)
    \item
    If \code{breaks} is given, then it determines intervals
    of the real number line
    which are mapped to each output value. It should be a numeric vector,
    of length at least 2, with entries that are in increasing order.
    Infinite values are allowed. Any number in the range
    between \code{breaks[i]} and \code{breaks[i+1]} will be mapped to the
    value \code{outputs[i]}. 
    The argument \code{outputs} should have length equal to
    \code{length(breaks) - 1}.
  }
  It is also permissible for \code{outputs} to be a single value,
  representing a trivial lookup table in which all data values are mapped to
  the same output value.

  The result is an object of class \code{"lut"}. 
  There is a \code{print} method for this class.
  Some plot commands in the \pkg{spatstat} package accept an object
  of this class as a specification of a lookup table.

  The result is also a function \code{f} which can be used to compute
  the output value assigned to any input data value. 
  That is, \code{f(x)} returns the output value assigned
  to \code{x}. This also works for vectors of input data values.
}
\value{
  A function, which is also an object of class \code{"lut"}.
}
\seealso{
  \code{\link{colourmap}}.
}
\examples{
  # lookup table for real numbers, using breakpoints
  cr <- lut(factor(c("low", "medium", "high")), breaks=c(0,5,10,15))
  cr
  cr(3.2)
  cr(c(3,5,7))
  # lookup table for discrete set of values
  ct <- lut(c(0,1), inputs=c(FALSE, TRUE))
  ct(TRUE)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}

\name{tileindex}
\alias{tileindex}
\title{
  Determine Which Tile Contains Each Given Point
}
\description{
  Given a tessellation and a list of spatial points,
  determine which tile of the tessellation contains each of the
  given points.
}
\usage{
tileindex(x, y, Z)
}
\arguments{
  \item{x,y}{
    Spatial coordinates.
    Numeric vectors of equal length.
    (Alternatively \code{y} may be missing
    and \code{x} may be an object containing
    spatial coordinates).
  }
  \item{Z}{
    A tessellation (object of class \code{"tess"}).
  }
}
\details{
  This function determines which tile of the tessellation \code{Z}
  contains each of the spatial points
  with coordinates \code{(x[i],y[i])}.

  The result is a factor, of the same length as \code{x} and \code{y},
  indicating which tile contains each point. The levels of the factor
  are the names of the tiles of \code{Z}.
  Values are \code{NA} if the corresponding point lies outside the tessellation.
}
\value{
  A factor, of the same length as \code{x} and \code{y},
  whose levels are the names of the tiles of \code{Z}.
}
\author{
  \spatstatAuthors
}
\seealso{
  \code{\link{cut.ppp}} and \code{\link{split.ppp}} to
  divide up the points of a point pattern according to
  a tessellation.

  \code{\link{as.function.tess}} to create a function whose
  value is the tile index.
}
\examples{
  X <- runifpoint(7)
  V <- dirichlet(X)
  tileindex(0.1, 0.4, V)
  tileindex(list(x=0.1, y=0.4), Z=V)
  tileindex(X, Z=V)
}
\keyword{spatial}
\keyword{manip}

\name{scaletointerval}  
\alias{scaletointerval}
\alias{scaletointerval.default}
\alias{scaletointerval.im}
\title{Rescale Data to Lie Between Specified Limits}
\description{
  Rescales a dataset so that the values range exactly between the
  specified limits.
}
\usage{
  scaletointerval(x, from=0, to=1, xrange=range(x))
  \method{scaletointerval}{default}(x, from=0, to=1, xrange=range(x))
  \method{scaletointerval}{im}(x, from=0, to=1, xrange=range(x))
}
\arguments{
  \item{x}{Data to be rescaled.}
  \item{from,to}{Lower and upper endpoints of the interval
    to which the values of \code{x} should be rescaled.
  }
  \item{xrange}{
    Optional range of values of \code{x} that should be mapped
    to the new interval.
  }
}
\details{
  These functions rescale a dataset \code{x}
  so that its values range exactly between the limits
  \code{from} and \code{to}.

  The method for pixel images (objects of class \code{"im"})
  applies this scaling to the pixel values of \code{x}.
  
  Rescaling cannot be performed if the values in \code{x} are not
  interpretable as numeric, or if the values in \code{x} are all equal.
}
\value{
  An object of the same type as \code{x}.
}
\seealso{
  \code{\link{scale}}
}
\examples{
  X <- as.im(function(x,y) {x+y+3}, unit.square())
  summary(X)
  Y <- scaletointerval(X)
  summary(Y)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}
\keyword{univar}

\name{area.owin}
\alias{area}
\alias{area.owin}
\alias{area.default}
\alias{volume.owin}
\title{Area of a Window}
\description{
  Computes the area of a window 
}
\usage{
 area(w)

 \method{area}{owin}(w)

 \method{area}{default}(w)

 \method{volume}{owin}(x)
}
\arguments{
  \item{w}{A window, whose area will be computed.
    This should be an object of class \code{\link{owin}},
    or can be given in any format acceptable to \code{\link{as.owin}()}.
  }
  \item{x}{Object of class \code{\link{owin}}}
}
\value{
  A numerical value giving the area of the window. 
}
\details{
  If the window \code{w} is of type \code{"rectangle"} or \code{"polygonal"},
  the area of this rectangular window is computed by analytic geometry.
  If \code{w} is of type \code{"mask"} 
  the area of the discrete raster approximation of the window is
  computed by summing the binary image values and adjusting for
  pixel size.

  The function \code{volume.owin} is identical to \code{area.owin}
  except for the argument name. It is a method for the generic function
  \code{volume}.
}
\seealso{
  \code{\link{perimeter}},
  \code{\link{diameter.owin}},
  \code{\link{owin.object}},
  \code{\link{as.owin}}
}
\examples{
  w <- unit.square()
  area(w)
       # returns 1.00000

  k <- 6
  theta <- 2 * pi * (0:(k-1))/k
  co <- cos(theta)
  si <- sin(theta)
  mas <- owin(c(-1,1), c(-1,1), poly=list(x=co, y=si))
  area(mas)
      # returns approx area of k-gon
  
  mas <- as.mask(square(2), eps=0.01)
  X <- raster.x(mas)
  Y <- raster.y(mas)
  mas$m <- ((X - 1)^2 + (Y - 1)^2 <= 1)
  area(mas)
       # returns 3.14 approx     

}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{methods.distfun}
\alias{methods.distfun} %DoNotExport
\alias{shift.distfun}
\alias{rotate.distfun}
\alias{scalardilate.distfun}
\alias{affine.distfun}
\alias{flipxy.distfun}
\alias{reflect.distfun}
\alias{rescale.distfun}
\Rdversion{1.1}
\title{
  Geometrical Operations for Distance Functions
}
\description{
  Methods for objects of the class \code{"distfun"}.
}
\usage{
\method{shift}{distfun}(X, \dots)

\method{rotate}{distfun}(X, \dots)

\method{scalardilate}{distfun}(X, \dots)

\method{affine}{distfun}(X, \dots)

\method{flipxy}{distfun}(X)

\method{reflect}{distfun}(X)

\method{rescale}{distfun}(X, s, unitname)
}
\arguments{
  \item{X}{
    Object of class \code{"distfun"} representing the distance function
    of a spatial object.
  }
  \item{\dots}{
    Arguments passed to the next method for the geometrical operation.
    See Details.
  }
  \item{s, unitname}{
    Arguments passed to the next method for \code{\link{rescale}}.
  }
}
\details{
  These are methods for the generic functions
  \code{\link{shift}},
  \code{\link{rotate}},
  \code{\link{scalardilate}},
  \code{\link{affine}},
  \code{\link{flipxy}} and \code{\link{reflect}}
  which perform geometrical operations on spatial objects,
  and for the generic \code{\link{rescale}} which changes the unit of length.

  The argument \code{X} should be an object of class \code{"distfun"}
  representing the distance function of a spatial object \code{Y}.
  Objects of class \code{"distfun"} are created by \code{\link{distfun}}.

  The methods apply the specified geometrical transformation to
  the original object \code{Y}, producing a new object \code{Z}
  of the same type as \code{Y}. They then create a new \code{distfun}
  object representing the distance function of \code{Z}.
}
\value{
  Another object of class \code{"distfun"}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{distfun}},
  \code{\link{methods.funxy}}.
}
\examples{
   (f <- distfun(letterR))
   plot(f)
   flipxy(f)
   shift(f, origin="midpoint")
   plot(rotate(f, angle=pi/2))

   (g <- distfun(lansing))
   rescale(g)
}
\keyword{spatial}
\keyword{methods}

\name{as.function.tess}
\alias{as.function.tess}
\title{
  Convert a Tessellation to a Function
}
\description{
  Convert a tessellation into a function of the \eqn{x} and \eqn{y} coordinates.
  The default function values are factor levels specifying which tile of the
  tessellation contains the point \eqn{(x,y)}.
}
\usage{
  \method{as.function}{tess}(x,\dots,values=NULL)
}
\arguments{
  \item{x}{
    A tessellation (object of class \code{"tess"}).
  }
  \item{values}{
    Optional. A vector giving the values of the function for each tile
    of \code{x}.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  This command converts a tessellation
  (object of class \code{"tess"}) to a \code{function(x,y)}
  where the arguments \code{x} and \code{y} are (vectors of) spatial
  coordinates. The corresponding function values are factor levels
  identifying which tile of the tessellation contains each point.
  Values are \code{NA} if the corresponding point lies outside the
  tessellation.

  If the argument \code{values} is given, then it determines the value
  of the function in each tile of \code{x}.
}
\value{
  A function in the \R language, also belonging to the
  class \code{"funxy"}.
}
\author{
  \spatstatAuthors
}
\seealso{
  \code{\link{tileindex}} for the low-level calculation of tile index.

  \code{\link{cut.ppp}} and \code{\link{split.ppp}} to
  divide up the points of a point pattern according to
  a tessellation.
}
\examples{
  X <- runifpoint(7)
  V <- dirichlet(X)
  f <- as.function(V)
  f(0.1, 0.4)
  plot(f)
}
\keyword{spatial}
\keyword{manip}

\name{funxy}
\Rdversion{1.1}
\alias{funxy}
\title{
  Spatial Function Class
}
\description{
  A simple class of functions of spatial location
}
\usage{
  funxy(f, W)
}
\arguments{
  \item{f}{
    A \code{function} in the \R language
    with arguments \code{x,y} (at least)
  }
  \item{W}{
    Window (object of class \code{"owin"}) inside which the
    function is well-defined.
  }
}
\details{
  This creates an object of class \code{"funxy"}.
  This is a simple mechanism for handling a function
  of spatial location \eqn{f(x,y)} to make it easier to display
  and manipulate.

  \code{f} should be a \code{function} in the \R language.
  The first two arguments of \code{f} must be named \code{x} and \code{y}
  respectively.

  \code{W} should be a window (object of class \code{"owin"}) inside which the
  function \code{f} is well-defined.

  The function \code{f} should be vectorised: that is,
  if \code{x} and \code{y} are numeric vectors of the same length
  \code{n}, then \code{v <- f(x,y)} should be a vector of length
  \code{n}.

  The resulting function \code{g <- funxy(f, W)} has the same formal
  arguments as \code{f} and can be called in the same way,
  \code{v <- g(x,y)} where \code{x} and \code{y} are numeric vectors.
  However it can also be called as \code{v <- g(X)}, where \code{X} is a point pattern
  (object of class \code{"ppp"} or \code{"lpp"})
  or a quadrature scheme (class \code{"quad"});
  the function will be evaluated at the points of \code{X}.
}
\value{
  A \code{function} with the same arguments as \code{f},
  which also belongs to the class \code{"funxy"}.
  This class has methods for
  \code{print}, \code{plot}, \code{contour} and \code{persp}.
}
\seealso{
  \code{\link{plot.funxy}},
  \code{\link{summary.funxy}}
}
\examples{
   f <- function(x,y) { x^2 + y^2 - 1} 
   g <- funxy(f, square(2))
   g
   ## evaluate function at any x, y coordinates
   g(0.2, 0.3)
   ## evaluate function at the points of a point pattern
   g(cells[1:4])
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{clarkevans.test}
\alias{clarkevans.test}
\title{Clark and Evans Test}
\description{
  Performs the Clark-Evans test of aggregation
  for a spatial point pattern.
}
\usage{
clarkevans.test(X, ...,
               correction="none",
               clipregion=NULL,
               alternative=c("two.sided", "less", "greater",
                             "clustered", "regular"),
               nsim=999)
}
\arguments{
  \item{X}{
    A spatial point pattern (object of class \code{"ppp"}).
  }
  \item{\dots}{Ignored.}
  \item{correction}{
    Character string.
    The type of edge correction to be applied.
    See \code{\link{clarkevans}}
  }
  \item{clipregion}{
    Clipping region for the guard area correction.
    A window (object of class \code{"owin"}).
    See \code{\link{clarkevans}}
  }
  \item{alternative}{
    String indicating the type of alternative for the
    hypothesis test. Partially matched.
  }
  \item{nsim}{
    Number of Monte Carlo simulations to perform, if a Monte Carlo
    p-value is required.
  }
}
\details{
  This command uses the Clark and Evans (1954) aggregation index \eqn{R}
  as the basis for a crude test of clustering or ordering of a point pattern.
  
  The Clark-Evans index is computed by the function
  \code{\link{clarkevans}}. See the help for \code{\link{clarkevans}}
  for information about the Clark-Evans index \eqn{R} and about
  the arguments \code{correction} and \code{clipregion}.

  This command performs a hypothesis test of clustering or ordering of
  the point pattern \code{X}. The null hypothesis is Complete
  Spatial Randomness, i.e.\ a uniform Poisson process. The alternative
  hypothesis is specified by the argument \code{alternative}:
  \itemize{
    \item \code{alternative="less"} or \code{alternative="clustered"}:
    the alternative hypothesis
    is that \eqn{R < 1} corresponding to a clustered point pattern;
    \item \code{alternative="greater"} or \code{alternative="regular"}:
    the alternative hypothesis
    is that \eqn{R > 1} corresponding to a regular or ordered point pattern;
    \item \code{alternative="two.sided"}:
    the alternative hypothesis is that \eqn{R \neq 1}{R != 1}
    corresponding to a clustered or regular pattern.
  }
  
  The Clark-Evans index \eqn{R} is computed for the data
  as described in \code{\link{clarkevans}}.

  If \code{correction="none"} and \code{nsim} is missing,
  the \eqn{p}-value for the test is computed by standardising
  \eqn{R} as proposed by Clark and Evans (1954) and referring the
  statistic to the standard Normal distribution.

  Otherwise, the \eqn{p}-value for the test is computed
  by Monte Carlo simulation of \code{nsim} realisations of
  Complete Spatial Randomness conditional on the
  observed number of points.
}
\value{
  An object of class \code{"htest"} representing the result of the test.
}
\references{
  Clark, P.J. and Evans, F.C. (1954)
  Distance to nearest neighbour as a measure of spatial
  relationships in populations. \emph{Ecology} \bold{35},
  445--453.
  
  Donnelly, K. (1978) Simulations to determine the variance
  and edge-effect of total nearest neighbour distance.
  In \emph{Simulation methods in archaeology},
  Cambridge University Press, pp 91--95.
}
\author{
  \adrian
}
\seealso{
  \code{\link{clarkevans}},
  \code{\link{hopskel.test}}
}
\examples{
  # Redwood data - clustered
  clarkevans.test(redwood)
  clarkevans.test(redwood, alternative="clustered")
  clarkevans.test(redwood, correction="cdf", nsim=39)
}
\keyword{spatial}
\keyword{nonparametric}
\keyword{htest}

\name{dirichletVertices}
\alias{dirichletVertices}
\alias{dirichletEdges}
\title{
   Vertices and Edges of Dirichlet Tessellation
}
\description{
  Computes the Dirichlet-Voronoi tessellation of a point pattern
  and extracts the vertices or edges of the tiles.
}
\usage{
dirichletVertices(X)

dirichletEdges(X)
}
\arguments{
  \item{X}{
    Point pattern (object of class \code{"ppp"}).
  }
}
\details{
  These function compute the Dirichlet-Voronoi tessellation of \code{X}
  (see \code{\link{dirichlet}})
  and extract the vertices or edges of the tiles of the tessellation.

  The Dirichlet vertices are the spatial locations which are locally
  farthest away from \code{X}, that is, where the distance function
  of \code{X} reaches a local maximum.

  The Dirichlet edges are the dividing lines equally distant between
  a pair of points of \code{X}.

  The Dirichlet tessellation of \code{X} is computed
  using \code{\link{dirichlet}}. The vertices or edges of all
  tiles of the tessellation are extracted.

  For \code{dirichletVertices}, any vertex which
  lies on the boundary of the window of \code{X} is deleted.
  The remaining vertices are returned, as a point pattern,
  without duplicated entries.
}
\value{
  \code{dirichletVertices}
  returns a point pattern (object of class \code{"ppp"}) in the same window
  as \code{X}.

  \code{dirichletEdges} returns a line segment pattern (object of
  class \code{"psp"}).
}
\seealso{
 \code{\link{dirichlet}},
 \code{\link{dirichletAreas}}
}
\examples{
  plot(dirichlet(cells))

  plot(dirichletVertices(cells), add=TRUE)

  ed <- dirichletEdges(cells)
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{math}
\keyword{manip}

\name{print.quad}
\alias{print.quad}
\title{Print a Quadrature Scheme}
\description{
  \code{print} method for a quadrature scheme.
}
\usage{
 \method{print}{quad}(x,\dots)
}
\arguments{
  \item{x}{
    A quadrature scheme object, typically obtained from
    \code{\link{quadscheme}}.
    An object of class \code{"quad"}.
  }
  \item{\dots}{Ignored.}
}
\value{
  none.
}
\details{
  This is the \code{print} method for the class \code{"quad"}.
  It prints simple information about the quadrature scheme.

  See \code{\link{quad.object}} for details of the class \code{"quad"}.
}
\seealso{
  \code{\link{quadscheme}},
  \code{\link{quad.object}},
  \code{\link{plot.quad}},
  \code{\link{summary.quad}}
}
\examples{
 data(cells)
 Q <- quadscheme(cells)
 Q
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{print}

\name{markstat}
\alias{markstat}
\title{Summarise Marks in Every Neighbourhood in a Point Pattern}
\description{
  Visit each point in a point pattern, find the neighbouring points,
  and summarise their marks
}
\usage{
   markstat(X, fun, N=NULL, R=NULL, \dots)
}
\arguments{
  \item{X}{
    A marked point pattern. 
    An object of class \code{"ppp"}.
  }
  \item{fun}{
    Function to be applied to the vector of marks.
  }
  \item{N}{
    Integer. If this argument is present,
    the neighbourhood of a point of \code{X} is defined to consist of the
    \code{N} points of \code{X} which are closest to it.
  }
  \item{R}{
    Nonnegative numeric value. If this argument is present,
    the neighbourhood of a point of \code{X} is defined to consist of
    all points of \code{X} which lie within a distance \code{R}
    of it.
  }
  \item{\dots}{
    extra arguments passed to the function \code{fun}.
    They must be given in the form \code{name=value}.
  }
}
\value{
  Similar to the result of \code{\link{apply}}.
  if each call to \code{fun} returns a single numeric value,
  the result is a vector of dimension \code{npoints(X)}, the number of points
  in \code{X}.
  If each call to \code{fun} returns a vector of the same length
  \code{m}, then the result is a matrix of dimensions \code{c(m,n)};
  note the transposition of the indices, as usual for the family of
  \code{apply} functions.
  If the calls to \code{fun} return vectors of different lengths,
  the result is a list of length \code{npoints(X)}. 
}
\details{
  This algorithm visits each point in the point pattern \code{X},
  determines which points of \code{X} are ``neighbours'' of the current
  point, extracts the marks of these neighbouring points,
  applies the function \code{fun} to the marks,
  and collects the value or values returned by \code{fun}.

  The definition of ``neighbours'' depends on the arguments
  \code{N} and \code{R}, exactly one of which must be given.

  If \code{N} is given, then the neighbours of the current
  point are the \code{N} points of \code{X} which are closest to
  the current point (including the current point itself).
  If \code{R} is given, then the neighbourhood of the current point
  consists of all points of \code{X} which lie closer than a distance \code{R}
  from the current point.

  Each point of \code{X} is visited; the neighbourhood
  of the current point is determined; the marks of these points
  are extracted as a vector \code{v}; then the function 
  \code{fun} is called as:
  
  \code{fun(v, \dots)}

  where \code{\dots} are the arguments passed from the call to
  \code{markstat}.

  The results of each call to \code{fun} are collected and returned
  according to the usual rules for \code{\link{apply}} and its
  relatives. See the section on \bold{Value}.

  This function is just a convenient wrapper for a common use of the
  function \code{\link{applynbd}}. For more complex tasks,
  use \code{\link{applynbd}}. 
  To simply tabulate the marks in every \code{R}-neighbourhood, use
  \code{\link{marktable}}.
}
\seealso{
  \code{\link{applynbd}},
  \code{\link{marktable}},
  \code{\link{ppp.object}},
  \code{\link{apply}}
}
\examples{
  trees <- longleaf
  \testonly{
     trees <- trees[seq(1, npoints(trees), by=6)]
  }

  # average diameter of 5 closest neighbours of each tree
  md <- markstat(trees, mean, N=5)

  # range of diameters of trees within 10 metre radius
  rd <- markstat(trees, range, R=10)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{programming}
 

\name{quantess}
\alias{quantess}
\alias{quantess.owin}
\alias{quantess.ppp}
\alias{quantess.im}
\title{Quantile Tessellation}
\description{
  Divide space into tiles which contain equal amounts of stuff.
}
\usage{
quantess(M, Z, n, \dots)

\method{quantess}{owin}(M, Z, n, \dots, type=2, origin=c(0,0), eps=NULL)

\method{quantess}{ppp}(M, Z, n, \dots, type=2, origin=c(0,0), eps=NULL)

\method{quantess}{im}(M, Z, n, \dots, type=2, origin=c(0,0))
}
\arguments{
  \item{M}{
    A spatial object (such as a window, point pattern
    or pixel image) determining the weight or amount of stuff
    at each location.
  }
  \item{Z}{
    A spatial covariate (a pixel image or a \code{function(x,y)})
    or one of the strings \code{"x"} or \code{"y"} indicating the
    Cartesian coordinates \eqn{x} or \eqn{y}, or one of the strings
    \code{"rad"} or \code{"ang"} indicating polar coordinates.
    The range of values of \code{Z} will be broken into \code{n}
    bands containing equal amounts of stuff.
  }
  \item{n}{
    Number of bands. A positive integer.
  }
  \item{type}{
    Integer specifying the rule for calculating quantiles.
    Passed to \code{\link[stats]{quantile.default}}.
  }
  \item{\dots}{
    Additional arguments passed to \code{\link{quadrats}} or
    \code{\link{tess}} defining another tessellation
    which should be intersected with the quantile tessellation.
  }
  \item{origin}{
    Location of the origin of polar coordinates,
    if \code{Z="rad"} or \code{Z="ang"}.
    Either a numeric vector of length 2 giving the location,
    or a point pattern containing only one point,
    or a list with two entries named \code{x} and \code{y},
    or one of the character strings
    \code{"centroid"}, \code{"midpoint"},
    \code{"left"}, \code{"right"}, \code{"top"}, \code{"bottom"},
    \code{"topleft"}, \code{"bottomleft"}, \code{"topright"} or
    \code{"bottomright"} (partially matched).
  }
  \item{eps}{
    Optional. The size of pixels in the approximation which is used
    to compute the quantiles. A positive numeric value, or vector of two
    positive numeric values.
  }
}
\details{
  A \emph{quantile tessellation} is a division of space into
  pieces which contain equal amounts of stuff.

  The function \code{quantess} 
  computes a quantile tessellation and 
  returns the tessellation itself.  
  The function \code{quantess} is generic, with methods for
  windows (class \code{"owin"}), point patterns (\code{"ppp"})
  and pixel images (\code{"im"}). 

  The first argument \code{M} (for mass) specifies the spatial
  distribution of stuff that is to be divided. If \code{M} is a window,
  the \emph{area} of the window is to be divided into \code{n} equal pieces.
  If \code{M} is a point pattern, the \emph{number of points} in the
  pattern is to be divided into \code{n} equal parts, as far as
  possible. If \code{M} is a pixel image, the pixel values are
  interpreted as weights, and the \emph{total weight} is to be divided
  into \code{n} equal parts.  

  The second argument
  \code{Z} is a spatial covariate. The range of values of \code{Z}
  will be divided into \code{n} bands, each containing
  the same total weight. That is, we determine the quantiles of \code{Z}
  with weights given by \code{M}.

  For convenience, additional arguments \code{\dots} can be given,
  to further subdivide the tiles of the tessellation. These arguments
  should be recognised by one of the functions
  \code{\link{quadrats}} or \code{\link{tess}}. The tessellation
  determined by these arguments is intersected with the quantile tessellation.

  The result of \code{quantess} is a tessellation of \code{as.owin(M)}
  determined by the quantiles of \code{Z}.
}
\value{
  A tessellation (object of class \code{"tess"}).
}
\author{
  Original idea by Ute Hahn.
  Implemented in \code{spatstat} by
  \spatstatAuthors.
}
\seealso{
  \code{\link{tess}}, \code{\link{quadrats}}, \code{\link{quantile}},
  \code{\link{tilenames}}
}
\examples{
  plot(quantess(letterR, "x", 5))

  plot(quantess(bronzefilter, "x", 6))
  points(unmark(bronzefilter))

  plot(quantess(letterR, "rad", 7, origin=c(2.8, 1.5)))
  plot(quantess(letterR, "ang", 7, origin=c(2.8, 1.5)))

  opa <- par(mar=c(0,0,2,5))
  A <- quantess(Window(bei), bei.extra$elev, 4)
  plot(A, ribargs=list(las=1))
  
  B <- quantess(bei, bei.extra$elev, 4)
  tilenames(B) <- paste(spatstat.utils::ordinal(1:4), "quartile")
  plot(B, ribargs=list(las=1))
  points(bei, pch=".", cex=2, col="white")
  par(opa)
}
\keyword{spatial}
\keyword{manip}

\name{bdist.points}
\alias{bdist.points}
\title{Distance to Boundary of Window}
\description{
  Computes the distances 
  from each point of a point pattern
  to the boundary of the window.
}
\usage{
 bdist.points(X)
}
\arguments{
  \item{X}{A point pattern (object of class \code{"ppp"}).}
}
\value{
  A numeric vector, giving the distances from each point of the pattern
  to the boundary of the window.
}
\details{
  This function computes, for each point \eqn{x_i}{x[i]}
  in the point pattern \code{X}, the shortest distance
  \eqn{d(x_i, W^c)}{dist(x[i], W')} from \eqn{x_i}{x[i]}
  to the boundary of the window \eqn{W} of observation.

  If the window \code{Window(X)} is of type \code{"rectangle"}
  or \code{"polygonal"}, then these distances are computed by
  analytic geometry and are exact, up to rounding errors.
  If the window is of type \code{"mask"} then the distances
  are computed using the real-valued distance transform,
  which is an approximation with maximum error equal to the width
  of one pixel in the mask.
}
\seealso{
  \code{\link{bdist.pixels}},
  \code{\link{bdist.tiles}},
  \code{\link{ppp.object}},
  \code{\link{erosion}}
}
\examples{
  data(cells)
  d <- bdist.points(cells)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{opening} 
\alias{opening}
\alias{opening.owin}
\alias{opening.psp}
\alias{opening.ppp}
\title{Morphological Opening}
\description{
  Perform morphological opening of a window, a line segment pattern
  or a point pattern.
}
\usage{
 opening(w, r, \dots)

 \method{opening}{owin}(w, r, \dots, polygonal=NULL)

 \method{opening}{ppp}(w, r, \dots)

 \method{opening}{psp}(w, r, \dots)
}
\arguments{
  \item{w}{
    A window (object of class \code{"owin"}
    or a line segment pattern (object of class \code{"psp"})
    or a point pattern (object of class \code{"ppp"}).
  }
  \item{r}{positive number: the radius of the opening.}
  \item{\dots}{
    extra arguments passed to \code{\link{as.mask}}
    controlling the pixel resolution, if a pixel approximation is used
  }
  \item{polygonal}{
    Logical flag indicating whether to compute a polygonal
    approximation to the erosion (\code{polygonal=TRUE}) or
    a pixel grid approximation (\code{polygonal=FALSE}).
  }
}
\value{
  If \code{r > 0}, an object of class \code{"owin"} representing the
  opened region. If \code{r=0}, the result is identical to \code{w}.
}
\details{
  The morphological opening (Serra, 1982)
  of a set \eqn{W} by a distance \eqn{r > 0}
  is the subset of points in \eqn{W} that can be 
  separated from the boundary of \eqn{W} by a circle of radius \eqn{r}.
  That is, a point \eqn{x} belongs to the opening 
  if it is possible to draw a circle of radius \eqn{r} (not necessarily
  centred on \eqn{x}) that has \eqn{x} on the inside
  and the boundary of \eqn{W} on the outside.
  The opened set is a subset of \code{W}.

  For a small radius \eqn{r}, the opening operation
  has the effect of smoothing out irregularities in the boundary of
  \eqn{W}. For larger radii, the opening operation removes promontories
  in the boundary. For very large radii, the opened set is empty.

  The algorithm applies \code{\link{erosion}} followed by
  \code{\link{dilation}}.
}
\seealso{
  \code{\link{closing}} for the opposite operation.

  \code{\link{dilation}}, \code{\link{erosion}} for the basic
  operations.  
  
  \code{\link{owin}},
  \code{\link{as.owin}} for information about windows.
}
\examples{
  v <- opening(letterR, 0.3)
  plot(letterR, type="n", main="opening")
  plot(v, add=TRUE, col="grey")
  plot(letterR, add=TRUE)
}
\references{
  Serra, J. (1982)
  Image analysis and mathematical morphology.
  Academic Press.
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{rescue.rectangle}
\alias{rescue.rectangle}
\title{Convert Window Back To Rectangle}
\description{
  Determines whether the given window is really a rectangle
  aligned with the coordinate axes, and if so, converts it to
  a rectangle object.
}
\usage{
 rescue.rectangle(W)
}
\arguments{
  \item{W}{A window (object of class \code{"owin"}).}
}
\value{
  Another object of class \code{"owin"} representing the
  same window.
}
\details{
  This function decides whether the window \code{W} is actually a rectangle
  aligned with the coordinate axes. This will be true if
  \code{W} is 
  \itemize{
    \item a rectangle (window object of type \code{"rectangle"});
    \item a polygon (window object of type \code{"polygonal"}
    with a single polygonal boundary) that is a rectangle aligned with
    the coordinate axes;
    \item a binary mask (window object of type \code{"mask"}) in which
    all the pixel entries are \code{TRUE}.
  }
  If so, the function returns
  this rectangle, a window object of type \code{"rectangle"}.
  If not, the function returns \code{W}.
}
\seealso{
  \code{\link{as.owin}},
  \code{\link{owin.object}}
}
\examples{
  w <- owin(poly=list(x=c(0,1,1,0),y=c(0,0,1,1)))
  rw <- rescue.rectangle(w)

  w <- as.mask(unit.square())
  rw <- rescue.rectangle(w)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{regularpolygon}
\alias{regularpolygon}
\alias{hexagon}
\title{
  Create A Regular Polygon
}
\description{
  Create a window object representing a regular (equal-sided) polygon.
}
\usage{
regularpolygon(n, edge = 1, centre = c(0, 0), \dots,
               align = c("bottom", "top", "left", "right", "no"))

hexagon(edge = 1, centre = c(0,0), \dots,
        align = c("bottom", "top", "left", "right", "no"))
}
\arguments{
  \item{n}{
    Number of edges in the polygon.
  }
  \item{edge}{
    Length of each edge in the polygon. A single positive number.
  }
  \item{centre}{
    Coordinates of the centre of the polygon.
    A numeric vector of length 2,
    or a \code{list(x,y)} giving the coordinates of exactly one point, or a
    point pattern (object of class \code{"ppp"}) containing exactly one point.
  }
  \item{align}{
    Character string specifying whether to align one of the edges with
    a vertical or horizontal boundary.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  The function \code{regularpolygon} creates a regular (equal-sided)
  polygon with \code{n} sides, centred at \code{centre},
  with sides of equal length \code{edge}.
  The function \code{hexagon} is the special case \code{n=6}.

  The orientation of the polygon is determined by the argument \code{align}.
  If \code{align="no"}, one vertex of the polygon is placed on the
  \eqn{x}-axis.
  Otherwise, an edge of the polygon is aligned with one side of
  the frame, specified by the value of \code{align}.
}
\value{
  A window (object of class \code{"owin"}).
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{disc}}, \code{\link{ellipse}},
  \code{\link{owin}}.

  \code{\link{hextess}} for hexagonal tessellations.
}
\examples{
  plot(hexagon())
  plot(regularpolygon(7))
  plot(regularpolygon(7, align="left"))
}
\keyword{spatial}
\keyword{datagen}

\name{edit.ppp}
\alias{edit.ppp}
\alias{edit.psp}
\alias{edit.im}
\title{
  Invoke Text Editor on Spatial Data
}
\description{
  Invokes a text editor allowing the user to inspect and change
  entries in a spatial dataset. 
}
\usage{
\method{edit}{ppp}(name, \dots)

\method{edit}{psp}(name, \dots)

\method{edit}{im}(name, \dots)
}
\arguments{
  \item{name}{
    A spatial dataset
    (object of class \code{"ppp"}, \code{"psp"} or \code{"im"}).
  }
  \item{\dots}{
    Other arguments passed to \code{\link[utils]{edit.data.frame}}.
  }
}
\details{
  The function \code{\link[utils]{edit}} is generic. These functions
  are methods for spatial
  objects of class \code{"ppp"}, \code{"psp"} and \code{"im"}.

  The spatial dataset \code{name} is converted to a data frame or array,
  and the text editor is invoked. The user can change the values of spatial
  coordinates or marks of the points in a point pattern,
  or the coordinates or marks of the segments in a segment pattern,
  or the pixel values in an image. 
  The names of the columns of marks can also be edited.

  If \code{name} is a pixel image, it is converted to a matrix 
  and displayed in the same spatial orientation as if
  the image had been plotted.

  Note that the original object \code{name} is not changed;
  the function returns the edited dataset.
}
\value{
  Object of the same kind as \code{name} containing the edited data.
}
\author{
\adrian


\rolf

and \ege

}
\seealso{
  \code{\link[utils]{edit.data.frame}},
  \code{\link{edit.hyperframe}}
}
\examples{
  if(interactive()) Z <- edit(cells)
}
\keyword{spatial}
\keyword{manip}

\name{plot.im}
\alias{plot.im}
\alias{image.im}
\title{Plot a Pixel Image}
\description{
  Plot a pixel image.
}
\usage{
  \method{plot}{im}(x, \dots,
                   main,
                   add=FALSE, clipwin=NULL,
                   col=NULL, valuesAreColours=NULL, log=FALSE,
                   ncolours=256, gamma=1, 
                   ribbon=show.all, show.all=!add,
                   ribside=c("right", "left", "bottom", "top"),
                   ribsep=0.15, ribwid=0.05, ribn=1024,
                   ribscale=1, ribargs=list(), riblab=NULL, colargs=list(),
                   useRaster=NULL, workaround=FALSE, zap=1,
                   do.plot=TRUE) 

  \method{image}{im}(x, \dots,
                   main,
                   add=FALSE, clipwin=NULL,
                   col=NULL, valuesAreColours=NULL, log=FALSE, 
                   ncolours=256, gamma=1, 
                   ribbon=show.all, show.all=!add,
                   ribside=c("right", "left", "bottom", "top"),
                   ribsep=0.15, ribwid=0.05, ribn=1024,
                   ribscale=1, ribargs=list(), riblab=NULL, colargs=list(),
                   useRaster=NULL, workaround=FALSE, zap=1,
                   do.plot=TRUE) 
}
\arguments{
  \item{x}{
    The pixel image to be plotted.
    An object of class \code{"im"} (see \code{\link{im.object}}).
  }
  \item{\dots}{
    Extra arguments passed to \code{\link[graphics]{image.default}}
    to control the plot. See Details.
  }
  \item{main}{Main title for the plot.}
  \item{add}{
    Logical value indicating whether to superimpose the image on the
    existing plot (\code{add=TRUE})
    or to initialise a new plot (\code{add=FALSE}, the default).
  }
  \item{clipwin}{
    Optional. A window (object of class \code{"owin"}).
    Only this subset of the image will be displayed.
  }
  \item{col}{
    Colours for displaying the pixel values.
    Either a character vector of colour values,
    an object of class \code{\link{colourmap}},
    or a \code{function} as described under Details.
  }
  \item{valuesAreColours}{
    Logical value. If \code{TRUE}, the pixel values of \code{x}
    are to be interpreted as colour values.
  }
  \item{log}{
    Logical value. If \code{TRUE}, the colour map will be
    evenly-spaced on a logarithmic scale.
  }
  \item{ncolours}{
    Integer. The default number of colours in the colour map
    for a real-valued image.
  }
  \item{gamma}{
    Exponent for the gamma correction of the colours.
    A single positive number.
  }
  \item{ribbon}{
    Logical flag indicating whether to display a ribbon
    showing the colour map. Default is \code{TRUE}
    for new plots and \code{FALSE} for added plots.
  }
  \item{show.all}{
    Logical value indicating whether to display all plot elements
    including the main title and colour ribbon. Default is \code{TRUE}
    for new plots and \code{FALSE} for added plots.
  }
  \item{ribside}{
    Character string indicating where to display the ribbon
    relative to the main image.
  }
  \item{ribsep}{
    Factor controlling the space between the ribbon and the image.
  }
  \item{ribwid}{
    Factor controlling the width of the ribbon.
  }
  \item{ribn}{
    Number of different values to display in the ribbon.
  }
  \item{ribscale}{
    Rescaling factor for tick marks. The values on the numerical scale
    printed beside the ribbon will be multiplied by this rescaling factor.
  }
  \item{ribargs}{
    List of additional arguments passed to
    \code{\link[graphics]{image.default}}, 
    \code{\link[graphics]{axis}} and
    \code{\link[grDevices]{axisTicks}}
    to control the display of the ribbon and its scale axis. These may override
    the \code{\dots} arguments. 
  }
  \item{riblab}{
    Text to be plotted in the margin near the ribbon.
    A character string or expression to be interpreted as text,
    or a list of arguments to be passed to
    \code{\link[graphics]{mtext}}.
  }
  \item{colargs}{
    List of additional arguments passed to
    \code{col} if it is a function.
  }
  \item{useRaster}{
    Logical value, passed to \code{\link[graphics]{image.default}}.
    Images are plotted using a bitmap raster if \code{useRaster=TRUE}
    or by drawing polygons if \code{useRaster=FALSE}.
    Bitmap raster display tends to produce better results,
    but is not supported on all graphics devices.
    The default is to use bitmap raster display if it is supported.
  }
  \item{workaround}{
    Logical value, specifying whether to use a workaround to avoid a bug
    which occurs with some device drivers in \R, in which the image
    has the wrong spatial orientation. See the section on
    \bold{Image is Displayed in Wrong Spatial Orientation} below.
  }
  \item{zap}{
    Noise threshold factor. A numerical value greater than or equal to 1.
    If the range of pixel values is less than
    \code{zap * .Machine$double.eps}, the image will be treated as
    constant. This avoids displaying images which should be constant
    but contain small numerical errors.
  }
  \item{do.plot}{
    Logical value indicating whether to actually plot the image
    and colour ribbon.
    Setting \code{do.plot=FALSE} will simply return the
    colour map and the bounding box that were chosen for the plot.
  }
}
\value{
  The colour map used. An object of class \code{"colourmap"}.

  Also has an attribute \code{"bbox"} giving a bounding box
  for the plot (containing the main colour image and the
  colour ribbon if plotted).
  If a ribbon was plotted, there is also an attribute
  \code{"bbox.legend"} giving a bounding box for the ribbon image.
  Text annotation occurs outside these bounding boxes.
}
\details{
  This is the \code{plot} method for the class \code{"im"}.
  [It is also the \code{image} method for \code{"im"}.]

  The pixel image \code{x} is displayed on the current plot device,
  using equal scales on the \code{x} and \code{y} axes.

  If \code{ribbon=TRUE}, a legend will be plotted.
  The legend consists of a colour ribbon and an axis with tick-marks,
  showing the correspondence between the pixel values and the colour map.

  Arguments \code{ribside, ribsep, ribwid} control the placement
  of the colour ribbon.
  By default, the ribbon is placed at the right of the main image.
  This can be changed using the argument \code{ribside}.
  The width of the ribbon is \code{ribwid} times the size of the pixel
  image, where `size' means the larger of the width and the height.
  The distance separating the ribbon and the image is \code{ribsep} times
  the size of the pixel image.

  The ribbon contains the colours representing \code{ribn}
  different numerical values, evenly spaced between the minimum and
  maximum pixel values in the image \code{x}, rendered according to
  the chosen colour map.

  The argument \code{ribargs} controls the annotation of the
  colour ribbon. It is a list of arguments to be passed to
  \code{\link[graphics]{image.default}}, 
  \code{\link[graphics]{axis}} and
  \code{\link[grDevices]{axisTicks}}.
  To plot the colour ribbon without the axis and
  tick-marks, use \code{ribargs=list(axes=FALSE)}.
  To ensure that the numerals or symbols printed next to the colour map
  are oriented horizontally, use \code{ribargs=list(las=1)}.
  To double the size of the numerals or symbols, 
  use \code{ribargs=list(cex.axis=2)}.
  To control the number of tick-marks, use \code{ribargs=list(nint=N)}
  where \code{N} is the desired number of intervals (so there will
  be \code{N+1} tickmarks, subject to the vagaries of \R internal code). 

  The argument \code{riblab} contains text that will be displayed
  in the margin next to the ribbon. 
  
  The argument \code{ribscale} is used
  to rescale the numerical values printed next to the colour map,
  for convenience. 
  For example if the pixel values in \code{x} range between 1000 and
  4000, it would be sensible to use \code{ribscale=1/1000} so that the
  colour map tickmarks would be labelled 1 to 4. 

  Normally the pixel values are displayed using the colours given in the
  argument \code{col}. This may be either
  \itemize{
    \item an explicit colour map (an object of class
    \code{"colourmap"}, created by the command \code{\link{colourmap}}).
    This is the best way to ensure
    that when we plot different images, the colour maps are consistent.
    \item a character vector or integer vector
    that specifies a set of colours.
    The colour mapping will be stretched to match the range of
    pixel values in the image \code{x}. The mapping of pixel values
    to colours is determined as follows.
    \describe{
      \item{logical-valued images:}{the values \code{FALSE} and
	\code{TRUE} are mapped to the colours \code{col[1]} and
	\code{col[2]} respectively.
	The vector \code{col} should have length 2.
      }
      \item{factor-valued images:}{the factor levels \code{levels(x)}
	are mapped to the entries of \code{col} in order. The vector
	\code{col} should have the same length as \code{levels(x)}.
      }
      \item{numeric-valued images:}{
	By default, the range of pixel values in \code{x}
	is divided into \code{n = length(col)}
	equal subintervals, which are mapped to the colours in \code{col}.
	(If \code{col} was not specified, it defaults to a vector of 255
	colours.)
      
	Alternatively if the argument \code{zlim} is given, it should be
	a vector of length 2 specifying an interval of real numbers.
	This interval will be used instead of the range of pixel
	values. The interval from \code{zlim[1]} to \code{zlim[2]} will be
	mapped to the colours in \code{col}. This facility enables the user to
	plot several images using a consistent colour map.

	Alternatively if the argument \code{breaks} is given,
	then this specifies
	the endpoints of the subintervals that are mapped to each colour.
	This is incompatible with \code{zlim}.
      
	The arguments \code{col} and \code{zlim} or \code{breaks}
	are then passed to the function \code{\link{image.default}}.
	For examples of the use of these arguments,
	see \code{\link{image.default}}.
      }
    }
    \item {
      a \code{function} in the \R language
      with an argument named \code{range}
      or \code{inputs}.
    
      If \code{col} is a function with an argument named \code{range},
      and if the pixel values of \code{x} are numeric values, 
      then the colour values will be determined by evaluating
      \code{col(range=range(x))}. The result of this evaluation
      should be a character vector containing colour values, or
      a \code{"colourmap"} object. Examples of such functions
      are \code{\link{beachcolours}} and \code{\link{beachcolourmap}}.
  
      If \code{col} is a function with an argument named \code{inputs},
      and if the pixel values of \code{x} are discrete values (integer,
      logical, factor or character), 
      then the colour values will be determined by evaluating
      \code{col(inputs=p)} where \code{p} is the set of possible pixel
      values. The result should be a character vector 
      containing colour values, or a \code{"colourmap"} object.
    }
    \item{
      a \code{function} in the \R language with first argument
      named \code{n}.
      The colour values will be determined by evaluating
      \code{col(n)} where \code{n} is the
      number of distinct pixel values, up to a maximum of 128.
      The result of this evaluation
      should be a character vector containing color values.
      Examples of such functions are
      \code{\link[grDevices]{heat.colors}},
      \code{\link[grDevices]{terrain.colors}},
      \code{\link[grDevices]{topo.colors}} and
      \code{\link[grDevices]{cm.colors}}. 
   }
  }

  If \code{spatstat.options("monochrome")} has been set to \code{TRUE}
  then \bold{all colours will be converted to grey scale values}.
  
  Other graphical parameters controlling the display of both the pixel image
  and the ribbon can be passed through the \code{...} arguments
  to the function \code{\link[graphics]{image.default}}.
  A parameter is handled only if it is one of the following:
  \itemize{
    \item a formal argument of \code{\link[graphics]{image.default}}
    that is operative when \code{add=TRUE}.
    \item one of the parameters
    \code{"main", "asp", "sub", "axes", "xlab", "ylab"}
    described in \code{\link[graphics]{plot.default}}.
    \item one of the parameters
    \code{"ann", "cex", "font", "cex.axis", "cex.lab", "cex.main", "cex.sub",
      "col.axis", "col.lab", "col.main", "col.sub",
      "font.axis", "font.lab", "font.main", "font.sub"}
    described in \code{\link[graphics]{par}}.
    \item the argument \code{box}, a logical value specifying whether
    a box should be drawn.
  }
  Images are plotted using a bitmap raster if \code{useRaster=TRUE}
  or by drawing polygons if \code{useRaster=FALSE}.
  Bitmap raster display (performed by \code{\link[graphics]{rasterImage}})
  tends to produce better results, but is not supported on all graphics devices.
  The default is to use bitmap raster display if it is
  supported according to \code{\link[grDevices]{dev.capabilities}}.

  Alternatively, the pixel values could be directly 
  interpretable as colour values in \R. That is, the pixel values
  could be character strings that represent colours, or
  values of a factor whose levels are character strings representing
  colours.
  \itemize{
    \item If \code{valuesAreColours=TRUE}, then the pixel values will
    be interpreted as colour values and displayed using these colours.
    \item If \code{valuesAreColours=FALSE}, then the pixel values will
    \emph{not} be interpreted as colour values, even if they could be.
    \item If \code{valuesAreColours=NULL}, the algorithm will guess
    what it should do. If the argument \code{col} is given,
    the pixel values will \emph{not} be interpreted as colour values. Otherwise,
    if all the pixel values are strings that represent colours, then
    they will be interpreted and displayed as colours.
  }
  If pixel values are interpreted as colours, 
  the arguments \code{col} and \code{ribbon} will be ignored,
  and a ribbon will not be plotted.

}
\section{Complex-valued images}{
  If the pixel values in \code{x} are complex numbers,
  they will be converted into four images containing the real and
  imaginary parts and the modulus and argument, 
  and plotted side-by-side using \code{\link{plot.imlist}}.
}
\section{Monochrome colours}{
  If \code{spatstat.options("monochrome")} has been set to \code{TRUE},
  then \bold{the image will be plotted in greyscale}.
  The colours are converted to grey scale values using
  \code{\link{to.grey}}.
  The choice of colour map still has an effect, since it determines
  the final grey scale values.

  Monochrome display can also be achieved by
  setting the graphics device parameter \code{colormodel="grey"}
  when starting a new graphics device, or in a call to
  \code{\link{ps.options}} or \code{\link{pdf.options}}.
}
\section{Image Looks Like Noise}{
  An image plot which looks like digital noise
  can be produced when the pixel values are almost exactly equal
  but include a tiny amount of numerical error.
  To check this, look at the numerals plotted next to the colour ribbon,
  or compute \code{diff(range(x))}, to determine whether the range
  of pixel values is almost zero. The behaviour can be suppressed
  by picking a larger value of the argument \code{zap}.
}
\section{Image Rendering Errors and Problems}{
  The help for \code{\link[graphics]{image.default}}
  and \code{\link[graphics]{rasterImage}} explains that
  errors may occur, or images may be rendered incorrectly, on some
  devices, depending on the availability of colours and other
  device-specific constraints.

  If the image is not displayed at all,
  try setting \code{useRaster=FALSE} in the call to \code{plot.im}.
  If the ribbon colours are not displayed, set
  \code{ribargs=list(useRaster=FALSE)}.

  Errors may occur on some graphics devices if the image is very
  large. If this happens, try setting \code{useRaster=FALSE} in the
  call to \code{plot.im}.

  The error message
  \code{useRaster=TRUE can only be used with a regular grid}
  means that the \eqn{x} and \eqn{y} coordinates of the pixels in the
  image are not perfectly equally spaced, due to numerical rounding.
  This occurs with some images created by earlier versions of \pkg{spatstat}.
  To repair the coordinates in an image \code{X}, type
  \code{X <- as.im(X)}.
}
\section{Image is Displayed in Wrong Spatial Orientation}{
  If the image is displayed in the wrong spatial orientation,
  and you created the image data directly, 
  please check that you understand the \pkg{spatstat} convention for
  the spatial orientation of pixel images. The row index of the matrix
  of pixel values corresponds to the increasing \eqn{y} coordinate; the
  column index of the matrix corresponds to the increasing \eqn{x} coordinate
  (Baddeley, Rubak and Turner, 2015, section 3.6.3, pages 66--67).

  Images can be displayed in the wrong spatial orientation
  on some devices, due to a bug in the device driver. This occurs only
  when the plot coordinates are \emph{reversed}, that is, when 
  the plot was initialised with coordinate limits \code{xlim, ylim}
  such that \code{xlim[1] > xlim[2]} or \code{ylim[1] > ylim[2]} or
  both. This bug is reported to occur only when \code{useRaster=TRUE}.
  To fix this, try setting \code{workaround=TRUE}, or if that is
  unsuccessful, \code{useRaster=FALSE}.
}
\seealso{
  \code{\link{im.object}},
  \code{\link{colourmap}},
  \code{\link{contour.im}},
  \code{\link{persp.im}},
  \code{\link{hist.im}},
  \code{\link[graphics]{image.default}},
  \code{\link{spatstat.options}}
}
\examples{
   # an image
   Z <- setcov(owin())
   plot(Z)
   plot(Z, ribside="bottom")
   # stretchable colour map
   plot(Z, col=rainbow)
   plot(Z, col=terrain.colors(128), axes=FALSE)
   # fixed colour map
   tc <- colourmap(rainbow(128), breaks=seq(-1,2,length=129))
   plot(Z, col=tc)
   # colour map function, with argument 'range'
   plot(Z, col=beachcolours, colargs=list(sealevel=0.5))
   # tweaking the plot
   plot(Z, main="La vie en bleu", col.main="blue", cex.main=1.5,
        box=FALSE,
        ribargs=list(col.axis="blue", col.ticks="blue", cex.axis=0.75))
   # add axes and axis labels
   plot(Z, axes=TRUE, ann=TRUE, xlab="Easting", ylab="Northing")
   # log scale
   V <- eval.im(exp(exp(Z+2))/1e4)
   plot(V, log=TRUE, main="Log scale")
   # it's complex
   Y <- exp(Z + V * 1i)
   plot(Y)
}
\references{
  Baddeley, A., Rubak, E. and Turner, R. (2015)
  \emph{Spatial Point Patterns: Methodology and Applications with R}.
  Chapman and Hall/CRC Press.
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
 
\keyword{hplot}

\name{plot.ppp}
\alias{plot.ppp}
\title{plot a Spatial Point Pattern}
\description{
  Plot a two-dimensional spatial point pattern
}
\usage{
 \method{plot}{ppp}(x, main, \dots, clipwin=NULL,
                    chars=NULL, cols=NULL,
                    use.marks=TRUE, which.marks=NULL,
                    add=FALSE, type=c("p","n"),
                    legend=TRUE,
                    leg.side=c("left", "bottom", "top", "right"),
                    leg.args=list(),
                    symap=NULL, maxsize=NULL, meansize=NULL, markscale=NULL,
                    zap=0.01,
                    show.window=show.all, show.all=!add, do.plot=TRUE,
                    multiplot=TRUE)
}
\arguments{
  \item{x}{
    The spatial point pattern to be plotted.
    An object of class \code{"ppp"},
    or data which can be converted into 
    this format by \code{\link{as.ppp}()}.
  }
  \item{main}{
    text to be displayed as a title above the plot.
  }
  \item{\dots}{
    extra arguments that will be passed to the plotting functions
    \code{\link{plot.default}}, \code{\link{points}} and/or
    \code{\link{symbols}}. Not all arguments will be recognised.
  }
  \item{clipwin}{
    Optional. A window (object of class \code{"owin"}).
    Only this subset of the image will be displayed.
  }
  \item{chars}{
    plotting character(s) used to plot points.
  }
  \item{cols}{
    the colour(s) used to plot points.
  }
  \item{use.marks}{
    logical flag; if \code{TRUE}, plot points using a different
    plotting symbol for each mark;
    if \code{FALSE}, only the locations of the points will be plotted,
    using \code{\link{points}()}.
  }
  \item{which.marks}{
    Index determining which column of marks to use,
    if the marks of \code{x} are a data frame.
    A character or integer vector identifying one or more
    columns of marks. 
    If \code{add=FALSE} then
    the default is to plot all columns of marks, in a series of separate
    plots.
    If \code{add=TRUE} then only one column of marks can be plotted,
    and the default is \code{which.marks=1}
    indicating the first column of marks.
  }
  \item{add}{
    logical flag; if \code{TRUE},
    just the points are plotted, over the existing plot.
    A new plot is not created, and
    the window is not plotted.
  }
  \item{type}{
    Type of plot: either \code{"p"} or \code{"n"}.
    If \code{type="p"} (the default), both the points and the observation window
    are plotted. If \code{type="n"}, only the window is plotted.
  }
  \item{legend}{
    Logical value indicating whether to add a legend showing the mapping
    between mark values and graphical symbols (for a marked point pattern).
  }
  \item{leg.side}{
    Position of legend relative to main plot.
  }
  \item{leg.args}{
    List of additional arguments passed to \code{\link{plot.symbolmap}}
    or \code{\link{symbolmap}} to control the legend.
    In addition to arguments documented under
    \code{\link{plot.symbolmap}}, and graphical arguments recognised
    by \code{\link{symbolmap}}, the list may also include the argument
    \code{sep} giving the separation between the main plot and the
    legend, or \code{sep.frac} giving the separation as a fraction
    of the relevant dimension (width or height) of the main plot.
  }
  \item{symap}{
    Optional. The graphical symbol map to be applied to the marks.
    An object of class \code{"symbolmap"}; see
    \code{\link{symbolmap}}.
  }
  \item{maxsize}{
    \emph{Maximum} physical size of the circles/squares plotted
    when \code{x} is a marked point pattern with 
    numerical marks.
    Incompatible with \code{meansize} and \code{markscale}.
    Ignored if \code{symap} is given.
  }
  \item{meansize}{
    \emph{Average} physical size of the circles/squares plotted
    when \code{x} is a marked point pattern with 
    numerical marks.
    Incompatible with \code{maxsize} and \code{markscale}.
    Ignored if \code{symap} is given.
  }
  \item{markscale}{
    physical scale factor determining the sizes of the
    circles/squares plotted when \code{x} is a marked point pattern with 
    numerical marks. Mark value will be multiplied by \code{markscale}
    to determine physical size.
    Incompatible with \code{maxsize} and \code{meansize}.
    Ignored if \code{symap} is given.
  }
  \item{zap}{
    Fraction between 0 and 1.
    When \code{x} is a marked point pattern with numerical marks,
    \code{zap} is the smallest mark value
    (expressed as a fraction of the maximum possible mark) that will
    be plotted. 
    Any points which have marks smaller in absolute value
    than \code{zap * max(abs(marks(x)))} will not be plotted.
  }
  \item{show.window}{
    Logical value indicating whether to plot the observation
    window of \code{x}.
  }
  \item{show.all}{
    Logical value indicating whether to plot everything
    including the main title and the observation window of \code{x}.
  }
  \item{do.plot}{
    Logical value determining whether to actually perform the plotting.
  }
  \item{multiplot}{
    Logical value giving permission to display multiple plots.
  }
}
\value{
  (Invisible) object of class \code{"symbolmap"}
  giving the correspondence between 
  mark values and plotting characters.
}
\details{
  This is the \code{plot} method for 
  point pattern datasets (of class \code{"ppp"}, see \code{\link{ppp.object}}).

  First the observation window \code{Window(x)} is plotted
  (if \code{show.window=TRUE}).
  Then the points themselves are plotted,
  in a fashion that depends on their marks,
  as follows.
  \describe{
    \item{unmarked point pattern:}{
      If the point pattern does not have marks, or if \code{use.marks = FALSE},
      then the locations of all points will be plotted 
      using a single plot character
    }
    \item{multitype point pattern:}{
      If \code{x$marks} is a factor, then 
      each level of the factor is 
      represented by a different plot character.
    }
    \item{continuous marks:}{
      If \code{x$marks} is a numeric vector,
      the marks are rescaled to the unit interval and
      each point is represented by a circle 
      with \emph{diameter} proportional to the rescaled mark
      (if the value is positive) or a square with \emph{side length}
      proportional to the absolute value of the rescaled mark
      (if the value is negative).
    }
    \item{other kinds of marks:}{
      If \code{x$marks} is neither numeric nor a factor,
      then each possible mark will be represented by a
      different plotting character. The default is to
      represent the \eqn{i}th smallest mark value by
      \code{points(..., pch=i)}.
    }
  }

  If there are several columns of marks, and if \code{which.marks} is
  missing or \code{NULL}, then
  \itemize{
    \item 
    if \code{add=FALSE} and \code{multiplot=TRUE}
    the default is to plot all columns of marks, in a series of separate
    plots, placed side-by-side. The plotting is coordinated by
    \code{\link{plot.listof}}, which calls \code{plot.ppp} to make each of
    the individual plots.
    \item
    Otherwise, only one column of marks can be plotted,
    and the default is \code{which.marks=1}
    indicating the first column of marks.
  }

  Plotting of the window \code{Window(x)} is performed by
  \code{\link{plot.owin}}. This plot may be modified 
  through the \code{...} arguments. In particular the
  extra argument \code{border} determines
  the colour of the window, if the window is not a binary mask.

  Plotting of the points themselves is performed
  by the function \code{\link{points}}, except for the case of
  continuous marks, where it is performed by \code{\link{symbols}}.
  Their plotting behaviour may be modified through the \code{...}
  arguments.

  The argument \code{chars} determines the plotting character
  or characters used to display the points (in all cases except
  for the case of continuous marks). For an unmarked point pattern,
  this should be a single integer or character determining a
  plotting character (see \code{par("pch")}).
  For a multitype point pattern, \code{chars} should be a vector
  of integers or characters, of the same length
  as \code{levels(x$marks)}, and then the \eqn{i}th level or type 
  will be plotted using character \code{chars[i]}.

  If \code{chars} is absent, but there is an extra argument
  \code{pch}, then this will determine the plotting character for
  all points.
  
  The argument \code{cols} determines the colour or colours used to
  display the points. For an unmarked point pattern,
  \code{cols} should be a character string
  determining a colour. For a multitype point pattern, \code{cols}
  should be a character vector, of the same length
  as \code{levels(marks(x))}: that is, there is one colour for each
  possible mark value. The \eqn{i}th level or type will
  be plotted using colour \code{cols[i]}. For a point pattern with
  continuous marks, \code{cols} can be either a character string
  or a character vector specifying colour values: the range of mark
  values will be mapped to the specified colours.

  If \code{cols} is absent, the colours used to plot the
  points may be determined by the extra argument \code{fg}
  (for multitype point patterns) or the extra argument \code{col}
  (for all other cases). Note that specifying \code{col} will also
  apply this colour to the window itself.

  The default colour for the points is a semi-transparent grey,
  if this is supported by the plot device. This behaviour can be
  suppressed (so that the default colour is non-transparent)
  by setting \code{spatstat.options(transparent=FALSE)}.

  The arguments \code{maxsize}, \code{meansize} and \code{markscale} 
  incompatible. They control the physical size of the circles and
  squares which represent the marks in a point pattern with continuous
  marks. The size of a circle is defined as its \emph{diameter};
  the size of a square is its side length.
  If \code{markscale} is given, then a mark value of \code{m}
  is plotted as a circle of diameter \code{m * markscale}
  (if \code{m} is positive) or a square of side \code{abs(m) * markscale}
  (if \code{m} is negative). If \code{maxsize} is given, then the
  largest mark in absolute value, \code{mmax=max(abs(marks(x)))},
  will be scaled to have physical size \code{maxsize}.
  If \code{meansize} is given, then the
  average absolute mark value, \code{mmean=mean(abs(marks(x)))},
  will be scaled to have physical size \code{meansize}.

  The user can set the default values of these plotting parameters
  using \code{\link{spatstat.options}("par.points")}.
  
  To zoom in (to view only a subset of the point pattern at higher
  magnification), use the graphical arguments
  \code{xlim} and \code{ylim} to specify the rectangular field of view.

  The value returned by this plot function is an object of
  class \code{"symbolmap"} representing the mapping from mark values
  to graphical symbols. See \code{\link{symbolmap}}.
  It can be used to make a suitable legend,
  or to ensure that two plots use the same graphics map.
}
\section{Removing White Space Around The Plot}{
  A frequently-asked question is: How do I remove the white space around
  the plot? Currently \code{plot.ppp} uses the base graphics system of
  \R, so the space around the plot is controlled by parameters 
  to \code{\link{par}}. To reduce the white space, change the
  parameter \code{mar}. Typically, \code{par(mar=rep(0.5, 4))} is
  adequate, if there are no annotations or titles outside the window.
}
\section{Drawing coordinate axes and axis labels}{
  Coordinate axes and axis labels are not drawn, by default.
  To draw coordinate axes, set \code{axes=TRUE}.
  To draw axis labels, set \code{ann=TRUE} and give values to the
  arguments \code{xlab} and \code{ylab}. See the Examples.
  Only the default style of axis is supported;
  for more control over the placement and style of axes,
  use the graphics commands
  \code{\link[graphics]{axis}} and \code{\link[graphics]{mtext}}.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{plot}},
  \code{\link{par}},
  \code{\link{points}},
  \code{\link{text.ppp}},
  \code{\link{plot.owin}},
  \code{\link{symbols}}.

  See also the command \code{iplot} in the \pkg{spatstat.gui} package.
}
\examples{
   plot(cells)

   plot(cells, pch=16)

   # make the plotting symbols larger (for publication at reduced scale)
   plot(cells, cex=2)

   # set it in spatstat.options
   oldopt <- spatstat.options(par.points=list(cex=2))
   plot(cells)
   spatstat.options(oldopt)

   # multitype 
   plot(lansing)

   # marked by a real number
   plot(longleaf)

   # just plot the points
   plot(longleaf, use.marks=FALSE)
   plot(unmark(longleaf)) # equivalent

   # point pattern with multiple marks
   plot(finpines)
   plot(finpines, which.marks="height")

   # controlling COLOURS of points
   plot(cells, cols="blue")
   plot(lansing, cols=c("black", "yellow", "green", 
                        "blue","red","pink"))
   plot(longleaf, fg="blue")

   # make window purple
   plot(lansing, border="purple")
   # make everything purple
   plot(lansing, border="purple", cols="purple", col.main="purple",
                 leg.args=list(col.axis="purple"))
 
   # controlling PLOT CHARACTERS for multitype pattern
   plot(lansing, chars = 11:16)
   plot(lansing, chars = c("o","h","m",".","o","o"))

   ## multitype pattern mapped to symbols
   plot(amacrine, shape=c("circles", "squares"), size=0.04)
   plot(amacrine, shape="arrows", direction=c(0,90), size=0.07)

   ## plot trees as trees!
   plot(lansing, shape="arrows", direction=90, cols=1:6)

   # controlling MARK SCALE for pattern with numeric marks
   plot(longleaf, markscale=0.1)
   plot(longleaf, maxsize=5)
   plot(longleaf, meansize=2)

   # draw circles of diameter equal to nearest neighbour distance
   plot(cells \%mark\% nndist(cells), markscale=1, legend=FALSE)

   # inspecting the symbol map
   v <- plot(amacrine)
   v

   ## variable colours ('cols' not 'col')
   plot(longleaf, cols=function(x) ifelse(x < 30, "red", "black"))

   ## re-using the same mark scale
   a <- plot(longleaf)
   juveniles <- longleaf[marks(longleaf) < 30]
   plot(juveniles, symap=a)

   ## numerical marks mapped to symbols of fixed size with variable colour
   ra <- range(marks(longleaf))
   colmap <- colourmap(terrain.colors(20), range=ra)
   ## filled plot characters are the codes 21-25
   ## fill colour is indicated by 'bg'
   sy <- symbolmap(pch=21, bg=colmap, range=ra)
   plot(longleaf, symap=sy)

   ## or more compactly..
   plot(longleaf, bg=terrain.colors(20), pch=21, cex=1)

   ## clipping
   plot(humberside)
   B <- owin(c(4810, 5190), c(4180, 4430))
   plot(B, add=TRUE, border="red")
   plot(humberside, clipwin=B, main="Humberside (clipped)")

   ## coordinate axes and labels
   plot(humberside, axes=TRUE)
   plot(humberside,            ann=TRUE, xlab="Easting", ylab="Northing")
   plot(humberside, axes=TRUE, ann=TRUE, xlab="Easting", ylab="Northing")
}

\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{hplot}

\name{rpoint}
\alias{rpoint}
\title{Generate N Random Points}
\description{
  Generate a random point pattern
  containing \eqn{n} independent, identically distributed random points
  with any specified distribution.
}
\usage{
 rpoint(n, f, fmax=NULL, win=unit.square(),
        \dots, giveup=1000, verbose=FALSE,
        nsim=1, drop=TRUE)
}
\arguments{
  \item{n}{
    Number of points to generate.
  }
  \item{f}{
    The probability density of the points,
    possibly un-normalised.
    Either a constant,
    a function \code{f(x,y,...)}, or a pixel image object.
  }
  \item{fmax}{
    An upper bound on the values of \code{f}.
    If missing, this number will be estimated.
  }
  \item{win}{
    Window in which to simulate the pattern.
    Ignored if \code{f} is a pixel image.
  }
  \item{\dots}{
    Arguments passed to the function \code{f}.
  }
  \item{giveup}{
    Number of attempts in the rejection method after which
    the algorithm should stop trying to generate new points.
  }
  \item{verbose}{
    Flag indicating whether to report details of performance
    of the simulation algorithm.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  A point pattern (an object of class \code{"ppp"})
  if \code{nsim=1}, or a list of point patterns if \code{nsim > 1}.
}
\details{
  This function generates \code{n} independent, identically distributed
  random points with common probability density proportional to
  \code{f}.

  The argument \code{f} may be
  \describe{
    \item{a numerical constant:}{
      uniformly distributed random points will
      be generated.
    }
    \item{a function:}{random points will be generated
      in the window \code{win} with probability density proportional
      to \code{f(x,y,...)} where \code{x} and \code{y} are the cartesian
      coordinates. The function \code{f} must accept 
      two \emph{vectors} of coordinates \code{x,y} and return the corresponding
      vector of function values. Additional arguments \code{...} of any kind
      may be passed to the function.
    }
    \item{a pixel image:}{if \code{f} is a pixel image object
      of class \code{"im"} (see \code{\link{im.object}}) then
      random points will be generated
      in the window of this pixel image, with probability density
      proportional to the pixel values of \code{f}.
    }
  }
  The algorithm is as follows:
  \itemize{
    \item
    If \code{f} is a constant, we invoke \code{\link{runifpoint}}.
    \item
    If \code{f} is a function, then we use the rejection method.
    Proposal points are generated from the uniform distribution.
    A proposal point \eqn{(x,y)} is accepted with probability
    \code{f(x,y,...)/fmax} and otherwise rejected.
    The algorithm continues until \code{n} points have been
    accepted. It gives up after \code{giveup * n} proposals
    if there are still fewer than \code{n} points.
    \item 
    If \code{f} is a pixel image, then a random sequence of 
    pixels is selected (using \code{\link{sample}})
    with probabilities proportional to the
    pixel values of \code{f}.  Then for each pixel in the sequence
    we generate a uniformly distributed random point in that pixel.
  }
  The algorithm for pixel images is more efficient than that for
  functions.
}
\seealso{
\code{\link{ppp.object}},
\code{\link{owin.object}},
\code{\link{runifpoint}}
}
\examples{
 # 100 uniform random points in the unit square
 X <- rpoint(100)

 # 100 random points with probability density proportional to x^2 + y^2
 X <- rpoint(100, function(x,y) { x^2 + y^2}, 1)

 # `fmax' may be omitted
 X <- rpoint(100, function(x,y) { x^2 + y^2})

 # irregular window
 data(letterR)
 X <- rpoint(100, function(x,y) { x^2 + y^2}, win=letterR)

 # make a pixel image 
 Z <- setcov(letterR)
 # 100 points with density proportional to pixel values
 X <- rpoint(100, Z)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{connected.ppp}  
\Rdversion{1.1}
\alias{connected.ppp}
\alias{connected.pp3}
\title{
  Connected Components of a Point Pattern
}
\description{
  Finds the topologically-connected components of a point pattern,
  when all pairs of points closer than a threshold distance are joined.
}
\usage{
\method{connected}{ppp}(X, R, \dots)

\method{connected}{pp3}(X, R, \dots)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"} or \code{"pp3"}).
  }
  \item{R}{
    Threshold distance. Pairs of points closer than \code{R} units apart
    will be joined together.
  }
  \item{\dots}{
    Other arguments, not recognised by these methods.
  }
}
\details{
  This function can be used to identify clumps of points in a point pattern.

  The function \code{connected} is generic. This file documents the methods for
  point patterns in dimension two or three (objects of class \code{"ppp"} or
  \code{"pp3"}).

  The point pattern \code{X} is first converted into an abstract graph
  by joining every pair of points that lie closer than \code{R} units
  apart. Then the connected components of this graph are identified.

  Two points in \code{X} belong to the same connected component if they
  can be reached by a series of steps between points of \code{X},
  each step being shorter than \code{R} units in length.

  The result is a vector of labels for the points of \code{X}
  where all the points in a connected component have the same label.
}
\value{
  A point pattern, equivalent to \code{X} except that the points
  have factor-valued marks, with levels corresponding to the
  connected components.
}
\seealso{
  \code{\link{connected.im}},
  \code{\link{im.object}}, 
  \code{\link{tess}}
}
\examples{
   Y <- connected(redwoodfull, 0.1)
   if(interactive()) {
    plot(Y, cols=1:length(levels(marks(Y))),
         main="connected(redwoodfull, 0.1)")
   }
   X <- osteo$pts[[1]]
   Z <- connected(X, 32)
   if(interactive()) {
    plot(Z, col=marks(Z), main="")
   }
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{complement.owin}
\alias{complement.owin}
\title{Take Complement of a Window}
\description{
  Take the set complement of a window, within its enclosing rectangle
  or in a larger rectangle.
}
\usage{
 complement.owin(w, frame=as.rectangle(w))
}
\arguments{
  \item{w}{
    an object of class \code{"owin"} describing
    a window of observation for a point pattern.
  }
  \item{frame}{
    Optional. The enclosing rectangle,
    with respect to which the set complement is taken.
  }
}
\value{
  Another object of class \code{"owin"} 
  representing the complement of the window, i.e. the inside
  of the window becomes the outside.
}
\details{
  This yields a window object (of class \code{"owin"},
  see \code{\link{owin.object}}) representing the set complement
  of \code{w} with respect to the rectangle \code{frame}.

  By default, \code{frame} is the enclosing box of \code{w}
  (originally specified by the arguments \code{xrange} and \code{yrange}
  given to \code{\link{owin}} when \code{w} was created).
  If \code{frame} is specified, it must be a rectangle (an object of
  class \code{"owin"} whose type is \code{"rectangle"}) and it must be
  larger than the enclosing box of \code{w}. This rectangle becomes the
  enclosing box for the resulting window.
  
  If \code{w} is a rectangle, then \code{frame} must be specified.
  Otherwise an error will occur (since the complement of \code{w} in
  itself is empty).

  For rectangular and polygonal windows, the complement is computed by
  reversing the sign of each boundary polygon, while for binary masks it is
  computed by negating the pixel values.
}
\seealso{
  \code{\link{owin}},
  \code{\link{owin.object}}
}
\examples{
   # rectangular
   a <- owin(c(0,1),c(0,1))
   b <- owin(c(-1,2),c(-1,2))
   bmina <- complement.owin(a, frame=b)
   # polygonal
   data(demopat)
   w <- Window(demopat)
   outside <- complement.owin(w)
   # mask
   w <- as.mask(Window(demopat))
   outside <- complement.owin(w)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{clickpoly}
\alias{clickpoly}
\title{Interactively Define a Polygon}
\description{
  Allows the user to create a polygon by
  point-and-click in the display.
}
\usage{
  clickpoly(add=FALSE, nv=NULL, np=1, \dots)
}
\arguments{
  \item{add}{
    Logical value indicating whether to create a new plot
    (\code{add=FALSE}) or draw over the existing plot (\code{add=TRUE}).
  }
  \item{nv}{
    Number of vertices of the polygon (if this is predetermined).
  }
  \item{np}{
    Number of polygons to create.
  }
  \item{\dots}{
    Arguments passed to \code{\link[graphics]{locator}} to control the
    interactive plot, and to \code{\link[graphics]{polygon}} to plot the
    polygons.
  }
}
\value{
  A window (object of class \code{"owin"}) representing the polygon.
}
\details{
  This function allows the user to create a polygonal window 
  by interactively clicking on the screen display.

  The user is prompted to point the mouse at any desired locations
  for the polygon vertices,
  and click the left mouse button to add each point.
  Interactive input stops after \code{nv} clicks (if \code{nv} was given)
  or when the middle mouse button is pressed.

  The return value is a window (object of class \code{"owin"})
  representing the polygon.

  This function uses the \R command \code{\link[graphics]{locator}} to
  input the mouse clicks. It only works on screen devices such as
  \sQuote{X11}, \sQuote{windows} and \sQuote{quartz}. Arguments that can be
  passed to \code{\link[graphics]{locator}} through \code{\dots} include
  \code{pch} (plotting character), \code{cex} (character expansion
  factor) and \code{col} (colour). See \code{\link[graphics]{locator}}
  and \code{\link[graphics]{par}}.

  Multiple polygons can also be drawn, by specifying
  \code{np > 1}. The polygons must be disjoint. The result is 
  a single window object consisting of all the polygons.
}
\seealso{
  \code{\link{identify.ppp}},
  \code{\link{clickbox}},
  \code{\link{clickppp}},
  \code{\link{clickdist}},
  \code{\link[graphics]{locator}}
}
\author{
  \adrian
  and \rolf.
}
\keyword{spatial}
\keyword{iplot}

\name{Extract.solist}
\alias{[.solist}
\alias{[<-.solist}
\title{Extract or Replace Subset of a List of Spatial Objects}
\description{
  Extract or replace some entries in a list of spatial objects,
  or extract a designated sub-region in each object.
}
\usage{
  \method{[}{solist}(x, i, \dots)

  \method{[}{solist}(x, i) <- value
}
\arguments{
  \item{x}{
    An object of class \code{"solist"} representing a list of
    two-dimensional spatial objects.
  }
  \item{i}{
    Subset index. Any valid subset index for vectors in the usual \R sense,
    or a window (object of class \code{"owin"}).
  }
  \item{value}{
    Replacement value for the subset. 
  }
  \item{\dots}{Ignored.}
}
\value{
  Another object of the same class as \code{x}.
}
\details{
  These are methods for extracting and replacing subsets
  for the class \code{"solist"}.

  The argument \code{x} should be an object of class \code{"solist"}
  representing a list of two-dimensional spatial objects.
  See \code{\link{solist}}.

  For the subset method, the subset index \code{i} can be either
  a vector index (specifying some elements of the list)
  or a spatial window (specifying a spatial sub-region). 

  For the replacement method,
  \code{i} must be a vector index: the designated elements will be
  replaced.
}
\seealso{
  \code{\link{solist}},
  \code{\link{plot.solist}},
  \code{\link{summary.solist}}
}
\examples{
   x <- solist(japanesepines, cells, redwood)
   x[2:3]
   x[square(0.5)]
   x[1] <- list(finpines)
 }
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{list}
\keyword{manip}

\name{tiles.empty}
\alias{tiles.empty}
\title{Check For Empty Tiles in a Tessellation}
\description{
  Checks whether each tile in a tessellation is empty or non-empty.
}
\usage{
tiles.empty(x)
}
\arguments{
  \item{x}{A tessellation (object of class \code{"tess"}).}
}
\details{
  A tessellation is a collection of disjoint spatial regions
  (called \emph{tiles}) that fit together to form a larger spatial
  region. See \code{\link{tess}}.

  It is possible for some tiles of a tessellation to be empty.
  For example, this can happen
  when the tessellation \code{x} is obtained by restricting
  another tessellation \code{y} to a smaller spatial domain \code{w}.

  The function \code{tiles.empty} checks whether each tile is empty
  or non-empty. The result is a logical vector,
  with entries equal to \code{TRUE} when the corresponding tile is
  empty. Results are given in the same order
  as the tiles would be listed by \code{tiles(x)}.
}
\value{
  A logical vector.
}
\seealso{
  \code{\link{tess}},
  \code{\link{tiles}},
  \code{\link{tilenames}},
  \code{\link{tile.areas}}
}
\examples{
  A <- tess(xgrid=0:2,ygrid=0:2)
  tiles.empty(A)
  v <- as.im(function(x,y){factor(round(x^2 + y^2))}, W=owin())
  E <- tess(image=v)
  tiles.empty(E)
}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{manip}

\name{km.rs}
\alias{km.rs}
\title{Kaplan-Meier and Reduced Sample Estimator using Histograms}
\description{
  Compute the Kaplan-Meier and Reduced Sample estimators of a
  survival time distribution function, using histogram techniques
}
\usage{
  km.rs(o, cc, d, breaks)
}
\arguments{
  \item{o}{vector of observed survival times
  }
  \item{cc}{vector of censoring times
  }
  \item{d}{vector of non-censoring indicators
  }
  \item{breaks}{Vector of breakpoints to be used to form histograms.
  }
}
\value{
  A list with five elements
  \item{rs}{Reduced-sample estimate of the survival time c.d.f. \eqn{F(t)}
  }
  \item{km}{Kaplan-Meier estimate of the survival time c.d.f. \eqn{F(t)}
  }
  \item{hazard}{corresponding Nelson-Aalen estimate of the
    hazard rate \eqn{\lambda(t)}{lambda(t)}
  }
  \item{r}{values of \eqn{t} for which \eqn{F(t)} is estimated
  }
  \item{breaks}{the breakpoints vector
  }
}
\details{
  This function is needed mainly for internal use in \pkg{spatstat},
  but may be useful in other applications where you want to form the
  Kaplan-Meier estimator from a huge dataset.

  Suppose \eqn{T_i}{T[i]} are the survival times of individuals
  \eqn{i=1,\ldots,M} with unknown distribution function \eqn{F(t)}
  which we wish to estimate. Suppose these times are right-censored
  by random censoring times \eqn{C_i}{C[i]}.
  Thus the observations consist of right-censored survival times
  \eqn{\tilde T_i = \min(T_i,C_i)}{T*[i] = min(T[i],C[i])}
  and non-censoring indicators
  \eqn{D_i = 1\{T_i \le C_i\}}{D[i] = 1(T[i] <= C[i])}
  for each \eqn{i}.

  The arguments to this function are 
  vectors \code{o}, \code{cc}, \code{d}
  of observed values of \eqn{\tilde T_i}{T*[i]}, \eqn{C_i}{C[i]}
  and \eqn{D_i}{D[i]} respectively.
  The function computes histograms and forms the reduced-sample
  and Kaplan-Meier estimates of  \eqn{F(t)} by
  invoking the functions \code{\link{kaplan.meier}}
  and \code{\link{reduced.sample}}.
  This is efficient if the lengths of \code{o}, \code{cc}, \code{d}
  (i.e. the number of observations) is large.

  The vectors \code{km} and \code{hazard} returned by \code{kaplan.meier}
  are (histogram approximations to) the Kaplan-Meier estimator
  of \eqn{F(t)} and its hazard rate \eqn{\lambda(t)}{lambda(t)}.
  Specifically, \code{km[k]} is an estimate of
  \code{F(breaks[k+1])}, and \code{lambda[k]} is an estimate of
  the average of \eqn{\lambda(t)}{lambda(t)} over the interval
  \code{(breaks[k],breaks[k+1])}. This approximation is exact only if the
  survival times are discrete and the 
  histogram breaks are fine enough to ensure that each interval
  \code{(breaks[k],breaks[k+1])} contains only one possible value of
  the survival time. 

  The vector \code{rs} is the reduced-sample estimator,
  \code{rs[k]} being the reduced sample estimate of \code{F(breaks[k+1])}.
  This value is exact, i.e. the use of histograms does not introduce any
  approximation error in the reduced-sample estimator.
}
\seealso{
  \code{\link{reduced.sample}},
  \code{\link{kaplan.meier}}
}
\author{\adrian
  
  
  and \rolf
  
  }
\keyword{spatial}
\keyword{nonparametric}

\name{diameter}
\alias{diameter}
\title{Diameter of an Object}
\description{
  Computes the diameter of an object such as a two-dimensional window
  or three-dimensional box. 
}
\usage{
 diameter(x)
}
\arguments{
  \item{x}{
    A window or other object whose diameter will be computed.
  }
}
\value{
  The numerical value of the diameter of the object.
}
\details{
  This function computes the diameter of an object
  such as a two-dimensional window or a three-dimensional box. 
  The diameter is the maximum distance 
  between any two points in the object.

  The function \code{diameter} is generic, with methods for
  the class \code{"owin"} (two-dimensional windows),
  \code{"box3"} (three-dimensional boxes),
  \code{"boxx"} (multi-dimensional boxes)
  and \code{"linnet"} (linear networks).
}
\seealso{
  \code{\link{diameter.owin}},
  \code{\link{diameter.box3}},
  \code{\link{diameter.boxx}},
  \code{\link{diameter.linnet}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{is.connected}  
\Rdversion{1.1}
\alias{is.connected}
\alias{is.connected.default}
\alias{is.connected.linnet}
\title{
  Determine Whether an Object is Connected
}
\description{
  Determine whether an object is 
  topologically connected.
}
\usage{
is.connected(X, \dots)

\method{is.connected}{default}(X, \dots)

\method{is.connected}{linnet}(X, \dots)
}
\arguments{
  \item{X}{
    A spatial object such as a pixel image (object of class \code{"im"}),
    a window (object of class \code{"owin"}) or a linear network
    (object of class \code{"linnet"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{connected}} to determine the
    connected components.
  }
}
\details{
  The command \code{is.connected(X)} returns \code{TRUE} if the object
  \code{X} consists of a single, topologically-connected piece,
  and returns \code{FALSE} if \code{X} consists of several pieces
  which are not joined together.
  
  The function \code{is.connected} is generic.
  The default method \code{is.connected.default} 
  works for many classes of objects, including windows (class \code{"owin"})
  and images (class \code{"im"}).
  There is a method for linear networks, \code{is.connected.linnet},
  described here, and a method for point patterns
  described in \code{\link{is.connected.ppp}}.
}
\value{
  A logical value.
}
\seealso{
  \code{\link{connected}},
  \code{\link{is.connected.ppp}}.
}
\examples{
  d <- distmap(cells, dimyx=256)
  X <- levelset(d, 0.07)
  plot(X)
  is.connected(X)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{nndist}
\alias{nndist}
\alias{nndist.ppp}
\alias{nndist.default}
\title{Nearest neighbour distances}
\description{
  Computes the distance from each point to its nearest neighbour
  in a point pattern. Alternatively computes the distance to the
  second nearest neighbour, or third nearest, etc.
}
\usage{
  nndist(X, \dots)
  \method{nndist}{ppp}(X, \dots, k=1, by=NULL, method="C")
  \method{nndist}{default}(X, Y=NULL, \dots, k=1, by=NULL, method="C")
}
\arguments{
  \item{X,Y}{
    Arguments specifying the locations of
    a set of points.
    For \code{nndist.ppp}, the argument \code{X} should be a point
    pattern (object of class \code{"ppp"}).
    For \code{nndist.default}, typically \code{X} and \code{Y} would be
    numeric vectors of equal length. Alternatively \code{Y} may be
    omitted and \code{X} may be
    a list with two components \code{x} and \code{y},
    or a matrix with two columns.
    Alternatively \code{X} can be
    a three-dimensional point pattern (class \code{"pp3"}),
    a higher-dimensional point pattern (class \code{"ppx"}),
    a point pattern on a linear network (class \code{"lpp"}),
    or a spatial pattern of line segments (class \code{"psp"}).
  }
  \item{\dots}{
    Ignored by \code{nndist.ppp}
    and \code{nndist.default}.
  }
  \item{k}{
    Integer, or integer vector. The algorithm will compute the distance to the
    \code{k}th nearest neighbour. 
  }
  \item{by}{
    Optional. A factor, which separates \code{X} into groups.
    The algorithm will compute the distance to
    the nearest point in each group. 
  }
  \item{method}{String specifying which method of calculation to use.
    Values are \code{"C"} and \code{"interpreted"}.
  }
}
\value{
  Numeric vector or matrix containing the 
  nearest neighbour distances for each point.

  If \code{k = 1} (the default), the return value is a
  numeric vector \code{v} such that \code{v[i]} is the
  nearest neighbour distance for the \code{i}th data point.
  
  If \code{k} is a single integer, then the return value is a
  numeric vector \code{v} such that \code{v[i]} is the
  \code{k}th nearest neighbour distance for the
  \code{i}th data point.

  If \code{k} is a vector, then the return value is a
  matrix \code{m} such that \code{m[i,j]} is the
  \code{k[j]}th nearest neighbour distance for the
  \code{i}th data point.

  If the argument \code{by} is given, then the result is a data frame
  containing the distances described above, from each point of \code{X},
  to the nearest point in each subset of \code{X}
  defined by the factor \code{by}.
}
\details{
  This function computes the Euclidean distance from each point
  in a point pattern to its nearest neighbour (the nearest other
  point of the pattern). If \code{k} is specified, it computes the
  distance to the \code{k}th nearest neighbour.

  The function \code{nndist} is generic, with
  a method for point patterns (objects of class \code{"ppp"}),
  and a default method for coordinate vectors.

  There are also methods for line segment patterns,
  \code{\link{nndist.psp}},
  three-dimensional point patterns, \code{\link{nndist.pp3}},
  higher-dimensional point patterns, \code{\link{nndist.ppx}}
  and point patterns on a linear network, \code{\link{nndist.lpp}};
  these are described in their own help files.
  Type \code{methods(nndist)} to see all available methods.

  The method for planar point patterns \code{nndist.ppp} expects a single
  point pattern argument \code{X} and returns the vector of its
  nearest neighbour distances.

  The default method expects that \code{X} and \code{Y} will determine
  the coordinates of a set of points. Typically \code{X} and
  \code{Y} would be numeric vectors of equal length. Alternatively
  \code{Y} may be omitted and \code{X} may be a list with two components
  named \code{x} and \code{y}, or a matrix or data frame with two columns.
  
  The argument \code{k} may be a single integer, or an integer vector.
  If it is a vector, then the \eqn{k}th nearest neighbour distances are
  computed for each value of \eqn{k} specified in the vector.

  If the argument \code{by} is given, it should be a \code{factor},
  of length equal to the number of points in \code{X}.
  This factor effectively partitions \code{X} into subsets,
  each subset associated with one of the levels of \code{X}.
  The algorithm will then compute, for each point of \code{X},
  the distance to the nearest neighbour \emph{in each subset}.

  The argument \code{method} is not normally used. It is
  retained only for checking the validity of the software.
  If \code{method = "interpreted"} then the distances are
  computed using interpreted R code only. If \code{method="C"}
  (the default) then C code is used. 
  The C code is faster by two to three orders of magnitude
  and uses much less memory.

  If there is only one point (if \code{x} has length 1),
  then a nearest neighbour distance of \code{Inf} is returned.
  If there are no points (if \code{x} has length zero)
  a numeric vector of length zero is returned.

  To identify \emph{which} point is the nearest neighbour of a given point,
  use \code{\link{nnwhich}}.

  To use the nearest neighbour distances for statistical inference,
  it is often advisable to use the edge-corrected empirical distribution,
  computed by \code{\link{Gest}}.

  To find the nearest neighbour distances from one point pattern
  to another point pattern, use \code{\link{nncross}}.
}
\section{Nearest neighbours of each type}{
  If \code{X} is a multitype point pattern 
  and \code{by=marks(X)}, then the algorithm will compute,
  for each point of \code{X}, the distance to the nearest neighbour
  of each type. See the Examples.

  To find the minimum distance from \emph{any} point of type \code{i}
  to the nearest point of type \code{j}, for all combinations of \code{i} and
  \code{j}, use \code{\link{minnndist}},
  or the \R function \code{\link[stats]{aggregate}} as
  suggested in the Examples.
}
\section{Warnings}{
  An infinite or \code{NA} value is returned if the
  distance is not defined (e.g. if there is only one point
  in the point pattern).
}
\seealso{
  \code{\link{nndist.psp}},
  \code{\link{nndist.pp3}},
  \code{\link{nndist.ppx}},
  \code{\link{nndist.lpp}},
  \code{\link{pairdist}},
  \code{\link{Gest}},
  \code{\link{nnwhich}},
  \code{\link{nncross}},
  \code{\link{minnndist}}, \code{\link{maxnndist}}.
}
\examples{
   data(cells)
   # nearest neighbours
   d <- nndist(cells)

   # second nearest neighbours
   d2 <- nndist(cells, k=2)

   # first, second and third nearest
   d1to3 <- nndist(cells, k=1:3)

   x <- runif(100)
   y <- runif(100)
   d <- nndist(x, y)

   # Stienen diagram
   plot(cells \%mark\% nndist(cells), markscale=1)

   # distance to nearest neighbour of each type
   nnda <- nndist(ants, by=marks(ants)) 
   head(nnda)
   # For nest number 1, the nearest Cataglyphis nest is 87.32125 units away

   # minimum distance between each pair of types
   minnndist(ants, by=marks(ants))

   # Use of 'aggregate':
   # _minimum_ distance between each pair of types
   aggregate(nnda, by=list(from=marks(ants)), min)
   # _mean_ nearest neighbour distances
   aggregate(nnda, by=list(from=marks(ants)), mean)
   # The mean distance from a Messor nest to
   # the nearest Cataglyphis nest is 59.02549 units
}
\author{Pavel Grabarnik
  \email{pavel.grabar@issp.serpukhov.su}
  and
  \adrian.
}
\keyword{spatial}
\keyword{math}

\name{as.matrix.owin}
\alias{as.matrix.owin}
\title{Convert Pixel Image to Matrix}
\description{
  Converts a pixel image to a matrix.
}
\usage{
  \method{as.matrix}{owin}(x, ...)
}
\arguments{
  \item{x}{A window (object of class \code{"owin"}).}
  \item{\dots}{Arguments passed to \code{\link{as.mask}} to control the
    pixel resolution.}
}
\details{
  The function \code{as.matrix.owin} converts a window to a
  logical matrux.

  It first converts the window \code{x} into a binary pixel mask
  using \code{\link{as.mask}}. It then extracts the pixel entries
  as a logical matrix.

  The resulting matrix has entries that are
  \code{TRUE} if the corresponding pixel is inside the window,
  and \code{FALSE} if it is outside.
  
  The function \code{as.matrix} is generic. The function
  \code{as.matrix.owin}
  is the method for windows (objects of class \code{"owin"}).

  Use \code{\link{as.im}} to convert a window to a pixel image.
}
\value{
  A logical matrix.
}
\examples{
  m <- as.matrix(letterR)
}
\seealso{
  \code{\link{as.matrix.im}},
  \code{\link{as.im}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}

\name{pppmatching.object}
\alias{pppmatching.object} %DoNotExport
\title{Class of Point Matchings}
\description{
  A class \code{"pppmatching"} to represent a matching of two planar
  point patterns.
  Optionally includes information about the construction of the matching
  and its associated distance between the point patterns.
}
\details{
  This class represents a (possibly weighted and incomplete) matching
  between two planar point patterns (objects of class \code{"ppp"}).

  A matching can be thought of as a bipartite weighted graph where
  the vertices are given by the two point patterns and edges of positive
  weights are drawn each time a point of the first point pattern is
  "matched" with a point of the second point pattern.

  If \code{m} is an object of type \code{pppmatching}, it contains the
  following elements
  \tabular{ll}{
    \code{pp1, pp2} \tab the two point patterns to be matched (vertices) \cr
    \code{matrix} \tab a matrix specifying which points are matched \cr
                  \tab and with what weights (edges) \cr
    \code{type} \tab (optional) a character string for the type of \cr
                 \tab the matching (one of \code{"spa"}, \code{"ace"} or \code{"mat"}) \cr
    \code{cutoff} \tab (optional) cutoff value for interpoint distances \cr
    \code{q} \tab (optional) the order for taking averages of \cr
             \tab interpoint distances \cr
    \code{distance} \tab (optional) the distance associated with the matching
  }

  The element \code{matrix} is a "generalized adjacency matrix".
  The numbers of rows
  and columns match the cardinalities of the first and second point
  patterns, respectively. The \code{[i,j]}-th entry is positive if
  the \code{i}-th point of \code{X} and the \code{j}-th point of
  \code{Y} are matched (zero otherwise) and its value then gives
  the corresponding weight of the match. For an unweighted matching
  all the weights are set to \eqn{1}.

  The optional elements are for saving details about matchings in the context of
  optimal point matching techniques. \code{type} can be one of \code{"spa"} (for
  "subpattern assignment"), \code{"ace"} (for "assignment only if
  cardinalities differ") or \code{"mat"} (for "mass transfer"). \code{cutoff}
  is a positive numerical value that specifies the maximal interpoint distance and
  \code{q} is a value in \eqn{[1,\infty]}{[1,Inf]} that gives the order of the average
  applied to the interpoint distances. See the help files for \code{\link{pppdist}}
  and \code{\link{matchingdist}} for detailed information about these elements.
  
  Objects of class \code{"pppmatching"} may be created by the function 
  \code{\link{pppmatching}}, and are most commonly obtained as output of the
  function \code{\link{pppdist}}. There are methods \code{plot}, \code{print} and
  \code{summary} for this class.
}
\author{
  Dominic Schuhmacher
  \email{dominic.schuhmacher@stat.unibe.ch}
  \url{http://www.dominic.schuhmacher.name}
}
\seealso{
  \code{\link{matchingdist}},
  \code{\link{pppmatching}},
  \code{\link{plot.pppmatching}}
}
\examples{
  # a random complete unweighted matching
  X <- runifpoint(10)
  Y <- runifpoint(10)
  am <- r2dtable(1, rep(1,10), rep(1,10))[[1]]
        # generates a random permutation matrix
  m <- pppmatching(X, Y, am)
  summary(m)
  m$matrix
  \dontrun{
    plot(m)
  }

  # an optimal complete unweighted matching
  m2 <- pppdist(X,Y)
  summary(m2)
  m2$matrix
  \dontrun{
    plot(m2)
  }
}
\keyword{spatial}
\keyword{attribute}

\name{methods.pp3}
\Rdversion{1.1}
\alias{methods.pp3} %DoNotExport
\alias{print.pp3}
\alias{summary.pp3}
\alias{print.summary.pp3}
\alias{unitname.pp3}
\alias{unitname<-.pp3}
\title{
  Methods for three-dimensional point patterns
}
\description{
  Methods for class \code{"pp3"}.
}
\usage{
  \method{print}{pp3}(x, ...)
  \method{print}{summary.pp3}(x, ...)
  \method{summary}{pp3}(object, ...)
  \method{unitname}{pp3}(x)
  \method{unitname}{pp3}(x) <- value
}
\arguments{
  \item{x,object}{
    Object of class \code{"pp3"}.
  }
  \item{\dots}{
    Ignored.
  }
  \item{value}{
    Name of the unit of length. See \code{\link{unitname}}.
  }
}
\details{
  These are methods for the generic functions
  \code{\link{print}}, \code{\link{summary}},
  \code{\link{unitname}} and  \code{\link{unitname<-}}
  for the class \code{"pp3"} of three-dimensional point patterns.

  The \code{print} and \code{summary} methods print a description
  of the point pattern.

  The \code{unitname} method extracts the name of the unit of
  length in which the point coordinates are expressed.
  The \code{unitname<-} method assigns the name of the unit of length.
}
\value{
  For \code{print.pp3} the value is \code{NULL}.
  For \code{unitname.pp3} an object of class \code{"units"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{pp3}},
  \code{\link{print}},
  \code{\link{unitname}}
  \code{\link{unitname<-}}
}
\examples{
   X <- pp3(runif(42),runif(42),runif(42), box3(c(0,1), unitname="mm"))
   X
   unitname(X)
   unitname(X) <- c("foot", "feet")
   summary(X)
}
\keyword{spatial}
\keyword{methods}

\name{bdist.pixels}
\alias{bdist.pixels}
\title{Distance to Boundary of Window}
\description{
  Computes the distances 
  from each pixel in a window to the boundary of the window.
}
\usage{
 bdist.pixels(w, \dots, style="image", method=c("C", "interpreted"))
}
\arguments{
  \item{w}{A window (object of class \code{"owin"}).}
  \item{\dots}{Arguments passed to \code{\link{as.mask}} to determine
    the pixel resolution.}
  \item{style}{Character string determining the format of
    the output: either \code{"matrix"}, \code{"coords"} or
    \code{"image"}.
  }
  \item{method}{Choice of algorithm to use when \code{w} is polygonal.}
}
\value{
  If \code{style="image"}, a pixel image (object of class \code{"im"})
  containing the distances from each pixel in the image raster
  to the boundary of the window.

  If \code{style="matrix"},
  a matrix giving the distances from each pixel in the image raster
  to the boundary of the window. Rows of this matrix correspond to
  the \eqn{y} coordinate and columns to the \eqn{x} coordinate.
  
  If \code{style="coords"}, a list with three components
  \code{x,y,z}, where \code{x,y} are vectors of length \eqn{m,n}
  giving the \eqn{x} and \eqn{y} coordinates respectively,
  and \code{z} is an \eqn{m \times n}{m x n} matrix such that
  \code{z[i,j]} is the distance from \code{(x[i],y[j])} to the
  boundary of the window. Rows of this matrix correspond to the
  \eqn{x} coordinate and columns to the \eqn{y} coordinate.
  This result can be plotted with \code{persp}, \code{image}
  or \code{contour}.
}
\details{
  This function computes, for each pixel \eqn{u}
  in the window \code{w}, the shortest distance
  \eqn{d(u, W^c)}{dist(u, W')} from \eqn{u}
  to the boundary of \eqn{W}.

  If the window is a binary mask then the distance from each pixel
  to the boundary is computed using the distance transform algorithm
  \code{\link{distmap.owin}}. The result is equivalent to
  \code{distmap(W, invert=TRUE)}.

  If the window is a rectangle or a polygonal region,
  the grid of pixels is determined by the arguments \code{"\dots"} 
  passed to \code{\link{as.mask}}. The distance from each pixel to the
  boundary is calculated exactly, using analytic geometry.
  This is slower but more accurate than in the case of a binary mask.

  For software testing purposes, there are two implementations
  available when \code{w} is a polygon: the default is \code{method="C"}
  which is much faster than \code{method="interpreted"}.
}
\seealso{
  \code{\link{owin.object}},
  \code{\link{erosion}},
  \code{\link{bdist.points}},
  \code{\link{bdist.tiles}},
  \code{\link{distmap.owin}}.
}
\examples{
  u <- owin(c(0,1),c(0,1))
  d <- bdist.pixels(u, eps=0.01)
  image(d)
  d <- bdist.pixels(u, eps=0.01, style="matrix")
  mean(d >= 0.1)
  # value is approx (1 - 2 * 0.1)^2 = 0.64
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{Extract.anylist}
\alias{[.anylist}
\alias{[<-.anylist}
\title{Extract or Replace Subset of a List of Things}
\description{
  Extract or replace a subset of a list of things.
}
\usage{
  \method{[}{anylist}(x, i, \dots)

  \method{[}{anylist}(x, i) <- value
}
\arguments{
  \item{x}{
    An object of class \code{"anylist"} representing a list of things.
  }
  \item{i}{
    Subset index. Any valid subset index in the usual \R sense.
  }
  \item{value}{
    Replacement value for the subset. 
  }
  \item{\dots}{Ignored.}
}
\value{
  Another object of class \code{"anylist"}.
}
\details{
  These are the methods for extracting and replacing subsets 
  for the class \code{"anylist"}.

  The argument \code{x} should be an object of class \code{"anylist"}
  representing a list of things. See \code{\link{anylist}}.

  The method replaces a designated
  subset of \code{x}, and returns an object of class \code{"anylist"}.
}
\seealso{
  \code{\link{anylist}},
  \code{\link{plot.anylist}},
  \code{\link{summary.anylist}}
}
\examples{
   x <- anylist(A=runif(10), B=runif(10), C=runif(10))
   x[1] <- list(A=rnorm(10))
 }
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{list}
\keyword{manip}

\name{incircle}
\alias{incircle}
\alias{inradius}
\title{Find Largest Circle Inside Window}
\description{
  Find the largest circle contained in a given window.
}
\usage{
incircle(W)

inradius(W)
}
\arguments{
  \item{W}{A window (object of class \code{"owin"}).}
}
\details{
  Given a window \code{W} of any type and shape, 
  the function \code{incircle} determines the largest circle
  that is contained inside \code{W}, while \code{inradius} computes its
  radius only.

  For non-rectangular windows, the incircle is computed approximately
  by finding the maximum of the distance map (see
  \code{\link{distmap}}) of the complement of the window.
}
\value{
  The result of \code{incircle} is 
  a list with entries \code{x,y,r} giving the location \code{(x,y)}
  and radius \code{r} of the incircle.

  The result of \code{inradius} is the numerical value of radius.
}
\seealso{
  \code{\link{centroid.owin}}
}
\examples{
  W <- square(1)
  Wc <- incircle(W)
  plot(W)
  plot(disc(Wc$r, c(Wc$x, Wc$y)), add=TRUE)

  plot(letterR)
  Rc <- incircle(letterR)
  plot(disc(Rc$r, c(Rc$x, Rc$y)), add=TRUE)

  W <- as.mask(letterR)
  plot(W)
  Rc <- incircle(W)
  plot(disc(Rc$r, c(Rc$x, Rc$y)), add=TRUE)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{quadscheme.logi}
\alias{quadscheme.logi}
\title{Generate a Logistic Regression Quadrature Scheme from a Point Pattern}
\description{
  Generates a logistic regression quadrature scheme (an object of class
  \code{"logiquad"} inheriting from \code{"quad"})
  from point patterns of data and dummy points.
}
\usage{
 quadscheme.logi(data, dummy, dummytype = "stratrand",
                 nd = NULL, mark.repeat = FALSE, \dots)
}
\arguments{
  \item{data}{
    The observed data point pattern.
    An object of class \code{"ppp"}
    or in a format recognised by \code{\link{as.ppp}()}
  }
  \item{dummy}{
    The pattern of dummy points for the quadrature. 
    An object of class \code{"ppp"}
    or in a format recognised by \code{\link{as.ppp}()}.
    If missing a sensible default is generated.
  }
  \item{dummytype}{
    The name of the type of dummy points to use when \code{"dummy"}
    is missing. Currently available
    options are: \code{"stratrand"} (default), \code{"binomial"},
    \code{"poisson"}, \code{"grid"} and \code{"transgrid"}.
  }
  \item{nd}{
    Integer, or integer vector of length 2 controlling the intensity of
    dummy points when \code{"dummy"} is missing.
  }
  \item{mark.repeat}{
    Repeating the dummy points for each level of a marked data pattern
    when \code{"dummy"} is missing. (See details.)
  }
  \item{\dots}{
    Ignored.
  }
} 
\value{
  An object of class \code{"logiquad"} inheriting from \code{"quad"}
  describing the quadrature scheme 
  (data points, dummy points, and quadrature weights)
  suitable as the argument \code{Q} of the function \code{\link{ppm}()} for
  fitting a point process model.

  The quadrature scheme can be inspected using the
  \code{print} and \code{plot} methods for objects
  of class \code{"quad"}. 
}
\details{
  This is the primary method for producing a quadrature schemes
  for use by \code{\link{ppm}} when the logistic regression
  approximation (Baddeley et al. 2013) to the pseudolikelihood of the
  model is applied (i.e. when \code{method="logi"} in \code{\link{ppm}}).
  
  The function \code{\link{ppm}} fits a point process model to an
  observed point pattern. When used with the option \code{method="logi"}
  it requires a quadrature scheme consisting of 
  the original data point pattern and an additional pattern of dummy points.
  Such quadrature schemes are represented by objects of class
  \code{"logiquad"}.

  Quadrature schemes are created by the function
  \code{quadscheme.logi}.
  The arguments \code{data} and \code{dummy} specify the data and dummy
  points, respectively. There is a sensible default for the dummy 
  points.
  Alternatively the dummy points
  may be specified arbitrarily and given in any format recognised by
  \code{\link{as.ppp}}.
 
  The quadrature region is the region over which we are
  integrating, and approximating integrals by finite sums.
  If \code{dummy} is a point pattern object (class \code{"ppp"})
  then the quadrature region is taken to be \code{Window(dummy)}.
  If \code{dummy} is just a list of \eqn{x, y} coordinates
  then the quadrature region defaults to the observation window
  of the data pattern, \code{Window(data)}.

  If \code{dummy} is missing, then a pattern of dummy points will be
  generated, taking account of the optional arguments \code{dummytype},
  \code{nd}, and \code{mark.repeat}.

  The currently accepted values for \code{dummytype} are:
  \itemize{
  \item \code{"grid"} where the frame of the window
  is divided into a \code{nd * nd} or \code{nd[1] * nd[2]} regular grid
  of tiles and the centers constitutes the dummy points.
  \item \code{"transgrid"} where a regular grid as above is translated
  by a random vector.
  \item \code{"stratrand"} where each point of a regular grid as above
  is randomly translated within its tile.
  \item \code{"binomial"} where \code{nd * nd} or \code{nd[1] * nd[2]}
  points are generated uniformly in the frame of the
  window.
  \code{"poisson"} where a homogeneous Poisson point process with
  intensity \code{nd * nd} or \code{nd[1] * nd[2]} is
  generated within the frame of observation window.
}
Then if the window is not rectangular, any dummy points
lying outside it are deleted. 

If \code{data} is a multitype point pattern the dummy points should also
be marked (with the same levels of the marks as \code{data}). If
\code{dummy} is missing and the dummy pattern is generated by
\code{quadscheme.logi} the default behaviour is to attach a uniformly
distributed mark (from the levels of the marks) to each dummy
point. Alternatively, if \code{mark.repeat=TRUE} each dummy point is
repeated as many times as there are levels of the marks with a distinct
mark value attached to it.

Finally, each point (data and dummy) is assigned the weight 1. The
weights are never used and only appear to be compatible with the class
\code{"quad"} from which the \code{"logiquad"} object inherits.
}
\references{
  Baddeley, A., Coeurjolly, J.-F., Rubak, E. and Waagepetersen, R. (2014)
  Logistic regression for spatial Gibbs point processes.
  \emph{Biometrika} \bold{101} (2) 377--392.
}
\seealso{
  \code{\link{ppm}},
  \code{\link{as.ppp}}
}
\examples{
  data(simdat)

  Q <- quadscheme.logi(simdat)
}
\author{\adrian
  
  ,
  \rolf 
  and \ege
  .
}
\keyword{spatial}
\keyword{datagen}

\name{plot.psp}
\alias{plot.psp}
\title{plot a Spatial Line Segment Pattern}
\description{
  Plot a two-dimensional line segment pattern
}
\usage{
  \method{plot}{psp}(x, \dots, main, add=FALSE,
                     show.all=!add, show.window=show.all,
                     which.marks=1,
                     style=c("colour", "width", "none"),
                     col=NULL, 
                     ribbon=show.all,
                     ribsep=0.15, ribwid=0.05, ribn=1024,
                     do.plot=TRUE)
}
\arguments{
  \item{x}{
    The line segment pattern to be plotted.
    An object of class \code{"psp"},
    or data which can be converted into 
    this format by \code{\link{as.psp}()}.
  }
  \item{\dots}{
    extra arguments that will be passed to the plotting functions
    \code{\link{segments}} (to plot the segments)
    and 
    \code{\link{plot.owin}} (to plot the observation window).
  }
  \item{main}{
    Character string giving a title for the plot.
  }
  \item{add}{
    Logical. If \code{TRUE}, the current plot is not erased;
    the segments are plotted on top of the
    current plot, and the window is not plotted (by default).
  }
  \item{show.all}{
    Logical value specifying whether to plot everything
    including the window, main title, and colour ribbon.
  }
  \item{show.window}{
    Logical value specifying whether to plot the window.
  }
  \item{which.marks}{
    Index determining which column of marks to use,
    if the marks of \code{x} are a data frame.
    A character string or an integer.
    Defaults to \code{1} indicating the first column of marks.
  }
  \item{style}{
    Character string specifying how to represent the
    mark value of each segment. If \code{style="colour"} (the default)
    segments are coloured according to their mark value.
    If \code{style="width"}, segments are drawn with a width
    proportional to their mark value.
    If \code{style="none"} the mark values are ignored.
  }
  \item{col}{
    Colour information.
    If \code{style="width"} or \code{style="none"},
    then \code{col} should be a single value,
    interpretable as a colour; the line segments will be plotted
    using this colour.
    If \code{style="colour"} and \code{x} has marks,
    then the mark values will be mapped
    to colours using the information in \code{col}, which should be a 
    colour map (object of class \code{"colourmap"})
    or a vector of colour values.
  }
  \item{ribbon}{
    Logical flag indicating whether to display a ribbon
    showing the colour map (in which mark values are associated with
    colours) when \code{style="colour"}.
  }
  \item{ribsep}{
    Factor controlling the space between the ribbon and the image.
  }
  \item{ribwid}{
    Factor controlling the width of the ribbon.
  }
  \item{ribn}{
    Number of different values to display in the ribbon.
  }
  \item{do.plot}{
    Logical value indicating whether to actually perform the plot.
  }
}
\value{
  (Invisibly) a colour map object specifying the association between
  marks and colours, if any. The return value also has an attribute
  \code{"bbox"} giving a bounding box for the plot.
}
\details{
  This is the \code{plot} method for 
  line segment pattern datasets (of class \code{"psp"},
  see \code{\link{psp.object}}).
  It plots both the observation window \code{Window(x)}
  and the line segments themselves.
  
  Plotting of the window \code{Window(x)} is performed by
  \code{\link{plot.owin}}. This plot may be modified 
  through the \code{...} arguments. 

  Plotting of the segments themselves is performed
  by the standard R function \code{\link{segments}}.
  Its plotting behaviour may also be modified through the \code{...}
  arguments.

  If the segments do not have marks (i.e. if \code{marks(x) = NULL})
  then 

  There are three different styles of plotting
  which apply when the segments have marks
  (i.e. when \code{marks(x)} is not null):
  \describe{
    \item{\code{style="colour"} (the default):}{
      Segments are plotted with different colours depending on their
      mark values.
      The colour map, associating mark values with colours,
      is determined by the argument \code{col}. The colour map will be
      displayed as a vertical colour ribbon to the right of the plot, if
      \code{ribbon=TRUE} (the default).
    }
    \item{\code{style="width"}:}{
      Segments are plotted with different widths depending on their
      mark values.
      The width map, associating mark values with line widths,
      is determined by \code{leg.scale} (see \code{\link{plot.linim}}).
      The width map will be displayed as a vertical stack of lines
      to the right of the plot, if \code{legend=TRUE} (the default).
    }
    \item{\code{style="none"}:}{
      Mark information is ignored.
    }
  }
  If \code{marks(x)} is a data frame, the default is to use the first
  column of \code{marks(x)} to determine the colours. To specify another
  column, use the argument \code{which.marks}.
}
\seealso{
  \code{\link{psp.object}},
  \code{\link{plot}},
  \code{\link{par}},
  \code{\link{plot.owin}},
  \code{\link{text.psp}},
  \code{\link{symbols}}
}
\examples{
  X <- psp(runif(20), runif(20), runif(20), runif(20), window=owin())
  plot(X)
  plot(X, lwd=3)
  lettuce <- sample(letters[1:4], 20, replace=TRUE)
  marks(X) <- data.frame(A=1:20, B=factor(lettuce))
  plot(X)
  plot(X, which.marks="B")
  plot(X, style="width", col="grey")
}

\author{
  \adrian
  and \rolf
}
\keyword{spatial}
 
 \keyword{hplot}

\name{solutionset}
\alias{solutionset}
\title{Evaluate Logical Expression Involving Pixel Images and Return
  Region Where Expression is True}
\description{
  Given a logical expression involving one or more pixel images,
  find all pixels where the expression is true,
  and assemble these pixels into a window.
}
\usage{
  solutionset(\dots, envir)
}
\arguments{
  \item{\dots}{An expression in the \R language, involving one or more
    pixel images.}
  \item{envir}{Optional. The environment in which to evaluate the
    expression.}
}
\details{
  Given a logical expression involving one or more pixel images,
  this function will find all pixels where the expression is true,
  and assemble these pixels into a spatial window.

  Pixel images in \code{spatstat}
  are represented by objects of class \code{"im"}
  (see \code{\link{im.object}}). These are essentially matrices of
  pixel values, with extra attributes recording the pixel dimensions,
  etc.

  Suppose \code{X} is a pixel image. Then \code{solutionset(abs(X) > 3)}
  will find all the pixels in \code{X} for which the pixel value
  is greater than 3 in absolute value, and return a window containing
  all these pixels.

  If \code{X} and \code{Y} are two pixel images,
  \code{solutionset(X > Y)} will find all pixels for which the
  pixel value of \code{X} is greater than the corresponding pixel value
  of \code{Y}, and return a window containing these pixels.
  
  In general, \code{\dots} can be any logical expression involving
  pixel images.

  The code first tries to evaluate the expression using
  \code{\link{eval.im}}.
  This is successful if the expression involves only
  (a) the \emph{names} of pixel images, (b) scalar
  constants, and (c) functions which are vectorised.
  There must be at least one pixel image in the expression.
  The expression \code{expr} must be vectorised.
  See the Examples.

  If this is unsuccessful, the code then tries to evaluate the
  expression using pixel arithmetic. This is successful if all the
  arithmetic operations in the expression are listed
  in \code{\link{Math.im}}. 
}
\value{
  A spatial window
  (object of class \code{"owin"}, see \code{\link{owin.object}}).
}
\seealso{
  \code{\link{im.object}},
  \code{\link{owin.object}},
  \code{\link{eval.im}},
  \code{\link{levelset}}
}
\examples{
  # test images
  X <- as.im(function(x,y) { x^2 - y^2 }, unit.square())
  Y <- as.im(function(x,y) { 3 * x + y  - 1}, unit.square())

  W <- solutionset(abs(X) > 0.1)
  W <- solutionset(X > Y)
  W <- solutionset(X + Y >= 1)

  area(solutionset(X < Y))

  solutionset(density(cells) > 20)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{programming}
\keyword{manip}

\name{project2segment}
\alias{project2segment}
\title{Move Point To Nearest Line}
\description{
  Given a point pattern and a line segment pattern,
  this function moves each point to the closest location
  on a line segment.
}
\usage{
project2segment(X, Y)
}
\arguments{
  \item{X}{A point pattern (object of class \code{"ppp"}).}
  \item{Y}{A line segment pattern (object of class \code{"psp"}).}
}
\details{
  For each point \code{x} in the point pattern \code{X}, this function
  finds the closest line segment \code{y} in the line segment pattern
  \code{Y}. It then `projects' the point \code{x} onto the line segment
  \code{y} by finding the position \code{z} along \code{y}
  which is closest to \code{x}. This position \code{z} is returned,
  along with supplementary information.
}
\value{
  A list with the following components. Each component has length equal to
  the number of points in \code{X}, and its entries correspond to the
  points of \code{X}.
  
  \item{Xproj }{
    Point pattern (object of class \code{"ppp"}
    containing the projected points.
  }
  \item{mapXY }{
    Integer vector identifying the nearest segment to each point.
  }
  \item{d}{
    Numeric vector of distances from each point of \code{X} to
    the corresponding projected point.
  }
  \item{tp}{
    Numeric vector giving the scaled parametric coordinate 
    \eqn{0 \le t_p \le 1}{0 <= tp <= 1} of the position of the projected point
    along the segment.
  }

  For example suppose \code{mapXY[2] = 5} and \code{tp[2] = 0.33}.
  Then \code{Y[5]} is the line segment lying closest to \code{X[2]}.
  The projection of the point \code{X[2]} onto the segment \code{Y[5]}
  is the point \code{Xproj[2]}, which lies one-third of the way
  between the first and second endpoints of the line segment \code{Y[5]}.

  }
\author{
  \adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{nearestsegment}} for a faster way to determine
  which segment is closest to each point.
}
\examples{
  X <- rstrat(square(1), 5)
  Y <- as.psp(matrix(runif(20), 5, 4), window=owin())
  plot(Y, lwd=3, col="green")
  plot(X, add=TRUE, col="red", pch=16)
  v <- project2segment(X,Y)
  Xproj <- v$Xproj
  plot(Xproj, add=TRUE, pch=16)
  arrows(X$x, X$y, Xproj$x, Xproj$y, angle=10, length=0.15, col="red")
}
\keyword{spatial}
\keyword{math}

\name{rjitter}
\alias{rjitter}
\title{Random Perturbation of a Point Pattern}
\description{
  Applies independent random displacements to each point in a point pattern.
}
\usage{
rjitter(X, radius, retry=TRUE, giveup = 10000, \dots, nsim=1, drop=TRUE)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"}).
  }
  \item{radius}{
    Scale of perturbations. A positive numerical value.
    The displacement vectors will be uniformly
    distributed in a circle of this radius.
    There is a sensible default.
  }
  \item{retry}{
    What to do when a perturbed point lies outside the window
    of the original point pattern. If \code{retry=FALSE},
    the point will be lost; if \code{retry=TRUE},
    the algorithm will try again.
  }
  \item{giveup}{
    Maximum number of unsuccessful attempts.
  }
  \item{\dots}{Ignored.}
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  A point pattern (an object of class \code{"ppp"})
  if \code{nsim=1}, or a list of point patterns if \code{nsim > 1},
  in the same window as \code{X}.
}
\details{
  Each of the points in the point pattern \code{X} is subjected to
  an independent random displacement. The displacement vectors are
  uniformly distributed in a circle of radius \code{radius}.

  If a displaced point lies outside the window, then if
  \code{retry=FALSE} the point will be lost. 

  However if \code{retry=TRUE}, the algorithm will try again: each time a
  perturbed point lies outside the window, the algorithm will reject it and
  generate another proposed perturbation of the original point,
  until one lies inside the window, or until \code{giveup} unsuccessful
  attempts have been made. In the latter case, any unresolved points
  will be included without any perturbation. The return value will
  always be a point pattern with the same number of points as \code{X}.
}
\examples{
   X <- rsyst(owin(), 10, 10)
   Y <- rjitter(X, 0.02)
   plot(Y)
   Z <- rjitter(X)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{plot.pppmatching}
\alias{plot.pppmatching}
\title{
  Plot a Point Matching
}
\description{
  Plot an object of class \code{"pppmatching"} which represents
  a matching of two planar point patterns.
}
\usage{
\method{plot}{pppmatching}(x, addmatch = NULL, main = NULL, \dots, adjust = 1)
}
\arguments{
  \item{x}{
    Point pattern matching object (class \code{"pppmatching"})
    to be plotted.
  }
  \item{addmatch}{
    Optional. A matrix indicating additional pairs of points that
    should be matched. See Details.
  }
  \item{main}{
    Main title for the plot. 
  }
  \item{\dots}{
    Additional arguments passed to other plot methods.
  }
  \item{adjust}{
    Adjustment factor for the widths of line segments.
    A positive number.
  }
}
\details{
  The object \code{x} represents a matching found between
  two point patterns \code{X} and \code{Y}. The matching may be
  incomplete. See \code{\link{pppmatching.object}} for further
  description.

  This function plots the matching by drawing the two point patterns
  \code{X} and \code{Y} as red and blue dots respectively, and drawing
  line segments between each pair of matched points. The width of the
  line segments is proportional to the strength of matching. The
  proportionality constant can be adjusted using the argument
  \code{adjust}.

  Additional graphics arguments \code{\dots} control the
  plotting of the window (and are passed to \code{\link{plot.owin}})
  and the plotting of the line segments 
  (and are passed to \code{\link{plot.psp}}, \code{\link{plot.linim}}
  and ultimately to the base graphics function
  \code{\link[graphics]{polygon}}).

  The argument \code{addmatch} is for use mainly by developers to
  study algorithms which update the matching.
  If \code{addmatch} is given, it should be a matrix with dimensions
  \code{npoints(X) * npoints(Y)}. If \code{addmatch[i,j] > 0} then
  a light grey line segment will be drawn between \code{X[i]} and \code{Y[j}.
}
\value{
  Null.
}
\author{
  Dominic Schuhmacher and \adrian.
}
\seealso{
  \code{\link{pppmatching.object}}
}
\examples{
  X <- runifpoint(7)
  Y <- runifpoint(7)
  am <- r2dtable(1, rep(10,7), rep(10,7))[[1]]/10
  m2 <- pppmatching(X, Y, am)
  plot(m2, adjust=0.3)
}
\keyword{spatial}
\keyword{hplot}

\name{plot.onearrow}
\alias{plot.onearrow}
\title{Plot an Arrow}
\description{Plots an object of class \code{"onearrow"}.}
\usage{
 \method{plot}{onearrow}(x, \dots,
  add = FALSE, main = "",
  retract = 0.05, headfraction = 0.25, headangle = 12, headnick = 0.1,
  col.head = NA, lwd.head = lwd, lwd = 1, col = 1,
  zap = FALSE, zapfraction = 0.07,
  pch = 1, cex = 1, do.plot = TRUE, do.points = FALSE, show.all = !add)
}
\arguments{
  \item{x}{
    Object of class \code{"onearrow"} to be plotted.
    This object is created by the command \code{\link{onearrow}}.
  }
  \item{\dots}{
    Additional graphics arguments passed to
    \code{\link[graphics]{segments}} to control the appearance of the line.
  }
  \item{add}{Logical value indicating whether to add graphics to the
    existing plot (\code{add=TRUE}) or to start a new plot
    (\code{add=FALSE}).
  }
  \item{main}{Main title for the plot.}
  \item{retract}{
    Fraction of length of arrow to remove at each end.
  }
  \item{headfraction}{
    Length of arrow head as a fraction of overall length of arrow.
  }
  \item{headangle}{
    Angle (in degrees) between the outer edge of the arrow head
    and the shaft of the arrow.
  }
  \item{headnick}{
    Size of the nick in the trailing edge of the arrow head
    as a fraction of length of arrow head. 
  }
  \item{col.head,lwd.head}{
    Colour and line style of the filled arrow head.
  }
  \item{col,lwd}{
    Colour and line style of the arrow shaft.
  }
  \item{zap}{
    Logical value indicating whether the arrow should
    include a Z-shaped (lightning-bolt) feature in the
    middle of the shaft.
  }
  \item{zapfraction}{
   Size of Z-shaped deviation as a fraction of total arrow length.
  }
  \item{pch,cex}{
    Plot character and character size for the two end points of the arrow,
    if \code{do.points=TRUE}.
  }
  \item{do.plot}{
    Logical. Whether to actually perform the plot.
  }
  \item{do.points}{
    Logical. Whether to display the two end points of the arrow as well.
  }
  \item{show.all}{
    Internal use only.
  }
}
\details{
  The argument \code{x} should be an object of class \code{"onearrow"}
  created by the command \code{\link{onearrow}}. 
}
\value{
  A window (class \code{"owin"}) enclosing the plotted graphics.
}
\examples{
  oa <- onearrow(cells[c(1, 42)])
  oa
  plot(oa)
  plot(oa, zap=TRUE, do.points=TRUE, col.head="pink", col="red")
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{onearrow}},
  \code{\link{yardstick}}
}
\keyword{spatial}
\keyword{hplot}

\name{runifpointOnLines}
\alias{runifpointOnLines}
\title{Generate N Uniform Random Points On Line Segments}
\description{
  Given a line segment pattern, generate a random point pattern
  consisting of \code{n} points uniformly distributed on the
  line segments.
}
\usage{
runifpointOnLines(n, L, nsim=1, drop=TRUE)
}
\arguments{
  \item{n}{Number of points to generate.}
  \item{L}{Line segment pattern (object of class \code{"psp"})
    on which the points should lie.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\details{
  This command generates a point pattern consisting of \code{n}
  independent random points, each point uniformly distributed on
  the line segment pattern. This means that, for each random point,
  \itemize{
    \item the
    probability of falling on a particular segment is proportional to the
    length of the segment; and
    \item given that the point falls on a particular segment,
    it has uniform probability density along that segment.
  }
  If \code{n} is a single integer, the result is an unmarked point
  pattern containing \code{n} points.
  If \code{n} is a vector of integers, the result is a marked point
  pattern, with \code{m} different types of points, where
  \code{m = length(n)},
  in which there are \code{n[j]} points of type \code{j}.
}
\value{
  If \code{nsim = 1}, 
  a point pattern (object of class \code{"ppp"}) with the same
  window as \code{L}.
  If \code{nsim > 1}, a list of point patterns.
}
\seealso{
  \code{\link{psp}},
  \code{\link{ppp}},
  \code{\link{pointsOnLines}},
  \code{\link{runifpoint}}
}
\examples{
  X <- psp(runif(10), runif(10), runif(10), runif(10),  window=owin())
  Y <- runifpointOnLines(20, X)
  plot(X, main="")
  plot(Y, add=TRUE)
  Z <- runifpointOnLines(c(5,5), X)
}
\author{\adrian
  and \rolf
}
\keyword{spatial}
\keyword{datagen}

\name{summary.listof}
\alias{summary.listof}
\title{Summary of a List of Things}
\description{
  Prints a useful summary of each item in a list of things.
}
\usage{
 \method{summary}{listof}(object, \dots)
}
\arguments{
  \item{object}{
    An object of class \code{"listof"}.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  This is a method for the generic function \code{\link{summary}}.

  An object of the class \code{"listof"} is effectively a list
  of things which are all of the same class.

  This function extracts a useful summary of each of the items in the list.
}
\seealso{
  \code{\link{summary}},
  \code{\link{plot.listof}}
}
\examples{
  x <- list(A=runif(10), B=runif(10), C=runif(10))
  class(x) <- c("listof", class(x))
  summary(x)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}

\name{tiles}
\alias{tiles}
\title{Extract List of Tiles in a Tessellation}
\description{
  Extracts a list of the tiles that make up a tessellation.
}
\usage{
tiles(x)
}
\arguments{
  \item{x}{A tessellation (object of class \code{"tess"}).}
}
\details{
  A tessellation is a collection of disjoint spatial regions
  (called \emph{tiles}) that fit together to form a larger spatial
  region. See \code{\link{tess}}.

  The tiles that make up the tessellation \code{x}
  are returned in a list.
}
\value{
  A list of windows (objects of class \code{"owin"}).
}
\seealso{
  \code{\link{tess}},
  \code{\link{tilenames}},
  \code{\link{tile.areas}},
  \code{\link{tiles.empty}}
}
\examples{
  A <- tess(xgrid=0:2,ygrid=0:2)
  tiles(A)
  v <- as.im(function(x,y){factor(round(x^2 + y^2))}, W=owin())
  E <- tess(image=v)
  tiles(E)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{dkernel}
\alias{dkernel}
\alias{pkernel}
\alias{qkernel}
\alias{rkernel}
\title{Kernel distributions and random generation}
\description{Density, distribution function, quantile function and random
  generation for several distributions used in kernel estimation
  for numerical data.
}
\usage{
dkernel(x, kernel = "gaussian", mean = 0, sd = 1)
pkernel(q, kernel = "gaussian", mean = 0, sd = 1, lower.tail = TRUE)
qkernel(p, kernel = "gaussian", mean = 0, sd = 1, lower.tail = TRUE)
rkernel(n, kernel = "gaussian", mean = 0, sd = 1)
}
\arguments{
  \item{x, q}{Vector of quantiles.}
  \item{p}{Vector of probabilities.}
  \item{kernel}{
    String name of the kernel.
    Options are
    \code{"gaussian"}, \code{"rectangular"},
    \code{"triangular"},
    \code{"epanechnikov"},
    \code{"biweight"},
    \code{"cosine"} and \code{"optcosine"}.
    (Partial matching is used).
  }
  \item{n}{Number of observations.}
  \item{mean}{Mean of distribution.}
  \item{sd}{Standard deviation of distribution.}
  \item{lower.tail}{logical; if \code{TRUE} (the default),
    then probabilities are \eqn{P(X \le x)}{P[X \le x]},
    otherwise, \eqn{P(X > x)}.
  }
}
\details{
  These functions give the
  probability density, cumulative distribution function,
  quantile function and random generation for several
  distributions used in kernel estimation for one-dimensional
  (numerical) data.

  The available kernels are those used in \code{\link[stats]{density.default}},
  namely \code{"gaussian"}, \code{"rectangular"},
    \code{"triangular"},
    \code{"epanechnikov"},
    \code{"biweight"},
    \code{"cosine"} and \code{"optcosine"}.
    For more information about these kernels,
    see \code{\link[stats]{density.default}}. 
  
  \code{dkernel} gives the probability density,
  \code{pkernel} gives the cumulative distribution function,
  \code{qkernel} gives the quantile function,
  and \code{rkernel} generates random deviates.
}
\value{
  A numeric vector.
  For \code{dkernel}, a vector of the same length as \code{x}
  containing the corresponding values of the probability density.
  For \code{pkernel}, a vector of the same length as \code{x}
  containing the corresponding values of the cumulative distribution function.
  For \code{qkernel}, a vector of the same length as \code{p}
  containing the corresponding quantiles.
  For \code{rkernel}, a vector of length \code{n}
  containing randomly generated values.
}
\examples{
  x <- seq(-3,3,length=100)
  plot(x, dkernel(x, "epa"), type="l",
           main=c("Epanechnikov kernel", "probability density"))
  plot(x, pkernel(x, "opt"), type="l",
           main=c("OptCosine kernel", "cumulative distribution function"))
  p <- seq(0,1, length=256)
  plot(p, qkernel(p, "biw"), type="l",
           main=c("Biweight kernel", "cumulative distribution function"))
  y <- rkernel(100, "tri")
  hist(y, main="Random variates from triangular density")
  rug(y)
}
\seealso{
  \code{\link[stats]{density.default}},
  \code{\link{kernel.factor}}
}
\author{\adrian
  \email{adrian@maths.uwa.edu.au}
  
  and Martin Hazelton
}
\keyword{methods}
\keyword{nonparametric}
\keyword{smooth}

\name{spatdim}
\alias{spatdim}
\title{Spatial Dimension of a Dataset}
\description{
  Extracts the spatial dimension of an object in the
  \pkg{spatstat} package.
}
\usage{spatdim(X, intrinsic=FALSE)}
\arguments{
  \item{X}{
    Object belonging to any class defined in the
    \pkg{spatstat} package.
  }
  \item{intrinsic}{
    Logical value indicating whether to return the
    number of intrinsic dimensions. See Details.
  }
}
\value{
  An integer, or \code{NA}.
}
\details{
  This function returns the number of spatial coordinate dimensions
  of the dataset \code{X}. The results for some of the more common
  types of objects are as follows:
  \tabular{ll}{
    \bold{object class} \tab \bold{dimension} \cr
    \code{"ppp"} \tab 2 \cr
    \code{"lpp"} \tab 2 \cr
    \code{"pp3"} \tab 3 \cr
    \code{"ppx"} \tab number of \emph{spatial} dimensions \cr
    \code{"owin"} \tab 2 \cr
    \code{"psp"} \tab 2 \cr
    \code{"ppm"} \tab 2
  }
  Note that time dimensions are not counted.

  Some spatial objects are lower-dimensional subsets of the
  space in which they live. This lower number of dimensions
  is returned if \code{intrinsic=TRUE}. For example, a dataset
  on a linear network
  (an object \code{X} of class \code{"linnet", "lpp", "linim", "linfun"}
  or \code{"lintess"}) returns \code{spatdim(X) = 2} but
  \code{spatdim(X, intrinsic=TRUE) = 1}.
  
  If \code{X} is not a recognised spatial object, the result is \code{NA}.
}
\author{
  \spatstatAuthors.
}
\examples{
  spatdim(lansing)
  A <- osteo$pts[[1]]
  spatdim(A)
  spatdim(domain(A))
  spatdim(chicago)
  spatdim(chicago, intrinsic=TRUE)
}

\name{rgbim}
\alias{rgbim}
\alias{hsvim}
\title{Create Colour-Valued Pixel Image}
\description{
  Creates an object of
  class \code{"im"} representing a two-dimensional pixel image
  whose pixel values are colours.
}
\usage{
  rgbim(R, G, B, A, maxColorValue=255, autoscale=FALSE)
  hsvim(H, S, V, A, autoscale=FALSE)
}
\arguments{
  \item{R,G,B}{
    Pixel images (objects of class \code{"im"}) or constants
    giving the red, green, and blue components of a colour, respectively.
  }
  \item{A}{
    Optional. Pixel image or constant value
    giving the alpha (transparency) component of a colour.
  }
  \item{maxColorValue}{
    Maximum colour channel value for \code{R,G,B,A}.
  }
  \item{H,S,V}{
    Pixel images (objects of class \code{"im"}) or constants
    giving the hue, saturation, and value components of
    a colour, respectively.
  }
  \item{autoscale}{
    Logical. If \code{TRUE}, input values are automatically rescaled
    to fit the permitted range. RGB values are scaled to lie between
    0 and \code{maxColorValue}. HSV values are scaled to lie between 0
    and 1.
  }
}
\details{
  These functions take three pixel images, with real or integer pixel values,
  and create a single pixel image whose pixel values are colours
  recognisable to \R.

  Some of the arguments may be constant numeric values, but
  at least one of the arguments must be a pixel image.
  The image arguments should be compatible (in array dimension
  and in spatial position).

  \code{rgbim} calls \code{\link{rgb}} to compute the colours,
  while \code{hsvim} calls \code{\link{hsv}}. See the help for the relevant
  function for more information about the meaning of the colour
  channels.
}
\seealso{
  \code{\link{im.object}},
  \code{\link{rgb}},
  \code{\link{hsv}}.

  See \code{\link[spatstat:colourtools]{colourtools}}
  for additional colour tools.
}
\examples{
  \testonly{
    op <- spatstat.options(npixel=32)
  }
  # create three images with values in [0,1]
  X <- setcov(owin())
  X <- eval.im(pmin(1,X))
  M <- Window(X)
  Y <- as.im(function(x,y){(x+1)/2}, W=M)
  Z <- as.im(function(x,y){(y+1)/2}, W=M)
  # convert 
  RGB <- rgbim(X, Y, Z, maxColorValue=1)
  HSV <- hsvim(X, Y, Z)
  opa <- par(mfrow=c(1,2))
  plot(RGB, valuesAreColours=TRUE)
  plot(HSV, valuesAreColours=TRUE)
  par(opa)
  \testonly{
    spatstat.options(op)
  }
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}
\keyword{datagen}
 
 

\name{anyNA.im}
\alias{anyNA.im}
\title{
  Check Whether Image Contains NA Values
}
\description{
  Checks whether any pixel values in a pixel image are \code{NA}
  (meaning that the pixel lies outside the domain of definition of the
  image). 
}
\usage{
  \method{anyNA}{im}(x, recursive = FALSE)
}
\arguments{
  \item{x}{
    A pixel image (object of class \code{"im"}).
  }
  \item{recursive}{
    Ignored.
  }
}
\details{
  The function \code{\link{anyNA}} is generic: \code{anyNA(x)} is a
  faster alternative to \code{any(is.na(x))}.

  This function \code{anyNA.im} is a method for the generic \code{anyNA}
  defined for pixel images. It returns the value \code{TRUE} if any of the pixel
  values in \code{x} are \code{NA}, and
  and otherwise returns \code{FALSE}.
}
\value{
  A single logical value.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{im.object}}
}
\examples{
  anyNA(as.im(letterR))
}
\keyword{spatial}
\keyword{methods}

\name{connected}  
\Rdversion{1.1}
\alias{connected}
\alias{connected.im}
\alias{connected.owin}
\title{
  Connected components
}
\description{
  Finds the topologically-connected components of a spatial object,
  such as the connected clumps of pixels in a binary image.
}
\usage{
connected(X, \dots)

\method{connected}{owin}(X, \dots, method="C")

\method{connected}{im}(X, \dots, background = NA, method="C")
}
\arguments{
  \item{X}{
    A spatial object such as a pixel image (object of class \code{"im"})
    or a window (object of class \code{"owin"}).
  }
  \item{background}{
    Optional. Treat pixels with this value 
    as being part of the background.
  }
  \item{method}{
    String indicating the algorithm to be used. Either \code{"C"}
    or \code{"interpreted"}. See Details.
  }
  \item{\dots}{
    Arguments passed to \code{\link{as.mask}} to determine the
    pixel resolution.
  }
}
\details{
  The function \code{connected} is generic, with methods
  for pixel images (class \code{"im"}) and windows (class \code{"owin"})
  described here. There are also methods for tessellations
  (\code{\link{connected.tess}}), point patterns
  (\code{\link{connected.ppp}} and \code{\link{connected.lpp}}),
  and linear networks (\code{\link{connected.linnet}}).
  
  The functions described here compute the connected component transform
  (Rosenfeld and Pfalz, 1966)
  of a binary image or binary mask. The argument \code{X} is first
  converted into a pixel image with logical values. Then the algorithm
  identifies the connected components (topologically-connected clumps
  of pixels) in the foreground.

  Two pixels belong to the same connected component if they have the value
  \code{TRUE} and if they are neighbours (in the 8-connected
  sense). This rule is applied repeatedly until it terminates.
  Then each connected component
  contains all the pixels that can be reached by stepping from neighbour
  to neighbour.

  If \code{method="C"}, the computation is performed by a compiled C language
  implementation of the classical algorithm of Rosenfeld and Pfalz
  (1966). If \code{method="interpreted"}, the computation is performed
  by an \R implementation of the algorithm of Park et al (2000). 

  The result is a factor-valued image, with levels that correspond to
  the connected components. The Examples show how to extract each
  connected component as a separate window object.
}
\value{
  A pixel image (object of class \code{"im"}) with factor values.
  The levels of the factor correspond to the connected components.
}
\references{
  Park, J.-M., Looney, C.G. and Chen, H.-C. (2000)
  Fast connected component labeling algorithm using a divide and conquer
  technique. Pages 373-376 in
  S.Y. Shin (ed) \emph{Computers and Their Applications:} Proceedings of
  the ISCA 15th International Conference on Computers and Their
  Applications, March 29-31, 2000, New Orleans, Louisiana USA. ISCA
  2000, ISBN 1-880843-32-3. 

  Rosenfeld, A. and Pfalz, J.L. (1966)
  Sequential operations in digital processing.
  \emph{Journal of the Association for Computing Machinery} \bold{13}
  471-494.
}
\seealso{
  \code{\link{connected.ppp}},
  \code{\link{connected.tess}},
  \code{\link{connected.lpp}},
  \code{\link{connected.linnet}},
  \code{\link{im.object}}, 
  \code{\link{tess}}
}
\section{Warnings}{
  It may be hard to distinguish different components 
  in the default plot because the colours of nearby components may be
  very similar. See the Examples for a randomised colour map.
  
  The algorithm for \code{method="interpreted"}
  can be very slow for large images (or images where
  the connected components include a large number of pixels).
}
\examples{
  d <- distmap(cells, dimyx=256)
  X <- levelset(d, 0.07)
  plot(X)
  Z <- connected(X)
  plot(Z)
  # or equivalently
  Z <- connected(d <= 0.07)

  # number of components
  nc <- length(levels(Z))
  # plot with randomised colour map
  plot(Z, col=hsv(h=sample(seq(0,1,length=nc), nc)))

  # how to extract the components as a list of windows
  W <- tiles(tess(image=Z))
}
\author{
  Original \R code by Julian Burgos, University of Washington.
  Adapted for \pkg{spatstat} by
  \adrian
  and \rolf.
}
\keyword{spatial}
\keyword{math}

\name{pcfinhom}
\alias{pcfinhom}
\title{
  Inhomogeneous Pair Correlation Function
}
\description{
  Estimates the inhomogeneous pair correlation function of
  a point pattern using kernel methods.
}
\usage{
pcfinhom(X, lambda = NULL, ..., r = NULL,
         kernel = "epanechnikov", bw = NULL, stoyan = 0.15,
         correction = c("translate", "Ripley"),
         divisor = c("r", "d"),
         renormalise = TRUE, normpower=1,
         update = TRUE, leaveoneout = TRUE,
         reciplambda = NULL,
         sigma = NULL, varcov = NULL, close=NULL)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"}).
  }
  \item{lambda}{
    Optional.
    Values of the estimated intensity function.
    Either a vector giving the intensity values
    at the points of the pattern \code{X},
    a pixel image (object of class \code{"im"}) giving the
    intensity values at all locations, a fitted point process model
    (object of class \code{"ppm"}, \code{"kppm"} or \code{"dppm"})
    or a \code{function(x,y)} which
    can be evaluated to give the intensity value at any location.
  }
  \item{r}{
    Vector of values for the argument \eqn{r} at which \eqn{g(r)} 
    should be evaluated. There is a sensible default.
  }
  \item{kernel}{
    Choice of smoothing kernel, passed to \code{\link{density.default}}.
  }
  \item{bw}{
    Bandwidth for smoothing kernel,
    passed to \code{\link{density.default}}.
    Either a single numeric value,
    or a character string specifying a bandwidth selection rule
    recognised by \code{\link{density.default}}.
    If \code{bw} is missing or \code{NULL},
    the default value is computed using
    Stoyan's rule of thumb: see \code{\link{bw.stoyan}}.
  }
  \item{\dots}{
    Other arguments passed to the kernel density estimation 
    function \code{\link{density.default}}.
  }
  \item{stoyan}{
    Coefficient for Stoyan's bandwidth selection rule;
    see \code{\link{bw.stoyan}}.
  }
  \item{correction}{
    Character string or character vector
    specifying the choice of edge correction.
    See \code{\link{Kest}} for explanation and options.
  }
  \item{divisor}{
    Choice of divisor in the estimation formula:
    either \code{"r"} (the default) or \code{"d"}.
    See \code{\link{pcf.ppp}}.
  }
  \item{renormalise}{
    Logical. Whether to renormalise the estimate. See Details.
  }
  \item{normpower}{
    Integer (usually either 1 or 2).
    Normalisation power. See Details.
  }
  \item{update}{
    Logical. If \code{lambda} is a fitted model
    (class \code{"ppm"}, \code{"kppm"} or \code{"dppm"})
    and \code{update=TRUE} (the default),
    the model will first be refitted to the data \code{X}
    (using \code{\link{update.ppm}} or \code{\link{update.kppm}})
    before the fitted intensity is computed.
    If \code{update=FALSE}, the fitted intensity of the
    model will be computed without re-fitting it to \code{X}.
  }
  \item{leaveoneout}{
    Logical value (passed to \code{\link{density.ppp}} or
    \code{\link{fitted.ppm}}) specifying whether to use a
    leave-one-out rule when calculating the intensity.
  }
  \item{reciplambda}{
    Alternative to \code{lambda}.
    Values of the estimated \emph{reciprocal} \eqn{1/\lambda}{1/lambda}
    of the intensity function.
    Either a vector giving the reciprocal intensity values
    at the points of the pattern \code{X},
    a pixel image (object of class \code{"im"}) giving the
    reciprocal intensity values at all locations,
    or a \code{function(x,y)} which can be evaluated to give the
    reciprocal intensity value at any location.
  }
  \item{sigma,varcov}{
    Optional arguments passed to  \code{\link{density.ppp}}
    to control the smoothing bandwidth, when \code{lambda} is
    estimated by kernel smoothing.
  }
  \item{close}{
    Advanced use only. Precomputed data. See section on Advanced Use.
  }
}
\details{
  The inhomogeneous pair correlation function \eqn{g_{\rm inhom}(r)}{ginhom(r)}
  is a summary of the dependence between points in a spatial point
  process that does not have a uniform density of points.

  The best intuitive interpretation is the following: the probability
  \eqn{p(r)} of finding two points at locations \eqn{x} and \eqn{y}
  separated by a distance \eqn{r} is equal to
  \deqn{
    p(r) = \lambda(x) lambda(y) g(r) \,{\rm d}x \, {\rm d}y
  }{
    p(r) = lambda(x) * lambda(y) * g(r) dx dy
  }
  where \eqn{\lambda}{lambda} is the intensity function
  of the point process.
  For a Poisson point process with intensity function
  \eqn{\lambda}{lambda}, this probability is 
  \eqn{p(r) = \lambda(x) \lambda(y)}{p(r) = lambda(x) * lambda(y)}
  so \eqn{g_{\rm inhom}(r) = 1}{ginhom(r) = 1}.

  The inhomogeneous pair correlation function 
  is related to the inhomogeneous \eqn{K} function through
  \deqn{
    g_{\rm inhom}(r) = \frac{K'_{\rm inhom}(r)}{2\pi r}
  }{
    ginhom(r) = Kinhom'(r)/ ( 2 * pi * r) 
  }
  where \eqn{K'_{\rm inhom}(r)}{Kinhom'(r)}
  is the derivative of \eqn{K_{\rm inhom}(r)}{Kinhom(r)}, the
  inhomogeneous \eqn{K} function. See \code{\link{Kinhom}} for information
  about \eqn{K_{\rm inhom}(r)}{Kinhom(r)}.

  The command \code{pcfinhom} estimates the inhomogeneous
  pair correlation using a modified version of
  the algorithm in \code{\link{pcf.ppp}}.   
  
  If \code{renormalise=TRUE} (the default), then the estimates 
  are multiplied by \eqn{c^{\mbox{normpower}}}{c^normpower} where 
  \eqn{
    c = \mbox{area}(W)/\sum (1/\lambda(x_i)).
  }{
    c = area(W)/sum[i] (1/lambda(x[i])).
  }
  This rescaling reduces the variability and bias of the estimate
  in small samples and in cases of very strong inhomogeneity.
  The default value of \code{normpower} is 1
  but the most sensible value is 2, which would correspond to rescaling
  the \code{lambda} values so that
  \eqn{
    \sum (1/\lambda(x_i)) = \mbox{area}(W).
  }{
    sum[i] (1/lambda(x[i])) = area(W).
  }
}
\value{
  A function value table (object of class \code{"fv"}).
  Essentially a data frame containing the variables
  \item{r}{
    the vector of values of the argument \eqn{r} 
    at which the inhomogeneous pair correlation function
    \eqn{g_{\rm inhom}(r)}{ginhom(r)} has been  estimated
  }
  \item{theo}{vector of values equal to 1,
    the theoretical value of \eqn{g_{\rm inhom}(r)}{ginhom(r)}
    for the Poisson process
  }
  \item{trans}{vector of values of \eqn{g_{\rm inhom}(r)}{ginhom(r)}
    estimated by translation correction
  }
  \item{iso}{vector of values of \eqn{g_{\rm inhom}(r)}{ginhom(r)}
    estimated by Ripley isotropic correction
  }
  as required.
}
\section{Advanced Use}{
  To perform the same computation using several different bandwidths \code{bw},
  it is efficient to use the argument \code{close}.
  This should be the result of \code{\link{closepairs}(X, rmax)}
  for a suitably large value of \code{rmax}, namely
  \code{rmax >= max(r) + 3 * bw}.
}
\seealso{
  \code{\link{pcf}}, 
  \code{\link{pcf.ppp}}, 
  \code{\link{bw.stoyan}},
  \code{\link{bw.pcf}},
  \code{\link{Kinhom}}
}
\examples{
  data(residualspaper)
  X <- residualspaper$Fig4b
  plot(pcfinhom(X, stoyan=0.2, sigma=0.1))
  fit <- ppm(X, ~polynom(x,y,2))
  plot(pcfinhom(X, lambda=fit, normpower=2))
}
\author{
  \spatstatAuthors.
} 
\keyword{spatial}
\keyword{nonparametric}

\name{hist.funxy}
\alias{hist.funxy}
\title{Histogram of Values of a Spatial Function}
\description{
  Computes and displays a histogram of the values of a spatial function
  of class \code{"funxy"}.
}
\usage{
  \method{hist}{funxy}(x, \dots, xname)
}
\arguments{
  \item{x}{A pixel image (object of class \code{"funxy"}).}
  \item{\dots}{
    Arguments passed to \code{\link{as.im}}
    or \code{\link{hist.im}}.
  }
  \item{xname}{
    Optional. Character string to be used as the
    name of the dataset \code{x}.
  }
}
\details{
  This function computes and (by default) displays a histogram
  of the values of the function \code{x}.

  An object of class \code{"funxy"} 
  describes a function of spatial location. It is a \code{function(x,y,..)}
  in the \R language, with additional attributes.

  The function \code{hist.funxy} is a method for the generic
  function \code{\link{hist}} for the class \code{"funxy"}.

  The function is first converted to a pixel image using \code{\link{as.im}},
  then \code{\link{hist.im}} is called to produce the histogram.

  Any arguments in \code{...} are passed to \code{\link{as.im}}
  to determine the pixel resolution, 
  or to \code{\link{hist.im}} to determine the histogram breaks
  and to control or suppress plotting.
  Useful arguments include \code{W} for the spatial domain,
  \code{eps,dimyx} for pixel resolution, \code{main} for the main title.
}
\value{
  An object of class \code{"histogram"} as returned
  by \code{\link[graphics:hist]{hist.default}}. This object can be
  plotted.
}
\seealso{
  \code{\link{spatialcdf}} for the cumulative distribution function
  of an image or function.

  \code{\link{hist}},
  \code{\link{hist.default}}.

  For other statistical graphics such as Q-Q plots,
  use \code{as.im(X)[]} to extract the pixel values of image \code{X},
  and apply the usual statistical graphics commands.
}
\examples{
  f <- funxy(function(x,y) {x^2}, unit.square())
  hist(f)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{methods}

\name{spatstat-deprecated}
\alias{as.psp.owin}
\alias{which.max.im}
\alias{circumradius}
\alias{circumradius.owin}
\alias{circumradius.ppp}
\alias{circumradius.linnet}
\title{Deprecated spatstat functions}
\description{
  Deprecated spatstat functions.
}
\usage{
\method{as.psp}{owin}(x, \dots, window=NULL, 
       check=spatstat.options("checksegments"), fatal=TRUE)
which.max.im(x)
circumradius(x, \dots)
\method{circumradius}{owin}(x, \dots)
\method{circumradius}{ppp}(x, \dots)
\method{circumradius}{linnet}(x, \dots)
}

\details{
  These functions are deprecated, and will eventually be deleted from
  the \pkg{spatstat} package.
  
  \code{as.psp.owin} has been replaced by \code{\link{edges}}.
  
  \code{which.max.im(x)} is replaced by
  \code{\link{im.apply}(x, which.max)}.

  \code{circumradius} is replaced by the more appropriately named
  \code{boundingradius}.
}
\keyword{internal}

\name{colouroutputs}
\alias{colouroutputs}
\alias{colouroutputs<-}
\title{
  Extract or Assign Colour Values in a Colour Map
}
\description{
  Extract the colour values in a colour map,
  or assign new colour values.
}
\usage{
colouroutputs(x)

colouroutputs(x) <- value
}

\arguments{
  \item{x}{
    A colour map (object of class \code{"colourmap"}).
  }
  \item{value}{
    A vector of values that can be interpreted as colours.
  }
}
\details{
  An object of class \code{"colourmap"} is effectively a function that
  maps its inputs (numbers or factor levels) to colour values.
  
  The command \code{colouroutputs(x)} extracts the colour values
  in the colour map \code{x}.

  The assignment \code{colouroutputs(x) <- value} replaces the colour
  values in the colour map \code{x} by the entries in 
  \code{value}. The replacement vector \code{value} should have the
  same length as \code{colouroutputs(x)}, and its entries should be
  interpretable as colours.

  To change only some of the colour values in a colour map,
  it may be easier to use \code{\link{tweak.colourmap}}.
}
\value{
  The result of \code{colouroutputs} is a character vector of
  colour values. The result of the assignment
  \code{colouroutputs(x) <- value}  is 
  another colour map (object of class \code{"colourmap"}).
}
\seealso{
  \code{\link{colourmap}}, 
  \code{\link{interp.colourmap}}, 
  \code{\link{tweak.colourmap}}, 
  \code{\link[spatstat:colourtools]{colourtools}}.
}
\examples{
  m <- colourmap(rainbow(5), range=c(0,1))
  m
  # reverse order of colours
  colouroutputs(m) <- rev(colouroutputs(m))
  m
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{color}

\name{CDF}
\alias{CDF}
\alias{CDF.density}
\title{
  Cumulative Distribution Function From Kernel Density Estimate
}
\description{
  Given a kernel estimate of a probability density,
  compute the corresponding cumulative distribution function.
}
\usage{
CDF(f, \dots)

\method{CDF}{density}(f, \dots, warn = TRUE)
}
\arguments{
  \item{f}{
    Density estimate (object of class \code{"density"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{warn}{
    Logical value indicating whether to issue a warning if the
    density estimate \code{f} had to be renormalised because it
    was computed in a restricted interval.
  }
}
\details{
  \code{CDF} is generic, with a method for class \code{"density"}.
  
  This calculates the cumulative distribution function
  whose probability density has been estimated and stored in the object
  \code{f}. The object \code{f} must belong to the class \code{"density"},
  and would typically have been obtained from a call to the function
  \code{\link[stats]{density}}.
}
\value{
  A function, which can be applied to any numeric value or vector of
  values.
}
\author{
  \spatstatAuthors
}
\seealso{
  \code{\link[stats]{density}},
  \code{\link{quantile.density}}
}
\examples{
   b <- density(runif(10))
   f <- CDF(b)
   f(0.5)
   plot(f)
}
\keyword{nonparametric}
\keyword{univar}

\name{reload.or.compute}
\alias{reload.or.compute}
\title{
  Compute Unless Previously Saved
}
\description{
  If the designated file does not yet exist,
  evaluate the expression and save the results in the file.
  If the file already exists, re-load the results from the file.
}
\usage{
reload.or.compute(filename, expr, objects = NULL,
                  destination = parent.frame(), force=FALSE)
}
\arguments{
  \item{filename}{
    Name of data file. A character string.
  }
  \item{expr}{
    \R language expression to be evaluated.
  }
  \item{objects}{
    Optional character vector of names of objects to be saved
    in \code{filename} after evaluating \code{expr},
    or names of objects that should be present in \code{filename}
    when loaded.
  }
  \item{destination}{
    Environment in which the resulting objects should be assigned.
  }
  \item{force}{
    Logical value indicating whether to perform the computation
    in any case.
  }
}
\details{
  This facility is useful for saving, and later re-loading, the results of
  time-consuming computations. It would typically be
  used in an \R script file or an \code{\link[utils]{Sweave}} document.

  If the file called \code{filename} does not yet exist,
  then \code{expr} will be evaluated
  and the results will be saved in \code{filename}.
  The optional argument \code{objects} specifies which results should be saved
  to the file: the default is to save all objects that were created
  by evaluating the expression. 

  If the file called \code{filename} already exists, then it
  will be loaded.
  The optional argument \code{objects} specifies the names
  of objects that should be present in the file; a warning is issued
  if any of them are missing.

  The resulting objects can be assigned into any desired \code{destination}.
  The default behaviour is equivalent to evaluating \code{expr}
  in the current environment.

  If \code{force=TRUE} then \code{expr} will be evaluated
  (regardless of whether the file already exists or not)
  and the results will be saved in \code{filename}, overwriting
  any previously-existing file with that name. This is a convenient
  way to force the code to re-compute everything
  in an \R script file or \code{\link[utils]{Sweave}} document.
}
\value{
  Character vector (invisible) giving the names of the objects computed
  or loaded.
}
\examples{
   \dontrun{
    if(FALSE) {
     reload.or.compute("mydata.rda", {
        x <- very.long.computation()
        y <- 42
      })
   }
   }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{utilities}

\name{summary.owin}
\alias{summary.owin}
\title{Summary of a Spatial Window}
\description{
  Prints a useful description of a window object.
}
\usage{
  \method{summary}{owin}(object, \dots)
}
\arguments{
  \item{object}{Window (object of class \code{"owin"}).}
  \item{\dots}{Ignored.}
}
\details{
  A useful description of the window \code{object} is printed.

  This is a method for the generic function \code{\link{summary}}.
}
\seealso{
  \code{\link{summary}},
  \code{\link{summary.ppp}},
  \code{\link{print.owin}}
}
\examples{
  summary(owin())  # the unit square

  data(demopat)
  W <- Window(demopat)  # weird polygonal window
  summary(W)           # describes it

  summary(as.mask(W))  # demonstrates current pixel resolution
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}

\name{rlinegrid}
\alias{rlinegrid}
\title{Generate grid of parallel lines with random displacement}
\description{
  Generates a grid of parallel lines, equally spaced, inside the
  specified window.
}
\usage{
  rlinegrid(angle = 45, spacing = 0.1, win = owin())
}
\arguments{
  \item{angle}{Common orientation of the lines, in degrees anticlockwise
    from the x axis.
  }
  \item{spacing}{Spacing between successive lines.}
  \item{win}{Window in which to generate the lines. An object of
    class \code{"owin"} or something acceptable to
    \code{\link{as.owin}}.
  }
}
\details{
  The grid is randomly displaced from the origin.
}
\value{
  A line segment pattern (object of class \code{"psp"}).
}
\seealso{
  \code{\link{psp}},
  \code{\link{rpoisline}}
}
\examples{
  plot(rlinegrid(30, 0.05))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{rags}
\alias{rags}
\title{
  Alternating Gibbs Sampler for Multitype Point Processes
}
\description{
  Simulate a realisation of a point process model using the
  alternating Gibbs sampler.
}
\usage{
rags(model, \dots, ncycles = 100)
}
\arguments{
  \item{model}{
    Data specifying some kind of point process model.
  }
  \item{\dots}{
    Additional arguments passed to other code.
  }
  \item{ncycles}{
    Number of cycles of the alternating Gibbs sampler that should be
    performed. 
  }
}
\details{
  The Alternating Gibbs Sampler for a multitype point process
  is an iterative simulation procedure. Each step of the sampler
  updates the pattern of points of a particular type \code{i},
  by drawing a realisation from the conditional distribution of
  points of type \code{i} given the points of all other types.
  Successive steps of the sampler update the points of type 1, then
  type 2, type 3, and so on. 

  This is an experimental implementation which currently works only
  for multitype hard core processes (see \code{\link{MultiHard}})
  in which there is no interaction between points of the same type. 

  The argument \code{model} should be an object describing a point
  process model. At the moment, the only permitted format for
  \code{model} is of the form \code{list(beta, hradii)} where
  \code{beta} gives the first order trend and \code{hradii} is the
  matrix of interaction radii. See \code{\link{ragsMultiHard}} for
  full details.
}
\value{
  A point pattern (object of class \code{"ppp"}).
}
\author{
  \adrian
}
\seealso{
  \code{\link{ragsMultiHard}},
  \code{\link{ragsAreaInter}}
}
\examples{
  mo <- list(beta=c(30, 20),
             hradii = 0.05 * matrix(c(0,1,1,0), 2, 2))
  rags(mo, ncycles=10)
}
\keyword{spatial}
\keyword{datagen}

\name{areaLoss}
\alias{areaLoss}
\title{Difference of Disc Areas}
\description{
  Computes the area of that part of a disc
  that is not covered by other discs.
}
\usage{
   areaLoss(X, r, ..., W=as.owin(X), subset=NULL,
                 exact=FALSE,
                 ngrid=spatstat.options("ngrid.disc"))
}
\arguments{
  \item{X}{
    Locations of the centres of discs.
    A point pattern (object of class \code{"ppp"}).
  }
  \item{r}{
    Disc radius, or vector of disc radii.
  }
  \item{\dots}{Ignored.}
  \item{W}{
    Optional. Window (object of class \code{"owin"})
    inside which the area should be calculated.
  }
  \item{subset}{
    Optional. Index identifying a subset of the points of \code{X}
    for which the area difference should be computed.
  }
  \item{exact}{
    Choice of algorithm.
    If \code{exact=TRUE}, areas are computed exactly using
    analytic geometry. If \code{exact=FALSE} then a faster algorithm
    is used to compute a discrete approximation to the areas.
  }
  \item{ngrid}{
    Integer. Number of points in the square grid used to compute
    the discrete approximation, when \code{exact=FALSE}.
  }
}
\value{
  A matrix with one row for each point in \code{X} (or \code{X[subset]})
  and one column for each value in \code{r}. 
}
\details{
  This function computes, for each point \code{X[i]} in \code{X}
  and for each radius \code{r},
  the area of that part of the disc of radius \code{r} centred at the
  location \code{X[i]} that is \emph{not} covered by any of the
  other discs of radius \code{r} centred at the points \code{X[j]}
  for \code{j} not equal to \code{i}.
  This area is important in some calculations related to
  the area-interaction model \code{\link{AreaInter}}.

  The result is a matrix, with one row for each point in \code{X}
  and one column for each entry of \code{r}. 
}
\seealso{
  \code{\link{AreaInter}},
  \code{\link{areaGain}},
  \code{\link{dilated.areas}}
}
\examples{
   data(cells)
   areaLoss(cells, 0.1)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{psp}
\alias{psp}
\title{Create a Line Segment Pattern}
\description{
  Creates an object of class \code{"psp"} representing 
  a line segment pattern in the two-dimensional plane.
}
\usage{
  psp(x0,y0, x1, y1, window, marks=NULL,
        check=spatstat.options("checksegments"))
}
\arguments{
  \item{x0}{Vector of \eqn{x} coordinates of first endpoint of each segment}
  \item{y0}{Vector of \eqn{y} coordinates of first endpoint of each segment}
  \item{x1}{Vector of \eqn{x} coordinates of second endpoint of each segment}
  \item{y1}{Vector of \eqn{y} coordinates of second endpoint of each segment}
  \item{window}{window of observation,
    an object of class \code{"owin"}}
  \item{marks}{(optional) vector or data frame of mark values}
  \item{check}{Logical value indicating whether to check that the line segments
    lie inside the window.}
}
\value{
  An object of class \code{"psp"} 
  describing a line segment pattern in the two-dimensional plane
  (see \code{\link{psp.object}}).
}
\details{
  In the \pkg{spatstat} library, a spatial pattern of line segments is
  described by an object of class \code{"psp"}. This function
  creates such objects.

  The vectors \code{x0}, \code{y0}, \code{x1} and \code{y1} must be
  numeric vectors of equal length. They are interpreted as the cartesian
  coordinates of the endpoints of the line segments.

  A line segment pattern is assumed to have been observed within a specific
  region of the plane called the observation window.
  An object of class \code{"psp"} representing a point pattern
  contains information specifying the observation window.
  This window must always be specified when creating a point pattern dataset;
  there is intentionally no default action of ``guessing'' the window
  dimensions from the data points alone. 

  The argument \code{window} must be an object of class
  \code{"owin"}. It is a full description of the window geometry,
  and could have been obtained from \code{\link{owin}} or
  \code{\link{as.owin}}, or by just extracting the observation window
  of another dataset, or by manipulating such windows.
  See \code{\link{owin}} or the Examples below.

  The optional argument \code{marks} is given if the line segment pattern
  is marked, i.e. if each line segment carries additional information.
  For example, line segments which are classified into two or more different
  types, or colours, may be regarded as having a mark which identifies
  which colour they are. 

  The object \code{marks} must be a vector of the same length
  as \code{x0}, or a data frame with number of rows equal to the
  length of \code{x0}.  The interpretation is that \code{marks[i]}
  or \code{marks[i,]} is the mark attached to the \eqn{i}th line
  segment.  If the marks are real numbers then \code{marks} should
  be a numeric vector, while if the marks takes only a finite number
  of possible values (e.g. colours or types) then \code{marks}
  should be a \code{factor}.

  See \code{\link{psp.object}} for a description of the class
  \code{"psp"}.

  Users would normally invoke \code{psp} to create a line segment pattern,
  and the function \code{\link{as.psp}} to convert data in another
  format into a line segment pattern.
}
\seealso{
  \code{\link{psp.object}},
  \code{\link{as.psp}},
  \code{\link{owin.object}},
  \code{\link{owin}},
  \code{\link{as.owin}}.

  Function for extracting information from a segment pattern:
  \code{\link{marks.psp}},
  \code{\link{summary.psp}},
  \code{\link{midpoints.psp}},
  \code{\link{lengths_psp}}
  \code{\link{angles.psp}},
  \code{\link{endpoints.psp}}

  Convert line segments to infinite lines: \code{\link{extrapolate.psp}}.
}
\examples{
  X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  m <- data.frame(A=1:10, B=letters[1:10])
  X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin(), marks=m)
}
\author{\adrian
  and \rolf.
}
\keyword{spatial}
\keyword{datagen}

\name{crossdist.psp} 
\alias{crossdist.psp}
\title{Pairwise distances between two different line segment patterns}
\description{
  Computes the distances between all pairs of line segments
  taken from two different line segment patterns.
}
\usage{
  \method{crossdist}{psp}(X, Y, \dots, method="C", type="Hausdorff")
}
\arguments{
  \item{X,Y}{
    Line segment patterns (objects of class \code{"psp"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{method}{String specifying which method of calculation to use.
    Values are \code{"C"} and \code{"interpreted"}.
    Usually not specified.
  }
  \item{type}{
    Type of distance to be computed. Options are
    \code{"Hausdorff"} and \code{"separation"}. Partial matching is used.
  }
}
\value{
  A matrix whose \code{[i,j]} entry is the distance
  from the \code{i}-th line segment in \code{X}
  to the \code{j}-th line segment in \code{Y}.
}
\details{
  This is a method for the generic function \code{\link{crossdist}}.

  Given two line segment patterns, 
  this function computes the distance from each line segment
  in the first pattern to each line segment in the second pattern,
  and returns a matrix containing these distances.

  The distances between line segments are measured in one of two ways:
  \itemize{
    \item if \code{type="Hausdorff"}, distances are computed
    in the Hausdorff metric. The Hausdorff
    distance between two line segments is the \emph{maximum} distance
    from any point on one of the segments to the nearest point on
    the other segment.
    \item if \code{type="separation"}, distances are computed
    as the \emph{minimum} distance from a point on one line segment to
    a point on the other line segment. For example, line segments which
    cross over each other have separation zero.
  }
  
  The argument \code{method} is not normally used. It is
  retained only for checking the validity of the software.
  If \code{method = "interpreted"} then the distances are
  computed using interpreted \R code only. If \code{method="C"}
  (the default) then compiled \code{C} code is used. 
  The \code{C} code is several times faster.
}
\seealso{
  \code{\link{pairdist}},
  \code{\link{nndist}},
  \code{\link{Gest}}
}
\examples{
   L1 <- psp(runif(5), runif(5), runif(5), runif(5), owin())
   L2 <- psp(runif(10), runif(10), runif(10), runif(10), owin())
   D <- crossdist(L1, L2)
   #result is a 5 x 10 matrix
   S <- crossdist(L1, L2, type="sep")
}
\author{
  \adrian
  
  
  and \rolf
  
}
\keyword{spatial}

\keyword{math}

\name{rLGCP}
\alias{rLGCP}
\title{Simulate Log-Gaussian Cox Process}
\description{
  Generate a random point pattern, a realisation of the
  log-Gaussian Cox process.
}
\usage{
 rLGCP(model="exp", mu = 0, param = NULL,
       \dots,
       win=NULL, saveLambda=TRUE, nsim=1, drop=TRUE)
}
\arguments{
  \item{model}{
    character string: the short name of a covariance model for
    the Gaussian random field. After adding the prefix \code{"RM"},
    the code will search for a function of this name
    in the \pkg{RandomFields} package.
  }
  \item{mu}{
    mean function of the Gaussian random field. Either a
    single number, a \code{function(x,y, ...)} or a pixel
    image (object of class \code{"im"}).
  }
  \item{param}{
    List of parameters for the covariance.
    Standard arguments are \code{var} and \code{scale}.
  }
  \item{\dots}{
    Additional parameters for the covariance,
    or arguments passed to \code{\link{as.mask}} to determine
    the pixel resolution.
  }
  \item{win}{
    Window in which to simulate the pattern.
    An object of class \code{"owin"}.
  }
  \item{saveLambda}{
    Logical. If \code{TRUE} (the default) then the
    simulated random intensity will also be saved,
    and returns as an attribute of the point pattern.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  A point pattern (object of class \code{"ppp"})
  or a list of point patterns.

  Additionally, the simulated intensity function for each point pattern is
  returned as an attribute \code{"Lambda"} of the point pattern,
  if \code{saveLambda=TRUE}.
}
\details{
  This function generates a realisation of a log-Gaussian Cox
  process (LGCP). This is a Cox point process in which
  the logarithm of the random intensity is a Gaussian random
  field with mean function \eqn{\mu} and covariance function
  \eqn{c(r)}. Conditional on the random intensity, the point process
  is a Poisson process with this intensity.

  The string \code{model} specifies the covariance 
  function of the Gaussian random field, and the parameters
  of the covariance are determined by \code{param} and \code{\dots}.

  To determine the covariance model, the string \code{model}
  is prefixed by \code{"RM"}, and a function of this name is
  sought in the \pkg{RandomFields} package. 
  For a list of available models see 
  \code{\link[RandomFields]{RMmodel}} in the
  \pkg{RandomFields} package. For example the
  \Matern covariance is specified by \code{model="matern"}, corresponding
  to the function \code{RMmatern} in the \pkg{RandomFields} package.

  Standard variance parameters (for all functions beginning with
  \code{"RM"} in the \pkg{RandomFields} package) are \code{var}
  for the variance at distance zero, and \code{scale} for the scale
  parameter. Other parameters are specified in the help files
  for the individual functions beginning with \code{"RM"}. For example
  the help file for \code{RMmatern} states that \code{nu} is a parameter
  for this model.

  This algorithm uses the function \code{\link[RandomFields]{RFsimulate}} in the
  \pkg{RandomFields} package to generate values of
  a Gaussian random field, with the specified mean function \code{mu}
  and the covariance specified by the arguments \code{model} and
  \code{param}, on the points of a regular grid. The exponential
  of this random field is taken as the intensity of a Poisson point
  process, and a realisation of the Poisson process is then generated by the 
  function \code{\link{rpoispp}} in the \pkg{spatstat} package.
  
  If the simulation window \code{win} is missing or \code{NULL},
  then it defaults to 
  \code{Window(mu)} if \code{mu} is a pixel image,
  and it defaults to the unit square otherwise.
  
  The LGCP model can be fitted to data using \code{\link{kppm}}.
}
\seealso{
\code{\link{rpoispp}},
\code{\link{rMatClust}},
\code{\link{rGaussPoisson}},
\code{\link{rNeymanScott}},
\code{\link{lgcp.estK}},
\code{\link{kppm}}
}
\references{
  \Moller, J., Syversveen, A. and Waagepetersen, R. (1998)
  Log Gaussian Cox Processes.
  \emph{Scandinavian Journal of Statistics} \bold{25}, 451--482.
}
\examples{
  if(require(RandomFields)) {
  # homogeneous LGCP with exponential covariance function
  X <- rLGCP("exp", 3, var=0.2, scale=.1)

  # inhomogeneous LGCP with Gaussian covariance function
  m <- as.im(function(x, y){5 - 1.5 * (x - 0.5)^2 + 2 * (y - 0.5)^2}, W=owin())
  X <- rLGCP("gauss", m, var=0.15, scale =0.5)
  plot(attr(X, "Lambda"))
  points(X)

  # inhomogeneous LGCP with Matern covariance function
  X <- rLGCP("matern", function(x, y){ 1 - 0.4 * x},
             var=2, scale=0.7, nu=0.5,
             win = owin(c(0, 10), c(0, 10)))
  plot(X)
  }
}
\author{Abdollah Jalilian and Rasmus Waagepetersen.
  Modified by \spatstatAuthors.
}
\keyword{spatial}
\keyword{datagen}

\name{polartess}
\alias{polartess}
\title{
  Tessellation Using Polar Coordinates
}
\description{
  Create a tessellation with tiles defined by polar coordinates
  (radius and angle).
}
\usage{
polartess(W, \dots, nradial = NULL, nangular = NULL,
          radii = NULL, angles = NULL,
          origin = NULL, sep = "x")
}
\arguments{
  \item{W}{
    A window (object of class \code{"owin"})
    or anything that can be coerced to a window using
    \code{\link{as.owin}}, such as a point pattern.
  }
  \item{\dots}{
    Ignored.
  }
  \item{nradial}{
    Number of \emph{tiles} in the radial direction.
    A single integer.
    Ignored if \code{radii} is given.
  }
  \item{nangular}{
    Number of \emph{tiles} in the angular coordinate.
    A single integer.
    Ignored if \code{angles} is given.
  }
  \item{radii}{
    The numeric values of the radii, defining the tiles in the radial
    direction. A numeric vector, of length at least 2, containing
    nonnegative numbers in increasing order. The value \code{Inf} is permitted.
  }
  \item{angles}{
    The numeric values of the angles defining the tiles in the
    angular coordinate. A numeric vector, of length at least 2,
    in increasing order, containing angles in radians.
  }
  \item{origin}{
    Location to be used as the origin of the polar coordinates.
    Either a numeric vector of length 2 giving the spatial location
    of the origin, or one of the strings
    \code{"centroid"}, \code{"midpoint"},
    \code{"left"}, \code{"right"}, \code{"top"}, \code{"bottom"},
    \code{"topleft"}, \code{"bottomleft"}, \code{"topright"} or
    \code{"bottomright"} indicating the location in the window.
  }
  \item{sep}{
    Argument passed to \code{\link{intersect.tess}}
    specifying the character string to be used as a separator
    when forming the names of the tiles.
  }
}
\details{
  A tessellation will be formed from tiles defined by
  intervals in the polar coordinates \eqn{r} (radial distance
  from the origin) or \eqn{\theta} (angle from the horizontal axis)
  or both. These tiles look like the cells on a dartboard.

  If the argument \code{radii} is given, tiles will be demarcated
  by circles centred at the origin, with the specified radii.
  If \code{radii} is absent but \code{nradial} is given,
  then \code{radii} will default to a sequence
  of \code{nradial+1} radii equally spaced from zero to the maximum
  possible radius. If neither \code{radii} nor \code{nradial} are given,
  the tessellation will not include circular arc boundaries.

  If the argument \code{angles} is given, tiles will be demarcated
  by lines emanating from the origin at the specified angles.
  The angular values can be any real numbers; they will be interpreted
  as angles in radians modulo \code{2*pi},
  but they must be an increasing sequence of numbers.
  If \code{angles} is absent but \code{nangular} is given,
  then \code{angles} will default to a sequence
  of \code{nangular+1} angles equally spaced from 0 to \code{2*pi}.
  If neither \code{angles} nor \code{nangular} are given,
  the tessellation will not include linear boundaries.
}
\value{
  A tessellation (object of class \code{"tess"}).
}
\author{
  \adrian.
}
\seealso{
  \code{\link{intersect.tess}}
  
  To construct other kinds of tessellations, see
  \code{\link{tess}},
  \code{\link{quadrats}}, \code{\link{hextess}},
  \code{\link{venn.tess}},
  \code{\link{dirichlet}}, \code{\link{delaunay}}, \code{\link{quantess}}
  and \code{\link{rpoislinetess}}.
  
}
\examples{
  Y <- c(2.8, 1.5)
  plot(polartess(letterR, nangular=6, radii=(0:4)/2, origin=Y),
       do.col=TRUE)
}
\keyword{spatial}
\keyword{manip}

\name{nncross}
\alias{nncross}
\alias{nncross.ppp}
\alias{nncross.default}
\title{Nearest Neighbours Between Two Patterns}
\description{
  Given two point patterns \code{X} and \code{Y},
  finds the nearest neighbour in \code{Y} of each point of \code{X}.
  Alternatively \code{Y} may be a line segment pattern.
}
\usage{
  nncross(X, Y, \dots)

  \method{nncross}{ppp}(X, Y,
          iX=NULL, iY=NULL,
          what = c("dist", "which"),
          \dots,
          k = 1,
          sortby=c("range", "var", "x", "y"),
          is.sorted.X = FALSE,
          is.sorted.Y = FALSE)

  \method{nncross}{default}(X, Y, \dots)
}
\arguments{
  \item{X}{Point pattern (object of class \code{"ppp"}).}
  \item{Y}{Either a point pattern (object of class \code{"ppp"})
    or a line segment pattern (object of class \code{"psp"}).}
  \item{iX, iY}{Optional identifiers, applicable only in the case where
    \code{Y} is a point pattern, used to determine whether a point in
    \code{X} is identical to a point in \code{Y}. See Details.
  }
  \item{what}{
    Character string specifying what information should be returned.
    Either the nearest neighbour distance (\code{"dist"}),
    the identifier of the nearest neighbour (\code{"which"}),
    or both.
  }
  \item{k}{
    Integer, or integer vector. The algorithm will compute the distance to the
    \code{k}th nearest neighbour.
  }
  \item{sortby}{
    Determines which coordinate to use to sort the point patterns.
    See Details.
  }
  \item{is.sorted.X, is.sorted.Y}{
    Logical values attesting whether the point patterns \code{X} and
    \code{Y} have been sorted. See Details.
  }
  \item{\dots}{Ignored.}
}
\details{
  Given two point patterns \code{X} and \code{Y} this
  function finds, for each point of \code{X}, 
  the nearest point of \code{Y}. The distance between these points
  is also computed.
  If the argument \code{k} is specified, then the \code{k}-th nearest
  neighbours will be found.

  Alternatively if \code{X} is a point pattern and \code{Y} is a line
  segment pattern, the function finds the nearest line segment to each point
  of \code{X}, and computes the distance.

  The return value is a data frame, with rows corresponding to
  the points of \code{X}.  The first column gives the nearest neighbour
  distances (i.e. the \code{i}th entry is the distance 
  from the \code{i}th point of \code{X} to the nearest element of
  \code{Y}). The second column gives the indices of the nearest
  neighbours (i.e.\ the \code{i}th entry is the index of
  the nearest element in \code{Y}.)
  If \code{what="dist"} then only the vector of distances is returned.
  If \code{what="which"} then only the vector of indices is returned.

  The argument \code{k} may be an integer or an integer vector.
  If it is a single integer, then the \code{k}-th nearest neighbours
  are computed. If it is a vector, then the \code{k[i]}-th nearest
  neighbours are computed for each entry \code{k[i]}. For example, setting
  \code{k=1:3} will compute the nearest, second-nearest and
  third-nearest neighbours. The result is a data frame.

  Note that this function is not symmetric in \code{X} and \code{Y}.
  To find the nearest neighbour in \code{X} of each point in \code{Y},
  where \code{Y} is a point pattern, use \code{nncross(Y,X)}.

  The arguments \code{iX} and \code{iY} are used when
  the two point patterns \code{X} and \code{Y} have some points in
  common.  In this situation \code{nncross(X, Y)} would return some zero
  distances. To avoid this, attach a unique integer identifier to
  each point, such that two points are identical if their
  identifying numbers are equal. Let \code{iX} be the vector of
  identifier values for the points in \code{X}, and \code{iY}
  the vector of identifiers for points in \code{Y}. Then the code
  will only compare two points if they have different values of the
  identifier. See the Examples.
}
\section{Sorting data and pre-sorted data}{
  Read this section if you care about the speed of computation.
  
  For efficiency, the algorithm sorts the point patterns \code{X} and \code{Y}
  into increasing order of the \eqn{x} coordinate
  or increasing order of the the \eqn{y} coordinate.
  Sorting is only an intermediate step;
  it does not affect the output, which is always given in the same
  order as the original data.
  
  By default (if \code{sortby="range"}),
  the sorting will occur on the coordinate that has the larger range of
  values (according to the frame of the enclosing window of \code{Y}).
  If \code{sortby = "var"}), sorting will occur on the coordinate that
  has the greater variance (in the pattern \code{Y}).
  Setting \code{sortby="x"} or \code{sortby = "y"} will specify that
  sorting should occur on the \eqn{x} or \eqn{y} coordinate, respectively.

  If the point pattern \code{X} is already
  sorted, then the corresponding argument \code{is.sorted.X}
  should be set to \code{TRUE}, and \code{sortby} should be set
  equal to \code{"x"} or \code{"y"} to indicate which coordinate
  is sorted.

  Similarly if \code{Y} is already sorted, then \code{is.sorted.Y}
  should be set to \code{TRUE}, and \code{sortby} should be set
  equal to \code{"x"} or \code{"y"} to indicate which coordinate
  is sorted.

  If both \code{X} and \code{Y} are sorted \emph{on the same coordinate
  axis} then both \code{is.sorted.X} and \code{is.sorted.Y}
  should be set to \code{TRUE}, and \code{sortby} should be set
  equal to \code{"x"} or \code{"y"} to indicate which coordinate
  is sorted.  
}
\value{
  A data frame, or a vector if the data frame would contain only one column.
  
  By default (if \code{what=c("dist", "which")} and \code{k=1})
  a data frame with two columns:
  \item{dist}{Nearest neighbour distance}
  \item{which}{Nearest neighbour index in \code{Y}}

  If \code{what="dist"} and \code{k=1}, a vector of nearest neighbour distances.

  If \code{what="which"} and \code{k=1}, a vector of nearest neighbour
  indices.

  If \code{k} is specified, the result is a data frame with
  columns containing the \code{k}-th nearest neighbour distances
  and/or nearest neighbour indices.
}
\seealso{
  \code{\link{nndist}} for nearest neighbour
  distances in a single point pattern.
}
\examples{
  # two different point patterns
  X <- runifpoint(15)
  Y <- runifpoint(20)
  N <- nncross(X,Y)$which
  # note that length(N) = 15
  plot(superimpose(X=X,Y=Y), main="nncross", cols=c("red","blue"))
  arrows(X$x, X$y, Y[N]$x, Y[N]$y, length=0.15)

  # third-nearest neighbour
  NXY <- nncross(X, Y, k=3)
  NXY[1:3,]
  # second and third nearest neighbours
  NXY <- nncross(X, Y, k=2:3)
  NXY[1:3,]

  # two patterns with some points in common
  Z <- runifpoint(50)
  X <- Z[1:30]
  Y <- Z[20:50]
  iX <- 1:30
  iY <- 20:50
  N <- nncross(X,Y, iX, iY)$which
  N <- nncross(X,Y, iX, iY, what="which") #faster
  plot(superimpose(X=X, Y=Y), main="nncross", cols=c("red","blue"))
  arrows(X$x, X$y, Y[N]$x, Y[N]$y, length=0.15)

  # point pattern and line segment pattern
  X <- runifpoint(15)
  Y <- rpoisline(10)
  N <- nncross(X,Y)
}
\author{
  \adrian
  
  ,
  \rolf
  ,
  and Jens Oehlschlaegel
}
\keyword{spatial}
\keyword{math}

\name{unique.ppp}
\alias{unique.ppp}
\alias{unique.ppx}
\title{Extract Unique Points from a Spatial Point Pattern}
\description{
  Removes any points that are identical to other points
  in a spatial point pattern.
}
\usage{
 \method{unique}{ppp}(x, \dots, warn=FALSE)

 \method{unique}{ppx}(x, \dots, warn=FALSE)
}
\arguments{
  \item{x}{
    A spatial point pattern
    (object of class \code{"ppp"} or \code{"ppx"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{duplicated.ppp}}
    or \code{\link{duplicated.data.frame}}.
  }
  \item{warn}{
    Logical. If \code{TRUE}, issue a warning message if any
    duplicated points were found.
  }
}
\value{
  Another point pattern object.
}
\details{
  These are methods for the generic function \code{unique} for 
  point pattern datasets (of class \code{"ppp"}, see
  \code{\link{ppp.object}}, or class \code{"ppx"}).

  This function removes duplicate points in \code{x},
  and returns a point pattern.

  Two points in a point pattern are deemed to be identical
  if their \eqn{x,y} coordinates are the same,
  \emph{and} their marks are the same (if they carry marks).
  This is the default rule: see \code{\link{duplicated.ppp}}
  for other options. 
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{duplicated.ppp}},
  \code{\link{multiplicity.ppp}}
}
\examples{
   X <- ppp(c(1,1,0.5), c(2,2,1), window=square(3))
   unique(X)
   unique(X, rule="deldir")
}

\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{methods}
 

\name{commonGrid}
\alias{commonGrid}
\title{Determine A Common Spatial Domain And Pixel Resolution}
\description{
  Determine a common spatial domain and pixel resolution for
  several spatial objects such as images, masks, windows
  and point patterns.
}
\usage{
commonGrid(\dots)
}
\arguments{
  \item{\dots}{
    Any number of pixel images (objects of class \code{"im"}),
    binary masks (objects of class \code{"owin"} of type \code{"mask"})
    or data which can be converted to binary masks by \code{\link{as.mask}}.
  } 
}
\details{
  This function determines a common spatial resolution and spatial domain
  for several spatial objects. 

  The arguments \code{\dots} may be pixel images,
  binary masks, or other spatial objects acceptable to \code{\link{as.mask}}.
  
  The common pixel grid is determined by inspecting all the pixel images
  and binary masks in the argument list, finding the pixel grid with the
  highest spatial resolution, and extending this pixel grid to cover the
  bounding box of all the spatial objects.

  The return value is a binary mask \code{M}, representing the bounding box
  at the chosen pixel resolution.
  Use \code{\link{as.im}(X, W=M)} to convert a pixel image \code{X} to this new
  pixel resolution.
  Use \code{\link{as.mask}(W, xy=M)} to convert a window \code{W}
  to a binary mask at this new pixel resolution.
  See the Examples.
}
\value{
  A binary mask (object of class \code{"owin"} and type \code{"mask"}).
}
\author{\adrian
  
  
  and \rolf
  
}
\examples{
   A <- setcov(square(1))
   G <- density(runifpoint(42), dimyx=16)
   H <- commonGrid(A, letterR, G)
   newR <- as.mask(letterR, xy=H)
   newG <- as.im(G, W=H)
}
\seealso{
  \code{\link{harmonise.im}},
  \code{\link{compatible.im}},
  \code{\link{as.im}}
}
\keyword{spatial}
\keyword{manip}

\name{uniquemap.ppp}
\alias{uniquemap}
\alias{uniquemap.ppp}
\alias{uniquemap.lpp}
\alias{uniquemap.ppx}
\title{
  Map Duplicate Entries to Unique Entries
}
\description{
  Determine whether points in a point pattern are duplicated,
  choose a unique representative for each set of duplicates,
  and map the duplicates to the unique representative.
}
\usage{
uniquemap(x)

\method{uniquemap}{ppp}(x)

\method{uniquemap}{lpp}(x)

\method{uniquemap}{ppx}(x)
}
\arguments{
  \item{x}{
    A point pattern (object of class \code{"ppp"}, \code{"lpp"},
    \code{"pp3"} or \code{"ppx"}).
  }
}
\details{
  The function \code{uniquemap} is generic, with methods for
  point patterns and data frames.

  This function determines whether any points of \code{x} are
  duplicated, and constructs a mapping of the indices of \code{x}
  so that all duplicates are mapped to a unique representative index.

  The result is an integer vector \code{u} such that
  \code{u[j] = i} if
  the points \code{x[i]} and \code{x[j]} are identical and
  point \code{i} has been chosen as the unique representative.
  The entry \code{u[i] = i} means either that point \code{i} is
  unique, or that it has been chosen as the unique representative
  of its equivalence class.
}
\value{
  An integer vector.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{unique.ppp}}, \code{\link{duplicated.ppp}},
  \code{\link{uniquemap.default}}
}
\examples{
  Y <- runifpoint(4)
  X <- Y[c(1,2,3,4,2,1)]
  uniquemap(X)
}
\keyword{spatial}
\keyword{methods}

\name{as.hyperframe} 
\Rdversion{1.1}
\alias{as.hyperframe}
\alias{as.hyperframe.default}
\alias{as.hyperframe.data.frame}
\alias{as.hyperframe.hyperframe}
\alias{as.hyperframe.listof}
\alias{as.hyperframe.anylist}
\title{
  Convert Data to Hyperframe
}
\description{
  Converts data from any suitable format into a hyperframe.
}
\usage{
as.hyperframe(x, \dots)

\method{as.hyperframe}{default}(x, \dots)

\method{as.hyperframe}{data.frame}(x, \dots, stringsAsFactors=FALSE)

\method{as.hyperframe}{hyperframe}(x, \dots)

\method{as.hyperframe}{listof}(x, \dots)

\method{as.hyperframe}{anylist}(x, \dots)
}
\arguments{
  \item{x}{
    Data in some other format.
  }
  \item{\dots}{
    Optional arguments passed to \code{\link{hyperframe}}.
  }
  \item{stringsAsFactors}{
    Logical. If \code{TRUE}, any column of the data frame \code{x} that
    contains character strings will be converted to a \code{factor}.
    If \code{FALSE}, no such conversion will occur.
  }
}
\details{
  A hyperframe is like a data frame, except that its entries
  can be objects of any kind.

  The generic function \code{as.hyperframe} converts any suitable kind
  of data into a hyperframe.

  There are methods for the classes
  \code{data.frame}, \code{listof}, \code{anylist} and a default method,
  all of which convert data that is like a hyperframe into
  a hyperframe object. (The method for the class \code{listof}
  and \code{anylist} converts a list of objects, of
  arbitrary type, into a hyperframe with one column.)
  These methods do not discard any information.

  There are also methods for other classes
  (see \code{\link{as.hyperframe.ppx}}) which extract
  the coordinates from a spatial dataset. These methods
  do discard some information.
}
\section{Conversion of Strings to Factors}{
  Note that \code{as.hyperframe.default}
  will convert a character vector to a factor.
  It behaves like \code{\link{as.data.frame}}.

  However \code{as.hyperframe.data.frame} does not convert strings to
  factors; it respects the structure of the data frame \code{x}.

  The behaviour can be changed using the argument \code{stringsAsFactors}.
}
\value{
  An object of class \code{"hyperframe"} created by
  \code{\link{hyperframe}}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{hyperframe}},
  \code{\link{as.hyperframe.ppx}}
}
\examples{
   df <- data.frame(x=runif(4),y=letters[1:4])
   as.hyperframe(df)

   sims <- list()
   for(i in 1:3) sims[[i]] <- rpoispp(42)
   as.hyperframe(as.listof(sims))
   as.hyperframe(as.solist(sims))
}
\keyword{spatial}
\keyword{manip}

\name{superimpose}  
\alias{superimpose}
\alias{superimpose.ppp}
\alias{superimpose.splitppp}
\alias{superimpose.ppplist}
\alias{superimpose.psp}
\alias{superimpose.default}
\title{Superimpose Several Geometric Patterns}
\description{
  Superimpose any number of point patterns or line segment patterns.
}
\usage{
  superimpose(\dots)

  \method{superimpose}{ppp}(\dots, W=NULL, check=TRUE)

  \method{superimpose}{psp}(\dots, W=NULL, check=TRUE)

  \method{superimpose}{splitppp}(\dots, W=NULL, check=TRUE)

  \method{superimpose}{ppplist}(\dots, W=NULL, check=TRUE)

  \method{superimpose}{default}(\dots)
}
\arguments{
  \item{\dots}{
    Any number of arguments, each of which represents either a point
    pattern or a line segment pattern or a list of point patterns.
  }
  \item{W}{
    Optional. Data determining the window for the resulting pattern.
    Either a window (object of class \code{"owin"}, or something
    acceptable to \code{\link{as.owin}}), or a function
    which returns a window, or one of the strings
    \code{"convex"}, \code{"rectangle"}, \code{"bbox"} or \code{"none"}.
  }
  \item{check}{
    Logical value (passed to \code{\link{ppp}} or \code{\link{psp}}
    as appropriate) determining whether to check the geometrical
    validity of the resulting pattern.
  }
}
\value{
  For \code{superimpose.ppp}, a point pattern (object of class \code{"ppp"}).
  For \code{superimpose.default}, either a point pattern
  (object of class \code{"ppp"}) or a \code{list(x,y)}.
  For \code{superimpose.psp}, a line segment pattern (object of class
  \code{"psp"}).
}
\details{
  This function is used to superimpose several geometric patterns
  of the same kind, producing a single pattern of the same kind.
  
  The function \code{superimpose} is generic, with methods
  for the class \code{ppp} of point patterns, the class \code{psp}
  of line segment patterns, and a default method. There is also
  a method for \code{lpp}, described separately in
  \code{\link{superimpose.lpp}}.

  The dispatch to a method is initially determined
  by the class of the \emph{first} argument in \code{\dots}.
  \itemize{
    \item \code{default}:  If the first argument is 
    \emph{not} an object of class \code{ppp} or \code{psp}, then the
    default method \code{superimpose.default} is executed.
    This checks the class of all arguments, and dispatches to the
    appropriate method. Arguments of class \code{ppplist} can be handled.
    \item \code{ppp}:
    If the first \code{\dots} argument is an object of
    class \code{ppp} then the method \code{superimpose.ppp}
    is executed. All arguments in \code{\dots}
    must be either \code{ppp} objects or lists
    with components \code{x} and \code{y}. The result will
    be an object of class \code{ppp}.
    \item psp:
    If the first \code{\dots} argument is an object of
    class \code{psp} then the \code{psp} method is dispatched and all
    \code{\dots} arguments must be \code{psp} objects.
    The result is a \code{psp} object.
  }

  The patterns are \emph{not} required to have the same window
  of observation. 

  The window for the superimposed pattern is controlled
  by the argument \code{W}.
  \itemize{
    \item
    If \code{W} is a window (object of class \code{"W"}
    or something acceptable to \code{\link{as.owin}})
    then this determines the window for the superimposed pattern.
    \item
    If \code{W} is \code{NULL}, or the character string \code{"none"},
    then windows are extracted from the geometric patterns,
    as follows. 
    For \code{superimpose.psp}, all arguments \code{\dots}
    are line segment patterns (objects of class \code{"psp"});
    their observation windows are extracted; the union of these
    windows is computed; and this union is taken to be the
    window for the superimposed pattern.
    For \code{superimpose.ppp} and \code{superimpose.default},
    the arguments \code{\dots} are inspected, and 
    any arguments which are point patterns (objects of class
    \code{"ppp"}) are selected; their observation windows
    are extracted, and the union of these windows
    is taken to be the window for the superimposed point pattern.
    For \code{superimpose.default} if none of the arguments
    is of class \code{"ppp"} then no window is computed
    and the result of \code{superimpose} is a \code{list(x,y)}.
    \item
    If \code{W} is one of the strings
    \code{"convex"}, \code{"rectangle"} or \code{"bbox"}
    then a window for the superimposed pattern
    is computed from the coordinates of the
    points or the line segments as follows.
    \describe{
      \item{\code{"bbox"}:}{the bounding box of the
      points or line segments (see \code{\link{bounding.box.xy}});}
      \item{\code{"convex"}:}{the Ripley-Rasson estimator
      of a convex window (see \code{\link{ripras}});}
      \item{\code{"rectangle"}:}{the Ripley-Rasson estimator
      of a rectangular window
      (using \code{\link{ripras}} with argument
      \code{shape="rectangle"}).}
    }
    \item
    If \code{W} is a function,
    then this function is used to compute
    a window for the superimposed pattern
    from the coordinates of the
    points or the line segments.
    The function should accept input of the form \code{list(x,y)}
    and is expected to return an object of class \code{"owin"}.
    Examples of such functions are
    \code{\link{ripras}} and \code{\link{bounding.box.xy}}.
  }
  The arguments \code{\dots} may be \emph{marked} patterns.
  The marks of each component pattern must have the same format.
  Numeric and character marks may be ``mixed''.  If there is such
  mixing then the numeric marks are coerced to character in the
  combining process. If the mark structures are all data frames,
  then these data frames must have the same number of columns and
  identical column names.

  If the arguments \code{\dots} are given in the form \code{name=value},
  then the \code{name}s will be used as an extra column of marks
  attached to the elements of the corresponding patterns.
}

\seealso{
  \code{\link{superimpose.lpp}},
  \code{\link{concatxy}}, \code{\link{quadscheme}}.
}
\examples{
  # superimposing point patterns
  p1  <- runifrect(30)
  p2  <- runifrect(42)
  s1  <- superimpose(p1,p2) # Unmarked pattern.
  p3  <- list(x=rnorm(20),y=rnorm(20))
  s2  <- superimpose(p3,p2,p1) # Default method gets called.
  s2a <- superimpose(p1,p2,p3) # Same as s2 except for order of points.
  s3  <- superimpose(clyde=p1,irving=p2) # Marked pattern; marks a factor
                                         # with levels "clyde" and "irving";
                                         # warning given.
  marks(p1) <- factor(sample(LETTERS[1:3],30,TRUE))
  marks(p2) <- factor(sample(LETTERS[1:3],42,TRUE))
  s5  <- superimpose(clyde=p1,irving=p2) # Marked pattern with extra column
  marks(p2) <- data.frame(a=marks(p2),b=runif(42))
  s6  <- try(superimpose(p1,p2)) # Gives an error.
  marks(p1) <- data.frame(a=marks(p1),b=1:30)
  s7  <- superimpose(p1,p2) # O.K.

  # how to make a 2-type point pattern with types "a" and "b"
  u <- superimpose(a = rpoispp(10), b = rpoispp(20))

  # how to make a 2-type point pattern with types 1 and 2
  u <- superimpose("1" = rpoispp(10), "2" = rpoispp(20))
 
  # superimposing line segment patterns
  X <- rpoisline(10)
  Y <- as.psp(matrix(runif(40), 10, 4), window=owin())
  Z <- superimpose(X, Y)

  # being unreasonable
  \dontrun{
   if(FALSE) {
    crud <- try(superimpose(p1,p2,X,Y)) # Gives an error, of course!
   }
  }
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{rescale.im}
\alias{rescale.im}
\title{Convert Pixel Image to Another Unit of Length}
\description{
  Converts a pixel image to 
  another unit of length.
}
\usage{
 \method{rescale}{im}(X, s, unitname)
}
\arguments{
  \item{X}{Pixel image (object of class \code{"im"}).}
  \item{s}{Conversion factor: the new units are \code{s} times the old units.}
  \item{unitname}{
    Optional. New name for the unit of length.
    See \code{\link{unitname}}.
  }
}
\value{
  Another pixel image (of class \code{"im"}),
  containing the same pixel values, but with pixel coordinates
  expressed in the new units.
}
\details{
  This is a method for the generic function \code{\link{rescale}}.

  The spatial coordinates of the pixels in \code{X}
  will be re-expressed
  in terms of a new unit of length that is \code{s} times the current
  unit of length given in \code{X}.
  (Thus, the coordinate values are \emph{divided} by \code{s},
  while the unit value is multiplied by \code{s}).

  If \code{s} is missing, then the coordinates will be re-expressed
  in \sQuote{native} units; for example if the current unit is
  equal to 0.1 metres, then the coordinates will be re-expressed in metres.

  The result is a pixel image representing the \emph{same} data
  but re-expressed in a different unit.

  Pixel values are unchanged. This may not be what you intended!
}
\seealso{
  \code{\link{im}},
  \code{\link{rescale}},
  \code{\link{unitname}},
  \code{\link{eval.im}}
}
\examples{
# Bramble Canes data: 1 unit = 9 metres
  data(bramblecanes)
# distance transform
  Z <- distmap(bramblecanes)
# convert to metres
# first alter the pixel values
  Zm <- eval.im(9 * Z)
# now rescale the pixel coordinates
  Z <- rescale(Zm, 1/9)
# or equivalently
  Z <- rescale(Zm)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{runifdisc}
\alias{runifdisc}
\title{Generate N Uniform Random Points in a Disc}
\description{
  Generate a random point pattern
  containing \eqn{n} independent uniform random points
  in a circular disc.
}
\usage{
 runifdisc(n, radius=1, centre=c(0,0), ..., nsim=1, drop=TRUE)
}
\arguments{
  \item{n}{
    Number of points.
  }
  \item{radius}{Radius of the circle.}
  \item{centre}{Coordinates of the centre of the circle.}
  \item{\dots}{
    Arguments passed to \code{\link{disc}} controlling the
    accuracy of approximation to the circle.
  }
  \item{nsim}{Number of simulated realisations to be generated.}
  \item{drop}{
    Logical. If \code{nsim=1} and \code{drop=TRUE} (the default), the
    result will be a point pattern, rather than a list 
    containing a point pattern.
  }
}
\value{
  A point pattern (an object of class \code{"ppp"})
  if \code{nsim=1}, or a list of point patterns if \code{nsim > 1}.
}
\details{
  This function generates \code{n} independent random points,
  uniformly distributed in a circular disc.

  It is faster (for a circular window) than the general
  code used in \code{\link{runifpoint}}.

  To generate random points in an ellipse, first generate points in a
  circle using \code{runifdisc},
  then transform to an ellipse using \code{\link{affine}},
  as shown in the examples.
  
  To generate random points in other windows, use
  \code{\link{runifpoint}}.
  To generate non-uniform random points, use \code{\link{rpoint}}.
}
\seealso{
\code{\link{disc}},
\code{\link{runifpoint}},
\code{\link{rpoint}}
}
\examples{
 # 100 random points in the unit disc
 plot(runifdisc(100))
 # 42 random points in the ellipse with major axis 3 and minor axis 1
 X <- runifdisc(42)
 Y <- affine(X, mat=diag(c(3,1)))
 plot(Y)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{nndist.ppx}
\alias{nndist.ppx}
\title{Nearest Neighbour Distances in Any Dimensions}
\description{
  Computes the distance from each point to its nearest neighbour
  in a multi-dimensional point pattern.
  Alternatively computes the distance to the
  second nearest neighbour, or third nearest, etc.
}
\usage{
  \method{nndist}{ppx}(X, \dots, k=1, by=NULL)
}
\arguments{
  \item{X}{
    Multi-dimensional point pattern
    (object of class \code{"ppx"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{coords.ppx}} to determine
    which coordinates should be used.
  }
  \item{k}{
    Integer, or integer vector. The algorithm will compute the distance to the
    \code{k}th nearest neighbour. 
  }
  \item{by}{
    Optional. A factor, which separates \code{X} into groups.
    The algorithm will compute the distance to
    the nearest point in each group. 
  }
}
\value{
  Numeric vector or matrix containing the 
  nearest neighbour distances for each point.

  If \code{k = 1} (the default), the return value is a
  numeric vector \code{v} such that \code{v[i]} is the
  nearest neighbour distance for the \code{i}th data point.
  
  If \code{k} is a single integer, then the return value is a
  numeric vector \code{v} such that \code{v[i]} is the
  \code{k}th nearest neighbour distance for the
  \code{i}th data point.

  If \code{k} is a vector, then the return value is a
  matrix \code{m} such that \code{m[i,j]} is the
  \code{k[j]}th nearest neighbour distance for the
  \code{i}th data point.
}
\details{
  This function computes the Euclidean distance from each point
  in a multi-dimensional
  point pattern to its nearest neighbour (the nearest other
  point of the pattern). If \code{k} is specified, it computes the
  distance to the \code{k}th nearest neighbour.

  The function \code{nndist} is generic; this function
  \code{nndist.ppx} is the method for the class \code{"ppx"}.

  The argument \code{k} may be a single integer, or an integer vector.
  If it is a vector, then the \eqn{k}th nearest neighbour distances are
  computed for each value of \eqn{k} specified in the vector.

  If there is only one point (if \code{x} has length 1),
  then a nearest neighbour distance of \code{Inf} is returned.
  If there are no points (if \code{x} has length zero)
  a numeric vector of length zero is returned.

  If the argument \code{by} is given, it should be a \code{factor},
  of length equal to the number of points in \code{X}.
  This factor effectively partitions \code{X} into subsets,
  each subset associated with one of the levels of \code{X}.
  The algorithm will then compute, for each point of \code{X},
  the distance to the nearest neighbour \emph{in each subset}.

  To identify \emph{which} point is the nearest neighbour of a given point,
  use \code{\link{nnwhich}}.

  To find the nearest neighbour distances from one point pattern
  to another point pattern, use \code{\link{nncross}}.
  
  By default, both spatial and temporal coordinates are extracted.
  To obtain the spatial distance between points in a space-time point
  pattern, set \code{temporal=FALSE}.
}
\section{Warnings}{
  An infinite or \code{NA} value is returned if the
  distance is not defined (e.g. if there is only one point
  in the point pattern).
}
\seealso{
  \code{\link{nndist}},
  \code{\link{pairdist}},
  \code{\link{nnwhich}}
}
\examples{
   df <- data.frame(x=runif(5),y=runif(5),z=runif(5),w=runif(5))
   X <- ppx(data=df)

   # nearest neighbours
   d <- nndist(X)

   # second nearest neighbours
   d2 <- nndist(X, k=2)

   # first, second and third nearest
   d1to3 <- nndist(X, k=1:3)

   # nearest neighbour distances to each group
   marks(X) <- factor(c("a","a", "b", "b", "b"))
   nndist(X, by=marks(X))
   nndist(X, by=marks(X), k=1:2)
}
\author{
  \adrian
}
\keyword{spatial}
\keyword{math}

\name{is.rectangle}
\alias{is.rectangle}
\alias{is.polygonal}
\alias{is.mask}
\title{Determine Type of Window}
\description{
  Determine whether a window is a
  rectangle, a polygonal region, or a binary mask.
}
\usage{
  is.rectangle(w)
  is.polygonal(w)
  is.mask(w)
}
\arguments{
  \item{w}{
    Window to be inspected. An object of class \code{"owin"}.
  }
}
\value{
  Logical value, equal to \code{TRUE} if \code{w} is a window
  of the specified type.
}
\details{
  These simple functions determine whether a window \code{w}
  (object of class \code{"owin"}) is
  a rectangle (\code{is.rectangle(w) = TRUE}), 
  a domain with polygonal boundary (\code{is.polygonal(w) = TRUE}),
  or 
  a binary pixel mask (\code{is.mask(w) = TRUE}). 
}
\seealso{
  \code{\link{owin}}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{coords}
\Rdversion{1.1}
\alias{coords}
\alias{coords.ppp}
\alias{coords.ppx}
\alias{coords.quad}
\alias{coords<-}
\alias{coords<-.ppp}
\alias{coords<-.ppx}
\title{
  Extract or Change Coordinates of a Spatial or Spatiotemporal Point Pattern
}
\description{
  Given any kind of spatial or space-time point pattern,
  this function extracts the
  (space and/or time and/or local) coordinates of the points
  and returns them as a data frame.
}
\usage{
  coords(x, ...)
  \method{coords}{ppp}(x, ...)
  \method{coords}{ppx}(x, ..., spatial = TRUE, temporal = TRUE, local=TRUE)
  coords(x, ...) <- value
  \method{coords}{ppp}(x, ...) <- value
  \method{coords}{ppx}(x, ..., spatial = TRUE, temporal = TRUE, local=TRUE) <- value
  \method{coords}{quad}(x, ...)
}
\arguments{
  \item{x}{
    A point pattern: either a two-dimensional point pattern
    (object of class \code{"ppp"}), a three-dimensional point pattern
    (object of class \code{"pp3"}), or a 
    general multidimensional space-time point pattern 
    (object of class \code{"ppx"})
    or a quadrature scheme (object of class \code{"quad"}).
  }
  \item{\dots}{
    Further arguments passed to methods.
  }
  \item{spatial,temporal,local}{
    Logical values indicating whether to extract spatial, temporal
    and local coordinates, respectively. The default is to return all
    such coordinates. (Only relevant to \code{ppx} objects).
  }
  \item{value}{
    New values of the coordinates. A numeric vector with one entry for each
    point in \code{x}, or a numeric matrix or data frame with one row
    for each point in \code{x}.
  }
}
\details{
  The function \code{coords} extracts the coordinates from
  a point pattern. The function \code{coords<-} replaces the coordinates
  of the point pattern with new values.
  
  Both functions \code{coords} and \code{coords<-} are generic, with methods for
  the classes \code{"ppp"}) and \code{"ppx"}.
  An object of class \code{"pp3"} also inherits from \code{"ppx"} and
  is handled by the method for \code{"ppx"}.
}
\value{
  \code{coords} returns a \code{data.frame} with one row for each point,
  containing the coordinates.
  \code{coords<-} returns the altered point pattern.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{ppx}},
  \code{\link{pp3}},
  \code{\link{ppp}},
  \code{as.hyperframe.ppx},
  \code{as.data.frame.ppx}.
}
\examples{
   df <- data.frame(x=runif(4),y=runif(4),t=runif(4))
   X <- ppx(data=df, coord.type=c("s","s","t"))
   coords(X)
   coords(X, temporal=FALSE)
   coords(X) <- matrix(runif(12), ncol=3)
}
\keyword{spatial}
\keyword{manip}

\name{angles.psp}
\alias{angles.psp}
\title{Orientation Angles of Line Segments}
\description{
  Computes the orientation angle of each line segment
  in a line segment pattern.
}
\usage{
  angles.psp(x, directed=FALSE)
}
\arguments{
  \item{x}{
    A line segment pattern (object of class \code{"psp"}).
  }
  \item{directed}{
    Logical flag. See details.
  }
}
\value{
  Numeric vector.
}
\details{
  For each line segment,  the angle of inclination to the \eqn{x}-axis
  (in radians) is computed,
  and the angles are returned as a numeric vector.

  If \code{directed=TRUE}, the directed angle of orientation
  is computed. The angle respects the
  sense of direction from \code{(x0,y0)} to \code{(x1,y1)}.
  The values returned are angles in the full range from \eqn{-\pi}{-\pi}
  to \eqn{\pi}{\pi}. The angle is computed as 
  \code{atan2(y1-y0,x1-x0)}. See \code{\link{atan2}}.

  If \code{directed=FALSE}, the undirected angle of orientation
  is computed. Angles differing by \eqn{\pi} are
  regarded as equivalent. The values returned are angles
  in the range from \eqn{0} to \eqn{\pi}{\pi}. These angles are
  computed by first computing the directed angle,
  then adding \eqn{\pi}{\pi} to any negative angles.
}
\seealso{
  \code{\link{psp}},
  \code{\link{marks.psp}},
  \code{\link{summary.psp}},
  \code{\link{midpoints.psp}},
  \code{\link{lengths_psp}},
  \code{\link{endpoints.psp}},
  \code{\link{extrapolate.psp}}.
}
\examples{
  a <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  b <- angles.psp(a)   
}
\author{
  \adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{union.quad}
\alias{union.quad}
\title{Union of Data and Dummy Points}
\description{
  Combines the data and dummy points of a quadrature scheme
  into a single point pattern.
}
\usage{
 union.quad(Q)
}
\arguments{
  \item{Q}{A quadrature scheme (an object of class \code{"quad"}).}
}
\value{
  A point pattern (of class \code{"ppp"}).
}
\details{
  The argument \code{Q} should be a quadrature scheme (an object of class
  \code{"quad"}, see \code{\link{quad.object}} for details).
  
  This function combines the data and dummy points of \code{Q}
  into a single point pattern. If either the data or the dummy points
  are marked, the result is a marked point pattern.

  The function \code{\link{as.ppp}} will perform the same task.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{as.ppp}}
}
\examples{
  data(simdat)
  Q <- quadscheme(simdat, default.dummy(simdat))
  U <- union.quad(Q)
  \dontrun{plot(U)}
  # equivalent:
  U <- as.ppp(Q)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{transmat}
\alias{transmat}
\title{
  Convert Pixel Array Between Different Conventions
}
\description{
  This function provides a simple way to convert arrays of pixel data
  between different display conventions.
}
\usage{
transmat(m, from, to)
}
\arguments{
  \item{m}{
    A matrix.
  }
  \item{from,to}{
    Specifications of the spatial arrangement of the pixels.
    See Details.
  }
}
\details{
  Pixel images are handled by many different software packages.
  In virtually all of these, the pixel values are stored in a matrix,
  and are accessed using the row and column indices of the matrix.
  However, different pieces of software use different conventions for
  mapping the matrix indices \eqn{[i,j]} to the spatial coordinates
  \eqn{(x,y)}.

  \itemize{
    \item
    In the \emph{Cartesian} convention, the first matrix index \eqn{i}
    is associated with the first Cartesian coordinate \eqn{x}, 
    and \eqn{j} is associated with \eqn{y}. This convention is used in
    \code{\link[graphics]{image.default}}.
    \item
    In the \emph{European reading order} convention, a matrix is displayed
    in the spatial coordinate system as it would be printed in a page of text:
    \eqn{i} is effectively associated with the negative \eqn{y} coordinate, 
    and \eqn{j} is associated with \eqn{x}. This convention is used in some
    image file formats.
    \item
    In the \code{spatstat} convention, \eqn{i}
    is associated with the increasing \eqn{y} coordinate, 
    and \eqn{j} is associated with \eqn{x}. This is also used in some
    image file formats.
  }
  To convert between these conventions, use the function
  \code{transmat}. If a matrix \code{m} contains pixel image data
  that is correctly displayed by software that uses the Cartesian convention,
  and we wish to convert it to the European reading convention, we can type
  \code{mm <- transmat(m, from="Cartesian", to="European")}.
  The transformed matrix \code{mm} will then be correctly displayed by
  software that uses the European convention. 

  Each of the arguments \code{from} and \code{to} can be one of the names
  \code{"Cartesian"}, \code{"European"} or \code{"spatstat"} (partially matched)
  or it can be a list specifying another convention. For example
  \code{to=list(x="-i", y="-j")!} specifies that rows of the output matrix 
  are expected to be displayed as vertical columns in the plot, 
  starting at the right side of the plot, as in the traditional 
  Chinese, Japanese and Korean writing order.
}
\value{
  Another matrix obtained by rearranging the entries of \code{m}.
}
\author{
\adrian


\rolf

and \ege

}
\examples{
  opa <- par(mfrow=c(1,2))
  # image in spatstat format
  Z <- bei.extra$elev
  plot(Z, main="plot.im", ribbon=FALSE)
  m <- as.matrix(Z)
  # convert matrix to format suitable for display by image.default
  Y <- transmat(m, from="spatstat", to="Cartesian")
  image(Y, asp=0.5, main="image.default", axes=FALSE)
  par(opa)
}
\keyword{spatial}
\keyword{hplot}
\keyword{manip}

\name{delaunayDistance}
\alias{delaunayDistance}
\title{Distance on Delaunay Triangulation}
\description{
  Computes the graph distance in the Delaunay triangulation
  of a point pattern.
}
\usage{
delaunayDistance(X)
}
\arguments{
  \item{X}{Spatial point pattern (object of class \code{"ppp"}).}
}
\details{
  The Delaunay triangulation of a spatial point pattern \code{X}
  is defined as follows. First the Dirichlet/Voronoi tessellation of \code{X}
  computed; see \code{\link{dirichlet}}. Then two points of \code{X}
  are defined to be Delaunay neighbours if their Dirichlet/Voronoi tiles
  share a common boundary. Every pair of Delaunay neighbours is
  joined by a straight line.

  The \emph{graph distance} 
  in the Delaunay triangulation between two points \code{X[i]} and \code{X[j]}
  is the minimum number of edges of the Delaunay triangulation
  that must be traversed to go from \code{X[i]} to \code{X[j]}.

  This command returns a matrix \code{D} such that
  \code{D[i,j]} is the graph distance
  between \code{X[i]} and \code{X[j]}.
}
\value{
  A symmetric square matrix with integer entries.
}
\seealso{
  \code{\link{delaunay}},
  \code{\link{delaunayNetwork}}
}
\examples{
  X <- runifpoint(20)
  M <- delaunayDistance(X)
  plot(delaunay(X), lty=3)
  text(X, labels=M[1, ], cex=2)
}
\author{
\adrian


\rolf

and \ege

}
\keyword{spatial}
\keyword{manip}

\name{rMosaicSet}
\alias{rMosaicSet}
\title{Mosaic Random Set}
\description{
  Generate a random set by taking a random selection of 
  tiles of a given tessellation.  
}
\usage{
rMosaicSet(X, p=0.5)
}
\arguments{
  \item{X}{
    A tessellation (object of class \code{"tess"}).
  }
  \item{p}{
    Probability of including a given tile.
    A number strictly between 0 and 1.
  }
}
\details{
  Given a tessellation \code{X}, this function 
  randomly selects some of the tiles of \code{X},
  including each tile with probability \eqn{p} independently of
  the other tiles. The selected tiles are then combined to form a
  set in the plane. 

  One application of this is Switzer's (1965) example of a random set
  which has a Markov property. It is constructed by generating \code{X}
  according to a Poisson line tessellation (see \code{\link{rpoislinetess}}).
}
\value{
  A window (object of class \code{"owin"}).
}
\references{
  Switzer, P.
  A random set process in the plane with a Markovian property.
  \emph{Annals of Mathematical Statistics} \bold{36} (1965) 1859--1863.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{rpoislinetess}},
  \code{\link{rMosaicField}}
}
\examples{
   # Switzer's random set
   X <- rpoislinetess(3)
   plot(rMosaicSet(X, 0.5), col="green", border=NA)

   # another example
   plot(rMosaicSet(dirichlet(runifpoint(30)), 0.4))
}
\keyword{spatial}
\keyword{datagen}

\name{pixelcentres}
\alias{pixelcentres}
\title{
  Extract Pixel Centres as Point Pattern
}
\description{
  Given a pixel image or binary mask window,
  extract the centres of all pixels and return them as a point pattern.
}
\usage{
pixelcentres(X, W = NULL, ...)
}
\arguments{
  \item{X}{
    Pixel image (object of class \code{"im"})
    or window (object of class \code{"owin"}).
  }
  \item{W}{
    Optional window to contain the resulting point pattern.
  }
  \item{\dots}{
    Optional arguments defining the pixel resolution.
  }
}
\details{
  If the argument \code{X} is a pixel image, the result is a point
  pattern, consisting of the centre of every pixel whose pixel value
  is not \code{NA}. 

  If \code{X} is a window which is a binary mask, the result is a point
  pattern consisting of the centre of every pixel inside the window
  (i.e. every pixel for which the mask value is \code{TRUE}).

  Otherwise, \code{X} is first converted to a window, then
  converted to a mask using \code{\link{as.mask}}, then handled as
  above.
}
\value{
  A point pattern (object of class \code{"ppp"}).
}
\seealso{
  \code{\link{raster.xy}}
}
\examples{
  pixelcentres(letterR, dimyx=5)
}
\author{\adrian
  
  ,
  \rolf
  
  and \ege
}
\keyword{spatial}
\keyword{manip}

\name{quadratcount}
\alias{quadratcount}
\alias{quadratcount.ppp}
\alias{quadratcount.splitppp}
\title{Quadrat counting for a point pattern}
\description{
  Divides window into quadrats and 
  counts the numbers of points in each quadrat.
}
\usage{
  quadratcount(X, \dots)

  \method{quadratcount}{ppp}(X, nx=5, ny=nx, \dots,
               xbreaks=NULL, ybreaks=NULL, tess=NULL)

  \method{quadratcount}{splitppp}(X, \dots)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppp"})
    or a split point pattern (object of class \code{"splitppp"}).
  }
  \item{nx,ny}{
    Numbers of rectangular quadrats in the \eqn{x} and \eqn{y} directions.
    Incompatible with \code{xbreaks} and \code{ybreaks}.
  }
  \item{\dots}{Additional arguments passed to \code{quadratcount.ppp}.}
  \item{xbreaks}{
    Numeric vector giving the \eqn{x} coordinates of the
    boundaries of the rectangular quadrats. Incompatible with \code{nx}.
  }
  \item{ybreaks}{
    Numeric vector giving the \eqn{y} coordinates of the
    boundaries of the rectangular quadrats. Incompatible with \code{ny}.
  }
  \item{tess}{
    Tessellation (object of class \code{"tess"} or something acceptable
    to \code{\link{as.tess}}) determining the quadrats. Incompatible
    with \code{nx,ny,xbreaks,ybreaks}.
  }
}
\value{
  The value of \code{quadratcount.ppp} is a 
  contingency table containing the number of points in each
  quadrat. The table is also an object of the
  special class \code{"quadratcount"}
  and there is a plot method for this class.

  The value of \code{quadratcount.splitppp} is a list of such
  contingency tables, each containing the quadrat counts for one of the
  component point patterns in \code{X}.
  This list also has the class \code{"solist"} which has
  print and plot methods.
}
\details{
  Quadrat counting is an elementary technique for analysing spatial
  point patterns. See Diggle (2003).

  \bold{If \code{X} is a point pattern}, then 
  by default, the window containing the point pattern \code{X} is divided into
  an \code{nx * ny} grid of rectangular tiles or `quadrats'.
  (If the window is not a rectangle, then these tiles are intersected
  with the window.)
  The number of points of \code{X} falling in each quadrat is
  counted. These numbers are returned as a contingency table.

  If \code{xbreaks} is given, it should be a numeric vector
  giving the \eqn{x} coordinates of the quadrat boundaries.
  If it is not given, it defaults to a
  sequence of \code{nx+1} values equally spaced
  over the range of \eqn{x} coordinates in the window \code{Window(X)}.

  Similarly if \code{ybreaks} is given, it should be a numeric
  vector giving the \eqn{y} coordinates of the quadrat boundaries.
  It defaults to a vector of \code{ny+1} values
  equally spaced over the range of \eqn{y} coordinates in the window.
  The lengths of \code{xbreaks} and \code{ybreaks} may be different.

  Alternatively, quadrats of any shape may be used.
  The argument \code{tess} can be a tessellation (object of class
  \code{"tess"}) whose tiles will serve as the quadrats.
  
  The algorithm counts the number of points of \code{X}
  falling in each quadrat, and returns these counts as a
  contingency table. 

  The return value is a \code{table} which can be printed neatly.
  The return value is also a member of the special class
  \code{"quadratcount"}. Plotting the object will display the
  quadrats, annotated by their counts. See the examples.

  To perform a chi-squared test based on the quadrat counts,
  use \code{\link{quadrat.test}}.
  
  To calculate an estimate of intensity based on the quadrat counts,
  use \code{\link{intensity.quadratcount}}.

  To extract the quadrats used in a \code{quadratcount} object,
  use \code{\link{as.tess}}.

  \bold{If \code{X} is a split point pattern} (object of class
  \code{"splitppp"} then quadrat counting will be performed on
  each of the components point patterns, and the resulting
  contingency tables will be returned in a list. This list can be
  printed or plotted.

  Marks attached to the points are ignored by \code{quadratcount.ppp}.
  To obtain a separate contingency table for each type of point
  in a multitype point pattern,
  first separate the different points using \code{\link{split.ppp}},
  then apply \code{quadratcount.splitppp}. See the Examples.
}
\note{
  To perform a chi-squared test based on the quadrat counts,
  use \code{\link{quadrat.test}}.
}
\section{Warning}{
  If \code{Q} is the result of \code{quadratcount}
  using rectangular tiles, then \code{as.numeric(Q)}
  extracts the counts \bold{in the wrong order}.
  To obtain the quadrat counts in the same order as the
  tiles of the corresponding tessellation would be listed,
  use \code{as.vector(t(Q))}, which works in all cases.
}
\seealso{
  \code{\link{plot.quadratcount}},
  \code{\link{intensity.quadratcount}},
  \code{\link{quadrats}},
  \code{\link{quadrat.test}},
  \code{\link{tess}},
  \code{\link{hextess}},
  \code{\link{quadratresample}},
  \code{\link{miplot}}
}
\references{
  Diggle, P.J. \emph{Statistical analysis of spatial point patterns}.
  Academic Press, 2003.

  Stoyan, D. and Stoyan, H. (1994)
  \emph{Fractals, random shapes and point fields:
  methods of geometrical statistics.}
  John Wiley and Sons.
} 
\examples{
 X <- runifpoint(50)
 quadratcount(X)
 quadratcount(X, 4, 5)
 quadratcount(X, xbreaks=c(0, 0.3, 1), ybreaks=c(0, 0.4, 0.8, 1))
 qX <-  quadratcount(X, 4, 5)

 # plotting:
 plot(X, pch="+")
 plot(qX, add=TRUE, col="red", cex=1.5, lty=2)

 # irregular window
 data(humberside)
 plot(humberside)
 qH <- quadratcount(humberside, 2, 3)
 plot(qH, add=TRUE, col="blue", cex=1.5, lwd=2)

 # multitype - split
 plot(quadratcount(split(humberside), 2, 3))
 
 # quadrats determined by tessellation:
 B <- dirichlet(runifpoint(6))
 qX <- quadratcount(X, tess=B)
 plot(X, pch="+")
 plot(qX, add=TRUE, col="red", cex=1.5, lty=2)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}
 
 

\name{uniquemap.default}
\alias{uniquemap.default}
\alias{uniquemap.data.frame}
\alias{uniquemap.matrix}
\title{
  Map Duplicate Entries to Unique Entries
}
\description{
  Determine whether entries in a vector
  (or rows in a matrix or data frame)
  are duplicated,
  choose a unique representative for each set of duplicates,
  and map the duplicates to the unique representative.
}
\usage{
\method{uniquemap}{default}(x)

\method{uniquemap}{data.frame}(x)

\method{uniquemap}{matrix}(x)
}
\arguments{
  \item{x}{
    A vector, data frame or matrix,
    or another type of data.
  }
}
\details{
  The function \code{\link{uniquemap}} is generic, with methods
  for point patterns, data frames, and a default method.

  The default method expects a vector. It determines whether any entries
  of the vector \code{x} are duplicated,
  and constructs a mapping of the indices of \code{x}
  so that all duplicates are mapped to a unique representative index.

  The result is an integer vector \code{u} such that
  \code{u[j] = i} if
  the entries \code{x[i]} and \code{x[j]} are identical and
  point \code{i} has been chosen as the unique representative.
  The entry \code{u[i] = i} means either that point \code{i} is
  unique, or that it has been chosen as the unique representative
  of its equivalence class.

  The method for \code{data.frame} determines whether any rows of the
  data frame \code{x} are duplicated, and constructs a mapping of the
  row indices so that all duplicate rows are mapped to a unique
  representative row.
}
\value{
  An integer vector.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{uniquemap}},
  \code{\link{uniquemap.ppp}}
}
\examples{
  x <- c(3, 5, 2, 4, 2, 3)
  uniquemap(x)

  df <- data.frame(A=x, B=42)
  uniquemap(df)

  z <- cbind(x, 10-x)
  uniquemap(z)
}
\keyword{spatial}
\keyword{methods}

\name{closetriples}
\alias{closetriples}
\title{
  Close Triples of Points
}
\description{
  Low-level function to find all close triples of points.
}
\usage{
closetriples(X, rmax)
}
\arguments{
  \item{X}{
    Point pattern (object of class \code{"ppp"} or \code{"pp3"}).
  }
  \item{rmax}{
    Maximum distance between each pair of points in a triple.
  }
}
\details{
  This low-level function
  finds all triples of points in a point pattern
  in which each pair lies closer than \code{rmax}.
}
\value{
  A data frame with columns \code{i,j,k} giving the indices of the
  points in each triple, and a column \code{diam} giving the diameter
  (maximum pairwise distance) in the triple.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{closepairs}},
  \code{\link{Tstat}}.
}
\examples{
   closetriples(redwoodfull, 0.02)
   closetriples(redwoodfull, 0.005)
}
\keyword{spatial}
\keyword{math}

\name{lengths_psp}
\alias{lengths.psp}
\alias{lengths_psp}
\title{Lengths of Line Segments}
\description{
  Computes the length of each line segment
  in a line segment pattern.
}
\usage{
  lengths.psp(x, squared=FALSE)
  lengths_psp(x, squared=FALSE)
}
\arguments{
  \item{x}{
    A line segment pattern (object of class \code{"psp"}).
  }
  \item{squared}{
    Logical value indicating whether to return
    the squared lengths (\code{squared=TRUE})
    or the lengths themselves (\code{squared=FALSE}, the default).
  }
}
\value{
  Numeric vector.
}
\details{
  The length of each line segment is computed
  and the lengths are returned as a numeric vector.

  Using squared lengths may be more efficient for some purposes,
  for example, to find the length of the shortest segment,
  \code{sqrt(min(lengths.psp(x, squared=TRUE)))}
  is faster than \code{min(lengths.psp(x))}.

  The functions \code{lengths.psp} and \code{lengths_psp} are
  identical. We recommend using the newer name \code{lengths_psp}.
}
\section{Change of name}{
  The name of this function has changed from \code{lengths.psp}
  to \code{lengths_psp}, because the old name \code{lengths.psp}
  could be misinterpreted as a method for \code{\link[base]{lengths}}.
  
  The older function name \code{lengths.psp} is retained temporarily,
  for consistency with older code and documentation.
  
  In future versions of \pkg{spatstat}, the 
  function name \code{lengths.psp} will be removed.
  The newer function name \code{lengths_psp} should be used.
}
\seealso{
  \code{\link{marks.psp}},
  \code{\link{summary.psp}},
  \code{\link{midpoints.psp}},
  \code{\link{angles.psp}},
  \code{\link{endpoints.psp}},
  \code{\link{extrapolate.psp}}.
}
\examples{
  a <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  b <- lengths_psp(a)   
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{ppx}
\Rdversion{1.1}
\alias{ppx}
\title{
  Multidimensional Space-Time Point Pattern
}
\description{
  Creates a multidimensional space-time point pattern
  with any kind of coordinates and marks.
}
\usage{
ppx(data, domain=NULL, coord.type=NULL, simplify=FALSE)
}
\arguments{
  \item{data}{
    The coordinates and marks of the points.
    A \code{data.frame} or \code{hyperframe}.
  }
  \item{domain}{
    Optional. The space-time domain containing the points.
    An object in some appropriate format, or \code{NULL}.
  }
  \item{coord.type}{
    Character vector specifying how each column of
    \code{data} should be interpreted: as a spatial coordinate,
    a temporal coordinate, a local coordinate or a mark.
    Entries are partially matched to the values
    \code{"spatial"},
    \code{"temporal"},
    \code{"local"} and
    \code{"mark"}.
  }
  \item{simplify}{
    Logical value indicating whether to simplify the result
    in special cases.
    If \code{simplify=TRUE}, a two-dimensional point pattern
    will be returned as an object of class \code{"ppp"},
    and a three-dimensional point pattern will be returned as
    an object of class \code{"pp3"}.
    If \code{simplify=FALSE} (the default) then the result is always
    an object of class \code{"ppx"}.
  }
}
\details{
  An object of class \code{"ppx"}
  represents a marked point pattern
  in multidimensional space and/or time. There may be any
  number of spatial coordinates, any number of temporal coordinates,
  any number of local coordinates,
  and any number of mark variables. The individual marks may be
  atomic (numeric values, factor values, etc) or objects of any kind.

  The argument \code{data} should contain the coordinates and marks of
  the points. It should be a \code{data.frame} or more generally a
  \code{hyperframe} (see \code{\link{hyperframe}}) with
  one row of data for each point.

  Each column of \code{data} is either
  a spatial coordinate, a temporal coordinate,
  a local coordinate, or a mark variable.
  The argument \code{coord.type} determines how each column is interpreted.
  It should be a character vector, of length equal to the number of
  columns of \code{data}. It should contain strings that partially match
  the values \code{"spatial"}, \code{"temporal"}, \code{"local"} and
  \code{"mark"}. (The first letters will be sufficient.)
  
  By default (if \code{coord.type} is missing or \code{NULL}),
  columns of numerical data are assumed to represent
  spatial coordinates, while other columns are assumed to be marks.
}
\value{
  Usually an object of class \code{"ppx"}.
  If \code{simplify=TRUE} the result may be an object of
  class \code{"ppp"} or \code{"pp3"}.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{pp3}},
  \code{\link{print.ppx}} 
}
\examples{
   df <- data.frame(x=runif(4),y=runif(4),t=runif(4),
                    age=rep(c("old", "new"), 2),
                    size=runif(4))
   X <- ppx(data=df, coord.type=c("s","s","t","m","m"))
   X

   val <- 20 * runif(4)
   E <- lapply(val, function(s) { rpoispp(s) })
   hf <- hyperframe(t=val, e=as.listof(E))
   Z <- ppx(data=hf, domain=c(0,1))
   Z
}
\keyword{spatial}
\keyword{datagen}

\name{MinkowskiSum}
\alias{MinkowskiSum}
\alias{\%(+)\%}  %DoNotExport 
%NAMESPACE export("%(+)%")
\alias{dilationAny}
\title{Minkowski Sum of Windows}
\description{
  Compute the Minkowski sum of two spatial windows.
}
\usage{
MinkowskiSum(A, B)

A \%(+)\% B

dilationAny(A, B)
}
\arguments{
  \item{A,B}{
    Windows (objects of class \code{"owin"}),
    point patterns (objects of class \code{"ppp"})
    or line segment patterns (objects of class \code{"psp"})
    in any combination.
  }
}
\value{
  A window (object of class \code{"owin"}) except that
  if \code{A} is a point pattern, then the result is an object of the same
  type as \code{B} (and vice versa).
}
\details{
  The operator \code{A \%(+)\% B} and function \code{MinkowskiSum(A,B)}
  are synonymous: they both compute the
  Minkowski sum of the windows \code{A} and \code{B}.
  The function \code{dilationAny} computes the Minkowski dilation
  \code{A \%(+)\% reflect(B)}.
  
  The Minkowski sum 
  of two spatial regions \eqn{A} and \eqn{B}
  is another region, formed by taking all possible pairs of points,
  one in \eqn{A} and one in \eqn{B}, and adding them as vectors.
  The Minkowski Sum \eqn{A \oplus B}{A \%(+)\% B}
  is the set of all points \eqn{a+b} where \eqn{a} is in \eqn{A}
  and \eqn{b} is in \eqn{B}.
  A few common facts about the Minkowski sum are:
  \itemize{
    \item
    The sum is symmetric:
    \eqn{A \oplus B = B \oplus A}{A \%(+)\% B = B \%(+)\% A}.
    \item
    If \eqn{B} is a single point, then \eqn{A \oplus B}{A \%(+)\% B}
    is a shifted copy of \eqn{A}.
    \item
    If \eqn{A} is a square of side length \eqn{a},
    and \eqn{B} is a square of side length \eqn{b},
    with sides that are parallel to the coordinate axes,
    then \eqn{A \oplus B}{A \%(+)\% B} is a square of side length \eqn{a+b}.
    \item
    If \eqn{A} and \eqn{B} are discs of radius \eqn{r} and \eqn{s}
    respectively, then \eqn{A \oplus B}{A \%(+)\% B} is a disc
    of redius \eqn{r+s}.
    \item
    If \eqn{B} is a disc of radius \eqn{r} centred at the origin,
    then \eqn{A \oplus B}{A \%(+)\% B} is equivalent to the
    \emph{morphological dilation} of \eqn{A} by distance \eqn{r}.
    See \code{\link{dilation}}.
  }

  The Minkowski dilation is the closely-related region
  \eqn{A \oplus (-B)}{A \%(+)\% (-B)}
  where \eqn{(-B)} is the reflection of \eqn{B} through the origin.
  The Minkowski dilation is the set of all vectors \eqn{z}
  such that, if \eqn{B} is shifted by \eqn{z}, the resulting set
  \eqn{B+z} has nonempty intersection with \eqn{A}.
  
  The algorithm currently computes the result as a polygonal
  window using the \pkg{polyclip} library. 
  It will be quite slow if applied to binary mask windows.

  The arguments \code{A} and \code{B} can also be point patterns or line
  segment patterns. These are interpreted as spatial regions, the
  Minkowski sum is computed, and the result is returned as an object of
  the most appropriate type. The Minkowski sum of two point patterns is
  another point pattern. The Minkowski sum of a point pattern and a line
  segment pattern is another line segment pattern.
}
\seealso{
  \code{\link{dilation}},
  \code{\link{erosionAny}}
}
\examples{
  B <- square(0.2)
  RplusB <- letterR \%(+)\% B

  opa <- par(mfrow=c(1,2))
  FR <- grow.rectangle(Frame(letterR), 0.3)
  plot(FR, main="")
  plot(letterR, add=TRUE, lwd=2, hatch=TRUE, hatchargs=list(texture=5))
  plot(shift(B, vec=c(3.675, 3)),
       add=TRUE, border="red", lwd=2)
  plot(FR, main="")
  plot(letterR, add=TRUE, lwd=2, hatch=TRUE, hatchargs=list(texture=5))
  plot(RplusB, add=TRUE, border="blue", lwd=2,
         hatch=TRUE, hatchargs=list(col="blue"))
  par(opa)

  plot(cells \%(+)\% square(0.1))
}
\author{
  \adrian
}
\keyword{spatial}
\keyword{math}

\name{rpoisline}
\alias{rpoisline}
\title{Generate Poisson Random Line Process}
\description{
  Generate a random pattern of line segments
  obtained from the Poisson line process.
}
\usage{
 rpoisline(lambda, win=owin())
}
\arguments{
  \item{lambda}{
    Intensity of the Poisson line process.
    A positive number.
  }
  \item{win}{
    Window in which to simulate the pattern.
    An object of class \code{"owin"}
    or something acceptable to \code{\link{as.owin}}.
  }
}
\value{
  A line segment pattern (an object of class \code{"psp"}).

  The result also has an attribute called \code{"lines"} (an object of
  class \code{"infline"} specifying the original infinite random lines)
  and an attribute \code{"linemap"} (an integer vector mapping the line
  segments to their parent lines).
}
\details{
  This algorithm generates a realisation
  of the uniform Poisson line process, and clips it to the window
  \code{win}.

  The argument \code{lambda} must be a positive number.
  It controls the intensity of the process. The expected number of
  lines intersecting a convex region of the plane is equal to
  \code{lambda} times the perimeter length of the region.
  The expected total length of the lines crossing a region of the plane
  is equal to \code{lambda * pi} times the area of the region.
}
\seealso{
\code{\link{psp}}
}
\examples{
 # uniform Poisson line process with intensity 10,
 # clipped to the unit square
 rpoisline(10)
}
\author{
  \adrian
  and \rolf
}
\keyword{spatial}
\keyword{datagen}

\name{is.marked.ppp}
\alias{is.marked.ppp}
\title{Test Whether A Point Pattern is Marked}
\description{
  Tests whether a point pattern has ``marks''
  attached to the points.
}
\usage{
  \method{is.marked}{ppp}(X, na.action="warn", \dots) 
}
\arguments{
  \item{X}{
    Point pattern (object of class \code{"ppp"})
  }
  \item{na.action}{
    String indicating what to do if \code{NA} values are
    encountered amongst the marks.
    Options are \code{"warn"}, \code{"fatal"} and \code{"ignore"}.
  }
  \item{\dots}{
    Ignored.
  }
}
\value{
  Logical value, equal to \code{TRUE} if \code{X} is a marked point pattern.
}
\details{
  ``Marks'' are observations attached to each point of a point pattern.
  For example the \code{\link[spatstat.data]{longleaf}} dataset
  contains the locations of trees, each tree being marked by its diameter;
  the \code{\link[spatstat.data]{amacrine}} dataset gives the locations of cells
  of two types (on/off) and the type of cell may be regarded as a mark attached
  to the location of the cell.

  This function tests whether the point pattern \code{X}
  contains or involves marked points.
  It is a method for the generic function \code{\link{is.marked}}.

  The argument \code{na.action} determines what action will be taken
  if the point pattern has a vector of marks but some or all of the
  marks are \code{NA}. Options are   \code{"fatal"} to cause a fatal
  error; \code{"warn"} to issue a warning and then return \code{TRUE};
  and \code{"ignore"} to take no action except returning \code{TRUE}.
}
\seealso{
  \code{\link{is.marked}},
  \code{\link{is.marked.ppm}}
}
\examples{
   data(cells)
   is.marked(cells)  #FALSE
   data(longleaf)
   is.marked(longleaf) #TRUE
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{marks}
\alias{marks}
\alias{marks.ppp}           
\alias{marks.ppx}           
\alias{marks<-}           
\alias{marks<-.ppp}           
\alias{marks<-.ppx}           
\alias{setmarks}
\alias{\%mark\%}  %DoNotExport 
%NAMESPACE export("%mark%")
\title{Marks of a Point Pattern}
\description{
  Extract or change the marks attached to
  a point pattern dataset.
}
\usage{
marks(x, \dots)

\method{marks}{ppp}(x, \dots, dfok=TRUE, drop=TRUE)           

\method{marks}{ppx}(x, \dots, drop=TRUE)

marks(x, \dots) <- value

\method{marks}{ppp}(x, \dots, dfok=TRUE, drop=TRUE) <- value

\method{marks}{ppx}(x, \dots) <- value

setmarks(x, value)

x \%mark\% value
}
\arguments{
  \item{x}{
    Point pattern dataset (object of class \code{"ppp"} or \code{"ppx"}).
  }
  \item{\dots}{
    Ignored.
  }
  \item{dfok}{
    Logical. If \code{FALSE}, data frames of marks are not permitted
    and will generate an error.
  }
  \item{drop}{
    Logical. If \code{TRUE}, a data frame consisting of a single column
    of marks will be converted to a vector or factor.
  }
  \item{value}{
    Replacement value. A vector, data frame or hyperframe of mark values,
    or \code{NULL}. 
  }
}
\value{
  For \code{marks(x)}, the result is a vector, factor, data frame or hyperframe,
  containing the mark values attached to the points of \code{x}.

  For \code{marks(x) <- value}, the result is the updated point pattern
  \code{x} (with the side-effect that the dataset \code{x} is updated in
  the current environment).

  For \code{setmarks(x,value)} and \code{x \%mark\% value}, the return value
  is the point pattern obtained by replacing the
  marks of \code{x} by \code{value}.
}
\details{
  These functions extract or change the marks
  attached to the points of the point pattern \code{x}.
  
  The expression \code{marks(x)} extracts the marks of \code{x}.
  The assignment \code{marks(x) <- value} assigns new marks to the
  dataset \code{x}, and updates the dataset \code{x} in the current
  environment. The expression \code{setmarks(x,value)}
  or equivalently \code{x \%mark\% value} returns a point pattern
  obtained by replacing the marks of \code{x} by \code{value}, but does
  not change the dataset \code{x} itself.
  
  For point patterns in two-dimensional space (objects of class
  \code{"ppp"}) the marks can be a vector, a factor, or a data frame.
  
  For general point patterns (objects of class "ppx") the
  marks can be a vector, a factor, a data frame or a
  hyperframe.
  
  For the assignment \code{marks(x) <- value}, the \code{value}
  should be a vector or factor of length equal to the number of
  points in \code{x}, or a data frame or hyperframe with as many rows
  as there are points in \code{x}. If \code{value} is a single value,
  or a data frame or hyperframe with one row, then it will be replicated
  so that the same marks will be attached to each point.
  
  To remove marks, use \code{marks(x) <- NULL} or
  \code{\link{unmark}(x)}.
  
  Use \code{\link{ppp}} or \code{\link{ppx}}
  to create point patterns in more general
  situations.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{ppx}},
  \code{\link{unmark}},
  \code{\link{hyperframe}}
}
\examples{
   X <- amacrine
   # extract marks
   m <- marks(X)
   # recode the mark values "off", "on" as 0, 1
   marks(X) <- as.integer(m == "on")
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}

\name{rpoislpp}
\alias{rpoislpp}
\title{
  Poisson Point Process on a Linear Network
}
\description{
  Generates a realisation of the Poisson point process
  with specified intensity on the given linear network.
}
\usage{
rpoislpp(lambda, L, \dots, nsim=1, drop=TRUE)
}
\arguments{
  \item{lambda}{
    Intensity of the Poisson process. 
    A single number, a \code{function(x,y)}, a pixel image
    (object of class \code{"im"}), or a vector of numbers,
    a list of functions, or a list of images.
  }
  \item{L}{
    A linear network (object of class \code{"linnet"},
    see \code{\link{linnet}}).
    Can be omitted in some cases: see Details.
  }
  \item{\dots}{
    Arguments passed to \code{\link{rpoisppOnLines}}.
  }
  \item{nsim}{Number of simulated realisations to generate.}
  \item{drop}{
    Logical value indicating what to do when \code{nsim=1}.
    If \code{drop=TRUE} (the default), the result is a point pattern.
    If \code{drop=FALSE}, the result is a list with one entry which is a
    point pattern.
  }
}
\details{
  This function uses \code{\link{rpoisppOnLines}}
  to generate the random points.

  Argument \code{L} can be omitted, and defaults to \code{as.linnet(lambda)},
  when \code{lambda} is a function on a linear network (class
  \code{"linfun"}) or a pixel image on a linear network (\code{"linim"}).
}
\value{
  If \code{nsim = 1} and \code{drop=TRUE},
  a point pattern on the linear network,
  i.e.\ an object of class \code{"lpp"}.
  Otherwise, a list of such point patterns.
}
\author{
  Ang Qi Wei \email{aqw07398@hotmail.com} and
  \adrian 
}
\seealso{
 \code{\link{runiflpp}},
 \code{\link{rlpp}},
 \code{\link{lpp}},
 \code{\link{linnet}}
}
\examples{
   X <- rpoislpp(5, simplenet)
   plot(X)
   # multitype
   X <- rpoislpp(c(a=5, b=5), simplenet)
}
\keyword{spatial}
\keyword{datagen}

\name{sidelengths.owin} 
\alias{sidelengths.owin}
\alias{shortside.owin}
\title{Side Lengths of Enclosing Rectangle of a Window}
\description{
  Computes the side lengths of the (enclosing rectangle of) a window.
}
\usage{
 \method{sidelengths}{owin}(x)

 \method{shortside}{owin}(x)
}
\arguments{
  \item{x}{
    A window whose side lengths will be computed.
    Object of class \code{"owin"}.
  }
}
\value{
  For \code{sidelengths.owin}, a numeric vector of length 2
  giving the side-lengths (\eqn{x} then \eqn{y}) of the enclosing rectangle.
  For \code{shortside.owin}, a numeric value.
}
\details{
  The functions \code{shortside} and \code{sidelengths} are generic.
  The functions documented here are the methods for the class \code{"owin"}.

  \code{sidelengths.owin} computes the 
  side-lengths of the enclosing rectangle of the window \code{x}.

  For safety, both functions give a warning if the window is not a rectangle.
  To suppress the warning, first convert the window to a rectangle
  using \code{\link{as.rectangle}}.

  \code{shortside.owin} computes the minimum of the two side-lengths.
}
\seealso{
  \code{\link{shortside}},   \code{\link{sidelengths}}
  for the generic functions.
  
  \code{\link{area.owin}},
  \code{\link{diameter.owin}},
  \code{\link{perimeter}}
  for other geometric calculations on \code{"owin"} objects.
  
  \code{\link{owin}},
  \code{\link{as.owin}}.
}
\examples{
  w <- owin(c(0,2),c(-1,3))
  sidelengths(w)
  shortside(as.rectangle(letterR))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{Extract.splitppp}
\alias{[.splitppp}
\alias{[<-.splitppp}
\title{Extract or Replace Sub-Patterns}
\description{
  Extract or replace some of the sub-patterns in a split point pattern.
}
\usage{
  \method{[}{splitppp}(x, ...)
  \method{[}{splitppp}(x, ...) <- value
}
\arguments{
  \item{x}{
    An object of class \code{"splitppp"}, representing a point pattern
    separated into a list of sub-patterns.
  }
  \item{\dots}{
    Subset index. Any valid subset index in the usual \R sense.
  }
  \item{value}{
    Replacement value for the subset. A list of point patterns.
  }
}
\value{
  Another object of class \code{"splitppp"}.
}
\details{
  These are subset methods for the class \code{"splitppp"}.

  The argument \code{x} should be an object of class \code{"splitppp"},
  representing a point pattern that has been separated into a
  list of sub-patterns. It is created by \code{\link{split.ppp}}.
  
  The methods extract or replace a designated
  subset of the list \code{x}, and return an object of class \code{"splitppp"}.
}
\seealso{
  \code{\link{split.ppp}},
  \code{\link{plot.splitppp}},
  \code{\link{summary.splitppp}}
}
\examples{
  data(amacrine)  # multitype point pattern
  y <- split(amacrine)
  y[1]
  y["off"]
  y[1] <- list(runifpoint(42, Window(amacrine)))
 }
\author{
  \adrian
  and
  \rolf
}
\keyword{spatial}
\keyword{manip}

\name{dilation} 
\alias{dilation}
\alias{dilation.owin}
\alias{dilation.ppp}
\alias{dilation.psp}
\title{Morphological Dilation}
\description{
  Perform morphological dilation of a window, a line segment pattern
  or a point pattern
}
\usage{
 dilation(w, r, \dots)
 \method{dilation}{owin}(w, r, \dots, polygonal=NULL, tight=TRUE)
 \method{dilation}{ppp}(w, r, \dots, polygonal=TRUE, tight=TRUE)
 \method{dilation}{psp}(w, r, \dots, polygonal=TRUE, tight=TRUE)
}
\arguments{
  \item{w}{
    A window (object of class \code{"owin"}
    or a line segment pattern (object of class \code{"psp"})
    or a point pattern (object of class \code{"ppp"}).
  }
  \item{r}{positive number: the radius of dilation.}
  \item{\dots}{extra arguments passed to \code{\link{as.mask}}
    controlling the pixel resolution, if the pixel approximation is
    used; or passed to \code{\link{disc}} if the polygonal approximation
    is used.
  }
  \item{polygonal}{
    Logical flag indicating whether to compute a polygonal
    approximation to the dilation (\code{polygonal=TRUE}) or
    a pixel grid approximation (\code{polygonal=FALSE}).
  }
  \item{tight}{
    Logical flag indicating whether the bounding frame of the window
    should be taken as the smallest rectangle enclosing the dilated region
    (\code{tight=TRUE}), or should be the
    dilation of the bounding frame of \code{w} (\code{tight=FALSE}).
  }
}
\value{
  If \code{r > 0}, an object of class \code{"owin"} representing the
  dilated region. If \code{r=0}, the result is identical to \code{w}.
}
\details{
  The morphological dilation of a set \eqn{W} by a distance \eqn{r > 0}
  is the set consisting of all points lying at most \eqn{r} units
  away from \eqn{W}. Effectively, dilation adds a margin of width
  \eqn{r} onto the set \eqn{W}.

  If \code{polygonal=TRUE} then a polygonal approximation
  to the dilation is computed.
  If \code{polygonal=FALSE} then a pixel approximation
  to the dilation is computed from the distance map of \code{w}.
  The arguments \code{"\dots"} are passed to \code{\link{as.mask}}
  to control the pixel resolution.

  When \code{w} is a window, the default (when \code{polygonal=NULL})
  is to compute a polygonal approximation if
  \code{w} is a rectangle or polygonal window, and to compute a
  pixel approximation if \code{w} is a window of type \code{"mask"}.
}
\seealso{
  \code{\link{erosion}} for the opposite operation.
  
  \code{\link{dilationAny}} for morphological dilation using any shape.
  
  \code{\link{owin}},
  \code{\link{as.owin}}
}
\examples{
  plot(dilation(redwood, 0.05))
  points(redwood)

  plot(dilation(letterR, 0.2))
  plot(letterR, add=TRUE, lwd=2, border="red")

  X <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  plot(dilation(X, 0.1))
  plot(X, add=TRUE, col="red")

}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{math}

\name{concatxy}
\alias{concatxy}
\title{Concatenate x,y Coordinate Vectors}
\description{
  Concatenate any number of pairs of \code{x} and \code{y}
  coordinate vectors.
}
\usage{
  concatxy(\dots)
}
\arguments{
  \item{\dots}{
    Any number of arguments, each of which is a structure
    containing elements \code{x} and \code{y}.
  }
}
\value{
  A list with two components \code{x} and \code{y}, which are the
  concatenations of all the corresponding \code{x} and \code{y}
  vectors in the argument list.
}
\details{
  This function can be used to superimpose two or more point patterns
  of unmarked points (but see also \code{\link{superimpose}} which is
  recommended). 
  
  It assumes that each of the arguments in
  \code{\dots} is a structure containing (at least) the elements
  \code{x} and \code{y}. It concatenates all the \code{x} elements
  into a vector \code{x}, and similarly for \code{y}, and returns these
  concatenated vectors.
}
\seealso{
  \code{\link{superimpose}},
  \code{\link{quadscheme}}
}
\examples{
  dat <- runifrect(30)
  xy <- list(x=runif(10),y=runif(10))
  new <- concatxy(dat, xy)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{manip}

\name{applynbd}
\alias{applynbd}
\title{Apply Function to Every Neighbourhood in a Point Pattern}
\description{
  Visit each point in a point pattern, find the neighbouring points,
  and apply a given function to them.
}
\usage{
   applynbd(X, FUN, N=NULL, R=NULL, criterion=NULL, exclude=FALSE, \dots)
}
\arguments{
  \item{X}{
    Point pattern. 
    An object of class \code{"ppp"},
    or data which can be converted into 
    this format by \code{\link{as.ppp}}.
  }
  \item{FUN}{
    Function to be applied to each neighbourhood.
    The arguments of \code{FUN} are described under \bold{Details}.
  }
  \item{N}{
    Integer. If this argument is present,
    the neighbourhood of a point of \code{X} is defined to consist of the
    \code{N} points of \code{X} which are closest to it.
  }
  \item{R}{
    Nonnegative numeric value. If this argument is present,
    the neighbourhood of a point of \code{X} is defined to consist of
    all points of \code{X} which lie within a distance \code{R}
    of it.
  }
  \item{criterion}{
    Function. If this argument is present, 
    the neighbourhood of a point of \code{X} is determined by
    evaluating this function. See under \bold{Details}.
  }
  \item{exclude}{
    Logical. If \code{TRUE} then the point currently being visited
    is excluded from its own neighbourhood.
  }
  \item{\dots}{
    extra arguments passed to the function \code{FUN}.
    They must be given in the form \code{name=value}.
  }
}
\value{
  Similar to the result of \code{\link{apply}}.
  If each call to \code{FUN} returns a single numeric value,
  the result is a vector of dimension \code{npoints(X)}, the number of points
  in \code{X}.
  If each call to \code{FUN} returns a vector of the same length
  \code{m}, then the result is a matrix of dimensions \code{c(m,n)};
  note the transposition of the indices, as usual for the family of
  \code{apply} functions.
  If the calls to \code{FUN} return vectors of different lengths,
  the result is a list of length \code{npoints(X)}. 
}
\details{
  This is an analogue of \code{\link{apply}}
  for point patterns. It visits each point in the point pattern \code{X},
  determines which points of \code{X} are ``neighbours'' of the current
  point, applies the function \code{FUN} to this neighbourhood,
  and collects the values returned by \code{FUN}.

  The definition of ``neighbours'' depends on the arguments
  \code{N}, \code{R} and \code{criterion}.
  Also the argument \code{exclude} determines whether
  the current point is excluded from its own neighbourhood.

  \itemize{
    \item 
    If \code{N} is given, then the neighbours of the current
    point are the \code{N} points of \code{X} which are closest to
    the current point (including the current point itself
    unless \code{exclude=TRUE}).
    \item
    If \code{R} is given, then the neighbourhood of the current point
    consists of all points of \code{X} which lie closer than a distance \code{R}
    from the current point.
    \item 
    If \code{criterion} is given, then it must be a function
    with two arguments \code{dist} and \code{drank} which will be
    vectors of equal length.
    The interpretation is that \code{dist[i]} will be the
    distance of a point from the current point, and
    \code{drank[i]} will be the rank of that distance (the three points
    closest to the current point will have rank 1, 2 and 3).
    This function must return a logical vector of the same length
    as \code{dist} and \code{drank} whose \code{i}-th entry is
    \code{TRUE} if the corresponding point should be included in
    the neighbourhood. See the examples below.
    \item
    If more than one of the arguments \code{N}, \code{R} and
    \code{criterion} is given, the neighbourhood is defined as
    the \emph{intersection} of the neighbourhoods specified by these arguments.
    For example if \code{N=3} and \code{R=5} then the neighbourhood
    is formed by finding the 3 nearest neighbours of current point,
    and retaining only those neighbours which lie closer than 5 units
    from the current point.
  }

  When \code{applynbd} is executed, 
  each point of \code{X} is visited, and the following happens
  for each point:
  \itemize{
    \item
    the neighbourhood of the current point is determined according
    to the chosen rule, and stored as a point pattern \code{Y};
    \item
    the function \code{FUN} is called as:
  
    \code{FUN(Y=Y, current=current, dists=dists, dranks=dranks, \dots)}

    where \code{current} is the location of the current point
    (in a format explained below),
    \code{dists} is a vector of distances from the current
    point to each of the points in \code{Y}, 
    \code{dranks} is a vector of the ranks of these distances
    with respect to the full point pattern \code{X},
    and \code{\dots} are the arguments passed from the call to
    \code{applynbd};
    \item
    The result of the call to \code{FUN} is stored.
  }
  The results of each call to \code{FUN} are collected and returned
  according to the usual rules for \code{\link{apply}} and its
  relatives. See the \bold{Value} section of this help file.

  The format of the argument \code{current} is as follows.
  If \code{X} is an unmarked point pattern, then \code{current} is a
  vector of length 2 containing the coordinates of the current point.
  If \code{X} is marked, then \code{current} is a point pattern
  containing exactly one point, so that \code{current$x} is its
  \eqn{x}-coordinate and \code{current$marks} is its mark value.
  In either case, the coordinates of the current point can be referred to as
  \code{current$x} and \code{current$y}.

  Note that \code{FUN} will be called exactly as described above,
  with each argument named explicitly. Care is required when writing the
  function \code{FUN} to ensure that
  the arguments will match up. See the Examples.

  See \code{\link{markstat}} for a common use of this function.

  To simply tabulate the marks in every \code{R}-neighbourhood, use
  \code{\link{marktable}}.
}
\seealso{
  \code{\link{ppp.object}},
  \code{\link{apply}},
  \code{\link{markstat}},
  \code{\link{marktable}}
}
\examples{
  redwood
  # count the number of points within radius 0.2 of each point of X
  nneighbours <- applynbd(redwood, R=0.2, function(Y, ...){npoints(Y)-1})
  # equivalent to:
  nneighbours <- applynbd(redwood, R=0.2, function(Y, ...){npoints(Y)}, exclude=TRUE)

  # compute the distance to the second nearest neighbour of each point
  secondnndist <- applynbd(redwood, N = 2,
                           function(dists, ...){max(dists)},
                           exclude=TRUE)

  # marked point pattern
  trees <- longleaf
  \testonly{
	# smaller dataset
	trees <- trees[seq(1, npoints(trees), by=80)]
  }
  # compute the median of the marks of all neighbours of a point
  # (see also 'markstat')
  dbh.med <- applynbd(trees, R=90, exclude=TRUE,
                 function(Y, ...) { median(marks(Y))})


  # ANIMATION explaining the definition of the K function
  # (arguments `fullpicture' and 'rad' are passed to FUN)

  if(interactive()) {
  showoffK <- function(Y, current, dists, dranks, fullpicture,rad) { 
	plot(fullpicture, main="")
	points(Y, cex=2)
        ux <- current[["x"]]
        uy <- current[["y"]]
	points(ux, uy, pch="+",cex=3)
	theta <- seq(0,2*pi,length=100)
	polygon(ux + rad * cos(theta), uy+rad*sin(theta))
	text(ux + rad/3, uy + rad/2,npoints(Y),cex=3)
	if(interactive()) Sys.sleep(if(runif(1) < 0.1) 1.5 else 0.3)
	return(npoints(Y))
  }
  applynbd(redwood, R=0.2, showoffK, fullpicture=redwood, rad=0.2, exclude=TRUE)

  # animation explaining the definition of the G function

  showoffG <- function(Y, current, dists, dranks, fullpicture) { 
	plot(fullpicture, main="")
	points(Y, cex=2)
        u <- current
	points(u[1],u[2],pch="+",cex=3)
	v <- c(Y$x[1],Y$y[1])
	segments(u[1],u[2],v[1],v[2],lwd=2)
	w <- (u + v)/2
	nnd <- dists[1]
	text(w[1],w[2],round(nnd,3),cex=2)
	if(interactive()) Sys.sleep(if(runif(1) < 0.1) 1.5 else 0.3)
	return(nnd)
  }

  applynbd(cells, N=1, showoffG, exclude=TRUE, fullpicture=cells)
  }
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{programming}
\keyword{iteration}
 

\name{pairdist.ppx}
\alias{pairdist.ppx}
\title{Pairwise Distances in Any Dimensions}
\description{
  Computes the matrix of distances between all pairs of points
  in a multi-dimensional point pattern.
}
\usage{
  \method{pairdist}{ppx}(X, \dots)
}
\arguments{
  \item{X}{
    A point pattern (object of class \code{"ppx"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{coords.ppx}} to determine
    which coordinates should be used.
  }
}
\value{
  A square matrix whose \code{[i,j]} entry is the distance
  between the points numbered \code{i} and \code{j}.
}
\details{
  This is a method for the generic function \code{pairdist}.

  Given a multi-dimensional point pattern \code{X}
  (an object of class \code{"ppx"}),
  this function computes the Euclidean distances between all pairs of
  points in \code{X}, and returns the matrix of distances.

  By default, both spatial and temporal coordinates are extracted.
  To obtain the spatial distance between points in a space-time point
  pattern, set \code{temporal=FALSE}.
}
\seealso{
  \code{\link{pairdist}},
  \code{\link{crossdist}},
  \code{\link{nndist}}
}
\examples{
   df <- data.frame(x=runif(4),y=runif(4),z=runif(4),w=runif(4))
   X <- ppx(data=df)
   pairdist(X)
}
\author{
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{stieltjes}
\alias{stieltjes}
\title{Compute Integral of Function Against Cumulative Distribution}
\description{
  Computes the Stieltjes integral 
  of a function \eqn{f} with respect to a function \eqn{M}.
}
\usage{
stieltjes(f, M, ...)
}
\arguments{
  \item{f}{
    The integrand. A function in the \R language.
  }
  \item{M}{
    The cumulative function against which \code{f} will be
    integrated. An object of class \code{"fv"} or \code{"stepfun"}.
  }
  \item{\dots}{
    Additional arguments passed to \code{f}.
  }
}
\details{
  This command computes the Stieltjes integral
  \deqn{I = \int f(x) dM(x)}{I = integral f(x) dM(x)}
  of a real-valued function \eqn{f(x)}
  with respect to a nondecreasing function \eqn{M(x)}.

  One common use of the Stieltjes integral is
  to find the mean value of a random variable from its
  cumulative distribution function \eqn{F(x)}. The mean value is
  the Stieltjes integral of \eqn{f(x)=x} with respect to \eqn{F(x)}.

  The argument \code{f} should be a \code{function} in the \R language.
  It should accept a numeric vector argument \code{x} and should return
  a numeric vector of the same length.

  The argument \code{M} should be either a step function
  (object of class \code{"stepfun"}) or a function value table
  (object of class \code{"fv"}, see \code{\link{fv.object}}).
  Objects of class \code{"stepfun"} are returned by
  \code{\link[stats]{ecdf}}, \code{\link{ewcdf}},
  \code{\link{spatialcdf}} and other utilities.
  Objects of class \code{"fv"} are returned
  by the commands \code{\link{Kest}}, \code{\link{Gest}}, etc.
}
\value{
  A list containing the value of the Stieltjes integral
  computed using each of the versions of the function \code{M}.
}
\seealso{
  \code{\link{fv.object}},
  \code{\link{Gest}}
}
\examples{
  # estimate cdf of nearest neighbour distance in redwood data
  G <- Gest(redwood)
  # compute estimate of mean nearest neighbour distance
  stieltjes(function(x){x}, G)
  # estimated probability of a distance in the interval [0.1,0.2]
  stieltjes(function(x,a,b){ (x >= a) & (x <= b)}, G, a=0.1, b=0.2)

  # stepfun example
  H <- spatialcdf(bei.extra$elev, normalise=TRUE)
  stieltjes(function(x){x}, H)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{default.dummy}
\alias{default.dummy}
\title{Generate a Default Pattern of Dummy Points}
\description{
  Generates a default pattern of dummy points
  for use in a quadrature scheme.
}
\usage{
 default.dummy(X, nd, random=FALSE, ntile=NULL, npix=NULL,
              quasi=FALSE, \dots, eps=NULL, verbose=FALSE)
}
\arguments{
  \item{X}{
    The observed data point pattern.
    An object of class \code{"ppp"}
    or in a format recognised by \code{\link{as.ppp}()}
  }
  \item{nd}{
    Optional. Integer, or integer vector of length 2, specifying an
    \code{nd * nd} or \code{nd[1] * nd[2]}
    rectangular array of dummy points. 
  }
  \item{random}{
    Logical value. If \code{TRUE}, the dummy points are
    generated randomly.
  }
  \item{quasi}{
    Logical value. If \code{TRUE}, the dummy points are
    generated by a quasirandom sequence.
  }
  \item{ntile}{
  	Optional. Integer or pair of integers specifying 
	the number of rows and columns of tiles used in the counting rule.
  }
  \item{npix}{
  	Optional. Integer or pair of integers specifying the
	number of rows and columns of pixels used in computing
	approximate areas.
  }
  \item{\dots}{
    Ignored.
  }
  \item{eps}{
    Optional. Grid spacing.
    A positive number, or a vector of two positive numbers, giving the
    horizontal and vertical spacing, respectively, of the grid of
    dummy points. Incompatible with \code{nd}.
  }
  \item{verbose}{
    If \code{TRUE}, information about the construction of the
    quadrature scheme is printed.
  }
} 
\value{
  A point pattern (an object of class \code{"ppp"},
  see \code{\link{ppp.object}}) containing the dummy points.
}
\details{
  This function provides a sensible default for the dummy points
  in a quadrature scheme.

  A quadrature scheme consists of 
  the original data point pattern, an additional pattern of dummy points,
  and a vector of quadrature weights for all these points.
  See \code{\link{quad.object}} for further information about
  quadrature schemes.

  If \code{random} and \code{quasi} are both false (the default),
  then the function creates dummy points
  in a regular \code{nd[1]} by \code{nd[1]} rectangular grid.
  If \code{random} is true and \code{quasi} is false,
  then the frame of the window is divided into
  an  \code{nd[1]} by \code{nd[1]} array of tiles, and one dummy point
  is generated at random inside each tile.
  If \code{quasi} is true, a quasirandom pattern of
  \code{nd[1] * nd[2]} points is generated.
  In all cases, the four corner points of the frame of the window
  are added. Then if the window is not rectangular, any dummy points
  lying outside it are deleted. 
  
  If \code{nd} is missing, 
  a default value (depending on the
  data pattern \code{X}) is computed by \code{default.ngrid}.

  Alternative functions for creating dummy patterns
  include \code{\link{corners}},
  \code{\link{gridcentres}},
  \code{\link{stratrand}} and
  \code{\link{spokes}}.
}
\seealso{
  \code{\link{quad.object}},
  \code{\link{quadscheme}},
  \code{\link{corners}},
  \code{\link{gridcentres}},
  \code{\link{stratrand}},
  \code{\link{spokes}}
}
\examples{
  data(simdat)
  P <- simdat
  D <- default.dummy(P, 100)
  \dontrun{plot(D)}
  Q <- quadscheme(P, D, "grid")
  \dontrun{plot(union.quad(Q))}
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{datagen}

\name{closing} 
\alias{closing}
\alias{closing.owin}
\alias{closing.ppp}
\alias{closing.psp}
\title{Morphological Closing}
\description{
  Perform morphological closing of a window, a line segment pattern
  or a point pattern.
}
\usage{
 closing(w, r, \dots)

 \method{closing}{owin}(w, r, \dots, polygonal=NULL)

 \method{closing}{ppp}(w, r, \dots, polygonal=TRUE)

 \method{closing}{psp}(w, r, \dots, polygonal=TRUE)
}
\arguments{
  \item{w}{
    A window (object of class \code{"owin"}
    or a line segment pattern (object of class \code{"psp"})
    or a point pattern (object of class \code{"ppp"}).
  }
  \item{r}{positive number: the radius of the closing.}
  \item{\dots}{extra arguments passed to \code{\link{as.mask}}
    controlling the pixel resolution, if a pixel approximation is used}
  \item{polygonal}{
    Logical flag indicating whether to compute a polygonal
    approximation to the erosion (\code{polygonal=TRUE}) or
    a pixel grid approximation (\code{polygonal=FALSE}).
  }
}
\value{
  If \code{r > 0}, an object of class \code{"owin"} representing the
  closed region. If \code{r=0}, the result is identical to \code{w}.
}
\details{
  The morphological closing (Serra, 1982)
  of a set \eqn{W} by a distance \eqn{r > 0}
  is the set of all points that cannot be
  separated from \eqn{W} by any circle of radius \eqn{r}.
  That is, a point \eqn{x} belongs to the closing \eqn{W*}
  if it is impossible to draw any circle of radius \eqn{r} that
  has \eqn{x} on the inside and \eqn{W} on the outside.
  The closing \eqn{W*} contains the original set \eqn{W}.

  For a small radius \eqn{r}, the closing operation
  has the effect of smoothing out irregularities in the boundary of
  \eqn{W}. For larger radii, the closing operation smooths out
  concave features in the boundary. For very large radii,
  the closed set \eqn{W*} becomes more and more convex.

  The algorithm applies \code{\link{dilation}} followed by
  \code{\link{erosion}}. 
}
\seealso{
  \code{\link{opening}} for the opposite operation.

  \code{\link{dilation}}, \code{\link{erosion}} for the basic
  operations.  
  
  \code{\link{owin}},
  \code{\link{as.owin}} for information about windows.
}
\examples{
  v <- closing(letterR, 0.25)
  plot(v, main="closing")
  plot(letterR, add=TRUE)

  plot(closing(cells, 0.1))
  points(cells)
}
\references{
  Serra, J. (1982)
  Image analysis and mathematical morphology.
  Academic Press.
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{math}

\name{interp.colourmap}
\alias{interp.colourmap}
\title{
  Interpolate smoothly between specified colours
}
\description{
  Given a colourmap object which maps numbers to colours,
  this function interpolates smoothly between the colours,
  yielding a new colour map.
}
\usage{
interp.colourmap(m, n = 512)
}

\arguments{
  \item{m}{
    A colour map (object of class \code{"colourmap"}).
  }
  \item{n}{
    Number of colour steps to be created in the new colour map.
  }
}
\details{
  Given a colourmap object \code{m}, which maps numerical values to
  colours, this function interpolates the mapping, 
  yielding a new colour map.

  This makes it easy to build a colour map that
  has smooth gradation between different colours or shades.
  First specify a small vector of numbers \code{x} which should be mapped to
  specific colours \code{y}. Use \code{m <- colourmap(y, inputs=x)}
  to create a colourmap that represents this simple
  mapping. Then apply \code{interp.colourmap(m)}
  to obtain a smooth transition between these points.
}
\value{
  Another colour map (object of class \code{"colourmap"}).
}
\seealso{
  \code{\link{colourmap}}, 
  \code{\link{tweak.colourmap}}, 
  \code{\link[spatstat:colourtools]{colourtools}}.
}
\examples{
  co <- colourmap(inputs=c(0, 0.5, 1), c("black", "red", "white"))
  plot(interp.colourmap(co))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{color}

\name{plot.listof}
\alias{plot.listof}
\title{Plot a List of Things}
\description{
  Plots a list of things
}
\usage{
  \method{plot}{listof}(x, \dots, main, arrange=TRUE,
   nrows=NULL, ncols=NULL, main.panel=NULL,
   mar.panel=c(2,1,1,2), hsep=0, vsep=0,
   panel.begin=NULL, panel.end=NULL, panel.args=NULL,
   panel.begin.args=NULL, panel.end.args=NULL, panel.vpad=0.2, 
   plotcommand="plot",
   adorn.left=NULL, adorn.right=NULL, adorn.top=NULL, adorn.bottom=NULL,
   adorn.size=0.2, equal.scales=FALSE, halign=FALSE, valign=FALSE)
}
\arguments{
  \item{x}{
    An object of the class \code{"listof"}.
    Essentially a list of objects.
  }
  \item{\dots}{
    Arguments passed to \code{\link{plot}} when generating each
    plot panel.
  }
  \item{main}{
    Overall heading for the plot.
  }
  \item{arrange}{
    Logical flag indicating whether to plot the objects
    side-by-side on a single page (\code{arrange=TRUE})
    or plot them individually in a succession of frames
    (\code{arrange=FALSE}).
  }
  \item{nrows,ncols}{
    Optional. The number of rows/columns in the plot layout
    (assuming \code{arrange=TRUE}).
    You can specify either or both of these numbers.
  }
  \item{main.panel}{
    Optional. A character string, or a vector of character strings,
    giving the headings for each of the objects.
  }
  \item{mar.panel}{
    Size of the margins outside each plot panel.
    A numeric vector of length 4 giving the bottom, left, top,
    and right margins in that order. (Alternatively the vector
    may have length 1 or 2 and will be replicated to length 4).
    See the section on \emph{Spacing between plots}.
  }
  \item{hsep,vsep}{
    Additional horizontal and vertical separation between plot panels,
    expressed in the same units as \code{mar.panel}. 
  }
  \item{panel.begin,panel.end}{
    Optional. Functions that will be executed before and after each panel is
    plotted. See Details.
  }
  \item{panel.args}{
    Optional. Function that determines different plot arguments
    for different panels. See Details.
  }
  \item{panel.begin.args}{
    Optional. List of additional arguments for \code{panel.begin}
    when it is a function.
  }
  \item{panel.end.args}{
    Optional. List of additional arguments for \code{panel.end}
    when it is a function.
  }
  \item{panel.vpad}{
    Amount of extra vertical space that should be allowed for the
    title of each panel, if a title will be displayed.
    Expressed as a fraction of the height of the panel.
    Applies only when \code{equal.scales=FALSE} (the default) and
    requires that the height of each panel can be determined.
  }
  \item{plotcommand}{
    Optional.
    Character string containing the name of the command that should be
    executed to plot each panel. 
  }
  \item{adorn.left,adorn.right,adorn.top,adorn.bottom}{
    Optional. Functions (with no arguments) that will be executed to
    generate additional plots at the margins (left, right, top and/or
    bottom, respectively) of the array of plots.
  }
  \item{adorn.size}{
    Relative width (as a fraction of the other panels' widths)
    of the margin plots.
  }
  \item{equal.scales}{
    Logical value indicating whether the components
    should be plotted at (approximately) the same physical scale. 
  }
  \item{halign,valign}{
    Logical values indicating whether panels in a column
    should be aligned to the same \eqn{x} coordinate system
    (\code{halign=TRUE}) and whether panels in a row should
    be aligned to the same \eqn{y} coordinate system (\code{valign=TRUE}).
    These are applicable only if \code{equal.scales=TRUE}.
  }
}
\value{
  Null.
}
\details{
  This is the \code{plot} method for the class \code{"listof"}.

  An object of class \code{"listof"} (defined in the base R package) represents
  a list of objects, all belonging to a common class.
  The base R package defines a method for printing these objects,
  \code{\link[base]{print.listof}},
  but does not define a method for \code{plot}.
  So here we have provided a method for \code{plot}.

  In the \pkg{spatstat} package, various functions produce
  an object of class \code{"listof"}, essentially a list of
  spatial objects of the same kind. 
  These objects can be plotted in a nice arrangement
  using \code{plot.listof}. See the Examples.

  The argument \code{panel.args} determines extra graphics parameters
  for each panel. It should be a function that will be called
  as \code{panel.args(i)} where \code{i} is the panel number.
  Its return value should be a list of graphics parameters that can
  be passed to the relevant \code{plot} method. These parameters
  override any parameters specified in the \code{\dots} arguments.

  The arguments \code{panel.begin} and \code{panel.end}
  determine graphics that will be plotted before and after
  each panel is plotted. They may be objects
  of some class that can be plotted
  with the generic \code{plot} command. Alternatively they
  may be functions that will be
  called as \code{panel.begin(i, y, main=main.panel[i])}
  and \code{panel.end(i, y, add=TRUE)} where \code{i} is the panel
  number and \code{y = x[[i]]}.

  If all entries of \code{x} are pixel images,
  the function \code{\link{image.listof}} is called to control
  the plotting. The arguments \code{equal.ribbon} and \code{col}
  can be used to determine the colour map or maps applied.

  If \code{equal.scales=FALSE} (the default), then the 
  plot panels will have equal height on the plot device
  (unless there is only one column of panels, in which case
  they will have equal width on the plot device). This means that the
  objects are plotted at different physical scales, by default.

  If \code{equal.scales=TRUE}, then the dimensions of the
  plot panels on the plot device will be proportional
  to the spatial dimensions of the
  corresponding components of \code{x}. This means that the
  objects will be plotted at \emph{approximately} equal physical scales.
  If these objects have very different spatial sizes,
  the plot command could fail (when it tries
  to plot the smaller objects at a tiny scale), with an error
  message that the figure margins are too large.

  The objects will be plotted at \emph{exactly} equal
  physical scales, and \emph{exactly} aligned on the device,
  under the following conditions:
  \itemize{
    \item
    every component of \code{x} is a spatial object
    whose position can be shifted by \code{\link{shift}};
    \item
    \code{panel.begin} and \code{panel.end} are either
    \code{NULL} or they are spatial objects 
    whose position can be shifted by \code{\link{shift}};
    \item
    \code{adorn.left}, 
    \code{adorn.right}, 
    \code{adorn.top} and 
    \code{adorn.bottom} are all \code{NULL}.
  }

  Another special case is when every component of \code{x} is an
  object of class \code{"fv"} representing a function.
  If \code{equal.scales=TRUE} then all these functions will 
  be plotted with the same axis scales
  (i.e. with the same \code{xlim} and the same \code{ylim}).
}
\section{Spacing between plots}{
  The spacing between individual plots is controlled by the parameters
  \code{mar.panel}, \code{hsep} and \code{vsep}.

  If \code{equal.scales=FALSE}, the plot panels are
  logically separate plots. The margins for each panel are
  determined by the argument \code{mar.panel} which becomes 
  the graphics parameter \code{mar}
  described in the help file for \code{\link{par}}.
  One unit of \code{mar} corresponds to one line of text in the margin.
  If \code{hsep} or \code{vsep} are present, \code{mar.panel}
  is augmented by \code{c(vsep, hsep, vsep, hsep)/2}.
  
  If \code{equal.scales=TRUE}, all the plot panels are drawn
  in the same coordinate system which represents a physical scale.
  The unit of measurement for \code{mar.panel[1,3]}
  is one-sixth of the greatest height of any object plotted in the same row
  of panels, and the unit for \code{mar.panel[2,4]} is one-sixth of the
  greatest width of any object plotted in the same column of panels.
  If \code{hsep} or \code{vsep} are present,
  they are interpreted in the same units as \code{mar.panel[2]}
  and \code{mar.panel[1]} respectively.
}
\seealso{
  \code{\link[base]{print.listof}},
  \code{\link{contour.listof}},
  \code{\link{image.listof}},
  \code{\link{density.splitppp}}
}
\section{Error messages}{
  If the error message \sQuote{Figure margins too large}
  occurs, this generally means that one of the
  objects had a much smaller physical scale than the others.
  Ensure that \code{equal.scales=FALSE}
  and increase the values of \code{mar.panel}.
}
\examples{
# Intensity estimate of multitype point pattern
 plot(D <- density(split(amacrine)))
 plot(D, main="", equal.ribbon=TRUE,
      panel.end=function(i,y,...){contour(y, ...)})

# list of 3D point patterns
 ape1 <- osteo[osteo$shortid==4, "pts", drop=TRUE]
 class(ape1)
 plot(ape1, main.panel="", mar.panel=0.1, hsep=0.7, vsep=1,
      cex=1.5, pch=21, bg='white')
}

\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{hplot}

\name{im.apply}
\alias{im.apply}
\title{
  Apply Function Pixelwise to List of Images 
}
\description{
  Returns a pixel image obtained by applying a function
  to the values of corresponding pixels in several pixel images.
}
\usage{
im.apply(X, FUN, \dots, fun.handles.na=FALSE, check=TRUE)
}
\arguments{
  \item{X}{
    A list of pixel images (objects of class \code{"im"}).
  }
  \item{FUN}{
    A function that can be applied to vectors,
    or a character string giving the name of such a function.
  }
  \item{\dots}{
    Additional arguments to \code{FUN}.
  }
  \item{fun.handles.na}{
    Logical value specifying what to do when the data include
    \code{NA} values. See Details.
  }
  \item{check}{
    Logical value specifying whether to check that the images in
    \code{X} are compatible (for example that they have the same
    grid of pixel locations) and to convert them to compatible images
    if necessary.
  }
}
\details{
  The argument \code{X} should be a list of pixel images
  (objects of class \code{"im"}). If the images do not have
  identical pixel grids, they will be converted to a common
  grid using \code{\link{harmonise.im}}.

  At each pixel location, the values of the images in \code{X}
  at that pixel will be extracted as a vector. The function
  \code{FUN} will be applied to this vector. The result (which should be
  a single value) becomes the pixel value of the resulting image.

  The argument \code{fun.handles.na} specifies what to do
  when some of the pixel values are \code{NA}.
  \itemize{
    \item If \code{fun.handles.na=FALSE} (the default),
    the function \code{FUN} is never applied to data that include
    \code{NA} values; the result is defined to be \code{NA} whenever
    the data contain \code{NA}.
    \item
    If \code{fun.handles.na=TRUE}, the function \code{FUN} will be applied to
    all pixel data, including those which contain \code{NA} values.
  }
}
\value{
  A pixel image (object of class \code{"im"}).
}
\seealso{
  \code{\link{eval.im}} for algebraic operations with images.
}
\examples{
  DA <- density(split(amacrine))
  DA
  im.apply(DA, max)
  im.apply(DA, sum)

  ## Example with incompatible patterns of NA values
  Z <- density(split(ants))
  B <- owin(c(438, 666), c(80, 310))
  Z[[1]][B] <- NA
  opa <- par(mfrow=c(2,2))
  plot(Z[[1]])
  plot(Z[[2]])
  #' Default action: NA -> NA
  plot(im.apply(Z, mean))
  #' Use NA handling in mean.default
  plot(im.apply(Z, mean, na.rm=TRUE, fun.handles.na=TRUE))
  par(opa)
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{manip}
\keyword{programming}

\name{crossing.psp}
\alias{crossing.psp}
\title{Crossing Points of Two Line Segment Patterns}
\description{
  Finds any crossing points between 
  two line segment patterns.
}
\usage{
  crossing.psp(A,B,fatal=TRUE,details=FALSE)
}
\arguments{
  \item{A,B}{
    Line segment patterns (objects of class \code{"psp"}).
  }
  \item{details}{
    Logical value indicating whether to return additional information.
    See below.
  }
  \item{fatal}{
    Logical value indicating what to do
    if the windows of \code{A} and \code{B} do not overlap.
    See Details.
  }
}
\value{
  Point pattern (object of class \code{"ppp"}).
}
\details{
  This function finds any crossing points between
  the line segment patterns \code{A} and \code{B}.

  A crossing point occurs whenever one of the line segments in \code{A}
  intersects one of the line segments in \code{B}, at a nonzero
  angle of intersection.

  The result is a point pattern consisting of all the intersection points.

  If \code{details=TRUE}, additional information is computed,
  specifying where each intersection point came from.
  The resulting point pattern has a data frame of marks, with columns
  named \code{iA, jB, tA, tB}. The marks \code{iA}
  and \code{jB} are the indices of the line segments in \code{A} and
  \code{B}, respectively, which produced each intersection point.
  The marks \code{tA} and \code{tB} are numbers between 0 and 1
  specifying the position of the intersection point along the
  original segments.

  If the windows \code{Window(A)} and \code{Window(B)} do not
  overlap, then an error will be reported if \code{fatal=TRUE},
  while if \code{fatal=FALSE} an error will not occur
  and the result will be \code{NULL}.
}
\seealso{
  \code{\link{selfcrossing.psp}},
  \code{\link{psp.object}},
  \code{\link{ppp.object}}.
}
\examples{
  a <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  b <- psp(runif(10), runif(10), runif(10), runif(10), window=owin())
  plot(a, col="green", main="crossing.psp")
  plot(b, add=TRUE, col="blue")
  P <- crossing.psp(a,b)
  plot(P, add=TRUE, col="red")
  as.data.frame(crossing.psp(a,b,details=TRUE))
}
\author{
  \adrian
  and \rolf
}
\keyword{spatial}
\keyword{manip}

\name{convexify}
\alias{convexify}
\title{
  Weil's Convexifying Operation
}
\description{
  Converts the window \code{W} into a convex set by rearranging
  the edges, preserving spatial orientation of each edge.
}
\usage{
 convexify(W, eps)
}
\arguments{
  \item{W}{
    A window (object of class \code{"owin"}).
  }
  \item{eps}{
    Optional. Minimum edge length of polygonal approximation,
    if \code{W} is not a polygon.
  }
}
\details{
  Weil (1995) defined a convexification operation 
  for windows \eqn{W} that belong to the convex ring (that is,
  for any \eqn{W} which is a finite union of convex sets).
  Note that this is \bold{not} the same as the convex hull.

  The convexified set \eqn{f(W)} has the same total boundary length as
  \eqn{W} and the same distribution of orientations of the boundary.
  If \eqn{W} is a polygonal set, then the convexification \eqn{f(W)}
  is obtained by rearranging all the edges of \eqn{W} in order of
  their spatial orientation.

  The argument \code{W} must be a window. If it is not already a polygonal
  window, it is first converted to one, using
  \code{\link{simplify.owin}}.
  The edges are sorted in increasing order of angular orientation
  and reassembled into a convex polygon.
}
\value{
  A window (object of class \code{"owin"}).
}
\references{
  Weil, W. (1995)
  The estimation of mean particle shape and mean
  particle number in overlapping particle systems in
  the plane. \emph{Advances in Applied Probability} \bold{27}, 102--119.
}
\author{
  \adrian
  
  
  \rolf
  
  and \ege
  
}
\seealso{
  \code{\link{convexhull}} for the convex hull of a window.
}
\examples{
  opa <- par(mfrow=c(1,2))
  plot(letterR)
  plot(convexify(letterR))
  par(opa)
}
\keyword{spatial}
\keyword{utilities}

\name{split.ppp}
\alias{split.ppp}
\alias{split<-.ppp}
\title{Divide Point Pattern into Sub-patterns}
\description{
  Divides a point pattern into several sub-patterns,
  according to their marks, or according to any user-specified grouping.
}
\usage{
  \method{split}{ppp}(x, f = marks(x), drop=FALSE, un=NULL, reduce=FALSE, \dots)
  \method{split}{ppp}(x, f = marks(x), drop=FALSE, un=NULL, \dots) <- value
}
\arguments{
  \item{x}{
    A two-dimensional point pattern.
    An object of class \code{"ppp"}.
  }
  \item{f}{
    Data determining the grouping. Either a factor,
    a logical vector,
    a pixel image with factor values, a tessellation, a window,
    or the name of one of the columns of marks.
  }
  \item{drop}{
    Logical. Determines whether empty groups will
    be deleted.
  }
  \item{un}{
    Logical. Determines whether the resulting subpatterns will be unmarked
    (i.e. whether marks will be removed	from the points in each
    subpattern). 
  }
  \item{reduce}{
    Logical. Determines whether to delete the column of marks used to split the
    pattern, when the marks are a data frame.
  }
  \item{\dots}{
    Other arguments are ignored.
  }
  \item{value}{
    List of point patterns.
  }
} 
\value{
  The value of \code{split.ppp} is a list of point patterns.
  The components of the list are named by the levels of \code{f}.
  The list also has the class \code{"splitppp"}.

  The assignment form \code{split<-.ppp} returns the updated
  point pattern \code{x}. 
}
\details{
  The function \code{split.ppp}
  divides up the points of the point pattern \code{x}
  into several sub-patterns according to the values of \code{f}.
  The result is a list of point patterns.
  
  The argument \code{f} may be
  \itemize{
    \item
    a factor, of length equal to the number of points in \code{x}.
    The levels of \code{f}
    determine the destination of each point in \code{x}.
    The \code{i}th point of \code{x} will be placed in the sub-pattern
    \code{split.ppp(x)$l} where \code{l = f[i]}.
    \item
    a pixel image (object of class \code{"im"}) with factor values.
    The pixel value of \code{f}
    at each point of \code{x} will be used as the classifying variable.
    \item
    a tessellation (object of class \code{"tess"}).
    Each point of \code{x} will be classified according to
    the tile of the tessellation into which it falls.
    \item
    a window (object of class \code{"owin"}).
    Each point of \code{x} will be classified according to
    whether it falls inside or outside this window.
    \item
    a character string, matching the name of one of the columns of
    marks, if \code{marks(x)} is a data frame. This column should
    be a factor.
  }
  If \code{f} is missing, then it will be determined by the
  marks of the point pattern. The pattern \code{x} can be either
  \itemize{
    \item 
    a multitype point pattern
    (a marked point pattern whose marks vector is a factor).
    Then \code{f} is taken to be the marks vector.
    The effect is that the points of each type
    are separated into different point patterns.
    \item 
    a marked point pattern with a data frame of marks, containing at least one
    column that is a factor. The first such column will be used to
    determine the splitting factor \code{f}.
  }

  Some of the sub-patterns created by the split
  may be empty. If \code{drop=TRUE}, then empty sub-patterns will
  be deleted from the list. If \code{drop=FALSE} then they are retained.

  The argument \code{un} determines how to handle marks 
  in the case where \code{x} is a marked point pattern.
  If \code{un=TRUE} then the marks of the 
  points will be discarded when they are split into groups,
  while if \code{un=FALSE} then the marks will be retained.

  If \code{f} and \code{un} are both missing,
  then the default is \code{un=TRUE} for multitype point patterns
  and \code{un=FALSE} for marked point patterns with a data frame of
  marks.

  If the marks of \code{x} are a data frame, then 
  \code{split(x, reduce=TRUE)} will discard only the column of marks
  that was used to split the pattern. This applies only when
  the argument \code{f} is missing.

  The result of \code{split.ppp} has class \code{"splitppp"}
  and can be plotted using \code{\link{plot.splitppp}}.

  The assignment function \code{split<-.ppp} 
  updates the point pattern \code{x} so that
  it satisfies \code{split(x, f, drop, un) = value}. The argument \code{value}
  is expected to be a list of point patterns, one for each level of
  \code{f}. These point patterns are expected to be compatible with the
  type of data in the original pattern \code{x}.

  Splitting can also be undone by the function
  \code{\link{superimpose}},
  but this typically changes the ordering of the data.
}
\seealso{
  \code{\link{cut.ppp}},
  \code{\link{plot.splitppp}},
  \code{\link{superimpose}},
  \code{\link{im}},
  \code{\link{tess}},
  \code{\link{ppp.object}}
}
\examples{

# (1) Splitting by marks

# Multitype point pattern: separate into types
 u <- split(amacrine)

# plot them
 plot(split(amacrine))

# the following are equivalent:
 amon <- split(amacrine)$on
 amon <- unmark(amacrine[amacrine$marks == "on"])
 amon <- subset(amacrine, marks == "on", -marks)
   
# the following are equivalent:
 amon <- split(amacrine, un=FALSE)$on
 amon <- amacrine[amacrine$marks == "on"]
   
# Scramble the locations of the 'on' cells
 X <- amacrine
 u <- split(X)
 u$on <- runifpoint(ex=amon)
 split(X) <- u

# Point pattern with continuous marks
 trees <- longleaf
 \testonly{
	# smaller dataset
	trees <- trees[seq(1, npoints(trees), by=80)]
 }
 # cut the range of tree diameters into three intervals
 # using cut.ppp
 long3 <- cut(trees, breaks=3)
 # now split them
 long3split <- split(long3)

# (2) Splitting by a factor

# Unmarked point pattern
  swedishpines
# cut & split according to nearest neighbour distance
  f <- cut(nndist(swedishpines), 3)
  u <- split(swedishpines, f)

# (3) Splitting over a tessellation
   tes <- tess(xgrid=seq(0,96,length=5),ygrid=seq(0,100,length=5))
   v <- split(swedishpines, tes)


# (4) how to apply an operation to selected points:
#  split into components, transform desired component, then un-split
#  e.g. apply random jitter to 'on' points only
  X <- amacrine
  Y <- split(X)
  Y$on <- rjitter(Y$on, 0.1)
  split(X) <- Y
}

\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{methods}
\keyword{manip}

\name{as.layered}
\alias{as.layered}
\alias{as.layered.default}
\alias{as.layered.ppp}
\alias{as.layered.splitppp}
\alias{as.layered.solist}
\alias{as.layered.listof}
\alias{as.layered.msr}
\title{Convert Data To Layered Object}
\description{
  Converts spatial data into a layered object.
}
\usage{
 as.layered(X)

 \method{as.layered}{default}(X)

 \method{as.layered}{ppp}(X)

 \method{as.layered}{splitppp}(X)

 \method{as.layered}{solist}(X)

 \method{as.layered}{listof}(X)

 \method{as.layered}{msr}(X)
}
\arguments{
  \item{X}{
    Some kind of spatial data.
  }
}
\value{
  An object of class \code{"layered"} (see \code{\link{layered}}).
}
\details{
  This function converts the object \code{X}
  into an object of class \code{"layered"}.

  The argument \code{X} should contain some kind of spatial data
  such as a point pattern, window, or pixel image.

  If \code{X} is a simple object then it will be converted into
  a \code{layered} object containing only one layer which is equivalent
  to \code{X}.

  If \code{X} can be interpreted as consisting of
  multiple layers of data, then the result will be a \code{layered}
  object consisting of these separate layers of data.
  \itemize{
    \item if \code{X} is a list of class \code{"listof"} or \code{"solist"},
    then \code{as.layered(X)} consists of several layers,
    one for each entry in the list \code{X};
    \item if \code{X} is a multitype point pattern,
    then \code{as.layered(X)} consists of several layers,
    each containing the sub-pattern consisting of points of one type;
    \item if \code{X} is a vector-valued measure,
    then \code{as.layered(X)} consists of several layers,
    each containing a scalar-valued measure.
  }
}
\seealso{
  \code{\link{layered}},
  \code{\link{split.ppp}}
}
\examples{
   as.layered(cells)
   as.layered(amacrine)

   P <- rpoispp(100)
   fit <- ppm(P ~ x+y)
   rs <- residuals(fit, type="score")
   as.layered(rs)
}
\author{\adrian
  
  
  \rolf
  
  and \ege
  
}
\keyword{spatial}
\keyword{manip}
 
 

\name{crossdist.ppx}
\alias{crossdist.ppx}
\title{Pairwise Distances Between Two Different Multi-Dimensional Point Patterns}
\description{
  Computes the distances between pairs of points
  taken from two different multi-dimensional point patterns.
}
\usage{
  \method{crossdist}{ppx}(X, Y, \dots)
}
\arguments{
  \item{X,Y}{
    Multi-dimensional point patterns (objects of class \code{"ppx"}).
  }
  \item{\dots}{
    Arguments passed to \code{\link{coords.ppx}} to determine
    which coordinates should be used.
  }
}
\value{
  A matrix whose \code{[i,j]} entry is the distance
  from the \code{i}-th point in \code{X}
  to the \code{j}-th point in \code{Y}.
}
\details{
  Given two point patterns in multi-dimensional space, 
  this function computes the Euclidean distance from each point
  in the first pattern to each point in the second pattern,
  and returns a matrix containing these distances.

  This is a method for the generic function \code{\link{crossdist}}
  for three-dimensional point patterns (objects of class \code{"ppx"}).

  This function expects two multidimensional
  point patterns \code{X} and \code{Y}, and returns the matrix
  whose \code{[i,j]} entry is the distance from \code{X[i]} to
  \code{Y[j]}.
  
  By default, both spatial and temporal coordinates are extracted.
  To obtain the spatial distance between points in a space-time point
  pattern, set \code{temporal=FALSE}.
}
\seealso{
  \code{\link{crossdist}},
  \code{\link{pairdist}},
  \code{\link{nndist}}
}
\examples{
   df <- data.frame(x=runif(3),y=runif(3),z=runif(3),w=runif(3))
   X <- ppx(data=df)
   df <- data.frame(x=runif(5),y=runif(5),z=runif(5),w=runif(5))
   Y <- ppx(data=df)
   d <- crossdist(X, Y)
}
\author{
  \adrian
  
  
}
\keyword{spatial}
\keyword{math}

\name{plot.splitppp}
\alias{plot.splitppp}
\title{Plot a List of Point Patterns}
\description{
  Plots a list of point patterns.
}
\usage{
  \method{plot}{splitppp}(x, \dots, main)
}
\arguments{
  \item{x}{
    A named list of point patterns,
    typically obtained from \code{\link{split.ppp}}.
  }
  \item{\dots}{
    Arguments passed to \code{\link{plot.listof}} which control the
    layout of the plot panels, their appearance, and the
    plot behaviour in individual plot panels.
  }
  \item{main}{
    Optional main title for the plot.
  }
}
\value{
  Null.
}
\details{
  This is the \code{plot} method for the class \code{"splitppp"}.
  It is typically used to plot the result of the function
  \code{\link{split.ppp}}.

  The argument \code{x} should be a named list of point patterns
  (objects of class \code{"ppp"}, see \code{\link{ppp.object}}).
  Each of these point patterns will be plotted in turn
  using \code{\link{plot.ppp}}.

  Plotting is performed by \code{\link{plot.listof}}.
}
\seealso{
  \code{\link{plot.listof}} for arguments controlling the plot.
  
  \code{\link{split.ppp}},
  \code{\link{plot.ppp}},
  \code{\link{ppp.object}}.
}
\section{Error messages}{
  If the error message \sQuote{Figure margins too large}
  occurs, ensure that \code{equal.scales=FALSE}
  and increase the values of \code{mar.panel}.
}
\examples{
# Multitype point pattern
 plot(split(amacrine))
 plot(split(amacrine), main="", 
     panel.begin=function(i, y, ...) { plot(density(y), ribbon=FALSE, ...) })
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{hplot}

\name{convexhull}
\alias{convexhull}
\title{Convex Hull}
\description{
  Computes the convex hull of a spatial object.
}
\usage{
convexhull(x)
}
\arguments{
  \item{x}{
    a window (object of class \code{"owin"}),
    a point pattern (object of class \code{"ppp"}),
    a line segment pattern (object of class \code{"psp"}),
    or an object that can be converted to a window
    by \code{\link{as.owin}}.
  }
}
\value{
  A window (an object of class \code{"owin"}).
}
\details{
  This function computes the convex hull of the spatial object \code{x}.
}
\seealso{
  \code{\link{owin}},
  \code{\link{convexhull.xy}},
  \code{\link{is.convex}}
}
\examples{
   data(demopat)
   W <- Window(demopat)
   plot(convexhull(W), col="lightblue", border=NA)
   plot(W, add=TRUE, lwd=2)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{utilities}

\name{as.function.owin}
\alias{as.function.owin}
\title{
  Convert Window to Indicator Function
}
\description{
  Converts a spatial window to a function of the
  \eqn{x} and \eqn{y} coordinates returning the value 1 inside the
  window and 0 outside.
}
\usage{
 \method{as.function}{owin}(x, \dots)
}
\arguments{
  \item{x}{
    Pixel image (object of class \code{"owin"}).
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  This command converts a spatial window
  (object of class \code{"owin"}) to a \code{function(x,y)}
  where the arguments \code{x} and \code{y} are (vectors of) spatial
  coordinates. This is the indicator function of the window:
  it returns the value 1 for locations inside the window,
  and returns 0 for values outside the window.
}
\value{
  A function in the \R language with arguments \code{x,y}.
  It also belongs to the class \code{"indicfun"} which has methods
  for \code{plot} and \code{print}.
}
\author{
  \spatstatAuthors.
}
\seealso{
  \code{\link{as.im.owin}}
}
\examples{
  W <- Window(humberside)
  f <- as.function(W)
  f
  f(5000, 4500)
  f(123456, 78910)
  X <- runifpoint(5, Frame(humberside))
  f(X)
  plot(f)
}
\keyword{spatial}
\keyword{manip}

\name{clickppp}
\alias{clickppp}
\title{Interactively Add Points}
\description{
  Allows the user to create a point pattern by
  point-and-click in the display.
}
\usage{
  clickppp(n=NULL, win=square(1), types=NULL, \dots, add=FALSE,
    main=NULL, hook=NULL)
}
\arguments{
  \item{n}{
    Number of points to be added (if this is predetermined).
  }
  \item{win}{
    Window in which to create the point pattern.
    An object of class \code{"owin"}.
  }
  \item{types}{
    Vector of types, when creating a multitype point pattern.
  }
  \item{\dots}{
    Optional extra arguments to be passed to \code{\link[graphics]{locator}}
    to control the display.
  }
  \item{add}{
    Logical value indicating whether to create a new plot
    (\code{add=FALSE}) or draw over the existing plot (\code{add=TRUE}).
  }
  \item{main}{
    Main heading for plot.
  }
  \item{hook}{For internal use only. Do not use this argument.}
}
\value{
  A point pattern (object of class \code{"ppp"}).
}
\details{
  This function allows the user to create a point pattern
  by interactively clicking on the screen display.

  First the window \code{win} is plotted on the current screen device.
  Then the user is prompted to point the mouse at any desired locations
  and click the left mouse button to add each point.
  Interactive input stops after \code{n} clicks (if \code{n} was given)
  or when the middle mouse button is pressed.

  The return value is a point pattern
  containing the locations of all the clicked points
  inside the original window \code{win},
  provided that all of the clicked locations were
  inside this window. Otherwise, the window is expanded to a box
  large enough to contain all the points (as well as containing
  the original window).

  If the argument \code{types} is given, then a multitype point pattern
  will be created. The user is
  prompted to input the locations of points of type \code{type[i]},
  for each successive index \code{i}. (If the argument \code{n} was
  given, there will be \code{n} points of \emph{each} type.)
  The return value is a multitype point pattern.
  
  This function uses the \R{} command \code{\link[graphics]{locator}} to
  input the mouse clicks. It only works on screen devices such as
  \sQuote{X11}, \sQuote{windows} and \sQuote{quartz}. Arguments that can be
  passed to \code{\link[graphics]{locator}} through \code{\dots} include
  \code{pch} (plotting character), \code{cex} (character expansion
  factor) and \code{col} (colour). See \code{\link[graphics]{locator}}
  and \code{\link[graphics]{par}}.
}
\seealso{
  \code{\link{identify.ppp}},
  \code{\link[graphics]{locator}},
  \code{\link{clickpoly}},
  \code{\link{clickbox}},
  \code{\link{clickdist}}
}
\author{Original by Dominic Schuhmacher.
  Adapted by \adrian and \rolf.
}
\keyword{spatial}
\keyword{iplot}

\name{relevel.im}
\alias{relevel.im}
\alias{relevel.ppp}
\alias{relevel.ppx}
\title{
  Reorder Levels of a Factor-Valued Image or Pattern
}
\description{
  For a pixel image with factor values, or a point pattern with
  factor-valued marks, the levels of the factor are re-ordered so that
  the level \code{ref} is first and the others are moved down.
}
\usage{
  \method{relevel}{im}(x, ref, \dots)

  \method{relevel}{ppp}(x, ref, \dots)

  \method{relevel}{ppx}(x, ref, \dots)
}
\arguments{
  \item{x}{
    A pixel image (object of class \code{"im"}) with factor values,
    or a point pattern (object of class \code{"ppp"}, \code{"ppx"},
    \code{"lpp"} or \code{"pp3"}) with
    factor-valued marks.
  }
  \item{ref}{
    The reference level.
  }
  \item{\dots}{
    Ignored.
  }
}
\details{
  These functions are methods for the generic
  \code{\link[stats]{relevel}}.

  If \code{x} is a pixel image (object of class \code{"im"}) with
  factor values, or a point pattern (object of class \code{"ppp"},
  \code{"ppx"}, \code{"lpp"} or \code{"pp3"})
  with factor-valued marks, the levels of the factor are changed so
  that the level specified by \code{ref} comes first. 
}
\value{
  Object of the same kind as \code{x}.
}
\author{
\adrian


\rolf

and \ege

}
\seealso{
  \code{\link{mergeLevels}}
}
\examples{
  amacrine
  relevel(amacrine, "on")
}
\keyword{manip}
\keyword{spatial}

\name{erosion}  
\alias{erosion}
\alias{erosion.owin}
\alias{erosion.ppp}
\alias{erosion.psp}
\title{Morphological Erosion by a Disc}
\description{
  Perform morphological erosion of a window, a line segment pattern
  or a point pattern by a disc.
}
\usage{
 erosion(w, r, \dots)
 \method{erosion}{owin}(w, r, shrink.frame=TRUE, \dots,
   strict=FALSE, polygonal=NULL)
 \method{erosion}{ppp}(w, r,\dots)
 \method{erosion}{psp}(w, r,\dots)
}
\arguments{
  \item{w}{
    A window (object of class \code{"owin"}
    or a line segment pattern (object of class \code{"psp"})
    or a point pattern (object of class \code{"ppp"}).
  }
  \item{r}{positive number: the radius of erosion.}
  \item{shrink.frame}{logical: if \code{TRUE}, erode the bounding
    rectangle as well.}
  \item{\dots}{extra arguments to \code{\link{as.mask}}
    controlling the pixel resolution, if pixel approximation is used.}
  \item{strict}{Logical flag determining the fate of boundary pixels,
    if pixel approximation is used. See details.}
  \item{polygonal}{
    Logical flag indicating whether to compute a polygonal
    approximation to the erosion (\code{polygonal=TRUE}) or
    a pixel grid approximation (\code{polygonal=FALSE}).
  }
}
\value{
  If \code{r > 0}, an object of class \code{"owin"} representing the
  eroded region (or \code{NULL} if this region is empty).
  If \code{r=0}, the result is identical to \code{w}.
}
\details{
  The morphological erosion of a set \eqn{W} by a distance \eqn{r > 0}
  is the subset 
  consisting of all points \eqn{x \in W}{x in W} such that the
  distance from \eqn{x} to the boundary of \eqn{W} is greater than
  or equal to \eqn{r}. In other words it is the result of trimming
  a margin of width \eqn{r} off the set \eqn{W}.

  If \code{polygonal=TRUE} then a polygonal approximation
  to the erosion is computed.
  If \code{polygonal=FALSE} then a pixel approximation
  to the erosion is computed from the distance map of \code{w}.
  The arguments \code{"\dots"} are passed to \code{\link{as.mask}}
  to control the pixel resolution.
  The erosion consists of all pixels whose distance
  from the boundary of \code{w} is strictly greater than \code{r} (if
  \code{strict=TRUE}) or is greater than or equal to \code{r} (if
  \code{strict=FALSE}).
  
  When \code{w} is a window, the default (when \code{polygonal=NULL})
  is to compute a polygonal approximation if
  \code{w} is a rectangle or polygonal window, and to compute a
  pixel approximation if \code{w} is a window of type \code{"mask"}.

  If \code{shrink.frame} is false, the resulting window is given the
  same outer, bounding rectangle as the original window \code{w}.
  If \code{shrink.frame} is true, the original bounding rectangle
  is also eroded by the same distance \code{r}.

  To simply compute the area of the eroded window,
  use \code{\link{eroded.areas}}.
}
\seealso{
  \code{\link{dilation}} for the opposite operation.

  \code{\link{erosionAny}} for morphological erosion using any shape.
  
  \code{\link{owin}},
  \code{\link{as.owin}},
  \code{\link{eroded.areas}}
}
\examples{
  plot(letterR, main="erosion(letterR, 0.2)")
  plot(erosion(letterR, 0.2), add=TRUE, col="red")
}
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{math}

\name{rescale.psp}
\alias{rescale.psp}
\title{Convert Line Segment Pattern to Another Unit of Length}
\description{
  Converts a line segment pattern dataset to 
  another unit of length.
}
\usage{
 \method{rescale}{psp}(X, s, unitname)
}
\arguments{
  \item{X}{Line segment pattern (object of class \code{"psp"}).}
  \item{s}{Conversion factor: the new units are \code{s} times the old units.}
  \item{unitname}{
    Optional. New name for the unit of length.
    See \code{\link{unitname}}.
  }
}
\value{
  Another line segment pattern (of class \code{"psp"}),
  representing the same data, but expressed in the new units.
}
\details{
  This is a method for the generic function \code{\link{rescale}}.

  The spatial coordinates in the line segment pattern \code{X}
  (and its window) will be re-expressed
  in terms of a new unit of length that is \code{s} times the current
  unit of length given in \code{X}.
  (Thus, the coordinate values are \emph{divided} by \code{s},
  while the unit value is multiplied by \code{s}).

  The result is a line segment pattern representing the \emph{same} data
  but re-expressed in a different unit.

  Mark values are unchanged.

  If \code{s} is missing, then the coordinates will be re-expressed
  in \sQuote{native} units; for example if the current unit is
  equal to 0.1 metres, then the coordinates will be re-expressed in metres.
}
\section{Note}{
  The result of this operation is equivalent to the original segment pattern.
  If you want to actually change the coordinates by
  a linear transformation, producing a segment pattern that is not
  equivalent to the original one, use \code{\link{affine}}.
}
\seealso{
  \code{\link{units}},
  \code{\link{affine}},
  \code{\link{rotate}},
  \code{\link{shift}}
}
\examples{
   data(copper)
   X <- copper$Lines
   X
   # data are in km
   # convert to metres
   rescale(X, 1/1000)

   # convert data and rename unit
   rescale(X, 1/1000, c("metre", "metres"))
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{tweak.colourmap}
\alias{tweak.colourmap}
\title{
  Change Colour Values in a Colour Map
}
\description{
  Assign new colour values to some of the entries in a colour map.
}
\usage{
tweak.colourmap(m, col, ..., inputs=NULL, range=NULL)
}

\arguments{
  \item{m}{
    A colour map (object of class \code{"colourmap"}).
  }
  \item{inputs}{
    Input values to the colour map, to be assigned new colours.
    Incompatible with \code{range}.
  }
  \item{range}{
    Numeric vector of length 2 specifying a range of numerical values
    which should be assigned a new colour.
    Incompatible with \code{inputs}.
  }
  \item{col}{
    Replacement colours for the specified \code{inputs} or
    the specified \code{range} of values.
  }
  \item{\dots}{Other arguments are ignored.}
}
\details{
  This function changes the colour map \code{m}
  by assigning new colours to each of the
  input values specified by \code{inputs},
  or by assigning a single new colour
  to the range of input values specified by \code{range}.
  
  The modified colour map is returned.
}
\value{
  Another colour map (object of class \code{"colourmap"}).
}
\seealso{
  \code{\link{colourmap}}, 
  \code{\link{interp.colourmap}}, 
  \code{\link{colouroutputs}}, 
  \code{\link[spatstat:colourtools]{colourtools}}.
}
\examples{
  co <- colourmap(rainbow(32), range=c(0,1))
  plot(tweak.colourmap(co, inputs=c(0.5, 0.6), "white"))
  plot(tweak.colourmap(co, range=c(0.5,0.6), "white"))
}
\author{
  \spatstatAuthors.
}
\keyword{spatial}
\keyword{color}

\name{affine.tess} 
\alias{reflect.tess} 
\alias{flipxy.tess} 
\alias{shift.tess} 
\alias{rotate.tess} 
\alias{scalardilate.tess} 
\alias{affine.tess} 
\title{Apply Geometrical Transformation To Tessellation}
\description{
  Apply various geometrical transformations of the plane
  to each tile in a tessellation.
}
\usage{
  \method{reflect}{tess}(X)

  \method{flipxy}{tess}(X)

  \method{shift}{tess}(X, \dots)

  \method{rotate}{tess}(X, angle=pi/2, \dots, centre=NULL)

  \method{scalardilate}{tess}(X, f, \dots)

  \method{affine}{tess}(X, mat=diag(c(1,1)), vec=c(0,0), \dots)
}
\arguments{
  \item{X}{Tessellation (object of class \code{"tess"}).}
  \item{angle}{
    Rotation angle in radians (positive values represent
    anticlockwise rotations).
  }
  \item{mat}{Matrix representing a linear transformation.}
  \item{vec}{Vector of length 2 representing a translation.}
  \item{f}{Positive number giving scale factor.}
  \item{\dots}{Arguments passed to other methods.}
  \item{centre}{
    Centre of rotation.
    Either a vector of length 2, or a character string
    (partially matched to \code{"centroid"}, \code{"midpoint"}
    or \code{"bottomleft"}).
    The default is the coordinate origin \code{c(0,0)}.
  }
}
\value{
  Another tessellation (of class \code{"tess"}) representing the
  result of applying the geometrical transformation.
}
\details{
  These are method for the generic functions 
\code{\link{reflect}},
\code{\link{flipxy}},
\code{\link{shift}},
\code{\link{rotate}}, 
\code{\link{scalardilate}},
\code{\link{affine}}
for tessellations (objects of class \code{"tess"}).

The individual tiles of the tessellation, and the window
containing the tessellation, are all subjected to the same
geometrical transformation.

The transformations are performed by the corresponding method
for windows (class \code{"owin"}) or images (class \code{"im"})
depending on the type of tessellation.

If the argument \code{origin} is used in \code{shift.tess}
it is interpreted as applying to the window containing the
tessellation. Then all tiles are shifted by the same vector.
}
\seealso{
  Generic functions
  \code{\link{reflect}},
  \code{\link{shift}},
  \code{\link{rotate}}, 
  \code{\link{scalardilate}},
  \code{\link{affine}}.

  Methods for windows:
  \code{\link{reflect.default}},
  \code{\link{shift.owin}},
  \code{\link{rotate.owin}}, 
  \code{\link{scalardilate.owin}},
  \code{\link{affine.owin}}.

  Methods for images:
  \code{\link{reflect.im}},
  \code{\link{shift.im}},
  \code{\link{rotate.im}}, 
  \code{\link{scalardilate.im}},
  \code{\link{affine.im}}.
}
\examples{
  live <- interactive()
  if(live) {
    H <- hextess(letterR, 0.2)
    plot(H)
    plot(reflect(H))
    plot(rotate(H, pi/3))
  } else H <- hextess(letterR, 0.6)

  # shear transformation
  shear <- matrix(c(1,0,0.6,1),2,2)
  sH <- affine(H, shear)
  if(live) plot(sH)
}
\author{\adrian
  
  
  and \rolf
  
}
\keyword{spatial}
\keyword{math}

\name{cbind.hyperframe}
\alias{cbind.hyperframe}
\alias{rbind.hyperframe}
\title{
  Combine Hyperframes by Rows or by Columns
}
\description{
  Methods for \code{cbind} and \code{rbind} for hyperframes.
}
\usage{
\method{cbind}{hyperframe}(...)
\method{rbind}{hyperframe}(...)
}
\arguments{
  \item{\dots}{
    Any number of hyperframes (objects of class \code{\link{hyperframe}}).
  }
}
\details{
  These are methods for \code{\link{cbind}}
  and \code{\link{rbind}} for hyperframes.
  
  Note that \emph{all} the arguments must be hyperframes (because of
  the peculiar dispatch rules of \code{\link{cbind}} and
  \code{\link{rbind}}).

  To combine a hyperframe with a data frame, one should either 
  convert the data frame to a hyperframe using
  \code{\link{as.hyperframe}}, or explicitly invoke the
  function \code{cbind.hyperframe} or \code{rbind.hyperframe}.
  
  In other words: if \code{h} is a hyperframe and \code{d} is a data frame,
  the result of \code{cbind(h,d)} will be the same as
  \code{cbind(as.data.frame(h), d)}, so that all hypercolumns
  of \code{h} will be deleted (and a warning will be issued).
  To combine \code{h} with \code{d}
  so that all columns of \code{h} are retained,
  type either \code{cbind(h, as.hyperframe(d))} or
  \code{cbind.hyperframe(h,d)}.
}
\value{
  Another hyperframe.
}
\author{\adrian
  
  
  and \rolf
  
}
\seealso{
  \code{\link{hyperframe}},
  \code{\link{as.hyperframe}}
}
\examples{
  lambda <- runif(5, min=10, max=30)
  X <- lapply(as.list(lambda), function(x) { rpoispp(x) })
  h <- hyperframe(lambda=lambda, X=X)
  g <- hyperframe(id=letters[1:5], Y=rev(X))
  gh <- cbind(h, g)
  hh <- rbind(h[1:2, ], h[3:5,])
}
\keyword{spatial}
\keyword{manip}

\name{Extract.listof}
\alias{[<-.listof}
\title{Extract or Replace Subset of a List of Things}
\description{
  Replace a subset of a list of things.
}
\usage{
  \method{[}{listof}(x, i) <- value
}
\arguments{
  \item{x}{
    An object of class \code{"listof"} representing a list of things
    which all belong to one class.
  }
  \item{i}{
    Subset index. Any valid subset index in the usual \R sense.
  }
  \item{value}{
    Replacement value for the subset. 
  }
}
\value{
  Another object of class \code{"listof"}.
}
\details{
  This is a subset replacement method for the class \code{"listof"}.

  The argument \code{x} should be an object of class \code{"listof"}
  representing a list of things that all belong to one class.

  The method replaces a designated
  subset of \code{x}, and returns an object of class \code{"listof"}.
}
\seealso{
  \code{\link{plot.listof}},
  \code{\link{summary.listof}}
}
\examples{
   x <- list(A=runif(10), B=runif(10), C=runif(10))
   class(x) <- c("listof", class(x))
   x[1] <- list(A=rnorm(10))
 }
\author{
  \spatstatAuthors
}
\keyword{spatial}
\keyword{manip}
